(*:Mathematica Version: 5.0 *)

(*:Package Version: 1.1 *)

(*:Copyright: Copyright 2004, Wellin, Gaylord and Kamin *)

(*:Title: IPM3`BaseConvert *)

(* From the book Introduction to Programming with Mathematica, 3rd Ed. *)
   
(*:Author: Paul R. Wellin and Richard J. Gaylord, and Samuel N. Kamin *)

(*:Summary:
	The code is presented and discussed
	in the book Introduction to Programming with Mathematica,
	3rd Edition, Cambridge University Press, 2004. *)

(* ******************************************************** *)

BeginPackage["IPM3`BaseConvert`"];

StringToInteger::usage = 
"StringToInteger[str, b] converts str to an integer, assuming it is \
written in base b. The default base is 10.";

StringToInteger::badbase = 
"Second argument, `1`, to StringToInteger is inappropriate base.";

StringToInteger::baddigs = 
"First argument, `1`, to StringToInteger has incorrect digits.";

StringToInteger::notstr = 
"First argument, `1`, to StringToInteger must be a string.";

StringToInteger::notint = 
"Second argument, `1`, to StringToInteger must be an integer.";

Begin["`Private`"];

StringToInteger[str_String, b_Integer:10] := 
  Module[{convert}, convert[lis_] := 
     Fold[(b*#1 + #2)& , 0, lis]; 
    Which[ !2 <= b <= 10, Message[
      StringToInteger::badbase, b], 
      !And @@ (Inequality[0, LessEqual, 
          numeralToInt[#1], Less, b] & ) /@ 
        ToCharacterCode[str], 
     Message[StringToInteger::baddigs, str], True, 
     convert[ToExpression /@ Characters[str]]]]
     
StringToInteger[str_, b_] :=
    Which[
       Not[StringQ[str]], Message[StringToInteger::notstr,str],
       Not[IntegerQ[str]], Message[StringToInteger::notint, b]];

numeralToInt[numeral_] := numeral - ToCharacterCode["0"][[1]];

End[];

EndPackage[]
