(*:Mathematica Version: 5.0 *)

(*:Package Version: 1.0 *)

(*:Copyright: Copyright 2004, Wellin, Gaylord, and Kamin *)

(*:Title: IPM3`Collatz *)

(* From the book Introduction to Programming with Mathematica, 3rd Ed. *)
   
(*:Author: Paul R. Wellin and Richard J. Gaylord, and Samuel N. Kamin *)

(*:Summary:
	This package provides functions for computing Collatz
	sequences. The code is presented and discussed
	in the book Introduction to Programming with Mathematica,
	3rd Edition, 2004. *)

(* ******************************************************** *)

BeginPackage["IPM3`Collatz`"]

CollatzSequence::usage="CollatzSequence[n] computes the sequence of Collatz iterates starting with initial value n. The sequence terminates as soon as it reaches the value 1.";

CollatzSequence::notint="First argument, `1`, to CollatzSequence must be a positive integer.";

Begin["`Private`"]

collatz[n_?EvenQ]:=n/2

collatz[n_?OddQ]:=3n+1

CollatzSequence[n_]:=
  If[IntegerQ[n] && n>=0,
    NestWhileList[collatz, n, #!=1&],
    Message[CollatzSequence::notint, n]
    ]

CollatzSequence[_,args__]/;
   Message[CollatzSequence::argx, CollatzSequence, Length[{args}]+1]:=Null

End[]

EndPackage[]


(* *********EXAMPLES******** *)
(*

--- Load the package ---
<<IPM3`Collatz`


--- Check the usage message ---
?CollatzSequence


--- Try out the public function on a valid input ---
CollatzSequence[23]


--- Try out with an invalid input ---
CollatzSequence[-5]


--- Try out with wrong number of arguments ---
CollatzSequence[3, 4]

*)
