(*:Mathematica Version: 5.0 *)

(*:Package Version: 1.0 *)

(*:Copyright: Copyright 2004, Wellin, Gaylord, and Kamin *)

(*:Title: IPM3`Life` *)

(* From the book Introduction to Programming with Mathematica, 3rd Ed. *)
   
(*:Author: Paul R. Wellin and Richard J. Gaylord and Samuel N. Kamin *)

(*:Summary:
	The code is presented and discussed
	in the book Introduction to Programming with Mathematica,
	3rd Edition, Cambridge University Press, 2004. *)

(* ******************************************************** *)

BeginPackage["IPM3`Life`",
	{"Utilities`FilterOptions`"}];

LifeGame::usage=
"LifeGame[n, steps] creates ...";

LifeGraphics::usage=
"LifeGraphics[lis, opts] creates ...";

Colors::usage=
"Colors is an option to LifeGraphics. To specify a different color scheme, use \
Colors->{0->RGBColor[...], 1->RGBColor[...].";


AnimateLife::usage=
"AnimateLife[lis, opts] creates ...";

Begin["`Private`"]; 

LifeGame[n_, steps_] := Module[{gameboard, liveNeighbors, update},
    gameboard = Table[Random[Integer], {n}, {n}];
    liveNeighbors[mat_] := Apply[Plus, Map[RotateRight[mat, #] &,
          {{-1,-1}, {-1,0}, {-1,1}, {0,-1}, {0,1}, 
          {1,-1}, {1,0}, {1,1}}]];
    update[1, 2] := 1;
    update[_, 3] := 1;
    update[_, _] := 0;
    SetAttributes[update, Listable];
    FixedPointList[update[#, liveNeighbors[#]] &, gameboard, steps]
    ]

LifeGame[n_, steps_, lifeform_List] := 
    Module[{init = Table[0, {n}, {n}], gameboard, liveNghbrs, update},
    gameboard = ReplacePart[
          init, 1, lifeform]; 
    liveNghbrs[mat_] := Apply[Plus, Map[RotateRight[mat,#]&, 
    	{{-1,-1}, {-1,0}, {-1,1}, {0,-1}, {0,1}, {1,-1}, {1,0}, {1,1}}]];
    update[1, 2] := 1;
    update[_, 3] := 1;
    update[_, _] := 0;
    Attributes[update] = Listable;
    FixedPointList[update[#, liveNghbrs[#]]&, 
       gameboard, steps]]    
    
Options[LifeGraphics]=
   Colors->{1->RGBColor[1,0,0],0->RGBColor[0,0,0]};

LifeGraphics[lis_,opts___Rule]:=Module[{colors},
    colors=Colors/.Flatten[{opts,Options[LifeGraphics]}];
    Map[
      Graphics[RasterArray[
            Reverse[#/.colors]],
          AspectRatio->Automatic]&,lis]]         

AnimateLife[lis_,opts___]:=
   Map[Show,LifeGraphics[lis,FilterOptions[LifeGraphics,opts]]]

End[];

EndPackage[]


(* Examples *)
(*

This shows only the last frame from the game after 75 iterations on a 100\[Cross]100 board.

Show[Last[LifeGraphics[LifeGame[100,75]]]]

AnimateLife[LifeGame[75, 100]];

AnimateLife[LifeGame[50, 5], 
   Colors -> {0->RGBColor[1,1,0], 1->RGBColor[0,1,1]}];

lg50=LifeGame[50,10,glider[20,20]];
AnimateLife[lg50];  

*)
