(*:Mathematica Version: 5.0 *)

(*:Package Version: 1.1 *)

(*:Copyright: Wellin, Gaylord, Kamin, Copyright 2004 *)

(*:Title: IPM3`PDL *)

(* From the book Introduction to Programming with Mathematica, 3rd Ed. *)
   
(*:Author: Paul R. Wellin and Richard J. Gaylord, and Samuel N. Kamin *)

(*:Summary:
	The code is presented and discussed
	in the book Introduction to Programming with Mathematica,
	3rd Edition, Cambridge University Press, 2004. *)

(* **************************************************************** *)

BeginPackage["IPM3`PDL`"]

Needs["IPM3`BaseConvert`"]

Lex::usage = "something";
Lexaux::usage = "something";
Parse::usage = "something";
ComputeShapes::usage = "something";
ConvertShapes::usage = "something";
ShowPicture::usage = "something";
PDL::usage = "something";

lparen;
rparen;
comma;
slash;
connecting;
containing;
square;
circle;
oval;
rectangle;
clear;
northeast;
southeast;
southwest;
northwest;
north;
east;
south;
west;
center;
eof;
Prod1;
Prod2;
Prod3;
Prod4;
Prod5;
Prod6;
Prod7;
Prod8;
Prod9;
Prod10;
Prod11;
Prod12;
Prod13;
Prod14;
Prod15;
Prod16;

Begin["`Private`"]

(* ************************************ *)
(*         LEXICAL ANALYSIS             *)

mainRules = {
  {"(", y___} -> {lparen, y},
  {")", y___} -> {rparen, y},
  {",", y___} -> {comma, y},
  {"/", y___} -> {slash, y},
  {"c", "o", "n", "n", "e", "c", "t", "i", "n", "g", y___}
     -> {connecting, y},
  {"c", "o", "n", "t", "a", "i", "n", "i", "n", "g", y___}
     -> {containing, y},
  {"s", "q", "u", "a", "r", "e", y___} -> {square, y},
  {"c", "i", "r", "c", "l", "e", y___} -> {circle, y},
  {"o", "v", "a", "l", y___} -> {oval, y},
  {"r", "e", "c", "t", "a", "n", "g", "l", "e", y___}
     -> {rectangle, y},
  {"c", "l", "e", "a", "r", y___} -> {clear, y},
  {"n", "e", y___} -> {northeast, y},
  {"s", "e", y___} -> {southeast, y},
  {"s", "w", y___} -> {southwest, y},
  {"n", "w", y___} -> {northwest, y},
  {"n", y___} -> {north, y},
  {"e", y___} -> {east, y},
  {"s", y___} -> {south, y},
  {"w", y___} -> {west, y},
  {"c", y___} -> {center, y}
};

convertDigits[L_]:= Map[If[DigitQ[#], StringToInteger[#], #]&, L]

numberRule = 
  {{m_?NumberQ, n_?NumberQ, y___} -> {10m+n, y}};

removeBlanks = { {" ", y___} -> {y} };

Lex[input_]:=
   Module[{inp = Map[FromCharacterCode, ToCharacterCode[input]]},
     Lexaux[Join[convertDigits[inp], {eof}] //. removeBlanks]]

Lexaux[{eof}]:= {eof}
Lexaux[input_]:=
   Module[{lexed = If[NumberQ[First[input]],
                      input //. numberRule,
                      input /. mainRules]},
     Join[{First[lexed]}, Lexaux[Rest[lexed] //. removeBlanks]]]

(* ************************************ *)
(*               PARSING                *)

prodsFor[picture]:= {1}
prodsFor[associations]:= {2, 3, 4}
prodsFor[connection]:= {5}
prodsFor[containment]:= {6}
prodsFor[shape]:= {7}
prodsFor[color]:= {8, 9}
prodsFor[primitive]:= {10, 11, 12, 13}
prodsFor[size]:= {14}
prodsFor[size2]:= {15, 16}

matches[1, t_]:= MemberQ[{clear, square, circle, oval, rectangle}, t]
matches[2, t_]:= Not[MemberQ[{connecting, containing}, t]]
matches[3, t_]:= MemberQ[{connecting}, t]
matches[4, t_]:= MemberQ[{containing}, t]
matches[5, t_]:= MemberQ[{connecting}, t]
matches[6, t_]:= MemberQ[{containing}, t]
matches[7, t_]:= MemberQ[{clear, square, circle, oval, rectangle}, t]
matches[8, t_]:= MemberQ[{square, circle, oval, rectangle}, t]
matches[9, t_]:= MemberQ[{clear}, t]
matches[10, t_]:= MemberQ[{square}, t]
matches[11, t_]:= MemberQ[{circle}, t]
matches[12, t_]:= MemberQ[{oval}, t]
matches[13, t_]:= MemberQ[{rectangle}, t]
matches[14, t_]:= MemberQ[{lparen}, t]
matches[15, t_]:= MemberQ[{rparen}, t]
matches[16, t_]:= MemberQ[{comma}, t]

Parse[tokens_]:= First[Parse[prodsFor[picture], tokens]]

Parse[{}, x_]:=
    (Print["Syntax error:  remaining input is ",
           Take[x, Min[Length[x], 10]], " ..."];
     Abort[])
Parse[{pn_, pns___}, tokens_]:=
    If[matches[pn, First[tokens]],  (* if pn applies *)
       Parse[pn, tokens],           (* parse using it *)
       Parse[{pns}, tokens]]        (* else try other prod's *)
 
Parse[1, tokens_] := Module[{part1 = Parse[prodsFor[shape], tokens], 
    part2},
    part2 = Parse[prodsFor[associations], part1[[2]]]; 
   {Prod1[part1[[1]], part2[[1]]], part2[[2]]}]
   
Parse[2, tokens_]:= {Prod2[], tokens}

Parse[3, tokens_] := Module[{
	part1 = Parse[prodsFor[connection], tokens], part2},
    part2 = Parse[prodsFor[associations], part1[[2]]]; 
   {Prod3[part1[[1]], part2[[1]]], part2[[2]]}]
   
Parse[4, tokens_] := Module[{
	part1 = Parse[prodsFor[containment], tokens], 
    	part2}, 
    	part2 = Parse[prodsFor[associations], part1[[2]]]; 
   {Prod4[part1[[1]], part2[[1]]], part2[[2]]}]
   
Parse[5, tokens_] := 
  Module[{part1 = Parse[prodsFor[picture], Drop[tokens, 5]]}, 
   {Prod5[tokens[[2]], tokens[[4]], part1[[1]]], Rest[part1[[2]]]}]
   
Parse[6, tokens_]:=
    Module[{part1 = Parse[prodsFor[picture], Drop[tokens, 5]]},
        {Prod6[tokens[[2]], tokens[[4]], part1[[1]]], Rest[part1[[2]]]}]

Parse[7, tokens_] := Module[{part1 = Parse[prodsFor[color], tokens],
	part2, part3},
    part2 = Parse[prodsFor[primitive], part1[[2]]]; 
    part3 = Parse[prodsFor[size], part2[[2]]]; 
   {Prod7[part1[[1]], part2[[1]], part3[[1]]], part3[[2]]}]
   
Parse[8, tokens_]:= {Prod8[], tokens}

Parse[9, tokens_]:= {Prod9[], Rest[tokens]}

Parse[10, tokens_]:= {Prod10[], Rest[tokens]}

Parse[11, tokens_]:= {Prod11[], Rest[tokens]}

Parse[12, tokens_]:= {Prod12[], Rest[tokens]}

Parse[13, tokens_]:= {Prod13[], Rest[tokens]}

Parse[14, tokens_]:=
    Module[{part1 = Parse[prodsFor[size2], Drop[tokens, 2]]},
      {Prod14[tokens[[2]], part1[[1]]], part1[[2]]}]

Parse[15, tokens_]:= {Prod15[], Rest[tokens]}

Parse[16, tokens_]:= {Prod16[tokens[[2]]], Drop[tokens, 3]}

(* ************************************** *)
(*          SHAPE COMPUTATION             *)
(* traverse parse tree, computing shape of 
   every picture (Prod1 node).  A "shape"
   is a list of seven items: center (a pair
   of numbers), primitive, color, distance
   from center to north, distance from center
   to east, angle to northeast, distance to
   northeast
*)

center[{c_, ___}]:= c
primitive[{_, p_, ___}]:= p
color[{_, _, c_, ___}]:= c
east[{_, _, _, e_, ___}]:= e
north[{_, _, _, _, n_, ___}]:= n
neangle[{_, _, _, _, _, a_, ___}]:= a
nedist[{_, _, _, _, _, _, d_}]:= d

angle[s_, north]:= Pi/2
angle[s_, south]:= -Pi/2
angle[s_, east]:= 0
angle[s_, west]:= Pi
angle[s_, northeast]:= neangle[s]
angle[s_, southeast]:= -neangle[s]
angle[s_, southwest]:= Pi+neangle[s]
angle[s_, northwest]:= Pi-neangle[s]

dist[s_, north]:= north[s]
dist[s_, south]:= north[s]
dist[s_, east]:= east[s]
dist[s_, west]:= east[s]
dist[s_, northeast]:= nedist[s]
dist[s_, southeast]:= nedist[s]
dist[s_, southwest]:= nedist[s]
dist[s_, northwest]:= nedist[s]

pointOf[s_, d_, delta_]:=
   center[s] + vector[angle[s, d], dist[s, d] + delta]
computeCenter[s_, p_, center]:= p
computeCenter[s_, p_, d_]:=
   p + vector[Pi+angle[s, d], dist[s, d]]
vector[theta_, r_]:= {r Cos[theta], r Sin[theta]}

separation = .1;

ComputeShapes[tree_]:= 
          Select[N[computeShapes[tree, {0, 0}, center]],
                 (color[#] =!= Prod9[])&]

computeShapes[Prod1[sh_, assoc_], p_, d_] := 
  Module[{s = computeShape[sh, p, d], as},
  	as = computeAssociatedShapes[assoc, s];
  	Join[{s}, as]]
      
computeAssociatedShapes[Prod2[], _]:= {}

computeAssociatedShapes[Prod3[Prod5[d1_, d2_, pic_], assoc_], s_] := 
  Module[{p = computePoint[s, d1, d2, connecting], ss},
	ss = computeShapes[pic, p, d2]; 
	Join[ss, computeAssociatedShapes[assoc, s]]]
   
computeAssociatedShapes[Prod4[Prod6[d1_, d2_, pic_], assoc_], s_] := 
  Module[{p = computePoint[s, d1, d2, containing], ss},
	ss = computeShapes[pic, p, d2]; 
	Join[ss, computeAssociatedShapes[assoc, s]]]
   
computePoint[s_, d1_, d2_, relation_]:=
   Which[d1===center, center[s],
         d2===center, pointOf[s, d1, 0],
         relation===connecting, pointOf[s, d1, separation],
         True, pointOf[s, d1, -separation]]

computeShape[s_, p_, d_]:=
   Module[{si = shapeInfo[s]},
     Join[{computeCenter[si, p, d]}, Rest[si]]]

shapeInfo[Prod7[color_, Prod10[], Prod14[i_, _]]]:=
   {0, square, color, i/2, i/2, Pi/4, i/Sqrt[2]}
shapeInfo[Prod7[color_, Prod11[], Prod14[i_, _]]]:=
   {0, circle, color, i/2, i/2, Pi/4, i/2}
ovalNE[a_, b_, theta_]:=
   a b Sqrt[(1 + Tan[theta]^2)/(a^2 + b^2 Tan[theta]^2)]
shapeInfo[Prod7[color_, Prod12[], Prod14[l_, Prod16[h_]]]]:=
   {0, oval, color, l/2, h/2, ArcTan[h/l], ovalNE[h/2, l/2, ArcTan[h/l]]}
shapeInfo[Prod7[color_, Prod13[], Prod14[l_, Prod16[h_]]]]:=
   {0, rectangle, color, l/2, h/2, ArcTan[h/l], Sqrt[h^2 + l^2]/2}


(* ************************************* *)
(*              OUTPUT                   *)
(* convert list of shapes to list of     *)
(* Mathematica graphics objects and draw *)

makeRectangle[p_, l_, h_]:=
  Line[{p, p+{0,h}, p+{l,h}, p+{l,0}, p}]
convertShape[s_] /; MemberQ[{square, rectangle}, s[[2]]]:=
     makeRectangle[pointOf[s, southwest, 0], 2 east[s], 2 north[s]]
convertShape[s_] /; MemberQ[{circle, rectangle}, s[[2]]]:=
     Circle[center[s], north[s]]
convertShape[s_] /; s[[2]] === oval:=
     Circle[center[s], {east[s], north[s]}]

ConvertShapes[ss_]:= Map[convertShape, ss]

ShowPicture[p_]:=
  Show[Graphics[p], AspectRatio->Automatic]

PDL[input_]:=
  ShowPicture[
    ConvertShapes[
      ComputeShapes[
        Parse[
          Lex[input]]]]]

End[]
EndPackage[]
