(*:Mathematica Version: 5.0 *)

(*:Package Version: 1.1 *)

(*:Copyright: Copyright 2004 *)

(*:Title: Trees *)

(* From the book Introduction to Programming with Mathematica, 3rd Ed. *)
   
(*:Author: Paul R. Wellin and Richard J. Gaylord, and Samuel N. Kamin *)

(*:Summary:
	The code is presented and discussed
	in the book Introduction to Programming with Mathematica,
	3rd Edition, Cambridge University Press, 2004. *)

BeginPackage["IPM3`Trees`"]

DrawTree::usage = "";
drawTree1::usage = "";
drawTree2::usage = "";
revtree::usage = "";
drawSepTree::usage = "";
placeTree::usage = "";
transGraphic::usage = "";
TR::usage = "";
enlargeDisk::usage = "";


Begin["`Private`"]

printTree[t_] := printTree[t, 0]

printTree[{lab_}, k_] := (printIndented[lab, 3k]; Null)
printTree[{lab_, lc_, rc_}, k_] :=
   (printIndented[lab, 3k]; Map[(printTree[#, k+1])&, {lc, rc}];)

printIndented[x_, spaces_] :=
   Print[Apply[StringJoin, Table[" ", {spaces}]], x]

zip[{}, _] := {}
zip[_, {}] := {}
zip[{x1_, y1___}, {x2_, y2___}] := Join[{{x1, x2}}, zip[{y1}, {y2}]]

extend[edges1_, edges2_, sep_] :=
   Join[edges1+sep,
        Take[edges2, Min[0, Length[edges1]-Length[edges2]]] - sep]

DrawTree[t_] := drawSepTree[TR[t][[1]], 0, 0]

drawSepTree[{}, lev_, xaxis_] := {AbsolutePointSize[5],Point[{xaxis,lev}]}
(* {Disk[{xaxis, lev}, 0.2]} *)
drawSepTree[{sep_, lc_, rc_}, lev_, xaxis_]  :=
   Join[{ {AbsolutePointSize[5], Point[{xaxis,lev}]},
   (* Disk[{xaxis, lev}, 0.2], *)
         Line[{{xaxis,lev}, {xaxis-sep, lev-1}}],
         Line[{{xaxis,lev}, {xaxis+sep, lev-1}}]},
        drawSepTree[lc, lev-1, xaxis-sep],
        drawSepTree[rc, lev-1, xaxis+sep]]

placeTree[{_}] := {{}, 0, 0}
placeTree[{_, lc_, rc_}] :=
   With[{left = placeTree[lc],
         right = placeTree[rc],
         minsep = 1.0},
      With[{sep = left[[3]] + right[[2]] + minsep},
         {{sep, left[[1]], right[[1]]},
          left[[2]]+sep/2, right[[3]]+sep/2}]]

TR[{_}] := {{}, {}, {}}
TR[{_, lc_, rc_}] :=
   With[{left = TR[lc],
         right = TR[rc],
         minsep = 2.0},
      With[{sep = (Max[0, Apply[Max, Map[Apply[Plus, #]&,
                         zip[left[[3]], right[[2]]]]]] + minsep) / 2},
         With[{newtree = {sep, left[[1]], right[[1]]},
               leftedge = Join[{sep}, extend[left[[2]], right[[2]], sep]],
               rightedge = Join[{sep}, extend[right[[3]], left[[3]], sep]]},
            {newtree, leftedge, rightedge}]]]

TR1[{x_}] := {{}, {width[x]/2}, {width[x]/2}}
TR1[{x_, lc_, rc_}] :=
   With[{left = TR1[lc],
         right = TR1[rc],
         minsep = 0.5},
      With[{sep = (Max[0, Apply[Max, Map[Apply[Plus, #]&,
                         zip[left[[3]], right[[2]]]]]] + minsep) / 2},
         With[{newtree = {sep, left[[1]], right[[1]]},
               leftedge = Join[{width[x]/2},
                               extend[left[[2]], right[[2]], sep]],
               rightedge = Join[{width[x]/2},
                                extend[right[[3]], left[[3]], sep]]},
            {newtree, leftedge, rightedge}]]]

width[t_] := 0

drawTree1[t_] := drawSepTree1[t, TR1[t][[1]], 0, 0]
drawTree2[t_] := drawSepTree2[t, TR1[t][[1]], 0, 0]

drawSepTree1[{lab_}, {}, lev_, xaxis_] :=
           {Text[settext[lab], {xaxis, lev}]}
        (* should be: drawBox[width[lab], lev, xaxis] *)

drawSepTree1[{lab_, lc_, rc_}, {sep_, ls_, rs_}, lev_, xaxis_]  :=
   With[{h1 = If[lab == "", 0, .3],
         h2 = If[lc[[1]] == "", 0, .3],
         h3 = If[rc[[1]] == "", 0, .3]},
      Join[{Text[settext[lab], {xaxis, lev}],
            Line[{{xaxis-sep*(h1/2), lev-h1/2},
                  {xaxis-sep+sep*(h2/2), lev-1+h2/2}}],
            Line[{{xaxis+sep*(h1/2), lev-h1/2},
                  {xaxis+sep-sep*(h3/2), lev-1+h3/2}}]},
         drawSepTree1[lc, ls, lev-1, xaxis-sep],
         drawSepTree1[rc, rs, lev-1, xaxis+sep]]]

drawSepTree2[{lab_}, {}, lev_, xaxis_] :=
           {Text[settext[lab], {xaxis, lev}]}
drawSepTree2[{lab_, lc_, rc_}, {sep_, ls_, rs_}, lev_, xaxis_]  :=
   With[{h1 = If[lab == "", 0, .5],
         h2 = If[lc[[1]] == "", 0, .5],
         h3 = If[rc[[1]] == "", 0, .5]},
      Join[{Text[settext[lab], {xaxis, lev}],
            Line[{{xaxis-sep*(h1/2), lev-h1/2},
                  {xaxis-sep+sep*(h2/2), lev-1+h2/2}}],
            Line[{{xaxis+sep*(h1/2), lev-h1/2},
                  {xaxis+sep-sep*(h3/2), lev-1+h3/2}}]},
         drawSepTree2[lc, ls, lev-1, xaxis-sep],
         drawSepTree2[rc, rs, lev-1, xaxis+sep]]]

         (* should be: drawBox[width[lab], lev, xaxis], *)
settext[lab_] := FontForm[lab, {"Helvetica", 9}]
drawBox[w_, y_, x_] :=  (* open rectangle of height ?, width w, centered
                         at (x, y) *)
      With[{height = .3},
        {Line[{{x-w/2, y-height/2}, {x-w/2, y+height/2}}],
         Line[{{x-w/2, y+height/2}, {x+w/2, y+height/2}}],
         Line[{{x+w/2, y+height/2}, {x+w/2, y-height/2}}],
         Line[{{x+w/2, y-height/2}, {x-w/2, y-height/2}}]}]

revtree[{x_}] := {x}
revtree[{x_, t1_, t2_}] := {x, revtree[t2], revtree[t1]}

transGraphic[Line[x_], o_] := Line[transGraphic[x, o]]
transGraphic[Disk[x_, w_], o_] := Disk[transGraphic[x, o], w]
transGraphic[Text[t_, x_], o_] := Text[t, transGraphic[x, o]]
transGraphic[{i_, j_}, o_] := {i+o, j}  /; NumberQ[i]
transGraphic[{p1__}, o_] := Map[transGraphic[#, o]&, {p1}]
transGraphic[{any_}, o_] := {transGraphic[any, o]}

enlargeDisk[Disk[x_, w_], f_] := Disk[x, w*f]
enlargeDisk[x_, f_] := x


End[]

EndPackage[]
