(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     37456,       1117]*)
(*NotebookOutlinePosition[     53105,       1514]*)
(*  CellTagsIndexPosition[     52503,       1488]*)
(*WindowFrame->Normal*)



Notebook[{
Cell["Introduction to Programming with Mathematica, 3rd Edition", "TitleBar",
  CellTags->{
  "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
    "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],

Cell[CellGroupData[{

Cell["Preface", "Chapter",
  CellTags->{
  "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
    "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],

Cell[CellGroupData[{

Cell["Technical computing environments", "PrefaceSection",
  CellTags->"IPM3.0.1"],

Cell["\<\
Computers are now an essential component of research and education in \
science, engineering, and almost all technical fields. Experimentalists \
routinely use computers to collect and analyze data while theoreticians use \
computers to manipulate equations numerically and symbolically. For both, \
computer simulation and modeling have become indispensable investigative \
tools. In response, technical education has changed to incorporate the use of \
computers into the curriculum both as a topic and as a medium for the \
presentation of technical material.\
\>", "TextFirst",
  CellTags->"IPM3.0.1"],

Cell["\<\
Advances in computer hardware have progressed to the point where researchers \
and students now have spectacularly powerful machinery at their fingertips. \
It is not uncommon for a first-year engineering student to have their own \
laptop which contains more raw computing power than most researchers could \
even dream of five years ago.\
\>", "Text",
  CellTags->"IPM3.0.1"],

Cell[TextData[{
  "Oftentimes, the spectacular gains in processing power have not been met \
with similar advances in software. It often seems as if the power and utility \
of software are inversely proportional to their ease of use. Fortunately, for \
those who do technical computing, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has long provided a fully integrated environment, including a programming \
language suitable for individuals who are not full-time programmers but who \
need to create programs to carry out their work. It includes:"
}], "Text",
  CellTags->{"i:1", "IPM3.0.1"}],

Cell["\<\
Built-in mathematical and graphical capabilities that are both powerful and \
flexible;\
\>", "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],

Cell["\<\
A programming language that can be used to extend its capabilities virtually \
without limit. That language is interactive, has the capability to perform \
both numeric and symbolic manipulation, makes broad use of pattern matching, \
and supports a functional style of programming favored by many computer \
scientists (while incorporating constructs for more conventional programming \
styles).\
\>", "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],

Cell["\<\
Extensive online help facilities that make it easy to learn about built-in \
functions and get their syntax correct.\
\>", "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],

Cell[TextData[{
  "The ability to connect ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to other computing environments and other languages."
}], "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],

Cell["\<\
A programmable interface in which all of the above elements are seamlessly \
integrated.\
\>", "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],

Cell[TextData[{
  "In this book, we focus on the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language. While there are many books, including the reference \
manual by Stephen Wolfram, ",
  StyleBox["The Mathematica Book",
    FontSlant->"Italic"],
  " (",
  ButtonBox["Wolfram 2003",
    ButtonData:>"C",
    ButtonStyle->"MainBookLink"],
  "), that discuss various aspects of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", there has been a need for a text explaining how to use the underlying \
programming language so that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\[CloseCurlyQuote]s capabilities can be fully utilized. This book explains \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\[CloseCurlyQuote]s programming language for the beginning programmer, \
first discussing the syntax and structure of the language and then focusing \
on how to use different programming styles to solve problems."
}], "Text",
  CellTags->"IPM3.0.1"]
}, Closed]],

Cell[CellGroupData[{

Cell["The audience for this book", "PrefaceSection",
  CellTags->"IPM3.0.2"],

Cell["This book was written for two distinct groups of readers:", "TextFirst",
  CellTags->"IPM3.0.2"],

Cell[TextData[{
  "Users who are familiar with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " but wish to deepen their understanding of the program and go beyond the \
most basic kinds of computations. This book introduces you to the syntax and \
structure of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language, thus helping you to see the bigger picture of how \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " works."
}], "BulletedList",
  CellTags->"IPM3.0.2"],

Cell[TextData[{
  "Those who want to learn to program and would like a friendly and useful \
language to start with. Most low-level languages, like C and Fortran, require \
many lines of complicated code in order to do something interesting, while \
many high-level languages, like Lisp, are a bit esoteric and difficult to \
master. Each of these traditional languages require an edit-compile-run cycle \
every time you wish to check your program. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", on the other hand, has a more natural syntax and provides very \
high-level built-in operations with which you can do a lot of interesting \
things right away. Since it is an interactive environment, you can prototype \
and test your code immediately, all in the same environment."
}], "BulletedList",
  CellTags->"IPM3.0.2"],

Cell[TextData[{
  "If you already have programming experience in one of the more traditional \
languages, you will often write ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programs in a procedural style, producing code that looks like \
\[OpenCurlyDoubleQuote]Fortran in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\[CloseCurlyDoubleQuote] for example. By understanding how ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programs can be constructed using the functional and rule-based styles of \
programming, much simpler and more efficient code can often be written. "
}], "Text",
  CellTags->"IPM3.0.2"]
}, Closed]],

Cell[CellGroupData[{

Cell["Changes in the new edition", "PrefaceSection",
  PageBreakAbove->True,
  CellTags->"IPM3.0.3"],

Cell[TextData[{
  "Since the last edition of this book was published, significant changes \
have occurred both in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and in its use throughout the world. Several new major versions of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " have been released (as of the publication date of this book, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " ",
  
  ValueBox["$VersionNumber"],
  " is the current version) and, with them, quite a few new functions and \
features, and some changes to existing functions. This new edition of ",
  StyleBox["An Introduction to Programming with Mathematica",
    FontSlant->"Italic"],
  " has kept pace, including information and examples throughout the book on \
the most important new features and functions."
}], "TextFirst",
  CellTags->"IPM3.0.3"],

Cell[TextData[{
  "Some of the new functions which are now discussed in this book include: ",
  StyleBox["ArrayDepth", "MR"],
  ", ",
  StyleBox["ArrayPlot", "MR"],
  ", ",
  StyleBox["ArrayQ", "MR"],
  ", ",
  StyleBox["ArrayRules", "MR"],
  ", ",
  StyleBox["BitAnd", "MR"],
  ", ",
  StyleBox["BitOr", "MR"],
  ", ",
  StyleBox["BitXor", "MR"],
  ", ",
  StyleBox["ButtonBox", "MR"],
  ", ",
  StyleBox["ButtonFunction", "MR"],
  ", ",
  StyleBox["ColumnAlignments", "MR"],
  ", ",
  StyleBox["DisplayForm", "MR"],
  ", ",
  StyleBox["Element", "MR"],
  ", ",
  StyleBox["EvaluationMonitor", "MR"],
  ", ",
  StyleBox["Export", "MR"],
  ", ",
  StyleBox["FindFit", "MR"],
  ", ",
  StyleBox["GridBox", "MR"],
  ", ",
  StyleBox["Import", "MR"],
  ", ",
  StyleBox["MakeBoxes", "MR"],
  ", ",
  StyleBox["MakeExpression", "MR"],
  ", ",
  StyleBox["Mean", "MR"],
  ", ",
  StyleBox["NestWhile", "MR"],
  ", ",
  StyleBox["NestWhileList", "MR"],
  ", ",
  StyleBox["Norm", "MR"],
  ". ",
  StyleBox["NotebookGet", "MR"],
  ", ",
  StyleBox["NotebookPut", "MR"],
  ", ",
  StyleBox["NumericQ", "MR"],
  ", ",
  StyleBox["PadLeft", "MR"],
  ", ",
  StyleBox["PadRight", "MR"],
  ", ",
  StyleBox["Pick", "MR"],
  ", ",
  StyleBox["PieceWise", "MR"],
  ", ",
  StyleBox["RegularExpression", "MR"],
  ", ",
  StyleBox["RowAlignments", "MR"],
  ", ",
  StyleBox["RowBox", "MR"],
  ", ",
  StyleBox["SelectionEvaluate", "MR"],
  ", ",
  StyleBox["SelectionMove", "MR"],
  ", ",
  StyleBox["Sequence", "MR"],
  ", ",
  StyleBox["SparseArray", "MR"],
  ", ",
  StyleBox["Split", "MR"],
  ", ",
  StyleBox["StepMonitor", "MR"],
  ", ",
  StyleBox["StringExpression", "MR"],
  ", ",
  StyleBox["StringCases", "MR"],
  ", ",
  StyleBox["StyleForm", "MR"],
  ", ",
  StyleBox["Subsets", "MR"],
  ", ",
  StyleBox["ToFileName", "MR"],
  ", ",
  StyleBox["TraditionalForm", "MR"],
  "."
}], "Text",
  CellTags->"IPM3.0.3"],

Cell[TextData[{
  "We have reorganized the structure of the chapters to make it easier to \
find information on broad topics of interest to the reader. There is a new \
chapter (Chapter 2) devoted specifically to the syntax and structure of the \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " language that has culled material from several chapters of the earlier \
edition. The old chapters on iteration and conditional definitions have been \
incorporated into a new chapter on procedural programming. Rule-based \
programming now gets its own chapter. The chapter on numerics has been \
extensively rewritten to take into account the many changes to numerics over \
the past several versions of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". The chapter on applications has also been reorganized and rewritten to \
better explain application development. And we have added an entirely new \
chapter on front end programming, something that was not feasible prior to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 3. See the section below, \[OpenCurlyDoubleQuote]Book Contents,\
\[CloseCurlyDoubleQuote] for details on each chapter."
}], "Text",
  CellTags->"IPM3.0.3"],

Cell[TextData[{
  "Throughout this new edition, many new examples have been worked into the \
text, in particular in Chapters 4, 5, 9, and 11. Exercises have been expanded \
over the previous edition, as have the solutions to the exercises. In \
addition to the printed solutions to exercises at the end of this book, they \
are also available as a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook online in several locations (see below)."
}], "Text",
  CellTags->"IPM3.0.3"],

Cell["\<\
Readers of the first two editions found a handful of mistakes, which have \
been corrected in this edition. We welcome further communication from our \
readers.\
\>", "Text",
  CellTags->"IPM3.0.3"]
}, Closed]],

Cell[CellGroupData[{

Cell["How to use this book", "PrefaceSection",
  CellTags->"IPM3.0.4"],

Cell["There are several ways in which this book should be useful:", \
"TextFirst",
  CellTags->"IPM3.0.4"],

Cell[TextData[{
  "As a primary text in an introductory course on programming. Perhaps the \
most obvious use would be in an introductory computer science course for \
students in science, engineering, or mathematics, who would solve technical \
problems of interest to them, while at the same time learning to program. The \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language supports many programming styles that are present in \
other programming languages, so programming skills learned using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can be readily transferred to these languages."
}], "BulletedList",
  CellTags->"IPM3.0.4"],

Cell[TextData[{
  "As a supplemental text in a course in which ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is being used as a tool for studying another technical subject. In these \
kinds of courses, the principles of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming can be introduced as needed during the course, although we \
have found that a structured introduction to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming at the beginning of the semester works best."
}], "BulletedList",
  CellTags->"IPM3.0.4"],

Cell[TextData[{
  "As a self-study book, particularly for ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " users who need to use its programming capabilities more fully or who are \
interested in understanding how the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language works."
}], "BulletedList",
  CellTags->"IPM3.0.4"],

Cell[TextData[{
  "As educators, we strongly believe in the axiom that one learns by doing. \
Hence we have not only included hundreds of exercises to this book, but have \
tried hard to make sure that the exercises are interesting and instructive. \
You will learn how to program in the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " language best by trying to solve problems. We strongly encourage you to \
try the examples in the text on your own, modifying them as you see fit, but \
also you should try to do as many of the exercises at the end of each section \
as you can. Quite a few concepts and features are explained in the exercises \
and their solutions so it is well worthwhile spending some time on those \
aspects of the book."
}], "Text",
  CellTags->"IPM3.0.4"],

Cell[TextData[{
  "When you work with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", you will generally find that you work with several different programming \
styles, deal with numerical issues as well as symbolics, crunch numbers but \
also use graphics, etc. Although topics such as functional programming, \
procedural programming, and rule-based programming are presented in separate \
chapters, you should not view such a division as ironclad. In fact, it is \
merely a convenience to enable us to discuss the various topics that make up \
the body of this book. We have tried to make the materials self-contained and \
have everything flow logically from beginning to end, but in some instances, \
this is quite difficult and, in fact, arbitrary. After digesting the first \
two or three chapters, you should feel free to jump around from chapter to \
chapter as you work through the materials in this book."
}], "Text",
  CellTags->"IPM3.0.4"]
}, Closed]],

Cell[CellGroupData[{

Cell["Conventions", "PrefaceSection",
  CellTags->"IPM3.0.5"],

Cell[TextData[{
  "All input and output in this book appear in a different font from the \
regular text. This is true for examples that appear in the middle of text, \
such as ",
  StyleBox["Expand[(a + b)^4]", "MR"],
  ", as well as displayed ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " code. So, for example, lines of input (what you type at your computer) \
appear as"
}], "TextFirst",
  CellTags->"IPM3.0.5"],

Cell[BoxData[
    \(3 + 5\)], "Input",
  CellLabel->"In[1]:=",
  CellLabelAutoDelete->False,
  CellTags->"IPM3.0.5"],

Cell[TextData[{
  "whereas all output (what ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " prints on your computer screen) appears in a slightly lighter font than \
the input."
}], "TextFirst",
  CellTags->"IPM3.0.5"],

Cell[BoxData[
    \(8\)], "Output",
  CellLabel->"Out[1]=",
  CellLabelAutoDelete->False,
  CellTags->"IPM3.0.5"],

Cell[TextData[{
  "You do not have to type the prompts ",
  StyleBox["In[1]:=", "SO10"],
  " or ",
  StyleBox["Out[1]=", "SO10"],
  "; ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will do that for you automatically."
}], "Text",
  CellTags->"IPM3.0.5"],

Cell[TextData[{
  "All of the programs that are defined in this book can be located in the \
index under the heading ",
  StyleBox["Programs",
    FontWeight->"Bold"],
  ". So, for example, the function ",
  StyleBox["runEncode", "MR"],
  " defined in Section 7.3 is listed in the index under ",
  StyleBox["Programs",
    FontWeight->"Bold"],
  ", ",
  StyleBox["runEncode", "MR"],
  "."
}], "Text",
  CellTags->"IPM3.0.5"]
}, Closed]],

Cell[CellGroupData[{

Cell["Book contents", "PrefaceSection",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "We should perhaps start with what is ",
  StyleBox["not",
    FontSlant->"Italic"],
  " in this book, namely, a complete list and explanation of the hundreds of \
operations built into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". For that list, the indispensable reference is ",
  StyleBox["The Mathematica Book",
    FontSlant->"Italic"],
  ", (",
  ButtonBox["Wolfram, 2003",
    ButtonData:>"C",
    ButtonStyle->"MainBookLink"],
  ")."
}], "TextFirst",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "This book is also not a complete or advanced treatment of programming in \
",
  StyleBox["Mathematica,",
    FontSlant->"Italic"],
  " nor is it a book about using ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "to solve problems from one particular discipline. So, for example, the \
reader will not find a discussion of object-oriented programming in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", nor of databases, nor a formal discussion of data types. Fortunately, \
there are other excellent sources of information on such topics, including \
books such as ",
  StyleBox["The Mathematica Programmer I",
    FontSlant->"Italic"],
  " and ",
  StyleBox["II",
    FontSlant->"Italic"],
  ", or ",
  StyleBox["Computer Science with Mathematica",
    FontSlant->"Italic"],
  StyleBox[": Theory and Practice for Science, Mathematics, and Engineering",
    FontSlant->"Italic"],
  ", all by Roman Maeder."
}], "Text",
  CellTags->{"i:2", "IPM3.0.6"}],

Cell[TextData[{
  "This book is about teaching you how to program with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". We assume that you have little, if any, programming experience (though \
you may already be a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " user), and we take you step by step through the various programming \
styles that are available in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and explain when to use what style."
}], "Text",
  CellTags->"IPM3.0.6"],

Cell["The chapter structure is as follows:", "Text",
  PageBreakAbove->True,
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 1 An Introduction to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". For the reader who has little or no knowledge of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", we have included a brief overview of some of its built-in capabilities, \
including graphics and numerical and symbolic computations. In addition, this \
chapter contains some of the fundamentals of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", such as how to use the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " interface and how to enter expressions. Experienced ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " users should feel free to skim this material."
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 2 The ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "Language. This chapter gives an introduction to the syntax and structure \
of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language. It introduces the basic construct in the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " language, an expression, and then discusses some of the building blocks \
that you will use over and over again in constructing your programs."
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell["\<\
Chapter 3 Lists. The heart of the book begins here, with a discussion of this \
most important of data types. Aside from numbers, no other data type is more \
useful in programming. We describe the use of built-in functions to create \
and manipulate lists. This chapter also discusses strings and characters as \
their manipulation is so similar to that for lists.\
\>", "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 4 Functional Programming. This chapter introduces functional \
programming, a style quite distinct from what is available in traditional \
languages. We discuss nested function calls, iterating functions, and \
introduce some of the more important higher-order functions, such as ",
  StyleBox["Map", "MR"],
  ", ",
  StyleBox["Apply", "MR"],
  ", and ",
  StyleBox["Thread", "MR"],
  ". A section on pure functions introduces this important construct borrowed \
from the lambda calculus. The chapter concludes with a section demonstrating \
how to construct \[OpenCurlyDoubleQuote]one-liners,\[CloseCurlyDoubleQuote] \
short programs that solve a particular problem in one line."
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 5 Procedural Programming. Procedures can be thought of as recipes. \
They spell out a sequence of steps to follow. They often include control \
structures that determine what action to take depending upon some property of \
an input. This programming style is more characteristic of conventional \
languages like C than it is of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", but is still important. This chapter introduces the concepts of loops, \
iteration, flow control, and conditional definitions, using some traditional \
problems such as Newton\[CloseCurlyQuote]s method and the Sieve of \
Eratosthenes to demonstrate these concepts."
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell["\<\
Chapter 6 Rule-Based Programming. This chapter, new to the third edition of \
this book, explains how rules can be used to transform expressions. Blanks, \
pattern matching, string patterns, and alternatives are discussed extensively \
and some classical problems from computer science are examined using \
rule-based approaches: encoding and sorting.\
\>", "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 7 Recursion. This method of programming, in which a function is \
defined in terms of itself, is heavily used in mathematics and computer \
science and has a natural implementation in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". For those with little experience programming recursively, we give a \
gentle introduction to thinking recursively and then use several well-known \
problems such as merge sort, Gaussian elimination, tree representation, and \
Huffman encoding to implement the ideas in this chapter. The chapter also \
includes discussions of dynamic programming (or caching)."
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 8 Numerics. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " contains a variety of types of numbers, both exact and approximate. This \
chapter discusses working with exact vs. inexact numbers and \
machine-precision vs. high-precision numbers. Efficiency issues are also \
presented including a discussion of packed arrays and sparse arrays, two data \
formats that are optimized for both speed and memory. The chapter concludes \
with a discussion of how to write programs that best take advantage of the \
numerical capabilities in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 9 Graphics Programming. This chapter introduces the basic concepts \
of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " graphics. It uses some of the techniques of the previous chapters to \
create graphics-based programs and to solve problems that are inherently \
graphical in nature. It also demonstrates how to include options in your \
programs."
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 10 Front End Programming. The front end is the graphical user \
interface to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Beginning in Version 3, many aspects of the front end became \
programmable and this functionality was further extended in Versions 4 and 5. \
This new chapter introduces the structure of the objects in the front end \
that can be manipulated using the techniques discussed in earlier chapters. \
It includes sections on notebooks and cells, cell data types, box structures, \
and buttons (a special kind of box). "
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 11 Examples and Applications. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is used to solve larger-scale programming problems. Included are an \
application to manipulate and visualize data, as well as ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " implementations of the Game of Life using functional and rule-based \
programming, random walks, and finally the creation of a mini \
picture-description language. Options and default values and documentation \
implementations are also discussed."
}], "BulletedList",
  CellTags->"IPM3.0.6"],

Cell[TextData[{
  "Chapter 12 Writing Packages. The last chapter introduces the method of \
organizing libraries of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions into convenient units called packages. Discussion of options, \
error-trapping, and messages are included."
}], "BulletedList",
  CellTags->"IPM3.0.6"]
}, Closed]],

Cell[CellGroupData[{

Cell["Supplemental packages and solutions", "PrefaceSection",
  CellTags->"IPM3.0.7"],

Cell[TextData[{
  "Materials for this book have been made available in electronic form. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebooks containing most of the examples, exercises, and solutions are \
included in an archive appropriate for several platforms: IPM3.zip for \
Windows, IPM3.tgz for Unix, and IPM3.hqx for Macintosh OS X. The file names \
correspond directly to the chapter structure. So, for example, the notebook \
for the first chapter is 01Introduction.nb."
}], "TextFirst",
  CellTags->{"i:3", "IPM3.0.7"}],

Cell[TextData[{
  "The archive also contains files with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programs that can be loaded into your ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " sessions as needed. These files (normally called packages), have a \
filename extension ending with ",
  StyleBox[".m", "MR"],
  ". "
}], "Text",
  CellTags->"IPM3.0.7"],

Cell["Archives for this book have been placed in two locations:", "Text",
  CellTags->"IPM3.0.7"],

Cell[TextData[{
  "Cambridge University Press website: ",
  ButtonBox["www.cambridge.org/0521846781",
    ButtonData:>{
      URL[ "http://www.cambridge.org/0521846781"], None},
    ButtonStyle->"Hyperlink"]
}], "BulletedList",
  CellTags->"IPM3.0.7"],

Cell[TextData[{
  "Wolfram Research website: ",
  ButtonBox["library.wolfram.com/infocenter/Books/5169/",
    ButtonData:>{
      URL[ "http://library.wolfram.com/infocenter/Books/5169/ "], None},
    ButtonStyle->"Hyperlink"]
}], "BulletedList",
  CellTags->"IPM3.0.7"],

Cell[TextData[{
  "The recommended way to use the notebooks and packages is to first unpack \
the archive on your computer. This will leave you with two \
directories/folders, one called IPM3 and the other RandomWalks. Move each of \
these folders and their contents inside one of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Applications directories on your computer. The recommended location is \
inside either ",
  StyleBox["$BaseDirectory", "MR"],
  " or ",
  StyleBox["$UserBaseDirectory", "MR"],
  " as files in these two directories will continue to be found even after \
you upgrade ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->{"i:4", "IPM3.0.7"}],

Cell["\<\
Here is the recommended location for where to install applications. The \
output will look different on different operating systems; the following was \
run in Windows XP.\
\>", "Text",
  CellTags->"IPM3.0.7"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ToFileName[{$BaseDirectory, "\<Applications\>"}]\)], "Input",
  CellTags->"IPM3.0.7"],

Cell[BoxData[
    \("C:\\Documents and Settings\\All Users\\Application \
Data\\Mathematica\\Applications\\"\)], "Output",
  CellTags->"IPM3.0.7"]
}, Closed]],

Cell[TextData[{
  "Once you have installed the directories in the above location, start up ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and, from the ",
  StyleBox["Help", "SR"],
  " menu, select ",
  StyleBox["Rebuild Help Index", "SR"],
  ". Once this is done, you will find the notebooks in the Help Browser by \
choosing the Add-ons & Links category."
}], "Text",
  CellTags->"IPM3.0.7"],

Cell[TextData[{
  "To load packages from the book, you need only to specify the directory and \
package name. For example, this loads the package ",
  StyleBox["RandomWalks.m", "MR"],
  ":"
}], "Text",
  CellTags->{"i:5", "IPM3.0.7"}],

Cell[BoxData[
    \(<< IPM3`RandomWalks`\)], "Input",
  CellTags->"IPM3.0.7"]
}, Closed]],

Cell[CellGroupData[{

Cell["Further information", "PrefaceSection",
  CellTags->"IPM3.0.8"],

Cell[TextData[{
  "While the basic aspects of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming are discussed in this book, there are a great many more \
things that can be said about ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language. The most comprehensive reference source is the \
manual that comes with each copy of the software: ",
  StyleBox["The Mathematica Book",
    FontSlant->"Italic"],
  " (",
  ButtonBox["Wolfram, 2003",
    ButtonData:>"C",
    ButtonStyle->"MainBookLink"],
  ")."
}], "TextFirst",
  CellTags->"IPM3.0.8"],

Cell[TextData[{
  "The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Information Center",
  " (",
  ButtonBox["library.wolfram.com/infocenter",
    ButtonData:>{
      URL[ "http://library.wolfram.com/infocenter"], None},
    ButtonStyle->"Hyperlink"],
  ") contains thousands of programming examples and notebooks from all areas \
of science and engineering. It is organized by collection (articles, \
courseware, demos, etc.), by subject, or you can simply search on any phrase \
that you wish. "
}], "Text",
  CellTags->{"i:6", "IPM3.0.8"}],

Cell[TextData[{
  "An Internet newsgroup exists that is devoted to all aspects of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ": ",
  StyleBox["comp.soft-sys.math.mathematica", "Program"],
  ". This moderated newsgroup has been in existence for many years and users \
have come to rely on the collective wisdom of the hundreds of contributors \
from around the world who respond to any question that is posed. An archive \
going back to 1989 exists and can easily be browsed or searched in a web \
browser by going to ",
  ButtonBox["forums.wolfram.com/mathgroup/",
    ButtonData:>{"forums.wolfram.com/mathgroup/", None},
    ButtonStyle->"Hyperlink"],
  "."
}], "Text",
  CellTags->{"i:7", "IPM3.0.8"}],

Cell[TextData[{
  StyleBox["The Mathematica Journal",
    FontSlant->"Italic"],
  " is a quarterly journal that publishes articles about all aspects of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". For subscription information, or to view issues online, go to the \
website ",
  ButtonBox["www.mathematica-journal.com",
    ButtonData:>{
      URL[ "http://www.mathematica-journal.com"], None},
    ButtonStyle->"Hyperlink"],
  ". "
}], "Text",
  CellTags->{"i:8", "IPM3.0.8"}],

Cell[TextData[{
  "Finally, there are now over 300 books about ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " or that use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to teach a subject. Some of these can be found in the bibliography at the \
end of this text. See ",
  ButtonBox["store.wolfram.com/catalog/books/ f",
    ButtonData:>{
      URL[ "http://store.wolfram.com/catalog/books/"], None},
    ButtonStyle->"Hyperlink"],
  "or a complete up-to-date publications list."
}], "Text",
  CellTags->"IPM3.0.8"]
}, Closed]],

Cell[CellGroupData[{

Cell["Colophon", "PrefaceSection",
  CellTags->"IPM3.0.9"],

Cell[TextData[{
  "The third edition of this book was produced from original ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebooks. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " 5 and 5.1 were used throughout. A ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " style sheet was customized containing page layout information consistent \
with the page layout as specified by a book designer. The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebooks were output to PostScript files and then distilled to PDF files \
which were sent to the printer electronically."
}], "TextFirst",
  CellTags->"IPM3.0.9"]
}, Closed]],

Cell[CellGroupData[{

Cell["Acknowledgments", "PrefaceSection",
  PageBreakAbove->True,
  CellTags->"IPM3.0.10"],

Cell[TextData[{
  "Many people have contributed to our understanding of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language over the years. At Wolfram Research these include \
Lou D\[CloseCurlyQuote]Andria, Harry Calkins, Andy Hunt, Rob Knapp, Dan \
Lichtblau, John Novak, Robby Villegas, and Dave Withoff. Also Roman Maeder \
has answered several esoteric questions about the patterns and the ordering \
of rules in ",
  StyleBox["Mathematica.",
    FontSlant->"Italic"]
}], "TextFirst",
  CellTags->{"i:9", "IPM3.0.10"}],

Cell[TextData[{
  "Thanks to Dana Scott for permission to use his material on sorting \
(Chapter 6) and Klaus Sutner for permission to use his material on encryption \
(also in Chapter 6) as well as interesting discussions on the pattern matcher \
in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". "
}], "Text",
  CellTags->{"i:10", "IPM3.0.10"}],

Cell["\<\
Paul Wellin would like to thank his wife Sheri for her support, \
understanding, and sense of humor throughout this project.\
\>", "Text",
  CellTags->"IPM3.0.10"],

Cell[TextData[{
  "Richard J. Gaylord would like to thank Shawn Sheridan who was his guru for \
both ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and the Macintosh."
}], "Text",
  CellTags->"IPM3.0.10"],

Cell["\<\
Samuel Kamin would like to acknowledge the Computer Science Department of the \
University of Illinois for its excellent working environment and the support \
of his colleagues there. Above all, he would like to thank Judy and Rebecca \
for their love and patience.\
\>", "Text",
  CellTags->"IPM3.0.10"],

Cell["\<\
Paul R. Wellin
wellin@wolfram.com\
\>", "TextFirst",
  CellMargins->{{Inherited, Inherited}, {0, 12}},
  CellTags->"IPM3.0.10"],

Cell["\<\
Richard J. Gaylord
gaylord@uiuc.edu\
\>", "TextFirst",
  CellMargins->{{Inherited, Inherited}, {0, 12}},
  CellTags->"IPM3.0.10"],

Cell["\<\
Samuel N. Kamin
kamin@cs.uiuc.edu\
\>", "TextFirst",
  CellMargins->{{Inherited, Inherited}, {0, 12}},
  CellTags->"IPM3.0.10"],

Cell[TextData[StyleBox["Ocotober 2004",
  FontSlant->"Italic"]], "TextFirst",
  CellMargins->{{Inherited, Inherited}, {0, 12}},
  CellTags->"IPM3.0.10"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 685}},
ScreenStyleEnvironment->"Working",
PrintingStyleEnvironment->"Production",
WindowSize->{580, 583},
WindowMargins->{{64, Automatic}, {Automatic, -4}},
PrintingCopies->1,
PrintingStartingPageNumber->11,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Cell[ 
        TextData[ {
          CounterBox[ "Page", CounterFunction :> RomanNumeral]}], 
        "PageNumber"], Inherited, Cell[ 
        TextData[ {"An Introduction to Programming with ", 
          StyleBox[ "Mathematica", "IT"]}], "Header"]}, {Cell[ 
        TextData[ {"Preface"}], "Header"], Inherited, Cell[ 
        TextData[ {
          CounterBox[ "Page", CounterFunction :> RomanNumeral]}], 
        "PageNumber"]}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
TaggingRules:>{"NextIndexingNumber" -> 11, "IndexEntries" -> {{"i:10", \
"Scott, Dana"}, {"i:10", "Sutner, Klaus"}, {"i:2", "Maeder, Roman"}, {"i:6", \
{
  StyleBox[ 
  "Mathematica", FontSlant -> "Italic"], " Information Center"}}, {"i:8", 
  StyleBox[ 
  "The Mathematica Journal", FontSlant -> 
    "Italic"]}, {"i:9", "Calkins, Harry"}, {"i:9", "Hunt, Andy"}, {"i:9", \
"Novak, John"}, {"i:1", 
  StyleBox[ 
  "Mathematica", FontSlant -> 
    "Italic"], "features"}, {"i:3", "IPM3 packages", "where to find"}, \
{"i:4", "IPM3 packages", "how to install"}, {"i:5", "IPM3 packages", "how to \
load"}, {"i:7", {
  StyleBox[ 
  "Mathematica", FontSlant -> 
    "Italic"], " newsgroup"}, "", ""}, {"i:9", "D\[CloseCurlyQuote]Andria, \
Lou", "", ""}, {"i:9", "Knapp, Rob", "", ""}, {"i:9", "Lichtblau, Dan", "", \
""}, {"i:9", "Maeder, Roman", "", ""}, {"i:9", "Villegas, Robby", "", ""}, \
{"i:9", "Withoff, Dave", "", ""}}, "PaginationCache" -> {{2004, 9, 2, 16, 55, \
31.8281344`9.255386087885707}, {1, 1, 1, 1, 1, 1, 1, 1, 1, 2, 2, 2, 2, 2, 2, \
2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 5, 5, 5, \
5, 5, 5, 5, 6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, \
8, 8, 8, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 10, 10, 10, 10, 10, 10, \
10, 10, 10}, Automatic}},
Magnification->1,
StyleDefinitions -> "IPM3Styles.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "IPM3.0.1"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[2200, 65, 82, 1, 48, "PrefaceSection",
      CellTags->"IPM3.0.1"],
    Cell[2285, 68, 615, 10, 128, "TextFirst",
      CellTags->"IPM3.0.1"],
    Cell[2903, 80, 387, 7, 90, "Text",
      CellTags->"IPM3.0.1"],
    Cell[3293, 89, 607, 11, 109, "Text",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[3903, 102, 152, 4, 31, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[4058, 108, 461, 8, 107, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[4522, 118, 181, 4, 50, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[4706, 124, 207, 6, 31, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[4916, 132, 153, 4, 31, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[5072, 138, 1007, 27, 128, "Text",
      CellTags->"IPM3.0.1"]},
  "IPM3.0.2"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[6116, 170, 76, 1, 34, "PrefaceSection",
      CellTags->"IPM3.0.2"],
    Cell[6195, 173, 102, 1, 33, "TextFirst",
      CellTags->"IPM3.0.2"],
    Cell[6300, 176, 519, 15, 88, "BulletedList",
      CellTags->"IPM3.0.2"],
    Cell[6822, 193, 834, 14, 164, "BulletedList",
      CellTags->"IPM3.0.2"],
    Cell[7659, 209, 646, 15, 90, "Text",
      CellTags->"IPM3.0.2"]},
  "IPM3.0.3"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[8342, 229, 100, 2, 34, "PrefaceSection",
      PageBreakAbove->True,
      CellTags->"IPM3.0.3"],
    Cell[8445, 233, 859, 21, 128, "TextFirst",
      CellTags->"IPM3.0.3"],
    Cell[9307, 256, 1909, 93, 166, "Text",
      CellTags->"IPM3.0.3"],
    Cell[11219, 351, 1209, 23, 204, "Text",
      CellTags->"IPM3.0.3"],
    Cell[12431, 376, 494, 10, 90, "Text",
      CellTags->"IPM3.0.3"],
    Cell[12928, 388, 208, 5, 52, "Text",
      CellTags->"IPM3.0.3"]},
  "IPM3.0.4"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[13173, 398, 70, 1, 34, "PrefaceSection",
      CellTags->"IPM3.0.4"],
    Cell[13246, 401, 106, 2, 33, "TextFirst",
      CellTags->"IPM3.0.4"],
    Cell[13355, 405, 676, 14, 126, "BulletedList",
      CellTags->"IPM3.0.4"],
    Cell[14034, 421, 570, 14, 88, "BulletedList",
      CellTags->"IPM3.0.4"],
    Cell[14607, 437, 359, 10, 69, "BulletedList",
      CellTags->"IPM3.0.4"],
    Cell[14969, 449, 791, 14, 147, "Text",
      CellTags->"IPM3.0.4"],
    Cell[15763, 465, 964, 16, 185, "Text",
      CellTags->"IPM3.0.4"]},
  "IPM3.0.5"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[16764, 486, 61, 1, 34, "PrefaceSection",
      CellTags->"IPM3.0.5"],
    Cell[16828, 489, 427, 11, 71, "TextFirst",
      CellTags->"IPM3.0.5"],
    Cell[17258, 502, 116, 4, 30, "Input",
      CellTags->"IPM3.0.5"],
    Cell[17377, 508, 229, 7, 52, "TextFirst",
      CellTags->"IPM3.0.5"],
    Cell[17609, 517, 113, 4, 29, "Output",
      CellTags->"IPM3.0.5"],
    Cell[17725, 523, 266, 10, 52, "Text",
      CellTags->"IPM3.0.5"],
    Cell[17994, 535, 424, 14, 71, "Text",
      CellTags->"IPM3.0.5"]},
  "IPM3.0.6"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[18455, 554, 63, 1, 34, "PrefaceSection",
      CellTags->"IPM3.0.6"],
    Cell[18521, 557, 514, 17, 71, "TextFirst",
      CellTags->"IPM3.0.6"],
    Cell[19038, 576, 992, 27, 128, "Text",
      CellTags->{"i:2", "IPM3.0.6"}],
    Cell[20033, 605, 514, 14, 90, "Text",
      CellTags->"IPM3.0.6"],
    Cell[20550, 621, 100, 2, 33, "Text",
      PageBreakAbove->True,
      CellTags->"IPM3.0.6"],
    Cell[20653, 625, 759, 20, 107, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[21415, 647, 542, 14, 88, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[21960, 663, 421, 7, 88, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[22384, 672, 751, 15, 126, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[23138, 689, 725, 13, 145, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[23866, 704, 407, 7, 88, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[24276, 713, 675, 12, 126, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[24954, 727, 676, 15, 126, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[25633, 744, 433, 10, 88, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[26069, 756, 623, 12, 107, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[26695, 770, 593, 13, 107, "BulletedList",
      CellTags->"IPM3.0.6"],
    Cell[27291, 785, 336, 8, 69, "BulletedList",
      CellTags->"IPM3.0.6"]},
  "IPM3.0.7"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[27664, 798, 85, 1, 34, "PrefaceSection",
      CellTags->"IPM3.0.7"],
    Cell[27752, 801, 544, 10, 109, "TextFirst",
      CellTags->{"i:3", "IPM3.0.7"}],
    Cell[28299, 813, 381, 12, 52, "Text",
      CellTags->"IPM3.0.7"],
    Cell[28683, 827, 97, 1, 33, "Text",
      CellTags->"IPM3.0.7"],
    Cell[28783, 830, 251, 7, 31, "BulletedList",
      CellTags->"IPM3.0.7"],
    Cell[29037, 839, 270, 7, 31, "BulletedList",
      CellTags->"IPM3.0.7"],
    Cell[29310, 848, 714, 18, 109, "Text",
      CellTags->{"i:4", "IPM3.0.7"}],
    Cell[30027, 868, 219, 5, 52, "Text",
      CellTags->"IPM3.0.7"],
    Cell[30271, 877, 105, 2, 30, "Input",
      CellTags->"IPM3.0.7"],
    Cell[30379, 881, 146, 3, 48, "Output",
      CellTags->"IPM3.0.7"],
    Cell[30540, 887, 405, 11, 71, "Text",
      CellTags->"IPM3.0.7"],
    Cell[30948, 900, 234, 6, 52, "Text",
      CellTags->{"i:5", "IPM3.0.7"}],
    Cell[31185, 908, 77, 2, 30, "Input",
      CellTags->"IPM3.0.7"]},
  "IPM3.0.8"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[31299, 915, 69, 1, 34, "PrefaceSection",
      CellTags->"IPM3.0.8"],
    Cell[31371, 918, 655, 21, 90, "TextFirst",
      CellTags->"IPM3.0.8"],
    Cell[32029, 941, 556, 15, 71, "Text",
      CellTags->{"i:6", "IPM3.0.8"}],
    Cell[32588, 958, 717, 16, 109, "Text",
      CellTags->{"i:7", "IPM3.0.8"}],
    Cell[33308, 976, 491, 14, 71, "Text",
      CellTags->{"i:8", "IPM3.0.8"}],
    Cell[33802, 992, 538, 15, 71, "Text",
      CellTags->"IPM3.0.8"]},
  "IPM3.0.9"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[34377, 1012, 58, 1, 34, "PrefaceSection",
      CellTags->"IPM3.0.9"],
    Cell[34438, 1015, 655, 17, 90, "TextFirst",
      CellTags->"IPM3.0.9"]},
  "IPM3.0.10"->{
    Cell[1754, 51, 220, 3, 29, "TitleBar",
      CellTags->{
      "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
        "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],
    Cell[35130, 1037, 90, 2, 34, "PrefaceSection",
      PageBreakAbove->True,
      CellTags->"IPM3.0.10"],
    Cell[35223, 1041, 554, 12, 90, "TextFirst",
      CellTags->{"i:9", "IPM3.0.10"}],
    Cell[35780, 1055, 359, 9, 71, "Text",
      CellTags->{"i:10", "IPM3.0.10"}],
    Cell[36142, 1066, 173, 4, 52, "Text",
      CellTags->"IPM3.0.10"],
    Cell[36318, 1072, 216, 7, 52, "Text",
      CellTags->"IPM3.0.10"],
    Cell[36537, 1081, 314, 6, 71, "Text",
      CellTags->"IPM3.0.10"],
    Cell[36854, 1089, 137, 5, 50, "TextFirst",
      CellTags->"IPM3.0.10"],
    Cell[36994, 1096, 139, 5, 50, "TextFirst",
      CellTags->"IPM3.0.10"],
    Cell[37136, 1103, 137, 5, 50, "TextFirst",
      CellTags->"IPM3.0.10"],
    Cell[37276, 1110, 152, 3, 31, "TextFirst",
      CellTags->"IPM3.0.10"]},
  "c:1"->{
    Cell[1999, 58, 176, 3, 96, "Chapter",
      CellTags->{
      "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
        "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}]},
  "i:1"->{
    Cell[3293, 89, 607, 11, 109, "Text",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[3903, 102, 152, 4, 31, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[4058, 108, 461, 8, 107, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[4522, 118, 181, 4, 50, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[4706, 124, 207, 6, 31, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}],
    Cell[4916, 132, 153, 4, 31, "BulletedList",
      CellTags->{"i:1", "IPM3.0.1"}]},
  "i:2"->{
    Cell[19038, 576, 992, 27, 128, "Text",
      CellTags->{"i:2", "IPM3.0.6"}]},
  "i:3"->{
    Cell[27752, 801, 544, 10, 109, "TextFirst",
      CellTags->{"i:3", "IPM3.0.7"}]},
  "i:4"->{
    Cell[29310, 848, 714, 18, 109, "Text",
      CellTags->{"i:4", "IPM3.0.7"}]},
  "i:5"->{
    Cell[30948, 900, 234, 6, 52, "Text",
      CellTags->{"i:5", "IPM3.0.7"}]},
  "i:6"->{
    Cell[32029, 941, 556, 15, 71, "Text",
      CellTags->{"i:6", "IPM3.0.8"}]},
  "i:7"->{
    Cell[32588, 958, 717, 16, 109, "Text",
      CellTags->{"i:7", "IPM3.0.8"}]},
  "i:8"->{
    Cell[33308, 976, 491, 14, 71, "Text",
      CellTags->{"i:8", "IPM3.0.8"}]},
  "i:9"->{
    Cell[35223, 1041, 554, 12, 90, "TextFirst",
      CellTags->{"i:9", "IPM3.0.10"}]},
  "i:10"->{
    Cell[35780, 1055, 359, 9, 71, "Text",
      CellTags->{"i:10", "IPM3.0.10"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"IPM3.0.1", 40239, 1180},
  {"IPM3.0.2", 41462, 1209},
  {"IPM3.0.3", 42262, 1228},
  {"IPM3.0.4", 43152, 1250},
  {"IPM3.0.5", 44109, 1273},
  {"IPM3.0.6", 45045, 1296},
  {"IPM3.0.7", 46823, 1340},
  {"IPM3.0.8", 48219, 1375},
  {"IPM3.0.9", 49107, 1396},
  {"IPM3.0.10", 49686, 1409},
  {"c:1", 50904, 1439},
  {"i:1", 51121, 1444},
  {"i:2", 51643, 1457},
  {"i:3", 51736, 1460},
  {"i:4", 51834, 1463},
  {"i:5", 51927, 1466},
  {"i:6", 52018, 1469},
  {"i:7", 52110, 1472},
  {"i:8", 52203, 1475},
  {"i:9", 52295, 1478},
  {"i:10", 52395, 1481}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 220, 3, 29, "TitleBar",
  CellTags->{
  "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", "IPM3.0.6", 
    "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],

Cell[CellGroupData[{
Cell[1999, 58, 176, 3, 96, "Chapter",
  CellTags->{
  "c:1", "IPM3.0.1", "IPM3.0.2", "IPM3.0.3", "IPM3.0.4", "IPM3.0.5", 
    "IPM3.0.6", "IPM3.0.7", "IPM3.0.8", "IPM3.0.9", "IPM3.0.10"}],

Cell[CellGroupData[{
Cell[2200, 65, 82, 1, 48, "PrefaceSection",
  CellTags->"IPM3.0.1"],
Cell[2285, 68, 615, 10, 128, "TextFirst",
  CellTags->"IPM3.0.1"],
Cell[2903, 80, 387, 7, 90, "Text",
  CellTags->"IPM3.0.1"],
Cell[3293, 89, 607, 11, 109, "Text",
  CellTags->{"i:1", "IPM3.0.1"}],
Cell[3903, 102, 152, 4, 31, "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],
Cell[4058, 108, 461, 8, 107, "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],
Cell[4522, 118, 181, 4, 50, "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],
Cell[4706, 124, 207, 6, 31, "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],
Cell[4916, 132, 153, 4, 31, "BulletedList",
  CellTags->{"i:1", "IPM3.0.1"}],
Cell[5072, 138, 1007, 27, 128, "Text",
  CellTags->"IPM3.0.1"]
}, Closed]],

Cell[CellGroupData[{
Cell[6116, 170, 76, 1, 34, "PrefaceSection",
  CellTags->"IPM3.0.2"],
Cell[6195, 173, 102, 1, 33, "TextFirst",
  CellTags->"IPM3.0.2"],
Cell[6300, 176, 519, 15, 88, "BulletedList",
  CellTags->"IPM3.0.2"],
Cell[6822, 193, 834, 14, 164, "BulletedList",
  CellTags->"IPM3.0.2"],
Cell[7659, 209, 646, 15, 90, "Text",
  CellTags->"IPM3.0.2"]
}, Closed]],

Cell[CellGroupData[{
Cell[8342, 229, 100, 2, 34, "PrefaceSection",
  PageBreakAbove->True,
  CellTags->"IPM3.0.3"],
Cell[8445, 233, 859, 21, 128, "TextFirst",
  CellTags->"IPM3.0.3"],
Cell[9307, 256, 1909, 93, 166, "Text",
  CellTags->"IPM3.0.3"],
Cell[11219, 351, 1209, 23, 204, "Text",
  CellTags->"IPM3.0.3"],
Cell[12431, 376, 494, 10, 90, "Text",
  CellTags->"IPM3.0.3"],
Cell[12928, 388, 208, 5, 52, "Text",
  CellTags->"IPM3.0.3"]
}, Closed]],

Cell[CellGroupData[{
Cell[13173, 398, 70, 1, 34, "PrefaceSection",
  CellTags->"IPM3.0.4"],
Cell[13246, 401, 106, 2, 33, "TextFirst",
  CellTags->"IPM3.0.4"],
Cell[13355, 405, 676, 14, 126, "BulletedList",
  CellTags->"IPM3.0.4"],
Cell[14034, 421, 570, 14, 88, "BulletedList",
  CellTags->"IPM3.0.4"],
Cell[14607, 437, 359, 10, 69, "BulletedList",
  CellTags->"IPM3.0.4"],
Cell[14969, 449, 791, 14, 147, "Text",
  CellTags->"IPM3.0.4"],
Cell[15763, 465, 964, 16, 185, "Text",
  CellTags->"IPM3.0.4"]
}, Closed]],

Cell[CellGroupData[{
Cell[16764, 486, 61, 1, 34, "PrefaceSection",
  CellTags->"IPM3.0.5"],
Cell[16828, 489, 427, 11, 71, "TextFirst",
  CellTags->"IPM3.0.5"],
Cell[17258, 502, 116, 4, 30, "Input",
  CellTags->"IPM3.0.5"],
Cell[17377, 508, 229, 7, 52, "TextFirst",
  CellTags->"IPM3.0.5"],
Cell[17609, 517, 113, 4, 29, "Output",
  CellTags->"IPM3.0.5"],
Cell[17725, 523, 266, 10, 52, "Text",
  CellTags->"IPM3.0.5"],
Cell[17994, 535, 424, 14, 71, "Text",
  CellTags->"IPM3.0.5"]
}, Closed]],

Cell[CellGroupData[{
Cell[18455, 554, 63, 1, 34, "PrefaceSection",
  CellTags->"IPM3.0.6"],
Cell[18521, 557, 514, 17, 71, "TextFirst",
  CellTags->"IPM3.0.6"],
Cell[19038, 576, 992, 27, 128, "Text",
  CellTags->{"i:2", "IPM3.0.6"}],
Cell[20033, 605, 514, 14, 90, "Text",
  CellTags->"IPM3.0.6"],
Cell[20550, 621, 100, 2, 33, "Text",
  PageBreakAbove->True,
  CellTags->"IPM3.0.6"],
Cell[20653, 625, 759, 20, 107, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[21415, 647, 542, 14, 88, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[21960, 663, 421, 7, 88, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[22384, 672, 751, 15, 126, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[23138, 689, 725, 13, 145, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[23866, 704, 407, 7, 88, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[24276, 713, 675, 12, 126, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[24954, 727, 676, 15, 126, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[25633, 744, 433, 10, 88, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[26069, 756, 623, 12, 107, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[26695, 770, 593, 13, 107, "BulletedList",
  CellTags->"IPM3.0.6"],
Cell[27291, 785, 336, 8, 69, "BulletedList",
  CellTags->"IPM3.0.6"]
}, Closed]],

Cell[CellGroupData[{
Cell[27664, 798, 85, 1, 34, "PrefaceSection",
  CellTags->"IPM3.0.7"],
Cell[27752, 801, 544, 10, 109, "TextFirst",
  CellTags->{"i:3", "IPM3.0.7"}],
Cell[28299, 813, 381, 12, 52, "Text",
  CellTags->"IPM3.0.7"],
Cell[28683, 827, 97, 1, 33, "Text",
  CellTags->"IPM3.0.7"],
Cell[28783, 830, 251, 7, 31, "BulletedList",
  CellTags->"IPM3.0.7"],
Cell[29037, 839, 270, 7, 31, "BulletedList",
  CellTags->"IPM3.0.7"],
Cell[29310, 848, 714, 18, 109, "Text",
  CellTags->{"i:4", "IPM3.0.7"}],
Cell[30027, 868, 219, 5, 52, "Text",
  CellTags->"IPM3.0.7"],

Cell[CellGroupData[{
Cell[30271, 877, 105, 2, 30, "Input",
  CellTags->"IPM3.0.7"],
Cell[30379, 881, 146, 3, 48, "Output",
  CellTags->"IPM3.0.7"]
}, Closed]],
Cell[30540, 887, 405, 11, 71, "Text",
  CellTags->"IPM3.0.7"],
Cell[30948, 900, 234, 6, 52, "Text",
  CellTags->{"i:5", "IPM3.0.7"}],
Cell[31185, 908, 77, 2, 30, "Input",
  CellTags->"IPM3.0.7"]
}, Closed]],

Cell[CellGroupData[{
Cell[31299, 915, 69, 1, 34, "PrefaceSection",
  CellTags->"IPM3.0.8"],
Cell[31371, 918, 655, 21, 90, "TextFirst",
  CellTags->"IPM3.0.8"],
Cell[32029, 941, 556, 15, 71, "Text",
  CellTags->{"i:6", "IPM3.0.8"}],
Cell[32588, 958, 717, 16, 109, "Text",
  CellTags->{"i:7", "IPM3.0.8"}],
Cell[33308, 976, 491, 14, 71, "Text",
  CellTags->{"i:8", "IPM3.0.8"}],
Cell[33802, 992, 538, 15, 71, "Text",
  CellTags->"IPM3.0.8"]
}, Closed]],

Cell[CellGroupData[{
Cell[34377, 1012, 58, 1, 34, "PrefaceSection",
  CellTags->"IPM3.0.9"],
Cell[34438, 1015, 655, 17, 90, "TextFirst",
  CellTags->"IPM3.0.9"]
}, Closed]],

Cell[CellGroupData[{
Cell[35130, 1037, 90, 2, 34, "PrefaceSection",
  PageBreakAbove->True,
  CellTags->"IPM3.0.10"],
Cell[35223, 1041, 554, 12, 90, "TextFirst",
  CellTags->{"i:9", "IPM3.0.10"}],
Cell[35780, 1055, 359, 9, 71, "Text",
  CellTags->{"i:10", "IPM3.0.10"}],
Cell[36142, 1066, 173, 4, 52, "Text",
  CellTags->"IPM3.0.10"],
Cell[36318, 1072, 216, 7, 52, "Text",
  CellTags->"IPM3.0.10"],
Cell[36537, 1081, 314, 6, 71, "Text",
  CellTags->"IPM3.0.10"],
Cell[36854, 1089, 137, 5, 50, "TextFirst",
  CellTags->"IPM3.0.10"],
Cell[36994, 1096, 139, 5, 50, "TextFirst",
  CellTags->"IPM3.0.10"],
Cell[37136, 1103, 137, 5, 50, "TextFirst",
  CellTags->"IPM3.0.10"],
Cell[37276, 1110, 152, 3, 31, "TextFirst",
  CellTags->"IPM3.0.10"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

