(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[   6675505,     103013]*)
(*NotebookOutlinePosition[   6762635,     105016]*)
(*  CellTagsIndexPosition[   6759500,     104908]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "Introduction to Programming with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "TitleBar",
  CellTags->{
  "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
    "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
    "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7"}],

Cell[CellGroupData[{

Cell[TextData[{
  "1 An introduction to ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "Chapter",
  CounterAssignments->{{"Chapter", 0}},
  CellTags->{"IPM3.1.0", "c:1"}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is a very large and seemingly complex system. It contains hundreds of \
functions for performing various tasks in science, mathematics, and \
engineering, including computing, programming, data analysis, knowledge \
representation, and visualization of information. In this introductory \
chapter, we introduce the elementary operations in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and give a sense of its computational and programming breadth and depth. \
In addition, we give some basic information that users of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " need to know, such as how to start ",
  StyleBox["Mathematica,",
    FontSlant->"Italic"],
  " how to get out of it, how to enter simple inputs and get answers, and \
finally how to use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\[CloseCurlyQuote]s documentation to get answers to questions about the \
system. "
}], "ChapterAbstract",
  CellTags->"IPM3.1.0"],

Cell[CellGroupData[{

Cell[TextData[{
  "A brief overview of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "Section",
  CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
  CellTags->{
  "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
    "IPM3.1.1.6", "c:2"}],

Cell[CellGroupData[{

Cell["Numerical computations", "Subsection",
  CellTags->{"i:1", "b:0.0", "IPM3.1.1.1", "c:3"}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has been aptly described as a sophisticated calculator. With it you can \
enter mathematical expressions and compute their values."
}], "TextFirst",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sin[ .86] - Log[\[Pi]]\ \((1 +  .08\/12)\)\^12\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[BoxData[
    \(\(-0.4818993389074404`\)\)], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell["You can store values in memory.", "TextFirst",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(rent = 350\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[BoxData[
    \(350\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(food = 175\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[BoxData[
    \(175\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(heat = 83\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[BoxData[
    \(83\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(rent + food + heat\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[BoxData[
    \(608\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[TextData[{
  "Yet ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " differs from calculators and simple computer programs in its ability to \
calculate exact results and to compute to an arbitrary degree of precision."
}], "TextFirst",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(1\/15 + 1\/35 + 1\/63\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[BoxData[
    \(1\/9\)], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(2\^500\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[BoxData[
    \(327339060789614187001318969682759915221664204604306478948329136809613379\
640467455488327009232590415715088668412756007100921725654588539305332852758937\
6\)], "Output",
  CellLabel->"Out[7]=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[\[Pi], 500]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[BoxData[
    \(3.1415926535897932384626433832795028841971693993751058209749445923078164\
062862089986280348253421170679821480865132823066470938446095505822317253594081\
284811174502841027019385211055596446229489549303819644288109756659334461284756\
482337867831652712019091456485669234603486104543266482133936072602491412737245\
870066063155881748815209209628292540917153643678925903600113305305488204665213\
841469519415116094330572703657595919530921861173819326117931051185480744623799\
62749567351885752724891227938183011949129833673363`500. \)], "Output",
  CellLabel->"Out[8]=",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Symbolic computations", "Subsection",
  CellTags->{"i:2", "b:0.1", "IPM3.1.1.2", "c:4"}],

Cell[TextData[{
  "One of the more powerful features of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is its ability to manipulate and compute with symbolic expressions. For \
example, you can factor polynomials and simplify trigonometric expressions."
}], "TextFirst",
  CellTags->{"b:0.1", "IPM3.1.1.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Factor[x\^5 - 1]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}],

Cell[BoxData[
    \(\((\(-1\) + x)\)\ \((1 + x + x\^2 + x\^3 + x\^4)\)\)], "Output",
  CellLabel->"Out[9]=",
  CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(TrigReduce[Sin[\[Theta]]\^3]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}],

Cell[BoxData[
    \(1\/4\ \((3\ Sin[\[Theta]] - Sin[3\ \[Theta]])\)\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]],

Cell[TextData[{
  "You can simplify expressions using assumptions about variables contained \
in those expressions. For example, if ",
  Cell[BoxData[
      \(TraditionalForm\`k\)]],
  " is assumed to be an integer, ",
  Cell[BoxData[
      \(TraditionalForm\`sin(2\ \[Pi]\ k + x)\)]],
  " simplifies to ",
  Cell[BoxData[
      \(TraditionalForm\`sin(x)\)]],
  "."
}], "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:0.1", "IPM3.1.1.2", "i:60"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Simplify[Sin[2  \[Pi]\ k + \ x], k \[Element] Integers]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}],

Cell[BoxData[
    \(Sin[x]\)], "Output",
  CellLabel->"Out[11]=",
  CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}]
}, Open  ]],

Cell["\<\
This computes the conditions for which a general quadratic polynomial will \
have both roots equal to each other.\
\>", "TextFirst",
  CellTags->{"b:0.1", "IPM3.1.1.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Reduce[\[Exists] \_\(x, a\ x\^2 + b\ x + c == 0\)\((\ \[ForAll] \_\(y, \
a\ y\^2 + b\ y + c == 0\)x == y)\), {a, b, c}]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}],

Cell[BoxData[
    \(\((a \[Equal] 0 && b \[NotEqual] 0)\) || \((a \[Equal] 0 && 
          b\ c \[NotEqual] 0)\) || \((a \[NotEqual] 0 && 
          c \[Equal] b\^2\/\(4\ a\))\)\)], "Output",
  CellLabel->"Out[12]=",
  CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]],

Cell["You can create functions that are defined piecewise.", "TextFirst",
  CellTags->"IPM3.1.1.2"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Piecewise[\ {{1, \ x \[Equal] 0}}, \ Sin[x]/x]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->"IPM3.1.1.2"],

Cell[BoxData[
    RowBox[{"\[Piecewise]", GridBox[{
          {"1", \(x \[Equal] 0\)},
          {\(Sin[x]\/x\), 
            TagBox["True",
              "PiecewiseDefault",
              AutoDelete->False,
              DeletionWarning->True]}
          },
        ColumnAlignments->{Left}]}]], "Output",
  CellLabel->"Out[13]=",
  CellTags->"IPM3.1.1.2"]
}, Open  ]],

Cell[TextData[{
  "The knowledge base of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " includes algorithms for solving polynomial equations, and computing \
integrals."
}], "TextFirst",
  CellTags->{"b:0.1", "IPM3.1.1.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Solve[x\^3 - a\ x + 1 == 0, x]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}],

Cell[BoxData[
    \({{x \[Rule] \(\((2\/3)\)\^\(1/3\)\ a\)\/\((\(-9\) + \@3\ \@\(27 - 4\ \
a\^3\))\)\^\(1/3\) + \((\(-9\) + \@3\ \@\(27 - 4\ \
a\^3\))\)\^\(1/3\)\/\(2\^\(1/3\)\ 3\^\(2/3\)\)}, {x \[Rule] \(-\(\(\((1 + \
\[ImaginaryI]\ \@3)\)\ a\)\/\(2\^\(2/3\)\ 3\^\(1/3\)\ \((\(-9\) + \@3\ \@\(27 \
- 4\ a\^3\))\)\^\(1/3\)\)\)\) - \(\((1 - \[ImaginaryI]\ \@3)\)\ \((\(-9\) + \
\@3\ \@\(27 - 4\ a\^3\))\)\^\(1/3\)\)\/\(2\ 2\^\(1/3\)\ 3\^\(2/3\)\)}, {x \
\[Rule] \(-\(\(\((1 - \[ImaginaryI]\ \@3)\)\ a\)\/\(2\^\(2/3\)\ 3\^\(1/3\)\ \
\((\(-9\) + \@3\ \@\(27 - 4\ a\^3\))\)\^\(1/3\)\)\)\) - \(\((1 + \
\[ImaginaryI]\ \@3)\)\ \((\(-9\) + \@3\ \@\(27 - 4\ a\^3\))\)\^\(1/3\)\)\/\(2\
\ 2\^\(1/3\)\ 3\^\(2/3\)\)}}\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Integral]\(1\/\(1 + x\^4\)\) \[DifferentialD]x\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}],

Cell[BoxData[
    \(\(1\/\(4\ \@2\)\) \((\(-2\)\ ArcTan[1 - \@2\ x] + 
          2\ ArcTan[1 + \@2\ x] - Log[\(-1\) + \@2\ x - x\^2] + 
          Log[1 + \@2\ x + x\^2])\)\)], "Output",
  CellLabel->"Out[15]=",
  CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Graphics", "Subsection",
  CellTags->{"b:0.2", "IPM3.1.1.3", "c:5"}],

Cell[TextData[{
  "The ability to visualize functions or sets of data often allows us greater \
insight into their structure and properties. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " provides a wide range of graphing capabilities. These include two- and \
three-dimensional plots of functions or data sets, contour and density plots \
of functions of two variables, bar charts, histograms and pie charts of data \
sets, and many packages designed for specific graphical purposes. In \
addition, the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language allows you to construct graphical images \
\[OpenCurlyDoubleQuote]from the ground up\[CloseCurlyDoubleQuote] using \
primitive elements, as we will see in Chapter 9."
}], "TextFirst",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[TextData[{
  "Here is a simple two-dimensional plot of the function ",
  Cell[BoxData[
      \(TraditionalForm\`sin(x + \(\@2\) \(sin(x\^2)\))\)]],
  "."
}], "Text",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Sin[x + \@2\ Sin[x\^2]], {x, \(-\[Pi]\), \[Pi]}]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.5 0.151576 0.309013 0.294306 [
[.04527 .29651 -7.59375 -11.875 ]
[.04527 .29651 7.59375 0 ]
[.19685 .29651 -7.59375 -11.875 ]
[.19685 .29651 7.59375 0 ]
[.34842 .29651 -7.59375 -11.875 ]
[.34842 .29651 7.59375 0 ]
[.65158 .29651 -4.5 -11.875 ]
[.65158 .29651 4.5 0 ]
[.80315 .29651 -4.5 -11.875 ]
[.80315 .29651 4.5 0 ]
[.95473 .29651 -4.5 -11.875 ]
[.95473 .29651 4.5 0 ]
[.4875 .01471 -15.1875 -5.9375 ]
[.4875 .01471 0 5.9375 ]
[.4875 .16186 -22.6875 -5.9375 ]
[.4875 .16186 0 5.9375 ]
[.4875 .45617 -16.5 -5.9375 ]
[.4875 .45617 0 5.9375 ]
[.4875 .60332 -9 -5.9375 ]
[.4875 .60332 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.04527 .30901 m
.04527 .31526 L
s
gsave
.04527 .29651 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.19685 .30901 m
.19685 .31526 L
s
gsave
.19685 .29651 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.34842 .30901 m
.34842 .31526 L
s
gsave
.34842 .29651 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.65158 .30901 m
.65158 .31526 L
s
gsave
.65158 .29651 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.80315 .30901 m
.80315 .31526 L
s
gsave
.80315 .29651 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.95473 .30901 m
.95473 .31526 L
s
gsave
.95473 .29651 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.125 Mabswid
.07559 .30901 m
.07559 .31276 L
s
.1059 .30901 m
.1059 .31276 L
s
.13622 .30901 m
.13622 .31276 L
s
.16653 .30901 m
.16653 .31276 L
s
.22716 .30901 m
.22716 .31276 L
s
.25748 .30901 m
.25748 .31276 L
s
.28779 .30901 m
.28779 .31276 L
s
.31811 .30901 m
.31811 .31276 L
s
.37874 .30901 m
.37874 .31276 L
s
.40905 .30901 m
.40905 .31276 L
s
.43937 .30901 m
.43937 .31276 L
s
.46968 .30901 m
.46968 .31276 L
s
.53032 .30901 m
.53032 .31276 L
s
.56063 .30901 m
.56063 .31276 L
s
.59095 .30901 m
.59095 .31276 L
s
.62126 .30901 m
.62126 .31276 L
s
.68189 .30901 m
.68189 .31276 L
s
.71221 .30901 m
.71221 .31276 L
s
.74252 .30901 m
.74252 .31276 L
s
.77284 .30901 m
.77284 .31276 L
s
.83347 .30901 m
.83347 .31276 L
s
.86378 .30901 m
.86378 .31276 L
s
.8941 .30901 m
.8941 .31276 L
s
.92441 .30901 m
.92441 .31276 L
s
.01496 .30901 m
.01496 .31276 L
s
.98504 .30901 m
.98504 .31276 L
s
.25 Mabswid
0 .30901 m
1 .30901 L
s
.5 .01471 m
.50625 .01471 L
s
gsave
.4875 .01471 -76.1875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.5 .16186 m
.50625 .16186 L
s
gsave
.4875 .16186 -83.6875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.5 .45617 m
.50625 .45617 L
s
gsave
.4875 .45617 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.5 .60332 m
.50625 .60332 L
s
gsave
.4875 .60332 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.5 .04414 m
.50375 .04414 L
s
.5 .07357 m
.50375 .07357 L
s
.5 .103 m
.50375 .103 L
s
.5 .13243 m
.50375 .13243 L
s
.5 .19129 m
.50375 .19129 L
s
.5 .22072 m
.50375 .22072 L
s
.5 .25015 m
.50375 .25015 L
s
.5 .27958 m
.50375 .27958 L
s
.5 .33844 m
.50375 .33844 L
s
.5 .36787 m
.50375 .36787 L
s
.5 .39731 m
.50375 .39731 L
s
.5 .42674 m
.50375 .42674 L
s
.5 .4856 m
.50375 .4856 L
s
.5 .51503 m
.50375 .51503 L
s
.5 .54446 m
.50375 .54446 L
s
.5 .57389 m
.50375 .57389 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .47726 m
.03279 .32329 L
.04262 .15066 L
.04758 .08707 L
.05035 .06098 L
.05288 .04309 L
.05524 .031 L
.05657 .02593 L
.0578 .02224 L
.05887 .01973 L
.06005 .01764 L
.06129 .01613 L
.06244 .01525 L
.0631 .01495 L
.06379 .01477 L
.06504 .01474 L
.06581 .01488 L
.06653 .0151 L
.06718 .01536 L
.06788 .01569 L
.07039 .01712 L
.07305 .01864 L
.07421 .0192 L
.07545 .01968 L
.07617 .01989 L
.07682 .02004 L
.0781 .0202 L
.07924 .0202 L
.08048 .02005 L
.08166 .01976 L
.08274 .01939 L
.0882 .01658 L
.08951 .0159 L
.09088 .0153 L
.09205 .01492 L
.09333 .01472 L
.09398 .01472 L
.09466 .0148 L
.0959 .01519 L
.09665 .0156 L
.09736 .01612 L
.09868 .01747 L
.09939 .01842 L
.10015 .01961 L
.10151 .02225 L
.10297 .02586 L
.10458 .03082 L
.10707 .04069 L
.10979 .05458 L
.11474 .08813 L
Mistroke
.12403 .17393 L
.13499 .28694 L
.14492 .37066 L
.1498 .40027 L
.15237 .41251 L
.15508 .42294 L
.15745 .43002 L
.15871 .43301 L
.16006 .43562 L
.1613 .4375 L
.16242 .43877 L
.16356 .43964 L
.16463 .44008 L
.16584 .44014 L
.16652 .43997 L
.16717 .43967 L
.16846 .43868 L
.16919 .43789 L
.16986 .43702 L
.17221 .43283 L
.1734 .43007 L
.17469 .42655 L
.17743 .41735 L
.18035 .40493 L
.18562 .37588 L
.19589 .29671 L
.207 .19007 L
.21661 .10163 L
.22207 .06158 L
.22469 .04627 L
.22717 .03441 L
.22957 .02552 L
.23093 .02166 L
.23219 .01885 L
.23325 .01705 L
.23441 .01566 L
.23563 .01487 L
.23677 .01473 L
.23803 .01524 L
.23867 .01576 L
.23936 .01652 L
.24059 .01836 L
.24173 .02063 L
.24432 .02757 L
.24708 .03759 L
.25275 .06508 L
.25811 .09722 L
.26814 .16444 L
.28643 .27539 L
.29144 .29885 L
Mistroke
.29683 .32018 L
.30149 .33543 L
.30657 .34884 L
.31189 .35954 L
.31434 .36339 L
.31694 .36679 L
.31952 .3695 L
.32186 .37141 L
.32299 .37215 L
.32421 .37283 L
.32528 .37332 L
.32642 .37374 L
.32774 .37409 L
.32897 .3743 L
.32971 .37437 L
.33042 .3744 L
.33106 .37439 L
.33175 .37435 L
.33243 .37428 L
.33317 .37416 L
.33395 .374 L
.33468 .37382 L
.33598 .3734 L
.33738 .37283 L
.33994 .37149 L
.34264 .36968 L
.34746 .36555 L
.35701 .35459 L
.36722 .3401 L
.38559 .31138 L
.40551 .28328 L
.41675 .27113 L
.4221 .26657 L
.4271 .26307 L
.43157 .26061 L
.43386 .2596 L
.43636 .25869 L
.43778 .25826 L
.43909 .25793 L
.44045 .25764 L
.44168 .25744 L
.44289 .25728 L
.44404 .25719 L
.44524 .25713 L
.44654 .25713 L
.44783 .25718 L
.44903 .25728 L
.45035 .25745 L
.45176 .2577 L
.45324 .25803 L
Mistroke
.45464 .25842 L
.45726 .25932 L
.46198 .26157 L
.46711 .26491 L
.47654 .27352 L
.48693 .28676 L
.49779 .30481 L
.50796 .32561 L
.54672 .43501 L
.56642 .50043 L
.57683 .53325 L
.58793 .56388 L
.59349 .57666 L
.59873 .58673 L
.60343 .59391 L
.60585 .59687 L
.60851 .59949 L
.60972 .60047 L
.61102 .60135 L
.61214 .60199 L
.61335 .60254 L
.61407 .6028 L
.61473 .60299 L
.61603 .60324 L
.61721 .60332 L
.61847 .60325 L
.61955 .60306 L
.62071 .60272 L
.62181 .60228 L
.62282 .60177 L
.62516 .60018 L
.62763 .59791 L
.63263 .59152 L
.63717 .58371 L
.64736 .56026 L
.66572 .50583 L
.67504 .47815 L
.6849 .45329 L
.69005 .44323 L
.69278 .4389 L
.69572 .4351 L
.6971 .43363 L
.69841 .43244 L
.69969 .43147 L
.70087 .43074 L
.70199 .4302 L
.70318 .4298 L
.7045 .42955 L
.70569 .42952 L
.7069 .42967 L
Mistroke
.70803 .42997 L
.70926 .4305 L
.71058 .43127 L
.71197 .43233 L
.71328 .43357 L
.71574 .43648 L
.72016 .4437 L
.72497 .45433 L
.7358 .48753 L
.7458 .52564 L
.7548 .56 L
.75953 .57576 L
.76449 .58923 L
.76715 .59477 L
.76855 .59718 L
.77005 .59933 L
.77143 .60091 L
.77268 .60201 L
.77394 .60278 L
.77513 .6032 L
.77634 .60332 L
.77762 .6031 L
.77872 .60265 L
.77992 .60186 L
.78121 .60068 L
.7826 .59905 L
.78511 .59517 L
.80489 .53953 L
.8103 .52438 L
.81327 .51761 L
.81607 .51258 L
.81843 .50948 L
.81908 .50883 L
.81976 .50825 L
.82099 .50746 L
.82164 .50718 L
.82233 .50699 L
.82358 .50693 L
.82435 .50707 L
.82507 .50733 L
.82572 .50768 L
.82642 .50817 L
.82761 .50927 L
.82889 .51083 L
.83121 .51465 L
.83396 .52079 L
.83647 .52782 L
.84712 .56737 L
.8518 .58504 L
.85428 .59283 L
Mistroke
.85564 .59639 L
.85692 .59916 L
.85813 .6012 L
.85945 .60268 L
.86064 .60328 L
.86174 .60317 L
.86278 .60243 L
.86392 .60088 L
.86511 .59837 L
.86622 .59518 L
.86749 .59052 L
.86883 .58433 L
.87123 .56993 L
.87382 .54969 L
.87668 .52183 L
.88658 .39051 L
.89562 .2556 L
.90055 .19111 L
.90514 .14189 L
.90996 .10267 L
.91251 .08696 L
.91524 .07368 L
.91773 .06436 L
.92009 .05778 L
.9222 .05349 L
.92339 .0517 L
.92449 .05042 L
.9257 .04941 L
.92699 .04877 L
.92821 .04858 L
.92932 .04876 L
.93062 .04939 L
.93136 .04996 L
.93204 .05063 L
.93329 .0522 L
.93463 .0544 L
.93602 .05733 L
.93753 .06126 L
.94025 .07063 L
.94291 .08289 L
.94535 .09722 L
.94903 .12499 L
.95297 .16333 L
.96006 .25296 L
.97619 .47726 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[16]:=",
  ImageSize->{307.563, 189.375},
  ImageMargins->{{42.4375, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004c0000_B000`400?l00000o`00003oo`3ooold
0?ooo`00o`3ooold0?ooo`00V@3oool00`000000oooo0?ooo`2G0?ooo`00V@3oool00`000000oooo
0?ooo`2G0?ooo`00T03oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo09L0oooo002@
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0U`3oool001D0oooo0`0000060?oo
o`<00000:03oool2000003h0oooo100000030?ooo`030000003oool0oooo00H0oooo0`00002G0?oo
o`005@3oool00`000000oooo0?ooo`0200000080oooo0P0000020?ooo`8000009P3oool200000003
0?ooo`000000oooo04<0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`2G0?ooo`00
503oool2000000<0oooo0`0000050?ooo`030000003oool0oooo02@0oooo00@000000?ooo`3oool0
0000A03oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo09L0oooo000D0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`0000130?ooo`80
0000203oool00`000000oooo0?ooo`2G0?ooo`004`3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo02<0oooo00D000000?ooo`3oool0oooo000000130?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool0U`3oool001<0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0R0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0BP3oool00`000000oooo
0?ooo`2G0?ooo`004`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`3oool00`3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo09L0oooo000C
0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0U`3oool001<0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0B@3oool00`000000oooo0?ooo`2G0?ooo`004`3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo0200oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`190?ooo`80
0000V03oool001<0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0P0?ooo`030000
003oool0oooo00D0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`1l0?ooo`<00000
603oool001<0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0O0?ooo`030000003o
ool0oooo00D0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`1l0?ooo`040000003o
ool00000000001L0oooo000B0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool07P3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0N`3o
ool01@000000oooo0?ooo`3oool0000001L0oooo000B0?ooo`030000003oool0oooo00l0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo04P0oooo00<0
00000?ooo`3oool0N`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo01@0oooo000B
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo01@0oooo000B0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo04L0oooo00<000000?oo
o`3oool0NP3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01@0oooo000B0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`070?ooo`03
0000003oool0oooo04L0oooo00<000000?ooo`3oool0NP3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo01<0oooo000B0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
7P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3oool0
N@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01<0oooo000A0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo04L0oooo0P00001j0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool04`3o
ool00140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool04`3oool00140oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0A`3oool00`00
0000oooo0?ooo`1h0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04P3oool00140
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00T0
oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool04P3oool00140oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`0L0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0AP3oool00`000000oooo
0?ooo`1h0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04P3oool00140oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0AP3oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool04P3oool00140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`1h
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04P3oool00140oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00T0oooo00<000000?oo
o`3oool0AP3oool00`000000oooo0?ooo`1h0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool04P3oool00140oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0L0?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0AP3oool00`000000oooo0?ooo`1h0?ooo`03
0000003oool0oooo00L0oooo00<000000?ooo`3oool04P3oool00140oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0
AP3oool2000007T0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0B0?ooo`004@3o
ool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`160?ooo`030000003oool0oooo07P0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0B0?ooo`004@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo01/0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`150?ooo`030000003oool0
oooo07L0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0B0?ooo`004@3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`150?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0A0?ooo`004@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01X0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo07L0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0A0?ooo`00403oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool00`000000oooo
0?ooo`150?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`0A0?ooo`00403oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01X0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo07L0oooo00<0
00000?ooo`3oool02@3oool00`000000oooo0?ooo`0A0?ooo`00403oool00`000000oooo0?ooo`0C
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`15
0?ooo`030000003oool0oooo07L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0A
0?ooo`00403oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo01X0oooo00<000000?oo
o`3oool02`3oool00`000000oooo0?ooo`150?ooo`800000M`3oool00`000000oooo0?ooo`0:0?oo
o`030000003oool0oooo0140oooo000@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3o
ool06@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3o
ool0MP3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0140oooo000@0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0<0?ooo`030000
003oool0oooo04@0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`0;0?ooo`030000
003oool0oooo0100oooo000@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool06@3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0MP3o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0100oooo000@0?ooo`030000003oool0
oooo01@0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0
oooo04@0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo0100oooo000@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0MP3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo0100oooo000@0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo04@0
oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0100
oooo000@0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0>0?ooo`030000003oool0oooo0340oooo0`0000030?ooo`030000003oool0oooo00800000
1`3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0
403oool00100oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0H0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0<@3oool010000000oooo0?ooo`0000040?ooo`040000
003oool0oooo000000H0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo00l0oooo000@0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo02/0oooo100000020?ooo`040000
003oool0oooo000000L0oooo00<000000?ooo`3oool0103oool3000007H0oooo00<000000?ooo`3o
ool0303oool00`000000oooo0?ooo`0?0?ooo`00403oool00`000000oooo0?ooo`0E0?ooo`030000
003oool0oooo01L0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0a0?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool000001P3oool00`000000oooo0?ooo`1f0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool03`3oool00100oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool0<@3oool010000000oooo0?ooo`0000040?ooo`<000001`3oool00`000000oooo0?ooo`1e0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool00100oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool0<@3oool010000000oooo0?ooo`0000050?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool0M@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00l0oooo000@0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo0340oooo0P0000060?ooo`<000001P3oool00`000000oooo0?ooo`1e0?ooo`03
0000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool00100oooo00<000000?ooo`3oool0
5@3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
@P3oool00`000000oooo0?ooo`1e0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
3`3oool000l0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0F0?ooo`030000003o
ool0oooo0100oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1e0?ooo`030000003o
ool0oooo00d0oooo00<000000?ooo`3oool03`3oool000l0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0@P3oool0
0`000000oooo0?ooo`1e0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03`3oool0
00l0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`1e0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03P3oool000l0oooo00<000000?ooo`3oool05P3oool00`000000
oooo0?ooo`0F0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0@P3oool2000007H0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`003`3oool00`000000oooo
0?ooo`0G0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04@3oool00`000000oooo
0?ooo`110?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0>0?ooo`003`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01D0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo07D0oooo00<0
00000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`003`3oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`11
0?ooo`030000003oool0oooo07D0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>
0?ooo`003`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01D0oooo00<000000?oo
o`3oool04@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo07D0oooo00<000000?oo
o`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`003`3oool00`000000oooo0?ooo`0G0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`110?ooo`03
0000003oool0oooo07@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`00
3`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
4`3oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo07@0oooo00<000000?ooo`3oool0
3`3oool00`000000oooo0?ooo`0>0?ooo`003`3oool00`000000oooo0?ooo`0G0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`100?ooo`030000003o
ool0oooo07@0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0>0?ooo`003`3oool0
0`000000oooo0?ooo`0G0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`100?ooo`030000003oool0oooo07@0oooo00<000000?ooo`3oool03`3oool0
0`000000oooo0?ooo`0>0?ooo`003`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`100?ooo`800000M@3oool00`00
0000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo000?0?ooo`030000003oool0oooo01L0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0400
oooo00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0
oooo000?0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0C0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0M03oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo00h0oooo000?0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0503oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo03l0oooo00<0
00000?ooo`3oool0M03oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo000?
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0M03oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo00d0oooo000>0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo03l0oooo00<000000?oo
o`3oool0M03oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00d0oooo000>0?ooo`03
0000003oool0oooo01T0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0D0?ooo`03
0000003oool0oooo03l0oooo00<000000?ooo`3oool0L`3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo00d0oooo000>0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0
L`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo000>0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0F0?ooo`030000003o
ool0oooo03h0oooo00<000000?ooo`3oool0L`3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo00d0oooo000>0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo03h0oooo0P00001d0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool03@3oool000h0oooo00<000000?ooo`3oool06@3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0?P3oool00`00
0000oooo0?ooo`1c0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03@3oool000h0
oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01H0
oooo00<000000?ooo`3oool0:P3oool8000000`0oooo00<000000?ooo`3oool0L`3oool00`000000
oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo000>0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo02T0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo07<0oooo
00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`003P3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool05P3oool00`000000oooo0?oo
o`0X0?ooo`030000003oool0oooo00P0oooo0P0000090?ooo`030000003oool0oooo07<0oooo00<0
00000?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`003P3oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0U
0?ooo`8000003@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo07<0oooo00<00000
0?ooo`3oool04@3oool00`000000oooo0?ooo`0=0?ooo`003P3oool010000000oooo0?ooo`3oool2
000001L0oooo00<000000?ooo`3oool04@3oool4000001H0oooo00<000000?ooo`3oool04P3oool0
0`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02P0oooo1000000J0?ooo`030000003oool0
oooo00h0oooo0P0000000`3oool000000?ooo`0>0?ooo`003P3oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool04@3oool2000001P0oooo00<00000
0?ooo`3oool04P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0100oooo00<00000
0?ooo`3oool0103oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02T0oooo00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo008000003`3oool0
00`0oooo100000040?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02`3oool40000
0080oooo00<000000?ooo`0000005`3oool00`000000oooo0?ooo`0;0?ooo`@000000`3oool00`00
0000oooo0?ooo`0=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00`3oool20000
02/0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`0000003P3oool000h0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo0100oooo00@000000?ooo`3oool000005`3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04`3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo00l0oooo00@000000?ooo`3oool000003P3oool0
00h0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
0100oooo00D000000?ooo`3oool0oooo0000000F0?ooo`030000003oool0oooo0180oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0403oool00`000000oooo0000000>0?ooo`003P3oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0403oool200000080oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0@0?ooo`800000303oool00`000000oooo0?ooo`0G
0?ooo`050000003oool0oooo0?ooo`000000:`3oool2000002X0oooo00@000000?ooo`3oool00000
6P3oool00`000000oooo0?ooo`0=0?ooo`060000003oool0oooo0000003oool000003P3oool000h0
oooo00@000000?ooo`3oool0oooo0P00000G0?ooo`030000003oool0oooo0100oooo00@000000?oo
o`0000000000603oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool06@3oool010000000oooo0?ooo`00000/0?ooo`030000003oool0oooo02T0oooo0P00
000J0?ooo`030000003oool0oooo00l0oooo0P0000020?ooo`030000003oool0oooo00`0oooo000>
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0I
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool06@3oool010000000oooo0?ooo`00
001d0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0303oool000d0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01T0oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`0K0?ooo`030000003oool0000007@0oooo00<00000
0?ooo`3oool04`3oool00`000000oooo0?ooo`0<0?ooo`003@3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`000000M03oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo00`0oooo000=0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool07P3oool2000007@0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0<0?ooo`00
0`3ooooo000002h000000`3oool000L0oooo00<000000?ooo`3oool00`3oool010000000oooo0?oo
o`0000090?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo00D0oooo0P0000080?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01P3oool00`000000oooo000000060?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01P3oool00`000000oooo000000070?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo
0P0000080?ooo`030000003oool0oooo00H0oooo00@000000?ooo`3oool000001@3oool00`000000
oooo0?ooo`060?ooo`003@3oool010000000oooo0?ooo`00000K0?ooo`030000003oool0oooo00h0
oooo0P00000L0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool01`3oool00`000000
oooo0?ooo`0Q0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo0100oooo00@000000?ooo`3oool000003P3oool000d0
oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0Q0?ooo`800000L`3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo00`0oooo000=0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool08P3oool2000007<0oooo00<000000?ooo`3oool04`3oool00`000000oooo
0?ooo`0<0?ooo`003@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0703oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0280oooo00<0
00000?ooo`000000LP3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00/0oooo000=
0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0L
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool08`3oool00`000000oooo0000001b
0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02`3oool000d0oooo00<000000?oo
o`3oool07@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo01d0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0S0?ooo`040000003oool0oooo00000740oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`0;0?ooo`003@3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo02@0oooo00@000000?ooo`3oool00000L@3oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo00/0oooo000=0?ooo`030000003oool0oooo01d0oooo00<000000?oo
o`3oool0303oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool0903oool2000000030?ooo`000000oooo0700oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0;0?ooo`003@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo02D0
oooo00D000000?ooo`3oool0oooo0000001`0?ooo`030000003oool0oooo01@0oooo00<000000?oo
o`3oool02`3oool000d0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo01l0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0V0?ooo`05
0000003oool0oooo0?ooo`000000L03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
00/0oooo000=0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0P0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool09`3oool00`000000
oooo0?ooo`020?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool0503oool00`000000
oooo0?ooo`0;0?ooo`00303oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo00`0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo02L0oooo
00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1]0?ooo`030000003oool0oooo01@0oooo
00<000000?ooo`3oool02`3oool000`0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0X0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?oo
o`0D0?ooo`030000003oool0oooo00/0oooo000<0?ooo`030000003oool0oooo01l0oooo00<00000
0?ooo`3oool02`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool0:@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo06/0oooo00<00000
0?ooo`3oool05@3oool00`000000oooo0?ooo`0;0?ooo`00303oool00`000000oooo0?ooo`0O0?oo
o`030000003oool0oooo00/0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`090?oo
o`030000003oool0oooo02X0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1[0?oo
o`030000003oool0oooo01D0oooo00<000000?ooo`3oool02`3oool000`0oooo00<000000?ooo`3o
ool07`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3o
ool0J`3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00/0oooo000<0?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0T0?ooo`800000
1P3oool00`000000oooo0?ooo`0/0?ooo`8000001@3oool00`000000oooo0?ooo`1Z0?ooo`030000
003oool0oooo01H0oooo00<000000?ooo`3oool02P3oool000`0oooo00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02D0oooo0P0000030?ooo`<00000;P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool05P3o
ool00`000000oooo0?ooo`0:0?ooo`00303oool00`000000oooo0?ooo`0O0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool09P3oool500000300oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool02P3oool0
00`0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
05/0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool02P3oool000`0oooo00<000000?ooo`3oool0803oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`1Y0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool02P3oool000`0oooo
00<000000?ooo`3oool0803oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo05`0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool02P3oool000`0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo05`0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`1X0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool02P3oool000`0oooo00<00000
0?ooo`3oool0803oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo05`0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool02P3oool000`0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`070?oo
o`030000003oool0oooo05`0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`1W0?oo
o`030000003oool0oooo01H0oooo00<000000?ooo`3oool02P3oool000`0oooo00<000000?ooo`3o
ool08@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo05d0oooo0P0000080?ooo`03
0000003oool0oooo06L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0:0?ooo`00
2`3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
G@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo06L0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`0:0?ooo`002`3oool00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool0G@3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo06H0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0:0?ooo`002`3oool0
0`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0G@3oool0
0`000000oooo0?ooo`080?ooo`030000003oool0oooo06H0oooo00<000000?ooo`3oool05P3oool0
0`000000oooo0?ooo`0:0?ooo`002`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
06H0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`090?ooo`002`3oool00`000000
oooo0?ooo`0S0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0GP3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo06@0oooo00<000000?ooo`3oool0603oool00`000000
oooo0?ooo`090?ooo`002`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00@0oooo
00<000000?ooo`3oool0GP3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo06@0oooo
00<000000?ooo`3oool0603oool00`000000oooo0?ooo`090?ooo`002`3oool00`000000oooo0?oo
o`0S0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0GP3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo06<0oooo00<000000?ooo`3oool0603oool00`000000oooo0?oo
o`090?ooo`002`3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00<0oooo00<00000
0?ooo`3oool0G`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo06<0oooo00<00000
0?ooo`3oool0603oool00`000000oooo0?ooo`090?ooo`002`3oool00`000000oooo0?ooo`0T0?oo
o`030000003oool0oooo0080oooo00<000000?ooo`3oool0G`3oool2000000/0oooo00<000000?oo
o`3oool0;@3oool2000003@0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`090?oo
o`002`3oool00`000000oooo0?ooo`0T0?ooo`040000003oool0oooo0?ooo`800000H@3oool00`00
0000oooo0?ooo`0;0?ooo`030000003oool0oooo02X0oooo1@00000c0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool02@3oool000/0oooo00<000000?ooo`3oool09@3oool010000000
oooo0?ooo`00001R0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0:@3oool20000
00@0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
00T0oooo000;0?ooo`030000003oool0oooo02D0oooo1000001R0?ooo`030000003oool0oooo00/0
oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0300
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`090?ooo`002`3oool00`000000oooo
0?ooo`0V0?ooo`800000H`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02L0oooo
00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01P0oooo
00<000000?ooo`3oool02@3oool000/0oooo00<000000?ooo`3oool0R`3oool00`000000oooo0?oo
o`0<0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`0^0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02@3oool000/0oooo00<00000
0?ooo`3oool0N@3oool3000000<0oooo00<000000?ooo`3oool00P0000070?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`070?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`090?ooo`002`3oool00`00
0000oooo0?ooo`1i0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool000001P3o
ool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02@3o
ool000X0oooo00<000000?ooo`3oool0NP3oool010000000oooo0?ooo`0000070?ooo`030000003o
ool0oooo00@0oooo0`00000=0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool02@3o
ool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02@3o
ool000X0oooo00<000000?ooo`3oool0NP3oool010000000oooo0?ooo`0000040?ooo`040000003o
ool0oooo000000H0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0T0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo00P0oooo000:0?ooo`030000003oool0oooo07X0oooo00@000000?ooo`3oool00000
103oool3000000L0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo00/0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo00P0oooo000:0?ooo`030000003oool0oooo07X0oooo00@000000?ooo`3oool00000
1@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0
6P3oool00`000000oooo0?ooo`080?ooo`002P3oool00`000000oooo0?ooo`1k0?ooo`8000001P3o
ool3000000H0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo00`0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo00P0oooo000:0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool03P3oool0
0`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0:`3oool0
0`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00P0oooo000:0?ooo`030000003oool0oooo
08`0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo
00P0oooo000:0?ooo`030000003oool0oooo08`0oooo00<000000?ooo`3oool03`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0:P3oool00`000000
oooo0?ooo`0J0?ooo`030000003oool0oooo00P0oooo002I0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo02X0oooo
00<000000?ooo`3oool09@3oool009T0oooo0P00000@0?ooo`030000003oool0oooo01h0oooo00<0
00000?ooo`3oool0403oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo02D0oooo002I
0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool09@3oool009T0oooo00<000000?oo
o`3oool0403oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo0140oooo00<000000?oo
o`3oool0:@3oool00`000000oooo0?ooo`0U0?ooo`00V@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo01`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0X0?ooo`03
0000003oool0oooo02D0oooo002I0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0
9@3oool009T0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo01<0oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0V0?ooo`00V@3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02H0oooo002I0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo
01@0oooo1000000>0?ooo`030000003oool0oooo02H0oooo002I0?ooo`030000003oool0oooo0180
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01@0
oooo00@000000?ooo`3oool000003P3oool00`000000oooo0?ooo`0V0?ooo`00V@3oool2000001<0
oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01<0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo02H0
oooo002I0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`0E0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0;0?ooo`030000003oool0oooo02H0oooo002I0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool05`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0140oooo00<0
00000?ooo`3oool0103oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo02H0oooo002I
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0G
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0:
0?ooo`030000003oool0oooo02H0oooo002I0?ooo`030000003oool0oooo01<0oooo00<000000?oo
o`3oool05P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo0100oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo02H0oooo002I0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0H0?ooo`03
0000003oool0oooo00l0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo02H0oooo002I0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0
503oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo02L0oooo002I0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo02L0oooo002I0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3oool0203oool0
0`000000oooo0?ooo`070?ooo`030000003oool0oooo02L0oooo002I0?ooo`8000005P3oool00`00
0000oooo0?ooo`0B0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool09`3oool009T0
oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01/0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool09`3oool009T0oooo00<000000?ooo`3oool05P3oool00`000000oooo
0?ooo`0A0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool09`3oool009T0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool09`3oool009T0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0@
0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0;
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool09`3oool009T0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01d0oooo00<000000?oo
o`3oool02P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00D0oooo00<000000?oo
o`3oool0:03oool009T0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0:03oool009T0oooo00<000000?ooo`3oool0
5`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
:03oool009T0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0>0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0:03oool009T0oooo0P00000I0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo00h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0X0?ooo`00V@3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool08@3oool00`00
0000oooo0?ooo`060?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0Y0?ooo`00V@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0100
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0Y0?ooo`00V@3oool00`000000oooo
0?ooo`0J0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool08`3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`0Y0?ooo`00V@3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo00P0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo00<0
00000?ooo`3oool00P3oool00`000000oooo0?ooo`0Y0?ooo`00V@3oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0180oooo00D000000?ooo`3oool0oooo0000000[0?ooo`00T03oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`0V0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04P3oool0
1@000000oooo0?ooo`3oool0000002/0oooo002@0?ooo`030000003oool0oooo00H0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo02L0oooo0P000003
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`000000;03oool00900oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`0N0?ooo`80000000<0oooo000000000000:`3oool40000
01H0oooo0`00000/0?ooo`00T03oool00`000000oooo0?ooo`060?ooo`<00000803oool00`000000
oooo0?ooo`0/0?ooo`800000603oool00`000000oooo0?ooo`0[0?ooo`00T03oool00`000000oooo
0?ooo`060?ooo`030000003oool0oooo09L0oooo002?0?ooo`800000203oool00`000000oooo0?oo
o`2G0?ooo`00T03oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo09L0oooo002I0?oo
o`030000003oool0oooo09L0oooo003o0?oooc@0oooo0000\
\>"],
  ImageRangeCache->{{{0, 306.563}, {188.375, 0}} -> {-3.37245, -1.06728, \
0.0220017, 0.0113315}},
  CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[16]=",
  CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}]
}, Open  ]],

Cell["\<\
You can combine two or more plots in a single graphic by enclosing them \
inside curly braces.\
\>", "TextFirst",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[{Sin[x], Sin[2\ x]}, {x, 0, 2\ \[Pi]}];\)\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.0238095 0.151576 0.309016 0.294303 [
[.17539 .29652 -4.5 -11.875 ]
[.17539 .29652 4.5 0 ]
[.32696 .29652 -4.5 -11.875 ]
[.32696 .29652 4.5 0 ]
[.47854 .29652 -4.5 -11.875 ]
[.47854 .29652 4.5 0 ]
[.63011 .29652 -4.5 -11.875 ]
[.63011 .29652 4.5 0 ]
[.78169 .29652 -4.5 -11.875 ]
[.78169 .29652 4.5 0 ]
[.93327 .29652 -4.5 -11.875 ]
[.93327 .29652 4.5 0 ]
[.01131 .01471 -15.1875 -5.9375 ]
[.01131 .01471 0 5.9375 ]
[.01131 .16187 -22.6875 -5.9375 ]
[.01131 .16187 0 5.9375 ]
[.01131 .45617 -16.5 -5.9375 ]
[.01131 .45617 0 5.9375 ]
[.01131 .60332 -9 -5.9375 ]
[.01131 .60332 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .30902 m
.17539 .31527 L
s
gsave
.17539 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.32696 .30902 m
.32696 .31527 L
s
gsave
.32696 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.47854 .30902 m
.47854 .31527 L
s
gsave
.47854 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.63011 .30902 m
.63011 .31527 L
s
gsave
.63011 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.78169 .30902 m
.78169 .31527 L
s
gsave
.78169 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.93327 .30902 m
.93327 .31527 L
s
gsave
.93327 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.125 Mabswid
.05412 .30902 m
.05412 .31277 L
s
.08444 .30902 m
.08444 .31277 L
s
.11476 .30902 m
.11476 .31277 L
s
.14507 .30902 m
.14507 .31277 L
s
.2057 .30902 m
.2057 .31277 L
s
.23602 .30902 m
.23602 .31277 L
s
.26633 .30902 m
.26633 .31277 L
s
.29665 .30902 m
.29665 .31277 L
s
.35728 .30902 m
.35728 .31277 L
s
.38759 .30902 m
.38759 .31277 L
s
.41791 .30902 m
.41791 .31277 L
s
.44822 .30902 m
.44822 .31277 L
s
.50885 .30902 m
.50885 .31277 L
s
.53917 .30902 m
.53917 .31277 L
s
.56948 .30902 m
.56948 .31277 L
s
.5998 .30902 m
.5998 .31277 L
s
.66043 .30902 m
.66043 .31277 L
s
.69074 .30902 m
.69074 .31277 L
s
.72106 .30902 m
.72106 .31277 L
s
.75138 .30902 m
.75138 .31277 L
s
.81201 .30902 m
.81201 .31277 L
s
.84232 .30902 m
.84232 .31277 L
s
.87264 .30902 m
.87264 .31277 L
s
.90295 .30902 m
.90295 .31277 L
s
.96358 .30902 m
.96358 .31277 L
s
.9939 .30902 m
.9939 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 .01471 m
.03006 .01471 L
s
gsave
.01131 .01471 -76.1875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.02381 .16187 m
.03006 .16187 L
s
gsave
.01131 .16187 -83.6875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.02381 .45617 m
.03006 .45617 L
s
gsave
.01131 .45617 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.02381 .60332 m
.03006 .60332 L
s
gsave
.01131 .60332 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.02381 .04414 m
.02756 .04414 L
s
.02381 .07357 m
.02756 .07357 L
s
.02381 .103 m
.02756 .103 L
s
.02381 .13243 m
.02756 .13243 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.06244 .38322 L
.10458 .45852 L
.14415 .51889 L
.18221 .56357 L
.20178 .58047 L
.21267 .58793 L
.22272 .59354 L
.23293 .59796 L
.23832 .59976 L
.24402 .60127 L
.24641 .60178 L
.24897 .60225 L
.25119 .60258 L
.25361 .60288 L
.25507 .60302 L
.25643 .60313 L
.25773 .60321 L
.2584 .60324 L
.25912 .60327 L
.25983 .60329 L
.26048 .60331 L
.26176 .60332 L
.26293 .60331 L
.26416 .60329 L
.26538 .60324 L
.26607 .60321 L
.26671 .60317 L
.26912 .60299 L
.27131 .60275 L
.27364 .60244 L
.27847 .60156 L
.2838 .60025 L
.29326 .59705 L
.30213 .59301 L
.32232 .58025 L
.34082 .56432 L
.38045 .51781 L
.41855 .45965 L
.45912 .38744 L
.49816 .31259 L
.53966 .23289 L
.57964 .1614 L
.61811 .10221 L
.63759 .07707 L
.65902 .05386 L
.67938 .03652 L
.69843 .02473 L
.70884 .02018 L
.71396 .01844 L
Mistroke
.71872 .01712 L
.72301 .01617 L
.72758 .01542 L
.7302 .01511 L
.73148 .01499 L
.73266 .0149 L
.73371 .01484 L
.73487 .01478 L
.73603 .01474 L
.73725 .01472 L
.73856 .01472 L
.73977 .01473 L
.7404 .01475 L
.7411 .01477 L
.74251 .01484 L
.74375 .01492 L
.74508 .01503 L
.74748 .01528 L
.74985 .0156 L
.75235 .01601 L
.7568 .01695 L
.76198 .01836 L
.7668 .01998 L
.77773 .02472 L
.78828 .0307 L
.79823 .03757 L
.81683 .05353 L
.85534 .09846 L
.89632 .16101 L
.93577 .23146 L
.97371 .3042 L
.97619 .30902 L
Mfstroke
.02381 .30902 m
.06244 .45263 L
.08255 .51497 L
.0932 .54236 L
.10458 .56657 L
.11009 .57624 L
.11531 .58409 L
.12038 .59047 L
.12507 .59525 L
.12951 .59877 L
.13178 .60018 L
.13422 .60141 L
.13549 .60193 L
.13666 .60234 L
.13784 .60268 L
.13895 .60293 L
.14022 .60314 L
.1409 .60322 L
.14161 .60328 L
.1423 .60331 L
.14295 .60332 L
.14415 .60328 L
.14531 .60316 L
.14656 .60297 L
.14773 .60271 L
.14881 .60241 L
.15143 .60144 L
.15392 .60019 L
.15886 .59679 L
.16423 .59169 L
.17383 .57909 L
.1827 .56357 L
.20137 .51989 L
.22197 .45702 L
.26217 .30798 L
.30086 .16432 L
.32051 .10346 L
.3309 .0766 L
.342 .05274 L
.35275 .03485 L
.35784 .0283 L
.36253 .02337 L
.36724 .01952 L
.36972 .01794 L
.37235 .01661 L
.37463 .01574 L
.37573 .01541 L
.37677 .01516 L
.37799 .01494 L
.37911 .0148 L
Mistroke
.38031 .01472 L
.38162 .01473 L
.38227 .01476 L
.38296 .01482 L
.38419 .01498 L
.38551 .01524 L
.38695 .01564 L
.38955 .0166 L
.39197 .01782 L
.39649 .02088 L
.40134 .02529 L
.40698 .0319 L
.41232 .03956 L
.42232 .05748 L
.44227 .1059 L
.4607 .16316 L
.50002 .3091 L
.53782 .44987 L
.55884 .51523 L
.56803 .53913 L
.57808 .56136 L
.58722 .57774 L
.59696 .59091 L
.60236 .59621 L
.60726 .59976 L
.60962 .60104 L
.61086 .6016 L
.61218 .60211 L
.61333 .60248 L
.61459 .60281 L
.61574 .60304 L
.61682 .60319 L
.61808 .60329 L
.61871 .60332 L
.61941 .60332 L
.62065 .60325 L
.62181 .60312 L
.62248 .60302 L
.62318 .60288 L
.62443 .60258 L
.62578 .60216 L
.62725 .6016 L
.62973 .6004 L
.63238 .59878 L
.63715 .59497 L
.64214 .58976 L
.64686 .58372 L
.6557 .56958 L
.67568 .52489 L
.69413 .47033 L
Mistroke
.7335 .32685 L
.77136 .18397 L
.79241 .1157 L
.81166 .06613 L
.82118 .04724 L
.8262 .0389 L
.83155 .03133 L
.8367 .02536 L
.84136 .02107 L
.84361 .01939 L
.84604 .01786 L
.84835 .01669 L
.85045 .01586 L
.85159 .0155 L
.85283 .01519 L
.85414 .01494 L
.85538 .01479 L
.85653 .01472 L
.85776 .01472 L
.85881 .01479 L
.85997 .01492 L
.86063 .01502 L
.86132 .01516 L
.86256 .01547 L
.86535 .01644 L
.86777 .0176 L
.87041 .01921 L
.87569 .02348 L
.88133 .02957 L
.89148 .0444 L
.90173 .06419 L
.91284 .09067 L
.933 .15021 L
.97393 .30026 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[17]:=",
  ImageSize->{309.875, 191.375},
  ImageMargins->{{42.6875, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004e0000_b000`400?l00000o`00003oo`3ooolf
0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`00o`3ooolf
0?ooo`006P3oool00`000000oooo0?ooo`3o0?oooaT0oooo000J0?ooo`030000003oool0oooo0?l0
oooo6@3oool00140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?oooaT0oooo
000A0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0o`3ooolI0?ooo`002P3oool4
000000<0oooo00<000000?ooo`3oool01P3oool300000680oooo1@00001R0?ooo`P000006`3oool5
000002L0oooo000A0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0H@3oool20000
00@0oooo00<000000?ooo`3oool0F`3oool4000000P0oooo0P00000G0?ooo`800000103oool30000
02D0oooo000A0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0H03oool00`000000
oooo0?ooo`050?ooo`800000F03oool3000000h0oooo0`00000C0?ooo`030000003oool0oooo00H0
oooo00<000000?ooo`3oool08`3oool00100oooo0P0000080?ooo`030000003oool0oooo05l0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`1C0?ooo`800000503oool200000100oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0R0?ooo`004@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo05d0oooo0P00000<0?ooo`030000003oool0oooo0500oooo0P00
000H0?ooo`8000003@3oool00`000000oooo0?ooo`0:0?ooo`8000008P3oool001X0oooo00<00000
0?ooo`3oool0G03oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo04d0oooo0P00000L
0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0O
0?ooo`006P3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo00d0oooo00<000000?oo
o`3oool0C03oool00`000000oooo0?ooo`0M0?ooo`8000002@3oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo01l0oooo000J0?ooo`030000003oool0oooo05/0oooo00<000000?ooo`3o
ool03`3oool00`000000oooo0?ooo`1:0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3o
ool01@3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01h0oooo000J0?ooo`030000
003oool0oooo05/0oooo00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`190?ooo`030000
003oool0oooo0280oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0?0?ooo`030000
003oool0oooo01h0oooo000J0?ooo`800000F`3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo04H0oooo0P00000V0?ooo`8000000`3oool00`000000oooo0?ooo`0A0?ooo`030000003o
ool0oooo01d0oooo000J0?ooo`030000003oool0oooo05X0oooo00<000000?ooo`3oool04@3oool0
0`000000oooo0?ooo`150?ooo`030000003oool0oooo02P0oooo00@000000?ooo`3oool00000503o
ool00`000000oooo0?ooo`0L0?ooo`006P3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0
oooo01<0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`0Z0?ooo`8000005@3oool0
0`000000oooo0?ooo`0L0?ooo`006P3oool00`000000oooo0?ooo`1I0?ooo`030000003oool0oooo
01<0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo
01@0oooo00<000000?ooo`3oool06`3oool001X0oooo00<000000?ooo`3oool0F03oool00`000000
oooo0?ooo`0E0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool0;03oool00`000000
oooo0000000E0?ooo`030000003oool0oooo01/0oooo000J0?ooo`030000003oool0oooo05P0oooo
00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo02d0oooo
00@000000?ooo`3oool000005@3oool00`000000oooo0?ooo`0J0?ooo`006P3oool00`000000oooo
0?ooo`1G0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0?@3oool00`000000oooo
0?ooo`0]0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool04P3oool00`000000oooo
0?ooo`0J0?ooo`006P3oool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo01L0oooo00<0
00000?ooo`3oool0?03oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`0I0?ooo`006P3oool2000005L0oooo00<00000
0?ooo`3oool06@3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo02h0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01T0oooo000J0?oo
o`030000003oool0oooo05H0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0i0?oo
o`030000003oool0oooo02l0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0A0?oo
o`030000003oool0oooo01T0oooo000J0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo01P0oooo000J0?ooo`030000
003oool0oooo05D0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0g0?ooo`030000
003oool0oooo02l0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0@0?ooo`030000
003oool0oooo01P0oooo000J0?ooo`030000003oool0oooo05D0oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo01L0oooo000J0?ooo`030000003oool0
oooo05@0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0
oooo0300oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0
oooo01L0oooo000J0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0c0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0303oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo01L0oooo000J0?ooo`030000003oool0oooo05@0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0300
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo01H0
oooo000J0?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool07`3oool00`000000oooo
0?ooo`0a0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool03`3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo01H0oooo000J0?ooo`800000E03oool00`000000oooo0?oo
o`0P0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?oo
o`0@0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool05P3oool001X0oooo00<00000
0?ooo`3oool0D`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02l0oooo00<00000
0?ooo`3oool0<@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool05@3oool001X0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo02h0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0B0?oo
o`030000003oool0oooo00d0oooo00<000000?ooo`3oool05@3oool001X0oooo00<000000?ooo`3o
ool0DP3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3o
ool0<@3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool05@3oool001X0oooo00<000000?ooo`3oool0DP3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo02`0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo00`0oooo00<000000?ooo`3oool0503oool001X0oooo00<000000?ooo`3oool0D@3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0<P3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0503o
ool001X0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0
oooo00`0oooo00<000000?ooo`3oool04`3oool001X0oooo00<000000?ooo`3oool0D@3oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0<`3oool00`00
0000oooo0?ooo`0F0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool04`3oool001X0
oooo0P00001A0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0:03oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool02`3oool00`000000
oooo0?ooo`0C0?ooo`006P3oool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo02D0oooo
00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0B0?ooo`006P3oool00`000000oooo0?oo
o`1?0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?oo
o`0d0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?oo
o`0B0?ooo`006P3oool00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo02L0oooo00<00000
0?ooo`3oool09P3oool00`000000oooo0?ooo`0d0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`0B0?ooo`006P3oool00`000000oooo0?ooo`1?0?oo
o`030000003oool0oooo02L0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0d0?oo
o`030000003oool0oooo01`0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0A0?oo
o`006P3oool00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool02P3oool00`000000oooo0?ooo`0A0?ooo`006P3oool00`000000oooo0?ooo`1>0?ooo`030000
003oool0oooo02T0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0e0?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0A0?ooo`00203o
ool3000000<0oooo00<000000?ooo`3oool00P0000070?ooo`030000003oool0oooo04h0oooo00<0
00000?ooo`3oool0:@3oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo03D0oooo00<0
00000?ooo`3oool07P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0100oooo0008
0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool000001P3oool00`000000oooo
0?ooo`1=0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`0e0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`0@0?ooo`000`3oool4000000050?ooo`000000oooo0?ooo`0000001`3oool00`000000oooo
0?ooo`040?ooo`<00000C@3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo0200oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo00P0oooo
00<000000?ooo`3oool0403oool000P0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000060?ooo`030000003oool0oooo04d0oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`0O0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool08@3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0100oooo00080?ooo`040000003oool0oooo000000@0
oooo0`0000070?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0;@3oool00`000000
oooo0?ooo`0N0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo00l0oooo00080?ooo`040000003oool0oooo000000D0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo02d0
oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0280
oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0?0?ooo`002@3oool2000000H0oooo
0`0000060?ooo`030000003oool0oooo04`0oooo00<000000?ooo`3oool0;@3oool00`000000oooo
0?ooo`0M0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool08`3oool00`000000oooo
0?ooo`070?ooo`030000003oool0oooo00l0oooo000J0?ooo`030000003oool0oooo04`0oooo00<0
00000?ooo`3oool0;@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo03L0oooo00<0
00000?ooo`3oool0903oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00l0oooo000J
0?ooo`030000003oool0oooo04/0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0J
0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo00h0oooo000J0?ooo`030000003oool0oooo04/0oooo00<000000?oo
o`3oool0;`3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo03P0oooo00<000000?oo
o`3oool09P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00h0oooo000J0?ooo`80
0000C03oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3o
ool0>03oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3o
ool03P3oool001X0oooo00<000000?ooo`3oool0B`3oool00`000000oooo0?ooo`0`0?ooo`030000
003oool0oooo01P0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool03@3oool001X0oooo00<000000?ooo`3oool0BP3o
ool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0>@3o
ool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03@3o
ool001X0oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool03@3oool001X0oooo00<000000?ooo`3oool0BP3oool00`00
0000oooo0?ooo`0a0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0>@3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool03@3oool001X0
oooo00<000000?ooo`3oool0BP3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0>P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool0B@3oool00`000000oooo
0?ooo`0c0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool0>P3oool00`000000oooo
0?ooo`0/0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0303oool001X0oooo00<0
00000?ooo`3oool0B@3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo01<0oooo00<0
00000?ooo`3oool0>`3oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool0B@3oool00`000000oooo0?ooo`0d
0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0^
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0303oool001X0oooo0P00001:0?oo
o`030000003oool0oooo03@0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0k0?oo
o`030000003oool0oooo02h0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0;0?oo
o`006P3oool00`000000oooo0?ooo`180?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3o
ool0403oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3o
ool00`3oool00`000000oooo0?ooo`0;0?ooo`006P3oool00`000000oooo0?ooo`180?ooo`030000
003oool0oooo03D0oooo00<000000?ooo`3oool0403oool00`000000oooo0?ooo`0l0?ooo`030000
003oool0oooo02l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0;0?ooo`006P3o
ool00`000000oooo0?ooo`180?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`170?ooo`030000003oool0
oooo03L0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0
oooo0380oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0:0?ooo`006P3oool00`00
0000oooo0?ooo`170?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool03@3oool00`00
0000oooo0?ooo`0m0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo03P0
oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo03@0
oooo00D000000?ooo`3oool0oooo0000000<0?ooo`006P3oool00`000000oooo0?ooo`170?ooo`03
0000003oool0oooo03P0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0m0?ooo`03
0000003oool0oooo03@0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`090?ooo`00
6P3oool2000004L0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo03h0oooo00<000000?ooo`3oool0=@3oool01@000000oooo0?ooo`3oool0000000/0
oooo000J0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0>@3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0=@3oool01@000000oooo
0?ooo`3oool0000000/0oooo000J0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3oool0
>P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo03h0oooo00<000000?ooo`3oool0
=`3oool010000000oooo0?ooo`00000;0?ooo`006P3oool00`000000oooo0?ooo`160?ooo`030000
003oool0oooo03X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0n0?ooo`030000
003oool0oooo03L0oooo00D000000?ooo`3oool0oooo0000000:0?ooo`006P3oool00`000000oooo
0?ooo`150?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`0o0?ooo`030000003oool0oooo03P0oooo00@000000?ooo`3oool000002P3oool001X0oooo
00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0h0?ooo`040000003oool0oooo000000X0
oooo000J0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool06P3oool00`000000oooo
0?ooo`0:0?ooo`@00000:@3oool2000000<0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo01h0oooo00<000000?ooo`3oool01P3oool2000002X0oooo
0P0000060?ooo`040000003oool0oooo000000T0oooo000J0?ooo`030000003oool0oooo02P0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo02P0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`050?ooo`040000003oool0oooo0000
02P0oooo00@000000?ooo`3oool000001@3oool010000000oooo0?ooo`0000090?ooo`006P3oool0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0Z0?ooo`040000003oool0oooo000000H0oooo00<000000?ooo`3oool0703o
ool5000001l0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0V0?ooo`040000003o
ool0oooo000000H0oooo00<000000?ooo`0000002@3oool001X0oooo0P00000Y0?ooo`030000003o
ool0oooo01T0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0Y0?ooo`030000003o
ool0oooo0080oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0L0?ooo`040000003o
ool0oooo000001l0oooo00<000000?ooo`3oool01P3oool010000000oooo0?ooo`00000X0?ooo`04
0000003oool0oooo000000H0oooo00<000000?ooo`0000002@3oool001X0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00h0oooo00<000000?ooo`3o
ool0:@3oool01@000000oooo0?ooo`3oool0000000@0oooo00<000000?ooo`3oool07P3oool00`00
0000oooo0000000O0?ooo`030000003oool0oooo00H0oooo0`00000Y0?ooo`<00000203oool00`00
0000oooo000000080?ooo`006P3oool00`000000oooo0?ooo`0W0?ooo`8000006`3oool00`000000
oooo0?ooo`0;0?ooo`040000003oool0oooo000002P0oooo00@000000?ooo`3oool000000`3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo01l0oooo0P00000O0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool0:03oool010000000oooo0?ooo`0000070?ooo`030000003o
ool0000000P0oooo000J0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0603oool0
0`000000oooo0?ooo`0=0?ooo`800000:P3oool2000000D0oooo00@000000?ooo`3oool000008`3o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo00L0oooo0`00000Y0?ooo`8000002@3o
ool2000000P0oooo000J0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0@03oool0
10000000oooo0?ooo`0000120?ooo`030000003oool0oooo03l0oooo0P0000080?ooo`006P3oool0
0`000000oooo0?ooo`130?ooo`030000003oool0oooo0400oooo00<000000?ooo`000000@`3oool0
0`000000oooo0?ooo`100?ooo`8000001`3oool001X0oooo00<000000?ooo`3oool0@`3oool00`00
0000oooo0?ooo`100?ooo`030000003oool0000004<0oooo00<000000?ooo`3oool0@03oool20000
00L0oooo000J0?ooo`030000003oool0oooo0480oooo00<000000?ooo`3oool0@P3oool00`000000
oooo0?ooo`120?ooo`030000003oool0oooo0440oooo00<000000?ooo`3oool01@3oool001<0oooo
o`00000S000000006P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo0P00
00080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00<0oooo0P0000000`3oool000000?ooo`070?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`050?ooo`030000003o
ool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`050000003o
ool0oooo0?ooo`0000000`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo
0@0000010?ooo`006P3oool2000002X0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?oo
o`0A0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool00P3oool010000000oooo0?oo
o`00000S0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3oool02P3oool00`000000oooo
0?ooo`0Y0?ooo`030000003oool0oooo0140oooo000J0?ooo`800000@P3oool00`000000oooo0?oo
o`110?ooo`040000003oool0oooo00000480oooo00<000000?ooo`3oool0BP3oool001X0oooo0`00
00110?ooo`030000003oool0oooo0400oooo00D000000?ooo`3oool0oooo000000110?ooo`030000
003oool0oooo04/0oooo000J0?ooo`<00000@@3oool00`000000oooo0?ooo`100?ooo`050000003o
ool0oooo0?ooo`000000@@3oool00`000000oooo0?ooo`1;0?ooo`006P3oool010000000oooo0000
0000000o0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0n0?ooo`030000003oool0oooo04/0oooo000J0?ooo`040000003oool00000000003l0oooo
00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo03d0oooo
00<000000?ooo`3oool0C03oool001X0oooo00D000000?ooo`000000oooo0000000n0?ooo`030000
003oool0oooo03l0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0m0?ooo`030000
003oool0oooo04`0oooo000J0?ooo`<0000000<0oooo0000003oool0?03oool00`000000oooo0?oo
o`100?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?oo
o`1<0?ooo`006P3oool01P000000oooo0?ooo`000000oooo000003`0oooo00<000000?ooo`3oool0
?`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0
C03oool001X0oooo00H000000?ooo`3oool000000?ooo`00000l0?ooo`030000003oool0oooo03l0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo04d0
oooo000J0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0>@3oool00`000000
oooo0?ooo`0n0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0>`3oool00`000000
oooo0?ooo`1=0?ooo`006P3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo03P0
oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool0C@3oool001X0oooo00D000000?ooo`3oool0oooo000000020?ooo`03
0000003oool0oooo03L0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo03X0oooo00<000000?ooo`3oool0C@3oool001X0oooo00D000000?ooo`3oool0
oooo000000020?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3oool0?P3oool00`000000
oooo0?ooo`090?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0CP3oool001X0oooo
00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3o
ool0?@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3o
ool0CP3oool001X0oooo0P0000020?ooo`050000003oool0oooo0?ooo`000000=`3oool00`000000
oooo0?ooo`0n0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0>03oool00`000000
oooo0?ooo`1>0?ooo`006P3oool00`000000oooo0?ooo`020?ooo`050000003oool0oooo0?ooo`00
0000=P3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3o
ool0>03oool00`000000oooo0?ooo`1>0?ooo`006P3oool00`000000oooo0?ooo`020?ooo`050000
003oool0oooo0?ooo`000000=P3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0=`3oool00`000000oooo0?ooo`1?0?ooo`006P3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0<P3oool00`000000oooo
0?ooo`0m0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0=`3oool00`000000oooo
0?ooo`1?0?ooo`006P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0<P3oool00`000000oooo0?ooo`0m0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool0=P3oool00`000000oooo0?ooo`1?0?ooo`006P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0l
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`1?
0?ooo`006P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0<03oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0=@3oool00`000000oooo0?ooo`1@0?ooo`006P3oool00`000000oooo0?ooo`030?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0l0?ooo`03
0000003oool0oooo0100oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`1@0?ooo`00
6P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0
;P3oool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
=03oool00`000000oooo0?ooo`1@0?ooo`006P3oool2000000D0oooo00<000000?ooo`3oool00`3o
ool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool04P3o
ool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo0540oooo000J0?ooo`030000003oool0
oooo00@0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0/0?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0c0?ooo`030000003oool0
oooo0540oooo000J0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`0/0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0503oool00`00
0000oooo0?ooo`0b0?ooo`030000003oool0oooo0540oooo000J0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo03/0
oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo0540
oooo000J0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`0[0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool05@3oool00`000000oooo
0?ooo`0a0?ooo`030000003oool0oooo0580oooo000J0?ooo`030000003oool0oooo00D0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo03X0oooo00<0
00000?ooo`3oool05P3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0580oooo000J
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`0Y
0?ooo`030000003oool0oooo03X0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0`
0?ooo`030000003oool0oooo0580oooo000J0?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo03T0oooo00<000000?oo
o`3oool0603oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo0580oooo000J0?ooo`80
00001`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool0>@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3o
ool0D`3oool001X0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo02H0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0I0?ooo`030000
003oool0oooo02l0oooo00<000000?ooo`3oool0D`3oool001X0oooo00<000000?ooo`3oool01P3o
ool00`000000oooo0?ooo`070?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0>@3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo02h0oooo00<000000?ooo`3oool0D`3o
ool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo02D0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo02h0oooo00<000000?ooo`3oool0D`3oool001X0oooo00<000000?ooo`3oool01`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3oool0>03oool00`00
0000oooo0?ooo`0L0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0E03oool001X0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo02<0
oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02d0
oooo00<000000?ooo`3oool0E03oool001X0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0=`3oool00`000000oooo
0?ooo`0N0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0E03oool000P0oooo0`00
00030?ooo`030000003oool0oooo008000001`3oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00L0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0g0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`1D0?ooo`00203oool0
10000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000H0oooo00<000000?ooo`3oool0
203oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
=`3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0
E@3oool000P0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`040?ooo`<00000
203oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool0
=`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0
E@3oool000P0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000060?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0O0?ooo`03
0000003oool0oooo03H0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0Z0?ooo`03
0000003oool0oooo05D0oooo00080?ooo`040000003oool0oooo000000@0oooo0`0000070?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0M0?ooo`03
0000003oool0oooo03H0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0Z0?ooo`03
0000003oool0oooo05D0oooo00080?ooo`040000003oool0oooo000000D0oooo00<000000?ooo`3o
ool01P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool07@3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo02<0oooo00<000000?ooo`3o
ool0:03oool00`000000oooo0?ooo`1F0?ooo`002@3oool2000000H0oooo0`0000060?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0oooo03D0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0X0?ooo`030000
003oool0oooo05H0oooo000J0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool02P3o
ool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3oool09@3o
ool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo05H0oooo000J0?ooo`030000003oool0
oooo00X0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0
oooo03D0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0
oooo05L0oooo000J0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool02P3oool00`00
0000oooo0?ooo`0I0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool09`3oool00`00
0000oooo0?ooo`0V0?ooo`030000003oool0oooo05L0oooo000J0?ooo`800000303oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0=03oool00`000000
oooo0?ooo`0X0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0F03oool001X0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo02D0oooo
00<000000?ooo`3oool0F03oool001X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?oo
o`0Z0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0F03oool001X0oooo00<00000
0?ooo`3oool0303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0=03oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0oooo02<0oooo00<00000
0?ooo`3oool0F@3oool001X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0=0?oo
o`030000003oool0oooo01<0oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0[0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool0F@3oool001X0oooo00<000000?ooo`3o
ool03@3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool0<P3oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3o
ool0F@3oool001X0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0=0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool0<`3oool00`000000oooo0?ooo`0]0?ooo`030000
003oool0oooo0240oooo00<000000?ooo`3oool0FP3oool001X0oooo00<000000?ooo`3oool03@3o
ool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0<P3o
ool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0FP3o
ool001X0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0
oooo00l0oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0
oooo01l0oooo00<000000?ooo`3oool0F`3oool001X0oooo0P00000?0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo
0300oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`1K0?ooo`006P3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03@3oool00`000000
oooo0?ooo`0a0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool07`3oool00`000000
oooo0?ooo`1K0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`1L0?ooo`006P3oool00`000000oooo0?oo
o`0?0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool02P3oool00`000000oooo0?oo
o`0`0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?oo
o`1L0?ooo`006P3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo03D0oooo00<00000
0?ooo`3oool07@3oool00`000000oooo0?ooo`1L0?ooo`006P3oool00`000000oooo0?ooo`0@0?oo
o`030000003oool0oooo0100oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0`0?oo
o`030000003oool0oooo03D0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`1M0?oo
o`006P3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool01`3oool00`000000oooo0?ooo`0_0?ooo`030000003oool0oooo03L0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`1M0?ooo`006P3oool00`000000oooo0?ooo`0A0?ooo`030000
003oool0oooo0140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0^0?ooo`030000
003oool0oooo03P0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1N0?ooo`006P3o
ool200000180oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo02h0oooo00<000000?ooo`3oool0>@3oool00`000000oooo0?ooo`0J0?ooo`030000003o
ool0oooo05h0oooo000J0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool04`3oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0>`3oool0
0`000000oooo0?ooo`0I0?ooo`030000003oool0oooo05h0oooo000J0?ooo`030000003oool0oooo
0180oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo
02`0oooo00<000000?ooo`3oool0?03oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo
05l0oooo000J0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool04`3oool01@000000
oooo0?ooo`3oool0000002l0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo05l0oooo000J0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool04`3oool010000000oooo0?ooo`00000^0?ooo`030000003oool0oooo03h0oooo00<000000?oo
o`3oool05`3oool00`000000oooo0?ooo`1O0?ooo`006P3oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo01@0oooo0P00000^0?ooo`030000003oool0oooo0400oooo00<000000?ooo`3o
ool05@3oool00`000000oooo0?ooo`1P0?ooo`006P3oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`120?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool0H03oool001X0oooo00<000000?ooo`3oool0503o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0000002/0oooo00<000000?ooo`3oool0@`3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0640oooo000J0?ooo`030000003oool0
oooo01D0oooo00<000000?ooo`3oool04@3oool01@000000oooo0?ooo`3oool0000002T0oooo00<0
00000?ooo`3oool0A@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo0640oooo000J
0?ooo`8000005P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0903oool2000004P0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`1R
0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00l0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo04T0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`1S0?ooo`006P3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo00h0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0P0?ooo`03
0000003oool0oooo04/0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1S0?ooo`00
6P3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
203oool2000001h0oooo0P00001>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0
I03oool001X0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`1?0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool0I@3oool001X0oooo00<000000?ooo`3oool06@3oool0
0`000000oooo0?ooo`090?ooo`030000003oool0oooo00`0oooo0P00000H0?ooo`800000D`3oool0
0`000000oooo0?ooo`0:0?ooo`030000003oool0oooo06D0oooo000A0?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool06P3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo
00l0oooo0P00000E0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool01`3oool20000
06P0oooo000A0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`060?ooo`030000003oool0oooo0140oooo1000000>0?ooo`<00000F03oool2000000H0
oooo00<000000?ooo`3oool0J03oool00140oooo00<000000?ooo`3oool01P3oool3000001`0oooo
2000000H0?ooo`h00000G03oool7000006/0oooo000A0?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool0o`3ooolI0?ooo`004@3oool00`000000oooo0?ooo`060?ooo`030000003oool0
oooo0?l0oooo6@3oool00100oooo0P0000080?ooo`030000003oool0oooo0?l0oooo6@3oool00140
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?oooaT0oooo000J0?ooo`030000
003oool0oooo0?l0oooo6@3oool00?l0oooo=P3oool00?l0oooo=P3oool00?l0oooo=P3oool00?l0
oooo=P3oool00?l0oooo=P3oool00001\
\>"],
  ImageRangeCache->{{{0, 308.875}, {190.375, 0}} -> {-0.599591, -1.11738, \
0.022792, 0.0117387}},
  CellTags->{"b:0.2", "IPM3.1.1.3"}]
}, Open  ]],

Cell["\<\
Here is a plot of the sinc function, given in the previous section.\
\>", "TextFirst",
  PageBreakAbove->True,
  CellTags->"IPM3.1.1.3"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
        Piecewise[\ {{1, \ x \[Equal] 0}}, \ Sin[x]/x], {x, \(-2\) \[Pi], 
          2  \[Pi]}];\)\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->"IPM3.1.1.3"],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.5 0.0757881 0.11976 0.48356 [
[.04527 .10726 -7.59375 -11.875 ]
[.04527 .10726 7.59375 0 ]
[.19685 .10726 -7.59375 -11.875 ]
[.19685 .10726 7.59375 0 ]
[.34842 .10726 -7.59375 -11.875 ]
[.34842 .10726 7.59375 0 ]
[.65158 .10726 -4.5 -11.875 ]
[.65158 .10726 4.5 0 ]
[.80315 .10726 -4.5 -11.875 ]
[.80315 .10726 4.5 0 ]
[.95473 .10726 -4.5 -11.875 ]
[.95473 .10726 4.5 0 ]
[.4875 .02305 -22.6875 -5.9375 ]
[.4875 .02305 0 5.9375 ]
[.4875 .21647 -16.5 -5.9375 ]
[.4875 .21647 0 5.9375 ]
[.4875 .31318 -16.5 -5.9375 ]
[.4875 .31318 0 5.9375 ]
[.4875 .4099 -16.5 -5.9375 ]
[.4875 .4099 0 5.9375 ]
[.4875 .50661 -16.5 -5.9375 ]
[.4875 .50661 0 5.9375 ]
[.4875 .60332 -9 -5.9375 ]
[.4875 .60332 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.04527 .11976 m
.04527 .12601 L
s
gsave
.04527 .10726 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.19685 .11976 m
.19685 .12601 L
s
gsave
.19685 .10726 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.34842 .11976 m
.34842 .12601 L
s
gsave
.34842 .10726 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.65158 .11976 m
.65158 .12601 L
s
gsave
.65158 .10726 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.80315 .11976 m
.80315 .12601 L
s
gsave
.80315 .10726 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.95473 .11976 m
.95473 .12601 L
s
gsave
.95473 .10726 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.125 Mabswid
.08317 .11976 m
.08317 .12351 L
s
.12106 .11976 m
.12106 .12351 L
s
.15895 .11976 m
.15895 .12351 L
s
.23474 .11976 m
.23474 .12351 L
s
.27264 .11976 m
.27264 .12351 L
s
.31053 .11976 m
.31053 .12351 L
s
.38632 .11976 m
.38632 .12351 L
s
.42421 .11976 m
.42421 .12351 L
s
.46211 .11976 m
.46211 .12351 L
s
.53789 .11976 m
.53789 .12351 L
s
.57579 .11976 m
.57579 .12351 L
s
.61368 .11976 m
.61368 .12351 L
s
.68947 .11976 m
.68947 .12351 L
s
.72736 .11976 m
.72736 .12351 L
s
.76526 .11976 m
.76526 .12351 L
s
.84105 .11976 m
.84105 .12351 L
s
.87894 .11976 m
.87894 .12351 L
s
.91683 .11976 m
.91683 .12351 L
s
.00738 .11976 m
.00738 .12351 L
s
.99262 .11976 m
.99262 .12351 L
s
.25 Mabswid
0 .11976 m
1 .11976 L
s
.5 .02305 m
.50625 .02305 L
s
gsave
.4875 .02305 -83.6875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.5 .21647 m
.50625 .21647 L
s
gsave
.4875 .21647 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.2) show
1.000 setlinewidth
grestore
.5 .31318 m
.50625 .31318 L
s
gsave
.4875 .31318 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.4) show
1.000 setlinewidth
grestore
.5 .4099 m
.50625 .4099 L
s
gsave
.4875 .4099 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.6) show
1.000 setlinewidth
grestore
.5 .50661 m
.50625 .50661 L
s
gsave
.4875 .50661 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.8) show
1.000 setlinewidth
grestore
.5 .60332 m
.50625 .60332 L
s
gsave
.4875 .60332 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.5 .04723 m
.50375 .04723 L
s
.5 .0714 m
.50375 .0714 L
s
.5 .09558 m
.50375 .09558 L
s
.5 .14394 m
.50375 .14394 L
s
.5 .16812 m
.50375 .16812 L
s
.5 .19229 m
.50375 .19229 L
s
.5 .24065 m
.50375 .24065 L
s
.5 .26483 m
.50375 .26483 L
s
.5 .28901 m
.50375 .28901 L
s
.5 .33736 m
.50375 .33736 L
s
.5 .36154 m
.50375 .36154 L
s
.5 .38572 m
.50375 .38572 L
s
.5 .43407 m
.50375 .43407 L
s
.5 .45825 m
.50375 .45825 L
s
.5 .48243 m
.50375 .48243 L
s
.5 .53079 m
.50375 .53079 L
s
.5 .55496 m
.50375 .55496 L
s
.5 .57914 m
.50375 .57914 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .11976 m
.06244 .07889 L
.08255 .05832 L
.10458 .03865 L
.11448 .03129 L
.12507 .0247 L
.1302 .02203 L
.13506 .01987 L
.13975 .01811 L
.14415 .01679 L
.14877 .01574 L
.15121 .01532 L
.15385 .015 L
.15516 .01488 L
.15586 .01483 L
.1566 .01479 L
.15731 .01476 L
.15796 .01474 L
.15921 .01472 L
.16043 .01472 L
.16174 .01476 L
.16298 .01483 L
.16413 .01492 L
.16634 .01515 L
.1684 .01546 L
.17066 .01589 L
.17309 .01646 L
.17805 .01798 L
.18257 .0198 L
.19248 .02523 L
.20165 .03207 L
.22241 .05403 L
.24108 .08135 L
.26168 .1193 L
.30188 .21287 L
.34057 .31776 L
.38171 .42955 L
.40223 .47993 L
.42133 .52104 L
.44093 .55583 L
.45944 .58057 L
.46926 .59017 L
.47443 .59419 L
.4799 .59767 L
.4845 .59996 L
.48695 .60093 L
.48957 .60179 L
.49198 .60242 L
.49418 .60284 L
.49523 .603 L
Mistroke
.49639 .60314 L
.49748 .60323 L
.49849 .60329 L
.49978 .60332 L
.50043 .60332 L
.50114 .6033 L
.5024 .60324 L
.50358 .60314 L
.50429 .60306 L
.50497 .60297 L
.50624 .60277 L
.50908 .60216 L
.51156 .60145 L
.51386 .60063 L
.51905 .59824 L
.52335 .59571 L
.52808 .59233 L
.53767 .58366 L
.55705 .55892 L
.57792 .52251 L
.61912 .42743 L
.65879 .31954 L
.69695 .21589 L
.73757 .12084 L
.75634 .08568 L
.77666 .05522 L
.7878 .04211 L
.79805 .03232 L
.80785 .02502 L
.81327 .02184 L
.81821 .01945 L
.82307 .0176 L
.82767 .01627 L
.8302 .01571 L
.83253 .01531 L
.8351 .01499 L
.8364 .01487 L
.83708 .01483 L
.83783 .01478 L
.83898 .01474 L
.8402 .01472 L
.84132 .01472 L
.84237 .01474 L
.84361 .0148 L
.84474 .01487 L
.84604 .01499 L
.84727 .01512 L
.84949 .01543 L
.85153 .01579 L
.85615 .01687 L
Mistroke
.86082 .01831 L
.86583 .02024 L
.87489 .02467 L
.89552 .03873 L
.93677 .07806 L
.97619 .11976 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[18]:=",
  ImageSize->{309.938, 191.438},
  ImageMargins->{{42.8125, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004e0000_b000`400?l00000o`00003oVP3oool0
0`000000oooo0?ooo`2H0?ooo`00VP3oool00`000000oooo0?ooo`2H0?ooo`00VP3oool00`000000
oooo0?ooo`2H0?ooo`00VP3oool00`000000oooo0?ooo`2H0?ooo`00VP3oool00`000000oooo0?oo
o`2H0?ooo`00;@3oool:00000540oooo0`0000030?ooo`030000003oool0000000<000001P3oool0
0`000000oooo0?ooo`1P0?ooo`/00000;@3oool002X0oooo0`00000:0?ooo`<00000CP3oool01000
0000oooo0?ooo`0000050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0GP3oool2
000000/0oooo0`00000Z0?ooo`00:@3oool00`000000oooo0?ooo`0>0?ooo`800000AP3oool40000
0080oooo00@000000?ooo`3oool000001P3oool00`000000oooo0?ooo`050?ooo`<00000G@3oool0
0`000000oooo0?ooo`0>0?ooo`800000:03oool002L0oooo0P00000C0?ooo`030000003oool0oooo
04T0oooo00@000000?ooo`3oool000001P3oool00`000000oooo0?ooo`050?ooo`030000003oool0
oooo05/0oooo0P00000C0?ooo`030000003oool0oooo02D0oooo000U0?ooo`8000005P3oool20000
04T0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo05T0oooo0P00000F0?ooo`8000009@3oool002@0oooo00<000000?ooo`3oool0603oool00`00
0000oooo0?ooo`160?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool000001P3o
ool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool08P3o
ool00280oooo0P00000L0?ooo`030000003oool0oooo04H0oooo0P0000060?ooo`8000001`3oool0
0`000000oooo0?ooo`1G0?ooo`030000003oool0oooo01X0oooo0P00000R0?ooo`008@3oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo05H0oooo00<000000?ooo`3oool0EP3oool00`00
0000oooo0?ooo`0M0?ooo`030000003oool0oooo01l0oooo000P0?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool0E@3oool00`000000oooo0?ooo`1E0?ooo`030000003oool0oooo01l0
oooo00<000000?ooo`3oool07P3oool001l0oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`1E0?ooo`030000003oool0oooo05@0oooo00<000000?ooo`3oool08@3oool00`000000oooo
0?ooo`0M0?ooo`007@3oool2000002@0oooo00<000000?ooo`3oool0E03oool2000005@0oooo00<0
00000?ooo`3oool08`3oool00`000000oooo0?ooo`0L0?ooo`00703oool00`000000oooo0?ooo`0U
0?ooo`030000003oool0oooo05<0oooo00<000000?ooo`3oool0D`3oool00`000000oooo0?ooo`0T
0?ooo`030000003oool0oooo01/0oooo000K0?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool0DP3oool00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo02H0oooo00<000000?oo
o`3oool06P3oool001X0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1A0?ooo`03
0000003oool0oooo0540oooo00<000000?ooo`3oool0:03oool2000001X0oooo000I0?ooo`030000
003oool0oooo02X0oooo00<000000?ooo`3oool0D@3oool00`000000oooo0?ooo`1@0?ooo`030000
003oool0oooo02/0oooo00<000000?ooo`3oool05`3oool001P0oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`1@0?ooo`030000003oool0oooo0500oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`0F0?ooo`005`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo04l0oooo00<000000?ooo`3oool0C`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0
oooo01D0oooo000F0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0CP3oool20000
04l0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0D0?ooo`005@3oool00`000000
oooo0?ooo`0a0?ooo`030000003oool0oooo04h0oooo00<000000?ooo`3oool0C@3oool00`000000
oooo0?ooo`0b0?ooo`030000003oool0oooo01<0oooo000D0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool0C@3oool00`000000oooo0?ooo`1=0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool04P3oool00140oooo0`00000]0?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool07P3oool4000002X0oooo00<000000?ooo`3oool0:@3oool4000001l0oooo00<0
00000?ooo`3oool02`3oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo008000003`3o
ool00100oooo00@000000?ooo`0000000000;@3oool00`000000oooo0?ooo`070?ooo`030000003o
ool0oooo01l0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0Z0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0X0?ooo`8000000P3o
ool00`000000oooo0?ooo`0<0?ooo`002P3oool400000080oooo0P0000000`3oool000000?ooo`0T
0?ooo`@0000000<0oooo0000000000000`0000090?ooo`030000003oool0oooo01P0oooo10000003
0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo00T0oooo1@00000Z0?ooo`040000003oool0oooo000000h0oooo000@
0?ooo`040000003oool0oooo000002X0oooo00@000000?ooo`3oool000002P3oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0:`3oool00`000000oooo
0?ooo`0L0?ooo`030000003oool0oooo00X0oooo00@000000?ooo`3oool00000:`3oool200000003
0?ooo`000000oooo00d0oooo000?0?ooo`@00000;03oool00`000000oooo0000000;0?ooo`030000
003oool0oooo01l0oooo00<000000?ooo`3oool0:03oool2000002d0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0000002/0oooo0`00000?0?ooo`003P3o
ool00`000000oooo000000020?ooo`030000003oool0oooo02X0oooo0P00000;0?ooo`030000003o
ool0oooo01`0oooo00@000000?ooo`3oool00000:P3oool00`000000oooo0?ooo`0Y0?ooo`040000
003oool0oooo000001`0oooo00<000000?ooo`3oool03@3oool2000002/0oooo00@000000?ooo`3o
ool000003P3oool000h0oooo00<000000?ooo`3oool00P00000^0?ooo`030000003oool0oooo00X0
oooo00<000000?ooo`3oool0703oool2000002/0oooo00<000000?ooo`3oool0:P3oool2000001d0
oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0Z0?ooo`<000003P3oool000d0oooo
00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`190?ooo`030000003oool0oooo04P0oooo
00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0;0?ooo`00303oool00`000000oooo0?oo
o`100?ooo`030000003oool0oooo04P0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?oo
o`100?ooo`030000003oool0oooo00X0oooo000;0?ooo`030000003oool0oooo0440oooo00<00000
0?ooo`3oool0B03oool00`000000oooo0?ooo`170?ooo`030000003oool0oooo0480oooo00<00000
0?ooo`3oool02@3oool000X0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`170?oo
o`030000003oool0oooo04L0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`080?oo
o`002@3oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo04L0oooo00<000000?ooo`3o
ool0AP3oool00`000000oooo0?ooo`150?ooo`030000003oool0oooo00L0oooo00000`3oool00000
0000003o0000038000000@3oool000<0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`090?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?oo
o`080?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool01@3oool010000000oooo0?oo
o`00000;0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`090?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`080?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0203oool01@000000oooo
0?ooo`3oool0000000L0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`080?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`03
0000003oool0oooo00T0oooo00<000000?ooo`3oool00@3oool000l0oooo00<000000?ooo`3oool0
:`3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
:`3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo00d0oooo001B0?ooo`030000003o
ool0oooo04D0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`1@0?ooo`00D`3oool0
0`000000oooo0?ooo`140?ooo`030000003oool0oooo04@0oooo00<000000?ooo`3oool0D@3oool0
05<0oooo00<000000?ooo`3oool0A03oool00`000000oooo0?ooo`140?ooo`030000003oool0oooo
0540oooo001D0?ooo`030000003oool0oooo04<0oooo00<000000?ooo`3oool0@`3oool00`000000
oooo0?ooo`1B0?ooo`00E03oool00`000000oooo0?ooo`130?ooo`800000A03oool00`000000oooo
0?ooo`1B0?ooo`00E03oool00`000000oooo0?ooo`130?ooo`030000003oool0oooo0480oooo00<0
00000?ooo`3oool0D`3oool005D0oooo00<000000?ooo`3oool0@P3oool00`000000oooo0?ooo`12
0?ooo`030000003oool0oooo05<0oooo001E0?ooo`030000003oool0oooo0480oooo00<000000?oo
o`3oool0@P3oool00`000000oooo0?ooo`1C0?ooo`00EP3oool00`000000oooo0?ooo`110?ooo`03
0000003oool0oooo0440oooo00<000000?ooo`3oool0E03oool005H0oooo00<000000?ooo`3oool0
@@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo05@0oooo001G0?ooo`030000003o
ool0oooo0400oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`1E0?ooo`00E`3oool0
0`000000oooo0?ooo`100?ooo`800000@@3oool00`000000oooo0?ooo`1E0?ooo`00E`3oool00`00
0000oooo0?ooo`100?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0EP3oool005P0
oooo00<000000?ooo`3oool0?`3oool00`000000oooo0?ooo`0o0?ooo`030000003oool0oooo05H0
oooo001H0?ooo`030000003oool0oooo03l0oooo00<000000?ooo`3oool0?P3oool00`000000oooo
0?ooo`1G0?ooo`00F@3oool00`000000oooo0?ooo`0n0?ooo`030000003oool0oooo03h0oooo00<0
00000?ooo`3oool0E`3oool005T0oooo00<000000?ooo`3oool0?P3oool00`000000oooo0?ooo`0m
0?ooo`030000003oool0oooo05P0oooo001J0?ooo`030000003oool0oooo03d0oooo00<000000?oo
o`3oool0?@3oool00`000000oooo0?ooo`1H0?ooo`00FP3oool00`000000oooo0?ooo`0m0?ooo`03
0000003oool0oooo03d0oooo00<000000?ooo`3oool0F03oool005X0oooo00<000000?ooo`3oool0
?@3oool2000003d0oooo00<000000?ooo`3oool0F@3oool005/0oooo00<000000?ooo`3oool0?03o
ool00`000000oooo0?ooo`0l0?ooo`030000003oool0oooo05T0oooo001K0?ooo`030000003oool0
oooo03`0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`1J0?ooo`00G03oool00`00
0000oooo0?ooo`0k0?ooo`030000003oool0oooo03/0oooo00<000000?ooo`3oool0FP3oool005`0
oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo05/0
oooo001M0?ooo`030000003oool0oooo02P0oooo0`0000030?ooo`030000003oool0000000<00000
1P3oool00`000000oooo0?ooo`0j0?ooo`030000003oool0oooo05/0oooo001M0?ooo`030000003o
ool0oooo02P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`060?ooo`030000
003oool0oooo03T0oooo00<000000?ooo`3oool0G03oool005d0oooo00<000000?ooo`3oool0:03o
ool010000000oooo0?ooo`0000060?ooo`030000003oool0oooo00D0oooo0`00000i0?ooo`030000
003oool0oooo05`0oooo001N0?ooo`030000003oool0oooo02L0oooo00@000000?ooo`3oool00000
1P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3oool0
G03oool005h0oooo00<000000?ooo`3oool09`3oool010000000oooo0?ooo`0000070?ooo`030000
003oool0oooo00@0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`1M0?ooo`00GP3o
ool00`000000oooo0?ooo`0W0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool0
00001P3oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo05d0oooo001O0?ooo`030000
003oool0oooo02L0oooo0P0000060?ooo`8000001`3oool00`000000oooo0?ooo`0h0?ooo`030000
003oool0oooo05d0oooo001O0?ooo`030000003oool0oooo03P0oooo00<000000?ooo`3oool0=`3o
ool00`000000oooo0?ooo`1N0?ooo`00H03oool00`000000oooo0?ooo`0g0?ooo`030000003oool0
oooo03L0oooo00<000000?ooo`3oool0GP3oool00600oooo00<000000?ooo`3oool0=`3oool00`00
0000oooo0?ooo`0g0?ooo`030000003oool0oooo05h0oooo001P0?ooo`030000003oool0oooo03L0
oooo0P00000g0?ooo`030000003oool0oooo05l0oooo001Q0?ooo`030000003oool0oooo03H0oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`1O0?ooo`00H@3oool00`000000oooo0?oo
o`0f0?ooo`030000003oool0oooo03H0oooo00<000000?ooo`3oool0G`3oool00640oooo00<00000
0?ooo`3oool0=P3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo0600oooo001R0?oo
o`030000003oool0oooo03D0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`1P0?oo
o`00HP3oool00`000000oooo0?ooo`0e0?ooo`030000003oool0oooo03D0oooo00<000000?ooo`3o
ool0H03oool006<0oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`0d0?ooo`030000
003oool0oooo0640oooo001S0?ooo`030000003oool0oooo03@0oooo0P00000e0?ooo`030000003o
ool0oooo0640oooo001S0?ooo`030000003oool0oooo03@0oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`1R0?ooo`00I03oool00`000000oooo0?ooo`0c0?ooo`030000003oool0oooo
03<0oooo00<000000?ooo`3oool0HP3oool006@0oooo00<000000?ooo`3oool0<`3oool00`000000
oooo0?ooo`0c0?ooo`030000003oool0oooo0680oooo001T0?ooo`030000003oool0oooo03<0oooo
00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`1S0?ooo`00I@3oool00`000000oooo0?oo
o`0b0?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0H`3oool006D0oooo00<00000
0?ooo`3oool0<P3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0oooo06<0oooo001V0?oo
o`030000003oool0oooo0340oooo0P00000b0?ooo`030000003oool0oooo06@0oooo001V0?ooo`03
0000003oool0oooo0340oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`1T0?ooo`00
IP3oool00`000000oooo0?ooo`0a0?ooo`030000003oool0oooo0340oooo00<000000?ooo`3oool0
I03oool006L0oooo00<000000?ooo`3oool0<03oool00`000000oooo0?ooo`0`0?ooo`030000003o
ool0oooo06D0oooo001W0?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool0<03oool0
0`000000oooo0?ooo`1U0?ooo`00I`3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo
0300oooo00<000000?ooo`3oool0I@3oool006P0oooo00<000000?ooo`3oool07@3oool3000000<0
oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02l0
oooo00<000000?ooo`3oool0IP3oool006P0oooo00<000000?ooo`3oool07@3oool010000000oooo
0?ooo`0000070?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0;`3oool00`000000
oooo0?ooo`1V0?ooo`00J@3oool00`000000oooo0?ooo`0L0?ooo`040000003oool0oooo000000@0
oooo1@0000050?ooo`<00000;`3oool00`000000oooo0?ooo`1V0?ooo`00J@3oool00`000000oooo
0?ooo`0L0?ooo`040000003oool0oooo000000@0oooo00@000000?ooo`3oool000001P3oool00`00
0000oooo0?ooo`0^0?ooo`030000003oool0oooo06L0oooo001Y0?ooo`030000003oool0oooo01`0
oooo00@000000?ooo`3oool000001@3oool00`000000oooo000000060?ooo`030000003oool0oooo
02h0oooo00<000000?ooo`3oool0I`3oool006X0oooo00<000000?ooo`3oool06`3oool010000000
oooo0?ooo`0000060?ooo`8000001P3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo
06L0oooo001Z0?ooo`030000003oool0oooo01`0oooo0P0000080?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`1X0?ooo`00J`3oool00`000000oooo
0?ooo`0/0?ooo`030000003oool0oooo02d0oooo00<000000?ooo`3oool0J03oool006/0oooo00<0
00000?ooo`3oool0;03oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo06T0oooo001[
0?ooo`030000003oool0oooo02`0oooo0P00000]0?ooo`030000003oool0oooo06T0oooo001/0?oo
o`030000003oool0oooo02/0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`1Y0?oo
o`00K03oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3o
ool0JP3oool006`0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`0[0?ooo`030000
003oool0oooo06X0oooo001]0?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3oool0:P3o
ool00`000000oooo0?ooo`1[0?ooo`00K@3oool00`000000oooo0?ooo`0Z0?ooo`030000003oool0
oooo02X0oooo00<000000?ooo`3oool0J`3oool006h0oooo00<000000?ooo`3oool0:@3oool00`00
0000oooo0?ooo`0Z0?ooo`030000003oool0oooo06/0oooo001^0?ooo`030000003oool0oooo02T0
oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`1/0?ooo`00KP3oool00`000000oooo
0?ooo`0Y0?ooo`800000:P3oool00`000000oooo0?ooo`1/0?ooo`00K`3oool00`000000oooo0?oo
o`0X0?ooo`030000003oool0oooo02P0oooo00<000000?ooo`3oool0K@3oool006l0oooo00<00000
0?ooo`3oool0:03oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo06d0oooo001_0?oo
o`030000003oool0oooo02P0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`1]0?oo
o`00L03oool00`000000oooo0?ooo`0W0?ooo`030000003oool0oooo02L0oooo00<000000?ooo`3o
ool0KP3oool00700oooo00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0W0?ooo`030000
003oool0oooo06h0oooo001a0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool09`3o
ool00`000000oooo0?ooo`1^0?ooo`00L@3oool00`000000oooo0?ooo`0V0?ooo`8000009`3oool0
0`000000oooo0?ooo`1_0?ooo`00L@3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo
02H0oooo00<000000?ooo`3oool0K`3oool00780oooo00<000000?ooo`3oool09@3oool00`000000
oooo0?ooo`0U0?ooo`030000003oool0oooo0700oooo001b0?ooo`030000003oool0oooo02D0oooo
00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`1`0?ooo`00L`3oool00`000000oooo0?oo
o`0T0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0L03oool007<0oooo00<00000
0?ooo`3oool04P3oool3000000<0oooo00<000000?ooo`3oool00P0000070?ooo`030000003oool0
oooo02@0oooo00<000000?ooo`3oool0L@3oool007<0oooo00<000000?ooo`3oool04P3oool01000
0000oooo0?ooo`0000040?ooo`040000003oool0oooo000000H0oooo00<000000?ooo`3oool0903o
ool00`000000oooo0?ooo`1a0?ooo`00M03oool00`000000oooo0?ooo`0A0?ooo`040000003oool0
oooo000000@0oooo00@000000?ooo`3oool000001P3oool3000002<0oooo00<000000?ooo`3oool0
LP3oool007@0oooo00<000000?ooo`3oool04@3oool010000000oooo0?ooo`0000040?ooo`040000
003oool0oooo000000H0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`1b0?ooo`00
M03oool00`000000oooo0?ooo`0A0?ooo`040000003oool0oooo000000@0oooo0`0000070?ooo`03
0000003oool0oooo02<0oooo00<000000?ooo`3oool0LP3oool007D0oooo00<000000?ooo`3oool0
403oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000H0oooo00<000000?oo
o`3oool08P3oool00`000000oooo0?ooo`1c0?ooo`00M@3oool00`000000oooo0?ooo`0A0?ooo`80
00001P3oool2000000L0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`1c0?ooo`00
MP3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0
M03oool007H0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0Q0?ooo`030000003o
ool0oooo07@0oooo001f0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool08@3oool0
0`000000oooo0?ooo`1d0?ooo`00M`3oool00`000000oooo0?ooo`0P0?ooo`8000008@3oool00`00
0000oooo0?ooo`1e0?ooo`00M`3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0200
oooo00<000000?ooo`3oool0M@3oool007L0oooo00<000000?ooo`3oool0803oool00`000000oooo
0?ooo`0O0?ooo`030000003oool0oooo07H0oooo001h0?ooo`030000003oool0oooo01l0oooo00<0
00000?ooo`3oool07`3oool00`000000oooo0?ooo`1f0?ooo`00N03oool00`000000oooo0?ooo`0O
0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0MP3oool007T0oooo00<000000?oo
o`3oool07P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo07L0oooo001i0?ooo`03
0000003oool0oooo01h0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`1g0?ooo`00
N@3oool00`000000oooo0?ooo`0N0?ooo`8000007P3oool00`000000oooo0?ooo`1h0?ooo`00NP3o
ool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0N03o
ool007X0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0
oooo07T0oooo001j0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool0703oool00`00
0000oooo0?ooo`1i0?ooo`00N`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo01`0
oooo00<000000?ooo`3oool0N@3oool007/0oooo00<000000?ooo`3oool0703oool00`000000oooo
0?ooo`0K0?ooo`030000003oool0oooo07X0oooo001l0?ooo`030000003oool0oooo01/0oooo00<0
00000?ooo`3oool06`3oool00`000000oooo0?ooo`1j0?ooo`00O03oool00`000000oooo0?ooo`0K
0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool0N`3oool007`0oooo00<000000?oo
o`3oool06`3oool2000001/0oooo00<000000?ooo`3oool0N`3oool007d0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`0J0?ooo`030000003oool0oooo07/0oooo001m0?ooo`030000
003oool0oooo01X0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`1l0?ooo`00OP3o
ool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0O03o
ool007h0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0
oooo07d0oooo001o0?ooo`030000003oool0oooo00H0oooo0`0000030?ooo`030000003oool0oooo
008000001`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo07d0oooo001o0?ooo`03
0000003oool0oooo00H0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000006
0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0OP3oool00800oooo00<000000?oo
o`3oool01@3oool010000000oooo0?ooo`0000040?ooo`040000003oool0oooo000000H0oooo0`00
000G0?ooo`030000003oool0oooo07h0oooo00200?ooo`030000003oool0oooo00D0oooo00@00000
0?ooo`3oool000001@3oool2000000L0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`1n0?ooo`00P@3oool00`000000oooo0?ooo`040?ooo`040000003oool0oooo000000@0oooo00@0
00000?ooo`3oool000001P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo07l0oooo
00210?ooo`030000003oool0oooo00@0oooo00@000000?ooo`3oool00000103oool010000000oooo
0?ooo`0000060?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0O`3oool00880oooo
00<000000?ooo`3oool0103oool2000000H0oooo0P0000070?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool0P03oool008<0oooo00<000000?ooo`3oool0503oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo0800oooo00230?ooo`030000003oool0oooo01@0oooo00<00000
0?ooo`3oool0503oool00`000000oooo0?ooo`210?ooo`00Q03oool00`000000oooo0?ooo`0C0?oo
o`8000005@3oool00`000000oooo0?ooo`210?ooo`00Q03oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo01<0oooo00<000000?ooo`3oool0PP3oool008D0oooo00<000000?ooo`3oool0
4P3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0oooo08<0oooo00250?ooo`030000003o
ool0oooo0180oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`230?ooo`00QP3oool0
0`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0Q03oool0
08H0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
08@0oooo00270?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0403oool00`000000
oooo0?ooo`250?ooo`00Q`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00l0oooo
00<000000?ooo`3oool0QP3oool008P0oooo00<000000?ooo`3oool03`3oool200000100oooo00<0
00000?ooo`3oool0QP3oool008T0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo08L0oooo00290?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool03@3oool00`000000oooo0?ooo`280?ooo`00RP3oool00`000000oooo0?ooo`0=0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0R@3oool008/0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo08T0oooo002<0?ooo`030000003o
ool0oooo00/0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`2:0?ooo`00S03oool0
0`000000oooo0?ooo`0;0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0R`3oool0
08d0oooo00<000000?ooo`3oool02P3oool2000000X0oooo00<000000?ooo`3oool0S03oool008h0
oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo08`0
oooo002?0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000oooo
0?ooo`2=0?ooo`00T03oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0oooo00<0
00000?ooo`3oool0SP3oool00940oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo08l0oooo002A0?ooo`8000001`3oool00`000000oooo0?ooo`040?oo
o`800000TP3oool00940oooo00@000000?ooo`00000000001@3oool00`000000oooo0?ooo`030?oo
o`030000003oool0oooo0980oooo002A0?ooo`040000003oool0oooo0?ooo`/00000U@3oool00940
oooo00<000000?ooo`3oool01@3oool4000009P0oooo002A0?ooo`030000003oool0oooo00H0oooo
00<000000?ooo`3oool0V03oool00900oooo0P0000080?ooo`030000003oool0oooo09P0oooo002A
0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0V03oool009X0oooo00<000000?oo
o`3oool0V03oool00?l0oooo=P3oool00001\
\>"],
  ImageRangeCache->{{{0, 308.938}, {190.438, 0}} -> {-6.66493, -0.247669, \
0.0431474, 0.00676247}},
  CellTags->"IPM3.1.1.3"]
}, Open  ]],

Cell[TextData[{
  "Here is a surface of constant negative curvature, represented \
parametrically by the three functions ",
  Cell[BoxData[
      \(TraditionalForm\`\[Rho]\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma]\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`\[Tau]\)]],
  ". This surface is often referred to as",
  " Dini\[CloseCurlyQuote]s surface."
}], "TextFirst",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[BoxData[{
    \(\(\[Rho] = Cos[\[Phi]] Sin[\[Theta]];\)\), "\n", 
    \(\(\[Sigma] = Sin[\[Phi]] Sin[\[Theta]];\)\), "\n", 
    \(\(\[Tau] = 
        0.2  \[Phi] + Cos[\[Theta]] + Log[Tan[\[Theta]\/2]];\)\)}], "Input",
  CellLabel->"In[19]:=",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ParametricPlot3D[{\[Rho], \[Sigma], \[Tau]}, {\[Phi], 0, 
          4  \[Pi]}, {\[Theta],  .05, 1}, Axes \[Rule] False, 
        Boxed \[Rule] False, PlotPoints \[Rule] 30, 
        AspectRatio \[Rule] 1.75];\)\)], "Input",
  CellLabel->"In[22]:=",
  CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.75 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
-0.631412 2.28006 -0.0604238 1.81042 [
[ 0 0 0 0 ]
[ 1 1.75 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.75 L
0 1.75 L
closepath
clip
newpath
.5 Mabswid
[ ] 0 setdash
0 0 .133 r
.52336 .26041 m .52548 .28243 L .53541 .37073 L p F P
0 g
s
0 0 .133 r
.53541 .37073 m .53745 .38885 L p .53565 .37246 L F P
0 g
s
0 0 .133 r
.52336 .26041 m .53541 .37073 L .53565 .37246 L F
0 0 .173 r
.54595 .43933 m .54954 .46063 L p .54661 .44252 L F P
0 g
s
0 0 .173 r
.53745 .38885 m .54595 .43933 L p .54293 .41968 L F P
0 g
s
0 0 .173 r
.54293 .41968 m .54595 .43933 L .54661 .44252 L F
.163 .064 .487 r
.53745 .38885 m .52548 .28243 L p .5307 .35087 L F P
0 g
s
0 0 .133 r
.53639 .37923 m .5339 .36376 L .52336 .26041 L p F P
0 g
s
0 0 .173 r
.5339 .36376 m .53745 .38885 L p .54293 .41968 L F P
0 g
s
0 0 .228 r
.54954 .46063 m .56169 .5149 L p .55689 .49186 L F P
0 g
s
.167 .082 .508 r
.54954 .46063 m .53745 .38885 L p .54063 .42278 L F P
0 g
s
0 0 .173 r
.54632 .44116 m .54455 .43268 L .53919 .39802 L p F P
0 g
s
0 0 .173 r
.53777 .38778 m .54954 .46063 L .53919 .39802 L F
0 0 .228 r
.54455 .43268 m .54954 .46063 L p .55689 .49186 L F P
0 g
s
.167 .082 .508 r
.54681 .48887 m .54954 .46063 L p .54063 .42278 L F P
0 g
s
0 0 .228 r
.56169 .5149 m .55528 .48419 L .55195 .46821 L p F P
0 g
s
0 0 .228 r
.55195 .46821 m .54455 .43268 L p .5495 .45642 L F P
0 g
s
0 0 .228 r
.5495 .45642 m .56169 .5149 L .55195 .46821 L F
.163 .064 .487 r
.53553 .41418 m .53745 .38885 L p .5307 .35087 L F P
0 g
s
0 0 .173 r
.53919 .39802 m .5339 .36376 L p .53777 .38778 L F P
0 g
s
.163 .064 .487 r
.52548 .28243 m .52435 .30463 L .53553 .41418 L p F P
0 g
s
.47 .288 .55 r
.52849 .43854 m .53553 .41418 L .52435 .30463 L p F P
0 g
s
.47 .288 .55 r
.52435 .30463 m .52016 .32629 L .52849 .43854 L p F P
0 g
s
.622 .403 .564 r
.51761 .46093 m .52849 .43854 L .52016 .32629 L p F P
0 g
s
.622 .403 .564 r
.52016 .32629 m .51367 .34678 L .51761 .46093 L p F P
0 g
s
.73 .491 .565 r
.50487 .48074 m .51761 .46093 L .51367 .34678 L p F P
0 g
s
.73 .491 .565 r
.51367 .34678 m .50608 .36573 L .50487 .48074 L p F P
0 g
s
.832 .579 .557 r
.49262 .49784 m .50487 .48074 L .50608 .36573 L p F P
0 g
s
.832 .579 .557 r
.50608 .36573 m .49877 .38306 L .49262 .49784 L p F P
0 g
s
.946 .691 .519 r
.48309 .51262 m .49262 .49784 L .49877 .38306 L p F P
0 g
s
.946 .691 .519 r
.49877 .38306 m .4931 .399 L .48309 .51262 L p F P
0 g
s
.949 .764 .304 r
.47807 .52591 m .48309 .51262 L .4931 .399 L p F P
0 g
s
.949 .764 .304 r
.4931 .399 m .49012 .41407 L .47807 .52591 L p F P
0 g
s
.167 .082 .508 r
.53745 .38885 m .53553 .41418 L .54681 .48887 L p F P
0 g
s
.47 .297 .564 r
.53688 .51573 m .54681 .48887 L .53553 .41418 L p F P
0 g
s
.47 .297 .564 r
.53553 .41418 m .52849 .43854 L .53688 .51573 L p F P
0 g
s
.619 .409 .575 r
.52157 .53982 m .53688 .51573 L .52849 .43854 L p F P
0 g
s
.619 .409 .575 r
.52849 .43854 m .51761 .46093 L .52157 .53982 L p F P
0 g
s
.977 .794 .378 r
.473 .59048 m .48309 .51262 L p .47318 .56406 L F P
0 g
s
.977 .794 .378 r
.46591 .60183 m .46925 .59648 L p .47318 .56406 L F P
0 g
s
.977 .794 .378 r
.46925 .59648 m .473 .59048 L p .47318 .56406 L F P
0 g
s
.977 .794 .378 r
.47807 .52591 m .46591 .60183 L p .47318 .56406 L F P
0 g
s
.977 .794 .378 r
.48309 .51262 m .47807 .52591 L p .47318 .56406 L F P
0 g
s
0 0 .262 r
.47807 .52591 m .48012 .50687 L p .47988 .51182 L F P
0 g
s
0 0 .262 r
.47853 .53882 m .47807 .52591 L p .47988 .51182 L F P
0 g
s
0 0 .262 r
.48012 .50687 m .49012 .41407 L p .486 .45844 L F P
0 g
s
0 0 .262 r
.486 .45844 m .47853 .53882 L .48012 .50687 L F
.49012 .41407 m .49041 .42895 L .47853 .53882 L p F P
0 g
s
.726 .495 .577 r
.50366 .56031 m .52157 .53982 L .51761 .46093 L p F P
0 g
s
.726 .495 .577 r
.51761 .46093 m .50487 .48074 L .50366 .56031 L p F P
0 g
s
.937 .692 .543 r
.473 .59048 m .48641 .57701 L .49262 .49784 L p F P
0 g
s
.937 .692 .543 r
.49262 .49784 m .48309 .51262 L .473 .59048 L p F P
0 g
s
.355 .152 .461 r
.48444 .55257 m .47853 .53882 L .49041 .42895 L p F P
0 g
s
.355 .152 .461 r
.49041 .42895 m .49393 .44437 L .48444 .55257 L p F P
0 g
s
.825 .582 .571 r
.48641 .57701 m .50366 .56031 L .50487 .48074 L p F P
0 g
s
.825 .582 .571 r
.50487 .48074 m .49262 .49784 L .48641 .57701 L p F P
0 g
s
.567 .323 .506 r
.49474 .56825 m .48444 .55257 L .49393 .44437 L p F P
0 g
s
.567 .323 .506 r
.49393 .44437 m .50006 .46102 L .49474 .56825 L p F P
0 g
s
.183 .109 .532 r
.55815 .54593 m .56169 .5149 L .54954 .46063 L p F P
0 g
s
.183 .109 .532 r
.54954 .46063 m .54681 .48887 L .55815 .54593 L p F P
0 g
s
.474 .31 .578 r
.54531 .57516 m .55815 .54593 L .54681 .48887 L p F P
0 g
s
.474 .31 .578 r
.54681 .48887 m .53688 .51573 L .54531 .57516 L p F P
0 g
s
.62 .418 .586 r
.52555 .60084 m .54531 .57516 L .53688 .51573 L p F P
0 g
s
.62 .418 .586 r
.53688 .51573 m .52157 .53982 L .52555 .60084 L p F P
0 g
s
.724 .501 .587 r
.52157 .53982 m .50366 .56031 L .50245 .6219 L p F P
0 g
s
.822 .587 .583 r
.4802 .63813 m .50245 .6219 L .50366 .56031 L p F P
0 g
s
.822 .587 .583 r
.50366 .56031 m .48641 .57701 L .4802 .63813 L p F P
0 g
s
.696 .429 .514 r
.50754 .5866 m .49474 .56825 L .50006 .46102 L p F P
0 g
s
.696 .429 .514 r
.50006 .46102 m .50767 .47932 L .50754 .5866 L p F P
0 g
s
.724 .501 .587 r
.50245 .6219 m .52555 .60084 L .52157 .53982 L p F P
0 g
s
.724 .508 .597 r
.52555 .60084 m .50245 .6219 L .50124 .67248 L p F P
0 g
s
.724 .508 .597 r
.50124 .67248 m .52952 .6509 L .52555 .60084 L p F P
0 g
s
.724 .515 .606 r
.50003 .71554 m .53349 .69349 L .52952 .6509 L p F P
0 g
s
.724 .515 .606 r
.52952 .6509 m .50124 .67248 L .50003 .71554 L p F P
0 g
s
.724 .521 .615 r
.49883 .75309 m .53743 .7306 L .53349 .69349 L p F P
0 g
s
.724 .521 .615 r
.53349 .69349 m .50003 .71554 L .49883 .75309 L p F P
0 g
s
.555 .351 .559 r
.52315 .59122 m .51747 .61376 L .524 .73568 L p F P
0 g
s
.808 .52 .501 r
.52047 .60789 m .50754 .5866 L .50767 .47932 L p F P
0 g
s
.808 .52 .501 r
.50767 .47932 m .51535 .49946 L .52047 .60789 L p F P
0 g
s
0 0 .287 r
.57388 .55846 m .56605 .52507 L .55528 .48419 L p F P
0 g
s
0 0 .287 r
.55528 .48419 m .56169 .5149 L .57388 .55846 L p F P
0 g
s
.989 .815 .43 r
.46591 .60183 m .45369 .65956 L p .45912 .64015 L F P
0 g
s
.989 .815 .43 r
.473 .59048 m .46591 .60183 L p .45912 .64015 L F P
0 g
s
0 0 .183 r
.46591 .60183 m .47807 .52591 L p .47417 .55524 L F P
0 g
s
0 0 .183 r
.46654 .61262 m .46591 .60183 L p .47417 .55524 L F P
0 g
s
0 0 .183 r
.47807 .52591 m .47853 .53882 L .46654 .61262 L p F P
0 g
s
.203 .136 .554 r
.56949 .5922 m .57388 .55846 L .56169 .5149 L p F P
0 g
s
.203 .136 .554 r
.56169 .5149 m .55815 .54593 L .56949 .5922 L p F P
0 g
s
.48 .324 .591 r
.55374 .62372 m .56949 .5922 L .55815 .54593 L p F P
0 g
s
.48 .324 .591 r
.55815 .54593 m .54531 .57516 L .55374 .62372 L p F P
0 g
s
.622 .427 .596 r
.54531 .57516 m .52555 .60084 L .52952 .6509 L p F P
0 g
s
.622 .427 .596 r
.52952 .6509 m .55374 .62372 L .54531 .57516 L p F P
0 g
s
.624 .435 .606 r
.53349 .69349 m .56215 .66485 L .55374 .62372 L p F P
0 g
s
.624 .435 .606 r
.55374 .62372 m .52952 .6509 L .53349 .69349 L p F P
0 g
s
.627 .444 .616 r
.56215 .66485 m .53349 .69349 L .53743 .7306 L p F P
0 g
s
.347 .196 .529 r
.53354 .71078 m .53831 .68443 L .52598 .56783 L p F P
0 g
s
.347 .196 .529 r
.52598 .56783 m .52315 .59122 L .53354 .71078 L p F P
0 g
s
.555 .351 .559 r
.524 .73568 m .53354 .71078 L .52315 .59122 L p F P
0 g
s
.926 .62 .444 r
.5311 .63184 m .52047 .60789 L .51535 .49946 L p F P
0 g
s
.926 .62 .444 r
.51535 .49946 m .52167 .52125 L .5311 .63184 L p F P
0 g
s
.931 .697 .562 r
.46287 .65021 m .4802 .63813 L .48641 .57701 L p F P
0 g
s
.931 .697 .562 r
.48641 .57701 m .473 .59048 L .46287 .65021 L p F P
0 g
s
.627 .444 .616 r
.53743 .7306 m .57052 .70057 L .56215 .66485 L p F P
0 g
s
.629 .453 .625 r
.57052 .70057 m .53743 .7306 L .54135 .76351 L p F P
0 g
s
.629 .453 .625 r
.54135 .76351 m .57883 .73211 L .57052 .70057 L p F P
0 g
s
.632 .461 .633 r
.57883 .73211 m .54135 .76351 L .54523 .79304 L p F P
0 g
s
.632 .461 .633 r
.54523 .79304 m .58708 .76033 L .57883 .73211 L p F P
0 g
s
.635 .469 .642 r
.58708 .76033 m .54523 .79304 L .54908 .8198 L p F P
0 g
s
.635 .469 .642 r
.54908 .8198 m .59525 .7858 L .58708 .76033 L p F P
0 g
s
.637 .477 .65 r
.59525 .7858 m .54908 .8198 L .55288 .84422 L p F P
0 g
s
.637 .477 .65 r
.55288 .84422 m .60333 .80898 L .59525 .7858 L p F P
0 g
s
.64 .485 .658 r
.60333 .80898 m .55288 .84422 L .55662 .86664 L p F P
0 g
s
.64 .485 .658 r
.55662 .86664 m .61131 .83018 L .60333 .80898 L p F P
0 g
s
.642 .493 .665 r
.61131 .83018 m .55662 .86664 L .56032 .88731 L p F P
0 g
s
.642 .493 .665 r
.56032 .88731 m .61917 .84966 L .61131 .83018 L p F P
0 g
s
0 0 .469 r
.57401 .8372 m .56177 .79133 L p .56409 .8047 L F P
0 g
s
0 0 .469 r
.57584 .8723 m .57401 .8372 L p .56409 .8047 L F P
0 g
s
.644 .501 .673 r
.61917 .84966 m .56032 .88731 L .56395 .90642 L p F P
0 g
s
.647 .508 .68 r
.62691 .86762 m .56395 .90642 L .56752 .92415 L p F P
0 g
s
.644 .501 .673 r
.56395 .90642 m .62691 .86762 L .61917 .84966 L p F P
0 g
s
.649 .516 .687 r
.63452 .88423 m .56752 .92415 L .57102 .94062 L p F P
0 g
s
.647 .508 .68 r
.56752 .92415 m .63452 .88423 L .62691 .86762 L p F P
0 g
s
0 0 .502 r
.58626 .87524 m .57401 .8372 L p .57578 .84653 L F P
0 g
s
0 0 .502 r
.58838 .91296 m .58626 .87524 L p .57578 .84653 L F P
0 g
s
.651 .523 .694 r
.64199 .89961 m .57102 .94062 L .57445 .95595 L p F P
0 g
s
.649 .516 .687 r
.57102 .94062 m .64199 .89961 L .63452 .88423 L p F P
0 g
s
.654 .53 .701 r
.6493 .91388 m .57445 .95595 L .5778 .97022 L p F P
0 g
s
.651 .523 .694 r
.57445 .95595 m .6493 .91388 L .64199 .89961 L p F P
0 g
s
0 .017 .533 r
.5985 .90756 m .58626 .87524 L p .58766 .88214 L F P
0 g
s
0 .017 .533 r
.60089 .94784 m .5985 .90756 L p .58766 .88214 L F P
0 g
s
.656 .537 .708 r
.65646 .92713 m .5778 .97022 L .58107 .98353 L p F P
0 g
s
.654 .53 .701 r
.5778 .97022 m .65646 .92713 L .6493 .91388 L p F P
0 g
s
.658 .545 .714 r
.66344 .93943 m .58107 .98353 L .58425 .99593 L p F P
0 g
s
.656 .537 .708 r
.58107 .98353 m .66344 .93943 L .65646 .92713 L p F P
0 g
s
.658 .545 .714 r
.58425 .99593 m .67025 .95087 L .66344 .93943 L p F P
0 g
s
.006 .055 .563 r
.61069 .93551 m .5985 .90756 L p .59962 .91291 L F P
0 g
s
.006 .055 .563 r
.61333 .9783 m .61069 .93551 L p .59962 .91291 L F P
0 g
s
.665 .566 .734 r
.68331 .97136 m .59036 1.01825 L .59328 1.02828 L p F P
0 g
s
.663 .559 .727 r
.59036 1.01825 m .68331 .97136 L .67688 .9615 L p F P
0 g
s
.663 .559 .727 r
.67688 .9615 m .58735 1.00749 L .59036 1.01825 L p F P
0 g
s
.66 .552 .721 r
.58735 1.00749 m .67688 .9615 L .67025 .95087 L p F P
0 g
s
.66 .552 .721 r
.67025 .95087 m .58425 .99593 L .58735 1.00749 L p F P
0 g
s
.04 .092 .59 r
.62282 .95999 m .61069 .93551 L p .61161 .9398 L F P
0 g
s
.04 .092 .59 r
.6257 1.00523 m .62282 .95999 L p .61161 .9398 L F P
0 g
s
.672 .588 .753 r
.70141 .99687 m .59881 1.04626 L .60143 1.0543 L p F P
0 g
s
.669 .581 .747 r
.59881 1.04626 m .70141 .99687 L .69559 .98901 L p F P
0 g
s
.669 .581 .747 r
.69559 .98901 m .5961 1.0376 L .59881 1.04626 L p F P
0 g
s
.667 .573 .74 r
.5961 1.0376 m .69559 .98901 L .68955 .98052 L p F P
0 g
s
.667 .573 .74 r
.68955 .98052 m .59328 1.02828 L .5961 1.0376 L p F P
0 g
s
.665 .566 .734 r
.59328 1.02828 m .68955 .98052 L .68331 .97136 L p F P
0 g
s
.071 .126 .614 r
.63486 .98162 m .62282 .95999 L p .62359 .96353 L F P
0 g
s
.071 .126 .614 r
.63797 1.02928 m .63486 .98162 L p .62359 .96353 L F P
0 g
s
0 0 .337 r
.63486 .98162 m .64681 1.00089 L p .64503 .99794 L F P
0 g
s
.662 .304 0 r
.62282 .95999 m .63486 .98162 L p .63287 .97809 L F P
0 g
s
.006 .055 .563 r
.5985 .90756 m .60089 .94784 L .61333 .9783 L p F P
0 g
s
.04 .092 .59 r
.61069 .93551 m .61333 .9783 L .6257 1.00523 L p F P
0 g
s
.071 .126 .614 r
.62282 .95999 m .6257 1.00523 L .63797 1.02928 L p F P
0 g
s
.662 .304 0 r
.59852 .91728 m .62282 .95999 L p .63287 .97809 L F P
0 g
s
.662 .304 0 r
.63486 .98162 m .60807 .93672 L .59852 .91728 L p F P
0 g
s
0 0 .337 r
.60807 .93672 m .63486 .98162 L p .64503 .99794 L F P
0 g
s
.715 .362 0 r
.61069 .93551 m .62282 .95999 L p .62054 .95563 L F P
0 g
s
0 .017 .533 r
.58626 .87524 m .58838 .91296 L .60089 .94784 L p F P
0 g
s
.715 .362 0 r
.58891 .89505 m .61069 .93551 L p .62054 .95563 L F P
0 g
s
.763 .416 0 r
.5985 .90756 m .61069 .93551 L p .60806 .92998 L F P
0 g
s
0 0 .502 r
.57401 .8372 m .57584 .8723 L .58838 .91296 L p F P
0 g
s
.763 .416 0 r
.57926 .86938 m .5985 .90756 L p .60806 .92998 L F P
0 g
s
.806 .467 0 r
.58626 .87524 m .5985 .90756 L p .5954 .90026 L F P
0 g
s
0 0 .469 r
.56177 .79133 m .56329 .82374 L .57584 .8723 L p F P
0 g
s
.806 .467 0 r
.56959 .83938 m .58626 .87524 L p .5954 .90026 L F P
0 g
s
.844 .515 0 r
.57401 .8372 m .58626 .87524 L p .58253 .8651 L F P
0 g
s
0 0 .434 r
.56329 .82374 m .56177 .79133 L p .55707 .77226 L F P
0 g
s
0 0 .434 r
.55069 .74129 m .56329 .82374 L .55622 .76522 L F
.54957 .73394 m .55077 .76357 L .56329 .82374 L p F P
0 g
s
.844 .515 0 r
.55992 .80373 m .57401 .8372 L p .58253 .8651 L F P
0 g
s
.876 .558 .046 r
.56177 .79133 m .57401 .8372 L p .56939 .82222 L F P
0 g
s
0 0 .399 r
.55077 .76357 m .54957 .73394 L p .5466 .71822 L F P
0 g
s
0 0 .399 r
.53978 .67628 m .55077 .76357 L .54597 .71131 L F
.53744 .65771 m .53831 .68443 L .55077 .76357 L p F P
0 g
s
.876 .558 .046 r
.55026 .7603 m .56177 .79133 L p .56939 .82222 L F P
0 g
s
0 0 .434 r
.56177 .79133 m .55622 .76522 L p .55707 .77226 L F P
0 g
s
0 0 .434 r
.55622 .76522 m .54957 .73394 L p .55069 .74129 L F P
0 g
s
.876 .558 .046 r
.55992 .80373 m .55026 .7603 L p .56939 .82222 L F P
0 g
s
.876 .558 .046 r
.57401 .8372 m .57143 .83106 L p .56939 .82222 L F P
0 g
s
.876 .558 .046 r
.57143 .83106 m .55992 .80373 L p .56939 .82222 L F P
0 g
s
.902 .597 .101 r
.54957 .73394 m .56177 .79133 L p .55585 .76725 L F P
0 g
s
.725 .534 .632 r
.49646 .81632 m .54523 .79304 L .54135 .76351 L p F P
0 g
s
.347 .207 .545 r
.54403 .79265 m .55077 .76357 L .53831 .68443 L p F P
0 g
s
.902 .597 .101 r
.54064 .70543 m .54957 .73394 L p .55585 .76725 L F P
0 g
s
0 0 .399 r
.54957 .73394 m .54597 .71131 L p .5466 .71822 L F P
0 g
s
0 0 .399 r
.54597 .71131 m .53744 .65771 L p .53978 .67628 L F P
0 g
s
.902 .597 .101 r
.55026 .7603 m .54064 .70543 L p .55585 .76725 L F P
0 g
s
.902 .597 .101 r
.56177 .79133 m .55858 .78274 L p .55585 .76725 L F P
0 g
s
.902 .597 .101 r
.55858 .78274 m .55026 .7603 L p .55585 .76725 L F P
0 g
s
.921 .632 .15 r
.53744 .65771 m .54957 .73394 L p .54175 .69076 L F P
0 g
s
0 0 .372 r
.53831 .68443 m .53744 .65771 L p .53585 .64554 L F P
0 g
s
0 0 .372 r
.53003 .59418 m .53831 .68443 L .5354 .63839 L F
.52545 .54426 m .52598 .56783 L .53831 .68443 L p F P
0 g
s
.921 .632 .15 r
.5311 .63184 m .53744 .65771 L p .54175 .69076 L F P
0 g
s
0 0 .372 r
.53744 .65771 m .5354 .63839 L p .53585 .64554 L F P
0 g
s
0 0 .372 r
.5354 .63839 m .52545 .54426 L p .53003 .59418 L F P
0 g
s
.725 .528 .624 r
.49764 .7864 m .54135 .76351 L .53743 .7306 L p F P
0 g
s
.725 .528 .624 r
.53743 .7306 m .49883 .75309 L .49764 .7864 L p F P
0 g
s
.725 .534 .632 r
.54135 .76351 m .49764 .7864 L .49646 .81632 L p F P
0 g
s
.347 .207 .545 r
.53831 .68443 m .53354 .71078 L .54403 .79265 L p F P
0 g
s
.921 .632 .15 r
.54064 .70543 m .5311 .63184 L p .54175 .69076 L F P
0 g
s
.921 .632 .15 r
.54957 .73394 m .54537 .72053 L p .54175 .69076 L F P
0 g
s
.921 .632 .15 r
.54537 .72053 m .54064 .70543 L p .54175 .69076 L F P
0 g
s
.931 .661 .182 r
.53744 .65771 m .5311 .63184 L .52167 .52125 L p F P
0 g
s
.931 .661 .182 r
.52167 .52125 m .52545 .54426 L .53744 .65771 L p F P
0 g
s
.989 .815 .43 r
.45369 .65956 m .46287 .65021 L .473 .59048 L p F P
0 g
s
.321 .111 .431 r
.47484 .62453 m .46654 .61262 L .47853 .53882 L p F P
0 g
s
.321 .111 .431 r
.47853 .53882 m .48444 .55257 L .47484 .62453 L p F P
0 g
s
.819 .592 .594 r
.47399 .68819 m .50124 .67248 L .50245 .6219 L p F P
0 g
s
.819 .592 .594 r
.50245 .6219 m .4802 .63813 L .47399 .68819 L p F P
0 g
s
.487 .337 .603 r
.56949 .5922 m .55374 .62372 L .56215 .66485 L p F P
0 g
s
.487 .337 .603 r
.56215 .66485 m .58082 .63112 L .56949 .5922 L p F P
0 g
s
.224 .164 .576 r
.57388 .55846 m .56949 .5922 L .58082 .63112 L p F P
0 g
s
.224 .164 .576 r
.58082 .63112 m .58606 .59473 L .57388 .55846 L p F P
0 g
s
0 0 .345 r
.56605 .52507 m .57388 .55846 L .58606 .59473 L p F P
0 g
s
0 0 .345 r
.58606 .59473 m .57682 .55873 L .56605 .52507 L p F P
0 g
s
.926 .702 .579 r
.45274 .69884 m .47399 .68819 L .4802 .63813 L p F P
0 g
s
.926 .702 .579 r
.4802 .63813 m .46287 .65021 L .45274 .69884 L p F P
0 g
s
.552 .3 .488 r
.48936 .63907 m .47484 .62453 L .48444 .55257 L p F P
0 g
s
.552 .3 .488 r
.48444 .55257 m .49474 .56825 L .48936 .63907 L p F P
0 g
s
.817 .598 .605 r
.46779 .7307 m .50003 .71554 L .50124 .67248 L p F P
0 g
s
.817 .598 .605 r
.50124 .67248 m .47399 .68819 L .46779 .7307 L p F P
0 g
s
.494 .351 .615 r
.58082 .63112 m .56215 .66485 L .57052 .70057 L p F P
0 g
s
.995 .83 .473 r
.45369 .65956 m .44146 .70617 L p .44564 .69523 L F P
0 g
s
.995 .83 .473 r
.46287 .65021 m .45369 .65956 L p .44564 .69523 L F P
0 g
s
0 0 .111 r
.45369 .65956 m .46591 .60183 L p .46328 .6171 L F P
0 g
s
0 0 .111 r
.45447 .66816 m .45369 .65956 L p .46328 .6171 L F P
0 g
s
0 0 .111 r
.46591 .60183 m .46654 .61262 L .45447 .66816 L p F P
0 g
s
.678 .448 .565 r
.51144 .75832 m .524 .73568 L .51747 .61376 L p F P
0 g
s
.678 .448 .565 r
.51747 .61376 m .50998 .63496 L .51144 .75832 L p F P
0 g
s
.494 .351 .615 r
.57052 .70057 m .59211 .66467 L .58082 .63112 L p F P
0 g
s
.244 .19 .595 r
.58606 .59473 m .58082 .63112 L .59211 .66467 L p F P
0 g
s
.244 .19 .595 r
.59211 .66467 m .59821 .6257 L .58606 .59473 L p F P
0 g
s
0 0 .4 r
.57682 .55873 m .58606 .59473 L .59821 .6257 L p F P
0 g
s
.781 .533 .562 r
.50998 .63496 m .50205 .65459 L .49813 .77833 L p F P
0 g
s
.688 .412 .5 r
.50742 .65728 m .48936 .63907 L .49474 .56825 L p F P
0 g
s
.688 .412 .5 r
.49474 .56825 m .50754 .5866 L .50742 .65728 L p F P
0 g
s
.995 .83 .473 r
.44146 .70617 m .45274 .69884 L .46287 .65021 L p F P
0 g
s
0 0 .4 r
.59821 .6257 m .58758 .58712 L .57682 .55873 L p F P
0 g
s
.781 .533 .562 r
.49813 .77833 m .51144 .75832 L .50998 .63496 L p F P
0 g
s
.816 .604 .615 r
.46163 .76768 m .49883 .75309 L .50003 .71554 L p F P
0 g
s
.816 .604 .615 r
.50003 .71554 m .46779 .7307 L .46163 .76768 L p F P
0 g
s
.921 .707 .595 r
.44263 .73992 m .46779 .7307 L .47399 .68819 L p F P
0 g
s
.921 .707 .595 r
.47399 .68819 m .45274 .69884 L .44263 .73992 L p F P
0 g
s
.501 .363 .626 r
.59211 .66467 m .57052 .70057 L .57883 .73211 L p F P
0 g
s
.501 .363 .626 r
.57883 .73211 m .60333 .6941 L .59211 .66467 L p F P
0 g
s
.264 .215 .614 r
.59821 .6257 m .59211 .66467 L .60333 .6941 L p F P
0 g
s
.293 .075 .402 r
.46518 .67816 m .45447 .66816 L .46654 .61262 L p F P
0 g
s
.293 .075 .402 r
.46654 .61262 m .47484 .62453 L .46518 .67816 L p F P
0 g
s
.801 .504 .486 r
.52566 .67953 m .50742 .65728 L .50754 .5866 L p F P
0 g
s
.801 .504 .486 r
.50754 .5866 m .52047 .60789 L .52566 .67953 L p F P
0 g
s
.814 .609 .625 r
.45551 .80042 m .49764 .7864 L .49883 .75309 L p F P
0 g
s
.814 .609 .625 r
.49883 .75309 m .46163 .76768 L .45551 .80042 L p F P
0 g
s
.813 .615 .634 r
.49764 .7864 m .45551 .80042 L .44944 .82976 L p F P
0 g
s
.888 .631 .543 r
.48653 .79587 m .49813 .77833 L .50205 .65459 L p F P
0 g
s
.888 .631 .543 r
.50205 .65459 m .49514 .67273 L .48653 .79587 L p F P
0 g
s
.507 .376 .636 r
.60333 .6941 m .57883 .73211 L .58708 .76033 L p F P
0 g
s
.264 .215 .614 r
.60333 .6941 m .61031 .65259 L .59821 .6257 L p F P
0 g
s
0 0 .452 r
.58758 .58712 m .59821 .6257 L .61031 .65259 L p F P
0 g
s
.917 .712 .61 r
.43257 .77546 m .46163 .76768 L .46779 .7307 L p F P
0 g
s
.917 .712 .61 r
.46779 .7307 m .44263 .73992 L .43257 .77546 L p F P
0 g
s
.507 .376 .636 r
.58708 .76033 m .61448 .72025 L .60333 .6941 L p F P
0 g
s
.283 .239 .631 r
.61031 .65259 m .60333 .6941 L .61448 .72025 L p F P
0 g
s
0 0 .452 r
.61031 .65259 m .59831 .61149 L .58758 .58712 L p F P
0 g
s
.92 .601 .424 r
.54064 .70543 m .52566 .67953 L .52047 .60789 L p F P
0 g
s
.92 .601 .424 r
.52047 .60789 m .5311 .63184 L .54064 .70543 L p F P
0 g
s
.998 .843 .51 r
.44146 .70617 m .42924 .74521 L p .43258 .73861 L F P
0 g
s
.998 .843 .51 r
.45274 .69884 m .44146 .70617 L p .43258 .73861 L F P
0 g
s
0 0 .041 r
.44146 .70617 m .45369 .65956 L p .45183 .66827 L F P
0 g
s
0 0 .041 r
.44236 .71255 m .44146 .70617 L p .45183 .66827 L F P
0 g
s
0 0 .041 r
.45369 .65956 m .45447 .66816 L .44236 .71255 L p F P
0 g
s
.998 .843 .51 r
.42924 .74521 m .44263 .73992 L .45274 .69884 L p F P
0 g
s
.726 .541 .64 r
.49529 .84344 m .54908 .8198 L .54523 .79304 L p F P
0 g
s
.726 .541 .64 r
.54523 .79304 m .49646 .81632 L .49529 .84344 L p F P
0 g
s
.813 .615 .634 r
.44944 .82976 m .49646 .81632 L .49764 .7864 L p F P
0 g
s
.514 .388 .647 r
.61448 .72025 m .58708 .76033 L .59525 .7858 L p F P
0 g
s
.993 .752 .453 r
.47878 .81152 m .48653 .79587 L .49514 .67273 L p F P
0 g
s
.993 .752 .453 r
.49514 .67273 m .49053 .68976 L .47878 .81152 L p F P
0 g
s
.726 .547 .648 r
.49414 .86822 m .55288 .84422 L .54908 .8198 L p F P
0 g
s
.726 .547 .648 r
.54908 .8198 m .49529 .84344 L .49414 .86822 L p F P
0 g
s
.283 .239 .631 r
.61448 .72025 m .62234 .67626 L .61031 .65259 L p F P
0 g
s
0 0 .5 r
.59831 .61149 m .61031 .65259 L .62234 .67626 L p F P
0 g
s
.541 .281 .47 r
.48394 .69148 m .46518 .67816 L .47484 .62453 L p F P
0 g
s
.541 .281 .47 r
.47484 .62453 m .48936 .63907 L .48394 .69148 L p F P
0 g
s
.913 .717 .623 r
.42256 .80676 m .45551 .80042 L .46163 .76768 L p F P
0 g
s
.913 .717 .623 r
.46163 .76768 m .43257 .77546 L .42256 .80676 L p F P
0 g
s
.811 .62 .643 r
.44344 .8563 m .49529 .84344 L .49646 .81632 L p F P
0 g
s
.811 .62 .643 r
.49646 .81632 m .44944 .82976 L .44344 .8563 L p F P
0 g
s
.52 .4 .656 r
.62553 .7437 m .59525 .7858 L .60333 .80898 L p F P
0 g
s
.514 .388 .647 r
.59525 .7858 m .62553 .7437 L .61448 .72025 L p F P
0 g
s
.301 .262 .647 r
.62234 .67626 m .61448 .72025 L .62553 .7437 L p F P
0 g
s
.727 .553 .656 r
.493 .89097 m .55662 .86664 L .55288 .84422 L p F P
0 g
s
.727 .553 .656 r
.55288 .84422 m .49414 .86822 L .493 .89097 L p F P
0 g
s
.81 .625 .652 r
.4375 .88049 m .49414 .86822 L .49529 .84344 L p F P
0 g
s
.81 .625 .652 r
.49529 .84344 m .44344 .8563 L .4375 .88049 L p F P
0 g
s
.526 .411 .665 r
.63647 .76491 m .60333 .80898 L .61131 .83018 L p F P
0 g
s
.52 .4 .656 r
.60333 .80898 m .63647 .76491 L .62553 .7437 L p F P
0 g
s
0 0 .5 r
.62234 .67626 m .60899 .63268 L .59831 .61149 L p F P
0 g
s
.727 .559 .663 r
.49188 .91195 m .56032 .88731 L .55662 .86664 L p F P
0 g
s
.727 .559 .663 r
.55662 .86664 m .493 .89097 L .49188 .91195 L p F P
0 g
s
.998 .853 .542 r
.41705 .77872 m .43257 .77546 L .44263 .73992 L p F P
0 g
s
0 0 0 r
.42924 .74521 m .44146 .70617 L p .44009 .71146 L F P
0 g
s
.998 .853 .542 r
.44263 .73992 m .42924 .74521 L p .41981 .77454 L F P
0 g
s
0 0 0 r
.43026 .74936 m .42924 .74521 L p .44009 .71146 L F P
0 g
s
.998 .853 .542 r
.42924 .74521 m .41705 .77872 L p .41981 .77454 L F P
0 g
s
0 0 .001 r
.47634 .82626 m .47878 .81152 L .49053 .68976 L p F P
0 g
s
0 0 .001 r
.49053 .68976 m .48909 .70627 L .47634 .82626 L p F P
0 g
s
.909 .721 .636 r
.41263 .83467 m .44944 .82976 L .45551 .80042 L p F P
0 g
s
.909 .721 .636 r
.45551 .80042 m .42256 .80676 L .41263 .83467 L p F P
0 g
s
.734 .663 .78 r
.47659 1.10914 m .61083 1.08074 L .60865 1.07493 L p F P
0 g
s
.681 .618 .779 r
.61083 1.08074 m .72719 1.02782 L .7225 1.02264 L p F P
0 g
s
.681 .618 .779 r
.7225 1.02264 m .60865 1.07493 L .61083 1.08074 L p F P
0 g
s
.318 .283 .661 r
.63427 .69728 m .62553 .7437 L .63647 .76491 L p F P
0 g
s
.301 .262 .647 r
.62553 .7437 m .63427 .69728 L .62234 .67626 L p F P
0 g
s
.728 .566 .671 r
.49078 .93137 m .56395 .90642 L .56032 .88731 L p F P
0 g
s
.728 .566 .671 r
.56032 .88731 m .49188 .91195 L .49078 .93137 L p F P
0 g
s
.809 .631 .661 r
.43164 .90266 m .493 .89097 L .49414 .86822 L p F P
0 g
s
.809 .631 .661 r
.49414 .86822 m .4375 .88049 L .43164 .90266 L p F P
0 g
s
.734 .663 .78 r
.60865 1.07493 m .47725 1.10318 L .47659 1.10914 L p F P
0 g
s
.733 .656 .773 r
.47725 1.10318 m .60865 1.07493 L .60635 1.0686 L p F P
0 g
s
.678 .61 .772 r
.60865 1.07493 m .7225 1.02264 L .71757 1.01699 L p F P
0 g
s
.678 .61 .772 r
.71757 1.01699 m .60635 1.0686 L .60865 1.07493 L p F P
0 g
s
0 0 .544 r
.60899 .63268 m .62234 .67626 L .63427 .69728 L p F P
0 g
s
.532 .422 .674 r
.64728 .78418 m .61131 .83018 L .61917 .84966 L p F P
0 g
s
.526 .411 .665 r
.61131 .83018 m .64728 .78418 L .63647 .76491 L p F P
0 g
s
.733 .656 .773 r
.60635 1.0686 m .47794 1.0967 L .47725 1.10318 L p F P
0 g
s
.733 .65 .766 r
.47794 1.0967 m .60635 1.0686 L .60394 1.06173 L p F P
0 g
s
.676 .603 .766 r
.60635 1.0686 m .71757 1.01699 L .71241 1.01082 L p F P
0 g
s
.676 .603 .766 r
.71241 1.01082 m .60394 1.06173 L .60635 1.0686 L p F P
0 g
s
.268 .04 .371 r
.45547 .72059 m .44236 .71255 L .45447 .66816 L p F P
0 g
s
.268 .04 .371 r
.45447 .66816 m .46518 .67816 L .45547 .72059 L p F P
0 g
s
.552 .357 .57 r
.53059 .81967 m .54403 .79265 L .53354 .71078 L p F P
0 g
s
.552 .357 .57 r
.53354 .71078 m .524 .73568 L .53059 .81967 L p F P
0 g
s
.781 .721 .796 r
.34679 1.1122 m .47659 1.10914 L .47725 1.10318 L p F P
0 g
s
.728 .572 .678 r
.4897 .94939 m .56752 .92415 L .56395 .90642 L p F P
0 g
s
.728 .572 .678 r
.56395 .90642 m .49078 .93137 L .4897 .94939 L p F P
0 g
s
.733 .65 .766 r
.60394 1.06173 m .47867 1.08967 L .47794 1.0967 L p F P
0 g
s
.733 .643 .759 r
.47867 1.08967 m .60394 1.06173 L .60143 1.0543 L p F P
0 g
s
.674 .595 .759 r
.60394 1.06173 m .71241 1.01082 L .70702 1.00413 L p F P
0 g
s
.674 .595 .759 r
.70702 1.00413 m .60143 1.0543 L .60394 1.06173 L p F P
0 g
s
0 0 0 r
.44146 .70617 m .44236 .71255 L .43026 .74936 L p F P
0 g
s
.682 .397 .484 r
.5073 .70945 m .48394 .69148 L .48936 .63907 L p F P
0 g
s
.682 .397 .484 r
.48936 .63907 m .50742 .65728 L .5073 .70945 L p F P
0 g
s
.808 .636 .669 r
.42586 .92307 m .49188 .91195 L .493 .89097 L p F P
0 g
s
.808 .636 .669 r
.493 .89097 m .43164 .90266 L .42586 .92307 L p F P
0 g
s
.781 .721 .796 r
.47725 1.10318 m .35022 1.10659 L .34679 1.1122 L p F P
0 g
s
.783 .715 .788 r
.35022 1.10659 m .47725 1.10318 L .47794 1.0967 L p F P
0 g
s
.733 .643 .759 r
.60143 1.0543 m .47943 1.08206 L .47867 1.08967 L p F P
0 g
s
.733 .637 .753 r
.47943 1.08206 m .60143 1.0543 L .59881 1.04626 L p F P
0 g
s
.672 .588 .753 r
.60143 1.0543 m .70702 1.00413 L .70141 .99687 L p F P
0 g
s
.729 .577 .685 r
.48864 .96614 m .57102 .94062 L .56752 .92415 L p F P
0 g
s
.729 .577 .685 r
.56752 .92415 m .4897 .94939 L .48864 .96614 L p F P
0 g
s
.733 .637 .753 r
.59881 1.04626 m .48022 1.07385 L .47943 1.08206 L p F P
0 g
s
.732 .631 .746 r
.48022 1.07385 m .59881 1.04626 L .5961 1.0376 L p F P
0 g
s
.783 .715 .788 r
.47794 1.0967 m .35382 1.10049 L .35022 1.10659 L p F P
0 g
s
.785 .71 .781 r
.35382 1.10049 m .47794 1.0967 L .47867 1.08967 L p F P
0 g
s
.538 .433 .683 r
.65795 .80177 m .61917 .84966 L .62691 .86762 L p F P
0 g
s
.532 .422 .674 r
.61917 .84966 m .65795 .80177 L .64728 .78418 L p F P
0 g
s
.732 .631 .746 r
.5961 1.0376 m .48105 1.065 L .48022 1.07385 L p F P
0 g
s
.732 .625 .739 r
.48105 1.065 m .5961 1.0376 L .59328 1.02828 L p F P
0 g
s
.729 .583 .692 r
.4876 .98174 m .57445 .95595 L .57102 .94062 L p F P
0 g
s
.729 .583 .692 r
.57102 .94062 m .48864 .96614 L .4876 .98174 L p F P
0 g
s
.906 .726 .648 r
.40279 .8598 m .44344 .8563 L .44944 .82976 L p F P
0 g
s
.906 .726 .648 r
.44944 .82976 m .41263 .83467 L .40279 .8598 L p F P
0 g
s
.785 .71 .781 r
.47867 1.08967 m .35759 1.09385 L .35382 1.10049 L p F P
0 g
s
.787 .705 .773 r
.35759 1.09385 m .47867 1.08967 L .47943 1.08206 L p F P
0 g
s
.732 .625 .739 r
.59328 1.02828 m .4819 1.05547 L .48105 1.065 L p F P
0 g
s
.732 .619 .733 r
.4819 1.05547 m .59328 1.02828 L .59036 1.01825 L p F P
0 g
s
.806 .641 .677 r
.42018 .94193 m .49078 .93137 L .49188 .91195 L p F P
0 g
s
.806 .641 .677 r
.49188 .91195 m .42586 .92307 L .42018 .94193 L p F P
0 g
s
.335 .304 .675 r
.6461 .71609 m .63647 .76491 L .64728 .78418 L p F P
0 g
s
.318 .283 .661 r
.63647 .76491 m .6461 .71609 L .63427 .69728 L p F P
0 g
s
.73 .589 .699 r
.48658 .99627 m .5778 .97022 L .57445 .95595 L p F P
0 g
s
.73 .589 .699 r
.57445 .95595 m .4876 .98174 L .48658 .99627 L p F P
0 g
s
.732 .619 .733 r
.59036 1.01825 m .48278 1.04524 L .4819 1.05547 L p F P
0 g
s
.731 .613 .726 r
.48278 1.04524 m .59036 1.01825 L .58735 1.00749 L p F P
0 g
s
.73 .595 .706 r
.48559 1.00983 m .58107 .98353 L .5778 .97022 L p F P
0 g
s
.73 .595 .706 r
.5778 .97022 m .48658 .99627 L .48559 1.00983 L p F P
0 g
s
.731 .613 .726 r
.58735 1.00749 m .48369 1.03425 L .48278 1.04524 L p F P
0 g
s
.731 .607 .719 r
.48369 1.03425 m .58735 1.00749 L .58425 .99593 L p F P
0 g
s
.787 .705 .773 r
.47943 1.08206 m .36152 1.08665 L .35759 1.09385 L p F P
0 g
s
.789 .7 .766 r
.36152 1.08665 m .47943 1.08206 L .48022 1.07385 L p F P
0 g
s
.73 .601 .712 r
.48463 1.02247 m .58425 .99593 L .58107 .98353 L p F P
0 g
s
.73 .601 .712 r
.58107 .98353 m .48559 1.00983 L .48463 1.02247 L p F P
0 g
s
.731 .607 .719 r
.58425 .99593 m .48463 1.02247 L .48369 1.03425 L p F P
0 g
s
.675 .452 .576 r
.51291 .84353 m .53059 .81967 L .524 .73568 L p F P
0 g
s
.675 .452 .576 r
.524 .73568 m .51144 .75832 L .51291 .84353 L p F P
0 g
s
.544 .444 .691 r
.66846 .81789 m .62691 .86762 L .63452 .88423 L p F P
0 g
s
.538 .433 .683 r
.62691 .86762 m .66846 .81789 L .65795 .80177 L p F P
0 g
s
.805 .646 .685 r
.4146 .95939 m .4897 .94939 L .49078 .93137 L p F P
0 g
s
.805 .646 .685 r
.49078 .93137 m .42018 .94193 L .4146 .95939 L p F P
0 g
s
.789 .7 .766 r
.48022 1.07385 m .36562 1.07886 L .36152 1.08665 L p F P
0 g
s
.791 .695 .759 r
.36562 1.07886 m .48022 1.07385 L .48105 1.065 L p F P
0 g
s
.791 .695 .759 r
.48105 1.065 m .36988 1.07044 L .36562 1.07886 L p F P
0 g
s
.792 .69 .751 r
.36988 1.07044 m .48105 1.065 L .4819 1.05547 L p F P
0 g
s
.804 .651 .693 r
.40913 .97559 m .48864 .96614 L .4897 .94939 L p F P
0 g
s
.804 .651 .693 r
.4897 .94939 m .4146 .95939 L .40913 .97559 L p F P
0 g
s
.996 .861 .571 r
.40492 .80799 m .42256 .80676 L .43257 .77546 L p F P
0 g
s
0 0 0 r
.41705 .77872 m .42924 .74521 L p .42819 .74855 L F P
0 g
s
.996 .861 .571 r
.43257 .77546 m .41705 .77872 L p .40725 .80525 L F P
0 g
s
0 0 0 r
.41817 .78063 m .41705 .77872 L p .42819 .74855 L F P
0 g
s
.996 .861 .571 r
.41705 .77872 m .40492 .80799 L p .40725 .80525 L F P
0 g
s
0 0 .544 r
.63427 .69728 m .6196 .65126 L .60899 .63268 L p F P
0 g
s
.792 .69 .751 r
.4819 1.05547 m .37429 1.06137 L .36988 1.07044 L p F P
0 g
s
.794 .685 .744 r
.37429 1.06137 m .4819 1.05547 L .48278 1.04524 L p F P
0 g
s
.99 .758 .492 r
.46691 .89457 m .47785 .88047 L .48653 .79587 L p F P
0 g
s
.99 .758 .492 r
.48653 .79587 m .47878 .81152 L .46691 .89457 L p F P
0 g
s
.716 .654 .1 r
.46691 .89457 m .47878 .81152 L p .46994 .86677 L F P
0 g
s
.716 .654 .1 r
.46345 .90737 m .46508 .90134 L p .46994 .86677 L F P
0 g
s
.716 .654 .1 r
.46508 .90134 m .46691 .89457 L p .46994 .86677 L F P
0 g
s
.716 .654 .1 r
.47634 .82626 m .46345 .90737 L p .46994 .86677 L F P
0 g
s
.716 .654 .1 r
.47878 .81152 m .47634 .82626 L p .46994 .86677 L F P
0 g
s
.167 .005 .403 r
.47634 .82626 m .47851 .80582 L p .4787 .81131 L F P
0 g
s
.167 .005 .403 r
.47972 .84125 m .47634 .82626 L p .4787 .81131 L F P
0 g
s
.167 .005 .403 r
.47851 .80582 m .48909 .70627 L p .48576 .75424 L F P
0 g
s
.167 .005 .403 r
.48576 .75424 m .47972 .84125 L .47851 .80582 L F
.48909 .70627 m .49111 .72298 L .47972 .84125 L p F P
0 g
s
.549 .454 .699 r
.6788 .83269 m .63452 .88423 L .64199 .89961 L p F P
0 g
s
.544 .444 .691 r
.63452 .88423 m .6788 .83269 L .66846 .81789 L p F P
0 g
s
.802 .656 .7 r
.40377 .99064 m .4876 .98174 L .48864 .96614 L p F P
0 g
s
.802 .656 .7 r
.48864 .96614 m .40913 .97559 L .40377 .99064 L p F P
0 g
s
0 .024 .584 r
.6196 .65126 m .63427 .69728 L .6461 .71609 L p F P
0 g
s
.794 .685 .744 r
.48278 1.04524 m .37886 1.05161 L .37429 1.06137 L p F P
0 g
s
.795 .68 .737 r
.37886 1.05161 m .48278 1.04524 L .48369 1.03425 L p F P
0 g
s
.795 .68 .737 r
.48369 1.03425 m .38357 1.0411 L .37886 1.05161 L p F P
0 g
s
.797 .675 .73 r
.38357 1.0411 m .48369 1.03425 L .48463 1.02247 L p F P
0 g
s
.801 .661 .708 r
.39852 1.00465 m .48658 .99627 L .4876 .98174 L p F P
0 g
s
.801 .661 .708 r
.4876 .98174 m .40377 .99064 L .39852 1.00465 L p F P
0 g
s
.902 .73 .659 r
.39306 .88259 m .4375 .88049 L .44344 .8563 L p F P
0 g
s
.902 .73 .659 r
.44344 .8563 m .40279 .8598 L .39306 .88259 L p F P
0 g
s
.797 .675 .73 r
.48463 1.02247 m .38842 1.02981 L .38357 1.0411 L p F P
0 g
s
.798 .67 .722 r
.38842 1.02981 m .48463 1.02247 L .48559 1.00983 L p F P
0 g
s
.8 .665 .715 r
.39341 1.01768 m .48559 1.00983 L .48658 .99627 L p F P
0 g
s
.8 .665 .715 r
.48658 .99627 m .39852 1.00465 L .39341 1.01768 L p F P
0 g
s
.798 .67 .722 r
.48559 1.00983 m .39341 1.01768 L .38842 1.02981 L p F P
0 g
s
.35 .323 .688 r
.65781 .73301 m .64728 .78418 L .65795 .80177 L p F P
0 g
s
.335 .304 .675 r
.64728 .78418 m .65781 .73301 L .6461 .71609 L p F P
0 g
s
.555 .464 .707 r
.68896 .84632 m .64199 .89961 L .6493 .91388 L p F P
0 g
s
.549 .454 .699 r
.64199 .89961 m .68896 .84632 L .6788 .83269 L p F P
0 g
s
.797 .49 .469 r
.53089 .73255 m .5073 .70945 L .50742 .65728 L p F P
0 g
s
.797 .49 .469 r
.50742 .65728 m .52566 .67953 L .53089 .73255 L p F P
0 g
s
.56 .474 .714 r
.69891 .85888 m .6493 .91388 L .65646 .92713 L p F P
0 g
s
.555 .464 .707 r
.6493 .91388 m .69891 .85888 L .68896 .84632 L p F P
0 g
s
.775 .536 .574 r
.49419 .86373 m .51291 .84353 L .51144 .75832 L p F P
0 g
s
.775 .536 .574 r
.51144 .75832 m .49813 .77833 L .49419 .86373 L p F P
0 g
s
.565 .484 .721 r
.70866 .87046 m .65646 .92713 L .66344 .93943 L p F P
0 g
s
.56 .474 .714 r
.65646 .92713 m .70866 .87046 L .69891 .85888 L p F P
0 g
s
.898 .734 .67 r
.38345 .90338 m .43164 .90266 L .4375 .88049 L p F P
0 g
s
.898 .734 .67 r
.4375 .88049 m .39306 .88259 L .38345 .90338 L p F P
0 g
s
.621 .584 .791 r
.72719 1.02782 m .80551 .95501 L .79906 .9509 L p F P
0 g
s
.621 .584 .791 r
.79906 .9509 m .7225 1.02264 L .72719 1.02782 L p F P
0 g
s
.571 .493 .728 r
.71819 .88115 m .66344 .93943 L .67025 .95087 L p F P
0 g
s
.565 .484 .721 r
.66344 .93943 m .71819 .88115 L .70866 .87046 L p F P
0 g
s
.365 .342 .7 r
.66937 .74831 m .65795 .80177 L .66846 .81789 L p F P
0 g
s
.35 .323 .688 r
.65795 .80177 m .66937 .74831 L .65781 .73301 L p F P
0 g
s
.616 .575 .785 r
.7225 1.02264 m .79906 .9509 L .79229 .94635 L p F P
0 g
s
.616 .575 .785 r
.79229 .94635 m .71757 1.01699 L .7225 1.02264 L p F P
0 g
s
.576 .503 .735 r
.72749 .89102 m .67025 .95087 L .67688 .9615 L p F P
0 g
s
.571 .493 .728 r
.67025 .95087 m .72749 .89102 L .71819 .88115 L p F P
0 g
s
.611 .566 .779 r
.71757 1.01699 m .79229 .94635 L .7852 .94136 L p F P
0 g
s
.611 .566 .779 r
.7852 .94136 m .71241 1.01082 L .71757 1.01699 L p F P
0 g
s
.606 .557 .773 r
.71241 1.01082 m .7852 .94136 L .77781 .93588 L p F P
0 g
s
.606 .557 .773 r
.77781 .93588 m .70702 1.00413 L .71241 1.01082 L p F P
0 g
s
.531 .261 .451 r
.47849 .73264 m .45547 .72059 L .46518 .67816 L p F P
0 g
s
.531 .261 .451 r
.46518 .67816 m .48394 .69148 L .47849 .73264 L p F P
0 g
s
.581 .512 .742 r
.73654 .90012 m .67688 .9615 L .68331 .97136 L p F P
0 g
s
.576 .503 .735 r
.67688 .9615 m .73654 .90012 L .72749 .89102 L p F P
0 g
s
.601 .548 .767 r
.70702 1.00413 m .77781 .93588 L .77012 .92989 L p F P
0 g
s
.601 .548 .767 r
.77012 .92989 m .70141 .99687 L .70702 1.00413 L p F P
0 g
s
.586 .521 .749 r
.74534 .90852 m .68331 .97136 L .68955 .98052 L p F P
0 g
s
.581 .512 .742 r
.68331 .97136 m .74534 .90852 L .73654 .90012 L p F P
0 g
s
0 0 0 r
.42924 .74521 m .43026 .74936 L .41817 .78063 L p F P
0 g
s
.596 .539 .761 r
.70141 .99687 m .77012 .92989 L .76214 .92336 L p F P
0 g
s
.596 .539 .761 r
.76214 .92336 m .69559 .98901 L .70141 .99687 L p F P
0 g
s
.591 .53 .755 r
.75387 .91625 m .68955 .98052 L .69559 .98901 L p F P
0 g
s
.586 .521 .749 r
.68955 .98052 m .75387 .91625 L .74534 .90852 L p F P
0 g
s
.591 .53 .755 r
.69559 .98901 m .76214 .92336 L .75387 .91625 L p F P
0 g
s
.993 .868 .597 r
.39287 .83389 m .41263 .83467 L .42256 .80676 L p F P
0 g
s
.993 .868 .597 r
.42256 .80676 m .40492 .80799 L .39287 .83389 L p F P
0 g
s
0 .072 .619 r
.63012 .66766 m .6461 .71609 L .65781 .73301 L p F P
0 g
s
0 .024 .584 r
.6461 .71609 m .63012 .66766 L .6196 .65126 L p F P
0 g
s
.894 .738 .681 r
.37397 .92243 m .42586 .92307 L .43164 .90266 L p F P
0 g
s
.894 .738 .681 r
.43164 .90266 m .38345 .90338 L .37397 .92243 L p F P
0 g
s
.916 .584 .399 r
.55026 .7603 m .53089 .73255 L .52566 .67953 L p F P
0 g
s
.916 .584 .399 r
.52566 .67953 m .54064 .70543 L .55026 .7603 L p F P
0 g
s
.38 .36 .711 r
.68078 .76217 m .66846 .81789 L .6788 .83269 L p F P
0 g
s
.365 .342 .7 r
.66846 .81789 m .68078 .76217 L .66937 .74831 L p F P
0 g
s
.88 .633 .56 r
.47785 .88047 m .49419 .86373 L .49813 .77833 L p F P
0 g
s
.88 .633 .56 r
.49813 .77833 m .48653 .79587 L .47785 .88047 L p F P
0 g
s
.481 .254 .492 r
.48835 .85764 m .47972 .84125 L .49111 .72298 L p F P
0 g
s
.481 .254 .492 r
.49111 .72298 m .49625 .74059 L .48835 .85764 L p F P
0 g
s
.243 .004 .339 r
.44576 .75542 m .43026 .74936 L .44236 .71255 L p F P
0 g
s
.243 .004 .339 r
.44236 .71255 m .45547 .72059 L .44576 .75542 L p F P
0 g
s
.89 .742 .691 r
.36464 .93994 m .42018 .94193 L .42586 .92307 L p F P
0 g
s
.89 .742 .691 r
.42586 .92307 m .37397 .92243 L .36464 .93994 L p F P
0 g
s
.887 .746 .701 r
.35546 .95608 m .4146 .95939 L .42018 .94193 L p F P
0 g
s
.887 .746 .701 r
.42018 .94193 m .36464 .93994 L .35546 .95608 L p F P
0 g
s
.394 .377 .721 r
.69201 .77477 m .6788 .83269 L .68896 .84632 L p F P
0 g
s
.38 .36 .711 r
.6788 .83269 m .69201 .77477 L .68078 .76217 L p F P
0 g
s
.822 .796 .828 r
.24342 1.09289 m .34679 1.1122 L .35022 1.10659 L p F P
0 g
s
0 .118 .651 r
.64055 .68223 m .65781 .73301 L .66937 .74831 L p F P
0 g
s
0 .072 .619 r
.65781 .73301 m .64055 .68223 L .63012 .66766 L p F P
0 g
s
.989 .874 .621 r
.38092 .85702 m .40279 .8598 L .41263 .83467 L p F P
0 g
s
.989 .874 .621 r
.41263 .83467 m .39287 .83389 L .38092 .85702 L p F P
0 g
s
.883 .75 .71 r
.34645 .97099 m .40913 .97559 L .4146 .95939 L p F P
0 g
s
.883 .75 .71 r
.4146 .95939 m .35546 .95608 L .34645 .97099 L p F P
0 g
s
.822 .796 .828 r
.35022 1.10659 m .2491 1.0881 L .24342 1.09289 L p F P
0 g
s
.827 .793 .82 r
.2491 1.0881 m .35022 1.10659 L .35382 1.10049 L p F P
0 g
s
.827 .793 .82 r
.35382 1.10049 m .25507 1.08284 L .2491 1.0881 L p F P
0 g
s
.832 .79 .812 r
.25507 1.08284 m .35382 1.10049 L .35759 1.09385 L p F P
0 g
s
.879 .754 .72 r
.33763 .98478 m .40377 .99064 L .40913 .97559 L p F P
0 g
s
.879 .754 .72 r
.40913 .97559 m .34645 .97099 L .33763 .98478 L p F P
0 g
s
.832 .79 .812 r
.35759 1.09385 m .26132 1.0771 L .25507 1.08284 L p F P
0 g
s
.837 .787 .804 r
.26132 1.0771 m .35759 1.09385 L .36152 1.08665 L p F P
0 g
s
.407 .393 .731 r
.70305 .78624 m .68896 .84632 L .69891 .85888 L p F P
0 g
s
.394 .377 .721 r
.68896 .84632 m .70305 .78624 L .69201 .77477 L p F P
0 g
s
.837 .787 .804 r
.36152 1.08665 m .26785 1.07083 L .26132 1.0771 L p F P
0 g
s
.841 .784 .796 r
.26785 1.07083 m .36152 1.08665 L .36562 1.07886 L p F P
0 g
s
.875 .757 .729 r
.32899 .99755 m .39852 1.00465 L .40377 .99064 L p F P
0 g
s
.875 .757 .729 r
.40377 .99064 m .33763 .98478 L .32899 .99755 L p F P
0 g
s
.354 .224 .562 r
.55457 .85541 m .56329 .82374 L .55077 .76357 L p F P
0 g
s
.354 .224 .562 r
.55077 .76357 m .54403 .79265 L .55457 .85541 L p F P
0 g
s
.841 .784 .796 r
.36562 1.07886 m .27463 1.064 L .26785 1.07083 L p F P
0 g
s
.846 .78 .788 r
.27463 1.064 m .36562 1.07886 L .36988 1.07044 L p F P
0 g
s
.871 .761 .738 r
.32055 1.00938 m .39341 1.01768 L .39852 1.00465 L p F P
0 g
s
.871 .761 .738 r
.39852 1.00465 m .32899 .99755 L .32055 1.00938 L p F P
0 g
s
.846 .78 .788 r
.36988 1.07044 m .28168 1.0566 L .27463 1.064 L p F P
0 g
s
.85 .777 .78 r
.28168 1.0566 m .36988 1.07044 L .37429 1.06137 L p F P
0 g
s
.867 .764 .746 r
.31232 1.02033 m .38842 1.02981 L .39341 1.01768 L p F P
0 g
s
.867 .764 .746 r
.39341 1.01768 m .32055 1.00938 L .31232 1.02033 L p F P
0 g
s
.85 .777 .78 r
.37429 1.06137 m .28898 1.04857 L .28168 1.0566 L p F P
0 g
s
.855 .774 .772 r
.28898 1.04857 m .37429 1.06137 L .37886 1.05161 L p F P
0 g
s
0 0 0 r
.40613 .80767 m .40492 .80799 L .41705 .77872 L p F P
0 g
s
0 0 0 r
.41705 .77872 m .41817 .78063 L .40613 .80767 L p F P
0 g
s
.863 .767 .755 r
.30431 1.03048 m .38357 1.0411 L .38842 1.02981 L p F P
0 g
s
.863 .767 .755 r
.38842 1.02981 m .31232 1.02033 L .30431 1.03048 L p F P
0 g
s
.855 .774 .772 r
.37886 1.05161 m .29653 1.03988 L .28898 1.04857 L p F P
0 g
s
.859 .771 .763 r
.29653 1.03988 m .37886 1.05161 L .38357 1.0411 L p F P
0 g
s
.859 .771 .763 r
.38357 1.0411 m .30431 1.03048 L .29653 1.03988 L p F P
0 g
s
.773 .542 .585 r
.51291 .84353 m .49419 .86373 L .49023 .92967 L p F P
0 g
s
.876 .637 .575 r
.46914 .94553 m .49023 .92967 L .49419 .86373 L p F P
0 g
s
.876 .637 .575 r
.49419 .86373 m .47785 .88047 L .46914 .94553 L p F P
0 g
s
.637 .381 .514 r
.50066 .87633 m .48835 .85764 L .49625 .74059 L p F P
0 g
s
.637 .381 .514 r
.49625 .74059 m .50356 .75964 L .50066 .87633 L p F P
0 g
s
.677 .383 .468 r
.50718 .75032 m .47849 .73264 L .48394 .69148 L p F P
0 g
s
.677 .383 .468 r
.48394 .69148 m .5073 .70945 L .50718 .75032 L p F P
0 g
s
.42 .408 .741 r
.7139 .7967 m .69891 .85888 L .70866 .87046 L p F P
0 g
s
.407 .393 .731 r
.69891 .85888 m .7139 .7967 L .70305 .78624 L p F P
0 g
s
.554 .367 .582 r
.53721 .88441 m .55457 .85541 L .54403 .79265 L p F P
0 g
s
.554 .367 .582 r
.54403 .79265 m .53059 .81967 L .53721 .88441 L p F P
0 g
s
0 .16 .679 r
.65086 .69521 m .66937 .74831 L .68078 .76217 L p F P
0 g
s
0 .118 .651 r
.66937 .74831 m .65086 .69521 L .64055 .68223 L p F P
0 g
s
.984 .879 .643 r
.36909 .87784 m .39306 .88259 L .40279 .8598 L p F P
0 g
s
.984 .879 .643 r
.40279 .8598 m .38092 .85702 L .36909 .87784 L p F P
0 g
s
.432 .423 .749 r
.72452 .80623 m .70866 .87046 L .71819 .88115 L p F P
0 g
s
.42 .408 .741 r
.70866 .87046 m .72452 .80623 L .7139 .7967 L p F P
0 g
s
.674 .459 .586 r
.51438 .90937 m .53721 .88441 L .53059 .81967 L p F P
0 g
s
.674 .459 .586 r
.53059 .81967 m .51291 .84353 L .51438 .90937 L p F P
0 g
s
.444 .438 .757 r
.73491 .81492 m .71819 .88115 L .72749 .89102 L p F P
0 g
s
.432 .423 .749 r
.71819 .88115 m .73491 .81492 L .72452 .80623 L p F P
0 g
s
.979 .883 .663 r
.35738 .89668 m .38345 .90338 L .39306 .88259 L p F P
0 g
s
.979 .883 .663 r
.39306 .88259 m .36909 .87784 L .35738 .89668 L p F P
0 g
s
.456 .452 .765 r
.74507 .82284 m .72749 .89102 L .73654 .90012 L p F P
0 g
s
.444 .438 .757 r
.72749 .89102 m .74507 .82284 L .73491 .81492 L p F P
0 g
s
.035 .2 .705 r
.66104 .7068 m .68078 .76217 L .69201 .77477 L p F P
0 g
s
0 .16 .679 r
.68078 .76217 m .66104 .7068 L .65086 .69521 L p F P
0 g
s
.217 0 .304 r
.43604 .7847 m .41817 .78063 L .43026 .74936 L p F P
0 g
s
.217 0 .304 r
.43026 .74936 m .44576 .75542 L .43604 .7847 L p F P
0 g
s
.521 .241 .431 r
.47303 .76616 m .44576 .75542 L .45547 .72059 L p F P
0 g
s
.521 .241 .431 r
.45547 .72059 m .47849 .73264 L .47303 .76616 L p F P
0 g
s
.467 .465 .773 r
.75496 .83005 m .73654 .90012 L .74534 .90852 L p F P
0 g
s
.456 .452 .765 r
.73654 .90012 m .75496 .83005 L .74507 .82284 L p F P
0 g
s
0 0 0 r
.39416 .83135 m .39287 .83389 L .40492 .80799 L p F P
0 g
s
0 0 0 r
.40492 .80799 m .40613 .80767 L .39416 .83135 L p F P
0 g
s
.773 .542 .585 r
.49023 .92967 m .51438 .90937 L .51291 .84353 L p F P
0 g
s
.752 .475 .512 r
.51437 .89782 m .50066 .87633 L .50356 .75964 L p F P
0 g
s
.752 .475 .512 r
.50356 .75964 m .51171 .78045 L .51437 .89782 L p F P
0 g
s
.795 .477 .451 r
.53615 .77422 m .50718 .75032 L .5073 .70945 L p F P
0 g
s
.795 .477 .451 r
.5073 .70945 m .53089 .73255 L .53615 .77422 L p F P
0 g
s
.478 .478 .779 r
.76459 .8366 m .74534 .90852 L .75387 .91625 L p F P
0 g
s
.467 .465 .773 r
.74534 .90852 m .76459 .8366 L .75496 .83005 L p F P
0 g
s
.973 .886 .682 r
.34583 .91381 m .37397 .92243 L .38345 .90338 L p F P
0 g
s
.973 .886 .682 r
.38345 .90338 m .35738 .89668 L .34583 .91381 L p F P
0 g
s
.489 .491 .786 r
.77394 .84256 m .75387 .91625 L .76214 .92336 L p F P
0 g
s
.478 .478 .779 r
.75387 .91625 m .77394 .84256 L .76459 .8366 L p F P
0 g
s
.076 .237 .727 r
.67108 .71717 m .69201 .77477 L .70305 .78624 L p F P
0 g
s
.035 .2 .705 r
.69201 .77477 m .67108 .71717 L .66104 .7068 L p F P
0 g
s
.499 .503 .792 r
.78299 .84795 m .76214 .92336 L .77012 .92989 L p F P
0 g
s
.489 .491 .786 r
.76214 .92336 m .78299 .84795 L .77394 .84256 L p F P
0 g
s
.509 .515 .798 r
.79175 .85282 m .77012 .92989 L .77781 .93588 L p F P
0 g
s
.499 .503 .792 r
.77012 .92989 m .79175 .85282 L .78299 .84795 L p F P
0 g
s
.519 .527 .804 r
.8002 .85722 m .77781 .93588 L .7852 .94136 L p F P
0 g
s
.509 .515 .798 r
.77781 .93588 m .8002 .85722 L .79175 .85282 L p F P
0 g
s
.529 .538 .809 r
.80832 .86116 m .7852 .94136 L .79229 .94635 L p F P
0 g
s
.519 .527 .804 r
.7852 .94136 m .80832 .86116 L .8002 .85722 L p F P
0 g
s
.538 .549 .814 r
.81611 .86468 m .79229 .94635 L .79906 .9509 L p F P
0 g
s
.529 .538 .809 r
.79229 .94635 m .81611 .86468 L .80832 .86116 L p F P
0 g
s
.967 .889 .699 r
.33444 .92944 m .36464 .93994 L .37397 .92243 L p F P
0 g
s
.967 .889 .699 r
.37397 .92243 m .34583 .91381 L .33444 .92944 L p F P
0 g
s
.548 .56 .819 r
.82356 .86782 m .79906 .9509 L .80551 .95501 L p F P
0 g
s
.538 .549 .814 r
.79906 .9509 m .82356 .86782 L .81611 .86468 L p F P
0 g
s
.548 .56 .819 r
.80551 .95501 m .83066 .8706 L .82356 .86782 L p F P
0 g
s
.913 .568 .372 r
.55992 .80373 m .53615 .77422 L .53089 .73255 L p F P
0 g
s
.913 .568 .372 r
.53089 .73255 m .55026 .7603 L .55992 .80373 L p F P
0 g
s
.115 .271 .747 r
.68096 .72645 m .70305 .78624 L .7139 .7967 L p F P
0 g
s
.076 .237 .727 r
.70305 .78624 m .68096 .72645 L .67108 .71717 L p F P
0 g
s
.675 .467 .596 r
.53721 .88441 m .51438 .90937 L .51586 .96335 L p F P
0 g
s
.865 .568 .484 r
.52693 .9221 m .51437 .89782 L .51171 .78045 L p F P
0 g
s
.865 .568 .484 r
.51171 .78045 m .51917 .80301 L .52693 .9221 L p F P
0 g
s
.96 .891 .715 r
.32324 .94372 m .35546 .95608 L .36464 .93994 L p F P
0 g
s
.96 .891 .715 r
.36464 .93994 m .33444 .92944 L .32324 .94372 L p F P
0 g
s
.651 .768 .295 r
.38226 .85228 m .38092 .85702 L .39287 .83389 L p F P
0 g
s
.651 .768 .295 r
.39287 .83389 m .39416 .83135 L .38226 .85228 L p F P
0 g
s
.364 .242 .577 r
.56511 .90646 m .57584 .8723 L .56329 .82374 L p F P
0 g
s
.364 .242 .577 r
.56329 .82374 m .55457 .85541 L .56511 .90646 L p F P
0 g
s
.191 0 .267 r
.42636 .80975 m .40613 .80767 L .41817 .78063 L p F P
0 g
s
.191 0 .267 r
.41817 .78063 m .43604 .7847 L .42636 .80975 L p F P
0 g
s
.557 .377 .593 r
.54382 .93736 m .56511 .90646 L .55457 .85541 L p F P
0 g
s
.557 .377 .593 r
.55457 .85541 m .53721 .88441 L .54382 .93736 L p F P
0 g
s
.151 .302 .764 r
.69067 .73476 m .7139 .7967 L .72452 .80623 L p F P
0 g
s
.115 .271 .747 r
.7139 .7967 m .69067 .73476 L .68096 .72645 L p F P
0 g
s
.954 .893 .73 r
.31222 .95681 m .34645 .97099 L .35546 .95608 L p F P
0 g
s
.954 .893 .73 r
.35546 .95608 m .32324 .94372 L .31222 .95681 L p F P
0 g
s
.672 .368 .451 r
.50706 .78351 m .47303 .76616 L .47849 .73264 L p F P
0 g
s
.672 .368 .451 r
.47849 .73264 m .50718 .75032 L .50706 .78351 L p F P
0 g
s
.675 .467 .596 r
.51586 .96335 m .54382 .93736 L .53721 .88441 L p F P
0 g
s
.946 .894 .744 r
.30142 .96882 m .33763 .98478 L .34645 .97099 L p F P
0 g
s
.946 .894 .744 r
.34645 .97099 m .31222 .95681 L .30142 .96882 L p F P
0 g
s
.676 .474 .605 r
.54382 .93736 m .51586 .96335 L .51733 1.00921 L p F P
0 g
s
.973 .669 .367 r
.53599 .94866 m .52693 .9221 L .51917 .80301 L p F P
0 g
s
.973 .669 .367 r
.51917 .80301 m .52456 .82701 L .53599 .94866 L p F P
0 g
s
.185 .332 .78 r
.70019 .7422 m .72452 .80623 L .73491 .81492 L p F P
0 g
s
.151 .302 .764 r
.72452 .80623 m .70019 .7422 L .69067 .73476 L p F P
0 g
s
.51 .221 .41 r
.46757 .79412 m .43604 .7847 L .44576 .75542 L p F P
0 g
s
.51 .221 .41 r
.44576 .75542 m .47303 .76616 L .46757 .79412 L p F P
0 g
s
.939 .895 .758 r
.29083 .97984 m .32899 .99755 L .33763 .98478 L p F P
0 g
s
.939 .895 .758 r
.33763 .98478 m .30142 .96882 L .29083 .97984 L p F P
0 g
s
.687 .81 .357 r
.37047 .87092 m .36909 .87784 L .38092 .85702 L p F P
0 g
s
.687 .81 .357 r
.38092 .85702 m .38226 .85228 L .37047 .87092 L p F P
0 g
s
.986 .765 .521 r
.45499 .95802 m .46914 .94553 L .47785 .88047 L p F P
0 g
s
.986 .765 .521 r
.47785 .88047 m .46691 .89457 L .45499 .95802 L p F P
0 g
s
.774 .713 .177 r
.45499 .95802 m .46691 .89457 L p .4551 .94793 L F P
0 g
s
.774 .713 .177 r
.45048 .9688 m .45174 .96581 L p .4551 .94793 L F P
0 g
s
.774 .713 .177 r
.45174 .96581 m .45499 .95802 L p .4551 .94793 L F P
0 g
s
.774 .713 .177 r
.46345 .90737 m .45048 .9688 L p .4551 .94793 L F P
0 g
s
.774 .713 .177 r
.46691 .89457 m .46345 .90737 L p .4551 .94793 L F P
0 g
s
.112 0 .36 r
.46345 .90737 m .46727 .8833 L p .46739 .88819 L F P
0 g
s
.112 0 .36 r
.46819 .92047 m .46345 .90737 L p .46739 .88819 L F P
0 g
s
.112 0 .36 r
.46727 .8833 m .47634 .82626 L p .47491 .84272 L F P
0 g
s
.112 0 .36 r
.47491 .84272 m .46819 .92047 L .46727 .8833 L F
.47634 .82626 m .47972 .84125 L .46819 .92047 L p F P
0 g
s
.932 .896 .77 r
.28049 .98996 m .32055 1.00938 L .32899 .99755 L p F P
0 g
s
.932 .896 .77 r
.32899 .99755 m .29083 .97984 L .28049 .98996 L p F P
0 g
s
.217 .359 .793 r
.70952 .74885 m .73491 .81492 L .74507 .82284 L p F P
0 g
s
.185 .332 .78 r
.73491 .81492 m .70952 .74885 L .70019 .7422 L p F P
0 g
s
.772 .548 .595 r
.48628 .98367 m .51586 .96335 L .51438 .90937 L p F P
0 g
s
.772 .548 .595 r
.51438 .90937 m .49023 .92967 L .48628 .98367 L p F P
0 g
s
.924 .896 .782 r
.27039 .99925 m .31232 1.02033 L .32055 1.00938 L p F P
0 g
s
.924 .896 .782 r
.32055 1.00938 m .28049 .98996 L .27039 .99925 L p F P
0 g
s
.916 .896 .794 r
.26055 1.00778 m .30431 1.03048 L .31232 1.02033 L p F P
0 g
s
.916 .896 .794 r
.31232 1.02033 m .27039 .99925 L .26055 1.00778 L p F P
0 g
s
.908 .896 .804 r
.25098 1.0156 m .29653 1.03988 L .30431 1.03048 L p F P
0 g
s
.908 .896 .804 r
.30431 1.03048 m .26055 1.00778 L .25098 1.0156 L p F P
0 g
s
.792 .463 .432 r
.54142 .80816 m .50706 .78351 L .50718 .75032 L p F P
0 g
s
.792 .463 .432 r
.50718 .75032 m .53615 .77422 L .54142 .80816 L p F P
0 g
s
.247 .385 .805 r
.71864 .75477 m .74507 .82284 L .75496 .83005 L p F P
0 g
s
.217 .359 .793 r
.74507 .82284 m .71864 .75477 L .70952 .74885 L p F P
0 g
s
.9 .896 .815 r
.24169 1.02276 m .28898 1.04857 L .29653 1.03988 L p F P
0 g
s
.9 .896 .815 r
.29653 1.03988 m .25098 1.0156 L .24169 1.02276 L p F P
0 g
s
.163 0 .227 r
.41671 .83144 m .39416 .83135 L .40613 .80767 L p F P
0 g
s
.163 0 .227 r
.40613 .80767 m .42636 .80975 L .41671 .83144 L p F P
0 g
s
.892 .895 .825 r
.2327 1.02931 m .28168 1.0566 L .28898 1.04857 L p F P
0 g
s
.892 .895 .825 r
.28898 1.04857 m .24169 1.02276 L .2327 1.02931 L p F P
0 g
s
.718 .848 .415 r
.35879 .8876 m .35738 .89668 L .36909 .87784 L p F P
0 g
s
.718 .848 .415 r
.36909 .87784 m .37047 .87092 L .35879 .8876 L p F P
0 g
s
.883 .895 .834 r
.22401 1.03529 m .27463 1.064 L .28168 1.0566 L p F P
0 g
s
.883 .895 .834 r
.28168 1.0566 m .2327 1.02931 L .22401 1.03529 L p F P
0 g
s
.838 .889 .877 r
.18544 1.05781 m .24342 1.09289 L .2491 1.0881 L p F P
0 g
s
.874 .894 .844 r
.21563 1.04073 m .26785 1.07083 L .27463 1.064 L p F P
0 g
s
.874 .894 .844 r
.27463 1.064 m .22401 1.03529 L .21563 1.04073 L p F P
0 g
s
.838 .889 .877 r
.2491 1.0881 m .19248 1.05419 L .18544 1.05781 L p F P
0 g
s
.847 .891 .869 r
.19248 1.05419 m .2491 1.0881 L .25507 1.08284 L p F P
0 g
s
.866 .893 .852 r
.20758 1.04567 m .26132 1.0771 L .26785 1.07083 L p F P
0 g
s
.866 .893 .852 r
.26785 1.07083 m .21563 1.04073 L .20758 1.04567 L p F P
0 g
s
.847 .891 .869 r
.25507 1.08284 m .19986 1.05015 L .19248 1.05419 L p F P
0 g
s
.856 .892 .861 r
.19986 1.05015 m .25507 1.08284 L .26132 1.0771 L p F P
0 g
s
.856 .892 .861 r
.26132 1.0771 m .20758 1.04567 L .19986 1.05015 L p F P
0 g
s
.275 .408 .816 r
.72754 .76004 m .75496 .83005 L .76459 .8366 L p F P
0 g
s
.247 .385 .805 r
.75496 .83005 m .72754 .76004 L .71864 .75477 L p F P
0 g
s
.561 .388 .604 r
.55041 .98226 m .57563 .94953 L .56511 .90646 L p F P
0 g
s
.561 .388 .604 r
.56511 .90646 m .54382 .93736 L .55041 .98226 L p F P
0 g
s
.374 .26 .593 r
.57584 .8723 m .56511 .90646 L .57563 .94953 L p F P
0 g
s
.872 .643 .588 r
.46043 .99862 m .48628 .98367 L .49023 .92967 L p F P
0 g
s
.872 .643 .588 r
.49023 .92967 m .46914 .94553 L .46043 .99862 L p F P
0 g
s
.374 .26 .593 r
.57563 .94953 m .58838 .91296 L .57584 .8723 L p F P
0 g
s
.302 .43 .825 r
.7362 .76471 m .76459 .8366 L .77394 .84256 L p F P
0 g
s
.275 .408 .816 r
.76459 .8366 m .7362 .76471 L .72754 .76004 L p F P
0 g
s
.676 .474 .605 r
.51733 1.00921 m .55041 .98226 L .54382 .93736 L p F P
0 g
s
.909 .55 .343 r
.56959 .83938 m .54142 .80816 L .53615 .77422 L p F P
0 g
s
.909 .55 .343 r
.53615 .77422 m .55992 .80373 L .56959 .83938 L p F P
0 g
s
.46 .226 .472 r
.48034 .93546 m .46819 .92047 L .47972 .84125 L p F P
0 g
s
.46 .226 .472 r
.47972 .84125 m .48835 .85764 L .48034 .93546 L p F P
0 g
s
.745 .881 .47 r
.34725 .9026 m .34583 .91381 L .35738 .89668 L p F P
0 g
s
.745 .881 .47 r
.35738 .89668 m .35879 .8876 L .34725 .9026 L p F P
0 g
s
.844 .515 0 r
.58626 .87524 m .56959 .83938 L .55992 .80373 L p F P
0 g
s
.667 .353 .433 r
.50694 .81112 m .46757 .79412 L .47303 .76616 L p F P
0 g
s
.667 .353 .433 r
.47303 .76616 m .50706 .78351 L .50694 .81112 L p F P
0 g
s
.5 .199 .387 r
.46212 .81784 m .42636 .80975 L .43604 .7847 L p F P
0 g
s
.5 .199 .387 r
.43604 .7847 m .46757 .79412 L .46212 .81784 L p F P
0 g
s
.327 .451 .833 r
.74462 .76884 m .77394 .84256 L .78299 .84795 L p F P
0 g
s
.302 .43 .825 r
.77394 .84256 m .74462 .76884 L .7362 .76471 L p F P
0 g
s
.351 .47 .84 r
.75279 .77246 m .78299 .84795 L .79175 .85282 L p F P
0 g
s
.327 .451 .833 r
.78299 .84795 m .75279 .77246 L .74462 .76884 L p F P
0 g
s
.771 .554 .604 r
.48234 1.02952 m .51733 1.00921 L .51586 .96335 L p F P
0 g
s
.771 .554 .604 r
.51586 .96335 m .48628 .98367 L .48234 1.02952 L p F P
0 g
s
.133 0 .185 r
.40712 .85039 m .38226 .85228 L .39416 .83135 L p F P
0 g
s
.133 0 .185 r
.39416 .83135 m .41671 .83144 L .40712 .85039 L p F P
0 g
s
.767 .909 .522 r
.33587 .91612 m .33444 .92944 L .34583 .91381 L p F P
0 g
s
.767 .909 .522 r
.34583 .91381 m .34725 .9026 L .33587 .91612 L p F P
0 g
s
.373 .488 .847 r
.76069 .77562 m .79175 .85282 L .8002 .85722 L p F P
0 g
s
.351 .47 .84 r
.79175 .85282 m .76069 .77562 L .75279 .77246 L p F P
0 g
s
.394 .504 .852 r
.76832 .77836 m .8002 .85722 L .80832 .86116 L p F P
0 g
s
.373 .488 .847 r
.8002 .85722 m .76832 .77836 L .76069 .77562 L p F P
0 g
s
.414 .52 .857 r
.77566 .78071 m .80832 .86116 L .81611 .86468 L p F P
0 g
s
.394 .504 .852 r
.80832 .86116 m .77566 .78071 L .76832 .77836 L p F P
0 g
s
.565 .398 .614 r
.55698 1.02127 m .58611 .98677 L .57563 .94953 L p F P
0 g
s
.565 .398 .614 r
.57563 .94953 m .55041 .98226 L .55698 1.02127 L p F P
0 g
s
.385 .277 .607 r
.58838 .91296 m .57563 .94953 L .58611 .98677 L p F P
0 g
s
.677 .482 .614 r
.5188 1.04914 m .55698 1.02127 L .55041 .98226 L p F P
0 g
s
.677 .482 .614 r
.55041 .98226 m .51733 1.00921 L .5188 1.04914 L p F P
0 g
s
.785 .932 .569 r
.32464 .92833 m .32324 .94372 L .33444 .92944 L p F P
0 g
s
.785 .932 .569 r
.33444 .92944 m .33582 .91651 L p .32464 .92833 L F P
0 g
s
.8 .951 .613 r
.32324 .94372 m .32461 .92873 L p .3136 .93938 L F P
0 g
s
.8 .951 .613 r
.32461 .92873 m .32464 .92833 L .3136 .93938 L p F P
0 g
s
.8 .951 .613 r
.3136 .93938 m .31222 .95681 L .32324 .94372 L p F P
0 g
s
.811 .967 .653 r
.31222 .95681 m .31357 .93979 L p .30276 .94939 L F P
0 g
s
.811 .967 .653 r
.31357 .93979 m .3136 .93938 L .30276 .94939 L p F P
0 g
s
.811 .967 .653 r
.30276 .94939 m .30142 .96882 L .31222 .95681 L p F P
0 g
s
.819 .979 .689 r
.30142 .96882 m .30273 .9498 L p .29213 .95845 L F P
0 g
s
.819 .979 .689 r
.30273 .9498 m .30276 .94939 L .29213 .95845 L p F P
0 g
s
.819 .979 .689 r
.29213 .95845 m .29083 .97984 L .30142 .96882 L p F P
0 g
s
.824 .988 .722 r
.29083 .97984 m .29211 .95886 L p .28173 .96666 L F P
0 g
s
.824 .988 .722 r
.29211 .95886 m .29213 .95845 L .28173 .96666 L p F P
0 g
s
.824 .988 .722 r
.28173 .96666 m .28049 .98996 L .29083 .97984 L p F P
0 g
s
.826 .994 .751 r
.28049 .98996 m .2817 .96706 L p .27156 .97407 L F P
0 g
s
.826 .994 .751 r
.2817 .96706 m .28173 .96666 L .27156 .97407 L p F P
0 g
s
.826 .994 .751 r
.27156 .97407 m .27039 .99925 L .28049 .98996 L p F P
0 g
s
.827 .998 .778 r
.27039 .99925 m .27154 .97448 L p .26164 .98077 L F P
0 g
s
.827 .998 .778 r
.27154 .97448 m .27156 .97407 L .26164 .98077 L p F P
0 g
s
.827 .998 .778 r
.26164 .98077 m .26055 1.00778 L .27039 .99925 L p F P
0 g
s
.826 1 .802 r
.26055 1.00778 m .26162 .98118 L p .25199 .98681 L F P
0 g
s
.826 1 .802 r
.26162 .98118 m .26164 .98077 L .25199 .98681 L p F P
0 g
s
.826 1 .802 r
.25199 .98681 m .25098 1.0156 L .26055 1.00778 L p F P
0 g
s
.823 1 .823 r
.25098 1.0156 m .25197 .98721 L p .24261 .99225 L F P
0 g
s
.823 1 .823 r
.25197 .98721 m .25199 .98681 L .24261 .99225 L p F P
0 g
s
0 0 0 r
.29374 .96678 m .24323 .99194 L p .25199 .98681 L F P
0 g
s
0 0 0 r
.30139 .96296 m .25264 .9865 L p .26164 .98077 L F P
0 g
s
.823 1 .823 r
.24261 .99225 m .24169 1.02276 L .25098 1.0156 L p F P
0 g
s
.819 .998 .843 r
.24169 1.02276 m .2426 .99264 L p .23352 .99712 L F P
0 g
s
.819 .998 .843 r
.2426 .99264 m .24261 .99225 L .23352 .99712 L p F P
0 g
s
0 0 0 r
.26714 .97954 m .30066 .96331 L .29374 .96678 L F
.30139 .96296 m .29374 .96678 L p .30066 .96331 L F P
0 g
s
.106 0 0 r
.38286 .95804 m .2874 .96996 L p .29374 .96678 L F P
0 g
s
0 0 0 r
.26592 .98009 m .30139 .96296 L p .26714 .97954 L F P
0 g
s
.396 .82 .647 r
.24261 .99225 m .25654 .98531 L .25814 .98437 L F
.28735 .96996 m .2863 .97009 L p .25814 .98437 L F P
0 g
s
.396 .82 .647 r
.25654 .98531 m .28735 .96996 L .25814 .98437 L F
.24261 .99225 m .27142 .97764 L .23412 .99681 L F
0 0 0 r
.25199 .98681 m .26592 .98009 L p .26714 .97954 L F P
0 g
s
.396 .82 .647 r
.23412 .99681 m .23352 .99712 L .24261 .99225 L p F P
0 g
s
0 0 0 r
.24323 .99194 m .24261 .99225 L .25199 .98681 L p F P
0 g
s
.427 .85 .7 r
.23352 .99712 m .26392 .98099 L .22531 1.00117 L F
0 0 0 r
.26164 .98077 m .30927 .95857 L .30484 .96104 L p F P
0 g
s
0 0 0 r
.30484 .96104 m .30139 .96296 L p .29754 .96468 L F P
0 g
s
0 0 0 r
.26164 .98077 m .30484 .96104 L .29754 .96468 L F
.456 .874 .749 r
.21625 1.00534 m .22473 1.00147 L p .23256 .99716 L F P
0 g
s
0 0 0 r
.25264 .9865 m .25199 .98681 L .26164 .98077 L p F P
0 g
s
0 0 0 r
.30927 .95857 m .26232 .98046 L p .27156 .97407 L F P
0 g
s
.456 .874 .749 r
.27217 .97534 m .21721 1.00483 L p .23256 .99716 L F P
0 g
s
.456 .874 .749 r
.21721 1.00483 m .21625 1.00534 L p .23256 .99716 L F P
0 g
s
.427 .85 .7 r
.27911 .97293 m .26535 .98015 L p .26392 .98099 L F P
0 g
s
.427 .85 .7 r
.26535 .98015 m .22531 1.00117 L p .26392 .98099 L F P
0 g
s
.106 0 0 r
.2874 .96996 m .2863 .97009 L .29374 .96678 L p F P
0 g
s
.396 .82 .647 r
.2863 .97009 m .27253 .97715 L p .27142 .97764 L F P
0 g
s
.396 .82 .647 r
.27253 .97715 m .23412 .99681 L p .27142 .97764 L F P
0 g
s
.427 .85 .7 r
.22531 1.00117 m .22473 1.00147 L .23352 .99712 L p F P
0 g
s
0 0 0 r
.26232 .98046 m .26164 .98077 L .27156 .97407 L p F P
0 g
s
0 0 0 r
.31736 .95355 m .27227 .97376 L p .28173 .96666 L F P
0 g
s
0 0 0 r
.27227 .97376 m .27156 .97407 L .28173 .96666 L p F P
0 g
s
0 0 0 r
.32564 .94786 m .28247 .96634 L p .29213 .95845 L F P
0 g
s
0 0 0 r
.28247 .96634 m .28173 .96666 L .29213 .95845 L p F P
0 g
s
0 0 0 r
.33411 .94143 m .29292 .95813 L p .30276 .94939 L F P
0 g
s
0 0 0 r
.29292 .95813 m .29213 .95845 L .30276 .94939 L p F P
0 g
s
0 0 0 r
.34275 .93416 m .30359 .94907 L p .3136 .93938 L F P
0 g
s
0 0 0 r
.30359 .94907 m .30276 .94939 L .3136 .93938 L p F P
0 g
s
0 0 0 r
.35156 .92599 m .31449 .93907 L p .32464 .92833 L F P
0 g
s
0 0 0 r
.31449 .93907 m .3136 .93938 L .32464 .92833 L p F P
0 g
s
.785 .932 .569 r
.33582 .91651 m .33587 .91612 L .32464 .92833 L p F P
0 g
s
.982 .772 .546 r
.44306 1.00944 m .46043 .99862 L .46914 .94553 L p F P
0 g
s
.982 .772 .546 r
.46914 .94553 m .45499 .95802 L .44306 1.00944 L p F P
0 g
s
.433 .535 .861 r
.78271 .78271 m .81611 .86468 L .82356 .86782 L p F P
0 g
s
.414 .52 .857 r
.81611 .86468 m .78271 .78271 L .77566 .78071 L p F P
0 g
s
.626 .363 .499 r
.49771 .95358 m .48034 .93546 L .48835 .85764 L p F P
0 g
s
.626 .363 .499 r
.48835 .85764 m .50066 .87633 L .49771 .95358 L p F P
0 g
s
.789 .449 .411 r
.54669 .83648 m .50694 .81112 L .50706 .78351 L p F P
0 g
s
.789 .449 .411 r
.50706 .78351 m .54142 .80816 L .54669 .83648 L p F P
0 g
s
.451 .549 .864 r
.78945 .78439 m .82356 .86782 L .83066 .8706 L p F P
0 g
s
.433 .535 .861 r
.82356 .86782 m .78945 .78439 L .78271 .78271 L p F P
0 g
s
.87 .648 .6 r
.45174 1.04352 m .48234 1.02952 L .48628 .98367 L p F P
0 g
s
.87 .648 .6 r
.48628 .98367 m .46043 .99862 L .45174 1.04352 L p F P
0 g
s
.385 .277 .607 r
.58611 .98677 m .60089 .94784 L .58838 .91296 L p F P
0 g
s
.102 0 .139 r
.3976 .86706 m .37047 .87092 L .38226 .85228 L p F P
0 g
s
.102 0 .139 r
.38226 .85228 m .40712 .85039 L .3976 .86706 L p F P
0 g
s
.451 .549 .864 r
.83066 .8706 m .79589 .78577 L .78945 .78439 L p F P
0 g
s
.771 .56 .614 r
.47842 1.0694 m .5188 1.04914 L .51733 1.00921 L p F P
0 g
s
.771 .56 .614 r
.51733 1.00921 m .48234 1.02952 L .47842 1.0694 L p F P
0 g
s
.489 .176 .363 r
.45669 .83819 m .41671 .83144 L .42636 .80975 L p F P
0 g
s
.489 .176 .363 r
.42636 .80975 m .46212 .81784 L .45669 .83819 L p F P
0 g
s
.905 .53 .31 r
.57926 .86938 m .54669 .83648 L .54142 .80816 L p F P
0 g
s
.905 .53 .31 r
.54142 .80816 m .56959 .83938 L .57926 .86938 L p F P
0 g
s
.661 .337 .413 r
.50682 .83446 m .46212 .81784 L .46757 .79412 L p F P
0 g
s
.661 .337 .413 r
.46757 .79412 m .50694 .81112 L .50682 .83446 L p F P
0 g
s
.816 .76 .245 r
.44306 1.00944 m .45499 .95802 L p .44091 1.00641 L F P
0 g
s
.816 .76 .245 r
.43749 1.01819 m .43852 1.01657 L p .44091 1.00641 L F P
0 g
s
.816 .76 .245 r
.43852 1.01657 m .44306 1.00944 L p .44091 1.00641 L F P
0 g
s
.816 .76 .245 r
.45048 .9688 m .43749 1.01819 L p .44091 1.00641 L F P
0 g
s
.816 .76 .245 r
.45499 .95802 m .45048 .9688 L p .44091 1.00641 L F P
0 g
s
.07 0 .319 r
.45048 .9688 m .45638 .94085 L p .4564 .94553 L F P
0 g
s
.07 0 .319 r
.45658 .97993 m .45048 .9688 L p .4564 .94553 L F P
0 g
s
.07 0 .319 r
.45638 .94085 m .46345 .90737 L p .46289 .91328 L F P
0 g
s
.07 0 .319 r
.46289 .91328 m .45658 .97993 L .45638 .94085 L F
.46345 .90737 m .46819 .92047 L .45658 .97993 L p F P
0 g
s
.806 .467 0 r
.5985 .90756 m .57926 .86938 L .56959 .83938 L p F P
0 g
s
.679 .489 .622 r
.52026 1.08449 m .5635 1.05574 L .55698 1.02127 L p F P
0 g
s
.679 .489 .622 r
.55698 1.02127 m .5188 1.04914 L .52026 1.08449 L p F P
0 g
s
.569 .408 .623 r
.58611 .98677 m .55698 1.02127 L .5635 1.05574 L p F P
0 g
s
.569 .408 .623 r
.5635 1.05574 m .59653 1.01953 L .58611 .98677 L p F P
0 g
s
.396 .294 .62 r
.60089 .94784 m .58611 .98677 L .59653 1.01953 L p F P
0 g
s
.745 .459 .498 r
.51708 .97554 m .49771 .95358 L .50066 .87633 L p F P
0 g
s
.745 .459 .498 r
.50066 .87633 m .51437 .89782 L .51708 .97554 L p F P
0 g
s
.069 0 .09 r
.38817 .88179 m .35879 .8876 L .37047 .87092 L p F P
0 g
s
.069 0 .09 r
.37047 .87092 m .3976 .86706 L .38817 .88179 L p F P
0 g
s
.77 .566 .623 r
.47453 1.10469 m .52026 1.08449 L .5188 1.04914 L p F P
0 g
s
.77 .566 .623 r
.5188 1.04914 m .47842 1.0694 L .47453 1.10469 L p F P
0 g
s
.867 .654 .612 r
.44309 1.08245 m .47842 1.0694 L .48234 1.02952 L p F P
0 g
s
.867 .654 .612 r
.48234 1.02952 m .45174 1.04352 L .44309 1.08245 L p F P
0 g
s
.396 .294 .62 r
.59653 1.01953 m .61333 .9783 L .60089 .94784 L p F P
0 g
s
.978 .779 .567 r
.43115 1.05268 m .45174 1.04352 L .46043 .99862 L p F P
0 g
s
.978 .779 .567 r
.46043 .99862 m .44306 1.00944 L .43115 1.05268 L p F P
0 g
s
.476 .152 .336 r
.45129 .8558 m .40712 .85039 L .41671 .83144 L p F P
0 g
s
.476 .152 .336 r
.41671 .83144 m .45669 .83819 L .45129 .8558 L p F P
0 g
s
.68 .496 .631 r
.5217 1.11618 m .56997 1.08659 L .5635 1.05574 L p F P
0 g
s
.68 .496 .631 r
.5635 1.05574 m .52026 1.08449 L .5217 1.11618 L p F P
0 g
s
.573 .418 .633 r
.59653 1.01953 m .5635 1.05574 L .56997 1.08659 L p F P
0 g
s
.786 .433 .389 r
.55196 .86051 m .50682 .83446 L .50694 .81112 L p F P
0 g
s
.786 .433 .389 r
.50694 .81112 m .54669 .83648 L .55196 .86051 L p F P
0 g
s
.858 .552 .468 r
.5348 1.00132 m .51708 .97554 L .51437 .89782 L p F P
0 g
s
.858 .552 .468 r
.51437 .89782 m .52693 .9221 L .5348 1.00132 L p F P
0 g
s
.573 .418 .633 r
.56997 1.08659 m .60687 1.04871 L .59653 1.01953 L p F P
0 g
s
.406 .31 .633 r
.61333 .9783 m .59653 1.01953 L .60687 1.04871 L p F P
0 g
s
.035 0 .038 r
.37884 .89486 m .34725 .9026 L .35879 .8876 L p F P
0 g
s
.035 0 .038 r
.35879 .8876 m .38817 .88179 L .37884 .89486 L p F P
0 g
s
.77 .572 .631 r
.47067 1.13631 m .5217 1.11618 L .52026 1.08449 L p F P
0 g
s
.77 .572 .631 r
.52026 1.08449 m .47453 1.10469 L .47067 1.13631 L p F P
0 g
s
.656 .32 .392 r
.5067 .85442 m .45669 .83819 L .46212 .81784 L p F P
0 g
s
.656 .32 .392 r
.46212 .81784 m .50682 .83446 L .5067 .85442 L p F P
0 g
s
.445 .203 .451 r
.47228 .99342 m .45658 .97993 L .46819 .92047 L p F P
0 g
s
.445 .203 .451 r
.46819 .92047 m .48034 .93546 L .47228 .99342 L p F P
0 g
s
.865 .659 .623 r
.4345 1.11678 m .47453 1.10469 L .47842 1.0694 L p F P
0 g
s
.865 .659 .623 r
.47842 1.0694 m .44309 1.08245 L .4345 1.11678 L p F P
0 g
s
.681 .503 .639 r
.52313 1.14486 m .57637 1.11445 L .56997 1.08659 L p F P
0 g
s
.681 .503 .639 r
.56997 1.08659 m .5217 1.11618 L .52313 1.14486 L p F P
0 g
s
.577 .427 .642 r
.60687 1.04871 m .56997 1.08659 L .57637 1.11445 L p F P
0 g
s
.899 .509 .275 r
.58891 .89505 m .55196 .86051 L .54669 .83648 L p F P
0 g
s
.899 .509 .275 r
.54669 .83648 m .57926 .86938 L .58891 .89505 L p F P
0 g
s
.763 .416 0 r
.61069 .93551 m .58891 .89505 L .57926 .86938 L p F P
0 g
s
.406 .31 .633 r
.60687 1.04871 m .6257 1.00523 L .61333 .9783 L p F P
0 g
s
.966 .647 .345 r
.54756 1.0302 m .5348 1.00132 L .52693 .9221 L p F P
0 g
s
.966 .647 .345 r
.52693 .9221 m .53599 .94866 L .54756 1.0302 L p F P
0 g
s
.77 .578 .64 r
.46686 1.1649 m .52313 1.14486 L .5217 1.11618 L p F P
0 g
s
.77 .578 .64 r
.5217 1.11618 m .47067 1.13631 L .46686 1.1649 L p F P
0 g
s
.577 .427 .642 r
.57637 1.11445 m .61712 1.07495 L .60687 1.04871 L p F P
0 g
s
.416 .326 .646 r
.6257 1.00523 m .60687 1.04871 L .61712 1.07495 L p F P
0 g
s
0 0 0 r
.36961 .90646 m .33587 .91612 L .34725 .9026 L p F P
0 g
s
0 0 0 r
.34725 .9026 m .37884 .89486 L .36961 .90646 L p F P
0 g
s
.819 .998 .843 r
.23352 .99712 m .2327 1.02931 L .24169 1.02276 L p F P
0 g
s
.974 .785 .587 r
.41928 1.08994 m .44309 1.08245 L .45174 1.04352 L p F P
0 g
s
.974 .785 .587 r
.45174 1.04352 m .43115 1.05268 L .41928 1.08994 L p F P
0 g
s
.463 .126 .308 r
.44592 .87113 m .3976 .86706 L .40712 .85039 L p F P
0 g
s
.463 .126 .308 r
.40712 .85039 m .45129 .8558 L .44592 .87113 L p F P
0 g
s
.683 .51 .647 r
.52454 1.17101 m .5827 1.13981 L .57637 1.11445 L p F P
0 g
s
.683 .51 .647 r
.57637 1.11445 m .52313 1.14486 L .52454 1.17101 L p F P
0 g
s
.581 .437 .651 r
.61712 1.07495 m .57637 1.11445 L .5827 1.13981 L p F P
0 g
s
.813 .995 .86 r
.22473 1.00147 m .22401 1.03529 L .2327 1.02931 L p F P
0 g
s
.813 .995 .86 r
.2327 1.02931 m .23352 .99712 L .22473 1.00147 L p F P
0 g
s
.863 .665 .633 r
.42597 1.14743 m .47067 1.13631 L .47453 1.10469 L p F P
0 g
s
.863 .665 .633 r
.47453 1.10469 m .4345 1.11678 L .42597 1.14743 L p F P
0 g
s
.807 .992 .876 r
.21625 1.00534 m .21563 1.04073 L .22401 1.03529 L p F P
0 g
s
.807 .992 .876 r
.22401 1.03529 m .22473 1.00147 L .21625 1.00534 L p F P
0 g
s
.769 .584 .648 r
.46308 1.19095 m .52454 1.17101 L .52313 1.14486 L p F P
0 g
s
.769 .584 .648 r
.52313 1.14486 m .46686 1.1649 L .46308 1.19095 L p F P
0 g
s
.848 .798 .305 r
.43115 1.05268 m .44306 1.00944 L p .42714 1.05226 L F P
0 g
s
.848 .798 .305 r
.4245 1.05937 m .42537 1.05849 L p .42714 1.05226 L F P
0 g
s
.848 .798 .305 r
.42537 1.05849 m .43115 1.05268 L p .42714 1.05226 L F P
0 g
s
.848 .798 .305 r
.43749 1.01819 m .4245 1.05937 L p .42714 1.05226 L F P
0 g
s
.848 .798 .305 r
.44306 1.00944 m .43749 1.01819 L p .42714 1.05226 L F P
0 g
s
.032 0 .277 r
.43749 1.01819 m .44603 .98572 L p .44591 .99047 L F P
0 g
s
.032 0 .277 r
.44492 1.02731 m .43749 1.01819 L p .44591 .99047 L F P
0 g
s
.032 0 .277 r
.44603 .98572 m .45048 .9688 L p .45031 .97067 L F P
0 g
s
.032 0 .277 r
.45031 .97067 m .44492 1.02731 L .44603 .98572 L F
.45048 .9688 m .45658 .97993 L .44492 1.02731 L p F P
0 g
s
.8 .987 .89 r
.20809 1.00877 m .20758 1.04567 L .21563 1.04073 L p F P
0 g
s
.8 .987 .89 r
.21563 1.04073 m .21625 1.00534 L .20809 1.00877 L p F P
0 g
s
.792 .981 .902 r
.20026 1.0118 m .19986 1.05015 L .20758 1.04567 L p F P
0 g
s
.792 .981 .902 r
.20758 1.04567 m .20809 1.00877 L .20026 1.0118 L p F P
0 g
s
.684 .517 .655 r
.52593 1.19498 m .58894 1.16302 L .5827 1.13981 L p F P
0 g
s
.684 .517 .655 r
.5827 1.13981 m .52454 1.17101 L .52593 1.19498 L p F P
0 g
s
.584 .446 .659 r
.62726 1.09873 m .5827 1.13981 L .58894 1.16302 L p F P
0 g
s
.581 .437 .651 r
.5827 1.13981 m .62726 1.09873 L .61712 1.07495 L p F P
0 g
s
.426 .341 .657 r
.63797 1.02928 m .61712 1.07495 L .62726 1.09873 L p F P
0 g
s
.416 .326 .646 r
.61712 1.07495 m .63797 1.02928 L .6257 1.00523 L p F P
0 g
s
0 0 0 r
.36052 .91679 m .32464 .92833 L .33587 .91612 L p F P
0 g
s
0 0 0 r
.33587 .91612 m .36961 .90646 L .36052 .91679 L p F P
0 g
s
.782 .417 .365 r
.55721 .88114 m .5067 .85442 L .50682 .83446 L p F P
0 g
s
.782 .417 .365 r
.50682 .83446 m .55196 .86051 L .55721 .88114 L p F P
0 g
s
.783 .975 .913 r
.19277 1.01444 m .19248 1.05419 L .19986 1.05015 L p F P
0 g
s
.783 .975 .913 r
.19986 1.05015 m .20026 1.0118 L .19277 1.01444 L p F P
0 g
s
.769 .59 .656 r
.45936 1.21481 m .52593 1.19498 L .52454 1.17101 L p F P
0 g
s
.769 .59 .656 r
.52454 1.17101 m .46308 1.19095 L .45936 1.21481 L p F P
0 g
s
.774 .968 .923 r
.18563 1.01674 m .18544 1.05781 L .19248 1.05419 L p F P
0 g
s
.774 .968 .923 r
.19248 1.05419 m .19277 1.01444 L .18563 1.01674 L p F P
0 g
s
.619 .347 .484 r
.49474 1.01088 m .47228 .99342 L .48034 .93546 L p F P
0 g
s
.619 .347 .484 r
.48034 .93546 m .49771 .95358 L .49474 1.01088 L p F P
0 g
s
.649 .302 .369 r
.50659 .87163 m .45129 .8558 L .45669 .83819 L p F P
0 g
s
.649 .302 .369 r
.45669 .83819 m .5067 .85442 L .50659 .87163 L p F P
0 g
s
.86 .67 .644 r
.41753 1.17507 m .46686 1.1649 L .47067 1.13631 L p F P
0 g
s
.86 .67 .644 r
.47067 1.13631 m .42597 1.14743 L .41753 1.17507 L p F P
0 g
s
.761 .697 .791 r
.40561 1.43932 m .54606 1.42163 L .54525 1.41557 L p F P
0 g
s
.708 .642 .782 r
.54606 1.42163 m .6794 1.37973 L .67575 1.37404 L p F P
0 g
s
.708 .642 .782 r
.67575 1.37404 m .54525 1.41557 L .54606 1.42163 L p F P
0 g
s
.686 .524 .662 r
.52731 1.21705 m .5951 1.18435 L .58894 1.16302 L p F P
0 g
s
.686 .524 .662 r
.58894 1.16302 m .52593 1.19498 L .52731 1.21705 L p F P
0 g
s
.588 .455 .667 r
.63728 1.1204 m .58894 1.16302 L .5951 1.18435 L p F P
0 g
s
.584 .446 .659 r
.58894 1.16302 m .63728 1.1204 L .62726 1.09873 L p F P
0 g
s
.769 .596 .664 r
.4557 1.23676 m .52731 1.21705 L .52593 1.19498 L p F P
0 g
s
.769 .596 .664 r
.52593 1.19498 m .45936 1.21481 L .4557 1.23676 L p F P
0 g
s
.761 .697 .791 r
.54525 1.41557 m .40777 1.43336 L .40561 1.43932 L p F P
0 g
s
.762 .69 .783 r
.40777 1.43336 m .54525 1.41557 L .5444 1.40896 L p F P
0 g
s
.706 .635 .775 r
.54525 1.41557 m .67575 1.37404 L .67191 1.36783 L p F P
0 g
s
.706 .635 .775 r
.67191 1.36783 m .5444 1.40896 L .54525 1.41557 L p F P
0 g
s
.97 .791 .606 r
.40747 1.1226 m .4345 1.11678 L .44309 1.08245 L p F P
0 g
s
.97 .791 .606 r
.44309 1.08245 m .41928 1.08994 L .40747 1.1226 L p F P
0 g
s
.715 .362 0 r
.62282 .95999 m .59852 .91728 L .58891 .89505 L p F P
0 g
s
.891 .486 .236 r
.55196 .86051 m .58891 .89505 L .59852 .91728 L p F P
0 g
s
.449 .098 .277 r
.4406 .88453 m .38817 .88179 L .3976 .86706 L p F P
0 g
s
.449 .098 .277 r
.3976 .86706 m .44592 .87113 L .4406 .88453 L p F P
0 g
s
.762 .69 .783 r
.5444 1.40896 m .41005 1.42686 L .40777 1.43336 L p F P
0 g
s
.763 .684 .775 r
.41005 1.42686 m .5444 1.40896 L .54351 1.40177 L p F P
0 g
s
.705 .627 .768 r
.5444 1.40896 m .67191 1.36783 L .66789 1.36106 L p F P
0 g
s
.705 .627 .768 r
.66789 1.36106 m .54351 1.40177 L .5444 1.40896 L p F P
0 g
s
.891 .486 .236 r
.59852 .91728 m .55721 .88114 L .55196 .86051 L p F P
0 g
s
.435 .356 .668 r
.65012 1.05091 m .62726 1.09873 L .63728 1.1204 L p F P
0 g
s
.426 .341 .657 r
.62726 1.09873 m .65012 1.05091 L .63797 1.02928 L p F P
0 g
s
.687 .531 .67 r
.52866 1.23744 m .60116 1.20403 L .5951 1.18435 L p F P
0 g
s
.687 .531 .67 r
.5951 1.18435 m .52731 1.21705 L .52866 1.23744 L p F P
0 g
s
.592 .464 .675 r
.64715 1.14022 m .5951 1.18435 L .60116 1.20403 L p F P
0 g
s
.588 .455 .667 r
.5951 1.18435 m .64715 1.14022 L .63728 1.1204 L p F P
0 g
s
0 0 0 r
.32464 .92833 m .36052 .91679 L .35156 .92599 L p F P
0 g
s
.763 .684 .775 r
.54351 1.40177 m .41243 1.41978 L .41005 1.42686 L p F P
0 g
s
.763 .677 .768 r
.41243 1.41978 m .54351 1.40177 L .54257 1.39398 L p F P
0 g
s
.704 .62 .761 r
.54351 1.40177 m .66789 1.36106 L .6637 1.3537 L p F P
0 g
s
.704 .62 .761 r
.6637 1.3537 m .54257 1.39398 L .54351 1.40177 L p F P
0 g
s
.769 .602 .672 r
.45209 1.25703 m .52866 1.23744 L .52731 1.21705 L p F P
0 g
s
.769 .602 .672 r
.52731 1.21705 m .4557 1.23676 L .45209 1.25703 L p F P
0 g
s
.858 .675 .654 r
.40919 1.20017 m .46308 1.19095 L .46686 1.1649 L p F P
0 g
s
.858 .675 .654 r
.46686 1.1649 m .41753 1.17507 L .40919 1.20017 L p F P
0 g
s
.763 .677 .768 r
.54257 1.39398 m .41492 1.4121 L .41243 1.41978 L p F P
0 g
s
.764 .671 .76 r
.41492 1.4121 m .54257 1.39398 L .5416 1.38555 L p F P
0 g
s
.703 .613 .754 r
.54257 1.39398 m .6637 1.3537 L .65933 1.34573 L p F P
0 g
s
.703 .613 .754 r
.65933 1.34573 m .5416 1.38555 L .54257 1.39398 L p F P
0 g
s
.768 .607 .679 r
.44855 1.27579 m .52998 1.25632 L .52866 1.23744 L p F P
0 g
s
.768 .607 .679 r
.52866 1.23744 m .45209 1.25703 L .44855 1.27579 L p F P
0 g
s
.688 .538 .677 r
.60116 1.20403 m .52866 1.23744 L .52998 1.25632 L p F P
0 g
s
.688 .538 .677 r
.52998 1.25632 m .60711 1.22222 L .60116 1.20403 L p F P
0 g
s
.595 .473 .683 r
.65688 1.15844 m .60116 1.20403 L .60711 1.22222 L p F P
0 g
s
.592 .464 .675 r
.60116 1.20403 m .65688 1.15844 L .64715 1.14022 L p F P
0 g
s
.764 .671 .76 r
.5416 1.38555 m .41751 1.40379 L .41492 1.4121 L p F P
0 g
s
.765 .665 .753 r
.41751 1.40379 m .5416 1.38555 L .54059 1.37645 L p F P
0 g
s
.701 .606 .747 r
.5416 1.38555 m .65933 1.34573 L .65479 1.33711 L p F P
0 g
s
.701 .606 .747 r
.65479 1.33711 m .54059 1.37645 L .5416 1.38555 L p F P
0 g
s
.765 .665 .753 r
.54059 1.37645 m .42021 1.3948 L .41751 1.40379 L p F P
0 g
s
.765 .659 .745 r
.42021 1.3948 m .54059 1.37645 L .53955 1.36664 L p F P
0 g
s
.7 .599 .74 r
.54059 1.37645 m .65479 1.33711 L .65008 1.32779 L p F P
0 g
s
.7 .599 .74 r
.65008 1.32779 m .53955 1.36664 L .54059 1.37645 L p F P
0 g
s
.102 .159 .637 r
.63486 .98162 m .63797 1.02928 L .65012 1.05091 L p F P
0 g
s
.768 .613 .687 r
.44508 1.29319 m .53128 1.27384 L .52998 1.25632 L p F P
0 g
s
.768 .613 .687 r
.52998 1.25632 m .44855 1.27579 L .44508 1.29319 L p F P
0 g
s
.69 .545 .684 r
.60711 1.22222 m .52998 1.25632 L .53128 1.27384 L p F P
0 g
s
.648 .601 .787 r
.6794 1.37973 m .78325 1.31613 L .77732 1.31126 L p F P
0 g
s
.648 .601 .787 r
.77732 1.31126 m .67575 1.37404 L .6794 1.37973 L p F P
0 g
s
.765 .659 .745 r
.53955 1.36664 m .423 1.38511 L .42021 1.3948 L p F P
0 g
s
.766 .653 .738 r
.423 1.38511 m .53955 1.36664 L .53847 1.35607 L p F P
0 g
s
.699 .592 .733 r
.53955 1.36664 m .65008 1.32779 L .64522 1.31775 L p F P
0 g
s
.699 .592 .733 r
.64522 1.31775 m .53847 1.35607 L .53955 1.36664 L p F P
0 g
s
.445 .37 .679 r
.66214 1.07048 m .63728 1.1204 L .64715 1.14022 L p F P
0 g
s
.435 .356 .668 r
.63728 1.1204 m .66214 1.07048 L .65012 1.05091 L p F P
0 g
s
.69 .545 .684 r
.53128 1.27384 m .61294 1.23907 L .60711 1.22222 L p F P
0 g
s
.599 .482 .691 r
.66645 1.1752 m .60711 1.22222 L .61294 1.23907 L p F P
0 g
s
.595 .473 .683 r
.60711 1.22222 m .66645 1.1752 L .65688 1.15844 L p F P
0 g
s
.768 .619 .694 r
.44168 1.30935 m .53256 1.29012 L .53128 1.27384 L p F P
0 g
s
.768 .619 .694 r
.53128 1.27384 m .44508 1.29319 L .44168 1.30935 L p F P
0 g
s
.691 .551 .691 r
.61294 1.23907 m .53128 1.27384 L .53256 1.29012 L p F P
0 g
s
.766 .653 .738 r
.53847 1.35607 m .42589 1.37467 L .423 1.38511 L p F P
0 g
s
.766 .647 .731 r
.42589 1.37467 m .53847 1.35607 L .53735 1.34471 L p F P
0 g
s
.697 .585 .726 r
.53847 1.35607 m .64522 1.31775 L .64019 1.30692 L p F P
0 g
s
.697 .585 .726 r
.64019 1.30692 m .53735 1.34471 L .53847 1.35607 L p F P
0 g
s
.644 .592 .781 r
.67575 1.37404 m .77732 1.31126 L .7711 1.30591 L p F P
0 g
s
.644 .592 .781 r
.7711 1.30591 m .67191 1.36783 L .67575 1.37404 L p F P
0 g
s
.856 .68 .663 r
.40094 1.22309 m .45936 1.21481 L .46308 1.19095 L p F P
0 g
s
.856 .68 .663 r
.46308 1.19095 m .40919 1.20017 L .40094 1.22309 L p F P
0 g
s
.766 .647 .731 r
.53735 1.34471 m .42888 1.36343 L .42589 1.37467 L p F P
0 g
s
.767 .642 .723 r
.42888 1.36343 m .53735 1.34471 L .5362 1.33249 L p F P
0 g
s
.696 .578 .719 r
.53735 1.34471 m .64019 1.30692 L .63502 1.29527 L p F P
0 g
s
.696 .578 .719 r
.63502 1.29527 m .5362 1.33249 L .53735 1.34471 L p F P
0 g
s
.768 .624 .702 r
.43835 1.32437 m .5338 1.30527 L .53256 1.29012 L p F P
0 g
s
.768 .624 .702 r
.53256 1.29012 m .44168 1.30935 L .43835 1.32437 L p F P
0 g
s
.692 .558 .698 r
.61866 1.25471 m .53256 1.29012 L .5338 1.30527 L p F P
0 g
s
.691 .551 .691 r
.53256 1.29012 m .61866 1.25471 L .61294 1.23907 L p F P
0 g
s
.767 .642 .723 r
.5362 1.33249 m .43195 1.35134 L .42888 1.36343 L p F P
0 g
s
.767 .636 .716 r
.43195 1.35134 m .5362 1.33249 L .53502 1.31937 L p F P
0 g
s
.695 .572 .712 r
.5362 1.33249 m .63502 1.29527 L .62971 1.28273 L p F P
0 g
s
.695 .572 .712 r
.62971 1.28273 m .53502 1.31937 L .5362 1.33249 L p F P
0 g
s
.767 .63 .709 r
.43511 1.33834 m .53502 1.31937 L .5338 1.30527 L p F P
0 g
s
.767 .63 .709 r
.5338 1.30527 m .43835 1.32437 L .43511 1.33834 L p F P
0 g
s
.694 .565 .705 r
.62425 1.26923 m .5338 1.30527 L .53502 1.31937 L p F P
0 g
s
.692 .558 .698 r
.5338 1.30527 m .62425 1.26923 L .61866 1.25471 L p F P
0 g
s
.641 .583 .774 r
.67191 1.36783 m .7711 1.30591 L .76458 1.30004 L p F P
0 g
s
.641 .583 .774 r
.76458 1.30004 m .66789 1.36106 L .67191 1.36783 L p F P
0 g
s
.767 .636 .716 r
.53502 1.31937 m .43511 1.33834 L .43195 1.35134 L p F P
0 g
s
.694 .565 .705 r
.53502 1.31937 m .62971 1.28273 L .62425 1.26923 L p F P
0 g
s
.603 .49 .698 r
.67583 1.19067 m .61294 1.23907 L .61866 1.25471 L p F P
0 g
s
.599 .482 .691 r
.61294 1.23907 m .67583 1.19067 L .66645 1.1752 L p F P
0 g
s
0 0 0 r
.3136 .93938 m .35156 .92599 L .34275 .93416 L p F P
0 g
s
.873 .831 .36 r
.41928 1.08994 m .43115 1.05268 L p .41365 1.09005 L F P
0 g
s
.873 .831 .36 r
.4245 1.05937 m .41153 1.09456 L p .41365 1.09005 L F P
0 g
s
.873 .831 .36 r
.43115 1.05268 m .4245 1.05937 L p .41365 1.09005 L F P
0 g
s
0 0 .234 r
.4245 1.05937 m .43652 1.02127 L p .43615 1.02643 L F P
0 g
s
0 0 .234 r
.43326 1.06646 m .4245 1.05937 L p .43615 1.02643 L F P
0 g
s
0 0 .234 r
.43652 1.02127 m .43749 1.01819 L p .43747 1.0184 L F P
0 g
s
0 0 .234 r
.43747 1.0184 m .43326 1.06646 L .43652 1.02127 L F
.873 .831 .36 r
.41153 1.09456 m .4123 1.09411 L p .41365 1.09005 L F P
0 g
s
.873 .831 .36 r
.4123 1.09411 m .41928 1.08994 L p .41365 1.09005 L F P
0 g
s
.637 .575 .768 r
.66789 1.36106 m .76458 1.30004 L .75778 1.29363 L p F P
0 g
s
.637 .575 .768 r
.75778 1.29363 m .6637 1.3537 L .66789 1.36106 L p F P
0 g
s
.965 .796 .623 r
.39574 1.1516 m .42597 1.14743 L .4345 1.11678 L p F P
0 g
s
.965 .796 .623 r
.4345 1.11678 m .40747 1.1226 L .39574 1.1516 L p F P
0 g
s
.606 .499 .706 r
.68504 1.20497 m .61866 1.25471 L .62425 1.26923 L p F P
0 g
s
.603 .49 .698 r
.61866 1.25471 m .68504 1.20497 L .67583 1.19067 L p F P
0 g
s
.74 .446 .484 r
.5198 1.0332 m .49474 1.01088 L .49771 .95358 L p F P
0 g
s
.74 .446 .484 r
.49771 .95358 m .51708 .97554 L .5198 1.0332 L p F P
0 g
s
.634 .566 .761 r
.6637 1.3537 m .75778 1.29363 L .7507 1.28664 L p F P
0 g
s
.634 .566 .761 r
.7507 1.28664 m .65933 1.34573 L .6637 1.3537 L p F P
0 g
s
.63 .558 .754 r
.65933 1.34573 m .7507 1.28664 L .74335 1.27905 L p F P
0 g
s
.63 .558 .754 r
.74335 1.27905 m .65479 1.33711 L .65933 1.34573 L p F P
0 g
s
.61 .507 .713 r
.69404 1.21818 m .62425 1.26923 L .62971 1.28273 L p F P
0 g
s
.606 .499 .706 r
.62425 1.26923 m .69404 1.21818 L .68504 1.20497 L p F P
0 g
s
.627 .549 .748 r
.65479 1.33711 m .74335 1.27905 L .73574 1.2708 L p F P
0 g
s
.627 .549 .748 r
.73574 1.2708 m .65008 1.32779 L .65479 1.33711 L p F P
0 g
s
.777 .399 .339 r
.56243 .89899 m .50659 .87163 L .5067 .85442 L p F P
0 g
s
.777 .399 .339 r
.5067 .85442 m .55721 .88114 L .56243 .89899 L p F P
0 g
s
.431 .18 .43 r
.46417 1.03925 m .44492 1.02731 L .45658 .97993 L p F P
0 g
s
.431 .18 .43 r
.45658 .97993 m .47228 .99342 L .46417 1.03925 L p F P
0 g
s
.453 .384 .689 r
.674 1.08826 m .64715 1.14022 L .65688 1.15844 L p F P
0 g
s
.445 .37 .679 r
.64715 1.14022 m .674 1.08826 L .66214 1.07048 L p F P
0 g
s
.434 .069 .244 r
.43533 .89626 m .37884 .89486 L .38817 .88179 L p F P
0 g
s
.434 .069 .244 r
.38817 .88179 m .4406 .88453 L .43533 .89626 L p F P
0 g
s
.613 .516 .72 r
.70284 1.23041 m .62971 1.28273 L .63502 1.29527 L p F P
0 g
s
.61 .507 .713 r
.62971 1.28273 m .70284 1.23041 L .69404 1.21818 L p F P
0 g
s
.854 .685 .672 r
.39282 1.24412 m .4557 1.23676 L .45936 1.21481 L p F P
0 g
s
.854 .685 .672 r
.45936 1.21481 m .40094 1.22309 L .39282 1.24412 L p F P
0 g
s
.642 .282 .345 r
.50647 .88655 m .44592 .87113 L .45129 .8558 L p F P
0 g
s
.642 .282 .345 r
.45129 .8558 m .50659 .87163 L .50647 .88655 L p F P
0 g
s
.623 .541 .741 r
.65008 1.32779 m .73574 1.2708 L .72788 1.26186 L p F P
0 g
s
.623 .541 .741 r
.72788 1.26186 m .64522 1.31775 L .65008 1.32779 L p F P
0 g
s
.616 .524 .727 r
.71142 1.24172 m .63502 1.29527 L .64019 1.30692 L p F P
0 g
s
.613 .516 .72 r
.63502 1.29527 m .71142 1.24172 L .70284 1.23041 L p F P
0 g
s
.62 .533 .734 r
.64522 1.31775 m .72788 1.26186 L .71977 1.25219 L p F P
0 g
s
.62 .533 .734 r
.71977 1.25219 m .64019 1.30692 L .64522 1.31775 L p F P
0 g
s
.616 .524 .727 r
.64019 1.30692 m .71977 1.25219 L .71142 1.24172 L p F P
0 g
s
.808 .768 .816 r
.28133 1.4339 m .40561 1.43932 L .40777 1.43336 L p F P
0 g
s
.131 .19 .658 r
.64681 1.00089 m .65012 1.05091 L .66214 1.07048 L p F P
0 g
s
0 0 .399 r
.64681 1.00089 m .65864 1.01814 L p .65704 1.01563 L F P
0 g
s
.102 .159 .637 r
.64681 1.00089 m .63486 .98162 L p .63552 .98462 L F P
0 g
s
0 0 .337 r
.61756 .95382 m .60807 .93672 L p .60975 .93949 L F P
0 g
s
0 0 .337 r
.64681 1.00089 m .61756 .95382 L p .60975 .93949 L F P
0 g
s
.853 .398 .098 r
.61756 .95382 m .62696 .96894 L p .62465 .96682 L F P
0 g
s
.853 .398 .098 r
.56762 .91455 m .61756 .95382 L p .62465 .96682 L F P
0 g
s
0 0 .399 r
.61756 .95382 m .64681 1.00089 L p .65704 1.01563 L F P
0 g
s
.102 .159 .637 r
.65012 1.05091 m .64681 1.00089 L p .63552 .98462 L F P
0 g
s
0 0 .234 r
.43749 1.01819 m .44492 1.02731 L .43326 1.06646 L p F P
0 g
s
.808 .768 .816 r
.40777 1.43336 m .28617 1.4285 L .28133 1.4339 L p F P
0 g
s
.812 .763 .807 r
.28617 1.4285 m .40777 1.43336 L .41005 1.42686 L p F P
0 g
s
.881 .46 .194 r
.55721 .88114 m .59852 .91728 L .60807 .93672 L p F P
0 g
s
.852 .69 .681 r
.38483 1.26348 m .45209 1.25703 L .4557 1.23676 L p F P
0 g
s
.852 .69 .681 r
.4557 1.23676 m .39282 1.24412 L .38483 1.26348 L p F P
0 g
s
0 0 0 r
.30276 .94939 m .34275 .93416 L .33411 .94143 L p F P
0 g
s
.462 .397 .699 r
.6857 1.10446 m .65688 1.15844 L .66645 1.1752 L p F P
0 g
s
.453 .384 .689 r
.65688 1.15844 m .6857 1.10446 L .674 1.08826 L p F P
0 g
s
.812 .763 .807 r
.41005 1.42686 m .29125 1.42258 L .28617 1.4285 L p F P
0 g
s
.816 .758 .799 r
.29125 1.42258 m .41005 1.42686 L .41243 1.41978 L p F P
0 g
s
.816 .758 .799 r
.41243 1.41978 m .29658 1.41612 L .29125 1.42258 L p F P
0 g
s
.819 .753 .79 r
.29658 1.41612 m .41243 1.41978 L .41492 1.4121 L p F P
0 g
s
.881 .46 .194 r
.60807 .93672 m .56243 .89899 L .55721 .88114 L p F P
0 g
s
.85 .695 .69 r
.37697 1.28134 m .44855 1.27579 L .45209 1.25703 L p F P
0 g
s
.85 .695 .69 r
.45209 1.25703 m .38483 1.26348 L .37697 1.28134 L p F P
0 g
s
.961 .801 .639 r
.38412 1.1776 m .41753 1.17507 L .42597 1.14743 L p F P
0 g
s
.961 .801 .639 r
.42597 1.14743 m .39574 1.1516 L .38412 1.1776 L p F P
0 g
s
.819 .753 .79 r
.41492 1.4121 m .30213 1.40908 L .29658 1.41612 L p F P
0 g
s
.822 .748 .782 r
.30213 1.40908 m .41492 1.4121 L .41751 1.40379 L p F P
0 g
s
.848 .7 .699 r
.36926 1.29786 m .44508 1.29319 L .44855 1.27579 L p F P
0 g
s
.848 .7 .699 r
.44855 1.27579 m .37697 1.28134 L .36926 1.29786 L p F P
0 g
s
.822 .748 .782 r
.41751 1.40379 m .30792 1.40143 L .30213 1.40908 L p F P
0 g
s
.825 .743 .774 r
.30792 1.40143 m .41751 1.40379 L .42021 1.3948 L p F P
0 g
s
.471 .41 .708 r
.69721 1.11927 m .66645 1.1752 L .67583 1.19067 L p F P
0 g
s
.462 .397 .699 r
.66645 1.1752 m .69721 1.11927 L .6857 1.10446 L p F P
0 g
s
.855 .538 .45 r
.54274 1.06035 m .5198 1.0332 L .51708 .97554 L p F P
0 g
s
.855 .538 .45 r
.51708 .97554 m .5348 1.00132 L .54274 1.06035 L p F P
0 g
s
.825 .743 .774 r
.42021 1.3948 m .31393 1.39314 L .30792 1.40143 L p F P
0 g
s
.828 .738 .766 r
.31393 1.39314 m .42021 1.3948 L .423 1.38511 L p F P
0 g
s
.845 .705 .708 r
.36171 1.31315 m .44168 1.30935 L .44508 1.29319 L p F P
0 g
s
.845 .705 .708 r
.44508 1.29319 m .36926 1.29786 L .36171 1.31315 L p F P
0 g
s
0 0 0 r
.29213 .95845 m .33411 .94143 L .32564 .94786 L p F P
0 g
s
.828 .738 .766 r
.423 1.38511 m .32015 1.38416 L .31393 1.39314 L p F P
0 g
s
.831 .734 .758 r
.32015 1.38416 m .423 1.38511 L .42589 1.37467 L p F P
0 g
s
.843 .71 .716 r
.35432 1.32733 m .43835 1.32437 L .44168 1.30935 L p F P
0 g
s
.843 .71 .716 r
.44168 1.30935 m .36171 1.31315 L .35432 1.32733 L p F P
0 g
s
.831 .734 .758 r
.42589 1.37467 m .32659 1.37446 L .32015 1.38416 L p F P
0 g
s
.833 .729 .749 r
.32659 1.37446 m .42589 1.37467 L .42888 1.36343 L p F P
0 g
s
.158 .22 .677 r
.65864 1.01814 m .66214 1.07048 L .674 1.08826 L p F P
0 g
s
.131 .19 .658 r
.66214 1.07048 m .65864 1.01814 L .64681 1.00089 L p F P
0 g
s
.841 .715 .725 r
.34711 1.34048 m .43511 1.33834 L .43835 1.32437 L p F P
0 g
s
.841 .715 .725 r
.43835 1.32437 m .35432 1.32733 L .34711 1.34048 L p F P
0 g
s
.833 .729 .749 r
.42888 1.36343 m .33324 1.36398 L .32659 1.37446 L p F P
0 g
s
.836 .724 .741 r
.33324 1.36398 m .42888 1.36343 L .43195 1.35134 L p F P
0 g
s
.416 .037 .208 r
.43011 .90655 m .36961 .90646 L .37884 .89486 L p F P
0 g
s
.416 .037 .208 r
.37884 .89486 m .43533 .89626 L .43011 .90655 L p F P
0 g
s
.838 .719 .733 r
.34008 1.35267 m .43195 1.35134 L .43511 1.33834 L p F P
0 g
s
.838 .719 .733 r
.43511 1.33834 m .34711 1.34048 L .34008 1.35267 L p F P
0 g
s
.836 .724 .741 r
.43195 1.35134 m .34008 1.35267 L .33324 1.36398 L p F P
0 g
s
.479 .423 .717 r
.70853 1.13283 m .67583 1.19067 L .68504 1.20497 L p F P
0 g
s
.471 .41 .708 r
.67583 1.19067 m .70853 1.13283 L .69721 1.11927 L p F P
0 g
s
.893 .859 .41 r
.39862 1.12517 m .40747 1.1226 L .41928 1.08994 L p F P
0 g
s
0 0 .189 r
.41153 1.09456 m .4245 1.05937 L p .42422 1.06316 L F P
0 g
s
.893 .859 .41 r
.41928 1.08994 m .41153 1.09456 L p .40036 1.1222 L F P
0 g
s
0 0 .189 r
.4216 1.09962 m .41153 1.09456 L p .42422 1.06316 L F P
0 g
s
.893 .859 .41 r
.41153 1.09456 m .39862 1.12517 L p .40036 1.1222 L F P
0 g
s
.956 .806 .654 r
.37261 1.20108 m .40919 1.20017 L .41753 1.17507 L p F P
0 g
s
.956 .806 .654 r
.41753 1.17507 m .38412 1.1776 L .37261 1.20108 L p F P
0 g
s
.487 .436 .725 r
.71963 1.14526 m .68504 1.20497 L .69404 1.21818 L p F P
0 g
s
.479 .423 .717 r
.68504 1.20497 m .71963 1.14526 L .70853 1.13283 L p F P
0 g
s
.634 .261 .319 r
.50635 .89953 m .4406 .88453 L .44592 .87113 L p F P
0 g
s
.634 .261 .319 r
.44592 .87113 m .50647 .88655 L .50635 .89953 L p F P
0 g
s
0 0 0 r
.28173 .96666 m .32564 .94786 L .31736 .95355 L p F P
0 g
s
.772 .379 .31 r
.56762 .91455 m .50647 .88655 L .50659 .87163 L p F P
0 g
s
.772 .379 .31 r
.50659 .87163 m .56243 .89899 L .56762 .91455 L p F P
0 g
s
.96 .626 .314 r
.55922 1.09142 m .54274 1.06035 L .5348 1.00132 L p F P
0 g
s
.96 .626 .314 r
.5348 1.00132 m .54756 1.0302 L .55922 1.09142 L p F P
0 g
s
.495 .448 .734 r
.7305 1.15667 m .69404 1.21818 L .70284 1.23041 L p F P
0 g
s
.487 .436 .725 r
.69404 1.21818 m .7305 1.15667 L .71963 1.14526 L p F P
0 g
s
.869 .431 .148 r
.56243 .89899 m .60807 .93672 L .61756 .95382 L p F P
0 g
s
.185 .248 .695 r
.67033 1.03364 m .674 1.08826 L .6857 1.10446 L p F P
0 g
s
.158 .22 .677 r
.674 1.08826 m .67033 1.03364 L .65864 1.01814 L p F P
0 g
s
.612 .332 .468 r
.49176 1.05598 m .46417 1.03925 L .47228 .99342 L p F P
0 g
s
.612 .332 .468 r
.47228 .99342 m .49474 1.01088 L .49176 1.05598 L p F P
0 g
s
.503 .46 .741 r
.74114 1.16713 m .70284 1.23041 L .71142 1.24172 L p F P
0 g
s
.495 .448 .734 r
.70284 1.23041 m .74114 1.16713 L .7305 1.15667 L p F P
0 g
s
.951 .81 .668 r
.36124 1.2224 m .40094 1.22309 L .40919 1.20017 L p F P
0 g
s
.951 .81 .668 r
.40919 1.20017 m .37261 1.20108 L .36124 1.2224 L p F P
0 g
s
0 0 0 r
.27156 .97407 m .31736 .95355 L .30927 .95857 L p F P
0 g
s
.511 .471 .749 r
.75152 1.17673 m .71142 1.24172 L .71977 1.25219 L p F P
0 g
s
.503 .46 .741 r
.71142 1.24172 m .75152 1.17673 L .74114 1.16713 L p F P
0 g
s
.398 .003 .169 r
.42497 .91558 m .36052 .91679 L .36961 .90646 L p F P
0 g
s
.398 .003 .169 r
.36961 .90646 m .43011 .90655 L .42497 .91558 L p F P
0 g
s
.869 .431 .148 r
.61756 .95382 m .56762 .91455 L .56243 .89899 L p F P
0 g
s
.518 .483 .757 r
.76163 1.18553 m .71977 1.25219 L .72788 1.26186 L p F P
0 g
s
.511 .471 .749 r
.71977 1.25219 m .76163 1.18553 L .75152 1.17673 L p F P
0 g
s
0 0 .189 r
.4245 1.05937 m .43326 1.06646 L .4216 1.09962 L p F P
0 g
s
.526 .494 .764 r
.77146 1.1936 m .72788 1.26186 L .73574 1.2708 L p F P
0 g
s
.518 .483 .757 r
.72788 1.26186 m .77146 1.1936 L .76163 1.18553 L p F P
0 g
s
.533 .505 .771 r
.78101 1.20098 m .73574 1.2708 L .74335 1.27905 L p F P
0 g
s
.526 .494 .764 r
.73574 1.2708 m .78101 1.20098 L .77146 1.1936 L p F P
0 g
s
.21 .274 .711 r
.68187 1.04762 m .6857 1.10446 L .69721 1.11927 L p F P
0 g
s
.185 .248 .695 r
.6857 1.10446 m .68187 1.04762 L .67033 1.03364 L p F P
0 g
s
.908 .882 .456 r
.38578 1.15213 m .39574 1.1516 L .40747 1.1226 L p F P
0 g
s
0 0 .142 r
.39862 1.12517 m .41153 1.09456 L p .41142 1.09708 L F P
0 g
s
.908 .882 .456 r
.40747 1.1226 m .39862 1.12517 L p .38725 1.15013 L F P
0 g
s
0 0 .142 r
.40998 1.12819 m .39862 1.12517 L p .41142 1.09708 L F P
0 g
s
.908 .882 .456 r
.39862 1.12517 m .38578 1.15213 L p .38725 1.15013 L F P
0 g
s
.54 .516 .777 r
.79025 1.20772 m .74335 1.27905 L .7507 1.28664 L p F P
0 g
s
.533 .505 .771 r
.74335 1.27905 m .79025 1.20772 L .78101 1.20098 L p F P
0 g
s
.946 .814 .682 r
.35002 1.24185 m .39282 1.24412 L .40094 1.22309 L p F P
0 g
s
.946 .814 .682 r
.40094 1.22309 m .36124 1.2224 L .35002 1.24185 L p F P
0 g
s
.548 .527 .784 r
.79918 1.21387 m .7507 1.28664 L .75778 1.29363 L p F P
0 g
s
.54 .516 .777 r
.7507 1.28664 m .79918 1.21387 L .79025 1.20772 L p F P
0 g
s
.576 .569 .808 r
.78325 1.31613 m .83882 1.23696 L .83159 1.23325 L p F P
0 g
s
.576 .569 .808 r
.83159 1.23325 m .77732 1.31126 L .78325 1.31613 L p F P
0 g
s
.555 .538 .79 r
.80779 1.21946 m .75778 1.29363 L .76458 1.30004 L p F P
0 g
s
.548 .527 .784 r
.75778 1.29363 m .80779 1.21946 L .79918 1.21387 L p F P
0 g
s
.569 .559 .803 r
.77732 1.31126 m .83159 1.23325 L .82401 1.22912 L p F P
0 g
s
.569 .559 .803 r
.82401 1.22912 m .7711 1.30591 L .77732 1.31126 L p F P
0 g
s
.562 .548 .797 r
.81607 1.22453 m .76458 1.30004 L .7711 1.30591 L p F P
0 g
s
.555 .538 .79 r
.76458 1.30004 m .81607 1.22453 L .80779 1.21946 L p F P
0 g
s
.562 .548 .797 r
.7711 1.30591 m .82401 1.22912 L .81607 1.22453 L p F P
0 g
s
.417 .156 .408 r
.45605 1.07683 m .43326 1.06646 L .44492 1.02731 L p F P
0 g
s
.417 .156 .408 r
.44492 1.02731 m .46417 1.03925 L .45605 1.07683 L p F P
0 g
s
.626 .238 .29 r
.50624 .91085 m .43533 .89626 L .4406 .88453 L p F P
0 g
s
.626 .238 .29 r
.4406 .88453 m .50635 .89953 L .50624 .91085 L p F P
0 g
s
0 0 .399 r
.62696 .96894 m .61756 .95382 L p .61908 .9562 L F P
0 g
s
0 0 .399 r
.65864 1.01814 m .62696 .96894 L p .61908 .9562 L F P
0 g
s
0 0 .459 r
.62696 .96894 m .65864 1.01814 L .67033 1.03364 L p F P
0 g
s
0 0 .459 r
.63627 .98235 m .62696 .96894 L p .62836 .97102 L F P
0 g
s
0 0 .459 r
.67033 1.03364 m .63627 .98235 L p .62836 .97102 L F P
0 g
s
0 0 .515 r
.63627 .98235 m .67033 1.03364 L .68187 1.04762 L p F P
0 g
s
0 0 .515 r
.64546 .99428 m .63627 .98235 L p .63756 .9842 L F P
0 g
s
0 0 .515 r
.68187 1.04762 m .64546 .99428 L p .63756 .9842 L F P
0 g
s
0 0 .567 r
.65453 1.0049 m .64546 .99428 L p .64666 .99594 L F P
0 g
s
0 0 .567 r
.69324 1.06025 m .65453 1.0049 L p .64666 .99594 L F P
0 g
s
0 .05 .615 r
.66346 1.01435 m .65453 1.0049 L p .65565 1.0064 L F P
0 g
s
0 .05 .615 r
.70443 1.07167 m .66346 1.01435 L p .65565 1.0064 L F P
0 g
s
0 .103 .658 r
.67224 1.02276 m .66346 1.01435 L p .66452 1.01572 L F P
0 g
s
0 .103 .658 r
.71542 1.08202 m .67224 1.02276 L p .66452 1.01572 L F P
0 g
s
0 .154 .696 r
.68086 1.03024 m .67224 1.02276 L p .67324 1.02403 L F P
0 g
s
0 .154 .696 r
.7262 1.09138 m .68086 1.03024 L p .67324 1.02403 L F P
0 g
s
0 .202 .73 r
.68931 1.03688 m .68086 1.03024 L p .68181 1.03142 L F P
0 g
s
0 .202 .73 r
.73675 1.09986 m .68931 1.03688 L p .68181 1.03142 L F P
0 g
s
0 .246 .76 r
.69757 1.04274 m .68931 1.03688 L p .69021 1.03797 L F P
0 g
s
0 .246 .76 r
.74705 1.10753 m .69757 1.04274 L p .69021 1.03797 L F P
0 g
s
0 .287 .786 r
.70564 1.04792 m .69757 1.04274 L p .69843 1.04378 L F P
0 g
s
0 .287 .786 r
.75711 1.11446 m .70564 1.04792 L p .69843 1.04378 L F P
0 g
s
0 .135 .587 r
.70564 1.04792 m .7135 1.05245 L p .71223 1.0517 L F P
0 g
s
.473 0 0 r
.69757 1.04274 m .70564 1.04792 L p .70432 1.04711 L F P
0 g
s
0 0 .567 r
.64546 .99428 m .68187 1.04762 L .69324 1.06025 L p F P
0 g
s
0 .05 .615 r
.65453 1.0049 m .69324 1.06025 L .70443 1.07167 L p F P
0 g
s
0 .103 .658 r
.66346 1.01435 m .70443 1.07167 L .71542 1.08202 L p F P
0 g
s
0 .154 .696 r
.67224 1.02276 m .71542 1.08202 L .7262 1.09138 L p F P
0 g
s
0 .202 .73 r
.68086 1.03024 m .7262 1.09138 L .73675 1.09986 L p F P
0 g
s
0 .246 .76 r
.68931 1.03688 m .73675 1.09986 L .74705 1.10753 L p F P
0 g
s
0 .287 .786 r
.69757 1.04274 m .74705 1.10753 L .75711 1.11446 L p F P
0 g
s
.473 0 0 r
.64773 1.01273 m .69757 1.04274 L .70432 1.04711 L F
.61155 .99075 m .63806 1.00677 L p .64773 1.01273 L F P
0 g
s
.473 0 0 r
.63806 1.00677 m .69757 1.04274 L p .64773 1.01273 L F P
0 g
s
.473 0 0 r
.70564 1.04792 m .616 .99466 L .61155 .99075 L p F P
0 g
s
0 .135 .587 r
.65263 1.01637 m .70564 1.04792 L .71223 1.0517 L F
.616 .99466 m .64258 1.01045 L p .65263 1.01637 L F P
0 g
s
0 .135 .587 r
.64258 1.01045 m .70564 1.04792 L p .65263 1.01637 L F P
0 g
s
0 .135 .587 r
.62033 .99797 m .616 .99466 L p .67915 1.03209 L F P
0 g
s
0 .135 .587 r
.7135 1.05245 m .68783 1.03744 L p .67915 1.03209 L F P
0 g
s
0 .135 .587 r
.68783 1.03744 m .62033 .99797 L p .67915 1.03209 L F P
0 g
s
0 .199 .653 r
.7135 1.05245 m .72114 1.05641 L p .65739 1.01945 L F P
0 g
s
0 .199 .653 r
.62033 .99797 m .64698 1.01355 L p .65739 1.01945 L F P
0 g
s
0 .199 .653 r
.64698 1.01355 m .7135 1.05245 L p .65739 1.01945 L F P
0 g
s
0 .199 .653 r
.62455 1.00073 m .62033 .99797 L p .62159 .99869 L F P
0 g
s
0 .199 .653 r
.62159 .99869 m .68648 1.03631 L .62455 1.00073 L F
.54 0 0 r
.607 .98618 m .68931 1.03688 L p .69619 1.04188 L F P
0 g
s
.54 0 0 r
.69757 1.04274 m .61155 .99075 L .607 .98618 L p F P
0 g
s
0 .259 .713 r
.62864 1.003 m .62455 1.00073 L p .62576 1.00142 L F P
0 g
s
0 .259 .713 r
.62576 1.00142 m .69361 1.03998 L .62864 1.003 L F
.602 .058 0 r
.68086 1.03024 m .68931 1.03688 L p .68786 1.03594 L F P
0 g
s
.602 .058 0 r
.60236 .98087 m .68086 1.03024 L p .68786 1.03594 L F P
0 g
s
.602 .058 0 r
.68931 1.03688 m .607 .98618 L .60236 .98087 L p F P
0 g
s
0 .316 .765 r
.6326 1.00482 m .62864 1.003 L p .62982 1.00366 L F P
0 g
s
0 .316 .765 r
.73573 1.06278 m .6326 1.00482 L p .62982 1.00366 L F P
0 g
s
0 .369 .81 r
.63643 1.00624 m .6326 1.00482 L p .63375 1.00545 L F P
0 g
s
0 .369 .81 r
.74266 1.06529 m .63643 1.00624 L p .63375 1.00545 L F P
0 g
s
.656 .119 0 r
.67224 1.02276 m .68086 1.03024 L p .67933 1.02922 L F P
0 g
s
.656 .119 0 r
.59762 .97475 m .67224 1.02276 L p .67933 1.02922 L F P
0 g
s
.656 .119 0 r
.68086 1.03024 m .60236 .98087 L .59762 .97475 L p F P
0 g
s
0 .416 .847 r
.64012 1.0073 m .63643 1.00624 L p .63755 1.00685 L F P
0 g
s
0 .416 .847 r
.74933 1.06739 m .64012 1.0073 L p .63755 1.00685 L F P
0 g
s
0 .41 .726 r
.64012 1.0073 m .64366 1.00803 L p .64243 1.0077 L F P
0 g
s
.243 0 0 r
.63643 1.00624 m .64012 1.0073 L p .63885 1.00695 L F P
0 g
s
.54 0 0 r
.68931 1.03688 m .69757 1.04274 L p .69619 1.04188 L F P
0 g
s
0 .259 .713 r
.70288 1.04523 m .62864 1.003 L p .69361 1.03998 L F P
0 g
s
0 .199 .653 r
.69547 1.04161 m .62455 1.00073 L p .68648 1.03631 L F P
0 g
s
0 .199 .653 r
.72114 1.05641 m .69547 1.04161 L p .68648 1.03631 L F P
0 g
s
0 .259 .713 r
.62455 1.00073 m .72114 1.05641 L .72855 1.05984 L p F P
0 g
s
0 .316 .765 r
.62864 1.003 m .72855 1.05984 L .73573 1.06278 L p F P
0 g
s
0 .369 .81 r
.6326 1.00482 m .73573 1.06278 L .74266 1.06529 L p F P
0 g
s
0 .416 .847 r
.63643 1.00624 m .74266 1.06529 L .74933 1.06739 L p F P
0 g
s
.243 0 0 r
.50494 .97054 m .63643 1.00624 L p .63885 1.00695 L F P
0 g
s
.317 0 0 r
.6326 1.00482 m .63643 1.00624 L p .63513 1.00588 L F P
0 g
s
.317 0 0 r
.50502 .96953 m .6326 1.00482 L p .63513 1.00588 L F P
0 g
s
.317 0 0 r
.63643 1.00624 m .50494 .97054 L .50502 .96953 L p F P
0 g
s
.243 0 0 r
.64012 1.0073 m .50486 .9712 L .50494 .97054 L p F P
0 g
s
0 .41 .726 r
.50486 .9712 m .64012 1.0073 L p .64243 1.0077 L F P
0 g
s
.386 0 0 r
.62864 1.003 m .6326 1.00482 L p .63125 1.00443 L F P
0 g
s
.386 0 0 r
.50511 .96814 m .62864 1.003 L p .63125 1.00443 L F P
0 g
s
.449 0 0 r
.62455 1.00073 m .62864 1.003 L p .62724 1.00258 L F P
0 g
s
.449 0 0 r
.5052 .96631 m .62455 1.00073 L p .62724 1.00258 L F P
0 g
s
0 .259 .713 r
.72855 1.05984 m .70288 1.04523 L p .69361 1.03998 L F P
0 g
s
.704 .176 0 r
.66346 1.01435 m .67224 1.02276 L p .67062 1.02164 L F P
0 g
s
.704 .176 0 r
.59279 .96772 m .66346 1.01435 L p .67062 1.02164 L F P
0 g
s
.746 .229 0 r
.65453 1.0049 m .66346 1.01435 L p .66174 1.0131 L F P
0 g
s
.746 .229 0 r
.58789 .95969 m .65453 1.0049 L p .66174 1.0131 L F P
0 g
s
.781 .278 0 r
.64546 .99428 m .65453 1.0049 L p .65269 1.0035 L F P
0 g
s
.781 .278 0 r
.58291 .95051 m .64546 .99428 L p .65269 1.0035 L F P
0 g
s
.81 .322 0 r
.63627 .98235 m .64546 .99428 L p .64349 .99269 L F P
0 g
s
.81 .322 0 r
.57787 .94006 m .63627 .98235 L p .64349 .99269 L F P
0 g
s
.834 .362 .043 r
.62696 .96894 m .63627 .98235 L p .63414 .98053 L F P
0 g
s
.834 .362 .043 r
.57277 .92814 m .62696 .96894 L p .63414 .98053 L F P
0 g
s
.941 .818 .695 r
.33896 1.25965 m .38483 1.26348 L .39282 1.24412 L p F P
0 g
s
.941 .818 .695 r
.39282 1.24412 m .35002 1.24185 L .33896 1.25965 L p F P
0 g
s
.234 .299 .726 r
.69324 1.06025 m .69721 1.11927 L .70853 1.13283 L p F P
0 g
s
.21 .274 .711 r
.69721 1.11927 m .69324 1.06025 L .68187 1.04762 L p F P
0 g
s
.765 .358 .279 r
.57277 .92814 m .50635 .89953 L .50647 .88655 L p F P
0 g
s
.765 .358 .279 r
.50647 .88655 m .56762 .91455 L .57277 .92814 L p F P
0 g
s
.377 0 .127 r
.4199 .92348 m .35156 .92599 L .36052 .91679 L p F P
0 g
s
.377 0 .127 r
.36052 .91679 m .42497 .91558 L .4199 .92348 L p F P
0 g
s
.936 .822 .707 r
.32809 1.27599 m .37697 1.28134 L .38483 1.26348 L p F P
0 g
s
.936 .822 .707 r
.38483 1.26348 m .33896 1.25965 L .32809 1.27599 L p F P
0 g
s
.736 .434 .468 r
.52255 1.07859 m .49176 1.05598 L .49474 1.01088 L p F P
0 g
s
.736 .434 .468 r
.49474 1.01088 m .5198 1.0332 L .52255 1.07859 L p F P
0 g
s
.919 .902 .498 r
.37304 1.17611 m .38412 1.1776 L .39574 1.1516 L p F P
0 g
s
0 0 .093 r
.38578 1.15213 m .39862 1.12517 L p .39861 1.12687 L F P
0 g
s
.919 .902 .498 r
.39574 1.1516 m .38578 1.15213 L p .3743 1.17475 L F P
0 g
s
0 0 .093 r
.3984 1.15312 m .38578 1.15213 L p .39861 1.12687 L F P
0 g
s
.919 .902 .498 r
.38578 1.15213 m .37304 1.17611 L p .3743 1.17475 L F P
0 g
s
.256 .323 .74 r
.70443 1.07167 m .70853 1.13283 L .71963 1.14526 L p F P
0 g
s
.234 .299 .726 r
.70853 1.13283 m .70443 1.07167 L .69324 1.06025 L p F P
0 g
s
.931 .825 .719 r
.31741 1.29102 m .36926 1.29786 L .37697 1.28134 L p F P
0 g
s
.931 .825 .719 r
.37697 1.28134 m .32809 1.27599 L .31741 1.29102 L p F P
0 g
s
.853 .398 .098 r
.62696 .96894 m .57277 .92814 L .56762 .91455 L p F P
0 g
s
.427 .85 .7 r
.23352 .99712 m .28566 .97042 L p .27911 .97293 L F P
0 g
s
.456 .874 .749 r
.22473 1.00147 m .27849 .97326 L p .27217 .97534 L F P
0 g
s
.456 .874 .749 r
.27849 .97326 m .27911 .97293 L .27217 .97534 L p F P
0 g
s
.481 .894 .792 r
.21625 1.00534 m .27156 .97567 L p .26548 .97736 L F P
0 g
s
.481 .894 .792 r
.27156 .97567 m .27217 .97534 L .26548 .97736 L p F P
0 g
s
.481 .894 .792 r
.26548 .97736 m .20809 1.00877 L .21625 1.00534 L p F P
0 g
s
.502 .908 .83 r
.20809 1.00877 m .26489 .97769 L p .25906 .97902 L F P
0 g
s
.502 .908 .83 r
.26489 .97769 m .26548 .97736 L .25906 .97902 L p F P
0 g
s
0 0 0 r
.36733 .96288 m .26003 .97888 L p .26548 .97736 L F P
0 g
s
.502 .908 .83 r
.25906 .97902 m .20026 1.0118 L .20809 1.00877 L p F P
0 g
s
.521 .918 .864 r
.20026 1.0118 m .25848 .97935 L p .25291 .98037 L F P
0 g
s
.521 .918 .864 r
.25848 .97935 m .25906 .97902 L .25291 .98037 L p F P
0 g
s
.078 0 0 r
.5047 .97164 m .36503 .96336 L p .36733 .96288 L F P
0 g
s
0 0 0 r
.31588 .9702 m .36989 .96235 L .36733 .96288 L F
.15 .691 .722 r
.36437 .96332 m .36383 .96329 L p .29329 .97388 L F P
0 g
s
.15 .691 .722 r
.28482 .97518 m .36437 .96332 L .29329 .97388 L F
.25906 .97902 m .28482 .97518 L .29329 .97388 L F
.25906 .97902 m .33157 .96814 L .25385 .98022 L F
0 0 0 r
.31644 .97008 m .371 .96219 L .31588 .9702 L F
.15 .691 .722 r
.36383 .96329 m .33871 .96716 L p .33157 .96814 L F P
0 g
s
.15 .691 .722 r
.33871 .96716 m .25385 .98022 L p .33157 .96814 L F P
0 g
s
0 0 0 r
.29119 .97366 m .371 .96219 L p .29926 .97256 L F P
0 g
s
0 0 0 r
.371 .96219 m .26648 .97722 L p .27217 .97534 L F P
0 g
s
0 0 0 r
.27217 .97534 m .37481 .96117 L .371 .96219 L p F P
0 g
s
.2 .729 .782 r
.25291 .98037 m .36383 .96329 L .36048 .96345 L p F P
0 g
s
.2 .729 .782 r
.31139 .97117 m .32792 .96857 L .31044 .97137 L F
.36048 .96345 m .33538 .96742 L p .32792 .96857 L F P
0 g
s
.2 .729 .782 r
.33538 .96742 m .31044 .97137 L p .32792 .96857 L F P
0 g
s
.078 0 0 r
.36503 .96336 m .36383 .96329 L .36733 .96288 L p F P
0 g
s
0 0 0 r
.371 .96219 m .36733 .96288 L p .36989 .96235 L F P
0 g
s
0 0 0 r
.29926 .97256 m .31644 .97008 L .31588 .9702 L F
.521 .918 .864 r
.25291 .98037 m .19277 1.01444 L .20026 1.0118 L p F P
0 g
s
.536 .924 .892 r
.19277 1.01444 m .25291 .98037 L .24704 .98142 L p F P
0 g
s
.2 .729 .782 r
.25291 .98037 m .31139 .97117 L .24796 .98127 L F
.31044 .97137 m .24796 .98127 L p .31139 .97117 L F P
0 g
s
0 0 0 r
.26548 .97736 m .29119 .97366 L p .29926 .97256 L F P
0 g
s
.2 .729 .782 r
.24796 .98127 m .24704 .98142 L .25291 .98037 L p F P
0 g
s
.15 .691 .722 r
.25385 .98022 m .25291 .98037 L .25906 .97902 L p F P
0 g
s
0 0 0 r
.26003 .97888 m .25906 .97902 L .26548 .97736 L p F P
0 g
s
0 0 0 r
.26648 .97722 m .26548 .97736 L .27217 .97534 L p F P
0 g
s
.007 0 0 r
.37481 .96117 m .2732 .9752 L p .27911 .97293 L F P
0 g
s
.007 0 0 r
.2732 .9752 m .27217 .97534 L .27911 .97293 L p F P
0 g
s
.427 .85 .7 r
.28566 .97042 m .2863 .97009 L .27911 .97293 L p F P
0 g
s
.925 .829 .731 r
.30694 1.30485 m .36171 1.31315 L .36926 1.29786 L p F P
0 g
s
.925 .829 .731 r
.36926 1.29786 m .31741 1.29102 L .30694 1.30485 L p F P
0 g
s
0 0 .142 r
.41153 1.09456 m .4216 1.09962 L .40998 1.12819 L p F P
0 g
s
.353 0 .08 r
.41491 .93039 m .34275 .93416 L .35156 .92599 L p F P
0 g
s
.353 0 .08 r
.35156 .92599 m .4199 .92348 L .41491 .93039 L p F P
0 g
s
.278 .345 .753 r
.71542 1.08202 m .71963 1.14526 L .7305 1.15667 L p F P
0 g
s
.256 .323 .74 r
.71963 1.14526 m .71542 1.08202 L .70443 1.07167 L p F P
0 g
s
.616 .214 .259 r
.50613 .92072 m .43011 .90655 L .43533 .89626 L p F P
0 g
s
.616 .214 .259 r
.43533 .89626 m .50624 .91085 L .50613 .92072 L p F P
0 g
s
.92 .832 .742 r
.29669 1.31759 m .35432 1.32733 L .36171 1.31315 L p F P
0 g
s
.92 .832 .742 r
.36171 1.31315 m .30694 1.30485 L .29669 1.31759 L p F P
0 g
s
.914 .835 .753 r
.28667 1.32934 m .34711 1.34048 L .35432 1.32733 L p F P
0 g
s
.914 .835 .753 r
.35432 1.32733 m .29669 1.31759 L .28667 1.32934 L p F P
0 g
s
.841 .86 .858 r
.1949 1.40998 m .28133 1.4339 L .28617 1.4285 L p F P
0 g
s
.908 .838 .763 r
.27689 1.34017 m .34008 1.35267 L .34711 1.34048 L p F P
0 g
s
.908 .838 .763 r
.34711 1.34048 m .28667 1.32934 L .27689 1.34017 L p F P
0 g
s
.841 .86 .858 r
.28617 1.4285 m .20167 1.40553 L .1949 1.40998 L p F P
0 g
s
.849 .859 .849 r
.20167 1.40553 m .28617 1.4285 L .29125 1.42258 L p F P
0 g
s
.927 .919 .537 r
.36041 1.19758 m .37261 1.20108 L .38412 1.1776 L p F P
0 g
s
.927 .919 .537 r
.38412 1.1776 m .37304 1.17611 L .36041 1.19758 L p F P
0 g
s
.853 .525 .431 r
.55071 1.10705 m .52255 1.07859 L .5198 1.0332 L p F P
0 g
s
.853 .525 .431 r
.5198 1.0332 m .54274 1.06035 L .55071 1.10705 L p F P
0 g
s
.849 .859 .849 r
.29125 1.42258 m .20879 1.4006 L .20167 1.40553 L p F P
0 g
s
.856 .857 .84 r
.20879 1.4006 m .29125 1.42258 L .29658 1.41612 L p F P
0 g
s
.299 .366 .764 r
.7262 1.09138 m .7305 1.15667 L .74114 1.16713 L p F P
0 g
s
.278 .345 .753 r
.7305 1.15667 m .7262 1.09138 L .71542 1.08202 L p F P
0 g
s
.903 .84 .774 r
.26738 1.35016 m .33324 1.36398 L .34008 1.35267 L p F P
0 g
s
.903 .84 .774 r
.34008 1.35267 m .27689 1.34017 L .26738 1.35016 L p F P
0 g
s
.856 .857 .84 r
.29658 1.41612 m .21623 1.39518 L .20879 1.4006 L p F P
0 g
s
.864 .855 .831 r
.21623 1.39518 m .29658 1.41612 L .30213 1.40908 L p F P
0 g
s
.758 .335 .246 r
.57787 .94006 m .50624 .91085 L .50635 .89953 L p F P
0 g
s
.758 .335 .246 r
.50635 .89953 m .57277 .92814 L .57787 .94006 L p F P
0 g
s
.897 .843 .784 r
.25813 1.35935 m .32659 1.37446 L .33324 1.36398 L p F P
0 g
s
.897 .843 .784 r
.33324 1.36398 m .26738 1.35016 L .25813 1.35935 L p F P
0 g
s
.864 .855 .831 r
.30213 1.40908 m .224 1.38922 L .21623 1.39518 L p F P
0 g
s
.871 .853 .822 r
.224 1.38922 m .30213 1.40908 L .30792 1.40143 L p F P
0 g
s
.89 .846 .794 r
.24915 1.36781 m .32015 1.38416 L .32659 1.37446 L p F P
0 g
s
.89 .846 .794 r
.32659 1.37446 m .25813 1.35935 L .24915 1.36781 L p F P
0 g
s
.871 .853 .822 r
.30792 1.40143 m .23208 1.3827 L .224 1.38922 L p F P
0 g
s
.877 .85 .813 r
.23208 1.3827 m .30792 1.40143 L .31393 1.39314 L p F P
0 g
s
.877 .85 .813 r
.31393 1.39314 m .24047 1.37557 L .23208 1.3827 L p F P
0 g
s
.884 .848 .803 r
.24047 1.37557 m .31393 1.39314 L .32015 1.38416 L p F P
0 g
s
.884 .848 .803 r
.32015 1.38416 m .24915 1.36781 L .24047 1.37557 L p F P
0 g
s
.606 .317 .452 r
.48877 1.0928 m .45605 1.07683 L .46417 1.03925 L p F P
0 g
s
.606 .317 .452 r
.46417 1.03925 m .49176 1.05598 L .48877 1.0928 L p F P
0 g
s
.404 .133 .385 r
.44793 1.1084 m .4216 1.09962 L .43326 1.06646 L p F P
0 g
s
.404 .133 .385 r
.43326 1.06646 m .45605 1.07683 L .44793 1.1084 L p F P
0 g
s
.319 .387 .775 r
.73675 1.09986 m .74114 1.16713 L .75152 1.17673 L p F P
0 g
s
.299 .366 .764 r
.74114 1.16713 m .73675 1.09986 L .7262 1.09138 L p F P
0 g
s
.328 0 .03 r
.41001 .9364 m .33411 .94143 L .34275 .93416 L p F P
0 g
s
.328 0 .03 r
.34275 .93416 m .41491 .93039 L .41001 .9364 L p F P
0 g
s
.834 .362 .043 r
.63627 .98235 m .57787 .94006 L .57277 .92814 L p F P
0 g
s
.953 .604 .28 r
.57093 1.14022 m .55071 1.10705 L .54274 1.06035 L p F P
0 g
s
.953 .604 .28 r
.54274 1.06035 m .55922 1.09142 L .57093 1.14022 L p F P
0 g
s
.338 .406 .785 r
.74705 1.10753 m .75152 1.17673 L .76163 1.18553 L p F P
0 g
s
.319 .387 .775 r
.75152 1.17673 m .74705 1.10753 L .73675 1.09986 L p F P
0 g
s
.604 .187 .224 r
.50602 .92933 m .42497 .91558 L .43011 .90655 L p F P
0 g
s
.604 .187 .224 r
.43011 .90655 m .50613 .92072 L .50602 .92933 L p F P
0 g
s
.932 .933 .573 r
.34791 1.21692 m .36124 1.2224 L .37261 1.20108 L p F P
0 g
s
.932 .933 .573 r
.37261 1.20108 m .36041 1.19758 L .34791 1.21692 L p F P
0 g
s
.536 .924 .892 r
.24704 .98142 m .18563 1.01674 L .19277 1.01444 L p F P
0 g
s
0 0 .093 r
.39862 1.12517 m .40998 1.12819 L .3984 1.15312 L p F P
0 g
s
.356 .425 .794 r
.75711 1.11446 m .76163 1.18553 L .77146 1.1936 L p F P
0 g
s
.338 .406 .785 r
.76163 1.18553 m .75711 1.11446 L .74705 1.10753 L p F P
0 g
s
.299 0 0 r
.40521 .9416 m .32564 .94786 L .33411 .94143 L p F P
0 g
s
.299 0 0 r
.33411 .94143 m .41001 .9364 L .40521 .9416 L p F P
0 g
s
.374 .442 .803 r
.7669 1.1207 m .77146 1.1936 L .78101 1.20098 L p F P
0 g
s
.356 .425 .794 r
.77146 1.1936 m .7669 1.1207 L .75711 1.11446 L p F P
0 g
s
.748 .309 .209 r
.58291 .95051 m .50613 .92072 L .50624 .91085 L p F P
0 g
s
.748 .309 .209 r
.50624 .91085 m .57787 .94006 L .58291 .95051 L p F P
0 g
s
.935 .944 .606 r
.33557 1.23442 m .35002 1.24185 L .36124 1.2224 L p F P
0 g
s
.935 .944 .606 r
.36124 1.2224 m .34791 1.21692 L .33557 1.23442 L p F P
0 g
s
.391 .459 .811 r
.77641 1.12631 m .78101 1.20098 L .79025 1.20772 L p F P
0 g
s
.374 .442 .803 r
.78101 1.20098 m .77641 1.12631 L .7669 1.1207 L p F P
0 g
s
.81 .322 0 r
.64546 .99428 m .58291 .95051 L .57787 .94006 L p F P
0 g
s
.408 .475 .818 r
.78562 1.13134 m .79025 1.20772 L .79918 1.21387 L p F P
0 g
s
.391 .459 .811 r
.79025 1.20772 m .78562 1.13134 L .77641 1.12631 L p F P
0 g
s
.733 .421 .452 r
.5253 1.11565 m .48877 1.0928 L .49176 1.05598 L p F P
0 g
s
.733 .421 .452 r
.49176 1.05598 m .52255 1.07859 L .5253 1.11565 L p F P
0 g
s
.591 .158 .187 r
.50591 .93681 m .4199 .92348 L .42497 .91558 L p F P
0 g
s
.591 .158 .187 r
.42497 .91558 m .50602 .92933 L .50591 .93681 L p F P
0 g
s
.267 0 0 r
.40051 .94608 m .31736 .95355 L .32564 .94786 L p F P
0 g
s
.267 0 0 r
.32564 .94786 m .40521 .9416 L .40051 .94608 L p F P
0 g
s
.423 .491 .825 r
.79454 1.13584 m .79918 1.21387 L .80779 1.21946 L p F P
0 g
s
.408 .475 .818 r
.79918 1.21387 m .79454 1.13584 L .78562 1.13134 L p F P
0 g
s
.936 .954 .636 r
.32339 1.2503 m .33896 1.25965 L .35002 1.24185 L p F P
0 g
s
.936 .954 .636 r
.35002 1.24185 m .33557 1.23442 L .32339 1.2503 L p F P
0 g
s
.439 .506 .831 r
.80314 1.13984 m .80779 1.21946 L .81607 1.22453 L p F P
0 g
s
.423 .491 .825 r
.80779 1.21946 m .80314 1.13984 L .79454 1.13584 L p F P
0 g
s
0 0 .041 r
.3869 1.17508 m .37304 1.17611 L .38578 1.15213 L p F P
0 g
s
0 0 .041 r
.38578 1.15213 m .3984 1.15312 L .3869 1.17508 L p F P
0 g
s
.39 .108 .36 r
.43982 1.13538 m .40998 1.12819 L .4216 1.09962 L p F P
0 g
s
.39 .108 .36 r
.4216 1.09962 m .44793 1.1084 L .43982 1.13538 L p F P
0 g
s
.454 .52 .837 r
.81142 1.14338 m .81607 1.22453 L .82401 1.22912 L p F P
0 g
s
.439 .506 .831 r
.81607 1.22453 m .81142 1.14338 L .80314 1.13984 L p F P
0 g
s
.468 .534 .842 r
.81936 1.14651 m .82401 1.22912 L .83159 1.23325 L p F P
0 g
s
.454 .52 .837 r
.82401 1.22912 m .81936 1.14651 L .81142 1.14338 L p F P
0 g
s
.736 .28 .168 r
.58789 .95969 m .50602 .92933 L .50613 .92072 L p F P
0 g
s
.736 .28 .168 r
.50613 .92072 m .58291 .95051 L .58789 .95969 L p F P
0 g
s
.482 .547 .847 r
.82696 1.14924 m .83159 1.23325 L .83882 1.23696 L p F P
0 g
s
.468 .534 .842 r
.83159 1.23325 m .82696 1.14924 L .81936 1.14651 L p F P
0 g
s
.6 .301 .434 r
.48578 1.12358 m .44793 1.1084 L .45605 1.07683 L p F P
0 g
s
.6 .301 .434 r
.45605 1.07683 m .48877 1.0928 L .48578 1.12358 L p F P
0 g
s
.232 0 0 r
.39592 .9499 m .30927 .95857 L .31736 .95355 L p F P
0 g
s
.232 0 0 r
.31736 .95355 m .40051 .94608 L .39592 .9499 L p F P
0 g
s
.935 .961 .664 r
.3114 1.26475 m .32809 1.27599 L .33896 1.25965 L p F P
0 g
s
.935 .961 .664 r
.33896 1.25965 m .32339 1.2503 L .3114 1.26475 L p F P
0 g
s
.482 .547 .847 r
.83882 1.23696 m .8342 1.15161 L .82696 1.14924 L p F P
0 g
s
.576 .126 .146 r
.5058 .94329 m .41491 .93039 L .4199 .92348 L p F P
0 g
s
.576 .126 .146 r
.4199 .92348 m .50591 .93681 L .5058 .94329 L p F P
0 g
s
.85 .511 .41 r
.55871 1.14535 m .5253 1.11565 L .52255 1.07859 L p F P
0 g
s
.85 .511 .41 r
.52255 1.07859 m .55071 1.10705 L .55871 1.14535 L p F P
0 g
s
.781 .278 0 r
.65453 1.0049 m .58789 .95969 L .58291 .95051 L p F P
0 g
s
.932 .967 .689 r
.29961 1.27792 m .31741 1.29102 L .32809 1.27599 L p F P
0 g
s
.932 .967 .689 r
.32809 1.27599 m .31184 1.26505 L p .29961 1.27792 L F P
0 g
s
.929 .972 .713 r
.31741 1.29102 m .30002 1.27822 L p .28803 1.28993 L F P
0 g
s
.929 .972 .713 r
.30002 1.27822 m .29961 1.27792 L .28803 1.28993 L p F P
0 g
s
.929 .972 .713 r
.28803 1.28993 m .30694 1.30485 L .31741 1.29102 L p F P
0 g
s
.924 .975 .735 r
.30694 1.30485 m .28843 1.29024 L p .27669 1.30089 L F P
0 g
s
.924 .975 .735 r
.28843 1.29024 m .28803 1.28993 L .27669 1.30089 L p F P
0 g
s
.924 .975 .735 r
.27669 1.30089 m .29669 1.31759 L .30694 1.30485 L p F P
0 g
s
.918 .978 .755 r
.29669 1.31759 m .27707 1.30121 L p .26559 1.31089 L F P
0 g
s
.918 .978 .755 r
.27707 1.30121 m .27669 1.30089 L .26559 1.31089 L p F P
0 g
s
0 0 0 r
.28936 1.29275 m .26599 1.31059 L p .27669 1.30089 L F P
0 g
s
.918 .978 .755 r
.26559 1.31089 m .28667 1.32934 L .29669 1.31759 L p F P
0 g
s
.912 .979 .774 r
.28667 1.32934 m .26559 1.31089 L .25475 1.32002 L p F P
0 g
s
.492 .821 .52 r
.26559 1.31089 m .28895 1.29306 L p .27947 1.30013 L F P
0 g
s
.492 .821 .52 r
.28895 1.29306 m .28936 1.29275 L .27947 1.30013 L p F P
0 g
s
.492 .821 .52 r
.27947 1.30013 m .25513 1.31971 L p .26559 1.31089 L F P
0 g
s
.492 .821 .52 r
.25513 1.31971 m .25475 1.32002 L .26559 1.31089 L p F P
0 g
s
0 0 0 r
.26599 1.31059 m .26559 1.31089 L .27669 1.30089 L p F P
0 g
s
.516 .85 .571 r
.25475 1.32002 m .27907 1.30045 L p .26981 1.30676 L F P
0 g
s
.516 .85 .571 r
.27907 1.30045 m .27947 1.30013 L .26981 1.30676 L p F P
0 g
s
.516 .85 .571 r
.26981 1.30676 m .24418 1.32834 L .25475 1.32002 L p F P
0 g
s
.538 .876 .618 r
.24418 1.32834 m .26943 1.30708 L p .26041 1.31268 L F P
0 g
s
.538 .876 .618 r
.26943 1.30708 m .26981 1.30676 L .26041 1.31268 L p F P
0 g
s
.538 .876 .618 r
.26041 1.31268 m .23391 1.33592 L .24418 1.32834 L p F P
0 g
s
.556 .898 .663 r
.23391 1.33592 m .26004 1.313 L p .25126 1.31795 L F P
0 g
s
.556 .898 .663 r
.26004 1.313 m .26041 1.31268 L .25126 1.31795 L p F P
0 g
s
.912 .979 .774 r
.25475 1.32002 m .27689 1.34017 L .28667 1.32934 L p F P
0 g
s
.904 .979 .792 r
.27689 1.34017 m .25475 1.32002 L .24418 1.32834 L p F P
0 g
s
.556 .898 .663 r
.25126 1.31795 m .22392 1.34281 L .23391 1.33592 L p F P
0 g
s
.573 .916 .705 r
.22392 1.34281 m .25091 1.31827 L p .2424 1.32264 L F P
0 g
s
.573 .916 .705 r
.25091 1.31827 m .25126 1.31795 L .2424 1.32264 L p F P
0 g
s
.573 .916 .705 r
.2424 1.32264 m .21425 1.34906 L .22392 1.34281 L p F P
0 g
s
.586 .931 .743 r
.21425 1.34906 m .24206 1.32296 L p .23382 1.32677 L F P
0 g
s
.586 .931 .743 r
.24206 1.32296 m .2424 1.32264 L .23382 1.32677 L p F P
0 g
s
.586 .931 .743 r
.23382 1.32677 m .20491 1.35471 L .21425 1.34906 L p F P
0 g
s
.597 .942 .777 r
.20491 1.35471 m .23349 1.32709 L p .22554 1.33041 L F P
0 g
s
.597 .942 .777 r
.23349 1.32709 m .23382 1.32677 L .22554 1.33041 L p F P
0 g
s
.597 .942 .777 r
.22554 1.33041 m .19589 1.35982 L .20491 1.35471 L p F P
0 g
s
.606 .95 .809 r
.19589 1.35982 m .22522 1.33073 L p .21757 1.33358 L F P
0 g
s
.606 .95 .809 r
.22522 1.33073 m .22554 1.33041 L .21757 1.33358 L p F P
0 g
s
.606 .95 .809 r
.21757 1.33358 m .18722 1.36441 L .19589 1.35982 L p F P
0 g
s
.612 .955 .837 r
.18722 1.36441 m .21726 1.3339 L p .20992 1.33633 L F P
0 g
s
.612 .955 .837 r
.21726 1.3339 m .21757 1.33358 L .20992 1.33633 L p F P
0 g
s
.904 .979 .792 r
.24418 1.32834 m .26738 1.35016 L .27689 1.34017 L p F P
0 g
s
.896 .979 .808 r
.26738 1.35016 m .24418 1.32834 L .23391 1.33592 L p F P
0 g
s
.612 .955 .837 r
.20992 1.33633 m .1789 1.36852 L .18722 1.36441 L p F P
0 g
s
.616 .958 .862 r
.1789 1.36852 m .20962 1.33665 L p .2026 1.3387 L F P
0 g
s
.616 .958 .862 r
.20962 1.33665 m .20992 1.33633 L .2026 1.3387 L p F P
0 g
s
0 0 0 r
.29378 1.31625 m .20337 1.33851 L p .20992 1.33633 L F P
0 g
s
0 0 0 r
.29898 1.31497 m .21072 1.33614 L p .21757 1.33358 L F P
0 g
s
.616 .958 .862 r
.2026 1.3387 m .17095 1.37218 L .1789 1.36852 L p F P
0 g
s
.618 .958 .884 r
.17095 1.37218 m .2026 1.3387 L .19561 1.3407 L p F P
0 g
s
0 0 0 r
.30442 1.31335 m .29898 1.31497 L p .2578 1.32439 L F P
0 g
s
0 0 0 r
.21757 1.33358 m .25992 1.32372 L p .2578 1.32439 L F P
0 g
s
0 0 0 r
.25992 1.32372 m .30442 1.31335 L p .2578 1.32439 L F P
0 g
s
0 0 0 r
.23727 1.32978 m .29809 1.31519 L .29378 1.31625 L F
.23106 1.33126 m .29898 1.31497 L p .23727 1.32978 L F P
0 g
s
.239 .749 .727 r
.2026 1.3387 m .29378 1.31625 L .28882 1.31721 L p F P
0 g
s
.065 0 0 r
.28994 1.31719 m .28994 1.31719 L p .29378 1.31625 L F P
0 g
s
.065 0 0 r
.42685 1.31497 m .28994 1.31719 L p .29378 1.31625 L F P
0 g
s
.065 0 0 r
.28994 1.31719 m .28882 1.31721 L .29378 1.31625 L p F P
0 g
s
0 0 0 r
.29898 1.31497 m .29378 1.31625 L p .29809 1.31519 L F P
0 g
s
0 0 0 r
.20992 1.33633 m .23106 1.33126 L p .23727 1.32978 L F P
0 g
s
.239 .749 .727 r
.24941 1.32703 m .26267 1.32373 L .24763 1.32759 L F
.28882 1.31721 m .26814 1.32242 L p .26267 1.32373 L F P
0 g
s
.239 .749 .727 r
.26814 1.32242 m .24763 1.32759 L p .26267 1.32373 L F P
0 g
s
.239 .749 .727 r
.2026 1.3387 m .24941 1.32703 L .19637 1.34051 L F
.24763 1.32759 m .19637 1.34051 L p .24941 1.32703 L F P
0 g
s
.239 .749 .727 r
.19637 1.34051 m .19561 1.3407 L .2026 1.3387 L p F P
0 g
s
0 0 0 r
.20337 1.33851 m .2026 1.3387 L .20992 1.33633 L p F P
0 g
s
0 0 0 r
.21072 1.33614 m .20992 1.33633 L .21757 1.33358 L p F P
0 g
s
0 0 0 r
.30442 1.31335 m .21839 1.33339 L p .22554 1.33041 L F P
0 g
s
0 0 0 r
.21839 1.33339 m .21757 1.33358 L .22554 1.33041 L p F P
0 g
s
0 0 0 r
.31007 1.31135 m .22639 1.33022 L p .23382 1.32677 L F P
0 g
s
0 0 0 r
.22639 1.33022 m .22554 1.33041 L .23382 1.32677 L p F P
0 g
s
0 0 0 r
.31594 1.30893 m .2347 1.32658 L p .2424 1.32264 L F P
0 g
s
0 0 0 r
.2347 1.32658 m .23382 1.32677 L .2424 1.32264 L p F P
0 g
s
0 0 0 r
.32202 1.30605 m .2433 1.32245 L p .25126 1.31795 L F P
0 g
s
0 0 0 r
.2433 1.32245 m .2424 1.32264 L .25126 1.31795 L p F P
0 g
s
.034 0 0 r
.32829 1.30266 m .2522 1.31777 L p .26041 1.31268 L F P
0 g
s
.034 0 0 r
.2522 1.31777 m .25126 1.31795 L .26041 1.31268 L p F P
0 g
s
.07 0 0 r
.33476 1.29871 m .26138 1.31249 L p .26981 1.30676 L F P
0 g
s
.07 0 0 r
.26138 1.31249 m .26041 1.31268 L .26981 1.30676 L p F P
0 g
s
.104 0 0 r
.3414 1.29415 m .27083 1.30658 L p .27947 1.30013 L F P
0 g
s
.104 0 0 r
.27083 1.30658 m .26981 1.30676 L .27947 1.30013 L p F P
0 g
s
0 0 0 r
.29949 1.28452 m .27711 1.30058 L p .28803 1.28993 L F P
0 g
s
0 0 0 r
.27711 1.30058 m .27669 1.30089 L .28803 1.28993 L p F P
0 g
s
0 0 0 r
.30982 1.27537 m .28848 1.28963 L p .29961 1.27792 L F P
0 g
s
0 0 0 r
.28848 1.28963 m .28803 1.28993 L .29961 1.27792 L p F P
0 g
s
.932 .967 .689 r
.31184 1.26505 m .3114 1.26475 L .29961 1.27792 L p F P
0 g
s
.194 0 0 r
.39144 .95313 m .30139 .96296 L .30927 .95857 L p F P
0 g
s
.194 0 0 r
.30927 .95857 m .39592 .9499 L .39144 .95313 L p F P
0 g
s
0 0 0 r
.37549 1.19455 m .36041 1.19758 L .37304 1.17611 L p F P
0 g
s
0 0 0 r
.37304 1.17611 m .3869 1.17508 L .37549 1.19455 L p F P
0 g
s
.944 .581 .242 r
.58265 1.18056 m .55871 1.14535 L .55071 1.10705 L p F P
0 g
s
.944 .581 .242 r
.55071 1.10705 m .57093 1.14022 L .58265 1.18056 L p F P
0 g
s
.723 .249 .123 r
.59279 .96772 m .50591 .93681 L .50602 .92933 L p F P
0 g
s
.723 .249 .123 r
.50602 .92933 m .58789 .95969 L .59279 .96772 L p F P
0 g
s
.558 .09 .101 r
.5057 .94889 m .41001 .9364 L .41491 .93039 L p F P
0 g
s
.558 .09 .101 r
.41491 .93039 m .5058 .94329 L .5057 .94889 L p F P
0 g
s
.151 0 0 r
.38709 .95583 m .29374 .96678 L .30139 .96296 L p F P
0 g
s
.151 0 0 r
.30139 .96296 m .39144 .95313 L .38709 .95583 L p F P
0 g
s
.746 .229 0 r
.66346 1.01435 m .59279 .96772 L .58789 .95969 L p F P
0 g
s
.376 .082 .334 r
.43174 1.15871 m .3984 1.15312 L .40998 1.12819 L p F P
0 g
s
.376 .082 .334 r
.40998 1.12819 m .43982 1.13538 L .43174 1.15871 L p F P
0 g
s
.73 .408 .435 r
.52806 1.14665 m .48578 1.12358 L .48877 1.0928 L p F P
0 g
s
.73 .408 .435 r
.48877 1.0928 m .5253 1.11565 L .52806 1.14665 L p F P
0 g
s
0 0 0 r
.36419 1.21191 m .34791 1.21692 L .36041 1.19758 L p F P
0 g
s
0 0 0 r
.36041 1.19758 m .37549 1.19455 L .36419 1.21191 L p F P
0 g
s
.106 0 0 r
.29374 .96678 m .38709 .95583 L .38286 .95804 L p F P
0 g
s
.047 .325 .808 r
.70564 1.04792 m .75711 1.11446 L .7669 1.1207 L p F P
0 g
s
.593 .285 .416 r
.48279 1.14975 m .43982 1.13538 L .44793 1.1084 L p F P
0 g
s
.593 .285 .416 r
.44793 1.1084 m .48578 1.12358 L .48279 1.14975 L p F P
0 g
s
.537 .052 .051 r
.50559 .95368 m .40521 .9416 L .41001 .9364 L p F P
0 g
s
.537 .052 .051 r
.41001 .9364 m .5057 .94889 L .50559 .95368 L p F P
0 g
s
.704 .176 0 r
.67224 1.02276 m .59762 .97475 L .59279 .96772 L p F P
0 g
s
.706 .214 .074 r
.59762 .97475 m .5058 .94329 L .50591 .93681 L p F P
0 g
s
.706 .214 .074 r
.50591 .93681 m .59279 .96772 L .59762 .97475 L p F P
0 g
s
.896 .979 .808 r
.23391 1.33592 m .25813 1.35935 L .26738 1.35016 L p F P
0 g
s
.057 0 0 r
.37876 .9598 m .27911 .97293 L .2863 .97009 L p F P
0 g
s
.057 0 0 r
.2863 .97009 m .38286 .95804 L .37876 .9598 L p F P
0 g
s
.093 .36 .827 r
.7135 1.05245 m .7669 1.1207 L .77641 1.12631 L p F P
0 g
s
.047 .325 .808 r
.7669 1.1207 m .7135 1.05245 L .70564 1.04792 L p F P
0 g
s
.888 .978 .823 r
.22392 1.34281 m .24915 1.36781 L .25813 1.35935 L p F P
0 g
s
.888 .978 .823 r
.25813 1.35935 m .23391 1.33592 L .22392 1.34281 L p F P
0 g
s
.879 .977 .838 r
.21425 1.34906 m .24047 1.37557 L .24915 1.36781 L p F P
0 g
s
.879 .977 .838 r
.24915 1.36781 m .22392 1.34281 L .21425 1.34906 L p F P
0 g
s
0 0 0 r
.35301 1.22745 m .33557 1.23442 L .34791 1.21692 L p F P
0 g
s
0 0 0 r
.34791 1.21692 m .36419 1.21191 L .35301 1.22745 L p F P
0 g
s
.848 .496 .388 r
.56671 1.17755 m .52806 1.14665 L .5253 1.11565 L p F P
0 g
s
.848 .496 .388 r
.5253 1.11565 m .55871 1.14535 L .56671 1.17755 L p F P
0 g
s
.869 .975 .851 r
.20491 1.35471 m .23208 1.3827 L .24047 1.37557 L p F P
0 g
s
.869 .975 .851 r
.24047 1.37557 m .21425 1.34906 L .20491 1.35471 L p F P
0 g
s
.007 0 0 r
.27911 .97293 m .37876 .9598 L .37481 .96117 L p F P
0 g
s
.36 .055 .305 r
.4237 1.17906 m .3869 1.17508 L .3984 1.15312 L p F P
0 g
s
.36 .055 .305 r
.3984 1.15312 m .43174 1.15871 L .4237 1.17906 L p F P
0 g
s
.137 .392 .844 r
.72114 1.05641 m .77641 1.12631 L .78562 1.13134 L p F P
0 g
s
.093 .36 .827 r
.77641 1.12631 m .72114 1.05641 L .7135 1.05245 L p F P
0 g
s
.859 .972 .863 r
.19589 1.35982 m .224 1.38922 L .23208 1.3827 L p F P
0 g
s
.859 .972 .863 r
.23208 1.3827 m .20491 1.35471 L .19589 1.35982 L p F P
0 g
s
.513 .01 0 r
.50549 .95775 m .40051 .94608 L .40521 .9416 L p F P
0 g
s
.513 .01 0 r
.40521 .9416 m .50559 .95368 L .50549 .95775 L p F P
0 g
s
.849 .969 .875 r
.18722 1.36441 m .21623 1.39518 L .224 1.38922 L p F P
0 g
s
.849 .969 .875 r
.224 1.38922 m .19589 1.35982 L .18722 1.36441 L p F P
0 g
s
.933 .555 .2 r
.55871 1.14535 m .58265 1.18056 L .59437 1.21474 L p F P
0 g
s
.838 .966 .886 r
.1789 1.36852 m .20879 1.4006 L .21623 1.39518 L p F P
0 g
s
.838 .966 .886 r
.21623 1.39518 m .18722 1.36441 L .1789 1.36852 L p F P
0 g
s
.685 .175 .02 r
.60236 .98087 m .5057 .94889 L .5058 .94329 L p F P
0 g
s
.685 .175 .02 r
.5058 .94329 m .59762 .97475 L .60236 .98087 L p F P
0 g
s
.827 .962 .897 r
.17095 1.37218 m .20167 1.40553 L .20879 1.4006 L p F P
0 g
s
.827 .962 .897 r
.20879 1.4006 m .1789 1.36852 L .17095 1.37218 L p F P
0 g
s
.933 .555 .2 r
.59437 1.21474 m .56671 1.17755 L .55871 1.14535 L p F P
0 g
s
.815 .958 .906 r
.16337 1.37544 m .1949 1.40998 L .20167 1.40553 L p F P
0 g
s
.815 .958 .906 r
.20167 1.40553 m .17095 1.37218 L .16337 1.37544 L p F P
0 g
s
.178 .422 .857 r
.72855 1.05984 m .78562 1.13134 L .79454 1.13584 L p F P
0 g
s
.137 .392 .844 r
.78562 1.13134 m .72855 1.05984 L .72114 1.05641 L p F P
0 g
s
0 0 0 r
.34196 1.24139 m .32339 1.2503 L .33557 1.23442 L p F P
0 g
s
0 0 0 r
.33557 1.23442 m .35301 1.22745 L .34196 1.24139 L p F P
0 g
s
.216 .449 .869 r
.73573 1.06278 m .79454 1.13584 L .80314 1.13984 L p F P
0 g
s
.178 .422 .857 r
.79454 1.13584 m .73573 1.06278 L .72855 1.05984 L p F P
0 g
s
.486 0 0 r
.50539 .96117 m .39592 .9499 L .40051 .94608 L p F P
0 g
s
.486 0 0 r
.40051 .94608 m .50549 .95775 L .50539 .96117 L p F P
0 g
s
.726 .394 .417 r
.53081 1.173 m .48279 1.14975 L .48578 1.12358 L p F P
0 g
s
.726 .394 .417 r
.48578 1.12358 m .52806 1.14665 L .53081 1.173 L p F P
0 g
s
.587 .268 .396 r
.47981 1.17227 m .43174 1.15871 L .43982 1.13538 L p F P
0 g
s
.587 .268 .396 r
.43982 1.13538 m .48279 1.14975 L .47981 1.17227 L p F P
0 g
s
.252 .473 .878 r
.74266 1.06529 m .80314 1.13984 L .81142 1.14338 L p F P
0 g
s
.216 .449 .869 r
.80314 1.13984 m .74266 1.06529 L .73573 1.06278 L p F P
0 g
s
.661 .132 0 r
.607 .98618 m .50559 .95368 L .5057 .94889 L p F P
0 g
s
.661 .132 0 r
.5057 .94889 m .60236 .98087 L .607 .98618 L p F P
0 g
s
.344 .027 .275 r
.41572 1.19694 m .37549 1.19455 L .3869 1.17508 L p F P
0 g
s
.344 .027 .275 r
.3869 1.17508 m .4237 1.17906 L .41572 1.19694 L p F P
0 g
s
.285 .496 .886 r
.74933 1.06739 m .81142 1.14338 L .81936 1.14651 L p F P
0 g
s
.252 .473 .878 r
.81142 1.14338 m .74933 1.06739 L .74266 1.06529 L p F P
0 g
s
0 0 0 r
.33107 1.25393 m .3114 1.26475 L .32339 1.2503 L p F P
0 g
s
0 0 0 r
.32339 1.2503 m .34196 1.24139 L .33107 1.25393 L p F P
0 g
s
.317 .516 .893 r
.75574 1.06912 m .81936 1.14651 L .82696 1.14924 L p F P
0 g
s
.285 .496 .886 r
.81936 1.14651 m .75574 1.06912 L .74933 1.06739 L p F P
0 g
s
.453 0 0 r
.5053 .96401 m .39144 .95313 L .39592 .9499 L p F P
0 g
s
.453 0 0 r
.39592 .9499 m .50539 .96117 L .5053 .96401 L p F P
0 g
s
.346 .535 .898 r
.76188 1.07053 m .82696 1.14924 L .8342 1.15161 L p F P
0 g
s
.317 .516 .893 r
.82696 1.14924 m .76188 1.07053 L .75574 1.06912 L p F P
0 g
s
.844 .48 .364 r
.57469 1.20507 m .53081 1.173 L .52806 1.14665 L p F P
0 g
s
.844 .48 .364 r
.52806 1.14665 m .56671 1.17755 L .57469 1.20507 L p F P
0 g
s
0 0 0 r
.32036 1.26521 m .29961 1.27792 L .3114 1.26475 L p F P
0 g
s
0 0 0 r
.3114 1.26475 m .33107 1.25393 L .32036 1.26521 L p F P
0 g
s
.631 .085 0 r
.61155 .99075 m .50549 .95775 L .50559 .95368 L p F P
0 g
s
.631 .085 0 r
.50559 .95368 m .607 .98618 L .61155 .99075 L p F P
0 g
s
.346 .535 .898 r
.8342 1.15161 m .76773 1.07164 L .76188 1.07053 L p F P
0 g
s
.92 .527 .156 r
.56671 1.17755 m .59437 1.21474 L .60606 1.24421 L p F P
0 g
s
.416 0 0 r
.5052 .96631 m .38709 .95583 L .39144 .95313 L p F P
0 g
s
.416 0 0 r
.39144 .95313 m .5053 .96401 L .5052 .96631 L p F P
0 g
s
.92 .527 .156 r
.60606 1.24421 m .57469 1.20507 L .56671 1.17755 L p F P
0 g
s
.327 0 .243 r
.4078 1.21272 m .36419 1.21191 L .37549 1.19455 L p F P
0 g
s
.327 0 .243 r
.37549 1.19455 m .41572 1.19694 L .4078 1.21272 L p F P
0 g
s
.579 .25 .375 r
.47685 1.1918 m .4237 1.17906 L .43174 1.15871 L p F P
0 g
s
.579 .25 .375 r
.43174 1.15871 m .47981 1.17227 L .47685 1.1918 L p F P
0 g
s
0 0 0 r
.29961 1.27792 m .32036 1.26521 L .30982 1.27537 L p F P
0 g
s
.722 .379 .397 r
.53356 1.19568 m .47981 1.17227 L .48279 1.14975 L p F P
0 g
s
.722 .379 .397 r
.48279 1.14975 m .53081 1.173 L .53356 1.19568 L p F P
0 g
s
.596 .033 0 r
.616 .99466 m .50539 .96117 L .50549 .95775 L p F P
0 g
s
.596 .033 0 r
.50549 .95775 m .61155 .99075 L .616 .99466 L p F P
0 g
s
.373 0 0 r
.50511 .96814 m .38286 .95804 L .38709 .95583 L p F P
0 g
s
.373 0 0 r
.38709 .95583 m .5052 .96631 L .50511 .96814 L p F P
0 g
s
0 0 0 r
.28803 1.28993 m .30982 1.27537 L .29949 1.28452 L p F P
0 g
s
.309 0 .209 r
.39996 1.22669 m .35301 1.22745 L .36419 1.21191 L p F P
0 g
s
.309 0 .209 r
.36419 1.21191 m .4078 1.21272 L .39996 1.22669 L p F P
0 g
s
.324 0 0 r
.50502 .96953 m .37876 .9598 L .38286 .95804 L p F P
0 g
s
.324 0 0 r
.38286 .95804 m .50511 .96814 L .50502 .96953 L p F P
0 g
s
.554 0 0 r
.62033 .99797 m .5053 .96401 L .50539 .96117 L p F P
0 g
s
.554 0 0 r
.50539 .96117 m .616 .99466 L .62033 .99797 L p F P
0 g
s
0 0 0 r
.27669 1.30089 m .29949 1.28452 L .28936 1.29275 L p F P
0 g
s
.903 .496 .108 r
.57469 1.20507 m .60606 1.24421 L .6177 1.26992 L p F P
0 g
s
.841 .464 .338 r
.58265 1.22888 m .53356 1.19568 L .53081 1.173 L p F P
0 g
s
.841 .464 .338 r
.53081 1.173 m .57469 1.20507 L .58265 1.22888 L p F P
0 g
s
.572 .231 .353 r
.4739 1.20885 m .41572 1.19694 L .4237 1.17906 L p F P
0 g
s
.572 .231 .353 r
.4237 1.17906 m .47685 1.1918 L .4739 1.20885 L p F P
0 g
s
.903 .496 .108 r
.6177 1.26992 m .58265 1.22888 L .57469 1.20507 L p F P
0 g
s
.27 0 0 r
.50494 .97054 m .37481 .96117 L .37876 .9598 L p F P
0 g
s
.27 0 0 r
.37876 .9598 m .50502 .96953 L .50494 .97054 L p F P
0 g
s
.505 0 0 r
.62455 1.00073 m .5052 .96631 L .5053 .96401 L p F P
0 g
s
.505 0 0 r
.5053 .96401 m .62033 .99797 L .62455 1.00073 L p F P
0 g
s
.289 0 .172 r
.39222 1.23907 m .34196 1.24139 L .35301 1.22745 L p F P
0 g
s
.289 0 .172 r
.35301 1.22745 m .39996 1.22669 L .39222 1.23907 L p F P
0 g
s
.718 .363 .376 r
.53629 1.21536 m .47685 1.1918 L .47981 1.17227 L p F P
0 g
s
.718 .363 .376 r
.47981 1.17227 m .53356 1.19568 L .53629 1.21536 L p F P
0 g
s
.21 0 0 r
.50486 .9712 m .371 .96219 L .37481 .96117 L p F P
0 g
s
.21 0 0 r
.37481 .96117 m .50494 .97054 L .50486 .9712 L p F P
0 g
s
.449 0 0 r
.62864 1.003 m .50511 .96814 L .5052 .96631 L p F P
0 g
s
.036 .458 .877 r
.64012 1.0073 m .74933 1.06739 L .75574 1.06912 L p F P
0 g
s
.883 .462 .057 r
.58265 1.22888 m .6177 1.26992 L .62927 1.29258 L p F P
0 g
s
.145 0 0 r
.50478 .97156 m .36733 .96288 L .371 .96219 L p F P
0 g
s
.145 0 0 r
.50486 .9712 m .50478 .97156 L p .43739 .96684 L F P
0 g
s
.078 0 0 r
.50478 .97156 m .5047 .97164 L p .4124 .96575 L F P
0 g
s
.078 0 0 r
.36733 .96288 m .40019 .96495 L p .4124 .96575 L F P
0 g
s
.078 0 0 r
.40019 .96495 m .50478 .97156 L p .4124 .96575 L F P
0 g
s
0 .566 .751 r
.36383 .96329 m .44096 .96786 L p .4406 .96776 L F P
0 g
s
0 .566 .751 r
.44096 .96786 m .5047 .97164 L p .4406 .96776 L F P
0 g
s
0 .566 .751 r
.5047 .97164 m .50463 .97148 L p .4406 .96776 L F P
0 g
s
0 .566 .751 r
.4406 .96776 m .4622 .96901 L .44127 .96795 L F
.50463 .97148 m .47289 .96971 L p .4622 .96901 L F P
0 g
s
0 .566 .751 r
.47289 .96971 m .44127 .96795 L p .4622 .96901 L F P
0 g
s
0 .566 .751 r
.36048 .96345 m .36383 .96329 L p .4406 .96776 L F P
0 g
s
0 .566 .751 r
.44127 .96795 m .36048 .96345 L p .4406 .96776 L F P
0 g
s
.145 0 0 r
.371 .96219 m .43658 .9666 L p .43739 .96684 L F P
0 g
s
.145 0 0 r
.43658 .9666 m .50486 .9712 L p .43739 .96684 L F P
0 g
s
.563 .211 .329 r
.47098 1.22379 m .4078 1.21272 L .41572 1.19694 L p F P
0 g
s
.563 .211 .329 r
.41572 1.19694 m .4739 1.20885 L .47098 1.22379 L p F P
0 g
s
.268 0 .132 r
.38457 1.25007 m .33107 1.25393 L .34196 1.24139 L p F P
0 g
s
.268 0 .132 r
.34196 1.24139 m .39222 1.23907 L .38457 1.25007 L p F P
0 g
s
.098 .494 .901 r
.64366 1.00803 m .75574 1.06912 L .76188 1.07053 L p F P
0 g
s
.036 .458 .877 r
.75574 1.06912 m .64366 1.00803 L .64012 1.0073 L p F P
0 g
s
.836 .445 .31 r
.59057 1.24968 m .53629 1.21536 L .53356 1.19568 L p F P
0 g
s
.836 .445 .31 r
.53356 1.19568 m .58265 1.22888 L .59057 1.24968 L p F P
0 g
s
.386 0 0 r
.6326 1.00482 m .50502 .96953 L .50511 .96814 L p F P
0 g
s
.883 .462 .057 r
.62927 1.29258 m .59057 1.24968 L .58265 1.22888 L p F P
0 g
s
.155 .526 .919 r
.64706 1.00848 m .76188 1.07053 L .76773 1.07164 L p F P
0 g
s
.098 .494 .901 r
.76188 1.07053 m .64706 1.00848 L .64366 1.00803 L p F P
0 g
s
.713 .347 .354 r
.53901 1.23254 m .4739 1.20885 L .47685 1.1918 L p F P
0 g
s
.713 .347 .354 r
.47685 1.1918 m .53629 1.21536 L .53901 1.23254 L p F P
0 g
s
.155 .526 .919 r
.76773 1.07164 m .6503 1.00866 L .64706 1.00848 L p F P
0 g
s
.245 0 .09 r
.37703 1.25984 m .32036 1.26521 L .33107 1.25393 L p F P
0 g
s
.245 0 .09 r
.33107 1.25393 m .38457 1.25007 L .37703 1.25984 L p F P
0 g
s
.86 .425 .003 r
.59057 1.24968 m .62927 1.29258 L .64075 1.31268 L p F P
0 g
s
.554 .189 .303 r
.46808 1.23693 m .39996 1.22669 L .4078 1.21272 L p F P
0 g
s
.554 .189 .303 r
.4078 1.21272 m .47098 1.22379 L .46808 1.23693 L p F P
0 g
s
.618 .958 .884 r
.19561 1.3407 m .16337 1.37544 L .17095 1.37218 L p F P
0 g
s
.221 0 .045 r
.36962 1.2685 m .30982 1.27537 L .32036 1.26521 L p F P
0 g
s
.221 0 .045 r
.32036 1.26521 m .37703 1.25984 L .36962 1.2685 L p F P
0 g
s
.83 .426 .279 r
.59844 1.26795 m .53901 1.23254 L .53629 1.21536 L p F P
0 g
s
.83 .426 .279 r
.53629 1.21536 m .59057 1.24968 L .59844 1.26795 L p F P
0 g
s
.86 .425 .003 r
.64075 1.31268 m .59844 1.26795 L .59057 1.24968 L p F P
0 g
s
0 .41 .726 r
.64366 1.00803 m .50478 .97156 L .50486 .9712 L p F P
0 g
s
.707 .329 .329 r
.54171 1.2476 m .47098 1.22379 L .4739 1.20885 L p F P
0 g
s
.707 .329 .329 r
.4739 1.20885 m .53901 1.23254 L .54171 1.2476 L p F P
0 g
s
.832 .384 0 r
.59844 1.26795 m .64075 1.31268 L .65212 1.3306 L p F P
0 g
s
0 .466 .789 r
.50478 .97156 m .64366 1.00803 L .64706 1.00848 L p F P
0 g
s
.195 0 0 r
.36234 1.27617 m .29949 1.28452 L .30982 1.27537 L p F P
0 g
s
.195 0 0 r
.30982 1.27537 m .36962 1.2685 L .36234 1.27617 L p F P
0 g
s
.544 .166 .275 r
.46522 1.24849 m .39222 1.23907 L .39996 1.22669 L p F P
0 g
s
.544 .166 .275 r
.39996 1.22669 m .46808 1.23693 L .46522 1.24849 L p F P
0 g
s
0 .466 .789 r
.64706 1.00848 m .5047 .97164 L .50478 .97156 L p F P
0 g
s
0 .517 .842 r
.5047 .97164 m .64706 1.00848 L .6503 1.00866 L p F P
0 g
s
0 .517 .842 r
.6503 1.00866 m .50463 .97148 L .5047 .97164 L p F P
0 g
s
.166 0 0 r
.35521 1.28295 m .28936 1.29275 L .29949 1.28452 L p F P
0 g
s
.166 0 0 r
.29949 1.28452 m .36234 1.27617 L .35521 1.28295 L p F P
0 g
s
.832 .384 0 r
.65212 1.3306 m .60624 1.28407 L .59844 1.26795 L p F P
0 g
s
.823 .404 .247 r
.53901 1.23254 m .59844 1.26795 L .60624 1.28407 L p F P
0 g
s
.823 .404 .247 r
.60624 1.28407 m .54171 1.2476 L .53901 1.23254 L p F P
0 g
s
.799 .339 0 r
.60624 1.28407 m .65212 1.3306 L .66337 1.34665 L p F P
0 g
s
.7 .309 .303 r
.54438 1.26085 m .46808 1.23693 L .47098 1.22379 L p F P
0 g
s
.7 .309 .303 r
.47098 1.22379 m .54171 1.2476 L .54438 1.26085 L p F P
0 g
s
.532 .141 .245 r
.46239 1.25866 m .38457 1.25007 L .39222 1.23907 L p F P
0 g
s
.532 .141 .245 r
.39222 1.23907 m .46522 1.24849 L .46239 1.25866 L p F P
0 g
s
.136 0 0 r
.34822 1.28892 m .27947 1.30013 L .28936 1.29275 L p F P
0 g
s
.136 0 0 r
.28936 1.29275 m .35521 1.28295 L .34822 1.28892 L p F P
0 g
s
.104 0 0 r
.27947 1.30013 m .34822 1.28892 L .3414 1.29415 L p F P
0 g
s
.761 .292 0 r
.61397 1.29836 m .66337 1.34665 L .67449 1.36106 L p F P
0 g
s
.799 .339 0 r
.66337 1.34665 m .61397 1.29836 L .60624 1.28407 L p F P
0 g
s
.815 .38 .211 r
.54171 1.2476 m .60624 1.28407 L .61397 1.29836 L p F P
0 g
s
.52 .114 .212 r
.4596 1.26761 m .37703 1.25984 L .38457 1.25007 L p F P
0 g
s
.52 .114 .212 r
.38457 1.25007 m .46239 1.25866 L .4596 1.26761 L p F P
0 g
s
.815 .38 .211 r
.61397 1.29836 m .54438 1.26085 L .54171 1.2476 L p F P
0 g
s
.693 .288 .275 r
.54703 1.2725 m .46522 1.24849 L .46808 1.23693 L p F P
0 g
s
.693 .288 .275 r
.46808 1.23693 m .54438 1.26085 L .54703 1.2725 L p F P
0 g
s
.07 0 0 r
.26981 1.30676 m .3414 1.29415 L .33476 1.29871 L p F P
0 g
s
.718 .24 0 r
.62161 1.31104 m .67449 1.36106 L .68545 1.37401 L p F P
0 g
s
.761 .292 0 r
.67449 1.36106 m .62161 1.31104 L .61397 1.29836 L p F P
0 g
s
.034 0 0 r
.26041 1.31268 m .33476 1.29871 L .32829 1.30266 L p F P
0 g
s
.506 .086 .177 r
.45686 1.27546 m .36962 1.2685 L .37703 1.25984 L p F P
0 g
s
.506 .086 .177 r
.37703 1.25984 m .4596 1.26761 L .45686 1.27546 L p F P
0 g
s
.804 .354 .172 r
.54438 1.26085 m .61397 1.29836 L .62161 1.31104 L p F P
0 g
s
.804 .354 .172 r
.62161 1.31104 m .54703 1.2725 L .54438 1.26085 L p F P
0 g
s
0 0 0 r
.25126 1.31795 m .32829 1.30266 L .32202 1.30605 L p F P
0 g
s
.685 .266 .244 r
.54964 1.28277 m .46239 1.25866 L .46522 1.24849 L p F P
0 g
s
.685 .266 .244 r
.46522 1.24849 m .54703 1.2725 L .54964 1.28277 L p F P
0 g
s
.67 .186 0 r
.62915 1.3223 m .68545 1.37401 L .69624 1.38568 L p F P
0 g
s
.718 .24 0 r
.68545 1.37401 m .62915 1.3223 L .62161 1.31104 L p F P
0 g
s
0 0 0 r
.2424 1.32264 m .32202 1.30605 L .31594 1.30893 L p F P
0 g
s
.49 .055 .139 r
.45416 1.28233 m .36234 1.27617 L .36962 1.2685 L p F P
0 g
s
.49 .055 .139 r
.36962 1.2685 m .45686 1.27546 L .45416 1.28233 L p F P
0 g
s
.792 .326 .13 r
.54703 1.2725 m .62161 1.31104 L .62915 1.3223 L p F P
0 g
s
0 0 0 r
.23382 1.32677 m .31594 1.30893 L .31007 1.31135 L p F P
0 g
s
.617 .129 0 r
.63658 1.33231 m .69624 1.38568 L .70684 1.39619 L p F P
0 g
s
.67 .186 0 r
.69624 1.38568 m .63658 1.33231 L .62915 1.3223 L p F P
0 g
s
.792 .326 .13 r
.62915 1.3223 m .54964 1.28277 L .54703 1.2725 L p F P
0 g
s
.675 .241 .211 r
.55221 1.2918 m .4596 1.26761 L .46239 1.25866 L p F P
0 g
s
.675 .241 .211 r
.46239 1.25866 m .54964 1.28277 L .55221 1.2918 L p F P
0 g
s
0 0 0 r
.22554 1.33041 m .31007 1.31135 L .30442 1.31335 L p F P
0 g
s
.472 .021 .097 r
.45152 1.28831 m .35521 1.28295 L .36234 1.27617 L p F P
0 g
s
.472 .021 .097 r
.36234 1.27617 m .45416 1.28233 L .45152 1.28831 L p F P
0 g
s
.777 .294 .085 r
.54964 1.28277 m .62915 1.3223 L .63658 1.33231 L p F P
0 g
s
.559 .071 0 r
.70684 1.39619 m .71725 1.40566 L p .67442 1.36803 L F P
0 g
s
.559 .071 0 r
.71725 1.40566 m .65107 1.34907 L .64389 1.3412 L p F P
0 g
s
0 0 .502 r
.71725 1.40566 m .72744 1.41418 L p .682 1.37544 L F P
0 g
s
0 0 .502 r
.65107 1.34907 m .67331 1.36809 L p .682 1.37544 L F P
0 g
s
0 0 .502 r
.67331 1.36809 m .71725 1.40566 L p .682 1.37544 L F P
0 g
s
0 0 .502 r
.72744 1.41418 m .70587 1.39609 L p .69835 1.38938 L F P
0 g
s
0 0 .502 r
.70587 1.39609 m .6581 1.35604 L p .69835 1.38938 L F P
0 g
s
0 .048 .562 r
.72744 1.41418 m .73741 1.42183 L p .6894 1.382 L F P
0 g
s
0 .048 .562 r
.6581 1.35604 m .6804 1.37473 L p .6894 1.382 L F P
0 g
s
0 .048 .562 r
.6804 1.37473 m .72744 1.41418 L p .6894 1.382 L F P
0 g
s
0 .048 .562 r
.66499 1.36217 m .6581 1.35604 L p .65933 1.35706 L F P
0 g
s
0 .048 .562 r
.65933 1.35706 m .70803 1.39746 L .66499 1.36217 L F
0 0 .502 r
.6581 1.35604 m .65107 1.34907 L p .69835 1.38938 L F P
0 g
s
.559 .071 0 r
.66608 1.36058 m .70684 1.39619 L p .67442 1.36803 L F P
0 g
s
.617 .129 0 r
.70684 1.39619 m .64389 1.3412 L .63658 1.33231 L p F P
0 g
s
0 .106 .619 r
.67172 1.36756 m .66499 1.36217 L p .66617 1.36313 L F P
0 g
s
0 .106 .619 r
.66617 1.36313 m .7175 1.4047 L .67172 1.36756 L F
0 .163 .671 r
.67172 1.36756 m .74713 1.42869 L .75661 1.43483 L p F P
0 g
s
0 .163 .671 r
.67827 1.37226 m .67172 1.36756 L p .67285 1.36846 L F P
0 g
s
0 .163 .671 r
.75661 1.43483 m .67827 1.37226 L p .67285 1.36846 L F P
0 g
s
0 .217 .718 r
.68465 1.37634 m .67827 1.37226 L p .67937 1.37311 L F P
0 g
s
0 .217 .718 r
.76581 1.4403 m .68465 1.37634 L p .67937 1.37311 L F P
0 g
s
0 .268 .759 r
.69084 1.37985 m .68465 1.37634 L p .68571 1.37715 L F P
0 g
s
0 .268 .759 r
.77474 1.44516 m .69084 1.37985 L p .68571 1.37715 L F P
0 g
s
0 .316 .796 r
.69684 1.38284 m .69084 1.37985 L p .69187 1.38062 L F P
0 g
s
0 .316 .796 r
.78338 1.44946 m .69684 1.38284 L p .69187 1.38062 L F P
0 g
s
0 .36 .827 r
.70263 1.38536 m .69684 1.38284 L p .69783 1.38358 L F P
0 g
s
0 .36 .827 r
.79172 1.45324 m .70263 1.38536 L p .69783 1.38358 L F P
0 g
s
0 .279 .65 r
.70263 1.38536 m .70821 1.38744 L p .70697 1.38697 L F P
0 g
s
.366 0 0 r
.69684 1.38284 m .70263 1.38536 L p .70135 1.38485 L F P
0 g
s
0 .106 .619 r
.72558 1.41122 m .67172 1.36756 L p .7175 1.4047 L F P
0 g
s
0 .048 .562 r
.71584 1.40407 m .66499 1.36217 L p .70803 1.39746 L F P
0 g
s
0 .048 .562 r
.73741 1.42183 m .71584 1.40407 L p .70803 1.39746 L F P
0 g
s
0 .106 .619 r
.66499 1.36217 m .73741 1.42183 L .74713 1.42869 L p F P
0 g
s
0 .217 .718 r
.67827 1.37226 m .75661 1.43483 L .76581 1.4403 L p F P
0 g
s
0 .268 .759 r
.68465 1.37634 m .76581 1.4403 L .77474 1.44516 L p F P
0 g
s
0 .316 .796 r
.69084 1.37985 m .77474 1.44516 L .78338 1.44946 L p F P
0 g
s
0 .36 .827 r
.69684 1.38284 m .78338 1.44946 L .79172 1.45324 L p F P
0 g
s
.366 0 0 r
.61798 1.35214 m .69684 1.38284 L .70135 1.38485 L F
.57314 1.33455 m .60529 1.3471 L p .61798 1.35214 L F P
0 g
s
.366 0 0 r
.60529 1.3471 m .69684 1.38284 L p .61798 1.35214 L F P
0 g
s
.366 0 0 r
.70263 1.38536 m .57516 1.33632 L .57314 1.33455 L p F P
0 g
s
0 .279 .65 r
.62046 1.35373 m .70263 1.38536 L .70697 1.38697 L F
.57516 1.33632 m .6074 1.34873 L p .62046 1.35373 L F P
0 g
s
0 .279 .65 r
.6074 1.34873 m .70263 1.38536 L p .62046 1.35373 L F P
0 g
s
0 .279 .65 r
.5771 1.3377 m .57516 1.33632 L p .66617 1.37129 L F P
0 g
s
0 .279 .65 r
.67736 1.37574 m .5771 1.3377 L p .66617 1.37129 L F P
0 g
s
0 .279 .65 r
.70821 1.38744 m .67736 1.37574 L p .66617 1.37129 L F P
0 g
s
.433 0 0 r
.69084 1.37985 m .69684 1.38284 L p .69551 1.38231 L F P
0 g
s
.433 0 0 r
.69684 1.38284 m .57314 1.33455 L .57105 1.33232 L p F P
0 g
s
0 .339 .715 r
.70821 1.38744 m .71357 1.38914 L p .62285 1.35494 L F P
0 g
s
0 .339 .715 r
.5771 1.3377 m .60944 1.34997 L p .62285 1.35494 L F P
0 g
s
0 .339 .715 r
.60944 1.34997 m .70821 1.38744 L p .62285 1.35494 L F P
0 g
s
0 .339 .715 r
.57897 1.33871 m .5771 1.3377 L p .58094 1.33914 L F P
0 g
s
0 .339 .715 r
.58094 1.33914 m .67127 1.37319 L .5802 1.33917 L F
.58094 1.33914 m .5802 1.33917 L .57897 1.33871 L F
0 .396 .772 r
.58075 1.3394 m .57897 1.33871 L p .58023 1.33918 L F P
0 g
s
0 .339 .715 r
.68273 1.37758 m .57897 1.33871 L p .67127 1.37319 L F P
0 g
s
0 .339 .715 r
.71357 1.38914 m .68273 1.37758 L p .67127 1.37319 L F P
0 g
s
.433 0 0 r
.57105 1.33232 m .69084 1.37985 L p .69551 1.38231 L F P
0 g
s
0 .396 .772 r
.71357 1.38914 m .7187 1.39048 L p .62514 1.35582 L F P
0 g
s
0 .396 .772 r
.57897 1.33871 m .6114 1.35086 L p .62514 1.35582 L F P
0 g
s
0 .396 .772 r
.6114 1.35086 m .71357 1.38914 L p .62514 1.35582 L F P
0 g
s
0 .396 .772 r
.58023 1.33918 m .67618 1.37473 L .58075 1.3394 L F
0 .447 .823 r
.58246 1.3398 m .58075 1.3394 L p .68089 1.37593 L F P
0 g
s
.495 0 0 r
.68465 1.37634 m .69084 1.37985 L p .68947 1.37928 L F P
0 g
s
.495 0 0 r
.5689 1.32961 m .68465 1.37634 L p .68947 1.37928 L F P
0 g
s
.495 0 0 r
.69084 1.37985 m .57105 1.33232 L .5689 1.32961 L p F P
0 g
s
0 .447 .823 r
.72361 1.39151 m .6928 1.38022 L p .68089 1.37593 L F P
0 g
s
0 .447 .823 r
.6928 1.38022 m .58246 1.3398 L p .68089 1.37593 L F P
0 g
s
.176 0 0 r
.58075 1.3394 m .58246 1.3398 L p .58123 1.33959 L F P
0 g
s
.176 0 0 r
.57892 1.33921 m .58075 1.3394 L .58123 1.33959 L F
.4287 1.31459 m .57951 1.33919 L p .57892 1.33921 L F P
0 g
s
.176 0 0 r
.57951 1.33919 m .58075 1.3394 L p .57892 1.33921 L F P
0 g
s
0 .396 .772 r
.7187 1.39048 m .68788 1.37907 L p .67618 1.37473 L F P
0 g
s
0 .396 .772 r
.68788 1.37907 m .58075 1.3394 L p .67618 1.37473 L F P
0 g
s
.551 0 0 r
.67827 1.37226 m .68465 1.37634 L p .68323 1.37574 L F P
0 g
s
.551 0 0 r
.56668 1.32635 m .67827 1.37226 L p .68323 1.37574 L F P
0 g
s
.6 .028 0 r
.67172 1.36756 m .67827 1.37226 L p .67679 1.37162 L F P
0 g
s
.6 .028 0 r
.5644 1.32249 m .67172 1.36756 L p .67679 1.37162 L F P
0 g
s
.643 .083 0 r
.66499 1.36217 m .67172 1.36756 L p .67017 1.36686 L F P
0 g
s
.643 .083 0 r
.56207 1.31797 m .66499 1.36217 L p .67017 1.36686 L F P
0 g
s
0 .106 .619 r
.74713 1.42869 m .72558 1.41122 L p .7175 1.4047 L F P
0 g
s
.559 .071 0 r
.64389 1.3412 m .66608 1.36058 L p .67442 1.36803 L F P
0 g
s
.453 0 .051 r
.44893 1.2935 m .34822 1.28892 L .35521 1.28295 L p F P
0 g
s
.453 0 .051 r
.35521 1.28295 m .45152 1.28831 L .44893 1.2935 L p F P
0 g
s
.663 .214 .175 r
.55475 1.29973 m .45686 1.27546 L .4596 1.26761 L p F P
0 g
s
.663 .214 .175 r
.4596 1.26761 m .55221 1.2918 L .55475 1.29973 L p F P
0 g
s
.777 .294 .085 r
.63658 1.33231 m .55221 1.2918 L .54964 1.28277 L p F P
0 g
s
.759 .26 .035 r
.55221 1.2918 m .63658 1.33231 L .64389 1.3412 L p F P
0 g
s
.43 0 .002 r
.4464 1.29797 m .3414 1.29415 L .34822 1.28892 L p F P
0 g
s
.43 0 .002 r
.34822 1.28892 m .44893 1.2935 L .4464 1.29797 L p F P
0 g
s
.65 .185 .135 r
.55724 1.30667 m .45416 1.28233 L .45686 1.27546 L p F P
0 g
s
.65 .185 .135 r
.45686 1.27546 m .55475 1.29973 L .55724 1.30667 L p F P
0 g
s
.759 .26 .035 r
.64389 1.3412 m .55475 1.29973 L .55221 1.2918 L p F P
0 g
s
.405 0 0 r
.44393 1.30178 m .33476 1.29871 L .3414 1.29415 L p F P
0 g
s
.405 0 0 r
.3414 1.29415 m .4464 1.29797 L .44393 1.30178 L p F P
0 g
s
.737 .221 0 r
.55475 1.29973 m .64389 1.3412 L .65107 1.34907 L p F P
0 g
s
.634 .152 .091 r
.55968 1.31272 m .45152 1.28831 L .45416 1.28233 L p F P
0 g
s
.634 .152 .091 r
.45416 1.28233 m .55724 1.30667 L .55968 1.31272 L p F P
0 g
s
.376 0 0 r
.44153 1.30499 m .32829 1.30266 L .33476 1.29871 L p F P
0 g
s
.376 0 0 r
.33476 1.29871 m .44393 1.30178 L .44153 1.30499 L p F P
0 g
s
.737 .221 0 r
.65107 1.34907 m .55724 1.30667 L .55475 1.29973 L p F P
0 g
s
.71 .179 0 r
.55724 1.30667 m .65107 1.34907 L .6581 1.35604 L p F P
0 g
s
.343 0 0 r
.4392 1.30767 m .32202 1.30605 L .32829 1.30266 L p F P
0 g
s
.343 0 0 r
.32829 1.30266 m .44153 1.30499 L .4392 1.30767 L p F P
0 g
s
.616 .117 .044 r
.56207 1.31797 m .44893 1.2935 L .45152 1.28831 L p F P
0 g
s
.616 .117 .044 r
.45152 1.28831 m .55968 1.31272 L .56207 1.31797 L p F P
0 g
s
.71 .179 0 r
.6581 1.35604 m .55968 1.31272 L .55724 1.30667 L p F P
0 g
s
.679 .133 0 r
.55968 1.31272 m .6581 1.35604 L .66499 1.36217 L p F P
0 g
s
.307 0 0 r
.43694 1.30986 m .31594 1.30893 L .32202 1.30605 L p F P
0 g
s
.307 0 0 r
.32202 1.30605 m .4392 1.30767 L .43694 1.30986 L p F P
0 g
s
.594 .078 0 r
.5644 1.32249 m .4464 1.29797 L .44893 1.2935 L p F P
0 g
s
.594 .078 0 r
.44893 1.2935 m .56207 1.31797 L .5644 1.32249 L p F P
0 g
s
.266 0 0 r
.43476 1.3116 m .31007 1.31135 L .31594 1.30893 L p F P
0 g
s
.266 0 0 r
.31594 1.30893 m .43694 1.30986 L .43476 1.3116 L p F P
0 g
s
.679 .133 0 r
.66499 1.36217 m .56207 1.31797 L .55968 1.31272 L p F P
0 g
s
.221 0 0 r
.43266 1.31294 m .30442 1.31335 L .31007 1.31135 L p F P
0 g
s
.221 0 0 r
.31007 1.31135 m .43476 1.3116 L .43266 1.31294 L p F P
0 g
s
.569 .035 0 r
.56668 1.32635 m .44393 1.30178 L .4464 1.29797 L p F P
0 g
s
.569 .035 0 r
.4464 1.29797 m .5644 1.32249 L .56668 1.32635 L p F P
0 g
s
.019 .401 .854 r
.70263 1.38536 m .79172 1.45324 L .79975 1.45654 L p F P
0 g
s
.643 .083 0 r
.67172 1.36756 m .5644 1.32249 L .56207 1.31797 L p F P
0 g
s
.172 0 0 r
.43064 1.31393 m .29898 1.31497 L .30442 1.31335 L p F P
0 g
s
.172 0 0 r
.30442 1.31335 m .43266 1.31294 L .43064 1.31393 L p F P
0 g
s
.075 .437 .876 r
.70821 1.38744 m .79975 1.45654 L .80746 1.45941 L p F P
0 g
s
.019 .401 .854 r
.79975 1.45654 m .70821 1.38744 L .70263 1.38536 L p F P
0 g
s
.539 0 0 r
.5689 1.32961 m .44153 1.30499 L .44393 1.30178 L p F P
0 g
s
.539 0 0 r
.44393 1.30178 m .56668 1.32635 L .5689 1.32961 L p F P
0 g
s
.12 0 0 r
.4287 1.31459 m .29378 1.31625 L .29898 1.31497 L p F P
0 g
s
.12 0 0 r
.29898 1.31497 m .43064 1.31393 L .4287 1.31459 L p F P
0 g
s
.127 .47 .894 r
.71357 1.38914 m .80746 1.45941 L .81484 1.46187 L p F P
0 g
s
.075 .437 .876 r
.80746 1.45941 m .71357 1.38914 L .70821 1.38744 L p F P
0 g
s
.6 .028 0 r
.67827 1.37226 m .56668 1.32635 L .5644 1.32249 L p F P
0 g
s
.175 .5 .908 r
.7187 1.39048 m .81484 1.46187 L .82188 1.46396 L p F P
0 g
s
.127 .47 .894 r
.81484 1.46187 m .7187 1.39048 L .71357 1.38914 L p F P
0 g
s
.065 0 0 r
.29378 1.31625 m .4287 1.31459 L .42685 1.31497 L p F P
0 g
s
.504 0 0 r
.57105 1.33232 m .4392 1.30767 L .44153 1.30499 L p F P
0 g
s
.504 0 0 r
.44153 1.30499 m .5689 1.32961 L .57105 1.33232 L p F P
0 g
s
.175 .5 .908 r
.82188 1.46396 m .72361 1.39151 L .7187 1.39048 L p F P
0 g
s
.551 0 0 r
.68465 1.37634 m .5689 1.32961 L .56668 1.32635 L p F P
0 g
s
.464 0 0 r
.57314 1.33455 m .43694 1.30986 L .4392 1.30767 L p F P
0 g
s
.464 0 0 r
.4392 1.30767 m .57105 1.33232 L .57314 1.33455 L p F P
0 g
s
.418 0 0 r
.57516 1.33632 m .43476 1.3116 L .43694 1.30986 L p F P
0 g
s
.418 0 0 r
.43694 1.30986 m .57314 1.33455 L .57516 1.33632 L p F P
0 g
s
.366 0 0 r
.5771 1.3377 m .43266 1.31294 L .43476 1.3116 L p F P
0 g
s
.366 0 0 r
.43476 1.3116 m .57516 1.33632 L .5771 1.3377 L p F P
0 g
s
.307 0 0 r
.57897 1.33871 m .43064 1.31393 L .43266 1.31294 L p F P
0 g
s
.307 0 0 r
.43266 1.31294 m .5771 1.3377 L .57897 1.33871 L p F P
0 g
s
.243 0 0 r
.58075 1.3394 m .4287 1.31459 L .43064 1.31393 L p F P
0 g
s
.243 0 0 r
.43064 1.31393 m .57897 1.33871 L .58075 1.3394 L p F P
0 g
s
0 .447 .823 r
.58075 1.3394 m .7187 1.39048 L .72361 1.39151 L p F P
0 g
s
.176 0 0 r
.58246 1.3398 m .42685 1.31497 L .4287 1.31459 L p F P
0 g
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[22]:=",
  ImageSize->{189.563, 331.875},
  ImageMargins->{{42.8125, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002m0001Bb000`400?l00000o`00003o_@3oool0
0;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3o
ool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00
_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?oo
o`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m
0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo
002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00680oooo00<000000?ooo`3oool0F03o
ool00680oooo00<000000?ooo`3oool0F03oool00680oooo00<000000?ooo`3oool0F03oool00680
oooo0P00001I0?ooo`00HP3oool2000005T0oooo001R0?ooo`800000F@3oool006<0oooo00<00000
0?ooo`3oool0E`3oool00680oooo00<0000002T@O03oool0F03oool00680oooo00<0000002T@O03o
ool0F03oool00680oooo00<0000002T@O03oool0F03oool00680oooo00<0000002T@O0000000F03o
ool00680oooo00<0000002T@O0000000F03oool00680oooo00<0000002T@O0000000F03oool00680
oooo00<0000002T@O0000000F03oool00640oooo0P0000000`0Y47`000000?ooo`1G0?ooo`00H@3o
ool4000005P0oooo001P0?ooo`030000002OIi00000000800000F03oool00600oooo00<0000009mW
T00000000P00001H0?ooo`00G`3oool20000000409mWT0000000000002T@O5P0oooo001O0?ooo`80
000000@0WfN@000000000000:A1lF03oool005l0oooo0P0000000`2OIi00000000000002000005L0
oooo001O0?ooo`80000000<0WfN@0000000000000P00001G0?ooo`00GP3oool30000000309mWT000
0000000000800000E`3oool005h0oooo0`0000000`2OIi00000000000002000005L0oooo001M0?oo
o`@000000P2OIi000`000000:A1l0000001G0?ooo`00G@3oool400000080WfN@0`00001G0?ooo`00
G03oool00`000000l[240000000200000080WfN@0`00001G0?ooo`00G03oool00`000000l[240000
000200000080WfN@0P0000000`0Z5880oooo0?ooo`1E0?ooo`00G03oool01`000000l[2400000000
0000^Wf@0000002OIi000P0000000`0Z588000000?ooo`1E0?ooo`00G03oool01`000000l[240=BD
SP000000^Wf@0000002OIi000P0000000`0Z588000000?ooo`1E0?ooo`00G03oool01`000000l[24
0=BDSP000000^Wf@0000002OIi000P0000000`0Z588000000?ooo`1E0?ooo`00G03oool01`000000
l[240=BDSP000000^Wf@0000002OIi000P0000000`0Z588000000?ooo`1E0?ooo`00G03oool20000
00090=BDSP000000^Wf@0000002OIi00000007Q<T0000000000/05H0oooo001L0?ooo`80000000L0
e9B>0000002jOI00000009mWT0000000N4b@00800000EP3oool005/0oooo00d0000005XVMP000000
e9B>0000002jOI00000009mWT0000000N4b@0000000Z5880000005D0oooo001K0?ooo`060000001J
9WH000000=BDSP000000^Wf@0`000000101hC900000002XDPP00001E0?ooo`00F`3oool01P000000
FRIf0000003DU8h000000;YmT0<0000000@0N4b@0000000Z58800000E@3oool005/0oooo00<00000
05XVMP2ADX400P000000102jOI00000009iXT`00000207Q<T0800000E@3oool005/0oooo00<00000
05XVMP2ADX400P0000001P2jOI00000009iXT`2NJ9<0000007Q<T0<00000E03oool005/0oooo00<0
000005XVMP2ADX401000000209iXT`030000001hC900000000800000E03oool005/0oooo0P000000
0`2ADX4000000000000200000080WVRC00D0000007Q<T000000000000000>P1D0?ooo`00F`3oool2
00000005095BP@00000000000;UnT`0000000P2NJ9<00`000000N4b@07Q<T002000005@0oooo001K
0?ooo`80000000D0TE:1000000000000^GjC0000000209iXT`030000001hC900N4b@00800000E03o
ool005X0oooo0`0000000`2ADX4000000000000200000080WVRC00<0000007Q<T01hC9000P00001D
0?ooo`00FP3oool300000003095BP@0000000000008000000P2NJ9<00`000000N4b@07Q<T0020000
05@0oooo001J0?ooo`<0000000D0TE:1000000000000cXF00000000309iXT`D00000D`3oool005X0
oooo0`000000102ADX4000000000003>QH0200000080WVRC0P0000000`1iCi<000000000001C0?oo
o`00FP3oool0100004<000000000002ADX42000000030<j5P000000000000080WVRC00<0000007U?
T`1iCi<00P00001C0?ooo`00FP3oool01`000000FRIf000000000000/Vf30000003>QH000P000002
09iXT`030000001iCi<0NDnC00800000D`3oool005X0oooo00L0000005XVMP00000000000;9]P`00
0000cXF00080000000<0WVRC0000000000000P1iCi<300000580oooo001J0?ooo`80000000D0TE:1
0000002bKH<000000<j5P0020000000309iXT`00000000000080NDnC0`00001B0?ooo`00FP3oool2
00000005095BP@000000/Vf30000003>QH000`000000102NJiH0000007U?T`1iCi<300000580oooo
001J0?ooo`80000000D0TE:10000002bKH<000000<j5P0030000000309i[UP000000NDnC0080NDnC
0`00001A0?ooo`00F@3oool300000005095BP@000000/Vf30000003>QH000`0000000`2NJiH00000
07U?T`0207U?T`030000000c8Xd000000540oooo001I0?ooo`<0000000D0TE:10000002bKH<00000
0<j5P0030000000309i[UP000000NDnC0080NDnC00<0000003<RS@000000D@3oool005T0oooo00/0
0000058LKP000000TE:10000002bKH<000000<j5P0000000kIia0000000209i[UP040000001iCi<0
000000000080<b:=00<000000?ooo`3oool0CP3oool005T0oooo00<00000058LKP0000000P000000
102bKH<000000<j5P03>QH0200000080WV^F00@0000007U?T`00000000000P0c8Xd00`000000oooo
0?ooo`1>0?ooo`00F@3oool00`000000DQa^00000002000000040;9]P`000000cXF00<j5P0800000
0P2NJiH200000080NU:G00<0000003<RS@000000D03oool005P0oooo0P0000001P1B76h0000008e<
O0000000/Vf300000080cXF00P00000209i[UP8000000P1jDYL010000000<b:=00000000001?0?oo
o`00F03oool300000080SDal00<000000;9]P`0000000P3>QH0200000080WV^F00H0000007YBU`1j
DYL0NU:G0000000c8Xd2000004l0oooo001H0?ooo`030000001B76h000000080SDal0`0000020<j5
P0<0000000<0WV^F0000001jDYL00P1jDYL20000000303TYT`000000oooo04h0oooo001H0?ooo`03
0000001B76h000000080SDal0`0000020<j5P0D00000101jDYL010000000>BVC00000000001>0?oo
o`00F03oool00`000000DQa^0000000208e<O0030000002`JGl000000080cXF00`0000000`2OKIP0
000007YBU`0307YBU`040000000i:I<0>BVC000004h0oooo001G0?ooo`80000000H0DQa^0000002=
C7`0SDal0000002`JGl2000000040<j5P0000000000000000080WffH00D0000007YBU`1jDYL0NU:G
0000000203TYT`030000003oool0oooo04`0oooo001G0?ooo`80000000H0DQa^0000002=C7`0SDal
0000002`JGl2000000040<j5P0000000000000000080WffH00<0000007YBU`1jDYL00P00000203TY
T`800000C@3oool005L0oooo00H00000058LKP1B76h0000008e<O00000020;1YO`060000003=PG`0
00000000003]WW4000000P2OKIP01`000000NU:G07YBU`000000O5JJ0000000i:I<00P00001=0?oo
o`00E`3oool00`000000DQa^0000000208e<O0040000002`JGl0/6Uo00000080cH5l00<000000>fN
L@0000000P2OKIP00`000000NU:G0000000207aFVP030000000i:I<0000000800000C03oool005L0
oooo00<00000058LKP0000000P2=C7`010000000/6Uo0;1YO`0000020<f1O0030000003]WW400000
00<0WffH0P00000207aFVP80000000<0?S2H000000000000C03oool005H0oooo00@0000004/CIP00
000000000P2=C7`010000000/6Uo0;1YO`0000020<f1O0<000000`2OKIP010000000O5JJ07aFVP1l
EYX200000080?S2H0P00001;0?ooo`00EP3oool010000000Ba=V04/CIP00000208e<O0040000002`
JGl0/6Uo00000080cH5l0`00000309m]V0030000001lEYX0O5JJ0080O5JJ00<0000003h`V00n<9P0
0P00001;0?ooo`00EP3oool00`000000Ba=V04/CIP020000000508e<O0000000/6Uo0;1YO`000000
0P3=PG`01@000000jiU/000000000000WffH00800000101lEYX00`000000?S2H03h`V003000004X0
oooo001F0?ooo`050000001;4fH0Ba=V0000002:AgP00P0000020;1YO`050000003=PG`0cH5l0000
003[VF`00`00000209m_V`030000001lEYX0O5JJ008000000P0n<9P3000004X0oooo001F0?ooo`03
0000001;4fH0000000<0RTMh00@000000;1YO`2`JGl000000P3=PG`010000000jiU/000000000003
09m_V`090000001lEYX0O5JJ0000001nFId0000003h`V0000000@cNM00800000B@3oool005D0oooo
0P0000000`1;4fH0000008Y7N00208Y7N080000000<0/6Uo0000003=PG`00P3=PG`3000000<0WfnK
00<0000007aFVP0000000P1nFId200000080@cNM0P0000190?ooo`00E@3oool20000000304/CIP00
0000RTMh0080RTMh00@000000:iUN`00000000000`3=PG`3000000<0WfnK00<0000007aFVP000000
0P1nFId2000000<0@cNM0P0000180?ooo`00E@3oool010000000Ba=V04/CIP00000308Y7N0040000
002^IG/0[VEk000000<0cH5l0`00000409m_V`030000001nFId0OUVM0080OUVM00@0000004<gW@13
=id0@cNM0`0000170?ooo`00E@3oool010000000Ba=V04/CIP00000308Y7N0030000002^IG/0[VEk
008000000P3=PG`2000000030>^Q9P2OKi/0WfnK0080WfnK00<0000007iIW@1nFId00P1nFId01@00
0000@cNM04<gW@000000B3fQ00800000A`3oool005D0oooo00<0000004/CIP0000000`2:AgP02000
0000[VEk0:iUN`2^IG/000000<amN0000000cH5l0P000000103[XBH0WfnK09m_V`2OKi/2000000<0
OUVM0P00000204<gW@0300000018?J40000000800000AP3oool005@0oooo00@0000004@:GP000000
00000`2:AgP01@000000[VEk0:iUN`2^IG/000000080c7eh0P0000001`3[VF`0000009m_V`2OKi/0
00000:1aW@0000000`1nFId01@000000P5bP00000013=id000000080B3fQ0`0000150?ooo`00E03o
ool010000000A0YN04@:GP00000308Y7N0050000002^IG/0[VEk0:iUN`0000000`3<OGP00`000000
jiU/0000000200000080X76M00@0000007iIW@1nFId000000P20G:000`000000@cNM0000000204Pm
X@<00000A@3oool005@0oooo00<0000004@:GP142Uh00`0000000`2:AgP000000:iUN`020:iUN`06
0000003<OGP0c7eh0<amN0000000jiU/0P0000040:1aW@030000001nFId000000080P5bP0P000003
04PmX@@00000A03oool005@0oooo00<0000004@:GP0000000`27@W<2000000<0[VEk00@000000<am
N03<OGP0c7eh100000040:1aW@8000001020G:001@000000B3fQ04PmX@000000CD>U00@00000@`3o
ool005<0oooo0P0000000`142Uh0000008M2L`0308M2L`8000000P2^IG/010000000c7eh0<amN03<
OGP3000000030>^Q9P2PLId0X76M0080X76M0P000004081LX00500000018?J40B3fQ0000001=@jD0
0`0000000`007ZH000000?ooo`110?ooo`00D`3oool010000000A0YN04@:GP00000408M2L`050000
002]HWL000000:iUN`0000000`3<OGP010000000jYEV0000003[XBH40:1aW@0300000020G:00P5bP
0080P5bP0P00000204PmX@030000001=@jD0CD>U00D00000@@3oool005<0oooo00@0000004@:GP14
2Uh000001027@W<00`000000[F9g0:eRM`02000000<0c7eh00@000000>ZEIP000000jj4V0`2PLId2
000000@0P5bP00D00000085PXP000000B3fQ0000000204e3Y@80000000@001:N000NYP00::d00000
@03oool00580oooo0P0000000`142Uh0000008M2L`0308M2L`030000002]HWL0[F9g0080[F9g00<0
00000<amN03<OGP00P3<OGP2000000040>^Q9P2PLId0X76M00000080XG>O00@00000081LX020G:00
00000P21H:8200000080CD>U00<000000558Z@0000001@00000o0?ooo`00DP3oool4000000@0Qd9c
00<000000:eRM`2]HWL00P2]HWL2000000<0c7eh1@0000030:5cW`0400000020G:00P5bP000000<0
PF2R00@0000004e3Y@1=@jD000000P1AB:T01P00000001:N000NYP00::d023>d000003h0oooo001B
0?ooo`030000000n0EH0?P5F008000000`27@W<00`000000[F9g0:eRM`020:eRM`070000003;NW<0
00000000003<OGP000000>ZEIP02000000@0XG>O00<00000081LX00000001021H:8010000000CD>U
04e3Y@0000020558Z@L00000?@3oool00540oooo0P00000203h1EP0600000025?Fh0000000000027
@W<00000102]HWL010000000bgYc0<]jL`3;NW<2000000030>ZEIP000000000000D0XG>O0P000003
085PXP8000000P1=@jD01@000000DDRY0558Z@000000EDf/00<0000000<023>d01<l^P000000?03o
ool00540oooo0P00000203h1EP0400000025?Fh0QCe^08DmKP800000102]HWL00`000000bgYc0<]j
L`020<]jL`040000003ZUFH00000000000D0XG>O00<00000085PXP21H:800P21H:8010000000Pf>U
0000000000030558Z@030000001ECJ`0EDf/00H00000>`3oool00540oooo0P00000203h1EP030000
0025?Fh0QCe^0080QCe^0P0000030:eRM`030000003;NW<0bgYc0080bgYc0`0000000`3VV1T0XG>O
0:5cW`020:5cW`8000000`21H:800`000000Pf>U08=SY@02000000<0DDRY00@0000005E=[01ECJ`0
FE:`0`0000000`0C?;X07DFo0000000j0?ooo`00D03oool200000080?P5F00<0000008DmKP25?Fh0
0`25?Fh01P000000[5ic000000000000[F9g000000@0bgYc0`0000001`3VV1T0XG>O0:5cW`2QLil0
00000:5fXP0000000P21H:800`000000Pf>U08=SY@0208=SY@040000001AB:T0DDRY000000<0EDf/
00<0FE:`0000000000001P00000h0?ooo`00D03oool200000080?P5F00<0000008DmKP25?Fh00`25
?Fh010000000[5ic0:aNL`2/GW<2000000@0bgYc00<000000>VAG`000000100000030:5fXP800000
1023HjD01@000000QFJW0000001AB:T000000080EDf/00<0000005UB/01ID[0010000000100MAKl0
9Tg300000000000g0?ooo`00D03oool00`000000=`1=00000002000000@0QCe^00<000000:aNL`2/
GW<00`2/GW<00`000000bgYc0<]jL`020<]jL`030000003YTEl00000008000001@2QMZ82000000@0
Pf>U00D0000008EVY`000000DDRY0000000205E=[0060000001ID[00FE:`05eG/`00000023>d0`00
0000100VCL<0000000000000000e0?ooo`00C`3oool2000000<0=`1=0P00000308DmKP030000002/
GW<0[5ic00<0[5ic0`0000030<]jL`040000003YTEl00000000000D0XGJR00<0000008=SY@23HjD0
0P23HjD010000000QFJW08EVY`25IZL200000080EDf/00@0000005UB/01ID[00GENc1@0000000`0V
CL<0000003MKb`03000003<0oooo001?0?ooo`030000000g04d0=`1=0080=`1=00H00000088hJ000
0000000008DmKP0000050:aNL`040000003:MVh0bWI^00000080bgYc00@000000>VAG`0000000000
102QMZ82000000<0Pf>U00<0000008EVY`25IZL00`25IZL01P000000EDf/05E=[0000000FE:`0000
0080GENc0P0000000`0C?;X00000000000020000000403MKb`0oH/h0000000000380oooo001?0?oo
o`050000000g04d0=`1=03L0C@0000001022>6P2000000D0[5ic00@000000<YfKP3:MVh0bWI^0`00
00000`3YTEl00000000000030:5fXP040000002RN:@0XWRT00000080Pf>U00<0000008EVY`25IZL0
0P25IZL200000080EDf/00<0000005UB/00000000P1MEk<00`1QG;H00000000000020000000502I=
``000000=e_;0000000oH/h00`00000`0?ooo`00CP3oool00`000000=`1=03L0C@0203L0C@030000
0022>6P0PSQX00<0PSQX0`0000030:aNL`030000003:MVh0bWI^00<0bWI^00@000000>VAG`3YTEl0
00000P2QMZ801`000000XWRT0:9hY02RN:@0000008=SY@0000001025IZL00`000000QVVZ08IYZP02
00000080FE:`00<0000005eG/`1MEk<00P1QG;H20000000801e5_`00000000000000000gFl/0?f;>
0000001=K]<4000002d0oooo001>0?ooo`8000000P0g04d00`000000PSQX088hJ004088hJ0030000
002ZFVh0ZUY^0080000000<0[5ic0000003:MVh0103:MVh2000000040>VAG`0000000000000000D0
XWRT0P00000408EVY`0300000026JJX0QVVZ008000000P1ID[000`000000GENc00000002065L]P03
06AP^0000000000000<0000000L0=e_;000000000000000004e^d`000000FGSG00@00000:P3oool0
04h0oooo00<000000300A00000000`000004088hJ0030000002ZFVh0ZUY^00<0ZUY^0P0000050<Yf
KP80000000<0jI5O0000000000001P2RN:@00`000000QFJW08EVY`0208EVY`0300000026JJX0QVVZ
0080QVVZ00P0000005UB/01ID[00000005eG/`000000HEbf00000080I62h0P0000001`0VCL<00000
000000000000?f;>0000001=K]<00P0000000`1IN=L0000006B1fP05000002H0oooo001=0?ooo`03
0000000`04@0<0140080<01400<0000007lbH`0000000P0000000`22>6P000000:YJKP050:YJKP<0
00000`3:MVh00`000000j8aG00000002000000D0XWRT0P00000308EVY`0300000026JJX0QVVZ0080
QVVZ00@0000008Q/[000000000000P1MEk<00`000000HEbf0000000206AP^00306QT^`0000000000
0080000000<0=e_;0000000000000`000000101IN=L000000000001TPMX20000000307><g@000000
oooo02D0oooo001=0?ooo`050000000`04@0<0140300A00000001@1o<V<3000000D0ZUY^00<00000
0<UbJ@39LVT00P0000020<YfKP040000003XS5L00000000000<0XWRT0P0000001@2SNZH0000008EV
Y`25IZL0000000@0QVVZ00<0000008Q/[028K:`00P00000205eG/`050000001QG;H0000006AP^000
00000P1XI;/30000000503MKb`000000000004IXd01=K]<00P0000001@1IN=L0000006B1fP000000
KXSL0080LhcM00<000000?ooo`3oool08`3oool004`0oooo00<000000300A00`04@00P0`04@00`00
0000Oc9S07lbH`0407lbH`040000002YEVT00000000000<0ZUY^00<000000<UbJ@39LVT00P39LVT3
000000060>R<E`00000000000000002RN:@00000102SNZH00`000000QFJW0000000408IYZP050000
0028K:`0R6b/08Q/[00000000P1MEk<01@000000HEbf0000001TH;P000000080J6Bk00<0JfRm0000
000000000P0000000`0oH/h00000000000020000000405Uhe`000000000006B1fP80000000<0LhcM
0000003oool0903oool004`0oooo0P0000020300A0030000001o<V<0Oc9S00@0Oc9S00<000000:UF
J@2YEVT00P2YEVT2000000030:YJKP000000bG9Y00D0bG9Y00<000000>R<E`3XS5L00`0000050:=j
YP8000001026JJX01P000000R6b/08Q/[028K:`0000008U^[P80000000L0HEbf0000001TH;P0I62h
0000001XI;/000000080JfRm0P0000000`0gFl/00000000000020000000905=ce@00000000000000
001TPMX0000006j8g01cS=d0000002D0oooo001<0?ooo`030000000Y03X0000000D000000`1o<V<0
0`000000ZEIY0:UFJ@040:UFJ@8000001P39LVT2000000030>R<E`000000h8h;00H0XgZV00<00000
08IYZP26JJX00P26JJX01@000000R6b/08Q/[028K:`000000080RFj^0P0000001`1QG;H0000006AP
^01TH;P0000006QT^`0000000P1[J;d01@1^K;l00000000000000000?f;>00@0000000<0FGSG0000
001TPMX00`0000000`1cS=d000000?ooo`0S0?ooo`00B`3oool01@000000:@0j02T0>P0Y03X00000
00@0OBeL1@0000060:UFJ@<000001039LVT010000000ihM?0000003XS5L2000000<0XgZV0`000003
08IYZP0500000028K:`0R6b/08Q/[00000001029KZh01@000000HEbf0000001TH;P000000080J6Bk
00<0000006]X_@0000000P1^K;l00`000000=e_;000000040000000805Uhe`000000000006B1fP00
0000KXSL07><g@00000T0?ooo`00B`3oool01@000000:@0j02T0>P0Y03X0000000L0OBeL00@00000
0:MAI00000000000102YEVT010000000bFiS0<U^H`0000030<UbJ@040000003WQdl000000>R<E`80
00000P2SNZH01@000000XgbX0:=lZ02SO:P000000080QVVZ00@0000008Q/[028K:`000001029KZh2
00000004065L]P000000I62h00000080J6Bk00<0000006]X_@0000000P1^K;l01@1aL<4000000000
00000000AVS@00<0000000<0FGSG0000001TPMX00`0000000`1cS=d000000?ooo`0R0?ooo`00BP3o
ool01@000000:@0j02T0>P0Y03X0000000L0OBeL00<000000:MAI02WDF@00P2WDF@3000000030:UF
J@000000bFiS0080bFiS0P0000020<UbJ@030000003WQdl0000000@00000102SO:P00`000000QVVZ
0000000308Q/[00500000029KZh0RFj^08U^[P0000000P2;LK001P000000I62h06AP^0000000J6Bk
00000080JfRm00<0000006i/_`0000000P1aL<400`1dLl<00000000000030000000405Uhe`000000
000006B1fP80000000<0LhcM0000003oool08P3oool004X0oooo2000000407d]G0030000002WDF@0
Ye5T00D0Ye5T0P0000050<U^H`<000000P3WQdl3000000H0XgbX00D0000008Q/[028K:`0R6b/0000
000408U^[P040000002;LK00Rg6`00000080I62h00<0000006QT^`0000000P1[J;d00`000000KVbo
00000002075``@80M7?300@0000004IXd0000000Dg?E0`0000000`1TPMX0000000000002000002<0
oooo00190?ooo`050000000R02l08P0_0280;`000000101i9UH6000000L0Ye5T0`0000050<U^H`04
0000003WQdl0ihM?0>N7C`800000102SO:P00`000000Y7jZ0000000208Q/[00300000029KZh0RFj^
0080RFj^00D0000008]a/02;LK00000008ad/P0200000080J6Bk00<0000006]X_@0000000P1^K;l0
0`000000LG310000000207Ac``0307Mga@00000000000080000000<0FGSG0000001TPMX00`000000
0`1cS=d000000?ooo`0Q0?ooo`00B@3oool01@0000008P0_0280;`0R02l0000000P0NBIF00<00000
0:I=GP0000000P0000040:MAI00300000038JUd0b6YM008000000`39KV<200000080ihM?00@00000
0=R3002SO:P0XgbX0P0000030:AnZP8000001029KZh010000000Rg6`08]a/000000208ad/P800000
0P1XI;/00`000000JfRm0000000206i/_`050000001aL<40000007Ac``0000000P1gMlD00`1jN/L0
000005=ce@030000000306B1fP000000KXSL008000008P3oool004P0oooo00<000000280;`0R02l0
1P00000407TVEP030000002VCEh0YTeN00<0YTeN0`0000000`2WDF@000000<QZG@030<QZG@800000
00@0bFiS0000003VPTH000000P3WQdl3000000D0Y7jZ00<0000008U^[P29KZh00P29KZh01@000000
Rg6`08]a/02;LK00000000<0S7Bb00<0000006QT^`0000000P1[J;d00`000000KVbo00000002075`
`@070000001dLl<0000007Mga@000000NW[707emb@020000000305Uhe`000000I87J00@000008P3o
ool004P0oooo1000000507HPCPH000001`2VCEh2000000H0b6YM0`0000000`3VPTH000000>N7C`02
000000@0Y7jZ0`00000308U^[P050000002;LK00Rg6`08]a/00000000`2<M;8010000000SWJd0000
0000000206]X_@030000001^K;l000000080LG3100H0000007Ac``000000MgO50000001jN/L207em
b@0707n0bP00000000000000001TPMX0000006j8g00200000240oooo00170?ooo`050000000J02<0
6P0S01X08`0000002@1f84h00`000000Y4QH00000002000000D0YTeN0`0000060<QZG@030000003V
PTH0iX9600<000000`2TOZX01@000000Y82/0:B0[02TP:`000000080RFj^00@0000008]a/02;LK00
00000`2<M;801`000000SWJd08if]02>M[@0000006]X_@0000000P1^K;l01@000000LG310000001d
Ll<000000080MgO50P0000001@1mOLT0000007n0bP1oP<X0PX?<00H000008@3oool004H0oooo00D0
000001X08`0J02<06P0S0000000907HPCP030000002TB5P0Y4QH00<0Y4QH10000000102VCEh00000
0<IVEP36IUH2000000@0b6YM0P0000020>J2AP80000000<0Y7jZ000000000000102TP:`00`000000
RFj^0000000208]a/0030000002<M;80S7Bb0080S7Bb00D0000008if]02>M[@0000008mi]P020000
0080KVbo00h00000075``@000000M7?30000001gMlD0000007Yja`000000OGg90000001oP<X00000
08:3c080Q8K=00<0QhW?0000001^R=`00P00000P0?ooo`00A@3ooolA000000P0Y4QH0P0000040<IV
EP8000000P38JUd00`000000i7`l000000020>J2AP8000001`2TP:`010000000Rg6`08]a/0000004
08ad/P040000002>M[@0SWJd000000<0SgVf00<0000006i/_`0000000P1aL<402P000000M7?30000
001gMlD0000007Yja`000000OGg90000001oP<X20000000408B6c@000000QhW?08V<d0<00000803o
ool004@0oooo00<0000001405`0A01L00P0A01L00`000000LQU6078IAP08078IAP030000002R@U40
000000<000001@2TB5P3000000@0aVIF0P0000001@38JUd000000>Al?0000000iX96008000001@2T
P:`00`000000YH:^0000000208]a/0050000002<M;80S7Bb08ad/P0000000P2>M[@01P000000SgVf
08mi]P2?NKH00000091k^08000000P1aL<402P000000M7?30000001gMlD0000007Yja`000000OGg9
0000001oP<X20000000308B6c@00000000000080RHc@00<0S8oA0000003oool07`3oool004@0oooo
00<0000001405`0A01L01`000005078IAP030000002R@U40XT9A00@0XT9A10000000102TB5P00000
0<EQC`35HDl2000000@0aVIF0P0000020>Al?0030000003VPTH0000000<0Y82/0P0000030:F2[P80
00000`2<M;8010000000SWJd08if]000000308mi]P090000002@NkP0T7^h091k^0000000LG310000
001dLl<000000080MgO50P000000101mOLT0000007n0bP1oP<X20000000408B6c@000000000008V<
d08000008@3oool004<0oooo100000000`0800T0000006lA?P0406lA?PL000002@2R@U42000000@0
aF5?0P0000030<IVEP8000000P3TO3`3000000030:B0[0000000YH:^00@0YH:^00D0000008ad/P2<
M;80S7Bb0000000208if]0050000002?NKH0SgVf08mi]P0000000`2@NkP04@000000TWjj00000000
0000M7?30000001gMlD0000007Yja`000000OGg90000001oP<X0000008:3c0000000Q8K=00800000
00<0RHc@0000000000008@3oool00480oooo00<0000000P02@0800T00`0800T00`000000Ka4n06lA
?P0706lA?P030000002P?4X0X3a:00@000001P2R@U43000000@0aF5?0P0000001036IUH000000>5e
<@0000020>Al?0<00000102UPZh010000000YXB_0:J4[`00000208ad/P040000002>M[@0SWJd0000
0080SgVf00D00000091k^02@NkP0T7^h00000003099n^P0:0000001dLl<0000007Mga@000000NW[7
0000001mOLT0000007n0bP80000000L0Q8K=00000027RLl0000008V<d02<Sm4000000240oooo0012
0?ooo`030000000800T0200901000000102P?4X400000080XT9A00<000000<=KA`33FdL00P000004
0<EQC`<0000000D0hGDa0000003TO3`0i7`l000000030:F2[P8000000`2VQ:l00`000000S7Bb0000
000208if]0040000002?NKH0SgVf000000<0T7^h00@00000099n^P2BO[X0TWjj0`00000207Mga@04
0000001jN/L0000000000080Oh3:0P0000001024Q/d0000000000029S=0200000280oooo00110?oo
o`H000002P1Z2CD010000000WCI209df@P2M=T84000000@0X3a:100000030<=KA`8000001035HDl0
1@000000hGDa0>5e<@000000i7`l0080000000<0YH:^0000002VQ:l01@2VQ:l010000000SWJd08if
]000000308mi]P040000002@NkP0T7^h000000<0TWjj00H0000009>0_02CP;`0000007Mga@000002
07Yja`80000000D0Oh3:00000022Pl`0000008B6c@020000000308V<d000000000000280oooo0010
0?oooa@000001@2M=T8400000080X3a:00@000000<9E?P00000000000`33FdL200000080aF5?0P00
00020>5e<@@000001@2VQ:l01@000000YXFa0000002>M[@0000000<0SgVf00@00000091k^02@NkP0
00000`2BO[X01@000000Th2l09>0_0000000U8>m0080000000T0NW[70000001mOLT0000007n0bP00
0000PX?<00000024Q/d00P0000000`29S=0000000000000R0?ooo`00@03oool5000000/0I@0[00@0
000009X_>@2J;cT0VRli1000000509df@P@000000P32ECh300000080`e]70P0000000`35HDl00000
000000020>5e<@<000000`2VQ:l2000000<0YXFa00D0000008mi]P2?NKH0SgVf00000003091k^004
0000002BO[X0TWjj00000080Th2l00D0000009B3_@2DPkd0000009J5_`020000000307emb@000000
Oh3:0080000000<0Q8K=00000027RLl00`00000S0?ooo`00?`3ooolE000000@0VRli1@00000209df
@P040000002oCcD00000000000<0`UDn0`0000001@33FdL000000000003NKRD000000080hGDa0P00
00000`2VQ:l00000000000030:J5/@030000002WQk<000000080SgVf00D00000091k^02@NkP0T7^h
00000002099n^P040000002CP;`0Th2l00000080U8>m00@0000009J5_`2FQKl0UXFo0P00000207n0
bP80000000@0Q8K=000000000000RHc@0P00000S0?ooo`00?P3oool5000000H0H00P2@00000209LX
;`P0000000<0VRli0000000000000P0000020;m?=@<000000`32ECh3000000030=YU6@3NKRD00000
00@000000`2VQK42000000<0YhNc0P000003091k^0040000002BO[X0TWjj00000080Th2l00@00000
09B3_@2DPkd000000`2FQKl020000000UhO10000001oP<X0000008:3c0000000Q8K=0P0000000`29
S=0000000000000S0?ooo`00?@3oool<000000D0FP0D00@0000009<P9@2C82D0Tb0U1@00000509LX
;`P000000P2oCcD300000080`UDn0P0000000`3JIAT000000=i^9@03000000030:J5/@0000000000
00D0YhNc00D00000091k^02@NkP0T7^h00000002099n^P040000002CP;`0Th2l00000080U8>m00P0
000009J5_`2FQKl0UXFo0000002GQl40000009R9`P80000000<0PX?<00000024Q/d00`0000000`2<
Sm4000000?ooo`0R0?ooo`00?03ooolG000000@0Tb0U1P00000209LX;`050000002i?al000000000
002lAb/00P0000030;m?=@80000000H0`UDn0000003EG0/0fVDI0000003NKRD3000000D0YhNc00H0
00000:N9]@2WRKD00000091k^0000002099n^P040000002CP;`0Th2l00000080U8>m00L0000009J5
_`2FQKl0UXFo0000002GQl4000000080V8W20`0000000`24Q/d0000008N9c`03000002@0oooo000k
0?ooo`D000001`1C00L;0000000308hG6@000000000000H000000P2C82D400000080^ClO1P000002
0;m?=@8000000P3EG0/2000000030=i^9@000000000000<0YhNc0P0000040:N9]@040000002BO[X0
TWjj00000080Th2l00@0000009B3_@2DPkd000000`2FQKl00`000000UhO10000000209R9`P050000
002IS<@0VHc400000024Q/d00P0000000`29S=0000000000000T0?ooo`00>P3ooolG0000000308T=
3@000000000000l000000`2i?al2000000030;a7:`00000000000080000000<0ce800000003EG0/0
1`0000040:N9]@050000002XRkH00000099n^P0000000P2CP;`010000000U8>m09B3_@00000309J5
_`030000002GQl4000000080V8W200D0000009V<a02IS<@0000009^>a@040000000308b?d@000000
oooo02<0oooo000i0?ooobd000000P2i?al3000000080;a7:`00000000000<mB00000000eE`;0000
003JIAT2000000@0YhVe0P0000030:R;]P040000002CP;`0Th2l00000080U8>m00L0000009J5_`2F
QKl0UXFo0000002GQl4000000080V8W200@0000009V<a02IS<@000000P2KS/D00`000000W9370000
0002000002D0oooo000h0?oooaL0000000<0L`000000000000005`0000020;To7`<0000000H0adL0
0<mB00000000eE`;0000003JIAT2000000030:N9]@000000000000<0Z8^f0`0000001@2CP;`00000
09B3_@2DPkd0000000<0UXFo00<0000009N7`@0000000P2HRL8010000000VHc409V<a000000209^>
a@060000002LT<L000000000002NULX000009@3oool003L0oooo=00000020;To7`80000000<0adL0
0000000000001@0000030:R;]P8000000`2XSKP010000000U8>m09B3_@00000309J5_`060000002G
Ql40UhO10000002HRL8000000P2IS<@01@000000Vhk509^>a@000000W9370080000000<0WYG:0000
003oool09@3oool003D0oooo>P0000000`2n>P00000000000002000000040=EL2`000000Z8^f0:R;
]P8000000`2XSKP30000000309B3_@000000UXFo0080UXFo00H0000009N7`@2GQl40000009R9`P00
000209V<a0050000002KS/D0Vhk50000002LT<L00P0000000`2NULX000000?ooo`0V0?ooo`00=03o
oolC000000@0h[ne5P000002078000d0000000H0_SX000000037A`00ce800000003EG0/2000000<0
Z8fh0P0000030:V?^P050000002FQKl0UXFo09J5_`0000000P2GQl4010000000V8W20000002IS<@2
00000080Vhk500H0000009b@a`000000000009jEbP00000X0?ooo`00<@3oool;000000@0l^Bn00<0
00000>71^03Q`KP0103Q`KP:000000<0cJNc00<020160000000000000P1a<g<01@000000WUQk09iH
N`2NF7/0000000<0_G9k100000000`1b000000000000000:000000030;hj00000000000000<00000
00<0Z8fh0000000000000`2YSkX01P000000ZY2k0:Z@^`000000UXFo00000080UhO100D0000009R9
`P000000VHc40000000309^>a@060000002LT<L000000000002NULX00000:@3oool002h0oooo1`00
00000`3Bo;P00000000000040?3U`P8000000P3bi;h00`000000hL6h0>71^00:0>71^0030000003=
Yk<0cJNc00@0cJNc0P0804H010000000LC=c074cL`00000309iHN`040000002mLW/0_G9k0;ebN`80
000000<0fheg000000000000400000000`0B89d00000000000030:V?^P030000002ZT;/0ZY2k0080
ZY2k0P00000209N7`@050000002HRL80000009V<a00000000P2KS/D20000000509b@a`0000000000
09jEbP000000:P3oool002`0oooo1`0000000`3Co/000000000000020>kUa@800000103`iL880000
00H0hL6h00@000000<fW/`3=Yk<0cJNc1000000200P0AP@000000`2NF7/010000000_G9k0;ebN`2m
LW/2000000030=^=M`000000mjEH01<0000000<0ZHnj0000000000000`2ZT;/2000000080:ZB_@00
0000UhO10000002HRL80000009V<a000000209^>a@030000002LT<L000000080000000<0WYG:0000
003oool0:P3oool002/0oooo1@000000103Cold000000000003Co/02000000@0k^G50`0000000`3`
iL8000000=o1^P060=o1^PX000000`3=ZKD00`00000020160000000306h^KP050000002NF7/0WUQk
09iHN`0000000`2mLW/01@000000fXUc000000000000mjEH008000000`2@IId?000000<0ZY2k0P00
00030:ZB_@80000000@0V8W20000002IS<@000000P2KS/D01`000000W9370000002MTlP0000009jE
bP000000;03oool002T0oooo1@0000000`3Bom8000000=?oc@0300000080k>G80P0000050>kUa@D0
00002P3O`KX00`000000cJVe0<fY]@040<fY]@030000000804H0000000<0KRi^0P00000209iHN`06
0000002mLW/0_G9k0;ebN`000000fXUc0P000000103gYEP000000:ekW@000004091UW@030000002A
J9l0000000X0000000<0ZY2k0000000000000`2ZT[d2000000050:^D_`000000000009V<a0000000
0P2KS/D01`000000W9370000002MTlP0000009jEbP000000;@3oool002L0oooo1`0000001@3Bom80
00000=?oc@000000j^G;008000000`3/iLP300000080k^G500<000000=k2_03N`[`00P3N`[`60000
00<0gl6j00<000000<fY]@3=ZKD00P3=ZKD40000000300P0AP000000KRi^0080KRi^00D0000009aD
M`000000WUQk000000030;ebN`080000003JRG<0fXUc0000003gYEP000000:ekW@000003091UW@05
0000002AJ9l0TFRO0000001UBih02P0000030:ZB_@80000000<0ZiBo0000000000000P2/U/002P00
0000Vhk509^>a@000000W9370000002MTlP0000009jEbP00000^0?ooo`009@3oool6000000070=3n
g000000000000=;odP000000doo=000000030>[Ub`800000103/iLP5000000P0g/:l200000020<bZ
]`030000000804H0000000<0KRi^00@0000009aDM`2LE7L0W5Ag0P0000040;ebN`030000003JRG<0
00000080000000D0[G^M0000002@IId0T6FM00000003095XW`030000001UBih0000000<0IdnR2@00
00000`2[U;l00000000000020:bF`0<0000000P0Vhk50000002LT<L0000009fCb0000000WYG:0000
02l0oooo000T0?ooo`D0000000H0c_gP000000000000dOoG0000003Bom82000000030>SUc@000000
000000<0j^G;0P0000020>cUb0030000003M`kh0gL>n0080gL>n1`0000030=k2_0030000003<Z[L0
c:Zg0080c:Zg100000000`0804H0000006h^KP0206h^KP030000002LE7L0W5Ag0080W5Ag00<00000
0;ebN`2mLW/00P2mLW/01@000000fXUc000000000000mjEH0080000000<0T6FM0000002AJ9l0102A
J9l00`000000IdnR06M?XP0206M?XP0306YCY@0000004R2M00L000000P2/U/0200000080[9S20P00
00001P2LT<L0000009fCb0000000WYG:00000300oooo000S0?ooo`<0000000@0b__V000000000000
c_gP0P000000103AomL000000=;odP0000040>SUc@8000000`3ZiL/5000000T0gL>n200000030<b[
^0030000000804H0000000<0KRi^00<0000009aDM`2LE7L00P2LE7L2000000<0_G9k00@000000=Z9
L`3JRG<0mJ1@0P0000000`2]Nid0000000000005095XW`080000001WCj80IdnR06M?XP000000JU>U
0000000B89d900000080[9S20P0000001`2]V/@000000000002MTlP0000009jEbP000000<@3oool0
02<0oooo0`000000103:nnH000000000003>oN02000000040=7oe`00000000000>KUd0800000103X
iLd2000000030>[Ub`000000g<C100<0g<C1200000030=g3_P030000003<ZkP0c:^h00H0c:^h0P00
000406h^KP030000002LE7L0W5Ag0080W5Ag00H000000;a_M`00000000000;ebN`0000020=Z9L`<0
000000<0[G^M0000002AJ9l0102AJ9l2000000<0IdnR00<0000006YCY@1ZDjD00`000002024`Z0L0
000000@0[I[4000000000000[Ic50P0000000`2NULX000000?ooo`0a0?ooo`008`3oool300000004
0<[kiP00000000000<kmh0<0000000<0i>GC0000000000000P3ViM02000000@0j>G=1@0000090=c4
`@`0000000<0000k0000001^;Vh00P1^;Vh00`000000W5Ag09aDM`0309aDM`040000002lKgL0_6mg
0;a_M`8000000P3JRG<00`000000mJ1@00000002000000@0TFRO00H00000099[XP2BJj80000006M?
XP00000406YCY@040000000J::<00000024`Z0P0000000@0[Ic5000000000000[Yk70P00000c0?oo
o`008`3oool3000000060<[kiP00000000000<kmh0000000h^GE0P0000020>CUd`8000000P3ViM02
00000080j>G=00<000000=_5``3KaL<00P3KaL<8000000<0g<C100<000000<b/^P3<[;X01P3<[;X0
0`000000000k000000020000000306h^KP000000W5Ag00<0W5Ag00<000000;a_M`2lKgL00`2lKgL0
20000000fXUc0=Z9L`000000mJ1@0000002]OIl000000`2AJ9l01`000000TV^R099[XP2BJj800000
06M?XP000000101ZDjD01P0000006RRS0000000Q<:P0000002Ph[@L0000000<0[Yk7000000000000
=@3oool00280oooo00D000000<Kgk000000000000<[kiP03000000040=oTf000000000000>;Ue@80
00000P3TiM<200000080i^G@0P00000=0=_5``h000000`1Z9fP2000000@0W5Ag00<000000;a_M`2l
KgL00`2lKgL2000000060=Z9L`000000mJ1@0000002]OIl000000`2AJ9l01`000000TV^R099[XP2B
Jj80000006M?XP0000000`1ZDjD01`000000KENX0000000000008C2X0000000X>:d02000000g0?oo
o`008P3oool01`000000a_O/000000000000b__V0000003Mi=X00P0000020=oTf0030000003RiMD0
h^GE008000000P3TiM<3000000@0f/K5200000040=_5``030000003;[[`0bjjl00L0bjjl00<0000k
0000001Z9fP00`1Z9fP300000080W5Ag00<000000;a_M`2lKgL00`2lKgL01P000000fXI^00000000
0000mJ1@00000080[GfO00<00000095XW`0000001@2BJj800`000000JU>U06YCY@0206YCY@070000
001]EjP000000000000Q<:P0000002Ph[@0:000003D0oooo000R0?ooo`0300000036mn`000000080
000000@0fnCL000000000000gNCJ0P0000000`3Oi=P00000000000020>;Ue@8000000P3TiM<00`00
0000f/K50=[6a@0:0=[6a@h0000000<0000k0000001Z9fP00P1Z9fP010000000Ve5c09]AL`2KDG<3
000000D0_6mg00P000000=Z6KP000000fXUc0000003eX50000000:emW`8000001P2BJj801@000000
JU>U06YCY@1ZDjD0000000<0KENX00H00000024`Z0000000:3R]0000000_?k4:000003<0oooo000R
0?ooo`0400000036mn`000000=STgP8000000P3Ki=`2000000050=gTfP000000gnCH0=oTf0000000
0`3RiMD2000000@0fLK7200000040=[6a@030000003;[kh0bjnn00L0bjnn00<000000000>`000000
0`1Z9fP00`000000Ve5c09]AL`0309]AL`030000002lKgL0_6mg00<0_6mg00<000000=Z6KP3JQVh0
0P000000103eX50000000:emW`000006099[XP8000000`1ZDjD02P000000KENX06eGZ01]EjP00000
024`Z0000000:3R]0000000_?k45000000030019b@000000000000@00000<@3oool00280oooo0P00
0000103Fhn0000000000003Hi=h200000080fnCL00D000000=gTfP00000000000=oTf00200000003
0>;Ue@000000f<O90080f<O91@0000050=W6aa000000101Z9fP00`000000Ve5c09]AL`0309]AL`<0
00000`2lKgL01P000000fXI^0=Z6KP3JQVh000000?FPD0<000001@2BJj801P000000TffT09=]Y000
0000JU>U000000<0KENX00T0000006mKZ`000000000002Ph[@000000;cna0000000eA[H01@000000
0`0GG=<0000000000004000002l0oooo000S0?ooo`8000000P3Fhn02000000040=STgP0000000000
0=_Tg080000000D0gNCJ000000000000gnCH000000090=S7b@H000000P3Ia/L00`000000b[300<Z`
`0070<Z``003000000000300000000@0JRMX00<0000009]AL`2KDG<00`2KDG<010000000^f]c0;][
L`0000020;a_M`090000003JQVh0fXI^0=Z6KP000000mJ1@0000002^OZ40000000@0TV^R00L00000
09=]Y02CKJ@0TffT0000001ZDjD0000000<0KENX00@0000006mKZ`00000000000P0X>:d200000003
03E6]P000000000000@0000000<05ecC02=Tf00]K=/01@00000]0?ooo`009@3oool200000080e^?P
00<000000=STgP3Hi=h00P0000001@3Ki=`000000000003Mi=X0000000<0elS;1P0000070=S7b@d0
000000H0000`000000000000000006XWJ000000609]AL`040000002kJg<0^f]c0;][L`<000000`3J
QVh3000000030:inX@000000TV^R0080TV^R00<0000009=]Y02CKJ@00`2CKJ@01@000000KENX06eG
Z01]EjP0000000<0Ke^[00<0000002Ph[@0X>:d00P0000000`0eA[H0000003]<^@050000000501ML
d`0SI=P0;FcK03MbgP10NN000`00000/0?ooo`009`3oool2000000030=KSh000000000000080f>CN
0P0000000`3Ki=`000000=K9c@020=K9c@H0000000<0elS;000000000000100000020=S7b@030000
003:/L80b[7200L0b[7200<000000=oDG00000000`1W8V83000000D0Ve5c00<000000;][L`2kJg<0
0`2kJg<010000000fXI^0=Z6KP3JQVh300000080[WjQ00<00000099[XP0000001P2CKJ@01@000000
KENX06eGZ01]EjP0000000<0Ke^[00<0LEj]0000000X>:d00P0000000`0eA[H0000003]<^@040000
000:0019b@000000000002=Tf0000000=g;N041ih01AQ>@0F8SU000002/0oooo000Y0?ooo`800000
00<0e^?P0000000000000P3Hi=h010000000e<[?0=C:c`3Db/l6000000030=K9c@000000000000@0
000000<0elS;0000000000002P0000000`000300gmAL0000000406LRHP040000002ICFl000000000
00<0Ve5c00<000000;][L`2kJg<00`2kJg<200000080fXI^00D000000?>JA`00000000000:inX@02
000000H0TffT0P00000206eGZ0050000001_Fj/0Ke^[06mKZ`0000000P1aGZd20000000602lo/@00
0000000003]<^@000000@E:m2P0000000`1AQ>@0F8SU0000000Z0?ooo`00:`3oool2000000030=KS
h0000000000000<0dl_A100000030=C:c`H0000000<0e/W=0000000000002`0000000`0003000000
0000000406LRHP040000002ICFl0VDe_09U=K`80000000<0Ve5c0000002kJg<0102kJg<00`000000
fH9X000000020=Z6KP<0000000<0[WjQ0000002CKJ@01@2CKJ@01P000000U6nV09A_YP000000KENX
000000<0Ke^[00H00000075N[@1aGZd0LEj]0000000_?k420000000303]<^@000000@E:m00`00000
00<0F8SU0000003oool0:03oool002d0oooo00@000000=;;d`3Bbm<0d/_C100000020=?;d@D00000
103Db/l>000000030000<0000000Ib9R00<0Ib9R00<0000009U=K`2ICFl0102ICFl2000000D0^f]c
00<000000=V2J00000000P3JQVh00`000000liY700000002000000D0TffT00@0000009A_YP2DKjH0
U6nV0P00000306mKZ`060000001aGZd0LEj]075N[@000000Lf:`0P0000001P0eA[H0000000000011
D[d0000004MH`0`00000:@3oool00300oooo100000020=;;d`D0000000<0dl_A0000000000000P00
00020=C:c`0300000038]LL0b;G700P0b;G700@0000T00000000000000000`1W8V800`000000VDe_
09U=K`0509U=K`<00000102kJg<02@000000fH9X0000003JQVh000000?>JA`000000[X2S00000003
09=]Y08000001@2DKjH01@000000Ke^[06mKZ`1_Fj/0000000<0LEj]00D0000007=R/00000000000
03E6]P0200000003045B_@000000AeS000<0000000<004W90000000000000P0000000`0gL]h00000
00000002000002P0oooo000e0?ooo`D0000000<0d/_C00000000000040000002000090060000001S
6e`000000000001W8V8000001`2ICFl00`000000^VQ_0;YXK`0200000080^f]c00<000000=V2J03I
PVP00P0000001`3cVTL000000:j0X`2^P:<0000009=]Y00000001P2DKjH200000080Ke^[00D00000
075N[@1aGZd0LEj]0000000307=R/0090000000eA[H0000000000011D[d0000004MH`0000000C5g3
00@0000000D05ecC0000000000000000041ih003000002L0oooo000j0?ooo`D0000000<0d/_C0000
000000002`0000000`0002@0000006<KG00306<KG0<000001@2ICFl00`000000^VQ_0;YXK`030;YX
K`<000000`3IPVP01@000000liY70?>JA`000000[X2S008000001P2DKjH01P000000UG:X09EbZ000
0000Ke^[000000<0LEj]00D0000007=R/01cH[00000007IU/P020000000803]<^@000000000004MH
`0000000C5g30000001AHlH40000000302=Tf0000000000000@000009P3oool003l0oooo3@000002
000090030000001S6e`0Ha]L00<0Ha]L00@0000009M8JP00000000000`2ICFl00`000000^VQ_0;YX
K`050;YXK`040000003IPVP0fH9X0=V2J080000000@0liY70000002^P:<000001P2DKjH010000000
UG:X09EbZ02ELZP2000000<0LEj]00H0000007=R/01cH[00Lf:`0000001fIK820000000803]<^@00
0000000004MH`0000000C5g30000001AHlH:0000000305R8i@000000oooo02@0oooo001;0?ooo`04
0000000002@0000T000000@0Ha]L00<0000009M8JP2GB6X00P2GB6X20000000309U=K`000000^VQ_
00H0^VQ_0P0000030=V2J0030000003cVTL0000000800000102DKjH2000000D0UG:X00D00000075N
[@1aGZd0LEj]0000000207=R/0080000001fIK80MVFb07IU/P000000>dbi00000011D[d200000005
04aM``000000DF?60000001FIlP02@0000000`1HR>D000000?ooo`0S0?ooo`00B`3oool00`000000
000T0000000506<KG0030000002GB6X0UdQZ00@0UdQZ0P0000070;YXK`030000003IOV<000000080
fH9X0`0000000`2^PZD0000009A_YP0209A_YP030000002ELZP0UG:X00@0UG:X0P000002075N[@05
0000001cH[00Lf:`07=R/00000000P1fIK801@000000N6Re000000000000@E:m0080000000L0C5g3
0000001AHlH0000005IWb0000000Ffc;00@0000000H0=g;N000000000000000005R8i@00000S0?oo
o`00BP3oool00`000000000G0000000300000080Ha]L00<0000009M8JP2GB6X01P2GB6X3000000D0
^VQ_00L000000=UnH`3IOV<000000=V2J0000000lI@m0080000000@0[X:U0000002DKjH000001P2E
LZP01P000000UWBZ09IdZP000000LEj]00000080Lf:`00H0000007IU/P1fIK80MVFb0000001hJ;D2
00000003045B_@000000AeS00080000000L0DF?60000001FIlP0000005]/b`000000Gg7=00@00000
00D0@7WP000000000000F8SU0000000R0?ooo`00BP3oool010000000000G00005`000002060DE@@0
00001`2GB6X00`000000^FAZ0;UTJP02000000<0^VQ_00@000000=UnH`3IOV<0fGiS0P000000103a
U3d000000000002^PZD2000000H0UG:X00@0000009IdZP2FM:X0UWBZ0P00000307=R/0040000001f
IK80MVFb000000<0N6Re00H00000045B_@000000AeS00000001<GL<20000000705IWb0000000Ffc;
0000001OLLd0000006Aec`0700000280oooo00190?ooo`040000000001L0000G000000D0H1AE00<0
000009I4I@0000000P00000409M8JP030000002iI6X0^FAZ0080^FAZ0P0000000`2jJ6l000000=Un
H`030=UnH`80000000@0lI@m0000002^PZD000001@2ELZP2000000D0UWBZ00@0000007=R/01cH[00
00000`1fIK801@000000N6Re07QX]@000000NVbg0080000000<0AeS00000001<GL<00P0000001`1F
IlP0000005]/b`000000Gg7=0000001TMLl01P0000000`1HR>D000000?ooo`0P0?ooo`00B@3oool3
00005`030000001P55D0H1AE00<0H1AE00<0000009I4I@2FA6D00P2FA6D30000000309M8JP000000
^FAZ00D0^FAZ0P0000050=UnH`030000003aU3d0000000800000102ELZP00`000000UWBZ09IdZP04
09IdZP8000000P1cH[0010000000MVFb07IU/P00000307QX]@090000001jK;L0NVbg00000017F<00
000004aM``000000DF?60080000000D0Ffc;0000001OLLd0000006Aec`020000000306amd`000000
00000080000000<0F8SU0000003oool07`3oool004P0oooo0P0000000`0001L00000060DE@04060D
E@030000002FA6D0UTAU00H0UTAU0`0000060;UTJP800000103IOV<300000080[hBW00@0000009Eb
Z02ELZP000001@2FM:X20000000409Mf[0000000Lf:`000000<0MVFb00@0000007QX]@1hJ;D00000
0P1jK;L01`000000O6ni000000000000C5g30000001AHlH00P0000001@1KK</0000005mac@000000
I7G?0080000000<0K7gC0000001`PM@01@00000O0?ooo`00B03oool200002P@000000`1P55D00`00
0000UTAU09I4I@0709I4I@040000002hHFD00000000000@0^FAZ00<000000=QjG@0000000`3IOV<0
20000000kXhc0?6D?@000000[hBW0000002ELZP000001@2FM:X00`000000UgJ/09Mf[00209Mf[080
00000P1fIK801@000000N6Re07QX]@1hJ;D000000080NVbg00T0000007a_^@1lKkT0000004aM``00
0000DF?60000001FIlP00P0000000`1OLLd0000006Aec`020000000306amd`000000L87D00<00000
00<0NhcH0000003oool07P3oool004L0oooo00@0000000002P0000X000000`1L3Th6000000H0UTAU
00<000000;QQI@2hHFD00P2hHFD2000000<0^FAZ00L000000=QjG@00000000000=UnH`000000kXhc
008000000P2_Q:L00`000000UWBZ09IdZP0309IdZP030000002GMZ`0UgJ/00<0UgJ/00D0000007IU
/P1fIK80MVFb0000000207QX]@040000001jK;L0NVbg00000080O6ni00T0000007ib^`0000000000
055SaP000000EVO80000001KK</00P0000001P1TMLl000000000001/OM<000000721e0<0000000<0
NhcH0000003oool07P3oool004L0oooo00<0000000002P0000001P1L3Th010000000U3mO09@oG`2D
?el400000080UTAU00<000000;QQI@2hHFD0102hHFD2000000030;UTJP000000f7YM0080f7YM00@0
00000=UnH`000000kXhc1000000409IdZP030000002GMZ`0UgJ/00<0UgJ/00D0000009Qi[P000000
MVFb0000000307QX]@040000001jK;L0NVbg00000080O6ni00/0000007ib^`1nL[/00000055SaP00
0000EVO80000001KK</0000005mac@030000000306amd`000000L87D00<0000000<0NhcH0000003o
ool07P3oool004H0oooo00@0000000002P0000X000001P1L3Th00`000000U3mO09@oG`0509@oG`<0
0000202hHFD2000000@0f7YM0P0000001@3^SS<000000:n6Z@2_QZT000000080UWBZ0P00000409Mf
[08000000P2HNJh00`000000MVFb0000000207QX]@040000001jK;L0NVbg00000080O6ni00D00000
07ib^`1nL[/00000081e_@020000000;05IWb0000000Ffc;0000001OLLd0000006Aec`000000K7gC
0000001`PM@00`0000000`1kS=P000000?ooo`0N0?ooo`00AP3oool;000000T0U3mO0`0000070;QQ
I@800000103HNUd010000000kXhc0>j><`0000020:n6Z@8000001@2GMZ`00`000000V7V^09Qi[P03
09Qi[P050000001hJ;D0N6Re07QX]@0000000P1jK;L010000000O6ni07a_^@00000207ib^`0;0000
0020MKd0P7Fm0000001FIlP0000005]/b`000000Gg7=0000001TMLl00`0000001P1`PM@000000000
00000000NhcH000001l0oooo00150?ooo`<000001`1H1TH00`000000TS]J00000002000000D0U3mO
00@000000;MLH02gG600]eaP0P0000050;QQI@040000003GMUH0000000000080f7YM0P0000001@3^
SS<000000:n6Z@2_QZT0000000D0UgJ/00<0000009Qi[P2HNJh00`2HNJh01@000000VG^`0000001h
J;D000000080NVbg00@0000007a_^@1lKkT000000P1nL[/01@000000P7Fm081e_@000000PWRo0080
000000D0Ffc;0000001OLLd0000006Aec`020000000306amd`1`PM@000000080000000<0NhcH0000
003oool07P3oool004@0oooo1000000605P6AP030000002B>eX0TS]J00<0TS]J100000000`2D?el0
00000;MLH0040;MLH08000000`2hHFD01@000000egIF0=MfEP3GMUH000000080f7YM0P000000103^
SS<0000000000000000409Mf[0030000002HNJh0V7V^0080V7V^0P00000209Uk/0030000001hJ;D0
00000080NVbg00<0000007a_^@0000000`1nL[/010000000P7Fm081e_@000002089h_`060000001K
K</0000005mac@000000I7G?0P0000000`1/OM<00000000000020000000307^<f0000000oooo01h0
oooo00140?ooo`H00000101H1TH00`000000TS]J098kFP07098kFP<000001P2gG602000000030;QQ
I@000000egIF00<0egIF00@000000=QjG@000000jhHW0P000000102_Qj/0000009Mf[02GMZ`20000
00@0V7V^00<0000009Uk/02INk000`2INk0010000000NVbg07Y/]`00000207a_^@040000001nL[/0
OW:k00000080P7Fm00D00000089h_`22N;l0000008Ak`@020000000305mac@000000I7G?00800000
00L0K7gC000000000000M8GF0000001kS=P0000001l0oooo00130?ooo`<00000101C03h7000000P0
TS]J00<000000;IHFP0000000P0000050;MLH08000001@3GMUH4000000030>j><`2_Qj/0[hN[0080
00001@2HNJh00`000000VG^`09Uk/00309Uk/0070000002JOK80000007Y/]`000000O6ni00000002
07ib^`0500000020MKd0P7Fm081e_@0000000P22N;l010000000Q7_100000026O/<20000000406Ae
c`000000000006amd`80000000D0M8GF000000000000NhcH0000000N0?ooo`00@P3oool4000000L0
D`0n00@00000090fE02@=U@0T3ID1P000002098kFP030000002fF5X0]UQJ0080]UQJ0`0000030;ML
H0<000001@3GMUH00`000000jhHW00000003000000D0V7V^00<0000009Uk/02INk000P2INk020000
00<0VWfb00@0000007a_^@1lKkT000000P1nL[/010000000P7Fm081e_@000002089h_`0400000024
Nl40Q7_100000080QWk300D0000006Aec`000000000006amd`020000000507B5eP000000000007^<
f00000007P3oool00480oooo0`00000805<0?P030000002@=U@0T3ID00L0T3ID100000060;IHFP<0
000000@0]eaP0000003FLDl00000103GMUH2000000040>^69`000000/8V]000000@0V7V^00<00000
09Uk/02INk000P2INk000`000000VWfb09Ym/P0209Ym/P0409]o]0000000O6ni00000080OW:k00@0
0000081e_@20MKd000000P22N;l010000000Q7_108Ak`@00000208In``0300000028PLD000000080
000000@0K7gC000000000000M8GF0P0000000`1kS=P000000?ooo`0M0?ooo`00@@3oool>000000X0
T3ID00<000000;ADE00000000P0000060;IHFP8000000P3FLDl200000080egIF00@000000>Mn6`3[
QRL0jhHW0P00000209Qi[P800000102INk001@000000VWfb09Ym/P2JOK80000000<0Vgnd0P000002
07ib^`0400000020MKd0P7Fm00000080PWRo00@0000008Ak`@24Nl4000000P26O/<208R1a@040000
002:Q<H0000006amd`80000000D0M8GF000000000000NhcH0000000N0?ooo`00@@3oool3000000L0
C`0e00@0000008d`C@2=<4d0SC1=1P000003090fE0030000002dE5@0]5AD00<0]5AD0`0000040;IH
FP<000000P3FLDl00`000000egIF0=MfEP02000000030>^69`000000/8V]00800000102INk020000
00<0VWfb00<0000009]o]02KOk@00P2KOk@010000000OW:k07ib^`000002081e_@0400000022N;l0
PWRo00000080Q7_100@0000008In``26O/<000000P28PLD00`2:Q<H0000008b6b0030000000507B5
eP000000000007^<f00000007P3oool00400oooo1000000304l0=@P000001P2=<4d5000000L0]5AD
0`0000001@2fF5X000000=A]A`3DKDL000000080eW5?0`0000000`3WOQ/000000>^69`02000000@0
VG^`00<0000009Ym/P2JOK800P2JOK801@000000Vgnd09]o]02KOk@000000080W86f00<0000007ib
^`0000000P20MKd00`000000PWRo0000000208Ak`@0400000026O/<0QWk300000080R87500P00000
08Z4aP2<Q/P0S8K80000002>RLX0000007B5eP80000000<0NhcH0000003oool07@3oool00400oooo
1`00000304T0;0030000002;:TH0RbY60080RbY61P00000308d`C@030000002cCdd0000000<00000
1P2dE5@2000000<0e6e700D000000=IaC`3FLDl0eW5?000000020>Mn6`<000000`2INk0010000000
VWfb09Ym/P2JOK82000000<0Vgnd00<0000009b1]P2LPKH00P2LPKH010000000P7Fm081e_@000002
089h_`0400000024Nl40Q7_100000080QWk300D0000008R1a@000000RXC60000000208b6b00308j9
bP000000Shc<00<0000000<0NhcH0000003oool07@3oool003l0oooo0`00000804T0;0030000002;
:TH0RbY600P0RbY61@0000040;=?C@<00000102dE5@00`000000dVLo000000020=A]A`80000000@0
eW5?000000000000ighK0P000000102`Rjl0000000000000000309Ym/P040000002KOk@0Vgnd09]o
]08000000`2LPKH020000000W8Bh09b4^0000000P7Fm00000022N;l000000P24Nl4010000000QWk3
08In``00000208R1a@8000000P2<Q/P01P000000SXW:08n<c02?S<`00000096?c@<000007P3oool0
03h0oooo4P00000708/ZAP030000002aBDH0/DU600<00000102cCdd4000000040;ADE0000000dVLo
000000<0e6e700D000000=IaC`00000000000>Mn6`03000000@0VWfb00D0000009]o]02KOk@0Vgnd
0000000409b1]P030000002LQ;P0W8Bh0080W8Bh00<00000089h_`0000000P24Nl4010000000QWk3
08In``00000208R1a@070000002:Q<H0000008b6b0000000SXW:0000000208n<c080000000<0Ti7>
0000003oool07P3oool003d0oooo0`00000804@08@030000002893h0R2@n00D0R2@n2@0000040;59
AP8000001P2cCdd200000080dVLo0P0000020=A]A`80000000<0hWH>0000003WOQ/00P00000209Ym
/P8000000`2KOk@00`000000W86f09b1]P0209b1]P090000002LQ;P0W8Bh09b4^0000000WHJj0000
0022N;l000000080Q7_100<0000008In``0000000P28PLD01`000000RXC60000002<Q/P0000008j9
bP0000000P2?S<`01@000000THo=0000002CTLh0000001l0oooo000l0?ooo`@000001014024<0000
00L0R2@n00<000000:m4?P0000000P0000030;59AP<00000102cCdd00`000000d64f000000020=9W
?`8000000P3DKDL2000000030>Mn6`000000/8f`008000000`2KOk@2000000<0W86f0P00000309b4
^0050000002MQ[X0WHJj09f6^P0000000P24Nl4020000000QWk308In``000000R8750000002:Q<H0
00000P2<Q/P02@000000SXW:0000002?S<`00000096?c@000000Ti7>0000000P0?ooo`00>`3oool9
000000<0?P0G00<0000008DM=P257CH01P257CH9000000<0[d@n0`0000030;59AP<0000000@0/dm=
0000003@HCH0d64f0P0000020=9W?`030000003DKDL000000080000000<0ighK0000000000000`2K
Ok@00`000000W86f09b1]P0209b1]P040000002LQ;P0W8Bh09b4^08000000P2MQ[X20000000609j8
^`000000Q7_100000026O/<000000P28PLD03@000000RXC60000002<Q/P0000008j9bP000000Shc<
0000002ASld0000009>AcP0000008@3oool003/0oooo0`00000403h05`d000001`257CH00`000000
[3df0:`m=P03000000<0[d@n100000020;59AP800000103@HCH00`000000dVLo0=9W?`0700000003
09]o]0000000000000<0W86f0P00000309b4^0050000002MQ[X0WHJj09f6^P000000102NR;/00`00
0000QWk30000000208R1a@070000002:Q<H0000008b6b0000000SXW:0000000208n<c080000000<0
Ti7>0000003oool08@3oool003X0oooo200000000`0h00/0000000000005000000D0PAH]2@000004
0:`m=P<00000102_A3h4000000030<eJ;0000000d64f0080d64f0P0000020=9W?`030000003LK000
000000<00000102LPKH010000000W8Bh09b4^02LQ;P2000000<0WHJj00@0000009j8^`2NR;/0WXRk
0`0000001026O/<0000008R1a@00000208Z4aP0;0000002<Q/P0000008j9bP000000Shc<0000002A
Sld0000009>AcP0000008P3oool003T0oooo0`0000000`0h00/00000000000040000000303400000
0000O@hS00@0O@hS20000004084F;@030000002Y=R`0000000<000000`2/?CH4000000<0[d@n0P00
00020<eJ;0040000003@HCH0d64f0=1Q=P80000000@0dVLo000000000000hWH>0P00000309b1]P05
0000002LQ;P0W8Bh09b4^00000000`2MQ[X2000000<0WXRk00D0000009n:_@2OR[d0WhZm00000002
08R1a@0=0000002:Q<H0000008b6b0000000SXW:0000002?S<`00000096?c@000000Ti7>0000000S
0?ooo`00>03oool500000080<@003P00000507d>8`L000000`2Y=R`3000000@0[3df0`0000001@2_
A3h000000<YC8@000000cEX/008000000`3@HCH5000000040;6@]0000000000000000080W8Bh0P00
000309f6^P040000002NR;/0WXRk09j8^`8000000P2OR[d2000000070:2<_`000000000008Z4aP00
0000S8K80000000208j9bP060000002?S<`000000000002CTLh00000903oool003L0oooo20000000
0`0Z000000000000000:0000000307P560000000000000T0000000<0YRlR0000000000000P000002
0:Tf;0@000000`2/?CH300000080bU<Q00@000000<eJ;000000000000`3@HCH5000000<0W8Bh00<0
000009f6^P2MQ[X00P2MQ[X01@000000WXRk09j8^`2NR;/0000000<0WhZm00<000000:2<_`2PS;l0
0`0000000`2:Q<H000000000000208j9bP040000002?S<`00000096?c@8000009@3oool003H0oooo
0P00000202X00140000000<0L`0=0000000000002@0000000`2R9QL0000000000008000000030:Tf
;0000000000000<0000000@0aT/E000000000000bU<Q0P000000103=FR`000000000003@HCH50000
000309b4^0000000000000<0WHJj0P00000309j8^`030000002OR[d0WhZm008000000P2PS;l03`00
0000XHk10:6>`@2QS/40000008b6b0000000SXW:0000002?S<`00000096?c@000000Ti7>0000000U
0?ooo`00=@3ooolE0000000306h000000000000000h000000P2R9QL9000000050:Tf;00000000000
00000036BaD00P000000103:Db4000000000003=FR`4000000030=a/00000000000000@0WHJj00@0
000009j8^`2NR;/0WXRk0P0000000`2OR[d00000000000020:2<_`8000000P2QS/42000000040::@
`P000000SXW:08j9bP80000000@0THo=0000002CTLh000009P3oool003@0oooo5@0000000`1W0000
00000000000C000000<0XRHG2P0000001036BaD000000000003:Db42000000030<eJ;0000000c5H0
00<000000`2MQ[X01@000000WXRk09j8^`2NR;/000000080WhZm00D000000:2<_`2PS;l0X8bo0000
00030:6>`@050000002RT<8000000000002STl@00P0000001@2?S<`00000096?c@2CTLh0000002L0
oooo000b0?ooo`D000000P0A000M00000003094800000000000000X00000102R9QL7000000040<I;
5@00000000000<YC8@<0000000<0c5H00000000000000`00000209j8^`80000000<0WhZm00000000
00000P2PS;l200000080XHk10P000000102RT<8000000:>Ca02STl@2000000050:BEaP000000THo=
0000002CTLh0:03oool00340oooo00<000000140000A0000:00000000`2A20000000000000080000
0080XRHG00<000000;D]00000000100000001036BaD000000000003:Db42000000030<aF00000000
000000<0WXRk00@0000009n:_@2OR[d000000P2PS;l2000000<0XHk100@000000::@`P0000000000
0P2STl@020000000Y9G6000000000000Y9O70000002CTLh00000:03oool00300oooo>`0000020;D]
00X0000000H0c5H00000002NR;/0WXRk0000002OR[d200000080X8bo00@000000:6>`@2QS/40XHk1
0P000000102RT<8000000:>Ca02STl@2000000030:BEaP000000Y9O70080000000<0YIW90000003o
ool0:03oool002d0oooo400000060>_D_@030000003G]KL0ekFg00`0ekFg6`0000020;D]00<00000
00<0`T880000000000000`0000001@3<EP00000009n:_@2OR[d000000080X8bo0P0000020:6>`@80
000000@0XY32000000000000Xi?40P0000001P2TULH000000000002TUlL000000:FIb@800000:P3o
ool002`0oooo200000000`3Yn/H00000000000060>WE`1P000001034XKDC0000000309T700000000
00000080]Bd00`0000000`32@PP0000000000004000000<0X8bo00H000000:6>`@2QS/40XHk10000
002RT<8200000080Xi?400D000000:BEaP00000000000:BGa`02000000030:FIb@000000oooo02/0
oooo000Y0?ooo`L000000P3Wn/X010000000jO[60>WjaP3Yn/H3000000T0jMG000<000000=Ng^@3G
]kT03@3G]kT010000000a:6e0<BQ]@34XKD6000000<0a::g3`0000001@2I1`000000000000000000
]Bd000<0000000<0`T880000000000000P0000020:2<_`030000002QS/40XHk10080000000@0XY32
0000002STl@0Xi?40P0000000`2TULH000000:BGa`02000000030:FIb@000000000002d0oooo000V
0?ooo`P0000000<0iO[>0000003Wn/X00P3Wn/X00`000000jO[6000000030>SF``H000000P3YeL00
0`000000ekNi0=Ng^@060=Ng^@`000002034X[L500000080a:Bi2`0000001@2I1`00000000000000
0000]Bd000@0000000<0WB400000000000000`2QS/4010000000XY320000000000020:>Ca0070000
002TULH000000000002TUlL000000:FIb@02000002l0oooo000T0?ooo`P0000000D0ho[B0000003U
n/h0iO[>000000030>OjbP<00000203Xe/<;000000P0e[Rk00<000000<BR]`34X[L01034X[L50000
00L0a:Bi00<000000;:C^00000000P0000000`2bUKT000000000000=0000000709dQ000000000000
0:6>`@000000XY32000000020:>Ca080000000<0Y9G60000002TUlL00P0000000`2UVLT000000?oo
o`0`0?ooo`008`3oool7000000050>7jeP000000ho[B0>?jdP0000000P3Un/h00`000000io[:0000
00030>OGaPH000000`3Xe/<00`000000e[Rk0=Jh^`0>0=Jh^`0400000034X[L0a::g0<BR]`@00000
2@34Y;T5000000<0/YFi0`0000020;:G^``0000000H0WB40000000000000XY320000002STl@20000
00060:BEaP00000000000:BGa`000000YIW90P00000b0?ooo`008P3oool6000000050=kif@000000
hO[F0>7jeP0000000P3Sn]800`000000iO[>0>GjcP03000000P0imO65`0000070<BT^@L000000`34
YK/00`000000/YFi0;:E^@0200000080/YNk0`0000020;>I_@/0000000D0WB40000000000000Xi?4
000000020:BEaP060000002TUlL000000000002UVLT00000=03oool00240oooo00<0000009kehP00
00000`0000030=kif@030000003Qn]H0000000<0ho[B0`0000020>GGb0H00000103WelH00`000000
e[Vm0=Ji_@0>0=Ji_@0400000034Y;T0a:Bi0<BT^@L000001`34YK/5000000<0/YNk0P0000030;>I
_@<000000P2cVkl<000000060:BEaP000000Y9O70:BGa`000000YIW90P00000e0?ooo`00803oool0
20000000W_GR000000000000fOSO0000003Kn=`000000`3NnMT01@000000hO[F0000003Sn]800000
00<0hmS;100000040>GGb1T000001`34YK/7000000<0a:Nl00@000000;:G^`2bUk/000000`2cVKd3
000000<0/i^o0`0000000`2cW<40000000000009000000050:BGa`000000YIW90:FIb@000000=`3o
ool001l0oooo00X0000009kehP00000000000=KghP000000fOSO0000003Kn=`000000`3NnMT00`00
0000hO[F00000002000000H0hmS;100000020>GGb0030000003E^[l0eKZo0100eKZo00<000000<BU
^`34YK/00P34YK/6000000L0a:Nl1@0000030;>I_@8000000P2cVkl4000000040;>L`@0000000000
000000<0]9k22@0000000`2UVLT000000?ooo`0h0?ooo`007P3oool3000000090=?fi@000000e_OR
0=KghP000000fOSO0000003Kn=`0000000<0g_WI00@000000>;Ic@3RfLd0h]W=100000060>?HbaT0
00001`34Yk`7000000<0`jRn00<000000;>I_@2cVKd00P000000102cVkl000000000000000030;>L
`@8000000P2dW/84000000030;BPa0000000000000L00000>P3oool001l0oooo00L000000=3ei`00
0000doKU0000003Fmn8000000080fOSO00D000000=_hg0000000g_WI000000020>3Id0@000000P3R
fLd500000080hmS;00<000000=Bk`P3D^l80403D^l800`000000a:Nl0<BW_0020<BW_0L000001`33
Z;h500000080/i^o0P0000030;>L`@<0000000@0]9k200000000000000000P2dX<@3000000050;BR
aP00000000000000002eY<P01@00000i0?ooo`00803oool01`000000d?GW0000003Cm^D000000=Kg
hP0000000P3In=l00`000000foSL000000070>3Id0<00000103RfLdI000000P0`jRn1`0000020<>Z
`0@000000`2cW<43000000040;BN`P000000000000000080]:340`0000001@2dX/H0000000000000
0000]JC800<000000P3oool6000003H0oooo000Q0?ooo`070000003@mNL000000=?fi@000000e_OR
000000020=Whg`040000003Nf]80g][B0=kJdPD000000`3PfM04000000030>;Ic@000000dkc40180
dkc400<000000<>X_P33Z;h00P33Z;h7000000<0`j[01P0000020<>[`P030000002cW<40/ic10080
000000@0]9k200000000000000000P2dX<@3000000030;BRaP000000000000D000001`3oool60000
03@0oooo000Q0?ooo`070000003@mNL0d?GW0000003Cm^D000000=KghP0200000080gM[D10000003
0=kJdPD000000P3PfM0I000000D0`j[01P0000050<>[`PD0000000<0]9k20000000000000`2dX<@:
000000`0oooo1P00000c0?ooo`008P3oool01P000000d?GW0=3ei`000000doKU00000080fm_G1000
00020=gJe0<000001@3Nf]8<000000X0dkk600<000000<>Z`033Z/000P33Z/05000000<0`j_23@00
00040;BPa0D0000000@0]JC80000000000000000403oool30000000401=_h0000000000000000340
oooo000S0?ooo`040000003@mNL0d?GW00000080fM_I100000000`3KfmL000000000000200000003
0=gJe00000000000008000000P3Nf]800`000000d[o80=:ob0080=:ob0h0000000<0`j_200000000
00001P0000020<>]a0H000000`33[lH00`000000]:340;BPa002000000040;BRaP00000000000;FT
b0<00000503oool700000300oooo000T0?ooo`8000000P3Gg=/4000000030=WKf@00000000000080
000000<0fm_G0000000000000P0000000`3Mf]@000000000000J000000@0`jg41P0000050<>_aPH0
000000@0]:;6000000000000]JC80`00000H0?ooo`L00000;`3oool002L0oooo0`0000020=OLf`@0
000000<0fM_I0000000000000P0000000`3KfmL000000000000:000000/0dL3:00<000000<>]a033
[L@00P33[L@5000000<0`jo63P0000000`2eY<P000000000000N0?ooo`L00000;@3oool002/0oooo
100000000`3Gg=/0000000000002000000030=WKf@000000000000/0d<;<3`0000000`33[lH00000
0000000D00000280oooo1`00000/0?ooo`00;`3oool4000000030=OLf`000000000000`0cl?>4000
00000`33/<P000000000000=000002T0oooo1`00000[0?ooo`00<`3oool3000000/0c/C@40000002
0<:bbPL00000<`3oool6000002X0oooo00110?oooa000000?@3oool7000002P0oooo002?0?ooo`@0
000000<04foP0000000000009`3oool00940oooo1P00000V0?ooo`00U03oool5000002@0oooo002F
0?ooo`@000008`3oool009T0oooo0P00000R0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool0
0;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3o
ool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00
_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?oo
o`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m
0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo
002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00_@3oool00;d0oooo002m0?ooo`00\
\>"],
  ImageRangeCache->{{{0, 188.563}, {330.875, 0}} -> {0.276926, 0.0320663, \
0.00232596, 0.00292934}},
  CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Working with data", "Subsection",
  CellTags->{"b:0.3", "IPM3.1.1.4", "c:6"}],

Cell[TextData[{
  "The ability to plot and visualize data is extremely important in \
engineering and all of the social, natural, and physical sciences. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can import and export data from other applications, plot the data in a \
variety of forms, and be used to perform numerical analysis on the data."
}], "TextFirst",
  CellTags->{"i:12", "b:0.3", "IPM3.1.1.4"}],

Cell[TextData[{
  "The file ",
  StyleBox["dataset.m", "MR"],
  " contains pairs of data points, in this case representing body mass",
  " vs. ",
  "heat production for 13 different animals. The data are given as (",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  "), where ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " represents the mass of the animal and ",
  Cell[BoxData[
      \(TraditionalForm\`r\)]],
  " the heat production in ",
  StyleBox["kcal",
    FontSlant->"Italic"],
  " per day. First we set up a platform independent path to the file and then \
import that file."
}], "Text",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(datafile = 
      ToFileName[{$BaseDirectory, "\<Applications\>", "\<IPM3\>", \
"\<DataFiles\>"}, "\<dataset.m\>"]\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}],

Cell[BoxData[
    \("C:\\Documents and Settings\\All Users\\Application \
Data\\Mathematica\\Applications\\IPM3\\DataFiles\\dataset.m"\)], "Output",
  CellLabel->"Out[23]=",
  CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(data = Import[datafile, "\<Table\>"]\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}],

Cell[BoxData[
    \({{0.06099`, 6.95099`}, {0.403`, 28.189`}, {0.62199`, 41.1`}, {2.50999`, 
        120.799`}, {2.95999`, 147.9`}, {3.33`, 182.8`}, {8.19999`, 
        368.8`}, {28.1999`, 981.299`}, {57.4`, 1303.29`}, {72.2999`, 
        1512.5`}, {340.199`, 7100.29`}, {711, 10101.1`}, {5000.`, 
        29894.9`}}\)], "Output",
  CellLabel->"Out[24]=",
  CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}]
}, Open  ]],

Cell[TextData[{
  "You can immediately plot the data using the ",
  StyleBox["ListPlot", "MR"],
  " function."
}], "TextFirst",
  CellTags->{"i:15", "b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[data, PlotStyle \[Rule] PointSize[ .02]];\)\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.0238095 0.00112102 0.0147151 3.3275e-005 [
[.24801 .00222 -9.5 -11.875 ]
[.24801 .00222 9.5 0 ]
[.47222 .00222 -9.5 -11.875 ]
[.47222 .00222 9.5 0 ]
[.69642 .00222 -9.5 -11.875 ]
[.69642 .00222 9.5 0 ]
[.92063 .00222 -9.5 -11.875 ]
[.92063 .00222 9.5 0 ]
[.01131 .0979 -24 -5.9375 ]
[.01131 .0979 0 5.9375 ]
[.01131 .18109 -24 -5.9375 ]
[.01131 .18109 0 5.9375 ]
[.01131 .26428 -24 -5.9375 ]
[.01131 .26428 0 5.9375 ]
[.01131 .34746 -29 -5.9375 ]
[.01131 .34746 0 5.9375 ]
[.01131 .43065 -29 -5.9375 ]
[.01131 .43065 0 5.9375 ]
[.01131 .51384 -29 -5.9375 ]
[.01131 .51384 0 5.9375 ]
[.01131 .59703 -29 -5.9375 ]
[.01131 .59703 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.24801 .01472 m
.24801 .02097 L
s
gsave
.24801 .00222 -70.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(200) show
1.000 setlinewidth
grestore
.47222 .01472 m
.47222 .02097 L
s
gsave
.47222 .00222 -70.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(400) show
1.000 setlinewidth
grestore
.69642 .01472 m
.69642 .02097 L
s
gsave
.69642 .00222 -70.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(600) show
1.000 setlinewidth
grestore
.92063 .01472 m
.92063 .02097 L
s
gsave
.92063 .00222 -70.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(800) show
1.000 setlinewidth
grestore
.125 Mabswid
.07986 .01472 m
.07986 .01847 L
s
.13591 .01472 m
.13591 .01847 L
s
.19196 .01472 m
.19196 .01847 L
s
.30407 .01472 m
.30407 .01847 L
s
.36012 .01472 m
.36012 .01847 L
s
.41617 .01472 m
.41617 .01847 L
s
.52827 .01472 m
.52827 .01847 L
s
.58432 .01472 m
.58432 .01847 L
s
.64037 .01472 m
.64037 .01847 L
s
.75247 .01472 m
.75247 .01847 L
s
.80853 .01472 m
.80853 .01847 L
s
.86458 .01472 m
.86458 .01847 L
s
.97668 .01472 m
.97668 .01847 L
s
.25 Mabswid
0 .01472 m
1 .01472 L
s
.02381 .0979 m
.03006 .0979 L
s
gsave
.01131 .0979 -85 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2500) show
1.000 setlinewidth
grestore
.02381 .18109 m
.03006 .18109 L
s
gsave
.01131 .18109 -85 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5000) show
1.000 setlinewidth
grestore
.02381 .26428 m
.03006 .26428 L
s
gsave
.01131 .26428 -85 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(7500) show
1.000 setlinewidth
grestore
.02381 .34746 m
.03006 .34746 L
s
gsave
.01131 .34746 -90 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10000) show
1.000 setlinewidth
grestore
.02381 .43065 m
.03006 .43065 L
s
gsave
.01131 .43065 -90 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(12500) show
1.000 setlinewidth
grestore
.02381 .51384 m
.03006 .51384 L
s
gsave
.01131 .51384 -90 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(15000) show
1.000 setlinewidth
grestore
.02381 .59703 m
.03006 .59703 L
s
gsave
.01131 .59703 -90 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(17500) show
1.000 setlinewidth
grestore
.125 Mabswid
.02381 .03135 m
.02756 .03135 L
s
.02381 .04799 m
.02756 .04799 L
s
.02381 .06463 m
.02756 .06463 L
s
.02381 .08127 m
.02756 .08127 L
s
.02381 .11454 m
.02756 .11454 L
s
.02381 .13118 m
.02756 .13118 L
s
.02381 .14782 m
.02756 .14782 L
s
.02381 .16445 m
.02756 .16445 L
s
.02381 .19773 m
.02756 .19773 L
s
.02381 .21437 m
.02756 .21437 L
s
.02381 .231 m
.02756 .231 L
s
.02381 .24764 m
.02756 .24764 L
s
.02381 .28092 m
.02756 .28092 L
s
.02381 .29755 m
.02756 .29755 L
s
.02381 .31419 m
.02756 .31419 L
s
.02381 .33083 m
.02756 .33083 L
s
.02381 .3641 m
.02756 .3641 L
s
.02381 .38074 m
.02756 .38074 L
s
.02381 .39738 m
.02756 .39738 L
s
.02381 .41401 m
.02756 .41401 L
s
.02381 .44729 m
.02756 .44729 L
s
.02381 .46393 m
.02756 .46393 L
s
.02381 .48056 m
.02756 .48056 L
s
.02381 .4972 m
.02756 .4972 L
s
.02381 .53048 m
.02756 .53048 L
s
.02381 .54711 m
.02756 .54711 L
s
.02381 .56375 m
.02756 .56375 L
s
.02381 .58039 m
.02756 .58039 L
s
.02381 .61366 m
.02756 .61366 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.02 w
.02388 .01495 Mdot
.02426 .01565 Mdot
.02451 .01608 Mdot
.02662 .01873 Mdot
.02713 .01964 Mdot
.02754 .0208 Mdot
.033 .02699 Mdot
.05542 .04737 Mdot
.08816 .05808 Mdot
.10486 .06504 Mdot
.40518 .25098 Mdot
.82086 .35083 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[25]:=",
  ImageSize->{309.75, 191.188},
  ImageMargins->{{39.625, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004e0000_b000`400?l00000o`00003oo`3ooolf
0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`00o`3ooolf
0?ooo`00F03oool4000000040?ooo`000000000000000080oooo0`00000e0?ooo`030000003oool0
0000008000000P3oool3000003@0oooo0P0000020?ooo`<000000P3oool3000003<0oooo0P000002
0?ooo`<000000P3oool300000180oooo001I0?ooo`050000003oool0oooo0?ooo`0000000P3oool0
0`000000oooo000000020?ooo`030000003oool0oooo0380oooo00<000000?ooo`0000000P3oool0
0`000000oooo000000020?ooo`030000003oool0oooo0300oooo00H000000?ooo`3oool000000?oo
o`0000020?ooo`030000003oool000000080oooo00<000000?ooo`3oool0;`3oool01P000000oooo
0?ooo`000000oooo00000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`0?0?oo
o`00FP3oool010000000oooo0?ooo`0000020?ooo`030000003oool000000080oooo00<000000?oo
o`3oool0;`3oool600000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`0`0?oo
o`060000003oool0oooo0000003oool000000P3oool00`000000oooo000000020?ooo`030000003o
ool0oooo02l0oooo00H000000?ooo`3oool000000?ooo`0000020?ooo`030000003oool000000080
oooo00<000000?ooo`3oool03`3oool005X0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo000000020?ooo`030000003oool0oooo02l0oooo00H000000?ooo`3oool000000?ooo`000002
0?ooo`030000003oool000000080oooo00<000000?ooo`3oool0<03oool01P000000oooo0?ooo`00
0000oooo00000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`0`0?ooo`800000
0P3oool01P000000oooo0?ooo`000000oooo00000080oooo00<000000?ooo`3oool03`3oool005/0
oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?ooo`030000003oool0oooo0300
oooo00D000000?ooo`000000oooo000000020?ooo`030000003oool000000080oooo00<000000?oo
o`3oool0<03oool300000080oooo00H000000?ooo`3oool000000?ooo`0000020?ooo`030000003o
ool0oooo02l0oooo00H000000?ooo`3oool000000?ooo`0000020?ooo`030000003oool000000080
oooo00<000000?ooo`3oool03`3oool005P0oooo00H000000?ooo`3oool000000?ooo`0000020?oo
o`030000003oool000000080oooo00<000000?ooo`3oool0<@3oool2000000070?ooo`000000oooo
0?ooo`000000oooo000000020?ooo`030000003oool0oooo0300oooo00H000000?ooo`3oool00000
0?ooo`0000020?ooo`030000003oool000000080oooo00<000000?ooo`3oool0;`3oool01P000000
oooo0?ooo`000000oooo00000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`0?
0?ooo`00F@3oool2000000<0oooo0P0000030?ooo`800000=@3oool00`000000oooo0?ooo`020000
00<0oooo0P00000d0?ooo`8000000`3oool2000000<0oooo0P00000c0?ooo`8000000`3oool20000
00<0oooo0P00000B0?ooo`00o`3ooolf0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo
000P0?ooo`030000003oool0oooo0?l0oooo4`3oool001l0oooo0`00003o0?oooa@0oooo000N0?oo
o`D00000o`3ooolC0?ooo`006@3ooooo000001d00000000N0?ooo`H00000303oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0303oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0=0?ooo`030000003oool0oooo00@0oooo000O0?ooo`D00000>`3oool00`000000oooo0?oo
o`0m0?ooo`030000003oool0oooo03`0oooo00<000000?ooo`3oool0?@3oool00`000000oooo0?oo
o`0D0?ooo`007`3oool600000?l0oooo4@3oool00200oooo1@00003o0?oooa40oooo000P0?ooo`D0
0000o`3ooolA0?ooo`00803oool400000?l0oooo4P3oool00200oooo00<000000?ooo`3oool01@3o
ool300000?l0oooo2`3oool00200oooo00<000000?ooo`3oool0103oool500000?l0oooo2P3oool0
0200oooo0P0000050?ooo`D00000o`3oool:0?ooo`00803oool00`000000oooo0?ooo`040?ooo`D0
00001@3oool300000?l0oooo0P3oool00200oooo00<000000?ooo`3oool01@3oool3000000D0oooo
1@00003o0?ooo`40oooo000P0?ooo`030000003oool0oooo00d0oooo1@000000103oool000000000
0000003l0?ooo`00803oool00`000000oooo0?ooo`0=0?ooo`X00000n`3oool00200oooo0P00000?
0?ooo`<0000000<0oooo0000000000000`00003k0?ooo`00803oool00`000000oooo0?ooo`0B0?oo
o`D00000n`3oool00200oooo00<000000?ooo`3oool04`3oool300000?`0oooo000P0?ooo`030000
003oool0oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool2
00000?l0oooo503oool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`001`3oool400000080
oooo0P0000020?ooo`<000000P3oool3000000L0oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00
203oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`0000000P3oool00`000000
oooo000000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00
2@3oool00`000000oooo0?ooo`030?ooo`030000003oool000000080oooo00<000000?ooo`000000
0P3oool00`000000oooo0?ooo`040?ooo`<00000o`3ooolC0?ooo`002@3oool010000000oooo0?oo
o`0000020?ooo`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo
0?ooo`040?ooo`030000003oool0oooo0?l0oooo4`3oool000X0oooo00<000000?ooo`0000000P00
00020?ooo`060000003oool0oooo0000003oool000000P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?l0oooo4`3oool000L0oooo00@000000?ooo`3oool000000P3oool01@000000
oooo0?ooo`3oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`040?oo
o`030000003oool0oooo0?l0oooo4`3oool000P0oooo0P0000030?ooo`<000000P3oool2000000<0
oooo0P0000070?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo0P00003o0?oooa@0oooo
000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3ooolC
0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0
oooo4`3oool00200oooo0P00003o0?oooa@0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3o
ool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oo
oa<0oooo000P0?ooo`800000o`3ooolD0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo
000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3ooolC
0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`800000o`3ooolD0?ooo`00
803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3o
ool000P0oooo0P0000020?ooo`<000000P3oool300000080oooo0`0000070?ooo`030000003oool0
oooo0?l0oooo4`3oool000L0oooo00H000000?ooo`3oool000000?ooo`0000020?ooo`030000003o
ool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`030000003o
ool0oooo0?l0oooo4`3oool000X0oooo00<000000?ooo`0000000P3oool00`000000oooo00000002
0?ooo`030000003oool000000080oooo00<000000?ooo`3oool0103oool300000?l0oooo4`3oool0
00L0oooo00H000000?ooo`3oool000000?ooo`0000020?ooo`030000003oool000000080oooo00<0
00000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo4`3o
ool000L0oooo0`0000020?ooo`060000003oool0oooo0000003oool000000P3oool00`000000oooo
000000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00203o
ool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo
000000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00203o
ool300000080oooo0P0000030?ooo`8000000`3oool2000000L0oooo00<000000?ooo`3oool0o`3o
oolC0?ooo`00803oool200000?l0oooo503oool00200oooo00<000000?ooo`3oool0o`3ooolC0?oo
o`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0oooo
4`3oool00200oooo0P00003o0?oooa@0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool0
0200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0
oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo0P00003o0?oooa@0oooo000P
0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3ooolC0?oo
o`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo06P0oooo
0`00002W0?ooo`00803oool2000006P0oooo1@00002V0?ooo`00803oool00`000000oooo0?ooo`1W
0?ooo`D00000YP3oool000P0oooo00<000000?ooo`3oool00P3oool200000080oooo0`0000020?oo
o`<000001`3oool00`000000oooo0?ooo`1W0?ooo`D00000YP3oool000P0oooo00D000000?ooo`3o
ool0oooo000000020?ooo`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo06P0oooo0`00002W0?ooo`00203oool00`000000
oooo0?ooo`040?ooo`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000
oooo0?ooo`040?ooo`<00000o`3ooolC0?ooo`002@3oool010000000oooo0?ooo`0000020?ooo`03
0000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?l0oooo4`3oool000T0oooo00<000000?ooo`3oool00`0000020?ooo`060000
003oool0oooo0000003oool000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
0?l0oooo4`3oool000X0oooo00@000000?ooo`3oool000000`3oool01P000000oooo0?ooo`000000
oooo00000080oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?oooa<0oooo0007
0?ooo`@000000P3oool300000080oooo0P0000030?ooo`8000001`3oool00`000000oooo0?ooo`3o
0?oooa<0oooo000P0?ooo`800000o`3ooolD0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0
oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3o
oolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`800000o`3ooolD0?oo
o`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0oooo
4`3oool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool200000?l0oooo503oool0
0200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0
oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3o
oolC0?ooo`00803oool200000?l0oooo503oool00200oooo00<000000?ooo`3oool0o`3ooolC0?oo
o`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo00040?ooo`030000003oool0oooo00<00000
0P3oool300000080oooo0`0000020?ooo`<000001`3oool00`000000oooo0?ooo`3o0?oooa<0oooo
00040?ooo`040000003oool0oooo00000080oooo00<000000?ooo`0000000P3oool00`000000oooo
000000020?ooo`030000003oool000000080oooo00<000000?ooo`3oool0103oool00`000000oooo
0?ooo`3M0?ooo`<00000<P3oool000@0oooo00@000000?ooo`3oool000000P3oool00`000000oooo
000000020?ooo`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo
0?ooo`040?ooo`<00000g03oool500000340oooo00040?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`0000000P3oool00`000000oooo000000020?ooo`030000003oool000000080oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3L0?ooo`D00000<@3oool000@0oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo000000020?ooo`030000003oool000000080oooo
00<000000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0=`0oooo
1@00000a0?ooo`000`3oool200000080oooo00H000000?ooo`3oool000000?ooo`0000020?ooo`03
0000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0=d0oooo0`00000b0?ooo`00103oool010000000oooo0?ooo`3oool2000000<0
oooo0P0000030?ooo`8000000`3oool2000000L0oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00
803oool200000?l0oooo503oool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool0
0`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200
oooo0P00003o0?oooa@0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<0
00000?ooo`3oool0o`3ooolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?oo
o`030000003oool0oooo0?l0oooo4`3oool00200oooo0P00003o0?oooa@0oooo000P0?ooo`030000
003oool0oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool0
0`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200
oooo0P00003o0?oooa@0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool000@0oooo00<0
00000?ooo`3oool0100000020?ooo`8000000P3oool300000080oooo0`0000070?ooo`030000003o
ool0oooo0?l0oooo4`3oool000@0oooo00D000000?ooo`3oool0oooo000000030?ooo`060000003o
ool0oooo0000003oool000000P3oool00`000000oooo000000020?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00103oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?ooo`030000
003oool0oooo00@0oooo0`00003o0?oooa<0oooo00040?ooo`030000003oool0oooo0080oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo000000020?ooo`030000003oool000000080oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?oooa<0oooo00040?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`0000000P0000020?ooo`060000003oool0oooo0000003oool0
00000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo4`3oool000<0oooo
0P0000020?ooo`040000003oool0oooo00000080oooo00D000000?ooo`3oool0oooo000000020?oo
o`030000003oool000000080oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?oo
oa<0oooo00040?ooo`040000003oool0oooo0?ooo`8000000`3oool300000080oooo0P0000030?oo
o`8000001`3oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`800000o`3ooolD0?ooo`00
803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3o
ool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oo
oa<0oooo000P0?ooo`800000o`3ooolD0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo
000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3ooolC
0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`800000o`3ooolD0?ooo`00
803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3o
ool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool200000?l0oooo503oool00200
oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo
00040?ooo`040000003oool0oooo0?ooo`8000000P3oool300000080oooo0`0000020?ooo`<00000
1`3oool00`000000oooo0?ooo`3o0?oooa<0oooo00040?ooo`040000003oool0oooo00000080oooo
00<000000?ooo`0000000P3oool00`000000oooo000000020?ooo`030000003oool000000080oooo
00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3o0?oooa<0oooo00040?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?ooo`030000003o
ool000000080oooo00<000000?ooo`3oool0103oool300000?l0oooo4`3oool000@0oooo00@00000
0?ooo`3oool000000P3oool00`000000oooo000000020?ooo`030000003oool000000080oooo00<0
00000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo4`3o
ool000@0oooo00<000000?ooo`3oool00`0000020?ooo`060000003oool0oooo0000003oool00000
0P3oool00`000000oooo000000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
o`3ooolC0?ooo`000`3oool2000000<0oooo00D000000?ooo`3oool0oooo000000020?ooo`030000
003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`030000
003oool0oooo0?l0oooo4`3oool000@0oooo00@000000?ooo`3oool0oooo0`0000020?ooo`800000
0`3oool2000000<0oooo0P0000070?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo0P00
003o0?oooa@0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<000000?oo
o`3oool0o`3ooolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`800000
o`3ooolD0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`030000003oool0
oooo0?l0oooo4`3oool00200oooo00<000000?ooo`3oool0o`3ooolC0?ooo`00803oool00`000000
oooo0?ooo`3o0?oooa<0oooo000P0?ooo`800000o`3ooolD0?ooo`00803oool00`000000oooo0?oo
o`3o0?oooa<0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00200oooo00<000000?oo
o`3oool0o`3ooolC0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo000P0?ooo`800000
o`3ooolD0?ooo`00803oool00`000000oooo0?ooo`3o0?oooa<0oooo00040?ooo`050000003oool0
oooo0?ooo`000000103oool200000080oooo0`0000020?ooo`<000001`3oool00`000000oooo0?oo
o`3o0?oooa<0oooo00040?ooo`050000003oool0oooo0?ooo`0000000`3oool01P000000oooo0?oo
o`000000oooo00000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?l0oooo4`3oool000@0oooo00D000000?ooo`3oool0oooo000000060?ooo`03
0000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`03
0000003oool0oooo0?l0oooo4`3oool000@0oooo00<000000?ooo`3oool00P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool000000080oooo00<000000?ooo`0000000P3oool00`000000
oooo0?ooo`040?ooo`<00000o`3ooolC0?ooo`00103oool00`000000oooo0?ooo`020?ooo`030000
003oool0oooo00<000000P3oool01P000000oooo0?ooo`000000oooo00000080oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`3o0?oooa<0oooo00030?ooo`8000001@3oool010000000
oooo0?ooo`0000030?ooo`060000003oool0oooo0000003oool000000P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0?l0oooo4`3oool000@0oooo00<000000?ooo`3oool010000002
0?ooo`<000000P3oool2000000<0oooo0P0000070?ooo`030000003oool0oooo0?l0oooo4`3oool0
0200oooo0P00003o0?oooa@0oooo000P0?ooo`030000003oool0oooo0?l0oooo4`3oool00?l0oooo
=P3oool00?l0oooo=P3oool00001\
\>"],
  ImageRangeCache->{{{0, 308.75}, {190.188, 0}} -> {-102.582, -1852.97, \
3.15269, 106.213}},
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]],

Cell["\<\
This plots the data on log\[Dash]log axes.\
\>", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"i:16", "b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(logplot = 
        ListPlot[Log[data], PlotStyle \[Rule] PointSize[ .02]];\)\)], "Input",\

  CellLabel->"In[26]:=",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.259252 0.0841754 -0.121689 0.070352 [
[.0909 .00651 -7.59375 -11.875 ]
[.0909 .00651 7.59375 0 ]
[.4276 .00651 -4.5 -11.875 ]
[.4276 .00651 4.5 0 ]
[.59595 .00651 -4.5 -11.875 ]
[.59595 .00651 4.5 0 ]
[.7643 .00651 -4.5 -11.875 ]
[.7643 .00651 4.5 0 ]
[.93266 .00651 -4.5 -11.875 ]
[.93266 .00651 4.5 0 ]
[.24675 .15972 -9 -5.9375 ]
[.24675 .15972 0 5.9375 ]
[.24675 .30042 -9 -5.9375 ]
[.24675 .30042 0 5.9375 ]
[.24675 .44113 -9 -5.9375 ]
[.24675 .44113 0 5.9375 ]
[.24675 .58183 -14 -5.9375 ]
[.24675 .58183 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.0909 .01901 m
.0909 .02526 L
s
gsave
.0909 .00651 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.4276 .01901 m
.4276 .02526 L
s
gsave
.4276 .00651 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.59595 .01901 m
.59595 .02526 L
s
gsave
.59595 .00651 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.7643 .01901 m
.7643 .02526 L
s
gsave
.7643 .00651 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.93266 .01901 m
.93266 .02526 L
s
gsave
.93266 .00651 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
.13299 .01901 m
.13299 .02276 L
s
.17508 .01901 m
.17508 .02276 L
s
.21716 .01901 m
.21716 .02276 L
s
.30134 .01901 m
.30134 .02276 L
s
.34343 .01901 m
.34343 .02276 L
s
.38552 .01901 m
.38552 .02276 L
s
.46969 .01901 m
.46969 .02276 L
s
.51178 .01901 m
.51178 .02276 L
s
.55387 .01901 m
.55387 .02276 L
s
.63804 .01901 m
.63804 .02276 L
s
.68013 .01901 m
.68013 .02276 L
s
.72222 .01901 m
.72222 .02276 L
s
.80639 .01901 m
.80639 .02276 L
s
.84848 .01901 m
.84848 .02276 L
s
.89057 .01901 m
.89057 .02276 L
s
.04881 .01901 m
.04881 .02276 L
s
.00673 .01901 m
.00673 .02276 L
s
.97474 .01901 m
.97474 .02276 L
s
.25 Mabswid
0 .01901 m
1 .01901 L
s
.25925 .15972 m
.2655 .15972 L
s
gsave
.24675 .15972 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.25925 .30042 m
.2655 .30042 L
s
gsave
.24675 .30042 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.25925 .44113 m
.2655 .44113 L
s
gsave
.24675 .44113 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.25925 .58183 m
.2655 .58183 L
s
gsave
.24675 .58183 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.125 Mabswid
.25925 .05419 m
.263 .05419 L
s
.25925 .08937 m
.263 .08937 L
s
.25925 .12454 m
.263 .12454 L
s
.25925 .19489 m
.263 .19489 L
s
.25925 .23007 m
.263 .23007 L
s
.25925 .26525 m
.263 .26525 L
s
.25925 .3356 m
.263 .3356 L
s
.25925 .37077 m
.263 .37077 L
s
.25925 .40595 m
.263 .40595 L
s
.25925 .4763 m
.263 .4763 L
s
.25925 .51148 m
.263 .51148 L
s
.25925 .54665 m
.263 .54665 L
s
.25925 .61701 m
.263 .61701 L
s
.25 Mabswid
.25925 0 m
.25925 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.02 w
.02381 .01472 Mdot
.18275 .11321 Mdot
.21928 .13974 Mdot
.33672 .21559 Mdot
.3506 .22983 Mdot
.36051 .24473 Mdot
.43637 .29411 Mdot
.54034 .36296 Mdot
.60017 .38292 Mdot
.61959 .39339 Mdot
.74996 .50218 Mdot
.812 .52698 Mdot
.97619 .60332 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[26]:=",
  ImageSize->{309.75, 191.188},
  ImageMargins->{{42.625, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004e0000_b000`400?l00000o`00003oo`3ooolf
0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`008`3oool4000005/0oooo1000
000`0?ooo`030000003oool0oooo02`0oooo0P00000`0?ooo`800000703oool002@0oooo00<00000
0?ooo`3oool0G03oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo02/0oooo00@00000
0?ooo`3oool00000;P3oool010000000oooo0?ooo`00000K0?ooo`007P3oool4000000<0oooo00<0
00000?ooo`3oool0G03oool00`000000oooo0?ooo`0/0?ooo`D00000;03oool010000000oooo0?oo
o`00000^0?ooo`040000003oool0oooo000001/0oooo000U0?ooo`030000003oool0oooo05`0oooo
00<000000?ooo`3oool0;03oool010000000oooo0?ooo`00000]0?ooo`040000003oool0oooo0000
02l0oooo0P00000L0?ooo`009P3oool00`000000oooo0?ooo`1L0?ooo`030000003oool0oooo02`0
oooo00<000000?ooo`000000;@3oool3000002l0oooo00@000000?ooo`3oool000006`3oool002<0
oooo00@000000?ooo`3oool00000F`3oool010000000oooo0?ooo`00000_0?ooo`800000;@3oool0
10000000oooo0?ooo`00000^0?ooo`040000003oool0oooo000001/0oooo000T0?ooo`800000;P3o
ool00`000000oooo0?ooo`0/0?ooo`800000<@3oool00`000000oooo0?ooo`0/0?ooo`800000<03o
ool2000001`0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo
0=h0oooo000>0?ooo`<00000@`3oool00`000000oooo0?ooo`3N0?ooo`003@3oool500000480oooo
00<000000?ooo`3oool0gP3oool000d0oooo1@0000120?ooo`030000003oool0oooo0=h0oooo0008
0?ooool000009P0000080?ooo`002P3oool010000000oooo0?ooo`3oool3000000D0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00X0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`0>0?ooo`008`3oool00`000000oooo0?ooo`0^0?oo
o`030000003oool0oooo02h0oooo00<000000?ooo`3oool0;P3oool00`000000oooo0?ooo`0^0?oo
o`030000003oool0oooo02h0oooo00<000000?ooo`3oool06P3oool005@0oooo00<000000?ooo`3o
ool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3o
ool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0
oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo0P00
003O0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N0?oo
o`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03o
ool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`00
0000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo
0?ooo`3N0?ooo`00E03oool200000=l0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?oo
o`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000
003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo000l0?ooo`<000005@3oool0
0`000000oooo0?ooo`3N0?ooo`00>`3oool5000001@0oooo00<000000?ooo`3oool0gP3oool003/0
oooo1@00000D0?ooo`030000003oool0oooo0=h0oooo000k0?ooo`D00000503oool00`000000oooo
0?ooo`3N0?ooo`00?03oool3000001D0oooo00<000000?ooo`3oool0gP3oool005@0oooo0P00003O
0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00A`3oool3000000X0oooo00<000000?ooo`3o
ool0gP3oool004H0oooo1@0000090?ooo`030000003oool0oooo0=h0oooo00160?ooo`D000002@3o
ool00`000000oooo0?ooo`3N0?ooo`00AP3oool5000000T0oooo00<000000?ooo`3oool0gP3oool0
04L0oooo0`00000:0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0
oooo001<0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0gP3oool004`0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`3N0?ooo`00B@3oool5000000H0oooo0`00003N
0?ooo`00B@3oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo0=h0oooo001:0?oo
o`030000003oool0000000L0oooo00<000000?ooo`3oool0gP3oool004/0oooo0P0000070?ooo`03
0000003oool0oooo0=h0oooo001<0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0
gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool0
05@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo
00<000000?ooo`3oool0gP3oool005@0oooo0P00003O0?ooo`00E03oool00`000000oooo0?ooo`3N
0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00
E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`0B0?ooo`<00000b@3o
ool005@0oooo00<000000?ooo`3oool04@3oool500000<P0oooo001D0?ooo`030000003oool0oooo
0140oooo1@0000380?ooo`00E03oool00`000000oooo0?ooo`0A0?ooo`D00000b03oool005@0oooo
00<000000?ooo`3oool04P3oool3000000040?ooo`000000000000000<D0oooo001D0?ooo`800000
5P3oool500000<@0oooo001D0?ooo`030000003oool0oooo01D0oooo1@0000340?ooo`00E03oool0
0`000000oooo0?ooo`0E0?ooo`D00000a03oool005@0oooo00<000000?ooo`3oool05P3oool60000
0<80oooo001D0?ooo`030000003oool0oooo01P0oooo1@0000310?ooo`00E03oool00`000000oooo
0?ooo`0H0?ooo`D00000`@3oool005@0oooo00<000000?ooo`3oool0603oool500000<40oooo001D
0?ooo`030000003oool0oooo01T0oooo0`0000320?ooo`00E03oool00`000000oooo0?ooo`3N0?oo
o`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03o
ool200000=l0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo
0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo
001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo02l0oooo0`00002/
0?ooo`00E03oool00`000000oooo0?ooo`0^0?ooo`D00000Z`3oool004X0oooo0P0000080?ooo`03
0000003oool0oooo02h0oooo1@00002[0?ooo`00B@3oool010000000oooo0?ooo`0000070?ooo`03
0000003oool0oooo02h0oooo1@00002[0?ooo`00B@3oool010000000oooo0?ooo`0000070?ooo`<0
0000;`3oool300000:`0oooo00190?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3o
ool0gP3oool004T0oooo0`0000080?ooo`030000003oool0oooo0=h0oooo00190?ooo`040000003o
ool0oooo000000L0oooo00<000000?ooo`3oool0gP3oool004X0oooo0P0000080?ooo`030000003o
ool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo
0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo
001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`800000g`3oool005@0oooo00<000000?oo
o`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0
gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool0
05@0oooo00<000000?ooo`3oool0C@3oool3000008h0oooo001D0?ooo`030000003oool0oooo04`0
oooo1@00002=0?ooo`00E03oool00`000000oooo0?ooo`1<0?ooo`D00000S@3oool005@0oooo00<0
00000?ooo`3oool0C03oool5000008d0oooo001D0?ooo`800000CP3oool3000008h0oooo001D0?oo
o`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo05l0oooo0`00001l0?ooo`00
E03oool00`000000oooo0?ooo`1N0?ooo`D00000N`3oool005@0oooo00<000000?ooo`3oool0GP3o
ool5000007/0oooo001D0?ooo`030000003oool0oooo05h0oooo1@000000103oool0000000000000
001g0?ooo`00E03oool00`000000oooo0?ooo`1O0?ooo`<0000000<0oooo0000000000000`00001f
0?ooo`00E03oool00`000000oooo0?ooo`1S0?ooo`D00000MP3oool005@0oooo00<000000?ooo`3o
ool0H`3oool5000007H0oooo001D0?ooo`030000003oool0oooo06@0oooo0`00001g0?ooo`00E03o
ool00`000000oooo0?ooo`3N0?ooo`00E03oool200000=l0oooo001D0?ooo`030000003oool0oooo
0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo
001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?oo
o`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001:0?ooo`800000
203oool00`000000oooo0?ooo`3N0?ooo`00B@3oool010000000oooo0?ooo`0000070?ooo`030000
003oool0oooo0=h0oooo00190?ooo`040000003oool0oooo000000L0oooo0`00003N0?ooo`00BP3o
ool2000000P0oooo00<000000?ooo`3oool0gP3oool004T0oooo00@000000?ooo`3oool000001`3o
ool00`000000oooo0?ooo`3N0?ooo`00B@3oool010000000oooo0?ooo`0000070?ooo`030000003o
ool0oooo0=h0oooo001:0?ooo`800000203oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`00
0000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo
0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3N
0?ooo`00E03oool200000=l0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000
003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0
oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo08X0
oooo0`00001A0?ooo`00E03oool00`000000oooo0?ooo`290?ooo`D00000D03oool005@0oooo00<0
00000?ooo`3oool0R@3oool500000500oooo001D0?ooo`030000003oool0oooo08T0oooo1@00001@
0?ooo`00E03oool2000008/0oooo0`00001A0?ooo`00E03oool00`000000oooo0?ooo`3N0?ooo`00
E03oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`2L0?ooo`<00000?`3o
ool005@0oooo00<000000?ooo`3oool0V`3oool5000003h0oooo001D0?ooo`030000003oool0oooo
09/0oooo1@00000n0?ooo`00E03oool00`000000oooo0?ooo`2K0?ooo`D00000?P3oool005@0oooo
00<000000?ooo`3oool0W03oool3000003l0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D
0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`030000003oool0oooo0=h0oooo001D0?ooo`80
0000g`3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3o
ool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0
oooo00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<0
00000?ooo`3oool0gP3oool004H0oooo00<000000?ooo`3oool00`0000080?ooo`030000003oool0
oooo0=h0oooo00160?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`3N0?ooo`00AP3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo00D0oooo0`00003N0?ooo`00AP3oool010000000oooo0?ooo`0000020?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0gP3oool004H0oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo0=h0oooo00150?ooo`8000000P3oool01000
0000oooo0?ooo`0000070?ooo`030000003oool0oooo0=h0oooo00160?ooo`040000003oool0oooo
0?ooo`800000203oool00`000000oooo0?ooo`3N0?ooo`00E03oool00`000000oooo0?ooo`3<0?oo
o`<000003`3oool005@0oooo00<000000?ooo`3oool0b`3oool5000000h0oooo001D0?ooo`030000
003oool0oooo0</0oooo1@00000>0?ooo`00E03oool00`000000oooo0?ooo`3;0?ooo`D000003P3o
ool005@0oooo00<000000?ooo`3oool0c03oool3000000l0oooo001D0?ooo`800000g`3oool00001
\
\>"],
  ImageRangeCache->{{{0, 308.75}, {190.188, 0}} -> {-3.42955, 1.24332, \
0.0407425, 0.048748}},
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]],

Cell[TextData[{
  "You can then fit a straight line to the log-data by performing a linear \
least squares fit. In this example, we are fitting to the model ",
  Cell[BoxData[
      \(TraditionalForm\`a + m\ x\)]],
  ", where ",
  Cell[BoxData[
      \(TraditionalForm\`a\)]],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`m\)]],
  " are the parameters to be determined in the model with variable ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  "."
}], "TextFirst",
  CellTags->{"i:17", "b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(f = FindFit[Log[data], a + m\ x, {a, m}, x]\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[BoxData[
    \({a \[Rule] 4.154367231072235`, 
      m \[Rule] 0.7614649634315508`}\)], "Output",
  CellLabel->"Out[27]=",
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]],

Cell["Here is a plot of the linear fit function.", "Text",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(fplot = Plot[a + m\ x /. f, {x, \(-3\), 9}];\)\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.261905 0.0793651 -0.105741 0.0644157 [
[.10317 .01059 -7.59375 -11.875 ]
[.10317 .01059 7.59375 0 ]
[.42063 .01059 -4.5 -11.875 ]
[.42063 .01059 4.5 0 ]
[.57937 .01059 -4.5 -11.875 ]
[.57937 .01059 4.5 0 ]
[.7381 .01059 -4.5 -11.875 ]
[.7381 .01059 4.5 0 ]
[.89683 .01059 -4.5 -11.875 ]
[.89683 .01059 4.5 0 ]
[.2494 .15192 -9 -5.9375 ]
[.2494 .15192 0 5.9375 ]
[.2494 .28075 -9 -5.9375 ]
[.2494 .28075 0 5.9375 ]
[.2494 .40959 -9 -5.9375 ]
[.2494 .40959 0 5.9375 ]
[.2494 .53842 -14 -5.9375 ]
[.2494 .53842 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.10317 .02309 m
.10317 .02934 L
s
gsave
.10317 .01059 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.42063 .02309 m
.42063 .02934 L
s
gsave
.42063 .01059 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.57937 .02309 m
.57937 .02934 L
s
gsave
.57937 .01059 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.7381 .02309 m
.7381 .02934 L
s
gsave
.7381 .01059 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.89683 .02309 m
.89683 .02934 L
s
gsave
.89683 .01059 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
.14286 .02309 m
.14286 .02684 L
s
.18254 .02309 m
.18254 .02684 L
s
.22222 .02309 m
.22222 .02684 L
s
.30159 .02309 m
.30159 .02684 L
s
.34127 .02309 m
.34127 .02684 L
s
.38095 .02309 m
.38095 .02684 L
s
.46032 .02309 m
.46032 .02684 L
s
.5 .02309 m
.5 .02684 L
s
.53968 .02309 m
.53968 .02684 L
s
.61905 .02309 m
.61905 .02684 L
s
.65873 .02309 m
.65873 .02684 L
s
.69841 .02309 m
.69841 .02684 L
s
.77778 .02309 m
.77778 .02684 L
s
.81746 .02309 m
.81746 .02684 L
s
.85714 .02309 m
.85714 .02684 L
s
.06349 .02309 m
.06349 .02684 L
s
.02381 .02309 m
.02381 .02684 L
s
.93651 .02309 m
.93651 .02684 L
s
.97619 .02309 m
.97619 .02684 L
s
.25 Mabswid
0 .02309 m
1 .02309 L
s
.2619 .15192 m
.26815 .15192 L
s
gsave
.2494 .15192 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.2619 .28075 m
.26815 .28075 L
s
gsave
.2494 .28075 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.2619 .40959 m
.26815 .40959 L
s
gsave
.2494 .40959 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.2619 .53842 m
.26815 .53842 L
s
gsave
.2494 .53842 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.125 Mabswid
.2619 .0553 m
.26565 .0553 L
s
.2619 .08751 m
.26565 .08751 L
s
.2619 .11971 m
.26565 .11971 L
s
.2619 .18413 m
.26565 .18413 L
s
.2619 .21634 m
.26565 .21634 L
s
.2619 .24855 m
.26565 .24855 L
s
.2619 .31296 m
.26565 .31296 L
s
.2619 .34517 m
.26565 .34517 L
s
.2619 .37738 m
.26565 .37738 L
s
.2619 .44179 m
.26565 .44179 L
s
.2619 .474 m
.26565 .474 L
s
.2619 .50621 m
.26565 .50621 L
s
.2619 .57062 m
.26565 .57062 L
s
.2619 .60283 m
.26565 .60283 L
s
.25 Mabswid
.2619 0 m
.2619 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .03859 L
.10458 .06463 L
.14415 .08909 L
.18221 .11261 L
.22272 .13765 L
.26171 .16175 L
.30316 .18736 L
.34309 .21204 L
.3815 .23578 L
.42237 .26104 L
.46172 .28536 L
.49955 .30874 L
.53984 .33364 L
.57861 .3576 L
.61984 .38308 L
.65954 .40762 L
.69774 .43123 L
.73838 .45635 L
.77751 .48053 L
.81909 .50623 L
.85916 .53099 L
.89771 .55482 L
.93871 .58016 L
.97619 .60332 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[28]:=",
  ImageSize->{310.875, 192},
  ImageMargins->{{42.6875, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004f0000`2000`400?l00000o`00003oo`3ooolg
0?ooo`00o`3ooolg0?ooo`00o`3ooolg0?ooo`00o`3ooolg0?ooo`009P3oool4000005L0oooo1000
000]0?ooo`030000003oool0oooo02X0oooo0P00000]0?ooo`8000009P3oool002L0oooo00<00000
0?ooo`3oool0F03oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo02T0oooo00@00000
0?ooo`3oool00000:`3oool010000000oooo0?ooo`00000U0?ooo`008@3oool4000000<0oooo00<0
00000?ooo`3oool0F03oool00`000000oooo0?ooo`0Y0?ooo`D00000:P3oool010000000oooo0?oo
o`00000[0?ooo`040000003oool0oooo000002D0oooo000X0?ooo`030000003oool0oooo05P0oooo
00<000000?ooo`3oool0:@3oool010000000oooo0?ooo`00000[0?ooo`040000003oool0oooo0000
02`0oooo0P00000V0?ooo`00:@3oool00`000000oooo0?ooo`1H0?ooo`030000003oool0oooo02T0
oooo00<000000?ooo`000000:`3oool3000002`0oooo00@000000?ooo`3oool000009@3oool002H0
oooo00@000000?ooo`3oool00000:P3oool00`000000oooo0?ooo`0Z0?ooo`040000003oool0oooo
000002`0oooo0P00000[0?ooo`040000003oool0oooo000002/0oooo00@000000?ooo`3oool00000
9@3oool002L0oooo0P00000[0?ooo`030000003oool0oooo02/0oooo0P00000^0?ooo`030000003o
ool0oooo02X0oooo0P00000]0?ooo`8000009P3oool005@0oooo00<000000?ooo`3oool0g`3oool0
05@0oooo00<000000?ooo`3oool0g`3oool005@0oooo00<000000?ooo`3oool0g`3oool000h0oooo
00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`3O0?ooo`003`3oool2000004<0oooo00<0
00000?ooo`3oool0g`3oool000L0ooooo`00000Y000000L0oooo000>0?ooo`040000003oool0oooo
0?ooo`8000001P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00P0oooo00<00000
0?ooo`3oool02@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0303oool001@0oooo0P00000?0?ooo`030000003oool0oooo02`0oooo00<000000?oo
o`3oool0;03oool00`000000oooo0?ooo`0/0?ooo`030000003oool0oooo02`0oooo00<000000?oo
o`3oool0:`3oool00`000000oooo0?ooo`0T0?ooo`005P3oool00`000000oooo0?ooo`0k0?ooo`03
0000003oool0oooo0=l0oooo000G0?ooo`800000>`3oool00`000000oooo0?ooo`3O0?ooo`006@3o
ool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo0=l0oooo000J0?ooo`800000>03oool0
0`000000oooo0?ooo`3O0?ooo`00703oool2000003H0oooo00<000000?ooo`3oool0g`3oool001h0
oooo0P00000d0?ooo`030000003oool0oooo0=l0oooo000P0?ooo`800000<P3oool00`000000oooo
0?ooo`3O0?ooo`008P3oool200000300oooo0P00003P0?ooo`00903oool2000002h0oooo00<00000
0?ooo`3oool0g`3oool002H0oooo00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`3O0?oo
o`009`3oool2000002/0oooo00<000000?ooo`3oool0g`3oool002T0oooo00<000000?ooo`3oool0
:03oool00`000000oooo0?ooo`3O0?ooo`00:P3oool2000002P0oooo00<000000?ooo`3oool0g`3o
ool002`0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`3O0?ooo`00;@3oool20000
02D0oooo00<000000?ooo`3oool0g`3oool002l0oooo00<000000?ooo`3oool08P3oool00`000000
oooo0?ooo`3O0?ooo`00<03oool200000280oooo0P00003P0?ooo`00<P3oool00`000000oooo0?oo
o`0O0?ooo`030000003oool0oooo0=l0oooo000c0?ooo`8000007`3oool00`000000oooo0?ooo`3O
0?ooo`00=@3oool2000001d0oooo00<000000?ooo`3oool0g`3oool003L0oooo0P00000K0?ooo`03
0000003oool0oooo0=l0oooo000i0?ooo`8000006@3oool00`000000oooo0?ooo`3O0?ooo`00>`3o
ool2000001L0oooo00<000000?ooo`3oool0g`3oool003d0oooo00<000000?ooo`3oool0503oool0
0`000000oooo0?ooo`3O0?ooo`00?P3oool2000001@0oooo00<000000?ooo`3oool0g`3oool00400
oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`3O0?ooo`00@@3oool200000140oooo
0P00003P0?ooo`00@`3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0=l0oooo0014
0?ooo`8000003P3oool00`000000oooo0?ooo`3O0?ooo`00AP3oool00`000000oooo0?ooo`0;0?oo
o`030000003oool0oooo0=l0oooo00170?ooo`8000002`3oool00`000000oooo0?ooo`3O0?ooo`00
B@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0=l0oooo001:0?ooo`800000203o
ool00`000000oooo0?ooo`3O0?ooo`00C03oool2000000H0oooo00<000000?ooo`3oool0g`3oool0
04d0oooo0P0000050?ooo`030000003oool0oooo0=l0oooo001:0?ooo`L000000`3oool300000=l0
oooo001:0?ooo`040000003oool0oooo000000<0oooo00@000000?ooo`3oool00000h@3oool004/0
oooo00<000000?ooo`000000103oool300000>40oooo001<0?ooo`8000001P3oool00`000000oooo
0?ooo`3O0?ooo`00C@3oool00`000000oooo0?ooo`040?ooo`<00000g`3oool005@0oooo00@00000
0?ooo`3oool00000gP3oool005@0oooo00@000000?ooo`3oool0oooo0P00003L0?ooo`00E03oool0
0`000000oooo0?ooo`030?ooo`030000003oool0oooo0=T0oooo001D0?ooo`030000003oool0oooo
00@0oooo0P00003I0?ooo`00E03oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0=H0
oooo001D0?ooo`800000203oool200000=H0oooo001D0?ooo`030000003oool0oooo00T0oooo00<0
00000?ooo`3oool0d`3oool005@0oooo00<000000?ooo`3oool02P3oool200000=<0oooo001D0?oo
o`030000003oool0oooo00`0oooo0P00003A0?ooo`00E03oool00`000000oooo0?ooo`0>0?ooo`03
0000003oool0oooo0<h0oooo001D0?ooo`030000003oool0oooo00l0oooo0P00003>0?ooo`00E03o
ool00`000000oooo0?ooo`0A0?ooo`800000c03oool005@0oooo00<000000?ooo`3oool04`3oool2
00000<X0oooo001D0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool0a`3oool005@0
oooo0P00000G0?ooo`800000a`3oool005@0oooo00<000000?ooo`3oool0603oool00`000000oooo
0?ooo`340?ooo`00E03oool00`000000oooo0?ooo`0I0?ooo`800000a03oool005@0oooo00<00000
0?ooo`3oool06`3oool200000<80oooo001D0?ooo`030000003oool0oooo01d0oooo00<000000?oo
o`3oool0_`3oool005@0oooo00<000000?ooo`3oool07P3oool200000;l0oooo001D0?ooo`030000
003oool0oooo0200oooo00<000000?ooo`3oool0_03oool005@0oooo00<000000?ooo`3oool08@3o
ool200000;`0oooo001D0?ooo`030000003oool0oooo02<0oooo0P00002j0?ooo`00E03oool00`00
0000oooo0?ooo`0U0?ooo`800000^03oool005@0oooo0P00000X0?ooo`800000]P3oool005@0oooo
00<000000?ooo`3oool0:@3oool200000;@0oooo001D0?ooo`030000003oool0oooo02/0oooo0P00
002b0?ooo`00E03oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0:l0oooo001D0?oo
o`030000003oool0oooo02h0oooo0P00002_0?ooo`00E03oool00`000000oooo0?ooo`0`0?ooo`03
0000003oool0oooo0:`0oooo001D0?ooo`030000003oool0oooo0340oooo0P00002/0?ooo`00B`3o
ool2000000L0oooo00<000000?ooo`3oool0<`3oool200000:X0oooo001:0?ooo`040000003oool0
oooo000000H0oooo00<000000?ooo`3oool0=@3oool00`000000oooo0?ooo`2W0?ooo`00BP3oool0
10000000oooo0?ooo`0000060?ooo`<00000=P3oool200000:L0oooo001:0?ooo`040000003oool0
oooo000000H0oooo00<000000?ooo`3oool0>03oool00`000000oooo0?ooo`2T0?ooo`00BP3oool3
000000L0oooo00<000000?ooo`3oool0>@3oool200000:@0oooo001:0?ooo`040000003oool0oooo
000000H0oooo00<000000?ooo`3oool0>`3oool00`000000oooo0?ooo`2Q0?ooo`00B`3oool20000
00L0oooo00<000000?ooo`3oool0?03oool200000:40oooo001D0?ooo`030000003oool0oooo03h0
oooo0P00002O0?ooo`00E03oool00`000000oooo0?ooo`100?ooo`030000003oool0oooo09`0oooo
001D0?ooo`030000003oool0oooo0440oooo0P00002L0?ooo`00E03oool00`000000oooo0?ooo`13
0?ooo`030000003oool0oooo09T0oooo001D0?ooo`030000003oool0oooo04@0oooo0P00002I0?oo
o`00E03oool2000004L0oooo00<000000?ooo`3oool0UP3oool005@0oooo00<000000?ooo`3oool0
A`3oool2000009H0oooo001D0?ooo`030000003oool0oooo04T0oooo00<000000?ooo`3oool0T`3o
ool005@0oooo00<000000?ooo`3oool0BP3oool2000009<0oooo001D0?ooo`030000003oool0oooo
04`0oooo00<000000?ooo`3oool0T03oool005@0oooo00<000000?ooo`3oool0C@3oool200000900
oooo001D0?ooo`030000003oool0oooo04l0oooo00<000000?ooo`3oool0S@3oool005@0oooo00<0
00000?ooo`3oool0D03oool2000008d0oooo001D0?ooo`030000003oool0oooo0580oooo0P00002;
0?ooo`00E03oool2000005D0oooo00<000000?ooo`3oool0R03oool005@0oooo00<000000?ooo`3o
ool0E@3oool2000008P0oooo001D0?ooo`030000003oool0oooo05L0oooo0P0000260?ooo`00E03o
ool00`000000oooo0?ooo`1I0?ooo`800000Q03oool005@0oooo00<000000?ooo`3oool0F`3oool0
0`000000oooo0?ooo`210?ooo`00E03oool00`000000oooo0?ooo`1L0?ooo`800000P@3oool005@0
oooo00<000000?ooo`3oool0GP3oool2000007l0oooo001D0?ooo`030000003oool0oooo0600oooo
00<000000?ooo`3oool0O03oool005@0oooo00<000000?ooo`3oool0H@3oool2000007`0oooo001D
0?ooo`030000003oool0oooo06<0oooo0P00001j0?ooo`00E03oool2000006H0oooo0P00001h0?oo
o`00E03oool00`000000oooo0?ooo`1W0?ooo`030000003oool0oooo07D0oooo001D0?ooo`030000
003oool0oooo06P0oooo0P00001e0?ooo`00E03oool00`000000oooo0?ooo`1Z0?ooo`030000003o
ool0oooo0780oooo001D0?ooo`030000003oool0oooo06/0oooo0P00001b0?ooo`00E03oool00`00
0000oooo0?ooo`1]0?ooo`800000L03oool005@0oooo00<000000?ooo`3oool0K`3oool00`000000
oooo0?ooo`1]0?ooo`00B`3oool2000000L0oooo00<000000?ooo`3oool0L03oool2000006d0oooo
001:0?ooo`040000003oool0oooo000000H0oooo00<000000?ooo`3oool0LP3oool00`000000oooo
0?ooo`1Z0?ooo`00BP3oool010000000oooo0?ooo`0000060?ooo`<00000L`3oool2000006X0oooo
001;0?ooo`8000001`3oool00`000000oooo0?ooo`1e0?ooo`800000J03oool004X0oooo00@00000
0?ooo`3oool000001P3oool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo06D0oooo001:
0?ooo`040000003oool0oooo000000H0oooo00<000000?ooo`3oool0N03oool2000006D0oooo001;
0?ooo`8000001`3oool00`000000oooo0?ooo`1j0?ooo`800000H`3oool005@0oooo00<000000?oo
o`3oool0O03oool200000640oooo001D0?ooo`030000003oool0oooo07h0oooo00<000000?ooo`3o
ool0GP3oool005@0oooo00<000000?ooo`3oool0O`3oool2000005h0oooo001D0?ooo`030000003o
ool0oooo0840oooo00<000000?ooo`3oool0F`3oool005@0oooo00<000000?ooo`3oool0PP3oool2
000005/0oooo001D0?ooo`800000Q@3oool00`000000oooo0?ooo`1H0?ooo`00E03oool00`000000
oooo0?ooo`250?ooo`800000F03oool005@0oooo00<000000?ooo`3oool0Q`3oool00`000000oooo
0?ooo`1E0?ooo`00E03oool00`000000oooo0?ooo`280?ooo`800000E@3oool005@0oooo00<00000
0?ooo`3oool0RP3oool00`000000oooo0?ooo`1B0?ooo`00E03oool00`000000oooo0?ooo`2;0?oo
o`800000DP3oool005@0oooo00<000000?ooo`3oool0S@3oool00`000000oooo0?ooo`1?0?ooo`00
E03oool00`000000oooo0?ooo`2>0?ooo`800000C`3oool005@0oooo00<000000?ooo`3oool0T03o
ool2000004d0oooo001D0?ooo`800000T`3oool00`000000oooo0?ooo`1:0?ooo`00E03oool00`00
0000oooo0?ooo`2C0?ooo`800000BP3oool005@0oooo00<000000?ooo`3oool0U@3oool00`000000
oooo0?ooo`170?ooo`00E03oool00`000000oooo0?ooo`2F0?ooo`800000A`3oool005@0oooo00<0
00000?ooo`3oool0V03oool2000004D0oooo001D0?ooo`030000003oool0oooo09X0oooo00<00000
0?ooo`3oool0@P3oool005@0oooo00<000000?ooo`3oool0V`3oool200000480oooo001D0?ooo`03
0000003oool0oooo09d0oooo00<000000?ooo`3oool0?`3oool005@0oooo00<000000?ooo`3oool0
WP3oool2000003l0oooo001D0?ooo`030000003oool0oooo0:00oooo0P00000m0?ooo`00E03oool2
00000:<0oooo00<000000?ooo`3oool0>P3oool005@0oooo00<000000?ooo`3oool0X`3oool20000
03X0oooo001D0?ooo`030000003oool0oooo0:D0oooo00<000000?ooo`3oool0=`3oool005@0oooo
00<000000?ooo`3oool0YP3oool2000003L0oooo001D0?ooo`030000003oool0oooo0:P0oooo0P00
000e0?ooo`00E03oool00`000000oooo0?ooo`2Z0?ooo`030000003oool0oooo0380oooo001D0?oo
o`030000003oool0oooo0:/0oooo0P00000b0?ooo`00A`3oool00`000000oooo0?ooo`03000000L0
oooo00<000000?ooo`3oool0[@3oool00`000000oooo0?ooo`0_0?ooo`00A`3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0[P3oool2000002l0
oooo00170?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0103oool300000;00
oooo0P00000]0?ooo`00A`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00@0
oooo00<000000?ooo`3oool0/P3oool00`000000oooo0?ooo`0Z0?ooo`00A`3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0/`3oool2000002X0
oooo00160?ooo`8000000P3oool010000000oooo0?ooo`0000060?ooo`030000003oool0oooo0;D0
oooo0P00000X0?ooo`00A`3oool010000000oooo0?ooo`3oool2000000L0oooo00<000000?ooo`3o
ool0]`3oool2000002H0oooo001D0?ooo`030000003oool0oooo0;T0oooo00<000000?ooo`3oool0
8`3oool005@0oooo00<000000?ooo`3oool0^P3oool2000002<0oooo001D0?ooo`030000003oool0
oooo0;`0oooo0P00000Q0?ooo`00E03oool00`000000oooo0?ooo`2n0?ooo`030000003oool0oooo
01h0oooo001D0?ooo`030000003oool0oooo0;l0oooo0P00000N0?ooo`00E03oool200000<80oooo
0P00000L0?ooo`00E03oool00`000000oooo0?ooo`330?ooo`8000006P3oool005@0oooo00<00000
0?ooo`3oool0a@3oool00`000000oooo0?ooo`0G0?ooo`00E03oool00`000000oooo0?ooo`360?oo
o`8000005`3oool005@0oooo00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`0D0?ooo`00
E03oool00`000000oooo0?ooo`390?ooo`800000503oool005@0oooo00<000000?ooo`3oool0b`3o
ool200000180oooo001D0?ooo`030000003oool0oooo0<d0oooo00<000000?ooo`3oool03`3oool0
05@0oooo00<000000?ooo`3oool0cP3oool2000000l0oooo001D0?ooo`800000d@3oool00`000000
oooo0?ooo`0<0?ooo`00E03oool00`000000oooo0?ooo`3O0?ooo`00E03oool00`000000oooo0?oo
o`3O0?ooo`00E03oool00`000000oooo0?ooo`3O0?ooo`00E03oool00`000000oooo0?ooo`3O0?oo
o`00\
\>"],
  ImageRangeCache->{{{0, 309.875}, {191, 0}} -> {-3.62032, 1.18073, \
0.0427289, 0.0526453}},
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]],

Cell["\<\
Finally, you can see how well the fitted function approximates the log plot \
by combining these last two graphics.\
\>", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"i:18", "b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[fplot, logplot];\)\)], "Input",
  CellLabel->"In[29]:=",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.261905 0.0793651 -0.105741 0.0644157 [
[.10317 .01059 -7.59375 -11.875 ]
[.10317 .01059 7.59375 0 ]
[.42063 .01059 -4.5 -11.875 ]
[.42063 .01059 4.5 0 ]
[.57937 .01059 -4.5 -11.875 ]
[.57937 .01059 4.5 0 ]
[.7381 .01059 -4.5 -11.875 ]
[.7381 .01059 4.5 0 ]
[.89683 .01059 -4.5 -11.875 ]
[.89683 .01059 4.5 0 ]
[.2494 .15192 -9 -5.9375 ]
[.2494 .15192 0 5.9375 ]
[.2494 .28075 -9 -5.9375 ]
[.2494 .28075 0 5.9375 ]
[.2494 .40959 -9 -5.9375 ]
[.2494 .40959 0 5.9375 ]
[.2494 .53842 -14 -5.9375 ]
[.2494 .53842 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.10317 .02309 m
.10317 .02934 L
s
gsave
.10317 .01059 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.42063 .02309 m
.42063 .02934 L
s
gsave
.42063 .01059 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.57937 .02309 m
.57937 .02934 L
s
gsave
.57937 .01059 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.7381 .02309 m
.7381 .02934 L
s
gsave
.7381 .01059 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.89683 .02309 m
.89683 .02934 L
s
gsave
.89683 .01059 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.125 Mabswid
.14286 .02309 m
.14286 .02684 L
s
.18254 .02309 m
.18254 .02684 L
s
.22222 .02309 m
.22222 .02684 L
s
.30159 .02309 m
.30159 .02684 L
s
.34127 .02309 m
.34127 .02684 L
s
.38095 .02309 m
.38095 .02684 L
s
.46032 .02309 m
.46032 .02684 L
s
.5 .02309 m
.5 .02684 L
s
.53968 .02309 m
.53968 .02684 L
s
.61905 .02309 m
.61905 .02684 L
s
.65873 .02309 m
.65873 .02684 L
s
.69841 .02309 m
.69841 .02684 L
s
.77778 .02309 m
.77778 .02684 L
s
.81746 .02309 m
.81746 .02684 L
s
.85714 .02309 m
.85714 .02684 L
s
.06349 .02309 m
.06349 .02684 L
s
.02381 .02309 m
.02381 .02684 L
s
.93651 .02309 m
.93651 .02684 L
s
.97619 .02309 m
.97619 .02684 L
s
.25 Mabswid
0 .02309 m
1 .02309 L
s
.2619 .15192 m
.26815 .15192 L
s
gsave
.2494 .15192 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.2619 .28075 m
.26815 .28075 L
s
gsave
.2494 .28075 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.2619 .40959 m
.26815 .40959 L
s
gsave
.2494 .40959 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.2619 .53842 m
.26815 .53842 L
s
gsave
.2494 .53842 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.125 Mabswid
.2619 .0553 m
.26565 .0553 L
s
.2619 .08751 m
.26565 .08751 L
s
.2619 .11971 m
.26565 .11971 L
s
.2619 .18413 m
.26565 .18413 L
s
.2619 .21634 m
.26565 .21634 L
s
.2619 .24855 m
.26565 .24855 L
s
.2619 .31296 m
.26565 .31296 L
s
.2619 .34517 m
.26565 .34517 L
s
.2619 .37738 m
.26565 .37738 L
s
.2619 .44179 m
.26565 .44179 L
s
.2619 .474 m
.26565 .474 L
s
.2619 .50621 m
.26565 .50621 L
s
.2619 .57062 m
.26565 .57062 L
s
.2619 .60283 m
.26565 .60283 L
s
.25 Mabswid
.2619 0 m
.2619 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .01472 m
.06244 .03859 L
.10458 .06463 L
.14415 .08909 L
.18221 .11261 L
.22272 .13765 L
.26171 .16175 L
.30316 .18736 L
.34309 .21204 L
.3815 .23578 L
.42237 .26104 L
.46172 .28536 L
.49955 .30874 L
.53984 .33364 L
.57861 .3576 L
.61984 .38308 L
.65954 .40762 L
.69774 .43123 L
.73838 .45635 L
.77751 .48053 L
.81909 .50623 L
.85916 .53099 L
.89771 .55482 L
.93871 .58016 L
.97619 .60332 L
s
.02 w
.03992 .01915 Mdot
.18978 .10934 Mdot
.22422 .13363 Mdot
.33494 .20308 Mdot
.34803 .21611 Mdot
.35738 .22976 Mdot
.4289 .27497 Mdot
.52693 .33801 Mdot
.58334 .35629 Mdot
.60165 .36588 Mdot
.72457 .46549 Mdot
.78307 .4882 Mdot
.93787 .55809 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[29]:=",
  ImageSize->{309.75, 191.188},
  ImageMargins->{{42.625, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004e0000_b000`400?l00000o`00003oo`3ooolf
0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`00o`3ooolf0?ooo`009P3oool4000005H0oooo1000
000^0?ooo`030000003oool0oooo02T0oooo0P00000]0?ooo`8000009P3oool002L0oooo00<00000
0?ooo`3oool0E`3oool00`000000oooo0?ooo`0^0?ooo`030000003oool0oooo02P0oooo00@00000
0?ooo`3oool00000:`3oool010000000oooo0?ooo`00000U0?ooo`008@3oool4000000<0oooo00<0
00000?ooo`3oool0E`3oool00`000000oooo0?ooo`0Z0?ooo`D00000:@3oool010000000oooo0?oo
o`00000[0?ooo`040000003oool0oooo000002D0oooo000X0?ooo`030000003oool0oooo05L0oooo
00<000000?ooo`3oool0:P3oool010000000oooo0?ooo`00000Z0?ooo`040000003oool0oooo0000
02`0oooo0P00000V0?ooo`00:@3oool00`000000oooo0?ooo`1G0?ooo`030000003oool0oooo02X0
oooo00<000000?ooo`000000:P3oool3000002`0oooo00@000000?ooo`3oool000009@3oool002H0
oooo00@000000?ooo`3oool00000:P3oool00`000000oooo0?ooo`0Y0?ooo`040000003oool0oooo
000002d0oooo0P00000Z0?ooo`040000003oool0oooo000002/0oooo00@000000?ooo`3oool00000
9@3oool002L0oooo0P00000[0?ooo`030000003oool0oooo02X0oooo0P00000_0?ooo`030000003o
ool0oooo02T0oooo0P00000]0?ooo`8000009P3oool005@0oooo00<000000?ooo`3oool0gP3oool0
05@0oooo00<000000?ooo`3oool0gP3oool000h0oooo00<000000?ooo`3oool00`0000100?ooo`03
0000003oool0oooo0=h0oooo000?0?ooo`H00000?`3oool00`000000oooo0?ooo`3N0?ooo`00403o
ool5000003l0oooo00<000000?ooo`3oool0gP3oool000L0ooooo`00000X000000L0oooo000>0?oo
o`030000003oool0oooo00D00000103oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
00T0oooo00<000000?ooo`3oool0303oool001H0oooo00<000000?ooo`3oool0303oool00`000000
oooo0?ooo`0/0?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0:`3oool00`000000
oooo0?ooo`0/0?ooo`030000003oool0oooo02/0oooo00<000000?ooo`3oool0903oool001L0oooo
0P00000k0?ooo`030000003oool0oooo0=h0oooo000I0?ooo`030000003oool0oooo03P0oooo00<0
00000?ooo`3oool0gP3oool001X0oooo0P00000h0?ooo`030000003oool0oooo0=h0oooo000L0?oo
o`800000=P3oool00`000000oooo0?ooo`3N0?ooo`007P3oool00`000000oooo0?ooo`0c0?ooo`03
0000003oool0oooo0=h0oooo000O0?ooo`800000<`3oool00`000000oooo0?ooo`3N0?ooo`008@3o
ool00`000000oooo0?ooo`0`0?ooo`800000g`3oool00280oooo0P00000`0?ooo`030000003oool0
oooo0=h0oooo000T0?ooo`800000;P3oool00`000000oooo0?ooo`3N0?ooo`009P3oool00`000000
oooo0?ooo`0[0?ooo`030000003oool0oooo0=h0oooo000W0?ooo`800000:`3oool00`000000oooo
0?ooo`3N0?ooo`00:@3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo0=h0oooo000Z
0?ooo`800000:03oool00`000000oooo0?ooo`3N0?ooo`00;03oool2000002H0oooo00<000000?oo
o`3oool0gP3oool002h0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`3N0?ooo`00
;`3oool2000002<0oooo00<000000?ooo`3oool0gP3oool00340oooo00<000000?ooo`3oool0803o
ool200000=l0oooo000b0?ooo`800000803oool00`000000oooo0?ooo`3N0?ooo`00=03oool20000
01h0oooo00<000000?ooo`3oool0gP3oool003H0oooo00<000000?ooo`3oool06`3oool00`000000
oooo0?ooo`3N0?ooo`00=`3oool2000001/0oooo00<000000?ooo`3oool0gP3oool003T0oooo0P00
00020?ooo`<00000503oool00`000000oooo0?ooo`3N0?ooo`00>`3oool6000001<0oooo00<00000
0?ooo`3oool0gP3oool003`0oooo1@00000C0?ooo`030000003oool0oooo0=h0oooo000l0?ooo`D0
00004`3oool00`000000oooo0?ooo`3N0?ooo`00?@3oool4000001<0oooo0P00003O0?ooo`00@@3o
ool200000140oooo00<000000?ooo`3oool0gP3oool004<0oooo0P00000?0?ooo`030000003oool0
oooo0=h0oooo00150?ooo`030000003oool0oooo00<000002@3oool00`000000oooo0?ooo`3N0?oo
o`00AP3oool6000000P0oooo00<000000?ooo`3oool0gP3oool004L0oooo1@0000080?ooo`030000
003oool0oooo0=h0oooo00170?ooo`D00000203oool00`000000oooo0?ooo`3N0?ooo`00B03oool4
000000P0oooo00<000000?ooo`3oool0gP3oool004`0oooo0P0000060?ooo`030000003oool0oooo
0=h0oooo001<0?ooo`030000003oool0000000D0oooo00<000000?ooo`3oool0gP3oool004T0oooo
1@0000000`3oool00000000000030?ooo`<00000gP3oool004T0oooo00@000000?ooo`3oool00000
103oool010000000oooo0?ooo`00003P0?ooo`00BP3oool00`000000oooo000000050?ooo`<00000
h03oool004/0oooo0P0000070?ooo`030000003oool0oooo0=h0oooo001<0?ooo`030000003oool0
oooo00D0oooo0`00003N0?ooo`00E03oool00`000000oooo0?ooo`0200000=`0oooo001D0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0f@3oool005@0oooo00<000000?ooo`3oool0
0`3oool200000=T0oooo001D0?ooo`030000003oool0oooo00D0oooo0P00003G0?ooo`00E03oool2
000000P0oooo0P00003E0?ooo`00E03oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0=80oooo001D0?ooo`030000003oool0oooo00X0oooo0P00003B0?ooo`00E03oool00`000000oooo
0?ooo`0<0?ooo`800000d03oool005@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?oo
o`0300000<X0oooo001D0?ooo`030000003oool0oooo00l0oooo1P0000390?ooo`00E03oool00`00
0000oooo0?ooo`0@0?ooo`D00000b@3oool005@0oooo00<000000?ooo`3oool0403oool500000<T0
oooo001D0?ooo`030000003oool0oooo0140oooo0`000000103oool000000000000000360?ooo`00
E03oool00`000000oooo0?ooo`0D0?ooo`D00000a@3oool005@0oooo0P00000E0?ooo`D00000a@3o
ool005@0oooo00<000000?ooo`3oool0503oool700000<<0oooo001D0?ooo`030000003oool0oooo
01D0oooo200000310?ooo`00E03oool00`000000oooo0?ooo`0G0?ooo`D0000000<0oooo0000003o
ool0_`3oool005@0oooo00<000000?ooo`3oool05`3oool500000080oooo0P00002n0?ooo`00E03o
ool00`000000oooo0?ooo`0G0?ooo`D00000103oool00`000000oooo0?ooo`2k0?ooo`00E03oool0
0`000000oooo0?ooo`0H0?ooo`<000001P3oool200000;/0oooo001D0?ooo`030000003oool0oooo
02<0oooo00<000000?ooo`3oool0^03oool005@0oooo00<000000?ooo`3oool0903oool200000;P0
oooo001D0?ooo`8000009`3oool00`000000oooo0?ooo`2e0?ooo`00E03oool00`000000oooo0?oo
o`0W0?ooo`800000]@3oool005@0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`2b
0?ooo`00E03oool00`000000oooo0?ooo`0Z0?ooo`800000/P3oool005@0oooo00<000000?ooo`3o
ool0;03oool00`000000oooo0?ooo`2_0?ooo`00E03oool00`000000oooo0?ooo`0]0?ooo`800000
[`3oool005@0oooo00<000000?ooo`3oool0;@3oool400000:d0oooo001:0?ooo`800000203oool0
0`000000oooo0?ooo`0/0?ooo`H00000[03oool004T0oooo00@000000?ooo`3oool000001`3oool0
0`000000oooo0?ooo`0/0?ooo`D0000000<0oooo000000000000ZP3oool004T0oooo00@000000?oo
o`3oool000001`3oool3000002`0oooo1@0000030?ooo`800000Z03oool004T0oooo00@000000?oo
o`3oool000001`3oool00`000000oooo0?ooo`0]0?ooo`<000001P3oool200000:H0oooo00190?oo
o`<00000203oool00`000000oooo0?ooo`0h0?ooo`030000003oool0oooo0:<0oooo00190?ooo`04
0000003oool0oooo000000L0oooo00<000000?ooo`3oool0>@3oool200000:<0oooo001:0?ooo`80
0000203oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0:00oooo001D0?ooo`030000
003oool0oooo03`0oooo0P00002P0?ooo`00E03oool00`000000oooo0?ooo`0n0?ooo`800000WP3o
ool005@0oooo00<000000?ooo`3oool0@03oool00`000000oooo0?ooo`2K0?ooo`00E03oool00`00
0000oooo0?ooo`110?ooo`800000V`3oool005@0oooo00<000000?ooo`3oool0@`3oool00`000000
oooo0?ooo`2H0?ooo`00E03oool2000004D0oooo0P00002H0?ooo`00E03oool00`000000oooo0?oo
o`160?ooo`800000UP3oool005@0oooo00<000000?ooo`3oool0B03oool00`000000oooo0?ooo`2C
0?ooo`00E03oool00`000000oooo0?ooo`190?ooo`800000T`3oool005@0oooo00<000000?ooo`3o
ool0B`3oool200000940oooo001D0?ooo`030000003oool0oooo04d0oooo0P00002?0?ooo`00E03o
ool00`000000oooo0?ooo`190?ooo`<000000`3oool00`000000oooo0?ooo`2<0?ooo`00E03oool0
0`000000oooo0?ooo`180?ooo`D000000`3oool2000008`0oooo001D0?ooo`030000003oool0oooo
04P0oooo1@0000050?ooo`030000003oool0oooo08T0oooo001D0?ooo`800000B@3oool5000000H0
oooo0P0000290?ooo`00E03oool00`000000oooo0?ooo`190?ooo`<000002@3oool2000008L0oooo
001D0?ooo`030000003oool0oooo05L0oooo00<000000?ooo`3oool00`0000210?ooo`00E03oool0
0`000000oooo0?ooo`1H0?ooo`H00000P03oool005@0oooo00<000000?ooo`3oool0F@3oool50000
0800oooo001D0?ooo`030000003oool0oooo05T0oooo1@000000103oool0000000000000001l0?oo
o`00E03oool00`000000oooo0?ooo`1J0?ooo`T00000N`3oool005@0oooo00<000000?ooo`3oool0
GP3oool5000007/0oooo001D0?ooo`030000003oool0oooo05h0oooo1@00001k0?ooo`00E03oool0
0`000000oooo0?ooo`1O0?ooo`@00000N`3oool005@0oooo0P00001T0?ooo`030000003oool0oooo
07P0oooo001D0?ooo`030000003oool0oooo06@0oooo0P00001h0?ooo`00E03oool00`000000oooo
0?ooo`1V0?ooo`030000003oool0oooo07D0oooo001D0?ooo`030000003oool0oooo06L0oooo0P00
001e0?ooo`00E03oool00`000000oooo0?ooo`1Y0?ooo`800000L`3oool005@0oooo00<000000?oo
o`3oool0J`3oool00`000000oooo0?ooo`1`0?ooo`00E03oool00`000000oooo0?ooo`1/0?ooo`80
0000L03oool004X0oooo0P0000080?ooo`030000003oool0oooo06h0oooo0P00001^0?ooo`00B@3o
ool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo0700oooo0P00001/0?ooo`00B@3o
ool010000000oooo0?ooo`0000070?ooo`<00000LP3oool00`000000oooo0?ooo`1Y0?ooo`00BP3o
ool2000000P0oooo00<000000?ooo`3oool0L`3oool2000006T0oooo00190?ooo`040000003oool0
oooo000000L0oooo00<000000?ooo`3oool0M@3oool00`000000oooo0?ooo`1V0?ooo`00B@3oool0
10000000oooo0?ooo`0000070?ooo`030000003oool0oooo07H0oooo0P00001V0?ooo`00BP3oool2
000000P0oooo00<000000?ooo`3oool0N03oool2000006@0oooo001D0?ooo`030000003oool0oooo
07X0oooo00<000000?ooo`3oool0H@3oool005@0oooo00<000000?ooo`3oool0N`3oool200000640
oooo001D0?ooo`030000003oool0oooo07d0oooo00<000000?ooo`3oool0GP3oool005@0oooo00<0
00000?ooo`3oool0OP3oool2000005h0oooo001D0?ooo`030000003oool0oooo0800oooo0P00001L
0?ooo`00E03oool2000008<0oooo00<000000?ooo`3oool0F@3oool005@0oooo00<000000?ooo`3o
ool0P`3oool2000005T0oooo001D0?ooo`030000003oool0oooo08D0oooo0P00001G0?ooo`00E03o
ool00`000000oooo0?ooo`270?ooo`800000E@3oool005@0oooo00<000000?ooo`3oool0R@3oool0
0`000000oooo0?ooo`1B0?ooo`00E03oool00`000000oooo0?ooo`230?ooo`<00000103oool20000
0580oooo001D0?ooo`030000003oool0oooo0880oooo1@0000050?ooo`030000003oool0oooo04l0
oooo001D0?ooo`030000003oool0oooo0880oooo1@0000060?ooo`800000C`3oool005@0oooo00<0
00000?ooo`3oool0PP3oool5000000P0oooo0P00001=0?ooo`00E03oool2000008@0oooo0`00000;
0?ooo`030000003oool0oooo04X0oooo001D0?ooo`030000003oool0oooo0980oooo0P00001:0?oo
o`00E03oool00`000000oooo0?ooo`2D0?ooo`030000003oool0oooo04L0oooo001D0?ooo`030000
003oool0oooo09D0oooo0`0000160?ooo`00E03oool00`000000oooo0?ooo`2D0?ooo`D00000A@3o
ool005@0oooo00<000000?ooo`3oool0U03oool6000004@0oooo001D0?ooo`030000003oool0oooo
09@0oooo1@0000000`3oool00000000000120?ooo`00E03oool00`000000oooo0?ooo`2E0?ooo`<0
0000103oool00`000000oooo0?ooo`0o0?ooo`00E03oool00`000000oooo0?ooo`2M0?ooo`800000
?`3oool005@0oooo00<000000?ooo`3oool0W`3oool2000003d0oooo001D0?ooo`800000XP3oool0
0`000000oooo0?ooo`0j0?ooo`00E03oool00`000000oooo0?ooo`2R0?ooo`800000>P3oool005@0
oooo00<000000?ooo`3oool0Y03oool00`000000oooo0?ooo`0g0?ooo`00E03oool00`000000oooo
0?ooo`2U0?ooo`800000=`3oool005@0oooo00<000000?ooo`3oool0Y`3oool2000003D0oooo001D
0?ooo`030000003oool0oooo0:T0oooo00<000000?ooo`3oool0<P3oool005@0oooo00<000000?oo
o`3oool0ZP3oool200000380oooo00160?ooo`030000003oool0oooo00<00000203oool00`000000
oooo0?ooo`2/0?ooo`030000003oool0oooo02l0oooo00160?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2]0?ooo`800000;`3oool004H0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`050?ooo`<00000[`3oool2000002d0
oooo00160?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`2a0?ooo`030000003oool0oooo02X0oooo00160?ooo`040000003oool0oooo00000080
oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`2b0?ooo`800000:P3oool004D0oooo
0P0000020?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0]03oool2000002P0
oooo00160?ooo`040000003oool0oooo0?ooo`800000203oool00`000000oooo0?ooo`2f0?ooo`80
00002P3oool3000001T0oooo001D0?ooo`030000003oool0oooo0;P0oooo00<000000?ooo`3oool0
1P3oool5000001P0oooo001D0?ooo`030000003oool0oooo0;T0oooo0P0000060?ooo`D00000603o
ool005@0oooo00<000000?ooo`3oool0^`3oool2000000@0oooo1@00000H0?ooo`00E03oool00`00
0000oooo0?ooo`2m0?ooo`030000003oool0oooo0080oooo0`00000I0?ooo`00E03oool00`000000
oooo0?ooo`2n0?ooo`8000007P3oool005@0oooo0P0000310?ooo`800000703oool005@0oooo00<0
00000?ooo`3oool0`P3oool2000001X0oooo001D0?ooo`030000003oool0oooo0<@0oooo00<00000
0?ooo`3oool05`3oool005@0oooo00<000000?ooo`3oool0a@3oool2000001L0oooo001D0?ooo`03
0000003oool0oooo0<L0oooo00<000000?ooo`3oool0503oool005@0oooo00<000000?ooo`3oool0
b03oool2000001@0oooo001D0?ooo`030000003oool0oooo0<X0oooo0P00000B0?ooo`00E03oool0
0`000000oooo0?ooo`3<0?ooo`030000003oool0oooo00l0oooo001D0?ooo`030000003oool0oooo
0<d0oooo0P00000?0?ooo`00E03oool200000=00oooo00<000000?ooo`3oool0303oool005@0oooo
00<000000?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool005@0oooo00<00000
0?ooo`3oool0gP3oool005@0oooo00<000000?ooo`3oool0gP3oool00001\
\>"],
  ImageRangeCache->{{{0, 308.75}, {190.188, 0}} -> {-3.62634, 1.17788, \
0.0429236, 0.0528852}},
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Programming", "Subsection",
  CellTags->{"b:0.4", "IPM3.1.1.5", "c:7"}],

Cell[TextData[{
  "With a copy of ",
  StyleBox["The Mathematica Book",
    FontSlant->"Italic"],
  " (",
  ButtonBox["Wolfram 2003",
    ButtonData:>"C",
    ButtonStyle->"MainBookLink"],
  ") or one of the many tutorial books (see, for example, Glynn and Gray \
1999) describing the vast array of computational tasks that can be performed \
with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", it would seem you can compute just about anything you might want. But \
that impression is mistaken. There are simply more kinds of calculations than \
could possibly be included in a single program. Whether you are interested in \
computing bowling scores or finding the mean square distance of a random walk \
on a torus, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " does not have a built-in function to do everything that a user could \
possibly want. What it ",
  StyleBox["does",
    FontSlant->"Italic"],
  " have \[Dash] and what really makes it the amazingly useful tool it is \
\[Dash] is the capability for users to define their own functions. This is \
called ",
  StyleBox["programming",
    FontSlant->"Italic"],
  ", and it is what this book is all about."
}], "TextFirst",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[TextData[{
  "Sometimes, the programs you create will be succinct and focused on a very \
specific task. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " possesses a rich set of tools that enable you to quickly and naturally \
translate the statement of a problem into a program. For example, the \
following program defines a test for perfect numbers, numbers that are equal \
to the sum of their proper divisors."
}], "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(PerfectQ[n_] := Apply[Plus, Divisors[n]] == 2  n\)], "Input",
  CellLabel->"In[30]:=",
  CellTags->{"i:20", "b:0.4", "IPM3.1.1.5"}],

Cell[TextData[{
  "We then define another function that selects those numbers from a range of \
integers that pass this ",
  StyleBox["PerfectQ", "MR"],
  " test."
}], "TextFirst",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(PerfectSearch[n_] := Select[Range[n], PerfectQ]\)], "Input",
  CellLabel->"In[31]:=",
  CellTags->{"i:21", "b:0.4", "IPM3.1.1.5"}],

Cell["This then finds all perfect numbers less than 1,000,000.", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PerfectSearch[10\^6]\)], "Input",
  CellLabel->"In[32]:=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \({6, 28, 496, 8128}\)], "Output",
  CellLabel->"Out[32]=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],

Cell[TextData[{
  "Here are two functions for representing regular polygons. The first \
defines the vertices of a regular ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-gon, while the second uses those vertices to create a polygon graphics \
object that can then be displayed with the built-in ",
  StyleBox["Show", "MR"],
  " function."
}], "Text",
  CellTags->{"i:22", "b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(vertices[n_Integer, r_:  1] := 
      Table[{r\ Cos[\(2  \[Alpha]\ \[Pi]\)\/n], 
          r\ Sin[\(2\ \[Alpha]\ \[Pi]\)\/n]}, {\[Alpha], 0, 
          n - 1}]\)], "Input",
  CellLabel->"In[33]:=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(RegularPolygon[n_] := 
      Graphics[Line[vertices[n] /. {a_, b__} \[Rule] {a, b, a}], 
        AspectRatio \[Rule] Automatic]\)], "Input",
  CellLabel->"In[34]:=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[RegularPolygon[5]]\)], "Input",
  CellLabel->"In[35]:=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.05146 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.449727 0.526463 0.525731 0.526463 [
[ 0 0 0 0 ]
[ 1 1.05146 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 1.05146 L
0 1.05146 L
closepath
clip
newpath
0 g
.5 Mabswid
[ ] 0 setdash
.97619 .52573 m
.61241 1.02643 L
.02381 .83518 L
.02381 .21628 L
.61241 .02503 L
.97619 .52573 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[35]:=",
  ImageSize->{164.875, 173.125},
  ImageMargins->{{46.875, 0}, {0, 1.0625}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40002T0000[B000`400?l00000o`00003oY03oool0
0:@0oooo002T0?ooo`00Y03oool006<0oooo0P00000o0?ooo`00H03oool300000080oooo00<00000
0?ooo`3oool0?03oool005d0oooo0`0000050?ooo`030000003oool0oooo03`0oooo001J0?ooo`<0
00002@3oool00`000000oooo0?ooo`0k0?ooo`00E`3oool3000000d0oooo00<000000?ooo`3oool0
>P3oool005@0oooo0`00000A0?ooo`030000003oool0oooo03T0oooo001A0?ooo`<00000503oool0
0`000000oooo0?ooo`0i0?ooo`00CP3oool3000001P0oooo00<000000?ooo`3oool0>03oool004/0
oooo0`00000L0?ooo`030000003oool0oooo03L0oooo00180?ooo`<000007`3oool00`000000oooo
0?ooo`0g0?ooo`00A@3oool3000002<0oooo00<000000?ooo`3oool0=P3oool00480oooo0`00000W
0?ooo`030000003oool0oooo03D0oooo000o0?ooo`<00000:`3oool00`000000oooo0?ooo`0d0?oo
o`00?03oool3000002h0oooo00<000000?ooo`3oool0=03oool003T0oooo0`00000b0?ooo`030000
003oool0oooo03<0oooo000f0?ooo`<00000=P3oool00`000000oooo0?ooo`0b0?ooo`00<P3oool4
000003X0oooo00<000000?ooo`3oool0<@3oool002l0oooo0`00000n0?ooo`030000003oool0oooo
0340oooo000/0?ooo`<00000@P3oool00`000000oooo0?ooo`0`0?ooo`00:@3oool3000004H0oooo
00<000000?ooo`3oool0;`3oool002H0oooo0`0000190?ooo`030000003oool0oooo02l0oooo000S
0?ooo`<00000C@3oool00`000000oooo0?ooo`0^0?ooo`00803oool300000540oooo00<000000?oo
o`3oool0;@3oool001d0oooo0`00001E0?ooo`030000003oool0oooo02`0oooo000J0?ooo`<00000
F03oool00`000000oooo0?ooo`0/0?ooo`005`3oool3000005`0oooo00<000000?ooo`3oool0:`3o
ool001@0oooo0`00001P0?ooo`030000003oool0oooo02X0oooo000A0?ooo`<00000H`3oool00`00
0000oooo0?ooo`0Z0?ooo`003P3oool3000006L0oooo00<000000?ooo`3oool0:@3oool000/0oooo
0`00001[0?ooo`030000003oool0oooo02P0oooo00080?ooo`<00000K`3oool00`000000oooo0?oo
o`0W0?ooo`001@3oool300000780oooo00<000000?ooo`3oool09`3oool000<0oooo0P00001f0?oo
o`030000003oool0oooo02H0oooo00030?ooo`030000003oool0oooo07H0oooo00<000000?ooo`3o
ool09@3oool000<0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?ooo`0U0?ooo`000`3o
ool00`000000oooo0?ooo`1g0?ooo`030000003oool0oooo02@0oooo00030?ooo`030000003oool0
oooo07P0oooo00<000000?ooo`3oool08`3oool000<0oooo00<000000?ooo`3oool0N@3oool00`00
0000oooo0?ooo`0R0?ooo`000`3oool00`000000oooo0?ooo`1i0?ooo`030000003oool0oooo0280
oooo00030?ooo`030000003oool0oooo07X0oooo00<000000?ooo`3oool08@3oool000<0oooo00<0
00000?ooo`3oool0N`3oool00`000000oooo0?ooo`0P0?ooo`000`3oool00`000000oooo0?ooo`1k
0?ooo`030000003oool0oooo0200oooo00030?ooo`030000003oool0oooo07`0oooo00<000000?oo
o`3oool07`3oool000<0oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`0N0?ooo`00
0`3oool00`000000oooo0?ooo`1n0?ooo`030000003oool0oooo01d0oooo00030?ooo`030000003o
ool0oooo07h0oooo00<000000?ooo`3oool07@3oool000<0oooo00<000000?ooo`3oool0O`3oool0
0`000000oooo0?ooo`0L0?ooo`000`3oool00`000000oooo0?ooo`200?ooo`030000003oool0oooo
01/0oooo00030?ooo`030000003oool0oooo0840oooo00<000000?ooo`3oool06P3oool000<0oooo
00<000000?ooo`3oool0P@3oool00`000000oooo0?ooo`0J0?ooo`000`3oool00`000000oooo0?oo
o`220?ooo`030000003oool0oooo01T0oooo00030?ooo`030000003oool0oooo08<0oooo00<00000
0?ooo`3oool0603oool000<0oooo00<000000?ooo`3oool0P`3oool00`000000oooo0?ooo`0H0?oo
o`000`3oool00`000000oooo0?ooo`240?ooo`030000003oool0oooo01L0oooo00030?ooo`030000
003oool0oooo08D0oooo00<000000?ooo`3oool05P3oool000<0oooo00<000000?ooo`3oool0QP3o
ool00`000000oooo0?ooo`0E0?ooo`000`3oool00`000000oooo0?ooo`260?ooo`030000003oool0
oooo01D0oooo00030?ooo`030000003oool0oooo08L0oooo00<000000?ooo`3oool0503oool000<0
oooo00<000000?ooo`3oool0R03oool00`000000oooo0?ooo`0C0?ooo`000`3oool00`000000oooo
0?ooo`280?ooo`030000003oool0oooo01<0oooo00030?ooo`030000003oool0oooo08T0oooo00<0
00000?ooo`3oool04P3oool000<0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`0A
0?ooo`000`3oool00`000000oooo0?ooo`2;0?ooo`030000003oool0oooo0100oooo00030?ooo`03
0000003oool0oooo08/0oooo00<000000?ooo`3oool0403oool000<0oooo00<000000?ooo`3oool0
S03oool00`000000oooo0?ooo`0?0?ooo`000`3oool00`000000oooo0?ooo`2=0?ooo`030000003o
ool0oooo00h0oooo00030?ooo`030000003oool0oooo08d0oooo00<000000?ooo`3oool03P3oool0
00<0oooo00<000000?ooo`3oool0SP3oool00`000000oooo0?ooo`0=0?ooo`000`3oool00`000000
oooo0?ooo`2?0?ooo`030000003oool0oooo00`0oooo00030?ooo`030000003oool0oooo0900oooo
00<000000?ooo`3oool02`3oool000<0oooo00<000000?ooo`3oool0T03oool00`000000oooo0?oo
o`0;0?ooo`000`3oool00`000000oooo0?ooo`2A0?ooo`030000003oool0oooo00X0oooo00030?oo
o`030000003oool0oooo0980oooo00<000000?ooo`3oool02@3oool000<0oooo00<000000?ooo`3o
ool0T`3oool00`000000oooo0?ooo`080?ooo`000`3oool00`000000oooo0?ooo`2C0?ooo`030000
003oool0oooo00P0oooo00030?ooo`030000003oool0oooo09@0oooo00<000000?ooo`3oool01`3o
ool000<0oooo00<000000?ooo`3oool0U@3oool00`000000oooo0?ooo`060?ooo`000`3oool00`00
0000oooo0?ooo`2E0?ooo`030000003oool0oooo00H0oooo00030?ooo`030000003oool0oooo09H0
oooo00<000000?ooo`3oool01@3oool000<0oooo00<000000?ooo`3oool0U`3oool00`000000oooo
0?ooo`040?ooo`000`3oool00`000000oooo0?ooo`2H0?ooo`030000003oool0oooo00<0oooo0003
0?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool00`3oool000<0oooo00<000000?oo
o`3oool0V@3oool00`000000oooo0?ooo`020?ooo`000`3oool00`000000oooo0?ooo`2H0?ooo`03
0000003oool0oooo00<0oooo00030?ooo`030000003oool0oooo09P0oooo00<000000?ooo`3oool0
0`3oool000<0oooo00<000000?ooo`3oool0U`3oool00`000000oooo0?ooo`040?ooo`000`3oool0
0`000000oooo0?ooo`2F0?ooo`030000003oool0oooo00D0oooo00030?ooo`030000003oool0oooo
09D0oooo00<000000?ooo`3oool01P3oool000<0oooo00<000000?ooo`3oool0U@3oool00`000000
oooo0?ooo`060?ooo`000`3oool00`000000oooo0?ooo`2D0?ooo`030000003oool0oooo00L0oooo
00030?ooo`030000003oool0oooo09<0oooo00<000000?ooo`3oool0203oool000<0oooo00<00000
0?ooo`3oool0T`3oool00`000000oooo0?ooo`080?ooo`000`3oool00`000000oooo0?ooo`2B0?oo
o`030000003oool0oooo00T0oooo00030?ooo`030000003oool0oooo0940oooo00<000000?ooo`3o
ool02P3oool000<0oooo00<000000?ooo`3oool0T03oool00`000000oooo0?ooo`0;0?ooo`000`3o
ool00`000000oooo0?ooo`2@0?ooo`030000003oool0oooo00/0oooo00030?ooo`030000003oool0
oooo08l0oooo00<000000?ooo`3oool0303oool000<0oooo00<000000?ooo`3oool0SP3oool00`00
0000oooo0?ooo`0=0?ooo`000`3oool00`000000oooo0?ooo`2=0?ooo`030000003oool0oooo00h0
oooo00030?ooo`030000003oool0oooo08d0oooo00<000000?ooo`3oool03P3oool000<0oooo00<0
00000?ooo`3oool0S03oool00`000000oooo0?ooo`0?0?ooo`000`3oool00`000000oooo0?ooo`2;
0?ooo`030000003oool0oooo0100oooo00030?ooo`030000003oool0oooo08/0oooo00<000000?oo
o`3oool0403oool000<0oooo00<000000?ooo`3oool0RP3oool00`000000oooo0?ooo`0A0?ooo`00
0`3oool00`000000oooo0?ooo`290?ooo`030000003oool0oooo0180oooo00030?ooo`030000003o
ool0oooo08P0oooo00<000000?ooo`3oool04`3oool000<0oooo00<000000?ooo`3oool0R03oool0
0`000000oooo0?ooo`0C0?ooo`000`3oool00`000000oooo0?ooo`270?ooo`030000003oool0oooo
01@0oooo00030?ooo`030000003oool0oooo08H0oooo00<000000?ooo`3oool05@3oool000<0oooo
00<000000?ooo`3oool0QP3oool00`000000oooo0?ooo`0E0?ooo`000`3oool00`000000oooo0?oo
o`250?ooo`030000003oool0oooo01H0oooo00030?ooo`030000003oool0oooo08@0oooo00<00000
0?ooo`3oool05`3oool000<0oooo00<000000?ooo`3oool0P`3oool00`000000oooo0?ooo`0H0?oo
o`000`3oool00`000000oooo0?ooo`230?ooo`030000003oool0oooo01P0oooo00030?ooo`030000
003oool0oooo0880oooo00<000000?ooo`3oool06@3oool000<0oooo00<000000?ooo`3oool0P@3o
ool00`000000oooo0?ooo`0J0?ooo`000`3oool00`000000oooo0?ooo`210?ooo`030000003oool0
oooo01X0oooo00030?ooo`030000003oool0oooo0800oooo00<000000?ooo`3oool06`3oool000<0
oooo00<000000?ooo`3oool0O`3oool00`000000oooo0?ooo`0L0?ooo`000`3oool00`000000oooo
0?ooo`1n0?ooo`030000003oool0oooo01d0oooo00030?ooo`030000003oool0oooo07h0oooo00<0
00000?ooo`3oool07@3oool000<0oooo00<000000?ooo`3oool0O@3oool00`000000oooo0?ooo`0N
0?ooo`000`3oool00`000000oooo0?ooo`1l0?ooo`030000003oool0oooo01l0oooo00030?ooo`03
0000003oool0oooo07/0oooo00<000000?ooo`3oool0803oool000<0oooo00<000000?ooo`3oool0
N`3oool00`000000oooo0?ooo`0P0?ooo`000`3oool00`000000oooo0?ooo`1j0?ooo`030000003o
ool0oooo0240oooo00030?ooo`030000003oool0oooo07T0oooo00<000000?ooo`3oool08P3oool0
00<0oooo00<000000?ooo`3oool0N@3oool00`000000oooo0?ooo`0R0?ooo`000`3oool00`000000
oooo0?ooo`1h0?ooo`030000003oool0oooo02<0oooo00030?ooo`030000003oool0oooo07L0oooo
00<000000?ooo`3oool0903oool000<0oooo00<000000?ooo`3oool0MP3oool00`000000oooo0?oo
o`0U0?ooo`000`3oool00`000000oooo0?ooo`1f0?ooo`030000003oool0oooo02D0oooo00030?oo
o`030000003oool0oooo07D0oooo00<000000?ooo`3oool09P3oool000<0oooo0P00001e0?ooo`03
0000003oool0oooo02L0oooo00050?ooo`<00000LP3oool00`000000oooo0?ooo`0W0?ooo`00203o
ool3000006h0oooo00<000000?ooo`3oool0:03oool000/0oooo0`00001Z0?ooo`030000003oool0
oooo02T0oooo000>0?ooo`@00000I@3oool00`000000oooo0?ooo`0Z0?ooo`004P3oool300000680
oooo00<000000?ooo`3oool0:P3oool001D0oooo0`00001N0?ooo`030000003oool0oooo02/0oooo
000H0?ooo`<00000FP3oool00`000000oooo0?ooo`0/0?ooo`006`3oool3000005L0oooo00<00000
0?ooo`3oool0;03oool001h0oooo0`00001C0?ooo`030000003oool0oooo02d0oooo000Q0?ooo`<0
0000C`3oool00`000000oooo0?ooo`0^0?ooo`00903oool3000004/0oooo00<000000?ooo`3oool0
;`3oool002L0oooo100000170?ooo`030000003oool0oooo02l0oooo000[0?ooo`<00000@`3oool0
0`000000oooo0?ooo`0`0?ooo`00;P3oool3000003l0oooo00<000000?ooo`3oool0<@3oool00340
oooo0`00000l0?ooo`030000003oool0oooo0340oooo000d0?ooo`<00000>03oool00`000000oooo
0?ooo`0b0?ooo`00=`3oool3000003@0oooo00<000000?ooo`3oool0<`3oool003X0oooo0`00000`
0?ooo`030000003oool0oooo03@0oooo000m0?ooo`@00000;03oool00`000000oooo0?ooo`0d0?oo
o`00@@3oool3000002P0oooo00<000000?ooo`3oool0=@3oool004@0oooo0`00000T0?ooo`030000
003oool0oooo03H0oooo00170?ooo`<00000803oool00`000000oooo0?ooo`0g0?ooo`00BP3oool3
000001d0oooo00<000000?ooo`3oool0=`3oool004d0oooo0`00000I0?ooo`030000003oool0oooo
03P0oooo001@0?ooo`<000005@3oool00`000000oooo0?ooo`0i0?ooo`00D`3oool300000180oooo
00<000000?ooo`3oool0>@3oool005H0oooo1000000=0?ooo`030000003oool0oooo03X0oooo001J
0?ooo`<000002@3oool00`000000oooo0?ooo`0k0?ooo`00G@3oool3000000D0oooo00<000000?oo
o`3oool0?03oool00600oooo0`0000020?ooo`030000003oool0oooo03`0oooo001S0?ooo`800000
?`3oool00:@0oooo002T0?ooo`00Y03oool00:@0oooo0000\
\>"],
  ImageRangeCache->{{{0, 163.875}, {172.125, 0}} -> {-0.855262, -0.99862, \
0.0116034, 0.0116034}},
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[35]=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],

Cell["\<\
As another example of a succinct program, here is an iterative function that \
implements the well-known Newton method for root finding.\
\>", "Text",
  CellTags->{"i:23", "b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(NewtonZero[f_, xi_] := 
      NestWhile[\((# - f[#]\/\(f'\)[#])\) &, xi, Unequal, 2]\)], "Input",
  CellLabel->"In[36]:=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(g[x_] := x\^3 - 2\ x\^2 + 1\)], "Input",
  CellLabel->"In[37]:=",
  AspectRatioFixed->True,
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(NewtonZero[g, 2.0]\)], "Input",
  CellLabel->"In[38]:=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(1.6180339887498951`\)], "Output",
  CellLabel->"Out[38]=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],

Cell["\<\
Of course, sometimes the task at hand requires a more involved program, \
stretching across several lines (or even pages) of code. For example, here is \
a slightly longer program to compute the score of a game of bowling, given a \
list of the number of pins scored by each ball.\
\>", "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(BowlingScore[pins_] := 
      Module[{score}, score[{x_, y_, z_}] := x + y + z; 
        score[{10, y_, z_, r___}] := 10 + y + z + score[{y, z, r}]; 
        score[{x_, y_, z_, r___}] := 
          x + y + z + score[{z, r}] /; x + y == 10; 
        score[{x_, y_, r___}] := x + y + score[{r}] /; x + y < 10; 
        score[If[
            pins\[LeftDoubleBracket]\(-2\)\[RightDoubleBracket] + 
                pins\[LeftDoubleBracket]\(-1\)\[RightDoubleBracket] \
\[GreaterEqual] 10, pins, Append[pins, 0]]]]\)], "Input",
  CellLabel->"In[39]:=",
  CellTags->{"i:24", "b:0.4", "IPM3.1.1.5"}],

Cell["\<\
Here is the computation for a \[OpenCurlyDoubleQuote]perfect\
\[CloseCurlyDoubleQuote] game \[Dash] 12 strikes in a row.\
\>", "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BowlingScore[{10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 
        10}]\)], "Input",
  CellLabel->"In[40]:=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[BoxData[
    \(300\)], "Output",
  CellLabel->"Out[40]=",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],

Cell[TextData[{
  "These examples use a variety of programming styles: functional \
programming, rule-based programming, the use of anonymous functions, and \
more. We do not expect you to understand the examples in this section at this \
point \[Dash] that is why we wrote this book! What you should understand is \
that in many ways ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is designed to be as broadly useful as possible and that there are many \
calculations for which ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " does not have a built-in function, so, to make full use of its many \
capabilities, you will sometimes need to program. The main purpose of this \
book is to show you how."
}], "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell["\<\
Another purpose is to teach you the basic principles of programming. These \
principles \[Dash] making assignments, defining rules, using conditionals, \
recursion, and iteration \[Dash] are applicable (with great differences in \
detail, to be sure) to all other programming languages.\
\>", "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Symbolic and interactive documents", "Subsection",
  CellTags->{"b:0.5", "IPM3.1.1.6", "c:8"}],

Cell[TextData[{
  "In addition to the computational tools that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " provides for what many professionals associate with technical computing, \
it also contains tools for creating and modifying the user interface to such \
tasks. These tools include hyperlinks for jumping to other locations within a \
document or across files, buttons to perform tasks that you might normally \
associate with a command-line interface, and tools to modify and manipulate \
the appearance and functionality of your ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebooks directly. In this section we will give a few short examples of \
what is possible, waiting until Chapter 10 for a methodical look at how to \
program these elements."
}], "TextFirst",
  CellTags->{"i:25", "b:0.5", "IPM3.1.1.6"}],

Cell["\<\
The first example takes the code necessary to display a polyhedron and puts \
it in a button. The two lines of code that could be evaluated normally in a \
notebook first load a package and then display an icosahedron in the \
notebook.\
\>", "Text",
  CellTags->{"i:26", "b:0.5", "IPM3.1.1.6"}],

Cell[BoxData[
    \(Needs["\<Graphics`Polyhedra`\>"]\)], "Input",
  CellLabel->"In[41]:=",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[Stellate[Polyhedron[Icosahedron]]]\)], "Input",
  CellLabel->"In[42]:=",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.00162 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
-0.0206914 1.08755 -7.41594e-017 1.08755 [
[ 0 0 0 0 ]
[ 1 1.00162 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.38643 1.00162 L
s
.38643 1.00162 m
.40217 .57872 L
s
.40217 .57872 m
.07192 .25073 L
s
.68538 0 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.68538 0 m
.07192 .25073 L
s
.40217 .57872 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.38643 1.00162 L
s
.38643 1.00162 m
.40217 .57872 L
s
0 0 m
1 0 L
1 1.00162 L
0 1.00162 L
closepath
clip
newpath
.5 Mabswid
.786 .442 .405 r
.44491 .44907 m .36477 .52184 L .55473 .57484 L closepath p F P
0 g
s
.37 .395 .757 r
.44491 .44907 m .55473 .57484 L .52308 .38117 L closepath p F P
0 g
s
0 0 0 r
.44491 .44907 m .52308 .38117 L .36477 .52184 L closepath p F P
0 g
s
.366 .042 .277 r
.39521 .65616 m .44391 .71565 L .55473 .57484 L closepath p F P
0 g
s
.702 .725 .868 r
.39521 .65616 m .55473 .57484 L .36477 .52184 L closepath p F P
0 g
s
0 0 .535 r
.39521 .65616 m .36477 .52184 L .44391 .71565 L closepath p F P
0 g
s
.299 .033 .331 r
.65615 .42467 m .55473 .57484 L .70544 .48388 L closepath p F P
0 g
s
.859 .738 .729 r
.65615 .42467 m .52308 .38117 L .55473 .57484 L closepath p F P
0 g
s
.641 .082 0 r
.65615 .42467 m .70544 .48388 L .52308 .38117 L closepath p F P
0 g
s
.873 .707 .674 r
.57791 .76621 m .55473 .57484 L .44391 .71565 L closepath p F P
0 g
s
.311 .298 .683 r
.57791 .76621 m .65788 .69455 L .55473 .57484 L closepath p F P
0 g
s
.6 .813 .982 r
.57791 .76621 m .44391 .71565 L .65788 .69455 L closepath p F P
0 g
s
.721 .348 .342 r
.74263 .62049 m .55473 .57484 L .65788 .69455 L closepath p F P
0 g
s
.643 .687 .876 r
.74263 .62049 m .70544 .48388 L .55473 .57484 L closepath p F P
0 g
s
.971 .845 .659 r
.74263 .62049 m .65788 .69455 L .70544 .48388 L closepath p F P
0 g
s
.698 .565 .701 r
.33115 .32429 m .36477 .52184 L .52308 .38117 L closepath p F P
0 g
s
0 0 .52 r
.33115 .32429 m .38251 .38653 L .36477 .52184 L closepath p F P
0 g
s
.608 .076 0 r
.33115 .32429 m .52308 .38117 L .38251 .38653 L closepath p F P
0 g
s
.706 .538 .659 r
.24235 .68284 m .44391 .71565 L .36477 .52184 L closepath p F P
0 g
s
0 0 0 r
.24235 .68284 m .36477 .52184 L .32852 .60768 L closepath p F P
0 g
s
.817 .41 .277 r
.19481 .46702 m .32852 .60768 L .36477 .52184 L closepath p F P
0 g
s
.712 .79 .912 r
.19481 .46702 m .36477 .52184 L .38251 .38653 L closepath p F P
0 g
s
.654 .533 .704 r
.69731 .27878 m .52308 .38117 L .70544 .48388 L closepath p F P
0 g
s
0 0 0 r
.69731 .27878 m .60797 .35949 L .52308 .38117 L closepath p F P
0 g
s
.217 .339 .774 r
.48589 .20782 m .52308 .38117 L .60797 .35949 L closepath p F P
0 g
s
.899 .805 .747 r
.48589 .20782 m .38251 .38653 L .52308 .38117 L closepath p F P
0 g
s
.559 .729 .954 r
.24235 .68284 m .32852 .60768 L .44391 .71565 L closepath p F P
0 g
s
.927 .746 .637 r
.33664 .82736 m .44391 .71565 L .32852 .60768 L closepath p F P
0 g
s
.666 .487 .633 r
.56015 .87859 m .65788 .69455 L .44391 .71565 L closepath p F P
0 g
s
.079 .258 .746 r
.56015 .87859 m .44391 .71565 L .52308 .72469 L closepath p F P
0 g
s
.145 0 0 r
.33664 .82736 m .52308 .72469 L .44391 .71565 L closepath p F P
0 g
s
.952 .764 .613 r
.69731 .27878 m .70544 .48388 L .60797 .35949 L closepath p F P
0 g
s
.633 .483 .662 r
.85199 .62068 m .70544 .48388 L .65788 .69455 L closepath p F P
0 g
s
.804 .349 .159 r
.85199 .62068 m .69976 .56825 L .70544 .48388 L closepath p F P
0 g
s
0 0 .084 r
.82611 .39367 m .70544 .48388 L .69976 .56825 L closepath p F P
0 g
s
.593 .717 .93 r
.82611 .39367 m .60797 .35949 L .70544 .48388 L closepath p F P
0 g
s
.702 .816 .936 r
.85199 .62068 m .65788 .69455 L .69976 .56825 L closepath p F P
0 g
s
.92 .834 .744 r
.56015 .87859 m .52308 .72469 L .65788 .69455 L closepath p F P
0 g
s
.67 .196 .118 r
.73868 .79032 m .65788 .69455 L .52308 .72469 L closepath p F P
0 g
s
.065 .115 .604 r
.73868 .79032 m .69976 .56825 L .65788 .69455 L closepath p F P
0 g
s
.624 .472 .656 r
.19481 .46702 m .38251 .38653 L .32852 .60768 L closepath p F P
0 g
s
.661 .476 .624 r
.48589 .20782 m .60797 .35949 L .38251 .38653 L closepath p F P
0 g
s
.635 .791 .957 r
.45664 .27423 m .38251 .38653 L .60797 .35949 L closepath p F P
0 g
s
.22 .231 .659 r
.45664 .27423 m .48535 .50673 L .38251 .38653 L closepath p F P
0 g
s
.947 .817 .683 r
.25792 .45112 m .32852 .60768 L .38251 .38653 L closepath p F P
0 g
s
.707 .293 .259 r
.25792 .45112 m .38251 .38653 L .48535 .50673 L closepath p F P
0 g
s
.628 .699 .896 r
.25792 .45112 m .48535 .50673 L .32852 .60768 L closepath p F P
0 g
s
.648 .528 .703 r
.33664 .82736 m .32852 .60768 L .52308 .72469 L closepath p F P
0 g
s
.722 .218 .033 r
.35521 .69402 m .52308 .72469 L .32852 .60768 L closepath p F P
0 g
s
.184 0 .222 r
.35521 .69402 m .32852 .60768 L .48535 .50673 L closepath p F P
0 g
s
.705 .534 .654 r
.82611 .39367 m .69976 .56825 L .60797 .35949 L closepath p F P
0 g
s
.892 .722 .664 r
.45664 .27423 m .60797 .35949 L .48535 .50673 L closepath p F P
0 g
s
0 .115 .646 r
.68527 .40123 m .60797 .35949 L .69976 .56825 L closepath p F P
0 g
s
.266 0 .159 r
.68527 .40123 m .48535 .50673 L .60797 .35949 L closepath p F P
0 g
s
.697 .564 .701 r
.73868 .79032 m .52308 .72469 L .69976 .56825 L closepath p F P
0 g
s
0 0 0 r
.62474 .66693 m .69976 .56825 L .52308 .72469 L closepath p F P
0 g
s
.296 .351 .75 r
.62474 .66693 m .52308 .72469 L .48535 .50673 L closepath p F P
0 g
s
.876 .757 .728 r
.35521 .69402 m .48535 .50673 L .52308 .72469 L closepath p F P
0 g
s
.697 .742 .886 r
.68527 .40123 m .69976 .56825 L .48535 .50673 L closepath p F P
0 g
s
.786 .409 .341 r
.62474 .66693 m .48535 .50673 L .69976 .56825 L closepath p F P
0 g
s
.25 Mabswid
.68538 0 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.68538 0 m
.07192 .25073 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[42]:=",
  ImageSize->{224.125, 224.25},
  ImageMargins->{{42.4375, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40003P0000h2000`400?l00000o`00003oU`3oool2
000004L0oooo002E0?ooo`80000000<0oooo000000000000AP3oool00980oooo0`0000030?ooo`80
0000AP3oool00900oooo0P0000060?ooo`030000003oool0000004D0oooo002>0?ooo`800000203o
ool010000000oooo0?ooo`0000140?ooo`00R`3oool3000000X0oooo00@000000?ooo`3oool00000
A03oool008T0oooo0P00000=0?ooo`050000003oool0oooo0?ooo`000000@`3oool008H0oooo0`00
000?0?ooo`050000003oool0oooo0?ooo`000000@`3oool008@0oooo0P00000C0?ooo`050000003o
ool0oooo0?ooo`000000@P3oool00840oooo0`00000E0?ooo`030000003oool0oooo0080oooo00<0
00000?ooo`3oool0?`3oool007l0oooo0P00000H0?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool0?`3oool007d0oooo0P00000J0?ooo`030000003oool0oooo00<0oooo00<000000?oo
o`3oool0?P3oool007X0oooo0`00000L0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3o
ool0?@3oool007P0oooo0P00000O0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0
?@3oool007D0oooo0`00000Q0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0?03o
ool007<0oooo0P00000T0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0?03oool0
0700oooo0`00000V0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0>`3oool006h0
oooo0P00000Y0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0>P3oool006`0oooo
0P00000[0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0>P3oool006T0oooo0`00
000]0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0>@3oool006L0oooo0P00000`
0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0>03oool006@0oooo0`00000b0?oo
o`030000003oool0oooo00T0oooo00<000000?ooo`3oool0>03oool00680oooo0P00000e0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0=`3oool005l0oooo0`00000g0?ooo`030000
003oool0oooo00X0oooo00<000000?ooo`3oool0=`3oool005d0oooo0P00000k0?ooo`030000003o
ool0oooo00X0oooo00<000000?ooo`3oool0=P3oool005/0oooo0P00000m0?ooo`030000003oool0
oooo00/0oooo00<000000?ooo`3oool0=@3oool005P0oooo0`00000o0?ooo`030000003oool0oooo
00/0oooo00<000000?ooo`3oool0=@3oool005H0oooo0P0000120?ooo`030000003oool0oooo00`0
oooo00<000000?ooo`3oool0=03oool005<0oooo0`0000140?ooo`030000003oool0oooo00d0oooo
00<000000?ooo`3oool0<`3oool00540oooo0P0000170?ooo`030000003oool0oooo00d0oooo00<0
00000?ooo`3oool0<`3oool004h0oooo0`0000190?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool0<P3oool004`0oooo0P00001<0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0<P3oool004X0oooo0P00001>0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3o
ool0<@3oool004L0oooo0`00001@0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
<03oool004D0oooo0P00001C0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0<03o
ool00480oooo0`00001E0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0;`3oool0
0400oooo0P00001H0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0;P3oool003d0
oooo0`00001J0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0;P3oool003/0oooo
0P00001M0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0;@3oool003T0oooo0P00
001P0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0;@3oool003H0oooo0`00001R
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool0;03oool003@0oooo0P00001U0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool0:`3oool00340oooo0`00001W0?ooo`03
0000003oool0oooo01@0oooo00<000000?ooo`3oool0:`3oool002l0oooo0P00001Z0?ooo`030000
003oool0oooo01D0oooo00<000000?ooo`3oool0:P3oool002`0oooo0`00001/0?ooo`030000003o
ool0oooo01H0oooo00<000000?ooo`3oool0:@3oool002X0oooo0P00001_0?ooo`030000003oool0
oooo01H0oooo00<000000?ooo`3oool0:@3oool002P0oooo0P00001a0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool0:03oool002D0oooo0`0000140?ooo`030000003oool0oooo02`0
oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0X0?ooo`008`3oool2000004H0oooo
00<000000:UiW`000000;@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo02L0oooo
000P0?ooo`<00000B03oool010000000ZGVO0:UiW`00000/0?ooo`030000003oool0oooo01T0oooo
00<000000?ooo`3oool09P3oool001h0oooo0P00001:0?ooo`050000002YNIl0ZGVO0:UiW`000000
;03oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo02H0oooo000K0?ooo`<00000C03o
ool00`000000ZGVO0:UiW`020:UiW`030000003oool0oooo02T0oooo00<000000?ooo`3oool06P3o
ool00`000000oooo0?ooo`0U0?ooo`006@3oool2000004h0oooo00<000000:UiW`2YNIl0102YNIl0
0`000000oooo0?ooo`0X0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool0903oool0
01L0oooo0P00001?0?ooo`030000002YNIl0ZGVO00H0ZGVO00<000000?ooo`3oool09`3oool00`00
0000oooo0?ooo`0K0?ooo`030000003oool0oooo02@0oooo000D0?ooo`<00000D@3oool00`000000
ZGVO0:UiW`070:UiW`030000003oool0oooo02H0oooo00<000000?ooo`3oool0703oool00`000000
oooo0?ooo`0S0?ooo`004P3oool2000005<0oooo00<000000:UiW`2YNIl02@2YNIl00`000000oooo
0?ooo`0V0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool08`3oool00100oooo0P00
001E0?ooo`030000002YNIl0ZGVO00T0ZGVO00<000000?ooo`3oool09P3oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo0280oooo000@0?ooo`800000E03oool00`000000ZGVO0:UiW`0;
0:UiW`030000003oool0oooo02D0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0Q
0?ooo`00403oool00`000000oooo0000001C0?ooo`030000002YNIl0ZGVO00`0ZGVO00<000000?oo
o`3oool0903oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0240oooo000@0?ooo`04
0000003oool0oooo00000540oooo00<000000:UiW`2YNIl03P2YNIl00`000000oooo0?ooo`0S0?oo
o`030000003oool0oooo01h0oooo00<000000?ooo`3oool0803oool000l0oooo00<000000?ooo`3o
ool00P3oool00`000000oooo0?ooo`1=0?ooo`030000002YNIl0ZGVO0100ZGVO00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo000?0?ooo`030000
003oool0oooo00<0oooo00<000000?ooo`3oool0C03oool200000140ZGVO00<000000?ooo`3oool0
8P3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo01l0oooo000?0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0BP3oool500000100ZGVO00<000000?ooo`3oool0803o
ool200000280oooo00<000000?ooo`3oool07P3oool000l0oooo00<000000?ooo`3oool01@3oool0
0`000000oooo0?ooo`190?ooo`<000000P3T^:T2000000l0ZGVO00<000000?ooo`3oool07@3oool4
00000280oooo00<000000?ooo`3oool07P3oool000l0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`170?ooo`80000000<0>3^X0000003T^:T00`3T^:T2000000h0ZGVO00<000000?oo
o`3oool06P3oool3000000030??3W0000000000002<0oooo00<000000?ooo`3oool07@3oool000l0
oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`150?ooo`8000000`0h>jP00`000000
i;RY0>BhZ@030>BhZ@8000003@2YNIl00`000000oooo0?ooo`0H0?ooo`<000000P3c`i`2000002@0
oooo00<000000?ooo`3oool0703oool000h0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`140?ooo`8000000`0h>jP00`000000i;RY0>BhZ@050>BhZ@030000002YNIl0ZGVO00/0ZGVO
00<000000?ooo`3oool05@3oool4000000<0ll>L0P00000T0?ooo`030000003oool0oooo01`0oooo
000>0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0@P3oool2000000@0>3^X00<0
00000>BhZ@3T^:T01P3T^:T2000000/0ZGVO00<000000?ooo`3oool04`3oool5000000@0ll>L0P00
000U0?ooo`030000003oool0oooo01/0oooo000>0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0@@3oool2000000@0>3^X00<000000>BhZ@3T^:T0203T^:T2000000X0ZGVO00<00000
0?ooo`3oool0403oool5000000H0ll>L0P00000V0?ooo`030000003oool0oooo01X0oooo000>0?oo
o`030000003oool0oooo00`0oooo00<000000?ooo`3oool0?`3oool2000000D0>3^X00<000000>Bh
Z@3T^:T02P3T^:T2000000T0ZGVO00<000000?ooo`3oool03P3oool5000000L0ll>L0P00000V0?oo
o`030000003oool0oooo01X0oooo000>0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3o
ool0?@3oool2000000H0>3^X00<000000>BhZ@3T^:T0303T^:T00`000000ZGVO0:UiW`070:UiW`03
0000003oool0oooo00/0oooo1P0000080??3W08000009`3oool00`000000oooo0?ooo`0I0?ooo`00
3P3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo03`0oooo0P00000603PkZ0030000
003T^:T0i;RY00d0i;RY0P0000080:UiW`030000003oool0oooo00P0oooo1`0000090??3W0030000
003oool0000002H0oooo00<000000?ooo`3oool06@3oool000h0oooo00<000000?ooo`3oool03`3o
ool00`000000oooo0?ooo`0j0?ooo`8000001`0h>jP00`000000i;RY0>BhZ@0?0>BhZ@8000001P2Y
NIl00`000000oooo0?ooo`070?ooo`L000002P3c`i`00`000000oooo0000000W0?ooo`030000003o
ool0oooo01P0oooo000=0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool09@3oool2
00000180oooo00<0000003PkZ00h>jP01`0h>jP00`000000i;RY0>BhZ@0@0>BhZ@8000001@2YNIl0
0`000000oooo0?ooo`040?ooo`P000002`3c`i`00`000000oooo0000000X0?ooo`030000003oool0
oooo01L0oooo000=0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0903oool60000
00d0oooo0P00000903PkZ0030000003T^:T0i;RY0180i;RY0P0000040:UiW`040000003oool0oooo
0?ooo`T000003@3c`i`2000002P0oooo00<000000?ooo`3oool05`3oool000d0oooo00<000000?oo
o`3oool04`3oool00`000000oooo0?ooo`0S0?ooo`030000000008D0000000<0Va<00`0000090?oo
o`8000002P0h>jP00`000000i;RY0>BhZ@0D0>BhZ@030000002YNIl0ZGVO0080ZGVO00<000000?oo
o`0000002000000>0??3W0800000:@3oool00`000000oooo0?ooo`0F0?ooo`003@3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo02<0oooo00<000000000Q@0000001@2K4`04000000D0
oooo00<0000003PkZ00h>jP02@0h>jP00`000000i;RY0>BhZ@0E0>BhZ@8000000`2YNIl9000000l0
ll>L0P00000Z0?ooo`030000003oool0oooo01D0oooo000=0?ooo`030000003oool0oooo01D0oooo
00<000000?ooo`3oool08P3oool01000000000250000Q@00000809/C00H000002`0h>jP00`000000
i;RY0>BhZ@0G0>BhZ@8000000P2YNIl600000140ll>L0P00000Z0?ooo`030000003oool0oooo01D0
oooo000=0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool08@3oool01@0000000025
0000Q@0008D0000000/0Va<000<0000003PkZ00h>jP02P0h>jP00`000000i;RY0>BhZ@0I0>BhZ@L0
00004P3c`i`2000002/0oooo00<000000?ooo`3oool0503oool000`0oooo00<000000?ooo`3oool0
603oool00`000000oooo0?ooo`0P0?ooo`030000000008D000250080002500<0000009/C002K4`00
1`2K4`000`000000>3^X03PkZ00;03PkZ0030000003T^:T0i;RY01/0i;RY1000000C0??3W0800000
:`3oool00`000000oooo0?ooo`0D0?ooo`00303oool00`000000oooo0?ooo`0I0?ooo`030000003o
ool0oooo01l0oooo00<000000000Q@0008D00P0008D00`000000Va<009/C000709/C00030000000h
>jP0>3^X00`0>3^X00<000000>BhZ@3T^:T0703T^:T400000140ll>L0P00000/0?ooo`030000003o
ool0oooo01<0oooo000<0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool07P3oool0
0`00000000250000Q@030000Q@030000002K4`00Va<000D0Va<000<0000003PkZ00h>jP03@0h>jP0
0`000000i;RY0>BhZ@0K0>BhZ@050000001402P000000000002dR:L01P00000;0??3W0800000;@3o
ool00`000000oooo0?ooo`0B0?ooo`00303oool00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo01h0oooo00<000000000Q@0008D00`0008D00`000000Va<009/C000309/C008000003`0h>jP0
0`000000i;RY0>BhZ@0J0>BhZ@030000001402P0A00X0080A00X0P0000050;B8Y`H000001@3c`i`2
000002d0oooo00<000000?ooo`3oool04P3oool000`0oooo00<000000?ooo`3oool0703oool00`00
0000oooo0?ooo`0M0?ooo`030000000008D0002500@0002500<0000009/C002K4`000P2K4`000`00
0000>3^X03PkZ00>03PkZ0030000003T^:T0i;RY01X0i;RY00<0000004@0:01402P0101402P00`00
0000]8RW0;B8Y`080;B8Y`L00000;P3oool00`000000oooo0?ooo`0A0?ooo`00303oool00`000000
oooo0?ooo`0M0?ooo`030000003oool0oooo01`0oooo00<000000000Q@0008D01@0008D010000000
Va<009/C0000000A03PkZ0030000003T^:T0i;RY01T0i;RY00<0000004@0:01402P01P1402P20000
00h0]8RW1P00000Z0?ooo`030000003oool0oooo0100oooo000<0?ooo`030000003oool0oooo01h0
oooo00<000000?ooo`3oool06`3oool00`00000000250000Q@060000Q@030000002K4`0000000140
>3^X00<000000>BhZ@3T^:T0603T^:T00`000000A00X04@0:00904@0:0800000302dR:L00`000000
]8RW0;B8Y`030;B8Y`H00000903oool00`000000oooo0?ooo`0@0?ooo`002`3oool00`000000oooo
0?ooo`0P0?ooo`030000003oool0oooo01X0oooo00<000000000Q@0008D01P0008D200000180>3^X
00<000000>BhZ@3T^:T05`3T^:T00`000000A00X04@0:00<04@0:08000002`2dR:L00`000000]8RW
0;B8Y`080;B8Y`H000007`3oool00`000000oooo0?ooo`0?0?ooo`002`3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo01X0oooo00<000000000Q@0008D00`0008D3000000030;A;@P00
0000>3^X0100>3^X00<000000>BhZ@3T^:T05P3T^:T00`000000A00X04@0:00?04@0:0030000002d
R:L0]8RW00P0]8RW00<000000;B8Y`2dR:L03P2dR:L6000001T0oooo00<000000?ooo`3oool03`3o
ool000/0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0I0?ooo`040000000008D0
00250000Q@<00000102dBd800`000000>3^X03PkZ00?03PkZ0030000003T^:T0i;RY01@0i;RY00<0
000004@0:01402P04@1402P2000000P0]8RW00<000000;B8Y`2dR:L0502dR:L4000001H0oooo00<0
00000?ooo`3oool03P3oool000/0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0H
0?ooo`D000001`2dBd800`000000>3^X03PkZ00>03PkZ0030000003T^:T0i;RY01@0i;RY00<00000
04@0:01402P04`1402P2000000H0]8RW00<000000;B8Y`2dR:L05P2dR:L00`000000oooo0?ooo`0F
0?ooo`030000003oool0oooo00d0oooo000;0?ooo`030000003oool0oooo02@0oooo00<000000?oo
o`3oool05P3oool4000000T0]4]200<0000003PkZ00h>jP03P0h>jP00`000000i;RY0>BhZ@0C0>Bh
Z@030000001402P0A00X01H0A00X00<000000;B8Y`2dR:L00`2dR:L00`000000]8RW0;B8Y`0F0;B8
Y`030000003oool0oooo01H0oooo00<000000?ooo`3oool03@3oool000/0oooo00<000000?ooo`3o
ool09@3oool00`000000oooo0?ooo`0C0?ooo`@00000302dBd800`000000>3^X03PkZ00=03PkZ003
0000003T^:T0i;RY0180i;RY00<0000004@0:01402P05@1402P3000000D0]8RW00<000000;B8Y`2d
R:L05@2dR:L00`000000oooo0?ooo`0G0?ooo`8000003P3oool000X0oooo00<000000?ooo`3oool0
9`3oool00`000000oooo0?ooo`0?0?ooo`D000003`2dBd800`000000>3^X03PkZ00<03PkZ0030000
003T^:T0i;RY0140i;RY00<0000004@0:01402P0501402P200000080/[gR00<000000;B8Y`2dR:L0
0`2dR:L00`000000]8RW0;B8Y`0D0;B8Y`030000003oool0oooo01D0oooo0`0000000`3oool00000
0?ooo`0=0?ooo`002P3oool00`000000oooo0?ooo`0X0?ooo`030000003oool0oooo00`0oooo0P00
000209mhY`030000002dBd80]4]20100]4]200<0000003PkZ00h>jP02`0h>jP00`000000i;RY0>Bh
Z@0@0>BhZ@030000001402P0A00X01<0A00X0P0000040;:mhP030000002dR:L0]8RW00<0]8RW00<0
00000;B8Y`2dR:L04`2dR:L00`000000oooo0?ooo`0C0?ooo`<00000103oool00`000000oooo0?oo
o`0<0?ooo`002P3oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00T0oooo0P000002
09mhY`800000502dBd800`000000>3^X03PkZ00:03PkZ0030000003T^:T0i;RY0100i;RY00<00000
04@0:01402P04@1402P2000000H0/[gR00<000000;B8Y`2dR:L00`2dR:L00`000000]8RW0;B8Y`0C
0;B8Y`030000003oool0oooo00l0oooo100000070?ooo`030000003oool0oooo00`0oooo000:0?oo
o`030000003oool0oooo02X0oooo00<000000?ooo`3oool01@3oool300000080WgRW0P00000G0;A;
@P030000000h>jP0>3^X00X0>3^X00<000000>BhZ@3T^:T03P3T^:T00`000000A00X04@0:00@04@0
:08000002@2b_N800`000000]8RW0;B8Y`020;B8Y`030000002dR:L0]8RW0180]8RW00<000000?oo
o`3oool03@3oool3000000`0oooo00<000000?ooo`3oool02`3oool000X0oooo00<000000?ooo`3o
ool0:`3oool00`000000oooo0?ooo`020?ooo`8000000`2ON:L2000001X0]4]200<0000003PkZ00h
>jP02@0h>jP00`000000i;RY0>BhZ@0=0>BhZ@030000001402P0A00X00l0A00X0P00000;0;:mhP03
0000002dR:L0]8RW0080]8RW00<000000;B8Y`2dR:L04@2dR:L00`000000oooo0?ooo`0;0?ooo`<0
00003`3oool00`000000oooo0?ooo`0;0?ooo`002P3oool00`000000oooo0?ooo`0/0?ooo`040000
003oool00000000000<0WgRW0P00000L0;A;@P030000000h>jP0>3^X00T0>3^X00<000000>BhZ@3T
^:T0303T^:T00`000000A00X04@0:00>04@0:08000003@2b_N800`000000]8RW0;B8Y`020;B8Y`03
0000002dR:L0]8RW0140]8RW00<000000?ooo`3oool0203oool300000180oooo00<000000?ooo`3o
ool02`3oool000X0oooo00<000000?ooo`3oool0:`3oool3000000<0WgRW0P00000O0;A;@P030000
000h>jP0>3^X00P0>3^X00<000000>BhZ@3T^:T02`3T^:T00`000000A00X04@0:00=04@0:0800000
3`2b_N800`000000]8RW0;B8Y`020;B8Y`030000002dR:L0]8RW0100]8RW00<000000?ooo`3oool0
1P3oool3000001D0oooo00<000000?ooo`3oool02`3oool000T0oooo00<000000?ooo`3oool0:P3o
ool2000000@0WgRW0P00000R0;A;@P030000000h>jP0>3^X00L0>3^X00<000000>BhZ@3T^:T02P3T
^:T00`000000A00X04@0:00<04@0:08000004@2b_N800`000000]8RW0;B8Y`020;B8Y`030000002d
R:L0]8RW00l0]8RW00<000000?ooo`3oool0103oool3000001P0oooo00<000000?ooo`3oool02`3o
ool000T0oooo00<000000?ooo`3oool0:03oool2000000@0WgRW0P00000U0;A;@P030000000h>jP0
>3^X00H0>3^X00<000000>BhZ@3T^:T02P3T^:T00`000000A00X04@0:00:04@0:08000004`2b_N80
0`000000]8RW0;B8Y`020;B8Y`030000002dR:L0]8RW00h0]8RW00<000000?ooo`3oool00P3oool3
000001/0oooo00<000000?ooo`3oool02`3oool000T0oooo00<000000?ooo`3oool09@3oool30000
00D0WgRW0`00000V0;A;@P030000000h>jP0>3^X00D0>3^X00<000000>BhZ@3T^:T02@3T^:T00`00
0000A00X04@0:00:04@0:0030000002b_N80/[gR01<0/[gR00<000000;B8Y`2dR:L00P2dR:L00`00
0000]8RW0;B8Y`0>0;B8Y`D000007P3oool00`000000oooo0?ooo`0;0?ooo`002@3oool00`000000
oooo0?ooo`0S0?ooo`800000202ON:L00`000000X;;U0:2bi@04000002<0]4]200<0000003PkZ00h
>jP01@0h>jP00`000000i;RY0>BhZ@070>BhZ@030000001402P0A00X00T0A00X0P00000F0;:mhP03
0000002dR:L0]8RW00<0]8RW00<000000;B8Y`2dR:L0302dR:L2000002<0oooo00<000000?ooo`3o
ool02P3oool000T0oooo00<000000?ooo`3oool08@3oool2000000/0WgRW00<000000:2bi@2P/^D0
0`2P/^D4000001l0]4]200<0000003PkZ00h>jP01@0h>jP00`000000i;RY0>BhZ@060>BhZ@030000
001402P0A00X00P0A00X0P00000H0;:mhP030000002dR:L0]8RW00<0]8RW00<000000;B8Y`2dR:L0
2`2dR:L00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00X0oooo00090?ooo`030000003o
ool0oooo01l0oooo0P00000=09mhY`030000002P/^D0X;;U00L0X;;U1000000L0;A;@P030000000h
>jP0>3^X00@0>3^X00<000000>BhZ@3T^:T01@3T^:T00`000000A00X04@0:00704@0:08000006`2b
_N800`000000]8RW0;B8Y`020;B8Y`030000002dR:L0]8RW00/0]8RW00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0:0?ooo`00203oool00`000000oooo0?ooo`0Q0?ooo`030000002ON:L0
WgRW00`0WgRW00<000000:2bi@2P/^D02P2P/^D5000001P0]4]200<0000003PkZ00h>jP00`0h>jP0
0`000000i;RY0>BhZ@050>BhZ@030000001402P0A00X00D0A00X0P00000M0;:mhP030000002dR:L0
]8RW0080]8RW00<000000;B8Y`2dR:L02P2dR:L00`000000oooo0?ooo`0T0?ooo`030000003oool0
oooo00X0oooo00080?ooo`030000003oool0oooo0280oooo00<0000009mhY`2ON:L02`2ON:L00`00
0000X;;U0:2bi@0?0:2bi@@000005@2dBd800`000000>3^X03PkZ00203PkZ0030000003T^:T0i;RY
00@0i;RY00<0000004@0:01402P0101402P2000001l0/[gR00<000000;B8Y`2dR:L00P2dR:L00`00
0000]8RW0;B8Y`090;B8Y`030000003oool0oooo02D0oooo00<000000?ooo`3oool02P3oool000P0
oooo00<000000?ooo`3oool08`3oool00`000000WgRW09mhY`0;09mhY`030000002P/^D0X;;U0180
X;;U1000000B0;A;@P050000000h>jP0>3^X03PkZ00000001@3T^:T00`000000A00X04@0:00304@0
:08000008@2b_N800`000000]8RW0;B8Y`020;B8Y`030000002dR:L0]8RW00P0]8RW00<000000?oo
o`3oool09P3oool00`000000oooo0?ooo`0:0?ooo`00203oool00`000000oooo0?ooo`0T0?ooo`03
0000002ON:L0WgRW00X0WgRW00<000000:2bi@2P/^D05P2P/^D4000000l0]4]200@0000003PkZ00h
>jP00000103T^:T00`000000A00X04@0:00204@0:08000008`2b_N800`000000]8RW0;B8Y`020;B8
Y`030000002dR:L0]8RW00P0]8RW00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0:0?oo
o`00203oool00`000000oooo0?ooo`0U0?ooo`030000002ON:L0WgRW00X0WgRW00<000000:2bi@2P
/^D06@2P/^D5000000/0]4]200<0000003PkZ00000000`3T^:T010000000A00X04@0:01402P20000
02D0/[gR00@000000;B8Y`2dR:L0]8RW0P0000090;B8Y`030000003oool0oooo02P0oooo00<00000
0?ooo`3oool02@3oool000P0oooo00<000000?ooo`3oool09P3oool00`000000WgRW09mhY`0909mh
Y`030000002P/^D0X;;U01h0X;;U100000070;A;@P060000000h>jP0>3^X0000003T^:T000000P14
02P2000002L0/[gR100000020;B8Y`030000002dR:L0]8RW00D0]8RW00<000000?ooo`3oool0:@3o
ool00`000000oooo0?ooo`090?ooo`00203oool00`000000oooo0?ooo`0W0?ooo`030000002ON:L0
WgRW00T0WgRW00<000000:2bi@2P/^D08@2P/^D4000000@0]4]200@0000003PkZ0000000i;RY0`00
000W0;:mhP<000001P2dR:L00`000000]8RW0;B8Y`030;B8Y`030000003oool0oooo02X0oooo00<0
00000?ooo`3oool02@3oool000L0oooo00<000000?ooo`3oool0:03oool00`000000WgRW09mhY`09
09mhY`030000002P/^D0X;;U02D0X;;U100000000`2dBd80000000000002000002H0/[gR0`000002
0;:mhP030000002dR:L0]8RW00@0]8RW00<000000;B8Y`2dR:L00`2dR:L00`000000oooo0?ooo`0Z
0?ooo`030000003oool0oooo00T0oooo00070?ooo`030000003oool0oooo02T0oooo00<0000009mh
Y`2ON:L0202ON:L00`000000X;;U0:2bi@0Y0:2bi@@000008`2b_N84000000D0/[gR00<000000;B8
Y`2dR:L01@2dR:L01@000000]8RW0;B8Y`2dR:L0000002d0oooo00<000000?ooo`3oool02@3oool0
00L0oooo00<000000?ooo`3oool0:P3oool00`000000WgRW09mhY`0809mhY`030000002P/^D0X;;U
02P0X;;U2000000L0;:mhP<000002P2b_N800`000000]8RW0;B8Y`050;B8Y`030000002dR:L00000
02h0oooo00<000000?ooo`3oool02@3oool000L0oooo00<000000?ooo`3oool0:`3oool00`000000
WgRW09mhY`0709mhY`030000002P/^D0X;;U02L0X;;U00H0000002l0>0000000000004]I_`000003
0<UXE`<000005P2b_N83000000d0/[gR00<000000;B8Y`2dR:L01P2dR:L2000002h0oooo00<00000
0?ooo`3oool02@3oool000L0oooo00<000000?ooo`3oool0;03oool00`000000WgRW09mhY`0709mh
Y`030000002P/^D0X;;U02@0X;;U0P000000100_03P000000>31^P00000204]I_`0300000039J5L0
bFQG00<0bFQG0`00000@0;:mhP<00000402b_N800`000000]8RW0;B8Y`060;B8Y`<00000;P3oool0
0`000000oooo0?ooo`080?ooo`001`3oool00`000000oooo0?ooo`0]0?ooo`030000002ON:L0WgRW
00H0WgRW00<000000:2bi@2P/^D08`2P/^D010000000;`0h02l0>00000030>31^P030000001;FKl0
000000P0bFQG100000090;:mhP<000004`2b_N800`000000]8RW0;B8Y`050;B8Y`050000003=FBP0
00000<eI:0000000;@3oool00`000000oooo0?ooo`080?ooo`001P3oool00`000000oooo0?ooo`0_
0?ooo`030000002ON:L0WgRW00H0WgRW00<000000:2bi@2P/^D0802P/^D2000000<0;`0h00D00000
0>31^P3P`KX0h<6j0000000204]I_`0300000039J5L0bFQG00T0bFQG0`0000030;:mhP<000005P2b
_N800`000000]8RW0;B8Y`040;B8Y`070000003=FBP0cETX0<eI:0000000cETX0000000/0?ooo`03
0000003oool0oooo00P0oooo00060?ooo`030000003oool0oooo0300oooo00<0000009mhY`2ON:L0
1@2ON:L00`000000X;;U0:2bi@0O0:2bi@030000000_03P0;`0h0080;`0h00<000000>31^P3P`KX0
0P3P`KX01@000000BeVo04]I_`1;FKl0000000d0bFQG1000000H0;:mhP030000002dR:L0]8RW00@0
]8RW00<000000<eI:03=FBP00P3=FBP00`000000cETX0000000[0?ooo`030000003oool0oooo00P0
oooo00060?ooo`030000003oool0oooo0340oooo00<0000009mhY`2ON:L01@2ON:L00`000000X;;U
0:2bi@0L0:2bi@800000100_03P00`000000h<6j0>31^P030>31^P030000001;FKl0BeVo0080BeVo
00<000000<UXE`39J5L01`39J5L3000000@0bFQG0`00000E0;:mhP030000002dR:L0]8RW00<0]8RW
00<000000<eI:03=FBP0103=FBP00`000000cETX0000000Z0?ooo`030000003oool0oooo00P0oooo
00060?ooo`030000003oool0oooo0380oooo00<0000009mhY`2ON:L0102ON:L00`000000X;;U0:2b
i@0K0:2bi@030000000_03P0;`0h00<0;`0h00<000000>31^P3P`KX0103P`KX00`000000BeVo04]I
_`0304]I_`0300000039J5L0bFQG00<0bFQG0`00000:0<UXE`@000004@2b_N800`000000]8RW0;B8
Y`020;B8Y`030000003=FBP0cETX00D0cETX00@000000<eI:03=FBP00000:@3oool00`000000oooo
0?ooo`080?ooo`001P3oool00`000000oooo0?ooo`0c0?ooo`030000002ON:L0WgRW00@0WgRW00<0
00000:2bi@2P/^D0602P/^D2000000H0;`0h00<000000>31^P3P`KX0103P`KX00`000000BeVo04]I
_`0404]I_`D000004@39J5L3000000h0/[gR00<000000;B8Y`2dR:L00P2dR:L00`000000cETX0<eI
:0060<eI:0040000003=FBP0cETX000002P0oooo00<000000?ooo`3oool0203oool000H0oooo00<0
00000?ooo`3oool0=03oool00`000000WgRW09mhY`0309mhY`030000002P/^D0X;;U01L0X;;U00<0
000002l0>00_03P01@0_03P00`000000h<6j0>31^P060>31^P040000001;FKl0BeVo04]I_`@00000
5`39J5L3000000`0/[gR00@000000;B8Y`2dR:L000002P3=FBP010000000cETX0<eI:000000X0?oo
o`030000003oool0oooo00L0oooo00060?ooo`030000003oool0oooo03D0oooo00<0000009mhY`2O
N:L00`2ON:L00`000000X;;U0:2bi@0D0:2bi@8000001`0_03P00`000000h<6j0>31^P070>31^P@0
00000`1;FKl00`000000bFQG0<UXE`0H0<UXE`@00000202b_N800`000000]8RW0000000<0<eI:004
0000003=FBP0cETX000002L0oooo00<000000?ooo`3oool01`3oool000D0oooo00<000000?ooo`3o
ool0=`3oool00`000000WgRW09mhY`0209mhY`030000002P/^D0X;;U0180X;;U0P00000902l0>003
0000003P`KX0h<6j00D0h<6j0`00000704]I_`0300000039J5L0bFQG01/0bFQG0`0000050;:mhP80
00003P3=FBP010000000cETX0<eI:000000V0?ooo`030000003oool0oooo00L0oooo00050?ooo`03
0000003oool0oooo03P0oooo00D0000009mhY`2ON:L0WgRW0000000C0:2bi@030000000_03P0;`0h
00P0;`0h00<000000>31^P3P`KX00`3P`KX300000080h<6j00<0000004]I_`1;FKl01P1;FKl00`00
0000bFQG0<UXE`0M0<UXE`@0000000<0/[gR0000000000003P3=FBP01@000000cETX0<eI:03=FBP0
000002D0oooo00<000000?ooo`3oool01`3oool000D0oooo00<000000?ooo`3oool0>@3oool01@00
0000WgRW09mhY`2ON:L000000100X;;U0P00000:02l0>0040000003P`KX0h<6j0>31^P<000001@3P
`KX00`000000BeVo04]I_`0704]I_`0300000039J5L0bFQG0200bFQG0`00000?0<eI:0050000003=
FBP0cETX0<eI:0000000903oool00`000000oooo0?ooo`070?ooo`001@3oool00`000000oooo0?oo
o`0i0?ooo`050000002ON:L0WgRW09mhY`0000003`2P/^D00`000000;`0h02l0>00:02l0>0@00000
203P`KX00`000000BeVo04]I_`0804]I_`0300000039J5L0bFQG01l0bFQG0P000000102cd>l00000
00000000000=0<eI:0050000003=FBP0cETX0<eI:00000008`3oool00`000000oooo0?ooo`070?oo
o`001@3oool00`000000oooo0?ooo`0j0?ooo`050000002ON:L0WgRW09mhY`000000302P/^D20000
00/0;`0h0`00000<0>31^P030000001;FKl0BeVo00P0BeVo00<000000<UXE`39J5L07@39J5L00`00
0000/Y2c000000040;?@k`<000002`3=FBP01@000000cETX0<eI:03=FBP000000280oooo00<00000
0?ooo`3oool01`3oool000D0oooo00<000000?ooo`3oool0>`3oool010000000WgRW09mhY`00000;
0:2bi@030000000_03P0;`0h00L0;`0h1@00000>0>31^P030000001;FKl0BeVo00T0BeVo00<00000
0<UXE`39J5L07039J5L00`000000/Y2c000000070;?@k`8000002P3=FBP01@000000cETX0<eI:03=
FBP000000280oooo00<000000?ooo`3oool01P3oool000@0oooo00<000000?ooo`3oool0?@3oool0
10000000WgRW09mhY`0000080:2bi@8000001`0_03P3000000@0;`0h00<000000>31^P3P`KX0303P
`KX00`000000BeVo04]I_`0904]I_`0300000039J5L0bFQG01/0bFQG00D000000;:@/`2bT;<0/Y2c
000000080;?@k`<000001`3=FBP00`000000cETX0<eI:0020<eI:0800000803oool00`000000oooo
0?ooo`060?ooo`00103oool00`000000oooo0?ooo`0n0?ooo`030000002ON:L0000000L0X;;U00<0
000002l0>00_03P0100_03P3000000H0;`0h00<000000>31^P3P`KX03@3P`KX00`000000BeVo04]I
_`0:04]I_`0300000039J5L0bFQG01T0bFQG00<000000;:@/`2bT;<00P2bT;<00`000000/m3_0;?@
k`090;?@k`<000001@3=FBP00`000000cETX0<eI:0030<eI:0030000003oool0oooo01d0oooo00<0
00000?ooo`3oool01P3oool000@0oooo00<000000?ooo`3oool0?P3oool20000000309mhY`000000
X;;U00<0X;;U0P00000402l0>0<00000200_03P00`000000h<6j0>31^P0>0>31^P030000001;FKl0
BeVo00/0BeVo00<000000<UXE`39J5L05`39J5L00`000000/Y2c0;:@/`030;:@/`030000002cd>l0
/m3_00`0/m3_0`0000030<eI:0030000003=FBP0cETX00<0cETX00<000000?ooo`3oool0703oool0
0`000000oooo0?ooo`060?ooo`00103oool00`000000oooo0?ooo`0m0?ooo`D000000`2P/^D01000
0000;`0h02l0>00_03P3000000X0;`0h00<000000>31^P3P`KX03`3P`KX00`000000BeVo04]I_`0<
04]I_`0300000039J5L0bFQG01H0bFQG00<000000;:@/`2bT;<00`2bT;<00`000000/m3_0;?@k`0?
0;?@k`<0000000<0cETX0000003=FBP0103=FBP00`000000oooo0?ooo`0K0?ooo`030000003oool0
oooo00H0oooo00040?ooo`030000003oool0oooo03`0oooo2@000000100_03P0000000000000000=
02l0>0030000003P`KX0h<6j0100h<6j00<0000004]I_`1;FKl0301;FKl00`000000bFQG0<UXE`0D
0<UXE`030000002bT;<0/Y2c00@0/Y2c00<000000;?@k`2cd>l04P2cd>l3000000D0cETX00<00000
0?ooo`3oool06P3oool00`000000oooo0?ooo`060?ooo`00103oool00`000000oooo0?ooo`0k0?oo
o`/000003`0_03P00`000000h<6j0>31^P0A0>31^P030000001;FKl0BeVo00d0BeVo00<000000<UX
E`39J5L04P39J5L00`000000/Y2c0;:@/`050;:@/`030000002cd>l0/m3_01@0/m3_0`0000040<eI
:0030000003oool0oooo01T0oooo00<000000?ooo`3oool01P3oool000@0oooo00<000000?ooo`3o
ool0>`3oool800000140;`0h00<000000>31^P3P`KX04P3P`KX00`000000BeVo04]I_`0>04]I_`03
00000039J5L0bFQG0100bFQG00<000000;:@/`2bT;<01`2bT;<00`000000/m3_0;?@k`0D0;?@k`03
0000002cd>l00000008000000P3=FBP00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00D0
oooo00030?ooo`030000003oool0oooo03/0oooo1`0000000`2?^_@000000000000@02l0>0030000
003P`KX0h<6j0180h<6j00<0000004]I_`1;FKl03`1;FKl00`000000bFQG0<UXE`0>0<UXE`030000
002bT;<0/Y2c00P0/Y2c00<000000;?@k`2cd>l05@2cd>l010000000/m3_0;?@k`2cd>l4000001X0
oooo00<000000?ooo`3oool01@3oool000<0oooo00<000000?ooo`3oool0>P3oool700000080Sk[d
0P00000?02l0>0030000003P`KX0h<6j01<0h<6j00<0000004]I_`1;FKl03`1;FKl00`000000bFQG
0<UXE`0>0<UXE`030000002bT;<0/Y2c00P0/Y2c00<000000;?@k`2cd>l05P2cd>l00`000000/m3_
0;?@k`030;?@k`8000006@3oool00`000000oooo0?ooo`050?ooo`000`3oool00`000000oooo0?oo
o`0g0?ooo`P00000102?^_@2000000h0;`0h00<000000>31^P3P`KX0503P`KX00`000000BeVo04]I
_`0@04]I_`0300000039J5L0bFQG00`0bFQG00<000000;:@/`2bT;<02@2bT;<00`000000/m3_0;?@
k`0F0;?@k`030000002cd>l0/m3_00<000006`3oool00`000000oooo0?ooo`050?ooo`000`3oool0
0`000000oooo0?ooo`0d0?ooo`<0000000<0oooo0000000000001000000508njm0030000002UQk<0
000000d0;`0h00<000000>31^P3P`KX05@3P`KX00`000000BeVo04]I_`0@04]I_`0300000039J5L0
bFQG00X0bFQG00<000000;:@/`2bT;<02P2bT;<00`000000/m3_0;?@k`0G0;?@k`8000007P3oool0
0`000000oooo0?ooo`050?ooo`000`3oool00`000000oooo0?ooo`0a0?ooo`<00000103oool50000
00H0Sk[d00<000000:F7/`000000300_03P00`000000h<6j0>31^P0F0>31^P030000001;FKl0BeVo
0140BeVo00<000000<UXE`39J5L02039J5L00`000000/Y2c0;:@/`0;0;:@/`030000002cd>l0/m3_
01@0/m3_0`0000000`3oool000000?ooo`0M0?ooo`030000003oool0oooo00D0oooo00030?ooo`03
0000003oool0oooo02h0oooo0`0000060?ooo`D000001`2?^_@00`000000YHNc0000000;02l0>003
0000003P`KX0h<6j01L0h<6j00<0000004]I_`1;FKl04P1;FKl00`000000bFQG0<UXE`070<UXE`03
0000002bT;<0/Y2c00`0/Y2c00<000000;?@k`2cd>l0402cd>l3000000D0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`050?ooo`000P3oool00`000000oooo0?ooo`0/0?ooo`<00000
203oool5000000P0Sk[d00@000000:F7/`2UQk<000002P0_03P00`000000h<6j0>31^P0G0>31^P03
0000001;FKl0BeVo01<0BeVo00<000000<UXE`39J5L01@39J5L00`000000/Y2c0;:@/`0=0;:@/`03
0000002cd>l0/m3_00d0/m3_0`0000090?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3o
ool01@3oool00080oooo00<000000?ooo`3oool0:@3oool3000000X0oooo1000000:08njm0040000
002UQk<0YHNc000000T0;`0h00<000000>31^P3P`KX0603P`KX00`000000BeVo04]I_`0D04]I_`03
00000039J5L0bFQG00<0bFQG00<000000;:@/`2bT;<03P2bT;<00`000000/m3_0;?@k`0;0;?@k`80
0000303oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo00@0oooo00020?ooo`030000
003oool0oooo02H0oooo0`00000=0?ooo`<000002`2?^_@010000000YHNc0:F7/`00000802l0>003
0000003P`KX0h<6j01T0h<6j00<0000004]I_`1;FKl0501;FKl00`000000bFQG0<UXE`020<UXE`03
0000002bT;<0/Y2c00l0/Y2c00<000000;?@k`2cd>l0202cd>l3000000l0oooo00<000000?ooo`3o
ool06P3oool00`000000oooo0?ooo`040?ooo`000P3oool00`000000oooo0?ooo`0R0?ooo`@00000
3`3oool3000000`0Sk[d00@000000:F7/`2UQk<00000200_03P00`000000h<6j0>31^P0J0>31^P03
0000001;FKl0BeVo01@0BeVo00D000000<UXE`39J5L0bFQG0000000A0;:@/`030000002cd>l0/m3_
00D0/m3_0`00000C0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0103oool00080
oooo00<000000?ooo`3oool07`3oool300000180oooo0`00000=08njm0050000002UQk<0YHNc0:F7
/`0000001P0_03P00`000000h<6j0>31^P0K0>31^P030000001;FKl0BeVo01D0BeVo00<000000<UX
E`0000004P2bT;<00`000000/m3_0;?@k`020;?@k`<000005`3oool00`000000oooo0?ooo`0H0?oo
o`030000003oool0oooo00@0oooo00020?ooo`030000003oool0oooo01`0oooo0`00000D0?ooo`<0
00003`2?^_@010000000YHNc0:F7/`00000502l0>0030000003P`KX0h<6j01`0h<6j00<0000004]I
_`1;FKl05P1;FKl00`000000/Y2c0;:@/`0B0;:@/`040000002cd>l00000000001/0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`040?ooo`000P3oool00`000000oooo0?ooo`0I0?oo
o`<000005`3oool00`000000Sk[d08njm00?08njm0040000002UQk<0YHNc000000@0;`0h00<00000
0>31^P3P`KX07@3P`KX00`000000BeVo04]I_`0D04]I_`8000005@2bT;<2000001d0oooo00<00000
0?ooo`3oool05`3oool00`000000oooo0?ooo`040?ooo`0000<0oooo0000003oool0603oool30000
01T0oooo00<0000008njm02?^_@0402?^_@01@000000YHNc0:F7/`2UQk<0000000<0;`0h00<00000
0>31^P3P`KX07@3P`KX00`000000BeVo04]I_`0B04]I_`8000005`2bT;<00`000000oooo0?ooo`0M
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0103oool000030?ooo`000000oooo
01D0oooo0`00000K0?ooo`<000004@2?^_@01@000000YHNc0:F7/`2UQk<000000080;`0h00<00000
0>31^P3P`KX07P3P`KX00`000000BeVo04]I_`0@04]I_`8000006@2bT;<00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool00`3oool000030?ooo`000000oooo
0180oooo0`00000Q0?ooo`H000002`2?^_@01`000000YHNc0:F7/`2UQk<0000002l0>00000008P3P
`KX00`000000BeVo04]I_`0>04]I_`030000002bT;<0/Y2c01T0/Y2c00<000000?ooo`3oool07`3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00<0oooo00000`3oool000000?ooo`0?
0?ooo`<00000:P3oool6000000D0Sk[d00<000000:F7/`2UQk<00P2UQk<200000280h<6j00<00000
04]I_`1;FKl0301;FKl2000001`0/Y2c00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0D
0?ooo`030000003oool0oooo00<0oooo00000`3oool000000?ooo`0<0?ooo`<00000<`3oool60000
00@0YHNc0`00000Q0>31^P030000001;FKl0BeVo00X0BeVo0P00000O0;:@/`030000003oool0oooo
01l0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`030?ooo`0000<0oooo0000003o
ool0203oool4000003/0oooo00<000000:F7/`2UQk<01@2UQk<5000001`0h<6j00<0000004]I_`1;
FKl0201;FKl200000240/Y2c00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0C0?ooo`03
0000003oool0oooo00<0oooo00000`000000oooo0?ooo`050?ooo`<00000?`3oool00`000000YHNc
0:F7/`0:0:F7/`H000005P3P`KX00`000000BeVo04]I_`0604]I_`8000008`2bT;<00`000000oooo
0?ooo`0Q0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00`3oool000030000003o
ool0oooo0080oooo0`0000120?ooo`030000002UQk<0YHNc0100YHNc1@00000A0>31^P030000001;
FKl0BeVo00D0BeVo00<000000;:@/`2bT;<08`2bT;<00`000000oooo0?ooo`0R0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool00`3oool000030000003oool0000000800000A@3oool0
0`000000YHNc0:F7/`0E0:F7/`D000003@3P`KX00`000000BeVo04]I_`0204]I_`8000009P2bT;<0
0`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool00P3oool0
00800000B03oool00`000000YHNc0:F7/`0J0:F7/`H000001`3P`KX00`000000BeVo04]I_`020000
02P0/Y2c00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo
0080oooo00000`3oool00000000000170?ooo`030000002UQk<0YHNc0200YHNc1@0000020>31^P<0
0000:`2bT;<00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3o
ool00P3oool000<0oooo00<000000?ooo`3oool0A03oool00`000000YHNc0:F7/`0U0:F7/`@00000
;02bT;<00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
0P3oool000@0oooo00<000000?ooo`3oool0@`3oool00`000000YHNc0:F7/`0U0:F7/`<0000000<0
jmFn0000000000000P00000X0;:@/`030000003oool0oooo02D0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`020?ooo`001@3oool2000004<0oooo00<000000:F7/`2UQk<08`2UQk<2
0000008054:n00<000000>_E_P3[eKh00`3[eKh3000002D0/Y2c00<000000?ooo`3oool09@3oool0
0`000000oooo0?ooo`0>0?ooo`030000003oool0oooo0080oooo00070?ooo`030000003oool0oooo
0400oooo00<000000:F7/`2UQk<08@2UQk<2000000D054:n00<000000>_E_P3[eKh01@3[eKh30000
0280/Y2c00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0oooo
0080oooo00080?ooo`030000003oool0oooo03l0oooo00<000000:F7/`2UQk<07`2UQk<2000000L0
54:n00<000000>_E_P3[eKh0203[eKh4000001h0/Y2c00<000000?ooo`3oool09`3oool00`000000
oooo0?ooo`0=0?ooo`030000003oool0oooo0040oooo00090?ooo`800000?`3oool00`000000YHNc
0:F7/`0N0:F7/`030000000D@[h054:n00L054:n00<000000>_E_P3[eKh0303[eKh3000001`0/Y2c
00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0040oooo
000;0?ooo`030000003oool0oooo03`0oooo00<000000:F7/`2UQk<0702UQk<2000000X054:n00<0
00000>_E_P3[eKh03`3[eKh4000001P0/Y2c00<000000?ooo`3oool0:03oool00`000000oooo0?oo
o`0;0?ooo`030000003oool0oooo0040oooo000<0?ooo`030000003oool0oooo03/0oooo00<00000
0:F7/`2UQk<06P2UQk<3000000`054:n00<000000>_E_P3[eKh04@3[eKh4000001D0/Y2c00<00000
0?ooo`3oool0:03oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo0040oooo000=0?oo
o`800000>`3oool00`000000YHNc0:F7/`0H0:F7/`8000000`3oool00`00000054:n01A2_P0901A2
_P030000003[eKh0jmFn0140jmFn00@000000?ooo`3oool0oooo1000000A0;:@/`030000003oool0
oooo02T0oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`010?ooo`003`3oool00`00
0000oooo0?ooo`0h0?ooo`030000002UQk<0YHNc01L0YHNc00<000000?ooo`3oool0103oool00`00
000054:n01A2_P0801A2_P030000003[eKh0jmFn0100jmFn00<000000?ooo`3oool01P3oool30000
00h0/Y2c00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
0040oooo000@0?ooo`800000>03oool00`000000YHNc0:F7/`0E0:F7/`8000001`3oool00`000000
54:n01A2_P0801A2_P030000003[eKh0jmFn0100jmFn00<000000?ooo`3oool02@3oool3000000/0
/Y2c00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0040
oooo000B0?ooo`030000003oool0oooo03H0oooo00<000000:F7/`2UQk<04P2UQk<2000000X0oooo
00<0000001A2_P0D@[h0200D@[h00`000000jmFn0>_E_P0>0>_E_P030000003oool0oooo00d0oooo
100000080;:@/`030000003oool0oooo02/0oooo00<000000?ooo`3oool0203oool100000040oooo
0@3oool001<0oooo00<000000?ooo`3oool0=@3oool00`000000YHNc0:F7/`0@0:F7/`8000003@3o
ool00`00000054:n01A2_P0701A2_P030000003[eKh0jmFn00h0jmFn00<000000?ooo`3oool04@3o
ool3000000D0/Y2c00<000000?ooo`3oool0:`3oool00`000000oooo0?ooo`080?ooo`4000000@3o
ool10?ooo`00503oool2000003D0oooo00<000000:F7/`2UQk<03P2UQk<2000000l0oooo00<00000
01A2_P0D@[h01`0D@[h00`000000jmFn0>_E_P0=0>_E_P030000003oool0oooo01D0oooo10000000
0`2bT;<000000?ooo`0]0?ooo`030000003oool0oooo00L0oooo0@0000010?ooo`40oooo000F0?oo
o`030000003oool0oooo0380oooo00<000000:F7/`2UQk<03@2UQk<00`000000oooo0?ooo`0@0?oo
o`030000000D@[h054:n00H054:n00<000000>_E_P3[eKh03@3[eKh00`000000oooo0?ooo`0I0?oo
o`800000;`3oool00`000000oooo0?ooo`060?ooo`4000000@3oool10?ooo`005`3oool00`000000
oooo0?ooo`0a0?ooo`030000002UQk<0YHNc00/0YHNc0P00000D0?ooo`030000000D@[h054:n00H0
54:n00<000000>_E_P3[eKh02`3[eKh00`000000oooo0?ooo`1<0?ooo`030000003oool0oooo00D0
oooo0@0000010?ooo`40oooo000H0?ooo`800000<@3oool00`000000YHNc0:F7/`090:F7/`800000
5P3oool00`00000054:n01A2_P0601A2_P030000003[eKh0jmFn00/0jmFn00<000000?ooo`3oool0
C@3oool00`000000oooo0?ooo`040?ooo`4000000@3oool10?ooo`006P3oool00`000000oooo0?oo
o`0^0?ooo`030000002UQk<0YHNc00L0YHNc0`00000H0?ooo`030000000D@[h054:n00D054:n00<0
00000>_E_P3[eKh02P3[eKh00`000000oooo0?ooo`1>0?ooo`030000003oool0oooo00@0oooo0@00
00010?ooo`40oooo000K0?ooo`030000003oool0oooo02d0oooo00<000000:F7/`2UQk<01P2UQk<0
10000000oooo0?ooo`00000I0?ooo`030000000D@[h054:n00@054:n00<000000>_E_P3[eKh02P3[
eKh00`000000oooo0?ooo`1?0?ooo`030000003oool0oooo00@0oooo0@0000010?ooo`00703oool2
000002d0oooo00<000000:F7/`2UQk<0102UQk<2000000<0oooo00<000000?ooo`3oool0603oool0
0`00000054:n01A2_P0301A2_P030000003[eKh0jmFn00T0jmFn00<000000?ooo`3oool0D@3oool0
0`000000oooo0?ooo`030?ooo`4000000@3oool001h0oooo00<000000?ooo`3oool0:P3oool00`00
0000YHNc0:F7/`020:F7/`8000001@3oool00`000000oooo0?ooo`0H0?ooo`030000000D@[h054:n
00@054:n00<000000>_E_P3[eKh0203[eKh00`000000oooo0?ooo`1B0?ooo`030000003oool0oooo
0080oooo0@0000010?ooo`007`3oool2000002X0oooo00<000000:F7/`2UQk<00P0000070?ooo`03
0000003oool0oooo01T0oooo00<0000001A2_P0D@[h00`0D@[h00`000000jmFn0>_E_P070>_E_P03
0000003oool0oooo05<0oooo00<000000?ooo`3oool00P3oool100000040oooo000Q0?ooo`030000
003oool0oooo02L0oooo0`0000090?ooo`030000003oool0oooo01X0oooo00<0000001A2_P0D@[h0
0P0D@[h00`000000jmFn0>_E_P060>_E_P030000003oool0oooo05D0oooo00D000000?ooo`3oool0
oooo000000010?ooo`008P3oool00`000000oooo0?ooo`0V0?ooo`030000003oool0oooo00T0oooo
00<000000?ooo`3oool06P3oool00`00000054:n01A2_P0201A2_P030000003[eKh0jmFn00H0jmFn
00<000000?ooo`3oool0EP3oool010000000oooo0?ooo`0000010?ooo`008`3oool200000380oooo
00<000000?ooo`3oool06`3oool00`00000054:n01A2_P0201A2_P030000003[eKh0jmFn00@0jmFn
00<000000?ooo`3oool0F03oool00`000000oooo000000010?ooo`009@3oool00`000000oooo0?oo
o`0_0?ooo`030000003oool0oooo01`0oooo00D0000001A2_P0D@[h054:n000000060>_E_P030000
003oool0oooo05T0oooo0@0000010?ooo`400000000V0?ooo`030000003oool0oooo02h0oooo00<0
00000?ooo`3oool0703oool01@00000054:n01A2_P0D@[h0000000D0jmFn00<000000?ooo`3oool0
FP3oool100000040oooo0@00000002L0oooo0P00000^0?ooo`030000003oool0oooo01d0oooo00@0
000001A2_P0D@[h000001@3[eKh00`000000oooo0?ooo`1K0?ooo`800000000Y0?ooo`030000003o
ool0oooo02/0oooo00<000000?ooo`3oool07P3oool01000000054:n01A2_P0000030>_E_P030000
003oool0oooo05/0oooo0`00000002X0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?oo
o`0O0?ooo`030000000D@[h0000000<0jmFn00<000000?ooo`3oool0E`3oool4000000<0oooo000[
0?ooo`800000:P3oool00`000000oooo0?ooo`0O0?ooo`030000000D@[h000000080jmFn00<00000
0?ooo`3oool0E03oool4000000L0oooo000]0?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool0803oool200000080jmFn00<000000?ooo`3oool0D03oool4000000/0oooo000^0?ooo`80
00009`3oool00`000000oooo0?ooo`0Q0?ooo`<00000C`3oool4000000l0oooo000`0?ooo`030000
003oool0oooo02@0oooo00<000000?ooo`3oool08@3oool3000004/0oooo1000000C0?ooo`00<@3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0280oooo00<000000?ooo`3oool0AP3o
ool4000001L0oooo000b0?ooo`8000008`3oool00`000000oooo0?ooo`1V0?ooo`D000006`3oool0
03@0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`1R0?ooo`@00000803oool003D0
oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`1N0?ooo`@00000903oool003H0oooo
0P00000O0?ooo`030000003oool0oooo05X0oooo1000000X0?ooo`00>03oool00`000000oooo0?oo
o`0L0?ooo`030000003oool0oooo05H0oooo1000000/0?ooo`00>@3oool2000001`0oooo00<00000
0?ooo`3oool0DP3oool400000300oooo000k0?ooo`030000003oool0oooo01T0oooo00<000000?oo
o`3oool0C@3oool5000003@0oooo000l0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3o
ool0B@3oool4000003T0oooo000m0?ooo`800000603oool00`000000oooo0?ooo`150?ooo`@00000
?@3oool003l0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`110?ooo`@00000@@3o
ool00400oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`0m0?ooo`@00000A@3oool0
0440oooo0P00000C0?ooo`030000003oool0oooo03X0oooo100000190?ooo`00@`3oool00`000000
oooo0?ooo`0@0?ooo`030000003oool0oooo03H0oooo1000001=0?ooo`00A03oool00`000000oooo
0?ooo`0?0?ooo`030000003oool0oooo0340oooo1@00001A0?ooo`00A@3oool2000000l0oooo00<0
00000?ooo`3oool0;@3oool4000005H0oooo00170?ooo`030000003oool0oooo00`0oooo00<00000
0?ooo`3oool0:@3oool4000005X0oooo00180?ooo`800000303oool00`000000oooo0?ooo`0U0?oo
o`@00000GP3oool004X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0Q0?ooo`@0
0000HP3oool004/0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0M0?ooo`@00000
IP3oool004`0oooo0P0000080?ooo`030000003oool0oooo01P0oooo1@00001Z0?ooo`00CP3oool0
0`000000oooo0?ooo`050?ooo`030000003oool0oooo01@0oooo1000001_0?ooo`00C`3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0100oooo1000001c0?ooo`00D03oool2000000@0
oooo00<000000?ooo`3oool0303oool4000007L0oooo001B0?ooo`050000003oool0oooo0?ooo`00
00002P3oool4000007/0oooo001C0?ooo`040000003oool0oooo000000H0oooo1000001o0?ooo`00
E03oool300000080oooo100000230?ooo`00EP3oool3000008L0oooo0000\
\>"],
  ImageRangeCache->{{{0, 223.125}, {223.25, 0}} -> {0.0185341, -4.60498*^-06, \
0.00412541, 0.00412541}},
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[42]=",
  CellTags->{"b:0.5", "IPM3.1.1.6"}]
}, Open  ]],

Cell["\<\
Here is a short program that creates a button containing the above two \
expressions.\
\>", "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell["\<\
Cell[BoxData[
    ButtonBox[
      RowBox[{\"Stellate\", \" \", \"Icosahedron\"}],
      ButtonFunction:>CompoundExpression[ 
          Needs[ \"Graphics`Polyhedra`\"], 
          Show[Stellate[Polyhedron[Icosahedron]]]
          ],
      ButtonEvaluator->Automatic],
  \"Input\",
  Active->True]\
\>", "Program",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[TextData[{
  "The formatted version of the above cell can be displayed by choosing ",
  StyleBox["Show Expression", "SR"],
  " from the ",
  StyleBox["Format", "SR"],
  " menu. When you do that, it will look like the following:"
}], "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[BoxData[
    ButtonBox[
      RowBox[{
        StyleBox["Stellate",
          "TB"], " ", 
        StyleBox["Icosahedron",
          "TB"]}],
      ButtonFunction:>CompoundExpression[ 
          Needs[ "Graphics`Polyhedra`"], 
          Show[ 
            Stellate[ 
              Polyhedron[ Icosahedron]]]],
      ButtonEvaluator->Automatic]], "DisplayFormula",
  Active->True,
  Evaluatable->False,
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[TextData[{
  "Clicking the button will cause the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " code in the ",
  StyleBox["ButtonFunction", "MR"],
  " to be immediately evaluated and the following graphics will then be \
displayed in your notebook."
}], "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[TextData[{
  "Functions are available to jump around to different parts of a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook and perform various actions. Here is a short piece of code that \
creates a button which, upon being clicked, moves the selection to the next \
cell and then evaluates that cell."
}], "Text",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell["\<\
Cell[TextData[{
  Cell[BoxData[
      ButtonBox[\"EVALUATE\",
        ButtonFunction:>FrontEndExecute[ {
            FrontEnd`SelectionMove[ 
              ButtonNotebook[ ], All, ButtonCell], 
            FrontEnd`SelectionMove[ 
              ButtonNotebook[ ], Next, Cell], 
            FrontEnd`SelectionEvaluate[ 
              ButtonNotebook[ ]]}],
        Active->True]]],
  StyleBox[\" MATHEMATICA INPUT\"]
}], \"Text\"]\
\>", "Program",
  PageBreakAbove->True,
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[TextData[{
  "The formatted version of the above cell can be displayed by choosing ",
  StyleBox["Show Expression", "SR"],
  " from the ",
  StyleBox["Format", "SR"],
  " menu. When you do that, it will look somewhat like the following \
(although we have removed some of the text formatting above to improve \
readability of the code). Clicking the ",
  StyleBox["EVALUATE", "SR"],
  " button will cause the input cell immediately following to be selected and \
then evaluated."
}], "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[BoxData[
    FormBox[
      RowBox[{
        ButtonBox[
          StyleBox["EVALUATE",
            FontFamily->"Helvetica",
            FontSize->10,
            FontWeight->"Bold"],
          ButtonFunction:>FrontEndExecute[ {
              FrontEnd`SelectionMove[ 
                ButtonNotebook[ ], All, ButtonCell], 
              FrontEnd`SelectionMove[ 
                ButtonNotebook[ ], Next, Cell], 
              FrontEnd`SelectionEvaluate[ 
                ButtonNotebook[ ]]}],
          Active->True,
          Background->GrayLevel[0.500008]], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontWeight->"Bold",
          FontSlant->"Italic",
          FontColor->GrayLevel[1]], 
        StyleBox["MATHEMATICA",
          FontFamily->"Helvetica",
          FontSize->10,
          FontWeight->"Bold",
          FontSlant->"Italic",
          FontColor->GrayLevel[1]], 
        StyleBox[" ",
          FontFamily->"Helvetica",
          FontWeight->"Bold",
          FontSlant->"Italic",
          FontColor->GrayLevel[1]], 
        StyleBox["INPUT",
          FontFamily->"Helvetica",
          FontSize->10,
          FontWeight->"Bold",
          FontSlant->"Italic",
          FontColor->GrayLevel[1]]}], 
      DefaultTraditionalFormatType]], "DisplayFormula",
  CellMargins->{{Inherited, 100}, {Inherited, Inherited}},
  Background->GrayLevel[0.500008],
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(3 \((4 + 5)\)\)], "Input",
  CellLabel->"In[43]:=",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[BoxData[
    \(27\)], "Output",
  CellLabel->"Out[43]=",
  CellTags->{"b:0.5", "IPM3.1.1.6"}]
}, Open  ]],

Cell[TextData[{
  "The following example demonstrates how you can use ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " functions to perform some of the user interface actions that you would \
normally associate with keyboard and mouse events. By using such techniques, \
you can create a specific set of actions that will follow certain \
evaluations. For example, if you were creating an electronic quiz for your \
students, you could include \[OpenCurlyDoubleQuote]hint\
\[CloseCurlyDoubleQuote] buttons within your class notebooks that would open \
a new notebook with hints and suggestions upon clicking. "
}], "Text",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[TextData[{
  "This creates a new notebook that contains three cells",
  " \[Dash] ",
  "a ",
  StyleBox["Section", "MR"],
  " cell, a ",
  StyleBox["Text", "MR"],
  " cell, and an ",
  StyleBox["Input", "MR"],
  " cell. Upon evaluation, the ",
  StyleBox["NotebookPut", "MR"],
  " command below will cause a new notebook to appear, containing the three \
specified cells. The screen shots below show what appears in the user \
interface after evaluating each of the preceding inputs."
}], "Text",
  CellTags->{"i:27", "b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(nb = 
      NotebookPut[\[IndentingNewLine]Notebook[{\[IndentingNewLine]Cell["\<\
Symbolic and Interactive Documents\>", "\<Section\>"], \
\[IndentingNewLine]Cell["\<Cells and notebooks are Mathematica \
expressions.\>", "\<Text\>"], \
\[IndentingNewLine]Cell["\<Integrate[Sin[x]/Cos[x],x]\>", \ "\<Input\>"]\
\[IndentingNewLine]}]]\)], "Input",
  CellLabel->"In[44]:=",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[BoxData[
    InterpretationBox[\(NotebookObject[\[LeftSkeleton]"Untitled-1"\
\[RightSkeleton]]\),
      NotebookObject[ 
        FrontEndObject[ 
          LinkObject[ "3dv_shm", 1, 1]], 16],
      Editable->False]], "Output",
  CellLabel->"Out[44]=",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .54907 
%%ImageSize: 378 208 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
2.5804e-017 0.00265252 -0.00265252 0.00265252 [
[ 0 0 0 0 ]
[ 1 .54907 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .54907 L
0 .54907 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00265 translate
1.00265 .55172 scale
1134 string
378 208 8 [378 0 0 208 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FEFEFEFEFEFEFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFC
FCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFC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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFBFBFB
FAFAFAF8F8F8F7F7F7F6F6F6F5F5F5F4F4F4F4F4F4F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F4F4F4F4F4F4
F5F5F5F6F6F6F7F7F7F8F8F8FAFAFAFBFBFBFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9
F8F8F8F6F6F6F5F5F5F3F3F3F1F1F1F0F0F0EFEFEFEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEFEFEFF0F0F0
F1F1F1F3F3F3F5F5F5F6F6F6F8F8F8F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7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FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7F5F5F5
F3F3F3F0F0F0EDEDEDEAEAEAE7E7E7E4E4E4E2E2E2E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E2E2E2E4E4E4
E7E7E7EAEAEAEDEDEDF0F0F0F3F3F3F5F5F5F7F7F7F9F9F9FBFBFBFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F3F3F3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FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F4F4F4F0F0F0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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F5F5F5F1F1F1EDEDED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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F6F6F6F3F3F3EFEFEFEAEAEA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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F5F5F5F1F1F1EDEDEDE7E7E7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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F4F4F4F0F0F0EBEBEBE4E4E4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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFBFBFBF9F9F9F6F6F6F4F4F4EFEFEFE9E9E9E2E2E2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FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F6F6F6F3F3F3EEEEEEE8E8E8E1E1E1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000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
ECECECBABABA9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FBDBDBDECECECE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
969696DBDBDBCACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACADBDBDBA3A3A3FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFF
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0FFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFF
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E0000000000000000009E9E9ED9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
DCDCDC515151B9B9B9E0E0E0D9D9D91A1A1A000000383838D9D9D9E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD1D1D1D0D0D0D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0
D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF969696FAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0E0E0E0E0E0E0
E0E0E0ADADAD545454E0E0E0ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6
D6D6D6D7D7D7D7D7D7D7D7D7D6D6D6D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6
959595D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D7D7D7
D7D7D7D7D7D7D6D6D6D6D6D6D6D6D6FFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCECECECECECECECECE
CECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFF
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9E0E0E0E0E0E0
E0E0E0DCDCDC515151B9B9B9ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595DCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFF
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2E0E0E0E0E0E0
E0E0E0E0E0E0C4C4C4484848D2D2D21A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000000000
000000000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFD4D4D4D4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595DCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF969696F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8969696FFFFFFD4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2DCDCDC494949
0000000D0D0DD2D2D26969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000
000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFDBDBDBDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696
969696FFFFFFE4E4E4E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E4E4E4
E4E4E4E4E4E4E4E4E4E3E3E3E4E4E4FFFFFF969696F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7969696FFFFFFDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9ADADAD545454
E0E0E06969697E7E7EC4C4C4484848D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFE4E4E4E4E4E4E4E4E4
3E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF969696F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5969696FFFFFFE4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E000000383838D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0ADADAD545454
E0E0E06969697E7E7EE0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF969696F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDEDEDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D9D9D9525252D9D9D9E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0C4C4C41D1D1DD2D2D2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFF
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
969696FFFFFFF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFFF969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
969696FAFAFAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFAFAFA969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0DADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
D6D6D6B0B0B0B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8A9A9A9CECECEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEDADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECE3E3E3ECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3E3E3E3E3E3E3ECFCFCF
CFCFCFCFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3E
D4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E
3E3E3EDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFE4E4E4E4E4E43E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E43E3E3E
E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFC6C6C6000000000000090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF121212000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000606060FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000606060
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFF7F7F7262626D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1B
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFC6C6C6606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFBFBFB545454000000414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECECF0F0F08E8E8EA0A0A0A0A0A0A0A0A0A0A0A0A0A0A09F9F9FA0A0A0A0A0A0A0A0A0
A0A0A0A0A0A0A0A0A0909090ECECEC696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDED3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3DEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C6000000
0000000000000F0F0FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB545454000000B4B4B4FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFF7F7F71D1D1D000000B4B4B4F7F7F71D1D1D000000B4B4B4FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2
000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878
B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF787878B4B4B4FFFFFFF7F7F71D1D1D000000B4B4B4FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D000000000000000000000000
0000000000008F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878B4B4B4
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECFDFDFD939393B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B5B5B5B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8939393ECECEC696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECE3E3E3ECECECECECECECECECECECECECECECE3E3E3E
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF3E3E3E3E3E3E
3E3E3ECFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3E
D4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3E3E3E3E3E3E3EDBDBDB
DBDBDBDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFE4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDED3E3E3EEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
ECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F79A9A9A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A9A9A9AF7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFF7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3F7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F1F1F1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE3E3E3A5A5A5767676767676
767676767676777777767676777777777777767676767676767676767676767676767676
767676767676767676A5A5A5E3E3E3FFFFFFFFFFFFE3E3E3A5A5A5767676767676767676
767676777777767676777777777777767676767676767676767676767676767676767676
767676767676A5A5A5E3E3E3FFFFFFFFFFFFEBEBEBA5A5A5565656565656565656565656
575757565656575757575757565656565656565656565656565656565656565656565656
565656A5A5A5EBEBEBFFFFFFFFFFFFFFFFFFFFFFFFF1F1F10000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E1E1E1FCFCFCFBFBFBFCFCFCFCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCA5A5A57676768D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D767676A5A5A5FCFCFCFCFCFCA5A5A57676768D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D767676A5A5A5FCFCFCFBFBFBA5A5A55656565B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B565656A5A5A5FCFCFCFCFCFCFCFCFCFCFCFCE1E1E10000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F75656565B5B5B6F6F6F717171727272727272
727272727272727272727272727272727272727272727272727272727272727272717171
6F6F6F5B5B5B565656F7F7F7F7F7F7F7F7F7F7F7F7E0E0E00000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DEDEDEF1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1
F1F1F1F1F1F1F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2
F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1767676B8B8B8ABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABB8B8B8767676F1F1F1F1F1F1767676B8B8B8ABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABB8B8B8767676F1F1F1F2F2F25656566F6F6F6F6F6F727272777777777777
777777777777777777777777777777777777777777777777777777777777777777727272
6F6F6F6F6F6F565656F1F1F1F1F1F1F1F1F1F2F2F2DEDEDE0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DCDCDCEBEBEBEBEBEBECECECEBEBEBEBEBEBA7A7A7A2A2A2A2A2A2A1A1A1A1A1A1
A0A0A09F9F9F9D9D9D9F9F9F9F9F9FA0A0A0A0A0A0A1A1A1A1A1A1A7A7A7EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBD4D4D4C5C5C5C5C5C5C5C5C5D7D7D7EBEBEBEBEBEB
EAEAEAC8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3EBEBEBE0E0E0C5C5C5C5C5C5
CCCCCCE9E9E9CECECEC7C7C7E9E9E9EBEBEBE0E0E0C5C5C5C5C5C5CCCCCCEAEAEAD4D4D4
C5C5C5DDDDDDEBEBEBE0E0E0C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEAEAEAC8C8C8
C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBE5E5E5C7C7C7D7D7D7EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1B1B1B1B1B1B1B1B1B1
B1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1
B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB565656777777727272747474777777787878
787878787878787878787878787878787878787878787878787878787878777777747474
727272777777565656EBEBEBEBEBEBEBEBEBEBEBEBDCDCDC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DADADAE5E5E5E5E5E5E5E5E5E5E5E58A8A8A9B9B9BACACACE9E9E9E9E9E9E9E9E9
EAEAEAC4C4C4858585ECECECEDEDEDEEEEEEE7E7E7F0F0F0F2F2F2A3A3A3E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5686868101010101010101010808080C6C6C6CFCFCFDEDEDE
262626515151CFCFCFE5E5E5949494151515A4A4A4DEDEDEB2B2B2101010101010414141
DBDBDB505050191919BABABAE3E3E3B2B2B2101010101010414141DFDFDF707070101010
929292E4E4E4AFAFAF101010101010151515BABABACCCCCCE4E4E4DDDDDD242424101010
101010101010151515A4A4A4DEDEDEE5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
C9C9C91C1C1C727272D4D4D4E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5767676C0C0C0B5B5B5B5B5B5
FFFFFF727272727272727272727272727272727272272727B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5C0C0C0767676E5E5E5E5E5E5767676C0C0C0B5B5B5B5B5B5FFFFFF
272727272727272727272727272727272727272727272727272727272727272727B5B5B5
B5B5B5B5B5B5C0C0C0767676E5E5E5E5E5E55656567777777777777979797A7A7AFFFFFF
FFFFFF7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7AFFFFFFFFFFFF7A7A7A797979
777777777777565656E5E5E5E5E5E5E5E5E5E5E5E5DADADA0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D9D9D9DFDFDFDFDFDFDFDFDFDFDFDFDEDEDEA0A0A0EAEAEAE9E9E9E5E5E5EAEAEA
EAEAEAD0D0D0747474EDEDEDEEEEEEADADADD0D0D0F2F2F2F3F3F3A3A3A3DFDFDFDEDEDE
DEDEDEDFDFDFDEDEDE6C6C6C151515ABABABDEDEDEC3C3C31C1C1C545454C6C6C6D6D6D6
252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4D5D5D54E4E4E383838C4C4C4DDDDDD
DBDBDB4E4E4E191919BABABAD9D9D94E4E4E383838C4C4C4DDDDDDD7D7D7252525383838
C4C4C4C2C2C21B1B1B515151D9D9D9C3C3C34C4C4CD8D8D8D7D7D7252525191919C5C5C5
DEDEDE8F8F8F151515A4A4A4D9D9D9DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
C3C3C31C1C1C727272D1D1D1DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBB
FFFFFF727272727272727272727272727272727272272727BBBBBBBBBBBBBBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBC3C3C3767676DEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBBFFFFFF
272727BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBFFFFFF272727BBBBBB
BBBBBBBBBBBBC3C3C3767676DFDFDFDEDEDE5656567777777878787B7B7B303030FFFFFF
FFFFFFFFFFFF7D7D7D7D7D7D7D7D7D7D7D7D7D7D7DFFFFFFFFFFFFFFFFFF3030307B7B7B
787878777777565656DEDEDEDEDEDEDFDFDFDEDEDED9D9D90000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D7D7D7D8D8D8D9D9D9D8D8D8D8D8D89090908F8F8F8E8E8EE9E9E9818181515151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000000D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3858585A3A3A3E6E6E6EAEAEAEAEAEA353535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000000D3D3D3CECECECECECECFCFCFCECECECECECE878787939393D3D3D3DBDBDBADADAD
C1C1C1B0B0B0A9A9A9AAAAAA8D8D8D919191B0B0B0CBCBCBEFEFEFA5A5A5CECECECECECE
CECECECFCFCFC7C7C7232323515151C8C8C8CECECECECECE656565191919B9B9B9C7C7C7
232323515151C7C7C7CECECE858585151515A4A4A4C7C7C7232323383838C1C1C1CECECE
CBCBCB494949191919B9B9B9C7C7C7232323383838C1C1C1CECECEC7C7C7232323383838
C1C1C1858585151515A4A4A4CACACAC7C7C7232323515151B1B1B11A1A1A727272C5C5C5
CECECE858585151515A4A4A4CDCDCDCECECECECECECECECECECECECECECECECECECECECE
B4B4B41A1A1A727272CACACACECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECE767676CECECECBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCECECE767676CECECECECECE767676CECECECBCBCBCBCBCBFFFFFF
272727CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBFFFFFF272727CBCBCB
CBCBCBCBCBCBCECECE767676CFCFCFCECECE5656567D7D7D828282858585848484858585
858585282828FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF282828858585858585848484858585
8282827D7D7D565656CECECECECECECECECECECECED3D3D3000000A8A8A8ADADADB3B3B3
BBBBBBC3C3C3CCCCCCD5D5D5DDDDDDE4E4E4EBEBEBF0F0F0F4F4F4F7F7F7F9F9F9FCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D0D0D0C9C9C9C9C9C9C9C9C9C9C9C97373739494947E7E7E797979A0A0A0D1D1D1
ADADADA5A5A5A6A6A68585855E5E5EADADADE5E5E5F6F6F6F8F8F8A4A4A4C9C9C9C8C8C8
C8C8C8C8C8C8C2C2C2232323515151C7C7C7C8C8C8C8C8C8626262191919B9B9B9C2C2C2
222222383838C1C1C1C2C2C2464646191919BCBCBCC1C1C1232323383838BFBFBFC5C5C5
C4C4C4464646191919B9B9B9C1C1C1232323383838BFBFBFC5C5C5C0C0C0222222383838
C0C0C0AFAFAF1A1A1A737373C5C5C59A9A9A1010108E8E8EC5C5C5464646191919B9B9B9
C5C5C57F7F7F151515A4A4A4C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C7C7C7
AEAEAE1A1A1A717171C7C7C7C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8767676D3D3D3CDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDD3D3D3767676C8C8C8C8C8C8767676D3D3D3CDCDCDCDCDCDFFFFFF
272727CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDFFFFFF272727CDCDCD
CDCDCDCDCDCDD3D3D3767676C8C8C8C8C8C8565656828282838383878787878787878787
878787878787282828FFFFFFFFFFFFFFFFFF282828878787878787878787878787878787
838383828282565656C9C9C9C9C9C9C9C9C9C8C8C8D0D0D0000000B2B2B2B6B6B6BBBBBB
C2C2C2C9C9C9D1D1D1D9D9D9E0E0E0E7E7E7EDEDEDF1F1F1F5F5F5F8F8F8FAFAFAFCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000CFCFCFC5C5C5C5C5C5C5C5C5C5C5C5C5C5C5979797B7B7B7ECECECB9B9B98B8B8B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000000CDCDCDC1C1C1C1C1C1C1C1C1C1C1C1757575999999A7A7A7EDEDEDC9C9C9737373
B0B0B08D8D8D7373734C4C4C626262BEBEBEB5B5B5B2B2B2ADADADA9A9A9C1C1C1C1C1C1
C1C1C1C1C1C1BBBBBB222222515151C4C4C4C1C1C1C1C1C15F5F5F191919B9B9B9C2C2C2
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1BBBBBB222222383838BEBEBEC2C2C2
C1C1C1C1C1C1C3C3C3C5C5C5BCBCBC222222383838BEBEBEC2C2C2BBBBBB222222383838
BEBEBEC2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C17C7C7C151515A4A4A4C3C3C3C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1A9A9A91A1A1A
101010101010717171C4C4C4C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1767676D8D8D8D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D8D8D8767676C1C1C1C1C1C1767676D8D8D8D5D5D5D5D5D5FFFFFF
272727FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF272727D5D5D5
D5D5D5D5D5D5D8D8D8767676C1C1C1C1C1C15656568484848888888A8A8A8A8A8A8A8A8A
8A8A8AFFFFFFFFFFFFFFFFFF282828FFFFFFFFFFFFFFFFFF8A8A8A8A8A8A8A8A8A8A8A8A
888888848484565656C1C1C1C1C1C1C1C1C1C1C1C1CDCDCD000000C6C6C6C8C8C8CCCCCC
D1D1D1D7D7D7DCDCDCE2E2E2E8E8E8EDEDEDF1F1F1F5F5F5F7F7F7F9F9F9FCFCFCFDFDFD
FDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
000000CCCCCCBABABABDBDBDBDBDBDBEBEBEBDBDBD9E9E9EE3E3E3B6B6B6969696828282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000000CACACAB8B8B8BABABABABABABABABA7C7C7C8D8D8D787878999999F0F0F0C2C2C2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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B88686869E9E9ED4D4D4F0F0F0F2F2F2CECECE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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8868686949494F2F2F2F3F3F3A0A0A0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000000C9C9C9B5B5B5B5B5B5B5B5B5B5B5B5777777A0A0A0E0E0E0F3F3F3E5E5E5ECECEC
F6F6F6F8F8F8F9F9F9909090FBFBFBFCFCFCFDFDFDFEFEFEFEFEFEA7A7A7B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5767676EDEDEDE9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9EDEDED767676B5B5B5B5B5B5767676EDEDEDE9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EDEDED767676B5B5B5B5B5B55656569B9B9B9E9E9EA5A5A5B2B2B2BABABA
BFBFBFBFBFBFC0C0C0BFBFBFBFBFBFC0C0C0BFBFBFBFBFBFC0C0C0BABABAB2B2B2A5A5A5
9E9E9E9B9B9B565656B5B5B5B5B5B5B5B5B5B5B5B5C9C9C9000000EEEEEEEFEFEFF0F0F0
F1F1F1F3F3F3F5F5F5F6F6F6F8F8F8F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000C9C9C9BABABAB1B1B1B1B1B1B1B1B1B1B1B18D8D8DAAAAAAAAAAAAAAAAAAAAAAAA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000000C3C3C3C9C9C9AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE8D8D8D767676CECECEFBFBFB
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFBFBFBCECECE7676768D8D8DAEAEAEAEAEAE8D8D8D767676CECECEFBFBFBFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FBFBFBCECECE7676768D8D8DAEAEAEAEAEAE999999565656B5B5B5CBCBCBDFDFDFE8E8E8
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8DFDFDFCBCBCB
B5B5B5565656999999AEAEAEAEAEAEAEAEAEC9C9C9C3C3C3000000F6F6F6F6F6F6F7F7F7
F8F8F8F8F8F8F9F9F9FAFAFAFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000C3C3C3D4D4D4B7B7B7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D565656565656565656565656
565656565656565656565656565656565656565656565656565656565656565656565656
5656568D8D8DA7A7A7A7A7A7A7A7A7B7B7B7D4D4D4C3C3C3000000F9F9F9F9F9F9F9F9F9
FAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000AAAAAAE7E7E7D4D4D4BEBEBEB3B3B3AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAB3B3B3BEBEBED4D4D4E7E7E7AAAAAA000000FBFBFBFBFBFBFCFCFC
FCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
0000008A8A8AD9D9D9FCFCFCE4E4E4D2D2D2CCCCCCBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBECCCCCCD2D2D2E4E4E4FCFCFCD9D9D98A8A8A000000FCFCFCFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000696969979797D9D9D9FCFCFCF2F2F2ECECECE3E3E3D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9E3E3E3ECECECF2F2F2FCFCFCD9D9D9979797696969000000FDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFF696969979797D3D3D3FCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFCFCFCFCFCFCD3D3D3979797696969FFFFFF000000FDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFF6969696969698A8A8A979797AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAA9797978A8A8A696969696969FFFFFFFFFFFF000000FEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{378, 208},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005j0000d2000`400?l00000o`00003oo`3ooomk
0?ooo`00o`3ooomk0?ooo`003P3ooooo0?knoUh0o_kn3`3oool000`0oooo103no_ko0?gmoEX0oOgm
103no_h=0?ooo`002P3oool30?kno_l0oOgmH03moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo5@0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?clo?l0no_kEP3kno/30?cl
o0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinEL0
nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_IF0?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`4
0?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gmo@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0
moOg0?KfmP3emOD00P3dm?Co0??cle@0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm00X0o?cl0?_kn`3inOT0
n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^k^iD0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0
m_Kf0?Shn03inOT0no_k0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj5@0j>SX00/0jNWY0>_[j`3]
kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0030?gmo@80o_kn103oool000<0oooo0100
o_kn0?gmo@3moOd0oOgm0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN5D0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3kno/0
o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`000P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0
m_Kf0??cl`3_knl0k>c/0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SHDP3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`00`3moOd20?knoP<0oooo
00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;R
hP3MgMd0fMWI0=GEe@3Bd]80d=3@o`3?clmB0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0oooo0040oooo00000`3o
ool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3A
dM40c<c<0<S8b036a/Ko0<G5aE80aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo0?knoP3no_h00P3moOd0
3`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl
_03o0;^k^e80^k^k00l0_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gmo@020?gmo@0?0?clo03j
n_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a
DP2a/K403`2b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd00P3moOd03`3lo?`0nOWi
0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYU80
YZJV00l0Z:RX0:f][@2c/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080oOgm00l0no_k0?Win@3f
m_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S80;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYiB09jN
WP0?0:2PX02UYJD0[Jf]0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0o?cl0?_kn`3inOT0m_Kf
0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l_;`0/[:b0:RXZ02PX:00o`2JVYYD09ZJVP0?0:2P
X02XZ:P0/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000006H000000100VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00003
0000003lo?`0o?cl0?l0o?clHP3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`00
0000o?cl0:^[Z`3o0:^[Zf40Zj^[0180o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFUP06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj9`2OWil00`2m_Kd0k>c/0>OWi`0_0>OWi`030>k^kP2=SHd0WinO00`0WinO00<0SHf=
0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3Ng]h0fm_K
00`0fm_K00@0g]kN09JFUP2FUYH0fm_K9`3:b/X00`3Kfm/0Xj>S0?_kn`0_0?_kn`0309JFUP3Ng]h0
fm_K00`0fm_K00<0g]kN09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo9`3@d=000`3oool0UYJF0?_kn`0_0?_kn`0309JFUP3oool0c/k>00`0c/k>00<0oooo
09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY
09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF
09JFUP3ooolW0<o?c`030?ooo`2FUYH0no_k02l0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ0000000=3@d0d];B0>3P
h03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3Lg=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph
0=WIf@0F0>3Ph00309JFUP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM401@3@d=0;0=7Ad@<0d=3@0`3AdM450=3@d0T0dM7A00<0oooo09JF
UP3jn_X0;`3jn_X00`2FUYH0oooo0<k>cP040<k>cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JF
UP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000
003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3P
h01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0103>c/h303hn?PD0c/k>
00@0oooo09JFUP2FUYH0oooo2`3Fe]H30=OGe`80e]KF00L0UIFE0=KFeP2EUID0e]KF09FEU@3Fe]H0
UIFE00X0e]KF0`3GemL30=KFeP030?ooo`2FUYH0nOWi02l0nOWi00<0UYJF0?ooo`3>c/h00`3>c/h3
03hn?PH0c/k>00<0oooo09JFUP3Ph>002@3Ph>020?ooo`80h>3P0P3oool05@3Ph>00JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3P
h0030?ooo`3Ph>00h>3P00@0h>3P0100a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0
d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0`3Ph>00203Lg=`0DE5A0;Vi^@2][Jd0E5AD0>3Ph01Y
JFT0WYjN5P3Ph>000`2FUYH0oooo0<o?c`030<o?c`<0?Shn1P3?cll0103oool0UYJF09JFUP3oool3
0=cLg0/0gMgM00T0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Lg=`;0=gMg@80
g=cL00<0gMgM0?ooo`2FUYH0<03inOT00`2FUYH0oooo0<o?c`040<o?c`<0?Shn1@3?cll00`3oool0
UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV
0=;BdP3Ph>00fMWI02HV9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103oool0UYJF09JFUP3oool3
0=cLg0/0gMgM00T0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Lg=`;0=gMg@80
g=cL00<0gMgM0?ooo`2FUYH0<03hn?P00`2FUYH0oooo0=CDe0050=CDe0<0?Shn103De=@00`3oool0
UYJF0>3Ph0080>3Ph080/k>c0P3Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00
103Ph>005`34a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00
fMWI02HV9P3Bd]80g=cL04U9B@0000003@d=0=;BdP1YJFT0WYjN00l0h>3P1@0000060>3Ph00409JF
UP3oool0fm_K0=_Kf`<0?Shn203Kfm/01@3oool0UYJF09JFUP3oool0i>CT0080hn?S0`3Ti>@50>?S
h`@0i>CT00P0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@80hn?S0`3Ti>@50>?Sh`@0i>CT
00@0hn?S0>CTi03oool0UYJF<03gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0?Shn0`3Kfm/00`3oool0
UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P01P0a<C404Q8B03IfMT0
fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0
h>3P06UYJ@1nOWh0a<C404Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0`3Zj^X80>WYj@090>[ZjP3oool0UIFE
0?ooo`2EUID0oooo09FEU@3oool0UIFE0080jNWY0`3Zj^X80>WYj@050>[ZjP3YjNT0jNWY0?ooo`2F
UYH0<03emOD00`2FUYH0oooo0>CTi0050>CTi0<0?Shn103Ti>@00`3oool0UYJF0>3Ph00=0>3Ph080
oooo01D0h>3P06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0030>3Ph00I08j>SP000000>3Ph0=WIf@3P
h>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00[Jf]05ADE03P
h>00JFUY07inOP3Ph>00JFUY09jNWP0@0>3Ph0030000003Ph>00h>3P00H0h>3P00<0UYJF0?ooo`3]
kNd00`3]kNd303hn?PH0kNg]00D0oooo09JFUP2FUYH0oooo0>[ZjP020>WYj@<0j^[Z203YjNT02@3Z
j^X0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@020>WYj@<0j^[Z203YjNT01@3Zj^X0jNWY
0>WYj@3oool0UYJF0300m?Cd00<0UYJF0?ooo`3]kNd0103]kNd303hn?PD0kNg]00<0oooo09JFUP3P
h>00303Ph>020;>c/`0F0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>006@3IfMT0
DU9B0=WIf@3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00
fMWI01XJ6P0000003@d=0=;BdP3Ph>00a<C401dM7@3Bd]80603Ph>000`2FUYH0oooo0?Cdm0040?Cd
m0<0?Shn1@3dm?@0103oool0UYJF09JFUP3oool?0>g]k@070?ooo`3]kNd0oooo0>g]k@3oool0kNg]
0?ooo`0A0>g]k@030?ooo`2FUYH0lO7a02l0lO7a00<0UYJF0?ooo`3dm?@00`3dm?@303hn?PH0m?Cd
00<0oooo09JFUP3Ph>00303Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3P
h>000`2FUYH0oooo0?clo0050?clo00303hn?P3lo?`0o?cl00@0o?cl00@0oooo09JFUP2FUYH0oooo
9`3]kNd00`3oool0UYJF0?3`l00_0?3`l00309JFUP3oool0o?cl00@0o?cl00<0?Shn0?clo03lo?`0
1@3lo?`00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0
UYJF0?ooo`3oool03@3oool209JFUP030?ooo`3gmoL0moOg02D0moOg00<0oooo09JFUP3_knl0;`3_
knl00`2FUYH0oooo0?ooo`0=0?ooo`0309JFUP3Ph>00h>3P00l0h>3P01@0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00=@3Ph>000`2FUYH0emOG0?ooo`0<0?ooo`040=OGe`2FUYH0UYJF0?[jnRL0oooo00<0
n_[j09JFUP3_knl0;`3_knl00`2FUYH0emOG0?ooo`0<0?ooo`030=OGe`2FUYH0h>3P0100h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`3Jf]X0TY:B0;Jf]P0;0;Fe]@050;Bd]02?Shl0
f][J0=KFeP2`/;009`2h^;P00`2YZJT0c/k>0>k^kP0_0>k^kP030=[JfP2BTY80][Jf00/0]KFe00<0
];Bd08n?S`3Jf]X04@3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJIL0WYjN00<0oooo0>3Ph03Ph>00=@3Ph>0A0>c/k2/0jn_[<03WinLA
0>c/k140h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?oo
o`030>c/k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`0<0<k>
cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?oo
o`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`050<k>
cP0303hn?P3>c/h0c/k>00@0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`040<k>
cP<0?Shn1@3>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00<0clo?1@0n?Sh40<o?
c`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3De=@303hn?P040=CDe00n?Sh0?Shn
03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooC@3oool0103/k>`0UYJF0?ooo`3Kfm/303hn?P<0fm_K0`0n
?Sh20=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocd0oooo1@1EEED20?ooo`D0EEEE103oool00`3/k>`0UYJF0?oo
o`020>CTi00303hn?P3Ti>@0i>CT00<0i>CT01X0?Shn0>CTi03Ti>@0i>CT0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0
kNg]01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYG`3oool01@36a/H00000000000092@T0d];B
01l0oooo00@0gmoO018B4P000000H61P603oool0103Ogml04Q8B0000001PH6090?ooo`0407QhN000
00002@T90=;BdR00oooo00@0gmoO018B4P000000H61P2@3oool0101hN7P0000000T92@3Bd]840?oo
o`030861P@3Bd]80oooo00X0oooo00@0N7Qh000000092@T0d];BBP3oool00`1EEED0oooo0?ooo`04
0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYHP3oool0
0`2RXZ80@D510?Ogm`0N0?ooo`030=oOg`0K6a/0d];B01T0oooo00<0gmoO01/K6`3Bd]802`3oool0
0`3Ogml06a/K0=;BdP020?ooo`030?Ogm`0V9RH0d];B01/0oooo00<0gmoO01/K6`3Bd]802`3oool0
0`3Ogml06a/K0=;BdP040?ooo`030<K6aP1PH600oooo00/0oooo00<0gmoO01/K6`3Bd]80BP3oool0
0`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3oool03@3oool0
5@2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY@P3oool00`3Ogml04Q8B000000020000000<00T92@3Bd]80N7Qh000000092@T0b/[:01dM
7@000000@D510?Ogm`3oool0a/K60P0000001011@D40moOg0?ooo`36a/H3000000040451@@3gmoL0
oooo0::RXP80000000D0TY:B024Q8@3_knl0oooo0::RXP020000000:0451@@3gmoL0oooo0?_kn`1D
E5@0000000l?3`2BTY800000061PH080oooo00L0a/K6000000000000@D510?Ogm`3oool0a/K600<0
000000L0@D510?Ogm`3oool0oooo0=oOg`0K6a/0d];B0080oooo00<0gmoO04i>CP1:BTX00P000000
0`2d];@0oooo0<K6aP050000000K0;Bd]01hN7P0000000T92@3:b/X07AdM00000011@D40moOg0?oo
o`3Ogml06a/K0=;BdP3oool0no_k05ADE00000002@T90<[:bP0M7Ad000000451@@3gmoL0oooo0=oO
g`0K6a/0d];B00<0oooo00<0N7Qh061PH03oool01@3oool03P1hN7P0000000000011@D40moOg0?oo
o`3Ogml04Q8B000000092@T0d];B0?ooo`3Ogml04Q8B0`0000005@2d];@0oooo0?ooo`3oool0gmoO
01/K6`3Bd]80oooo0?_kn`1DE5@0000000T92@3:b/X07AdM00000011@D40moOg0?ooo`3Ogml06a/K
0=;BdP040?ooo`0A0=oOg`0Q8B40kno_0?ooo`3kno/0E5AD000000092@T0b/[:01dM7@000000@D51
0?Ogm`3oool0gmoO01/K6`3Bd]80BP3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo
0080oooo00<0k>c/09JFUP3GemL0303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool00`3Ogml06a/K0=;BdP020?oo
o`0P0?_kn`1DE5@0];Bd0?ooo`3Ogml08B4Q0>o_k`3oool0no_k05ADE02d];@0oooo07QhN02?Shl0
a/K60000002d];@0oooo0?Ogm`0M7Ad0];Bd0<K6aP092@T0d];B0?ooo`1hN7P03`l?0>o_k`3oool0
no_k05ADE02d];@30?ooo`060<K6aP11@D40moOg0?Ogm`0M7Ad0H61P0P3oool03@3kno/0E5AD0;Bd
]03oool0N7Qh08n?S`36a/H000000;Bd]03oool0moOg01dM7@2d];@00P3oool00`3Ogml06a/K0=;B
dP020?ooo`070=oOg`0K6a/0d];B0?ooo`3Ogml06a/K0=;BdP020?ooo`030=oOg`0K6a/0d];B0080
oooo00L0no_k05ADE02d];@0oooo0=oOg`0Q8B40kno_0080oooo00<0gmoO01/K6`3Bd]800`3oool0
1@2RXZ80@D510??cl`1DE5@0Shn?00<0oooo00<0gmoO01/K6`3Bd]800`3oool00`3Ogml06a/K0=;B
dP030?ooo`0E0?_kn`1DE5@0@D510?Ogm`3oool0N7Qh08n?S`3Ogml04Q8B0;Bd]03oool0a/K600T9
2@3Bd]80gmoO024Q8@3_knl0oooo0=oOg`0K6a/0d];B0080oooo00<0gmoO01/K6`3Bd]800`3oool0
1@2RXZ80@D510??cl`1DE5@0Shn?00<0oooo00<0gmoO01/K6`3Bd]80103oool00`3kno/0E5AD08n?
S`030?ooo`050::RXP11@D40lo?c05ADE02?Shl00`3oool00`3Ogml06a/K0=;BdP1:0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0f][J09:BTP2f][H:0;Fe]@0G
0;Bd]02?Shl0f][J06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJD@0oooo00<0gmoO01/K6`3Bd]800P3oool02`3kno/0E5AD0;Bd]03oool0gmoO
024Q8@3_knl0oooo0?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL0103oool02`1hN7P0Shn?
0?ooo`3oool0gmoO024Q8@3_knl0oooo0?_kn`1DE5@0];Bd00<0oooo00H0a/K600T92@3Bd]80oooo
0::RXP1PH6020?ooo`030?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL01P3oool00`3Ogml0
6a/K0=;BdP060?ooo`030?Ogm`0M7Ad0];Bd0080oooo00<0gmoO01/K6`3Bd]800P3oool01`3kno/0
E5AD0;Bd]03oool0gmoO024Q8@3_knl00P3oool00`3Ogml06a/K0=;BdP040?ooo`0407QhN018B4P0
@D510?Ogm`<0oooo00<0gmoO01/K6`3Bd]80103oool00`1hN7P0H61P0?ooo`020?ooo`0307QhN011
@D40moOg00@0oooo00<0XZ:R061PH03oool00P3oool02@1hN7P0Shn?0?ooo`3oool0moOg01dM7@00
00003`l?0>o_k`020?ooo`030=oOg`0K6a/0d];B00@0oooo00@0N7Qh04Q8B011@D40moOg0`3oool0
0`3Ogml06a/K0=;BdP0;0?ooo`0407QhN018B4P0@D510?Ogm`<0oooo00<0gmoO01/K6`3Bd]80BP3o
ool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_
0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00<0
gmoO01/K6`3Bd]800P3oool02`3kno/0E5AD08n?S`3oool0gmoO024Q8@3_knl0oooo0?Ogm`0V9RH0
d];B00<0oooo00<0N7Qh0000000000000`0000000`2?Shl0N7Qh08n?S`020?ooo`070=oOg`0Q8B40
kno_0?ooo`3kno/0E5AD061PH0040?ooo`050::RXP00000000000000001PH6000P3oool00`3gmoL0
9RHV0=;BdP030?ooo`0307QhN0000000000000<0000000H0Shn?0?ooo`3oool0gmoO01/K6`3Bd]84
0?ooo`050=oOg`0B4Q80000000l?3`3_knl00P3oool00`3Ogml06a/K0=;BdP020?ooo`070?_kn`1D
E5@0Shn?0?ooo`3Ogml08B4Q0>o_k`020?ooo`030=oOg`0K6a/0d];B00@0oooo00<0XZ:R0000002?
Shl0103oool00`3Ogml06a/K0=;BdP040?ooo`030=oOg`0K6a/0d];B0080oooo00<0N7Qh061PH03o
ool0103oool02@2RXZ80H61P0?ooo`3oool0no_k05ADE02?Shl0no_k05ADE0020000000300l?3`3_
knl0oooo0080oooo00<0gmoO01/K6`3Bd]80103oool00`2RXZ80000008n?S`040?ooo`030=oOg`0K
6a/0d];B00/0oooo00<0XZ:R0000002?Shl0103oool00`3Ogml06a/K0=;BdP1:0?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool00`3Ogml06a/K0=;B
dP020?ooo`0;0?_kn`1DE5@0De=C0>o_k`36a/H0@D510?Ogm`3oool0moOg02HV9P3Bd]800`3oool0
5@36a/H02@T90=;BdP3oool0a/K600T92@3Bd]80a/K600T92@3Bd]80oooo07QhN00?3`l0kno_0?oo
o`3kno/0E5AD0000001PH600a/K6061PH0040?ooo`070::RXP1PH600oooo0?ooo`3gmoL09RHV0=;B
dP030?ooo`070<K6aP092@T0d];B0?ooo`36a/H02@T90=;BdP020?ooo`030=oOg`0K6a/0d];B00<0
oooo00@0N7Qh000000000000Shn?103oool00`3Ogml06a/K0=;BdP020?ooo`070?_kn`1DE5@0De=C
0>o_k`36a/H0@D510?Ogm`020?ooo`030=oOg`0K6a/0d];B00<0oooo00D0a/K600T92@3:b/X07AdM
0;Bd]0030?ooo`030=oOg`0K6a/0d];B00D0oooo00H0N7Qh061PH03oool0oooo07QhN01PH6050?oo
o`0>0=oOg`0B4Q80];Bd0?ooo`36a/H02@T90=;BdP3kno/0E5AD0;Bd]03oool0a/K60451@@3gmoL2
0?ooo`030=oOg`0K6a/0d];B00<0oooo00D0a/K600T92@3:b/X07AdM0;Bd]0030?ooo`030=oOg`0K
6a/0d];B00X0oooo00D0a/K600T92@3:b/X07AdM0;Bd]0030?ooo`030=oOg`0K6a/0d];B04X0oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU40?ooo`030=oO
g`0K6a/0d];B0080oooo00T0N7Qh0000002/[:`07AdM00T92@3Bd]80oooo0=oOg`0B4Q800`000000
1011@D40moOg0?ooo`36a/H3000000040;Bd]03oool0oooo0::RXP80000000`0TY:B018B4P0?3`l0
clo?018B4P000000[:b/01dM7@000000];Bd0?_kn`1DE5@20000000500l?3`3_knl0oooo0=oOg`0B
4Q800`0000001011@D40moOg0?ooo`36a/H3000000070;Bd]03oool0oooo0?ooo`3Ogml06a/K0=;B
dP020?ooo`030?_kn`1DE5@0];Bd00@0oooo00D0no_k05ADE00000002@T90=;BdP020?ooo`0607Qh
N0000000[:b/01dM7@092@T0d];B0`3oool00`3Ogml06a/K0=;BdP020?ooo`070::RXP0000002@T9
0;:b/P0B4Q80000008n?S`020?ooo`030=oOg`0K6a/0d];B00D0oooo00L0gmoO01/K6`3Bd]80oooo
07QhN011@D40moOg0080oooo00P0N7Qh0;Bd]03oool0gmoO018B4P0000002@T90=;BdP80oooo00H0
gmoO018B4P00000000000451@@3gmoL20?ooo`030=oOg`0K6a/0d];B0080oooo00L0XZ:R00000009
2@T0/[:b018B4P000000Shn?0080oooo00<0gmoO01/K6`3Bd]802@3oool01`2RXZ80000000T92@2b
/[804Q8B0000002?Shl00P3oool00`3Ogml06a/K0=;BdP1:0?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool00`3Ogml06a/K0=;BdP0:0?ooo`030?Og
m`0V9RH0d];B0200oooo00<0moOg02HV9P3Bd]80303oool00`3Ogml06a/K0=;BdP020?ooo`070?_k
n`1DE5@0Shn?0?ooo`2RXZ803`l?0>o_k`0@0?ooo`030=oOg`0K6a/0d];B00/0oooo00<0gmoO01/K
6`3Bd]801P3oool02P1hN7P0H61P0?ooo`3kno/0E5AD0451@@3gmoL0gmoO018B4P2d];@@0?ooo`03
0=oOg`0K6a/0d];B00/0oooo00<0gmoO01/K6`3Bd]804P3oool00`3Ogml06a/K0=;BdP1:0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY@P3oool00`3Ogml0
4Q8B000000020000000300T92@3Bd]80oooo00L0oooo00<0moOg02HV9P3Bd]80803oool00`3gmoL0
9RHV0=;BdP0<0?ooo`040=oOg`0B4Q800000061PH080oooo00H0moOg01dM7@000000QhN702`/;03_
knl20?ooo`030::RXP0?3`l0kno_00/0oooo00@0gmoO018B4P000000H61P2@3oool0101hN7P00000
00T92@3Bd]860?ooo`030=oOg`0Q8B40kno_0080oooo00D0N7Qh00000011@D40L71`0;Bd]00@0?oo
o`040=oOg`0B4Q800000061PH0T0oooo00@0N7Qh000000092@T0d];B4@3oool0101hN7P0000000T9
2@3Bd]9:0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolm0?ooo`D0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo?@3oool505EEE@H0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o03oool01@3kno/0E5AD00000011@D40moOg03l0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWm0?ooo`030?Ogm`0/;2`0kno_0400oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWm0?ooo`030?Ogm`0/;2`0kno_0400
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTn0?ooo`05
07QhN000000000000451@@3gmoL00P3oool03`1hN7P0000000l?3`3_knl0XZ:R00000011@D40VYZJ
00000011@D40`l?303lo?`0000003`l?0>o_k`030?ooo`0=0::RXP000000000000000011@D40L71`
0000001PH600XZ:R00000011@D40moOg07QhN0030000000<0;Bd]03oool0oooo0?ooo`1hN7P00000
061PH02RXZ8000000451@@3gmoL0XZ:R0P0000003@2d];@0oooo0?ooo`1hN7P000000;Bd]03oool0
N7Qh0000000?3`l0kno_0?_kn`1DE5@00P000000102d];@0oooo0?ooo`2RXZ82000000040;Bd]03o
ool0oooo0::RXP80000000d0];Bd0?ooo`1hN7P00000061PH036a/H0000000l?3`2k^k/0?clo0000
000?3`l0kno_00<0oooo00h0XZ:R00000000000000000451@@1`L70000000451@@3gmoL0oooo07Qh
N00000003`l?0>o_k`80oooo00X0no_k05ADE00000002@T90=;BdP3oool0N7Qh0;Bd]03kno/0E5AD
0P0000000`11@D40VYZJ000000020000000J0451@@3gmoL0N7Qh0000002d];@0N7Qh0000001PH600
XZ:R00000011@D40moOg07QhN00000003`l?0>o_k`1hN7P00000061PH01hN7P000000451@@2JVYX0
00000451@@2JVYX30000000>0451@@3gmoL0N7Qh0000002d];@0XZ:R00000011@D40lo?c05ADE000
00002@T90=;BdP2RXZ83000000030451@@3gmoL0oooo0080oooo00/0N7Qh0000000?3`l0jn_[05AD
E01PH600gmoO018B4P11@D40kno_01dM7@020000000K00T92@3Bd]80oooo07QhN0000000@D510?Og
m`3oool0N7Qh0000000?3`l0kno_0<_;b`0o?cl0000000l?3`3_knl0bl_;03lo?`0000003`l?0>o_
k`2RXZ8000000451@@3gmoL0XZ:R008000000100];Bd0?ooo`1hN7P00000061PH02RXZ8000000451
@@33`l<0?clo0000000?3`l0kno_0=oOg`0K6a/0d];B4P3oool00`1EEED0oooo0?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo01L0N7Qh0451@@3gmoL0oooo0=oOg`1CDe<0
moOg0<K6aP1PH600oooo0?Ogm`1mOGd0no_k05eMG@3Bd]80no_k05eMG@3Bd]80bl_;0=;BdP3oool0
a/K608n?S`030?ooo`0B07QhN02d];@0no_k05eMG@3Bd]80moOg02`/;03_knl0no_k05eMG@3Bd]80
a/K600l?3`3_knl0no_k05ADE00?3`l0kno_0P3oool05`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3O
gml06a/K0=;BdP3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP3oool0a/K6061PH03oool0moOg07emO@3O
gml0De=C0080moOg01T09RHV0=;BdP3Ogml06a/K0=;BdP3kno/0GEeM0=;BdP3Ogml06a/K0=;BdP3k
no/0GEeM0=;BdP3gmoL0;2`/0>o_k`1hN7P0Shn?0?ooo`3;bl/0d];B0?ooo`36a/H0Shn?00<0oooo
00h0N7Qh0;Bd]03kno/0GEeM0=;BdP3gmoL0;2`/0>o_k`3oool0a/K6061PH03oool0moOg07emO@@0
oooo03h0[Jf]0>o_k`3oool0gmoO01/K6`1^KVh0moOg0?ooo`2RXZ80Shn?0?ooo`1hN7P0];Bd0?_k
n`1MGEd0d];B0?_kn`1MGEd0d];B0?ooo`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP36a/H0H61P0?oo
o`3gmoL0OGem0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0];Bd07QhN02d];@0no_k05eM
G@3Bd]80no_k05eMG@3Bd]80oooo0?_kn`1MGEd0d];B0::RXP1PH600oooo0?Ogm`2TY:@0N7Qh0;Bd
]03kno/0GEeM0=;BdP<0oooo03P0a/K6061PH03oool0moOg07emO@3Ogml0LW9b0?_kn`1MGEd0d];B
0?Ogm`0V9RH0d];B0?ooo`3gmoL09RHV0=;BdP3gmoL0;2`/0>o_k`3oool0a/K6061PH03oool0moOg
07emO@3;bl/0d];B0?ooo`36a/H0Shn?0<_;b`3Bd]80oooo0<K6aP2?Shl0no_k05eMG@3Bd]80gmoO
01/K6`3Bd]80no_k05eMG@3Bd]80moOg02`/;03_knl0no_k05eMG@3Bd]80bl_;0=;BdP3oool0a/K6
08n?SaD0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030=oOg`0K6a/0d];B00<0oooo00@0a/K60;Bd]02RXZ80];Bd0`3oool01P3kno/0GEeM0=;B
dP3kno/0GEeM0=;BdP80oooo00<0XZ:R0000002d];@00`3oool04@36a/H03`l?0>_[j`1MGEd0d];B
0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP2?Shl0oooo0?_kn`1MGEd0d];B00<0oooo00P0moOg
02`/;03_knl0no_k05eMG@3Bd]80XZ:R061PH080oooo00P0XZ:R08n?S`3kno/0GEeM0=;BdP3oool0
XZ:R0;Bd]0<0oooo00P0gmoO05=CD`3gmoL0oooo0::RXP2?Shl0XZ:R061PH080oooo00@0XZ:R08n?
S`2RXZ80H61P0P3oool01`2RXZ80Shn?0?Ogm`0V9RH0UiNG0451@@3gmoL00`3oool00`2RXZ800000
0;Bd]0030?ooo`0;0<K6aP0?3`l0jn_[05eMG@3Bd]80moOg02`/;03_knl0oooo0::RXP2d];@01`3o
ool0702][Jd0kno_0?ooo`1hN7P0Shn?0<K6aP2d];@0oooo0::RXP2?Shl0oooo0<K6aP0?3`l0jn_[
05eMG@3Bd]80no_k05eMG@3Bd]80oooo0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP2d];@30?oo
o`0G0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0];Bd0<K6aP0?3`l0jn_[05eMG@3Bd]80
no_k05eMG@3Bd]80oooo0?_kn`1MGEd0d];B07QhN02d];@00`3oool01@36a/H03`l?0>_[j`1MGEd0
d];B00<0oooo00<0XZ:R0;Bd]03oool00`3oool01`3;bl/0?clo08n?S`3oool0moOg02`/;03_knl0
0P3oool0202RXZ80Shn?0?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd1@3oool00`2RXZ8000000;Bd]002
0?ooo`080::RXP000000];Bd0?_kn`1MGEd0d];B0::RXP1PH6020?ooo`080::RXP2?Shl0moOg02`/
;03_knl0no_k05eMG@3Bd]820?ooo`030::RXP000000];Bd01D0oooo00<0EEEE0?ooo`3oool0103o
ool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg
0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030::RXP11@D40moOg00D0oooo00l0
XZ:R0000000000000000061PH03kno/0GEeM0=;BdP3kno/0GEeM0=;BdP3Ogml04Q8B0451@@3gmoL0
1@3oool0403gmoL07AdM00T92@3Bd]80moOg02`/;03_knl0no_k05eMG@3Bd]80XZ:R08n?S`3oool0
no_k05eMG@3Bd]830?ooo`080?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP2?Shl20?ooo`0707Qh
N02?Shl0no_k05eMG@3Bd]80oooo0::RXP0300000009061PH03Ogml0De=C0?Ogm`3oool0XZ:R08n?
S`2RXZ80Shn?0080oooo00@0N7Qh08n?S`2RXZ80Shn?0P3oool01P1hN7P0Shn?0?Ogm`0V9RH0?clo
061PH080oooo00@0gmoO018B4P11@D40moOg1@3oool02@3gmoL07AdM00T92@3Bd]80moOg02`/;03_
knl0oooo0::RXP0300000003061PH03oool0oooo0080oooo00X0[Jf]0>o_k`3gmoL0;2`/0>o_k`3k
no/0X:2P0?ooo`2RXZ80Shn?0P3oool03`3gmoL07AdM00T92@3Bd]80no_k05eMG@3Bd]80oooo0?Og
m`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP030000000H061PH03gmoL0;2`/0>o_k`3kno/0GEeM0=;B
dP3kno/0E5AD0;Bd]03oool0moOg01dM7@092@T0d];B0?_kn`1MGEd0d];B0?ooo`3kno/0GEeM0=;B
dP21PH40d];B103oool0103gmoL07AdM00T92@3Bd]830?ooo`0=0::RXP00000000000000001PH600
oooo0<K6aP0?3`l0kno_0?ooo`3gmoL0;2`/0>o_k`020?ooo`070::RXP2?Shl0moOg02`/;03_knl0
oooo0::RXP030000000@061PH03Ogml04Q8B0451@@3gmoL0oooo0=oOg`0B4Q80@D510?Ogm`3oool0
no_k05eMG@3Bd]80XZ:R08n?S`80oooo00`0N7Qh08n?S`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3O
gml04Q8B0451@@3gmoLF0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY?03oool00`2RXZ80@D510?Ogm`050?ooo`0@0<K6aP2?Shl0oooo07QhN02?Shl0no_k
05eMG@3Bd]80no_k05eMG@3Bd]80a/K608n?S`3oool0PH610=;BdP<0oooo0140XZ:R08n?S`3kno/0
GEeM0<k>cP1DE5@00000061PH03gmoL09RHV0=;BdP3Ogml08B4Q0>o_k`3Ogml06a/K0=;BdP020?oo
o`0c0?_kn`1DE5@00000061PH03gmoL09RHV0=;BdP3Ogml0De=C0?Ogm`3Ogml04Q8B0;Bd]03Kfm/0
GEeM0=;BdP3oool0a/K608n?S`3oool0N7Qh08n?S`3Ogml04Q8B0;Bd]03kno/0E5AD0;Bd]03Ogml0
De=C0?Ogm`3Ogml04Q8B0;Bd]03Ogml0De=C0?Ogm`3Ogml04Q8B0;Bd]03gmoL0;2`/0>o_k`3;bl/0
d];B0?ooo`36a/H0Shn?0?ooo`21PH40d];B00<0oooo00h0XZ:R08n?S`3kno/0GEeM0<k>cP1DE5@0
000008n?S`3oool0a/K608n?S`3oool0N7Qh08n?S`@0oooo00@0[Jf]0>o_k`2RXZ80Shn?0P3oool0
>02XZ:P0d];B0::RXP2?Shl0oooo0::RXP2?Shl0no_k05eMG@3Bd]80fm_K05eMG@3Bd]80oooo0?Og
m`0M7Ad0H61P0?Ogm`0V9RH0d];B0<K6aP2?Shl0oooo07QhN02;Rh/0E5AD0000002?Shl0moOg01dM
7@1PH600no_k05eMG@3Bd]80XZ:R08n?S`3kno/0GEeM0=;BdP3Kfm/0GEeM0=;BdP3oool0N7Qh00T9
2@3Bd]80XZ:R08n?S`3oool0N7Qh0;Bd]02RXZ80Shn?0?_kn`1MGEd0d];B0`3oool0>036a/H0Shn?
0?ooo`1hN7P0Shn?0?Ogm`0/;2`0inOW0:BTY03kno/0E5AD0000001PH600oooo0=oOg`0B4Q80/;2`
05ADE0000000Shn?0?ooo`36a/H0Shn?0?ooo`1hN7P0Shn?0<K6aP2?Shl0oooo0861P@3Bd]80a/K6
08n?S`3oool0PH610=;BdP1hN7P02@T90=;BdP3Ogml0De=C0?Ogm`3Ogml04Q8B0;2`/01DE5@00000
061PH03gmoL09RHV0=;BdP36a/H0Shn?0?ooo`21PH40d];B5@3oool00`1EEED0oooo0?ooo`040?oo
o`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<0XZ:R0451@@3gmoL01P3oool03`1h
N7P000000451@@3gmoL0no_k05eMG@3Bd]80no_k05eMG@3Bd]80no_k05ADE0000000UYJF0=_Kf`03
0?ooo`040=oOg`0B4Q8000000451@@80moOg00L0;2`/0;Jf]P000000@D510?Ogm`3oool0a/K60080
000000<02@T90=;BdP3oool00P3oool0203gmoL0;2`/0;Jf]P000000@D510?Ogm`3oool0XZ:R0P00
00000`2?Shl0oooo0<K6aP020000000;0;Bd]03oool0N7Qh00000011@D40moOg0=oOg`1CDe<0L71`
0000001PH6000P3oool0102RXZ8000000000002?Shl20?ooo`0@0::RXP000000000008n?S`3oool0
moOg02`/;03_knl0XZ:R00000011@D40lo?c05ADE0000000UYJF0=_Kf`<0oooo00@0gmoO018B4P00
0000@D510P3gmoL0200/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg103oool0102][Jd0inOW
02`/;03_knl20?ooo`0;0?Ogm`1mOGd0XZ:R08n?S`3oool0gmoO018B4P000000@D510?Ogm`36a/H0
0P0000006`2d];@0moOg02`/;02f][H000000451@@3gmoL0oooo07QhN0000000@D510?Ogm`3Ogml0
De=C09ZJVP000000@D510;jn_P000000@D510?Ogm`3Ogml04Q8B00000011@D40moOg0<K6aP020000
000>0;Bd]03gmoL09RHV0=;BdP3oool0N7Qh0000000?3`l0kno_0=oOg`0B4Q8000000451@@3gmoL4
0?ooo`0<07QhN0000000@D510??cl`1DE5@0000009:BTP0B4Q80Shn?0?Ogm`0/;2`0][Jf0P000000
6P2?Shl0oooo0?Ogm`0/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg0?_kn`1DE5@0000009JF
UP3Kfm/0no_k05ADE0000000UYJF0=_Kf`3gmoL09RHV0=;BdP3oool0XZ:R0P0000003@2?Shl0oooo
0?Ogm`0/;2`0][Jf00000011@D40moOg0?_kn`1DE5@0000009JFUP3Kfm/05@3oool00`1EEED0oooo
0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd
0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<0gmoO01/K6`3Bd]80
0`3oool00`2XZ:P0d];B0?ooo`040?ooo`060?_kn`1MGEd0d];B0?_kn`1MGEd0d];B5P3oool00`3k
no/0GEeM0=;BdP0@0?ooo`030861P@3Bd]80oooo00D0oooo00<0gmoO05=CD`3gmoL03`3oool00`3g
moL0;2`/0>o_k`0M0?ooo`030:f][@2BTY80H61P00@0oooo00<0[Jf]09:BTP2?Shl01`3oool01P21
PH40d];B0?ooo`3gmoL0;2`/0>o_kaL0oooo00<0PH610=;BdP3oool0G@3oool00`1EEED0oooo0?oo
o`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo00L0N7Qh0451@@3gmoL0oooo
0::RXP092@T0d];B00D0oooo00H0N7Qh00T92@3Bd]80N7Qh00T92@3Bd]8F0?ooo`0307QhN0092@T0
d];B0100oooo00<0Z:RX0=;BdP3oool0103oool0103gmoL07AdM0451@@3gmoL>0?ooo`040?_kn`1D
E5@03`l?0>o_kad0oooo00<0XZ:R00T92@3Bd]80103oool00`3Ogml04Q8B08n?S`070?ooo`060:RX
Z03Bd]80no_k05ADE00?3`l0kno_5`3oool00`2XZ:P0d];B0?ooo`1M0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?P3oool01P2RXZ8000000000002?Shl0
Z:RX0=;BdPD0oooo00H0moOg02HV9P3Bd]80moOg02HV9P3Bd]8F0?ooo`030?_kn`1MGEd0d];B01P0
oooo00<0gmoO05=CD`3gmoL03`3oool00`3Ogml08B4Q0>o_k`0K0?ooo`040?_kn`1DE5@00000061P
H0H0oooo00@0N7Qh00000011@D40moOg203oool00`3Ogml08B4Q0>o_k`0J0?ooo`030?Ogm`0V9RH0
d];B0340oooo00<0moOg02HV9P3Bd]809P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo?P3oool405EEE@H0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolm0?ooo`D0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0l?3`08j>SP050:2PX00309nOW`2PX:00
X:2P00@0X:2P01H0T92@0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0
g]kN00`0dm?C01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
odP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S
0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0
emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocd0oooo1@1EEED60?ooo`03
05EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJBH0oooo00D0gmoO018B4P0000003`l?0>o_k`3o0?oooaH0oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTV0?ooo`050=oOg`0B
4Q8000000000001PH600o`3ooolF0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY:03oool0103Ogml04Q8B00T92@3Bd];o0?oooaD0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTN0?ooo`030<K6aP00000000000080000000<0
Shn?0?ooo`3oool00P3oool0103gmoL07AdM0000002?Shl30?ooo`0C0=oOg`0B4Q802@T90=;BdP3o
ool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_knl0no_k05ADE0000000[:b/01dM7@02
0000000308n?S`3oool0oooo0080oooo00H0moOg01dM7@000000000000T92@3Bd]820?ooo`080?Og
m`0M7Ad000000;Bd]03gmoL07AdM0000002d];@20?ooo`060<K6aP000000000000000011@D40moOg
1P3oool0302RXZ80000000000011@D40kno_01dM7@092@T0d];B0?ooo`1hN7P00000061PH080oooo
00<0XZ:R0000001PH6000`3oool01`2RXZ80000000000011@D40VYZJ0000001PH6001@3oool0203O
gml04Q8B00T92@3Bd]80oooo07QhN0000000H61P0P3oool00`2RXZ800000061PH0020?ooo`070=oO
g`0B4Q80000000T92@3Bd]80oooo07QhN0030000000708n?S`3oool0oooo0?Ogm`0M7Ad02@T90=;B
dP030?ooo`080::RXP00000000000451@@3_knl07AdM00T92@3Bd]820?ooo`060<K6aP0000000000
00000011@D40moOg0P3oool0203Ogml04Q8B000000092@T0b/[:01dM7@000000];Bd0`3oool00`1h
N7P00000061PH0050?ooo`0507QhN000000000000000002?Shl01P3oool00`3gmoL07AdM00000005
0000000308n?S`3oool0oooo00<0oooo00H0moOg01dM7@000000000000T92@3Bd]840?ooo`060<K6
aP000000000000000011@D40moOg0`3oool05P1hN7P0000000000011@D40VYZJ0000001PH600oooo
0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_knl20?oo
o`0507QhN000000000000000002?Shl00`3oool00`1hN7P00000061PH0020?ooo`030::RXP000000
H61P0080oooo00L0gmoO018B4P0000002@T90=;BdP3kno/0E5AD00<0000000<0@D510?Ogm`3oool0
3P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAd0oooo00<0
XZ:R000000000000100000001@2?Shl0oooo0?ooo`3oool0a/K60080000001@0@D510?Ogm`3oool0
oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_knl0
no_k05ADE0H0000000@0Shn?0?ooo`3oool0a/K61P000000302d];@0oooo0?Ogm`0M7Ad000000;Bd
]03gmoL07AdM0000002d];@0oooo0<K6aPD0000000<03`l?0>o_k`3oool00`3oool00`36a/H00000
00000004000000060451@@3gmoL0oooo07QhN0000000H61P0P3oool00`2RXZ800000061PH0020?oo
o`030::RXP000000000000@0000000<0H61P0?ooo`3oool00`3oool0203Ogml04Q8B00T92@3Bd]80
oooo07QhN0000000H61P0P3oool00`2RXZ800000061PH0020?ooo`0607QhN0000000000000T92@3B
d]80N7Qh1@0000001P2?Shl0oooo0?Ogm`0M7Ad02@T90=;BdP80oooo00<0a/K60000000000001000
00001011@D40moOg0?ooo`36a/H50000000400l?3`3_knl0oooo07QhN080000000D02@T90<[:bP0M
7Ad000000;Bd]0020?ooo`050?Ogm`0M7Ad0000000l?3`3_knl00`3oool00`1hN7P0000000000003
0000000308n?S`3oool0oooo00<0oooo00<0moOg01dM7@0000001P0000001@11@D40moOg0?ooo`3o
ool0a/K600H0000000@0];Bd0?ooo`3oool0a/K61@000000100?3`l0kno_0?ooo`36a/H60000000B
061PH03oool0gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6aP0000003`l?
0>o_k`3oool0N7Qh1@0000001P2?Shl0oooo0?ooo`1hN7P00000061PH080oooo00<0XZ:R0000001P
H6000P3oool01P1hN7P00000000000092@T0c/k>05ADE0D0000000<0@D510?Ogm`3oool03@3oool0
0`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo00@0moOg01dM
7@000000Shn?0P3oool0102RXZ80000000l?3`3_knl20?ooo`0507QhN0000000000000T92@3Bd]80
0P3oool08@3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K60000000?3`l0
kno_0?_kn`1DE5@0000000l?3`3_knl0oooo0::RXP0000003`l?0>o_k`3kno/0E5AD00000011@D40
moOg0?ooo`1hN7P000000451@@020?Ogm`0@01dM7@000000];Bd0?Ogm`0M7Ad000000;Bd]03gmoL0
7AdM0000002?Shl0oooo0?_kn`1DE5@0000008n?S`@0oooo00d0XZ:R00000011@D40moOg0?_kn`1D
E5@000000451@@3gmoL0oooo07QhN0000000H61P0080oooo00d0XZ:R0000001PH600oooo0?Ogm`0M
7Ad00000061PH03oool0moOg01dM7@000000H61P00D0oooo00P0gmoO018B4P092@T0d];B0?ooo`1h
N7P00000061PH080oooo01H0XZ:R0000001PH600oooo0?_kn`1DE5@0000008n?S`3oool0gmoO018B
4P000000];Bd0?ooo`36a/H0000000l?3`3_knl0moOg01dM7@092@T0d];B0P3oool0202RXZ800000
0451@@3gmoL0no_k05ADE0000000@D510P3gmoL04@0M7Ad0000008n?S`3oool0no_k05ADE0000000
Shn?0?_kn`1DE5@0000008n?S`3oool0moOg01dM7@000000];Bd0080oooo00D0XZ:R0000001dM7@0
00000;Bd]0020?ooo`090=oOg`0B4Q8000000;Bd]03oool0a/K60000000?3`l0kno_00@0oooo00@0
moOg01dM7@000000];Bd0`3oool03P3Ogml04Q8B0000002?Shl0oooo0?_kn`1DE5@000000451@@3g
moL0oooo07QhN0000000@D510P3gmoL0300M7Ad0000008n?S`3oool0no_k05ADE0000000Shn?0?oo
o`1hN7P000000451@@80moOg01l07AdM0000001PH600oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO
018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_knl0gmoO018B4P000000];Bd0?ooo`36a/H00000
00l?3`3_knl0oooo07QhN0000000H61P0080oooo0180XZ:R0000001PH600oooo0?_kn`1DE5@00000
08n?S`3oool0a/K6000000092@T0d];B0?ooo`3gmoL07AdM00T92@3Bd]8>0?ooo`0305EEE@3oool0
oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0
i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703oool01036a/H0000000l?3`3_knl2
0?ooo`0;0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0][Jf0000001PH6000P3oool04P3O
gml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K60000000?3`l0kno_0?_kn`1D
E5@0000008n?S`80oooo00P0moOg01dM7@092@T0d];B0=oOg`0B4Q802@T90=;BdP80oooo0100moOg
01dM7@092@T0d];B0?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@0gmoO018B4P092@T0d];B
2@3oool03@36a/H000000000002d];@0oooo0::RXP000000@D510?Ogm`3oool0N7Qh0000001PH600
0P3oool0202RXZ800000061PH03oool0gmoO018B4P092@T0d];B0P3oool00`2RXZ800000061PH005
0?ooo`080=oOg`0B4Q802@T90=;BdP3oool0N7Qh0000001PH6020?ooo`0=0::RXP000000H61P0?oo
o`3kno/0E5AD0000002d];@0oooo0::RXP000000@D510?Ogm`050?ooo`040?Ogm`0M7Ad02@T90=;B
dP80oooo00d0a/K6000000000000];Bd0?ooo`2RXZ8000000451@@3gmoL0gmoO018B4P092@T0d];B
00D0oooo01H0no_k05ADE0000000];Bd0?ooo`3gmoL07AdM0000002d];@0oooo0?_kn`1DE5@03`l?
0<o?c`0B4Q80@D510?Ogm`3oool0XZ:R00000011@D40moOg2@3oool0103gmoL07AdM0000002d];@4
0?ooo`080<K6aP0000003`l?0>o_k`3Ogml04Q8B00T92@3Bd]820?ooo`080?Ogm`0M7Ad02@T90=;B
dP3Ogml04Q8B00T92@3Bd]860?ooo`0307QhN0000000H61P0080oooo01H0N7Qh0000001PH600oooo
0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_knl0XZ:R
00000011@D40moOg1P3oool00`1hN7P00000061PH0020?ooo`080::RXP000000H61P0?ooo`3kno/0
E5AD0000002d];@40?ooo`060<K6aP0000000000000000092@T0d];B3P3oool00`1EEED0oooo0?oo
o`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CT
i03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJB40oooo0200moOg01dM7@0000003`l?0>o_
k`3oool0XZ:R0000001PH600moOg01dM7@0?3`l0kno_0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oO
g`0B4Q802@T90=;BdP3oool0a/K60000000?3`l0kno_0?_kn`1DE5@000000;Bd]080oooo00P0moOg
01dM7@000000];Bd0=oOg`0B4Q803`l?0>o_k`80oooo0100moOg01dM7@000000];Bd0?Ogm`0M7Ad0
00000;Bd]03gmoL07AdM0000002d];@0gmoO018B4P0?3`l0kno_2@3oool00`3kno/0E5AD00000004
000000060451@@3gmoL0oooo07QhN0000000H61P0P3oool0202RXZ800000061PH03oool0gmoO018B
4P0?3`l0kno_0P3oool00`2RXZ800000061PH0050?ooo`080=oOg`0B4Q802@T90=;BdP3oool0N7Qh
0000001PH6020?ooo`0:0::RXP000000H61P0?ooo`3kno/0E5AD0000002d];@0oooo0::RXPL00000
00D0];Bd0?Ogm`0M7Ad000000;Bd]0020?ooo`030?_kn`1DE5@0000000@0000000H0@D510?Ogm`3O
gml04Q8B00l?3`3_knl50?ooo`0C0?_kn`1DE5@000000;Bd]03oool0moOg01dM7@000000];Bd0?oo
o`3Ogml04Q8B0451@@3clo<0E5AD00T92@3Bd]80oooo0::RXP07000000030;Bd]03oool0oooo0080
oooo00@0moOg01dM7@000000];Bd103oool0203Ogml04Q8B00T92@3Bd]80gmoO018B4P0?3`l0kno_
0P3oool0203gmoL07AdM0000002d];@0gmoO018B4P0?3`l0kno_1P3oool00`1hN7P00000061PH002
0?ooo`0C0::RXP000000H61P0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q802@T90=;BdP3o
ool0a/K60000000?3`l0kno_0::RXP07000000050;Bd]03oool0N7Qh0000001PH6000P3oool0202R
XZ800000061PH03oool0no_k05ADE0000000];Bd0P3oool00`2RXZ8000000000000300000003061P
H03oool0oooo00d0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTO0?ooo`030::RXP00000000000080000001d0Shn?0?ooo`3kno/0E5AD0000002d];@0oooo07Qh
N0000000];Bd0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q802@T90=;BdP3oool0a/K60000
000?3`l0kno_0?_kn`1DE5@000000;Bd]0020?ooo`080?Ogm`0M7Ad02@T90=;BdP3Ogml04Q8B00T9
2@3Bd]820?ooo`0@0?Ogm`0M7Ad02@T90=;BdP3gmoL07AdM0000002d];@0moOg01dM7@000000];Bd
0=oOg`0B4Q802@T90=;BdP`0oooo00X0gmoO018B4P00000000000451@@3gmoL0oooo07QhN0000000
H61P0P3oool0202RXZ800000061PH03oool0gmoO018B4P092@T0d];B0P3oool00`2RXZ800000061P
H0050?ooo`080=oOg`0B4Q802@T90=;BdP3oool0N7Qh0000001PH6020?ooo`0:0::RXP000000H61P
0?ooo`3kno/0E5AD0000002d];@0oooo0<K6aPH0000000H02@T90=;BdP3gmoL07AdM0000002d];@5
0?ooo`0:0=oOg`0B4Q80000000000011@D40moOg0=oOg`0B4Q802@T90=;BdPD0oooo01<0no_k05AD
E0000000];Bd0?ooo`3gmoL07AdM0000002d];@0oooo07QhN0000000Shn?0?ooo`2RXZ80000008n?
S`3oool0a/K600H0000000<02@T90=;BdP3oool00`3oool0103gmoL07AdM0000002d];@40?ooo`08
0?Ogm`0M7Ad02@T90=;BdP3Ogml04Q8B00T92@3Bd]820?ooo`080?Ogm`0M7Ad02@T90=;BdP3Ogml0
4Q8B00T92@3Bd]860?ooo`0307QhN0000000H61P0080oooo01<0XZ:R0000001PH600oooo0=oOg`0B
4Q802@T90=;BdP3oool0gmoO018B4P092@T0d];B0?ooo`36a/H0000000l?3`3_knl0a/K600H00000
00H02@T90=;BdP3oool0N7Qh0000001PH6020?ooo`0;0::RXP000000H61P0?ooo`3kno/0E5AD0000
002d];@0oooo0?Ogm`0M7Ad00`0000000`1PH600oooo0?ooo`0?0?ooo`0305EEE@3oool0oooo00@0
oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cL
g03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7@3oool00`3gmoL07AdM000000030000000B0;Bd
]03oool0oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K600000011@D40moOg0=oOg`0B4Q800000061P
H03oool0a/K60P000000:`2?Shl0oooo0::RXP0000003`l?0>o_k`3kno/0E5AD00000011@D40moOg
0?ooo`2RXZ80000000l?3`3_knl0no_k05ADE0000000@D510?Ogm`3oool0N7Qh0000000?3`l0kno_
0?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@0moOg01dM7@000000Shn?0?ooo`3gmoL07AdM
00T92@3Bd]80103oool0703gmoL07AdM00l?3`3_knl0oooo0::RXP000000@D510?Ogm`3oool0N7Qh
000000092@T0d];B0?_kn`1DE5@00000061PH03oool0moOg01dM7@000000H61P0?ooo`3gmoL07AdM
0000001PH6050?ooo`0P0=oOg`0B4Q802@T90=;BdP3oool0N7Qh000000092@T0d];B0?_kn`1DE5@0
0000061PH03oool0no_k05ADE0000000];Bd0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0=oOg`0B4Q80
3`l?0>o_k`3gmoL07AdM0000001PH6020?ooo`080?Ogm`0M7Ad03`l?0>o_k`3oool0XZ:R00000011
@D420?Ogm`0S01dM7@000000Shn?0?ooo`3gmoL07AdM00T92@3Bd]80no_k05ADE0000000];Bd0?oo
o`3gmoL07AdM0000002d];@0moOg01dM7@092@T0d];B0?ooo`3Ogml04Q8B0451@@3gmoL0gmoO018B
4P0?3`l0kno_0?ooo`3Ogml04Q8B00l?3`3_knl0103oool0103gmoL07AdM0000002d];@40?ooo`0K
0?Ogm`0M7Ad02@T90=;BdP3kno/0E5AD00000011@D40moOg0?ooo`1hN7P0000000l?3`3_knl0moOg
01dM7@000000Shn?0?ooo`3gmoL07AdM00T92@3Bd]80oooo07QhN0000000H61P0080oooo00X0XZ:R
0000001PH600oooo0=oOg`0B4Q800000061PH03oool0a/K60P0000009P2?Shl0oooo0::RXP000000
3`l?0>o_k`3Ogml04Q8B00l?3`3_knl0oooo0=oOg`0B4Q803`l?0>o_k`3oool0N7Qh000000092@T0
d];B0?_kn`1DE5@00000061PH03oool0no_k05ADE0000000];Bd0?ooo`3Ogml04Q8B00l?3`3_knl0
oooo0::RXP000000H61P3`3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJAd0oooo00D0N7Qh0000000000003`l?0>o_k`040?ooo`030::RXP000000H61P0080oooo
00H0moOg01dM7@092@T0d];B0=oOg`0B4Q8:00000004061PH03oool0no_k05ADE0H0000000D0Shn?
0?ooo`3oool0gmoO018B4P050000000<0;Bd]03oool0moOg01dM7@000000];Bd0?Ogm`0M7Ad00000
0;Bd]03oool0a/K61@0000000`11@D40moOg0?ooo`040?ooo`0307QhN0000000000000<0000000@0
H61P0?ooo`3oool0N7Qh1P000000102?Shl0oooo0?ooo`2RXZ8600000003061PH03oool0oooo00<0
oooo00H0gmoO018B4P092@T0d];B0?ooo`1hN7P60000000308n?S`3kno/0E5AD00<0000000<03`l?
0>o_k`1hN7P01@000000102?Shl0oooo0?Ogm`0M7Ad3000000030451@@3gmoL0N7Qh00D0000000@0
H61P0?ooo`3oool0a/K61@0000000`11@D40lo?c05ADE0030000000@00l?3`3WinL07AdM0000002d
];@0a/K600000011@D40moOg0?ooo`3kno/0E5AD0000002d];@0oooo07QhN0D0000000<0Shn?0?oo
o`3oool00`3oool0103gmoL07AdM0000002d];@40?ooo`070=oOg`0B4Q802@T90=;BdP3oool0gmoO
018B4P05000000040;Bd]03oool0oooo0<K6aPD0000000H0@D510?Ogm`3oool0N7Qh0000001PH602
0?ooo`060::RXP000000H61P0?ooo`3Ogml04Q8B2P000000101PH600oooo0?ooo`1hN7P500000004
08n?S`3oool0oooo07QhN0H0000000<0Shn?0?_kn`1DE5@00`0000000`0?3`l0jn_[05ADE0050000
000308n?S`3oool0oooo00d0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTL0?ooo`040?_kn`1DE5@0000008n?S`80oooo00L0N7Qh0000001PH600moOg01dM7@00
0000];Bd00<0oooo01H0N7Qh0000002?Shl0gmoO018B4P092@T0/[:b018B4P0000003`l?0>o_k`3O
gml04Q8B0000000?3`l0kno_0?ooo`3kno/0E5AD0000002`/;00E5AD0P0000000`2?Shl0oooo0?oo
o`020?ooo`060?Ogm`0M7Ad00000000000092@T0d];B0P3oool0203gmoL07AdM0000002d];@0moOg
01dM7@000000];Bd0P3oool01P36a/H00000000000000000@D510?Ogm`L0oooo00H0N7Qh00000000
000000000451@@3gmoL20?ooo`0407QhN0000000H61P07QhN080000000<0H61P0?ooo`3oool00P3o
ool01`2RXZ80000000000011@D40VYZJ0000001PH6001@3oool02@3Ogml04Q8B00T92@3Bd]80oooo
07QhN0000000H61P07QhN00200000004061PH03oool0no_k05ADE0<0000000@03`l?0>o_k`3oool0
XZ:R0`000000302?Shl0oooo0?ooo`3gmoL07AdM00T92@2GUiL0000000T92@3Bd]80oooo07QhN0<0
000000<0@D510?Ogm`3oool00P3oool02036a/H00000000000000000@D510?Ogm`3kno/0E5AD0`00
0000200?3`l0inOW01dM7@000000];Bd07QhN0000000Shn?0`3oool00`1hN7P00000061PH0020?oo
o`050::RXP00000000000000002?Shl01P3oool0103gmoL07AdM0000002d];@40?ooo`040::RXP00
00003`l?0>o_k`80oooo00H0moOg01dM7@000000000000T92@3Bd]840?ooo`060<K6aP0000000000
00000011@D40moOg0P3oool00`1hN7P00000061PH0020?ooo`0A0::RXP000000H61P0?ooo`3Ogml0
4Q8B00T92@2b/[804Q8B0000000?3`l0kno_0=oOg`0B4Q80000000l?3`3_knl00`3oool01@2RXZ80
0000000000000000Shn?00<0oooo00@0N7Qh0000001PH600N7Qh0P000000101PH600oooo0?_kn`1D
E5@30000000400l?3`3_knl0moOg01dM7@<0000000<0H61P0?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd609FEU@0I
0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUY703oool02@3kno/0E5AD0000002?Shl0oooo0?Ogm`0M7Ad00000
08n?S`0H0?ooo`040?_kn`1DE5@000000;Bd]100oooo00@0moOg01dM7@000000];Bd:P3oool00`2R
XZ800000061PH0050?ooo`040=oOg`0B4Q802@T90=;BdPX0oooo00@0no_k05ADE0000000];Bd8P3o
ool0103kno/0E5AD0000002d];@K0?ooo`040?Ogm`0M7Ad000000;Bd]0<0oooo00@0a/K600000000
0000Shn??`3oool0103kno/0E5AD0000002d];@H0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTM0?ooo`030::RXP000000000000<0000000<03`l?0>o_k`3oool05`3oool0103kno/0
E5AD0000002d];@@0?ooo`080?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@V0?ooo`030::R
XP000000H61P00D0oooo00@0gmoO018B4P092@T0d];B2P3oool0103kno/0E5AD0000002d];@R0?oo
o`090?_kn`1DE5@000000;Bd]03oool0moOg01dM7@000000];Bd01H0oooo00<0moOg01dM7@000000
1P0000000`11@D40moOg0?ooo`0n0?ooo`040?_kn`1DE5@000000;Bd]1P0oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3@
d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJAh0oooo00H0a/K6000000000000000000l?3`3_knlI0?ooo`040?_k
n`1DE5@000000;Bd]100oooo00P0moOg01dM7@000000];Bd0?Ogm`0M7Ad000000;Bd]2H0oooo00<0
XZ:R0000001PH6001@3oool0103Ogml04Q8B00T92@3Bd]8<0?ooo`0307QhN02d];@0oooo02<0oooo
00L0N7Qh0;Bd]03oool0moOg01dM7@000000];Bd01H0oooo00<0moOg01dM7@0000001@0000000`2?
Shl0oooo0?ooo`110?ooo`0307QhN02d];@0oooo01L0oooo00<0EEEE0?ooo`3oool0103oool00`1E
EED0oooo0?ooo`020?ooo`050>c/k02FUYH0oooo0?Ogm`3]kNd01P3oool06P3Fe]H0d=3@0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd609FEU@0I0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom10?oo
o`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0
kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo
0080oooo00D0k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J0=KFeP3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln
0?ooo`@0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Z
j^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?oo
o`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooool000007@0000050?ooo`D0
EEEE0P3oool505EEE@@0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM
0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooodd0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0ooooC@3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S
0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0ooooC@3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
ooooC@3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooom=0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0
gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooom=0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[Z
jP0M0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF
0=OGe`0<0?ooo`0F0=OGe`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/0?gmo@2CTi<01@2h^;P00`2e]KD0^;Rh
0;Rh^0040;Rh^00F09>CT`3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom=0?ooo`030>c/k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF
0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom=0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF
0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3>c/h00`0n?Sh0c/k>
0<k>cP030<k>cP0J03hn?P3>c/h0c/k>0<k>cP3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`040>c/k02FUYH0oooo
0<o?c`<0?Shn0`3?cll303hn?P80clo?01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`02
0=CDe0<0?Shn00@0e=CD03hn?P0n?Sh0?Shn0`3De=@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02F
UYH0oooo00<0fm_K1@0n?Sh40=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0103T
i>@303hn?PD0i>CT01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`050>g]k@0303hn?P3]
kNd0kNg]00@0kNg]01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`0<0?Cdm00F0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02F
UYH0oooo00d0oooo01D0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
000004d0000000@0k>c/0=[JfP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`000000moOg09ZJVP3o07YjNV00NWYj01<0VYZJ
0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`000000moOg0;no_`3o0;no_f40_kno
0180moOg0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`3Shn<0o`3Shn=Q0>?S
h`0B0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003alO40oooo0?l0oooo6P3o
ool00`3Shn<0YJFU07IfMP0307IfMP0407MgM`1fMWH0MgMg07MgM`T0MWIf00H0YJFU0>?Sh`3oool0
oooo0>?Sh`2UYJD407IfMP0407MgM`1fMWH0MgMg07MgM`T0MWIf00H0YJFU0>?Sh`3oool0oooo0>_[
j`2UYJD405IFEP0405MGE`1FEUH0EeMG05MGE`T0EUIF00<0YJFU0>_[j`3oool00`3oool04P3alO40
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000hN7Q0?clo03kno/20?clo0030?_kn`3l
o?`0o?cl00@0o?cl00<0no_k0?clo03lo?`00P3lo?`00`3kno/0o?cl0?_kn`050?clo0030?_kn`3l
o?`0no_k0640no_kWP3lo?`00`2UYJD0MWIf08f=S@0@08f=S@0607IfMP2UYJD0o?cl0?clo02UYJD0
MWIf4@2=SHd01P1fMWH0YJFU0?clo03kno/0YJFU05IFEQ40Fe]K00<0EUIF0:FUY@3lo?`00`3lo?`0
4P3QhN40000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000h>3P0?Ogm`3o0?OgmaX0moOg
00<0MWIf08f=S@2RXZ80402RXZ801P2=SHd0MWIf0?Ogm`3gmoL0MWIf08f=SA40XZ:R00P0SHf=07If
MP3gmoL0moOg05IFEP1KFe/0Kfm_075aL@d0LW9b00@0LG5a06m_K`1KFe/0EUIF103gmoL04P3Ph>00
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000g]kN0?7al@020?7al@80l_;b00<0lO7a
0?;blP3bl_80103alO400`3bl_80lO7a0?7al@030?7al@80l_;b00<0lO7a0?;blP3alO400P3bl_;o
0?7al@80lO7a00<0MWIf0;Rh^02[Zj/0402[Zj/01P2h^;P0MWIf0?7al@3alO40MWIf0;Rh^140Zj^[
00D0^;Rh07IfMP3alO40l_;b05IFEP0206m_K`03079bLP1gMgL0MgMg00/0MgMg00@0LW9b06m_K`1_
Kfl0EUIF0`3alO404`3bl_80g]kN0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003L
g=`0jn_[0>_[j`3/k>`00P3[jn/00`2WYjL0XZ:R0::RXP020:6QX@030:2PX02OWil0WIfM0080WinO
0P2PX:020:6QX@030:NWY`3[jn/0jn_[00D0jn_[00D0e=CD0<G5a@35aLD0aLG50=OGe`020>_[j`03
0>[ZjP38b<P0d=3@0080jn_[00D0f][J0<K6aP3Shn<0jn_[0>3Ph0020<G5a@070<c<c03YjNT0c/k>
0<O7a`3YjNT0jn_[0>3Ph0020<G5a@070<c<c03Zj^X0e=CD0<G5a@3MgMd0jn_[0>3Ph0020<G5a@06
0<K6aP3Shn<0jn_[0>_[j`3Zj^X0b<S80`35aLD00`36a/H0hn?S0>_[j`070>_[j`030>GUi@37alL0
emOG02H0jn_[HP3/k>`l0>_[j`0407IfMP2k^k/0/K6a0;6a/@P0oooo1`2a/K401P2k^k/0MWIf0>_[
j`3[jn/0MWIf0;^k^`80/K6a303oool30;6a/@090;^k^`1fMWH0jn_[0>_[j`1FEUH0MgMg079bLP1d
M7@0MgMg00/0N7Qh00D0MgMg07AdM01bLW80MgMg05IFEP040>_[j`0B0=cLg0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000030000003Jf]X0iNGU00<0iNGU00<0RXZ:09^KV`2/[:`00`3YjNT02P3Z
j^X0a<C408F5Q@3/k>`0kNg]0>k^kP3WinL0l?3`0?;blP2SXj<60>GUi@0A06QXJ00@4100410@010@
4020P800a/K60<o?c`3Ng]h09RHV055AD@3?cll0iNGU09BDU00E5AD0Y:BT0=kNgP2b/[800P0@4100
1`11@D40fm_K051@D00I6AT0^[Zj0>?Sh`2b/[800P0@41001`11@D40gmoO071`L00@4100TY:B0>CT
i02_[jl00P0@41001P0E5AD0^[Zj0<c<c03Ti>@0gMgM02@T90<0410@00<05ADE0:BTY03Ng]h01`3U
iND01039bLT071`L079bLP3De=C40>GUi@0507IfMP30`<00]KFe0;Fe]@3oool01P1bLW800`0W9bL0
]KFe0;Fe]@050;Fe]@060<30`01fMWH0iNGU0>GUi@1fMWH0`<300P2e]KD00`3oool09bLW02LW9`09
02LW9`<0]KFe00D0`<3007IfMP3UiND0iNGU05IFEP0207MgM`0407UiN@1jNWX0oooo0?ooo`L0NWYj
0P3oool01@1jNWX0NGUi07MgM`1gMgL0EUIF00@0iNGU0180f][J0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000<000000=WIf@3Ogml00`3Ogml01@3Ng]h0X:2P0>[ZjP3YjNT0iNGU0080j^[Z00X0
d=3@07AdM03]kNd0k^k^0:f][@3@d=00l_;b0??cl`2SXj<0gmoO0P3Ng]h0=03Ogml0g]kN06a/K00E
5AD0Zj^[0=kNgP33`l<071`L05ADE036a/H0e]KF02DU9@1ADE40cLg=0=kNgP2?Shl05ADE0:BTY03E
eMD0CTi>03Ph>034a<@0gMgM0=_Kf`1>CTh06ATI0;Zj^P3IfMT0CTi>03Ph>034a<@0gMgM0=OGe`0U
9BD0>3Ph0<C4a032`/806a/K055AD@3IfMT0`l?304a<C03Hf=P0emOG02DU9@0I6AT0aLG50=kNgP2?
Shl05ADE0:BTY03IfMT70=kNgP040<?3``0L71`0LW9b0=7AdL@0g]kN00D0MWIf0<?3``2k^k/0^k^k
0?ooo`06079bLP0302LW9`2k^k/0^k^k00D0^k^k00H0`l?307IfMP3Ng]h0g]kN07IfMP33`l<20;^k
^`030?ooo`0W9bL0^k^k00L0^k^k00<0oooo02LW9`2k^k/00P2k^k/02@33`l<0MWIf0=oOg`3Ng]h0
EUIF07MgM`1hN7P0Ng]k030`<0030?ooo`D0OGem0`3oool01@0`<300Ng]k07QhN01gMgL0EUIF0080
g]kN01@0gmoO0=kNgP3IfMT0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000emOG0=SH
f03IfMT20=SHf00@092@T02?Shl0SXj>0>WYj@21PH40DE5A08N7Q`3Jf]X0YZJV0:VYZ@2/[:`0GemO
0?;blP3clo<0m?Cd0:BTY0<0f=SH00D0fMWI0=;BdP0T92@0DE5A0<_;b`020=SHf00P06YZJP0I6AT0
^[Zj0=3@d00T92@0DE5A0<_;b`3Hf=P0Rh^;01DE5@2TY:@0cLg=02@T900h>3P0`l?30=OGe`3EeMD0
C4a<01TI6@2j^[X0d=3@02@T900h>3P0`l?30=OGe`3AdM4092@T03Ph>033`l<0RXZ:01DE5@2TY:@4
0<G5a@0:09jNWP0@4100LG5a0=7Ad@3Hf=P0Rh^;01DE5@2TY:@0e=CD0=;BdP80aLG500P0a/K60=OG
e`3Hf=P0f=SH0;fm_@0K6a/0LW9b0<k>cRH0f=SHHP3IfMTl0=SHf00507IfMP37alL0_kno0;no_`3o
ool01`0W9bL70;no_`060<O7a`1fMWH0f=SH0=SHf01fMWH0alO70P2o_kl00`3oool09bLW0;no_`07
0;no_`030?ooo`0W9bL0_kno0080_kno00D0alO707IfMP3Hf=P0f=SH05IFEP0207]kN`0307inOP1o
Ogl0:2PX00<0oooo0`1oOgl30?ooo`0302PX:01oOgl0OWin0080Ng]k00<0EUIF0=WIf@3Hf=P00P3H
f=P04P3GemL0000009ZJVP2PX:00Z:RX0;:b/P2l_;`0a/K60=3@d03IfMT0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000eMGE0=?Cd`030=?Cd`03
08F5Q@2SXj<0i^KV0080j^[Z00/0=CDe07moO`3?cll0_[jn08^;R`1NGUh0F5QH06QXJ02SXj<0bLW9
0:ZZZP040=?Cd`040<c<c00T92@0DE5A0<[:bP80dm?C01h0IfMW01TI6@2i^KT0bl_;02@T901ADE40
b/[:0=?Cd`28R8P05ADE0:BTY039bLT092@T03Ph>031`L40dm?C0<o?c`1:BTX06ATI0;Vi^@3;bl/0
92@T03Ph>031`L40dm?C0<c<c00T92@0>3Ph0<71`@1WIfL5010@400:055AD@2MWId0410@08j>SP3<
c<`0dm?C08R8R00E5AD0Y:BT0:6QX@80410@00<06QXJ0<K6aP3Cdm<00P3Cdm<0102h^;P06a/K079b
LP3<c<c40=?Cd`0307IfMP3;bl/0a/K60100a/K600H0bl_;07IfMP3Cdm<0dm?C07IfMP3;bl/20<K6
aP030?ooo`0W9bL0a/K600L0a/K600<0oooo02LW9`36a/H00P36a/H01`3;bl/0MWIf0=?Cd`3Cdm<0
EUIF07emO@1oOgl00`23Ph<01@0X:2P0oooo0?ooo`3oool0Ph>300<0oooo00L0:2PX08>3P`23Ph<0
Ph>307moO`1mOGd0EUIF00@0dm?C0140eMGE0000002PX:00YJFU0:f][@2f][H0_kno0<S8b03Bd]80
f][J0>;RhP3YjNT0kno_0?Cdm03fm_H0nOWi0?_kn`030?gmo@40o_kn0@3oool10?ooo`0000D00000
0=?Cd`3>c/h0c/k>0<o?c`020<k>cP0?08N7Q`2CTi<0dm?C0=_Kf`2][Jd0`L710;2`/02YZJT0ZZZZ
08f=S@2ATI40/;2`0<_;b`3_knl0YJFU00<0c/k>00D0clo?0<O7a`0S8b<0DE5A0<S8b0020<k>cP0]
06EUI@0I6AT0^KVi0<O7a`0S8b<0DE5A0<O7a`3>c/h0QHF501DE5@2TY:@0alO702<S8`0h>3P0`L71
0<k>cP3;bl/0BDU901TI6@2i^KT0alO702<S8`0h>3P0`L710<k>cP37alL08b<S03Ph>031`L40QHF5
01DE5@2TY:@0b/[:0<O7a`0S8b<0DE5A0;6a/@0J6QX0LW9b0<G5a@3>c/h0QHF501DE5@2TY:@0cLg=
00L0c/k>00@0];Bd01XJ6P1bLW80b/[:9P3>c/iR0<o?cc`0c/k>00<0MWIf0<k>cP3;bl/0403;bl/0
1P3>c/h0MWIf0<k>cP3>c/h0MWIf0<k>cP80bl_;00<0oooo02LW9`3;bl/01`3;bl/00`3oool09bLW
0<_;b`020<_;b`090<k>cP1fMWH0clo?0<k>cP1FEUH0OGem08:2PP25QHD0Q8B40080QHF500<0:2PX
0?ooo`3oool00`3oool0200X:2P0QHF508F5Q@24Q8@0QHF508:2PP1mOGd0EUIF103>c/h04@3Cdm<0
00000:RXZ02][Jd0/k>c0;^k^`33`l<0c<c<0=GEe@3MgMd0i>CT0>_[j`3`l?00m?Cd0?Ogm`3inOT0
o?cl00<0oOgm0@3no_h10?ooo`40oooo00000`000000d=3@0<W9b@030<W9b@0A07=cL`2DU9@0OWin
07UiN@2PX:00dM7A0:f][@2UYJD0YZJV08F5Q@1NGUh0[Jf]0>GUi@3fm_H0n?Sh0:BTY039bLT00`38
b<P01032`/808b<S055AD@37alL20<S8b00/069RHP0I6AT0^KVi0<;2`P0R8R80>3Ph0<71`@32`/80
ATI601TI6@2l_;`0`L7102<S8`0h>3P0_kno0<G5a@34a<@0ATI601TI6@2i^KT0`L7102<S8`0h>3P0
_kno0<G5a@30`<008R8R03Ph>030`<00[jn_01XJ6P1cLg<0aLG509ZJVP0@4100SXj>0<G5a@16ATH0
6ATI0;Vi^@35aLD0Ogmo01DE5@2TY:@70<S8b0050<O7a`2^[Zh06QXJ075aL@37alL09P38b<QR0<W9
bC`0b<S800<0MWIf0=?Cd`3=cLd0403=cLd01P3Cdm<0MWIf0<S8b038b<P0MWIf0=?Cd`80cLg=00<0
oooo02LW9`3=cLd01`3=cLd00`3oool09bLW0<g=c@020<g=c@070=?Cd`1fMWH0b<S80<S8b01FEUH0
PX:208>3P`0508N7Q`0502PX:03oool0oooo0?ooo`0X:2P01@27QhL00`23Ph<0PX:205IFEP030<W9
b@0B0<S8b03@d=0000000;:b/P2f][H0^k^k0<;2`P39bLT0dM7A0=WIf@3Ph>00inOW0>g]k@3alO40
mOGe0?Shn03jn_X0o?cl0`3moOd10?knoP40oooo0@3oool000030000003?cll0aLG500@0aLG500l0
UiNG0;Ng]`3/k>`0^KVi08^;R`2g]kL0ZZZZ0:BTY01nOWh0LG5a0<C4a03IfMT0m_Kf0?Win@2UYJD0
1035aLD0102o_kl08R8R055AD@36a/H20<G5a@06065QH@0I6AT0^KVi0;no_`0R8R80GEeM0P0@4100
100E5AD0Y:BT0<;2`P15ADD3010@4006055AD@1QHF40410@01TI6@2f][H0ADE50`0@4100201ADE40
_kno028R8P0h>3P0_kno0<G5a@2/[:`06QXJ0P0@41001@1bLW80aLG50<G5a@32`/80ADE500<0410@
00<05ADE0:BTY036a/H01@35aLD01P2/[:`0G5aL0:f][@0J6QX0LG5a0<K6a/@0aLG500<0MWIf0=GE
e@3AdM40403AdM401P3EeMD0MWIf0<G5a@35aLD0MWIf0=GEe@80dM7A00<0oooo02LW9`3AdM401`3A
dM400`3oool09bLW0=7Ad@020=7Ad@070=GEe@1fMWH0aLG50<G5a@1FEUH0PX:208N7Q`0508V9R@D0
oooo1@29RHT00`27QhL0PX:205IFEP040<G5a@0A0<o?c`000000_;bl0;no_`33`l<0bLW90=3@d03G
emL0gMgM0>CTi03Zj^X0kno_0??cl`3fm_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`00
0000cLg=0<71`@030<71`@0@07EeM@2IVIT0YjNW0>g]k@39bLT0Lg=c0;2`/02=SHd0Lg=c04a<C01R
HV80_[jn0;Fe]@2b/[80[Jf]0:VYZ@@0`L7100@0^k^k028R8P1ADE40a<C40P31`L40101OGel06ATI
0;Vi^@32`/870<71`@050;^k^`0R8R80>3Ph0;jn_P32`/800P31`L403033`l<0aLG50;bl_00R8R80
>3Ph0;jn_P32`/80^k^k028R8P0h>3P0_[jn0<;2`P/0`L7100@0O7al01DE5@2TY:@0`l?31@31`L40
1P2YZJT06QXJ010@400@4100LG5a0<C4a<@0`L7100<0MWIf0=SHf03EeMD0403EeMD01P3Hf=P0MWIf
0<71`@31`L40MWIf0=SHf080eMGE00<0oooo02LW9`3oool0203oool01P0W9bL0eMGE0=GEe@3EeMD0
f=SH07IfMP80`L7100<0EUIF08B4Q028R8P0102:RXX30?ooo`0402PX:03oool0oooo0?ooo`@0RXZ:
00<0R8R808B4Q01FEUH01031`L40403=cLd000000<K6aP38b<P0c<c<0=7Ad@3GemL0g=cL0>;RhP3X
j>P0kNg]0?7al@3emOD0moOg0?Win@3lo?`30?gmo@80o_kn0P3oool000030000003<c<`0^[Zj0080
_Kfm0140_[jn0;fm_@2NWYh0hn?S0;Jf]P2FUYH0PX:209>CT`24Q8@0PX:204i>CP0/;2`0W9bL0:^[
Z`3fm_H0n?Sh0:NWY`040;fm_@040;Ng]`0Q8B40DE5A0<C4a080_Kfm00@0GEeM01TI6@2i^KT0_kno
202m_Kd00`1MGEd0=cLg0;Rh^0020;fm_@0;05eMG@0I6AT0^KVi0;no_`1MGEd0=cLg0;Rh^02m_Kd0
]kNg024Q8@0h>3P00P2n_[h;0;fm_@0407YjNP0E5AD0Y:BT0;no_`H0_Kfm00D0YJFU01XJ6P0@4100
L71`0<;2`P0V0;fm_F80_[jn?02m_Kd00`1fMWH0f][J0=GEe@0@0=GEe@060=[JfP1fMWH0_Kfm0;fm
_@1fMWH0f][J0P3EeMD00`3oool0LW9b079bLP08079bLP0>02LW9`3EeMD0eMGE0=GEe@3Jf]X0MWIf
0;fm_@2n_[h0EUIF08N7Q`2:RXX0SHf=08j>SP2=SHd30?ooo`0302PX:02=SHd0:2PX00<0oooo0`2=
SHd0102:RXX0QhN705IFEP2n_[h20;fm_@0A0;Zj^P3<c<`000000=3@d03Bd]80eMGE0=WIf@3MgMd0
h^;R0>OWi`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`00`3moOd00`3no_h0oooo0?ooo`010?oo
o`0000<000000<[:bP2h^;P00`2j^[X0401lO7`0SHf=07QhN02IVIT0l?3`0<;2`P2WYjL0bl_;0;bl
_02SXj<0KFe]07emO@2/[:`0no_k0?clo02VYZH30;Zj^P040;^k^`2d];@08B4Q04e=C@<0^[Zj00<0
Fe]K01TI6@2_[jl03P2j^[X00`1KFe/06ATI0:n_[`050;Zj^P040;Bd]00Q8B40=cLg0;Bd]0`0^[Zj
00<0MgMg01@D502KVi/02@2j^[X00`1KFe/0Jf][0;Zj^P340;Zj^P0307IfMP3Hf=P0f][J0100f][J
00H0f=SH07IfMP2j^[X0^[Zj07IfMP3Hf=P20=[JfP030?ooo`1bLW80LW9b00P0LW9b00d09bLW0=[J
fP3Jf]X0f][J0=SHf01fMWH0^[Zj0;^k^`1FEUH0Rh^;08n?S`2>SXh0Shn?00<0oooo00D0:2PX08n?
S`2?Shl0Shn?02PX:0030?ooo`0508n?S`2>SXh0Shn?08^;R`1FEUH00`2j^[X0402h^;P0b/[:0000
003IfMT0f][J0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`30?gmo@80
o_kn0`3oool0000300000039bLT0^;Rh00<0^;Rh0100QXJ609jNWP3De=@0l?3`0?;blP3>c/h0`L71
0=gMg@3Bd]80XZ:R0<c<c039bLT0RHV90?clo03moOd0YjNWo`2h^;P70;Rh^00307IfMP3GemL0gMgM
0100gMgM00H0emOG07IfMP2h^;P0^;Rh07IfMP3GemL30=gMg@/09bLW0`3MgMd02@3GemL0MWIf0;Rh
^02h^;P0EUIF092@T02ATI40Ti>C09BDU0020?ooo`0302PX:02HV9P0V9RH00<0V9RH00P0:2PX0?oo
o`3oool0U9BD09>CT`2ATI40T92@05IFEP@0^;Rh00l0bLW90000003QhN40h^;R0>CTi03WinL0j^[Z
0>g]k@3`l?00lo?c0?Gem@3gmoL0nOWi0?_kn`3lo?`00`3moOd00`3no_h0oooo0?ooo`020?ooo`00
00<000000<W9b@2h^;P0102h^;P03`26QXH0U9BD0?;blP3clo<0X:2P0>SXj03fm_H0fm_K09VIV@3j
n_X0no_k0=cLg02f][H0o_kn0:NWY`3o0;Rh^0L0^;Rh00<0MWIf0=KFeP3QhN40403QhN401P3Fe]H0
MWIf0;Rh^02h^;P0MWIf0=KFeQ40hN7Q00T0e]KF07IfMP2h^;P0^;Rh05IFEP2EUID0UiNG09bLW02R
XZ800P0X:2P01@2[Zj/0[Jf]0:^[Z`2[Zj/0[Jf]0080Zj^[0P0X:2P01@2RXZ80W9bL09NGU`2EUID0
EUIF00@0^;Rh00h0bLW90000003Xj>P0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0
no_k0?clo0<0oOgm0P3no_h40?ooo`0000<000000<W9b@2e]KD00`2e]KD03@1gMgL0X:2P0>3Ph03c
lo<0iNGU0>c/k03fm_H0n?Sh0?Win@2@T900no_k0?clo03moOd00P3no_h00`2WYjL0]KFe0;Fe]@3o
0;Fe]@D0]KFe00<0MWIf0>g]k@3YjNT0403YjNT01P3]kNd0MWIf0;Fe]@2e]KD0MWIf0>g]kA40jNWY
00X0kNg]07IfMP2e]KD0]KFe05IFEP2KVi/0WYjN0:FUY@2b/[80^[Zj0P2o_kl01030`<00_kno0;no
_`30`<020;no_`070<30`02j^[X0/[:b0:FUY@2NWYh0Vi^K05IFEP040;Fe]@0=0<W9b@000000k^k^
0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k0?clo0030?gmo@80o_kn1@3oool00003
00000039bLT0^[Zj00@0/K6a00<0SHf=0:ZZZP2ZZZX01@2ZZZX00`2WYjL0ZZZZ0:ZZZP040:ZZZ_l0
/K6a1`2a/K400`1fMWH0e=CD0?;blP0@0?;blP060=CDe01fMWH0/K6a0;6a/@1fMWH0e=CD4@3bl_80
1P3De=@0MWIf0;6a/@2a/K40EUIF0:>SX`80Z:RX3@33`l<20:RXZ0030:>SX`1FEUH0/K6a0080/K6a
00@0^[Zj0<W9b@000000lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3lo?`0103moOd0
0`3no_h0oooo0?ooo`040?ooo`0000<000000<?3``39bLT0o`2^[ZhJ0:j^[P0408f=S@1fMWH0c/k>
0?_kn`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^[Zh0102=SHd0MWIf0<k>cP3kno/=0?ooo`040?_k
n`3>c/h0MWIf08f=S@80[Zj^00H0VIVI05IFEP2e]KD0bl_;0=oOg`3Xj>P90>OWi`060>SXj03Ogml0
bl_;0;Fe]@1FEUH0VIVI0`2^[Zh00`39bLT0`l?3000000020?KfmP050?Ogm`3hn?P0n?Sh0?Win@3j
n_X00P3lo?`40?gmo@80o_kn1P3oool0000400000033`l<0e=CD0;Ng]ol0YjNW6P2WYjL00`2=SHd0
MWIf07IfMP0?07IfMP0308f=S@2WYjL0YjNW0080YjNW00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0
YjNW0:NWY`020:NWY`0308f=S@1FEUH0EUIF00l0EUIF00P0SHf=0:NWY`2WYjL0YjNW0;Ng]`3De=@0
`l?3000000<0nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00001P000000ZZZZ0>OW
i`3De=@0_[jn0;>c/ol0ZZZZFP2ZZZX01P2c/k<0_[jn0=CDe03WinL0ZZZZ00000080no_k0`3lo?`5
0?gmo@<0o_kn203oool000070000002:RXX0fMWI0?clo03Ti>@0d];B0<c<c03o0;jn_UP0_[jn00P0
c<c<0=;BdP3Ti>@0o?cl0=WIf@2:RXX000000?clo0P0oOgm0P3no_h:0?ooo`0000P0000006UYJ@2G
UiL0fMWI0?clo03bl_80k>c/0>?Shol0fMWIEP3IfMT0203Shn<0k>c/0?;blP3lo?`0fMWI09NGU`1Y
JFT000001`3moOd30?knoP/0oooo00001@000000oooo06UYJ@2GUiL0dm?C0080o?clo`3ooomH0?oo
o`80o?cl00D0dm?C09NGU`1YJFT0oooo000000040?gmo@@0o_kn3@3oool000030000003oool0oooo
0080JFUY00<0RXZ:09NGU`2ZZZX0o`2ZZZYG0:ZZZP0409NGU`2:RXX0JFUY06UYJ@80oooo00<00000
0?knoP3no_h0103no_h?0?ooo`00o`00001V000001D0oooo0000\
\>"],
  ImageRangeCache->{{{126.938, 390.5}, {377.688, 233.125}} -> {-181.964, \
334.815, 1.00234, 1.00234}},
  CellTags->"IPM3.1.1.6"]
}, Open  ]],

Cell["\<\
This moves the selection bar past the last cell in the above notebook.\
\>", "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SelectionMove[nb, Next, Cell, 4]\)], "Input",
  CellLabel->"In[45]:=",
  CellTags->{"i:28", "b:0.5", "IPM3.1.1.6"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .54907 
%%ImageSize: 378 208 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
2.5804e-017 0.00265252 -0.00265252 0.00265252 [
[ 0 0 0 0 ]
[ 1 .54907 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .54907 L
0 .54907 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00265 translate
1.00265 .55172 scale
1134 string
378 208 8 [378 0 0 208 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FEFEFEFEFEFEFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFC
FCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFC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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFBFBFB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FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9
F8F8F8F6F6F6F5F5F5F3F3F3F1F1F1F0F0F0EFEFEFEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEFEFEFF0F0F0
F1F1F1F3F3F3F5F5F5F6F6F6F8F8F8F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7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FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7F5F5F5
F3F3F3F0F0F0EDEDEDEAEAEAE7E7E7E4E4E4E2E2E2E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E2E2E2E4E4E4
E7E7E7EAEAEAEDEDEDF0F0F0F3F3F3F5F5F5F7F7F7F9F9F9FBFBFBFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F3F3F3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FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F4F4F4F0F0F0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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F5F5F5F1F1F1EDEDED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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F6F6F6F3F3F3EFEFEFEAEAEA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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F5F5F5F1F1F1EDEDEDE7E7E7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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F4F4F4F0F0F0EBEBEBE4E4E4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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFBFBFBF9F9F9F6F6F6F4F4F4EFEFEFE9E9E9E2E2E2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FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F6F6F6F3F3F3EEEEEEE8E8E8E1E1E1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000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
ECECECBABABA9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FBDBDBDECECECE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
969696DBDBDBCACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACADBDBDBA3A3A3FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFF
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0FFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFF
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E0000000000000000009E9E9ED9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
DCDCDC515151B9B9B9E0E0E0D9D9D91A1A1A000000383838D9D9D9E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD1D1D1D0D0D0D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0
D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF969696FAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0E0E0E0E0E0E0
E0E0E0ADADAD545454E0E0E0ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6
D6D6D6D7D7D7D7D7D7D7D7D7D6D6D6D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6
959595D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D7D7D7
D7D7D7D7D7D7D6D6D6D6D6D6D6D6D6FFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCECECECECECECECECE
CECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFF
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9E0E0E0E0E0E0
E0E0E0DCDCDC515151B9B9B9ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595DCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFF
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2E0E0E0E0E0E0
E0E0E0E0E0E0C4C4C4484848D2D2D21A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000000000
000000000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFD4D4D4D4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595DCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF969696F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8969696FFFFFFD4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2DCDCDC494949
0000000D0D0DD2D2D26969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000
000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFDBDBDBDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696
969696FFFFFFE4E4E4E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E4E4E4
E4E4E4E4E4E4E4E4E4E3E3E3E4E4E4FFFFFF969696F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7969696FFFFFFDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9ADADAD545454
E0E0E06969697E7E7EC4C4C4484848D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFE4E4E4E4E4E4E4E4E4
3E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF969696F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5969696FFFFFFE4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E000000383838D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0ADADAD545454
E0E0E06969697E7E7EE0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF969696F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDEDEDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D9D9D9525252D9D9D9E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0C4C4C41D1D1DD2D2D2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFF
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
969696FFFFFFF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFFF969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
969696FAFAFAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFAFAFA969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0DADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
D6D6D6B0B0B0B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8A9A9A9CECECEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEDADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECE3E3E3ECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3E3E3E3E3E3E3ECFCFCF
CFCFCFCFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3E
D4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E
3E3E3EDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFE4E4E4E4E4E43E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E43E3E3E
E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFC6C6C6000000000000090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF121212000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000606060FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000606060
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFF7F7F7262626D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1B
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFC6C6C6606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
DFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFF7F7F7262626D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F7F7F7262626D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFBFBFB5454548F8F8FFFFFFF
A2A2A20F0F0FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878606060FFFFFFFBFBFB545454414141
F7F7F7DFDFDF121212B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1B
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFBFBFB545454000000414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFA2A2A2414141F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C6C6C68F8F8FFFFFFF7878788F8F8FFBFBFB5D5D5DD2D2D2FBFBFB5D5D5DD2D2D2C6C6C6
8F8F8FFFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFA2A2A28F8F8FFBFBFB5D5D5DCECECE
545454000000606060F7F7F7262626D2D2D2DFDFDF212121EFEFEFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFFBFBFB545454000000606060F7F7F7262626D2D2D2DFDFDF535353F7F7F7
DFDFDF121212B4B4B4DBDBDB5D5D5DD2D2D2FFFFFFC6C6C68F8F8FFFFFFF7878788F8F8F
DFDFDF121212B4B4B4FBFBFB545454B4B4B4DFDFDF535353F7F7F7DFDFDF121212B4B4B4
DFDFDF535353F7F7F7DFDFDF121212B4B4B4F7F7F72C2C2CEFEFEFCBCBCBD2D2D2FFFFFF
C6C6C68F8F8FFFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFA2A2A28F8F8FFBFBFB5D5D5D
CECECE5454540000008F8F8FFFFFFFC6C6C68F8F8FFFFFFF7878788F8F8FFFFFFFFFFFFF
FFFFFFFFFFFFADADADEFEFEFA2A2A28F8F8FFFFFFFFFFFFFA8A8A8D2D2D2A2A2A28F8F8F
FFFFFFA2A2A28F8F8FFBFBFB5D5D5DD2D2D2DBDBDB5D5D5DD2D2D2FFFFFFF7F7F71D1D1D
606060F7F7F7262626D2D2D2C6C6C68F8F8FFFFFFF7878788B8B8B5454540000008F8F8F
F7F7F71D1D1D606060FBFBFB5D5D5DD2D2D2A2A2A28F8F8FFBFBFB5D5D5DD2D2D2DBDBDB
5D5D5DD2D2D2FFFFFF787878090909D2D2D2A2A2A28F8F8FFFFFFF787878B4B4B4A2A2A2
8F8F8FFBFBFB5D5D5DD2D2D2FFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFF7878788F8F8F
F7F7F72C2C2CE7E7E7A4A4A4FBFBFB545454000000606060FFFFFFDFDFDF121212B0B0B0
5454540000008F8F8FFFFFFFC6C6C68F8F8FFFFFFF7878788F8F8FC6C6C68F8F8FFFFFFF
818181D2D2D2C6C6C68F8F8FFFFFFF818181D2D2D2787878090909D2D2D2DFDFDF535353
F7F7F7DFDFDF121212B0B0B0545454000000606060F7F7F7262626D2D2D2C6C6C68F8F8F
FFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDED3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3DEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000000000606060FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2
0000000000000000000000008F8F8FFFFFFFFBFBFB545454000000B4B4B4FFFFFF787878
000000B4B4B4FFFFFFDFDFDF121212090909D2D2D2FFFFFFDFDFDF121212090909D2D2D2
FFFFFFC6C6C60000000F0F0FEFEFEFFBFBFB545454000000B4B4B4FFFFFFFFFFFFF7F7F7
1D1D1D090909D2D2D2DFDFDF121212090909D2D2D2FFFFFFFFFFFFF7F7F71D1D1D090909
D2D2D2F7F7F71D1D1D000000B4B4B4F7F7F71D1D1D000000B4B4B4DFDFDF121212090909
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFDFDFDF121212000000000000414141F7F7F7FFFFFF787878000000606060FFFFFF
FFFFFFA2A2A2000000606060FFFFFFDFDFDF121212090909D2D2D2FFFFFFFFFFFFA2A2A2
000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFF
787878000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFFBFBFB545454000000
B4B4B4FFFFFFC6C6C6000000000000000000000000000000000000090909D2D2D2F7F7F7
1D1D1D000000B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000000000
414141F7F7F7DFDFDF121212090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB
545454000000B4B4B4FFFFFFF7F7F71D1D1D000000B4B4B4FFFFFF7878780000008F8F8F
FFFFFFA2A2A20000008F8F8FFFFFFFC6C6C6000000000000000000000000000000000000
090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D000000B4B4B4FFFFFFFFFFFF
FFFFFFFFFFFFF7F7F71D1D1D090909D2D2D2DFDFDF121212090909D2D2D2FFFFFFFFFFFF
F7F7F71D1D1D090909D2D2D2DFDFDF121212090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF787878000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFDFDFDF
121212090909D2D2D2FFFFFFDFDFDF121212090909D2D2D2FFFFFFC6C6C60000000F0F0F
EFEFEFC6C6C6000000000000000000000000000000000000090909D2D2D2FFFFFF787878
000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFFBFBFB545454000000B4B4B4
FFFFFFF7F7F71D1D1D000000000000000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECECFDFDFD939393B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B5B5B5B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8939393ECECEC696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECE3E3E3ECECECECECECECECECECECECECECECE3E3E3E
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF3E3E3E3E3E3E
3E3E3ECFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3E
D4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3E3E3E3E3E3E3EDBDBDB
DBDBDBDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFE4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDED3E3E3EEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F79A9A9A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A9A9A9AF7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFF7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3F7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F1F1F1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE3E3E3A5A5A5767676767676
767676767676777777767676777777777777767676767676767676767676767676767676
767676767676767676A5A5A5E3E3E3FFFFFFFFFFFFE3E3E3A5A5A5767676767676767676
767676777777767676777777777777767676767676767676767676767676767676767676
767676767676A5A5A5E3E3E3FFFFFFFFFFFFEBEBEBA5A5A5565656565656565656565656
575757565656575757575757565656565656565656565656565656565656565656565656
565656A5A5A5EBEBEBFFFFFFFFFFFFFFFFFFFFFFFFF1F1F10000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E1E1E1FCFCFCFBFBFBFCFCFCFCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCA5A5A57676768D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D767676A5A5A5FCFCFCFCFCFCA5A5A57676768D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D767676A5A5A5FCFCFCFBFBFBA5A5A55656565B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B565656A5A5A5FCFCFCFCFCFCFCFCFCFCFCFCE1E1E10000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F75656565B5B5B6F6F6F717171727272727272
727272727272727272727272727272727272727272727272727272727272727272717171
6F6F6F5B5B5B565656F7F7F7F7F7F7F7F7F7F7F7F7E0E0E00000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DEDEDEF1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1
F1F1F1F1F1F1F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2
F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1767676B8B8B8ABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABB8B8B8767676F1F1F1F1F1F1767676B8B8B8ABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABB8B8B8767676F1F1F1F2F2F25656566F6F6F6F6F6F727272777777777777
777777777777777777777777777777777777777777777777777777777777777777727272
6F6F6F6F6F6F565656F1F1F1F1F1F1F1F1F1F2F2F2DEDEDE0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DCDCDCEBEBEBEBEBEBECECECEBEBEBEBEBEBA7A7A7A2A2A2A2A2A2A1A1A1A1A1A1
A0A0A09F9F9F9D9D9D9F9F9F9F9F9FA0A0A0A0A0A0A1A1A1A1A1A1A7A7A7EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBD4D4D4C5C5C5C5C5C5C5C5C5D7D7D7EBEBEBEBEBEB
EAEAEAC8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3EBEBEBE0E0E0C5C5C5C5C5C5
CCCCCCE9E9E9CECECEC7C7C7E9E9E9EBEBEBE0E0E0C5C5C5C5C5C5CCCCCCEAEAEAD4D4D4
C5C5C5DDDDDDEBEBEBE0E0E0C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEAEAEAC8C8C8
C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBE5E5E5C7C7C7D7D7D7EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1B1B1B1B1B1B1B1B1B1
B1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1
B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB565656777777727272747474777777787878
787878787878787878787878787878787878787878787878787878787878777777747474
727272777777565656EBEBEBEBEBEBEBEBEBEBEBEBDCDCDC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DADADAE5E5E5E5E5E5E5E5E5E5E5E58A8A8A9B9B9BACACACE9E9E9E9E9E9E9E9E9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000000D9D9D9DFDFDFDFDFDFDFDFDFDFDFDFDEDEDEA0A0A0EAEAEAE9E9E9E5E5E5EAEAEA
EAEAEAD0D0D0747474EDEDEDEEEEEEADADADD0D0D0F2F2F2F3F3F3A3A3A3DFDFDFDEDEDE
DEDEDEDFDFDFDEDEDE6C6C6C151515ABABABDEDEDEC3C3C31C1C1C545454C6C6C6D6D6D6
252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4D5D5D54E4E4E383838C4C4C4DDDDDD
DBDBDB4E4E4E191919BABABAD9D9D94E4E4E383838C4C4C4DDDDDDD7D7D7252525383838
C4C4C4C2C2C21B1B1B515151D9D9D9C3C3C34C4C4CD8D8D8D7D7D7252525191919C5C5C5
DEDEDE8F8F8F151515A4A4A4D9D9D9DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
C3C3C31C1C1C727272D1D1D1DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBB
FFFFFF727272727272727272727272727272727272272727BBBBBBBBBBBBBBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBC3C3C3767676DEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBBFFFFFF
272727BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBFFFFFF272727BBBBBB
BBBBBBBBBBBBC3C3C3767676DFDFDFDEDEDE5656567777777878787B7B7B303030FFFFFF
FFFFFFFFFFFF7D7D7D7D7D7D7D7D7D7D7D7D7D7D7DFFFFFFFFFFFFFFFFFF3030307B7B7B
787878777777565656DEDEDEDEDEDEDFDFDFDEDEDED9D9D90000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D7D7D7D8D8D8D9D9D9D8D8D8D8D8D89090908F8F8F8E8E8EE9E9E9818181515151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000000D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3858585A3A3A3E6E6E6EAEAEAEAEAEA353535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000000D3D3D3CECECECECECECFCFCFCECECECECECE878787939393D3D3D3DBDBDBADADAD
C1C1C1B0B0B0A9A9A9AAAAAA8D8D8D919191B0B0B0CBCBCBEFEFEFA5A5A5CECECECECECE
CECECECFCFCFC7C7C7232323515151C8C8C8CECECECECECE656565191919B9B9B9C7C7C7
232323515151C7C7C7CECECE858585151515A4A4A4C7C7C7232323383838C1C1C1CECECE
CBCBCB494949191919B9B9B9C7C7C7232323383838C1C1C1CECECEC7C7C7232323383838
C1C1C1858585151515A4A4A4CACACAC7C7C7232323515151B1B1B11A1A1A727272C5C5C5
CECECE858585151515A4A4A4CDCDCDCECECECECECECECECECECECECECECECECECECECECE
B4B4B41A1A1A727272CACACACECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECE767676CECECECBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCECECE767676CECECECECECE767676CECECECBCBCBCBCBCBFFFFFF
272727CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBFFFFFF272727CBCBCB
CBCBCBCBCBCBCECECE767676CFCFCFCECECE5656567D7D7D828282858585848484858585
858585282828FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF282828858585858585848484858585
8282827D7D7D565656CECECECECECECECECECECECED3D3D3000000A8A8A8ADADADB3B3B3
BBBBBBC3C3C3CCCCCCD5D5D5DDDDDDE4E4E4EBEBEBF0F0F0F4F4F4F7F7F7F9F9F9FCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D0D0D0C9C9C9C9C9C9C9C9C9C9C9C97373739494947E7E7E797979A0A0A0D1D1D1
ADADADA5A5A5A6A6A68585855E5E5EADADADE5E5E5F6F6F6F8F8F8A4A4A4C9C9C9C8C8C8
C8C8C8C8C8C8C2C2C2232323515151C7C7C7C8C8C8C8C8C8626262191919B9B9B9C2C2C2
222222383838C1C1C1C2C2C2464646191919BCBCBCC1C1C1232323383838BFBFBFC5C5C5
C4C4C4464646191919B9B9B9C1C1C1232323383838BFBFBFC5C5C5C0C0C0222222383838
C0C0C0AFAFAF1A1A1A737373C5C5C59A9A9A1010108E8E8EC5C5C5464646191919B9B9B9
C5C5C57F7F7F151515A4A4A4C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C7C7C7
AEAEAE1A1A1A717171C7C7C7C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8767676D3D3D3CDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDD3D3D3767676C8C8C8C8C8C8767676D3D3D3CDCDCDCDCDCDFFFFFF
272727CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDFFFFFF272727CDCDCD
CDCDCDCDCDCDD3D3D3767676C8C8C8C8C8C8565656828282838383878787878787878787
878787878787282828FFFFFFFFFFFFFFFFFF282828878787878787878787878787878787
838383828282565656C9C9C9C9C9C9C9C9C9C8C8C8D0D0D0000000B2B2B2B6B6B6BBBBBB
C2C2C2C9C9C9D1D1D1D9D9D9E0E0E0E7E7E7EDEDEDF1F1F1F5F5F5F8F8F8FAFAFAFCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000CFCFCFC5C5C5C5C5C5C5C5C5C5C5C5C5C5C5979797B7B7B7ECECECB9B9B98B8B8B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000000CDCDCDC1C1C1C1C1C1C1C1C1C1C1C1757575999999A7A7A7EDEDEDC9C9C9737373
B0B0B08D8D8D7373734C4C4C626262BEBEBEB5B5B5B2B2B2ADADADA9A9A9C1C1C1C1C1C1
C1C1C1C1C1C1BBBBBB222222515151C4C4C4C1C1C1C1C1C15F5F5F191919B9B9B9C2C2C2
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1BBBBBB222222383838BEBEBEC2C2C2
C1C1C1C1C1C1C3C3C3C5C5C5BCBCBC222222383838BEBEBEC2C2C2BBBBBB222222383838
BEBEBEC2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C17C7C7C151515A4A4A4C3C3C3C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1A9A9A91A1A1A
101010101010717171C4C4C4C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1767676D8D8D8D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D8D8D8767676C1C1C1C1C1C1767676D8D8D8D5D5D5D5D5D5FFFFFF
272727FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF272727D5D5D5
D5D5D5D5D5D5D8D8D8767676C1C1C1C1C1C15656568484848888888A8A8A8A8A8A8A8A8A
8A8A8AFFFFFFFFFFFFFFFFFF282828FFFFFFFFFFFFFFFFFF8A8A8A8A8A8A8A8A8A8A8A8A
888888848484565656C1C1C1C1C1C1C1C1C1C1C1C1CDCDCD000000C6C6C6C8C8C8CCCCCC
D1D1D1D7D7D7DCDCDCE2E2E2E8E8E8EDEDEDF1F1F1F5F5F5F7F7F7F9F9F9FCFCFCFDFDFD
FDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
000000CCCCCCBABABABDBDBDBDBDBDBEBEBEBDBDBD9E9E9EE3E3E3B6B6B6969696828282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000000CACACAB8B8B8BABABABABABABABABA7C7C7C8D8D8D787878999999F0F0F0C2C2C2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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B88686869E9E9ED4D4D4F0F0F0F2F2F2CECECE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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8868686949494F2F2F2F3F3F3A0A0A0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000000C9C9C9B5B5B5B5B5B5B5B5B5B5B5B5777777A0A0A0E0E0E0F3F3F3E5E5E5ECECEC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000000C9C9C9BABABAB1B1B1B1B1B1B1B1B1B1B1B18D8D8DAAAAAAAAAAAAAAAAAAAAAAAA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000000C3C3C3C9C9C9AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE8D8D8D767676CECECEFBFBFB
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFBFBFBCECECE7676768D8D8DAEAEAEAEAEAE8D8D8D767676CECECEFBFBFBFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FBFBFBCECECE7676768D8D8DAEAEAEAEAEAE999999565656B5B5B5CBCBCBDFDFDFE8E8E8
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8DFDFDFCBCBCB
B5B5B5565656999999AEAEAEAEAEAEAEAEAEC9C9C9C3C3C3000000F6F6F6F6F6F6F7F7F7
F8F8F8F8F8F8F9F9F9FAFAFAFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000C3C3C3D4D4D4B7B7B7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D565656565656565656565656
565656565656565656565656565656565656565656565656565656565656565656565656
5656568D8D8DA7A7A7A7A7A7A7A7A7B7B7B7D4D4D4C3C3C3000000F9F9F9F9F9F9F9F9F9
FAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000AAAAAAE7E7E7D4D4D4BEBEBEB3B3B3AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAB3B3B3BEBEBED4D4D4E7E7E7AAAAAA000000FBFBFBFBFBFBFCFCFC
FCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
0000008A8A8AD9D9D9FCFCFCE4E4E4D2D2D2CCCCCCBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBECCCCCCD2D2D2E4E4E4FCFCFCD9D9D98A8A8A000000FCFCFCFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000696969979797D9D9D9FCFCFCF2F2F2ECECECE3E3E3D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9E3E3E3ECECECF2F2F2FCFCFCD9D9D9979797696969000000FDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFF696969979797D3D3D3FCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFCFCFCFCFCFCD3D3D3979797696969FFFFFF000000FDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFF6969696969698A8A8A979797AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAA9797978A8A8A696969696969FFFFFFFFFFFF000000FEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{378, 208},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005j0000d2000`400?l00000o`00003oo`3ooomk
0?ooo`00o`3ooomk0?ooo`003P3ooooo0?knoUh0o_kn3`3oool000`0oooo103no_ko0?gmoEX0oOgm
103no_h=0?ooo`002P3oool30?kno_l0oOgmH03moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo5@0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?clo?l0no_kEP3kno/30?cl
o0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinEL0
nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_IF0?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`4
0?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gmo@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0
moOg0?KfmP3emOD00P3dm?Co0??cle@0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm00X0o?cl0?_kn`3inOT0
n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^k^iD0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0
m_Kf0?Shn03inOT0no_k0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj5@0j>SX00/0jNWY0>_[j`3]
kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0030?gmo@80o_kn103oool000<0oooo0100
o_kn0?gmo@3moOd0oOgm0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN5D0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3kno/0
o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`000P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0
m_Kf0??cl`3_knl0k>c/0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SHDP3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`00`3moOd20?knoP<0oooo
00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;R
hP3MgMd0fMWI0=GEe@3Bd]80d=3@o`3?clmB0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0oooo0040oooo00000`3o
ool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3A
dM40c<c<0<S8b036a/Ko0<G5aE80aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo0?knoP3no_h00P3moOd0
3`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl
_03o0;^k^e80^k^k00l0_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gmo@020?gmo@0?0?clo03j
n_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a
DP2a/K403`2b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd00P3moOd03`3lo?`0nOWi
0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYU80
YZJV00l0Z:RX0:f][@2c/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080oOgm00l0no_k0?Win@3f
m_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S80;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYiB09jN
WP0?0:2PX02UYJD0[Jf]0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0o?cl0?_kn`3inOT0m_Kf
0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l_;`0/[:b0:RXZ02PX:00o`2JVYYD09ZJVP0?0:2P
X02XZ:P0/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000006H000000100VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00003
0000003lo?`0o?cl0?l0o?clHP3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`00
0000o?cl0:^[Z`3o0:^[Zf40Zj^[0180o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFUP06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj9`2OWil00`2m_Kd0k>c/0>OWi`0_0>OWi`030>k^kP2=SHd0WinO00`0WinO00<0SHf=
0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3Ng]h0fm_K
00`0fm_K00@0g]kN09JFUP2FUYH0fm_K9`3:b/X00`3Kfm/0Xj>S0?_kn`0_0?_kn`0309JFUP3Ng]h0
fm_K00`0fm_K00<0g]kN09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo9`3@d=000`3oool0UYJF0?_kn`0_0?_kn`0309JFUP3oool0c/k>00`0c/k>00<0oooo
09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY
09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF
09JFUP3ooolW0<o?c`030?ooo`2FUYH0no_k02l0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ0000000=3@d0d];B0>3P
h03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3Lg=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph
0=WIf@0F0>3Ph00309JFUP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM401@3@d=0;0=7Ad@<0d=3@0`3AdM450=3@d0T0dM7A00<0oooo09JF
UP3jn_X0;`3jn_X00`2FUYH0oooo0<k>cP040<k>cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JF
UP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000
003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3P
h01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0103>c/h303hn?PD0c/k>
00@0oooo09JFUP2FUYH0oooo2`3Fe]H30=OGe`80e]KF00L0UIFE0=KFeP2EUID0e]KF09FEU@3Fe]H0
UIFE00X0e]KF0`3GemL30=KFeP030?ooo`2FUYH0nOWi02l0nOWi00<0UYJF0?ooo`3>c/h00`3>c/h3
03hn?PH0c/k>00<0oooo09JFUP3Ph>002@3Ph>020?ooo`80h>3P0P3oool05@3Ph>00JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3P
h0030?ooo`3Ph>00h>3P00@0h>3P0100a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0
d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0`3Ph>00203Lg=`0DE5A0;Vi^@2][Jd0E5AD0>3Ph01Y
JFT0WYjN5P3Ph>000`2FUYH0oooo0<o?c`030<o?c`<0?Shn1P3?cll0103oool0UYJF09JFUP3oool3
0=cLg0/0gMgM00T0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Lg=`;0=gMg@80
g=cL00<0gMgM0?ooo`2FUYH0<03inOT00`2FUYH0oooo0<o?c`040<o?c`<0?Shn1@3?cll00`3oool0
UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV
0=;BdP3Ph>00fMWI02HV9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103oool0UYJF09JFUP3oool3
0=cLg0/0gMgM00T0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Lg=`;0=gMg@80
g=cL00<0gMgM0?ooo`2FUYH0<03hn?P00`2FUYH0oooo0=CDe0050=CDe0<0?Shn103De=@00`3oool0
UYJF0>3Ph0080>3Ph080/k>c0P3Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00
103Ph>005`34a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00
fMWI02HV9P3Bd]80g=cL04U9B@0000003@d=0=;BdP1YJFT0WYjN00l0h>3P1@0000060>3Ph00409JF
UP3oool0fm_K0=_Kf`<0?Shn203Kfm/01@3oool0UYJF09JFUP3oool0i>CT0080hn?S0`3Ti>@50>?S
h`@0i>CT00P0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@80hn?S0`3Ti>@50>?Sh`@0i>CT
00@0hn?S0>CTi03oool0UYJF<03gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0?Shn0`3Kfm/00`3oool0
UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P01P0a<C404Q8B03IfMT0
fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0
h>3P06UYJ@1nOWh0a<C404Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0`3Zj^X80>WYj@090>[ZjP3oool0UIFE
0?ooo`2EUID0oooo09FEU@3oool0UIFE0080jNWY0`3Zj^X80>WYj@050>[ZjP3YjNT0jNWY0?ooo`2F
UYH0<03emOD00`2FUYH0oooo0>CTi0050>CTi0<0?Shn103Ti>@00`3oool0UYJF0>3Ph00=0>3Ph080
oooo01D0h>3P06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0030>3Ph00I08j>SP000000>3Ph0=WIf@3P
h>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00[Jf]05ADE03P
h>00JFUY07inOP3Ph>00JFUY09jNWP0@0>3Ph0030000003Ph>00h>3P00H0h>3P00<0UYJF0?ooo`3]
kNd00`3]kNd303hn?PH0kNg]00D0oooo09JFUP2FUYH0oooo0>[ZjP020>WYj@<0j^[Z203YjNT02@3Z
j^X0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@020>WYj@<0j^[Z203YjNT01@3Zj^X0jNWY
0>WYj@3oool0UYJF0300m?Cd00<0UYJF0?ooo`3]kNd0103]kNd303hn?PD0kNg]00<0oooo09JFUP3P
h>00303Ph>020;>c/`0F0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>006@3IfMT0
DU9B0=WIf@3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00
fMWI01XJ6P0000003@d=0=;BdP3Ph>00a<C401dM7@3Bd]80603Ph>000`2FUYH0oooo0?Cdm0040?Cd
m0<0?Shn1@3dm?@0103oool0UYJF09JFUP3oool?0>g]k@070?ooo`3]kNd0oooo0>g]k@3oool0kNg]
0?ooo`0A0>g]k@030?ooo`2FUYH0lO7a02l0lO7a00<0UYJF0?ooo`3dm?@00`3dm?@303hn?PH0m?Cd
00<0oooo09JFUP3Ph>00303Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3P
h>000`2FUYH0oooo0?clo0050?clo00303hn?P3lo?`0o?cl00@0o?cl00@0oooo09JFUP2FUYH0oooo
9`3]kNd00`3oool0UYJF0?3`l00_0?3`l00309JFUP3oool0o?cl00@0o?cl00<0?Shn0?clo03lo?`0
1@3lo?`00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0
UYJF0?ooo`3oool03@3oool209JFUP030?ooo`3gmoL0moOg02D0moOg00<0oooo09JFUP3_knl0;`3_
knl00`2FUYH0oooo0?ooo`0=0?ooo`0309JFUP3Ph>00h>3P00l0h>3P01@0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00=@3Ph>000`2FUYH0emOG0?ooo`0<0?ooo`040=OGe`2FUYH0UYJF0?[jnRL0oooo00<0
n_[j09JFUP3_knl0;`3_knl00`2FUYH0emOG0?ooo`0<0?ooo`030=OGe`2FUYH0h>3P0100h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`3Jf]X0TY:B0;Jf]P0;0;Fe]@050;Bd]02?Shl0
f][J0=KFeP2`/;009`2h^;P00`2YZJT0c/k>0>k^kP0_0>k^kP030=[JfP2BTY80][Jf00/0]KFe00<0
];Bd08n?S`3Jf]X04@3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJIL0WYjN00<0oooo0>3Ph03Ph>00=@3Ph>0A0>c/k2/0jn_[<03WinLA
0>c/k140h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?oo
o`030>c/k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`0<0<k>
cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?oo
o`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`050<k>
cP0303hn?P3>c/h0c/k>00@0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`040<k>
cP<0?Shn1@3>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00<0clo?1@0n?Sh40<o?
c`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3De=@303hn?P040=CDe00n?Sh0?Shn
03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooC@3oool0103/k>`0UYJF0?ooo`3Kfm/303hn?P<0fm_K0`0n
?Sh20=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3Ti>@00`0n?Sh0i>CT0>CT
i0030>CTi00J03hn?P3Ti>@0i>CT0>CTi03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00`0
kNg]01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`0<0?Cdm00F0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0
oooo00<0k>c/09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00d0
oooo01D0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo
00@0k>c/0=[JfP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo000004d0000000X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
ooooC@3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
ooooC@3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolm0?ooo`D0EEEE0P3oool505EEE@@0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJEl0
oooo00D0a/K60000000000002@T90=;BdP0O0?ooo`040=oOg`0B4Q800000061PH1P0oooo00@0gmoO
018B4P000000H61P2@3oool0101hN7P0000000T92@3Bd]8P0?ooo`040=oOg`0B4Q800000061PH0T0
oooo00@0N7Qh000000092@T0d];B103oool00`21PH40d];B0?ooo`0:0?ooo`0407QhN00000002@T9
0=;BdTX0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFUR
0?ooo`030::RXP11@D40moOg01h0oooo00<0gmoO01/K6`3Bd]806@3oool00`3Ogml06a/K0=;BdP0;
0?ooo`030=oOg`0K6a/0d];B0080oooo00<0moOg02HV9P3Bd]806`3oool00`3Ogml06a/K0=;BdP0;
0?ooo`030=oOg`0K6a/0d];B00@0oooo00<0a/K6061PH03oool02`3oool00`3Ogml06a/K0=;BdP1:
0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY@P3oool0
0`3Ogml04Q8B000000020000000<00T92@3Bd]80N7Qh000000092@T0b/[:01dM7@000000@D510?Og
m`3oool0a/K60P0000001011@D40moOg0?ooo`36a/H3000000040451@@3gmoL0oooo0::RXP800000
00D0TY:B024Q8@3_knl0oooo0::RXP020000000:0451@@3gmoL0oooo0?_kn`1DE5@0000000l?3`2B
TY800000061PH080oooo00L0a/K6000000000000@D510?Ogm`3oool0a/K600<0000000L0@D510?Og
m`3oool0oooo0=oOg`0K6a/0d];B0080oooo00<0gmoO04i>CP1:BTX00P0000000`2d];@0oooo0<K6
aP050000000K0;Bd]01hN7P0000000T92@3:b/X07AdM00000011@D40moOg0?ooo`3Ogml06a/K0=;B
dP3oool0no_k05ADE00000002@T90<[:bP0M7Ad000000451@@3gmoL0oooo0=oOg`0K6a/0d];B00<0
oooo00<0N7Qh061PH03oool01@3oool03P1hN7P0000000000011@D40moOg0?ooo`3Ogml04Q8B0000
00092@T0d];B0?ooo`3Ogml04Q8B0`0000005@2d];@0oooo0?ooo`3oool0gmoO01/K6`3Bd]80oooo
0?_kn`1DE5@0000000T92@3:b/X07AdM00000011@D40moOg0?ooo`3Ogml06a/K0=;BdP040?ooo`0A
0=oOg`0Q8B40kno_0?ooo`3kno/0E5AD000000092@T0b/[:01dM7@000000@D510?Ogm`3oool0gmoO
01/K6`3Bd]80BP3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0
inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJD@0oooo00<0gmoO01/K6`3Bd]800P3oool0803kno/0E5AD0;Bd]03oool0gmoO024Q8@3_knl0
oooo0?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0<K6aP000000];Bd0?ooo`3gmoL07AdM0;Bd]036a/H0
2@T90=;BdP3oool0N7Qh00l?3`3_knl0oooo0?_kn`1DE5@0];Bd0`3oool01P36a/H0@D510?Ogm`3g
moL07AdM061PH080oooo00d0no_k05ADE02d];@0oooo07QhN02?Shl0a/K60000002d];@0oooo0?Og
m`0M7Ad0];Bd0080oooo00<0gmoO01/K6`3Bd]800P3oool01`3Ogml06a/K0=;BdP3oool0gmoO01/K
6`3Bd]800P3oool00`3Ogml06a/K0=;BdP020?ooo`070?_kn`1DE5@0];Bd0?ooo`3Ogml08B4Q0>o_
k`020?ooo`030=oOg`0K6a/0d];B00<0oooo00D0XZ:R0451@@3clo<0E5AD08n?S`030?ooo`030=oO
g`0K6a/0d];B00<0oooo00<0gmoO01/K6`3Bd]800`3oool05@3kno/0E5AD0451@@3gmoL0oooo07Qh
N02?Shl0gmoO018B4P2d];@0oooo0<K6aP092@T0d];B0=oOg`0Q8B40kno_0?ooo`3Ogml06a/K0=;B
dP020?ooo`030=oOg`0K6a/0d];B00<0oooo00D0XZ:R0451@@3clo<0E5AD08n?S`030?ooo`030=oO
g`0K6a/0d];B00@0oooo00<0no_k05ADE02?Shl00`3oool01@2RXZ80@D510??cl`1DE5@0Shn?00<0
oooo00<0gmoO01/K6`3Bd]80BP3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJD@0oooo00<0gmoO01/K6`3Bd]800P3oool02`3kno/0E5AD0;Bd]03oool0gmoO
024Q8@3_knl0oooo0?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL0103oool02`1hN7P0Shn?
0?ooo`3oool0gmoO024Q8@3_knl0oooo0?_kn`1DE5@0];Bd00<0oooo00H0a/K600T92@3Bd]80oooo
0::RXP1PH6020?ooo`030?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL01P3oool00`3Ogml0
6a/K0=;BdP060?ooo`030?Ogm`0M7Ad0];Bd0080oooo00<0gmoO01/K6`3Bd]800P3oool01`3kno/0
E5AD0;Bd]03oool0gmoO024Q8@3_knl00P3oool00`3Ogml06a/K0=;BdP040?ooo`0407QhN018B4P0
@D510?Ogm`<0oooo00<0gmoO01/K6`3Bd]80103oool00`1hN7P0H61P0?ooo`020?ooo`0307QhN011
@D40moOg00@0oooo00<0XZ:R061PH03oool00P3oool02@1hN7P0Shn?0?ooo`3oool0moOg01dM7@00
00003`l?0>o_k`020?ooo`030=oOg`0K6a/0d];B00@0oooo00@0N7Qh04Q8B011@D40moOg0`3oool0
0`3Ogml06a/K0=;BdP0;0?ooo`0407QhN018B4P0@D510?Ogm`<0oooo00<0gmoO01/K6`3Bd]80BP3o
ool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_
0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00<0
gmoO01/K6`3Bd]800P3oool02`3kno/0E5AD08n?S`3oool0gmoO024Q8@3_knl0oooo0?Ogm`0V9RH0
d];B00<0oooo00<0N7Qh0000000000000`0000000`2?Shl0N7Qh08n?S`020?ooo`070=oOg`0Q8B40
kno_0?ooo`3kno/0E5AD061PH0040?ooo`050::RXP00000000000000001PH6000P3oool00`3gmoL0
9RHV0=;BdP030?ooo`0307QhN0000000000000<0000000H0Shn?0?ooo`3oool0gmoO01/K6`3Bd]84
0?ooo`050=oOg`0B4Q80000000l?3`3_knl00P3oool00`3Ogml06a/K0=;BdP020?ooo`070?_kn`1D
E5@0Shn?0?ooo`3Ogml08B4Q0>o_k`020?ooo`030=oOg`0K6a/0d];B00@0oooo00<0XZ:R0000002?
Shl0103oool00`3Ogml06a/K0=;BdP040?ooo`030=oOg`0K6a/0d];B0080oooo00<0N7Qh061PH03o
ool0103oool02@2RXZ80H61P0?ooo`3oool0no_k05ADE02?Shl0no_k05ADE0020000000300l?3`3_
knl0oooo0080oooo00<0gmoO01/K6`3Bd]80103oool00`2RXZ80000008n?S`040?ooo`030=oOg`0K
6a/0d];B00/0oooo00<0XZ:R0000002?Shl0103oool00`3Ogml06a/K0=;BdP1:0?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool00`3Ogml06a/K0=;B
dP020?ooo`0;0?_kn`1DE5@0De=C0>o_k`36a/H0@D510?Ogm`3oool0moOg02HV9P3Bd]800`3oool0
5@36a/H02@T90=;BdP3oool0a/K600T92@3Bd]80a/K600T92@3Bd]80oooo07QhN00?3`l0kno_0?oo
o`3kno/0E5AD0000001PH600a/K6061PH0040?ooo`070::RXP1PH600oooo0?ooo`3gmoL09RHV0=;B
dP030?ooo`070<K6aP092@T0d];B0?ooo`36a/H02@T90=;BdP020?ooo`030=oOg`0K6a/0d];B00<0
oooo00@0N7Qh000000000000Shn?103oool00`3Ogml06a/K0=;BdP020?ooo`070?_kn`1DE5@0De=C
0>o_k`36a/H0@D510?Ogm`020?ooo`030=oOg`0K6a/0d];B00<0oooo00D0a/K600T92@3:b/X07AdM
0;Bd]0030?ooo`030=oOg`0K6a/0d];B00D0oooo00H0N7Qh061PH03oool0oooo07QhN01PH6050?oo
o`0>0=oOg`0B4Q80];Bd0?ooo`36a/H02@T90=;BdP3kno/0E5AD0;Bd]03oool0a/K60451@@3gmoL2
0?ooo`030=oOg`0K6a/0d];B00<0oooo00D0a/K600T92@3:b/X07AdM0;Bd]0030?ooo`030=oOg`0K
6a/0d];B00X0oooo00D0a/K600T92@3:b/X07AdM0;Bd]0030?ooo`030=oOg`0K6a/0d];B04X0oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU40?ooo`030=oO
g`0K6a/0d];B0080oooo00T0N7Qh0000002/[:`07AdM00T92@3Bd]80oooo0=oOg`0B4Q800`000000
1011@D40moOg0?ooo`36a/H3000000040;Bd]03oool0oooo0::RXP80000000`0TY:B018B4P0?3`l0
clo?018B4P000000[:b/01dM7@000000];Bd0?_kn`1DE5@20000000500l?3`3_knl0oooo0=oOg`0B
4Q800`0000001011@D40moOg0?ooo`36a/H3000000070;Bd]03oool0oooo0?ooo`3Ogml06a/K0=;B
dP020?ooo`030?_kn`1DE5@0];Bd00@0oooo00D0no_k05ADE00000002@T90=;BdP020?ooo`0607Qh
N0000000[:b/01dM7@092@T0d];B0`3oool00`3Ogml06a/K0=;BdP020?ooo`070::RXP0000002@T9
0;:b/P0B4Q80000008n?S`020?ooo`030=oOg`0K6a/0d];B00D0oooo00L0gmoO01/K6`3Bd]80oooo
07QhN011@D40moOg0080oooo00P0N7Qh0;Bd]03oool0gmoO018B4P0000002@T90=;BdP80oooo00H0
gmoO018B4P00000000000451@@3gmoL20?ooo`030=oOg`0K6a/0d];B0080oooo00L0XZ:R00000009
2@T0/[:b018B4P000000Shn?0080oooo00<0gmoO01/K6`3Bd]802@3oool01`2RXZ80000000T92@2b
/[804Q8B0000002?Shl00P3oool00`3Ogml06a/K0=;BdP1:0?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool00`3Ogml06a/K0=;BdP0:0?ooo`030?Og
m`0V9RH0d];B0200oooo00<0moOg02HV9P3Bd]80303oool00`3Ogml06a/K0=;BdP020?ooo`070?_k
n`1DE5@0Shn?0?ooo`2RXZ803`l?0>o_k`0@0?ooo`030=oOg`0K6a/0d];B00/0oooo00<0gmoO01/K
6`3Bd]801P3oool02P1hN7P0H61P0?ooo`3kno/0E5AD0451@@3gmoL0gmoO018B4P2d];@@0?ooo`03
0=oOg`0K6a/0d];B00/0oooo00<0gmoO01/K6`3Bd]804P3oool00`3Ogml06a/K0=;BdP1:0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY@P3oool00`3Ogml0
4Q8B000000020000000300T92@3Bd]80oooo00L0oooo00<0moOg02HV9P3Bd]80803oool00`3gmoL0
9RHV0=;BdP0<0?ooo`040=oOg`0B4Q800000061PH080oooo00H0moOg01dM7@000000QhN702`/;03_
knl20?ooo`030::RXP0?3`l0kno_00/0oooo00@0gmoO018B4P000000H61P2@3oool0101hN7P00000
00T92@3Bd]860?ooo`030=oOg`0Q8B40kno_0080oooo00D0N7Qh00000011@D40L71`0;Bd]00@0?oo
o`040=oOg`0B4Q800000061PH0T0oooo00@0N7Qh000000092@T0d];B4@3oool0101hN7P0000000T9
2@3Bd]9:0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolm0?ooo`D0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo?@3oool505EEE@H0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o03oool01@3kno/0E5AD00000011@D40moOg03l0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWm0?ooo`030?Ogm`0/;2`0kno_0400oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWm0?ooo`030?Ogm`0/;2`0kno_0400
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTn0?ooo`05
07QhN000000000000451@@3gmoL00P3oool03`1hN7P0000000l?3`3_knl0XZ:R00000011@D40VYZJ
00000011@D40`l?303lo?`0000003`l?0>o_k`030?ooo`0=0::RXP000000000000000011@D40L71`
0000001PH600XZ:R00000011@D40moOg07QhN0030000000<0;Bd]03oool0oooo0?ooo`1hN7P00000
061PH02RXZ8000000451@@3gmoL0XZ:R0P0000003@2d];@0oooo0?ooo`1hN7P000000;Bd]03oool0
N7Qh0000000?3`l0kno_0?_kn`1DE5@00P000000102d];@0oooo0?ooo`2RXZ82000000040;Bd]03o
ool0oooo0::RXP80000000d0];Bd0?ooo`1hN7P00000061PH036a/H0000000l?3`2k^k/0?clo0000
000?3`l0kno_00<0oooo00h0XZ:R00000000000000000451@@1`L70000000451@@3gmoL0oooo07Qh
N00000003`l?0>o_k`80oooo00X0no_k05ADE00000002@T90=;BdP3oool0N7Qh0;Bd]03kno/0E5AD
0P0000000`11@D40VYZJ000000020000000J0451@@3gmoL0N7Qh0000002d];@0N7Qh0000001PH600
XZ:R00000011@D40moOg07QhN00000003`l?0>o_k`1hN7P00000061PH01hN7P000000451@@2JVYX0
00000451@@2JVYX30000000>0451@@3gmoL0N7Qh0000002d];@0XZ:R00000011@D40lo?c05ADE000
00002@T90=;BdP2RXZ83000000030451@@3gmoL0oooo0080oooo00/0N7Qh0000000?3`l0jn_[05AD
E01PH600gmoO018B4P11@D40kno_01dM7@020000000K00T92@3Bd]80oooo07QhN0000000@D510?Og
m`3oool0N7Qh0000000?3`l0kno_0<_;b`0o?cl0000000l?3`3_knl0bl_;03lo?`0000003`l?0>o_
k`2RXZ8000000451@@3gmoL0XZ:R008000000100];Bd0?ooo`1hN7P00000061PH02RXZ8000000451
@@33`l<0?clo0000000?3`l0kno_0=oOg`0K6a/0d];B4P3oool00`1EEED0oooo0?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo01L0N7Qh0451@@3gmoL0oooo0=oOg`1CDe<0
moOg0<K6aP1PH600oooo0?Ogm`1mOGd0no_k05eMG@3Bd]80no_k05eMG@3Bd]80bl_;0=;BdP3oool0
a/K608n?S`030?ooo`0B07QhN02d];@0no_k05eMG@3Bd]80moOg02`/;03_knl0no_k05eMG@3Bd]80
a/K600l?3`3_knl0no_k05ADE00?3`l0kno_0P3oool05`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3O
gml06a/K0=;BdP3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP3oool0a/K6061PH03oool0moOg07emO@3O
gml0De=C0080moOg01T09RHV0=;BdP3Ogml06a/K0=;BdP3kno/0GEeM0=;BdP3Ogml06a/K0=;BdP3k
no/0GEeM0=;BdP3gmoL0;2`/0>o_k`1hN7P0Shn?0?ooo`3;bl/0d];B0?ooo`36a/H0Shn?00<0oooo
00h0N7Qh0;Bd]03kno/0GEeM0=;BdP3gmoL0;2`/0>o_k`3oool0a/K6061PH03oool0moOg07emO@@0
oooo03h0[Jf]0>o_k`3oool0gmoO01/K6`1^KVh0moOg0?ooo`2RXZ80Shn?0?ooo`1hN7P0];Bd0?_k
n`1MGEd0d];B0?_kn`1MGEd0d];B0?ooo`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP36a/H0H61P0?oo
o`3gmoL0OGem0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0];Bd07QhN02d];@0no_k05eM
G@3Bd]80no_k05eMG@3Bd]80oooo0?_kn`1MGEd0d];B0::RXP1PH600oooo0?Ogm`2TY:@0N7Qh0;Bd
]03kno/0GEeM0=;BdP<0oooo03P0a/K6061PH03oool0moOg07emO@3Ogml0LW9b0?_kn`1MGEd0d];B
0?Ogm`0V9RH0d];B0?ooo`3gmoL09RHV0=;BdP3gmoL0;2`/0>o_k`3oool0a/K6061PH03oool0moOg
07emO@3;bl/0d];B0?ooo`36a/H0Shn?0<_;b`3Bd]80oooo0<K6aP2?Shl0no_k05eMG@3Bd]80gmoO
01/K6`3Bd]80no_k05eMG@3Bd]80moOg02`/;03_knl0no_k05eMG@3Bd]80bl_;0=;BdP3oool0a/K6
08n?SaD0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030=oOg`0K6a/0d];B00<0oooo00@0a/K60;Bd]02RXZ80];Bd0`3oool01P3kno/0GEeM0=;B
dP3kno/0GEeM0=;BdP80oooo00<0XZ:R0000002d];@00`3oool04@36a/H03`l?0>_[j`1MGEd0d];B
0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP2?Shl0oooo0?_kn`1MGEd0d];B00<0oooo00P0moOg
02`/;03_knl0no_k05eMG@3Bd]80XZ:R061PH080oooo00P0XZ:R08n?S`3kno/0GEeM0=;BdP3oool0
XZ:R0;Bd]0<0oooo00P0gmoO05=CD`3gmoL0oooo0::RXP2?Shl0XZ:R061PH080oooo00@0XZ:R08n?
S`2RXZ80H61P0P3oool01`2RXZ80Shn?0?Ogm`0V9RH0UiNG0451@@3gmoL00`3oool00`2RXZ800000
0;Bd]0030?ooo`0;0<K6aP0?3`l0jn_[05eMG@3Bd]80moOg02`/;03_knl0oooo0::RXP2d];@01`3o
ool0702][Jd0kno_0?ooo`1hN7P0Shn?0<K6aP2d];@0oooo0::RXP2?Shl0oooo0<K6aP0?3`l0jn_[
05eMG@3Bd]80no_k05eMG@3Bd]80oooo0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP2d];@30?oo
o`0G0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0];Bd0<K6aP0?3`l0jn_[05eMG@3Bd]80
no_k05eMG@3Bd]80oooo0?_kn`1MGEd0d];B07QhN02d];@00`3oool01@36a/H03`l?0>_[j`1MGEd0
d];B00<0oooo00<0XZ:R0;Bd]03oool00`3oool01`3;bl/0?clo08n?S`3oool0moOg02`/;03_knl0
0P3oool0202RXZ80Shn?0?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd1@3oool00`2RXZ8000000;Bd]002
0?ooo`080::RXP000000];Bd0?_kn`1MGEd0d];B0::RXP1PH6020?ooo`080::RXP2?Shl0moOg02`/
;03_knl0no_k05eMG@3Bd]820?ooo`030::RXP000000];Bd01D0oooo00<0EEEE0?ooo`3oool0103o
ool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg
0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030::RXP11@D40moOg00D0oooo00l0
XZ:R0000000000000000061PH03kno/0GEeM0=;BdP3kno/0GEeM0=;BdP3Ogml04Q8B0451@@3gmoL0
1@3oool0403gmoL07AdM00T92@3Bd]80moOg02`/;03_knl0no_k05eMG@3Bd]80XZ:R08n?S`3oool0
no_k05eMG@3Bd]830?ooo`080?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP2?Shl20?ooo`0707Qh
N02?Shl0no_k05eMG@3Bd]80oooo0::RXP0300000009061PH03Ogml0De=C0?Ogm`3oool0XZ:R08n?
S`2RXZ80Shn?0080oooo00@0N7Qh08n?S`2RXZ80Shn?0P3oool01P1hN7P0Shn?0?Ogm`0V9RH0?clo
061PH080oooo00@0gmoO018B4P11@D40moOg1@3oool02@3gmoL07AdM00T92@3Bd]80moOg02`/;03_
knl0oooo0::RXP0300000003061PH03oool0oooo0080oooo00X0[Jf]0>o_k`3gmoL0;2`/0>o_k`3k
no/0X:2P0?ooo`2RXZ80Shn?0P3oool03`3gmoL07AdM00T92@3Bd]80no_k05eMG@3Bd]80oooo0?Og
m`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP030000000H061PH03gmoL0;2`/0>o_k`3kno/0GEeM0=;B
dP3kno/0E5AD0;Bd]03oool0moOg01dM7@092@T0d];B0?_kn`1MGEd0d];B0?ooo`3kno/0GEeM0=;B
dP21PH40d];B103oool0103gmoL07AdM00T92@3Bd]830?ooo`0=0::RXP00000000000000001PH600
oooo0<K6aP0?3`l0kno_0?ooo`3gmoL0;2`/0>o_k`020?ooo`070::RXP2?Shl0moOg02`/;03_knl0
oooo0::RXP030000000@061PH03Ogml04Q8B0451@@3gmoL0oooo0=oOg`0B4Q80@D510?Ogm`3oool0
no_k05eMG@3Bd]80XZ:R08n?S`80oooo00`0N7Qh08n?S`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3O
gml04Q8B0451@@3gmoLF0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
0`3/k>`0l?3`08j>SP050:2PX00309nOW`2PX:00X:2P00@0X:2P01H0T92@0>c/k01YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<0XZ:R
0451@@3gmoL01@3oool04036a/H0Shn?0?ooo`1hN7P0Shn?0?_kn`1MGEd0d];B0?_kn`1MGEd0d];B
0<K6aP2?Shl0oooo0861P@3Bd]830?ooo`0A0::RXP2?Shl0no_k05eMG@3>c/h0E5AD0000001PH600
moOg02HV9P3Bd]80gmoO024Q8@3_knl0gmoO01/K6`3Bd]800P3oool0<`3kno/0E5AD0000001PH600
moOg02HV9P3Bd]80gmoO05=CD`3gmoL0gmoO018B4P2d];@0fm_K05eMG@3Bd]80oooo0<K6aP2?Shl0
oooo07QhN02?Shl0gmoO018B4P2d];@0no_k05ADE02d];@0gmoO05=CD`3gmoL0gmoO018B4P2d];@0
gmoO05=CD`3gmoL0gmoO018B4P2d];@0moOg02`/;03_knl0bl_;0=;BdP3oool0a/K608n?S`3oool0
PH610=;BdP030?ooo`0>0::RXP2?Shl0no_k05eMG@3>c/h0E5AD0000002?Shl0oooo0<K6aP2?Shl0
oooo07QhN02?Shl40?ooo`040:f][@3_knl0XZ:R08n?S`80oooo03P0Z:RX0=;BdP2RXZ80Shn?0?oo
o`2RXZ80Shn?0?_kn`1MGEd0d];B0=_Kf`1MGEd0d];B0?ooo`3gmoL07AdM061PH03gmoL09RHV0=;B
dP36a/H0Shn?0?ooo`1hN7P0Rh^;05ADE0000000Shn?0?Ogm`0M7Ad0H61P0?_kn`1MGEd0d];B0::R
XP2?Shl0no_k05eMG@3Bd]80fm_K05eMG@3Bd]80oooo07QhN0092@T0d];B0::RXP2?Shl0oooo07Qh
N02d];@0XZ:R08n?S`3kno/0GEeM0=;BdP<0oooo03P0a/K608n?S`3oool0N7Qh08n?S`3gmoL0;2`/
0>OWi`2TY:@0no_k05ADE0000000H61P0?ooo`3Ogml04Q8B0;2`/01DE5@0000008n?S`3oool0a/K6
08n?S`3oool0N7Qh08n?S`36a/H0Shn?0?ooo`21PH40d];B0<K6aP2?Shl0oooo0861P@3Bd]80N7Qh
00T92@3Bd]80gmoO05=CD`3gmoL0gmoO018B4P2`/;00E5AD0000001PH600moOg02HV9P3Bd]80a/K6
08n?S`3oool0PH610=;BdQD0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?oo
o`030>c/k02FUYH0g]kN00`0dm?C01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<0XZ:R0451@@3gmoL01P3oool03`1h
N7P000000451@@3gmoL0no_k05eMG@3Bd]80no_k05eMG@3Bd]80no_k05ADE0000000UYJF0=_Kf`03
0?ooo`040=oOg`0B4Q8000000451@@80moOg00L0;2`/0;Jf]P000000@D510?Ogm`3oool0a/K60080
000000<02@T90=;BdP3oool00P3oool0203gmoL0;2`/0;Jf]P000000@D510?Ogm`3oool0XZ:R0P00
00000`2?Shl0oooo0<K6aP020000000;0;Bd]03oool0N7Qh00000011@D40moOg0=oOg`1CDe<0L71`
0000001PH6000P3oool0102RXZ8000000000002?Shl20?ooo`0@0::RXP000000000008n?S`3oool0
moOg02`/;03_knl0XZ:R00000011@D40lo?c05ADE0000000UYJF0=_Kf`<0oooo00@0gmoO018B4P00
0000@D510P3gmoL0200/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg103oool0102][Jd0inOW
02`/;03_knl20?ooo`0;0?Ogm`1mOGd0XZ:R08n?S`3oool0gmoO018B4P000000@D510?Ogm`36a/H0
0P0000006`2d];@0moOg02`/;02f][H000000451@@3gmoL0oooo07QhN0000000@D510?Ogm`3Ogml0
De=C09ZJVP000000@D510;jn_P000000@D510?Ogm`3Ogml04Q8B00000011@D40moOg0<K6aP020000
000>0;Bd]03gmoL09RHV0=;BdP3oool0N7Qh0000000?3`l0kno_0=oOg`0B4Q8000000451@@3gmoL4
0?ooo`0<07QhN0000000@D510??cl`1DE5@0000009:BTP0B4Q80Shn?0?Ogm`0/;2`0][Jf0P000000
6P2?Shl0oooo0?Ogm`0/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg0?_kn`1DE5@0000009JF
UP3Kfm/0no_k05ADE0000000UYJF0=_Kf`3gmoL09RHV0=;BdP3oool0XZ:R0P0000003@2?Shl0oooo
0?Ogm`0/;2`0][Jf00000011@D40moOg0?_kn`1DE5@0000009JFUP3Kfm/05@3oool00`1EEED0oooo
0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>CTi03MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<0gmoO01/K6`3Bd]800`3o
ool00`2XZ:P0d];B0?ooo`040?ooo`060?_kn`1MGEd0d];B0?_kn`1MGEd0d];B5P3oool00`3kno/0
GEeM0=;BdP0@0?ooo`030861P@3Bd]80oooo00D0oooo00<0gmoO05=CD`3gmoL03`3oool00`3gmoL0
;2`/0>o_k`0M0?ooo`030:f][@2BTY80H61P00@0oooo00<0[Jf]09:BTP2?Shl01`3oool01P21PH40
d];B0?ooo`3gmoL0;2`/0>o_kaL0oooo00<0PH610=;BdP3oool0G@3oool00`1EEED0oooo0?ooo`04
0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3L
g=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo00L0N7Qh0451@@3gmoL0oooo0::RXP09
2@T0d];B00D0oooo00H0N7Qh00T92@3Bd]80N7Qh00T92@3Bd]8F0?ooo`0307QhN0092@T0d];B0100
oooo00<0Z:RX0=;BdP3oool0103oool0103gmoL07AdM0451@@3gmoL>0?ooo`040?_kn`1DE5@03`l?
0>o_kad0oooo00<0XZ:R00T92@3Bd]80103oool00`3Ogml04Q8B08n?S`070?ooo`060:RXZ03Bd]80
no_k05ADE00?3`l0kno_5`3oool00`2XZ:P0d];B0?ooo`1M0?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03L
g=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY?P3oool01P2RXZ8000000000002?Shl0Z:RX0=;BdPD0
oooo00H0moOg02HV9P3Bd]80moOg02HV9P3Bd]8F0?ooo`030?_kn`1MGEd0d];B01P0oooo00<0gmoO
05=CD`3gmoL03`3oool00`3Ogml08B4Q0>o_k`0K0?ooo`040?_kn`1DE5@00000061PH0H0oooo00@0
N7Qh00000011@D40moOg203oool00`3Ogml08B4Q0>o_k`0J0?ooo`030?Ogm`0V9RH0d];B0340oooo
00<0moOg02HV9P3Bd]809P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo?P3oool405EEE@H0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom10?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolm0?ooo`D0EEEE
1P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom80?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M
0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?oo
o`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0
gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]1P2EUID06@3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo
0?ooo`020?ooo`050>c/k02FUYH0oooo0?Ogm`3]kNd01P3oool06P3Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2E
UID06@3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`050>c/
k02FUYH0oooo0?Ogm`3]kNd01P3oool06P3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?@3oool5
05EEE@H0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo
0?ooo`020?ooo`050>c/k02FUYH0oooo0?Ogm`3]kNd01P3oool06P3Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJBH0oooo00D0gmoO018B4P0000003`l?0>o_k`3o0?oooaH0oooo00<0EEEE0?ooo`3oool0103o
ool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3AdM40clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJBH0oooo00D0gmoO018B4P0000000000061PH03o0?oooaH0oooo00<0EEEE0?oo
o`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`050>c/k02FUYH0oooo0?Ogm`3]kNd01P3oool0
6P3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJBP0oooo00@0gmoO018B4P092@T0d];Bo`3ooolE0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7P3oool00`36a/H00000
000000020000000308n?S`3oool0oooo0080oooo00@0moOg01dM7@000000Shn?0`3oool04`3Ogml0
4Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K60000000?3`l0kno_0?_kn`1DE5@0
00000:b/[00M7Ad00P0000000`2?Shl0oooo0?ooo`020?ooo`060?Ogm`0M7Ad00000000000092@T0
d];B0P3oool0203gmoL07AdM0000002d];@0moOg01dM7@000000];Bd0P3oool01P36a/H000000000
00000000@D510?Ogm`H0oooo00`0XZ:R000000000000@D510>o_k`0M7Ad02@T90=;BdP3oool0N7Qh
0000001PH6020?ooo`030::RXP000000H61P00<0oooo00L0XZ:R000000000000@D5109ZJVP000000
H61P00D0oooo00P0gmoO018B4P092@T0d];B0?ooo`1hN7P00000061PH080oooo00<0XZ:R0000001P
H6000P3oool01`3Ogml04Q8B000000092@T0d];B0?ooo`1hN7P00`0000001`2?Shl0oooo0?ooo`3g
moL07AdM00T92@3Bd]800`3oool0202RXZ80000000000011@D40kno_01dM7@092@T0d];B0P3oool0
1P36a/H00000000000000000@D510?Ogm`80oooo00P0gmoO018B4P0000002@T90<[:bP0M7Ad00000
0;Bd]0<0oooo00<0N7Qh0000001PH6001@3oool01@1hN7P00000000000000000Shn?00H0oooo00<0
moOg01dM7@0000001@0000000`2?Shl0oooo0?ooo`030?ooo`060?Ogm`0M7Ad00000000000092@T0
d];B103oool01P36a/H00000000000000000@D510?Ogm`<0oooo01H0N7Qh000000000000@D5109ZJ
VP000000H61P0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K60000
000?3`l0kno_0P3oool01@1hN7P00000000000000000Shn?00<0oooo00<0N7Qh0000001PH6000P3o
ool00`2RXZ800000061PH0020?ooo`070=oOg`0B4Q80000000T92@3Bd]80no_k05ADE00300000003
0451@@3gmoL0oooo00h0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTM0?ooo`030::RXP000000000000@0000000D0Shn?0?ooo`3oool0oooo0<K6aP020000000D
0451@@3gmoL0oooo0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K6
0000000?3`l0kno_0?_kn`1DE5@60000000408n?S`3oool0oooo0<K6aPH0000000`0];Bd0?ooo`3g
moL07AdM0000002d];@0moOg01dM7@000000];Bd0?ooo`36a/H50000000300l?3`3_knl0oooo00<0
oooo00<0a/K6000000000000100000001P11@D40moOg0?ooo`1hN7P00000061PH080oooo00<0XZ:R
0000001PH6000P3oool00`2RXZ8000000000000400000003061PH03oool0oooo00<0oooo00P0gmoO
018B4P092@T0d];B0?ooo`1hN7P00000061PH080oooo00<0XZ:R0000001PH6000P3oool01P1hN7P0
0000000000092@T0d];B07QhN0D0000000H0Shn?0?ooo`3gmoL07AdM00T92@3Bd]820?ooo`030<K6
aP000000000000@0000000@0@D510?Ogm`3oool0a/K61@000000100?3`l0kno_0?ooo`1hN7P20000
000500T92@3:b/X07AdM0000002d];@00P3oool01@3gmoL07AdM0000000?3`l0kno_00<0oooo00<0
N7Qh0000000000000`0000000`2?Shl0oooo0?ooo`030?ooo`030?Ogm`0M7Ad0000000H0000000D0
@D510?Ogm`3oool0oooo0<K6aP06000000040;Bd]03oool0oooo0<K6aPD0000000@03`l?0>o_k`3o
ool0a/K61P0000004P1PH600oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?oo
o`36a/H0000000l?3`3_knl0oooo07QhN0D0000000H0Shn?0?ooo`3oool0N7Qh0000001PH6020?oo
o`030::RXP000000H61P0080oooo00H0N7Qh0000000000002@T90<k>cP1DE5@5000000030451@@3g
moL0oooo00d0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL
0?ooo`040?Ogm`0M7Ad0000008n?S`80oooo00@0XZ:R0000000?3`l0kno_0P3oool01@1hN7P00000
000000092@T0d];B0080oooo0240gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00l?3`3_knl0oooo
0<K6aP0000003`l?0>o_k`3kno/0E5AD0000000?3`l0kno_0?ooo`2RXZ80000000l?3`3_knl0no_k
05ADE0000000@D510?Ogm`3oool0N7Qh00000011@D400P3gmoL0400M7Ad000000;Bd]03gmoL07AdM
0000002d];@0moOg01dM7@000000Shn?0?ooo`3kno/0E5AD0000002?Shl40?ooo`0=0::RXP000000
@D510?Ogm`3kno/0E5AD00000011@D40moOg0?ooo`1hN7P00000061PH0020?ooo`0=0::RXP000000
H61P0?ooo`3gmoL07AdM0000001PH600oooo0?Ogm`0M7Ad00000061PH0050?ooo`080=oOg`0B4Q80
2@T90=;BdP3oool0N7Qh0000001PH6020?ooo`0F0::RXP000000H61P0?ooo`3kno/0E5AD0000002?
Shl0oooo0=oOg`0B4Q8000000;Bd]03oool0a/K60000000?3`l0kno_0?Ogm`0M7Ad02@T90=;BdP80
oooo00P0XZ:R00000011@D40moOg0?_kn`1DE5@000000451@@80moOg01407AdM0000002?Shl0oooo
0?_kn`1DE5@0000008n?S`3kno/0E5AD0000002?Shl0oooo0?Ogm`0M7Ad000000;Bd]0020?ooo`05
0::RXP000000M7Ad0000002d];@00P3oool02@3Ogml04Q8B0000002d];@0oooo0<K6aP0000003`l?
0>o_k`040?ooo`040?Ogm`0M7Ad000000;Bd]0<0oooo00h0gmoO018B4P000000Shn?0?ooo`3kno/0
E5AD00000011@D40moOg0?ooo`1hN7P000000451@@80moOg00`07AdM0000002?Shl0oooo0?_kn`1D
E5@0000008n?S`3oool0N7Qh00000011@D420?Ogm`0O01dM7@000000H61P0?ooo`3Ogml04Q8B00T9
2@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K60000000?3`l0kno_0=oOg`0B4Q8000000;Bd
]03oool0a/K60000000?3`l0kno_0?ooo`1hN7P00000061PH0020?ooo`0B0::RXP000000H61P0?oo
o`3kno/0E5AD0000002?Shl0oooo0<K6aP0000002@T90=;BdP3oool0moOg01dM7@092@T0d];B3P3o
ool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo00@0a/K6
0000000?3`l0kno_0P3oool02`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0;Jf]P000000
H61P0080oooo0180gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6aP000000
3`l?0>o_k`3kno/0E5AD0000002?Shl20?ooo`080?Ogm`0M7Ad02@T90=;BdP3Ogml04Q8B00T92@3B
d]820?ooo`0@0?Ogm`0M7Ad02@T90=;BdP3gmoL07AdM0000002d];@0moOg01dM7@000000];Bd0=oO
g`0B4Q802@T90=;BdPT0oooo00d0a/K6000000000000];Bd0?ooo`2RXZ8000000451@@3gmoL0oooo
07QhN0000000H61P0080oooo00P0XZ:R0000001PH600oooo0=oOg`0B4Q802@T90=;BdP80oooo00<0
XZ:R0000001PH6001@3oool0203Ogml04Q8B00T92@3Bd]80oooo07QhN0000000H61P0P3oool03@2R
XZ800000061PH03oool0no_k05ADE0000000];Bd0?ooo`2RXZ8000000451@@3gmoL01@3oool0103g
moL07AdM00T92@3Bd]820?ooo`0=0<K6aP00000000000;Bd]03oool0XZ:R00000011@D40moOg0=oO
g`0B4Q802@T90=;BdP050?ooo`0F0?_kn`1DE5@000000;Bd]03oool0moOg01dM7@000000];Bd0?oo
o`3kno/0E5AD00l?3`3?cll04Q8B0451@@3gmoL0oooo0::RXP000000@D510?Ogm`T0oooo00@0moOg
01dM7@000000];Bd103oool02036a/H0000000l?3`3_knl0gmoO018B4P092@T0d];B0P3oool0203g
moL07AdM00T92@3Bd]80gmoO018B4P092@T0d];B1P3oool00`1hN7P00000061PH0020?ooo`0F07Qh
N0000000H61P0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K60000
000?3`l0kno_0::RXP000000@D510?Ogm`H0oooo00<0N7Qh0000001PH6000P3oool0202RXZ800000
061PH03oool0no_k05ADE0000000];Bd103oool01P36a/H000000000000000002@T90=;BdPh0oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTQ0?ooo`0P0?Ogm`0M
7Ad0000000l?3`3_knl0oooo0::RXP000000H61P0?Ogm`0M7Ad03`l?0>o_k`3oool0gmoO018B4P09
2@T0d];B0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0<K6aP0000003`l?0>o_k`3kno/0E5AD0000002d
];@20?ooo`080?Ogm`0M7Ad000000;Bd]03Ogml04Q8B00l?3`3_knl20?ooo`0@0?Ogm`0M7Ad00000
0;Bd]03gmoL07AdM0000002d];@0moOg01dM7@000000];Bd0=oOg`0B4Q803`l?0>o_k`T0oooo00<0
no_k05ADE0000000100000001P11@D40moOg0?ooo`1hN7P00000061PH080oooo00P0XZ:R0000001P
H600oooo0=oOg`0B4Q803`l?0>o_k`80oooo00<0XZ:R0000001PH6001@3oool0203Ogml04Q8B00T9
2@3Bd]80oooo07QhN0000000H61P0P3oool02P2RXZ800000061PH03oool0no_k05ADE0000000];Bd
0?ooo`2RXZ87000000050;Bd]03gmoL07AdM0000002d];@00P3oool00`3kno/0E5AD000000040000
00060451@@3gmoL0gmoO018B4P0?3`l0kno_1@3oool04`3kno/0E5AD0000002d];@0oooo0?Ogm`0M
7Ad000000;Bd]03oool0gmoO018B4P11@D40lo?c05ADE0092@T0d];B0?ooo`2RXZ801`0000000`2d
];@0oooo0?ooo`020?ooo`040?Ogm`0M7Ad000000;Bd]0@0oooo00P0gmoO018B4P092@T0d];B0=oO
g`0B4Q803`l?0>o_k`80oooo00P0moOg01dM7@000000];Bd0=oOg`0B4Q803`l?0>o_k`H0oooo00<0
N7Qh0000001PH6000P3oool04`2RXZ800000061PH03oool0gmoO018B4P092@T0d];B0?ooo`3Ogml0
4Q8B00T92@3Bd]80oooo0<K6aP0000003`l?0>o_k`2RXZ801`0000001@2d];@0oooo07QhN0000000
H61P0080oooo00P0XZ:R0000001PH600oooo0?_kn`1DE5@000000;Bd]080oooo00<0XZ:R00000000
00000`0000000`1PH600oooo0?ooo`0=0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY7`3oool00`2RXZ800000000000020000000M08n?S`3oool0no_k05ADE000
0000];Bd0?ooo`1hN7P000000;Bd]03oool0gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00T92@3B
d]80oooo0<K6aP0000003`l?0>o_k`3kno/0E5AD0000002d];@00P3oool0203gmoL07AdM00T92@3B
d]80gmoO018B4P092@T0d];B0P3oool0403gmoL07AdM00T92@3Bd]80moOg01dM7@000000];Bd0?Og
m`0M7Ad000000;Bd]03Ogml04Q8B00T92@3Bd]8<0?ooo`0:0=oOg`0B4Q80000000000011@D40moOg
0?ooo`1hN7P00000061PH080oooo00P0XZ:R0000001PH600oooo0=oOg`0B4Q802@T90=;BdP80oooo
00<0XZ:R0000001PH6001@3oool0203Ogml04Q8B00T92@3Bd]80oooo07QhN0000000H61P0P3oool0
2P2RXZ800000061PH03oool0no_k05ADE0000000];Bd0?ooo`36a/H60000000600T92@3Bd]80moOg
01dM7@000000];Bd1@3oool02P3Ogml04Q8B000000000000@D510?Ogm`3Ogml04Q8B00T92@3Bd]85
0?ooo`0C0?_kn`1DE5@000000;Bd]03oool0moOg01dM7@000000];Bd0?ooo`1hN7P0000008n?S`3o
ool0XZ:R0000002?Shl0oooo0<K6aP060000000300T92@3Bd]80oooo00<0oooo00@0moOg01dM7@00
0000];Bd103oool0203gmoL07AdM00T92@3Bd]80gmoO018B4P092@T0d];B0P3oool0203gmoL07AdM
00T92@3Bd]80gmoO018B4P092@T0d];B1P3oool00`1hN7P00000061PH0020?ooo`0C0::RXP000000
H61P0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q802@T90=;BdP3oool0a/K60000000?3`l0
kno_0<K6aP060000000600T92@3Bd]80oooo07QhN0000000H61P0P3oool02`2RXZ800000061PH03o
ool0no_k05ADE0000000];Bd0?ooo`3gmoL07AdM00<0000000<0H61P0?ooo`3oool03`3oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAd0oooo00<0moOg01dM7@00
00000`0000004P2d];@0oooo0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6aP000000@D510?Ogm`3O
gml04Q8B0000001PH600oooo0<K6aP80000002/0Shn?0?ooo`2RXZ80000000l?3`3_knl0no_k05AD
E0000000@D510?Ogm`3oool0XZ:R0000000?3`l0kno_0?_kn`1DE5@000000451@@3gmoL0oooo07Qh
N00000003`l?0>o_k`3gmoL07AdM0000002d];@0moOg01dM7@000000];Bd0?Ogm`0M7Ad0000008n?
S`3oool0moOg01dM7@092@T0d];B00@0oooo01`0moOg01dM7@0?3`l0kno_0?ooo`2RXZ8000000451
@@3gmoL0oooo07QhN00000002@T90=;BdP3kno/0E5AD0000001PH600oooo0?Ogm`0M7Ad00000061P
H03oool0moOg01dM7@000000H61P1@3oool0803Ogml04Q8B00T92@3Bd]80oooo07QhN00000002@T9
0=;BdP3kno/0E5AD0000001PH600oooo0?_kn`1DE5@000000;Bd]03oool0gmoO018B4P0?3`l0kno_
0?ooo`3Ogml04Q8B00l?3`3_knl0moOg01dM7@000000H61P0P3oool0203gmoL07AdM00l?3`3_knl0
oooo0::RXP000000@D510P3gmoL08`0M7Ad0000008n?S`3oool0moOg01dM7@092@T0d];B0?_kn`1D
E5@000000;Bd]03oool0moOg01dM7@000000];Bd0?Ogm`0M7Ad02@T90=;BdP3oool0gmoO018B4P11
@D40moOg0=oOg`0B4Q803`l?0>o_k`3oool0gmoO018B4P0?3`l0kno_00@0oooo00@0moOg01dM7@00
0000];Bd103oool06`3gmoL07AdM00T92@3Bd]80no_k05ADE0000000@D510?Ogm`3oool0N7Qh0000
000?3`l0kno_0?Ogm`0M7Ad0000008n?S`3oool0moOg01dM7@092@T0d];B0?ooo`1hN7P00000061P
H0020?ooo`0:0::RXP000000H61P0?ooo`3Ogml04Q8B0000001PH600oooo0<K6aP80000002H0Shn?
0?ooo`2RXZ80000000l?3`3_knl0gmoO018B4P0?3`l0kno_0?ooo`3Ogml04Q8B00l?3`3_knl0oooo
07QhN00000002@T90=;BdP3kno/0E5AD0000001PH600oooo0?_kn`1DE5@000000;Bd]03oool0gmoO
018B4P0?3`l0kno_0?ooo`2RXZ800000061PH0l0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFTM0?ooo`0507QhN0000000000000l?3`3_knl0103oool00`2RXZ80
0000061PH0020?ooo`060?Ogm`0M7Ad02@T90=;BdP3Ogml04Q8B2P000000101PH600oooo0?_kn`1D
E5@60000000508n?S`3oool0oooo0=oOg`0B4Q801@000000302d];@0oooo0?Ogm`0M7Ad000000;Bd
]03gmoL07AdM0000002d];@0oooo0<K6aPD0000000<0@D510?Ogm`3oool0103oool00`1hN7P00000
0000000300000004061PH03oool0oooo07QhN0H0000000@0Shn?0?ooo`3oool0XZ:R1P0000000`1P
H600oooo0?ooo`030?ooo`060=oOg`0B4Q802@T90=;BdP3oool0N7Qh1P0000000`2?Shl0no_k05AD
E0030000000300l?3`3_knl0N7Qh00D0000000@0Shn?0?ooo`3gmoL07AdM0`0000000`11@D40moOg
07QhN00500000004061PH03oool0oooo0<K6aPD0000000<0@D510??cl`1DE5@00`000000400?3`l0
inOW01dM7@000000];Bd0<K6aP000000@D510?Ogm`3oool0no_k05ADE0000000];Bd0?ooo`1hN7P5
0000000308n?S`3oool0oooo00<0oooo00@0moOg01dM7@000000];Bd103oool01`3Ogml04Q8B00T9
2@3Bd]80oooo0=oOg`0B4Q801@000000102d];@0oooo0?ooo`36a/H5000000060451@@3gmoL0oooo
07QhN0000000H61P0P3oool01P2RXZ800000061PH03oool0gmoO018B4PX0000000@0H61P0?ooo`3o
ool0N7Qh1@000000102?Shl0oooo0?ooo`1hN7P60000000308n?S`3kno/0E5AD00<0000000<03`l?
0>_[j`1DE5@01@0000000`2?Shl0oooo0?ooo`0=0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUY703oool0103kno/0E5AD0000002?Shl20?ooo`0707QhN0000000
H61P0?Ogm`0M7Ad000000;Bd]0030?ooo`0F07QhN0000000Shn?0=oOg`0B4Q802@T90;:b/P0B4Q80
000000l?3`3_knl0gmoO018B4P0000003`l?0>o_k`3oool0no_k05ADE0000000/;2`05ADE0800000
00<0Shn?0?ooo`3oool00P3oool01P3gmoL07AdM0000000000002@T90=;BdP80oooo00P0moOg01dM
7@000000];Bd0?Ogm`0M7Ad000000;Bd]080oooo00H0a/K600000000000000000451@@3gmoL70?oo
o`0607QhN0000000000000000011@D40moOg0P3oool0101hN7P00000061PH01hN7P200000003061P
H03oool0oooo0080oooo00L0XZ:R000000000000@D5109ZJVP000000H61P00D0oooo00T0gmoO018B
4P092@T0d];B0?ooo`1hN7P00000061PH01hN7P00P000000101PH600oooo0?_kn`1DE5@300000004
00l?3`3_knl0oooo0::RXP<0000000`0Shn?0?ooo`3oool0moOg01dM7@092@T0UiNG000000092@T0
d];B0?ooo`1hN7P3000000030451@@3gmoL0oooo0080oooo00P0a/K600000000000000000451@@3g
moL0no_k05ADE0<0000000P03`l?0>OWi`0M7Ad000000;Bd]01hN7P0000008n?S`<0oooo00<0N7Qh
0000001PH6000P3oool01@2RXZ800000000000000000Shn?00H0oooo00@0moOg01dM7@000000];Bd
103oool0102RXZ80000000l?3`3_knl20?ooo`060?Ogm`0M7Ad00000000000092@T0d];B103oool0
1P36a/H00000000000000000@D510?Ogm`80oooo00<0N7Qh0000001PH6000P3oool04@2RXZ800000
061PH03oool0gmoO018B4P092@T0/[:b018B4P0000003`l?0>o_k`3Ogml04Q8B0000000?3`l0kno_
00<0oooo00D0XZ:R000000000000000008n?S`030?ooo`0407QhN0000000H61P07QhN080000000@0
H61P0?ooo`3kno/0E5AD0`000000100?3`l0kno_0?Ogm`0M7Ad300000003061PH03oool0oooo00h0
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`090?_k
n`1DE5@0000008n?S`3oool0moOg01dM7@000000Shn?01P0oooo00@0no_k05ADE0000000];Bd403o
ool0103gmoL07AdM0000002d];@Z0?ooo`030::RXP000000H61P00D0oooo00@0gmoO018B4P092@T0
d];B2P3oool0103kno/0E5AD0000002d];@R0?ooo`040?_kn`1DE5@000000;Bd]1/0oooo00@0moOg
01dM7@000000];Bd0`3oool01036a/H000000000002?Shlo0?ooo`040?_kn`1DE5@000000;Bd]1P0
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTM0?ooo`030::R
XP000000000000<0000000<03`l?0>o_k`3oool05`3oool0103kno/0E5AD0000002d];@@0?ooo`08
0?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@V0?ooo`030::RXP000000H61P00D0oooo00@0
gmoO018B4P092@T0d];B2P3oool0103kno/0E5AD0000002d];@R0?ooo`090?_kn`1DE5@000000;Bd
]03oool0moOg01dM7@000000];Bd01H0oooo00<0moOg01dM7@0000001P0000000`11@D40moOg0?oo
o`0n0?ooo`040?_kn`1DE5@000000;Bd]1P0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTN0?ooo`060<K6aP00000000000000000?3`l0kno_6@3oool0103kno/0
E5AD0000002d];@@0?ooo`080?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@V0?ooo`030::R
XP000000H61P00D0oooo00@0gmoO018B4P092@T0d];B303oool00`1hN7P0];Bd0?ooo`0S0?ooo`07
07QhN02d];@0oooo0?Ogm`0M7Ad000000;Bd]00F0?ooo`030?Ogm`0M7Ad0000000D0000000<0Shn?
0?ooo`3oool0@@3oool00`1hN7P0];Bd0?ooo`0G0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3oooln0?ooo`@0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0emOG00`0oooo
01H0emOG09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJA/0ooooo`00000M000000D0oooo1@1EEED20?ooo`D0EEEE103oool00`3/k>`0oOgm
09>CT`050;Rh^0030;Fe]@2h^;P0^;Rh00@0^;Rh01H0Ti>C0>c/k01YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom=0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0
n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd
0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom=0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0
n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom=0?ooo`030>c/k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF
0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom=0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF
0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3>c/h00`0n?Sh0c/k>
0<k>cP030<k>cP0J03hn?P3>c/h0c/k>0<k>cP3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`040>c/k02FUYH0oooo
0<o?c`<0?Shn0`3?cll303hn?P80clo?01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`02
0=CDe0<0?Shn00@0e=CD03hn?P0n?Sh0?Shn0`3De=@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02F
UYH0oooo00<0fm_K1@0n?Sh40=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0103T
i>@303hn?PD0i>CT01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`050>g]k@0303hn?P3]
kNd0kNg]00@0kNg]01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`0<0?Cdm00F0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02F
UYH0oooo00d0oooo01D0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooodd0oooo00@0k>c/0=[JfP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`000000moOg09ZJVP3o07YjNV00NWYj01<0VYZJ
0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`000000moOg0;no_`3o0;no_f40_kno
0180moOg0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`3Shn<0o`3Shn=Q0>?S
h`0B0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003alO40oooo0?l0oooo6P3o
ool00`3Shn<0YJFU07IfMP0307IfMP0407MgM`1fMWH0MgMg07MgM`T0MWIf00H0YJFU0>?Sh`3oool0
oooo0>?Sh`2UYJD407IfMP0407MgM`1fMWH0MgMg07MgM`T0MWIf00H0YJFU0>?Sh`3oool0oooo0>_[
j`2UYJD405IFEP0405MGE`1FEUH0EeMG05MGE`T0EUIF00<0YJFU0>_[j`3oool00`3oool04P3alO40
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000hN7Q0?clo03kno/20?clo0030?_kn`3l
o?`0o?cl00@0o?cl00<0no_k0?clo03lo?`00P3lo?`00`3kno/0o?cl0?_kn`050?clo0030?_kn`3l
o?`0no_k0640no_kWP3lo?`00`2UYJD0MWIf08f=S@0@08f=S@0607IfMP2UYJD0o?cl0?clo02UYJD0
MWIf4@2=SHd01P1fMWH0YJFU0?clo03kno/0YJFU05IFEQ40Fe]K00<0EUIF0:FUY@3lo?`00`3lo?`0
4P3QhN40000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000h>3P0?Ogm`3o0?OgmaX0moOg
00<0MWIf08f=S@2RXZ80402RXZ801P2=SHd0MWIf0?Ogm`3gmoL0MWIf08f=SA40XZ:R00P0SHf=07If
MP3gmoL0moOg05IFEP1KFe/0Kfm_075aL@d0LW9b00@0LG5a06m_K`1KFe/0EUIF103gmoL04P3Ph>00
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000g]kN0?7al@020?7al@80l_;b00<0lO7a
0?;blP3bl_80103alO400`3bl_80lO7a0?7al@030?7al@80l_;b00<0lO7a0?;blP3alO400P3bl_;o
0?7al@80lO7a00<0MWIf0;Rh^02[Zj/0402[Zj/01P2h^;P0MWIf0?7al@3alO40MWIf0;Rh^140Zj^[
00D0^;Rh07IfMP3alO40l_;b05IFEP0206m_K`03079bLP1gMgL0MgMg00/0MgMg00@0LW9b06m_K`1_
Kfl0EUIF0`3alO404`3bl_80g]kN0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003L
g=`0jn_[0>_[j`3/k>`00P3[jn/00`2WYjL0XZ:R0::RXP020:6QX@030:2PX02OWil0WIfM0080WinO
0P2PX:020:6QX@030:NWY`3[jn/0jn_[00D0jn_[00D0e=CD0<G5a@35aLD0aLG50=OGe`020>_[j`03
0>[ZjP38b<P0d=3@0080jn_[00D0f][J0<K6aP3Shn<0jn_[0>3Ph0020<G5a@070<c<c03YjNT0c/k>
0<O7a`3YjNT0jn_[0>3Ph0020<G5a@070<c<c03Zj^X0e=CD0<G5a@3MgMd0jn_[0>3Ph0020<G5a@06
0<K6aP3Shn<0jn_[0>_[j`3Zj^X0b<S80`35aLD00`36a/H0hn?S0>_[j`070>_[j`030>GUi@37alL0
emOG02H0jn_[HP3/k>`l0>_[j`0407IfMP2k^k/0/K6a0;6a/@P0oooo1`2a/K401P2k^k/0MWIf0>_[
j`3[jn/0MWIf0;^k^`80/K6a303oool30;6a/@090;^k^`1fMWH0jn_[0>_[j`1FEUH0MgMg079bLP1d
M7@0MgMg00/0N7Qh00D0MgMg07AdM01bLW80MgMg05IFEP040>_[j`0B0=cLg0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000030000003Jf]X0iNGU00<0iNGU00<0RXZ:09^KV`2/[:`00`3YjNT02P3Z
j^X0a<C408F5Q@3/k>`0kNg]0>k^kP3WinL0l?3`0?;blP2SXj<60>GUi@0A06QXJ00@4100410@010@
4020P800a/K60<o?c`3Ng]h09RHV055AD@3?cll0iNGU09BDU00E5AD0Y:BT0=kNgP2b/[800P0@4100
1`11@D40fm_K051@D00I6AT0^[Zj0>?Sh`2b/[800P0@41001`11@D40gmoO071`L00@4100TY:B0>CT
i02_[jl00P0@41001P0E5AD0^[Zj0<c<c03Ti>@0gMgM02@T90<0410@00<05ADE0:BTY03Ng]h01`3U
iND01039bLT071`L079bLP3De=C40>GUi@0507IfMP30`<00]KFe0;Fe]@3oool01P1bLW800`0W9bL0
]KFe0;Fe]@050;Fe]@060<30`01fMWH0iNGU0>GUi@1fMWH0`<300P2e]KD00`3oool09bLW02LW9`09
02LW9`<0]KFe00D0`<3007IfMP3UiND0iNGU05IFEP0207MgM`0407UiN@1jNWX0oooo0?ooo`L0NWYj
0P3oool01@1jNWX0NGUi07MgM`1gMgL0EUIF00@0iNGU0180f][J0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000<000000=WIf@3Ogml00`3Ogml01@3Ng]h0X:2P0>[ZjP3YjNT0iNGU0080j^[Z00X0
d=3@07AdM03]kNd0k^k^0:f][@3@d=00l_;b0??cl`2SXj<0gmoO0P3Ng]h0=03Ogml0g]kN06a/K00E
5AD0Zj^[0=kNgP33`l<071`L05ADE036a/H0e]KF02DU9@1ADE40cLg=0=kNgP2?Shl05ADE0:BTY03E
eMD0CTi>03Ph>034a<@0gMgM0=_Kf`1>CTh06ATI0;Zj^P3IfMT0CTi>03Ph>034a<@0gMgM0=OGe`0U
9BD0>3Ph0<C4a032`/806a/K055AD@3IfMT0`l?304a<C03Hf=P0emOG02DU9@0I6AT0aLG50=kNgP2?
Shl05ADE0:BTY03IfMT70=kNgP040<?3``0L71`0LW9b0=7AdL@0g]kN00D0MWIf0<?3``2k^k/0^k^k
0?ooo`06079bLP0302LW9`2k^k/0^k^k00D0^k^k00H0`l?307IfMP3Ng]h0g]kN07IfMP33`l<20;^k
^`030?ooo`0W9bL0^k^k00L0^k^k00<0oooo02LW9`2k^k/00P2k^k/02@33`l<0MWIf0=oOg`3Ng]h0
EUIF07MgM`1hN7P0Ng]k030`<0030?ooo`D0OGem0`3oool01@0`<300Ng]k07QhN01gMgL0EUIF0080
g]kN01@0gmoO0=kNgP3IfMT0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000emOG0=SH
f03IfMT20=SHf00@092@T02?Shl0SXj>0>WYj@21PH40DE5A08N7Q`3Jf]X0YZJV0:VYZ@2/[:`0GemO
0?;blP3clo<0m?Cd0:BTY0<0f=SH00D0fMWI0=;BdP0T92@0DE5A0<_;b`020=SHf00P06YZJP0I6AT0
^[Zj0=3@d00T92@0DE5A0<_;b`3Hf=P0Rh^;01DE5@2TY:@0cLg=02@T900h>3P0`l?30=OGe`3EeMD0
C4a<01TI6@2j^[X0d=3@02@T900h>3P0`l?30=OGe`3AdM4092@T03Ph>033`l<0RXZ:01DE5@2TY:@4
0<G5a@0:09jNWP0@4100LG5a0=7Ad@3Hf=P0Rh^;01DE5@2TY:@0e=CD0=;BdP80aLG500P0a/K60=OG
e`3Hf=P0f=SH0;fm_@0K6a/0LW9b0<k>cRH0f=SHHP3IfMTl0=SHf00507IfMP37alL0_kno0;no_`3o
ool01`0W9bL70;no_`060<O7a`1fMWH0f=SH0=SHf01fMWH0alO70P2o_kl00`3oool09bLW0;no_`07
0;no_`030?ooo`0W9bL0_kno0080_kno00D0alO707IfMP3Hf=P0f=SH05IFEP0207]kN`0307inOP1o
Ogl0:2PX00<0oooo0`1oOgl30?ooo`0302PX:01oOgl0OWin0080Ng]k00<0EUIF0=WIf@3Hf=P00P3H
f=P04P3GemL0000009ZJVP2PX:00Z:RX0;:b/P2l_;`0a/K60=3@d03IfMT0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000eMGE0=?Cd`030=?Cd`03
08F5Q@2SXj<0i^KV0080j^[Z00/0=CDe07moO`3?cll0_[jn08^;R`1NGUh0F5QH06QXJ02SXj<0bLW9
0:ZZZP040=?Cd`040<c<c00T92@0DE5A0<[:bP80dm?C01h0IfMW01TI6@2i^KT0bl_;02@T901ADE40
b/[:0=?Cd`28R8P05ADE0:BTY039bLT092@T03Ph>031`L40dm?C0<o?c`1:BTX06ATI0;Vi^@3;bl/0
92@T03Ph>031`L40dm?C0<c<c00T92@0>3Ph0<71`@1WIfL5010@400:055AD@2MWId0410@08j>SP3<
c<`0dm?C08R8R00E5AD0Y:BT0:6QX@80410@00<06QXJ0<K6aP3Cdm<00P3Cdm<0102h^;P06a/K079b
LP3<c<c40=?Cd`0307IfMP3;bl/0a/K60100a/K600H0bl_;07IfMP3Cdm<0dm?C07IfMP3;bl/20<K6
aP030?ooo`0W9bL0a/K600L0a/K600<0oooo02LW9`36a/H00P36a/H01`3;bl/0MWIf0=?Cd`3Cdm<0
EUIF07emO@1oOgl00`23Ph<01@0X:2P0oooo0?ooo`3oool0Ph>300<0oooo00L0:2PX08>3P`23Ph<0
Ph>307moO`1mOGd0EUIF00@0dm?C0140eMGE0000002PX:00YJFU0:f][@2f][H0_kno0<S8b03Bd]80
f][J0>;RhP3YjNT0kno_0?Cdm03fm_H0nOWi0?_kn`030?gmo@40o_kn0@3oool10?ooo`0000D00000
0=?Cd`3>c/h0c/k>0<o?c`020<k>cP0?08N7Q`2CTi<0dm?C0=_Kf`2][Jd0`L710;2`/02YZJT0ZZZZ
08f=S@2ATI40/;2`0<_;b`3_knl0YJFU00<0c/k>00D0clo?0<O7a`0S8b<0DE5A0<S8b0020<k>cP0]
06EUI@0I6AT0^KVi0<O7a`0S8b<0DE5A0<O7a`3>c/h0QHF501DE5@2TY:@0alO702<S8`0h>3P0`L71
0<k>cP3;bl/0BDU901TI6@2i^KT0alO702<S8`0h>3P0`L710<k>cP37alL08b<S03Ph>031`L40QHF5
01DE5@2TY:@0b/[:0<O7a`0S8b<0DE5A0;6a/@0J6QX0LW9b0<G5a@3>c/h0QHF501DE5@2TY:@0cLg=
00L0c/k>00@0];Bd01XJ6P1bLW80b/[:9P3>c/iR0<o?cc`0c/k>00<0MWIf0<k>cP3;bl/0403;bl/0
1P3>c/h0MWIf0<k>cP3>c/h0MWIf0<k>cP80bl_;00<0oooo02LW9`3;bl/01`3;bl/00`3oool09bLW
0<_;b`020<_;b`090<k>cP1fMWH0clo?0<k>cP1FEUH0OGem08:2PP25QHD0Q8B40080QHF500<0:2PX
0?ooo`3oool00`3oool0200X:2P0QHF508F5Q@24Q8@0QHF508:2PP1mOGd0EUIF103>c/h04@3Cdm<0
00000:RXZ02][Jd0/k>c0;^k^`33`l<0c<c<0=GEe@3MgMd0i>CT0>_[j`3`l?00m?Cd0?Ogm`3inOT0
o?cl00<0oOgm0@3no_h10?ooo`40oooo00000`000000d=3@0<W9b@030<W9b@0A07=cL`2DU9@0OWin
07UiN@2PX:00dM7A0:f][@2UYJD0YZJV08F5Q@1NGUh0[Jf]0>GUi@3fm_H0n?Sh0:BTY039bLT00`38
b<P01032`/808b<S055AD@37alL20<S8b00/069RHP0I6AT0^KVi0<;2`P0R8R80>3Ph0<71`@32`/80
ATI601TI6@2l_;`0`L7102<S8`0h>3P0_kno0<G5a@34a<@0ATI601TI6@2i^KT0`L7102<S8`0h>3P0
_kno0<G5a@30`<008R8R03Ph>030`<00[jn_01XJ6P1cLg<0aLG509ZJVP0@4100SXj>0<G5a@16ATH0
6ATI0;Vi^@35aLD0Ogmo01DE5@2TY:@70<S8b0050<O7a`2^[Zh06QXJ075aL@37alL09P38b<QR0<W9
bC`0b<S800<0MWIf0=?Cd`3=cLd0403=cLd01P3Cdm<0MWIf0<S8b038b<P0MWIf0=?Cd`80cLg=00<0
oooo02LW9`3=cLd01`3=cLd00`3oool09bLW0<g=c@020<g=c@070=?Cd`1fMWH0b<S80<S8b01FEUH0
PX:208>3P`0508N7Q`0502PX:03oool0oooo0?ooo`0X:2P01@27QhL00`23Ph<0PX:205IFEP030<W9
b@0B0<S8b03@d=0000000;:b/P2f][H0^k^k0<;2`P39bLT0dM7A0=WIf@3Ph>00inOW0>g]k@3alO40
mOGe0?Shn03jn_X0o?cl0`3moOd10?knoP40oooo0@3oool000030000003?cll0aLG500@0aLG500l0
UiNG0;Ng]`3/k>`0^KVi08^;R`2g]kL0ZZZZ0:BTY01nOWh0LG5a0<C4a03IfMT0m_Kf0?Win@2UYJD0
1035aLD0102o_kl08R8R055AD@36a/H20<G5a@06065QH@0I6AT0^KVi0;no_`0R8R80GEeM0P0@4100
100E5AD0Y:BT0<;2`P15ADD3010@4006055AD@1QHF40410@01TI6@2f][H0ADE50`0@4100201ADE40
_kno028R8P0h>3P0_kno0<G5a@2/[:`06QXJ0P0@41001@1bLW80aLG50<G5a@32`/80ADE500<0410@
00<05ADE0:BTY036a/H01@35aLD01P2/[:`0G5aL0:f][@0J6QX0LG5a0<K6a/@0aLG500<0MWIf0=GE
e@3AdM40403AdM401P3EeMD0MWIf0<G5a@35aLD0MWIf0=GEe@80dM7A00<0oooo02LW9`3AdM401`3A
dM400`3oool09bLW0=7Ad@020=7Ad@070=GEe@1fMWH0aLG50<G5a@1FEUH0PX:208N7Q`0508V9R@D0
oooo1@29RHT00`27QhL0PX:205IFEP040<G5a@0A0<o?c`000000_;bl0;no_`33`l<0bLW90=3@d03G
emL0gMgM0>CTi03Zj^X0kno_0??cl`3fm_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`00
0000cLg=0<71`@030<71`@0@07EeM@2IVIT0YjNW0>g]k@39bLT0Lg=c0;2`/02=SHd0Lg=c04a<C01R
HV80_[jn0;Fe]@2b/[80[Jf]0:VYZ@@0`L7100@0^k^k028R8P1ADE40a<C40P31`L40101OGel06ATI
0;Vi^@32`/870<71`@050;^k^`0R8R80>3Ph0;jn_P32`/800P31`L403033`l<0aLG50;bl_00R8R80
>3Ph0;jn_P32`/80^k^k028R8P0h>3P0_[jn0<;2`P/0`L7100@0O7al01DE5@2TY:@0`l?31@31`L40
1P2YZJT06QXJ010@400@4100LG5a0<C4a<@0`L7100<0MWIf0=SHf03EeMD0403EeMD01P3Hf=P0MWIf
0<71`@31`L40MWIf0=SHf080eMGE00<0oooo02LW9`3oool0203oool01P0W9bL0eMGE0=GEe@3EeMD0
f=SH07IfMP80`L7100<0EUIF08B4Q028R8P0102:RXX30?ooo`0402PX:03oool0oooo0?ooo`@0RXZ:
00<0R8R808B4Q01FEUH01031`L40403=cLd000000<K6aP38b<P0c<c<0=7Ad@3GemL0g=cL0>;RhP3X
j>P0kNg]0?7al@3emOD0moOg0?Win@3lo?`30?gmo@80o_kn0P3oool000030000003<c<`0^[Zj0080
_Kfm0140_[jn0;fm_@2NWYh0hn?S0;Jf]P2FUYH0PX:209>CT`24Q8@0PX:204i>CP0/;2`0W9bL0:^[
Z`3fm_H0n?Sh0:NWY`040;fm_@040;Ng]`0Q8B40DE5A0<C4a080_Kfm00@0GEeM01TI6@2i^KT0_kno
202m_Kd00`1MGEd0=cLg0;Rh^0020;fm_@0;05eMG@0I6AT0^KVi0;no_`1MGEd0=cLg0;Rh^02m_Kd0
]kNg024Q8@0h>3P00P2n_[h;0;fm_@0407YjNP0E5AD0Y:BT0;no_`H0_Kfm00D0YJFU01XJ6P0@4100
L71`0<;2`P0V0;fm_F80_[jn?02m_Kd00`1fMWH0f][J0=GEe@0@0=GEe@060=[JfP1fMWH0_Kfm0;fm
_@1fMWH0f][J0P3EeMD00`3oool0LW9b079bLP08079bLP0>02LW9`3EeMD0eMGE0=GEe@3Jf]X0MWIf
0;fm_@2n_[h0EUIF08N7Q`2:RXX0SHf=08j>SP2=SHd30?ooo`0302PX:02=SHd0:2PX00<0oooo0`2=
SHd0102:RXX0QhN705IFEP2n_[h20;fm_@0A0;Zj^P3<c<`000000=3@d03Bd]80eMGE0=WIf@3MgMd0
h^;R0>OWi`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`00`3moOd00`3no_h0oooo0?ooo`010?oo
o`0000<000000<[:bP2h^;P00`2j^[X0401lO7`0SHf=07QhN02IVIT0l?3`0<;2`P2WYjL0bl_;0;bl
_02SXj<0KFe]07emO@2/[:`0no_k0?clo02VYZH30;Zj^P040;^k^`2d];@08B4Q04e=C@<0^[Zj00<0
Fe]K01TI6@2_[jl03P2j^[X00`1KFe/06ATI0:n_[`050;Zj^P040;Bd]00Q8B40=cLg0;Bd]0`0^[Zj
00<0MgMg01@D502KVi/02@2j^[X00`1KFe/0Jf][0;Zj^P340;Zj^P0307IfMP3Hf=P0f][J0100f][J
00H0f=SH07IfMP2j^[X0^[Zj07IfMP3Hf=P20=[JfP030?ooo`1bLW80LW9b00P0LW9b00d09bLW0=[J
fP3Jf]X0f][J0=SHf01fMWH0^[Zj0;^k^`1FEUH0Rh^;08n?S`2>SXh0Shn?00<0oooo00D0:2PX08n?
S`2?Shl0Shn?02PX:0030?ooo`0508n?S`2>SXh0Shn?08^;R`1FEUH00`2j^[X0402h^;P0b/[:0000
003IfMT0f][J0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`30?gmo@80
o_kn0`3oool0000300000039bLT0^;Rh00<0^;Rh0100QXJ609jNWP3De=@0l?3`0?;blP3>c/h0`L71
0=gMg@3Bd]80XZ:R0<c<c039bLT0RHV90?clo03moOd0YjNWo`2h^;P70;Rh^00307IfMP3GemL0gMgM
0100gMgM00H0emOG07IfMP2h^;P0^;Rh07IfMP3GemL30=gMg@/09bLW0`3MgMd02@3GemL0MWIf0;Rh
^02h^;P0EUIF092@T02ATI40Ti>C09BDU0020?ooo`0302PX:02HV9P0V9RH00<0V9RH00P0:2PX0?oo
o`3oool0U9BD09>CT`2ATI40T92@05IFEP@0^;Rh00l0bLW90000003QhN40h^;R0>CTi03WinL0j^[Z
0>g]k@3`l?00lo?c0?Gem@3gmoL0nOWi0?_kn`3lo?`00`3moOd00`3no_h0oooo0?ooo`020?ooo`00
00<000000<W9b@2h^;P0102h^;P03`26QXH0U9BD0?;blP3clo<0X:2P0>SXj03fm_H0fm_K09VIV@3j
n_X0no_k0=cLg02f][H0o_kn0:NWY`3o0;Rh^0L0^;Rh00<0MWIf0=KFeP3QhN40403QhN401P3Fe]H0
MWIf0;Rh^02h^;P0MWIf0=KFeQ40hN7Q00T0e]KF07IfMP2h^;P0^;Rh05IFEP2EUID0UiNG09bLW02R
XZ800P0X:2P01@2[Zj/0[Jf]0:^[Z`2[Zj/0[Jf]0080Zj^[0P0X:2P01@2RXZ80W9bL09NGU`2EUID0
EUIF00@0^;Rh00h0bLW90000003Xj>P0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0
no_k0?clo0<0oOgm0P3no_h40?ooo`0000<000000<W9b@2e]KD00`2e]KD03@1gMgL0X:2P0>3Ph03c
lo<0iNGU0>c/k03fm_H0n?Sh0?Win@2@T900no_k0?clo03moOd00P3no_h00`2WYjL0]KFe0;Fe]@3o
0;Fe]@D0]KFe00<0MWIf0>g]k@3YjNT0403YjNT01P3]kNd0MWIf0;Fe]@2e]KD0MWIf0>g]kA40jNWY
00X0kNg]07IfMP2e]KD0]KFe05IFEP2KVi/0WYjN0:FUY@2b/[80^[Zj0P2o_kl01030`<00_kno0;no
_`30`<020;no_`070<30`02j^[X0/[:b0:FUY@2NWYh0Vi^K05IFEP040;Fe]@0=0<W9b@000000k^k^
0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k0?clo0030?gmo@80o_kn1@3oool00003
00000039bLT0^[Zj00@0/K6a00<0SHf=0:ZZZP2ZZZX01@2ZZZX00`2WYjL0ZZZZ0:ZZZP040:ZZZ_l0
/K6a1`2a/K400`1fMWH0e=CD0?;blP0@0?;blP060=CDe01fMWH0/K6a0;6a/@1fMWH0e=CD4@3bl_80
1P3De=@0MWIf0;6a/@2a/K40EUIF0:>SX`80Z:RX3@33`l<20:RXZ0030:>SX`1FEUH0/K6a0080/K6a
00@0^[Zj0<W9b@000000lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3lo?`0103moOd0
0`3no_h0oooo0?ooo`040?ooo`0000<000000<?3``39bLT0o`2^[ZhJ0:j^[P0408f=S@1fMWH0c/k>
0?_kn`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^[Zh0102=SHd0MWIf0<k>cP3kno/=0?ooo`040?_k
n`3>c/h0MWIf08f=S@80[Zj^00H0VIVI05IFEP2e]KD0bl_;0=oOg`3Xj>P90>OWi`060>SXj03Ogml0
bl_;0;Fe]@1FEUH0VIVI0`2^[Zh00`39bLT0`l?3000000020?KfmP050?Ogm`3hn?P0n?Sh0?Win@3j
n_X00P3lo?`40?gmo@80o_kn1P3oool0000400000033`l<0e=CD0;Ng]ol0YjNW6P2WYjL00`2=SHd0
MWIf07IfMP0?07IfMP0308f=S@2WYjL0YjNW0080YjNW00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0
YjNW0:NWY`020:NWY`0308f=S@1FEUH0EUIF00l0EUIF00P0SHf=0:NWY`2WYjL0YjNW0;Ng]`3De=@0
`l?3000000<0nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00001P000000ZZZZ0>OW
i`3De=@0_[jn0;>c/ol0ZZZZFP2ZZZX01P2c/k<0_[jn0=CDe03WinL0ZZZZ00000080no_k0`3lo?`5
0?gmo@<0o_kn203oool000070000002:RXX0fMWI0?clo03Ti>@0d];B0<c<c03o0;jn_UP0_[jn00P0
c<c<0=;BdP3Ti>@0o?cl0=WIf@2:RXX000000?clo0P0oOgm0P3no_h:0?ooo`0000P0000006UYJ@2G
UiL0fMWI0?clo03bl_80k>c/0>?Shol0fMWIEP3IfMT0203Shn<0k>c/0?;blP3lo?`0fMWI09NGU`1Y
JFT000001`3moOd30?knoP/0oooo00001@000000oooo06UYJ@2GUiL0dm?C0080o?clo`3ooomH0?oo
o`80o?cl00D0dm?C09NGU`1YJFT0oooo000000040?gmo@@0o_kn3@3oool000030000003oool0oooo
0080JFUY00<0RXZ:09NGU`2ZZZX0o`2ZZZYG0:ZZZP0409NGU`2:RXX0JFUY06UYJ@80oooo00<00000
0?knoP3no_h0103no_h?0?ooo`00o`00001V000001D0oooo0000\
\>"],
  ImageRangeCache->{{{126.938, 390.5}, {575.625, 431.063}} -> {-181.964, \
618.245, 1.00234, 1.00234}},
  CellTags->"IPM3.1.1.6"]
}, Open  ]],

Cell["We then select the most previous cell.", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SelectionMove[nb, Previous, Cell]\)], "Input",
  CellLabel->"In[46]:=",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .54907 
%%ImageSize: 378 208 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
2.5804e-017 0.00265252 -0.00265252 0.00265252 [
[ 0 0 0 0 ]
[ 1 .54907 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .54907 L
0 .54907 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00265 translate
1.00265 .55172 scale
1134 string
378 208 8 [378 0 0 208 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FEFEFEFEFEFEFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFC
FCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFC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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFBFBFB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FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9
F8F8F8F6F6F6F5F5F5F3F3F3F1F1F1F0F0F0EFEFEFEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEFEFEFF0F0F0
F1F1F1F3F3F3F5F5F5F6F6F6F8F8F8F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7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FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7F5F5F5
F3F3F3F0F0F0EDEDEDEAEAEAE7E7E7E4E4E4E2E2E2E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E2E2E2E4E4E4
E7E7E7EAEAEAEDEDEDF0F0F0F3F3F3F5F5F5F7F7F7F9F9F9FBFBFBFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F3F3F3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FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F4F4F4F0F0F0
ECECECE7E7E7E2E2E2DDDDDDD9D9D9D5D5D5D2D2D2D0D0D0CFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFD0D0D0D2D2D2D5D5D5
D9D9D9DDDDDDE2E2E2E7E7E7ECECECF0F0F0F4F4F4F6F6F6F8F8F8FAFAFAFCFCFCFDFDFD
FDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F5F5F5F1F1F1EDEDED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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F6F6F6F3F3F3EFEFEFEAEAEA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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F5F5F5F1F1F1EDEDEDE7E7E7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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F4F4F4F0F0F0EBEBEBE4E4E4
DDDDDDD5D5D5CCCCCCC3C3C3BBBBBBB3B3B3ADADADA8A8A8A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A8A8A8ADADADB3B3B3
BBBBBBC3C3C3CCCCCCD5D5D5DDDDDDE4E4E4EBEBEBF0F0F0F4F4F4F7F7F7F9F9F9FCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFBFBFBF9F9F9F6F6F6F4F4F4EFEFEFE9E9E9E2E2E2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FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F6F6F6F3F3F3EEEEEEE8E8E8E1E1E1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000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
ECECECBABABA9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FBDBDBDECECECE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
969696DBDBDBCACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACADBDBDBA3A3A3FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFF
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0FFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFF
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E0000000000000000009E9E9ED9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
DCDCDC515151B9B9B9E0E0E0D9D9D91A1A1A000000383838D9D9D9E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD1D1D1D0D0D0D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0
D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF969696FAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0E0E0E0E0E0E0
E0E0E0ADADAD545454E0E0E0ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6
D6D6D6D7D7D7D7D7D7D7D7D7D6D6D6D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6
959595D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D7D7D7
D7D7D7D7D7D7D6D6D6D6D6D6D6D6D6FFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCECECECECECECECECE
CECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFF
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9E0E0E0E0E0E0
E0E0E0DCDCDC515151B9B9B9ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595DCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFF
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2E0E0E0E0E0E0
E0E0E0E0E0E0C4C4C4484848D2D2D21A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000000000
000000000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFD4D4D4D4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595DCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF969696F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8969696FFFFFFD4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2DCDCDC494949
0000000D0D0DD2D2D26969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000
000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFDBDBDBDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696
969696FFFFFFE4E4E4E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E4E4E4
E4E4E4E4E4E4E4E4E4E3E3E3E4E4E4FFFFFF969696F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7969696FFFFFFDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9ADADAD545454
E0E0E06969697E7E7EC4C4C4484848D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFE4E4E4E4E4E4E4E4E4
3E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF969696F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5969696FFFFFFE4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E000000383838D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0ADADAD545454
E0E0E06969697E7E7EE0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF969696F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDEDEDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D9D9D9525252D9D9D9E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0C4C4C41D1D1DD2D2D2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFF
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
969696FFFFFFF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFFF969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
969696FAFAFAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFAFAFA969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0DADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
D6D6D6B0B0B0B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8A9A9A9CECECEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEDADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECE3E3E3ECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3E3E3E3E3E3E3ECFCFCF
CFCFCFCFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3E
D4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E
3E3E3EDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFE4E4E4E4E4E43E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E43E3E3E
E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000AAAAAAAAAAAAAAAAAAAAAAAA
AAAAAA000000FFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFC6C6C6000000000000090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF121212000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000606060FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000606060
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFF7F7F7262626D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1B
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFC6C6C6606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
DFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFBFBFB545454535353EFEFEFC6C6C6414141F7F7F7
FFFFFFF7F7F7262626D2D2D2FFFFFFFFFFFFFFFFFFC6C6C6090909D2D2D2FFFFFFC6C6C6
090909D2D2D2C6C6C6090909D2D2D2FFFFFF7878780F0F0FEFEFEFFFFFFFFBFBFB545454
000000606060C6C6C6606060FFFFFFFFFFFFFFFFFFFFFFFFA2A2A2606060FFFFFFFFFFFF
F7F7F7262626D2D2D2FFFFFFFFFFFFFFFFFFC6C6C6090909D2D2D2FFFFFFC6C6C6090909
D2D2D2FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFFFFFF787878000000000000
8F8F8FFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFBFBFB545454
535353EFEFEFC6C6C6414141F7F7F7FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFC6C6C6090909CACACA1D1D1DB4B4B4FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878606060FFFFFFFFFFFF787878606060FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212B4B4B4FFFFFFC6C6C6090909D2D2D2FBFBFB
545454B4B4B4FFFFFFC6C6C6414141F7F7F7FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
FFFFFFFFFFFFC6C6C6090909CACACA1D1D1DB4B4B4FFFFFFFFFFFFFFFFFFDFDFDF1B1B1B
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C6
090909CACACA1D1D1DB4B4B4FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000AAAAAAAAAAAAAAAAAAAAAAAA
AAAAAA000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFBFBFB545454000000414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2
0000000000000000000000008F8F8FFFFFFFFBFBFB545454000000B4B4B4FFFFFF787878
000000B4B4B4FFFFFFDFDFDF121212090909D2D2D2FFFFFFDFDFDF121212090909D2D2D2
FFFFFFC6C6C60000000F0F0FEFEFEFFBFBFB545454000000B4B4B4FFFFFFFFFFFFF7F7F7
1D1D1D090909D2D2D2DFDFDF121212090909D2D2D2FFFFFFFFFFFFF7F7F71D1D1D090909
D2D2D2F7F7F71D1D1D000000B4B4B4F7F7F71D1D1D000000B4B4B4DFDFDF121212090909
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFDFDFDF121212000000000000414141F7F7F7FFFFFF787878000000606060FFFFFF
FFFFFFA2A2A2000000606060FFFFFFDFDFDF121212090909D2D2D2FFFFFFFFFFFFA2A2A2
000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFF
787878000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFFBFBFB545454000000
B4B4B4FFFFFFC6C6C6000000000000000000000000000000000000090909D2D2D2F7F7F7
1D1D1D000000B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000000000
414141F7F7F7DFDFDF121212090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB
545454000000B4B4B4FFFFFFF7F7F71D1D1D000000B4B4B4FFFFFF7878780000008F8F8F
FFFFFFA2A2A20000008F8F8FFFFFFFC6C6C6000000000000000000000000000000000000
090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D000000B4B4B4FFFFFFFFFFFF
FFFFFFFFFFFFF7F7F71D1D1D090909D2D2D2DFDFDF121212090909D2D2D2FFFFFFFFFFFF
F7F7F71D1D1D090909D2D2D2DFDFDF121212090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF787878000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFDFDFDF
121212090909D2D2D2FFFFFFDFDFDF121212090909D2D2D2FFFFFFC6C6C60000000F0F0F
EFEFEFC6C6C6000000000000000000000000000000000000090909D2D2D2FFFFFF787878
000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFFBFBFB545454000000B4B4B4
FFFFFFF7F7F71D1D1D000000000000000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000000000
0F0F0FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2000000606060FFFFFFFFFFFFF7F7F7
1D1D1D090909D2D2D2DFDFDF121212000000000000000000000000000000000000000000
000000000000000000606060FFFFFFFBFBFB545454000000000000000000000000000000
0000008F8F8FFFFFFFFFFFFFDFDFDF121212000000000000000000000000000000B4B4B4
FFFFFFF7F7F71D1D1D000000B4B4B4F7F7F71D1D1D000000B4B4B4FFFFFFC6C6C6000000
000000000000000000000000414141F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878
000000000000000000000000000000606060FFFFFFFFFFFF787878000000000000000000
0000000000000000008F8F8FFFFFFFFFFFFFA2A2A2000000000000000000000000000000
000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFF
7878780000000000000000000000000000000000008F8F8FFBFBFB545454000000000000
0000000F0F0FEFEFEF7878780000000000000000000000000000008F8F8FFFFFFFF7F7F7
1D1D1D000000000000000000414141F7F7F7787878000000000000000000000000000000
606060FFFFFFFFFFFFC6C6C6000000000000000000000000000000414141F3F3F3545454
0000000000000000000F0F0FE7E7E71D1D1D000000B4B4B4C6C6C6000000414141F7F7F7
FFFFFFFBFBFB545454000000B4B4B4FFFFFF787878000000000000000000000000000000
8F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D000000B4B4B4FFFFFFFFFFFF
FFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFFDFDFDF121212000000000000000000
000000000000B4B4B4FFFFFFFFFFFFC6C6C6000000000000000000000000000000414141
F7F7F7FFFFFF787878000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFDFDFDF
121212000000000000000000000000000000000000000000000000000000000000606060
FFFFFFFFFFFF7878780000000000000000000000000000008F8F8FFFFFFFFFFFFF787878
0000000000000000000000000000000000008F8F8FFBFBFB545454000000000000000000
0F0F0FEBEBEB5454540000000000000000000000000000008F8F8FFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECECFDFDFD939393B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B5B5B5B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8939393ECECEC696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECE3E3E3ECECECECECECECECECECECECECECECE3E3E3E
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF3E3E3E3E3E3E
3E3E3ECFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3E
D4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3E3E3E3E3E3E3EDBDBDB
DBDBDBDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFE4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDED3E3E3EEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F79A9A9A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A9A9A9AF7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFF7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3F7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F1F1F1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE3E3E3A5A5A5767676767676
767676767676777777767676777777777777767676767676767676767676767676767676
767676767676767676A5A5A5E3E3E3FFFFFFFFFFFFE3E3E3A5A5A5767676767676767676
767676777777767676777777777777767676767676767676767676767676767676767676
767676767676A5A5A5E3E3E3FFFFFFFFFFFFEBEBEBA5A5A5565656565656565656565656
575757565656575757575757565656565656565656565656565656565656565656565656
565656A5A5A5EBEBEBFFFFFFFFFFFFFFFFFFFFFFFFF1F1F10000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E1E1E1FCFCFCFBFBFBFCFCFCFCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCA5A5A57676768D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D767676A5A5A5FCFCFCFCFCFCA5A5A57676768D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D767676A5A5A5FCFCFCFBFBFBA5A5A55656565B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B565656A5A5A5FCFCFCFCFCFCFCFCFCFCFCFCE1E1E10000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F75656565B5B5B6F6F6F717171727272727272
727272727272727272727272727272727272727272727272727272727272727272717171
6F6F6F5B5B5B565656F7F7F7F7F7F7F7F7F7F7F7F7E0E0E00000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DEDEDEF1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1
F1F1F1F1F1F1F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2
F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1767676B8B8B8ABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABB8B8B8767676F1F1F1F1F1F1767676B8B8B8ABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABB8B8B8767676F1F1F1F2F2F25656566F6F6F6F6F6F727272777777777777
777777777777777777777777777777777777777777777777777777777777777777727272
6F6F6F6F6F6F565656F1F1F1F1F1F1F1F1F1F2F2F2DEDEDE0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DCDCDCEBEBEBEBEBEBECECECEBEBEBEBEBEBA7A7A7A2A2A2A2A2A2A1A1A1A1A1A1
A0A0A09F9F9F9D9D9D9F9F9F9F9F9FA0A0A0A0A0A0A1A1A1A1A1A1A7A7A7EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBD4D4D4C5C5C5C5C5C5C5C5C5D7D7D7EBEBEBEBEBEB
EAEAEAC8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3EBEBEBE0E0E0C5C5C5C5C5C5
CCCCCCE9E9E9CECECEC7C7C7E9E9E9EBEBEBE0E0E0C5C5C5C5C5C5CCCCCCEAEAEAD4D4D4
C5C5C5DDDDDDEBEBEBE0E0E0C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEAEAEAC8C8C8
C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBE5E5E5C7C7C7D7D7D7EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1B1B1B1B1B1B1B1B1B1
B1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1
B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB565656777777727272747474777777787878
787878787878787878787878787878787878787878787878787878787878777777747474
727272777777565656EBEBEBEBEBEBEBEBEBEBEBEBDCDCDC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DADADAE5E5E5E5E5E5E5E5E5E5E5E58A8A8A9B9B9BACACACE9E9E9E9E9E9E9E9E9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000000D9D9D9DFDFDFDFDFDFDFDFDFDFDFDFDEDEDEA0A0A0EAEAEAE9E9E9E5E5E5EAEAEA
EAEAEAD0D0D0747474EDEDEDEEEEEEADADADD0D0D0F2F2F2F3F3F3A3A3A3DFDFDFDEDEDE
DEDEDEDFDFDFDEDEDE6C6C6C151515ABABABDEDEDEC3C3C31C1C1C545454C6C6C6D6D6D6
252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4D5D5D54E4E4E383838C4C4C4DDDDDD
DBDBDB4E4E4E191919BABABAD9D9D94E4E4E383838C4C4C4DDDDDDD7D7D7252525383838
C4C4C4C2C2C21B1B1B515151D9D9D9C3C3C34C4C4CD8D8D8D7D7D7252525191919C5C5C5
DEDEDE8F8F8F151515A4A4A4D9D9D9DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
C3C3C31C1C1C727272D1D1D1DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBB
FFFFFF727272727272727272727272727272727272272727BBBBBBBBBBBBBBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBC3C3C3767676DEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBBFFFFFF
272727BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBFFFFFF272727BBBBBB
BBBBBBBBBBBBC3C3C3767676DFDFDFDEDEDE5656567777777878787B7B7B303030FFFFFF
FFFFFFFFFFFF7D7D7D7D7D7D7D7D7D7D7D7D7D7D7DFFFFFFFFFFFFFFFFFF3030307B7B7B
787878777777565656DEDEDEDEDEDEDFDFDFDEDEDED9D9D90000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D7D7D7D8D8D8D9D9D9D8D8D8D8D8D89090908F8F8F8E8E8EE9E9E9818181515151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000000D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3858585A3A3A3E6E6E6EAEAEAEAEAEA353535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000000D3D3D3CECECECECECECFCFCFCECECECECECE878787939393D3D3D3DBDBDBADADAD
C1C1C1B0B0B0A9A9A9AAAAAA8D8D8D919191B0B0B0CBCBCBEFEFEFA5A5A5CECECECECECE
CECECECFCFCFC7C7C7232323515151C8C8C8CECECECECECE656565191919B9B9B9C7C7C7
232323515151C7C7C7CECECE858585151515A4A4A4C7C7C7232323383838C1C1C1CECECE
CBCBCB494949191919B9B9B9C7C7C7232323383838C1C1C1CECECEC7C7C7232323383838
C1C1C1858585151515A4A4A4CACACAC7C7C7232323515151B1B1B11A1A1A727272C5C5C5
CECECE858585151515A4A4A4CDCDCDCECECECECECECECECECECECECECECECECECECECECE
B4B4B41A1A1A727272CACACACECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECE767676CECECECBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCECECE767676CECECECECECE767676CECECECBCBCBCBCBCBFFFFFF
272727CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBFFFFFF272727CBCBCB
CBCBCBCBCBCBCECECE767676CFCFCFCECECE5656567D7D7D828282858585848484858585
858585282828FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF282828858585858585848484858585
8282827D7D7D565656CECECECECECECECECECECECED3D3D3000000A8A8A8ADADADB3B3B3
BBBBBBC3C3C3CCCCCCD5D5D5DDDDDDE4E4E4EBEBEBF0F0F0F4F4F4F7F7F7F9F9F9FCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D0D0D0C9C9C9C9C9C9C9C9C9C9C9C97373739494947E7E7E797979A0A0A0D1D1D1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000000CFCFCFC5C5C5C5C5C5C5C5C5C5C5C5C5C5C5979797B7B7B7ECECECB9B9B98B8B8B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000000CDCDCDC1C1C1C1C1C1C1C1C1C1C1C1757575999999A7A7A7EDEDEDC9C9C9737373
B0B0B08D8D8D7373734C4C4C626262BEBEBEB5B5B5B2B2B2ADADADA9A9A9C1C1C1C1C1C1
C1C1C1C1C1C1BBBBBB222222515151C4C4C4C1C1C1C1C1C15F5F5F191919B9B9B9C2C2C2
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1BBBBBB222222383838BEBEBEC2C2C2
C1C1C1C1C1C1C3C3C3C5C5C5BCBCBC222222383838BEBEBEC2C2C2BBBBBB222222383838
BEBEBEC2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C17C7C7C151515A4A4A4C3C3C3C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1A9A9A91A1A1A
101010101010717171C4C4C4C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1767676D8D8D8D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D8D8D8767676C1C1C1C1C1C1767676D8D8D8D5D5D5D5D5D5FFFFFF
272727FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF272727D5D5D5
D5D5D5D5D5D5D8D8D8767676C1C1C1C1C1C15656568484848888888A8A8A8A8A8A8A8A8A
8A8A8AFFFFFFFFFFFFFFFFFF282828FFFFFFFFFFFFFFFFFF8A8A8A8A8A8A8A8A8A8A8A8A
888888848484565656C1C1C1C1C1C1C1C1C1C1C1C1CDCDCD000000C6C6C6C8C8C8CCCCCC
D1D1D1D7D7D7DCDCDCE2E2E2E8E8E8EDEDEDF1F1F1F5F5F5F7F7F7F9F9F9FCFCFCFDFDFD
FDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
000000CCCCCCBABABABDBDBDBDBDBDBEBEBEBDBDBD9E9E9EE3E3E3B6B6B6969696828282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000000CACACAB8B8B8BABABABABABABABABA7C7C7C8D8D8D787878999999F0F0F0C2C2C2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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B88686869E9E9ED4D4D4F0F0F0F2F2F2CECECE
C1C1C1DDDDDDD2D2D2A2A2A2CCCCCCC9C9C9898989FCFCFCFDFDFDA7A7A7B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8767676D7D7D7DDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDD7D7D7767676B8B8B8B8B8B8767676D7D7D7DDDDDDDDDDDDDDDDDD
272727272727272727272727272727272727272727272727272727272727272727DDDDDD
DDDDDDDDDDDDD7D7D7767676B8B8B8B8B8B8565656909090919191939393949494FFFFFF
FFFFFF282828989898989898989898989898989898282828FFFFFFFFFFFF949494939393
919191909090565656B8B8B8B8B8B8B8B8B8B8B8B8C9C9C9000000E1E1E1E2E2E2E4E4E4
E7E7E7EAEAEAEDEDEDF0F0F0F3F3F3F5F5F5F7F7F7F9F9F9FBFBFBFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8868686949494F2F2F2F3F3F3A0A0A0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000000C9C9C9B5B5B5B5B5B5B5B5B5B5B5B5777777A0A0A0E0E0E0F3F3F3E5E5E5ECECEC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000000C9C9C9BABABAB1B1B1B1B1B1B1B1B1B1B1B18D8D8DAAAAAAAAAAAAAAAAAAAAAAAA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000000C3C3C3C9C9C9AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE8D8D8D767676CECECEFBFBFB
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFBFBFBCECECE7676768D8D8DAEAEAEAEAEAE8D8D8D767676CECECEFBFBFBFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FBFBFBCECECE7676768D8D8DAEAEAEAEAEAE999999565656B5B5B5CBCBCBDFDFDFE8E8E8
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8DFDFDFCBCBCB
B5B5B5565656999999AEAEAEAEAEAEAEAEAEC9C9C9C3C3C3000000F6F6F6F6F6F6F7F7F7
F8F8F8F8F8F8F9F9F9FAFAFAFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000C3C3C3D4D4D4B7B7B7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D565656565656565656565656
565656565656565656565656565656565656565656565656565656565656565656565656
5656568D8D8DA7A7A7A7A7A7A7A7A7B7B7B7D4D4D4C3C3C3000000F9F9F9F9F9F9F9F9F9
FAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000AAAAAAE7E7E7D4D4D4BEBEBEB3B3B3AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAB3B3B3BEBEBED4D4D4E7E7E7AAAAAA000000FBFBFBFBFBFBFCFCFC
FCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
0000008A8A8AD9D9D9FCFCFCE4E4E4D2D2D2CCCCCCBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBECCCCCCD2D2D2E4E4E4FCFCFCD9D9D98A8A8A000000FCFCFCFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000696969979797D9D9D9FCFCFCF2F2F2ECECECE3E3E3D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9E3E3E3ECECECF2F2F2FCFCFCD9D9D9979797696969000000FDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFF696969979797D3D3D3FCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFCFCFCFCFCFCD3D3D3979797696969FFFFFF000000FDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFF6969696969698A8A8A979797AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAA9797978A8A8A696969696969FFFFFFFFFFFF000000FEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{378, 208},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005j0000d2000`400?l00000o`00003oo`3ooomk
0?ooo`00o`3ooomk0?ooo`003P3ooooo0?knoUh0o_kn3`3oool000`0oooo103no_ko0?gmoEX0oOgm
103no_h=0?ooo`002P3oool30?kno_l0oOgmH03moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo5@0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?clo?l0no_kEP3kno/30?cl
o0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinEL0
nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_IF0?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`4
0?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gmo@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0
moOg0?KfmP3emOD00P3dm?Co0??cle@0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm00X0o?cl0?_kn`3inOT0
n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^k^iD0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0
m_Kf0?Shn03inOT0no_k0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj5@0j>SX00/0jNWY0>_[j`3]
kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0030?gmo@80o_kn103oool000<0oooo0100
o_kn0?gmo@3moOd0oOgm0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN5D0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3kno/0
o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`000P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0
m_Kf0??cl`3_knl0k>c/0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SHDP3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`00`3moOd20?knoP<0oooo
00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;R
hP3MgMd0fMWI0=GEe@3Bd]80d=3@o`3?clmB0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0oooo0040oooo00000`3o
ool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3A
dM40c<c<0<S8b036a/Ko0<G5aE80aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo0?knoP3no_h00P3moOd0
3`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl
_03o0;^k^e80^k^k00l0_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gmo@020?gmo@0?0?clo03j
n_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a
DP2a/K403`2b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd00P3moOd03`3lo?`0nOWi
0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYU80
YZJV00l0Z:RX0:f][@2c/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080oOgm00l0no_k0?Win@3f
m_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S80;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYiB09jN
WP0?0:2PX02UYJD0[Jf]0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0o?cl0?_kn`3inOT0m_Kf
0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l_;`0/[:b0:RXZ02PX:00o`2JVYYD09ZJVP0?0:2P
X02XZ:P0/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000006H000000100VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00003
0000003lo?`0o?cl0?l0o?clHP3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`00
0000o?cl0:^[Z`3o0:^[Zf40Zj^[0180o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFUP06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj9`2OWil00`2m_Kd0k>c/0>OWi`0_0>OWi`030>k^kP2=SHd0WinO00`0WinO00<0SHf=
0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3Ng]h0fm_K
00`0fm_K00@0g]kN09JFUP2FUYH0fm_K9`3:b/X00`3Kfm/0Xj>S0?_kn`0_0?_kn`0309JFUP3Ng]h0
fm_K00`0fm_K00<0g]kN09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo9`3@d=000`3oool0UYJF0?_kn`0_0?_kn`0309JFUP3oool0c/k>00`0c/k>00<0oooo
09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY
09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF
09JFUP3ooolW0<o?c`030?ooo`2FUYH0no_k02l0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ0000000=3@d0d];B0>3P
h03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3Lg=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph
0=WIf@0F0>3Ph00309JFUP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM401@3@d=0;0=7Ad@<0d=3@0`3AdM450=3@d0T0dM7A00<0oooo09JF
UP3jn_X0;`3jn_X00`2FUYH0oooo0<k>cP040<k>cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JF
UP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000
003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3P
h01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0103>c/h303hn?PD0c/k>
00@0oooo09JFUP2FUYH0oooo2`3Fe]H30=OGe`80e]KF00L0UIFE0=KFeP2EUID0e]KF09FEU@3Fe]H0
UIFE00X0e]KF0`3GemL30=KFeP030?ooo`2FUYH0nOWi02l0nOWi00<0UYJF0?ooo`3>c/h00`3>c/h3
03hn?PH0c/k>00<0oooo09JFUP3Ph>002@3Ph>020?ooo`80h>3P0P3oool05@3Ph>00JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3P
h0030?ooo`3Ph>00h>3P00@0h>3P0100a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0
d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0`3Ph>00203Lg=`0DE5A0;Vi^@2][Jd0E5AD0>3Ph01Y
JFT0WYjN5P3Ph>000`2FUYH0oooo0<o?c`030<o?c`<0?Shn1P3?cll0103oool0UYJF09JFUP3oool3
0=cLg0/0gMgM00T0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Lg=`;0=gMg@80
g=cL00<0gMgM0?ooo`2FUYH0<03inOT00`2FUYH0oooo0<o?c`040<o?c`<0?Shn1@3?cll00`3oool0
UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV
0=;BdP3Ph>00fMWI02HV9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103oool0UYJF09JFUP3oool3
0=cLg0/0gMgM00T0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Lg=`;0=gMg@80
g=cL00<0gMgM0?ooo`2FUYH0<03hn?P00`2FUYH0oooo0=CDe0050=CDe0<0?Shn103De=@00`3oool0
UYJF0>3Ph0080>3Ph080/k>c0P3Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00
103Ph>005`34a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00
fMWI02HV9P3Bd]80g=cL04U9B@0000003@d=0=;BdP1YJFT0WYjN00l0h>3P1@0000060>3Ph00409JF
UP3oool0fm_K0=_Kf`<0?Shn203Kfm/01@3oool0UYJF09JFUP3oool0i>CT0080hn?S0`3Ti>@50>?S
h`@0i>CT00P0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@80hn?S0`3Ti>@50>?Sh`@0i>CT
00@0hn?S0>CTi03oool0UYJF<03gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0?Shn0`3Kfm/00`3oool0
UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P01P0a<C404Q8B03IfMT0
fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0
h>3P06UYJ@1nOWh0a<C404Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0`3Zj^X80>WYj@090>[ZjP3oool0UIFE
0?ooo`2EUID0oooo09FEU@3oool0UIFE0080jNWY0`3Zj^X80>WYj@050>[ZjP3YjNT0jNWY0?ooo`2F
UYH0<03emOD00`2FUYH0oooo0>CTi0050>CTi0<0?Shn103Ti>@00`3oool0UYJF0>3Ph00=0>3Ph080
oooo01D0h>3P06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0030>3Ph00I08j>SP000000>3Ph0=WIf@3P
h>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00[Jf]05ADE03P
h>00JFUY07inOP3Ph>00JFUY09jNWP0@0>3Ph0030000003Ph>00h>3P00H0h>3P00<0UYJF0?ooo`3]
kNd00`3]kNd303hn?PH0kNg]00D0oooo09JFUP2FUYH0oooo0>[ZjP020>WYj@<0j^[Z203YjNT02@3Z
j^X0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@020>WYj@<0j^[Z203YjNT01@3Zj^X0jNWY
0>WYj@3oool0UYJF0300m?Cd00<0UYJF0?ooo`3]kNd0103]kNd303hn?PD0kNg]00<0oooo09JFUP3P
h>00303Ph>020;>c/`0F0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>006@3IfMT0
DU9B0=WIf@3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00
fMWI01XJ6P0000003@d=0=;BdP3Ph>00a<C401dM7@3Bd]80603Ph>000`2FUYH0oooo0?Cdm0040?Cd
m0<0?Shn1@3dm?@0103oool0UYJF09JFUP3oool?0>g]k@070?ooo`3]kNd0oooo0>g]k@3oool0kNg]
0?ooo`0A0>g]k@030?ooo`2FUYH0lO7a02l0lO7a00<0UYJF0?ooo`3dm?@00`3dm?@303hn?PH0m?Cd
00<0oooo09JFUP3Ph>00303Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3P
h>000`2FUYH0oooo0?clo0050?clo00303hn?P3lo?`0o?cl00@0o?cl00@0oooo09JFUP2FUYH0oooo
9`3]kNd00`3oool0UYJF0?3`l00_0?3`l00309JFUP3oool0o?cl00@0o?cl00<0?Shn0?clo03lo?`0
1@3lo?`00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0
UYJF0?ooo`3oool03@3oool209JFUP030?ooo`3gmoL0moOg02D0moOg00<0oooo09JFUP3_knl0;`3_
knl00`2FUYH0oooo0?ooo`0=0?ooo`0309JFUP3Ph>00h>3P00l0h>3P01@0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00=@3Ph>000`2FUYH0emOG0?ooo`0<0?ooo`040=OGe`2FUYH0UYJF0?[jnRL0oooo00<0
n_[j09JFUP3_knl0;`3_knl00`2FUYH0emOG0?ooo`0<0?ooo`030=OGe`2FUYH0h>3P0100h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`3Jf]X0TY:B0;Jf]P0;0;Fe]@050;Bd]02?Shl0
f][J0=KFeP2`/;009`2h^;P00`2YZJT0c/k>0>k^kP0_0>k^kP030=[JfP2BTY80][Jf00/0]KFe00<0
];Bd08n?S`3Jf]X04@3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJIL0WYjN00<0oooo0>3Ph03Ph>00=@3Ph>0A0>c/k2/0jn_[<03WinLA
0>c/k140h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?oo
o`030>c/k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`0<0<k>
cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?oo
o`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`050<k>
cP0303hn?P3>c/h0c/k>00@0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`040<k>
cP<0?Shn1@3>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00<0clo?1@0n?Sh40<o?
c`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3De=@303hn?P040=CDe00n?Sh0?Shn
03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooC@3oool0103/k>`0UYJF0?ooo`3Kfm/303hn?P<0fm_K0`0n
?Sh20=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3Ti>@00`0n?Sh0i>CT0>CT
i0030>CTi00J03hn?P3Ti>@0i>CT0>CTi03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00`0
kNg]01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`0<0?Cdm00F0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0
oooo00<0k>c/09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00d0
oooo01D0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo
00@0k>c/0=[JfP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
ooooC@3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocd0oooo1@00000;0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc`0oooo1`00000:0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<000000:ZZZP2Z
ZZX00`2ZZZX00`000000oooo05EEE@0405EEE@@0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool500000003
0:ZZZP000000oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooll0?ooo`D0
000000<0ZZZZ0000003oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFUO0?ooo`05
0<K6aP000000000000T92@3Bd]807`3oool0103Ogml04Q8B0000001PH60H0?ooo`040=oOg`0B4Q80
0000061PH0T0oooo00@0N7Qh000000092@T0d];B803oool0103Ogml04Q8B0000001PH6090?ooo`04
07QhN00000002@T90=;BdP@0oooo00<0PH610=;BdP3oool02P3oool0101hN7P0000000T92@3Bd]95
0?ooo`D0000000<0ZZZZ0000003oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFUR
0?ooo`030::RXP11@D40moOg01h0oooo00<0gmoO01/K6`3Bd]806@3oool00`3Ogml06a/K0=;BdP0;
0?ooo`030=oOg`0K6a/0d];B0080oooo00<0moOg02HV9P3Bd]806`3oool00`3Ogml06a/K0=;BdP0;
0?ooo`030=oOg`0K6a/0d];B00@0oooo00<0a/K6061PH03oool02`3oool00`3Ogml06a/K0=;BdP15
0?ooo`D0000000<0ZZZZ0000003oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU2
0?ooo`030=oOg`0B4Q8000000080000000`02@T90=;BdP1hN7P0000000T92@3:b/X07AdM00000011
@D40moOg0?ooo`36a/H2000000040451@@3gmoL0oooo0<K6aP<0000000@0@D510?Ogm`3oool0XZ:R
0P0000001@2BTY808B4Q0>o_k`3oool0XZ:R0080000000X0@D510?Ogm`3oool0no_k05ADE0000000
3`l?09:BTP000000H61P0P3oool01`36a/H0000000000011@D40moOg0?ooo`36a/H00`0000001`11
@D40moOg0?ooo`3oool0gmoO01/K6`3Bd]800P3oool00`3Ogml0CTi>04Y:BP02000000030;Bd]03o
ool0a/K600D0000001/0];Bd07QhN00000002@T90<[:bP0M7Ad000000451@@3gmoL0oooo0=oOg`0K
6a/0d];B0?ooo`3kno/0E5AD000000092@T0b/[:01dM7@000000@D510?Ogm`3oool0gmoO01/K6`3B
d]800`3oool00`1hN7P0H61P0?ooo`050?ooo`0>07QhN000000000000451@@3gmoL0oooo0=oOg`0B
4Q80000000T92@3Bd]80oooo0=oOg`0B4Q830000000E0;Bd]03oool0oooo0?ooo`3Ogml06a/K0=;B
dP3oool0no_k05ADE00000002@T90<[:bP0M7Ad000000451@@3gmoL0oooo0=oOg`0K6a/0d];B00@0
oooo0140gmoO024Q8@3_knl0oooo0?_kn`1DE5@0000000T92@3:b/X07AdM00000011@D40moOg0?oo
o`3Ogml06a/K0=;BdP150?ooo`D0000000<0ZZZZ0000003oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFU40?ooo`030=oOg`0K6a/0d];B0080oooo0200no_k05ADE02d];@0oooo0=oO
g`0Q8B40kno_0?ooo`3kno/0E5AD0;Bd]03oool0N7Qh08n?S`36a/H000000;Bd]03oool0moOg01dM
7@2d];@0a/K600T92@3Bd]80oooo07QhN00?3`l0kno_0?ooo`3kno/0E5AD0;Bd]0<0oooo00H0a/K6
0451@@3gmoL0moOg01dM7@1PH6020?ooo`0=0?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0<K6aP000000
];Bd0?ooo`3gmoL07AdM0;Bd]0020?ooo`030=oOg`0K6a/0d];B0080oooo00L0gmoO01/K6`3Bd]80
oooo0=oOg`0K6a/0d];B0080oooo00<0gmoO01/K6`3Bd]800P3oool01`3kno/0E5AD0;Bd]03oool0
gmoO024Q8@3_knl00P3oool00`3Ogml06a/K0=;BdP030?ooo`050::RXP11@D40lo?c05ADE02?Shl0
0`3oool00`3Ogml06a/K0=;BdP030?ooo`030=oOg`0K6a/0d];B00<0oooo01D0no_k05ADE011@D40
moOg0?ooo`1hN7P0Shn?0=oOg`0B4Q80];Bd0?ooo`36a/H02@T90=;BdP3Ogml08B4Q0>o_k`3oool0
gmoO01/K6`3Bd]800P3oool00`3Ogml06a/K0=;BdP030?ooo`050::RXP11@D40lo?c05ADE02?Shl0
0`3oool00`3Ogml06a/K0=;BdP040?ooo`030?_kn`1DE5@0Shn?00<0oooo00D0XZ:R0451@@3clo<0
E5AD08n?S`030?ooo`030=oOg`0K6a/0d];B04D0oooo1@0000000`2ZZZX000000?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00<0gmoO01/K6`3Bd]800P3oool02`3kno/0
E5AD0;Bd]03oool0gmoO024Q8@3_knl0oooo0?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL0
103oool02`1hN7P0Shn?0?ooo`3oool0gmoO024Q8@3_knl0oooo0?_kn`1DE5@0];Bd00<0oooo00H0
a/K600T92@3Bd]80oooo0::RXP1PH6020?ooo`030?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3g
moL01P3oool00`3Ogml06a/K0=;BdP060?ooo`030?Ogm`0M7Ad0];Bd0080oooo00<0gmoO01/K6`3B
d]800P3oool01`3kno/0E5AD0;Bd]03oool0gmoO024Q8@3_knl00P3oool00`3Ogml06a/K0=;BdP04
0?ooo`0407QhN018B4P0@D510?Ogm`<0oooo00<0gmoO01/K6`3Bd]80103oool00`1hN7P0H61P0?oo
o`020?ooo`0307QhN011@D40moOg00@0oooo00<0XZ:R061PH03oool00P3oool02@1hN7P0Shn?0?oo
o`3oool0moOg01dM7@0000003`l?0>o_k`020?ooo`030=oOg`0K6a/0d];B00@0oooo00@0N7Qh04Q8
B011@D40moOg0`3oool00`3Ogml06a/K0=;BdP0;0?ooo`0407QhN018B4P0@D510?Ogm`<0oooo00<0
gmoO01/K6`3Bd]80A@3oool5000000030:ZZZP000000oooo00@0oooo00<0EEEE0?ooo`3oool00P3o
ool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYA03oool00`3Ogml06a/K0=;BdP020?ooo`0;0?_kn`1DE5@0Shn?0?ooo`3Ogml0
8B4Q0>o_k`3oool0moOg02HV9P3Bd]800`3oool00`1hN7P00000000000030000000308n?S`1hN7P0
Shn?0080oooo00L0gmoO024Q8@3_knl0oooo0?_kn`1DE5@0H61P00@0oooo00D0XZ:R000000000000
0000061PH0020?ooo`030?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0000000000000`0000001P2?Shl0
oooo0?ooo`3Ogml06a/K0=;BdP@0oooo00D0gmoO018B4P0000003`l?0>o_k`020?ooo`030=oOg`0K
6a/0d];B0080oooo00L0no_k05ADE02?Shl0oooo0=oOg`0Q8B40kno_0080oooo00<0gmoO01/K6`3B
d]80103oool00`2RXZ80000008n?S`040?ooo`030=oOg`0K6a/0d];B00@0oooo00<0gmoO01/K6`3B
d]800P3oool00`1hN7P0H61P0?ooo`040?ooo`090::RXP1PH600oooo0?ooo`3kno/0E5AD08n?S`3k
no/0E5AD0080000000<03`l?0>o_k`3oool00P3oool00`3Ogml06a/K0=;BdP040?ooo`030::RXP00
0000Shn?00@0oooo00<0gmoO01/K6`3Bd]802`3oool00`2RXZ80000008n?S`040?ooo`030=oOg`0K
6a/0d];B04D0oooo1@0000000`2ZZZX000000?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3k
no/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJD@0oooo00<0gmoO01/K6`3Bd]800P3oool02`3kno/0E5AD05=CD`3_knl0a/K60451@@3g
moL0oooo0?Ogm`0V9RH0d];B00<0oooo01D0a/K600T92@3Bd]80oooo0<K6aP092@T0d];B0<K6aP09
2@T0d];B0?ooo`1hN7P03`l?0>o_k`3oool0no_k05ADE0000000H61P0<K6aP1PH600103oool01`2R
XZ80H61P0?ooo`3oool0moOg02HV9P3Bd]800`3oool01`36a/H02@T90=;BdP3oool0a/K600T92@3B
d]800P3oool00`3Ogml06a/K0=;BdP030?ooo`0407QhN0000000000008n?S`@0oooo00<0gmoO01/K
6`3Bd]800P3oool01`3kno/0E5AD05=CD`3_knl0a/K60451@@3gmoL00P3oool00`3Ogml06a/K0=;B
dP030?ooo`050<K6aP092@T0b/[:01dM7@2d];@00`3oool00`3Ogml06a/K0=;BdP050?ooo`0607Qh
N01PH600oooo0?ooo`1hN7P0H61P1@3oool03P3Ogml04Q8B0;Bd]03oool0a/K600T92@3Bd]80no_k
05ADE02d];@0oooo0<K6aP11@D40moOg0P3oool00`3Ogml06a/K0=;BdP030?ooo`050<K6aP092@T0
b/[:01dM7@2d];@00`3oool00`3Ogml06a/K0=;BdP0:0?ooo`050<K6aP092@T0b/[:01dM7@2d];@0
0`3oool00`3Ogml06a/K0=;BdP150?ooo`D0000000<0ZZZZ0000003oool0103oool00`1EEED0oooo
0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0
n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFU40?ooo`030=oOg`0K6a/0d];B0080oooo00T0N7Qh0000002/[:`0
7AdM00T92@3Bd]80oooo0=oOg`0B4Q800`0000001011@D40moOg0?ooo`36a/H3000000040;Bd]03o
ool0oooo0::RXP80000000`0TY:B018B4P0?3`l0clo?018B4P000000[:b/01dM7@000000];Bd0?_k
n`1DE5@20000000500l?3`3_knl0oooo0=oOg`0B4Q800`0000001011@D40moOg0?ooo`36a/H30000
00070;Bd]03oool0oooo0?ooo`3Ogml06a/K0=;BdP020?ooo`030?_kn`1DE5@0];Bd00@0oooo00D0
no_k05ADE00000002@T90=;BdP020?ooo`0607QhN0000000[:b/01dM7@092@T0d];B0`3oool00`3O
gml06a/K0=;BdP020?ooo`070::RXP0000002@T90;:b/P0B4Q80000008n?S`020?ooo`030=oOg`0K
6a/0d];B00D0oooo00L0gmoO01/K6`3Bd]80oooo07QhN011@D40moOg0080oooo00P0N7Qh0;Bd]03o
ool0gmoO018B4P0000002@T90=;BdP80oooo00H0gmoO018B4P00000000000451@@3gmoL20?ooo`03
0=oOg`0K6a/0d];B0080oooo00L0XZ:R000000092@T0/[:b018B4P000000Shn?0080oooo00<0gmoO
01/K6`3Bd]802@3oool01`2RXZ80000000T92@2b/[804Q8B0000002?Shl00P3oool00`3Ogml06a/K
0=;BdP150?ooo`D0000000<0ZZZZ0000003oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFU40?ooo`030=oOg`0K6a/0d];B00X0oooo00<0moOg02HV9P3Bd]80803oool00`3gmoL09RHV
0=;BdP0<0?ooo`030=oOg`0K6a/0d];B0080oooo00L0no_k05ADE02?Shl0oooo0::RXP0?3`l0kno_
0100oooo00<0gmoO01/K6`3Bd]802`3oool00`3Ogml06a/K0=;BdP060?ooo`0:07QhN01PH600oooo
0?_kn`1DE5@0@D510?Ogm`3Ogml04Q8B0;Bd]100oooo00<0gmoO01/K6`3Bd]802`3oool00`3Ogml0
6a/K0=;BdP0B0?ooo`030=oOg`0K6a/0d];B04D0oooo1@0000000`2ZZZX000000?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJD80oooo00<0gmoO018B4P0000000P0000000`092@T0
d];B0?ooo`070?ooo`030?Ogm`0V9RH0d];B0200oooo00<0moOg02HV9P3Bd]80303oool0103Ogml0
4Q8B0000001PH6020?ooo`060?Ogm`0M7Ad0000008N7Q`0/;2`0kno_0P3oool00`2RXZ803`l?0>o_
k`0;0?ooo`040=oOg`0B4Q800000061PH0T0oooo00@0N7Qh000000092@T0d];B1P3oool00`3Ogml0
8B4Q0>o_k`020?ooo`0507QhN0000000@D51071`L02d];@0403oool0103Ogml04Q8B0000001PH609
0?ooo`0407QhN00000002@T90=;BdQ40oooo00@0N7Qh000000092@T0d];BA@3oool5000000030:ZZ
ZP000000oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooll0?ooo`D00000
00<0ZZZZ0000003oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo
1@0000000`2ZZZX000000?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool5000000030:ZZZP000000oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3oooll0?ooo`D0000000<0ZZZZ0000003oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oooc`0oooo00<000000:ZZZP2ZZZX00`2ZZZX00`000000oooo0?ooo`030?ooo`0305EE
E@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool7000000D0oooo00<0EEEE0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolm0?ooo`D000001P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EE
E@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EE
E@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EE
E@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EE
E@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocd0oooo1@1EEED60?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJO`0oooo
00D0no_k05ADE0000000@D510?Ogm`0o0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo@3oool00`3gmoL0;2`/0>o_k`100?ooo`0305EEE@3oool0oooo00@0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo@3oool00`3gmoL0;2`/0>o_k`100?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?P3oool01@1hN7P0
000000000011@D40moOg0080oooo00l0N7Qh0000000?3`l0kno_0::RXP000000@D5109ZJVP000000
@D510<?3``0o?cl0000000l?3`3_knl00`3oool03@2RXZ800000000000000000@D51071`L0000000
H61P0::RXP000000@D510?Ogm`1hN7P00`000000302d];@0oooo0?ooo`3oool0N7Qh0000001PH600
XZ:R00000011@D40moOg0::RXP80000000d0];Bd0?ooo`3oool0N7Qh0000002d];@0oooo07QhN000
00003`l?0>o_k`3kno/0E5AD0080000000@0];Bd0?ooo`3oool0XZ:R0P000000102d];@0oooo0?oo
o`2RXZ820000000=0;Bd]03oool0N7Qh0000001PH600a/K60000000?3`l0^k^k03lo?`0000003`l?
0>o_k`030?ooo`0>0::RXP000000000000000011@D40L71`00000011@D40moOg0?ooo`1hN7P00000
00l?3`3_knl20?ooo`0:0?_kn`1DE5@0000000T92@3Bd]80oooo07QhN02d];@0no_k05ADE0800000
00<0@D5109ZJVP0000000P0000006P11@D40moOg07QhN0000000];Bd07QhN0000000H61P0::RXP00
0000@D510?Ogm`1hN7P0000000l?3`3_knl0N7Qh0000001PH600N7Qh00000011@D40VYZJ00000011
@D40VYZJ0`0000003P11@D40moOg07QhN0000000];Bd0::RXP000000@D510??cl`1DE5@0000000T9
2@3Bd]80XZ:R0`0000000`11@D40moOg0?ooo`020?ooo`0;07QhN00000003`l?0>_[j`1DE5@0H61P
0=oOg`0B4Q80@D510>o_k`0M7Ad00P0000006`092@T0d];B0?ooo`1hN7P000000451@@3gmoL0oooo
07QhN00000003`l?0>o_k`3;bl/0?clo0000000?3`l0kno_0<_;b`0o?cl0000000l?3`3_knl0XZ:R
00000011@D40moOg0::RXP020000000@0;Bd]03oool0N7Qh0000001PH600XZ:R00000011@D40`l?3
03lo?`0000003`l?0>o_k`3Ogml06a/K0=;BdQ80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTm0?ooo`0G07QhN011@D40moOg0?ooo`3Ogml0De=C0?Ogm`36
a/H0H61P0?ooo`3gmoL0OGem0?_kn`1MGEd0d];B0?_kn`1MGEd0d];B0<_;b`3Bd]80oooo0<K6aP2?
Shl00`3oool04P1hN7P0];Bd0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0<K6aP0?
3`l0kno_0?_kn`1DE5@03`l?0>o_k`80oooo01L0moOg02`/;03_knl0no_k05eMG@3Bd]80gmoO01/K
6`3Bd]80no_k05eMG@3Bd]80no_k05eMG@3Bd]80oooo0<K6aP1PH600oooo0?Ogm`1mOGd0gmoO05=C
D`020?Ogm`0I02HV9P3Bd]80gmoO01/K6`3Bd]80no_k05eMG@3Bd]80gmoO01/K6`3Bd]80no_k05eM
G@3Bd]80moOg02`/;03_knl0N7Qh08n?S`3oool0bl_;0=;BdP3oool0a/K608n?S`030?ooo`0>07Qh
N02d];@0no_k05eMG@3Bd]80moOg02`/;03_knl0oooo0<K6aP1PH600oooo0?Ogm`1mOGd40?ooo`0n
0:f][@3_knl0oooo0=oOg`0K6a/0KVi^0?Ogm`3oool0XZ:R08n?S`3oool0N7Qh0;Bd]03kno/0GEeM
0=;BdP3kno/0GEeM0=;BdP3oool0moOg02`/;03_knl0no_k05eMG@3Bd]80a/K6061PH03oool0moOg
07emO@3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]01hN7P0];Bd0?_kn`1MGEd0d];B
0?_kn`1MGEd0d];B0?ooo`3kno/0GEeM0=;BdP2RXZ80H61P0?ooo`3gmoL0Y:BT07QhN02d];@0no_k
05eMG@3Bd]830?ooo`0h0<K6aP1PH600oooo0?Ogm`1mOGd0gmoO079bLP3kno/0GEeM0=;BdP3gmoL0
9RHV0=;BdP3oool0moOg02HV9P3Bd]80moOg02`/;03_knl0oooo0<K6aP1PH600oooo0?Ogm`1mOGd0
bl_;0=;BdP3oool0a/K608n?S`3;bl/0d];B0?ooo`36a/H0Shn?0?_kn`1MGEd0d];B0=oOg`0K6a/0
d];B0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0<_;b`3Bd]80oooo0<K6aP2?ShlE
0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool0
0`3Ogml06a/K0=;BdP030?ooo`040<K6aP2d];@0XZ:R0;Bd]0<0oooo00H0no_k05eMG@3Bd]80no_k
05eMG@3Bd]820?ooo`030::RXP000000];Bd00<0oooo0140a/K600l?3`3[jn/0GEeM0=;BdP3gmoL0
;2`/0>o_k`3kno/0GEeM0=;BdP2RXZ80Shn?0?ooo`3kno/0GEeM0=;BdP030?ooo`080?Ogm`0/;2`0
kno_0?_kn`1MGEd0d];B0::RXP1PH6020?ooo`080::RXP2?Shl0no_k05eMG@3Bd]80oooo0::RXP2d
];@30?ooo`080=oOg`1CDe<0moOg0?ooo`2RXZ80Shn?0::RXP1PH6020?ooo`040::RXP2?Shl0XZ:R
061PH080oooo00L0XZ:R08n?S`3gmoL09RHV09NGU`11@D40moOg00<0oooo00<0XZ:R0000002d];@0
0`3oool02`36a/H03`l?0>_[j`1MGEd0d];B0?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd00L0oooo01`0
[Jf]0>o_k`3oool0N7Qh08n?S`36a/H0];Bd0?ooo`2RXZ80Shn?0?ooo`36a/H03`l?0>_[j`1MGEd0
d];B0?_kn`1MGEd0d];B0?ooo`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP2RXZ80];Bd0`3oool05`3g
moL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]036a/H03`l?0>_[j`1MGEd0d];B0?_kn`1M
GEd0d];B0?ooo`3kno/0GEeM0=;BdP1hN7P0];Bd00<0oooo00D0a/K600l?3`3[jn/0GEeM0=;BdP03
0?ooo`030::RXP2d];@0oooo00<0oooo00L0bl_;03lo?`2?Shl0oooo0?Ogm`0/;2`0kno_0080oooo
00P0XZ:R08n?S`3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]0D0oooo00<0XZ:R0000002d];@00P3oool0
202RXZ8000000;Bd]03kno/0GEeM0=;BdP2RXZ80H61P0P3oool0202RXZ80Shn?0?Ogm`0/;2`0kno_
0?_kn`1MGEd0d];B0P3oool00`2RXZ8000000;Bd]00E0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`2RXZ80@D510?Ogm`050?ooo`0?0::RXP00
000000000000001PH600no_k05eMG@3Bd]80no_k05eMG@3Bd]80gmoO018B4P11@D40moOg00D0oooo
0100moOg01dM7@092@T0d];B0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0::RXP2?Shl0oooo0?_kn`1M
GEd0d];B0`3oool0203gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP2RXZ80Shn?0P3oool01`1hN7P0Shn?
0?_kn`1MGEd0d];B0?ooo`2RXZ800`0000002@1PH600gmoO05=CD`3gmoL0oooo0::RXP2?Shl0XZ:R
08n?S`020?ooo`0407QhN02?Shl0XZ:R08n?S`80oooo00H0N7Qh08n?S`3gmoL09RHV03lo?`1PH602
0?ooo`040=oOg`0B4Q80@D510?Ogm`D0oooo00T0moOg01dM7@092@T0d];B0?Ogm`0/;2`0kno_0?oo
o`2RXZ800`0000000`1PH600oooo0?ooo`020?ooo`0:0:f][@3_knl0moOg02`/;03_knl0no_k0:2P
X03oool0XZ:R08n?S`80oooo00l0moOg01dM7@092@T0d];B0?_kn`1MGEd0d];B0?ooo`3gmoL0;2`/
0>o_k`3kno/0GEeM0=;BdP2RXZ800`000000601PH600moOg02`/;03_knl0no_k05eMG@3Bd]80no_k
05ADE02d];@0oooo0?Ogm`0M7Ad02@T90=;BdP3kno/0GEeM0=;BdP3oool0no_k05eMG@3Bd]80PH61
0=;BdP@0oooo00@0moOg01dM7@092@T0d];B0`3oool03@2RXZ800000000000000000H61P0?ooo`36
a/H03`l?0>o_k`3oool0moOg02`/;03_knl00P3oool01`2RXZ80Shn?0?Ogm`0/;2`0kno_0?ooo`2R
XZ800`000000401PH600gmoO018B4P11@D40moOg0?ooo`3Ogml04Q8B0451@@3gmoL0oooo0?_kn`1M
GEd0d];B0::RXP2?Shl20?ooo`0<07QhN02?Shl0moOg02`/;03_knl0no_k05eMG@3Bd]80gmoO018B
4P11@D40moOg5P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/
0?3`l02>SXh01@2PX:000`2OWil0X:2P0:2PX0040:2PX00F092@T03/k>`0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030::RXP11@D40
moOg00D0oooo0100a/K608n?S`3oool0N7Qh08n?S`3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP36a/H0
Shn?0?ooo`21PH40d];B0`3oool04@2RXZ80Shn?0?_kn`1MGEd0c/k>05ADE0000000H61P0?Ogm`0V
9RH0d];B0=oOg`0Q8B40kno_0=oOg`0K6a/0d];B0080oooo03<0no_k05ADE0000000H61P0?Ogm`0V
9RH0d];B0=oOg`1CDe<0moOg0=oOg`0B4Q80];Bd0=_Kf`1MGEd0d];B0?ooo`36a/H0Shn?0?ooo`1h
N7P0Shn?0=oOg`0B4Q80];Bd0?_kn`1DE5@0];Bd0=oOg`1CDe<0moOg0=oOg`0B4Q80];Bd0=oOg`1C
De<0moOg0=oOg`0B4Q80];Bd0?Ogm`0/;2`0kno_0<_;b`3Bd]80oooo0<K6aP2?Shl0oooo0861P@3B
d]800`3oool03P2RXZ80Shn?0?_kn`1MGEd0c/k>05ADE0000000Shn?0?ooo`36a/H0Shn?0?ooo`1h
N7P0Shn?103oool0102][Jd0kno_0::RXP2?Shl20?ooo`0h0:RXZ03Bd]80XZ:R08n?S`3oool0XZ:R
08n?S`3kno/0GEeM0=;BdP3Kfm/0GEeM0=;BdP3oool0moOg01dM7@1PH600moOg02HV9P3Bd]80a/K6
08n?S`3oool0N7Qh08^;R`1DE5@0000008n?S`3gmoL07AdM061PH03kno/0GEeM0=;BdP2RXZ80Shn?
0?_kn`1MGEd0d];B0=_Kf`1MGEd0d];B0?ooo`1hN7P02@T90=;BdP2RXZ80Shn?0?ooo`1hN7P0];Bd
0::RXP2?Shl0no_k05eMG@3Bd]830?ooo`0h0<K6aP2?Shl0oooo07QhN02?Shl0moOg02`/;03WinL0
Y:BT0?_kn`1DE5@00000061PH03oool0gmoO018B4P2`/;00E5AD0000002?Shl0oooo0<K6aP2?Shl0
oooo07QhN02?Shl0a/K608n?S`3oool0PH610=;BdP36a/H0Shn?0?ooo`21PH40d];B07QhN0092@T0
d];B0=oOg`1CDe<0moOg0=oOg`0B4Q80/;2`05ADE0000000H61P0?Ogm`0V9RH0d];B0<K6aP2?Shl0
oooo0861P@3Bd]8E0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool00`3/
k>`0UYJF0=kNgP0<0=?Cd`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030::RXP11@D40moOg00H0oooo00l0N7Qh0000
0011@D40moOg0?_kn`1MGEd0d];B0?_kn`1MGEd0d];B0?_kn`1DE5@0000009JFUP3Kfm/00`3oool0
103Ogml04Q8B00000011@D420?Ogm`0702`/;02f][H000000451@@3gmoL0oooo0<K6aP0200000003
00T92@3Bd]80oooo0080oooo00P0moOg02`/;02f][H000000451@@3gmoL0oooo0::RXP80000000<0
Shn?0?ooo`36a/H00P0000002`2d];@0oooo07QhN0000000@D510?Ogm`3Ogml0De=C071`L0000000
H61P0080oooo00@0XZ:R000000000000Shn?0P3oool0402RXZ8000000000002?Shl0oooo0?Ogm`0/
;2`0kno_0::RXP000000@D510??cl`1DE5@0000009JFUP3Kfm/30?ooo`040=oOg`0B4Q8000000451
@@80moOg00P0;2`/09:BTP092@T0d];B07QhN0000000@D510?Ogm`@0oooo00@0[Jf]0>OWi`0/;2`0
kno_0P3oool02`3gmoL0OGem0::RXP2?Shl0oooo0=oOg`0B4Q8000000451@@3gmoL0a/K600800000
01/0];Bd0?Ogm`0/;2`0][Jf00000011@D40moOg0?ooo`1hN7P000000451@@3gmoL0gmoO05=CD`2J
VYX000000451@@2n_[h000000451@@3gmoL0gmoO018B4P000000@D510?Ogm`36a/H00P0000003P2d
];@0moOg02HV9P3Bd]80oooo07QhN00000003`l?0>o_k`3Ogml04Q8B00000011@D40moOg103oool0
301hN7P000000451@@3clo<0E5AD0000002BTY804Q8B08n?S`3gmoL0;2`/0;Jf]P80000001X0Shn?
0?ooo`3gmoL0;2`/09:BTP092@T0d];B07QhN0000000@D510?Ogm`3kno/0E5AD0000002FUYH0fm_K
0?_kn`1DE5@0000009JFUP3Kfm/0moOg02HV9P3Bd]80oooo0::RXP80000000d0Shn?0?ooo`3gmoL0
;2`/0;Jf]P000000@D510?Ogm`3kno/0E5AD0000002FUYH0fm_K01D0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0
gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030=oOg`0K6a/0d];B00<0oooo00<0
Z:RX0=;BdP3oool0103oool01P3kno/0GEeM0=;BdP3kno/0GEeM0=;BdQH0oooo00<0no_k05eMG@3B
d]80403oool00`21PH40d];B0?ooo`050?ooo`030=oOg`1CDe<0moOg00l0oooo00<0moOg02`/;03_
knl07@3oool00`2][Jd0TY:B061PH0040?ooo`030:f][@2BTY80Shn?00L0oooo00H0PH610=;BdP3o
ool0moOg02`/;03_knlG0?ooo`030861P@3Bd]80oooo05d0oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cL
g03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTm0?ooo`0707QhN011@D40moOg0?ooo`2RXZ802@T90=;B
dP050?ooo`0607QhN0092@T0d];B07QhN0092@T0d];B5P3oool00`1hN7P02@T90=;BdP0@0?ooo`03
0:RXZ03Bd]80oooo00@0oooo00@0moOg01dM7@11@D40moOg3P3oool0103kno/0E5AD00l?3`3_knlM
0?ooo`030::RXP092@T0d];B00@0oooo00<0gmoO018B4P2?Shl01`3oool01P2XZ:P0d];B0?_kn`1D
E5@03`l?0>o_kaL0oooo00<0Z:RX0=;BdP3oool0G@3oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJCh0oooo00H0XZ:R000000000000Shn?0:RXZ03Bd]850?ooo`06
0?Ogm`0V9RH0d];B0?Ogm`0V9RH0d];B5P3oool00`3kno/0GEeM0=;BdP0H0?ooo`030=oOg`1CDe<0
moOg00l0oooo00<0gmoO024Q8@3_knl06`3oool0103kno/0E5AD0000001PH6060?ooo`0407QhN000
0000@D510?Ogm`P0oooo00<0gmoO024Q8@3_knl06P3oool00`3gmoL09RHV0=;BdP0a0?ooo`030?Og
m`0V9RH0d];B02H0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oood40oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oood40oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
och0oooo101EEED60?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0
oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?@3oool505EEE@H0oooo
00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3M
gMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
B03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0
gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooom80?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@H0UIFE01T0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool0
0P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF
0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocd0oooo1@1EEED6
0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd609FEU@0I0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0
0P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTV
0?ooo`050=oOg`0B4Q80000000l?3`3_knl0o`3ooolF0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTV0?ooo`050=oOg`0B4Q8000000000001PH600o`3ooolF0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFTX0?ooo`040=oOg`0B4Q802@T90=;Bd_l0oooo5@3oool00`1EEED0
oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[Z
jP0M0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAh0oooo00<0a/K6000000000000
0P0000000`2?Shl0oooo0?ooo`020?ooo`040?Ogm`0M7Ad0000008n?S`<0oooo01<0gmoO018B4P09
2@T0d];B0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6aP0000003`l?0>o_k`3kno/0E5AD0000002/
[:`07AdM0080000000<0Shn?0?ooo`3oool00P3oool01P3gmoL07AdM0000000000002@T90=;BdP80
oooo00P0moOg01dM7@000000];Bd0?Ogm`0M7Ad000000;Bd]080oooo00H0a/K60000000000000000
0451@@3gmoL60?ooo`0<0::RXP00000000000451@@3_knl07AdM00T92@3Bd]80oooo07QhN0000000
H61P0P3oool00`2RXZ800000061PH0030?ooo`070::RXP00000000000451@@2JVYX00000061PH005
0?ooo`080=oOg`0B4Q802@T90=;BdP3oool0N7Qh0000001PH6020?ooo`030::RXP000000H61P0080
oooo00L0gmoO018B4P0000002@T90=;BdP3oool0N7Qh00<0000000L0Shn?0?ooo`3oool0moOg01dM
7@092@T0d];B00<0oooo00P0XZ:R000000000000@D510>o_k`0M7Ad02@T90=;BdP80oooo00H0a/K6
00000000000000000451@@3gmoL20?ooo`080=oOg`0B4Q80000000T92@3:b/X07AdM0000002d];@3
0?ooo`0307QhN0000000H61P00D0oooo00D0N7Qh000000000000000008n?S`060?ooo`030?Ogm`0M
7Ad0000000D0000000<0Shn?0?ooo`3oool00`3oool01P3gmoL07AdM0000000000002@T90=;BdP@0
oooo00H0a/K600000000000000000451@@3gmoL30?ooo`0F07QhN000000000000451@@2JVYX00000
061PH03oool0gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6aP0000003`l?
0>o_k`80oooo00D0N7Qh000000000000000008n?S`030?ooo`0307QhN0000000H61P0080oooo00<0
XZ:R0000001PH6000P3oool01`3Ogml04Q8B000000092@T0d];B0?_kn`1DE5@00`0000000`11@D40
moOg0?ooo`0>0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
7@3oool00`2RXZ800000000000040000000508n?S`3oool0oooo0?ooo`36a/H00P0000005011@D40
moOg0?ooo`3oool0gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6aP000000
3`l?0>o_k`3kno/0E5AD1P000000102?Shl0oooo0?ooo`36a/H60000000<0;Bd]03oool0moOg01dM
7@000000];Bd0?Ogm`0M7Ad000000;Bd]03oool0a/K61@0000000`0?3`l0kno_0?ooo`030?ooo`03
0<K6aP000000000000@0000000H0@D510?Ogm`3oool0N7Qh0000001PH6020?ooo`030::RXP000000
H61P0080oooo00<0XZ:R000000000000100000000`1PH600oooo0?ooo`030?ooo`080=oOg`0B4Q80
2@T90=;BdP3oool0N7Qh0000001PH6020?ooo`030::RXP000000H61P0080oooo00H0N7Qh00000000
00002@T90=;BdP1hN7P50000000608n?S`3oool0moOg01dM7@092@T0d];B0P3oool00`36a/H00000
00000004000000040451@@3gmoL0oooo0<K6aPD0000000@03`l?0>o_k`3oool0N7Qh0P0000001@09
2@T0b/[:01dM7@000000];Bd0080oooo00D0moOg01dM7@0000003`l?0>o_k`030?ooo`0307QhN000
0000000000<0000000<0Shn?0?ooo`3oool00`3oool00`3gmoL07AdM00000006000000050451@@3g
moL0oooo0?ooo`36a/H01P000000102d];@0oooo0?ooo`36a/H50000000400l?3`3_knl0oooo0<K6
aPH000000180H61P0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K6
0000000?3`l0kno_0?ooo`1hN7P50000000608n?S`3oool0oooo07QhN0000000H61P0P3oool00`2R
XZ800000061PH0020?ooo`0607QhN0000000000000T92@3>c/h0E5AD1@0000000`11@D40moOg0?oo
o`0=0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703oool0
103gmoL07AdM0000002?Shl20?ooo`040::RXP0000003`l?0>o_k`80oooo00D0N7Qh000000000000
2@T90=;BdP020?ooo`0Q0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0
000000l?3`3_knl0no_k05ADE00000003`l?0>o_k`3oool0XZ:R0000000?3`l0kno_0?_kn`1DE5@0
00000451@@3gmoL0oooo07QhN0000000@D510080moOg01007AdM0000002d];@0moOg01dM7@000000
];Bd0?Ogm`0M7Ad0000008n?S`3oool0no_k05ADE0000000Shn?103oool03@2RXZ8000000451@@3g
moL0no_k05ADE0000000@D510?Ogm`3oool0N7Qh0000001PH6000P3oool03@2RXZ800000061PH03o
ool0moOg01dM7@000000H61P0?ooo`3gmoL07AdM0000001PH6001@3oool0203Ogml04Q8B00T92@3B
d]80oooo07QhN0000000H61P0P3oool05P2RXZ800000061PH03oool0no_k05ADE0000000Shn?0?oo
o`3Ogml04Q8B0000002d];@0oooo0<K6aP0000003`l?0>o_k`3gmoL07AdM00T92@3Bd]820?ooo`08
0::RXP000000@D510?Ogm`3kno/0E5AD00000011@D420?Ogm`0A01dM7@000000Shn?0?ooo`3kno/0
E5AD0000002?Shl0no_k05ADE0000000Shn?0?ooo`3gmoL07AdM0000002d];@00P3oool01@2RXZ80
000007AdM0000000];Bd0080oooo00T0gmoO018B4P000000];Bd0?ooo`36a/H0000000l?3`3_knl0
103oool0103gmoL07AdM0000002d];@30?ooo`0>0=oOg`0B4Q80000008n?S`3oool0no_k05ADE000
0000@D510?Ogm`3oool0N7Qh00000011@D420?Ogm`0<01dM7@000000Shn?0?ooo`3kno/0E5AD0000
002?Shl0oooo07QhN0000000@D510P3gmoL07`0M7Ad00000061PH03oool0gmoO018B4P092@T0d];B
0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6aP0000003`l?0>o_k`3Ogml04Q8B0000002d];@0oooo
0<K6aP0000003`l?0>o_k`3oool0N7Qh0000001PH6000P3oool04P2RXZ800000061PH03oool0no_k
05ADE0000000Shn?0?ooo`36a/H0000000T92@3Bd]80oooo0?Ogm`0M7Ad02@T90=;BdPh0oooo00<0
EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`040<K6aP000000
3`l?0>o_k`80oooo00/0gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00l?3`2f][H00000061PH002
0?ooo`0B0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_
knl0no_k05ADE0000000Shn?0P3oool0203gmoL07AdM00T92@3Bd]80gmoO018B4P092@T0d];B0P3o
ool0403gmoL07AdM00T92@3Bd]80moOg01dM7@000000];Bd0?Ogm`0M7Ad000000;Bd]03Ogml04Q8B
00T92@3Bd]890?ooo`0=0<K6aP00000000000;Bd]03oool0XZ:R00000011@D40moOg0?ooo`1hN7P0
0000061PH0020?ooo`080::RXP000000H61P0?ooo`3Ogml04Q8B00T92@3Bd]820?ooo`030::RXP00
0000H61P00D0oooo00P0gmoO018B4P092@T0d];B0?ooo`1hN7P00000061PH080oooo00d0XZ:R0000
001PH600oooo0?_kn`1DE5@000000;Bd]03oool0XZ:R00000011@D40moOg00D0oooo00@0moOg01dM
7@092@T0d];B0P3oool03@36a/H000000000002d];@0oooo0::RXP000000@D510?Ogm`3Ogml04Q8B
00T92@3Bd]801@3oool05P3kno/0E5AD0000002d];@0oooo0?Ogm`0M7Ad000000;Bd]03oool0no_k
05ADE00?3`l0clo?018B4P11@D40moOg0?ooo`2RXZ8000000451@@3gmoL90?ooo`040?Ogm`0M7Ad0
00000;Bd]0@0oooo00P0a/K60000000?3`l0kno_0=oOg`0B4Q802@T90=;BdP80oooo00P0moOg01dM
7@092@T0d];B0=oOg`0B4Q802@T90=;BdPH0oooo00<0N7Qh0000001PH6000P3oool05P1hN7P00000
061PH03oool0gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6aP0000003`l?
0>o_k`2RXZ8000000451@@3gmoL60?ooo`0307QhN0000000H61P0080oooo00P0XZ:R0000001PH600
oooo0?_kn`1DE5@000000;Bd]0@0oooo00H0a/K6000000000000000000T92@3Bd]8>0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY8@3oool0803gmoL07AdM0000
000?3`l0kno_0?ooo`2RXZ800000061PH03gmoL07AdM00l?3`3_knl0oooo0=oOg`0B4Q802@T90=;B
dP3oool0gmoO018B4P092@T0d];B0?ooo`36a/H0000000l?3`3_knl0no_k05ADE0000000];Bd0P3o
ool0203gmoL07AdM0000002d];@0gmoO018B4P0?3`l0kno_0P3oool0403gmoL07AdM0000002d];@0
moOg01dM7@000000];Bd0?Ogm`0M7Ad000000;Bd]03Ogml04Q8B00l?3`3_knl90?ooo`030?_kn`1D
E5@0000000@0000000H0@D510?Ogm`3oool0N7Qh0000001PH6020?ooo`080::RXP000000H61P0?oo
o`3Ogml04Q8B00l?3`3_knl20?ooo`030::RXP000000H61P00D0oooo00P0gmoO018B4P092@T0d];B
0?ooo`1hN7P00000061PH080oooo00X0XZ:R0000001PH600oooo0?_kn`1DE5@000000;Bd]03oool0
XZ:R1`0000001@2d];@0moOg01dM7@000000];Bd0080oooo00<0no_k05ADE0000000100000001P11
@D40moOg0=oOg`0B4Q803`l?0>o_k`D0oooo01<0no_k05ADE0000000];Bd0?ooo`3gmoL07AdM0000
002d];@0oooo0=oOg`0B4Q80@D510??cl`1DE5@02@T90=;BdP3oool0XZ:R00L0000000<0];Bd0?oo
o`3oool00P3oool0103gmoL07AdM0000002d];@40?ooo`080=oOg`0B4Q802@T90=;BdP3Ogml04Q8B
00l?3`3_knl20?ooo`080?Ogm`0M7Ad000000;Bd]03Ogml04Q8B00l?3`3_knl60?ooo`0307QhN000
0000H61P0080oooo01<0XZ:R0000001PH600oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P09
2@T0d];B0?ooo`36a/H0000000l?3`3_knl0XZ:R00L0000000D0];Bd0?ooo`1hN7P00000061PH002
0?ooo`080::RXP000000H61P0?ooo`3kno/0E5AD0000002d];@20?ooo`030::RXP000000000000<0
000000<0H61P0?ooo`3oool03@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJAl0oooo00<0XZ:R0000000000000P0000007@2?Shl0oooo0?_kn`1DE5@000000;Bd
]03oool0N7Qh0000002d];@0oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P092@T0d];B0?oo
o`36a/H0000000l?3`3_knl0no_k05ADE0000000];Bd0080oooo00P0moOg01dM7@092@T0d];B0=oO
g`0B4Q802@T90=;BdP80oooo0100moOg01dM7@092@T0d];B0?Ogm`0M7Ad000000;Bd]03gmoL07AdM
0000002d];@0gmoO018B4P092@T0d];B303oool02P3Ogml04Q8B000000000000@D510?Ogm`3oool0
N7Qh0000001PH6020?ooo`080::RXP000000H61P0?ooo`3Ogml04Q8B00T92@3Bd]820?ooo`030::R
XP000000H61P00D0oooo00P0gmoO018B4P092@T0d];B0?ooo`1hN7P00000061PH080oooo00X0XZ:R
0000001PH600oooo0?_kn`1DE5@000000;Bd]03oool0a/K61P0000001P092@T0d];B0?Ogm`0M7Ad0
00000;Bd]0D0oooo00X0gmoO018B4P00000000000451@@3gmoL0gmoO018B4P092@T0d];B1@3oool0
4`3kno/0E5AD0000002d];@0oooo0?Ogm`0M7Ad000000;Bd]03oool0N7Qh0000002?Shl0oooo0::R
XP000000Shn?0?ooo`36a/H01P0000000`092@T0d];B0?ooo`030?ooo`040?Ogm`0M7Ad000000;Bd
]0@0oooo00P0moOg01dM7@092@T0d];B0=oOg`0B4Q802@T90=;BdP80oooo00P0moOg01dM7@092@T0
d];B0=oOg`0B4Q802@T90=;BdPH0oooo00<0N7Qh0000001PH6000P3oool04`2RXZ800000061PH03o
ool0gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0<K6aP0000003`l?0>o_k`36
a/H01P0000001P092@T0d];B0?ooo`1hN7P00000061PH080oooo00/0XZ:R0000001PH600oooo0?_k
n`1DE5@000000;Bd]03oool0moOg01dM7@0300000003061PH03oool0oooo00l0oooo00<0EEEE0?oo
o`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTM0?ooo`030?Ogm`0M7Ad0000000<0
00000180];Bd0?ooo`3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H000000451@@3gmoL0gmoO018B
4P000000H61P0?ooo`36a/H20000000[08n?S`3oool0XZ:R0000000?3`l0kno_0?_kn`1DE5@00000
0451@@3gmoL0oooo0::RXP0000003`l?0>o_k`3kno/0E5AD00000011@D40moOg0?ooo`1hN7P00000
00l?3`3_knl0moOg01dM7@000000];Bd0?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002?Shl0oooo
0?Ogm`0M7Ad02@T90=;BdP040?ooo`0L0?Ogm`0M7Ad03`l?0>o_k`3oool0XZ:R00000011@D40moOg
0?ooo`1hN7P0000000T92@3Bd]80no_k05ADE0000000H61P0?ooo`3gmoL07AdM0000001PH600oooo
0?Ogm`0M7Ad00000061PH0D0oooo0200gmoO018B4P092@T0d];B0?ooo`1hN7P0000000T92@3Bd]80
no_k05ADE0000000H61P0?ooo`3kno/0E5AD0000002d];@0oooo0=oOg`0B4Q803`l?0>o_k`3oool0
gmoO018B4P0?3`l0kno_0?Ogm`0M7Ad00000061PH080oooo00P0moOg01dM7@0?3`l0kno_0?ooo`2R
XZ8000000451@@80moOg02<07AdM0000002?Shl0oooo0?Ogm`0M7Ad02@T90=;BdP3kno/0E5AD0000
002d];@0oooo0?Ogm`0M7Ad000000;Bd]03gmoL07AdM00T92@3Bd]80oooo0=oOg`0B4Q80@D510?Og
m`3Ogml04Q8B00l?3`3_knl0oooo0=oOg`0B4Q803`l?0>o_k`040?ooo`040?Ogm`0M7Ad000000;Bd
]0@0oooo01/0moOg01dM7@092@T0d];B0?_kn`1DE5@000000451@@3gmoL0oooo07QhN00000003`l?
0>o_k`3gmoL07AdM0000002?Shl0oooo0?Ogm`0M7Ad02@T90=;BdP3oool0N7Qh0000001PH6000P3o
ool02P2RXZ800000061PH03oool0gmoO018B4P000000H61P0?ooo`36a/H20000000V08n?S`3oool0
XZ:R0000000?3`l0kno_0=oOg`0B4Q803`l?0>o_k`3oool0gmoO018B4P0?3`l0kno_0?ooo`1hN7P0
000000T92@3Bd]80no_k05ADE0000000H61P0?ooo`3kno/0E5AD0000002d];@0oooo0=oOg`0B4Q80
3`l?0>o_k`3oool0XZ:R0000001PH60?0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY7@3oool01@1hN7P000000000000?3`l0kno_00@0oooo00<0XZ:R0000001P
H6000P3oool01P3gmoL07AdM00T92@3Bd]80gmoO018B4PX0000000@0H61P0?ooo`3kno/0E5AD1P00
00001@2?Shl0oooo0?ooo`3Ogml04Q8B00D0000000`0];Bd0?ooo`3gmoL07AdM0000002d];@0moOg
01dM7@000000];Bd0?ooo`36a/H5000000030451@@3gmoL0oooo00@0oooo00<0N7Qh000000000000
0`000000101PH600oooo0?ooo`1hN7P60000000408n?S`3oool0oooo0::RXPH0000000<0H61P0?oo
o`3oool00`3oool01P3Ogml04Q8B00T92@3Bd]80oooo07QhN0H0000000<0Shn?0?_kn`1DE5@00`00
00000`0?3`l0kno_07QhN0050000000408n?S`3oool0moOg01dM7@<0000000<0@D510?Ogm`1hN7P0
1@000000101PH600oooo0?ooo`36a/H5000000030451@@3clo<0E5AD00<0000001003`l?0>OWi`0M
7Ad000000;Bd]036a/H000000451@@3gmoL0oooo0?_kn`1DE5@000000;Bd]03oool0N7Qh1@000000
0`2?Shl0oooo0?ooo`030?ooo`040?Ogm`0M7Ad000000;Bd]0@0oooo00L0gmoO018B4P092@T0d];B
0?ooo`3Ogml04Q8B00D0000000@0];Bd0?ooo`3oool0a/K61@0000001P11@D40moOg0?ooo`1hN7P0
0000061PH080oooo00H0XZ:R0000001PH600oooo0=oOg`0B4Q8:00000004061PH03oool0oooo07Qh
N0D0000000@0Shn?0?ooo`3oool0N7Qh1P0000000`2?Shl0no_k05ADE0030000000300l?3`3[jn/0
E5AD00D0000000<0Shn?0?ooo`3oool03@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0
oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJA`0oooo00@0no_k05ADE0000000Shn?0P3oool01`1hN7P00000061PH03g
moL07AdM0000002d];@00`3oool05P1hN7P0000008n?S`3Ogml04Q8B00T92@2b/[804Q8B0000000?
3`l0kno_0=oOg`0B4Q80000000l?3`3_knl0oooo0?_kn`1DE5@000000;2`/01DE5@20000000308n?
S`3oool0oooo0080oooo00H0moOg01dM7@000000000000T92@3Bd]820?ooo`080?Ogm`0M7Ad00000
0;Bd]03gmoL07AdM0000002d];@20?ooo`060<K6aP000000000000000011@D40moOg1`3oool01P1h
N7P00000000000000000@D510?Ogm`80oooo00@0N7Qh0000001PH600N7Qh0P0000000`1PH600oooo
0?ooo`020?ooo`070::RXP00000000000451@@2JVYX00000061PH0050?ooo`090=oOg`0B4Q802@T9
0=;BdP3oool0N7Qh0000001PH600N7Qh0080000000@0H61P0?ooo`3kno/0E5AD0`000000100?3`l0
kno_0?ooo`2RXZ830000000<08n?S`3oool0oooo0?Ogm`0M7Ad02@T909NGU`0000002@T90=;BdP3o
ool0N7Qh0`0000000`11@D40moOg0?ooo`020?ooo`080<K6aP000000000000000011@D40moOg0?_k
n`1DE5@30000000800l?3`3WinL07AdM0000002d];@0N7Qh0000002?Shl30?ooo`0307QhN0000000
H61P0080oooo00D0XZ:R000000000000000008n?S`060?ooo`040?Ogm`0M7Ad000000;Bd]0@0oooo
00@0XZ:R0000000?3`l0kno_0P3oool01P3gmoL07AdM0000000000002@T90=;BdP@0oooo00H0a/K6
00000000000000000451@@3gmoL20?ooo`0307QhN0000000H61P0080oooo0140XZ:R0000001PH600
oooo0=oOg`0B4Q802@T90;:b/P0B4Q80000000l?3`3_knl0gmoO018B4P0000003`l?0>o_k`030?oo
o`050::RXP00000000000000002?Shl00`3oool0101hN7P00000061PH01hN7P200000004061PH03o
ool0no_k05ADE0<0000000@03`l?0>o_k`3gmoL07AdM0`0000000`1PH600oooo0?ooo`0>0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703oool02@3kno/0E5AD
0000002?Shl0oooo0?Ogm`0M7Ad0000008n?S`0H0?ooo`040?_kn`1DE5@000000;Bd]100oooo00@0
moOg01dM7@000000];Bd:P3oool00`2RXZ800000061PH0050?ooo`040=oOg`0B4Q802@T90=;BdPX0
oooo00@0no_k05ADE0000000];Bd8P3oool0103kno/0E5AD0000002d];@K0?ooo`040?Ogm`0M7Ad0
00000;Bd]0<0oooo00@0a/K6000000000000Shn??`3oool0103kno/0E5AD0000002d];@H0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7@3oool00`2RXZ800000
000000030000000300l?3`3_knl0oooo01L0oooo00@0no_k05ADE0000000];Bd403oool0203gmoL0
7AdM0000002d];@0moOg01dM7@000000];Bd9P3oool00`2RXZ800000061PH0050?ooo`040=oOg`0B
4Q802@T90=;BdPX0oooo00@0no_k05ADE0000000];Bd8P3oool02@3kno/0E5AD0000002d];@0oooo
0?Ogm`0M7Ad000000;Bd]00F0?ooo`030?Ogm`0M7Ad0000000H0000000<0@D510?Ogm`3oool0?P3o
ool0103kno/0E5AD0000002d];@H0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY7P3oool01P36a/H000000000000000003`l?0>o_kaT0oooo00@0no_k05ADE000
0000];Bd403oool0203gmoL07AdM0000002d];@0moOg01dM7@000000];Bd9P3oool00`2RXZ800000
061PH0050?ooo`040=oOg`0B4Q802@T90=;BdP`0oooo00<0N7Qh0;Bd]03oool08`3oool01`1hN7P0
];Bd0?ooo`3gmoL07AdM0000002d];@05P3oool00`3gmoL07AdM000000050000000308n?S`3oool0
oooo0440oooo00<0N7Qh0;Bd]03oool05`3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0
oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo
0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo?P3oool405EEE@H0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom10?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OG
e`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTK0?ooool000007@0000050?ooo`D0EEEE0P3oool505EEE@@0oooo00<0k>c/0?gmo@2CTi<0
1@2h^;P00`2e]KD0^;Rh0;Rh^0040;Rh^00F09>CT`3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
C@3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooom=0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
C@3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k03^k^h0SHf=00`0WinO01H0SHf=0>KViP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
C@3oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0
303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
C@3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0
303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo0080c/k>00<0?Shn0<k>cP3>c/h0
0`3>c/h06P0n?Sh0c/k>0<k>cP3>c/h0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool0103/k>`0UYJF0?ooo`3?cll3
03hn?P<0clo?0`0n?Sh20<o?c`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3De=@3
03hn?P040=CDe00n?Sh0?Shn03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?oo
o`030=_Kf`D0?Shn103Kfm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00@0i>CT0`0n
?Sh50>CTi00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool01@3]kNd00`0n?Sh0kNg]0>g]
k@040>g]k@0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oom=0?ooo`030>c/k02FUYH0oooo00`0o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?oo
o`0=0?ooo`0E09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3GemL0303oool05P3GemL0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=
0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3Jf]X0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`2JVYX0o`1jNWYP07YjNP0C09ZJVP3gmoL0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`2o_kl0o`2o_kmQ0;no_`0B0?Og
m`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003gmoL0hn?S0?l0hn?SH@3Shn<04P3g
moL0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000lO7a0?ooo`3o0?oooaX0oooo00<0
hn?S0:FUY@1fMWH00`1fMWH0101gMgL0MWIf07MgM`1gMgL907IfMP060:FUY@3Shn<0oooo0?ooo`3S
hn<0YJFU101fMWH0101gMgL0MWIf07MgM`1gMgL907IfMP060:FUY@3Shn<0oooo0?ooo`3[jn/0YJFU
101FEUH0101GEeL0EUIF05MGE`1GEeL905IFEP030:FUY@3[jn/0oooo00<0oooo0180lO7a0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000>7Qh@3lo?`0no_k0P3lo?`00`3kno/0o?cl0?cl
o0040?clo0030?_kn`3lo?`0o?cl0080o?cl00<0no_k0?clo03kno/01@3lo?`00`3kno/0o?cl0?_k
n`1Q0?_knih0o?cl00<0YJFU07IfMP2=SHd0402=SHd01P1fMWH0YJFU0?clo03lo?`0YJFU07IfMQ40
SHf=00H0MWIf0:FUY@3lo?`0no_k0:FUY@1FEUHA05]KF`0305IFEP2UYJD0o?cl00<0o?cl0180hN7Q
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000>3Ph03gmoL0o`3gmoLJ0?Ogm`0307If
MP2=SHd0XZ:R0100XZ:R00H0SHf=07IfMP3gmoL0moOg07IfMP2=SHdA0::RXP0808f=S@1fMWH0moOg
0?Ogm`1FEUH0Fe]K06m_K`1aLG4=079bLP04075aL@1_Kfl0Fe]K05IFEP@0moOg0180h>3P0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=kNgP3alO400P3alO420?;blP030?7al@3bl_80
l_;b00@0lO7a00<0l_;b0?7al@3alO400`3alO420?;blP030?7al@3bl_80lO7a0080l_;bo`3alO42
0?7al@0307IfMP2h^;P0Zj^[0100Zj^[00H0^;Rh07IfMP3alO40lO7a07IfMP2h^;PA0:^[Z`050;Rh
^01fMWH0lO7a0?;blP1FEUH00P1_Kfl00`1bLW80MgMg07MgM`0;07MgM`04079bLP1_Kfl0Kfm_05IF
EP<0lO7a01<0l_;b0=kNgP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001@000000g=cL0>_[
j`3[jn/0k>c/0080jn_[00<0YjNW0::RXP2RXZ800P2QXJ400`2PX:00WinO09fMW@0209nOW`80X:2P
0P2QXJ400`2WYjL0jn_[0>_[j`050>_[j`050=CDe035aLD0aLG50<G5a@3GemL00P3[jn/00`3Zj^X0
b<S80=3@d0020>_[j`050=[JfP36a/H0hn?S0>_[j`3Ph>000P35aLD01`3<c<`0jNWY0<k>cP37alL0
jNWY0>_[j`3Ph>000P35aLD01`3<c<`0j^[Z0=CDe035aLD0gMgM0>_[j`3Ph>000P35aLD01P36a/H0
hn?S0>_[j`3[jn/0j^[Z0<S8b0<0aLG500<0a/K60>?Sh`3[jn/01`3[jn/00`3UiND0alO70=OGe`0V
0>_[jf80k>c/?03[jn/0101fMWH0^k^k0;6a/@2a/K480?ooo`L0/K6a00H0^k^k07IfMP3[jn/0jn_[
07IfMP2k^k/20;6a/@`0oooo0`2a/K402@2k^k/0MWIf0>_[j`3[jn/0EUIF07MgM`1bLW80M7Ad07Mg
M`0;07QhN00507MgM`1dM7@0LW9b07MgM`1FEUH0103[jn/04P3Lg=`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo00000`000000f][J0>GUi@030>GUi@0308Z:RP2KVi/0[:b/00<0jNWY00X0j^[Z0<C4
a025QHD0k>c/0>g]k@3^k^h0inOW0?3`l03bl_80Xj>S1P3UiND04@1XJ6P0410@010@400@4100P820
0<K6aP3?cll0g]kN02HV9P1ADE40clo?0>GUi@2DU9@05ADE0:BTY03Ng]h0/[:b0080410@00L0@D51
0=_Kf`1@D5006ATI0;Zj^P3Shn<0/[:b0080410@00L0@D510=oOg`1`L700410@09:BTP3Ti>@0[jn_
0080410@00H05ADE0;Zj^P3<c<`0i>CT0=gMg@0T92@3010@400301DE5@2TY:@0g]kN00L0iNGU00@0
bLW901`L701bLW80e=CDa03UiND01@1fMWH0`<300;Fe]@2e]KD0oooo00H0LW9b00<09bLW0;Fe]@2e
]KD01@2e]KD01P30`<00MWIf0>GUi@3UiND0MWIf0<30`080]KFe00<0oooo02LW9`0W9bL02@0W9bL3
0;Fe]@050<30`01fMWH0iNGU0>GUi@1FEUH00P1gMgL0101iNGT0NWYj0?ooo`3oool707YjNP80oooo
00D0NWYj07UiN@1gMgL0MgMg05IFEP040>GUi@0B0=[JfP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000030000003IfMT0gmoO00<0gmoO00D0g]kN0:2PX03Zj^X0jNWY0>GUi@020>[ZjP0:0=3@d01d
M7@0kNg]0>k^kP2][Jd0d=3@0?;blP3clo<0Xj>S0=oOg`80g]kN03@0gmoO0=kNgP1/K6`05ADE0:^[
Z`3Ng]h0`l?301`L701DE5@0a/K60=KFeP0U9BD0DE5A0<g=c@3Ng]h0Shn?01DE5@2TY:@0eMGE04i>
CP0h>3P0a<C40=gMg@3Kfm/0CTi>01TI6@2j^[X0fMWI04i>CP0h>3P0a<C40=gMg@3GemL09BDU03Ph
>034a<@0`/;201/K6`1ADE40fMWI0<?3``1<C4`0f=SH0=OGe`0U9BD06ATI0<G5a@3Ng]h0Shn?01DE
5@2TY:@0fMWI1`3Ng]h01033`l<071`L079bLP3AdM740=kNgP0507IfMP33`l<0^k^k0;^k^`3oool0
1P1bLW800`0W9bL0^k^k0;^k^`050;^k^`060<?3``1fMWH0g]kN0=kNgP1fMWH0`l?30P2k^k/00`3o
ool09bLW0;^k^`070;^k^`030?ooo`0W9bL0^k^k0080^k^k00T0`l?307IfMP3Ogml0g]kN05IFEP1g
MgL0N7Qh07]kN`0`<3000`3oool507emO@<0oooo00D0<30`07]kN`1hN7P0MgMg05IFEP020=kNgP0D
0=oOg`3Ng]h0fMWI0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000=OGe`3Hf=P0fMWI
0P3Hf=P0402@T900Shn?08j>SP3YjNT0PH61055AD@27QhL0f][J0:JVYP2YZJT0[:b/05mOG`3bl_80
lo?c0?Cdm02TY:@30=SHf0050=WIf@3Bd]8092@T055AD@3;bl/00P3Hf=P0801ZJVX06ATI0;Zj^P3@
d=0092@T055AD@3;bl/0f=SH08^;R`0E5AD0Y:BT0<g=c@0T92@0>3Ph0<?3``3GemL0eMGE04a<C00I
6AT0^[Zj0=3@d00T92@0>3Ph0<?3``3GemL0dM7A02@T900h>3P0`l?308Z:RP0E5AD0Y:BT1035aLD0
2P2NWYh0410@075aL@3AdM40f=SH08^;R`0E5AD0Y:BT0=CDe03Bd]820<G5a@080<K6aP3GemL0f=SH
0=SHf02m_Kd06a/K079bLP3>c/hV0=SHf680fMWI?03Hf=P01@1fMWH0alO70;no_`2o_kl0oooo00L0
9bLW1`2o_kl01P37alL0MWIf0=SHf03Hf=P0MWIf0<O7a`80_kno00<0oooo02LW9`2o_kl01`2o_kl0
0`3oool09bLW0;no_`020;no_`050<O7a`1fMWH0f=SH0=SHf01FEUH00P1kNg/00`1nOWh0Ogmo02PX
:0030?ooo`<0Ogmo0`3oool00`0X:2P0Ogmo07inOP0207]kN`0305IFEP3IfMT0f=SH0080f=SH0180
emOG0000002JVYX0X:2P0:RXZ02b/[80_;bl0<K6aP3@d=00fMWI0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=GEe@3Cdm<00`3Cdm<00`25QHD0
Xj>S0>KViP020>[ZjP0;03De=@1oOgl0clo?0;jn_P2;Rh/0GUiN05QHF01XJ6P0Xj>S0<W9b@2ZZZX0
103Cdm<0103<c<`092@T055AD@3:b/X20=?Cd`0N06MWI`0I6AT0^KVi0<_;b`0T92@0DE5A0<[:bP3C
dm<0R8R801DE5@2TY:@0bLW902@T900h>3P0`L710=?Cd`3?cll0BTY:01TI6@2i^KT0bl_;02@T900h
>3P0`L710=?Cd`3<c<`092@T03Ph>031`L40IfMW1@0@41002P1ADE40WIfM010@402>SXh0c<c<0=?C
d`28R8P05ADE0:BTY02QXJ42010@400301XJ6P36a/H0dm?C0080dm?C00@0^;Rh01/K6`1bLW80c<c<
a03Cdm<00`1fMWH0bl_;0<K6aP0@0<K6aP060<_;b`1fMWH0dm?C0=?Cd`1fMWH0bl_;0P36a/H00`3o
ool09bLW0<K6aP070<K6aP030?ooo`0W9bL0a/K60080a/K600L0bl_;07IfMP3Cdm<0dm?C05IFEP1m
OGd0Ogmo00<0Ph>300D0:2PX0?ooo`3oool0oooo08>3P`030?ooo`0702PX:023Ph<0Ph>308>3P`1o
Ogl0OGem05IFEP040=?Cd`0A0=GEe@000000X:2P0:FUY@2][Jd0][Jf0;no_`38b<P0d];B0=[JfP3R
h^80jNWY0>o_k`3dm?@0m_Kf0?Win@3kno/00`3moOd10?knoP40oooo0@3oool000050000003Cdm<0
c/k>0<k>cP3?cll00P3>c/h03`27QhL0Ti>C0=?Cd`3Kfm/0[Jf]0<71`@2`/;00ZJVY0:ZZZP2=SHd0
TI6A0;2`/03;bl/0kno_0:FUY@030<k>cP050<o?c`37alL08b<S055AD@38b<P00P3>c/h0;@1UIFD0
6ATI0;Vi^@37alL08b<S055AD@37alL0c/k>08F5Q@0E5AD0Y:BT0<O7a`0S8b<0>3Ph0<71`@3>c/h0
bl_;04U9B@0I6AT0^KVi0<O7a`0S8b<0>3Ph0<71`@3>c/h0alO702<S8`0h>3P0`L7108F5Q@0E5AD0
Y:BT0<[:bP37alL08b<S055AD@2a/K406QXJ079bLP35aLD0c/k>08F5Q@0E5AD0Y:BT0<g=c@070<k>
cP040;Bd]00J6QX0LW9b0<[:bRH0c/k>HP3?clll0<k>cP0307IfMP3>c/h0bl_;0100bl_;00H0c/k>
07IfMP3>c/h0c/k>07IfMP3>c/h20<_;b`030?ooo`0W9bL0bl_;00L0bl_;00<0oooo02LW9`3;bl/0
0P3;bl/02@3>c/h0MWIf0<o?c`3>c/h0EUIF07emO@22PX80QHF508B4Q00208F5Q@0302PX:03oool0
oooo00<0oooo00P0:2PX08F5Q@25QHD0Q8B408F5Q@22PX80OGem05IFEP@0c/k>0140dm?C0000002X
Z:P0[Jf]0;>c/`2k^k/0`l?30<c<c03EeMD0gMgM0>CTi03[jn/0l?3`0?Cdm03gmoL0nOWi0?clo003
0?gmo@40o_kn0@3oool10?ooo`0000<000000=3@d039bLT00`39bLT04@1cLg<0U9BD07inOP1iNGT0
X:2P0=7Ad@2][Jd0YJFU0:JVYP25QHD0GUiN0:f][@3UiND0m_Kf0?Shn02TY:@0bLW900<0b<S800@0
`/;202<S8`1ADE40alO70P38b<P0;01RHV806ATI0;Vi^@32`/808R8R03Ph>031`L40`/;204I6AP0I
6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35aLD0a<C404I6AP0I6AT0^KVi0<71`@0S8b<0>3Ph0;no_`35
aLD0`<30028R8P0h>3P0`<300:n_[`0J6QX0Lg=c0<G5a@2JVYX0410@08j>SP35aLD0ATI601TI6@2i
^KT0aLG507moO`0E5AD0Y:BT1`38b<P01@37alL0[Zj^01XJ6P1aLG40alO702H0b<S8HP39bLTl0<S8
b00307IfMP3Cdm<0cLg=0100cLg=00H0dm?C07IfMP38b<P0b<S807IfMP3Cdm<20<g=c@030?ooo`0W
9bL0cLg=00L0cLg=00<0oooo02LW9`3=cLd00P3=cLd01`3Cdm<0MWIf0<S8b038b<P0EUIF08:2PP23
Ph<01@27QhL01@0X:2P0oooo0?ooo`3oool0:2PX00D0QhN700<0Ph>308:2PP1FEUH00`39bLT04P38
b<P0d=3@0000002b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3h
n?P0n_[j0?clo0<0oOgm0@3no_h10?ooo`40oooo00000`000000clo?0<G5a@040<G5a@0?09NGU`2g
]kL0k>c/0;Vi^@2;Rh/0]kNg0:ZZZP2TY:@0OWin075aL@34a<@0fMWI0?KfmP3inOT0YJFU00@0aLG5
00@0_kno028R8P1ADE40a/K60P35aLD01P1QHF406ATI0;Vi^@2o_kl08R8R05eMG@80410@00@05ADE
0:BTY032`/80ADE50`0@41001P1ADE40HF5Q010@400I6AT0][Jf04E5A@<0410@00P0DE5A0;no_`0R
8R80>3Ph0;no_`35aLD0[:b/01XJ6P80410@00D0LW9b0<G5a@35aLD0`/;204E5A@03010@400301DE
5@2TY:@0a/K600D0aLG500H0[:b/05aLG02][Jd06QXJ075aL@36a/K40<G5a@0307IfMP3EeMD0dM7A
0100dM7A00H0eMGE07IfMP35aLD0aLG507IfMP3EeMD20=7Ad@030?ooo`0W9bL0dM7A00L0dM7A00<0
oooo02LW9`3AdM400P3AdM401`3EeMD0MWIf0<G5a@35aLD0EUIF08:2PP27QhL01@29RHT50?ooo`D0
RHV900<0QhN708:2PP1FEUH01035aLD04@3?cll000000;bl_02o_kl0`l?30<W9b@3@d=00emOG0=gM
g@3Ti>@0j^[Z0>o_k`3clo<0m_Kf0?Shn03kno/0o?cl0080oOgm0P3no_h20?ooo`0000<000000<g=
c@31`L400`31`L40401eMGD0VIVI0:NWY`3]kNd0bLW907=cL`2`/;00SHf=07=cL`1<C4`0HV9R0;jn
_P2e]KD0/[:b0:f][@2YZJT40<71`@040;^k^`0R8R80DE5A0<C4a080`L7100@0GemO01TI6@2i^KT0
`/;21`31`L401@2k^k/08R8R03Ph>02n_[h0`/;20080`L7100`0`l?30<G5a@2l_;`08R8R03Ph>02n
_[h0`/;20;^k^`0R8R80>3Ph0;jn_P32`/8;0<71`@0407alO00E5AD0Y:BT0<?3``D0`L7100H0ZJVY
01XJ6P0@4100410@075aL@34a<C40<71`@0307IfMP3Hf=P0eMGE0100eMGE00H0f=SH07IfMP31`L40
`L7107IfMP3Hf=P20=GEe@030?ooo`0W9bL0oooo00P0oooo00H09bLW0=GEe@3EeMD0eMGE0=SHf01f
MWH20<71`@0305IFEP24Q8@0R8R800@0RXZ:0`3oool0100X:2P0oooo0?ooo`3oool408Z:RP0308R8
R024Q8@0EUIF00@0`L710100cLg=00000036a/H0b<S80<c<c03AdM40emOG0=cLg03Rh^80j>SX0>g]
k@3alO40mOGe0?Ogm`3inOT0o?cl0`3moOd20?knoP80oooo00000`000000c<c<0;Zj^P020;fm_@0A
0;jn_P2m_Kd0WYjN0>?Sh`2f][H0UYJF08:2PP2CTi<0Q8B408:2PP1>CTh0;2`/09bLW02[Zj/0m_Kf
0?Shn02WYjL0102m_Kd0102g]kL08B4Q055AD@34a<@20;fm_@0405eMG@0I6AT0^KVi0;no_`P0_Kfm
00<0GEeM03Lg=`2h^;P00P2m_Kd02`1MGEd06ATI0;Vi^@2o_kl0GEeM03Lg=`2h^;P0_Kfm0;Ng]`0Q
8B40>3Ph0080_[jn2`2m_Kd0101jNWX05ADE0:BTY02o_kl60;fm_@050:FUY@0J6QX0410@071`L032
`/809P2m_KeR0;jn_S`0_Kfm00<0MWIf0=[JfP3EeMD0403EeMD01P3Jf]X0MWIf0;fm_@2m_Kd0MWIf
0=[JfP80eMGE00<0oooo079bLP1bLW80201bLW803P0W9bL0eMGE0=GEe@3EeMD0f][J07IfMP2m_Kd0
_[jn05IFEP27QhL0RXZ:08f=S@2>SXh0SHf=0`3oool00`0X:2P0SHf=02PX:0030?ooo`<0SHf=00@0
RXZ:08N7Q`1FEUH0_[jn0P2m_Kd04@2j^[X0c<c<0000003@d=00d];B0=GEe@3IfMT0gMgM0>;RhP3W
inL0k>c/0?3`l03dm?@0m_Kf0?Shn03jn_X0o?cl00<0oOgm00<0o_kn0?ooo`3oool00@3oool00003
0000003:b/X0^;Rh00<0^[Zj0100O7al08f=S@1hN7P0VIVI0?3`l032`/80YjNW0<_;b`2l_;`0Xj>S
06e]K@1mOGd0[:b/0?_kn`3lo?`0YZJV0`2j^[X0102k^k/0];Bd024Q8@1=CDd30;Zj^P0305]KF`0I
6AT0[jn_00h0^[Zj00<0Fe]K01TI6@2_[jl01@2j^[X0102d];@08B4Q03Lg=`2d];@<0;Zj^P0307Mg
M`0D51@0Vi^K00T0^[Zj00<0Fe]K06][J`2j^[X0a02j^[X00`1fMWH0f=SH0=[JfP0@0=[JfP060=SH
f01fMWH0^[Zj0;Zj^P1fMWH0f=SH0P3Jf]X00`3oool0LW9b079bLP08079bLP0=02LW9`3Jf]X0f][J
0=[JfP3Hf=P0MWIf0;Zj^P2k^k/0EUIF08^;R`2?Shl0SXj>08n?S`030?ooo`0502PX:02?Shl0Shn?
08n?S`0X:2P00`3oool01@2?Shl0SXj>08n?S`2;Rh/0EUIF00<0^[Zj0100^;Rh0<[:bP000000fMWI
0=[JfP3MgMd0h>3P0>CTi03Xj>P0k>c/0>o_k`3clo<0m_Kf0?Shn03jn_X0o?cl0`3moOd20?knoP<0
oooo00000`000000bLW90;Rh^0030;Rh^00@08J6QP2NWYh0e=CD0?3`l03bl_80c/k>0<71`@3MgMd0
d];B0::RXP3<c<`0bLW908V9R@3lo?`0oOgm0:NWYol0^;Rh1`2h^;P00`1fMWH0emOG0=gMg@0@0=gM
g@060=OGe`1fMWH0^;Rh0;Rh^01fMWH0emOG0`3MgMd;02LW9`<0gMgM00T0emOG07IfMP2h^;P0^;Rh
05IFEP2@T900TI6A09>CT`2DU9@00P3oool00`0X:2P0V9RH09RHV00309RHV00802PX:03oool0oooo
09BDU02CTi<0TI6A092@T01FEUH40;Rh^00?0<W9b@000000hN7Q0>;RhP3Ti>@0inOW0>[ZjP3]kNd0
l?3`0??cl`3emOD0moOg0?Win@3kno/0o?cl00<0oOgm00<0o_kn0?ooo`3oool00P3oool000030000
0039bLT0^;Rh00@0^;Rh00l0QXJ609BDU03bl_80lo?c0:2PX03Xj>P0m_Kf0=_Kf`2IVIT0n_[j0?_k
n`3Lg=`0][Jf0?knoP2WYjL0o`2h^;P70;Rh^00307IfMP3Fe]H0hN7Q0100hN7Q00H0e]KF07IfMP2h
^;P0^;Rh07IfMP3Fe]HA0>7Qh@090=KFeP1fMWH0^;Rh0;Rh^01FEUH0UIFE09NGU`2LW9`0XZ:R0080
:2PX00D0Zj^[0:f][@2[Zj/0Zj^[0:f][@020:^[Z`80:2PX00D0XZ:R09bLW02GUiL0UIFE05IFEP04
0;Rh^00>0<W9b@000000j>SX0>WYj@3[jn/0kNg]0>o_k`3alO40m?Cd0?KfmP3gmoL0nOWi0?_kn`3l
o?`30?gmo@80o_kn103oool0000300000039bLT0]KFe00<0]KFe00d0MgMg0:2PX03Ph>00lo?c0>GU
i@3/k>`0m_Kf0?Shn03inOT0T92@0?_kn`3lo?`0oOgm0080o_kn00<0YjNW0;Fe]@2e]KD0o`2e]KD5
0;Fe]@0307IfMP3]kNd0jNWY0100jNWY00H0kNg]07IfMP2e]KD0]KFe07IfMP3]kNdA0>WYj@0:0>g]
k@1fMWH0]KFe0;Fe]@1FEUH0Vi^K09jNWP2UYJD0/[:b0;Zj^P80_kno00@0`<300;no_`2o_kl0`<30
0P2o_kl01`30`<00^[Zj0;:b/P2UYJD0WYjN09^KV`1FEUH0102e]KD03@39bLT000000>k^kP3_knl0
l?3`0?7al@3clo<0mOGe0?KfmP3hn?P0nOWi0?_kn`3lo?`00`3moOd20?knoPD0oooo00000`000000
bLW90;Zj^P040;6a/@0308f=S@2ZZZX0ZZZZ00D0ZZZZ00<0YjNW0:ZZZP2ZZZX0102ZZZ[o0;6a/@L0
/K6a00<0MWIf0=CDe03bl_80403bl_801P3De=@0MWIf0;6a/@2a/K40MWIf0=CDe140l_;b00H0e=CD
07IfMP2a/K40/K6a05IFEP2SXj<20:RXZ0d0`l?30P2XZ:P00`2SXj<0EUIF0;6a/@020;6a/@040;Zj
^P39bLT000000??cl`80m?Cd00L0mOGe0?KfmP3gmoL0n?Sh0?[jnP3kno/0o?cl00@0oOgm00<0o_kn
0?ooo`3oool0103oool0000300000033`l<0bLW90?l0[Zj^6P2^[Zh0102=SHd0MWIf0<k>cP3kno/=
0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00@0SHf=07IfMP3>c/h0no_k3@3oool0103kno/0c/k>
07IfMP2=SHd20:j^[P0609VIV@1FEUH0]KFe0<_;b`3Ogml0j>SX2@3WinL01P3Xj>P0gmoO0<_;b`2e
]KD0EUIF09VIV@<0[Zj^00<0bLW90<?3``0000000P3fm_H01@3gmoL0n?Sh0?Shn03inOT0n_[j0080
o?cl103moOd20?knoPH0oooo000010000000`l?30=CDe02g]kOo0:NWYaX0YjNW00<0SHf=07IfMP1f
MWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=S@1fMWH0MWIf00l0MWIf00<0SHf=0:NWY`2W
YjL00P2WYjL00`2=SHd0EUIF05IFEP0?05IFEP0808f=S@2WYjL0YjNW0:NWY`2g]kL0e=CD0<?3``00
00030?Win@040?[jnP3kno/0o?cl0?clo0D0oOgm0P3no_h70?ooo`0000H000000:ZZZP3WinL0e=CD
0;jn_P2c/k?o0:ZZZUX0ZZZZ00H0/k>c0;jn_P3De=@0inOW0:ZZZP0000020?_kn`<0o?cl1@3moOd3
0?knoPP0oooo00001`000000RXZ:0=WIf@3lo?`0i>CT0=;BdP3<c<`0o`2n_[iH0;jn_P080<c<c03B
d]80i>CT0?clo03IfMT0RXZ:0000003lo?`80?gmo@80o_kn2P3oool000080000001YJFT0UiNG0=WI
f@3lo?`0l_;b0>c/k03Shn?o0=WIfEH0fMWI00P0hn?S0>c/k03bl_80o?cl0=WIf@2GUiL0JFUY0000
00L0oOgm0`3no_h;0?ooo`0000D000000?ooo`1YJFT0UiNG0=?Cd`020?clo?l0ooooF03oool20?cl
o0050=?Cd`2GUiL0JFUY0?ooo`000000103moOd40?knoPd0oooo00000`000000oooo0?ooo`0206UY
J@0308Z:RP2GUiL0ZZZZ0?l0ZZZZE`2ZZZX0102GUiL0RXZ:06UYJ@1YJFT20?ooo`030000003no_h0
o_kn00@0o_kn3`3oool00?l00000IP00000E0?ooo`00\
\>"],
  ImageRangeCache->{{{126.938, 390.5}, {295, 150.438}} -> {-181.964, 216.413, \
1.00234, 1.00234}},
  CellTags->"IPM3.1.1.6"]
}, Open  ]],

Cell["Finally, we evaluate the selected cell.", "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SelectionEvaluate[nb]\)], "Input",
  CellLabel->"In[47]:=",
  CellTags->{"i:29", "b:0.5", "IPM3.1.1.6"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .54907 
%%ImageSize: 378 208 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
2.5804e-017 0.00265252 -0.00265252 0.00265252 [
[ 0 0 0 0 ]
[ 1 .54907 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .54907 L
0 .54907 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00265 translate
1.00265 .55172 scale
1134 string
378 208 8 [378 0 0 208 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FEFEFEFEFEFEFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFC
FCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFC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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFBFBFB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FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9
F8F8F8F6F6F6F5F5F5F3F3F3F1F1F1F0F0F0EFEFEFEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEFEFEFF0F0F0
F1F1F1F3F3F3F5F5F5F6F6F6F8F8F8F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7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FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7F5F5F5
F3F3F3F0F0F0EDEDEDEAEAEAE7E7E7E4E4E4E2E2E2E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E2E2E2E4E4E4
E7E7E7EAEAEAEDEDEDF0F0F0F3F3F3F5F5F5F7F7F7F9F9F9FBFBFBFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F3F3F3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FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F4F4F4F0F0F0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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F5F5F5F1F1F1EDEDED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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F6F6F6F3F3F3EFEFEFEAEAEA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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F5F5F5F1F1F1EDEDEDE7E7E7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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F4F4F4F0F0F0EBEBEBE4E4E4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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFBFBFBF9F9F9F6F6F6F4F4F4EFEFEFE9E9E9E2E2E2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FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F6F6F6F3F3F3EEEEEEE8E8E8E1E1E1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000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
ECECECBABABA9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FBDBDBDECECECE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
969696DBDBDBCACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACADBDBDBA3A3A3FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFF
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0FFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFF
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E0000000000000000009E9E9ED9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
DCDCDC515151B9B9B9E0E0E0D9D9D91A1A1A000000383838D9D9D9E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD1D1D1D0D0D0D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0
D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF969696FAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0E0E0E0E0E0E0
E0E0E0ADADAD545454E0E0E0ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6
D6D6D6D7D7D7D7D7D7D7D7D7D6D6D6D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6
959595D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D7D7D7
D7D7D7D7D7D7D6D6D6D6D6D6D6D6D6FFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCECECECECECECECECE
CECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFF
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9E0E0E0E0E0E0
E0E0E0DCDCDC515151B9B9B9ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595DCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFF
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2E0E0E0E0E0E0
E0E0E0E0E0E0C4C4C4484848D2D2D21A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000000000
000000000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFD4D4D4D4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595DCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF969696F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8969696FFFFFFD4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2DCDCDC494949
0000000D0D0DD2D2D26969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000
000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFDBDBDBDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696
969696FFFFFFE4E4E4E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E4E4E4
E4E4E4E4E4E4E4E4E4E3E3E3E4E4E4FFFFFF969696F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7969696FFFFFFDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9ADADAD545454
E0E0E06969697E7E7EC4C4C4484848D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFE4E4E4E4E4E4E4E4E4
3E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF969696F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5969696FFFFFFE4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E000000383838D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0ADADAD545454
E0E0E06969697E7E7EE0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF
959595E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF969696F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDEDEDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D9D9D9525252D9D9D9E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0C4C4C41D1D1DD2D2D2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFF
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
969696FFFFFFF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFFF969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
969696FAFAFAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFAFAFA969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0DADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
D6D6D6B0B0B0B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8A9A9A9CECECEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEDADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECE3E3E3ECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3E3E3E3E3E3E3ECFCFCF
CFCFCFCFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3E
D4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E
3E3E3EDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFE4E4E4E4E4E43E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E43E3E3E
E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF555555555555555555555555555555FFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFF7F7F75E5E5EFFFFFFFFFFFFFFFFFFFBFBFBA0A0A0FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFBA0A0A0
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFA5A5A5F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA5A5A5F7F7F7FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2000000000000000000B4B4B4
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB545454090909D2D2D2FFFFFFFFFFFF
F7F7F71D1D1D0000000909099797978F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB545454
090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF
121212414141F7F7F7FFFFFFFFFFFFFFFFFFDFDFDF121212414141F7F7F7FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF555555555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF555555FFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF555555FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFC6C6C6000000000000090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF121212000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000606060FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212000000606060
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFF7F7F7262626D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1B
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFC6C6C6606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFF
555555FFFFFFFFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF555555
FFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
DFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFBFBFB545454B4B4B4FFFFFFDFDFDF212121EFEFEF
FFFFFFFBFBFB545454B4B4B4FFFFFF7878788F8F8FC6C6C6000000B4B4B4FFFFFFF7F7F7
1D1D1DB4B4B4C6C6C6090909D2D2D2FFFFFF7878780F0F0FEFEFEFFFFFFFFBFBFB545454
B4B4B4FFFFFFFFFFFFFFFFFFC6C6C6414141F7F7F7F7F7F71D1D1D606060FFFFFFFFFFFF
FBFBFB545454B4B4B4FFFFFF7878788F8F8FC6C6C6000000B4B4B4FFFFFFF7F7F71D1D1D
B4B4B4FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
DFDFDF1B1B1BD2D2D2FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFBFBFB545454
B4B4B4FFFFFFDFDFDF212121EFEFEFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFA2A2A2414141F3F3F35454548F8F8FFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFFFFFFFBFBFB545454414141
F7F7F7FFFFFF7878788F8F8FDFDFDF121212B4B4B4FFFFFFC6C6C6090909D2D2D2DFDFDF
212121EFEFEFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
FFFFFFFFFFFFA2A2A2414141F3F3F35454548F8F8FFFFFFFFFFFFFFFFFFFDFDFDF1B1B1B
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFBFBFB5454548F8F8FFFFFFFFFFFFFFFFFFFA2A2A2
414141F3F3F35454548F8F8FFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFF555555555555
FFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
DFDFDF1B1B1BD2D2D2FFFFFFFFFFFF787878000000ACACAC1D1D1D090909D2D2D2FFFFFF
DFDFDF121212000000000000000000414141F7F7F7FFFFFFC6C6C6000000000000000000
B4B4B4FFFFFFFFFFFFA2A2A20000000000009292921212120F0F0FCFCFCF121212000000
ACACAC1D1D1D000000B4B4B4FBFBFB5454540000000000000F0F0FEFEFEFFFFFFFDFDFDF
121212000000000000000000414141F7F7F7FFFFFFC6C6C6000000000000000000B4B4B4
FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFBFBFB545454B4B4B4FFFFFF
FFFFFFFFFFFFFFFFFFFBFBFB545454000000090909D2D2D2FFFFFFFFFFFF787878000000
ACACAC1D1D1D090909D2D2D2FFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
A2A2A2000000090909B2B2B21212120000008F8F8FFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF787878414141F7F7F7
FFFFFFFFFFFF787878B4B4B4FFFFFFDFDFDF121212000000090909D2D2D2FFFFFFFFFFFF
DFDFDF121212000000000000414141F7F7F7FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFF
FFFFFFA2A2A2000000090909B2B2B21212120000008F8F8FFFFFFFFFFFFFDFDFDF1B1B1B
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2000000
090909B2B2B21212120000008F8F8FFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF555555555555555555555555555555FFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECECF0F0F08E8E8EA0A0A0A0A0A0A0A0A0A0A0A0A0A0A09F9F9FA0A0A0A0A0A0A0A0A0
A0A0A0A0A0A0A0A0A0909090ECECEC696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696DEDEDED3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3DEDEDE969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFBFBFB545454000000414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2FFFFFFFFFFFFFFFFFFC6C6C6B4B4B4
A2A2A2B4B4B4FFFFFFFFFFFFFFFFFFFBFBFB5D5D5DD2D2D2FBFBFB5D5D5DD2D2D2FFFFFF
FFFFFFA2A2A2000000B4B4B4FFFFFFFFFFFFFFFFFFC6C6C60F0F0FEBEBEB5D5D5DD2D2D2
F7F7F72C2C2CEFEFEFFBFBFB5D5D5DD2D2D2A2A2A28F8F8FFFFFFFFBFBFB5D5D5DD2D2D2
FFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFBFBFB5D5D5DD2D2D2A2A2A2606060FFFFFF
FFFFFFA2A2A28F8F8FFBFBFB5D5D5DD2D2D2FFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFF
DFDFDF535353F7F7F7FFFFFFA2A2A28F8F8FA2A2A2606060FFFFFFFFFFFFA2A2A28F8F8F
A2A2A2606060FFFFFFFFFFFFA2A2A28F8F8FF7F7F7262626979797414141F7F7F7FFFFFF
FFFFFFFFFFFFA2A2A2000000B4B4B4FFFFFFFFFFFFFFFFFFC6C6C60F0F0FEBEBEB5D5D5D
D2D2D2F7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFADADADEFEFEFFFFFFF7878788F8F8FC6C6C6B4B4B4FFFFFFA2A2A28F8F8F
FFFFFFC6C6C60F0F0FEBEBEB5D5D5DD2D2D2FBFBFB5D5D5DD2D2D2FFFFFFF7F7F72C2C2C
EFEFEFFBFBFB5D5D5DD2D2D2A2A2A2B4B4B4FFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEF
FBFBFB5D5D5DD2D2D2FBFBFB545454B4B4B4C6C6C60F0F0FEBEBEB5D5D5DD2D2D2FBFBFB
5D5D5DD2D2D2FFFFFFFBFBFB5D5D5DD2D2D2787878B4B4B4FFFFFFFFFFFFFFFFFFC6C6C6
0F0F0FEBEBEB5D5D5DD2D2D2FFFFFFFFFFFFFFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFF
FFFFFFCBCBCB3F3F3F8F8F8FFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFA2A2A28F8F8F
F7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2
000000B4B4B4FFFFFFFFFFFFA2A2A2000000B4B4B4FBFBFB5D5D5DD2D2D2A2A2A2606060
FFFFFFFFFFFFA2A2A28F8F8FF7F7F72C2C2CEFEFEFFBFBFB5D5D5DD2D2D2FFFFFFFFFFFF
A2A2A2000000B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595959595959595
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6
D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6
D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C60000000F0F0FEFEFEF
FFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFFDFDFDF1212120F0F0FB6B6B6000000
606060FFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFFDFDFDF1212120F0F0FEFEFEF
FFFFFFC6C6C60000000F0F0FEFEFEFFBFBFB5454540000008F8F8FFFFFFFFFFFFFF7F7F7
1D1D1D090909D2D2D2DFDFDF121212090909D2D2D2FFFFFFFFFFFFF7F7F71D1D1D090909
D2D2D2F7F7F71D1D1D000000B4B4B4F7F7F71D1D1D000000B4B4B4DFDFDF121212090909
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C6000000
000000B4B4B4FFFFFFA2A2A2000000414141F7F7F7FFFFFF787878000000606060FFFFFF
FFFFFFA2A2A2000000606060FFFFFFDFDFDF121212090909D2D2D2FFFFFFFFFFFFA2A2A2
000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFF
787878000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFFBFBFB545454000000
B4B4B4FFFFFFA2A2A2000000414141F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F7
1D1D1D090909D2D2D2FFFFFFFFFFFFC6C6C6000000000000B4B4B4FFFFFFA2A2A2000000
414141F7F7F7DFDFDF121212090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB
545454000000B4B4B4FFFFFFF7F7F71D1D1D000000B4B4B4FFFFFFFBFBFB5454540F0F0F
CFCFCF121212414141F7F7F7FFFFFFA2A2A2000000414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D000000B4B4B4FFFFFFFFFFFF
FFFFFFFFFFFFC6C6C60000000F0F0FEFEFEFDFDFDF121212090909D2D2D2FFFFFFFFFFFF
F7F7F71D1D1D090909D2D2D2DFDFDF121212090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF787878000000606060FFFFFFFFFFFF787878000000606060FFFFFFDFDFDF
121212090909D2D2D2FFFFFFDFDFDF1212120F0F0FEFEFEFFFFFFFC6C6C60000000F0F0F
EFEFEFA2A2A2000000414141F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878
000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFFBFBFB545454000000B4B4B4
FFFFFFFFFFFFFFFFFFFFFFFFC6C6C6000000000000000000090909D2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB5454540000008F8F8F
FFFFFFFFFFFF787878000000606060F7F7F71D1D1D000000B4B4B4FFFFFFFFFFFFFFFFFF
7878780000008F8F8FDFDFDF121212090909B2B2B21212120000000F0F0FEFEFEFDFDFDF
1212120000000F0F0FEFEFEFFFFFFFFBFBFB545454000000B0B0B0545454000000000000
8F8F8FFFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D000000000000090909D2D2D2FFFFFF
FFFFFFF7F7F71D1D1D000000B4B4B4F7F7F71D1D1D000000B4B4B4FFFFFFFFFFFFC6C6C6
000000000000000000414141F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
787878000000000000000000414141F7F7F7FFFFFFFFFFFF787878000000606060787878
000000000000606060FFFFFFFFFFFFFFFFFFFFFFFFA2A2A20000000000004141419A9A9A
000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212090909D2D2D2FFFFFF
787878000000606060787878000000000000606060FFFFFFFBFBFB545454000000000000
0000000F0F0FEFEFEFFFFFFFA2A2A20000000000000000008F8F8FFFFFFFFFFFFFF7F7F7
1D1D1D090909979797000000090909D2D2D2FFFFFF787878000000000000000000414141
F7F7F7FFFFFFFFFFFFFFFFFFC6C6C6000000000000000000414141F7F7F7FBFBFB545454
0000000000000000000F0F0FE7E7E71D1D1D000000B4B4B47878780000008F8F8FFFFFFF
FFFFFFFFFFFF787878000000606060FFFFFFFFFFFFA2A2A20000000000000000008F8F8F
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D000000B4B4B4FFFFFFFFFFFF
FFFFFFFFFFFFA2A2A20000000F0F0FEFEFEFFFFFFFFFFFFFF7F7F71D1D1D000000000000
090909D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFC6C6C6000000000000000000414141F7F7F7
FFFFFFFFFFFF787878000000606060FFFFFFFFFFFFA2A2A2000000606060FFFFFFDFDFDF
121212090909B2B2B21212120000000F0F0FEFEFEFDFDFDF1212120000000F0F0FEFEFEF
FFFFFFFFFFFFFFFFFFA2A2A20000000000000000008F8F8FFFFFFFFFFFFFFFFFFF787878
000000606060787878000000000000606060FFFFFFFBFBFB545454000000000000000000
0F0F0FEFEFEFF7F7F71D1D1D000000000000000000606060FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
E7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9
FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCECECECECECE3E3E3ECECECECECECECECECECECECECECECE3E3E3E
CECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF3E3E3E3E3E3E
3E3E3ECFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3E
D4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3E3E3E3E3E3E3EDBDBDB
DBDBDBDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555
555555FFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFE4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDED3E3E3EEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
ECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F79A9A9A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A9A9A9AF7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFF7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3F7F7F70000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F1F1F1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE3E3E3A5A5A5767676767676
767676767676777777767676777777777777767676767676767676767676767676767676
767676767676767676A5A5A5E3E3E3FFFFFFFFFFFFE3E3E3A5A5A5767676767676767676
767676777777767676777777777777767676767676767676767676767676767676767676
767676767676A5A5A5E3E3E3FFFFFFFFFFFFEBEBEBA5A5A5565656565656565656565656
575757565656575757575757565656565656565656565656565656565656565656565656
565656A5A5A5EBEBEBFFFFFFFFFFFFFFFFFFFFFFFFF1F1F10000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E1E1E1FCFCFCFBFBFBFCFCFCFCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCA5A5A57676768D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D767676A5A5A5FCFCFCFCFCFCA5A5A57676768D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D767676A5A5A5FCFCFCFBFBFBA5A5A55656565B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B565656A5A5A5FCFCFCFCFCFCFCFCFCFCFCFCE1E1E10000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F75656565B5B5B6F6F6F717171727272727272
727272727272727272727272727272727272727272727272727272727272727272717171
6F6F6F5B5B5B565656F7F7F7F7F7F7F7F7F7F7F7F7E0E0E00000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DEDEDEF1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1
F1F1F1F1F1F1F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2
F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1767676B8B8B8ABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABB8B8B8767676F1F1F1F1F1F1767676B8B8B8ABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABB8B8B8767676F1F1F1F2F2F25656566F6F6F6F6F6F727272777777777777
777777777777777777777777777777777777777777777777777777777777777777727272
6F6F6F6F6F6F565656F1F1F1F1F1F1F1F1F1F2F2F2DEDEDE0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DCDCDCEBEBEBEBEBEBECECECEBEBEBEBEBEBA7A7A7A2A2A2A2A2A2A1A1A1A1A1A1
A0A0A09F9F9F9D9D9D9F9F9F9F9F9FA0A0A0A0A0A0A1A1A1A1A1A1A7A7A7EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBD4D4D4C5C5C5C5C5C5C5C5C5D7D7D7EBEBEBEBEBEB
EAEAEAC8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3EBEBEBE0E0E0C5C5C5C5C5C5
CCCCCCE9E9E9CECECEC7C7C7E9E9E9EBEBEBE0E0E0C5C5C5C5C5C5CCCCCCEAEAEAD4D4D4
C5C5C5DDDDDDEBEBEBE0E0E0C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEAEAEAC8C8C8
C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBE5E5E5C7C7C7D7D7D7EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1B1B1B1B1B1B1B1B1B1
B1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1
B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB565656777777727272747474777777787878
787878787878787878787878787878787878787878787878787878787878777777747474
727272777777565656EBEBEBEBEBEBEBEBEBEBEBEBDCDCDC0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DADADAE5E5E5E5E5E5E5E5E5E5E5E58A8A8A9B9B9BACACACE9E9E9E9E9E9E9E9E9
EAEAEAC4C4C4858585ECECECEDEDEDEEEEEEE7E7E7F0F0F0F2F2F2A3A3A3E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5686868101010101010101010808080C6C6C6CFCFCFDEDEDE
262626515151CFCFCFE5E5E5949494151515A4A4A4DEDEDEB2B2B2101010101010414141
DBDBDB505050191919BABABAE3E3E3B2B2B2101010101010414141DFDFDF707070101010
929292E4E4E4AFAFAF101010101010151515BABABACCCCCCE4E4E4DDDDDD242424101010
101010101010151515A4A4A4DEDEDEE5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
C9C9C91C1C1C727272D4D4D4E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5767676C0C0C0B5B5B5B5B5B5
FFFFFF727272727272727272727272727272727272272727B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5C0C0C0767676E5E5E5E5E5E5767676C0C0C0B5B5B5B5B5B5FFFFFF
272727272727272727272727272727272727272727272727272727272727272727B5B5B5
B5B5B5B5B5B5C0C0C0767676E5E5E5E5E5E55656567777777777777979797A7A7AFFFFFF
FFFFFF7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7AFFFFFFFFFFFF7A7A7A797979
777777777777565656E5E5E5E5E5E5E5E5E5E5E5E5DADADA0000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D9D9D9DFDFDFDFDFDFDFDFDFDFDFDFDEDEDEA0A0A0EAEAEAE9E9E9E5E5E5EAEAEA
EAEAEAD0D0D0747474EDEDEDEEEEEEADADADD0D0D0F2F2F2F3F3F3A3A3A3DFDFDFDEDEDE
DEDEDEDFDFDFDEDEDE6C6C6C151515ABABABDEDEDEC3C3C31C1C1C545454C6C6C6D6D6D6
252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4D5D5D54E4E4E383838C4C4C4DDDDDD
DBDBDB4E4E4E191919BABABAD9D9D94E4E4E383838C4C4C4DDDDDDD7D7D7252525383838
C4C4C4C2C2C21B1B1B515151D9D9D9C3C3C34C4C4CD8D8D8D7D7D7252525191919C5C5C5
DEDEDE8F8F8F151515A4A4A4D9D9D9DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
C3C3C31C1C1C727272D1D1D1DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBB
FFFFFF727272727272727272727272727272727272272727BBBBBBBBBBBBBBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBC3C3C3767676DEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBBFFFFFF
272727BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBFFFFFF272727BBBBBB
BBBBBBBBBBBBC3C3C3767676DFDFDFDEDEDE5656567777777878787B7B7B303030FFFFFF
FFFFFFFFFFFF7D7D7D7D7D7D7D7D7D7D7D7D7D7D7DFFFFFFFFFFFFFFFFFF3030307B7B7B
787878777777565656DEDEDEDEDEDEDFDFDFDEDEDED9D9D90000009A9A9A9E9E9EA6A6A6
B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D7D7D7D8D8D8D9D9D9D8D8D8D8D8D89090908F8F8F8E8E8EE9E9E9818181515151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000000D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3858585A3A3A3E6E6E6EAEAEAEAEAEA353535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000000D3D3D3CECECECECECECFCFCFCECECECECECE878787939393D3D3D3DBDBDBADADAD
C1C1C1B0B0B0A9A9A9AAAAAA8D8D8D919191B0B0B0CBCBCBEFEFEFA5A5A5CECECECECECE
CECECECFCFCFC7C7C7232323515151C8C8C8CECECECECECE656565191919B9B9B9C7C7C7
232323515151C7C7C7CECECE858585151515A4A4A4C7C7C7232323383838C1C1C1CECECE
CBCBCB494949191919B9B9B9C7C7C7232323383838C1C1C1CECECEC7C7C7232323383838
C1C1C1858585151515A4A4A4CACACAC7C7C7232323515151B1B1B11A1A1A727272C5C5C5
CECECE858585151515A4A4A4CDCDCDCECECECECECECECECECECECECECECECECECECECECE
B4B4B41A1A1A727272CACACACECECECECECECECECECECECECECECECECECECDCDCDC7C7C7
CDCDCDC7C7C7CACACACECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECE767676CECECECBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCECECE767676CECECECECECE767676CECECECBCBCBCBCBCBFFFFFF
272727CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBFFFFFF272727CBCBCB
CBCBCBCBCBCBCECECE767676CFCFCFCECECE5656567D7D7D828282858585848484858585
858585282828FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF282828858585858585848484858585
8282827D7D7D565656CECECECECECECECECECECECED3D3D3000000A8A8A8ADADADB3B3B3
BBBBBBC3C3C3CCCCCCD5D5D5DDDDDDE4E4E4EBEBEBF0F0F0F4F4F4F7F7F7F9F9F9FCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D0D0D0C9C9C9C9C9C9C9C9C9C9C9C97373739494947E7E7E797979A0A0A0D1D1D1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000000CFCFCFC5C5C5C5C5C5C5C5C5C5C5C5C5C5C5979797B7B7B7ECECECB9B9B98B8B8B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000000CDCDCDC1C1C1C1C1C1C1C1C1C1C1C1757575999999A7A7A7EDEDEDC9C9C9737373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000000CCCCCCBABABABDBDBDBDBDBDBEBEBEBDBDBD9E9E9EE3E3E3B6B6B6969696828282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000000CACACAB8B8B8BABABABABABABABABA7C7C7C8D8D8D787878999999F0F0F0C2C2C2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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B88686869E9E9ED4D4D4F0F0F0F2F2F2CECECE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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8868686949494F2F2F2F3F3F3A0A0A0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000000C9C9C9B5B5B5B5B5B5B5B5B5B5B5B5777777A0A0A0E0E0E0F3F3F3E5E5E5ECECEC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000000C9C9C9BABABAB1B1B1B1B1B1B1B1B1B1B1B18D8D8DAAAAAAAAAAAAAAAAAAAAAAAA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000000C3C3C3C9C9C9AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE8D8D8D767676CECECEFBFBFB
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFBFBFBCECECE7676768D8D8DAEAEAEAEAEAE8D8D8D767676CECECEFBFBFBFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FBFBFBCECECE7676768D8D8DAEAEAEAEAEAE999999565656B5B5B5CBCBCBDFDFDFE8E8E8
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8DFDFDFCBCBCB
B5B5B5565656999999AEAEAEAEAEAEAEAEAEC9C9C9C3C3C3000000F6F6F6F6F6F6F7F7F7
F8F8F8F8F8F8F9F9F9FAFAFAFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000C3C3C3D4D4D4B7B7B7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D565656565656565656565656
565656565656565656565656565656565656565656565656565656565656565656565656
5656568D8D8DA7A7A7A7A7A7A7A7A7B7B7B7D4D4D4C3C3C3000000F9F9F9F9F9F9F9F9F9
FAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000AAAAAAE7E7E7D4D4D4BEBEBEB3B3B3AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAB3B3B3BEBEBED4D4D4E7E7E7AAAAAA000000FBFBFBFBFBFBFCFCFC
FCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
0000008A8A8AD9D9D9FCFCFCE4E4E4D2D2D2CCCCCCBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBECCCCCCD2D2D2E4E4E4FCFCFCD9D9D98A8A8A000000FCFCFCFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000696969979797D9D9D9FCFCFCF2F2F2ECECECE3E3E3D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9E3E3E3ECECECF2F2F2FCFCFCD9D9D9979797696969000000FDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFF696969979797D3D3D3FCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFCFCFCFCFCFCD3D3D3979797696969FFFFFF000000FDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFF6969696969698A8A8A979797AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAA9797978A8A8A696969696969FFFFFFFFFFFF000000FEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{378, 208},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005j0000d2000`400?l00000o`00003oo`3ooomk
0?ooo`00o`3ooomk0?ooo`003P3ooooo0?knoUh0o_kn3`3oool000`0oooo103no_ko0?gmoEX0oOgm
103no_h=0?ooo`002P3oool30?kno_l0oOgmH03moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo5@0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?clo?l0no_kEP3kno/30?cl
o0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinEL0
nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_IF0?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`4
0?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gmo@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0
moOg0?KfmP3emOD00P3dm?Co0??cle@0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm00X0o?cl0?_kn`3inOT0
n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^k^iD0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0
m_Kf0?Shn03inOT0no_k0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj5@0j>SX00/0jNWY0>_[j`3]
kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0030?gmo@80o_kn103oool000<0oooo0100
o_kn0?gmo@3moOd0oOgm0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN5D0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3kno/0
o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`000P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0
m_Kf0??cl`3_knl0k>c/0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SHDP3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`00`3moOd20?knoP<0oooo
00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;R
hP3MgMd0fMWI0=GEe@3Bd]80d=3@o`3?clmB0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0oooo0040oooo00000`3o
ool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3A
dM40c<c<0<S8b036a/Ko0<G5aE80aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo0?knoP3no_h00P3moOd0
3`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl
_03o0;^k^e80^k^k00l0_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gmo@020?gmo@0?0?clo03j
n_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a
DP2a/K403`2b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd00P3moOd03`3lo?`0nOWi
0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYU80
YZJV00l0Z:RX0:f][@2c/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080oOgm00l0no_k0?Win@3f
m_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S80;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYiB09jN
WP0?0:2PX02UYJD0[Jf]0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0o?cl0?_kn`3inOT0m_Kf
0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l_;`0/[:b0:RXZ02PX:00o`2JVYYD09ZJVP0?0:2P
X02XZ:P0/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000006H000000100VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00003
0000003lo?`0o?cl0?l0o?clHP3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`00
0000o?cl0:^[Z`3o0:^[Zf40Zj^[0180o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFUP06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj9`2OWil00`2m_Kd0k>c/0>OWi`0_0>OWi`030>k^kP2=SHd0WinO00`0WinO00<0SHf=
0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3Ng]h0fm_K
00`0fm_K00@0g]kN09JFUP2FUYH0fm_K9`3:b/X00`3Kfm/0Xj>S0?_kn`0_0?_kn`0309JFUP3Ng]h0
fm_K00`0fm_K00<0g]kN09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo9`3@d=000`3oool0UYJF0?_kn`0_0?_kn`0309JFUP3oool0c/k>00`0c/k>00<0oooo
09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY
09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF
09JFUP3ooolW0<o?c`030?ooo`2FUYH0no_k02l0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ0000000=3@d0d];B0>3P
h03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3Lg=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph
0=WIf@0F0>3Ph00309JFUP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM401@3@d=0;0=7Ad@<0d=3@0`3AdM450=3@d0T0dM7A00<0oooo09JF
UP3jn_X0;`3jn_X00`2FUYH0oooo0<k>cP040<k>cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JF
UP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000
003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3P
h01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0103>c/h303hn?PD0c/k>
00@0oooo09JFUP2FUYH0oooo2`3Fe]H30=OGe`80e]KF00L0UIFE0=KFeP2EUID0e]KF09FEU@3Fe]H0
UIFE00X0e]KF0`3GemL30=KFeP030?ooo`2FUYH0nOWi02l0nOWi00<0UYJF0?ooo`3>c/h00`3>c/h3
03hn?PH0c/k>00<0oooo09JFUP3Ph>002@3Ph>020?ooo`80h>3P0P3oool05@3Ph>00JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3P
h0030?ooo`3Ph>00h>3P00@0h>3P0100a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0
d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0`3Ph>00203Lg=`0DE5A0;Vi^@2][Jd0E5AD0>3Ph01Y
JFT0WYjN5P3Ph>000`2FUYH0oooo0<o?c`030<o?c`<0?Shn1P3?cll0103oool0UYJF09JFUP3oool3
0=cLg0/0gMgM00T0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Lg=`;0=gMg@80
g=cL00<0gMgM0?ooo`2FUYH0<03inOT00`2FUYH0oooo0<o?c`040<o?c`<0?Shn1@3?cll00`3oool0
UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV
0=;BdP3Ph>00fMWI02HV9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103oool0UYJF09JFUP3oool3
0=cLg0/0gMgM00T0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Lg=`;0=gMg@80
g=cL00<0gMgM0?ooo`2FUYH0<03hn?P00`2FUYH0oooo0=CDe0050=CDe0<0?Shn103De=@00`3oool0
UYJF0>3Ph0080>3Ph080/k>c0P3Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00
103Ph>005`34a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00
fMWI02HV9P3Bd]80g=cL04U9B@0000003@d=0=;BdP1YJFT0WYjN00l0h>3P1@0000060>3Ph00409JF
UP3oool0fm_K0=_Kf`<0?Shn203Kfm/01@3oool0UYJF09JFUP3oool0i>CT0080hn?S0`3Ti>@50>?S
h`@0i>CT00P0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@80hn?S0`3Ti>@50>?Sh`@0i>CT
00@0hn?S0>CTi03oool0UYJF<03gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0?Shn0`3Kfm/00`3oool0
UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P01P0a<C404Q8B03IfMT0
fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0
h>3P06UYJ@1nOWh0a<C404Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0`3Zj^X80>WYj@090>[ZjP3oool0UIFE
0?ooo`2EUID0oooo09FEU@3oool0UIFE0080jNWY0`3Zj^X80>WYj@050>[ZjP3YjNT0jNWY0?ooo`2F
UYH0<03emOD00`2FUYH0oooo0>CTi0050>CTi0<0?Shn103Ti>@00`3oool0UYJF0>3Ph00=0>3Ph080
oooo01D0h>3P06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0030>3Ph00I08j>SP000000>3Ph0=WIf@3P
h>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00[Jf]05ADE03P
h>00JFUY07inOP3Ph>00JFUY09jNWP0@0>3Ph0030000003Ph>00h>3P00H0h>3P00<0UYJF0?ooo`3]
kNd00`3]kNd303hn?PH0kNg]00D0oooo09JFUP2FUYH0oooo0>[ZjP020>WYj@<0j^[Z203YjNT02@3Z
j^X0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@020>WYj@<0j^[Z203YjNT01@3Zj^X0jNWY
0>WYj@3oool0UYJF0300m?Cd00<0UYJF0?ooo`3]kNd0103]kNd303hn?PD0kNg]00<0oooo09JFUP3P
h>00303Ph>020;>c/`0F0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>006@3IfMT0
DU9B0=WIf@3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00
fMWI01XJ6P0000003@d=0=;BdP3Ph>00a<C401dM7@3Bd]80603Ph>000`2FUYH0oooo0?Cdm0040?Cd
m0<0?Shn1@3dm?@0103oool0UYJF09JFUP3oool?0>g]k@070?ooo`3]kNd0oooo0>g]k@3oool0kNg]
0?ooo`0A0>g]k@030?ooo`2FUYH0lO7a02l0lO7a00<0UYJF0?ooo`3dm?@00`3dm?@303hn?PH0m?Cd
00<0oooo09JFUP3Ph>00303Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3P
h>000`2FUYH0oooo0?clo0050?clo00303hn?P3lo?`0o?cl00@0o?cl00@0oooo09JFUP2FUYH0oooo
9`3]kNd00`3oool0UYJF0?3`l00_0?3`l00309JFUP3oool0o?cl00@0o?cl00<0?Shn0?clo03lo?`0
1@3lo?`00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0
UYJF0?ooo`3oool03@3oool209JFUP030?ooo`3gmoL0moOg02D0moOg00<0oooo09JFUP3_knl0;`3_
knl00`2FUYH0oooo0?ooo`0=0?ooo`0309JFUP3Ph>00h>3P00l0h>3P01@0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00=@3Ph>000`2FUYH0emOG0?ooo`0<0?ooo`040=OGe`2FUYH0UYJF0?[jnRL0oooo00<0
n_[j09JFUP3_knl0;`3_knl00`2FUYH0emOG0?ooo`0<0?ooo`030=OGe`2FUYH0h>3P0100h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`3Jf]X0TY:B0;Jf]P0;0;Fe]@050;Bd]02?Shl0
f][J0=KFeP2`/;009`2h^;P00`2YZJT0c/k>0>k^kP0_0>k^kP030=[JfP2BTY80][Jf00/0]KFe00<0
];Bd08n?S`3Jf]X04@3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJIL0WYjN00<0oooo0>3Ph03Ph>00=@3Ph>0A0>c/k2/0jn_[<03WinLA
0>c/k140h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom=0?oo
o`030>c/k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`0<0<k>
cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`00001=0000
00030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`050<k>
cP0303hn?P3>c/h0c/k>00@0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool00`3/k>`0UYJF0?ooo`040<k>
cP<0?Shn1@3>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00<0clo?1@0n?Sh40<o?
c`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3De=@303hn?P040=CDe00n?Sh0?Shn
03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooC@3oool0103/k>`0UYJF0?ooo`3Kfm/303hn?P<0fm_K0`0n
?Sh20=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool00P3Ti>@00`0n?Sh0i>CT0>CT
i0030>CTi00J03hn?P3Ti>@0i>CT0>CTi03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`D0EEEE0P3oool505EEE@80
oooo1@1EEED40?ooo`030>c/k02FUYH0oooo00`0kNg]01H0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo>P3oool00`1EEED0
oooo0?ooo`040?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0
UYJF0?ooo`0<0?Cdm00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTY0?ooo`80EEEE2P3oool205EEEB<0oooo00D0gmoO018B4P00
000000000;Bd]0030?ooo`040?_kn`1DE5@02@T90=;BdQP0oooo00@0no_k05ADE0092@T0d];B2@3o
ool0103Ogml04Q8B0451@@3gmoL30?ooo`040=oOg`0B4Q80@D510?Ogmj<0oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3o
ool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY:@3oool00`1EEED0oooo0?ooo`0:0?ooo`0305EEE@3oool0oooo02@0
oooo00<0moOg05iNGP3oool00P3oool00`3kno/0X:2P0?ooo`0I0?ooo`030?_kn`2PX:00oooo00`0
oooo00<0YJFU0?Ogm`3oool0103oool00`2UYJD0moOg0?ooo`2R0?ooo`0305EEE@3oool0oooo00@0
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00d0
oooo01D0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJA/0oooo0`1EEED30?ooo`<0EEEE00<0oooo05EEE@1EEED00P3oool00`1EEED0oooo
0?ooo`020?ooo`0305EEE@3oool0oooo0080EEEE0`3oool00`1EEED0oooo0?ooo`0B0?ooo`030::R
XP000000000000<0000000<0Shn?0?ooo`2RXZ800`000000102?Shl0oooo0?ooo`2RXZ8200000003
0451@@1SHf<0kno_0080oooo00<0no_k0:2PX03oool00`3oool01@3Ogml04Q8B000000000000H61P
0080oooo00P0XZ:R000000000000000008n?S`3oool0moOg01dM7@<0000000H0];Bd0?ooo`3oool0
oooo0?_kn`2PX:030?ooo`070::RXP0000003`l?0>OWi`0M7Ad0000008n?S`030?ooo`030:FUY@3g
moL0oooo00@0oooo00<0YJFU0?Ogm`3oool0XP3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTJ0?ooo`0705EEE@3oool0oooo0?ooo`1EEED0oooo05EEE@020?ooo`0305EEE@3oool0EEEE00<0
oooo00<0EEEE0?ooo`3oool00P3oool00`1EEED0oooo05EEE@020?ooo`0405EEE@3oool0oooo05EE
EAD0oooo00P0moOg05iNGP3oool0oooo0<K6aP2?Shl0moOg05iNGP<0oooo00@0Hf=S0>o_k`3Ogml0
De=C0P3oool00`2RXZ803`l?0>o_k`020?ooo`030?_kn`2PX:00oooo00<0oooo00P0GEeM0=;BdP3o
ool0oooo07QhN02d];@0moOg05iNGP<0oooo00P0Hf=S0>o_k`3gmoL0;2`/0>o_k`3oool0gmoO05=C
D`<0oooo00<0no_k0:2PX03oool00`3oool01@1hN7P0H61P0?ooo`1hN7P0H61P00@0oooo00<0YJFU
0?Ogm`3oool0103oool00`2UYJD0moOg0?ooo`2R0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0
];Bd08n?S`3Jf]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY6P3oool01`1EEED0oooo0?ooo`3oool0EEEE0?ooo`1EEED00P3oool00`1EEED0
oooo05EEE@030?ooo`0305EEE@3oool0EEEE00D0EEEE0P3oool01@1EEED0oooo0?ooo`1EEED0oooo
00@0EEEE203oool00`3kno/0E5AD000000020000000500l?3`3_knl0oooo0?Ogm`1NGUh00P3oool0
1036a/H0Shn?0<K6aP2?Shl30?ooo`040::RXP2d];@0a/K60;Bd]080oooo00<0moOg02`/;03_knl0
0P3oool00`3kno/0X:2P0?ooo`020?ooo`030<K6aP2?Shl0oooo00@0oooo00<0a/K608n?S`3oool0
0P3oool00`2RXZ80];Bd0?ooo`030?ooo`030=oOg`1CDe<0oooo0080oooo00<0no_k0:2PX03oool0
103oool00`1hN7P0B4Q8061PH0050?ooo`030:FUY@3gmoL0oooo00@0oooo00<0YJFU0?Ogm`3oool0
XP3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY6P3oool01`1EEED0oooo0?ooo`3oool0EEEE0?ooo`1EEED00P3oool00`1E
EED0oooo05EEE@030?ooo`0305EEE@3oool0EEEE0080oooo00<0EEEE0?ooo`1EEED00P3oool0101E
EED0oooo0?ooo`1EEEDE0?ooo`030?Ogm`1NGUh0oooo00<0oooo00<0a/K608n?S`3oool00P3oool0
102RXZ80];Bd0<K6aP2d];@20?ooo`030?Ogm`0/;2`0kno_0080oooo00<0no_k0:2PX03oool00P3o
ool00`2RXZ80];Bd0?ooo`040?ooo`030<K6aP2?Shl0oooo0080oooo00@0XZ:R0;Bd]03oool0a/K6
0`0000001P2d];@0oooo0?ooo`3oool0no_k0:2PX0D0oooo00<0XZ:R0000001PH6001@3oool00`2U
YJD0moOg0?ooo`040?ooo`030:FUY@3gmoL0oooo0:80oooo00<0EEEE0?ooo`3oool0103oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAX0oooo00L0EEEE0?oo
o`3oool0oooo05EEE@3oool0EEEE0080oooo101EEED20?ooo`0705EEE@3oool0oooo05EEE@3oool0
EEEE0?ooo`0305EEE@<0oooo00<0EEEE0?ooo`1EEED00`1EEED@0?ooo`030?Ogm`1NGUh0oooo00<0
oooo00h0moOg02HV9P3Bd]80oooo0=oOg`0Q8B40kno_0=oOg`0Q8B40kno_0?ooo`1hN7P03`l?0>o_
k`80oooo00<0no_k0:2PX03oool00P3oool00`2RXZ80];Bd0?ooo`040?ooo`0>0?Ogm`0V9RH0d];B
0?ooo`3Ogml08B4Q0>o_k`3oool0PH610=;BdP3oool0a/K600l?3`3_knl20?ooo`030?_kn`2PX:00
oooo00<0oooo00D0XZ:R0451@@3oool0N7Qh0;Bd]0040?ooo`030:FUY@3gmoL0oooo00@0oooo00<0
YJFU0?Ogm`3oool0XP3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY6P3oool01@1EEED0oooo0?ooo`3oool0EEEE00H0oooo
00D0EEEE0?ooo`3oool0oooo05EEE@030?ooo`80EEEE00D0oooo05EEE@3oool0oooo05EEE@020?oo
o`0305EEE@3oool0oooo01<0oooo00<0moOg05iNGP3oool0103oool01@3Ogml04Q8B000000092@T0
d];B0080oooo00H0a/K6000000000000H61P05ADE011@D420?ooo`030?_kn`2PX:00oooo0080oooo
00<0a/K608n?S`3oool01@3oool01@3Ogml04Q8B000000092@T0d];B0080oooo00H0gmoO018B4P00
00003`l?04i>CP3_knl20?ooo`030?_kn`2PX:00oooo0080oooo00L0a/K60000000?3`l0inOW01dM
7@000000];Bd00<0oooo00<0YJFU0?Ogm`3oool0103oool00`2UYJD0moOg0?ooo`2R0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTK0?ooo`<0EEEE2`3oool205EEE@<0oooo00<0EEEE0?ooo`3oool00P1EEED20?ooo`80EEEE
5@3oool00`3gmoL0GUiN0?ooo`0C0?ooo`030?_kn`2PX:00oooo0080oooo00L0moOg01dM7@2d];@0
oooo0=oOg`0B4Q80Shn?0100oooo00<0no_k0:2PX03oool0303oool00`2UYJD0moOg0?ooo`040?oo
o`030:FUY@3gmoL0oooo09l0oooo101EEED60?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?oo
o`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYB`3oool01@2RXZ800000000000000000];Bd0180oooo00@0no_k
05ADE0092@T0d];B0P3oool01P3gmoL07AdM000000092@T0UiNG08n?Sa00oooo00@0no_k05ADE009
2@T0d];B2@3oool0103Ogml04Q8B0451@@3gmoL30?ooo`040=oOg`0B4Q80@D510?Ogmj<0oooo00<0
EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3k
no/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo>@3oool205EEE@H0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?oo
o`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00<0EEEE0?ooo`1EEED01P3oool00`1EEED0oooo0?oo
o`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=`3oool0101EEED0oooo0?oo
o`1EEED60?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`0505EEE@3oool0oooo0?ooo`1EEED01P3oool00`1EEED0oooo0?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=@3oool605EEE@H0oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo1@1EEED60?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolj0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocX0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJBH0oooo0P1EEED:0?ooo`80EEEE:`3oool01@36a/H0
0000000000092@T0d];B01l0oooo00@0gmoO018B4P000000H61P603oool0103Ogml04Q8B0000001P
H6090?ooo`0407QhN00000002@T90=;BdR00oooo00@0gmoO018B4P000000H61P2@3oool0101hN7P0
000000T92@3Bd]840?ooo`030861P@3Bd]80oooo00X0oooo00@0N7Qh000000092@T0d];B@`3oool0
0`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY9P3oool00`1EEED0oooo0?ooo`0:0?ooo`0305EEE@3oool0oooo02`0oooo00<0XZ:R
0451@@3gmoL07P3oool00`3Ogml06a/K0=;BdP0I0?ooo`030=oOg`0K6a/0d];B00/0oooo00<0gmoO
01/K6`3Bd]800P3oool00`3gmoL09RHV0=;BdP0K0?ooo`030=oOg`0K6a/0d];B00/0oooo00<0gmoO
01/K6`3Bd]80103oool00`36a/H0H61P0?ooo`0;0?ooo`030=oOg`0K6a/0d];B04<0oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0
inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJAh0oooo00<0EEEE0?ooo`1EEED00P3oool0101EEED0oooo0?ooo`1EEED40?ooo`0305EEE@3o
ool0oooo0080EEEE0`3oool0101EEED0oooo0?ooo`1EEED;0?ooo`030=oOg`0B4Q80000000800000
00`02@T90=;BdP1hN7P0000000T92@3:b/X07AdM00000011@D40moOg0?ooo`36a/H2000000040451
@@3gmoL0oooo0<K6aP<0000000@0@D510?Ogm`3oool0XZ:R0P0000001@2BTY808B4Q0>o_k`3oool0
XZ:R0080000000X0@D510?Ogm`3oool0no_k05ADE00000003`l?09:BTP000000H61P0P3oool01`36
a/H0000000000011@D40moOg0?ooo`36a/H00`0000001`11@D40moOg0?ooo`3oool0gmoO01/K6`3B
d]800P3oool00`3Ogml0CTi>04Y:BP02000000030;Bd]03oool0a/K600D0000001/0];Bd07QhN000
00002@T90<[:bP0M7Ad000000451@@3gmoL0oooo0=oOg`0K6a/0d];B0?ooo`3kno/0E5AD00000009
2@T0b/[:01dM7@000000@D510?Ogm`3oool0gmoO01/K6`3Bd]800`3oool00`1hN7P0H61P0?ooo`05
0?ooo`0>07QhN000000000000451@@3gmoL0oooo0=oOg`0B4Q80000000T92@3Bd]80oooo0=oOg`0B
4Q830000000E0;Bd]03oool0oooo0?ooo`3Ogml06a/K0=;BdP3oool0no_k05ADE00000002@T90<[:
bP0M7Ad000000451@@3gmoL0oooo0=oOg`0K6a/0d];B00@0oooo0140gmoO024Q8@3_knl0oooo0?_k
n`1DE5@0000000T92@3:b/X07AdM00000011@D40moOg0?ooo`3Ogml06a/K0=;BdP130?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTN0?ooo`0305EEE@3oool0EEEE0080oooo00@0EEEE0?ooo`3oool0EEEE103oool00`1EEED0
oooo05EEE@020?ooo`0405EEE@3oool0oooo05EEEA00oooo00<0gmoO01/K6`3Bd]800P3oool0803k
no/0E5AD0;Bd]03oool0gmoO024Q8@3_knl0oooo0?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0<K6aP00
0000];Bd0?ooo`3gmoL07AdM0;Bd]036a/H02@T90=;BdP3oool0N7Qh00l?3`3_knl0oooo0?_kn`1D
E5@0];Bd0`3oool01P36a/H0@D510?Ogm`3gmoL07AdM061PH080oooo00d0no_k05ADE02d];@0oooo
07QhN02?Shl0a/K60000002d];@0oooo0?Ogm`0M7Ad0];Bd0080oooo00<0gmoO01/K6`3Bd]800P3o
ool01`3Ogml06a/K0=;BdP3oool0gmoO01/K6`3Bd]800P3oool00`3Ogml06a/K0=;BdP020?ooo`07
0?_kn`1DE5@0];Bd0?ooo`3Ogml08B4Q0>o_k`020?ooo`030=oOg`0K6a/0d];B00<0oooo00D0XZ:R
0451@@3clo<0E5AD08n?S`030?ooo`030=oOg`0K6a/0d];B00<0oooo00<0gmoO01/K6`3Bd]800`3o
ool05@3kno/0E5AD0451@@3gmoL0oooo07QhN02?Shl0gmoO018B4P2d];@0oooo0<K6aP092@T0d];B
0=oOg`0Q8B40kno_0?ooo`3Ogml06a/K0=;BdP020?ooo`030=oOg`0K6a/0d];B00<0oooo00D0XZ:R
0451@@3clo<0E5AD08n?S`030?ooo`030=oOg`0K6a/0d];B00@0oooo00<0no_k05ADE02?Shl00`3o
ool01@2RXZ80@D510??cl`1DE5@0Shn?00<0oooo00<0gmoO01/K6`3Bd]80@`3oool00`1EEED0oooo
0?ooo`040?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
7P3oool00`1EEED0oooo05EEE@020?ooo`0505EEE@3oool0oooo05EEE@3oool01P1EEED20?ooo`04
05EEE@3oool0oooo05EEE@@0oooo101EEED80?ooo`030=oOg`0K6a/0d];B0080oooo00/0no_k05AD
E02d];@0oooo0=oOg`0Q8B40kno_0?ooo`3gmoL09RHV0=;BdP030?ooo`0307QhN011@D40moOg00@0
oooo00/0N7Qh08n?S`3oool0oooo0=oOg`0Q8B40kno_0?ooo`3kno/0E5AD0;Bd]0030?ooo`060<K6
aP092@T0d];B0?ooo`2RXZ80H61P0P3oool00`3gmoL09RHV0=;BdP030?ooo`0307QhN011@D40moOg
00H0oooo00<0gmoO01/K6`3Bd]801P3oool00`3gmoL07AdM0;Bd]0020?ooo`030=oOg`0K6a/0d];B
0080oooo00L0no_k05ADE02d];@0oooo0=oOg`0Q8B40kno_0080oooo00<0gmoO01/K6`3Bd]80103o
ool0101hN7P0B4Q80451@@3gmoL30?ooo`030=oOg`0K6a/0d];B00@0oooo00<0N7Qh061PH03oool0
0P3oool00`1hN7P0@D510?Ogm`040?ooo`030::RXP1PH600oooo0080oooo00T0N7Qh08n?S`3oool0
oooo0?Ogm`0M7Ad0000000l?3`3_knl00P3oool00`3Ogml06a/K0=;BdP040?ooo`0407QhN018B4P0
@D510?Ogm`<0oooo00<0gmoO01/K6`3Bd]802`3oool0101hN7P0B4Q80451@@3gmoL30?ooo`030=oO
g`0K6a/0d];B04<0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJAh0oooo00<0EEEE0?ooo`1EEED00P3oool01P1EEED0oooo
0?ooo`1EEED0oooo05EEE@80oooo00<0EEEE0?ooo`1EEED00P3oool0101EEED0oooo0?ooo`1EEED@
0?ooo`030=oOg`0K6a/0d];B0080oooo00/0no_k05ADE02?Shl0oooo0=oOg`0Q8B40kno_0?ooo`3g
moL09RHV0=;BdP030?ooo`0307QhN0000000000000<0000000<0Shn?07QhN02?Shl00P3oool01`3O
gml08B4Q0>o_k`3oool0no_k05ADE01PH600103oool01@2RXZ800000000000000000H61P0080oooo
00<0moOg02HV9P3Bd]800`3oool00`1hN7P00000000000030000000608n?S`3oool0oooo0=oOg`0K
6a/0d];B103oool01@3Ogml04Q8B0000000?3`l0kno_0080oooo00<0gmoO01/K6`3Bd]800P3oool0
1`3kno/0E5AD08n?S`3oool0gmoO024Q8@3_knl00P3oool00`3Ogml06a/K0=;BdP040?ooo`030::R
XP000000Shn?00@0oooo00<0gmoO01/K6`3Bd]80103oool00`3Ogml06a/K0=;BdP020?ooo`0307Qh
N01PH600oooo00@0oooo00T0XZ:R061PH03oool0oooo0?_kn`1DE5@0Shn?0?_kn`1DE5@00P000000
0`0?3`l0kno_0?ooo`020?ooo`030=oOg`0K6a/0d];B00@0oooo00<0XZ:R0000002?Shl0103oool0
0`3Ogml06a/K0=;BdP0;0?ooo`030::RXP000000Shn?00@0oooo00<0gmoO01/K6`3Bd]80@`3oool0
0`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY7P3oool00`1EEED0oooo05EEE@0205EEE@<0oooo00L0EEEE0?ooo`3oool0EEEE0?oo
o`1EEED0oooo00<0EEEE0`3oool01@1EEED0oooo0?ooo`1EEED0oooo00@0EEEE203oool00`3Ogml0
6a/K0=;BdP020?ooo`0;0?_kn`1DE5@0De=C0>o_k`36a/H0@D510?Ogm`3oool0moOg02HV9P3Bd]80
0`3oool05@36a/H02@T90=;BdP3oool0a/K600T92@3Bd]80a/K600T92@3Bd]80oooo07QhN00?3`l0
kno_0?ooo`3kno/0E5AD0000001PH600a/K6061PH0040?ooo`070::RXP1PH600oooo0?ooo`3gmoL0
9RHV0=;BdP030?ooo`070<K6aP092@T0d];B0?ooo`36a/H02@T90=;BdP020?ooo`030=oOg`0K6a/0
d];B00<0oooo00@0N7Qh000000000000Shn?103oool00`3Ogml06a/K0=;BdP020?ooo`070?_kn`1D
E5@0De=C0>o_k`36a/H0@D510?Ogm`020?ooo`030=oOg`0K6a/0d];B00<0oooo00D0a/K600T92@3:
b/X07AdM0;Bd]0030?ooo`030=oOg`0K6a/0d];B00D0oooo00H0N7Qh061PH03oool0oooo07QhN01P
H6050?ooo`0>0=oOg`0B4Q80];Bd0?ooo`36a/H02@T90=;BdP3kno/0E5AD0;Bd]03oool0a/K60451
@@3gmoL20?ooo`030=oOg`0K6a/0d];B00<0oooo00D0a/K600T92@3:b/X07AdM0;Bd]0030?ooo`03
0=oOg`0K6a/0d];B00X0oooo00D0a/K600T92@3:b/X07AdM0;Bd]0030?ooo`030=oOg`0K6a/0d];B
04<0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo
0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJAh0oooo00<0EEEE0?ooo`3oool01@3oool0101EEED0oooo0?ooo`3oool2
05EEE@050?ooo`1EEED0oooo0?ooo`1EEED00P3oool00`1EEED0oooo0?ooo`0>0?ooo`030=oOg`0K
6a/0d];B0080oooo00T0N7Qh0000002/[:`07AdM00T92@3Bd]80oooo0=oOg`0B4Q800`0000001011
@D40moOg0?ooo`36a/H3000000040;Bd]03oool0oooo0::RXP80000000`0TY:B018B4P0?3`l0clo?
018B4P000000[:b/01dM7@000000];Bd0?_kn`1DE5@20000000500l?3`3_knl0oooo0=oOg`0B4Q80
0`0000001011@D40moOg0?ooo`36a/H3000000070;Bd]03oool0oooo0?ooo`3Ogml06a/K0=;BdP02
0?ooo`030?_kn`1DE5@0];Bd00@0oooo00D0no_k05ADE00000002@T90=;BdP020?ooo`0607QhN000
0000[:b/01dM7@092@T0d];B0`3oool00`3Ogml06a/K0=;BdP020?ooo`070::RXP0000002@T90;:b
/P0B4Q80000008n?S`020?ooo`030=oOg`0K6a/0d];B00D0oooo00L0gmoO01/K6`3Bd]80oooo07Qh
N011@D40moOg0080oooo00P0N7Qh0;Bd]03oool0gmoO018B4P0000002@T90=;BdP80oooo00H0gmoO
018B4P00000000000451@@3gmoL20?ooo`030=oOg`0K6a/0d];B0080oooo00L0XZ:R000000092@T0
/[:b018B4P000000Shn?0080oooo00<0gmoO01/K6`3Bd]802@3oool01`2RXZ80000000T92@2b/[80
4Q8B0000002?Shl00P3oool00`3Ogml06a/K0=;BdP130?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00
lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTN0?ooo`0305EEE@3o
ool0oooo00D0oooo0P1EEED30?ooo`0305EEE@3oool0oooo0080EEEE0P3oool205EEEA00oooo00<0
gmoO01/K6`3Bd]802P3oool00`3gmoL09RHV0=;BdP0P0?ooo`030?Ogm`0V9RH0d];B00`0oooo00<0
gmoO01/K6`3Bd]800P3oool01`3kno/0E5AD08n?S`3oool0XZ:R00l?3`3_knl0403oool00`3Ogml0
6a/K0=;BdP0;0?ooo`030=oOg`0K6a/0d];B00H0oooo00X0N7Qh061PH03oool0no_k05ADE011@D40
moOg0=oOg`0B4Q80];Bd403oool00`3Ogml06a/K0=;BdP0;0?ooo`030=oOg`0K6a/0d];B0180oooo
00<0gmoO01/K6`3Bd]80@`3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY@P3oool00`3Ogml04Q8B000000020000000300T9
2@3Bd]80oooo00L0oooo00<0moOg02HV9P3Bd]80803oool00`3gmoL09RHV0=;BdP0<0?ooo`040=oO
g`0B4Q800000061PH080oooo00H0moOg01dM7@000000QhN702`/;03_knl20?ooo`030::RXP0?3`l0
kno_00/0oooo00@0gmoO018B4P000000H61P2@3oool0101hN7P0000000T92@3Bd]860?ooo`030=oO
g`0Q8B40kno_0080oooo00D0N7Qh00000011@D40L71`0;Bd]00@0?ooo`040=oOg`0B4Q800000061P
H0T0oooo00@0N7Qh000000092@T0d];B4@3oool0101hN7P0000000T92@3Bd]930?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocX0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo>P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolj0?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocX0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool505EEE@80oooo1@1EEED60?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`030>c/k03`
l?00SXj>00D0X:2P00<0WinO0:2PX02PX:00102PX:005P2@T900k>c/06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3o
ool0oooo0080oooo00<0k>c/09JFUP3Ng]h0303Cdm<05P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
odP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT
0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0
emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolm0?ooo`D0EEEE1P3oool00`1EEED0oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWl0?ooo`050?_kn`1DE5@000000451@@3gmoL0?`3oool00`1EEED0oooo0?ooo`040?oo
o`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0
gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOd0oooo00<0moOg02`/;03_knl0@03oool00`1EEED0
oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOd0oooo00<0moOg02`/;03_knl0
@03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJCh0oooo00D0
N7Qh000000000000@D510?Ogm`020?ooo`0?07QhN00000003`l?0>o_k`2RXZ8000000451@@2JVYX0
00000451@@33`l<0?clo0000000?3`l0kno_00<0oooo00d0XZ:R00000000000000000451@@1`L700
0000061PH02RXZ8000000451@@3gmoL0N7Qh00<0000000`0];Bd0?ooo`3oool0oooo07QhN0000000
H61P0::RXP000000@D510?Ogm`2RXZ820000000=0;Bd]03oool0oooo07QhN0000000];Bd0?ooo`1h
N7P0000000l?3`3_knl0no_k05ADE002000000040;Bd]03oool0oooo0::RXP80000000@0];Bd0?oo
o`3oool0XZ:R0P0000003@2d];@0oooo07QhN0000000H61P0<K6aP0000003`l?0;^k^`0o?cl00000
00l?3`3_knl00`3oool03P2RXZ800000000000000000@D51071`L0000000@D510?Ogm`3oool0N7Qh
0000000?3`l0kno_0P3oool02P3kno/0E5AD000000092@T0d];B0?ooo`1hN7P0];Bd0?_kn`1DE5@2
000000030451@@2JVYX000000080000001X0@D510?Ogm`1hN7P000000;Bd]01hN7P00000061PH02R
XZ8000000451@@3gmoL0N7Qh0000000?3`l0kno_07QhN0000000H61P07QhN0000000@D5109ZJVP00
0000@D5109ZJVP<0000000h0@D510?Ogm`1hN7P000000;Bd]02RXZ8000000451@@3clo<0E5AD0000
00092@T0d];B0::RXP<0000000<0@D510?Ogm`3oool00P3oool02`1hN7P0000000l?3`3[jn/0E5AD
061PH03Ogml04Q8B0451@@3_knl07AdM0080000001/02@T90=;BdP3oool0N7Qh00000011@D40moOg
0?ooo`1hN7P0000000l?3`3_knl0bl_;03lo?`0000003`l?0>o_k`3;bl/0?clo0000000?3`l0kno_
0::RXP000000@D510?Ogm`2RXZ800P000000402d];@0oooo07QhN0000000H61P0::RXP000000@D51
0<?3``0o?cl0000000l?3`3_knl0gmoO01/K6`3Bd]8B0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY?@3oool05`1hN7P0@D510?Ogm`3oool0gmoO05=CD`3gmoL0
a/K6061PH03oool0moOg07emO@3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP3;bl/0d];B0?ooo`36a/H0
Shn?00<0oooo0180N7Qh0;Bd]03kno/0GEeM0=;BdP3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP36a/H0
3`l?0>o_k`3kno/0E5AD00l?3`3_knl20?ooo`0G0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0=oOg`0K
6a/0d];B0?_kn`1MGEd0d];B0?_kn`1MGEd0d];B0?ooo`36a/H0H61P0?ooo`3gmoL0OGem0=oOg`1C
De<00P3gmoL06@0V9RH0d];B0=oOg`0K6a/0d];B0?_kn`1MGEd0d];B0=oOg`0K6a/0d];B0?_kn`1M
GEd0d];B0?Ogm`0/;2`0kno_07QhN02?Shl0oooo0<_;b`3Bd]80oooo0<K6aP2?Shl00`3oool03P1h
N7P0];Bd0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?ooo`36a/H0H61P0?ooo`3gmoL0OGem103oool0
?P2][Jd0kno_0?ooo`3Ogml06a/K06i^KP3gmoL0oooo0::RXP2?Shl0oooo07QhN02d];@0no_k05eM
G@3Bd]80no_k05eMG@3Bd]80oooo0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0<K6aP1PH600oooo0?Og
m`1mOGd0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0N7Qh0;Bd]03kno/0GEeM0=;B
dP3kno/0GEeM0=;BdP3oool0no_k05eMG@3Bd]80XZ:R061PH03oool0moOg0:BTY01hN7P0];Bd0?_k
n`1MGEd0d];B0`3oool0>036a/H0H61P0?ooo`3gmoL0OGem0=oOg`1bLW80no_k05eMG@3Bd]80moOg
02HV9P3Bd]80oooo0?Ogm`0V9RH0d];B0?Ogm`0/;2`0kno_0?ooo`36a/H0H61P0?ooo`3gmoL0OGem
0<_;b`3Bd]80oooo0<K6aP2?Shl0bl_;0=;BdP3oool0a/K608n?S`3kno/0GEeM0=;BdP3Ogml06a/K
0=;BdP3kno/0GEeM0=;BdP3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3;bl/0d];B0?ooo`36a/H0Shn?
5@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<0
gmoO01/K6`3Bd]800`3oool01036a/H0];Bd0::RXP2d];@30?ooo`060?_kn`1MGEd0d];B0?_kn`1M
GEd0d];B0P3oool00`2RXZ8000000;Bd]0030?ooo`0A0<K6aP0?3`l0jn_[05eMG@3Bd]80moOg02`/
;03_knl0no_k05eMG@3Bd]80XZ:R08n?S`3oool0no_k05eMG@3Bd]800`3oool0203gmoL0;2`/0>o_
k`3kno/0GEeM0=;BdP2RXZ80H61P0P3oool0202RXZ80Shn?0?_kn`1MGEd0d];B0?ooo`2RXZ80];Bd
0`3oool0203Ogml0De=C0?Ogm`3oool0XZ:R08n?S`2RXZ80H61P0P3oool0102RXZ80Shn?0::RXP1P
H6020?ooo`070::RXP2?Shl0moOg02HV9P2GUiL0@D510?Ogm`030?ooo`030::RXP000000];Bd00<0
oooo00/0a/K600l?3`3[jn/0GEeM0=;BdP3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]0070?ooo`0L0:f]
[@3_knl0oooo07QhN02?Shl0a/K60;Bd]03oool0XZ:R08n?S`3oool0a/K600l?3`3[jn/0GEeM0=;B
dP3kno/0GEeM0=;BdP3oool0moOg02`/;03_knl0no_k05eMG@3Bd]80XZ:R0;Bd]0<0oooo01L0moOg
02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0a/K600l?3`3[jn/0GEeM0=;BdP3kno/0GEeM
0=;BdP3oool0no_k05eMG@3Bd]80N7Qh0;Bd]0030?ooo`050<K6aP0?3`l0jn_[05eMG@3Bd]800`3o
ool00`2RXZ80];Bd0?ooo`030?ooo`070<_;b`0o?cl0Shn?0?ooo`3gmoL0;2`/0>o_k`020?ooo`08
0::RXP2?Shl0moOg02`/;03_knl0oooo0::RXP2d];@50?ooo`030::RXP000000];Bd0080oooo00P0
XZ:R0000002d];@0no_k05eMG@3Bd]80XZ:R061PH080oooo00P0XZ:R08n?S`3gmoL0;2`/0>o_k`3k
no/0GEeM0=;BdP80oooo00<0XZ:R0000002d];@05@3oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KF
eP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJC`0oooo00<0XZ:R0451@@3gmoL01@3oool03`2RXZ8000000000
00000000H61P0?_kn`1MGEd0d];B0?_kn`1MGEd0d];B0=oOg`0B4Q80@D510?Ogm`050?ooo`0@0?Og
m`0M7Ad02@T90=;BdP3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP2RXZ80Shn?0?ooo`3kno/0GEeM0=;B
dP<0oooo00P0moOg02`/;03_knl0no_k05eMG@3Bd]80XZ:R08n?S`80oooo00L0N7Qh08n?S`3kno/0
GEeM0=;BdP3oool0XZ:R00<0000000T0H61P0=oOg`1CDe<0moOg0?ooo`2RXZ80Shn?0::RXP2?Shl0
0P3oool0101hN7P0Shn?0::RXP2?Shl20?ooo`0607QhN02?Shl0moOg02HV9P0o?cl0H61P0P3oool0
103Ogml04Q8B0451@@3gmoL50?ooo`090?Ogm`0M7Ad02@T90=;BdP3gmoL0;2`/0>o_k`3oool0XZ:R
00<0000000<0H61P0?ooo`3oool00P3oool02P2][Jd0kno_0?Ogm`0/;2`0kno_0?_kn`2PX:00oooo
0::RXP2?Shl20?ooo`0?0?Ogm`0M7Ad02@T90=;BdP3kno/0GEeM0=;BdP3oool0moOg02`/;03_knl0
no_k05eMG@3Bd]80XZ:R00<0000001P0H61P0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0
];Bd0?ooo`3gmoL07AdM00T92@3Bd]80no_k05eMG@3Bd]80oooo0?_kn`1MGEd0d];B0861P@3Bd]84
0?ooo`040?Ogm`0M7Ad02@T90=;BdP<0oooo00d0XZ:R0000000000000000061PH03oool0a/K600l?
3`3_knl0oooo0?Ogm`0/;2`0kno_0080oooo00L0XZ:R08n?S`3gmoL0;2`/0>o_k`3oool0XZ:R00<0
00000100H61P0=oOg`0B4Q80@D510?Ogm`3oool0gmoO018B4P11@D40moOg0?ooo`3kno/0GEeM0=;B
dP2RXZ80Shn?0P3oool0301hN7P0Shn?0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0=oOg`0B4Q80@D51
0?OgmaH0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030::RXP11@D40moOg00D0oooo0100a/K608n?S`3oool0N7Qh08n?S`3kno/0GEeM0=;BdP3kno/0
GEeM0=;BdP36a/H0Shn?0?ooo`21PH40d];B0`3oool04@2RXZ80Shn?0?_kn`1MGEd0c/k>05ADE000
0000H61P0?Ogm`0V9RH0d];B0=oOg`0Q8B40kno_0=oOg`0K6a/0d];B0080oooo03<0no_k05ADE000
0000H61P0?Ogm`0V9RH0d];B0=oOg`1CDe<0moOg0=oOg`0B4Q80];Bd0=_Kf`1MGEd0d];B0?ooo`36
a/H0Shn?0?ooo`1hN7P0Shn?0=oOg`0B4Q80];Bd0?_kn`1DE5@0];Bd0=oOg`1CDe<0moOg0=oOg`0B
4Q80];Bd0=oOg`1CDe<0moOg0=oOg`0B4Q80];Bd0?Ogm`0/;2`0kno_0<_;b`3Bd]80oooo0<K6aP2?
Shl0oooo0861P@3Bd]800`3oool03P2RXZ80Shn?0?_kn`1MGEd0c/k>05ADE0000000Shn?0?ooo`36
a/H0Shn?0?ooo`1hN7P0Shn?103oool0102][Jd0kno_0::RXP2?Shl20?ooo`0h0:RXZ03Bd]80XZ:R
08n?S`3oool0XZ:R08n?S`3kno/0GEeM0=;BdP3Kfm/0GEeM0=;BdP3oool0moOg01dM7@1PH600moOg
02HV9P3Bd]80a/K608n?S`3oool0N7Qh08^;R`1DE5@0000008n?S`3gmoL07AdM061PH03kno/0GEeM
0=;BdP2RXZ80Shn?0?_kn`1MGEd0d];B0=_Kf`1MGEd0d];B0?ooo`1hN7P02@T90=;BdP2RXZ80Shn?
0?ooo`1hN7P0];Bd0::RXP2?Shl0no_k05eMG@3Bd]830?ooo`0h0<K6aP2?Shl0oooo07QhN02?Shl0
moOg02`/;03WinL0Y:BT0?_kn`1DE5@00000061PH03oool0gmoO018B4P2`/;00E5AD0000002?Shl0
oooo0<K6aP2?Shl0oooo07QhN02?Shl0a/K608n?S`3oool0PH610=;BdP36a/H0Shn?0?ooo`21PH40
d];B07QhN0092@T0d];B0=oOg`1CDe<0moOg0=oOg`0B4Q80/;2`05ADE0000000H61P0?Ogm`0V9RH0
d];B0<K6aP2?Shl0oooo0861P@3Bd]8E0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY?03oool00`2RXZ80@D510?Ogm`060?ooo`0?07QhN0000000@D510?Ogm`3k
no/0GEeM0=;BdP3kno/0GEeM0=;BdP3kno/0E5AD0000002FUYH0fm_K00<0oooo00@0gmoO018B4P00
0000@D510P3gmoL01`0/;2`0][Jf00000011@D40moOg0?ooo`36a/H00P0000000`092@T0d];B0?oo
o`020?ooo`080?Ogm`0/;2`0][Jf00000011@D40moOg0?ooo`2RXZ820000000308n?S`3oool0a/K6
0080000000/0];Bd0?ooo`1hN7P000000451@@3gmoL0gmoO05=CD`1`L7000000061PH0020?ooo`04
0::RXP000000000008n?S`80oooo0100XZ:R000000000000Shn?0?ooo`3gmoL0;2`/0>o_k`2RXZ80
00000451@@3clo<0E5AD0000002FUYH0fm_K0`3oool0103Ogml04Q8B00000011@D420?Ogm`0802`/
;02BTY802@T90=;BdP1hN7P000000451@@3gmoL40?ooo`040:f][@3WinL0;2`/0>o_k`80oooo00/0
moOg07emO@2RXZ80Shn?0?ooo`3Ogml04Q8B00000011@D40moOg0<K6aP020000000K0;Bd]03gmoL0
;2`/0;Jf]P000000@D510?Ogm`3oool0N7Qh00000011@D40moOg0=oOg`1CDe<0VYZJ00000011@D40
_[jn00000011@D40moOg0=oOg`0B4Q8000000451@@3gmoL0a/K60080000000h0];Bd0?Ogm`0V9RH0
d];B0?ooo`1hN7P0000000l?3`3_knl0gmoO018B4P000000@D510?Ogm`@0oooo00`0N7Qh00000011
@D40lo?c05ADE0000000TY:B018B4P2?Shl0moOg02`/;02f][H20000000J08n?S`3oool0moOg02`/
;02BTY802@T90=;BdP1hN7P000000451@@3gmoL0no_k05ADE0000000UYJF0=_Kf`3kno/0E5AD0000
002FUYH0fm_K0?Ogm`0V9RH0d];B0?ooo`2RXZ820000000=08n?S`3oool0moOg02`/;02f][H00000
0451@@3gmoL0no_k05ADE0000000UYJF0=_Kf`0E0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUY?03oool00`3Ogml06a/K0=;BdP030?ooo`030:RXZ03Bd]80oooo
00@0oooo00H0no_k05eMG@3Bd]80no_k05eMG@3Bd]8F0?ooo`030?_kn`1MGEd0d];B0100oooo00<0
PH610=;BdP3oool01@3oool00`3Ogml0De=C0?Ogm`0?0?ooo`030?Ogm`0/;2`0kno_01d0oooo00<0
[Jf]09:BTP1PH600103oool00`2][Jd0TY:B08n?S`070?ooo`060861P@3Bd]80oooo0?Ogm`0/;2`0
kno_5`3oool00`21PH40d];B0?ooo`1M0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0dM7A0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTm
0?ooo`0707QhN011@D40moOg0?ooo`2RXZ802@T90=;BdP050?ooo`0607QhN0092@T0d];B07QhN009
2@T0d];B5P3oool00`1hN7P02@T90=;BdP0@0?ooo`030:RXZ03Bd]80oooo00@0oooo00@0moOg01dM
7@11@D40moOg3P3oool0103kno/0E5AD00l?3`3_knlM0?ooo`030::RXP092@T0d];B00@0oooo00<0
gmoO018B4P2?Shl01`3oool01P2XZ:P0d];B0?_kn`1DE5@03`l?0>o_kaL0oooo00<0Z:RX0=;BdP3o
ool0G@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00D0k>c/09JFUP3o
ool0moOg0>g]k@060?ooo`0J0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?P3oool01P2RXZ8000000000
002?Shl0Z:RX0=;BdPD0oooo00H0moOg02HV9P3Bd]80moOg02HV9P3Bd]8F0?ooo`030?_kn`1MGEd0
d];B01P0oooo00<0gmoO05=CD`3gmoL03`3oool00`3Ogml08B4Q0>o_k`0K0?ooo`040?_kn`1DE5@0
0000061PH0H0oooo00@0N7Qh00000011@D40moOg203oool00`3Ogml08B4Q0>o_k`0J0?ooo`030?Og
m`0V9RH0d];B0340oooo00<0moOg02HV9P3Bd]809P3oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd609FEU@0I0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool01@3/
k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]1P2EUID06@3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool405EEE@H0oooo00<0EEEE0?oo
o`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oood40oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?@3oool505EE
E@H0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oom80?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CT
i03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`03
05EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM
0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooodP0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0
oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0
oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gM
g@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF
0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooom80?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
ooooB03oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3S
hn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocd0oooo1@1EEED60?ooo`0305EE
E@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KF
eP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJBH0oooo00D0gmoO018B4P0000003`l?0>o_k`3o0?oooaH0oooo00<0
EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0emOG00`0oooo01H0
emOG09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJBH0oooo00D0gmoO018B4P0000000000061PH03o0?oooaH0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k03moOd0Ti>C00D0^;Rh00<0]KFe0;Rh^02h^;P0
102h^;P05P2CTi<0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY:03oool0103Ogml04Q8B00T92@3Bd];o0?oooaD0oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTN0?ooo`030<K6aP00000000000080
000000<0Shn?0?ooo`3oool00P3oool0103gmoL07AdM0000002?Shl30?ooo`0C0=oOg`0B4Q802@T9
0=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_knl0no_k05ADE0000000[:b/
01dM7@020000000308n?S`3oool0oooo0080oooo00H0moOg01dM7@000000000000T92@3Bd]820?oo
o`080?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@20?ooo`060<K6aP000000000000000011
@D40moOg1P3oool0302RXZ80000000000011@D40kno_01dM7@092@T0d];B0?ooo`1hN7P00000061P
H080oooo00<0XZ:R0000001PH6000`3oool01`2RXZ80000000000011@D40VYZJ0000001PH6001@3o
ool0203Ogml04Q8B00T92@3Bd]80oooo07QhN0000000H61P0P3oool00`2RXZ800000061PH0020?oo
o`070=oOg`0B4Q80000000T92@3Bd]80oooo07QhN0030000000708n?S`3oool0oooo0?Ogm`0M7Ad0
2@T90=;BdP030?ooo`080::RXP00000000000451@@3_knl07AdM00T92@3Bd]820?ooo`060<K6aP00
0000000000000011@D40moOg0P3oool0203Ogml04Q8B000000092@T0b/[:01dM7@000000];Bd0`3o
ool00`1hN7P00000061PH0050?ooo`0507QhN000000000000000002?Shl01P3oool00`3gmoL07AdM
000000050000000308n?S`3oool0oooo00<0oooo00H0moOg01dM7@000000000000T92@3Bd]840?oo
o`060<K6aP000000000000000011@D40moOg0`3oool05P1hN7P0000000000011@D40VYZJ0000001P
H600oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_
knl20?ooo`0507QhN000000000000000002?Shl00`3oool00`1hN7P00000061PH0020?ooo`030::R
XP000000H61P0080oooo00L0gmoO018B4P0000002@T90=;BdP3kno/0E5AD00<0000000<0@D510?Og
m`3oool03P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^
kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JAd0oooo00<0XZ:R000000000000100000001@2?Shl0oooo0?ooo`3oool0a/K60080000001@0@D51
0?Ogm`3oool0oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H00000
00l?3`3_knl0no_k05ADE0H0000000@0Shn?0?ooo`3oool0a/K61P000000302d];@0oooo0?Ogm`0M
7Ad000000;Bd]03gmoL07AdM0000002d];@0oooo0<K6aPD0000000<03`l?0>o_k`3oool00`3oool0
0`36a/H0000000000004000000060451@@3gmoL0oooo07QhN0000000H61P0P3oool00`2RXZ800000
061PH0020?ooo`030::RXP000000000000@0000000<0H61P0?ooo`3oool00`3oool0203Ogml04Q8B
00T92@3Bd]80oooo07QhN0000000H61P0P3oool00`2RXZ800000061PH0020?ooo`0607QhN0000000
000000T92@3Bd]80N7Qh1@0000001P2?Shl0oooo0?Ogm`0M7Ad02@T90=;BdP80oooo00<0a/K60000
00000000100000001011@D40moOg0?ooo`36a/H50000000400l?3`3_knl0oooo07QhN080000000D0
2@T90<[:bP0M7Ad000000;Bd]0020?ooo`050?Ogm`0M7Ad0000000l?3`3_knl00`3oool00`1hN7P0
0000000000030000000308n?S`3oool0oooo00<0oooo00<0moOg01dM7@0000001P0000001@11@D40
moOg0?ooo`3oool0a/K600H0000000@0];Bd0?ooo`3oool0a/K61@000000100?3`l0kno_0?ooo`36
a/H60000000B061PH03oool0gmoO018B4P092@T0d];B0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0<K6
aP0000003`l?0>o_k`3oool0N7Qh1@0000001P2?Shl0oooo0?ooo`1hN7P00000061PH080oooo00<0
XZ:R0000001PH6000P3oool01P1hN7P00000000000092@T0c/k>05ADE0D0000000<0@D510?Ogm`3o
ool03@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0
oooo00@0moOg01dM7@000000Shn?0P3oool0102RXZ80000000l?3`3_knl20?ooo`0507QhN0000000
000000T92@3Bd]800P3oool08@3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0
a/K60000000?3`l0kno_0?_kn`1DE5@0000000l?3`3_knl0oooo0::RXP0000003`l?0>o_k`3kno/0
E5AD00000011@D40moOg0?ooo`1hN7P000000451@@020?Ogm`0@01dM7@000000];Bd0?Ogm`0M7Ad0
00000;Bd]03gmoL07AdM0000002?Shl0oooo0?_kn`1DE5@0000008n?S`@0oooo00d0XZ:R00000011
@D40moOg0?_kn`1DE5@000000451@@3gmoL0oooo07QhN0000000H61P0080oooo00d0XZ:R0000001P
H600oooo0?Ogm`0M7Ad00000061PH03oool0moOg01dM7@000000H61P00D0oooo00P0gmoO018B4P09
2@T0d];B0?ooo`1hN7P00000061PH080oooo01H0XZ:R0000001PH600oooo0?_kn`1DE5@0000008n?
S`3oool0gmoO018B4P000000];Bd0?ooo`36a/H0000000l?3`3_knl0moOg01dM7@092@T0d];B0P3o
ool0202RXZ8000000451@@3gmoL0no_k05ADE0000000@D510P3gmoL04@0M7Ad0000008n?S`3oool0
no_k05ADE0000000Shn?0?_kn`1DE5@0000008n?S`3oool0moOg01dM7@000000];Bd0080oooo00D0
XZ:R0000001dM7@000000;Bd]0020?ooo`090=oOg`0B4Q8000000;Bd]03oool0a/K60000000?3`l0
kno_00@0oooo00@0moOg01dM7@000000];Bd0`3oool03P3Ogml04Q8B0000002?Shl0oooo0?_kn`1D
E5@000000451@@3gmoL0oooo07QhN0000000@D510P3gmoL0300M7Ad0000008n?S`3oool0no_k05AD
E0000000Shn?0?ooo`1hN7P000000451@@80moOg01l07AdM0000001PH600oooo0=oOg`0B4Q802@T9
0=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0000000l?3`3_knl0gmoO018B4P000000];Bd
0?ooo`36a/H0000000l?3`3_knl0oooo07QhN0000000H61P0080oooo0180XZ:R0000001PH600oooo
0?_kn`1DE5@0000008n?S`3oool0a/K6000000092@T0d];B0?ooo`3gmoL07AdM00T92@3Bd]8>0?oo
o`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703oool01036
a/H0000000l?3`3_knl20?ooo`0;0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0][Jf0000
001PH6000P3oool04P3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K60000
000?3`l0kno_0?_kn`1DE5@0000008n?S`80oooo00P0moOg01dM7@092@T0d];B0=oOg`0B4Q802@T9
0=;BdP80oooo0100moOg01dM7@092@T0d];B0?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@0
gmoO018B4P092@T0d];B2@3oool03@36a/H000000000002d];@0oooo0::RXP000000@D510?Ogm`3o
ool0N7Qh0000001PH6000P3oool0202RXZ800000061PH03oool0gmoO018B4P092@T0d];B0P3oool0
0`2RXZ800000061PH0050?ooo`080=oOg`0B4Q802@T90=;BdP3oool0N7Qh0000001PH6020?ooo`0=
0::RXP000000H61P0?ooo`3kno/0E5AD0000002d];@0oooo0::RXP000000@D510?Ogm`050?ooo`04
0?Ogm`0M7Ad02@T90=;BdP80oooo00d0a/K6000000000000];Bd0?ooo`2RXZ8000000451@@3gmoL0
gmoO018B4P092@T0d];B00D0oooo01H0no_k05ADE0000000];Bd0?ooo`3gmoL07AdM0000002d];@0
oooo0?_kn`1DE5@03`l?0<o?c`0B4Q80@D510?Ogm`3oool0XZ:R00000011@D40moOg2@3oool0103g
moL07AdM0000002d];@40?ooo`080<K6aP0000003`l?0>o_k`3Ogml04Q8B00T92@3Bd]820?ooo`08
0?Ogm`0M7Ad02@T90=;BdP3Ogml04Q8B00T92@3Bd]860?ooo`0307QhN0000000H61P0080oooo01H0
N7Qh0000001PH600oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P0?3`l0kno_0?ooo`36a/H0
000000l?3`3_knl0XZ:R00000011@D40moOg1P3oool00`1hN7P00000061PH0020?ooo`080::RXP00
0000H61P0?ooo`3kno/0E5AD0000002d];@40?ooo`060<K6aP0000000000000000092@T0d];B3P3o
ool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_
0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJB40oooo0200
moOg01dM7@0000003`l?0>o_k`3oool0XZ:R0000001PH600moOg01dM7@0?3`l0kno_0?ooo`3Ogml0
4Q8B00T92@3Bd]80oooo0=oOg`0B4Q802@T90=;BdP3oool0a/K60000000?3`l0kno_0?_kn`1DE5@0
00000;Bd]080oooo00P0moOg01dM7@000000];Bd0=oOg`0B4Q803`l?0>o_k`80oooo0100moOg01dM
7@000000];Bd0?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@0gmoO018B4P0?3`l0kno_2@3o
ool00`3kno/0E5AD00000004000000060451@@3gmoL0oooo07QhN0000000H61P0P3oool0202RXZ80
0000061PH03oool0gmoO018B4P0?3`l0kno_0P3oool00`2RXZ800000061PH0050?ooo`080=oOg`0B
4Q802@T90=;BdP3oool0N7Qh0000001PH6020?ooo`0:0::RXP000000H61P0?ooo`3kno/0E5AD0000
002d];@0oooo0::RXPL0000000D0];Bd0?Ogm`0M7Ad000000;Bd]0020?ooo`030?_kn`1DE5@00000
00@0000000H0@D510?Ogm`3Ogml04Q8B00l?3`3_knl50?ooo`0C0?_kn`1DE5@000000;Bd]03oool0
moOg01dM7@000000];Bd0?ooo`3Ogml04Q8B0451@@3clo<0E5AD00T92@3Bd]80oooo0::RXP070000
00030;Bd]03oool0oooo0080oooo00@0moOg01dM7@000000];Bd103oool0203Ogml04Q8B00T92@3B
d]80gmoO018B4P0?3`l0kno_0P3oool0203gmoL07AdM0000002d];@0gmoO018B4P0?3`l0kno_1P3o
ool00`1hN7P00000061PH0020?ooo`0C0::RXP000000H61P0?ooo`3Ogml04Q8B00T92@3Bd]80oooo
0=oOg`0B4Q802@T90=;BdP3oool0a/K60000000?3`l0kno_0::RXP07000000050;Bd]03oool0N7Qh
0000001PH6000P3oool0202RXZ800000061PH03oool0no_k05ADE0000000];Bd0P3oool00`2RXZ80
00000000000300000003061PH03oool0oooo00d0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTO0?ooo`030::RXP00000000000080000001d0Shn?0?ooo`3k
no/0E5AD0000002d];@0oooo07QhN0000000];Bd0?ooo`3Ogml04Q8B00T92@3Bd]80oooo0=oOg`0B
4Q802@T90=;BdP3oool0a/K60000000?3`l0kno_0?_kn`1DE5@000000;Bd]0020?ooo`080?Ogm`0M
7Ad02@T90=;BdP3Ogml04Q8B00T92@3Bd]820?ooo`0@0?Ogm`0M7Ad02@T90=;BdP3gmoL07AdM0000
002d];@0moOg01dM7@000000];Bd0=oOg`0B4Q802@T90=;BdP`0oooo00X0gmoO018B4P0000000000
0451@@3gmoL0oooo07QhN0000000H61P0P3oool0202RXZ800000061PH03oool0gmoO018B4P092@T0
d];B0P3oool00`2RXZ800000061PH0050?ooo`080=oOg`0B4Q802@T90=;BdP3oool0N7Qh0000001P
H6020?ooo`0:0::RXP000000H61P0?ooo`3kno/0E5AD0000002d];@0oooo0<K6aPH0000000H02@T9
0=;BdP3gmoL07AdM0000002d];@50?ooo`0:0=oOg`0B4Q80000000000011@D40moOg0=oOg`0B4Q80
2@T90=;BdPD0oooo01<0no_k05ADE0000000];Bd0?ooo`3gmoL07AdM0000002d];@0oooo07QhN000
0000Shn?0?ooo`2RXZ80000008n?S`3oool0a/K600H0000000<02@T90=;BdP3oool00`3oool0103g
moL07AdM0000002d];@40?ooo`080?Ogm`0M7Ad02@T90=;BdP3Ogml04Q8B00T92@3Bd]820?ooo`08
0?Ogm`0M7Ad02@T90=;BdP3Ogml04Q8B00T92@3Bd]860?ooo`0307QhN0000000H61P0080oooo01<0
XZ:R0000001PH600oooo0=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P092@T0d];B0?ooo`36a/H0
000000l?3`3_knl0a/K600H0000000H02@T90=;BdP3oool0N7Qh0000001PH6020?ooo`0;0::RXP00
0000H61P0?ooo`3kno/0E5AD0000002d];@0oooo0?Ogm`0M7Ad00`0000000`1PH600oooo0?ooo`0?
0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7@3oool0
0`3gmoL07AdM000000030000000B0;Bd]03oool0oooo0=oOg`0B4Q803`l?0>o_k`3oool0a/K60000
0011@D40moOg0=oOg`0B4Q800000061PH03oool0a/K60P000000:`2?Shl0oooo0::RXP0000003`l?
0>o_k`3kno/0E5AD00000011@D40moOg0?ooo`2RXZ80000000l?3`3_knl0no_k05ADE0000000@D51
0?Ogm`3oool0N7Qh0000000?3`l0kno_0?Ogm`0M7Ad000000;Bd]03gmoL07AdM0000002d];@0moOg
01dM7@000000Shn?0?ooo`3gmoL07AdM00T92@3Bd]80103oool0703gmoL07AdM00l?3`3_knl0oooo
0::RXP000000@D510?Ogm`3oool0N7Qh000000092@T0d];B0?_kn`1DE5@00000061PH03oool0moOg
01dM7@000000H61P0?ooo`3gmoL07AdM0000001PH6050?ooo`0P0=oOg`0B4Q802@T90=;BdP3oool0
N7Qh000000092@T0d];B0?_kn`1DE5@00000061PH03oool0no_k05ADE0000000];Bd0?ooo`3Ogml0
4Q8B00l?3`3_knl0oooo0=oOg`0B4Q803`l?0>o_k`3gmoL07AdM0000001PH6020?ooo`080?Ogm`0M
7Ad03`l?0>o_k`3oool0XZ:R00000011@D420?Ogm`0S01dM7@000000Shn?0?ooo`3gmoL07AdM00T9
2@3Bd]80no_k05ADE0000000];Bd0?ooo`3gmoL07AdM0000002d];@0moOg01dM7@092@T0d];B0?oo
o`3Ogml04Q8B0451@@3gmoL0gmoO018B4P0?3`l0kno_0?ooo`3Ogml04Q8B00l?3`3_knl0103oool0
103gmoL07AdM0000002d];@40?ooo`0K0?Ogm`0M7Ad02@T90=;BdP3kno/0E5AD00000011@D40moOg
0?ooo`1hN7P0000000l?3`3_knl0moOg01dM7@000000Shn?0?ooo`3gmoL07AdM00T92@3Bd]80oooo
07QhN0000000H61P0080oooo00X0XZ:R0000001PH600oooo0=oOg`0B4Q800000061PH03oool0a/K6
0P0000009P2?Shl0oooo0::RXP0000003`l?0>o_k`3Ogml04Q8B00l?3`3_knl0oooo0=oOg`0B4Q80
3`l?0>o_k`3oool0N7Qh000000092@T0d];B0?_kn`1DE5@00000061PH03oool0no_k05ADE0000000
];Bd0?ooo`3Ogml04Q8B00l?3`3_knl0oooo0::RXP000000H61P3`3oool00`1EEED0oooo0?ooo`04
0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAd0oooo00D0N7Qh0000000000003`l?0>o_
k`040?ooo`030::RXP000000H61P0080oooo00H0moOg01dM7@092@T0d];B0=oOg`0B4Q8:00000004
061PH03oool0no_k05ADE0H0000000D0Shn?0?ooo`3oool0gmoO018B4P050000000<0;Bd]03oool0
moOg01dM7@000000];Bd0?Ogm`0M7Ad000000;Bd]03oool0a/K61@0000000`11@D40moOg0?ooo`04
0?ooo`0307QhN0000000000000<0000000@0H61P0?ooo`3oool0N7Qh1P000000102?Shl0oooo0?oo
o`2RXZ8600000003061PH03oool0oooo00<0oooo00H0gmoO018B4P092@T0d];B0?ooo`1hN7P60000
000308n?S`3kno/0E5AD00<0000000<03`l?0>o_k`1hN7P01@000000102?Shl0oooo0?Ogm`0M7Ad3
000000030451@@3gmoL0N7Qh00D0000000@0H61P0?ooo`3oool0a/K61@0000000`11@D40lo?c05AD
E0030000000@00l?3`3WinL07AdM0000002d];@0a/K600000011@D40moOg0?ooo`3kno/0E5AD0000
002d];@0oooo07QhN0D0000000<0Shn?0?ooo`3oool00`3oool0103gmoL07AdM0000002d];@40?oo
o`070=oOg`0B4Q802@T90=;BdP3oool0gmoO018B4P05000000040;Bd]03oool0oooo0<K6aPD00000
00H0@D510?Ogm`3oool0N7Qh0000001PH6020?ooo`060::RXP000000H61P0?ooo`3Ogml04Q8B2P00
0000101PH600oooo0?ooo`1hN7P50000000408n?S`3oool0oooo07QhN0H0000000<0Shn?0?_kn`1D
E5@00`0000000`0?3`l0jn_[05ADE0050000000308n?S`3oool0oooo00d0oooo00<0EEEE0?ooo`3o
ool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`040?_kn`1DE5@0000008n?
S`80oooo00L0N7Qh0000001PH600moOg01dM7@000000];Bd00<0oooo01H0N7Qh0000002?Shl0gmoO
018B4P092@T0/[:b018B4P0000003`l?0>o_k`3Ogml04Q8B0000000?3`l0kno_0?ooo`3kno/0E5AD
0000002`/;00E5AD0P0000000`2?Shl0oooo0?ooo`020?ooo`060?Ogm`0M7Ad00000000000092@T0
d];B0P3oool0203gmoL07AdM0000002d];@0moOg01dM7@000000];Bd0P3oool01P36a/H000000000
00000000@D510?Ogm`L0oooo00H0N7Qh00000000000000000451@@3gmoL20?ooo`0407QhN0000000
H61P07QhN080000000<0H61P0?ooo`3oool00P3oool01`2RXZ80000000000011@D40VYZJ0000001P
H6001@3oool02@3Ogml04Q8B00T92@3Bd]80oooo07QhN0000000H61P07QhN00200000004061PH03o
ool0no_k05ADE0<0000000@03`l?0>o_k`3oool0XZ:R0`000000302?Shl0oooo0?ooo`3gmoL07AdM
00T92@2GUiL0000000T92@3Bd]80oooo07QhN0<0000000<0@D510?Ogm`3oool00P3oool02036a/H0
0000000000000000@D510?Ogm`3kno/0E5AD0`000000200?3`l0inOW01dM7@000000];Bd07QhN000
0000Shn?0`3oool00`1hN7P00000061PH0020?ooo`050::RXP00000000000000002?Shl01P3oool0
103gmoL07AdM0000002d];@40?ooo`040::RXP0000003`l?0>o_k`80oooo00H0moOg01dM7@000000
000000T92@3Bd]840?ooo`060<K6aP000000000000000011@D40moOg0P3oool00`1hN7P00000061P
H0020?ooo`0A0::RXP000000H61P0?ooo`3Ogml04Q8B00T92@2b/[804Q8B0000000?3`l0kno_0=oO
g`0B4Q80000000l?3`3_knl00`3oool01@2RXZ800000000000000000Shn?00<0oooo00@0N7Qh0000
001PH600N7Qh0P000000101PH600oooo0?_kn`1DE5@30000000400l?3`3_knl0moOg01dM7@<00000
00<0H61P0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJA`0oooo00T0no_k05ADE0000000Shn?0?ooo`3gmoL07AdM0000002?Shl0603oool0
103kno/0E5AD0000002d];@@0?ooo`040?Ogm`0M7Ad000000;Bd]2X0oooo00<0XZ:R0000001PH600
1@3oool0103Ogml04Q8B00T92@3Bd]8:0?ooo`040?_kn`1DE5@000000;Bd]280oooo00@0no_k05AD
E0000000];Bd6`3oool0103gmoL07AdM0000002d];@30?ooo`040<K6aP000000000008n?Scl0oooo
00@0no_k05ADE0000000];Bd603oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJAd0oooo00<0XZ:R0000000000000`0000000`0?3`l0kno_0?ooo`0G0?ooo`04
0?_kn`1DE5@000000;Bd]100oooo00P0moOg01dM7@000000];Bd0?Ogm`0M7Ad000000;Bd]2H0oooo
00<0XZ:R0000001PH6001@3oool0103Ogml04Q8B00T92@3Bd]8:0?ooo`040?_kn`1DE5@000000;Bd
]280oooo00T0no_k05ADE0000000];Bd0?ooo`3gmoL07AdM0000002d];@05P3oool00`3gmoL07AdM
00000006000000030451@@3gmoL0oooo03h0oooo00@0no_k05ADE0000000];Bd603oool00`1EEED0
oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0
i^KV06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUY7P3oool01P36a/H000000000000000003`l?0>o_kaT0oooo00@0no_k05ADE0000000];Bd403o
ool0203gmoL07AdM0000002d];@0moOg01dM7@000000];Bd9P3oool00`2RXZ800000061PH0050?oo
o`040=oOg`0B4Q802@T90=;BdP`0oooo00<0N7Qh0;Bd]03oool08`3oool01`1hN7P0];Bd0?ooo`3g
moL07AdM0000002d];@05P3oool00`3gmoL07AdM000000050000000308n?S`3oool0oooo0440oooo
00<0N7Qh0;Bd]03oool05`3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00<0k>c/09JFUP3Ng]h0303Kfm/05P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>
01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo@@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom10?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood40oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo0080c/k>
00<0?Shn0<k>cP3>c/h00`3>c/h06P0n?Sh0c/k>0<k>cP3>c/h0oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@@3oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0clo?0`0n?Sh3
0<o?c`<0?Shn0P3?cll05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`@0EEEE1P3oool00`1EEED0oooo0?ooo`02
0?ooo`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh30=CDe00F0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oood40oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02F
UYH0oooo00<0fm_K1@0n?Sh40=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooool000007@0000050?ooo`D0EEEE0P3o
ool505EEE@@0oooo00<0k>c/09JFUP3oool0103Ti>@303hn?PD0i>CT01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3o
ool00`3/k>`0UYJF0?ooo`050>g]k@0303hn?P3]kNd0kNg]00@0kNg]01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3o
ool00`3/k>`0UYJF0?ooo`0<0?Cdm00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00<0k>c/09JFUP3oool0303l
o?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooom=0?ooo`030>c/k02FUYH0oooo00d0oooo01D0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooC@3oool0
0`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooodd0oooo00@0k>c/0=[JfP2BTY80][Jf2P2e
]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
0`000000moOg09ZJVP3o07YjNV00NWYj01<0VYZJ0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo00000`000000moOg0;no_`3o0;no_f40_kno0180moOg0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000<000000?Ogm`3Shn<0o`3Shn=Q0>?Sh`0B0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000030000003alO40oooo0?l0oooo6P3oool00`3Shn<0YJFU07IfMP0307IfMP0407MgM`1f
MWH0MgMg07MgM`T0MWIf00H0YJFU0>?Sh`3oool0oooo0>?Sh`2UYJD407IfMP0407MgM`1fMWH0MgMg
07MgM`T0MWIf00H0YJFU0>?Sh`3oool0oooo0>_[j`2UYJD405IFEP0405MGE`1FEUH0EeMG05MGE`T0
EUIF00<0YJFU0>_[j`3oool00`3oool04P3alO40000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000hN7Q0?clo03kno/20?clo0030?_kn`3lo?`0o?cl00@0o?cl00<0no_k0?clo03lo?`00P3l
o?`00`3kno/0o?cl0?_kn`050?clo0030?_kn`3lo?`0no_k0640no_kWP3lo?`00`2UYJD0MWIf08f=
S@0@08f=S@0607IfMP2UYJD0o?cl0?clo02UYJD0MWIf4@2=SHd01P1fMWH0YJFU0?clo03kno/0YJFU
05IFEQ40Fe]K00<0EUIF0:FUY@3lo?`00`3lo?`04P3QhN40000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo00000`000000h>3P0?Ogm`3o0?OgmaX0moOg00<0MWIf08f=S@2RXZ80402RXZ801P2=SHd0MWIf
0?Ogm`3gmoL0MWIf08f=SA40XZ:R00P0SHf=07IfMP3gmoL0moOg05IFEP1KFe/0Kfm_075aL@d0LW9b
00@0LG5a06m_K`1KFe/0EUIF103gmoL04P3Ph>00000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
0`000000g]kN0?7al@020?7al@80l_;b00<0lO7a0?;blP3bl_80103alO400`3bl_80lO7a0?7al@03
0?7al@80l_;b00<0lO7a0?;blP3alO400P3bl_;o0?7al@80lO7a00<0MWIf0;Rh^02[Zj/0402[Zj/0
1P2h^;P0MWIf0?7al@3alO40MWIf0;Rh^140Zj^[00D0^;Rh07IfMP3alO40l_;b05IFEP0206m_K`03
079bLP1gMgL0MgMg00/0MgMg00@0LW9b06m_K`1_Kfl0EUIF0`3alO404`3bl_80g]kN0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000050000003Lg=`0jn_[0>_[j`3/k>`00P3[jn/00`2WYjL0XZ:R
0::RXP020:6QX@030:2PX02OWil0WIfM0080WinO0P2PX:020:6QX@030:NWY`3[jn/0jn_[00D0jn_[
00D0e=CD0<G5a@35aLD0aLG50=OGe`020>_[j`030>[ZjP38b<P0d=3@0080jn_[00D0f][J0<K6aP3S
hn<0jn_[0>3Ph0020<G5a@070<c<c03YjNT0c/k>0<O7a`3YjNT0jn_[0>3Ph0020<G5a@070<c<c03Z
j^X0e=CD0<G5a@3MgMd0jn_[0>3Ph0020<G5a@060<K6aP3Shn<0jn_[0>_[j`3Zj^X0b<S80`35aLD0
0`36a/H0hn?S0>_[j`070>_[j`030>GUi@37alL0emOG02H0jn_[HP3/k>`l0>_[j`0407IfMP2k^k/0
/K6a0;6a/@P0oooo1`2a/K401P2k^k/0MWIf0>_[j`3[jn/0MWIf0;^k^`80/K6a303oool30;6a/@09
0;^k^`1fMWH0jn_[0>_[j`1FEUH0MgMg079bLP1dM7@0MgMg00/0N7Qh00D0MgMg07AdM01bLW80MgMg
05IFEP040>_[j`0B0=cLg0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003Jf]X0iNGU
00<0iNGU00<0RXZ:09^KV`2/[:`00`3YjNT02P3Zj^X0a<C408F5Q@3/k>`0kNg]0>k^kP3WinL0l?3`
0?;blP2SXj<60>GUi@0A06QXJ00@4100410@010@4020P800a/K60<o?c`3Ng]h09RHV055AD@3?cll0
iNGU09BDU00E5AD0Y:BT0=kNgP2b/[800P0@41001`11@D40fm_K051@D00I6AT0^[Zj0>?Sh`2b/[80
0P0@41001`11@D40gmoO071`L00@4100TY:B0>CTi02_[jl00P0@41001P0E5AD0^[Zj0<c<c03Ti>@0
gMgM02@T90<0410@00<05ADE0:BTY03Ng]h01`3UiND01039bLT071`L079bLP3De=C40>GUi@0507If
MP30`<00]KFe0;Fe]@3oool01P1bLW800`0W9bL0]KFe0;Fe]@050;Fe]@060<30`01fMWH0iNGU0>GU
i@1fMWH0`<300P2e]KD00`3oool09bLW02LW9`0902LW9`<0]KFe00D0`<3007IfMP3UiND0iNGU05IF
EP0207MgM`0407UiN@1jNWX0oooo0?ooo`L0NWYj0P3oool01@1jNWX0NGUi07MgM`1gMgL0EUIF00@0
iNGU0180f][J0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=WIf@3Ogml00`3Ogml0
1@3Ng]h0X:2P0>[ZjP3YjNT0iNGU0080j^[Z00X0d=3@07AdM03]kNd0k^k^0:f][@3@d=00l_;b0??c
l`2SXj<0gmoO0P3Ng]h0=03Ogml0g]kN06a/K00E5AD0Zj^[0=kNgP33`l<071`L05ADE036a/H0e]KF
02DU9@1ADE40cLg=0=kNgP2?Shl05ADE0:BTY03EeMD0CTi>03Ph>034a<@0gMgM0=_Kf`1>CTh06ATI
0;Zj^P3IfMT0CTi>03Ph>034a<@0gMgM0=OGe`0U9BD0>3Ph0<C4a032`/806a/K055AD@3IfMT0`l?3
04a<C03Hf=P0emOG02DU9@0I6AT0aLG50=kNgP2?Shl05ADE0:BTY03IfMT70=kNgP040<?3``0L71`0
LW9b0=7AdL@0g]kN00D0MWIf0<?3``2k^k/0^k^k0?ooo`06079bLP0302LW9`2k^k/0^k^k00D0^k^k
00H0`l?307IfMP3Ng]h0g]kN07IfMP33`l<20;^k^`030?ooo`0W9bL0^k^k00L0^k^k00<0oooo02LW
9`2k^k/00P2k^k/02@33`l<0MWIf0=oOg`3Ng]h0EUIF07MgM`1hN7P0Ng]k030`<0030?ooo`D0OGem
0`3oool01@0`<300Ng]k07QhN01gMgL0EUIF0080g]kN01@0gmoO0=kNgP3IfMT0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000emOG0=SHf03IfMT20=SHf00@092@T02?Shl0SXj>0>WYj@21
PH40DE5A08N7Q`3Jf]X0YZJV0:VYZ@2/[:`0GemO0?;blP3clo<0m?Cd0:BTY0<0f=SH00D0fMWI0=;B
dP0T92@0DE5A0<_;b`020=SHf00P06YZJP0I6AT0^[Zj0=3@d00T92@0DE5A0<_;b`3Hf=P0Rh^;01DE
5@2TY:@0cLg=02@T900h>3P0`l?30=OGe`3EeMD0C4a<01TI6@2j^[X0d=3@02@T900h>3P0`l?30=OG
e`3AdM4092@T03Ph>033`l<0RXZ:01DE5@2TY:@40<G5a@0:09jNWP0@4100LG5a0=7Ad@3Hf=P0Rh^;
01DE5@2TY:@0e=CD0=;BdP80aLG500P0a/K60=OGe`3Hf=P0f=SH0;fm_@0K6a/0LW9b0<k>cRH0f=SH
HP3IfMTl0=SHf00507IfMP37alL0_kno0;no_`3oool01`0W9bL70;no_`060<O7a`1fMWH0f=SH0=SH
f01fMWH0alO70P2o_kl00`3oool09bLW0;no_`070;no_`030?ooo`0W9bL0_kno0080_kno00D0alO7
07IfMP3Hf=P0f=SH05IFEP0207]kN`0307inOP1oOgl0:2PX00<0oooo0`1oOgl30?ooo`0302PX:01o
Ogl0OWin0080Ng]k00<0EUIF0=WIf@3Hf=P00P3Hf=P04P3GemL0000009ZJVP2PX:00Z:RX0;:b/P2l
_;`0a/K60=3@d03IfMT0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo00000`000000eMGE0=?Cd`030=?Cd`0308F5Q@2SXj<0i^KV0080j^[Z00/0=CDe07moO`3?
cll0_[jn08^;R`1NGUh0F5QH06QXJ02SXj<0bLW90:ZZZP040=?Cd`040<c<c00T92@0DE5A0<[:bP80
dm?C01h0IfMW01TI6@2i^KT0bl_;02@T901ADE40b/[:0=?Cd`28R8P05ADE0:BTY039bLT092@T03Ph
>031`L40dm?C0<o?c`1:BTX06ATI0;Vi^@3;bl/092@T03Ph>031`L40dm?C0<c<c00T92@0>3Ph0<71
`@1WIfL5010@400:055AD@2MWId0410@08j>SP3<c<`0dm?C08R8R00E5AD0Y:BT0:6QX@80410@00<0
6QXJ0<K6aP3Cdm<00P3Cdm<0102h^;P06a/K079bLP3<c<c40=?Cd`0307IfMP3;bl/0a/K60100a/K6
00H0bl_;07IfMP3Cdm<0dm?C07IfMP3;bl/20<K6aP030?ooo`0W9bL0a/K600L0a/K600<0oooo02LW
9`36a/H00P36a/H01`3;bl/0MWIf0=?Cd`3Cdm<0EUIF07emO@1oOgl00`23Ph<01@0X:2P0oooo0?oo
o`3oool0Ph>300<0oooo00L0:2PX08>3P`23Ph<0Ph>307moO`1mOGd0EUIF00@0dm?C0140eMGE0000
002PX:00YJFU0:f][@2f][H0_kno0<S8b03Bd]80f][J0>;RhP3YjNT0kno_0?Cdm03fm_H0nOWi0?_k
n`030?gmo@40o_kn0@3oool10?ooo`0000D000000=?Cd`3>c/h0c/k>0<o?c`020<k>cP0?08N7Q`2C
Ti<0dm?C0=_Kf`2][Jd0`L710;2`/02YZJT0ZZZZ08f=S@2ATI40/;2`0<_;b`3_knl0YJFU00<0c/k>
00D0clo?0<O7a`0S8b<0DE5A0<S8b0020<k>cP0]06EUI@0I6AT0^KVi0<O7a`0S8b<0DE5A0<O7a`3>
c/h0QHF501DE5@2TY:@0alO702<S8`0h>3P0`L710<k>cP3;bl/0BDU901TI6@2i^KT0alO702<S8`0h
>3P0`L710<k>cP37alL08b<S03Ph>031`L40QHF501DE5@2TY:@0b/[:0<O7a`0S8b<0DE5A0;6a/@0J
6QX0LW9b0<G5a@3>c/h0QHF501DE5@2TY:@0cLg=00L0c/k>00@0];Bd01XJ6P1bLW80b/[:1P3>c/h0
1@3=cLd0alO70<g=c@37alL0b/[:01/0c/k>HP3?clll0<k>cP0307IfMP3>c/h0bl_;0100bl_;00H0
c/k>07IfMP3>c/h0c/k>07IfMP3>c/h20<_;b`030?ooo`0W9bL0bl_;00L0bl_;00<0oooo02LW9`3;
bl/00P3;bl/02@3>c/h0MWIf0<o?c`3>c/h0EUIF07emO@22PX80QHF508B4Q00208F5Q@0302PX:03o
ool0oooo00<0oooo00P0:2PX08F5Q@25QHD0Q8B408F5Q@22PX80OGem05IFEP@0c/k>0140dm?C0000
002XZ:P0[Jf]0;>c/`2k^k/0`l?30<c<c03EeMD0gMgM0>CTi03[jn/0l?3`0?Cdm03gmoL0nOWi0?cl
o0030?gmo@40o_kn0@3oool10?ooo`0000<000000=3@d039bLT00`39bLT04@1cLg<0U9BD07inOP1i
NGT0X:2P0=7Ad@2][Jd0YJFU0:JVYP25QHD0GUiN0:f][@3UiND0m_Kf0?Shn02TY:@0bLW900<0b<S8
00@0`/;202<S8`1ADE40alO70P38b<P0;01RHV806ATI0;Vi^@32`/808R8R03Ph>031`L40`/;204I6
AP0I6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35aLD0a<C404I6AP0I6AT0^KVi0<71`@0S8b<0>3Ph0;no
_`35aLD0`<30028R8P0h>3P0`<300:n_[`0J6QX0Lg=c0<G5a@2JVYX0410@08j>SP35aLD0ATI601TI
6@2i^KT0aLG507moO`0E5AD0Y:BT1`38b<P01@37alL0[Zj^01XJ6P1aLG40alO700D0b<S800H0/k>c
04E5A@2QXJ40C4a<06i^KP37alLK0<S8b680bLW9?038b<P00`1fMWH0dm?C0<g=c@0@0<g=c@060=?C
d`1fMWH0b<S80<S8b01fMWH0dm?C0P3=cLd00`3oool09bLW0<g=c@070<g=c@030?ooo`0W9bL0cLg=
0080cLg=00L0dm?C07IfMP38b<P0b<S805IFEP22PX80Ph>300D0QhN700D0:2PX0?ooo`3oool0oooo
02PX:00508N7Q`0308>3P`22PX80EUIF00<0bLW90180b<S80=3@d0000000/[:b0;Jf]P2k^k/0`/;2
0<W9b@3AdM40fMWI0>3Ph03WinL0kNg]0?7al@3emOD0n?Sh0?[jnP3lo?`30?gmo@40o_kn0@3oool1
0?ooo`0000<000000<o?c`35aLD01035aLD03`2GUiL0]kNg0>c/k02i^KT0Rh^;0;Ng]`2ZZZX0Y:BT
07inOP1aLG40a<C40=WIf@3fm_H0nOWi0:FUY@040<G5a@040;no_`0R8R80DE5A0<K6aP80aLG500H0
HF5Q01TI6@2i^KT0_kno028R8P1MGEd2010@400401DE5@2TY:@0`/;204E5A@<0410@00H0DE5A065Q
H@0@41006ATI0;Jf]P15ADD3010@4008055AD@2o_kl08R8R03Ph>02o_kl0aLG50:b/[00J6QX2010@
4005079bLP35aLD0aLG50<;2`P15ADD00`0@41000`0E5AD0Y:BT0<K6aP050<G5a@060:b/[01LG5`0
[Jf]01XJ6P1aLG40a/K61@35aLD01@2o_kl08R8R030`<00J6QX0SXj>0;X0aLG500<0MWIf0=GEe@3A
dM40403AdM401P3EeMD0MWIf0<G5a@35aLD0MWIf0=GEe@80dM7A00<0oooo02LW9`3AdM401`3AdM40
0`3oool09bLW0=7Ad@020=7Ad@070=GEe@1fMWH0aLG50<G5a@1FEUH0PX:208N7Q`0508V9R@D0oooo
1@29RHT00`27QhL0PX:205IFEP040<G5a@0A0<o?c`000000_;bl0;no_`33`l<0bLW90=3@d03GemL0
gMgM0>CTi03Zj^X0kno_0??cl`3fm_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`000000
cLg=0<71`@030<71`@0@07EeM@2IVIT0YjNW0>g]k@39bLT0Lg=c0;2`/02=SHd0Lg=c04a<C01RHV80
_[jn0;Fe]@2b/[80[Jf]0:VYZ@@0`L7100@0^k^k028R8P1ADE40a<C40P31`L40101OGel06ATI0;Vi
^@32`/870<71`@050;^k^`0R8R80>3Ph0;jn_P32`/800P31`L403033`l<0aLG50;bl_00R8R80>3Ph
0;jn_P32`/80^k^k028R8P0h>3P0_[jn0<;2`P/0`L7100@0O7al01DE5@2TY:@0`l?31@31`L401P2Y
ZJT06QXJ010@400@4100LG5a0<C4a0D0`L7100L0GemO010@400@4100410@03Ph>02n_[h0`/;20;P0
`L7100<0MWIf0=SHf03EeMD0403EeMD01P3Hf=P0MWIf0<71`@31`L40MWIf0=SHf080eMGE00<0oooo
02LW9`3oool0203oool01P0W9bL0eMGE0=GEe@3EeMD0f=SH07IfMP80`L7100<0EUIF08B4Q028R8P0
102:RXX30?ooo`0402PX:03oool0oooo0?ooo`@0RXZ:00<0R8R808B4Q01FEUH01031`L40403=cLd0
00000<K6aP38b<P0c<c<0=7Ad@3GemL0g=cL0>;RhP3Xj>P0kNg]0?7al@3emOD0moOg0?Win@3lo?`3
0?gmo@80o_kn0P3oool000030000003<c<`0^[Zj0080_Kfm0140_[jn0;fm_@2NWYh0hn?S0;Jf]P2F
UYH0PX:209>CT`24Q8@0PX:204i>CP0/;2`0W9bL0:^[Z`3fm_H0n?Sh0:NWY`040;fm_@040;Ng]`0Q
8B40DE5A0<C4a080_Kfm00@0GEeM01TI6@2i^KT0_kno202m_Kd00`1MGEd0=cLg0;Rh^0020;fm_@0;
05eMG@0I6AT0^KVi0;no_`1MGEd0=cLg0;Rh^02m_Kd0]kNg024Q8@0h>3P00P2n_[h;0;fm_@0407Yj
NP0E5AD0Y:BT0;no_`H0_Kfm00D0YJFU01XJ6P0@4100L71`0<;2`P050;fm_@0607YjNP0d=3@0?3`l
05eMG@0h>3P0^;Rh6`2m_KeR0;jn_S`0_Kfm00<0MWIf0=[JfP3EeMD0403EeMD01P3Jf]X0MWIf0;fm
_@2m_Kd0MWIf0=[JfP80eMGE00<0oooo079bLP1bLW80201bLW803P0W9bL0eMGE0=GEe@3EeMD0f][J
07IfMP2m_Kd0_[jn05IFEP27QhL0RXZ:08f=S@2>SXh0SHf=0`3oool00`0X:2P0SHf=02PX:0030?oo
o`<0SHf=00@0RXZ:08N7Q`1FEUH0_[jn0P2m_Kd04@2j^[X0c<c<0000003@d=00d];B0=GEe@3IfMT0
gMgM0>;RhP3WinL0k>c/0?3`l03dm?@0m_Kf0?Shn03jn_X0o?cl00<0oOgm00<0o_kn0?ooo`3oool0
0@3oool000030000003:b/X0^;Rh00<0^[Zj0100O7al08f=S@1hN7P0VIVI0?3`l032`/80YjNW0<_;
b`2l_;`0Xj>S06e]K@1mOGd0[:b/0?_kn`3lo?`0YZJV0`2j^[X0102k^k/0];Bd024Q8@1=CDd30;Zj
^P0305]KF`0I6AT0[jn_00h0^[Zj00<0Fe]K01TI6@2_[jl01@2j^[X0102d];@08B4Q03Lg=`2d];@<
0;Zj^P0307MgM`0D51@0Vi^K00T0^[Zj00<0Fe]K06][J`2j^[X01P2j^[X00`2d];@08B4Q06][J`2k
0;Zj^P0307IfMP3Hf=P0f][J0100f][J00H0f=SH07IfMP2j^[X0^[Zj07IfMP3Hf=P20=[JfP030?oo
o`1bLW80LW9b00P0LW9b00d09bLW0=[JfP3Jf]X0f][J0=SHf01fMWH0^[Zj0;^k^`1FEUH0Rh^;08n?
S`2>SXh0Shn?00<0oooo00D0:2PX08n?S`2?Shl0Shn?02PX:0030?ooo`0508n?S`2>SXh0Shn?08^;
R`1FEUH00`2j^[X0402h^;P0b/[:0000003IfMT0f][J0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??c
l`3fm_H0n?Sh0?[jnP3lo?`30?gmo@80o_kn0`3oool0000300000039bLT0^;Rh00<0^;Rh0100QXJ6
09jNWP3De=@0l?3`0?;blP3>c/h0`L710=gMg@3Bd]80XZ:R0<c<c039bLT0RHV90?clo03moOd0YjNW
o`2h^;P70;Rh^00307IfMP3GemL0gMgM0100gMgM00H0emOG07IfMP2h^;P0^;Rh07IfMP3GemL30=gM
g@/09bLW0`3MgMd02@3GemL0MWIf0;Rh^02h^;P0EUIF092@T02ATI40Ti>C09BDU0020?ooo`0302PX
:02HV9P0V9RH00<0V9RH00P0:2PX0?ooo`3oool0U9BD09>CT`2ATI40T92@05IFEP@0^;Rh00l0bLW9
0000003QhN40h^;R0>CTi03WinL0j^[Z0>g]k@3`l?00lo?c0?Gem@3gmoL0nOWi0?_kn`3lo?`00`3m
oOd00`3no_h0oooo0?ooo`020?ooo`0000<000000<W9b@2h^;P0102h^;P03`26QXH0U9BD0?;blP3c
lo<0X:2P0>SXj03fm_H0fm_K09VIV@3jn_X0no_k0=cLg02f][H0o_kn0:NWY`3o0;Rh^0L0^;Rh00<0
MWIf0=KFeP3QhN40403QhN401P3Fe]H0MWIf0;Rh^02h^;P0MWIf0=KFeQ40hN7Q00T0e]KF07IfMP2h
^;P0^;Rh05IFEP2EUID0UiNG09bLW02RXZ800P0X:2P01@2[Zj/0[Jf]0:^[Z`2[Zj/0[Jf]0080Zj^[
0P0X:2P01@2RXZ80W9bL09NGU`2EUID0EUIF00@0^;Rh00h0bLW90000003Xj>P0jNWY0>_[j`3]kNd0
kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0<0oOgm0P3no_h40?ooo`0000<000000<W9b@2e
]KD00`2e]KD03@1gMgL0X:2P0>3Ph03clo<0iNGU0>c/k03fm_H0n?Sh0?Win@2@T900no_k0?clo03m
oOd00P3no_h00`2WYjL0]KFe0;Fe]@3o0;Fe]@D0]KFe00<0MWIf0>g]k@3YjNT0403YjNT01P3]kNd0
MWIf0;Fe]@2e]KD0MWIf0>g]kA40jNWY00X0kNg]07IfMP2e]KD0]KFe05IFEP2KVi/0WYjN0:FUY@2b
/[80^[Zj0P2o_kl01030`<00_kno0;no_`30`<020;no_`070<30`02j^[X0/[:b0:FUY@2NWYh0Vi^K
05IFEP040;Fe]@0=0<W9b@000000k^k^0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0030?gmo@80o_kn1@3oool0000300000039bLT0^[Zj00@0/K6a00<0SHf=0:ZZZP2ZZZX01@2Z
ZZX00`2WYjL0ZZZZ0:ZZZP040:ZZZ_l0/K6a1`2a/K400`1fMWH0e=CD0?;blP0@0?;blP060=CDe01f
MWH0/K6a0;6a/@1fMWH0e=CD4@3bl_801P3De=@0MWIf0;6a/@2a/K40EUIF0:>SX`80Z:RX3@33`l<2
0:RXZ0030:>SX`1FEUH0/K6a0080/K6a00@0^[Zj0<W9b@000000lo?c0P3dm?@01`3emOD0m_Kf0?Og
m`3hn?P0n_[j0?_kn`3lo?`0103moOd00`3no_h0oooo0?ooo`040?ooo`0000<000000<?3``39bLT0
o`2^[ZhJ0:j^[P0408f=S@1fMWH0c/k>0?_kn`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^[Zh0102=
SHd0MWIf0<k>cP3kno/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00H0VIVI05IFEP2e]KD0bl_;
0=oOg`3Xj>P90>OWi`060>SXj03Ogml0bl_;0;Fe]@1FEUH0VIVI0`2^[Zh00`39bLT0`l?300000002
0?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool0000400000033`l<0
e=CD0;Ng]ol0YjNW6P2WYjL00`2=SHd0MWIf07IfMP0?07IfMP0308f=S@2WYjL0YjNW0080YjNW00<0
SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=S@1FEUH0EUIF00l0EUIF00P0
SHf=0:NWY`2WYjL0YjNW0;Ng]`3De=@0`l?3000000<0nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd2
0?knoPL0oooo00001P000000ZZZZ0>OWi`3De=@0_[jn0;>c/ol0ZZZZFP2ZZZX01P2c/k<0_[jn0=CD
e03WinL0ZZZZ00000080no_k0`3lo?`50?gmo@<0o_kn203oool000070000002:RXX0fMWI0?clo03T
i>@0d];B0<c<c03o0;jn_UP0_[jn00P0c<c<0=;BdP3Ti>@0o?cl0=WIf@2:RXX000000?clo0P0oOgm
0P3no_h:0?ooo`0000P0000006UYJ@2GUiL0fMWI0?clo03bl_80k>c/0>?Shol0fMWIEP3IfMT0203S
hn<0k>c/0?;blP3lo?`0fMWI09NGU`1YJFT000001`3moOd30?knoP/0oooo00001@000000oooo06UY
J@2GUiL0dm?C0080o?clo`3ooomH0?ooo`80o?cl00D0dm?C09NGU`1YJFT0oooo000000040?gmo@@0
o_kn3@3oool000030000003oool0oooo0080JFUY00<0RXZ:09NGU`2ZZZX0o`2ZZZYG0:ZZZP0409NG
U`2:RXX0JFUY06UYJ@80oooo00<000000?knoP3no_h0103no_h?0?ooo`00o`00001V000001D0oooo
0000\
\>"],
  ImageRangeCache->{{{126.938, 390.5}, {492.938, 348.375}} -> {-181.964, \
499.843, 1.00234, 1.00234}},
  CellTags->"IPM3.1.1.6"]
}, Open  ]],

Cell["\<\
In Chapter 10 we will give a detailed discussion of how to modify and \
manipulate the user interface through the use of the symbolic programming \
techniques that are discussed throughout this book.\
\>", "Text",
  CellTags->{"b:0.5", "IPM3.1.1.6"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "Section",
  CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
  CellTags->{
  "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
    "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],

Cell[TextData[{
  "Before you can do any serious work, you will need to know how to get a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session started, how to stop it, and how to get out of trouble when you \
get into it. These procedures depend somewhat on the system you are using. \
You should read the system-specific information that came with your copy of \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "; and you may need to consult a local ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " guru if our advice here is not applicable to your system."
}], "TextFirst",
  CellTags->"IPM3.1.2.0"],

Cell[CellGroupData[{

Cell[TextData[{
  "Getting into and out of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "Subsection",
  CellTags->{"b:1.0", "IPM3.1.2.1", "c:10"}],

Cell[TextData[{
  "The most commonly used interface is often referred to as a notebook \
interface in which the user creates and works in interactive documents. \
Personal computers running Windows, Macintosh operating systems, Linux, and \
most flavors of Unix all support this graphical user interface, which \
normally starts up automatically when you begin your ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session."
}], "TextFirst",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "There are some situations where you may want to start up ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " from a command prompt and issue commands directly through that interface, \
bypassing the notebook interface entirely. For example, you may have a very \
long computation that you need to run in batch mode. Typically, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is started up on these systems by typing ",
  StyleBox["math", "MR"],
  " at a command prompt. We will not discuss using ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " through a command prompt any further. If you are interested in this mode \
you should consult the documentation that came with your copy of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Starting ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and first computations"
}], "Subsubsection",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "To start ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " you will have to find and then double-click on the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " icon on your computer, which will look something like this:"
}], "TextFirst",
  CellTags->{"i:30", "b:1.0", "IPM3.1.2.1"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40000^00009B000`400?l0
0000o`00003o3P3oool01@3ooo/0o_om0?ooo@3oo_l0o_oo0080oooo0`3o
o_l00`3nool0ooom0?ooo`020?ono`070?kon`3oool0ooko0?omoP3oool0
o_kn0?ono`0:0?ooo`002@3oool20?knoP80oooo00<0oogn0?ono`3mooh0
0P3oool04@3oo_l0oook0?ono`3oood0oooo0?kno03oo_l0oooo0?_ooP3n
ool0ooom0?ono`3oool0o?oo0?koo@3oool0ooom00/0oooo000;0?ooo`03
0?knoP3oool0oOol0080ooko0P3oool01`2KJRD0i;fD0?gmo@3lool0oOon
0?ooo`3ooOh00P3oood20?ooo`040?koo@3nool0o_oo0?knoP80oooo00<0
ooko0?ooo`3oool0203oool000d0oooo00P0ooko0?koo@3oood0oO_l0?_o
oP20Eb40jim10>o_k`80oooo00X0o_ko0?koo@3oo_l0ookm0?koo`3lo?`0
m_Kh0?gmo`3ooOh0ooko3@3oool000d0oooo01L0ooom0?goo03oool0lo3O
0=W6Y@1IAbl0mJl00=?AdP3gm_@0ooom0?ooo`3nool0ooko0?ooo@3mnol0
dKFY0>?J_@3ooOd0o_oo0?_on@3no_l0o_ok0?gooP0:0?ooo`001`3oool0
1P3no_h0oooo0?knoP3oool0l?3`0?KfmP80oooo00l0o_kl0?Kel@3;[1H0
TTd<0>ZI0@3T]4H0fMSM0?koo`3no_h0o_oo0?ooo`3jn_`0aJVM0=b65`3_
kNh00P3oood30?ooo`030?ono`3oool0oooo00P0oooo00060?ooo`0M0?oo
o@3oool0o_oo0?ooo`3oo_l0jlo10=7@cP3YjNT0no_k0?ooo`3kn_D0d<<2
0;1m003FSP00nZ400<_0ZP3Be=<0l>k_0??cm@3Shn<0bkRJ0>AJ0@3P^VL0
oooo0?koo`3no_h0ooko0?ooo`3oo_l02`3oool000L0oooo01`0ooom0?ko
o@3oool0ooom0;aU8P3a/9X0`l?10=3?e03MgMd0h^KW0<g30P3/b`@0j9h1
0?2T003[XQl0c[VN0<C4a03:^Z<0bhdh0>Md0@3dM@00e/no0?ooo`3nool0
ooko0?kno03oool0o_kn2`3oool000H0oooo00<0o_oo0?ono`3oool00P3o
ool0503Hbk/0c5D10<Uh@`2kTW`0`kJ`0;:PRP3R`@00dKT10=JU003aY000
lId00?6C003^SPH0jX@00>b8003BLP40f7T?0<o?c`3nool0oooo0P3oood0
0`3gm_@0jnOX0>SZj@0:0?ooo`001P3oool07P3oo_l0oogn0?koo`3oool0
o_om0?koo`33KR/0aED00;aC002eB`00^7L10>OV002^X040a:l00>jW0@3d
XP40kiL10>n;003YR@00hWP20=1Z003ZOR<0a<C40=;BdP3Qgn00h=kO0=gK
gP3:b/X0e<[80??cl@X0oooo00060?ooo`0N0?gmn`3lo_d0oooo0?[im`3i
n?H0o?cl0<>4F`3ME@00bUP00:e40P3AW`80gML00;:N002n[`00b:D10?BL
003dUP00jXP10>b40@3>J@40d5l00<9X4P2j]jP0a<C60<C6`@34`lP0`<G1
0<:2EP3YgG/0oook2P3oool000H0oooo01h0lo;`0>3Ng`3IgN00fMWK0<O9
aP3;bLX0`XYY0=IA0P3>B000dfP00>7<0@3LdP00dl800;V[0P2XWP00]9@1
0<RA0@3=P`00`g430:UT001l?000N6L00:A[502cM4P0VEPl0:5P:02`LP<0
lOJ20?ooo@3oo_l:0?ooo`001P3oool07P3loOl0jn_Y0=_C`03<cLP0a<G7
0<?4aP3?HRX0hVL00>>9003_a@00j]d00=kB003FcP00_kD00;2R0`2FS@00
P8<007Ye001fJ`00H4l005i<0P2gT`40[F430<>0003PV@00om820?[j803o
ood0oogo0?koo`X0oooo00060?ooo`0K0?neV03[aZ`0bZUf0=fQ2`3nbQH0
kK8F0?C4003bb`40mN/00>kN0P3/e`00iMP00<k60@33^@00YY`008F00P24
O000NFl006mU0`1VB000Z9L00>KA003Z^P00nLX20?Nj003n_`80in;L0080
oooo00<0o_om0?ooo`3oool0203oool000030?ooo`3no_h0oooo0080oooo
01h0o_kn0:JD903GLP00dEL10;i:0@3GX`40mmT00?GO003ed`00mmX20?GK
0P3[d@00jmd20=kI0@35`@00Wil007aj0@1fM@00KF0205e6002<L080f]<0
0>74003b]P00o/820?nk0P3LaWd0eMGE0?ooo@3nool;0?ooo`000P3no_h4
0?ooo`0N0?ooo@3Ni<P0]<QW0;VD002kQ@00i:l00?F_003_^`D0l;/00?G<
003_f000lMd00>kV0039c000TYL006ic001TF`00Fd/005Lo0@36/080iLD0
0?:l0P3n/`00o[400?Ne6@34a<@0a<G70?[jo03nood0ooko2P3oool0000T
0?knoP3oool0oooo0?ooo`3no_h0oooo0?koo`3mo_l0oooo0>oJY@34WP/0
di400>B[0P3a^`00lK000>Za003[/`00lkd10?cH003jn`00LXH304M30`1=
?`00BC@007eH003d`P<0lJD10?^S103mYP00oiT00>jc<`35`l@0`LG40<O5
b03QhMl0kNg_2P3oool000@0oooo0200o_kn0?ooo`3oo_l0ooom0?ooo`3o
ood0n>[@0<R>003LU`00n:810?>J003aY@40kjd10>j/003fUP<0n98104@V
000^6P00F3T003hJ0P2YH040nYh00?FO003hX000o9H30?nO003oVP80c<6]
0<C4`P39bL/0b<S60<C4a0X0oooo00000`3oool0o_kn0?ooo`040?ooo`0M
0?coo`3ooOl0o_kn0?koo`3?Q`00fIT30>J@003dV@00kYH00>fD0P3VR000
gfX10:i1001Q4`00F24204lP0P115`00_5d00?f70@3dS@00nHl00?Z@003k
X@80oY/00?FR803VgM@0lo?c0?Gel`3dmOL02P3oool000H0oooo01h0ooom
0?koo`3oool0ooko0>k]lP3?R@<0ehH10>B6003/U000li<10>^40@3EK`00
`5P10:Hl0020:000K24105dQ00194P00_D/20?md003gN0<0mg/00?R:003k
S000ni820?nH003oW0X0ooko0?ooo`3oo_l:0?ooo`001P3oool07P3nool0
o_kn0?ono`3fmO40inOW0=1n0@3CQ040eg<10>Mj0`3/O000gFl00<9M0@2/
@@00Ycd008P[001d8@40KB40058M002]A000o6P00?U[003gN@40oWL10?]m
0P3kQ@00nh800?n;0P3mR0D0o/J;0?ojm`X0oooo00080?ooo`0L0?ooo@3l
ool0ajf:0<ie0034KP40`f/10>:4003NM000bU400;Hn0@31E080]TH009D`
0P23:P00M1X206PK0@2;;000of810?aY003iL@40n7D00?Uh003iQ`00nJ0N
0?JiAP3S`400c:LU0>RnK0X0oooo00060?ooo`0N0?kmo`3oood0o_oj0>C>
``32JP00_VT00;MU0@3>G@40m6000<=C0@3HKP00gI400?2j003jXP80hHX0
0:=N0026B`00I2T006TC0P3SE@40mFP00?Y^003gM000kkMH0=C;[034a<H0
gmoO0?koo`3oo_d0o?go2P3oool000H0oooo01`0o_om0?WelP37X840_FX0
0<R24P34Rc40/f@80=YH003kP000kjd10>kE003hcP00kk`10?6T003/T000
dG/00;I`002BH@80Id000:m;003gH`40nVD00?Ym4@3no_`0n?[g0>KVj033
aL@0oOon0P3oood:0?ooo`001P3oool0103kno/0f<^O0<c9PP3ZjL/20?oo
o`0H0=^TJ`2nN0T0UWP00>O>0P3``@80l;h00>BW003`X@00i8d00<Yd0`2k
M000]6d109aU001iG000]FH10?EV003kL@`0ooko0?ooo@3ooo/0m_Kf0?Sf
m`3oool0o_oo2P3oool000H0oooo01d0oogn0?gooP3oo_l0oOol0?ooo@3n
noH0n?OS08]F8P29H`00n=H20>jf003][P40jI`00>^M003VS080c7`10=9n
0@2fKP00YFL208]P0@23GP<0dF@00?m/0P3mh]40ooom0?omo`3nood0no_k
0?koo`0;0?ooo`001P3oool07P3oo_l0oooo0?koo@3nood0oooo0?koo@3k
nod0Ue<80>_0003ba`00kk`10>j/003^Y@40m:000=^A003=RP00b7P00<1h
002cN0@0YG0@09QT0@28I`00jfP00?f`Q03mooh0oooo0?ooo@3no_h0ooom
0?omoPX0oooo00060?ooo`0N0?ooo@3oo_l0oOon0?ooo`3oo_l0ooom0=k<
`P2eM`00oM`30?Fd0P3hb2H0nmma0?;1@03[X@00hID30>J;002nN@00giXI
0>_[h@3>c<l0jN;H0<:/LP1/G@`0ofH00?KSe@3ooOl0oooo0?koo`3mo_l0
o?oo2P3oool000L0oooo01T0ooko0?ooo`3nool0oooo0?omo`2>Q8D0mNT0
0>^k0`3kjIl0oooo0?ono`3oood0lMQn0>JI003:Q`80d7`00?gZP`3oo_l0
kno_0?GgmP3no_h0kO;/07eR2`3aNRP0ooom0080oooo00<0ooom0?ooo`3o
ool02@3oool000H0oooo01D0o_oo0?ooo`3oood0o_oo0?ono`3oo_d0XFH0
0?on:P3nno@0o_kn0?koo`3oood0o_oo0?ooo@3KZAH0fXP00<R10@3jnl40
ooom0?ono`3oood00P3oool01P3jn_X0ZHiI0?OLb`3ooOh0ooom0?ono`/0
oooo00060?ooo`030?ooo@3lool0ooko0080oooo00<0d[6N0?oN7`3oool0
0`3oool0203nood0oooo0?ono`3Vhl00jhX00?BQ7`3oood0o_oo0P3oo_l0
1P3oood0o_om0?ono`3oood0lNgZ0?oon`80oooo00<0ooko0?ooo`3oool0
203oool000H0oooo00L0o_oo0?ooo@3oood0o_oo0?oon`3Qb7H0o_og0080
oooo00@0ooom0?kno`3nool0o_kl0P3oo_l0303?SA<0kJ@T0?ooo`3nool0
o_om0?ooo`3no_`0ooko0?ooo`3oood0ookl0?ooo@d0oooo00060?ooo`06
0?ooo@3oo_l0oooo0?koo`3oool0o_oc1@3oool20?koo`0>0?kno03oool0
bJUN0>RU8`3oool0o_kn0?ooo`3nool0ooom0?knoP3oool0o_kn0?ooo@3n
o_h=0?ooo`001P3oool0103ooOl0o_om0?coo`3oood20?ooo`060?ono`3o
ool0ooom0?omoP3oood0o_kn0P3nool0103nood0h]Bg0>N_=P3moO/20?oo
o`030?koo`3oo_`0oooo0080oooo00<0o_kn0?ooo`3oool0303oool000L0
oooo00P0o_kn0?ono`3no_l0ooom0?koo`3oo_l0oooo0?kno080oooo00<0
ooom0?ooo`3no_h00P3oool01`3no_h0ooko0?ooo@3ooOh0o_oo0?ooo`3n
ood00`3oool00`3no_h0oooo0?ooo`0;0?ooo`001P3oool20?ono`070?oo
n`3oool0o?oo0?ono`3lood0oooo0?knoP020?ooo`090?knoP3oool0ooom
0?ooo`3oo_l0o?oo0?knoP3oool0ooko00H0oooo00<0o_kn0?ooo`3oool0
2`3oool000H0oooo00`0o_oo0?kon`3oool0o_kl0?ooo`3no_h0o_oo0?kn
oP3oool0o_kn0?kno03oood30?ooo`0;0?ono`3ooo/0oooo0?kno`3oo_l0
o_kn0?ooo`3ooo/0o_kn0?ooo`3no_h00`3oool00`3no_h0oooo0?ooo`08
0?ooo`00\
\>"], "Picture",
  ImageSize->{45.9375, 36.9375},
  ImageMargins->{{72.8125, 0}, {0, 1.375}},
  ImageRegion->{{0, 1}, {0, 1}},
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "The computer will then load parts of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " into its memory and soon a blank window will appear on the screen. This \
window is the visual interface to a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook and it has many features that are useful to the user."
}], "TextFirst",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell["\<\
Notebooks allow you to write text, perform computations, write and run \
programs, and create graphics all in one document. Notebooks also have many \
of the features of common word processors, so those familiar with word \
processing will find the notebook interface easy to learn. In addition, the \
notebook provides features for outlining material which you may find useful \
for giving talks and demonstrations.\
\>", "Text",
  CellTags->{"i:31", "b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "When a blank notebook first appears on the screen (either from just \
starting ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " or from selecting ",
  StyleBox["New", "SR"],
  " in the ",
  StyleBox["File", "SR"],
  " menu), you can start typing immediately. For example, if you type ",
  StyleBox["N[Pi,200]", "MR"],
  " press \[ShiftKey]\[LeftModified]\[EnterKey]\[RightModified] (hold down \
the Shift key while pressing the Enter key) to evaluate an expression. ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will evaluate the result and print the 200-decimal digit approximation to \
",
  Cell[BoxData[
      \(TraditionalForm\`\[Pi]\)]],
  " on the screen."
}], "Text",
  CellTags->{"i:32", "b:1.0", "IPM3.1.2.1"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .55108 
%%ImageSize: 373 206 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
4.0766e-017 0.00268817 -0.00268817 0.00268817 [
[ 0 0 0 0 ]
[ 1 .55108 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .55108 L
0 .55108 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00269 translate
1.00269 .55377 scale
1119 string
373 206 8 [373 0 0 206 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FEFEFEFEFEFEFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFC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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFBFBFB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FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9
F8F8F8F6F6F6F5F5F5F3F3F3F1F1F1F0F0F0EFEFEFEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEFEFEFF0F0F0F1F1F1F3F3F3F5F5F5F6F6F6F8F8F8
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7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FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7F5F5F5
F3F3F3F0F0F0EDEDEDEAEAEAE7E7E7E4E4E4E2E2E2E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E2E2E2E4E4E4E7E7E7EAEAEAEDEDEDF0F0F0F3F3F3
F5F5F5F7F7F7F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F3F3F3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FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F4F4F4F0F0F0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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F5F5F5F1F1F1EDEDED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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F6F6F6F3F3F3EFEFEFEAEAEA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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F5F5F5F1F1F1EDEDEDE7E7E7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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F4F4F4F0F0F0EBEBEBE4E4E4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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFBFBFBF9F9F9F6F6F6F4F4F4EFEFEFE9E9E9E2E2E2
DADADAD2D2D2C8C8C8BFBFBFB6B6B6ADADADA5A5A5A0A0A09E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9EA0A0A0A5A5A5ADADADB6B6B6BFBFBFC8C8C8D2D2D2DADADA
E2E2E2E9E9E9EFEFEFF4F4F4F6F6F6F9F9F9FBFBFBFDFDFDFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F6F6F6F3F3F3EEEEEEE8E8E8E1E1E1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000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
ECECECBABABA9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9FBDBDBDECECECE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
969696DBDBDBCACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACADBDBDBA3A3A3FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFB969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFFE0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0FFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFB969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFFE0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFFFFFFF969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFB969696FFFFFFCECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E0000000000000000009E9E9ED9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
DCDCDC515151B9B9B9E0E0E0D9D9D91A1A1A000000383838D9D9D9E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD1D1D1D0D0D0D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D1D1D1D1D1D1
D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1FFFFFF969696FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFA969696FFFFFFCECECECECECECECECECECECECECECE3E3E3ECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0E0E0E0E0E0E0
E0E0E0ADADAD545454E0E0E0ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6
D7D7D7D7D7D7D7D7D7D6D6D6D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6959595
D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D7D7D7D7D7D7D7D7D7D6D6D6
D6D6D6D6D6D6FFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9969696FFFFFFCECECECECECECECECECECECE3E3E3E3E3E3E3E3E3ECECECE
CECECECECECECECECECECECECECECEFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFFE0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9E0E0E0E0E0E0
E0E0E0DCDCDC515151B9B9B9ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595
DCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDC
DCDCDCDDDDDDFFFFFF969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9969696FFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3E
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFFE0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2E0E0E0E0E0E0
E0E0E0E0E0E0C4C4C4484848D2D2D21A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000000000
000000000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFD4D4D4D4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDCDCDCFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595
DCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDC
DCDCDCDDDDDDFFFFFF969696F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8969696FFFFFFD4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D43E3E3E3E3E3E
3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2DCDCDC494949
0000000D0D0DD2D2D26969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000
000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFDBDBDBDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696
969696FFFFFFE4E4E4E3E3E3E3E3E3E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595
E3E3E3E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E4E4E4
E3E3E3E4E4E4FFFFFF969696F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7969696FFFFFFDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9ADADAD545454
E0E0E06969697E7E7EC4C4C4484848D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFE4E4E4E4E4E4E4E4E4
3E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595
E9E9E9EAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEA
E9E9E9E9E9E9FFFFFF969696F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5969696FFFFFFE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E
3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E000000383838D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0ADADAD545454
E0E0E06969697E7E7EE0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9EAEAEAFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595
E9E9E9EAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEA
E9E9E9E9E9E9FFFFFF969696F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDED3E3E3E3E3E3E3E3E3E
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D9D9D9525252D9D9D9E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0C4C4C41D1D1DD2D2D2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDFFFFFF969696F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDFFFFFF969696F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
969696FFFFFFF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7FFFFFF969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEF969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
969696FAFAFAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFAFAFA969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEF969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0DADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
D6D6D6B0B0B0B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8A9A9A9CECECEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADAE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB6969699E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECECEEEEEE8D8D8D9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696DEDEDEDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFCECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFCECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFCECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFCECECECECECE
CECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000ECECEC969696FFFFFFCECECECECECE
CECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECEFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFCFCFCFCFCFCF
CFCFCF3E3E3E3E3E3E3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFD4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFE4E4E4E4E4E4
3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E43E3E3EE4E4E4E4E4E4E4E4E4FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFF555555555555
555555555555555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696D7D7D7FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECECDADADA929292B6B6B6B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF6363633F3F3F000000414141FFFFFF
FFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFFFFFFFF7878780000000000008F8F8FFFFFFF
FFFFFF6363633F3F3F000000414141FFFFFFFFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFF
F7F7F75E5E5EFFFFFFFFFFFF818181D2D2D2FFFFFF7878780000000000008F8F8FFFFFFF
F7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4F7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4
FFFFFF7878780000000000008F8F8FFFFFFFFFFFFF6363633F3F3F000000414141FFFFFF
FFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFFFFFFFF7878780000000000008F8F8FFFFFFF
F7F7F75E5E5EFFFFFFFFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFF
FFFFFF787878B4B4B4C6C6C68F8F8FFFFFFFFFFFFF7878780000000000008F8F8FFFFFFF
FFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFFFFFFFF787878000000000000606060FFFFFF
FFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFFFFFFFF787878B4B4B4C6C6C68F8F8FFFFFFF
FFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFFFFFFFF
FFFFFFFFFFFFC6C6C68F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFF
F7F7F75E5E5EFFFFFFFFFFFF818181D2D2D2FFFFFF6363633F3F3F000000414141FFFFFF
FFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFFF7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4
FFFFFF7878780000000000008F8F8FFFFFFFFFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFF
FFFFFFFFFFFFC6C6C68F8F8FFFFFFFFFFFFFFFFFFF787878B4B4B4C6C6C68F8F8FFFFFFF
FFFFFF7878780000000000008F8F8FFFFFFFF7F7F75E5E5EFFFFFFFFFFFF818181D2D2D2
FFFFFF7878780000000000008F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0
F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDEAEAEAEAEAEAE4E4E4DCDCDCDCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1212128F8F8FFFFFFF
FFFFFFC6C6C60000008F8F8FFFFFFFFFFFFFFFFFFFC6C6C60000000000008F8F8FFFFFFF
DFDFDF1212120000000000000F0F0FEFEFEFFFFFFFC6C6C60000008F8F8FFFFFFFFFFFFF
FFFFFFFFFFFFA2A2A2000000000000B4B4B4FFFFFFC6C6C60000000000008F8F8FFFFFFF
FFFFFFDFDFDF121212000000B4B4B4FFFFFFFFFFFFC6C6C60000000000008F8F8FFFFFFF
FFFFFFC6C6C60000000000008F8F8FFFFFFFFFFFFFDFDFDF121212000000B4B4B4FFFFFF
DFDFDF1212120000000000000F0F0FEFEFEFFFFFFF787878000000000000606060FFFFFF
FFFFFFC6C6C60000008F8F8FFFFFFFFFFFFFFFFFFFDFDFDF121212000000B4B4B4FFFFFF
FFFFFF787878000000000000606060FFFFFFFFFFFFA2A2A2000000000000B4B4B4FFFFFF
FFFFFFA2A2A20000000000008F8F8FFFFFFFFFFFFFDFDFDF121212000000B4B4B4FFFFFF
FFFFFFC6C6C60000000000008F8F8FFFFFFFDFDFDF1212120000000000000F0F0FEFEFEF
FFFFFFFFFFFFDFDFDF1212128F8F8FFFFFFFFFFFFFC6C6C60000000000008F8F8FFFFFFF
FFFFFFFFFFFFDFDFDF1212128F8F8FFFFFFFFFFFFFFFFFFFDFDFDF1212128F8F8FFFFFFF
FFFFFF787878000000000000606060FFFFFFFFFFFFC6C6C60000000000008F8F8FFFFFFF
FFFFFFA2A2A20000000000008F8F8FFFFFFFFFFFFFDFDFDF121212000000B4B4B4FFFFFF
FFFFFFDFDFDF121212000000B4B4B4FFFFFFDFDFDF1212120000000000000F0F0FEFEFEF
FFFFFFA2A2A2000000000000B4B4B4FFFFFFFFFFFFC6C6C60000008F8F8FFFFFFFFFFFFF
FFFFFFFFFFFFA2A2A2000000000000B4B4B4FFFFFFFFFFFFDFDFDF1212128F8F8FFFFFFF
FFFFFFDFDFDF121212000000B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDED959595959595959595959595959595959595D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDED959595959595959595959595959595959595D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555
FFFFFFFFFFFFFFFFFF555555FFFFFF555555FFFFFFFFFFFF555555FFFFFF555555FFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFF555555FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFC6C6C68F8F8FFFFFFFFFFFFFFFFFFF787878B4B4B4C6C6C68F8F8FFFFFFF
FFFFFFFFFFFFC6C6C68F8F8FFFFFFFFFFFFFFFFFFF787878000000000000606060FFFFFF
F7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFF
FFFFFF6363633F3F3F000000414141FFFFFFFFFFFF787878000000000000606060FFFFFF
FFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFFFFFFFF787878000000000000606060FFFFFF
FFFFFF7878780000000000008F8F8FFFFFFFF7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4
FFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFFF7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4
FFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFFFFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFF
FFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFFFFFFFF7878780000000000008F8F8FFFFFFF
FFFFFF787878B4B4B4C6C6C68F8F8FFFFFFFFFFFFF6363633F3F3F000000414141FFFFFF
FFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFFFFFFFF6363633F3F3F000000414141FFFFFF
FFFFFF787878B4B4B4C6C6C68F8F8FFFFFFFFFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFF
FFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFFFFFFFF7878780000000000008F8F8FFFFFFF
FFFFFFFFFFFFFFFFFFF7F7F75E5E5EFFFFFFFFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFF
FFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFFF7F7F72C2C2CEFEFEFFFFFFFA2A2A2B4B4B4
FFFFFF787878000000000000606060FFFFFFF7F7F75E5E5EFFFFFFFFFFFF818181D2D2D2
FFFFFFFFFFFFFFFFFFF7F7F7C7C7C7FFFFFFFFFFFF7878780000000000008F8F8FFFFFFF
FFFFFF7878780000000000008F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
555555555555555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555555555FFFFFFFFFFFF555555FFFFFFFFFFFF555555555555FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E4E4E4DDDDDDDDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDEAEAEAEAEAEAE4E4E4DDDDDDDDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF555555555555555555555555555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDED
EDEDEDE9E9E9E9E9E9E3E3E3DCDCDCDCDCDCD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A28F8F8FFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF787878B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A28F8F8FFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696D7D7D7FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF555555FFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFF555555
FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFF555555FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F7F7F71D1D1D000000414141F3F3F3545454B4B4B4FFFFFFFFFFFFA2A2A28F8F8FFFFFFF
FFFFFFA2A2A2000000090909D2D2D2FFFFFFFFFFFFC6C6C6000000000000000000000000
B4B4B4FFFFFFFFFFFFC6C6C6414141F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFA2A2A2000000
0000000000000F0F0FEFEFEFFFFFFFA2A2A20000000000008F8F8FFFFFFFFFFFFFA2A2A2
0000000000008F8F8FFFFFFFFFFFFFFFFFFFA2A2A28F8F8FFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECECFDFDFD939393B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B5B5B5B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8939393ECECEC
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF555555FFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFF555555
FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFC6C6C6414141F7F7F7A2A2A2000000B4B4B4FFFFFFFFFFFFA2A2A28F8F8FFFFFFF
FFFFFFFFFFFFA2A2A28F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A28F8F8FFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878
414141F7F7F7FFFFFFFFFFFFDFDFDF1B1B1BD2D2D2DFDFDF1B1B1BD2D2D2DFDFDF1B1B1B
D2D2D2DFDFDF1B1B1BD2D2D2FFFFFFFFFFFFA2A2A28F8F8FFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECECEEEEEE8D8D8D9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555
555555FFFFFFFFFFFF555555FFFFFFFFFFFF555555555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFCECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFCECECECECECE
3E3E3ECECECECECECECECECECECECECECECE3E3E3ECECECECECECECECECEFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFCFCFCF3E3E3E
3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFD4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D43E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF555555555555555555555555555555555555FFFFFFFFFFFF555555555555
555555555555555555FFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFDBDBDBDBDBDB
DBDBDB3E3E3E3E3E3E3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFE4E4E4E4E4E4
E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFEDEDEDEDEDED
EDEDEDEDEDEDEDEDED3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFF4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECEC969696D7D7D7FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFECECECDADADA929292B6B6B6B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
696969ABABABFCFCFC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000F7F7F79A9A9A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A9A9A9AF7F7F70000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000F7F7F7BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFF7F7F70000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3F7F7F70000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000F1F1F1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFE3E3E3A5A5A5767676767676767676767676777777767676777777
777777767676767676767676767676767676767676767676767676767676A5A5A5E3E3E3
FFFFFFFFFFFFE3E3E3A5A5A5767676767676767676767676777777767676777777777777
767676767676767676767676767676767676767676767676767676A5A5A5E3E3E3FFFFFF
FFFFFFEBEBEBA5A5A5565656565656565656565656575757565656575757575757565656
565656565656565656565656565656565656565656565656A5A5A5EBEBEBFFFFFFFFFFFF
FFFFFFFFFFFFF1F1F10000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000E1E1E1FCFCFCFBFBFBFCFCFCFCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC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000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A28D8D8D767676
F7F7F7F7F7F77676768D8D8DA2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7
F7F7F75656565B5B5B6F6F6F717171727272727272727272727272727272727272727272
7272727272727272727272727272727272727171716F6F6F5B5B5B565656F7F7F7F7F7F7
F7F7F7F7F7F7E0E0E00000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000DEDEDEF1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1
F1F1F1F1F1F1F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2
F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1767676B8B8B8ABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABB8B8B8767676
F1F1F1F1F1F1767676B8B8B8ABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABB8B8B8767676F1F1F1
F2F2F25656566F6F6F6F6F6F727272777777777777777777777777777777777777777777
7777777777777777777777777777777777777272726F6F6F6F6F6F565656F1F1F1F1F1F1
F1F1F1F2F2F2DEDEDE0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000DCDCDCEBEBEBEBEBEBECECECEBEBEBEBEBEBA7A7A7A2A2A2A2A2A2A1A1A1A1A1A1
A0A0A09F9F9F9D9D9D9F9F9F9F9F9FA0A0A0A0A0A0A1A1A1A1A1A1A7A7A7EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBD4D4D4C5C5C5C5C5C5C5C5C5D7D7D7EBEBEBEBEBEB
EAEAEAC8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3EBEBEBE0E0E0C5C5C5C5C5C5
CCCCCCE9E9E9CECECEC7C7C7E9E9E9EBEBEBE0E0E0C5C5C5C5C5C5CCCCCCEAEAEAD4D4D4
C5C5C5DDDDDDEBEBEBE0E0E0C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEAEAEAC8C8C8
C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBE5E5E5C7C7C7D7D7D7EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFB1B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1BBBBBB767676
EBEBEBEBEBEB767676BBBBBBB1B1B1B1B1B1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEB
EBEBEB565656777777727272747474777777787878787878787878787878787878787878
787878787878787878787878787878777777747474727272777777565656EBEBEBEBEBEB
EBEBEBEBEBEBDCDCDC0000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000DADADAE5E5E5E5E5E5E5E5E5E5E5E58A8A8A9B9B9BACACACE9E9E9E9E9E9E9E9E9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000000D9D9D9DFDFDFDFDFDFDFDFDFDFDFDFDEDEDEA0A0A0EAEAEAE9E9E9E5E5E5EAEAEA
EAEAEAD0D0D0747474EDEDEDEEEEEEADADADD0D0D0F2F2F2F3F3F3A3A3A3DFDFDFDEDEDE
DEDEDEDFDFDFDEDEDE6C6C6C151515ABABABDEDEDEC3C3C31C1C1C545454C6C6C6D6D6D6
252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4D5D5D54E4E4E383838C4C4C4DDDDDD
DBDBDB4E4E4E191919BABABAD9D9D94E4E4E383838C4C4C4DDDDDDD7D7D7252525383838
C4C4C4C2C2C21B1B1B515151D9D9D9C3C3C34C4C4CD8D8D8D7D7D7252525191919C5C5C5
DEDEDE8F8F8F151515A4A4A4D9D9D9DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
C3C3C31C1C1C727272D1D1D1DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBBFFFFFF727272727272727272727272
727272727272272727BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBC3C3C3767676
DEDEDEDEDEDE767676C3C3C3BBBBBBBBBBBBFFFFFF272727BBBBBBBBBBBBBBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBBBBBBBFFFFFF272727BBBBBBBBBBBBBBBBBBC3C3C3767676DFDFDF
DEDEDE5656567777777878787B7B7B303030FFFFFFFFFFFFFFFFFF7D7D7D7D7D7D7D7D7D
7D7D7D7D7D7DFFFFFFFFFFFFFFFFFF3030307B7B7B787878777777565656DEDEDEDEDEDE
DFDFDFDEDEDED9D9D90000009A9A9A9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8
E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000D7D7D7D8D8D8D9D9D9D8D8D8D8D8D89090908F8F8F8E8E8EE9E9E9818181515151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000000D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3858585A3A3A3E6E6E6EAEAEAEAEAEA353535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000000D3D3D3CECECECECECECFCFCFCECECECECECE878787939393D3D3D3DBDBDBADADAD
C1C1C1B0B0B0A9A9A9AAAAAA8D8D8D919191B0B0B0CBCBCBEFEFEFA5A5A5CECECECECECE
CECECECFCFCFC7C7C7232323515151C8C8C8CECECECECECE656565191919B9B9B9C7C7C7
232323515151C7C7C7CECECE858585151515A4A4A4C7C7C7232323383838C1C1C1CECECE
CBCBCB494949191919B9B9B9C7C7C7232323383838C1C1C1CECECEC7C7C7232323383838
C1C1C1858585151515A4A4A4CACACAC7C7C7232323515151B1B1B11A1A1A727272C5C5C5
CECECE858585151515A4A4A4CDCDCDCECECECECECECECECECECECECECECECECECECECECE
B4B4B41A1A1A727272CACACACECECECECECECECECECECECECECECECECECECDCDCDC7C7C7
CDCDCDC7C7C7CACACACECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECE767676CECECECBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCECECE767676
CECECECECECE767676CECECECBCBCBCBCBCBFFFFFF272727CBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCBCBCBFFFFFF272727CBCBCBCBCBCBCBCBCBCECECE767676CFCFCF
CECECE5656567D7D7D828282858585848484858585858585282828FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF2828288585858585858484848585858282827D7D7D565656CECECECECECE
CECECECECECED3D3D3000000A8A8A8ADADADB3B3B3BBBBBBC3C3C3CCCCCCD5D5D5DDDDDD
E4E4E4EBEBEBF0F0F0F4F4F4F7F7F7F9F9F9FCFCFCFDFDFDFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFF
000000D0D0D0C9C9C9C9C9C9C9C9C9C9C9C97373739494947E7E7E797979A0A0A0D1D1D1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000000CFCFCFC5C5C5C5C5C5C5C5C5C5C5C5C5C5C5979797B7B7B7ECECECB9B9B98B8B8B
B7B7B7AAAAAAA4A4A47E7E7E717171C4C4C4D9D9D9F6F6F6F9F9F9A5A5A5C5C5C5C5C5C5
C5C5C5C5C5C5BFBFBF222222515151C6C6C6C5C5C5C5C5C5616161191919B9B9B9BFBFBF
2222225D5D5D101010101010151515A4A4A4C2C2C2454545101010101010101010515151
616161101010191919B6B6B6454545101010101010101010515151BFBFBF222222383838
BFBFBFC5C5C5ACACAC1A1A1A101010101010727272C5C5C5C5C5C5C2C2C2454545101010
101010101010151515A4A4A4C6C6C6C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5ACACAC5C5C5C
ADADAD1A1A1A717171C6C6C6C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5BFBFBF222222303030
1A1A1A8E8E8EC5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5767676D5D5D5D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D5D5D5767676
C5C5C5C5C5C5767676D5D5D5D1D1D1D1D1D1FFFFFF272727D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF272727D1D1D1D1D1D1D1D1D1D5D5D5767676C5C5C5
C5C5C5565656828282878787898989898989898989898989898989FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF898989898989898989898989898989878787828282565656C5C5C5C5C5C5
C5C5C5C5C5C5CFCFCF000000BCBCBCBFBFBFC3C3C3C9C9C9D0D0D0D7D7D7DDDDDDE4E4E4
EAEAEAEFEFEFF3F3F3F6F6F6F8F8F8FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFEFEFEFFFFFF
FFFFFF
000000CDCDCDC1C1C1C1C1C1C1C1C1C1C1C1757575999999A7A7A7EDEDEDC9C9C9737373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000000CCCCCCBABABABDBDBDBDBDBDBEBEBEBDBDBD9E9E9EE3E3E3B6B6B6969696828282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000000CACACAB8B8B8BABABABABABABABABA7C7C7C8D8D8D787878999999F0F0F0C2C2C2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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B88686869E9E9ED4D4D4F0F0F0F2F2F2CECECE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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8868686949494F2F2F2F3F3F3A0A0A0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000000C9C9C9B5B5B5B5B5B5B5B5B5B5B5B5777777A0A0A0E0E0E0F3F3F3E5E5E5ECECEC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000000C9C9C9BABABAB1B1B1B1B1B1B1B1B1B1B1B18D8D8DAAAAAAAAAAAAAAAAAAAAAAAA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000000C3C3C3C9C9C9AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAE8D8D8D767676CECECEFBFBFBFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFBCECECE7676768D8D8D
AEAEAEAEAEAE8D8D8D767676CECECEFBFBFBFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFBCECECE7676768D8D8DAEAEAE
AEAEAE999999565656B5B5B5CBCBCBDFDFDFE8E8E8E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8DFDFDFCBCBCBB5B5B5565656999999AEAEAEAEAEAE
AEAEAEC9C9C9C3C3C3000000F6F6F6F6F6F6F7F7F7F8F8F8F8F8F8F9F9F9FAFAFAFCFCFC
FCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
000000C3C3C3D4D4D4B7B7B7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676767676767676767676767676767676767676
7676767676767676767676767676767676767676767676767676767676768D8D8DA7A7A7
A7A7A7A7A7A7A7A7A78D8D8D767676767676767676767676767676767676767676767676
7676767676767676767676767676767676767676767676767676768D8D8DA7A7A7A7A7A7
A7A7A7A7A7A78D8D8D565656565656565656565656565656565656565656565656565656
5656565656565656565656565656565656565656565656568D8D8DA7A7A7A7A7A7A7A7A7
B7B7B7D4D4D4C3C3C3000000F9F9F9F9F9F9F9F9F9FAFAFAFBFBFBFCFCFCFCFCFCFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
000000AAAAAAE7E7E7D4D4D4BEBEBEB3B3B3AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAB3B3B3BEBEBE
D4D4D4E7E7E7AAAAAA000000FBFBFBFBFBFBFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
0000008A8A8AD9D9D9FCFCFCE4E4E4D2D2D2CCCCCCBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBECCCCCCD2D2D2E4E4E4
FCFCFCD9D9D98A8A8A000000FCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
000000696969979797D9D9D9FCFCFCF2F2F2ECECECE3E3E3D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9E3E3E3ECECECF2F2F2FCFCFC
D9D9D9979797696969000000FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
000000FFFFFF696969979797D3D3D3FCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFCFCFCFCFCFCD3D3D3
979797696969FFFFFF000000FDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
000000FFFFFFFFFFFF6969696969698A8A8A979797AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA9797978A8A8A696969
696969FFFFFFFFFFFF000000FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{373, 206},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005e0000cR000`400?l00000o`00003oo`3ooomf
0?ooo`00o`3ooomf0?ooo`003P3ooooo0?knoUT0o_kn3`3oool000`0oooo103no_ko0?gmoED0oOgm
103no_h=0?ooo`002P3oool30?kno_l0oOgmF`3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo4l0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?clo?l0no_kD@3kno/30?cl
o0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinE80
nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_IA0?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`4
0?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gmo@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0
moOg0?KfmP3emOD00P3dm?Co0??cldl0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm00X0o?cl0?_kn`3inOT0
n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^k^i?0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0
m_Kf0?Shn03inOT0no_k0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj4l0j>SX00/0jNWY0>_[j`3]
kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0030?gmo@80o_kn103oool000<0oooo0100
o_kn0?gmo@3moOd0oOgm0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN5?0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3kno/0
o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`000P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0
m_Kf0??cl`3_knl0k>c/0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SHC@3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`00`3moOd20?knoP<0oooo
00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;R
hP3MgMd0fMWI0=GEe@3Bd]80d=3@o`3?clm=0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0oooo0040oooo00000`3o
ool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3A
dM40c<c<0<S8b036a/Ko0<G5aDd0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo0?knoP3no_h00P3moOd0
3`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl
_03o0;^k^dd0^k^k00l0_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gmo@020?gmo@0?0?clo03j
n_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a
C@2a/K403`2b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd00P3moOd03`3lo?`0nOWi
0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYTd0
YZJV00l0Z:RX0:f][@2c/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080oOgm00l0no_k0?Win@3f
m_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S80;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYi=09jN
WP0?0:2PX02UYJD0[Jf]0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0o?cl0?_kn`3inOT0m_Kf
0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l_;`0/[:b0:RXZ02PX:00o`2JVYY?09ZJVP0?0:2P
X02XZ:P0/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o0000064000000100VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00003
0000003lo?`0o?cl0?l0o?clG@3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`00
0000o?cl0:^[Z`3o0:^[Ze`0Zj^[0180o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFUK06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj902OWil00`2m_Kd0k>c/0>OWi`0]0>OWi`030>k^kP2=SHd0WinO00`0WinO00<0SHf=
0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3Ng]h0fm_K
00`0fm_K00@0g]kN09JFUP2FUYH0fm_K903:b/X00`3Kfm/0Xj>S0?_kn`0]0?_kn`0309JFUP3Ng]h0
fm_K00`0fm_K00<0g]kN09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo903@d=000`3oool0UYJF0?_kn`0]0?_kn`0309JFUP3oool0c/k>00`0c/k>00<0oooo
09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY
09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF
09JFUP3ooolT0<o?c`030?ooo`2FUYH0no_k02d0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ0000000=3@d0d];B0>3P
h03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3Lg=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph
0=WIf@0F0>3Ph00309JFUP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM401@3@d=0:0=7Ad@80d=3@0`3AdM450=3@d0P0dM7A00<0oooo09JF
UP3jn_X0;@3jn_X00`2FUYH0oooo0<k>cP040<k>cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JF
UP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000
003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3P
h01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0103>c/h303hn?PD0c/k>
00@0oooo09JFUP2FUYH0oooo2P3Fe]H30=OGe`80e]KF00L0UIFE0=KFeP2EUID0e]KF09FEU@3Fe]H0
UIFE00P0e]KF0`3GemL30=KFeP030?ooo`2FUYH0nOWi02d0nOWi00<0UYJF0?ooo`3>c/h00`3>c/h3
03hn?PH0c/k>00<0oooo09JFUP3Ph>002@3Ph>020?ooo`80h>3P0P3oool05@3Ph>00JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3P
h0030?ooo`3Ph>00h>3P00@0h>3P0100a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0
d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0`3Ph>00203Lg=`0DE5A0;Vi^@2][Jd0E5AD0>3Ph01Y
JFT0WYjN5P3Ph>000`2FUYH0oooo0<o?c`030<o?c`<0?Shn1P3?cll0103oool0UYJF09JFUP3oool3
0=cLg0X0gMgM00X0g=cL0?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID0g=cL2P3MgMd20=cL
g0030=gMg@3oool0UYJF02h0nOWi00<0UYJF0?ooo`3?cll0103?cll303hn?PD0clo?00<0oooo09JF
UP3Ph>00203Ph>020;>c/`0J0?ooo`3Ph>00/k>c0;>c/`3oool0h>3P06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3P
h>00h>3P00@0h>3P0100a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0=WIf@0V9RH0bl_;02HV9P3B
d]80h>3P0=WIf@0V9RH0d];B103Ph>001`34a<@0B4Q80=;BdP0J6QX0000000d=3@3Bd]802P3Ph>07
000000D0h>3P00<0UYJF0?ooo`3De=@00P3De=@303hn?PL0e=CD00@0oooo09JFUP2FUYH0oooo0`3L
g=`:0=gMg@0:0=cLg03oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0=cLg0X0gMgM0P3Lg=`0
0`3MgMd0oooo09JFUP0^0?Shn00309JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0
h>3P00P0h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3P
h00G0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B0>3Ph03IfMT0
9RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2NWYh03`3Ph>05000000H0h>3P00@0UYJF0?oo
o`3Kfm/0fm_K0`0n?Sh80=_Kf`050?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0D0hn?S103T
i>@02@3oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0>?Sh`020>CTi0D0hn?S103Ti>@0103S
hn<0i>CT0?ooo`2FUYH^0?Ogm`0309JFUP3oool0fm_K00H0fm_K0`0n?Sh30=_Kf`030?ooo`2FUYH0
h>3P0100h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000
o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>006034a<@0B4Q80=WIf@3IfMT0
9RHV0=;BdP3Ph>00a<C404Q8B03Bd]809RHV0=;BdP3Ph>00a<C404Q8B03IfMT0[Jf]05ADE03Ph>00
JFUY07inOP34a<@0B4Q80=WIf@l0h>3P0`0000070>3Ph00309JFUP3oool0i>CT0080i>CT0`0n?Sh7
0>CTi0050?ooo`2FUYH0UYJF0?ooo`3Zj^X00P3YjNT20>[ZjPP0jNWY00X0j^[Z0?ooo`2EUID0oooo
09FEU@3oool0UIFE0?ooo`2EUID0jNWY0P3Zj^X80>WYj@050>[ZjP3YjNT0jNWY0?ooo`2FUYH0;P3e
mOD00`2FUYH0oooo0>CTi0050>CTi0<0?Shn103Ti>@00`3oool0UYJF0>3Ph00=0>3Ph080oooo01D0
h>3P06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[
06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0030>3Ph00I08j>SP000000>3Ph0=WIf@3Ph>00JFUY
09jNWP3Ph>00SXj>05ADE03Ph>00JFUY09jNWP3Ph>00SXj>05ADE03Ph>00[Jf]05ADE03Ph>00JFUY
07inOP3Ph>00JFUY09jNWP0@0>3Ph0030000003Ph>00h>3P00H0h>3P00<0UYJF0?ooo`3]kNd00`3]
kNd303hn?PH0kNg]00D0oooo09JFUP2FUYH0oooo0>[ZjP020>WYj@80j^[Z203YjNT02P3Zj^X0oooo
09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@3YjNT20>[ZjPP0jNWY00D0j^[Z0>WYj@3YjNT0oooo
09JFUP0^0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]k@030?ooo`2FUYH0h>3P00`0h>3P
0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P01T0fMWI059BDP3IfMT0
h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0
000000d=3@3Bd]80h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@303hn?PD0
m?Cd00@0oooo09JFUP2FUYH0oooo3P3]kNd01`3oool0kNg]0?ooo`3]kNd0oooo0>g]k@3oool03`3]
kNd00`3oool0UYJF0?7al@0]0?7al@0309JFUP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2F
UYH0h>3P00`0h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF
0?ooo`3lo?`01@3lo?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?ooob@0kNg]00<0
oooo09JFUP3`l?00;@3`l?000`2FUYH0oooo0?clo0040?clo00303hn?P3lo?`0o?cl00D0o?cl00<0
oooo09JFUP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0
oooo00d0oooo0P2FUYH00`3oool0moOg0?Ogm`0R0?Ogm`030?ooo`2FUYH0kno_02d0kno_00<0UYJF
0?ooo`3oool03@3oool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P
03D0h>3P00<0UYJF0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XT0?ooo`030?[jnP2FUYH0
kno_02d0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3P
h0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e]KD01@2d];@0Shn?0=[JfP3Fe]H0
/;2`02@0^;Rh00<0ZJVY0<k>cP3^k^h0;@3^k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0
f][J0140h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFVG09jNWP030?ooo`3Ph>00h>3P03D0h>3P4@3/k>`X0>_[jbh0inOW4@3/k>`A0>3P
h00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooom80?ooo`030>c/k03^k^h0SHf=00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`3/k>`0
UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooom80?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`3/k>`0
UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0k>c/09JFUP3oool01@3>c/h00`0n?Sh0
c/k>0<k>cP040<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo000004P0000000<0k>c/09JFUP3oool0103>c/h303hn?PD0
c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0ooooB03oool00`3/k>`0UYJF0?ooo`030<o?c`D0?Shn103?cll05P3oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooom80?ooo`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh30=CD
e00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooodP0oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn0P3Kfm/0
5P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooom80?ooo`030>c/k02FUYH0oooo0080i>CT00<0?Shn0>CTi03Ti>@00`3Ti>@0
6P0n?Sh0i>CT0>CTi03Ti>@0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`3/k>`0UYJF0?ooo`0<0>g]k@0F0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooodP0oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh0?ooo`D0EEEE
0P3oool505EEE@@0oooo00<0k>c/09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooll0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02F
UYH0emOG00`0oooo01H0emOG09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00@0k>c/0=[JfP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU;0?ooo`0405AD
E0000000000008n?S`80oooo00D0moOg01dM7@0000002@T90=;BdP020?ooo`040::RXP0000002@T9
0=;BdP80oooo00/0XZ:R000000092@T0d];B0?ooo`3gmoL07AdM000000092@T0d];B0<K6aP030000
000300l?3`3_knl0a/K600<0000000@03`l?0>o_k`3oool0E5AD0P0000002P2?Shl0oooo0?ooo`3o
ool0XZ:R000000092@T0d];B0?ooo`1hN7P20000000408n?S`3oool0oooo05ADE080000000@0Shn?
0?ooo`3oool0XZ:R0P0000002P2?Shl0oooo0?ooo`3oool0XZ:R000000092@T0d];B0?ooo`1DE5@2
0000000408n?S`3oool0oooo0::RXP80000000@0H61P0?ooo`3oool0XZ:R0P000000102?Shl0oooo
0?ooo`2RXZ8200000004061PH03oool0oooo07QhN080000000<0Shn?0?ooo`36a/H00`000000100?
3`l0kno_0?ooo`2RXZ820000000308n?S`3oool0oooo07P0oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFU>0?ooo`0806=SH`3_knl0oooo07QhN02d];@0oooo
0?Ogm`37alL30?ooo`030<K6aP2?Shl0oooo00<0oooo00d0a/K608n?S`3oool0oooo07QhN02d];@0
oooo0?Ogm`37alL0oooo05ADE00?3`l0kno_00<0oooo00<0E5AD00l?3`3_knl01P3oool00`1SHf<0
kno_0?ooo`020?ooo`050<K6aP2?Shl0oooo0?Ogm`1mOGd00P3oool00`23Ph<0oooo0?ooo`030?oo
o`0406=SH`3_knl0gmoO05=CD`80oooo00<0GEeM0=;BdP3oool00P3oool00`36a/H0Shn?0?ooo`04
0?ooo`0:06=SH`3_knl0gmoO05=CD`3oool0moOg02HV9P3Bd]80oooo08>3P`80oooo00X0Hf=S0>o_
k`3Ogml0De=C0?ooo`3gmoL09RHV0=;BdP3gmoL0OGem0P3oool01P23Ph<0oooo0?ooo`1DE5@03`l?
0>o_k`<0oooo00D0Ph>30?ooo`3oool0Hf=S0>o_k`1i0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYB`3oool03@36a/H00000000000000000];Bd0?Ogm`0M
7Ad0Shn?0?ooo`2RXZ80];Bd0?Ogm`0M7Ad00P00000010092@T0d];B0?Ogm`0M7Ad20000000800T9
2@3Bd]80moOg01dM7@2?Shl0oooo0::RXP2d];@20?ooo`030<K6aP000000];Bd00<0oooo00<0a/K6
0000002d];@00P3oool01`36a/H00000000000000000];Bd0?Ogm`0M7Ad00P0000001`092@T0d];B
0?ooo`23Ph<0oooo0?Ogm`1mOGd00P3oool01@36a/H00000000000000000];Bd00@0oooo00@0PH61
0=;BdP3gmoL07AdM0P00000010092@T0d];B0?ooo`36a/H3000000090;Bd]03oool0oooo07QhN000
0000];Bd0?ooo`3gmoL0GUiN0080oooo00L0PH610=;BdP3oool0oooo07QhN0000000];Bd0080oooo
00@0Ph>30?ooo`3gmoL0OGem0`3oool01P36a/H000000;Bd]03oool0moOg05iNGP80oooo00<0PH61
0=;BdP3oool0N03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0
inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJDX0oooo00/0moOg02`/;03_knl0oooo0::RXP2d];@0oooo06=SH`0o?cl000000451@@020?oo
o`0407QhN02d];@0a/K608n?S`80oooo00@0N7Qh0;Bd]036a/H0Shn?0P3oool0101SHf<0?clo0000
0011@D440?ooo`030?Ogm`37alL0oooo00<0oooo00h0moOg0<O7a`3oool0moOg02`/;03_knl0oooo
0::RXP2d];@0oooo07QhN02d];@0a/K608n?S`80oooo00d0N7Qh000000000000Shn?0?ooo`3gmoL0
;2`/0>o_k`3oool0XZ:R0;Bd]03oool0N7Qh0080000000h0H61P0?ooo`3oool0N7Qh0;Bd]036a/H0
Shn?0?ooo`3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]0<0oooo00D0moOg05iNGP3oool0moOg05iNGP02
0?ooo`030861P@3Bd]80oooo0080oooo00D0moOg05iNGP3oool0oooo07QhN0020000000308n?S`3o
ool0oooo0080oooo00D0moOg0<O7a`3oool0moOg05iNGP020?ooo`030861P@3Bd]80oooo07P0oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`
l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU:0?ooo`090?Og
m`0/;2`0kno_0?ooo`1SHf<0kno_0?ooo`36a/H0@D5100@0oooo00@0moOg02HV9P2GUiL0Shn?0P3o
ool0103gmoL09RHV09NGU`2?Shl20?ooo`030<K6aP11@D40oooo0080oooo02L0gmoO05=CD`3oool0
moOg02`/;03_knl0gmoO05=CD`3oool0moOg02`/;03_knl0moOg02`/;03_knl0oooo06=SH`3_knl0
oooo0?Ogm`0V9RH0UiNG08n?S`3oool0moOg05iNGP3oool0moOg05iNGP3oool0moOg02`/;03_knl0
oooo06=SH`3_knl0oooo0861P@3Bd]80103oool0;`3gmoL09RHV09NGU`2?Shl0oooo0?Ogm`0/;2`0
kno_0?ooo`1SHf<0kno_0?Ogm`0V9RH0d];B0?Ogm`0/;2`0kno_0?ooo`1MGEd0d];B0?Ogm`0/;2`0
kno_0?Ogm`0V9RH0d];B0?Ogm`0/;2`0kno_0?Ogm`1NGUh0oooo0?Ogm`1NGUh0oooo0=oOg`1CDe<0
oooo0?Ogm`0/;2`0kno_0?ooo`1MGEd0d];B0?Ogm`0/;2`0kno_07T0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU;0?ooo`040<K6aP000000000008n?S`<0
oooo00@0XZ:R000000000000];Bd0P3oool00`3Ogml04Q8B08n?S`030?ooo`030=oOg`0B4Q80Shn?
00<0oooo00H0XZ:R000000000000];Bd0?ooo`2RXZ820000000408n?S`3oool0oooo0::RXP800000
00@0Shn?0?ooo`3oool0a/K60P0000001`2?Shl0oooo0?ooo`3oool0gmoO018B4P2?Shl00P3oool0
102RXZ8000000000002d];@20?ooo`040<K6aP000000000008n?S`80oooo00@0N7Qh000000000000
H61P0`3oool00`3Ogml04Q8B08n?S`020?ooo`040<K6aP000000000008n?S`80oooo00@0gmoO018B
4P000000];Bd0P3oool0103Ogml04Q8B0000002d];@20?ooo`040=oOg`0B4Q8000000;Bd]080oooo
00@0XZ:R000000000000];Bd0P3oool0102RXZ8000000000002?Shl20?ooo`040=oOg`0B4Q800000
0;Bd]7X0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU;
0?ooo`0405ADE0000000000008n?S`<0oooo00H0XZ:R000000092@T0d];B0?ooo`2RXZ8200000004
08n?S`3oool0oooo07QhN080000000@0Shn?0?ooo`3gmoL07AdM0P0000000`092@T0d];B0<K6aP03
0000000400l?3`3_knl0oooo07QhN080000000X0Shn?0?ooo`3oool0oooo0::RXP0000002@T90=;B
dP3oool0N7Qh0P000000102?Shl0oooo0?Ogm`0M7Ad20000000400T92@3Bd]80moOg01dM7@800000
00@02@T90=;BdP3gmoL07AdM0P0000001P092@T0d];B0?ooo`3oool0XZ:R0;Bd]0@0oooo00H0XZ:R
000000092@T0d];B0?ooo`2RXZ820000000408n?S`3oool0oooo0::RXP80000000<0Shn?0?ooo`36
a/H00`000000100?3`l0kno_0?ooo`1hN7P20000000:08n?S`3oool0oooo0?ooo`2RXZ80000000T9
2@3Bd]80moOg01dM7@80000000@02@T90=;BdP3oool0XZ:R0P0000000`2?Shl0oooo0<K6aP030000
000600l?3`3_knl0oooo0?ooo`2RXZ80];Bd0`3oool01`2RXZ8000000000002?Shl0oooo0?Ogm`0M
7Ad00P00000010092@T0d];B0?ooo`1DE5@20000000408n?S`3oool0oooo0::RXP80000000@0H61P
0?ooo`3oool0N7Qh0P000000102?Shl0oooo0?ooo`2RXZ820000000308n?S`3oool0a/K600<00000
00@03`l?0>o_k`3gmoL07AdM0P00000010092@T0d];B0?Ogm`0M7Ad20000000400T92@3Bd]80oooo
0::RXP80000000@0Shn?0?ooo`3oool0XZ:R0P000000102?Shl0oooo0?ooo`2RXZ820000000408n?
S`3oool0oooo0::RXP80000000<0Shn?0?ooo`3oool00P3oool00`2RXZ80];Bd0?ooo`0C0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYCP3oool00`1SHf<0
kno_0?ooo`020?ooo`050<K6aP2?Shl0oooo0?ooo`23Ph<00P3oool0101SHf<0kno_0?Ogm`1mOGd2
0?ooo`0608>3P`3oool0oooo0?ooo`36a/H0Shn?0`3oool00`1DE5@03`l?0>o_k`020?ooo`050?Og
m`1mOGd0oooo0?ooo`23Ph<0103oool01@36a/H0Shn?0?ooo`3gmoL0OGem0080oooo00H0Ph>30?oo
o`3oool0oooo0<K6aP2?Shl40?ooo`030<K6aP2?Shl0oooo00<0oooo00<0a/K608n?S`3oool00`3o
ool00`3gmoL0OGem0?ooo`040?ooo`050<K6aP2?Shl0oooo0=oOg`1CDe<00P3oool0101MGEd0d];B
0?ooo`23Ph<20?ooo`0606=SH`3_knl0oooo05ADE00?3`l0kno_0P3oool01@3gmoL0OGem0?ooo`3o
ool0Ph>300@0oooo00<0a/K608n?S`3oool00P3oool00`36a/H0Shn?0?ooo`020?ooo`0908>3P`3o
ool0oooo06=SH`3_knl0oooo05ADE00?3`l0kno_00@0oooo00<0moOg07emO@3oool00P3oool01@23
Ph<0oooo0?ooo`1SHf<0kno_0080oooo00<0a/K608n?S`3oool01@3oool02P1SHf<0kno_0=oOg`1C
De<0oooo0?Ogm`0V9RH0d];B0?Ogm`1mOGd20?ooo`0408>3P`3oool0gmoO05=CD`80oooo00H0GEeM
0=;BdP3oool0E5AD00l?3`3_knl40?ooo`030<K6aP2?Shl0oooo00<0oooo00<0a/K608n?S`3oool0
0P3oool01`23Ph<0oooo0?ooo`1SHf<0kno_0=oOg`1CDe<00P3oool0101MGEd0d];B0=oOg`1CDe<2
0?ooo`0405eMG@3Bd]80gmoO05=CD`80oooo00<0GEeM0=;BdP3oool0603oool00`1EEED0oooo0?oo
o`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD/0oooo00L0a/K60000000000000000
0;Bd]03gmoL07AdM0080000000@02@T90=;BdP3gmoL0GUiN0P3oool01`21PH40d];B0?ooo`23Ph<0
oooo0?Ogm`1mOGd00`3oool00`36a/H0Shn?0?ooo`030?ooo`030<K6aP000000];Bd0080oooo00L0
Ph>30?ooo`3gmoL0OGem0?ooo`3gmoL07AdM0080000000L02@T90=;BdP3oool0Ph>30?ooo`3gmoL0
OGem00<0oooo00<0a/K608n?S`3oool00`3oool00`36a/H0Shn?0?ooo`030?ooo`030<K6aP2?Shl0
oooo00@0oooo00D0QhN70>o_k`3oool0moOg01dM7@020000000300T92@3Bd]80oooo00<0oooo00@0
PH610=;BdP3gmoL0GUiN0P3oool01`21PH40d];B0?ooo`3oool0a/K60000002d];@00P3oool01`23
Ph<0oooo0?Ogm`1mOGd0oooo0?Ogm`0M7Ad00P0000001P092@T0d];B0?ooo`3oool0a/K608n?S`80
oooo00H0moOg05iNGP3oool0oooo0861P@3Bd]820?ooo`030<K6aP000000];Bd00@0oooo00D0QhN7
0>o_k`3oool0moOg05iNGP020?ooo`060861P@3Bd]80oooo0?ooo`36a/H0Shn?0`3oool01@36a/H0
0000000000000000];Bd0080oooo00<0N7Qh0000002d];@00P3oool01023Ph<0oooo0?Ogm`1mOGd5
0?ooo`070861P@3Bd]80oooo0?ooo`36a/H000000;Bd]0030?ooo`030<K6aP2?Shl0oooo00<0oooo
00H0a/K608n?S`3oool0oooo0?Ogm`1NGUh20?ooo`030861P@3Bd]80oooo00<0oooo00<0PH610=;B
dP3oool00`3oool00`21PH40d];B0?ooo`030?ooo`060861P@3Bd]80oooo0?ooo`36a/H0];Bd5@3o
ool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_
0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJDX0oooo00h0
moOg02`/;03_knl0oooo0::RXP2d];@0oooo07QhN02d];@0a/K608n?S`3oool0moOg05iNGP80oooo
00@0PH610=;BdP3oool0N7Qh0P0000001P2?Shl0oooo0?ooo`3oool0a/K608n?S`D0oooo00D0moOg
0<O7a`3oool0oooo07QhN0020000000708n?S`3oool0oooo07QhN02d];@0a/K608n?S`020?ooo`04
07QhN0000000000008n?S`<0oooo00<0a/K608n?S`3oool00`3oool00`36a/H0Shn?0?ooo`030?oo
o`030<K6aP2?Shl0oooo00@0oooo00P0a/K608n?S`3oool0oooo07QhN02d];@0a/K608n?S`80oooo
00L0N7Qh000000000000H61P0?ooo`3gmoL0GUiN0080oooo00<0PH610=;BdP3oool00P3oool01@3g
moL0alO70?ooo`3oool0N7Qh0080000000L0Shn?0?ooo`3oool0N7Qh0;Bd]036a/H0Shn?00<0oooo
00H0a/K608n?S`3oool0oooo0?Ogm`1NGUh20?ooo`030861P@3Bd]80oooo0080oooo00<0moOg0<O7
a`3oool00`3oool01@36a/H0Shn?0?ooo`3gmoL0GUiN0080oooo00H0PH610=;BdP3oool0oooo0<K6
aP2?Shl20?ooo`060?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd0`3oool01@3gmoL0GUiN0?ooo`3oool0
N7Qh0080000000@0Shn?0?ooo`3oool0N7Qh0P0000000`1PH600oooo0?ooo`020?ooo`030?Ogm`37
alL0oooo0080oooo00<0a/K608n?S`3oool00`3oool01P36a/H0Shn?0?ooo`3oool0moOg05iNGP80
oooo00@0PH610=;BdP3oool0N7Qh0P000000101PH600oooo0?ooo`1hN7P200000004061PH03oool0
oooo07QhN080000000<0H61P0?ooo`3oool0603oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJDX0oooo00/0moOg02`/;03_knl0oooo06=SH`3_knl0oooo0?Og
m`0V9RH0UiNG08n?S`020?ooo`0:05eMG@3Bd]80moOg02`/;03_knl0moOg05iNGP3oool0moOg05iN
GP80oooo00<0Hf=S0;Jf]P2?Shl00P3oool02`3Ogml0De=C0?ooo`3gmoL0;2`/0>o_k`3gmoL0GUiN
0?ooo`3gmoL0GUiN0080oooo00X0moOg02HV9P2GUiL0Shn?0?ooo`3gmoL0GUiN0?ooo`3gmoL0GUiN
0P3oool00`1SHf<0][Jf08n?S`030?ooo`0306=SH`2f][H0Shn?00<0oooo00<0Hf=S0;Jf]P2?Shl0
0P3oool01@3Ogml0LW9b0?ooo`3gmoL0GUiN0080oooo00@0moOg02HV9P2GUiL0Shn?0P3oool00`21
PH40d];B0?ooo`030?ooo`0@05eMG@3Bd]80moOg02`/;03_knl0gmoO05=CD`3oool0moOg02`/;03_
knl0moOg05iNGP3oool0moOg05iNGP80oooo00@0moOg02HV9P2GUiL0Shn?0P3oool00`1SHf<0][Jf
08n?S`030?ooo`0@05eMG@3Bd]80moOg02`/;03_knl0gmoO05=CD`3oool0moOg02`/;03_knl0gmoO
079bLP3oool0moOg05iNGP80oooo00T0GEeM0=;BdP3gmoL0;2`/0>o_k`3oool0Hf=S0;Jf]P2?Shl0
0P3oool04@3gmoL0;2`/0>o_k`3oool0Hf=S0>o_k`3gmoL09RHV0=;BdP3gmoL0;2`/0>o_k`3gmoL0
GUiN0?ooo`3gmoL0GUiN0080oooo00<0PH610=;BdP3oool00P3oool02P3Ogml0De=C0?ooo`3gmoL0
;2`/0>o_k`3oool0Hf=S0;Jf]P2?Shl30?ooo`0306=SH`2f][H0Shn?00<0oooo00P0GEeM0=;BdP3g
moL0;2`/0>o_k`3oool0PH610=;BdP@0oooo00<0PH610=;BdP3oool00`3oool00`21PH40d];B0?oo
o`030?ooo`030<K6aP2?Shl0oooo01D0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?oo
o`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFU;0?ooo`040<K6aP000000000008n?S`<0oooo00<0gmoO018B4P2?Shl0
0P3oool0103Ogml04Q8B0000002d];@20?ooo`040::RXP00000000000;Bd]080oooo00<0a/K60000
002?Shl00`3oool0102RXZ8000000000002?Shl20?ooo`040::RXP00000000000;Bd]0<0oooo00<0
gmoO018B4P2?Shl00P3oool0102RXZ8000000000002d];@20?ooo`030<K6aP000000Shn?00<0oooo
00<0a/K60000002?Shl00`3oool00`36a/H0000008n?S`020?ooo`060=oOg`0B4Q8000000000000?
3`l0kno_0P3oool00`3Ogml04Q8B08n?S`020?ooo`0407QhN00000000000061PH080oooo00@0gmoO
018B4P000000];Bd0P3oool0102RXZ8000000000002?Shl20?ooo`040::RXP00000000000;Bd]0<0
oooo00<0gmoO018B4P2?Shl00P3oool00`36a/H0000008n?S`030?ooo`040=oOg`0B4Q8000000;Bd
]080oooo00L0XZ:R000000000000Shn?0?ooo`3Ogml04Q8B0080000000L03`l?0>o_k`3oool0gmoO
018B4P000000];Bd0080oooo00<0a/K60000002?Shl00`3oool01036a/H000000000002?Shl20?oo
o`040=oOg`0B4Q8000000;Bd]080oooo00@0XZ:R000000000000];Bd0P3oool0101hN7P000000000
001PH6020?ooo`040::RXP000000000008n?S`80oooo00<0a/K60000002?Shl00`3oool00`36a/H0
000008n?S`030?ooo`040=oOg`0B4Q8000000;Bd]080oooo00@0N7Qh000000000000H61P0P3oool0
101hN7P000000000001PH6020?ooo`0407QhN00000000000061PH1X0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0
103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU;0?ooo`070?Ogm`0M7Ad0000000T92@3B
d]80oooo0::RXP020000000408n?S`3oool0moOg01dM7@80000000@02@T90=;BdP3oool0XZ:R0P00
00000`1PH600oooo0<K6aP030000000400l?3`3_knl0oooo07QhN080000000<0Shn?0?ooo`36a/H0
0`000000100?3`l0kno_0?ooo`2RXZ8200000004061PH03oool0oooo0::RXP80000000h0Shn?0?oo
o`3oool0moOg01dM7@0000002@T90=;BdP3oool0moOg01dM7@0000002@T90=;BdP80oooo00@0XZ:R
000000092@T0d];B0P3oool00`2RXZ80];Bd0?ooo`020?ooo`040::RXP000000000008n?S`80oooo
00@0E5AD000000000000Shn?0P3oool0102RXZ8000000000001PH6020?ooo`0407QhN00000000000
08n?S`<0oooo00@0XZ:R000000092@T0d];B0P3oool0302RXZ80000000T92@3Bd]80oooo0?Ogm`0M
7Ad0000000T92@3Bd]80oooo0::RXP80000000@0Shn?0?ooo`3oool0E5AD0P000000102?Shl0oooo
0?ooo`2RXZ820000000408n?S`3oool0oooo0::RXP80000000@0Shn?0?ooo`3oool0XZ:R0P000000
102?Shl0oooo0?ooo`2RXZ820000000408n?S`3oool0oooo07QhN080000000<0Shn?0?ooo`36a/H0
0`0000000`0?3`l0kno_0<K6aP030000000400l?3`3_knl0oooo0::RXP80000000@0H61P0?ooo`3g
moL07AdM0P0000001P092@T0d];B0?ooo`3oool0XZ:R0;Bd]080oooo00P0a/K60000000000000000
00l?3`3_knl0oooo0::RXP80000000@0Shn?0?ooo`3oool0XZ:R0P000000101PH600oooo0?ooo`2R
XZ820000000308n?S`3oool0oooo0080oooo00<0XZ:R0;Bd]03oool04`3oool00`1EEED0oooo0?oo
o`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD/0oooo00L0N7Qh0;Bd]03oool0moOg
0<O7a`3Ogml0De=C0080oooo00H0GEeM0=;BdP3oool0oooo0<K6aP2?Shl20?ooo`0:0=oOg`1CDe<0
oooo0?Ogm`0V9RH0d];B0?ooo`1DE5@03`l?0>o_k`80oooo00D0moOg07emO@3oool0oooo08>3P`02
0?ooo`0305ADE00?3`l0kno_0080oooo00P0gmoO05=CD`3oool0moOg02HV9P3Bd]80oooo08>3P`80
oooo00h0Hf=S0>o_k`3oool0N7Qh0;Bd]03oool0moOg0<O7a`3oool0N7Qh0;Bd]03oool0moOg0<O7
a`<0oooo00<0a/K608n?S`3oool00P3oool00`3gmoL0OGem0?ooo`020?ooo`0508>3P`3oool0oooo
06=SH`3_knl0103oool02P1SHf<0kno_0=oOg`1CDe<0oooo0?Ogm`0V9RH0d];B0?Ogm`1mOGd20?oo
o`0308>3P`3oool0oooo0080oooo00<0a/K608n?S`3oool00`3oool02`36a/H0Shn?0?ooo`3oool0
N7Qh0;Bd]03oool0moOg0<O7a`3oool0Ph>30080oooo00<0Hf=S0>o_k`3oool00`3oool0101SHf<0
kno_0=oOg`1CDe<20?ooo`0405eMG@3Bd]80gmoO05=CD`80oooo00@0GEeM0=;BdP3oool0Ph>30P3o
ool0101SHf<0kno_0=oOg`1CDe<20?ooo`0405eMG@3Bd]80moOg07emO@80oooo00H0Ph>30?ooo`3o
ool0E5AD00l?3`3_knl30?ooo`0305ADE00?3`l0kno_0080oooo00H0gmoO05=CD`3oool0moOg02HV
9P3Bd]820?ooo`030<K6aP2?Shl0oooo00<0oooo00<0moOg07emO@3oool00P3oool00`1DE5@03`l?
0>o_k`020?ooo`0>0=oOg`1CDe<0oooo0?ooo`1MGEd0d];B0=oOg`1CDe<0oooo0?Ogm`0V9RH0d];B
0=oOg`1CDe<20?ooo`0305eMG@3Bd]80oooo01P0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFU:0?ooo`060?Ogm`0M7Ad0Shn?0?ooo`2RXZ80];Bd103oool0
1P21PH40d];B0?ooo`3oool0a/K608n?S`@0oooo00<0N7Qh0000002d];@00`3oool00`36a/H00000
0;Bd]0020?ooo`0408>3P`3oool0moOg07emO@<0oooo00<0a/K60000002d];@00`3oool01P1hN7P0
00000;Bd]03oool0moOg05iNGP80oooo0100PH610=;BdP3gmoL07AdM08n?S`3oool0XZ:R0;Bd]03g
moL07AdM08n?S`3oool0XZ:R0;Bd]03gmoL07AdM0P0000000`092@T0d];B0?ooo`020?ooo`0508N7
Q`3_knl0oooo0?Ogm`1NGUh00P3oool01021PH40d];B0?ooo`36a/H3000000060;Bd]03oool0oooo
07QhN0000000];Bd0P3oool01`23Ph<0oooo0?Ogm`1mOGd0oooo0?Ogm`0M7Ad00P00000010092@T0
d];B0?Ogm`0M7Ad20000000:00T92@3Bd]80moOg01dM7@2?Shl0oooo0::RXP2d];@0moOg05iNGP80
oooo00@0PH610=;BdP3oool0a/K60`0000000`2d];@0oooo0?ooo`020?ooo`030861P@3Bd]80oooo
00<0oooo00@0PH610=;BdP3gmoL0GUiN0P3oool00`21PH40d];B0?ooo`030?ooo`070861P@3Bd]80
oooo08>3P`3oool0moOg07emO@030?ooo`030<K6aP000000];Bd00<0oooo00<0a/K60000002d];@0
0`3oool00`1hN7P000000;Bd]0030?ooo`030<K6aP2?Shl0oooo00@0oooo00<0QhN70>o_k`3oool0
0P3oool00`36a/H000000;Bd]0050?ooo`070861P@3Bd]80oooo0?ooo`1hN7P000000;Bd]0050?oo
o`060861P@3Bd]80oooo0?ooo`36a/H0];Bd5@3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJD/0oooo00@0Hf=S03lo?`000000@D510P3oool0101hN7P00000
0000001PH6030?ooo`030<K6aP2?Shl0oooo00@0oooo00<0moOg05iNGP3oool00`3oool01@3gmoL0
alO70?ooo`3oool0N7Qh0080000000<0Shn?0?ooo`3oool00P3oool00`3gmoL0alO70?ooo`030?oo
o`050?Ogm`1NGUh0oooo0?Ogm`1NGUh00P3oool01`21PH40d];B0?ooo`1SHf<0?clo00000011@D40
0P3oool0101SHf<0?clo00000011@D420?ooo`0407QhN02d];@0a/K608n?S`@0oooo00D0a/K608n?
S`3oool0moOg05iNGP020?ooo`080861P@3Bd]80moOg02`/;03_knl0oooo0::RXP2d];@30?ooo`05
0?Ogm`1NGUh0oooo0?ooo`1hN7P00P0000001`2?Shl0oooo0?ooo`1hN7P0];Bd0<K6aP2?Shl00P3o
ool0101hN7P0];Bd0<K6aP2?Shl20?ooo`0706=SH`0o?cl000000451@@3oool0moOg05iNGP020?oo
o`0:0861P@3Bd]80moOg02`/;03_knl0oooo0::RXP2d];@0oooo07QhN080000000@0H61P0?ooo`3o
ool0N7Qh0P000000101PH600oooo0?Ogm`1NGUh20?ooo`040861P@3Bd]80oooo07QhN080000000@0
H61P0?ooo`3oool0N7Qh0P0000000`2?Shl0oooo0?ooo`020?ooo`030?Ogm`37alL0oooo00<0oooo
00<0moOg0<O7a`3oool00`3oool00`3gmoL0GUiN0?ooo`020?ooo`030<K6aP2?Shl0oooo00@0oooo
00<0a/K608n?S`3oool00`3oool01@3gmoL0alO70?ooo`3oool0N7Qh0080000000<0H61P0?ooo`3o
ool00P3oool01@3gmoL0GUiN0?ooo`3oool0N7Qh0080000000<0H61P0?ooo`3oool0603oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD/0oooo00<0a/K60451
@@3oool00`3oool00`21PH40d];B0?ooo`030?ooo`0306=SH`2f][H0Shn?0080oooo02L0moOg02HV
9P3Bd]80moOg02`/;03_knl0gmoO05=CD`3oool0moOg02`/;03_knl0moOg05iNGP3oool0moOg05iN
GP3oool0gmoO05=CD`3oool0moOg02`/;03_knl0moOg02HV9P3Bd]80moOg02`/;03_knl0oooo05eM
G@3Bd]80moOg02`/;03_knl0oooo0<K6aP11@D40103oool00`36a/H0@D510?ooo`030?ooo`0:0?Og
m`0V9RH0UiNG08n?S`3oool0gmoO079bLP3oool0moOg05iNGP80oooo01H0GEeM0=;BdP3gmoL0;2`/
0>o_k`3gmoL0;2`/0>o_k`3oool0Hf=S0>o_k`3gmoL09RHV0=;BdP3gmoL0;2`/0>o_k`3gmoL0GUiN
0?ooo`3gmoL0GUiN0P3oool0103gmoL09RHV09NGU`2?Shl20?ooo`040?Ogm`0V9RH0UiNG08n?S`80
oooo00<0a/K60451@@3oool00`3oool03P1MGEd0d];B0?Ogm`0/;2`0kno_0?Ogm`0/;2`0kno_0?oo
o`1SHf<0kno_0?ooo`21PH40d];B103oool00`21PH40d];B0?ooo`030?ooo`0805eMG@3Bd]80moOg
02`/;03_knl0oooo0861P@3Bd]830?ooo`0L0?Ogm`1NGUh0oooo0?Ogm`1NGUh0oooo0=oOg`1CDe<0
oooo0?Ogm`0/;2`0kno_0=oOg`1CDe<0oooo0?Ogm`0/;2`0kno_0?Ogm`0V9RH0d];B0?Ogm`0/;2`0
kno_0?ooo`1SHf<0][Jf08n?S`80oooo00l0gmoO079bLP3oool0moOg05iNGP3oool0gmoO05=CD`3o
ool0moOg02`/;03_knl0oooo0861P@3Bd]800`3oool02@3gmoL09RHV0=;BdP3gmoL0;2`/0>o_k`3o
ool0PH610=;BdP040?ooo`030<K6aP2?Shl0oooo01D0oooo00<0EEEE0?ooo`3oool0103oool00`1E
EED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFU<0?ooo`060::RXP00000000000;Bd]03oool0N7Qh0P00
00001P1PH600oooo0?ooo`36a/H0000008n?S`<0oooo00@0gmoO018B4P000000];Bd0P3oool0102R
XZ8000000000002?Shl20?ooo`040::RXP00000000000;Bd]080oooo00@0XZ:R000000000000Shn?
0P3oool0103Ogml04Q8B0000002d];@20?ooo`040=oOg`0B4Q8000000;Bd]0<0oooo00@0XZ:R0000
00000000];Bd0P3oool0102RXZ8000000000002d];@20?ooo`060=oOg`0B4Q80Shn?0?ooo`3Ogml0
4Q8B0P0000001`0?3`l0kno_0?ooo`3Ogml04Q8B0000002d];@00P3oool01036a/H000000000002?
Shl20?ooo`040=oOg`0B4Q8000000;Bd]080oooo00@0XZ:R000000000000];Bd0`3oool00`3Ogml0
4Q8B08n?S`030?ooo`030=oOg`0B4Q80Shn?00<0oooo00T0XZ:R000000000000];Bd0?ooo`3Ogml0
4Q8B0000002d];@00P3oool01036a/H000000000002?Shl20?ooo`0407QhN00000000000061PH080
oooo00@0N7Qh000000000000H61P0P3oool0103Ogml04Q8B0000002d];@20?ooo`0407QhN0000000
0000061PH080oooo00@0XZ:R000000000000];Bd0P3oool0102RXZ8000000000002?Shl20?ooo`04
0::RXP000000000008n?S`80oooo00@0gmoO018B4P000000];Bd0P3oool00`36a/H0000008n?S`02
0?ooo`080=oOg`0B4Q8000000000000?3`l0kno_0?ooo`2RXZ820000000408n?S`3oool0oooo07Qh
N080000000L0H61P0?ooo`3oool0gmoO018B4P000000];Bd0080oooo00@0N7Qh000000000000H61P
6P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
?03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD/0oooo
00H0moOg01dM7@0000002@T90=;BdP36a/H30000000400l?3`3_knl0oooo07QhN080000000T0Shn?
0?ooo`3oool0moOg01dM7@0000002@T90=;BdP36a/H00`000000100?3`l0kno_0?ooo`2RXZ820000
000408n?S`3oool0oooo07QhN080000000@0Shn?0?ooo`3oool0E5AD0P000000102?Shl0oooo0?oo
o`1DE5@20000000408n?S`3oool0oooo07QhN080000000T0Shn?0?ooo`3oool0moOg01dM7@000000
2@T90=;BdP36a/H00`000000100?3`l0kno_0?ooo`1hN7P20000000408n?S`3oool0oooo0::RXP80
000000@0Shn?0?ooo`3oool0XZ:R0P0000002P1PH600oooo0?ooo`3oool0XZ:R000000092@T0d];B
0?ooo`1hN7P20000000308n?S`3oool0a/K600<0000000@03`l?0>o_k`3oool0XZ:R0P000000102?
Shl0oooo0?ooo`2RXZ8200000009061PH03oool0oooo0?ooo`2RXZ80000000T92@3Bd]80a/K600<0
000000@03`l?0>o_k`3gmoL07AdM0P00000010092@T0d];B0?Ogm`0M7Ad20000000600T92@3Bd]80
oooo0?ooo`2RXZ80];Bd0`3oool0102RXZ8000000000002?Shl20?ooo`050?Ogm`0M7Ad0000000T9
2@3Bd]800P3oool00`2RXZ80];Bd0?ooo`020?ooo`0405ADE0000000000008n?S`80oooo00H0N7Qh
000000000000Shn?0?ooo`36a/H30000000400l?3`3_knl0moOg01dM7@80000000X02@T90=;BdP3o
ool0oooo0::RXP0000002@T90=;BdP3oool0N7Qh0P000000102?Shl0oooo0?ooo`2RXZ8200000004
08n?S`3oool0oooo07QhN080000000<0Shn?0?ooo`3oool00P3oool00`2RXZ80];Bd0?ooo`0C0?oo
o`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYB`3oool02@1h
N7P0];Bd0?ooo`3gmoL0alO70?ooo`1DE5@03`l?0>o_k`020?ooo`050?Ogm`1mOGd0oooo0?ooo`23
Ph<00P3oool02@1hN7P0];Bd0?ooo`3gmoL0alO70?ooo`1DE5@03`l?0>o_k`030?ooo`0708>3P`3o
ool0oooo06=SH`3_knl0moOg07emO@020?ooo`0308>3P`3oool0oooo00<0oooo00<0Hf=S0>o_k`3o
ool00`3oool0101SHf<0kno_0?Ogm`1mOGd20?ooo`0<08>3P`3oool0oooo07QhN02d];@0oooo0?Og
m`37alL0oooo05ADE00?3`l0kno_0P3oool01@3gmoL0OGem0?ooo`3oool0Ph>30080oooo00/0Ph>3
0?ooo`3oool0Hf=S0>o_k`3Ogml0De=C0?ooo`3gmoL09RHV0=;BdP030?ooo`050<K6aP2?Shl0oooo
0?Ogm`1mOGd00P3oool01P23Ph<0oooo0?ooo`1DE5@03`l?0>o_k`80oooo00`0gmoO05=CD`3oool0
oooo05eMG@3Bd]80gmoO05=CD`3oool0moOg02HV9P3Bd]830?ooo`070<K6aP2?Shl0oooo0?ooo`1D
E5@03`l?0>o_k`040?ooo`030<K6aP2?Shl0oooo00<0oooo00<0a/K608n?S`3oool00`3oool00`3g
moL0OGem0?ooo`020?ooo`0;08>3P`3oool0oooo06=SH`3_knl0oooo07QhN02d];@0oooo0?Ogm`37
alL00P3oool00`3gmoL0OGem0?ooo`050?ooo`0406=SH`3_knl0moOg07emO@80oooo00H0Ph>30?oo
o`3oool0E5AD00l?3`3_knl40?ooo`030<K6aP2?Shl0oooo00@0oooo00D0a/K608n?S`3oool0moOg
07emO@020?ooo`0408>3P`3oool0oooo08>3P`80oooo00@0Hf=S0>o_k`3gmoL0OGem0P3oool00`23
Ph<0oooo0?ooo`0H0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYBP3oool01P3gmoL07AdM08n?S`3oool0XZ:R0;Bd]080oooo00<0a/K60000002d];@00P3o
ool02`23Ph<0oooo0?Ogm`1mOGd0oooo0?Ogm`0M7Ad0Shn?0?ooo`2RXZ80];Bd0080oooo00H0a/K6
0000002d];@0oooo0?Ogm`1NGUh20?ooo`070861P@3Bd]80oooo08>3P`3oool0moOg07emO@020?oo
o`070<K6aP00000000000000002d];@0oooo0<K6aP030000000=0;Bd]03oool0Ph>30?ooo`3gmoL0
OGem0?ooo`3gmoL07AdM08n?S`3oool0XZ:R0;Bd]0020?ooo`030<K6aP000000];Bd0080oooo00L0
Ph>30?ooo`3gmoL0OGem0?ooo`3gmoL0GUiN0080oooo00X0PH610=;BdP3oool0oooo07QhN0000000
];Bd0?ooo`3gmoL07AdM0P0000001`092@T0d];B0?ooo`23Ph<0oooo0?Ogm`1mOGd00`3oool00`36
a/H000000;Bd]0050?ooo`0:0861P@3Bd]80oooo0?ooo`1hN7P000000;Bd]03oool0moOg01dM7@80
000000L02@T90=;BdP3oool0oooo0<K6aP000000];Bd00<0oooo00<0a/K608n?S`3oool00`3oool0
0`36a/H0Shn?0?ooo`040?ooo`0508N7Q`3_knl0oooo0?Ogm`1NGUh00P3oool02021PH40d];B0?Og
m`0M7Ad0Shn?0?ooo`2RXZ80];Bd0`3oool01@27QhL0kno_0?ooo`3oool0a/K600<0000000H0];Bd
0?ooo`23Ph<0oooo0?Ogm`1mOGd30?ooo`030<K6aP000000];Bd00<0oooo00H0a/K608n?S`3oool0
oooo0?Ogm`0M7Ad20000000:00T92@3Bd]80oooo08>3P`3oool0moOg07emO@3oool0moOg05iNGP80
oooo00L0PH610=;BdP3oool0Ph>30?ooo`3gmoL0OGem00<0oooo00<0a/K60;Bd]03oool0503oool0
0`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`
l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD/0oooo00@0Hf=S
03lo?`000000@D51103oool01@3gmoL0alO70?ooo`3oool0N7Qh0080000000L0Shn?0?ooo`3oool0
Hf=S03lo?`000000@D5100@0oooo00D0moOg0<O7a`3oool0moOg05iNGP020?ooo`040861P@3Bd]80
oooo07QhN08000000100Shn?0?ooo`3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]03gmoL0;2`/0>o_k`3o
ool0XZ:R0;Bd]03oool0N7Qh0P0000001`2?Shl0oooo0?ooo`1SHf<0?clo00000011@D40103oool0
1@3gmoL0alO70?ooo`3oool0N7Qh0080000000@0Shn?0?ooo`3gmoL0GUiN0P3oool00`21PH40d];B
0?ooo`020?ooo`080?Ogm`1NGUh0oooo0?ooo`1hN7P0];Bd0<K6aP2?Shl20?ooo`0407QhN0000000
000008n?S`@0oooo00D0moOg0<O7a`3oool0oooo07QhN00200000003061PH03oool0oooo0080oooo
00P0moOg05iNGP3oool0oooo07QhN02d];@0a/K608n?S`@0oooo00<0moOg0<O7a`3oool00P3oool0
0`36a/H0Shn?0?ooo`030?ooo`030<K6aP2?Shl0oooo00@0oooo00D0a/K608n?S`3oool0moOg05iN
GP020?ooo`070861P@3Bd]80oooo06=SH`0o?cl000000451@@040?ooo`0;0<K6aP2?Shl0oooo0?Og
m`0/;2`0kno_0?ooo`2RXZ80];Bd0?ooo`1hN7P00P0000000`2?Shl0oooo0?ooo`020?ooo`030?Og
m`37alL0oooo0080oooo00<0a/K608n?S`3oool00P3oool0101hN7P0];Bd0<K6aP2?Shl20?ooo`07
07QhN0000000000008n?S`3oool0moOg05iNGP020?ooo`040861P@3Bd]80oooo07QhN080000000<0
Shn?0?ooo`3oool0603oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3k
no/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJD/0oooo00<0a/K60451@@3oool00P3oool02`3Ogml0De=C0?ooo`3gmoL0;2`/0>o_k`3g
moL0GUiN0?ooo`3gmoL0GUiN0080oooo00<0a/K60451@@3oool00P3oool08`3Ogml0De=C0?ooo`3g
moL0;2`/0>o_k`3oool0GEeM0=;BdP3gmoL0;2`/0>o_k`3gmoL0GUiN0?ooo`3gmoL0GUiN0?ooo`3g
moL0;2`/0>o_k`3oool0Hf=S0>o_k`3gmoL0;2`/0>o_k`3oool0Hf=S0>o_k`3gmoL0GUiN0?ooo`3g
moL0GUiN0080oooo00<0a/K60451@@3oool00P3oool02`3Ogml0De=C0?ooo`3gmoL0;2`/0>o_k`3g
moL0GUiN0?ooo`3gmoL0GUiN0080oooo0200GEeM0=;BdP3gmoL0;2`/0>o_k`3gmoL09RHV0=;BdP3g
moL0;2`/0>o_k`3oool0moOg02HV9P2GUiL0Shn?0?ooo`3gmoL0GUiN0?ooo`3gmoL0GUiN0?ooo`3O
gml0De=C0?ooo`3gmoL0;2`/0>o_k`3oool0PH610=;BdP<0oooo01H0moOg02HV9P3Bd]80moOg02`/
;03_knl0oooo0?Ogm`0V9RH0UiNG08n?S`3oool0gmoO05=CD`3oool0moOg02`/;03_knl0oooo06=S
H`2f][H0Shn?0`3oool00`1SHf<0][Jf08n?S`020?ooo`050=oOg`1bLW80oooo0?Ogm`1NGUh00P3o
ool0201MGEd0d];B0?Ogm`0/;2`0kno_0?ooo`36a/H0@D510`3oool0703Ogml0LW9b0?ooo`3gmoL0
GUiN0?ooo`3gmoL0;2`/0>o_k`3oool0Hf=S0>o_k`3gmoL0GUiN0?ooo`3gmoL0GUiN0?ooo`3Ogml0
De=C0?ooo`3gmoL0;2`/0>o_k`3oool0Hf=S0;Jf]P2?Shl30?ooo`0:0?Ogm`0V9RH0UiNG08n?S`3o
ool0moOg05iNGP3oool0moOg05iNGP80oooo00X0GEeM0=;BdP3gmoL0;2`/0>o_k`3gmoL0GUiN0?oo
o`3gmoL0GUiN0P3oool00`36a/H0Shn?0?ooo`0E0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool00`3/k>`0l?3`08j>SP050:2PX00309nOW`2PX:00X:2P00@0X:2P01H0T92@
0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJD`0oooo00H0XZ:R000000000000];Bd0?ooo`2RXZ820000000408n?S`3oool0oooo0::RXP80
000000D0];Bd0?ooo`3oool0oooo0::RXP02000000030;Bd]03oool0XZ:R0080000000L0Shn?0?oo
o`3oool0gmoO018B4P000000];Bd0080oooo00@0XZ:R000000000000];Bd0P3oool01036a/H00000
0000002?Shl20?ooo`040<K6aP000000000008n?S`80oooo00@0XZ:R000000000000];Bd0`3oool0
1P2RXZ8000000000002d];@0oooo0::RXP80000000@0Shn?0?ooo`3oool0XZ:R0P0000001`2d];@0
oooo0?ooo`3Ogml04Q8B0000002d];@00P3oool0103Ogml04Q8B0000002d];@30?ooo`030=oOg`0B
4Q80Shn?0080oooo00@0XZ:R000000000000];Bd0P3oool0102RXZ8000000000002?Shl20?ooo`04
07QhN00000000000061PH080oooo00@0gmoO018B4P000000];Bd0`3oool00`3Ogml04Q8B08n?S`02
0?ooo`040::RXP000000000008n?S`80oooo00<0a/K60000002?Shl00`3oool00`36a/H0000008n?
S`020?ooo`0;0=oOg`0B4Q8000000000000?3`l0kno_0?ooo`3Ogml04Q8B0000002d];@00`3oool0
1P2RXZ8000000000002d];@0gmoO018B4P80000000@03`l?0>o_k`3oool0a/K60P000000102?Shl0
oooo0?ooo`2RXZ82000000040;Bd]03oool0oooo0::RXP80000000H0Shn?0?ooo`3oool0a/K60000
002?Shl40?ooo`030=oOg`0B4Q80Shn?0080oooo00@0XZ:R000000000000];Bd0P3oool0103Ogml0
4Q8B0000002d];@20?ooo`040::RXP00000000000;Bd]1X0oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0g]kN00`0dm?C01H0g]kN09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool0
0`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>CTi03MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1EEED0
oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[Z
jP0M0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1EEED0oooo
0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1EEED0oooo0?oo
o`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CT
i03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1EEED0oooo0?ooo`04
0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03M
gMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1EEED0oooo0?ooo`040?oo
o`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0
gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1EEED0oooo0?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?03oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJD`0oooo00H0XZ:R000000092@T0d];B0?Ogm`0M7Ad200000004
00T92@3Bd]80oooo05ADE080000000H0Shn?0?ooo`3oool0oooo0::RXP2d];@20?ooo`0>0?Ogm`0M
7Ad00000000000092@T0d];B0?ooo`3gmoL07AdM000000092@T0d];B0?ooo`1DE5@20000000408n?
S`3oool0oooo0::RXP80000000@0H61P0?ooo`3oool0E5AD0P000000102?Shl0oooo0?ooo`1DE5@2
0000000408n?S`3oool0oooo0::RXP80000000H0H61P0?ooo`3oool0oooo0::RXP2d];@30?ooo`07
0::RXP000000000008n?S`3oool0moOg01dM7@020000000400T92@3Bd]80oooo0::RXP80000000@0
Shn?0?ooo`3oool0XZ:R0P000000102?Shl0oooo0?ooo`1hN7P20000000308n?S`3oool0a/K600<0
000000@03`l?0>o_k`3oool0XZ:R0P000000102?Shl0oooo0?ooo`1DE5@20000000608n?S`3oool0
oooo0?ooo`2RXZ80];Bd103oool01P2RXZ80000000T92@3Bd]80oooo05ADE080000000P0Shn?0?oo
o`3oool0oooo0::RXP0000002@T90=;BdP80oooo00H0XZ:R000000092@T0d];B0?ooo`2RXZ820000
000408n?S`3oool0oooo05ADE080000000<0Shn?0?ooo`36a/H00`000000100?3`l0kno_0?ooo`2R
XZ8200000004061PH03oool0oooo0::RXP80000000H0Shn?0?ooo`3oool0oooo0::RXP2d];@30?oo
o`0707QhN0000000000008n?S`3oool0moOg01dM7@020000000800T92@3Bd]80oooo0?Ogm`0M7Ad0
000000T92@3Bd]820?ooo`060::RXP0000002@T90=;BdP3oool0XZ:R0P0000000`2?Shl0oooo0?oo
o`020?ooo`030::RXP2d];@0oooo01<0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFU=0?ooo`030<K6aP2?Shl0oooo0080oooo00<0a/K608n?S`3oool01@3oool0
1P1SHf<0kno_0?ooo`3oool0moOg07emO@@0oooo00<0a/K608n?S`3oool00P3oool01@1hN7P0];Bd
0?ooo`3gmoL0alO700@0oooo00P0Hf=S0>o_k`3Ogml0De=C0?ooo`3gmoL09RHV0=;BdP@0oooo00<0
Hf=S0>o_k`3oool00`3oool0201SHf<0kno_0=oOg`1CDe<0oooo0?Ogm`0V9RH0d];B0P3oool01P3g
moL0OGem0?ooo`3oool0gmoO05=CD`80oooo00H0GEeM0=;BdP3oool0oooo0<K6aP2?Shl30?ooo`07
08>3P`3oool0oooo06=SH`3_knl0gmoO05=CD`020?ooo`0405eMG@3Bd]80moOg07emO@80oooo00H0
Ph>30?ooo`3oool0E5AD00l?3`3_knl30?ooo`0508>3P`3oool0oooo06=SH`3_knl0103oool01P1S
Hf<0kno_0?ooo`3oool0moOg07emO@D0oooo00<0a/K608n?S`3oool0103oool00`1SHf<0kno_0?oo
o`020?ooo`030<K6aP2?Shl0oooo00<0oooo00D0a/K608n?S`3oool0gmoO05=CD`020?ooo`0305eM
G@3Bd]80oooo00<0oooo00H0Hf=S0>o_k`3oool0E5AD00l?3`3_knl20?ooo`080=oOg`1CDe<0oooo
0?Ogm`0V9RH0d];B0?ooo`23Ph<20?ooo`0606=SH`3_knl0oooo0?ooo`3gmoL0OGem0P3oool01@3g
moL0OGem0?ooo`3oool0Ph>300<0oooo00<0a/K608n?S`3oool00P3oool01@1hN7P0];Bd0?ooo`3g
moL0alO700<0oooo00D0a/K608n?S`3oool0oooo08>3P`020?ooo`0306=SH`3_knl0oooo01P0oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU:0?ooo`060?Ogm`0M
7Ad00000000000092@T0d];B0P3oool00`36a/H0Shn?0?ooo`020?ooo`050<K6aP00000000000000
002d];@00`3oool00`27QhL0kno_0?ooo`020?ooo`0<0<K6aP2?Shl0oooo0?ooo`3gmoL07AdM08n?
S`3oool0XZ:R0;Bd]03oool0a/K60`0000001P2d];@0oooo0?ooo`1hN7P000000;Bd]080oooo00L0
a/K600000000000000000;Bd]03oool0a/K600<0000000H0];Bd0?ooo`3oool0N7Qh0000002d];@4
0?ooo`0308N7Q`3_knl0oooo00@0oooo00H0PH610=;BdP3oool0oooo0<K6aP2?Shl20?ooo`060?Og
m`1NGUh0oooo0?ooo`21PH40d];B103oool01`21PH40d];B0?ooo`23Ph<0oooo0?Ogm`1mOGd00`3o
ool01P36a/H000000;Bd]03oool0moOg05iNGP80oooo00@0PH610=;BdP3oool0a/K60`0000002@2d
];@0oooo0?ooo`3oool0QhN70>o_k`3oool0moOg01dM7@020000000400T92@3Bd]80oooo0<K6aP<0
000000<0];Bd0?Ogm`0M7Ad00P00000010092@T0d];B0?Ogm`0M7Ad20000000300T92@3Bd]80oooo
00<0oooo00@0PH610=;BdP3oool0a/K60`0000001P2d];@0oooo0?ooo`36a/H000000;Bd]0<0oooo
00H0N7Qh0000002d];@0oooo0?Ogm`1NGUh20?ooo`030861P@3Bd]80oooo0080oooo00P0QhN70>o_
k`3oool0oooo08>3P`3oool0moOg07emO@<0oooo00`0a/K608n?S`3oool0oooo0?Ogm`0M7Ad0Shn?
0?ooo`2RXZ80];Bd0?Ogm`0M7Ad20000000400T92@3Bd]80moOg05iNGP80oooo00H0PH610=;BdP3o
ool0oooo0<K6aP2d];@E0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYB`3oool0101hN7P0];Bd0<K6aP2?Shl30?ooo`0:0<K6aP2?Shl0oooo0?ooo`3gmoL0;2`/
0>o_k`3oool0XZ:R0;Bd]0<0oooo00<0a/K608n?S`3oool00P3oool00`36a/H0Shn?0?ooo`020?oo
o`0;06=SH`0o?cl000000451@@3oool0moOg02`/;03_knl0oooo0::RXP2d];@00`3oool03`3gmoL0
GUiN0?ooo`3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]03gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]0030?oo
o`030?Ogm`1NGUh0oooo00<0oooo00D0a/K608n?S`3oool0oooo07QhN00200000006061PH03oool0
oooo0?ooo`36a/H0Shn?0P3oool0203gmoL0GUiN0?ooo`3oool0PH610=;BdP3oool0N7Qh0P000000
101PH600oooo0?ooo`1hN7P20000000308n?S`3oool0oooo0080oooo00D0moOg0<O7a`3oool0moOg
05iNGP020?ooo`080861P@3Bd]80moOg02`/;03_knl0oooo0::RXP2d];@30?ooo`0D0<K6aP2?Shl0
oooo0?ooo`1hN7P0];Bd0<K6aP2?Shl0oooo0?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd0?ooo`1hN7P0
];Bd0<K6aP2?Shl20?ooo`0407QhN02d];@0a/K608n?S`80oooo00/0N7Qh000000000000H61P0?oo
o`3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]0030?ooo`030?Ogm`37alL0oooo00<0oooo00D0moOg05iN
GP3oool0moOg05iNGP020?ooo`030861P@3Bd]80oooo0080oooo00D0a/K608n?S`3oool0oooo07Qh
N0020000000608n?S`3oool0oooo0?ooo`36a/H0Shn?0`3oool0101SHf<0?clo00000011@D420?oo
o`0707QhN02d];@0a/K608n?S`3oool0moOg05iNGP020?ooo`030861P@3Bd]80oooo01P0oooo00<0
EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU;0?ooo`040?Ogm`0V9RH0
UiNG08n?S`80oooo00<0Hf=S0;Jf]P2?Shl00P3oool02`3gmoL0;2`/0>o_k`3oool0Hf=S0>o_k`3O
gml0LW9b0?ooo`3gmoL0GUiN0080oooo00<0Hf=S0;Jf]P2?Shl00`3oool00`36a/H0@D510?ooo`02
0?ooo`0S0?Ogm`0/;2`0kno_0?ooo`1SHf<0kno_0?Ogm`0V9RH0d];B0?Ogm`0/;2`0kno_0?Ogm`0/
;2`0kno_0?ooo`1SHf<0kno_0?Ogm`0/;2`0kno_0?ooo`1SHf<0kno_0?Ogm`0V9RH0d];B0?Ogm`0/
;2`0kno_0=oOg`1bLW80oooo0?Ogm`1NGUh00P3oool00`21PH40d];B0?ooo`030?ooo`0306=SH`2f
][H0Shn?00<0oooo00P0GEeM0=;BdP3gmoL0;2`/0>o_k`3oool0PH610=;BdP<0oooo01d0moOg05iN
GP3oool0moOg05iNGP3oool0gmoO05=CD`3oool0moOg02`/;03_knl0oooo05eMG@3Bd]80moOg02`/
;03_knl0moOg02`/;03_knl0oooo06=SH`3_knl0gmoO079bLP3oool0moOg05iNGP020?ooo`0@0?Og
m`0V9RH0UiNG08n?S`3oool0moOg02`/;03_knl0oooo06=SH`3_knl0oooo0?Ogm`0V9RH0UiNG08n?
S`80oooo00@0moOg02HV9P2GUiL0Shn?0P3oool00`21PH40d];B0?ooo`020?ooo`0S0?Ogm`0/;2`0
kno_0?ooo`1SHf<0kno_0=oOg`1CDe<0oooo0?Ogm`0/;2`0kno_0?Ogm`0V9RH0d];B0?Ogm`0/;2`0
kno_0?ooo`1MGEd0d];B0?Ogm`0/;2`0kno_0=oOg`1bLW80oooo0?Ogm`1NGUh0oooo0?Ogm`1NGUh0
oooo0?Ogm`1NGUh00P3oool00`1SHf<0][Jf08n?S`030?ooo`030<K6aP11@D40oooo00<0oooo00@0
moOg02HV9P2GUiL0Shn?0P3oool0201MGEd0d];B0?Ogm`0/;2`0kno_0?ooo`36a/H0Shn?5P3oool0
0`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD`0oooo00<0gmoO018B
4P2?Shl00P3oool00`36a/H0000008n?S`030?ooo`070<K6aP000000000008n?S`3oool0gmoO018B
4P020000000600l?3`3_knl0oooo0<K6aP000000Shn?103oool01P2RXZ8000000000002d];@0oooo
0<K6aP80000000L0Shn?0?ooo`3oool0gmoO018B4P000000];Bd0080oooo00@0a/K6000000000000
Shn?0P3oool01036a/H000000000002?Shl20?ooo`070=oOg`0B4Q8000000;Bd]03oool0gmoO018B
4P020000000400l?3`3_knl0oooo07QhN080000000H0H61P0?ooo`3oool0a/K60000002?Shl30?oo
o`040=oOg`0B4Q8000000;Bd]080oooo00@0N7Qh000000000000H61P0P3oool0102RXZ8000000000
002d];@20?ooo`040::RXP000000000008n?S`80oooo00@0gmoO018B4P000000];Bd0P3oool01`36
a/H000000000002?Shl0oooo0=oOg`0B4Q800P0000001`0?3`l0kno_0?ooo`3oool0gmoO018B4P2?
Shl00P3oool01036a/H000000000002?Shl30?ooo`030=oOg`0B4Q80Shn?00<0oooo00<0gmoO018B
4P2?Shl00P3oool0101hN7P000000000001PH6020?ooo`040<K6aP000000000008n?S`80oooo00@0
XZ:R000000000000Shn?0P3oool0103Ogml04Q8B0000002d];@20?ooo`070=oOg`0B4Q8000000;Bd
]03oool0gmoO018B4P020000000400l?3`3_knl0oooo0::RXP80000000H0];Bd0?ooo`3oool0a/K6
0000002?Shl40?ooo`040::RXP00000000000;Bd]080oooo00<0gmoO018B4P2?Shl00P3oool0103O
gml04Q8B0000002d];@J0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oooll0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3oooll0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3oooll0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
ooll0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooll
0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooll0?oo
o`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@H0UIFE01T0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?ooo`3oool0103o
ool00`1EEED0oooo0?ooo`020?ooo`050>c/k02FUYH0oooo0?Ogm`3]kNd01P3oool06P3Fe]H0d=3@
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJBh0oooo0P1EEED50?ooo`80EEEEo`3oool50?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0d=3@
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFT^0?ooo`0305EEE@3oool0oooo00D0oooo00<0EEEE0?ooo`3oool0o`3o
ool30?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?oo
o`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTP0?ooo`<0EEEE0`3oool305EE
E@030?ooo`1EEED0EEEE0080oooo00D0EEEE0?ooo`3oool0oooo05EEE@030?ooo`0305EEE@3oool0
oooo00`0oooo00@0XZ:R000000000000H61P0P3oool00`3gmoL07AdM0451@@020?ooo`060?Ogm`0M
7Ad00000000000092@T0d];B0P3oool01P2RXZ80000000T92@3Bd]80moOg01dM7@80000000@02@T9
0=;BdP3oool0XZ:R0P000000102?Shl0oooo0?ooo`1DE5@20000000308n?S`3oool0a/K600<00000
00X03`l?0>o_k`3oool0moOg01dM7@0000002@T90=;BdP3oool0XZ:R0P000000102?Shl0oooo0?oo
o`2RXZ8200000004061PH03oool0oooo0::RXP80000000@0Shn?0?ooo`3oool0N7Qh0P000000102?
Shl0oooo0?ooo`1DE5@20000000608n?S`3oool0oooo0?ooo`2RXZ80];Bd0`3oool0101DE5@00000
0000002?Shl20?ooo`060::RXP0000000000061PH03oool0a/K60`000000100?3`l0kno_0?ooo`2R
XZ8200000004061PH03oool0oooo07QhN080000000l0Shn?0?ooo`3oool0oooo0::RXP0000002@T9
0=;BdP3oool0moOg01dM7@0000002@T90=;BdP36a/H00`000000200?3`l0kno_0?ooo`3gmoL07AdM
000000092@T0d];B0P3oool01P2RXZ80000000T92@3Bd]80oooo0::RXP80000000@0H61P0?ooo`3o
ool0XZ:R0P000000101PH600oooo0?ooo`1hN7P20000000408n?S`3oool0oooo0::RXP80000000<0
H61P0?ooo`36a/H00`0000001P0?3`l0kno_0?ooo`3oool0XZ:R0;Bd]0<0oooo00@0E5AD00000000
0000Shn?0P3oool0102RXZ8000000000002?Shl20?ooo`060::RXP000000000008n?S`3oool0a/K6
0`000000100?3`l0kno_0?ooo`1hN7P20000000408n?S`3oool0oooo07QhN080000000<0Shn?0?oo
o`3oool00P3oool00`2RXZ80];Bd0?ooo`0C0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTO0?ooo`0705EEE@3oool0oooo0?ooo`1EEED0oooo05EEE@020?ooo`0305EEE@3oool0EEEE00<0
oooo00D0EEEE0?ooo`3oool0oooo05EEE@030?ooo`0305EEE@3oool0oooo00/0oooo00H0gmoO05=C
D`3oool0moOg02HV9P3Bd]880?ooo`030<K6aP2?Shl0oooo00@0oooo00<0a/K608n?S`3oool00P3o
ool01P36a/H0Shn?0?ooo`3oool0gmoO05=CD`80oooo00<0GEeM0=;BdP3oool00`3oool01P1SHf<0
kno_0?ooo`1DE5@03`l?0>o_k`<0oooo00L0N7Qh0;Bd]03oool0moOg0<O7a`3Ogml0De=C0080oooo
00X0GEeM0=;BdP3Ogml0De=C0?ooo`3gmoL09RHV0=;BdP3Ogml0De=C0P3oool0101MGEd0d];B0?Og
m`1mOGd20?ooo`0308>3P`3oool0oooo00<0oooo00H0Hf=S0>o_k`3oool0oooo0?Ogm`1mOGd60?oo
o`0<06=SH`3_knl0gmoO05=CD`3oool0moOg02HV9P3Bd]80oooo05ADE00?3`l0kno_0P3oool0203O
gml0De=C0?ooo`3gmoL09RHV0=;BdP3gmoL0OGem0P3oool00`23Ph<0oooo0?ooo`020?ooo`0=0<K6
aP2?Shl0oooo0?ooo`1hN7P0];Bd0?ooo`3gmoL0alO70?ooo`1DE5@03`l?0>o_k`030?ooo`0507Qh
N02d];@0oooo0?Ogm`37alL00`3oool04@36a/H0Shn?0?ooo`3Ogml0De=C0?ooo`3gmoL09RHV0=;B
dP3Ogml0De=C0?ooo`3gmoL09RHV0=;BdP3gmoL0OGem0080oooo00`0Ph>30?ooo`3Ogml0De=C0?oo
o`3gmoL09RHV0=;BdP3oool0E5AD00l?3`3_knl40?ooo`030?Ogm`1mOGd0oooo00D0oooo00@0Hf=S
0>o_k`3Ogml0De=C0P3oool0101MGEd0d];B0?ooo`23Ph<20?ooo`0606=SH`3_knl0oooo05ADE00?
3`l0kno_0P3oool0203gmoL0OGem0?ooo`3oool0Ph>30?ooo`3gmoL0OGem0P3oool00`23Ph<0oooo
0?ooo`0H0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF
0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTO0?ooo`0705EEE@3oool0
oooo0?ooo`1EEED0oooo05EEE@020?ooo`0305EEE@3oool0EEEE00<0oooo00D0EEEE0?ooo`3oool0
oooo05EEE@030?ooo`0305EEE@3oool0EEEE00<0EEEE2P3oool00`1hN7P000000;Bd]0090?ooo`06
0<K6aP2?Shl0oooo0?ooo`3gmoL07AdM0P0000001P092@T0d];B0?ooo`3oool0a/K608n?S`H0oooo
00@0PH610=;BdP3oool0a/K60`0000003@2d];@0oooo0?ooo`36a/H000000;Bd]03oool0moOg01dM
7@2?Shl0oooo0::RXP2d];@0103oool01`21PH40d];B0?ooo`3oool0N7Qh0000002d];@01@3oool0
1`21PH40d];B0?ooo`23Ph<0oooo0?Ogm`1mOGd00P3oool01@36a/H00000000000000000];Bd00<0
oooo00D0QhN70>o_k`3oool0oooo0<K6aP03000000060;Bd]03oool0oooo07QhN0000000];Bd0`3o
ool00`36a/H000000;Bd]0030?ooo`0307QhN0000000];Bd0080oooo00L0Ph>30?ooo`3gmoL0OGem
0?ooo`3gmoL07AdM0080000000P02@T90=;BdP3gmoL07AdM08n?S`3oool0XZ:R0;Bd]080oooo00`0
a/K60000002d];@0oooo0?Ogm`0M7Ad0Shn?0?ooo`2RXZ80];Bd0?Ogm`0M7Ad20000000700T92@3B
d]80oooo0?ooo`1hN7P000000;Bd]0030?ooo`0307QhN0000000];Bd0080oooo00@0Ph>30?ooo`3g
moL0OGem0`3oool00`1hN7P000000;Bd]0030?ooo`030<K6aP000000];Bd00@0oooo00D0QhN70>o_
k`3oool0oooo0<K6aP03000000030;Bd]03oool0oooo0080oooo00@0PH610=;BdP3gmoL0GUiN0P3o
ool01`21PH40d];B0?ooo`3oool0a/K60000002d];@00P3oool01023Ph<0oooo0?Ogm`1mOGd20?oo
o`0408>3P`3oool0moOg07emO@<0oooo00<0a/K60;Bd]03oool0503oool00`1EEED0oooo0?ooo`04
0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd609FEU@0I0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY7`3oool01`1EEED0oooo0?ooo`3oool0EEEE0?ooo`1EEED00P3oool00`1E
EED0oooo05EEE@030?ooo`0505EEE@3oool0oooo0?ooo`1EEED00`3oool00`1EEED0oooo0?ooo`0>
0?ooo`030?Ogm`1NGUh0oooo00P0oooo00<0a/K608n?S`3oool00P3oool0101hN7P0];Bd0<K6aP2?
Shl30?ooo`030<K6aP2?Shl0oooo0080oooo00/0N7Qh000000000000H61P0?ooo`3gmoL0;2`/0>o_
k`3oool0XZ:R0;Bd]0030?ooo`080?Ogm`37alL0oooo0?ooo`1SHf<0?clo00000011@D420?ooo`04
07QhN00000000000061PH0@0oooo00D0moOg05iNGP3oool0oooo07QhN00200000004061PH03oool0
oooo07QhN080000000P0Shn?0?ooo`3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]0<0oooo00T0a/K608n?
S`3oool0moOg02`/;03_knl0oooo0::RXP2d];@00`3oool00`3gmoL0GUiN0?ooo`030?ooo`030?Og
m`37alL0oooo00<0oooo00D0moOg05iNGP3oool0oooo07QhN0020000000708n?S`3oool0oooo07Qh
N02d];@0a/K608n?S`020?ooo`0406=SH`0o?cl000000451@@@0oooo00P0moOg0<O7a`3oool0oooo
06=SH`0o?cl000000451@@80oooo00@0N7Qh0;Bd]036a/H0Shn?103oool00`3gmoL0GUiN0?ooo`03
0?ooo`050?Ogm`1NGUh0oooo0?ooo`1hN7P00P0000000`2?Shl0oooo0?ooo`020?ooo`030?Ogm`1N
GUh0oooo00<0oooo00<0moOg0<O7a`3oool00`3oool02`36a/H0Shn?0?ooo`3gmoL0;2`/0>o_k`3o
ool0XZ:R0;Bd]03oool0N7Qh0080000000@0H61P0?ooo`3gmoL0GUiN0P3oool00`21PH40d];B0?oo
o`020?ooo`050?Ogm`37alL0oooo0?ooo`1hN7P00P000000102?Shl0oooo0?ooo`1hN7P200000003
08n?S`3oool0oooo01D0oooo101EEED60?ooo`0305EEE@3oool0oooo0080oooo00D0k>c/09JFUP3o
ool0moOg0>g]k@060?ooo`0J0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7`3oool01`1EEED0oooo0?oo
o`3oool0EEEE0?ooo`1EEED00P3oool405EEE@80oooo00D0EEEE0?ooo`3oool0oooo05EEE@030?oo
o`0305EEE@3oool0EEEE00<0EEEE203oool01P3gmoL09RHV0=;BdP3gmoL0;2`/0>o_k`L0oooo00<0
Hf=S0;Jf]P2?Shl00`3oool0103gmoL09RHV09NGU`2?Shl20?ooo`0306=SH`2f][H0Shn?00<0oooo
00<0PH610=;BdP3oool00P3oool03`3gmoL0;2`/0>o_k`3oool0Hf=S0>o_k`3Ogml0De=C0?ooo`3g
moL0;2`/0>o_k`3oool0a/K60451@@040?ooo`030861P@3Bd]80oooo0080oooo00T0moOg02HV9P3B
d]80moOg02`/;03_knl0oooo0861P@3Bd]800`3oool0;`3gmoL0GUiN0?ooo`3gmoL0GUiN0?ooo`3g
moL0;2`/0>o_k`3oool0Hf=S0>o_k`3Ogml0LW9b0?ooo`3gmoL0GUiN0?ooo`3gmoL0;2`/0>o_k`3o
ool0Hf=S0>o_k`3gmoL09RHV0=;BdP3gmoL0;2`/0>o_k`3Ogml0De=C0?ooo`3gmoL0;2`/0>o_k`3g
moL09RHV0=;BdP3gmoL0;2`/0>o_k`3gmoL0GUiN0?ooo`3gmoL0GUiN0080oooo00@0moOg02HV9P2G
UiL0Shn?0P3oool00`36a/H0@D510?ooo`020?ooo`090=oOg`1CDe<0oooo0?Ogm`0/;2`0kno_0?oo
o`36a/H0@D5100@0oooo0380moOg02HV9P2GUiL0Shn?0?ooo`3gmoL09RHV0=;BdP3gmoL0;2`/0>o_
k`3gmoL09RHV0=;BdP3gmoL0;2`/0>o_k`3gmoL0GUiN0?ooo`3gmoL0GUiN0?ooo`3gmoL09RHV0=;B
dP3gmoL0;2`/0>o_k`3Ogml0De=C0?ooo`3gmoL0;2`/0>o_k`3Ogml0LW9b0?ooo`3gmoL0GUiN0?oo
o`3gmoL0;2`/0>o_k`3oool0Hf=S0>o_k`3oool0PH610=;BdP@0oooo01H0GEeM0=;BdP3gmoL0;2`/
0>o_k`3Ogml0De=C0?ooo`3gmoL0;2`/0>o_k`3gmoL0GUiN0?ooo`3gmoL0GUiN0?ooo`3gmoL0GUiN
0?ooo`3gmoL0GUiN0P3oool00`36a/H0Shn?0?ooo`0E0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0
e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY7`3oool01@1EEED0oooo0?ooo`3oool0EEEE00H0oooo00D0
EEEE0?ooo`3oool0oooo05EEE@020?ooo`80EEEE0`3oool00`1EEED0oooo0?ooo`0<0?ooo`040=oO
g`0B4Q8000000;Bd]0P0oooo00<0a/K60000002?Shl0103oool00`3Ogml04Q8B08n?S`020?ooo`03
0<K6aP000000Shn?00<0oooo00@0N7Qh000000000000H61P0P3oool01036a/H000000000002?Shl2
0?ooo`040::RXP000000000008n?S`<0oooo00H0XZ:R000000000000];Bd0?ooo`1hN7P200000007
061PH03oool0oooo0=oOg`0B4Q8000000;Bd]0020?ooo`0407QhN00000000000061PH080oooo00@0
XZ:R000000000000];Bd0P3oool01`36a/H000000000002?Shl0oooo0=oOg`0B4Q800P000000100?
3`l0kno_0?ooo`36a/H20000000708n?S`3oool0oooo0=oOg`0B4Q8000000;Bd]0020?ooo`040::R
XP000000000008n?S`80oooo00@0gmoO018B4P000000];Bd0P3oool0102RXZ8000000000002d];@3
0?ooo`030=oOg`0B4Q80Shn?00<0oooo00H0XZ:R000000000000];Bd0?ooo`2RXZ820000000508n?
S`3oool0oooo0?ooo`2RXZ800P0000001P2d];@0oooo0?ooo`3Ogml04Q8B08n?S`80oooo00@0gmoO
018B4P000000];Bd0P3oool0103Ogml04Q8B0000002d];@20?ooo`040::RXP00000000000;Bd]080
oooo00@0gmoO018B4P000000];Bd0P3oool01`2RXZ8000000000002?Shl0oooo0=oOg`0B4Q800P00
0000100?3`l0kno_0?ooo`36a/H20000000408n?S`3oool0oooo07QhN080000000L0H61P0?ooo`3o
ool0gmoO018B4P000000];Bd0080oooo00@0XZ:R000000000000Shn?0P3oool0102RXZ8000000000
002d];@20?ooo`040::RXP00000000000;Bd]1T0oooo0P1EEED60?ooo`0305EEE@3oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJB00oooo0`1EEED;0?ooo`80EEEE0P3oool00`1EEED0oooo0?ooo`0205EEEOl0oooo
0`3oool00`1EEED0oooo05EEE@060?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo>@3o
ool0101EEED0oooo0?ooo`1EEED60?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo>03o
ool01@1EEED0oooo0?ooo`3oool0EEEE00H0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolg0?ooo`H0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood<0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo@`3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3S
hn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood<0oooo00<0EEEE0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooom30?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gM
g@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood<0oooo00<0EEEE0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooom30?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[Z
jP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oood<0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh
0?ooo`D0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc`0oooo00<0EEEE0?oo
o`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0g=cL0=cLg03Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT[0?ooo`80EEEE1@3oool205EEEAT0
oooo00<0XZ:R0000001PH6003P3oool00`3kno/0Ph>30?Ogm`0H0?ooo`0307QhN0000000Shn?0;l0
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT[0?ooo`0305EE
E@3oool0oooo00D0oooo00<0EEEE0?ooo`3oool05`3oool00`2RXZ80Shn?0?ooo`0?0?ooo`0307Qh
N02d];@0oooo01P0oooo00<0XZ:R08n?S`3oool0_P3oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00<0k>c/09JFUP3GemL0303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY8`3oool00`1EEED0oooo
05EEE@020?ooo`0405EEE@3oool0oooo05EEE@<0oooo00D0EEEE0?ooo`3oool0oooo05EEE@020?oo
o`0305EEE@3oool0oooo00/0oooo00L0moOg01dM7@000000@D510??cl`1DE5@0];Bd0080oooo00P0
XZ:R08n?S`3oool0oooo0::RXP0000002@T90=;BdP80oooo00<0a/K60000000000000P0000001P2d
];@0oooo0?ooo`36a/H0@D510?Ogm`@0oooo00P0XZ:R000000000000000000l?3`3_knl0oooo0::R
XP80000000@0Shn?0?ooo`3oool0XZ:R0P0000001P2?Shl0oooo0?ooo`3oool0XZ:R08n?Skl0oooo
00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k03moOd0Ti>C00D0^;Rh
00<0]KFe0;Rh^02h^;P0102h^;P05P2CTi<0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY8`3oool00`1EEED0oooo05EEE@020?ooo`0405EE
E@3oool0oooo05EEE@<0oooo00D0EEEE0?ooo`3oool0oooo05EEE@0A0?ooo`060<K6aP11@D40moOg
0::RXP000000];Bd0P3oool00`2RXZ80Shn?0?ooo`020?ooo`030::RXP2?Shl0oooo00@0oooo00<0
XZ:R08n?S`3oool02`3oool00`1hN7P0@D510?Ogm`020?ooo`0<0=oOg`0K6a/0d];B0=oOg`0K6a/0
d];B0=oOg`0K6a/0d];B0=oOg`0K6a/0d];B0P3oool00`2RXZ80Shn?0?ooo`2n0?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0k^k^08f=S@0<09nOW`0F08f=S@3V
i^H0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTS0?ooo`0305EEE@3oool0EEEE0080oooo00@0EEEE0?ooo`3oool0EEEE0`3oool01@1EEED0oooo
0?ooo`3oool0EEEE00@0oooo101EEED90?ooo`060<K6aP11@D40emOG04i>CP1:BTX0];Bd0P3oool0
0`2RXZ80Shn?0?ooo`020?ooo`050::RXP000000000000l?3`3_knl00P3oool00`2RXZ80Shn?0?oo
o`0<0?ooo`0@0=oOg`0Q8B40kno_0?ooo`3Ogml0De=C0?Ogm`3kno/0GEeM0=;BdP3Ogml0De=C0?Og
m`3kno/0GEeM0=;BdP80oooo00<0XZ:R08n?S`3oool0_P3oool00`1EEED0oooo0?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00<0k>c/09JFUP3Ng]h0303Kfm/05P3Ng]h0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY8`3oool00`1EEED0
oooo05EEE@020?ooo`0405EEE@3oool0oooo05EEE@<0oooo00D0EEEE0?ooo`3oool0oooo05EEE@0A
0?ooo`060<K6aP0l?3`0De=C0<k>cP1DE5@0];Bd0P3oool00`2RXZ80Shn?0?ooo`020?ooo`090::R
XP2?Shl0oooo0::RXP1PH600moOg01dM7@000000Shn?00h0oooo00l0gmoO05=CD`3gmoL0gmoO05=C
D`3gmoL0no_k05eMG@3Bd]80gmoO05=CD`3gmoL0no_k05eMG@3Bd]800P3oool00`2RXZ80Shn?0?oo
o`2n0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0?oo
o`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTS0?ooo`0305EEE@3oool0EEEE0080EEEE0`3oool01@1EEED0oooo0?oo
o`3oool0EEEE00<0oooo00D0EEEE0?ooo`3oool0EEEE0?ooo`0405EEE@T0oooo00H0a/K60000002?
Shl0no_k05ADE02d];@20?ooo`030::RXP2?Shl0oooo0080oooo00D0XZ:R08n?S`3oool0a/K6061P
H00?0?ooo`0B0=oOg`1CDe<0moOg0=oOg`0Q8B40kno_0=oOg`0K6a/0d];B0=oOg`0K6a/0d];B0=oO
g`0K6a/0d];B0=oOg`0K6a/0d];B0P3oool00`2RXZ80Shn?0?ooo`2n0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTS0?oo
o`0305EEE@3oool0oooo00D0oooo00<0EEEE0?ooo`3oool00P1EEED30?ooo`0305EEE@3oool0oooo
00h0oooo00h0gmoO018B4P11@D40lo?c05ADE00000003`l?0>o_k`3oool0XZ:R0000001PH600oooo
0::RXP<0000000H02@T90=;BdP3oool0oooo07QhN02d];@<0?ooo`0407QhN0000000000008n?S`80
oooo00@0XZ:R000000000000Shn?0P3oool0102RXZ8000000000002?Shl20?ooo`0307QhN0000000
Shn?0;l0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0
oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJB<0oooo00<0EEEE0?ooo`3oool01@3oool205EEE@80oooo00<0EEEE
0?ooo`3oool00P1EEEGo0?ooo`L0oooo0P1EEED60?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/
09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolj0?ooo`0305EEE@3oool0EEEE00H0oooo00<0EEEE
0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`020<k>cP0303hn?P3>c/h0c/k>00<0c/k>01X0?Shn
0<k>cP3>c/h0c/k>0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocT0oooo00@0EEEE0?ooo`3oool0EEEE1P3oool00`1EEED0
oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0<o?c`<0?Shn0`3?cll303hn?P80clo?01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo>03oool01@1EEED0oooo0?ooo`3oool0EEEE00H0oooo00<0EEEE0?ooo`3oool00P3oool0
0`3/k>`0UYJF0?ooo`020=CDe0<0?Shn00@0e=CD03hn?P0n?Sh0?Shn0`3De=@05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolg0?ooo`H0EEEE0P3oool505EEE@@0oooo00<0k>c/09JFUP3oool00`3Kfm/503hn?P@0fm_K01H0
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0ooooB03oool00`3/k>`0UYJF0?ooo`040>CTi0<0?Shn1@3Ti>@05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oom80?ooo`030>c/k02FUYH0oooo00D0kNg]00<0?Shn0>g]k@3]kNd0103]kNd05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oom80?ooo`030>c/k02FUYH0oooo00`0m?Cd01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool00`3/k>`0UYJF0?oo
o`0<0?clo00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooodP0oooo00<0k>c/09JFUP3oool03@3oool05@2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom8
0?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0ooooB03oool0103/k>`0f][J09:BTP2f
][H:0;Fe]@0G0;Bd]02?Shl0f][J06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000030000003gmoL0VYZJ0?l0NWYjF`1jNWX04`2JVYX0moOg0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000030000003gmoL0_kno0?l0_knoG02o_kl04P3gmoL0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo00000`000000moOg0>?Sh`3o0>?She`0hn?S0180moOg0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000<000000?7al@3oool0o`3ooolE0?ooo`030>?Sh`2UYJD0MWIf00<0MWIf00@0
MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0hn?S0:FUY@@0MWIf00@0MgMg07If
MP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0jn_[0:FUY@@0EUIF00@0EeMG05IFEP1GEeL0
EeMG2@1FEUH00`2UYJD0jn_[0?ooo`030?ooo`0B0?7al@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003QhN40o?cl0?_kn`80o?cl00<0no_k0?clo03lo?`0103lo?`00`3kno/0o?cl0?cl
o0020?clo0030?_kn`3lo?`0no_k00D0o?cl00<0no_k0?clo03kno/0G`3kno^K0?clo0030:FUY@1f
MWH0SHf=0100SHf=00H0MWIf0:FUY@3lo?`0o?cl0:FUY@1fMWHA08f=S@0607IfMP2UYJD0o?cl0?_k
n`2UYJD0EUIF4@1KFe/00`1FEUH0YJFU0?clo0030?clo00B0>7Qh@000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000030000003Ph>00moOg0?l0moOg5@3gmoL00`1fMWH0SHf=0::RXP0@0::RXP0608f=
S@1fMWH0moOg0?Ogm`1fMWH0SHf=4@2RXZ80202=SHd0MWIf0?Ogm`3gmoL0EUIF05]KF`1_Kfl0LG5a
3@1bLW80101aLG40Kfm_05]KF`1FEUH40?Ogm`0B0>3Ph0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000030000003Ng]h0lO7a0080lO7a0P3bl_800`3alO40l_;b0?;blP040?7al@030?;blP3alO40
lO7a00<0lO7a0P3bl_800`3alO40l_;b0?7al@020?;bl_`0lO7a00<0MWIf0;Rh^02[Zj/0402[Zj/0
1P2h^;P0MWIf0?7al@3alO40MWIf0;Rh^140Zj^[00D0^;Rh07IfMP3alO40l_;b05IFEP0206m_K`03
079bLP1gMgL0MgMg00/0MgMg00@0LW9b06m_K`1_Kfl0EUIF0`3alO404`3bl_80g]kN0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000050000003Lg=`0jn_[0>_[j`3/k>`00P3[jn/00`2WYjL0XZ:R
0::RXP020:6QX@030:2PX02OWil0WIfM0080WinO0P2PX:020:6QX@030:NWY`3[jn/0jn_[00D0jn_[
00D0e=CD0<G5a@35aLD0aLG50=OGe`020>_[j`030>[ZjP38b<P0d=3@0080jn_[00D0f][J0<K6aP3S
hn<0jn_[0>3Ph0020<G5a@070<c<c03YjNT0c/k>0<O7a`3YjNT0jn_[0>3Ph0020<G5a@070<c<c03Z
j^X0e=CD0<G5a@3MgMd0jn_[0>3Ph0020<G5a@060<K6aP3Shn<0jn_[0>_[j`3Zj^X0b<S80`35aLD0
0`36a/H0hn?S0>_[j`070>_[j`030>GUi@37alL0emOG02@0jn_[H@3/k>`j0>_[j`0407IfMP2k^k/0
/K6a0;6a/@P0oooo1`2a/K401P2k^k/0MWIf0>_[j`3[jn/0MWIf0;^k^`80/K6a303oool30;6a/@09
0;^k^`1fMWH0jn_[0>_[j`1FEUH0MgMg079bLP1dM7@0MgMg00/0N7Qh00D0MgMg07AdM01bLW80MgMg
05IFEP040>_[j`0B0=cLg0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003Jf]X0iNGU
00<0iNGU00<0RXZ:09^KV`2/[:`00`3YjNT02P3Zj^X0a<C408F5Q@3/k>`0kNg]0>k^kP3WinL0l?3`
0?;blP2SXj<60>GUi@0A06QXJ00@4100410@010@4020P800a/K60<o?c`3Ng]h09RHV055AD@3?cll0
iNGU09BDU00E5AD0Y:BT0=kNgP2b/[800P0@41001`11@D40fm_K051@D00I6AT0^[Zj0>?Sh`2b/[80
0P0@41001`11@D40gmoO071`L00@4100TY:B0>CTi02_[jl00P0@41001P0E5AD0^[Zj0<c<c03Ti>@0
gMgM02@T90<0410@00<05ADE0:BTY03Ng]h01`3UiND01039bLT071`L079bLP3De=Bo0>GUi@0507If
MP30`<00]KFe0;Fe]@3oool01P1bLW800`0W9bL0]KFe0;Fe]@050;Fe]@060<30`01fMWH0iNGU0>GU
i@1fMWH0`<300P2e]KD00`3oool09bLW02LW9`0902LW9`<0]KFe00D0`<3007IfMP3UiND0iNGU05IF
EP0207MgM`0407UiN@1jNWX0oooo0?ooo`L0NWYj0P3oool01@1jNWX0NGUi07MgM`1gMgL0EUIF00@0
iNGU0180f][J0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=WIf@3Ogml00`3Ogml0
1@3Ng]h0X:2P0>[ZjP3YjNT0iNGU0080j^[Z00X0d=3@07AdM03]kNd0k^k^0:f][@3@d=00l_;b0??c
l`2SXj<0gmoO0P3Ng]h0=03Ogml0g]kN06a/K00E5AD0Zj^[0=kNgP33`l<071`L05ADE036a/H0e]KF
02DU9@1ADE40cLg=0=kNgP2?Shl05ADE0:BTY03EeMD0CTi>03Ph>034a<@0gMgM0=_Kf`1>CTh06ATI
0;Zj^P3IfMT0CTi>03Ph>034a<@0gMgM0=OGe`0U9BD0>3Ph0<C4a032`/806a/K055AD@3IfMT0`l?3
04a<C03Hf=P0emOG02DU9@0I6AT0aLG50=kNgP2?Shl05ADE0:BTY03IfMT70=kNgP040<?3``0L71`0
LW9b0=7AdKl0g]kN00D0MWIf0<?3``2k^k/0^k^k0?ooo`06079bLP0302LW9`2k^k/0^k^k00D0^k^k
00H0`l?307IfMP3Ng]h0g]kN07IfMP33`l<20;^k^`030?ooo`0W9bL0^k^k00L0^k^k00<0oooo02LW
9`2k^k/00P2k^k/02@33`l<0MWIf0=oOg`3Ng]h0EUIF07MgM`1hN7P0Ng]k030`<0030?ooo`D0OGem
0`3oool01@0`<300Ng]k07QhN01gMgL0EUIF0080g]kN01@0gmoO0=kNgP3IfMT0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000emOG0=SHf03IfMT20=SHf00@092@T02?Shl0SXj>0>WYj@21
PH40DE5A08N7Q`3Jf]X0YZJV0:VYZ@2/[:`0GemO0?;blP3clo<0m?Cd0:BTY0<0f=SH00D0fMWI0=;B
dP0T92@0DE5A0<_;b`020=SHf00P06YZJP0I6AT0^[Zj0=3@d00T92@0DE5A0<_;b`3Hf=P0Rh^;01DE
5@2TY:@0cLg=02@T900h>3P0`l?30=OGe`3EeMD0C4a<01TI6@2j^[X0d=3@02@T900h>3P0`l?30=OG
e`3AdM4092@T03Ph>033`l<0RXZ:01DE5@2TY:@40<G5a@0:09jNWP0@4100LG5a0=7Ad@3Hf=P0Rh^;
01DE5@2TY:@0e=CD0=;BdP80aLG500P0a/K60=OGe`3Hf=P0f=SH0;fm_@0K6a/0LW9b0<k>cR@0f=SH
H@3IfMTj0=SHf00507IfMP37alL0_kno0;no_`3oool01`0W9bL70;no_`060<O7a`1fMWH0f=SH0=SH
f01fMWH0alO70P2o_kl00`3oool09bLW0;no_`070;no_`030?ooo`0W9bL0_kno0080_kno00D0alO7
07IfMP3Hf=P0f=SH05IFEP0207]kN`0307inOP1oOgl0:2PX00<0oooo0`1oOgl30?ooo`0302PX:01o
Ogl0OWin0080Ng]k00<0EUIF0=WIf@3Hf=P00P3Hf=P04P3GemL0000009ZJVP2PX:00Z:RX0;:b/P2l
_;`0a/K60=3@d03IfMT0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo00000`000000eMGE0=?Cd`030=?Cd`0308F5Q@2SXj<0i^KV0080j^[Z00/0=CDe07moO`3?
cll0_[jn08^;R`1NGUh0F5QH06QXJ02SXj<0bLW90:ZZZP040=?Cd`040<c<c00T92@0DE5A0<[:bP80
dm?C01h0IfMW01TI6@2i^KT0bl_;02@T901ADE40b/[:0=?Cd`28R8P05ADE0:BTY039bLT092@T03Ph
>031`L40dm?C0<o?c`1:BTX06ATI0;Vi^@3;bl/092@T03Ph>031`L40dm?C0<c<c00T92@0>3Ph0<71
`@1WIfL5010@400:055AD@2MWId0410@08j>SP3<c<`0dm?C08R8R00E5AD0Y:BT0:6QX@80410@00<0
6QXJ0<K6aP3Cdm<00P3Cdm<0102h^;P06a/K079bLP3<c<bo0=?Cd`0307IfMP3;bl/0a/K60100a/K6
00H0bl_;07IfMP3Cdm<0dm?C07IfMP3;bl/20<K6aP030?ooo`0W9bL0a/K600L0a/K600<0oooo02LW
9`36a/H00P36a/H01`3;bl/0MWIf0=?Cd`3Cdm<0EUIF07emO@1oOgl00`23Ph<01@0X:2P0oooo0?oo
o`3oool0Ph>300<0oooo00L0:2PX08>3P`23Ph<0Ph>307moO`1mOGd0EUIF00@0dm?C0140eMGE0000
002PX:00YJFU0:f][@2f][H0_kno0<S8b03Bd]80f][J0>;RhP3YjNT0kno_0?Cdm03fm_H0nOWi0?_k
n`030?gmo@40o_kn0@3oool10?ooo`0000D000000=?Cd`3>c/h0c/k>0<o?c`020<k>cP0?08N7Q`2C
Ti<0dm?C0=_Kf`2][Jd0`L710;2`/02YZJT0ZZZZ08f=S@2ATI40/;2`0<_;b`3_knl0YJFU00<0c/k>
00D0clo?0<O7a`0S8b<0DE5A0<S8b0020<k>cP0]06EUI@0I6AT0^KVi0<O7a`0S8b<0DE5A0<O7a`3>
c/h0QHF501DE5@2TY:@0alO702<S8`0h>3P0`L710<k>cP3;bl/0BDU901TI6@2i^KT0alO702<S8`0h
>3P0`L710<k>cP37alL08b<S03Ph>031`L40QHF501DE5@2TY:@0b/[:0<O7a`0S8b<0DE5A0;6a/@0J
6QX0LW9b0<G5a@3>c/h0QHF501DE5@2TY:@0cLg=00L0c/k>00@0];Bd01XJ6P1bLW80b/[:1P3>c/h0
1@3=cLd0alO70<g=c@37alL0b/[:01T0c/k>H@3?cllj0<k>cP0307IfMP3>c/h0bl_;0100bl_;00H0
c/k>07IfMP3>c/h0c/k>07IfMP3>c/h20<_;b`030?ooo`0W9bL0bl_;00L0bl_;00<0oooo02LW9`3;
bl/00P3;bl/02@3>c/h0MWIf0<o?c`3>c/h0EUIF07emO@22PX80QHF508B4Q00208F5Q@0302PX:03o
ool0oooo00<0oooo00P0:2PX08F5Q@25QHD0Q8B408F5Q@22PX80OGem05IFEP@0c/k>0140dm?C0000
002XZ:P0[Jf]0;>c/`2k^k/0`l?30<c<c03EeMD0gMgM0>CTi03[jn/0l?3`0?Cdm03gmoL0nOWi0?cl
o0030?gmo@40o_kn0@3oool10?ooo`0000<000000=3@d039bLT00`39bLT04@1cLg<0U9BD07inOP1i
NGT0X:2P0=7Ad@2][Jd0YJFU0:JVYP25QHD0GUiN0:f][@3UiND0m_Kf0?Shn02TY:@0bLW900<0b<S8
00@0`/;202<S8`1ADE40alO70P38b<P0;01RHV806ATI0;Vi^@32`/808R8R03Ph>031`L40`/;204I6
AP0I6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35aLD0a<C404I6AP0I6AT0^KVi0<71`@0S8b<0>3Ph0;no
_`35aLD0`<30028R8P0h>3P0`<300:n_[`0J6QX0Lg=c0<G5a@2JVYX0410@08j>SP35aLD0ATI601TI
6@2i^KT0aLG507moO`0E5AD0Y:BT1`38b<P01@37alL0[Zj^01XJ6P1aLG40alO700D0b<S800H0/k>c
04E5A@2QXJ40C4a<06i^KP37alLI0<S8b640bLW9>P38b<P00`1fMWH0dm?C0<g=c@0@0<g=c@060=?C
d`1fMWH0b<S80<S8b01fMWH0dm?C0P3=cLd00`3oool09bLW0<g=c@070<g=c@030?ooo`0W9bL0cLg=
0080cLg=00L0dm?C07IfMP38b<P0b<S805IFEP22PX80Ph>300D0QhN700D0:2PX0?ooo`3oool0oooo
02PX:00508N7Q`0308>3P`22PX80EUIF00<0bLW90180b<S80=3@d0000000/[:b0;Jf]P2k^k/0`/;2
0<W9b@3AdM40fMWI0>3Ph03WinL0kNg]0?7al@3emOD0n?Sh0?[jnP3lo?`30?gmo@40o_kn0@3oool1
0?ooo`0000<000000<o?c`35aLD01035aLD03`2GUiL0]kNg0>c/k02i^KT0Rh^;0;Ng]`2ZZZX0Y:BT
07inOP1aLG40a<C40=WIf@3fm_H0nOWi0:FUY@040<G5a@040;no_`0R8R80DE5A0<K6aP80aLG500H0
HF5Q01TI6@2i^KT0_kno028R8P1MGEd2010@400401DE5@2TY:@0`/;204E5A@<0410@00H0DE5A065Q
H@0@41006ATI0;Jf]P15ADD3010@4008055AD@2o_kl08R8R03Ph>02o_kl0aLG50:b/[00J6QX2010@
4005079bLP35aLD0aLG50<;2`P15ADD00`0@41000`0E5AD0Y:BT0<K6aP050<G5a@060:b/[01LG5`0
[Jf]01XJ6P1aLG40a/K61@35aLD01@2o_kl08R8R030`<00J6QX0SXj>0;D0aLG500<0MWIf0=GEe@3A
dM40403AdM401P3EeMD0MWIf0<G5a@35aLD0MWIf0=GEe@80dM7A00<0oooo02LW9`3AdM401`3AdM40
0`3oool09bLW0=7Ad@020=7Ad@070=GEe@1fMWH0aLG50<G5a@1FEUH0PX:208N7Q`0508V9R@D0oooo
1@29RHT00`27QhL0PX:205IFEP040<G5a@0A0<o?c`000000_;bl0;no_`33`l<0bLW90=3@d03GemL0
gMgM0>CTi03Zj^X0kno_0??cl`3fm_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`000000
cLg=0<71`@030<71`@0@07EeM@2IVIT0YjNW0>g]k@39bLT0Lg=c0;2`/02=SHd0Lg=c04a<C01RHV80
_[jn0;Fe]@2b/[80[Jf]0:VYZ@@0`L7100@0^k^k028R8P1ADE40a<C40P31`L40101OGel06ATI0;Vi
^@32`/870<71`@050;^k^`0R8R80>3Ph0;jn_P32`/800P31`L403033`l<0aLG50;bl_00R8R80>3Ph
0;jn_P32`/80^k^k028R8P0h>3P0_[jn0<;2`P/0`L7100@0O7al01DE5@2TY:@0`l?31@31`L401P2Y
ZJT06QXJ010@400@4100LG5a0<C4a0D0`L7100L0GemO010@400@4100410@03Ph>02n_[h0`/;20;<0
`L7100<0MWIf0=SHf03EeMD0403EeMD01P3Hf=P0MWIf0<71`@31`L40MWIf0=SHf080eMGE00<0oooo
02LW9`3oool0203oool01P0W9bL0eMGE0=GEe@3EeMD0f=SH07IfMP80`L7100<0EUIF08B4Q028R8P0
102:RXX30?ooo`0402PX:03oool0oooo0?ooo`@0RXZ:00<0R8R808B4Q01FEUH01031`L40403=cLd0
00000<K6aP38b<P0c<c<0=7Ad@3GemL0g=cL0>;RhP3Xj>P0kNg]0?7al@3emOD0moOg0?Win@3lo?`3
0?gmo@80o_kn0P3oool000030000003<c<`0^[Zj0080_Kfm0140_[jn0;fm_@2NWYh0hn?S0;Jf]P2F
UYH0PX:209>CT`24Q8@0PX:204i>CP0/;2`0W9bL0:^[Z`3fm_H0n?Sh0:NWY`040;fm_@040;Ng]`0Q
8B40DE5A0<C4a080_Kfm00@0GEeM01TI6@2i^KT0_kno202m_Kd00`1MGEd0=cLg0;Rh^0020;fm_@0;
05eMG@0I6AT0^KVi0;no_`1MGEd0=cLg0;Rh^02m_Kd0]kNg024Q8@0h>3P00P2n_[h;0;fm_@0407Yj
NP0E5AD0Y:BT0;no_`H0_Kfm00D0YJFU01XJ6P0@4100L71`0<;2`P050;fm_@0607YjNP0d=3@0?3`l
05eMG@0h>3P0^;Rh6@2m_KeQ0;jn_SX0_Kfm00<0MWIf0=[JfP3EeMD0403EeMD01P3Jf]X0MWIf0;fm
_@2m_Kd0MWIf0=[JfP80eMGE00<0oooo079bLP1bLW80201bLW803P0W9bL0eMGE0=GEe@3EeMD0f][J
07IfMP2m_Kd0_[jn05IFEP27QhL0RXZ:08f=S@2>SXh0SHf=0`3oool00`0X:2P0SHf=02PX:0030?oo
o`<0SHf=00@0RXZ:08N7Q`1FEUH0_[jn0P2m_Kd04@2j^[X0c<c<0000003@d=00d];B0=GEe@3IfMT0
gMgM0>;RhP3WinL0k>c/0?3`l03dm?@0m_Kf0?Shn03jn_X0o?cl00<0oOgm00<0o_kn0?ooo`3oool0
0@3oool000030000003:b/X0^;Rh00<0^[Zj0100O7al08f=S@1hN7P0VIVI0?3`l032`/80YjNW0<_;
b`2l_;`0Xj>S06e]K@1mOGd0[:b/0?_kn`3lo?`0YZJV0`2j^[X0102k^k/0];Bd024Q8@1=CDd30;Zj
^P0305]KF`0I6AT0[jn_00h0^[Zj00<0Fe]K01TI6@2_[jl01@2j^[X0102d];@08B4Q03Lg=`2d];@<
0;Zj^P0307MgM`0D51@0Vi^K00T0^[Zj00<0Fe]K06][J`2j^[X01P2j^[X00`2d];@08B4Q06][J`2f
0;Zj^P0307IfMP3Hf=P0f][J0100f][J00H0f=SH07IfMP2j^[X0^[Zj07IfMP3Hf=P20=[JfP030?oo
o`1bLW80LW9b00P0LW9b00d09bLW0=[JfP3Jf]X0f][J0=SHf01fMWH0^[Zj0;^k^`1FEUH0Rh^;08n?
S`2>SXh0Shn?00<0oooo00D0:2PX08n?S`2?Shl0Shn?02PX:0030?ooo`0508n?S`2>SXh0Shn?08^;
R`1FEUH00`2j^[X0402h^;P0b/[:0000003IfMT0f][J0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??c
l`3fm_H0n?Sh0?[jnP3lo?`30?gmo@80o_kn0`3oool0000300000039bLT0^;Rh00<0^;Rh0100QXJ6
09jNWP3De=@0l?3`0?;blP3>c/h0`L710=gMg@3Bd]80XZ:R0<c<c039bLT0RHV90?clo03moOd0YjNW
o`2h^;P20;Rh^00307IfMP3GemL0gMgM0100gMgM00H0emOG07IfMP2h^;P0^;Rh07IfMP3GemL30=gM
g@/09bLW0`3MgMd02@3GemL0MWIf0;Rh^02h^;P0EUIF092@T02ATI40Ti>C09BDU0020?ooo`0302PX
:02HV9P0V9RH00<0V9RH00P0:2PX0?ooo`3oool0U9BD09>CT`2ATI40T92@05IFEP@0^;Rh00l0bLW9
0000003QhN40h^;R0>CTi03WinL0j^[Z0>g]k@3`l?00lo?c0?Gem@3gmoL0nOWi0?_kn`3lo?`00`3m
oOd00`3no_h0oooo0?ooo`020?ooo`0000<000000<W9b@2h^;P0102h^;P03`26QXH0U9BD0?;blP3c
lo<0X:2P0>SXj03fm_H0fm_K09VIV@3jn_X0no_k0=cLg02f][H0o_kn0:NWY`3o0;Rh^080^;Rh00<0
MWIf0=KFeP3QhN40403QhN401P3Fe]H0MWIf0;Rh^02h^;P0MWIf0=KFeQ40hN7Q00T0e]KF07IfMP2h
^;P0^;Rh05IFEP2EUID0UiNG09bLW02RXZ800P0X:2P01@2[Zj/0[Jf]0:^[Z`2[Zj/0[Jf]0080Zj^[
0P0X:2P01@2RXZ80W9bL09NGU`2EUID0EUIF00@0^;Rh00h0bLW90000003Xj>P0jNWY0>_[j`3]kNd0
kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0<0oOgm0P3no_h40?ooo`0000<000000<W9b@2e
]KD00`2e]KD03@1gMgL0X:2P0>3Ph03clo<0iNGU0>c/k03fm_H0n?Sh0?Win@2@T900no_k0?clo03m
oOd00P3no_h00`2WYjL0]KFe0;Fe]@3o0;Fe]@0307IfMP3]kNd0jNWY0100jNWY00H0kNg]07IfMP2e
]KD0]KFe07IfMP3]kNdA0>WYj@0:0>g]k@1fMWH0]KFe0;Fe]@1FEUH0Vi^K09jNWP2UYJD0/[:b0;Zj
^P80_kno00@0`<300;no_`2o_kl0`<300P2o_kl01`30`<00^[Zj0;:b/P2UYJD0WYjN09^KV`1FEUH0
102e]KD03@39bLT000000>k^kP3_knl0l?3`0?7al@3clo<0mOGe0?KfmP3hn?P0nOWi0?_kn`3lo?`0
0`3moOd20?knoPD0oooo00000`000000bLW90;Zj^P040;6a/@0308f=S@2ZZZX0ZZZZ00D0ZZZZ00<0
YjNW0:ZZZP2ZZZX0102ZZZ[o0;6a/@80/K6a00<0MWIf0=CDe03bl_80403bl_801P3De=@0MWIf0;6a
/@2a/K40MWIf0=CDe140l_;b00H0e=CD07IfMP2a/K40/K6a05IFEP2SXj<20:RXZ0d0`l?30P2XZ:P0
0`2SXj<0EUIF0;6a/@020;6a/@040;Zj^P39bLT000000??cl`80m?Cd00L0mOGe0?KfmP3gmoL0n?Sh
0?[jnP3kno/0o?cl00@0oOgm00<0o_kn0?ooo`3oool0103oool0000300000033`l<0bLW90?l0[Zj^
5@2^[Zh0102=SHd0MWIf0<k>cP3kno/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00@0SHf=07If
MP3>c/h0no_k3@3oool0103kno/0c/k>07IfMP2=SHd20:j^[P0609VIV@1FEUH0]KFe0<_;b`3Ogml0
j>SX2@3WinL01P3Xj>P0gmoO0<_;b`2e]KD0EUIF09VIV@<0[Zj^00<0bLW90<?3``0000000P3fm_H0
1@3gmoL0n?Sh0?Shn03inOT0n_[j0080o?cl103moOd20?knoPH0oooo000010000000`l?30=CDe02g
]kOo0:NWYaD0YjNW00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=S@1f
MWH0MWIf00l0MWIf00<0SHf=0:NWY`2WYjL00P2WYjL00`2=SHd0EUIF05IFEP0?05IFEP0808f=S@2W
YjL0YjNW0:NWY`2g]kL0e=CD0<?3``0000030?Win@040?[jnP3kno/0o?cl0?clo0D0oOgm0P3no_h7
0?ooo`0000H000000:ZZZP3WinL0e=CD0;jn_P2c/k?o0:ZZZUD0ZZZZ00H0/k>c0;jn_P3De=@0inOW
0:ZZZP0000020?_kn`<0o?cl1@3moOd30?knoPP0oooo00001`000000RXZ:0=WIf@3lo?`0i>CT0=;B
dP3<c<`0o`2n_[iC0;jn_P080<c<c03Bd]80i>CT0?clo03IfMT0RXZ:0000003lo?`80?gmo@80o_kn
2P3oool000080000001YJFT0UiNG0=WIf@3lo?`0l_;b0>c/k03Shn?o0=WIfE40fMWI00P0hn?S0>c/
k03bl_80o?cl0=WIf@2GUiL0JFUY000000L0oOgm0`3no_h;0?ooo`0000D000000?ooo`1YJFT0UiNG
0=?Cd`020?clo?l0ooooD`3oool20?clo0050=?Cd`2GUiL0JFUY0?ooo`000000103moOd40?knoPd0
oooo00000`000000oooo0?ooo`0206UYJ@0308Z:RP2GUiL0ZZZZ0?l0ZZZZDP2ZZZX0102GUiL0RXZ:
06UYJ@1YJFT20?ooo`030000003no_h0o_kn00@0o_kn3`3oool00?l00000H@00000E0?ooo`00\
\>"],
  ImageRangeCache->{{{126.938, 387}, {330.563, 187.375}} -> {-181.906, \
269.267, 1.0022, 1.0022}}],

Cell[TextData[{
  "Notice that when you evaluate an expression in a notebook, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " adds input and output prompts. In the example notebook above, these are \
denoted ",
  StyleBox["In[1]:=", "MR"],
  " and ",
  StyleBox["Out[1]=", "MR"],
  ". These prompts can be thought of as markers (or labels) that you can \
refer to during your ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session.",
  " "
}], "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .86649 
%%ImageSize: 368 319 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
1.79978e-017 0.0027248 -0.0027248 0.0027248 [
[ 0 0 0 0 ]
[ 1 .86649 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .86649 L
0 .86649 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00272 translate
1.00272 .86921 scale
1104 string
368 319 8 [368 0 0 319 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FEFEFEFEFEFEFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFC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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFBFBFB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FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9
F8F8F8F6F6F6F5F5F5F3F3F3F1F1F1F0F0F0EFEFEFEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EFEFEFF0F0F0F1F1F1F3F3F3F5F5F5F6F6F6F8F8F8F9F9F9FBFBFBFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7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FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7F5F5F5
F3F3F3F0F0F0EDEDEDEAEAEAE7E7E7E4E4E4E2E2E2E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E2E2E2E4E4E4E7E7E7EAEAEAEDEDEDF0F0F0F3F3F3F5F5F5F7F7F7F9F9F9FBFBFBFCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F3F3F3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FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F4F4F4F0F0F0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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F5F5F5F1F1F1EDEDED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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F6F6F6F3F3F3EFEFEFEAEAEA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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F5F5F5F1F1F1EDEDEDE7E7E7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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F4F4F4F0F0F0EBEBEBE4E4E4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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFBFBFBF9F9F9F6F6F6F4F4F4EFEFEFE9E9E9E2E2E2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FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F6F6F6F3F3F3EEEEEEE8E8E8E1E1E1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000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
ECECECBABABA9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FBDBDBD
ECECECE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7EEEEEE8D8D8D9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
8D8D8DE6E6E6E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
969696DBDBDBCACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACADBDBDB
A3A3A3FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696DEDEDEDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DEDEDE969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0
FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0FFFFFF
969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3
B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF
969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3
B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E0000000000000000009E9E9ED9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
DCDCDC515151B9B9B9E0E0E0D9D9D91A1A1A000000383838D9D9D9E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD1D1D1D0D0D0D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D0D0D0
D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF
969696FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA969696FFFFFFCECECE
CECECECECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECECECECE
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0E0E0E0E0E0E0
E0E0E0ADADAD545454E0E0E0ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6
D7D7D7D7D7D7D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6D6D6D6
D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D7D7D7D7D7D7D6D6D6D6D6D6D6D6D6FFFFFF
969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCECECE
CECECECECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECECECECE
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9E0E0E0E0E0E0
E0E0E0DCDCDC515151B9B9B9ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595DCDCDCDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF
969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3
B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2E0E0E0E0E0E0
E0E0E0E0E0E0C4C4C4484848D2D2D21A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000000000
000000000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFD4D4D4D4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595DCDCDCDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF
969696F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8969696FFFFFFD4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3
B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2DCDCDC494949
0000000D0D0DD2D2D26969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000
000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFDBDBDBDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696
969696FFFFFFE4E4E4E3E3E3E3E3E3E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E4E4E4E4E4E4FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595E3E3E3E4E4E4
E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E4E4E4FFFFFF
969696F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7969696FFFFFFDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDB
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9ADADAD545454
E0E0E06969697E7E7EC4C4C4484848D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFE4E4E4E4E4E4E4E4E4
3E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595E9E9E9EAEAEA
EAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF
969696F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5969696FFFFFFE4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E000000383838D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0ADADAD545454
E0E0E06969697E7E7EE0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595E9E9E9EAEAEA
EAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF
969696F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4969696FFFFFFEDEDED
EDEDEDEDEDEDEDEDEDEDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D9D9D9525252D9D9D9E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0C4C4C41D1D1DD2D2D2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF
969696F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1969696FFFFFFF4F4F4
F4F4F4F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF
969696F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0969696FFFFFFFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
969696FFFFFFF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFFF
969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
969696FAFAFAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFAFAFA
969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696D7D7D7FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
D7D7D7969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0DADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
D6D6D6B0B0B0B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8A9A9A9
CECECEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEDADADA929292B6B6B6
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B4
8F8F8FDADADAE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7ECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECE3E3E3ECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECE3E3E3E3E3E3E3E3E3E
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3E3E3E3E
3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E
3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDB
3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFE4E4E4E4E4E43E3E3EE4E4E4E4E4E4E4E4E4E4E4E4
E4E4E43E3E3EE4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020202020FFFFFFFFFFFF202020202020202020202020202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B29A9A9AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000D9D9D9FFFFFF
FFFFFFFFFFFFFFFFFFB2B2B2FFFFFFFFFFFF6868680000000000008C8C8CBDBDBDF0F0F0
FFFFFF0000007C7C7CFFFFFFFFFFFF0000007C7C7CFFFFFFFFFFFF7C7C7C000000FFFFFF
FFFFFF000000686868FFFFFFFFFFFF6868684D4D4DFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFB2B2B2FFFFFFE1E1E1000000000000C7C7C7E9E9E9FFFFFFC7C7C70000009A9A9A
FFFFFFFFFFFFF0F0F0000000E1E1E1FFFFFF4D4D4D000000FFFFFFE9E9E9000000A7A7A7
FFFFFFE9E9E9000000000000E1E1E1FFFFFF4D4D4D8C8C8CFFFFFF9A9A9A0000004D4D4D
8C8C8CBDBDBDF0F0F0D9D9D99A9A9A000000C7C7C7FFFFFFD9D9D9000000D0D0D0F0F0F0
9A9A9A000000F0F0F0F0F0F0000000E1E1E1FFFFFF4D4D4D000000FFFFFFFFFFFFE9E9E9
000000A7A7A7FFFFFFFFFFFFF0F0F00000009A9A9AFFFFFFB2B2B2000000F0F0F0FFFFFF
FFFFFFF0F0F0000000E1E1E1FFFFFF4D4D4D000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E9A7A7A7000000000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFBDBDBDC7C7C7D9D9D97C7C7C0000007C7C7CFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000D9D9D9FFFFFF
FFFFFFFFFFFFFFFFFF7C7C7CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF686868686868FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFBDBDBD000000000000E9E9E9FFFFFFFFFFFFFFFFFFE1E1E18C8C8C000000
686868FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF9A9A9A000000E9E9E9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF686868686868FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0C7C7C74D4D4DFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFBDBDBD4D4D4D0000007C7C7CE1E1E1FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D98C8C8C0000000000008C8C8CD9D9D9
FFFFFFE9E9E9A7A7A70000008C8C8CD9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF8C8C8C686868FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFBDBDBD686868000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B2000000
000000A7A7A7E9E9E9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE9E9E97C7C7C000000BDBDBDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF686868686868FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020202020202020202020202020FFFFFFFFFFFF202020202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFF202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF202020202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF202020202020202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFE1E1E1A7A7A7000000A7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE1E1E1
A7A7A7000000A7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECECF0F0F08E8E8EA0A0A0A0A0A0A0A0A0A0A0A0A0A0A09F9F9FA0A0A0
A0A0A0A0A0A0A0A0A0A0A0A0A0A0A0909090ECECEC696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020202020202020202020202020FFFFFFFFFFFF202020202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696DEDEDED3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3DEDEDE969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000C7C7C7000000
D9D9D9D9D9D9000000E1E1E1000000D9D9D9BDBDBDA7A7A7C7C7C7A7A7A7BDBDBDFFFFFF
C7C7C7A7A7A7F0F0F09A9A9AFFFFFFD9D9D9000000E1E1E1C7C7C70000007C7C7CBDBDBD
A7A7A7A7A7A7C7C7C7000000E1E1E1FFFFFF0000000000007C7C7CFFFFFFC7C7C7000000
7C7C7CBDBDBDFFFFFFD9D9D9000000E1E1E1FFFFFFC7C7C70000007C7C7CBDBDBDFFFFFF
000000BDBDBDBDBDBDA7A7A7D9D9D9000000E1E1E1FFFFFFC7C7C70000007C7C7CBDBDBD
000000C7C7C77C7C7CC7C7C70000007C7C7CBDBDBDD9D9D9000000E1E1E1000000D9D9D9
FFFFFF0000009A9A9A000000A7A7A7000000C7C7C7FFFFFF0000009A9A9ABDBDBD000000
F0F0F0BDBDBDA7A7A7C7C7C7A7A7A7BDBDBDD9D9D9000000E1E1E1000000D9D9D9FFFFFF
000000A7A7A7000000C7C7C7D9D9D9000000E1E1E1000000D9D9D9A7A7A7A7A7A7C7C7C7
000000E1E1E1FFFFFFFFFFFFFFFFFFE1E1E19A9A9AFFFFFFD9D9D9000000F0F0F0000000
C7C7C77C7C7CFFFFFF0000009A9A9A000000D9D9D9000000A7A7A7000000C7C7C7D9D9D9
000000E1E1E1D9D9D9000000F0F0F09A9A9ABDBDBDC7C7C77C7C7C000000A7A7A7000000
C7C7C7FFFFFF0000009A9A9ABDBDBD000000F0F0F0BDBDBDA7A7A7C7C7C7A7A7A7BDBDBD
D9D9D9000000E1E1E1000000D9D9D9FFFFFFBDBDBDA7A7A7C7C7C7A7A7A7BDBDBDD9D9D9
000000E1E1E1FFFFFFFFFFFFD9D9D9000000F0F0F0D9D9D9000000E1E1E1000000D9D9D9
D9D9D9000000E1E1E1BDBDBDA7A7A7C7C7C7A7A7A7BDBDBDD9D9D9000000E1E1E1FFFFFF
D9D9D9000000E1E1E1000000E1E1E10000009A9A9A000000A7A7A7000000C7C7C7FFFFFF
0000009A9A9ABDBDBD000000F0F0F0BDBDBDA7A7A7C7C7C7A7A7A7BDBDBDD9D9D9000000
E1E1E1000000D9D9D9FFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000F0F0F0000000
D9D9D9D9D9D9000000A7A7A7000000FFFFFFC7C7C7BDBDBDC7C7C7BDBDBDC7C7C7686868
000000D9D9D9000000E1E1E1686868E1E1E1000000FFFFFFA7A7A7BDBDBDC7C7C79A9A9A
D9D9D9BDBDBDC7C7C7000000FFFFFFFFFFFF9A9A9ABDBDBD000000686868A7A7A7BDBDBD
C7C7C79A9A9AFFFFFFD9D9D90000007C7C7CA7A7A7A7A7A7BDBDBDC7C7C79A9A9AE1E1E1
000000C7C7C7A7A7A7A7A7A7D9D9D90000007C7C7CA7A7A7A7A7A7BDBDBDC7C7C79A9A9A
9A9A9AC7C7C7686868A7A7A7BDBDBDC7C7C79A9A9AD9D9D9000000A7A7A7000000FFFFFF
E1E1E10000007C7C7CA7A7A79A9A9A000000FFFFFFE1E1E10000007C7C7CE1E1E1000000
FFFFFFC7C7C7BDBDBDC7C7C7BDBDBDC7C7C7D9D9D9000000A7A7A7000000FFFFFFFFFFFF
A7A7A79A9A9A000000FFFFFFD9D9D9000000A7A7A7000000FFFFFFD9D9D9BDBDBDC7C7C7
000000FFFFFFFFFFFFFFFFFF7C7C7C000000F0F0F0000000E1E1E1000000FFFFFF9A9A9A
C7C7C7686868D9D9D9000000E1E1E1000000FFFFFFA7A7A79A9A9A000000FFFFFFFFFFFF
000000FFFFFFE1E1E1000000FFFFFFA7A7A7BDBDBD686868F0F0F0A7A7A79A9A9A000000
FFFFFFE1E1E10000007C7C7CE1E1E1000000FFFFFFC7C7C7BDBDBDC7C7C7BDBDBDC7C7C7
D9D9D9000000A7A7A7000000FFFFFFFFFFFFC7C7C7BDBDBDC7C7C7BDBDBDC7C7C7C7C7C7
0000009A9A9AFFFFFFFFFFFFE1E1E1000000FFFFFFD9D9D9000000A7A7A7000000FFFFFF
C7C7C70000009A9A9AC7C7C7BDBDBDC7C7C7BDBDBDC7C7C7D9D9D90000007C7C7CA7A7A7
D9D9D9000000A7A7A7000000BDBDBD000000E1E1E1A7A7A79A9A9A000000FFFFFFE1E1E1
0000007C7C7CE1E1E1000000FFFFFFC7C7C7BDBDBDC7C7C7BDBDBDC7C7C7D9D9D9000000
A7A7A7000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFF0F0F0000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9
000000FFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C7C7C79A9A9A9A9A9AFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFC7C7C79A9A9A9A9A9AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE1E1E1FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFD9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
D9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000F0F0F0A7A7A7
000000A7A7A7E1E1E1000000C7C7C77C7C7C000000A7A7A7FFFFFFFFFFFFFFFFFF000000
A7A7A7000000C7C7C7D9D9D9000000E1E1E1000000D9D9D9A7A7A7A7A7A7C7C7C7000000
E1E1E1FFFFFFFFFFFFFFFFFFC7C7C70000007C7C7CBDBDBDD9D9D9000000E1E1E1000000
D9D9D9FFFFFF0000000000007C7C7CFFFFFFD9D9D9000000F0F0F0D9D9D9000000E1E1E1
000000D9D9D9C7C7C70000007C7C7CBDBDBDC7C7C70000007C7C7CBDBDBDD9D9D9000000
E1E1E1FFFFFFD9D9D9000000F0F0F0D9D9D9000000E1E1E1000000D9D9D9FFFFFF000000
0000007C7C7CFFFFFF000000A7A7A7FFFFFFD9D9D9000000F0F0F0D9D9D9000000E1E1E1
000000D9D9D9A7A7A7000000A7A7A7E1E1E1D9D9D9000000E1E1E1FFFFFF0000009A9A9A
000000D9D9D9A7A7A7A7A7A7C7C7C7000000E1E1E1D9D9D9000000F0F0F0D9D9D9000000
E1E1E1000000D9D9D9FFFFFF0000000000007C7C7CFFFFFFFFFFFFFFFFFFA7A7A7000000
A7A7A7E1E1E1BDBDBDA7A7A7C7C7C7A7A7A7BDBDBDD9D9D9000000E1E1E1000000E1E1E1
0000009A9A9AFFFFFF000000BDBDBDBDBDBDA7A7A7FFFFFF0000009A9A9A000000D9D9D9
FFFFFF0000009A9A9ABDBDBD000000F0F0F0D9D9D9000000E1E1E1000000D9D9D9FFFFFF
0000000000007C7C7CFFFFFF000000A7A7A7FFFFFFFFFFFFFFFFFF000000BDBDBDBDBDBD
A7A7A7D9D9D9000000E1E1E1FFFFFFBDBDBDA7A7A7C7C7C7A7A7A7BDBDBDFFFFFF000000
0000007C7C7CFFFFFFD9D9D9000000E1E1E1FFFFFF000000A7A7A7000000C7C7C7D9D9D9
000000E1E1E1000000E1E1E10000009A9A9AD9D9D9000000E1E1E1000000E1E1E1000000
9A9A9AD9D9D9000000F0F0F0D9D9D9000000E1E1E1000000D9D9D9FFFFFF000000000000
7C7C7CFFFFFF000000A7A7A7FFFFFFFFFFFFFFFFFFA7A7A7A7A7A7C7C7C7000000E1E1E1
000000A7A7A7000000C7C7C7FFFFFF0000009A9A9A000000A7A7A7000000C7C7C7FFFFFF
FFFFFFFFFFFF000000A7A7A7000000C7C7C7D9D9D9000000E1E1E1000000D9D9D9000000
A7A7A7000000C7C7C7D9D9D9000000E1E1E1FFFFFFE1E1E17C7C7CFFFFFF000000C7C7C7
7C7C7CD9D9D9000000F0F0F0000000C7C7C77C7C7C000000FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE1E1E1FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFE1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000
D9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF0000007C7C7CBDBDBDD9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC7C7C7
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFBDBDBDF0F0F0FFFFFF7C7C7CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0
BDBDBDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000E1E1E1000000
D9D9D9FFFFFF0000009A9A9A000000D9D9D9D9D9D9000000E1E1E1000000D9D9D9A7A7A7
A7A7A7C7C7C7000000E1E1E1D9D9D9000000E1E1E1FFFFFFC7C7C70000007C7C7CBDBDBD
A7A7A7A7A7A7C7C7C7000000E1E1E1000000C7C7C77C7C7CFFFFFFBDBDBDA7A7A7C7C7C7
A7A7A7BDBDBDD9D9D9000000E1E1E1FFFFFFFFFFFFD9D9D9000000E1E1E1FFFFFF000000
9A9A9A000000D9D9D9D9D9D9000000E1E1E1000000D9D9D9A7A7A7000000A7A7A7E1E1E1
FFFFFF0000009A9A9ABDBDBD000000F0F0F0BDBDBDA7A7A7C7C7C7A7A7A7BDBDBDD9D9D9
000000E1E1E1000000D9D9D9000000C7C7C77C7C7CFFFFFFFFFFFFD9D9D9000000E1E1E1
BDBDBDA7A7A7C7C7C7A7A7A7BDBDBDD9D9D9000000E1E1E1FFFFFFFFFFFFFFFFFF000000
BDBDBDBDBDBDA7A7A7C7C7C70000007C7C7CBDBDBDD9D9D9000000E1E1E1FFFFFFD9D9D9
000000E1E1E1BDBDBDA7A7A7C7C7C7A7A7A7BDBDBDD9D9D9000000E1E1E1FFFFFFD9D9D9
000000E1E1E1000000E1E1E10000009A9A9AD9D9D9000000F0F0F0D9D9D9000000E1E1E1
000000D9D9D9FFFFFF0000000000007C7C7CFFFFFFFFFFFFFFFFFFE1E1E19A9A9AFFFFFF
000000A7A7A7000000C7C7C7D9D9D9000000E1E1E1FFFFFFD9D9D9000000F0F0F0BDBDBD
A7A7A7C7C7C7A7A7A7BDBDBDFFFFFF0000009A9A9A000000D9D9D9000000C7C7C77C7C7C
FFFFFFFFFFFFFFFFFF0000009A9A9A000000A7A7A7000000C7C7C7000000C7C7C77C7C7C
D9D9D9000000C7C7C7000000D9D9D9000000C7C7C77C7C7CFFFFFFFFFFFFD9D9D9000000
F0F0F0D9D9D9000000E1E1E1000000D9D9D9FFFFFFFFFFFF000000C7C7C77C7C7CA7A7A7
000000A7A7A7E1E1E1D9D9D9000000F0F0F0C7C7C70000007C7C7CBDBDBDD9D9D9000000
E1E1E1000000D9D9D9A7A7A7000000A7A7A7E1E1E1C7C7C70000007C7C7CBDBDBD000000
A7A7A7FFFFFFFFFFFFFFFFFFD9D9D9000000E1E1E1000000E1E1E10000009A9A9A000000
A7A7A7000000C7C7C7FFFFFF0000009A9A9ABDBDBD000000E1E1E1000000D9D9D9C7C7C7
0000007C7C7CBDBDBDD9D9D9000000E1E1E1000000E1E1E10000009A9A9A000000A7A7A7
000000C7C7C7FFFFFF0000009A9A9AE1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000A7A7A7000000
FFFFFFD9D9D9000000E1E1E1000000FFFFFFD9D9D9000000A7A7A7000000FFFFFFD9D9D9
BDBDBDC7C7C7000000FFFFFFD9D9D90000007C7C7CA7A7A7A7A7A7BDBDBDC7C7C79A9A9A
D9D9D9BDBDBDC7C7C7000000FFFFFF9A9A9AC7C7C7686868FFFFFFC7C7C7BDBDBDC7C7C7
BDBDBDC7C7C7C7C7C70000009A9A9AFFFFFFFFFFFFC7C7C70000009A9A9AD9D9D9000000
E1E1E1000000FFFFFFD9D9D9000000A7A7A7000000FFFFFFA7A7A7A7A7A7686868BDBDBD
E1E1E10000007C7C7CE1E1E1000000FFFFFFC7C7C7BDBDBDC7C7C7BDBDBDC7C7C7D9D9D9
000000A7A7A7000000FFFFFF9A9A9AC7C7C7686868FFFFFFFFFFFFC7C7C70000009A9A9A
C7C7C7BDBDBDC7C7C7BDBDBDC7C7C7D9D9D90000007C7C7CA7A7A7FFFFFFE1E1E1000000
C7C7C7A7A7A7A7A7A7A7A7A7BDBDBDC7C7C79A9A9AD9D9D90000007C7C7CA7A7A7C7C7C7
0000009A9A9AC7C7C7BDBDBDC7C7C7BDBDBDC7C7C7D9D9D90000007C7C7CA7A7A7D9D9D9
000000A7A7A7000000BDBDBD000000E1E1E1E1E1E1000000FFFFFFD9D9D9000000A7A7A7
000000FFFFFFFFFFFF9A9A9ABDBDBD000000686868FFFFFF7C7C7C000000F0F0F0000000
8C8C8C9A9A9A000000FFFFFFD9D9D90000007C7C7CA7A7A7E1E1E1000000FFFFFFC7C7C7
BDBDBDC7C7C7BDBDBDC7C7C7D9D9D9000000E1E1E1000000FFFFFF9A9A9AC7C7C7686868
FFFFFFFFFFFFE1E1E10000007C7C7CA7A7A79A9A9A000000FFFFFF9A9A9AC7C7C7686868
FFFFFF000000F0F0F0000000D9D9D99A9A9AC7C7C7686868FFFFFFFFFFFFE1E1E1000000
FFFFFFD9D9D9000000A7A7A7000000FFFFFFFFFFFFFFFFFF9A9A9AC7C7C7686868A7A7A7
A7A7A7686868BDBDBDE1E1E1000000FFFFFFA7A7A7BDBDBDC7C7C79A9A9AD9D9D9000000
A7A7A7000000FFFFFFA7A7A7A7A7A7686868BDBDBDA7A7A7BDBDBDC7C7C79A9A9AFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000A7A7A7000000BDBDBD000000E1E1E1A7A7A7
9A9A9A000000FFFFFFE1E1E10000007C7C7CF0F0F0000000A7A7A7000000FFFFFFA7A7A7
BDBDBDC7C7C79A9A9AD9D9D9000000A7A7A7000000BDBDBD000000E1E1E1A7A7A79A9A9A
000000FFFFFFE1E1E10000007C7C7CF0F0F0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFF0F0F0000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFF0F0F0000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFC7C7C79A9A9A9A9A9AFFFFFFFFFFFFC7C7C79A9A9A9A9A9AFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC7C7C79A9A9A
9A9A9AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C7C7C79A9A9A9A9A9AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
D9D9D9000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595
959595959595D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595
959595959595D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF9A9A9A7C7C7CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000007C7C7C
BDBDBDD9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF0000007C7C7CBDBDBDD9D9D9FFFFFFFFFFFFFFFFFFFFFFFF
9A9A9A7C7C7CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
D9D9D9000000D9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF9A9A9A
7C7C7CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595
959595959595D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFD9D9D9E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBDBDBDF0F0F0
FFFFFF7C7C7CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFBDBDBDF0F0F0FFFFFF7C7C7CFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFD9D9D9E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
D9D9D9E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE1E1E1FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B2000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC7C7C7D0D0D0FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE1E1E10000007C7C7CFFFFFFFFFFFFFFFFFF000000B2B2B2FFFFFFFFFFFF
B2B2B2000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000
7C7C7CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000B2B2B2FFFFFFFFFFFFB2B2B2000000
FFFFFFFFFFFFFFFFFFFFFFFF0000008C8C8CFFFFFFFFFFFFF0F0F0FFFFFF000000B2B2B2
FFFFFFFFFFFFFFFFFFFFFFFFE1E1E1000000D9D9D9FFFFFFFFFFFFD9D9D9000000E1E1E1
FFFFFFFFFFFFC7C7C7000000E1E1E1FFFFFFFFFFFF8C8C8C000000F0F0F0FFFFFFFFFFFF
FFFFFF6868687C7C7CFFFFFFFFFFFF9A9A9A000000FFFFFFFFFFFFFFFFFFD0D0D0000000
9A9A9AE9E9E9FFFFFFD9D9D9A7A7A7FFFFFFFFFFFF0000008C8C8CFFFFFFFFFFFFF0F0F0
FFFFFF000000B2B2B2FFFFFFFFFFFFE1E1E1000000D9D9D9FFFFFFFFFFFFD9D9D9000000
E1E1E1FFFFFFFFFFFFFFFFFF000000B2B2B2FFFFFFFFFFFFB2B2B2000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0000008C8C8CFFFFFFFFFFFFF0F0F0E1E1E1000000
D9D9D9FFFFFFFFFFFFD9D9D9000000E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E9
7C7C7CFFFFFFFFFFFFFFFFFFFFFFFF7C7C7C8C8C8CFFFFFFFFFFFFFFFFFFFFFFFF9A9A9A
000000FFFFFFFFFFFFFFFFFF000000B2B2B2C7C7C7E1E1E14D4D4DA7A7A7BDBDBDFFFFFF
FFFFFF686868A7A7A7E1E1E1BDBDBDFFFFFFA7A7A74D4D4DFFFFFFBDBDBDE9E9E9A7A7A7
F0F0F0FFFFFFFFFFFFFFFFFF0000009A9A9AF0F0F0BDBDBDFFFFFFFFFFFFD9D9D9000000
D9D9D9FFFFFFFFFFFF9A9A9A686868FFFFFFFFFFFFD9D9D9000000C7C7C7E1E1E1D0D0D0
000000B2B2B2C7C7C7E1E1E14D4D4DA7A7A7BDBDBDFFFFFFFFFFFF686868A7A7A7E1E1E1
BDBDBDFFFFFF8C8C8C686868B2B2B2E1E1E1FFFFFF000000A7A7A7FFFFFFB2B2B2F0F0F0
000000B2B2B2C7C7C7E1E1E14D4D4DA7A7A7BDBDBDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B2000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B2FFFFFF
4D4D4D000000F0F0F0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000
7C7C7CFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFE1E1E1000000B2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B2000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE1E1E1000000B2B2B2FFFFFFFFFFFFFFFFFF
FFFFFFE9E9E9D9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFE1E1E1000000B2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF8C8C8CFFFFFFB2B2B27C7C7CFFFFFFFFFFFFFFFFFFFFFFFFC7C7C7E9E9E9
E9E9E9000000BDBDBDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF8C8C8CA7A7A7FFFFFFFFFFFFFFFFFFBDBDBD000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF8C8C8C
A7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD0D0D08C8C8CFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020
202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020
202020202020202020FFFFFFFFFFFF202020202020202020202020202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFDFDFD939393B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B5B5B5B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8939393ECECEC696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECE3E3E3ECECECECECECECECECECECECE
CECECE3E3E3ECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF
3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E
3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3E3E3E3E
3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFE4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3E
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDED3E3E3EEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000ECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F79A9A9A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A9A9A9AF7F7F70000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFF7F7F70000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3F7F7F70000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F1F1F1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE3E3E3A5A5A5
767676767676767676767676777777767676777777777777767676767676767676767676
767676767676767676767676767676A5A5A5E3E3E3FFFFFFFFFFFFE3E3E3A5A5A5767676
767676767676767676777777767676777777777777767676767676767676767676767676
767676767676767676767676A5A5A5E3E3E3FFFFFFFFFFFFEBEBEBA5A5A5565656565656
565656565656575757565656575757575757565656565656565656565656565656565656
565656565656565656A5A5A5EBEBEBFFFFFFFFFFFFFFFFFFFFFFFFF1F1F10000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E1E1E1FCFCFCFBFBFBFCFCFCFCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFBFBFBFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCA5A5A5767676
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D767676A5A5A5FCFCFCFCFCFCA5A5A57676768D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D8D
8D8D8D8D8D8D8D8D8D8D8D8D767676A5A5A5FCFCFCFBFBFBA5A5A55656565B5B5B5B5B5B
5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B
5B5B5B5B5B5B5B5B5B565656A5A5A5FCFCFCFCFCFCFCFCFCFCFCFCE1E1E10000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F77676768D8D8D
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F77676768D8D8DA2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F75656565B5B5B6F6F6F717171
727272727272727272727272727272727272727272727272727272727272727272727272
7272727171716F6F6F5B5B5B565656F7F7F7F7F7F7F7F7F7F7F7F7E0E0E00000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DEDEDEF1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1
F1F1F1F1F1F1F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2
F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1767676B8B8B8
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABB8B8B8767676F1F1F1F1F1F1767676B8B8B8ABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABB8B8B8767676F1F1F1F2F2F25656566F6F6F6F6F6F727272
777777777777777777777777777777777777777777777777777777777777777777777777
7777777272726F6F6F6F6F6F565656F1F1F1F1F1F1F1F1F1F2F2F2DEDEDE0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DCDCDCEBEBEBEBEBEBECECECEBEBEBEBEBEBA7A7A7A2A2A2A2A2A2A1A1A1A1A1A1
A0A0A09F9F9F9D9D9D9F9F9F9F9F9FA0A0A0A0A0A0A1A1A1A1A1A1A7A7A7EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBCECECEC5C5C5C5C5C5D0D0D0EBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBE5E5E5C7C7C7D7D7D7EBEBEBEBEBEBEBEBEBCECECECCCCCCEAEAEAEAEAEA
C8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3EBEBEBE0E0E0C5C5C5C5C5C5CCCCCC
E9E9E9C8C8C8D0D0D0EBEBEBEBEBEBEBEBEBEBEBEBE0E0E0C5C5C5C5C5C5C5C5C5DDDDDD
EBEBEBEBEBEBEAEAEAC8C8C8C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBE0E0E0C5C5C5
C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEAEAEAC8C8C8C5C5C5C5C5C5D7D7D7EBEBEB
EBEBEBE0E0E0C5C5C5C5C5C5CCCCCCE9E9E9CECECEC7C7C7E9E9E9EBEBEBE0E0E0C5C5C5
C5C5C5C5C5C5DDDDDDEBEBEBEAEAEAC8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3
EBEBEBD4D4D4C5C5C5DEDEDEECECECEBEBEBC8C8C8D0D0D0ECECECECECECDBDBDBC6C6C6
E3E3E3E6E6E6C7C7C7C5C5C5C5C5C5C5C5C5CCCCCCEBEBEBECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB767676BBBBBB
B1B1B1B1B1B1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1B1B1B1
B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB767676BBBBBBB1B1B1
B1B1B1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFB1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB565656777777727272747474
777777787878787878787878787878787878787878787878787878787878787878787878
777777747474727272777777565656EBEBEBEBEBEBEBEBEBEBEBEBDCDCDC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DADADAE5E5E5E5E5E5E5E5E5E5E5E58A8A8A9B9B9BACACACE9E9E9E9E9E9E9E9E9
EAEAEAC4C4C4858585ECECECEDEDEDEEEEEEE7E7E7F0F0F0F2F2F2A3A3A3E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E1E1E14B4B4B1010101010105B5B5BC7C7C7CFCFCFE5E5E5E5E5E5
E5E5E5C9C9C91C1C1C727272D4D4D4E5E5E5E2E2E2505050393939C5C5C5DDDDDD262626
515151CFCFCFE5E5E5949494151515A4A4A4DEDEDEB2B2B2101010101010414141D7D7D7
262626515151CFCFCFE5E5E5E5E5E5E5E5E5AFAFAF101010101010101010999999C6C6C6
DEDEDEDDDDDD242424101010101010101010151515A4A4A4DEDEDEB2B2B2101010101010
101010101010151515A4A4A4DEDEDEDDDDDD242424101010101010838383CDCDCDD9D9D9
B2B2B2101010101010414141DBDBDB505050191919BABABAE3E3E3AFAFAF101010101010
101010999999C6C6C6D7D7D7262626515151CFCFCFE5E5E5949494151515A4A4A4DEDEDE
7070701010108E8E8ED9D9D9DDDDDD262626515151CFCFCFE5E5E5949494151515A4A4A4
C2C2C21C1C1C101010101010101010404040C6C6C6C6C6C6E2E2E2E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5767676C0C0C0
B5B5B5B5B5B5FFFFFF727272727272727272727272727272727272272727B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5C0C0C0767676E5E5E5E5E5E5767676C0C0C0B5B5B5
B5B5B5FFFFFF272727272727272727272727272727272727272727272727272727272727
272727B5B5B5B5B5B5B5B5B5C0C0C0767676E5E5E5E5E5E5565656777777777777797979
7A7A7AFFFFFFFFFFFF7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7AFFFFFFFFFFFF
7A7A7A797979777777777777565656E5E5E5E5E5E5E5E5E5E5E5E5DADADA0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D9D9D9DFDFDFDFDFDFDFDFDFDFDFDFDEDEDEA0A0A0EAEAEAE9E9E9E5E5E5EAEAEA
EAEAEAD0D0D0747474EDEDEDEEEEEEADADADD0D0D0F2F2F2F3F3F3A3A3A3DFDFDFDEDEDE
DEDEDEDFDFDFDEDEDE6B6B6B191919C5C5C5D7D7D7252525555555C6C6C6DDDDDDDEDEDE
DEDEDEC3C3C31C1C1C727272D1D1D1DEDEDEDBDBDB4E4E4E393939C4C4C4D6D6D6252525
515151CDCDCDDEDEDE8F8F8F151515A4A4A4D5D5D54E4E4E383838C4C4C4DDDDDDD7D7D7
252525515151CDCDCDDEDEDEDEDEDEC3C3C31B1B1B727272D9D9D98F8F8F151515A5A5A5
C6C6C6252525191919C5C5C5DEDEDE8F8F8F151515A4A4A4D2D2D2252525383838C7C7C7
DBDBDB4E4E4E151515A4A4A4D2D2D2252525383838CFCFCFDBDBDB4E4E4E9F9F9FDBDBDB
4E4E4E383838C4C4C4DDDDDDDBDBDB4E4E4E191919BABABAC2C2C21B1B1B727272D9D9D9
8F8F8F151515A5A5A5C6C6C6252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4D9D9D9
6C6C6C1010109F9F9FDEDEDED7D7D7252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4
BEBEBE1C1C1C727272D1D1D1DBDBDB4E4E4E191919B9B9B9D5D5D5DEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE767676C3C3C3
BBBBBBBBBBBBFFFFFF727272727272727272727272727272727272272727BBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBC3C3C3767676DEDEDEDEDEDE767676C3C3C3BBBBBB
BBBBBBFFFFFF272727BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBFFFFFF
272727BBBBBBBBBBBBBBBBBBC3C3C3767676DFDFDFDEDEDE5656567777777878787B7B7B
303030FFFFFFFFFFFFFFFFFF7D7D7D7D7D7D7D7D7D7D7D7D7D7D7DFFFFFFFFFFFFFFFFFF
3030307B7B7B787878777777565656DEDEDEDEDEDEDFDFDFDEDEDED9D9D90000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D7D7D7D8D8D8D9D9D9D8D8D8D8D8D89090908F8F8F8E8E8EE9E9E9818181515151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000000D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3858585A3A3A3E6E6E6EAEAEAEAEAEA353535
7F7F7FCFCFCFBEBEBE8B8B8B5E5E5E585858686868A3A3A3C9C9C9AAAAAAD3D3D3D3D3D3
D3D3D3D3D3D3A4A4A41010108E8E8ECECECED3D3D3A4A4A41010108E8E8ECCCCCCD3D3D3
D3D3D3B8B8B81B1B1B727272CCCCCCD3D3D3CFCFCF4A4A4A393939C1C1C1CCCCCC242424
515151CACACAD3D3D3888888151515A4A4A4C9C9C9242424383838C1C1C1D3D3D3CCCCCC
242424515151CACACAD3D3D3D3D3D3676767191919B9B9B9D0D0D0CCCCCC242424515151
9D9D9D1010108E8E8ECCCCCCD3D3D3888888151515A4A4A4C9C9C9242424515151CACACA
D3D3D3888888151515A4A4A4868686151515A4A4A4CECECED3D3D3D3D3D3D3D3D3CCCCCC
242424383838C1C1C1D3D3D3CFCFCF4A4A4A191919B9B9B9676767191919B9B9B9D0D0D0
CCCCCC242424515151C3C3C3242424515151CACACAD3D3D3888888151515A4A4A4D0D0D0
D3D3D3D3D3D3D3D3D3D3D3D3CCCCCC242424515151CACACAD3D3D3888888151515A4A4A4
B6B6B61B1B1B727272CCCCCCD3D3D3B8B8B81B1B1B717171CDCDCDD3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3767676CBCBCB
C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6
C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6CBCBCB767676D3D3D3D3D3D3767676CBCBCBC6C6C6
C6C6C6FFFFFF272727C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6FFFFFF
272727C6C6C6C6C6C6C6C6C6CBCBCB767676D3D3D3D3D3D35656567D7D7D7F7F7F838383
838383838383282828FFFFFFFFFFFFFFFFFF838383FFFFFFFFFFFFFFFFFF282828838383
8383838383837F7F7F7D7D7D565656D3D3D3D3D3D3D3D3D3D3D3D3D5D5D5000000A0A0A0
A5A5A5ADADADB6B6B6BFBFBFC8C8C8D2D2D2DADADAE2E2E2E9E9E9EFEFEFF4F4F4F6F6F6
F9F9F9FBFBFBFDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D3D3D3CECECECECECECFCFCFCECECECECECE878787939393D3D3D3DBDBDBADADAD
C1C1C1B0B0B0A9A9A9AAAAAA8D8D8D919191B0B0B0CBCBCBEFEFEFA5A5A5CECECECECECE
CECECECFCFCF8585851010108E8E8ECBCBCBCECECEA0A0A0101010717171CACACACECECE
CECECEB4B4B41A1A1A727272CACACACECECECBCBCB494949393939C1C1C1C7C7C7232323
515151C7C7C7CECECE858585151515A4A4A4C7C7C7232323383838C1C1C1CECECEC7C7C7
232323515151C5C5C5CDCDCDCDCDCD848484151515A4A4A4CACACAC7C7C7232323515151
B1B1B11A1A1A727272C5C5C5CECECE858585151515A4A4A4C6C6C6232323515151C8C8C8
CECECE858585151515A4A4A49F9F9F1010108E8E8EC7C7C7CECECECECECEC8C8C8C6C6C6
232323383838C1C1C1CECECECBCBCB494949191919B9B9B9848484151515A4A4A4CACACA
C7C7C7232323515151C4C4C4232323515151C7C7C7CECECE858585151515A4A4A4CECECE
CECECECECECECECECECFCFCFC8C8C8232323515151C7C7C7CECECE858585151515A4A4A4
B4B4B41A1A1A727272CACACACFCFCFA1A1A11010108E8E8ECECECECFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE767676CECECE
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCECECE767676CECECECECECE767676CECECECBCBCB
CBCBCBFFFFFF272727CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBFFFFFF
272727CBCBCBCBCBCBCBCBCBCECECE767676CFCFCFCECECE5656567D7D7D828282858585
848484858585858585282828FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF282828858585858585
8484848585858282827D7D7D565656CECECECECECECECECECECECED3D3D3000000A8A8A8
ADADADB3B3B3BBBBBBC3C3C3CCCCCCD5D5D5DDDDDDE4E4E4EBEBEBF0F0F0F4F4F4F7F7F7
F9F9F9FCFCFCFDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D0D0D0C9C9C9C9C9C9C9C9C9C9C9C97373739494947E7E7E797979A0A0A0D1D1D1
ADADADA5A5A5A6A6A68585855E5E5EADADADE5E5E5F6F6F6F8F8F8A4A4A4C9C9C9C8C8C8
C8C8C8C8C8C88181811010108E8E8EC7C7C7C8C8C89C9C9C101010717171C7C7C7C8C8C8
C7C7C7AEAEAE1A1A1A717171C7C7C7C8C8C8C5C5C5464646383838C0C0C0C2C2C2222222
383838C1C1C1C2C2C2464646191919BCBCBCC1C1C1232323383838BFBFBFC5C5C5C0C0C0
222222151515A5A5A5B1B1B1ABABABAEAEAE1A1A1A737373C5C5C57F7F7F1010108F8F8F
C5C5C5464646191919B9B9B9C5C5C57F7F7F151515A4A4A4C1C1C1222222515151C7C7C7
C8C8C8818181151515A4A4A4C1C1C1222222393939C0C0C0C5C5C5676767A5A5A5C1C1C1
232323383838BFBFBFC5C5C5C4C4C4464646191919B9B9B9AFAFAF1A1A1A737373C5C5C5
7F7F7F1010108F8F8FC2C2C2222222383838C1C1C1C2C2C2464646191919BCBCBCC8C8C8
C8C8C8C8C8C8C8C8C8C9C9C9C2C2C2222222383838C1C1C1C2C2C2464646191919BCBCBC
B0B0B01A1A1A717171C5C5C5C2C2C2464646191919BDBDBDC9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9
C9C9C9C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8767676D3D3D3
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDD3D3D3767676C8C8C8C8C8C8767676D3D3D3CDCDCD
CDCDCDFFFFFF272727CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDFFFFFF
272727CDCDCDCDCDCDCDCDCDD3D3D3767676C8C8C8C8C8C8565656828282838383878787
878787878787878787878787282828FFFFFFFFFFFFFFFFFF282828878787878787878787
878787878787838383828282565656C9C9C9C9C9C9C9C9C9C8C8C8D0D0D0000000B2B2B2
B6B6B6BBBBBBC2C2C2C9C9C9D1D1D1D9D9D9E0E0E0E7E7E7EDEDEDF1F1F1F5F5F5F8F8F8
FAFAFAFCFCFCFDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000CFCFCFC5C5C5C5C5C5C5C5C5C5C5C5C5C5C5979797B7B7B7ECECECB9B9B98B8B8B
B7B7B7AAAAAAA4A4A47E7E7E717171C4C4C4D9D9D9F6F6F6F9F9F9A5A5A5C5C5C5C5C5C5
C5C5C5C5C5C59999991010108D8D8DC6C6C6C5C5C59999991010108E8E8EC6C6C6ACACAC
5C5C5CADADAD1A1A1A717171C6C6C6C5C5C5C2C2C2454545383838BFBFBFBFBFBF222222
5D5D5D101010101010151515A4A4A4C2C2C2454545101010101010101010515151BFBFBF
2222224E4E4E454545101010515151C5C5C5ACACAC1A1A1A1010101010108D8D8DC5C5C5
C5C5C5C2C2C2454545101010101010101010151515A4A4A4BFBFBF222222515151C5C5C5
C5C5C57F7F7F151515A4A4A4C5C5C5C2C2C2454545101010101010393939BCBCBC454545
101010101010101010515151616161101010191919B9B9B9C5C5C5ACACAC1A1A1A101010
1010108D8D8DC5C5C5BFBFBF2222225D5D5D101010101010151515A4A4A4C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5BFBFBF2222225D5D5D101010101010151515A4A4A4C5C5C5
ACACAC1A1A1A101010101010101010191919B9B9B9C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5767676D5D5D5
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D5D5D5767676C5C5C5C5C5C5767676D5D5D5D1D1D1
D1D1D1FFFFFF272727D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF
272727D1D1D1D1D1D1D1D1D1D5D5D5767676C5C5C5C5C5C5565656828282878787898989
898989898989898989898989FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF898989898989898989
898989898989878787828282565656C5C5C5C5C5C5C5C5C5C5C5C5CFCFCF000000BCBCBC
BFBFBFC3C3C3C9C9C9D0D0D0D7D7D7DDDDDDE4E4E4EAEAEAEFEFEFF3F3F3F6F6F6F8F8F8
FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
000000CDCDCDC1C1C1C1C1C1C1C1C1C1C1C1757575999999A7A7A7EDEDEDC9C9C9737373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000000CCCCCCBABABABDBDBDBDBDBDBEBEBEBDBDBD9E9E9EE3E3E3B6B6B6969696828282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000000CACACAB8B8B8BABABABABABABABABA7C7C7C8D8D8D787878999999F0F0F0C2C2C2
A7A7A7CBCBCBBCBCBCA3A3A36D6D6D7D7D7DACACACFBFBFBFCFCFCA6A6A6BABABABABABA
BABABABBBBBBBABABABABABA5B5B5B101010101010373737B4B4B4BABABABABABABABABA
BABABABABABA5B5B5B6B6B6BBABABABABABAB7B7B7424242373737B4B4B4BABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABA7777771414149B9B9BBABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABA5B5B5B191919AFAFAFBABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
A2A2A21919196B6B6BBABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA
BABABABABABABABABABABABABABABABABABABABABABABABABABABABABABA767676D8D8D8
DADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADA
DADADADADADADADADADADADADADADAD8D8D8767676BABABABABABA767676D8D8D8DADADA
DADADAFFFFFF727272727272727272727272727272727272727272727272727272727272
272727DADADADADADADADADAD8D8D8767676BABABABBBBBB5656568B8B8B8F8F8F8E8E8E
8F8F8FFFFFFFFFFFFFFFFFFF2828288F8F8F8F8F8F8F8F8F282828FFFFFFFFFFFFFFFFFF
8F8F8F8E8E8E8F8F8F8B8B8B565656BABABABABABABABABAB8B8B8CACACA000000D9D9D9
DADADADDDDDDE0E0E0E4E4E4E8E8E8ECECECEFEFEFF3F3F3F6F6F6F8F8F8FAFAFAFCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B88686869E9E9ED4D4D4F0F0F0F2F2F2CECECE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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8868686949494F2F2F2F3F3F3A0A0A0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000000C9C9C9B5B5B5B5B5B5B5B5B5B5B5B5777777A0A0A0E0E0E0F3F3F3E5E5E5ECECEC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000000C9C9C9BABABAB1B1B1B1B1B1B1B1B1B1B1B18D8D8DAAAAAAAAAAAAAAAAAAAAAAAA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000000C3C3C3C9C9C9AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE8D8D8D767676
CECECEFBFBFBFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFBFBFBCECECE7676768D8D8DAEAEAEAEAEAE8D8D8D767676CECECE
FBFBFBFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFBFBFBCECECE7676768D8D8DAEAEAEAEAEAE999999565656B5B5B5CBCBCB
DFDFDFE8E8E8E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8
DFDFDFCBCBCBB5B5B5565656999999AEAEAEAEAEAEAEAEAEC9C9C9C3C3C3000000F6F6F6
F6F6F6F7F7F7F8F8F8F8F8F8F9F9F9FAFAFAFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFD
FEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000C3C3C3D4D4D4B7B7B7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A78D8D8D
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676767676767676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676767676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D565656565656
565656565656565656565656565656565656565656565656565656565656565656565656
5656565656565656568D8D8DA7A7A7A7A7A7A7A7A7B7B7B7D4D4D4C3C3C3000000F9F9F9
F9F9F9F9F9F9FAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000AAAAAAE7E7E7D4D4D4BEBEBEB3B3B3AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAB3B3B3BEBEBED4D4D4E7E7E7AAAAAA000000FBFBFB
FBFBFBFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
0000008A8A8AD9D9D9FCFCFCE4E4E4D2D2D2CCCCCCBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBECCCCCCD2D2D2E4E4E4FCFCFCD9D9D98A8A8A000000FCFCFC
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000696969979797D9D9D9FCFCFCF2F2F2ECECECE3E3E3D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9E3E3E3ECECECF2F2F2FCFCFCD9D9D9979797696969000000FDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFF696969979797D3D3D3FCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFCFCFCFCFCFCD3D3D3979797696969FFFFFF000000FDFDFD
FDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFF6969696969698A8A8A979797AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAA9797978A8A8A696969696969FFFFFFFFFFFF000000FEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{368, 319},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005`0001?b000`400?l00000o`00003oo`3oooma
0?ooo`00o`3oooma0?ooo`003P3ooooo0?knoU@0o_kn3`3oool000`0oooo103no_ko0?gmoE00oOgm
103no_h=0?ooo`002P3oool30?kno_l0oOgmEP3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo4X0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?clo?l0no_kC03kno/30?cl
o0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinDd0
nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_I<0?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`4
0?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gmo@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0
moOg0?KfmP3emOD00P3dm?Co0??cldX0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm00X0o?cl0?_kn`3inOT0
n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^k^i:0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0
m_Kf0?Shn03inOT0no_k0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj4X0j>SX00/0jNWY0>_[j`3]
kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0030?gmo@80o_kn103oool000<0oooo0100
o_kn0?gmo@3moOd0oOgm0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN5:0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3kno/0
o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`000P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0
m_Kf0??cl`3_knl0k>c/0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SHB03Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`00`3moOd20?knoP<0oooo
00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;R
hP3MgMd0fMWI0=GEe@3Bd]80d=3@o`3?clm80<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0oooo0040oooo00000`3o
ool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3A
dM40c<c<0<S8b036a/Ko0<G5aDP0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo0?knoP3no_h00P3moOd0
3`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl
_03o0;^k^dP0^k^k00l0_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gmo@020?gmo@0?0?clo03j
n_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a
B02a/K403`2b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd00P3moOd03`3lo?`0nOWi
0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYTP0
YZJV00l0Z:RX0:f][@2c/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080oOgm00l0no_k0?Win@3f
m_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S80;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYi809jN
WP0?0:2PX02UYJD0[Jf]0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0o?cl0?_kn`3inOT0m_Kf
0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l_;`0/[:b0:RXZ02PX:00o`2JVYY:09ZJVP0?0:2P
X02XZ:P0/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000005`000000100VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00003
0000003lo?`0o?cl0?l0o?clF03lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`00
0000o?cl0:^[Z`3o0:^[ZeL0Zj^[0180o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFUF06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj8@2OWil00`2m_Kd0k>c/0>OWi`0[0>OWi`030>k^kP2=SHd0WinO00`0WinO00<0SHf=
0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3Ng]h0fm_K
00`0fm_K00@0g]kN09JFUP2FUYH0fm_K8@3:b/X00`3Kfm/0Xj>S0?_kn`0[0?_kn`0309JFUP3Ng]h0
fm_K00`0fm_K00<0g]kN09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo8@3@d=000`3oool0UYJF0?_kn`0[0?_kn`0309JFUP3oool0c/k>00`0c/k>00<0oooo
09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY
09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF
09JFUP3ooolQ0<o?c`030?ooo`2FUYH0no_k02/0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ0000000=3@d0d];B0>3P
h03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3Lg=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph
0=WIf@0F0>3Ph00309JFUP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM40103@d=090=7Ad@<0d=3@0P3AdM440=3@d0P0dM7A00<0oooo09JF
UP3jn_X0:`3jn_X00`2FUYH0oooo0<k>cP040<k>cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JF
UP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000
003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3P
h01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0103>c/h303hn?PD0c/k>
00@0oooo09JFUP2FUYH0oooo2P3Fe]H20=OGe`080=KFeP2EUID0e]KF09FEU@3Fe]H0UIFE0=KFeP2E
UID80=KFeP80emOG0`3Fe]H00`3oool0UYJF0?Win@0[0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n
?Sh60<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>00
0`3oool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph034a<@0B4Q80=;B
dP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY
09jNWQH0h>3P00<0UYJF0?ooo`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3L
g=`90=gMg@090?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID0g=cL00T0gMgM0P3Lg=`00`3M
gMd0oooo09JFUP0/0?Win@0309JFUP3oool0clo?00@0clo?0`0n?Sh50<o?c`030?ooo`2FUYH0h>3P
00P0h>3P0P2c/k<06P3oool0h>3P0;>c/`2c/k<0oooo0>3Ph01YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B0>3P
h03IfMT09RHV0=;BdP@0h>3P00L0a<C404Q8B03Bd]806QXJ0000000=3@d0d];B00X0h>3P1`000005
0>3Ph00309JFUP3oool0e=CD0080e=CD0`0n?Sh70=CDe0040?ooo`2FUYH0UYJF0?ooo`<0g=cL2@3M
gMd02@3oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0=cLg0090=gMg@80g=cL00<0gMgM0?oo
o`2FUYH0;03hn?P00`2FUYH0oooo0=CDe0050=CDe0<0?Shn103De=@00`3oool0UYJF0>3Ph0080>3P
h080/k>c0P3Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>005`34a<@0
B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV9P3Bd]80
g=cL04U9B@0000003@d=0=;BdP1YJFT0WYjN00l0h>3P1@0000060>3Ph00409JFUP3oool0fm_K0=_K
f`<0?Shn203Kfm/01@3oool0UYJF09JFUP3oool0i>CT0080hn?S0P3Ti>@50>?Sh`80i>CT00T0oooo
09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@3Shn<00P3Ti>@50>?Sh`<0i>CT00@0hn?S0>CTi03o
ool0UYJF;03gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3P
h00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0
JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80
h>3P0<C4a018B4P0d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0
a<C404Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0?Shn1`3Ti>@01@3o
ool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X70>WYj@090?ooo`2EUID0oooo09FEU@3oool0UIFE
0?ooo`2EUID0jNWY0080j^[Z1`3YjNT01@3Zj^X0jNWY0>WYj@3oool0UYJF02`0mOGe00<0UYJF0?oo
o`3Ti>@01@3Ti>@303hn?P@0i>CT00<0oooo09JFUP3Ph>003@3Ph>020?ooo`0E0>3Ph01YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0
h>3P00<0oooo0>3Ph03Ph>000`3Ph>006@2>SXh0000003Ph>03IfMT0h>3P06UYJ@2NWYh0h>3P08j>
SP1DE5@0h>3P06UYJ@2NWYh0h>3P08j>SP1DE5@0h>3P0:f][@1DE5@0h>3P06UYJ@1nOWh0h>3P06UY
J@2NWYh0403Ph>000`000000h>3P0>3Ph0060>3Ph00309JFUP3oool0kNg]00<0kNg]0`0n?Sh60>g]
k@050?ooo`2FUYH0UYJF0?ooo`3Zj^X00P3YjNT20>[ZjPL0jNWY00T0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@3YjNT00P3Zj^X70>WYj@050>[ZjP3YjNT0jNWY0?ooo`2FUYH0;03dm?@00`2F
UYH0oooo0>g]k@040>g]k@<0?Shn1@3]kNd00`3oool0UYJF0>3Ph00<0>3Ph080/k>c01H0oooo0>3P
h01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00I0=WIf@1BDU80fMWI0>3Ph03IfMT06QXJ0000
000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0>3P
h034a<@07AdM0=;BdP0H0>3Ph00309JFUP3oool0m?Cd00@0m?Cd0`0n?Sh50?Cdm0040?ooo`2FUYH0
UYJF0?ooo``0kNg]00L0oooo0>g]k@3oool0kNg]0?ooo`3]kNd0oooo00h0kNg]00<0oooo09JFUP3a
lO40:`3alO400`2FUYH0oooo0?Cdm0030?Cdm0<0?Shn1P3dm?@00`3oool0UYJF0>3Ph00<0>3Ph080
/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0o?cl00D0o?cl
00<0?Shn0?clo03lo?`0103lo?`0103oool0UYJF09JFUP3ooolQ0>g]k@030?ooo`2FUYH0l?3`02/0
l?3`00<0UYJF0?ooo`3lo?`0103lo?`00`0n?Sh0o?cl0?clo0050?clo0030?ooo`2FUYH0h>3P0100
h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[
Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`2FUYH0oooo0?ooo`0=0?ooo`80UYJF
00<0oooo0?Ogm`3gmoL07`3gmoL00`3oool0UYJF0>o_k`0[0>o_k`0309JFUP3oool0oooo00d0oooo
00<0UYJF0>3Ph03Ph>003`3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3G
emL0oooo00`0oooo00@0emOG09JFUP2FUYH0n_[j8@3oool00`3jn_X0UYJF0>o_k`0[0>o_k`0309JF
UP3GemL0oooo00`0oooo00<0emOG09JFUP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph0030=[JfP2BTY80][Jf00/0]KFe00D0];Bd08n?S`3Jf]X0e]KF0;2`/00Q0;Rh^0030:VY
Z@3>c/h0k^k^02/0k^k^00<0f][J09:BTP2f][H02`2e]KD00`2d];@0Shn?0=[JfP0A0>3Ph00D06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYU`2N
WYh00`3oool0h>3P0>3Ph00e0>3Ph140k>c/9@3[jn//0>OWia40k>c/4@3Ph>00501YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool0
0`3/k>`0k^k^08f=S@0<09nOW`0F08f=S@3Vi^H0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood<0oooo00<0k>c/09JFUP3Ng]h0303Kfm/0
5P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooom30?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool0
0`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood<0oooo00<0k>c/09JFUP3oool0303>c/h0
5P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooom30?ooo`030>c/k02FUYH0oooo00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h0
5P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooom30?ooo`030>c/k02FUYH0oooo00@0c/k>0`0n?Sh50<k>cP0F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oood<0oooo00<0k>c/09JFUP3oool00`3?cll503hn?P@0clo?01H0oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool0
0`3/k>`0UYJF0?ooo`020=CDe0<0?Shn00@0e=CD03hn?P0n?Sh0?Shn0`3De=@05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oom30?ooo`040>c/k02FUYH0oooo0=_Kf`<0?Shn0`3Kfm/303hn?P80fm_K01H0oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
@`3oool00`3/k>`0UYJF0?ooo`020>CTi00303hn?P3Ti>@0i>CT00<0i>CT01X0?Shn0>CTi03Ti>@0
i>CT0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?oood<0oooo00<0k>c/09JFUP3oool0303]kNd05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?oo
o`030>c/k02FUYH0oooo00`0m?Cd01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool00`3/k>`0UYJF0?ooo`0<0?cl
o00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?oood<0oooo00<0k>c/09JFUP3oool03@3oool05@2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`03
0>c/k02FUYH0emOG00`0oooo01H0emOG09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool0103/k>`0f][J09:BTP2f][H:0;Fe
]@0G0;Bd]02?Shl0f][J06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
od<0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo@`3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool/0?ooo`D0820P1@3oool2020P
8080oooo1@0P82040?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFU@0?ooo`060;:b/P000000J6QX08b<S02b/[80d=3@f@3o
ool00`0P8200oooo0?ooo`020?ooo`<0820P1P3oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFU@0?ooo`030000003QhN40oooo0080oooo00<0_Kfm0;:b/P3oool0e`3oool0100P8200oooo0?oo
o`3oool4020P80H0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYD03oool00`37alL0hN7Q
0?ooo`020?ooo`030<O7a`000000oooo0=L0oooo00<0820P0?ooo`3oool01@0P82060?ooo`03020P
803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJE40oooo00<0S8b<0000000000000P0000000`1=CDd0oooo
0?ooo`3F0?ooo`03020P803oool0820P00D0820P1P3oool00`0P8200oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTI0?ooo`0:0>7Qh@2WYjL000000000002WYjL0fMWI0?ooo`2<S8`0CDe=0?3`l0D0000000<0
J6QX0?ooo`3oool02P3oool01P3@d=00O7al000000000000S8b<0>7Qh@<0oooo00L0O7al09ZJVP2b
/[80VYZJ08b<S03QhN40/[:b0080000000H0alO70;:b/P000000CDe=0<O7a`3@d=02000000040;:b
/P3oool0000008b<S080/[:b00<0_Kfm0>WYj@3oool00`3oool02P2b/[8000000:NWY`3YjNT0oooo
09ZJVP3YjNT0oooo0>7Qh@2WYjL2000000030:NWY`3YjNT0oooo0080CDe=00l0fMWI0;:b/P1=CDd0
d=3@0?ooo`000000J6QX0;fm_@3`l?00000009ZJVP2m_Kd000000:NWY`3`l?000P3oool03P37alL0
00000000002JVYX0jNWY0?ooo`1XJ6P000000?3`l03oool0YjNW0000003YjNT0oooo0P000000202m
_Kd0oooo04e=C@1=CDd0fMWI0;:b/P1=CDd0d=3@0P3oool00`000000J6QX0;fm_@020?3`l00904e=
C@2JVYX0jNWY0?ooo`2b/[80000007alO03QhN40oooo0080CDe=00@0fMWI0;:b/P1=CDd0d=3@Q03o
ool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3o
ool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY6`3oool2000000<0oooo00D0CDe=0000003YjNT0hN7Q04e=C@030;:b/P0308b<
S03IfMT0oooo00T0oooo00/0alO7000000000000fMWI0?ooo`3YjNT0VYZJ0>WYj@3oool0l?3`0<O7
a`030?ooo`0506QXJ01=CDd0oooo0000001lO7`00P3oool01P000000O7al0?ooo`3oool0O7al0000
0080oooo0P3@d=090?ooo`060:NWY`3`l?00oooo0?ooo`1=CDd0O7al0`3oool00`1=CDd000000>WY
j@020?ooo`0=04e=C@1XJ6P0_Kfm0;:b/P000000_Kfm0=3@d01XJ6P000000?3`l02WYjL0S8b<0000
00020>WYj@060;:b/P2<S8`0oooo0?ooo`000000CDe=0P3`l?00:`2WYjL0l?3`0<O7a`000000alO7
0?ooo`3YjNT000000:NWY`3oool0YjNW0000003QhN40_Kfm04e=C@1XJ6P0_Kfm0;:b/P000000_Kfm
0=3@d03oool0J6QX0000003`l?00YjNW0?ooo`1=CDd0J6QX0<O7a`3QhN40000006QXJ03oool0YjNW
0?3`l01=CDd0J6QX0;fm_@2b/[8000000;fm_@3@d=00P`3oool00`0P8200oooo0?ooo`040?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6`3oool2000000<0
oooo00H0l?3`0>WYj@3oool0oooo0=3@d02JVYX30?ooo`030?3`l03oool0oooo00P0oooo00<0CDe=
0000002b/[800`3oool00`3`l?00YjNW0?ooo`020?ooo`080<O7a`2JVYX0CDe=0000001XJ6P0oooo
0000001lO7`20?ooo`060000001lO7`0oooo0?ooo`1lO7`000000P3oool01P2JVYX000000:NWY`2W
YjL0S8b<0>7Qh@D0oooo00@0hN7Q0=3@d03oool0oooo0P0000001P2m_Kd0oooo0?ooo`2JVYX00000
0<O7a`80oooo01D0YjNW0000003oool0alO70000002JVYX0jNWY0;fm_@000000hN7Q0?ooo`2m_Kd0
00000;fm_@3oool0jNWY0000003@d=00oooo0000001lO7`0103oool05`3`l?00000004e=C@3`l?00
oooo09ZJVP000000hN7Q0>WYj@000000VYZJ0?3`l02WYjL000000?ooo`37alL0000009ZJVP3YjNT0
oooo0;fm_@000000hN7Q0080oooo00H0/[:b0000003IfMT0oooo04e=C@1lO7`30?ooo`070:NWY`00
0000oooo0<O7a`000000VYZJ0>WYj@230?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3o
ool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooo`800000203oool00`2b/[80
VYZJ0?ooo`0<0?ooo`80000000<0fMWI0?ooo`3oool00P3oool0102b/[80oooo0?ooo`1XJ6P20000
000608b<S02m_Kd0l?3`0?ooo`000000O7al0P3oool01P000000O7al0?ooo`3oool0O7al00000080
oooo00H0000006QXJ03oool0oooo06QXJ01=CDd60?ooo`030;:b/P3oool0hN7Q0080000000H0alO7
0>WYj@3oool0alO70000002JVYX20?ooo`0<0?3`l0000000hN7Q0?ooo`1=CDd000000?ooo`3YjNT0
00000:NWY`3oool0jNWY0P0000005P3QhN40oooo04e=C@2<S8`0oooo09ZJVP000000CDe=08b<S02m
_Kd0l?3`0=WIf@2JVYX000000<O7a`3oool0fMWI0000003@d=00l?3`09ZJVP0000020?3`l0050000
003QhN40oooo04e=C@0000000P3oool00`3YjNT000000:NWY`020?ooo`070?3`l0000000VYZJ0?oo
o`2b/[8000000?3`l0020?ooo`060?3`l0000000hN7Q0?ooo`1=CDd00000Q03oool00`0P8200oooo
0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
6`3oool2000000T0oooo00<0J6QX0;:b/P3oool02`3oool2000000030=WIf@3oool0oooo0080oooo
00D0O7al0?ooo`3oool0000006QXJ0030?ooo`080;:b/P3oool0000007alO03oool0jNWY0000001=
CDd20?3`l08000000P3oool01`1lO7`0J6QX0?ooo`3oool0J6QX0000003YjNT01@3oool02`2m_Kd0
oooo0=WIf@000000O7al0?ooo`2WYjL0oooo0>7Qh@000000CDe=00<0oooo00T0alO709ZJVP3oool0
alO70000003@d=00oooo06QXJ01=CDd00P3oool02`1=CDd000000:NWY`3@d=00J6QX0000003oool0
l?3`06QXJ01=CDd0l?3`0080J6QX00L0alO70>WYj@000000/[:b0:NWY`3YjNT0J6QX0080alO700X0
/[:b0000002m_Kd0oooo0<O7a`2JVYX0oooo0<O7a`000000d=3@0P3oool01`1XJ6P0CDe=0?ooo`3`
l?00_Kfm09ZJVP0000000P3`l?002`1XJ6P0alO70?3`l02m_Kd0l?3`0<O7a`2JVYX0oooo0<O7a`00
0000d=3@08<0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJA/0oooo0P0000090?ooo`030=3@d0000000fMWI00/0oooo0P00
00000`3IfMT0oooo0?ooo`020?ooo`0D07alO03oool0oooo0>7Qh@2<S8`0YjNW0;:b/P1XJ6P0VYZJ
0:NWY`000000O7al0?ooo`2m_Kd0S8b<06QXJ02b/[80CDe=00000037alL20?ooo`070>7Qh@2<S8`0
S8b<0;:b/P2JVYX0CDe=0=3@d0050?ooo`090<O7a`3YjNT0/[:b0000002JVYX0oooo0>7Qh@2m_Kd0
oooo0080000000D0l?3`0?ooo`3oool0oooo0=WIf@020>7Qh@0606QXJ02JVYX0S8b<04e=C@000000
O7al0P2WYjL01@000000hN7Q0;:b/P000000S8b<0080oooo0140l?3`0;:b/P37alL0VYZJ07alO03Y
jNT0/[:b0000001=CDd0l?3`0:NWY`000000/[:b0?ooo`2m_Kd0CDe=0;:b/P020?ooo`0G0=WIf@3Q
hN40hN7Q06QXJ02JVYX0oooo08b<S01=CDd0000007alO02WYjL0l?3`0;:b/P1=CDd0hN7Q0?ooo`3`
l?00YjNW08b<S01=CDd0jNWY0?ooo`3IfMT00P3QhN400`1XJ6P0VYZJ0?ooo`220?ooo`03020P803o
ool0oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTK0?ooo`8000002P3oool00`000000J6QX0?ooo`0:0?ooo`80000000<0fMWI0?ooo`3oool00P3o
ool00`1lO7`0oooo0?ooo`070?ooo`030>WYj@37alL0oooo0080oooo00<0l?3`0?ooo`3oool0403o
ool01@3`l?00alO70:NWY`000000/[:b0080oooo00D0_Kfm0>WYj@2<S8`000000=3@d0040?ooo`0<
0?3`l037alL0_Kfm0000003oool0jNWY0000002WYjL0oooo0=WIf@000000YjNW6@3oool0103`l?00
alO70;fm_@0000020?ooo`030>WYj@000000YjNW00d0oooo00@0l?3`0<O7a`2m_Kd00000P`3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY6`3oool2000000T0oooo00<0l?3`0000000000002`3oool2000000030=WIf@3oool0
oooo0080oooo00<0O7al0?ooo`3oool0803oool0102b/[80O7al0000003@d=030?ooo`040:NWY`2m
_Kd000000:NWY`X0oooo00H0VYZJ0:NWY`3oool0oooo0000001XJ6PP0?ooo`0309ZJVP2WYjL0oooo
09<0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo
0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJAT0oooo00@0jNWY0:NWY`00000000001P3oool01P2m_Kd0alO70=WIf@1l
O7`0000007alO0/0oooo0P0000000`3IfMT0oooo0?ooo`020?ooo`0307alO03oool0oooo00L0oooo
0P1XJ6PG0?ooo`040;fm_@00000000000>WYj@<0oooo00@0hN7Q08b<S0000000J6QX3P3oool00`2J
VYX000000>WYj@0T0?ooo`80J6QXS`3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo
00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6P3oool00`3`l?00alO704e=C@070?oo
o`050;fm_@1=CDd0000007alO03QhN402@3oool0303IfMT0S8b<000000000000S8b<0=WIf@3oool0
jNWY0:NWY`000000S8b<0=WIf@L0oooo00<0S8b<06QXJ03oool05@3oool0102m_Kd0J6QX00000000
00050?ooo`050;:b/P00000000000:NWY`3YjNT02`3oool0103YjNT0O7al0000002m_KdT0?ooo`80
J6QXS`3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3o
ool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`3o
ool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo;@3oool4020P80H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo;03oool5020P8080oooo1@0P82060?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooob`0oooo1@0P82050?ooo`80820P1P3oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`3oool00P3oool3020P80H0oooo00<0820P0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3oool`0?ooo`04020P803oool0oooo0?ooo`@0820P1P3oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`3oool01@0P82060?oo
o`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo020P8005020P
80H0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6@3oool03@3QhN40YjNW000000000000
YjNW0=WIf@3oool0S8b<04e=C@3`l?00oooo0>7Qh@2WYjL00P0000000`2WYjL0fMWI0?ooo`070?oo
o`060000002WYjL0fMWI0?ooo`3YjNT0VYZJ0P0000000`1lO7`0fMWI0?ooo`030?ooo`0>0>WYj@2<
S8`000000000002<S8`0VYZJ04e=C@2JVYX0l?3`0?ooo`2b/[80000004e=C@2b/[820?ooo`0<0;:b
/P00000000000<O7a`3oool0alO707alO0000000CDe=09ZJVP3YjNT0/[:b0P0000000`2b/[80oooo
0?ooo`050?ooo`050=WIf@2JVYX0O7al0:NWY`2b/[800P2WYjL00`3YjNT0oooo0?ooo`030?ooo`03
06QXJ03`l?00oooo0080oooo00X0alO707alO0000000CDe=09ZJVP3YjNT0/[:b0000001=CDd0/[:b
1@3oool01@1XJ6P0l?3`0?ooo`3oool0/[:b0080000000P0alO70?ooo`37alL0O7al0000001=CDd0
VYZJ0>WYj@80oooo00H0jNWY06QXJ03oool0oooo0=3@d02WYjL80?ooo`080<O7a`2<S8`0YjNW0:NW
Y`2<S8`0alO70?ooo`2b/[820000000>0;:b/P3oool0oooo0?ooo`2b/[8000000:NWY`3YjNT0oooo
09ZJVP3YjNT0oooo0>7Qh@2WYjL2000000030:NWY`3YjNT0oooo0080CDe=0100fMWI0;:b/P1=CDd0
d=3@0?ooo`000000J6QX0;fm_@3`l?00oooo0000002JVYX0_Kfm0000002WYjL0l?3`0P3oool03@37
alL000000000002JVYX0jNWY06QXJ0000000l?3`0?ooo`2WYjL000000>WYj@3oool00P0000000`2m
_Kd0oooo0?ooo`0204e=C@0C0=WIf@2b/[80CDe=0=3@d03oool0000006QXJ02m_Kd0l?3`0?ooo`3`
l?00CDe=09ZJVP3YjNT0/[:b0000001lO7`0hN7Q0?ooo`0204e=C@040=WIf@2b/[80CDe=0=3@d2/0
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJA/0oooo0P0000030?ooo`0304e=C@000000jNWY00<0oooo0P00000:0?ooo`03
09ZJVP3`l?00oooo0080oooo00@0alO7000000000000l?3`1`3oool2000000050=WIf@3oool0VYZJ
0000002JVYX00P3oool00`000000O7al0?ooo`030?ooo`0=0000001lO7`0oooo0;:b/P000000J6QX
0=3@d03oool0l?3`0:NWY`3oool0000007alO0070?ooo`030:NWY`000000S8b<00@0oooo00<0YjNW
00000037alL00`3oool00`37alL000000;:b/P020?ooo`0:0;:b/P000000J6QX0=3@d03oool0l?3`
0:NWY`3oool0000007alO0D0oooo00<0alO70000002b/[800`3oool02P000000O7al0?ooo`2b/[80
000006QXJ03@d=00oooo0?3`l02WYjL20?ooo`070;:b/P000000alO70?ooo`2<S8`000000?3`l006
0?ooo`030;:b/P000000d=3@0080oooo00H0d=3@0000002b/[80oooo0000001lO7`50?ooo`060:NW
Y`3`l?00oooo0?ooo`1=CDd0O7al0`3oool00`1=CDd000000>WYj@020?ooo`0>04e=C@1XJ6P0_Kfm
0;:b/P000000_Kfm0=3@d01XJ6P000000?3`l02WYjL0oooo08b<S00000020>WYj@060;:b/P2<S8`0
oooo0?ooo`000000CDe=0P3`l?006@2WYjL0^k^k00000037alL0oooo0>WYj@000000YjNW0?ooo`2W
YjL000000>7Qh@2m_Kd0oooo04e=C@1XJ6P0_Kfm0;:b/P000000_Kfm0=3@d01XJ6P000000?3`l02W
YjL00P3oool03`1=CDd0J6QX0<O7a`000000J6QX0?ooo`2WYjL0l?3`04e=C@1XJ6P0_Kfm0;:b/P00
0000_Kfm0=3@d00Z0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooo`8000000`3oool00`3`l?00jNWY0?ooo`030?oo
o`8000002P3oool00`3IfMT0alO70?ooo`020?ooo`0307alO0000000YjNW00P0oooo0P0000000`3I
fMT0oooo0=3@d00200000080oooo00<0000007alO03oool00`3oool01P000000O7al0?ooo`000000
CDe=0?3`l0D0oooo00<0000007alO03oool01@3oool0103@d=0000000000003`l?050?ooo`0304e=
C@000000fMWI0080oooo00P0CDe=0000002JVYX0l?3`0?ooo`000000CDe=0?3`l0D0oooo00<00000
07alO03oool0103oool0101=CDd0000009ZJVP3`l?020?ooo`060000001lO7`0oooo0000001=CDd0
l?3`1@3oool0203`l?0000000000001lO7`0l?3`0000001=CDd0/[:b1P3oool00`1=CDd000000?oo
o`030?ooo`050000001=CDd0oooo0000001lO7`01@3oool0103QhN40d=3@0?ooo`3oool200000006
0;fm_@3oool0oooo09ZJVP000000alO70P3oool02P2WYjL000000?ooo`37alL0000009ZJVP3YjNT0
_Kfm0000003QhN420?ooo`0:0;fm_@000000_Kfm0?ooo`3YjNT000000=3@d03oool0000007alO0<0
oooo01L0l?3`0000001=CDd0l?3`0?ooo`2JVYX000000>7Qh@3YjNT0000009ZJVP3`l?00oooo0:NW
Y`000000oooo0<O7a`000000VYZJ0>WYj@2m_Kd000000>7Qh@030?ooo`050;:b/P000000fMWI04e=
C@1lO7`00`3oool01`2WYjL000000?ooo`37alL0000009ZJVP3YjNT0:P3oool00`0P8200oooo0?oo
o`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6`3o
ool2000000T0oooo0P00000;0?ooo`@0O7al0P0000000`3YjNT0oooo0?ooo`060?ooo`80000000D0
fMWI0?ooo`3@d=00000004e=C@020?ooo`030000001lO7`0oooo00<0oooo00D0000007alO03oool0
000004e=C@050;:b/P030?ooo`000000O7al00H0oooo00<0O7al0000002<S8`01P3oool0302JVYX0
000008b<S03oool0fMWI0000002JVYX0jNWY0;:b/P3oool0000004e=C@D0/[:b00<0oooo0000001l
O7`0103oool01@3IfMT0000009ZJVP3YjNT0/[:b0080oooo00D0000007alO03oool0000004e=C@05
0;:b/P0:0?ooo`2b/[8000000;:b/P3IfMT0J6QX00000037alL0YjNW0?3`l0D0oooo00<0CDe=0000
003oool00`3oool01@000000CDe=0?ooo`000000O7al00H0oooo00<0/[:b0?ooo`3QhN400P000000
1P37alL0jNWY0?ooo`37alL0000009ZJVP80oooo00X0l?3`0000003QhN40oooo04e=C@000000oooo
0>WYj@000000YjNW0P3oool0903YjNT000000000003QhN40oooo04e=C@2<S8`0oooo09ZJVP000000
CDe=08b<S02m_Kd0c<c<09ZJVP000000alO70?ooo`3IfMT000000=3@d03`l?00VYZJ0000003`l?00
oooo0?3`l0000000hN7Q0?ooo`1=CDd000000?ooo`3YjNT000000:NWY`<0oooo00H0l?3`0000002J
VYX0/[:b0000003`l?020?ooo`060?3`l0000000hN7Q0?ooo`1=CDd00000:`3oool00`0P8200oooo
0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
6`3oool2000000T0oooo0P00000;0?ooo`060;fm_@3`l?00oooo0;:b/P000000O7al2@3oool20000
00070=WIf@3oool0VYZJ0000002m_Kd0oooo0=WIf@020000000:0=3@d02JVYX0/[:b0?ooo`000000
O7al0?ooo`2b/[8000000?3`l080oooo00D0J6QX07alO03oool0000007alO0060?ooo`80000000<0
/[:b0?ooo`3oool0103oool03@2b/[8000000000003oool0O7al0000003QhN40oooo0:NWY`3`l?00
/[:b0000003`l?000P3oool00`1XJ6P0O7al0=WIf@020000000A0=3@d02JVYX0/[:b0?ooo`1lO7`0
00000>7Qh@3oool0YjNW0?3`l03oool0000007alO03oool0/[:b0000003`l?000P3oool0301XJ6P0
O7al0?ooo`1=CDd000000?3`l03oool0CDe=0000003`l?00jNWY0;:b/PD0oooo00<0/[:b0000003@
d=000P3oool01P3@d=0000000;:b/P3oool0000007alO0H0oooo00/0_Kfm0?ooo`3IfMT0000007al
O03oool0YjNW0?ooo`3QhN40000004e=C@030?ooo`090<O7a`2JVYX0oooo0<O7a`000000d=3@0?oo
o`1XJ6P0CDe=00<0oooo01<0CDe=0000002WYjL0d=3@06QXJ0000000oooo0?3`l01XJ6P0CDe=0?3`
l01XJ6P0DE5A0>WYj@000000/[:b0:NWY`3YjNT0J6QX0080alO700<0/[:b0000002m_Kd00P3oool0
2@37alL0VYZJ0?ooo`37alL000000=3@d03oool0J6QX04e=C@020?ooo`0@0?3`l02m_Kd0VYZJ0000
003Rh^80J6QX0<O7a`3`l?00_Kfm0?3`l037alL0VYZJ0?ooo`37alL000000=3@d2X0oooo00<0820P
0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0
inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJA/0oooo0P0000090?ooo`8000002`3oool01P3YjNT0/[:b0?ooo`1=CDd000000=3@d0T0oooo
0P00000209ZJVP0906QXJ03@d=00oooo0?ooo`37alL0VYZJ08b<S02m_Kd000000080YjNW00/00000
07alO03oool0oooo0;:b/P2JVYX0/[:b0:NWY`2JVYX0l?3`09ZJVP020000000307alO037alL0oooo
00<0oooo0P0000000`2b/[80oooo0?ooo`040?ooo`040;:b/P00000000000:NWY`8000000100alO7
0>7Qh@1lO7`0S8b<0>WYj@2b/[80VYZJ0;:b/P2WYjL0VYZJ0?3`l037alL0VYZJ08b<S02m_Kd00000
0P2WYjL2000000030<O7a`3QhN40O7al0080S8b<01D0000007alO03oool0oooo0;:b/P2JVYX0/[:b
0:NWY`2JVYX0l?3`08b<S0000000CDe=0=WIf@2m_Kd0000004e=C@3IfMT0alO700000037alL01@3o
ool02037alL0S8b<0:NWY`2WYjL0S8b<0<O7a`3`l?00VYZJ0P0000000`1lO7`0alO70?ooo`030?oo
o`090<O7a`3YjNT0/[:b0000002JVYX0oooo0>7Qh@2m_Kd0oooo0080000000D0l?3`0?ooo`3oool0
oooo0=WIf@020>7Qh@0>06QXJ02JVYX0S8b<04e=C@000000O7al0:NWY`3oool0YjNW0000003QhN40
/[:b0000002<S8`20?ooo`0@0?3`l02b/[80alO709ZJVP1aLG40/[:b0000001=CDd0l?3`0:NWY`00
0000/[:b0?ooo`2m_Kd0CDe=0;:b/P<0oooo01H0fMWI0>7Qh@3QhN40J6QX09ZJVP2<S8`0CDe=0000
001lO7`0YjNW0?ooo`3`l?00/[:b04e=C@3QhN40l?3`0:NWY`2<S8`0CDe=0>WYj@3oool0fMWI0P3Q
hN400`1XJ6P0VYZJ0?ooo`0Y0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103o
ool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg
0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooo`8000002@3oool2000000`0oooo00@0
YjNW0=3@d0000000CDe=2P3oool2000000050=WIf@3`l?00S8b<04e=C@3QhN400`3oool00`3`l?00
oooo0?ooo`020?ooo`030>WYj@37alL0oooo00P0oooo00<0000007alO03oool01@3oool00`2<S8`0
000009ZJVP060?ooo`0308b<S0000000O7al0100oooo00<0l?3`0?ooo`3oool02@3oool00`3YjNT0
alO70?ooo`0O0?ooo`030000001lO7`0oooo00D0oooo00D0l?3`0<O7a`2WYjL000000;:b/P020?oo
o`050;fm_@3YjNT0S8b<0000003@d=00103oool0203`l?00alO70;fm_@000000oooo0>WYj@000000
YjNW0P3oool00`3IfMT000000:NWY`0I0?ooo`080?3`l037alL0_Kfm0000003oool0jNWY0000002W
YjL=0?ooo`040?3`l037alL0_Kfm000002X0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo
0?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd
0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA/0oooo0P0000090?ooo`800000
303oool0103IfMT000000000002b/[8:0?ooo`80000000D0fMWI0?ooo`3IfMT0000004e=C@0C0?oo
o`0306QXJ01lO7`0oooo00D0oooo00<0fMWI0000001=CDd01@3oool0103`l?0000000000003@d=0n
0?ooo`0306QXJ01lO7`0oooo00H0oooo00@0/[:b07alO0000000d=3@0`3oool0102WYjL0_Kfm0000
002WYjL:0?ooo`0309ZJVP2WYjL0oooo0080oooo00<0000006QXJ03oool07P3oool00`2JVYX0YjNW
0?ooo`0j0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo
0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0
n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFTI0?ooo`040>WYj@2WYjL00000000000L0oooo00@0jNWY0:NWY`00
000000003@3oool00`1=CDd000000>WYj@0:0?ooo`80000000D0fMWI0?ooo`2b/[80000004e=C@08
0?ooo`80J6QX2@3oool00`2JVYX0O7al0?ooo`060?ooo`030<O7a`000000VYZJ00@0oooo00<0VYZJ
0000002WYjL07@3oool206QXJ200oooo00<0VYZJ07alO03oool01P3oool0102m_Kd000000000003Y
jNT30?ooo`040>7Qh@2<S8`0000006QXJ0l0oooo00<0VYZJ0000003YjNT0903oool206QXJ3D0oooo
00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJAX0oooo00<0l?3`0<O7a`1=CDd0203oool00`3`l?00alO704e=C@0=0?ooo`030<O7
a`2JVYX0oooo00P0oooo00@0jNWY09ZJVP00000000000P2JVYX00`000000VYZJ0>WYj@080?ooo`03
08b<S01XJ6P0oooo00P0oooo00<0jNWY06QXJ03oool00P2m_Kd50?ooo`030>WYj@2WYjL0YjNW0080
/[:b00<0O7al09ZJVP3IfMT07P3oool00`2<S8`0J6QX0?ooo`0O0?ooo`030>WYj@1XJ6P0oooo0080
_Kfm0`3oool0102m_Kd0J6QX0000000000050?ooo`050;:b/P00000000000:NWY`3YjNT0303oool0
103YjNT0O7al0000002m_KdT0?ooo`80J6QX=@3oool00`0P8200oooo0?ooo`040?ooo`03020P803o
ool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYG@3oool0103QhN40YjNW0000
002WYjM>0?ooo`040>7Qh@2WYjL000000:NWYg`0oooo00<0820P0?ooo`3oool0103oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3oool]0?ooo`@0820P1P3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0
oooo00<0820P0?ooo`3oool00P3oool00`3/k>`0l?3`08j>SP050:2PX00309nOW`2PX:00X:2P00@0
X:2P01H0T92@0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo;03oool5020P8080oooo1@0P82060?ooo`03020P803oool0oooo0080
oooo00<0k>c/09JFUP3Ng]h0303Cdm<05P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0
820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0
e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3o
ool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM
0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200
oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
ooln0?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?S
h`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03
020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803oool0
oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0
oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gM
g@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolc0?ooo`D0820P1P3oool00`0P8200oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTm0?ooo`040000001lO7`0_Kfm0=WIf@h0oooo00<0fMWI0000003IfMT0i03oool00`0P
8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo00@0_Kfm0?3`l03o
ool0O7al3`3oool00`000000oooo0?ooo`3S0?ooo`03020P803oool0oooo00@0oooo00<0820P0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY703oool0103IfMT000000<O7a`0000020=WIf@0F0000003QhN400000
0=WIf@2m_Kd0YjNW0<O7a`2WYjL0_Kfm0?ooo`37alL0YjNW0?3`l02JVYX0oooo0=WIf@000000hN7Q
0<O7a`000000O7al0;fm_@80YjNW00@0alO70000003QhN40oooo0P0000004@1lO7`0oooo0<O7a`00
0000O7al0;fm_@3oool0fMWI0000003QhN40oooo0<O7a`000000O7al0;fm_@3oool000000080_Kfm
03H0YjNW0=WIf@000000hN7Q0?ooo`37alL0000007alO02m_Kd000000<O7a`1lO7`0alO70000001l
O7`0_Kfm0=WIf@000000hN7Q0000003IfMT0oooo0000002JVYX000000:NWY`000000alO70?ooo`00
0000VYZJ0;fm_@000000l?3`0;fm_@2WYjL0alO70:NWY`2m_Kd0fMWI0000003QhN4000000=WIf@3o
ool000000:NWY`000000alO70=WIf@000000hN7Q0000003IfMT20:NWY`030<O7a`000000hN7Q00<0
oooo03T0hN7Q09ZJVP3oool0fMWI0000003`l?0000000<O7a`1lO7`0oooo0000002JVYX000000=WI
f@000000YjNW00000037alL0fMWI0000003QhN40fMWI0000003`l?00VYZJ0;fm_@37alL0O7al0000
002WYjL000000<O7a`3oool0000009ZJVP2m_Kd000000?3`l02m_Kd0YjNW0<O7a`2WYjL0_Kfm0=WI
f@000000hN7Q0000003IfMT0oooo0;fm_@2WYjL0alO70:NWY`2m_Kd0fMWI0000003QhN400P3oool0
1`3IfMT000000?3`l03IfMT000000>7Qh@0000000P3IfMT0:0000000hN7Q0;fm_@2WYjL0alO70:NW
Y`2m_Kd0fMWI0000003QhN40oooo0=WIf@000000hN7Q0000003QhN40000009ZJVP000000YjNW0000
0037alL0oooo0000002JVYX0_Kfm0000003`l?00_Kfm0:NWY`37alL0YjNW0;fm_@3IfMT000000>7Q
h@000000fMWI0?ooo`00000i0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY7@3oool2000000080=WIf@3oool0oooo0000003oool000000?ooo`0000030?ooo`06
0000003oool0000007alO02WYjL0J6QX0P3oool01`000000oooo0000003`l?00oooo0?3`l0000000
0P3oool02`000000oooo0?ooo`000000alO707alO03oool000000?3`l03oool0l?3`0080oooo00T0
00000?ooo`3oool000000?3`l03oool0l?3`0?ooo`0000000P3oool00`000000oooo000000020?oo
o`030000003`l?00oooo0080l?3`00X0_Kfm000000000000l?3`0?ooo`3`l?00oooo0000003oool0
00000P3oool01@000000oooo06QXJ03`l?0000000080oooo00H000000?ooo`3oool000000?ooo`00
00030?ooo`050000003oool000000?ooo`0000000P3oool00`1XJ6P0l?3`000000020?ooo`050000
003oool000000?ooo`0000000P3oool00`000000oooo0?ooo`020?ooo`0A0000001lO7`0oooo0?oo
o`000000oooo0?3`l02m_Kd000000?ooo`000000oooo0000003oool0J6QX0?3`l00000000P3oool0
10000000oooo0?ooo`0000020?ooo`0607alO03oool0hN7Q06QXJ03`l?0000000P3oool01P000000
oooo0?ooo`000000oooo000000<0oooo00D000000?ooo`000000oooo000000020?ooo`070000003o
ool0oooo0?ooo`000000oooo000000040?ooo`060000003oool0oooo0000003oool000000P3oool0
0`000000oooo000000030?ooo`030000003oool0000000<0oooo00T000000?ooo`000000oooo07al
O037alL0J6QX0?3`l00000000P3oool01P000000oooo0?ooo`000000oooo000000<0oooo00D00000
0?ooo`000000oooo0000000l0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY7@3oool00`000000VYZJ0?3`l0020?ooo`050000003oool000000?ooo`0000000`3o
ool02`000000l?3`0000003`l?0000000?3`l037alL0oooo0000003oool0000000<0alO700@00000
0?ooo`3oool000000P3oool01@000000oooo0000003oool0000000<0alO70P3oool010000000oooo
0?ooo`0000030<O7a`070?ooo`000000oooo0?ooo`000000oooo000000020?ooo`0800000037alL0
alO70<O7a`000000VYZJ0?3`l00000030<O7a`040?ooo`000000oooo00000080oooo00D000000?oo
o`3oool0alO7000000020?ooo`060000003oool0oooo0000003oool000000`3oool01@000000oooo
0000003oool0000000<0oooo00T0alO70000003oool0oooo0000003oool000000?ooo`0000000P3o
ool00`000000oooo0?ooo`020?ooo`0=0000003oool0_Kfm0?ooo`000000oooo0000002JVYX0l?3`
0?ooo`000000oooo000000020?ooo`050<O7a`000000oooo0?ooo`0000000P3oool01@000000oooo
0?ooo`3IfMT0alO70080oooo00D0alO70000003oool0oooo000000020?ooo`030000003oool00000
00<0oooo00D000000?ooo`000000oooo000000020?ooo`070000003oool0oooo0?ooo`000000oooo
000000040?ooo`060000003oool0oooo0000003oool000000P3oool00`000000oooo000000030?oo
o`030000003oool0000000<0oooo00T000000?ooo`000000oooo07alO037alL0oooo0<O7a`000000
0P3oool01P000000oooo0?ooo`000000oooo000000<0oooo00D000000?ooo`000000oooo0000000l
0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7@3oool00`00
0000l?3`000000020=WIf@0J0000002WYjL000000?ooo`37alL0_Kfm0<O7a`2m_Kd0alO706QXJ000
0000fMWI0000003QhN40J6QX0>7Qh@000000oooo0:NWY`2m_Kd0alO709ZJVP3IfMT0_Kfm0<O7a`00
00020?ooo`0<09ZJVP2m_Kd0000006QXJ02WYjL0_Kfm0<O7a`2JVYX0oooo0=WIf@000000O7al0P2W
YjL01P2m_Kd0alO709ZJVP3QhN4000000<O7a`80YjNW00<0fMWI0000001lO7`00P2WYjL09@2m_Kd0
alO709ZJVP2JVYX0alO706QXJ02WYjL0_Kfm0<O7a`2JVYX0fMWI0000002WYjL000000?ooo`3QhN40
000007alO02WYjL0VYZJ0000003oool0hN7Q0000001lO7`0hN7Q0000003oool0alO70;fm_@37alL0
_Kfm0<O7a`3IfMT000000:NWY`0000000P3oool03@2WYjL0VYZJ0000003oool0fMWI0000002WYjL0
00000?ooo`3IfMT0_Kfm0<O7a`0000000`3oool04P1lO7`000000?3`l0000000hN7Q0000003oool0
VYZJ0<O7a`1XJ6P0fMWI0000003QhN4000000?ooo`2WYjL0VYZJ00000080oooo01`000000?ooo`3Q
hN4000000?ooo`2WYjL0_Kfm06QXJ03`l?00YjNW09ZJVP000000oooo0>7Qh@000000O7al0>7Qh@00
0000oooo0<O7a`2m_Kd0alO70;fm_@37alL0fMWI0000002WYjL000000P3oool01037alL0_Kfm0<O7
a`2m_Kd20<O7a`0b0000002JVYX0oooo0?ooo`3QhN4000000?ooo`3IfMT000000:NWY`000000oooo
0<O7a`000000VYZJ0<O7a`2m_Kd0alO70;fm_@37alL0fMWI0000001lO7`0YjNW0=WIf@000000YjNW
0000002m_Kd000000>7Qh@2WYjL0VYZJ0000003oool0hN7Q0000001lO7`0hN7Q0000003oool0alO7
0;fm_@37alL0_Kfm0<O7a`3IfMT000000:NWY`00000l0?ooo`03020P803oool0oooo00@0oooo00<0
820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY7@3oool00`000000oooo0?ooo`0B0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool02P3oool00`3`l?00oooo0?ooo`0:0?ooo`030?3`l03oool0
oooo0100oooo00<0l?3`06QXJ03oool0103oool01@3`l?00J6QX0?ooo`3QhN40YjNW01P0oooo00<0
00000?ooo`3oool01@3oool00`3QhN40YjNW0?ooo`0=0?ooo`040000003oool0hN7Q0:NWY`T0oooo
00D0l?3`06QXJ03oool0hN7Q0:NWY`0B0?ooo`060000003oool0oooo0?ooo`3QhN40YjNW1`3oool0
0`000000oooo0?ooo`050?ooo`050?3`l03oool0oooo0?ooo`3`l?002@3oool01@3`l?00J6QX0?oo
o`3QhN40YjNW04H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTM0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`1>0?ooo`030000003oool0oooo01D0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0:0?ooo`030000003oool0oooo05h0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTL0?ooo`030=WIf@000000oooo0180oooo00<0fMWI0000003oool01P3oool0
0`3`l?0000000?ooo`0f0?ooo`030000003oool0oooo01D0oooo00<0l?3`0000003oool01`3oool0
0`000000oooo0?ooo`0;0?ooo`050=WIf@000000oooo0?ooo`0000003@3oool00`000000oooo0?oo
o`0@0?ooo`030<O7a`2JVYX0VYZJ0080oooo00<000000?ooo`3oool01@3oool00`37alL0VYZJ09ZJ
VP0G0?ooo`030000003oool0oooo04@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTM0?ooo`030>7Qh@3oool0oooo0180oooo00<0hN7Q0?ooo`3oool01P3oool0
0`3IfMT0oooo0?ooo`1>0?ooo`030=WIf@3oool0oooo01D0oooo00<0hN7Q0?ooo`3oool0S`3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=`3oool00`0P
8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=`3oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[Z
jP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJBH0oooo00<0alO70?ooo`3oool0
703oool010000000O7al0;fm_@3IfMTM0?ooo`060000001lO7`0_Kfm0=WIf@3oool0alO78`3oool0
10000000O7al0;fm_@3IfMTC0?ooo`030=WIf@000000fMWI0180oooo00H0000007alO02m_Kd0fMWI
0?ooo`37alL30?ooo`030=WIf@000000fMWI00/0oooo00@0000007alO02m_Kd0fMWI7`3oool01P00
0000O7al0;fm_@3IfMT0oooo0<O7abL0oooo00<0VYZJ07alO03oool02P3oool00`37alL0oooo0?oo
o`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY9P3oool0
0`3`l?00_Kfm0?ooo`0L0?ooo`040;fm_@3`l?00oooo07alO1d0oooo00L0_Kfm0?3`l03oool0O7al
0?ooo`3`l?00_Kfm0280oooo00@0_Kfm0?3`l03oool0O7al503oool00`000000oooo0?ooo`0A0?oo
o`070;fm_@3`l?00oooo07alO03oool0l?3`0;fm_@030?ooo`030000003oool0oooo00X0oooo00@0
_Kfm0?3`l03oool0O7al7`3oool01`2m_Kd0l?3`0?ooo`1lO7`0oooo0?3`l02m_Kd09`3oool00`3I
fMT0hN7Q0?ooo`090?ooo`030?3`l03oool0oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P
8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`0<0=WIf@000000l?3`0:NWY`000000YjNW0>7Qh@00
0000alO707alO0000000YjNW0`3oool02@000000YjNW00000037alL0fMWI0000003QhN4000000=WI
f@020:NWY`030<O7a`000000hN7Q00<0oooo00X0alO70000001lO7`0_Kfm0=WIf@000000hN7Q0000
003IfMT0oooo0P0000007`1lO7`0oooo0=WIf@000000l?3`0=WIf@000000hN7Q0000003IfMT0alO7
0000001lO7`0_Kfm0<O7a`000000O7al0;fm_@3IfMT000000>7Qh@3oool0fMWI0000003`l?00fMWI
0000003QhN4000000=WIf@3oool00P0000006@1lO7`0oooo0000002WYjL0oooo0=WIf@000000l?3`
0=WIf@000000hN7Q0000003IfMT0YjNW0000002WYjL0hN7Q0=WIf@000000hN7Q0?ooo`000000VYZJ
0000003IfMT00P2WYjL03037alL000000>7Qh@3IfMT000000?3`l03IfMT000000>7Qh@000000fMWI
0?ooo`80000001H0O7al0?ooo`3oool0oooo0:NWY`000000YjNW0>7Qh@2m_Kd0YjNW0<O7a`2WYjL0
_Kfm0=WIf@000000hN7Q0000003QhN40000009ZJVP3oool000000P2m_Kd04P2WYjL0oooo0000002J
VYX000000=WIf@3oool0000009ZJVP2m_Kd000000?3`l03IfMT000000>7Qh@000000fMWI0?ooo`80
000000@0O7al0?ooo`000000YjNW0`3oool03P000000_Kfm0;fm_@2WYjL0fMWI0000003QhN40oooo
0;fm_@2WYjL0alO70:NWY`2m_Kd0oooo0P0000008@1lO7`0oooo0=WIf@000000hN7Q0?ooo`000000
YjNW00000037alL0fMWI0000003QhN4000000>7Qh@000000VYZJ0=WIf@000000hN7Q0000003QhN40
000009ZJVP3IfMT000000?3`l03IfMT000000>7Qh@000000fMWI0?ooo`020000000407alO03oool0
00000:NWY`<0oooo0P2WYjL03P37alL000000>7Qh@000000YjNW00000037alL0oooo0000002JVYX0
00000:NWY`000000alO70`3oool07P000000YjNW00000037alL0fMWI0000003QhN4000000=WIf@00
0000YjNW00000037alL0fMWI0000003QhN40oooo0>7Qh@1lO7`0oooo00000037alL0O7al0=WIf@00
0000l?3`00000037alL0O7al000000H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTM0?ooo`030000003oool0000000<0oooo00<0l?3`0;fm_@0000001@3oool0
0`1XJ6P0l?3`000000020?ooo`050000003oool000000?ooo`0000000P3oool00`000000oooo0?oo
o`020?ooo`080000003`l?00oooo0?3`l03oool000000?ooo`0000020?ooo`0300000037alL0O7al
0080oooo014000000?ooo`3oool000000?ooo`000000oooo0000003`l?00oooo0?3`l0000000l?3`
0?ooo`3`l?00oooo000000030?ooo`060000003oool0oooo0000003oool000000P3oool00`000000
alO707alO0050?ooo`080000003oool0oooo0000003oool000000?ooo`0000040?ooo`080000003o
ool0oooo0000003oool000000?ooo`0000020?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`0000000P3oool00`000000alO707alO0030?ooo`050000003oool0oooo0?ooo`0000000`3o
ool02P000000oooo0000003oool000000?ooo`1lO7`0alO70?ooo`0000020?ooo`050000003oool0
00000?ooo`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool000000080oooo
00<000000<O7a`1lO7`01P3oool01P000000oooo0?ooo`000000oooo00000080oooo00T000000?oo
o`3oool0oooo0000003oool000000<O7a`1lO7`00P3oool01P000000oooo0?ooo`1XJ6P0l?3`0000
0080oooo00l000000?ooo`000000oooo07alO037alL0oooo0000003oool000000?ooo`1lO7`0alO7
0?ooo`0000000P3oool00`000000oooo000000020?ooo`0300000037alL0O7al00H0oooo00P00000
0?ooo`3oool000000?ooo`1XJ6P0l?3`00000080oooo00D000000?ooo`1XJ6P0l?3`000000040?oo
o`0306QXJ03`l?0000000080oooo00L000000?ooo`000000oooo06QXJ03`l?0000000080oooo00D0
00000?ooo`3oool0J6QX09ZJVP020?3`l0080;fm_@000000oooo0000003oool0l?3`0;fm_@000007
0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7@3oool01000
0000oooo0000003oool20?3`l0030000002JVYX0l?3`00H0oooo00T0alO70000003oool0oooo0000
003oool000000?ooo`0000000P3oool00`000000oooo0?ooo`020?ooo`0800000037alL0alO70<O7
a`3oool000000?ooo`0000020?ooo`030000003oool000000080oooo00P000000?ooo`3oool00000
0?ooo`000000oooo000000<0alO700H000000<O7a`37alL0alO70?ooo`0000030?ooo`060000003o
ool0oooo0000003oool000000P3oool00`000000oooo000000050?ooo`090000003oool0oooo0000
003oool000000?ooo`000000oooo0080l?3`00T0oooo0000003oool0oooo0000003oool000000?oo
o`0000000P3oool010000000oooo0?ooo`0000020?ooo`030000003oool000000080oooo00<00000
0?ooo`0000000`3oool01@000000oooo0?3`l03`l?00000000<0oooo00X000000?ooo`000000oooo
0000003oool0O7al0<O7a`3oool000000P3oool01@000000oooo0000003oool000000080oooo00@0
00000?ooo`3oool000000P3oool00`000000oooo000000020?ooo`030000003oool0000000H0oooo
00H000000?ooo`3oool000000?ooo`0000020?ooo`090000003oool0oooo0?ooo`000000oooo0000
003oool000000080oooo00H000000?ooo`3oool0oooo0<O7a`0000020?ooo`0?0000003oool00000
0?ooo`1lO7`0alO70?ooo`000000oooo0000003oool0O7al0<O7a`3oool000000080oooo00<00000
0?ooo`0000000P3oool00`000000oooo000000060?ooo`040000003oool0oooo00000080oooo00D0
alO70000003oool0oooo000000020?ooo`030<O7a`000000oooo00@0oooo00L0alO70000003oool0
oooo0000003oool000000080oooo00D0alO70000003oool0oooo000000020?ooo`0<0000003oool0
alO70000002JVYX0l?3`0?ooo`000000oooo0000002JVYX0l?3`1`3oool00`0P8200oooo0?ooo`04
0?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3L
g=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo00<0hN7Q0000003oool00P2WYjL01@1X
J6P0_Kfm09ZJVP37alL0J6QX00D0oooo00d0YjNW09ZJVP000000oooo0=WIf@000000YjNW0000003o
ool0fMWI0;fm_@37alL0000000@0oooo00P0YjNW0;fm_@37alL0VYZJ0=WIf@000000YjNW00000080
oooo01l0VYZJ0;fm_@000000J6QX0>7Qh@000000oooo0=WIf@000000YjNW0000003oool0YjNW0;fm
_@37alL0VYZJ0:NWY`2m_Kd0alO709ZJVP3IfMT0000007alO02WYjL0hN7Q0000003oool0fMWI0000
002WYjL000000080oooo00@0VYZJ0;fm_@000000J6QX0`3oool0203QhN4000000?ooo`3IfMT00000
0:NWY`000000oooo0P2WYjL05P1XJ6P0_Kfm0?ooo`000000oooo0=WIf@000000hN7Q0000003oool0
fMWI0;fm_@37alL000000?ooo`3QhN4000000?ooo`3IfMT000000:NWY`0000020?ooo`0409ZJVP2m
_Kd0000006QXJ080oooo0P2WYjL03@1XJ6P0_Kfm0<O7a`2m_Kd0alO70;fm_@37alL0fMWI0000002W
YjL000000;fm_@0000000P3QhN404`000000alO70:NWY`2WYjL0fMWI0000003QhN4000000?ooo`3Q
hN40000007alO03QhN4000000?ooo`3IfMT000000:NWY`0000000P3oool0102JVYX0_Kfm0000001X
J6P40?ooo`030>7Qh@000000alO70080YjNW0140fMWI0000001lO7`0YjNW0<O7a`2m_Kd0alO70;fm
_@37alL0oooo09ZJVP2m_Kd0000006QXJ03IfMT0000007alO0020:NWY`0@09ZJVP000000oooo0=WI
f@000000YjNW0000002m_Kd000000>7Qh@3IfMT000000:NWY`000000_Kfm00000080hN7Q00H00000
0?ooo`3IfMT000000:NWY`0000020?ooo`0409ZJVP2m_Kd0000006QXJ0D0oooo00l0fMWI0;fm_@37
alL000000?ooo`2WYjL0VYZJ0000003oool0hN7Q0000001lO7`0YjNW09ZJVP000000103oool0302W
YjL0VYZJ0000003oool0fMWI0000002WYjL000000?ooo`2WYjL0VYZJ00000080oooo00l000000?oo
o`2WYjL000000?3`l01XJ6P0CDe=0<O7a`1XJ6P0hN7Q0000003oool0VYZJ0<O7a`1XJ6P01`3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo00<0hN7Q0:NW
Y`3oool0603oool00`000000oooo0?ooo`0@0?ooo`030>7Qh@2WYjL0oooo00h0oooo00D0l?3`0?oo
o`3oool0hN7Q0:NWY`0>0?ooo`030>7Qh@2WYjL0oooo00X0oooo00<000000?ooo`3oool01`3oool0
10000000oooo0>7Qh@2WYjLG0?ooo`030?3`l03oool0oooo00d0oooo00D0l?3`06QXJ03oool0hN7Q
0:NWY`0E0?ooo`030?3`l03oool0oooo00/0oooo00<0l?3`0?ooo`3oool01@3oool00`3`l?00oooo
0?ooo`040?ooo`030?3`l03oool0oooo00<0oooo00<0hN7Q0:NWY`3oool04P3oool00`000000oooo
0?ooo`030?ooo`030?3`l01XJ6P0oooo01D0oooo00<000000?ooo`3oool01P3oool00`3QhN40YjNW
0?ooo`0:0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY=`3o
ool00`000000oooo0?ooo`110?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0NP3o
ool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo01<0oooo00<0820P0?ooo`3oool0103o
ool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL
0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTM0?ooo`030000003oool0oooo01H0oooo00<0l?3`
0000003oool04P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00d0oooo00<00000
0?ooo`3oool0203oool00`3IfMT000000?ooo`070?ooo`050?3`l0000000oooo0?ooo`000000:`3o
ool00`000000oooo0?ooo`0g0?ooo`030000003oool0oooo0100oooo00<0l?3`0000003oool06`3o
ool00`3IfMT000000?ooo`080?ooo`030000003oool0oooo00T0oooo00<0820P0?ooo`3oool0103o
ool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL
0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTg0?ooo`030=WIf@3oool0oooo0440oooo00<0hN7Q
0?ooo`3oool01`3oool00`3IfMT0oooo0?ooo`1j0?ooo`030=WIf@3oool0oooo01/0oooo00<0hN7Q
0?ooo`3oool04`3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=`3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=`3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJGh0oooo
00<0fMWI0000003IfMT09@3oool010000000O7al0;fm_@3IfMUI0?ooo`030<O7a`3oool0oooo0300
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFUo0?ooo`030000
003oool0oooo02@0oooo00@0_Kfm0?3`l03oool0O7alF@3oool00`3`l?00_Kfm0?ooo`0`0?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703oool02@3IfMT00000
0>7Qh@000000fMWI0?ooo`000000VYZJ000000020=WIf@040000003QhN4000000=WIf@80YjNW00/0
alO70000003QhN40fMWI0000003QhN40oooo0<O7a`000000O7al0;fm_@020:NWY`0?0<O7a`000000
hN7Q00000037alL0O7al0?ooo`2m_Kd0YjNW0<O7a`2WYjL0_Kfm0=WIf@000000hN7Q0080oooo00L0
fMWI0000003QhN40oooo0000002JVYX000000080fMWI01/000000>7Qh@000000fMWI0:NWY`000000
YjNW0>7Qh@3oool0000009ZJVP2m_Kd000000?3`l02m_Kd0YjNW0<O7a`2WYjL0_Kfm0=WIf@000000
hN7Q0000003IfMT000000<O7a`1lO7`00P3oool02`3IfMT000000>7Qh@2m_Kd0YjNW0<O7a`2WYjL0
_Kfm0=WIf@000000hN7Q00<0oooo02P000000;fm_@2m_Kd0YjNW0<O7a`000000O7al0;fm_@3IfMT0
00000>7Qh@3oool0fMWI0000003QhN40_Kfm0:NWY`37alL0YjNW0;fm_@3IfMT000000>7Qh@3oool0
fMWI0000003QhN4000000>7Qh@000000VYZJ0=WIf@000000l?3`0=WIf@000000hN7Q0000003IfMT0
oooo0P0000007`1lO7`0oooo0?ooo`3oool0hN7Q09ZJVP3oool000000:NWY`000000alO70=WIf@00
0000hN7Q0?ooo`3IfMT000000?3`l02m_Kd0YjNW0<O7a`2WYjL0_Kfm0?ooo`000000VYZJ0000003I
fMT000000<O7a`1lO7`00`3oool04@000000VYZJ0000002WYjL000000<O7a`000000alO707alO03I
fMT000000<O7a`000000fMWI00000037alL0O7al0080oooo00P0fMWI0000003`l?00fMWI0000003Q
hN4000000=WIf@80oooo01d000000<O7a`1lO7`0YjNW0000002WYjL0hN7Q0=WIf@000000l?3`0<O7
a`000000O7al0;fm_@3IfMT000000>7Qh@000000fMWI0:NWY`000000YjNW0>7Qh@37alL0000007al
O02m_Kd000000:NWY`030?ooo`0V0=WIf@000000hN7Q0000003QhN40000009ZJVP000000YjNW0000
0037alL0oooo0000002JVYX0_Kfm0000003QhN4000000=WIf@37alL0000007alO02m_Kd0fMWI0000
003QhN4000000>7Qh@000000VYZJ0000002WYjL000000<O7a`3oool0000009ZJVP3QhN480?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7@3oool00`000000oooo
000000020?ooo`030000003oool000000080oooo00D000000?ooo`000000oooo000000020?ooo`04
0000003oool0oooo00000080oooo00D000000?3`l03oool0l?3`000000020?ooo`070000003oool0
l?3`0;fm_@000000oooo000000030?ooo`030000003oool0000000@0oooo00H000000?ooo`3oool0
00000?ooo`0000020?ooo`050000003oool000000?ooo`000000103oool01P000000oooo0?ooo`00
0000oooo000000<0oooo00T000000?ooo`000000oooo0000003oool0l?3`0;fm_@0000000`3oool0
0`000000oooo000000030?ooo`030000003oool0000000@0oooo00<000000?ooo`3oool00P000000
1@3`l?00oooo0?3`l03oool0000000<0oooo00<000000?ooo`0000000`3oool00`000000oooo0000
00030?ooo`080000003oool000000?ooo`1lO7`0alO70?ooo`0000020?ooo`030000003oool00000
0080oooo00<000000<O7a`1lO7`00`3oool01P000000O7al0?ooo`1XJ6P0l?3`00000080oooo00L0
00000?ooo`3oool0oooo0000003oool0000000<0oooo00T000000?ooo`000000oooo0000003oool0
l?3`0;fm_@0000000`3oool02P000000oooo06QXJ03`l?0000000?ooo`3`l?00_Kfm0000003oool2
000000050=WIf@3oool0l?3`0;fm_@0000000`3oool01P000000oooo0?ooo`000000oooo000000<0
oooo00@0l?3`0;fm_@0000000000103oool030000000oooo0000003`l?00oooo0?3`l03oool00000
0?ooo`000000oooo000000<0oooo00@000000?3`l03oool0l?3`1P3oool02@000000oooo0000003o
ool0O7al0<O7a`1XJ6P0l?3`000000020?ooo`0E0000003oool0oooo0000003oool000000?ooo`00
0000l?3`0?ooo`3`l?00oooo0000003oool000000?ooo`1lO7`0alO706QXJ03`l?0000000080oooo
00<000000?ooo`3oool0203oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3@d=00clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJAd0oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?ooo`050000003o
ool000000?ooo`0000000P3oool010000000oooo0?ooo`0000020?ooo`0500000037alL0alO70<O7
a`0000000P3oool01`000000oooo0000002JVYX0l?3`0?ooo`0000000`3oool00`000000oooo0000
00040?ooo`060000003oool0oooo0000003oool000000P3oool01P000000oooo0000003oool00000
0?ooo`80l?3`00L0oooo0000003oool0oooo0000003oool0000000<0oooo00T000000?ooo`000000
oooo0000003oool0000009ZJVP3`l?000`3oool00`000000oooo000000030?ooo`030000003oool0
000000@0oooo00<000000?ooo`3oool00P0000030<O7a`030?ooo`000000oooo0080oooo00<00000
0?ooo`0000000`3oool00`000000oooo000000030?ooo`080000003oool000000?ooo`1lO7`0alO7
0?ooo`0000020?ooo`030000003oool000000080oooo00<000000?ooo`0000000`3oool01P000000
oooo0;fm_@3oool0alO700000080oooo00L000000?ooo`3oool0oooo0000003oool0000000<0oooo
00T000000?ooo`000000oooo0000003oool0000009ZJVP3`l?000`3oool04@000000oooo0?ooo`37
alL000000?ooo`000000VYZJ0?3`l03oool0000009ZJVP3`l?00oooo0000002JVYX0l?3`00<0oooo
00H000000?ooo`3oool000000?ooo`0000030?ooo`050000002JVYX0l?3`0000003oool00P3`l?00
103oool000000?ooo`0000030<O7a`070?ooo`000000oooo0000003oool000000?ooo`020?3`l004
00000037alL0alO70<O7a`H0oooo00T000000?ooo`000000oooo07alO037alL0oooo0<O7a`000000
0P3oool020000000oooo0?ooo`000000oooo0000003oool000000`37alL02P3oool000000?ooo`00
0000oooo07alO037alL0oooo0<O7a`0000020?ooo`050000003oool0oooo0?ooo`1lO7`01P3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAd0oooo01H000000:NW
Y`000000oooo0=WIf@000000hN7Q0000003oool0fMWI0000002WYjL000000?ooo`3IfMT0_Kfm0<O7
a`000000oooo0=WIf@000000O7al0P2WYjL0402m_Kd0alO709ZJVP3IfMT0_Kfm0<O7a`000000oooo
09ZJVP37alL0J6QX0?ooo`37alL0_Kfm0<O7a`2m_Kd20<O7a`0A0000002JVYX0oooo0?ooo`37alL0
000009ZJVP3IfMT000000>7Qh@000000oooo0=WIf@000000YjNW0000003oool00P2WYjL05@1XJ6P0
_Kfm0>7Qh@000000O7al0>7Qh@000000oooo0<O7a`2m_Kd0alO70;fm_@37alL0fMWI0000002WYjL0
00000?ooo`2JVYX0alO706QXJ0020?ooo`0@0<O7a`000000VYZJ0<O7a`2m_Kd0alO70;fm_@37alL0
fMWI0000001lO7`0YjNW0?ooo`3QhN4000000<O7a`<0YjNW01T0_Kfm0<O7a`2JVYX0fMWI0000001l
O7`0YjNW0<O7a`000000VYZJ0<O7a`2m_Kd0alO70;fm_@37alL0fMWI0000001lO7`0YjNW0=WIf@00
0000YjNW0000002m_Kd000000080hN7Q00H000000?ooo`3IfMT000000:NWY`0000020?ooo`0Q09ZJ
VP2m_Kd0000006QXJ03oool0O7al0000003`l?00000008b<S02JVYX000000?ooo`3IfMT0000007al
O02WYjL0hN7Q0000003oool0alO70;fm_@37alL0_Kfm0<O7a`3IfMT000000>7Qh@000000oooo09ZJ
VP37alL0J6QX0080oooo0180hN7Q0000001lO7`0YjNW09ZJVP000000oooo09ZJVP37alL0J6QX0?oo
o`000000l?3`0000003IfMT0VYZJ0<O7a`1XJ6P20?ooo`070>7Qh@000000oooo0=WIf@000000YjNW
000000030?ooo`0309ZJVP37alL0J6QX0080YjNW00h0J6QX0;fm_@3QhN4000000?ooo`2WYjL0_Kfm
0<O7a`2JVYX0fMWI0000002WYjL000000?ooo`80YjNW00H0J6QX0;fm_@2WYjL0_Kfm0<O7a`2JVYX5
0?ooo`0V0=WIf@000000YjNW0000002m_Kd000000>7Qh@2WYjL0VYZJ0000003oool0hN7Q0000001l
O7`0l?3`0000002WYjL000000?ooo`2WYjL0_Kfm0<O7a`2JVYX0fMWI0000002WYjL000000;fm_@00
0000hN7Q0:NWY`2JVYX000000?ooo`3QhN40000007alO03`l?080?ooo`03020P803oool0oooo00@0
oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cL
g03Lg=`0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7@3oool00`000000oooo0?ooo`0>0?ooo`040000
003oool0oooo0?3`l0T0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`020?ooo`03
0000003oool0oooo00d0oooo00D0l?3`06QXJ03oool0hN7Q0:NWY`0A0?ooo`030000003oool0oooo
00D0oooo00<0l?3`0?ooo`3oool02`3oool01@3`l?00oooo0?ooo`3oool0000000L0oooo00D0l?3`
0?ooo`3oool0oooo0?3`l0050?ooo`030>7Qh@2WYjL0oooo01@0oooo00D0l?3`0?ooo`3oool0hN7Q
0:NWY`0@0?ooo`030?3`l01XJ6P0oooo00P0oooo00<000000?ooo`3oool01P3oool00`3QhN40YjNW
0?ooo`0>0?ooo`030>7Qh@2WYjL0oooo01L0oooo00<0l?3`0?ooo`3oool01`3oool01@3`l?00J6QX
0?ooo`3oool0000000P0oooo00<0l?3`0?ooo`3oool01`3oool00`3`l?00J6QX0?ooo`090?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@H0UIFE01T0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTM0?ooo`030000003oool0oooo00h0oooo00<00000
0?ooo`3oool02P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0080oooo00<00000
0?ooo`3oool08`3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo04H0oooo00<00000
0?ooo`3oool0?`3oool00`000000oooo0?ooo`0L0?ooo`03020P803oool0oooo00@0oooo00<0820P
0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTL0?ooo`030=WIf@000000oooo00h0oooo00<0l?3`0000003oool02P3oool00`3`l?00
00000?ooo`0:0?ooo`030<O7a`2JVYX0VYZJ0080oooo00<0alO709ZJVP2JVYX04@3oool00`000000
oooo0?ooo`0?0?ooo`030<O7a`2JVYX0VYZJ01L0oooo00<0alO709ZJVP2JVYX0403oool00`000000
oooo0?ooo`0G0?ooo`030000003oool0oooo01P0oooo00<0fMWI0000003oool0203oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo02<0oooo00<0fMWI0000003oool07@3oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd609FE
U@0I0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY7@3oool00`3QhN40oooo0?ooo`0>0?ooo`030=WIf@3oool0
oooo00X0oooo00<0fMWI0?ooo`3oool0U`3oool00`3QhN40oooo0?ooo`0o0?ooo`030>7Qh@3oool0
oooo01`0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`050>c/k02FUYH0
oooo0?Ogm`3]kNd01P3oool06P3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=`3oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd609FE
U@0I0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0820P
0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFU[0?ooo`0309ZJVP1lO7`0oooo0100oooo00@0000007alO02m_Kd0fMWI<P3oool01000
0000O7al0;fm_@3IfMT40?ooo`0309ZJVP1lO7`0oooo01D0oooo00<0fMWI0000003IfMT0503oool0
0`2JVYX0O7al0?ooo`180?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU/0?ooo`030=WI
f@3QhN40oooo00l0oooo00@0_Kfm0?3`l03oool0O7al<P3oool0102m_Kd0l?3`0?ooo`1lO7`50?oo
o`030=WIf@3QhN40oooo01D0oooo00<000000?ooo`3oool0503oool00`3IfMT0hN7Q0?ooo`170?oo
o`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0
kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`0^0:NWY`2JVYX0oooo0:NWY`3o
ool0fMWI0000003IfMT000000<O7a`000000l?3`00000037alL0oooo0000002WYjL0VYZJ0?ooo`00
0000_Kfm0?3`l03oool0alO707alO03oool000000?ooo`000000alO700000037alL000000?3`l000
0000alO70?ooo`2m_Kd0J6QX0000002m_Kd0oooo00000037alL000000?3`l0@0oooo00H0fMWI0000
003`l?0000000<O7a`1lO7`30?ooo`040000002WYjL000000<O7a`D0oooo00X0hN7Q09ZJVP3oool0
alO70000001lO7`0_Kfm0=WIf@000000hN7Q0P3oool00`3QhN40O7al0?ooo`040?ooo`0:0=WIf@00
0000hN7Q0000002WYjL000000<O7a`3IfMT000000>7Qh@80oooo0P0000001P1lO7`0oooo0<O7a`00
0000O7al0;fm_@80oooo00T000000:NWY`000000alO70=WIf@000000hN7Q0000003IfMT00P2WYjL0
0`37alL000000>7Qh@030?ooo`0K00000037alL0O7al0<O7a`000000O7al0;fm_@37alL0000007al
O02m_Kd0fMWI0000003QhN4000000>7Qh@000000VYZJ0=WIf@000000l?3`0=WIf@000000hN7Q0000
003IfMT0oooo0080000000P0O7al0?ooo`3IfMT000000>7Qh@3oool0hN7Q07alO0D0oooo0180YjNW
0000002WYjL0hN7Q0;fm_@2WYjL0alO70:NWY`2m_Kd0fMWI0000003QhN4000000>7Qh@000000VYZJ
0?ooo`0000020;fm_@0<0:NWY`3IfMT000000>7Qh@37alL0000007alO02m_Kd0VYZJ0<O7a`000000
O7al103oool06P000000alO707alO03oool0hN7Q07alO03oool000000<O7a`1lO7`0oooo0000002J
VYX0_Kfm0000001lO7`0_Kfm0=WIf@000000hN7Q0000003QhN40000009ZJVP3oool000001@3oool0
1`2m_Kd000000=WIf@3oool0000009ZJVP3QhN400`3oool07`2WYjL000000:NWY`3QhN40_Kfm0:NW
Y`37alL0YjNW0;fm_@3IfMT000000>7Qh@000000fMWI0?ooo`000000VYZJ0000002WYjL000000<O7
a`3IfMT000000?3`l03IfMT000000>7Qh@000000fMWI00000037alL01P3oool00`0P8200oooo0?oo
o`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CT
i03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAd0oooo01D0alO70?ooo`000000alO70?oo
o`000000oooo06QXJ03QhN4000000>7Qh@000000oooo0?3`l01XJ6P0fMWI0?3`l02WYjL000000?oo
o`3`l?000P3oool03P000000l?3`07alO02m_Kd0YjNW0;fm_@1RHV80hN7Q0000003QhN4000000?oo
o`3`l?00oooo0P0000001P3oool0l?3`06QXJ03QhN4000000>7Qh@D0oooo00D000000?ooo`3`l?00
_Kfm000000030?ooo`0306QXJ03`l?00000000H0oooo00T0000007alO03oool000000?3`l03oool0
l?3`0?ooo`0000000`3oool00`1XJ6P0VYZJ0?3`l0050?ooo`050000003oool0J6QX0?3`l0000000
0P3oool030000000oooo0?ooo`3oool000000<O7a`1lO7`0oooo0000003`l?00oooo0?3`l080oooo
00<0J6QX0?3`l00000000P3oool01@000000oooo0000003oool000000080oooo00<000000?ooo`3o
ool00P3oool0503`l?00_Kfm000000000000l?3`0?ooo`3`l?0000000?3`l03oool0l?3`0?ooo`00
0000oooo0000003oool0O7al0<O7a`3oool000000P3oool00`000000oooo000000020?ooo`030000
0037alL0O7al0080oooo00H000000?ooo`3oool0J6QX09ZJVP3`l?040?ooo`050000003oool0oooo
0?ooo`0000000`3oool02P000000oooo0000003oool000000?ooo`1lO7`0alO70?ooo`0000020?oo
o`0;0000003oool000000?ooo`000000l?3`0?ooo`3`l?00oooo0;fm_@0000001@3oool01P3`l?00
_Kfm0000003oool0J6QX09ZJVP80l?3`0100_Kfm0000003oool000000?ooo`000000l?3`0?ooo`3`
l?00oooo0000003oool000000?ooo`1lO7`0alO7203oool010000000oooo0?ooo`0000050?ooo`05
0000003oool0oooo0?ooo`0000000`3oool01@000000oooo0000003oool000000080oooo00D00000
0?ooo`1XJ6P0l?3`000000020?ooo`090000003oool0oooo0000003oool000000?ooo`3`l?00_Kfm
00H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTM0?ooo`06
0<O7a`3oool0000006QXJ03oool0O7al0`37alL03P1lO7`0_Kfm0:NWY`2m_Kd0oooo0;fm_@000000
oooo06QXJ0000000alO70;fm_@3oool0hN7Q0P2JVYX0103QhN4000000=WIf@1XJ6P20<O7a`0807al
O02m_Kd0YjNW0;fm_@3oool0l?3`06QXJ02ATI420?ooo`80alO700<0O7al0;fm_@3oool0103oool0
1@000000oooo0000002JVYX0l?3`00@0oooo00<0alO70000003oool01@3oool010000000oooo0;fm
_@0000030<O7a`030?ooo`000000oooo0080oooo00<000000?ooo`37alL01@3oool01@000000oooo
0?ooo`37alL000000080oooo00T000000?ooo`3oool0oooo0000003oool000000?ooo`0000000`37
alL30?ooo`090<O7a`000000oooo0?ooo`000000oooo0000003oool000000080oooo00<000000?oo
o`3oool00P3oool010000000VYZJ0?3`l00000030<O7a`0=00000037alL0alO70<O7a`3oool00000
0?ooo`000000oooo07alO037alL0oooo000000020?ooo`030000003oool000000080oooo00<00000
0?ooo`0000000P3oool01P000000oooo0?ooo`000000oooo0<O7a`@0oooo00D000000?ooo`3`l?00
l?3`000000030?ooo`0:0000003oool000000?ooo`000000oooo07alO037alL0oooo00000080oooo
00D000000?ooo`000000oooo000000030<O7a`030?ooo`000000VYZJ00D0oooo00h0000009ZJVP3`
l?00oooo0000003oool0alO70000002JVYX0l?3`0?ooo`000000oooo000000<0alO700L0oooo0000
003oool000000?ooo`1lO7`0alO700P0oooo00@000000?ooo`3oool000001@3oool01@000000oooo
0?3`l03`l?00000000<0oooo00D000000?ooo`000000oooo000000020?ooo`050000003oool0oooo
0<O7a`0000000P3oool02@000000oooo0?ooo`000000oooo0000003oool0000009ZJVP060?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7@3oool02P37alL0hN7Q
0000003QhN40YjNW0<O7a`2JVYX0oooo0<O7a`1lO7`30000000C0;fm_@3oool000000<O7a`000000
l?3`0:NWY`000000oooo0>7Qh@2JVYX0J6QX0=WIf@000000hN7Q0000003oool0alO707alO0030000
000:0;fm_@3oool0YjNW096AT@2m_Kd0O7al0?ooo`37alL0O7al000000@0oooo00H0hN7Q0000003o
ool0VYZJ0<O7a`1XJ6P30?ooo`030:NWY`2JVYX0000000D0oooo00/0O7al0000003`l?00000008b<
S02m_Kd0alO709ZJVP3IfMT0000007alO0020:NWY`040000003`l?00J6QX0<O7a`@0oooo01<00000
0?ooo`2WYjL0VYZJ0000003oool0fMWI0000001lO7`0YjNW0?ooo`2JVYX0_Kfm0000001XJ6P0YjNW
0;fm_@37alL0VYZJ0080oooo00d0YjNW09ZJVP000000oooo0=WIf@000000YjNW0000003oool0fMWI
0;fm_@37alL0000000@0oooo0140VYZJ0<O7a`1XJ6P0YjNW0;fm_@37alL0VYZJ0:NWY`2m_Kd0alO7
09ZJVP3IfMT000000:NWY`000000_Kfm000000020>7Qh@060000003oool0fMWI0000002WYjL00000
0P3oool0302JVYX0_Kfm0000001XJ6P0oooo0000003oool0YjNW0000003`l?00J6QX0<O7a`<0oooo
0P2WYjL03@1XJ6P0_Kfm0<O7a`2m_Kd0alO70;fm_@37alL0fMWI0000002WYjL000000;fm_@000000
0P3QhN403`000000alO70:NWY`2WYjL0oooo0000003oool0YjNW0;fm_@37alL0VYZJ0<O7a`000000
_Kfm09ZJVP040?ooo`0H09ZJVP37alL0J6QX0:NWY`000000l?3`06QXJ01=CDd0alO706QXJ03QhN40
000007alO02WYjL0_Kfm0<O7a`2JVYX0fMWI0000002WYjL000000;fm_@000000hN7Q203oool01P00
0000oooo0>7Qh@000000O7al0?3`l0<0oooo0P2WYjL07@1XJ6P0_Kfm0<O7a`2m_Kd0alO70;fm_@37
alL0fMWI0000002WYjL000000?ooo`3QhN40000007alO02WYjL0VYZJ0000003oool0hN7Q0000003o
ool0fMWI0000002WYjL000000?ooo`2JVYX0alO700H0oooo00<0820P0?ooo`3oool0103oool00`0P
8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03F
e]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTM0?ooo`070=WIf@2m_Kd000000?ooo`2WYjL0hN7Q00000005
0?ooo`040000003oool0oooo00000180oooo00<000000?ooo`3oool0303oool00`3QhN40YjNW0?oo
o`0G0?ooo`030?3`l03oool0oooo00T0oooo00<000000?ooo`3oool0103oool00`3`l?00oooo0?oo
o`0G0?ooo`030000003oool0oooo00h0oooo00<0l?3`0?ooo`3oool00`3oool00`3QhN40YjNW0?oo
o`0;0?ooo`030000003oool0oooo0140oooo00<0l?3`0?ooo`3oool02@3oool00`000000oooo0?oo
o`0E0?ooo`030?3`l01XJ6P0oooo00D0oooo00<0l?3`0?ooo`3oool02`3oool010000000oooo0?3`
l01XJ6PC0?ooo`030?3`l01XJ6P0oooo00@0oooo00<0hN7Q0:NWY`3oool03@3oool00`0P8200oooo
0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M
0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAh0oooo0P0000020?ooo`030;fm_@00
0000oooo00L0oooo00<0VYZJ0;fm_@3oool0503oool00`2WYjL0oooo0?ooo`0^0?ooo`030000003o
ool0oooo01h0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo02/0oooo00<000000?ooo`3oool0:`3oool00`0P8200oooo0?ooo`040?ooo`03020P803o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJAd0oooo00<0hN7Q0000001lO7`00P3oool00`3QhN4000000;fm_@07
0?ooo`030:NWY`1XJ6P0oooo0200oooo00<000000?ooo`3oool08@3oool00`3IfMT000000?ooo`0N
0?ooo`030?3`l0000000oooo01H0oooo00<000000?ooo`3oool02@3oool00`3IfMT000000?ooo`0M
0?ooo`030=WIf@000000oooo02/0oooo00<0_Kfm0000003IfMT07@3oool00`000000oooo0?ooo`0<
0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYM03oool00`3Q
hN40oooo0?ooo`0N0?ooo`030=WIf@3oool0oooo0280oooo00<0hN7Q0?ooo`3oool07@3oool00`3Q
hN40oooo0?ooo`1F0?ooo`@0820P1P3oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocL0
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc<0oooo
1@0P82060?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M
0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?oo
o`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0
g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
och0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT
0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0
820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0
emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo?P3oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3o
ool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM
0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oooln0?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<`3oool5020P80H0
oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gM
g@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo
00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3M
gMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0
820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0820P
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0820P0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7A
d@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY3@3oool01P3YjNT0/[:b06QXJ0000000YjNW0<O7a`L0oooo00D0/[:b0000002<S8`0
/[:b0>WYj@020?ooo`090<O7a`2WYjL000000000002<S8`0alO70:NWY`000000J6QX0080fMWI00@0
O7al0000002WYjL0jNWY1@3oool02`2b/[80S8b<000000000000YjNW0<O7a`3oool0fMWI0:NWY`00
0000J6QX0080fMWI00@0O7al0000002WYjL0jNWY0P3oool02@37alL0CDe=07alO02WYjL0_Kfm0:NW
Y`000000J6QX0;:b/P040?ooo`060>7Qh@2JVYX0YjNW0:NWY`2JVYX0hN7Q103oool01`2b/[80CDe=
07alO02WYjL0VYZJ04e=C@2JVYX00`3oool0203QhN40CDe=06QXJ02b/[80YjNW07alO02b/[80jNWY
0P3oool01@3@d=00J6QX0000001=CDd0/[:b00<0oooo00T0alO704e=C@1lO7`0YjNW0;fm_@2JVYX0
000006QXJ037alL00P3oool01P3QhN40VYZJ0:NWY`2WYjL0VYZJ0>7Qh@80oooo00@0fMWI0:NWY`00
0000J6QX0P3IfMT0101lO7`000000:NWY`3YjNT50?ooo`080<O7a`1=CDd0O7al0:NWY`3QhN40oooo
0>7Qh@2JVYX20:NWY`0309ZJVP3QhN40oooo00@0oooo00D0/[:b04e=C@000000/[:b0>WYj@020?oo
o`060:NWY`3`l?00oooo0?ooo`3IfMT0YjNW0P0000001@2WYjL0d=3@0?ooo`000000O7al0080hN7Q
00<000000<O7a`3oool00P3oool00`000000S8b<0=3@d0020?ooo`0607alO02WYjL0oooo0000001l
O7`0hN7Q103oool01037alL0000008b<S03`l?020?ooo`0709ZJVP1=CDd0oooo0?ooo`3YjNT00000
0<O7a`020?ooo`0706QXJ0000000_Kfm0>WYj@3oool0000007alO0020>7Qh@0300000037alL0oooo
0080oooo00<0000008b<S03@d=000P3oool00`000000O7al0>WYj@020?ooo`070;:b/P000000O7al
0>WYj@3oool0000007alO0020>7Qh@0300000037alL0oooo02/0oooo00<0820P0?ooo`3oool0103o
ool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM
0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFT?0?ooo`030;:b/P000000oooo00T0oooo00<0alO7
0=3@d03oool0103oool00`3QhN40000007alO0030?ooo`060000002b/[80oooo0?ooo`2b/[800000
2@3oool00`000000O7al0?ooo`040?ooo`060000002b/[80oooo0?ooo`2b/[800000103oool02000
0000S8b<0?ooo`3oool0l?3`0?ooo`000000/[:b103oool00`3QhN4000000=WIf@020?ooo`030=WI
f@000000hN7Q0080oooo00<0alO70000003QhN400P3oool00`2<S8`000000?3`l0030?ooo`0606QX
J01lO7`0oooo0?ooo`2JVYX000000`3oool01`3@d=00000009ZJVP3YjNT0oooo0=WIf@2WYjL00P3o
ool020000000S8b<0?ooo`3oool0l?3`0?ooo`000000/[:b0P3oool00`3QhN4000000=WIf@020?oo
o`030=WIf@000000hN7Q00<0oooo00H000000;:b/P3oool0oooo0;:b/P0000070?ooo`080000002<
S8`0oooo0?ooo`3`l?00hN7Q0000003IfMT20?ooo`030=WIf@000000hN7Q00D0oooo00<0jNWY07al
O03oool00`3oool00`1lO7`0S8b<0?ooo`030?ooo`0309ZJVP000000oooo0080oooo00L000000;:b
/P37alL0hN7Q04e=C@2WYjL0_Kfm0080oooo00`0J6QX0:NWY`3QhN40_Kfm0?ooo`2WYjL0CDe=0?oo
o`2m_Kd0jNWY0:NWY`3`l?030?ooo`040000002JVYX0l?3`0;fm_@80oooo00<0fMWI0000003IfMT0
0P3oool0402JVYX0J6QX0?ooo`3oool0fMWI00000037alL0hN7Q0=3@d0000000/[:b0<O7a`3QhN40
CDe=0:NWY`2m_Kd20?ooo`0F06QXJ02WYjL0hN7Q0;fm_@3oool0S8b<06QXJ02b/[80hN7Q0?ooo`00
0000YjNW0?ooo`2b/[80l?3`0000002b/[80alO70>7Qh@1=CDd0YjNW0;fm_B/0oooo00<0820P0?oo
o`3oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT?0?ooo`030;:b/P000000oooo00T0
oooo00<0l?3`0:NWY`3oool0103oool00`2WYjL000000<O7a`030?ooo`060000002b/[80oooo0?oo
o`2b/[8000002@3oool00`000000O7al0?ooo`040?ooo`060000002b/[80oooo0?ooo`2b/[800000
103oool00`000000/[:b0?ooo`030?ooo`030000002b/[80oooo00<0oooo00<0S8b<04e=C@3oool0
0`3oool01P1=CDd0S8b<0?ooo`3oool0J6QX08b<S0<0oooo00<0/[:b0000003oool00`3oool01P00
0000/[:b0?ooo`3oool0/[:b000000<0oooo00<0J6QX07alO03oool01P3oool00`000000/[:b0?oo
o`030?ooo`060000002b/[80oooo0?ooo`2<S8`0CDe=103oool00`1=CDd0S8b<0?ooo`020?ooo`06
0000002b/[80oooo0?ooo`2b/[8000001`3oool00`000000/[:b0?ooo`020?ooo`0308b<S01=CDd0
oooo00<0oooo00<0CDe=08b<S03oool01@3oool00`2WYjL0oooo0?ooo`020?ooo`0306QXJ0000000
fMWI00<0oooo00<0alO70000003YjNT00P3oool209ZJVP0?0?ooo`3@d=00YjNW09ZJVP3QhN40fMWI
0?ooo`2b/[80J6QX0?ooo`3`l?00jNWY0>7Qh@000000hN7Q0080oooo00<0fMWI0:NWY`3oool00P3o
ool010000000/[:b0?ooo`3YjNT30?ooo`0704e=C@1lO7`0oooo0?ooo`3YjNT000000=WIf@020?oo
o`0I07alO01=CDd0oooo0=WIf@2ATI40VYZJ0?ooo`3@d=00YjNW09ZJVP3QhN40fMWI0?ooo`2b/[80
J6QX0?ooo`3`l?00jNWY0=3@d0000000jNWY0>7Qh@3oool000000;:b/P020?ooo`090?3`l02JVYX0
VYZJ0?ooo`3@d=00YjNW09ZJVP3QhN40fMWI02X0oooo00<0820P0?ooo`3oool0103oool00`0P8200
oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFT?0?ooo`030;:b/P000000oooo00X0oooo00<0YjNW0?ooo`3oool0
0P3oool00`3`l?000000000000040?ooo`060000002b/[80oooo0?ooo`2b/[8000002@3oool00`00
0000O7al0?ooo`040?ooo`060000002b/[80oooo0?ooo`2b/[800000103oool00`000000/[:b0?oo
o`030?ooo`030000002b/[80oooo00<0oooo00<0000007alO03oool00`3oool01P1lO7`000000?oo
o`3oool000000;:b/P<0oooo00<0/[:b0000003oool00`3oool01P000000/[:b0?ooo`3oool0/[:b
000000<0oooo00<000000;:b/P3oool01P3oool00`000000/[:b0?ooo`030?ooo`060000002b/[80
oooo0?ooo`000000O7al103oool00`1lO7`000000?ooo`020?ooo`060000002b/[80oooo0?ooo`2b
/[8000001`3oool00`000000/[:b0?ooo`020?ooo`030000001lO7`0oooo00<0oooo00<0O7al0000
003oool01@3oool00`2m_Kd0jNWY0?ooo`020?ooo`040000002<S8`0S8b<0?3`l080oooo00<0hN7Q
0000003@d=000P3oool01`3IfMT000000?ooo`3oool0VYZJ0000003`l?000P3oool00`3QhN400000
0>WYj@030?ooo`030000002JVYX0oooo0080oooo00H000000>WYj@3oool0oooo0000002b/[850?oo
o`030<O7a`000000alO70080oooo00X0O7al0000003QhN40oooo0=WIf@000000alO70?ooo`3IfMT0
00000P3oool00`2JVYX000000?3`l0020?ooo`030>7Qh@000000jNWY00<0oooo00H0CDe=0;:b/P3o
ool0oooo08b<S02WYjL30?ooo`070=WIf@000000oooo0?ooo`2JVYX000000?3`l00[0?ooo`03020P
803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY3`3oool00`2b/[8000000?oo
o`0:0?ooo`030=3@d02JVYX0/[:b0080/[:b00<0VYZJ0000002WYjL0103oool01P000000/[:b0?oo
o`3oool0/[:b000000T0oooo00<0000007alO03oool0103oool01P000000/[:b0?ooo`3oool0/[:b
000000@0oooo00<000000;:b/P3oool00`3oool00`000000/[:b0?ooo`030?ooo`030000001lO7`0
oooo00<0oooo00H0O7al0000003oool0oooo0000002b/[830?ooo`030;:b/P000000oooo00<0oooo
00H000000;:b/P3oool0oooo0;:b/P0000030?ooo`030000002b/[80oooo00H0oooo00<000000;:b
/P3oool00`3oool01P000000/[:b0?ooo`3oool0000007alO0@0oooo00<0O7al0000003oool00P3o
ool01P000000/[:b0?ooo`3oool0/[:b000000L0oooo00<000000;:b/P3oool00P3oool00`000000
O7al0?ooo`030?ooo`0307alO0000000oooo00D0oooo00<0jNWY0;fm_@3oool00P3oool010000000
_Kfm0>WYj@2b/[830?ooo`030000002WYjL0oooo0080oooo00H0O7al0<O7a`3oool0jNWY0000002m
_Kd30?ooo`0300000037alL0oooo0080oooo00<0VYZJ0000003QhN400P3oool0202JVYX0YjNW0?oo
o`3oool0S8b<06QXJ02b/[80d=3@0P3oool20?3`l00;0000002<S8`0d=3@0?ooo`3IfMT000000;:b
/P3`l?00oooo08b<S01=CDd00P3oool01P1lO7`0alO70?ooo`3YjNT000000;fm_@<0oooo00<00000
0<O7a`3oool00P3oool01P2b/[80CDe=0?ooo`3oool0alO706QXJ0@0oooo00H0O7al0<O7a`3oool0
jNWY0000002m_Kd[0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUY3`3oool00`2b/[8000000?ooo`0;0?ooo`0709ZJVP3oool0oooo0?ooo`1XJ6P000000>WYj@04
0?ooo`060000002b/[80oooo0?ooo`2b/[8000002@3oool00`000000O7al0?ooo`040?ooo`060000
002b/[80oooo0?ooo`2b/[800000103oool00`000000/[:b0?ooo`030?ooo`030000002b/[80oooo
00<0oooo00<0S8b<04e=C@3oool00`3oool0101=CDd0S8b<0?ooo`3oool209ZJVP<0oooo00<0/[:b
0000003oool00`3oool01P000000/[:b0?ooo`3oool0/[:b000000<0oooo0P2<S8`30?ooo`060=WI
f@3QhN40oooo0?ooo`000000/[:b103oool01P000000/[:b0?ooo`3oool0S8b<04e=C@@0oooo00<0
CDe=08b<S03oool00P3oool01P000000/[:b0?ooo`3oool0/[:b000000L0oooo00<000000;:b/P3o
ool00P3oool00`2<S8`0CDe=0?ooo`030?ooo`0304e=C@2<S8`0oooo00H0oooo00T0VYZJ0?ooo`3o
ool0fMWI0000003IfMT0oooo0;fm_@3QhN400P3oool207alO0<0oooo00H0jNWY06QXJ03oool0oooo
09ZJVP1=CDd30?ooo`0309ZJVP2<S8`0oooo0080oooo00L0d=3@0000002WYjL0l?3`0?ooo`2b/[80
J6QX0080oooo00h0fMWI04e=C@3oool0S8b<0;:b/P3oool0/[:b0?ooo`2WYjL0J6QX0=WIf@3QhN40
oooo04e=C@80_Kfm0P3YjNT01@000000alO70?ooo`3YjNT0J6QX0080oooo00<0VYZJ04e=C@3oool0
0P3oool0102JVYX0S8b<0?ooo`3oool20?3`l0060000003@d=00oooo0?ooo`1XJ6P0jNWY0`3oool0
1P3YjNT0J6QX0?ooo`3oool0VYZJ04e=CB/0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFT?0?ooo`030;:b/P000000oooo00/0oooo00H0YjNW0?3`l03oool0jNWY
0000001XJ6P40?ooo`030=3@d0000000O7al0080l?3`00<0CDe=06QXJ03oool0203oool00`000000
O7al0?ooo`030?ooo`030=3@d0000000O7al0080l?3`00<0CDe=06QXJ03oool00`3oool00`000000
/[:b0?ooo`020?ooo`0:0=3@d0000000CDe=0=3@d02WYjL0J6QX0?ooo`3QhN4000000=WIf@80oooo
00<0fMWI0000003QhN400P3oool00`3YjNT0CDe=0?3`l0020?ooo`030:NWY`000000oooo00<0oooo
00H000000;:b/P3oool0oooo0;:b/P0000030?ooo`030>7Qh@000000hN7Q0080oooo0P1XJ6P20?oo
o`030000002b/[80oooo0080oooo00<0l?3`0000002b/[800P3oool00`3QhN4000000=WIf@020?oo
o`030=WIf@000000hN7Q0080oooo00<0d=3@0000001lO7`00P3`l?000`1=CDd0J6QX0?ooo`060?oo
o`030000002b/[80oooo0080oooo00<0hN7Q0000003IfMT00P3oool00`3IfMT000000>7Qh@070?oo
o`050:NWY`3`l?00oooo0=WIf@0000000`3oool01@2WYjL0oooo0?ooo`2b/[80000000@0oooo00H0
alO70;:b/P3oool0fMWI0000003@d=020?ooo`030<O7a`000000oooo0080oooo00L0l?3`0000002m
_Kd0YjNW0>WYj@2WYjL0000000<0oooo00H0VYZJ0=WIf@3IfMT000000?ooo`2b/[820>7Qh@090000
003YjNT0d=3@0>7Qh@2b/[80J6QX0?ooo`2b/[80alO70080J6QX0P3oool01P37alL0/[:b0?ooo`3I
fMT000000=3@d080oooo00P0alO70000003oool0oooo0>7Qh@37alL0VYZJ08b<S080oooo00`0hN7Q
0;:b/P3oool0/[:b0=3@d03oool0alO70;:b/P3oool0fMWI0000003@d=0Z0?ooo`03020P803oool0
oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY3`3oool00`2b/[8000000?ooo`0;0?oo
o`060>7Qh@37alL0oooo0;:b/P000000alO70`3oool00`3`l?00/[:b06QXJ0020;:b/P030000001=
CDd0hN7Q00T0oooo00<0000007alO03oool00P3oool00`3`l?00/[:b06QXJ0020;:b/P030000001=
CDd0hN7Q0080oooo00@0l?3`08b<S0000000CDe=0P1lO7`01`3@d=00/[:b06QXJ02b/[80alO70000
001XJ6P00P3oool01P3QhN40VYZJ0:NWY`2WYjL0VYZJ0>7Qh@@0oooo00H0hN7Q0:NWY`2b/[80/[:b
04e=C@0000020?ooo`090=WIf@2<S8`000000;:b/P3oool0/[:b06QXJ0000000jNWY00<0oooo00X0
hN7Q09ZJVP2WYjL0YjNW08b<S03QhN40l?3`08b<S0000000CDe=0P1lO7`0103@d=00O7al0000002b
/[830?ooo`060>7Qh@2JVYX0YjNW0:NWY`2JVYX0hN7Q0P3oool00`3`l?00/[:b06QXJ0020;:b/P03
0000001=CDd0hN7Q00D0oooo00@0l?3`08b<S0000000CDe=0P1lO7`0103IfMT0oooo0>7Qh@2JVYX2
0:NWY`0309ZJVP3QhN40oooo00L0oooo00D0fMWI0<O7a`3oool0_Kfm000000030?ooo`060>WYj@2m
_Kd0oooo0=3@d0000000fMWI103oool00`3@d=00/[:b09ZJVP0208b<S0080?ooo`2<S8`0J6QX0000
001XJ6P0O7al0=WIf@3oool208b<S0040?ooo`2b/[80CDe=07alO0<0oooo00`0l?3`0;:b/P2<S8`0
S8b<0?ooo`3`l?00S8b<0;:b/P000000_Kfm0?ooo`3@d=0204e=C@80oooo00<0d=3@04e=C@000000
0`3oool00`3@d=00/[:b09ZJVP0208b<S00:0?ooo`2<S8`0J6QX0000001XJ6P0O7al0=WIf@3YjNT0
S8b<000000<0oooo00@0hN7Q0=3@d0000000/[:b0P3oool00`3@d=00/[:b09ZJVP0208b<S2X0oooo
00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT?0?ooo`030;:b/P00
0000oooo00`0oooo00D0/[:b0?ooo`1=CDd000000?3`l0060?ooo`030?3`l03oool0oooo00/0oooo
00<0000007alO03oool01@3oool00`3`l?00oooo0?ooo`050?ooo`030>7Qh@000000/[:b00D0oooo
00<0l?3`0?ooo`3oool04@3oool00`2b/[8000000?ooo`0D0?ooo`030>7Qh@000000/[:b00@0oooo
00<0jNWY0=WIf@3oool03@3oool00`3`l?00oooo0?ooo`080?ooo`030>7Qh@000000/[:b01<0oooo
00@0S8b<0?ooo`2b/[80O7al103oool01@37alL0jNWY0>WYj@000000_Kfm00`0oooo00<0S8b<0:NW
Y`3oool00P3oool00`2m_Kd000000?ooo`0T0?ooo`0308b<S02WYjL0oooo03`0oooo00<0820P0?oo
o`3oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT?0?ooo`030;:b/P000000oooo00`0
oooo00@0_Kfm0=WIf@000000VYZJ5@3oool00`000000O7al0?ooo`0>0?ooo`030>7Qh@37alL0oooo
01P0oooo00<0/[:b0000003oool05@3oool00`3QhN40alO70?ooo`0O0?ooo`030>7Qh@37alL0oooo
0180oooo00@0VYZJ0>WYj@1lO7`0YjNW1@3oool0102b/[80oooo0000002JVYX<0?ooo`030<O7a`1=
CDd0oooo0080oooo00<0jNWY0000003@d=00903oool00`37alL0CDe=0?ooo`0l0?ooo`03020P803o
ool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z
01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY3@3oool0103YjNT0/[:b06QXJ000
000=0?ooo`040?3`l000000000000=3@d1D0oooo00<0000007alO03oool0:@3oool00`2b/[800000
0?ooo`1<0?ooo`040<O7a`2b/[80J6QX0;:b/PD0oooo00@0jNWY0;fm_@2<S8`0J6QX4P3oool00`1X
J6P0VYZJ0?ooo`1R0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUY3P3oool00`3QhN40S8b<0000000>0?ooo`0308b<S0000000oooo01D0oooo00<0000007alO03o
ool0:@3oool00`2b/[8000000?ooo`0J0?ooo`0308b<S02b/[80oooo02l0oooo00@0l?3`00000000
0000fMWI1P3oool0102m_Kd0S8b<0000003`l?0A0?ooo`030:NWY`1=CDd0oooo02T0oooo00<0YjNW
0<O7a`3oool0=P3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JA00oooo00<0VYZJ0?ooo`3oool0303oool00`3@d=00YjNW0?ooo`0C0?ooo`060;:b/P2<S8`00000
0000002WYjL0_Kfm9`3oool00`37alL0J6QX0000000K0?ooo`0307alO02WYjL0oooo02h0oooo00D0
/[:b06QXJ000000000000>WYj@070?ooo`0406QXJ0000000O7al0;:b/Pl0oooo00@0fMWI07alO000
0000hN7Q:@3oool00`1XJ6P0YjNW0?ooo`0f0?ooo`03020P803oool0oooo00@0oooo00<0820P0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYH`3oool00`3@d=00S8b<0?ooo`3@0?ooo`03020P803oool0oooo00@0
oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gM
g@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo
00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3M
gMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0
820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolg0?ooo`03020P803oool0oooo00@0oooo00<0820P
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooold0?ooo`@0820P1P3oool00`0P8200oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0d=3@0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocL0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oooc<0oooo1@0P82020?ooo`D0820P103oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OG
e`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oood<0oooo00<0k>c/0?gmo@2CTi<01@2h^;P00`2e]KD0^;Rh0;Rh^0040;Rh^00F09>C
T`3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oood<0oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`030>c/
k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oood<0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`030>c/
k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo@`3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oood<0oooo00<0k>c/09JFUP3oool00P3>c/h00`0n?Sh0c/k>0<k>cP030<k>cP0J03hn
?P3>c/h0c/k>0<k>cP3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`040>c/k02FUYH0oooo0<o?c`<0?Shn0`3?cll3
03hn?P80clo?01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo@`3oool00`3/k>`0UYJF0?ooo`020=CDe0<0?Shn00@0e=CD
03hn?P0n?Sh0?Shn0`3De=@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`030>c/k02FUYH0oooo00<0fm_K1@0n
?Sh40=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oood<0oooo00<0k>c/09JFUP3oool0103Ti>@303hn?PD0i>CT01H0
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo@`3oool00`3/k>`0UYJF0?ooo`050>g]k@0303hn?P3]kNd0kNg]00@0kNg]01H0
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo@`3oool00`3/k>`0UYJF0?ooo`0<0?Cdm00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oood<0oooo00<0
k>c/09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooom30?ooo`030>c/k02FUYH0oooo00d0oooo01D0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo@`3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo000004<0000000@0k>c/
0=[JfP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo00000`000000moOg09ZJVP3o07YjNUH0NWYj01<0VYZJ0?Ogm`000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo00000`000000moOg0;no_`3o0;no_eL0_kno0180moOg0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`3Shn<0o`3Shn=G0>?Sh`0B0?Ogm`000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000030000003alO40oooo0?l0oooo403oool00`3Shn<0YJFU07If
MP0307IfMP0407MgM`1fMWH0MgMg07MgM`T0MWIf00H0YJFU0>?Sh`3oool0oooo0>?Sh`2UYJD407If
MP0407MgM`1fMWH0MgMg07MgM`T0MWIf00H0YJFU0>?Sh`3oool0oooo0>_[j`2UYJD405IFEP0405MG
E`1FEUH0EeMG05MGE`T0EUIF00<0YJFU0>_[j`3oool00`3oool04P3alO40000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000hN7Q0?clo03kno/20?clo0030?_kn`3lo?`0o?cl00@0o?cl00<0
no_k0?clo03lo?`00P3lo?`00`3kno/0o?cl0?_kn`050?clo0030?_kn`3lo?`0no_k05h0no_kU`3l
o?`00`2UYJD0MWIf08f=S@0@08f=S@0607IfMP2UYJD0o?cl0?clo02UYJD0MWIf4@2=SHd01P1fMWH0
YJFU0?clo03kno/0YJFU05IFEQ40Fe]K00<0EUIF0:FUY@3lo?`00`3lo?`04P3QhN40000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo00000`000000h>3P0?Ogm`3o0?Ogma00moOg00<0MWIf08f=S@2RXZ80
402RXZ801P2=SHd0MWIf0?Ogm`3gmoL0MWIf08f=SA40XZ:R00P0SHf=07IfMP3gmoL0moOg05IFEP1K
Fe/0Kfm_075aL@d0LW9b00@0LG5a06m_K`1KFe/0EUIF103gmoL04P3Ph>00000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo00000`000000g]kN0?7al@020?7al@80l_;b00<0lO7a0?;blP3bl_80103alO40
0`3bl_80lO7a0?7al@030?7al@80l_;b00<0lO7a0?;blP3alO400P3bl_;g0?7al@0307IfMP2h^;P0
Zj^[0100Zj^[00H0^;Rh07IfMP3alO40lO7a07IfMP2h^;PA0:^[Z`050;Rh^01fMWH0lO7a0?;blP1F
EUH00P1_Kfl00`1bLW80MgMg07MgM`0;07MgM`04079bLP1_Kfl0Kfm_05IFEP<0lO7a01<0l_;b0=kN
gP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001@000000g=cL0>_[j`3[jn/0k>c/0080jn_[
00<0YjNW0::RXP2RXZ800P2QXJ400`2PX:00WinO09fMW@0209nOW`80X:2P0P2QXJ400`2WYjL0jn_[
0>_[j`050>_[j`040<k>cP35aLD0aLG50=3@d0D0jn_[00<0iNGU0<O7a`3GemL00`3[jn/01P3>c/h0
c<c<0>[ZjP3Zj^X0b<S80=3@d080jn_[00D0f][J0<K6aP3Shn<0jn_[0>3Ph0020<G5a@040<c<c03Y
jNT0b<S80=3@d0@0jn_[00D0h>3P0<G5a@35aLD0aLG50=gMg@020>_[j`030>[ZjP38b<P0aLG50080
aLG500@0a/K60>?Sh`3[jn/0h>3P1035aLD01@36a/H0hn?S0>_[j`3Zj^X0b<S80080aLG500@0emOG
0>_[j`3[jn/0h>3P0P35aLD01`3<c<`0jNWY0<k>cP37alL0jNWY0>_[j`3Ph>000`35aLD01@3MgMd0
jn_[0>[ZjP38b<P0d=3@0080jn_[00/0f][J0<K6aP3Shn<0jn_[0=CDe035aLD0g]kN0>c/k03[jn/0
b<S80=3@d0020>c/k0050=_Kf`36a/H0hn?S0>KViP37alL00`35aLD00`3<c<`0jn_[0>c/k01<0>c/
k3T0jn_[00@0MWIf0;^k^`2a/K40/K6a203oool70;6a/@060;^k^`1fMWH0jn_[0>_[j`1fMWH0^k^k
0P2a/K4<0?ooo`<0/K6a00T0^k^k07IfMP3[jn/0jn_[05IFEP1gMgL0LW9b07AdM01gMgL02`1hN7P0
1@1gMgL0M7Ad079bLP1gMgL0EUIF00@0jn_[0180g=cL0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000<000000=[JfP3UiND00`3UiND00`2:RXX0Vi^K0:b/[0030>WYj@0:0>[ZjP34a<@0QHF50>c/
k03]kNd0k^k^0>OWi`3`l?00l_;b0:>SX`D0iNGU00L0hN7Q04];B`0@4100410@05]KF`37alL0clo?
00<0iNGU01<0bLW901`L701bLW80e=CD0>GUi@3Rh^80D51@03Ti>@35aLD0gMgM02HV9P1ADE40clo?
0>GUi@2DU9@05ADE0:BTY03Ng]h0/[:b0080410@00D0@D510=OGe`0V9RH0DE5A0<o?c`030>GUi@09
0:n_[`0@4100410@010@402IVIT0a/K60=kNgP3MgMd092@T00<0410@00@05ADE0:BTY03Ng]h0/[:b
100@41001@0E5AD0Y:BT0=kNgP3MgMd092@T0080410@00@0Ph>30<g=c@3IfMT0/[:b0P0@41001`11
@D40fm_K051@D00I6AT0^[Zj0>?Sh`2_[jl00`0@41006@2IVIT0a/K60=OGe`0V9RH0DE5A0<o?c`3U
iND0U9BD01DE5@2TY:@0g]kN071`L00@4100SXj>0=WIf@3MgMd09RHV055AD@3?cll0iNGU09BDU00E
5AD0Y:BT0<;2`P0L71`00`0@41001010@400a/K60<K6aP3Rh^:50>GUi@0507IfMP30`<00]KFe0;Fe
]@3oool01P1bLW800`0W9bL0]KFe0;Fe]@050;Fe]@060<30`01fMWH0iNGU0>GUi@1fMWH0`<300P2e
]KD00`3oool09bLW02LW9`0902LW9`<0]KFe00D0`<3007IfMP3UiND0iNGU05IFEP0207MgM`0407Ui
N@1jNWX0oooo0?ooo`L0NWYj0P3oool01@1jNWX0NGUi07MgM`1gMgL0EUIF00@0iNGU0180f][J0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=WIf@3Ogml00`3Ogml01@3Ng]h0X:2P0>[Z
jP3YjNT0iNGU0080j^[Z00X0d=3@07AdM03]kNd0k^k^0:f][@3@d=00l_;b0??cl`2SXj<0gmoO0P3N
g]h02P3Ogml0g]kN06][J`0I6AT0aLG50=OGe`0U9BD0EEEE0<K6aP3MgMd20=kNgP0J0<?3``0L71`0
LW9b0=7Ad@3Ng]h0fm_K04i>CP0i>CT0a<C40=KFeP0U9BD0DE5A0<g=c@3Ng]h0Shn?01DE5@2TY:@0
eMGE04i>CP0h>3P0a<C40=gMg@3GemL09BDU055AD@3=cLd20=kNgP1<0<?3``0K6a/0LW9b0=WIf@2?
Shl05ADE0:FUY@36a/H09BDU01TI6@35aLD0g]kN08n?S`0E5AD0Y:BT0=;BdP0U9BD0>3Ph0<O7a`3K
fm/0CTi>01DE5@2TY:@0d];B02DU9@0h>3P0clo?0=_Kf`1>CTh0WinO0=_Kf`1>CTh0>3Ph0<C4a03M
gMd0fm_K04i>CP0I6AT0^[Zj0<;2`P0K6a/0LW9b0=WIf@2?Shl05ADE0:FUY@36a/H09BDU055AD@3=
cLd0g]kN08n?S`0E5AD0Y:BT0=WIf@1/K6`0410@09nOW`3Ng]h0emOG02DU9@1ADE40cLg=0=kNgP2?
Shl05ADE0:BTY02n_[h071`L079bLP3AdM40fm_K04i>CP0I6AT0^KVi0=GEeHD0g]kN00D0MWIf0<?3
``2k^k/0^k^k0?ooo`06079bLP0302LW9`2k^k/0^k^k00D0^k^k00H0`l?307IfMP3Ng]h0g]kN07If
MP33`l<20;^k^`030?ooo`0W9bL0^k^k00L0^k^k00<0oooo02LW9`2k^k/00P2k^k/02@33`l<0MWIf
0=oOg`3Ng]h0EUIF07MgM`1hN7P0Ng]k030`<0030?ooo`D0OGem0`3oool01@0`<300Ng]k07QhN01g
MgL0EUIF0080g]kN01@0gmoO0=kNgP3IfMT0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000emOG0=SHf03IfMT20=SHf00@092@T02?Shl0SXj>0>WYj@21PH40DE5A08N7Q`3Jf]X0YZJV0:VY
Z@2/[:`0GemO0?;blP3clo<0m?Cd0:BTY0<0f=SH00X0fMWI0;jn_P0K6a/0LG5a0=7Ad@3Hf=P0Rh^;
01TI6@2i^KT0dM7A0P3Hf=P06P2m_Kd06a/K079bLP3>c/h0f=SH0=GEe@1<C4`0>CTi0<?3``3AdM40
92@T055AD@3;bl/0f=SH08^;R`0E5AD0Y:BT0<g=c@0T92@0>3Ph0<?3``3GemL0dM7A02@T901ADE40
bl_;0P3Hf=P05P2;Rh/05ADE0:BTY03GemL0dM7A02@T901ADE40WYjN010@401aLG40dM7A0=SHf02;
Rh/05ADE0:BTY03=cLd092@T055AD@3;bl/0f=SH08^;R`0E5AD20:BTY003010@402>SXh0e=CD00<0
f=SH01T0dM7A02@T900h>3P0`l?30=OGe`3EeMD0C4a<01TI6@2j^[X0RXZ:01DE5@2TY:@0emOG0=7A
d@0T92@0DE5A0<C4a00T92@0DE5A0<_;b`3Hf=P0Rh^;01DE5@2TY:@0e=CD00<0f=SH0180fMWI0=;B
dP0T92@0DE5A0<_;b`3IfMT0Rh^;01DE5@2TY:@0^[Zj01/K6`1bLW80c/k>0=WIf@2YZJT0410@08j>
SP3>c/i<0=WIfCT0f=SH00D0MWIf0<O7a`2o_kl0_kno0?ooo`0702LW9`L0_kno00H0alO707IfMP3H
f=P0f=SH07IfMP37alL20;no_`030?ooo`0W9bL0_kno00L0_kno00<0oooo02LW9`2o_kl00P2o_kl0
1@37alL0MWIf0=SHf03Hf=P0EUIF0080Ng]k00<0OWin07moO`0X:2P00`3oool307moO`<0oooo00<0
:2PX07moO`1nOWh00P1kNg/00`1FEUH0fMWI0=SHf0020=SHf00B0=OGe`000000VYZJ0:2PX02XZ:P0
/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000030000003EeMD0dm?C00<0dm?C00<0QHF50:>SX`3Vi^H00P3Zj^X02`0e=CD0
Ogmo0<o?c`2n_[h0Rh^;05iNGP1HF5P0J6QX0:>SX`39bLT0ZZZZ00@0dm?C00T0Y:BT010@402>SXh0
c/k>0=?Cd`2TY:@0410@08j>SP3<c<`00P3Cdm<06P2h^;P06a/K079bLP3<c<`0dm?C0<o?c`1:BTX0
>CTi0<71`@3<c<`092@T055AD@3:b/X0dm?C08R8R00E5AD0Y:BT0<W9b@0T92@0>3Ph0<71`@3Cdm<0
c<c<02@T901ADE40b/[:0P3Cdm<06`1WIfL06ATI0;Vi^@3@d=00c<c<02@T901ADE40WIfM010@402>
SXh0c<c<0=?Cd`28R8P05ADE0:BTY039bLT092@T055AD@3:b/X0dm?C08R8R00E5AD0Y:BT08J6QP0E
5AD0Y:BT0<k>cP030=?Cd`0I0<c<c00T92@0>3Ph0<71`@3Cdm<0clo?04Y:BP0I6AT0^KVi06MWI`0I
6AT0^KVi0=3@d03<c<`092@T055AD@33`l<092@T055AD@3:b/X0dm?C08R8R00E5AD0Y:BT0=3@d004
0=?Cd`0A0<c<c00T92@0DE5A0<[:bP3Cdm<0R8R801DE5@2TY:@0][Jf01/K6`1bLW80c<c<0=?Cd`2h
^;P06a/K075aL@3=cLd0Q@3Cdm<00`1fMWH0bl_;0<K6aP0@0<K6aP060<_;b`1fMWH0dm?C0=?Cd`1f
MWH0bl_;0P36a/H00`3oool09bLW0<K6aP070<K6aP030?ooo`0W9bL0a/K60080a/K600L0bl_;07If
MP3Cdm<0dm?C05IFEP1mOGd0Ogmo00<0Ph>300D0:2PX0?ooo`3oool0oooo08>3P`030?ooo`0702PX
:023Ph<0Ph>308>3P`1oOgl0OGem05IFEP040=?Cd`0A0=GEe@000000X:2P0:FUY@2][Jd0][Jf0;no
_`38b<P0d];B0=[JfP3Rh^80jNWY0>o_k`3dm?@0m_Kf0?Win@3kno/00`3moOd10?knoP40oooo0@3o
ool000050000003Cdm<0c/k>0<k>cP3?cll00P3>c/h03`27QhL0Ti>C0=?Cd`3Kfm/0[Jf]0<71`@2`
/;00ZJVY0:ZZZP2=SHd0TI6A0;2`/03;bl/0kno_0:FUY@030<k>cP0:0<o?c`25QHD0410@08j>SP3;
bl/0c/k>0:2PX00@4100LG5a0<[:bP80c/k>01X0];Bd01XJ6P1bLW80b/[:0<k>cP3;bl/0BDU903Ti
>@31`L40alO702<S8`1ADE40alO70<k>cP25QHD05ADE0:BTY037alL08b<S03Ph>031`L40c/k>0<O7
a`0S8b<0DE5A0<G5a@80cLg=01/0Q8B401DE5@2TY:@0b/[:0<O7a`0S8b<0DE5A0;6a/@0J6QX0LW9b
0<G5a@3>c/h0QHF501DE5@2TY:@0a/K602<S8`1ADE40b<S80<k>cP25QHD05ADE0:BTY02OWil0410@
08j>SP37alL00P3>c/h06@38b<P0a/K602<S8`0h>3P0`L710<k>cP3;bl/0BDU901TI6@2i^KT0Q8B4
01DE5@2TY:@0b/[:0<O7a`0S8b<0DE5A0<C4a00S8b<0DE5A0<O7a`3>c/h0QHF501DE5@2TY:@0103>
c/h04P3?cll0b<S802<S8`1ADE40alO70<k>cP25QHD05ADE0:BTY02d];@06QXJ079bLP3:b/X0clo?
0:6QX@0@4100SXj>0<k>cT`0clo?>@3>c/h00`1fMWH0c/k>0<_;b`0@0<_;b`060<k>cP1fMWH0c/k>
0<k>cP1fMWH0c/k>0P3;bl/00`3oool09bLW0<_;b`070<_;b`030?ooo`0W9bL0bl_;0080bl_;00T0
c/k>07IfMP3?cll0c/k>05IFEP1mOGd0PX:208F5Q@24Q8@00P25QHD00`0X:2P0oooo0?ooo`030?oo
o`0802PX:025QHD0QHF508B4Q025QHD0PX:207emO@1FEUH40<k>cP0A0=?Cd`000000Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Win@3lo?`00`3moOd10?kn
oP40oooo0@3oool000030000003@d=00bLW900<0bLW90140Lg=c09BDU01nOWh0NGUi0:2PX03AdM40
[Jf]0:FUY@2VYZH0QHF505iNGP2][Jd0iNGU0?KfmP3hn?P0Y:BT0<W9b@030<S8b01M0861P@0@4100
SXj>0<O7a`38b<P0W9bL010@401aLG40alO70<S8b037alL0[Zj^01XJ6P1aLG40alO70<S8b035aLD0
ATI603Ph>030`<00`/;2028R8P0h>3P0`L710<;2`P16ATH06ATI0;bl_031`L408b<S03Ph>02o_kl0
aLG50<30`00R8R805ADE0:FUY@2a/K40Zj^[0:j^[P0J6QX0Lg=c0<G5a@1oOgl0410@08n?S`35aLD0
ATI601TI6@2i^KT0aLG507moO`0E5AD0Y:BT0<71`@0R8R80DE5A0<O7a`38b<P0PH6101DE5@2TY:@0
`L71028R8P0i>CT0`<300<G5a@1WIfL0YJFU0<71`@0S8b<0>3Ph0;no_`35aLD0a<C404I6AP0I6AT0
^KVi0:n_[`0J6QX0Lg=c0<G5a@1oOgl0410@08n?S`32`/808R8R03Ph>031`L40`/;204I6AP0I6AT0
_;bl00@0b<S80140bLW90<;2`P0R8R80>3Ph0<71`@32`/80ATI601TI6@2l_;`0/;2`01XJ6P1aLG40
aLG50<;2`P16ATH06ATI0;fm_@1=0<W9bCT0b<S800<0MWIf0=?Cd`3=cLd0403=cLd01P3Cdm<0MWIf
0<S8b038b<P0MWIf0=?Cd`80cLg=00<0oooo02LW9`3=cLd01`3=cLd00`3oool09bLW0<g=c@020<g=
c@070=?Cd`1fMWH0b<S80<S8b01FEUH0PX:208>3P`0508N7Q`0502PX:03oool0oooo0?ooo`0X:2P0
1@27QhL00`23Ph<0PX:205IFEP030<W9b@0B0<S8b03@d=0000000;:b/P2f][H0^k^k0<;2`P39bLT0
dM7A0=WIf@3Ph>00inOW0>g]k@3alO40mOGe0?Shn03jn_X0o?cl0`3moOd10?knoP40oooo0@3oool0
00030000003?cll0aLG500@0aLG500l0UiNG0;Ng]`3/k>`0^KVi08^;R`2g]kL0ZZZZ0:BTY01nOWh0
LG5a0<C4a03IfMT0m_Kf0?Win@2UYJD01035aLD04`2IVIT0410@08f=S@36a/H0aLG509VIV@0@4100
SXj>0<K6aP2/[:`0G5aL0:f][@0J6QX0LG5a0<K6aP35aLD0`/;204E5A@0h>3P00P2o_kl0200R8R80
GEeM010@400@41005ADE0:BTY032`/80ADE50`0@41002P1ADE40_kno028R8P1>CTh0ADE5010@401A
DE40aLG50:b/[00J6QX2010@400508f=S@35aLD0aLG50<;2`P15ADD00`0@41001@0E5AD0Y:BT0;no
_`0R8R80DE5A0080aLG500H0Ogmo01DE5@2TY:@0aLG50<;2`P15ADD2010@400303Ti>@2l_;`0ADE5
00<0410@00P0DE5A065QH@0@41006ATI0;Vi^@35aLD0[:b/01XJ6P80410@00D0SHf=0<G5a@2o_kl0
8R8R05eMG@02010@400301DE5@2TY:@0aLG500D0aLG500<0_kno028R8P1MGEd00P0@41001@0E5AD0
Y:BT0<G5a@2/[:`06QXJ00<0410@00<06ATI0;Vi^@35aLD0QP35aLD00`1fMWH0eMGE0=7Ad@0@0=7A
d@060=GEe@1fMWH0aLG50<G5a@1fMWH0eMGE0P3AdM400`3oool09bLW0=7Ad@070=7Ad@030?ooo`0W
9bL0dM7A0080dM7A00L0eMGE07IfMP35aLD0aLG505IFEP22PX80QhN700D0RHV91@3oool508V9R@03
08N7Q`22PX80EUIF00@0aLG50140clo?0000002l_;`0_kno0<?3``39bLT0d=3@0=OGe`3MgMd0i>CT
0>[ZjP3_knl0lo?c0?KfmP3hn?P0no_k0?clo0020?gmo@80o_kn0P3oool000030000003=cLd0`L71
00<0`L710100MGEe09VIV@2WYjL0kNg]0<W9b@1cLg<0/;2`08f=S@1cLg<0C4a<069RHP2n_[h0]KFe
0;:b/P2][Jd0ZJVY1031`L402`2k^k/08R8R075aL@35aLD0`/;207alO00E5AD0Xj>S0<;2`P2YZJT0
6QXJ0080410@00P0LG5a0<C4a031`L40_[jn04A4A00h>3P0_[jn0<;2`PL0`L7100D0^k^k028R8P0h
>3P0_[jn0<;2`P0B0<71`@0407alO00E5AD0Y:BT0<?3``h0`L7100D0^k^k028R8P0h>3P0_[jn0<;2
`P020<71`@030<?3``35aLD0`/;201/0`L7100@0ZJVY01XJ6P1aLG40a<C4RP31`L400`1fMWH0f=SH
0=GEe@0@0=GEe@060=SHf01fMWH0`L710<71`@1fMWH0f=SH0P3EeMD00`3oool09bLW0?ooo`080?oo
o`0602LW9`3EeMD0eMGE0=GEe@3Hf=P0MWIf0P31`L400`1FEUH0Q8B408R8R00408Z:RP<0oooo00@0
:2PX0?ooo`3oool0oooo102:RXX00`28R8P0Q8B405IFEP040<71`@0@0<g=c@000000a/K60<S8b03<
c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?oo
o`0000<000000<c<c02j^[X00P2m_Kd04@2n_[h0_Kfm09jNWP3Shn<0][Jf09JFUP22PX80Ti>C08B4
Q022PX80CTi>02`/;02LW9`0Zj^[0?KfmP3hn?P0YjNW00D0_Kfm00L0NWYj01TI6@2f][H0`/;204I6
AP0h>3P0^;Rh0080_Kfm00T0YJFU01XJ6P0@4100L71`0<;2`P2m_Kd0^[Zj04=3@`0h>3P00P2n_[h8
0;fm_@0305eMG@0g=cL0^;Rh01<0_Kfm00@0NWYj01DE5@2TY:@0_kno3`2m_Kd00`1MGEd0=cLg0;Rh
^0020;fm_@0405eMG@0I6AT0^KVi0;no_a80_Kfm2@2n_[h0102VYZH06QXJ075aL@32`/9A0;jn_ST0
_Kfm00<0MWIf0=[JfP3EeMD0403EeMD01P3Jf]X0MWIf0;fm_@2m_Kd0MWIf0=[JfP80eMGE00<0oooo
079bLP1bLW80201bLW803P0W9bL0eMGE0=GEe@3EeMD0f][J07IfMP2m_Kd0_[jn05IFEP27QhL0RXZ:
08f=S@2>SXh0SHf=0`3oool00`0X:2P0SHf=02PX:0030?ooo`<0SHf=00@0RXZ:08N7Q`1FEUH0_[jn
0P2m_Kd04@2j^[X0c<c<0000003@d=00d];B0=GEe@3IfMT0gMgM0>;RhP3WinL0k>c/0?3`l03dm?@0
m_Kf0?Shn03jn_X0o?cl00<0oOgm00<0o_kn0?ooo`3oool00@3oool000030000003:b/X0^;Rh00<0
^[Zj0100O7al08f=S@1hN7P0VIVI0?3`l032`/80YjNW0<_;b`2l_;`0Xj>S06e]K@1mOGd0[:b/0?_k
n`3lo?`0YZJV0`2j^[X0102k^k/0^[Zj0;Zj^P1KFe/2010@400303Lg=`2d];@0^[Zj00@0^[Zj00P0
Fe]K06][J`2j^[X0^[Zj0;Ng]`12@T80=cLg0;Bd]1l0^[Zj00<0MgMg01@D502KVi/05@2j^[X00`1K
Fe/06ATI0:n_[`0L0;Zj^P030::RXP0I6AT0Jf][08/0^[Zj00<0MWIf0=SHf03Jf]X0403Jf]X01P3H
f=P0MWIf0;Zj^P2j^[X0MWIf0=SHf080f][J00<0oooo079bLP1bLW80201bLW803@0W9bL0f][J0=[J
fP3Jf]X0f=SH07IfMP2j^[X0^k^k05IFEP2;Rh/0Shn?08j>SP2?Shl00`3oool01@0X:2P0Shn?08n?
S`2?Shl0:2PX00<0oooo00D0Shn?08j>SP2?Shl0Rh^;05IFEP030;Zj^P0@0;Rh^03:b/X000000=WI
f@3Jf]X0gMgM0>3Ph03Ti>@0j>SX0>c/k03_knl0lo?c0?KfmP3hn?P0n_[j0?clo0<0oOgm0P3no_h3
0?ooo`0000<000000<W9b@2h^;P00`2h^;P04026QXH0WYjN0=CDe03`l?00l_;b0<k>cP31`L40gMgM
0=;BdP2RXZ80c<c<0<W9b@29RHT0o?cl0?gmo@2WYjOl0;Rh^00307IfMP3GemL0gMgM0100gMgM00H0
emOG07IfMP2h^;P0^;Rh07IfMP3GemL30=gMg@/09bLW0`3MgMd02@3GemL0MWIf0;Rh^02h^;P0EUIF
092@T02ATI40Ti>C09BDU0020?ooo`0302PX:02HV9P0V9RH00<0V9RH00P0:2PX0?ooo`3oool0U9BD
09>CT`2ATI40T92@05IFEP@0^;Rh00l0bLW90000003QhN40h^;R0>CTi03WinL0j^[Z0>g]k@3`l?00
lo?c0?Gem@3gmoL0nOWi0?_kn`3lo?`00`3moOd00`3no_h0oooo0?ooo`020?ooo`0000<000000<W9
b@2h^;P0102h^;P03`26QXH0U9BD0?;blP3clo<0X:2P0>SXj03fm_H0fm_K09VIV@3jn_X0no_k0=cL
g02f][H0o_kn0:NWY`3l0;Rh^00307IfMP3Fe]H0hN7Q0100hN7Q00H0e]KF07IfMP2h^;P0^;Rh07If
MP3Fe]HA0>7Qh@090=KFeP1fMWH0^;Rh0;Rh^01FEUH0UIFE09NGU`2LW9`0XZ:R0080:2PX00D0Zj^[
0:f][@2[Zj/0Zj^[0:f][@020:^[Z`80:2PX00D0XZ:R09bLW02GUiL0UIFE05IFEP040;Rh^00>0<W9
b@000000j>SX0>WYj@3[jn/0kNg]0>o_k`3alO40m?Cd0?KfmP3gmoL0nOWi0?_kn`3lo?`30?gmo@80
o_kn103oool0000300000039bLT0]KFe00<0]KFe00d0MgMg0:2PX03Ph>00lo?c0>GUi@3/k>`0m_Kf
0?Shn03inOT0T92@0?_kn`3lo?`0oOgm0080o_kn00<0YjNW0;Fe]@2e]KD0nP2e]KD00`1fMWH0kNg]
0>WYj@0@0>WYj@060>g]k@1fMWH0]KFe0;Fe]@1fMWH0kNg]4@3YjNT02P3]kNd0MWIf0;Fe]@2e]KD0
EUIF09^KV`2NWYh0YJFU0;:b/P2j^[X20;no_`040<30`02o_kl0_kno0<30`080_kno00L0`<300;Zj
^P2b/[80YJFU09jNWP2KVi/0EUIF00@0]KFe00d0bLW90000003^k^h0kno_0?3`l03alO40lo?c0?Ge
m@3fm_H0n?Sh0?Win@3kno/0o?cl00<0oOgm0P3no_h50?ooo`0000<000000<W9b@2j^[X0102a/K40
0`2=SHd0ZZZZ0:ZZZP050:ZZZP030:NWY`2ZZZX0ZZZZ00@0ZZZZo02a/K400`1fMWH0e=CD0?;blP0@
0?;blP060=CDe01fMWH0/K6a0;6a/@1fMWH0e=CD4@3bl_801P3De=@0MWIf0;6a/@2a/K40EUIF0:>S
X`80Z:RX3@33`l<20:RXZ0030:>SX`1FEUH0/K6a0080/K6a00@0^[Zj0<W9b@000000lo?c0P3dm?@0
1`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3lo?`0103moOd00`3no_h0oooo0?ooo`040?ooo`0000<0
00000<?3``39bLT0o`2^[Zh@0:j^[P0408f=S@1fMWH0c/k>0?_kn`d0oooo00@0no_k0<k>cP1fMWH0
SHf=0P2^[Zh0102=SHd0MWIf0<k>cP3kno/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00H0VIVI
05IFEP2e]KD0bl_;0=oOg`3Xj>P90>OWi`060>SXj03Ogml0bl_;0;Fe]@1FEUH0VIVI0`2^[Zh00`39
bLT0`l?3000000020?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool0
000400000033`l<0e=CD0;Ng]ol0YjNW402WYjL00`2=SHd0MWIf07IfMP0?07IfMP0308f=S@2WYjL0
YjNW0080YjNW00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=S@1FEUH0
EUIF00l0EUIF00P0SHf=0:NWY`2WYjL0YjNW0;Ng]`3De=@0`l?3000000<0nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00001P000000ZZZZ0>OWi`3De=@0_[jn0;>c/ol0ZZZZD02ZZZX0
1P2c/k<0_[jn0=CDe03WinL0ZZZZ00000080no_k0`3lo?`50?gmo@<0o_kn203oool000070000002:
RXX0fMWI0?clo03Ti>@0d];B0<c<c03o0;jn_Th0_[jn00P0c<c<0=;BdP3Ti>@0o?cl0=WIf@2:RXX0
00000?clo0P0oOgm0P3no_h:0?ooo`0000P0000006UYJ@2GUiL0fMWI0?clo03bl_80k>c/0>?Shol0
fMWIC03IfMT0203Shn<0k>c/0?;blP3lo?`0fMWI09NGU`1YJFT000001`3moOd30?knoP/0oooo0000
1@000000oooo06UYJ@2GUiL0dm?C0080o?clo`3ooom>0?ooo`80o?cl00D0dm?C09NGU`1YJFT0oooo
000000040?gmo@@0o_kn3@3oool000030000003oool0oooo0080JFUY00<0RXZ:09NGU`2ZZZX0o`2Z
ZZY=0:ZZZP0409NGU`2:RXX0JFUY06UYJ@80oooo00<000000?knoP3no_h0103no_h?0?ooo`00o`00
001L000001D0oooo0000\
\>"],
  ImageRangeCache->{{{126.938, 383.5}, {628.563, 406.313}} -> {-181.678, \
582.367, 1.00159, 1.00159}}],

Cell[TextData[{
  "You should also note that when you started typing ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " placed a ",
  StyleBox["bracket",
    FontSlant->"Italic"],
  " on the far right side of the window that enclosed the ",
  StyleBox["cell",
    FontSlant->"Italic"],
  " that you were working in. These ",
  StyleBox["cell brackets",
    FontSlant->"Italic"],
  " are helpful for organizational purposes within the notebook. \
Double-clicking on cell brackets will open any collapsed cells, or close any \
open cells as can be seen in the previous screen shot."
}], "Text",
  CellTags->{"i:33", "b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "Double-clicking on the cell bracket containing the ",
  StyleBox["1.1 A Brief Overview of ",
    FontWeight->"Bold"],
  StyleBox["Mathematica",
    FontWeight->"Bold",
    FontSlant->"Italic"],
  " cell will open the cell to display its contents:"
}], "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .86649 
%%ImageSize: 368 319 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
1.79978e-017 0.0027248 -0.0027248 0.0027248 [
[ 0 0 0 0 ]
[ 1 .86649 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .86649 L
0 .86649 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00272 translate
1.00272 .86921 scale
1104 string
368 319 8 [368 0 0 319 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FEFEFEFEFEFEFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFEFEFEFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFCFCFCFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFCFCFCFCFCFCFBFBFBFAFAFAF9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9FAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFC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FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFDFDFDFCFCFCFBFBFB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FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9
F8F8F8F6F6F6F5F5F5F3F3F3F1F1F1F0F0F0EFEFEFEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EFEFEFF0F0F0F1F1F1F3F3F3F5F5F5F6F6F6F8F8F8F9F9F9FBFBFBFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7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FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F7F7F7F5F5F5
F3F3F3F0F0F0EDEDEDEAEAEAE7E7E7E4E4E4E2E2E2E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E2E2E2E4E4E4E7E7E7EAEAEAEDEDEDF0F0F0F3F3F3F5F5F5F7F7F7F9F9F9FBFBFBFCFCFC
FDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F3F3F3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FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F6F6F6F4F4F4F0F0F0
ECECECE7E7E7E2E2E2DDDDDDD9D9D9D5D5D5D2D2D2D0D0D0CFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFD0D0D0
D2D2D2D5D5D5D9D9D9DDDDDDE2E2E2E7E7E7ECECECF0F0F0F4F4F4F6F6F6F8F8F8FAFAFA
FCFCFCFDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F5F5F5F1F1F1EDEDED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FFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F6F6F6F3F3F3EFEFEFEAEAEA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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFAFAFAF8F8F8F5F5F5F1F1F1EDEDEDE7E7E7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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCF9F9F9F7F7F7F4F4F4F0F0F0EBEBEBE4E4E4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FFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFBFBFBF9F9F9F6F6F6F4F4F4EFEFEFE9E9E9E2E2E2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FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F6F6F6F3F3F3EEEEEEE8E8E8E1E1E1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000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000000000000000000000000000000000000000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969696969696969696969696969
696969696969696969696969696969696969696969696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EEEEEE8D8D8D9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6
ECECECBABABA9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9FBDBDBD
ECECECE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7EEEEEE8D8D8D9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
8D8D8DE6E6E6E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696DEDEDEDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696
969696DBDBDBCACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACADBDBDB
A3A3A3FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696DEDEDEDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DEDEDE969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0E0E0E0
FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0
D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0D0FFFFFF
969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0B3B3B3
B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF
969696FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB969696FFFFFFCECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0B3B3B3
B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E0000000000000000009E9E9ED9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
DCDCDC515151B9B9B9E0E0E0D9D9D91A1A1A000000383838D9D9D9E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD1D1D1D0D0D0D1D1D1D0D0D0D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D0D0D0
D0D0D0D0D0D0D0D0D0D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF
969696FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA969696FFFFFFCECECE
CECECECECECECECECECECECE3E3E3ECECECECECECECECECECECECECECECECECECECECECE
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0E0E0E0E0E0E0
E0E0E0ADADAD545454E0E0E0ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCECECECECECECECECE
CECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECEFFFFFF969696
969696FFFFFFD6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6
D7D7D7D7D7D7D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6959595D6D6D6D6D6D6
D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D6D7D7D7D7D7D7D6D6D6D6D6D6D6D6D6FFFFFF
969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCECECE
CECECECECECECECECE3E3E3E3E3E3E3E3E3ECECECECECECECECECECECECECECECECECECE
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
FFFFFFFFFFFFE0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9E0E0E0E0E0E0
E0E0E0DCDCDC515151B9B9B9ADADAD545454E0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFCFCFCFCFCFCFCFCFCF
CFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595DCDCDCDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF
969696F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9969696FFFFFFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3
B3B3B3FFFFFFE0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2E0E0E0E0E0E0
E0E0E0E0E0E0C4C4C4484848D2D2D21A1A1A0000000D0D0DD2D2D2E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000000000
000000000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFD4D4D4D4D4D4D4D4D4
3E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4FFFFFF969696
969696FFFFFFDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDFFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595DCDCDCDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDCDCDCDCDDDDDDFFFFFF
969696F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8969696FFFFFFD4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4D4D4D4
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3
B3B3B3E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0D9D9D9262626CBCBCB262626D2D2D2E0E0E0D9D9D9262626D2D2D2DCDCDC494949
0000000D0D0DD2D2D26969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000000000
000000E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFDBDBDBDBDBDB3E3E3E
3E3E3E3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696
969696FFFFFFE4E4E4E3E3E3E3E3E3E4E4E4E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E4E4E4E4E4E4FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595E3E3E3E4E4E4
E4E4E4E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E4E4E4E4E4E4E4E4E4E3E3E3E4E4E4FFFFFF
969696F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7969696FFFFFFDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3EDBDBDBDBDBDBDBDBDB
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C4C4C4484848D9D9D9D9D9D9262626D2D2D2
E0E0E0C4C4C4484848D2D2D2262626D2D2D2E0E0E0C4C4C4484848D9D9D9ADADAD545454
E0E0E06969697E7E7EC4C4C4484848D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000000000000000
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFE4E4E4E4E4E4E4E4E4
3E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595E9E9E9EAEAEA
EAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF
969696F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5969696FFFFFFE4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3EE4E4E4E4E4E4E4E4E4E4E4E4
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0FFFFFFFFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E08E8E8E000000383838D9D9D9E0E0E06969699E9E9E
E0E0E08E8E8E545454E0E0E06969699E9E9EE0E0E08E8E8E545454E0E0E0ADADAD545454
E0E0E06969697E7E7EE0E0E06969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0000000E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFEDEDEDEDEDEDEDEDED
EDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696
969696FFFFFFEAEAEAE9E9E9E9E9E9EAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9FFFFFF959595FFFFFF959595FFFFFF959595FFFFFF959595E9E9E9EAEAEA
EAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAE9E9E9E9E9E9FFFFFF
969696F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4969696FFFFFFEDEDED
EDEDEDEDEDEDEDEDEDEDEDED3E3E3E3E3E3E3E3E3EEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3FFFFFFE0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D9D9D9525252D9D9D9E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A0000000D0D0DD2D2D2E0E0E0D9D9D91A1A1A
0000000D0D0DD2D2D2E0E0E0C4C4C41D1D1DD2D2D2E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFF4F4F4F4F4F4F4F4F4
F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDFFFFFFEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF
969696F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1969696FFFFFFF4F4F4
F4F4F4F4F4F4F4F4F43E3E3E3E3E3E3E3E3EF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696
969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF
969696F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0969696FFFFFFFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC3E3E3EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696
969696FFFFFFF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFFF
969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0969696D7D7D7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696
969696FAFAFAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFAFAFA
969696EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF969696D7D7D7FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
D7D7D7969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0FFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0DADADA929292B6B6B6B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA
D6D6D6B0B0B0B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8
B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8A9A9A9
CECECEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEDADADA929292B6B6B6
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B4
8F8F8FDADADAE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB6969699E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EFFFFFF
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBE7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7ECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECECEEEEEE8D8D8D9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F8D8D8DE6E6E6696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECE3E3E3ECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECE3E3E3E3E3E3E3E3E3E
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCFCFCFCFCFCFCFCFCF3E3E3E3E3E3E3E3E3E3E3E3E
3E3E3ECFCFCFCFCFCFCFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E
3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFBFBFB838383EFEFEF1D1D1D000000606060F7F7F77D7D7DF7F7F7A4A4A4FFFFFF
878787EFEFEFADADADEBEBEB838383F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000090909D2D2D2FFFFFFF7F7F7
1D1D1D0000000F0F0FEFEFEFF7F7F71D1D1D0000000F0F0FEFEFEFF7F7F71D1D1D000000
0000008F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFF7F7F75E5E5EF7F7F7
F7F7F71D1D1D0000000F0F0FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020FFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020202020202020202020202020202020FFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020202020202020202020202020FFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020FFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020202020202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB545454B4B4B4FFFFFFFFFFFFFFFFFF
A2A2A2414141F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFF7F7F77D7D7DDFDFDF9A9A9AFBFBFBA0A0A0DFDFDF9A9A9AFFFFFFFFFFFFC6C6C6
8F8F8FFBFBFBA0A0A0FFFFFFFFFFFFC6C6C6000000000000414141F7F7F7FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2606060FFFFFFFFFFFF787878
000000000000000000606060C6C6C68F8F8FFFFFFFA2A2A2B4B4B4FFFFFFA2A2A2B4B4B4
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D0000000000008F8F8FFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878B4B4B4FFFFFFFFFFFFFFFFFFA2A2A28F8F8F
C6C6C6606060FFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFF7F7F71D1D1D0000000F0F0FEFEFEFF7F7F71D1D1D000000090909D2D2D2
FFFFFF787878000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020FFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020FFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020202020202020202020202020202020FFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFA7A7A7686868FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020202020202020202020202020FFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFA8A8A8D2D2D2FFFFFFFFFFFF878787CFCFCF9A9A9AFBFBFBA0A0A0DFDFDF
9A9A9ADFDFDF9A9A9AFFFFFFFFFFFFA5A5A5F7F7F7FBFBFBA0A0A0C6C6C6000000000000
414141F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFDFDFDF121212000000000000B4B4B4F7F7F77D7D7DFFFFFF878787EFEFEFFFFFFF
FFFFFF636363EFEFEFFFFFFFF7F7F77D7D7DFFFFFF878787EFEFEFFFFFFF545454000000
0000008F8F8FFFFFFF5454540000000000008F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFA2A2A2000000000000B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFBFBFB
838383F7F7F7787878606060FFFFFFA2A2A2B4B4B4F7F7F71D1D1DB4B4B4FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFDFDFDF535353FFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFC6C6C65757575E5E5EFFFFFFDFDFDF9A9A9AFFFFFF818181D2D2D2DFDFDF
727272636363EFEFEFFFFFFFDFDFDF9A9A9AFFFFFF818181D2D2D2DFDFDF727272FFFFFF
818181D2D2D2DFDFDF727272FFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020FFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020FFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020202020202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020202020202020202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
A2A2A28F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F7
5E5E5EF7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2
0000000000000000008F8F8F787878000000000000000000B4B4B4FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFBFBFB636363EFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C6
8F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F7F7F72C2C2CEFEFEFFFFFFFFBFBFB5454548F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2
B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFDFDFDF727272FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF818181
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFF818181D2D2D2FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFF7F7F75E5E5EF7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB636363
EFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F7F7F72C2C2CEFEFEFFFFFFFC6C6C6606060FBFBFB545454B4B4B4FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F7
1D1D1D0F0F0FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F71D1D1D0F0F0FEFEFEFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB5454540000000F0F0F
EFEFEFFFFFFFFFFFFFF7F7F71D1D1D0F0F0FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFF7F7F71D1D1D0F0F0FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF8F8F82B2B2B
E3E3E3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C6
0000000F0F0FEFEFEFFFFFFFFBFBFB545454000000414141F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFF8F8F82B2B2BE3E3E3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFBFBFB838383EFEFEF1D1D1D000000606060F7F7F77D7D7DFBFBFB545454090909
D2D2D2FFFFFFADADADEBEBEB838383F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF121212606060FFFFFFFFFFFFC6C6C6
0000000F0F0FEFEFEFF7F7F71D1D1D0000000000000F0F0FEFEFEFFFFFFFFFFFFFF7F7F7
2C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF787878000000606060
FFFFFFDFDFDF4E4E4E4A4A4A0F0F0FEFEFEFFFFFFFFFFFFFFBFBFB636363EFEFEFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF818181
D2D2D2FFFFFFFFFFFFF7F7F71D1D1D0000000F0F0FEFEFEFF7F7F71D1D1D000000000000
090909D2D2D2FFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFC6C6C68F8F8F818181
D2D2D2FFFFFFFBFBFB636363EFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB636363
EFEFEFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFBFBFB636363EFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBFBFB636363
EFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFF7F7F72C2C2CEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF818181
D2D2D2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF1B1B1BD2D2D2
FFFFFFFFFFFF787878000000606060FFFFFFFBFBFB545454000000414141F7F7F7FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFF7F7F75E5E5EF7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA2A2A2
B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFC6C6C6414141F3F3F3545454B4B4B4C6C6C6606060FFFFFF787878B4B4B4FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFDFDFDF727272FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C6
8F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFC6C6C6606060FFFFFFA2A2A2B4B4B4C6C6C6414141F7F7F7787878B4B4B4FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFA2A2A2B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF7F7F7
5E5E5EF7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFC6C6C6606060FFFFFFA2A2A2B4B4B4FFFFFF787878000000606060FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFBFBFB636363EFEFEFFFFFFFFFFFFFA2A2A20000000F0F0FEFEFEF
7878780000000F0F0FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF202020202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
A2A2A28F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFC6C6C6414141F3F3F3545454B4B4B4C6C6C6606060FFFFFFA2A2A2B4B4B4FFFFFF
FFFFFFFFFFFFFFFFFFC6C6C68F8F8FFFFFFFFFFFFFFFFFFFFFFFFFA8A8A8D2D2D2FFFFFF
FFFFFF787878B4B4B4FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020FFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF787878000000606060FFFFFFFBFBFB5454540000000F0F0FEFEFEFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA8A8A8D2D2D2FFFFFF
FFFFFFFFFFFF878787EFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF202020FFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC6C6C6090909D2D2D2FFFFFF
C6C6C6000000606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF202020FFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020202020202020202020202020202020FFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBDBDBD8C8C8CFFFFFFFFFFFF
FFFFFF000000FFFFFFFFFFFFFFFFFFD9D9D9000000E1E1E1FFFFFF000000A7A7A7BDBDBD
000000D9D9D9E1E1E1000000D9D9D9FFFFFFFFFFFFFFFFFFD9D9D9000000E1E1E1FFFFFF
000000A7A7A7000000A7A7A7FFFFFFFFFFFFFFFFFFFFFFFFF0F0F0000000FFFFFFFFFFFF
D9D9D9A7A7A7D0D0D0A7A7A7D9D9D9FFFFFF7C7C7C7C7C7CBDBDBD000000BDBDBDFFFFFF
FFFFFFFFFFFFD0D0D0000000B2B2B2BDBDBD0000008C8C8CD0D0D0000000D9D9D9000000
D9D9D9E1E1E1000000D9D9D9FFFFFFFFFFFFFFFFFFD0D0D06868688C8C8CA7A7A7FFFFFF
D9D9D9000000D9D9D9E1E1E1000000D9D9D9FFFFFF000000A7A7A7BDBDBD6868688C8C8C
A7A7A7FFFFFFD9D9D9000000D9D9D9FFFFFFFFFFFFFFFFFFD9D9D9000000E1E1E1D9D9D9
000000E1E1E1D9D9D90000000000008C8C8CD0D0D0000000D9D9D9000000A7A7A7BDBDBD
000000D9D9D9E1E1E1000000D9D9D9D0D0D06868688C8C8CA7A7A7FFFFFFD9D9D9000000
E1E1E1D9D9D9000000E1E1E1D9D9D90000000000008C8C8CD0D0D0000000D9D9D9000000
A7A7A7BDBDBD000000E1E1E1D0D0D0000000B2B2B2BDBDBD0000008C8C8CD0D0D0000000
D9D9D9000000E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD0D0D06868688C8C8CA7A7A7
FFFFFFA7A7A7B2B2B2B2B2B2000000E1E1E1000000BDBDBDBDBDBDD0D0D0D9D9D9000000
D9D9D9FFFFFFD0D0D06868688C8C8CA7A7A7FFFFFF7C7C7CBDBDBDA7A7A7FFFFFF7C7C7C
BDBDBDA7A7A7FFFFFFD9D9D9000000E1E1E1D9D9D9A7A7A7D0D0D0A7A7A7D9D9D9D9D9D9
000000D9D9D9E1E1E1000000D9D9D97C7C7CBDBDBDA7A7A7FFFFFFFFFFFFFFFFFF000000
8C8C8CD0D0D0000000D9D9D9000000D9D9D9E1E1E1000000D9D9D9D0D0D0A7A7A7BDBDBD
D0D0D0000000E1E1E1FFFFFFFFFFFFD0D0D0000000B2B2B2BDBDBDD9D9D9A7A7A7D0D0D0
A7A7A7D9D9D9D9D9D9000000E1E1E1D9D9D9000000E1E1E1D9D9D9000000E1E1E1000000
BDBDBDBDBDBDD0D0D0FFFFFF7C7C7C7C7C7CBDBDBD000000BDBDBD000000A7A7A7BDBDBD
6868688C8C8CA7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000BDBDBDFFFFFFFFFFFF8C8C8C
7C7C7CFFFFFFFFFFFFA7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFF0F0F0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
B2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
B2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFB2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFA7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9
000000D9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFA7A7A7686868
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9000000D9D9D9FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD9D9D9
E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
B2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDED959595959595959595959595
959595959595D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFBDBDBD686868F0F0F0D0D0D0D0D0D0FFFFFF
D0D0D0000000D0D0D0000000D9D9D9000000E1E1E1FFFFFF000000D0D0D0FFFFFFD9D9D9
A7A7A7686868D9D9D9FFFFFF000000A7A7A7D0D0D0FFFFFFA7A7A7B2B2B2E1E1E1686868
FFFFFF000000D9D9D9000000D9D9D9000000E1E1E1FFFFFF000000D0D0D0FFFFFF7C7C7C
8C8C8CFFFFFF000000A7A7A7F0F0F0000000D9D9D9000000E1E1E1FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFD9D9D9000000D9D9D9E1E1E1000000D9D9D90000008C8C8CD0D0D0000000
7C7C7CBDBDBDA7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000D0D0D0BDBDBDD9D9D9
D0D0D06868688C8C8CA7A7A7FFFFFFD0D0D06868688C8C8CA7A7A7FFFFFFD9D9D9000000
D9D9D9E1E1E1000000D9D9D9FFFFFFFFFFFFFFFFFF0000008C8C8CD0D0D0000000D9D9D9
000000BDBDBDBDBDBDD0D0D0FFFFFF000000A7A7A7BDBDBD000000E1E1E1FFFFFFF0F0F0
000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD0D0D0A7A7A7BDBDBDD0D0D0000000
BDBDBD6868688C8C8CA7A7A7FFFFFF7C7C7CBDBDBDA7A7A7FFFFFFD0D0D0000000B2B2B2
BDBDBDD9D9D9000000D9D9D9FFFFFFD9D9D9000000E1E1E1FFFFFF000000D0D0D0BDBDBD
D9D9D9D0D0D06868688C8C8CA7A7A7FFFFFFD0D0D0A7A7A7BDBDBDD0D0D0000000E1E1E1
FFFFFFFFFFFFFFFFFF0000008C8C8CD0D0D00000007C7C7CBDBDBDA7A7A7FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF0000008C8C8CD0D0D0000000D9D9D9FFFFFFFFFFFFFFFFFF7C7C7C
BDBDBDA7A7A7FFFFFFD9D9D9A7A7A7D0D0D0A7A7A7D9D9D9FFFFFF000000BDBDBDBDBDBD
D0D0D0D9D9D9000000D9D9D9E1E1E1000000D9D9D9D9D9D9000000E1E1E17C7C7CBDBDBD
A7A7A7FFFFFFFFFFFF000000A7A7A7BDBDBD000000E1E1E1D0D0D0000000B2B2B2BDBDBD
0000008C8C8CD0D0D0000000D9D9D9000000A7A7A7BDBDBD6868688C8C8CA7A7A7FFFFFF
D0D0D0A7A7A7BDBDBDD0D0D0000000E1E1E1FFFFFFFFFFFFFFFFFFFFFFFFD0D0D0000000
B2B2B2BDBDBD0000008C8C8CD0D0D0000000D9D9D9000000E1E1E1D0D0D0000000B2B2B2
BDBDBDFFFFFF7C7C7C7C7C7CBDBDBD000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DCDCDC
DCDCDCD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E4E4E4DDDDDD
DDDDDDD7D7D7D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D1D1D1CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E97C7C7C4D4D4DD0D0D0
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC7C7C7000000FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDE9E9E9E9E9E9E3E3E3DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E9000000E9E9E9
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF8C8C8C7C7C7CD9D9D9FFFFFFFFFFFFFFFFFF
8C8C8CE9E9E9FFFFFFBDBDBD4D4D4DD0D0D09A9A9A686868E9E9E9BDBDBD000000F0F0F0
686868B2B2B2FFFFFF4D4D4D9A9A9AE9E9E9FFFFFFA7A7A7000000A7A7A7D0D0D0FFFFFF
686868B2B2B2FFFFFFFFFFFFFFFFFF8C8C8C686868E1E1E18C8C8C4D4D4DBDBDBDFFFFFF
0000007C7C7CD9D9D9000000C7C7C7FFFFFF000000A7A7A7F0F0F0FFFFFFFFFFFFFFFFFF
FFFFFFB2B2B2A7A7A7B2B2B27C7C7CA7A7A7FFFFFFC7C7C7A7A7A77C7C7CA7A7A7FFFFFF
BDBDBD000000F0F0F0686868B2B2B2FFFFFF4D4D4D9A9A9AE9E9E9FFFFFF4D4D4D8C8C8C
8C8C8CA7A7A7FFFFFFFFFFFFBDBDBD4D4D4DD0D0D09A9A9A686868E9E9E94D4D4D8C8C8C
D0D0D0FFFFFF0000007C7C7CD9D9D9000000C7C7C7FFFFFF4D4D4D8C8C8CD0D0D0FFFFFF
8C8C8C686868E1E1E1C7C7C7A7A7A77C7C7CA7A7A7FFFFFF6868689A9A9AFFFFFF8C8C8C
686868D9D9D9F0F0F04D4D4DB2B2B2B2B2B2FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF7F7F7EDEDEDEDEDEDEAEAEAEAEAEAE4E4E4DDDDDD
DDDDDDD6D6D6D0D0D0CFCFCFD0D0D0FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
202020FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020
202020FFFFFFFFFFFF202020202020202020202020202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020202020202020202020202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E9B2B2B2FFFFFF4D4D4D000000
D0D0D0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000
9A9A9A9A9A9A686868D0D0D0FFFFFFFFFFFFC7C7C79A9A9A8C8C8CBDBDBD000000A7A7A7
A7A7A70000007C7C7CFFFFFFFFFFFFB2B2B29A9A9AB2B2B2A7A7A79A9A9AF0F0F09A9A9A
0000000000007C7C7CC7C7C7FFFFFFFFFFFFFFFFFFFFFFFF000000000000B2B2B2FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB2B2B2000000000000A7A7A7000000000000C7C7C7
E1E1E17C7C7C8C8C8CE9E9E9B2B2B29A9A9AB2B2B2A7A7A79A9A9AF0F0F0C7C7C79A9A9A
8C8C8CBDBDBD000000A7A7A7A7A7A7000000000000C7C7C7E1E1E17C7C7C8C8C8C8C8C8C
0000007C7C7CFFFFFFFFFFFFB2B2B29A9A9AB2B2B2A7A7A79A9A9AF0F0F08C8C8C000000
4D4D4DD9D9D9BDBDBD0000004D4D4DD9D9D9C7C7C7000000C7C7C7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFC7C7C78C8C8CA7A7A7A7A7A78C8C8CC7C7C7F0F0F09A9A9A000000000000
7C7C7CC7C7C7FFFFFFFFFFFFFFFFFFFFFFFFC7C7C7E9E9E9B2B2B20000009A9A9AFFFFFF
E1E1E1BDBDBDFFFFFF000000000000F0F0F0FFFFFFFFFFFFFFFFFFD9D9D9E1E1E1E1E1E1
6868689A9A9A8C8C8C4D4D4D0000007C7C7CA7A7A7FFFFFFA7A7A7000000E1E1E1B2B2B2
0000008C8C8CFFFFFFFFFFFFF0F0F0B2B2B2C7C7C79A9A9A717171B2B2B20000004D4D4D
F0F0F0A7A7A7000000B2B2B2FFFFFFBDBDBD4D4D4DB2B2B2FFFFFFFFFFFFFFFFFFD9D9D9
E1E1E1E1E1E16868689A9A9A8C8C8C4D4D4D0000007C7C7CA7A7A7FFFFFFF0F0F0B2B2B2
4D4D4DE1E1E1F0F0F0A7A7A78C8C8C4D4D4DE9E9E9FFFFFFD9D9D9E1E1E1E1E1E1686868
9A9A9AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFE7E7E7EEEEEEEFEFEFEFEFEFF0F0F0F1F1F1F4F4F4F5F5F5F7F7F7F8F8F8
F9F9F9F9F9F9FAFAFAFBFBFBFBFBFBFBFBFBE7E7E7696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0C7C7C74D4D4DFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF0F0F0C7C7C74D4D4DFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFC7C7C79A9A9AFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E99A9A9A000000000000
9A9A9A9A9A9A0000009A9A9AE9E9E9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF8C8C8C686868FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E9E9E9686868FFFFFFBDBDBDBDBDBDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E9A7A7A7
A7A7A7B2B2B2B2B2B27C7C7C9A9A9AD9D9D9FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
8C8C8C686868FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E9686868
FFFFFFBDBDBDBDBDBDFFFFFFFFFFFFFFFFFFBDBDBD686868000000000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFB2B2B2000000000000A7A7A7E9E9E9FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE9E9E97C7C7C000000BDBDBD
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
686868686868FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696DEDEDEDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDEDEDE969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFE1E1E1A7A7A7000000A7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE1E1E1
A7A7A7000000A7A7A7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF202020202020202020202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFF000000000000000000
000000000000DFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF202020202020202020202020202020FFFFFF000000DFDFDFDFDFDF
DFDFDFDFDFDFDFDFDF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCECECECECECE3E3E3ECECECECECECECECECECECECE
CECECE3E3E3ECECECECECECECECECEFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000000000
000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFCFCFCF3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF
3E3E3E3E3E3E3E3E3ECFCFCFCFCFCFFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000000000
000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFD4D4D4D4D4D43E3E3E3E3E3E3E3E3ED4D4D43E3E3E
3E3E3E3E3E3ED4D4D4D4D4D4D4D4D4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFDBDBDBDBDBDBDBDBDB3E3E3E3E3E3E3E3E3E3E3E3E
3E3E3EDBDBDBDBDBDBDBDBDBDBDBDBFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFE4E4E4E4E4E4E4E4E4E4E4E43E3E3E3E3E3E3E3E3E
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFEDEDEDEDEDEDEDEDEDEDEDEDEDEDED3E3E3EEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECEC969696D7D7D7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7969696696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000FCFCFCABABAB696969FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF202020FFFFFFFFFFFF
FFFFFFFFFFFFECECECDADADA929292B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FDADADA696969ABABABFCFCFC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F79A9A9A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A
7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A9A9A9AF7F7F70000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFF7F7F70000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3F7F7F70000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000F1F1F1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE3E3E3A5A5A5
767676767676767676767676777777767676777777777777767676767676767676767676
767676767676767676767676767676A5A5A5E3E3E3FFFFFFFFFFFFE3E3E3A5A5A5767676
767676767676767676777777767676777777777777767676767676767676767676767676
767676767676767676767676A5A5A5E3E3E3FFFFFFFFFFFFEBEBEBA5A5A5565656565656
565656565656575757565656575757575757565656565656565656565656565656565656
565656565656565656A5A5A5EBEBEBFFFFFFFFFFFFFFFFFFFFFFFFF1F1F10000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000E1E1E1FCFCFCFBFBFBFCFCFCFCFCFCFBFBFBFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC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000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F77676768D8D8D
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F77676768D8D8DA2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A28D8D8D767676F7F7F7F7F7F75656565B5B5B6F6F6F717171
727272727272727272727272727272727272727272727272727272727272727272727272
7272727171716F6F6F5B5B5B565656F7F7F7F7F7F7F7F7F7F7F7F7E0E0E00000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DEDEDEF1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1
F1F1F1F1F1F1F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F2F2F2F2F2F2F1F1F1F2F2F2
F1F1F1F2F2F2F2F2F2F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1767676B8B8B8
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABB8B8B8767676F1F1F1F1F1F1767676B8B8B8ABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABB8B8B8767676F1F1F1F2F2F25656566F6F6F6F6F6F727272
777777777777777777777777777777777777777777777777777777777777777777777777
7777777272726F6F6F6F6F6F565656F1F1F1F1F1F1F1F1F1F2F2F2DEDEDE0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DCDCDCEBEBEBEBEBEBECECECEBEBEBEBEBEBA7A7A7A2A2A2A2A2A2A1A1A1A1A1A1
A0A0A09F9F9F9D9D9D9F9F9F9F9F9FA0A0A0A0A0A0A1A1A1A1A1A1A7A7A7EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBCECECEC5C5C5C5C5C5D0D0D0EBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBE5E5E5C7C7C7D7D7D7EBEBEBEBEBEBEBEBEBCECECECCCCCCEAEAEAEAEAEA
C8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3EBEBEBE0E0E0C5C5C5C5C5C5CCCCCC
E9E9E9C8C8C8D0D0D0EBEBEBEBEBEBEBEBEBEBEBEBE0E0E0C5C5C5C5C5C5C5C5C5DDDDDD
EBEBEBEBEBEBEAEAEAC8C8C8C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBE0E0E0C5C5C5
C5C5C5C5C5C5C5C5C5C6C6C6E3E3E3EBEBEBEAEAEAC8C8C8C5C5C5C5C5C5D7D7D7EBEBEB
EBEBEBE0E0E0C5C5C5C5C5C5CCCCCCE9E9E9CECECEC7C7C7E9E9E9EBEBEBE0E0E0C5C5C5
C5C5C5C5C5C5DDDDDDEBEBEBEAEAEAC8C8C8D0D0D0EBEBEBEBEBEBDADADAC6C6C6E3E3E3
EBEBEBD4D4D4C5C5C5DEDEDEECECECEBEBEBC8C8C8D0D0D0ECECECECECECDBDBDBC6C6C6
E3E3E3E6E6E6C7C7C7C5C5C5C5C5C5C5C5C5CCCCCCEBEBEBECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB767676BBBBBB
B1B1B1B1B1B1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFB1B1B1B1B1B1
B1B1B1B1B1B1B1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB767676BBBBBBB1B1B1
B1B1B1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFB1B1B1B1B1B1B1B1B1BBBBBB767676EBEBEBEBEBEB565656777777727272747474
777777787878787878787878787878787878787878787878787878787878787878787878
777777747474727272777777565656EBEBEBEBEBEBEBEBEBEBEBEBDCDCDC0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000DADADAE5E5E5E5E5E5E5E5E5E5E5E58A8A8A9B9B9BACACACE9E9E9E9E9E9E9E9E9
EAEAEAC4C4C4858585ECECECEDEDEDEEEEEEE7E7E7F0F0F0F2F2F2A3A3A3E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E1E1E14B4B4B1010101010105B5B5BC7C7C7CFCFCFE5E5E5E5E5E5
E5E5E5C9C9C91C1C1C727272D4D4D4E5E5E5E2E2E2505050393939C5C5C5DDDDDD262626
515151CFCFCFE5E5E5949494151515A4A4A4DEDEDEB2B2B2101010101010414141D7D7D7
262626515151CFCFCFE5E5E5E5E5E5E5E5E5AFAFAF101010101010101010999999C6C6C6
DEDEDEDDDDDD242424101010101010101010151515A4A4A4DEDEDEB2B2B2101010101010
101010101010151515A4A4A4DEDEDEDDDDDD242424101010101010838383CDCDCDD9D9D9
B2B2B2101010101010414141DBDBDB505050191919BABABAE3E3E3AFAFAF101010101010
101010999999C6C6C6D7D7D7262626515151CFCFCFE5E5E5949494151515A4A4A4DEDEDE
7070701010108E8E8ED9D9D9DDDDDD262626515151CFCFCFE5E5E5949494151515A4A4A4
C2C2C21C1C1C101010101010101010404040C6C6C6C6C6C6E2E2E2E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5767676C0C0C0
B5B5B5B5B5B5FFFFFF727272727272727272727272727272727272272727B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5C0C0C0767676E5E5E5E5E5E5767676C0C0C0B5B5B5
B5B5B5FFFFFF272727272727272727272727272727272727272727272727272727272727
272727B5B5B5B5B5B5B5B5B5C0C0C0767676E5E5E5E5E5E5565656777777777777797979
7A7A7AFFFFFFFFFFFF7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7AFFFFFFFFFFFF
7A7A7A797979777777777777565656E5E5E5E5E5E5E5E5E5E5E5E5DADADA0000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D9D9D9DFDFDFDFDFDFDFDFDFDFDFDFDEDEDEA0A0A0EAEAEAE9E9E9E5E5E5EAEAEA
EAEAEAD0D0D0747474EDEDEDEEEEEEADADADD0D0D0F2F2F2F3F3F3A3A3A3DFDFDFDEDEDE
DEDEDEDFDFDFDEDEDE6B6B6B191919C5C5C5D7D7D7252525555555C6C6C6DDDDDDDEDEDE
DEDEDEC3C3C31C1C1C727272D1D1D1DEDEDEDBDBDB4E4E4E393939C4C4C4D6D6D6252525
515151CDCDCDDEDEDE8F8F8F151515A4A4A4D5D5D54E4E4E383838C4C4C4DDDDDDD7D7D7
252525515151CDCDCDDEDEDEDEDEDEC3C3C31B1B1B727272D9D9D98F8F8F151515A5A5A5
C6C6C6252525191919C5C5C5DEDEDE8F8F8F151515A4A4A4D2D2D2252525383838C7C7C7
DBDBDB4E4E4E151515A4A4A4D2D2D2252525383838CFCFCFDBDBDB4E4E4E9F9F9FDBDBDB
4E4E4E383838C4C4C4DDDDDDDBDBDB4E4E4E191919BABABAC2C2C21B1B1B727272D9D9D9
8F8F8F151515A5A5A5C6C6C6252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4D9D9D9
6C6C6C1010109F9F9FDEDEDED7D7D7252525515151CDCDCDDEDEDE8F8F8F151515A4A4A4
BEBEBE1C1C1C727272D1D1D1DBDBDB4E4E4E191919B9B9B9D5D5D5DEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE767676C3C3C3
BBBBBBBBBBBBFFFFFF727272727272727272727272727272727272272727BBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBC3C3C3767676DEDEDEDEDEDE767676C3C3C3BBBBBB
BBBBBBFFFFFF272727BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBFFFFFF
272727BBBBBBBBBBBBBBBBBBC3C3C3767676DFDFDFDEDEDE5656567777777878787B7B7B
303030FFFFFFFFFFFFFFFFFF7D7D7D7D7D7D7D7D7D7D7D7D7D7D7DFFFFFFFFFFFFFFFFFF
3030307B7B7B787878777777565656DEDEDEDEDEDEDFDFDFDEDEDED9D9D90000009A9A9A
9E9E9EA6A6A6B1B1B1BBBBBBC5C5C5CFCFCFD8D8D8E1E1E1E8E8E8EEEEEEF3F3F3F6F6F6
F9F9F9FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D7D7D7D8D8D8D9D9D9D8D8D8D8D8D89090908F8F8F8E8E8EE9E9E9818181515151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000000D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3858585A3A3A3E6E6E6EAEAEAEAEAEA353535
7F7F7FCFCFCFBEBEBE8B8B8B5E5E5E585858686868A3A3A3C9C9C9AAAAAAD3D3D3D3D3D3
D3D3D3D3D3D3A4A4A41010108E8E8ECECECED3D3D3A4A4A41010108E8E8ECCCCCCD3D3D3
D3D3D3B8B8B81B1B1B727272CCCCCCD3D3D3CFCFCF4A4A4A393939C1C1C1CCCCCC242424
515151CACACAD3D3D3888888151515A4A4A4C9C9C9242424383838C1C1C1D3D3D3CCCCCC
242424515151CACACAD3D3D3D3D3D3676767191919B9B9B9D0D0D0CCCCCC242424515151
9D9D9D1010108E8E8ECCCCCCD3D3D3888888151515A4A4A4C9C9C9242424515151CACACA
D3D3D3888888151515A4A4A4868686151515A4A4A4CECECED3D3D3D3D3D3D3D3D3CCCCCC
242424383838C1C1C1D3D3D3CFCFCF4A4A4A191919B9B9B9676767191919B9B9B9D0D0D0
CCCCCC242424515151C3C3C3242424515151CACACAD3D3D3888888151515A4A4A4D0D0D0
D3D3D3D3D3D3D3D3D3D3D3D3CCCCCC242424515151CACACAD3D3D3888888151515A4A4A4
B6B6B61B1B1B727272CCCCCCD3D3D3B8B8B81B1B1B717171CDCDCDD3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3767676CBCBCB
C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6
C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6CBCBCB767676D3D3D3D3D3D3767676CBCBCBC6C6C6
C6C6C6FFFFFF272727C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6C6FFFFFF
272727C6C6C6C6C6C6C6C6C6CBCBCB767676D3D3D3D3D3D35656567D7D7D7F7F7F838383
838383838383282828FFFFFFFFFFFFFFFFFF838383FFFFFFFFFFFFFFFFFF282828838383
8383838383837F7F7F7D7D7D565656D3D3D3D3D3D3D3D3D3D3D3D3D5D5D5000000A0A0A0
A5A5A5ADADADB6B6B6BFBFBFC8C8C8D2D2D2DADADAE2E2E2E9E9E9EFEFEFF4F4F4F6F6F6
F9F9F9FBFBFBFDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D3D3D3CECECECECECECFCFCFCECECECECECE878787939393D3D3D3DBDBDBADADAD
C1C1C1B0B0B0A9A9A9AAAAAA8D8D8D919191B0B0B0CBCBCBEFEFEFA5A5A5CECECECECECE
CECECECFCFCF8585851010108E8E8ECBCBCBCECECEA0A0A0101010717171CACACACECECE
CECECEB4B4B41A1A1A727272CACACACECECECBCBCB494949393939C1C1C1C7C7C7232323
515151C7C7C7CECECE858585151515A4A4A4C7C7C7232323383838C1C1C1CECECEC7C7C7
232323515151C5C5C5CDCDCDCDCDCD848484151515A4A4A4CACACAC7C7C7232323515151
B1B1B11A1A1A727272C5C5C5CECECE858585151515A4A4A4C6C6C6232323515151C8C8C8
CECECE858585151515A4A4A49F9F9F1010108E8E8EC7C7C7CECECECECECEC8C8C8C6C6C6
232323383838C1C1C1CECECECBCBCB494949191919B9B9B9848484151515A4A4A4CACACA
C7C7C7232323515151C4C4C4232323515151C7C7C7CECECE858585151515A4A4A4CECECE
CECECECECECECECECECFCFCFC8C8C8232323515151C7C7C7CECECE858585151515A4A4A4
B4B4B41A1A1A727272CACACACFCFCFA1A1A11010108E8E8ECECECECFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE767676CECECE
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCECECE767676CECECECECECE767676CECECECBCBCB
CBCBCBFFFFFF272727CBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBFFFFFF
272727CBCBCBCBCBCBCBCBCBCECECE767676CFCFCFCECECE5656567D7D7D828282858585
848484858585858585282828FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF282828858585858585
8484848585858282827D7D7D565656CECECECECECECECECECECECED3D3D3000000A8A8A8
ADADADB3B3B3BBBBBBC3C3C3CCCCCCD5D5D5DDDDDDE4E4E4EBEBEBF0F0F0F4F4F4F7F7F7
F9F9F9FCFCFCFDFDFDFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
000000D0D0D0C9C9C9C9C9C9C9C9C9C9C9C97373739494947E7E7E797979A0A0A0D1D1D1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000000CFCFCFC5C5C5C5C5C5C5C5C5C5C5C5C5C5C5979797B7B7B7ECECECB9B9B98B8B8B
B7B7B7AAAAAAA4A4A47E7E7E717171C4C4C4D9D9D9F6F6F6F9F9F9A5A5A5C5C5C5C5C5C5
C5C5C5C5C5C59999991010108D8D8DC6C6C6C5C5C59999991010108E8E8EC6C6C6ACACAC
5C5C5CADADAD1A1A1A717171C6C6C6C5C5C5C2C2C2454545383838BFBFBFBFBFBF222222
5D5D5D101010101010151515A4A4A4C2C2C2454545101010101010101010515151BFBFBF
2222224E4E4E454545101010515151C5C5C5ACACAC1A1A1A1010101010108D8D8DC5C5C5
C5C5C5C2C2C2454545101010101010101010151515A4A4A4BFBFBF222222515151C5C5C5
C5C5C57F7F7F151515A4A4A4C5C5C5C2C2C2454545101010101010393939BCBCBC454545
101010101010101010515151616161101010191919B9B9B9C5C5C5ACACAC1A1A1A101010
1010108D8D8DC5C5C5BFBFBF2222225D5D5D101010101010151515A4A4A4C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5BFBFBF2222225D5D5D101010101010151515A4A4A4C5C5C5
ACACAC1A1A1A101010101010101010191919B9B9B9C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5767676D5D5D5
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1
D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D5D5D5767676C5C5C5C5C5C5767676D5D5D5D1D1D1
D1D1D1FFFFFF272727D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1D1FFFFFF
272727D1D1D1D1D1D1D1D1D1D5D5D5767676C5C5C5C5C5C5565656828282878787898989
898989898989898989898989FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF898989898989898989
898989898989878787828282565656C5C5C5C5C5C5C5C5C5C5C5C5CFCFCF000000BCBCBC
BFBFBFC3C3C3C9C9C9D0D0D0D7D7D7DDDDDDE4E4E4EAEAEAEFEFEFF3F3F3F6F6F6F8F8F8
FBFBFBFCFCFCFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
000000CDCDCDC1C1C1C1C1C1C1C1C1C1C1C1757575999999A7A7A7EDEDEDC9C9C9737373
B0B0B08D8D8D7373734C4C4C626262BEBEBEB5B5B5B2B2B2ADADADA9A9A9C1C1C1C1C1C1
C1C1C1C1C1C1BBBBBB222222717171C5C5C5C2C2C27C7C7C151515A3A3A3C2C2C2A9A9A9
1A1A1A101010101010717171C4C4C4C1C1C1BEBEBE444444383838BEBEBEC2C2C2C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1BBBBBB222222383838BEBEBEC2C2C2C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C17C7C7C151515A4A4A4C3C3C3C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1BBBBBB
222222383838BEBEBEC2C2C2C1C1C1C1C1C1C3C3C3C5C5C5C2C2C2C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
A9A9A91A1A1A717171C4C4C4C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1767676D8D8D8
D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D8D8D8767676C1C1C1C1C1C1767676D8D8D8D5D5D5
D5D5D5FFFFFF272727FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
272727D5D5D5D5D5D5D5D5D5D8D8D8767676C1C1C1C1C1C15656568484848888888A8A8A
8A8A8A8A8A8A8A8A8AFFFFFFFFFFFFFFFFFF282828FFFFFFFFFFFFFFFFFF8A8A8A8A8A8A
8A8A8A8A8A8A888888848484565656C1C1C1C1C1C1C1C1C1C1C1C1CDCDCD000000C6C6C6
C8C8C8CCCCCCD1D1D1D7D7D7DCDCDCE2E2E2E8E8E8EDEDEDF1F1F1F5F5F5F7F7F7F9F9F9
FCFCFCFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
000000CCCCCCBABABABDBDBDBDBDBDBEBEBEBDBDBD9E9E9EE3E3E3B6B6B6969696828282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000000CACACAB8B8B8BABABABABABABABABA7C7C7C8D8D8D787878999999F0F0F0C2C2C2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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B88686869E9E9ED4D4D4F0F0F0F2F2F2CECECE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000000C9C9C9B8B8B8B8B8B8B8B8B8B8B8B8B8B8B8868686949494F2F2F2F3F3F3A0A0A0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000000C9C9C9B5B5B5B5B5B5B5B5B5B5B5B5777777A0A0A0E0E0E0F3F3F3E5E5E5ECECEC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000000C9C9C9BABABAB1B1B1B1B1B1B1B1B1B1B1B18D8D8DAAAAAAAAAAAAAAAAAAAAAAAA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000000C3C3C3C9C9C9AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE8D8D8D767676
CECECEFBFBFBFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFBFBFBCECECE7676768D8D8DAEAEAEAEAEAE8D8D8D767676CECECE
FBFBFBFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFBFBFBCECECE7676768D8D8DAEAEAEAEAEAE999999565656B5B5B5CBCBCB
DFDFDFE8E8E8E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8
DFDFDFCBCBCBB5B5B5565656999999AEAEAEAEAEAEAEAEAEC9C9C9C3C3C3000000F6F6F6
F6F6F6F7F7F7F8F8F8F8F8F8F9F9F9FAFAFAFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFD
FEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000C3C3C3D4D4D4B7B7B7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7
A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A7A78D8D8D
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676767676767676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D767676
767676767676767676767676767676767676767676767676767676767676767676767676
7676767676767676767676768D8D8DA7A7A7A7A7A7A7A7A7A7A7A78D8D8D565656565656
565656565656565656565656565656565656565656565656565656565656565656565656
5656565656565656568D8D8DA7A7A7A7A7A7A7A7A7B7B7B7D4D4D4C3C3C3000000F9F9F9
F9F9F9F9F9F9FAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000AAAAAAE7E7E7D4D4D4BEBEBEB3B3B3AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAB3B3B3BEBEBED4D4D4E7E7E7AAAAAA000000FBFBFB
FBFBFBFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
0000008A8A8AD9D9D9FCFCFCE4E4E4D2D2D2CCCCCCBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBECCCCCCD2D2D2E4E4E4FCFCFCD9D9D98A8A8A000000FCFCFC
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000696969979797D9D9D9FCFCFCF2F2F2ECECECE3E3E3D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9E3E3E3ECECECF2F2F2FCFCFCD9D9D9979797696969000000FDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFF696969979797D3D3D3FCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFCFCFCFCFCFCD3D3D3979797696969FFFFFF000000FDFDFD
FDFDFDFDFDFDFDFDFDFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFFFFFFF6969696969698A8A8A979797AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAA9797978A8A8A696969696969FFFFFFFFFFFF000000FEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{368, 319},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005`0001?b000`400?l00000o`00003oo`3oooma
0?ooo`00o`3oooma0?ooo`003P3ooooo0?knoU@0o_kn3`3oool000`0oooo103no_ko0?gmoE00oOgm
103no_h=0?ooo`002P3oool30?kno_l0oOgmEP3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo4X0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?clo?l0no_kC03kno/30?cl
o0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinDd0
nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_I<0?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`4
0?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gmo@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0
moOg0?KfmP3emOD00P3dm?Co0??cldX0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm00X0o?cl0?_kn`3inOT0
n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^k^i:0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0
m_Kf0?Shn03inOT0no_k0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj4X0j>SX00/0jNWY0>_[j`3]
kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0030?gmo@80o_kn103oool000<0oooo0100
o_kn0?gmo@3moOd0oOgm0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN5:0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3kno/0
o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`000P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0
m_Kf0??cl`3_knl0k>c/0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SHB03Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`00`3moOd20?knoP<0oooo
00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;R
hP3MgMd0fMWI0=GEe@3Bd]80d=3@o`3?clm80<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0oooo0040oooo00000`3o
ool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3A
dM40c<c<0<S8b036a/Ko0<G5aDP0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo0?knoP3no_h00P3moOd0
3`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl
_03o0;^k^dP0^k^k00l0_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gmo@020?gmo@0?0?clo03j
n_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a
B02a/K403`2b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd00P3moOd03`3lo?`0nOWi
0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYTP0
YZJV00l0Z:RX0:f][@2c/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080oOgm00l0no_k0?Win@3f
m_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S80;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYi809jN
WP0?0:2PX02UYJD0[Jf]0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0o?cl0?_kn`3inOT0m_Kf
0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l_;`0/[:b0:RXZ02PX:00o`2JVYY:09ZJVP0?0:2P
X02XZ:P0/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000005`000000100VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00003
0000003lo?`0o?cl0?l0o?clF03lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`00
0000o?cl0:^[Z`3o0:^[ZeL0Zj^[0180o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFUF06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj8@2OWil00`2m_Kd0k>c/0>OWi`0[0>OWi`030>k^kP2=SHd0WinO00`0WinO00<0SHf=
0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3Ng]h0fm_K
00`0fm_K00@0g]kN09JFUP2FUYH0fm_K8@3:b/X00`3Kfm/0Xj>S0?_kn`0[0?_kn`0309JFUP3Ng]h0
fm_K00`0fm_K00<0g]kN09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo8@3@d=000`3oool0UYJF0?_kn`0[0?_kn`0309JFUP3oool0c/k>00`0c/k>00<0oooo
09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY
09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF
09JFUP3ooolQ0<o?c`030?ooo`2FUYH0no_k02/0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ0000000=3@d0d];B0>3P
h03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3Lg=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph
0=WIf@0F0>3Ph00309JFUP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM40103@d=090=7Ad@<0d=3@0P3AdM440=3@d0P0dM7A00<0oooo09JF
UP3jn_X0:`3jn_X00`2FUYH0oooo0<k>cP040<k>cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JF
UP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000
003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3P
h01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0103>c/h303hn?PD0c/k>
00@0oooo09JFUP2FUYH0oooo2P3Fe]H20=OGe`080=KFeP2EUID0e]KF09FEU@3Fe]H0UIFE0=KFeP2E
UID80=KFeP80emOG0`3Fe]H00`3oool0UYJF0?Win@0[0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n
?Sh60<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>00
0`3oool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph034a<@0B4Q80=;B
dP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY
09jNWQH0h>3P00<0UYJF0?ooo`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3L
g=`90=gMg@090?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID0g=cL00T0gMgM0P3Lg=`00`3M
gMd0oooo09JFUP0/0?Win@0309JFUP3oool0clo?00@0clo?0`0n?Sh50<o?c`030?ooo`2FUYH0h>3P
00P0h>3P0P2c/k<06P3oool0h>3P0;>c/`2c/k<0oooo0>3Ph01YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B0>3P
h03IfMT09RHV0=;BdP@0h>3P00L0a<C404Q8B03Bd]806QXJ0000000=3@d0d];B00X0h>3P1`000005
0>3Ph00309JFUP3oool0e=CD0080e=CD0`0n?Sh70=CDe0040?ooo`2FUYH0UYJF0?ooo`<0g=cL2@3M
gMd02@3oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0=cLg0090=gMg@80g=cL00<0gMgM0?oo
o`2FUYH0;03hn?P00`2FUYH0oooo0=CDe0050=CDe0<0?Shn103De=@00`3oool0UYJF0>3Ph0080>3P
h080/k>c0P3Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>005`34a<@0
B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV9P3Bd]80
g=cL04U9B@0000003@d=0=;BdP1YJFT0WYjN00l0h>3P1@0000060>3Ph00409JFUP3oool0fm_K0=_K
f`<0?Shn203Kfm/01@3oool0UYJF09JFUP3oool0i>CT0080hn?S0P3Ti>@50>?Sh`80i>CT00T0oooo
09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@3Shn<00P3Ti>@50>?Sh`<0i>CT00@0hn?S0>CTi03o
ool0UYJF;03gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3P
h00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0
JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80
h>3P0<C4a018B4P0d];B02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0
a<C404Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0?Shn1`3Ti>@01@3o
ool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X70>WYj@090?ooo`2EUID0oooo09FEU@3oool0UIFE
0?ooo`2EUID0jNWY0080j^[Z1`3YjNT01@3Zj^X0jNWY0>WYj@3oool0UYJF02`0mOGe00<0UYJF0?oo
o`3Ti>@01@3Ti>@303hn?P@0i>CT00<0oooo09JFUP3Ph>003@3Ph>020?ooo`0E0>3Ph01YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0
h>3P00<0oooo0>3Ph03Ph>000`3Ph>006@2>SXh0000003Ph>03IfMT0h>3P06UYJ@2NWYh0h>3P08j>
SP1DE5@0h>3P06UYJ@2NWYh0h>3P08j>SP1DE5@0h>3P0:f][@1DE5@0h>3P06UYJ@1nOWh0h>3P06UY
J@2NWYh0403Ph>000`000000h>3P0>3Ph0060>3Ph00309JFUP3oool0kNg]00<0kNg]0`0n?Sh60>g]
k@050?ooo`2FUYH0UYJF0?ooo`3Zj^X00P3YjNT20>[ZjPL0jNWY00T0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@3YjNT00P3Zj^X70>WYj@050>[ZjP3YjNT0jNWY0?ooo`2FUYH0;03dm?@00`2F
UYH0oooo0>g]k@040>g]k@<0?Shn1@3]kNd00`3oool0UYJF0>3Ph00<0>3Ph080/k>c01H0oooo0>3P
h01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UY
J@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00I0=WIf@1BDU80fMWI0>3Ph03IfMT06QXJ0000
000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0>3P
h034a<@07AdM0=;BdP0H0>3Ph00309JFUP3oool0m?Cd00@0m?Cd0`0n?Sh50?Cdm0040?ooo`2FUYH0
UYJF0?ooo``0kNg]00L0oooo0>g]k@3oool0kNg]0?ooo`3]kNd0oooo00h0kNg]00<0oooo09JFUP3a
lO40:`3alO400`2FUYH0oooo0?Cdm0030?Cdm0<0?Shn1P3dm?@00`3oool0UYJF0>3Ph00<0>3Ph080
/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0o?cl00D0o?cl
00<0?Shn0?clo03lo?`0103lo?`0103oool0UYJF09JFUP3ooolQ0>g]k@030?ooo`2FUYH0l?3`02/0
l?3`00<0UYJF0?ooo`3lo?`0103lo?`00`0n?Sh0o?cl0?clo0050?clo0030?ooo`2FUYH0h>3P0100
h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[
Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`2FUYH0oooo0?ooo`0=0?ooo`80UYJF
00<0oooo0?Ogm`3gmoL07`3gmoL00`3oool0UYJF0>o_k`0[0>o_k`0309JFUP3oool0oooo00d0oooo
00<0UYJF0>3Ph03Ph>003`3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3Ph00309JFUP3G
emL0oooo00`0oooo00@0emOG09JFUP2FUYH0n_[j8@3oool00`3jn_X0UYJF0>o_k`0[0>o_k`0309JF
UP3GemL0oooo00`0oooo00<0emOG09JFUP3Ph>00403Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph0030=[JfP2BTY80][Jf00/0]KFe00D0];Bd08n?S`3Jf]X0e]KF0;2`/00Q0;Rh^0030:VY
Z@3>c/h0k^k^02/0k^k^00<0f][J09:BTP2f][H02`2e]KD00`2d];@0Shn?0=[JfP0A0>3Ph00D06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYU`2N
WYh00`3oool0h>3P0>3Ph00e0>3Ph140k>c/9@3[jn//0>OWia40k>c/4@3Ph>00501YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo:@3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`030>c/k03^k^h0SHf=00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo:@3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo:@3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo:@3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo:@3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo:@3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h0
5P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolN0?ooo`D0820P1P3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0
oooo00@0c/k>0`0n?Sh50<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooob80oooo00<0820P0?ooo`3oool0103oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00<0clo?1@0n?Sh40<o?c`0F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooob80oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0
oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh30=CDe00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTJ0?ooo`030?_kn`1D
E5@0];Bd00<0oooo00<0XZ:R0451@@3gmoL0oP3oool00`0P8200oooo0?ooo`040?ooo`03020P803o
ool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803o
ool0oooo0080oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn0P3Kfm/05P3oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
5@3oool0402XZ:P0d];B08N7Q`3_knl0a/K60;Bd]027QhL0kno_0::RXP0000002@T90=;BdP2][Jd0
kno_0:RXZ03Bd]8>0?ooo`0@07QhN00000002@T90=;BdP3gmoL07AdM0000000?3`l0kno_0?Ogm`0M
7Ad0000000l?3`3_knl0gmoO018B4P<0000000H0];Bd0?ooo`3oool0oooo0?Ogm`0M7Ad200000005
08n?S`3oool0oooo0?ooo`2RXZ800`0000001`2?Shl0oooo0?_kn`1MGEd0d];B0?ooo`36a/H00P00
00000`0?3`l0kno_0?ooo`2o0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103o
ool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3o
ool00`3/k>`0UYJF0?ooo`020>CTi00303hn?P3Ti>@0i>CT00<0i>CT01X0?Shn0>CTi03Ti>@0i>CT
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTE0?ooo`080=oOg`2JVYX0Z:RX0=;BdP3Ogml0VYZJ0<_;b`3Bd]820?ooo`040?_kn`2P
X:00gmoO0;jn_P80oooo00D0a/K6000000000000@D510?Ogm`090?ooo`0C0::RXP2d];@0oooo0?oo
o`1hN7P0];Bd0?ooo`36a/H0H61P07QhN02d];@0oooo0<K6aP1PH600N7Qh0;Bd]03oool0XZ:R08n?
S`0=0?ooo`030?Ogm`0/;2`0kno_00<0oooo00<0N7Qh0;Bd]03oool00`3oool00`2RXZ80Shn?0?oo
o`2o0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000
0004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool00`3/k>`0UYJF0?oo
o`0<0>g]k@0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTE0?ooo`0>0?Ogm`1mOGd0gmoO09ZJVP3kno/0X:2P0=oOg`2JVYX0oooo
0?Ogm`0M7Ad0];Bd0?_kn`2PX:020?ooo`050<K6aP00000000000451@@3gmoL02@3oool04`2RXZ80
];Bd0?ooo`3oool0N7Qh08n?S`3oool0XZ:R061PH01hN7P0Shn?0?ooo`2RXZ80H61P07QhN02d];@0
oooo07QhN02?Shl0103oool01@3gmoL07AdM000000000000Shn?00@0oooo00<0moOg02`/;03_knl0
0`3oool00`36a/H0H61P0?ooo`030?ooo`030::RXP2?Shl0oooo0;l0oooo00<0820P0?ooo`3oool0
103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0
103oool00`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0m?Cd01H0oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAD0oooo
0140no_k08>3P`3_knl07AdM0000001PH600moOg07emO@3gmoL0Y:BT0?ooo`27QhL0kno_0:f][@3[
jn/0Ph>30?Ogm`0<0?ooo`0A07QhN00000002@T90=;BdP3oool0moOg01dM7@0000003`l?0>o_k`3g
moL07AdM0000000?3`l0kno_0?Ogm`0M7Ad00P0000000`2?Shl0oooo0?ooo`0;0?ooo`030?Ogm`0/
;2`0kno_00<0oooo00P0moOg05iNGP3gmoL0moOg01dM7@0000003`l?0>o_kl00oooo00<0820P0?oo
o`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000
003oool0103oool00`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0o?cl01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JAH0oooo00<0[Jf]0>o_k`3oool00`3oool0201hN7P0H61P07QhN0000000H61P0?Ogm`0M7Ad0];Bd
3`3oool00`2RXZ80];Bd0?ooo`0>0?ooo`030::RXP2?Shl0oooo00`0oooo00`0moOg02`/;03_knl0
oooo0<K6aP1PH600oooo0861P@3Bd]80moOg05iNGP3gmoO10?ooo`80820P1P3oool00`0P8200oooo
0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo
0?ooo`020?ooo`030>c/k02FUYH0oooo00d0oooo01D0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC<0oooo00D0gmoO018B4P000000@D51
0?Ogm`0;0?ooo`030<K6aP000000Shn?00`0oooo00H0a/K60000000?3`l0kno_0?ooo`36a/H20000
000700T92@3Bd]80moOg01dM7@0000002@T90=;BdP2n0?ooo`03020P803oool0820P00H0oooo00<0
820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0
820P0?ooo`3oool00P3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooal0oooo00@0
820P0?ooo`3oool0820P1P3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@00000
00<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0
]KFe01L0];Bd08n?S`3Jf]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`05020P803oool0oooo0?ooo`0P82001P3oool00`0P
8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooad0oooo1P0P82020?ooo`D0820P1P3oool00`0P
8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oool`0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P
8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oool`0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P
8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oool`0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P
8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oool`0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P
8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolN0?ooo`D0820P0P3oool5020P80H0oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo8P3oool00`0P
8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo8P3oool00`0P
8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJB00oooo00<0no_k05AD
E02d];@00`3oool00`2RXZ80@D510?Ogm`3h0?ooo`03020P803oool0oooo00@0oooo00<0820P0?oo
o`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?oo
o`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUY5P3oool01P3kno/0E5AD00000011@D40moOg0::RXP80000000H0
];Bd0::RXP1PH600QhN70>o_k`2RXZ820000000308n?S`2][Jd0kno_00h0oooo00P0N7Qh0000000?
3`l0kno_0?ooo`1hN7P000000451@@80oooo00<0N7Qh00000011@D40g@3oool00`0P8200oooo0?oo
o`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?oo
o`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAH0oooo01@0a/K60;Bd]03oool0gmoO
079bLP3;bl/0d];B0<K6aP2?Shl0bl_;0=;BdP3;bl/0d];B0?_kn`1DE5@0H61P0?ooo`3Ogml0_[jn
0<K6aP80000000<0@D510?Ogm`3oool01`3oool03`36a/H0H61P0?ooo`1hN7P0];Bd0<K6aP1PH600
oooo07QhN02d];@0gmoO079bLP3oool0Z:RX0=;BdP3I0?ooo`@0820P1P3oool00`0P8200oooo0?oo
o`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?oo
o`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTF0?ooo`0=0:RXZ03Bd]80oooo0?ooo`27QhL0clo?09ZJVP3kno/0X:2P
0=oOg`2JVYX0gmoO09ZJVP020?ooo`050<K6aP2?Shl0no_k0:2PX036a/H00P0000000`11@D40moOg
0?ooo`080?ooo`030?Ogm`0M7Ad0H61P00@0oooo00L0XZ:R0;Bd]036a/H0Shn?0?ooo`2RXZ80];Bd
0=`0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool00000
00@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTF0?oo
o`0D0=oOg`1bLW80oooo0?_kn`23Ph<0kno_07emO@3oool0YJFU0;jn_P000000[:b/07emO@3gmoL0
OGem0?ooo`27QhL0kno_0:f][@3_knl?0?ooo`0<0861P@3Bd]80moOg01dM7@0000003`l?0>o_k`36
a/H0Shn?0?ooo`2RXZ80];Bdf`3oool2020P80H0oooo00<0820P0?ooo`3oool0103oool00`0P8200
oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUY5`3oool0102RXZ8000000000002d];@40?ooo`0;0?_kn`23Ph<0moOg07QhN01PH600N7Qh0000
001PH600moOg01dM7@2d];@0303oool01`3Ogml08B4Q0>o_k`21PH40d];B0?Ogm`1NGUh00`3oool0
1@3Ogml0De=C0?ooo`21PH40d];B0=X0oooo00<0820P0?ooo`0P82001P3oool00`0P8200oooo0?oo
o`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?oo
o`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTg0?ooo`0=0<K6aP000000@D510?ooo`3gmoL07AdM0000000?3`l0kno_
0?ooo`2RXZ800000061PH03J0?ooo`04020P803oool0oooo020P80H0oooo00<0820P0?ooo`3oool0
103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolN0?ooo`05020P803oool0oooo0?ooo`0P82001P3oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?oooad0oooo1P0P82060?ooo`03020P803oool0oooo00@0
oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0
000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobT0
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0
oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolY0?ooo`03020P803oool0oooo00@0
oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0
000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobT0
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0
oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolY0?ooo`03020P803oool0oooo00@0
oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0
000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobT0
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0
oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`D0820P1P3oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooob80oooo00<0820P0?ooo`3oool0103oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTJ0?ooo`030?_kn`1DE5@0];Bd00<0oooo00<0XZ:R0451@@3g
moL0oP3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAD0
oooo00T0Z:RX0=;BdP27QhL0kno_0<K6aP2d];@0QhN70>o_k`2RXZ800P0000001@2?Shl0[Jf]0>o_
k`2XZ:P0d];B00h0oooo00H0N7Qh000000000000];Bd0?Ogm`0M7Ad20000000805MGE`0M7Ad02@T9
0=;BdP3Ogml08B4Q0>o_k`36a/H20000000308n?S`3oool0oooo0080oooo00D0moOg01dM7@000000
000008n?S`030?ooo`060<K6aP000000000000l?3`3_knl0a/K60P0000001P0?3`l0kno_0?ooo`1h
N7P00000061PH<40oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P
803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTE0?ooo`0>0=oOg`2JVYX0Z:RX0=;BdP3Ogml0VYZJ0<_;b`3Bd]80no_k05ADE01PH600oooo
0=oOg`2n_[h20?ooo`050<K6aP00000000000451@@3gmoL02@3oool01P2RXZ80];Bd0?ooo`3oool0
N7Qh08n?S`<0oooo00P0a/K608n?S`3oool0no_k0>?Sh`3oool0XZ:R0;Bd]140oooo00<0XZ:R08n?
S`3oool00P3oool01`2RXZ80Shn?0<K6aP11@D40lo?c05ADE02d];@0`03oool00`0P8200oooo0?oo
o`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?oo
o`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAD0oooo00P0moOg07emO@3Ogml0VYZJ
0?_kn`2PX:00gmoO09ZJVP80oooo00@0a/K608n?S`3kno/0X:2P0P3oool01@36a/H0000000000011
@D40moOg00T0oooo00D0XZ:R061PH03oool0oooo07QhN00300000009061PH036a/H0Shn?0?ooo`2R
XZ80];Bd0?ooo`2RXZ80];Bd00H0oooo00D0moOg01dM7@000000000008n?S`060?ooo`0307QhN02d
];@0oooo0080oooo00L0XZ:R08n?S`36a/H0H61P0?ooo`2RXZ80];Bd0<00oooo00<0820P0?ooo`3o
ool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003o
ool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTE0?ooo`0A0?_kn`23Ph<0kno_01dM
7@000000H61P0?Ogm`1mOGd0moOg07emO@3oool0QhN70>o_k`2][Jd0jn_[08>3P`3gmoL0303oool0
2`2RXZ800000000000000000@D510>o_k`0M7Ad0000000T92@3:b/X07AdM0080000000<03`l?0>o_
k`36a/H00P0000000`092@T0d];B0?ooo`0<0?ooo`030=oOg`0B4Q80@D510080moOg00T07AdM0000
000?3`l0kno_0<K6aP1PH600oooo0::RXP2d];@0`03oool00`0P8200oooo0?ooo`040?ooo`03020P
803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P
803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJAH0oooo00<0[Jf]0>o_k`3oool00`3oool0201hN7P0H61P
07QhN0000000H61P0?Ogm`0M7Ad0];Bd7P3oool00`2RXZ80];Bd0?ooo`0@0?ooo`0507QhN02d];@0
moOg05iNGP3gmoL00P3oool01@36a/H0@D510??cl`1DE5@0];Bd0;l0oooo0P0P82060?ooo`03020P
803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P
803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJE80oooo00h0moOg01dM7@0000003`l?0>o_k`3gmoL07AdM
000000092@T0d];B0?ooo`1hN7P00000061PH;l0oooo00<0820P0?ooo`0P82001P3oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?oooal0oooo00@0820P0?ooo`3oool0820P1P3oool00`0P
8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00D0820P0?ooo`3oool0oooo020P8006
0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`04
0?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7@3oool6020P8080oooo1@0P8206
0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`04
0?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo00000004
000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool`
0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`04
0?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo00000004
000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool`
0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFVd0?ooo`030:NWY`1XJ6P0oooo06d0oooo1@0P82060?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFVe
0?ooo`030=WIf@3QhN40oooo0700oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004
000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY]P3oool0
0`2b/[80oooo0?ooo`1_0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`00000010000000
0`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`
l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC<0oooo00`0l?3`
0;fm_@000000_Kfm0?3`l03oool0fMWI0:NWY`3@d=00YjNW0=WIf@3oool207alO0030;fm_@000000
_Kfm0080oooo00h0d=3@0000002b/[80_Kfm0000002<S8`0d=3@0000003IfMT000000=WIf@3QhN40
00000=WIf@<0oooo00<0O7al0;fm_@2WYjL00P3oool01P000000YjNW0;fm_@2WYjL0d=3@0:NWY`80
fMWI00L000000=WIf@3oool0d=3@06QXJ02<S8`0YjNW00H0oooo00/0S8b<0?ooo`3oool0000008b<
S03@d=0000000=WIf@000000hN7Q0?ooo`0207alO00:0;fm_@000000_Kfm06QXJ02<S8`0YjNW0?oo
o`1lO7`0_Kfm0:NWY`<0oooo00T0fMWI0000003QhN40fMWI0000003IfMT0hN7Q0000003IfMT00P3o
ool06@3IfMT000000>7Qh@3IfMT000000>7Qh@3IfMT000000;fm_@1XJ6P0S8b<0:NWY`3oool0fMWI
0000003QhN40fMWI0000003QhN40fMWI0000002m_Kd0YjNW0=3@d02WYjL00P3IfMT01P000000fMWI
0?ooo`3oool0l?3`000000<0oooo00<000000>7Qh@3oool0J`3oool00`0P8200oooo0?ooo`040?oo
o`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFTe0?ooo`050000003oool0oooo0?ooo`1lO7`00`3oool00`1XJ6P0oooo00000002
0?ooo`0>0000003oool0oooo0?ooo`000000l?3`0?ooo`3`l?00YjNW0>7Qh@3oool000000?ooo`00
00020?ooo`030000003oool0oooo0080oooo00<0l?3`0>7Qh@0000000P3oool00`000000oooo07al
O0030?ooo`0306QXJ03oool000000080oooo00@000000?3`l03oool0l?3`1@3oool02P2m_Kd00000
0?3`l03oool0YjNW0>7Qh@3oool000000?ooo`0000020?ooo`030000003oool0oooo0080000000L0
l?3`0?ooo`3`l?00oooo0?3`l03QhN40000000@0oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`020?ooo`040000003oool0oooo00000080oooo0P0000000`3`l?00oooo0?3`l002
0?ooo`040000003oool0oooo00000080oooo00<0000007alO03oool00P3oool00`1XJ6P0oooo0000
00030?ooo`030;:b/P1XJ6P0_Kfm0080oooo00<0l?3`0?ooo`3oool0J`3oool00`0P8200oooo0?oo
o`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?oo
o`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTe0?ooo`050000003oool0oooo0?ooo`0000000`3oool00`000000oooo
000000020?ooo`050000003oool0oooo0?ooo`000000103oool01@3QhN40d=3@0000003oool00000
0080oooo00<000000?ooo`3oool00P3oool00`2b/[80000008b<S0020?ooo`030000003oool00000
00<0oooo00<000000?ooo`0000000P3oool010000000d=3@0=3@d03@d=050?ooo`0306QXJ03IfMT0
_Kfm0080oooo00D0hN7Q0=3@d0000000oooo000000020?ooo`030000003oool0oooo008000000`3@
d=00103oool0/[:b0000002<S8`40?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3o
ool00P3oool010000000oooo0?ooo`0000020?ooo`8000000`3@d=020?ooo`040000003oool0oooo
00000080oooo0P0000030?ooo`030000003oool0000000<0oooo00<0J6QX0=WIf@2m_Kd0L03oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTd0?ooo`030;:b/P000000_Kfm0080oooo00L0J6QX
0?ooo`3oool0oooo06QXJ03oool000000080oooo00T000000?ooo`3oool0oooo07alO03oool0l?3`
0;fm_@1XJ6P00P3oool00`000000oooo000000020?ooo`030000003oool0oooo0080oooo00P00000
0?3`l03`l?00oooo0=3@d0000000S8b<06QXJ0<0oooo00<0J6QX0?ooo`0000000P3oool0101XJ6P0
oooo0?ooo`1XJ6P50?ooo`050000003oool0/[:b0?ooo`1XJ6P00P3oool00`000000oooo00000002
0?ooo`050000003oool0oooo0000001XJ6P00P3oool00`1XJ6P0oooo000000020?3`l0<0oooo00D0
fMWI0000003oool0oooo000000020?ooo`030000003oool0oooo0080oooo00@000000?ooo`3oool0
00000P3oool01@000000J6QX0?ooo`3oool0J6QX0080oooo00@000000?ooo`3oool000000P3oool0
0`000000J6QX0?ooo`020?ooo`0306QXJ03oool0000000<0oooo00@000000?ooo`3IfMT0hN7QK`3o
ool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103o
ool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg
0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTc0?ooo`0@0?3`l0000000d=3@0;fm_@3`l?00
oooo0=WIf@2b/[80d=3@0:NWY`3IfMT0d=3@0000003oool0d=3@000000<0oooo00/0fMWI0;:b/P2W
YjL0S8b<0=3@d02m_Kd0J6QX08b<S03@d=0000000=3@d00207alO0@0oooo01@0YjNW0;fm_@000000
oooo0?3`l01lO7`0oooo0=WIf@2b/[80d=3@0:NWY`3IfMT0_Kfm0000002<S8`000000?3`l02b/[80
d=3@0;fm_@@0oooo00<0S8b<0000003`l?000P2WYjL0503@d=00_Kfm06QXJ02<S8`0oooo0000003o
ool0d=3@0000003oool0d=3@0000003`l?00/[:b0=3@d02m_Kd0oooo0:NWY`2m_Kd000000`3oool0
1P3QhN40_Kfm0?ooo`3@d=0000000=3@d080O7al0`3oool0902m_Kd000000=3@d01lO7`000000=3@
d0000000YjNW0?3`l02b/[80d=3@0;fm_@3oool0_Kfm0000003@d=00O7al0000003@d=0000000:NW
Y`3IfMT0/[:b0=3@d02WYjL0fMWI0;fm_@000000S8b<0000002WYjL000000?3`l03@d=00J6QX0?3`
l6h0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo
00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY<`3oool01@2<S8`000000?ooo`3oool0
/[:b0440oooo00<000000?ooo`3oool06`3oool00`3`l?00oooo0?ooo`0:0?ooo`030?3`l03oool0
oooo0840oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000
oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY<P3oool0202m_Kd0000007alO03o
ool0hN7Q07alO02m_Kd0l?3`?P3oool00`3IfMT000000?ooo`0A0?ooo`030000003oool0oooo09T0
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0
oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYN@3oool00`2b/[80oooo0?ooo`2/0?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo9P3oool4020P80H0oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003o
ool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobD0oooo1@0P82060?ooo`03
020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03
020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03
020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooah0oooo1@0P82020?ooo`D0820P1P3oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolR0?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolR0?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY803oool00`3kno/0
E5AD0;Bd]0030?ooo`030::RXP11@D40moOg0?P0oooo00<0820P0?ooo`3oool0103oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTF0?ooo`060?_kn`1DE5@000000451@@3gmoL0XZ:R0P000000
2`2d];@0XZ:R061PH027QhL0kno_0?ooo`3Ogml0VYZJ0?ooo`2][Jd0kno_01<0oooo00<0N7Qh0000
0011@D400P3oool00`36a/H000000;Bd]0030?ooo`0:05ADE0000000];Bd0?Ogm`0M7Ad000000451
@@3oool0gmoO018B4P80000000l0];Bd0?Ogm`0M7Ad000000451@@3oool0gmoO018B4P000000@D51
0?ooo`3Ogml04Q8B00000011@D40_`3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo
00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo
0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJAH0oooo01@0a/K60;Bd]03oool0gmoO079bLP3;bl/0d];B0<K6aP2?Shl0
bl_;0=;BdP3;bl/0d];B0?ooo`3kno/0X:2P0?ooo`3Ogml0_[jn0<K6aP80000000<0@D510?Ogm`3o
ool01`3oool02P3gmoL07AdM000000000000Shn?0=oOg`1bLW80oooo0:RXZ03Bd]870?ooo`080?Og
m`1NGUh0oooo0<K6aP2d];@0oooo0:RXZ03Bd]820?ooo`0806=SH`3_knl0oooo0<K6aP2d];@0oooo
0:RXZ03Bd]830?ooo`030:RXZ03Bd]80oooo0080oooo00<0Z:RX0=;BdP3oool0^P3oool4020P80H0
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0
oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY5P3oool03@2XZ:P0d];B0?ooo`3oool0QhN7
0<o?c`2JVYX0no_k0:2PX03Ogml0VYZJ0=oOg`2JVYX00P3oool01@2UYJD0moOg0?_kn`2PX:00a/K6
0080000000<0@D510?Ogm`3oool0303oool01@36a/H0Shn?0?ooo`2RXZ80];Bd00D0oooo00X0gmoO
018B4P00000000000;Bd]03gmoL0OGem0?ooo`27QhL0kno_0P3oool02P1SHf<0kno_0?ooo`3gmoL0
OGem0?ooo`27QhL0kno_0?ooo`1DE5@20000000308n?S`3oool0E5AD0080000000<0Shn?0?ooo`3o
ool0_03oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAH0
oooo01@0gmoO079bLP3oool0no_k08>3P`3_knl0OGem0?ooo`2UYJD0_[jn0000002/[:`0OGem0?_k
n`1DE5@02@T90=;BdP3oool0[Jf]0>o_ka40oooo00D0a/K608n?S`3oool0XZ:R0;Bd]0050?ooo`09
0?Ogm`0/;2`0inOW05iNGP3oool0moOg01dM7@000000@D5100<0oooo0180Hf=S0>o_k`3oool0moOg
01dM7@000000@D510?ooo`3Ogml0LW9b0?ooo`3Ogml0VYZJ0=oOg`1bLW80oooo0=oOg`2JVYZm0?oo
o`80820P1P3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000
003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTG0?ooo`040::RXP0000000000
0;Bd]0@0oooo00/0no_k08>3P`3gmoL0N7Qh061PH03oool0XZ:R0;Bd]03gmoL07AdM0;Bd]00A0?oo
o`050=oOg`1CDe<0oooo0861P@3Bd]801P3oool07@36a/H0EeMG05iNGP3oool0gmoO09ZJVP3oool0
PH610=;BdP3Ogml0LW9b06=SH`3_knl0oooo0=oOg`2JVYX0oooo0861P@3Bd]80gmoO079bLP3oool0
PH610=;BdP3Ogml0LW9b0?ooo`21PH40d];B0;`0oooo00<0820P0?ooo`0P82001P3oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030::RXP000000H61P00P0oooo00L0N7Qh0451@@3o
ool0oooo05ADE0000000H61P0080oooo00<0N7Qh00l?3`3_knl00P3oool00`1DE5@00000061PH002
0?ooo`0307QhN0000000H61P0080oooo00<0N7Qh0000001PH600_03oool0100P8200oooo0?ooo`0P
82060?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?oo
o`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool01@0P8200oooo0?oo
o`3oool0820P00H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oO
g`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolM0?ooo`H0820P
1P3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0
103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobT0oooo00<0820P0?ooo`3oool0
103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolY0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000
100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0
103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobT0oooo00<0820P0?ooo`3oool0
103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolY0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000
100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0
103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobT0oooo00<0820P0?ooo`3oool0
103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolY0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000
100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
7P3oool5020P80H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oO
g`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYS03oool00`2RXZ80Shn?
0?ooo`0T0?ooo`030<K6aP2?Shl0oooo06/0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo
0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo
0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0
n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFV;0?ooo`030?Ogm`1NGUh0moOg01D0oooo00H0XZ:R000000000000
000008n?S`1hN7P3000000030;Bd]03oool0oooo00@0oooo00<0no_k06=SH`3_knl0J`3oool00`0P
8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJH/0oooo00<0a/K608n?
S`3oool05P3oool01`3gmoL0;2`/0>o_k`3oool0no_k05ADE02?Shl02@3oool00`2RXZ80];Bd0?oo
o`1Z0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000
0004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYR`3o
ool00`2RXZ80];Bd0?ooo`0F0?ooo`030?Ogm`0/;2`0kno_00<0oooo00<0XZ:R0;Bd]03oool01`3o
ool00`3Ogml0LW9b0?ooo`1Z0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0103o
ool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3o
ool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYR`3oool00`21PH40d];B0?ooo`0F0?ooo`030?Ogm`0/;2`0kno_00@0oooo00<0
PH610=;BdP3oool01P3oool00`3gmoL0GUiN0?Ogm`1Z0?ooo`03020P803oool0oooo00@0oooo00<0
820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0
820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYRP3oool00`3kno/0Hf=S0>o_k`0G0?ooo`090?Ogm`0/
;2`0kno_0?ooo`36a/H0H61P0?_kn`1DE5@0];Bd00L0oooo00<0moOg02`/;03_knl0JP3oool00`0P
8200oooo0?ooo`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3O
gml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD<0oooo00@0moOg01dM
7@0?3`l0kno_403oool0103gmoL07AdM00l?3`3_knlE0?ooo`050?_kn`1DE5@0000000l?3`3_knl0
0P3oool0103gmoL07AdM00l?3`3_knl60?ooo`040?Ogm`0M7Ad03`l?0>o_k`D0oooo00<0n?Sh02/[
:`3Shn<05P3oool02P36a/H0000000l?3`3_knl0oooo0?_kn`1DE5@000000451@@3gmoL70?ooo`03
0?Shn00[:b/0hn?S06X0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03
020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTJ0?ooo`030?_kn`1DE5@0];Bd00<0oooo00<0XZ:R0451@@3gmoL0803oool00`3gmoL0
;2`/0>o_k`0B0?ooo`030?_kn`1SHf<0kno_01P0oooo00L0XZ:R0;Bd]03oool0oooo0?Ogm`0/;2`0
kno_00P0oooo00<0no_k06=SH`3_knl01@3oool00`3hn?P0:b/[0>?Sh`0W0?ooo`030?Shn00[:b/0
hn?S06X0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTE
0?ooo`0@0:RXZ03Bd]80QhN70>o_k`36a/H0];Bd08N7Q`3_knl0oooo0=oOg`2JVYX0oooo0:f][@3_
knl0Z:RX0=;BdPd0oooo00H0XZ:R0000000000003`l?0>o_k`1hN7P30000000;05MGE`0M7Ad02@T9
0=;BdP3Ogml08B4Q0>o_k`3oool0moOg02`/;03_knl00P3oool03P3Ogml06a/K0=;BdP3oool0no_k
05ADE0000000@D510?Ogm`3oool0XZ:R0000000?3`l0kno_0P3oool00`3kno/0Hf=S0>o_k`0;0?oo
o`0<0?_kn`1DE5@0000000000011@D40kno_01dM7@0000003`l?0>o_k`3gmoL07AdM0P0000004@2?
Shl0oooo0?ooo`3gmoL0;2`/0>o_k`3oool0no_k05eMG@3Bd]80oooo0::RXP2?Shl0oooo0?_kn`1S
Hf<0kno_00D0oooo00D0n?Sh02/[:`3Shn<0oooo0::RXP030000000308n?S`3oool0oooo0080oooo
00<0no_k06=SH`3_knl06P3oool00`3hn?P0:b/[0>?Sh`1Z0?ooo`03020P803oool0oooo00@0oooo
00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo
00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY5@3oool03P3Ogml0VYZJ0:RXZ03Bd]80gmoO09ZJ
VP3;bl/0d];B0?ooo`3kno/0X:2P0?ooo`3Ogml0_[jn0P3oool01@36a/H0000000000011@D40moOg
00P0oooo00l0XZ:R08n?S`3oool0XZ:R08n?S`3oool0moOg02`/;03_knl0oooo0<K6aP2?Shl0oooo
0?_kn`3Shn<00P3oool00`3gmoL0;2`/0>o_k`060?ooo`0:0<K6aP1PH600oooo07QhN02d];@0moOg
02`/;03_knl0XZ:R08n?S`80oooo00<0no_k06=SH`3_knl0103oool01@3gmoL07AdM000000000000
Shn?00<0oooo00h0PH610=;BdP3Ogml0De=C071`L02d];@0oooo0<K6aP1PH600N7Qh0;Bd]03oool0
N7Qh08n?S`80oooo00<0moOg02`/;03_knl00P3oool02@3gmoL0GUiN0??cl`1SHf<0kno_0?ooo`3k
no/0Hf=S0>o_k`050?ooo`030?Shn00[:b/0hn?S0080oooo00<0moOg02`/;03_knl0103oool01@36
a/H00000000000000000];Bd00<0oooo00<0moOg01dM7@000000300000001@071`L04Q8B00000009
2@T0d];B0080oooo00<0n?Sh02/[:`3Shn<0JP3oool00`0P8200oooo0?ooo`040?ooo`03020P803o
ool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJAD0oooo00P0moOg07emO@3Ogml0VYZJ0?_kn`2PX:00gmoO09ZJ
VP80oooo00@0YJFU0?Ogm`3kno/0X:2P0P3oool01@36a/H0000000000011@D40moOg00X0oooo00d0
a/K60000002d];@0oooo0?Ogm`0/;2`0kno_0?ooo`36a/H0Shn?0?ooo`2RXZ80];Bd0080oooo00<0
moOg02`/;03_knl01P3oool02P36a/H0@D510?Ogm`1hN7P0];Bd0=oOg`0K6a/0d];B0::RXP2?Shl2
0?ooo`030?_kn`1SHf<0kno_00`0oooo00h0PH610=;BdP3oool0oooo07QhN02?Shl0oooo0::RXP1P
H600N7Qh0;Bd]03oool0N7Qh08n?S`80oooo00<0moOg02`/;03_knl00`3oool0202RXZ80/;2`06=S
H`3_knl0oooo0?_kn`1SHf<0kno_1@3oool00`3kno/0Hf=S0>o_k`020?ooo`030?Ogm`0/;2`0kno_
00D0oooo00<0no_k06=SH`3_knl06P3oool00`3kno/0Hf=S0>o_k`1Z0?ooo`03020P803oool0oooo
00@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo
00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY5@3oool04@3kno/0Ph>30>o_k`0M7Ad0
0000061PH03gmoL0OGem0?_kn`1DE5@02@T90=;BdP3oool0[Jf]0>_[j`23Ph<0moOg00`0oooo00<0
gmoO018B4P1PH6000P3oool01P36a/H0000000l?3`3_knl0moOg01dM7@80000000L03`l?0>o_k`3o
ool0oooo0?Ogm`0/;2`0kno_00L0oooo00T0N7Qh0000001PH600oooo0=oOg`1>CTh0BTY:00l?3`3_
knl00P3oool00`3kno/0Hf=S0>o_k`0<0?ooo`0;0861P@3Bd]80oooo0?ooo`3gmoL07AdM0000000?
3`l0kno_0?Ogm`0M7Ad00P0000001P092@T0d];B0?ooo`3gmoL0;2`/0>o_k`<0oooo00P0a/K608n?
S`21PH40d];B0?ooo`3kno/0Hf=S0>o_k`D0oooo00<0no_k06=SH`3_knl00P3oool00`3gmoL0;2`/
0>o_k`0R0?ooo`030?_kn`1SHf<0kno_06X0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo
0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo
0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0
n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFTF0?ooo`030:f][@3_knl0oooo00<0oooo00P0N7Qh061PH03oool0
XZ:R0;Bd]03gmoL07AdM0;Bd]0h0oooo00D0a/K6061PH03oool0XZ:R0;Bd]00<0?ooo`030?Ogm`0/
;2`0kno_00H0oooo00P0a/K6061PH03oool0XZ:R0;Bd]03kno/0E5AD08n?S`@0oooo00<0no_k06=S
H`3_knl0303oool00`21PH40d];B0?ooo`0=0?ooo`030?Ogm`0/;2`0kno_0080oooo00<0N7Qh0000
000000000P000000102d];@0no_k06=SH`3_knl50?ooo`030?_kn`1SHf<0kno_0080oooo00<0moOg
02`/;03_knl08P3oool00`3kno/0Hf=S0>o_k`1Z0?ooo`03020P803oool0oooo00@0oooo00<0820P
0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P
0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Wi
n@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY<P3oool02@3kno/0E5AD000000000000];Bd0?ooo`3gmoL0
GUiN0?Ogm`080?ooo`040?Ogm`0M7Ad03`l?0>o_k`D0oooo00l0no_k05ADE00000003`l?0>o_k`3o
ool0no_k05ADE0000000Shn?0?ooo`3gmoL07AdM00l?3`3_knl02`3oool00`3kno/0E5AD08n?S`0>
0?ooo`040?Ogm`0M7Ad03`l?0>o_k`H0oooo00@0moOg01dM7@0?3`l0kno_1@3oool0203kno/0Hf=S
0>o_k`3oool0a/K60000000?3`l0kno_8P3oool00`3kno/0Hf=S0>o_k`1Z0?ooo`03020P803oool0
oooo00@0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000
oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYRP3oool00`3kno/0Hf=S0>o_k`0W
0?ooo`030?Ogm`0/;2`0kno_06X0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`04
0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFV;0?ooo`030861P@3Bd]80oooo01<0oooo00<0gmoO01/K6`3Bd]800P3oool0
2@1hN7P00000061PH03oool0no_k05ADE0000000@D510?Ogm`050?ooo`030?Ogm`1NGUh0moOg06X0
oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0
000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFV;0?ooo`03
0::RXP2d];@0oooo01L0oooo00X0a/K60451@@3clo<0E5AD0;Bd]036a/H0H61P0?ooo`1hN7P0];Bd
1@3oool00`3Ogml0LW9b0?ooo`1Z0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`3oool0
103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYR`3oool00`36a/H0Shn?0?ooo`0G0?ooo`0:0<K6aP1PH600oooo0::RXP2d
];@0a/K60451@@3gmoL0N7Qh0;Bd]0D0oooo00<0XZ:R0;Bd]03oool0JP3oool00`0P8200oooo0?oo
o`040?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?oo
o`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJH/0oooo00<0moOg05iNGP3gmoL05`3o
ool02@36a/H0H61P0?ooo`2RXZ80];Bd0?ooo`1hN7P00000061PH0050?ooo`030?_kn`1SHf<0kno_
0080oooo00P0XZ:R0000000?3`l0kno_07QhN00000003`l?0>o_kf00oooo0P0P82060?ooo`03020P
803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P
803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJH`0oooo00<0XZ:R08n?S`3oool05P3oool02P36a/H0@D51
0??cl`1DE5@0];Bd0<K6aP1PH600oooo0::RXP2d];@40?ooo`030<K6aP2?Shl0oooo00<0oooo00H0
Z:RX0=;BdP3oool0oooo07QhN02d];AP0?ooo`03020P803oool0820P00H0oooo00<0820P0?ooo`3o
ool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYYP3oool02@1hN7P00000061PH03oool0no_k05ADE00000003`l?0>o_
k`0:0?ooo`030:RXZ03Bd]80oooo0080oooo00<0QhN70>o_k`3oool0G@3oool0100P8200oooo0?oo
o`0P82060?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml00000
0?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd
0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJKP0oooo00L0a/K600T92@3Bd]80
oooo0<K6aP000000H61P05h0oooo00D0820P0?ooo`3oool0oooo020P80060?ooo`03020P803oool0
oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo7@3oool6020P8080oooo1@0P82060?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc00oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo00000004000000030=oOg`000000
oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc00oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo00000004000000030=oOg`000000
oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooobD0oooo1@0P82060?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobT0oooo00<0820P0?ooo`3oool0103oool0
0`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolY0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`00000010000000
0`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`
l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo:@3oool0
0`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool0
0`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`0608b<S03oool0000008b<S03@d=000000
0P3IfMT03P000000hN7Q07alO01lO7`0_Kfm0000002m_Kd0d=3@06QXJ02<S8`0YjNW07alO02m_Kd0
YjNW0P3oool00`000000hN7Q0?ooo`3a0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA/0
oooo00L0_Kfm0000003`l?00YjNW0>7Qh@3oool000000080oooo00<000000?ooo`0000000P3oool0
1@000000oooo0000003`l?00oooo0080l?3`00D0hN7Q0000003oool0oooo0?3`l03c0?ooo`03020P
803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P
803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJA/0oooo00L0J6QX0=WIf@2m_Kd0oooo0>7Qh@3@d=000000
0080oooo00<000000?ooo`0000000P3oool00`000000oooo000000030=3@d0030;:b/P000000S8b<
0?H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo
00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6`3oool010000000oooo0;:b/P1XJ6P2
0?ooo`060000003oool0oooo0000003oool000000P3oool00`000000oooo06QXJ0020?ooo`0406QX
J0000000l?3`0?3`l?H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6P3oool0202<S8`0
00000?3`l02WYjL0R8R80;fm_@1XJ6P0S8b<0P3oool03P000000d=3@0000003oool0d=3@0000003o
ool0l?3`0;:b/P3@d=00_Kfm0:NWY`2m_Kd00000mP3oool00`0P8200oooo0?ooo`040?ooo`03020P
803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTT0?ooo`030000003oool0oooo0?l0oooo0P3oool00`0P8200oooo0?ooo`040?ooo`03020P
803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OW
i`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k
0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTS0?ooo`030=WIf@000000oooo0?l0oooo0`3oool00`0P8200oooo0?ooo`040?ooo`03020P
803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`030>c/
k03`l?00SXj>00D0X:2P00<0WinO0:2PX02PX:00102PX:005P2@T900k>c/06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY903oool00`2b/[80oooo
0?ooo`3o0?ooo`80oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oO
g`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool00`3/k>`0UYJF0=kNgP0<0=?Cd`0F0=kN
gP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooobT0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oO
g`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0i>CT0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolY0?ooo`03020P803o
ool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTo0?oo
o`030:NWY`1XJ6P0oooo07l0oooo00<0fMWI0000003IfMT0BP3oool00`3IfMT000000=WIf@0G0?oo
o`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?oo
o`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0
gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJCX0oooo00<0d=3@0?ooo`3oool00`3oool00`3IfMT0
hN7Q0?ooo`1o0?ooo`030000003oool0oooo04X0oooo00<000000?ooo`3oool05P3oool00`0P8200
oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200
oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFTM0?ooo`050?3`l03oool0oooo0?ooo`3`l?00603oool00`3`l?00
/[:b0?ooo`040?ooo`030;:b/P3oool0oooo07h0oooo00<000000?ooo`3oool0BP3oool00`000000
oooo0?ooo`0F0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml0
00000?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo00<0_Kfm08b<S03oool0
0P3oool040000000oooo0?ooo`3oool0fMWI0000003QhN40oooo0000002WYjL0_Kfm0000003IfMT0
hN7Q0000003IfMT30?ooo`080=WIf@000000hN7Q0?ooo`000000YjNW0000002WYjL40?ooo`0:0?3`
l0000000oooo0?ooo`3IfMT0YjNW0=3@d02WYjL0fMWI0?ooo`80O7al00<0_Kfm0000002m_Kd00`3o
ool03P3@d=0000000;:b/P2m_Kd0000008b<S03@d=0000000=WIf@000000fMWI0>7Qh@000000fMWI
0`3oool05P3@d=00J6QX08b<S02WYjL0oooo0=WIf@000000fMWI0>7Qh@000000fMWI0?ooo`000000
YjNW0;fm_@1XJ6P0S8b<0:NWY`3oool0fMWI0000003IfMT30?ooo`070=WIf@000000hN7Q0=WIf@00
0000hN7Q0=WIf@020000000H08b<S03@d=0000000=WIf@000000YjNW0;fm_@000000fMWI0>7Qh@00
0000fMWI0=3@d01XJ6P0S8b<0:NWY`3oool0fMWI0000003QhN40fMWI0000003QhN40fMWI0P000000
502<S8`0d=3@0000003IfMT000000:NWY`2m_Kd000000>7Qh@3@d=0000000;:b/P2m_Kd0000008b<
S03@d=0000000=WIf@000000hN7Q1@3oool01P3@d=00J6QX08b<S02WYjL0oooo0:NWY`80/[:b00<0
00000>7Qh@0000000P2m_Kd06@3@d=00fMWI0000003IfMT0oooo0=3@d01XJ6P0S8b<0:NWY`3oool0
O7al0;fm_@2WYjL0oooo07alO02m_Kd0YjNW0?ooo`3IfMT000000>7Qh@3IfMT0YjNW0=3@d02WYjL0
0P3IfMT020000000fMWI0>7Qh@000000fMWI07alO02m_Kd0YjNW0`3oool040000000S8b<0=3@d000
0000fMWI0000003IfMT0hN7Q0000003IfMT0d=3@0:NWY`2m_Kd0d=3@0000003QhN420?ooo`080=3@
d0000000/[:b0;fm_@3IfMT0YjNW0=3@d02WYjL20=WIf@090000003QhN40fMWI0000003QhN40fMWI
0000003QhN4000000080_Kfm00d0d=3@0?ooo`1lO7`0O7al0;fm_@000000_Kfm0000002WYjL0_Kfm
06QXJ02<S8`0YjNW00T0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703oool01`1lO7`00000
0>7Qh@3oool0fMWI0000003@d=000`3oool010000000oooo0?ooo`0000020?ooo`040000003oool0
oooo000000D0oooo00H000000?ooo`3oool000000?ooo`3`l?050?ooo`050;:b/P1XJ6P0_Kfm0?oo
o`1lO7`00`3oool00`1XJ6P0oooo000000020?ooo`030000003oool0oooo0080oooo00X000000?3`
l03oool0l?3`0:NWY`3QhN40oooo0000003oool000000P3oool00`000000oooo0?ooo`020?ooo`04
0000003`l?00oooo0?3`l080oooo00@000000?ooo`3oool000000P3oool01P000000oooo0000003`
l?00oooo0?3`l080oooo00<000000?ooo`3oool00`3oool010000000oooo0?ooo`0000020?ooo`07
0000002WYjL0hN7Q0?ooo`000000oooo000000020?ooo`090000003oool0oooo0000003oool00000
0?3`l03oool0l?3`0080oooo00@000000?ooo`3oool000000P3oool01`000000YjNW0>7Qh@3oool0
00000?ooo`0000000P3oool030000000oooo0000003`l?00oooo0?3`l02WYjL0hN7Q0?ooo`000000
oooo000000H0oooo00@000000?3`l03oool0l?3`0P3oool01@2b/[80J6QX0=3@d03oool000000080
oooo00<0J6QX0?ooo`0000000P3oool030000000l?3`0?ooo`3`l?00oooo0?3`l03QhN4000000?oo
o`3`l?00hN7Q00000080oooo00<000000?ooo`1lO7`00`3oool00`1XJ6P0oooo000000020?ooo`05
0000003oool0l?3`0>7Qh@0000000`3oool01P2WYjL0hN7Q0?ooo`000000oooo00000080oooo00@0
00000?ooo`000000l?3`0P3oool02@000000oooo0?ooo`3oool000000?3`l03oool0l?3`07alO003
0?ooo`0306QXJ03oool000000080oooo00H000000?ooo`3oool000000?ooo`0000020?ooo`0306QX
J03oool000000080oooo00P000000?ooo`000000oooo0000003`l?00oooo0?3`l0T0oooo00<0820P
0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUY703oool01`000000hN7Q0;:b/P3oool0O7al0:NWY`3@d=000`3o
ool010000000oooo0?ooo`0000020?ooo`040000003oool0oooo000000D0oooo00@000000?ooo`3o
ool000001`3oool01@1XJ6P0fMWI0;fm_@3oool0000000<0oooo00<000000?ooo`0000000P3oool0
0`000000oooo0?ooo`020?ooo`030000003oool0oooo0080oooo00D0hN7Q0=3@d0000000oooo0000
00020?ooo`030000003oool0oooo0080oooo00@000000=3@d03@d=00d=3@0P3oool010000000oooo
0?ooo`0000020?ooo`030000003oool0000000<0d=3@0P3oool00`000000oooo0?ooo`030?ooo`04
0000003oool0oooo00000080oooo00L000000?ooo`3QhN40d=3@0000003oool000000080oooo00H0
00000?ooo`3oool000000?ooo`0000030=3@d080oooo00@000000?ooo`3oool000000P3oool01`00
0000oooo0>7Qh@3@d=0000000?ooo`0000000P3oool00`000000oooo000000040?ooo`050>7Qh@3@
d=0000000?ooo`0000001P3oool010000000d=3@0=3@d03@d=020?ooo`050=WIf@000000oooo0?oo
o`0000000P3oool00`000000oooo000000020?ooo`0<0000003@d=00d=3@0=3@d03oool0/[:b0000
002<S8`0oooo0;:b/P000000S8b<0P3oool00`000000oooo000000030?ooo`030000003oool00000
0080oooo00D000000?ooo`2b/[80000008b<S0040?ooo`050>7Qh@3@d=0000000?ooo`0000000P3o
ool00`000000oooo000000030?ooo`050000003oool0oooo0?ooo`0000000`3oool01`000000oooo
0?ooo`3oool000000?ooo`0000000P3oool01P000000oooo0?ooo`000000oooo00000080oooo00<0
00000?ooo`0000000P3oool01@000000oooo0000003oool0000000<0d=3@2@3oool00`0P8200oooo
0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTK0?ooo`0B0>7Qh@000000oooo0=3@d03QhN4000000>7Qh@3IfMT0l?3`
0?ooo`3IfMT000000?ooo`3@d=00000008b<S03IfMT000000P3oool00`000000oooo0?ooo`020?oo
o`070=WIf@000000oooo0=3@d0000000S8b<0=WIf@050?ooo`050000003oool0fMWI0>7Qh@1XJ6P0
0`3oool00`1XJ6P0oooo000000020?ooo`030000003oool0oooo0080oooo00D0O7al0?ooo`3`l?00
_Kfm06QXJ0020?ooo`030000003oool000000080oooo00<000000?ooo`3oool00P3oool0101XJ6P0
oooo0?ooo`1XJ6P20?ooo`090000003oool0oooo0000003oool0d=3@0000002<S8`0J6QX0080oooo
00@0J6QX0?ooo`3oool000001@3oool010000000oooo0?ooo`0000020?ooo`0:0000001XJ6P0oooo
0?ooo`000000d=3@0000002<S8`0fMWI00000080oooo00<000000?ooo`1XJ6P00P3oool0101XJ6P0
oooo0?ooo`0000020?ooo`050000003oool0oooo0000001XJ6P00P3oool030000000d=3@0000002<
S8`0fMWI0000003oool0O7al0?ooo`3`l?00_Kfm06QXJ080oooo00<000000?ooo`0000001P3oool0
101XJ6P0oooo0?ooo`1XJ6P20?ooo`0:06QXJ02<S8`0hN7Q0?ooo`000000oooo0?3`l0000000oooo
00000080oooo00H0J6QX0?ooo`3oool0J6QX0?ooo`0000020?3`l0090?ooo`000000l?3`0?3`l03o
ool0fMWI0000003oool0J6QX00<0oooo00<0J6QX0?ooo`0000000P3oool00`000000oooo00000002
0?3`l0<0oooo00H0J6QX0?ooo`3oool000000?ooo`0000020?ooo`030000003oool0O7al00<0oooo
00T000000?ooo`3oool0oooo07alO03oool0l?3`0;fm_@1XJ6P00`3oool00`1XJ6P0oooo00000002
0?ooo`0;0000003oool0oooo0000003oool000000?ooo`3`l?0000000?ooo`0000000P3oool01@00
0000d=3@0000002<S8`0J6QX0080oooo00<0J6QX0?ooo`3oool01`3oool00`0P8200oooo0?ooo`04
0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFTK0?ooo`060:NWY`1lO7`0oooo0?ooo`1lO7`000000P3oool01`2m_Kd0oooo0>7Q
h@2m_Kd0oooo0?3`l01lO7`00P3oool010000000d=3@06QXJ01lO7`40?ooo`050>7Qh@2m_Kd0oooo
0?3`l01lO7`01P3oool03`2WYjL000000?3`l03@d=00J6QX0=3@d02b/[80d=3@0:NWY`3IfMT0d=3@
0000003oool0d=3@000000040?ooo`0;0=WIf@2b/[80YjNW08b<S03@d=00_Kfm06QXJ02<S8`0d=3@
0000003@d=000P1lO7`40?ooo`080?3`l02b/[80d=3@0;fm_@3oool0d=3@0000003@d=0207alO00=
0?ooo`3`l?00O7al0?ooo`3`l?00/[:b0=3@d02m_Kd0oooo0;fm_@000000S8b<000000020?ooo`0>
0;fm_@000000d=3@07alO0000000d=3@0000002WYjL0d=3@0;fm_@1XJ6P0S8b<0?3`l01lO7`20?oo
o`0V0000003@d=00J6QX07alO03oool0l?3`0;:b/P3@d=00_Kfm0?ooo`2m_Kd000000=3@d01lO7`0
00000=3@d0000000YjNW0=3@d02m_Kd0J6QX08b<S03`l?00O7al0?ooo`3QhN40_Kfm0?ooo`3IfMT0
/[:b0:NWY`2<S8`0d=3@0;fm_@1XJ6P0S8b<0?ooo`0000060?ooo`0=0?3`l02b/[80d=3@0;fm_@3o
ool0d=3@0000003QhN4000000;fm_@000000_Kfm06QXJ0020;fm_@0K0000002<S8`000000?3`l02b
/[80d=3@0;fm_@3oool0YjNW0;fm_@000000oooo0:NWY`2m_Kd000000?ooo`3QhN40_Kfm0?ooo`3I
fMT0/[:b0=3@d02WYjL0fMWI0=3@d0000000d=3@0080O7al00@0oooo0:NWY`2m_Kd000000`3oool0
1`3@d=00_Kfm06QXJ02<S8`0d=3@0000003@d=000P1lO7`01P3oool0l?3`0;:b/P3@d=00_Kfm0000
00<0oooo0280fMWI0;:b/P2WYjL0S8b<0=WIf@2b/[80d=3@0:NWY`3IfMT0_Kfm0000003@d=00O7al
0000003@d=0000000:NWY`3IfMT000000;fm_@1XJ6P0_Kfm0=3@d0000000oooo0=3@d0000000l?3`
07alO03oool0l?3`0;:b/P3@d=00_Kfm2@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?oo
o`060000002m_Kd0oooo0?ooo`2<S8`0O7al0P3oool00`2WYjL0oooo0?ooo`060?ooo`030000003o
ool0oooo04h0oooo00<0l?3`0?ooo`3oool0303oool00`000000oooo0?ooo`080?ooo`030?3`l03o
ool0oooo01L0oooo00<000000?ooo`3oool03P3oool00`3`l?00oooo0?ooo`0d0?ooo`030000003o
ool0oooo00/0oooo00<0l?3`0?ooo`3oool01@3oool00`3`l?00oooo0?ooo`0F0?ooo`03020P803o
ool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@
d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJAX0oooo00L0_Kfm0000002WYjL0oooo0=WIf@000000J6QX0080l?3`
0P2<S8`00`3oool000000?ooo`030?ooo`030>7Qh@000000oooo00L0oooo00<000000?ooo`3oool0
D`3oool00`3QhN4000000?ooo`0H0?ooo`030000003oool0oooo00L0oooo00<0fMWI0000003oool0
9@3oool00`000000oooo0?ooo`0M0?ooo`030>7Qh@000000oooo02d0oooo00<0820P0?ooo`3oool0
103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY;03oool00`2b/[80oooo0?ooo`1M0?ooo`030;:b/P3oool0oooo0280oooo00<0
/[:b0?ooo`3oool0A@3oool00`2WYjL0oooo0?ooo`0/0?ooo`03020P803oool0oooo00@0oooo00<0
820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO
0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Z
j^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobT0oooo00<0820P0?oo
o`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolY0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0
103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFUo0?ooo`030=WIf@000000fMWI00P0oooo00<0
YjNW06QXJ03oool0C`3oool00`3IfMT000000=WIf@190?ooo`03020P803oool0oooo00@0oooo00<0
820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJH00oooo00<000000?ooo`3oool0203oool00`3IfMT0hN7Q0?ooo`1?0?ooo`030000003oool0
oooo04P0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000
oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYP03oool00`000000oooo0?ooo`090?oo
o`030;:b/P3oool0oooo04h0oooo00<000000?ooo`3oool0B03oool00`0P8200oooo0?ooo`040?oo
o`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAX0oooo00<0
_Kfm06QXJ03`l?000P3@d=00;P3oool0d=3@0000003@d=0000000=WIf@000000hN7Q0?ooo`000000
d=3@0?ooo`3IfMT0YjNW06QXJ03IfMT0oooo0000002WYjL0d=3@0?ooo`2WYjL0/[:b0>7Qh@1XJ6P0
oooo0000003IfMT000000=WIf@000000hN7Q0?ooo`000000d=3@0?ooo`1lO7`0S8b<0?ooo`000000
YjNW0?3`l0000000fMWI0000003QhN450?ooo`0=0=WIf@000000fMWI0>7Qh@000000fMWI0000002<
S8`0d=3@0000001lO7`0_Kfm0:NWY`050?ooo`0D0000003@d=00_Kfm0=WIf@3@d=00J6QX08b<S02W
YjL0oooo0=3@d01XJ6P0S8b<0:NWY`3oool0fMWI0000003IfMT0hN7Q0000003IfMT30?ooo`060000
002<S8`0d=3@0000003IfMT000000P2m_Kd02P3@d=00oooo0000002WYjL0_Kfm0000003QhN40oooo
0?3`l00000060?ooo`0Y0=3@d02WYjL0_Kfm0=3@d0000000_Kfm06QXJ02<S8`0YjNW0?ooo`1lO7`0
_Kfm0:NWY`3oool0d=3@0000002b/[80_Kfm0=WIf@000000fMWI0?ooo`3IfMT000000>7Qh@3oool0
00000=3@d02m_Kd0fMWI0=3@d01XJ6P0S8b<0:NWY`3oool0d=3@0:NWY`2m_Kd0d=3@0000003QhN40
0`3oool01`000000S8b<0=3@d0000000O7al0;fm_@2WYjL01@3oool01@000000S8b<0=3@d0000000
fMWI00<0oooo00/0O7al0;fm_@2WYjL0oooo0=WIf@2WYjL0d=3@0:NWY`3IfMT0oooo000000020;fm
_@060=3@d03IfMT000000=WIf@3QhN4000000P3IfMT01@000000hN7Q07alO02m_Kd0YjNW0080oooo
01/000000:NWY`2m_Kd000000>7Qh@3@d=0000000;:b/P2m_Kd0000008b<S03@d=0000000=WIf@00
0000YjNW0;fm_@1XJ6P0S8b<0:NWY`3oool0d=3@0:NWY`2m_Kd0d=3@0000003QhN40103oool0403@
d=0000000;:b/P2m_Kd0000008b<S03@d=0000000=WIf@000000hN7Q0=3@d0000000/[:b0;fm_@3o
ool207alO0030;fm_@000000oooo00D0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000
0004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool01@3/k>`0UYJF0?oo
o`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooo`0>0;fm_@3oool0_Kfm
0000003`l?00oooo0000003QhN40J6QX0>7Qh@000000_Kfm0?3`l01XJ6P20>7Qh@060?ooo`000000
oooo0=WIf@3QhN4000000P3oool03P3QhN40l?3`0000003oool000000>7Qh@1lO7`0d=3@069RHP3Q
hN4000000;fm_@3`l?00J6QX0P3QhN402`2m_Kd0YjNW0>7Qh@000000oooo0?3`l01XJ6P0hN7Q0000
002m_Kd0l?3`00D0oooo00`000000?ooo`3oool000000?ooo`2WYjL0hN7Q0?ooo`000000l?3`0>7Q
h@0000050?ooo`0=0000003oool0oooo06QXJ0000000l?3`0?ooo`3`l?00oooo0000003`l?00oooo
0?3`l0020?ooo`040000003oool0oooo000000@0oooo00H0YjNW0>7Qh@3oool000000?ooo`000002
0?ooo`0306QXJ03oool000000080oooo00H000000?ooo`3oool0/[:b06QXJ02m_Kd50?ooo`040000
003`l?00oooo0?ooo`80000000h0l?3`0?ooo`3`l?00oooo0?3`l03QhN4000000?ooo`000000l?3`
0?ooo`3`l?00oooo000000<0oooo00@000000?ooo`3oool000000P3oool0201XJ6P000000?3`l03o
ool0l?3`0?ooo`000000l?3`0P3oool00`000000oooo0?ooo`020?ooo`070:NWY`3QhN40oooo0000
003`l?00hN7Q000000050?ooo`040:NWY`3QhN40oooo000000@0oooo00D0l?3`0>7Qh@000000oooo
07alO0030?ooo`0306QXJ03oool000000080oooo00<0J6QX0?ooo`0000000P3oool020000000oooo
0?ooo`000000oooo0?3`l03QhN4000000P3oool05`000000oooo0?ooo`000000oooo0000003`l?00
oooo0?3`l02WYjL0hN7Q0?ooo`000000oooo0000003oool000000?3`l03oool0l?3`0?ooo`000000
l?3`0080oooo00<000000?ooo`3oool00`3oool04@000000l?3`0?ooo`3`l?00YjNW0>7Qh@3oool0
00000?ooo`000000oooo0000003`l?00oooo0?3`l03oool000000080oooo00<000000?ooo`3oool0
103oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0
103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3@d=00
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJA/0oooo00X0d=3@0?ooo`2<S8`0S8b<0=3@d03oool0S8b<0;fm_@2b/[80
fMWI0P2WYjL00`3oool0_Kfm0:NWY`020?ooo`0706QXJ03QhN40oooo07alO0000000_Kfm0=WIf@02
0?ooo`080000002m_Kd0/[:b0000003IfMT0YjNW0;:b/P3IfMT20:NWY`030?ooo`2m_Kd0YjNW0080
oooo00<000000?ooo`1XJ6P00P3oool0102b/[80fMWI0:NWY`2WYjL60?ooo`040000003oool0oooo
00000080oooo00H0hN7Q0=3@d0000000/[:b0000002<S8`50?ooo`030000003oool0oooo00800000
0`3@d=000`3oool000000=3@d0020=3@d080oooo00@000000?ooo`3oool000001@3oool01@3QhN40
d=3@0000003oool000000080oooo00<000000?ooo`0000000P3oool01P000000oooo0?ooo`1XJ6P0
fMWI0;fm_@D0oooo00@000000?ooo`3oool0oooo0P0000030=3@d0060?ooo`2b/[80000008b<S03o
ool00000103oool01@000000oooo0?ooo`3oool000000080oooo00<000000?ooo`3oool00P000003
0=3@d0030?ooo`000000oooo0080oooo00<000000?ooo`3oool00`3oool01P3QhN40d=3@0000002b
/[80000008b<S0H0oooo00<0hN7Q0=3@d0000000103oool01@2b/[80000008b<S03oool0000000<0
oooo00<000000?ooo`0000000P3oool00`000000oooo000000020?ooo`080000003oool0oooo0000
003oool0/[:b0000002<S8`20?ooo`060000003oool0oooo0000003oool00000103oool01`3QhN40
d=3@0000003oool000000?ooo`0000000`3@d=000`3oool000000?ooo`020?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool00P3oool01`3QhN40d=3@0000003oool000000?ooo`000000
103oool010000000oooo0?ooo`0000060?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00D0k>c/09JFUP3o
ool0moOg0>g]k@060?ooo`0J0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6`3oool20>7Qh@0`07alO02b
/[80l?3`0=WIf@3@d=00O7al0?3`l01lO7`0l?3`0000003oool0YjNW0000002WYjL0oooo0;fm_@00
0000oooo07alO02m_Kd0/[:b0=3@d01lO7`0d=3@0;fm_@000000l?3`06QXJ02m_Kd0J6QX0?3`l01l
O7`0l?3`0000003oool0YjNW0000002WYjL0hN7Q06QXJ03IfMT0_Kfm0>7Qh@3oool0l?3`07alO03`
l?0000001P3oool01P000000oooo0?ooo`000000oooo06QXJ080oooo0P0000020?3`l0D0oooo00D0
00000?ooo`3oool0000006QXJ0020?ooo`0306QXJ03oool0J6QX0080oooo00@0J6QX0?ooo`3oool0
00000P3oool00`000000oooo0?ooo`020?ooo`0>06QXJ03oool0oooo0000003oool000000?ooo`3`
l?0000000=3@d0000000S8b<0=WIf@0000020?ooo`040000003oool0fMWI0>7Qh@@0oooo00H0O7al
0?ooo`3oool0oooo0000001XJ6P20?ooo`0306QXJ03oool000000080l?3`00L0oooo07alO03oool0
l?3`0;fm_@3oool000000080oooo00D0fMWI0000003oool0oooo000000020?ooo`070000001XJ6P0
oooo0?ooo`1XJ6P0oooo07alO0030?ooo`030000003oool0oooo0080oooo00<0J6QX0?ooo`3oool0
0P0000020?3`l0D0oooo00@0J6QX0?ooo`3oool00000103oool01@000000l?3`0?3`l03oool0J6QX
00<0oooo00P0J6QX0?ooo`000000oooo0?3`l0000000oooo00000080oooo00H000000?ooo`3IfMT0
00000?ooo`0000020?3`l00<0?ooo`3@d=00000008b<S03IfMT000000?ooo`1lO7`0oooo0?3`l02m
_Kd0J6QX0P3oool01@000000d=3@0000002<S8`0J6QX0080oooo00<0J6QX0?ooo`1lO7`00`3oool0
0`000000oooo0?ooo`030?ooo`0507alO03oool0l?3`0;fm_@1XJ6P00P3oool02P000000oooo0000
003oool0O7al0?ooo`3`l?00_Kfm0?ooo`0000020?ooo`030000003oool0oooo00@0oooo00<0820P
0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTL0?ooo`0?0=3@d0000000d=3@0?ooo`3IfMT0hN7Q0000003oool0fMWI0;:b/P000000hN7Q
0?ooo`1XJ6P0l?3`0080oooo01D000000=3@d0000000oooo0;:b/P3@d=0000000=WIf@2b/[80J6QX
0;fm_@000000hN7Q0000003oool0fMWI0;:b/P000000hN7Q0?ooo`1XJ6P00P3`l?002P2b/[80S8b<
0?ooo`2m_Kd000000?3`l03IfMT0/[:b0000003QhN450?ooo`0<0000003@d=00J6QX07alO03oool0
d=3@0;fm_@1XJ6P0S8b<0:NWY`2m_Kd000001@3oool04@000000d=3@0:NWY`3@d=00l?3`0;:b/P3@
d=00_Kfm0?ooo`3`l?00/[:b0=3@d02m_Kd0oooo0=3@d0000000d=3@0080O7al103oool02`3@d=00
_Kfm06QXJ02<S8`0fMWI0000002m_Kd0J6QX0;fm_@3`l?00O7al0080oooo00P000000?ooo`2WYjL0
00000?3`l03@d=00J6QX0?3`l0<0oooo01P0l?3`0;:b/P3@d=00_Kfm0000003`l?00/[:b0=3@d02m
_Kd0oooo0:NWY`2m_Kd000000?ooo`3IfMT0/[:b0:NWY`2<S8`0_Kfm0000002<S8`000000>7Qh@2m
_Kd20?ooo`0>0000003@d=00YjNW0=3@d03`l?00/[:b0=3@d02m_Kd0oooo0?3`l02b/[80d=3@0;fm
_@0000040?ooo`070=3@d02m_Kd0J6QX08b<S02WYjL0_Kfm000000050?ooo`040=3@d02m_Kd0J6QX
08b<S0@0oooo00P0YjNW0;fm_@000000oooo0=WIf@2b/[80d=3@0:NWY`80fMWI02`000000;fm_@1X
J6P0_Kfm0?ooo`000000d=3@06QXJ01lO7`0oooo0>7Qh@2m_Kd0oooo0:NWY`2m_Kd000000?ooo`3`
l?00O7al0?ooo`3QhN40_Kfm0?ooo`3IfMT0/[:b0:NWY`2<S8`0d=3@0;fm_@1XJ6P0S8b<0?3`l01l
O7`0oooo0?3`l02b/[80d=3@0;fm_@3oool0l?3`0;:b/P3@d=00_Kfm000000D0oooo01@0fMWI0;:b
/P2WYjL0S8b<0=3@d02m_Kd0J6QX08b<S03oool000000?ooo`3IfMT0/[:b0:NWY`2<S8`0d=3@0000
003oool0d=3@000000H0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]
00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`030=3@d0000000hN7Q0080oooo00<0
d=3@0000003oool02@3oool00`000000l?3`0?ooo`0S0?ooo`030000003oool0oooo00h0oooo00<0
00000?ooo`3oool06@3oool00`3`l?00oooo0?ooo`050?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool04`3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo0200oooo00<0
l?3`0?ooo`3oool00P3oool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo00`0oooo00<0
00000?ooo`3oool03P3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0
gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
1P2EUID06@3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJA/0oooo00<0hN7Q08b<S00000000`3oool0103QhN40
00000:NWY`3`l?080?ooo`0306QXJ02b/[80oooo01D0oooo00<000000?ooo`3oool02P3oool00`3Q
hN4000000?ooo`0>0?ooo`030>7Qh@000000oooo0240oooo00<0fMWI0000003oool0303oool00`3Q
hN4000000?ooo`0A0?ooo`040000003oool0hN7Q000000/0oooo00<0hN7Q0000003oool09@3oool0
0`3QhN4000000?ooo`040?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool04P3oool0
0`3QhN4000000?ooo`0<0?ooo`030=WIf@000000oooo00l0oooo00<0820P0?ooo`3oool0103oool0
0`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool0
1@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFUC0?ooo`03
0;:b/P3oool0oooo00h0oooo00<0/[:b0?ooo`3oool08@3oool00`2b/[80oooo0?ooo`0<0?ooo`03
0:NWY`3oool0oooo01<0oooo00<0/[:b0?ooo`3oool02P3oool00`2WYjL0oooo0?ooo`0U0?ooo`03
0;:b/P3oool0oooo02<0oooo00<0YjNW0?ooo`3oool0303oool00`2b/[80oooo0?ooo`0>0?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03
020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo9P3oool4020P80H0oooo00<0820P0?ooo`000000
100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo:@3o
ool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103o
ool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM
0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobD0oooo1@0P82060?ooo`03020P803oool0
000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
oc00oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo
0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo<03oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0
oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gM
g@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03020P803oool0000000@00000
00<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0
820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo<03oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03020P803oool0000000@0000000<0gmoO0000
003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobD0oooo1@0P82060?ooo`03
020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFUT0?ooo`040>WYj@1lO7`0CDe=0=3@d<00oooo00<0820P0?ooo`3oool0103oool00`0P8200
oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
I@3oool00`37alL000000?ooo`300?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`000000
100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJFD0oooo00<0
jNWY0000003YjNT0`03oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0
gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTJ0?ooo`0308b<S01lO7`0
fMWI00<0oooo01X0S8b<0>WYj@3oool0_Kfm04e=C@3@d=00VYZJ06QXJ03YjNT0_Kfm0000003`l?00
J6QX0;:b/P3oool0CDe=09ZJVP3YjNT0oooo0:NWY`000000YjNW0=3@d03oool0J6QX0;:b/P<0oooo
0100S8b<06QXJ03QhN40S8b<04e=C@2m_Kd0oooo0000001lO7`0fMWI00000037alL0oooo0000002W
YjL0l?3`103oool05P2b/[80YjNW0;:b/P1lO7`0YjNW0?ooo`37alL0YjNW07alO02WYjL0oooo0;fm
_@000000l?3`06QXJ02b/[80oooo04e=C@2JVYX0jNWY0?ooo`1=CDd208b<S00W0:NWY`3oool0oooo
0;fm_@1=CDd0d=3@09ZJVP1XJ6P0jNWY04e=C@2<S8`0d=3@0?ooo`000000O7al0=WIf@000000alO7
0?ooo`1=CDd0S8b<0=3@d03oool0S8b<06QXJ03QhN40alO70:NWY`1lO7`0YjNW0?ooo`1XJ6P0VYZJ
0?ooo`2<S8`0J6QX0=WIf@3`l?00CDe=0080/[:bUP3oool00`0P8200oooo0?ooo`040?ooo`03020P
803oool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTK0?ooo`0@0;fm_@3oool0oooo0?ooo`3@d=0000000<O7a`3oool0d=3@0000003IfMT0_Kfm04e=
C@3IfMT0l?3`00000080alO700h000000?3`l02b/[80CDe=0>7Qh@3`l?0000000:NWY`3oool0l?3`
0>7Qh@2WYjL000000?3`l080oooo0100_Kfm04e=C@3@d=0000000;:b/P3`l?00jNWY04e=C@2b/[80
_Kfm0000002m_Kd0fMWI08b<S02JVYX0_Kfm0`3oool00`2WYjL0CDe=0?ooo`020?ooo`0908b<S03o
ool000000=WIf@3`l?0000000;:b/P3`l?0000000080alO701`000000?3`l02b/[80CDe=0>7Qh@3`
l?00VYZJ0:NWY`3oool0CDe=0<O7a`3oool0d=3@0000003IfMT0_Kfm04e=C@3IfMT0S8b<09ZJVP3Q
hN40oooo04e=C@2b/[80_Kfm0000002m_Kd0fMWI0P2JVYX0403QhN40oooo0;fm_@1=CDd0d=3@0000
003IfMT0l?3`0000002b/[80alO70000003YjNT0alO70000003IfMT20?3`l0030>WYj@000000jNWY
09D0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo
00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT
0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6`3oool0102m_Kd0oooo0?ooo`3`l?0204e=
C@0J0;:b/P3oool0oooo0000002m_Kd0d=3@0000003QhN40oooo09ZJVP1=CDd0l?3`0000002WYjL0
jNWY0000003IfMT0oooo04e=C@1lO7`0/[:b0>7Qh@3oool0hN7Q0000003@d=020?ooo`050?3`l000
0000d=3@06QXJ02b/[800P3oool02037alL0CDe=0?ooo`1XJ6P0VYZJ0?ooo`37alL00000103oool0
0`000000O7al0?ooo`020?ooo`80l?3`01@000000;fm_@3oool0YjNW04e=C@3oool0VYZJ04e=C@3`
l?0000000:NWY`3YjNT000000=WIf@3oool0alO704e=C@3oool0YjNW08b<S080oooo00@000000;fm
_@3@d=0000000P3QhN406`000000l?3`0?ooo`37alL0CDe=0?ooo`1XJ6P0VYZJ0?ooo`3QhN400000
0?3`l03oool0l?3`0000003@d=0000000;fm_@3oool0YjNW04e=C@3`l?0000000;:b/P3`l?000000
0<O7a`020?ooo`0309ZJVP1XJ6P0oooo09D0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo
00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6`3o
ool08@2m_Kd0oooo0?ooo`2WYjL000000=WIf@2b/[80oooo0=WIf@2JVYX0CDe=0?ooo`1=CDd0YjNW
0;:b/P3@d=00S8b<0;fm_@2WYjL0VYZJ0;fm_@2<S8`0VYZJ0?ooo`3@d=00000009ZJVP2<S8`0/[:b
0=3@d01=CDd0YjNW0=WIf@020=3@d080VYZJ00d0fMWI06QXJ03oool0fMWI0?ooo`1XJ6P0alO70=3@
d0000000l?3`0?ooo`000000fMWI00<0oooo00<0000007alO03oool0103oool0:P2b/[80J6QX0?oo
o`3IfMT000000;:b/P3@d=00S8b<0;fm_@2WYjL0VYZJ0;fm_@2<S8`0VYZJ0?3`l02b/[8000000>WY
j@2b/[8000000?ooo`3IfMT0VYZJ04e=C@3oool0CDe=0:NWY`2m_Kd0000008b<S037alL0oooo06QX
J037alL0d=3@0000003`l?00_Kfm0000002<S8`0alO70=3@d080VYZJ00/0/[:b06QXJ03oool0fMWI
0000003IfMT0S8b<0:NWY`37alL0VYZJ07alO0020?ooo`040000002m_Kd0hN7Q0?3`l9@0oooo00<0
820P0?ooo`3oool0103oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0
820P0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY6`3oool01`3QhN40fMWI0=WIf@000000VYZJ0?ooo`37alL0
0P3YjNT03P2b/[8000000>7Qh@2m_Kd000000=WIf@2m_Kd000000>WYj@2<S8`000000?3`l02JVYX0
O7al0P3oool0403QhN40VYZJ0=3@d02<S8`0hN7Q07alO02<S8`0alO70000003`l?00VYZJ04e=C@3o
ool0alO70:NWY`0000020?3`l00509ZJVP2WYjL0CDe=0;:b/P3oool00P2JVYX30?ooo`0306QXJ000
0000l?3`00D0oooo01d0/[:b0;fm_@37alL0alO70=WIf@2m_Kd000000>WYj@2<S8`000000?3`l02J
VYX0O7al0?ooo`3`l?00000007alO01=CDd0O7al0?ooo`3YjNT0/[:b0000003QhN40_Kfm0000003o
ool0_Kfm06QXJ0020?ooo`0?0?3`l02JVYX0YjNW04e=C@2b/[80oooo0;fm_@1XJ6P0oooo0?3`l02J
VYX0CDe=0?ooo`2b/[80_Kfm0080alO700H0hN7Q0:NWY`1=CDd0hN7Q06QXJ00000020?ooo`80alO7
00<0O7al0>7Qh@3oool0T`3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@00000
00<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`070;fm_@1X
J6P000000?3`l03oool0jNWY0<O7a`0X0?ooo`030=3@d0000000l?3`0080oooo00<0alO700000037
alL00`3oool00`3IfMT0oooo0?ooo`0>0?ooo`0308b<S02JVYX0oooo00T0oooo00<0l?3`0=WIf@3o
ool01`3oool00`3`l?00fMWI0?ooo`2X0?ooo`03020P803oool0oooo00@0oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00<0k>c/09JFUP3G
emL0303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY703oool00`1lO7`000000;fm_@030?ooo`030;:b/P3oool0oooo01X0
oooo00<0fMWI0?3`l03oool02P3oool00`000000alO70?ooo`020?ooo`0309ZJVP000000l?3`0080
oooo00<0J6QX0?3`l03oool0:@3oool00`3IfMT0l?3`0?ooo`2S0?ooo`03020P803oool0oooo00@0
oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080
oooo00<0k>c/0?gmo@2CTi<01@2h^;P00`2e]KD0^;Rh0;Rh^0040;Rh^00F09>CT`3/k>`0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooo`03
0;:b/P000000J6QX00<0oooo00<0alO704e=C@2m_Kd06`3oool00`000000fMWI0?ooo`090?ooo`03
0>7Qh@1=CDd0S8b<00@0oooo00<0alO707alO02b/[800P2WYjL00`3YjNT0oooo0?ooo`0X0?ooo`03
0000003IfMT0oooo0:<0oooo00<0820P0?ooo`3oool0103oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolY0?ooo`03
020P803oool0oooo00@0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo9P3oool4020P80H0oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo:@3oool00`0P8200oooo0?ooo`040?ooo`03020P803oool0000000@0000000<0gmoO0000003o
ool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobD0oooo1@0P82020?ooo`D0
820P00<0oooo0000000000000`0000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo<P3oool5000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolb0?ooo`D0000000<0gmoO0000003oool0103oool00`0P8200
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?oooc80oooo1@0000000`3Ogml000000?ooo`040?ooo`03
020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<P3oool5000000030=oOg`000000oooo00@0
oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolb0?ooo`D0000000<0gmoO0000003o
ool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc80oooo1@0000000`3Ogml0
00000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<P3oool500000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolb0?ooo`D0
000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc80
oooo1@0000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo<P3oool5000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolb0?ooo`D0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc80oooo1@0000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo;03oool5020P80030?ooo`000000000000<0000000<0gmoO0000003o
ool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo<03oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03020P803oool0000000@0000000<0gmoO0000003o
ool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc00oooo00<0820P0?ooo`00
0000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAT0
oooo00d0hN7Q0:NWY`00000000000:NWY`3IfMT0oooo08b<S01=CDd0l?3`0?ooo`3QhN40YjNW0080
000000<0YjNW0=WIf@3oool01`3oool01P000000YjNW0=WIf@3oool0jNWY09ZJVP80000000<0O7al
0=WIf@3oool00`3oool03P3YjNT0S8b<000000000000S8b<09ZJVP1=CDd0VYZJ0?3`l03oool0/[:b
0000001=CDd0/[:b0P3oool0302b/[80000000000037alL0oooo0<O7a`1lO7`0000004e=C@2JVYX0
jNWY0;:b/P80000000<0/[:b0?ooo`3oool01@3oool01@3IfMT0VYZJ07alO02WYjL0/[:b0080YjNW
00<0jNWY0?ooo`3oool00`3oool00`1XJ6P0l?3`0?ooo`020?ooo`0:0<O7a`1lO7`0000004e=C@2J
VYX0jNWY0;:b/P000000CDe=0;:b/PD0oooo00D0J6QX0?3`l03oool0oooo0;:b/P02000000080<O7
a`3oool0alO707alO0000000CDe=09ZJVP3YjNT20?ooo`060>WYj@1XJ6P0oooo0?ooo`3@d=00YjNW
203oool02037alL0S8b<0:NWY`2WYjL0S8b<0<O7a`3oool0/[:b0P0000003P2b/[80oooo0?ooo`3o
ool0/[:b0000002WYjL0jNWY0?ooo`2JVYX0jNWY0?ooo`3QhN40YjNW0P0000000`2WYjL0jNWY0?oo
o`0204e=C@0@0=WIf@2b/[80CDe=0=3@d03oool0000006QXJ02m_Kd0l?3`0?ooo`000000VYZJ0;fm
_@000000YjNW0?3`l080oooo00d0alO7000000000000VYZJ0>WYj@1XJ6P000000?3`l03oool0YjNW
0000003YjNT0oooo0080000000<0_Kfm0?ooo`3oool00P1=CDd04`3IfMT0/[:b04e=C@3@d=00oooo
0000001XJ6P0_Kfm0?3`l03oool0l?3`04e=C@2JVYX0jNWY0;:b/P000000O7al0>7Qh@3oool00P1=
CDd0103IfMT0/[:b04e=C@3@d=0[0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0
103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooo`8000000`3oool00`1=CDd00000
0>WYj@030?ooo`8000002P3oool00`2JVYX0l?3`0?ooo`020?ooo`040<O7a`00000000000?3`l0L0
oooo0P0000001@3IfMT0oooo09ZJVP000000VYZJ0080oooo00<0000007alO03oool00`3oool03@00
0000O7al0?ooo`2b/[80000006QXJ03@d=00oooo0?3`l02WYjL0oooo0000001lO7`01`3oool00`2W
YjL0000008b<S0040?ooo`030:NWY`000000alO700<0oooo00<0alO70000002b/[800P3oool02P2b
/[80000006QXJ03@d=00oooo0?3`l02WYjL0oooo0000001lO7`50?ooo`030<O7a`000000/[:b00<0
oooo00X0000007alO03oool0/[:b0000001XJ6P0d=3@0?ooo`3`l?00YjNW0P3oool01`2b/[800000
0<O7a`3oool0S8b<0000003`l?001P3oool00`2b/[8000000=3@d0020?ooo`060=3@d0000000/[:b
0?ooo`000000O7al1@3oool01P2WYjL0l?3`0?ooo`3oool0CDe=07alO0<0oooo00<0CDe=0000003Y
jNT00P3oool03P1=CDd0J6QX0;fm_@2b/[8000000;fm_@3@d=00J6QX0000003`l?00YjNW0?ooo`2<
S8`000000P3YjNT01P2b/[80S8b<0?ooo`3oool0000004e=C@80l?3`01T0YjNW0;^k^`000000alO7
0?ooo`3YjNT000000:NWY`3oool0YjNW0000003QhN40_Kfm0?ooo`1=CDd0J6QX0;fm_@2b/[800000
0;fm_@3@d=00J6QX0000003`l?00YjNW0080oooo00l0CDe=06QXJ037alL0000006QXJ03oool0YjNW
0?3`l01=CDd0J6QX0;fm_@2b/[8000000;fm_@3@d=00:P3oool00`0P8200oooo0000000400000003
0=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6`3oool2000000<0
oooo00<0l?3`0>WYj@3oool00`3oool2000000X0oooo00<0fMWI0<O7a`3oool00P3oool00`1lO7`0
00000:NWY`080?ooo`80000000<0fMWI0?ooo`3@d=000P0000020?ooo`030000001lO7`0oooo00<0
oooo00H0000007alO03oool0000004e=C@3`l?050?ooo`030000001lO7`0oooo00D0oooo00@0d=3@
000000000000l?3`1@3oool00`1=CDd000000=WIf@020?ooo`0804e=C@000000VYZJ0?3`l03oool0
000004e=C@3`l?050?ooo`030000001lO7`0oooo00@0oooo00@0CDe=0000002JVYX0l?3`0P3oool0
1P000000O7al0?ooo`000000CDe=0?3`l0D0oooo00P0l?3`000000000000O7al0?3`l0000000CDe=
0;:b/PH0oooo00<0CDe=0000003oool00`3oool01@000000CDe=0?ooo`000000O7al00D0oooo00@0
hN7Q0=3@d03oool0oooo0P0000001P2m_Kd0oooo0?ooo`2JVYX000000<O7a`80oooo00X0YjNW0000
003oool0alO70000002JVYX0jNWY0;fm_@000000hN7Q0P3oool02P2m_Kd000000;fm_@3oool0jNWY
0000003@d=00oooo0000001lO7`30?ooo`0G0?3`l0000000CDe=0?3`l03oool0VYZJ0000003QhN40
jNWY0000002JVYX0l?3`0?ooo`2WYjL000000?ooo`37alL0000009ZJVP3YjNT0_Kfm0000003QhN40
0`3oool01@2b/[8000000=WIf@1=CDd0O7al00<0oooo00L0YjNW0000003oool0alO70000002JVYX0
jNWY02X0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJA/0oooo0P0000090?ooo`8000002`3oool407alO080000000<0jNWY
0?ooo`3oool01P3oool2000000050=WIf@3oool0d=3@0000001=CDd00P3oool00`000000O7al0?oo
o`030?ooo`050000001lO7`0oooo0000001=CDd01@2b/[800`3oool0000007alO0060?ooo`0307al
O0000000S8b<00H0oooo00`0VYZJ0000002<S8`0oooo0=WIf@000000VYZJ0>WYj@2b/[80oooo0000
001=CDd50;:b/P030?ooo`000000O7al00@0oooo00D0fMWI0000002JVYX0jNWY0;:b/P020?ooo`05
0000001lO7`0oooo0000001=CDd01@2b/[802P3oool0/[:b0000002b/[80fMWI06QXJ0000000alO7
0:NWY`3`l?050?ooo`0304e=C@000000oooo00<0oooo00D0000004e=C@3oool0000007alO0060?oo
o`030;:b/P3oool0hN7Q0080000000H0alO70>WYj@3oool0alO70000002JVYX20?ooo`0:0?3`l000
0000hN7Q0?ooo`1=CDd000000?ooo`3YjNT000000:NWY`80oooo02@0jNWY000000000000hN7Q0?oo
o`1=CDd0S8b<0?ooo`2JVYX0000004e=C@2<S8`0_Kfm0<c<c02JVYX000000<O7a`3oool0fMWI0000
003@d=00l?3`09ZJVP000000l?3`0?ooo`3`l?0000000>7Qh@3oool0CDe=0000003oool0jNWY0000
002WYjL30?ooo`060?3`l0000000VYZJ0;:b/P000000l?3`0P3oool01P3`l?0000000>7Qh@3oool0
CDe=000002/0oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P803o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJA/0oooo0P0000090?ooo`8000002`3oool01P2m_Kd0l?3`0?oo
o`2b/[80000007alO0T0oooo0P0000001`3IfMT0oooo09ZJVP000000_Kfm0?ooo`3IfMT00P000000
2P3@d=00VYZJ0;:b/P3oool0000007alO03oool0/[:b0000003`l?020?ooo`0506QXJ01lO7`0oooo
0000001lO7`01P3oool2000000030;:b/P3oool0oooo00@0oooo00d0/[:b000000000000oooo07al
O0000000hN7Q0?ooo`2WYjL0l?3`0;:b/P000000l?3`0080oooo00<0J6QX07alO03IfMT00P000000
4@3@d=00VYZJ0;:b/P3oool0O7al0000003QhN40oooo0:NWY`3`l?00oooo0000001lO7`0oooo0;:b
/P000000l?3`0080oooo00`0J6QX07alO03oool0CDe=0000003`l?00oooo04e=C@000000l?3`0>WY
j@2b/[850?ooo`030;:b/P000000d=3@0080oooo00H0d=3@0000002b/[80oooo0000001lO7`60?oo
o`0;0;fm_@3oool0fMWI0000001lO7`0oooo0:NWY`3oool0hN7Q0000001=CDd00`3oool02@37alL0
VYZJ0?ooo`37alL000000=3@d03oool0J6QX04e=C@030?ooo`0C04e=C@000000YjNW0=3@d01XJ6P0
00000?ooo`3`l?00J6QX04e=C@3`l?00J6QX055AD@3YjNT000000;:b/P2WYjL0jNWY06QXJ0020<O7
a`030;:b/P000000_Kfm0080oooo00T0alO709ZJVP3oool0alO70000003@d=00oooo06QXJ01=CDd0
0P3oool0403`l?00_Kfm09ZJVP000000h^;R06QXJ037alL0l?3`0;fm_@3`l?00alO709ZJVP3oool0
alO70000003@d=0Z0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103oool00`0P
8200oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooo`8000002@3oool2000000/0oooo00H0jNWY0;:b
/P3oool0CDe=0000003@d=090?ooo`8000000P2JVYX02@1XJ6P0d=3@0?ooo`3oool0alO709ZJVP2<
S8`0_Kfm000000020:NWY`0;0000001lO7`0oooo0?ooo`2b/[80VYZJ0;:b/P2WYjL0VYZJ0?3`l02J
VYX00P0000000`1lO7`0alO70?ooo`030?ooo`80000000<0/[:b0?ooo`3oool0103oool0102b/[80
00000000002WYjL20000000@0<O7a`3QhN40O7al08b<S03YjNT0/[:b09ZJVP2b/[80YjNW09ZJVP3`
l?00alO709ZJVP2<S8`0_Kfm00000080YjNW0P0000000`37alL0hN7Q07alO00208b<S00E0000001l
O7`0oooo0?ooo`2b/[80VYZJ0;:b/P2WYjL0VYZJ0?3`l02<S8`0000004e=C@3IfMT0_Kfm0000001=
CDd0fMWI0<O7a`000000alO700D0oooo00P0alO708b<S02WYjL0YjNW08b<S037alL0l?3`09ZJVP80
000000<0O7al0<O7a`3oool00`3oool02@37alL0jNWY0;:b/P000000VYZJ0?ooo`3QhN40_Kfm0?oo
o`02000000050?3`l03oool0oooo0?ooo`3IfMT00P3QhN403P1XJ6P0VYZJ08b<S01=CDd0000007al
O02WYjL0oooo0:NWY`000000hN7Q0;:b/P000000S8b<0P3oool0403`l?00/[:b0<O7a`2JVYX0LG5a
0;:b/P000000CDe=0?3`l02WYjL000000;:b/P3oool0_Kfm04e=C@2b/[830?ooo`0F0=WIf@3QhN40
hN7Q06QXJ02JVYX0S8b<04e=C@000000O7al0:NWY`3oool0l?3`0;:b/P1=CDd0hN7Q0?3`l02WYjL0
S8b<04e=C@3YjNT0oooo0=WIf@80hN7Q00<0J6QX09ZJVP3oool0:@3oool00`0P8200oooo00000004
000000030=oOg`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6`3oool2
000000T0oooo0P00000<0?ooo`040:NWY`3@d=00000004e=C@X0oooo0P0000001@3IfMT0l?3`08b<
S01=CDd0hN7Q00<0oooo00<0l?3`0?ooo`3oool00P3oool00`3YjNT0alO70?ooo`080?ooo`030000
001lO7`0oooo00D0oooo00<0S8b<0000002JVYX01P3oool00`2<S8`0000007alO00@0?ooo`030?3`
l03oool0oooo00T0oooo00<0jNWY0<O7a`3oool07`3oool00`000000O7al0?ooo`050?ooo`050?3`
l037alL0YjNW0000002b/[800P3oool01@2m_Kd0jNWY08b<S0000000d=3@00@0oooo00P0l?3`0<O7
a`2m_Kd000000?ooo`3YjNT000000:NWY`80oooo00<0fMWI0000002WYjL06@3oool0203`l?00alO7
0;fm_@000000oooo0>WYj@000000YjNW3@3oool0103`l?00alO70;fm_@00000Z0?ooo`03020P803o
ool0000000@0000000<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTK0?ooo`8000002@3oool2000000`0oooo00@0fMWI000000000000/[:b2P3oool2000000050=WI
f@3oool0fMWI0000001=CDd04`3oool00`1XJ6P0O7al0?ooo`050?ooo`030=WIf@000000CDe=00D0
oooo00@0l?3`000000000000d=3@?P3oool00`1XJ6P0O7al0?ooo`060?ooo`040;:b/P1lO7`00000
0=3@d0<0oooo00@0YjNW0;fm_@000000YjNW2P3oool00`2JVYX0YjNW0?ooo`020?ooo`030000001X
J6P0oooo01h0oooo00<0VYZJ0:NWY`3oool0>P3oool00`0P8200oooo00000004000000030=oOg`00
0000oooo00@0oooo00<0820P0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6@3oool0103YjNT0YjNW0000
000000070?ooo`040>WYj@2WYjL00000000000d0oooo00<0CDe=0000003YjNT02P3oool200000005
0=WIf@3oool0/[:b0000001=CDd0203oool206QXJ0T0oooo00<0VYZJ07alO03oool01P3oool00`37
alL0000009ZJVP040?ooo`0309ZJVP000000YjNW01d0oooo0P1XJ6PP0?ooo`0309ZJVP1lO7`0oooo
00H0oooo00@0_Kfm000000000000jNWY0`3oool0103QhN40S8b<0000001XJ6P?0?ooo`0309ZJVP00
0000jNWY02@0oooo0P1XJ6Pe0?ooo`03020P803oool0000000@0000000<0gmoO0000003oool0103o
ool00`0P8200oooo0?ooo`020?ooo`030>c/k03^k^h0SHf=00`0WinO01H0SHf=0>KViP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJAX0oooo00<0
l?3`0<O7a`1=CDd0203oool00`3`l?00alO704e=C@0=0?ooo`030<O7a`2JVYX0oooo00P0oooo00@0
jNWY09ZJVP00000000000P2JVYX00`000000VYZJ0>WYj@080?ooo`0308b<S01XJ6P0oooo00P0oooo
00<0jNWY06QXJ03oool00P2m_Kd50?ooo`030>WYj@2WYjL0YjNW0080/[:b00<0O7al09ZJVP3IfMT0
7P3oool00`2<S8`0J6QX0?ooo`0O0?ooo`030>WYj@1XJ6P0oooo0080_Kfm0`3oool0102m_Kd0J6QX
0000000000050?ooo`050;:b/P00000000000:NWY`3YjNT0303oool0103YjNT0O7al0000002m_KdT
0?ooo`80J6QX=@3oool00`0P8200oooo00000004000000030=oOg`000000oooo00@0oooo00<0820P
0?ooo`3oool00P3oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFUM0?ooo`040>7Qh@2WYjL0
00000:NWYdh0oooo00@0hN7Q0:NWY`000000YjNWO03oool00`0P8200oooo00000004000000030=oO
g`000000oooo00@0oooo00<0820P0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oooc00oooo00<0820P0?ooo`000000100000000`3Ogml000000?ooo`040?ooo`03020P
803oool0oooo0080oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool]0?ooo`@0820P
00<0oooo0000000000000`0000000`3Ogml000000?ooo`040?ooo`03020P803oool0oooo0080oooo
00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool`0?ooo`03020P803oool0000000@00000
00<0gmoO0000003oool0103oool00`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>
01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo;03oool5020P80030?ooo`000000gmoO00@0gmoO00<000000?ooo`3oool0
0`3oool00`0P8200oooo0?ooo`020?ooo`030>c/k02FUYH0oooo0080c/k>00<0?Shn0<k>cP3>c/h0
0`3>c/h06P0n?Sh0c/k>0<k>cP3>c/h0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<P3oool7000000D0oooo00<0820P0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3?cll303hn?P<0clo?0`0n?Sh20<o?c`0F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc<0oooo1@0000060?ooo`03020P803oool0oooo0080oooo00<0k>c/09JFUP3oool00P3De=@3
03hn?P040=CDe00n?Sh0?Shn03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo?P3oool00`0P8200oooo0?oo
o`020?ooo`030>c/k02FUYH0oooo00<0fm_K1@0n?Sh40=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0
820P0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`040>CTi0<0?Shn1@3Ti>@05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
ooln0?ooo`03020P803oool0oooo0080oooo00<0k>c/09JFUP3oool01@3]kNd00`0n?Sh0kNg]0>g]
k@040>g]k@0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool00`3/k>`0UYJF0?oo
o`0<0?Cdm00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool00`3/k>`0UYJF0?oo
o`0<0?clo00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool00`3/k>`0UYJF0?oo
o`0=0?ooo`0E09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool00`3/k>`0UYJF0=OGe`0<
0?ooo`0F0=OGe`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooch0oooo00<0820P0?ooo`3oool00P3oool0103/k>`0f][J09:BTP2f
][H:0;Fe]@0G0;Bd]02?Shl0f][J06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000030000003gmoL0VYZJ0?l0NWYjEP1jNWX04`2JVYX0moOg0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000030000003gmoL0_kno0?l0_knoE`2o_kl04P3gmoL0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo00000`000000moOg0>?Sh`3o0>?SheL0hn?S0180moOg0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000<000000?7al@3oool0o`3oool@0?ooo`030>?Sh`2UYJD0MWIf00<0MWIf00@0
MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0hn?S0:FUY@@0MWIf00@0MgMg07If
MP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0jn_[0:FUY@@0EUIF00@0EeMG05IFEP1GEeL0
EeMG2@1FEUH00`2UYJD0jn_[0?ooo`030?ooo`0B0?7al@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003QhN40o?cl0?_kn`80o?cl00<0no_k0?clo03lo?`0103lo?`00`3kno/0o?cl0?cl
o0020?clo0030?_kn`3lo?`0no_k00D0o?cl00<0no_k0?clo03kno/0GP3kno^G0?clo0030:FUY@1f
MWH0SHf=0100SHf=00H0MWIf0:FUY@3lo?`0o?cl0:FUY@1fMWHA08f=S@0607IfMP2UYJD0o?cl0?_k
n`2UYJD0EUIF4@1KFe/00`1FEUH0YJFU0?clo0030?clo00B0>7Qh@000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000030000003Ph>00moOg0?l0moOg403gmoL00`1fMWH0SHf=0::RXP0@0::RXP0608f=
S@1fMWH0moOg0?Ogm`1fMWH0SHf=4@2RXZ80202=SHd0MWIf0?Ogm`3gmoL0EUIF05]KF`1_Kfl0LG5a
3@1bLW80101aLG40Kfm_05]KF`1FEUH40?Ogm`0B0>3Ph0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000030000003Ng]h0lO7a0080lO7a0P3bl_800`3alO40l_;b0?;blP040?7al@030?;blP3alO40
lO7a00<0lO7a0P3bl_800`3alO40l_;b0?7al@020?;bl_L0lO7a00<0MWIf0;Rh^02[Zj/0402[Zj/0
1P2h^;P0MWIf0?7al@3alO40MWIf0;Rh^140Zj^[00D0^;Rh07IfMP3alO40l_;b05IFEP0206m_K`03
079bLP1gMgL0MgMg00/0MgMg00@0LW9b06m_K`1_Kfl0EUIF0`3alO404`3bl_80g]kN0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000050000003Lg=`0jn_[0>_[j`3/k>`00P3[jn/00`2WYjL0XZ:R
0::RXP020:6QX@030:2PX02OWil0WIfM0080WinO0P2PX:020:6QX@030:NWY`3[jn/0jn_[00D0jn_[
00@0c/k>0<G5a@35aLD0d=3@1@3[jn/00`3UiND0alO70=OGe`030>_[j`060<k>cP3<c<`0j^[Z0>[Z
jP38b<P0d=3@0P3[jn/01@3Jf]X0a/K60>?Sh`3[jn/0h>3P0080aLG500@0c<c<0>WYj@38b<P0d=3@
103[jn/01@3Ph>00aLG50<G5a@35aLD0gMgM0080jn_[00<0j^[Z0<S8b035aLD00P35aLD01036a/H0
hn?S0>_[j`3Ph>040<G5a@050<K6aP3Shn<0jn_[0>[ZjP38b<P00P35aLD0103GemL0jn_[0>_[j`3P
h>020<G5a@070<c<c03YjNT0c/k>0<O7a`3YjNT0jn_[0>3Ph0030<G5a@050=gMg@3[jn/0j^[Z0<S8
b03@d=000P3[jn/02`3Jf]X0a/K60>?Sh`3[jn/0e=CD0<G5a@3Ng]h0k>c/0>_[j`38b<P0d=3@0080
k>c/00D0fm_K0<K6aP3Shn<0i^KV0<O7a`030<G5a@030<c<c03[jn/0k>c/04`0k>c/>@3[jn/0101f
MWH0^k^k0;6a/@2a/K480?ooo`L0/K6a00H0^k^k07IfMP3[jn/0jn_[07IfMP2k^k/20;6a/@`0oooo
0`2a/K402@2k^k/0MWIf0>_[j`3[jn/0EUIF07MgM`1bLW80M7Ad07MgM`0;07QhN00507MgM`1dM7@0
LW9b07MgM`1FEUH0103[jn/04P3Lg=`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000
f][J0>GUi@030>GUi@0308Z:RP2KVi/0[:b/00<0jNWY00X0j^[Z0<C4a025QHD0k>c/0>g]k@3^k^h0
inOW0?3`l03bl_80Xj>S1@3UiND01`3QhN40Bd];010@400@4100Fe]K0<O7a`3?cll00`3UiND04`39
bLT071`L079bLP3De=@0iNGU0>;RhP1@D500>CTi0<G5a@3MgMd09RHV055AD@3?cll0iNGU09BDU00E
5AD0Y:BT0=kNgP2b/[800P0@41001@11@D40emOG02HV9P1ADE40clo?00<0iNGU00T0[jn_010@400@
4100410@09VIV@36a/H0g]kN0=gMg@0T92@00`0@4100100E5AD0Y:BT0=kNgP2b/[84010@400501DE
5@2TY:@0g]kN0=gMg@0T92@00P0@41001023Ph<0cLg=0=WIf@2b/[82010@40070451@@3Kfm/0D51@
01TI6@2j^[X0hn?S0:n_[`03010@400I09VIV@36a/H0emOG02HV9P1ADE40clo?0>GUi@2DU9@05ADE
0:BTY03Ng]h0L71`010@402>SXh0fMWI0=gMg@0V9RH0DE5A0<o?c`3UiND0U9BD01DE5@2TY:@0`/;2
01`L7003010@40040410@036a/H0a/K60>;RhXD0iNGU00D0MWIf0<30`02e]KD0]KFe0?ooo`06079b
LP0302LW9`2e]KD0]KFe00D0]KFe00H0`<3007IfMP3UiND0iNGU07IfMP30`<020;Fe]@030?ooo`0W
9bL09bLW00T09bLW0`2e]KD01@30`<00MWIf0>GUi@3UiND0EUIF0080MgMg00@0NGUi07YjNP3oool0
oooo1`1jNWX20?ooo`0507YjNP1iNGT0MgMg07MgM`1FEUH0103UiND04P3Jf]X0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo00000`000000fMWI0=oOg`030=oOg`050=kNgP2PX:00j^[Z0>WYj@3UiND0
0P3Zj^X02P3@d=00M7Ad0>g]k@3^k^h0[Jf]0=3@d03bl_80lo?c0:>SX`3Ogml20=kNgP0:0=oOg`3N
g]h0Jf][01TI6@35aLD0emOG02DU9@1EEED0a/K60=gMg@80g]kN01X0`l?301`L701bLW80dM7A0=kN
gP3Kfm/0CTi>03Ti>@34a<@0e]KF02DU9@1ADE40cLg=0=kNgP2?Shl05ADE0:BTY03EeMD0CTi>03Ph
>034a<@0gMgM0=OGe`0U9BD0DE5A0<g=c@80g]kN04`0`l?301/K6`1bLW80fMWI08n?S`0E5AD0YJFU
0<K6aP0U9BD06ATI0<G5a@3Ng]h0Shn?01DE5@2TY:@0d];B02DU9@0h>3P0alO70=_Kf`1>CTh05ADE
0:BTY03Bd]809BDU03Ph>03?cll0fm_K04i>CP2OWil0fm_K04i>CP0h>3P0a<C40=gMg@3Kfm/0CTi>
01TI6@2j^[X0`/;201/K6`1bLW80fMWI08n?S`0E5AD0YJFU0<K6aP0U9BD0DE5A0<g=c@3Ng]h0Shn?
01DE5@2TY:@0fMWI06a/K00@4100WinO0=kNgP3GemL09BDU055AD@3=cLd0g]kN08n?S`0E5AD0Y:BT
0;jn_P0L71`0LW9b0=7Ad@3Kfm/0CTi>01TI6@2i^KT0eMGEQ@3Ng]h01@1fMWH0`l?30;^k^`2k^k/0
oooo00H0LW9b00<09bLW0;^k^`2k^k/01@2k^k/01P33`l<0MWIf0=kNgP3Ng]h0MWIf0<?3``80^k^k
00<0oooo02LW9`2k^k/01`2k^k/00`3oool09bLW0;^k^`020;^k^`090<?3``1fMWH0gmoO0=kNgP1F
EUH0MgMg07QhN01kNg/0<30`00<0oooo1@1mOGd30?ooo`05030`<01kNg/0N7Qh07MgM`1FEUH00P3N
g]h0503Ogml0g]kN0=WIf@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003GemL0f=SH
0=WIf@80f=SH0100T92@08n?S`2>SXh0jNWY0861P@1ADE40QhN70=[JfP2VYZH0ZJVY0:b/[01OGel0
l_;b0??cl`3dm?@0Y:BT0`3Hf=P02P3IfMT0_[jn01/K6`1aLG40dM7A0=SHf02;Rh/06ATI0;Vi^@3A
dM420=SHf00J0;fm_@0K6a/0LW9b0<k>cP3Hf=P0eMGE04a<C00i>CT0`l?30=7Ad@0T92@0DE5A0<_;
b`3Hf=P0Rh^;01DE5@2TY:@0cLg=02@T900h>3P0`l?30=OGe`3AdM4092@T055AD@3;bl/20=SHf00F
08^;R`0E5AD0Y:BT0=OGe`3AdM4092@T055AD@2NWYh0410@075aL@3AdM40f=SH08^;R`0E5AD0Y:BT
0<g=c@0T92@0DE5A0<_;b`3Hf=P0Rh^;01DE5@80Y:BT00<0410@08j>SP3De=@00`3Hf=P06@3AdM40
92@T03Ph>033`l<0emOG0=GEe@1<C4`06ATI0;Zj^P2:RXX05ADE0:BTY03GemL0dM7A02@T901ADE40
a<C402@T901ADE40bl_;0=SHf02;Rh/05ADE0:BTY03De=@00`3Hf=P04P3IfMT0d];B02@T901ADE40
bl_;0=WIf@2;Rh/05ADE0:BTY02j^[X06a/K079bLP3>c/h0fMWI0:VYZ@0@4100SXj>0<k>cT`0fMWI
>@3Hf=P01@1fMWH0alO70;no_`2o_kl0oooo00L09bLW1`2o_kl01P37alL0MWIf0=SHf03Hf=P0MWIf
0<O7a`80_kno00<0oooo02LW9`2o_kl01`2o_kl00`3oool09bLW0;no_`020;no_`050<O7a`1fMWH0
f=SH0=SHf01FEUH00P1kNg/00`1nOWh0Ogmo02PX:0030?ooo`<0Ogmo0`3oool00`0X:2P0Ogmo07in
OP0207]kN`0305IFEP3IfMT0f=SH0080f=SH0180emOG0000002JVYX0X:2P0:RXZ02b/[80_;bl0<K6
aP3@d=00fMWI0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000<000000=GEe@3Cdm<00`3Cdm<00`25QHD0Xj>S0>KViP020>[ZjP0;03De=@1oOgl0clo?0;jn
_P2;Rh/0GUiN05QHF01XJ6P0Xj>S0<W9b@2ZZZX0103Cdm<02@2TY:@0410@08j>SP3>c/h0dm?C0:BT
Y00@4100SXj>0<c<c0020=?Cd`0J0;Rh^00K6a/0LW9b0<c<c03Cdm<0clo?04Y:BP0i>CT0`L710<c<
c00T92@0DE5A0<[:bP3Cdm<0R8R801DE5@2TY:@0bLW902@T900h>3P0`L710=?Cd`3<c<`092@T055A
D@3:b/X20=?Cd`0K06MWI`0I6AT0^KVi0=3@d03<c<`092@T055AD@2MWId0410@08j>SP3<c<`0dm?C
08R8R00E5AD0Y:BT0<W9b@0T92@0DE5A0<[:bP3Cdm<0R8R801DE5@2TY:@0QXJ601DE5@2TY:@0c/k>
00<0dm?C01T0c<c<02@T900h>3P0`L710=?Cd`3?cll0BTY:01TI6@2i^KT0IfMW01TI6@2i^KT0d=3@
0<c<c00T92@0DE5A0<?3``0T92@0DE5A0<[:bP3Cdm<0R8R801DE5@2TY:@0d=3@00@0dm?C0140c<c<
02@T901ADE40b/[:0=?Cd`28R8P05ADE0:BTY02f][H06a/K079bLP3<c<`0dm?C0;Rh^00K6a/0LG5a
0<g=c@250=?Cd`0307IfMP3;bl/0a/K60100a/K600H0bl_;07IfMP3Cdm<0dm?C07IfMP3;bl/20<K6
aP030?ooo`0W9bL0a/K600L0a/K600<0oooo02LW9`36a/H00P36a/H01`3;bl/0MWIf0=?Cd`3Cdm<0
EUIF07emO@1oOgl00`23Ph<01@0X:2P0oooo0?ooo`3oool0Ph>300<0oooo00L0:2PX08>3P`23Ph<0
Ph>307moO`1mOGd0EUIF00@0dm?C0140eMGE0000002PX:00YJFU0:f][@2f][H0_kno0<S8b03Bd]80
f][J0>;RhP3YjNT0kno_0?Cdm03fm_H0nOWi0?_kn`030?gmo@40o_kn0@3oool10?ooo`0000D00000
0=?Cd`3>c/h0c/k>0<o?c`020<k>cP0?08N7Q`2CTi<0dm?C0=_Kf`2][Jd0`L710;2`/02YZJT0ZZZZ
08f=S@2ATI40/;2`0<_;b`3_knl0YJFU00<0c/k>00X0clo?08F5Q@0@4100SXj>0<_;b`3>c/h0X:2P
010@401aLG40b/[:0P3>c/h06P2d];@06QXJ079bLP3:b/X0c/k>0<_;b`19BDT0>CTi0<71`@37alL0
8b<S055AD@37alL0c/k>08F5Q@0E5AD0Y:BT0<O7a`0S8b<0>3Ph0<71`@3>c/h0alO702<S8`1ADE40
aLG50P3=cLd06`24Q8@05ADE0:BTY03:b/X0alO702<S8`1ADE40/K6a01XJ6P1bLW80aLG50<k>cP25
QHD05ADE0:BTY036a/H08b<S055AD@38b<P0c/k>08F5Q@0E5AD0Y:BT09nOW`0@4100SXj>0<O7a`02
0<k>cP0I0<S8b036a/H08b<S03Ph>031`L40c/k>0<_;b`19BDT06ATI0;Vi^@24Q8@05ADE0:BTY03:
b/X0alO702<S8`1ADE40a<C402<S8`1ADE40alO70<k>cP25QHD05ADE0:BTY0040<k>cP0B0<o?c`38
b<P08b<S055AD@37alL0c/k>08F5Q@0E5AD0Y:BT0;Bd]00J6QX0LW9b0<[:bP3?cll0XJ6Q010@402>
SXh0c/k>C03?clli0<k>cP0307IfMP3>c/h0bl_;0100bl_;00H0c/k>07IfMP3>c/h0c/k>07IfMP3>
c/h20<_;b`030?ooo`0W9bL0bl_;00L0bl_;00<0oooo02LW9`3;bl/00P3;bl/02@3>c/h0MWIf0<o?
c`3>c/h0EUIF07emO@22PX80QHF508B4Q00208F5Q@0302PX:03oool0oooo00<0oooo00P0:2PX08F5
Q@25QHD0Q8B408F5Q@22PX80OGem05IFEP@0c/k>0140dm?C0000002XZ:P0[Jf]0;>c/`2k^k/0`l?3
0<c<c03EeMD0gMgM0>CTi03[jn/0l?3`0?Cdm03gmoL0nOWi0?clo0030?gmo@40o_kn0@3oool10?oo
o`0000<000000=3@d039bLT00`39bLT04@1cLg<0U9BD07inOP1iNGT0X:2P0=7Ad@2][Jd0YJFU0:JV
YP25QHD0GUiN0:f][@3UiND0m_Kf0?Shn02TY:@0bLW900<0b<S805d0PH61010@402>SXh0alO70<S8
b02LW9`0410@075aL@37alL0b<S80<O7a`2^[Zh06QXJ075aL@37alL0b<S80<G5a@16ATH0>3Ph0<30
`032`/808R8R03Ph>031`L40`/;204I6AP0I6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35aLD0`<30028R
8P0E5AD0YJFU0;6a/@2[Zj/0[Zj^01XJ6P1cLg<0aLG507moO`0@4100Shn?0<G5a@16ATH06ATI0;Vi
^@35aLD0Ogmo01DE5@2TY:@0`L71028R8P1ADE40alO70<S8b021PH405ADE0:BTY031`L408R8R03Ti
>@30`<00aLG506MWI`2UYJD0`L7102<S8`0h>3P0_kno0<G5a@34a<@0ATI601TI6@2i^KT0[jn_01XJ
6P1cLg<0aLG507moO`0@4100Shn?0<;2`P0R8R80>3Ph0<71`@32`/80ATI601TI6@2l_;`01038b<P0
4@39bLT0`/;2028R8P0h>3P0`L710<;2`P16ATH06ATI0;bl_02`/;006QXJ075aL@35aLD0`/;204I6
AP0I6AT0_Kfm04d0bLW9>@38b<P00`1fMWH0dm?C0<g=c@0@0<g=c@060=?Cd`1fMWH0b<S80<S8b01f
MWH0dm?C0P3=cLd00`3oool09bLW0<g=c@070<g=c@030?ooo`0W9bL0cLg=0080cLg=00L0dm?C07If
MP38b<P0b<S805IFEP22PX80Ph>300D0QhN700D0:2PX0?ooo`3oool0oooo02PX:00508N7Q`0308>3
P`22PX80EUIF00<0bLW90180b<S80=3@d0000000/[:b0;Jf]P2k^k/0`/;20<W9b@3AdM40fMWI0>3P
h03WinL0kNg]0?7al@3emOD0n?Sh0?[jnP3lo?`30?gmo@40o_kn0@3oool10?ooo`0000<000000<o?
c`35aLD01035aLD03`2GUiL0]kNg0>c/k02i^KT0Rh^;0;Ng]`2ZZZX0Y:BT07inOP1aLG40a<C40=WI
f@3fm_H0nOWi0:FUY@040<G5a@0C09VIV@0@4100SHf=0<K6aP35aLD0VIVI010@402>SXh0a/K60:b/
[01LG5`0[Jf]01XJ6P1aLG40a/K60<G5a@32`/80ADE503Ph>0020;no_`08028R8P1MGEd0410@010@
400E5AD0Y:BT0<;2`P15ADD3010@400:055AD@2o_kl08R8R04i>CP15ADD0410@055AD@35aLD0[:b/
01XJ6P80410@00D0SHf=0<G5a@35aLD0`/;204E5A@03010@400501DE5@2TY:@0_kno028R8P1ADE40
0P35aLD01P1oOgl05ADE0:BTY035aLD0`/;204E5A@80410@00<0>CTi0;bl_015ADD00`0@4100201A
DE40HF5Q010@400I6AT0^KVi0<G5a@2/[:`06QXJ0P0@41001@2=SHd0aLG50;no_`0R8R80GEeM0080
410@00<05ADE0:BTY035aLD01@35aLD00`2o_kl08R8R05eMG@02010@400501DE5@2TY:@0aLG50:b/
[00J6QX00`0@41000`0I6AT0^KVi0<G5a@260<G5a@0307IfMP3EeMD0dM7A0100dM7A00H0eMGE07If
MP35aLD0aLG507IfMP3EeMD20=7Ad@030?ooo`0W9bL0dM7A00L0dM7A00<0oooo02LW9`3AdM400P3A
dM401`3EeMD0MWIf0<G5a@35aLD0EUIF08:2PP27QhL01@29RHT50?ooo`D0RHV900<0QhN708:2PP1F
EUH01035aLD04@3?cll000000;bl_02o_kl0`l?30<W9b@3@d=00emOG0=gMg@3Ti>@0j^[Z0>o_k`3c
lo<0m_Kf0?Shn03kno/0o?cl0080oOgm0P3no_h20?ooo`0000<000000<g=c@31`L400`31`L40401e
MGD0VIVI0:NWY`3]kNd0bLW907=cL`2`/;00SHf=07=cL`1<C4`0HV9R0;jn_P2e]KD0/[:b0:f][@2Y
ZJT40<71`@0;0;^k^`0R8R80LG5a0<G5a@32`/80O7al01DE5@2SXj<0`/;20:VYZ@0J6QX00P0@4100
201aLG40a<C40<71`@2n_[h0A4A403Ph>02n_[h0`/;21`31`L401@2k^k/08R8R03Ph>02n_[h0`/;2
0180`L7100@0O7al01DE5@2TY:@0`l?33P31`L401@2k^k/08R8R03Ph>02n_[h0`/;20080`L7100<0
`l?30<G5a@32`/806`31`L40102YZJT06QXJ075aL@34a<B:0<71`@0307IfMP3Hf=P0eMGE0100eMGE
00H0f=SH07IfMP31`L40`L7107IfMP3Hf=P20=GEe@030?ooo`0W9bL0oooo00P0oooo00H09bLW0=GE
e@3EeMD0eMGE0=SHf01fMWH20<71`@0305IFEP24Q8@0R8R800@0RXZ:0`3oool0100X:2P0oooo0?oo
o`3oool408Z:RP0308R8R024Q8@0EUIF00@0`L710100cLg=00000036a/H0b<S80<c<c03AdM40emOG
0=cLg03Rh^80j>SX0>g]k@3alO40mOGe0?Ogm`3inOT0o?cl0`3moOd20?knoP80oooo00000`000000
c<c<0;Zj^P020;fm_@0A0;jn_P2m_Kd0WYjN0>?Sh`2f][H0UYJF08:2PP2CTi<0Q8B408:2PP1>CTh0
;2`/09bLW02[Zj/0m_Kf0?Shn02WYjL01@2m_Kd01`1jNWX06ATI0;Jf]P32`/80ATI603Ph>02h^;P0
0P2m_Kd02@2UYJD06QXJ010@401`L700`/;20;fm_@2j^[X0@d=303Ph>0020;jn_PP0_Kfm00<0GEeM
03Lg=`2h^;P04`2m_Kd0101jNWX05ADE0:BTY02o_kl?0;fm_@0305eMG@0g=cL0^;Rh0080_Kfm00@0
GEeM01TI6@2i^KT0_kno4P2m_Kd90;jn_P040:JVYP0J6QX0LG5a0<;2`U40_[jn>@2m_Kd00`1fMWH0
f][J0=GEe@0@0=GEe@060=[JfP1fMWH0_Kfm0;fm_@1fMWH0f][J0P3EeMD00`3oool0LW9b079bLP08
079bLP0>02LW9`3EeMD0eMGE0=GEe@3Jf]X0MWIf0;fm_@2n_[h0EUIF08N7Q`2:RXX0SHf=08j>SP2=
SHd30?ooo`0302PX:02=SHd0:2PX00<0oooo0`2=SHd0102:RXX0QhN705IFEP2n_[h20;fm_@0A0;Zj
^P3<c<`000000=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OWi`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[j
nP3lo?`00`3moOd00`3no_h0oooo0?ooo`010?ooo`0000<000000<[:bP2h^;P00`2j^[X0401lO7`0
SHf=07QhN02IVIT0l?3`0<;2`P2WYjL0bl_;0;bl_02SXj<0KFe]07emO@2/[:`0no_k0?clo02VYZH3
0;Zj^P040;^k^`2j^[X0^[Zj05]KF`80410@00<0=cLg0;Bd]02j^[X0102j^[X0201KFe/0Jf][0;Zj
^P2j^[X0]kNg0492@P0g=cL0];Bd7`2j^[X00`1gMgL051@D09^KV`0E0;Zj^P0305]KF`0I6AT0[jn_
01`0^[Zj00<0XZ:R01TI6@1[Jf/0R`2j^[X00`1fMWH0f=SH0=[JfP0@0=[JfP060=SHf01fMWH0^[Zj
0;Zj^P1fMWH0f=SH0P3Jf]X00`3oool0LW9b079bLP08079bLP0=02LW9`3Jf]X0f][J0=[JfP3Hf=P0
MWIf0;Zj^P2k^k/0EUIF08^;R`2?Shl0SXj>08n?S`030?ooo`0502PX:02?Shl0Shn?08n?S`0X:2P0
0`3oool01@2?Shl0SXj>08n?S`2;Rh/0EUIF00<0^[Zj0100^;Rh0<[:bP000000fMWI0=[JfP3MgMd0
h>3P0>CTi03Xj>P0k>c/0>o_k`3clo<0m_Kf0?Shn03jn_X0o?cl0`3moOd20?knoP<0oooo00000`00
0000bLW90;Rh^0030;Rh^00@08J6QP2NWYh0e=CD0?3`l03bl_80c/k>0<71`@3MgMd0d];B0::RXP3<
c<`0bLW908V9R@3lo?`0oOgm0:NWYo`0^;Rh00<0MWIf0=OGe`3MgMd0403MgMd01P3GemL0MWIf0;Rh
^02h^;P0MWIf0=OGe`<0gMgM2`0W9bL30=gMg@090=OGe`1fMWH0^;Rh0;Rh^01FEUH0T92@096AT@2C
Ti<0U9BD0080oooo00<0:2PX09RHV02HV9P00`2HV9P0200X:2P0oooo0?ooo`2DU9@0Ti>C096AT@2@
T900EUIF102h^;P03`39bLT000000>7Qh@3Rh^80i>CT0>OWi`3Zj^X0kNg]0?3`l03clo<0mOGe0?Og
m`3inOT0no_k0?clo0030?gmo@030?knoP3oool0oooo0080oooo00000`000000bLW90;Rh^0040;Rh
^00?08J6QP2DU9@0l_;b0??cl`2PX:00j>SX0?KfmP3Kfm/0VIVI0?[jnP3kno/0g=cL0;Jf]P3no_h0
YjNW0?`0^;Rh00<0MWIf0=KFeP3QhN40403QhN401P3Fe]H0MWIf0;Rh^02h^;P0MWIf0=KFeQ40hN7Q
00T0e]KF07IfMP2h^;P0^;Rh05IFEP2EUID0UiNG09bLW02RXZ800P0X:2P01@2[Zj/0[Jf]0:^[Z`2[
Zj/0[Jf]0080Zj^[0P0X:2P01@2RXZ80W9bL09NGU`2EUID0EUIF00@0^;Rh00h0bLW90000003Xj>P0
jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0<0oOgm0P3no_h40?ooo`00
00<000000<W9b@2e]KD00`2e]KD03@1gMgL0X:2P0>3Ph03clo<0iNGU0>c/k03fm_H0n?Sh0?Win@2@
T900no_k0?clo03moOd00P3no_h00`2WYjL0]KFe0;Fe]@3j0;Fe]@0307IfMP3]kNd0jNWY0100jNWY
00H0kNg]07IfMP2e]KD0]KFe07IfMP3]kNdA0>WYj@0:0>g]k@1fMWH0]KFe0;Fe]@1FEUH0Vi^K09jN
WP2UYJD0/[:b0;Zj^P80_kno00@0`<300;no_`2o_kl0`<300P2o_kl01`30`<00^[Zj0;:b/P2UYJD0
WYjN09^KV`1FEUH0102e]KD03@39bLT000000>k^kP3_knl0l?3`0?7al@3clo<0mOGe0?KfmP3hn?P0
nOWi0?_kn`3lo?`00`3moOd20?knoPD0oooo00000`000000bLW90;Zj^P040;6a/@0308f=S@2ZZZX0
ZZZZ00D0ZZZZ00<0YjNW0:ZZZP2ZZZX0102ZZZ[l0;6a/@0307IfMP3De=@0l_;b0100l_;b00H0e=CD
07IfMP2a/K40/K6a07IfMP3De=@A0?;blP060=CDe01fMWH0/K6a0;6a/@1FEUH0Xj>S0P2XZ:P=0<?3
``80Z:RX00<0Xj>S05IFEP2a/K400P2a/K40102j^[X0bLW90000003clo<20?Cdm0070?Gem@3fm_H0
moOg0?Shn03jn_X0no_k0?clo0040?gmo@030?knoP3oool0oooo00@0oooo00000`000000`l?30<W9
b@3o0:j^[Q00[Zj^00@0SHf=07IfMP3>c/h0no_k3@3oool0103kno/0c/k>07IfMP2=SHd20:j^[P04
08f=S@1fMWH0c/k>0?_kn`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^[Zh01P2IVIT0EUIF0;Fe]@3;
bl/0gmoO0>SXj0T0inOW00H0j>SX0=oOg`3;bl/0]KFe05IFEP2IVIT30:j^[P030<W9b@33`l<00000
0080m_Kf00D0moOg0?Shn03hn?P0nOWi0?[jnP020?clo0@0oOgm0P3no_h60?ooo`0000@000000<?3
``3De=@0]kNgo`2WYjL@0:NWY`0308f=S@1fMWH0MWIf00l0MWIf00<0SHf=0:NWY`2WYjL00P2WYjL0
0`2=SHd0MWIf07IfMP0?07IfMP0308f=S@2WYjL0YjNW0080YjNW00<0SHf=05IFEP1FEUH03`1FEUH0
202=SHd0YjNW0:NWY`2WYjL0]kNg0=CDe033`l<000000`3inOT0103jn_X0no_k0?clo03lo?`50?gm
o@80o_kn1`3oool000060000002ZZZX0inOW0=CDe02n_[h0/k>co`2ZZZY@0:ZZZP060;>c/`2n_[h0
e=CD0>OWi`2ZZZX000000P3kno/30?clo0D0oOgm0`3no_h80?ooo`0000L0000008Z:RP3IfMT0o?cl
0>CTi03Bd]80c<c<0?l0_[jnCP2n_[h0203<c<`0d];B0>CTi03lo?`0fMWI08Z:RP000000o?cl203m
oOd20?knoPX0oooo000020000000JFUY09NGU`3IfMT0o?cl0?;blP3/k>`0hn?So`3IfMU<0=WIf@08
0>?Sh`3/k>`0l_;b0?clo03IfMT0UiNG06UYJ@0000070?gmo@<0o_kn2`3oool000050000003oool0
JFUY09NGU`3Cdm<00P3lo?co0?ooodh0oooo0P3lo?`01@3Cdm<0UiNG06UYJ@3oool0000000@0oOgm
103no_h=0?ooo`0000<000000?ooo`3oool00P1YJFT00`2:RXX0UiNG0:ZZZP3o0:ZZZTd0ZZZZ00@0
UiNG08Z:RP1YJFT0JFUY0P3oool00`000000o_kn0?knoP040?knoPl0oooo003o000005`000005@3o
ool00001\
\>"],
  ImageRangeCache->{{{126.938, 383.5}, {439, 216.75}} -> {-181.678, 311.133, \
1.00159, 1.00159}}],

Cell[TextData[{
  "Using cell brackets in this manner allows you to organize your work in an \
orderly manner, as well as to outline material. For a complete description of \
cell brackets and many other interface features, you should consult the \
documentation that came with your version of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "For information on other features such as saving, printing, and editing \
notebooks, consult the manuals that came with your version of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Entering input", "Subsubsection",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell["\<\
New input can be entered whenever there is a horizontal line that runs across \
the width of the notebook. If one is not present where you wish to place an \
input cell, move the cursor up and down until it changes to a horizontal bar \
and then click the mouse once. A horizontal line should now appear across the \
width of the window. You can immediately start typing and an input cell will \
be created.\
\>", "TextFirst",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "Input can be entered exactly as it appears in this book. To get ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to evaluate an expression that you have entered, ",
  "press \[ShiftKey]-\[LeftModified]\[EnterKey]\[RightModified]; that is, \
hold down the Shift key and then press the Enter key."
}], "Text",
  CellTags->{"i:34", "b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "You can enter mathematical expressions in a traditional looking \
two-dimensional format using either palettes for quick entry of template \
expressions, or keyboard equivalents. For example, the following expression \
can be entered by using the Basic Input palette, or through a series of \
keystrokes. For details of inputting mathematical expressions, read your user \
documentation or read the section on ",
  StyleBox["2D Expression Input", "SR"],
  " in the Help Browser."
}], "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Integral]\(1\/\(1 - x\^3\)\) \[DifferentialD]x\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[BoxData[
    \(ArcTan[\(1 + 2\ x\)\/\@3]\/\@3 - 1\/3\ Log[\(-1\) + x] + 
      1\/6\ Log[1 + x + x\^2]\)], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[TextData[{
  "As noted previously, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " enters the ",
  StyleBox["In", "MR"],
  " and ",
  StyleBox["Out", "MR"],
  " prompts for you. You do not type these prompts. You will see them ",
  StyleBox["after",
    FontSlant->"Italic"],
  " you evaluate your input."
}], "Text",
  CellTags->{"i:35", "b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "You can refer to the result of the previous calculation using the symbol \
",
  StyleBox["%", "MR"],
  "."
}], "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(2\^64\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[BoxData[
    \(18446744073709551616\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(% + 1\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[BoxData[
    \(18446744073709551617\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[TextData[{
  "You can also refer to the result of any earlier calculation using its ",
  StyleBox["Out[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  StyleBox["]", "MR"],
  " label or, equivalently, ",
  StyleBox["%", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`i\)]],
  "."
}], "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Out[1]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[BoxData[
    \(ArcTan[\(1 + 2\ x\)\/\@3]\/\@3 - 1\/3\ Log[\(-1\) + x] + 
      1\/6\ Log[1 + x + x\^2]\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%2\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[BoxData[
    \(18446744073709551616\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
  "Ending a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session"
}], "Subsubsection",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "To end your ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " session, choose ",
  StyleBox["Exit", "SR"],
  " from the ",
  StyleBox["File", "SR"],
  " menu. You will be prompted to save any unsaved open notebooks."
}], "TextFirst",
  CellTags->{"i:36", "b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Getting out of trouble", "Subsubsection",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "From time to time, you will type an input which will cause ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " to misbehave in some way, perhaps by just going silent for a long time \
(if, for example, you have inadvertently asked it to do something very \
difficult) or perhaps by printing out screen after screen of not terribly \
useful information. In this case, you can try to \
\[OpenCurlyDoubleQuote]interrupt\[CloseCurlyDoubleQuote] the calculation. How \
you do this depends on your computer\[CloseCurlyQuote]s operating system:"
}], "TextFirst",
  CellTags->{"i:37", "b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "Macintosh: type \[CommandKey]\[LeftModified]",
  StyleBox[".", "MR"],
  "\[RightModified] (the Command key and the period) and then type ",
  StyleBox["a", "MR"]
}], "BulletedList",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "Windows 95/98/NT/2000/XP: type \[AltKey]\[LeftModified]",
  StyleBox[".", "MR"],
  "\[RightModified] (the Alt key and the period)"
}], "BulletedList",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "Unix: type \[ControlKey]-\[LeftModified]",
  StyleBox[".", "MR"],
  "\[RightModified] and then type ",
  StyleBox["a", "MR"],
  " and then \[ReturnKey]"
}], "BulletedList",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[TextData[{
  "These attempts to stop the computation will sometimes fail. If after \
waiting a reasonable amount of time (say, a few minutes), ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " still seems to be stuck, you will have to \[OpenCurlyDoubleQuote]kill the \
kernel.\[CloseCurlyDoubleQuote] (Before attempting to kill the kernel, try to \
convince yourself that the computation is really in a loop from which it will \
not return and that it is not just an intensive computation that requires a \
lot of time.) Killing the kernel is accomplished by selecting ",
  StyleBox["Quit Kernel", "SR"],
  " from the ",
  StyleBox["Kernel", "SR"],
  " menu. The kernel can then be restarted without killing the front end by \
first selecting ",
  StyleBox["Start Kernel \[RightPointer] Local", "SR"],
  " under the ",
  StyleBox["Kernel", "SR"],
  " menu, or you can simply evaluate a command in a notebook and a new kernel \
should start up automatically."
}], "Text",
  CellTags->{"i:38", "b:1.0", "IPM3.1.2.1"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The syntax of inputs", "Subsection",
  CellTags->{"b:1.1", "IPM3.1.2.2", "c:11"}],

Cell["\<\
You can enter mathematical expressions in a linear syntax using arithmetic \
operators common to almost all computer languages.\
\>", "TextFirst",
  CellTags->{"i:39", "b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(39/13\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(3\)], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "Alternately, you can enter this expression in the traditional form by \
typing ",
  StyleBox["39", "MR"],
  ", \[ControlKey]\[LeftModified]",
  StyleBox["/", "MR"],
  "\[RightModified], then ",
  StyleBox["13", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(39\/13\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(3\)], "Output",
  CellLabel->"Out[7]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "The caret (",
  StyleBox["^", "MR"],
  ") is used for exponentiation."
}], "TextFirst",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(2^5\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(32\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "You can enter this expression in a more traditional typeset form by typing \
",
  StyleBox["2", "MR"],
  ", \[ControlKey]\[LeftModified]",
  StyleBox["^", "MR"],
  "\[RightModified], and then ",
  StyleBox["5", "MR"],
  "."
}], "TextFirst",
  CellTags->{"i:40", "b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(2\^5\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(32\)], "Output",
  CellLabel->"Out[9]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " includes several different ways of entering typeset expressions, either \
directly from the keyboard as we did above, or via palettes available from \
the ",
  StyleBox["File", "SR"],
  " menu. Below is a brief table showing some of the more commonly used \
typeset expressions and how they are entered through the keyboard. You should \
read your documentation and become comfortable using these input interfaces \
so that you can easily enter the kinds of expressions in this book."
}], "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    FormBox[GridBox[{
          {
            StyleBox["Expression",
              "TB"], 
            StyleBox["FullForm",
              "TB"], 
            StyleBox[\(Keyboard\ shortcut\),
              "TB"]},
          {
            SuperscriptBox[
              StyleBox["x",
                FontWeight->"Plain"], "2"], 
            RowBox[{
              StyleBox["SuperscriptBox",
                "MR"], 
              StyleBox["[",
                "MR"], 
              RowBox[{
                StyleBox["x",
                  "MR",
                  FontSlant->"Plain"], 
                StyleBox[",",
                  "MR"], 
                StyleBox["2",
                  "MR"]}], 
              StyleBox["]",
                "MR"]}], 
            RowBox[{
              RowBox[{
                StyleBox["x",
                  "MR",
                  FontSlant->"Plain"], 
                StyleBox[" ",
                  "TR"], 
                StyleBox[
                  RowBox[{"\[ControlKey]\[LeftModified]", 
                    StyleBox["6",
                      "MR"], "\[RightModified]"}]]}], ",", 
              StyleBox["2",
                "MR"]}]},
          {
            SubscriptBox[
              StyleBox["x",
                FontWeight->"Plain"], "i"], 
            RowBox[{
              StyleBox["SubscriptBox",
                "MR"], 
              StyleBox["[",
                "MR"], 
              RowBox[{
                StyleBox["x",
                  "MR",
                  FontSlant->"Plain"], 
                StyleBox[",",
                  "MR"], 
                StyleBox["i",
                  "MR",
                  FontSlant->"Plain"]}], 
              StyleBox["]",
                "MR"]}], 
            RowBox[{
              RowBox[{
                RowBox[{
                  StyleBox["x",
                    "MR",
                    FontSlant->"Plain"], 
                  StyleBox[" ",
                    "TR"], "\[ControlKey]\[LeftModified]"}], 
                StyleBox["-",
                  "MR"], "\[RightModified]"}], ",", 
              StyleBox["i",
                "MR",
                FontSlant->"Plain"]}]},
          {
            FractionBox[
              StyleBox["x",
                FontWeight->"Plain"], "y"], 
            RowBox[{
              StyleBox["FractionBox",
                "MR"], 
              StyleBox["[",
                "MR"], 
              RowBox[{
                StyleBox["x",
                  "MR",
                  FontSlant->"Plain"], 
                StyleBox[",",
                  "MR"], 
                StyleBox["y",
                  "MR",
                  FontSlant->"Plain"]}], 
              StyleBox["]",
                "MR"]}], 
            RowBox[{
              RowBox[{
                StyleBox["x",
                  "MR",
                  FontSlant->"Plain"], 
                StyleBox[" ",
                  "TR"], 
                RowBox[{"\[ControlKey]\[LeftModified]", 
                  StyleBox["/",
                    "MR"], "\[RightModified]"}]}], ",", 
              StyleBox["y",
                "MR",
                FontSlant->"Plain"]}]},
          {
            StyleBox[
              SqrtBox[
                StyleBox["x",
                  FontWeight->"Plain"]],
              FontWeight->"Plain"], 
            RowBox[{
              StyleBox["SqrtBox",
                "MR"], 
              StyleBox["[",
                "MR"], 
              StyleBox["x",
                "MR",
                FontSlant->"Plain"], 
              StyleBox["]",
                "MR"]}], 
            RowBox[{
              StyleBox[
                RowBox[{"\[ControlKey]\[LeftModified]", 
                  StyleBox["2",
                    "MR"], "\[RightModified]"}]], ",", 
              StyleBox["x",
                "MR",
                FontSlant->"Plain"]}]},
          {
            StyleBox[\(x \[GreaterEqual] y\),
              FontWeight->"Plain"], 
            RowBox[{
              StyleBox["GreaterEqual",
                "MR"], 
              StyleBox["[",
                "MR"], 
              RowBox[{
                StyleBox["x",
                  "MR",
                  FontSlant->"Plain"], 
                StyleBox[",",
                  "MR"], 
                StyleBox["y",
                  "MR",
                  FontSlant->"Plain"]}], 
              StyleBox["]",
                "MR"]}], 
            RowBox[{
              RowBox[{
                RowBox[{
                  StyleBox["x",
                    "MR",
                    FontSlant->"Plain"], " ", "\[EscapeKey]"}], 
                StyleBox[">=",
                  "MR"], "\[EscapeKey]"}], ",", 
              StyleBox["y",
                "MR",
                FontSlant->"Plain"]}]}
          },
        ColumnAlignments->{Left},
        GridFrame->True,
        GridFrameMargins->True,
        RowLines->{True, False},
        ColumnLines->True], DefaultTraditionalFormatType]], "DisplayTable",
  CellTags->{"i:41", "b:1.1", "IPM3.1.2.2"}],

Cell["Entering typeset expressions", "NumberedTable",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[TextData[{
  "Y",
  "ou can indicate multiplication by simply putting a space between the two \
factors, as in mathematics. You can also use the asterisk (",
  StyleBox["*", "MR"],
  ") for that purpose, as is traditional in most computer languages."
}], "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(2\ 5\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(10\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(2*5\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(10\)], "Output",
  CellLabel->"Out[11]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " also gives operations the same precedence as in mathematics. In \
particular, multiplication and division have a higher precedence than \
addition and subtraction, so that ",
  StyleBox["3 + 4 * 5", "MR"],
  " equals 23 and not 35."
}], "Text",
  CellTags->{"i:42", "b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(3 + 4\ 5\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(23\)], "Output",
  CellLabel->"Out[12]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell["\<\
Functions are also written as they are in mathematics books, except that \
function names are capitalized and their arguments are enclosed in square \
brackets.\
\>", "Text",
  PageBreakAbove->True,
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Factor[x\^5 - 1]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(\((\(-1\) + x)\)\ \((1 + x + x\^2 + x\^3 + x\^4)\)\)], "Output",
  CellLabel->"Out[13]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "Almost all of the built-in functions are spelled out in full, as in the \
above example. The exceptions to this rule are well-known abbreviations such \
as ",
  StyleBox["D", "MR"],
  " for differentiation, ",
  StyleBox["Sqrt", "MR"],
  " for square roots, ",
  StyleBox["Log", "MR"],
  " for logarithms, and ",
  StyleBox["Det", "MR"],
  " for the determinant of a matrix. Spelling out the name of a function in \
full is quite useful when you are not sure whether a function exists to \
perform a particular task. For example, if we wanted to compute the conjugate \
of a complex number, an educated guess would be:"
}], "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Conjugate[3 + 4\ \[ImaginaryI]]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(3 - 4\ \[ImaginaryI]\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "Whereas square brackets ",
  StyleBox["[", "MR"],
  " and ",
  StyleBox["]", "MR"],
  " are used to enclose the arguments to functions, curly braces ",
  StyleBox["{", "MR"],
  " and ",
  StyleBox["}", "MR"],
  " are used to indicate a ",
  StyleBox["list",
    FontSlant->"Italic"],
  " or range of values. Lists are a basic data type in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and are used to represent vectors and matrices (and tensors of any \
dimension), as well as additional arguments to functions such as in ",
  StyleBox["Plot", "MR"],
  " and ",
  StyleBox["Integrate", "MR"],
  "."
}], "Text",
  CellTags->{"i:43", "b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \({a, b, c} . {x, y, z}\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(a\ x + b\ y + c\ z\)], "Output",
  CellLabel->"Out[15]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[
        Sin[x + \@2\ Sin[x]], {x, \(-2\)\ \[Pi], 2\ \[Pi]}];\)\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.5 0.0757881 0.30902 0.294311 [
[.04527 .29652 -7.59375 -11.875 ]
[.04527 .29652 7.59375 0 ]
[.19685 .29652 -7.59375 -11.875 ]
[.19685 .29652 7.59375 0 ]
[.34842 .29652 -7.59375 -11.875 ]
[.34842 .29652 7.59375 0 ]
[.65158 .29652 -4.5 -11.875 ]
[.65158 .29652 4.5 0 ]
[.80315 .29652 -4.5 -11.875 ]
[.80315 .29652 4.5 0 ]
[.95473 .29652 -4.5 -11.875 ]
[.95473 .29652 4.5 0 ]
[.4875 .01471 -15.1875 -5.9375 ]
[.4875 .01471 0 5.9375 ]
[.4875 .16186 -22.6875 -5.9375 ]
[.4875 .16186 0 5.9375 ]
[.4875 .45618 -16.5 -5.9375 ]
[.4875 .45618 0 5.9375 ]
[.4875 .60333 -9 -5.9375 ]
[.4875 .60333 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.04527 .30902 m
.04527 .31527 L
s
gsave
.04527 .29652 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.19685 .30902 m
.19685 .31527 L
s
gsave
.19685 .29652 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.34842 .30902 m
.34842 .31527 L
s
gsave
.34842 .29652 -68.5938 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.65158 .30902 m
.65158 .31527 L
s
gsave
.65158 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.80315 .30902 m
.80315 .31527 L
s
gsave
.80315 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.95473 .30902 m
.95473 .31527 L
s
gsave
.95473 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.125 Mabswid
.08317 .30902 m
.08317 .31277 L
s
.12106 .30902 m
.12106 .31277 L
s
.15895 .30902 m
.15895 .31277 L
s
.23474 .30902 m
.23474 .31277 L
s
.27264 .30902 m
.27264 .31277 L
s
.31053 .30902 m
.31053 .31277 L
s
.38632 .30902 m
.38632 .31277 L
s
.42421 .30902 m
.42421 .31277 L
s
.46211 .30902 m
.46211 .31277 L
s
.53789 .30902 m
.53789 .31277 L
s
.57579 .30902 m
.57579 .31277 L
s
.61368 .30902 m
.61368 .31277 L
s
.68947 .30902 m
.68947 .31277 L
s
.72736 .30902 m
.72736 .31277 L
s
.76526 .30902 m
.76526 .31277 L
s
.84105 .30902 m
.84105 .31277 L
s
.87894 .30902 m
.87894 .31277 L
s
.91683 .30902 m
.91683 .31277 L
s
.00738 .30902 m
.00738 .31277 L
s
.99262 .30902 m
.99262 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.5 .01471 m
.50625 .01471 L
s
gsave
.4875 .01471 -76.1875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.5 .16186 m
.50625 .16186 L
s
gsave
.4875 .16186 -83.6875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.5 .45618 m
.50625 .45618 L
s
gsave
.4875 .45618 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.5 .60333 m
.50625 .60333 L
s
gsave
.4875 .60333 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.5 .04414 m
.50375 .04414 L
s
.5 .07357 m
.50375 .07357 L
s
.5 .103 m
.50375 .103 L
s
.5 .13243 m
.50375 .13243 L
s
.5 .1913 m
.50375 .1913 L
s
.5 .22073 m
.50375 .22073 L
s
.5 .25016 m
.50375 .25016 L
s
.5 .27959 m
.50375 .27959 L
s
.5 .33845 m
.50375 .33845 L
s
.5 .36788 m
.50375 .36788 L
s
.5 .39731 m
.50375 .39731 L
s
.5 .42674 m
.50375 .42674 L
s
.5 .48561 m
.50375 .48561 L
s
.5 .51504 m
.50375 .51504 L
s
.5 .54447 m
.50375 .54447 L
s
.5 .5739 m
.50375 .5739 L
s
.25 Mabswid
.5 0 m
.5 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.03279 .39192 L
.04262 .47411 L
.04758 .50961 L
.05288 .5419 L
.05785 .56624 L
.06244 .58332 L
.06508 .59075 L
.06788 .59672 L
.06933 .59905 L
.07069 .60077 L
.07146 .60154 L
.07218 .60214 L
.07302 .60268 L
.07379 .60303 L
.07507 .60332 L
.07579 .60331 L
.07644 .60321 L
.07771 .60274 L
.07891 .60197 L
.07959 .6014 L
.08031 .60069 L
.0816 .59916 L
.08451 .59451 L
.0869 .58953 L
.08946 .58311 L
.09408 .56903 L
.10458 .52782 L
.12439 .43491 L
.13484 .3876 L
.14594 .34339 L
.15578 .31096 L
.16664 .28311 L
.17139 .27354 L
.17651 .26494 L
.181 .25884 L
.18579 .25373 L
.18838 .25155 L
.19074 .2499 L
.19314 .24857 L
.19536 .24762 L
.19661 .2472 L
.19794 .24685 L
.19928 .24659 L
.20003 .24648 L
.20074 .24641 L
.20202 .24635 L
.20319 .24636 L
.20384 .2464 L
.20454 .24646 L
Mistroke
.20583 .24664 L
.20652 .24676 L
.20726 .24693 L
.20861 .24729 L
.21113 .24819 L
.21407 .24959 L
.21678 .2512 L
.22693 .25974 L
.23669 .2711 L
.24579 .28379 L
.26647 .31634 L
.28514 .34432 L
.29498 .35631 L
.30037 .36166 L
.30543 .36575 L
.31069 .36895 L
.31348 .37017 L
.31505 .37071 L
.31649 .3711 L
.31772 .37136 L
.31887 .37153 L
.32013 .37165 L
.32083 .37169 L
.32147 .3717 L
.32276 .37165 L
.32349 .37159 L
.32416 .37151 L
.3254 .37129 L
.32671 .37098 L
.3291 .37017 L
.33167 .36896 L
.33409 .36746 L
.3363 .3658 L
.34171 .3605 L
.34684 .35378 L
.35648 .33642 L
.36672 .31087 L
.38514 .24653 L
.40481 .1575 L
.42286 .07483 L
.43226 .04092 L
.437 .02832 L
.43968 .02284 L
.44218 .01892 L
.44348 .01737 L
.44417 .01669 L
.44489 .01608 L
.44553 .01563 L
.44623 .01524 L
.44746 .01481 L
Mistroke
.44865 .01472 L
.44993 .01497 L
.45115 .01557 L
.45225 .01641 L
.45362 .01786 L
.4549 .01961 L
.45778 .02506 L
.46032 .03157 L
.46303 .04029 L
.46839 .06297 L
.47332 .08994 L
.48445 .16968 L
.50463 .35224 L
.51372 .43324 L
.52347 .5076 L
.52906 .54184 L
.53416 .56673 L
.53661 .57645 L
.53928 .58531 L
.54182 .59205 L
.54414 .59683 L
.54647 .6003 L
.5477 .60159 L
.54899 .60256 L
.55012 .60308 L
.55117 .60331 L
.55232 .60327 L
.55354 .60291 L
.55424 .60255 L
.55489 .60213 L
.55634 .60085 L
.55763 .59937 L
.55898 .59744 L
.56393 .58754 L
.5691 .57288 L
.57393 .55589 L
.58494 .50909 L
.60353 .42116 L
.62378 .33749 L
.63478 .30295 L
.64497 .2786 L
.65471 .26205 L
.66015 .25551 L
.6625 .25326 L
.66508 .25117 L
.66744 .24959 L
.66998 .24826 L
.67137 .24768 L
.67265 .24724 L
.67394 .24689 L
Mistroke
.67514 .24664 L
.67624 .24648 L
.67742 .24638 L
.67859 .24634 L
.67989 .24639 L
.681 .24649 L
.68221 .24667 L
.68334 .2469 L
.68438 .24717 L
.68703 .24807 L
.68948 .24918 L
.69506 .25264 L
.69998 .2567 L
.70522 .26195 L
.72662 .29081 L
.74679 .32289 L
.76516 .34928 L
.77019 .35523 L
.77559 .36076 L
.78025 .36473 L
.78533 .36812 L
.7881 .36953 L
.79068 .37053 L
.79194 .37091 L
.79314 .3712 L
.79438 .37144 L
.79504 .37153 L
.79575 .37161 L
.7969 .37168 L
.79816 .37169 L
.79948 .37161 L
.80071 .37145 L
.80186 .37124 L
.80291 .37098 L
.80525 .37019 L
.80796 .3689 L
.81045 .36734 L
.81609 .36248 L
.82105 .35657 L
.82632 .34857 L
.83575 .32953 L
.84588 .30208 L
.86419 .23458 L
.88393 .14346 L
.89435 .09491 L
.90007 .07067 L
.90546 .0506 L
.91066 .03462 L
.91339 .02786 L
.91628 .02212 L
Mistroke
.91871 .01848 L
.92009 .01694 L
.92138 .01585 L
.92257 .01517 L
.92365 .01481 L
.9249 .01472 L
.92608 .01496 L
.92718 .01547 L
.92839 .01636 L
.92965 .01767 L
.93084 .01924 L
.93296 .02292 L
.93524 .02812 L
.94044 .04485 L
.9452 .06607 L
.95288 .11149 L
.96001 .16454 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[16]:=",
  ImageSize->{286, 176.688},
  ImageMargins->{{28, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004N0000/2000`400?l00000o`00003oo`3ooolO
0?ooo`00o`3ooolO0?ooo`00SP3oool00`000000oooo0?ooo`2=0?ooo`00SP3oool00`000000oooo
0?ooo`2=0?ooo`00QP3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo08d0oooo0026
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0S@3oool007h0oooo100000040?oo
o`030000003oool0oooo00D0oooo0`00001b0?ooo`<00000603oool007h0oooo00@000000?ooo`3o
ool00000103oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo0780oooo00@000000?oo
o`3oool000005`3oool007d0oooo00D000000?ooo`3oool0oooo000000040?ooo`030000003oool0
oooo00D0oooo00<000000?ooo`3oool0L@3oool01@000000oooo0?ooo`3oool0000001L0oooo001l
0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool00P0000070?ooo`030000003oool0
oooo0700oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0D0?ooo`00O03oool00`00
0000oooo0?ooo`040?ooo`040000003oool0oooo000000L0oooo00<000000?ooo`3oool0K`3oool0
0`000000oooo0?ooo`040?ooo`030000003oool0oooo01@0oooo001l0?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`1_0?ooo`030000003oool0oooo
00D0oooo00<000000?ooo`3oool04`3oool007/0oooo00<000000?ooo`3oool01P3oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo06l0oooo00<000000?ooo`3oool01@3oool00`000000
oooo0?ooo`0C0?ooo`00N`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00L0oooo
00<000000?ooo`3oool0K`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0180oooo
001k0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01`3oool2000006l0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`0B0?ooo`00N`3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0KP3oool00`000000oooo0?ooo`07
0?ooo`030000003oool0oooo0180oooo001j0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`1^0?ooo`030000003oool0oooo00L0oooo00<000000?oo
o`3oool04P3oool007X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`060?ooo`03
0000003oool0oooo06h0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`0B0?ooo`00
NP3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0
K@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo0140oooo001j0?ooo`030000003o
ool0oooo00P0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`1]0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool04@3oool007T0oooo00<000000?ooo`3oool02@3oool0
0`000000oooo0?ooo`060?ooo`030000003oool0oooo06`0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0A0?ooo`00N@3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool0K03oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
0140oooo001i0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool01@3oool2000006d0
oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`0A0?ooo`00N@3oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0K03oool00`000000oooo
0?ooo`0:0?ooo`030000003oool0oooo0140oooo001i0?ooo`030000003oool0oooo00X0oooo00<0
00000?ooo`3oool01@3oool00`000000oooo0?ooo`1/0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0403oool007P0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`05
0?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0@
0?ooo`00N03oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0J`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo0100oooo001h0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1[0?ooo`03
0000003oool0oooo00`0oooo00<000000?ooo`3oool0403oool007P0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool0
303oool00`000000oooo0?ooo`0@0?ooo`00M`3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo00@0oooo00<000000?ooo`3oool0J`3oool00`000000oooo0?ooo`0<0?ooo`030000003o
ool0oooo0100oooo001g0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0103oool2
000006`0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0?0?ooo`00M`3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00<0oooo00<000000?ooo`3oool0JP3oool00`00
0000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo001g0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1Z0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool03`3oool007L0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool03P3oool00`000000oooo
0?ooo`0?0?ooo`00MP3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0JP3oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo00l0oooo001f
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`1Y
0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool03`3oool007H0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo06T0oooo00<000000?oo
o`3oool03`3oool00`000000oooo0?ooo`0?0?ooo`00MP3oool00`000000oooo0?ooo`0?0?ooo`03
0000003oool0oooo00<0oooo00<000000?ooo`3oool0J@3oool00`000000oooo0?ooo`0@0?ooo`03
0000003oool0oooo00h0oooo001f0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0
0`3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0
3P3oool007D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`020?ooo`800000JP3o
ool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo00h0oooo001e0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`1X0?ooo`030000003oool0
oooo0140oooo00<000000?ooo`3oool03P3oool007D0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo06P0oooo00<000000?ooo`3oool04@3oool00`00
0000oooo0?ooo`0>0?ooo`00M@3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0080
oooo00<000000?ooo`3oool0J03oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00h0
oooo001d0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool00P3oool00`000000oooo
0?ooo`1X0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool03P3oool007@0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo06P0oooo00<0
00000?ooo`3oool04P3oool00`000000oooo0?ooo`0=0?ooo`00M03oool00`000000oooo0?ooo`05
0?ooo`<000000`3oool00`000000oooo0?ooo`0200000080oooo00<000000?ooo`3oool00P3oool0
0`000000oooo0?ooo`1W0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3oool03@3oool0
07@0oooo00<000000?ooo`3oool01@3oool010000000oooo0?ooo`0000040?ooo`040000003oool0
oooo00000080oooo00D000000?ooo`3oool0oooo0000001Y0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool03@3oool007@0oooo00<000000?ooo`3oool0100000001@3oool000000?oo
o`3oool0000000L0oooo00@000000?ooo`3oool000000`3oool3000006L0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`0=0?ooo`00L`3oool00`000000oooo0?ooo`060?ooo`040000
003oool0oooo000000@0oooo00@000000?ooo`3oool000000P3oool01@000000oooo0?ooo`3oool0
000006T0oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`0=0?ooo`00L`3oool00`00
0000oooo0?ooo`060?ooo`040000003oool0oooo000000@0oooo0`0000030?ooo`050000003oool0
oooo0?ooo`000000J03oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo00d0oooo001c
0?ooo`030000003oool0oooo00H0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?oo
o`020?ooo`050000003oool0oooo0?ooo`000000J03oool00`000000oooo0?ooo`0D0?ooo`030000
003oool0oooo00d0oooo001c0?ooo`030000003oool0oooo00L0oooo0P0000060?ooo`<000000P3o
ool01@000000oooo0?ooo`3oool0000006P0oooo00<000000?ooo`3oool0503oool00`000000oooo
0?ooo`0=0?ooo`00LP3oool00`000000oooo0?ooo`0E0?ooo`050000003oool0oooo0?ooo`000000
J03oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo00`0oooo001b0?ooo`030000003o
ool0oooo01D0oooo00D000000?ooo`3oool0oooo0000001W0?ooo`030000003oool0oooo01H0oooo
00<000000?ooo`3oool0303oool00780oooo00<000000?ooo`3oool05@3oool01@000000oooo0?oo
o`3oool0000006L0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0<0?ooo`00LP3o
ool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00800000IP3oool00`000000oooo0?oo
o`0F0?ooo`030000003oool0oooo00`0oooo001a0?ooo`030000003oool0oooo01L0oooo00@00000
0?ooo`3oool00000I`3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo00`0oooo001a
0?ooo`030000003oool0oooo01L0oooo00@000000?ooo`3oool00000IP3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo00`0oooo001a0?ooo`030000003oool0oooo01L0oooo00@00000
0?ooo`3oool00000IP3oool00`000000oooo0?ooo`0G0?ooo`030000003oool0oooo00`0oooo001a
0?ooo`030000003oool0oooo01L0oooo00@000000?ooo`3oool00000IP3oool00`000000oooo0?oo
o`0G0?ooo`030000003oool0oooo00`0oooo001`0?ooo`030000003oool0oooo01P0oooo00@00000
0?ooo`3oool00000IP3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo00/0oooo001`
0?ooo`030000003oool0oooo01P0oooo00@000000?ooo`3oool00000I@3oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo00/0oooo001`0?ooo`030000003oool0oooo01P0oooo00@00000
0?ooo`3oool00000I@3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo00/0oooo001`
0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool00P00001T0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool02`3oool006l0oooo00<000000?ooo`3oool06@3oool01000
0000oooo0?ooo`00001U0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool02`3oool0
06l0oooo00<000000?ooo`3oool06P3oool00`000000oooo0000001T0?ooo`030000003oool0oooo
01X0oooo00<000000?ooo`3oool02`3oool006l0oooo00<000000?ooo`3oool06P3oool00`000000
oooo0000001T0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02`3oool006l0oooo
00<000000?ooo`3oool06P3oool00`000000oooo0000001T0?ooo`030000003oool0oooo01X0oooo
00<000000?ooo`3oool02`3oool006h0oooo00<000000?ooo`3oool06`3oool00`000000oooo0000
001T0?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool02P3oool006h0oooo00<00000
0?ooo`3oool06`3oool00`000000oooo0000001S0?ooo`030000003oool0oooo01`0oooo00<00000
0?ooo`3oool02P3oool003X0oooo1000000`0?ooo`030000003oool0oooo01/0oooo00<000000?oo
o`000000;`3oool400000300oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0:0?oo
o`00>03oool2000000@0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`0K0?ooo`04
0000003oool00000000002d0oooo0P0000030?ooo`800000;P3oool00`000000oooo0?ooo`0L0?oo
o`030000003oool0oooo00X0oooo000g0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0:`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0000002`0oooo0P0000070?ooo`03
0000003oool0oooo02/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0:0?ooo`00
=P3oool00`000000oooo0?ooo`070?ooo`800000:`3oool00`000000oooo0?ooo`0L0?ooo`030000
003oool0000002`0oooo00<000000?ooo`3oool01`3oool2000002X0oooo00<000000?ooo`3oool0
7@3oool00`000000oooo0?ooo`0:0?ooo`00=@3oool00`000000oooo0?ooo`0:0?ooo`030000003o
ool0oooo02P0oooo00<000000?ooo`3oool0703oool00`000000oooo0000000[0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0M0?ooo`030000003o
ool0oooo00X0oooo000d0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool09`3oool0
0`000000oooo0?ooo`0M0?ooo`800000:P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0
oooo02L0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0:0?ooo`004@3oool20000
0240oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
01d0oooo100000040?ooo`030000003oool0oooo01h0oooo0P00000W0?ooo`@000003P3oool00`00
0000oooo0?ooo`0H0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool05`3oool20000
00D0oooo00<000000?ooo`3oool02@3oool00100oooo00@000000?ooo`3oool000007`3oool00`00
0000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo01h0oooo0P00000X0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool0603oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo
01L0oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`090?ooo`002`3oool40000
00050?ooo`000000oooo0?ooo`0000007`3oool00`000000oooo00000003000000030?ooo`000000
000000<000001P3oool00`000000oooo0?ooo`0G0?ooo`@000000`3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo01h0oooo0P00000X0?ooo`800000403oool00`000000oooo0?ooo`0D
0?ooo`D000002`3oool00`000000oooo0?ooo`0G0?ooo`040000003oool0oooo000000@0oooo00<0
00000?ooo`3oool02@3oool00100oooo00@000000?ooo`3oool000007P3oool00`000000oooo0?oo
o`050?ooo`040000003oool0oooo000000P0oooo00<000000?ooo`3oool07@3oool00`000000oooo
0?ooo`020?ooo`030000003oool0oooo01l0oooo0`00000V0?ooo`030000003oool000000140oooo
00<000000?ooo`3oool04`3oool010000000oooo0?ooo`00000<0?ooo`030000003oool0oooo01L0
oooo00@000000?ooo`3oool00000103oool00`000000oooo0?ooo`090?ooo`00403oool3000001h0
oooo00<000000?ooo`3oool01`3oool00`000000oooo000000080?ooo`030000003oool0oooo01h0
oooo00D000000?ooo`3oool0oooo0000000Q0?ooo`8000009`3oool010000000oooo0?ooo`00000A
0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`000000303oool00`000000oooo0?ooo`0G
0?ooo`<000001@3oool00`000000oooo0?ooo`090?ooo`00403oool010000000oooo0?ooo`00000M
0?ooo`030000003oool0oooo00P0oooo0P0000090?ooo`030000003oool0oooo01X0oooo00@00000
0?ooo`3oool000000`3oool00`000000oooo0?ooo`0O0?ooo`8000009P3oool200000080oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`0D0?ooo`8000002`3oool00`000000oooo0?oo
o`0H0?ooo`040000003oool0oooo000000@0oooo00<000000?ooo`3oool02@3oool00140oooo0P00
000M0?ooo`030000003oool0oooo00X0oooo00<000000?ooo`3oool0203oool00`000000oooo0?oo
o`0J0?ooo`800000103oool00`000000oooo0?ooo`0O0?ooo`8000009P3oool010000000oooo0000
0000000C0?ooo`030000003oool0oooo01@0oooo00<000000?ooo`3oool02@3oool00`000000oooo
0?ooo`0I0?ooo`8000001@3oool00`000000oooo0?ooo`090?ooo`00<03oool00`000000oooo0?oo
o`0E0?ooo`030000003oool0oooo01l0oooo00<000000?ooo`3oool0803oool2000002H0oooo00<0
00000?ooo`3oool05@3oool00`000000oooo0?ooo`0O0?ooo`030000003oool0oooo0200oooo00<0
00000?ooo`3oool02@3oool002l0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0N
0?ooo`030000003oool0oooo0200oooo0P00000V0?ooo`030000003oool0oooo01H0oooo00<00000
0?ooo`3oool07P3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo00P0oooo000_0?oo
o`030000003oool0oooo01P0oooo00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0Q0?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?ooo`0M0?oo
o`030000003oool0oooo0280oooo00<000000?ooo`3oool0203oool002l0oooo00<000000?ooo`3o
ool0603oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo0240oooo00<000000?ooo`3o
ool08`3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01`0oooo00<000000?ooo`3o
ool08P3oool00`000000oooo0?ooo`080?ooo`00;P3oool00`000000oooo0?ooo`0J0?ooo`030000
003oool0oooo01/0oooo00<000000?ooo`3oool08P3oool00`000000oooo0?ooo`0S0?ooo`030000
003oool0oooo01T0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo00P0oooo00030?ooool00000600000040?ooo`001@3oool01@000000oooo0?ooo`3o
ool0000000D0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`080?ooo`030000003o
ool0oooo00H0oooo0P00000:0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01@3o
ool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0
1`3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
0P3oool00`000000oooo0?ooo`080?ooo`030000003oool0oooo00L0oooo00D000000?ooo`3oool0
oooo000000060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0203oool00`000000
oooo0?ooo`070?ooo`030000003oool0oooo00@0oooo00090?ooo`030000003oool0oooo0240oooo
00<000000?ooo`3oool07@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo02<0oooo
00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`0M0?ooo`030000003oool0oooo01L0oooo
00<000000?ooo`3oool0;`3oool000T0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0N0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?oo
o`0Q0?ooo`030000003oool0oooo01d0oooo00<000000?ooo`3oool05`3oool00`000000oooo0?oo
o`0_0?ooo`002P3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo01h0oooo00<00000
0?ooo`3oool05P3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0240oooo00<00000
0?ooo`3oool07P3oool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo0300oooo000:0?oo
o`030000003oool0oooo01l0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0E0?oo
o`030000003oool0oooo02@0oooo00<000000?ooo`3oool0803oool00`000000oooo0?ooo`0P0?oo
o`030000003oool0oooo01@0oooo00<000000?ooo`3oool0<03oool000X0oooo00<000000?ooo`3o
ool07`3oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo01<0oooo00<000000?ooo`3o
ool09@3oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3o
ool0503oool00`000000oooo0?ooo`0`0?ooo`002P3oool00`000000oooo0?ooo`0O0?ooo`030000
003oool0oooo0280oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`0U0?ooo`800000
803oool00`000000oooo0?ooo`0R0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
<@3oool000X0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0S0?ooo`030000003o
ool0oooo0140oooo00<000000?ooo`3oool09P3oool200000200oooo00<000000?ooo`3oool08`3o
ool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo0340oooo000:0?ooo`030000003oool0
oooo01h0oooo00<000000?ooo`3oool0903oool00`000000oooo0?ooo`0@0?ooo`030000003oool0
oooo02H0oooo0P00000P0?ooo`030000003oool0oooo02@0oooo00<000000?ooo`3oool0403oool0
0`000000oooo0?ooo`0a0?ooo`002P3oool00`000000oooo0?ooo`0N0?ooo`030000003oool0oooo
02D0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`0W0?ooo`800000803oool00`00
0000oooo0?ooo`0T0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0<P3oool000X0
oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00h0
oooo00<000000?ooo`3oool09`3oool2000001l0oooo00<000000?ooo`3oool09P3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo0380oooo000;0?ooo`030000003oool0oooo01`0oooo
00<000000?ooo`3oool09`3oool00`000000oooo0?ooo`0<0?ooo`030000003oool0oooo02P0oooo
0P00000O0?ooo`030000003oool0oooo02H0oooo00<000000?ooo`3oool03@3oool00`000000oooo
0?ooo`0c0?ooo`002`3oool00`000000oooo0?ooo`0L0?ooo`030000003oool0oooo02P0oooo00<0
00000?ooo`3oool02P3oool00`000000oooo0?ooo`0Y0?ooo`8000007`3oool00`000000oooo0?oo
o`0W0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool0=03oool000/0oooo00<00000
0?ooo`3oool0703oool00`000000oooo0?ooo`0Y0?ooo`030000003oool0oooo00T0oooo00<00000
0?ooo`3oool0:@3oool2000001l0oooo00<000000?ooo`3oool0:03oool2000000X0oooo00<00000
0?ooo`3oool0=@3oool000/0oooo00<000000?ooo`3oool0703oool00`000000oooo0?ooo`0Z0?oo
o`8000001`3oool2000002`0oooo0P00000N0?ooo`030000003oool0oooo02/0oooo00<000000?oo
o`3oool01`3oool00`000000oooo0?ooo`0e0?ooo`002`3oool00`000000oooo0?ooo`0K0?ooo`03
0000003oool0oooo02d0oooo00<000000?ooo`3oool00`3oool2000002d0oooo0`00000M0?ooo`03
0000003oool0oooo02`0oooo0P0000050?ooo`800000>03oool000/0oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`0^0?ooo`D00000;`3oool00`000000oooo0000000M0?ooo`030000
003oool0oooo02h0oooo1@00000j0?ooo`002`3oool00`000000oooo0?ooo`0K0?ooo`030000003o
ool0oooo0680oooo00<000000?ooo`0000007@3oool00`000000oooo0?ooo`1]0?ooo`002`3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0680oooo00<000000?ooo`0000007@3oool0
0`000000oooo0?ooo`1]0?ooo`00303oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo
06<0oooo00<000000?ooo`000000703oool00`000000oooo0?ooo`1^0?ooo`00303oool00`000000
oooo0?ooo`0I0?ooo`030000003oool0oooo06<0oooo00<000000?ooo`000000703oool00`000000
oooo0?ooo`1^0?ooo`00303oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo06<0oooo
00<000000?ooo`000000703oool00`000000oooo0?ooo`1^0?ooo`00303oool00`000000oooo0?oo
o`0I0?ooo`030000003oool0oooo06<0oooo00<000000?ooo`000000703oool00`000000oooo0?oo
o`1^0?ooo`00303oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo06<0oooo0P000000
0`3oool000000?ooo`0J0?ooo`030000003oool0oooo06h0oooo000<0?ooo`030000003oool0oooo
01P0oooo00<000000?ooo`3oool0I03oool010000000oooo0?ooo`00000J0?ooo`030000003oool0
oooo06l0oooo000<0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0I03oool01000
0000oooo0?ooo`00000J0?ooo`030000003oool0oooo06l0oooo000=0?ooo`030000003oool0oooo
01L0oooo00<000000?ooo`3oool0I03oool010000000oooo0?ooo`00000J0?ooo`030000003oool0
oooo06l0oooo000=0?ooo`030000003oool0oooo01L0oooo00<000000?ooo`3oool0I03oool01000
0000oooo0?ooo`00000J0?ooo`030000003oool0oooo06l0oooo000=0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0I@3oool010000000oooo0?ooo`00000I0?ooo`030000003oool0
oooo0700oooo000=0?ooo`030000003oool0oooo01H0oooo00<000000?ooo`3oool0I@3oool01000
0000oooo0?ooo`00000I0?ooo`030000003oool0oooo0700oooo000=0?ooo`030000003oool0oooo
01H0oooo00<000000?ooo`3oool0I@3oool01@000000oooo0?ooo`3oool0000001P0oooo00<00000
0?ooo`3oool0L03oool000d0oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`1U0?oo
o`8000000P3oool00`000000oooo0?ooo`0F0?ooo`030000003oool0oooo0700oooo000=0?ooo`03
0000003oool0oooo01D0oooo00<000000?ooo`3oool0IP3oool01@000000oooo0?ooo`3oool00000
01P0oooo00<000000?ooo`3oool0L03oool000d0oooo00<000000?ooo`3oool05@3oool00`000000
oooo0?ooo`1V0?ooo`050000003oool0oooo0?ooo`0000005`3oool00`000000oooo0?ooo`1a0?oo
o`003P3oool00`000000oooo0?ooo`0D0?ooo`030000003oool0oooo06H0oooo00D000000?ooo`3o
ool0oooo0000000G0?ooo`030000003oool0oooo0740oooo000>0?ooo`030000003oool0oooo01@0
oooo00<000000?ooo`3oool0IP3oool01@000000oooo0?ooo`3oool0000001L0oooo00<000000?oo
o`3oool0L@3oool000h0oooo00<000000?ooo`3oool0503oool00`000000oooo0?ooo`1V0?ooo`05
0000003oool0oooo0?ooo`0000005`3oool00`000000oooo0?ooo`1a0?ooo`003P3oool00`000000
oooo0?ooo`0C0?ooo`030000003oool0oooo06L0oooo00<000000?ooo`3oool00P3oool00`000000
oooo0?ooo`0D0?ooo`030000003oool0oooo0740oooo000>0?ooo`030000003oool0oooo01<0oooo
00<000000?ooo`3oool0E@3oool3000000<0oooo00<000000?ooo`3oool00P0000070?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool04`3oool00`000000oooo0?ooo`1b0?ooo`003P3o
ool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo05D0oooo00@000000?ooo`3oool00000
103oool010000000oooo0?ooo`0000060?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`1b0?ooo`003P3oool00`000000oooo0?ooo`0C0?ooo`030000
003oool0oooo05D0oooo00@000000?ooo`3oool000001`3oool00`000000oooo0?ooo`040?ooo`<0
00000P3oool00`000000oooo0?ooo`0C0?ooo`030000003oool0oooo0780oooo000>0?ooo`030000
003oool0oooo01<0oooo00<000000?ooo`3oool0E@3oool010000000oooo0?ooo`0000040?ooo`04
0000003oool0oooo000000H0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo0780oooo000?0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3o
ool0EP3oool010000000oooo0?ooo`0000040?ooo`<000001`3oool00`000000oooo0?ooo`020?oo
o`030000003oool0oooo0180oooo00<000000?ooo`3oool0L`3oool000l0oooo00<000000?ooo`3o
ool04@3oool00`000000oooo0?ooo`1F0?ooo`040000003oool0oooo000000D0oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0180oooo00<000000?oo
o`3oool0L`3oool000l0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`1G0?ooo`80
00001P3oool3000000H0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0A0?ooo`03
0000003oool0oooo07<0oooo000?0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
J03oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool0
L`3oool000l0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`1X0?ooo`030000003o
ool0oooo00<0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`1c0?ooo`003`3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo06T0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0@0?ooo`030000003oool0oooo07@0oooo000?0?ooo`030000003oool0oooo
0100oooo00<000000?ooo`3oool0J@3oool2000000@0oooo00<000000?ooo`3oool0403oool00`00
0000oooo0?ooo`1d0?ooo`003`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo06T0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo07@0
oooo000@0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0J@3oool00`000000oooo
0?ooo`030?ooo`030000003oool0oooo0100oooo00<000000?ooo`3oool0M03oool00100oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`1Y0?ooo`030000003oool0oooo00@0oooo00<0
00000?ooo`3oool03`3oool00`000000oooo0?ooo`1d0?ooo`00403oool00`000000oooo0?ooo`0?
0?ooo`030000003oool0oooo06T0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0>
0?ooo`030000003oool0oooo07D0oooo000@0?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0JP3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo00h0oooo00<000000?oo
o`3oool0M@3oool00100oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1Z0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`1e0?ooo`00
403oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo06X0oooo00<000000?ooo`3oool0
103oool00`000000oooo0?ooo`0>0?ooo`030000003oool0oooo07D0oooo000@0?ooo`030000003o
ool0oooo00h0oooo00<000000?ooo`3oool0JP3oool2000000D0oooo00<000000?ooo`3oool03P3o
ool00`000000oooo0?ooo`1e0?ooo`004@3oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo06X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0=0?ooo`030000003oool0
oooo07H0oooo000A0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0J`3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool0MP3oool00140
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`1[0?ooo`030000003oool0oooo00D0
oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`1f0?ooo`004@3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo06/0oooo00<000000?ooo`3oool01@3oool00`000000oooo
0?ooo`0<0?ooo`030000003oool0oooo07H0oooo000B0?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0J`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00/0oooo00<0
00000?ooo`3oool0M`3oool00180oooo00<000000?ooo`3oool02P3oool00`000000oooo0?ooo`1/
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`1g
0?ooo`004P3oool00`000000oooo0?ooo`0:0?ooo`030000003oool0oooo06`0oooo00<000000?oo
o`3oool01@3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo07L0oooo000B0?ooo`03
0000003oool0oooo00X0oooo00<000000?ooo`3oool0K03oool2000000H0oooo00<000000?ooo`3o
ool02`3oool00`000000oooo0?ooo`1g0?ooo`004P3oool00`000000oooo0?ooo`0:0?ooo`030000
003oool0oooo06`0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo07P0oooo000B0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0K@3o
ool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3oool0N03o
ool001<0oooo00<000000?ooo`3oool0203oool00`000000oooo0?ooo`1]0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`1h0?ooo`004`3oool00`00
0000oooo0?ooo`080?ooo`030000003oool0oooo06d0oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`090?ooo`030000003oool0oooo07P0oooo000C0?ooo`030000003oool0oooo00P0
oooo00<000000?ooo`3oool0K@3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00L0
oooo00<000000?ooo`3oool0N@3oool001<0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`1^0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool01`3oool00`000000oooo
0?ooo`1i0?ooo`004`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo06h0oooo00<0
00000?ooo`3oool01`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo07X0oooo000C
0?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3oool0KP3oool2000000P0oooo00<00000
0?ooo`3oool01P3oool00`000000oooo0?ooo`1j0?ooo`00503oool00`000000oooo0?ooo`050?oo
o`030000003oool0oooo06l0oooo00<000000?ooo`3oool01`3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo07X0oooo000D0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0K`3oool00`000000oooo0?ooo`070?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3o
ool0N`3oool001@0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`1_0?ooo`030000
003oool0oooo00P0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`1k0?ooo`00503o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0700oooo00<000000?ooo`3oool0203o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo07/0oooo000E0?ooo`030000003oool0
oooo00<0oooo00<000000?ooo`3oool0L03oool00`000000oooo0?ooo`090?ooo`030000003oool0
oooo0080oooo00<000000?ooo`3oool0O03oool001D0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`1Y0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`020?ooo`030000003oool0oooo07`0oooo000F0?ooo`050000003oool0oooo0?oo
o`000000J`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00T0oooo00D000000?oo
o`3oool0oooo0000001o0?ooo`005`3oool3000006`0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`0:0?ooo`@00000O`3oool001P0oooo00<000000?ooo`3oool0J`3oool00`000000
oooo0?ooo`050?ooo`<000002`3oool200000800oooo00260?ooo`030000003oool0oooo00D0oooo
00<000000?ooo`3oool0S@3oool008D0oooo0P0000070?ooo`030000003oool0oooo08d0oooo0026
0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0S@3oool008h0oooo00<000000?oo
o`3oool0S@3oool00?l0oooo7`3oool00001\
\>"],
  ImageRangeCache->{{{0, 285}, {175.688, 0}} -> {-6.75626, -1.0725, \
0.0474123, 0.0122092}},
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Integrate[Cos[x], {x, a, b}]", "Input",
  CellLabel->"In[17]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(\(-Sin[a]\) + Sin[b]\)], "Output",
  CellLabel->"Out[17]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "In the ",
  StyleBox["Plot", "MR"],
  " example, the list ",
  StyleBox["{x,-2\[Pi],2\[Pi]}", "MR"],
  " indicates that the function ",
  Cell[BoxData[
      \(TraditionalForm\`sin(x + \(\@2\) \(sin(x)\))\)],
    FormatType->StandardForm],
  " is to be plotted over an interval as ",
  Cell[BoxData[
      \(TraditionalForm\`x\)],
    FormatType->StandardForm],
  " takes on values from ",
  Cell[BoxData[
      \(TraditionalForm\`\(-2\) \[Pi]\)],
    FormatType->StandardForm],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`2  \[Pi]\)],
    FormatType->StandardForm],
  ". The ",
  StyleBox["Integrate", "MR"],
  " expression above is equivalent to the mathematical expression ",
  Cell[BoxData[
      \(TraditionalForm\`\[Integral]\_a\%b\( cos(x)\)\ \[DifferentialD]x\)],
    FormatType->StandardForm],
  ". "
}], "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has very powerful list-manipulating capabilities that will be explored in \
detail in Chapter 3."
}], "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[TextData[{
  "When you end an expression with a semicolon (",
  StyleBox[";", "MR"],
  "), ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " computes its value but does not display it. This is very helpful when the \
result of the expression would be very long and you do not need to see it. In \
the following example, we first create a list of the integers from 1 to \
10,000, suppressing their display with the semicolon, and then compute their \
sum and average."
}], "Text",
  CellTags->{"i:44", "b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(\(nums = Range[10000];\)\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell["Apply[Plus, nums]", "Input",
  CellLabel->"In[19]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(50005000\)], "Output",
  CellLabel->"Out[19]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(%\/Length[nums]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(10001\/2\)], "Output",
  CellLabel->"Out[20]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "An expression can be entered on multiple lines, but only if ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can tell that it is not finished after the first line. For example, you \
can enter ",
  StyleBox["3*", "MR"],
  " on one line and ",
  StyleBox["4", "MR"],
  " on the next."
}], "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell["\<\
3 *

4\
\>", "Input",
  CellLabel->"In[21]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(12\)], "Output",
  CellLabel->"Out[21]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "But you cannot enter ",
  StyleBox["3", "MR"],
  " on the first line and ",
  StyleBox["*4", "MR"],
  " on the second."
}], "TextFirst",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell["\<\
3
*4\
\>", "Input",
  CellLabel->"In[22]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(3\)], "Output",
  CellLabel->"Out[22]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell["If you use parentheses, you can avoid this problem.", "TextFirst",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell["\<\
(3
 *4)\
\>", "Input",
  CellLabel->"In[23]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(12\)], "Output",
  CellLabel->"Out[23]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[TextData[{
  "With the notebook interface, you can input as many lines as you like \
within an input cell; ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will evaluate them all when you enter \[ShiftKey]\[LeftModified]\
\[EnterKey]\[RightModified] still obeying the rules stated above for any \
incomplete lines."
}], "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[TextData[{
  "Finally, you can enter a ",
  StyleBox["comment",
    FontSlant->"Italic"],
  " \[Dash] some words that are not evaluated",
  " \[Dash] ",
  "by entering the words between ",
  StyleBox["(*", "MR"],
  " and ",
  StyleBox["*)", "MR"],
  "."
}], "Text",
  PageBreakAbove->True,
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{

Cell["\<\
D[Sin[x],    (* differentiate Sin[x]   *) 
    {x, 1}]  (* with respect to x once *) \
\>", "Input",
  CellLabel->"In[24]:=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[BoxData[
    \(Cos[x]\)], "Output",
  CellLabel->"Out[24]=",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Alternate input syntax", "Subsection",
  CellTags->{"b:1.2", "IPM3.1.2.3", "c:12"}],

Cell[TextData[{
  "There are several different ways to write expressions in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Usually, you will simply use the traditional notation, ",
  StyleBox["fun", "IT"],
  StyleBox["[", "MR"],
  StyleBox["x", "IT"],
  StyleBox["]", "MR"],
  ", for example. But you should be aware of several alternatives to this \
syntax that are widely used."
}], "TextFirst",
  CellTags->{"i:45", "b:1.2", "IPM3.1.2.3"}],

Cell["\<\
Here is an example using the standard function notation for writing a \
function with one argument.\
\>", "Text",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(N[\[Pi]]\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[BoxData[
    \(3.141592653589793`\)], "Output",
  CellLabel->"Out[25]=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell["This uses a prefix operator.", "TextFirst",
  CellTags->{"i:46", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(N@\[Pi]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[BoxData[
    \(3.141592653589793`\)], "Output",
  CellLabel->"Out[26]=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell["Here is a postfix operator notation.", "TextFirst",
  CellTags->{"i:47", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Pi] // N\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[BoxData[
    \(3.141592653589793`\)], "Output",
  CellLabel->"Out[27]=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell[TextData[{
  "For functions with two arguments, you can use an infix notation. The \
following expression is identical to ",
  StyleBox["N[\[Pi],30]", "MR"],
  "."
}], "Text",
  CellTags->{"i:48", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Pi]~N~30\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[BoxData[
    \(3.141592653589793238462643383279502884197167382228`30. \)], "Output",
  CellLabel->"Out[28]=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell[TextData[{
  "Finally, many people prefer to use a more traditional syntax when entering \
and working with mathematical expressions. You can compute an integral using \
standard ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " syntax."
}], "Text",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[1/Sin[x], x]\)], "Input",
  CellLabel->"In[29]:=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[BoxData[
    \(\(-Log[Cos[x\/2]]\) + Log[Sin[x\/2]]\)], "Output",
  CellLabel->"Out[29]=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell["\<\
The same integral, represented in a more traditional manner, can be entered \
from palettes or keyboard shortcuts.\
\>", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"i:49", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\[Integral]1\/Sin[x]\ \[DifferentialD]x\)], "Input",
  CellLabel->"In[30]:=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[BoxData[
    \(\(-Log[Cos[x\/2]]\) + Log[Sin[x\/2]]\)], "Output",
  CellLabel->"Out[30]=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell[TextData[{
  "Many mathematical functions have traditional symbols associated with their \
operations and when available, these can be used instead of the fully \
spelled-out names. For example, you can compute the intersection of two sets \
using the ",
  StyleBox["Intersection", "MR"],
  " function."
}], "Text",
  CellTags->{"i:50", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Intersection[{a, b, c, d, e}, {b, f, a, z}]\)], "Input",
  CellLabel->"In[31]:=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[BoxData[
    \({a, b}\)], "Output",
  CellLabel->"Out[31]=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell["\<\
Or you can do the same computation using more traditional notation.\
\>", "TextFirst",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \({a, b, c, d, e} \[Intersection] {b, f, a, z}\)], "Input",
  CellLabel->"In[32]:=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[BoxData[
    \({a, b}\)], "Output",
  CellLabel->"Out[32]=",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell["\<\
To learn how to enter these and other notations quickly, either from palettes \
or directly from the keyboard using shortcuts, refer to the 2D Expression \
Input section in the Front End category of the Help Browser.\
\>", "Text",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The front end and the kernel", "Subsection",
  CellTags->{"b:1.3", "IPM3.1.2.4", "c:13"}],

Cell[TextData[{
  "When you work in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " you are actually working with two separate programs. They are referred to \
as the ",
  StyleBox["front end", "TermDefinition"],
  " and the ",
  StyleBox["kernel", "TermDefinition"],
  ". The front end is the user interface. It consists of the notebooks that \
you work in together with the menu system, palettes (which are really just \
notebooks), and any element that accepts input from the keyboard or mouse. \
The kernel is the program that does the calculations. So a typical operation \
between the user (you) and ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " consists of the following steps, where the program that is invoked in \
each step is indicated in parentheses:"
}], "TextFirst",
  CellTags->{"i:51", "b:1.3", "IPM3.1.2.4"}],

Cell["enter input in the notebook (front end)", "BulletedList",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],

Cell["\<\
send input to the kernel to be evaluated by pressing \[ShiftKey]-\[EnterKey] \
(front end)\
\>", "BulletedList",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],

Cell["\<\
kernel does the computation and sends it back to the front end (kernel)\
\>", "BulletedList",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],

Cell["result is displayed in the notebook (front end)", "BulletedList",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],

Cell[TextData[{
  "There is one remaining piece that we have not yet mentioned; that is ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  ". Since the kernel and front end are two separate programs, a means of \
communication is necessary for these two programs to \
\[OpenCurlyDoubleQuote]talk\[CloseCurlyDoubleQuote] to each other. That \
communication protocol is called ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " and it comes bundled with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". It operates behind the scenes, completely transparent to the user. "
}], "Text",
  CellTags->{"i:52", "b:1.3", "IPM3.1.2.4"}],

Cell[TextData[{
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " is a very general communications protocol that is not limited to front \
end",
  " \[Dash] ",
  "kernel communication, but can also be used to set up communication between \
the front end and other programs on your computer, programs like compiled C \
and Fortran code. It can also be used to connect a kernel to a word processor \
or spreadsheet or many other programs. "
}], "Text",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],

Cell[TextData[{
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " programming is beyond the scope of this book, but if you are interested, \
there are several books and articles that discuss it (see the References at \
the end of this book)."
}], "Text",
  CellTags->{"b:1.3", "IPM3.1.2.4"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Errors", "Subsection",
  CellTags->{"b:1.4", "IPM3.1.2.5", "c:14"}],

Cell[TextData[{
  "In the course of using and programming in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", you will encounter various sorts of errors, some obvious, some very \
subtle, some easily rectified, and others not. We have already mentioned that \
it is possible to send ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " into an infinite loop from which it cannot return. In this section, we \
discuss those situations where ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " does finish the computation, but without giving you the answer you \
expected."
}], "TextFirst",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell["\<\
Perhaps the most frequent error you will make is misspelling the name of a \
function. Here is an illustration of the kind of thing that will usually \
happen in this case.\
\>", "Text",
  CellTags->{"i:53", "b:1.4", "IPM3.1.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sine[1.5]\)], "Input",
  CellLabel->"In[33]:=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    RowBox[{\(General::"spell"\), \(\(:\)\(\ \)\), "\<\"Possible spelling \
error: new symbol name \\\"\\!\\(Sine\\)\\\" is similar to existing symbols \
\\!\\({Line, Sin, Sinh}\\). \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::spell\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[33]:=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    \(Sine[1.5`]\)], "Output",
  CellLabel->"Out[33]=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]],

Cell[TextData[{
  "Whenever you type a name that is ",
  StyleBox["close",
    FontSlant->"Italic"],
  " to an existing name, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will print a warning message like the one above. You may often use such \
names intentionally, in which case these messages can be annoying. In that \
case, it is best to turn off the warnings."
}], "Text",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    \(Off[General::spell]\)], "Input",
  CellLabel->"In[34]:=",
  CellTags->{"i:54", "b:1.4", "IPM3.1.2.5"}],

Cell[TextData[{
  "Now, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will not report that function names might be misspelled; and, when it can \
not find a definition associated with a misspelled function, it returns your \
input unevaluated."
}], "Text",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Intergate[x\^2, x]\)], "Input",
  CellLabel->"In[35]:=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    \(Intergate[x\^2, x]\)], "Output",
  CellLabel->"Out[35]=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]],

Cell[TextData[{
  " You can turn these spell warnings back on by evaluating ",
  StyleBox["On[General::spell]", "MR"],
  "."
}], "Text",
  PageBreakAbove->True,
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    \(On[General::spell]\)], "Input",
  CellLabel->"In[36]:=",
  CellTags->{"i:55", "b:1.4", "IPM3.1.2.5"}],

Cell[TextData[{
  "Having your original expression returned unevaluated \[Dash] as if this \
were perfectly normal",
  " \[Dash] ",
  "is a problem you will often run into. Aside from misspelling a function \
name, or simply using a function that does not exist, another case where this \
occurs is when you give the wrong number of arguments to a function, \
especially to a user-defined function. For example, the ",
  StyleBox["BowlingScore", "MR"],
  " function takes a single list argument; if we accidentally leave out the \
list braces, then we are actually giving ",
  StyleBox["BowlingScore", "MR"],
  " 12 arguments."
}], "Text",
  CellTags->{"i:56", "b:1.4", "IPM3.1.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(BowlingScore[10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10]\)], "Input",\

  CellLabel->"In[37]:=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    \(BowlingScore[10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 
      10]\)], "Output",
  CellLabel->"Out[37]=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]],

Cell[TextData[{
  "Of course, some kinds of inputs cause genuine error messages. Syntax \
errors, as shown above, are one example. The built-in functions are designed \
to usually warn you of such errors in input. In the first example below, we \
have supplied the ",
  StyleBox["Log", "MR"],
  " function with an incorrect number of arguments (it expects one or two \
arguments only). In the second example, ",
  StyleBox["FactorInteger", "MR"],
  " operates on integers only and so the real number argument causes the \
error condition."
}], "Text",
  CellTags->{"i:57", "b:1.4", "IPM3.1.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Log[2, 16, 3]\)], "Input",
  CellLabel->"In[38]:=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    RowBox[{\(Log::"argt"\), \(\(:\)\(\ \)\), "\<\"\\!\\(Log\\) called with \
\\!\\(3\\) arguments; \\!\\(1\\) or \\!\\(2\\) arguments are expected. \\!\\(\
\\*ButtonBox[\\\"More\[Ellipsis]\\\", ButtonStyle->\\\"RefGuideLinkText\\\", \
ButtonFrame->None, ButtonData:>\\\"General::argt\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[38]:=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    \(Log[2, 16, 3]\)], "Output",
  CellLabel->"Out[38]=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[12.5]\)], "Input",
  CellLabel->"In[39]:=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    RowBox[{\(FactorInteger::"facn"\), \(\(:\)\(\ \)\), "\<\"Argument \
\\!\\(12.5`\\) in \\!\\(FactorInteger[12.5`]\\) is not an exact number. \
\\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", ButtonStyle->\\\"RefGuideLinkText\
\\\", ButtonFrame->None, ButtonData:>\\\"FactorInteger::facn\\\"]\\)\"\>"}]], \
"Message",
  CellLabel->"From In[39]:=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[BoxData[
    \(FactorInteger[12.5`]\)], "Output",
  CellLabel->"Out[39]=",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Getting help", "Subsection",
  CellTags->{"b:1.5", "IPM3.1.2.6", "c:15"}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " contains a vast array of documentation that you can access in a variety \
of ways. It is also designed so that you can create new documentation for \
your own functions and program in such a way that users of your programs can \
get help in exactly the same way as they would for ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\[CloseCurlyQuote]s built-in functions."
}], "TextFirst",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],

Cell[TextData[{
  "If you are aware of the name of a function but are unsure of its syntax or \
what it does, the easiest way to find out about it is to evaluate ",
  StyleBox["?function", "MR"],
  ". For example, here is the usage message for ",
  StyleBox["ParametricPlot", "MR"],
  "."
}], "Text",
  CellTags->{"i:58", "b:1.5", "IPM3.1.2.6"}],

Cell[CellGroupData[{

Cell["?ParametricPlot", "Input",
  CellLabel->"In[40]:=",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],

Cell[BoxData[
    RowBox[{"\<\"ParametricPlot[{fx, fy}, {u, umin, umax}] produces a \
parametric plot of a curve with x and y coordinates fx and fy generated as a \
function of t. ParametricPlot[{{fx, fy}, {gx, gy}, ... }, {u, umin, umax}] \
plots several parametric curves.\"\>", " ", 
      ButtonBox[
        StyleBox["More\[Ellipsis]",
          "SR"],
        ButtonData:>"ParametricPlot",
        Active->True,
        ButtonStyle->"RefGuideLink"]}]], "Print",
  CellTags->{"b:1.5", "Info3302590721-5746852", "IPM3.1.2.6"}]
}, Open  ]],

Cell[TextData[{
  "Also, if you were not sure of the spelling of a command (",
  StyleBox["Integrate", "MR"],
  ", for example), you could type the following to display all built-in \
functions that start with ",
  StyleBox["Integ", "MR"],
  "."
}], "Text",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],

Cell[CellGroupData[{

Cell["?Integ*", "Input",
  CellLabel->"In[41]:=",
  Editable->False,
  CellTags->{"b:1.5", "IPM3.1.2.6"}],

Cell[BoxData[GridBox[{
        {
          StyleBox["System`",
            FontFamily->"Helvetica",
            FontSize->10,
            FontWeight->"Bold"]},
        {GridBox[{
              {
                ButtonBox[
                  StyleBox["Integer",
                    "Hyperlink"],
                  
                  ButtonFunction:>(
                    Internal`PutInformation[ #, LongForm -> False]&),
                  ButtonEvaluator->Automatic,
                  ButtonData:>{"Info3302590721-1942882", "System`Integer"},
                  ButtonFrame->"None",
                  ButtonNote->"System`"], 
                ButtonBox[
                  StyleBox["IntegerExponent",
                    "Hyperlink"],
                  
                  ButtonFunction:>(
                    Internal`PutInformation[ #, LongForm -> False]&),
                  ButtonEvaluator->Automatic,
                  
                  ButtonData:>{"Info3302590721-1942882", 
                    "System`IntegerExponent"},
                  ButtonFrame->"None",
                  ButtonNote->"System`"], 
                ButtonBox[
                  StyleBox["IntegerQ",
                    "Hyperlink"],
                  
                  ButtonFunction:>(
                    Internal`PutInformation[ #, LongForm -> False]&),
                  ButtonEvaluator->Automatic,
                  ButtonData:>{"Info3302590721-1942882", "System`IntegerQ"},
                  ButtonFrame->"None",
                  ButtonNote->"System`"], 
                ButtonBox[
                  StyleBox["Integrate",
                    "Hyperlink"],
                  
                  ButtonFunction:>(
                    Internal`PutInformation[ #, LongForm -> False]&),
                  ButtonEvaluator->Automatic,
                  ButtonData:>{"Info3302590721-1942882", "System`Integrate"},
                  ButtonFrame->"None",
                  ButtonNote->"System`"], ""},
              {
                ButtonBox[
                  StyleBox["IntegerDigits",
                    "Hyperlink"],
                  
                  ButtonFunction:>(
                    Internal`PutInformation[ #, LongForm -> False]&),
                  ButtonEvaluator->Automatic,
                  
                  ButtonData:>{"Info3302590721-1942882", 
                    "System`IntegerDigits"},
                  ButtonFrame->"None",
                  ButtonNote->"System`"], 
                ButtonBox[
                  StyleBox["IntegerPart",
                    "Hyperlink"],
                  
                  ButtonFunction:>(
                    Internal`PutInformation[ #, LongForm -> False]&),
                  ButtonEvaluator->Automatic,
                  
                  ButtonData:>{"Info3302590721-1942882", 
                    "System`IntegerPart"},
                  ButtonFrame->"None",
                  ButtonNote->"System`"], 
                ButtonBox[
                  StyleBox["Integers",
                    "Hyperlink"],
                  
                  ButtonFunction:>(
                    Internal`PutInformation[ #, LongForm -> False]&),
                  ButtonEvaluator->Automatic,
                  ButtonData:>{"Info3302590721-1942882", "System`Integers"},
                  ButtonFrame->"None",
                  ButtonNote->"System`"], "", ""}
              },
            RowMinHeight->{1, 1.05}]}
        },
      RowSpacings->{2, 3},
      ColumnsEqual->True]], "Print",
  CellMargins->{{20, Inherited}, {Inherited, Inherited}},
  ShowCellLabel->False,
  CellFrameMargins->{{Inherited, Inherited}, {14, 14}},
  Background->RGBColor[0.964706, 0.929412, 0.839216],
  ButtonBoxOptions->{Active->True},
  GridBoxOptions->{ColumnAlignments->{Left}},
  CellTags->{"b:1.5", "IPM3.1.2.6"}]
}, Open  ]],

Cell[TextData[{
  "Clicking on one of these links will produce a short usage statement about \
that function. For example, if you were to click on the ",
  StyleBox["Integrate", "MR"],
  " link, here is what would be displayed in your notebook."
}], "Text",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],

Cell[BoxData[
    RowBox[{"\<\"Integrate[f, x] gives the indefinite integral of f with \
respect to x. Integrate[f, {x, xmin, xmax}] gives the definite integral of f \
with respect to x from xmin to xmax. Integrate[f, {x, xmin, xmax}, {y, ymin, \
ymax}] gives a multiple definite integral of f with respect to x and y.\"\>", 
      " ", 
      ButtonBox["More\[Ellipsis]",
        ButtonData:>"Integrate",
        Active->True,
        ButtonStyle->"RefGuideLink"]}]], "Print",
  CellMargins->{{Inherited, Inherited}, {Inherited, 6}},
  CellTags->{"b:1.5", "IPM3.1.2.6"}],

Cell[TextData[{
  "Clicking the ",
  StyleBox["More\[Ellipsis]", "SR"],
  " hyperlink would take you directly to the Help Browser where a much more \
detailed explanation of this function can be found."
}], "Text",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],

Cell[TextData[{
  "You can also get help by highlighting any ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " function and pressing the ",
  StyleBox["F1", "SR"],
  " key on your keyboard. This will take you directly into the documentation \
for that function in the Help Browser."
}], "Text",
  CellTags->{"b:1.5", "IPM3.1.2.6"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Help Browser", "Subsection",
  CellTags->{"b:1.6", "IPM3.1.2.7", "c:16"}],

Cell[TextData[{
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " contains a very useful addition to the help system called the Help \
Browser. The Help Browser allows you to search for functions easily and it \
provides extensive documentation and examples."
}], "TextFirst",
  CellTags->{"i:59", "b:1.6", "IPM3.1.2.7"}],

Cell[TextData[{
  "To start the Help Browser, select ",
  StyleBox["Help Browser\[Ellipsis]", "SR"],
  " under the ",
  StyleBox["Help", "SR"],
  " menu. You should quickly see something like the following:"
}], "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .64103 
%%ImageSize: 391 251 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
4.33681e-019 0.0025641 -0.0025641 0.0025641 [
[ 0 0 0 0 ]
[ 1 .64103 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .64103 L
0 .64103 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00256 translate
1.00256 .64359 scale
1173 string
391 251 8 [391 0 0 251 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFBFBFB
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F8F8F8
F7F7F7F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F7F7F7F8F8F8F9F9F9FBFBFBFCFCFCFDFDFD
FDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBFAFAFAF8F8F8F6F6F6F3F3F3
F2F2F2F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F2F2F2F3F3F3F6F6F6F8F8F8FAFAFAFBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFBFBFBF9F9F9F7F7F7F5F5F5F1F1F1EEEEEE
EBEBEBE8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8EBEBEBEEEEEEF1F1F1F5F5F5F7F7F7F9F9F9
FBFBFBFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFAFAFAF7F7F7F5F5F5F1F1F1EDEDEDE8E8E8
E4E4E4E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E4E4E4E8E8E8EDEDEDF1F1F1F5F5F5F7F7F7
FAFAFAFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F5F5F5F1F1F1EBEBEBE5E5E5DEDEDE
D9D9D9D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D5D5D5D9D9D9DEDEDEE5E5E5EBEBEBF1F1F1F5F5F5
F8F8F8FBFBFBFCFCFCFDFDFDFDFDFDFFFFFFFFFFFF
FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCF9F9F9F6F6F6F1F1F1EDEDEDE5E5E5DCDCDCD4D4D4
CCCCCCC7C7C7C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C7C7C7CCCCCCD4D4D4DCDCDCE5E5E5EDEDEDF1F1F1
F6F6F6F9F9F9FCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFFFEFEFEFDFDFDFDFDFDFBFBFBF8F8F8F3F3F3EEEEEEE8E8E8DEDEDED4D4D4C8C8C8
BFBFBFB8B8B8B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B8B8B8BFBFBFC8C8C8D4D4D4DEDEDEE8E8E8EEEEEE
F3F3F3F8F8F8FBFBFBFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFFFEFEFEFDFDFDFDFDFDFAFAFAF7F7F7F2F2F2EBEBEBE4E4E4D9D9D9CCCCCCBFBFBF
B3B3B3AAAAAAA6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6AAAAAAB3B3B3BFBFBFCCCCCCD9D9D9E4E4E4EBEBEB
F2F2F2F7F7F7FAFAFAFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFFFEFEFEFDFDFDFCFCFCFAFAFAF6F6F6F0F0F0E8E8E8E1E1E1D5D5D5C7C7C7B8B8B8
AAAAAA9F9F9F9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9F9F9FAAAAAAB8B8B8C7C7C7D5D5D5E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC9A9A9A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A9A9A9AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFEFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0BDBDBD9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F969696E6E6E6D3D3D39F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9FBDBDBDE9E9E9E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7EEEEEE9696969F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
969696E6E6E6E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0C2C2C2D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4E1E1E1969696C1C1C1CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDEDEDEDCBCBCBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFB969696E1E1E1D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
E1E1E1969696E0E0E0E0E0E0E0E0E0D4D4D4D9D9D9EFEFEFC9C9C9D9D9D9EFEFEFC9C9C9
ECECECE0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACACECECECECECECECECECECECECECECECECECECECECE
CECECECECECEE6E6E6969696CACACACFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFFFFFFFC8C8C8FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFB969696E6E6E6CECECECECECECECECECECECECECECECECECECECECECECECE
E6E6E6969696E0E0E0E0E0E0E0E0E0C9C9C9B3B3B3E0E0E0B3B3B3B3B3B3E0E0E0B3B3B3
C9C9C9E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0B7B7B74343436C6C6CAFAFAF5C5C5C5E5E5E939393
E0E0E07E7E7E707070707070E0E0E0E0E0E0DEDEDE7F7F7FB3B3B3C3C3C3535353505050
CDCDCDE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACACECECECECECECECECECECECE626262868686CECECE
CECECECECECEE6E6E6969696CACACAD3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3B3B3B3D3D3D3C2C2C2B2B2B2
C2C2C2D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D3D3D3D3D3D3FFFFFFC7C7C7F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9969696E6E6E6CECECECECECE8686863E3E3EAAAAAACECECECECECECECECE
E6E6E6969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EFEFEFE7E7E7E0E0E0
EFEFEFE0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0868686D9D9D97F7F7FD2D2D2D2D2D2484848
D2D2D27C7C7CE0E0E0868686D9D9D9E0E0E0E0E0E0DCDCDC858585ADADAD9A9A9A696969
BFBFBFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACACFCFCFCFCFCFCFCFCF3E3E3E3E3E3ECFCFCFCFCFCF
CFCFCFCFCFCFE7E7E7969696CACACADCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDC
DCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCEDEDED959595FFFFFFCACACA959595
CACACADCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDC
DCDCDCDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCFFFFFFC7C7C7F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9969696E7E7E7CFCFCFCFCFCFCFCFCF3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF
E7E7E7969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3EFEFEFB3B3B3
D9D9D9E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0868686D9D9D97F7F7FD2D2D2DCDCDC262626
CBCBCB7C7C7CE0E0E07F7F7FD2D2D2E0E0E0E0E0E0E0E0E0868686D2D2D20D0D0D0D0D0D
D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0707070000000000000000000000000
707070E0E0E0E0E0E0E0E0E0CACACAD4D4D4D4D4D48989893E3E3E898989D4D4D4D4D4D4
D4D4D4D4D4D4E9E9E9969696CACACADCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDC
DCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCEDEDED959595FFFFFFCACACA959595
CACACADCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDC
DCDCDCDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCFFFFFFC7C7C7F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8969696E9E9E9D4D4D4D4D4D4D4D4D4D4D4D43E3E3E3E3E3ED4D4D4D4D4D4
E9E9E9969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0B3B3B3
C9C9C9E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0868686D9D9D97F7F7FD2D2D2D7D7D7373737
CECECE7C7C7CE0E0E0828282D5D5D58989897070703B3B3B9F9F9F737373DEDEDEE0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0707070000000000000707070
E0E0E0E0E0E0E0E0E0E0E0E0CACACADFDFDFDFDFDF6767673E3E3EB6B6B6DFDFDFDFDFDF
DFDFDFDFDFDFEFEFEF969696CACACAE7E7E7E6E6E6E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7F2F2F2959595FFFFFFCACACA959595
CACACAE6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E6E6E6E6E6E6E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E7E7E7E6E6E6FFFFFFC5C5C5F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6969696EFEFEFDFDFDFDFDFDFDFDFDFDFDFDF6565653E3E3EB8B8B8DFDFDF
EFEFEF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
EFEFEFE0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E08E8E8E1C1C1CD9D9D9A4A4A49E9E9EB7B7B7545454
E0E0E0838383E0E0E0717171E0E0E0808080E0E0E0696969AFAFAF696969BFBFBFE0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0707070E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACAEDEDEDEDEDEDEDEDED3E3E3E3E3E3EEDEDEDEDEDED
EDEDEDEDEDEDF6F6F6969696CACACAEAEAEAE9E9E9EAEAEAEAEAEAEAEAEAEAEAEAEAEAEA
EAEAEAEAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEA
EAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAF4F4F4959595FFFFFFCACACA959595
CACACAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEA
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9FFFFFFC5C5C5F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4969696F6F6F6EDEDEDEDEDEDEDEDED3E3E3E3E3E3EEDEDEDEDEDEDEDEDED
F6F6F6969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3
D9D9D9E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0BABABADCDCDCDEDEDE7D7D7D737373D9D9D9
E0E0E0ACACAC707070A7A7A7E0E0E0ACACAC707070767676DCDCDCD2D2D2ABABABE0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACAF8F8F8F8F8F8F8F8F8F8F8F86D6D6D9D9D9DF8F8F8
F8F8F8F8F8F8FBFBFB969696CACACAEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDF1F1F1EDEDEDF6F6F6F1F1F1EDEDED
F1F1F1EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDFFFFFFC3C3C3F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0969696FBFBFBF8F8F8F8F8F89D9D9D3E3E3ECACACAF8F8F8F8F8F8F8F8F8
FBFBFB969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C9C9C9
D4D4D4E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF969696CACACAF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7FFFFFFC2C2C2EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEF969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0B6B6B6DADADADADADADADADADADADADADADADADADADADADA
DADADADADADAC6C6C6B8B8B8C5C5C5DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBD1D1D1D0D0D0EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEB8B8B8C7C7C7DADADADADADADADADADADADADADADADADADADADADADADADA
C6C6C6B8B8B8E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9EEFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE9E9E9E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECE3E3E3CACACAC5C5C5C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C4C4C4
E1E1E1EBEBEB8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF4F4F4F1F1F1F2F2F2F2F2F2F2F2F2F2F2F2F2F2F2F1F1F1F0F0F0
F1F1F1DDDDDD8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF0F0F0E9E9E9EAEAEAEBEBEBEAEAEAE9E9E9E8E8E8E6E6E6E5E5E5
F0F0F0CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF1F1F1EBEBEBECECECECECECE2E2E2EAEAEAEAEAEAE8E8E8E6E6E6
F1F1F1CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF2F2F2EDEDEDEEEEEEEEEEEEC8C8C8C8C8C8E9E9E9E9E9E9E8E8E8
F2F2F2CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF2F2F2EEEEEEE4E4E4C8C8C8D1D1D1C8C8C8C8C8C8E1E1E1E9E9E9
F2F2F2CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF3F3F3EFEFEFC8C8C8C8C8C8ECECECEAEAEAC8C8C8C8C8C8EAEAEA
F2F2F2CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF4F4F4F0F0F0DCDCDCEFEFEFEDEDEDEBEBEBEBEBEBD9D9D9EAEAEA
F2F2F2CBCBCB8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF4F4F4F1F1F1F0F0F0EFEFEFEFEFEFEBEBEBEBEBEBEBEBEBEBEBEB
F2F2F2CFCFCF8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF6F6F6F5F5F5F2F2F2F0F0F0EFEFEFEEEEEEEEEEEEECECECEBEBEB
F2F2F2D6D6D68A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF9F9F9F3F3F3EFEFEFEDEDEDECECECEBEBEBEAEAEAE9E9E9E7E7E7
F5F5F5E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFDFDFDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFEBEBEB8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBECECEC8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECE3E3E3CACACAC5C5C5C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C4C4C4
E1E1E1EBEBEB8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF4F4F4F1F1F1F2F2F2F2F2F2F2F2F2F2F2F2F2F2F2F1F1F1F0F0F0
F1F1F1DDDDDD8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF0F0F0E9E9E9EAEAEAEBEBEBEAEAEAE9E9E9E8E8E8E6E6E6E5E5E5
F0F0F0CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF1F1F1EBEBEBECECECECECECEBEBEBEAEAEAEAEAEAE8E8E8E6E6E6
F1F1F1CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF2F2F2EDEDEDDBDBDBEEEEEEEDEDEDEBEBEBE9E9E9D8D8D8E8E8E8
F2F2F2CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF2F2F2EEEEEEC8C8C8C8C8C8EDEDEDEBEBEBC8C8C8C8C8C8E9E9E9
F2F2F2CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF2F2F2EFEFEFE5E5E5C8C8C8D1D1D1C8C8C8C8C8C8E1E1E1E9E9E9
F2F2F2CACACA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF4F4F4F0F0F0EFEFEFEFEFEFC8C8C8C8C8C8EBEBEBEAEAEAEAEAEA
F2F2F2CBCBCB8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF4F4F4F1F1F1F0F0F0EFEFEFE5E5E5EBEBEBEBEBEBEBEBEBEBEBEB
F2F2F2CFCFCF8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF6F6F6F5F5F5F2F2F2F0F0F0EFEFEFEEEEEEEEEEEEECECECEBEBEB
F2F2F2D6D6D68A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECF9F9F9F3F3F3EFEFEFEDEDEDECECECEBEBEBEAEAEAE9E9E9E7E7E7
F5F5F5E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFECECECFDFDFDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFEBEBEB8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9999996E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0EEEEEEC6C6C6BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
8686869999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0EEEEEEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
8686869999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
D5D5D59999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999FCFCFC
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999FCFCFC999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999FCFCFC999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
9999999999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDFA5A5A5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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDFA5A5A5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000000FCFCFC8A8A8A999999FFFFFFFFFFFF7878780000000000006D6D6DDFDFDFA5A5A5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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFF8E8E8ED7D7D7616161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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFF838383F7F7F7FFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFBBBBBB7F7F7F7F7F7FB6B6B6FFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECEF5F5F5969696FEFEFE
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696EBEBEB
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFD7D7D7CACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFDDDDDD6C6C6C7F7F7F7E7E7E848484DCDCDC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000000FCFCFC8A8A8A999999FFFFFFFFFFFF808080EFEFEFFFFFFFFFFFFF787878C9C9C9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000000FCFCFC8A8A8A999999FFFFFFFFFFFF939393FFFFFFFFFFFF3C3C3C000000C9C9C9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000000FCFCFC8A8A8A999999FFFFFFFFFFFF898989F7F7F7FFFFFFFFFFFFC0C0C0E3E3E3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000000FCFCFC8A8A8A999999FFFFFFFFFFFFBBBBBB606060FFFFFFEFEFEF212121F7F7F7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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFBBBBBB7F7F7F818181E8E8E8FFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFF8888887B7B7B7F7F7F979797DFDFDF8C8C8C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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDFA5A5A5
FFFFFF787878000000B4B4B4999999F7F7F7FFFFFFDFDFDF090909B4B4B4FFFFFFFFFFFF
C6C6C67F7F7FF7F7F77C7C7CDFDFDF535353F9F9F95D5D5DBABABA8F8F8FFDFDFD5D5D5D
D2D2D2FFFFFFFFFFFF999999EFEFEF6E6E6EEBEBEB636363DADADA0F0F0FF3F3F3262626
D8D8D8535353F7F7F7999999F7F7F7EFEFEF535353959595F7F7F7FFFFFFFFFFFFA2A2A2
C7C7C7FFFFFFFFFFFFFFFFFFFFFFFF787878000000D9D9D9FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFF3F3F3EEEEEEEFEFEF
F0F0F0F1F1F1F4F4F4F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDF858585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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDFA3A3A3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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFEFEFEFD2D2D2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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFCCCCCCFBFBFBD0D0D0E2E2E2B1B1B1F9F9F9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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F3F3F3797979B6B6B6636363DADADA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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999D6D6D6969696B1B1B1636363E8E8E8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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999A0A0A0D2D2D2A8A8A8636363E7E7E7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000000FCFCFC8A8A8A999999FFFFFFFFFFFF9999999C9C9CF7F7F7B8B8B8636363F7F7F7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000000FCFCFC8A8A8A999999FFFFFFFFFFFF858585939393FFFFFFBEBEBE2E2E2EF7F7F7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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696FBFBFB
EDEDEDEDEDED959595959595959595959595CFCFCFD0D0D0FFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFBFBFBA9A9A9FFFFFFFFFFFFFFFFFFC6C6C69F9F9F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000000FCFCFC8A8A8A999999FFFFFFFFFFFF939393FFFFFFFFFFFFFDFDFD5D5D5DD8D8D8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000000FCFCFC8A8A8A999999FFFFFFFFFFFFADADAD4747477F7F7F5D5D5D7A7A7AEDEDED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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFBFBFB2C2C2CEFEFEF999999F7F7F7EFEFEF
5353538A8A8AD2D2D2F7F7F7686868DFDFDFA5A5A5FFFFFF989898DFDFDF1B1B1BE4E4E4
262626D8D8D81B1B1BE8E8E8DFDFDF535353F7F7F72C2C2CE7E7E75353538A8A8AD2D2D2
FFFFFF818181FFFFFFFFFFFFDFDFDFA5A5A5FFFFFFFFFFFFFFFFFFDFDFDF888888F7F7F7
7C7C7CDFDFDF1B1B1BCCCCCC090909CCCCCC6060607D7D7DD2D2D2F7F7F77C7C7CDFDFDF
A5A5A5FBFBFB979797DFDFDF535353EBEBEB535353F7F7F72C2C2CE7E7E7535353F7F7F7
999999D7D7D7888888FFFFFFFFFFFF0000000000007F7F7FFFFFFFF5F5F5969696FBFBFB
EDEDEDEDEDEDE9E9E9E3E3E3DCDCDCD6D6D6CFCFCFD0D0D0FFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFABABAB919191AAAAAAFFFFFFEFEFEF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFF787878C7C7C7FFFFFFEFEFEF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFF7D7D7DD2D2D2BBBBBB
000000474747D2D2D2DFDFDFA5A5A5DFDFDFA5A5A5DFDFDFA5A5A5A2A2A2000000303030
FFFFFF999999F7F7F7EFEFEF535353F7F7F77878780000007F7F7FF7F7F70E0E0E575757
1D1D1DC9C9C9535353FBFBFBFFFFFFFDFDFD545454000000474747FFFFFFD0D0D0000000
5A5A5AFFFFFFDFDFDFA5A5A5DFDFDFA5A5A5DFDFDF959595484848121212474747FFFFFF
BBBBBB000000474747D2D2D2F7F7F72F2F2FEFEFEF0E0E0E575757686868F7F7F71D1D1D
8C8C8C535353CCCCCCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696E6E6E6
CECECE3E3E3ECECECECECECECECECE3E3E3ECECECECECECEFFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7EFEFEF525252D2D2D28C8C8C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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7818181959595D2D2D2999999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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999A8A8A8A2A2A2ACACACD2D2D2999999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000000FCFCFC8A8A8A999999FFFFFFFFFFFF787878606060FFFFFFACACACD2D2D2FFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFF7D7D7DD2D2D2FFFFFFACACACD2D2D2FFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC999999FCFCFC
999999CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC999999FCFCFC999999CCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCC999999FCFCFC999999CCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCC9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999ADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADAD9E9E9EAEAEAEADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
AEAEAEBEBEBE8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999CACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACAA3A3A3C3C3C3CACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4A3A3A3C3C3C3D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDF000000DFDFDFDFDFDFDFDFDF0000006F6F6FDFDFDF000000000000000000
DFDFDFDFDFDF000000000000000000DFDFDFDFDFDFDFDFDFDFDFDF6F6F6F000000000000
DFDFDF6F6F6F000000DFDFDF6F6F6F000000DFDFDF000000000000DFDFDFDEDEDEDEDEDE
DEDEDEDEDEDE000000000000000000000000DEDEDEDEDEDEDEDEDE000000000000000000
0000006F6F6F0000006F6F6F0000006F6F6FDFDFDF0000006F6F6F0000006F6F6FDFDFDF
000000DFDFDFDFDFDF000000000000DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDEDEDEA3A3A3C3C3C3DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF0000006F6F6FDFDFDFDFDFDF000000
000000DFDFDF0000006F6F6FDFDFDF000000000000000000DFDFDFDFDFDFDFDFDFDFDFDF
000000DFDFDFDFDFDFDFDFDF0000006F6F6FDEDEDE6F6F6F000000000000000000DEDEDE
6F6F6F0000006F6F6F000000DEDEDE6F6F6F000000DEDEDE000000000000000000DEDEDE
6F6F6F0000006F6F6F0000006F6F6F000000DFDFDF6F6F6F000000000000000000DFDFDF
0000000000006F6F6F000000DFDFDF000000000000000000DFDFDFDFDFDF000000000000
0000006F6F6FDFDFDFDFDFDFDFDFDF0000000000000000006F6F6FDFDFDFDFDFDF6F6F6F
000000000000DFDFDFDFDFDF000000000000000000DFDFDF6F6F6F000000DFDFDF000000
000000DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDEDEDE
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC7E7E7E7E7E7E7E7E7EFCFCFCFCFCFCFCFCFC
7E7E7E7E7E7E7E7E7EBDBDBD7E7E7EBDBDBD7E7E7E7E7E7EFCFCFC7E7E7EBDBDBDFCFCFC
FCFCFCFCFCFC7E7E7EBDBDBD7E7E7EBDBDBDFCFCFC7E7E7EFCFCFCFCFCFCFCFCFCFCFCFC
7E7E7EFCFCFCFCFCFCFCFCFCFCFCFCBDBDBD7E7E7E7E7E7E7E7E7EBDBDBD7E7E7EFCFCFC
BDBDBD7E7E7EFCFCFC7E7E7E7E7E7E7E7E7EFCFCFCFCFCFC7E7E7EBDBDBD7E7E7EBDBDBD
FCFCFC7E7E7E7E7E7E7E7E7EFCFCFC7E7E7EBDBDBDFCFCFC7E7E7EBDBDBDFCFCFC7E7E7E
7E7E7EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E43A3A3A707070707070707070000000ACACAC000000727272E4E4E4000000
E4E4E4000000727272E4E4E4000000E4E4E4E4E4E4E4E4E4E4E4E4000000E4E4E4727272
000000727272000000E4E4E4727272000000ACACAC747474383838747474E4E4E4E4E4E4
E4E4E4727272000000727272000000E4E4E4E4E4E4E4E4E4E4E4E4000000727272E4E4E4
E4E4E4E4E4E4000000727272000000727272E4E4E4000000727272000000383838000000
ACACACE4E4E4747474727272000000ACACACE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4A3A3A3C3C3C3E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4000000727272E4E4E4E4E4E4000000
000000E4E4E4000000727272000000727272E4E4E4747474ACACACE4E4E4E4E4E4E4E4E4
000000E4E4E4000000727272000000727272E4E4E4000000E4E4E4E4E4E4000000E4E4E4
000000E4E4E4727272000000E4E4E4727272000000727272000000E4E4E4747474747474
727272000000727272000000727272000000E4E4E4000000E4E4E4727272000000727272
000000E4E4E4727272000000727272000000E4E4E4747474747474727272000000E4E4E4
000000727272E4E4E4E4E4E4E4E4E4000000727272E4E4E4000000E4E4E4E4E4E4000000
E4E4E4727272000000727272000000E4E4E4727272000000727272000000383838747474
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E000000FCFCFC000000
0000007E7E7E0000007E7E7E0000007E7E7E0000000000007E7E7E000000FCFCFCFCFCFC
FCFCFCFCFCFC0000007E7E7E0000007E7E7EFCFCFC000000FCFCFCFCFCFCFCFCFCFCFCFC
000000FCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC0000000000007E7E7E000000FCFCFC
7E7E7E0000007E7E7E000000FCFCFC0000000000007E7E7E000000FCFCFC0000007E7E7E
000000000000FCFCFC0000007E7E7E0000007E7E7EFCFCFC0000007E7E7E0000007E7E7E
0000007E7E7EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECEC767676000000ECECEC000000000000ECECEC000000767676ECECEC000000
ECECEC000000767676ECECEC000000ECECEC000000000000ECECEC000000ECECEC767676
000000767676000000ECECEC767676000000ECECEC000000767676ECECECECECECECECEC
ECECEC767676000000000000000000000000ECECECECECECECECEC000000767676ECECEC
ECECECECECEC000000767676000000767676ECECEC000000767676000000000000ECECEC
ECECECECECECECECEC000000ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECA3A3A3C3C3C3ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECEC000000767676ECECECECECEC000000
000000ECECEC000000767676000000000000000000000000767676ECECECECECECECECEC
000000000000000000000000000000767676ECECEC767676000000000000000000ECECEC
000000ECECEC767676000000ECECEC767676000000767676000000000000000000000000
767676000000767676000000767676000000ECECEC767676000000000000000000767676
000000ECECEC767676000000767676000000ECECECECECECECECECECECEC000000000000
000000767676ECECECECECECECECEC000000767676ECECEC000000ECECECECECEC000000
ECECEC767676000000767676000000ECECEC767676000000767676000000767676ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E000000FCFCFC000000
000000FCFCFC0000007E7E7E0000007E7E7E0000000000007E7E7E000000FCFCFCFCFCFC
FCFCFCFCFCFC0000007E7E7E0000007E7E7EFCFCFC000000FCFCFCFCFCFCFCFCFCFCFCFC
000000FCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E0000007E7E7E000000FCFCFC
7E7E7E0000007E7E7E000000FCFCFCFCFCFCFCFCFC7E7E7E000000FCFCFC0000007E7E7E
000000000000FCFCFC0000007E7E7E0000007E7E7EFCFCFC0000007E7E7EFCFCFC7E7E7E
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECEC767676000000ECECEC000000000000ECECEC000000767676ECECEC000000
ECECEC000000767676ECECEC000000ECECECECECECECECECECECEC000000ECECEC767676
000000767676000000000000767676000000767676000000767676000000ECECECECECEC
ECECECECECEC000000767676ECECECECECECECECECECECECECECEC000000767676ECECEC
ECECECECECEC000000767676000000000000ECECEC000000767676000000000000000000
ECECECECECEC000000767676000000767676ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECA3A3A3C3C3C3ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECEC000000767676ECECECECECEC000000
000000767676000000767676000000767676ECECEC000000767676ECECECECECECECECEC
000000000000000000000000000000767676ECECECECECECECECECECECEC000000ECECEC
000000ECECEC767676000000000000767676000000767676000000ECECEC000000000000
767676000000767676000000767676000000ECECECECECECECECEC767676000000767676
000000ECECEC767676000000767676000000ECECEC000000000000ECECECECECECECECEC
000000767676ECECECECECECECECEC000000000000000000767676ECECECECECEC000000
ECECEC767676000000767676000000ECECEC767676000000767676000000000000ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000007E7E7E3F3F3F7E7E7EFCFCFC000000
000000FCFCFC0000007E7E7E0000007E7E7E0000000000007E7E7E000000FCFCFC7E7E7E
7E7E7EFCFCFC0000007E7E7E0000003F3F3FFCFCFC000000FCFCFCFCFCFCFCFCFCFCFCFC
0000007E7E7E7E7E7EFCFCFCFCFCFC000000FCFCFC7E7E7E0000007E7E7E0000007E7E7E
7E7E7E0000007E7E7E000000FCFCFC7E7E7E7E7E7E7E7E7E000000FCFCFC0000007E7E7E
000000000000FCFCFC0000007E7E7E0000003F3F3FFCFCFC0000007E7E7E7E7E7E3F3F3F
7E7E7EBDBDBDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4000000000000000000F4F4F4F4F4F4F4F4F47A7A7A7A7A7A000000
F4F4F4F4F4F47A7A7A7A7A7A000000F4F4F4F4F4F4F4F4F4F4F4F4B7B7B77A7A7A7A7A7A
F4F4F4B7B7B77A7A7A7A7A7A7A7A7AF4F4F4F4F4F47A7A7A7A7A7AF4F4F4F4F4F4F4F4F4
F4F4F4B6B6B60000003C3C3CF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4000000797979F4F4F4
F4F4F4F4F4F47A7A7AB7B7B77A7A7A7A7A7A7A7A7A7A7A7AF4F4F40000007979797A7A7A
B7B7B7F4F4F4F4F4F47A7A7A7A7A7AF4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4A3A3A3C3C3C3F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4000000797979F4F4F4F4F4F4000000
0000007A7A7A7A7A7AF4F4F4F4F4F47A7A7A7A7A7A7A7A7AF4F4F4F4F4F4F4F4F4F4F4F4
000000000000F4F4F4797979000000797979F4F4F4B7B7B77A7A7A7A7A7AB7B7B7F4F4F4
0000007A7A7A7979790000007A7A7A7A7A7AF4F4F4F4F4F47A7A7A7A7A7A7A7A7AF4F4F4
B7B7B77A7A7A7A7A7A7A7A7A7A7A7AF4F4F4F4F4F4B7B7B77A7A7A7A7A7AF4F4F4797979
0000007A7A7AB7B7B77A7A7AF4F4F47A7A7A7A7A7A7A7A7AF4F4F4F4F4F47A7A7A7A7A7A
7A7A7AF4F4F4F4F4F4F4F4F4F4F4F4000000797979F4F4F4000000F4F4F4F4F4F4B7B7B7
7A7A7A7A7A7AF4F4F4F4F4F47A7A7A7A7A7A7A7A7AF4F4F4797979000000B7B7B77A7A7A
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E000000FCFCFC000000
000000FCFCFC0000007E7E7E0000007E7E7E0000000000007E7E7E0000007E7E7EFCFCFC
FCFCFCFCFCFC0000007E7E7E0000000000000000007E7E7EFCFCFCFCFCFCFCFCFCFCFCFC
000000FCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E0000007E7E7E000000000000
000000FCFCFCFCFCFC000000000000000000FCFCFC7E7E7E0000007E7E7E0000007E7E7E
FCFCFC000000000000000000FCFCFC000000000000000000000000FCFCFCFCFCFC000000
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFA000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA000000
FAFAFAFAFAFAFAFAFAFAFAFA000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFA7D7D7D000000000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAA3A3A3C3C3C3FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFAFAFAFA000000
000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
000000FAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
000000FAFAFA7D7D7D000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA7D7D7D
000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFA000000FAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA7D7D7D000000FAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC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000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000000000000000FCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC0000007E7E7E000000000000FCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFC0000007E7E7EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
000000000000000000000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000007E7E7E
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFF9F9F9A3A3A3C3C3C3FDFDFDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF4F4F4
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999F8F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE6E6E6C5C5C5D3D3D3D5D5D5F3F3F3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFAFAFAD5D5D5
D3D3D3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AA6A6A6BEBEBEC4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4BEBEBEA6A6A6ACACACA3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3BFBFBFE3E3E3E3E3E3D4D4D4B6B6B6A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A9A9A9CECECE
E3E3E3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AC6C6C6AFAFAFC4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4B1B1B1C6C6C6C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1B2B2B2E3E3E3A3A3A3C1C1C1C2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C3C3C3A3A3A3C1C1C1C2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C3C3C3A3A3A3C1C1C1C2C2C2C2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE1E1E1B9B9B9B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2ACACAC9C9C9CB2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B24B4B4B4B4B4B
4B4B4B7E7E7EB2B2B2BFBFBFDEDEDECCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCB7B7B7E3E3E3A3A3A3CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C3C3C3A3A3A3CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C3C3C3A3A3A3CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9C3C3C3A3A3A3D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9000000D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9BEBEBEE3E3E3A3A3A3D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
C3C3C3A3A3A3D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
C3C3C3A3A3A3D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0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000000FCFCFC8A8A8AE0E0E0C1C1C1E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000000000E8E8E8
E8E8E8E8E8E8000000747474E8E8E8000000747474E8E8E8000000E8E8E8000000E8E8E8
E8E8E8000000E8E8E8000000E8E8E8E8E8E8E8E8E8E8E8E8747474000000E8E8E8E8E8E8
E8E8E8E8E8E8000000747474E8E8E8000000E8E8E8000000E8E8E8E8E8E8000000E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8C3C3C3A3A3A3E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000747474
E8E8E8747474000000E8E8E8747474000000E8E8E8E8E8E8E8E8E8000000000000E8E8E8
000000747474E8E8E8000000E8E8E8000000747474E8E8E8000000E8E8E8000000E8E8E8
E8E8E8000000E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8747474000000E8E8E8000000
747474E8E8E8000000E8E8E8E8E8E8000000E8E8E8000000E8E8E8747474000000E8E8E8
747474000000E8E8E8E8E8E8E8E8E8000000000000E8E8E8000000747474E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8C5C5C5E3E3E3A3A3A3E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000747474E8E8E8
E8E8E8747474000000E8E8E8000000747474000000000000E8E8E8000000747474000000
747474E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
C3C3C3A3A3A3E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000E8E8E8E8E8E8747474000000E8E8E8
000000747474E8E8E8E8E8E8E8E8E8000000747474000000747474000000747474000000
000000E8E8E8000000747474E8E8E8747474000000E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
C3C3C3A3A3A3E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000747474000000747474000000747474
E8E8E8000000747474E8E8E8000000E8E8E8E8E8E8000000000000E8E8E8000000747474
E8E8E8000000E8E8E8E8E8E8E8E8E8E8E8E8000000E8E8E8E8E8E8E8E8E8E8E8E8747474
000000E8E8E8000000747474E8E8E8000000747474000000747474E8E8E8000000E8E8E8
000000747474E8E8E8E8E8E8E8E8E8747474000000000000E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECEC000000000000767676
767676ECECEC000000767676ECECEC000000767676ECECEC000000ECECEC000000767676
ECECEC000000ECECEC000000ECECECECECECECECECECECEC767676000000767676767676
ECECECECECEC0000003B3B3BECECEC000000ECECEC000000ECECECECECEC000000ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECC3C3C3A3A3A3ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC000000767676
7676763B3B3B000000ECECEC767676000000767676000000767676000000000000ECECEC
000000767676ECECEC000000ECECEC0000003B3B3BECECEC000000ECECEC000000ECECEC
ECECEC000000ECECECECECECECECECECECEC7676767676763B3B3B767676ECECEC000000
767676ECECECB1B1B1767676767676000000ECECEC000000ECECEC767676000000ECECEC
767676000000767676000000767676000000000000ECECEC000000767676ECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECC7C7C7E3E3E3A3A3A3ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECEC000000767676ECECEC
ECECEC767676000000ECECEC000000767676000000000000ECECEC000000767676000000
767676ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3A3A3A3ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECEC000000ECECECECECEC767676000000ECECEC
0000003B3B3B767676000000767676000000767676000000767676000000767676000000
000000ECECEC0000007676767676763B3B3B767676767676ECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3A3A3A3ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECEC000000000000000000000000000000767676
ECECECECECEC767676767676000000ECECEC3B3B3B767676767676B1B1B1000000767676
ECECEC000000767676767676000000ECECEC000000ECECECECECECECECECECECEC767676
000000ECECEC0000003B3B3BECECEC000000767676000000767676ECECEC000000ECECEC
0000003B3B3B767676000000ECECEC3B3B3B767676000000B1B1B1ECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000000000F3F3F3
F3F3F3F3F3F3000000000000F3F3F3F3F3F3000000000000797979F3F3F3000000000000
000000797979F3F3F3000000000000F3F3F3F3F3F3F3F3F3797979000000F3F3F3F3F3F3
F3F3F3F3F3F3000000000000000000797979F3F3F3797979000000000000000000F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3C3C3C3A3A3A3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000797979
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000000000000000F3F3F3000000000000797979
000000000000F3F3F3000000F3F3F3000000000000000000797979F3F3F3797979000000
000000000000F3F3F3F3F3F3F3F3F3797979000000F3F3F3F3F3F3F3F3F3F3F3F3000000
000000F3F3F3797979000000000000797979F3F3F3000000000000797979000000000000
F3F3F3000000000000000000F3F3F3F3F3F3000000000000000000797979F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3CBCBCBE3E3E3A3A3A3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000797979F3F3F3
F3F3F3F3F3F3000000000000000000F3F3F3000000000000F3F3F3000000797979000000
000000F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
C3C3C3A3A3A3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000F3F3F3F3F3F3797979000000F3F3F3
F3F3F3000000000000000000F3F3F3000000000000000000000000000000F3F3F3F3F3F3
000000000000000000F3F3F3F3F3F3000000000000F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
C3C3C3A3A3A3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000000000F3F3F3797979000000797979
F3F3F3F3F3F3000000000000797979F3F3F3797979000000000000F3F3F3000000000000
F3F3F3797979000000000000797979F3F3F3000000000000F3F3F3F3F3F3F3F3F3797979
000000F3F3F3000000000000000000000000F3F3F3F3F3F3000000000000000000F3F3F3
F3F3F3000000000000797979F3F3F3000000F3F3F3000000797979F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5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000000FCFCFC8A8A8AE0E0E0C1C1C1FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC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000000FCFCFC8A8A8AE0E0E0C1C1C1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFC3C3C3A3A3A3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFD1D1D1E3E3E3A3A3A3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C3C3C3A3A3A3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C3C3C3A3A3A3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C6C6C6F2F2F2FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8AE0E0E0DADADAC2C2C2F3F3F3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8AE0E0E0E0E0E0DADADAC6C6C6C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C4C4C4D7D7D7E1E1E1E1E1E1DADADAC6C6C6C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C8C8C8DCDCDCE1E1E1E1E1E1D2D2D2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C8C8C8D7D7D7
E1E1E1E1E1E1D2D2D2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C4C4C4D7D7D7
E1E1E1E1E1E1D2D2D2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C4C4C4D7D7D7
E1E1E1E1E1E18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E4E4E4E8E8E8E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBE9E9E9E9E9E9EAEAEAEAEAEAE6E6E6E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E3E3E3E7E7E7E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAEBEBEBEBEBEB
EBEBEBEBEBEBE9E9E9E9E9E9E9E9E9EAEAEAE6E6E6E0E0E0E0E0E0E0E0E0E4E4E4E8E8E8
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE9E9E9
E9E9E9EAEAEAEAEAEAE6E6E6E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D4D4D4C7C7C7C7C7C7C6C6C6C7C7C7C7C7C7C7C7C7
C7C7C7C8C8C8C9C9C9C8C8C8C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7
C8C8C8C9C9C98A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0CFCFCFC3C3C3
C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3
C3C3C3C7C7C7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0CFCFCFC3C3C3C3C3C3
C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3
C7C7C7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0DDDDDDBABABA505050343434343434343434343434343434343434343434343434
343434343434343434343434343434454545868686F1F1F1E0E0E0E0E0E0E0E0E0E0E0E0
DDDDDDE2E2E2717171343434343434343434343434343434343434343434343434343434
3434343434343434343434343C3C3C868686F1F1F1E0E0E0E0E0E0DDDDDDBABABA505050
343434343434343434343434343434343434343434343434343434343434343434343434
343434454545868686F1F1F1E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C7C7C7FEFEFEFDFDFDEFEFEFEAEAEAE9E9E9E8E8E8
E9E9E9E0E0E08E8E8EDCDCDCE6E6E6E4E4E4E4E4E4E3E3E3E4E4E4E3E3E3E2E2E2E2E2E2
E2E2E2B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D6D6D6D2D2D2EBEBEB
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1DEDEDECCCCCCE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D6D6D6D2D2D2EBEBEBF1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
DEDEDECCCCCCE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABE0E0E0E0E0E0
E0E0E0D9D9D96666667A7A7A898989898989898989898989898989898989898989898989
8989898989898989898989898989898989894B4B4BBABABAE0E0E0E0E0E0E0E0E0E0E0E0
D9D9D9818181939393C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C6C6C6696969BABABAE0E0E0E0E0E0D9D9D9767676BBBBBB
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C4C4C4696969BABABAE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C7C7C7FCFCFCF7F7F7F0F0F0E7E7E7E6E6E6E5E5E5
E6E6E6E8E8E8A9A9A9B7B7B7E6E6E6E3E3E3E2E2E2E2E2E2D3D3D3D9D9D9E0E0E0DFDFDF
DDDDDDB0B0B08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1EDEDEDF1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1D8D8D8B0B0B0A1A1A1A2A2A2CACACAF1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1C3C3C3E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1B8B8B8A3A3A39C9C9CB9B9B9DADADAF1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1C3C3C3E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D5D5D5FFFFFF000000FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFABABABE0E0E0E0E0E0
E0E0E0D6D6D67C7C7CFFFFFFF7F7F7F7F7F7000000F7F7F7F7F7F7B9B9B9F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFFFC4C4C48D8D8DE0E0E0E0E0E0E0E0E0E0E0E0
D6D6D6343434FAFAFAF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7C2C2C2666666ABABABF7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7FFFFFF8D8D8DE0E0E0E0E0E0D6D6D6999999F7F7F7
F7F7F7F7F7F7F7F7F7D2D2D28E8E8E7D7D7D7777777B7B7B8C8C8CDBDBDBF7F7F7F7F7F7
F7F7F7F7F7F7FFFFFF8D8D8DE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D6D6D6D5D5D5D6D6D6C1C1C1A6A6A6B3B3B3
BABABAACACACA0A0A09090907B7B7B6060609B9B9BC9C9C9A7A7A79D9D9DC0C0C0E7E7E7
E5E5E5AFAFAF8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D4D4D4DCDCDCEEEEEE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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0CFCFCFC3C3C3
C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3
C3C3C3C7C7C7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0CFCFCFC3C3C3C3C3C3
C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3
C7C7C7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABE0E0E0E0E0E0
E0E0E0D7D7D7B6B6B6505050343434343434343434343434343434343434343434343434
343434343434343434343434343434454545818181DCDCDCE0E0E0E0E0E0E0E0E0E0E0E0
D7D7D7DCDCDC707070343434343434343434343434343434343434343434343434343434
3434343434343434343434343C3C3C818181DCDCDCE0E0E0E0E0E0D7D7D7B6B6B6505050
343434343434343434343434343434343434343434343434343434343434343434343434
343434454545818181DCDCDCE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D7D7D7D6D6D6D5D5D5D6D6D6D1D1D1717171
B0B0B0A4A4A4ABABAB919191B4B4B49A9A9A9696968181817E7E7EC1C1C1D6D6D6D4D4D4
D0D0D0B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0DEDEDED6D6D6D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4CFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFD4D4D4D4D4D4D5D5D5D6D6D6DADADAE0E0E0E0E0E0E0E0E0E0E0E0
DEDEDED7D7D7D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D1D1D1CFCFCFCFCFCF
CFCFCFCFCFCFD4D4D4D4D4D4D4D4D4D6D6D6DADADAE0E0E0E0E0E0DEDEDED6D6D6D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFD4D4D4
D4D4D4D5D5D5D6D6D6DADADAE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D6D6D6D5D5D5D4D4D4D6D6D6C2C2C2808080
C7C7C7D0D0D0D2D2D2A3A3A3A8A8A8ADADADD3D3D3CBCBCB8888887E7E7ED1D1D1D5D5D5
D1D1D1B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D6D6D6D5D5D5D4D4D4D6D6D69C9C9CCDCDCD
D7D7D7D5D5D5D5D5D5CACACA9C9C9CB9B9B9D7D7D7D6D6D6D4D4D4B2B2B2BFBFBFD5D5D5
D1D1D1B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D7D7D7D5D5D5D4D4D4D6D6D6C6C6C6D6D6D6
D4D4D4D4D4D4D5D5D5D6D6D6ABABABB5B5B5D6D6D6D5D5D5D5D5D5D6D6D6D5D5D5D5D5D5
D1D1D1B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D6D6D6CECECECECECECECECECECECECECECE
CECECECECECECECECECFCFCFC4C4C4C4C4C4CECECECECECECECECECECECECECECECECECE
CBCBCBB3B3B38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C9C9C9BBBBBBB1B1B1B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B4B4B48A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000F7F7F7A4A4A49C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9CA4A4A4F7F7F70000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3F7F7F70000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000E9E9E9FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFBFBFBE3E3E3EEEEEEFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDE0E0E08D8D8D818181818181818181818181828282818181818181818181818181
818181818181818181A8A8A8FDFDFDFDFDFDA8A8A8818181818181818181818181828282
8282828181818181818181818181818181818181818D8D8DE0E0E0FDFDFDA2A2A2585858
5858585858585959595858585959595858585858585858585858585858585858587D7D7D
E2E2E2FDFDFDFDFDFDE9E9E90000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F77B7B7B727272DCDCDCF7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7B6B6B68D8D8DA2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2818181F7F7F7F7F7F7818181A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A28D8D8DB6B6B6F7F7F75858586F6F6F
7171717272727272727272727272727272727272727272727272727272727070705B5B5B
A6A6A6F7F7F7F7F7F7E0E0E00000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000DDDDDDEEEEEEEEEEEEEEEEEECCCCCCCACACAC9C9C9C9C9C9C8C8C8C7C7C7C8C8C8
C8C8C8C8C8C8C9C9C9CCCCCCEEEEEEEEEEEEEEEEEEE5E5E5E2E2E2EEEEEEE8E8E8E9E9E9
EEEEEEDFDFDFECECECDDDDDDDBDBDBE4E4E4E0E0E0EEEEEEE1E1E1DBDBDBE5E5E5DCDCDC
E0E0E0EEEEEEE5E5E5E2E2E2EEEEEEE1E1E1DBDBDBDBDBDBECECECEBEBEBE0E0E0EEEEEE
E1E1E1E7E7E7EEEEEEE1E1E1E7E7E7E6E6E6DBDBDBDFDFDFDCDCDCE4E4E4EBEBEBDBDBDB
DCDCDCEDEDEDDDDDDDEDEDEDEEEEEEE5E5E5DBDBDBDFDFDFEEEEEEE6E6E6DBDBDBDBDBDB
E0E0E0E4E4E4EEEEEEEEEEEEEDEDEDDCDCDCE0E0E0EEEEEEEEEEEEDFDFDFEDEDEDEBEBEB
DBDBDBDBDBDBE2E2E2EEEEEEE8E8E8DBDBDB737373727272CFCFCFDBDBDBDCDCDCEDEDED
EEEEEEEEEEEEEEEEEEEDEDEDDBDBDBDBDBDBDBDBDBEDEDEDEDEDEDDCDCDCE0E0E0EEEEEE
EEEEEEEBEBEBDBDBDBDBDBDBE7E7E7EEEEEEEEEEEEE2E2E2EAEAEAEEEEEEE4E4E4EEEEEE
EEEEEEE3E3E3DBDBDBDCDCDCEDEDEDEBEBEBDBDBDBDBDBDBE2E2E2EEEEEEE5E5E5E0E0E0
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEB1B1B1B9B9B9AEAEAEC1C1C1D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5AEAEAEAEAEAE
AEAEAEAEAEAEAEAEAE979797EEEEEEEEEEEE979797AEAEAEC1C1C1D5D5D5D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5AEAEAEAEAEAEB9B9B9B1B1B1EEEEEE646464707070
747474777777777777777777777777777777777777777777777777777777717171737373
A1A1A1EEEEEEEEEEEEDDDDDD0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000DADADAE5E5E5E5E5E5B7B7B79B9B9BCACACAE9E9E9E9E9E9D7D7D7858585ECECEC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000000D9D9D9DFDFDFDFDFDFDEDEDEA0A0A0E9E9E9E5E5E5EAEAEADDDDDD747474EDEDED
ADADADE1E1E1F3F3F3A3A3A3DEDEDEDEDEDEC6C6C6101010AFAFAF6C6C6C383838CACACA
4E4E4E6C6C6C101010B1B1B14E4E4E101010B1B1B14E4E4E7E7E7EDDDDDD7E7E7E515151
CDCDCDB6B6B6151515B1B1B11B1B1B959595C3C3C34C4C4CCDCDCD1C1C1CA1A1A1ADADAD
4F4F4FD5D5D5ADADAD4F4F4F8989894F4F4FD5D5D52F2F2F8E8E8ED5D5D5434343C4C4C4
DCDCDC4E4E4E696969D6D6D6252525838383DBDBDB7676768F8F8F4F4F4FD5D5D54E4E4E
4F4F4FD9D9D9DEDEDEDEDEDE7E7E7E515151CDCDCDDEDEDE6C6C6C696969C2C2C2363636
D9D9D9C3C3C3929292D7D7D72E2E2EC4C4C46F6F6F727272D1D1D19494941A1A1AC5C5C5
DEDEDEDEDEDED7D7D72525258F8F8FDEDEDE404040A4A4A4777777515151CDCDCDDEDEDE
C3C3C3464646D9D9D9525252A5A5A5CDCDCD7E7E7E383838CCCCCC6B6B6B696969D5D5D5
D7D7D7A7A7A7C3C3C3464646B7B7B7363636D9D9D9C3C3C3929292D7D7D73B3B3BCDCDCD
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEAAAAAAC3C3C3BBBBBBDDDDDD727272727272727272727272272727BBBBBBBBBBBB
BBBBBBBBBBBBBBBBBB9C9C9CDEDEDEDEDEDE9C9C9CBBBBBBDDDDDD272727BBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBDDDDDD272727BBBBBBBBBBBBC3C3C3AAAAAADEDEDE666666787878
555555FFFFFFFFFFFFBEBEBE7D7D7D7D7D7D7D7D7DFFFFFFFFFFFF303030797979777777
9A9A9ADEDEDEDEDEDED9D9D90000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000D6D6D6D5D5D5D5D5D5B0B0B0999999D1D1D1B5B5B5434343ABABABB2B2B28F8F8F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000000D3D3D3CECECECFCFCFCECECE878787B3B3B3DBDBDBADADADB8B8B8A9A9A99B9B9B
919191BDBDBDEFEFEFA5A5A5CECECECECECECCCCCC4949495A5A5A7272729E9E9E676767
151515B9B9B9A0A0A0101010101010151515B5B5B52323237C7C7CCECECE757575515151
C7C7C7A9A9A9151515949494151515B7B7B7C7C7C72323238181811A1A1A9C9C9CA0A0A0
4F4F4FCACACAA0A0A04F4F4FCDCDCD585858101010121212A4A4A4C7C7C72D2D2DC1C1C1
CCCCCC494949696969A0A0A0101010AAAAAACECECECBCBCBCDCDCD585858101010101010
5C5C5CCECECECECECECECECE757575515151C5C5C5C5C5C56161616969698484845C5C5C
CACACAC7C7C73A3A3AC4C4C42D2D2DC1C1C1676767727272CACACAC1C1C11A1A1A9F9F9F
CECECECECECEC7C7C72323238B8B8BC5C5C53D3D3DC2C2C2757575515151C5C5C5CDCDCD
8484845C5C5CCACACA757575515151C4C4C43A3A3A9E9E9E2525258F8F8F4F4F4FB9B9B9
9E9E9E141414C0C0C0CCCCCC8585855C5C5CCACACAC7C7C73A3A3AC4C4C43A3A3AC5C5C5
CDCDCDCDCDCDCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECEA2A2A2CECECECBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBA2A2A2CECECECECECEA2A2A2CBCBCBE5E5E5272727CBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBE5E5E5272727CBCBCBCBCBCBCECECEA2A2A2CECECE696969828282
848484858585858585939393FFFFFFFFFFFFFFFFFF5656568585858484848383837D7D7D
929292CECECECECECED3D3D3000000AAAAAAB3B3B3BFBFBFCCCCCCD9D9D9E4E4E4EBEBEB
F2F2F2F7F7F7FAFAFAFDFDFDFDFDFDFEFEFEFFFFFF
000000CFCFCFC7C7C7C7C7C7B1B1B1959595A6A6A6ACACACAEAEAEACACACA5A5A5747474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000000CDCDCDC1C1C1C1C1C19B9B9B999999CACACAC9C9C97373739E9E9E737373575757
BEBEBEB3B3B3ADADADA9A9A9C1C1C1C1C1C1C1C1C1969696303030C4C4C4C1C1C1464646
717171C2C2C2C1C1C1C1C1C1C1C1C1C1C1C1BEBEBE2222227B7B7BC2C2C26E6E6E515151
C4C4C4C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1BBBBBB2D2D2DBEBEBE
C1C1C1C1C1C1C4C4C4C2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C16E6E6E515151C4C4C4C1C1C15F5F5F696969C2C2C2C1C1C1
C1C1C1C1C1C1C1C1C1BBBBBB2D2D2DBEBEBEC1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1BBBBBB2222228A8A8ABEBEBE3E3E3EBEBEBEC1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C19B9B9BD8D8D8D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5A7A7A7C1C1C1C1C1C1A7A7A7D5D5D5EAEAEA272727FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF272727D5D5D5D5D5D5D8D8D89B9B9BC1C1C16D6D6D888888
8A8A8A8A8A8A8A8A8AFFFFFFFFFFFF939393FFFFFFC4C4C48A8A8A8A8A8A898989848484
8B8B8BC1C1C1C1C1C1CDCDCD000000C7C7C7CCCCCCD4D4D4DCDCDCE5E5E5EDEDEDF1F1F1
F6F6F6F9F9F9FCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
000000CBCBCBBABABABBBBBBACACAC959595AAAAAAC3C3C3A2A2A2A2A2A29F9F9F626262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000000C9C9C9B8B8B8B8B8B89F9F9F9E9E9EE2E2E2F2F2F2CECECECFCFCFD2D2D2B7B7B7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000000C9C9C9B8B8B8B8B8B8B8B8B8868686C3C3C3F3F3F3A0A0A0EFEFEFDBDBDBC9C9C9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000000C9C9C9B5B5B5B3B3B3A3A3A3969696C9C9C9C7C7C7CBCBCBD0D0D0D1D1D1B6B6B6
D3D3D3D3D3D3D4D4D4A8A8A8B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3949494E0E0E0EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDABABABB3B3B3B3B3B3ABABABEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE0E0E0949494B3B3B37A7A7AA3A3A3
B0B0B0BEBEBEC1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1BEBEBEBABABAA4A4A49F9F9F
848484B3B3B3B5B5B5C9C9C9000000F0F0F0F2F2F2F3F3F3F6F6F6F8F8F8FAFAFAFBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFF
000000C3C3C3BBBBBBAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAE9D9D9D767676CECECEFDFDFDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFDFDFDCECECE818181AEAEAEAEAEAE818181CECECEFDFDFDFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDCECECE7676769D9D9DAEAEAE777777B5B5B5
D5D5D5E8E8E8E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8DFDFDFC0C0C0565656
A3A3A3AEAEAEBBBBBBC3C3C3000000F6F6F6F7F7F7F8F8F8F9F9F9FBFBFBFCFCFCFDFDFD
FDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
000000B6B6B6D1D1D1B2B2B2AAAAAAA8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A89B9B9B909090909090909090909090909090909090909090909090909090
909090909090909090A2A2A2A8A8A8A8A8A8A2A2A2909090909090909090909090909090
9090909090909090909090909090909090909090909B9B9BA8A8A8A8A8A8A2A2A2808080
8080808080808080808080808080808080808080808080808080808080808080809B9B9B
AAAAAAB2B2B2D1D1D1B6B6B6000000FAFAFAFAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
0000008A8A8AEAEAEAE4E4E4CFCFCFBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
CFCFCFE4E4E4EAEAEA8A8A8A000000FCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000B4B4B49C9C9CE7E7E7F5F5F5F1F1F1ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECF1F1F1
F5F5F5E7E7E79C9C9CB4B4B4000000FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFB4B4B4696969909090AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
909090696969B4B4B4FFFFFF000000FEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{391, 251},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000670000nb000`400?l00000o`00003oo`3ooon8
0?ooo`002P3ooooo0?knoW@0o_kn2P3oool000L0oooo0P3no_ko0?gmoGH0oOgm0P3no_h70?ooo`00
1P3oool20?knoPD0oOgmo`3lo?a^0?clo0D0oOgm0P3no_h60?ooo`00103oool20?knoP<0oOgm0P3l
o?`00`3kno/0n_[j0?[jnP3o0?[jnVh0n_[j00<0no_k0?clo03lo?`00`3moOd20?knoP@0oooo0004
0?ooo`090?knoP3moOd0oOgm0?gmo@3lo?`0no_k0?Win@3hn?P0moOg0?l0m_KfKP3fm_H01@3gmoL0
n?Sh0?Win@3kno/0o?cl00<0oOgm00<0o_kn0?ooo`3oool00P3oool000<0oooo00X0o_kn0?gmo@3m
oOd0o?cl0?_kn`3jn_X0n?Sh0?KfmP3clo<0l_;bo`3`l?1^0?3`l0070?;blP3clo<0m_Kf0?Shn03j
n_X0no_k0?clo0020?gmo@030?knoP3oool0oooo0040oooo00020?ooo`80o_kn0P3moOd01`3kno/0
nOWi0?Ogm`3emOD0lO7a0>k^kP3[jn/0o`3Xj>Q^0>SXj0070>_[j`3^k^h0lO7a0?Gem@3gmoL0nOWi
0?_kn`020?gmo@80o_kn0P3oool00080oooo00/0o_kn0?gmo@3moOd0o?cl0?[jnP3gmoL0mOGe0?7a
l@3]kNd0j>SX0>CTi03o0>7QhFh0hN7Q00P0i>CT0>SXj03]kNd0lO7a0?Gem@3gmoL0n_[j0?clo080
oOgm0@3no_h10?ooo`40oooo00020?ooo`80oOgm00X0o?cl0?_kn`3hn?P0mOGe0?7al@3[jn/0iNGU
0=kNgP3IfMT0eMGEo`3Cdm=/0=?Cd`0:0=GEe@3IfMT0g]kN0>GUi@3[jn/0lO7a0?Gem@3hn?P0no_k
0?clo080oOgm0P3oool0000>0?ooo`3no_h0oOgm0?gmo@3lo?`0nOWi0?KfmP3alO40kNg]0>GUi@3L
g=`0e=CD0<c<c037alOo0<G5aF`0aLG500X0alO70<c<c03De=@0g=cL0>GUi@3]kNd0lO7a0?KfmP3i
nOT0o?cl0P3moOd10?knoP40oooo00003P3oool0o_kn0?gmo@3moOd0no_k0?Shn03clo<0k^k^0>SX
j03Ng]h0e=CD0<S8b02o_kl0^;Rho`2f][I/0;Jf]P0:0;Rh^02o_kl0b<S80=CDe03Ng]h0j>SX0>k^
kP3clo<0n?Sh0?_kn`80oOgm0@3no_h10?ooo`0000h0oooo0?knoP3moOd0oOgm0?[jnP3gmoL0l_;b
0>_[j`3Ti>@0fMWI0<c<c02o_kl0/k>c0:ZZZ_l0YZJVK02VYZH02P2ZZZX0/k>c0;no_`3<c<`0fMWI
0>CTi03[jn/0l_;b0?Ogm`3jn_X20?gmo@40o_kn0@3oool0000>0?ooo`3no_h0oOgm0?clo03jn_X0
m_Kf0?3`l03Xj>P0hN7Q0=GEe@37alL0^;Rh0:ZZZP2OWioo09bLW6`0W9bL00d0WinO0:ZZZP2h^;P0
alO70=GEe@3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo003o000007X0000000d0
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
0`000000o?cl0?clo03o0?clo7H0o?cl00h0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`
l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl09ZJVP3o08Z:RW@0RXZ:0100VYZJ
0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn
0@3oool000030000003lo?`0RXZ:06X0oooo00<0kno_0>3Ph03Ph>00903Ph>000`2m_Kd0WinO09nO
W`0709nOW`0309JFUP3Vi^H0dm?C06/0WinO00<0_Kfm0>WYj@3WinL0C`3WinL00`3^k^h0UYJF09nO
W`0709nOW`0309JFUP3Vi^H0h>3P00/0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000040000003lo?`0RXZ:09jNWVT0
h>3P00<0kno_0>3Ph03Ph>00903Ph>000`32`/80e=CD0=CDe0070=CDe0030>7Qh@2FUYH0`L7106/0
cLg=00<0kNg]0<_;b`3kno/0C`3kno/00`2FUYH0hN7Q0=CDe0070=CDe0030>7Qh@2FUYH0h>3P0080
h>3P01T0e=CD0=WIf@3_knl0bLW90=WIf@3_knl0bLW90>c/k03Ph>00RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000
o?cl08Z:RP2NWYiY0>3Ph0030>o_k`3Ph>00h>3P02@0h>3P00<0b/[:0<k>cP3>c/h01`3>c/h00`3V
i^H0UYJF0<[:bP1[0<o?c`030?ooo`38b<P0no_k04l0no_k00<0UYJF0>KViP3>c/h01`3>c/h00`3V
i^H0UYJF0>3Ph0020>3Ph0030<W9b@2c/k<0h>3P0080/k>c01@0h>3P0;>c/`39bLT0h>3P08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40
oooo000010000000o?cl08Z:RP2NWYiY0>3Ph00=0>o_k`3Ph>00h>3P0>3Ph02g]kL0@d=306a/K02_
[jl0G5aL05iNGP2CTi<0h>3P07inOP02071`L080h>3P00L0g]kN07moO`2c/k<0`l?305=CD`1@D500
cLg=00l0h>3P00<0b/[:0<k>cP3>c/h00P3>c/h00`1RHV80QXJ60<k>cP020<k>cP030>KViP2FUYH0
b/[:01`0dm?C203De=@?0=?Cd`050;>c/`3Cdm<0`/;20;:b/P32`/80:@3Cdm<80=CDe080dm?C00<0
oooo0<O7a`3inOT0C`3inOT01`2FUYH0i^KV0<k>cP3>c/h0QXJ603hn?P2ZZZX00`3>c/h00`3Vi^H0
UYJF0>3Ph0060>3Ph00E0>o_k`3WinL0h>3P0>o_k`3Ph>00RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:
RP2NWYiY0>3Ph0030>o_k`3Ph>00h>3P0080h>3P00<0QXJ60=WIf@1oOgl00P3Bd]801P18B4P0d];B
07alO03Ph>00QXJ60=WIf@80h>3P00H0g=cL08F5Q@2][Jd0VYZJ06UYJ@2o_kl?0>3Ph0040<[:bP3?
cll0clo?0<o?c`80?Shn103?cll00`3WinL0UYJF0<[:bP020=cLg280gMgM3P3Lg=`01P3]kNd0UIFE
0?ooo`3:b/X0UIFE0<[:bPl0g=cL8P3MgMd20=cLg0030?ooo`37alL0nOWi04l0nOWi00<0UYJF0>OW
i`3?cll00P3?cll203hn?P<0clo?00<0inOW09JFUP3Ph>001@3Ph>020;>c/`0D0>o_k`2c/k<0fMWI
0>3Ph02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl
0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0WYjNJ@3Ph>000`3_knl0h>3P0>3Ph0020>3Ph00;
08J6QP3IfMT0Ogmo0=;BdP3Lg=`09RHV0<_;b`1lO7`0h>3P07moO`3Bd]800`3Ph>001@26QXH0d];B
00d=3@0=3@d0fMWI00H0h>3P00<0L71`0000000000000P0000001@1`L700h>3P0>3Ph03Ph>00b/[:
0080e=CD00<0RHV903hn?P29RHT0103De=@00`3YjNT0UYJF0<[:bP020=cLg280gMgM3P3Lg=`01P3]
kNd0UIFE0?ooo`3:b/X0UIFE0<[:bPl0g=cL8P3MgMd20=cLg0030?ooo`37alL0n?Sh04l0n?Sh00<0
UYJF0>WYj@3De=@00`3De=@203hn?P80e=CD00<0jNWY09JFUP3Ph>001@3Ph>020;>c/`0D0>3Ph02c
/k<0bLW90>3Ph02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3j
n_X0o?cl0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0WYjNJ@3Ph>000`3_knl0h>3P0>3Ph002
0>3Ph00A08J6QP3IfMT0Ogmo0=;BdP3GemL0=cLg0<k>cP1lO7`0h>3P08:2PP3EeMD0RHV9071`L00k
>c/0WinO07=cL`3Ng]h02@3Ph>00101`L70000000000001`L7040>3Ph0060<[:bP3Ogml0gmoO06MW
I`0n?Sh0][Jf103Ogml01@3_knl0UYJF0<[:bP3WinL0i^KV00T0inOW6P3Vi^H=0>OWi`060?;blP2E
UID0oooo0<[:bP2EUID0b/[:3`3Vi^H80>OWiaX0i^KV00@0inOW0>KViP3oool0aLG5D03fm_H00`2F
UYH0kno_0=oOg`030=oOg`0606EUI@0n?Sh0^;Rh0=oOg`3_knl0UYJF2P3Ph>004P3_knl0h>3P08Z:
RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?kn
oP40oooo000010000000o?cl08Z:RP2NWYiY0>3Ph00F0>o_k`3Ph>00h>3P0>3Ph02>SXh071`L0=WI
f@2TY:@0WYjN0;Ng]`1DE5@0h>3P08>3P`3Ph>00LG5a0>3Ph020P800h>3P06UYJ@2_[jl0JFUY0;no
_`X0h>3P00<0L71`0>3Ph03Ph>00103Ph>00103:b/X0kNg]0>g]k@3]kNd203hn?P@0kNg]00D0m_Kf
09JFUP3:b/X0j^[Z0>WYj@090>[ZjQX0jNWY3@3Zj^X01P3dm?@0UIFE0?ooo`3:b/X0UIFE0<[:bPl0
jNWY203Zj^XJ0>WYj@040>[ZjP3YjNT0oooo0<G5aE00m?Cd00<0UYJF0?KfmP3]kNd00P3]kNd203hn
?P<0kNg]00<0m_Kf09JFUP3Ph>00203Ph>004`2c/k<0fMWI0>3Ph02:RXX0o?cl0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003l
o?`0RXZ:09jNWVT0h>3P00<0kno_0>3Ph03Ph>000P3Ph>004@2j^[X0g=cL0=kNgP1mOGd0Lg=c0=WI
f@3Ph>00[:b/071`L02WYjL0h>3P0:b/[01`L700MWIf0=cLg03Bd]80Zj^[0140h>3P00<0b/[:0?Sh
n03hn?P00P3hn?P00`1]KFd0WIfM0?Shn0020?Shn0030?_kn`2FUYH0b/[:0380kNg]00H0lO7a0>g]
k@3fm_H0lO7a0>g]k@3alO4c0>g]k@030?ooo`33`l<0l?3`04l0l?3`00L0UYJF0?_kn`3hn?P0n?Sh
09fMW@0n?Sh0b/[:00<0n?Sh00<0no_k09JFUP3Ph>00203Ph>004`39bLT0e=CD0>3Ph02:RXX0o?cl
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3o
ool000040000003lo?`0RXZ:09jNWVT0h>3P00<0kno_0>3Ph03Ph>00903Ph>000`3:b/X0oooo0?oo
o`080?ooo`0309JFUP3:b/X0moOg06X0moOg00<0oooo0<;2`P3_knl0C`3_knl00`2FUYH0oooo0?oo
o`080?ooo`0309JFUP3Ph>00h>3P00X0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000040000003lo?`0RXZ:09jNWVT0
h>3P00<0kno_0>3Ph03Ph>00903Ph>000`2f][H0f][J0=[JfP070=[JfP030<K6aP2h^;P0aLG506/0
fm_K00<0dM7A0=3@d03^k^h0C`3^k^h00`2h^;P0alO70=[JfP070=[JfP030<K6aP2h^;P0h>3P00/0
h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0@3oool000030000003lo?`0RXZ:06X0WYjN00<0kno_0>3Ph03Ph>00903Ph>0<0>c/
k6d0jn_[00<0jNWY0>OWi`3WinL0CP3WinL<0>c/k0`0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0
RXZ:0?l0ooooJ03oool0103/k>`0hn?S0<[:bP35aLD50<71`@0C0<C4a03QhN40jn_[08Z:RP3lo?`0
000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?oo
o`0000<000000?clo02:RXX0o`3ooomX0?ooo`030>c/k03dm?@0lO7a00D0l_;b01@0lO7a0?3`l03a
lO40gMgM08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3l
o?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo01`0k>c/0?3`l03YjNT0j^[Z
0>_[j`3Zj^X0jNWY0>SXj03Vi^H0iNGU0?3`l03:b/X0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0
ooooJ03oool00`3/k>`0lO7a0>_[j`020>c/k00G0>;RhP3Zj^X0j^[Z0>SXj03Vi^H0lO7a0<[:bP2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000030000003lo?`0RXZ:0?l0ooooJ03oool00`3/k>`0l_;b0>g]k@020>k^kP80b<S8
0P3YjNT04`3Xj>P0l_;b0<[:bP2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0
l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0ooooJ03oool01P3/
k>`0l_;b0>k^kP3Ti>@0b<S80=7Ad@80b<S801@0hN7Q0>WYj@3bl_80b/[:08Z:RP3lo?`0000009bL
W02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`00
0000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0??cl`3_knl00P38b<P05`3/k>`0j^[Z0<S8b038b<P0
j^[Z0?;blP3:b/X0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00H0k>c/0?Cdm03`
l?00g=cL0>o_k`3]kNd20>_[j`0D0=WIf@3Zj^X0l_;b0<_;b`2:RXX0o?cl0000002LW9`0YZJV0;Jf
]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:
RXX0o`3ooomX0?ooo`040>c/k03dm?@0lO7a0?3`l080kno_103[jn/04P3bl_80clo?08Z:RP3lo?`0
000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo
00000`000000o?cl08Z:RP3o0?ooofP0oooo00H0k>c/0?KfmP3emOD0l_;b0?3`l03_knl20>k^kP0D
0>c/k03[jn/0l_;b0=KFeP2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`
0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:RXX0o`3ooomX0?ooo`0L0>c/k03i
nOT0lo?c0>o_k`3]kNd0k>c/0>_[j`3Zj^X0jNWY0>OWi`3emOD0h>3P08Z:RP3lo?`0000009bLW02V
YZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000
o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?gmo@3oool0203oool04@3[jn/0RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`00
0000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
0`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo
00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?cl
o0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040
oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:
0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn
0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0
RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/
k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03m
oOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/0
4@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203k
no/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0
203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3k
no/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03f
m_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_k
n`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/
0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0
k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40
j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo
00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3Q
hN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0
oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?oo
ofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o
0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:
RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl
08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000
o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`00
0000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
0`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo
00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?cl
o0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040
oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:
0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn
0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0
RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/
k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03m
oOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/0
4@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203k
no/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0
203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3k
no/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03f
m_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_k
n`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/
0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0
k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40
j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo
00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3Q
hN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0
oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?oo
ofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o
0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:
RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl
08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000
o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`00
0000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo0000000
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
0`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo
00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:0?cl
o0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040
oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0RXZ:
0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn
0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/k>`0
RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/04@3/
k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03m
oOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203kno/0
4@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?_kn`3kno/0203k
no/04@3/k>`0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00@0k>c/0>?Sh`3:b/X0
aLG51@31`L404`34a<@0hN7Q0>_[j`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3X
j>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0ooooJ03oool0
0`3/k>`0m?Cd0?7al@050?;blP0D0?7al@3`l?00lO7a0=gMg@2:RXX0o?cl0000002LW9`0YZJV0;Jf
]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:
RXX0o`3ooomX0?ooo`0L0>c/k03`l?00jNWY0>[ZjP3[jn/0j^[Z0>WYj@3Xj>P0i^KV0>GUi@3`l?00
b/[:08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0
oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?7al@3[jn/00P3/k>`0
5`3[jn/0j^[Z0>[ZjP3Xj>P0i^KV0?7al@3:b/X0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?oo
ofP0oooo01`0k>c/0?;blP3]kNd0fm_K0>k^kP3]kNd0jn_[0>WYj@3Hf=P0j>SX0?;blP3:b/X0RXZ:
0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn
0@3oool000030000003lo?`0RXZ:0?l0ooooJ03oool00`3/k>`0l_;b0>k^kP020<S8b00G0>g]k@3[
jn/0b<S80<S8b03YjNT0l_;b0<[:bP2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3X
j>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0ooooJ03oool0
1P3/k>`0l_;b0>o_k`3UiND0b<S80=7Ad@80b<S801@0hN7Q0>WYj@3bl_80b/[:08Z:RP3lo?`00000
09bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo0000
0`000000o?cl08Z:RP3o0?ooofP0oooo00<0k>c/0?Cdm03`l?000P3_knl20<S8b00E0>_[j`3Zj^X0
j^[Z0?;blP3;bl/0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo00H0k>c/0?Cdm03a
lO40l?3`0>o_k`3UiND40>_[j`0B0?;blP3?cll0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0oooo
J03oool01P3/k>`0m_Kf0?Gem@3bl_80l?3`0>o_k`80k^k^01@0k>c/0>_[j`3bl_80e]KF08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40
oooo00000`000000o?cl08Z:RP3o0?ooofP0oooo01`0k>c/0?Win@3clo<0kno_0>g]k@3/k>`0jn_[
0>[ZjP3YjNT0inOW0?Gem@3Ph>00RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0ooooJ03oool00`3/
k>`0oOgm0?ooo`080?ooo`0A0>_[j`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3X
j>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIVOl0KVi^LP1^
KVh04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIWo0>3Ph6L0h>3P00<0k^k^0<K6aP2o
_kl01`2o_kl04P26QXH0VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`
l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo000010000000o?cl08Z:RP2IVIWo0>3Ph6L0h>3P00<0
k^k^0?ooo`3oool01`3oool04P26QXH0VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo000010000000o?cl08Z:RP2IVIWo0>k^
kVL0k^k^2P3emOD04P3EeMD0VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SX
j03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP1L09VIV@030?clo02I
VIT0VIVI05X0VIVI00<0o?cl09VIV@2IVIT0FP2IVIT00`3lo?`0VIVI09VIV@1J09VIV@0@08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40
oooo000010000000o?cl08Z:RP2IVIT=0?ooo`040?_kn`26QXH0Ogmo0>?ShaP0oooo00@0no_k08J6
QP1oOgl0hn?S803oool00`3jn_X0m_Kf0<[:bP080<o?c`050<K6aP3hn?P0VIVI0?clo02IVIT0FP3o
ool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`00000
09bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`00
00@000000?clo02:RXX0VIVI3`3oool00`3gmoL0O7al0?ooo`0I0?ooo`030?Ogm`1lO7`0oooo01l0
oooo00<0mOGe09JFUP3Lg=`0203Kfm/01@3Ng]h0b/[:09VIV@3lo?`0VIVI05X0oooo00<0VIVI0?cl
o02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf
]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0
RXZ:09VIV@80oooo00P0VIVI0?Ogm`3oool0oooo0=oOg`2UYJD0oooo0=3@d08000000240fMWI0?oo
o`1SHf<03`l?08f=S@3Ogml0De=C0?_kn`3gmoL03Ph>05MGE`1XJ6P0gmoO05=CD`3[jn/0De=C0>_[
j`1CDe<0UIFE0?Ogm`3Ogml0YJFU0=oOg`2EUID0De=C09FEU@3gmoL0no_k09NGU`3oool0Hf=S00l?
3`2=SHd0803oool00`3emOD0UYJF0>KViP080<k>cP050?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool0
0`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bL
W02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@0
00000?clo02:RXX0VIVI0P3oool0:`2IVIT0moOg0?ooo`3oool0gmoO0:FUY@3oool0RHV90?Ogm`3k
no/0RHV90=kNgP2NWYh0no_k07moO`3Ogml0De=C0?_kn`36a/H0UiNG0=kNgP1dM7@0gmoO05=CD`3[
jn/0De=C0>_[j`1CDe<0UIFE0?Ogm`3Ogml0YJFU0=oOg`2EUID0De=C09FEU@3gmoL0no_k09NGU`3N
g]h0WYjN0?_kn`1oOgl0703oool01`1oOgl0oooo0?ooo`3oool0mOGe09JFUP3Vi^H0203>c/h01@3o
ool0b/[:09VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J
0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3j
n_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIV@80oooo00P0N7Qh000000000000
KFe]0=oOg`2UYJD0oooo09>CT`80oooo0240UYJF0<K6aP2_[jl0oooo09JFUP3Ogml0De=C0?_kn`3o
ool0N7Qh0000001]KFd0gmoO05=CD`3[jn/0De=C0>_[j`1CDe<0UIFE0?Ogm`3Ogml0YJFU0=oOg`2E
UID0De=C09FEU@3gmoL0no_k09NGU`36a/H0[jn_0?ooo`2FUYH0703oool200000080oooo00<0mOGe
09JFUP3Vi^H00`3>c/h00`0n?Sh0c/k>0<k>cP020<k>cP050?ooo`3:b/X0VIVI0?clo02IVIT0FP3o
ool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`00000
09bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`00
00@000000?clo02:RXX0VIVI0P3oool04`2IVIT0moOg0?ooo`2>SXh0emOG065QH@27QhL0Y:BT07Mg
M`1kNg/0XJ6Q0?_kn`1_Kfl0Ph>307YjNP3Ogml0=3@d08Z:RP3WinL00P1kNg/05P2NWYh0gmoO04A4
A01kNg/0G5aL07]kN`2:RXX0TI6A09>CT`1SHf<0MgMg06=SH`2d];@0De=C08j>SP26QXH0Ng]k09jN
WP3kno/0Kfm_08>3P`1jNWXL0?ooo`80000000D0_kno0?ooo`3emOD0UYJF0>KViP020<k>cP03069R
HP0n?Sh0HV9R00<0c/k>00D0oooo0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0
oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIT2
0?ooo`0509VIV@3gmoL0oooo08>3P`3gmoL0@P3oool020000000oooo0?ooo`3oool0mOGe09JFUP3Y
jNT0e=CD0P0n?Sh00`3De=@0?Shn03hn?P020=CDe0050?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool0
0`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bL
W02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@0
00000?clo02:RXX0VIVI0P3oool0102k^k/0Ogmo07moO`2f][HM0?ooo`030?Ogm`2YZJT0oOgm02L0
oooo0100mOGe09JFUP3_knl0TI6A03hn?P2h^;P0gmoO0;Rh^00n?Sh0^;Rh0=oOg`3oool0b/[:09VI
V@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI
08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm
0?knoP010?ooo`0000@000000?clo02:RXX0VIVIC@3oool00`3emOD0UYJF0?KfmP080>g]k@050?oo
o`3:b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0
oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[j
nP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVIC@3oool00`3emOD0UYJF0?Win@08
0?Cdm0050?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3l
o?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`
l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000D000000?clo02:RXX0VIVI0?ooo`1<0<k>cP03
0?Gem@2FUYH0o_kn00P0oOgm00D0oooo0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI
05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2I
VIU=0?ooo`030?Gem@2FUYH0jn_[00P0oooo00D0emOG0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VI
V@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000
o?cl08Z:RP2IVIT>0?ooo`030?Ogm`2YZJT0oOgm03`0oooo00H0m?Cd0>CTi039bLT0d];B0=?Cd`3D
e=@20=KFeP<0emOG00D0aLG50>o_k`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo
00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3Q
hN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIT>0?oo
o`030>o_k`1CDe<0no_k03`0oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0
n_[j0P3kno/0103clo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?cl
o02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIT20?ooo`0O0=gMg@1/
K6`0Ogmo07inOP24Q8@0g=cL05=CD`3kno/0g]kN03hn?P26QXH04a<C0=3@d00d=3@0OWin05YJFP3X
j>P0VIVI0?Ogm`2/[:`0d];B09VIV@3clo<0PX:207QhN01ZJVX0/K6a05YJFP1oOgl0LG5a0<?3``02
0?ooo`0T08j>SP1oOgl0D51@0<?3``3Ogml0YJFU0?_kn`2GUiL0h^;R07IfMP2=SHd0=cLg0>SXj03o
ool0l?3`05QHF01kNg/0OGem075aL@3Ti>@0HF5Q07UiN@1]KFd0jn_[04a<C01gMgL0HV9R0=;BdP2I
VIT0moOg0:b/[03Bd]80h^;R07IfMP2=SHd0Q8B41P3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cd
m03gmoL0n?Sh0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0
FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0
dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VI
V@80oooo01T0P8200>o_k`3oool0oooo07QhN039bLT0De=C0?_kn`36a/H03`l?0??cl`0V9RH0f=SH
079bLP3oool0N7Qh0;Bd]02IVIT0moOg0:b/[03Bd]80VIVI0=OGe`1CDe<0no_k00<0oooo00<0N7Qh
0000002d];@00P3oool03@1ZJVX0kno_08j>SP3Bd]80gmoO0:FUY@3kno/0UiNG0::RXP37alL0no_k
05eMG@3Xj>P01@3oool04P2RXZ80ZZZZ061PH03oool0N7Qh0<W9b@1CDe<0moOg08j>SP3Bd]80VIVI
0?Ogm`2/[:`0d];B0::RXP37alL0no_k09NGU`80oooo00h000000?ooo`3oool0oooo0??cl`3^k^h0
kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool0
0`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bL
W02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@0
00000?clo02:RXX0VIVI0P3oool02P2CTi<0oooo0?ooo`0l?3`000000<W9b@1CDe<0no_k0?ooo`3O
gml200T92@0A0=SHf01bLW80oooo07QhN02d];@0VIVI0?Ogm`2/[:`0d];B09VIV@3GemL0De=C0?_k
n`3oool0kno_018B4P0`<300103oool03@3_knl04Q8B00@4103Bd]80gmoO0:FUY@3kno/0UiNG0::R
XP37alL0no_k05eMG@3Xj>P00`3oool0503kno/07AdM00l?3`3Jf]X0H61P0?ooo`1hN7P0bLW905=C
D`3gmoL0[:b/0=;BdP2IVIT0moOg0:b/[03Bd]80XZ:R0<O7a`3kno/0UiNG0P3oool200000080oooo
00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?cl
o02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:
0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn
0040oooo000010000000o?cl08Z:RP2IVIT20?ooo`0O08V9R@3gmoL0oooo0?ooo`30`<00hn?S03@d
=02:RXX0inOW07YjNP1kNg/0ATI60>_[j`14A4@0QhN705YJFP3Xj>P0RHV908j>SP1PH600moOg09VI
V@3clo<0PX:207QhN01HF5P0^k^k075aL@1mOGd0FUYJ0>SXj0020?ooo`0T0;Bd]01kNg/0H61P0?Og
m`3Ogml0IFEU07]kN`2NWYh0h^;R06a/K02=SHd0=cLg0>SXj03oool0moOg06EUI@21PH40moOg0?oo
o`3gmoL0HF5Q07UiN@1JFUX0hn?S05=CD`3gmoL0[:b/0=;BdP2BTY80QXJ6061PH03gmoL0h^;R06a/
K02=SHd0Q8B40P3oool20000000<0;no_`3oool0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Sh
n03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0
VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40
j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIT20?ooo`06
0;^k^`1PH600oooo0>o_k`0Q8B40moOg2`3oool00`2IVIT0moOg0?ooo`0G0?ooo`0:0?_kn`1MGEd0
j>SX0?ooo`3_knl08B4Q0?Ogm`3Ogml06a/K0>SXj0h0oooo00D0no_k09NGU`3oool0oooo00000003
0?ooo`0:0??cl`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VI
V@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI
08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm
0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0`3oool0102k^k/0Ogmo0861P@3Xj>P<0?ooo`06
0<c<c03kno/0oooo0?ooo`3<c<`0no_k503oool00`3moOd0[Zj^0??cl`020?ooo`040>;RhP1oOgl0
Ogmo0=WIf@l0oooo00<0oOgm0<_;b`3oool01@3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03g
moL0n?Sh0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3o
ool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C
0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIVDd0
oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI
0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0
RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo000010000000o?cl08Z:RP2IVIU=0?ooo`0:0??cl`3^k^h0kno_0?3`l03alO40m?Cd
0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J
0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI
C@3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??cl`2I
VIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VI
V@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gm
o@3no_h00@3oool000040000003lo?`0RXZ:09VIVDd0oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3d
m?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI
05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2I
VIT20?ooo`0@08R8R01kNg/0Ogmo09NGU`3Ogml0S8b<05YJFP1oOgl0LG5a0<?3``2ATI40W9bL09:B
TP1oOgl0N7Qh0<?3``80oooo00d0g]kN05iNGP2>SXh0JVYZ0=oOg`1CDe<0nOWi05eMG@3Jf]X0LG5a
08B4Q00g=cL0d];B0080oooo01d0VIVI0?Gem@26QXH0no_k06=SH`3Vi^H0?Shn08J6QP0C4a<0e]KF
03Ph>02LW9`0UiNG0?Ogm`3_knl0De=C0;Zj^P1bLW80Ogmo0861P@3[jn/0K6a/07moO`2IVIT0c/k>
05YJFP1oOgl0LG5a0>7Qh@0=0?ooo`0:0??cl`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi
0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3l
o?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`
l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3oool0402IVIT0
moOg0?ooo`3oool0gmoO0:FUY@3oool0N7Qh0000002d];@0VIVI0?Ogm`3oool0gmoO00T92@2d];@2
0?ooo`0=0<K6aP1oOgl0moOg07alO03Ogml0De=C0?Win@1MGEd0^[Zj08n?S`3moOd0GEeM0=;BdP02
0?ooo`0B09VIV@3_knl0KVi^0>_[j`1SHf<0f][J00l?3`3clo<09RHV0=SHf01CDe<0moOg09VIV@3g
moL0kno_05=CD`2EUID0moOg0P3oool00`2RXZ80alO70?ooo`030?ooo`0307QhN0000000fMWI00T0
oooo00h000000?ooo`3oool0oooo0??cl`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[j
nP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0
VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00
m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3oool0402IVIT0moOg
0?ooo`3oool0gmoO08F5Q@1@D500UIFE09:BTP3Xj>P0VIVI0?Ogm`28R8P0Ogmo09jNWP3Xj>P20?oo
o`0=0>o_k`2>SXh0Ng]k07emO@3Ogml0>SXj0?KfmP14A4@0b/[:06QXJ03dm?@0?3`l0=;BdP020?oo
o`0C09VIV@2_[jl0_Kfm0:b/[01SHf<0kno_09VIV@20P8006QXJ0=oOg`1CDe<0moOg08^;R`3Ti>@0
kno_05=CD`2?Shl0i>CT0?ooo`020<30`008059BDP1oOgl0I6AT0<30`01@D500UIFE09:BTP3clo<9
0?ooo`80000000`0_kno0?ooo`3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X2
0?_kn`040??cl`2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VI
V@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?Kf
mP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIV@80oooo00l0VIVI0?Ogm`3o
ool0oooo0=oOg`2SXj<0E5AD0000002?Shl0no_k02XZ:P11@D40Xj>S00000017AdL0103oool0301A
DE4000000:n_[`3Ogml0JFUY00h>3P1PH600oooo07QhN00N7Qh0De=C0=;BdP80oooo01`0VIVI09bL
W03gmoL0NGUi06=SH`3gmoL0XZ:R0000001PH600YjNW00000011@D40TY:B04Y:BP1_Kfl0De=C0>_[
j`0B4Q80000007moO`3oool0DE5A0000002_[jl0no_k05ADE0000000Shn?2P3oool200000080oooo
00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?cl
o02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:
0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn
0040oooo000010000000o?cl08Z:RP2IVIT20?ooo`0609VIV@3gmoL0oooo0?ooo`3_knl0d];B103o
ool00`2QXJ40moOg0?ooo`0?0?ooo`030?gmo@1MGEd0d];B0080oooo00H0QHF509>CT`3no_h0Ogmo
02h^;P3gmoL30?ooo`030?Gem@1HF5P0moOg0080oooo00<0moOg0:VYZ@3moOd05@3oool03P1oOgl0
oooo0?ooo`3oool0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103c
lo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool0
4@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIU=0?ooo`0:0??cl`3^k^h0kno_0?3`l03a
lO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl
09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf
0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:
RXX0VIVIC@3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`04
0??cl`2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?oo
o`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0
o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIVDd0oooo00X0lo?c0>k^kP3_knl0l?3`
0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3l
o?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl
08Z:RP2IVIU=0?ooo`0:0??cl`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k
00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0
oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[j
nP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3oool0<03<c<`0no_k0=3@d03R
h^80/K6a0?Win@2>SXh0UIFE08j>SP3IfMT0a<C40:2PX03:b/X0no_k0=GEe@3Xj>P0oooo0:RXZ01o
Ogl0emOG0>o_k`3Bd]80kno_0:VYZ@3emOD0ZJVY0<;2`P1oOgl0W9bL0=WIf@3kno/0UiNG0>_[j`3B
d]80kno_08B4Q027QhL0moOg0<C4a01oOgl0W9bL0=WIf@3<c<`0no_k0?ooo`3gmoL0ZJVY0?gmo@80
oooo01<0^k^k07moO`2SXj<0j>SX0<c<c03kno/0eMGE0>SXj03oool0^k^k07moO`2o_kl0no_k09NG
U`3[jn/0ZJVY0>GUi@1oOgl0[:b/00H0oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Sh
n03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0
VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40
j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIT20?ooo`15
09VIV@3clo<0NGUi0;Jf]P1SHf<0f][J061PH03Rh^802@T90=;BdP2IVIT0moOg09VIV@3gmoL0[:b/
0=;BdP3Ogml0R8R80?ooo`23Ph<0gmoO0:FUY@3Ogml0De=C0>_[j`1CDe<0Shn?0?ooo`1WIfL0d];B
0=oOg`2UYJD0gmoO0:6QX@0V9RH0j>SX0<K6aP11@D40Shn?0?ooo`1WIfL0d];B09VIV@3gmoL0oooo
0>o_k`1CDe<0no_k0?ooo`3_knl08B4Q0>o_k`1mOGd0d];B09VIV@3gmoL0[:b/0=;BdP3gmoL0O7al
0?ooo`23Ph<0emOG0:FUY@3Ogml0De=C07alO03_knl0SXj>00H0oooo00<0mOGe0?3`l02GUiL00`2P
X:000`2OWil0X:2P0:2PX0020:2PX005092@T03emOD0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl
09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf
0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:
RXX0VIVI0P3oool0902IVIT0e]KF09JFUP2a/K40Hf=S0>SXj01gMgL0OWin01LG5`3Bd]80VIVI0?Og
m`2IVIT0moOg0:b/[03Bd]80XZ:R06=SH`1oOgl0UiNG0=oOg`2UYJD0gmoO05=CD`3[jn/0De=C0:RX
Z020P800BDU90=;BdP3Ogml0YJFU0=oOg`2EUID0De=C0?_kn`80oooo01H0[:b/0820P019BDT0d];B
09VIV@3gmoL0oooo0>o_k`1CDe<0no_k0?ooo`3_knl0De=C0?Ogm`2MWId0d];B09VIV@3gmoL0[:b/
0=;BdP3Ogml0YJFU0P3oool01`3Ogml0YJFU0=oOg`1CDe<0Shn?0?ooo`2k^k/00P3oool01`000000
Ogmo0?ooo`3oool0mOGe09JFUP3YjNT00P3Ph>00103Ng]h0fm_K0=SHf03De=@20=7Ad@050>k^kP3:
b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo
0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3l
o?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3oool0A@2IVIT0X:2P0=;BdP2XZ:P0
Hf=S0>OWi`1CDe<0moOg02`/;03_knl0VIVI0?Ogm`1mOGd0d];B096AT@3_knl0gmoO0:FUY@3oool0
V9RH0=oOg`1fMWH0a/K600T92@3<c<`0H61P09VIV@3gmoL0TI6A0>o_k`3Ogml0YJFU0=oOg`2QXJ40
9RHV0>SXj02RXZ80H61P09VIV@3gmoL0TI6A0>o_k`2IVIT0moOg0?ooo`3_knl04Q8B00000011@D40
jn_[05=CD`3gmoL0[:b/0=;BdP20P800kno_096AT@3_knl0moOg07alO03oool0PH610=oOg`2UYJD0
gmoO05=CD`1lO7`0kno_08j>SP020?ooo`80000000D0Ogmo0?ooo`3emOD0UYJF0?_kn`020>g]k@04
0>WYj@3Shn<0g=cL0=KFeP80d=3@00D0oooo0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0
VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:
RP2IVIT20?ooo`0?09VIV@2LW9`0moOg0;Rh^01SHf<0moOg0::RXP000000H61P0?_kn`0Z:RX0@D51
09:BTP1:BTX0<30`0080oooo02P0DE5A0000002_[jl0gmoO065QH@000000/;2`02XZ:P2d];@0d=3@
0000000`<300no_k05ADE00P8200emOG0:FUY@3Ogml02@T900l?3`3_knl0d=3@0000000`<300oooo
09VIV@3gmoL0oooo0>o_k`1CDe<0no_k0?ooo`3_knl0De=C0?Ogm`2/[:`0d];B0:BTY00M7Ad0<30`
0P3oool02P1hN7P0000007emO@1DE5@0820P0=OGe`1CDe<0c<c<00000017AdL20?ooo`8000000P3o
ool00`3emOD0UYJF0?_kn`020>g]k@040>[ZjP3Ti>@0g=cL0=KFeP80d=3@00D0oooo0<[:bP2IVIT0
o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0
RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo000010000000o?cl08Z:RP2IVIT20?ooo`0608F5Q@2CTi<0oooo0;jn_P0^;Rh0moOg
103oool0102QXJ40moOg09VIV@3gmoL@0?ooo`040>_[j`2WYjL0kno_0=;BdPP0oooo00P0VIVI0?Og
m`3oool0kno_038b<P1mOGd0Ogmo0>?Sh``0oooo00D0jn_[0:NWY`3_knl0ZJVY0?gmo@040?ooo`07
07moO`3oool0oooo0?ooo`3emOD0UYJF0?_kn`020>g]k@040>WYj@3Shn<0gMgM0=KFeP80d=3@00D0
oooo0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0
FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIU=0?ooo`030?Gem@2FUYH0no_k
0080kNg]102EUID01`3?cll0d=3@0?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VI
V@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5
a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0
VIVIC@3oool01@3emOD0UYJF0?_kn`3]kNd0m_Kf00<0b/[:00P0]KFe0<o?c`3@d=00oooo0<[:bP2I
VIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VI
V@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gm
o@3no_h00@3oool000040000003lo?`0RXZ:09VIVDd0oooo00@0mOGe09JFUP3kno/0kNg]103oool0
203Fe]H0d=3@0=3@d03oool0b/[:09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`03
09VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI2@3oool0
1034a<@0Ogmo07]kN`2QXJ4V0?ooo`0309VIV@3gmoL0oooo01L0oooo00D0mOGe09JFUP3kno/0kNg]
0?KfmP030<[:bP080;Jf]P3@d=00d=3@0?ooo`3:b/X0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0
VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00001`000000o?cl08Z:
RP2IVIT0oooo0?_kn`2YZJT00`3oool07036a/H0WinO05=CD`3Ng]h0000000l?3`2=SHd0oooo055A
D@000000[jn_0=oOg`1^KVh0:RXZ08n?S`3_knl0De=C0?_kn`3gmoL07AdM02/[:`0M7Ad0bLW905=C
D`3[jn/04Q8B0000001oOgl30?ooo`0C0:NWY`00000000000<O7a`3oool0DE5A0000003IfMT0gmoO
05=CD`3[jn/0De=C0>_[j`1CDe<0Shn?018B4P000000alO70?ooo`0203`l?00=09:BTP3gmoL07AdM
09RHV00M7Ad0:b/[01dM7@3EeMD07AdM0451@@2EUID0moOg055AD@060?ooo`030?Gem@2FUYH0no_k
0080kNg]102EUID20=3@d0050?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J
0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI
0P3oool08@2CTi<0oooo0?ooo`3moOd0GEeM0=SHf01CDe<0SHf=0>o_k`3gmoL0J6QX0=oOg`28R8P0
oooo08>3P`3Ogml06a/K0>CTi00V9RH0f=SH05=CD`3kno/0a/K6061PH03Rh^802@T90=SHf01CDe<0
RXZ:0=;BdP3oool0Ph>30?Ogm`020?ooo`0R08>3P`3oool0moOg05IFEP3Ogml0R8R80?Ogm`1lO7`0
gmoO05=CD`3[jn/0De=C0>_[j`1CDe<0O7al0>o_k`3gmoL0O7al0=oOg`28R8P0gmoO07IfMP3Ogml0
De=C0=kNgP1PH600h^;R00T92@3Hf=P0De=C0?Ogm`2IVIT0emOG08R8R0H0oooo00@0mOGe09JFUP3k
no/0kNg]103oool0203Fe]H0clo?0=3@d03oool0b/[:09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl
09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf
0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:
RXX0VIVI0P3oool0202][Jd0AdM707moO`1MGEd0NWYj0>g]k@1CDe<0Shn?0P3oool0502FUYH0XZ:R
06=SH`1oOgl0UiNG0=oOg`1CDe<0no_k0::RXP2g]kL0De=C0?_kn`3Rh^80MgMg07inOP0G5aL0f=SH
05=CD`2EUID0moOg103oool00`3[jn/0VIVI0?ooo`020?ooo`0;0<K6aP2_[jl0oooo09JFUP3Ogml0
De=C0>_[j`1CDe<0jn_[05=CD`2TY:@00P3oool04@2FUYH0gmoO0:FUY@3inOT0RHV90=oOg`1CDe<0
k>c/07MgM`1nOWh05aLG0=SHf01CDe<0moOg09VIV@2n_[h0[jn_0080oooo00L0000007moO`3oool0
oooo0?Gem@2FUYH0no_k0080kNg]00/0jNWY0>?Sh`3Lg=`0e]KF0<o?c`3@d=00oooo0<[:bP2IVIT0
o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0
RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo000010000000o?cl08Z:RP2IVIT20?ooo`0P0?_kn`0/;2`0kno_09VIV@3gmoL0kno_
05=CD`2:RXX0d];B0?Ogm`1XJ6P0gmoO0:FUY@3oool0V9RH0=oOg`0K6a/0i>CT02HV9P3Hf=P06a/K
0>SXj03Ogml0De=C0?Ogm`0/;2`0inOW05=CD`2:RXX0d];B0?ooo`21PH420?ooo`030=oOg`2UYJD0
oooo0080oooo01h0gmoO08R8R03gmoL0O7al0=oOg`0K6a/0c<c<00T92@3<c<`0H61P07emO@3Bd]80
moOg07alO03Ogml0YJFU0?_kn`2GUiL0gmoO05=CD`3[jn/0De=C0?Ogm`0/;2`0inOW05=CD`3gmoL0
VIVI0=OGe`28R8P20?ooo`80000000D0Ogmo0?ooo`3emOD0UYJF0?_kn`020>g]k@0;0>WYj@3Shn<0
g=cL0=KFeP3?cll0d=3@0?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?oo
o`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0`3o
ool07`2[Zj/0TI6A0:ZZZP3oool0kno_05=CD`3/k>`0Ogmo08N7Q`36a/H0oooo0:RXZ01oOgl0emOG
0=oOg`1@D500T92@0<O7a`3gmoL0YZJV09FEU@3gmoL0d=3@07moO`2_[jl0moOg0:VYZ@3emOD0R8R8
07moO`2o_kl00P3oool08`3kno/0R8R80?ooo`3kno/0[Jf]0?ooo`2XZ:P0Ogmo0>?Sh`3_knl0][Jf
09BDU03GemL0U9BD0=WIf@3Bd]80ZJVY07moO`3Shn<0kno_0=;BdP3moOd0b/[:09VIV@0Y:BT0bl_;
0=3@d01oOgl0[jn_0<_;b`0Y:BT0W9bL0<[:bP3kno/0Z:RX0080oooo00L0000007moO`3oool0oooo
0?Gem@2FUYH0lo?c0080m_Kf00@0m?Cd0?7al@3]kNd0j^[Z0P3WinL01@3[jn/0b/[:09VIV@3lo?`0
VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0
o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h0
0@3oool000040000003lo?`0RXZ:09VIV@@0oooo00H0N7Qh0<O7a`3oool0kno_05=CD`3kno/70?oo
o`030=oOg`1CDe<0no_k00P0oooo00<0kno_05=CD`3kno/01P3oool0102WYjL0000000000037alLB
0?ooo`030?Ogm`1NGUh0no_k00<0oooo00D0no_k05iNGP3gmoL0VIVI0?Ogm`070?ooo`030?Gem@3m
oOd0YJFU00<0^;Rh00<0]KFe0;Rh^02h^;P00P2h^;P01@2CTi<0mOGe09VIV@3lo?`0VIVI05X0oooo
00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00004
0000003lo?`0RXZ:09VIVDd0oooo00<0mOGe0<;2`P2i^KT0202m_Kd01@2e]KD0g]kN09VIV@3lo?`0
VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0
o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h0
0@3oool000040000003lo?`0RXZ:09VIVDd0oooo00<0mOGe09JFUP3Vi^H0203>c/h01@3oool0b/[:
09VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A
09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl
0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIVDd0oooo00<0mOGe09JFUP3Vi^H0203>c/h0
1@3oool0b/[:09VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VI
V@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?Kf
mP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIVC<0oooo00<0kno_05=CD`3g
moL05`3oool00`3emOD0UYJF0>KViP080<k>cP050?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool00`2I
VIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02V
YZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@00000
0?clo02:RXX0VIVI0P3oool0A@2IVIT0moOg0?ooo`1mOGd0d];B0;^k^`000000AdM70=;BdP3Ogml0
YJFU0=oOg`2UYJD0gmoO0:FUY@2RXZ800000030`<03oool0VIVI0?Ogm`3_knl0De=C0?Ogm`1hN7P0
000007moO`3gmoL03Ph>05MGE`0M7Ad0bLW905=CD`3kno/0oooo0?gmo@1DE5@0000004M7A`3oool0
d=3@0000001JFUX0oooo0=oOg`2UYJD0gmoO0:FUY@3Ogml0UIFE04Q8B00B4Q80AdM70?ooo`2k^k/0
000004M7A`3Bd]80moOg02l_;`3_knl03Ph>05MGE`1XJ6P0moOg01dM7@2<S8`0De=C0<c<c0060?oo
o`050?Gem@2FUYH0i^KV0<k>cP0n?Sh00`3>c/h0200n?Sh0c/k>0<k>cP3oool0b/[:09VIV@3lo?`0
VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP01
0?ooo`0000@000000?clo02:RXX0VIVI0P3oool0A@2IVIT0moOg0>o_k`1BDU80d];B08b<S03clo<0
QHF50=;BdP3Ogml0YJFU0=oOg`2UYJD0gmoO08F5Q@1HF5P0moOg0=gMg@37alL0VIVI0?Ogm`3_knl0
De=C0>?Sh`2@T900oooo0<71`@32`/80UiNG0=kNgP0G5aL0f=SH05=CD`3kno/0oooo0:JVYP2o_kl0
oooo0<C4a037alL0RHV90?Ogm`2TY:@0`l?30=oOg`2UYJD0gmoO0:FUY@3Ogml0UIFE04U9B@3gmoL0
Y:BT0<?3``2<S8`0lo?c08F5Q@3Bd]80gmoO0:FUY@36a/H0UiNG0=kNgP1dM7@0gmoO05=CD`3[jn/0
De=C08F5Q@020?ooo`0D07moO`3oool0oooo0?ooo`3emOD0UYJF0>SXj029RHT0?Shn069RHP3AdM40
HV9R03hn?P2][Jd0dM7A0?ooo`3:b/X0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo
00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3Q
hN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIT20?oo
o`0?09VIV@3gmoL0PH6109FEU@3Bd]80VIVI0?Ogm`2/[:`0d];B0=oOg`2UYJD0gmoO0:FUY@3Ogml0
MWIf00<0000000L0H61P09VIV@3gmoL0kno_05=CD`3GemL0YJFU00<0oooo00T0N7Qh000000092@T0
f=SH05=CD`3kno/0oooo09VIV@3gmoL00`3oool07@2CTi<0oooo0;^k^`2d];@0gmoO0:FUY@3Ogml0
YJFU0=oOg`2EUID0LW9b0?ooo`2k^k/0];Bd09VIV@3gmoL0[:b/0=;BdP3Ogml0YJFU0?ooo`1hN7P0
000006e]K@3Ogml0De=C0>_[j`1CDe<0Shn?0080oooo0P0000020?ooo`030?Gem@2FUYH0kNg]0080
fm_K0`0n?Sh30=_Kf`050?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?oo
o`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3o
ool0702IVIT0Z:RX0::RXP2/[:`0d];B09VIV@3gmoL0[:b/0=;BdP3Ogml0Jf][06=SH`1gMgL0Hf=S
0;bl_01jNWX0Ng]k0820P037alL0RHV908j>SP2_[jl0De=C0??cl`1jNWX0Ogmo0820P03_knl207]k
N`0O04I6AP3[jn/0De=C0?_kn`3oool0UIFE0??cl`3oool0m_Kf0?7al@2XZ:P0MgMg06YZJP3Xj>P0
gmoO06][J`1SHf<0MgMg06=SH`2d];@0A4A409>CT`1ZJVX0j>SX09VIV@3gmoL0[:b/0=3@d02IVIT0
HV9R0>_[j`0207]kN`0609fMW@2IVIT0:BTY0;^k^`1CDe<0Y:BT0P3oool2000000050;no_`3oool0
mOGe09JFUP3clo<00P3Xj>P00`2o_kl0?Shn0;no_`030>SXj0050?ooo`3:b/X0VIVI0?clo02IVIT0
FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0
000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?oo
o`0000@000000?clo02:RXX0VIVI0P3oool01@1hN7P0H61P0?ooo`2/[:`0d];B01X0oooo00T0kno_
05=CD`3kno/0oooo0;^k^`2?Shl0oooo08j>SP3Bd]804P3oool00`3Ogml0YJFU0?ooo`030?ooo`03
0=oOg`1CDe<0no_k00@0oooo00L000000?ooo`3oool0oooo0?Gem@2FUYH0nOWi00P0m?Cd00D0oooo
0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3o
ool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIT20?ooo`0507emO@3Bd]80oooo0:b/
[03Bd]80403oool00`3_knl0De=C0?Ogm`070?ooo`080>o_k`1CDe<0no_k0?ooo`3moOd0E5AD0000
0017AdLC0?ooo`030?Ogm`2ZZZX0oooo00<0oooo00D0moOg05iNGP3[jn/0De=C0?_kn`060?ooo`03
0?Gem@2FUYH0oOgm00P0o?cl00D0oooo0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI
05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2I
VIU=0?ooo`030?Gem@2FUYH0mOGe00P0oooo00D0jn_[0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VI
V@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000
o?cl08Z:RP2IVIU=0?ooo`030?Gem@3Jf]X0Y:BT00L0]KFe00H0];Bd08n?S`3/k>`0VIVI0?clo02I
VIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?cl
o0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040
oooo000010000000o?cl08Z:RP2IVIUJ0<c<c00309VIV@3lo?`0VIVI05X0c<c<00<0VIVI0?clo02I
VIT0FP3<c<`00`2IVIT0o?cl09VIV@1J0<c<c00A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000050000003lo?`0RXZ:
09VIV@3moOd0o`3lo?a`0?clo00B0?gmo@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:09VIV@3m
oOd0o`3lo?a`0?clo00B0?gmo@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40
j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:09VIV@3moOd0o`3l
o?a`0?clo00B0?gmo@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:09VIV@3moOd0o`3lo?a`0?cl
o00B0?gmo@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:09VIV@3moOd0o`3lo?a`0?clo00B0?gm
o@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0@3oool000040000003lo?`0RXZ:09VIVGT0o?cln03oool04@2IVIT0RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo
000010000000o?cl08Z:RP2IVIUh0?clo00309VIV@2][Jd0[Jf]06P0[Jf]00<0WYjN0:j^[P2][Jd0
RP2][Jd04P2^[Zh0_[jn08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00
m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo000010000000o?cl08Z:RP2IVIUh0?clo00309VIV@3:b/X0
b/[:06P0b/[:00<0Xj>S0<?3``3:b/X0RP3:b/X04P33`l<0hn?S08Z:RP3lo?`0000009bLW02VYZH0
][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo000010000000o?cl
08Z:RP2IVIUh0?clo00309VIV@3De=@0e=CD06P0e=CD00<0Xj>S0<?3``3De=@0RP3De=@04P33`l<0
hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0
oOgm0?knoP40oooo000010000000o?cl08Z:RP2IVIUh0?clo00309VIV@3Ogml0gmoO01<0gmoO00L0
00000=oOg`3Ogml0gmoO0000001_Kfl0gmoO00<000000P3Ogml3000000@0gmoO00X0Kfm_00000000
0000gmoO06m_K`000000gmoO06m_K`000000gmoO0P0000000`3Ogml0g]kN0=kNgP020=kNgP@00000
0`3Ng]h40000000<06m_K`000000Kfm_0000001_Kfl0gmoO0000001_Kfl0000006m_K`3Ogml00000
0P3Ogml2000001D0gmoO00<0g]kN0:>SX`33`l<06P3Ogml010000000Kfm_0=oOg`3Ogml200000004
0=oOg`000000Kfm_0=oOg`<00000103Ogml020000000gmoO0=oOg`3Ogml0000006m_K`3Ng]h0Kfm_
0`0000002@3Ng]h0Kfm_0000001_Kfl000000=kNgP1_Kfl000000=kNgP03000000090=kNgP1_Kfl0
000006m_K`000000Kfm_0000003Ogml0Kfm_00<0000000H0gmoO000000000000Kfm_0000003Ogml3
00000080gmoO0`000000101_Kfl0gmoO0=oOg`3Ogml30000000406m_K`3Ogml0gmoO06m_K`800000
0P3Ogml3000000040=oOg`1_Kfl000000=oOg`8000005P3Ogml04`3Ng]h0`l?30>?Sh`2:RXX0o?cl
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3o
ool000040000003lo?`0RXZ:09VIVAX0o?cl0`1nOWh30?clo0<0OWin00<0_Kfm07inOP2m_Kd00P1n
OWh00`3lo?`0OWin0;fm_@030?clo00607inOP2m_Kd0OWin0;fm_@3lo?`0OWin103lo?`00`1nOWh0
o?cl0?clo0020?clo00:0;fm_@1nOWh0OWin07inOP2m_Kd0OWin0?clo02m_Kd0OWin0?clo0<0OWin
0P3lo?`01@1nOWh0_Kfm07inOP2m_Kd0o?cl00<0OWin00L0o?cl07inOP2m_Kd0o?cl07inOP2m_Kd0
o?cl0080OWin6`3lo?`00`2IVIT0i>CT0>CTi00C0>CTi00?03Xj>P1`L700L71`071`L0000000[:b/
0000001bLW80i>CT0000003Ti>@00000079bLP3Ti>@0000000@0i>CT00d000000>CTi01bLW800000
079bLP000000i>CT079bLP000000[:b/07AdM00h>3P0M7Ad00<0i>CT00@0LW9b0000001bLW800000
103Ti>@00`000000LW9b0>CTi0020>CTi00@0000001bLW800000079bLP3Ti>@00000079bLP000000
>3Ph0000002/[:`0i>CT07AdM01bLW8000000:b/[1D0i>CT00<0Xj>S0<?3``3Ti>@06@3Ti>@01000
0000LW9b0>CTi03Ti>@2000000080>CTi0000000LW9b0000001bLW80i>CT07AdM02/[:`30>CTi008
0000003Ti>@00000079bLP000000LW9b0>CTi00000020>CTi00<0000003Ti>@000000>CTi01bLW80
00000>CTi01bLW800000079bLP000000i>CT0P1dM7@04`1bLW800000079bLP000000LW9b0000003T
i>@000000>CTi01bLW800000079bLP000000i>CT079bLP000000LW9b0000003Ti>@00P1dM7@01@1b
LW8000000>CTi0000000LW9b00<0i>CT00@00000079bLP3Ti>@000000P3Ti>@03@000000i>CT079b
LP000000LW9b0000003Ti>@0LW9b0000001bLW80000003Ph>01dM7@0603Ti>@04P33`l<0hn?S08Z:
RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?kn
oP40oooo000010000000o?cl08Z:RP2IVITJ0?clo0050000003lo?`0OWin0000003lo?`00P000000
1@1nOWh0000007inOP000000OWin0080000000<0OWin0000003lo?`00`3lo?`01P000000OWin0000
001nOWh0o?cl000000@0o?cl00<000000?clo03lo?`00P3lo?`030000000o?cl000000000000OWin
0000003lo?`0OWin0000001nOWh000000?clo080000000D0OWin0000003lo?`0000007inOP020000
000<0?clo0000000OWin0000001nOWh0o?cl0000001nOWh0000007inOP000000OWin6P3lo?`00`2I
VIT0k>c/0>c/k00C0>c/k00307IfMP000000k>c/0080000000/0k>c/0000001fMWH0k>c/0000003/
k>`0000007IfMP3/k>`000000>c/k0020000000=0>c/k0000000k>c/07IfMP000000MWIf0000003/
k>`0MWIf0000003/k>`0000007IfMP040>c/k00307IfMP0000000000008000000`3/k>`00`000000
MWIf0>c/k0020>c/k0070000001fMWH0000007IfMP3/k>`0000007IfMP02000000@0k>c/00<00000
0>c/k03/k>`05@3/k>`00`2SXj<0`l?30>c/k00I0>c/k0040000001fMWH0k>c/0>c/k080000000<0
k>c/0000001fMWH010000000101fMWH0k>c/0>c/k03/k>`50000000307IfMP3/k>`0MWIf00<00000
00T0k>c/0000003/k>`0MWIf0000003/k>`0MWIf0000001fMWH010000000201fMWH0000007IfMP00
0000MWIf0000003/k>`0MWIf0`0000001`1fMWH000000>c/k01fMWH0000007IfMP000000103/k>`3
0000000807IfMP3/k>`0k>c/0>c/k0000000MWIf0>c/k00000020>c/k00<0000003/k>`0MWIf0000
001fMWH000000>c/k01fMWH0000007IfMP000000MWIf6@3/k>`04P33`l<0hn?S08Z:RP3lo?`00000
09bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo0000
10000000o?cl08Z:RP2IVITJ0?clo0050000003lo?`0OWin0000003lo?`00P0000001@3lo?`00000
07inOP000000OWin0080000000<0OWin0000003lo?`00`3lo?`01P000000OWin0000001nOWh0o?cl
000000@0o?cl00<000000?clo03lo?`00P3lo?`02`000000o?cl07inOP000000OWin0000003lo?`0
OWin0000001nOWh0000000<0o?cl00D0OWin0000003lo?`0000007inOP020000000;0?clo0000000
OWin0000001nOWh0o?cl0000001nOWh0o?cl07inOP0000006`3lo?`00`2IVIT0k>c/0>c/k00C0>c/
k00307IfMP000000k>c/0080000000X0k>c/0000001fMWH0k>c/0000003/k>`0000007IfMP3/k>`0
0000103/k>`01@000000k>c/07IfMP000000MWIf0080000000H0MWIf0000001fMWH0000007IfMP00
00040>c/k0030000001fMWH0k>c/00@0k>c/00<0000007IfMP3/k>`00P3/k>`01`000000MWIf0000
00000000k>c/0000001fMWH00`0000020>c/k0040000001fMWH0000007IfMQD0k>c/00<0Xj>S0<?3
``3/k>`06@3/k>`010000000MWIf0>c/k03/k>`20000000807IfMP000000MWIf0000001fMWH0k>c/
0000001fMWH30>c/k0D0000000<0MWIf0>c/k03/k>`00P3/k>`01@000000k>c/0000003/k>`0MWIf
0080000000D0MWIf0000001fMWH000000>c/k0020000000607IfMP000000MWIf0000001fMWH00000
0`3/k>`02P1fMWH0000007IfMP000000k>c/07IfMP000000MWIf0000003/k>`2000000<0k>c/00<0
000007IfMP3/k>`00P3/k>`30000000=07IfMP3/k>`0k>c/0000003/k>`0MWIf0000001fMWH00000
0>c/k01fMWH0000007IfMP02000001T0k>c/0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf
]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000@000000?clo02:
RXX0VIVI6P3lo?`01@000000OWin03lo?`1nOWh0o?cl0080000000D0o?cl0000001nOWh0000007in
OP020000000307inOP000000o?cl0080OWin00L0o?cl0000001nOWh0000003lo?`3lo?`0000000@0
o?cl00<0000007inOP1nOWh00P3lo?`01P000000o?cl07inOP000000OWin00000080OWin00@00000
07inOP000000o?cl0`1nOWh010000000o?cl0000001nOWh2000000070?clo0000000OWin0000000o
?cl0o?cl0000000207inOP0303lo?`1nOWh0_Kfm01X0o?cl00<0VIVI0?Cdm03dm?@0503dm?@30000
00<0m?Cd0P1jNWX00`000000m?Cd0?Cdm00207YjNP030000003dm?@0m?Cd0080m?Cd00D0]kNg07Yj
NP1jNWX0m?Cd0;Ng]`0307YjNP80m?Cd0P1jNWX40?Cdm0030;Jf]P000000?3`l00D0m?Cd00<00000
07UiN@3dm?@00P3dm?@00`1jNWX0]kNg07YjNP0307YjNP050?Cdm0000000NGUi07YjNP2g]kL00P3d
m?@207YjNQH0m?Cd00<0Xj>S0<?3``3dm?@06@3dm?@010000000NGUi0?Cdm03dm?@200000080NWYj
0P3dm?@307YjNP@0m?Cd0P0000001P3dm?@0NGUi0000001iNGT0m?Cd0;Ng]`80NWYj00H0]kNg0?Cd
m0000000NWYj07UiN@00000207YjNP80m?Cd0`1jNWX00`3dm?@0]kNg07YjNP0307YjNP80m?Cd00X0
]kNg07YjNP1jNWX0m?Cd07UiN@000000NWYj0;Ng]`1jNWX0m?Cd0`1jNWX20?Cdm0<0NWYj103dm?@0
10000000NGUi0?Cdm00000020?Cdm0030;Ng]`1jNWX0NWYj0080m?Cd0`1jNWX01@3dm?@0NGUi0000
002g]kL0NWYj01P0m?Cd0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Q
h@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0VIVI6P3lo?`0
1@000000o?cl07inOP000000o?cl0080000000D0o?cl0000001nOWh0000007inOP020000000307in
OP000000OWin00<0o?cl00<0000007inOP0000000P0000000`1nOWh0o?cl0?clo0020?clo0030000
003lo?`0o?cl0080o?cl00D000000?clo01nOWh0000007inOP0300000080o?cl0`0000001`3lo?`0
OWin0000001nOWh0000007inOP3lo?`00`0000000`3lo?`000000000000200000080o?cl0P00000K
0?clo00309VIV@3jn_X0n_[j01D0n_[j00<000000?[jnP3jn_X0103jn_X00`000000n_[j0?[jnP02
0?[jnP030000003jn_X0n_[j0180n_[j00<0OGem0000000000001@3jn_X00`000000OGem0?[jnP09
0?[jnP030000001mOGd0n_[j01/0n_[j00<0Xj>S0<?3``3jn_X06@3jn_X010000000OGem0?[jnP3j
n_X2000000/0n_[j00H000000?[jnP3jn_X0n_[j0000001mOGd60?[jnP040000003jn_X0OGem0000
01<0n_[j00<0OGem0000003jn_X03`3jn_X010000000OGem0?[jnP00000;0?[jnP0307emO@000000
n_[j01T0n_[j0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0
l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0VIVI6P3lo?`010000000
o?cl07inOP0000080?clo080000000<0OWin0000003lo?`03@3lo?`00`000000o?cl0?clo00@0?cl
o00307inOP000000o?cl02X0o?cl00<0VIVI0?ooo`3oool05@3oool00`1oOgl0oooo0?ooo`040?oo
o`0307moO`3oool0oooo0080oooo00<0Ogmo0?ooo`3oool04`3oool00`1oOgl0_kno0?ooo`040?oo
o`0307moO`2o_kl0oooo0080oooo00<0Ogmo0;no_`3oool0103oool00`1oOgl0_kno0?ooo`0K0?oo
o`030:>SX`33`l<0oooo01P0oooo101oOgl00`3oool0Ogmo07moO`0;0?ooo`0607moO`3oool0oooo
0?ooo`1oOgl0_kno203oool00`2o_kl0Ogmo0?ooo`0E0?ooo`030;no_`1oOgl0oooo00`0oooo0`1o
Ogl00`2o_kl0oooo0?ooo`090?ooo`030;no_`1oOgl0oooo01T0oooo0180`l?30>?Sh`2:RXX0o?cl
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?oo
o`0000@000000?clo02:RXX0VIVI6P3lo?`3000000L0o?cl00@0000007inOP00000000001P3lo?`0
0`000000OWin0?clo0070?clo0@000004P3lo?`00`000000OWin0?clo00W0?clo00309VIV@3oool0
oooo06L0oooo00@0nOWi0:>SX`33`l<0oOgmR@3oool04`3dm?@0`l?30>?Sh`2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00004
0000003lo?`0RXZ:09VIVGP0o?cl00<0VIVI0?Shn03oool0I`3oool01@3Vi^H0aLG50=?Cd`3EeMD0
lo?c08L0oooo01@0n_[j0=GEe@3Cdm<0hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00001@000000o?cl08Z:RP2VYZH0_[jn
07H0a<C400<0_[jn0:JVYP2/[:`0J02SXj<01@2o_kl0hn?S0>?Sh`3De=@0][Jf08L0Xj>S01@0ZJVY
0<k>cP3Shn<0hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf
0?[jnP3lo?`0oOgm0?knoP40oooo00001@000000o?cl08Z:RP36a/H0[jn_07H0a<C400<0/K6a0<K6
aP31`L40:`31`L401@2b/[80hn?S0:>SX`31`L40`/;203H0`L7100@0`l?30:>SX`31`L40`/;2>031
`L400`33`l<0Xj>S0<71`@020<;2`Tl0`L710180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf
]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000D000000?clo02:
RXX0hN7Q0;Vi^@170;:b/P030:b/[02LW9`0/[:b02L0/[:b0`1;Bd/0101nOWh0/[:b0;no_`3Ng]h/
0<c<c0030;Ng]`3Shn<0Xj>S03P0c<c<00<0`l?30:>SX`3<c<`0>@3<c<`00`33`l<0Xj>S0<c<c01A
0<c<c00B0<?3``3Shn<0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03f
m_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:0>3Ph031`L40A`3IfMT00`33`l<0
Xj>S0=WIf@0Z0=WIf@030000003IfMT0fMWI02d0fMWI00<0_[jn0>?Sh`2SXj<0>03IfMT00`33`l<0
Xj>S0=WIf@0i0=WIf@030<?3``2SXj<0fMWI0540fMWI0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0
YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000D00000
0?clo02:RXX0h>3P0<71`@0@0>3Ph08000000`3Ph>002@000000Kfm_0>3Ph01_Kfl0>3Ph071`L00g
=cL0h>3P000000020>3Ph0040000003Ph>00=cLg071`L0<0gmoO00`0Kfm_0000001`L700L71`0:RX
Z03Ogml0000006m_K`3Ph>0000000>3Ph00g=cL2071`L0030000003Ph>00h>3P00h0h>3P00<0gmoO
0<?3``2SXj<0403Ph>000`1_Kfl0>3Ph071`L0020000000D0>3Ph02WYjL00000071`L0000000YjNW
06m_K`000000Z:RX06m_K`0h>3P0h>3P0000003Ph>00000006m_K`3Ph>0000000>3Ph00g=cL2071`
L00>0000003Ph>00h>3P0=oOg`2WYjL00000071`L00h>3P0Kfm_0=oOg`1_Kfl0>3Ph0=oOg`0g=cL2
071`L00A0000003Ph>0000000>3Ph02WYjL00000071`L02WYjL00000071`L0000000YjNW06m_K`00
0000L71`0000001_Kfl03`3Ph>000`31`L40hn?S0:>SX`0A0>3Ph00@0000001_Kfl0h>3P0>3Ph02W
YjL00000071`L0000000YjNW06m_K`000000>3Ph0000001_Kfl0000006m_K`<0gmoO4`3Ph>000`3O
gml0`l?30:>SX`0@0>3Ph0040000001`L700L71`03Lg=`80h>3P00X0Kfm_03Ph>01`L70000000:NW
Y`000000Kfm_0000001_Kfl000000P1_Kfl020000000L71`0000002WYjL0Kfm_03Ph>0000000Kfm_
3`3Ph>000`3Ogml0`l?30:>SX`0@0>3Ph00D0000001_Kfl0Kfm_0:NWY`000000Kfm_0>3Ph01_Kfl0
>3Ph071`L0000000h>3P03Lg=`1`L70000000:NWY`1_Kfl0>3Ph0>3Ph00g=cL2071`L00303Lg=`3P
h>00000000@0gmoO00L0Kfm_0000003Ogml0000006m_K`3Ogml000000080Kfm_00d0>3Ph071`L000
0000h>3P06m_K`0h>3P0L71`03Lg=`3Ph>0000000>3Ph0000000Kfm_00h0h>3P01<0gmoO0<?3``3S
hn<0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03m
oOd0o_kn0040oooo00001@000000o?cl08Z:RP3Ph>00`L710100j>SX0P0000030>SXj0090000001d
M7@0j>SX0000001dM7@0j>SX0000003Xj>P000000080j>SX00<000000>SXj0000000103Xj>P00`1d
M7@000000>SXj0030>SXj0060000001dM7@0j>SX0000003Xj>P000000P3Xj>P00`000000j>SX0>SX
j00?0>SXj0030<?3``2SXj<0j>SX00l0j>SX00P0000007AdM03Xj>P0M7Ad0000003Xj>P0M7Ad0000
00<0j>SX0P000000303Xj>P0000007AdM03Xj>P000000>SXj0000000M7Ad0>SXj0000000j>SX0000
0080j>SX00<000000>SXj03Xj>P0103Xj>P01`1dM7@000000>SXj0000000M7Ad0>SXj00000000P3X
j>P02@000000j>SX0000003Xj>P0M7Ad0000003Xj>P0M7Ad000000030>SXj080000000<0j>SX0000
001dM7@03`3Xj>P00`35aLD0hn?S0:>SX`0A0>SXj0090000001dM7@0j>SX0>SXj01dM7@000000>SX
j0000000M7Ad0080000000D0j>SX0000001dM7@0000007AdM00G0>SXj0030<?3``2SXj<0j>SX00l0
j>SX00P000000>SXj03Xj>P0M7Ad0000003Xj>P0000007AdM0<0j>SX00H0000007AdM0000000M7Ad
0000001dM7@2000000060>SXj0000000M7Ad0>SXj01dM7@000004@3Xj>P00`33`l<0Xj>S0>SXj00?
0>SXj00;0000001dM7@0000007AdM0000000M7Ad0>SXj0000000M7Ad0>SXj00000000P3Xj>P20000
00050>SXj0000000M7Ad0>SXj0000000103Xj>P00`000000j>SX0>SXj0020>SXj00?07AdM0000000
j>SX0000001dM7@0j>SX0000001dM7@0000007AdM03Xj>P000000>SXj0000000M7Ad00<0j>SX00<0
M7Ad000000000000403Xj>P04P33`l<0hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00001@000000o?cl08Z:RP3Ph>00`L71
0100k>c/0P00000207IfMP0?0>c/k0000000MWIf0>c/k0000000MWIf0>c/k0000000k>c/0000001f
MWH0k>c/0000003/k>`0000000@0k>c/00@0MWIf0000001fMWH0MWIf0P3/k>`01P000000>c/k0>c/
k0000000k>c/00000080k>c/00<000000>c/k03/k>`03`3/k>`00`33`l<0Xj>S0>c/k00?0>c/k00;
0000001fMWH0MWIf03/k>`000000k>c/07IfMP000000MWIf0000001fMWH00P000000303/k>`00000
07IfMP3/k>`000000>c/k0000000>c/k0>c/k0000000k>c/00000080k>c/00<000000>c/k03/k>`0
0P3/k>`207IfMP0703/k>`1fMWH0k>c/0000001fMWH0k>c/0;6a/@0207IfMP0<0000003/k>`00000
0>c/k01fMWH000000>c/k01fMWH0000007IfMP000000MWIf0P0000000`3/k>`0000007IfMP0?0>c/
k0030<O7a`3Shn<0Xj>S0140k>c/00T0000007IfMP3/k>`0k>c/07IfMP000000k>c/0000001fMWH0
0P0000001@3/k>`0000007IfMP000000MWIf01L0k>c/00<0`l?30:>SX`3/k>`03`3/k>`04@000000
k>c/0>c/k01fMWH000000>c/k0000000>c/k07IfMP000000MWIf0000001fMWH0000007IfMP000000
MWIf0080000000D0k>c/0000001fMWH0MWIf03/k>`0207IfMQ00k>c/00<0`l?30:>SX`3/k>`03`3/
k>`50000000307IfMP3/k>`0k>c/0080MWIf00<000000>c/k00k>c/00P1fMWH01@2a/K40000007If
MP3/k>`000000080MWIf00<000000>c/k0000000103/k>`05P1fMWH000000>c/k0000000>c/k0>c/
k0000000MWIf0000001fMWH0k>c/0000003/k>`0000003/k>`1fMWH000000>c/k00k>c/0MWIf0000
002a/K4?0>c/k00B0<?3``3Shn<0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:0>3Ph031`L40403clo<2
000000<0lo?c0P0000020??cl`80000000<0NGUi0??cl`0000000P000000101iNGT0lo?c00000000
00030??cl`0307UiN@000000lo?c00<0lo?c0`0000000`1iNGT0lo?c07UiN@0300000140lo?c00<0
`l?30:>SX`3clo<03`3clo<00`000000NGUi0??cl`040??cl`<0000000@0lo?c000000000000NGUi
0P0000000`3clo<000000??cl`030000000307UiN@3clo<0NGUi00<000000`3clo<00`1iNGT00000
0??cl`030??cl`80000000H0lo?c07UiN@000000000007UiN@3clo<20000000407UiN@0000000000
0??cl`<000000P3clo<30000000307UiN@3clo<0lo?c00d0lo?c00<0bl_;0>?Sh`2SXj<04@3clo<0
0`000000NGUi0??cl`020??cl`<0000000H0lo?c000000000000lo?c0000001iNGT2000001L0lo?c
00<0`l?30:>SX`3clo<03`3clo<01@000000lo?c0??cl`1iNGT000000080lo?c0`0000000`3clo<0
00000000000300000080lo?c0`0000020??cl`8000004@3clo<00`33`l<0Xj>S0??cl`0?0??cl`80
000000@0lo?c07UiN@000000NGUi0P3clo<20000000307UiN@3clo<0NGUi0080000000D0lo?c0000
00000000lo?c07UiN@020000000407UiN@3clo<00000000000<0lo?c00<0NGUi0000003clo<01000
00020??cl`<000000P3clo<20000000607UiN@3clo<000000??cl`000000NGUi3`3clo<04P33`l<0
hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0
oOgm0?knoP40oooo00001@000000o?cl08Z:RP3Ph>00`L710100mOGe0P00000@0?Gem@030000003e
mOD0mOGe0080mOGe00<0NWYj0000003emOD02`3emOD00`000000mOGe0?Gem@0?0?Gem@030<?3``2S
Xj<0mOGe00l0mOGe00<0000007YjNP3emOD0203emOD2000000030?Gem@000000NWYj00l0mOGe00<0
NWYj0000003emOD00`3emOD00`000000NWYj0?Gem@070?Gem@0307YjNP000000mOGe00P0mOGe00<0
000007YjNP3emOD03P3emOD00`3<c<`0hn?S0:>SX`0A0?Gem@030000001jNWX0mOGe02@0mOGe00<0
`l?30:>SX`3emOD03`3emOD01@000000mOGe0?Gem@1jNWX0000002D0mOGe00<0`l?30:>SX`3emOD0
3`3emOD2000000040?Gem@1jNWX0000007YjNPX0mOGe00<0000007YjNP3emOD02P3emOD00`1jNWX0
00000?Gem@080?Gem@030000003emOD0mOGe01L0mOGe0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0
YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000D00000
0?clo02:RXX0h>3P0<71`@0@0?clo08000000P1mOGd00`2l_;`0o?cl0?clo00;0?clo00307moO`3l
o?`0o?cl0080o?cl00D0OWin0000001mOGd0OGem0;bl_0090?clo0030000003lo?`0o?cl00l0o?cl
00<0`l?30:>SX`3lo?`03`3lo?`01@1oOgl0?Shn07emO@0n?Sh0Ogmo00H0o?cl0P1oOgl01@3lo?`0
Ogmo0;jn_P3lo?`0OGem00d0o?cl00P0_[jn0000001mOGd0?Shn07moO`3lo?`0Ogmo0;jn_PP0o?cl
00<0_[jn07moO`3lo?`0203lo?`00`000000OWin0?clo00>0?clo0030<o?c`3Shn<0Xj>S0100o?cl
00@0OGem0000000n?Sh0OGem903lo?`00`33`l<0Xj>S0?clo00?0?clo0040000001mOGd0OGem03lo
?bH0o?cl00<0`l?30:>SX`3lo?`03`3lo?`01P000000OWin0?clo03lo?`0000007inOPX0o?cl00<0
Ogmo0;jn_P3lo?`02P3lo?`00`1nOWh000000?clo0080?clo0030000003lo?`0o?cl01L0o?cl0180
`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0
o?cl0?gmo@3no_h10?ooo`0000D000000?clo02:RXX0h>3P0<71`@170?ooo`030<?3``2SXj<0oooo
05X0oooo00<0dM7A0>?Sh`2SXj<0>03oool00`33`l<0Xj>S0?ooo`0i0?ooo`030<?3``2SXj<0oooo
0540oooo0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`
0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000L000000?clo02:RXX0h>3P0<K6aP3bl_80o_kn04<0
oooo00H0o_kn0>k^kP37alL0[:b/0?;blP3no_iH0?ooo`050?knoP3=cLd0hn?S0:b/[03hn?P0=@3o
ool01@3no_h0k^k^0<O7a`2/[:`0n?Sh03L0oooo00D0o_kn0>k^kP37alL0[:b/0?Shn01?0?ooo`0D
0?knoP3^k^h0alO70>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`
0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000L000000?clo02:RXX0h>3P0=[JfP32`/80lo?c04<0
oooo00H0moOg0<;2`P3Kfm/0e=CD0<;2`P3clo=H0?ooo`050?3`l03;bl/0hn?S0=CDe03EeMD0=@3o
ool01@3`l?00`/;20=_Kf`3De=@0eMGE03L0oooo00D0moOg0<;2`P3Kfm/0e=CD0=GEe@1?0?ooo`0D
0?Ogm`32`/80fm_K0>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`
0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:RXX00P3Ph>000`3Jf]X0a/K60<71
`@120<71`@060<C4a03GemL0hN7Q0>7Qh@3Jf]X0a/K6F031`L401@38b<P0g=cL0>7Qh@3QhN40d];B
03D0`L7100D0b<S80=OGe`3QhN40hN7Q0=;BdP0g0<71`@050<C4a03GemL0hN7Q0>7Qh@3Bd]80C`31
`L405034a<@0emOG0>7Qh@3QhN40RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0h>3PM03Ph>00402:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h10?ooo`0000<000000?clo02:RXX0o`3Ph>1d0>3Ph00@08Z:RP3lo?`0000009bLW02VYZH0][Jf
0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:
RP3o0>3Ph7@0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03f
m_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0h>3PM03Ph>00402:RXX0o?cl
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?oo
o`0000<000000?clo02:RXX0o`3Ph>1d0>3Ph00@08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o0>3P
h7@0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0=L0h>3P00<0i>CT0>SXj03YjNT01@3YjNT5
0>_[j`80jNWY0P3Zj^X00`3Vi^H0h>3P0>3Ph0030>3Ph0030>?Sh`3WinL0jNWY00D0jNWY00<0j^[Z
0>_[j`3[jn/00P3[jn/30>WYj@030>[ZjP3Vi^H0h>3P0080h>3P00<0i>CT0>SXj03YjNT01@3YjNT5
0>_[j`80jNWY0P3Zj^X00`3Vi^H0h>3P0>3Ph0170>3Ph0040=CDe037alL0alO70<K6aP@0alO700<0
b<S80<W9b@38b<P02037alL04P38b<P0bLW908Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP070>3Ph003
0<o?c`33`l<0`l?300`0`l?300<0alO70>3Ph03Ph>001@3Ph>000`3?cll0`l?30<?3``0<0<?3``03
0<O7a`3Ph>00h>3P0:H0h>3P00<0gMgM0;Zj^P1@D5003@0d=3@00`15ADD0QXJ60?7al@040>3Ph003
0=gMg@3Rh^80LG5a00d0=3@d00<0?3`l08J6QP3alO400P3Ph>000`3MgMd0^[Zj051@D00=03@d=003
04E5A@26QXH0lO7a04T0h>3P00`0alO70?knoP3moOd0kno_0>[ZjP3YjNT0j>SX0>WYj@3Ph>00SXj>
0=cLg03Vi^H20>CTi0030>?Sh`3Ti>@0hn?S00<0h^;R0140/K6a08Z:RP3lo?`0000009bLW02VYZH0
][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?cl
o02:RXX01P3Ph>000`3Fe]H0d];B0>_[j`0=0?7al@030=kNgP3<c<`0h>3P00@0h>3P00<0e]KF0=;B
dP3[jn/03@3alO400`3Ng]h0c<c<0>3Ph0040>3Ph9l0Zj^[0`3Ph>000`3IfMT0IVIV07YjNP0>08V9
R@0304];B`2j^[X0h>3P00<0h>3P00<0fMWI0861P@2CTi<03@38b<P00`36a/H0JFUY0;Zj^P020>3P
h0030=WIf@1fMWH0^k^k00d0b<S800<0a<C406UYJ@2j^[X0B@3Ph>003@37alL0o?cl0?Ogm`3`l?00
inOW0>KViP3UiND0i^KV0>SXj02YZJT0]kNg0>KViP3Shn<00P3Rh^805P3Cdm<0fMWI0>3Ph03Ogml0
gMgM0;2`/02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0
o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:RXX01P3Ph>000`3AdM40kNg]0?7al@0=0?7al@03
0>o_k`33`l<0h>3P00@0h>3P00<0dM7A0>g]k@3alO403@3alO400`3_knl0`l?30>3Ph0040>3Ph003
0=GEe@3oool0oooo09/0oooo00L0Zj^[0>3Ph03Ph>00h>3P0=KFeP1cLg<0eMGE00d0_Kfm00<0eMGE
0:^[Z`2CTi<0103Ph>000`3Fe]H0?3`l0=KFeP0>0<_;b`060=gMg@2CTi<0h>3P0>3Ph03Fe]H0Shn?
3`3;bl/00`3MgMd0Ti>C0>3Ph0180>3Ph00<0<K6aP3moOd0moOg0?Gem@3YjNT0gMgM0>SXj03WinL0
iNGU08:2PP2JVYX0e]KF0P3Shn<00`33`l<0^KVi0=kNgP020=oOg`0B0=gMg@2`/;00RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool0
00030000003lo?`0RXZ:00H0h>3P00<0dM7A0?7al@3alO403P3alO400`33`l<0h>3P0>3Ph0030>3P
h0030=7Ad@3alO40lO7a00h0lO7a00<0`l?30>3Ph03Ph>000`3Ph>000`3EeMD0oooo0?ooo`2K0?oo
o`070:^[Z`3Ph>00h>3P0>3Ph03Fe]H0Kfm_0?ooo`0=0=CDe0030?ooo`2][Jd0SHf=00@0h>3P00<0
e]KF03@d=03YjNT03P3De=@01P3oool0SHf=0>3Ph03Ph>00e]KF09VIV@l0e=CD00<0oooo08f=S@3P
h>00B03Ph>004@37alL0lo?c0?_kn`3emOD0n?Sh0;Bd]02c/k<0g]kN0;Rh^02j^[X0VYZJ0:FUY@2c
/k<0XZ:R08f=S@3@d=00iNGU0080hN7Q0180gmoO0;2`/02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:RXX0
1P3Ph>000`3AdM40lO7a0?7al@070?7al@030>3Ph03WinL0lO7a00@0lO7a00<0`l?30>3Ph03Ph>00
0`3Ph>000`3AdM40lO7a0?7al@030?7al@030=gMg@3alO40lO7a00P0lO7a00<0`l?30>3Ph03Ph>00
0`3Ph>000`3EeMD0oooo0000002K0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03Fe]H0LG5a0?ooo`030=[J
fP80K6a/00H0Xj>S0=[JfP3Jf]X0Xj>S06a/K02SXj<20=[JfP030?ooo`2^[Zh0SHf=00@0h>3P00<0
e]KF03@d=03/k>`03P3Jf]X01P3oool0SHf=0>3Ph03Ph>00e]KF09VIV@l0f][J00<0oooo08f=S@3P
h>00B03Ph>000`37alL0emOG0>c/k0020?Shn00P0=SHf01lO7`0Ogmo0;>c/`34a<@0^[Zj0:j^[P2K
Vi/0WIfM09nOW`2e]KD0hN7Q0=oOg`37alL0dM7A0;2`/02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:RXX0
1P3Ph>000`3AdM40lO7a0?7al@060?7al@030=kNgP2n_[h0dm?C00D0lO7a00<0`l?30>3Ph03Ph>00
0`3Ph>000`3AdM40lO7a0?7al@030?7al@030;Rh^03EeMD0lO7a00P0lO7a00<0`l?30>3Ph03Ph>00
0`3Ph>000`3EeMD0oooo0000002K0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03Fe]H0Lg=c0?ooo`020=_K
f`040000003Kfm/0fm_K00000080fm_K00<0KFe]0=_Kf`1]KFd00P3Kfm/00`3oool0/k>c08f=S@04
0>3Ph0030=KFeP0d=3@0kNg]00<0fm_K00<0Zj^[09:BTP2BTY800P2BTY800`2ATI40]KFe0=_Kf`03
0=_Kf`060?ooo`2=SHd0h>3P0>3Ph03Fe]H0VIVI1P3Kfm/00`3;bl/0/k>c0=_Kf`060=_Kf`030?oo
o`2=SHd0h>3P04P0h>3P02D0b<S80=KFeP3Cdm<0l?3`0?_kn`3Kfm/0Lg=c0:FUY@32`/80^[Zj0;Vi
^@2SXj<0W9bL09BDU01jNWX0IfMW07UiN@1gMgL0VIVI0>3Ph02`/;00RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000
o?cl08Z:RP060>3Ph0030=7Ad@3alO40lO7a00@0lO7a00D0i^KV0=KFeP2g]kL0/[:b0=3@d0050?7a
l@030<?3``3Ph>00h>3P00<0h>3P00<0dM7A0?7al@3alO400`3alO40102i^KT0/k>c0;Vi^@3Rh^87
0?7al@030<?3``3Ph>00h>3P00<0h>3P00<0eMGE0?ooo`000000V`3oool01`2[Zj/0h>3P0>3Ph03P
h>00e]KF07EeM@3oool00P3Shn<010000000hn?S0>?Sh`1aLG420>?Sh`03075aL@3Shn<0LG5a0080
hn?S00<0oooo0;Ng]`2=SHd0103Ph>000`3Fe]H0=3@d0?3`l0030>?Sh`040<K6aP26QXH0QXJ608^;
R`80P82000<0eMGE0>?Sh`3Shn<00P3Shn<01P3oool0SHf=0>3Ph03Ph>00e]KF09VIV@H0hn?S00<0
[:b/092@T02f][H01P3Shn<00`3oool0SHf=0>3Ph0180>3Ph00U0<S8b03Hf=P0[Zj^0<C4a03Ogml0
bLW90;fm_@33`l<0`L710;no_`2a/K40XZ:R08^;R`1bLW80OWin0;Ng]`2^[Zh0b<S80>CTi03Rh^80
[jn_08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0
oOgm0?knoP010?ooo`0000<000000?clo02:RXX01P3Ph>000`3AdM40lO7a0?7al@040?7al@050;Vi
^@2b/[80/[:b0:n_[`3<c<`01@3alO400`33`l<0h>3P0>3Ph0030>3Ph0030=7Ad@3alO40lO7a00<0
lO7a00H0^KVi0;:b/P2b/[80/[:b0;^k^`36a/H50?7al@030<?3``3Ph>00h>3P00<0h>3P00<0eMGE
0?ooo`000000V`3oool01`2[Zj/0h>3P0>3Ph03Ph>00e]KF07QhN03oool00P3/k>`010000000k>c/
0000000000020>c/k00307IfMP3/k>`0MWIf0080k>c/00<0oooo0;fm_@2=SHd0103Ph>000`3Fe]H0
=3@d0?Gem@040>c/k0050:^[Z`1dM7@0Ng]k05mOG`2e]KD01@3/k>`01P3oool0SHf=0>3Ph03Ph>00
e]KF09VIV@D0k>c/00D0a/K609BDU02>SXh0T92@0=GEe@050>c/k0030?ooo`2=SHd0h>3P04P0h>3P
02D0b<S80=WIf@2i^KT0S8b<09:BTP2o_kl0alO70<G5a@32`/80_[jn09bLW02?Shl0Ogmo07AdM02c
/k<0a<C409^KV`3@d=00iNGU0>?Sh`2_[jl0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3Q
hN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP060>3Ph003
0=7Ad@3alO40lO7a00@0lO7a00D0[Jf]0:^[Z`2_[jl0[Jf]0<c<c0050?7al@030<?3``3Ph>00h>3P
00<0h>3P00<0dM7A0?7al@3alO400`3alO401P2i^KT0/K6a0:b/[02SXj<0XJ6Q0;^k^`D0lO7a00<0
`l?30>3Ph03Ph>000`3Ph>000`3EeMD0oooo0000002K0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03Fe]H0
NWYj0?ooo`020?;blP030000003bl_80l_;b00<0l_;b00<0NGUi0000001iNGT00P3bl_800`3oool0
`<3008f=S@040>3Ph0030=KFeP0d=3@0n?Sh00@0l_;b00D0b/[:08f=S@1[Jf/0GUiN0=GEe@050?;b
lP060?ooo`2=SHd0h>3P0>3Ph03Fe]H0VIVI1@3bl_801@2^[Zh0SXj>08j>SP2>SXh0b<S800D0l_;b
00<0oooo08f=S@3Ph>00B03Ph>009@38b<P0f=SH0=SHf035aLD0T92@0::RXP32`/80bl_;0;Jf]P2Y
ZJT0Vi^K0861P@1eMGD0DE5A0<71`@2a/K40YJFU0=_Kf`3UiND0hn?S0:n_[`2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00003
0000003lo?`0RXZ:00H0h>3P00<0dM7A0?7al@3alO40103alO401@3Hf=P0/;2`0:6QX@2RXZ80b/[:
00D0lO7a00<0`l?30>3Ph03Ph>000`3Ph>000`3AdM40lO7a0?7al@030?7al@050;Rh^02SXj<0W9bL
0;Vi^@3Jf]X01P3alO400`33`l<0h>3P0>3Ph0030>3Ph0030=GEe@3oool0000009/0oooo00L0Zj^[
0>3Ph03Ph>00h>3P0=KFeP1lO7`0oooo0080moOg00@000000?Ogm`3gmoL0^KVi1`3gmoL00`3oool0
a<C408f=S@040>3Ph0030=KFeP0d=3@0n_[j00D0moOg00<0`/;206IVIP2[Zj/01P3gmoL01P3oool0
SHf=0>3Ph03Ph>00e]KF09VIV@@0moOg00L0d];B08j>SP1mOGd0MgMg07]kN`2<S8`0fm_K00@0moOg
00<0oooo08f=S@3Ph>00B03Ph>009@38b<P0e]KF0=GEe@3Fe]H0`L710:JVYP2c/k<0^[Zj0:b/[02P
X:00T92@07]kN`1PH600Vi^K0<W9b@2WYjL0WIfM0<30`03WinL0iNGU0:n_[`2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00003
0000003lo?`0RXZ:00H0h>3P00<0dM7A0?7al@3alO401P3alO400`3GemL0Vi^K0<C4a0050?7al@03
0<?3``3Ph>00h>3P00<0h>3P00<0dM7A0?7al@3alO400`3alO400`2/[:`0Y:BT0<W9b@080?7al@03
0<?3``3Ph>00h>3P00<0h>3P00<0eMGE0?ooo`000000V`3oool01`2[Zj/0h>3P0>3Ph03Ph>00e]KF
07inOP3oool00`3kno/20000000307emO@3kno/0no_k00D0no_k00<0oooo0<[:bP2=SHd0103Ph>00
0`3Fe]H0=3@d0?gmo@060?_kn`030;Rh^03kno/0no_k00D0no_k00H0oooo08f=S@3Ph>00h>3P0=KF
eP2IVIT40?_kn`0309ZJVP1ZJVX0GUiN0080GEeM00<0HF5Q0:6QX@3kno/00`3kno/00`3oool0SHf=
0>3Ph0180>3Ph00U0<S8b03Fe]H0eMGE0=KFeP3Cdm<0WIfM0;Bd]02RXZ80UiNG08n?S`1kNg/0Kfm_
06QXJ028R8P0/k>c0:j^[P2][Jd0VIVI0;fm_@3Ph>00[jn_08Z:RP3lo?`0000009bLW02VYZH0][Jf
0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:
RXX01P3Ph>000`3AdM40lO7a0?7al@080?7al@030>?Sh`3alO40lO7a00<0lO7a00<0`l?30>3Ph03P
h>000`3Ph>000`3AdM40lO7a0?7al@030?7al@030=KFeP3UiND0lO7a00P0lO7a00<0`l?30>3Ph03P
h>000`3Ph>000`3EeMD0oooo0000002K0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03Fe]H0P8200?ooo`0=
0?gmo@030?ooo`3=cLd0SHf=00@0h>3P00<0e]KF03@d=03no_h03P3moOd01P3oool0SHf=0>3Ph03P
h>00e]KF09VIV@l0oOgm00<0oooo08f=S@3Ph>00B03Ph>009@38b<P0e]KF0=GEe@3Cdm<0YjNW09>C
T`2XZ:P0Ti>C08n?S`1fMWH0KVi^05MGE`1MGEd0Hf=S09RHV02FUYH0XZ:R0;Bd]02m_Kd0emOG0:n_
[`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gm
o@3no_h00@3oool000030000003lo?`0RXZ:00H0h>3P00<0dM7A0?7al@3alO403P3alO400`33`l<0
h>3P0>3Ph0030>3Ph0030=7Ad@3alO40lO7a00h0lO7a00<0`l?30>3Ph03Ph>000`3Ph>000`3EeMD0
oooo0000002K0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03EeMD0PX:20?ooo`0=0?gmo@030?ooo`3AdM40
S8b<00@0h>3P00<0eMGE03@d=03no_h03P3moOd01P3oool0S8b<0>3Ph03Ph>00eMGE09VIV@l0oOgm
00<0oooo08b<S03Ph>00B03Ph>009@38b<P0f=SH0=3@d02UYJD0VYZJ096AT@2MWId0Rh^;096AT@2V
YZH0dM7A0<;2`P1KFe/0=SHf08N7Q`2>SXh0fMWI0?gmo@3inOT0mOGe0:j^[P2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00003
0000003lo?`0RXZ:00H0h>3P00<0e=CD0=cLg03^k^h03@3alO400`3Ti>@0bLW90>3Ph0040>3Ph003
0=CDe03Lg=`0k^k^00d0lO7a00<0i>CT0<W9b@3Ph>00103Ph>000`3EeMD0oooo0?ooo`2K0?ooo`06
0:^[Z`3Ph>00h>3P0>3Ph03Fe]H0Ogmo3`3Kfm/00`2>SXh0YjNW0>3Ph0030>3Ph0030=KFeP1`L700
`l?300d0oooo00<0m_Kf0::RXP2WYjL00P3Ph>000`3Fe]H0RHV90>KViP0=0?ooo`030>k^kP2RXZ80
YjNW04T0h>3P0140b<S80=SHf038b<P0_Kfm0=3@d031`L40OGem0:NWY`3=cLd0[Jf]0;>c/`2WYjL0
UiNG07MgM`1dM7@0VIVI0=gMg@030>GUi@0A0:n_[`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0
dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:00L0
h>3P00<0clo?0<?3``33`l<03033`l<00`37alL0h>3P0>3Ph0050>3Ph0030<o?c`33`l<0`l?300`0
`l?300<0alO70>3Ph03Ph>00103Ph>2O0:^[Z`<0h>3P00<0emOG0;Jf]P1@D5003@0d=3@00`15ADD0
PH610=cLg0040>3Ph0030=OGe`3Lg=`0L71`00d0=3@d00<0?3`l0861P@3Lg=`00P3Ph>000`3GemL0
][Jf051@D00=03@d=00304E5A@21PH40g=cL04T0h>3P02D0b<S80=OGe`3Fe]H0eMGE0=KFeP3AdM40
LG5a0;2`/02TY:@0Zj^[096AT@2d];@0VYZJ09JFUP21PH40OWin0<71`@3Fe]H0e=CD0=3@d02a/K40
RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo00000`000000o?cl08Z:RP3F0>3Ph0030=kNgP3Fe]H0e=CD00H0e=CD1@3?cll20=CD
e0030=GEe@3Fe]H0f][J00@0h>3P00<0g]kN0=OGe`3De=@01P3De=@00`3AdM40clo?0<o?c`020<o?
c`<0e=CD00H0e]KF0=[JfP3Ph>00h>3P0=kNgP3Fe]H70=CDe0D0clo?0P3De=@00`3EeMD0e]KF0=[J
fP190>3Ph00U0<S8b03Fe]H0eMGE0=CDe03Fe]H0`/;20820P037alL0d=3@0=;BdP2SXj<0Z:RX0:f]
[@3Cdm<0bl_;08R8R01nOWh0dM7A0=GEe@3AdM40/K6a08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5
a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX0
o`3Ph>1O0>3Ph0080<S8b03Fe]H0eMGE0=CDe03Fe]H0W9bL0<g=c@3GemL20=GEe@0K0<[:bP2LW9`0
^KVi0=OGe`3Fe]H0e=CD0;:b/P2o_kl0eMGE0=7Ad@2a/K40RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:
RP3o0>3Ph5l0h>3P00L0b<S80=OGe`3EeMD0e=CD0=KFeP36a/H0e]KF0080e=CD00D0eMGE0=KFeP2[
Zj/0]KFe0=KFeP020=GEe@0E0=KFeP3EeMD0eMGE0=7Ad@2a/K40RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl
08Z:RP3o0>3Ph5l0h>3P00<0b<S80=KFeP3>c/h01`3>c/h00`3?cll0a<C40<C4a0060<k>cP0B0<_;
b`2c/k<0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0h>3PG`3Ph>000`39bLT0^k^k0;6a/@0A0;:b
/P0A0;Bd]02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0
o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0h>3PM03Ph>00402:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<0
00000?clo02:RXX0o`3Ph>1d0>3Ph00@08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o0>3Ph7@0h>3P
0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03m
oOd0o_kn0@3oool000030000003gmoL0Y:BT0?l0W9bLM02LW9`0402TY:@0moOg0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?Og
m`3Shn<0o`3Shn=e0>?Sh`0?0?Ogm`000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03f
m_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000jNWY0?gmo@1G0?gmo@030?_kn`3Shn<0k^k^
0>P0oOgm00<0h>3P08f=S@21PH400`21PH400`22PX80PH610861P@050861P@040:RXZ03moOd0oOgm
0:RXZ0@0PH610P22PX860861P@0408f=S@3Ph>00oOgm0::RXP<0F5QH00<0FEUI05QHF01IFET01P1H
F5P04`1mOGd0h^;R0?gmo@3moOd0jNWY0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`
0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003Ph>00moOg05L0moOg00<0Ng]k079bLP3L
g=`0j03gmoL00`2f][H0SHf=0::RXP0;0::RXP040861P@3gmoL0moOg0861P@`0XZ:R00H0SHf=0;Jf
]P3gmoL0F5QH06m_K`1aLG49079bLP03071`L01KFe/0YZJV0080moOg00l0h>3P0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003M
gMd0k^k^0080k^k^00H0c<c<0<[:bP39bLT0bLW90<S8b037alL30<S8b0030<W9b@3<c<`0k^k^0080
k^k^01L0iNGU0>;RhP3^k^h0j>SX0>WYj@3^k^h0gmoO0>c/k03MgMd0fm_K0>CTi03Ph>00k^k^0>7Q
h@3Kfm/0iNGU0=cLg03Ph>00k^k^0>GUi@3Rh^80k^k^0>7Qh@020=_Kf`0J0>c/k03[jn/0h>3P0>k^
kP3QhN40inOW0>k^kP3QhN40inOW0>KViP3Kfm/0gmoO0=cLg03Ti>@0jn_[0=_Kf`3Lg=`0kNg]0=gM
g@3]kNd0k^k^0>GUi@3Kfm/0gmoO0>k^kP3Vi^H20=_Kf`070>3Ph03Ti>@0k^k^0>k^kP3]kNd0g=cL
0>3Ph0020>k^kP030=oOg`3]kNd0jn_[0080fm_K00X0h^;R0>k^kP3Xj>P0fm_K07=cL`1bLW80clo?
0=_Kf`3Lg=`0kNg]0`3^k^h0103]kNd0fm_K0=_Kf`3Kfm/20>g]k@050=cLg03Ph>00k^k^0>k^kP3[
jn/00P3Kfm/01`3WinL0k^k^0>k^kP3Rh^80j^[Z0>k^kP3Ti>@00P3^k^h01@3Shn<0fm_K0=cLg03]
kNd0jn_[0080fm_K00@0h^;R0>k^kP3UiND0h>3P`@3^k^h0102a/K40^KVi0:j^[P31`L450=GEe@D0
[Zj^00H0UiNG0>k^kP3^k^h0UiNG0:j^[P31`L480=GEe@80[Zj^00H0^KVi0;6a/@3^k^h0I6AT071`
L01dM7@907MgM`03075aL@1cLg<0XJ6Q0080k^k^00l0gMgM0000002LW9`0YZJV0;Jf]P35aLD0dm?C
0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000070000003Jf]X0iNGU0>GUi@2g
]kL0Vi^K0<[:bP020>WYj@070=OGe`25QHD0k>c/0>k^kP3[jn/0l_;b0:>SX`020>GUi@0h0;bl_00E
5AD0_[jn0;>c/`1gMgL0emOG071`L01/K6`0C4a<010@4025QHD06a/K0:FUY@2b/[80410@0451@@1n
OWh0DE5A0<o?c`2l_;`05ADE0<71`@2_[jl0410@01DE5@2j^[X0b/[:01`L702SXj<0/k>c04m?C`3I
fMT0/k>c04m?C`3Fe]H0;Rh^010@401@D500LG5a0=WIf@1QHF40410@08j>SP1@D500JFUY0>?Sh`3M
gMd06QXJ010@402XZ:P0e]KF02h^;P0@4100QHF504I6AP3IfMT20>GUi@0?08:2PP1ADE40clo?0>GU
i@1`L700JFUY0>?Sh`1OGel0410@01DE5@33`l<0i>CT0410@02BTY80LW9b0080410@00<05ADE0<30
`03De=@00P3UiND02@3Ng]h09RHV010@400@4100Kfm_0<K6aP1nOWh0DE5A0<o?c`020>GUi@0G05mO
G`0@4100E5AD0<K6aP3Ng]h0ZZZZ08n?S`3GemL0/k>c08f=S@3Shn<0bLW901HF5P0@4100Ogmo0=CD
e01OGel0410@01DE5@33`l<0gMgM03/k>`3?cll0`@3UiND0102][Jd0`<300;Fe]@3Jf]X4079bLP03
02LW9`2e]KD0]KFe00<0]KFe00H0Vi^K0>GUi@3UiND0Vi^K0;Fe]@3Jf]X802LW9`80]KFe00H0`<30
0:f][@3UiND0IVIV07MgM`1iNGT20?ooo`@0NWYj00H0_;bl0?ooo`1jNWX0N7Qh07MgM`2MWId20>GU
i@0?0=[JfP000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo000040000000fMWI0=oOg`3Ogml0g]kN0:2PX03YjNT0iNGU0>[ZjP3MgMd0M7Ad0>g]
k@2][Jd0hN7Q0??cl`2SXj<20=kNgP0h0<K6aP0@4100[jn_06a/K00h>3P0b/[:04i>CP1/K6`0410@
0;6a/@1>CTh0410@0;6a/@1>CTh0OWin0=gMg@1nOWh0DE5A0<g=c@2f][H05ADE0;6a/@0K6a/0UIFE
0<?3``1<C4`0cLg=01`L702QXJ40[Jf]04m?C`3EeMD0[Jf]04m?C`29RHT0Cdm?0=GEe@0_;bl0SXj>
0=GEe@13@d<0a<C40=cLg01>CTh0JFUY0=KFeP0U9BD0Ph>30=_Kf`1fMWH0Shn?04m?C`3EeMD0CTi>
04m?C`3IfMT20=kNgP0D07inOP1ADE40cLg=0=kNgP1/K6`0JFUY0<;2`P0f=SH0fMWI0<?3``2BTY80
emOG02h^;P34a<@0Kfm_079bLP3AdM40U9BD01XJ6P35aLD20=kNgP0R0=OGe`0U9BD0Shn?0=kNgP10
@400Y:BT07MgM`1ADE40cLg=0=kNgP33`l<0ATI60=WIf@1BDU80YJFU0<g=c@1nOWh0>3Ph0<c<c01[
Jf/0JFUY0=GEe@3GemL0YjNW0<?3``16ATH0]kNg03Hf=P3IfMT0`l?309:BTP3GemL0>c/k0<g=cL40
g]kN00@0ZZZZ0<?3``2k^k/0gMgM101bLW800`0W9bL0^k^k0;^k^`030;^k^`0709bLW03Ng]h0g]kN
09bLW02k^k/0gMgM02LW9`050;^k^`0:0=gMg@0W9bL0^k^k0;^k^`33`l<0ZZZZ0=kNgP1VIVH0N7Qh
05EEE@80oooo00@0_[jn07emO@1mOGd0OGem0P3oool0100`<300NGUi07MgM`2JVYX20=kNgP0?0=WI
f@000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040
oooo000040000000e]KF0=GEe@3EeMD0/;2`09VIV@3AdM40]KFe04=3@`2[Zj/0/[:b08n?S`1KFe/0
^k^k0=kNgP2WYjL20=GEe@0H0<g=c@0O7al0TY:B02h^;P0i>CT0YJFU01lO7`29RHT04Q8B0:^[Z`1d
M7@05ADE0;Ng]`0T92@0O7al0=GEe@1iNGT0DE5A0<[:bP2_[jl05ADE08f=S@0B4Q80HV9R0P1ZJVX0
5P2NWYh06a/K09nOW`2VYZH0Cdm?0<o?c`2VYZH0Cdm?09>CT`1?Cdl0bLW904A4A02TY:@0bl_;02h^
;P32`/80dm?C04];B`1YJFT0UiNG018B4P2e]KD20=GEe@0609NGU`1?Cdl0bLW907AdM01LG5`0d];B
0P3EeMD0201iNGT0DE5A0<[:bP3EeMD0J6QX06UYJ@1hN7P0=SHf0P1ZJVX02P1jNWX0`l?302h^;P32
`/80JVYZ079bLP3=cLd0alO701/K6`2OWil20=GEe@0L0<k>cP0T92@0SHf=0=GEe@1RHV80LW9b07If
MP1ADE40b/[:0=GEe@1iNGT0HV9R0=?Cd`1iNGT0DE5A0<[:bP1>CTh0=CDe09BDU00d=3@0CTi>0<S8
b03EeMD0Shn?04M7A`1SHf<0N7Qh03Hf=P80JVYZ00@0NWYj0<?3``0j>SX0b/[:KP3Fe]IC0=GEe@04
0:FUY@39bLT0`/;20=;BdPD0MWIf1@32`/801`2OWil0eMGE0=GEe@2OWil0`/;20>3Ph00W9bL01@32
`/805P3Ph>009bLW0<;2`P32`/80bLW90:FUY@3EeMD0J6QX07emO@20P800EEEE09>CT`3oool0_kno
0:2PX03oool0bLW905EEE@21PH40OWin07alO02EUID20=GEe@0?0=KFeP000000WinO0:ZZZP2h^;P0
alO70=GEe@3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000040000000dm?C0<k>
cP3?cll0c/k>08N7Q`2c/k<0fm_K0:f][@2h^;P0ZJVY09^KV`2ATI40_Kfm0>o_k`2UYJD20<k>cP09
0<c<c019BDT0FUYJ079bLP2NWYh0IfMW01DE5@2i^KT0X:2P0080410@02T05ADE0;Fe]@0S8b<0O7al
0<k>cP1eMGD0DE5A0<O7a`2YZJT05ADE09BDU00E5AD0]kNg0<O7a`0S8b<0PH6101XJ6P2LW9`0X:2P
04m?C`3:b/X0X:2P04m?C`3=cLd0F5QH010@400B4Q80Y:BT0<O7a`0];Bd0`L710<c<c019BDT0JFUY
0:2PX00@4100ZZZZ0<k>cP3;bl/0cLg=05QHF002010@400605aLG03>c/h0c/k>0<k>cP1eMGD0DE5A
0P35aLD0401QHF40JFUY08B4Q01LG5`0b/[:0<O7a`0j>SX0a<C402d];@31`L40IfMW079bLP3:b/X0
`L7101XJ6P2OWil20<k>cP0R0<O7a`0S8b<0Rh^;0<G5a@0m?Cd0`/;207EeM@1ADE40aLG50<g=c@24
Q8@0G5aL0<[:bP1eMGD0DE5A0<C4a00j>SX0WYjN02DU9@2?Shl0Cdm?0;Vi^@2NWYh051@D0<30`03<
c<`0QHF505aLG03:b/X0alO703Xj>P34a<@0>SXj0<G5a@80cLg=K03?clmC0<k>cP030::RXP3>c/h0
bl_;00/0bl_;00L0XZ:R0<k>cP3>c/h0XZ:R0<_;b`3UiND09bLW00D0bl_;00X0iNGU02LW9`3;bl/0
bl_;0<k>cP2RXZ80c/k>06UYJ@22PX80Q8B40P25QHD02P2CTi<0oooo0?ooo`3oool0EUIF08F5Q@24
Q8@0Ph>307emO@2BTY820<k>cP0<0=?Cd`000000ZZZZ0;>c/`2o_kl0c<c<0=WIf@3Ti>@0jn_[0?;b
lP3gmoL0n_[j0P3moOd10?knoP40oooo000040000000clo?0<O7a`37alL0/K6a09FEU@2VYZH0[:b/
0:j^[P2/[:`0YJFU07AdM02h^;P0j^[Z0?Shn02TY:@30<K6aP0f071`L00L71`0Y:BT0;Rh^00c<c<0
=CDe0<?3``2DU9@0JVYZ03/k>`1NGUh0X:2P01TI6@15ADD0Rh^;071`L00T92@0IfMW04];B`1NGUh0
_Kfm06=SH`1HF5P0EEEE0451@@2[Zj/07AdM05mOG`17AdL0G5aL06][J`17AdL0O7al0<K6aP1oOgl0
JVYZ04a<C02o_kl0Ph>301hN7P1WIfL0SXj>02/[:`1YJFT0`l?3079bLP1CDe<0JVYZ0861P@36a/H0
Ogmo06YZJP0k>c/0Shn?0`36a/H0501aLG40<30`06][J`1ZJVX0>3Ph06UYJ@2j^[X0E5AD06YZJP1E
EED0MGEe0<30`00];Bd0_kno06=SH`10@400JVYZ059BDP1LG5`0_[jn0P36a/H09030`<008R8R04e=
C@1NGUh0H61P0<K6aP1aLG40<C4a07EeM@1_Kfl0^KVi05ADE01ZJVX0BTY:0:RXZ01`L700IFEU0<G5
a@1KFe/0`l?307=cL`1SHf<0WIfM04=3@`1ZJVX0Q8B40;Zj^P1DE5@0JVYZ05EEE@1eMGD0`<3002TY
:@1eMGD0Kfm_0<K6aV`0alO7D`36a/H00`2NWYh0e=CD0<o?c`0;0<o?c`070:BTY036a/H0a/K60:BT
Y03?cll0inOW02LW9`050<o?c`090>OWi`0W9bL0clo?0<o?c`3De=@0WYjN0<K6aP1/K6`0QHF500<0
R8R800@0SHf=0?ooo`3oool0Ti>C0`28R8P04@26QXH0PX:208j>SP37alL0a/K60<o?c`000000^;Rh
0;no_`38b<P0e=CD0=kNgP3Xj>P0k^k^0??cl`3hn?P0no_k0080oOgm0@3no_h10?ooo`0001000000
0<g=c@31`L40`L7109^KV`2IVIT0b/[:0<W9b@1cLg<0WYjN07=cL`1GEeL0_[jn0;>c/`2][Jd0ZJVY
0`31`L401`2FUYH0<30`0<C4a031`L40ATI6075aL@32`/801031`L401`2n_[h08R8R07]kN`32`/80
KVi^055AD@34a<@05031`L400`2k^k/0;Bd]0;jn_P020<71`@030<C4a032`/80`L7100/0`L7100L0
KVi^055AD@34a<@0`L7105mOG`1YJFT0`/;200@0`L7100<0^k^k02d];@2n_[h02031`L401P2k^k/0
8R8R08Z:RP2n_[h0?Shn0;jn_]d0`L7100<0Vi^K0=SHf03EeMD02`3EeMD01`2WYjL0`L710<71`@2W
YjL0eMGE0>[ZjP0W9bL01P3oool0200W9bL0eMGE0=GEe@3Hf=P0Vi^K0<71`@1]KFd0R8R80`2:RXX2
0?ooo`0309>CT`3oool0a<C40080RXZ:00<0RHV908B4Q02;Rh/00P31`L40303=cLd000000<O7a`3<
c<`0e=CD0=cLg03UiND0kNg]0?7al@3fm_H0nOWi0?clo080oOgm0@3no_h10?ooo`0000P000000<_;
b`2j^[X0^k^k0:b/[02EUID0ZZZZ0<?3``80XZ:R00H0WinO069RHP2<S8`0dM7A0?[jnP2VYZH30;^k
^`070:b/[01CDe<0_;bl0;^k^`1OGel0U9BD0;bl_0050;^k^`0608^;R`2HV9P0^k^k06][J`1?Cdl0
_kno5@2k^k/01P22PX80^KVi0;^k^`1LG5`0IVIV0;bl_0`0^k^k00L0Jf][04m?C`2o_kl0^k^k05aL
G01VIVH0_;bl00@0^k^k00<0]KFe02`/;02i^KT0202k^k/01P2e]KD08B4Q04e=C@1WIfL0EeMG0;bl
_1/0^k^kK`2l_;aC0;^k^`0309RHV03IfMT0emOG00/0emOG00H0YjNW0;^k^`2k^k/0YjNW0=OGe`3[
jn/7079bLP0C02LW9`3GemL0emOG0=WIf@2HV9P0_;bl06m_K`2<S8`0SHf=0<K6aP3oool0bLW905]K
F`1dM7@0Ti>C0?ooo`36a/H0SXj>08b<S00208V9R@0>0;^k^`2j^[X0bl_;0000003EeMD0fMWI0=kN
gP3UiND0jn_[0?7al@3emOD0n?Sh0?_kn`3lo?`20?gmo@80oooo000040000000bLW90;Rh^02h^;P0
WinO09jNWP3Rh^80l_;b0<k>cP3?cll0d];B0;Ng]`39bLT0`/;20?gmo@2WYjOo0;Rh^3H0^;Rh00<0
UiNG0=OGe`3MgMd02`3MgMd0102VYZH0^;Rh0;Rh^02VYZH20=gMg@P09bLW0P3MgMd01P3GemL0UiNG
0;Rh^01cLg<0TI6A09>CT`80oooo00X0H61P09RHV02HV9P0V9RH09>CT`3oool0U9BD09:BTP2@T900
QhN70P2h^;P02`39bLT000000>7Qh@3Ti>@0j>SX0>g]k@3alO40mOGe0?Ogm`3jn_X0o?cl0080oOgm
0@3no_h10?ooo`40oooo00000`000000bLW90;Rh^0020;Rh^00;08J6QP33`l<0lo?c0:2PX03_knl0
fm_K0<W9b@3kno/0bLW90?knoP2WYjL0o`2h^;Pf0;Rh^00309NGU`3Fe]H0hN7Q00/0hN7Q00@0YZJV
0;Rh^02h^;P0YZJV303QhN401P3Fe]H0UiNG0;Rh^01eMGD0UiNG09nOW`80:2PX00X0[:b/0:^[Z`2/
[:`0Zj^[06UYJ@0X:2P0XZ:R09VIV@2EUID0QhN70P2h^;P02P39bLT000000>SXj03[jn/0k^k^0?7a
l@3emOD0moOg0?Win@3kno/20?gmo@80o_kn0P3oool0000<00000039bLT0]KFe0;>c/`2SXj<0UYJF
0<W9b@37alL0bl_;0=3@d03AdM40][Jf0P3Cdm<00`3De=@0Z:RX0;>c/`3o0;>c/cD0/k>c00<0U9BD
0>3Ph03]kNd02`3]kNd0102[Zj/0/k>c0;>c/`2[Zj/<0>g]k@070>3Ph02DU9@0/k>c07YjNP2SXj<0
/;2`0;jn_P060<71`@0A0;jn_P2j^[X0Y:BT09nOW`24Q8@0/k>c0;Fe]@39bLT000000?3`l03bl_80
lo?c0?KfmP3hn?P0n_[j0?_kn`3lo?`00P3moOd00`3no_h0oooo0?ooo`010?ooo`0000<000000<?3
``2k^k/0o`2^[Zi30:j^[P0409fMW@1fMWH0c/k>0?gmo@P0oooo00<0oOgm0<k>cP21PH400P2^[Zh0
0`21PH40c/k>0?gmo@080?ooo`090?gmo@3>c/h0MWIf09fMW@2^[Zh0MgMg0;Fe]@3EeMD0j>SX00H0
inOW00l0j>SX0=oOg`30`<00EUIF0:>SX`2^[Zh0^k^k0<?3``000000m_Kf0?Ogm`3hn?P0nOWi0?_k
n`3lo?`00`3moOd00`3no_h0oooo0?ooo`020?ooo`0000D000000;Jf]P3AdM40/[:b0:ZZZP3o0:RX
Z480Z:RX00<0Vi^K092@T02@T9002P2@T900102RXZ80Z:RX0:RXZ02RXZ8<092@T00409^KV`2XZ:P0
Z:RX0::RXP`0P82000H0Vi^K0:ZZZP2b/[80dM7A0;Jf]P0000020?[jnP030?_kn`3lo?`0o?cl00<0
oOgm0P3no_h40?ooo`0000D0000008Z:RP3Zj^X0i>CT0<o?c`3o0;jn_W00_[jn00H0clo?0>CTi03Z
j^X0RXZ:0000003lo?`50?gmo@80o_kn1P3oool000060000002d];@0W9bL0>OWi`3emOD0lO7ao`3/
k>a^0>c/k0060?7al@3emOD0inOW09bLW02d];@000001@3moOd20?knoPL0oooo00001@000000oooo
0;Bd]01YJFT0T92@0?l0ZZZZL02ZZZX01@2@T900JFUY0;Bd]03oool0000000@0o_kn2P3oool00?l0
0000NP00000>0?ooo`00\
\>"],
  ImageRangeCache->{{{126.938, 399.625}, {289.5, 114.813}} -> {-181.795, \
165.313, 1.00181, 1.00181}},
  CellTags->"IPM3.1.2.7"],

Cell[TextData[{
  "Notice the eight category tabs near the top of the Help Browser window. \
Choosing the Add-ons & Links tab will give you access to all of the packages \
that come with each implementation of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Similarly, choosing The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Book tab will give you access to the entire ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " book that ships with each professional version of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],

Cell["\<\
Suppose you were looking for information about three-dimensional parametric \
graphics. First click the Built-in Functions tab, then select Graphics and \
Sound on the left, then 3D Plots and finally ParametricPlot3D. The Help \
Browser should look like this:\
\>", "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .64103 
%%ImageSize: 391 251 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
4.33681e-019 0.0025641 -0.0025641 0.0025641 [
[ 0 0 0 0 ]
[ 1 .64103 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 0 m
1 0 L
1 .64103 L
0 .64103 L
closepath
clip
newpath
% Start of colorimage (RGB)
p
0 -0.00256 translate
1.00256 .64359 scale
1173 string
391 251 8 [391 0 0 251 0 0] {
\tcurrentfile
\t1 index
\treadhexstring
\tpop
}  false 3 Mcolorimage
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFE
FEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFDFDFD
FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFCFCFCFBFBFB
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFDFDFDFCFCFCFBFBFBF9F9F9F8F8F8
F7F7F7F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F7F7F7F8F8F8F9F9F9FBFBFBFCFCFCFDFDFD
FDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFBFBFBFAFAFAF8F8F8F6F6F6F3F3F3
F2F2F2F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F2F2F2F3F3F3F6F6F6F8F8F8FAFAFAFBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFEFEFEFDFDFDFDFDFDFBFBFBF9F9F9F7F7F7F5F5F5F1F1F1EEEEEE
EBEBEBE8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8EBEBEBEEEEEEF1F1F1F5F5F5F7F7F7F9F9F9
FBFBFBFDFDFDFDFDFDFEFEFEFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFEFEFEFDFDFDFDFDFDFCFCFCFAFAFAF7F7F7F5F5F5F1F1F1EDEDEDE8E8E8
E4E4E4E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1
E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E1E4E4E4E8E8E8EDEDEDF1F1F1F5F5F5F7F7F7
FAFAFAFCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFF
FFFFFFFFFFFFFDFDFDFDFDFDFCFCFCFBFBFBF8F8F8F5F5F5F1F1F1EBEBEBE5E5E5DEDEDE
D9D9D9D5D5D5D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D5D5D5D9D9D9DEDEDEE5E5E5EBEBEBF1F1F1F5F5F5
F8F8F8FBFBFBFCFCFCFDFDFDFDFDFDFFFFFFFFFFFF
FFFFFFFEFEFEFDFDFDFDFDFDFCFCFCF9F9F9F6F6F6F1F1F1EDEDEDE5E5E5DCDCDCD4D4D4
CCCCCCC7C7C7C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5
C5C5C5C5C5C5C5C5C5C5C5C5C5C5C5C7C7C7CCCCCCD4D4D4DCDCDCE5E5E5EDEDEDF1F1F1
F6F6F6F9F9F9FCFCFCFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFFFEFEFEFDFDFDFDFDFDFBFBFBF8F8F8F3F3F3EEEEEEE8E8E8DEDEDED4D4D4C8C8C8
BFBFBFB8B8B8B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6
B6B6B6B6B6B6B6B6B6B6B6B6B6B6B6B8B8B8BFBFBFC8C8C8D4D4D4DEDEDEE8E8E8EEEEEE
F3F3F3F8F8F8FBFBFBFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFFFEFEFEFDFDFDFDFDFDFAFAFAF7F7F7F2F2F2EBEBEBE4E4E4D9D9D9CCCCCCBFBFBF
B3B3B3AAAAAAA6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6
A6A6A6A6A6A6A6A6A6A6A6A6A6A6A6AAAAAAB3B3B3BFBFBFCCCCCCD9D9D9E4E4E4EBEBEB
F2F2F2F7F7F7FAFAFAFDFDFDFDFDFDFEFEFEFFFFFF
FFFFFFFEFEFEFDFDFDFCFCFCFAFAFAF6F6F6F0F0F0E8E8E8E1E1E1D5D5D5C7C7C7B8B8B8
AAAAAA9F9F9F9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9F9F9FAAAAAAB8B8B8C7C7C7D5D5D5E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
0000000000000000000000000000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC9A9A9A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A
8A8A8A8A8A8A9A9A9AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFEFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0BDBDBD9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F969696E6E6E6D3D3D39F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
9F9F9F9F9F9F9F9F9F9F9F9FBDBDBDE9E9E9E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7EEEEEE9696969F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
969696E6E6E6E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0C2C2C2D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4E1E1E1969696C1C1C1CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCD
CDCDCDCDCDCDCDCDCDCDCDCDEDEDEDCBCBCBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFB969696E1E1E1D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
E1E1E1969696E0E0E0E0E0E0E0E0E0D4D4D4D9D9D9EFEFEFC9C9C9D9D9D9EFEFEFC9C9C9
ECECECE0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACACECECECECECECECECECECECECECECECECECECECECE
CECECECECECEE6E6E6969696CACACACFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFFFFFFFC8C8C8FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFB
FBFBFBFBFBFB969696E6E6E6CECECECECECECECECECECECECECECECECECECECECECECECE
E6E6E6969696E0E0E0E0E0E0E0E0E0C9C9C9B3B3B3E0E0E0B3B3B3B3B3B3E0E0E0B3B3B3
C9C9C9E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0B7B7B74343436C6C6CAFAFAF5C5C5C5E5E5E939393
E0E0E07E7E7E707070707070E0E0E0E0E0E0DEDEDE7F7F7FB3B3B3C3C3C3535353505050
CDCDCDE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACACECECECECECECECECECECECE626262868686CECECE
CECECECECECEE6E6E6969696CACACAD3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3B3B3B3D3D3D3C2C2C2B2B2B2
C2C2C2D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3
D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D3D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D3D3D3D3D3D3FFFFFFC7C7C7F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9969696E6E6E6CECECECECECE8686863E3E3EAAAAAACECECECECECECECECE
E6E6E6969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0EFEFEFE7E7E7E0E0E0
EFEFEFE0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0868686D9D9D97F7F7FD2D2D2D2D2D2484848
D2D2D27C7C7CE0E0E0868686D9D9D9E0E0E0E0E0E0DCDCDC858585ADADAD9A9A9A696969
BFBFBFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACACFCFCFCFCFCFCFCFCF3E3E3E3E3E3ECFCFCFCFCFCF
CFCFCFCFCFCFE7E7E7969696CACACADCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDC
DCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCEDEDED959595FFFFFFCACACA959595
CACACADCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDC
DCDCDCDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCFFFFFFC7C7C7F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9
F9F9F9F9F9F9969696E7E7E7CFCFCFCFCFCFCFCFCF3E3E3E3E3E3ECFCFCFCFCFCFCFCFCF
E7E7E7969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3EFEFEFB3B3B3
D9D9D9E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0868686D9D9D97F7F7FD2D2D2DCDCDC262626
CBCBCB7C7C7CE0E0E07F7F7FD2D2D2E0E0E0E0E0E0E0E0E0868686D2D2D20D0D0D0D0D0D
D9D9D9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0707070000000000000000000000000
707070E0E0E0E0E0E0E0E0E0CACACAD4D4D4D4D4D48989893E3E3E898989D4D4D4D4D4D4
D4D4D4D4D4D4E9E9E9969696CACACADCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDC
DCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCEDEDED959595FFFFFFCACACA959595
CACACADCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDCDC
DCDCDCDCDCDCDCDCDCDCDCDCDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDDD
DDDDDDDDDDDDDCDCDCDCDCDCFFFFFFC7C7C7F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8F8
F8F8F8F8F8F8969696E9E9E9D4D4D4D4D4D4D4D4D4D4D4D43E3E3E3E3E3ED4D4D4D4D4D4
E9E9E9969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3B3B3B3E0E0E0B3B3B3
C9C9C9E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0868686D9D9D97F7F7FD2D2D2D7D7D7373737
CECECE7C7C7CE0E0E0828282D5D5D58989897070703B3B3B9F9F9F737373DEDEDEE0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0707070000000000000707070
E0E0E0E0E0E0E0E0E0E0E0E0CACACADFDFDFDFDFDF6767673E3E3EB6B6B6DFDFDFDFDFDF
DFDFDFDFDFDFEFEFEF969696CACACAE7E7E7E6E6E6E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7F2F2F2959595FFFFFFCACACA959595
CACACAE6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E6E6E6E6E6E6E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6
E6E6E6E6E6E6E7E7E7E6E6E6FFFFFFC5C5C5F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6F6
F6F6F6F6F6F6969696EFEFEFDFDFDFDFDFDFDFDFDFDFDFDF6565653E3E3EB8B8B8DFDFDF
EFEFEF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
EFEFEFE0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E08E8E8E1C1C1CD9D9D9A4A4A49E9E9EB7B7B7545454
E0E0E0838383E0E0E0717171E0E0E0808080E0E0E0696969AFAFAF696969BFBFBFE0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0707070E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACAEDEDEDEDEDEDEDEDED3E3E3E3E3E3EEDEDEDEDEDED
EDEDEDEDEDEDF6F6F6969696CACACAEAEAEAE9E9E9EAEAEAEAEAEAEAEAEAEAEAEAEAEAEA
EAEAEAEAEAEAEAEAEAEAEAEAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEA
EAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAF4F4F4959595FFFFFFCACACA959595
CACACAE9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEA
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9
E9E9E9E9E9E9EAEAEAE9E9E9FFFFFFC5C5C5F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4969696F6F6F6EDEDEDEDEDEDEDEDED3E3E3E3E3E3EEDEDEDEDEDEDEDEDED
F6F6F6969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0B3B3B3
D9D9D9E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0BABABADCDCDCDEDEDE7D7D7D737373D9D9D9
E0E0E0ACACAC707070A7A7A7E0E0E0ACACAC707070767676DCDCDCD2D2D2ABABABE0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACAF8F8F8F8F8F8F8F8F8F8F8F86D6D6D9D9D9DF8F8F8
F8F8F8F8F8F8FBFBFB969696CACACAEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDF1F1F1EDEDEDF6F6F6F1F1F1EDEDED
F1F1F1EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDFFFFFFC3C3C3F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F0F0F0F0F0F0969696FBFBFBF8F8F8F8F8F89D9D9D3E3E3ECACACAF8F8F8F8F8F8F8F8F8
FBFBFB969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C9C9C9
D4D4D4E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0CACACAFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFF969696CACACAF7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7FFFFFFC2C2C2EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEFEF
EFEFEFEFEFEF969696FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF969696E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9EE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0EFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0B6B6B6DADADADADADADADADADADADADADADADADADADADADA
DADADADADADAC6C6C6B8B8B8C5C5C5DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBD1D1D1D0D0D0EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEB8B8B8C7C7C7DADADADADADADADADADADADADADADADADADADADADADADADA
C6C6C6B8B8B8E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E
9E9E9EEFEFEFE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE9E9E9E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7
E7E7E7E7E7E7ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECB8B8B8000000
000000414141ECECECECECECECECECD0D0D0AAAAAAECECECECECEC979797B7B7B7ECECEC
4E4E4EB7B7B7ECECECB2B2B2C2C2C2ECECECECECEC979797B7B7B7ECECEC686868ECECEC
9E9E9EE6E6E6555555B2B2B2C2C2C2878787DDDDDDECECECECECECECECECDEDEDE4A4A4A
E9E9E91C1C1CC8C8C8ECECEC797979D7D7D7ECECECECECECECECECA3A3A3DDDDDDECECEC
ECECEC565656D7D7D7ECECEC7A7A7AECECECD0D0D0AAAAAAECECECECECECECECECECECEC
5A5A5ADDDDDDECECECE6E6E6A0A0A0ECECECA3A3A3DDDDDDECECEC8D8D8DECECEC818181
A5A5A5848484C2C2C2ECECEC787878ECECECC1C1C1A5A5A5ECECEC797979C2C2C2ECECEC
ECECEC9B9B9BECECECECECEC979797A5A5A5ECECECECECECD2D2D2585858ECECECECECEC
ECECECECECECECECECECECECECECECAFAFAF0C0C0CDDDDDDECECECECECECECECECECECEC
ECECECB8B8B8A2A2A2ECECECECECECECECECECECECECECECECECECECECECECECEC727272
747474ECECECECECECECECECECECECECECECECECEC888888ECECECECECECECECECECECEC
ECECECECECECECECECECECEC9D9D9DECECECECECEC7272720C0C0CE4E4E4ECECECECECEC
ECECECC1C1C1A5A5A5ECECEC9D9D9DECECECECECEC727272747474ECECECECECECECECEC
ECECEC9D9D9DECECEC8D8D8DE5E5E5B8B8B8A2A2A2B8B8B8A2A2A2D0D0D0979797B8B8B8
585858DEDEDE1D1D1DE4E4E4ECECEC3F3F3FDDDDDDECECECECECECECECECD2D2D2838383
DEDEDE4A4A4AE5E5E5888888ECECEC888888ECECEC9D9D9DE9E9E94E4E4E747474E9E9E9
717171A5A5A5ECECECB8B8B8838383ECECECECECEC3F3F3FDDDDDDECECECECECECECECEC
ECECECECECECECECECECECEC9E9E9EECECECD0D0D07B7B7B707070979797ECECECAFAFAF
0C0C0CDDDDDDECECECECECECECECECD0D0D07B7B7B7070704A4A4ADADADA4A4A4ACECECE
585858D2D2D2585858ECECEC8D8D8DE5E5E5888888ECECEC767676DDDDDDECECEC727272
0C0C0CE4E4E4ECECECECECECECECEC767676DADADA707070979797D0D0D0979797B8B8B8
A2A2A2B8B8B8585858B2B2B2C2C2C29B9B9B0C0C0CE3E3E3797979696969C8C8C8ECECEC
9D9D9DECECECECECECECECEC5A5A5AE4E4E4ECECECECECECECECEC848484C2C2C2ECECEC
848484C2C2C2878787DDDDDDDEDEDE181818C2C2C29F9F9FC2C2C2626262DDDDDD9B9B9B
0C0C0CD6D6D6828282E6E6E6141414878787A2A2A2E6E6E6979797888888ECECECADADAD
BDBDBDD7D7D7B8B8B8838383ECECECECECEC8B8B8BE9E9E98C8C8CECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECBDBDBD9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
969696E6E6E68A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECB8B8B8000000
000000414141ECECECECECECECECECD0D0D0595959767676BFBFBFD2D2D26D6D6D767676
5C5C5CB7B7B7ECECECB2B2B2C2C2C2ECECECECECECD2D2D26D6D6D7676766F6F6FECECEC
9E9E9EE6E6E6555555B2B2B2C2C2C2747474767676767676797979D7D7D7DEDEDE696969
ECECECECECECECECECECECEC797979D7D7D7ECECECECECECECECECA3A3A3DDDDDDECECEC
DEDEDE696969ECECECECECECECECECECECECD0D0D0595959767676BFBFBFECECECECECEC
5A5A5ADDDDDDECECECB8B8B8B7B7B7ECECECC1C1C1A5A5A5ECECEC9C9C9CECECECECECEC
ECECECECECECECECECECECEC959595E4E4E4C1C1C1A5A5A5ECECECC4C4C4767676ECECEC
ECECEC9B9B9BECECECD6D6D68D8D8DC8C8C8ECECECECECECE4E4E4505050DFDFDFECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECDCDCDC919191ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECEC969696D3D3D3ECECECECECECECECEC
ECECECECECECECECECECECECAFAFAFB7B7B7ECECECECECECECECECECECECECECECECECEC
ECECECA7A7A7C8C8C8ECECEC979797E1E1E1ECECECECECECECECECECECECECECECECECEC
ECECEC979797E1E1E1B0B0B0646464D4D4D45F5F5FD4D4D4767676EAEAEA959595E9E9E9
4A4A4AA5A5A5959595B5B5B5ECECECECECECECECECECECECECECECECECECE4E4E4525252
E5E5E57676766464648F8F8F7474748F8F8FA2A2A28C8C8CE8E8E8EAEAEA808080ECECEC
818181E1E1E1ECECECD2D2D28D8D8DD1D1D1ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECC1C1C1AAAAAAECECECEAEAEA787878DEDEDE737373ECECECECECEC
ECECECECECECECECECECECECECECECEAEAEA787878DEDEDE454545C6C6C67676769C9C9C
4A4A4AA4A4A44A4A4AA2A2A2B0B0B0B3B3B3999999747474B6B6B67E7E7EECECECECECEC
ECECECECECECECECECECECECECECECAEAEAE7E7E7EDEDEDE737373EAEAEA6D6D6DD4D4D4
5F5F5FD4D4D44A4A4AA1A1A1777777E9E9E9838383B5B5B5D0D0D0333333E6E6E6ECECEC
AFAFAFB7B7B7ECECECECECEC5A5A5AE4E4E4ECECECECECECECECEC878787DDDDDDECECEC
C1C1C1838383878787DDDDDDC1C1C16D6D6DECECECB8B8B88383838D8D8DECECEC9F9F9F
C2C2C2E6E6E6828282E9E9E98C8C8C757575CFCFCFECECECECECEC7474747676768C8C8C
DEDEDE737373727272C8C8C8ECECECECECECA5A5A57B7B7B787878ECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECC2C2C2D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
E1E1E19696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECB8B8B8000000
000000414141ECECECECECECECECECD0D0D0AAAAAAECECEC9E9E9EECECECCECECEECECEC
979797B7B7B7ECECECAFAFAF070707BDBDBD838383ECECECCECECEECECEC8D8D8DECECEC
686868D0D0D01818189D9D9DC2C2C28D8D8DECECECDEDEDE4A4A4AECECECDEDEDE696969
ECECECECECECECECECECECEC727272646464B8B8B8B7B7B7ECECECA3A3A3DDDDDDECECEC
ECECEC4E4E4EC8C8C8D0D0D06D6D6DECECECD0D0D0AAAAAAECECEC9E9E9EECECECECECEC
5A5A5ADDDDDDECECECE6E6E6727272ECECEC767676DDDDDDECECEC9C9C9CECECECECECEC
ECECECECECECD0D0D0111111747474ECECECC1C1C1A5A5A5ECECECD0D0D0696969ECECEC
ECECEC9B9B9BECECECECECEC979797A5A5A5ECECECECECECECECEC727272C8C8C8ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECEC8B8B8BECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECAFAFAFA5A5A5ECECECECECECECECEC
ECECECECECECECECECECECEC9D9D9DECECECECECECECECECECECECECECECECECECECECEC
ECECECC1C1C1A5A5A5ECECECC1C1C1A5A5A5ECECECECECECECECECECECECECECECECECEC
ECECECC1C1C1A5A5A5CFCFCF6565655A5A5A6F6F6F515151808080ECECEC9C9C9CD7D7D7
B1B1B16C6C6CC0C0C07D7D7DECECECECECECECECECECECECECECECECECECECECEC979797
BEBEBEC8C8C86565658181813535357D7D7D585858C1C1C1585858ECECEC515151DDDDDD
7C7C7C8E8E8EECECECB8B8B8838383ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECEC9E9E9EECECECB8B8B8515151DDDDDD9C9C9CECECECECECEC
ECECECECECECECECECECECECECECECB8B8B8515151DDDDDDD0D0D0A0A0A0C8C8C85F5F5F
A9A9A9434343A9A9A9585858DEDEDE696969C4C4C47F7F7F8D8D8D1D1D1DE4E4E4ECECEC
ECECECECECECECECECECECECECECEC9F9F9F1D1D1DDDDDDD9C9C9CD7D7D79696965A5A5A
6F6F6F515151A9A9A9A2A2A2979797A2A2A2979797747474979797616161BDBDBDECECEC
9D9D9DECECECECECECECECEC5A5A5AE4E4E4ECECECECECECE9E9E9272727585858ECECEC
707070A2A2A2272727838383DEDEDE4A4A4AE5E5E5707070A5A5A5767676DDDDDD747474
C2C2C2E6E6E6828282E9E9E98C8C8C979797B7B7B7727272A5A5A59D9D9DECECEC7A7A7A
B8B8B8B7B7B7797979C2C2C2ECECECECECEC8D8D8DE9E9E98C8C8CECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECCACACACECECECECECECECECECECECECECECECECECECECECECECECE
E6E6E69696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECD2D2D2767676
767676969696ECECECECECECECECECD0D0D0AAAAAAECECEC949494ECECEC929292767676
929292ECECECE9E9E97E7E7EC8C8C87D7D7DE4E4E4ECECEC929292767676BFBFBFD2D2D2
868686767676B9B9B97D7D7DE4E4E4D6D6D6767676777777D7D7D7ECECEC9D9D9D343434
767676A2A2A2EAEAEA848484797979A3A3A37C7C7CE8E8E8AFAFAF797979E4E4E4ECECEC
ECECECE9E9E97D7D7D797979C1C1C1ECECECD0D0D0AAAAAAECECEC949494ECECECECECEC
5A5A5ADDDDDDECECECECECECAEAEAE767676A8A8A8ECECECD2D2D24E4E4E7676768C8C8C
ECECECCDCDCDA2A2A2ECECEC888888ECECECC1C1C1A5A5A5EAEAEA6D6D6DA2A2A2ECECEC
ECECEC9B9B9BECECECECECEC979797A5A5A5ECECECECECECD6D6D6686868747474ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECC1C1C1464646C5C5C5ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECEC949494252525D8D8D8ECECECECECEC
ECECECECECECECECECECECEC9D9D9DECECECECECECECECECECECECECECECECECECECECEC
ECECECC1C1C1A5A5A5ECECECAEAEAE4F4F4FDFDFDFECECECECECECECECECECECECECECEC
ECECECAEAEAE4F4F4FAAAAAA7C7C7CE1E1E19D9D9DE9E9E99D9D9DECECEC959595E1E1E1
848484BBBBBB848484BDBDBDECECECECECECECECECECECECECECECECECECE5E5E57E7E7E
8F8F8F8484847C7C7CB2B2B2B7B7B7B6B6B6B7B7B7ECECECC1C1C17676769B9B9B767676
A8A8A8C1C1C1BFBFBFB8B8B8838383ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECEC9E9E9EECECECECECECA0A0A0E8E8E8B9B9B9E4E4E4ECECEC
ECECECECECECECECECECECECECECECECECECA0A0A0E8E8E8E9E9E9B5B5B5848484AEAEAE
848484D0D0D0848484B7B7B7CDCDCD7C7C7CB2B2B2929292B4B4B4929292EAEAEAECECEC
ECECECECECECECECECECECECECECECCDCDCD929292E8E8E8B9B9B9E1E1E1808080E1E1E1
9D9D9DE9E9E9848484D1D1D1ECECEC9B9B9BB3B3B37D7D7D797979D7D7D7AAAAAAECECEC
9D9D9DECECECECECECECECEC5A5A5AE4E4E4ECECECECECECECECECB9B9B9CACACA767676
969696ECECECB9B9B9B9B9B9A8A8A8C1C1C1767676969696ECECECDFDFDF767676515151
BFBFBF848484B0B0B0AFAFAFA8A8A8ECECEC9292927C7C7CE4E4E4CDCDCD767676BDBDBD
EAEAEA898989BBBBBBDBDBDBECECECECECECAEAEAE767676BDBDBDECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECCACACACECECECECECECECECEAAAAAACECECECECECECECECECECECE
E6E6E69696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECE6E6E61C1C1C000000000000A2A2A2ECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECEC767676DDDDDDECECEC
ECECECECECECECECECECECECECECECE6E6E61C1C1C000000000000A2A2A2ECECEC393939
0C0C0CDDDDDDECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECEC727272000000646464ECECEC5C5C5C000000000000A5A5A5ECECECECECEC
ECECEC272727838383ECECEC979797838383ECECECECECECECECECECECEC8B8B8BB8B8B8
838383ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECAFAFAFA5A5A5B8B8B8B7B7B7ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECEC727272AEAEAE838383ECECEC
ECECECECECECECECECECECEC8D8D8DECECECECECECECECECECECECECECECECECECECECEC
ECECECC1C1C1838383ECECECECECECBDBDBDD7D7D7ECECECECECECECECECECECECECECEC
ECECECECECECBDBDBDD7D7D7ECECECECECECECECECECECECECECECECECEC767676DDDDDD
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
BFBFBFECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECEC979797838383ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECEC8D8D8DECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECDEDEDE1D1D1DE4E4E4ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
8D8D8DECECECD2D2D20000000C0C0CE4E4E4ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC3C3C3C
C2C2C2ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECCACACACECECECECECECECECE3E3E3E3E3E3ECECECECECECECECECE
E6E6E69696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECDEDEDE7E7E7EDADADAECECECECECECECECECC3C3C3767676
C8C8C8ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECEBEBEB9D9D9D767676DADADAECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECEAEAEA898989C8C8C8ECECEC
ECECECECECECECECECD2D2D2A8A8A8ECECECECECECECECECECECECECECECECECECECECEC
ECECECE5E5E57E7E7EC8C8C8ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECDFDFDF797979D7D7D7ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECAEAEAEC8C8C8ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECD2D2D2
A8A8A8ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECC5C5C5
D7D7D7ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECCACACAD1D1D1ACACAC3E3E3E6363633E3E3E3E3E3EACACACD1D1D1
E8E8E89696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECCACACADBDBDB3E3E3E3E3E3EDBDBDBDBDBDB3E3E3E3E3E3EDBDBDB
EDEDED9696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECCACACAE4E4E4919191E4E4E4E4E4E4E4E4E4E4E4E4919191E4E4E4
F1F1F19696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECCACACAF0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0F0
F7F7F79696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECCACACAFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECC0C0C0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F5F5F59696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECB6B6B6B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
A1A1A1DADADA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECEC9B9B9B0000008F8F8FECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECEAEAEA4E4E4E000000
8F8F8FECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECEC393939585858ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECE9E9E9282828E4E4E4ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE6E6E6282828
E4E4E4ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECB8B8B8C8C8C8ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECD4D4D48484847C7C7CE5E5E5222222727272797979
D7D7D7D6D6D6767676767676929292C2C2C2767676797979E1E1E1AFAFAF797979E4E4E4
ECECECECECECEAEAEA898989797979E1E1E1AFAFAF777777969696D7D7D7929292929292
EAEAEAECECECCDCDCDECECECECECECCDCDCD767676B0B0B0ECECECECECECE6E6E6222222
727272797979E1E1E1C1C1C1767676767676929292909090929292D5D5D5767676767676
929292AAAAAA8C8C8CAFAFAF848484BDBDBD848484E8E8E8AFAFAF797979E4E4E4CDCDCD
767676BBBBBB767676929292D5D5D5767676949494767676767676767676A2A2A2CDCDCD
767676B0B0B0ECECEC929292767676767676DADADAECECECEBEBEB9D9D9D767676C8C8C8
ECECECECECECEAEAEA9D9D9DECECECECECECECECECECECEC9B9B9B767676848484E8E8E8
ECECECECECECE5E5E5BDBDBDECECECD6D6D6767676767676929292909090929292E8E8E8
9B9B9B929292999999767676848484E7E7E79D9D9D767676C8C8C8D6D6D6767676BDBDBD
AFAFAF767676B0B0B0ECECECECECECAFAFAF9F9F9FECECECECECECECECECECECECCDCDCD
ECECECCDCDCDECECECECECEC929292A2A2A29B9B9B929292999999929292E7E7E7898989
797979C3C3C3767676B1B1B1767676929292EAEAEAECECEC929292929292EAEAEAAFAFAF
767676888888D7D7D7929292A2A2A29B9B9B929292D0D0D08484847C7C7CE8E8E8ECECEC
DFDFDF767676767676B3B3B3767676BDBDBDECECEC9B9B9B767676C8C8C8CDCDCD767676
A8A8A8767676A8A8A8767676929292EAEAEAECECEC929292C8C8C8BCBCBCECECECAFAFAF
DADADAAFAFAF969696C1C1C19F9F9FAFAFAFC8C8C8DFDFDF767676C7C7C7ECECECECECEC
AFAFAF767676DADADAC1C1C1767676C8C8C8ECECECECECECECECEC929292C8C8C8BCBCBC
ECECECBBBBBBECECECAFAFAF969696C1C1C1777777C8C8C87A7A7A797979CCCCCC767676
C8C8C8B0B0B0D7D7D7ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECBDBDBDD7D7D7B8B8B8B4B4B4232323D7D7D7E6E6E6
2323239A9A9AA5A5A59F9F9FC2C2C2777777ECECECE6E6E6A7A7A7585858E9E9E9747474
ECECECECECEC979797A2A2A2E6E6E6BEBEBE282828E1E1E11C1C1C0C0C0C7F7F7FDDDDDD
ECECECECECEC555555E5E5E5ECECEC888888ECECECADADADECECECECECECE6E6E6232323
D7D7D7E6E6E6727272727272C7C7C7565656C2C2C2878787DDDDDDAFAFAFA5A5A59F9F9F
C2C2C2E6E6E6828282E9E9E98C8C8CE9E9E9797979B8B8B8585858E9E9E97474749F9F9F
C2C2C2E9E9E9282828DDDDDDEAEAEA565656D5D5D5565656D7D7D7ECECECB1B1B1888888
ECECECADADADB8B8B8747474E9E9E94E4E4E747474ECECECDEDEDE4A4A4AE5E5E5232323
C2C2C2ECECECB8B8B81C1C1CE5E5E5ECECECECECECECECEC8B8B8BE9E9E98C8C8CECECEC
ECECECECECECAFAFAF393939E5E5E5AFAFAFA5A5A59F9F9FC2C2C2878787DDDDDDECECEC
9F9F9FC2C2C28B8B8BE9E9E98C8C8CD0D0D04A4A4AE5E5E5232323D5D5D5565656BDBDBD
585858ECECECADADADECECECECECECB8B8B8B7B7B7ECECECECECECECECECE9E9E9535353
ECECEC686868E5E5E5ECECEC878787DDDDDD9F9F9FC2C2C29F9F9FC2C2C2979797A2A2A2
E6E6E6BEBEBE282828E3E3E3565656C2C2C2ECECECECECEC878787DDDDDDE9E9E9282828
DDDDDD8181810C0C0C7F7F7FDDDDDD9F9F9FC2C2C2BDBDBDD7D7D7B8B8B8B7B7B7ECECEC
ECECEC979797838383E9E9E9282828E4E4E4D0D0D06D6D6DE9E9E9565656D4D4D4282828
E3E3E3565656D5D5D54E4E4EA5A5A5ECECECECECEC9D9D9DECECEC8D8D8DE5E5E5B8B8B8
A2A2A2B8B8B8A2A2A2D0D0D0979797B8B8B8585858DEDEDE1D1D1DE4E4E4ECECECEAEAEA
565656C2C2C2DEDEDE181818D5D5D5565656D7D7D7ECECECECECEC9D9D9DECECEC8D8D8D
E5E5E5888888ECECECB8B8B8A2A2A2797979D5D5D54E4E4E747474E9E9E9717171A5A5A5
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECDEDEDE737373727272D7D7D7282828E4E4E4ECECEC
979797AAAAAA797979545454C2C2C2949494ECECECECECECC1C1C1525252767676A2A2A2
ECECECECECEC757575CFCFCFECECECE9E9E9282828E3E3E3565656C2C2C2878787DDDDDD
ECECECA1A1A1929292A3A3A3E8E8E87474747676768C8C8CECECECECECECE6E6E6282828
E4E4E4ECECEC8D8D8DD2D2D27A7A7A2E2E2EC2C2C2878787DDDDDDDFDFDF797979545454
C2C2C2E6E6E6828282E9E9E98C8C8CE9E9E9797979979797525252767676A2A2A29F9F9F
C2C2C2E9E9E9282828DDDDDDEAEAEA565656D7D7D7E9E9E9848484E4E4E4ECECEC747474
7676768C8C8C979797B7B7B7E9E9E9565656D7D7D7ECECECDEDEDE4A4A4AE8E8E8979797
838383D5D5D5434343C2C2C29B9B9BECECECECECECECECECA5A5A57B7B7B787878ECECEC
ECECECECECEC787878D6D6D6A3A3A3DDDDDD797979545454C2C2C2878787DDDDDDECECEC
9F9F9FC2C2C2A5A5A57B7B7B787878D0D0D04A4A4AE8E8E8979797B6B6B6565656ACACAC
5252527676768C8C8CECECECECECECDCDCDC999999ECECECECECECECECEC9595959F9F9F
969696959595A1A1A1ECECEC878787DDDDDD9F9F9FC2C2C29F9F9FC2C2C2757575CFCFCF
ECECECE9E9E9282828E3E3E3565656C2C2C2ECECECECECEC878787DDDDDDE9E9E9282828
DDDDDD9F9F9FC2C2C2878787DDDDDD9F9F9FC2C2C2DEDEDE737373727272DADADAECECEC
ECECEC979797838383E9E9E9282828E4E4E4979797ACACACECECEC848484B4B4B4282828
E3E3E3565656D5D5D54E4E4EA5A5A5ECECECECECEC979797E1E1E1B0B0B0646464D4D4D4
5F5F5FD4D4D4767676EAEAEA959595E9E9E94A4A4AA5A5A5959595B5B5B5ECECECEAEAEA
565656C2C2C2C1C1C16D6D6DECECEC848484B7B7B7ECECECECECEC979797E1E1E1B0B0B0
6464648F8F8F747474D4D4D4767676A1A1A1AFAFAFEAEAEA808080ECECEC818181E1E1E1
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECB8B8B8B7B7B7797979868686000000A2A2A2D0D0D0
1111119E9E9E8383839F9F9FC2C2C28D8D8DECECEC727272AEAEAE838383AFAFAF838383
ECECECECECEC979797B7B7B7727272C5C5C5282828E3E3E3565656BFBFBF272727838383
E9E9E9282828E4E4E4BDBDBDC2C2C29D9D9DECECEC7A7A7AECECECEAEAEA4E4E4E000000
A2A2A2D0D0D05B5B5B979797B6B6B6565656BFBFBF272727838383C1C1C18383839F9F9F
BFBFBF4E4E4E414141E6E6E63A3A3AE9E9E98C8C8CB8B8B8838383AFAFAF838383919191
C2C2C29B9B9B000000838383AFAFAF070707C1C1C1DDDDDDDEDEDE4A4A4AE5E5E59D9D9D
ECECEC7A7A7AD0D0D07D7D7DD0D0D0181818D7D7D7ECECECDEDEDE111111C7C7C74E4E4E
A5A5A5707070A5A5A5ECECEC9C9C9CD7D7D7ECECECECECEC8D8D8DE9E9E98C8C8CECECEC
ECECECE6E6E6828282ECECECBDBDBDACACAC8383839F9F9FBFBFBF272727838383ECECEC
3C3C3CC2C2C28D8D8DE9E9E98C8C8CD0D0D0111111C7C7C74E4E4EC7C7C7565656BDBDBD
838383ECECEC7A7A7AECECECECECECECECEC9E9E9EECECECECECECDEDEDE4A4A4AE7E7E7
565656D7D7D7BDBDBDC2C2C2818181585858848484C2C2C23C3C3CC2C2C2979797B7B7B7
727272C5C5C51C1C1C9F9F9F232323C2C2C2ECECECE9E9E9272727838383E9E9E9282828
DDDDDD9F9F9FBFBFBF272727585858848484C2C2C2B8B8B8B7B7B7797979D7D7D7ECECEC
ECECEC9797978383839B9B9B000000B7B7B7D0D0D0979797D0D0D0111111787878000000
B4B4B41C1C1CA0A0A0565656C2C2C2ECECECECECECC1C1C1A5A5A5CFCFCF6565655A5A5A
6F6F6F515151808080ECECEC9C9C9CD7D7D7B1B1B16C6C6CC0C0C07D7D7DECECECDCDCDC
565656C2C2C2DEDEDE4A4A4ADADADA111111C8C8C8ECECECECECECC1C1C1A5A5A5CFCFCF
656565818181353535515151808080ECECEC777777ECECEC515151DDDDDD7C7C7C8E8E8E
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECEAEAEA898989BBBBBBE2E2E28A8A8AA0A0A0767676
B7B7B7E5E5E57E7E7E848484E8E8E8CDCDCD7676767C7C7CE8E8E8AFAFAF848484E8E8E8
ECECECECECECECECEC9292927C7C7CB2B2B27C7C7CC9C9C9797979D7D7D7B9B9B9B9B9B9
767676767676C6C6C69D9D9D7C7C7CC9C9C9767676BDBDBDECECECECECECE9E9E98A8A8A
A0A0A0767676D1D1D1DEDEDE7A7A7A929292E8E8E8B9B9B9B9B9B9A1A1A17E7E7E848484
E8E8E8DEDEDEACACAC767676B0B0B0767676BDBDBDECECECAFAFAF848484E8E8E8878787
616161D8D8D88A8A8AB9B9B9A6A6A6878787D4D4D4767676767676767676929292CBCBCB
767676BDBDBDECECECA4A4A47474742E2E2ED7D7D7ECECECDEDEDE4A4A4A8E8E8EA2A2A2
E9E9E97D7D7D7C7C7CE1E1E17E7E7EB0B0B0ECECECECECECAEAEAE767676BDBDBDECECEC
EBEBEB9D9D9D8C8C8CEAEAEA9D9D9DA9A9A97E7E7E848484E8E8E8B9B9B9B9B9B9797979
ADADADD7D7D7AEAEAE767676BDBDBDD0D0D04A4A4A8E8E8EA2A2A2EAEAEA565656D7D7D7
AFAFAF767676BDBDBDECECECECECECDEDEDE676767D3D3D3ECECECB6B6B6767676B2B2B2
767676BCBCBC9D9D9D929292858585C3C3C3848484E8E8E8B8B8B8D7D7D7ECECEC929292
7C7C7CE5E5E52828289C9C9C929292E8E8E8ECECECECECECB9B9B9B9B9B97676767C7C7C
E4E4E4949494D7D7D7B9B9B9CACACA848484E8E8E8EAEAEA898989BBBBBBE3E3E3ECECEC
D6D6D64B4B4B414141BBBBBB8A8A8A999999D7D7D79B9B9B767676B7B7B7E5E5E59B9B9B
999999929292A2A2A2929292E8E8E8ECECECECECECAEAEAE4F4F4FAAAAAA7C7C7CE1E1E1
9D9D9DE9E9E99D9D9DECECEC959595E1E1E1848484BBBBBB848484BDBDBDECECECDFDFDF
3C3C3C616161DADADAC1C1C1767676B7B7B7ECECECECECECECECECAEAEAE4F4F4FAAAAAA
7C7C7CB2B2B2B7B7B7E9E9E99D9D9DECECECD6D6D67676769B9B9B767676A8A8A8C1C1C1
929292EAEAEAECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECC4C4C4
C2C2C2ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECEC727272070707D7D7D7ECECEC818181393939E8E8E8ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECE9E9E91C1C1C393939E8E8E8ECECECAFAFAF070707D7D7D7
ECECECECECECECECECECECECECECECECECECD4D4D4C2C2C2ECECECECECECECECECECECEC
ECECECECECECECECECE9E9E92D2D2DDDDDDDECECECECECECECECECECECECECECECECECEC
ECECEC9B9B9B0C0C0CE4E4E4ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECB8B8B8838383ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECBDBDBDD7D7D7ECECECECECEC
ECECECECECECECECECECECEC767676DDDDDDECECECECECECECECECECECECECECECECECEC
9C9C9CC2C2C2ECECECECECECECECECECECECECECECECECECECECECECECECBDBDBDD7D7D7
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECE9E9E9232323D7D7D7ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECE9E9E9565656D7D7D7ECECECDEDEDE4A4A4AE8E8E8ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
848484C2C2C2ECECECECECECECECECD0D0D04A4A4AE8E8E8ECECECE9E9E9232323D7D7D7
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECEC767676DDDDDDECECECECECEC848484C2C2C2ECECECECECEC
ECECECDEDEDE1D1D1DE4E4E4ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECEC727272030303C2C2C2ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECB1B1B1ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECEC767676000000ECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECE5E5E57E7E7EC7C7C7ECECECECECECECECEC
ECECEC767676000000ECECECECECECECECECECECECECECECECECECECECECECECECDEDEDE
909090ECECECECECECECECECECECECECECECECECEC3B3B3BECECECECECECECECECECECEC
ECECECECECECECECECECECECE5E5E57E7E7EA2A2A2ECECECECECECECECECECECEC3B3B3B
767676000000ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3767676BDBDBDECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECEC272727838383ECECECE6E6E61C1C1CBABABA6969698D8D8DECECECECECECECECEC
ECECEC767676ECECECECECECECECEC888888ECECECECECECECECECECECECECECECD0D0D0
808080838383ECECECECECECECECECECECECECECEC767676000000ECECECB8B8B8585858
ECECECECECECECECECECECECDEDEDE696969979797B7B7B7ECECECECECECECECECECECEC
000000ECECECECECECB8B8B8747474ECECECD2D2D2585858ECECECECECECECECECECECEC
ECECECE9E9E91C1C1CA5A5A5ECECECECECECD2D2D2585858ECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECB8B8B8A2A2A2ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECD2D2D20C0C0CDDDDDDECECECB8B8B8060606DDDDDDECECECECECECECECEC
E9E9E9282828E4E4E4ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECEC9B9B9BECECECECECEC979797838383ECECECECECEC878787DDDDDDECECECECECEC
ECECEC767676000000ECECECECECEC747474C2C2C2ECECECECECECECECECECECECECECEC
E9E9E9282828E4E4E4ECECECECECECECECECECECEC767676000000ECECECD0D0D0181818
D7D7D7ECECECECECECECECECECECECECECECE6E6E6828282ECECECECECECECECEC767676
ECECEC000000ECECECECECEC8D8D8DECECECDEDEDE181818C2C2C2ECECECECECECECECEC
ECECECC1C1C1838383ECECECECECECECECECDEDEDE181818D7D7D7ECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECD0D0D06D6D6DECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECEC979797838383ECECECECECECA3A3A3DDDDDDECECECECECECECECEC
E9E9E9282828E4E4E4ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECD2D2D2767676
767676969696ECECECECECECE9E9E9767676555555A2A2A2ECECECC1C1C16F6F6F767676
474747848484DEDEDE5D5D5D616161969696ECECECC1C1C16F6F6F767676444444ACACAC
505050D1D1D13838388B8B8B646464A2A2A26E6E6E7676767C7C7CE4E4E4E5E5E55E5E5E
7474744F4F4FD3D3D37E7E7E3C3C3C6B6B6BB7B7B7EAEAEA8484845151516E6E6E797979
E1E1E19393936F6F6F7676768E8E8EE9E9E9767676555555A2A2A2ECECECE9E9E97D7D7D
2D2D2D6E6E6EA8A8A8E9E9E9757575767676727272E4E4E4ECECEC9E9E9E767676616161
C8C8C8B1B1B16969697676766E6E6EECECEC8E8E8E525252767676686868D7D7D7ECECEC
ECECEC9B9B9BECECECECECEC979797A5A5A5ECECECECECECC0C0C07E7E7EECECECECECEC
ECECEC767676ECECECB1B1B1ECECEC989898323232E4E4E4ECECECECECECECECECECECEC
ECECEC959595B5B5B5ECECECECECECECECEC767676B1B1B1767676ECECECE9E9E9474747
8B8B8BECECECECECECECECECECECECECECECECECEC898989ECECECECECECECECECB1B1B1
767676767676B1B1B1ECECEC9D9D9DECECECEAEAEA474747323232E8E8E8ECECECECECEC
ECECECC1C1C1A5A5A5ECECEC6B6B6BC8C8C8EAEAEA4747478B8B8BECECECECECECECECEC
ECECEC6B6B6BC8C8C88D8D8DE8E8E8959595B5B5B5959595717171B3B3B3747474959595
7E7E7ED8D8D80E0E0EC3C3C3E9E9E93C3C3CE4E4E4ECECECECECECECECECC0C0C0414141
BEBEBE666666E8E8E8898989ECECEC606060B7B7B77474747878787979793E3E3E787878
8E8E8E525252DADADAB8B8B8838383ECECECECECECA3A3A3DDDDDDECECECECECECECECEC
E9E9E91F1F1F6B6B6B767676A4A4A4ACACAC141414B9B9B9DEDEDE5C5C5C747474AAAAAA
D2D2D23F3F3F7171712727279C9C9C4D4D4D7272722E2E2E676767858585767676919191
D2D2D26D6D6D797979A8A8A89D9D9D767676888888ECECECECECEC8484846363632B2B2B
7D7D7DEAEAEAECECECA7A7A7616161BABABA1414149A9A9A7575757D7D7D5D5D5D8B8B8B
DDDDDD656565767676909090D2D2D26D6D6D797979CACACAECECECECECECDFDFDF3F3F3F
7171712727278888882B2B2B7D7D7D5D5D5D9A9A9A6C6C6C7D7D7D7171716F6F6FCECECE
6D6D6D797979AAAAAA141414ADADAD2B2B2B7D7D7D9B9B9B767676888888C0C0C0545454
DADADA5757576A6A6A7D7D7DC1C1C1141414ADADAD2B2B2B7D7D7D696969747474545454
BCBCBC545454E8E8E8ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECB8B8B8000000
000000414141ECECECECECECECECECD0D0D0AAAAAAECECECECECECB8B8B8A2A2A2ECECEC
B8B8B8B7B7B7ECECECB2B2B2C2C2C2ECECECECECECB8B8B8A2A2A2ECECEC9D9D9DECECEC
9E9E9EE6E6E6555555B2B2B2C2C2C29D9D9DECECECECECECECECECECECECDEDEDE696969
ECECECECECECECECECECECEC797979D7D7D7ECECECECECECECECECA3A3A3DDDDDDECECEC
DEDEDE696969ECECECECECECECECECECECECD0D0D0AAAAAAECECECECECECECECECECECEC
5A5A5ADDDDDDECECECB8B8B8B7B7B7ECECECC1C1C1A5A5A5ECECEC9C9C9CECECECECECEC
ECECECECECECECECECECECECA3A3A3DDDDDDC1C1C1A5A5A5ECECECB8B8B8838383ECECEC
ECECEC9B9B9BECECECECECEC979797A5A5A5ECECECECECECDEDEDE4A4A4AE8E8E8ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECD0D0D0979797ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECEC8D8D8DE5E5E5ECECECECECECECECEC
ECECECECECECECECECECECEC9D9D9DECECECECECECECECECECECECECECECECECECECECEC
ECECECC1C1C1A5A5A5ECECEC8D8D8DE5E5E5ECECECECECECECECECECECECECECECECECEC
ECECEC8D8D8DE5E5E59F9F9FC2C2C2E6E6E6626262E6E6E6727272E9E9E98C8C8CE6E6E6
232323C7C7C7727272C8C8C8ECECECECECECECECECECECECECECECECECECDEDEDE4A4A4A
E7E7E7565656C2C2C2848484A2A2A2848484C2C2C28B8B8BECECECE9E9E9797979ECECEC
767676DDDDDDECECECB8B8B8838383ECECECECECECA3A3A3DDDDDDECECECECECECECECEC
E9E9E9282828E4E4E4ECECEC8D8D8DE6E6E6282828E4E4E4979797A2A2A2ECECEC8D8D8D
979797838383EAEAEA565656D4D4D4282828E3E3E3565656C2C2C28B8B8BECECECECECEC
979797C8C8C8ECECECECECECECECEC979797525252ECECECECECECB8B8B8737373565656
C2C2C2ECECECECECEC9F9F9FC2C2C2E9E9E9282828DDDDDD9F9F9FC2C2C2878787DDDDDD
C1C1C1A5A5A5ECECECECECEC979797C8C8C8ECECECC1C1C10000000000008D8D8D838383
EAEAEA565656D5D5D5565656C2C2C2878787DDDDDD9F9F9FC2C2C29B9B9BA5A5A5979797
C8C8C8ECECECE9E9E9282828E3E3E3565656C2C2C2ECECEC979797525252E9E9E98C8C8C
979797585858ECECEC979797B4B4B4282828E3E3E3565656C2C2C2626262DADADA8C8C8C
E9E9E98C8C8CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECB8B8B8000000
000000414141ECECECECECECECECECD0D0D0595959767676989898ECECEC838383767676
4B4B4BB7B7B7ECECECB0B0B0646464D4D4D4B7B7B7ECECEC838383767676676767ECECEC
838383DBDBDB363636A7A7A7C2C2C26C6C6C7676766F6F6F282828D7D7D7DEDEDE696969
ECECECECECECECECECECECEC7575759D9D9DD2D2D2D1D1D1ECECECA3A3A3DDDDDDECECEC
E5E5E55B5B5BDADADADEDEDEACACACECECECD0D0D0595959767676989898ECECECECECEC
5A5A5ADDDDDDECECECCFCFCF949494ECECEC9B9B9BC1C1C1ECECEC9C9C9CECECECECECEC
ECECECECECECDEDEDE7E7E7E7E7E7EECECECC1C1C1A5A5A5ECECECD0D0D0696969ECECEC
ECECEC9B9B9BECECECD6D6D68D8D8DC8C8C8ECECECECECECEBEBEB646464CFCFCFECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECEAEAEA8B8B8BECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECA7A7A7B3B3B3ECECECECECECECECEC
ECECECECECECECECECECECECAFAFAFB7B7B7ECECECECECECECECECECECECECECECECECEC
ECECECA7A7A7C8C8C8ECECECB1B1B1C1C1C1ECECECECECECECECECECECECECECECECECEC
ECECECB1B1B1C1C1C1C8C8C83636368E8E8E6565658989897D7D7DECECEC9D9D9DE1E1E1
919191787878BCBCBC8F8F8FECECECECECECECECECECECECECECECECECECEBEBEB787878
D1D1D1AFAFAF3636368D8D8D3D3D3D8B8B8B6D6D6DA7A7A79E9E9EECECEC6C6C6CE4E4E4
848484B9B9B9ECECECD2D2D28D8D8DD1D1D1ECECECA3A3A3DDDDDDECECECECECECECECEC
C2C2C2141414C3C3C3DEDEDE7373739A9A9A141414CDCDCDB3B3B3A7A7A7DEDEDE6A6A6A
B3B3B3505050E0E0E0373737D4D4D4282828E3E3E3565656C2C2C2959595ECECECBBBBBB
A7A7A75B5B5B3939396D6D6D868686929292C2C2C2ECECECECECECC1C1C19B9B9B2E2E2E
C2C2C2ECECECECECEC989898C2C2C2E9E9E92222229A9A9A919191C0C0C0575757B0B0B0
C9C9C9414141767676535353A7A7A75B5B5B393939ACACACECECECECECECCFCFCF505050
E0E0E0373737B7B7B72E2E2EC0C0C0575757B0B0B09090908D8D8D9D9D9DB3B3B3A7A7A7
5B5B5B393939828282141414C1C1C13C3C3CC2C2C2868686929292C2C2C2ADADAD787878
B3B3B3666666E3E3E3414141969696141414C1C1C13C3C3CC2C2C2BBBBBB828282787878
E9E9E98C8C8CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECB8B8B8000000
000000414141ECECECECECECECECECD0D0D0AAAAAAECECEC9D9D9DECECEC393939000000
393939ECECECE6E6E6111111A5A5A50E0E0EDDDDDDECECEC393939000000929292B8B8B8
2020200000008686860E0E0EDDDDDDC1C1C1000000030303C2C2C2ECECEC5C5C5C000000
000000585858E9E9E91C1C1C0707075B5B5B0C0C0CE4E4E4727272060606DDDDDDECECEC
ECECECE6E6E60E0E0E070707979797ECECECD0D0D0AAAAAAECECEC9D9D9DECECECECECEC
5A5A5ADDDDDDECECECECECEC707070000000646464ECECECB8B8B80000000000002C2C2C
ECECECECECECECECECECECEC888888ECECECC1C1C1A5A5A5ECECECB8B8B8838383ECECEC
ECECEC9B9B9BECECECECECEC979797A5A5A5ECECECECECECC1C1C1000000525252ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECEC979797000000A5A5A5ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECEC4B4B4B000000C8C8C8ECECECECECEC
ECECECECECECECECECECECEC9D9D9DECECECECECECECECECECECECECECECECECECECECEC
ECECECC1C1C1A5A5A5ECECEC707070070707D7D7D7ECECECECECECECECECECECECECECEC
ECECEC7070700707076C6C6C0C0C0CD6D6D64F4F4FE6E6E64F4F4FECECEC3F3F3FD6D6D6
1C1C1C8B8B8B1C1C1C8F8F8FECECECECECECECECECECECECECECECECECECDEDEDE111111
6161611C1C1C0C0C0C797979838383818181838383ECECEC9797970000004B4B4B000000
646464979797929292B8B8B8838383ECECECECECECA3A3A3DDDDDDECECECECECECECECEC
E9E9E9282828545454000000B7B7B7E6E6E6282828474747C2C2C24B4B4B000000B7B7B7
ECECECB8B8B80000000707076C6C6C0C0C0CA7A7A7070707C2C2C2AFAFAF000000747474
ECECEC393939393939E5E5E59B9B9B000000AAAAAAECECECECECECD0D0D0080808393939
E5E5E5ECECECECECEC5C5C5C000000C5C5C5282828A9A9A91C1C1CE5E5E5878787878787
6464647272720000008F8F8FECECEC393939393939E8E8E8ECECECECECECECECECB8B8B8
000000070707D4D4D4232323C2C2C28D8D8D8F8F8F1C1C1CB1B1B11C1C1CE5E5E5ECECEC
393939393939E5E5E5282828545454393939E5E5E59B9B9B000000AAAAAAE6E6E6727272
ECECEC979797000000838383E9E9E9282828545454393939E5E5E57070700000008F8F8F
E9E9E98C8C8CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECE9E9E9767676555555767676969696ECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE5E5E5AAAAAA
ECECECECECECECECECECECECECECECECECECECECECECECECECECECB1B1B1E4E4E4ECECEC
ECECECECECECECECECECECECECECECE9E9E9767676555555767676969696ECECEC929292
333333DDDDDDECECECECECECECECECECECECECECECECECECECECECC4C4C4ECECECECECEC
ECECECCDCDCD656565767676767676ECECEC8E8E8E525252747474646464D7D7D7ECECEC
ECECEC616161B7B7B7ECECEC979797949494ECECECECECECECECECDEDEDE919191D2D2D2
B7B7B7ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECEC9E9E9EC5C5C5D2D2D2D1D1D1ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECE5E5E55E5E5ECBCBCBB7B7B7ECECEC
ECECECECECECECECECECECEC959595ECECECECECECECECECECECECECECECECECECECECEC
ECECECC1C1C1949494ECECECECECECAAAAAADFDFDFECECECECECECECECECECECECECECEC
ECECECECECECAAAAAADFDFDFECECECECECECECECECECECECECECECECECECB1B1B1E4E4E4
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
BEBEBEECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECA7A7A7838383ECECECD2D2D2545454DDDDDDECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECCCCCCC2E2E2ED7D7D7ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECBBBBBBC2C2C2C2C2C21A1A1ADDDDDDECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
CCCCCC2E2E2ED7D7D7ECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ADADAD787878ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECD0D0D0111111C8C8C8ECECECECECECECECEC9B9B9B000000
A5A5A5ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECEAEAEA4E4E4E000000C8C8C8ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECE9E9E9272727A5A5A5ECECEC
ECECECECECECECECECB8B8B8646464ECECECECECECECECECECECECECECECECECECECECEC
ECECECDEDEDE111111A5A5A5ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECD2D2D2070707C2C2C2ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECEAEAEA565656D7D7D7ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECDEDEDE1D1D1DDDDDDDECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
EAEAEA565656D4D4D4232323C2C2C2ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE6E6E6727272
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
E6E6E6727272ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ACECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECEE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFE7E7E7EEEEEEEFEFEFF0F0F0F4F4F4F6F6F6F8F8F8F9F9F9F9F9F9FBFBFB
FBFBFBE7E7E78A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF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000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF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000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF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000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF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000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF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000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF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000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF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000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDCDCDC
262626000000282828DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACAF7F7F7EDEDEDE9E9E9E6E6E6DCDCDCDCDCDCD3D3D3CFCFCF
E7E7E79696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACAF7F7F7EDEDEDEAEAEAE7E7E7DCDCDCDCDCDCD3D3D3CFCFCF
E7E7E79696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECBFBFBFDBDBDBDBDBDBDBDBDBDADADADBDBDBDBDBDBDBDBDBDBDBDB
C8C8C8C1C1C18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECBDBDBD9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F9F
969696E6E6E68A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECC2C2C2D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
E1E1E19696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACACECECECECECECECECECECECECECECECECECECECECECECECE
E6E6E69696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACACECECECECECECECECECECECECECECECECECECECECECECECE
E6E6E69696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACACECECE868686CECECECECECECECECECECECE868686CECECE
E6E6E69696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACACFCFCF3E3E3E3E3E3ECFCFCFCFCFCF3E3E3E3E3E3ECFCFCF
E7E7E79696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACAD7D7D7B2B2B23E3E3E6363633E3E3E3E3E3EB2B2B2D7D7D7
EBEBEB9696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACAE4E4E4E4E4E4E4E4E43E3E3E3E3E3EE4E4E4E4E4E4E4E4E4
F1F1F19696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACAF0F0F0F0F0F0F0F0F0C4C4C4F0F0F0F0F0F0F0F0F0F0F0F0
F7F7F79696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECCACACAFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECC0C0C0FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
F5F5F59696968A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8ADFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFECECECB6B6B6B6B6B6B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
A1A1A1DADADA8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A9999996E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E6E
6E6E6E9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0EEEEEEC6C6C6BFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBFBF
8686869999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0EEEEEEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
8686869999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEF5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5
D5D5D59999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999FCFCFC
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999FCFCFC999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999FCFCFC999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
999999999999999999999999999999999999999999999999999999999999999999999999
9999999999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDFA5A5A5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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDFA5A5A5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000000FCFCFC8A8A8A999999FFFFFFFFFFFF7878780000000000006D6D6DDFDFDFA5A5A5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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFF8E8E8ED7D7D7616161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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFF838383F7F7F7FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFF5F5F5969696E9E9E9
D4D4D43E3E3E3E3E3ED4D4D43E3E3E3E3E3ED4D4D4D4D4D4FFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFDFDFDF888888FFFFFF7D7D7DD2D2D2FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFD5D5D5D
D2D2D2FFFFFFFFFFFFBBBBBB606060FFFFFFDFDFDF212121F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFF5F5F5969696E9E9E9D4D4D43E3E3E3E3E3E
D4D4D43E3E3E3E3E3ED4D4D4D4D4D4FFFFFFCACACA999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFBBBBBB7F7F7F7F7F7FB6B6B6FFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696F6F6F6
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696F6F6F6EDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDFFFFFFCACACA999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFCECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECEF5F5F5969696FEFEFE
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
E2E2E2979797FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696FEFEFEFDFDFDFDFDFDFDFDFD
FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFFFFFFCACACA999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999DADADAB5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5AFAFAF777777B3B3B3B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5CFCFCFE4E4E4C9C9C9
D2D2D2D3D3D3D4D4D4D6D6D6D6D6D6D7D7D7D7D7D7D7D7D7C5C5C5EFEFEF999999FCFCFC
999999FFFFFFFFFFFFC6C6C65757577F7F7F818181FBFBFBA8A8A87777777F7F7F999999
DFDFDFA5A5A5FBFBFB979797CECECE5A5A5A7F7F7F717171D1D1D15353538D8D8D9C9C9C
FBFBFBAFAFAF575757FBFBFBFFFFFFFFFFFFDFDFDF535353FBFBFBFFFFFFEFEFEF535353
A4A4A4777777747474E8E8E8EBEBEB6A6A6ACACACA6C6C6C7F7F7F9A9A9AFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF4F4F4E4E4E4C9C9C9D2D2D2D3D3D3D4D4D4
D6D6D6D6D6D6D7D7D7D7D7D7D7D7D7C5C5C5EFEFEF999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999DADADAB5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5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000000FCFCFC8A8A8A999999DADADAB5B5B59D9D9D4D4D4D5A5A5A5959595D5D5D9C9C9C
3A3A3AB2B2B29D9D9D2C2C2C5F5F5F0D0D0D949494242424595959404040A5A5A56C6C6C
AFAFAF7979799696966C6C6CACACAC5C5C5C5555554B4B4B7D7D7D3F3F3F5A5A5A505050
8A8A8AB5B5B5B5B5B56464645A5A5A3939398B8B8B9E9E9E747474B2B2B26B6B6BA0A0A0
535353646464262626A5A5A5B5B5B5AAAAAA3E3E3E575757595959505050A2A2A2444444
5656564E4E4EA7A7A73636365555554545459696966C6C6CAFAFAF797979969696A0A0A0
5353536464645E5E5EB5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5CECECEEEEEEEEFEFEF
F0F0F0F1F1F1F4F4F4F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC
999999FFFFFFFFFFFFC6C6C6AFAFAFFFFFFFBBBBBBB4B4B47575757B7B7B7F7F7F646464
DFDFDF969696F9F9F9929292C0C0C0505050959595929292E3E3E3535353959595F7F7F7
EDEDED868686EDEDED949494FFFFFFFFFFFFDFDFDF3232327D7D7D676767B2B2B2535353
858585F7F7F7A4A4A4C3C3C3DFDFDFA5A5A5C0C0C0686868ABABABBDBDBDFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000000000BFBFBFFFFFFFF3F3F3EEEEEEEFEFEFF0F0F0F1F1F1F4F4F4
F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999DADADAB5B5B55A5A5AAAAAAAB5B5B5B5B5B55555558F8F8F
3A3A3AB2B2B28C8C8C0B0B0BACACAC1B1B1B9A9A9A525252B5B5B55555558181816C6C6C
AFAFAF7979799696966C6C6C9898983A3A3AB2B2B2B5B5B5B5B5B5B5B5B5555555000000
818181B5B5B5B5B5B54B4B4BAAAAAA6565659696969E9E9E747474B2B2B26B6B6B727272
8D8D8DB2B2B2414141A5A5A5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5727272797979454545
B5B5B55555558F8F8F3A3A3AAFAFAF6565659696966C6C6CAFAFAF797979969696727272
8D8D8DB2B2B26B6B6BB5B5B5B5B5B5000000B5B5B5B5B5B5B5B5B5CECECEEEEEEEEFEFEF
F0F0F0F1F1F1F4F4F4F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC
999999FFFFFFFFFFFFC6C6C6AFAFAFFFFFFFACACACD2D2D2D0D0D0000000000000AFAFAF
DFDFDF434343000000AFAFAFFBFBFB5454540000008F8F8FEFEFEF4E4E4E252525414141
707070C7C7C7FFFFFF939393FFFFFFFFFFFFDFDFDF535353FBFBFBDFDFDF959595535353
CCCCCC000000474747FBFBFB545454202020F3F3F32A2A2A000000C7C7C7FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000000000FFFFFFFFFFFFF3F3F3EEEEEEEFEFEFF0F0F0F1F1F1F4F4F4
F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999DADADAB5B5B5686868B5B5B5B5B5B52A2A2A0000008F8F8F
3A3A3AB2B2B2B5B5B59E9E9E0606060707079A9A9A525252B5B5B55555558181816C6C6C
AFAFAF7979799696966C6C6C9898983A3A3AB2B2B2B5B5B5A9A9A90D0D0D222222B5B5B5
B5B5B5B5B5B5B5B5B5A9A9A90D0D0D0303039696969E9E9E747474B2B2B26B6B6B727272
8D8D8DB2B2B2414141A5A5A5B5B5B5B5B5B5B5B5B5B2B2B21414140B0B0B9B9B9B454545
B5B5B55555558F8F8F3A3A3AAFAFAF7979799696966C6C6CAFAFAF797979969696727272
8D8D8DB2B2B26B6B6BB5B5B5B5B5B5000000000000B5B5B5B5B5B5CECECEEEEEEEEFEFEF
F0F0F0F1F1F1F4F4F4F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC
999999FFFFFFFFFFFFC6C6C6AFAFAFFFFFFF838383F7F7F7FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999F7F7F7
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDF535353FBFBFBC6C6C68C8C8C535353
FBFBFBFFFFFFFFFFFFFFFFFFDFDFDFA5A5A5FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFF3F3F3EEEEEEEFEFEFF0F0F0F1F1F1F4F4F4
F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999DADADAB5B5B5616161AFAFAFB5B5B5B5B5B5888888A1A1A1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000000FCFCFC8A8A8A999999DADADAB5B5B5858585454545B5B5B5A9A9A9171717AFAFAF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000000FCFCFC8A8A8A999999DADADAB5B5B5B5B5B58585855A5A5A5C5C5CA5A5A5B5B5B5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000000FCFCFC8A8A8A999999DADADAB5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5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000000FCFCFC8A8A8A999999ECECECDADADADADADADADADADADADADADADADADADADADADA
DADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADA
DADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADA
DADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADA
DADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADA
DADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADA
DADADADADADADADADADADADADADADADADADADADADADADADADADADAE0E0E0EEEEEEEFEFEF
F0F0F0F1F1F1F4F4F4F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC
999999FFFFFFFFFFFFFFFFFFD3D3D37F7F7FA3A3A3FFFFFFE7E7E77F7F7F7F7F7FECECEC
EFEFEFD2D2D2FDFDFDCBCBCBFBFBFB8E8E8ECDCDCDD0D0D07F7F7FD9D9D9FFFFFFBBBBBB
7F7F7FA3A3A3E8E8E8CCCCCCFBFBFBFFFFFFFFFFFFFFFFFFCCCCCCFBFBFBFFFFFFFFFFFF
CCCCCCFBFBFBA8A8A87F7F7FD9D9D9FDFDFD8E8E8ECCCCCCA9A9A97F7F7FA0A0A0FBFBFB
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF3F3F3EEEEEEEFEFEFF0F0F0F1F1F1F4F4F4
F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF3F3F3EEEEEEEFEFEF
F0F0F0F1F1F1F4F4F4F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC
999999FFFFFFFFFFFFF7F7F78D8D8DFFFFFFFFFFFFFFFFFF939393FFFFFFFFFFFF969696
DFDFDFA5A5A5FBFBFB979797DFDFDF535353DEDEDE606060FFFFFF787878C9C9C9535353
F7F7F78E8E8ED2D2D2999999F7F7F7FFFFFFFFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFF
999999BEBEBEAFAFAFFFFFFF787878C9C9C9535353FBFBFBFFFFFF787878000000B4B4B4
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFF3F3F3EEEEEEEFEFEFF0F0F0F1F1F1F4F4F4
F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFF8888887B7B7B7F7F7F979797DFDFDF8C8C8C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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDFA5A5A5
FFFFFF787878000000B4B4B4999999F7F7F7FFFFFFDFDFDF090909B4B4B4FFFFFFFFFFFF
C6C6C67F7F7FF7F7F77C7C7CDFDFDF535353F9F9F95D5D5DBABABA8F8F8FFDFDFD5D5D5D
D2D2D2FFFFFFFFFFFF999999EFEFEF6E6E6EEBEBEB636363DADADA0F0F0FF3F3F3262626
D8D8D8535353F7F7F7999999F7F7F7EFEFEF535353959595F7F7F7FFFFFFFFFFFFA2A2A2
C7C7C7FFFFFFFFFFFFFFFFFFFFFFFF787878000000D9D9D9FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFF3F3F3EEEEEEEFEFEF
F0F0F0F1F1F1F4F4F4F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC
999999FFFFFFFFFFFFF7F7F78D8D8DFFFFFFEDEDEDE3E3E3D0D0D0000000000000C7C7C7
DFDFDF434343000000ADADAD5454540000009C9C9CA2A2A20000008F8F8FEFEFEF535353
F7F7F7ACACACD2D2D29696964A4A4A7F7F7FFFFFFFFFFFFF999999F7F7F7EFEFEF535353
959595F7F7F75151510000008F8F8FA7A7A70000009A9A9A5454540000008F8F8FFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000000000FFFFFFFFFFFFF3F3F3EEEEEEEFEFEFF0F0F0F1F1F1F4F4F4
F7F7F7F8F8F8F9F9F9FAFAFAFBFBFBFBFBFBF3F3F3999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDF858585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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFDFDFDFA3A3A3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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFFFFFFFFEFEFEFD2D2D2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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFCCCCCCFBFBFBD0D0D0E2E2E2B1B1B1F9F9F9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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F3F3F3797979B6B6B6636363DADADA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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999D6D6D6969696B1B1B1636363E8E8E8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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999A0A0A0D2D2D2A8A8A8636363E7E7E7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000000FCFCFC8A8A8A999999FFFFFFFFFFFF9999999C9C9CF7F7F7B8B8B8636363F7F7F7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000000FCFCFC8A8A8A999999FFFFFFFFFFFF858585939393FFFFFFBEBEBE2E2E2EF7F7F7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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFBFBFBA9A9A9FFFFFFFFFFFFFFFFFFC6C6C69F9F9F
535353DEDEDE0000000F0F0F8D8D8DFFFFFF515151000000AFAFAFDFDFDF6E6E6E2A2A2A
8F8F8FEFEFEF535353FBFBFBF7F7F71D1D1D2B2B2B1D1D1DC9C9C9535353EBEBEB121212
0000007F7F7FFFFFFFFFFFFFFFFFFFA7A7A7000000000000C7C7C7FFFFFF515151000000
D9D9D9DFDFDF535353EBEBEB535353EBEBEB5353538F8F8F121212000000C7C7C7FFFFFF
3C3C3C3C3C3C929292F7F7F71D1D1D9898981D1D1D2B2B2B1D1D1DD5D5D51D1D1D414141
959595F7F7F7515151FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696FBFBFB
EDEDEDEDEDED959595959595959595959595D0D0D0D0D0D0FFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFBFBFB262626E8E8E8C6C6C6AFAFAFFFFFFFFFFFFF969696
FFFFFFFFFFFFDFDFDF090909000000090909D8D8D85353538F8F8FFFFFFFBBBBBBB4B4B4
DFDFDFA5A5A5DFDFDF090909606060FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFF000000000000FFFFFFFFFFFFF5F5F5969696FBFBFBEDEDEDEDEDED959595
959595959595959595D0D0D0D0D0D0FFFFFFCACACA999999FCFCFC999999FFFFFFFFFFFF
DFDFDF090909000000303030DFDFDFA3A3A3545454000000202020F7F7F78A8A8A414141
959595F7F7F7FFFFFFFFFFFFDFDFDFA5A5A5A2A2A2000000B4B4B4FFFFFF8A8A8A414141
B7B7B7000000474747FFFFFFC6C6C60000000000007F7F7FFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFF939393FFFFFFFFFFFFFDFDFD5D5D5DD8D8D8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000000FCFCFC8A8A8A999999FFFFFFFFFFFFADADAD4747477F7F7F5D5D5D7A7A7AEDEDED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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFBFBFB2C2C2CEFEFEF999999F7F7F7EFEFEF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFABABAB919191AAAAAAFFFFFFEFEFEF
535353ECECEC7F7F7F878787C6C6C6FFFFFFA8A8A87F7F7FD7D7D7DFDFDF505050909090
C7C7C7F7F7F7A6A6A6959595F7F7F7D0D0D07F7F7FAFAFAFF7F7F7A9A9A9F5F5F5888888
7F7F7FBFBFBFFFFFFFFFFFFFFBFBFB888888FFFFFFFBFBFBADADADFFFFFFA8A8A87F7F7F
E3E3E3EFEFEFB6B6B6949494D7D7D7949494D9D9D9D2D2D2A9A9A97F7F7FE3E3E3EFEFEF
D2D2D2FDFDFDCACACA999999292929CBCBCBD0D0D07F7F7FAFAFAFCBCBCB2929299C9C9C
CACACAFBFBFBA8A8A8FFFFFFFFFFFF0000007F7F7FFFFFFFFFFFFFF5F5F5969696F3F3F3
F6F6F6F6F6F6F4F4F4F1F1F1EDEDEDEAEAEAE7E7E7E7E7E7EBEBEBCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696F3F3F3F6F6F6F6F6F6F4F4F4
F1F1F1EDEDEDEAEAEAE7E7E7E7E7E7EBEBEBCACACA999999FCFCFC999999FFFFFFFFFFFF
DFDFDFA5A5A5FFFFFFFFFFFFEFEFEFD1D1D1A9A9A97F7F7FA3A3A3FDFDFD6161619C9C9C
979797F7F7F7FFFFFF8E8E8ED7D7D7A5A5A5D0D0D07F7F7FC7C7C7FDFDFD6161619C9C9C
C7C7C7FFFFFF919191EFEFEFC6C6C6AFAFAFFFFFFF979797E4E4E4FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFF787878C7C7C7FFFFFFEFEFEF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696E6E6E6
CECECECECECECECECECECECECECECECECECECECECECECECEFFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696E6E6E6CECECECECECECECECE
CECECECECECECECECECECECECECECEFFFFFFCACACA999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7FFFFFF7D7D7DD2D2D2BBBBBB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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7EFEFEF525252D2D2D28C8C8C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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999F7F7F7818181959595D2D2D2999999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000000FCFCFC8A8A8A999999FFFFFFFFFFFF999999A8A8A8A2A2A2ACACACD2D2D2999999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000000FCFCFC8A8A8A999999FFFFFFFFFFFF787878606060FFFFFFACACACD2D2D2FFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFEFEFEF535353FBFBFBFFFFFFBBBBBB8F8F8FFFFFFF8E8E8ED2D2D2FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFDFDFDFA5A5A5FFFFFFFFFFFFFFFFFFFFFFFFDFDFDF535353
FBFBFBFFFFFFFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFF5F5F5969696F9F9F9
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFFCACACA999999FCFCFC
999999FFFFFFFFFFFFFFFFFFA8A8A8FFFFFFFFFFFFDFDFDF686868EFEFEFFFFFFFEFEFEF
535353FBFBFBFFFFFFFFFFFFFFFFFFDFDFDFA5A5A5FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFEFEFEF535353FBFBFBFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999F7F7F7
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFEFEFEF535353F7F7F7
FFFFFFDFDFDFA5A5A5FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF999999
F7F7F7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFE2E2E2
8F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5969696F9F9F9F4F4F4F4F4F4F4F4F4
F4F4F4F4F4F4F4F4F4F4F4F4F4F4F4FFFFFFCACACA999999FCFCFC999999FFFFFFFFFFFF
DFDFDFA5A5A5FFFFFF838383D7D7D7A5A5A5FFFFFFFFFFFFFFFFFFFFFFFF999999F7F7F7
939393FFFFFFF7F7F78D8D8DC6C6C6AFAFAFFFFFFF838383F7F7F7FFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FFFFFFFFFFFF7D7D7DD2D2D2FFFFFFACACACD2D2D2FFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8A999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5DADADAA4A4A4
B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FECECEC999999FCFCFC
999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF5F5F5DADADAA4A4A4B5B5B5B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B5B5B5B5B4B4B48F8F8FECECEC999999FCFCFC999999FFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFF999999FCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC999999FCFCFC
999999CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC999999FCFCFC999999CCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCC999999FCFCFC999999CCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCC9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FDFDFDFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FDFDFD9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFF9999998A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999ADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADAD9E9E9EAEAEAEADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
ADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADADAD
AEAEAEBEBEBE8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999CACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACAA3A3A3C3C3C3CACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
CACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACACA
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4A3A3A3C3C3C3D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDF000000DFDFDFDFDFDFDFDFDF0000006F6F6FDFDFDF000000000000000000
DFDFDFDFDFDF000000000000000000DFDFDFDFDFDFDFDFDFDFDFDF6F6F6F000000000000
DFDFDF6F6F6F000000DFDFDF6F6F6F000000DFDFDF000000000000DFDFDFDEDEDEDEDEDE
DEDEDEDEDEDE000000000000000000000000DEDEDEDEDEDEDEDEDE000000000000000000
0000006F6F6F0000006F6F6F0000006F6F6FDFDFDF0000006F6F6F0000006F6F6FDFDFDF
000000DFDFDFDFDFDF000000000000DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDEDEDEA3A3A3C3C3C3DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF0000006F6F6FDFDFDFDFDFDF000000
000000DFDFDF0000006F6F6FDFDFDF000000000000000000DFDFDFDFDFDFDFDFDFDFDFDF
000000DFDFDFDFDFDFDFDFDF0000006F6F6FDEDEDE6F6F6F000000000000000000DEDEDE
6F6F6F0000006F6F6F000000DEDEDE6F6F6F000000DEDEDE000000000000000000DEDEDE
6F6F6F0000006F6F6F0000006F6F6F000000DFDFDF6F6F6F000000000000000000DFDFDF
0000000000006F6F6F000000DFDFDF000000000000000000DFDFDFDFDFDF000000000000
0000006F6F6FDFDFDFDFDFDFDFDFDF0000000000000000006F6F6FDFDFDFDFDFDF6F6F6F
000000000000DFDFDFDFDFDF000000000000000000DFDFDF6F6F6F000000DFDFDF000000
000000DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDF
DFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDEDEDE
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC7E7E7E7E7E7E7E7E7EFCFCFCFCFCFCFCFCFC
7E7E7E7E7E7E7E7E7EBDBDBD7E7E7EBDBDBD7E7E7E7E7E7EFCFCFC7E7E7EBDBDBDFCFCFC
FCFCFCFCFCFC7E7E7EBDBDBD7E7E7EBDBDBDFCFCFC7E7E7EFCFCFCFCFCFCFCFCFCFCFCFC
7E7E7EFCFCFCFCFCFCFCFCFCFCFCFCBDBDBD7E7E7E7E7E7E7E7E7EBDBDBD7E7E7EFCFCFC
BDBDBD7E7E7EFCFCFC7E7E7E7E7E7E7E7E7EFCFCFCFCFCFC7E7E7EBDBDBD7E7E7EBDBDBD
FCFCFC7E7E7E7E7E7E7E7E7EFCFCFC7E7E7EBDBDBDFCFCFC7E7E7EBDBDBDFCFCFC7E7E7E
7E7E7EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E43A3A3A707070707070707070000000ACACAC000000727272E4E4E4000000
E4E4E4000000727272E4E4E4000000E4E4E4E4E4E4E4E4E4E4E4E4000000E4E4E4727272
000000727272000000E4E4E4727272000000ACACAC747474383838747474E4E4E4E4E4E4
E4E4E4727272000000727272000000E4E4E4E4E4E4E4E4E4E4E4E4000000727272E4E4E4
E4E4E4E4E4E4000000727272000000727272E4E4E4000000727272000000383838000000
ACACACE4E4E4747474727272000000ACACACE4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4A3A3A3C3C3C3E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4000000727272E4E4E4E4E4E4000000
000000E4E4E4000000727272000000727272E4E4E4747474ACACACE4E4E4E4E4E4E4E4E4
000000E4E4E4000000727272000000727272E4E4E4000000E4E4E4E4E4E4000000E4E4E4
000000E4E4E4727272000000E4E4E4727272000000727272000000E4E4E4747474747474
727272000000727272000000727272000000E4E4E4000000E4E4E4727272000000727272
000000E4E4E4727272000000727272000000E4E4E4747474747474727272000000E4E4E4
000000727272E4E4E4E4E4E4E4E4E4000000727272E4E4E4000000E4E4E4E4E4E4000000
E4E4E4727272000000727272000000E4E4E4727272000000727272000000383838747474
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4E4
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E000000FCFCFC000000
0000007E7E7E0000007E7E7E0000007E7E7E0000000000007E7E7E000000FCFCFCFCFCFC
FCFCFCFCFCFC0000007E7E7E0000007E7E7EFCFCFC000000FCFCFCFCFCFCFCFCFCFCFCFC
000000FCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC0000000000007E7E7E000000FCFCFC
7E7E7E0000007E7E7E000000FCFCFC0000000000007E7E7E000000FCFCFC0000007E7E7E
000000000000FCFCFC0000007E7E7E0000007E7E7EFCFCFC0000007E7E7E0000007E7E7E
0000007E7E7EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECEC767676000000ECECEC000000000000ECECEC000000767676ECECEC000000
ECECEC000000767676ECECEC000000ECECEC000000000000ECECEC000000ECECEC767676
000000767676000000ECECEC767676000000ECECEC000000767676ECECECECECECECECEC
ECECEC767676000000000000000000000000ECECECECECECECECEC000000767676ECECEC
ECECECECECEC000000767676000000767676ECECEC000000767676000000000000ECECEC
ECECECECECECECECEC000000ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECA3A3A3C3C3C3ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECEC000000767676ECECECECECEC000000
000000ECECEC000000767676000000000000000000000000767676ECECECECECECECECEC
000000000000000000000000000000767676ECECEC767676000000000000000000ECECEC
000000ECECEC767676000000ECECEC767676000000767676000000000000000000000000
767676000000767676000000767676000000ECECEC767676000000000000000000767676
000000ECECEC767676000000767676000000ECECECECECECECECECECECEC000000000000
000000767676ECECECECECECECECEC000000767676ECECEC000000ECECECECECEC000000
ECECEC767676000000767676000000ECECEC767676000000767676000000767676ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E000000FCFCFC000000
000000FCFCFC0000007E7E7E0000007E7E7E0000000000007E7E7E000000FCFCFCFCFCFC
FCFCFCFCFCFC0000007E7E7E0000007E7E7EFCFCFC000000FCFCFCFCFCFCFCFCFCFCFCFC
000000FCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E0000007E7E7E000000FCFCFC
7E7E7E0000007E7E7E000000FCFCFCFCFCFCFCFCFC7E7E7E000000FCFCFC0000007E7E7E
000000000000FCFCFC0000007E7E7E0000007E7E7EFCFCFC0000007E7E7EFCFCFC7E7E7E
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECEC767676000000ECECEC000000000000ECECEC000000767676ECECEC000000
ECECEC000000767676ECECEC000000ECECECECECECECECECECECEC000000ECECEC767676
000000767676000000000000767676000000767676000000767676000000ECECECECECEC
ECECECECECEC000000767676ECECECECECECECECECECECECECECEC000000767676ECECEC
ECECECECECEC000000767676000000000000ECECEC000000767676000000000000000000
ECECECECECEC000000767676000000767676ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECA3A3A3C3C3C3ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECEC000000767676ECECECECECEC000000
000000767676000000767676000000767676ECECEC000000767676ECECECECECECECECEC
000000000000000000000000000000767676ECECECECECECECECECECECEC000000ECECEC
000000ECECEC767676000000000000767676000000767676000000ECECEC000000000000
767676000000767676000000767676000000ECECECECECECECECEC767676000000767676
000000ECECEC767676000000767676000000ECECEC000000000000ECECECECECECECECEC
000000767676ECECECECECECECECEC000000000000000000767676ECECECECECEC000000
ECECEC767676000000767676000000ECECEC767676000000767676000000000000ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC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000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E000000FCFCFC000000
000000FCFCFC0000007E7E7E0000007E7E7E0000000000007E7E7E0000007E7E7EFCFCFC
FCFCFCFCFCFC0000007E7E7E0000000000000000007E7E7EFCFCFCFCFCFCFCFCFCFCFCFC
000000FCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC7E7E7E0000007E7E7E000000000000
000000FCFCFCFCFCFC000000000000000000FCFCFC7E7E7E0000007E7E7E0000007E7E7E
FCFCFC000000000000000000FCFCFC000000000000000000000000FCFCFCFCFCFC000000
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFA000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA000000
FAFAFAFAFAFAFAFAFAFAFAFA000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFA7D7D7D000000000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAA3A3A3C3C3C3FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFAFAFAFA000000
000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
000000FAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
000000FAFAFA7D7D7D000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA7D7D7D
000000FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFA0000007D7D7DFAFAFA000000FAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA7D7D7D000000FAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
FAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFAFA
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC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000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000000000000000FCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC0000007E7E7E000000000000FCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFC0000007E7E7EFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
000000000000000000000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000007E7E7E
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFF9F9F9A3A3A3C3C3C3FDFDFDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF4F4F4
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8A999999FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFC999999F8F8F8FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFE6E6E6C5C5C5D3D3D3D5D5D5F3F3F3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFAFAFAD5D5D5
D3D3D3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AA6A6A6BEBEBEC4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4BEBEBEA6A6A6ACACACA3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3BFBFBFE3E3E3E3E3E3D4D4D4B6B6B6A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3
A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A3A9A9A9CECECE
E3E3E3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AC6C6C6AFAFAFC4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4C4
C4C4C4C4C4C4C4C4C4B1B1B1C6C6C6C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1B2B2B2E3E3E3A3A3A3C1C1C1C2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C3C3C3A3A3A3C1C1C1C2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C3C3C3A3A3A3C1C1C1C2C2C2C2C2C2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE1E1E1B9B9B9B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2ACACAC9C9C9CB2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B24B4B4B4B4B4B
4B4B4B7E7E7EB2B2B2BFBFBFDEDEDECCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCB7B7B7E3E3E3A3A3A3CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C3C3C3A3A3A3CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C3C3C3A3A3A3CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9C3C3C3A3A3A3D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9000000D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9BEBEBEE3E3E3A3A3A3D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
C3C3C3A3A3A3D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
C3C3C3A3A3A3D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9D9
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0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000000FCFCFC8A8A8AE0E0E0C1C1C1E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000000000E8E8E8
E8E8E8E8E8E8000000747474E8E8E8000000747474E8E8E8000000E8E8E8000000E8E8E8
E8E8E8000000E8E8E8000000E8E8E8E8E8E8E8E8E8E8E8E8747474000000E8E8E8E8E8E8
E8E8E8E8E8E8000000747474E8E8E8000000E8E8E8000000E8E8E8E8E8E8000000E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8C3C3C3A3A3A3E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000747474
E8E8E8747474000000E8E8E8747474000000E8E8E8E8E8E8E8E8E8000000000000E8E8E8
000000747474E8E8E8000000E8E8E8000000747474E8E8E8000000E8E8E8000000E8E8E8
E8E8E8000000E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8747474000000E8E8E8000000
747474E8E8E8000000E8E8E8E8E8E8000000E8E8E8000000E8E8E8747474000000E8E8E8
747474000000E8E8E8E8E8E8E8E8E8000000000000E8E8E8000000747474E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8C5C5C5E3E3E3A3A3A3E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000747474E8E8E8
E8E8E8747474000000E8E8E8000000747474000000000000E8E8E8000000747474000000
747474E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
C3C3C3A3A3A3E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000E8E8E8E8E8E8747474000000E8E8E8
000000747474E8E8E8E8E8E8E8E8E8000000747474000000747474000000747474000000
000000E8E8E8000000747474E8E8E8747474000000E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
C3C3C3A3A3A3E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8000000747474000000747474000000747474
E8E8E8000000747474E8E8E8000000E8E8E8E8E8E8000000000000E8E8E8000000747474
E8E8E8000000E8E8E8E8E8E8E8E8E8E8E8E8000000E8E8E8E8E8E8E8E8E8E8E8E8747474
000000E8E8E8000000747474E8E8E8000000747474000000747474E8E8E8000000E8E8E8
000000747474E8E8E8E8E8E8E8E8E8747474000000000000E8E8E8E8E8E8E8E8E8E8E8E8
E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8E8
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1ECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECEC000000000000767676
767676ECECEC000000767676ECECEC000000767676ECECEC000000ECECEC000000767676
ECECEC000000ECECEC000000ECECECECECECECECECECECEC767676000000767676767676
ECECECECECEC0000003B3B3BECECEC000000ECECEC000000ECECECECECEC000000ECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECC3C3C3A3A3A3ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC000000767676
7676763B3B3B000000ECECEC767676000000767676000000767676000000000000ECECEC
000000767676ECECEC000000ECECEC0000003B3B3BECECEC000000ECECEC000000ECECEC
ECECEC000000ECECECECECECECECECECECEC7676767676763B3B3B767676ECECEC000000
767676ECECECB1B1B1767676767676000000ECECEC000000ECECEC767676000000ECECEC
767676000000767676000000767676000000000000ECECEC000000767676ECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECC7C7C7E3E3E3A3A3A3ECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECEC000000767676ECECEC
ECECEC767676000000ECECEC000000767676000000000000ECECEC000000767676000000
767676ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3A3A3A3ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECEC000000ECECECECECEC767676000000ECECEC
0000003B3B3B767676000000767676000000767676000000767676000000767676000000
000000ECECEC0000007676767676763B3B3B767676767676ECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3A3A3A3ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECEC000000000000000000000000000000767676
ECECECECECEC767676767676000000ECECEC3B3B3B767676767676B1B1B1000000767676
ECECEC000000767676767676000000ECECEC000000ECECECECECECECECECECECEC767676
000000ECECEC0000003B3B3BECECEC000000767676000000767676ECECEC000000ECECEC
0000003B3B3B767676000000ECECEC3B3B3B767676000000B1B1B1ECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000000000F3F3F3
F3F3F3F3F3F3000000000000F3F3F3F3F3F3000000000000797979F3F3F3000000000000
000000797979F3F3F3000000000000F3F3F3F3F3F3F3F3F3797979000000F3F3F3F3F3F3
F3F3F3F3F3F3000000000000000000797979F3F3F3797979000000000000000000F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3C3C3C3A3A3A3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000797979
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000000000000000F3F3F3000000000000797979
000000000000F3F3F3000000F3F3F3000000000000000000797979F3F3F3797979000000
000000000000F3F3F3F3F3F3F3F3F3797979000000F3F3F3F3F3F3F3F3F3F3F3F3000000
000000F3F3F3797979000000000000797979F3F3F3000000000000797979000000000000
F3F3F3000000000000000000F3F3F3F3F3F3000000000000000000797979F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3CBCBCBE3E3E3A3A3A3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000797979F3F3F3
F3F3F3F3F3F3000000000000000000F3F3F3000000000000F3F3F3000000797979000000
000000F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
C3C3C3A3A3A3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000F3F3F3F3F3F3797979000000F3F3F3
F3F3F3000000000000000000F3F3F3000000000000000000000000000000F3F3F3F3F3F3
000000000000000000F3F3F3F3F3F3000000000000F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
C3C3C3A3A3A3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3000000000000F3F3F3797979000000797979
F3F3F3F3F3F3000000000000797979F3F3F3797979000000000000F3F3F3000000000000
F3F3F3797979000000000000797979F3F3F3000000000000F3F3F3F3F3F3F3F3F3797979
000000F3F3F3000000000000000000000000F3F3F3F3F3F3000000000000000000F3F3F3
F3F3F3000000000000797979F3F3F3000000F3F3F3000000797979F3F3F3F3F3F3F3F3F3
F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3F3
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5F5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000000FCFCFC8A8A8AE0E0E0C1C1C1FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000000000007D7D7D
7D7D7DBCBCBCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFC7F7F7FFCFCFCFCFCFCFCFCFCFCFCFC7E7E7E0000007D7D7D7D7D7D
BCBCBCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCC3C3C3A3A3A3FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC7F7F7F3E3E3E
7D7D7D3E3E3E7F7F7FFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC7F7F7F7F7F7FFCFCFC
7F7F7FBEBEBEFCFCFC7D7D7DFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCBEBEBE0000007D7D7D3E3E3E7F7F7FFCFCFC7F7F7F
BEBEBEFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCBEBEBE7F7F7FFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000007E7E7EFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCCFCFCFE3E3E3A3A3A3FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC7D7D7D0000003E3E3E7D7D7D
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
C3C3C3A3A3A3FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000007D7D7D7D7D7D3F3F3FFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
C3C3C3A3A3A3FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC0000007E7E7EFCFCFCFCFCFC0000007E7E7E
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC7F7F7FBEBEBE
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC7E7E7E
000000FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC000000FCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
FCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFC
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C1C1C1FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFC3C3C3A3A3A3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFD1D1D1E3E3E3A3A3A3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C3C3C3A3A3A3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C3C3C3A3A3A3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
C3C3C3E3E3E38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0C6C6C6F2F2F2FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8AE0E0E0DADADAC2C2C2F3F3F3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFF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000000FCFCFC8A8A8AE0E0E0E0E0E0DADADAC6C6C6C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C4C4C4D7D7D7E1E1E1E1E1E1DADADAC6C6C6C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C8C8C8DCDCDCE1E1E1E1E1E1D2D2D2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C8C8C8D7D7D7
E1E1E1E1E1E1D2D2D2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C4C4C4D7D7D7
E1E1E1E1E1E1D2D2D2C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1
C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C4C4C4D7D7D7
E1E1E1E1E1E18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E4E4E4E8E8E8E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EBEBEBEBEBEB
EBEBEBEBEBEBEBEBEBE9E9E9E9E9E9EAEAEAEAEAEAE6E6E6E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E3E3E3E7E7E7E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EAEAEAEBEBEBEBEBEB
EBEBEBEBEBEBE9E9E9E9E9E9E9E9E9EAEAEAE6E6E6E0E0E0E0E0E0E0E0E0E4E4E4E8E8E8
E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9E9EBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE9E9E9
E9E9E9EAEAEAEAEAEAE6E6E6E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D4D4D4C7C7C7C7C7C7C6C6C6C7C7C7C7C7C7C7C7C7
C7C7C7C8C8C8C9C9C9C8C8C8C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7C7
C8C8C8C9C9C98A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0CFCFCFC3C3C3
C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3
C3C3C3C7C7C7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0CFCFCFC3C3C3C3C3C3
C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3
C7C7C7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0DDDDDDBABABA505050343434343434343434343434343434343434343434343434
343434343434343434343434343434454545868686F1F1F1E0E0E0E0E0E0E0E0E0E0E0E0
DDDDDDE2E2E2717171343434343434343434343434343434343434343434343434343434
3434343434343434343434343C3C3C868686F1F1F1E0E0E0E0E0E0DDDDDDBABABA505050
343434343434343434343434343434343434343434343434343434343434343434343434
343434454545868686F1F1F1E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C7C7C7FEFEFEFDFDFDEFEFEFEAEAEAE9E9E9E8E8E8
E9E9E9E0E0E08E8E8EDCDCDCE6E6E6E4E4E4E4E4E4E3E3E3E4E4E4E3E3E3E2E2E2E2E2E2
E2E2E2B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D6D6D6D2D2D2EBEBEB
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1DEDEDECCCCCCE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D6D6D6D2D2D2EBEBEBF1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
DEDEDECCCCCCE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABE0E0E0E0E0E0
E0E0E0D9D9D96666667A7A7A898989898989898989898989898989898989898989898989
8989898989898989898989898989898989894B4B4BBABABAE0E0E0E0E0E0E0E0E0E0E0E0
D9D9D9818181939393C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C8C8C8C8C8C8C8C8C8C6C6C6696969BABABAE0E0E0E0E0E0D9D9D9767676BBBBBB
C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8C8
C8C8C8C4C4C4696969BABABAE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C7C7C7FCFCFCF7F7F7F0F0F0E7E7E7E6E6E6E5E5E5
E6E6E6E8E8E8A9A9A9B7B7B7E6E6E6E3E3E3E2E2E2E2E2E2D3D3D3D9D9D9E0E0E0DFDFDF
DDDDDDB0B0B08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1EDEDEDF1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1DEDEDEBEBEBED3D3D3F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1C3C3C3E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1B8B8B8D5D5D5F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1C3C3C3E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D5D5D5FFFFFFFFFFFF7F7F7FFFFFFF
FFFFFFFFFFFFFFFFFF000000000000000000FFFFFF7F7F7FFFFFFF7F7F7F000000000000
7F7F7F000000FFFFFF000000FFFFFF000000FFFFFF7F7F7F0000007F7F7FFFFFFF7F7F7F
FFFFFF000000FFFFFF000000FFFFFF000000000000FFFFFFFFFFFF7F7F7FFFFFFFFFFFFF
FFFFFF7F7F7FFFFFFFFFFFFF000000000000FFFFFFFFFFFF7F7F7FFFFFFF000000000000
FFFFFF7F7F7F000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFABABABE0E0E0E0E0E0
E0E0E0D6D6D6737373FFFFFFDBDBDBDBDBDB000000DBDBDBDBDBDB000000DBDBDBDBDBDB
6D6D6DDBDBDB6D6D6DDBDBDBDBDBDBFFFFFFB3B3B38D8D8DE0E0E0E0E0E0E0E0E0E0E0E0
D6D6D6343434EDEDEDDBDBDBDBDBDBDBDBDBABABAB929292929292929292929292919191
B5B5B5DBDBDBDBDBDBDBDBDBDBDBDBFFFFFF8D8D8DE0E0E0E0E0E0D6D6D6999999DBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBCBCBCBB3B3B3DBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBFFFFFF8D8D8DE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D6D6D6D3D3D3F0F0F0FBFBFBDBDBDB737373
A5A5A5C2C2C2BABABAB9B9B9A3A3A39C9C9C9494947A7A7A676767797979777777999999
E0E0E0B0B0B08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1ADADADABABABAFAFAFADADADCCCCCCF1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1C3C3C3E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1F1F1F1
F1F1F1F1F1F1B9B9B9B1B1B1ACACACA3A3A3A1A1A1BBBBBBF1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1C3C3C3E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D5D5D5FFFFFFFFFFFF000000000000
000000FFFFFFFFFFFFFFFFFFFFFFFF7F7F7FFFFFFF7F7F7FFFFFFFFFFFFFFFFFFFFFFFFF
7F7F7F000000FFFFFF000000FFFFFF000000FFFFFF7F7F7FFFFFFF7F7F7FFFFFFF7F7F7F
FFFFFF000000FFFFFF0000007F7F7FFFFFFFFFFFFF000000FFFFFF000000000000000000
FFFFFF7F7F7FFFFFFF000000FFFFFFFFFFFF7F7F7F000000FFFFFFFFFFFFFFFFFF000000
FFFFFF7F7F7FFFFFFFFFFFFFFFFFFF000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFABABABE0E0E0E0E0E0
E0E0E0D6D6D67A7A7AFFFFFFF2F2F2F2F2F2000000F2F2F2F2F2F2F2F2F2F2F2F2F2F2F2
797979000000797979F2F2F2F2F2F2FFFFFFC0C0C08D8D8DE0E0E0E0E0E0E0E0E0E0E0E0
D6D6D6343434F8F8F8F2F2F2F2F2F2F2F2F2F2F2F2CACACA8D8D8D6B6B6B5E5E5ED5D5D5
F2F2F2F2F2F2F2F2F2F2F2F2F2F2F2FFFFFF8D8D8DE0E0E0E0E0E0D6D6D6999999F2F2F2
F2F2F2F2F2F2F2F2F2F2F2F2AEAEAE8E8E8E8E8E8E8E8E8EC8C8C8F2F2F2F2F2F2F2F2F2
F2F2F2F2F2F2FFFFFF8D8D8DE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D8D8D8D8D8D8C5C5C5909090A2A2A2C2C2C2
CBCBCBB6B6B6A9A9A99B9B9B818181757575515151C1C1C1B1B1B1A5A5A5DBDBDBE5E5E5
E3E3E3AFAFAF8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D1D1D1F1F1F1F1F1F1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000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D4D4D4DCDCDCEEEEEE
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
F1F1F1E4E4E4C9C9C9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D4D4D4DCDCDCEEEEEEF1F1F1
F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1F1
E4E4E4C9C9C9E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0D5D5D5FFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFABABABE0E0E0E0E0E0
E0E0E0D6D6D67F7F7FDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB
DBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDBDB8E8E8EA7A7A7E0E0E0E0E0E0E0E0E0E0E0E0
D6D6D6707070C3C3C3FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFF6F6F6A2A2A2A7A7A7E0E0E0E0E0E0D6D6D6898989E6E6E6
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFEEEEEEA2A2A2A7A7A7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D8D8D8C8C8C8BDBDBDD0D0D0C1C1C17D7D7D
A7A7A7CDCDCDADADADB3B3B3A7A7A7979797777777747474999999DDDDDDE5E5E5E5E5E5
E5E5E5AFAFAF8A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0CFCFCFC3C3C3
C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3
C3C3C3C7C7C7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0CFCFCFC3C3C3C3C3C3
C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3C3
C7C7C7E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0ABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABAB
ABABABABABABABABABABABABABABABABABABABABABABABABABABABABABABE0E0E0E0E0E0
E0E0E0D7D7D7B6B6B6505050343434343434343434343434343434343434343434343434
343434343434343434343434343434454545818181DCDCDCE0E0E0E0E0E0E0E0E0E0E0E0
D7D7D7DCDCDC707070343434343434343434343434343434343434343434343434343434
3434343434343434343434343C3C3C818181DCDCDCE0E0E0E0E0E0D7D7D7B6B6B6505050
343434343434343434343434343434343434343434343434343434343434343434343434
343434454545818181DCDCDCE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D7D7D7D6D6D6D5D5D5D6D6D6D1D1D1717171
B0B0B0A4A4A4ABABAB919191B4B4B49A9A9A9696968181817E7E7EC1C1C1D6D6D6D4D4D4
D0D0D0B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0DEDEDED6D6D6D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4CFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFD4D4D4D4D4D4D5D5D5D6D6D6DADADAE0E0E0E0E0E0E0E0E0E0E0E0
DEDEDED7D7D7D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D1D1D1CFCFCFCFCFCF
CFCFCFCFCFCFD4D4D4D4D4D4D4D4D4D6D6D6DADADAE0E0E0E0E0E0DEDEDED6D6D6D4D4D4
D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4D4CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFD4D4D4
D4D4D4D5D5D5D6D6D6DADADAE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D6D6D6D5D5D5D4D4D4D6D6D6C2C2C2808080
C7C7C7D0D0D0D2D2D2A3A3A3A8A8A8ADADADD3D3D3CBCBCB8888887E7E7ED1D1D1D5D5D5
D1D1D1B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D6D6D6D5D5D5D4D4D4D6D6D69C9C9CCDCDCD
D7D7D7D5D5D5D5D5D5CACACA9C9C9CB9B9B9D7D7D7D6D6D6D4D4D4B2B2B2BFBFBFD5D5D5
D1D1D1B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D7D7D7D5D5D5D4D4D4D6D6D6C6C6C6D6D6D6
D4D4D4D4D4D4D5D5D5D6D6D6ABABABB5B5B5D6D6D6D5D5D5D5D5D5D6D6D6D5D5D5D5D5D5
D1D1D1B1B1B18A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C8C8C8D6D6D6CECECECECECECECECECECECECECECE
CECECECECECECECECECFCFCFC4C4C4C4C4C4CECECECECECECECECECECECECECECECECECE
CBCBCBB3B3B38A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0C9C9C9BBBBBBB1B1B1B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2B2
B2B2B2B4B4B48A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000FCFCFC8A8A8AE0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0E0
E0E0E0E0E0E08A8A8AFCFCFC0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000F7F7F7A4A4A49C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C9C
9C9C9C9C9C9CA4A4A4F7F7F70000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000F7F7F7E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3E3
E3E3E3E3E3E3E3E3E3F7F7F70000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000E9E9E9FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFD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000000E0E0E0F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F77B7B7B727272DCDCDCF7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7F7
F7F7F7B6B6B68D8D8DA2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2818181F7F7F7F7F7F7818181A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2
A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A2A28D8D8DB6B6B6F7F7F75858586F6F6F
7171717272727272727272727272727272727272727272727272727272727070705B5B5B
A6A6A6F7F7F7F7F7F7E0E0E00000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000DDDDDDEEEEEEEEEEEEEEEEEECCCCCCCACACAC9C9C9C9C9C9C8C8C8C7C7C7C8C8C8
C8C8C8C8C8C8C9C9C9CCCCCCEEEEEEEEEEEEEEEEEEE5E5E5E2E2E2EEEEEEE8E8E8E9E9E9
EEEEEEDFDFDFECECECDDDDDDDBDBDBE4E4E4E0E0E0EEEEEEE1E1E1DBDBDBE5E5E5DCDCDC
E0E0E0EEEEEEE5E5E5E2E2E2EEEEEEE1E1E1DBDBDBDBDBDBECECECEBEBEBE0E0E0EEEEEE
E1E1E1E7E7E7EEEEEEE1E1E1E7E7E7E6E6E6DBDBDBDFDFDFDCDCDCE4E4E4EBEBEBDBDBDB
DCDCDCEDEDEDDDDDDDEDEDEDEEEEEEE5E5E5DBDBDBDFDFDFEEEEEEE6E6E6DBDBDBDBDBDB
E0E0E0E4E4E4EEEEEEEEEEEEEDEDEDDCDCDCE0E0E0EEEEEEEEEEEEDFDFDFEDEDEDEBEBEB
DBDBDBDBDBDBE2E2E2EEEEEEE8E8E8DBDBDB737373727272CFCFCFDBDBDBDCDCDCEDEDED
EEEEEEEEEEEEEEEEEEEDEDEDDBDBDBDBDBDBDBDBDBEDEDEDEDEDEDDCDCDCE0E0E0EEEEEE
EEEEEEEBEBEBDBDBDBDBDBDBE7E7E7EEEEEEEEEEEEE2E2E2EAEAEAEEEEEEE4E4E4EEEEEE
EEEEEEE3E3E3DBDBDBDCDCDCEDEDEDEBEBEBDBDBDBDBDBDBE2E2E2EEEEEEE5E5E5E0E0E0
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEEE
EEEEEEB1B1B1B9B9B9AEAEAEC1C1C1D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5AEAEAEAEAEAE
AEAEAEAEAEAEAEAEAE979797EEEEEEEEEEEE979797AEAEAEC1C1C1D5D5D5D5D5D5D5D5D5
D5D5D5D5D5D5D5D5D5D5D5D5D5D5D5AEAEAEAEAEAEB9B9B9B1B1B1EEEEEE646464707070
747474777777777777777777777777777777777777777777777777777777717171737373
A1A1A1EEEEEEEEEEEEDDDDDD0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000DADADAE5E5E5E5E5E5B7B7B79B9B9BCACACAE9E9E9E9E9E9D7D7D7858585ECECEC
EEEEEEEBEBEBF2F2F2A3A3A3E5E5E5E5E5E5BCBCBC151515BEBEBEB3B3B3777777D7D7D7
7070706C6C6C4C4C4C1010108585851B1B1BA5A5A5B2B2B21010104141417E7E7E515151
CFCFCFBCBCBC151515C1C1C1AFAFAF101010151515BABABACACACA1C1C1CA3A3A3B3B3B3
4F4F4FD9D9D9B3B3B34F4F4FD6D6D62E2E2E101010505050717171D9D9D9616161101010
8E8E8E505050696969E3E3E3DDDDDD1A1A1A101010A8A8A8D6D6D62E2E2E101010858585
464646D9D9D9E5E5E5E5E5E5828282515151CFCFCFE5E5E5707070696969E3E3E35F5F5F
101010151515C3C3C3E4E4E4404040929292727272101010101010151515C0C0C0D4D4D4
E5E5E5E5E5E5DEDEDE2626261010101010106F6F6FC6C6C67E7E7E515151CFCFCFE5E5E5
E5E5E55F5F5F101010545454C6C6C6DEDEDEAAAAAA8F8F8FD7D7D7B3B3B38D8D8DE3E3E3
C9C9C91616161010107F7F7FD4D4D45F5F5F101010151515C3C3C3DDDDDD3B3B3BCFCFCF
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5E5
E5E5E5ADADADC0C0C0B5B5B5DADADA727272727272727272727272272727B5B5B5B5B5B5
B5B5B5B5B5B5B5B5B59B9B9BE5E5E5E5E5E59B9B9BB5B5B5DADADA272727272727272727
272727272727272727272727272727B5B5B5B5B5B5C0C0C0ADADADE5E5E5666666777777
797979FFFFFFFFFFFF7A7A7A7A7A7A7A7A7A7A7A7ABCBCBCFFFFFF7A7A7A787878777777
9D9D9DE5E5E5E5E5E5DADADA0000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000D9D9D9DFDFDFDFDFDFDEDEDEA0A0A0E9E9E9E5E5E5EAEAEADDDDDD747474EDEDED
ADADADE1E1E1F3F3F3A3A3A3DEDEDEDEDEDEC6C6C6101010AFAFAF6C6C6C383838CACACA
4E4E4E6C6C6C101010B1B1B14E4E4E101010B1B1B14E4E4E7E7E7EDDDDDD7E7E7E515151
CDCDCDB6B6B6151515B1B1B11B1B1B959595C3C3C34C4C4CCDCDCD1C1C1CA1A1A1ADADAD
4F4F4FD5D5D5ADADAD4F4F4F8989894F4F4FD5D5D52F2F2F8E8E8ED5D5D5434343C4C4C4
DCDCDC4E4E4E696969D6D6D6252525838383DBDBDB7676768F8F8F4F4F4FD5D5D54E4E4E
4F4F4FD9D9D9DEDEDEDEDEDE7E7E7E515151CDCDCDDEDEDE6C6C6C696969C2C2C2363636
D9D9D9C3C3C3929292D7D7D72E2E2EC4C4C46F6F6F727272D1D1D19494941A1A1AC5C5C5
DEDEDEDEDEDED7D7D72525258F8F8FDEDEDE404040A4A4A4777777515151CDCDCDDEDEDE
C3C3C3464646D9D9D9525252A5A5A5CDCDCD7E7E7E383838CCCCCC6B6B6B696969D5D5D5
D7D7D7A7A7A7C3C3C3464646B7B7B7363636D9D9D9C3C3C3929292D7D7D73B3B3BCDCDCD
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE
DEDEDEAAAAAAC3C3C3BBBBBBDDDDDD727272727272727272727272272727BBBBBBBBBBBB
BBBBBBBBBBBBBBBBBB9C9C9CDEDEDEDEDEDE9C9C9CBBBBBBDDDDDD272727BBBBBBBBBBBB
BBBBBBBBBBBBBBBBBBDDDDDD272727BBBBBBBBBBBBC3C3C3AAAAAADEDEDE666666787878
555555FFFFFFFFFFFFBEBEBE7D7D7D7D7D7D7D7D7DFFFFFFFFFFFF303030797979777777
9A9A9ADEDEDEDEDEDED9D9D90000009C9C9CA6A6A6B6B6B6C5C5C5D3D3D3E1E1E1E8E8E8
F0F0F0F6F6F6FAFAFAFCFCFCFDFDFDFEFEFEFFFFFF
000000D6D6D6D5D5D5D5D5D5B0B0B0999999D1D1D1B5B5B5434343ABABABB2B2B28F8F8F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000000D3D3D3CECECECFCFCFCECECE878787B3B3B3DBDBDBADADADB8B8B8A9A9A99B9B9B
919191BDBDBDEFEFEFA5A5A5CECECECECECECCCCCC4949495A5A5A7272729E9E9E676767
151515B9B9B9A0A0A0101010101010151515B5B5B52323237C7C7CCECECE757575515151
C7C7C7A9A9A9151515949494151515B7B7B7C7C7C72323238181811A1A1A9C9C9CA0A0A0
4F4F4FCACACAA0A0A04F4F4FCDCDCD585858101010121212A4A4A4C7C7C72D2D2DC1C1C1
CCCCCC494949696969A0A0A0101010AAAAAACECECECBCBCBCDCDCD585858101010101010
5C5C5CCECECECECECECECECE757575515151C5C5C5C5C5C56161616969698484845C5C5C
CACACAC7C7C73A3A3AC4C4C42D2D2DC1C1C1676767727272CACACAC1C1C11A1A1A9F9F9F
CECECECECECEC7C7C72323238B8B8BC5C5C53D3D3DC2C2C2757575515151C5C5C5CDCDCD
8484845C5C5CCACACA757575515151C4C4C43A3A3A9E9E9E2525258F8F8F4F4F4FB9B9B9
9E9E9E141414C0C0C0CCCCCC8585855C5C5CCACACAC7C7C73A3A3AC4C4C43A3A3AC5C5C5
CDCDCDCDCDCDCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCFCF
CFCFCFCFCFCFCECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECE
CECECEA2A2A2CECECECBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBA2A2A2CECECECECECEA2A2A2CBCBCBE5E5E5272727CBCBCBCBCBCB
CBCBCBCBCBCBCBCBCBE5E5E5272727CBCBCBCBCBCBCECECEA2A2A2CECECE696969828282
848484858585858585939393FFFFFFFFFFFFFFFFFF5656568585858484848383837D7D7D
929292CECECECECECED3D3D3000000AAAAAAB3B3B3BFBFBFCCCCCCD9D9D9E4E4E4EBEBEB
F2F2F2F7F7F7FAFAFAFDFDFDFDFDFDFEFEFEFFFFFF
000000CFCFCFC7C7C7C7C7C7B1B1B1959595A6A6A6ACACACAEAEAEACACACA5A5A5747474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000000CDCDCDC1C1C1C1C1C19B9B9B999999CACACAC9C9C97373739E9E9E737373575757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000000CBCBCBBABABABBBBBBACACAC959595AAAAAAC3C3C3A2A2A2A2A2A29F9F9F626262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000000C9C9C9B8B8B8B8B8B89F9F9F9E9E9EE2E2E2F2F2F2CECECECFCFCFD2D2D2B7B7B7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000000C9C9C9B8B8B8B8B8B8B8B8B8868686C3C3C3F3F3F3A0A0A0EFEFEFDBDBDBC9C9C9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000000C9C9C9B5B5B5B3B3B3A3A3A3969696C9C9C9C7C7C7CBCBCBD0D0D0D1D1D1B6B6B6
D3D3D3D3D3D3D4D4D4A8A8A8B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3B3
B3B3B3949494E0E0E0EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDABABABB3B3B3B3B3B3ABABABEDEDEDEDEDEDEDEDEDEDEDEDEDEDED
EDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDEDE0E0E0949494B3B3B37A7A7AA3A3A3
B0B0B0BEBEBEC1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1C1BEBEBEBABABAA4A4A49F9F9F
848484B3B3B3B5B5B5C9C9C9000000F0F0F0F2F2F2F3F3F3F6F6F6F8F8F8FAFAFAFBFBFB
FCFCFCFDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFF
000000C3C3C3BBBBBBAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAEAE
AEAEAE9D9D9D767676CECECEFDFDFDFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFDFDFDCECECE818181AEAEAEAEAEAE818181CECECEFDFDFDFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFDFDFDCECECE7676769D9D9DAEAEAE777777B5B5B5
D5D5D5E8E8E8E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E7E8E8E8DFDFDFC0C0C0565656
A3A3A3AEAEAEBBBBBBC3C3C3000000F6F6F6F7F7F7F8F8F8F9F9F9FBFBFBFCFCFCFDFDFD
FDFDFDFDFDFDFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
000000B6B6B6D1D1D1B2B2B2AAAAAAA8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8A8
A8A8A8A8A8A89B9B9B909090909090909090909090909090909090909090909090909090
909090909090909090A2A2A2A8A8A8A8A8A8A2A2A2909090909090909090909090909090
9090909090909090909090909090909090909090909B9B9BA8A8A8A8A8A8A2A2A2808080
8080808080808080808080808080808080808080808080808080808080808080809B9B9B
AAAAAAB2B2B2D1D1D1B6B6B6000000FAFAFAFAFAFAFBFBFBFCFCFCFCFCFCFDFDFDFDFDFD
FDFDFDFEFEFEFEFEFEFFFFFFFFFFFFFFFFFFFFFFFF
0000008A8A8AEAEAEAE4E4E4CFCFCFBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
BEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBEBE
CFCFCFE4E4E4EAEAEA8A8A8A000000FCFCFCFDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFE
FEFEFEFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000B4B4B49C9C9CE7E7E7F5F5F5F1F1F1ECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECEC
ECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECECF1F1F1
F5F5F5E7E7E79C9C9CB4B4B4000000FDFDFDFDFDFDFDFDFDFDFDFDFDFDFDFEFEFEFEFEFE
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000FFFFFFB4B4B4696969909090AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
909090696969B4B4B4FFFFFF000000FEFEFEFEFEFEFEFEFEFEFEFEFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000000000000000000000000000000000000000000000
000000000000000000000000000000FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF
pop
P
% End of image
% End of Graphics
MathPictureEnd
\
\>"], "Picture",
  ImageSize->{391, 251},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000670000nb000`400?l00000o`00003oo`3ooon8
0?ooo`002P3ooooo0?knoW@0o_kn2P3oool000L0oooo0P3no_ko0?gmoGH0oOgm0P3no_h70?ooo`00
1P3oool20?knoPD0oOgmo`3lo?a^0?clo0D0oOgm0P3no_h60?ooo`00103oool20?knoP<0oOgm0P3l
o?`00`3kno/0n_[j0?[jnP3o0?[jnVh0n_[j00<0no_k0?clo03lo?`00`3moOd20?knoP@0oooo0004
0?ooo`090?knoP3moOd0oOgm0?gmo@3lo?`0no_k0?Win@3hn?P0moOg0?l0m_KfKP3fm_H01@3gmoL0
n?Sh0?Win@3kno/0o?cl00<0oOgm00<0o_kn0?ooo`3oool00P3oool000<0oooo00X0o_kn0?gmo@3m
oOd0o?cl0?_kn`3jn_X0n?Sh0?KfmP3clo<0l_;bo`3`l?1^0?3`l0070?;blP3clo<0m_Kf0?Shn03j
n_X0no_k0?clo0020?gmo@030?knoP3oool0oooo0040oooo00020?ooo`80o_kn0P3moOd01`3kno/0
nOWi0?Ogm`3emOD0lO7a0>k^kP3[jn/0o`3Xj>Q^0>SXj0070>_[j`3^k^h0lO7a0?Gem@3gmoL0nOWi
0?_kn`020?gmo@80o_kn0P3oool00080oooo00/0o_kn0?gmo@3moOd0o?cl0?[jnP3gmoL0mOGe0?7a
l@3]kNd0j>SX0>CTi03o0>7QhFh0hN7Q00P0i>CT0>SXj03]kNd0lO7a0?Gem@3gmoL0n_[j0?clo080
oOgm0@3no_h10?ooo`40oooo00020?ooo`80oOgm00X0o?cl0?_kn`3hn?P0mOGe0?7al@3[jn/0iNGU
0=kNgP3IfMT0eMGEo`3Cdm=/0=?Cd`0:0=GEe@3IfMT0g]kN0>GUi@3[jn/0lO7a0?Gem@3hn?P0no_k
0?clo080oOgm0P3oool0000>0?ooo`3no_h0oOgm0?gmo@3lo?`0nOWi0?KfmP3alO40kNg]0>GUi@3L
g=`0e=CD0<c<c037alOo0<G5aF`0aLG500X0alO70<c<c03De=@0g=cL0>GUi@3]kNd0lO7a0?KfmP3i
nOT0o?cl0P3moOd10?knoP40oooo00003P3oool0o_kn0?gmo@3moOd0no_k0?Shn03clo<0k^k^0>SX
j03Ng]h0e=CD0<S8b02o_kl0^;Rho`2f][I/0;Jf]P0:0;Rh^02o_kl0b<S80=CDe03Ng]h0j>SX0>k^
kP3clo<0n?Sh0?_kn`80oOgm0@3no_h10?ooo`0000h0oooo0?knoP3moOd0oOgm0?[jnP3gmoL0l_;b
0>_[j`3Ti>@0fMWI0<c<c02o_kl0/k>c0:ZZZ_l0YZJVK02VYZH02P2ZZZX0/k>c0;no_`3<c<`0fMWI
0>CTi03[jn/0l_;b0?Ogm`3jn_X20?gmo@40o_kn0@3oool0000>0?ooo`3no_h0oOgm0?clo03jn_X0
m_Kf0?3`l03Xj>P0hN7Q0=GEe@37alL0^;Rh0:ZZZP2OWioo09bLW6`0W9bL00d0WinO0:ZZZP2h^;P0
alO70=GEe@3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo003o000007X0000000d0
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
0`000000o?cl0?clo03o0?clo7H0o?cl00h0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`
l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl09ZJVP3o08Z:RW@0RXZ:0100VYZJ
0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn
0@3oool000030000003lo?`0RXZ:06X0oooo00<0kno_0>3Ph03Ph>00903Ph>000`2m_Kd0WinO09nO
W`0709nOW`0309JFUP3Vi^H0dm?C06/0WinO00<0_Kfm0>WYj@3WinL0C`3WinL00`3^k^h0UYJF09nO
W`0709nOW`0309JFUP3Vi^H0h>3P00/0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000040000003lo?`0RXZ:09jNWVT0
h>3P00<0kno_0>3Ph03Ph>00903Ph>000`32`/80e=CD0=CDe0070=CDe0030>7Qh@2FUYH0`L7106/0
cLg=00<0kNg]0<_;b`3kno/0C`3kno/00`2FUYH0hN7Q0=CDe0070=CDe0030>7Qh@2FUYH0h>3P0080
h>3P01T0e=CD0=WIf@3_knl0bLW90=WIf@3_knl0bLW90>c/k03Ph>00RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000
o?cl08Z:RP2NWYiY0>3Ph0030>o_k`3Ph>00h>3P02@0h>3P00<0b/[:0<k>cP3>c/h01`3>c/h00`3V
i^H0UYJF0<[:bP1[0<o?c`030?ooo`38b<P0no_k04l0no_k00<0UYJF0>KViP3>c/h01`3>c/h00`3V
i^H0UYJF0>3Ph0020>3Ph0030<W9b@2c/k<0h>3P0080/k>c01@0h>3P0;>c/`39bLT0h>3P08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40
oooo000010000000o?cl08Z:RP2NWYiY0>3Ph00=0>o_k`3Ph>00h>3P0>3Ph02g]kL0@d=306a/K02_
[jl0G5aL05iNGP2CTi<0h>3P07inOP02071`L080h>3P00L0g]kN07moO`2c/k<0`l?305=CD`1@D500
cLg=00l0h>3P00<0b/[:0<k>cP3>c/h00P3>c/h00`1RHV80QXJ60<k>cP020<k>cP030>KViP2FUYH0
b/[:01`0dm?C203De=@?0=?Cd`050;>c/`3Cdm<0`/;20;:b/P32`/80:@3Cdm<80=CDe080dm?C00<0
oooo0<O7a`3inOT0C`3inOT01`2FUYH0i^KV0<k>cP3>c/h0QXJ603hn?P2ZZZX00`3>c/h00`3Vi^H0
UYJF0>3Ph0060>3Ph00E0>o_k`3WinL0h>3P0>o_k`3Ph>00RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:
RP2NWYiY0>3Ph0030>o_k`3Ph>00h>3P0080h>3P00<0QXJ60=WIf@1oOgl00P3Bd]801P18B4P0d];B
07alO03Ph>00QXJ60=WIf@80h>3P00H0g=cL08F5Q@2][Jd0VYZJ06UYJ@2o_kl?0>3Ph0040<[:bP3?
cll0clo?0<o?c`80?Shn103?cll00`3WinL0UYJF0<[:bP020=cLg280gMgM3P3Lg=`01P3]kNd0UIFE
0?ooo`3:b/X0UIFE0<[:bPl0g=cL8P3MgMd20=cLg0030?ooo`37alL0nOWi04l0nOWi00<0UYJF0>OW
i`3?cll00P3?cll203hn?P<0clo?00<0inOW09JFUP3Ph>001@3Ph>020;>c/`0D0>o_k`2c/k<0fMWI
0>3Ph02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl
0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0WYjNJ@3Ph>000`3_knl0h>3P0>3Ph0020>3Ph00;
08J6QP3IfMT0Ogmo0=;BdP3Lg=`09RHV0<_;b`1lO7`0h>3P07moO`3Bd]800`3Ph>001@26QXH0d];B
00d=3@0=3@d0fMWI00H0h>3P00<0L71`0000000000000P0000001@1`L700h>3P0>3Ph03Ph>00b/[:
0080e=CD00<0RHV903hn?P29RHT0103De=@00`3YjNT0UYJF0<[:bP020=cLg280gMgM3P3Lg=`01P3]
kNd0UIFE0?ooo`3:b/X0UIFE0<[:bPl0g=cL8P3MgMd20=cLg0030?ooo`37alL0n?Sh04l0n?Sh00<0
UYJF0>WYj@3De=@00`3De=@203hn?P80e=CD00<0jNWY09JFUP3Ph>001@3Ph>020;>c/`0D0>3Ph02c
/k<0bLW90>3Ph02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3j
n_X0o?cl0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0WYjNJ@3Ph>000`3_knl0h>3P0>3Ph002
0>3Ph00A08J6QP3IfMT0Ogmo0=;BdP3GemL0=cLg0<k>cP1lO7`0h>3P08:2PP3EeMD0RHV9071`L00k
>c/0WinO07=cL`3Ng]h02@3Ph>00101`L70000000000001`L7040>3Ph0060<[:bP3Ogml0gmoO06MW
I`0n?Sh0][Jf103Ogml01@3_knl0UYJF0<[:bP3WinL0i^KV00T0inOW6P3Vi^H=0>OWi`060?;blP2E
UID0oooo0<[:bP2EUID0b/[:3`3Vi^H80>OWiaX0i^KV00@0inOW0>KViP3oool0aLG5D03fm_H00`2F
UYH0kno_0=oOg`030=oOg`0606EUI@0n?Sh0^;Rh0=oOg`3_knl0UYJF2P3Ph>004P3_knl0h>3P08Z:
RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?kn
oP40oooo000010000000o?cl08Z:RP2NWYiY0>3Ph00F0>o_k`3Ph>00h>3P0>3Ph02>SXh071`L0=WI
f@2TY:@0WYjN0;Ng]`1DE5@0h>3P08>3P`3Ph>00LG5a0>3Ph020P800h>3P06UYJ@2_[jl0JFUY0;no
_`X0h>3P00<0L71`0>3Ph03Ph>00103Ph>00103:b/X0kNg]0>g]k@3]kNd203hn?P@0kNg]00D0m_Kf
09JFUP3:b/X0j^[Z0>WYj@090>[ZjQX0jNWY3@3Zj^X01P3dm?@0UIFE0?ooo`3:b/X0UIFE0<[:bPl0
jNWY203Zj^XJ0>WYj@040>[ZjP3YjNT0oooo0<G5aE00m?Cd00<0UYJF0?KfmP3]kNd00P3]kNd203hn
?P<0kNg]00<0m_Kf09JFUP3Ph>00203Ph>004`2c/k<0fMWI0>3Ph02:RXX0o?cl0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003l
o?`0RXZ:09jNWVT0h>3P00<0kno_0>3Ph03Ph>000P3Ph>004@2j^[X0g=cL0=kNgP1mOGd0Lg=c0=WI
f@3Ph>00[:b/071`L02WYjL0h>3P0:b/[01`L700MWIf0=cLg03Bd]80Zj^[0140h>3P00<0b/[:0?Sh
n03hn?P00P3hn?P00`1]KFd0WIfM0?Shn0020?Shn0030?_kn`2FUYH0b/[:0380kNg]00H0lO7a0>g]
k@3fm_H0lO7a0>g]k@3alO4c0>g]k@030?ooo`33`l<0l?3`04l0l?3`00L0UYJF0?_kn`3hn?P0n?Sh
09fMW@0n?Sh0b/[:00<0n?Sh00<0no_k09JFUP3Ph>00203Ph>004`39bLT0e=CD0>3Ph02:RXX0o?cl
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3o
ool000040000003lo?`0RXZ:09jNWVT0h>3P00<0kno_0>3Ph03Ph>00903Ph>000`3:b/X0oooo0?oo
o`080?ooo`0309JFUP3:b/X0moOg06X0moOg00<0oooo0<;2`P3_knl0C`3_knl00`2FUYH0oooo0?oo
o`080?ooo`0309JFUP3Ph>00h>3P00X0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000040000003lo?`0RXZ:09jNWVT0
h>3P00<0kno_0>3Ph03Ph>00903Ph>000`2f][H0f][J0=[JfP070=[JfP030<K6aP2h^;P0aLG506/0
fm_K00<0dM7A0=3@d03^k^h0C`3^k^h00`2h^;P0alO70=[JfP070=[JfP030<K6aP2h^;P0h>3P00/0
h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0@3oool000030000003lo?`0RXZ:06X0WYjN00<0kno_0>3Ph03Ph>00903Ph>0<0>c/
k6d0jn_[00<0jNWY0>OWi`3WinL0CP3WinL<0>c/k0`0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0
RXZ:01<0k>c/00@0^;Rh000000000000@D510`3/k>`0303@d=00ZZZZ0>c/k03/k>`0UiNG0;Ng]`3/
k>`0CTi>0;Ng]`3/k>`0/[:b0<;2`P80k>c/00`0UiNG0;Ng]`3/k>`0J6QX0>c/k02NWYh0i^KV05EE
E@2b/[80`/;208N7Q`3MgMd30>c/k0080=kNgP1:BTX0jNWY01`L7038b<P0k>c/07UiN@3GemL30>c/
k00;0:>SX`3MgMd0k>c/0>c/k01FEUH0emOG0>c/k01jNWX0k>c/0=3@d02ZZZX0103/k>`05`1JFUX0
gMgM0>c/k03Vi^H0X:2P0>c/k02SXj<0gMgM0>c/k02=SHd0k>c/0861P@2UYJD0Q8B40<;2`P3/k>`0
N7Qh0>c/k031`L40YJFU0>c/k01iNGT0`/;20080k>c/00D0Vi^K0>c/k03/k>`0UiNG0:FUY@020>c/
k0030=;BdP1HF5P0k>c/00H0k>c/00<0[jn_00`<303MgMd01@3/k>`00`2h^;P0XZ:R0>c/k0070>c/
k003079bLP1dM7@0k>c/00D0k>c/00<0R8R80>c/k03/k>`01P3/k>`01P2MWId0k>c/0>c/k01bLW80
30`<0>CTi0<0k>c/00@0`L710:FUY@3/k>`0WIfM0P3/k>`00`1bLW80M7Ad0>c/k0030>c/k00B09fM
W@3/k>`0SHf=0>GUi@2h^;P0XZ:R0;Rh^02RXZ80d=3@09NGU`2h^;P0F5QH0=kNgP0M7Ad0i>CT0>c/
k00o?cl0gMgM0`3/k>`04`3Bd]80Ph>30=kNgP1:BTX0iNGU08R8R03/k>`0R8R80>c/k02MWId0jNWY
04i>CP1dM7@0jNWY075aL@2UYJD0k>c/0;Rh^023Ph<00P3/k>`00`0o?cl0gMgM0>c/k0060>c/k00:
09jNWP3/k>`0d=3@07]kN`1`L700UiNG0>c/k02_[jl030`<0=gMg@<0k>c/01D0d=3@07]kN`1`L700
BTY:0=[JfP1:BTX0c/k>05QHF03Bd]80F5QH0>c/k02=SHd0iNGU08R8R03/k>`0MWIf0=gMg@3/k>`0
LW9b00`<303Ti>@00`3/k>`0501fMWH0f][J071`L02GUiL0d=3@09NGU`2h^;P0XZ:R0;Rh^01HF5P0
/[:b0<;2`P2KVi/030`<0>?Sh`1iNGT0JFUY0<S8b03/k>`0WIfM0`3/k>`00`1JFUX0i>CT0>c/k002
0>c/k00O08B4Q032`/80k>c/08B4Q032`/80QhN70=gMg@3Ng]h061PH0<;2`P2OWil0`/;2069RHP3M
gMd0Vi^K00`<303Fe]H0PX:20>KViP0D51@0QhN70::RXP3Vi^H0UiNG08R8R03/k>`0[Jf]0;fm_@3G
emL0^;Rh08>3P`020>c/k00308^;R`3YjNT0S8b<01h0k>c/00<0_Kfm09nOW`2OWil01P2OWil04P2F
UYH0i^KV08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3l
o?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP0C0>c/k0040;Rh^000000000000451@@<0k>c/
00`0d=3@05UIF@1fMWH0_kno0=;BdP1]KFd0MWIf05aLG02g]kL0k>c/0;:b/P32`/820>c/k00;0=;B
dP1]KFd0MWIf06m_K`3/k>`0WYjN0>KViP1EEED0/[:b0<;2`P1dM7@00P1fMWH0101iNGT0emOG0=kN
gP1YJFT40>c/k00307UiN@3GemL0k>c/0080k>c/00D0Xj>S0=gMg@3/k>`0g]kN06UYJ@040>c/k004
0=3@d01IFET0MWIf0;no_`80k>c/00X0FUYJ0=gMg@3/k>`0^;Rh0;Ng]`3/k>`0`L710:FUY@3/k>`0
W9bL1P3/k>`01`2EUID0i>CT0<71`@2UYJD0k>c/0<C4a01fMWH00P3/k>`01@2KVi/0k>c/0=KFeP2=
SHd0b<S80080k>c/00<0i>CT051@D03Ogml03P3/k>`00`3Lg=`0TI6A0>c/k00?0>c/k00309JFUP3C
dm<0k>c/00H0k>c/00<0[jn_0;Ng]`3/k>`01P3/k>`01@2WYjL0b<S80>c/k02GUiL0hN7Q00L0k>c/
00l0UiNG0>7Qh@2`/;00I6AT0=CDe01OGel0e=CD07IfMP3Zj^X0UIFE0>WYj@1:BTX0YJFU09FEU@2e
]KD01P3/k>`0503Ti>@0DU9B0>GUi@1fMWH0I6AT08n?S`1dM7@0Shn?0::RXP2<S8`0j>SX0>[ZjP20
P800k>c/0861P@3QhN40k>c/0=;BdP2=SHd0dM7A2@3/k>`01`31`L40ZZZZ0>c/k03Zj^X0N7Qh0=kN
gP1cLg<01`3/k>`04@3Zj^X0N7Qh0=kNgP15ADD0a/K607IfMP2LW9`0BTY:0:BTY01:BTX0XZ:R0;2`
/02c/k<0VIVI07AdM02f][H0OWin00L0k>c/01D0[Zj^07inOP3Ng]h0Lg=c0>[ZjP1]KFd0e=CD05mO
G`3De=@0BTY:0:6QX@1gMgL0jNWY08>3P`2e]KD0d=3@03<c<`3Vi^H0k>c/0:n_[`2g]kL00P3/k>`0
0`1JFUX0i>CT0>c/k0020>c/k00F08N7Q`3MgMd0k>c/0<71`@23Ph<0QhN70=gMg@31`L40KFe]0>c/
k02h^;P0Ph>308f=S@3/k>`0WinO0<;2`P3Vi^H0PX:20>WYj@2<S8`0MGEe0<o?c`80k>c/00L0M7Ad
07IfMP2<S8`0g]kN07=cL`1bLW80b<S80080k>c/00<0YJFU07]kN`1hN7P07P3/k>`00`32`/80e=CD
0=CDe0060=CDe00B0>7Qh@2FUYH0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:01<0k>c/00@0^;Rh0000
00000000@D510`3/k>`07`3@d=00ZZZZ0>c/k02NWYh0k>c/0<k>cP3/k>`0UiNG0;Ng]`3/k>`0[jn_
00L71`2m_Kd0Ph>30>c/k03>c/h0k>c/08f=S@3/k>`0J6QX0=3@d00H61P0WIfM0<;2`P2=SHd0k>c/
0=kNgP1:BTX0k>c/0=kNgP1YJFT0103/k>`01`1bLW80I6AT0;Rh^02g]kL0k>c/0:>SX`3MgMd00P3/
k>`02@1>CTh0b<S80=3@d01]KFd0k>c/0=3@d02ZZZX0k>c/09jNWP020>c/k00:05YJFP3MgMd0k>c/
0>KViP1bLW80k>c/07IfMP3MgMd0k>c/09bLW0@0k>c/00T0d=3@014A4@1dM7@0k>c/0<71`@2UYJD0
k>c/0=3@d01YJFT00P3/k>`01@2KVi/0k>c/0>c/k02GUiL0YJFU00<0k>c/00<0LW9b0<S8b03/k>`0
3P3/k>`00`2;Rh/0k>c/0>c/k00>0>c/k0030:n_[`2UYJD0k>c/00H0k>c/00<0WIfM0>c/k03/k>`0
1P3/k>`01@31`L40YJFU0>c/k031`L40YJFU00L0k>c/00l0`L710:FUY@3?cll0IFEU05YJFP1_Kfl0
DE5A0820P03/k>`0W9bL0=OGe`2a/K40K6a/0<30`01mOGd01`3/k>`04P2GUiL0_[jn0<S8b01UIFD0
PH6103De=@1mOGd0F5QH0<71`@1HF5P0k>c/055AD@3MgMd0O7al08j>SP3/k>`0^;Rh08>3P`/0k>c/
00H0WYjN0>c/k02h^;P0DE5A0=gMg@2LW9`70>c/k00B0;Rh^01ADE40gMgM0=3@d02PX:00b<S805mO
G`2YZJT0@d=30:VYZ@1HF5P0g]kN06UYJ@34a<@0Ogmo08f=S@0M7Ad0i>CT1P3/k>`0502OWil07AdM
0=gMg@2LW9`0emOG09JFUP1JFUX0Kfm_055AD@2YZJT0XZ:R09NGU`2RXZ80UiNG07AdM02GUiL0HF5Q
0;fm_@3/k>`0WIfM0`3/k>`0901JFUX0i>CT0>c/k03/k>`0jNWY02LW9`1HF5P0k>c/071`L02RXZ80
9bLW08>3P`3Ng]h0BTY:0>GUi@1`L700YJFU07IfMP3MgMd0M7Ad0<;2`P3Vi^H0PX:20>WYj@2<S8`0
UiNG0;Ng]`1bLW80YJFU09fMW@3/k>`0NWYj0;Rh^02g]kL0NGUi0<;2`P80k>c/00<0SHf=0>WYj@2<
S8`07P3/k>`00`3:b/X0c/k>0<k>cP060<k>cP0B0>KViP2FUYH0RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0
RXZ:01<0k>c/00@0d];B07IfMP1fMWH0UYJF0`3/k>`0:P3@d=00ZZZZ0>c/k02DU9@0k>c/09:BTP1f
MWH0TY:B0>c/k03YjNT0OWin0<S8b01mOGd0i>CT0>c/k02BTY80MWIf0;no_`3Bd]80QXJ607IfMP2i
^KT0OGem0>CTi03Fe]H0MWIf07MgM`3GemL0k>c/09fMW@0d=3@0MWIf0::RXP3Zj^X0Q8B407UiN@2S
Xj<0O7al0>SXj02_[jl0NGUi0>CTi080k>c/00T0jNWY07emO@1iNGT0`L710>c/k03@d=00ZZZZ0>c/
k02DU9@00P3/k>`05`1JFUX0gMgM0>c/k03/k>`0[Zj^07IfMP2XZ:P0k>c/0=;BdP1>CTh0MWIf08b<
S03/k>`0cLg=0::RXP3/k>`0R8R80>c/k031`L40YJFU0>[ZjP1]KFd0XZ:R0080k>c/00D0Vi^K0>c/
k03/k>`0UiNG0:FUY@020>c/k0030=KFeP1XJ6P0M7Ad00h0k>c/00<0`L7104I6AP35aLD03`3/k>`0
0`2DU9@09BDU0=SHf0060>c/k00309fMW@3/k>`0k>c/00H0k>c/00H0`L710:FUY@3/k>`0[Zj^04m?
C`3Ogml60>c/k00?0:j^[P1?Cdl0ZZZZ07alO03QhN40WIfM0>WYj@2MWId0k>c/09FEU@3QhN40Q8B4
0;^k^`24Q8@0_Kfm00H0k>c/01<0iNGU07inOP2?Shl0Q8B407alO02b/[80]kNg0;Jf]P2g]kL0k>c/
0<71`@1fMWH0Vi^K07IfMP2XZ:P0`L710;no_`2h^;P0Ph>300/0k>c/00L0WYjN0>c/k03/k>`0X:2P
0>SXj02i^KT0i>CT00L0k>c/0140X:2P0>SXj03YjNT0]KFe08B4Q02^[Zh0Q8B40=3@d024Q8@0]kNg
0<g=c@1lO7`0/[:b09:BTP2d];@0TY:B0>[ZjP060>c/k00D0<g=c@2BTY80j>SX0;Vi^@3QhN40P820
0>7Qh@2MWId0jNWY08B4Q03AdM40k>c/09^KV`2c/k<0OGem07UiN@3GemL0ZZZZ0>c/k02MWId30>c/
k00305YJFP3Ti>@0k>c/0080k>c/00D0^KVi0<[:bP1fMWH0UYJF0>c/k0020;Vi^@0H0:RXZ031`L40
MWIf09JFUP3/k>`0gmoO07IfMP1ADE40_kno08B4Q02`/;00[jn_0:RXZ03/k>`0TY:B07alO03Ti>@0
cLg=07IfMP2m_Kd0j^[Z08V9R@2k^k/0fm_K0P3/k>`00`2^[Zh0MWIf0;fm_@0N0>c/k0050<[:bP3>
c/h0c/k>0<k>cP2ZZZX0103>c/h04P3Vi^H0UYJF08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP0I0>c/
k0050>KViP0L71`000000000002RXZ80903/k>`00`1fMWH0gMgM0>c/k0050>c/k0090>KViP0L71`0
00000000002RXZ80k>c/03Ti>@0<30`0gMgM00`0k>c/00D0LW9b0000001TI6@0k>c/05aLG0020000
00090:FUY@3/k>`0k>c/0>c/k00W9bL0Ph>30>c/k02GUiL0Ph>300@0k>c/00<0Rh^;0;Rh^023Ph<0
3@3/k>`0102_[jl0YJFU0;Rh^02g]kL>0>c/k003079bLP2^[Zh0Ph>300D0k>c/00<0SHf=0>c/k03/
k>`01P3/k>`01P31`L40Ph>30>c/k03/k>`0_Kfm0=OGe`L0k>c/00<0_Kfm0=OGe`3/k>`01@3/k>`0
0`1fMWH0gMgM0>c/k00;0>c/k0030;no_`3/k>`0k>c/00`0k>c/00<0UiNG08>3P`3/k>`02P3/k>`0
0`2=SHd0k>c/0>c/k00E0>c/k0030=kNgP0M7Ad0i>CT01d0k>c/00H0SHf=0>c/k03Bd]80000000`<
303Ti>@A0>c/k00303`l?032`/80k>c/0340k>c/00@0b/[:0<k>cP3>c/h0c/k>0P0n?Sh30<k>cP0B
0>KViP2FUYH0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:06d0k>c/00<0g]kN07inOP3Jf]X00`3/k>`0
0`33`l<0MWIf0<S8b00=0>c/k0040>_[j`2MWId0MWIf0=[JfPh0k>c/00<0j^[Z08V9R@38b<P0103/
k>`00`3Bd]80Z:RX0>c/k0070>c/k0030>GUi@1nOWh0b<S802h0k>c/00<0gmoO07UiN@3GemL02`3/
k>`00`2^[Zh0b<S80>c/k00d0>c/k0030=;BdP2XZ:P0k>c/01D0k>c/00<0aLG50=OGe`3/k>`0<@3/
k>`01@3:b/X0dM7A0:b/[00n?Sh0Hf=S0080?Shn01@0[:b/0=7Ad@3Xj>P0UYJF08Z:RP3lo?`00000
09bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo0000
0`000000o?cl08Z:RP3o0>c/k6T0k>c/00@0b/[:0=_Kf`0n?Sh0?Shn0P3Kfm/203hn?P0C0=_Kf`3]
kNd0UYJF08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3l
o?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX0o`3/k>aY0>c/k0030<[:bP3Ti>@0TI6A00@0
i>CT01@0TI6A0>CTi03alO40UYJF08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SX
j03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o0>c/k6T0k>c/00<0
b/[:0?3`l03`l?001P3`l?004P3gmoL0UYJF08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o0>c/k6T0
k>c/00<0b/[:0?clo03lo?`01P3lo?`04P3moOd0UYJF08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5
a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o
0>c/k6T0k>c/00<0`<300?ooo`3oool01P3oool04P3emOD0UYJF08Z:RP3lo?`0000009bLW02VYZH0
][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl
08Z:RP3o0>c/k6T0k>c/0P2f][H70;Fe]@0B0:6QX@3Jf]X0RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:
0?l0k>c/J03/k>`01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000030000003lo?`0RXZ:01d0k>c/00<0Vi^K0000002?Shl08P3/k>`0103Zj^X0CTi>
0000002?Shl`0>c/k00303Ti>@1HF5P0k>c/0>h0k>c/00L0inOW0>k^kP3_knl0l?3`0?Cdm03fm_H0
n?Sh0080nOWi0P3kno/04@3WinL0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP0M0>c/k0030>WYj@0X
:2P0i>CT02<0k>c/00<0i^KV02PX:03Ti>@0<@3/k>`00`2h^;P0b<S80>c/k03]0>c/k0070>OWi`3^
k^h0kno_0?3`l03dm?@0m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3lo?`0000009bLW02VYZH0
][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?cl
o02:RXX06P3/k>`02@3De=@0Q8B407alO03UiND08R8R079bLP1iNGT0emOG0=KFeP0207IfMP0809:B
TP32`/80MWIf07UiN@3QhN40[jn_07UiN@3Ti>@20>c/k0080>[ZjP29RHT0NGUi0>7Qh@2_[jl0MgMg
09JFUP3GemL209:BTP030>[ZjP3/k>`0cLg=0080k>c/00<0cLg=07IfMP2`/;000P3/k>`01P3Vi^H0
8R8R079bLP1iNGT0hN7Q0<71`@80MWIf00@0TY:B092@T02BTY80eMGE0P1fMWH04`2BTY80ZZZZ08b<
S02_[jl0Q8B40;fm_@24Q8@0j>SX0:n_[`1iNGT0i>CT0<g=c@1fMWH0^k^k07IfMP2BTY80eMGE07If
MP2DU9@00`1fMWH01P2RXZ80cLg=07IfMP2`/;00k>c/09:BTP80MWIf00H0f][J0>c/k03[jn/0WIfM
07IfMP38b<P20>c/k0030>[ZjP2MWId0k>c/00<0k>c/00@0Vi^K07IfMP24Q8@0j>SX0P3/k>`0103U
iND0_Kfm0>c/k03Fe]H207IfMP0C09:BTP2@T900TY:B0>SXj02KVi/0TY:B09VIV@1fMWH0Q8B40>OW
i`2MWId0MWIf0<S8b03Fe]H0MWIf0;fm_@2_[jl0MWIf0;2`/0020>c/k0030:n_[`2OWil0k>c/00<0
k>c/00<0cLg=0>c/k03=cLd00P3/k>`0402BTY80XZ:R09^KV`2BTY80VIVI09:BTP3WinL0RHV907Ui
N@33`l<0MWIf0;6a/@1fMWH0TY:B0>[ZjP3/k>`209:BTP0?0>[ZjP2_[jl0MWIf08R8R03GemL0TY:B
0::RXP2KVi/0TY:B0=3@d024Q8@0O7al0>SXj03/k>`0gmoO0080MWIf01l0/k>c07IfMP2m_Kd0k>c/
09^KV`1fMWH0b<S80<g=c@1fMWH0Z:RX07IfMP2XZ:P0MWIf09:BTP3Zj^X0k>c/09:BTP38b<P0_;bl
0>c/k02_[jl0f][J0:n_[`2FUYH0`L7109nOW`2_[jl0b<S80=oOg`1fMWH0alO70080k>c/00H0[jn_
07IfMP3Jf]X0`L7107IfMP38b<P30>c/k00B09:BTP38b<P0_;bl0>c/k02k^k/0k>c/0:n_[`2FUYH0
`L7107MgM`38b<P0NWYj07UiN@3<c<`0MWIf0<S8b02`/;00emOGG`3/k>`01`3WinL0k^k^0>o_k`3`
l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0
dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:01X0
k>c/01<0_Kfm0=OGe`2h^;P0];Bd02<S8`3GemL0i^KV02<S8`2JVYX0YJFU09nOW`32`/80MgMg0>c/
k03Vi^H0YjNW05QHF03YjNT0M7Ad0080k>c/00X0UiNG0::RXP3Vi^H0_[jn02PX:03QhN4071`L00`<
301oOgl0gMgM0P3/k>`01P1EEED0iNGU0>c/k028R8P0k>c/0:f][@80k>c/00@0i^KV02<S8`3GemL0
i^KV0P1bLW80<@37alL0EUIF0<;2`P27QhL0gMgM0:n_[`2UYJD0WinO0<;2`P3Vi^H0PX:20>WYj@2<
S8`0jNWY07UiN@2h^;P0F5QH0>WYj@1dM7@0WinO0<;2`P3YjNT0:2PX0=gMg@3Zj^X0EUIF0=GEe@1F
EUH0emOG0>c/k02a/K40R8R80>c/k02][Jd0^;Rh07AdM03YjNT0CTi>07AdM03/k>`0g]kN04Y:BP3U
iND08b<S0<;2`P3/k>`0^;Rh01`L703UiND00`3/k>`00`2;Rh/0jNWY08b<S0030>c/k00I0:n_[`0i
>CT0iNGU0:n_[`2UYJD0WinO0<;2`P27QhL0gMgM0>c/k02OWil0`/;208^;R`3YjNT0S8b<0=3@d01:
BTX0iNGU02<S8`3EeMD0EUIF0;fm_@1HF5P0k>c/0:f][@020>c/k0030;Rh^02g]kL0k>c/0080k>c/
01@0jNWY05=CD`3/k>`0J6QX0>GUi@3/k>`0QhN70=gMg@2OWil0`/;209nOW`32`/80UiNG0::RXP3V
i^H0_[jn02PX:03Shn<0EUIF0<;2`P80k>c/00l0QhN70=gMg@3YjNT0:2PX0=gMg@21PH4030`<07mo
O`3MgMd0WinO0<;2`P2m_Kd0emOG0;Rh^02g]kL00P3/k>`0402GUiL0Ph>30>WYj@0X:2P0i>CT0=3@
d01]KFd0jNWY05IFEP3De=@0:2PX0>?Sh`1FEUH0eMGE04i>CP2UYJD20>c/k00H09fMW@3/k>`0SHf=
0>GUi@2h^;P0XZ:R0;Rh^02RXZ80d=3@09NGU`2h^;P0F5QH0=kNgP0M7Ad0i>CT0>c/k03Zj^X0EUIF
0<;2`P3Ng]h061PH0=GEe@1FEUH0emOG0P3/k>`03`2MWId0k>c/08f=S@3UiND0R8R80>c/k02h^;P0
XZ:R07UiN@3EeMD0CTi>07AdM03YjNT0LG5a0:FUY@1R0>c/k0070>OWi`3^k^h0kno_0?3`l03dm?@0
m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX06P3/k>`03@3N
g]h0Lg=c079bLP3GemL0:2PX0>CTi03/k>`0UiNG0:ZZZP1iNGT0E5AD0<;2`P2DU9@00P3/k>`01031
`L40DU9B07IfMP2RXZ820>c/k00B07EeM@3?cll0k>c/0>WYj@0X:2P0hn?S05IFEP32`/80QhN70=gM
g@3/k>`0XJ6Q09:BTP2SXj<0j>SX07AdM01fMWH0S8b<0P3/k>`0=`3Vi^H0:2PX0>CTi03/k>`0SHf=
0=;BdP1jNWX0;Rh^0<;2`P27QhL0gMgM0=oOg`1iNGT0E5AD0<;2`P3Vi^H0PX:20>WYj@2<S8`0jNWY
07UiN@2GUiL0DU9B07IfMP2RXZ80WinO0<;2`P3YjNT0:2PX0=gMg@3Zj^X0EUIF0=OGe`3YjNT0Q8B4
0>CTi03/k>`0M7Ad07IfMP2<S8`0UiNG0;Ng]`3YjNT0EUIF0=OGe`3/k>`0g]kN04Y:BP3Xj>P0UiNG
08>3P`3EeMD0@d=30<;2`P2KVi/00`3/k>`00`2UYJD0Ng]k07QhN0030>c/k00I07QhN03Fe]H0Xj>S
0=gMg@1iNGT0E5AD0<;2`P27QhL0gMgM0>c/k02OWil0`/;20:FUY@1kNg/0N7Qh0=3@d01:BTX0j>SX
09NGU`2f][H0EUIF0:b/[01BDU80MWIf08b<S0020>c/k0030=cLg02IVIT0k>c/0080k>c/01@0UIFE
09nOW`2FUYH0UIFE0:6QX@3/k>`0QhN70=gMg@2OWil0`/;209nOW`32`/80MGEe0<o?c`3/k>`0jNWY
02PX:03Shn<0EUIF0<;2`P80k>c/00l0QhN70=gMg@3YjNT0:2PX0=gMg@2OWil0`/;208N7Q`3MgMd0
WinO0<;2`P3Ng]h0Lg=c079bLP3Jf]X00P3/k>`0402GUiL0Ph>30>WYj@0X:2P0i>CT09NGU`2/[:`0
k>c/08B4Q02d];@0:2PX0>?Sh`1FEUH0eMGE04i>CP2UYJD20>c/k00H09NGU`3QhN40/;2`06ATI03D
e=@0GemO0=CDe01fMWH0j^[Z09FEU@3YjNT0BTY:0:FUY@2EUID0]KFe0>c/k03Zj^X0EUIF0<;2`P31
`L40KFe]0>c/k024Q8@0]kNg0P3/k>`03`2GUiL0hN7Q0;2`/01TI6@0Shn?07AdM03De=@0MWIf0:6Q
X@2_[jl0j^[Z0820P03/k>`0PH610>7Qh@1R0>c/k0070>OWi`3^k^h0kno_0?3`l03dm?@0m_Kf0?Sh
n0020?Win@80no_k0140inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`
l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX06P3/k>`04`2h^;P0]kNg
07UiN@26QXH000000::RXP3@d=004A4A09jNWP23Ph<0WinO0<;2`P2=SHd0k>c/079bLP2^[Zh0Ph>3
0:n_[`23Ph<00P3/k>`0C02GUiL0]kNg079bLP35aLD0:2PX0>?Sh`1FEUH0_kno02LW9`23Ph<0jNWY
02PX:03Ti>@0_Kfm0<;2`P2MWId0k>c/07YjNP3/k>`0j^[Z04i>CP000000XZ:R0=3@d01KFe/0UiNG
0;Jf]P1FEUH0_kno02LW9`23Ph<0`L7108>3P`2OWil0_kno04i>CP11@D40i^KV03Xj>P3YjNT0S8b<
0;Rh^023Ph<0[jn_08>3P`2ATI40`/;209^KV`000000Ph>30:n_[`071`L0`L710=gMg@3Ng]h0BTY:
0>GUi@2MWId0k>c/07YjNP3@d=00OGem0=3@d00H61P0emOG0>c/k03Ng]h04A4A0<O7a`1>CTh0YJFU
071`L02UYJD0k>c/09bLW03GemL20>c/k00308f=S@3YjNT0S8b<0080k>c/01X0i^KV08:2PP3/k>`0
_Kfm0:b/[023Ph<0WinO0;no_`0W9bL0Ph>30>c/k00l?3`0`/;208f=S@3YjNT0S8b<0=3@d00A4A40
alO704i>CP37alL0EUIF0;fm_@23Ph<0k>c/07YjNP<0k>c/02T0WYjN0>c/k03/k>`0g]kN04Y:BP3W
inL0EUIF0=OGe`2m_Kd0`/;20861P@1HF5P0Q8B40<;2`P0l?3`0`/;209NGU`2g]kL0LW9b0<G5a@0L
71`0WinO02<S8`32`/80k>c/0>WYj@0W9bL0Ph>30>WYj@0X:2P0gMgM09nOW`2o_kl09bLW05QHF024
Q8@0`/;20;Rh^02g]kL0NGUi0=OGe`020>c/k00@09NGU`23Ph<0Vi^K0000002g]kL0d=3@09NGU`3@
d=004A4A07QhN0000000];Bd01`L702PX:00EUIF0<;2`P80k>c/01P0`L710:FUY@3?cll0IFEU05YJ
FP1_Kfl0DE5A0820P03/k>`0W9bL0=OGe`2a/K40K6a/0<30`01mOGd0k>c/0=cLg01FEUH0`/;20=kN
gP1:BTX0f][J014A4@38b<P20>c/k00?0<71`@2UYJD0clo?06EUI@21PH40=CDe055AD@20P800k>c/
07MgM`3/k>`0DE5A0=gMg@1lO7`0SXj>0680k>c/00L0inOW0>k^kP3_knl0l?3`0?Cdm03fm_H0n?Sh
0080nOWi0P3kno/04@3WinL0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP0J0>c/k00C0>[ZjP29RHT0
^k^k0>;RhP2:RXX0X:2P07IfMP2g]kL0iNGU07inOP24Q8@0j>SX0<g=c@1fMWH0O7al0>SXj02_[jl0
Q8B40>SXj0030>c/k00709:BTP1lO7`0/[:b07alO039bLT0NGUi0=OGe`020;Vi^@80MWIf00H0a/K6
09fMW@1lO7`0bLW907IfMP2m_Kd20>c/k0090>WYj@2:RXX0X:2P07IfMP3AdM40g]kN07YjNP2BTY80
j>SX0080^KVi01H0XJ6Q07inOP24Q8@0j>SX0=kNgP2/[:`0MWIf0;2`/01fMWH0_Kfm0>c/k02_[jl0
Q8B40>SXj027QhL0HF5Q0=SHf02:RXX0^KVi0:JVYP27QhL0e=CD0`1fMWH0502BTY80bl_;07IfMP2m
_Kd0k>c/0:BTY01dM7@0;Rh^0=OGe`3/k>`0g]kN04Y:BP2>SXh0XZ:R0>WYj@1mOGd0O7al0>7Qh@1n
OWh0/;2`0P3/k>`03@2^[Zh0MWIf0;fm_@3/k>`0jn_[09fMW@2<S8`0j^[Z09fMW@2YZJT0OWin08B4
Q03Xj>P00P2i^KT0401iNGT0[Jf]0=OGe`2^[Zh0MWIf0;fm_@3@d=00BTY:08j>SP2RXZ80j^[Z05IF
EP3GemL0[jn_07IfMP2m_Kd20>c/k00I0=kNgP1WIfL0dm?C0>c/k02f][H0MWIf0;:b/P1fMWH0_;bl
09fMW@2BTY80QHF50<?3``24Q8@0j>SX0;Rh^03GemL0k>c/09:BTP1lO7`0iNGU02PX:02LW9`0TY:B
0>SXj0020>c/k080^KVi01l0MWIf07alO03Ti>@0U9BD0=OGe`2i^KT0b/[:08B4Q03Xj>P0j^[Z08V9
R@2k^k/0hn?S0>c/k03Fe]H0Bd];0451@@2k^k/0RXZ:09VIV@3GemL0Vi^K07IfMP2g]kL0iNGU09^K
V`2IVIT0TY:B0::RXP2BTY80j>SX0080k>c/01L0[Zj^04m?C`2ZZZX0O7al0>7Qh@2MWId0jNWY09fM
W@3/k>`0UIFE0>7Qh@24Q8@0^k^k08B4Q02m_Kd0k>c/0=oOg`0l?3`0HF5Q0=[JfP31`L40MWIf0;Ng
]`030>c/k00A0:j^[P1?Cdl0ZZZZ07alO02b/[80]kNg0>WYj@2MWId0k>c/0=KFeP1fMWH0Vi^K07If
MP2XZ:P0`L7109:BTP3Zj^X0H03/k>`01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT2
0?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3j
n_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:05`0k>c/00<0a<C40<;2`P3/k>`03P3/
k>`01`1bLW801`L70=OGe`3/k>`0PH6103Ti>@3Xj>P07@3/k>`0203YjNT071`L03Ti>@3Xj>P0k>c/
0:n_[`071`L0emOG1P3/k>`00`3De=@0`/;20>c/k0060>c/k0030>WYj@0];Bd0gMgM00L0k>c/00<0
Vi^K00`<303Ti>@05@3/k>`00`2h^;P0Ph>30>c/k00@0>c/k0030;fm_@3GemL0k>c/00D0k>c/00<0
MWIf0=gMg@3/k>`01@3/k>`00`2LW9`0`/;20>c/k0070>c/k0030;fm_@3GemL0k>c/06d0k>c/00L0
inOW0>k^kP3_knl0l?3`0?Cdm03fm_H0n?Sh0080nOWi0P3kno/04@3WinL0RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`00
0000o?cl08Z:RP1Q0>c/k0030>WYj@0S8b<0emOG00T0k>c/00L0jNWY05IFEP3GemL0k>c/0=kNgP1:
BTX0j>SX01T0k>c/00<0Q8B40<;2`P3/k>`00P3/k>`01`3@d=00BTY:0>SXj03/k>`0jNWY02<S8`3G
emL0403/k>`01P1fMWH0gMgM0>c/k03/k>`0Q8B40<;2`P<0k>c/00<0g]kN01dM7@3Ti>@05P3/k>`0
0`1bLW800`<30<;2`P2I0>c/k0070>OWi`3^k^h0kno_0?3`l03dm?@0m_Kf0?Shn0020?Win@80no_k
0140inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3l
o?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX0o`3/k>aX0>c/k0070>OWi`3^k^h0kno_0?3`
l03dm?@0m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX0o`3/
k>aX0>c/k0070>OWi`3^k^h0kno_0?3`l03dm?@0m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP01
0?ooo`0000<000000?clo02:RXX0QP3/k>`00`2a/K40k>c/0>c/k03N0>c/k0070>OWi`3^k^h0kno_
0?3`l03dm?@0m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5
a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX0
QP3/k>`00`1fMWH000000>c/k03N0>c/k0070>OWi`3^k^h0kno_0?3`l03dm?@0m_Kf0?Shn0020?Wi
n@80no_k0140inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf
0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX0K`3/k>`00`3UiND0OWin0<O7a`04
0>c/k00307IfMP000000k>c/00L0k>c/00<0g]kN092@T03/k>`01@3/k>`00`0k>c/0k>c/0>c/k006
0>c/k0030>GUi@1nOWh0XZ:R00@0k>c/00<0>c/k07IfMP000000AP3/k>`00`33`l<0MWIf0;fm_@23
0>c/k0070>OWi`3^k^h0kno_0?3`l03dm?@0m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3lo?`0
000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?oo
o`0000<000000?clo02:RXX0JP3/k>`0200W9bL0Ph>30>c/k03Vi^H071`L0;Zj^P1YJFT0SHf=103/
k>`01@1fMWH0k>c/0>c/k03/k>`0R8R800D0k>c/00<0d=3@0820P023Ph<01@3/k>`01@1fMWH00000
0>c/k02h^;P0F5QH00@0k>c/00@0g]kN06UYJ@2GUiL0]kNg103/k>`020000000k>c/0>c/k02h^;P0
M7Ad0>c/k03Bd]80F5QH1@3/k>`00`3YjNT071`L0:FUY@020>c/k0030=;BdP1HF5P0k>c/01<0k>c/
00<0^;Rh0::RXP3/k>`04`3/k>`01`3Bd]8030`<0=gMg@3/k>`0^;Rh00H61P3MgMd00`3/k>`00`3Y
jNT0:2PX0>CTi0230>c/k0070>OWi`3^k^h0kno_0?3`l03dm?@0m_Kf0?Shn0020?Win@80no_k0140
inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0
oOgm0?knoP010?ooo`0000<000000?clo02:RXX0JP3/k>`01@2KVi/0k>c/0>c/k02GUiL0Ph>30080
k>c/00<0QhN70=gMg@3/k>`00P3/k>`01P1fMWH000000>c/k03/k>`0M7Ad0<;2`PD0k>c/00<0jNWY
02PX:03Ti>@0103/k>`01P1fMWH000000>c/k03@d=0061PH0=OGe`D0k>c/00<0i^KV08:2PP3/k>`0
0P3/k>`00`1fMWH0k>c/000000020>c/k00508f=S@3/k>`0g]kN01PH6032`/80103/k>`00`31`L40
Ph>30>c/k0020>c/k0030=kNgP0H61P0emOG01<0k>c/00<0d=3@06e]K@3/k>`0503/k>`01P2GUiL0
Ph>30>c/k03/k>`0Xj>S0=gMg@<0k>c/00<0jNWY02PX:03Ti>@0P`3/k>`01`3WinL0k^k^0>o_k`3`
l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0
dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:01<0
k>c/00@0d];B07IfMP1fMWH0UYJF0P3/k>`0C`3YjNT0MWIf05EEE@2RXZ80k>c/0<71`@1_Kfl0MWIf
04M7A`24Q8@0g]kN05eMG@1QHF40UYJF0>c/k031`L40Kfm_07IfMP14A4@0[:b/051@D03AdM40>3Ph
08^;R`1TI6@0XZ:R06i^KP1fMWH0O7al0>CTi03UiND0GUiN07AdM01?Cdl0dm?C07inOP0l?3`0Jf][
0;Ng]`3Zj^X0Q8B4055AD@1^KVh0NGUi0>7Qh@2CTi<0Kfm_07IfMP2>SXh0jNWY07IfMP1EEED0XZ:R
0>c/k03YjNT0OGem02d];@1^KVh0Z:RX0>WYj@1eMGD0MWIf079bLP3Ti>@0k>c/09jNWP1fMWH0HF5Q
0<S8b02a/K40JFUY07IfMP1^KVh0k>c/08j>SP1BDU80MWIf06QXJ03GemL00P3/k>`01@2KVi/0k>c/
0>c/k02GUiL0YJFU0080k>c/00<0`<3007inOP3/k>`00P3/k>`01`1fMWH0k>c/0;6a/@3/k>`0V9RH
038b<P3Ti>@01@3/k>`00`2EUID0]KFe0>c/k0020>c/k00707IfMP2a/K40MWIf0>c/k03YjNT0AdM7
08^;R`060>c/k00508V9R@3/k>`0k>c/0>c/k02a/K400P1fMWH0202a/K40k>c/09fMW@3/k>`0j^[Z
04M7A`0b<S80j>SX0`3/k>`02031`L40YJFU0>c/k01[Jf/0b<S80>[ZjP17AdL0Rh^;103/k>`04P1[
Jf/0b<S808f=S@3Xj>P0UIFE0;Fe]@2EUID0LG5a0;>c/`1dM7@0UIFE07inOP3Hf=P03Ph>0<?3``3Y
jNT0?3`l0>CTi0<0k>c/01<0`<300451@@2n_[h0IVIV0>SXj029RHT0k>c/061PH02g]kL0M7Ad07Qh
N01iNGT0?Shn07QhN02>SXh0DU9B0=[JfP2h^;P0Ph>30080k>c/00<0Xj>S0=gMg@3/k>`00P3/k>`0
7`3YjNT07alO06][J`1fMWH0Y:BT0:b/[00D51@0^KVi0=kNgP1LG5`0M7Ad0:ZZZP3Bd]80?clo075a
L@0W9bL0W9bL04e=C@1bLW80;Rh^06MWI`25QHD0MWIf096AT@3Bd]80KFe]07UiN@2XZ:P0WIfM07If
MP28R8P00P3/k>`05`24Q8@0Hf=S02/[:`1mOGd0j^[Z0>c/k02WYjL0HF5Q0;Zj^P0D51@0VYZJ07Ee
M@1mOGd0GEeM08^;R`3MgMd0IFEU07IfMP2@T900d];B06e]K@1iNGT0b/[:0080k>c/02T0gmoO03lo
?`1aLG409bLW08R8R00[:b/0OGem05eMG@2JVYX0K6a/07emO@1aLG40Kfm_0<k>cP1]KFd0NGUi0:ZZ
ZP0D51@0[Jf]02/[:`1mOGd0Vi^K07IfMP28R8P0`<3005ADE03Jf]X0EeMG06YZJP1mOGd0`L7101@D
502][Jd0:b/[07emO@1YJFT0M7Ad05ADE02l_;`0E5AD0>SXj00S0>c/k0070>OWi`3^k^h0kno_0?3`
l03dm?@0m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX04`3/
k>`0102h^;P0000000000011@D430>c/k00<0=3@d02ZZZX0k>c/0>c/k02h^;P0XZ:R0>c/k02h^;P0
]kNg0>c/k02b/[80`/;20P3/k>`02`2h^;P0XZ:R0>c/k02MWId0k>c/09jNWP3Vi^H0EEEE0;:b/P32
`/80WIfM00@0k>c/00<0g]kN06UYJ@3/k>`00`3/k>`00`1iNGT0emOG0>c/k0020>c/k0050:>SX`3M
gMd0k>c/0=kNgP1YJFT0103/k>`00`3@d=00ZZZZ0>c/k0030>c/k00:05YJFP3MgMd0k>c/0;Rh^02g
]kL0k>c/0<71`@2UYJD0k>c/09bLW0H0k>c/00L0Xj>S0=gMg@31`L40YJFU0>c/k02h^;P0Ph>30080
k>c/00D0Vi^K0>c/k03/k>`0UiNG0:FUY@020>c/k0030=kNgP1:BTX0j>SX00h0k>c/00<0d=3@09NG
U`3/k>`03`3/k>`00`2=SHd0iNGU0>c/k0060>c/k00309fMW@3/k>`0k>c/00H0k>c/00D0`L710:FU
Y@3/k>`0SHf=0>GUi@070>c/k00?08f=S@3UiND0WinO0<;2`P3Vi^H0HV9R0>KViP1bLW80jNWY08b<
S03Vi^H08b<S0<O7a`1bLW80b<S800H0k>c/01<0g]kN04Y:BP3WinL0EUIF0<;2`P24Q8@0XZ:R08B4
Q032`/80Rh^;0>c/k03YjNT0NGUi0>c/k01fMWH0gMgM0>c/k02h^;P0Ph>30080k>c/00<0Xj>S0=gM
g@3/k>`00P3/k>`05P3YjNT0:2PX0>CTi03/k>`0SHf=0>KViP0X:2P0i>CT09NGU`2RXZ80k>c/08f=
S@2GUiL0Ph>30>[ZjP1FEUH0e=CD02PX:03Shn<0EUIF0<;2`P2;Rh/20>c/k00309NGU`38b<P0k>c/
0080k>c/00P0UiNG059BDP3/k>`0k>c/0;Rh^01cLg<0EUIF0<;2`P80k>c/00/0WinO0<;2`P3YjNT0
:2PX0=gMg@2OWil0`/;208N7Q`3MgMd0`L710:FUY@020>c/k00409NGU`38b<P0k>c/0<71`@800000
02P0SHf=08>3P`3Zj^X0EUIF0=GEe@1FEUH0`/;208N7Q`3MgMd0WinO0<;2`P2KVi/0YJFU09NGU`38
b<P0k>c/0>WYj@0X:2P0hn?S05IFEP32`/80k>c/09NGU`1BDU80jNWY08b<S02GUiL0F5QH0>c/k02G
UiL0];Bd02PX:03Shn<0EUIF0<;2`P1RHV80f][J08b<S03YjNT0S8b<903/k>`01`3WinL0k^k^0>o_
k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:
01<0k>c/00@0^;Rh000000000000@D510`3/k>`07`3@d=00FEUI07IfMP2HV9P0k>c/08>3P`1fMWH0
Bd];0;Ng]`3/k>`0/;2`06ATI03De=@0]kNg0>c/k023Ph<0MWIf06MWI`3/k>`0Ph>30=_Kf`0f=SH0
YjNW0<;2`P1/K6`0MWIf06m_K`0X:2P0emOG0=kNgP1YJFT0103/k>`04P1eMGD0WIfM0=;BdP3AdM40
k>c/0:>SX`3MgMd0k>c/0>GUi@1KFe/0f][J0=kNgP2/[:`0k>c/0=3@d01IFET0MWIf09RHV080k>c/
00X0FUYJ0=gMg@3/k>`0clo?09BDU03/k>`0Vi^K0<71`@3/k>`0W9bL103/k>`02@3Ng]h0OWin07in
OP3/k>`0`L710:FUY@3/k>`0d=3@06UYJ@020>c/k00509^KV`3/k>`0e]KF08f=S@38b<P00P3/k>`0
0`3[jn/0I6AT0<o?c`0>0>c/k0030>[ZjP2;Rh/0k>c/00l0k>c/00<0YjNW0;>c/`3/k>`01P3/k>`0
0`2_[jl0]kNg0>c/k0060>c/k0050:NWY`38b<P0k>c/0;6a/@31`L401`3/k>`03`2a/K40`L710<S8
b00f=SH0SXj>06EUI@29RHT0OGem0>c/k02MWId0hN7Q096AT@1hN7P0_;bl08n?S`060>c/k00G0>_[
j`1hN7P0dM7A0:n_[`0f=SH0SHf=03dm?@2;Rh/0KFe]0:NWY`2NWYh0k>c/06a/K03Ti>@0Q8B40;Vi
^@3/k>`0d];B08f=S@3AdM40k>c/0:>SX`3MgMd00`3/k>`07`32`/8051@D0<?3``3Ng]h0Lg=c09ZJ
VP0D51@0cLg=0;>c/`2WYjL0g]kN06YZJP2c/k<0D51@0>3Ph00g=cL0e=CD02PX:03Shn<0EUIF0<;2
`P2EUID0k>c/0;^k^`2WYjL0Fe]K03Ti>@1]KFd0QXJ609:BTP32`/800P3/k>`01031`L40Vi^K02h^
;P32`/820>c/k00A09RHV032`/80jNWY028R8P2JVYX0TI6A0<30`01GEeL0/;2`0<W9b@11@D40MWIf
05=CD`2WYjL0Fe]K03Ti>@2/[:`00P3/k>`0:03?cll0D51@0>3Ph00g=cL0]kNg02h^;P30`<00EeMG
0;2`/02@T900SHf=09fMW@2c/k<0YjNW05]KF`0i>CT0PX:201@D5031`L40?3`l0<;2`P26QXH0TY:B
0<;2`P2][Jd0N7Qh0;>c/`1VIVH0hn?S0451@@2FUYH051@D0<71`@0l?3`0`/;20;^k^`22PX80N7Qh
0>WYj@2<S8`T0>c/k0070>OWi`3^k^h0kno_0?3`l03dm?@0m_Kf0?Shn0020?Win@80no_k0140inOW
08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm
0?knoP010?ooo`0000<000000?clo02:RXX04`3/k>`0102h^;P0000000000011@D430>c/k00N0=3@
d02ZZZX0k>c/09fMW@3/k>`0>CTi0000000i>CT0k>c/0>KViP0A4A40YJFU00h>3P3MgMd0k>c/03Ti
>@000000TY:B0;Rh^00P8200000008J6QP0>3Ph0gMgM0<71`@0000000`<30<;2`P3/k>`0G5aL0P00
00002P1HF5P0jNWY01`L70071`L0Fe]K00`<303Ti>@0LW9b00H61P3MgMd20>c/k0090>KViP0>3Ph0
1`L709NGU`3/k>`0d=3@0:ZZZP3/k>`0WIfM0080k>c/00T0FUYJ0=gMg@3/k>`0k>c/071`L0000000
I6AT0>c/k02h^;P00P0000000`0/;2`0k>c/0>c/k0020>c/k00708R8R03/k>`0`L710:FUY@3/k>`0
^;Rh08>3P`020>c/k00509^KV`3/k>`0k>c/09NGU`2UYJD00P3/k>`00`31`L400000059BDP0>0>c/
k00309NGU`000000YJFU00l0k>c/00<0Bd];00000038b<P01P3/k>`00`2MWId0k>c/0>c/k0060>c/
k0060<71`@2UYJD0k>c/071`L0071`L0emOG1P3/k>`03`1`L7001`L706a/K00<30`0e]KF04m?C`3V
i^H0Cdm?0>c/k00o?cl0e]KF01`L702;Rh/071`L08n?S`060>c/k00C0=kNgP0A4A40HF5Q01`L700<
30`0NGUi08>3P`21PH40Ph>30>c/k02GUiL0000004];B`000000I6AT09NGU`2BTY80^;Rh08>3P`02
0>c/k0030:>SX`3MgMd0k>c/0080k>c/01T0jNWY02PX:01DE5@000000;Ng]`3Vi^H0:2PX04M7A`32
`/80Bd];0000002g]kL0k>c/0;Rh^00000001`L706a/K00<30`0YjNW00L71`32`/80[jn_0000001d
M7@0k>c/0080>CTi00@0iNGU09^KV`000000ZZZZ0P3/k>`0103@d=0020P803Ti>@3UiND20>c/k007
05aLG0000000aLG502PX:02YZJT071`L0>GUi@0208N7Q`0506ATI01bLW80000008n?S`3/k>`00P0i
>CT04@3Xj>P0k>c/0>c/k03/k>`0^;Rh000000071`L0e=CD02<S8`32`/80SHf=08n?S`0L71`0/K6a
01`L703UiND0k>c/0080>CTi01P0iNGU02PX:01DE5@0>CTi0>GUi@2KVi/000000:ZZZP3Vi^H0LW9b
0>c/k02GUiL0000008>3P`3YjNT0:2PX05ADE00i>CT0iNGU071`L0000000Shn?0>WYj@2<S8`T0>c/
k0070>OWi`3^k^h0kno_0?3`l03dm?@0m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3lo?`00000
09bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`00
00<000000?clo02:RXX06@3/k>`01@3YjNT0MWIf05EEE@1fMWH0UYJF01T0k>c/00<0iNGU0:ZZZP3/
k>`0203/k>`00`2a/K40i>CT0>c/k0050>c/k0090>WYj@1fMWH0EEEE07IfMP2FUYH0k>c/09:BTP0c
<c<0gMgM00L0k>c/00H0a<C40>c/k03/k>`0k>c/0<g=c@1UIFD207IfMP060>c/k02>SXh0DU9B07Ad
M01TI6@0emOG0P3/k>`01@1QHF40]kNg0>c/k02GUiL0U9BD00<0k>c/00@0g]kN096AT@3Bd]80]kNg
3@3/k>`0102NWYh0aLG50=;BdP3AdM4=0>c/k0040>GUi@1NGUh0bl_;0;Ng]`D0k>c/00<0UIFE0>c/
k03/k>`01P3/k>`01P31`L40U9BD0>c/k03/k>`0ZZZZ0=oOg`L0k>c/00<0ZZZZ0=oOg`3/k>`01@3/
k>`00`2a/K40i>CT0>c/k00;0>c/k0030;jn_P3/k>`0k>c/00`0k>c/00H0YjNW08>3P`3/k>`0d];B
05ADE03MgMdA0>c/k0030<c<c00^;Rh0emOG01H0k>c/00D0^k^k0<;2`P32`/806QXJ0=gMg@0@0>c/
k0030<c<c00^;Rh0emOG0240k>c/00<0[Jf]07QhN03/k>`08`3/k>`01`3WinL0k^k^0>o_k`3`l?00
m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C
0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:06d0k>c/
00<0d=3@014A4@38b<P00`3/k>`00`2KVi/000000:FUY@0=0>c/k0040>[ZjP1>CTh000000<S8b0h0
k>c/00<0jNWY02LW9`2UYJD0103/k>`00`2h^;P0I6AT0>c/k0070>c/k0030=kNgP0A4A40YJFU02h0
k>c/00<0d];B00L71`32`/805@3/k>`00`3Zj^X0EUIF0=OGe`0H0>c/k0030=kNgP0M7Ad0gMgM0100
k>c/00D0j^[Z05IFEP3De=@08b<S0<;2`P0A0>c/k0030>KViP1bLW80k>c/00/0k>c/00<0i^KV079b
LP3/k>`08`3/k>`01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000030000003lo?`0RXZ:0?l0k>c/J03/k>`01`3WinL0k^k^0>o_k`3`l?00m?Cd0?Kf
mP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3X
j>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0k>c/J03/k>`0
1`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00003
0000003lo?`0RXZ:0?l0k>c/J03/k>`01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT2
0?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3j
n_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0k>c/J03/k>`01`3WinL0k^k^0>o_
k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:
0?l0k>c/J03/k>`01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000030000003lo?`0RXZ:0?l0k>c/J03/k>`01`3WinL0k^k^0>o_k`3`l?00m?Cd0?Kf
mP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3X
j>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0k>c/J03/k>`0
1`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00003
0000003lo?`0RXZ:0?l0k>c/J03/k>`01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT2
0?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3j
n_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0c/k>J03>c/h01`3WinL0k^k^0>o_
k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:
0?l0gmoOJ03Ogml01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000030000003lo?`0RXZ:0?l0gmoOJ03Ogml01`3WinL0k^k^0>o_k`3`l?00m?Cd0?Kf
mP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3X
j>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0gmoOJ03Ogml0
1`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00003
0000003lo?`0RXZ:0?l0gmoOJ03Ogml01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT2
0?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3j
n_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0gmoOJ03Ogml01`3WinL0k^k^0>o_
k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:
0?l0gmoOJ03Ogml01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000030000003lo?`0RXZ:01<0gmoO00<0g=cL02HV9P0000000P0000001@2l_;`0gmoO
0=oOg`2h^;P09RHV0080000000D0>CTi000000000000ZZZZ04e=C@040000000403@d=03Kfm/0gMgM
05eMG@80000000h0C4a<02HV9P000000FUYJ00P8200:2PX0_kno0000001@D500YZJV0000001=CDd0
^;Rh03dm?@@0000000D0S8b<0=oOg`3Ogml0^;Rh03Lg=`02000000030492@P2l_;`0U9BD00@00000
00@06QXJ0=OGe`3Ogml0KVi^1@0000001025QHD0gmoO0=;BdP1>CTh30000000403Hf=P2l_;`0g=cL
02HV9P<0000000@0_;bl0=oOg`3Ogml0YZJV1@000000100d=3@0fm_K0=;BdP1>CTh20000000301XJ
6P1^KVh0e=CD0080gmoO00P0^;Rh01/K6`00000000000492@P2l_;`0^KVi03Lg=`80000000@03@d=
06i^KP3:b/X0U9BD100000000`0l?3`0RHV90=gMg@3I0=oOg`070>OWi`3^k^h0kno_0?3`l03dm?@0
m_Kf0?Shn0020?Win@80no_k0140inOW08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX0503Ogml00`37
alL000000:b/[0040=oOg`0707YjNP0d=3@0fm_K0=oOg`2DU9@02PX:0<o?c`020=oOg`030:JVYP06
1PH0b/[:00<0gmoO0180fMWI02PX:03GemL0gmoO0=cLg01=CDd0K6a/0=oOg`1oOgl0clo?0<O7a`1@
D500gmoO0:JVYP061PH0_Kfm014A4@1/K6`70=oOg`0606i^KP061PH0b/[:0=oOg`1HF5P0VYZJ0P3O
gml00`1^KVh0GEeM0=oOg`050=oOg`030<O7a`000000[:b/00<0gmoO00<0KVi^02PX:03Ogml00P3O
gml01P1HF5P0VYZJ0=oOg`37alL000000:b/[0H0gmoO00<0alO70000002/[:`00P3Ogml00`3MgMd0
CDe=02PX:0020=oOg`0306i^KP000000QHF50080gmoO01D0KVi^05eMG@3Ogml0gmoO05QHF02JVYX0
YZJV03@d=03Kfm/0gmoO0;Rh^0000000D51@0=oOg`1^KVh0GEeM0=oOg`3MgMd0CDe=0000002/[:`0
f@3Ogml01`3WinL0k^k^0>o_k`3`l?00m?Cd0?KfmP3hn?P00P3inOT20?_kn`0A0>OWi`2:RXX0o?cl
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3o
ool000030000003lo?`0RXZ:01@0gmoO00D0alO70000001FEUH0Kfm_09JFUP020=oOg`030<_;b`1H
F5P000000080000000L02PX:0<o?c`3Ogml0gmoO0:JVYP030`<0^k^k00@0gmoO00<0QhN700000000
00000P0000002P1/K6`0gmoO07YjNP1eMGD0alO700D51@2d];@0YZJV00H61P2b/[860000000606AT
I03Ogml0gmoO06i^KP0:2PX0emOG1@3Ogml00`1^KVh0C4a<0=oOg`050=oOg`030<O7a`000000[:b/
0080gmoO00<0fMWI01`L701hN7P00`3Ogml0202=SHd0emOG0=oOg`37alL0000005IFEP1_Kfl0UYJF
103Ogml00`37alL000000:b/[0020=oOg`030=;BdP0A4A40SXj>0080gmoO00<0gMgM05eMG@1MGEd0
0P3Ogml00`1^KVh0K6a/0=oOg`070=oOg`060:JVYP0820P0I6AT0=oOg`1^KVh0GEeM0P3Ogml00`3J
f]X0=3@d07QhN03I0=oOg`040>WYj@3Fe]H0alO70<S8b080b/[:0`3<c<`04`3=cLd0bLW90>WYj@2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000030000003lo?`0RXZ:01@0gmoO00<0alO70000000000000P0000000`1@D500gmoO
0=oOg`030=oOg`0307YjNP0:2PX0clo?0080gmoO00@0YZJV000000051@D0clo?1P3Ogml03P3IfMT0
71`L06a/K03Ogml0NWYj00H61P2b/[80000003@d=02RXZ801PH60;fm_@0A4A40GEeM0P3Ogml00`37
alL0000009NGU`020=oOg`0306i^KP0:2PX0emOG00D0gmoO00@0KVi^0000000:2PX0emOG103Ogml0
0`37alL000000:b/[0030=oOg`0906i^KP0J6QX0emOG0=oOg`3IfMT04A4A0;Fe]@3Ogml0alO700@0
000000L0D51@0=oOg`3Ogml0gmoO0<O7a`000000[:b/0080gmoO00<0gMgM04e=C@0X:2P00P3Ogml0
0`1^KVh0000008F5Q@020=oOg`0306i^KP1/K6`0gmoO00D0gmoO00@0U9BD000000000000D51@0P3O
gml01`1^KVh0GEeM0=oOg`3Ogml0g=cL04e=C@1/K6`0f@3Ogml00`3/k>`0^[Zj0=?Cd`070=?Cd`0B
0=SHf02FUYH0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:01@0gmoO00T0alO70000002/[:`0gmoO0:2P
X0000000RHV90=SHf00o?cl00P0000003@0=3@d0Ti>C0=oOg`3Ogml0TI6A000000061PH0SXj>00h>
3P000000H61P0=oOg`28R8P00`0000003@0J6QX0^KVi0<G5a@0820P06QXJ02@T901aLG40H61P0000
001BDU80fm_K0<O7a`0m?Cd00P0000001@0D51@0M7Ad0=_Kf`3Ogml0B4Q800@0000000<0CDe=0=oO
g`2h^;P00P0000001`1GEeL0MGEe00L71`0J6QX0`l?30=WIf@0L71`00P0000001P2/[:`0gmoO0=oO
g`3Ogml0g=cL05YJFP80000000/06QXJ02h^;P32`/80gmoO0<O7a`000000[:b/0=oOg`2PX:000000
08V9R@020=oOg`030<O7a`000000[:b/00<0gmoO00T0d];B05MGE`000000000002PX:01bLW80e=CD
0=oOg`18B4P010000000301=CDd0gmoO0=oOg`3Ogml0^KVi06m_K`0m?Cd0:b/[0<[:bP3Ogml0KVi^
05eMG@80gmoO00<0d=3@02l_;`1hN7P0f@3Ogml01P3/k>`0b/[:0?Ogm`3]kNd0jNWY0>KViP80g=cL
01@0dm?C0<o?c`3WinL0UYJF08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`
l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP0D0=oOg`070<O7a`000000
[:b/0=oOg`2DU9@0000009ZJVP0X0=oOg`0306i^KP0:2PX0emOG01/0gmoO00L0alO70000002/[:`0
gmoO09BDU0000000VYZJ0080gmoO00<0alO70000002/[:`0303Ogml00`1^KVh0K6a/0=oOg`070=oO
g`0;0;Rh^0000000VYZJ0=oOg`1^KVh0GEeM0=oOg`3Lg=`071`L0000002l_;`0f@3Ogml01P3/k>`0
b/[:0?Ogm`3]kNd0j^[Z0>OWi`80g=cL01@0dm?C0<o?c`3WinL0UYJF08Z:RP3lo?`0000009bLW02V
YZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000
o?cl08Z:RP0C0=oOg`030=cLg00V9RH000000080000000<051@D08V9R@3Kfm/0:03Ogml00`1^KVh0
2PX:0=OGe`0=0=oOg`0309BDU0061PH0b/[:00X0gmoO00<0g=cL02HV9P0000000P0000001@0D51@0
RHV90=_Kf`3Lg=`071`L0080000000<0[:b/0=oOg`3Ogml02P3Ogml00`1^KVh0K6a/0=oOg`030=oO
g`080=OGe`0F5QH0Bd];06m_K`1:BTX0:b/[0<[:bP2DU9@40000000304e=C@2GUiL0gmoO0=T0gmoO
00H0k>c/0<[:bP3gmoL0kNg]0>WYj@3Vi^H20=cLg00D0=?Cd`3?cll0inOW09JFUP2:RXX0o?cl0000
002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`00
00<000000?clo02:RXX0P03Ogml0103Lg=`09RHV0000000X:2SS0=oOg`060>c/k03:b/X0moOg0>g]
k@3YjNT0i^KV0P3Lg=`0503Cdm<0clo?0>OWi`2FUYH0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0
gmoOJ03Ogml01P3/k>`0b/[:0?Ogm`3]kNd0j^[Z0>OWi`80g=cL01@0dm?C0<o?c`3WinL0UYJF08Z:
RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?kn
oP40oooo00000`000000o?cl08Z:RP3o0=oOgfP0gmoO00<0k>c/0;no_`3Kfm/00P3Kfm/00`3Jf]X0
fm_K0=_Kf`020=_Kf`0B0<S8b031`L40RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40
j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0gmoOJ03Ogml0
0`3/k>`0_Kfm09nOW`0709nOW`0B09JFUP3Vi^H0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0gmoO
J03Ogml00`3/k>`0`/;20=CDe0070=CDe00B0>7Qh@2FUYH0RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:
0?l0gmoOJ03Ogml00`3/k>`0b/[:0<k>cP070<k>cP0B0>KViP2FUYH0RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003l
o?`0RXZ:0?l0gmoOJ03Ogml00`3/k>`0b/[:0<k>cP070<k>cP0B0>KViP2FUYH0RXZ:0?clo0000000
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool00003
0000003lo?`0RXZ:0?l0gmoOJ03Ogml0103/k>`0b/[:0<k>cP26QXH40<k>cP0D08J6QP3>c/h0i^KV
09JFUP2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl
0?gmo@3no_h10?ooo`0000<000000?clo02:RXX0o`3OgmmX0=oOg`030>c/k03:b/X0clo?0080?Shn
0P3?cll203hn?P0C0<o?c`3WinL0UYJF08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX0o`3OgmmX0=oO
g`060>c/k03:b/X0emOG0;:b/P0n?Sh0Hf=S0P0n?Sh0502b/[80emOG0>_[j`2FUYH0RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool0
00030000003lo?`0RXZ:0?l0gmoOJ03Ogml00`3/k>`0b/[:0>CTi0020>CTi080?Shn0`3Ti>@04P3a
lO40UYJF08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3l
o?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o0=oOgfP0gmoO00<0k>c/0<[:bP3`l?000P3`
l?000`34a<@0l?3`0?3`l0020?3`l00B0?Ogm`2FUYH0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0
gmoOJ03Ogml00`3/k>`0b/[:0?clo0070?clo00B0?gmo@2FUYH0RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0
RXZ:0?l0gmoOJ03Ogml00`3/k>`0`<300?ooo`070?ooo`0B0?Gem@2FUYH0RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000
003lo?`0RXZ:0?l0gmoOJ03Ogml00`3/k>`0][Jf0;Jf]P070;Fe]@0B0:6QX@3Jf]X0RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool0
00040000003lo?`0RXZ:09VIVOl0KVi^LP1^KVh04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:
RP2IVIWo0>3Ph6L0h>3P00<0k^k^0<K6aP2o_kl01`2o_kl04P26QXH0VIVI08Z:RP3lo?`0000009bL
W02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00001000
0000o?cl08Z:RP2IVIWo0>3Ph6L0h>3P00<0k^k^0?ooo`3oool01`3oool04P26QXH0VIVI08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40
oooo000010000000o?cl08Z:RP2IVIWo0>k^kVL0k^k^2P3emOD04P3EeMD0VIVI08Z:RP3lo?`00000
09bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo0000
0`000000o?cl08Z:RP1L09VIV@030?clo02IVIT0VIVI05X0VIVI00<0o?cl09VIV@2IVIT0FP2IVIT0
0`3lo?`0VIVI09VIV@1J09VIV@0@08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SX
j03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo000010000000o?cl08Z:RP2IVIT=0?ooo`040?_k
n`26QXH0Ogmo0>?ShaP0oooo00@0no_k08J6QP1oOgl0hn?S803oool00`3jn_X0m_Kf0<[:bP080<o?
c`050<K6aP3hn?P0VIVI0?clo02IVIT0C@3oool00`3jn_X0m_Kf0<[:bP080<o?c`050<K6aP3hn?P0
VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0
YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000
003lo?`0RXZ:09VIV@l0oooo00<0moOg07alO03oool06@3oool00`3gmoL0O7al0?ooo`0O0?ooo`03
0?Gem@2FUYH0g=cL00P0fm_K00D0g]kN0<[:bP2IVIT0o?cl09VIV@1=0?ooo`030?Gem@2FUYH0g=cL
00P0fm_K00D0g]kN0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI
08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm
0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3oool0202IVIT0moOg0?ooo`3oool0gmoO0:FU
Y@3oool0d=3@0P0000008@3IfMT0oooo06=SH`0?3`l0SHf=0=oOg`1CDe<0no_k0?Ogm`0>3Ph0EeMG
06QXJ03Ogml0De=C0>_[j`1CDe<0jn_[05=CD`2EUID0moOg0=oOg`2UYJD0gmoO09FEU@1CDe<0UIFE
0?Ogm`3kno/0UiNG0?ooo`1SHf<03`l?08f=S@0P0?ooo`030?Gem@2FUYH0i^KV00P0c/k>00D0oooo
0<[:bP2IVIT0o?cl09VIV@030?ooo`05055AD@000000AdM70?ooo`3@d=000P0000003P3IfMT0oooo
03`l?00l?3`0TY:B0=oOg`1CDe<0nOWi05eMG@3Xj>P0XZ:R01hN7P1CDe<0d];B0`3oool0901hN7P0
00000000002d];@0oooo0::RXP000000H61P0>o_k`1CDe<0nOWi05eMG@3Bd]80d=3@0000000`<300
oooo09VIV@3gmoL0oooo08Z:RP000000>SXj0;Bd]02:RXX0@D510=OGe`2UYJD0XZ:R0000002d];@0
kno_05=CD`3gmoL0[:b/0=;BdPh0oooo00<0mOGe09JFUP3Vi^H0203>c/h01@3oool0b/[:09VIV@3l
o?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl
08Z:RP2IVIT20?ooo`0[09VIV@3gmoL0oooo0?ooo`3Ogml0YJFU0?ooo`29RHT0moOg0?_kn`29RHT0
g]kN09jNWP3kno/0Ogmo0=oOg`1CDe<0no_k0<K6aP2GUiL0g]kN07AdM03Ogml0De=C0>_[j`1CDe<0
jn_[05=CD`2EUID0moOg0=oOg`2UYJD0gmoO09FEU@1CDe<0UIFE0?Ogm`3kno/0UiNG0=kNgP2NWYh0
no_k07moO`0L0?ooo`0707moO`3oool0oooo0?ooo`3emOD0UYJF0>KViP080<k>cP050?ooo`3:b/X0
VIVI0?clo02IVIT00P3oool05P3Rh^80_kno0?ooo`2k^k/0XJ6Q08V9R@3gmoL0no_k08V9R@3Ogml0
UYJF0>_[j`1iNGT0gmoO05=CD`3inOT0GEeM0<?3``1XJ6P0m?Cd03`l?03Bd]820?ooo`0U09fMW@2W
YjL0oooo0?gmo@1VIVH0^[Zj05QHF03kno/0^k^k0=?Cd`1CDe<0nOWi05eMG@3Bd]80S8b<0?Ogm`3M
gMd0alO709VIV@3gmoL0oooo07inOP3gmoL0NWYj0=;BdP2IVIT0moOg0=oOg`2>SXh0@4100?Win@1?
Cdl0d=3@05=CD`3gmoL0[:b/0=;BdP0:0?ooo`0707moO`3oool0oooo0?ooo`3emOD0UYJF0>KViP08
0<k>cP050?ooo`3:b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000040000003lo?`0RXZ:09VIV@80oooo00P0N7Qh000000000000KFe]0=oOg`2UYJD0
oooo09>CT`80oooo0240UYJF0<K6aP2_[jl0oooo09JFUP3Ogml0De=C0?_kn`3oool0N7Qh0000001]
KFd0gmoO05=CD`3[jn/0De=C0>_[j`1CDe<0UIFE0?Ogm`3Ogml0YJFU0=oOg`2EUID0De=C09FEU@3g
moL0no_k09NGU`36a/H0[jn_0?ooo`2FUYH0703oool200000080oooo00<0mOGe09JFUP3Vi^H00`3>
c/h00`0n?Sh0c/k>0<k>cP020<k>cP050?ooo`3:b/X0VIVI0?clo02IVIT0103oool0103gmoL05QHF
0>o_k`2CTi<20?ooo`0>09JFUP3Ogml0YJFU0?_kn`2GUiL0gmoO05=CD`3inOT0GEeM0;Zj^P2?Shl0
oOgm05eMG@3Bd]820?ooo`0309>CT`3oool0^k^k0080000000X0Zj^[000000000000000009nOW`1C
De<0nOWi05eMG@3Bd]80DE5A0P000000501PH600VIVI0?Ogm`3oool0kno_018B4P0410@0d];B09VI
V@3gmoL0gmoO08R8R01PH600oooo07QhN039bLT0De=C0?Ogm`2/[:`0d];B2P3oool200000080oooo
00<0mOGe09JFUP3Vi^H00`3>c/h00`0n?Sh0c/k>0<k>cP020<k>cP050?ooo`3:b/X0VIVI0?clo02I
VIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:
09VIV@80oooo01<0VIVI0?Ogm`3oool0SXj>0=OGe`1QHF40QhN70:BTY01gMgL0Ng]k0:6QX@3kno/0
Kfm_08>3P`1jNWX0gmoO03@d=02:RXX0inOW0080Ng]k01H0WYjN0=oOg`14A4@0Ng]k05aLG01kNg/0
RXZ:096AT@2CTi<0Hf=S07MgM`1SHf<0];Bd05=CD`2>SXh0QXJ607]kN`2NWYh0no_k06m_K`23Ph<0
NWYj703oool2000000050;no_`3oool0mOGe09JFUP3Vi^H00P3>c/h00`1RHV80?Shn069RHP030<k>
cP050?ooo`3:b/X0VIVI0?clo02IVIT00P3oool05P3_knl0L71`08N7Q`3kno/0oooo0:RXZ01gMgL0
Ng]k0:6QX@3Ogml0YJFU0?_kn`2GUiL0gmoO04E5A@20P800ATI60>c/k01LG5`0Q8B403Lg=`3Bd]82
0?ooo`0U08V9R@3gmoL0oooo0?ooo`30`<00jn_[07YjNP1mOGd0PH610=?Cd`15ADD0P82004I6AP3g
moL0];Bd07]kN`20P800alO708V9R@2>SXh0QhN70;2`/01kNg/0H61P0?Gem@1QHF40W9bL0=_Kf`2M
WId0HF5Q07UiN@1JFUX0hn?S04E5A@26QXH0H61P0?Ogm`0:0?ooo`80000000D0_kno0?ooo`3emOD0
UYJF0>KViP020<k>cP03069RHP0n?Sh0HV9R00<0c/k>00D0oooo0<[:bP2IVIT0o?cl09VIV@1J0?oo
o`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3o
ool01@2IVIT0moOg0?ooo`23Ph<0moOg0480oooo00P000000?ooo`3oool0oooo0?Gem@2FUYH0jNWY
0=CDe080?Shn00<0e=CD03hn?P0n?Sh00P3De=@01@3oool0b/[:09VIV@3lo?`0VIVI0080oooo00D0
gmoO08R8R03oool0OGem0=;BdP0>0?ooo`030?gmo@1MGEd0d];B0080oooo00H0^k^k061PH03oool0
gmoO024Q8@3gmoLC0?ooo`0309VIV@3gmoL0oooo01<0oooo00P000000?ooo`3oool0oooo0?Gem@2F
UYH0jNWY0=CDe080?Shn00<0e=CD03hn?P0n?Sh00P3De=@01@3oool0b/[:09VIV@3lo?`0VIVI05X0
oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?C
d`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIT2
0?ooo`040;^k^`1oOgl0Ogmo0;Jf]Qd0oooo00<0moOg0:VYZ@3moOd09`3oool0403emOD0UYJF0>o_
k`2ATI40?Shn0;Rh^03Ogml0^;Rh03hn?P2h^;P0gmoO0?ooo`3:b/X0VIVI0?clo02IVIT30?ooo`03
0;6a/@1oOgl0[:b/00l0oooo00<0o_kn0:j^[P3Xj>P00`3oool0102k^k/0Ogmo08B4Q03Xj>PD0?oo
o`040=CDe03kno/0kno_0=;BdQH0oooo0100mOGe09JFUP3_knl0TI6A03hn?P2h^;P0gmoO0;Rh^00n
?Sh0^;Rh0=oOg`3oool0b/[:09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VI
V@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gm
o@3no_h00@3oool000040000003lo?`0RXZ:09VIVDd0oooo00<0mOGe09JFUP3fm_H0203]kNd01@3o
ool0b/[:09VIV@3lo?`0VIVI04d0oooo00<0mOGe09JFUP3fm_H0203]kNd01@3oool0b/[:09VIV@3l
o?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl
08Z:RP2IVIU=0?ooo`030?Gem@2FUYH0nOWi00P0m?Cd00D0oooo0<[:bP2IVIT0o?cl09VIV@1=0?oo
o`030?Gem@2FUYH0nOWi00P0m?Cd00D0oooo0<[:bP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0
VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00
m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000D000000?clo02:RXX0VIVI0?ooo`1<0<k>cP030?Ge
m@2FUYH0o_kn00P0oOgm00D0oooo0<[:bP2IVIT0o?cl09VIV@0G0?ooo`030>;RhP2GUiL0oooo03<0
oooo00<0mOGe09JFUP3no_h0203moOd01@3oool0b/[:09VIV@3lo?`0VIVI05X0oooo00<0VIVI0?cl
o02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIU=0?ooo`030?Gem@2F
UYH0jn_[00P0oooo00D0emOG0<[:bP2IVIT0o?cl09VIV@0H0?ooo`030:b/[03Bd]80oooo0380oooo
00<0mOGe09JFUP3[jn/0203oool01@3GemL0b/[:09VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02I
VIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03f
m_H0n_[j0?clo03moOd0o_kn0040oooo00001@000000o?cl08Z:RP2IVIT0f][J00d0]KFe00<0[jn_
07MgM`2c/k<0?02e]KD01P3?cll0i>CT0<W9b@3Bd]80dm?C0=CDe080e]KF0`3GemL01@35aLD0kno_
09VIV@3lo?`0VIVI0080oooo01T0a/K605MGE`1oOgl0PH610?_kn`2XZ:P0MgMg07moO`2IVIT0gmoO
0:FUY@3kno/0UiNG0<k>cP1JFUX0Ogmo075aL@3AdM40De=C08f=S@2LW9`0no_k0:n_[`1GEeL0no_k
0080oooo0100gmoO05=CD`3kno/0oooo0>o_k`1CDe<0Y:BT07MgM`1dM7@0j>SX0>_[j`1ZJVX0b/[:
06a/K01oOgl0VYZJ803oool01P3dm?@0i>CT0<W9b@3Bd]80dm?C0=CDe080e]KF0`3GemL01@35aLD0
kno_09VIV@3lo?`0VIVI05X0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
1@000000o?cl08Z:RP2IVIT0f][J00d0]KFe00<0ZJVY03Xj>P2b/[80?02e]KD02P3>c/h0k^k^0>o_
k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIV@80oooo00H0
a/K60:n_[`3oool0[:b/0=;BdP2HV9P30?ooo`040=oOg`2UYJD0no_k09NGU`80oooo00X0N7Qh0000
0039bLT0De=C09FEU@3gmoL0oooo09>CT`2RXZ80alO70P3oool0403Ogml0De=C0?_kn`3oool0kno_
05=CD`2?Shl0oooo0;^k^`2d];@0gmoO0:FUY@3oool0kno_018B4P1JFUXL0?ooo`0>0000003oool0
oooo0?ooo`3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??cl`2I
VIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00009@000000
o?cl08Z:RP2IVIT0f][J0;Fe]@2MWId0CDe=05YJFP1IFET0GEeM09bLW00j>SX0/[:b09fMW@0/;2`0
GemO00d=3@2DU9@092@T05UIF@10@400YJFU06a/K02_[jl0NGUi09JFUP1/K6`0[:b/05aLG01EEED0
Bd];07emO@0o?cl0FUYJ051@D02:RXX00P2e]KD0901TI6@0FUYJ03Ti>@2;Rh/0WYjN07AdM02b/[80
Jf][0:2PX01CDe<0I6AT02HV9P2UYJD0]KFe0:ZZZP0n?Sh0EeMG05UIF@1@D500XZ:R04A4A01FEUH0
CTi>0:NWY`0f=SH0EEEE04E5A@2FUYH0K6a/0:n_[`1iNGT0UYJF0:2PX01CDe<0I6AT05iNGPH0]KFe
00X0c/k>0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?cl
o02IVIT20?ooo`0I0<K6aP2_[jl0oooo0;^k^`2d];@0MGEe07]kN`1oOgl0I6AT0=oOg`2FUYH0nOWi
09:BTP30`<00D51@09FEU@2BTY80hn?S05=CD`2EUID0moOg0>g]k@26QXH0kNg]09BDU0020?ooo`0@
0=oOg`0b<S80OGem06MWI`2b/[80De=C08F5Q@3gmoL0Y:BT0<?3``3Ogml0YJFU0<30`01XJ6P0Zj^[
0;fm_A`0oooo0P000000302o_kl0oooo0??cl`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi
0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000080000003lo?`0RXZ:09VIV@3Jf]X0]KFe05YJFP2ZZZX20;Fe]@0E05EEE@2?Shl0
>SXj0;:b/P2<S8`02`/;0:b/[00K6a/0VYZJ059BDP2e]KD0EEEE0861P@1/K6`0[jn_07UiN@2FUYH0
K6a/09RHV00j>SX0/[:b00<0]KFe00<0EEEE00000021PH400P2e]KD03@1;Bd/0ZZZZ06EUI@2FUYH0
WYjN07AdM02b/[80Jf][079bLP2=SHd0/[:b0451@@2UYJD01@2e]KD04P1bLW80NGUi04E5A@2e]KD0
EEEE08n?S`0j>SX0[jn_06EUI@2FUYH0K6a/0:n_[`1iNGT0UYJF079bLP2=SHd0/[:b06][J`80]KFe
00h000000;Fe]@2e]KD0]KFe0<k>cP3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80
no_k00@0lo?c09VIV@3lo?`0VIVI0P3oool01P36a/H0[jn_0?ooo`2/[:`0d];B0=3@d08000000140
[jn_0=oOg`13@d<000000:n_[`3kno/0E5AD0000002?Shl0kno_04i>CP0U9BD0@D51071`L037alL0
oooo09>CT`020?ooo`0@0=oOg`1CDe<0no_k0=oOg`2EUID0De=C0<c<c0000000AdM70?_kn`1DE5@0
820P0??cl`0Z:RX000000<O7aa`0oooo0P0000020?ooo`0:0??cl`3^k^h0kno_0?3`l03alO40m?Cd
0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J
0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3j
n_X0o?cl0?gmo@3no_h00@3oool000070000003lo?`0RXZ:09VIV@3Jf]X0]KFe06QXJ0020;Fe]@0J
02XZ:P000000Shn?03Xj>P2b/[80]KFe09jNWP061PH01`L709ZJVP1BDU80]KFe05EEE@21PH40K6a/
0:n_[`1iNGT0UYJF06a/K02HV9P0>SXj0;:b/P2e]KD0ZJVY00d=3@0R8R840;Fe]@0=0:VYZ@0=3@d0
0`<309JFUP2NWYh0M7Ad0;:b/P1[Jf/0LW9b08f=S@2b/[80@D510:FUY@030;Fe]@0D0;:b/P0D51@0
2`/;09^KV`15ADD0]KFe05EEE@2?Shl0>SXj0:n_[`1iNGT0UYJF06a/K02_[jl0NGUi09JFUP1bLW80
SHf=0;:b/P1[Jf/20;Fe]@8000000P2e]KD02P3>c/h0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh
0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIV@80oooo00D0a/K60:n_[`3oool0Ph>30?Ogm`0>
0?ooo`0309VIV@3gmoL0oooo00D0oooo00L0gmoO05=CD`3kno/0a/K608b<S01CDe<0no_k00<0oooo
00<0gmoO0:FUY@3oool07`3oool03P000000oooo0?ooo`3oool0lo?c0>k^kP3_knl0l?3`0?7al@3d
m?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI
05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf
0?[jnP3lo?`0oOgm0?knoP010?ooo`0000P000000?clo02:RXX0VIVI0=[JfP2e]KD0HF5Q0:n_[`80
]KFe01/0R8R80:6QX@0U9BD0HV9R0:BTY01FEUH0EeMG038b<P2VYZH0<30`05mOG`10@400YJFU065Q
H@1UIFD0A4A40:n_[`1/K6`0[:b/05aLG01EEED0?Shn08B4Q01@D500FEUI0410@02UYJD00P2e]KD0
901oOgl0EeMG04A4A02_[jl0WYjN04M7A`1GEeL0L71`0:2PX01<C4`0I6AT02HV9P2UYJD0]KFe0:n_
[`17AdL0Fe]K0:n_[`2e]KD0[jn_04A4A01FEUH0@4100:6QX@0j>SX0[jn_07UiN@2FUYH0IfMW05mO
G`14A4@0[jn_0:2PX01<C4`0I6AT05iNGP80]KFe0P0000003027QhL0]KFe0<k>cP3^k^h0kno_0?3`
l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVI0P3oool0103Rh^80
Ogmo07moO`2f][H=0?ooo`040?Ogm`2YZJT0d];B0?_kn`H0oooo00L0kno_08R8R01oOgl0Q8B40>_[
j`2YZJT0oOgm00<0oooo00<0no_k0=CDe03oool08`3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cd
m03gmoL0n?Sh0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0
FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0040oooo000030000000o?cl08Z:RP2IVIT0f][J0;Fe]@25QHD0ADE50;Fe
]@2YZJT05aLG0:n_[`/0]KFe00<0K6a/0:n_[`2e]KD05`2e]KD02P2b/[80@D510:FUY@2e]KD0ZJVY
01LG5`2_[jl0WYjN01<C4`2UYJD>0;Fe]@050;:b/P1[Jf/0]KFe0;Fe]@0000000`2e]KD02P3>c/h0
k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIVDd0
oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI
0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0
][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000D000000?cl
o02:RXX0VIVI0=[JfP020;Fe]@0408F5Q@1JFUX0G5aL0:FUY@`0]KFe00H0T92@0;:b/P2e]KD0]KFe
092@T02b/[8D0;Fe]@030;>c/`1kNg/0[Jf]0080]KFe00@0X:2P05YJFP1JFUX0Vi^K3`2e]KD00`2c
/k<0T92@0;Fe]@050;Fe]@0:0<k>cP3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80
no_k00@0lo?c09VIV@3lo?`0VIVIC@3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh
0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool04@2I
VIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03m
oOd0o_kn0040oooo00001@000000o?cl08Z:RP2IVIT0f][J04`0]KFe00X0c/k>0>k^kP3_knl0l?3`
0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIU=0?ooo`0:0??cl`3^
k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3o
ool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C
0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000050000003lo?`0RXZ:09VIV@3/
k>`0C03Jf]X02P3Ph>00k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??c
l`2IVIT0o?cl09VIV@<0oooo00D0dm?C07moO`2SXj<0oooo0>OWi`0207moO`0B0>c/k03_knl0d];B
0?gmo@3;bl/0no_k08j>SP3=cLd0d=3@07moO`3IfMT0oooo0;^k^`1oOgl0Xj>S0>SXj03<c<`0no_k
0`3oool0403<c<`0no_k0?ooo`3oool0c<c<0?_kn`2XZ:P0Ogmo0=WIf@3moOd0SXj>0<c<c02YZJT0
Ogmo0:2PX03kno/N0?ooo`0:0??cl`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80
no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3o
ool000040000003lo?`0RXZ:09VIVDd0oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Sh
n03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIT30?ooo`0I08>3P`3oool0RXZ:08n?S`20P800
kno_0?Ogm`1lO7`0gmoO0:FUY@3kno/0UiNG0=oOg`1CDe<0jn_[024Q8@3clo<09RHV0=SHf00Q8B40
kno_07emO@3Bd]80VIVI0?Ogm`030?ooo`0@09VIV@3gmoL0oooo0?ooo`2IVIT0emOG08R8R03gmoL0
9RHV0=SHf01CDe<0c<c<061PH03oool0XZ:R08n?Sah0oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3d
m?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI
05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf
0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVIC@3oool02P3clo<0k^k^0>o_
k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIV@80oooo00<0
moOg08f=S@3oool00P3oool05@2CTi<0oooo0?ooo`2FUYH0gmoO0:FUY@3kno/0UiNG0=oOg`1CDe<0
g]kN061PH03oool0N7Qh0<W9b@1CDe<0moOg08j>SP3Bd]80VIVI0?Ogm`030?ooo`0@09VIV@3gmoL0
oooo0?ooo`2IVIT0_[jn0:n_[`3oool0N7Qh0<W9b@1CDe<0no_k0?ooo`1hN7P000000;Bd]1X0oooo
00h000000?ooo`3oool0oooo0??cl`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80
no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3o
ool000040000003lo?`0RXZ:09VIV@80oooo0100R8R807]kN`1oOgl0UiNG0=oOg`2<S8`0FUYJ07mo
O`1aLG40`l?3096AT@2LW9`0TY:B07moO`1hN7P0`l?30P3oool03@3Ng]h0GUiN08j>SP1ZJVX0gmoO
05=CD`3inOT0GEeM0=[JfP1aLG40Q8B403Lg=`3Bd]800P3oool07@2IVIT0mOGe08J6QP3kno/0Hf=S
0>KViP0n?Sh0QXJ601<C4`3Fe]H0>3Ph09bLW02GUiL0moOg0>o_k`1CDe<0^[Zj079bLP1oOgl0PH61
0>_[j`1/K6`0Ogmo09VIV@3>c/h0FUYJ07moO`1aLG40hN7Q00d0oooo00X0lo?c0>k^kP3_knl0l?3`
0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIT20?ooo`030=oOg`2U
YJD0oooo0080oooo01D0RHV90?Ogm`3kno/0RHV90=oOg`2FUYH0nOWi09:BTP3Ogml0De=C0>CTi010
@400nOWi04m?C`3@d=00De=C0?Ogm`2/[:`0d];B08^;R`3Ti>@00`3oool04028R8P0Ng]k07=cL`29
RHT0UiNG0<[:bP2KVi/0no_k04m?C`3@d=00De=C0=_Kf`1@D500UIFE09:BTP3Xj>PJ0?ooo`800000
00`0_kno0?ooo`3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??c
l`2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00001000
0000o?cl08Z:RP2IVIT20?ooo`0@09VIV@3gmoL0oooo0?ooo`3Ogml0YJFU0?ooo`1hN7P000000;Bd
]02IVIT0moOg0?ooo`3Ogml02@T90;Bd]080oooo00d0a/K607moO`3gmoL0O7al0=oOg`1CDe<0nOWi
05eMG@2j^[X0Shn?0?gmo@1MGEd0d];B0080oooo0180VIVI0>o_k`1^KVh0jn_[06=SH`3Jf]X03`l?
0??cl`0V9RH0f=SH05=CD`3gmoL0VIVI0?Ogm`3_knl0De=C09FEU@3gmoL20?ooo`030::RXP37alL0
oooo00<0oooo00<0N7Qh0000003IfMT02@3oool03P000000oooo0?ooo`3oool0lo?c0>k^kP3_knl0
l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103clo<0VIVI0?clo02IVIT20?ooo`060?Og
m`2=SHd0oooo0>g]k@3Shn<0d=3@0P0000004`37alL0gmoO04=3@`000000[Jf]05ADE0000000W9bL
0::RXP000000Shn?0>o_k`1CDe<0moOg0:b/[03Bd]80UYJF04Y:BP1oOgl00P3oool03`2IVIT0moOg
0>o_k`1CDe<0UIFE0?Ogm`1ADE40000008n?S`2WYjL0000009ZJVP1DE5@0000008n?S`0K0?ooo`80
00000P3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??c
l`2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00001000
0000o?cl08Z:RP2IVIT20?ooo`0@09VIV@3gmoL0oooo0?ooo`3Ogml0QHF5051@D02EUID0TY:B0>SX
j02IVIT0moOg08R8R01oOgl0WYjN0>SXj080oooo00d0kno_08j>SP1kNg/0OGem0=oOg`0j>SX0m_Kf
04A4A03:b/X0J6QX0?Cdm00l?3`0d];B0080oooo01<0VIVI0:n_[`2m_Kd0[:b/06=SH`3_knl0VIVI
0820P00J6QX0gmoO05=CD`3gmoL0Rh^;0>CTi03_knl0De=C08n?S`3Ti>@0oooo0080`<3000P0DU9B
07moO`1TI6@0`<30051@D02EUID0TY:B0??cl`T0oooo0P000000302o_kl0oooo0??cl`3^k^h0kno_
0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVI0`3oool0102E
UID0Ogmo06YZJP3Xj>P80?ooo`030>_[j`1HF5P0no_k00d0oooo00H0R8R807]kN`1cLg<0RHV909NG
U`3gmoL30?ooo`030?Gem@1HF5P0no_k01h0oooo00h0Ogmo0?ooo`3oool0oooo0??cl`3^k^h0kno_
0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2I
VIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3X
j>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIV@80oooo00l0
VIVI0?Ogm`3oool0oooo0=oOg`2SXj<0E5AD0000002?Shl0no_k02XZ:P11@D40Xj>S00000017AdL0
103oool0301ADE4000000:n_[`3Ogml0JFUY00h>3P1PH600oooo07QhN00N7Qh0De=C0=;BdP80oooo
01`0VIVI09bLW03gmoL0NGUi06=SH`3gmoL0XZ:R0000001PH600YjNW00000011@D40TY:B04Y:BP1_
Kfl0De=C0>_[j`0B4Q80000007moO`3oool0DE5A0000002_[jl0no_k05ADE0000000Shn?2P3oool2
00000080oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0103c
lo<0VIVI0?clo02IVIU=0?ooo`0:0??cl`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[j
nP80no_k00@0lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0
o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h0
0@3oool000040000003lo?`0RXZ:09VIV@80oooo00H0VIVI0?Ogm`3oool0oooo0>o_k`3Bd]840?oo
o`030:6QX@3gmoL0oooo00l0oooo00<0oOgm05eMG@3Bd]800P3oool01P25QHD0Ti>C0?knoP1oOgl0
;Rh^0?Ogm`<0oooo00<0mOGe05QHF03gmoL00P3oool00`3gmoL0ZJVY0?gmo@0E0?ooo`0>07moO`3o
ool0oooo0?ooo`3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??c
l`2IVIT0o?cl09VIVDd0oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j
0P3kno/0103clo<0VIVI0?clo02IVIUJ0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP01
0?ooo`0000@000000?clo02:RXX0VIVIC@3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0
n?Sh0?Win@3jn_X20?_kn`050??cl`2IVIT0o?cl09VIV@3Jf]X0C02e]KD02P3>c/h0k^k^0>o_k`3`
l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`040??cl`2IVIT0o?cl09VIVEX0oooo00<0VIVI
0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIU=0?ooo`0:0??c
l`3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00D0lo?c09VIV@3lo?`0VIVI
0=[JfP1<0;Fe]@0:0<k>cP3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00@0
lo?c09VIV@3lo?`0VIVIFP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00004
0000003lo?`0RXZ:09VIVDd0oooo00X0lo?c0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0
n_[j0P3kno/01@3clo<0VIVI0?clo02IVIT0f][J0080]KFe00@0Kfm_05YJFP2QXJ40X:2P0P1JFUX0
1P1PH600/[:b0;Fe]@2e]KD0ZJVY09BDU080]KFe00`0[jn_07MgM`2RXZ80FUYJ07YjNP2e]KD0/[:b
06][J`2a/K40JFUY05YJFP2ATI4`0;Fe]@0:0<k>cP3^k^h0kno_0?3`l03alO40m?Cd0?Ogm`3hn?P0
nOWi0?[jnP80no_k0640lo?c09VIV@3lo?`0VIVI0=7Ad@3Jf]X0Y:BT0;no_`3Jf]X0_kno0:BTY02o
_kl0f][J0:BTY02o_kl0f][J0;no_`2TY:@0_kno0=[JfP2TY:@0_kno0=[JfP2o_kl0Y:BT0;no_`3J
f]X0Y:BT0;no_`3Jf]X0_kno0:BTY02o_kl0f][J0:BTY02o_kl0f][J0;no_`2TY:@0_kno0=[JfP2T
Y:@0_kno0=[JfP2o_kl0Y:BT0;no_`3Jf]X0Y:BT0;no_`3Jf]X0_kno0:BTY02o_kl0f][J0:BTY02o
_kl0f][J0;no_`2TY:@0_kno0=[JfP2TY:@0_kno0=[JfP2o_kl0Y:BT0;no_`3Jf]X0Y:BT0;no_`3J
f]X0_kno0:BTY02o_kl0f][J0;no_`2TY:@0f][J0;no_`2TY:@0_kno0=[JfP2o_kl0Y:BT0=[JfP2o
_kl0Y:BT0;no_`3Jf]X0_kno0:BTY03Jf]X0dM7A09VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP01
0?ooo`0000@000000?clo02:RXX0VIVIC@3oool02P3clo<0k^k^0>o_k`3`l?00lO7a0?Cdm03gmoL0
n?Sh0?Win@3jn_X20?_kn`0?0??cl`2IVIT0o?cl09VIV@3Jf]X0]KFe08b<S01mOGd0]KFe06][J`2<
S8`0OGem0;Fe]@2<S8`0KVi^0080]KFe0100WYjN07AdM02e]KD0]KFe0:VYZ@0j>SX0EeMG0:ZZZP1U
IFD0UYJF09jNWP1dM7@0LW9b07emO@2e]KD0K6a/<02e]KD02P3>c/h0k^k^0>o_k`3`l?00lO7a0?Cd
m03gmoL0n?Sh0?Win@3jn_X20?_kn`050??cl`2IVIT0o?cl09VIV@3Jf]X0F02e]KD0103Jf]X0VIVI
0?clo02IVIUJ0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0
l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIV@80oooo0300c<c<
0?_kn`3@d=00h^;R0;6a/@3inOT0SXj>09FEU@2>SXh0fMWI0<C4a02PX:00b/[:0?_kn`3EeMD0j>SX
0?ooo`2XZ:P0Ogmo0=OGe`3_knl0d];B0>o_k`2YZJT0mOGe0:VYZ@32`/80Ogmo09bLW03IfMT0no_k
09NGU`3[jn/0d];B0>o_k`24Q8@0QhN70?Ogm`34a<@0Ogmo09bLW03IfMT0c<c<0?_kn`3oool0moOg
0:VYZ@3moOd20?ooo`0C0;^k^`1oOgl0Xj>S0>SXj03<c<`0no_k0=GEe@3Xj>P0oooo0;^k^`1oOgl0
_kno0?_kn`2GUiL0jn_[0:VYZ@3UiND0Ogmo0:b/[0060?ooo`0:0??cl`3^k^h0kno_0?3`l03alO40
m?Cd0?Ogm`3hn?P0nOWi0?[jnP80no_k00D0lo?c09VIV@3lo?`0VIVI0=[JfP040;Fe]@0606][J`2<
S8`0OGem0;Fe]@2c/k<0Jf][0P2e]KD0402NWYh0?Cdm05YJFP1NGUh0XJ6Q03Xj>P1UIFD0]KFe08F5
Q@21PH40WYjN07AdM02YZJT0EeMG02TY:@1jNWX/0;Fe]@0>0000001JFUX0]KFe0;Fe]@3>c/h0k^k^
0>o_k`3`l?00lO7a0?Cdm03gmoL0n?Sh0?Win@3jn_X20?_kn`080??cl`2IVIT0o?cl09VIV@2o_kl0
]KFe0:VYZ@2DU9@20;Fe]@0R0;:b/P1OGel0NGUi06ATI02RXZ80MgMg0;>c/`2b/[80GemO07UiN@1T
I6@0XZ:R07MgM`2][Jd0MgMg0:f][@1gMgL0/[:b07MgM`1JFUX0VIVI0;:b/P1[Jf/0YZJV07MgM`2c
/k<0ZJVY09BDU02YZJT0GEeM061PH02_[jl0T92@0;:b/P80]KFe0100T92@0;:b/P2CTi<0FUYJ09^K
V`2c/k<0I6AT096AT@25QHD0FUYJ07AdM02e]KD0OGem05YJFP1MGEd0[jn_7`2e]KD0102o_kl0VIVI
0?clo02IVIUJ0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0
l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIV@80oooo04D0VIVI
0??cl`1iNGT0][Jf06=SH`3Jf]X0H61P0>;RhP092@T0d];B09VIV@3gmoL0VIVI0?Ogm`2/[:`0d];B
0=oOg`28R8P0oooo08>3P`3Ogml0YJFU0=oOg`1CDe<0jn_[05=CD`2?Shl0oooo06MWI`3Bd]80gmoO
0:FUY@3Ogml0XJ6Q02HV9P3Xj>P0a/K60451@@2?Shl0oooo06MWI`3Bd]80VIVI0?Ogm`3oool0kno_
05=CD`3kno/0oooo0>o_k`0Q8B40kno_07emO@3Bd]80VIVI0?Ogm`2/[:`0d];B0?Ogm`1lO7`0oooo
08>3P`3GemL0YJFU0=oOg`1CDe<0O7al0>o_k`2>SXh01P3oool00`3emOD0l?3`09NGU`030:2PX003
09nOW`2PX:00X:2P0080X:2P00H0T92@0?Gem@2IVIT0o?cl09VIV@3Jf]X20;Fe]@0509>CT`000000
Vi^K08b<S01mOGd00P2e]KD04`1[Jf/0]KFe0;Fe]@2NWYh0M7Ad0;Fe]@2<S8`0Hf=S03Xj>P1GEeL0
ZZZZ06EUI@2FUYH0WYjN07AdM01bLW80SHf=0:n_[`1HF5P0;02e]KD20000000<05YJFP2e]KD0c/k>
0>k^kP3_knl0l?3`0?7al@3dm?@0moOg0?Shn03inOT0n_[j0P3kno/0203clo<0VIVI0?clo02IVIT0
Y:BT0;Fe]@2NWYh0M7Ad0P2e]KD08P2<S8`0OGem08b<S0071`L0VYZJ03Xj>P2b/[80S8b<07emO@2<
S8`01`L709ZJVP0j>SX0YZJV03Xj>P2VYZH0>SXj09RHV01PH600]KFe05eMG@2NWYh0M7Ad09jNWP0j
>SX0/[:b09jNWP1aLG406a/K0:FUY@2<S8`0;Rh^06UYJ@2_[jl20;Fe]@0A06a/K02HV9P05aLG0:b/
[00K6a/0VYZJ03Xj>P2MWId0ADE50;Fe]@25QHD0PH6106QXJ02e]KD0X:2P02h^;P2_[jl07P2e]KD0
102TY:@0VIVI0?clo02IVIUJ0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C
0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIV@80
oooo02@0VIVI0=KFeP2FUYH0/K6a06=SH`3Xj>P0MgMg07inOP0G5aL0d];B09VIV@3gmoL0VIVI0?Og
m`2/[:`0d];B0::RXP1SHf<0Ogmo09NGU`3Ogml0YJFU0=oOg`1CDe<0jn_[05=CD`2XZ:P0P82004U9
B@3Bd]80gmoO0:FUY@3Ogml0UIFE05=CD`3kno/20?ooo`0F0:b/[020P800BDU90=;BdP2IVIT0moOg
0?ooo`3_knl0De=C0?_kn`3oool0kno_05=CD`3gmoL0WIfM0=;BdP2IVIT0moOg0:b/[03Bd]80gmoO
0:FUY@80oooo00L0gmoO0:FUY@3Ogml0De=C08n?S`3oool0^k^k0080oooo00L0000007moO`3oool0
oooo0?Gem@2FUYH0jNWY0080h>3P00@0g]kN0=_Kf`3Hf=P0e=CD0P3AdM40403^k^h0b/[:09VIV@3l
o?`0VIVI0=[JfP2e]KD0X:2P09VIV@2_[jl0I6AT08b<S01mOGd0]KFe09nOW`1/K6`20;Fe]@0@09jN
WP1dM7@0]KFe09FEU@1VIVH0>SXj0:6QX@1JFUX0M7Ad0;>c/`1/K6`0ADE50;6a/@1YJFT0FUYJ0:6Q
XB`0]KFe00L0000005YJFP2e]KD0]KFe0=3@d033`l<0]kNg00P0^KVi00`0]kNg0=oOg`2IVIT0o?cl
09VIV@2o_kl0]KFe09jNWP0m?Cd0FUYJ0861P@2PX:0205MGE`05014A4@2JVYX0>SXj0;:b/P2PX:00
0P1GEeL0500A4A40VYZJ03Xj>P2VYZH0>SXj0:JVYP0j>SX0KFe]04I6AP1JFUX0Jf][09jNWP1dM7@0
WYjN03Xj>P2b/[80WYjN06UYJ@0j>SX0/[:b0P2e]KD0301PH600EeMG05aLG02UYJD0K6a/08J6QP15
ADD0]KFe05EEE@2?Shl0>SXj0;:b/P80]KFe00L0S8b<07=cL`1XJ6P0]KFe0;Bd]01;Bd/0Rh^;01h0
]KFe00@0_kno09VIV@3lo?`0VIVIFP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2I
VIT20?ooo`1509VIV@2PX:00d];B0:RXZ01SHf<0inOW05=CD`3gmoL0;2`/0>o_k`2IVIT0moOg07em
O@3Bd]80TI6A0>o_k`3Ogml0YJFU0?ooo`2HV9P0gmoO07IfMP36a/H02@T90<c<c01PH600VIVI0?Og
m`2ATI40kno_0=oOg`2UYJD0gmoO0:6QX@0V9RH0j>SX0::RXP1PH600VIVI0?Ogm`2ATI40kno_09VI
V@3gmoL0oooo0>o_k`0B4Q8000000451@@3[jn/0De=C0?Ogm`2/[:`0d];B0820P03_knl0TI6A0>o_
k`3gmoL0O7al0?ooo`21PH40gmoO0:FUY@3Ogml0De=C07alO03_knl0SXj>0080oooo0P0000001@1o
Ogl0oooo0?Gem@2FUYH0no_k0080kNg]00@0jNWY0>?Sh`3Lg=`0e]KF0P3@d=001P3oool0b/[:09VI
V@3lo?`0VIVI0=[JfP80]KFe00@0:RXZ0000002KVi/0S8b<0P0000002`071`L0YJFU0;Fe]@2e]KD0
WYjN00H61P0000001`L709ZJVP0j>SX0/[:b00<0]KFe00<0[jn_07QhN02e]KD0<`2e]KD00`3@d=00
UYJF0?_kn`020>g]k@040>WYj@3Ti>@0gMgM0=KFeP80d=3@03`0oooo0<[:bP2IVIT0o?cl09VIV@3J
f]X0]KFe09jNWP1dM7@0]KFe05eMG@2HV9P0M7Ad0:n_[`0O7al0Y:BT01<C4`2UYJD0WYjN07AdM02_
[jl07alO0:BTY00C4a<0TI6A00L71`2ATI40ADE509jNWP1dM7@0]KFe06a/K02NWYh0M7Ad09jNWP0C
4a<0YJFU09jNWP1aLG406a/K0:FUY@1bLW80ADE506a/K02_[jl0X:2P02h^;P1YJFT0V9RH01LG5`2/
[:`06a/K09ZJVP0j>SX0/[:b0;Fe]@1bLW80@4100;Fe]@1XJ6P20;Fe]@0305EEE@21PH40]KFe01d0
]KFe00@0f][J09VIV@3lo?`0VIVIFP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG5
0=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2I
VIT20?ooo`0?09VIV@2LW9`0moOg0;Rh^01SHf<0moOg0::RXP000000H61P0?_kn`0Z:RX0@D5109:B
TP1:BTX0<30`0080oooo02P0DE5A0000002_[jl0gmoO065QH@000000/;2`02XZ:P2d];@0d=3@0000
000`<300no_k05ADE00P8200emOG0:FUY@3Ogml02@T900l?3`3_knl0d=3@0000000`<300oooo09VI
V@3gmoL0oooo0>o_k`1CDe<0no_k0?ooo`3_knl0De=C0?Ogm`2/[:`0d];B0:BTY00M7Ad0<30`0P3o
ool02P1hN7P0000007emO@1DE5@0820P0=OGe`1CDe<0c<c<00000017AdL20?ooo`8000000P3oool0
0`3emOD0UYJF0?_kn`020>g]k@040>[ZjP3Ti>@0g=cL0=KFeP80d=3@00H0oooo0<[:bP2IVIT0o?cl
09VIV@3Jf]Y<0;Fe]@030=3@d02FUYH0no_k0080kNg]00@0jNWY0>?Sh`3Lg=`0e]KF0P3@d=00=`3o
ool0b/[:09VIV@3lo?`0VIVI0:BTY02e]KD0WYjN07AdM02e]KD0K6a/0:n_[`0i>CT0000004E5A@2Y
ZJT0=cLg01lO7`2ZZZX0>CTi00000015ADD0ZJVY04i>CP0M7Ad0OWin01dM7@21PH40]KFe03Ti>@00
0000Ng]k03Xj>P0G5aL0V9RH03Lg=`0O7al0Ti>C07AdM02NWYh01PH600/;2`2ZZZX0K6a/0:n_[`2Y
ZJT0>SXj06UYJ@2_[jl0LW9b0000001VIVH0MWIf0000001^KVh00P2e]KD01`1WIfL0ZZZZ06QXJ02e
]KD0/k>c0451@@2FUYH07P2e]KD0102TY:@0VIVI0?clo02IVIUJ0?ooo`0A09VIV@2:RXX0o?cl0000
002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool0
00040000003lo?`0RXZ:09VIV@80oooo00H0QHF509>CT`3oool0_[jn02h^;P3gmoL40?ooo`040:6Q
X@3gmoL0VIVI0?Ogma00oooo00@0jn_[0:NWY`3_knl0d];B203oool0202IVIT0moOg0?ooo`3_knl0
<S8b07emO@1oOgl0hn?S303oool01@3[jn/0YjNW0>o_k`2YZJT0oOgm00@0oooo00L0Ogmo0?ooo`3o
ool0oooo0?Gem@2FUYH0no_k0080kNg]00@0jNWY0>?Sh`3MgMd0e]KF0P3@d=001P3oool0b/[:09VI
V@3lo?`0VIVI0=[JfT`0]KFe00<0d=3@09JFUP3kno/00P3]kNd0103YjNT0i>CT0=cLg03Fe]H20=3@
d00<0?ooo`3:b/X0VIVI0?clo02IVIT0_kno0;Fe]@2NWYh0?Cdm05YJFP1EEED0/[:b502e]KD00`2V
YZH0MWIf0;Fe]@020;Fe]@030:VYZ@2DU9@0]KFe00<0]KFe00H0H61P05MGE`1ADE40HV9R06YZJP2_
[jl30;Fe]@0<0:f][@0n?Sh0YjNW04e=C@1JFUX0De=C0:n_[`1GEeL0FUYJ055AD@1RHV80/[:b7P2e
]KD0102o_kl0VIVI0?clo02IVIUJ0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0
dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VI
VDd0oooo00<0mOGe09JFUP3kno/00P3]kNd409FEU@070<o?c`3@d=00oooo0<[:bP2IVIT0o?cl09VI
V@1=0?ooo`030?Gem@2FUYH0no_k0080kNg]102EUID20=3@d0060?ooo`3:b/X0VIVI0?clo02IVIT0
f][JF02e]KD0103Jf]X0VIVI0?clo02IVIUJ0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf
]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0
RXZ:09VIVDd0oooo00D0mOGe09JFUP3kno/0kNg]0?KfmP030<[:bP080;Fe]@3?cll0d=3@0?ooo`3:
b/X0VIVI0?clo02IVIU=0?ooo`050?Gem@2FUYH0no_k0>g]k@3fm_H00`3:b/X0I@2e]KD0clo?0=3@
d03oool0b/[:09VIV@3lo?`0VIVI0;no_`1oOgl0]KFe09ZJVP1oOgl0VYZJ0;Fe]@2JVYX0Ogmo0;Fe
]@2JVYX0Ogmo09ZJVP2e]KD0VYZJ07moO`2e]KD0VYZJ07moO`2JVYX0]KFe09ZJVP1oOgl0]KFe09ZJ
VP1oOgl0VYZJ0;Fe]@2JVYX0Ogmo0;Fe]@2JVYX0Ogmo09ZJVP2e]KD0VYZJ07moO`2e]KD0VYZJ07mo
O`2JVYX0]KFe09ZJVP1oOgl0]KFe09ZJVP1oOgl0VYZJ0;Fe]@2JVYX0Ogmo0;Fe]@2JVYX0Ogmo09ZJ
VP2e]KD0VYZJ07moO`2e]KD0VYZJ07moO`2JVYX0]KFe09ZJVP1oOgl0]KFe09ZJVP1oOgl0VYZJ0;Fe
]@2JVYX0Ogmo09ZJVP2e]KD0Ogmo09ZJVP2e]KD0VYZJ07moO`2JVYX0]KFe07moO`2JVYX0]KFe09ZJ
VP1oOgl0VYZJ0;Fe]@1oOgl0_kno09VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bL
W02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@0
00000?clo02:RXX0VIVIC@3oool0103emOD0UYJF0?_kn`3]kNd40?ooo`080=KFeP3@d=00d=3@0?oo
o`3:b/X0VIVI0?clo02IVIT20?ooo`0507QhN0000000000004M7A`36a/H00P0000001P0?3`l0moOg
0?ooo`3oool0gmoO0:FUY@80oooo00`0kno_05=CD`3<c<`0000005YJFP3oool0moOg02l_;`3clo<0
:RXZ0000002LW9``0?ooo`040?Gem@2FUYH0no_k0>g]k@@0oooo00P0e]KF0<o?c`3@d=00oooo0<[:
bP2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00001000
0000o?cl08Z:RP2IVIT90?ooo`040<C4a01oOgl0Ng]k0:6QXBH0oooo00<0VIVI0?Ogm`3oool05`3o
ool01@3emOD0UYJF0?_kn`3]kNd0m_Kf00<0b/[:00P0][Jf0=3@d03@d=00oooo0<[:bP2IVIT0o?cl
09VIV@80oooo00T0kno_08:2PP3Xj>P0oooo0<K6aP2_[jl0oooo0>3Ph02IVIT00P3oool0403Ogml0
YJFU0?ooo`3oool0kno_05=CD`25QHD0moOg0:BTY033`l<0gmoO0:FUY@3@d=00clo?08R8R01iNGT/
0?ooo`0907moO`3oool0oooo0?ooo`3emOD0UYJF0?_kn`3]kNd0m_Kf00<0b/[:00P0]KFe0=3@d03@
d=00oooo0<[:bP2IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?cl
o0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040
oooo00001`000000o?cl08Z:RP2IVIT0oooo0?_kn`2YZJT00`3oool07036a/H0WinO05=CD`3Ng]h0
000000l?3`2=SHd0oooo055AD@000000[jn_0=oOg`1^KVh0:RXZ08n?S`3_knl0De=C0?_kn`3gmoL0
7AdM02/[:`0M7Ad0bLW905=CD`3[jn/04Q8B0000001oOgl30?ooo`0C0:NWY`00000000000<O7a`3o
ool0DE5A0000003IfMT0gmoO05=CD`3[jn/0De=C0>_[j`1CDe<0Shn?018B4P000000alO70?ooo`02
03`l?00=09:BTP3gmoL07AdM09RHV00M7Ad0:b/[01dM7@3EeMD07AdM0451@@2EUID0moOg055AD@06
0?ooo`030?Gem@2FUYH0no_k0080kNg]102EUID20=3@d0050?ooo`3:b/X0VIVI0?clo02IVIT00`3o
ool01@3kno/09RHV0>SXj036a/H0[jn_0080oooo0180UYJF0?ooo`3oool0gmoO00T92@0000002@T9
0=SHf01CDe<0Shn?0?ooo`2k^k/0];Bd0=oOg`2UYJD0gmoO00T92@1PH60]0?ooo`8000000P3oool0
0`3emOD0UYJF0?_kn`020>g]k@@0UIFE0P3@d=001@3oool0b/[:09VIV@3lo?`0VIVI0080oooo00h0
gmoO00T92@000000<30`0=oOg`2SXj<0E5AD0000000P8200moOg08Z:RP11@D40UIFE0?Ogm`80oooo
00d0gmoO0:FUY@2RXZ8000000;Bd]03oool0RXZ:0451@@2g]kL0000004M7A`3oool0a/K600800000
00<0Ogmo0?ooo`3oool0=P3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002L
W9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool00004
0000003lo?`0RXZ:09VIV@80oooo0240Ti>C0?ooo`3oool0oOgm05eMG@3Hf=P0De=C08f=S@3_knl0
moOg06QXJ03Ogml0R8R80?ooo`23Ph<0gmoO01/K6`3Ti>@09RHV0=SHf01CDe<0no_k0<K6aP1PH600
h^;R00T92@3Hf=P0De=C08Z:RP3Bd]80oooo08>3P`3gmoL00P3oool08P23Ph<0oooo0?Ogm`1FEUH0
gmoO08R8R03gmoL0O7al0=oOg`1CDe<0jn_[05=CD`3[jn/0De=C07alO03_knl0moOg07alO03Ogml0
R8R80=oOg`1fMWH0gmoO05=CD`3Ng]h0H61P0>;RhP092@T0f=SH05=CD`3gmoL0VIVI0=OGe`28R8P6
0?ooo`040?Gem@2FUYH0no_k0>g]k@@0oooo00P0e]KF0<o?c`3@d=00oooo0<[:bP2IVIT0o?cl09VI
V@@0oooo00@0moOg08f=S@36a/H0[jn_0P3oool04`2FUYH0oooo0?ooo`3Ogml0YJFU0?ooo`36a/H0
S8b<05=CD`1lO7`0kno_08j>SP3Bd]80gmoO0:FUY@2RXZ80alO70?Ogm`1lO7`0;03oool200000006
07moO`3oool0mOGe09JFUP3kno/0kNg]103oool0203Fe]H0clo?0=3@d03oool0b/[:09VIV@3lo?`0
VIVI0P3oool03P3Ogml0YJFU0?ooo`3oool0gmoO07IfMP1PH600oooo0=3@d02?Shl0VIVI0?Ogm`2I
VIT0moOg0P3oool04@3Ogml0UIFE024Q8@3clo<09RHV0=;BdP2IVIT0moOg09>CT`3oool0^k^k0;Bd
]036a/H0[jn_0?ooo`23Ph<0moOg03L0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?cl
o0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040
oooo000010000000o?cl08Z:RP2IVIT20?ooo`080:f][@17AdL0Ogmo05eMG@1jNWX0kNg]05=CD`2?
Shl20?ooo`0D09JFUP2RXZ80Hf=S07moO`2GUiL0gmoO05=CD`3kno/0XZ:R0;Ng]`1CDe<0no_k0>;R
hP1gMgL0OWin01LG5`3Hf=P0De=C09FEU@3gmoL40?ooo`030>_[j`2IVIT0oooo0080oooo00/0a/K6
0:n_[`3oool0UYJF0=oOg`1CDe<0jn_[05=CD`3[jn/0De=C0:BTY0020?ooo`0A09JFUP3Ogml0YJFU
0?Win@29RHT0gmoO05=CD`3/k>`0MgMg07inOP0G5aL0f=SH05=CD`3gmoL0VIVI0;jn_P2_[jl00P3o
ool01`000000Ogmo0?ooo`3oool0mOGe09JFUP3kno/00P3]kNd02`3YjNT0hn?S0=cLg03Fe]H0clo?
0=3@d03oool0b/[:09VIV@3lo?`0VIVI0080oooo00T0d=3@0=OGe`3moOd0SXj>0<K6aP2_[jl0oooo
0>3Ph02IVIT00P3oool0403Ogml0YJFU0?ooo`3Bd]80T92@05=CD`3Shn<0Ogmo0:>SX`3moOd0VIVI
069RHP3inOT0U9BD07moO`3Shn</0?ooo`070000001oOgl0oooo0?ooo`3emOD0UYJF0?_kn`020>g]
k@0;0>WYj@3Shn<0g=cL0=KFeP3?cll0d=3@0?ooo`3:b/X0VIVI0?clo02IVIT00P3oool0603Ogml0
YJFU0?ooo`3oool0gmoO09fMW@28R8P0Kfm_05aLG03IfMT0VIVI0?Ogm`28R8P0Ng]k07moO`2f][H0
gmoO08R8R01PH600oooo07QhN02d];@0VIVI0?Ogm`80oooo00L0aLG50:6QX@36a/H0[jn_0?ooo`2c
/k<0`l?303L0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f
][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl
08Z:RP2IVIT20?ooo`0P0?_kn`0/;2`0kno_09VIV@3gmoL0kno_05=CD`2:RXX0d];B0?Ogm`1XJ6P0
gmoO0:FUY@3oool0V9RH0=oOg`0K6a/0i>CT02HV9P3Hf=P06a/K0>SXj03Ogml0De=C0?Ogm`0/;2`0
inOW05=CD`2:RXX0d];B0?ooo`21PH420?ooo`030=oOg`2UYJD0oooo0080oooo01h0gmoO08R8R03g
moL0O7al0=oOg`0K6a/0c<c<00T92@3<c<`0H61P07emO@3Bd]80moOg07alO03Ogml0YJFU0?_kn`2G
UiL0gmoO05=CD`3[jn/0De=C0?Ogm`0/;2`0inOW05=CD`3gmoL0VIVI0=OGe`28R8P20?ooo`800000
00D0Ogmo0?ooo`3emOD0UYJF0?_kn`020>g]k@0;0>WYj@3Shn<0g=cL0=KFeP3?cll0d=3@0?ooo`3:
b/X0VIVI0?clo02IVIT00P3oool01@3kno/0:RXZ0000002_[jl0a/K60080000000/02@T90>SXj03o
ool0oooo0=oOg`092@T0000000T92@3Hf=P0De=C0?_kn`030?ooo`030?Ogm`2ZZZX0oooo03<0oooo
00<0mOGe09JFUP3kno/00P3]kNd02`3YjNT0hn?S0=cLg03Fe]H0clo?0=3@d03oool0b/[:09VIV@3l
o?`0VIVI0080oooo0240gmoO0:FUY@3oool0oooo0=oOg`1fMWH0Shn?0?ooo`2>SXh0d];B09VIV@3g
moL0VIVI0?Ogm`3oool0Ph>30=OGe`2EUID08B4Q0??cl`0V9RH0d];B09VIV@3gmoL0oooo0::RXP1J
FUX0oooo0<K6aP2_[jl0oooo0;^k^`2d];@0=`3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000040000003lo?`0RXZ:09VIV@<0oooo01l0Zj^[096AT@2ZZZX0oooo0>o_k`1CDe<0
k>c/07moO`27QhL0a/K60?ooo`2XZ:P0Ogmo0=OGe`3Ogml0D51@092@T037alL0moOg0:JVYP2EUID0
moOg0=3@d01oOgl0[jn_0?Ogm`2YZJT0mOGe08R8R01oOgl0_kno0080oooo02<0no_k08R8R03oool0
no_k0:f][@3oool0Z:RX07moO`3Shn<0kno_0;Jf]P2DU9@0emOG09BDU03IfMT0d];B0:VYZ@1oOgl0
hn?S0>o_k`3Bd]80oOgm0<[:bP2IVIT0:BTY0<_;b`3@d=00Ogmo0:n_[`3;bl/0:BTY09bLW03:b/X0
no_k0:RXZ0020?ooo`070000001oOgl0oooo0?ooo`3emOD0UYJF0??cl`020?KfmP040?Cdm03alO40
kNg]0>[ZjP80inOW00D0jn_[0<[:bP2IVIT0o?cl09VIV@1=0?ooo`030?Gem@2FUYH0lo?c0080m_Kf
00@0m?Cd0?7al@3]kNd0j^[Z0P3WinL01@3[jn/0b/[:09VIV@3lo?`0VIVI0080oooo0240gmoO0:FU
Y@3oool0oooo0>o_k`3AdM40ZJVY07moO`2SXj<0oOgm065QH@2LW9`0UiNG0?Ogm`3oool0SXj>0=OG
e`2UYJD0d=3@07moO`37alL0oOgm065QH@2LW9`0alO70?ooo`2ATI40kno_0<K6aP2_[jl0oooo09NG
U`3Ti>@0=`3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf
]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003lo?`0
RXZ:09VIV@@0oooo00H0N7Qh0<O7a`3oool0kno_05=CD`3kno/70?ooo`030=oOg`1CDe<0no_k00P0
oooo00<0kno_05=CD`3kno/01P3oool0102WYjL0000000000037alLB0?ooo`030?Ogm`1NGUh0no_k
00<0oooo00D0no_k05iNGP3gmoL0VIVI0?Ogm`070?ooo`030?Gem@3moOd0YJFU00<0^;Rh00<0]KFe
0;Rh^02h^;P00P2h^;P01@2CTi<0mOGe09VIV@3lo?`0VIVI04d0oooo00<0mOGe0?gmo@2UYJD00`2h
^;P00`2e]KD0^;Rh0;Rh^0020;Rh^00509>CT`3emOD0VIVI0?clo02IVIT00P3oool01P3Ogml0YJFU
0?ooo`3oool0gmoO0:FUY@@0oooo00<0ZZZZ0?Ogm`1hN7P00P0000000`1]KFd0gmoO0:FUY@040?oo
o`070:ZZZP3gmoL0^k^k0000001JFUX0oooo0<K6aP020000000306e]K@3oool0oooo03H0oooo00<0
VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40
j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVIU=0?ooo`03
0?Gem@32`/80^KVi00P0_Kfm00H0]KFe0=kNgP2IVIT0o?cl09VIV@3ooom<0<k>cP030?Gem@32`/80
^KVi00P0_Kfm00D0]KFe0=kNgP2IVIT0o?cl09VIV@1J0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140
VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0
oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVIC@3oool00`3emOD0UYJF0>KViP080<k>cP05
0?ooo`3:b/X0VIVI0?clo02IVIT0C@3oool00`3emOD0UYJF0>KViP080<k>cP050?ooo`3:b/X0VIVI
0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003l
o?`0RXZ:09VIVDd0oooo00<0mOGe09JFUP3Vi^H0203>c/h01@3oool0b/[:09VIV@3lo?`0VIVI04d0
oooo00<0mOGe09JFUP3Vi^H0203>c/h01@3oool0b/[:09VIV@3lo?`0VIVI05X0oooo00<0VIVI0?cl
o02IVIT0FP3oool04@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0040oooo000010000000o?cl08Z:RP2IVITc0?ooo`030>o_k`1C
De<0moOg01L0oooo00<0mOGe09JFUP3Vi^H0203>c/h01@3oool0b/[:09VIV@3lo?`0VIVI00<0oooo
00<0_;bl0<O7a`3oool01P3oool00`2IVIT0moOg0?ooo`0b0?ooo`070?gmo@2>SXh0Ogmo06YZJP3X
j>P0a/K60<?3``050?ooo`030?Gem@2FUYH0i^KV00P0c/k>00D0oooo0<[:bP2IVIT0o?cl09VIV@1J
0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI
0P3oool0A@2IVIT0moOg0?ooo`1mOGd0d];B0;^k^`000000AdM70=;BdP3Ogml0YJFU0=oOg`2UYJD0
gmoO0:FUY@2RXZ800000030`<03oool0VIVI0?Ogm`3_knl0De=C0?Ogm`1hN7P0000007moO`3gmoL0
3Ph>05MGE`0M7Ad0bLW905=CD`3kno/0oooo0?gmo@1DE5@0000004M7A`3oool0d=3@0000001JFUX0
oooo0=oOg`2UYJD0gmoO0:FUY@3Ogml0UIFE04Q8B00B4Q80AdM70?ooo`2k^k/0000004M7A`3Bd]80
moOg02l_;`3_knl03Ph>05MGE`1XJ6P0moOg01dM7@2<S8`0De=C0<c<c0060?ooo`050?Gem@2FUYH0
i^KV0<k>cP0n?Sh00`3>c/h0200n?Sh0c/k>0<k>cP3oool0b/[:09VIV@3lo?`0VIVI0`3oool00`2Y
ZJT0no_k0:VYZ@030?ooo`100<K6aP2OWil0De=C08n?S`0B4Q80AdM70?ooo`3Ogml0YJFU0?_kn`2G
UiL0moOg00h>3P1GEeL07AdM0<W9b@1^KVh0:RXZ08n?S`3oool0XZ:R0000000`<300oooo08Z:RP11
@D40UIFE0?Ogm`1hN7P0000000l?3`3clo<07AdM02/[:`0M7Ad0bLW905=CD`3gmoL0oooo0=oOg`09
2@T00000030`<03Ogml0De=C0:>SX`000000820P0?Ogm`2:RXX0@D510=OGe`2EUID0De=C0?Win@1M
GEd0j>SX0<K6aP000000>SXj0;Bd]03moOd0Hf=S0?Ogm`@0oooo00D0mOGe09JFUP3Vi^H0c/k>03hn
?P030<k>cP0803hn?P3>c/h0c/k>0?ooo`3:b/X0VIVI0?clo02IVIT20?ooo`0=0=oOg`2UYJD0oooo
0?ooo`3Ogml0YJFU0::RXP000000FUYJ0?ooo`2:RXX0@D510;Ng]`02000000050<O7a`36a/H00000
0000001oOgl0A03oool00`2IVIT0o?cl09VIV@1J0?ooo`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000003l
o?`0RXZ:09VIV@80oooo04D0VIVI0?Ogm`3_knl0DU9B0=;BdP2<S8`0lo?c08F5Q@3Bd]80gmoO0:FU
Y@3Ogml0YJFU0=oOg`25QHD0F5QH0?Ogm`3MgMd0alO709VIV@3gmoL0kno_05=CD`3Shn<0T92@0?oo
o`31`L40`/;209NGU`3Ng]h05aLG0=SHf01CDe<0no_k0?ooo`2VYZH0_kno0?ooo`34a<@0alO708V9
R@3gmoL0Y:BT0<?3``3Ogml0YJFU0=oOg`2UYJD0gmoO09FEU@19BDT0moOg0:BTY033`l<0S8b<0??c
l`25QHD0d];B0=oOg`2UYJD0a/K609NGU`3Ng]h0M7Ad0=oOg`1CDe<0jn_[05=CD`25QHD00P3oool0
501oOgl0oooo0?ooo`3oool0mOGe09JFUP3Xj>P0RHV903hn?P1RHV80dM7A069RHP0n?Sh0[Jf]0=7A
d@3oool0b/[:09VIV@3lo?`0VIVI0P3oool0103[jn/0VIVI0?ooo`2>SXh207moO`0Z07emO@1?Cdl0
hN7Q05=CD`2@T900moOg0:BTY033`l<0gmoO0:FUY@3kno/0UiNG0<K6aP2GUiL0g]kN01LG5`3Hf=P0
=cLg0>o_k`1TI6@0^;Rh05QHF03gmoL0gMgM0<O7a`2IVIT0moOg09VIV@3Shn<0T92@0?ooo`3Rh^80
/;2`03Lg=`3Zj^X05aLG0=SHf01CDe<0moOg0?ooo`3Ogml0YJFU0P3oool05@3Ogml0De=C0;bl_03_
knl0K6a/0:6QX@2IVIT0moOg0=oOg`2EUID0De=C0?Win@1MGEd0f=SH04I6AP3gmoL0XZ:R0;Bd]03o
ool0SHf=0=3@d0040?ooo`0@0?Gem@2FUYH0j>SX08V9R@0n?Sh0HV9R0=7Ad@1RHV80?Shn0:f][@3A
dM40oooo0<[:bP2IVIT0o?cl09VIV@80oooo00d0gmoO0:FUY@3oool0oooo0=oOg`2>SXh0@4100?Og
m`2TY:@0`l?309VIV@3gmoL0bLW90080oooo00H0UiNG0<K6aP2_[jl0oooo09NGU`3Ti>A30?ooo`03
09VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3oool0
3`2IVIT0moOg0861P@2EUID0d];B09VIV@3gmoL0[:b/0=;BdP3Ogml0YJFU0=oOg`2UYJD0gmoO07If
MP0300000007061PH02IVIT0moOg0>o_k`1CDe<0emOG0:FUY@030?ooo`0907QhN00000002@T90=SH
f01CDe<0no_k0?ooo`2IVIT0moOg00<0oooo01d0Ti>C0?ooo`2k^k/0];Bd0=oOg`2UYJD0gmoO0:FU
Y@3Ogml0UIFE079bLP3oool0^k^k0;Bd]02IVIT0moOg0:b/[03Bd]80gmoO0:FUY@3oool0N7Qh0000
001]KFd0gmoO05=CD`3[jn/0De=C08n?S`020?ooo`8000000P3oool00`3emOD0UYJF0>g]k@020=_K
f`<0?Shn0`3Kfm/01@3oool0b/[:09VIV@3lo?`0VIVI0080oooo01/0gmoO0:FUY@3oool0d=3@08n?
S`3oool0N7Qh0;Bd]03_knl0De=C0:BTY03oool0^k^k0;Bd]03Ogml0YJFU0?_kn`2GUiL0oooo07Qh
N00000002@T90=SHf01CDe<0no_k0::RXP2HV9P00`0000001P1PH600VIVI0?Ogm`2IVIT0emOG0:FU
Y@<0oooo00T0gmoO00T92@092@T0f=SH05=CD`3gmoL0oooo0=oOg`2UYJD00P3oool05@3Ogml0De=C
07AdM0000000[jn_0?ooo`2IVIT0moOg0=oOg`2EUID0De=C0?Win@1MGEd0c<c<061PH03oool0^k^k
0;Bd]03oool0XZ:R0<O7a`040?ooo`030?Gem@2FUYH0kNg]0080fm_K0`0n?Sh30=_Kf`050?ooo`3:
b/X0VIVI0?clo02IVIT00P3oool0303Ogml02@T90000001]KFd0gmoO08R8R01PH600oooo0;^k^`2d
];@0VIVI0?Ogm`<0oooo00H0UYJF0<K6aP2_[jl0oooo0;^k^`2d];A30?ooo`0309VIV@3lo?`0VIVI
05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf
0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI0P3oool0702IVIT0Z:RX0::R
XP2/[:`0d];B09VIV@3gmoL0[:b/0=;BdP3Ogml0Jf][06=SH`1gMgL0Hf=S0;bl_01jNWX0Ng]k0820
P037alL0RHV908j>SP2_[jl0De=C0??cl`1jNWX0Ogmo0820P03_knl207]kN`0O04I6AP3[jn/0De=C
0?_kn`3oool0UIFE0??cl`3oool0m_Kf0?7al@2XZ:P0MgMg06YZJP3Xj>P0gmoO06][J`1SHf<0MgMg
06=SH`2d];@0A4A409>CT`1ZJVX0j>SX09VIV@3gmoL0[:b/0=3@d02IVIT0HV9R0>_[j`0207]kN`06
09fMW@2IVIT0:BTY0;^k^`1CDe<0Y:BT0P3oool2000000050;no_`3oool0mOGe09JFUP3clo<00P3X
j>P00`2o_kl0?Shn0;no_`030>SXj0050?ooo`3:b/X0VIVI0?clo02IVIT00P3oool04`3kno/0Vi^K
0?ooo`3moOd0DU9B0<W9b@23Ph<0no_k0>o_k`1CDe<0SHf=09>CT`1ZJVX0j>SX0=oOg`1QHF40Ng]k
09jNWP3_knl00P1kNg/06`16ATH0jn_[03@d=024Q8@0FUYJ0>_[j`1jNWX0Ng]k0820P035aLD0HF5Q
09bLW02GUiL0lo?c07YjNP1oOgl0F5QH0<_;b`1jNWX0Ng]k04I6AP3[jn/0De=C0?Ogm`3oool0gmoO
0:FUY@020?ooo`0E0=oOg`1CDe<0Vi^K07moO`1ZJVX0i^KV065QH@2LW9`0fm_K09FEU@15ADD0P820
04I6AP3inOT0MWIf06UYJ@1RHV80];Bd0?knoP22PX80j>SX00@0oooo00<0mOGe09JFUP3clo<00P3X
j>P00`2o_kl0?Shn0;no_`030>SXj0050?ooo`3:b/X0VIVI0?clo02IVIT00P3oool05@3Ogml0YJFU
0?ooo`2>SXh0emOG09fMW@1QHF40MgMg06YZJP3Vi^H0HF5Q09bLW03moOd0d=3@07]kN`2c/k<0a/K6
0:n_[`3oool0/k>c0<?3``130?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`00000
09bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`00
00@000000?clo02:RXX0VIVI0P3oool01@1hN7P0H61P0?ooo`2/[:`0d];B01X0oooo00T0kno_05=C
D`3kno/0oooo0;^k^`2?Shl0oooo08j>SP3Bd]804P3oool00`3Ogml0YJFU0?ooo`030?ooo`030=oO
g`1CDe<0no_k00@0oooo00L000000?ooo`3oool0oooo0?Gem@2FUYH0nOWi00P0m?Cd00D0oooo0<[:
bP2IVIT0o?cl09VIV@030?ooo`0:0:RXZ03oool0oooo0=oOg`1XJ6P0kno_0?ooo`3_knl0De=C0?_k
n`<0oooo00<0gmoO0:FUY@3oool01@3oool00`3_knl0De=C0?_kn`060?ooo`0309VIV@3gmoL0oooo
00P0oooo00H0kno_05=CD`3gmoL0oooo0=oOg`2UYJD80?ooo`0309VIV@3gmoL0oooo00T0oooo00<0
h^;R08n?S`3oool0103oool00`3emOD0UYJF0?Win@080?Cdm0050?ooo`3:b/X0VIVI0?clo02IVIT0
0P3oool01P3Ogml0YJFU0?ooo`23Ph<0emOG0:FUY@@0oooo00/0VIVI0?Ogm`2CTi<0oooo0?Ogm`2=
SHd0a/K60:n_[`3oool0Ph>30?Ogm`130?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP01
0?ooo`0000@000000?clo02:RXX0VIVI0P3oool01@1mOGd0d];B0?ooo`2/[:`0d];B0100oooo00<0
kno_05=CD`3gmoL01`3oool0203_knl0De=C0?_kn`3oool0oOgm05ADE0000000AdM74`3oool00`3g
moL0ZZZZ0?ooo`030?ooo`050?Ogm`1NGUh0jn_[05=CD`3kno/01P3oool00`3emOD0UYJF0?gmo@08
0?clo0050?ooo`3:b/X0VIVI0?clo02IVIT00`3oool01@3@d=00Shn?0?ooo`3oool0Ph>30080oooo
00<0kno_05=CD`3kno/00`3oool00`3Ogml0YJFU0?ooo`050?ooo`030>o_k`1CDe<0no_k00H0oooo
00@0ZZZZ0?Ogm`2IVIT0moOg1`3oool01P3_knl0De=C0?Ogm`3oool0gmoO0:FUY@80oooo00<0gmoO
05=CD`3kno/00`3oool0102ZZZX0moOg0=oOg`2UYJD80?ooo`030:ZZZP3gmoL0oooo00@0oooo00<0
mOGe09JFUP3moOd0203lo?`01@3oool0b/[:09VIV@3lo?`0VIVI0080oooo00H0gmoO00T92@000000
KFe]0=oOg`2UYJD40?ooo`030:ZZZP3gmoL0^k^k0080000000D0fMWI0<K6aP000000000006e]K@14
0?ooo`0309VIV@3lo?`0VIVI05X0oooo0140VIVI08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000@000000?clo02:RXX0VIVI
C@3oool00`3emOD0UYJF0?Gem@080?ooo`050>_[j`3:b/X0VIVI0?clo02IVIT0C@3oool00`3emOD0
UYJF0?Gem@080?ooo`050>_[j`3:b/X0VIVI0?clo02IVIT0FP3oool00`2IVIT0o?cl09VIV@1J0?oo
o`0A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0
o?cl0?gmo@3no_h00@3oool000040000003lo?`0RXZ:09VIVDd0oooo00<0mOGe0=[JfP2TY:@01`2e
]KD01P2d];@0Shn?0>c/k02IVIT0o?cl09VIVDd0oooo00<0mOGe0=[JfP2TY:@01`2e]KD01P2d];@0
Shn?0>c/k02IVIT0o?cl09VIVEX0oooo00<0VIVI0?clo02IVIT0FP3oool04@2IVIT0RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo
000010000000o?cl08Z:RP2IVIUJ0<c<c00309VIV@3lo?`0VIVI05X0c<c<00<0VIVI0?clo02IVIT0
FP3<c<`00`2IVIT0o?cl09VIV@1J0<c<c00A09VIV@2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0
dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000050000003lo?`0RXZ:09VI
V@3moOd0o`3lo?a`0?clo00B0?gmo@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3Q
hN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:09VIV@3moOd0
o`3lo?a`0?clo00B0?gmo@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX
0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:09VIV@3moOd0o`3lo?a`
0?clo00B0?gmo@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03f
m_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:09VIV@3moOd0o`3lo?a`0?clo00B
0?gmo@2IVIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j
0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:09VIV@3moOd0o`3lo?a`0?clo00B0?gmo@2I
VIT0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03m
oOd0o_kn0@3oool000040000003lo?`0RXZ:09VIVGT0o?cln03oool04@2IVIT0RXZ:0?clo0000000
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
10000000o?cl08Z:RP2IVIUh0?clo00309VIV@2][Jd0[Jf]06P0[Jf]00<0WYjN0:j^[P2][Jd0RP2]
[Jd04P2^[Zh0_[jn08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf
0?[jnP3lo?`0oOgm0?knoP40oooo000010000000o?cl08Z:RP2IVIUh0?clo00309VIV@3:b/X0b/[:
06P0b/[:00<0Xj>S0<?3``3:b/X0RP3:b/X04P33`l<0hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf
0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo000010000000o?cl08Z:
RP2IVIUh0?clo00309VIV@3De=@0e=CD06P0e=CD00<0Xj>S0<?3``3De=@0RP3De=@04P33`l<0hn?S
08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm
0?knoP40oooo000010000000o?cl08Z:RP2IVIUh0?clo00309VIV@3Ogml0gmoO01<0gmoO00L00000
0=oOg`3Ogml0gmoO0000001_Kfl0gmoO00<000000P3Ogml3000000@0gmoO00X0Kfm_000000000000
gmoO06m_K`000000gmoO06m_K`000000gmoO0P0000000`3Ogml0g]kN0=kNgP020=kNgP@000000`3N
g]h40000000<06m_K`000000Kfm_0000001_Kfl0gmoO0000001_Kfl0000006m_K`3Ogml000000P3O
gml2000001D0gmoO00<0g]kN0:>SX`33`l<06P3Ogml010000000Kfm_0=oOg`3Ogml2000000040=oO
g`000000Kfm_0=oOg`<00000103Ogml020000000gmoO0=oOg`3Ogml0000006m_K`3Ng]h0Kfm_0`00
00002@3Ng]h0Kfm_0000001_Kfl000000=kNgP1_Kfl000000=kNgP03000000090=kNgP1_Kfl00000
06m_K`000000Kfm_0000003Ogml0Kfm_00<0000000H0gmoO000000000000Kfm_0000003Ogml30000
0080gmoO0`000000101_Kfl0gmoO0=oOg`3Ogml30000000406m_K`3Ogml0gmoO06m_K`8000000P3O
gml3000000040=oOg`1_Kfl000000=oOg`8000005P3Ogml04`3Ng]h0`l?30>?Sh`2:RXX0o?cl0000
002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool0
00040000003lo?`0RXZ:09VIVAX0o?cl0`1nOWh30?clo0<0OWin00<0_Kfm07inOP2m_Kd00P1nOWh0
0`3lo?`0OWin0;fm_@030?clo00607inOP2m_Kd0OWin0;fm_@3lo?`0OWin103lo?`00`1nOWh0o?cl
0?clo0020?clo00:0;fm_@1nOWh0OWin07inOP2m_Kd0OWin0?clo02m_Kd0OWin0?clo0<0OWin0P3l
o?`01@1nOWh0_Kfm07inOP2m_Kd0o?cl00<0OWin00L0o?cl07inOP2m_Kd0o?cl07inOP2m_Kd0o?cl
0080OWin6`3lo?`00`2IVIT0i>CT0>CTi00C0>CTi00?03Xj>P1`L700L71`071`L0000000[:b/0000
001bLW80i>CT0000003Ti>@00000079bLP3Ti>@0000000@0i>CT00d000000>CTi01bLW800000079b
LP000000i>CT079bLP000000[:b/07AdM00h>3P0M7Ad00<0i>CT00@0LW9b0000001bLW800000103T
i>@00`000000LW9b0>CTi0020>CTi00@0000001bLW800000079bLP3Ti>@00000079bLP000000>3Ph
0000002/[:`0i>CT07AdM01bLW8000000:b/[1D0i>CT00<0Xj>S0<?3``3Ti>@06@3Ti>@010000000
LW9b0>CTi03Ti>@2000000080>CTi0000000LW9b0000001bLW80i>CT07AdM02/[:`30>CTi0080000
003Ti>@00000079bLP000000LW9b0>CTi00000020>CTi00<0000003Ti>@000000>CTi01bLW800000
0>CTi01bLW800000079bLP000000i>CT0P1dM7@04`1bLW800000079bLP000000LW9b0000003Ti>@0
00000>CTi01bLW800000079bLP000000i>CT079bLP000000LW9b0000003Ti>@00P1dM7@01@1bLW80
00000>CTi0000000LW9b00<0i>CT00@00000079bLP3Ti>@000000P3Ti>@03@000000i>CT079bLP00
0000LW9b0000003Ti>@0LW9b0000001bLW80000003Ph>01dM7@0603Ti>@04P33`l<0hn?S08Z:RP3l
o?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40
oooo000010000000o?cl08Z:RP2IVITJ0?clo0050000003lo?`0OWin0000003lo?`00P0000001@1n
OWh0000007inOP000000OWin0080000000<0OWin0000003lo?`00`3lo?`01P000000OWin0000001n
OWh0o?cl000000@0o?cl00<000000?clo03lo?`00P3lo?`030000000o?cl000000000000OWin0000
003lo?`0OWin0000001nOWh000000?clo080000000D0OWin0000003lo?`0000007inOP020000000<
0?clo0000000OWin0000001nOWh0o?cl0000001nOWh0000007inOP000000OWin6P3lo?`00`2IVIT0
k>c/0>c/k00C0>c/k00307IfMP000000k>c/0080000000/0k>c/0000001fMWH0k>c/0000003/k>`0
000007IfMP3/k>`000000>c/k0020000000=0>c/k0000000k>c/07IfMP000000MWIf0000003/k>`0
MWIf0000003/k>`0000007IfMP040>c/k00307IfMP0000000000008000000`3/k>`00`000000MWIf
0>c/k0020>c/k0070000001fMWH0000007IfMP3/k>`0000007IfMP02000000@0k>c/00<000000>c/
k03/k>`05@3/k>`00`2SXj<0`l?30>c/k00I0>c/k0040000001fMWH0k>c/0>c/k080000000<0k>c/
0000001fMWH010000000101fMWH0k>c/0>c/k03/k>`50000000307IfMP3/k>`0MWIf00<0000000T0
k>c/0000003/k>`0MWIf0000003/k>`0MWIf0000001fMWH010000000201fMWH0000007IfMP000000
MWIf0000003/k>`0MWIf0`0000001`1fMWH000000>c/k01fMWH0000007IfMP000000103/k>`30000
000807IfMP3/k>`0k>c/0>c/k0000000MWIf0>c/k00000020>c/k00<0000003/k>`0MWIf0000001f
MWH000000>c/k01fMWH0000007IfMP000000MWIf6@3/k>`04P33`l<0hn?S08Z:RP3lo?`0000009bL
W02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00001000
0000o?cl08Z:RP2IVITJ0?clo0050000003lo?`0OWin0000003lo?`00P0000001@3lo?`0000007in
OP000000OWin0080000000<0OWin0000003lo?`00`3lo?`01P000000OWin0000001nOWh0o?cl0000
00@0o?cl00<000000?clo03lo?`00P3lo?`02`000000o?cl07inOP000000OWin0000003lo?`0OWin
0000001nOWh0000000<0o?cl00D0OWin0000003lo?`0000007inOP020000000;0?clo0000000OWin
0000001nOWh0o?cl0000001nOWh0o?cl07inOP0000006`3lo?`00`2IVIT0k>c/0>c/k00C0>c/k003
07IfMP000000k>c/0080000000X0k>c/0000001fMWH0k>c/0000003/k>`0000007IfMP3/k>`00000
103/k>`01@000000k>c/07IfMP000000MWIf0080000000H0MWIf0000001fMWH0000007IfMP000004
0>c/k0030000001fMWH0k>c/00@0k>c/00<0000007IfMP3/k>`00P3/k>`01`000000MWIf00000000
0000k>c/0000001fMWH00`0000020>c/k0040000001fMWH0000007IfMQD0k>c/00<0Xj>S0<?3``3/
k>`06@3/k>`010000000MWIf0>c/k03/k>`20000000807IfMP000000MWIf0000001fMWH0k>c/0000
001fMWH30>c/k0D0000000<0MWIf0>c/k03/k>`00P3/k>`01@000000k>c/0000003/k>`0MWIf0080
000000D0MWIf0000001fMWH000000>c/k0020000000607IfMP000000MWIf0000001fMWH000000`3/
k>`02P1fMWH0000007IfMP000000k>c/07IfMP000000MWIf0000003/k>`2000000<0k>c/00<00000
07IfMP3/k>`00P3/k>`30000000=07IfMP3/k>`0k>c/0000003/k>`0MWIf0000001fMWH000000>c/
k01fMWH0000007IfMP02000001T0k>c/0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0
VIVI6P3lo?`01@000000OWin03lo?`1nOWh0o?cl0080000000D0o?cl0000001nOWh0000007inOP02
0000000307inOP000000o?cl0080OWin00L0o?cl0000001nOWh0000003lo?`3lo?`0000000@0o?cl
00<0000007inOP1nOWh00P3lo?`01P000000o?cl07inOP000000OWin00000080OWin00@0000007in
OP000000o?cl0`1nOWh010000000o?cl0000001nOWh2000000070?clo0000000OWin0000000o?cl0
o?cl0000000207inOP0303lo?`1nOWh0_Kfm01X0o?cl00<0VIVI0?Cdm03dm?@0503dm?@3000000<0
m?Cd0P1jNWX00`000000m?Cd0?Cdm00207YjNP030000003dm?@0m?Cd0080m?Cd00D0]kNg07YjNP1j
NWX0m?Cd0;Ng]`0307YjNP80m?Cd0P1jNWX40?Cdm0030;Jf]P000000?3`l00D0m?Cd00<0000007Ui
N@3dm?@00P3dm?@00`1jNWX0]kNg07YjNP0307YjNP050?Cdm0000000NGUi07YjNP2g]kL00P3dm?@2
07YjNQH0m?Cd00<0Xj>S0<?3``3dm?@06@3dm?@010000000NGUi0?Cdm03dm?@200000080NWYj0P3d
m?@307YjNP@0m?Cd0P0000001P3dm?@0NGUi0000001iNGT0m?Cd0;Ng]`80NWYj00H0]kNg0?Cdm000
0000NWYj07UiN@00000207YjNP80m?Cd0`1jNWX00`3dm?@0]kNg07YjNP0307YjNP80m?Cd00X0]kNg
07YjNP1jNWX0m?Cd07UiN@000000NWYj0;Ng]`1jNWX0m?Cd0`1jNWX20?Cdm0<0NWYj103dm?@01000
0000NGUi0?Cdm00000020?Cdm0030;Ng]`1jNWX0NWYj0080m?Cd0`1jNWX01@3dm?@0NGUi0000002g
]kL0NWYj01P0m?Cd0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3X
j>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0VIVI6P3lo?`01@00
0000o?cl07inOP000000o?cl0080000000D0o?cl0000001nOWh0000007inOP020000000307inOP00
0000OWin00<0o?cl00<0000007inOP0000000P0000000`1nOWh0o?cl0?clo0020?clo0030000003l
o?`0o?cl0080o?cl00D000000?clo01nOWh0000007inOP0300000080o?cl0`0000001`3lo?`0OWin
0000001nOWh0000007inOP3lo?`00`0000000`3lo?`000000000000200000080o?cl0P00000K0?cl
o00309VIV@3jn_X0n_[j01D0n_[j00<000000?[jnP3jn_X0103jn_X00`000000n_[j0?[jnP020?[j
nP030000003jn_X0n_[j0180n_[j00<0OGem0000000000001@3jn_X00`000000OGem0?[jnP090?[j
nP030000001mOGd0n_[j01/0n_[j00<0Xj>S0<?3``3jn_X06@3jn_X010000000OGem0?[jnP3jn_X2
000000/0n_[j00H000000?[jnP3jn_X0n_[j0000001mOGd60?[jnP040000003jn_X0OGem000001<0
n_[j00<0OGem0000003jn_X03`3jn_X010000000OGem0?[jnP00000;0?[jnP0307emO@000000n_[j
01T0n_[j0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`
0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000@000000?clo02:RXX0VIVI6P3lo?`010000000o?cl
07inOP0000080?clo080000000<0OWin0000003lo?`03@3lo?`00`000000o?cl0?clo00@0?clo003
07inOP000000o?cl02X0o?cl00<0VIVI0?ooo`3oool05@3oool00`1oOgl0oooo0?ooo`040?ooo`03
07moO`3oool0oooo0080oooo00<0Ogmo0?ooo`3oool04`3oool00`1oOgl0_kno0?ooo`040?ooo`03
07moO`2o_kl0oooo0080oooo00<0Ogmo0;no_`3oool0103oool00`1oOgl0_kno0?ooo`0K0?ooo`03
0:>SX`33`l<0oooo01P0oooo101oOgl00`3oool0Ogmo07moO`0;0?ooo`0607moO`3oool0oooo0?oo
o`1oOgl0_kno203oool00`2o_kl0Ogmo0?ooo`0E0?ooo`030;no_`1oOgl0oooo00`0oooo0`1oOgl0
0`2o_kl0oooo0?ooo`090?ooo`030;no_`1oOgl0oooo01T0oooo0180`l?30>?Sh`2:RXX0o?cl0000
002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`00
00@000000?clo02:RXX0VIVI6P3lo?`3000000L0o?cl00@0000007inOP00000000001P3lo?`00`00
0000OWin0?clo0070?clo0@000004P3lo?`00`000000OWin0?clo00W0?clo00309VIV@3oool0oooo
06L0oooo00@0nOWi0:>SX`33`l<0oOgmR@3oool04`3dm?@0`l?30>?Sh`2:RXX0o?cl0000002LW9`0
YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000040000
003lo?`0RXZ:09VIVGP0o?cl00<0VIVI0?Shn03oool0I`3oool01@3Vi^H0aLG50=?Cd`3EeMD0lo?c
08L0oooo01@0n_[j0=GEe@3Cdm<0hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00001@000000o?cl08Z:RP2VYZH0_[jn07H0
a<C400<0_[jn0:JVYP2/[:`0J02SXj<01@2o_kl0hn?S0>?Sh`3De=@0][Jf08L0Xj>S01@0ZJVY0<k>
cP3Shn<0hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[j
nP3lo?`0oOgm0?knoP40oooo00001@000000o?cl08Z:RP36a/H0[jn_07H0a<C400<0/K6a0<K6aP31
`L40:`31`L401@2b/[80hn?S0:>SX`31`L40`/;203H0`L7100@0`l?30:>SX`31`L40`/;2>031`L40
0`33`l<0Xj>S0<71`@020<;2`Tl0`L710180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35
aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000D000000?clo02:RXX0
hN7Q0;Vi^@170;:b/P030:b/[02LW9`0/[:b02L0/[:b0`1;Bd/0101nOWh0/[:b0;no_`3Ng]h/0<c<
c0030;Ng]`3Shn<0Xj>S03P0c<c<00<0`l?30:>SX`3<c<`0>@3<c<`00`33`l<0Xj>S0<c<c01A0<c<
c00B0<?3``3Shn<0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:0>3Ph031`L40A`3IfMT00`33`l<0Xj>S
0=WIf@0Z0=WIf@030000003IfMT0fMWI02d0fMWI00<0_[jn0>?Sh`2SXj<0>03IfMT00`33`l<0Xj>S
0=WIf@0i0=WIf@030<?3``2SXj<0fMWI0540fMWI0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000D000000?cl
o02:RXX0h>3P0<71`@0@0>3Ph08000000`3Ph>002@000000Kfm_0>3Ph01_Kfl0>3Ph071`L00g=cL0
h>3P000000020>3Ph0040000003Ph>00=cLg071`L0<0gmoO00`0Kfm_0000001`L700L71`0:RXZ03O
gml0000006m_K`3Ph>0000000>3Ph00g=cL2071`L0030000003Ph>00h>3P00h0h>3P00<0gmoO0<?3
``2SXj<0403Ph>000`1_Kfl0>3Ph071`L0020000000D0>3Ph02WYjL00000071`L0000000YjNW06m_
K`000000Z:RX06m_K`0h>3P0h>3P0000003Ph>00000006m_K`3Ph>0000000>3Ph00g=cL2071`L00>
0000003Ph>00h>3P0=oOg`2WYjL00000071`L00h>3P0Kfm_0=oOg`1_Kfl0>3Ph0=oOg`0g=cL2071`
L00A0000003Ph>0000000>3Ph02WYjL00000071`L02WYjL00000071`L0000000YjNW06m_K`000000
L71`0000001_Kfl03`3Ph>000`31`L40hn?S0:>SX`0A0>3Ph00@0000001_Kfl0h>3P0>3Ph02WYjL0
0000071`L0000000YjNW06m_K`000000>3Ph0000001_Kfl0000006m_K`<0gmoO4`3Ph>000`3Ogml0
`l?30:>SX`0@0>3Ph0040000001`L700L71`03Lg=`80h>3P00X0Kfm_03Ph>01`L70000000:NWY`00
0000Kfm_0000001_Kfl000000P1_Kfl020000000L71`0000002WYjL0Kfm_03Ph>0000000Kfm_3`3P
h>000`3Ogml0`l?30:>SX`0@0>3Ph00D0000001_Kfl0Kfm_0:NWY`000000Kfm_0>3Ph01_Kfl0>3Ph
071`L0000000h>3P03Lg=`1`L70000000:NWY`1_Kfl0>3Ph0>3Ph00g=cL2071`L00303Lg=`3Ph>00
000000@0gmoO00L0Kfm_0000003Ogml0000006m_K`3Ogml000000080Kfm_00d0>3Ph071`L0000000
h>3P06m_K`0h>3P0L71`03Lg=`3Ph>0000000>3Ph0000000Kfm_00h0h>3P01<0gmoO0<?3``3Shn<0
RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0
o_kn0040oooo00001@000000o?cl08Z:RP3Ph>00`L710100j>SX0P0000030>SXj0090000001dM7@0
j>SX0000001dM7@0j>SX0000003Xj>P000000080j>SX00<000000>SXj0000000103Xj>P00`1dM7@0
00000>SXj0030>SXj0060000001dM7@0j>SX0000003Xj>P000000P3Xj>P00`000000j>SX0>SXj00?
0>SXj0030<?3``2SXj<0j>SX00l0j>SX00P0000007AdM03Xj>P0M7Ad0000003Xj>P0M7Ad000000<0
j>SX0P000000303Xj>P0000007AdM03Xj>P000000>SXj0000000M7Ad0>SXj0000000j>SX00000080
j>SX00<000000>SXj03Xj>P0103Xj>P01`1dM7@000000>SXj0000000M7Ad0>SXj00000000P3Xj>P0
2@000000j>SX0000003Xj>P0M7Ad0000003Xj>P0M7Ad000000030>SXj080000000<0j>SX0000001d
M7@03`3Xj>P00`35aLD0hn?S0:>SX`0A0>SXj0090000001dM7@0j>SX0>SXj01dM7@000000>SXj000
0000M7Ad0080000000D0j>SX0000001dM7@0000007AdM00G0>SXj0030<?3``2SXj<0j>SX00l0j>SX
00P000000>SXj03Xj>P0M7Ad0000003Xj>P0000007AdM0<0j>SX00H0000007AdM0000000M7Ad0000
001dM7@2000000060>SXj0000000M7Ad0>SXj01dM7@000004@3Xj>P00`33`l<0Xj>S0>SXj00?0>SX
j00;0000001dM7@0000007AdM0000000M7Ad0>SXj0000000M7Ad0>SXj00000000P3Xj>P200000005
0>SXj0000000M7Ad0>SXj0000000103Xj>P00`000000j>SX0>SXj0020>SXj00?07AdM0000000j>SX
0000001dM7@0j>SX0000001dM7@0000007AdM03Xj>P000000>SXj0000000M7Ad00<0j>SX00<0M7Ad
000000000000403Xj>P04P33`l<0hn?S08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00001@000000o?cl08Z:RP3Ph>00`L710100
k>c/0P00000207IfMP0?0>c/k0000000MWIf0>c/k0000000MWIf0>c/k0000000k>c/0000001fMWH0
k>c/0000003/k>`0000000@0k>c/00@0MWIf0000001fMWH0MWIf0P3/k>`01P000000>c/k0>c/k000
0000k>c/00000080k>c/00<000000>c/k03/k>`03`3/k>`00`33`l<0Xj>S0>c/k00?0>c/k00;0000
001fMWH0MWIf03/k>`000000k>c/07IfMP000000MWIf0000001fMWH00P000000303/k>`0000007If
MP3/k>`000000>c/k0000000>c/k0>c/k0000000k>c/00000080k>c/00<000000>c/k03/k>`00P3/
k>`207IfMP0703/k>`1fMWH0k>c/0000001fMWH0k>c/0;6a/@0207IfMP0<0000003/k>`000000>c/
k01fMWH000000>c/k01fMWH0000007IfMP000000MWIf0P0000000`3/k>`0000007IfMP0?0>c/k003
0<O7a`3Shn<0Xj>S0140k>c/00T0000007IfMP3/k>`0k>c/07IfMP000000k>c/0000001fMWH00P00
00001@3/k>`0000007IfMP000000MWIf01L0k>c/00<0`l?30:>SX`3/k>`03`3/k>`04@000000k>c/
0>c/k01fMWH000000>c/k0000000>c/k07IfMP000000MWIf0000001fMWH0000007IfMP000000MWIf
0080000000D0k>c/0000001fMWH0MWIf03/k>`0207IfMQ00k>c/00<0`l?30:>SX`3/k>`03`3/k>`5
0000000307IfMP3/k>`0k>c/0080MWIf00<000000>c/k00k>c/00P1fMWH01@2a/K40000007IfMP3/
k>`000000080MWIf00<000000>c/k0000000103/k>`05P1fMWH000000>c/k0000000>c/k0>c/k000
0000MWIf0000001fMWH0k>c/0000003/k>`0000003/k>`1fMWH000000>c/k00k>c/0MWIf0000002a
/K4?0>c/k00B0<?3``3Shn<0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000050000003lo?`0RXZ:0>3Ph031`L40403clo<20000
00<0lo?c0P0000020??cl`80000000<0NGUi0??cl`0000000P000000101iNGT0lo?c000000000003
0??cl`0307UiN@000000lo?c00<0lo?c0`0000000`1iNGT0lo?c07UiN@0300000140lo?c00<0`l?3
0:>SX`3clo<03`3clo<00`000000NGUi0??cl`040??cl`<0000000@0lo?c000000000000NGUi0P00
00000`3clo<000000??cl`030000000307UiN@3clo<0NGUi00<000000`3clo<00`1iNGT000000??c
l`030??cl`80000000H0lo?c07UiN@000000000007UiN@3clo<20000000407UiN@00000000000??c
l`<000000P3clo<30000000307UiN@3clo<0lo?c00d0lo?c00<0bl_;0>?Sh`2SXj<04@3clo<00`00
0000NGUi0??cl`020??cl`<0000000H0lo?c000000000000lo?c0000001iNGT2000001L0lo?c00<0
`l?30:>SX`3clo<03`3clo<01@000000lo?c0??cl`1iNGT000000080lo?c0`0000000`3clo<00000
0000000300000080lo?c0`0000020??cl`8000004@3clo<00`33`l<0Xj>S0??cl`0?0??cl`800000
00@0lo?c07UiN@000000NGUi0P3clo<20000000307UiN@3clo<0NGUi0080000000D0lo?c00000000
0000lo?c07UiN@020000000407UiN@3clo<00000000000<0lo?c00<0NGUi0000003clo<010000002
0??cl`<000000P3clo<20000000607UiN@3clo<000000??cl`000000NGUi3`3clo<04P33`l<0hn?S
08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm
0?knoP40oooo00001@000000o?cl08Z:RP3Ph>00`L710100mOGe0P00000@0?Gem@030000003emOD0
mOGe0080mOGe00<0NWYj0000003emOD02`3emOD00`000000mOGe0?Gem@0?0?Gem@030<?3``2SXj<0
mOGe00l0mOGe00<0000007YjNP3emOD0203emOD2000000030?Gem@000000NWYj00l0mOGe00<0NWYj
0000003emOD00`3emOD00`000000NWYj0?Gem@070?Gem@0307YjNP000000mOGe00P0mOGe00<00000
07YjNP3emOD03P3emOD00`3<c<`0hn?S0:>SX`0A0?Gem@030000001jNWX0mOGe02@0mOGe00<0`l?3
0:>SX`3emOD03`3emOD01@000000mOGe0?Gem@1jNWX0000002D0mOGe00<0`l?30:>SX`3emOD03`3e
mOD2000000040?Gem@1jNWX0000007YjNPX0mOGe00<0000007YjNP3emOD02P3emOD00`1jNWX00000
0?Gem@080?Gem@030000003emOD0mOGe01L0mOGe0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000D000000?cl
o02:RXX0h>3P0<71`@0@0?clo08000000P1mOGd00`2l_;`0o?cl0?clo00;0?clo00307moO`3lo?`0
o?cl0080o?cl00D0OWin0000001mOGd0OGem0;bl_0090?clo0030000003lo?`0o?cl00l0o?cl00<0
`l?30:>SX`3lo?`03`3lo?`01@1oOgl0?Shn07emO@0n?Sh0Ogmo00H0o?cl0P1oOgl01@3lo?`0Ogmo
0;jn_P3lo?`0OGem00d0o?cl00P0_[jn0000001mOGd0?Shn07moO`3lo?`0Ogmo0;jn_PP0o?cl00<0
_[jn07moO`3lo?`0203lo?`00`000000OWin0?clo00>0?clo0030<o?c`3Shn<0Xj>S0100o?cl00@0
OGem0000000n?Sh0OGem903lo?`00`33`l<0Xj>S0?clo00?0?clo0040000001mOGd0OGem03lo?bH0
o?cl00<0`l?30:>SX`3lo?`03`3lo?`01P000000OWin0?clo03lo?`0000007inOPX0o?cl00<0Ogmo
0;jn_P3lo?`02P3lo?`00`1nOWh000000?clo0080?clo0030000003lo?`0o?cl01L0o?cl0180`l?3
0>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl
0?gmo@3no_h10?ooo`0000D000000?clo02:RXX0h>3P0<71`@170?ooo`030<?3``2SXj<0oooo05X0
oooo00<0dM7A0>?Sh`2SXj<0>03oool00`33`l<0Xj>S0?ooo`0i0?ooo`030<?3``2SXj<0oooo0540
oooo0180`l?30>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?Kf
mP3jn_X0o?cl0?gmo@3no_h10?ooo`0000L000000?clo02:RXX0h>3P0<K6aP3bl_80o_kn04<0oooo
00H0o_kn0>k^kP37alL0[:b/0?;blP3no_iH0?ooo`050?knoP3=cLd0hn?S0:b/[03hn?P0=@3oool0
1@3no_h0k^k^0<O7a`2/[:`0n?Sh03L0oooo00D0o_kn0>k^kP37alL0[:b/0?Shn01?0?ooo`0D0?kn
oP3^k^h0alO70>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?Kf
mP3jn_X0o?cl0?gmo@3no_h10?ooo`0000L000000?clo02:RXX0h>3P0=[JfP32`/80lo?c04<0oooo
00H0moOg0<;2`P3Kfm/0e=CD0<;2`P3clo=H0?ooo`050?3`l03;bl/0hn?S0=CDe03EeMD0=@3oool0
1@3`l?00`/;20=_Kf`3De=@0eMGE03L0oooo00D0moOg0<;2`P3Kfm/0e=CD0=GEe@1?0?ooo`0D0?Og
m`32`/80fm_K0>?Sh`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?Kf
mP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:RXX00P3Ph>000`3Jf]X0a/K60<71`@12
0<71`@060<C4a03GemL0hN7Q0>7Qh@3Jf]X0a/K6F031`L401@38b<P0g=cL0>7Qh@3QhN40d];B03D0
`L7100D0b<S80=OGe`3QhN40hN7Q0=;BdP0g0<71`@050<C4a03GemL0hN7Q0>7Qh@3Bd]80C`31`L40
5034a<@0emOG0>7Qh@3QhN40RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0h>3PM03Ph>00402:RXX0
o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h1
0?ooo`0000<000000?clo02:RXX0o`3Ph>1d0>3Ph00@08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5
a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o
0>3Ph7@0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0h>3PM03Ph>00402:RXX0o?cl0000
002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`00
00<000000?clo02:RXX0o`3Ph>1d0>3Ph00@08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0
hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP3o0>3Ph7@0
h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0@3oool000030000003lo?`0RXZ:0=L0h>3P00<0i>CT0>SXj03YjNT01@3YjNT50>_[
j`80jNWY0P3Zj^X00`3Vi^H0h>3P0>3Ph0030>3Ph0030>?Sh`3WinL0jNWY00D0jNWY00<0j^[Z0>_[
j`3[jn/00P3[jn/30>WYj@030>[ZjP3Vi^H0h>3P0080h>3P00<0i>CT0>SXj03YjNT01@3YjNT50>_[
j`80jNWY0P3Zj^X00`3Vi^H0h>3P0>3Ph0170>3Ph0040=CDe037alL0alO70<K6aP@0alO700<0b<S8
0<W9b@38b<P02037alL04P38b<P0bLW908Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3Cdm<0hN7Q
0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:RP070>3Ph0030<o?
c`33`l<0`l?300`0`l?300<0alO70>3Ph03Ph>001@3Ph>000`3?cll0`l?30<?3``0<0<?3``030<O7
a`3Ph>00h>3P0:H0h>3P00<0gMgM0;Zj^P1@D5003@0d=3@00`15ADD0QXJ60?7al@040>3Ph0030=gM
g@3Rh^80LG5a00d0=3@d00<0?3`l08J6QP3alO400P3Ph>000`3MgMd0^[Zj051@D00=03@d=00304E5
A@26QXH0lO7a04T0h>3P00`0alO70?knoP3moOd0kno_0>[ZjP3YjNT0j>SX0>WYj@3Ph>00SXj>0=cL
g03Vi^H20>CTi0030>?Sh`3Ti>@0hn?S00<0h^;R0140/K6a08Z:RP3lo?`0000009bLW02VYZH0][Jf
0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:
RXX01P3Ph>000`3Fe]H0d];B0>_[j`0=0?7al@030=kNgP3<c<`0h>3P00@0h>3P00<0e]KF0=;BdP3[
jn/03@3alO400`3Ng]h0c<c<0>3Ph0040>3Ph9l0Zj^[0`3Ph>000`3IfMT0IVIV07YjNP0>08V9R@03
04];B`2j^[X0h>3P00<0h>3P00<0fMWI0861P@2CTi<03@38b<P00`36a/H0JFUY0;Zj^P020>3Ph003
0=WIf@1fMWH0^k^k00d0b<S800<0a<C406UYJ@2j^[X0B@3Ph>003@37alL0o?cl0?Ogm`3`l?00inOW
0>KViP3UiND0i^KV0>SXj02YZJT0]kNg0>KViP3Shn<00P3Rh^805P3Cdm<0fMWI0>3Ph03Ogml0gMgM
0;2`/02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl
0?gmo@3no_h10?ooo`0000<000000?clo02:RXX01P3Ph>000`3AdM40kNg]0?7al@0=0?7al@030>o_
k`33`l<0h>3P00@0h>3P00<0dM7A0>g]k@3alO403@3alO400`3_knl0`l?30>3Ph0040>3Ph0030=GE
e@3oool0oooo09/0oooo00L0Zj^[0>3Ph03Ph>00h>3P0=KFeP1cLg<0eMGE00d0_Kfm00<0eMGE0:^[
Z`2CTi<0103Ph>000`3Fe]H0?3`l0=KFeP0>0<_;b`060=gMg@2CTi<0h>3P0>3Ph03Fe]H0Shn?3`3;
bl/00`3MgMd0Ti>C0>3Ph0180>3Ph00<0<K6aP3moOd0moOg0?Gem@3YjNT0gMgM0>SXj03WinL0iNGU
08:2PP2JVYX0e]KF0P3Shn<00`33`l<0^KVi0=kNgP020=oOg`0B0=gMg@2`/;00RXZ:0?clo0000000
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0@3oool00003
0000003lo?`0RXZ:00H0h>3P00<0dM7A0?7al@3alO403P3alO400`33`l<0h>3P0>3Ph0030>3Ph003
0=7Ad@3alO40lO7a00h0lO7a00<0`l?30>3Ph03Ph>000`3Ph>000`3EeMD0oooo0?ooo`2K0?ooo`07
0:^[Z`3Ph>00h>3P0>3Ph03Fe]H0Kfm_0?ooo`0=0=CDe0030?ooo`2][Jd0SHf=00@0h>3P00<0e]KF
03@d=03YjNT03P3De=@01P3oool0SHf=0>3Ph03Ph>00e]KF09VIV@l0e=CD00<0oooo08f=S@3Ph>00
B03Ph>004@37alL0lo?c0?_kn`3emOD0n?Sh0;Bd]02c/k<0g]kN0;Rh^02j^[X0VYZJ0:FUY@2c/k<0
XZ:R08f=S@3@d=00iNGU0080hN7Q0180gmoO0;2`/02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0
dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:RXX01P3P
h>000`3AdM40lO7a0?7al@070?7al@030>3Ph03WinL0lO7a00@0lO7a00<0`l?30>3Ph03Ph>000`3P
h>000`3AdM40lO7a0?7al@030?7al@030=gMg@3alO40lO7a00P0lO7a00<0`l?30>3Ph03Ph>000`3P
h>000`3EeMD0oooo0?ooo`2K0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03Fe]H0LG5a0?ooo`030=[JfP80
K6a/00H0Xj>S0=[JfP3Jf]X0Xj>S06a/K02SXj<20=[JfP030?ooo`2^[Zh0SHf=00@0h>3P00<0e]KF
03@d=03/k>`03P3Jf]X01P3oool0SHf=0>3Ph03Ph>00e]KF09VIV@l0f][J00<0oooo08f=S@3Ph>00
B03Ph>000`37alL0emOG0>c/k0020?Shn00P0=SHf01lO7`0Ogmo0;>c/`34a<@0^[Zj0:j^[P2KVi/0
WIfM09nOW`2e]KD0hN7Q0=oOg`37alL0dM7A0;2`/02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0
dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?ooo`0000<000000?clo02:RXX01P3P
h>000`3AdM40lO7a0?7al@060?7al@030=kNgP2n_[h0dm?C00D0lO7a00<0`l?30>3Ph03Ph>000`3P
h>000`3AdM40lO7a0?7al@030?7al@030;Rh^03EeMD0lO7a00P0lO7a00<0`l?30>3Ph03Ph>000`3P
h>00103EeMD0oooo0?ooo`1oOgl40?ooo`<0000000@0oooo07moO`3oool0Ogmo0P0000004@1oOgl0
00000?ooo`000000oooo0000003oool0Ogmo0000001oOgl0oooo07moO`3oool000000?ooo`000000
oooo008000000P3oool01@1oOgl0oooo0?ooo`3oool0Ogmo0080oooo0P0000020?ooo`0607moO`3o
ool000000000003oool0Ogmo0P00001U0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03Fe]H0Lg=c0?ooo`02
0=_Kf`040000003Kfm/0fm_K00000080fm_K00<0KFe]0=_Kf`1]KFd00P3Kfm/00`3oool0/k>c08f=
S@040>3Ph0030=KFeP0d=3@0kNg]00<0fm_K00<0Zj^[09:BTP2BTY800P2BTY800`2ATI40]KFe0=_K
f`030=_Kf`060?ooo`2=SHd0h>3P0>3Ph03Fe]H0VIVI1P3Kfm/00`3;bl/0/k>c0=_Kf`060=_Kf`03
0?ooo`2=SHd0h>3P04P0h>3P02D0b<S80=KFeP3Cdm<0l?3`0?_kn`3Kfm/0Lg=c0:FUY@32`/80^[Zj
0;Vi^@2SXj<0W9bL09BDU01jNWX0IfMW07UiN@1gMgL0VIVI0>3Ph02`/;00RXZ:0?clo0000000W9bL
0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`00
0000o?cl08Z:RP060>3Ph0030=7Ad@3alO40lO7a00@0lO7a00D0i^KV0=KFeP2g]kL0/[:b0=3@d005
0?7al@030<?3``3Ph>00h>3P00<0h>3P00<0dM7A0?7al@3alO400`3alO40102i^KT0/k>c0;Vi^@3R
h^870?7al@030<?3``3Ph>00h>3P00<0h>3P00@0eMGE0?ooo`3oool0Ogmo0`3oool0201oOgl0oooo
0?ooo`1oOgl0oooo07moO`3oool0Ogmo0P3oool04@1oOgl000000?ooo`000000oooo0000003oool0
Ogmo0?ooo`2o_kl0oooo07moO`3oool000000?ooo`000000Ogmo0080oooo00<0Ogmo0?ooo`1oOgl0
0`3oool00`1oOgl0oooo000000020?ooo`0407moO`000000oooo07moO`80oooo00@0000007moO`3o
ool0oooo0P1oOgmS0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03Fe]H0MGEe0?ooo`020>?Sh`040000003S
hn<0hn?S075aL@80hn?S00<0LG5a0>?Sh`1aLG400P3Shn<00`3oool0]kNg08f=S@040>3Ph0030=KF
eP0d=3@0l?3`00<0hn?S00@0a/K608J6QP26QXH0Rh^;0P20P8000`3EeMD0hn?S0>?Sh`020>?Sh`06
0?ooo`2=SHd0h>3P0>3Ph03Fe]H0VIVI1P3Shn<00`2/[:`0T92@0;Jf]P060>?Sh`030?ooo`2=SHd0
h>3P04P0h>3P02D0b<S80=SHf02^[Zh0a<C40=oOg`39bLT0_Kfm0<?3``31`L40_kno0;6a/@2RXZ80
Rh^;079bLP1nOWh0]kNg0:j^[P38b<P0i>CT0>;RhP2_[jl0RXZ:0?clo0000000W9bL0:JVYP2f][H0
aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:
RP060>3Ph0030=7Ad@3alO40lO7a00@0lO7a00D0^KVi0;:b/P2b/[80[jn_0<c<c0050?7al@030<?3
``3Ph>00h>3P00<0h>3P00<0dM7A0?7al@3alO400`3alO401P2i^KT0/[:b0;:b/P2b/[80^k^k0<K6
aPD0lO7a00<0`l?30>3Ph03Ph>000`3Ph>00103EeMD0oooo0?ooo`1oOgl40?ooo`<0000000@0oooo
07moO`3oool0Ogmo0P0000001`1oOgl000000?ooo`000000oooo0000003oool00`0000001`3oool0
Ogmo0?ooo`000000oooo0000001oOgl0103oool01`1oOgl0oooo0?ooo`3oool0Ogmo0?ooo`000000
0P3oool00`1oOgl000000?ooo`030?ooo`030000001oOgl0oooo0080oooo00<000000?ooo`3oool0
H@3oool01`2[Zj/0h>3P0>3Ph03Ph>00e]KF07QhN03oool00P3/k>`010000000k>c/000000000002
0>c/k00307IfMP3/k>`0MWIf0080k>c/00<0oooo0;fm_@2=SHd0103Ph>000`3Fe]H0=3@d0?Gem@04
0>c/k0050:^[Z`1dM7@0Ng]k05mOG`2e]KD01@3/k>`01P3oool0SHf=0>3Ph03Ph>00e]KF09VIV@D0
k>c/00D0a/K609BDU02>SXh0T92@0=GEe@050>c/k0030?ooo`2=SHd0h>3P04P0h>3P02D0b<S80=WI
f@2i^KT0S8b<09:BTP2o_kl0alO70<G5a@32`/80_[jn09bLW02?Shl0Ogmo07AdM02c/k<0a<C409^K
V`3@d=00iNGU0>?Sh`2_[jl0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP060>3Ph0030=7Ad@3alO40
lO7a00@0lO7a00D0[Jf]0:^[Z`2_[jl0[Jf]0<c<c0050?7al@030<?3``3Ph>00h>3P00<0h>3P00<0
dM7A0?7al@3alO400`3alO401P2i^KT0/K6a0:b/[02SXj<0XJ6Q0;^k^`D0lO7a00<0`l?30>3Ph03P
h>000`3Ph>000`3EeMD0oooo0?ooo`03000000@0oooo00<0Ogmo0?ooo`1oOgl0103oool04@1oOgl0
00000?ooo`000000oooo0000003oool0Ogmo0?ooo`1oOgl0oooo07moO`3oool000000?ooo`000000
Ogmo0080oooo00<000000?ooo`0000000P000000103oool0Ogmo0?ooo`0000020?ooo`0307moO`00
0000oooo0080oooo00<000000?ooo`1oOgl00`3oool00`000000oooo0?ooo`1Q0?ooo`070:^[Z`3P
h>00h>3P0>3Ph03Fe]H0NWYj0?ooo`020?;blP030000003bl_80l_;b00<0l_;b00<0NGUi0000001i
NGT00P3bl_800`3oool0`<3008f=S@040>3Ph0030=KFeP0d=3@0n?Sh00@0l_;b00D0b/[:08f=S@1[
Jf/0GUiN0=GEe@050?;blP060?ooo`2=SHd0h>3P0>3Ph03Fe]H0VIVI1@3bl_801@2^[Zh0SXj>08j>
SP2>SXh0b<S800D0l_;b00<0oooo08f=S@3Ph>00B03Ph>009@38b<P0f=SH0=SHf035aLD0T92@0::R
XP32`/80bl_;0;Jf]P2YZJT0Vi^K0861P@1eMGD0DE5A0<71`@2a/K40YJFU0=_Kf`3UiND0hn?S0:n_
[`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gm
o@3no_h00@3oool000030000003lo?`0RXZ:00H0h>3P00<0dM7A0?7al@3alO40103alO401@3Hf=P0
/;2`0:6QX@2RXZ80b/[:00D0lO7a00<0`l?30>3Ph03Ph>000`3Ph>000`3AdM40lO7a0?7al@030?7a
l@050;Rh^02SXj<0W9bL0;Vi^@3Jf]X01P3alO400`33`l<0h>3P0>3Ph0030>3Ph0040=GEe@3oool0
oooo07moO`80oooo00T000000?ooo`1oOgl0Ogmo0;no_`3oool0Ogmo0?ooo`2o_kl00P1oOgl01@3o
ool0Ogmo07moO`3oool0Ogmo0080oooo00X0_kno07moO`2o_kl0oooo03lo?`3oool0Ogmo0;no_`1o
Ogl0oooo0P1oOgl20?ooo`0507moO`3oool0oooo0000001oOgl00P3oool207moO`030?ooo`000000
_kno00<0oooo00<0000007moO`3oool00P3oool00`000000oooo0?ooo`1Q0?ooo`070:^[Z`3Ph>00
h>3P0>3Ph03Fe]H0O7al0?ooo`020?Ogm`040000003gmoL0moOg0;Vi^@L0moOg00<0oooo0<C4a02=
SHd0103Ph>000`3Fe]H0=3@d0?[jnP050?Ogm`030<;2`P1VIVH0Zj^[00H0moOg00H0oooo08f=S@3P
h>00h>3P0=KFeP2IVIT40?Ogm`070=;BdP2>SXh0OGem07MgM`1kNg/0S8b<0=_Kf`040?Ogm`030?oo
o`2=SHd0h>3P04P0h>3P02D0b<S80=KFeP3EeMD0e]KF0<71`@2VYZH0/k>c0;Zj^P2/[:`0X:2P092@
T01kNg/0H61P09^KV`39bLT0YjNW09fMW@30`<00inOW0>GUi@2_[jl0RXZ:0?clo0000000W9bL0:JV
YP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000
o?cl08Z:RP060>3Ph0030=7Ad@3alO40lO7a00H0lO7a00<0emOG09^KV`34a<@01@3alO400`33`l<0
h>3P0>3Ph0030>3Ph0030=7Ad@3alO40lO7a00<0lO7a00<0[:b/0:BTY039bLT0203alO400`33`l<0
h>3P0>3Ph0030>3Ph0040=GEe@3oool0oooo07moO`80oooo00<000000?ooo`3oool04`3oool00`1o
Ogl0oooo0?ooo`070?ooo`0507moO`3oool0oooo0000001oOgl01@3oool00`000000oooo00000002
0?ooo`050000001oOgl0oooo0?ooo`000000I03oool01`2[Zj/0h>3P0>3Ph03Ph>00e]KF07inOP3o
ool00`3kno/20000000307emO@3kno/0no_k00D0no_k00<0oooo0<[:bP2=SHd0103Ph>000`3Fe]H0
=3@d0?gmo@060?_kn`030;Rh^03kno/0no_k00D0no_k00H0oooo08f=S@3Ph>00h>3P0=KFeP2IVIT4
0?_kn`0309ZJVP1ZJVX0GUiN0080GEeM00<0HF5Q0:6QX@3kno/00`3kno/00`3oool0SHf=0>3Ph018
0>3Ph00U0<S8b03Fe]H0eMGE0=KFeP3Cdm<0WIfM0;Bd]02RXZ80UiNG08n?S`1kNg/0Kfm_06QXJ028
R8P0/k>c0:j^[P2][Jd0VIVI0;fm_@3Ph>00[jn_08Z:RP3lo?`0000009bLW02VYZH0][Jf0<G5a@3C
dm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<000000?clo02:RXX01P3P
h>000`3AdM40lO7a0?7al@080?7al@030>?Sh`3alO40lO7a00<0lO7a00<0`l?30>3Ph03Ph>000`3P
h>000`3AdM40lO7a0?7al@030?7al@030=KFeP3UiND0lO7a00P0lO7a00<0`l?30>3Ph03Ph>000`3P
h>000`3EeMD0oooo0?ooo`0307moOaX0oooo00<0Ogmo0?ooo`3oool00`3oool307moO`030?ooo`2o
_kl0oooo00L0oooo0P1oOgl0103oool0_kno07moO`1oOgmU0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03F
e]H0P8200?ooo`0=0?gmo@030?ooo`3=cLd0SHf=00@0h>3P00<0e]KF03@d=03no_h03P3moOd01P3o
ool0SHf=0>3Ph03Ph>00e]KF09VIV@l0oOgm00<0oooo08f=S@3Ph>00B03Ph>009@38b<P0e]KF0=GE
e@3Cdm<0YjNW09>CT`2XZ:P0Ti>C08n?S`1fMWH0KVi^05MGE`1MGEd0Hf=S09RHV02FUYH0XZ:R0;Bd
]02m_Kd0emOG0:n_[`2:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?Kf
mP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:00H0h>3P00<0dM7A0?7al@3alO40
3P3alO400`33`l<0h>3P0>3Ph0030>3Ph0030=7Ad@3alO40lO7a00h0lO7a00<0`l?30>3Ph03Ph>00
0`3Ph>000`3EeMD0oooo0?ooo`2K0?ooo`070:^[Z`3Ph>00h>3P0>3Ph03EeMD0PX:20?ooo`0=0?gm
o@030?ooo`3AdM40S8b<00@0h>3P00<0eMGE03@d=03no_h03P3moOd01P3oool0S8b<0>3Ph03Ph>00
eMGE09VIV@l0oOgm00<0oooo08b<S03Ph>00B03Ph>009@38b<P0f=SH0=3@d02UYJD0VYZJ096AT@2M
WId0Rh^;096AT@2VYZH0dM7A0<;2`P1KFe/0=SHf08N7Q`2>SXh0fMWI0?gmo@3inOT0mOGe0:j^[P2:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h00@3oool000030000003lo?`0RXZ:00H0h>3P00<0e=CD0=cLg03^k^h03@3alO400`3Ti>@0bLW9
0>3Ph0040>3Ph0030=CDe03Lg=`0k^k^00d0lO7a00<0i>CT0<W9b@3Ph>00103Ph>000`3EeMD0oooo
0?ooo`2K0?ooo`060:^[Z`3Ph>00h>3P0>3Ph03Fe]H0Ogmo3`3Kfm/00`2>SXh0YjNW0>3Ph0030>3P
h0030=KFeP1`L700`l?300d0oooo00<0m_Kf0::RXP2WYjL00P3Ph>000`3Fe]H0RHV90>KViP0=0?oo
o`030>k^kP2RXZ80YjNW04T0h>3P0140b<S80=SHf038b<P0_Kfm0=3@d031`L40OGem0:NWY`3=cLd0
[Jf]0;>c/`2WYjL0UiNG07MgM`1dM7@0VIVI0=gMg@030>GUi@0A0:n_[`2:RXX0o?cl0000002LW9`0
YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000
003lo?`0RXZ:00L0h>3P00<0clo?0<?3``33`l<03033`l<00`37alL0h>3P0>3Ph0050>3Ph0030<o?
c`33`l<0`l?300`0`l?300<0alO70>3Ph03Ph>00103Ph>2O0:^[Z`<0h>3P00<0emOG0;Jf]P1@D500
3@0d=3@00`15ADD0PH610=cLg0040>3Ph0030=OGe`3Lg=`0L71`00d0=3@d00<0?3`l0861P@3Lg=`0
0P3Ph>000`3GemL0][Jf051@D00=03@d=00304E5A@21PH40g=cL04T0h>3P02D0b<S80=OGe`3Fe]H0
eMGE0=KFeP3AdM40LG5a0;2`/02TY:@0Zj^[096AT@2d];@0VYZJ09JFUP21PH40OWin0<71`@3Fe]H0
e=CD0=3@d02a/K40RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0040oooo00000`000000o?cl08Z:RP3F0>3Ph0030=kNgP3Fe]H0e=CD00H0
e=CD1@3?cll20=CDe0030=GEe@3Fe]H0f][J00@0h>3P00<0g]kN0=OGe`3De=@01P3De=@00`3AdM40
clo?0<o?c`020<o?c`<0e=CD00H0e]KF0=[JfP3Ph>00h>3P0=kNgP3Fe]H70=CDe0D0clo?0P3De=@0
0`3EeMD0e]KF0=[JfP190>3Ph00U0<S8b03Fe]H0eMGE0=CDe03Fe]H0`/;20820P037alL0d=3@0=;B
dP2SXj<0Z:RX0:f][@3Cdm<0bl_;08R8R01nOWh0dM7A0=GEe@3AdM40/K6a08Z:RP3lo?`0000009bL
W02VYZH0][Jf0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP010?ooo`0000<0
00000?clo02:RXX0o`3Ph>1O0>3Ph0080<S8b03Fe]H0eMGE0=CDe03Fe]H0W9bL0<g=c@3GemL20=GE
e@0K0<[:bP2LW9`0^KVi0=OGe`3Fe]H0e=CD0;:b/P2o_kl0eMGE0=7Ad@2a/K40RXZ:0?clo0000000
W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
0`000000o?cl08Z:RP3o0>3Ph5l0h>3P00L0b<S80=OGe`3EeMD0e=CD0=KFeP36a/H0e]KF0080e=CD
00D0eMGE0=KFeP2[Zj/0]KFe0=KFeP020=GEe@0E0=KFeP3EeMD0eMGE0=7Ad@2a/K40RXZ:0?clo000
0000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo
00000`000000o?cl08Z:RP3o0>3Ph5l0h>3P00<0b<S80=KFeP3>c/h01`3>c/h00`3?cll0a<C40<C4
a0060<k>cP0B0<_;b`2c/k<0RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`
l03fm_H0n_[j0?clo03moOd0o_kn0@3oool000030000003lo?`0RXZ:0?l0h>3PG`3Ph>000`39bLT0
^k^k0;6a/@0A0;:b/P0A0;Bd]02:RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0
l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003lo?`0RXZ:0?l0h>3PM03Ph>00402:
RXX0o?cl0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3n
o_h10?ooo`0000<000000?clo02:RXX0o`3Ph>1d0>3Ph00@08Z:RP3lo?`0000009bLW02VYZH0][Jf
0<G5a@3Cdm<0hN7Q0>SXj03`l?00m_Kf0?[jnP3lo?`0oOgm0?knoP40oooo00000`000000o?cl08Z:
RP3o0>3Ph7@0h>3P0100RXZ:0?clo0000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03f
m_H0n_[j0?clo03moOd0o_kn0@3oool000030000003gmoL0Y:BT0?l0W9bLM02LW9`0402TY:@0moOg
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h10?oo
o`0000<000000?Ogm`3Shn<0o`3Shn=e0>?Sh`0?0?Ogm`000000W9bL0:JVYP2f][H0aLG50=?Cd`3Q
hN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo00000`000000jNWY0?gmo@1G0?gmo@03
0?_kn`3Shn<0k^k^0>P0oOgm00<0h>3P08f=S@21PH400`21PH400`22PX80PH610861P@050861P@04
0:RXZ03moOd0oOgm0:RXZ0@0PH610P22PX860861P@0408f=S@3Ph>00oOgm0::RXP<0F5QH00<0FEUI
05QHF01IFET01P1HF5P04`1mOGd0h^;R0?gmo@3moOd0jNWY0000002LW9`0YZJV0;Jf]P35aLD0dm?C
0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000030000003Ph>00moOg05L0moOg
00<0Ng]k079bLP3Lg=`0j03gmoL00`2f][H0SHf=0::RXP0;0::RXP040861P@3gmoL0moOg0861P@`0
XZ:R00H0SHf=0;Jf]P3gmoL0F5QH06m_K`1aLG49079bLP03071`L01KFe/0YZJV0080moOg00l0h>3P
0000002LW9`0YZJV0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3o
ool000030000003MgMd0k^k^0080k^k^00H0c<c<0<[:bP39bLT0bLW90<S8b037alL30<S8b0030<W9
b@3<c<`0k^k^0080k^k^01L0iNGU0>;RhP3^k^h0j>SX0>WYj@3^k^h0gmoO0>c/k03MgMd0fm_K0>CT
i03Ph>00k^k^0>7Qh@3Kfm/0iNGU0=cLg03Ph>00k^k^0>GUi@3Rh^80k^k^0>7Qh@020=_Kf`0J0>c/
k03[jn/0h>3P0>k^kP3QhN40inOW0>k^kP3QhN40inOW0>KViP3Kfm/0gmoO0=cLg03Ti>@0jn_[0=_K
f`3Lg=`0kNg]0=gMg@3]kNd0k^k^0>GUi@3Kfm/0gmoO0>k^kP3Vi^H20=_Kf`070>3Ph03Ti>@0k^k^
0>k^kP3]kNd0g=cL0>3Ph0020>k^kP030=oOg`3]kNd0jn_[0080fm_K00X0h^;R0>k^kP3Xj>P0fm_K
07=cL`1bLW80clo?0=_Kf`3Lg=`0kNg]0`3^k^h0103]kNd0fm_K0=_Kf`3Kfm/20>g]k@050=cLg03P
h>00k^k^0>k^kP3[jn/00P3Kfm/01`3WinL0k^k^0>k^kP3Rh^80j^[Z0>k^kP3Ti>@00P3^k^h01@3S
hn<0fm_K0=cLg03]kNd0jn_[0080fm_K00@0h^;R0>k^kP3UiND0h>3P`@3^k^h0102a/K40^KVi0:j^
[P31`L450=GEe@D0[Zj^00H0UiNG0>k^kP3^k^h0UiNG0:j^[P31`L480=GEe@80[Zj^00H0^KVi0;6a
/@3^k^h0I6AT071`L01dM7@907MgM`03075aL@1cLg<0XJ6Q0080k^k^00l0gMgM0000002LW9`0YZJV
0;Jf]P35aLD0dm?C0>7Qh@3Xj>P0l?3`0?KfmP3jn_X0o?cl0?gmo@3no_h00@3oool000070000003J
f]X0iNGU0>GUi@2g]kL0Vi^K0<[:bP020>WYj@070=OGe`25QHD0k>c/0>k^kP3[jn/0l_;b0:>SX`02
0>GUi@0h0;bl_00E5AD0_[jn0;>c/`1gMgL0emOG071`L01/K6`0C4a<010@4025QHD06a/K0:FUY@2b
/[80410@0451@@1nOWh0DE5A0<o?c`2l_;`05ADE0<71`@2_[jl0410@01DE5@2j^[X0b/[:01`L702S
Xj<0/k>c04m?C`3IfMT0/k>c04m?C`3Fe]H0;Rh^010@401@D500LG5a0=WIf@1QHF40410@08j>SP1@
D500JFUY0>?Sh`3MgMd06QXJ010@402XZ:P0e]KF02h^;P0@4100QHF504I6AP3IfMT20>GUi@0?08:2
PP1ADE40clo?0>GUi@1`L700JFUY0>?Sh`1OGel0410@01DE5@33`l<0i>CT0410@02BTY80LW9b0080
410@00<05ADE0<30`03De=@00P3UiND02@3Ng]h09RHV010@400@4100Kfm_0<K6aP1nOWh0DE5A0<o?
c`020>GUi@0G05mOG`0@4100E5AD0<K6aP3Ng]h0ZZZZ08n?S`3GemL0/k>c08f=S@3Shn<0bLW901HF
5P0@4100Ogmo0=CDe01OGel0410@01DE5@33`l<0gMgM03/k>`3?cll0`@3UiND0102][Jd0`<300;Fe
]@3Jf]X4079bLP0302LW9`2e]KD0]KFe00<0]KFe00H0Vi^K0>GUi@3UiND0Vi^K0;Fe]@3Jf]X802LW
9`80]KFe00H0`<300:f][@3UiND0IVIV07MgM`1iNGT20?ooo`@0NWYj00H0_;bl0?ooo`1jNWX0N7Qh
07MgM`2MWId20>GUi@0?0=[JfP000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0
n_[j0?clo03moOd0o_kn0040oooo000040000000fMWI0=oOg`3Ogml0g]kN0:2PX03YjNT0iNGU0>[Z
jP3MgMd0M7Ad0>g]k@2][Jd0hN7Q0??cl`2SXj<20=kNgP0h0<K6aP0@4100[jn_06a/K00h>3P0b/[:
04i>CP1/K6`0410@0;6a/@1>CTh0410@0;6a/@1>CTh0OWin0=gMg@1nOWh0DE5A0<g=c@2f][H05ADE
0;6a/@0K6a/0UIFE0<?3``1<C4`0cLg=01`L702QXJ40[Jf]04m?C`3EeMD0[Jf]04m?C`29RHT0Cdm?
0=GEe@0_;bl0SXj>0=GEe@13@d<0a<C40=cLg01>CTh0JFUY0=KFeP0U9BD0Ph>30=_Kf`1fMWH0Shn?
04m?C`3EeMD0CTi>04m?C`3IfMT20=kNgP0D07inOP1ADE40cLg=0=kNgP1/K6`0JFUY0<;2`P0f=SH0
fMWI0<?3``2BTY80emOG02h^;P34a<@0Kfm_079bLP3AdM40U9BD01XJ6P35aLD20=kNgP0R0=OGe`0U
9BD0Shn?0=kNgP10@400Y:BT07MgM`1ADE40cLg=0=kNgP33`l<0ATI60=WIf@1BDU80YJFU0<g=c@1n
OWh0>3Ph0<c<c01[Jf/0JFUY0=GEe@3GemL0YjNW0<?3``16ATH0]kNg03Hf=P3IfMT0`l?309:BTP3G
emL0>c/k0<g=cL40g]kN00@0ZZZZ0<?3``2k^k/0gMgM101bLW800`0W9bL0^k^k0;^k^`030;^k^`07
09bLW03Ng]h0g]kN09bLW02k^k/0gMgM02LW9`050;^k^`0:0=gMg@0W9bL0^k^k0;^k^`33`l<0ZZZZ
0=kNgP1VIVH0N7Qh05EEE@80oooo00@0_[jn07emO@1mOGd0OGem0P3oool0100`<300NGUi07MgM`2J
VYX20=kNgP0?0=WIf@000000W9bL0:JVYP2f][H0aLG50=?Cd`3QhN40j>SX0?3`l03fm_H0n_[j0?cl
o03moOd0o_kn0040oooo000040000000e]KF0=GEe@3EeMD0/;2`09VIV@3AdM40]KFe04=3@`2[Zj/0
/[:b08n?S`1KFe/0^k^k0=kNgP2WYjL20=GEe@0H0<g=c@0O7al0TY:B02h^;P0i>CT0YJFU01lO7`29
RHT04Q8B0:^[Z`1dM7@05ADE0;Ng]`0T92@0O7al0=GEe@1iNGT0DE5A0<[:bP2_[jl05ADE08f=S@0B
4Q80HV9R0P1ZJVX05P2NWYh06a/K09nOW`2VYZH0Cdm?0<o?c`2VYZH0Cdm?09>CT`1?Cdl0bLW904A4
A02TY:@0bl_;02h^;P32`/80dm?C04];B`1YJFT0UiNG018B4P2e]KD20=GEe@0609NGU`1?Cdl0bLW9
07AdM01LG5`0d];B0P3EeMD0201iNGT0DE5A0<[:bP3EeMD0J6QX06UYJ@1hN7P0=SHf0P1ZJVX02P1j
NWX0`l?302h^;P32`/80JVYZ079bLP3=cLd0alO701/K6`2OWil20=GEe@0L0<k>cP0T92@0SHf=0=GE
e@1RHV80LW9b07IfMP1ADE40b/[:0=GEe@1iNGT0HV9R0=?Cd`1iNGT0DE5A0<[:bP1>CTh0=CDe09BD
U00d=3@0CTi>0<S8b03EeMD0Shn?04M7A`1SHf<0N7Qh03Hf=P80JVYZ00@0NWYj0<?3``0j>SX0b/[:
KP3Fe]IC0=GEe@040:FUY@39bLT0`/;20=;BdPD0MWIf1@32`/801`2OWil0eMGE0=GEe@2OWil0`/;2
0>3Ph00W9bL01@32`/805P3Ph>009bLW0<;2`P32`/80bLW90:FUY@3EeMD0J6QX07emO@20P800EEEE
09>CT`3oool0_kno0:2PX03oool0bLW905EEE@21PH40OWin07alO02EUID20=GEe@0?0=KFeP000000
WinO0:ZZZP2h^;P0alO70=GEe@3QhN40j>SX0?3`l03fm_H0n_[j0?clo03moOd0o_kn0040oooo0000
40000000dm?C0<k>cP3?cll0c/k>08N7Q`2c/k<0fm_K0:f][@2h^;P0ZJVY09^KV`2ATI40_Kfm0>o_
k`2UYJD20<k>cP090<c<c019BDT0FUYJ079bLP2NWYh0IfMW01DE5@2i^KT0X:2P0080410@02T05ADE
0;Fe]@0S8b<0O7al0<k>cP1eMGD0DE5A0<O7a`2YZJT05ADE09BDU00E5AD0]kNg0<O7a`0S8b<0PH61
01XJ6P2LW9`0X:2P04m?C`3:b/X0X:2P04m?C`3=cLd0F5QH010@400B4Q80Y:BT0<O7a`0];Bd0`L71
0<c<c019BDT0JFUY0:2PX00@4100ZZZZ0<k>cP3;bl/0cLg=05QHF002010@400605aLG03>c/h0c/k>
0<k>cP1eMGD0DE5A0P35aLD0401QHF40JFUY08B4Q01LG5`0b/[:0<O7a`0j>SX0a<C402d];@31`L40
IfMW079bLP3:b/X0`L7101XJ6P2OWil20<k>cP0R0<O7a`0S8b<0Rh^;0<G5a@0m?Cd0`/;207EeM@1A
DE40aLG50<g=c@24Q8@0G5aL0<[:bP1eMGD0DE5A0<C4a00j>SX0WYjN02DU9@2?Shl0Cdm?0;Vi^@2N
WYh051@D0<30`03<c<`0QHF505aLG03:b/X0alO703Xj>P34a<@0>SXj0<G5a@80cLg=K03?clmC0<k>
cP030::RXP3>c/h0bl_;00/0bl_;00L0XZ:R0<k>cP3>c/h0XZ:R0<_;b`3UiND09bLW00D0bl_;00X0
iNGU02LW9`3;bl/0bl_;0<k>cP2RXZ80c/k>06UYJ@22PX80Q8B40P25QHD02P2CTi<0oooo0?ooo`3o
ool0EUIF08F5Q@24Q8@0Ph>307emO@2BTY820<k>cP0<0=?Cd`000000ZZZZ0;>c/`2o_kl0c<c<0=WI
f@3Ti>@0jn_[0?;blP3gmoL0n_[j0P3moOd10?knoP40oooo000040000000clo?0<O7a`37alL0/K6a
09FEU@2VYZH0[:b/0:j^[P2/[:`0YJFU07AdM02h^;P0j^[Z0?Shn02TY:@30<K6aP0f071`L00L71`0
Y:BT0;Rh^00c<c<0=CDe0<?3``2DU9@0JVYZ03/k>`1NGUh0X:2P01TI6@15ADD0Rh^;071`L00T92@0
IfMW04];B`1NGUh0_Kfm06=SH`1HF5P0EEEE0451@@2[Zj/07AdM05mOG`17AdL0G5aL06][J`17AdL0
O7al0<K6aP1oOgl0JVYZ04a<C02o_kl0Ph>301hN7P1WIfL0SXj>02/[:`1YJFT0`l?3079bLP1CDe<0
JVYZ0861P@36a/H0Ogmo06YZJP0k>c/0Shn?0`36a/H0501aLG40<30`06][J`1ZJVX0>3Ph06UYJ@2j
^[X0E5AD06YZJP1EEED0MGEe0<30`00];Bd0_kno06=SH`10@400JVYZ059BDP1LG5`0_[jn0P36a/H0
9030`<008R8R04e=C@1NGUh0H61P0<K6aP1aLG40<C4a07EeM@1_Kfl0^KVi05ADE01ZJVX0BTY:0:RX
Z01`L700IFEU0<G5a@1KFe/0`l?307=cL`1SHf<0WIfM04=3@`1ZJVX0Q8B40;Zj^P1DE5@0JVYZ05EE
E@1eMGD0`<3002TY:@1eMGD0Kfm_0<K6aV`0alO7D`36a/H00`2NWYh0e=CD0<o?c`0;0<o?c`070:BT
Y036a/H0a/K60:BTY03?cll0inOW02LW9`050<o?c`090>OWi`0W9bL0clo?0<o?c`3De=@0WYjN0<K6
aP1/K6`0QHF500<0R8R800@0SHf=0?ooo`3oool0Ti>C0`28R8P04@26QXH0PX:208j>SP37alL0a/K6
0<o?c`000000^;Rh0;no_`38b<P0e=CD0=kNgP3Xj>P0k^k^0??cl`3hn?P0no_k0080oOgm0@3no_h1
0?ooo`00010000000<g=c@31`L40`L7109^KV`2IVIT0b/[:0<W9b@1cLg<0WYjN07=cL`1GEeL0_[jn
0;>c/`2][Jd0ZJVY0`31`L401`2FUYH0<30`0<C4a031`L40ATI6075aL@32`/801031`L401`2n_[h0
8R8R07]kN`32`/80KVi^055AD@34a<@05031`L400`2k^k/0;Bd]0;jn_P020<71`@030<C4a032`/80
`L7100/0`L7100L0KVi^055AD@34a<@0`L7105mOG`1YJFT0`/;200@0`L7100<0^k^k02d];@2n_[h0
2031`L401P2k^k/08R8R08Z:RP2n_[h0?Shn0;jn_]d0`L7100<0Vi^K0=SHf03EeMD02`3EeMD01`2W
YjL0`L710<71`@2WYjL0eMGE0>[ZjP0W9bL01P3oool0200W9bL0eMGE0=GEe@3Hf=P0Vi^K0<71`@1]
KFd0R8R80`2:RXX20?ooo`0309>CT`3oool0a<C40080RXZ:00<0RHV908B4Q02;Rh/00P31`L40303=
cLd000000<O7a`3<c<`0e=CD0=cLg03UiND0kNg]0?7al@3fm_H0nOWi0?clo080oOgm0@3no_h10?oo
o`0000P000000<_;b`2j^[X0^k^k0:b/[02EUID0ZZZZ0<?3``80XZ:R00H0WinO069RHP2<S8`0dM7A
0?[jnP2VYZH30;^k^`070:b/[01CDe<0_;bl0;^k^`1OGel0U9BD0;bl_0050;^k^`0608^;R`2HV9P0
^k^k06][J`1?Cdl0_kno5@2k^k/01P22PX80^KVi0;^k^`1LG5`0IVIV0;bl_0`0^k^k00L0Jf][04m?
C`2o_kl0^k^k05aLG01VIVH0_;bl00@0^k^k00<0]KFe02`/;02i^KT0202k^k/01P2e]KD08B4Q04e=
C@1WIfL0EeMG0;bl_1/0^k^kK`2l_;aC0;^k^`0309RHV03IfMT0emOG00/0emOG00H0YjNW0;^k^`2k
^k/0YjNW0=OGe`3[jn/7079bLP0C02LW9`3GemL0emOG0=WIf@2HV9P0_;bl06m_K`2<S8`0SHf=0<K6
aP3oool0bLW905]KF`1dM7@0Ti>C0?ooo`36a/H0SXj>08b<S00208V9R@0>0;^k^`2j^[X0bl_;0000
003EeMD0fMWI0=kNgP3UiND0jn_[0?7al@3emOD0n?Sh0?_kn`3lo?`20?gmo@80oooo000040000000
bLW90;Rh^02h^;P0WinO09jNWP3Rh^80l_;b0<k>cP3?cll0d];B0;Ng]`39bLT0`/;20?gmo@2WYjOo
0;Rh^3H0^;Rh00<0UiNG0=OGe`3MgMd02`3MgMd0102VYZH0^;Rh0;Rh^02VYZH20=gMg@P09bLW0P3M
gMd01P3GemL0UiNG0;Rh^01cLg<0TI6A09>CT`80oooo00X0H61P09RHV02HV9P0V9RH09>CT`3oool0
U9BD09:BTP2@T900QhN70P2h^;P02`39bLT000000>7Qh@3Ti>@0j>SX0>g]k@3alO40mOGe0?Ogm`3j
n_X0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`000000bLW90;Rh^0020;Rh^00;08J6QP33`l<0
lo?c0:2PX03_knl0fm_K0<W9b@3kno/0bLW90?knoP2WYjL0o`2h^;Pf0;Rh^00309NGU`3Fe]H0hN7Q
00/0hN7Q00@0YZJV0;Rh^02h^;P0YZJV303QhN401P3Fe]H0UiNG0;Rh^01eMGD0UiNG09nOW`80:2PX
00X0[:b/0:^[Z`2/[:`0Zj^[06UYJ@0X:2P0XZ:R09VIV@2EUID0QhN70P2h^;P02P39bLT000000>SX
j03[jn/0k^k^0?7al@3emOD0moOg0?Win@3kno/20?gmo@80o_kn0P3oool0000<00000039bLT0]KFe
0;>c/`2SXj<0UYJF0<W9b@37alL0bl_;0=3@d03AdM40][Jf0P3Cdm<00`3De=@0Z:RX0;>c/`3o0;>c
/cD0/k>c00<0U9BD0>3Ph03]kNd02`3]kNd0102[Zj/0/k>c0;>c/`2[Zj/<0>g]k@070>3Ph02DU9@0
/k>c07YjNP2SXj<0/;2`0;jn_P060<71`@0A0;jn_P2j^[X0Y:BT09nOW`24Q8@0/k>c0;Fe]@39bLT0
00000?3`l03bl_80lo?c0?KfmP3hn?P0n_[j0?_kn`3lo?`00P3moOd00`3no_h0oooo0?ooo`010?oo
o`0000<000000<?3``2k^k/0o`2^[Zi30:j^[P0409fMW@1fMWH0c/k>0?gmo@P0oooo00<0oOgm0<k>
cP21PH400P2^[Zh00`21PH40c/k>0?gmo@080?ooo`090?gmo@3>c/h0MWIf09fMW@2^[Zh0MgMg0;Fe
]@3EeMD0j>SX00H0inOW00l0j>SX0=oOg`30`<00EUIF0:>SX`2^[Zh0^k^k0<?3``000000m_Kf0?Og
m`3hn?P0nOWi0?_kn`3lo?`00`3moOd00`3no_h0oooo0?ooo`020?ooo`0000D000000;Jf]P3AdM40
/[:b0:ZZZP3o0:RXZ480Z:RX00<0Vi^K092@T02@T9002P2@T900102RXZ80Z:RX0:RXZ02RXZ8<092@
T00409^KV`2XZ:P0Z:RX0::RXP`0P82000H0Vi^K0:ZZZP2b/[80dM7A0;Jf]P0000020?[jnP030?_k
n`3lo?`0o?cl00<0oOgm0P3no_h40?ooo`0000D0000008Z:RP3Zj^X0i>CT0<o?c`3o0;jn_W00_[jn
00H0clo?0>CTi03Zj^X0RXZ:0000003lo?`50?gmo@80o_kn1P3oool000060000002d];@0W9bL0>OW
i`3emOD0lO7ao`3/k>a^0>c/k0060?7al@3emOD0inOW09bLW02d];@000001@3moOd20?knoPL0oooo
00001@000000oooo0;Bd]01YJFT0T92@0?l0ZZZZL02ZZZX01@2@T900JFUY0;Bd]03oool0000000@0
o_kn2P3oool00?l00000NP00000>0?ooo`00\
\>"],
  ImageRangeCache->{{{126.938, 399.625}, {597.813, 423.125}} -> {-181.795, \
606.555, 1.00181, 1.00181}},
  CellTags->"IPM3.1.2.7"],

Cell[TextData[{
  "Notice that in the main window, the Help Browser has displayed information \
about the ",
  StyleBox["ParametricPlot3D", "MR"],
  " function. This is identical to the usage message you would get if you \
entered ",
  StyleBox["?ParametricPlot3D", "MR"],
  "."
}], "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],

Cell["\<\
Alternatively, you could have clicked the Master Index tab and searched for \
\[OpenCurlyDoubleQuote]ParametricPlot3D\[CloseCurlyDoubleQuote] or even \
simply \[OpenCurlyDoubleQuote]parametric\[CloseCurlyDoubleQuote] and then \
browsed through the index to find what you were looking for.\
\>", "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],

Cell["\<\
Many additional features are available in the Help Browser and you are \
advised to consult your documentation for a complete list and description.\
\>", "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 685}},
AutoGeneratedPackage->None,
ScreenStyleEnvironment->"Working",
PrintingStyleEnvironment->"Production",
ShowPageBreaks->False,
WindowSize->{627, 579},
WindowMargins->{{1, Automatic}, {Automatic, -1}},
PrintingCopies->1,
PrintingStartingPageNumber->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"An Introduction to Programming with ", 
          StyleBox[ "Mathematica", "IT"]}], "Header"]}, {Cell[ 
        TextData[ {"1 An introduction to ", 
          StyleBox[ "Mathematica", "IT"]}], "Header"], Inherited, 
      Inherited}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False},
ShowSelection->True,
TaggingRules:>{"NextIndexingNumber" -> 62, "IndexEntries" -> {{"i:1", \
"Numerical computations"}, {"i:10", 
  StyleBox[ "ParametricPlot3D", "MR"]}, {"i:13", 
  StyleBox[ "ToFileName", "MR"]}, {"i:14", 
  StyleBox[ "Import", "MR"]}, {"i:15", 
  StyleBox[ 
  "ListPlot", 
    "MR"]}, {"i:16", "Log-log plots"}, {"i:17", "Fitting data to a model"}, \
{"i:18", "Combining plots"}, {"i:2", "Symbolic computations"}, {"i:20", 
  StyleBox[ "PerfectQ", "MR"]}, {"i:21", 
  StyleBox[ 
  "PerfectSearch", 
    "MR"]}, {"i:22", "Regular polygons"}, {"i:23", "Newton\[CloseCurlyQuote]s \
method"}, {"i:27", 
  StyleBox[ "NotebookPut", "MR"]}, {"i:28", 
  StyleBox[ "SelectionMove", "MR"]}, {"i:29", 
  StyleBox[ 
  "SelectionEvaluate", 
    "MR"]}, {"i:31", "Notebooks"}, {"i:32", "Evaluating input"}, {"i:33", \
"Cell brackets"}, {"i:34", "Entering input"}, {"i:35", {
  StyleBox[ "In", "MR"], " prompt"}}, {"i:35", {
  StyleBox[ "Out", "MR"], " prompt"}}, {"i:36", {"Quitting ", 
  StyleBox[ 
  "Mathematica", FontSlant -> 
    "Italic"], " session"}}, {"i:37", "Interrupting computations"}, {"i:38", \
"Aborting computations"}, {"i:4", 
  StyleBox[ "TrigReduce", "MR"]}, {"i:44", {
  StyleBox[ "CompoundExpression", "MR"], " (", 
  StyleBox[ 
  ";", "MR"], ")"}}, {"i:46", "Prefix input operator"}, {"i:47", "Postfix \
input operator"}, {"i:48", "Infix operator"}, {"i:50", 
  StyleBox[ "Intersection", "MR"]}, {"i:51", "Front end"}, {"i:52", 
  StyleBox[ "MathLink", FontSlant -> "Italic"]}, {"i:54", 
  StyleBox[ "Off", "MR"]}, {"i:55", 
  StyleBox[ "On", "MR"]}, {"i:58", {
  StyleBox[ 
  "?", "MR"], " (information escape)"}}, {"i:59", "Help Browser"}, {"i:6", 
  StyleBox[ 
  "Reduce", "MR"]}, {"i:60", "Simplifying algebraic expressions"}, {"i:61", 
  StyleBox[ "Simplify", "MR"]}, {"i:7", 
  StyleBox[ "Solve", "MR"]}, {"i:8", 
  StyleBox[ 
  "Integrate", 
    "MR"]}, {"i:1", "Computations", "numeric"}, {"i:10", "Parametric \
functions", "plotting"}, {"i:10", "Plotting", "functions represented \
parametrically"}, {"i:12", "Data", "importing"}, {"i:15", "Data", \
"plotting"}, {"i:16", "Data", "plotting log-log"}, {"i:17", "Data", "fitting \
to a model"}, {"i:18", "Plots", "combining"}, {"i:2", "Computations", \
"symbolic"}, {"i:22", "Polygons", "regular"}, {"i:23", "Root finding", \
"Newton\[CloseCurlyQuote]s method"}, {"i:25", "Programming", "symbolic \
documents"}, {"i:25", "Symbolic documents", "programming"}, {"i:26", \
"Programming", "buttons"}, {"i:30", 
  StyleBox[ "Mathematica", FontSlant -> "Italic"], "starting up"}, {"i:31", 
  StyleBox[ "Mathematica", FontSlant -> "Italic"], "notebooks"}, {"i:32", 
  StyleBox[ 
  "Mathematica", FontSlant -> 
    "Italic"], "evaluating input"}, {"i:34", "Input", "entering"}, {"i:34", \
"Input", "evaluating"}, {"i:36", 
  StyleBox[ 
  "Mathematica", FontSlant -> 
    "Italic"], "quitting session"}, {"i:37", "Computations", "interrupting"}, \
{"i:38", "Computations", "aborting"}, {"i:38", "Kernel", "killing"}, {"i:39", 
  "Entering input", "fractions"}, {"i:39", "Inputs", "syntax"}, {"i:40", \
"Entering input", "superscripts"}, {"i:41", "Typeset expressions", "entering \
from keyboard"}, {"i:42", "Precedence", "arithmetic operators"}, {"i:43", \
"Lists", "notation"}, {"i:45", "Input", "syntax"}, {"i:46", "Input", "prefix \
operator"}, {"i:47", "Input", "postfix operator"}, {"i:48", "Input", "infix \
operator"}, {"i:49", "Input", "traditional representations"}, {"i:49", \
"Mathematical expressions", "traditional representations"}, {"i:51", 
  StyleBox[ "Mathematica", FontSlant -> "Italic"], "front end"}, {"i:51", 
  StyleBox[ 
  "Mathematica", FontSlant -> 
    "Italic"], "kernel"}, {"i:53", "Errors", "mispelling"}, {"i:56", \
"Errors", "syntax"}, {"i:58", 
  StyleBox[ "Mathematica", FontSlant -> "Italic"], "getting help"}, {"i:59", 
  StyleBox[ 
  "Mathematica", FontSlant -> 
    "Italic"], "Help Browser"}, {"i:9", "Plotting", "functions of one \
variable"}, {"i:17", 
  StyleBox[ 
  "FindFit", "MR"], "", ""}, {"i:24", "Bowling program", "", ""}, {"i:27", 
  StyleBox[ "Notebook", "MR"], "", ""}, {"i:3", 
  StyleBox[ "Factor", "MR"], "", ""}, {"i:44", {
  StyleBox[ ";", "MR"], ", ", 
  StyleBox[ 
  "CompoundExpression", 
    "MR"]}, "", ""}, {"i:51", "Kernel", "", ""}, {"i:57", "Errors", \
"arguments to functions", ""}, {"i:9", 
  StyleBox[ 
  "Plot", "MR"], "", ""}}, "PaginationCache" -> {{2004, 9, 2, 17, 39, \
10.4935904`8.77349907010875}, {1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, \
2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, 3, 3, \
3, 3, 3, 3, 3, 4, 4, 4, 4, 4, 4, 4, 4, 4, 5, 5, 5, 5, 5, 5, 5, 5, 6, 6, 6, 6, \
6, 6, 6, 6, 6, 6, 7, 7, 7, 7, 7, 7, 7, 7, 7, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 9, \
9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 9, 10, 10, 10, 10, 10, 10, 10, 10, 10, \
10, 10, 11, 11, 11, 11, 11, 11, 11, 11, 11, 12, 12, 12, 12, 12, 12, 12, 13, \
13, 13, 13, 13, 13, 14, 14, 14, 14, 14, 14, 14, 14, 14, 15, 15, 15, 15, 15, \
15, 15, 15, 15, 15, 15, 15, 16, 16, 16, 16, 17, 17, 17, 17, 17, 17, 17, 17, \
18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, 18, \
19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 19, 20, 20, \
20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 20, 21, 21, 21, 21, 21, 21, 21, \
21, 21, 21, 21, 21, 21, 21, 22, 22, 22, 22, 22, 22, 22, 22, 22, 22, 22, 22, \
22, 22, 22, 22, 22, 23, 23, 23, 23, 23, 23, 23, 23, 23, 23, 23, 23, 23, 23, \
23, 23, 23, 23, 23, 23, 23, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, 24, \
24, 24, 24, 24, 24, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, 25, \
25, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 26, 27, 27, \
27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 27, 28, 28, 28, 28, 28, 29, 29, 30, \
30, 30, 30, 30, 30, 30}, Automatic}},
CellLabelAutoDelete->False,
Magnification->1,
StyleDefinitions -> "IPM3Styles.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "IPM3.1.0"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2135, 63, 181, 6, 98, "Chapter",
      CounterAssignments->{{"Chapter", 0}},
      CellTags->{"IPM3.1.0", "c:1"}],
    Cell[2319, 71, 1037, 24, 148, "ChapterAbstract",
      CellTags->"IPM3.1.0"]},
  "IPM3.1.1.1"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[3381, 99, 276, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
      CellTags->{
      "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
        "IPM3.1.1.6", "c:2"}],
    Cell[3682, 111, 95, 1, 44, "Subsection",
      CellTags->{"i:1", "b:0.0", "IPM3.1.1.1", "c:3"}],
    Cell[3780, 114, 257, 6, 52, "TextFirst",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4062, 124, 140, 3, 43, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4205, 129, 119, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4339, 135, 89, 1, 33, "TextFirst",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4453, 140, 104, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4560, 145, 98, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4695, 153, 104, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4802, 158, 98, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4937, 166, 103, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5043, 171, 97, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5177, 179, 112, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5292, 184, 98, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5405, 190, 286, 7, 52, "TextFirst",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5716, 201, 115, 3, 42, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5834, 206, 99, 3, 42, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5970, 214, 100, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[6073, 219, 250, 5, 67, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[6360, 229, 107, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[6470, 234, 625, 9, 162, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}]},
  "IPM3.1.1.2"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[3381, 99, 276, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
      CellTags->{
      "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
        "IPM3.1.1.6", "c:2"}],
    Cell[7144, 249, 94, 1, 44, "Subsection",
      CellTags->{"i:2", "b:0.1", "IPM3.1.1.2", "c:4"}],
    Cell[7241, 252, 319, 7, 52, "TextFirst",
      CellTags->{"b:0.1", "IPM3.1.1.2"}],
    Cell[7585, 263, 117, 3, 31, "Input",
      CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}],
    Cell[7705, 268, 152, 3, 29, "Output",
      CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}],
    Cell[7894, 276, 130, 3, 31, "Input",
      CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}],
    Cell[8027, 281, 150, 3, 42, "Output",
      CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}],
    Cell[8192, 287, 451, 14, 52, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:60"}],
    Cell[8668, 305, 158, 3, 30, "Input",
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}],
    Cell[8829, 310, 110, 3, 29, "Output",
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}],
    Cell[8954, 316, 179, 4, 33, "TextFirst",
      CellTags->{"b:0.1", "IPM3.1.1.2"}],
    Cell[9158, 324, 221, 4, 33, "Input",
      CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}],
    Cell[9382, 330, 260, 5, 46, "Output",
      CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}],
    Cell[9657, 338, 99, 1, 33, "TextFirst",
      CellTags->"IPM3.1.1.2"],
    Cell[9781, 343, 130, 3, 30, "Input",
      CellTags->"IPM3.1.1.2"],
    Cell[9914, 348, 357, 11, 51, "Output",
      CellTags->"IPM3.1.1.2"],
    Cell[10286, 362, 235, 7, 33, "TextFirst",
      CellTags->{"b:0.1", "IPM3.1.1.2"}],
    Cell[10546, 373, 132, 3, 31, "Input",
      CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}],
    Cell[10681, 378, 788, 12, 172, "Output",
      CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}],
    Cell[11506, 395, 150, 3, 42, "Input",
      CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}],
    Cell[11659, 400, 254, 5, 70, "Output",
      CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}]},
  "IPM3.1.1.3"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[3381, 99, 276, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
      CellTags->{
      "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
        "IPM3.1.1.6", "c:2"}],
    Cell[11962, 411, 74, 1, 44, "Subsection",
      CellTags->{"b:0.2", "IPM3.1.1.3", "c:5"}],
    Cell[12039, 414, 816, 16, 128, "TextFirst",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[12858, 432, 207, 6, 33, "Text",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[13090, 442, 155, 3, 33, "Input",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
    Cell[13248, 447, 55928, 1377, 193, 17886, 902, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
    Cell[69179, 1826, 199, 5, 29, "Output",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
    Cell[69393, 1834, 160, 4, 33, "TextFirst",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[69578, 1842, 143, 3, 30, "Input",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[69724, 1847, 51230, 1241, 195, 16127, 802, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[120969, 3091, 146, 4, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->"IPM3.1.1.3"],
    Cell[121140, 3099, 190, 5, 30, "Input",
      CellTags->"IPM3.1.1.3"],
    Cell[121333, 3106, 38115, 1041, 195, 16544, 769, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->"IPM3.1.1.3"],
    Cell[159463, 4150, 440, 14, 52, "TextFirst",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[159906, 4166, 285, 6, 83, "Input",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[160216, 4176, 315, 6, 50, "Input",
      CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}],
    Cell[160534, 4184, 202685, 8488, 335, 150201, 7835, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}]},
  "IPM3.1.1.4"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[3381, 99, 276, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
      CellTags->{
      "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
        "IPM3.1.1.6", "c:2"}],
    Cell[363268, 12678, 83, 1, 44, "Subsection",
      CellTags->{"b:0.3", "IPM3.1.1.4", "c:6"}],
    Cell[363354, 12681, 420, 8, 71, "TextFirst",
      CellTags->{"i:12", "b:0.3", "IPM3.1.1.4"}],
    Cell[363777, 12691, 699, 23, 71, "Text",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[364501, 12718, 217, 5, 70, "Input",
      CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}],
    Cell[364721, 12725, 218, 4, 48, "Output",
      CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}],
    Cell[364976, 12734, 139, 3, 30, "Input",
      CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}],
    Cell[365118, 12739, 400, 7, 86, "Output",
      CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}],
    Cell[365533, 12749, 172, 5, 33, "TextFirst",
      CellTags->{"i:15", "b:0.3", "IPM3.1.1.4"}],
    Cell[365730, 12758, 149, 3, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[365882, 12763, 29311, 818, 195, 13497, 617, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[395208, 13584, 140, 4, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:16", "b:0.3", "IPM3.1.1.4"}],
    Cell[395373, 13592, 175, 5, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[395551, 13599, 21459, 646, 195, 11260, 514, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[417025, 14248, 521, 16, 52, "TextFirst",
      CellTags->{"i:17", "b:0.3", "IPM3.1.1.4"}],
    Cell[417571, 14268, 138, 3, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[417712, 14273, 164, 4, 29, "Output",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[417891, 14280, 95, 1, 33, "Text",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[418011, 14285, 143, 3, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[418157, 14290, 23874, 692, 195, 11488, 533, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[442046, 14985, 213, 5, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:18", "b:0.3", "IPM3.1.1.4"}],
    Cell[442284, 14994, 120, 3, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[442407, 14999, 24429, 709, 195, 11739, 547, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.3", "IPM3.1.1.4"}]},
  "IPM3.1.1.5"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[3381, 99, 276, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
      CellTags->{
      "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
        "IPM3.1.1.6", "c:2"}],
    Cell[466885, 15714, 77, 1, 44, "Subsection",
      CellTags->{"b:0.4", "IPM3.1.1.5", "c:7"}],
    Cell[466965, 15717, 1246, 31, 166, "TextFirst",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[468214, 15750, 479, 10, 90, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[468696, 15762, 151, 3, 30, "Input",
      CellTags->{"i:20", "b:0.4", "IPM3.1.1.5"}],
    Cell[468850, 15767, 217, 6, 33, "TextFirst",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[469070, 15775, 150, 3, 30, "Input",
      CellTags->{"i:21", "b:0.4", "IPM3.1.1.5"}],
    Cell[469223, 15780, 138, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[469386, 15786, 115, 3, 31, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[469504, 15791, 114, 3, 29, "Output",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[469633, 15797, 403, 10, 71, "Text",
      CellTags->{"i:22", "b:0.4", "IPM3.1.1.5"}],
    Cell[470039, 15809, 254, 6, 63, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[470296, 15817, 222, 5, 50, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[470543, 15826, 118, 3, 30, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[470664, 15831, 10747, 175, 178, 733, 46, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[481414, 16008, 192, 5, 29, "Output",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[481621, 16016, 205, 4, 52, "Text",
      CellTags->{"i:23", "b:0.4", "IPM3.1.1.5"}],
    Cell[481829, 16022, 179, 4, 44, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482011, 16028, 148, 4, 30, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482184, 16036, 113, 3, 30, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482300, 16041, 115, 3, 29, "Output",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482430, 16047, 341, 6, 71, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482774, 16055, 611, 12, 130, "Input",
      CellTags->{"i:24", "b:0.4", "IPM3.1.1.5"}],
    Cell[483388, 16069, 181, 4, 33, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[483594, 16077, 166, 4, 30, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[483763, 16083, 99, 3, 29, "Output",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[483877, 16089, 771, 16, 128, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[484651, 16107, 347, 6, 71, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}]},
  "IPM3.1.1.6"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[3381, 99, 276, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
      CellTags->{
      "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
        "IPM3.1.1.6", "c:2"}],
    Cell[485035, 16118, 100, 1, 44, "Subsection",
      CellTags->{"b:0.5", "IPM3.1.1.6", "c:8"}],
    Cell[485138, 16121, 849, 16, 128, "TextFirst",
      CellTags->{"i:25", "b:0.5", "IPM3.1.1.6"}],
    Cell[485990, 16139, 305, 6, 71, "Text",
      CellTags->{"i:26", "b:0.5", "IPM3.1.1.6"}],
    Cell[486298, 16147, 127, 3, 30, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[486450, 16154, 134, 3, 30, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[486587, 16159, 46022, 846, 228, 6511, 353, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[532612, 17007, 194, 5, 29, "Output",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[532821, 17015, 151, 4, 33, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[532975, 17021, 360, 12, 195, "Program",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[533338, 17035, 287, 7, 52, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[533628, 17044, 443, 15, 24, "DisplayFormula",
      Evaluatable->False,
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[534074, 17061, 320, 9, 52, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[534397, 17072, 378, 8, 71, "Text",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[534778, 17082, 516, 16, 246, "Program",
      PageBreakAbove->True,
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[535297, 17100, 538, 12, 90, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[535838, 17114, 1434, 42, 41, "DisplayFormula",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[537297, 17160, 108, 3, 30, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[537408, 17165, 98, 3, 29, "Output",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[537521, 17171, 669, 12, 109, "Text",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[538193, 17185, 546, 15, 90, "Text",
      CellTags->{"i:27", "b:0.5", "IPM3.1.1.6"}],
    Cell[538764, 17204, 427, 9, 130, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[539194, 17215, 293, 8, 29, "Output",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[539490, 17225, 571347, 7848, 220, 479236, 6707, "GraphicsData", 
    "PostScript", "Picture",
      CellTags->"IPM3.1.1.6"],
    Cell[1110852, 25076, 136, 3, 33, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[1111013, 25083, 135, 3, 30, "Input",
      CellTags->{"i:28", "b:0.5", "IPM3.1.1.6"}],
    Cell[1111151, 25088, 571347, 7848, 220, 479236, 6707, "GraphicsData", 
    "PostScript", "Picture",
      CellTags->"IPM3.1.1.6"],
    Cell[1682513, 32939, 120, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[1682658, 32945, 128, 3, 30, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[1682789, 32950, 571578, 7851, 220, 479236, 6707, "GraphicsData", 
    "PostScript", "Picture",
      CellTags->"IPM3.1.1.6"],
    Cell[2254382, 40804, 97, 1, 33, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[2254504, 40809, 124, 3, 30, "Input",
      CellTags->{"i:29", "b:0.5", "IPM3.1.1.6"}],
    Cell[2254631, 40814, 578751, 7940, 220, 479236, 6707, "GraphicsData", 
    "PostScript", "Picture",
      CellTags->"IPM3.1.1.6"],
    Cell[2833397, 48757, 260, 5, 52, "Text",
      CellTags->{"b:0.5", "IPM3.1.1.6"}]},
  "IPM3.1.2.0"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
    Cell[2833999, 48778, 634, 15, 90, "TextFirst",
      CellTags->"IPM3.1.2.0"]},
  "IPM3.1.2.1"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
    Cell[2834658, 48797, 159, 5, 44, "Subsection",
      CellTags->{"b:1.0", "IPM3.1.2.1", "c:10"}],
    Cell[2834820, 48804, 488, 10, 90, "TextFirst",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2835311, 48816, 841, 20, 128, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2836177, 48840, 168, 6, 43, "Subsubsection",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2836348, 48848, 319, 9, 52, "TextFirst",
      CellTags->{"i:30", "b:1.0", "IPM3.1.2.1"}],
    Cell[2836670, 48859, 6733, 113, 51, 6586, 109, "GraphicsData", 
    "Bitmap", "Picture",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2843406, 48974, 401, 10, 71, "TextFirst",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2843810, 48986, 485, 8, 90, "Text",
      CellTags->{"i:31", "b:1.0", "IPM3.1.2.1"}],
    Cell[2844298, 48996, 761, 21, 90, "Text",
      CellTags->{"i:32", "b:1.0", "IPM3.1.2.1"}],
    Cell[3419477, 56975, 511, 16, 71, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[4287062, 68815, 645, 17, 90, "Text",
      CellTags->{"i:33", "b:1.0", "IPM3.1.2.1"}],
    Cell[4287710, 68834, 315, 9, 52, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5164154, 80779, 403, 9, 71, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5164560, 80790, 264, 7, 52, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5164861, 80802, 76, 1, 43, "Subsubsection",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5164940, 80805, 473, 8, 90, "TextFirst",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5165416, 80815, 381, 8, 52, "Text",
      CellTags->{"i:34", "b:1.0", "IPM3.1.2.1"}],
    Cell[5165800, 80825, 547, 10, 90, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5166372, 80839, 142, 3, 42, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5166517, 80844, 182, 4, 63, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5166714, 80851, 378, 13, 52, "Text",
      CellTags->{"i:35", "b:1.0", "IPM3.1.2.1"}],
    Cell[5167095, 80866, 174, 6, 33, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167294, 80876, 99, 3, 30, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167396, 80881, 115, 3, 29, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167548, 80889, 99, 3, 30, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167650, 80894, 115, 3, 29, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167780, 80900, 345, 12, 33, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168150, 80916, 100, 3, 30, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168253, 80921, 182, 4, 63, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168472, 80930, 96, 3, 30, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168571, 80935, 115, 3, 29, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168735, 80944, 153, 6, 43, "Subsubsection",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168891, 80952, 306, 10, 52, "TextFirst",
      CellTags->{"i:36", "b:1.0", "IPM3.1.2.1"}],
    Cell[5169234, 80967, 84, 1, 43, "Subsubsection",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5169321, 80970, 626, 11, 90, "TextFirst",
      CellTags->{"i:37", "b:1.0", "IPM3.1.2.1"}],
    Cell[5169950, 80983, 237, 6, 31, "BulletedList",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5170190, 80991, 206, 5, 31, "BulletedList",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5170399, 80998, 228, 7, 31, "BulletedList",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5170630, 81007, 1029, 21, 147, "Text",
      CellTags->{"i:38", "b:1.0", "IPM3.1.2.1"}]},
  "IPM3.1.2.2"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
    Cell[5171708, 81034, 87, 1, 44, "Subsection",
      CellTags->{"b:1.1", "IPM3.1.2.2", "c:11"}],
    Cell[5171798, 81037, 201, 4, 52, "TextFirst",
      CellTags->{"i:39", "b:1.1", "IPM3.1.2.2"}],
    Cell[5172024, 81045, 99, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172126, 81050, 96, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172237, 81056, 296, 10, 33, "TextFirst",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172558, 81070, 100, 3, 42, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172661, 81075, 96, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172772, 81081, 143, 5, 33, "TextFirst",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172940, 81090, 97, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5173040, 81095, 97, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5173152, 81101, 304, 10, 33, "TextFirst",
      CellTags->{"i:40", "b:1.1", "IPM3.1.2.2"}],
    Cell[5173481, 81115, 98, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5173582, 81120, 97, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5173694, 81126, 605, 12, 109, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5174302, 81140, 5084, 167, 155, "DisplayTable",
      CellTags->{"i:41", "b:1.1", "IPM3.1.2.2"}],
    Cell[5179389, 81309, 90, 1, 22, "NumberedTable",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5179482, 81312, 304, 7, 52, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5179811, 81323, 99, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5179913, 81328, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180048, 81336, 98, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180149, 81341, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180262, 81347, 361, 9, 52, "Text",
      CellTags->{"i:42", "b:1.1", "IPM3.1.2.2"}],
    Cell[5180648, 81360, 103, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180754, 81365, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180867, 81371, 245, 6, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5181137, 81381, 111, 3, 31, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5181251, 81386, 146, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5181412, 81392, 687, 16, 109, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5182124, 81412, 126, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5182253, 81417, 116, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5182384, 81423, 687, 22, 90, "Text",
      CellTags->{"i:43", "b:1.1", "IPM3.1.2.2"}],
    Cell[5183096, 81449, 116, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5183215, 81454, 114, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5183366, 81462, 165, 4, 33, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5183534, 81468, 44186, 1154, 180, 16464, 806, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5227757, 82627, 107, 2, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5227867, 82631, 116, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5227998, 82637, 884, 29, 71, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5228885, 82668, 218, 6, 33, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5229106, 82676, 538, 12, 90, "Text",
      CellTags->{"i:44", "b:1.1", "IPM3.1.2.2"}],
    Cell[5229647, 82690, 119, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5229791, 82697, 96, 2, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5229890, 82701, 104, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230031, 82709, 110, 3, 44, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230144, 82714, 104, 3, 42, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230263, 82720, 361, 11, 52, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230649, 82735, 93, 6, 66, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230745, 82743, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230858, 82749, 192, 7, 33, "TextFirst",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231075, 82760, 91, 5, 48, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231169, 82767, 97, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231281, 82773, 109, 1, 33, "TextFirst",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231415, 82778, 94, 5, 48, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231512, 82785, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231625, 82791, 379, 9, 52, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5232007, 82802, 331, 13, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5232363, 82819, 172, 5, 48, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5232538, 82826, 102, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}]},
  "IPM3.1.2.3"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
    Cell[5232689, 82835, 89, 1, 44, "Subsection",
      CellTags->{"b:1.2", "IPM3.1.2.3", "c:12"}],
    Cell[5232781, 82838, 455, 12, 52, "TextFirst",
      CellTags->{"i:45", "b:1.2", "IPM3.1.2.3"}],
    Cell[5233239, 82852, 160, 4, 33, "Text",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5233424, 82860, 103, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5233530, 82865, 114, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5233659, 82871, 94, 1, 33, "TextFirst",
      CellTags->{"i:46", "b:1.2", "IPM3.1.2.3"}],
    Cell[5233778, 82876, 102, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5233883, 82881, 114, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234012, 82887, 102, 1, 33, "TextFirst",
      CellTags->{"i:47", "b:1.2", "IPM3.1.2.3"}],
    Cell[5234139, 82892, 105, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234247, 82897, 114, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234376, 82903, 225, 6, 52, "Text",
      CellTags->{"i:48", "b:1.2", "IPM3.1.2.3"}],
    Cell[5234626, 82913, 105, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234734, 82918, 151, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234900, 82924, 300, 8, 52, "Text",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5235225, 82936, 117, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5235345, 82941, 132, 3, 40, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5235492, 82947, 212, 5, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:49", "b:1.2", "IPM3.1.2.3"}],
    Cell[5235729, 82956, 134, 3, 44, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5235866, 82961, 132, 3, 40, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236013, 82967, 365, 8, 71, "Text",
      CellTags->{"i:50", "b:1.2", "IPM3.1.2.3"}],
    Cell[5236403, 82979, 138, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236544, 82984, 102, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236661, 82990, 133, 3, 33, "TextFirst",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236819, 82997, 139, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236961, 83002, 102, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5237078, 83008, 277, 5, 52, "Text",
      CellTags->{"b:1.2", "IPM3.1.2.3"}]},
  "IPM3.1.2.4"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
    Cell[5237392, 83018, 95, 1, 44, "Subsection",
      CellTags->{"b:1.3", "IPM3.1.2.4", "c:13"}],
    Cell[5237490, 83021, 849, 19, 128, "TextFirst",
      CellTags->{"i:51", "b:1.3", "IPM3.1.2.4"}],
    Cell[5238342, 83042, 100, 1, 31, "BulletedList",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5238445, 83045, 159, 4, 31, "BulletedList",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5238607, 83051, 140, 3, 31, "BulletedList",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5238750, 83056, 108, 1, 31, "BulletedList",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5238861, 83059, 645, 15, 90, "Text",
      CellTags->{"i:52", "b:1.3", "IPM3.1.2.4"}],
    Cell[5239509, 83076, 490, 11, 90, "Text",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5240002, 83089, 294, 7, 52, "Text",
      CellTags->{"b:1.3", "IPM3.1.2.4"}]},
  "IPM3.1.2.5"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
    Cell[5240333, 83101, 73, 1, 44, "Subsection",
      CellTags->{"b:1.4", "IPM3.1.2.5", "c:14"}],
    Cell[5240409, 83104, 649, 16, 90, "TextFirst",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5241061, 83122, 241, 5, 52, "Text",
      CellTags->{"i:53", "b:1.4", "IPM3.1.2.5"}],
    Cell[5241327, 83131, 104, 3, 30, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5241434, 83136, 427, 7, 55, "Message",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5241864, 83145, 106, 3, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5241985, 83151, 428, 11, 71, "Text",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5242416, 83164, 122, 3, 30, "Input",
      CellTags->{"i:54", "b:1.4", "IPM3.1.2.5"}],
    Cell[5242541, 83169, 306, 8, 52, "Text",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5242872, 83181, 113, 3, 31, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5242988, 83186, 114, 3, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5243117, 83192, 197, 6, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5243317, 83200, 121, 3, 30, "Input",
      CellTags->{"i:55", "b:1.4", "IPM3.1.2.5"}],
    Cell[5243441, 83205, 684, 14, 109, "Text",
      CellTags->{"i:56", "b:1.4", "IPM3.1.2.5"}],
    Cell[5244150, 83223, 157, 4, 30, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5244310, 83229, 163, 4, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5244488, 83236, 596, 12, 109, "Text",
      CellTags->{"i:57", "b:1.4", "IPM3.1.2.5"}],
    Cell[5245109, 83252, 108, 3, 30, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5245220, 83257, 393, 6, 37, "Message",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5245616, 83265, 109, 3, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5245762, 83273, 114, 3, 30, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5245879, 83278, 396, 7, 37, "Message",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5246278, 83287, 116, 3, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}]},
  "IPM3.1.2.6"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
    Cell[5246443, 83296, 79, 1, 44, "Subsection",
      CellTags->{"b:1.5", "IPM3.1.2.6", "c:15"}],
    Cell[5246525, 83299, 504, 11, 71, "TextFirst",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5247032, 83312, 345, 8, 52, "Text",
      CellTags->{"i:58", "b:1.5", "IPM3.1.2.6"}],
    Cell[5247402, 83324, 94, 2, 30, "Input",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5247499, 83328, 529, 11, 77, "Print",
      CellTags->{"b:1.5", "Info3302590721-5746852", "IPM3.1.2.6"}],
    Cell[5248043, 83342, 294, 8, 52, "Text",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5248362, 83354, 105, 3, 30, "Input",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5248470, 83359, 3822, 96, 84, "Print",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5252307, 83458, 294, 6, 52, "Text",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5252604, 83466, 571, 11, 99, "Print",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5253178, 83479, 251, 6, 52, "Text",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5253432, 83487, 340, 9, 52, "Text",
      CellTags->{"b:1.5", "IPM3.1.2.6"}]},
  "IPM3.1.2.7"->{
    Cell[1754, 51, 356, 8, 29, "TitleBar",
      CellTags->{
      "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
        "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
        "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", 
        "IPM3.1.2.7"}],
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
    Cell[5253809, 83501, 83, 1, 44, "Subsection",
      CellTags->{"b:1.6", "IPM3.1.2.7", "c:16"}],
    Cell[5253895, 83504, 327, 7, 52, "TextFirst",
      CellTags->{"i:59", "b:1.6", "IPM3.1.2.7"}],
    Cell[5254225, 83513, 256, 7, 52, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[5254484, 83522, 687612, 9445, 263, 597963, 8334, "GraphicsData", 
    "PostScript", "Picture",
      CellTags->"IPM3.1.2.7"],
    Cell[5942099, 92969, 615, 17, 90, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[5942717, 92988, 320, 6, 71, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[5943040, 92996, 731532, 9987, 263, 597963, 8334, "GraphicsData", 
    "PostScript", "Picture",
      CellTags->"IPM3.1.2.7"],
    Cell[6674575, 102985, 327, 9, 52, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[6674905, 102996, 349, 6, 52, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[6675257, 103004, 208, 4, 52, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}]},
  "c:1"->{
    Cell[2135, 63, 181, 6, 98, "Chapter",
      CounterAssignments->{{"Chapter", 0}},
      CellTags->{"IPM3.1.0", "c:1"}]},
  "c:2"->{
    Cell[3381, 99, 276, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
      CellTags->{
      "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
        "IPM3.1.1.6", "c:2"}]},
  "i:1"->{
    Cell[3682, 111, 95, 1, 44, "Subsection",
      CellTags->{"i:1", "b:0.0", "IPM3.1.1.1", "c:3"}]},
  "b:0.0"->{
    Cell[3682, 111, 95, 1, 44, "Subsection",
      CellTags->{"i:1", "b:0.0", "IPM3.1.1.1", "c:3"}],
    Cell[3780, 114, 257, 6, 52, "TextFirst",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4062, 124, 140, 3, 43, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4205, 129, 119, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4339, 135, 89, 1, 33, "TextFirst",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4453, 140, 104, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4560, 145, 98, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4695, 153, 104, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4802, 158, 98, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[4937, 166, 103, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5043, 171, 97, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5177, 179, 112, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5292, 184, 98, 3, 29, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5405, 190, 286, 7, 52, "TextFirst",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5716, 201, 115, 3, 42, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5834, 206, 99, 3, 42, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[5970, 214, 100, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[6073, 219, 250, 5, 67, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[6360, 229, 107, 3, 30, "Input",
      CellTags->{"b:0.0", "IPM3.1.1.1"}],
    Cell[6470, 234, 625, 9, 162, "Output",
      CellTags->{"b:0.0", "IPM3.1.1.1"}]},
  "c:3"->{
    Cell[3682, 111, 95, 1, 44, "Subsection",
      CellTags->{"i:1", "b:0.0", "IPM3.1.1.1", "c:3"}]},
  "i:2"->{
    Cell[7144, 249, 94, 1, 44, "Subsection",
      CellTags->{"i:2", "b:0.1", "IPM3.1.1.2", "c:4"}]},
  "b:0.1"->{
    Cell[7144, 249, 94, 1, 44, "Subsection",
      CellTags->{"i:2", "b:0.1", "IPM3.1.1.2", "c:4"}],
    Cell[7241, 252, 319, 7, 52, "TextFirst",
      CellTags->{"b:0.1", "IPM3.1.1.2"}],
    Cell[7585, 263, 117, 3, 31, "Input",
      CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}],
    Cell[7705, 268, 152, 3, 29, "Output",
      CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}],
    Cell[7894, 276, 130, 3, 31, "Input",
      CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}],
    Cell[8027, 281, 150, 3, 42, "Output",
      CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}],
    Cell[8192, 287, 451, 14, 52, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:60"}],
    Cell[8668, 305, 158, 3, 30, "Input",
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}],
    Cell[8829, 310, 110, 3, 29, "Output",
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}],
    Cell[8954, 316, 179, 4, 33, "TextFirst",
      CellTags->{"b:0.1", "IPM3.1.1.2"}],
    Cell[9158, 324, 221, 4, 33, "Input",
      CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}],
    Cell[9382, 330, 260, 5, 46, "Output",
      CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}],
    Cell[10286, 362, 235, 7, 33, "TextFirst",
      CellTags->{"b:0.1", "IPM3.1.1.2"}],
    Cell[10546, 373, 132, 3, 31, "Input",
      CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}],
    Cell[10681, 378, 788, 12, 172, "Output",
      CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}],
    Cell[11506, 395, 150, 3, 42, "Input",
      CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}],
    Cell[11659, 400, 254, 5, 70, "Output",
      CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}]},
  "c:4"->{
    Cell[7144, 249, 94, 1, 44, "Subsection",
      CellTags->{"i:2", "b:0.1", "IPM3.1.1.2", "c:4"}]},
  "i:3"->{
    Cell[7585, 263, 117, 3, 31, "Input",
      CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}],
    Cell[7705, 268, 152, 3, 29, "Output",
      CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}]},
  "i:4"->{
    Cell[7894, 276, 130, 3, 31, "Input",
      CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}],
    Cell[8027, 281, 150, 3, 42, "Output",
      CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}]},
  "i:60"->{
    Cell[8192, 287, 451, 14, 52, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:60"}]},
  "i:61"->{
    Cell[8668, 305, 158, 3, 30, "Input",
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}],
    Cell[8829, 310, 110, 3, 29, "Output",
      CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}]},
  "i:6"->{
    Cell[9158, 324, 221, 4, 33, "Input",
      CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}],
    Cell[9382, 330, 260, 5, 46, "Output",
      CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}]},
  "i:7"->{
    Cell[10546, 373, 132, 3, 31, "Input",
      CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}],
    Cell[10681, 378, 788, 12, 172, "Output",
      CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}]},
  "i:8"->{
    Cell[11506, 395, 150, 3, 42, "Input",
      CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}],
    Cell[11659, 400, 254, 5, 70, "Output",
      CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}]},
  "b:0.2"->{
    Cell[11962, 411, 74, 1, 44, "Subsection",
      CellTags->{"b:0.2", "IPM3.1.1.3", "c:5"}],
    Cell[12039, 414, 816, 16, 128, "TextFirst",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[12858, 432, 207, 6, 33, "Text",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[13090, 442, 155, 3, 33, "Input",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
    Cell[13248, 447, 55928, 1377, 193, 17886, 902, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
    Cell[69179, 1826, 199, 5, 29, "Output",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
    Cell[69393, 1834, 160, 4, 33, "TextFirst",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[69578, 1842, 143, 3, 30, "Input",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[69724, 1847, 51230, 1241, 195, 16127, 802, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[159463, 4150, 440, 14, 52, "TextFirst",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[159906, 4166, 285, 6, 83, "Input",
      CellTags->{"b:0.2", "IPM3.1.1.3"}],
    Cell[160216, 4176, 315, 6, 50, "Input",
      CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}],
    Cell[160534, 4184, 202685, 8488, 335, 150201, 7835, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}]},
  "c:5"->{
    Cell[11962, 411, 74, 1, 44, "Subsection",
      CellTags->{"b:0.2", "IPM3.1.1.3", "c:5"}]},
  "i:9"->{
    Cell[13090, 442, 155, 3, 33, "Input",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
    Cell[13248, 447, 55928, 1377, 193, 17886, 902, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
    Cell[69179, 1826, 199, 5, 29, "Output",
      CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}]},
  "i:10"->{
    Cell[160216, 4176, 315, 6, 50, "Input",
      CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}],
    Cell[160534, 4184, 202685, 8488, 335, 150201, 7835, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}]},
  "b:0.3"->{
    Cell[363268, 12678, 83, 1, 44, "Subsection",
      CellTags->{"b:0.3", "IPM3.1.1.4", "c:6"}],
    Cell[363354, 12681, 420, 8, 71, "TextFirst",
      CellTags->{"i:12", "b:0.3", "IPM3.1.1.4"}],
    Cell[363777, 12691, 699, 23, 71, "Text",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[364501, 12718, 217, 5, 70, "Input",
      CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}],
    Cell[364721, 12725, 218, 4, 48, "Output",
      CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}],
    Cell[364976, 12734, 139, 3, 30, "Input",
      CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}],
    Cell[365118, 12739, 400, 7, 86, "Output",
      CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}],
    Cell[365533, 12749, 172, 5, 33, "TextFirst",
      CellTags->{"i:15", "b:0.3", "IPM3.1.1.4"}],
    Cell[365730, 12758, 149, 3, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[365882, 12763, 29311, 818, 195, 13497, 617, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[395208, 13584, 140, 4, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:16", "b:0.3", "IPM3.1.1.4"}],
    Cell[395373, 13592, 175, 5, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[395551, 13599, 21459, 646, 195, 11260, 514, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[417025, 14248, 521, 16, 52, "TextFirst",
      CellTags->{"i:17", "b:0.3", "IPM3.1.1.4"}],
    Cell[417571, 14268, 138, 3, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[417712, 14273, 164, 4, 29, "Output",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[417891, 14280, 95, 1, 33, "Text",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[418011, 14285, 143, 3, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[418157, 14290, 23874, 692, 195, 11488, 533, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[442046, 14985, 213, 5, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:18", "b:0.3", "IPM3.1.1.4"}],
    Cell[442284, 14994, 120, 3, 30, "Input",
      CellTags->{"b:0.3", "IPM3.1.1.4"}],
    Cell[442407, 14999, 24429, 709, 195, 11739, 547, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.3", "IPM3.1.1.4"}]},
  "c:6"->{
    Cell[363268, 12678, 83, 1, 44, "Subsection",
      CellTags->{"b:0.3", "IPM3.1.1.4", "c:6"}]},
  "i:12"->{
    Cell[363354, 12681, 420, 8, 71, "TextFirst",
      CellTags->{"i:12", "b:0.3", "IPM3.1.1.4"}]},
  "i:13"->{
    Cell[364501, 12718, 217, 5, 70, "Input",
      CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}],
    Cell[364721, 12725, 218, 4, 48, "Output",
      CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}]},
  "i:14"->{
    Cell[364976, 12734, 139, 3, 30, "Input",
      CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}],
    Cell[365118, 12739, 400, 7, 86, "Output",
      CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}]},
  "i:15"->{
    Cell[365533, 12749, 172, 5, 33, "TextFirst",
      CellTags->{"i:15", "b:0.3", "IPM3.1.1.4"}]},
  "i:16"->{
    Cell[395208, 13584, 140, 4, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:16", "b:0.3", "IPM3.1.1.4"}]},
  "i:17"->{
    Cell[417025, 14248, 521, 16, 52, "TextFirst",
      CellTags->{"i:17", "b:0.3", "IPM3.1.1.4"}]},
  "i:18"->{
    Cell[442046, 14985, 213, 5, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:18", "b:0.3", "IPM3.1.1.4"}]},
  "b:0.4"->{
    Cell[466885, 15714, 77, 1, 44, "Subsection",
      CellTags->{"b:0.4", "IPM3.1.1.5", "c:7"}],
    Cell[466965, 15717, 1246, 31, 166, "TextFirst",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[468214, 15750, 479, 10, 90, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[468696, 15762, 151, 3, 30, "Input",
      CellTags->{"i:20", "b:0.4", "IPM3.1.1.5"}],
    Cell[468850, 15767, 217, 6, 33, "TextFirst",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[469070, 15775, 150, 3, 30, "Input",
      CellTags->{"i:21", "b:0.4", "IPM3.1.1.5"}],
    Cell[469223, 15780, 138, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[469386, 15786, 115, 3, 31, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[469504, 15791, 114, 3, 29, "Output",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[469633, 15797, 403, 10, 71, "Text",
      CellTags->{"i:22", "b:0.4", "IPM3.1.1.5"}],
    Cell[470039, 15809, 254, 6, 63, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[470296, 15817, 222, 5, 50, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[470543, 15826, 118, 3, 30, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[470664, 15831, 10747, 175, 178, 733, 46, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[481414, 16008, 192, 5, 29, "Output",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[481621, 16016, 205, 4, 52, "Text",
      CellTags->{"i:23", "b:0.4", "IPM3.1.1.5"}],
    Cell[481829, 16022, 179, 4, 44, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482011, 16028, 148, 4, 30, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482184, 16036, 113, 3, 30, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482300, 16041, 115, 3, 29, "Output",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482430, 16047, 341, 6, 71, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[482774, 16055, 611, 12, 130, "Input",
      CellTags->{"i:24", "b:0.4", "IPM3.1.1.5"}],
    Cell[483388, 16069, 181, 4, 33, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[483594, 16077, 166, 4, 30, "Input",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[483763, 16083, 99, 3, 29, "Output",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[483877, 16089, 771, 16, 128, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}],
    Cell[484651, 16107, 347, 6, 71, "Text",
      CellTags->{"b:0.4", "IPM3.1.1.5"}]},
  "c:7"->{
    Cell[466885, 15714, 77, 1, 44, "Subsection",
      CellTags->{"b:0.4", "IPM3.1.1.5", "c:7"}]},
  "i:20"->{
    Cell[468696, 15762, 151, 3, 30, "Input",
      CellTags->{"i:20", "b:0.4", "IPM3.1.1.5"}]},
  "i:21"->{
    Cell[469070, 15775, 150, 3, 30, "Input",
      CellTags->{"i:21", "b:0.4", "IPM3.1.1.5"}]},
  "i:22"->{
    Cell[469633, 15797, 403, 10, 71, "Text",
      CellTags->{"i:22", "b:0.4", "IPM3.1.1.5"}]},
  "i:23"->{
    Cell[481621, 16016, 205, 4, 52, "Text",
      CellTags->{"i:23", "b:0.4", "IPM3.1.1.5"}]},
  "i:24"->{
    Cell[482774, 16055, 611, 12, 130, "Input",
      CellTags->{"i:24", "b:0.4", "IPM3.1.1.5"}]},
  "b:0.5"->{
    Cell[485035, 16118, 100, 1, 44, "Subsection",
      CellTags->{"b:0.5", "IPM3.1.1.6", "c:8"}],
    Cell[485138, 16121, 849, 16, 128, "TextFirst",
      CellTags->{"i:25", "b:0.5", "IPM3.1.1.6"}],
    Cell[485990, 16139, 305, 6, 71, "Text",
      CellTags->{"i:26", "b:0.5", "IPM3.1.1.6"}],
    Cell[486298, 16147, 127, 3, 30, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[486450, 16154, 134, 3, 30, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[486587, 16159, 46022, 846, 228, 6511, 353, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[532612, 17007, 194, 5, 29, "Output",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[532821, 17015, 151, 4, 33, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[532975, 17021, 360, 12, 195, "Program",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[533338, 17035, 287, 7, 52, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[533628, 17044, 443, 15, 24, "DisplayFormula",
      Evaluatable->False,
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[534074, 17061, 320, 9, 52, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[534397, 17072, 378, 8, 71, "Text",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[534778, 17082, 516, 16, 246, "Program",
      PageBreakAbove->True,
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[535297, 17100, 538, 12, 90, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[535838, 17114, 1434, 42, 41, "DisplayFormula",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[537297, 17160, 108, 3, 30, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[537408, 17165, 98, 3, 29, "Output",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[537521, 17171, 669, 12, 109, "Text",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[538193, 17185, 546, 15, 90, "Text",
      CellTags->{"i:27", "b:0.5", "IPM3.1.1.6"}],
    Cell[538764, 17204, 427, 9, 130, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[539194, 17215, 293, 8, 29, "Output",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[1110852, 25076, 136, 3, 33, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[1111013, 25083, 135, 3, 30, "Input",
      CellTags->{"i:28", "b:0.5", "IPM3.1.1.6"}],
    Cell[1682513, 32939, 120, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[1682658, 32945, 128, 3, 30, "Input",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[2254382, 40804, 97, 1, 33, "TextFirst",
      CellTags->{"b:0.5", "IPM3.1.1.6"}],
    Cell[2254504, 40809, 124, 3, 30, "Input",
      CellTags->{"i:29", "b:0.5", "IPM3.1.1.6"}],
    Cell[2833397, 48757, 260, 5, 52, "Text",
      CellTags->{"b:0.5", "IPM3.1.1.6"}]},
  "c:8"->{
    Cell[485035, 16118, 100, 1, 44, "Subsection",
      CellTags->{"b:0.5", "IPM3.1.1.6", "c:8"}]},
  "i:25"->{
    Cell[485138, 16121, 849, 16, 128, "TextFirst",
      CellTags->{"i:25", "b:0.5", "IPM3.1.1.6"}]},
  "i:26"->{
    Cell[485990, 16139, 305, 6, 71, "Text",
      CellTags->{"i:26", "b:0.5", "IPM3.1.1.6"}]},
  "i:27"->{
    Cell[538193, 17185, 546, 15, 90, "Text",
      CellTags->{"i:27", "b:0.5", "IPM3.1.1.6"}]},
  "i:28"->{
    Cell[1111013, 25083, 135, 3, 30, "Input",
      CellTags->{"i:28", "b:0.5", "IPM3.1.1.6"}]},
  "i:29"->{
    Cell[2254504, 40809, 124, 3, 30, "Input",
      CellTags->{"i:29", "b:0.5", "IPM3.1.1.6"}]},
  "c:9"->{
    Cell[2833706, 48768, 290, 8, 41, "Section",
      CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
      CellTags->{
      "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
        "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}]},
  "b:1.0"->{
    Cell[2834658, 48797, 159, 5, 44, "Subsection",
      CellTags->{"b:1.0", "IPM3.1.2.1", "c:10"}],
    Cell[2834820, 48804, 488, 10, 90, "TextFirst",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2835311, 48816, 841, 20, 128, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2836177, 48840, 168, 6, 43, "Subsubsection",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2836348, 48848, 319, 9, 52, "TextFirst",
      CellTags->{"i:30", "b:1.0", "IPM3.1.2.1"}],
    Cell[2836670, 48859, 6733, 113, 51, 6586, 109, "GraphicsData", 
    "Bitmap", "Picture",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2843406, 48974, 401, 10, 71, "TextFirst",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[2843810, 48986, 485, 8, 90, "Text",
      CellTags->{"i:31", "b:1.0", "IPM3.1.2.1"}],
    Cell[2844298, 48996, 761, 21, 90, "Text",
      CellTags->{"i:32", "b:1.0", "IPM3.1.2.1"}],
    Cell[3419477, 56975, 511, 16, 71, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[4287062, 68815, 645, 17, 90, "Text",
      CellTags->{"i:33", "b:1.0", "IPM3.1.2.1"}],
    Cell[4287710, 68834, 315, 9, 52, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5164154, 80779, 403, 9, 71, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5164560, 80790, 264, 7, 52, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5164861, 80802, 76, 1, 43, "Subsubsection",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5164940, 80805, 473, 8, 90, "TextFirst",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5165416, 80815, 381, 8, 52, "Text",
      CellTags->{"i:34", "b:1.0", "IPM3.1.2.1"}],
    Cell[5165800, 80825, 547, 10, 90, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5166372, 80839, 142, 3, 42, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5166517, 80844, 182, 4, 63, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5166714, 80851, 378, 13, 52, "Text",
      CellTags->{"i:35", "b:1.0", "IPM3.1.2.1"}],
    Cell[5167095, 80866, 174, 6, 33, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167294, 80876, 99, 3, 30, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167396, 80881, 115, 3, 29, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167548, 80889, 99, 3, 30, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167650, 80894, 115, 3, 29, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5167780, 80900, 345, 12, 33, "Text",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168150, 80916, 100, 3, 30, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168253, 80921, 182, 4, 63, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168472, 80930, 96, 3, 30, "Input",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168571, 80935, 115, 3, 29, "Output",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168735, 80944, 153, 6, 43, "Subsubsection",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5168891, 80952, 306, 10, 52, "TextFirst",
      CellTags->{"i:36", "b:1.0", "IPM3.1.2.1"}],
    Cell[5169234, 80967, 84, 1, 43, "Subsubsection",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5169321, 80970, 626, 11, 90, "TextFirst",
      CellTags->{"i:37", "b:1.0", "IPM3.1.2.1"}],
    Cell[5169950, 80983, 237, 6, 31, "BulletedList",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5170190, 80991, 206, 5, 31, "BulletedList",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5170399, 80998, 228, 7, 31, "BulletedList",
      CellTags->{"b:1.0", "IPM3.1.2.1"}],
    Cell[5170630, 81007, 1029, 21, 147, "Text",
      CellTags->{"i:38", "b:1.0", "IPM3.1.2.1"}]},
  "c:10"->{
    Cell[2834658, 48797, 159, 5, 44, "Subsection",
      CellTags->{"b:1.0", "IPM3.1.2.1", "c:10"}]},
  "i:30"->{
    Cell[2836348, 48848, 319, 9, 52, "TextFirst",
      CellTags->{"i:30", "b:1.0", "IPM3.1.2.1"}]},
  "i:31"->{
    Cell[2843810, 48986, 485, 8, 90, "Text",
      CellTags->{"i:31", "b:1.0", "IPM3.1.2.1"}]},
  "i:32"->{
    Cell[2844298, 48996, 761, 21, 90, "Text",
      CellTags->{"i:32", "b:1.0", "IPM3.1.2.1"}]},
  "i:33"->{
    Cell[4287062, 68815, 645, 17, 90, "Text",
      CellTags->{"i:33", "b:1.0", "IPM3.1.2.1"}]},
  "i:34"->{
    Cell[5165416, 80815, 381, 8, 52, "Text",
      CellTags->{"i:34", "b:1.0", "IPM3.1.2.1"}]},
  "i:35"->{
    Cell[5166714, 80851, 378, 13, 52, "Text",
      CellTags->{"i:35", "b:1.0", "IPM3.1.2.1"}]},
  "i:36"->{
    Cell[5168891, 80952, 306, 10, 52, "TextFirst",
      CellTags->{"i:36", "b:1.0", "IPM3.1.2.1"}]},
  "i:37"->{
    Cell[5169321, 80970, 626, 11, 90, "TextFirst",
      CellTags->{"i:37", "b:1.0", "IPM3.1.2.1"}]},
  "i:38"->{
    Cell[5170630, 81007, 1029, 21, 147, "Text",
      CellTags->{"i:38", "b:1.0", "IPM3.1.2.1"}]},
  "b:1.1"->{
    Cell[5171708, 81034, 87, 1, 44, "Subsection",
      CellTags->{"b:1.1", "IPM3.1.2.2", "c:11"}],
    Cell[5171798, 81037, 201, 4, 52, "TextFirst",
      CellTags->{"i:39", "b:1.1", "IPM3.1.2.2"}],
    Cell[5172024, 81045, 99, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172126, 81050, 96, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172237, 81056, 296, 10, 33, "TextFirst",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172558, 81070, 100, 3, 42, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172661, 81075, 96, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172772, 81081, 143, 5, 33, "TextFirst",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5172940, 81090, 97, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5173040, 81095, 97, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5173152, 81101, 304, 10, 33, "TextFirst",
      CellTags->{"i:40", "b:1.1", "IPM3.1.2.2"}],
    Cell[5173481, 81115, 98, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5173582, 81120, 97, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5173694, 81126, 605, 12, 109, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5174302, 81140, 5084, 167, 155, "DisplayTable",
      CellTags->{"i:41", "b:1.1", "IPM3.1.2.2"}],
    Cell[5179389, 81309, 90, 1, 22, "NumberedTable",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5179482, 81312, 304, 7, 52, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5179811, 81323, 99, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5179913, 81328, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180048, 81336, 98, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180149, 81341, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180262, 81347, 361, 9, 52, "Text",
      CellTags->{"i:42", "b:1.1", "IPM3.1.2.2"}],
    Cell[5180648, 81360, 103, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180754, 81365, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5180867, 81371, 245, 6, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5181137, 81381, 111, 3, 31, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5181251, 81386, 146, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5181412, 81392, 687, 16, 109, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5182124, 81412, 126, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5182253, 81417, 116, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5182384, 81423, 687, 22, 90, "Text",
      CellTags->{"i:43", "b:1.1", "IPM3.1.2.2"}],
    Cell[5183096, 81449, 116, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5183215, 81454, 114, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5183366, 81462, 165, 4, 33, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5183534, 81468, 44186, 1154, 180, 16464, 806, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5227757, 82627, 107, 2, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5227867, 82631, 116, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5227998, 82637, 884, 29, 71, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5228885, 82668, 218, 6, 33, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5229106, 82676, 538, 12, 90, "Text",
      CellTags->{"i:44", "b:1.1", "IPM3.1.2.2"}],
    Cell[5229647, 82690, 119, 3, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5229791, 82697, 96, 2, 30, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5229890, 82701, 104, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230031, 82709, 110, 3, 44, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230144, 82714, 104, 3, 42, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230263, 82720, 361, 11, 52, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230649, 82735, 93, 6, 66, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230745, 82743, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5230858, 82749, 192, 7, 33, "TextFirst",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231075, 82760, 91, 5, 48, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231169, 82767, 97, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231281, 82773, 109, 1, 33, "TextFirst",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231415, 82778, 94, 5, 48, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231512, 82785, 98, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5231625, 82791, 379, 9, 52, "Text",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5232007, 82802, 331, 13, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5232363, 82819, 172, 5, 48, "Input",
      CellTags->{"b:1.1", "IPM3.1.2.2"}],
    Cell[5232538, 82826, 102, 3, 29, "Output",
      CellTags->{"b:1.1", "IPM3.1.2.2"}]},
  "c:11"->{
    Cell[5171708, 81034, 87, 1, 44, "Subsection",
      CellTags->{"b:1.1", "IPM3.1.2.2", "c:11"}]},
  "i:39"->{
    Cell[5171798, 81037, 201, 4, 52, "TextFirst",
      CellTags->{"i:39", "b:1.1", "IPM3.1.2.2"}]},
  "i:40"->{
    Cell[5173152, 81101, 304, 10, 33, "TextFirst",
      CellTags->{"i:40", "b:1.1", "IPM3.1.2.2"}]},
  "i:41"->{
    Cell[5174302, 81140, 5084, 167, 155, "DisplayTable",
      CellTags->{"i:41", "b:1.1", "IPM3.1.2.2"}]},
  "i:42"->{
    Cell[5180262, 81347, 361, 9, 52, "Text",
      CellTags->{"i:42", "b:1.1", "IPM3.1.2.2"}]},
  "i:43"->{
    Cell[5182384, 81423, 687, 22, 90, "Text",
      CellTags->{"i:43", "b:1.1", "IPM3.1.2.2"}]},
  "i:44"->{
    Cell[5229106, 82676, 538, 12, 90, "Text",
      CellTags->{"i:44", "b:1.1", "IPM3.1.2.2"}]},
  "b:1.2"->{
    Cell[5232689, 82835, 89, 1, 44, "Subsection",
      CellTags->{"b:1.2", "IPM3.1.2.3", "c:12"}],
    Cell[5232781, 82838, 455, 12, 52, "TextFirst",
      CellTags->{"i:45", "b:1.2", "IPM3.1.2.3"}],
    Cell[5233239, 82852, 160, 4, 33, "Text",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5233424, 82860, 103, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5233530, 82865, 114, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5233659, 82871, 94, 1, 33, "TextFirst",
      CellTags->{"i:46", "b:1.2", "IPM3.1.2.3"}],
    Cell[5233778, 82876, 102, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5233883, 82881, 114, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234012, 82887, 102, 1, 33, "TextFirst",
      CellTags->{"i:47", "b:1.2", "IPM3.1.2.3"}],
    Cell[5234139, 82892, 105, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234247, 82897, 114, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234376, 82903, 225, 6, 52, "Text",
      CellTags->{"i:48", "b:1.2", "IPM3.1.2.3"}],
    Cell[5234626, 82913, 105, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234734, 82918, 151, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5234900, 82924, 300, 8, 52, "Text",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5235225, 82936, 117, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5235345, 82941, 132, 3, 40, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5235492, 82947, 212, 5, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:49", "b:1.2", "IPM3.1.2.3"}],
    Cell[5235729, 82956, 134, 3, 44, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5235866, 82961, 132, 3, 40, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236013, 82967, 365, 8, 71, "Text",
      CellTags->{"i:50", "b:1.2", "IPM3.1.2.3"}],
    Cell[5236403, 82979, 138, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236544, 82984, 102, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236661, 82990, 133, 3, 33, "TextFirst",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236819, 82997, 139, 3, 30, "Input",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5236961, 83002, 102, 3, 29, "Output",
      CellTags->{"b:1.2", "IPM3.1.2.3"}],
    Cell[5237078, 83008, 277, 5, 52, "Text",
      CellTags->{"b:1.2", "IPM3.1.2.3"}]},
  "c:12"->{
    Cell[5232689, 82835, 89, 1, 44, "Subsection",
      CellTags->{"b:1.2", "IPM3.1.2.3", "c:12"}]},
  "i:45"->{
    Cell[5232781, 82838, 455, 12, 52, "TextFirst",
      CellTags->{"i:45", "b:1.2", "IPM3.1.2.3"}]},
  "i:46"->{
    Cell[5233659, 82871, 94, 1, 33, "TextFirst",
      CellTags->{"i:46", "b:1.2", "IPM3.1.2.3"}]},
  "i:47"->{
    Cell[5234012, 82887, 102, 1, 33, "TextFirst",
      CellTags->{"i:47", "b:1.2", "IPM3.1.2.3"}]},
  "i:48"->{
    Cell[5234376, 82903, 225, 6, 52, "Text",
      CellTags->{"i:48", "b:1.2", "IPM3.1.2.3"}]},
  "i:49"->{
    Cell[5235492, 82947, 212, 5, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"i:49", "b:1.2", "IPM3.1.2.3"}]},
  "i:50"->{
    Cell[5236013, 82967, 365, 8, 71, "Text",
      CellTags->{"i:50", "b:1.2", "IPM3.1.2.3"}]},
  "b:1.3"->{
    Cell[5237392, 83018, 95, 1, 44, "Subsection",
      CellTags->{"b:1.3", "IPM3.1.2.4", "c:13"}],
    Cell[5237490, 83021, 849, 19, 128, "TextFirst",
      CellTags->{"i:51", "b:1.3", "IPM3.1.2.4"}],
    Cell[5238342, 83042, 100, 1, 31, "BulletedList",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5238445, 83045, 159, 4, 31, "BulletedList",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5238607, 83051, 140, 3, 31, "BulletedList",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5238750, 83056, 108, 1, 31, "BulletedList",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5238861, 83059, 645, 15, 90, "Text",
      CellTags->{"i:52", "b:1.3", "IPM3.1.2.4"}],
    Cell[5239509, 83076, 490, 11, 90, "Text",
      CellTags->{"b:1.3", "IPM3.1.2.4"}],
    Cell[5240002, 83089, 294, 7, 52, "Text",
      CellTags->{"b:1.3", "IPM3.1.2.4"}]},
  "c:13"->{
    Cell[5237392, 83018, 95, 1, 44, "Subsection",
      CellTags->{"b:1.3", "IPM3.1.2.4", "c:13"}]},
  "i:51"->{
    Cell[5237490, 83021, 849, 19, 128, "TextFirst",
      CellTags->{"i:51", "b:1.3", "IPM3.1.2.4"}]},
  "i:52"->{
    Cell[5238861, 83059, 645, 15, 90, "Text",
      CellTags->{"i:52", "b:1.3", "IPM3.1.2.4"}]},
  "b:1.4"->{
    Cell[5240333, 83101, 73, 1, 44, "Subsection",
      CellTags->{"b:1.4", "IPM3.1.2.5", "c:14"}],
    Cell[5240409, 83104, 649, 16, 90, "TextFirst",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5241061, 83122, 241, 5, 52, "Text",
      CellTags->{"i:53", "b:1.4", "IPM3.1.2.5"}],
    Cell[5241327, 83131, 104, 3, 30, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5241434, 83136, 427, 7, 55, "Message",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5241864, 83145, 106, 3, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5241985, 83151, 428, 11, 71, "Text",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5242416, 83164, 122, 3, 30, "Input",
      CellTags->{"i:54", "b:1.4", "IPM3.1.2.5"}],
    Cell[5242541, 83169, 306, 8, 52, "Text",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5242872, 83181, 113, 3, 31, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5242988, 83186, 114, 3, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5243117, 83192, 197, 6, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5243317, 83200, 121, 3, 30, "Input",
      CellTags->{"i:55", "b:1.4", "IPM3.1.2.5"}],
    Cell[5243441, 83205, 684, 14, 109, "Text",
      CellTags->{"i:56", "b:1.4", "IPM3.1.2.5"}],
    Cell[5244150, 83223, 157, 4, 30, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5244310, 83229, 163, 4, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5244488, 83236, 596, 12, 109, "Text",
      CellTags->{"i:57", "b:1.4", "IPM3.1.2.5"}],
    Cell[5245109, 83252, 108, 3, 30, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5245220, 83257, 393, 6, 37, "Message",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5245616, 83265, 109, 3, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5245762, 83273, 114, 3, 30, "Input",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5245879, 83278, 396, 7, 37, "Message",
      CellTags->{"b:1.4", "IPM3.1.2.5"}],
    Cell[5246278, 83287, 116, 3, 29, "Output",
      CellTags->{"b:1.4", "IPM3.1.2.5"}]},
  "c:14"->{
    Cell[5240333, 83101, 73, 1, 44, "Subsection",
      CellTags->{"b:1.4", "IPM3.1.2.5", "c:14"}]},
  "i:53"->{
    Cell[5241061, 83122, 241, 5, 52, "Text",
      CellTags->{"i:53", "b:1.4", "IPM3.1.2.5"}]},
  "i:54"->{
    Cell[5242416, 83164, 122, 3, 30, "Input",
      CellTags->{"i:54", "b:1.4", "IPM3.1.2.5"}]},
  "i:55"->{
    Cell[5243317, 83200, 121, 3, 30, "Input",
      CellTags->{"i:55", "b:1.4", "IPM3.1.2.5"}]},
  "i:56"->{
    Cell[5243441, 83205, 684, 14, 109, "Text",
      CellTags->{"i:56", "b:1.4", "IPM3.1.2.5"}]},
  "i:57"->{
    Cell[5244488, 83236, 596, 12, 109, "Text",
      CellTags->{"i:57", "b:1.4", "IPM3.1.2.5"}]},
  "b:1.5"->{
    Cell[5246443, 83296, 79, 1, 44, "Subsection",
      CellTags->{"b:1.5", "IPM3.1.2.6", "c:15"}],
    Cell[5246525, 83299, 504, 11, 71, "TextFirst",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5247032, 83312, 345, 8, 52, "Text",
      CellTags->{"i:58", "b:1.5", "IPM3.1.2.6"}],
    Cell[5247402, 83324, 94, 2, 30, "Input",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5247499, 83328, 529, 11, 77, "Print",
      CellTags->{"b:1.5", "Info3302590721-5746852", "IPM3.1.2.6"}],
    Cell[5248043, 83342, 294, 8, 52, "Text",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5248362, 83354, 105, 3, 30, "Input",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5248470, 83359, 3822, 96, 84, "Print",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5252307, 83458, 294, 6, 52, "Text",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5252604, 83466, 571, 11, 99, "Print",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5253178, 83479, 251, 6, 52, "Text",
      CellTags->{"b:1.5", "IPM3.1.2.6"}],
    Cell[5253432, 83487, 340, 9, 52, "Text",
      CellTags->{"b:1.5", "IPM3.1.2.6"}]},
  "c:15"->{
    Cell[5246443, 83296, 79, 1, 44, "Subsection",
      CellTags->{"b:1.5", "IPM3.1.2.6", "c:15"}]},
  "i:58"->{
    Cell[5247032, 83312, 345, 8, 52, "Text",
      CellTags->{"i:58", "b:1.5", "IPM3.1.2.6"}]},
  "Info3302590721-5746852"->{
    Cell[5247499, 83328, 529, 11, 77, "Print",
      CellTags->{"b:1.5", "Info3302590721-5746852", "IPM3.1.2.6"}]},
  "b:1.6"->{
    Cell[5253809, 83501, 83, 1, 44, "Subsection",
      CellTags->{"b:1.6", "IPM3.1.2.7", "c:16"}],
    Cell[5253895, 83504, 327, 7, 52, "TextFirst",
      CellTags->{"i:59", "b:1.6", "IPM3.1.2.7"}],
    Cell[5254225, 83513, 256, 7, 52, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[5942099, 92969, 615, 17, 90, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[5942717, 92988, 320, 6, 71, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[6674575, 102985, 327, 9, 52, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[6674905, 102996, 349, 6, 52, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}],
    Cell[6675257, 103004, 208, 4, 52, "Text",
      CellTags->{"b:1.6", "IPM3.1.2.7"}]},
  "c:16"->{
    Cell[5253809, 83501, 83, 1, 44, "Subsection",
      CellTags->{"b:1.6", "IPM3.1.2.7", "c:16"}]},
  "i:59"->{
    Cell[5253895, 83504, 327, 7, 52, "TextFirst",
      CellTags->{"i:59", "b:1.6", "IPM3.1.2.7"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"IPM3.1.0", 6682652, 103157},
  {"IPM3.1.1.1", 6683181, 103169},
  {"IPM3.1.1.2", 6685425, 103221},
  {"IPM3.1.1.3", 6687778, 103274},
  {"IPM3.1.1.4", 6689980, 103323},
  {"IPM3.1.1.5", 6692826, 103385},
  {"IPM3.1.1.6", 6695874, 103453},
  {"IPM3.1.2.0", 6699746, 103539},
  {"IPM3.1.2.1", 6700413, 103553},
  {"IPM3.1.2.2", 6704640, 103644},
  {"IPM3.1.2.3", 6710540, 103775},
  {"IPM3.1.2.4", 6713612, 103842},
  {"IPM3.1.2.5", 6715050, 103872},
  {"IPM3.1.2.6", 6717754, 103931},
  {"IPM3.1.2.7", 6719444, 103967},
  {"c:1", 6721018, 104001},
  {"c:2", 6721154, 104005},
  {"i:1", 6721394, 104011},
  {"b:0.0", 6721509, 104014},
  {"c:3", 6723214, 104055},
  {"i:2", 6723327, 104058},
  {"b:0.1", 6723442, 104061},
  {"c:4", 6725035, 104097},
  {"i:3", 6725148, 104100},
  {"i:4", 6725341, 104105},
  {"i:60", 6725535, 104110},
  {"i:61", 6725672, 104114},
  {"i:6", 6725867, 104119},
  {"i:7", 6726060, 104124},
  {"i:8", 6726257, 104129},
  {"b:0.2", 6726454, 104134},
  {"c:5", 6727804, 104164},
  {"i:9", 6727911, 104167},
  {"i:10", 6728255, 104175},
  {"b:0.3", 6728516, 104181},
  {"c:6", 6730823, 104232},
  {"i:12", 6730934, 104235},
  {"i:13", 6731046, 104238},
  {"i:14", 6731250, 104243},
  {"i:15", 6731454, 104248},
  {"i:16", 6731566, 104251},
  {"i:17", 6731706, 104255},
  {"i:18", 6731819, 104258},
  {"b:0.4", 6731960, 104262},
  {"c:7", 6734469, 104319},
  {"i:20", 6734580, 104322},
  {"i:21", 6734688, 104325},
  {"i:22", 6734796, 104328},
  {"i:23", 6734904, 104331},
  {"i:24", 6735011, 104334},
  {"b:0.5", 6735122, 104337},
  {"c:8", 6737916, 104400},
  {"i:25", 6738028, 104403},
  {"i:26", 6738142, 104406},
  {"i:27", 6738249, 104409},
  {"i:28", 6738357, 104412},
  {"i:29", 6738466, 104415},
  {"c:9", 6738574, 104418},
  {"b:1.0", 6738850, 104424},
  {"c:10", 6742505, 104504},
  {"i:30", 6742619, 104507},
  {"i:31", 6742732, 104510},
  {"i:32", 6742840, 104513},
  {"i:33", 6742949, 104516},
  {"i:34", 6743058, 104519},
  {"i:35", 6743166, 104522},
  {"i:36", 6743275, 104525},
  {"i:37", 6743389, 104528},
  {"i:38", 6743503, 104531},
  {"b:1.1", 6743615, 104534},
  {"c:11", 6748943, 104654},
  {"i:39", 6749056, 104657},
  {"i:40", 6749169, 104660},
  {"i:41", 6749283, 104663},
  {"i:42", 6749403, 104666},
  {"i:43", 6749511, 104669},
  {"i:44", 6749620, 104672},
  {"b:1.2", 6749730, 104675},
  {"c:12", 6752230, 104731},
  {"i:45", 6752343, 104734},
  {"i:46", 6752457, 104737},
  {"i:47", 6752569, 104740},
  {"i:48", 6752682, 104743},
  {"i:49", 6752790, 104746},
  {"i:50", 6752931, 104750},
  {"b:1.3", 6753040, 104753},
  {"c:13", 6753906, 104772},
  {"i:51", 6754019, 104775},
  {"i:52", 6754134, 104778},
  {"b:1.4", 6754244, 104781},
  {"c:14", 6756376, 104829},
  {"i:53", 6756489, 104832},
  {"i:54", 6756597, 104835},
  {"i:55", 6756706, 104838},
  {"i:56", 6756815, 104841},
  {"i:57", 6756925, 104844},
  {"b:1.5", 6757036, 104847},
  {"c:15", 6758154, 104872},
  {"i:58", 6758267, 104875},
  {"Info3302590721-5746852", 6758393, 104878},
  {"b:1.6", 6758522, 104881},
  {"c:16", 6759261, 104898},
  {"i:59", 6759374, 104901}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 356, 8, 29, "TitleBar",
  CellTags->{
  "IPM3.1.0", "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", 
    "IPM3.1.1.5", "IPM3.1.1.6", "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", 
    "IPM3.1.2.3", "IPM3.1.2.4", "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7"}],

Cell[CellGroupData[{
Cell[2135, 63, 181, 6, 98, "Chapter",
  CounterAssignments->{{"Chapter", 0}},
  CellTags->{"IPM3.1.0", "c:1"}],
Cell[2319, 71, 1037, 24, 148, "ChapterAbstract",
  CellTags->"IPM3.1.0"],

Cell[CellGroupData[{
Cell[3381, 99, 276, 8, 41, "Section",
  CounterAssignments->{{"Chapter", 1}, {"Section", 0}},
  CellTags->{
  "IPM3.1.1.1", "IPM3.1.1.2", "IPM3.1.1.3", "IPM3.1.1.4", "IPM3.1.1.5", 
    "IPM3.1.1.6", "c:2"}],

Cell[CellGroupData[{
Cell[3682, 111, 95, 1, 44, "Subsection",
  CellTags->{"i:1", "b:0.0", "IPM3.1.1.1", "c:3"}],
Cell[3780, 114, 257, 6, 52, "TextFirst",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[CellGroupData[{
Cell[4062, 124, 140, 3, 43, "Input",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],
Cell[4205, 129, 119, 3, 29, "Output",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],
Cell[4339, 135, 89, 1, 33, "TextFirst",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[CellGroupData[{
Cell[4453, 140, 104, 3, 30, "Input",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],
Cell[4560, 145, 98, 3, 29, "Output",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[4695, 153, 104, 3, 30, "Input",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],
Cell[4802, 158, 98, 3, 29, "Output",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[4937, 166, 103, 3, 30, "Input",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],
Cell[5043, 171, 97, 3, 29, "Output",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5177, 179, 112, 3, 30, "Input",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],
Cell[5292, 184, 98, 3, 29, "Output",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],
Cell[5405, 190, 286, 7, 52, "TextFirst",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],

Cell[CellGroupData[{
Cell[5716, 201, 115, 3, 42, "Input",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],
Cell[5834, 206, 99, 3, 42, "Output",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5970, 214, 100, 3, 30, "Input",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],
Cell[6073, 219, 250, 5, 67, "Output",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[6360, 229, 107, 3, 30, "Input",
  CellTags->{"b:0.0", "IPM3.1.1.1"}],
Cell[6470, 234, 625, 9, 162, "Output",
  CellTags->{"b:0.0", "IPM3.1.1.1"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7144, 249, 94, 1, 44, "Subsection",
  CellTags->{"i:2", "b:0.1", "IPM3.1.1.2", "c:4"}],
Cell[7241, 252, 319, 7, 52, "TextFirst",
  CellTags->{"b:0.1", "IPM3.1.1.2"}],

Cell[CellGroupData[{
Cell[7585, 263, 117, 3, 31, "Input",
  CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}],
Cell[7705, 268, 152, 3, 29, "Output",
  CellTags->{"i:3", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[7894, 276, 130, 3, 31, "Input",
  CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}],
Cell[8027, 281, 150, 3, 42, "Output",
  CellTags->{"i:4", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]],
Cell[8192, 287, 451, 14, 52, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:0.1", "IPM3.1.1.2", "i:60"}],

Cell[CellGroupData[{
Cell[8668, 305, 158, 3, 30, "Input",
  CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}],
Cell[8829, 310, 110, 3, 29, "Output",
  CellTags->{"b:0.1", "IPM3.1.1.2", "i:61"}]
}, Open  ]],
Cell[8954, 316, 179, 4, 33, "TextFirst",
  CellTags->{"b:0.1", "IPM3.1.1.2"}],

Cell[CellGroupData[{
Cell[9158, 324, 221, 4, 33, "Input",
  CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}],
Cell[9382, 330, 260, 5, 46, "Output",
  CellTags->{"i:6", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]],
Cell[9657, 338, 99, 1, 33, "TextFirst",
  CellTags->"IPM3.1.1.2"],

Cell[CellGroupData[{
Cell[9781, 343, 130, 3, 30, "Input",
  CellTags->"IPM3.1.1.2"],
Cell[9914, 348, 357, 11, 51, "Output",
  CellTags->"IPM3.1.1.2"]
}, Open  ]],
Cell[10286, 362, 235, 7, 33, "TextFirst",
  CellTags->{"b:0.1", "IPM3.1.1.2"}],

Cell[CellGroupData[{
Cell[10546, 373, 132, 3, 31, "Input",
  CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}],
Cell[10681, 378, 788, 12, 172, "Output",
  CellTags->{"i:7", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[11506, 395, 150, 3, 42, "Input",
  CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}],
Cell[11659, 400, 254, 5, 70, "Output",
  CellTags->{"i:8", "b:0.1", "IPM3.1.1.2"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[11962, 411, 74, 1, 44, "Subsection",
  CellTags->{"b:0.2", "IPM3.1.1.3", "c:5"}],
Cell[12039, 414, 816, 16, 128, "TextFirst",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],
Cell[12858, 432, 207, 6, 33, "Text",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[CellGroupData[{
Cell[13090, 442, 155, 3, 33, "Input",
  CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
Cell[13248, 447, 55928, 1377, 193, 17886, 902, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}],
Cell[69179, 1826, 199, 5, 29, "Output",
  CellTags->{"i:9", "b:0.2", "IPM3.1.1.3"}]
}, Open  ]],
Cell[69393, 1834, 160, 4, 33, "TextFirst",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[CellGroupData[{
Cell[69578, 1842, 143, 3, 30, "Input",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],
Cell[69724, 1847, 51230, 1241, 195, 16127, 802, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->{"b:0.2", "IPM3.1.1.3"}]
}, Open  ]],
Cell[120969, 3091, 146, 4, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->"IPM3.1.1.3"],

Cell[CellGroupData[{
Cell[121140, 3099, 190, 5, 30, "Input",
  CellTags->"IPM3.1.1.3"],
Cell[121333, 3106, 38115, 1041, 195, 16544, 769, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->"IPM3.1.1.3"]
}, Open  ]],
Cell[159463, 4150, 440, 14, 52, "TextFirst",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],
Cell[159906, 4166, 285, 6, 83, "Input",
  CellTags->{"b:0.2", "IPM3.1.1.3"}],

Cell[CellGroupData[{
Cell[160216, 4176, 315, 6, 50, "Input",
  CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}],
Cell[160534, 4184, 202685, 8488, 335, 150201, 7835, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->{"i:10", "b:0.2", "IPM3.1.1.3"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[363268, 12678, 83, 1, 44, "Subsection",
  CellTags->{"b:0.3", "IPM3.1.1.4", "c:6"}],
Cell[363354, 12681, 420, 8, 71, "TextFirst",
  CellTags->{"i:12", "b:0.3", "IPM3.1.1.4"}],
Cell[363777, 12691, 699, 23, 71, "Text",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{
Cell[364501, 12718, 217, 5, 70, "Input",
  CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}],
Cell[364721, 12725, 218, 4, 48, "Output",
  CellTags->{"i:13", "b:0.3", "IPM3.1.1.4"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[364976, 12734, 139, 3, 30, "Input",
  CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}],
Cell[365118, 12739, 400, 7, 86, "Output",
  CellTags->{"i:14", "b:0.3", "IPM3.1.1.4"}]
}, Open  ]],
Cell[365533, 12749, 172, 5, 33, "TextFirst",
  CellTags->{"i:15", "b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{
Cell[365730, 12758, 149, 3, 30, "Input",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],
Cell[365882, 12763, 29311, 818, 195, 13497, 617, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]],
Cell[395208, 13584, 140, 4, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"i:16", "b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{
Cell[395373, 13592, 175, 5, 30, "Input",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],
Cell[395551, 13599, 21459, 646, 195, 11260, 514, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]],
Cell[417025, 14248, 521, 16, 52, "TextFirst",
  CellTags->{"i:17", "b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{
Cell[417571, 14268, 138, 3, 30, "Input",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],
Cell[417712, 14273, 164, 4, 29, "Output",
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]],
Cell[417891, 14280, 95, 1, 33, "Text",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{
Cell[418011, 14285, 143, 3, 30, "Input",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],
Cell[418157, 14290, 23874, 692, 195, 11488, 533, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]],
Cell[442046, 14985, 213, 5, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"i:18", "b:0.3", "IPM3.1.1.4"}],

Cell[CellGroupData[{
Cell[442284, 14994, 120, 3, 30, "Input",
  CellTags->{"b:0.3", "IPM3.1.1.4"}],
Cell[442407, 14999, 24429, 709, 195, 11739, 547, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->{"b:0.3", "IPM3.1.1.4"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[466885, 15714, 77, 1, 44, "Subsection",
  CellTags->{"b:0.4", "IPM3.1.1.5", "c:7"}],
Cell[466965, 15717, 1246, 31, 166, "TextFirst",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[468214, 15750, 479, 10, 90, "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[468696, 15762, 151, 3, 30, "Input",
  CellTags->{"i:20", "b:0.4", "IPM3.1.1.5"}],
Cell[468850, 15767, 217, 6, 33, "TextFirst",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[469070, 15775, 150, 3, 30, "Input",
  CellTags->{"i:21", "b:0.4", "IPM3.1.1.5"}],
Cell[469223, 15780, 138, 2, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[CellGroupData[{
Cell[469386, 15786, 115, 3, 31, "Input",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[469504, 15791, 114, 3, 29, "Output",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],
Cell[469633, 15797, 403, 10, 71, "Text",
  CellTags->{"i:22", "b:0.4", "IPM3.1.1.5"}],
Cell[470039, 15809, 254, 6, 63, "Input",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[470296, 15817, 222, 5, 50, "Input",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[CellGroupData[{
Cell[470543, 15826, 118, 3, 30, "Input",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[470664, 15831, 10747, 175, 178, 733, 46, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[481414, 16008, 192, 5, 29, "Output",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],
Cell[481621, 16016, 205, 4, 52, "Text",
  CellTags->{"i:23", "b:0.4", "IPM3.1.1.5"}],
Cell[481829, 16022, 179, 4, 44, "Input",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[482011, 16028, 148, 4, 30, "Input",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[CellGroupData[{
Cell[482184, 16036, 113, 3, 30, "Input",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[482300, 16041, 115, 3, 29, "Output",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],
Cell[482430, 16047, 341, 6, 71, "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[482774, 16055, 611, 12, 130, "Input",
  CellTags->{"i:24", "b:0.4", "IPM3.1.1.5"}],
Cell[483388, 16069, 181, 4, 33, "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],

Cell[CellGroupData[{
Cell[483594, 16077, 166, 4, 30, "Input",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[483763, 16083, 99, 3, 29, "Output",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],
Cell[483877, 16089, 771, 16, 128, "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}],
Cell[484651, 16107, 347, 6, 71, "Text",
  CellTags->{"b:0.4", "IPM3.1.1.5"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[485035, 16118, 100, 1, 44, "Subsection",
  CellTags->{"b:0.5", "IPM3.1.1.6", "c:8"}],
Cell[485138, 16121, 849, 16, 128, "TextFirst",
  CellTags->{"i:25", "b:0.5", "IPM3.1.1.6"}],
Cell[485990, 16139, 305, 6, 71, "Text",
  CellTags->{"i:26", "b:0.5", "IPM3.1.1.6"}],
Cell[486298, 16147, 127, 3, 30, "Input",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{
Cell[486450, 16154, 134, 3, 30, "Input",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[486587, 16159, 46022, 846, 228, 6511, 353, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[532612, 17007, 194, 5, 29, "Output",
  CellTags->{"b:0.5", "IPM3.1.1.6"}]
}, Open  ]],
Cell[532821, 17015, 151, 4, 33, "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[532975, 17021, 360, 12, 195, "Program",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[533338, 17035, 287, 7, 52, "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[533628, 17044, 443, 15, 24, "DisplayFormula",
  Evaluatable->False,
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[534074, 17061, 320, 9, 52, "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[534397, 17072, 378, 8, 71, "Text",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[534778, 17082, 516, 16, 246, "Program",
  PageBreakAbove->True,
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[535297, 17100, 538, 12, 90, "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[535838, 17114, 1434, 42, 41, "DisplayFormula",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{
Cell[537297, 17160, 108, 3, 30, "Input",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[537408, 17165, 98, 3, 29, "Output",
  CellTags->{"b:0.5", "IPM3.1.1.6"}]
}, Open  ]],
Cell[537521, 17171, 669, 12, 109, "Text",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[538193, 17185, 546, 15, 90, "Text",
  CellTags->{"i:27", "b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{
Cell[538764, 17204, 427, 9, 130, "Input",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[539194, 17215, 293, 8, 29, "Output",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[539490, 17225, 571347, 7848, 220, 479236, 6707, "GraphicsData", \
"PostScript", "Picture",
  CellTags->"IPM3.1.1.6"]
}, Open  ]],
Cell[1110852, 25076, 136, 3, 33, "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{
Cell[1111013, 25083, 135, 3, 30, "Input",
  CellTags->{"i:28", "b:0.5", "IPM3.1.1.6"}],
Cell[1111151, 25088, 571347, 7848, 220, 479236, 6707, "GraphicsData", \
"PostScript", "Picture",
  CellTags->"IPM3.1.1.6"]
}, Open  ]],
Cell[1682513, 32939, 120, 2, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{
Cell[1682658, 32945, 128, 3, 30, "Input",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],
Cell[1682789, 32950, 571578, 7851, 220, 479236, 6707, "GraphicsData", \
"PostScript", "Picture",
  CellTags->"IPM3.1.1.6"]
}, Open  ]],
Cell[2254382, 40804, 97, 1, 33, "TextFirst",
  CellTags->{"b:0.5", "IPM3.1.1.6"}],

Cell[CellGroupData[{
Cell[2254504, 40809, 124, 3, 30, "Input",
  CellTags->{"i:29", "b:0.5", "IPM3.1.1.6"}],
Cell[2254631, 40814, 578751, 7940, 220, 479236, 6707, "GraphicsData", \
"PostScript", "Picture",
  CellTags->"IPM3.1.1.6"]
}, Open  ]],
Cell[2833397, 48757, 260, 5, 52, "Text",
  CellTags->{"b:0.5", "IPM3.1.1.6"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2833706, 48768, 290, 8, 41, "Section",
  CounterAssignments->{{"Chapter", 1}, {"Section", 1}},
  CellTags->{
  "IPM3.1.2.0", "IPM3.1.2.1", "IPM3.1.2.2", "IPM3.1.2.3", "IPM3.1.2.4", 
    "IPM3.1.2.5", "IPM3.1.2.6", "IPM3.1.2.7", "c:9"}],
Cell[2833999, 48778, 634, 15, 90, "TextFirst",
  CellTags->"IPM3.1.2.0"],

Cell[CellGroupData[{
Cell[2834658, 48797, 159, 5, 44, "Subsection",
  CellTags->{"b:1.0", "IPM3.1.2.1", "c:10"}],
Cell[2834820, 48804, 488, 10, 90, "TextFirst",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[2835311, 48816, 841, 20, 128, "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[CellGroupData[{
Cell[2836177, 48840, 168, 6, 43, "Subsubsection",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[2836348, 48848, 319, 9, 52, "TextFirst",
  CellTags->{"i:30", "b:1.0", "IPM3.1.2.1"}],
Cell[2836670, 48859, 6733, 113, 51, 6586, 109, "GraphicsData", "Bitmap", \
"Picture",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[2843406, 48974, 401, 10, 71, "TextFirst",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[2843810, 48986, 485, 8, 90, "Text",
  CellTags->{"i:31", "b:1.0", "IPM3.1.2.1"}],
Cell[2844298, 48996, 761, 21, 90, "Text",
  CellTags->{"i:32", "b:1.0", "IPM3.1.2.1"}],
Cell[2845062, 49019, 574412, 7954, 218, 468456, 6643, "GraphicsData", \
"PostScript", "Picture"],
Cell[3419477, 56975, 511, 16, 71, "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[3419991, 56993, 867068, 11820, 331, 715075, 9940, "GraphicsData", \
"PostScript", "Picture"],
Cell[4287062, 68815, 645, 17, 90, "Text",
  CellTags->{"i:33", "b:1.0", "IPM3.1.2.1"}],
Cell[4287710, 68834, 315, 9, 52, "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[4288028, 68845, 876123, 11932, 331, 715075, 9940, "GraphicsData", \
"PostScript", "Picture"],
Cell[5164154, 80779, 403, 9, 71, "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5164560, 80790, 264, 7, 52, "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5164861, 80802, 76, 1, 43, "Subsubsection",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5164940, 80805, 473, 8, 90, "TextFirst",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5165416, 80815, 381, 8, 52, "Text",
  CellTags->{"i:34", "b:1.0", "IPM3.1.2.1"}],
Cell[5165800, 80825, 547, 10, 90, "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[CellGroupData[{
Cell[5166372, 80839, 142, 3, 42, "Input",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5166517, 80844, 182, 4, 63, "Output",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],
Cell[5166714, 80851, 378, 13, 52, "Text",
  CellTags->{"i:35", "b:1.0", "IPM3.1.2.1"}],
Cell[5167095, 80866, 174, 6, 33, "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[CellGroupData[{
Cell[5167294, 80876, 99, 3, 30, "Input",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5167396, 80881, 115, 3, 29, "Output",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5167548, 80889, 99, 3, 30, "Input",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5167650, 80894, 115, 3, 29, "Output",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],
Cell[5167780, 80900, 345, 12, 33, "Text",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],

Cell[CellGroupData[{
Cell[5168150, 80916, 100, 3, 30, "Input",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5168253, 80921, 182, 4, 63, "Output",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5168472, 80930, 96, 3, 30, "Input",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5168571, 80935, 115, 3, 29, "Output",
  CellTags->{"b:1.0", "IPM3.1.2.1"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5168735, 80944, 153, 6, 43, "Subsubsection",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5168891, 80952, 306, 10, 52, "TextFirst",
  CellTags->{"i:36", "b:1.0", "IPM3.1.2.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5169234, 80967, 84, 1, 43, "Subsubsection",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5169321, 80970, 626, 11, 90, "TextFirst",
  CellTags->{"i:37", "b:1.0", "IPM3.1.2.1"}],
Cell[5169950, 80983, 237, 6, 31, "BulletedList",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5170190, 80991, 206, 5, 31, "BulletedList",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5170399, 80998, 228, 7, 31, "BulletedList",
  CellTags->{"b:1.0", "IPM3.1.2.1"}],
Cell[5170630, 81007, 1029, 21, 147, "Text",
  CellTags->{"i:38", "b:1.0", "IPM3.1.2.1"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5171708, 81034, 87, 1, 44, "Subsection",
  CellTags->{"b:1.1", "IPM3.1.2.2", "c:11"}],
Cell[5171798, 81037, 201, 4, 52, "TextFirst",
  CellTags->{"i:39", "b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5172024, 81045, 99, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5172126, 81050, 96, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5172237, 81056, 296, 10, 33, "TextFirst",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5172558, 81070, 100, 3, 42, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5172661, 81075, 96, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5172772, 81081, 143, 5, 33, "TextFirst",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5172940, 81090, 97, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5173040, 81095, 97, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5173152, 81101, 304, 10, 33, "TextFirst",
  CellTags->{"i:40", "b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5173481, 81115, 98, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5173582, 81120, 97, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5173694, 81126, 605, 12, 109, "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5174302, 81140, 5084, 167, 155, "DisplayTable",
  CellTags->{"i:41", "b:1.1", "IPM3.1.2.2"}],
Cell[5179389, 81309, 90, 1, 22, "NumberedTable",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5179482, 81312, 304, 7, 52, "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5179811, 81323, 99, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5179913, 81328, 98, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5180048, 81336, 98, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5180149, 81341, 98, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5180262, 81347, 361, 9, 52, "Text",
  CellTags->{"i:42", "b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5180648, 81360, 103, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5180754, 81365, 98, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5180867, 81371, 245, 6, 52, "Text",
  PageBreakAbove->True,
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5181137, 81381, 111, 3, 31, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5181251, 81386, 146, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5181412, 81392, 687, 16, 109, "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5182124, 81412, 126, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5182253, 81417, 116, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5182384, 81423, 687, 22, 90, "Text",
  CellTags->{"i:43", "b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5183096, 81449, 116, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5183215, 81454, 114, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5183366, 81462, 165, 4, 33, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5183534, 81468, 44186, 1154, 180, 16464, 806, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5227757, 82627, 107, 2, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5227867, 82631, 116, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5227998, 82637, 884, 29, 71, "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5228885, 82668, 218, 6, 33, "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5229106, 82676, 538, 12, 90, "Text",
  CellTags->{"i:44", "b:1.1", "IPM3.1.2.2"}],
Cell[5229647, 82690, 119, 3, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5229791, 82697, 96, 2, 30, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5229890, 82701, 104, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5230031, 82709, 110, 3, 44, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5230144, 82714, 104, 3, 42, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5230263, 82720, 361, 11, 52, "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5230649, 82735, 93, 6, 66, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5230745, 82743, 98, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5230858, 82749, 192, 7, 33, "TextFirst",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5231075, 82760, 91, 5, 48, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5231169, 82767, 97, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5231281, 82773, 109, 1, 33, "TextFirst",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5231415, 82778, 94, 5, 48, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5231512, 82785, 98, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]],
Cell[5231625, 82791, 379, 9, 52, "Text",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5232007, 82802, 331, 13, 52, "Text",
  PageBreakAbove->True,
  CellTags->{"b:1.1", "IPM3.1.2.2"}],

Cell[CellGroupData[{
Cell[5232363, 82819, 172, 5, 48, "Input",
  CellTags->{"b:1.1", "IPM3.1.2.2"}],
Cell[5232538, 82826, 102, 3, 29, "Output",
  CellTags->{"b:1.1", "IPM3.1.2.2"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5232689, 82835, 89, 1, 44, "Subsection",
  CellTags->{"b:1.2", "IPM3.1.2.3", "c:12"}],
Cell[5232781, 82838, 455, 12, 52, "TextFirst",
  CellTags->{"i:45", "b:1.2", "IPM3.1.2.3"}],
Cell[5233239, 82852, 160, 4, 33, "Text",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{
Cell[5233424, 82860, 103, 3, 30, "Input",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],
Cell[5233530, 82865, 114, 3, 29, "Output",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],
Cell[5233659, 82871, 94, 1, 33, "TextFirst",
  CellTags->{"i:46", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{
Cell[5233778, 82876, 102, 3, 30, "Input",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],
Cell[5233883, 82881, 114, 3, 29, "Output",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],
Cell[5234012, 82887, 102, 1, 33, "TextFirst",
  CellTags->{"i:47", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{
Cell[5234139, 82892, 105, 3, 30, "Input",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],
Cell[5234247, 82897, 114, 3, 29, "Output",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],
Cell[5234376, 82903, 225, 6, 52, "Text",
  CellTags->{"i:48", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{
Cell[5234626, 82913, 105, 3, 30, "Input",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],
Cell[5234734, 82918, 151, 3, 29, "Output",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],
Cell[5234900, 82924, 300, 8, 52, "Text",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{
Cell[5235225, 82936, 117, 3, 30, "Input",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],
Cell[5235345, 82941, 132, 3, 40, "Output",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],
Cell[5235492, 82947, 212, 5, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"i:49", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{
Cell[5235729, 82956, 134, 3, 44, "Input",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],
Cell[5235866, 82961, 132, 3, 40, "Output",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],
Cell[5236013, 82967, 365, 8, 71, "Text",
  CellTags->{"i:50", "b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{
Cell[5236403, 82979, 138, 3, 30, "Input",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],
Cell[5236544, 82984, 102, 3, 29, "Output",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],
Cell[5236661, 82990, 133, 3, 33, "TextFirst",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],

Cell[CellGroupData[{
Cell[5236819, 82997, 139, 3, 30, "Input",
  CellTags->{"b:1.2", "IPM3.1.2.3"}],
Cell[5236961, 83002, 102, 3, 29, "Output",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],
Cell[5237078, 83008, 277, 5, 52, "Text",
  CellTags->{"b:1.2", "IPM3.1.2.3"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5237392, 83018, 95, 1, 44, "Subsection",
  CellTags->{"b:1.3", "IPM3.1.2.4", "c:13"}],
Cell[5237490, 83021, 849, 19, 128, "TextFirst",
  CellTags->{"i:51", "b:1.3", "IPM3.1.2.4"}],
Cell[5238342, 83042, 100, 1, 31, "BulletedList",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],
Cell[5238445, 83045, 159, 4, 31, "BulletedList",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],
Cell[5238607, 83051, 140, 3, 31, "BulletedList",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],
Cell[5238750, 83056, 108, 1, 31, "BulletedList",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],
Cell[5238861, 83059, 645, 15, 90, "Text",
  CellTags->{"i:52", "b:1.3", "IPM3.1.2.4"}],
Cell[5239509, 83076, 490, 11, 90, "Text",
  CellTags->{"b:1.3", "IPM3.1.2.4"}],
Cell[5240002, 83089, 294, 7, 52, "Text",
  CellTags->{"b:1.3", "IPM3.1.2.4"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5240333, 83101, 73, 1, 44, "Subsection",
  CellTags->{"b:1.4", "IPM3.1.2.5", "c:14"}],
Cell[5240409, 83104, 649, 16, 90, "TextFirst",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5241061, 83122, 241, 5, 52, "Text",
  CellTags->{"i:53", "b:1.4", "IPM3.1.2.5"}],

Cell[CellGroupData[{
Cell[5241327, 83131, 104, 3, 30, "Input",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5241434, 83136, 427, 7, 55, "Message",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5241864, 83145, 106, 3, 29, "Output",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]],
Cell[5241985, 83151, 428, 11, 71, "Text",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5242416, 83164, 122, 3, 30, "Input",
  CellTags->{"i:54", "b:1.4", "IPM3.1.2.5"}],
Cell[5242541, 83169, 306, 8, 52, "Text",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],

Cell[CellGroupData[{
Cell[5242872, 83181, 113, 3, 31, "Input",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5242988, 83186, 114, 3, 29, "Output",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]],
Cell[5243117, 83192, 197, 6, 33, "Text",
  PageBreakAbove->True,
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5243317, 83200, 121, 3, 30, "Input",
  CellTags->{"i:55", "b:1.4", "IPM3.1.2.5"}],
Cell[5243441, 83205, 684, 14, 109, "Text",
  CellTags->{"i:56", "b:1.4", "IPM3.1.2.5"}],

Cell[CellGroupData[{
Cell[5244150, 83223, 157, 4, 30, "Input",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5244310, 83229, 163, 4, 29, "Output",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]],
Cell[5244488, 83236, 596, 12, 109, "Text",
  CellTags->{"i:57", "b:1.4", "IPM3.1.2.5"}],

Cell[CellGroupData[{
Cell[5245109, 83252, 108, 3, 30, "Input",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5245220, 83257, 393, 6, 37, "Message",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5245616, 83265, 109, 3, 29, "Output",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5245762, 83273, 114, 3, 30, "Input",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5245879, 83278, 396, 7, 37, "Message",
  CellTags->{"b:1.4", "IPM3.1.2.5"}],
Cell[5246278, 83287, 116, 3, 29, "Output",
  CellTags->{"b:1.4", "IPM3.1.2.5"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[5246443, 83296, 79, 1, 44, "Subsection",
  CellTags->{"b:1.5", "IPM3.1.2.6", "c:15"}],
Cell[5246525, 83299, 504, 11, 71, "TextFirst",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],
Cell[5247032, 83312, 345, 8, 52, "Text",
  CellTags->{"i:58", "b:1.5", "IPM3.1.2.6"}],

Cell[CellGroupData[{
Cell[5247402, 83324, 94, 2, 30, "Input",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],
Cell[5247499, 83328, 529, 11, 77, "Print",
  CellTags->{"b:1.5", "Info3302590721-5746852", "IPM3.1.2.6"}]
}, Open  ]],
Cell[5248043, 83342, 294, 8, 52, "Text",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],

Cell[CellGroupData[{
Cell[5248362, 83354, 105, 3, 30, "Input",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],
Cell[5248470, 83359, 3822, 96, 84, "Print",
  CellTags->{"b:1.5", "IPM3.1.2.6"}]
}, Open  ]],
Cell[5252307, 83458, 294, 6, 52, "Text",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],
Cell[5252604, 83466, 571, 11, 99, "Print",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],
Cell[5253178, 83479, 251, 6, 52, "Text",
  CellTags->{"b:1.5", "IPM3.1.2.6"}],
Cell[5253432, 83487, 340, 9, 52, "Text",
  CellTags->{"b:1.5", "IPM3.1.2.6"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[5253809, 83501, 83, 1, 44, "Subsection",
  CellTags->{"b:1.6", "IPM3.1.2.7", "c:16"}],
Cell[5253895, 83504, 327, 7, 52, "TextFirst",
  CellTags->{"i:59", "b:1.6", "IPM3.1.2.7"}],
Cell[5254225, 83513, 256, 7, 52, "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],
Cell[5254484, 83522, 687612, 9445, 263, 597963, 8334, "GraphicsData", \
"PostScript", "Picture",
  CellTags->"IPM3.1.2.7"],
Cell[5942099, 92969, 615, 17, 90, "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],
Cell[5942717, 92988, 320, 6, 71, "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],
Cell[5943040, 92996, 731532, 9987, 263, 597963, 8334, "GraphicsData", \
"PostScript", "Picture",
  CellTags->"IPM3.1.2.7"],
Cell[6674575, 102985, 327, 9, 52, "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],
Cell[6674905, 102996, 349, 6, 52, "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}],
Cell[6675257, 103004, 208, 4, 52, "Text",
  CellTags->{"b:1.6", "IPM3.1.2.7"}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

