(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    230399,       7796]*)
(*NotebookOutlinePosition[    391430,      11418]*)
(*  CellTagsIndexPosition[    387560,      11278]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "Introduction to Programming with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "TitleBar",
  CellTags->{
  "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
    "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
    "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
    "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
    "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
    "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Chapter"],
  " Functional programming"
}], "Chapter",
  CounterAssignments->{{"Chapter", 3}},
  CellTags->{"IPM3.4.0", "c:1"}],

Cell[TextData[{
  "Programming in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is essentially a matter of writing user-defined functions that work like \
mathematical functions; when applied to specific values, they perform \
computations producing results. In fact, these functions can operate on \
arbitrary expressions, including other functions. This ",
  StyleBox["functional",
    FontSlant->"Italic"],
  " style of programming distinguishes ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " from more traditional procedural languages like C and Fortran, and a \
facility at functional programming is essential for taking full advantage of \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\[CloseCurlyQuote]s powerful language to solve your computational tasks."
}], "ChapterAbstract",
  CellTags->"IPM3.4.0"],

Cell[CellGroupData[{

Cell["Introduction", "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 0}},
  CellTags->{"b:11", "IPM3.4.1", "c:2"}],

Cell["\<\
Functions are objects that operate on expressions and output unique \
expressions for each input. We can think of functions as mathematicians do. \
For example, here is a definition for a function of two variables.\
\>", "TextFirst",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \(f[x_, y_] := Cos[x] + Sin[y]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell["\<\
You can evaluate the function for numeric or symbolic values.\
\>", "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(f[\[Pi], 1.6]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \(\(-0.0004263969584948901`\)\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(f[\[Theta], \[Rho]]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \(Cos[\[Theta]] + Sin[\[Rho]]\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[TextData[{
  "Functions can be significantly more complicated objects. Below is a \
function that operates on functions. Like the function ",
  StyleBox["f", "MR"],
  " above it takes two arguments, but, in this case, its arguments are a \
function or expression, and a list containing the variable of integration and \
the integration limits."
}], "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[Exp[I\ \[Pi]\ x], {x, a, b}]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \(\(\[ImaginaryI]\ \((\[ExponentialE]\^\(\[ImaginaryI]\ a\ \[Pi]\) - \
\[ExponentialE]\^\(\[ImaginaryI]\ b\ \[Pi]\))\)\)\/\[Pi]\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell["\<\
This particular function can be also be called with a function and a \
variable.\
\>", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[Exp[I\ \[Pi]\ x], x]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \(\(-\(\(\[ImaginaryI]\ \[ExponentialE]\^\(\[ImaginaryI]\ \[Pi]\ x\)\)\/\
\[Pi]\)\)\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell["\<\
Here is a function that also takes two arguments and operates on functions, \
but it returns a graphical object as its value.\
\>", "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot[Sin[x + \(\@2\) Sin[x]], {x, 0, 2  \[Pi]}]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.0238095 0.151576 0.309017 0.294303 [
[.17539 .29652 -4.5 -11.875 ]
[.17539 .29652 4.5 0 ]
[.32696 .29652 -4.5 -11.875 ]
[.32696 .29652 4.5 0 ]
[.47854 .29652 -4.5 -11.875 ]
[.47854 .29652 4.5 0 ]
[.63011 .29652 -4.5 -11.875 ]
[.63011 .29652 4.5 0 ]
[.78169 .29652 -4.5 -11.875 ]
[.78169 .29652 4.5 0 ]
[.93327 .29652 -4.5 -11.875 ]
[.93327 .29652 4.5 0 ]
[.01131 .01471 -15.1875 -5.9375 ]
[.01131 .01471 0 5.9375 ]
[.01131 .16187 -22.6875 -5.9375 ]
[.01131 .16187 0 5.9375 ]
[.01131 .45617 -16.5 -5.9375 ]
[.01131 .45617 0 5.9375 ]
[.01131 .60332 -9 -5.9375 ]
[.01131 .60332 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .30902 m
.17539 .31527 L
s
gsave
.17539 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.32696 .30902 m
.32696 .31527 L
s
gsave
.32696 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.47854 .30902 m
.47854 .31527 L
s
gsave
.47854 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.63011 .30902 m
.63011 .31527 L
s
gsave
.63011 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.78169 .30902 m
.78169 .31527 L
s
gsave
.78169 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.93327 .30902 m
.93327 .31527 L
s
gsave
.93327 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.125 Mabswid
.05412 .30902 m
.05412 .31277 L
s
.08444 .30902 m
.08444 .31277 L
s
.11476 .30902 m
.11476 .31277 L
s
.14507 .30902 m
.14507 .31277 L
s
.2057 .30902 m
.2057 .31277 L
s
.23602 .30902 m
.23602 .31277 L
s
.26633 .30902 m
.26633 .31277 L
s
.29665 .30902 m
.29665 .31277 L
s
.35728 .30902 m
.35728 .31277 L
s
.38759 .30902 m
.38759 .31277 L
s
.41791 .30902 m
.41791 .31277 L
s
.44822 .30902 m
.44822 .31277 L
s
.50885 .30902 m
.50885 .31277 L
s
.53917 .30902 m
.53917 .31277 L
s
.56948 .30902 m
.56948 .31277 L
s
.5998 .30902 m
.5998 .31277 L
s
.66043 .30902 m
.66043 .31277 L
s
.69074 .30902 m
.69074 .31277 L
s
.72106 .30902 m
.72106 .31277 L
s
.75138 .30902 m
.75138 .31277 L
s
.81201 .30902 m
.81201 .31277 L
s
.84232 .30902 m
.84232 .31277 L
s
.87264 .30902 m
.87264 .31277 L
s
.90295 .30902 m
.90295 .31277 L
s
.96358 .30902 m
.96358 .31277 L
s
.9939 .30902 m
.9939 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 .01471 m
.03006 .01471 L
s
gsave
.01131 .01471 -76.1875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.02381 .16187 m
.03006 .16187 L
s
gsave
.01131 .16187 -83.6875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.02381 .45617 m
.03006 .45617 L
s
gsave
.01131 .45617 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.02381 .60332 m
.03006 .60332 L
s
gsave
.01131 .60332 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.02381 .04414 m
.02756 .04414 L
s
.02381 .07357 m
.02756 .07357 L
s
.02381 .103 m
.02756 .103 L
s
.02381 .13244 m
.02756 .13244 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.04262 .39574 L
.06244 .47797 L
.08255 .54353 L
.09396 .57052 L
.09905 .57999 L
.10458 .58843 L
.11004 .59488 L
.11243 .59713 L
.11508 .59919 L
.1174 .60066 L
.11989 .60187 L
.12127 .60238 L
.12253 .60275 L
.12379 .60303 L
.125 .60321 L
.12617 .6033 L
.12723 .60332 L
.12845 .60326 L
.1296 .60313 L
.13067 .60294 L
.13166 .60272 L
.13388 .60201 L
.13613 .60103 L
.13859 .59966 L
.14357 .59597 L
.14881 .59083 L
.15378 .58486 L
.16308 .57112 L
.184 .53076 L
.22233 .44115 L
.26311 .35261 L
.28336 .3176 L
.30238 .29128 L
.32054 .2721 L
.33076 .2638 L
.34013 .25769 L
.34993 .25277 L
.35905 .24949 L
.36429 .24814 L
.36906 .24725 L
.3717 .24689 L
.37309 .24674 L
.37456 .2466 L
.37584 .24651 L
.37653 .24647 L
.37726 .24643 L
.37855 .24638 L
.37975 .24635 L
.38044 .24634 L
Mistroke
.38109 .24634 L
.3823 .24635 L
.3836 .24639 L
.38433 .24641 L
.38501 .24644 L
.38738 .2466 L
.38857 .2467 L
.38988 .24683 L
.39264 .24718 L
.3956 .24764 L
.40092 .24873 L
.41145 .25176 L
.42103 .25548 L
.44015 .26528 L
.46092 .27883 L
.50176 .31043 L
.54108 .34062 L
.56081 .35331 L
.57888 .36251 L
.58833 .36619 L
.59827 .36911 L
.60344 .37022 L
.60824 .37098 L
.611 .37129 L
.61351 .3715 L
.6149 .37158 L
.61563 .37162 L
.61641 .37165 L
.61708 .37167 L
.61783 .37168 L
.61914 .37169 L
.62044 .37168 L
.62164 .37165 L
.62296 .37159 L
.62363 .37156 L
.62437 .37151 L
.62692 .3713 L
.62932 .37102 L
.6338 .37029 L
.63862 .36919 L
.64393 .36761 L
.64952 .3655 L
.65953 .36054 L
.66906 .35437 L
.67924 .34612 L
.6976 .32678 L
.71892 .29684 L
.73907 .26125 L
.7775 .17728 L
.79696 .13084 L
Mistroke
.81839 .08216 L
.82819 .06241 L
.83872 .04401 L
.84867 .03011 L
.85344 .02485 L
.85776 .02096 L
.86253 .01771 L
.86515 .01642 L
.86635 .01594 L
.86762 .01552 L
.86871 .01523 L
.8699 .01498 L
.87099 .01483 L
.87201 .01474 L
.87321 .01472 L
.8743 .01476 L
.87556 .0149 L
.87676 .01512 L
.87795 .01541 L
.87926 .01584 L
.88192 .01701 L
.88438 .01847 L
.88669 .02018 L
.89189 .02522 L
.89749 .03254 L
.90707 .04962 L
.91754 .07477 L
.93865 .14421 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[6]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{43, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`006P3oool0
0`000000oooo0?ooo`3o0?ooo`@0oooo000J0?ooo`030000003oool0oooo0?l0oooo103oool00140
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooo`@0oooo000A0?ooo`030000
003oool0oooo00H0oooo00<000000?ooo`3oool0o`3oool40?ooo`002P3oool4000000<0oooo00<0
00000?ooo`3oool01P3oool200000=l0oooo1000000Q0?ooo`004@3oool00`000000oooo0?ooo`06
0?ooo`030000003oool0oooo0=d0oooo0P0000020?ooo`<000007`3oool00140oooo00<000000?oo
o`3oool01P3oool00`000000oooo0?ooo`3K0?ooo`8000001P3oool00`000000oooo0?ooo`0M0?oo
o`00403oool2000000P0oooo00<000000?ooo`3oool0fP3oool00`000000oooo0?ooo`070?ooo`80
00007@3oool00140oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3I0?ooo`030000
003oool0oooo00T0oooo00<000000?ooo`3oool06`3oool001X0oooo00<000000?ooo`3oool0f03o
ool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01X0oooo000J0?ooo`030000003oool0
oooo0=L0oooo00<000000?ooo`3oool03@3oool00`000000oooo0?ooo`0I0?ooo`006P3oool00`00
0000oooo0?ooo`3G0?ooo`030000003oool0oooo00d0oooo00<000000?ooo`3oool06@3oool001X0
oooo0P00003G0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0603oool001X0oooo
00<000000?ooo`3oool0eP3oool00`000000oooo0?ooo`0?0?ooo`030000003oool0oooo01P0oooo
000J0?ooo`030000003oool0oooo0=D0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?oo
o`0G0?ooo`006P3oool00`000000oooo0?ooo`3E0?ooo`030000003oool0oooo0140oooo00<00000
0?ooo`3oool05`3oool001X0oooo00<000000?ooo`3oool0e03oool00`000000oooo0?ooo`0C0?oo
o`030000003oool0oooo01H0oooo000J0?ooo`030000003oool0oooo0=@0oooo00<000000?ooo`3o
ool04`3oool00`000000oooo0?ooo`0F0?ooo`006P3oool00`000000oooo0?ooo`3C0?ooo`030000
003oool0oooo01@0oooo00<000000?ooo`3oool05P3oool001X0oooo00<000000?ooo`3oool0d`3o
ool00`000000oooo0?ooo`0E0?ooo`030000003oool0oooo01D0oooo000J0?ooo`800000d`3oool0
0`000000oooo0?ooo`0F0?ooo`030000003oool0oooo01D0oooo000J0?ooo`030000003oool0oooo
0=80oooo00<000000?ooo`3oool05P3oool00`000000oooo0?ooo`0E0?ooo`006P3oool00`000000
oooo0?ooo`3A0?ooo`030000003oool0oooo01P0oooo00<000000?ooo`3oool0503oool001X0oooo
00<000000?ooo`3oool0d@3oool00`000000oooo0?ooo`0H0?ooo`030000003oool0oooo01@0oooo
000J0?ooo`030000003oool0oooo0=00oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`0D0?ooo`006P3oool00`000000oooo0?ooo`3@0?ooo`030000003oool0oooo01X0oooo00<00000
0?ooo`3oool04`3oool001X0oooo00<000000?ooo`3oool0c`3oool00`000000oooo0?ooo`0K0?oo
o`030000003oool0oooo01<0oooo000J0?ooo`030000003oool0oooo0<l0oooo00<000000?ooo`3o
ool06`3oool00`000000oooo0?ooo`0C0?ooo`006P3oool200000<l0oooo00<000000?ooo`3oool0
7@3oool00`000000oooo0?ooo`0B0?ooo`006P3oool00`000000oooo0?ooo`3>0?ooo`030000003o
ool0oooo01d0oooo00<000000?ooo`3oool04P3oool001X0oooo00<000000?ooo`3oool0c@3oool0
0`000000oooo0?ooo`0N0?ooo`030000003oool0oooo0180oooo000J0?ooo`030000003oool0oooo
0<d0oooo00<000000?ooo`3oool07P3oool00`000000oooo0?ooo`0B0?ooo`006P3oool00`000000
oooo0?ooo`3<0?ooo`030000003oool0oooo0200oooo00<000000?ooo`3oool04@3oool001X0oooo
00<000000?ooo`3oool0c03oool00`000000oooo0?ooo`0P0?ooo`030000003oool0oooo0140oooo
000J0?ooo`030000003oool0oooo0</0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?oo
o`0A0?ooo`006P3oool00`000000oooo0?ooo`3;0?ooo`030000003oool0oooo0280oooo00<00000
0?ooo`3oool0403oool001X0oooo0P00003<0?ooo`030000003oool0oooo0280oooo00<000000?oo
o`3oool0403oool001X0oooo00<000000?ooo`3oool0bP3oool00`000000oooo0?ooo`0S0?ooo`03
0000003oool0oooo0100oooo000J0?ooo`030000003oool0oooo0<X0oooo00<000000?ooo`3oool0
903oool00`000000oooo0?ooo`0?0?ooo`006P3oool00`000000oooo0?ooo`390?ooo`030000003o
ool0oooo02D0oooo00<000000?ooo`3oool03`3oool001X0oooo00<000000?ooo`3oool0b@3oool0
0`000000oooo0?ooo`0U0?ooo`030000003oool0oooo00l0oooo000J0?ooo`030000003oool0oooo
0<T0oooo00<000000?ooo`3oool09@3oool00`000000oooo0?ooo`0?0?ooo`00203oool3000000<0
oooo00<000000?ooo`3oool00P0000070?ooo`030000003oool0oooo0<P0oooo00<000000?ooo`3o
ool09`3oool00`000000oooo0?ooo`0>0?ooo`00203oool010000000oooo0?ooo`0000040?ooo`04
0000003oool0oooo000000H0oooo00<000000?ooo`3oool0b03oool00`000000oooo0?ooo`0W0?oo
o`030000003oool0oooo00h0oooo00030?ooo`@0000000D0oooo0000003oool0oooo000000070?oo
o`030000003oool0oooo00@0oooo0P0000390?ooo`030000003oool0oooo02L0oooo00<000000?oo
o`3oool03P3oool000P0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`000006
0?ooo`030000003oool0oooo0<L0oooo00<000000?ooo`3oool0:03oool00`000000oooo0?ooo`0>
0?ooo`00203oool010000000oooo0?ooo`0000040?ooo`<000001`3oool00`000000oooo0?ooo`37
0?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool03@3oool000P0oooo00@000000?oo
o`3oool000001@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0<H0oooo00<00000
0?ooo`3oool0:P3oool00`000000oooo0?ooo`0=0?ooo`002@3oool2000000H0oooo0`0000060?oo
o`030000003oool0oooo0<H0oooo00<000000?ooo`3oool0:P3oool00`000000oooo0?ooo`0=0?oo
o`006P3oool00`000000oooo0?ooo`360?ooo`030000003oool0oooo02X0oooo00<000000?ooo`3o
ool03@3oool001X0oooo00<000000?ooo`3oool0a@3oool00`000000oooo0?ooo`0[0?ooo`030000
003oool0oooo00d0oooo000J0?ooo`030000003oool0oooo0<D0oooo00<000000?ooo`3oool0;03o
ool00`000000oooo0?ooo`0<0?ooo`006P3oool200000<D0oooo00<000000?ooo`3oool0;@3oool0
0`000000oooo0?ooo`0<0?ooo`006P3oool00`000000oooo0?ooo`340?ooo`030000003oool0oooo
02d0oooo00<000000?ooo`3oool0303oool001X0oooo00<000000?ooo`3oool0``3oool00`000000
oooo0?ooo`0^0?ooo`030000003oool0oooo00`0oooo000J0?ooo`030000003oool0oooo0<<0oooo
00<000000?ooo`3oool0;`3oool00`000000oooo0?ooo`0;0?ooo`006P3oool00`000000oooo0?oo
o`320?ooo`030000003oool0oooo0300oooo00<000000?ooo`3oool02`3oool001X0oooo00<00000
0?ooo`3oool0`P3oool00`000000oooo0?ooo`0`0?ooo`030000003oool0oooo00/0oooo000J0?oo
o`030000003oool0oooo0<40oooo00<000000?ooo`3oool0<@3oool00`000000oooo0?ooo`0;0?oo
o`006P3oool200000<80oooo00<000000?ooo`3oool0<P3oool00`000000oooo0?ooo`0:0?ooo`00
6P3oool00`000000oooo0?ooo`310?ooo`030000003oool0oooo0380oooo00<000000?ooo`3oool0
2P3oool001X0oooo00<000000?ooo`3oool0`03oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo00X0oooo000J0?ooo`030000003oool0oooo0<00oooo00<000000?ooo`3oool0<`3oool0
0`000000oooo0?ooo`0:0?ooo`006P3oool00`000000oooo0?ooo`2o0?ooo`030000003oool0oooo
03@0oooo00<000000?ooo`3oool02P3oool001X0oooo00<000000?ooo`3oool0_`3oool00`000000
oooo0?ooo`0e0?ooo`030000003oool0oooo00T0oooo000J0?ooo`030000003oool0oooo0;h0oooo
00<000000?ooo`3oool0=P3oool00`000000oooo0?ooo`090?ooo`006P3oool00`000000oooo0?oo
o`1J0?ooo`H00000GP3oool00`000000oooo0?ooo`0f0?ooo`030000003oool0oooo00T0oooo000J
0?ooo`800000E`3oool4000000H0oooo0`00001J0?ooo`030000003oool0oooo03L0oooo00<00000
0?ooo`3oool02@3oool001X0oooo00<000000?ooo`3oool0D`3oool3000000d0oooo0`00001G0?oo
o`030000003oool0oooo03P0oooo00<000000?ooo`3oool0203oool001X0oooo00<000000?ooo`3o
ool0D@3oool2000001<0oooo0`00001C0?ooo`030000003oool0oooo03T0oooo00<000000?ooo`3o
ool0203oool001X0oooo00<000000?ooo`3oool0D03oool00`000000oooo0?ooo`0F0?ooo`800000
D@3oool00`000000oooo0?ooo`0i0?ooo`030000003oool0oooo00P0oooo000J0?ooo`030000003o
ool0oooo04l0oooo00<000000?ooo`3oool06@3oool2000004h0oooo00<000000?ooo`3oool0>P3o
ool00`000000oooo0?ooo`080?ooo`006P3oool00`000000oooo0?ooo`0U0?ooo`030000003oool0
oooo02<0oooo1000000N0?ooo`030000003oool0oooo00D0oooo0P00000X0?ooo`030000003oool0
oooo01T0oooo00<000000?ooo`3oool0203oool2000002H0oooo0P0000080?ooo`030000003oool0
oooo00P0oooo000J0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool0903oool00`00
0000oooo0?ooo`0O0?ooo`800000103oool010000000oooo0?ooo`00000W0?ooo`030000003oool0
oooo01P0oooo00<000000?ooo`3oool0203oool010000000oooo0?ooo`00000T0?ooo`040000003o
ool0oooo000000P0oooo00<000000?ooo`3oool01`3oool001X0oooo00<000000?ooo`3oool09@3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool000000280oooo00<000000?ooo`3oool0103o
ool00`000000oooo0?ooo`0R0?ooo`D000006@3oool00`000000oooo0?ooo`0;0?ooo`030000003o
ool0oooo0280oooo00@000000?ooo`3oool00000203oool00`000000oooo0?ooo`070?ooo`006P3o
ool2000002H0oooo00<000000?ooo`3oool08P3oool010000000oooo0?ooo`00000S0?ooo`030000
003oool0oooo0080oooo00<000000?ooo`3oool08`3oool010000000oooo0?ooo`00000I0?ooo`03
0000003oool0oooo00T0oooo00@000000?ooo`3oool00000903oool010000000oooo0?ooo`000008
0?ooo`030000003oool0oooo00L0oooo000J0?ooo`030000003oool0oooo02D0oooo00<000000?oo
o`3oool08@3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo0240oooo0P0000030?oo
o`030000003oool0oooo02<0oooo00<000000?ooo`000000603oool00`000000oooo0?ooo`0:0?oo
o`<000009@3oool3000000T0oooo00<000000?ooo`3oool01`3oool001X0oooo00<000000?ooo`3o
ool0903oool200000280oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`0S0?oo
o`040000003oool0oooo000002H0oooo0P00000H0?ooo`030000003oool0oooo00/0oooo00<00000
0?ooo`3oool0903oool010000000oooo0?ooo`0000090?ooo`030000003oool0oooo00H0oooo000J
0?ooo`030000003oool0oooo02D0oooo00<000000?ooo`3oool07`3oool00`000000oooo0?ooo`02
0?ooo`8000009`3oool2000002P0oooo00<000000?ooo`3oool05@3oool00`000000oooo0?ooo`0<
0?ooo`<000009@3oool2000000X0oooo00<000000?ooo`3oool01P3oool001X0oooo00<000000?oo
o`3oool0AP3oool00`000000oooo0?ooo`0]0?ooo`800000?`3oool00`000000oooo0?ooo`100?oo
o`030000003oool0oooo00H0oooo000J0?ooo`030000003oool0oooo04H0oooo00<000000?ooo`3o
ool0;`3oool00`000000oooo0?ooo`0k0?ooo`030000003oool0oooo0440oooo00<000000?ooo`3o
ool01P3oool001X0oooo00<000000?ooo`3oool0A@3oool00`000000oooo0?ooo`0a0?ooo`030000
003oool0oooo03T0oooo00<000000?ooo`3oool0@`3oool00`000000oooo0?ooo`050?ooo`006P3o
ool00`000000oooo0?ooo`140?ooo`030000003oool0oooo03<0oooo0P00000i0?ooo`030000003o
ool0oooo04<0oooo00<000000?ooo`3oool01@3oool001<0ooooo`00000>000000006P3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool00`3oool00`00
0000oooo000000080?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01@3oool00`00
0000oooo0?ooo`040?ooo`8000001`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00H0oooo00<000000?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo
00D0oooo00D000000?ooo`3oool0oooo000000030?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00D0oooo00<00000
0?ooo`3oool01@3oool00`000000oooo0?ooo`050?ooo`4000000@3oool10?ooo`006P3oool00`00
0000oooo0?ooo`0U0?ooo`030000003oool0oooo01X0oooo00<000000?ooo`3oool02@3oool00`00
0000oooo0?ooo`0U0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool07@3oool00`00
0000oooo0?ooo`0A0?ooo`030000003oool0oooo0140oooo00<000000?ooo`3oool09P3oool00`00
0000oooo0?ooo`0@0?ooo`006P3oool200000480oooo00<000000?ooo`3oool0?03oool00`000000
oooo0?ooo`0_0?ooo`030000003oool0oooo04h0oooo000J0?ooo`800000@P3oool00`000000oooo
0?ooo`0m0?ooo`800000;`3oool00`000000oooo0?ooo`1>0?ooo`006P3oool200000440oooo00<0
00000?ooo`3oool0@03oool00`000000oooo0?ooo`0[0?ooo`030000003oool0oooo04l0oooo000J
0?ooo`800000@@3oool00`000000oooo0?ooo`110?ooo`030000003oool0oooo02T0oooo00<00000
0?ooo`3oool0D03oool001X0oooo00<000000?ooo`000000?`3oool00`000000oooo0?ooo`130?oo
o`800000:03oool00`000000oooo0?ooo`1A0?ooo`006P3oool3000003l0oooo00<000000?ooo`3o
ool0A@3oool00`000000oooo0?ooo`0T0?ooo`030000003oool0oooo0580oooo000J0?ooo`030000
003oool0000003h0oooo00<000000?ooo`3oool0A`3oool00`000000oooo0?ooo`0R0?ooo`030000
003oool0oooo05<0oooo000J0?ooo`030000003oool0000003h0oooo00<000000?ooo`3oool0B03o
ool200000240oooo00<000000?ooo`3oool0E03oool001X0oooo00<000000?ooo`000000?@3oool0
0`000000oooo0?ooo`1;0?ooo`030000003oool0oooo01`0oooo0P00001G0?ooo`006P3oool01000
0000oooo0?ooo`00000l0?ooo`030000003oool0oooo04`0oooo0P00000K0?ooo`030000003oool0
oooo05L0oooo000J0?ooo`040000003oool0oooo000003`0oooo00<000000?ooo`3oool0CP3oool2
000001L0oooo0P00001J0?ooo`006P3oool010000000oooo0?ooo`00000k0?ooo`030000003oool0
oooo0540oooo0P00000D0?ooo`030000003oool0oooo05X0oooo000J0?ooo`040000003oool0oooo
000003/0oooo00<000000?ooo`3oool0D`3oool3000000l0oooo0P00001M0?ooo`006P3oool20000
00030?ooo`000000oooo03T0oooo00<000000?ooo`3oool0E`3oool3000000P0oooo1000001O0?oo
o`006P3oool01@000000oooo0?ooo`3oool0000003T0oooo00<000000?ooo`3oool0FP3oool80000
06<0oooo000J0?ooo`050000003oool0oooo0?ooo`000000>03oool00`000000oooo0?ooo`360?oo
o`006P3oool01@000000oooo0?ooo`3oool0000003P0oooo00<000000?ooo`3oool0aP3oool001X0
oooo00D000000?ooo`3oool0oooo0000000g0?ooo`030000003oool0oooo0<L0oooo000J0?ooo`03
0000003oool0oooo0080oooo00<000000?ooo`3oool0=03oool00`000000oooo0?ooo`370?ooo`00
6P3oool00`000000oooo0?ooo`020?ooo`030000003oool0oooo03<0oooo00<000000?ooo`3oool0
b03oool001X0oooo00<000000?ooo`3oool00P3oool00`000000oooo0?ooo`0c0?ooo`030000003o
ool0oooo0<P0oooo000J0?ooo`8000000`3oool00`000000oooo0?ooo`0b0?ooo`030000003oool0
oooo0<T0oooo000J0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0<P3oool00`00
0000oooo0?ooo`390?ooo`006P3oool00`000000oooo0?ooo`030?ooo`030000003oool0oooo0300
oooo00<000000?ooo`3oool0bP3oool001X0oooo00<000000?ooo`3oool00`3oool00`000000oooo
0?ooo`0`0?ooo`030000003oool0oooo0<X0oooo000J0?ooo`030000003oool0oooo00<0oooo00<0
00000?ooo`3oool0;`3oool00`000000oooo0?ooo`3;0?ooo`006P3oool00`000000oooo0?ooo`03
0?ooo`030000003oool0oooo02l0oooo00<000000?ooo`3oool0b`3oool001X0oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`0]0?ooo`030000003oool0oooo0<`0oooo000J0?ooo`03
0000003oool0oooo00@0oooo00<000000?ooo`3oool0;@3oool00`000000oooo0?ooo`3<0?ooo`00
6P3oool2000000D0oooo00<000000?ooo`3oool0;03oool00`000000oooo0?ooo`3=0?ooo`006P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo02`0oooo00<000000?ooo`3oool0c@3o
ool001X0oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`0[0?ooo`030000003oool0
oooo0<h0oooo000J0?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool0:P3oool00`00
0000oooo0?ooo`3>0?ooo`006P3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02X0
oooo00<000000?ooo`3oool0cP3oool000P0oooo0`0000030?ooo`030000003oool0oooo00800000
1`3oool00`000000oooo0?ooo`050?ooo`030000003oool0oooo02T0oooo00<000000?ooo`3oool0
c`3oool000P0oooo00@000000?ooo`3oool00000103oool010000000oooo0?ooo`0000060?ooo`03
0000003oool0oooo00D0oooo00<000000?ooo`3oool0:@3oool00`000000oooo0?ooo`3?0?ooo`00
203oool010000000oooo0?ooo`0000070?ooo`030000003oool0oooo00@0oooo0P0000070?ooo`03
0000003oool0oooo02L0oooo00<000000?ooo`3oool0d03oool000P0oooo00@000000?ooo`3oool0
0000103oool010000000oooo0?ooo`0000060?ooo`030000003oool0oooo00H0oooo00<000000?oo
o`3oool09`3oool00`000000oooo0?ooo`3@0?ooo`00203oool010000000oooo0?ooo`0000040?oo
o`<000001`3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo02H0oooo00<000000?oo
o`3oool0d@3oool000P0oooo00@000000?ooo`3oool000001@3oool00`000000oooo0?ooo`060?oo
o`030000003oool0oooo00H0oooo00<000000?ooo`3oool09P3oool00`000000oooo0?ooo`3A0?oo
o`002@3oool2000000H0oooo0`0000060?ooo`030000003oool0oooo00L0oooo00<000000?ooo`3o
ool0903oool00`000000oooo0?ooo`3B0?ooo`006P3oool00`000000oooo0?ooo`070?ooo`030000
003oool0oooo02@0oooo00<000000?ooo`3oool0dP3oool001X0oooo00<000000?ooo`3oool01`3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo0=<0oooo000J0?ooo`030000003oool0
oooo00L0oooo00<000000?ooo`3oool08`3oool00`000000oooo0?ooo`3C0?ooo`006P3oool20000
00T0oooo00<000000?ooo`3oool08@3oool00`000000oooo0?ooo`3D0?ooo`006P3oool00`000000
oooo0?ooo`080?ooo`030000003oool0oooo0240oooo00<000000?ooo`3oool0e03oool001X0oooo
00<000000?ooo`3oool0203oool00`000000oooo0?ooo`0Q0?ooo`030000003oool0oooo0=@0oooo
000J0?ooo`030000003oool0oooo00P0oooo00<000000?ooo`3oool0803oool00`000000oooo0?oo
o`3E0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000003oool0oooo01l0oooo00<00000
0?ooo`3oool0e@3oool001X0oooo00<000000?ooo`3oool02@3oool00`000000oooo0?ooo`0N0?oo
o`030000003oool0oooo0=H0oooo000J0?ooo`030000003oool0oooo00T0oooo00<000000?ooo`3o
ool07P3oool00`000000oooo0?ooo`3F0?ooo`006P3oool00`000000oooo0?ooo`090?ooo`030000
003oool0oooo01d0oooo00<000000?ooo`3oool0e`3oool001X0oooo0P00000;0?ooo`030000003o
ool0oooo01`0oooo00<000000?ooo`3oool0e`3oool001X0oooo00<000000?ooo`3oool02P3oool0
0`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0=P0oooo000J0?ooo`030000003oool0oooo
00X0oooo00<000000?ooo`3oool06`3oool00`000000oooo0?ooo`3H0?ooo`006P3oool00`000000
oooo0?ooo`0;0?ooo`030000003oool0oooo01T0oooo00<000000?ooo`3oool0f@3oool001X0oooo
00<000000?ooo`3oool02`3oool00`000000oooo0?ooo`0I0?ooo`030000003oool0oooo0=T0oooo
000J0?ooo`030000003oool0oooo00/0oooo00<000000?ooo`3oool06@3oool00`000000oooo0?oo
o`3I0?ooo`006P3oool00`000000oooo0?ooo`0;0?ooo`030000003oool0oooo01P0oooo00<00000
0?ooo`3oool0fP3oool001X0oooo00<000000?ooo`3oool0303oool00`000000oooo0?ooo`0G0?oo
o`030000003oool0oooo0=X0oooo000J0?ooo`8000003@3oool00`000000oooo0?ooo`0F0?ooo`03
0000003oool0oooo0=/0oooo000J0?ooo`030000003oool0oooo00`0oooo00<000000?ooo`3oool0
5P3oool00`000000oooo0?ooo`3K0?ooo`006P3oool00`000000oooo0?ooo`0=0?ooo`030000003o
ool0oooo01@0oooo00<000000?ooo`3oool0g03oool001X0oooo00<000000?ooo`3oool03@3oool0
0`000000oooo0?ooo`0D0?ooo`030000003oool0oooo0=`0oooo000J0?ooo`030000003oool0oooo
00h0oooo00<000000?ooo`3oool04P3oool00`000000oooo0?ooo`3M0?ooo`006P3oool00`000000
oooo0?ooo`0>0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0g@3oool001X0oooo
00<000000?ooo`3oool03`3oool00`000000oooo0?ooo`0@0?ooo`030000003oool0oooo0=h0oooo
000J0?ooo`030000003oool0oooo00l0oooo00<000000?ooo`3oool0403oool00`000000oooo0?oo
o`3N0?ooo`006P3oool200000140oooo00<000000?ooo`3oool03P3oool00`000000oooo0?ooo`3O
0?ooo`006P3oool00`000000oooo0?ooo`0A0?ooo`030000003oool0oooo00`0oooo00<000000?oo
o`3oool0h03oool001X0oooo00<000000?ooo`3oool04@3oool00`000000oooo0?ooo`0<0?ooo`03
0000003oool0oooo0>00oooo000J0?ooo`030000003oool0oooo0180oooo00<000000?ooo`3oool0
2P3oool00`000000oooo0?ooo`3Q0?ooo`006P3oool00`000000oooo0?ooo`0B0?ooo`030000003o
ool0oooo00T0oooo00<000000?ooo`3oool0hP3oool001X0oooo00<000000?ooo`3oool04`3oool2
000000P0oooo00<000000?ooo`3oool0h`3oool00140oooo00<000000?ooo`3oool01P3oool00`00
0000oooo0?ooo`0D0?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool0h`3oool00140
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`0E0?ooo`<000000P3oool300000>H0
oooo000A0?ooo`030000003oool0oooo00H0oooo0P00000H0?ooo`<00000j@3oool00140oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooo`@0oooo000A0?ooo`030000003oool0
oooo00H0oooo00<000000?ooo`3oool0o`3oool40?ooo`00403oool2000000P0oooo00<000000?oo
o`3oool0o`3oool40?ooo`004@3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0
oooo103oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0oooo8@3oool00?l0
oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{153.125, 330.625}, {336.25, 226.938}} -> {-6.75526, \
3.54132, 0.0247448, 0.0127444}},
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell["\<\
Programming involves writing a set of instructions to be applied for some \
appropriate input. Whereas procedural programs provide a step-by-step set of \
instructions, functional programming involves the application of functions to \
their arguments. For example, here is a traditional procedural approach to \
switching the elements in a list of pairs.\
\>", "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(lis = {{\[Alpha], 1}, {\[Beta], 2}, {\[Gamma], 3}}\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \({{\[Alpha], 1}, {\[Beta], 2}, {\[Gamma], 3}}\)], "Output",
  CellLabel->"Out[7]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(temp = lis;\)\), "\n", 
    \(\(Do[{temp[\([i, 1]\)], temp[\([i, 2]\)]} = {lis[\([i, 2]\)], 
            lis[\([i, 1]\)]}, \[IndentingNewLine]{i, 1, 
          Length[lis]}];\)\), "\n", 
    \(temp\)}], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \({{1, \[Alpha]}, {2, \[Beta]}, {3, \[Gamma]}}\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[TextData[{
  "We first allocate an empty array ",
  StyleBox["temp", "MR"],
  ", of the same size as ",
  StyleBox["lis", "MR"],
  "; then we put elements into ",
  StyleBox["temp", "MR"],
  " one by one as we loop over ",
  StyleBox["lis", "MR"],
  "; finally we return the value of ",
  StyleBox["temp", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell["Here is a simpler procedure using a structured iteration.", "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Table[{lis[\([i, 2]\)], lis[\([i, 1]\)]}, {i, 1, 3}]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \({{1, \[Alpha]}, {2, \[Beta]}, {3, \[Gamma]}}\)], "Output",
  CellLabel->"Out[11]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell["\<\
And here is a functional approach to solving the same problem.\
\>", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Reverse, lis]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[BoxData[
    \({{1, \[Alpha]}, {2, \[Beta]}, {3, \[Gamma]}}\)], "Output",
  CellLabel->"Out[12]=",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[TextData[{
  "This simple example illustrates several of the key features of functional \
programming. A functional approach often allows for a more direct \
implementation of the solution to many problems, especially when list \
manipulations are involved. Notice that the procedural approach required \
setting up a list structure and then looping over the list as ",
  StyleBox["i", "MR"],
  " takes on successive values, whereas the functional approach simply \
applied the ",
  StyleBox["Reverse", "MR"],
  " function to the list directly."
}], "Text",
  CellTags->{"i:1", "b:11", "IPM3.4.1"}],

Cell[TextData[{
  "Up to this point, we have described fairly simple functions and stayed \
focused on the built-in functions present in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". In this chapter we will first take a look at some of the most powerful \
and useful functional programming constructs in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and then discuss the creation of our own functions, using many of the \
list and string manipulating functions discussed earlier. It is well \
worthwhile to spend time familiarizing yourself with these functions by \
playing around with them; for example, create various lists and apply \
built-in functions to them. Having a larger vocabulary of built-in functions \
will not only make it easier to follow the programs and do the exercises \
here, but will enhance your own programming skills as well."
}], "Text",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Functions for manipulating expressions", "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
  CellTags->{
  "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
    "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],

Cell[TextData[{
  "Three of the most powerful and commonly used functions by experienced ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programmers are ",
  StyleBox["Map", "MR"],
  ", ",
  StyleBox["Apply", "MR"],
  ", and ",
  StyleBox["Thread", "MR"],
  ". They provide very sophisticated ways of manipulating expressions in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Becoming familiar with them is essential to functional programming in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". In this section we will discuss their syntax and look at some simple \
examples of their use. We will also briefly look at some related functions \
(",
  StyleBox["Inner", "MR"],
  " and ",
  StyleBox["Outer", "MR"],
  "), which will prove useful in manipulating the structure of your \
expressions. These higher-order functions will be used throughout the rest of \
this book."
}], "TextFirst",
  CellTags->"IPM3.4.2.0"],

Cell[CellGroupData[{

Cell["Map", "Subsection",
  PageBreakAbove->True,
  CellTags->{"b:12.0", "IPM3.4.2.1", "c:4"}],

Cell[TextData[{
  StyleBox["Map", "MR"],
  " applies a function to each element in a list. "
}], "TextFirst",
  CellTags->{"i:2", "b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Head, {3, 22\/7, \[Pi]}]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[BoxData[
    \({Integer, Rational, Symbol}\)], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],

Cell[TextData[{
  "This is illustrated using an undefined function ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  " and a simple linear list."
}], "TextFirst",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[f, {a, b, c}]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[BoxData[
    \({f[a], f[b], f[c]}\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],

Cell[TextData[{
  "More generally, mapping a function ",
  StyleBox["f", "MR"],
  " over the expression ",
  StyleBox["g[a,b,c]", "MR"],
  " essentially wraps the function ",
  StyleBox["f", "MR"],
  " around each of the ",
  StyleBox["elements",
    FontSlant->"Italic"],
  " of ",
  StyleBox["g", "MR"],
  "."
}], "Text",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[f, g[a, b, c]]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[BoxData[
    \(g[f[a], f[b], f[c]]\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],

Cell[TextData[{
  "So this general computation is identical to ",
  StyleBox["Map[f,{a,b,c}]", "MR"],
  ", except in that example ",
  StyleBox["g", "MR"],
  " is replaced with ",
  StyleBox["List", "MR"],
  " (remember that ",
  StyleBox["FullForm[{a,b,c}]", "MR"],
  " is ",
  StyleBox["List[a,b,c]", "MR"],
  ")."
}], "TextFirst",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[TextData[{
  "The real power of the ",
  StyleBox["Map", "MR"],
  " function is that you can map ",
  StyleBox["any",
    FontSlant->"Italic"],
  " function across any expression for which that function makes sense. Using \
the ",
  StyleBox["Reverse", "MR"],
  " function with ",
  StyleBox["Map", "MR"],
  ", you can reverse the order of elements in each list of a nested list."
}], "Text",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Reverse, {{a, b}, {c, d}, {e, f}}]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[BoxData[
    \({{b, a}, {d, c}, {f, e}}\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],

Cell["\<\
The elements in each of the inner lists in a nested list can be sorted.\
\>", "TextFirst",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Sort, {{2, 6, 3, 5}, {7, 4, 1, 3}}]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[BoxData[
    \({{2, 3, 5, 6}, {1, 3, 4, 7}}\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],

Cell[TextData[{
  "Often, you will need to define your own function to perform some \
computation on every element of a list. This is the sort of computation that \
",
  StyleBox["Map", "MR"],
  " is expressly designed for. Here is a list of three elements."
}], "Text",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[BoxData[
    \(\(vec = {2, \[Pi], \[Gamma]};\)\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell["\<\
If we wished to square each element and add 1, we could first define a \
function that performs this computation on its arguments.\
\>", "TextFirst",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[BoxData[
    \(f[x_] := x\^2 + 1\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[TextData[{
  "Mapping this function over ",
  StyleBox["vec", "MR"],
  ", will then wrap ",
  StyleBox["f", "MR"],
  " around each element and evaluate ",
  StyleBox["f", "MR"],
  " of those elements."
}], "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[f, vec]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[BoxData[
    \({5, 1 + \[Pi]\^2, 1 + \[Gamma]\^2}\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],

Cell["\<\
Later in this chapter we will look at even simpler ways of performing such \
computations.\
\>", "Text",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Thread and MapThread", "Subsection",
  CellTags->{"b:12.1", "IPM3.4.2.2", "c:5"}],

Cell[TextData[{
  "The ",
  StyleBox["Thread", "MR"],
  " function exchanges operations with arguments that are lists."
}], "TextFirst",
  CellTags->{"i:3", "b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Thread[g[{a, b, c}, {x, y, z}]]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    \({g[a, x], g[b, y], g[c, z]}\)], "Output",
  CellLabel->"Out[9]=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],

Cell[TextData[{
  "You can accomplish something quite similar with ",
  StyleBox["MapThread", "MR"],
  ". It differs from ",
  StyleBox["Thread", "MR"],
  " in that it takes two arguments",
  " \[Dash] ",
  "the function that you are mapping and a list of two (or more) lists as \
arguments of the function. It creates a new list in which the corresponding \
elements of the old lists are paired (or zipped together)."
}], "Text",
  CellTags->{"i:4", "b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MapThread[g, {{a, b, c}, {x, y, z}}]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    \({g[a, x], g[b, y], g[c, z]}\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],

Cell[TextData[{
  "With ",
  StyleBox["Thread", "MR"],
  ", you can fundamentally change the structure of the expressions you are \
working with. For example, this threads the ",
  StyleBox["Equal", "MR"],
  " function over the two lists given as its arguments."
}], "Text",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Thread[Equal[{a, b, c}, {x, y, z}]]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    \({a \[Equal] x, b \[Equal] y, c \[Equal] z}\)], "Output",
  CellLabel->"Out[11]=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[FullForm, %]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[
          StyleBox[\(Equal[a, x]\),
            ShowSpecialCharacters->False,
            ShowStringCharacters->True,
            NumberMarks->True],
          FullForm], ",", 
        TagBox[
          StyleBox[\(Equal[b, y]\),
            ShowSpecialCharacters->False,
            ShowStringCharacters->True,
            NumberMarks->True],
          FullForm], ",", 
        TagBox[
          StyleBox[\(Equal[c, z]\),
            ShowSpecialCharacters->False,
            ShowStringCharacters->True,
            NumberMarks->True],
          FullForm]}], "}"}]], "Output",
  CellLabel->"Out[12]=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],

Cell[TextData[{
  "Here is another example of the use of ",
  StyleBox["Thread", "MR"],
  ". We start off with a list of variables and a list of values. "
}], "Text",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    \(\(vars = {x\_1, x\_2, x\_3, x\_4, x\_5};\)\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    \(\(values = {1.2, 2.5, 5.7, 8.21, 6.66};\)\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell["From these two lists, we create a list of rules.", "TextFirst",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Thread[Rule[vars, values]]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    \({x\_1 \[Rule] 1.2`, x\_2 \[Rule] 2.5`, x\_3 \[Rule] 5.7`, 
      x\_4 \[Rule] 8.21`, x\_5 \[Rule] 6.66`}\)], "Output",
  CellLabel->"Out[15]=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],

Cell[TextData[{
  "Notice how we started with a ",
  StyleBox["rule of lists",
    FontSlant->"Italic"],
  " and ",
  StyleBox["Thread", "MR"],
  " produced a ",
  StyleBox["list of rules",
    FontSlant->"Italic"],
  ". In this way, you might think of ",
  StyleBox["Thread", "MR"],
  " as a generalization of ",
  StyleBox["Transpose", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[TextData[{
  "Here are a few more examples of ",
  StyleBox["MapThread", "MR"],
  ". This raises each element in the first list to the power given by the \
corresponding element in the second list."
}], "Text",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MapThread[Power, {{2, 6, 3}, {5, 1, 2}}]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    \({32, 6, 9}\)], "Output",
  CellLabel->"Out[16]=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],

Cell[TextData[{
  "Using ",
  StyleBox["Trace", "MR"],
  ", you can view some of the intermediate steps that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " performs in doing this calculation."
}], "TextFirst",
  CellTags->{"i:5", "b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MapThread[Power, {{2, 6, 3}, {5, 1, 2}}] // Trace\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        TagBox[\(MapThread[Power, {{2, 6, 3}, {5, 1, 2}}]\),
          HoldForm], ",", 
        TagBox[\({2\^5, 6\^1, 3\^2}\),
          HoldForm], ",", 
        RowBox[{"{", 
          RowBox[{
            TagBox[\(2\^5\),
              HoldForm], ",", 
            TagBox["32",
              HoldForm]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            TagBox[\(6\^1\),
              HoldForm], ",", 
            TagBox["6",
              HoldForm]}], "}"}], ",", 
        RowBox[{"{", 
          RowBox[{
            TagBox[\(3\^2\),
              HoldForm], ",", 
            TagBox["9",
              HoldForm]}], "}"}], ",", 
        TagBox[\({32, 6, 9}\),
          HoldForm]}], "}"}]], "Output",
  CellLabel->"Out[17]=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],

Cell[TextData[{
  "Using the ",
  StyleBox["List", "MR"],
  " function, the corresponding elements in the three lists are placed in a \
list structure (note that ",
  StyleBox["Transpose", "MR"],
  " would do the same thing)."
}], "TextFirst",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MapThread[List, {{5, 3, 2}, {6, 4, 9}, {4, 1, 4}}]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[BoxData[
    \({{5, 6, 4}, {3, 4, 1}, {2, 9, 4}}\)], "Output",
  CellLabel->"Out[18]=",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["The Listable attribute", "Subsection",
  CellTags->{"b:12.2", "IPM3.4.2.3", "c:6"}],

Cell[TextData[{
  "Many of the built-in functions that take a single argument have the \
property that, when a list is the argument, the function is automatically \
applied to all of the elements in the list. In other words, these functions \
are automatically mapped on to the elements of the list. For example, the ",
  StyleBox["Log", "MR"],
  " function has this attribute."
}], "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Log[{a, E, 1}]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \({Log[a], 1, 0}\)], "Output",
  CellLabel->"Out[19]=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell[TextData[{
  "This is the same result you get using the ",
  StyleBox["Map", "MR"],
  " function."
}], "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Log, {a, E, 1}]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \({Log[a], 1, 0}\)], "Output",
  CellLabel->"Out[20]=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell["\<\
Many of the built-in functions that take two or more arguments have the \
property that, when multiple lists are the arguments, the function is \
automatically applied to all of the corresponding elements in the list. In \
other words, these functions are automatically threaded on to the elements of \
the list.\
\>", "Text",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \({4, 6, 3} + {5, 1, 2}\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \({9, 7, 5}\)], "Output",
  CellLabel->"Out[21]=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell[TextData[{
  "This gives the same result as using the ",
  StyleBox["Plus", "MR"],
  " function with ",
  StyleBox["MapThread", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MapThread[Plus, {{4, 6, 3}, {5, 1, 2}}]\)], "Input",
  CellLabel->"In[22]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \({9, 7, 5}\)], "Output",
  CellLabel->"Out[22]=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell[TextData[{
  "Functions that are either automatically mapped or threaded on to the \
elements of list arguments are said to be ",
  StyleBox["Listable", "MR"],
  ". Many of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\[CloseCurlyQuote]s built-in functions have this ",
  StyleBox["Attribute", "MR"],
  ". "
}], "Text",
  CellTags->{"i:6", "b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Attributes[Log]\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \({Listable, NumericFunction, Protected}\)], "Output",
  CellLabel->"Out[23]=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Attributes[Plus]\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \({Flat, Listable, NumericFunction, OneIdentity, Orderless, 
      Protected}\)], "Output",
  CellLabel->"Out[24]=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell[TextData[{
  "By default, functions that you define do not have any attributes \
associated with them. So, for example, if you define a function ",
  StyleBox["g", "MR"],
  ", say, it will not automatically be threaded over a list."
}], "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(g[{{a, b}, {c, d}}]\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \(g[{{a, b}, {c, d}}]\)], "Output",
  CellLabel->"Out[25]=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell[TextData[{
  "If you want your function to have the ability to thread over lists, give \
it the ",
  StyleBox["Listable", "MR"],
  " attribute using ",
  StyleBox["SetAttributes", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \(SetAttributes[g, Listable]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->{"i:7", "b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(g[{{a, b}, {c, d}}]\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \({{g[a], g[b]}, {g[c], g[d]}}\)], "Output",
  CellLabel->"Out[27]=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell["\<\
Note that clearing a symbol only clears values associated with that symbol. \
It does not clear any attributes associated with the symbol.\
\>", "TextFirst",
  CellTags->{"i:8", "b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \(Clear[g]\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?g\)\)], "Input",
  CellLabel->"In[29]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell["Global`g", "Print",
  CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {\(Attributes[g] = {Listable}\)}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "g"],
      Editable->False]], "Print",
  CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}]
}, Closed]],

Cell[TextData[{
  "To clear attributes, you need to use ",
  StyleBox["Remove", "MR"],
  "."
}], "TextFirst",
  CellTags->{"i:9", "b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    \(Remove[g]\)], "Input",
  CellLabel->"In[30]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[TextData[{
  "Now there is no remaining information associated with ",
  StyleBox["g", "MR"],
  "."
}], "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?g\)\)], "Input",
  CellLabel->"In[31]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[BoxData[
    RowBox[{\(Information::"notfound"\), \(\(:\)\(\ \)\), "\<\"Symbol \
\\!\\(\\\"g\\\"\\) not found. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"General::notfound\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[31]:=",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Apply", "Subsection",
  CellTags->{"b:12.3", "IPM3.4.2.4", "c:7"}],

Cell[TextData[{
  "Whereas ",
  StyleBox["Map", "MR"],
  " is used to perform the same operation on each element of an expression, \
",
  StyleBox["Apply", "MR"],
  " is used to change the structure of an expression."
}], "TextFirst",
  CellTags->{"i:10", "b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[f, g[a, b, c]]\)], "Input",
  CellLabel->"In[32]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \(f[a, b, c]\)], "Output",
  CellLabel->"Out[32]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "The function ",
  StyleBox["f", "MR"],
  " was applied to the expression ",
  StyleBox["g[a,b,c]", "MR"],
  " and ",
  StyleBox["Apply", "MR"],
  " replaced the head of ",
  StyleBox["g[a,b,c]", "MR"],
  " with ",
  StyleBox["f", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[TextData[{
  "If the second argument is a list, applying ",
  StyleBox["f", "MR"],
  " to that expression simply replaces its head (",
  StyleBox["List", "MR"],
  ") with ",
  StyleBox["f", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[f, {a, b, c}]\)], "Input",
  CellLabel->"In[33]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \(f[a, b, c]\)], "Output",
  CellLabel->"Out[33]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "The following computation shows the same thing, except we are using the \
internal representation of the list ",
  StyleBox["{a,b,c}", "MR"],
  " here to better see how the structure is changed."
}], "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[f, List[a, b, c]]\)], "Input",
  CellLabel->"In[34]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \(f[a, b, c]\)], "Output",
  CellLabel->"Out[34]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "We see that the elements of ",
  StyleBox["List", "MR"],
  " are now the arguments of ",
  StyleBox["f", "MR"],
  ". Essentially, you should think of ",
  StyleBox["Apply[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  StyleBox[",", "MR"],
  Cell[BoxData[
      FormBox[
        StyleBox["expr",
          "IT"], TraditionalForm]]],
  StyleBox["]", "MR"],
  " as replacing the head of ",
  Cell[BoxData[
      FormBox[
        StyleBox["expr",
          "IT"], TraditionalForm]]],
  " with ",
  Cell[BoxData[
      \(TraditionalForm\`f\)]],
  ". "
}], "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, {1, 2, 3, 4}]\)], "Input",
  CellLabel->"In[35]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \(10\)], "Output",
  CellLabel->"Out[35]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "Here, ",
  StyleBox["List[1,2,3,4]", "MR"],
  " has been changed to ",
  StyleBox["Plus[1,2,3,4]", "MR"],
  " or, in other words, the head ",
  StyleBox["List", "MR"],
  " has been replaced by ",
  StyleBox["Plus", "MR"],
  ". "
}], "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[TextData[{
  StyleBox["Plus[a,b,c,d]", "MR"],
  " is the internal representation of the sum of these four symbols that you \
would normally write ",
  StyleBox["a+b+c+d", "MR"],
  "."
}], "Text",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plus[a, b, c, d]\)], "Input",
  CellLabel->"In[36]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \(a + b + c + d\)], "Output",
  CellLabel->"Out[36]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell["This list conversion can be applied to an entire list.", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[f, {{1, 2, 3}, {5, 6, 7}}]\)], "Input",
  CellLabel->"In[37]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \(f[{1, 2, 3}, {5, 6, 7}]\)], "Output",
  CellLabel->"Out[37]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell["This is just vector addition.", "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, {{1, 2, 3}, {5, 6, 7}}]\)], "Input",
  CellLabel->"In[38]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \({6, 8, 10}\)], "Output",
  CellLabel->"Out[38]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "One important distinction between ",
  StyleBox["Map", "MR"],
  " and ",
  StyleBox["Apply", "MR"],
  " that you should be aware of concerns the level of the expression at which \
each operate. By default, ",
  StyleBox["Map", "MR"],
  " operates at level 1. That is, in ",
  StyleBox["Map[", "MR"],
  StyleBox["f", "IT"],
  StyleBox[",\[ThinSpace]", "MR"],
  StyleBox["expr", "IT"],
  StyleBox["]", "MR"],
  ", ",
  StyleBox["f", "IT"],
  " will be applied to each element at the top level of ",
  StyleBox["expr", "IT"],
  ". So, for example, if ",
  StyleBox["expr", "IT"],
  " consists of a nested list, ",
  StyleBox["f", "IT"],
  " will be applied to each of the sublists, but not deeper, by default."
}], "Text",
  CellTags->{"i:11", "b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[f, {{a, b}, {c, d}}]\)], "Input",
  CellLabel->"In[39]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \({{1 + a\^2, 1 + b\^2}, {1 + c\^2, 1 + d\^2}}\)], "Output",
  CellLabel->"Out[39]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "If you wish to apply ",
  StyleBox["f", "MR"],
  " at a deeper level, then you have to specify that explicitly using a third \
argument to ",
  StyleBox["Map", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[f, {{a, b}, {c, d}}, {2}]\)], "Input",
  CellLabel->"In[40]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \({{1 + a\^2, 1 + b\^2}, {1 + c\^2, 1 + d\^2}}\)], "Output",
  CellLabel->"Out[40]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  StyleBox["Apply", "MR"],
  ", on the other hand, operates at level 0. That is, in ",
  StyleBox["Apply[", "MR"],
  StyleBox["f", "IT"],
  StyleBox[",\[ThinSpace]", "MR"],
  StyleBox["expr", "IT"],
  StyleBox["]", "MR"],
  ", ",
  StyleBox["Apply", "MR"],
  " looks at the part 0 of ",
  StyleBox["expr", "IT"],
  " (",
  "that is",
  ", its ",
  StyleBox["Head", "MR"],
  ") and replaces it with ",
  StyleBox["f", "IT"],
  "."
}], "Text",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[f, {{a, b}, {c, d}}]\)], "Input",
  CellLabel->"In[41]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \(f[{a, b}, {c, d}]\)], "Output",
  CellLabel->"Out[41]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "Again, if you wish to apply ",
  StyleBox["f", "MR"],
  " at a different level, then you have to specify that explicitly using a \
third argument to ",
  StyleBox["Apply", "MR"],
  "."
}], "Text",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[f, {{a, b}, {c, d}}, 1]\)], "Input",
  CellLabel->"In[42]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \({f[a, b], f[c, d]}\)], "Output",
  CellLabel->"Out[42]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "For example, to apply ",
  StyleBox["Plus", "MR"],
  " to each of the inner lists, you need to specify that ",
  StyleBox["Apply", "MR"],
  " will operate at level 1."
}], "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, {{1, 2, 3}, {5, 6, 7}}, {1}]\)], "Input",
  CellLabel->"In[43]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \({6, 18}\)], "Output",
  CellLabel->"Out[43]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],

Cell[TextData[{
  "If you are a little unsure of what has just happened, consider the \
following example and, instead of ",
  StyleBox["f", "MR"],
  ", think of ",
  StyleBox["Plus", "MR"],
  "."
}], "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[f, {{1, 2, 3}, {5, 6, 7}}, {1}]\)], "Input",
  CellLabel->"In[44]:=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[BoxData[
    \({f[1, 2, 3], f[5, 6, 7]}\)], "Output",
  CellLabel->"Out[44]=",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Inner and Outer", "Subsection",
  CellTags->{"b:12.4", "IPM3.4.2.5", "c:8"}],

Cell[TextData[{
  "The ",
  StyleBox["Outer", "MR"],
  " function applies a function to all of the combinations of the elements in \
several lists. This is a generalization of the mathematical ",
  StyleBox["outer product", "TermDefinition"],
  "."
}], "TextFirst",
  CellTags->{"i:12", "b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Outer[f, {a, b}, {2, 3, 4}]\)], "Input",
  CellLabel->"In[45]:=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[BoxData[
    \({{f[a, 2], f[a, 3], f[a, 4]}, {f[b, 2], f[b, 3], f[b, 4]}}\)], "Output",\

  CellLabel->"Out[45]=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],

Cell[TextData[{
  "Using the ",
  StyleBox["List", "MR"],
  " function as an argument, you can create lists of ordered pairs that \
combine the elements of several lists."
}], "TextFirst",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Outer[List, {a, b}, {2, 3, 4}]\)], "Input",
  CellLabel->"In[46]:=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[BoxData[
    \({{{a, 2}, {a, 3}, {a, 4}}, {{b, 2}, {b, 3}, {b, 4}}}\)], "Output",
  CellLabel->"Out[46]=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],

Cell[TextData[{
  "Using ",
  StyleBox["Inner", "MR"],
  ", you can thread a function on to several lists and then use the result as \
the argument to another function."
}], "TextFirst",
  CellTags->{"i:13", "b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Inner[f, {a, b, c}, {d, e, f}, g]\)], "Input",
  CellLabel->"In[47]:=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[BoxData[
    \(g[f[a, d], f[b, e], f[c, f]]\)], "Output",
  CellLabel->"Out[47]=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],

Cell["\<\
This function lets you carry out some interesting operations.\
\>", "Text",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Inner[Times, {x\_1, y\_1, z\_1}, {x\_2, y\_2, z\_2}, Plus]\)], "Input",
  CellLabel->"In[48]:=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[BoxData[
    \(x\_1\ x\_2 + y\_1\ y\_2 + z\_1\ z\_2\)], "Output",
  CellLabel->"Out[48]=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Inner[List, {a, b, c}, {d, e, f}, Plus]\)], "Input",
  CellLabel->"In[49]:=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[BoxData[
    \({a + b + c, d + e + f}\)], "Output",
  CellLabel->"Out[49]=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],

Cell[TextData[{
  "Looking at these two examples, you can see that ",
  StyleBox["Inner", "MR"],
  " is really a generalization of the mathematical dot product."
}], "TextFirst",
  CellTags->{"i:14", "b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Dot[{x\_1, y\_1, z\_1}, {x\_2, y\_2, z\_2}]\)], "Input",
  CellLabel->"In[50]:=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[BoxData[
    \(x\_1\ x\_2 + y\_1\ y\_2 + z\_1\ z\_2\)], "Output",
  CellLabel->"Out[50]=",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:12.4", "IPM3.4.2.6", "c:9"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Write a function ",
  StyleBox["addPair[{x,y}]", "MR"],
  " that adds the elements in a pair. Then use your ",
  StyleBox["addPair", "MR"],
  " function to sum each pair from the following."
}], "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[BoxData[
    \(\(data = {{1, 2}, {2, 3}, {3, 4}, {4, 5}, {5, 6}};\)\)], "Input",
  Evaluatable->False,
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[TextData[{
  "Your output should look like ",
  StyleBox["{3,5,7,9,11}", "MR"],
  "."
}], "ExercisesNoLabel",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],

Cell[TextData[{
  "Use ",
  StyleBox["Apply", "MR"],
  " to add the elements in each pair from a list of pairs of numbers such as \
in the previous exercise."
}], "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[TextData[{
  "A matrix can be rotated by performing a number of successive operations. \
Rotate the matrix ",
  StyleBox["{{1,2,3},{4,5,6}}", "MR"],
  " clockwise by 90 degrees, obtaining ",
  StyleBox["{{4,1},{5,2},{6,3}}", "MR"],
  ", in two steps. Use ",
  StyleBox["TableForm", "MR"],
  " to display the results."
}], "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[CellGroupData[{

Cell[TextData[{
  "While matrices can easily be added using ",
  StyleBox["Plus", "MR"],
  ", matrix multiplication is more complicated. The ",
  StyleBox["Dot", "MR"],
  " function, written as a single period, can be used."
}], "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[CellGroupData[{

Cell[BoxData[
    \({{1, 2}, {3, 4}} . {x, y}\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[BoxData[
    \({x + 2\ y, 3\ x + 4\ y}\)], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],

Cell[TextData[{
  "Perform matrix multiplication on ",
  StyleBox["{{1,2},{3,4}}", "MR"],
  " and ",
  StyleBox["{x,y}", "MR"],
  " without using ",
  StyleBox["Dot", "MR"],
  ". (This can be done in two or three steps.)"
}], "ExercisesNoLabel",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  StyleBox["FactorInteger[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  StyleBox["]", "MR"],
  " returns a nested list of prime factors and their exponents for the number \
",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "."
}], "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(FactorInteger[3628800]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[BoxData[
    \({{2, 8}, {3, 4}, {5, 2}, {7, 1}}\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],

Cell[TextData[{
  "Use ",
  StyleBox["Apply", "MR"],
  " to reconstruct the number from this nested list."
}], "ExercisesNoLabel",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],

Cell[TextData[{
  "Repeat the above exercise but instead use ",
  StyleBox["Inner", "MR"],
  " to construct the original number ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " from the factorization given by ",
  StyleBox["FactorInteger[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  StyleBox["]", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:13", "IPM3.4.2.6"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Using ",
  StyleBox["Inner", "MR"],
  ", write a function ",
  StyleBox["div[", "MR"],
  StyleBox["vecs", "IT"],
  StyleBox[",", "MR"],
  StyleBox["vars", "IT"],
  StyleBox["]", "MR"],
  " that computes the divergence of an ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-dimensional vector field ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["vecs",
            FontSlant->"Italic"], "=", \({e\_1, e\_2, \[Ellipsis], e\_n}\)}], 
        TraditionalForm]]],
  " dependent upon ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " variables ",
  Cell[BoxData[
      FormBox[
        RowBox[{
          StyleBox["vars",
            FontSlant->"Italic"], "=", \({v\_1, v\_2, \[Ellipsis], v\_n}\)}], 
        TraditionalForm]]],
  ". The divergence is given by the sum of the pairwise partial derivatives."
}], "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6", "i:59"}],

Cell[BoxData[
    \(TraditionalForm\`\[PartialD]e\_1\/\[PartialD]v\_1 + \[PartialD]e\_2\/\
\[PartialD]v\_2 + \[Ellipsis] + \[PartialD]e\_n\/\[PartialD]v\_n\)], \
"DisplayFormula",
  PageBreakAbove->False,
  PageBreakBelow->True,
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Iterating functions", "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 2}},
  CellTags->{"b:13", "IPM3.4.3.1", "IPM3.4.3.2", "c:10"}],

Cell[TextData[{
  "A commonly performed task in computer science and mathematics is to \
repeatedly apply a function to some expression. Iterating functions has a \
long and rich tradition in the history of computing. Perhaps the most famous \
example is Newton\[CloseCurlyQuote]s method for root finding. Chaos theory \
rests on studying how iterated functions behave under small perturbations of \
their initial conditions or starting values. In this section, we will \
introduce several functions available in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " for function iteration. In later chapters we will apply these and other \
programming constructs to look at some applications of iteration, including \
Newton\[CloseCurlyQuote]s method."
}], "TextFirst",
  CellTags->{"i:15", "b:13", "IPM3.4.3.1"}],

Cell[TextData[{
  "The ",
  StyleBox["Nest", "MR"],
  " function is used to iterate functions. Here, ",
  StyleBox["g", "MR"],
  " is iterated (or applied to) ",
  StyleBox["a", "MR"],
  " four times."
}], "Text",
  CellTags->{"i:16", "b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Nest[g, a, 4]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \(g[g[g[g[a]]]]\)], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[TextData[{
  "The ",
  StyleBox["NestList", "MR"],
  " function displays all of the intermediate values of the ",
  StyleBox["Nest", "MR"],
  " operation."
}], "TextFirst",
  CellTags->{"i:17", "b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(NestList[g, a, 4]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \({a, g[a], g[g[a]], g[g[g[a]]], g[g[g[g[a]]]]}\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[TextData[{
  "Using a starting value of 0.85, this generates a list of ten iterates of \
the ",
  StyleBox["Cos", "MR"],
  " function."
}], "TextFirst",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(NestList[Cos, 0.85, 10]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \({0.85`, 0.6599831458849822`, 0.7900025649271041`, 0.703843493645559`, 
      0.7623604975064312`, 0.72320779552426`, 0.7496866935497157`, 
      0.7319023947789758`, 0.7439044053660816`, 0.7358302433371221`, 
      0.7412737472749241`}\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[TextData[{
  "The list elements above are the values of ",
  StyleBox["0.85", "MR"],
  ", ",
  StyleBox["Cos[0.85]", "MR"],
  ", ",
  StyleBox["Cos[Cos[0.85]]", "MR"],
  ", and so on."
}], "TextFirst",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \({0.85, Cos[0.85], Cos[Cos[0.85]], Cos[Cos[Cos[0.85]]]}\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \({0.85`, 0.6599831458849822`, 0.7900025649271041`, 
      0.703843493645559`}\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[TextData[{
  "In fact, the iterates of the cosine function tend towards a fixed point \
which can be obtained with ",
  StyleBox["FixedPoint", "MR"],
  ". This function is particularly useful when you do not know how many \
iterations to perform on a function whose iterations eventually settle down."
}], "Text",
  CellTags->{"i:18", "b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(FixedPoint[Cos, 0.85]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \(0.7390851332151607`\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[TextData[{
  "Whereas ",
  StyleBox["Nest", "MR"],
  " and ",
  StyleBox["NestList", "MR"],
  " operate on functions of one variable, ",
  StyleBox["Fold", "MR"],
  " and ",
  StyleBox["FoldList", "MR"],
  " generalize this notion by iterating a function of two arguments. In the \
following example, the function ",
  StyleBox["f", "MR"],
  " is first applied to a starting value ",
  StyleBox["x", "MR"],
  " and the first element from a list, then this result is used as the first \
argument of the next iteration, with the second argument coming from the \
second element in the list, and so on."
}], "Text",
  CellTags->{"i:19", "b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Fold[f, x, {a, b, c}]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \(f[f[f[x, a], b], c]\)], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[TextData[{
  "If ",
  StyleBox["FoldList", "MR"],
  " is used, then you will see all of the intermediate results of the ",
  StyleBox["Fold", "MR"],
  " operation."
}], "TextFirst",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(FoldList[f, x, {a, b, c}]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \({x, f[x, a], f[f[x, a], b], f[f[f[x, a], b], c]}\)], "Output",
  CellLabel->"Out[7]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[TextData[{
  "It is easy to see what is going on with the ",
  StyleBox["FoldList", "MR"],
  " function by working with an arithmetic operator. This generates \
\[OpenCurlyDoubleQuote]running sums.\[CloseCurlyDoubleQuote]"
}], "TextFirst",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(FoldList[Plus, 0, {a, b, c, d}]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \({0, a, a + b, a + b + c, a + b + c + d}\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(FoldList[Plus, 0, {1, 2, 3, 4, 5}]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[BoxData[
    \({0, 1, 3, 6, 10, 15}\)], "Output",
  CellLabel->"Out[9]=",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:13", "IPM3.4.3.2", "c:11"}],

Cell[TextData[{
  "Determine the locations after each step of a ten-step one-dimensional \
random walk. (Recall that you have already generated the step ",
  StyleBox["directions",
    FontSlant->"Italic"],
  " in Exercise 3 at the end of Section 3.2.)"
}], "ExerciseItem",
  CellTags->{"b:13", "IPM3.4.3.2"}],

Cell["\<\
Create a list of the step locations of a ten-step random walk on a square \
lattice.\
\>", "ExerciseItem",
  CellTags->{"b:13", "IPM3.4.3.2"}],

Cell[TextData[{
  "Using ",
  StyleBox["Fold", "MR"],
  ", create a function ",
  StyleBox["fac[", "MR"],
  StyleBox["n", "IT"],
  StyleBox["]", "MR"],
  " that takes in an integer ",
  StyleBox["n",
    FontSlant->"Italic"],
  " as argument and returns the factorial of ",
  StyleBox["n",
    FontSlant->"Italic"],
  "; that is, ",
  Cell[BoxData[
      \(TraditionalForm\`\(n(
          n - 1)\) \((n - 
            2)\) \[CenterEllipsis]3\[CenterDot]2\[CenterDot]1\)]],
  "."
}], "ExerciseItem",
  CellTags->{"b:13", "IPM3.4.3.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Programs as functions", "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 3}},
  CellTags->{"IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", "c:12"}],

Cell["\<\
A computer program is a set of instructions (a recipe) for carrying out a \
computation. When a program is evaluated with appropriate inputs, the \
computation is performed and the result is returned. In this sense, a program \
is a mathematical function and the inputs to a program are the arguments of \
the function. Executing a program is equivalent to applying a function to its \
arguments or, as it is often referred, making a function call.\
\>", "TextFirst",
  CellTags->"IPM3.4.4.0"],

Cell[CellGroupData[{

Cell["User-defined functions", "Subsection",
  CellTags->{"b:14.0", "IPM3.4.4.1", "c:13"}],

Cell[TextData[{
  "While there are a great many built-in functions in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " that can be used to carry out computations, we invariably find ourselves \
needing customized functions. For example, once we have written a program to \
compute some values for some particular inputs, we might want to perform the \
same set of operations on different inputs. We would therefore like to create \
our own ",
  StyleBox["user-defined",
    FontSlant->"Italic"],
  " functions that we could then apply in the same way as we call a built-in \
function",
  " \[Dash] ",
  "by entering the function name and specific argument values. We will start \
with the proper syntax (or grammar) to use when writing a function \
definition."
}], "TextFirst",
  CellTags->{"i:20", "b:14.0", "IPM3.4.4.1"}],

Cell["\<\
The function definition looks very much like a mathematical equation: a \
left-hand side and a right-hand side separated by a colon-equal sign.\
\>", "Text",
  CellTags->{"i:21", "b:14.0", "IPM3.4.4.1"}],

Cell[TextData[{
  StyleBox["name", "TI"],
  "[",
  Cell[BoxData[
      RowBox[{
        FormBox[
          SubscriptBox[
            StyleBox["arg",
              "TI"], "1"],
          "TraditionalForm"], "_"}]]],
  ",",
  Cell[BoxData[
      RowBox[{
        FormBox[
          SubscriptBox[
            StyleBox["arg",
              "TI"], "2"],
          "TraditionalForm"], "_"}]]],
  ",\[Ellipsis],",
  Cell[BoxData[
      RowBox[{
        FormBox[
          SubscriptBox[
            StyleBox["arg",
              "TI"], 
            StyleBox["n",
              "TI"]],
          "TraditionalForm"], "_"}]]],
  "]:= ",
  StyleBox["body", "TI"]
}], "Program",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],

Cell[TextData[{
  "The left-hand side starts with a symbol. This symbol is referred to as the \
",
  StyleBox["function name", "TermDefinition"],
  " (or sometimes just as the function, as in \[OpenCurlyDoubleQuote]the sine \
function\[CloseCurlyDoubleQuote]). The function name is followed by a set of \
square brackets, inside of which are a sequence of symbols ending with ",
  StyleBox["blanks", "TermDefinition"],
  ". These symbols are referred to as the ",
  StyleBox["function argument names", "TermDefinition"],
  ", or just the ",
  StyleBox["function arguments", "TermDefinition"],
  "."
}], "Text",
  CellTags->{"i:22", "b:14.0", "IPM3.4.4.1"}],

Cell[TextData[{
  "The right-hand side of a user-defined function definition is called the ",
  StyleBox["body", "TermDefinition"],
  " of the function. The body can be either a single expression (a \
one-liner), or a series of expressions (a compound function), both of which \
will be discussed in detail shortly. Argument names from the left-hand side \
appear on the right-hand side without blanks. Basically, the right-hand side \
is a formula stating what computations are to be done when the function is \
called with specific values of the arguments. "
}], "Text",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],

Cell[TextData[{
  "When a user-defined function is defined with a delayed assignment (",
  StyleBox[":=", "MR"],
  "), nothing is returned. Thereafter, calling the function by entering the \
left-hand side of the function definition with specific values of the \
arguments causes the body of the function to be computed with the specific \
argument values substituted where the argument names occur. In other words, \
when using delayed assignments, the body of your function is only evaluated \
when the function is called, not when it is first defined."
}], "Text",
  CellTags->{"i:23", "b:14.0", "IPM3.4.4.1"}],

Cell[TextData[{
  "A simple example of a user-defined function is ",
  StyleBox["square", "MR"],
  " which squares a value (it is a good idea to use a function name that \
indicates the purpose of the function)."
}], "Text",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],

Cell[BoxData[
    \(square[x_] := x\^2\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],

Cell["\<\
After entering a function definition, you call the function in the same way \
that a built-in function is applied to an argument.\
\>", "Text",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(square[5]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],

Cell[BoxData[
    \(25\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:14.0", "IPM3.4.4.1"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Building up programs", "Subsection",
  CellTags->{"b:14.1", "IPM3.4.4.2", "c:14"}],

Cell[TextData[{
  "The ability to use the output of one function as the input of another is \
one of the keys to functional programming. A mathematician would call this \
\[OpenCurlyDoubleQuote]composition of functions.\[CloseCurlyDoubleQuote] In \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", this sequential application of several functions is known as a ",
  StyleBox["nested function call", "TermDefinition"],
  ". Nested function calls are not limited to using a single function \
repeatedly, such as with the built-in ",
  StyleBox["Nest", "MR"],
  " and ",
  StyleBox["Fold", "MR"],
  " functions."
}], "TextFirst",
  CellTags->{"i:24", "b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Cos[Sin[Tan[4.0]]]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(0.6090525263647699`\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["\<\
To see the above computation more clearly, we can step through the \
computation.\
\>", "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Tan[4.0]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(1.1578212823495775`\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Sin[%]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(0.9159308854343666`\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Cos[%]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(0.6090525263647699`\)], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "Wrapping the ",
  StyleBox["Trace", "MR"],
  " function around the computation lets us see all of the intermediate \
expressions that are used in this evaluation."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Trace[Cos[Sin[Tan[4.0]]]]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                TagBox[\(Tan[4.`]\),
                  HoldForm], ",", 
                TagBox["1.1578212823495775`",
                  HoldForm]}], "}"}], ",", 
            TagBox[\(Sin[1.1578212823495775`]\),
              HoldForm], ",", 
            TagBox["0.9159308854343666`",
              HoldForm]}], "}"}], ",", 
        TagBox[\(Cos[0.9159308854343666`]\),
          HoldForm], ",", 
        TagBox["0.6090525263647699`",
          HoldForm]}], "}"}]], "Output",
  CellLabel->"Out[7]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["\<\
You can read nested functions in much the same way that they are created, \
starting with the innermost functions and working towards the outermost \
functions. For example, the following expression determines whether all of \
the elements in a list are even numbers.\
\>", "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[And, Map[EvenQ, {2, 4, 6, 7, 8}]]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(False\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "Let us step through the computation much the same as ",
  StyleBox["Mathematica ",
    FontSlant->"Italic"],
  "does, from the inside out."
}], "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Map the predicate ",
  StyleBox["EvenQ", "MR"],
  " to every element in the list ",
  StyleBox["{2,4,6,7,8}", "MR"],
  "."
}], "ItemizedText",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[EvenQ, {2, 4, 6, 7, 8}]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({True, True, True, False, True}\)], "Output",
  CellLabel->"Out[9]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Apply the logical function ",
  StyleBox["And", "MR"],
  " to the result of the previous step."
}], "ItemizedText",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[And, %]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(False\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["\<\
Finally, here is a definition that can be used on arbitrary lists.\
\>", "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(setEvenQ[lis_] := Apply[And, Map[EvenQ, lis]]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(setEvenQ[{11, 5, 1, 18, 16, 6, 17, 6}]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(False\)], "Output",
  CellLabel->"Out[12]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["\<\
Another, more complicated, example returns the elements in a list of positive \
numbers that are bigger than all of the preceding numbers in the list.\
\>", "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Union[Rest[FoldList[Max, 0, {3, 1, 6, 5, 4, 8, 7}]]]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({3, 6, 8}\)], "Output",
  CellLabel->"Out[13]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "The ",
  StyleBox["Trace", "MR"],
  " of the function call shows the intermediate steps of the computation."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Trace[Union[Rest[FoldList[Max, 0, {3, 1, 6, 5, 4, 8, 7}]]]]\)], "Input",\

  CellLabel->"In[14]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        RowBox[{"{", 
          RowBox[{
            RowBox[{"{", 
              RowBox[{
                TagBox[\(FoldList[Max, 0, {3, 1, 6, 5, 4, 8, 7}]\),
                  HoldForm], ",", 
                RowBox[{"{", 
                  RowBox[{
                    TagBox[\(Max[0, 3]\),
                      HoldForm], ",", 
                    TagBox["3",
                      HoldForm]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    TagBox[\(Max[3, 1]\),
                      HoldForm], ",", 
                    TagBox[\(Max[1, 3]\),
                      HoldForm], ",", 
                    TagBox["3",
                      HoldForm]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    TagBox[\(Max[3, 6]\),
                      HoldForm], ",", 
                    TagBox["6",
                      HoldForm]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    TagBox[\(Max[6, 5]\),
                      HoldForm], ",", 
                    TagBox[\(Max[5, 6]\),
                      HoldForm], ",", 
                    TagBox["6",
                      HoldForm]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    TagBox[\(Max[6, 4]\),
                      HoldForm], ",", 
                    TagBox[\(Max[4, 6]\),
                      HoldForm], ",", 
                    TagBox["6",
                      HoldForm]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    TagBox[\(Max[6, 8]\),
                      HoldForm], ",", 
                    TagBox["8",
                      HoldForm]}], "}"}], ",", 
                RowBox[{"{", 
                  RowBox[{
                    TagBox[\(Max[8, 7]\),
                      HoldForm], ",", 
                    TagBox[\(Max[7, 8]\),
                      HoldForm], ",", 
                    TagBox["8",
                      HoldForm]}], "}"}], ",", 
                TagBox[\({0, 3, 3, 6, 6, 6, 8, 8}\),
                  HoldForm]}], "}"}], ",", 
            TagBox[\(Rest[{0, 3, 3, 6, 6, 6, 8, 8}]\),
              HoldForm], ",", 
            TagBox[\({3, 3, 6, 6, 6, 8, 8}\),
              HoldForm]}], "}"}], ",", 
        TagBox[\(Union[{3, 3, 6, 6, 6, 8, 8}]\),
          HoldForm], ",", 
        TagBox[\({3, 6, 8}\),
          HoldForm]}], "}"}]], "Output",
  CellLabel->"Out[14]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["This computation can be described as follows:", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "The ",
  StyleBox["FoldList", "MR"],
  " function is first applied to the function ",
  StyleBox["Max", "MR"],
  ", ",
  StyleBox["0", "MR"],
  ", and the list ",
  StyleBox["{3,1,6,5,4,8,7}", "MR"],
  " (look at the ",
  StyleBox["Trace", "MR"],
  " of this computation to see what ",
  StyleBox["FoldList", "MR"],
  " is doing here)."
}], "BulletedList",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(FoldList[Max, 0, {3, 1, 6, 5, 4, 8, 7}]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({0, 3, 3, 6, 6, 6, 8, 8}\)], "Output",
  CellLabel->"Out[15]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "The ",
  StyleBox["Rest", "MR"],
  " function is then applied to the result of the previous step to remove the \
first element of the list."
}], "BulletedList",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Rest[%]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({3, 3, 6, 6, 6, 8, 8}\)], "Output",
  CellLabel->"Out[16]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "Finally, the ",
  StyleBox["Union", "MR"],
  " function is applied to the result of the previous step to remove \
duplicates."
}], "BulletedList",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Union[%]\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({3, 6, 8}\)], "Output",
  CellLabel->"Out[17]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["Here is the function definition.", "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(maxima[x_] := Union[Rest[FoldList[Max, 0, x]]]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->{"i:25", "b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "Applying ",
  StyleBox["maxima", "MR"],
  " to a list of numbers produces a list of all those numbers that are larger \
than any number that comes before it."
}], "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(maxima[{4, 2, 7, 3, 4, 9, 14, 11, 17}]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({4, 7, 9, 14, 17}\)], "Output",
  CellLabel->"Out[19]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["\<\
Notice that in each of the nested functions described here, the argument of \
the first function was explicitly referred to, but the expressions that were \
manipulated in the succeeding function calls were not identified other than \
as the results of the previous steps (that is, as the results of the \
preceding function applications).\
\>", "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "Here is an interesting application of building up a program with nested \
functions \[Dash] the creation of a deck of cards. (",
  StyleBox["Hint",
    FontSlant->"Italic"],
  ": The suit icons are entered by typing in ",
  StyleBox["\\[ClubSuit", "MR"],
  "\[VeryThinSpace]",
  StyleBox["]", "MR"],
  ", ",
  StyleBox["\\[DiamondSuit", "MR"],
  "\[VeryThinSpace]",
  StyleBox["]", "MR"],
  ", etc.)"
}], "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(cardDeck = 
      Flatten[Outer[
          List, {\[ClubSuit], \[DiamondSuit], \[HeartSuit], \[SpadeSuit]}, 
          Join[Range[2, 10], {J, Q, K, A}]], 1]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({{\[ClubSuit], 2}, {\[ClubSuit], 3}, {\[ClubSuit], 4}, {\[ClubSuit], 
        5}, {\[ClubSuit], 6}, {\[ClubSuit], 7}, {\[ClubSuit], 
        8}, {\[ClubSuit], 9}, {\[ClubSuit], 10}, {\[ClubSuit], 
        J}, {\[ClubSuit], Q}, {\[ClubSuit], K}, {\[ClubSuit], 
        A}, {\[DiamondSuit], 2}, {\[DiamondSuit], 3}, {\[DiamondSuit], 
        4}, {\[DiamondSuit], 5}, {\[DiamondSuit], 6}, {\[DiamondSuit], 
        7}, {\[DiamondSuit], 8}, {\[DiamondSuit], 9}, {\[DiamondSuit], 
        10}, {\[DiamondSuit], J}, {\[DiamondSuit], Q}, {\[DiamondSuit], 
        K}, {\[DiamondSuit], A}, {\[HeartSuit], 2}, {\[HeartSuit], 
        3}, {\[HeartSuit], 4}, {\[HeartSuit], 5}, {\[HeartSuit], 
        6}, {\[HeartSuit], 7}, {\[HeartSuit], 8}, {\[HeartSuit], 
        9}, {\[HeartSuit], 10}, {\[HeartSuit], J}, {\[HeartSuit], 
        Q}, {\[HeartSuit], K}, {\[HeartSuit], A}, {\[SpadeSuit], 
        2}, {\[SpadeSuit], 3}, {\[SpadeSuit], 4}, {\[SpadeSuit], 
        5}, {\[SpadeSuit], 6}, {\[SpadeSuit], 7}, {\[SpadeSuit], 
        8}, {\[SpadeSuit], 9}, {\[SpadeSuit], 10}, {\[SpadeSuit], 
        J}, {\[SpadeSuit], Q}, {\[SpadeSuit], K}, {\[SpadeSuit], 
        A}}\)], "Output",
  CellLabel->"Out[20]=",
  FontSize->8,
  CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "You might think of ",
  StyleBox["cardDeck", "MR"],
  " as a name for the expression given on the right-hand side of the \
immediate definition, or you might think of ",
  StyleBox["cardDeck", "MR"],
  " as defining a function with zero arguments."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "To understand what is going on here, we will build up this program from \
scratch. First we form a list of the number and face cards in a suit by \
combining a list of the numbers 2 through 10, ",
  StyleBox["Range[2,10]", "MR"],
  ", with a four-element list representing the jack, queen, king, and ace, ",
  StyleBox["{J,Q,K,A}", "MR"],
  "."
}], "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Join[Range[2, 10], {J, Q, K, A}]\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({2, 3, 4, 5, 6, 7, 8, 9, 10, J, Q, K, A}\)], "Output",
  CellLabel->"Out[21]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "Now we pair each of the 13 elements in this list with each of the four \
elements in the list representing the card suits ",
  StyleBox["{\[ClubSuit],\[DiamondSuit],\[HeartSuit],\[SpadeSuit]}", "MR"],
  ". This produces a list of 52 ordered pairs representing the cards in a \
deck, where the king of clubs, for example, is represented by ",
  StyleBox["{\[ClubSuit],K}", "MR"],
  "). "
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Outer[
      List, {\[ClubSuit], \[DiamondSuit], \[HeartSuit], \[SpadeSuit]}, %]\)], \
"Input",
  CellLabel->"In[22]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({{{\[ClubSuit], 2}, {\[ClubSuit], 3}, {\[ClubSuit], 4}, {\[ClubSuit], 
          5}, {\[ClubSuit], 6}, {\[ClubSuit], 7}, {\[ClubSuit], 
          8}, {\[ClubSuit], 9}, {\[ClubSuit], 10}, {\[ClubSuit], 
          J}, {\[ClubSuit], Q}, {\[ClubSuit], K}, {\[ClubSuit], 
          A}}, {{\[DiamondSuit], 2}, {\[DiamondSuit], 3}, {\[DiamondSuit], 
          4}, {\[DiamondSuit], 5}, {\[DiamondSuit], 6}, {\[DiamondSuit], 
          7}, {\[DiamondSuit], 8}, {\[DiamondSuit], 9}, {\[DiamondSuit], 
          10}, {\[DiamondSuit], J}, {\[DiamondSuit], Q}, {\[DiamondSuit], 
          K}, {\[DiamondSuit], A}}, {{\[HeartSuit], 2}, {\[HeartSuit], 
          3}, {\[HeartSuit], 4}, {\[HeartSuit], 5}, {\[HeartSuit], 
          6}, {\[HeartSuit], 7}, {\[HeartSuit], 8}, {\[HeartSuit], 
          9}, {\[HeartSuit], 10}, {\[HeartSuit], J}, {\[HeartSuit], 
          Q}, {\[HeartSuit], K}, {\[HeartSuit], A}}, {{\[SpadeSuit], 
          2}, {\[SpadeSuit], 3}, {\[SpadeSuit], 4}, {\[SpadeSuit], 
          5}, {\[SpadeSuit], 6}, {\[SpadeSuit], 7}, {\[SpadeSuit], 
          8}, {\[SpadeSuit], 9}, {\[SpadeSuit], 10}, {\[SpadeSuit], 
          J}, {\[SpadeSuit], Q}, {\[SpadeSuit], K}, {\[SpadeSuit], 
          A}}}\)], "Output",
  CellLabel->"Out[22]=",
  FontSize->8,
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["\<\
While we now have all of the cards in the deck, they are grouped by suit in a \
nested list. We therefore un-nest the list:\
\>", "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Flatten[%, 1]\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({{\[ClubSuit], 2}, {\[ClubSuit], 3}, {\[ClubSuit], 4}, {\[ClubSuit], 
        5}, {\[ClubSuit], 6}, {\[ClubSuit], 7}, {\[ClubSuit], 
        8}, {\[ClubSuit], 9}, {\[ClubSuit], 10}, {\[ClubSuit], 
        J}, {\[ClubSuit], Q}, {\[ClubSuit], K}, {\[ClubSuit], 
        A}, {\[DiamondSuit], 2}, {\[DiamondSuit], 3}, {\[DiamondSuit], 
        4}, {\[DiamondSuit], 5}, {\[DiamondSuit], 6}, {\[DiamondSuit], 
        7}, {\[DiamondSuit], 8}, {\[DiamondSuit], 9}, {\[DiamondSuit], 
        10}, {\[DiamondSuit], J}, {\[DiamondSuit], Q}, {\[DiamondSuit], 
        K}, {\[DiamondSuit], A}, {\[HeartSuit], 2}, {\[HeartSuit], 
        3}, {\[HeartSuit], 4}, {\[HeartSuit], 5}, {\[HeartSuit], 
        6}, {\[HeartSuit], 7}, {\[HeartSuit], 8}, {\[HeartSuit], 
        9}, {\[HeartSuit], 10}, {\[HeartSuit], J}, {\[HeartSuit], 
        Q}, {\[HeartSuit], K}, {\[HeartSuit], A}, {\[SpadeSuit], 
        2}, {\[SpadeSuit], 3}, {\[SpadeSuit], 4}, {\[SpadeSuit], 
        5}, {\[SpadeSuit], 6}, {\[SpadeSuit], 7}, {\[SpadeSuit], 
        8}, {\[SpadeSuit], 9}, {\[SpadeSuit], 10}, {\[SpadeSuit], 
        J}, {\[SpadeSuit], Q}, {\[SpadeSuit], K}, {\[SpadeSuit], 
        A}}\)], "Output",
  CellLabel->"Out[23]=",
  FontSize->8,
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["Voila! ", "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "The step-by-step construction that we used here, applying one function at \
a time, checking each function call separately, is a very efficient way to ",
  StyleBox["prototype", "TermDefinition"],
  " your programs in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". We will use this technique again in the next example."
}], "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "We will perform what is called a ",
  StyleBox["perfect shuffle",
    FontSlant->"Italic"],
  ", consisting of cutting the deck in half and then interleaving the cards \
from the two halves. Rather than working with the large list of 52 ordered \
pairs during the prototyping, we will use a short made-up list. A short list \
of an even number of ordered integers is a good choice for the task."
}], "Text",
  CellTags->{"i:27", "b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(d = Range[6]\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({1, 2, 3, 4, 5, 6}\)], "Output",
  CellLabel->"Out[24]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["We first divide the list into two equal-sized lists.", "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Partition[d, Length[d]/2]\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({{1, 2, 3}, {4, 5, 6}}\)], "Output",
  CellLabel->"Out[25]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "We now want to interleave these two lists to form ",
  StyleBox["{1,4,2,5,3,6}", "MR"],
  ". The first step is to pair the corresponding elements in each of the two \
lists above. This can be done using the ",
  StyleBox["Transpose", "MR"],
  " function."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Transpose[%]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({{1, 4}, {2, 5}, {3, 6}}\)], "Output",
  CellLabel->"Out[26]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "We now un-nest the interior lists using the ",
  StyleBox["Flatten", "MR"],
  " function. We could flatten our simple list using ",
  StyleBox["Flatten[\[Ellipsis]]", "MR"],
  ", but, since we know that ultimately we will be dealing with ordered pairs \
rather than integers, we will use ",
  StyleBox["Flatten[\[Ellipsis],1]", "MR"],
  " as we did in creating the card deck."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Flatten[%, 1]\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({1, 4, 2, 5, 3, 6}\)], "Output",
  CellLabel->"Out[27]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["\<\
That does the job. Given this prototype, it is easy to write the actual \
function to perform a perfect shuffle on a deck of cards. Notice we have \
generalized this shuffle to lists of arbitrary length.\
\>", "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(shuffle[lis_] := 
      Flatten[Transpose[Partition[lis, Length[lis]/2]], 1]\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->{"i:28", "b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(shuffle[cardDeck]\)], "Input",
  CellLabel->"In[29]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({{\[ClubSuit], 2}, {\[HeartSuit], 2}, {\[ClubSuit], 3}, {\[HeartSuit], 
        3}, {\[ClubSuit], 4}, {\[HeartSuit], 4}, {\[ClubSuit], 
        5}, {\[HeartSuit], 5}, {\[ClubSuit], 6}, {\[HeartSuit], 
        6}, {\[ClubSuit], 7}, {\[HeartSuit], 7}, {\[ClubSuit], 
        8}, {\[HeartSuit], 8}, {\[ClubSuit], 9}, {\[HeartSuit], 
        9}, {\[ClubSuit], 10}, {\[HeartSuit], 10}, {\[ClubSuit], 
        J}, {\[HeartSuit], J}, {\[ClubSuit], Q}, {\[HeartSuit], 
        Q}, {\[ClubSuit], K}, {\[HeartSuit], K}, {\[ClubSuit], 
        A}, {\[HeartSuit], A}, {\[DiamondSuit], 2}, {\[SpadeSuit], 
        2}, {\[DiamondSuit], 3}, {\[SpadeSuit], 3}, {\[DiamondSuit], 
        4}, {\[SpadeSuit], 4}, {\[DiamondSuit], 5}, {\[SpadeSuit], 
        5}, {\[DiamondSuit], 6}, {\[SpadeSuit], 6}, {\[DiamondSuit], 
        7}, {\[SpadeSuit], 7}, {\[DiamondSuit], 8}, {\[SpadeSuit], 
        8}, {\[DiamondSuit], 9}, {\[SpadeSuit], 9}, {\[DiamondSuit], 
        10}, {\[SpadeSuit], 10}, {\[DiamondSuit], J}, {\[SpadeSuit], 
        J}, {\[DiamondSuit], Q}, {\[SpadeSuit], Q}, {\[DiamondSuit], 
        K}, {\[SpadeSuit], K}, {\[DiamondSuit], A}, {\[SpadeSuit], 
        A}}\)], "Output",
  CellLabel->"Out[29]=",
  FontSize->8,
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["\<\
Let us take this example one step further and construct a function that deals \
cards from a card deck. We will construct this function in stages using the \
prototyping method we showed earlier.\
\>", "Text",
  CellTags->{"i:30", "b:14.1", "IPM3.4.4.2"}],

Cell["\<\
First we need to define a function that removes a single element from a \
randomly chosen position in a list.\
\>", "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(removeRand[lis_] := 
      Delete[lis, Random[Integer, {1, Length[lis]}]]\)], "Input",
  CellLabel->"In[30]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "The function ",
  StyleBox["removeRand", "MR"],
  " first uses the ",
  StyleBox["Random", "MR"],
  " function to randomly choose an integer ",
  Cell[BoxData[
      \(TraditionalForm\`k\)],
    FormatType->StandardForm],
  " between 1 and the length of the list, and then uses the ",
  StyleBox["Delete", "MR"],
  " function to remove the ",
  Cell[BoxData[
      \(TraditionalForm\`k\)],
    FormatType->StandardForm],
  "th element of the list. For example, if a list has 10 elements, an integer \
between 1 and 10, say 6, is randomly determined and the element in the sixth \
position in the list is then removed from the list."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(lis = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10};\)\), "\[IndentingNewLine]", 
    \(removeRand[lis]\)}], "Input",
  CellLabel->"In[31]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({2, 3, 4, 5, 6, 7, 8, 9, 10}\)], "Output",
  CellLabel->"Out[32]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[TextData[{
  "Now we want to make a function call that applies the ",
  StyleBox["removeRand", "MR"],
  " function to the ",
  StyleBox["cardDeck", "MR"],
  " list, then applies the ",
  StyleBox["removeRand", "MR"],
  " function to the resulting list, then applies the ",
  StyleBox["removeRand", "MR"],
  " function to the resulting list, and so on, a total of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " times. The way to carry out this operation is with the ",
  StyleBox["Nest", "MR"],
  " function."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell["Nest[removeRand, cardDeck, n]", "Program",
  Evaluatable->False,
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "Lastly, we want the cards that are removed from ",
  StyleBox["cardDeck", "MR"],
  " rather than those that remain."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell["Complement[cardDeck, Nest[removeRand, cardDeck, n]]", "Program",
  Evaluatable->False,
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[TextData[{
  "Now, we write this up formally into the user-defined ",
  StyleBox["deal", "MR"],
  " function."
}], "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \(deal[n_] := 
      Complement[cardDeck, Nest[removeRand, cardDeck, n]]\)], "Input",
  CellLabel->"In[33]:=",
  CellTags->{"i:29", "b:14.1", "IPM3.4.4.2"}],

Cell["Let us try it out.", "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(deal[5]\)], "Input",
  CellLabel->"In[34]:=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[BoxData[
    \({{\[ClubSuit], 3}, {\[ClubSuit], K}, {\[DiamondSuit], 
        2}, {\[HeartSuit], K}, {\[SpadeSuit], J}}\)], "Output",
  CellLabel->"Out[34]=",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell["Not a bad hand!", "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:14.1", "IPM3.4.4.3", "c:15"}],

Cell[CellGroupData[{

Cell[TextData[{
  "One of the games in the Illinois State Lottery is based on choosing ",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  " numbers, each between 0 and 9, with duplicates allowed; in practice, a \
selection is made from containers of numbered ping pong balls. We can model \
this game using a simple user-defined function, which we will call ",
  StyleBox["pick", "MR"],
  " (after the official lottery names of ",
  StyleBox["Pick 3",
    FontSlant->"Italic"],
  " and ",
  StyleBox["Pick 4",
    FontSlant->"Italic"],
  ")."
}], "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[BoxData[
    \(pick[n_] := Table[Random[Integer, {0, 9}], {n}]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(pick[4]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[BoxData[
    \({0, 9, 0, 4}\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:14.1", "IPM3.4.4.3"}]
}, Closed]],

Cell[TextData[{
  "This program can be generalized to perform ",
  StyleBox["random sampling with replacement",
    FontSlant->"Italic"],
  " on any list. Write a function ",
  StyleBox["chooseWithReplacement[", "MR"],
  Cell[BoxData[
      FormBox[
        StyleBox["lis",
          FontSlant->"Italic"], TraditionalForm]],
    FormatType->StandardForm],
  StyleBox[",", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  StyleBox["]", "MR"],
  ", where ",
  Cell[BoxData[
      FormBox[
        StyleBox["lis",
          FontSlant->"Italic"], TraditionalForm]],
    FormatType->StandardForm],
  " is the list, ",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  " is the number of elements being chosen and the following is a typical \
result."
}], "ExercisesNoLabel",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(chooseWithReplacement[{a, b, c, d, e, f, g, h}, 3]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[BoxData[
    \({h, b, f}\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:14.1", "IPM3.4.4.3"}]
}, Closed]]
}, Closed]],

Cell[TextData[{
  "Write your own user-defined functions using the ",
  StyleBox["ToCharacterCode", "MR"],
  " and ",
  StyleBox["FromCharacterCode", "MR"],
  " functions to perform the same operations as ",
  StyleBox["StringInsert", "MR"],
  " and ",
  StyleBox["StringDrop", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[TextData[{
  "Create a function ",
  StyleBox["distance[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`a\)],
    FormatType->StandardForm],
  StyleBox[",", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`b\)],
    FormatType->StandardForm],
  StyleBox["]", "MR"],
  " that finds the distance between two points ",
  Cell[BoxData[
      \(TraditionalForm\`a\)],
    FormatType->StandardForm],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)],
    FormatType->StandardForm],
  " in the plane. "
}], "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3", "i:57"}],

Cell[TextData[{
  "Write a user-defined function ",
  StyleBox["interleave2", "MR"],
  " that interleaves the elements of two lists of unequal length. (You have \
already seen how to interleave lists of equal length using ",
  StyleBox["Partition", "MR"],
  " earlier in this section.) Your function should take the lists ",
  StyleBox["{1,2,3}", "MR"],
  " and ",
  StyleBox["{a,b,c,d}", "MR"],
  " as inputs and return ",
  StyleBox["{1,a,2,b,3,c,d}", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell["\<\
Write a nested function call that creates a deck of cards and performs a \
perfect shuffle on it.\
\>", "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[TextData[{
  "Write nested function calls using the ",
  StyleBox["ToCharacterCode", "MR"],
  " and ",
  StyleBox["FromCharacterCode", "MR"],
  " functions to perform the same operations as the built-in ",
  StyleBox["StringJoin", "MR"],
  " and ",
  StyleBox["StringReverse", "MR"],
  " functions."
}], "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Auxiliary functions", "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
  CellTags->{
  "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
    "IPM3.4.5.5", "c:16"}],

Cell[TextData[{
  "There are several major drawbacks to the ",
  StyleBox["deal", "MR"],
  " function created in the previous section. In order to use ",
  StyleBox["deal", "MR"],
  ", the definition of ",
  StyleBox["removeRand", "MR"],
  " and the value of ",
  StyleBox["cardDeck", "MR"],
  " must be entered before calling ",
  StyleBox["deal", "MR"],
  ". It would be much more convenient if we could incorporate these functions \
within the ",
  StyleBox["deal", "MR"],
  " function definition itself. In the next section, we will show how this \
can be done."
}], "TextFirst",
  CellTags->"IPM3.4.5.0"],

Cell[CellGroupData[{

Cell["Compound functions", "Subsection",
  CellTags->{"b:15.0", "IPM3.4.5.1", "c:17"}],

Cell[TextData[{
  "The left-hand side of a ",
  StyleBox["compound function", "TermDefinition"],
  " is the same as that of any user-defined function. The right-hand side \
consists of consecutive expressions enclosed in parentheses and separated by \
semicolons."
}], "TextFirst",
  CellTags->{"i:31", "b:15.0", "IPM3.4.5.1"}],

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["name",
          "TI"], "[", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["arg",
                  "TI"], "1"],
              "TraditionalForm"], "_"}], ",", 
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["arg",
                  "TI"], "2"],
              "TraditionalForm"], "_"}], ",", "\[Ellipsis]", ",", 
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["arg",
                  "TI"], 
                StyleBox["n",
                  "TI"]],
              "TraditionalForm"], "_"}]}], "]"}], ":=", 
      RowBox[{"(", 
        RowBox[{
          FormBox[
            SubscriptBox[
              StyleBox["expr",
                "TI"], "1"],
            "TraditionalForm"], ";", 
          FormBox[
            SubscriptBox[
              StyleBox["expr",
                "TI"], "2"],
            "TraditionalForm"], ";", "\[Ellipsis]", ";", 
          FormBox[
            SubscriptBox[
              StyleBox["expr",
                "TI"], 
              StyleBox["m",
                "TI"]],
            "TraditionalForm"]}], ")"}]}]], "Program",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[TextData[{
  "The expressions can be user-defined functions (also known as ",
  StyleBox["auxiliary", "TermDefinition"],
  " functions), value declarations, and function calls. When a compound \
function is evaluated with particular argument values, these expressions are \
evaluated in order and the result of the evaluation of the last expression is \
returned (by adding a semicolon after ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["expr",
            FontSlant->"Italic"], "n"], TraditionalForm]]],
  ", the display of the final evaluation result can also be suppressed)."
}], "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[TextData[{
  "We will work with the ",
  StyleBox["deal", "MR"],
  " function to illustrate how a compound function is created. We need the \
following three expressions."
}], "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[BoxData[
    \(\(cardDeck = 
        Flatten[Outer[
            List, {\[ClubSuit], \[DiamondSuit], \[HeartSuit], \[SpadeSuit]}, 
            Join[Range[2, 10], {J, Q, K, A}]], 1];\)\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[BoxData[
    \(removeRand[lis_] := 
      Delete[lis, Random[Integer, {1, Length[lis]}]]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[BoxData[
    \(deal[n_] := 
      Complement[cardDeck, Nest[removeRand, cardDeck, n]]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell["\<\
The conversion to a compound function is easily done. We will first remove \
the old definitions.\
\>", "TextFirst",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[BoxData[
    \(Clear[deal, cardDeck, removeRand]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell["Now we can create and enter the new definition.", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[BoxData[
    \(deal[
        n_] := \((\[IndentingNewLine]cardDeck = 
          Flatten[Outer[
              List, {\[ClubSuit], \[DiamondSuit], \[HeartSuit], \
\[SpadeSuit]}, Join[Range[2, 10], {J, Q, K, A}]], 
            1]; \[IndentingNewLine]removeRand[lis_] := 
          Delete[lis, Random[Integer, {1, Length[lis]}]]; 
        Complement[cardDeck, 
          Nest[removeRand, cardDeck, n]]\[IndentingNewLine])\)\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"i:32", "b:15.0", "IPM3.4.5.1"}],

Cell["Let us check that this works.", "TextFirst",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(deal[5]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[BoxData[
    \({{\[ClubSuit], 3}, {\[DiamondSuit], 2}, {\[HeartSuit], 
        3}, {\[HeartSuit], 4}, {\[HeartSuit], Q}}\)], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:15.0", "IPM3.4.5.1"}]
}, Closed]],

Cell[TextData[{
  "A couple of things should be pointed out about the right-hand side of a \
compound function definition. Since the expressions on the right-hand side \
are evaluated in order, value declarations and auxiliary function definitions \
should be given ",
  StyleBox["before",
    FontSlant->"Italic"],
  " they are used and the argument names used on the left-hand side of \
auxiliary function definitions ",
  StyleBox["must",
    FontSlant->"Italic"],
  " differ from the argument names used by the compound function itself."
}], "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[TextData[{
  "Finally, when we enter a compound function definition, we are entering not \
only the function but also the auxiliary functions and the value \
declarations. If we then remove the function definition using ",
  StyleBox["Clear", "MR"],
  ", the auxiliary function definitions and value declarations remain. This \
can cause a problem if we subsequently try to use the names of these \
auxiliary functions and values elsewhere."
}], "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell["\<\
So how does the global rule base treat compound functions? When a compound \
function definition is entered, a rewrite rule corresponding to the entire \
definition is created. Each time the compound function is subsequently \
called, rewrite rules are created from the auxiliary function definitions and \
value declarations within the compound function.\
\>", "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?cardDeck\)\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell["Global`cardDeck", "Print",
  CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(cardDeck = {{\[ClubSuit], 2}, {\[ClubSuit], 
                        3}, {\[ClubSuit], 4}, {\[ClubSuit], 5}, {\[ClubSuit], 
                        6}, {\[ClubSuit], 7}, {\[ClubSuit], 8}, {\[ClubSuit], 
                        9}, {\[ClubSuit], 10}, {\[ClubSuit], 
                        J}, {\[ClubSuit], Q}, {\[ClubSuit], K}, {\[ClubSuit], 
                        A}, {\[DiamondSuit], 2}, {\[DiamondSuit], 
                        3}, {\[DiamondSuit], 4}, {\[DiamondSuit], 
                        5}, {\[DiamondSuit], 6}, {\[DiamondSuit], 
                        7}, {\[DiamondSuit], 8}, {\[DiamondSuit], 
                        9}, {\[DiamondSuit], 10}, {\[DiamondSuit], 
                        J}, {\[DiamondSuit], Q}, {\[DiamondSuit], 
                        K}, {\[DiamondSuit], A}, {\[HeartSuit], 
                        2}, {\[HeartSuit], 3}, {\[HeartSuit], 
                        4}, {\[HeartSuit], 5}, {\[HeartSuit], 
                        6}, {\[HeartSuit], 7}, {\[HeartSuit], 
                        8}, {\[HeartSuit], 9}, {\[HeartSuit], 
                        10}, {\[HeartSuit], J}, {\[HeartSuit], 
                        Q}, {\[HeartSuit], K}, {\[HeartSuit], 
                        A}, {\[SpadeSuit], 2}, {\[SpadeSuit], 
                        3}, {\[SpadeSuit], 4}, {\[SpadeSuit], 
                        5}, {\[SpadeSuit], 6}, {\[SpadeSuit], 
                        7}, {\[SpadeSuit], 8}, {\[SpadeSuit], 
                        9}, {\[SpadeSuit], 10}, {\[SpadeSuit], 
                        J}, {\[SpadeSuit], Q}, {\[SpadeSuit], 
                        K}, {\[SpadeSuit], A}}\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "cardDeck"],
      Editable->False]], "Print",
  FontSize->8,
  CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}]
}, Closed]],

Cell["\<\
It is considered bad programming practice to leave auxiliary definitions in \
the global rule base that are not explicitly needed by the user of your \
function. In fact, it could interfere with a user\[CloseCurlyQuote]s \
workspace and cause unintended problems.\
\>", "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[TextData[{
  "To prevent these additional rewrite rules from being placed in the global \
rule base, you can localize their names by using the ",
  StyleBox["Module", "MR"],
  " construct in the compound function definition. This is what we discuss \
next."
}], "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Localizing names: Module", "Subsection",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2", "c:18"}],

Cell[TextData[{
  "When a user-defined function is written, it is generally a good idea to \
isolate the ",
  StyleBox["names",
    FontSlant->"Italic"],
  " of values and functions defined on the right-hand side from the outside \
world in order to avoid any conflict with the use of a name elsewhere in the \
session (for example, ",
  StyleBox["cardDeck", "MR"],
  " might be used elsewhere to represent a pinochle deck). This can be done \
by wrapping the right-hand side of the function definition in the built-in ",
  StyleBox["Module", "MR"],
  " function."
}], "TextFirst",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],

Cell[BoxData[
    RowBox[{
      RowBox[{
        StyleBox["name",
          "TI"], "[", 
        RowBox[{
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["arg",
                  "TI"], "1"],
              "TraditionalForm"], "_"}], ",", 
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["arg",
                  "TI"], "2"],
              "TraditionalForm"], "_"}], ",", "\[Ellipsis]", ",", 
          RowBox[{
            FormBox[
              SubscriptBox[
                StyleBox["arg",
                  "TI"], 
                StyleBox["n",
                  "TI"]],
              "TraditionalForm"], "_"}]}], "]"}], ":=", 
      RowBox[{"Module", "[", 
        RowBox[{
          RowBox[{"{", 
            RowBox[{
              FormBox[
                SubscriptBox[
                  StyleBox["name",
                    "TI"], "1"],
                "TraditionalForm"], ",", 
              RowBox[{
                SubscriptBox[
                  StyleBox["name",
                    "TI"], "2"], "=", 
                StyleBox["value",
                  "TI"]}], ",", "\[Ellipsis]"}], "}"}], ",", 
          "\[IndentingNewLine]", 
          StyleBox["expr",
            "TI"]}], "]"}]}]], "Program",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],

Cell[TextData[{
  "The first argument of the ",
  StyleBox["Module", "MR"],
  " function is a list of the names we want to localize. If we wish, we can \
assign values to these names, as is shown with ",
  Cell[BoxData[
      FormBox[
        SubscriptBox[
          StyleBox["name",
            FontSlant->"Italic"], "2"], TraditionalForm]]],
  " above (the assigned value is only an initial value and can be changed \
subsequently). The list is separated from the right-hand side by a comma and \
so the parentheses enclosing the right-hand side of a compound function are \
not needed."
}], "TextFirst",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],

Cell[TextData[{
  "We can demonstrate the use of ",
  StyleBox["Module", "MR"],
  " with the ",
  StyleBox["deal", "MR"],
  " function."
}], "Text",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],

Cell[BoxData[
    \(Clear[deal]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],

Cell[BoxData[
    \(deal[n_] := 
      Module[{cardDeck, removeRand}, \[IndentingNewLine]cardDeck = 
          Flatten[Outer[
              List, {\[ClubSuit], \[DiamondSuit], \[HeartSuit], \
\[SpadeSuit]}, Join[Range[2, 10], {J, Q, K, A}]], 
            1]; \[IndentingNewLine]removeRand[lis_] := 
          Delete[lis, 
            Random[Integer, {1, Length[lis]}]]; \[IndentingNewLine]Complement[
          cardDeck, Nest[removeRand, cardDeck, n]]]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],

Cell[TextData[{
  "Briefly, when ",
  StyleBox["Module", "MR"],
  " is encountered, the symbols that are being localized (",
  StyleBox["cardDeck", "MR"],
  " and ",
  StyleBox["removeRand", "MR"],
  " in the above example) are temporarily given new and unique names and all \
occurrences of those symbols in the body of the ",
  StyleBox["Module", "MR"],
  " are given those new names as well. In this way, these unique and \
temporary names, which are local to the function will not interfere with any \
functions outside of the ",
  StyleBox["Module", "MR"],
  "."
}], "Text",
  CellTags->{"b:15.1", "IPM3.4.5.2"}],

Cell[TextData[{
  "It is generally a good idea to wrap the right-hand side of all compound \
function definitions in the ",
  StyleBox["Module", "MR"],
  " function. Another way to avoid conflicts in the use of names of auxiliary \
function definitions is to use a function that can be applied without being \
given a name. Such functions are called ",
  StyleBox["pure functions", "TermDefinition"],
  ", which we discuss in Section 4.6."
}], "Text",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Localizing values: Block", "Subsection",
  CellTags->{"b:15.2", "IPM3.4.5.3", "c:19"}],

Cell[TextData[{
  "Occasionally, you will need to localize a ",
  StyleBox["value",
    FontSlant->"Italic"],
  " associated with a symbol without localizing the symbol name itself. For \
example, you may have a recursive computation that requires you to \
temporarily reset the system variable ",
  StyleBox["$RecursionLimit", "MR"],
  ". You can do this with ",
  StyleBox["Block", "MR"],
  ", thereby only localizing the ",
  StyleBox["value",
    FontSlant->"Italic"],
  " of ",
  StyleBox["$RecursionLimit", "MR"],
  " during the evaluation inside the ",
  StyleBox["Block", "MR"],
  "."
}], "TextFirst",
  CellTags->{"i:34", "b:15.2", "IPM3.4.5.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Block[{$RecursionLimit = 20}, \[IndentingNewLine]x = 
        g[x]\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],

Cell[BoxData[
    RowBox[{\($RecursionLimit::"reclim"\), \(\(:\)\(\ \)\), "\<\"Recursion \
depth of \\!\\(20\\) exceeded. \\!\\(\\*ButtonBox[\\\"More\[Ellipsis]\\\", \
ButtonStyle->\\\"RefGuideLinkText\\\", ButtonFrame->None, \
ButtonData:>\\\"$RecursionLimit::reclim\\\"]\\)\"\>"}]], "Message",
  CellLabel->"From In[10]:=",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],

Cell[BoxData[
    \(g[g[
        g[g[g[g[g[
                  g[g[g[g[
                          g[g[g[g[
                                  g[g[g[
                                        Hold[
                                        g[x]]]]]]]]]]]]]]]]]]]]\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"b:15.2", "IPM3.4.5.3"}]
}, Closed]],

Cell[TextData[{
  "Notice the global value of ",
  StyleBox["$RecursionLimit", "MR"],
  " is unchanged."
}], "TextFirst",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \($RecursionLimit\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],

Cell[BoxData[
    \(256\)], "Output",
  CellLabel->"Out[11]=",
  CellTags->{"b:15.2", "IPM3.4.5.3"}]
}, Closed]],

Cell[TextData[{
  "This construct is similar to what is done for the iterators in ",
  StyleBox["Table", "MR"],
  ", ",
  StyleBox["Do", "MR"],
  ", ",
  StyleBox["Sum", "MR"],
  ", and ",
  StyleBox["Product", "MR"],
  ". "
}], "TextFirst",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],

Cell[TextData[{
  StyleBox["Module", "MR"],
  ", on the other hand, would create an entirely new symbol, ",
  StyleBox["$RecursionLimit$", "MR"],
  StyleBox["nn", "IT"],
  " that would have nothing to do with the global variable ",
  StyleBox["$RecursionLimit", "MR"],
  ", and so ",
  StyleBox["Module", "MR"],
  " would be inappropriate for this particular task."
}], "Text",
  CellTags->{"b:15.2", "IPM3.4.5.3"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Localizing constants: With", "Subsection",
  CellTags->{"b:15.3", "IPM3.4.5.4", "c:20"}],

Cell[TextData[{
  "Another scoping construct is available when you simply need to localize \
constants. If, in the body of your function, you use a variable that is \
assigned a constant once and never changes, then ",
  StyleBox["With", "MR"],
  " is the preferred means to localize that constant."
}], "TextFirst",
  CellTags->{"i:35", "b:15.3", "IPM3.4.5.4"}],

Cell[TextData[{
  "This sets the global variable ",
  StyleBox["y", "MR"],
  " to have the value 5."
}], "Text",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[BoxData[
    \(\(y = 5;\)\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[TextData[{
  "Here is a simple function that initializes ",
  StyleBox["y", "MR"],
  " as a local constant."
}], "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[BoxData[
    \(f[x_] := 
      With[{y = x + 1}, \[IndentingNewLine]y\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[TextData[{
  "We see the global symbol is unchanged and it does not interfere with the \
local symbol ",
  StyleBox["y", "MR"],
  " inside the ",
  StyleBox["With", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(y\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[BoxData[
    \(5\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(f[2]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[BoxData[
    \(3\)], "Output",
  CellLabel->"Out[15]=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}]
}, Closed]],

Cell[TextData[{
  "Using ",
  StyleBox["With", "MR"],
  ", you can initialize local constants with the values of global symbols. \
For example:"
}], "TextFirst",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[BoxData[
    \(With[{y = y}, \[IndentingNewLine]g[x_] := 
        x + y\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[TextData[{
  "This shows that the global value for ",
  StyleBox["y", "MR"],
  " was inserted inside ",
  StyleBox["g", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?g\)\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell["Global`g", "Print",
  CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}],

Cell[BoxData[
    InterpretationBox[GridBox[{
          {GridBox[{
                {\(g[x$_] := x$ + 5\)}
                },
              GridBaseline->{Baseline, {1, 1}},
              ColumnWidths->0.999,
              ColumnAlignments->{Left}]}
          },
        GridBaseline->{Baseline, {1, 1}},
        ColumnAlignments->{Left}],
      Definition[ "g"],
      Editable->False]], "Print",
  CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}]
}, Closed]],

Cell[TextData[{
  "Resetting the global value of ",
  StyleBox["y", "MR"],
  " has no effect on the localized ",
  StyleBox["y", "MR"],
  " inside the ",
  StyleBox["With", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[BoxData[
    \(\(y = 1;\)\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(g[5]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[BoxData[
    \(10\)], "Output",
  CellLabel->"Out[19]=",
  CellTags->{"b:15.3", "IPM3.4.5.4"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:15.3", "IPM3.4.5.5", "c:21"}],

Cell[TextData[{
  "Write a compound function definition for the location of steps taken in an \
",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  "-step random walk on a square lattice. ",
  StyleBox["Hint",
    FontSlant->"Italic"],
  ": Use the definition for the step increments of the walk as an auxiliary \
function. "
}], "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[TextData[{
  "The ",
  StyleBox["PerfectSearch", "MR"],
  " function defined in Section 1.1 is impractical for checking large numbers \
because it has to check all numbers from 1 through ",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  ". If you already know the perfect numbers below 500, say, it is \
inefficient to check all numbers from 1 to 1,000 if you are only looking for \
perfect numbers in the range 500 to 1,000. Modify ",
  StyleBox["searchPerfect", "MR"],
  " so that it accepts two numbers as input and computes all perfect numbers \
between the inputs. For example, ",
  StyleBox["PerfectSearch[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`a\)],
    FormatType->StandardForm],
  StyleBox[",", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`b\)],
    FormatType->StandardForm],
  StyleBox["]", "MR"],
  " will produce a list of all perfect numbers in the range from ",
  Cell[BoxData[
      \(TraditionalForm\`a\)],
    FormatType->StandardForm],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`b\)],
    FormatType->StandardForm],
  "."
}], "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5", "i:58"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Overload the ",
  StyleBox["PerfectSearch", "MR"],
  " function to compute all ",
  StyleBox["3-perfect", "TermDefinition"],
  " numbers. A 3-perfect number is such that the sum of its divisors equals \
",
  StyleBox["three",
    FontSlant->"Italic"],
  " times the number. For example, 120 is 3-perfect since it is equal to \
three times the sum of its divisors."
}], "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, Divisors[120]]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[BoxData[
    \(360\)], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:15.3", "IPM3.4.5.5"}]
}, Closed]],

Cell["Find the only other 3-perfect number under 1,000.", "ExercisesNoLabel",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[TextData[{
  "You can overload ",
  StyleBox["PerfectSearch", "MR"],
  " as defined in Exercise 2 above by defining a three-argument version ",
  StyleBox["PerfectSearch[", "MR"],
  StyleBox["a", "IT"],
  StyleBox[",", "MR"],
  StyleBox["b", "IT"],
  StyleBox[",3]", "MR"],
  "."
}], "ExercisesNoLabel",
  CellTags->{"b:15.3", "IPM3.4.5.5"}]
}, Closed]],

Cell[TextData[{
  "Overload ",
  StyleBox["PerfectSearch", "MR"],
  " to find the three 4-perfect numbers less than 2,200,000."
}], "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[TextData[{
  "Redefine ",
  StyleBox["PerfectSearch", "MR"],
  " so that it accepts as input a number ",
  Cell[BoxData[
      \(TraditionalForm\`k\)],
    FormatType->StandardForm],
  ", and two numbers ",
  Cell[BoxData[
      \(TraditionalForm\`a\)],
    FormatType->StandardForm],
  " and ",
  Cell[BoxData[
      \(TraditionalForm\`b\)],
    FormatType->StandardForm],
  ", and computes all ",
  Cell[BoxData[
      \(TraditionalForm\`k\)],
    FormatType->StandardForm],
  "-perfect numbers in the range from ",
  Cell[BoxData[
      \(TraditionalForm\`a\)],
    FormatType->StandardForm],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`b\)],
    FormatType->StandardForm],
  ". For example, ",
  StyleBox["PerfectSearch[1,30,2]", "MR"],
  " would compute all 2-perfect numbers in the range from 1 to 30 and, hence, \
would output ",
  StyleBox["{6,28}", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[TextData[{
  "If ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma](n)\)],
    FormatType->StandardForm],
  " is defined to be the sum of the divisors of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  ", then ",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  " is called ",
  StyleBox["superperfect", "TermDefinition"],
  " if ",
  Cell[BoxData[
      \(TraditionalForm\`\[Sigma](\[Sigma](n)) = 2  n\)],
    FormatType->StandardForm],
  ". Write a function ",
  StyleBox["SuperPerfectSearch[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`a\)],
    FormatType->StandardForm],
  StyleBox[",", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`b\)],
    FormatType->StandardForm],
  StyleBox["]", "MR"],
  " that finds all superperfect numbers in the range from ",
  Cell[BoxData[
      \(TraditionalForm\`a\)],
    FormatType->StandardForm],
  " to ",
  Cell[BoxData[
      \(TraditionalForm\`b\)],
    FormatType->StandardForm],
  "."
}], "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Often in processing files you will be presented with expressions that need \
to be converted into a format that can be more easily manipulated inside ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". For example, a file may contain dates in the form 20030515 to represent \
May 15 2003.",
  " ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " represents its dates as a list ",
  StyleBox["{", "MR"],
  StyleBox["year", "IT"],
  StyleBox[",", "MR"],
  StyleBox["month", "IT"],
  StyleBox[",", "MR"],
  StyleBox["day", "IT"],
  StyleBox[",", "MR"],
  StyleBox["hour", "IT"],
  StyleBox[",", "MR"],
  StyleBox["minutes", "IT"],
  StyleBox[",", "MR"],
  StyleBox["seconds", "IT"],
  StyleBox["}", "MR"],
  ". Write a function ",
  StyleBox["convertToDate[", "MR"],
  StyleBox["n", "IT"],
  StyleBox["]", "MR"],
  " to convert a number consisting of eight digits such as ",
  StyleBox["20030515", "MR"],
  " into a list of the form ",
  StyleBox["{2003,5,15}", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"i:53", "b:15.3", "IPM3.4.5.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(convertToDate[20030515]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[BoxData[
    \({2003, 5, 15}\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:15.3", "IPM3.4.5.5"}]
}, Closed]]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pure functions", "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 5}},
  CellTags->{"b:16", "IPM3.4.6.1", "IPM3.4.6.2", "c:22"}],

Cell[TextData[{
  "A ",
  StyleBox["pure function", "TermDefinition"],
  " is a function that does not have a name and that can be used \
\[OpenCurlyDoubleQuote]on the spot\[CloseCurlyDoubleQuote]; that is, at the \
moment it is created. This is often convenient, especially if the function is \
only going to be used once or as an argument to a higher-order function, such \
as ",
  StyleBox["Map", "MR"],
  ", ",
  StyleBox["Fold", "MR"],
  ", or ",
  StyleBox["Nest", "MR"],
  ". The built-in function ",
  StyleBox["Function", "MR"],
  " is used to create a pure function."
}], "TextFirst",
  CellTags->{"i:36", "b:16", "IPM3.4.6.1"}],

Cell[TextData[{
  "The basic form of a pure function is ",
  StyleBox["Function[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  StyleBox[",", "MR"],
  Cell[BoxData[
      FormBox[
        StyleBox["body",
          FontSlant->"Italic"], TraditionalForm]]],
  StyleBox["]", "MR"],
  " for a pure function with a single variable ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  " (any symbol can be used for the variable), and ",
  StyleBox["Function[{", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  StyleBox[",", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  StyleBox[",\[Ellipsis]},", "MR"],
  Cell[BoxData[
      FormBox[
        StyleBox["body",
          FontSlant->"Italic"], TraditionalForm]]],
  StyleBox["]", "MR"],
  " for a pure function with more than one variable. The ",
  StyleBox["body",
    FontSlant->"Italic"],
  " looks like the right-hand side of a user-defined function definition, \
with the variables ",
  Cell[BoxData[
      \(TraditionalForm\`x\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`y\)]],
  ", \[Ellipsis], where argument names would be."
}], "Text",
  CellTags->{"i:37", "b:16", "IPM3.4.6.1"}],

Cell[TextData[{
  "As an example, the ",
  StyleBox["square", "MR"],
  " function we created earlier can be written as a pure function."
}], "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Function[z, z\^2]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(Function[z, z\^2]\)], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[TextData[{
  "There is also a standard input form that can be used in writing a pure \
function which is easier to write than the ",
  StyleBox["Function", "MR"],
  " notation but can be a bit cryptic to read. The right-hand side of the \
function definition is rewritten by replacing the variable by the pound \
symbol (",
  StyleBox["#", "MR"],
  ") and ending the expression with the ampersand symbol (",
  StyleBox["&", "MR"],
  ") to indicate that this is a pure function."
}], "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(#\^2 &\)], "Input",
  Evaluatable->False,
  CellTags->{"i:38", "b:16", "IPM3.4.6.1"}],

Cell[TextData[{
  "If there is more than one variable, ",
  StyleBox["#1", "MR"],
  ", ",
  StyleBox["#2", "MR"],
  ", and so on are used."
}], "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[TextData[{
  "A pure function can be used exactly like more conventional looking \
functions, by following the function with the argument values enclosed in \
square brackets. First we show the pure function using ",
  StyleBox["Function", "MR"],
  "."
}], "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Function[z, z\^2]\)[6]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(36\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell["\<\
Here is the same thing, but using the more cryptic shorthand notation (the \
parentheses in the following example are purely for readability and can be \
omitted if you wish).\
\>", "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\((#\^2 &)\)[6]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(36\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell["\<\
We can, if we wish, give a pure function a name and then use that name to \
call the function later. This has the same effect as defining the function in \
the more traditional manner.\
\>", "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(\(squared = \((#\^2)\) &;\)\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(squared[6]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(36\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[TextData[{
  "Pure functions are very commonly used with higher-order functions like ",
  StyleBox["Map", "MR"],
  " and ",
  StyleBox["Apply", "MR"],
  ", so before going further, let us first look at a few simple examples of \
the use of pure functions."
}], "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell["Here is a list of numbers.", "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(\(lis = {2, \(-5\), 6.1};\)\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[TextData[{
  "Now suppose we wished to square each number and then add 1 to it. The pure \
function that does this is: ",
  Cell[BoxData[
      FormBox[
        StyleBox[\(#\^2 + 1 &\),
          "MR"], TraditionalForm]]],
  ". So that is what we need to map across this list."
}], "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[#\^2 + 1 &, lis]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \({5, 26, 38.209999999999994`}\)], "Output",
  CellLabel->"Out[7]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[TextData[{
  "In the next example we will create a set of data and then use the ",
  StyleBox["Select", "MR"],
  " function to filter out outliers."
}], "Text",
  CellTags->{"b:16", "IPM3.4.6.1", "i:60"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(data = {24.39001, 29.669, 9.321, 20.8856, 23.4736, 22.1488, 24.7434, 
        22.1619, 21.1039, 24.8177, 27.1331, 25.8705, 39.7676, 
        24.7762}\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \({24.39001`, 29.669`, 9.321`, 20.8856`, 23.4736`, 22.1488`, 24.7434`, 
      22.1619`, 21.1039`, 24.8177`, 27.1331`, 25.8705`, 39.7676`, 
      24.7762`}\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell["A plot of the data shows there are two outliers.", "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(ListPlot[data, PlotStyle \[Rule] PointSize[ .02]];\)\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.0238095 0.0680272 -0.165482 0.0193323 [
[.15986 -0.00399 -4.5 -11.875 ]
[.15986 -0.00399 4.5 0 ]
[.29592 -0.00399 -4.5 -11.875 ]
[.29592 -0.00399 4.5 0 ]
[.43197 -0.00399 -4.5 -11.875 ]
[.43197 -0.00399 4.5 0 ]
[.56803 -0.00399 -4.5 -11.875 ]
[.56803 -0.00399 4.5 0 ]
[.70408 -0.00399 -7 -11.875 ]
[.70408 -0.00399 7 0 ]
[.84014 -0.00399 -7 -11.875 ]
[.84014 -0.00399 7 0 ]
[.97619 -0.00399 -7 -11.875 ]
[.97619 -0.00399 7 0 ]
[.01131 .02784 -14 -5.9375 ]
[.01131 .02784 0 5.9375 ]
[.01131 .1245 -14 -5.9375 ]
[.01131 .1245 0 5.9375 ]
[.01131 .22117 -14 -5.9375 ]
[.01131 .22117 0 5.9375 ]
[.01131 .31783 -14 -5.9375 ]
[.01131 .31783 0 5.9375 ]
[.01131 .41449 -14 -5.9375 ]
[.01131 .41449 0 5.9375 ]
[.01131 .51115 -14 -5.9375 ]
[.01131 .51115 0 5.9375 ]
[.01131 .60781 -14 -5.9375 ]
[.01131 .60781 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.15986 .00851 m
.15986 .01476 L
s
gsave
.15986 -0.00399 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.29592 .00851 m
.29592 .01476 L
s
gsave
.29592 -0.00399 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.43197 .00851 m
.43197 .01476 L
s
gsave
.43197 -0.00399 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.56803 .00851 m
.56803 .01476 L
s
gsave
.56803 -0.00399 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(8) show
1.000 setlinewidth
grestore
.70408 .00851 m
.70408 .01476 L
s
gsave
.70408 -0.00399 -68 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.84014 .00851 m
.84014 .01476 L
s
gsave
.84014 -0.00399 -68 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(12) show
1.000 setlinewidth
grestore
.97619 .00851 m
.97619 .01476 L
s
gsave
.97619 -0.00399 -68 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(14) show
1.000 setlinewidth
grestore
.125 Mabswid
.05782 .00851 m
.05782 .01226 L
s
.09184 .00851 m
.09184 .01226 L
s
.12585 .00851 m
.12585 .01226 L
s
.19388 .00851 m
.19388 .01226 L
s
.22789 .00851 m
.22789 .01226 L
s
.2619 .00851 m
.2619 .01226 L
s
.32993 .00851 m
.32993 .01226 L
s
.36395 .00851 m
.36395 .01226 L
s
.39796 .00851 m
.39796 .01226 L
s
.46599 .00851 m
.46599 .01226 L
s
.5 .00851 m
.5 .01226 L
s
.53401 .00851 m
.53401 .01226 L
s
.60204 .00851 m
.60204 .01226 L
s
.63605 .00851 m
.63605 .01226 L
s
.67007 .00851 m
.67007 .01226 L
s
.7381 .00851 m
.7381 .01226 L
s
.77211 .00851 m
.77211 .01226 L
s
.80612 .00851 m
.80612 .01226 L
s
.87415 .00851 m
.87415 .01226 L
s
.90816 .00851 m
.90816 .01226 L
s
.94218 .00851 m
.94218 .01226 L
s
.25 Mabswid
0 .00851 m
1 .00851 L
s
.02381 .02784 m
.03006 .02784 L
s
gsave
.01131 .02784 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(10) show
1.000 setlinewidth
grestore
.02381 .1245 m
.03006 .1245 L
s
gsave
.01131 .1245 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(15) show
1.000 setlinewidth
grestore
.02381 .22117 m
.03006 .22117 L
s
gsave
.01131 .22117 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(20) show
1.000 setlinewidth
grestore
.02381 .31783 m
.03006 .31783 L
s
gsave
.01131 .31783 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(25) show
1.000 setlinewidth
grestore
.02381 .41449 m
.03006 .41449 L
s
gsave
.01131 .41449 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(30) show
1.000 setlinewidth
grestore
.02381 .51115 m
.03006 .51115 L
s
gsave
.01131 .51115 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(35) show
1.000 setlinewidth
grestore
.02381 .60781 m
.03006 .60781 L
s
gsave
.01131 .60781 -75 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(40) show
1.000 setlinewidth
grestore
.125 Mabswid
.02381 .04717 m
.02756 .04717 L
s
.02381 .06651 m
.02756 .06651 L
s
.02381 .08584 m
.02756 .08584 L
s
.02381 .10517 m
.02756 .10517 L
s
.02381 .14384 m
.02756 .14384 L
s
.02381 .16317 m
.02756 .16317 L
s
.02381 .1825 m
.02756 .1825 L
s
.02381 .20183 m
.02756 .20183 L
s
.02381 .2405 m
.02756 .2405 L
s
.02381 .25983 m
.02756 .25983 L
s
.02381 .27916 m
.02756 .27916 L
s
.02381 .29849 m
.02756 .29849 L
s
.02381 .33716 m
.02756 .33716 L
s
.02381 .35649 m
.02756 .35649 L
s
.02381 .37582 m
.02756 .37582 L
s
.02381 .39516 m
.02756 .39516 L
s
.02381 .43382 m
.02756 .43382 L
s
.02381 .45315 m
.02756 .45315 L
s
.02381 .47249 m
.02756 .47249 L
s
.02381 .49182 m
.02756 .49182 L
s
.02381 .53048 m
.02756 .53048 L
s
.02381 .54981 m
.02756 .54981 L
s
.02381 .56915 m
.02756 .56915 L
s
.02381 .58848 m
.02756 .58848 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.02 w
.09184 .30603 Mdot
.15986 .40809 Mdot
.22789 .01472 Mdot
.29592 .23829 Mdot
.36395 .28832 Mdot
.43197 .26271 Mdot
.5 .31287 Mdot
.56803 .26296 Mdot
.63605 .24251 Mdot
.70408 .3143 Mdot
.77211 .35906 Mdot
.84014 .33466 Mdot
.90816 .60332 Mdot
.97619 .3135 Mdot
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[9]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{24, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/B000`400?l00000o`00003oo`3ooolQ
0?ooo`00o`3ooolQ0?ooo`00o`3ooolQ0?ooo`00>@3oool4000002<0oooo00<000000?ooo`3oool0
7P3oool200000240oooo0P00000P0?ooo`030000003oool0oooo00<000007P3oool00`000000oooo
0?ooo`04000001`0oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0:0?ooo`00>P3o
ool00`000000oooo0?ooo`0S0?ooo`030000003oool0oooo01d0oooo00@000000?ooo`3oool00000
7`3oool010000000oooo0?ooo`00000O0?ooo`040000003oool0oooo00000080oooo00<000000?oo
o`3oool06`3oool01@000000oooo0?ooo`3oool0000001h0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`0:0?ooo`00>`3oool00`000000oooo0?ooo`0O0?ooo`D000007P3oool01000
0000oooo0?ooo`00000O0?ooo`040000003oool0oooo000001l0oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo0?ooo`0K0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3oool0
6`3oool00`000000oooo0?ooo`05000000/0oooo000k0?ooo`030000003oool0oooo01l0oooo00@0
00000?ooo`3oool000007`3oool010000000oooo0?ooo`00000P0?ooo`800000803oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo01/0oooo00<000000?ooo`3oool00P3oool00`00
0000oooo0?ooo`0K0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool02P3oool0
03`0oooo00<000000?ooo`3oool07`3oool00`000000oooo0000000O0?ooo`<00000803oool01000
0000oooo0?ooo`00000O0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool06`3o
ool00`000000oooo0?ooo`030?ooo`030000003oool0oooo01X0oooo00L000000?ooo`3oool0oooo
0000003oool0000000`0oooo000i0?ooo`040000003oool0oooo00000280oooo0P00000O0?ooo`04
0000003oool0oooo000001l0oooo00@000000?ooo`3oool000007P3oool200000080oooo00@00000
0?ooo`3oool00000703oool200000080oooo00@000000?ooo`3oool000006`3oool2000000@0oooo
0P00000<0?ooo`00>P3oool2000002@0oooo00<000000?ooo`3oool07P3oool200000240oooo0P00
000P0?ooo`040000003oool0oooo0?ooo`8000007P3oool010000000oooo0?ooo`3oool2000001d0
oooo00<000000?ooo`3oool00`3oool00`000000oooo0?ooo`0:0?ooo`00o`3ooolQ0?ooo`00o`3o
oolQ0?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo
0?l0oooo1P3oool001P0oooo00<000000?ooo`3oool0<@3oool300000=40oooo000B0?ooool00000
1P0000090?ooo`00603oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool01P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo00H0oooo00<0
00000?ooo`3oool00`3oool5000000H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`050?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00D0oooo00<000000?ooo`3oool01P3oool00`000000oooo0?oo
o`060?ooo`030000003oool0oooo00H0oooo00<000000?ooo`3oool03@3oool001P0oooo00<00000
0?ooo`3oool0<03oool500000=00oooo000;0?ooo`030000003oool0oooo00<000001`3oool00`00
0000oooo0?ooo`0a0?ooo`<00000d@3oool000/0oooo00@000000?ooo`3oool000000P3oool00`00
0000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo1P3oool000/0oooo00@000000?ooo`3o
ool000000P3oool00`000000oooo0?ooo`040?ooo`<00000o`3oool60?ooo`002`3oool010000000
oooo0?ooo`0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool60?oo
o`002`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00@0oooo00<000000?oo
o`3oool0o`3oool60?ooo`002P3oool200000080oooo00@000000?ooo`3oool000001P3oool00`00
0000oooo0?ooo`3o0?ooo`H0oooo000;0?ooo`040000003oool0oooo0?ooo`8000001`3oool00`00
0000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`800000o`3oool70?ooo`00603oool00`000000oooo
0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool001P0oooo00<00000
0?ooo`3oool0o`3oool60?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`80
0000o`3oool70?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003o
ool0oooo0?l0oooo1P3oool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool00`00
0000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`800000o`3oool70?ooo`00603oool00`000000oooo
0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool001P0oooo00<00000
0?ooo`3oool0o`3oool60?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`80
0000o`3oool70?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003o
ool0oooo0?l0oooo1P3oool000/0oooo00@000000?ooo`3oool0oooo0P0000070?ooo`030000003o
ool0oooo0?l0oooo1P3oool000/0oooo00@000000?ooo`3oool000000P3oool00`000000oooo0?oo
o`040?ooo`030000003oool0oooo0?l0oooo1P3oool000/0oooo00<000000?ooo`3oool00`3oool0
0`000000oooo0?ooo`040?ooo`<00000o`3oool60?ooo`002`3oool010000000oooo0?ooo`000002
0?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool60?ooo`002`3oool00`00
0000oooo0?ooo`03000000L0oooo00<000000?ooo`3oool0o`3oool60?ooo`002P3oool2000000<0
oooo00<000000?ooo`3oool01P3oool00`000000oooo0?ooo`3o0?ooo`H0oooo000;0?ooo`040000
003oool0oooo0?ooo`<000001P3oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`800000
o`3oool70?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0
oooo0?l0oooo1P3oool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool00`000000
oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`800000o`3oool70?ooo`00603oool00`000000oooo0?oo
o`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool001P0oooo00<000000?oo
o`3oool0o`3oool60?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`800000
o`3oool70?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0
oooo0?l0oooo1P3oool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool00`000000
oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`800000o`3oool70?ooo`00603oool00`000000oooo0?oo
o`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool000T0oooo10000000103o
ool000000000000000070?ooo`030000003oool0oooo0?l0oooo1P3oool000X0oooo00D000000?oo
o`3oool0oooo000000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool6
0?ooo`002`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo00@0oooo0`00003o
0?ooo`H0oooo000;0?ooo`040000003oool0oooo00000080oooo00<000000?ooo`3oool0103oool0
0`000000oooo0?ooo`3o0?ooo`H0oooo000<0?ooo`030000003oool000000080oooo00<000000?oo
o`3oool0103oool00`000000oooo0?ooo`3o0?ooo`H0oooo00090?ooo`060000003oool0oooo0000
003oool000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo04<0oooo0`00002o
0?ooo`002P3oool2000000<0oooo0P0000070?ooo`030000003oool0oooo0480oooo1@00001D0?oo
o`<00000I`3oool001P0oooo0P0000130?ooo`D00000D`3oool5000006H0oooo000H0?ooo`030000
003oool0oooo0480oooo1@00001C0?ooo`D00000IP3oool001P0oooo00<000000?ooo`3oool0@`3o
ool3000005@0oooo1@00001V0?ooo`00603oool00`000000oooo0?ooo`2K0?ooo`<00000I`3oool0
01P0oooo00<000000?ooo`3oool0IP3oool300000240oooo0`00001h0?ooo`00603oool2000006H0
oooo1@00000O0?ooo`D00000M`3oool001P0oooo00<000000?ooo`3oool0I@3oool5000001l0oooo
1@00001g0?ooo`00603oool00`000000oooo0?ooo`1U0?ooo`D000007`3oool5000007L0oooo000H
0?ooo`030000003oool0oooo06H0oooo0`00000Q0?ooo`<00000N03oool001P0oooo00<000000?oo
o`3oool0o`3oool60?ooo`00603oool200000?l0oooo1`3oool001P0oooo00<000000?ooo`3oool0
E@3oool300000:d0oooo000H0?ooo`030000003oool0oooo05@0oooo1@00002/0?ooo`00603oool0
0`000000oooo0?ooo`1D0?ooo`D00000[03oool001P0oooo00<000000?ooo`3oool0E03oool50000
0:`0oooo000H0?ooo`8000003`3oool3000004@0oooo0`00002]0?ooo`00603oool00`000000oooo
0?ooo`0=0?ooo`D00000l`3oool001P0oooo00<000000?ooo`3oool03@3oool5000006H0oooo0`00
000b0?ooo`<00000A03oool3000000h0oooo00090?ooo`@000000P3oool2000000L0oooo00<00000
0?ooo`3oool03@3oool5000006D0oooo1@00000`0?ooo`D00000@P3oool5000000d0oooo000:0?oo
o`050000003oool0oooo0?ooo`0000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0
oooo00h0oooo0`00001V0?ooo`D00000<03oool500000480oooo1@00000=0?ooo`002`3oool00`00
0000oooo0?ooo`030?ooo`030000003oool0oooo00@0oooo0`00001g0?ooo`D00000<03oool50000
0480oooo1@00000=0?ooo`002`3oool010000000oooo0?ooo`0000020?ooo`030000003oool0oooo
00@0oooo00<000000?ooo`3oool0N03oool300000380oooo0`0000140?ooo`<000003P3oool000`0
oooo00<000000?ooo`0000000P0000070?ooo`030000003oool0oooo0?l0oooo1P3oool000T0oooo
00@000000?ooo`3oool000000P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0=40
oooo0`00000a0?ooo`002P3oool2000000<0oooo0`0000060?ooo`030000003oool0oooo0=00oooo
1@00000`0?ooo`00603oool200000=40oooo1@00000`0?ooo`00603oool00`000000oooo0?ooo`3@
0?ooo`D00000<03oool001P0oooo00<000000?ooo`3oool0d@3oool300000340oooo000H0?ooo`03
0000003oool0oooo0?l0oooo1P3oool001P0oooo00<000000?ooo`3oool0_`3oool3000004<0oooo
000H0?ooo`800000_`3oool500000480oooo000H0?ooo`030000003oool0oooo0;h0oooo1@000012
0?ooo`00603oool00`000000oooo0?ooo`2n0?ooo`D00000@P3oool001P0oooo00<000000?ooo`3o
ool0_`3oool3000004<0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool001P0oooo0P00
003o0?ooo`L0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool001P0oooo00<000000?oo
o`3oool0o`3oool60?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000
003oool0oooo0?l0oooo1P3oool001P0oooo0P00003o0?ooo`L0oooo000H0?ooo`030000003oool0
oooo0?l0oooo1P3oool001P0oooo00<000000?ooo`3oool0803oool300000>80oooo000:0?ooo`80
00000P3oool3000000L0oooo00<000000?ooo`3oool07`3oool500000>40oooo00090?ooo`060000
003oool0oooo0000003oool000000P3oool00`000000oooo0?ooo`040?ooo`030000003oool0oooo
01l0oooo1@00003Q0?ooo`00303oool00`000000oooo000000020?ooo`030000003oool0oooo00@0
oooo0`00000O0?ooo`D00000h@3oool000/0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0200oooo0`00003R0?ooo`00303oool00`000000oooo
000000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0o`3oool60?ooo`002@3o
ool01P000000oooo0?ooo`000000oooo00000080oooo00<000000?ooo`3oool0103oool00`000000
oooo0?ooo`3o0?ooo`H0oooo000:0?ooo`8000000`3oool2000000L0oooo00<000000?ooo`3oool0
o`3oool60?ooo`00603oool200000?l0oooo1`3oool001P0oooo00<000000?ooo`3oool0o`3oool6
0?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo0?l0
oooo1P3oool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool200000?l0oooo1`3o
ool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool00`000000oooo0?ooo`3o0?oo
o`H0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool001P0oooo00<000000?ooo`3oool0
o`3oool60?ooo`00603oool200000?l0oooo1`3oool001P0oooo00<000000?ooo`3oool0o`3oool6
0?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo0?l0
oooo1P3oool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool200000?l0oooo1`3o
ool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool00`000000oooo0?ooo`3o0?oo
o`H0oooo000:0?ooo`8000000`3oool2000000L0oooo00<000000?ooo`3oool0o`3oool60?ooo`00
2@3oool01P000000oooo0?ooo`000000oooo00000080oooo00<000000?ooo`3oool0103oool00`00
0000oooo0?ooo`3o0?ooo`H0oooo000<0?ooo`030000003oool0oooo0080oooo00<000000?ooo`3o
ool0103oool300000?l0oooo1P3oool000/0oooo00@000000?ooo`3oool000000P3oool00`000000
oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo1P3oool000`0oooo00<000000?ooo`000000
0P0000070?ooo`030000003oool0oooo0?l0oooo1P3oool000T0oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo0?ooo`060?ooo`030000003oool0oooo0?l0oooo1P3oool000X0oooo0P00
00030?ooo`<000001P3oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`800000o`3oool7
0?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo0?l0
oooo1P3oool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool00`000000oooo0?oo
o`3o0?ooo`H0oooo000H0?ooo`800000o`3oool70?ooo`00603oool00`000000oooo0?ooo`3o0?oo
o`H0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool001P0oooo00<000000?ooo`3oool0
o`3oool60?ooo`00603oool00`000000oooo0?ooo`3o0?ooo`H0oooo000H0?ooo`030000003oool0
oooo0?l0oooo1P3oool001P0oooo0P00003o0?ooo`L0oooo000H0?ooo`030000003oool0oooo0?l0
oooo1P3oool001P0oooo00<000000?ooo`3oool0o`3oool60?ooo`00603oool00`000000oooo0?oo
o`3o0?ooo`H0oooo000H0?ooo`030000003oool0oooo0?l0oooo1P3oool001P0oooo0P00003o0?oo
o`L0oooo000H0?ooo`030000003oool0oooo0>80oooo0`00000P0?ooo`00303oool00`000000oooo
00000002000000L0oooo00<000000?ooo`3oool0h@3oool5000001l0oooo000<0?ooo`030000003o
ool000000080oooo00<000000?ooo`3oool0103oool00`000000oooo0?ooo`3Q0?ooo`D000007`3o
ool000T0oooo1P0000020?ooo`030000003oool0oooo00@0oooo00<000000?ooo`3oool0h@3oool5
000001l0oooo00090?ooo`060000003oool0oooo0000003oool000000P3oool00`000000oooo0?oo
o`040?ooo`<00000hP3oool300000200oooo000:0?ooo`050000003oool000000?ooo`0000000P3o
ool00`000000oooo0?ooo`040?ooo`030000003oool0oooo0?l0oooo1P3oool000/0oooo0P000000
1@3oool000000?ooo`3oool0000000H0oooo00<000000?ooo`3oool0o`3oool60?ooo`00303oool0
0`000000oooo0?ooo`0200000?l0oooo403oool00?l0oooo8@3oool00?l0oooo8@3oool00001\
\>"],
  ImageRangeCache->{{{164.938, 342.438}, {553.938, 444.625}} -> {-16.5079, \
149.171, 0.0567424, 0.199667}},
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[TextData[{
  "The ",
  StyleBox["Select", "MR"],
  " function takes two arguments \[LongDash] the first is the expression from \
which it will select elements, and the second argument is a function that \
must return ",
  StyleBox["True", "MR"],
  " or ",
  StyleBox["False", "MR"],
  ". ",
  StyleBox["Select[", "MR"],
  StyleBox["expr", "IT"],
  StyleBox[",", "MR"],
  StyleBox["test", "IT"],
  StyleBox["]", "MR"],
  " will then select those elements from ",
  StyleBox["expr",
    FontSlant->"Italic"],
  " that return ",
  StyleBox["True", "MR"],
  " when ",
  StyleBox["test",
    FontSlant->"Italic"],
  " is applied to them."
}], "TextFirst",
  CellTags->{"i:39", "b:16", "IPM3.4.6.1"}],

Cell[TextData[{
  "Suppose we wish to exclude all data points that lie outside of the range \
20 to 30. Then we need a function that returns ",
  StyleBox["True", "MR"],
  " if its argument is in that range. "
}], "Text",
  PageBreakAbove->True,
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Select[data, 20 \[LessEqual] # \[LessEqual] 30 &]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \({24.39001`, 29.669`, 20.8856`, 23.4736`, 22.1488`, 24.7434`, 22.1619`, 
      21.1039`, 24.8177`, 27.1331`, 25.8705`, 24.7762`}\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[TextData[{
  "A good way to become comfortable with pure functions is to see them in \
action, so we will convert some of the functions we defined earlier into pure \
functions, showing both the ",
  StyleBox["(\[Ellipsis]#\[Ellipsis])&", "MR"],
  " and the ",
  StyleBox["Function", "MR"],
  " forms so that you can decide which you prefer to use."
}], "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell["\<\
This function tests whether all the elements of a list are even.\
\>", "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(areEltsEven[lis_] := Apply[And, Map[EvenQ, lis]]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"i:40", "b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(areEltsEven[{2, 4, 5, 8}]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(False\)], "Output",
  CellLabel->"Out[12]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell["Here it is written using pure functions.", "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Function[lis, Apply[And, Map[EvenQ, lis]]]\)[{2, 4, 5, 8}]\)], "Input",\

  CellLabel->"In[13]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(False\)], "Output",
  CellLabel->"Out[13]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(\((Apply[And, Map[EvenQ, #1]])\) &\)[{2, 4, 5, 8}]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \(False\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell["\<\
This function returns each element in the list greater than all previous \
elements.\
\>", "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell["maxima[x_] := Union[Rest[FoldList[Max, 0, x]]]", "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(maxima[{2, 6, 3, 7, 9, 2}]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \({2, 6, 7, 9}\)], "Output",
  CellLabel->"Out[16]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell["Here it is written using pure functions.", "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Function[x, Union[Rest[FoldList[Max, 0, x]]]]\)[{2, 6, 3, 7, 9, 
        2}]\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \({2, 6, 7, 9}\)], "Output",
  CellLabel->"Out[17]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Union[Rest[FoldList[Max, 0, #]]] &\)[{2, 6, 3, 7, 9, 2}]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \({2, 6, 7, 9}\)], "Output",
  CellLabel->"Out[18]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[TextData[{
  "We can also create nested pure functions. For example, this maps the pure \
squaring function over the three-element list ",
  StyleBox["{3,2,7}", "MR"],
  "."
}], "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[#\^2 &, {3, 2, 7}]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \({9, 4, 49}\)], "Output",
  CellLabel->"Out[19]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[TextData[{
  "When dealing with nested pure functions, the shorthand notation can be \
used for each of the pure functions but care needs to be taken to avoid \
confusion as to which ",
  StyleBox["#", "MR"],
  " variable belongs to which ",
  "pure",
  " function. This can be avoided by using ",
  StyleBox["Function", "MR"],
  ", in which case different variable names can be used."
}], "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Function[y, Map[Function[x, x\^2], y]]\)[{3, 2, 7}]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[BoxData[
    \({9, 4, 49}\)], "Output",
  CellLabel->"Out[20]=",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:16", "IPM3.4.6.2", "c:23"}],

Cell["\<\
Write a function to sum the squares of the elements of a numeric list.\
\>", "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[TextData[{
  "Write a function to sum the digits of any integer. You will need the ",
  StyleBox["IntegerDigits", "MR"],
  " function (use ",
  StyleBox["?IntegerDigits", "MR"],
  ", or look up ",
  ButtonBox["IntegerDigits",
    ButtonStyle->"RefGuideLink"],
  " in the Help Browser to find out about this function)."
}], "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Using the definition of the ",
  StyleBox["distance", "MR"],
  " function from Exercise 3 of Section 4.4, write a new function ",
  StyleBox["diameter[", "MR"],
  StyleBox["pts", "IT"],
  StyleBox["]", "MR"],
  " that, given a set of points in the plane, finds the maximum distance \
between all pairs of points. Try to incorporate the ",
  StyleBox["distance", "MR"],
  " function into ",
  StyleBox["diameter", "MR"],
  " without naming it explicitly; that is, use it as a pure function. \
Consider using ",
  StyleBox["Distribute", "MR"],
  " to get the set of all pairs of points."
}], "ExerciseItem",
  CellTags->{"i:41", "b:16", "IPM3.4.6.2"}],

Cell[BoxData[
    \(\(pts = {p\_1, p\_2, p\_3};\)\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Distribute[{pts, pts}, List]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[BoxData[
    \({{p\_1, p\_1}, {p\_1, p\_2}, {p\_1, p\_3}, {p\_2, p\_1}, {p\_2, 
        p\_2}, {p\_2, p\_3}, {p\_3, p\_1}, {p\_3, p\_2}, {p\_3, 
        p\_3}}\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:16", "IPM3.4.6.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Take the ",
  StyleBox["removeRand", "MR"],
  " function defined in Section 4.4 and rewrite it as a pure function."
}], "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell["\<\
removeRand[lis_] := 
Delete[lis, Random[Integer, {1, Length[lis]}]]\
\>", "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:16", "IPM3.4.6.2"}]
}, Closed]],

Cell[TextData[{
  "Convert the ",
  StyleBox["deal", "MR"],
  " function developed earlier into one that uses pure functions. Use the \
pure function version of the ",
  StyleBox["removeRand", "MR"],
  " function from the previous exercise in your new ",
  StyleBox["deal", "MR"],
  " function definition."
}], "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[TextData[{
  "Create a function ",
  StyleBox["RepUnit[", "MR"],
  StyleBox["n", "IT"],
  StyleBox["]", "MR"],
  " that generates integers of length ",
  StyleBox["n",
    FontSlant->"Italic"],
  " consisting entirely of 1s. For example ",
  StyleBox["RepUnit[7]", "MR"],
  " should produce 1111111."
}], "ExerciseItem",
  CellTags->{"i:42", "b:16", "IPM3.4.6.2"}],

Cell[TextData[{
  "Create a function ",
  StyleBox["chooseWithoutReplacement[", "MR"],
  StyleBox["lis", "IT"],
  StyleBox[",", "MR"],
  StyleBox["n", "IT"],
  StyleBox["]", "MR"],
  " that is a generalization of the ",
  StyleBox["deal", "MR"],
  " function in that it will work with ",
  StyleBox["any",
    FontSlant->"Italic"],
  " list."
}], "ExerciseItem",
  CellTags->{"i:43", "b:16", "IPM3.4.6.2"}],

Cell[TextData[{
  "Write a pure function that moves a random walker from one location on a \
square lattice to one of the four adjoining locations with equal probability. \
For example, starting at ",
  StyleBox["{0,0}", "MR"],
  ", the function should return either ",
  StyleBox["{0,1}", "MR"],
  ", ",
  StyleBox["{0,-1}", "MR"],
  ", ",
  StyleBox["{1,0}", "MR"],
  " or ",
  StyleBox["{-1,0}", "MR"],
  " with equal likelihood. Now, use this ",
  "pure",
  " function with ",
  StyleBox["NestList", "MR"],
  " to generate the list of step locations for an ",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  "-step random walk starting at ",
  StyleBox["{0,0}", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Create a function ",
  StyleBox["WordsStartingWith[", "MR"],
  StyleBox["lis", "IT"],
  StyleBox[",", "MR"],
  StyleBox["char", "IT"],
  StyleBox["]", "MR"],
  " that outputs all those words in ",
  StyleBox["lis",
    FontSlant->"Italic"],
  " that begin with the character ",
  StyleBox["char",
    FontSlant->"Italic"],
  ". As a sample list, you can use the dictionary.dat file that comes with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "ExerciseItem",
  CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],

Cell["Here is a platform-independent path to the dictionary file.", \
"ExercisesNoLabel",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(wordfile = 
      ToFileName[{$InstallationDirectory, "\<Documentation\>", "\<English\>", \
"\<Demos\>", "\<DataFiles\>"}, "\<dictionary.dat\>"]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[BoxData[
    \("C:\\Program Files\\Wolfram \
Research\\Mathematica\\5.1\\Documentation\\English\\Demos\\DataFiles\\\
dictionary.dat"\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:16", "IPM3.4.6.2"}]
}, Closed]],

Cell[TextData[{
  "This reads in the file using ",
  StyleBox["ReadList", "MR"],
  ", specifying the type of data we are reading in as a ",
  StyleBox["Word", "MR"],
  "."
}], "ExercisesNoLabel",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[BoxData[
    \(\(words = ReadList[wordfile, Word];\)\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:16", "IPM3.4.6.2"}]
}, Closed]],

Cell[TextData[{
  "Modify Exercise 9 above so that ",
  StyleBox["WordsStartingWith", "MR"],
  " accepts a string of arbitrary length as its second argument."
}], "ExerciseItem",
  CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{

Cell[TextData[{
  "A naive approach to polynomial arithmetic would require three additions \
and six multiplies to carry out the arithmetic in the expression ",
  Cell[BoxData[
      \(TraditionalForm\`a\ x\^3 + b\ x\^2 + c\ x + d\)]],
  ". Using Horner\[CloseCurlyQuote]s method for fast polynomial \
multiplication, this expression can be represented as ",
  Cell[BoxData[
      \(TraditionalForm\`d + x(c + x(b + a\ x))\)]],
  ", where there are now half as many multiplies. In general, the number of \
multiplies for an ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  "-degree polynomial is given by:"
}], "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2", "i:55"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Binomial[n + 1, 2]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->"IPM3.4.6.2"],

Cell[BoxData[
    \(1\/2\ n\ \((1 + n)\)\)], "Output",
  CellLabel->"Out[6]=",
  CellTags->"IPM3.4.6.2"]
}, Closed]],

Cell[TextData[{
  "This, of course, grows quadratically with ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ", whereas Horner\[CloseCurlyQuote]s method grows linearly. Create a \
function ",
  StyleBox["Horner[", "MR"],
  StyleBox["lis", "IT"],
  StyleBox[",", "MR"],
  StyleBox["var", "IT"],
  StyleBox["]", "MR"],
  " that implements Horner\[CloseCurlyQuote]s method for polynomial \
multiplication. Here is some sample input and the corresponding output that \
your function should generate."
}], "ExercisesNoLabel",
  CellTags->"IPM3.4.6.2"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Horner[{a, b, c, d}, x]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->"IPM3.4.6.2"],

Cell[BoxData[
    \(d + x\ \((c + x\ \((b + a\ x)\))\)\)], "Output",
  CellLabel->"Out[7]=",
  CellTags->"IPM3.4.6.2"]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Expand[%]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->"IPM3.4.6.2"],

Cell[BoxData[
    \(d + c\ x + b\ x\^2 + a\ x\^3\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->"IPM3.4.6.2"]
}, Closed]]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["One-liners", "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 6}},
  CellTags->{
  "IPM3.4.7.0", "IPM3.4.7.1", "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4", 
    "c:24"}],

Cell["\<\
In the simplest version of a user-defined function, there are no value \
declarations or auxiliary function definitions; the right-hand side is a \
single nested function call whose arguments are the names of the arguments on \
the left-hand side, without the blanks. These \
\[OpenCurlyDoubleQuote]one-liners\[CloseCurlyDoubleQuote] are fantastically \
useful and so we will discuss them in the context of three examples, one from \
electrical engineering (computing Hamming distance), one from ancient history \
(the Josephus problem), and the last a simple and practical problem (counting \
change).\
\>", "TextFirst",
  CellTags->"IPM3.4.7.0"],

Cell[CellGroupData[{

Cell["Hamming distance", "Subsection",
  CellTags->{"b:17.0", "IPM3.4.7.1", "c:25"}],

Cell[TextData[{
  "When a code is transmitted over a channel in the presence of noise, errors \
will often occur. The task of channel coding is to represent the source \
information in a manner that minimizes the error probability in decoding. ",
  StyleBox["Hamming distance", "TermDefinition"],
  " is used in source coding to represent an information source with the \
minimum number of symbols. For two lists of binary symbols, the Hamming \
distance is defined as the number of nonmatching elements and so gives a \
measure of the how well these two lists match up."
}], "TextFirst",
  CellTags->{"i:45", "b:17.0", "IPM3.4.7.1"}],

Cell[TextData[{
  "Let us first think about how we might determine if two binary symbols are \
identical. ",
  StyleBox["SameQ[", "MR"],
  StyleBox["x", "IT"],
  StyleBox[",", "MR"],
  StyleBox["y", "IT"],
  StyleBox["]", "MR"],
  " will return ",
  StyleBox["True", "MR"],
  " if ",
  StyleBox["x",
    FontSlant->"Italic"],
  " and ",
  StyleBox["y",
    FontSlant->"Italic"],
  " are identical."
}], "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \({SameQ[0, 0], SameQ[1, 0], SameQ[1, 1]}\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \({True, False, True}\)], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[TextData[{
  "So we need to thread ",
  StyleBox["SameQ", "MR"],
  " over the two lists of binary numbers"
}], "TextFirst",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(MapThread[SameQ, {{1, 0, 0, 1, 1}, {0, 1, 0, 1, 0}}]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \({False, False, True, True, False}\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[TextData[{
  "and then count up the occurrences of ",
  StyleBox["False", "MR"],
  "."
}], "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Count[%, False]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \(3\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[TextData[{
  "So a first definition of ",
  StyleBox["HammingDistance", "MR"],
  " could be accomplished by putting these last",
  " ",
  "two pieces together."
}], "TextFirst",
  CellTags->{"i:46", "b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \(HammingDistance[lis1_, lis2_] : \[IndentingNewLine]Count[
        MapThread[SameQ, {lis1, lis2}], False]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(HammingDistance[{1, 0, 0, 1, 1}, {0, 1, 0, 1, 0}]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \(3\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[TextData[{
  "We might try to solve this problem by a more direct approach. Since we are \
dealing with binary information, we could use some of the logical binary \
operators built into ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell["Here is our transposed list again.", "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(lis = Transpose[{{1, 0, 0, 1, 1}, {0, 1, 0, 1, 0}}]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \({{1, 0}, {0, 1}, {0, 0}, {1, 1}, {1, 0}}\)], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[TextData[{
  StyleBox["BitXor[", "MR"],
  StyleBox["x", "IT"],
  StyleBox[",", "MR"],
  StyleBox["y", "IT"],
  StyleBox["]", "MR"],
  " returns the bitwise XOR of ",
  StyleBox["x",
    FontSlant->"Italic"],
  " and ",
  StyleBox["y",
    FontSlant->"Italic"],
  ". So if ",
  StyleBox["x",
    FontSlant->"Italic"],
  " and ",
  StyleBox["y",
    FontSlant->"Italic"],
  " can only be among the binary integers 0 or 1, ",
  StyleBox["BitXor", "MR"],
  " will return 0 whenever they are the same and will return 1 whenever they \
are different."
}], "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1", "i:56"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[BitXor, {{0, 0}, {1, 0}, {1, 1}}, {1}]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \({0, 1, 0}\)], "Output",
  CellLabel->"Out[7]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[TextData[{
  "Here then is ",
  StyleBox["BitXor", "MR"],
  " applied to ",
  StyleBox["lis", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[BitXor, lis, {1}]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \({1, 1, 0, 0, 1}\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell["And here are the number of 1s that occur in that list.", "TextFirst",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, %]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \(3\)], "Output",
  CellLabel->"Out[9]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[TextData[{
  "Summing up, our function ",
  StyleBox["HammingDistance2", "MR"],
  " first pairs up the lists (",
  StyleBox["Transpose", "MR"],
  "), then determines which pairs contain different elements (apply ",
  StyleBox["BitXor", "MR"],
  "), and finally counts up the number of 1s (",
  StyleBox["Apply[Plus,\[Ellipsis]]", "MR"],
  ")."
}], "TextFirst",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \(HammingDistance2[lis1_, lis2_] := 
      Apply[Plus, \[IndentingNewLine]Apply[BitXor, 
          Transpose[{lis1, lis2}], {1}]\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(HammingDistance2[{1, 0, 0, 1, 1}, {0, 1, 0, 1, 0}]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \(3\)], "Output",
  CellLabel->"Out[11]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell["\<\
Let us compare the running times of these implementations using a large data \
set, in this case two lists consisting of one million 0s and 1s.\
\>", "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \(\(data1 = Table[Random[Integer], {10\^6}];\)\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \(\(data2 = Table[Random[Integer], {10\^6}];\)\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[HammingDistance[data1, data2]]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \({1.1620000000000001`\ Second, 499801}\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Timing[HammingDistance2[data1, data2]]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[BoxData[
    \({1.392`\ Second, 499801}\)], "Output",
  CellLabel->"Out[15]=",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[TextData[{
  "Although these times do not look too bad, they are in fact too slow for \
any serious work with signal processing. The exercises ask you to write an \
implementation of ",
  StyleBox["HammingDistance", "MR"],
  " that runs about two orders of magnitude faster than those presented \
here."
}], "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell["\<\
As an aside, the above computations are not a bad check on the built-in \
random number generator \[LongDash] we would expect that about one half of \
the paired up lists would contain different elements.\
\>", "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[CellGroupData[{

Cell["The Josephus problem", "Subsection",
  CellTags->{"b:17.1", "IPM3.4.7.2", "c:26"}],

Cell["\<\
Flavius Josephus was a Jewish historian during the Roman\[Dash]Jewish war of \
the first century AD. Through his writings comes the following story:\
\>", "TextFirst",
  CellTags->{"i:49", "b:17.1", "IPM3.4.7.2"}],

Cell["\<\
The Romans had chased a group of ten Jews into a cave and were about to \
attack. Rather than die at the hands of their enemy, the group chose to \
commit suicide one by one. Legend has it though, that they decided to go \
around their circle of ten individuals and eliminate every other person until \
only one was left. \
\>", "Quote",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[TextData[{
  "Who was the last to survive? Although a bit macabre, this problem has a \
definite mathematical interpretation that lends itself well to a functional \
style of programming. We will start by changing the problem a bit (the \
importance of rewording a problem can hardly be overstated; the key to most \
problem-solving resides in turning something we can not work with into \
something we can work with). We will restate the problem as follows: ",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  " people are lined up. The first person is moved to the end of the line, \
the second person is removed from the line, the third person is moved to the \
end of the line, and so on until only one person remains in the line."
}], "Text",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[TextData[{
  "The statement of the problem indicates that there is a repetitive action, \
performed over and over again. It involves the use of the ",
  StyleBox["RotateLeft", "MR"],
  " function (move the person at the front of the line to the back of the \
line) followed by the use of the ",
  StyleBox["Rest", "MR"],
  " function (remove the next person from the line)."
}], "Text",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Rest[RotateLeft[#]] &\)[{a, b, c, d}]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    \({c, d, a}\)], "Output",
  CellLabel->"Out[16]=",
  CellTags->{"b:17.1", "IPM3.4.7.2"}]
}, Closed]],

Cell[TextData[{
  "At this point it is already pretty clear where this computation is headed. \
We want to take a list and, using the ",
  StyleBox["Nest", "MR"],
  " function, perform the ",
  "pure",
  " function call (",
  StyleBox["Rest[RotateLeft[#])&", "MR"],
  " on the list until only one element remains. A list of ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " elements will need ",
  Cell[BoxData[
      \(TraditionalForm\`n - 1\)]],
  " calls. So we can now write the function, to which we give the apt name ",
  StyleBox["survivor", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    \(survivor[lis_] := 
      Nest[Rest[RotateLeft[#]] &, lis, Length[lis] - 1]\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->{"i:50", "b:17.1", "IPM3.4.7.2"}],

Cell[TextData[{
  "Trying out the ",
  StyleBox["survivor", "MR"],
  " function on a list of ten, we see that the fifth position will be the \
position of the survivor."
}], "TextFirst",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(survivor[Range[10]]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    \({5}\)], "Output",
  CellLabel->"Out[18]=",
  CellTags->{"b:17.1", "IPM3.4.7.2"}]
}, Closed]],

Cell[TextData[{
  "Tracing the applications of ",
  StyleBox["RotateLeft", "MR"],
  " in this example gives a very clear picture of what is going on. The \
following form of ",
  StyleBox["TracePrint", "MR"],
  " shows only the results of the applications of ",
  StyleBox["RotateLeft", "MR"],
  " that occur during evaluation of the expression ",
  StyleBox["survivor[Range[6]]", "MR"],
  "."
}], "TextFirst",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TracePrint[survivor[Range[6]], RotateLeft]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"    ", "\[InvisibleSpace]", 
        TagBox["RotateLeft",
          HoldForm]}],
      SequenceForm[ 
        Indent[ 4], 
        HoldForm[ RotateLeft]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"   ", "\[InvisibleSpace]", 
        TagBox[\({2, 3, 4, 5, 6, 1}\),
          HoldForm]}],
      SequenceForm[ 
        Indent[ 3], 
        HoldForm[ {2, 3, 4, 5, 6, 1}]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"    ", "\[InvisibleSpace]", 
        TagBox["RotateLeft",
          HoldForm]}],
      SequenceForm[ 
        Indent[ 4], 
        HoldForm[ RotateLeft]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"   ", "\[InvisibleSpace]", 
        TagBox[\({4, 5, 6, 1, 3}\),
          HoldForm]}],
      SequenceForm[ 
        Indent[ 3], 
        HoldForm[ {4, 5, 6, 1, 3}]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"    ", "\[InvisibleSpace]", 
        TagBox["RotateLeft",
          HoldForm]}],
      SequenceForm[ 
        Indent[ 4], 
        HoldForm[ RotateLeft]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"   ", "\[InvisibleSpace]", 
        TagBox[\({6, 1, 3, 5}\),
          HoldForm]}],
      SequenceForm[ 
        Indent[ 3], 
        HoldForm[ {6, 1, 3, 5}]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"    ", "\[InvisibleSpace]", 
        TagBox["RotateLeft",
          HoldForm]}],
      SequenceForm[ 
        Indent[ 4], 
        HoldForm[ RotateLeft]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"   ", "\[InvisibleSpace]", 
        TagBox[\({3, 5, 1}\),
          HoldForm]}],
      SequenceForm[ 
        Indent[ 3], 
        HoldForm[ {3, 5, 1}]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"    ", "\[InvisibleSpace]", 
        TagBox["RotateLeft",
          HoldForm]}],
      SequenceForm[ 
        Indent[ 4], 
        HoldForm[ RotateLeft]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    InterpretationBox[
      RowBox[{"   ", "\[InvisibleSpace]", 
        TagBox[\({1, 5}\),
          HoldForm]}],
      SequenceForm[ 
        Indent[ 3], 
        HoldForm[ {1, 5}]],
      Editable->False]], "Print",
  CellLabel->"From In[19]:=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[BoxData[
    \({5}\)], "Output",
  CellLabel->"Out[19]=",
  CellMargins->{{Inherited, Inherited}, {4, Inherited}},
  CellTags->{"b:17.1", "IPM3.4.7.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pocket change", "Subsection",
  CellTags->{"b:17.2", "IPM3.4.7.3", "c:27"}],

Cell["\<\
As another example, we will write a program to perform an operation most of \
us do every day: calculating how much change we have in our pocket. Suppose \
we have the following collection of coins.\
\>", "TextFirst",
  CellTags->{"i:51", "b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(coins = {p, p, q, n, d, d, p, q, q, p}\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \({p, p, q, n, d, d, p, q, q, p}\)], "Output",
  CellLabel->"Out[20]=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],

Cell[TextData[{
  "Assume ",
  Cell[BoxData[
      \(TraditionalForm\`p\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  ", ",
  Cell[BoxData[
      \(TraditionalForm\`d\)]],
  ", and ",
  Cell[BoxData[
      \(TraditionalForm\`q\)]],
  " represent pennies, nickels, dimes, and quarters, respectively. Let us \
start by using the ",
  StyleBox["Count", "MR"],
  " function to determine the number of pennies we have."
}], "TextFirst",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Count[coins, p]\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \(4\)], "Output",
  CellLabel->"Out[21]=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],

Cell["\<\
This works. So let us do the same thing for all of the coin types.\
\>", "TextFirst",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \({Count[coins, p], Count[coins, n], Count[coins, d], 
      Count[coins, q]}\)], "Input",
  CellLabel->"In[22]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \({4, 1, 2, 3}\)], "Output",
  CellLabel->"Out[22]=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],

Cell[TextData[{
  "Looking at this list, it is apparent that there ought to be a more compact \
way of writing the list. If we ",
  StyleBox["Map", "MR"],
  " a pure function involving ",
  StyleBox["Count", "MR"],
  " and ",
  StyleBox["coins", "MR"],
  " on to the list ",
  StyleBox["{p,n,d,q}", "MR"],
  ", it should do the job."
}], "Text",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[\((Count[coins, #1] &)\), {p, n, d, q}]\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \({4, 1, 2, 3}\)], "Output",
  CellLabel->"Out[23]=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],

Cell[TextData[{
  "Now that we know how many coins of each type we have, we want to calculate \
how much change we have. We first do the calculation ",
  StyleBox["manually",
    FontSlant->"Italic"],
  " to see what we get for an answer (so we will know when our program \
works)."
}], "TextFirst",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(4\ 1 + 1\ 5 + 2\ 10 + 3\ 25\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \(104\)], "Output",
  CellLabel->"Out[24]=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],

Cell[TextData[{
  "From the above computation we see that the lists ",
  StyleBox["{4,1,2,3}", "MR"],
  " and ",
  StyleBox["{1,5,10,25}", "MR"],
  " are first multiplied together element-wise and then the elements of the \
result are added. This suggests a few possibilities."
}], "TextFirst",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Apply[Plus, \(({4, 1, 2, 3}\ {1, 5, 10, 25})\)]\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \(104\)], "Output",
  CellLabel->"Out[25]=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],

Cell[CellGroupData[{

Cell[BoxData[
    \({4, 1, 2, 3} . {1, 5, 10, 25}\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \(104\)], "Output",
  CellLabel->"Out[26]=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],

Cell["\<\
Either of these operations are suitable for the job (to coin a phrase, \
\[OpenCurlyDoubleQuote]there\[CloseCurlyQuote]s not a penny, nickel, quarter, \
or dime\[CloseCurlyQuote]s worth of difference\[CloseCurlyDoubleQuote]). We \
will write the one-liner using the first method.\
\>", "TextFirst",
  CellTags->{"i:52", "b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \(pocketChange[x_] := 
      Apply[Plus, 
        Map[\((Count[x, #] &)\), {p, n, d, q}]\ {1, 5, 10, 25}]\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(pocketChange[coins]\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[BoxData[
    \(104\)], "Output",
  CellLabel->"Out[28]=",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:17.2", "IPM3.4.7.4", "c:28"}],

Cell[TextData[{
  "Write a function to compute the Hamming distance of two binary lists \
(assumed to be of equal length), using ",
  StyleBox["Select", "MR"],
  " and an appropriate predicate function."
}], "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[TextData[{
  "All of the implementations of Hamming distance discussed so far are a bit \
slow for large datasets. You can get a significant speedup in running times \
by using functions that are optimized for working with numbers (a topic we \
discuss in detail in Chapter 8). Write an implementation of Hamming distance \
using the ",
  StyleBox["Total", "MR"],
  " function and then compare running times with the other versions discussed \
in this chapter."
}], "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell["\<\
One of the best ways to learn how to write programs is to practice reading \
code. We list below a number of one-liner function definitions along with a \
very brief explanation of what these user-defined functions do and a typical \
input and output. Deconstruct these programs to see what they do and then \
reconstruct them as compound functions without any pure functions.\
\>", "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{

Cell["\<\
Determine the frequencies with which distinct elements appear in a list.\
\>", "ExerciseSubItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \(frequencies[lis_] := 
      Map[\(({#, Count[lis, #]})\) &, Union[lis]]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(frequencies[{a, a, b, b, b, a, c, c}]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \({{a, 3}, {b, 3}, {c, 2}}\)], "Output",
  CellLabel->"Out[2]=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Divide up a list into parts each of whose lengths are given by the second \
argument.\
\>", "ExerciseSubItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \(split1[lis_, 
        parts_] := \((Inner[Take[lis, {#1, #2}] &, Drop[#1, \(-1\)] + 1, 
              Rest[#1], List] &)\)[FoldList[Plus, 0, parts]]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(split1[Range[10], {2, 5, 0, 3}]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \({{1, 2}, {3, 4, 5, 6, 7}, {}, {8, 9, 10}}\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]],

Cell["\<\
This is the same as the previous program, done in a different way.\
\>", "ExercisesNoLabel",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell["\<\
split2[lis_, parts_] := 
   Map[(Take[lis, # + {1, 0}])&,
      Partition[FoldList[Plus, 0, parts], 2, 1]]\
\>", "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[{
  "Another game in the Illinois State Lottery is based on choosing ",
  Cell[BoxData[
      \(TraditionalForm\`n\)],
    FormatType->StandardForm],
  " numbers, each between 0 and ",
  Cell[BoxData[
      \(TraditionalForm\`s\)],
    FormatType->StandardForm],
  " with no duplicates allowed. Write a user-defined function called ",
  StyleBox["lotto", "MR"],
  " (after the official lottery names of ",
  StyleBox["Little Lotto",
    FontSlant->"Italic"],
  " and ",
  StyleBox["Big Lotto",
    FontSlant->"Italic"],
  ") to perform ",
  StyleBox["sampling without replacement",
    FontSlant->"Italic"],
  " on an arbitrary list. (",
  StyleBox["Note",
    FontSlant->"Italic"],
  ": The difference between this function and the function ",
  StyleBox["chooseWithoutReplacement", "MR"],
  " is that the order of selection is needed here.)"
}], "ExerciseSubItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \(lotto1[lis_, 
        n_] := \((Flatten[
              Rest[MapThread[Complement, {RotateRight[#], #}, 1]]] &)\)[
        NestList[Delete[#, Random[Integer, {1, Length[#]}]] &, lis, 
          n]]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(lotto1[Range[10], 5]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \({10, 3, 2, 7, 6}\)], "Output",
  CellLabel->"Out[7]=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]],

Cell["\<\
This is the same as the previous program, done in a different way.\
\>", "ExercisesNoLabel",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \(lotto2[lis_, n_] := 
      Take[\(Transpose[
            Sort[Transpose[{Table[Random[], {Length[lis]}], 
                  lis}]]]\)\[LeftDoubleBracket]2\[RightDoubleBracket], 
        n]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[TextData[{
  "As the ",
  StyleBox["split", "MR"],
  " and ",
  StyleBox["lotto", "MR"],
  " programs illustrate, user-defined functions can be written in several \
ways. The choice as to which version of a program to use has to be based on \
efficiency. A program whose development time was shorter and which runs \
faster is ",
  StyleBox["better",
    FontSlant->"Italic"],
  " than a program which took more time to develop and which runs more \
slowly. Although concise ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programs tend to run fastest, when execution speed is a primary concern \
(when dealing with very large lists) it is a good idea to take various \
programming approaches and perform ",
  StyleBox["Timing", "MR"],
  " tests to determine the fastest program."
}], "ExercisesNoLabel",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]],

Cell[TextData[{
  "Use the ",
  StyleBox["Timing", "MR"],
  " function to determine when (in terms of the relative sizes of the list \
and the number of elements being chosen) it is preferable to use the \
different versions of the ",
  StyleBox["lotto", "MR"],
  " function."
}], "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[TextData[{
  "Rewrite the ",
  StyleBox["pocketChange", "MR"],
  " function in two different ways \[LongDash] one, using ",
  StyleBox["Dot", "MR"],
  ", and the other using ",
  StyleBox["Inner", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{

Cell["\<\
Make change with quarters, dimes, nickels, and pennies using the fewest \
coins.\
\>", "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \(makeChange[x_] := 
      Quotient[FoldList[Mod, x, {25, 10, 5}], {25, 10, 5, 1}]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(makeChange[119]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[BoxData[
    \({4, 1, 1, 4}\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]]
}, Closed]],

Cell[TextData[{
  "Write a one-liner to create a list of the step locations of a \
two-dimensional random walk that is not restricted to a lattice. ",
  StyleBox["Hint",
    FontSlant->"Italic"],
  ": Each step length must be the same, so the sum of the squares of the ",
  Cell[BoxData[
      \(TraditionalForm\`x\)],
    FormatType->StandardForm],
  "- and ",
  Cell[BoxData[
      \(TraditionalForm\`y\)],
    FormatType->StandardForm],
  "-components of each step should be equal to 1."
}], "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[TextData[{
  "Write a one-liner version of ",
  StyleBox["convertToDate", "MR"],
  " as described in Exercise 7 from Section 4.5. Consider the built-in \
function ",
  StyleBox["FromDigits", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"i:54", "b:17.2", "IPM3.4.7.4"}]
}, Closed]]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 685}},
AutoGeneratedPackage->None,
ScreenStyleEnvironment->"Working",
PrintingStyleEnvironment->"Production",
ShowPageBreaks->False,
WindowSize->{610, 601},
WindowMargins->{{8, Automatic}, {Automatic, 4}},
PrintingCopies->1,
PrintingStartingPageNumber->75,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"An Introduction to Programming with ", 
          StyleBox[ "Mathematica", "IT"]}], "Header"]}, {Cell[ 
        TextData[ {"4 Functional programming"}], "Header"], Inherited, 
      Inherited}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False},
ShowSelection->True,
TaggingRules:>{"NextIndexingNumber" -> 61, "IndexEntries" -> {{"i:10", 
  StyleBox[ "Apply", "MR"]}, {"i:14", {"Dot product, (", 
  StyleBox[ "Dot", "MR"], ")"}}, {"i:16", 
  StyleBox[ "Nest", "MR"]}, {"i:17", 
  StyleBox[ "NestList", "MR"]}, {"i:18", {"Fixed point iteration, (", 
  StyleBox[ "FixedPoint", "MR"], ")"}}, {"i:19", 
  StyleBox[ "Fold", "MR"]}, {"i:19", 
  StyleBox[ "FoldList", "MR"]}, {"i:2", 
  StyleBox[ 
  "Map", "MR"]}, {"i:20", "User-defined functions"}, {"i:27", "Perfect \
shuffle"}, {"i:3", 
  StyleBox[ 
  "Thread", 
    "MR"]}, {"i:30", "Dealing cards"}, {"i:31", "Auxiliary functions"}, \
{"i:31", "Compound functions"}, {"i:33", {"Localizing names, (", 
  StyleBox[ "Module", "MR"], ")"}}, {"i:33", 
  StyleBox[ "Module", "MR"]}, {"i:34", {"Localizing values, (", 
  StyleBox[ "Block", "MR"], ")"}}, {"i:34", 
  StyleBox[ "Block", "MR"]}, {"i:35", {"Localizing constants, (", 
  StyleBox[ 
  "With", "MR"], ")"}}, {"i:36", "Anonymous functions"}, {"i:36", "Pure \
functions"}, {"i:37", 
  StyleBox[ "Function", "MR"]}, {"i:39", 
  StyleBox[ "Select", "MR"]}, {"i:4", 
  StyleBox[ 
  "MapThread", 
    "MR"]}, {"i:41", "Diameter of point set"}, {"i:41", "Distance function"}, \
{"i:41", 
  StyleBox[ "Distribute", "MR"]}, {"i:45", "Hamming distance"}, {"i:48", 
  StyleBox[ 
  "Timing", 
    "MR"]}, {"i:49", "Josephus problem"}, {"i:5", {"Tracing evaluations, (", 
  StyleBox[ "Trace", "MR"], ")"}}, {"i:5", 
  StyleBox[ 
  "Trace", "MR"]}, {"i:51", "Counting change"}, {"i:53", "Converting date \
formats"}, {"i:55", "Horner\[CloseCurlyQuote]s method"}, {"i:57", "Distance \
function"}, {"i:59", {"Divergence (", 
  StyleBox[ "div", "MR"], ")"}}, {"i:7", 
  StyleBox[ "SetAttributes", "MR"]}, {"i:8", {"Clearing values, (", 
  StyleBox[ "Clear", "MR"], ")"}}, {"i:9", {"Removing symbols, (", 
  StyleBox[ 
  "Remove", 
    "MR"], ")"}}, {"i:1", "Functional programming", "features"}, {"i:11", \
"Level specifications", {
  StyleBox[ "Map", "MR"], " and ", 
  StyleBox[ "Apply", "MR"]}}, {"i:11", 
  StyleBox[ "Apply", "MR"], "level specification"}, {"i:11", 
  StyleBox[ 
  "Map", "MR"], "level specification"}, {"i:14", "Matrices", \
{"multiplication, (", 
  StyleBox[ 
  "Dot", "MR"], ")"}}, {"i:15", "Functions", "iteration"}, {"i:18", \
"Iteration", {"fixed point, (", 
  StyleBox[ 
  "FixedPoint", 
    "MR"], ")"}}, {"i:20", "Functions", "user-defined"}, {"i:21", \
"Functions", "syntax"}, {"i:22", "Functions", "arguments"}, {"i:23", \
"Functions", "assignments"}, {"i:24", "Functions", "nested calls"}, {"i:25", \
"Programs", 
  StyleBox[ "maxima", "MR"]}, {"i:26", "Programs", 
  StyleBox[ "cardDeck", "MR"]}, {"i:28", "Programs", 
  StyleBox[ "shuffle", "MR"]}, {"i:29", "Programs", 
  StyleBox[ 
  "deal", "MR"]}, {"i:31", "Functions", "auxiliary"}, {"i:31", "Functions", \
"compound"}, {"i:32", "Programs", 
  StyleBox[ "deal", "MR"]}, {"i:33", "Functions", {"localizing names, (", 
  StyleBox[ 
  "Module", "MR"], ")"}}, {"i:34", "Functions", {"localizing values, (", 
  StyleBox[ 
  "Block", "MR"], ")"}}, {"i:35", "Functions", {"localizing constants, (", 
  StyleBox[ 
  "With", "MR"], ")"}}, {"i:36", "Functions", "anonymous"}, {"i:36", \
"Functions", "pure"}, {"i:40", "Programs", 
  StyleBox[ "areEltsEven", "MR"]}, {"i:41", "Programs", 
  StyleBox[ "diameter", "MR"]}, {"i:41", "Programs", 
  StyleBox[ "distance", "MR"]}, {"i:43", "Programs", 
  StyleBox[ "chooseWithoutReplacement", "MR"]}, {"i:46", "Programs", 
  StyleBox[ "HammingDistance", "MR"]}, {"i:50", "Programs", {
  StyleBox[ 
  "survivor", "MR"], " (to Josephus problem)"}}, {"i:52", "Programs", 
  StyleBox[ "pocketChange", "MR"]}, {"i:53", "Programs", 
  StyleBox[ "convertToDate", "MR"]}, {"i:54", "Programs", 
  StyleBox[ 
  "convertToDate", 
    "MR"]}, {"i:55", "Polynomials", "multiplication using Horner\
\[CloseCurlyQuote]s method"}, {"i:56", "Bitwise logical operators", 
  StyleBox[ 
  "BitXor", 
    "MR"]}, {"i:58", "Perfect numbers", "searching for"}, {"i:58", \
"Programs", 
  StyleBox[ "PerfectSearch", "MR"]}, {"i:59", "Programs", 
  StyleBox[ 
  "div", "MR"]}, {"i:60", "Data", "removing outliers"}, {"i:60", "Outliers", \
"removing from datasets"}, {"i:7", "Attributes", {"setting, (", 
  StyleBox[ 
  "SetAttributes", "MR"], ")"}}, {"i:8", "Symbols", {"clearing values, (", 
  StyleBox[ "Clear", "MR"], ")"}}, {"i:9", "Symbols", {"removing, (", 
  StyleBox[ 
  "Remove", "MR"], ")"}}, {"i:12", "Outer products", {"generalized, ", 
  StyleBox[ 
  "Outer", "MR"]}, ""}, {"i:13", "Inner products", {"generalized, ", 
  StyleBox[ 
  "Inner", "MR"]}, ""}, {"i:15", "Iteration", "functions", ""}, {"i:19", \
"Iteration", {"functions with two arguments, (", 
  StyleBox[ "Fold", "MR"], ")"}, ""}, {"i:35", 
  StyleBox[ "With", "MR"], "", ""}, {"i:38", {
  StyleBox[ "#", "MR"], ", ", 
  StyleBox[ 
  "Slot", "MR"], " (argument to pure function)"}, "", ""}, {"i:42", \
"Programs", 
  StyleBox[ "RepUnit", "MR"], ""}, {"i:42", {"Repeating units, (", 
  StyleBox[ "RepUnit", "MR"], ")"}, "", ""}, {"i:57", "Programs", 
  StyleBox[ 
  "distance", 
    "MR"], ""}, {"i:6", "Attributes", {"threading across expressions (", 
  StyleBox[ "Listable", "MR"], ")"}, ""}, {"i:6", {
  StyleBox[ 
  "Listable", 
    "MR"], " attribute"}, "", ""}}, "PaginationCache" -> {{2004, 9, 2, 17, \
39, 13.6581408`8.887966539192949}, {75, 75, 75, 75, 75, 75, 75, 75, 75, 75, \
75, 75, 75, 75, 76, 76, 76, 76, 76, 76, 76, 76, 76, 76, 76, 76, 76, 76, 76, \
76, 77, 77, 77, 77, 77, 77, 77, 77, 77, 77, 78, 78, 78, 78, 78, 78, 78, 78, \
78, 78, 78, 78, 78, 78, 78, 78, 78, 78, 78, 78, 78, 79, 79, 79, 79, 79, 79, \
79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 79, 80, 80, \
80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, 80, \
81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, 81, \
81, 81, 81, 82, 82, 82, 82, 82, 82, 82, 82, 82, 82, 82, 82, 82, 82, 82, 82, \
82, 82, 82, 82, 82, 82, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, 83, \
83, 83, 83, 83, 83, 83, 83, 83, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, \
84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 84, 85, 85, 85, 85, 85, \
85, 85, 85, 85, 85, 85, 85, 85, 85, 85, 85, 85, 86, 86, 86, 86, 86, 86, 86, \
86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 86, 87, 87, 87, 87, 87, 87, 87, 87, \
87, 87, 87, 87, 87, 87, 87, 87, 87, 87, 88, 88, 88, 88, 88, 88, 88, 88, 88, \
89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, 89, \
89, 89, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, 90, \
90, 90, 90, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, 91, \
91, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 92, 93, 93, 93, 93, 93, \
93, 93, 93, 93, 93, 93, 93, 93, 93, 93, 93, 93, 94, 94, 94, 94, 94, 94, 94, \
94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 94, 95, 95, 95, 95, 95, 95, 95, 95, \
95, 95, 95, 95, 95, 95, 95, 96, 96, 96, 96, 96, 96, 96, 96, 96, 96, 96, 96, \
96, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 97, 98, 98, 98, 98, 98, 98, \
98, 98, 98, 98, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, 99, \
99, 99, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, 100, \
100, 100, 100, 100, 100, 100, 100, 101, 101, 101, 101, 101, 101, 101, 101, \
101, 101, 101, 101, 101, 101, 102, 102, 102, 102, 102, 102, 102, 102, 102, \
102, 102, 102, 102, 102, 102, 102, 102, 103, 103, 103, 103, 103, 103, 103, \
103, 103, 103, 103, 103, 103, 103, 103, 104, 104, 104, 104, 104, 104, 104, \
104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, 104, \
104, 104, 104, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, 105, \
105, 105, 105, 105, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, 106, \
106, 106, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, 107, \
107, 107, 107, 107, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, \
108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 108, 109, 109, 109, 109, \
109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 109, 110, 110, 110, \
110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, 110, \
110, 110, 110, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, 111, \
111, 111, 111, 111, 111, 111, 111, 111, 111, 112, 112, 112, 112, 112, 112, \
112, 112, 112, 112, 112, 112, 112, 112, 112, 112, 113, 113, 113, 113, 113, \
113, 113, 113, 113, 113, 113, 113, 113, 114, 114, 114, 114, 114, 114, 114, \
114, 114, 114, 114}, Automatic}},
CellLabelAutoDelete->False,
Magnification->1,
StyleDefinitions -> "IPM3Styles.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "IPM3.4.0"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[2330, 66, 159, 6, 96, "Chapter",
      CounterAssignments->{{"Chapter", 3}},
      CellTags->{"IPM3.4.0", "c:1"}],
    Cell[2492, 74, 849, 20, 132, "ChapterAbstract",
      CellTags->"IPM3.4.0"]},
  "IPM3.4.1"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[3366, 98, 128, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 0}},
      CellTags->{"b:11", "IPM3.4.1", "c:2"}],
    Cell[3497, 102, 277, 5, 52, "TextFirst",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[3777, 109, 119, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[3899, 114, 119, 3, 33, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4043, 121, 104, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4150, 126, 119, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4306, 134, 110, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4419, 139, 119, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4553, 145, 395, 8, 71, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4973, 157, 129, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5105, 162, 217, 4, 48, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5337, 169, 167, 5, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5529, 178, 121, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5653, 183, 173, 4, 46, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5841, 190, 183, 4, 52, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[6049, 198, 138, 3, 33, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[6190, 203, 33273, 947, 181, 14713, 712, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[39466, 1152, 188, 5, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[39669, 1160, 412, 7, 90, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40106, 1171, 141, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40250, 1176, 136, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40423, 1184, 292, 7, 90, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40718, 1193, 137, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40870, 1199, 374, 13, 52, "TextFirst",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41247, 1214, 107, 1, 33, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41379, 1219, 144, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41526, 1224, 137, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41678, 1230, 149, 4, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41852, 1238, 109, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41964, 1243, 137, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[42116, 1249, 603, 12, 109, "Text",
      CellTags->{"i:1", "b:11", "IPM3.4.1"}],
    Cell[42722, 1263, 926, 17, 147, "Text",
      CellTags->{"b:11", "IPM3.4.1"}]},
  "IPM3.4.2.0"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[43685, 1285, 240, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
      CellTags->{
      "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
        "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],
    Cell[43928, 1291, 951, 26, 128, "TextFirst",
      CellTags->"IPM3.4.2.0"]},
  "IPM3.4.2.1"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[43685, 1285, 240, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
      CellTags->{
      "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
        "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],
    Cell[44904, 1321, 94, 2, 44, "Subsection",
      PageBreakAbove->True,
      CellTags->{"b:12.0", "IPM3.4.2.1", "c:4"}],
    Cell[45001, 1325, 154, 4, 33, "TextFirst",
      CellTags->{"i:2", "b:12.0", "IPM3.4.2.1"}],
    Cell[45180, 1333, 123, 3, 42, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45306, 1338, 123, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45444, 1344, 203, 6, 33, "TextFirst",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45672, 1354, 112, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45787, 1359, 114, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45916, 1365, 361, 14, 52, "Text",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[46302, 1383, 113, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[46418, 1388, 115, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[46548, 1394, 371, 13, 52, "TextFirst",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[46922, 1409, 435, 13, 71, "Text",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47382, 1426, 133, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47518, 1431, 120, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47653, 1437, 138, 3, 33, "TextFirst",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47816, 1444, 134, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47953, 1449, 124, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48092, 1455, 308, 7, 52, "Text",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48403, 1464, 126, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48532, 1469, 197, 4, 52, "TextFirst",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48732, 1475, 112, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48847, 1480, 285, 10, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[49157, 1494, 106, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[49266, 1499, 130, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[49411, 1505, 152, 4, 33, "Text",
      CellTags->{"b:12.0", "IPM3.4.2.1"}]},
  "IPM3.4.2.2"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[43685, 1285, 240, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
      CellTags->{
      "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
        "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],
    Cell[49600, 1514, 87, 1, 44, "Subsection",
      CellTags->{"b:12.1", "IPM3.4.2.2", "c:5"}],
    Cell[49690, 1517, 181, 5, 33, "TextFirst",
      CellTags->{"i:3", "b:12.1", "IPM3.4.2.2"}],
    Cell[49896, 1526, 126, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[50025, 1531, 123, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[50163, 1537, 475, 11, 71, "Text",
      CellTags->{"i:4", "b:12.1", "IPM3.4.2.2"}],
    Cell[50663, 1552, 132, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[50798, 1557, 124, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[50937, 1563, 312, 8, 52, "Text",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[51274, 1575, 131, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[51408, 1580, 139, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[51584, 1588, 112, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[51699, 1593, 702, 22, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[52416, 1618, 204, 5, 33, "Text",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[52623, 1625, 138, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[52764, 1630, 137, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[52904, 1635, 107, 1, 33, "TextFirst",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[53036, 1640, 122, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[53161, 1645, 201, 4, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[53377, 1652, 405, 15, 52, "TextFirst",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[53785, 1669, 253, 6, 52, "Text",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[54063, 1679, 136, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[54202, 1684, 107, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[54324, 1690, 266, 8, 33, "TextFirst",
      CellTags->{"i:5", "b:12.1", "IPM3.4.2.2"}],
    Cell[54615, 1702, 145, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[54763, 1707, 831, 28, 48, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[55609, 1738, 281, 8, 52, "TextFirst",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[55915, 1750, 146, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[56064, 1755, 130, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}]},
  "IPM3.4.2.3"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[43685, 1285, 240, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
      CellTags->{
      "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
        "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],
    Cell[56243, 1764, 89, 1, 44, "Subsection",
      CellTags->{"b:12.2", "IPM3.4.2.3", "c:6"}],
    Cell[56335, 1767, 433, 8, 71, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[56793, 1779, 110, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[56906, 1784, 111, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57032, 1790, 158, 5, 33, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57215, 1799, 115, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57333, 1804, 111, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57459, 1810, 374, 7, 71, "Text",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57858, 1821, 117, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57978, 1826, 106, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[58099, 1832, 200, 7, 33, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[58324, 1843, 135, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[58462, 1848, 106, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[58583, 1854, 382, 11, 52, "Text",
      CellTags->{"i:6", "b:12.2", "IPM3.4.2.3"}],
    Cell[58990, 1869, 111, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59104, 1874, 135, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59276, 1882, 112, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59391, 1887, 172, 4, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59578, 1894, 292, 6, 52, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59895, 1904, 115, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60013, 1909, 116, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60144, 1915, 252, 8, 52, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60399, 1925, 129, 3, 30, "Input",
      CellTags->{"i:7", "b:12.2", "IPM3.4.2.3"}],
    Cell[60553, 1932, 115, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60671, 1937, 125, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60811, 1943, 212, 4, 52, "TextFirst",
      CellTags->{"i:8", "b:12.2", "IPM3.4.2.3"}],
    Cell[61026, 1949, 104, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[61155, 1956, 102, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[61260, 1961, 89, 1, 20, "Print",
      CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}],
    Cell[61352, 1964, 300, 8, 20, "Print",
      CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}],
    Cell[61667, 1975, 154, 5, 33, "TextFirst",
      CellTags->{"i:9", "b:12.2", "IPM3.4.2.3"}],
    Cell[61824, 1982, 105, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[61932, 1987, 183, 6, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[62140, 1997, 102, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[62245, 2002, 351, 6, 19, "Message",
      CellTags->{"b:12.2", "IPM3.4.2.3"}]},
  "IPM3.4.2.4"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[43685, 1285, 240, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
      CellTags->{
      "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
        "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],
    Cell[62645, 2014, 72, 1, 44, "Subsection",
      CellTags->{"b:12.3", "IPM3.4.2.4", "c:7"}],
    Cell[62720, 2017, 280, 8, 52, "TextFirst",
      CellTags->{"i:10", "b:12.3", "IPM3.4.2.4"}],
    Cell[63025, 2029, 116, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63144, 2034, 107, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63266, 2040, 316, 13, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63585, 2055, 262, 9, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63872, 2068, 115, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63990, 2073, 107, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[64112, 2079, 269, 6, 52, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[64406, 2089, 119, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[64528, 2094, 107, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[64650, 2100, 633, 25, 52, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[65308, 2129, 121, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[65432, 2134, 99, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[65546, 2140, 302, 11, 52, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[65851, 2153, 238, 7, 52, "Text",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66114, 2164, 112, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66229, 2169, 110, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66354, 2175, 137, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66516, 2181, 128, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66647, 2186, 120, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66782, 2192, 88, 1, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66895, 2197, 131, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[67029, 2202, 107, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[67151, 2208, 784, 24, 90, "Text",
      CellTags->{"i:11", "b:12.3", "IPM3.4.2.4"}],
    Cell[67960, 2236, 120, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68083, 2241, 141, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68239, 2247, 246, 8, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68510, 2259, 125, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68638, 2264, 141, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68794, 2270, 495, 20, 52, "Text",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69314, 2294, 122, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69439, 2299, 114, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69568, 2305, 253, 8, 52, "Text",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69846, 2317, 125, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69974, 2322, 115, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70104, 2328, 241, 7, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70370, 2339, 136, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70509, 2344, 104, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70628, 2350, 275, 9, 52, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70928, 2363, 133, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[71064, 2368, 121, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}]},
  "IPM3.4.2.5"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[43685, 1285, 240, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
      CellTags->{
      "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
        "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],
    Cell[71234, 2377, 82, 1, 44, "Subsection",
      CellTags->{"b:12.4", "IPM3.4.2.5", "c:8"}],
    Cell[71319, 2380, 311, 8, 52, "TextFirst",
      CellTags->{"i:12", "b:12.4", "IPM3.4.2.5"}],
    Cell[71655, 2392, 123, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[71781, 2397, 157, 4, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[71953, 2404, 226, 6, 52, "TextFirst",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[72204, 2414, 126, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[72333, 2419, 149, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[72497, 2425, 232, 6, 52, "TextFirst",
      CellTags->{"i:13", "b:12.4", "IPM3.4.2.5"}],
    Cell[72754, 2435, 129, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[72886, 2440, 125, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73026, 2446, 123, 3, 33, "Text",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73174, 2453, 154, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73331, 2458, 133, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73501, 2466, 135, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73639, 2471, 119, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73773, 2477, 224, 5, 52, "TextFirst",
      CellTags->{"i:14", "b:12.4", "IPM3.4.2.5"}],
    Cell[74022, 2486, 139, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[74164, 2491, 133, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}]},
  "IPM3.4.2.6"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[43685, 1285, 240, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
      CellTags->{
      "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
        "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],
    Cell[74346, 2500, 84, 1, 77, "ExerciseSubsection",
      CellTags->{"b:12.4", "IPM3.4.2.6", "c:9"}],
    Cell[74455, 2505, 267, 7, 46, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[74725, 2514, 145, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[74873, 2519, 152, 5, 27, "ExercisesNoLabel",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[75040, 2527, 216, 6, 46, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[75259, 2535, 380, 10, 65, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[75664, 2549, 282, 7, 46, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[75971, 2560, 120, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[76094, 2565, 119, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[76228, 2571, 283, 9, 46, "ExercisesNoLabel",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[76548, 2585, 317, 11, 27, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[76890, 2600, 117, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[77010, 2605, 128, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[77153, 2611, 168, 5, 27, "ExercisesNoLabel",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[77336, 2619, 386, 13, 46, "ExerciseItem",
      CellTags->{"b:13", "IPM3.4.2.6"}],
    Cell[77747, 2636, 920, 31, 65, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6", "i:59"}],
    Cell[78670, 2669, 266, 6, 38, "DisplayFormula",
      PageBreakAbove->False,
      PageBreakBelow->True,
      CellTags->{"b:12.4", "IPM3.4.2.6"}]},
  "IPM3.4.3.1"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[78997, 2682, 152, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 2}},
      CellTags->{"b:13", "IPM3.4.3.1", "IPM3.4.3.2", "c:10"}],
    Cell[79152, 2686, 819, 14, 128, "TextFirst",
      CellTags->{"i:15", "b:13", "IPM3.4.3.1"}],
    Cell[79974, 2702, 257, 9, 33, "Text",
      CellTags->{"i:16", "b:13", "IPM3.4.3.1"}],
    Cell[80256, 2715, 106, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[80365, 2720, 107, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[80487, 2726, 221, 7, 33, "TextFirst",
      CellTags->{"i:17", "b:13", "IPM3.4.3.1"}],
    Cell[80733, 2737, 110, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[80846, 2742, 139, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81000, 2748, 193, 6, 33, "TextFirst",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81218, 2758, 116, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81337, 2763, 329, 6, 48, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81681, 2772, 242, 9, 33, "TextFirst",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81948, 2785, 147, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[82098, 2790, 170, 4, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[82283, 2797, 362, 7, 71, "Text",
      CellTags->{"i:18", "b:13", "IPM3.4.3.1"}],
    Cell[82670, 2808, 114, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[82787, 2813, 113, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[82915, 2819, 661, 18, 90, "Text",
      CellTags->{"i:19", "b:13", "IPM3.4.3.1"}],
    Cell[83601, 2841, 114, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[83718, 2846, 113, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[83846, 2852, 222, 7, 33, "TextFirst",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84093, 2863, 118, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84214, 2868, 142, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84371, 2874, 280, 6, 52, "TextFirst",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84676, 2884, 124, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84803, 2889, 133, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84973, 2897, 127, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[85103, 2902, 114, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}]},
  "IPM3.4.3.2"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[78997, 2682, 152, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 2}},
      CellTags->{"b:13", "IPM3.4.3.1", "IPM3.4.3.2", "c:10"}],
    Cell[85254, 2910, 83, 1, 77, "ExerciseSubsection",
      CellTags->{"b:13", "IPM3.4.3.2", "c:11"}],
    Cell[85340, 2913, 309, 7, 46, "ExerciseItem",
      CellTags->{"b:13", "IPM3.4.3.2"}],
    Cell[85652, 2922, 152, 4, 27, "ExerciseItem",
      CellTags->{"b:13", "IPM3.4.3.2"}],
    Cell[85807, 2928, 534, 20, 46, "ExerciseItem",
      CellTags->{"b:13", "IPM3.4.3.2"}]},
  "IPM3.4.4.0"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[86390, 2954, 174, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 3}},
      CellTags->{
      "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", "c:12"}],
    Cell[86567, 2958, 503, 8, 90, "TextFirst",
      CellTags->"IPM3.4.4.0"]},
  "IPM3.4.4.1"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[86390, 2954, 174, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 3}},
      CellTags->{
      "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", "c:12"}],
    Cell[87095, 2970, 90, 1, 44, "Subsection",
      CellTags->{"b:14.0", "IPM3.4.4.1", "c:13"}],
    Cell[87188, 2973, 834, 18, 128, "TextFirst",
      CellTags->{"i:20", "b:14.0", "IPM3.4.4.1"}],
    Cell[88025, 2993, 213, 4, 52, "Text",
      CellTags->{"i:21", "b:14.0", "IPM3.4.4.1"}],
    Cell[88241, 2999, 703, 31, 42, "Program",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[88947, 3032, 656, 14, 90, "Text",
      CellTags->{"i:22", "b:14.0", "IPM3.4.4.1"}],
    Cell[89606, 3048, 610, 10, 109, "Text",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[90219, 3060, 613, 10, 109, "Text",
      CellTags->{"i:23", "b:14.0", "IPM3.4.4.1"}],
    Cell[90835, 3072, 262, 6, 52, "Text",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[91100, 3080, 113, 3, 30, "Input",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[91216, 3085, 191, 4, 52, "Text",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[91432, 3093, 104, 3, 30, "Input",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[91539, 3098, 98, 3, 29, "Output",
      CellTags->{"b:14.0", "IPM3.4.4.1"}]},
  "IPM3.4.4.2"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[86390, 2954, 174, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 3}},
      CellTags->{
      "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", "c:12"}],
    Cell[91686, 3107, 88, 1, 44, "Subsection",
      CellTags->{"b:14.1", "IPM3.4.4.2", "c:14"}],
    Cell[91777, 3110, 683, 16, 90, "TextFirst",
      CellTags->{"i:24", "b:14.1", "IPM3.4.4.2"}],
    Cell[92485, 3130, 113, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[92601, 3135, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[92731, 3141, 148, 4, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[92904, 3149, 103, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93010, 3154, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93162, 3162, 101, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93266, 3167, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93418, 3175, 101, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93522, 3180, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93652, 3186, 237, 6, 52, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93914, 3196, 120, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[94037, 3201, 673, 20, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[94725, 3224, 329, 6, 71, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95079, 3234, 134, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95216, 3239, 101, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95332, 3245, 208, 6, 33, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95565, 3255, 199, 7, 27, "ItemizedText",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95789, 3266, 122, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95914, 3271, 127, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96090, 3280, 172, 5, 27, "ItemizedText",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96287, 3289, 109, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96399, 3294, 102, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96516, 3300, 133, 3, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96652, 3305, 141, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96818, 3312, 134, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96955, 3317, 102, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97072, 3323, 212, 4, 52, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97309, 3331, 148, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97460, 3336, 106, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97581, 3342, 182, 5, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97788, 3351, 157, 4, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97948, 3357, 2568, 70, 124, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[100531, 3430, 128, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[100662, 3434, 413, 15, 50, "BulletedList",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101100, 3453, 135, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101238, 3458, 121, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101374, 3464, 217, 6, 50, "BulletedList",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101616, 3474, 103, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101722, 3479, 118, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101855, 3485, 203, 6, 31, "BulletedList",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102083, 3495, 104, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102190, 3500, 106, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102311, 3506, 86, 1, 33, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102400, 3509, 150, 3, 30, "Input",
      CellTags->{"i:25", "b:14.1", "IPM3.4.4.2"}],
    Cell[102553, 3514, 227, 6, 52, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102805, 3524, 134, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102942, 3529, 114, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[103071, 3535, 401, 7, 71, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[103475, 3544, 469, 15, 52, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[103969, 3563, 260, 6, 70, "Input",
      CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}],
    Cell[104232, 3571, 1278, 21, 62, "Output",
      CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}],
    Cell[105525, 3595, 322, 8, 52, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[105850, 3605, 413, 9, 71, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[106288, 3618, 128, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[106419, 3623, 137, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[106571, 3629, 460, 9, 71, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[107056, 3642, 178, 5, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[107237, 3649, 1312, 21, 72, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[108564, 3673, 190, 4, 52, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[108779, 3681, 109, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[108891, 3686, 1270, 21, 62, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[110176, 3710, 66, 1, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[110245, 3713, 401, 9, 71, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[110649, 3724, 472, 9, 90, "Text",
      CellTags->{"i:27", "b:14.1", "IPM3.4.4.2"}],
    Cell[111146, 3737, 108, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111257, 3742, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111387, 3748, 111, 1, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111523, 3753, 121, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111647, 3758, 119, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111781, 3764, 329, 8, 52, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112135, 3776, 108, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112246, 3781, 121, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112382, 3787, 450, 10, 71, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112857, 3801, 109, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112969, 3806, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[113099, 3812, 265, 5, 52, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[113367, 3819, 180, 4, 30, "Input",
      CellTags->{"i:28", "b:14.1", "IPM3.4.4.2"}],
    Cell[113572, 3827, 113, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[113688, 3832, 1270, 21, 62, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[114973, 3856, 265, 5, 52, "Text",
      CellTags->{"i:30", "b:14.1", "IPM3.4.4.2"}],
    Cell[115241, 3863, 176, 4, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[115420, 3869, 169, 4, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[115592, 3875, 706, 19, 90, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[116323, 3898, 189, 4, 50, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[116515, 3904, 125, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[116655, 3910, 576, 16, 71, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117234, 3928, 108, 2, 42, "Program",
      Evaluatable->False,
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117345, 3932, 190, 5, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117538, 3939, 130, 2, 42, "Program",
      Evaluatable->False,
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117671, 3943, 170, 5, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117844, 3950, 174, 4, 30, "Input",
      CellTags->{"i:29", "b:14.1", "IPM3.4.4.2"}],
    Cell[118021, 3956, 77, 1, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[118123, 3961, 103, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[118229, 3966, 201, 4, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[118445, 3973, 74, 1, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}]},
  "IPM3.4.4.3"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[86390, 2954, 174, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 3}},
      CellTags->{
      "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", "c:12"}],
    Cell[118568, 3980, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:14.1", "IPM3.4.4.3", "c:15"}],
    Cell[118678, 3985, 632, 17, 84, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[119313, 4004, 142, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[119480, 4011, 102, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[119585, 4016, 108, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[119708, 4022, 875, 29, 65, "ExercisesNoLabel",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[120608, 4055, 145, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[120756, 4060, 105, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[120888, 4067, 348, 11, 46, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[121239, 4080, 575, 21, 27, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3", "i:57"}],
    Cell[121817, 4103, 524, 14, 84, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[122344, 4119, 167, 4, 27, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[122514, 4125, 362, 11, 46, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}]},
  "IPM3.4.5.0"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[122925, 4142, 208, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
      CellTags->{
      "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "c:16"}],
    Cell[123136, 4148, 609, 17, 90, "TextFirst",
      CellTags->"IPM3.4.5.0"]},
  "IPM3.4.5.1"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[122925, 4142, 208, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
      CellTags->{
      "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "c:16"}],
    Cell[123770, 4169, 86, 1, 44, "Subsection",
      CellTags->{"b:15.0", "IPM3.4.5.1", "c:17"}],
    Cell[123859, 4172, 327, 7, 52, "TextFirst",
      CellTags->{"i:31", "b:15.0", "IPM3.4.5.1"}],
    Cell[124189, 4181, 1309, 45, 43, "Program",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[125501, 4228, 663, 14, 90, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[126167, 4244, 226, 6, 52, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[126396, 4252, 262, 6, 70, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[126661, 4260, 168, 4, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[126832, 4266, 165, 4, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[127000, 4272, 164, 4, 33, "TextFirst",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[127167, 4278, 128, 3, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[127298, 4283, 130, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[127431, 4287, 507, 11, 150, "Input",
      CellTags->{"i:32", "b:15.0", "IPM3.4.5.1"}],
    Cell[127941, 4300, 88, 1, 33, "TextFirst",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[128054, 4305, 102, 3, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[128159, 4310, 201, 4, 29, "Output",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[128375, 4317, 591, 13, 90, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[128969, 4332, 496, 9, 90, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[129468, 4343, 417, 7, 90, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[129910, 4354, 108, 3, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[130021, 4359, 96, 1, 20, "Print",
      CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}],
    Cell[130120, 4362, 2082, 38, 53, "Print",
      CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}],
    Cell[132217, 4403, 325, 6, 71, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[132545, 4411, 312, 7, 52, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}]},
  "IPM3.4.5.2"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[122925, 4142, 208, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
      CellTags->{
      "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "c:16"}],
    Cell[132894, 4423, 100, 1, 44, "Subsection",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2", "c:18"}],
    Cell[132997, 4426, 627, 14, 90, "TextFirst",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[133627, 4442, 1348, 44, 61, "Program",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[134978, 4488, 652, 15, 90, "TextFirst",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135633, 4505, 194, 7, 33, "Text",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135830, 4514, 114, 3, 30, "Input",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135947, 4519, 535, 11, 130, "Input",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[136485, 4532, 617, 16, 90, "Text",
      CellTags->{"b:15.1", "IPM3.4.5.2"}],
    Cell[137105, 4550, 497, 10, 90, "Text",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}]},
  "IPM3.4.5.3"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[122925, 4142, 208, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
      CellTags->{
      "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "c:16"}],
    Cell[137639, 4565, 92, 1, 44, "Subsection",
      CellTags->{"b:15.2", "IPM3.4.5.3", "c:19"}],
    Cell[137734, 4568, 655, 19, 90, "TextFirst",
      CellTags->{"i:34", "b:15.2", "IPM3.4.5.3"}],
    Cell[138414, 4591, 182, 4, 70, "Input",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[138599, 4597, 363, 6, 19, "Message",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[138965, 4605, 333, 9, 29, "Output",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[139313, 4617, 159, 5, 33, "TextFirst",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[139497, 4626, 111, 3, 30, "Input",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[139611, 4631, 100, 3, 29, "Output",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[139726, 4637, 279, 11, 33, "TextFirst",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[140008, 4650, 415, 11, 71, "Text",
      CellTags->{"b:15.2", "IPM3.4.5.3"}]},
  "IPM3.4.5.4"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[122925, 4142, 208, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
      CellTags->{
      "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "c:16"}],
    Cell[140460, 4666, 94, 1, 44, "Subsection",
      CellTags->{"b:15.3", "IPM3.4.5.4", "c:20"}],
    Cell[140557, 4669, 362, 7, 71, "TextFirst",
      CellTags->{"i:35", "b:15.3", "IPM3.4.5.4"}],
    Cell[140922, 4678, 150, 5, 33, "Text",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141075, 4685, 106, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141184, 4690, 192, 6, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141379, 4698, 170, 4, 70, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141552, 4704, 237, 8, 33, "TextFirst",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141814, 4716, 97, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141914, 4721, 98, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142049, 4729, 100, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142152, 4734, 98, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142265, 4740, 199, 6, 33, "TextFirst",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142467, 4748, 172, 4, 70, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142642, 4754, 192, 7, 33, "TextFirst",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142859, 4765, 102, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142964, 4770, 89, 1, 20, "Print",
      CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}],
    Cell[143056, 4773, 460, 13, 20, "Print",
      CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}],
    Cell[143531, 4789, 240, 9, 33, "TextFirst",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[143774, 4800, 106, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[143905, 4807, 100, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[144008, 4812, 99, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.4"}]},
  "IPM3.4.5.5"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[122925, 4142, 208, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
      CellTags->{
      "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "c:16"}],
    Cell[144156, 4821, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:15.3", "IPM3.4.5.5", "c:21"}],
    Cell[144244, 4824, 414, 12, 46, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[144661, 4838, 1167, 33, 122, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5", "i:58"}],
    Cell[145853, 4875, 441, 12, 65, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[146319, 4891, 121, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[146443, 4896, 99, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[146557, 4902, 115, 1, 27, "ExercisesNoLabel",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[146675, 4905, 346, 11, 46, "ExercisesNoLabel",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[147036, 4919, 185, 5, 27, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[147224, 4926, 940, 34, 65, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[148167, 4962, 1069, 39, 46, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[149261, 5005, 1072, 34, 103, "ExerciseItem",
      CellTags->{"i:53", "b:15.3", "IPM3.4.5.5"}],
    Cell[150358, 5043, 118, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[150479, 5048, 109, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.5"}]},
  "IPM3.4.6.1"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[150661, 5059, 147, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 5}},
      CellTags->{"b:16", "IPM3.4.6.1", "IPM3.4.6.2", "c:22"}],
    Cell[150811, 5063, 638, 17, 90, "TextFirst",
      CellTags->{"i:36", "b:16", "IPM3.4.6.1"}],
    Cell[151452, 5082, 1174, 39, 90, "Text",
      CellTags->{"i:37", "b:16", "IPM3.4.6.1"}],
    Cell[152629, 5123, 184, 5, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[152838, 5132, 110, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[152951, 5137, 111, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[153077, 5143, 536, 12, 90, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[153616, 5157, 105, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"i:38", "b:16", "IPM3.4.6.1"}],
    Cell[153724, 5162, 192, 7, 33, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[153919, 5171, 305, 7, 52, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154249, 5182, 117, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154369, 5187, 96, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154480, 5193, 240, 5, 52, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154745, 5202, 108, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154856, 5207, 96, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154967, 5213, 249, 5, 52, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155219, 5220, 120, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155364, 5227, 103, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155470, 5232, 96, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155581, 5238, 309, 8, 52, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155893, 5248, 78, 1, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155974, 5251, 120, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[156097, 5256, 335, 9, 52, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[156457, 5269, 113, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[156573, 5274, 122, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[156710, 5280, 209, 5, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1", "i:60"}],
    Cell[156944, 5289, 242, 5, 70, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[157189, 5296, 246, 5, 48, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[157450, 5304, 105, 1, 33, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[157580, 5309, 147, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[157730, 5314, 29192, 905, 181, 16645, 745, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[186937, 6222, 699, 25, 71, "TextFirst",
      CellTags->{"i:39", "b:16", "IPM3.4.6.1"}],
    Cell[187639, 6249, 281, 7, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[187945, 6260, 143, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[188091, 6265, 222, 4, 48, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[188328, 6272, 402, 9, 71, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[188733, 6283, 124, 3, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[188860, 6288, 150, 3, 30, "Input",
      CellTags->{"i:40", "b:16", "IPM3.4.6.1"}],
    Cell[189035, 6295, 119, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189157, 6300, 100, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189272, 6306, 97, 1, 33, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189394, 6311, 156, 4, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189553, 6317, 100, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189690, 6325, 146, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189839, 6330, 100, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189954, 6336, 144, 4, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190101, 6342, 124, 2, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190250, 6348, 120, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190373, 6353, 107, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190495, 6359, 97, 1, 33, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190617, 6364, 172, 4, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190792, 6370, 107, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190936, 6378, 152, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191091, 6383, 107, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191213, 6389, 226, 6, 52, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191464, 6399, 116, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191583, 6404, 105, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191703, 6410, 438, 11, 71, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[192166, 6425, 147, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[192316, 6430, 105, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}]},
  "IPM3.4.6.2"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[150661, 5059, 147, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 5}},
      CellTags->{"b:16", "IPM3.4.6.1", "IPM3.4.6.2", "c:22"}],
    Cell[192458, 6438, 83, 1, 77, "ExerciseSubsection",
      CellTags->{"b:16", "IPM3.4.6.2", "c:23"}],
    Cell[192544, 6441, 138, 3, 27, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[192685, 6446, 379, 10, 46, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[193089, 6460, 668, 17, 84, "ExerciseItem",
      CellTags->{"i:41", "b:16", "IPM3.4.6.2"}],
    Cell[193760, 6479, 122, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[193907, 6486, 121, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[194031, 6491, 238, 5, 48, "Output",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[194318, 6502, 190, 5, 27, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[194511, 6509, 152, 5, 48, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[194678, 6517, 362, 10, 46, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[195043, 6529, 369, 12, 46, "ExerciseItem",
      CellTags->{"i:42", "b:16", "IPM3.4.6.2"}],
    Cell[195415, 6543, 406, 14, 46, "ExerciseItem",
      CellTags->{"i:43", "b:16", "IPM3.4.6.2"}],
    Cell[195824, 6559, 766, 25, 84, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[196615, 6588, 541, 18, 46, "ExerciseItem",
      CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],
    Cell[197159, 6608, 125, 2, 27, "ExercisesNoLabel",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[197309, 6614, 237, 5, 70, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[197549, 6621, 211, 5, 48, "Output",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[197775, 6629, 231, 7, 27, "ExercisesNoLabel",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[198009, 6638, 130, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[198154, 6644, 222, 5, 46, "ExerciseItem",
      CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],
    Cell[198401, 6653, 674, 15, 84, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2", "i:55"}],
    Cell[199100, 6672, 101, 3, 30, "Input",
      CellTags->"IPM3.4.6.2"],
    Cell[199204, 6677, 104, 3, 42, "Output",
      CellTags->"IPM3.4.6.2"],
    Cell[199323, 6683, 551, 15, 65, "ExercisesNoLabel",
      CellTags->"IPM3.4.6.2"],
    Cell[199899, 6702, 106, 3, 30, "Input",
      CellTags->"IPM3.4.6.2"],
    Cell[200008, 6707, 118, 3, 29, "Output",
      CellTags->"IPM3.4.6.2"],
    Cell[200163, 6715, 92, 3, 30, "Input",
      CellTags->"IPM3.4.6.2"],
    Cell[200258, 6720, 112, 3, 29, "Output",
      CellTags->"IPM3.4.6.2"]},
  "IPM3.4.7.0"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[200443, 6731, 185, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 6}},
      CellTags->{
      "IPM3.4.7.0", "IPM3.4.7.1", "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4", 
        "c:24"}],
    Cell[200631, 6737, 657, 11, 109, "TextFirst",
      CellTags->"IPM3.4.7.0"]},
  "IPM3.4.7.1"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[200443, 6731, 185, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 6}},
      CellTags->{
      "IPM3.4.7.0", "IPM3.4.7.1", "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4", 
        "c:24"}],
    Cell[201313, 6752, 84, 1, 44, "Subsection",
      CellTags->{"b:17.0", "IPM3.4.7.1", "c:25"}],
    Cell[201400, 6755, 634, 10, 109, "TextFirst",
      CellTags->{"i:45", "b:17.0", "IPM3.4.7.1"}],
    Cell[202037, 6767, 448, 18, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[202510, 6789, 134, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[202647, 6794, 115, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[202777, 6800, 166, 5, 33, "TextFirst",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[202968, 6809, 147, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203118, 6814, 129, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203262, 6820, 170, 6, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203457, 6830, 110, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203570, 6835, 97, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203682, 6841, 228, 7, 33, "TextFirst",
      CellTags->{"i:46", "b:17.0", "IPM3.4.7.1"}],
    Cell[203913, 6850, 199, 4, 50, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204137, 6858, 144, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204284, 6863, 97, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204396, 6869, 302, 8, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204701, 6879, 88, 1, 33, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204814, 6884, 146, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204963, 6889, 136, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[205114, 6895, 608, 23, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1", "i:56"}],
    Cell[205747, 6922, 139, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[205889, 6927, 105, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206009, 6933, 166, 7, 33, "TextFirst",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206200, 6944, 118, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206321, 6949, 111, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206447, 6955, 113, 1, 33, "TextFirst",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206585, 6960, 109, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206697, 6965, 97, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206809, 6971, 403, 11, 52, "TextFirst",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207215, 6984, 242, 5, 70, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207482, 6993, 146, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207631, 6998, 98, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207744, 7004, 205, 4, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207952, 7010, 140, 3, 31, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[208095, 7015, 140, 3, 31, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[208260, 7022, 141, 3, 30, "Input",
      CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}],
    Cell[208404, 7027, 142, 3, 29, "Output",
      CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}],
    Cell[208583, 7035, 134, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[208720, 7040, 121, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[208856, 7046, 358, 8, 71, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[209217, 7056, 266, 5, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}]},
  "IPM3.4.7.2"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[200443, 6731, 185, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 6}},
      CellTags->{
      "IPM3.4.7.0", "IPM3.4.7.1", "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4", 
        "c:24"}],
    Cell[209520, 7066, 88, 1, 44, "Subsection",
      CellTags->{"b:17.1", "IPM3.4.7.2", "c:26"}],
    Cell[209611, 7069, 223, 4, 52, "TextFirst",
      CellTags->{"i:49", "b:17.1", "IPM3.4.7.2"}],
    Cell[209837, 7075, 385, 7, 103, "Quote",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[210225, 7084, 821, 14, 128, "Text",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[211049, 7100, 429, 9, 71, "Text",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[211503, 7113, 135, 3, 30, "Input",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[211641, 7118, 106, 3, 29, "Output",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[211762, 7124, 621, 18, 90, "TextFirst",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[212386, 7144, 178, 4, 30, "Input",
      CellTags->{"i:50", "b:17.1", "IPM3.4.7.2"}],
    Cell[212567, 7150, 224, 6, 52, "TextFirst",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[212816, 7160, 115, 3, 30, "Input",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[212934, 7165, 100, 3, 29, "Output",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[213049, 7171, 448, 12, 71, "TextFirst",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[213522, 7187, 138, 3, 30, "Input",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[213663, 7192, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[214031, 7205, 382, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[214416, 7218, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[214784, 7231, 376, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[215163, 7244, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[215531, 7257, 370, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[215904, 7270, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[216272, 7283, 364, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[216639, 7296, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[217007, 7309, 358, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[217368, 7322, 157, 4, 26, "Output",
      CellTags->{"b:17.1", "IPM3.4.7.2"}]},
  "IPM3.4.7.3"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[200443, 6731, 185, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 6}},
      CellTags->{
      "IPM3.4.7.0", "IPM3.4.7.1", "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4", 
        "c:24"}],
    Cell[217574, 7332, 81, 1, 44, "Subsection",
      CellTags->{"b:17.2", "IPM3.4.7.3", "c:27"}],
    Cell[217658, 7335, 273, 5, 52, "TextFirst",
      CellTags->{"i:51", "b:17.2", "IPM3.4.7.3"}],
    Cell[217956, 7344, 134, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218093, 7349, 127, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218235, 7355, 486, 18, 52, "TextFirst",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218746, 7377, 111, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218860, 7382, 98, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218973, 7388, 133, 3, 33, "TextFirst",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219131, 7395, 171, 4, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219305, 7401, 109, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219429, 7407, 383, 12, 52, "Text",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219837, 7423, 139, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219979, 7428, 109, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[220103, 7434, 337, 8, 52, "TextFirst",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[220465, 7446, 123, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[220591, 7451, 100, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[220706, 7457, 332, 8, 52, "TextFirst",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221063, 7469, 143, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221209, 7474, 100, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221346, 7482, 125, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221474, 7487, 100, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221589, 7493, 354, 6, 52, "TextFirst",
      CellTags->{"i:52", "b:17.2", "IPM3.4.7.3"}],
    Cell[221946, 7501, 199, 5, 50, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[222170, 7510, 115, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[222288, 7515, 100, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}]},
  "IPM3.4.7.4"->{
    Cell[1754, 51, 551, 11, 29, "TitleBar",
      CellTags->{
      "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
        "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
        "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
        "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
        "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],
    Cell[200443, 6731, 185, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 6}},
      CellTags->{
      "IPM3.4.7.0", "IPM3.4.7.1", "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4", 
        "c:24"}],
    Cell[222437, 7524, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:17.2", "IPM3.4.7.4", "c:28"}],
    Cell[222525, 7527, 261, 6, 46, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[222789, 7535, 524, 10, 84, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[223316, 7547, 446, 7, 84, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[223787, 7558, 145, 3, 27, "ExerciseSubItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[223935, 7563, 166, 4, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224126, 7571, 132, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224261, 7576, 120, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224430, 7585, 158, 4, 27, "ExerciseSubItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224591, 7591, 243, 5, 70, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224859, 7600, 126, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224988, 7605, 137, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[225140, 7611, 140, 3, 27, "ExercisesNoLabel",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[225283, 7616, 193, 6, 66, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[225513, 7627, 918, 27, 103, "ExerciseSubItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[226434, 7656, 291, 7, 70, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[226750, 7667, 115, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[226868, 7672, 112, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[226995, 7678, 140, 3, 27, "ExercisesNoLabel",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[227138, 7683, 283, 7, 90, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[227424, 7692, 858, 21, 122, "ExercisesNoLabel",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[228297, 7716, 334, 9, 46, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[228634, 7727, 272, 9, 46, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[228931, 7740, 150, 4, 27, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[229084, 7746, 175, 4, 50, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[229284, 7754, 111, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[229398, 7759, 109, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[229534, 7766, 548, 15, 65, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[230085, 7783, 274, 8, 46, "ExerciseItem",
      CellTags->{"i:54", "b:17.2", "IPM3.4.7.4"}]},
  "c:1"->{
    Cell[2330, 66, 159, 6, 96, "Chapter",
      CounterAssignments->{{"Chapter", 3}},
      CellTags->{"IPM3.4.0", "c:1"}]},
  "b:11"->{
    Cell[3366, 98, 128, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 0}},
      CellTags->{"b:11", "IPM3.4.1", "c:2"}],
    Cell[3497, 102, 277, 5, 52, "TextFirst",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[3777, 109, 119, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[3899, 114, 119, 3, 33, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4043, 121, 104, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4150, 126, 119, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4306, 134, 110, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4419, 139, 119, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4553, 145, 395, 8, 71, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[4973, 157, 129, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5105, 162, 217, 4, 48, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5337, 169, 167, 5, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5529, 178, 121, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5653, 183, 173, 4, 46, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[5841, 190, 183, 4, 52, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[6049, 198, 138, 3, 33, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[6190, 203, 33273, 947, 181, 14713, 712, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[39466, 1152, 188, 5, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[39669, 1160, 412, 7, 90, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40106, 1171, 141, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40250, 1176, 136, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40423, 1184, 292, 7, 90, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40718, 1193, 137, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[40870, 1199, 374, 13, 52, "TextFirst",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41247, 1214, 107, 1, 33, "Text",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41379, 1219, 144, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41526, 1224, 137, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41678, 1230, 149, 4, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41852, 1238, 109, 3, 30, "Input",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[41964, 1243, 137, 3, 29, "Output",
      CellTags->{"b:11", "IPM3.4.1"}],
    Cell[42116, 1249, 603, 12, 109, "Text",
      CellTags->{"i:1", "b:11", "IPM3.4.1"}],
    Cell[42722, 1263, 926, 17, 147, "Text",
      CellTags->{"b:11", "IPM3.4.1"}]},
  "c:2"->{
    Cell[3366, 98, 128, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 0}},
      CellTags->{"b:11", "IPM3.4.1", "c:2"}]},
  "i:1"->{
    Cell[42116, 1249, 603, 12, 109, "Text",
      CellTags->{"i:1", "b:11", "IPM3.4.1"}]},
  "c:3"->{
    Cell[43685, 1285, 240, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
      CellTags->{
      "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
        "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}]},
  "b:12.0"->{
    Cell[44904, 1321, 94, 2, 44, "Subsection",
      PageBreakAbove->True,
      CellTags->{"b:12.0", "IPM3.4.2.1", "c:4"}],
    Cell[45001, 1325, 154, 4, 33, "TextFirst",
      CellTags->{"i:2", "b:12.0", "IPM3.4.2.1"}],
    Cell[45180, 1333, 123, 3, 42, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45306, 1338, 123, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45444, 1344, 203, 6, 33, "TextFirst",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45672, 1354, 112, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45787, 1359, 114, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[45916, 1365, 361, 14, 52, "Text",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[46302, 1383, 113, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[46418, 1388, 115, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[46548, 1394, 371, 13, 52, "TextFirst",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[46922, 1409, 435, 13, 71, "Text",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47382, 1426, 133, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47518, 1431, 120, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47653, 1437, 138, 3, 33, "TextFirst",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47816, 1444, 134, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[47953, 1449, 124, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48092, 1455, 308, 7, 52, "Text",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48403, 1464, 126, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48532, 1469, 197, 4, 52, "TextFirst",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48732, 1475, 112, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[48847, 1480, 285, 10, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[49157, 1494, 106, 3, 30, "Input",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[49266, 1499, 130, 3, 29, "Output",
      CellTags->{"b:12.0", "IPM3.4.2.1"}],
    Cell[49411, 1505, 152, 4, 33, "Text",
      CellTags->{"b:12.0", "IPM3.4.2.1"}]},
  "c:4"->{
    Cell[44904, 1321, 94, 2, 44, "Subsection",
      PageBreakAbove->True,
      CellTags->{"b:12.0", "IPM3.4.2.1", "c:4"}]},
  "i:2"->{
    Cell[45001, 1325, 154, 4, 33, "TextFirst",
      CellTags->{"i:2", "b:12.0", "IPM3.4.2.1"}]},
  "b:12.1"->{
    Cell[49600, 1514, 87, 1, 44, "Subsection",
      CellTags->{"b:12.1", "IPM3.4.2.2", "c:5"}],
    Cell[49690, 1517, 181, 5, 33, "TextFirst",
      CellTags->{"i:3", "b:12.1", "IPM3.4.2.2"}],
    Cell[49896, 1526, 126, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[50025, 1531, 123, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[50163, 1537, 475, 11, 71, "Text",
      CellTags->{"i:4", "b:12.1", "IPM3.4.2.2"}],
    Cell[50663, 1552, 132, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[50798, 1557, 124, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[50937, 1563, 312, 8, 52, "Text",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[51274, 1575, 131, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[51408, 1580, 139, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[51584, 1588, 112, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[51699, 1593, 702, 22, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[52416, 1618, 204, 5, 33, "Text",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[52623, 1625, 138, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[52764, 1630, 137, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[52904, 1635, 107, 1, 33, "TextFirst",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[53036, 1640, 122, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[53161, 1645, 201, 4, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[53377, 1652, 405, 15, 52, "TextFirst",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[53785, 1669, 253, 6, 52, "Text",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[54063, 1679, 136, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[54202, 1684, 107, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[54324, 1690, 266, 8, 33, "TextFirst",
      CellTags->{"i:5", "b:12.1", "IPM3.4.2.2"}],
    Cell[54615, 1702, 145, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[54763, 1707, 831, 28, 48, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[55609, 1738, 281, 8, 52, "TextFirst",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[55915, 1750, 146, 3, 30, "Input",
      CellTags->{"b:12.1", "IPM3.4.2.2"}],
    Cell[56064, 1755, 130, 3, 29, "Output",
      CellTags->{"b:12.1", "IPM3.4.2.2"}]},
  "c:5"->{
    Cell[49600, 1514, 87, 1, 44, "Subsection",
      CellTags->{"b:12.1", "IPM3.4.2.2", "c:5"}]},
  "i:3"->{
    Cell[49690, 1517, 181, 5, 33, "TextFirst",
      CellTags->{"i:3", "b:12.1", "IPM3.4.2.2"}]},
  "i:4"->{
    Cell[50163, 1537, 475, 11, 71, "Text",
      CellTags->{"i:4", "b:12.1", "IPM3.4.2.2"}]},
  "i:5"->{
    Cell[54324, 1690, 266, 8, 33, "TextFirst",
      CellTags->{"i:5", "b:12.1", "IPM3.4.2.2"}]},
  "b:12.2"->{
    Cell[56243, 1764, 89, 1, 44, "Subsection",
      CellTags->{"b:12.2", "IPM3.4.2.3", "c:6"}],
    Cell[56335, 1767, 433, 8, 71, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[56793, 1779, 110, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[56906, 1784, 111, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57032, 1790, 158, 5, 33, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57215, 1799, 115, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57333, 1804, 111, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57459, 1810, 374, 7, 71, "Text",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57858, 1821, 117, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[57978, 1826, 106, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[58099, 1832, 200, 7, 33, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[58324, 1843, 135, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[58462, 1848, 106, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[58583, 1854, 382, 11, 52, "Text",
      CellTags->{"i:6", "b:12.2", "IPM3.4.2.3"}],
    Cell[58990, 1869, 111, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59104, 1874, 135, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59276, 1882, 112, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59391, 1887, 172, 4, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59578, 1894, 292, 6, 52, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[59895, 1904, 115, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60013, 1909, 116, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60144, 1915, 252, 8, 52, "TextFirst",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60399, 1925, 129, 3, 30, "Input",
      CellTags->{"i:7", "b:12.2", "IPM3.4.2.3"}],
    Cell[60553, 1932, 115, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60671, 1937, 125, 3, 29, "Output",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[60811, 1943, 212, 4, 52, "TextFirst",
      CellTags->{"i:8", "b:12.2", "IPM3.4.2.3"}],
    Cell[61026, 1949, 104, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[61155, 1956, 102, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[61260, 1961, 89, 1, 20, "Print",
      CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}],
    Cell[61352, 1964, 300, 8, 20, "Print",
      CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}],
    Cell[61667, 1975, 154, 5, 33, "TextFirst",
      CellTags->{"i:9", "b:12.2", "IPM3.4.2.3"}],
    Cell[61824, 1982, 105, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[61932, 1987, 183, 6, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[62140, 1997, 102, 3, 30, "Input",
      CellTags->{"b:12.2", "IPM3.4.2.3"}],
    Cell[62245, 2002, 351, 6, 19, "Message",
      CellTags->{"b:12.2", "IPM3.4.2.3"}]},
  "c:6"->{
    Cell[56243, 1764, 89, 1, 44, "Subsection",
      CellTags->{"b:12.2", "IPM3.4.2.3", "c:6"}]},
  "i:6"->{
    Cell[58583, 1854, 382, 11, 52, "Text",
      CellTags->{"i:6", "b:12.2", "IPM3.4.2.3"}]},
  "i:7"->{
    Cell[60399, 1925, 129, 3, 30, "Input",
      CellTags->{"i:7", "b:12.2", "IPM3.4.2.3"}]},
  "i:8"->{
    Cell[60811, 1943, 212, 4, 52, "TextFirst",
      CellTags->{"i:8", "b:12.2", "IPM3.4.2.3"}]},
  "Info3302592863-4682924"->{
    Cell[61260, 1961, 89, 1, 20, "Print",
      CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}],
    Cell[61352, 1964, 300, 8, 20, "Print",
      CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}]},
  "i:9"->{
    Cell[61667, 1975, 154, 5, 33, "TextFirst",
      CellTags->{"i:9", "b:12.2", "IPM3.4.2.3"}]},
  "b:12.3"->{
    Cell[62645, 2014, 72, 1, 44, "Subsection",
      CellTags->{"b:12.3", "IPM3.4.2.4", "c:7"}],
    Cell[62720, 2017, 280, 8, 52, "TextFirst",
      CellTags->{"i:10", "b:12.3", "IPM3.4.2.4"}],
    Cell[63025, 2029, 116, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63144, 2034, 107, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63266, 2040, 316, 13, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63585, 2055, 262, 9, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63872, 2068, 115, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[63990, 2073, 107, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[64112, 2079, 269, 6, 52, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[64406, 2089, 119, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[64528, 2094, 107, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[64650, 2100, 633, 25, 52, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[65308, 2129, 121, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[65432, 2134, 99, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[65546, 2140, 302, 11, 52, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[65851, 2153, 238, 7, 52, "Text",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66114, 2164, 112, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66229, 2169, 110, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66354, 2175, 137, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66516, 2181, 128, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66647, 2186, 120, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66782, 2192, 88, 1, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[66895, 2197, 131, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[67029, 2202, 107, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[67151, 2208, 784, 24, 90, "Text",
      CellTags->{"i:11", "b:12.3", "IPM3.4.2.4"}],
    Cell[67960, 2236, 120, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68083, 2241, 141, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68239, 2247, 246, 8, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68510, 2259, 125, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68638, 2264, 141, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[68794, 2270, 495, 20, 52, "Text",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69314, 2294, 122, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69439, 2299, 114, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69568, 2305, 253, 8, 52, "Text",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69846, 2317, 125, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[69974, 2322, 115, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70104, 2328, 241, 7, 33, "TextFirst",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70370, 2339, 136, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70509, 2344, 104, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70628, 2350, 275, 9, 52, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[70928, 2363, 133, 3, 30, "Input",
      CellTags->{"b:12.3", "IPM3.4.2.4"}],
    Cell[71064, 2368, 121, 3, 29, "Output",
      CellTags->{"b:12.3", "IPM3.4.2.4"}]},
  "c:7"->{
    Cell[62645, 2014, 72, 1, 44, "Subsection",
      CellTags->{"b:12.3", "IPM3.4.2.4", "c:7"}]},
  "i:10"->{
    Cell[62720, 2017, 280, 8, 52, "TextFirst",
      CellTags->{"i:10", "b:12.3", "IPM3.4.2.4"}]},
  "i:11"->{
    Cell[67151, 2208, 784, 24, 90, "Text",
      CellTags->{"i:11", "b:12.3", "IPM3.4.2.4"}]},
  "b:12.4"->{
    Cell[71234, 2377, 82, 1, 44, "Subsection",
      CellTags->{"b:12.4", "IPM3.4.2.5", "c:8"}],
    Cell[71319, 2380, 311, 8, 52, "TextFirst",
      CellTags->{"i:12", "b:12.4", "IPM3.4.2.5"}],
    Cell[71655, 2392, 123, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[71781, 2397, 157, 4, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[71953, 2404, 226, 6, 52, "TextFirst",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[72204, 2414, 126, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[72333, 2419, 149, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[72497, 2425, 232, 6, 52, "TextFirst",
      CellTags->{"i:13", "b:12.4", "IPM3.4.2.5"}],
    Cell[72754, 2435, 129, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[72886, 2440, 125, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73026, 2446, 123, 3, 33, "Text",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73174, 2453, 154, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73331, 2458, 133, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73501, 2466, 135, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73639, 2471, 119, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[73773, 2477, 224, 5, 52, "TextFirst",
      CellTags->{"i:14", "b:12.4", "IPM3.4.2.5"}],
    Cell[74022, 2486, 139, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[74164, 2491, 133, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.5"}],
    Cell[74346, 2500, 84, 1, 77, "ExerciseSubsection",
      CellTags->{"b:12.4", "IPM3.4.2.6", "c:9"}],
    Cell[74455, 2505, 267, 7, 46, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[74725, 2514, 145, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[74873, 2519, 152, 5, 27, "ExercisesNoLabel",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[75040, 2527, 216, 6, 46, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[75259, 2535, 380, 10, 65, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[75664, 2549, 282, 7, 46, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[75971, 2560, 120, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[76094, 2565, 119, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[76228, 2571, 283, 9, 46, "ExercisesNoLabel",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[76548, 2585, 317, 11, 27, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[76890, 2600, 117, 3, 30, "Input",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[77010, 2605, 128, 3, 29, "Output",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[77153, 2611, 168, 5, 27, "ExercisesNoLabel",
      CellTags->{"b:12.4", "IPM3.4.2.6"}],
    Cell[77747, 2636, 920, 31, 65, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6", "i:59"}],
    Cell[78670, 2669, 266, 6, 38, "DisplayFormula",
      PageBreakAbove->False,
      PageBreakBelow->True,
      CellTags->{"b:12.4", "IPM3.4.2.6"}]},
  "c:8"->{
    Cell[71234, 2377, 82, 1, 44, "Subsection",
      CellTags->{"b:12.4", "IPM3.4.2.5", "c:8"}]},
  "i:12"->{
    Cell[71319, 2380, 311, 8, 52, "TextFirst",
      CellTags->{"i:12", "b:12.4", "IPM3.4.2.5"}]},
  "i:13"->{
    Cell[72497, 2425, 232, 6, 52, "TextFirst",
      CellTags->{"i:13", "b:12.4", "IPM3.4.2.5"}]},
  "i:14"->{
    Cell[73773, 2477, 224, 5, 52, "TextFirst",
      CellTags->{"i:14", "b:12.4", "IPM3.4.2.5"}]},
  "c:9"->{
    Cell[74346, 2500, 84, 1, 77, "ExerciseSubsection",
      CellTags->{"b:12.4", "IPM3.4.2.6", "c:9"}]},
  "b:13"->{
    Cell[77336, 2619, 386, 13, 46, "ExerciseItem",
      CellTags->{"b:13", "IPM3.4.2.6"}],
    Cell[78997, 2682, 152, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 2}},
      CellTags->{"b:13", "IPM3.4.3.1", "IPM3.4.3.2", "c:10"}],
    Cell[79152, 2686, 819, 14, 128, "TextFirst",
      CellTags->{"i:15", "b:13", "IPM3.4.3.1"}],
    Cell[79974, 2702, 257, 9, 33, "Text",
      CellTags->{"i:16", "b:13", "IPM3.4.3.1"}],
    Cell[80256, 2715, 106, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[80365, 2720, 107, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[80487, 2726, 221, 7, 33, "TextFirst",
      CellTags->{"i:17", "b:13", "IPM3.4.3.1"}],
    Cell[80733, 2737, 110, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[80846, 2742, 139, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81000, 2748, 193, 6, 33, "TextFirst",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81218, 2758, 116, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81337, 2763, 329, 6, 48, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81681, 2772, 242, 9, 33, "TextFirst",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[81948, 2785, 147, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[82098, 2790, 170, 4, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[82283, 2797, 362, 7, 71, "Text",
      CellTags->{"i:18", "b:13", "IPM3.4.3.1"}],
    Cell[82670, 2808, 114, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[82787, 2813, 113, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[82915, 2819, 661, 18, 90, "Text",
      CellTags->{"i:19", "b:13", "IPM3.4.3.1"}],
    Cell[83601, 2841, 114, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[83718, 2846, 113, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[83846, 2852, 222, 7, 33, "TextFirst",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84093, 2863, 118, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84214, 2868, 142, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84371, 2874, 280, 6, 52, "TextFirst",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84676, 2884, 124, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84803, 2889, 133, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[84973, 2897, 127, 3, 30, "Input",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[85103, 2902, 114, 3, 29, "Output",
      CellTags->{"b:13", "IPM3.4.3.1"}],
    Cell[85254, 2910, 83, 1, 77, "ExerciseSubsection",
      CellTags->{"b:13", "IPM3.4.3.2", "c:11"}],
    Cell[85340, 2913, 309, 7, 46, "ExerciseItem",
      CellTags->{"b:13", "IPM3.4.3.2"}],
    Cell[85652, 2922, 152, 4, 27, "ExerciseItem",
      CellTags->{"b:13", "IPM3.4.3.2"}],
    Cell[85807, 2928, 534, 20, 46, "ExerciseItem",
      CellTags->{"b:13", "IPM3.4.3.2"}]},
  "i:59"->{
    Cell[77747, 2636, 920, 31, 65, "ExerciseItem",
      CellTags->{"b:12.4", "IPM3.4.2.6", "i:59"}]},
  "c:10"->{
    Cell[78997, 2682, 152, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 2}},
      CellTags->{"b:13", "IPM3.4.3.1", "IPM3.4.3.2", "c:10"}]},
  "i:15"->{
    Cell[79152, 2686, 819, 14, 128, "TextFirst",
      CellTags->{"i:15", "b:13", "IPM3.4.3.1"}]},
  "i:16"->{
    Cell[79974, 2702, 257, 9, 33, "Text",
      CellTags->{"i:16", "b:13", "IPM3.4.3.1"}]},
  "i:17"->{
    Cell[80487, 2726, 221, 7, 33, "TextFirst",
      CellTags->{"i:17", "b:13", "IPM3.4.3.1"}]},
  "i:18"->{
    Cell[82283, 2797, 362, 7, 71, "Text",
      CellTags->{"i:18", "b:13", "IPM3.4.3.1"}]},
  "i:19"->{
    Cell[82915, 2819, 661, 18, 90, "Text",
      CellTags->{"i:19", "b:13", "IPM3.4.3.1"}]},
  "c:11"->{
    Cell[85254, 2910, 83, 1, 77, "ExerciseSubsection",
      CellTags->{"b:13", "IPM3.4.3.2", "c:11"}]},
  "c:12"->{
    Cell[86390, 2954, 174, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 3}},
      CellTags->{
      "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", "c:12"}]},
  "b:14.0"->{
    Cell[87095, 2970, 90, 1, 44, "Subsection",
      CellTags->{"b:14.0", "IPM3.4.4.1", "c:13"}],
    Cell[87188, 2973, 834, 18, 128, "TextFirst",
      CellTags->{"i:20", "b:14.0", "IPM3.4.4.1"}],
    Cell[88025, 2993, 213, 4, 52, "Text",
      CellTags->{"i:21", "b:14.0", "IPM3.4.4.1"}],
    Cell[88241, 2999, 703, 31, 42, "Program",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[88947, 3032, 656, 14, 90, "Text",
      CellTags->{"i:22", "b:14.0", "IPM3.4.4.1"}],
    Cell[89606, 3048, 610, 10, 109, "Text",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[90219, 3060, 613, 10, 109, "Text",
      CellTags->{"i:23", "b:14.0", "IPM3.4.4.1"}],
    Cell[90835, 3072, 262, 6, 52, "Text",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[91100, 3080, 113, 3, 30, "Input",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[91216, 3085, 191, 4, 52, "Text",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[91432, 3093, 104, 3, 30, "Input",
      CellTags->{"b:14.0", "IPM3.4.4.1"}],
    Cell[91539, 3098, 98, 3, 29, "Output",
      CellTags->{"b:14.0", "IPM3.4.4.1"}]},
  "c:13"->{
    Cell[87095, 2970, 90, 1, 44, "Subsection",
      CellTags->{"b:14.0", "IPM3.4.4.1", "c:13"}]},
  "i:20"->{
    Cell[87188, 2973, 834, 18, 128, "TextFirst",
      CellTags->{"i:20", "b:14.0", "IPM3.4.4.1"}]},
  "i:21"->{
    Cell[88025, 2993, 213, 4, 52, "Text",
      CellTags->{"i:21", "b:14.0", "IPM3.4.4.1"}]},
  "i:22"->{
    Cell[88947, 3032, 656, 14, 90, "Text",
      CellTags->{"i:22", "b:14.0", "IPM3.4.4.1"}]},
  "i:23"->{
    Cell[90219, 3060, 613, 10, 109, "Text",
      CellTags->{"i:23", "b:14.0", "IPM3.4.4.1"}]},
  "b:14.1"->{
    Cell[91686, 3107, 88, 1, 44, "Subsection",
      CellTags->{"b:14.1", "IPM3.4.4.2", "c:14"}],
    Cell[91777, 3110, 683, 16, 90, "TextFirst",
      CellTags->{"i:24", "b:14.1", "IPM3.4.4.2"}],
    Cell[92485, 3130, 113, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[92601, 3135, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[92731, 3141, 148, 4, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[92904, 3149, 103, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93010, 3154, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93162, 3162, 101, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93266, 3167, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93418, 3175, 101, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93522, 3180, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93652, 3186, 237, 6, 52, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[93914, 3196, 120, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[94037, 3201, 673, 20, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[94725, 3224, 329, 6, 71, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95079, 3234, 134, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95216, 3239, 101, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95332, 3245, 208, 6, 33, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95565, 3255, 199, 7, 27, "ItemizedText",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95789, 3266, 122, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[95914, 3271, 127, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96090, 3280, 172, 5, 27, "ItemizedText",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96287, 3289, 109, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96399, 3294, 102, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96516, 3300, 133, 3, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96652, 3305, 141, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96818, 3312, 134, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[96955, 3317, 102, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97072, 3323, 212, 4, 52, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97309, 3331, 148, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97460, 3336, 106, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97581, 3342, 182, 5, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97788, 3351, 157, 4, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[97948, 3357, 2568, 70, 124, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[100531, 3430, 128, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[100662, 3434, 413, 15, 50, "BulletedList",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101100, 3453, 135, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101238, 3458, 121, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101374, 3464, 217, 6, 50, "BulletedList",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101616, 3474, 103, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101722, 3479, 118, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[101855, 3485, 203, 6, 31, "BulletedList",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102083, 3495, 104, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102190, 3500, 106, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102311, 3506, 86, 1, 33, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102400, 3509, 150, 3, 30, "Input",
      CellTags->{"i:25", "b:14.1", "IPM3.4.4.2"}],
    Cell[102553, 3514, 227, 6, 52, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102805, 3524, 134, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[102942, 3529, 114, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[103071, 3535, 401, 7, 71, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[103475, 3544, 469, 15, 52, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[103969, 3563, 260, 6, 70, "Input",
      CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}],
    Cell[104232, 3571, 1278, 21, 62, "Output",
      CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}],
    Cell[105525, 3595, 322, 8, 52, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[105850, 3605, 413, 9, 71, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[106288, 3618, 128, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[106419, 3623, 137, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[106571, 3629, 460, 9, 71, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[107056, 3642, 178, 5, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[107237, 3649, 1312, 21, 72, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[108564, 3673, 190, 4, 52, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[108779, 3681, 109, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[108891, 3686, 1270, 21, 62, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[110176, 3710, 66, 1, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[110245, 3713, 401, 9, 71, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[110649, 3724, 472, 9, 90, "Text",
      CellTags->{"i:27", "b:14.1", "IPM3.4.4.2"}],
    Cell[111146, 3737, 108, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111257, 3742, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111387, 3748, 111, 1, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111523, 3753, 121, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111647, 3758, 119, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[111781, 3764, 329, 8, 52, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112135, 3776, 108, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112246, 3781, 121, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112382, 3787, 450, 10, 71, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112857, 3801, 109, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[112969, 3806, 115, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[113099, 3812, 265, 5, 52, "Text",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[113367, 3819, 180, 4, 30, "Input",
      CellTags->{"i:28", "b:14.1", "IPM3.4.4.2"}],
    Cell[113572, 3827, 113, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[113688, 3832, 1270, 21, 62, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[114973, 3856, 265, 5, 52, "Text",
      CellTags->{"i:30", "b:14.1", "IPM3.4.4.2"}],
    Cell[115241, 3863, 176, 4, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[115420, 3869, 169, 4, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[115592, 3875, 706, 19, 90, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[116323, 3898, 189, 4, 50, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[116515, 3904, 125, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[116655, 3910, 576, 16, 71, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117234, 3928, 108, 2, 42, "Program",
      Evaluatable->False,
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117345, 3932, 190, 5, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117538, 3939, 130, 2, 42, "Program",
      Evaluatable->False,
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117671, 3943, 170, 5, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[117844, 3950, 174, 4, 30, "Input",
      CellTags->{"i:29", "b:14.1", "IPM3.4.4.2"}],
    Cell[118021, 3956, 77, 1, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[118123, 3961, 103, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[118229, 3966, 201, 4, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[118445, 3973, 74, 1, 33, "TextFirst",
      CellTags->{"b:14.1", "IPM3.4.4.2"}],
    Cell[118568, 3980, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:14.1", "IPM3.4.4.3", "c:15"}],
    Cell[118678, 3985, 632, 17, 84, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[119313, 4004, 142, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[119480, 4011, 102, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[119585, 4016, 108, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[119708, 4022, 875, 29, 65, "ExercisesNoLabel",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[120608, 4055, 145, 3, 30, "Input",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[120756, 4060, 105, 3, 29, "Output",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[120888, 4067, 348, 11, 46, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[121239, 4080, 575, 21, 27, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3", "i:57"}],
    Cell[121817, 4103, 524, 14, 84, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[122344, 4119, 167, 4, 27, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}],
    Cell[122514, 4125, 362, 11, 46, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3"}]},
  "c:14"->{
    Cell[91686, 3107, 88, 1, 44, "Subsection",
      CellTags->{"b:14.1", "IPM3.4.4.2", "c:14"}]},
  "i:24"->{
    Cell[91777, 3110, 683, 16, 90, "TextFirst",
      CellTags->{"i:24", "b:14.1", "IPM3.4.4.2"}]},
  "i:25"->{
    Cell[102400, 3509, 150, 3, 30, "Input",
      CellTags->{"i:25", "b:14.1", "IPM3.4.4.2"}]},
  "i:26"->{
    Cell[103969, 3563, 260, 6, 70, "Input",
      CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}],
    Cell[104232, 3571, 1278, 21, 62, "Output",
      CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}]},
  "i:27"->{
    Cell[110649, 3724, 472, 9, 90, "Text",
      CellTags->{"i:27", "b:14.1", "IPM3.4.4.2"}]},
  "i:28"->{
    Cell[113367, 3819, 180, 4, 30, "Input",
      CellTags->{"i:28", "b:14.1", "IPM3.4.4.2"}]},
  "i:30"->{
    Cell[114973, 3856, 265, 5, 52, "Text",
      CellTags->{"i:30", "b:14.1", "IPM3.4.4.2"}]},
  "i:29"->{
    Cell[117844, 3950, 174, 4, 30, "Input",
      CellTags->{"i:29", "b:14.1", "IPM3.4.4.2"}]},
  "c:15"->{
    Cell[118568, 3980, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:14.1", "IPM3.4.4.3", "c:15"}]},
  "i:57"->{
    Cell[121239, 4080, 575, 21, 27, "ExerciseItem",
      CellTags->{"b:14.1", "IPM3.4.4.3", "i:57"}]},
  "c:16"->{
    Cell[122925, 4142, 208, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
      CellTags->{
      "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
        "IPM3.4.5.5", "c:16"}]},
  "b:15.0"->{
    Cell[123770, 4169, 86, 1, 44, "Subsection",
      CellTags->{"b:15.0", "IPM3.4.5.1", "c:17"}],
    Cell[123859, 4172, 327, 7, 52, "TextFirst",
      CellTags->{"i:31", "b:15.0", "IPM3.4.5.1"}],
    Cell[124189, 4181, 1309, 45, 43, "Program",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[125501, 4228, 663, 14, 90, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[126167, 4244, 226, 6, 52, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[126396, 4252, 262, 6, 70, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[126661, 4260, 168, 4, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[126832, 4266, 165, 4, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[127000, 4272, 164, 4, 33, "TextFirst",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[127167, 4278, 128, 3, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[127298, 4283, 130, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[127431, 4287, 507, 11, 150, "Input",
      CellTags->{"i:32", "b:15.0", "IPM3.4.5.1"}],
    Cell[127941, 4300, 88, 1, 33, "TextFirst",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[128054, 4305, 102, 3, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[128159, 4310, 201, 4, 29, "Output",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[128375, 4317, 591, 13, 90, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[128969, 4332, 496, 9, 90, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[129468, 4343, 417, 7, 90, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[129910, 4354, 108, 3, 30, "Input",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[130021, 4359, 96, 1, 20, "Print",
      CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}],
    Cell[130120, 4362, 2082, 38, 53, "Print",
      CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}],
    Cell[132217, 4403, 325, 6, 71, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}],
    Cell[132545, 4411, 312, 7, 52, "Text",
      CellTags->{"b:15.0", "IPM3.4.5.1"}]},
  "c:17"->{
    Cell[123770, 4169, 86, 1, 44, "Subsection",
      CellTags->{"b:15.0", "IPM3.4.5.1", "c:17"}]},
  "i:31"->{
    Cell[123859, 4172, 327, 7, 52, "TextFirst",
      CellTags->{"i:31", "b:15.0", "IPM3.4.5.1"}]},
  "i:32"->{
    Cell[127431, 4287, 507, 11, 150, "Input",
      CellTags->{"i:32", "b:15.0", "IPM3.4.5.1"}]},
  "Info3302592864-7158086"->{
    Cell[130021, 4359, 96, 1, 20, "Print",
      CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}],
    Cell[130120, 4362, 2082, 38, 53, "Print",
      CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}]},
  "i:33"->{
    Cell[132894, 4423, 100, 1, 44, "Subsection",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2", "c:18"}],
    Cell[132997, 4426, 627, 14, 90, "TextFirst",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[133627, 4442, 1348, 44, 61, "Program",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[134978, 4488, 652, 15, 90, "TextFirst",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135633, 4505, 194, 7, 33, "Text",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135830, 4514, 114, 3, 30, "Input",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135947, 4519, 535, 11, 130, "Input",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[137105, 4550, 497, 10, 90, "Text",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}]},
  "b:15.1"->{
    Cell[132894, 4423, 100, 1, 44, "Subsection",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2", "c:18"}],
    Cell[132997, 4426, 627, 14, 90, "TextFirst",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[133627, 4442, 1348, 44, 61, "Program",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[134978, 4488, 652, 15, 90, "TextFirst",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135633, 4505, 194, 7, 33, "Text",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135830, 4514, 114, 3, 30, "Input",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[135947, 4519, 535, 11, 130, "Input",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
    Cell[136485, 4532, 617, 16, 90, "Text",
      CellTags->{"b:15.1", "IPM3.4.5.2"}],
    Cell[137105, 4550, 497, 10, 90, "Text",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}]},
  "c:18"->{
    Cell[132894, 4423, 100, 1, 44, "Subsection",
      CellTags->{"i:33", "b:15.1", "IPM3.4.5.2", "c:18"}]},
  "b:15.2"->{
    Cell[137639, 4565, 92, 1, 44, "Subsection",
      CellTags->{"b:15.2", "IPM3.4.5.3", "c:19"}],
    Cell[137734, 4568, 655, 19, 90, "TextFirst",
      CellTags->{"i:34", "b:15.2", "IPM3.4.5.3"}],
    Cell[138414, 4591, 182, 4, 70, "Input",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[138599, 4597, 363, 6, 19, "Message",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[138965, 4605, 333, 9, 29, "Output",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[139313, 4617, 159, 5, 33, "TextFirst",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[139497, 4626, 111, 3, 30, "Input",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[139611, 4631, 100, 3, 29, "Output",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[139726, 4637, 279, 11, 33, "TextFirst",
      CellTags->{"b:15.2", "IPM3.4.5.3"}],
    Cell[140008, 4650, 415, 11, 71, "Text",
      CellTags->{"b:15.2", "IPM3.4.5.3"}]},
  "c:19"->{
    Cell[137639, 4565, 92, 1, 44, "Subsection",
      CellTags->{"b:15.2", "IPM3.4.5.3", "c:19"}]},
  "i:34"->{
    Cell[137734, 4568, 655, 19, 90, "TextFirst",
      CellTags->{"i:34", "b:15.2", "IPM3.4.5.3"}]},
  "b:15.3"->{
    Cell[140460, 4666, 94, 1, 44, "Subsection",
      CellTags->{"b:15.3", "IPM3.4.5.4", "c:20"}],
    Cell[140557, 4669, 362, 7, 71, "TextFirst",
      CellTags->{"i:35", "b:15.3", "IPM3.4.5.4"}],
    Cell[140922, 4678, 150, 5, 33, "Text",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141075, 4685, 106, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141184, 4690, 192, 6, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141379, 4698, 170, 4, 70, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141552, 4704, 237, 8, 33, "TextFirst",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141814, 4716, 97, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[141914, 4721, 98, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142049, 4729, 100, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142152, 4734, 98, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142265, 4740, 199, 6, 33, "TextFirst",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142467, 4748, 172, 4, 70, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142642, 4754, 192, 7, 33, "TextFirst",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142859, 4765, 102, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[142964, 4770, 89, 1, 20, "Print",
      CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}],
    Cell[143056, 4773, 460, 13, 20, "Print",
      CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}],
    Cell[143531, 4789, 240, 9, 33, "TextFirst",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[143774, 4800, 106, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[143905, 4807, 100, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[144008, 4812, 99, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.4"}],
    Cell[144156, 4821, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:15.3", "IPM3.4.5.5", "c:21"}],
    Cell[144244, 4824, 414, 12, 46, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[144661, 4838, 1167, 33, 122, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5", "i:58"}],
    Cell[145853, 4875, 441, 12, 65, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[146319, 4891, 121, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[146443, 4896, 99, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[146557, 4902, 115, 1, 27, "ExercisesNoLabel",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[146675, 4905, 346, 11, 46, "ExercisesNoLabel",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[147036, 4919, 185, 5, 27, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[147224, 4926, 940, 34, 65, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[148167, 4962, 1069, 39, 46, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[149261, 5005, 1072, 34, 103, "ExerciseItem",
      CellTags->{"i:53", "b:15.3", "IPM3.4.5.5"}],
    Cell[150358, 5043, 118, 3, 30, "Input",
      CellTags->{"b:15.3", "IPM3.4.5.5"}],
    Cell[150479, 5048, 109, 3, 29, "Output",
      CellTags->{"b:15.3", "IPM3.4.5.5"}]},
  "c:20"->{
    Cell[140460, 4666, 94, 1, 44, "Subsection",
      CellTags->{"b:15.3", "IPM3.4.5.4", "c:20"}]},
  "i:35"->{
    Cell[140557, 4669, 362, 7, 71, "TextFirst",
      CellTags->{"i:35", "b:15.3", "IPM3.4.5.4"}]},
  "Info3302592864-4738886"->{
    Cell[142964, 4770, 89, 1, 20, "Print",
      CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}],
    Cell[143056, 4773, 460, 13, 20, "Print",
      CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}]},
  "c:21"->{
    Cell[144156, 4821, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:15.3", "IPM3.4.5.5", "c:21"}]},
  "i:58"->{
    Cell[144661, 4838, 1167, 33, 122, "ExerciseItem",
      CellTags->{"b:15.3", "IPM3.4.5.5", "i:58"}]},
  "i:53"->{
    Cell[149261, 5005, 1072, 34, 103, "ExerciseItem",
      CellTags->{"i:53", "b:15.3", "IPM3.4.5.5"}]},
  "b:16"->{
    Cell[150661, 5059, 147, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 5}},
      CellTags->{"b:16", "IPM3.4.6.1", "IPM3.4.6.2", "c:22"}],
    Cell[150811, 5063, 638, 17, 90, "TextFirst",
      CellTags->{"i:36", "b:16", "IPM3.4.6.1"}],
    Cell[151452, 5082, 1174, 39, 90, "Text",
      CellTags->{"i:37", "b:16", "IPM3.4.6.1"}],
    Cell[152629, 5123, 184, 5, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[152838, 5132, 110, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[152951, 5137, 111, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[153077, 5143, 536, 12, 90, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[153616, 5157, 105, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"i:38", "b:16", "IPM3.4.6.1"}],
    Cell[153724, 5162, 192, 7, 33, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[153919, 5171, 305, 7, 52, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154249, 5182, 117, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154369, 5187, 96, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154480, 5193, 240, 5, 52, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154745, 5202, 108, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154856, 5207, 96, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[154967, 5213, 249, 5, 52, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155219, 5220, 120, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155364, 5227, 103, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155470, 5232, 96, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155581, 5238, 309, 8, 52, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155893, 5248, 78, 1, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[155974, 5251, 120, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[156097, 5256, 335, 9, 52, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[156457, 5269, 113, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[156573, 5274, 122, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[156710, 5280, 209, 5, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1", "i:60"}],
    Cell[156944, 5289, 242, 5, 70, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[157189, 5296, 246, 5, 48, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[157450, 5304, 105, 1, 33, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[157580, 5309, 147, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[157730, 5314, 29192, 905, 181, 16645, 745, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[186937, 6222, 699, 25, 71, "TextFirst",
      CellTags->{"i:39", "b:16", "IPM3.4.6.1"}],
    Cell[187639, 6249, 281, 7, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[187945, 6260, 143, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[188091, 6265, 222, 4, 48, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[188328, 6272, 402, 9, 71, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[188733, 6283, 124, 3, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[188860, 6288, 150, 3, 30, "Input",
      CellTags->{"i:40", "b:16", "IPM3.4.6.1"}],
    Cell[189035, 6295, 119, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189157, 6300, 100, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189272, 6306, 97, 1, 33, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189394, 6311, 156, 4, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189553, 6317, 100, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189690, 6325, 146, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189839, 6330, 100, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[189954, 6336, 144, 4, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190101, 6342, 124, 2, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190250, 6348, 120, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190373, 6353, 107, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190495, 6359, 97, 1, 33, "TextFirst",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190617, 6364, 172, 4, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190792, 6370, 107, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[190936, 6378, 152, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191091, 6383, 107, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191213, 6389, 226, 6, 52, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191464, 6399, 116, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191583, 6404, 105, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[191703, 6410, 438, 11, 71, "Text",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[192166, 6425, 147, 3, 31, "Input",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[192316, 6430, 105, 3, 29, "Output",
      CellTags->{"b:16", "IPM3.4.6.1"}],
    Cell[192458, 6438, 83, 1, 77, "ExerciseSubsection",
      CellTags->{"b:16", "IPM3.4.6.2", "c:23"}],
    Cell[192544, 6441, 138, 3, 27, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[192685, 6446, 379, 10, 46, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[193089, 6460, 668, 17, 84, "ExerciseItem",
      CellTags->{"i:41", "b:16", "IPM3.4.6.2"}],
    Cell[193760, 6479, 122, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[193907, 6486, 121, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[194031, 6491, 238, 5, 48, "Output",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[194318, 6502, 190, 5, 27, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[194511, 6509, 152, 5, 48, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[194678, 6517, 362, 10, 46, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[195043, 6529, 369, 12, 46, "ExerciseItem",
      CellTags->{"i:42", "b:16", "IPM3.4.6.2"}],
    Cell[195415, 6543, 406, 14, 46, "ExerciseItem",
      CellTags->{"i:43", "b:16", "IPM3.4.6.2"}],
    Cell[195824, 6559, 766, 25, 84, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[196615, 6588, 541, 18, 46, "ExerciseItem",
      CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],
    Cell[197159, 6608, 125, 2, 27, "ExercisesNoLabel",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[197309, 6614, 237, 5, 70, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[197549, 6621, 211, 5, 48, "Output",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[197775, 6629, 231, 7, 27, "ExercisesNoLabel",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[198009, 6638, 130, 3, 30, "Input",
      CellTags->{"b:16", "IPM3.4.6.2"}],
    Cell[198154, 6644, 222, 5, 46, "ExerciseItem",
      CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],
    Cell[198401, 6653, 674, 15, 84, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2", "i:55"}]},
  "c:22"->{
    Cell[150661, 5059, 147, 2, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 5}},
      CellTags->{"b:16", "IPM3.4.6.1", "IPM3.4.6.2", "c:22"}]},
  "i:36"->{
    Cell[150811, 5063, 638, 17, 90, "TextFirst",
      CellTags->{"i:36", "b:16", "IPM3.4.6.1"}]},
  "i:37"->{
    Cell[151452, 5082, 1174, 39, 90, "Text",
      CellTags->{"i:37", "b:16", "IPM3.4.6.1"}]},
  "i:38"->{
    Cell[153616, 5157, 105, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"i:38", "b:16", "IPM3.4.6.1"}]},
  "i:60"->{
    Cell[156710, 5280, 209, 5, 33, "Text",
      CellTags->{"b:16", "IPM3.4.6.1", "i:60"}]},
  "i:39"->{
    Cell[186937, 6222, 699, 25, 71, "TextFirst",
      CellTags->{"i:39", "b:16", "IPM3.4.6.1"}]},
  "i:40"->{
    Cell[188860, 6288, 150, 3, 30, "Input",
      CellTags->{"i:40", "b:16", "IPM3.4.6.1"}]},
  "c:23"->{
    Cell[192458, 6438, 83, 1, 77, "ExerciseSubsection",
      CellTags->{"b:16", "IPM3.4.6.2", "c:23"}]},
  "i:41"->{
    Cell[193089, 6460, 668, 17, 84, "ExerciseItem",
      CellTags->{"i:41", "b:16", "IPM3.4.6.2"}]},
  "i:42"->{
    Cell[195043, 6529, 369, 12, 46, "ExerciseItem",
      CellTags->{"i:42", "b:16", "IPM3.4.6.2"}]},
  "i:43"->{
    Cell[195415, 6543, 406, 14, 46, "ExerciseItem",
      CellTags->{"i:43", "b:16", "IPM3.4.6.2"}]},
  "i:44"->{
    Cell[196615, 6588, 541, 18, 46, "ExerciseItem",
      CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],
    Cell[198154, 6644, 222, 5, 46, "ExerciseItem",
      CellTags->{"i:44", "b:16", "IPM3.4.6.2"}]},
  "i:55"->{
    Cell[198401, 6653, 674, 15, 84, "ExerciseItem",
      CellTags->{"b:16", "IPM3.4.6.2", "i:55"}]},
  "c:24"->{
    Cell[200443, 6731, 185, 4, 29, "Section",
      CounterAssignments->{{"Chapter", 4}, {"Section", 6}},
      CellTags->{
      "IPM3.4.7.0", "IPM3.4.7.1", "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4", 
        "c:24"}]},
  "b:17.0"->{
    Cell[201313, 6752, 84, 1, 44, "Subsection",
      CellTags->{"b:17.0", "IPM3.4.7.1", "c:25"}],
    Cell[201400, 6755, 634, 10, 109, "TextFirst",
      CellTags->{"i:45", "b:17.0", "IPM3.4.7.1"}],
    Cell[202037, 6767, 448, 18, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[202510, 6789, 134, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[202647, 6794, 115, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[202777, 6800, 166, 5, 33, "TextFirst",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[202968, 6809, 147, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203118, 6814, 129, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203262, 6820, 170, 6, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203457, 6830, 110, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203570, 6835, 97, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[203682, 6841, 228, 7, 33, "TextFirst",
      CellTags->{"i:46", "b:17.0", "IPM3.4.7.1"}],
    Cell[203913, 6850, 199, 4, 50, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204137, 6858, 144, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204284, 6863, 97, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204396, 6869, 302, 8, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204701, 6879, 88, 1, 33, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204814, 6884, 146, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[204963, 6889, 136, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[205114, 6895, 608, 23, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1", "i:56"}],
    Cell[205747, 6922, 139, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[205889, 6927, 105, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206009, 6933, 166, 7, 33, "TextFirst",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206200, 6944, 118, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206321, 6949, 111, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206447, 6955, 113, 1, 33, "TextFirst",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206585, 6960, 109, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206697, 6965, 97, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[206809, 6971, 403, 11, 52, "TextFirst",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207215, 6984, 242, 5, 70, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207482, 6993, 146, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207631, 6998, 98, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207744, 7004, 205, 4, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[207952, 7010, 140, 3, 31, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[208095, 7015, 140, 3, 31, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[208260, 7022, 141, 3, 30, "Input",
      CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}],
    Cell[208404, 7027, 142, 3, 29, "Output",
      CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}],
    Cell[208583, 7035, 134, 3, 30, "Input",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[208720, 7040, 121, 3, 29, "Output",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[208856, 7046, 358, 8, 71, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}],
    Cell[209217, 7056, 266, 5, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1"}]},
  "c:25"->{
    Cell[201313, 6752, 84, 1, 44, "Subsection",
      CellTags->{"b:17.0", "IPM3.4.7.1", "c:25"}]},
  "i:45"->{
    Cell[201400, 6755, 634, 10, 109, "TextFirst",
      CellTags->{"i:45", "b:17.0", "IPM3.4.7.1"}]},
  "i:46"->{
    Cell[203682, 6841, 228, 7, 33, "TextFirst",
      CellTags->{"i:46", "b:17.0", "IPM3.4.7.1"}]},
  "i:56"->{
    Cell[205114, 6895, 608, 23, 52, "Text",
      CellTags->{"b:17.0", "IPM3.4.7.1", "i:56"}]},
  "i:48"->{
    Cell[208260, 7022, 141, 3, 30, "Input",
      CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}],
    Cell[208404, 7027, 142, 3, 29, "Output",
      CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}]},
  "b:17.1"->{
    Cell[209520, 7066, 88, 1, 44, "Subsection",
      CellTags->{"b:17.1", "IPM3.4.7.2", "c:26"}],
    Cell[209611, 7069, 223, 4, 52, "TextFirst",
      CellTags->{"i:49", "b:17.1", "IPM3.4.7.2"}],
    Cell[209837, 7075, 385, 7, 103, "Quote",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[210225, 7084, 821, 14, 128, "Text",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[211049, 7100, 429, 9, 71, "Text",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[211503, 7113, 135, 3, 30, "Input",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[211641, 7118, 106, 3, 29, "Output",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[211762, 7124, 621, 18, 90, "TextFirst",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[212386, 7144, 178, 4, 30, "Input",
      CellTags->{"i:50", "b:17.1", "IPM3.4.7.2"}],
    Cell[212567, 7150, 224, 6, 52, "TextFirst",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[212816, 7160, 115, 3, 30, "Input",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[212934, 7165, 100, 3, 29, "Output",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[213049, 7171, 448, 12, 71, "TextFirst",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[213522, 7187, 138, 3, 30, "Input",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[213663, 7192, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[214031, 7205, 382, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[214416, 7218, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[214784, 7231, 376, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[215163, 7244, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[215531, 7257, 370, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[215904, 7270, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[216272, 7283, 364, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[216639, 7296, 365, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[217007, 7309, 358, 11, 24, "Print",
      CellTags->{"b:17.1", "IPM3.4.7.2"}],
    Cell[217368, 7322, 157, 4, 26, "Output",
      CellTags->{"b:17.1", "IPM3.4.7.2"}]},
  "c:26"->{
    Cell[209520, 7066, 88, 1, 44, "Subsection",
      CellTags->{"b:17.1", "IPM3.4.7.2", "c:26"}]},
  "i:49"->{
    Cell[209611, 7069, 223, 4, 52, "TextFirst",
      CellTags->{"i:49", "b:17.1", "IPM3.4.7.2"}]},
  "i:50"->{
    Cell[212386, 7144, 178, 4, 30, "Input",
      CellTags->{"i:50", "b:17.1", "IPM3.4.7.2"}]},
  "b:17.2"->{
    Cell[217574, 7332, 81, 1, 44, "Subsection",
      CellTags->{"b:17.2", "IPM3.4.7.3", "c:27"}],
    Cell[217658, 7335, 273, 5, 52, "TextFirst",
      CellTags->{"i:51", "b:17.2", "IPM3.4.7.3"}],
    Cell[217956, 7344, 134, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218093, 7349, 127, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218235, 7355, 486, 18, 52, "TextFirst",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218746, 7377, 111, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218860, 7382, 98, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[218973, 7388, 133, 3, 33, "TextFirst",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219131, 7395, 171, 4, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219305, 7401, 109, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219429, 7407, 383, 12, 52, "Text",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219837, 7423, 139, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[219979, 7428, 109, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[220103, 7434, 337, 8, 52, "TextFirst",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[220465, 7446, 123, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[220591, 7451, 100, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[220706, 7457, 332, 8, 52, "TextFirst",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221063, 7469, 143, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221209, 7474, 100, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221346, 7482, 125, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221474, 7487, 100, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[221589, 7493, 354, 6, 52, "TextFirst",
      CellTags->{"i:52", "b:17.2", "IPM3.4.7.3"}],
    Cell[221946, 7501, 199, 5, 50, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[222170, 7510, 115, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[222288, 7515, 100, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.3"}],
    Cell[222437, 7524, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:17.2", "IPM3.4.7.4", "c:28"}],
    Cell[222525, 7527, 261, 6, 46, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[222789, 7535, 524, 10, 84, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[223316, 7547, 446, 7, 84, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[223787, 7558, 145, 3, 27, "ExerciseSubItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[223935, 7563, 166, 4, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224126, 7571, 132, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224261, 7576, 120, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224430, 7585, 158, 4, 27, "ExerciseSubItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224591, 7591, 243, 5, 70, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224859, 7600, 126, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[224988, 7605, 137, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[225140, 7611, 140, 3, 27, "ExercisesNoLabel",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[225283, 7616, 193, 6, 66, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[225513, 7627, 918, 27, 103, "ExerciseSubItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[226434, 7656, 291, 7, 70, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[226750, 7667, 115, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[226868, 7672, 112, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[226995, 7678, 140, 3, 27, "ExercisesNoLabel",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[227138, 7683, 283, 7, 90, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[227424, 7692, 858, 21, 122, "ExercisesNoLabel",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[228297, 7716, 334, 9, 46, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[228634, 7727, 272, 9, 46, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[228931, 7740, 150, 4, 27, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[229084, 7746, 175, 4, 50, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[229284, 7754, 111, 3, 30, "Input",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[229398, 7759, 109, 3, 29, "Output",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[229534, 7766, 548, 15, 65, "ExerciseItem",
      CellTags->{"b:17.2", "IPM3.4.7.4"}],
    Cell[230085, 7783, 274, 8, 46, "ExerciseItem",
      CellTags->{"i:54", "b:17.2", "IPM3.4.7.4"}]},
  "c:27"->{
    Cell[217574, 7332, 81, 1, 44, "Subsection",
      CellTags->{"b:17.2", "IPM3.4.7.3", "c:27"}]},
  "i:51"->{
    Cell[217658, 7335, 273, 5, 52, "TextFirst",
      CellTags->{"i:51", "b:17.2", "IPM3.4.7.3"}]},
  "i:52"->{
    Cell[221589, 7493, 354, 6, 52, "TextFirst",
      CellTags->{"i:52", "b:17.2", "IPM3.4.7.3"}]},
  "c:28"->{
    Cell[222437, 7524, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:17.2", "IPM3.4.7.4", "c:28"}]},
  "i:54"->{
    Cell[230085, 7783, 274, 8, 46, "ExerciseItem",
      CellTags->{"i:54", "b:17.2", "IPM3.4.7.4"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"IPM3.4.0", 240175, 7991},
  {"IPM3.4.1", 240900, 8005},
  {"IPM3.4.2.0", 244221, 8082},
  {"IPM3.4.2.1", 245068, 8098},
  {"IPM3.4.2.2", 248089, 8164},
  {"IPM3.4.2.3", 251324, 8234},
  {"IPM3.4.2.4", 255260, 8319},
  {"IPM3.4.2.5", 259778, 8419},
  {"IPM3.4.2.6", 262149, 8469},
  {"IPM3.4.3.1", 264583, 8520},
  {"IPM3.4.3.2", 267614, 8586},
  {"IPM3.4.4.0", 268682, 8606},
  {"IPM3.4.4.1", 269477, 8621},
  {"IPM3.4.4.2", 271278, 8658},
  {"IPM3.4.4.3", 280703, 8868},
  {"IPM3.4.5.0", 282640, 8907},
  {"IPM3.4.5.1", 283475, 8923},
  {"IPM3.4.5.2", 286357, 8984},
  {"IPM3.4.5.3", 287987, 9016},
  {"IPM3.4.5.4", 289650, 9050},
  {"IPM3.4.5.5", 292354, 9107},
  {"IPM3.4.6.1", 294447, 9149},
  {"IPM3.4.6.2", 300353, 9282},
  {"IPM3.4.7.0", 303545, 9350},
  {"IPM3.4.7.1", 304367, 9366},
  {"IPM3.4.7.2", 308787, 9463},
  {"IPM3.4.7.3", 311763, 9527},
  {"IPM3.4.7.4", 314739, 9591},
  {"c:1", 318153, 9663},
  {"b:11", 318290, 9667},
  {"c:2", 321101, 9736},
  {"i:1", 321261, 9740},
  {"c:3", 321363, 9743},
  {"b:12.0", 321623, 9749},
  {"c:4", 323889, 9802},
  {"i:2", 324026, 9806},
  {"b:12.1", 324138, 9809},
  {"c:5", 326618, 9866},
  {"i:3", 326727, 9869},
  {"i:4", 326836, 9872},
  {"i:5", 326941, 9875},
  {"b:12.2", 327053, 9878},
  {"c:6", 330234, 9950},
  {"i:6", 330343, 9953},
  {"i:7", 330448, 9956},
  {"i:8", 330553, 9959},
  {"Info3302592863-4682924", 330681, 9962},
  {"i:9", 330916, 9967},
  {"b:12.3", 331028, 9970},
  {"c:7", 334791, 10057},
  {"i:10", 334901, 10060},
  {"i:11", 335012, 10063},
  {"b:12.4", 335121, 10066},
  {"c:8", 338312, 10138},
  {"i:12", 338422, 10141},
  {"i:13", 338533, 10144},
  {"i:14", 338644, 10147},
  {"c:9", 338754, 10150},
  {"b:13", 338872, 10153},
  {"i:59", 341864, 10221},
  {"c:10", 341979, 10224},
  {"i:15", 342160, 10228},
  {"i:16", 342271, 10231},
  {"i:17", 342375, 10234},
  {"i:18", 342484, 10237},
  {"i:19", 342588, 10240},
  {"c:11", 342693, 10243},
  {"c:12", 342810, 10246},
  {"b:14.0", 343020, 10251},
  {"c:13", 344117, 10276},
  {"i:20", 344228, 10279},
  {"i:21", 344341, 10282},
  {"i:22", 344447, 10285},
  {"i:23", 344554, 10288},
  {"b:14.1", 344664, 10291},
  {"c:14", 354605, 10515},
  {"i:24", 354716, 10518},
  {"i:25", 354828, 10521},
  {"i:26", 354936, 10524},
  {"i:27", 355142, 10529},
  {"i:28", 355249, 10532},
  {"i:30", 355357, 10535},
  {"i:29", 355464, 10538},
  {"c:15", 355572, 10541},
  {"i:57", 355692, 10544},
  {"c:16", 355808, 10547},
  {"b:15.0", 356056, 10553},
  {"c:17", 358196, 10601},
  {"i:31", 358308, 10604},
  {"i:32", 358420, 10607},
  {"Info3302592864-7158086", 358548, 10610},
  {"i:33", 358788, 10615},
  {"b:15.1", 359591, 10632},
  {"c:18", 360479, 10651},
  {"b:15.2", 360602, 10654},
  {"c:19", 361523, 10675},
  {"i:34", 361635, 10678},
  {"b:15.3", 361750, 10681},
  {"c:20", 365050, 10753},
  {"i:35", 365162, 10756},
  {"Info3302592864-4738886", 365292, 10759},
  {"c:21", 365531, 10764},
  {"i:58", 365651, 10767},
  {"i:53", 365769, 10770},
  {"b:16", 365887, 10773},
  {"c:22", 373246, 10940},
  {"i:36", 373428, 10944},
  {"i:37", 373539, 10947},
  {"i:38", 373646, 10950},
  {"i:60", 373778, 10954},
  {"i:39", 373883, 10957},
  {"i:40", 373994, 10960},
  {"c:23", 374100, 10963},
  {"i:41", 374218, 10966},
  {"i:42", 374332, 10969},
  {"i:43", 374446, 10972},
  {"i:44", 374560, 10975},
  {"i:55", 374774, 10980},
  {"c:24", 374888, 10983},
  {"b:17.0", 375122, 10989},
  {"c:25", 378814, 11073},
  {"i:45", 378926, 11076},
  {"i:46", 379040, 11079},
  {"i:56", 379152, 11082},
  {"i:48", 379260, 11085},
  {"b:17.1", 379466, 11090},
  {"c:26", 381714, 11141},
  {"i:49", 381826, 11144},
  {"i:50", 381938, 11147},
  {"b:17.2", 382048, 11150},
  {"c:27", 386976, 11259},
  {"i:51", 387088, 11262},
  {"i:52", 387200, 11265},
  {"c:28", 387312, 11268},
  {"i:54", 387432, 11271}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 551, 11, 29, "TitleBar",
  CellTags->{
  "IPM3.4.0", "IPM3.4.1", "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", 
    "IPM3.4.2.3", "IPM3.4.2.4", "IPM3.4.2.5", "IPM3.4.2.6", "IPM3.4.3.1", 
    "IPM3.4.3.2", "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", 
    "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
    "IPM3.4.5.5", "IPM3.4.6.1", "IPM3.4.6.2", "IPM3.4.7.0", "IPM3.4.7.1", 
    "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4"}],

Cell[CellGroupData[{
Cell[2330, 66, 159, 6, 96, "Chapter",
  CounterAssignments->{{"Chapter", 3}},
  CellTags->{"IPM3.4.0", "c:1"}],
Cell[2492, 74, 849, 20, 132, "ChapterAbstract",
  CellTags->"IPM3.4.0"],

Cell[CellGroupData[{
Cell[3366, 98, 128, 2, 40, "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 0}},
  CellTags->{"b:11", "IPM3.4.1", "c:2"}],
Cell[3497, 102, 277, 5, 52, "TextFirst",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[3777, 109, 119, 3, 30, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[3899, 114, 119, 3, 33, "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{
Cell[4043, 121, 104, 3, 30, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[4150, 126, 119, 3, 29, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[4306, 134, 110, 3, 30, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[4419, 139, 119, 3, 29, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],
Cell[4553, 145, 395, 8, 71, "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{
Cell[4973, 157, 129, 3, 30, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[5105, 162, 217, 4, 48, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],
Cell[5337, 169, 167, 5, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{
Cell[5529, 178, 121, 3, 30, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[5653, 183, 173, 4, 46, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],
Cell[5841, 190, 183, 4, 52, "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{
Cell[6049, 198, 138, 3, 33, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[6190, 203, 33273, 947, 181, 14713, 712, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[39466, 1152, 188, 5, 29, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],
Cell[39669, 1160, 412, 7, 90, "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{
Cell[40106, 1171, 141, 3, 30, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[40250, 1176, 136, 3, 29, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[40423, 1184, 292, 7, 90, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[40718, 1193, 137, 3, 29, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],
Cell[40870, 1199, 374, 13, 52, "TextFirst",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[41247, 1214, 107, 1, 33, "Text",
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{
Cell[41379, 1219, 144, 3, 30, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[41526, 1224, 137, 3, 29, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],
Cell[41678, 1230, 149, 4, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:11", "IPM3.4.1"}],

Cell[CellGroupData[{
Cell[41852, 1238, 109, 3, 30, "Input",
  CellTags->{"b:11", "IPM3.4.1"}],
Cell[41964, 1243, 137, 3, 29, "Output",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],
Cell[42116, 1249, 603, 12, 109, "Text",
  CellTags->{"i:1", "b:11", "IPM3.4.1"}],
Cell[42722, 1263, 926, 17, 147, "Text",
  CellTags->{"b:11", "IPM3.4.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[43685, 1285, 240, 4, 29, "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 1}},
  CellTags->{
  "IPM3.4.2.0", "IPM3.4.2.1", "IPM3.4.2.2", "IPM3.4.2.3", "IPM3.4.2.4", 
    "IPM3.4.2.5", "IPM3.4.2.6", "c:3"}],
Cell[43928, 1291, 951, 26, 128, "TextFirst",
  CellTags->"IPM3.4.2.0"],

Cell[CellGroupData[{
Cell[44904, 1321, 94, 2, 44, "Subsection",
  PageBreakAbove->True,
  CellTags->{"b:12.0", "IPM3.4.2.1", "c:4"}],
Cell[45001, 1325, 154, 4, 33, "TextFirst",
  CellTags->{"i:2", "b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{
Cell[45180, 1333, 123, 3, 42, "Input",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[45306, 1338, 123, 3, 29, "Output",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],
Cell[45444, 1344, 203, 6, 33, "TextFirst",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{
Cell[45672, 1354, 112, 3, 30, "Input",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[45787, 1359, 114, 3, 29, "Output",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],
Cell[45916, 1365, 361, 14, 52, "Text",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{
Cell[46302, 1383, 113, 3, 30, "Input",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[46418, 1388, 115, 3, 29, "Output",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],
Cell[46548, 1394, 371, 13, 52, "TextFirst",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[46922, 1409, 435, 13, 71, "Text",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{
Cell[47382, 1426, 133, 3, 30, "Input",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[47518, 1431, 120, 3, 29, "Output",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],
Cell[47653, 1437, 138, 3, 33, "TextFirst",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{
Cell[47816, 1444, 134, 3, 30, "Input",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[47953, 1449, 124, 3, 29, "Output",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],
Cell[48092, 1455, 308, 7, 52, "Text",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[48403, 1464, 126, 3, 30, "Input",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[48532, 1469, 197, 4, 52, "TextFirst",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[48732, 1475, 112, 3, 30, "Input",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[48847, 1480, 285, 10, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:12.0", "IPM3.4.2.1"}],

Cell[CellGroupData[{
Cell[49157, 1494, 106, 3, 30, "Input",
  CellTags->{"b:12.0", "IPM3.4.2.1"}],
Cell[49266, 1499, 130, 3, 29, "Output",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],
Cell[49411, 1505, 152, 4, 33, "Text",
  CellTags->{"b:12.0", "IPM3.4.2.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[49600, 1514, 87, 1, 44, "Subsection",
  CellTags->{"b:12.1", "IPM3.4.2.2", "c:5"}],
Cell[49690, 1517, 181, 5, 33, "TextFirst",
  CellTags->{"i:3", "b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{
Cell[49896, 1526, 126, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[50025, 1531, 123, 3, 29, "Output",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],
Cell[50163, 1537, 475, 11, 71, "Text",
  CellTags->{"i:4", "b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{
Cell[50663, 1552, 132, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[50798, 1557, 124, 3, 29, "Output",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],
Cell[50937, 1563, 312, 8, 52, "Text",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{
Cell[51274, 1575, 131, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[51408, 1580, 139, 3, 29, "Output",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],

Cell[CellGroupData[{
Cell[51584, 1588, 112, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[51699, 1593, 702, 22, 29, "Output",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],
Cell[52416, 1618, 204, 5, 33, "Text",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[52623, 1625, 138, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[52764, 1630, 137, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[52904, 1635, 107, 1, 33, "TextFirst",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{
Cell[53036, 1640, 122, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[53161, 1645, 201, 4, 29, "Output",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],
Cell[53377, 1652, 405, 15, 52, "TextFirst",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[53785, 1669, 253, 6, 52, "Text",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{
Cell[54063, 1679, 136, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[54202, 1684, 107, 3, 29, "Output",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],
Cell[54324, 1690, 266, 8, 33, "TextFirst",
  CellTags->{"i:5", "b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{
Cell[54615, 1702, 145, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[54763, 1707, 831, 28, 48, "Output",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]],
Cell[55609, 1738, 281, 8, 52, "TextFirst",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],

Cell[CellGroupData[{
Cell[55915, 1750, 146, 3, 30, "Input",
  CellTags->{"b:12.1", "IPM3.4.2.2"}],
Cell[56064, 1755, 130, 3, 29, "Output",
  CellTags->{"b:12.1", "IPM3.4.2.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[56243, 1764, 89, 1, 44, "Subsection",
  CellTags->{"b:12.2", "IPM3.4.2.3", "c:6"}],
Cell[56335, 1767, 433, 8, 71, "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[56793, 1779, 110, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[56906, 1784, 111, 3, 29, "Output",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],
Cell[57032, 1790, 158, 5, 33, "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[57215, 1799, 115, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[57333, 1804, 111, 3, 29, "Output",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],
Cell[57459, 1810, 374, 7, 71, "Text",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[57858, 1821, 117, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[57978, 1826, 106, 3, 29, "Output",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],
Cell[58099, 1832, 200, 7, 33, "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[58324, 1843, 135, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[58462, 1848, 106, 3, 29, "Output",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],
Cell[58583, 1854, 382, 11, 52, "Text",
  CellTags->{"i:6", "b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[58990, 1869, 111, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[59104, 1874, 135, 3, 29, "Output",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],

Cell[CellGroupData[{
Cell[59276, 1882, 112, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[59391, 1887, 172, 4, 29, "Output",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],
Cell[59578, 1894, 292, 6, 52, "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[59895, 1904, 115, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[60013, 1909, 116, 3, 29, "Output",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],
Cell[60144, 1915, 252, 8, 52, "TextFirst",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[60399, 1925, 129, 3, 30, "Input",
  CellTags->{"i:7", "b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[60553, 1932, 115, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[60671, 1937, 125, 3, 29, "Output",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]],
Cell[60811, 1943, 212, 4, 52, "TextFirst",
  CellTags->{"i:8", "b:12.2", "IPM3.4.2.3"}],
Cell[61026, 1949, 104, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[61155, 1956, 102, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[61260, 1961, 89, 1, 20, "Print",
  CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}],
Cell[61352, 1964, 300, 8, 20, "Print",
  CellTags->{"b:12.2", "Info3302592863-4682924", "IPM3.4.2.3"}]
}, Closed]],
Cell[61667, 1975, 154, 5, 33, "TextFirst",
  CellTags->{"i:9", "b:12.2", "IPM3.4.2.3"}],
Cell[61824, 1982, 105, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[61932, 1987, 183, 6, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:12.2", "IPM3.4.2.3"}],

Cell[CellGroupData[{
Cell[62140, 1997, 102, 3, 30, "Input",
  CellTags->{"b:12.2", "IPM3.4.2.3"}],
Cell[62245, 2002, 351, 6, 19, "Message",
  CellTags->{"b:12.2", "IPM3.4.2.3"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[62645, 2014, 72, 1, 44, "Subsection",
  CellTags->{"b:12.3", "IPM3.4.2.4", "c:7"}],
Cell[62720, 2017, 280, 8, 52, "TextFirst",
  CellTags->{"i:10", "b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[63025, 2029, 116, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[63144, 2034, 107, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[63266, 2040, 316, 13, 33, "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[63585, 2055, 262, 9, 33, "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[63872, 2068, 115, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[63990, 2073, 107, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[64112, 2079, 269, 6, 52, "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[64406, 2089, 119, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[64528, 2094, 107, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[64650, 2100, 633, 25, 52, "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[65308, 2129, 121, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[65432, 2134, 99, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[65546, 2140, 302, 11, 52, "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[65851, 2153, 238, 7, 52, "Text",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[66114, 2164, 112, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[66229, 2169, 110, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[66354, 2175, 137, 2, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[66516, 2181, 128, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[66647, 2186, 120, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[66782, 2192, 88, 1, 33, "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[66895, 2197, 131, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[67029, 2202, 107, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[67151, 2208, 784, 24, 90, "Text",
  CellTags->{"i:11", "b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[67960, 2236, 120, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[68083, 2241, 141, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[68239, 2247, 246, 8, 33, "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[68510, 2259, 125, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[68638, 2264, 141, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[68794, 2270, 495, 20, 52, "Text",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[69314, 2294, 122, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[69439, 2299, 114, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[69568, 2305, 253, 8, 52, "Text",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[69846, 2317, 125, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[69974, 2322, 115, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[70104, 2328, 241, 7, 33, "TextFirst",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[70370, 2339, 136, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[70509, 2344, 104, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]],
Cell[70628, 2350, 275, 9, 52, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:12.3", "IPM3.4.2.4"}],

Cell[CellGroupData[{
Cell[70928, 2363, 133, 3, 30, "Input",
  CellTags->{"b:12.3", "IPM3.4.2.4"}],
Cell[71064, 2368, 121, 3, 29, "Output",
  CellTags->{"b:12.3", "IPM3.4.2.4"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[71234, 2377, 82, 1, 44, "Subsection",
  CellTags->{"b:12.4", "IPM3.4.2.5", "c:8"}],
Cell[71319, 2380, 311, 8, 52, "TextFirst",
  CellTags->{"i:12", "b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{
Cell[71655, 2392, 123, 3, 30, "Input",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],
Cell[71781, 2397, 157, 4, 29, "Output",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],
Cell[71953, 2404, 226, 6, 52, "TextFirst",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{
Cell[72204, 2414, 126, 3, 30, "Input",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],
Cell[72333, 2419, 149, 3, 29, "Output",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],
Cell[72497, 2425, 232, 6, 52, "TextFirst",
  CellTags->{"i:13", "b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{
Cell[72754, 2435, 129, 3, 30, "Input",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],
Cell[72886, 2440, 125, 3, 29, "Output",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],
Cell[73026, 2446, 123, 3, 33, "Text",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{
Cell[73174, 2453, 154, 3, 30, "Input",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],
Cell[73331, 2458, 133, 3, 29, "Output",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],

Cell[CellGroupData[{
Cell[73501, 2466, 135, 3, 30, "Input",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],
Cell[73639, 2471, 119, 3, 29, "Output",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]],
Cell[73773, 2477, 224, 5, 52, "TextFirst",
  CellTags->{"i:14", "b:12.4", "IPM3.4.2.5"}],

Cell[CellGroupData[{
Cell[74022, 2486, 139, 3, 30, "Input",
  CellTags->{"b:12.4", "IPM3.4.2.5"}],
Cell[74164, 2491, 133, 3, 29, "Output",
  CellTags->{"b:12.4", "IPM3.4.2.5"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[74346, 2500, 84, 1, 77, "ExerciseSubsection",
  CellTags->{"b:12.4", "IPM3.4.2.6", "c:9"}],

Cell[CellGroupData[{
Cell[74455, 2505, 267, 7, 46, "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],
Cell[74725, 2514, 145, 3, 30, "Input",
  Evaluatable->False,
  CellTags->{"b:12.4", "IPM3.4.2.6"}],
Cell[74873, 2519, 152, 5, 27, "ExercisesNoLabel",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],
Cell[75040, 2527, 216, 6, 46, "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],
Cell[75259, 2535, 380, 10, 65, "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[CellGroupData[{
Cell[75664, 2549, 282, 7, 46, "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[CellGroupData[{
Cell[75971, 2560, 120, 3, 30, "Input",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],
Cell[76094, 2565, 119, 3, 29, "Output",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],
Cell[76228, 2571, 283, 9, 46, "ExercisesNoLabel",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],

Cell[CellGroupData[{
Cell[76548, 2585, 317, 11, 27, "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],

Cell[CellGroupData[{
Cell[76890, 2600, 117, 3, 30, "Input",
  CellTags->{"b:12.4", "IPM3.4.2.6"}],
Cell[77010, 2605, 128, 3, 29, "Output",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],
Cell[77153, 2611, 168, 5, 27, "ExercisesNoLabel",
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]],
Cell[77336, 2619, 386, 13, 46, "ExerciseItem",
  CellTags->{"b:13", "IPM3.4.2.6"}],

Cell[CellGroupData[{
Cell[77747, 2636, 920, 31, 65, "ExerciseItem",
  CellTags->{"b:12.4", "IPM3.4.2.6", "i:59"}],
Cell[78670, 2669, 266, 6, 38, "DisplayFormula",
  PageBreakAbove->False,
  PageBreakBelow->True,
  CellTags->{"b:12.4", "IPM3.4.2.6"}]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[78997, 2682, 152, 2, 29, "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 2}},
  CellTags->{"b:13", "IPM3.4.3.1", "IPM3.4.3.2", "c:10"}],
Cell[79152, 2686, 819, 14, 128, "TextFirst",
  CellTags->{"i:15", "b:13", "IPM3.4.3.1"}],
Cell[79974, 2702, 257, 9, 33, "Text",
  CellTags->{"i:16", "b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{
Cell[80256, 2715, 106, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[80365, 2720, 107, 3, 29, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],
Cell[80487, 2726, 221, 7, 33, "TextFirst",
  CellTags->{"i:17", "b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{
Cell[80733, 2737, 110, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[80846, 2742, 139, 3, 29, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],
Cell[81000, 2748, 193, 6, 33, "TextFirst",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{
Cell[81218, 2758, 116, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[81337, 2763, 329, 6, 48, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],
Cell[81681, 2772, 242, 9, 33, "TextFirst",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{
Cell[81948, 2785, 147, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[82098, 2790, 170, 4, 29, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],
Cell[82283, 2797, 362, 7, 71, "Text",
  CellTags->{"i:18", "b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{
Cell[82670, 2808, 114, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[82787, 2813, 113, 3, 29, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],
Cell[82915, 2819, 661, 18, 90, "Text",
  CellTags->{"i:19", "b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{
Cell[83601, 2841, 114, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[83718, 2846, 113, 3, 29, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],
Cell[83846, 2852, 222, 7, 33, "TextFirst",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{
Cell[84093, 2863, 118, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[84214, 2868, 142, 3, 29, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],
Cell[84371, 2874, 280, 6, 52, "TextFirst",
  CellTags->{"b:13", "IPM3.4.3.1"}],

Cell[CellGroupData[{
Cell[84676, 2884, 124, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[84803, 2889, 133, 3, 29, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[84973, 2897, 127, 3, 30, "Input",
  CellTags->{"b:13", "IPM3.4.3.1"}],
Cell[85103, 2902, 114, 3, 29, "Output",
  CellTags->{"b:13", "IPM3.4.3.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[85254, 2910, 83, 1, 77, "ExerciseSubsection",
  CellTags->{"b:13", "IPM3.4.3.2", "c:11"}],
Cell[85340, 2913, 309, 7, 46, "ExerciseItem",
  CellTags->{"b:13", "IPM3.4.3.2"}],
Cell[85652, 2922, 152, 4, 27, "ExerciseItem",
  CellTags->{"b:13", "IPM3.4.3.2"}],
Cell[85807, 2928, 534, 20, 46, "ExerciseItem",
  CellTags->{"b:13", "IPM3.4.3.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[86390, 2954, 174, 2, 29, "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 3}},
  CellTags->{
  "IPM3.4.4.0", "IPM3.4.4.1", "IPM3.4.4.2", "IPM3.4.4.3", "c:12"}],
Cell[86567, 2958, 503, 8, 90, "TextFirst",
  CellTags->"IPM3.4.4.0"],

Cell[CellGroupData[{
Cell[87095, 2970, 90, 1, 44, "Subsection",
  CellTags->{"b:14.0", "IPM3.4.4.1", "c:13"}],
Cell[87188, 2973, 834, 18, 128, "TextFirst",
  CellTags->{"i:20", "b:14.0", "IPM3.4.4.1"}],
Cell[88025, 2993, 213, 4, 52, "Text",
  CellTags->{"i:21", "b:14.0", "IPM3.4.4.1"}],
Cell[88241, 2999, 703, 31, 42, "Program",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],
Cell[88947, 3032, 656, 14, 90, "Text",
  CellTags->{"i:22", "b:14.0", "IPM3.4.4.1"}],
Cell[89606, 3048, 610, 10, 109, "Text",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],
Cell[90219, 3060, 613, 10, 109, "Text",
  CellTags->{"i:23", "b:14.0", "IPM3.4.4.1"}],
Cell[90835, 3072, 262, 6, 52, "Text",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],
Cell[91100, 3080, 113, 3, 30, "Input",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],
Cell[91216, 3085, 191, 4, 52, "Text",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],

Cell[CellGroupData[{
Cell[91432, 3093, 104, 3, 30, "Input",
  CellTags->{"b:14.0", "IPM3.4.4.1"}],
Cell[91539, 3098, 98, 3, 29, "Output",
  CellTags->{"b:14.0", "IPM3.4.4.1"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[91686, 3107, 88, 1, 44, "Subsection",
  CellTags->{"b:14.1", "IPM3.4.4.2", "c:14"}],
Cell[91777, 3110, 683, 16, 90, "TextFirst",
  CellTags->{"i:24", "b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[92485, 3130, 113, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[92601, 3135, 115, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[92731, 3141, 148, 4, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[92904, 3149, 103, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[93010, 3154, 115, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[CellGroupData[{
Cell[93162, 3162, 101, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[93266, 3167, 115, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],

Cell[CellGroupData[{
Cell[93418, 3175, 101, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[93522, 3180, 115, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[93652, 3186, 237, 6, 52, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[93914, 3196, 120, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[94037, 3201, 673, 20, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[94725, 3224, 329, 6, 71, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[95079, 3234, 134, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[95216, 3239, 101, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[95332, 3245, 208, 6, 33, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[95565, 3255, 199, 7, 27, "ItemizedText",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[95789, 3266, 122, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[95914, 3271, 127, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[96090, 3280, 172, 5, 27, "ItemizedText",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[96287, 3289, 109, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[96399, 3294, 102, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[96516, 3300, 133, 3, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[96652, 3305, 141, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[96818, 3312, 134, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[96955, 3317, 102, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[97072, 3323, 212, 4, 52, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[97309, 3331, 148, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[97460, 3336, 106, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[97581, 3342, 182, 5, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[97788, 3351, 157, 4, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[97948, 3357, 2568, 70, 124, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[100531, 3430, 128, 2, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[100662, 3434, 413, 15, 50, "BulletedList",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[101100, 3453, 135, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[101238, 3458, 121, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[101374, 3464, 217, 6, 50, "BulletedList",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[101616, 3474, 103, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[101722, 3479, 118, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[101855, 3485, 203, 6, 31, "BulletedList",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[102083, 3495, 104, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[102190, 3500, 106, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[102311, 3506, 86, 1, 33, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[102400, 3509, 150, 3, 30, "Input",
  CellTags->{"i:25", "b:14.1", "IPM3.4.4.2"}],
Cell[102553, 3514, 227, 6, 52, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[102805, 3524, 134, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[102942, 3529, 114, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[103071, 3535, 401, 7, 71, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[103475, 3544, 469, 15, 52, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[103969, 3563, 260, 6, 70, "Input",
  CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}],
Cell[104232, 3571, 1278, 21, 62, "Output",
  CellTags->{"i:26", "b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[105525, 3595, 322, 8, 52, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[105850, 3605, 413, 9, 71, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[106288, 3618, 128, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[106419, 3623, 137, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[106571, 3629, 460, 9, 71, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[107056, 3642, 178, 5, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[107237, 3649, 1312, 21, 72, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[108564, 3673, 190, 4, 52, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[108779, 3681, 109, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[108891, 3686, 1270, 21, 62, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[110176, 3710, 66, 1, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[110245, 3713, 401, 9, 71, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[110649, 3724, 472, 9, 90, "Text",
  CellTags->{"i:27", "b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[111146, 3737, 108, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[111257, 3742, 115, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[111387, 3748, 111, 1, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[111523, 3753, 121, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[111647, 3758, 119, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[111781, 3764, 329, 8, 52, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[112135, 3776, 108, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[112246, 3781, 121, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[112382, 3787, 450, 10, 71, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[112857, 3801, 109, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[112969, 3806, 115, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[113099, 3812, 265, 5, 52, "Text",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[113367, 3819, 180, 4, 30, "Input",
  CellTags->{"i:28", "b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[113572, 3827, 113, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[113688, 3832, 1270, 21, 62, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[114973, 3856, 265, 5, 52, "Text",
  CellTags->{"i:30", "b:14.1", "IPM3.4.4.2"}],
Cell[115241, 3863, 176, 4, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[115420, 3869, 169, 4, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[115592, 3875, 706, 19, 90, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[116323, 3898, 189, 4, 50, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[116515, 3904, 125, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[116655, 3910, 576, 16, 71, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[117234, 3928, 108, 2, 42, "Program",
  Evaluatable->False,
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[117345, 3932, 190, 5, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[117538, 3939, 130, 2, 42, "Program",
  Evaluatable->False,
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[117671, 3943, 170, 5, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[117844, 3950, 174, 4, 30, "Input",
  CellTags->{"i:29", "b:14.1", "IPM3.4.4.2"}],
Cell[118021, 3956, 77, 1, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],

Cell[CellGroupData[{
Cell[118123, 3961, 103, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.2"}],
Cell[118229, 3966, 201, 4, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]],
Cell[118445, 3973, 74, 1, 33, "TextFirst",
  CellTags->{"b:14.1", "IPM3.4.4.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[118568, 3980, 85, 1, 77, "ExerciseSubsection",
  CellTags->{"b:14.1", "IPM3.4.4.3", "c:15"}],

Cell[CellGroupData[{
Cell[118678, 3985, 632, 17, 84, "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],
Cell[119313, 4004, 142, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[CellGroupData[{
Cell[119480, 4011, 102, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],
Cell[119585, 4016, 108, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.3"}]
}, Closed]],
Cell[119708, 4022, 875, 29, 65, "ExercisesNoLabel",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],

Cell[CellGroupData[{
Cell[120608, 4055, 145, 3, 30, "Input",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],
Cell[120756, 4060, 105, 3, 29, "Output",
  CellTags->{"b:14.1", "IPM3.4.4.3"}]
}, Closed]]
}, Closed]],
Cell[120888, 4067, 348, 11, 46, "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],
Cell[121239, 4080, 575, 21, 27, "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3", "i:57"}],
Cell[121817, 4103, 524, 14, 84, "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],
Cell[122344, 4119, 167, 4, 27, "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}],
Cell[122514, 4125, 362, 11, 46, "ExerciseItem",
  CellTags->{"b:14.1", "IPM3.4.4.3"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[122925, 4142, 208, 4, 29, "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 4}},
  CellTags->{
  "IPM3.4.5.0", "IPM3.4.5.1", "IPM3.4.5.2", "IPM3.4.5.3", "IPM3.4.5.4", 
    "IPM3.4.5.5", "c:16"}],
Cell[123136, 4148, 609, 17, 90, "TextFirst",
  CellTags->"IPM3.4.5.0"],

Cell[CellGroupData[{
Cell[123770, 4169, 86, 1, 44, "Subsection",
  CellTags->{"b:15.0", "IPM3.4.5.1", "c:17"}],
Cell[123859, 4172, 327, 7, 52, "TextFirst",
  CellTags->{"i:31", "b:15.0", "IPM3.4.5.1"}],
Cell[124189, 4181, 1309, 45, 43, "Program",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[125501, 4228, 663, 14, 90, "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[126167, 4244, 226, 6, 52, "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[126396, 4252, 262, 6, 70, "Input",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[126661, 4260, 168, 4, 30, "Input",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[126832, 4266, 165, 4, 30, "Input",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[127000, 4272, 164, 4, 33, "TextFirst",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[127167, 4278, 128, 3, 30, "Input",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[127298, 4283, 130, 2, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[127431, 4287, 507, 11, 150, "Input",
  CellTags->{"i:32", "b:15.0", "IPM3.4.5.1"}],
Cell[127941, 4300, 88, 1, 33, "TextFirst",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[CellGroupData[{
Cell[128054, 4305, 102, 3, 30, "Input",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[128159, 4310, 201, 4, 29, "Output",
  CellTags->{"b:15.0", "IPM3.4.5.1"}]
}, Closed]],
Cell[128375, 4317, 591, 13, 90, "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[128969, 4332, 496, 9, 90, "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[129468, 4343, 417, 7, 90, "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],

Cell[CellGroupData[{
Cell[129910, 4354, 108, 3, 30, "Input",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[130021, 4359, 96, 1, 20, "Print",
  CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}],
Cell[130120, 4362, 2082, 38, 53, "Print",
  CellTags->{"b:15.0", "Info3302592864-7158086", "IPM3.4.5.1"}]
}, Closed]],
Cell[132217, 4403, 325, 6, 71, "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}],
Cell[132545, 4411, 312, 7, 52, "Text",
  CellTags->{"b:15.0", "IPM3.4.5.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[132894, 4423, 100, 1, 44, "Subsection",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2", "c:18"}],
Cell[132997, 4426, 627, 14, 90, "TextFirst",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
Cell[133627, 4442, 1348, 44, 61, "Program",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
Cell[134978, 4488, 652, 15, 90, "TextFirst",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
Cell[135633, 4505, 194, 7, 33, "Text",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
Cell[135830, 4514, 114, 3, 30, "Input",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
Cell[135947, 4519, 535, 11, 130, "Input",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}],
Cell[136485, 4532, 617, 16, 90, "Text",
  CellTags->{"b:15.1", "IPM3.4.5.2"}],
Cell[137105, 4550, 497, 10, 90, "Text",
  CellTags->{"i:33", "b:15.1", "IPM3.4.5.2"}]
}, Closed]],

Cell[CellGroupData[{
Cell[137639, 4565, 92, 1, 44, "Subsection",
  CellTags->{"b:15.2", "IPM3.4.5.3", "c:19"}],
Cell[137734, 4568, 655, 19, 90, "TextFirst",
  CellTags->{"i:34", "b:15.2", "IPM3.4.5.3"}],

Cell[CellGroupData[{
Cell[138414, 4591, 182, 4, 70, "Input",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],
Cell[138599, 4597, 363, 6, 19, "Message",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],
Cell[138965, 4605, 333, 9, 29, "Output",
  CellTags->{"b:15.2", "IPM3.4.5.3"}]
}, Closed]],
Cell[139313, 4617, 159, 5, 33, "TextFirst",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],

Cell[CellGroupData[{
Cell[139497, 4626, 111, 3, 30, "Input",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],
Cell[139611, 4631, 100, 3, 29, "Output",
  CellTags->{"b:15.2", "IPM3.4.5.3"}]
}, Closed]],
Cell[139726, 4637, 279, 11, 33, "TextFirst",
  CellTags->{"b:15.2", "IPM3.4.5.3"}],
Cell[140008, 4650, 415, 11, 71, "Text",
  CellTags->{"b:15.2", "IPM3.4.5.3"}]
}, Closed]],

Cell[CellGroupData[{
Cell[140460, 4666, 94, 1, 44, "Subsection",
  CellTags->{"b:15.3", "IPM3.4.5.4", "c:20"}],
Cell[140557, 4669, 362, 7, 71, "TextFirst",
  CellTags->{"i:35", "b:15.3", "IPM3.4.5.4"}],
Cell[140922, 4678, 150, 5, 33, "Text",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[141075, 4685, 106, 3, 30, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[141184, 4690, 192, 6, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[141379, 4698, 170, 4, 70, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[141552, 4704, 237, 8, 33, "TextFirst",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[CellGroupData[{
Cell[141814, 4716, 97, 3, 30, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[141914, 4721, 98, 3, 29, "Output",
  CellTags->{"b:15.3", "IPM3.4.5.4"}]
}, Closed]],

Cell[CellGroupData[{
Cell[142049, 4729, 100, 3, 30, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[142152, 4734, 98, 3, 29, "Output",
  CellTags->{"b:15.3", "IPM3.4.5.4"}]
}, Closed]],
Cell[142265, 4740, 199, 6, 33, "TextFirst",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[142467, 4748, 172, 4, 70, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[142642, 4754, 192, 7, 33, "TextFirst",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[CellGroupData[{
Cell[142859, 4765, 102, 3, 30, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[142964, 4770, 89, 1, 20, "Print",
  CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}],
Cell[143056, 4773, 460, 13, 20, "Print",
  CellTags->{"b:15.3", "Info3302592864-4738886", "IPM3.4.5.4"}]
}, Closed]],
Cell[143531, 4789, 240, 9, 33, "TextFirst",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[143774, 4800, 106, 3, 30, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],

Cell[CellGroupData[{
Cell[143905, 4807, 100, 3, 30, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.4"}],
Cell[144008, 4812, 99, 3, 29, "Output",
  CellTags->{"b:15.3", "IPM3.4.5.4"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[144156, 4821, 85, 1, 77, "ExerciseSubsection",
  CellTags->{"b:15.3", "IPM3.4.5.5", "c:21"}],
Cell[144244, 4824, 414, 12, 46, "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],
Cell[144661, 4838, 1167, 33, 122, "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5", "i:58"}],

Cell[CellGroupData[{
Cell[145853, 4875, 441, 12, 65, "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[CellGroupData[{
Cell[146319, 4891, 121, 3, 30, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],
Cell[146443, 4896, 99, 3, 29, "Output",
  CellTags->{"b:15.3", "IPM3.4.5.5"}]
}, Closed]],
Cell[146557, 4902, 115, 1, 27, "ExercisesNoLabel",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],
Cell[146675, 4905, 346, 11, 46, "ExercisesNoLabel",
  CellTags->{"b:15.3", "IPM3.4.5.5"}]
}, Closed]],
Cell[147036, 4919, 185, 5, 27, "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],
Cell[147224, 4926, 940, 34, 65, "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],
Cell[148167, 4962, 1069, 39, 46, "ExerciseItem",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],

Cell[CellGroupData[{
Cell[149261, 5005, 1072, 34, 103, "ExerciseItem",
  CellTags->{"i:53", "b:15.3", "IPM3.4.5.5"}],

Cell[CellGroupData[{
Cell[150358, 5043, 118, 3, 30, "Input",
  CellTags->{"b:15.3", "IPM3.4.5.5"}],
Cell[150479, 5048, 109, 3, 29, "Output",
  CellTags->{"b:15.3", "IPM3.4.5.5"}]
}, Closed]]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[150661, 5059, 147, 2, 29, "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 5}},
  CellTags->{"b:16", "IPM3.4.6.1", "IPM3.4.6.2", "c:22"}],
Cell[150811, 5063, 638, 17, 90, "TextFirst",
  CellTags->{"i:36", "b:16", "IPM3.4.6.1"}],
Cell[151452, 5082, 1174, 39, 90, "Text",
  CellTags->{"i:37", "b:16", "IPM3.4.6.1"}],
Cell[152629, 5123, 184, 5, 33, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[152838, 5132, 110, 3, 31, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[152951, 5137, 111, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[153077, 5143, 536, 12, 90, "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[153616, 5157, 105, 3, 30, "Input",
  Evaluatable->False,
  CellTags->{"i:38", "b:16", "IPM3.4.6.1"}],
Cell[153724, 5162, 192, 7, 33, "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[153919, 5171, 305, 7, 52, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[154249, 5182, 117, 3, 31, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[154369, 5187, 96, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[154480, 5193, 240, 5, 52, "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[154745, 5202, 108, 3, 31, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[154856, 5207, 96, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[154967, 5213, 249, 5, 52, "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[155219, 5220, 120, 3, 31, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[155364, 5227, 103, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[155470, 5232, 96, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[155581, 5238, 309, 8, 52, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[155893, 5248, 78, 1, 33, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[155974, 5251, 120, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[156097, 5256, 335, 9, 52, "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[156457, 5269, 113, 3, 31, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[156573, 5274, 122, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[156710, 5280, 209, 5, 33, "Text",
  CellTags->{"b:16", "IPM3.4.6.1", "i:60"}],

Cell[CellGroupData[{
Cell[156944, 5289, 242, 5, 70, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[157189, 5296, 246, 5, 48, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[157450, 5304, 105, 1, 33, "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[157580, 5309, 147, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[157730, 5314, 29192, 905, 181, 16645, 745, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[186937, 6222, 699, 25, 71, "TextFirst",
  CellTags->{"i:39", "b:16", "IPM3.4.6.1"}],
Cell[187639, 6249, 281, 7, 52, "Text",
  PageBreakAbove->True,
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[187945, 6260, 143, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[188091, 6265, 222, 4, 48, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[188328, 6272, 402, 9, 71, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[188733, 6283, 124, 3, 33, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[188860, 6288, 150, 3, 30, "Input",
  CellTags->{"i:40", "b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[189035, 6295, 119, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[189157, 6300, 100, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[189272, 6306, 97, 1, 33, "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[189394, 6311, 156, 4, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[189553, 6317, 100, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[189690, 6325, 146, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[189839, 6330, 100, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[189954, 6336, 144, 4, 33, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[190101, 6342, 124, 2, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[190250, 6348, 120, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[190373, 6353, 107, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[190495, 6359, 97, 1, 33, "TextFirst",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[190617, 6364, 172, 4, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[190792, 6370, 107, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[190936, 6378, 152, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[191091, 6383, 107, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[191213, 6389, 226, 6, 52, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[191464, 6399, 116, 3, 31, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[191583, 6404, 105, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],
Cell[191703, 6410, 438, 11, 71, "Text",
  CellTags->{"b:16", "IPM3.4.6.1"}],

Cell[CellGroupData[{
Cell[192166, 6425, 147, 3, 31, "Input",
  CellTags->{"b:16", "IPM3.4.6.1"}],
Cell[192316, 6430, 105, 3, 29, "Output",
  CellTags->{"b:16", "IPM3.4.6.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[192458, 6438, 83, 1, 77, "ExerciseSubsection",
  CellTags->{"b:16", "IPM3.4.6.2", "c:23"}],
Cell[192544, 6441, 138, 3, 27, "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],
Cell[192685, 6446, 379, 10, 46, "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{
Cell[193089, 6460, 668, 17, 84, "ExerciseItem",
  CellTags->{"i:41", "b:16", "IPM3.4.6.2"}],
Cell[193760, 6479, 122, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{
Cell[193907, 6486, 121, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.2"}],
Cell[194031, 6491, 238, 5, 48, "Output",
  CellTags->{"b:16", "IPM3.4.6.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[194318, 6502, 190, 5, 27, "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],
Cell[194511, 6509, 152, 5, 48, "Input",
  CellTags->{"b:16", "IPM3.4.6.2"}]
}, Closed]],
Cell[194678, 6517, 362, 10, 46, "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],
Cell[195043, 6529, 369, 12, 46, "ExerciseItem",
  CellTags->{"i:42", "b:16", "IPM3.4.6.2"}],
Cell[195415, 6543, 406, 14, 46, "ExerciseItem",
  CellTags->{"i:43", "b:16", "IPM3.4.6.2"}],
Cell[195824, 6559, 766, 25, 84, "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{
Cell[196615, 6588, 541, 18, 46, "ExerciseItem",
  CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],
Cell[197159, 6608, 125, 2, 27, "ExercisesNoLabel",
  CellTags->{"b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{
Cell[197309, 6614, 237, 5, 70, "Input",
  CellTags->{"b:16", "IPM3.4.6.2"}],
Cell[197549, 6621, 211, 5, 48, "Output",
  CellTags->{"b:16", "IPM3.4.6.2"}]
}, Closed]],
Cell[197775, 6629, 231, 7, 27, "ExercisesNoLabel",
  CellTags->{"b:16", "IPM3.4.6.2"}],
Cell[198009, 6638, 130, 3, 30, "Input",
  CellTags->{"b:16", "IPM3.4.6.2"}]
}, Closed]],
Cell[198154, 6644, 222, 5, 46, "ExerciseItem",
  CellTags->{"i:44", "b:16", "IPM3.4.6.2"}],

Cell[CellGroupData[{
Cell[198401, 6653, 674, 15, 84, "ExerciseItem",
  CellTags->{"b:16", "IPM3.4.6.2", "i:55"}],

Cell[CellGroupData[{
Cell[199100, 6672, 101, 3, 30, "Input",
  CellTags->"IPM3.4.6.2"],
Cell[199204, 6677, 104, 3, 42, "Output",
  CellTags->"IPM3.4.6.2"]
}, Closed]],
Cell[199323, 6683, 551, 15, 65, "ExercisesNoLabel",
  CellTags->"IPM3.4.6.2"],

Cell[CellGroupData[{
Cell[199899, 6702, 106, 3, 30, "Input",
  CellTags->"IPM3.4.6.2"],
Cell[200008, 6707, 118, 3, 29, "Output",
  CellTags->"IPM3.4.6.2"]
}, Closed]],

Cell[CellGroupData[{
Cell[200163, 6715, 92, 3, 30, "Input",
  CellTags->"IPM3.4.6.2"],
Cell[200258, 6720, 112, 3, 29, "Output",
  CellTags->"IPM3.4.6.2"]
}, Closed]]
}, Closed]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[200443, 6731, 185, 4, 29, "Section",
  CounterAssignments->{{"Chapter", 4}, {"Section", 6}},
  CellTags->{
  "IPM3.4.7.0", "IPM3.4.7.1", "IPM3.4.7.2", "IPM3.4.7.3", "IPM3.4.7.4", 
    "c:24"}],
Cell[200631, 6737, 657, 11, 109, "TextFirst",
  CellTags->"IPM3.4.7.0"],

Cell[CellGroupData[{
Cell[201313, 6752, 84, 1, 44, "Subsection",
  CellTags->{"b:17.0", "IPM3.4.7.1", "c:25"}],
Cell[201400, 6755, 634, 10, 109, "TextFirst",
  CellTags->{"i:45", "b:17.0", "IPM3.4.7.1"}],
Cell[202037, 6767, 448, 18, 52, "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[202510, 6789, 134, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[202647, 6794, 115, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[202777, 6800, 166, 5, 33, "TextFirst",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[202968, 6809, 147, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[203118, 6814, 129, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[203262, 6820, 170, 6, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[203457, 6830, 110, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[203570, 6835, 97, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[203682, 6841, 228, 7, 33, "TextFirst",
  CellTags->{"i:46", "b:17.0", "IPM3.4.7.1"}],
Cell[203913, 6850, 199, 4, 50, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[204137, 6858, 144, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[204284, 6863, 97, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[204396, 6869, 302, 8, 52, "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[204701, 6879, 88, 1, 33, "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[204814, 6884, 146, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[204963, 6889, 136, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[205114, 6895, 608, 23, 52, "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1", "i:56"}],

Cell[CellGroupData[{
Cell[205747, 6922, 139, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[205889, 6927, 105, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[206009, 6933, 166, 7, 33, "TextFirst",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[206200, 6944, 118, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[206321, 6949, 111, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[206447, 6955, 113, 1, 33, "TextFirst",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[206585, 6960, 109, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[206697, 6965, 97, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[206809, 6971, 403, 11, 52, "TextFirst",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[207215, 6984, 242, 5, 70, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[207482, 6993, 146, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[207631, 6998, 98, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[207744, 7004, 205, 4, 52, "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[207952, 7010, 140, 3, 31, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[208095, 7015, 140, 3, 31, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],

Cell[CellGroupData[{
Cell[208260, 7022, 141, 3, 30, "Input",
  CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}],
Cell[208404, 7027, 142, 3, 29, "Output",
  CellTags->{"i:48", "b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[208583, 7035, 134, 3, 30, "Input",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[208720, 7040, 121, 3, 29, "Output",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],
Cell[208856, 7046, 358, 8, 71, "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}],
Cell[209217, 7056, 266, 5, 52, "Text",
  CellTags->{"b:17.0", "IPM3.4.7.1"}]
}, Closed]],

Cell[CellGroupData[{
Cell[209520, 7066, 88, 1, 44, "Subsection",
  CellTags->{"b:17.1", "IPM3.4.7.2", "c:26"}],
Cell[209611, 7069, 223, 4, 52, "TextFirst",
  CellTags->{"i:49", "b:17.1", "IPM3.4.7.2"}],
Cell[209837, 7075, 385, 7, 103, "Quote",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[210225, 7084, 821, 14, 128, "Text",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[211049, 7100, 429, 9, 71, "Text",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[CellGroupData[{
Cell[211503, 7113, 135, 3, 30, "Input",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[211641, 7118, 106, 3, 29, "Output",
  CellTags->{"b:17.1", "IPM3.4.7.2"}]
}, Closed]],
Cell[211762, 7124, 621, 18, 90, "TextFirst",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[212386, 7144, 178, 4, 30, "Input",
  CellTags->{"i:50", "b:17.1", "IPM3.4.7.2"}],
Cell[212567, 7150, 224, 6, 52, "TextFirst",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[CellGroupData[{
Cell[212816, 7160, 115, 3, 30, "Input",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[212934, 7165, 100, 3, 29, "Output",
  CellTags->{"b:17.1", "IPM3.4.7.2"}]
}, Closed]],
Cell[213049, 7171, 448, 12, 71, "TextFirst",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],

Cell[CellGroupData[{
Cell[213522, 7187, 138, 3, 30, "Input",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[213663, 7192, 365, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[214031, 7205, 382, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[214416, 7218, 365, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[214784, 7231, 376, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[215163, 7244, 365, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[215531, 7257, 370, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[215904, 7270, 365, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[216272, 7283, 364, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[216639, 7296, 365, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[217007, 7309, 358, 11, 24, "Print",
  CellTags->{"b:17.1", "IPM3.4.7.2"}],
Cell[217368, 7322, 157, 4, 26, "Output",
  CellTags->{"b:17.1", "IPM3.4.7.2"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[217574, 7332, 81, 1, 44, "Subsection",
  CellTags->{"b:17.2", "IPM3.4.7.3", "c:27"}],
Cell[217658, 7335, 273, 5, 52, "TextFirst",
  CellTags->{"i:51", "b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{
Cell[217956, 7344, 134, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],
Cell[218093, 7349, 127, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],
Cell[218235, 7355, 486, 18, 52, "TextFirst",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{
Cell[218746, 7377, 111, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],
Cell[218860, 7382, 98, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],
Cell[218973, 7388, 133, 3, 33, "TextFirst",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{
Cell[219131, 7395, 171, 4, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],
Cell[219305, 7401, 109, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],
Cell[219429, 7407, 383, 12, 52, "Text",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{
Cell[219837, 7423, 139, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],
Cell[219979, 7428, 109, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],
Cell[220103, 7434, 337, 8, 52, "TextFirst",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{
Cell[220465, 7446, 123, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],
Cell[220591, 7451, 100, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],
Cell[220706, 7457, 332, 8, 52, "TextFirst",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{
Cell[221063, 7469, 143, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],
Cell[221209, 7474, 100, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],

Cell[CellGroupData[{
Cell[221346, 7482, 125, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],
Cell[221474, 7487, 100, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]],
Cell[221589, 7493, 354, 6, 52, "TextFirst",
  CellTags->{"i:52", "b:17.2", "IPM3.4.7.3"}],
Cell[221946, 7501, 199, 5, 50, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],

Cell[CellGroupData[{
Cell[222170, 7510, 115, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.3"}],
Cell[222288, 7515, 100, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.3"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[222437, 7524, 85, 1, 77, "ExerciseSubsection",
  CellTags->{"b:17.2", "IPM3.4.7.4", "c:28"}],
Cell[222525, 7527, 261, 6, 46, "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[222789, 7535, 524, 10, 84, "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[223316, 7547, 446, 7, 84, "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{
Cell[223787, 7558, 145, 3, 27, "ExerciseSubItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[223935, 7563, 166, 4, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{
Cell[224126, 7571, 132, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[224261, 7576, 120, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[224430, 7585, 158, 4, 27, "ExerciseSubItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[224591, 7591, 243, 5, 70, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{
Cell[224859, 7600, 126, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[224988, 7605, 137, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]],
Cell[225140, 7611, 140, 3, 27, "ExercisesNoLabel",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[225283, 7616, 193, 6, 66, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]],

Cell[CellGroupData[{
Cell[225513, 7627, 918, 27, 103, "ExerciseSubItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[226434, 7656, 291, 7, 70, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{
Cell[226750, 7667, 115, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[226868, 7672, 112, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]],
Cell[226995, 7678, 140, 3, 27, "ExercisesNoLabel",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[227138, 7683, 283, 7, 90, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[227424, 7692, 858, 21, 122, "ExercisesNoLabel",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]],
Cell[228297, 7716, 334, 9, 46, "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[228634, 7727, 272, 9, 46, "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{
Cell[228931, 7740, 150, 4, 27, "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[229084, 7746, 175, 4, 50, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],

Cell[CellGroupData[{
Cell[229284, 7754, 111, 3, 30, "Input",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[229398, 7759, 109, 3, 29, "Output",
  CellTags->{"b:17.2", "IPM3.4.7.4"}]
}, Closed]]
}, Closed]],
Cell[229534, 7766, 548, 15, 65, "ExerciseItem",
  CellTags->{"b:17.2", "IPM3.4.7.4"}],
Cell[230085, 7783, 274, 8, 46, "ExerciseItem",
  CellTags->{"i:54", "b:17.2", "IPM3.4.7.4"}]
}, Closed]]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

