(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[    949260,      17935]*)
(*NotebookOutlinePosition[   1040469,      19981]*)
(*  CellTagsIndexPosition[   1037468,      19875]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "Introduction to Programming with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "TitleBar",
  CellTags->{
  "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
    "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3",
     "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", "IPM3.10.4.3", 
    "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", "IPM3.10.5.4",
     "IPM3.10.5.5", "IPM3.10.5.6"}],

Cell[CellGroupData[{

Cell[TextData[{
  
  CounterBox["Chapter"],
  " Front end programming"
}], "Chapter",
  CounterAssignments->{{"Chapter", 9}},
  CellTags->{"IPM3.10.0", "c:1"}],

Cell[TextData[{
  "In this chapter we extend the programming concepts we have covered thus \
far to the objects that comprise the user interface, or front end. Because \
the objects that the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " user interacts with are themselves ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " expressions, all of the tools that you use to do computations can also be \
used to create, manipulate, and alter cells and notebooks themselves. We will \
first look at the underlying structure of these objects and then discuss ways \
of manipulating them directly from within ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "."
}], "ChapterAbstract",
  CellTags->"IPM3.10.0"],

Cell[CellGroupData[{

Cell["Introduction", "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 0}},
  CellTags->{"b:42", "IPM3.10.1", "c:2"}],

Cell[TextData[{
  "Up until this point, we have been primarily concerned with learning about \
programming constructs and styles so that we can write programs to manipulate \
data or solve problems from science, engineering, or mathematics. We have \
taken for granted that the space in which we do our experimenting, \
prototyping, and documenting has been the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook, an interface that has some similarities to a word processor \
document."
}], "TextFirst",
  CellTags->{"b:42", "IPM3.10.1"}],

Cell[TextData[{
  "It is not uncommon now to add interactive elements to your documents to \
make them more useful for yourself or the intended reader of your documents. \
With programs, documentation, and papers all being created and used in \
electronic format, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " provides a seamless and well-integrated interface to these elements."
}], "Text",
  CellTags->{"b:42", "IPM3.10.1"}],

Cell[TextData[{
  "Another tool that is useful, especially for educators, are buttons that \
allow you to hide your program code behind a familiar and easy-to-use \
interface element \[Dash] the button. The user clicks on a button and an \
action happens that is determined by the underlying code. For example, you \
might want to have calculus students quickly plot Taylor polynomial \
approximations to a function together with the original function but do not \
want them to spend time learning the syntax of such commands in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". You could easily program an interface that would only require them to \
fill in a few parameters before clicking a button to produce the desired \
plot."
}], "Text",
  CellTags->{"b:42", "IPM3.10.1"}],

Cell["\<\
In this chapter we will discuss the structure of cell and notebook \
expressions, look at a few basic functions for manipulating these \
expressions, and then create several simple examples that give a flavor of \
the kind of things that can be done with front end programming.\
\>", "Text",
  CellTags->{"b:42", "IPM3.10.1"}],

Cell[TextData[{
  "Before we begin we should mention that this chapter is not intended as a \
complete discussion of front end programming. An entire book could certainly \
be written on this topic alone. This book is intended to give you an \
introduction to the many aspects of programming with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " and front end programming is certainly an appropriate topic for that \
introduction. But there are several areas that cannot be included here, \
either because of space limitations or because they do not fit under the \
introductory nature of this book. These topics include front end options and \
front end tokens. An understanding of each of these topics is quite important \
for more advanced front end programming. The interested reader can delve \
further into this subject by looking in the Front End category of the Help \
Browser or by searching the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " Information Center online at ",
  ButtonBox["library.wolfram.com/infocenter",
    ButtonData:>{
      URL[ "http://library.wolfram.com/infocenter"], None},
    ButtonStyle->"Hyperlink"],
  "."
}], "Text",
  CellTags->{"b:42", "IPM3.10.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The structure of cells and notebooks", "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 1}},
  CellTags->{
  "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", "IPM3.10.2.3", "c:3"}],

Cell[TextData[{
  "We have spent a lot of time in this book focusing on the structure of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " expressions. In Chapter 2 we indicated that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " expressions are of the form ",
  StyleBox["h[", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`e\_1\)]],
  StyleBox[",", "MR"],
  Cell[BoxData[
      \(TraditionalForm\`e\_2\)]],
  StyleBox[",\[Ellipsis]]", "MR"],
  " where ",
  StyleBox["h", "MR"],
  " is the head of the expression and the ",
  Cell[BoxData[
      \(TraditionalForm\`e\_i\)]],
  " are the elements which may themselves be ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " expressions. We even went so far as to say that everything in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " is an expression. In this section we will learn that this statement \
extends to elements of the front end, specifically to notebooks and cells. "
}], "TextFirst",
  CellTags->"IPM3.10.2.0"],

Cell[CellGroupData[{

Cell["Notebook expressions", "Subsection",
  CellTags->{"b:43.0", "IPM3.10.2.1", "c:4"}],

Cell[TextData[{
  "Notebooks are ASCII files, meaning that you can open them in a text editor \
and view their contents directly. If you were to do that, you would see that \
the underlying expression is a ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " function called a ",
  StyleBox["Notebook", "MR"],
  ". The notebook would look like this:"
}], "TextFirst",
  CellTags->{"i:1", "b:43.0", "IPM3.10.2.1"}],

Cell[TextData[{
  "Notebook[{\n    Cell[",
  StyleBox["string", "TI"],
  ",",
  StyleBox["style", "TI"],
  ",",
  StyleBox["options", "TI"],
  "],\n    Cell[",
  StyleBox["string", "TI"],
  ",",
  StyleBox["style", "TI"],
  ",",
  StyleBox["options", "TI"],
  "],\n    \[Ellipsis]\n    },\n  ",
  StyleBox["options", "TI"],
  "]"
}], "Program",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell[TextData[{
  "In other words, the ",
  StyleBox["Notebook", "MR"],
  " is a function whose first argument is a list of one or more ",
  StyleBox["Cell", "MR"],
  " objects, followed by some options. The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " kernel does not do anything with this practically. It is the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " front end that knows how to render this expression as the familiar \
notebook."
}], "Text",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell["\<\
For example, here is a very simple notebook that you could write in a text \
editor (of course there is no reason to do that). \
\>", "Text",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell["\<\
Notebook[{
    Cell[\"Demo notebook\", \"Section\"],
    Cell[\"This is a text cell.\",\"Text\"],
    Cell[\"1+2+3\", \"Input\"]
    }]\
\>", "Program",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell[TextData[{
  "The ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " front end renders this expression in the familiar manner, a window."
}], "TextFirst",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005;0000fR000`400?l0
0000o`00003oo`3ooom<0?ooo`00o`3ooom<0?ooo`003P3ooooo0?knoRl0
o_kn3`3oool000`0oooo103no_ko0?gmoB/0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgm<@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo2D0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k9`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinBP0nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_HW0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clbD0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hU0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj2D0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4U0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH8`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllS0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aB<0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^b<0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a8`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYR<0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhS09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXU09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000003L000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?cl<`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[Zc80Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFTa06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj3`2OWil00`2m_Kd0k>c/0>OWi`0H0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K3`3:b/X0
0`3Kfm/0Xj>S0?_kn`0H0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo3`3@d=000`3oool0UYJF0?_kn`0H0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3oool?0<o?c`03
0?ooo`2FUYH0no_k01P0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h0203oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM40d=3@0`3AdM400`3@d=00dM7A0=3@d005
0=7Ad@030?ooo`2FUYH0n_[j01P0n_[j00<0UYJF0?ooo`3>c/h0103>c/h0
0`0n?Sh0c/k>0<k>cP050<k>cP030?ooo`2FUYH0h>3P0100h>3P01@0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3P
h03Ph>00103Ph>003`34a<@0B4Q80=WIf@3Ph>00JFUY09jNWP3Ph>00SXj>
05ADE03Ph>00JFUY09jNWP3Ph>00SXj>05ADE0040>3Ph0080:f][@1DE5@0
h>3P0:f][@1DE5@0h>3P06UYJ@2NWYhF0>3Ph00309JFUP3oool0c/k>00@0
c/k>0`0n?Sh50<k>cP040?ooo`2FUYH0UYJF0?ooo`@0e]KF00P0UIFE0=OG
e`2EUID0e]KF09FEU@3Fe]H0UIFE0=OGe`<0e]KF00<0oooo09JFUP3inOT0
603inOT00`2FUYH0oooo0<k>cP030<k>cP<0?Shn1P3>c/h00`3oool0UYJF
0>3Ph0090>3Ph080oooo0P3Ph>020?ooo`0E0>3Ph01YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
1@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103P
h>004034a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00a<C404Q8B03Bd]80
9RHV0=;BdP3Ph>00a<C404Q8B03IfMT30>3Ph0080=cLg01ADE40^KVi0:f]
[@1DE5@0h>3P06UYJ@2NWYhF0>3Ph00309JFUP3oool0clo?00<0clo?0`0n
?Sh60<o?c`040?ooo`2FUYH0UYJF0?ooo`<0g=cL00T0oooo09FEU@3oool0
UIFE0?ooo`2EUID0oooo09FEU@3MgMd00P3Lg=`00`3MgMd0oooo09JFUP0I
0?Win@0309JFUP3oool0clo?00@0clo?0`0n?Sh50<o?c`030?ooo`2FUYH0
h>3P00P0h>3P0P2c/k<06P3oool0h>3P0;>c/`2c/k<0oooo0>3Ph01YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P
0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT0
9RHV0<_;b`0V9RH0d];B0>3Ph03IfMT09RHV0=;BdP@0h>3P00L0a<C404Q8
B03Bd]806QXJ0000000=3@d0d];B00X0h>3P1`0000050>3Ph00309JFUP3o
ool0e=CD0080e=CD0`0n?Sh70=CDe0040?ooo`2FUYH0UYJF0?ooo`<0g=cL
00T0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@3MgMd00P3Lg=`0
0`3MgMd0oooo09JFUP0I0?Shn00309JFUP3oool0e=CD00D0e=CD0`0n?Sh4
0=CDe0030?ooo`2FUYH0h>3P00P0h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00
501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00G0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h03IfMT09RHV0<_;b`0V9RH0d];B0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU9
0000000=3@d0d];B06UYJ@2NWYh03`3Ph>05000000H0h>3P00@0UYJF0?oo
o`3Kfm/0fm_K0`0n?Sh80=_Kf`050?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3S
hn<0203oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0P3Ti>@0103S
hn<0i>CT0?ooo`2FUYHI0?Ogm`0309JFUP3oool0fm_K00H0fm_K0`0n?Sh3
0=_Kf`030?ooo`2FUYH0h>3P0100h>3P01@0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000
o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>006034
a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00a<C404Q8B03Bd]809RHV0=;B
dP3Ph>00a<C404Q8B03IfMT0[Jf]05ADE03Ph>00JFUY07inOP34a<@0B4Q8
0=WIf@l0h>3P0`0000070>3Ph00309JFUP3oool0i>CT0080i>CT0`0n?Sh7
0>CTi0050?ooo`2FUYH0UYJF0?ooo`3Zj^X00P3YjNT02P3oool0UIFE0?oo
o`2EUID0oooo09FEU@3oool0UIFE0>WYj@3Zj^X20>WYj@030?ooo`2FUYH0
mOGe01P0mOGe00<0UYJF0?ooo`3Ti>@01@3Ti>@303hn?P@0i>CT00<0oooo
09JFUP3Ph>003@3Ph>020?ooo`0E0>3Ph01YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001@000000
o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>000`3Ph>006@2>
SXh0000003Ph>03IfMT0h>3P06UYJ@2NWYh0h>3P08j>SP1DE5@0h>3P06UY
J@2NWYh0h>3P08j>SP1DE5@0h>3P0:f][@1DE5@0h>3P06UYJ@1nOWh0h>3P
06UYJ@2NWYh0403Ph>000`000000h>3P0>3Ph0060>3Ph00309JFUP3oool0
kNg]00<0kNg]0`0n?Sh60>g]k@050?ooo`2FUYH0UYJF0?ooo`3Zj^X00P3Y
jNT02P3oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0>WYj@3Zj^X2
0>WYj@030?ooo`2FUYH0m?Cd01P0m?Cd00<0UYJF0?ooo`3]kNd0103]kNd3
03hn?PD0kNg]00<0oooo09JFUP3Ph>00303Ph>020;>c/`0F0?ooo`3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00103Ph>006@3IfMT0DU9B0=WIf@3Ph>00fMWI01XJ6P000000
3@d=0=;BdP3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00fMWI01XJ6P00
00003@d=0=;BdP3Ph>00a<C401dM7@3Bd]80603Ph>000`2FUYH0oooo0?Cd
m0040?Cdm0<0?Shn1@3dm?@0103oool0UYJF09JFUP3oool30>g]k@070?oo
o`3]kNd0oooo0>g]k@3oool0kNg]0?ooo`050>g]k@030?ooo`2FUYH0lO7a
01P0lO7a00<0UYJF0?ooo`3dm?@00`3dm?@303hn?PH0m?Cd00<0oooo09JF
UP3Ph>00303Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl
0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`2FUYH0
oooo0?clo0050?clo00303hn?P3lo?`0o?cl00@0o?cl00@0oooo09JFUP2F
UYH0oooo3`3]kNd00`3oool0UYJF0?3`l00H0?3`l00309JFUP3oool0o?cl
00@0o?cl00<0?Shn0?clo03lo?`01@3lo?`00`3oool0UYJF0>3Ph00@0>3P
h00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph003
0?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3oool03@3oool209JFUP03
0?ooo`3gmoL0moOg00d0moOg00<0oooo09JFUP3_knl0603_knl00`2FUYH0
oooo0?ooo`0=0?ooo`0309JFUP3Ph>00h>3P00l0h>3P01@0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00
=@3Ph>000`2FUYH0emOG0?ooo`0<0?ooo`040=OGe`2FUYH0UYJF0?[jnPl0
oooo00<0n_[j09JFUP3_knl0603_knl00`2FUYH0emOG0?ooo`0<0?ooo`03
0=OGe`2FUYH0h>3P0100h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[
Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`3Jf]X0TY:B
0;Jf]P0;0;Fe]@050;Bd]02?Shl0f][J0=KFeP2`/;003`2h^;P00`2YZJT0
c/k>0>k^kP0H0>k^kP030=[JfP2BTY80][Jf00/0]KFe00<0];Bd08n?S`3J
f]X04@3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJIL0WYjN
00<0oooo0>3Ph03Ph>00=@3Ph>0A0>c/k1<0jn_[6@3WinLA0>c/k140h>3P
01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0
k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0g]kN00`0fm_K01H0
g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3o
ool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool0303>
c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolN0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF0?oo
o`050<k>cP0303hn?P3>c/h0c/k>00@0c/k>01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF0?oo
o`040<k>cP<0?Shn1@3>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0oooo00<0clo?1@0n
?Sh40<o?c`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooah0oooo00<0k>c/09JFUP3oool00P3De=@303hn?P040=CDe00n
?Sh0?Shn03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF0?ooo`3Kfm/303hn?P<0
fm_K0`0n?Sh20=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool00P3Ti>@00`0n?Sh0
i>CT0>CTi0030>CTi00J03hn?P3Ti>@0i>CT0>CTi03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0
oooo00`0kNg]01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo7P3oool00`3/k>`0UYJF0?ooo`0<0?Cdm00F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/
09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0oooo00d0oooo01D0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo3P3oool505EE
E@80oooo1@1EEED40?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo4P3oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/0=[J
fP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFU20?ooo`030=oOg`0B4Q8000000080000000<02@T9
0=;BdP3oool01P3oool00`2RXZ80000000000002000000030451@@3gmoL0
oooo00H0oooo00<0no_k05ADE00000000P0000000`2d];@0oooo0?ooo`2[
0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Wi
n@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYA03oool00`3Ogml06a/K0=;BdP040?ooo`030=oOg`0K6a/0
d];B00<0oooo00<0N7Qh00l?3`3_knl01@3oool00`3Ogml06a/K0=;BdP02
0?ooo`070=oOg`1CDe<0moOg0?ooo`3Ogml08B4Q0>o_k`2/0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
A03oool00`3Ogml06a/K0=;BdP040?ooo`030=oOg`0K6a/0d];B00@0oooo
00<0a/K600T92@3Bd]80103oool00`3Ogml06a/K0=;BdP060?ooo`030?Og
m`0V9RH0d];B0:`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo
0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFU40?ooo`030=oOg`0K6a/0d];B0080oooo
00<0N7Qh0000000000000`0000002@1PH600oooo0?ooo`3oool0gmoO01/K
6`3Bd]80oooo07QhN00500000003061PH03oool0oooo0080oooo00<0XZ:R
00l?3`3_knl0[03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJD@0oooo00<0gmoO01/K6`3Bd]80103oool0
0`3Ogml06a/K0=;BdP060?ooo`030::RXP11@D40moOg0080oooo00<0gmoO
01/K6`3Bd]80103oool01036a/H0000000T92@3Bd]:]0?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03o
ool00`3Ogml06a/K0=;BdP040?ooo`030=oOg`0K6a/0d];B0080oooo00L0
gmoO024Q8@3_knl0oooo0<K6aP11@D40moOg0080oooo00<0gmoO01/K6`3B
d]801P3oool00`36a/H0@D510?Ogm`2/0?ooo`0305EEE@3oool0oooo00@0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY@P3oool01@3gmoL0
7AdM000000092@T0d];B00T0oooo00H0moOg01dM7@2d];@0oooo07QhN01P
H6080?ooo`070?_kn`1DE5@0];Bd0?ooo`36a/H0@D510?Ogm`2/0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYA03oool00`1hN7P02@T90=;BdP0:0?ooo`050=oOg`0B4Q80000000l?
3`3_knl02@3oool01@3Ogml04Q8B000000092@T0d];B0:d0oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooo`h0oooo1@1EEED60?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Wi
n@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI
0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`
l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0
inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo6@3oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0EEEE
0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Wi
n@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool>
0?ooo`D0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTm0?ooo`050?_kn`1DE5@000000000001PH6000P3o
ool03P1hN7P00000061PH02RXZ8000000451@@2JVYX000000451@@33`l<0
?clo0000000?3`l0kno_0`3oool0202RXZ8000000451@@33`l<0?clo0000
000?3`l0kno_0`3oool01P2RXZ800000000000000000@D510?Ogm`<0oooo
0140N7Qh0000002d];@0oooo07QhN00000003`l?0>_[j`1DE5@0H61P0=oO
g`0B4Q80@D510?Ogm`1hN7P000000;Bd]0030?ooo`0C0?_kn`1DE5@00000
00T92@3Bd]80oooo07QhN00000003`l?0>o_k`2RXZ8000000451@@2JVYX0
00000451@@3GemL06a/K0=;BdP210?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?`3oool00`2RXZ80Shn?
0?ooo`020?ooo`0>0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1MGEd0
d];B0<_;b`3Bd]80oooo0<K6aP2?Shl30?ooo`080?_kn`1MGEd0d];B0<_;
b`3Bd]80oooo0<K6aP2?Shl30?ooo`0507QhN02d];@0no_k05eMG@3Bd]80
0`3oool04@3kno/0GEeM0=;BdP3oool0a/K6061PH03oool0moOg07emO@3O
gml0LW9b0?_kn`1MGEd0d];B0?_kn`1MGEd0d];B00@0oooo0100XZ:R061P
H03oool0moOg0:BTY036a/H0H61P0?ooo`3gmoL0OGem0?_kn`1MGEd0d];B
0?_kn`1MGEd0d];BQ03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJCl0oooo00<0XZ:R08n?S`3oool00P3o
ool02@3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP020?oo
o`030::RXP000000];Bd00<0oooo00<0no_k05eMG@3Bd]800P3oool00`2R
XZ8000000;Bd]0030?ooo`050<K6aP0?3`l0jn_[05eMG@3Bd]800`3oool0
1P3kno/0GEeM0=;BdP3oool0XZ:R0;Bd]0@0oooo00L0bl_;03lo?`2?Shl0
oooo0?_kn`1MGEd0d];B00@0oooo00<0N7Qh0;Bd]03oool00P3oool00`2R
XZ80];Bd0?ooo`020?ooo`060?_kn`1MGEd0d];B0?_kn`1MGEd0d];BQ03o
ool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJCl0oooo00<0XZ:R08n?S`3oool00P3oool03@3gmoL0;2`/0>o_
k`3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP3Ogml04Q8B0451@@3gmoL0103o
ool01`3kno/0GEeM0=;BdP3Ogml04Q8B0451@@3gmoL01@3oool0103gmoL0
7AdM00T92@3Bd]830?ooo`050?_kn`1MGEd0d];B0?ooo`2RXZ800`000000
2@1PH600oooo0<K6aP0?3`l0kno_0?ooo`3kno/0GEeM0=;BdP040?ooo`03
0861P@3Bd]80oooo0080oooo00/0XZ:R0000000000000000061PH03kno/0
GEeM0=;BdP3kno/0GEeM0=;BdP240?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?`3oool00`2RXZ80Shn?
0?ooo`020?ooo`0>0?Ogm`0M7Ad0H61P0?Ogm`0V9RH0d];B07QhN0092@T0
d];B0<K6aP2?Shl0oooo0861P@3Bd]830?ooo`0807QhN0092@T0d];B0<K6
aP2?Shl0oooo0861P@3Bd]830?ooo`050::RXP2?Shl0no_k05eMG@3Bd]80
0`3oool04@3Kfm/0GEeM0=;BdP3oool0a/K608n?S`3oool0N7Qh08n?S`3g
moL0;2`/0>OWi`2TY:@0oooo0=_Kf`1MGEd0d];B00@0oooo0100XZ:R08n?
S`3oool0N7Qh0;Bd]036a/H0Shn?0?ooo`1hN7P0Shn?0?_kn`1MGEd0d];B
0?_kn`1MGEd0d];BQ03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJCl0oooo00<0XZ:R08n?S`3oool00P3o
ool01@3gmoL0;2`/0;Jf]P000000@D510080moOg00L09RHV0=;BdP3kno/0
E5AD0000002FUYH0fm_K00<0oooo00P0moOg02HV9P3Bd]80no_k05ADE000
0000UYJF0=_Kf`<0oooo00D0gmoO018B4P000000@D510?Ogm`030?ooo`0?
0<K6aP00000000000;Bd]03oool0N7Qh00000011@D40lo?c05ADE0000000
TY:B018B4P2?Shl0a/K60080000000<0];Bd0?ooo`3oool00P3oool03`1h
N7P0000000l?3`3_knl0oooo07QhN0000000@D510?Ogm`3kno/0GEeM0=;B
dP3kno/0GEeM0=;BdP240?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY?`3oool00`2RXZ80Shn?0?ooo`02
0?ooo`030?Ogm`0/;2`0kno_0280oooo00<0PH610=;BdP3oool02`3oool0
0`21PH40d];B0?ooo`0=0?ooo`060?_kn`1MGEd0d];B0?_kn`1MGEd0d];B
Q03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJC`0oooo00/0Z:RX0=;BdP3oool0XZ:R08n?S`3oool0a/K6
08^;R`1DE5@03`l?0>o_k`0R0?ooo`030:RXZ03Bd]80oooo00/0oooo00<0
Z:RX0=;BdP3oool03@3oool01P1hN7P02@T90=;BdP1hN7P02@T90=;BdX@0
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030::RXP000000000000@0000000@0Shn?0=oOg`0Q
8B40kno_0`3oool00`3gmoL09RHV0=;BdP080?ooo`030?Ogm`0V9RH0d];B
02l0oooo00H0moOg02HV9P3Bd]80moOg02HV9P3Bd]:40?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3o
ool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool?0?ooo`@0EEEE1P3o
ool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00
lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0
EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooo`h0oooo1@1EEED60?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolI0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_
0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3k
no/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo6@3o
ool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00
lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3o
ool0oooo0080oooo00<0k>c/0?3`l02>SXh01@2PX:000`2OWil0X:2P0:2P
X0040:2PX00F092@T03/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0=kN
gP0<0=?Cd`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI
0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo6@3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gM
g@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OG
e`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI0?oo
o`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo3P3oool505EEE@H0
oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
703oool00`1hN7P000000000000500000003061PH03oool0oooo00@0oooo
00<0no_k05ADE00000000P0000005@0?3`l0kno_0?ooo`3oool0moOg01dM
7@000000@D510?Ogm`3oool0moOg01dM7@000000@D510?Ogm`3oool0moOg
01dM7@000000@D510?Ogm`030?ooo`030<K6aP00000000000080000000<0
];Bd0?ooo`3oool0203oool0101hN7P0000000l?3`3_knl20?ooo`050?Og
m`0M7Ad000000451@@3gmoL00`3oool00`36a/H000000000000200000003
0;Bd]03oool0oooo00<0oooo00T0a/K600000000000000000451@@3gmoL0
oooo0?_kn`1DE5@00`0000000`0?3`l0kno_0?ooo`020?ooo`0507QhN000
00003`l?0>_[j`1DE5@00P0000000`0?3`l0kno_0?ooo`040?ooo`030<K6
aP00000000000080000000<0];Bd0?ooo`3oool0103oool00`36a/H00000
00000002000000030;Bd]03oool0oooo0080oooo00@0N7Qh000000092@T0
d];B0P3oool01036a/H000000000002?ShmQ0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703oool00`1hN7P0
0000000000060000000300T92@3Bd]80oooo0080oooo00<0gmoO018B4P00
00001@0000005P2d];@0oooo0?Ogm`0M7Ad000000451@@3gmoL0oooo0?Og
m`0M7Ad000000451@@3gmoL0oooo0?Ogm`0M7Ad000000451@@3gmoL0oooo
0?_kn`1DE5@6000000030451@@3gmoL0oooo00L0oooo00@0N7Qh0000000?
3`l0kno_0P3oool0203gmoL07AdM00000011@D40moOg0?ooo`3kno/0E5AD
1P0000001P11@D40moOg0?ooo`3oool0moOg01dM7@<0000000@0H61P0?oo
o`3Ogml04Q8B1P0000001`2d];@0oooo0?ooo`1hN7P0000000T92@0o?cl0
100000001P092@T0d];B0?ooo`3oool0no_k05ADE0H0000000H0@D510?Og
m`3oool0oooo0?_kn`1DE5@60000000>0451@@3gmoL0oooo0?ooo`1hN7P0
000000T92@3Bd]80oooo0?_kn`1DE5@0000000l?3`3_knmQ0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703o
ool00`1hN7P00000000000070000000500l?3`3_knl0oooo0?_kn`1DE5@0
0P0000006P2d];@0oooo0?Ogm`0M7Ad0000000l?3`3_knl0moOg01dM7@00
0000@D510?Ogm`3oool0moOg01dM7@000000@D510?Ogm`3oool0moOg01dM
7@000000@D510?Ogm`3oool0XZ:R0P000000101PH600oooo0?ooo`2RXZ82
0000000308n?S`3oool0oooo00H0oooo00@0N7Qh0000000?3`l0kno_0P3o
ool01`3gmoL07AdM00000011@D40moOg0?ooo`2RXZ800P000000101PH600
oooo0?ooo`2RXZ820000000708n?S`3oool0oooo0=oOg`0B4Q800000061P
H0020?ooo`0=0?_kn`1DE5@000000000002d];@0oooo0?Ogm`0M7Ad00000
00l?3`3_knl0oooo07QhN0020000000400l?3`3_knl0oooo0<K6aP800000
00@0@D510?Ogm`3oool0XZ:R0P000000101PH600oooo0?ooo`2RXZ820000
000408n?S`3oool0oooo0::RXP80000000@0H61P0?ooo`3oool0XZ:R0P00
00002@2?Shl0oooo0?ooo`1hN7P0000000T92@3Bd]80oooo0<K6aP020000
000308n?S`3oool0oooo0600oooo00<0EEEE0?ooo`3oool0103oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`0407QhN00000002@T90=;B
dP<0oooo00X0moOg01dM7@000000000008n?S`3oool0gmoO018B4P000000
H61P1`3oool0403gmoL07AdM00000011@D40moOg0?ooo`3gmoL07AdM0000
0011@D40moOg0?ooo`3gmoL07AdM00000011@D420?Ogm`0401dM7@000000
@D510?Ogm`80oooo00D0no_k05ADE00000003`l?0>o_k`070?ooo`0407Qh
N00000003`l?0>o_k`80oooo00@0moOg01dM7@000000@D510P3gmoL0100M
7Ad000000451@@3gmoL20?ooo`0:0?_kn`1DE5@0000000l?3`3_knl0oooo
0=oOg`0B4Q800000061PH080oooo00@0gmoO018B4P000000H61P203oool0
101hN7P000000000002d];@30?ooo`0907QhN00000002@T90=;BdP3gmoL0
7AdM00000011@D40moOg0080oooo00X0no_k05ADE00000003`l?0>o_k`3g
moL07AdM00000011@D40moOg0P3oool03`3kno/0E5AD0000000?3`l0kno_
0?ooo`1hN7P0000000T92@3Bd]80no_k05ADE00000003`l?0>o_k`1R0?oo
o`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUY703oool0101hN7P0000000T92@3Bd]840?ooo`060=oOg`0B4Q800000
0451@@3gmoL0a/K60P0000000`2?Shl0oooo0?ooo`050?ooo`0E0?Ogm`0M
7Ad000000451@@3gmoL0oooo0?Ogm`0M7Ad000000451@@3gmoL0oooo0?Og
m`0M7Ad000000451@@3gmoL0gmoO018B4P000000H61P00@0oooo00@0XZ:R
000000000000];Bd1`3oool0101hN7P0000000l?3`3_knl20?ooo`090?Og
m`0M7Ad000000451@@3gmoL0gmoO018B4P000000H61P00@0oooo00T0XZ:R
000000000000];Bd0?ooo`3Ogml04Q8B0000001PH6000P3oool01036a/H0
00000000002?Shl80?ooo`0407QhN00000002@T90=;BdP<0oooo00P0XZ:R
000000000000];Bd0=oOg`0B4Q800000061PH0@0oooo00P0XZ:R00000000
0000];Bd0=oOg`0B4Q800000061PH0@0oooo00H0XZ:R000000000000];Bd
0?ooo`1hN7P20000000500l?3`2BTY8000000000002d];@0H`3oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0
oooo00@0N7Qh000000092@T0d];B103oool01P3kno/0E5AD0000000?3`l0
kno_0<K6aPT0000001<0];Bd0?Ogm`0M7Ad000000451@@3gmoL0oooo0?Og
m`0M7Ad000000451@@3gmoL0oooo0?Ogm`0M7Ad000000451@@3gmoL0a/K6
0080000000<0Shn?0?ooo`3oool00P3oool0102RXZ8000000000002d];@7
0?ooo`0407QhN00000003`l?0>o_k`80oooo00H0moOg01dM7@000000@D51
0?Ogm`36a/H20000000308n?S`3oool0oooo0080oooo00T0XZ:R00000000
0000];Bd0?ooo`3Ogml04Q8B0000001PH6000P3oool00`36a/H000000000
0007000000060;Bd]03oool0N7Qh0000000?3`l0kno_0`3oool01@2RXZ80
00000000002d];@0a/K60080000000<0Shn?0?ooo`3oool00P3oool01@2R
XZ8000000000002d];@0a/K60080000000<0Shn?0?ooo`3oool00P3oool0
1P2RXZ8000000000002d];@0oooo07QhN0D0000000<0@D510?Ogm`3oool0
HP3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJA`0oooo00@0N7Qh000000092@T0d];B1@3oool01@1hN7P00000
00T92@3Bd]80a/K600T0000001H0];Bd0?Ogm`0M7Ad000000451@@3gmoL0
oooo0?Ogm`0M7Ad000000451@@3gmoL0oooo0?Ogm`0M7Ad000000451@@3g
moL0gmoO018B4P000000H61P103oool0102RXZ8000000000002d];@70?oo
o`0407QhN00000002@T90=;BdP80oooo00T0moOg01dM7@000000@D510?Og
m`3Ogml04Q8B0000001PH600103oool02@2RXZ8000000000002d];@0oooo
0=oOg`0B4Q800000061PH0020?ooo`030<K6aP000000000000L0000000H0
];Bd0?ooo`1hN7P0000000l?3`3_knl30?ooo`080::RXP00000000000;Bd
]03Ogml04Q8B0000001PH6040?ooo`080::RXP00000000000;Bd]03Ogml0
4Q8B0000001PH6040?ooo`090::RXP00000000000;Bd]03oool0N7Qh0000
00092@T0?clo0080000000<0];Bd0?ooo`3oool0HP3oool00`1EEED0oooo
0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo00@0
N7Qh000000092@T0d];B1@3oool0201hN7P0000000T92@3Bd]80gmoO018B
4P000000Shn?0`3oool0502RXZ80000000T92@3Bd]80moOg01dM7@000000
3`l?0>o_k`3oool0moOg01dM7@0000003`l?0>o_k`3oool0moOg01dM7@00
0000@D510P3gmoL0100M7Ad000000451@@3gmoL20?ooo`050?_kn`1DE5@0
000000l?3`3_knl01`3oool0101hN7P000000000002d];@20?ooo`040?Og
m`0M7Ad000000451@@80moOg00@07AdM00000011@D40moOg0P3oool02P3k
no/0E5AD0000000?3`l0kno_0?ooo`3Ogml04Q8B0000001PH6020?ooo`04
0=oOg`0B4Q80000008n?S`<0oooo00H0XZ:R000000092@T0d];B0?ooo`1h
N7P20000000=0;Bd]03oool0oooo0?ooo`1hN7P0000000T92@3Bd]80moOg
01dM7@000000@D510?Ogm`020?ooo`0:0?_kn`1DE5@0000000l?3`3_knl0
moOg01dM7@000000@D510?Ogm`80oooo00d0no_k05ADE00000003`l?0>o_
k`3oool0N7Qh000000092@T0b/[:01dM7@000000Shn?06@0oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?oo
o`0407QhN00000002@T90=;BdPD0oooo0140N7Qh000000092@T0d];B0?oo
o`1hN7P0000000l?3`3_knl0oooo0?Ogm`0M7Ad00000061PH03oool0moOg
01dM7@020000000308n?S`3oool0a/K600<0000000<0Shn?0?ooo`36a/H0
0P000000101PH600oooo0?ooo`2RXZ8200000004061PH03oool0oooo0::R
XP80000000<0H61P0?ooo`3oool01P3oool01`1hN7P000000000000?3`l0
kno_0?ooo`2RXZ800P000000101PH600oooo0?ooo`2RXZ8200000004061P
H03oool0oooo0::RXP80000000L0H61P0?ooo`3oool0gmoO018B4P000000
H61P00<0oooo00T0N7Qh0000000?3`l0kno_0?ooo`3gmoL07AdM0000001P
H6000P3oool01`1hN7P000000000000?3`l0kno_0?ooo`36a/H00P000000
1011@D40moOg0?ooo`2RXZ8200000004061PH03oool0oooo0::RXP800000
00@0H61P0?ooo`3oool0XZ:R0P000000101PH600oooo0?ooo`2RXZ820000
0008061PH03oool0oooo07QhN00000002@T90=;BdP36a/H20000000308n?
S`3oool0oooo0640oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTL0?ooo`0407QhN00000002@T90=;BdP@0oooo
00P0no_k05ADE00000003`l?0>o_k`3oool0moOg01dM7@D0000000D03`l?
0>o_k`3oool0moOg01dM7@0=000000050;Bd]03oool0oooo0?_kn`1DE5@0
1P0000000`11@D40moOg0?ooo`070?ooo`0407QhN00000002@T903lo?`D0
000000D0];Bd0?ooo`3oool0no_k05ADE006000000040451@@3gmoL0oooo
07QhN0D0000000@0H61P0?ooo`3gmoL07AdM1@0000001@0?3`l0kno_0?oo
o`3oool0N7Qh00L0000000H02@T90=;BdP3oool0oooo0?_kn`1DE5@60000
00060451@@3gmoL0oooo0?ooo`3kno/0E5AD1P0000002P11@D40moOg0?oo
o`3oool0N7Qh000000092@T0d];B0?ooo`1hN7P2000000030;Bd]03oool0
oooo0600oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3@d=00clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo
00@0N7Qh000000092@T0d];B103oool01@3Ogml04Q8B00000011@D40moOg
0080oooo00<0no_k05ADE00000000P0000002@11@D40moOg0?ooo`3oool0
moOg01dM7@000000Shn?0<K6aP030000000308n?S`3oool0a/K600<00000
00<0];Bd0?ooo`3oool00`3oool00`36a/H00000000000020000000308n?
S`3oool0oooo00P0oooo00D0N7Qh0000000?3`l0kno_07QhN00300000003
0;Bd]03oool0oooo00<0oooo00<0a/K60000000000000P0000001@2?Shl0
oooo0?ooo`3oool0N7Qh00D0000000D0H61P0?ooo`3oool0no_k05ADE003
000000030451@@3gmoL0oooo0080oooo00D0N7Qh0000000?3`l0jn_[05AD
E0020000000300l?3`3_knl0oooo00@0oooo00<0a/K60000000000000P00
00000`2?Shl0oooo0?ooo`040?ooo`030<K6aP00000000000080000000<0
Shn?0?ooo`3oool00P3oool02P1hN7P0000000T92@3Bd]80oooo0?Ogm`0M
7Ad0000000T92@3Bd]9Q0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTL0?ooo`0407QhN00000002@T90=;BdP<0oooo00D0gmoO018B
4P000000000008n?S`180?ooo`040=oOg`0B4Q800000061PH0h0oooo00@0
N7Qh0000000?3`l0kno_803oool0101hN7P0000000T92@3Bd]9W0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0d=3@0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`0307QhN0000000
000000L0000000<03`l?0>o_k`3oool0A`3oool0103Ogml04Q8B0000001P
H60>0?ooo`0407QhN00000003`l?0>o_kb00oooo00@0N7Qh000000092@T0
d];BI`3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00D0k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703o
ool00`1hN7P00000000000060000000300T92@3Bd]80oooo04T0oooo00<0
XZ:R0000001PH6003P3oool0101hN7P0000000l?3`3_knlP0?ooo`0407Qh
N00000002@T90=;BdVL0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJA`0oooo00<0N7Qh0000000000001@0000000`11@D40moOg0?ooo`1;
0?ooo`030=oOg`1bLW80oooo00d0oooo00@0N7Qh0000000?3`l0kno_803o
ool0101hN7P0000000T92@3Bd]9W0?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0
oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2E
UID06@3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo4P3oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00D0k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0
i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03L
g=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3oool?0?ooo`@0EEEE1P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?oo
o`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooond0
00001@3oool505EEE@80oooo1@1EEED40?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
oah0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0
gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooah0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?S
h`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooah0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[Z
jP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolN0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM
0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooah0oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolN0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3T
i>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3GemL0
303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolN0?ooo`030>c/k03moOd0Ti>C00D0^;Rh00<0]KFe0;Rh^02h^;P0
102h^;P05P2CTi<0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolN0?ooo`030>c/k03^k^h0SHf=00`0WinO01H0SHf=0>KViP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF
0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/
k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo
00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0oooo0080c/k>
00<0?Shn0<k>cP3>c/h00`3>c/h06P0n?Sh0c/k>0<k>cP3>c/h0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool0103/
k>`0UYJF0?ooo`3?cll303hn?P<0clo?0`0n?Sh20<o?c`0F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/
09JFUP3oool00P3De=@303hn?P040=CDe00n?Sh0?Shn03hn?P<0e=CD01H0
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3o
ool00`3/k>`0UYJF0?ooo`030=_Kf`D0?Shn103Kfm/05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02F
UYH0oooo00@0i>CT0`0n?Sh50>CTi00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool01@3]
kNd00`0n?Sh0kNg]0>g]k@040>g]k@0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool0303d
m?@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolN0?ooo`030>c/k02FUYH0oooo00`0o?cl01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF0?oo
o`0=0?ooo`0E09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooah0oooo00<0k>c/09JFUP3GemL0303oool05P3GemL0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`00000N000000040>c/k03Jf]X0
TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3Jf]X0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000<00000
0?Ogm`2JVYX0o`1jNWXa07YjNP0C09ZJVP3gmoL0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`2o_kl0
o`2o_klb0;no_`0B0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000030000003gmoL0hn?S0?l0hn?S<P3Shn<04P3g
moL0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
00000`000000lO7a0?ooo`3Z0?ooo`030>?Sh`2UYJD0MWIf00<0MWIf00@0
MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0hn?S0:FU
Y@@0MWIf00@0MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3o
ool0jn_[0:FUY@@0EUIF00@0EeMG05IFEP1GEeL0EeMG2@1FEUH00`2UYJD0
jn_[0?ooo`030?ooo`0B0?7al@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003QhN40o?cl0?_kn`80o?cl00<0
no_k0?clo03lo?`0103lo?`00`3kno/0o?cl0?clo0020?clo0030?_kn`3l
o?`0no_k00D0o?cl00<0no_k0?clo03kno/0D@3kno]o0?clo0030:FUY@1f
MWH0SHf=0100SHf=00H0MWIf0:FUY@3lo?`0o?cl0:FUY@1fMWHA08f=S@06
07IfMP2UYJD0o?cl0?_kn`2UYJD0EUIF4@1KFe/00`1FEUH0YJFU0?clo003
0?clo00B0>7Qh@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000030000003Ph>00moOg0>X0moOg00<0MWIf08f=S@2RXZ80
402RXZ801P2=SHd0MWIf0?Ogm`3gmoL0MWIf08f=SA40XZ:R00P0SHf=07If
MP3gmoL0moOg05IFEP1KFe/0Kfm_075aL@d0LW9b00@0LG5a06m_K`1KFe/0
EUIF103gmoL04P3Ph>00000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo00000`000000g]kN0?7al@020?7al@80l_;b00<0lO7a
0?;blP3bl_80103alO400`3bl_80lO7a0?7al@030?7al@80l_;b00<0lO7a
0?;blP3alO400P3bl_;B0?7al@0307IfMP2h^;P0Zj^[0100Zj^[00H0^;Rh
07IfMP3alO40lO7a07IfMP2h^;PA0:^[Z`050;Rh^01fMWH0lO7a0?;blP1F
EUH00P1_Kfl00`1bLW80MgMg07MgM`0;07MgM`04079bLP1_Kfl0Kfm_05IF
EP<0lO7a01<0l_;b0=kNgP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo00001@000000g=cL0>_[j`3[jn/0k>c/0080jn_[
00<0YjNW0::RXP2RXZ800P2QXJ400`2PX:00WinO09fMW@0209nOW`80X:2P
0P2QXJ400`2WYjL0jn_[0>_[j`050>_[j`050=CDe035aLD0aLG50<G5a@3G
emL00P3[jn/00`3Zj^X0b<S80=3@d0020>_[j`050=[JfP36a/H0hn?S0>_[
j`3Ph>000P35aLD01`3<c<`0jNWY0<k>cP37alL0jNWY0>_[j`3Ph>000P35
aLD01`3<c<`0j^[Z0=CDe035aLD0gMgM0>_[j`3Ph>000P35aLD01P36a/H0
hn?S0>_[j`3[jn/0j^[Z0<S8b0<0aLG500<0a/K60>?Sh`3[jn/01`3[jn/0
0`3UiND0alO70=OGe`0F0>_[je<0k>c/;03[jn/0101fMWH0^k^k0;6a/@2a
/K480?ooo`L0/K6a00H0^k^k07IfMP3[jn/0jn_[07IfMP2k^k/20;6a/@`0
oooo0`2a/K402@2k^k/0MWIf0>_[j`3[jn/0EUIF07MgM`1bLW80M7Ad07Mg
M`0;07QhN00507MgM`1dM7@0LW9b07MgM`1FEUH0103[jn/04P3Lg=`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`00
0000f][J0>GUi@030>GUi@0308Z:RP2KVi/0[:b/00<0jNWY00X0j^[Z0<C4
a025QHD0k>c/0>g]k@3^k^h0inOW0?3`l03bl_80Xj>S1P3UiND04@1XJ6P0
410@010@400@4100P8200<K6aP3?cll0g]kN02HV9P1ADE40clo?0>GUi@2D
U9@05ADE0:BTY03Ng]h0/[:b0080410@00L0@D510=_Kf`1@D5006ATI0;Zj
^P3Shn<0/[:b0080410@00L0@D510=oOg`1`L700410@09:BTP3Ti>@0[jn_
0080410@00H05ADE0;Zj^P3<c<`0i>CT0=gMg@0T92@3010@400301DE5@2T
Y:@0g]kN00L0iNGU00@0bLW901`L701bLW80e=CDU@3UiND01@1fMWH0`<30
0;Fe]@2e]KD0oooo00H0LW9b00<09bLW0;Fe]@2e]KD01@2e]KD01P30`<00
MWIf0>GUi@3UiND0MWIf0<30`080]KFe00<0oooo02LW9`0W9bL02@0W9bL3
0;Fe]@050<30`01fMWH0iNGU0>GUi@1FEUH00P1gMgL0101iNGT0NWYj0?oo
o`3oool707YjNP80oooo00D0NWYj07UiN@1gMgL0MgMg05IFEP040>GUi@0B
0=[JfP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000030000003IfMT0gmoO00<0gmoO00D0g]kN0:2PX03Zj^X0jNWY0>GU
i@020>[ZjP0:0=3@d01dM7@0kNg]0>k^kP2][Jd0d=3@0?;blP3clo<0Xj>S
0=oOg`80g]kN03@0gmoO0=kNgP1/K6`05ADE0:^[Z`3Ng]h0`l?301`L701D
E5@0a/K60=KFeP0U9BD0DE5A0<g=c@3Ng]h0Shn?01DE5@2TY:@0eMGE04i>
CP0h>3P0a<C40=gMg@3Kfm/0CTi>01TI6@2j^[X0fMWI04i>CP0h>3P0a<C4
0=gMg@3GemL09BDU03Ph>034a<@0`/;201/K6`1ADE40fMWI0<?3``1<C4`0
f=SH0=OGe`0U9BD06ATI0<G5a@3Ng]h0Shn?01DE5@2TY:@0fMWI1`3Ng]h0
1033`l<071`L079bLP3AdM6E0=kNgP0507IfMP33`l<0^k^k0;^k^`3oool0
1P1bLW800`0W9bL0^k^k0;^k^`050;^k^`060<?3``1fMWH0g]kN0=kNgP1f
MWH0`l?30P2k^k/00`3oool09bLW0;^k^`070;^k^`030?ooo`0W9bL0^k^k
0080^k^k00T0`l?307IfMP3Ogml0g]kN05IFEP1gMgL0N7Qh07]kN`0`<300
0`3oool507emO@<0oooo00D0<30`07]kN`1hN7P0MgMg05IFEP020=kNgP0D
0=oOg`3Ng]h0fMWI0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000=OGe`3Hf=P0fMWI0P3Hf=P0402@T900Shn?
08j>SP3YjNT0PH61055AD@27QhL0f][J0:JVYP2YZJT0[:b/05mOG`3bl_80
lo?c0?Cdm02TY:@30=SHf0050=WIf@3Bd]8092@T055AD@3;bl/00P3Hf=P0
801ZJVX06ATI0;Zj^P3@d=0092@T055AD@3;bl/0f=SH08^;R`0E5AD0Y:BT
0<g=c@0T92@0>3Ph0<?3``3GemL0eMGE04a<C00I6AT0^[Zj0=3@d00T92@0
>3Ph0<?3``3GemL0dM7A02@T900h>3P0`l?308Z:RP0E5AD0Y:BT1035aLD0
2P2NWYh0410@075aL@3AdM40f=SH08^;R`0E5AD0Y:BT0=CDe03Bd]820<G5
a@080<K6aP3GemL0f=SH0=SHf02m_Kd06a/K079bLP3>c/hF0=SHf5<0fMWI
;03Hf=P01@1fMWH0alO70;no_`2o_kl0oooo00L09bLW1`2o_kl01P37alL0
MWIf0=SHf03Hf=P0MWIf0<O7a`80_kno00<0oooo02LW9`2o_kl01`2o_kl0
0`3oool09bLW0;no_`020;no_`050<O7a`1fMWH0f=SH0=SHf01FEUH00P1k
Ng/00`1nOWh0Ogmo02PX:0030?ooo`<0Ogmo0`3oool00`0X:2P0Ogmo07in
OP0207]kN`0305IFEP3IfMT0f=SH0080f=SH0180emOG0000002JVYX0X:2P
0:RXZ02b/[80_;bl0<K6aP3@d=00fMWI0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=GEe@3C
dm<00`3Cdm<00`25QHD0Xj>S0>KViP020>[ZjP0;03De=@1oOgl0clo?0;jn
_P2;Rh/0GUiN05QHF01XJ6P0Xj>S0<W9b@2ZZZX0103Cdm<0103<c<`092@T
055AD@3:b/X20=?Cd`0N06MWI`0I6AT0^KVi0<_;b`0T92@0DE5A0<[:bP3C
dm<0R8R801DE5@2TY:@0bLW902@T900h>3P0`L710=?Cd`3?cll0BTY:01TI
6@2i^KT0bl_;02@T900h>3P0`L710=?Cd`3<c<`092@T03Ph>031`L40IfMW
1@0@41002P1ADE40WIfM010@402>SXh0c<c<0=?Cd`28R8P05ADE0:BTY02Q
XJ42010@400301XJ6P36a/H0dm?C0080dm?C00@0^;Rh01/K6`1bLW80c<c<
U@3Cdm<00`1fMWH0bl_;0<K6aP0@0<K6aP060<_;b`1fMWH0dm?C0=?Cd`1f
MWH0bl_;0P36a/H00`3oool09bLW0<K6aP070<K6aP030?ooo`0W9bL0a/K6
0080a/K600L0bl_;07IfMP3Cdm<0dm?C05IFEP1mOGd0Ogmo00<0Ph>300D0
:2PX0?ooo`3oool0oooo08>3P`030?ooo`0702PX:023Ph<0Ph>308>3P`1o
Ogl0OGem05IFEP040=?Cd`0A0=GEe@000000X:2P0:FUY@2][Jd0][Jf0;no
_`38b<P0d];B0=[JfP3Rh^80jNWY0>o_k`3dm?@0m_Kf0?Win@3kno/00`3m
oOd10?knoP40oooo0@3oool000050000003Cdm<0c/k>0<k>cP3?cll00P3>
c/h03`27QhL0Ti>C0=?Cd`3Kfm/0[Jf]0<71`@2`/;00ZJVY0:ZZZP2=SHd0
TI6A0;2`/03;bl/0kno_0:FUY@030<k>cP050<o?c`37alL08b<S055AD@38
b<P00P3>c/h0;@1UIFD06ATI0;Vi^@37alL08b<S055AD@37alL0c/k>08F5
Q@0E5AD0Y:BT0<O7a`0S8b<0>3Ph0<71`@3>c/h0bl_;04U9B@0I6AT0^KVi
0<O7a`0S8b<0>3Ph0<71`@3>c/h0alO702<S8`0h>3P0`L7108F5Q@0E5AD0
Y:BT0<[:bP37alL08b<S055AD@2a/K406QXJ079bLP35aLD0c/k>08F5Q@0E
5AD0Y:BT0<g=c@070<k>cP040;Bd]00J6QX0LW9b0<[:bQH0c/k>D`3?cll/
0<k>cP0307IfMP3>c/h0bl_;0100bl_;00H0c/k>07IfMP3>c/h0c/k>07If
MP3>c/h20<_;b`030?ooo`0W9bL0bl_;00L0bl_;00<0oooo02LW9`3;bl/0
0P3;bl/02@3>c/h0MWIf0<o?c`3>c/h0EUIF07emO@22PX80QHF508B4Q002
08F5Q@0302PX:03oool0oooo00<0oooo00P0:2PX08F5Q@25QHD0Q8B408F5
Q@22PX80OGem05IFEP@0c/k>0140dm?C0000002XZ:P0[Jf]0;>c/`2k^k/0
`l?30<c<c03EeMD0gMgM0>CTi03[jn/0l?3`0?Cdm03gmoL0nOWi0?clo003
0?gmo@40o_kn0@3oool10?ooo`0000<000000=3@d039bLT00`39bLT04@1c
Lg<0U9BD07inOP1iNGT0X:2P0=7Ad@2][Jd0YJFU0:JVYP25QHD0GUiN0:f]
[@3UiND0m_Kf0?Shn02TY:@0bLW900<0b<S800@0`/;202<S8`1ADE40alO7
0P38b<P0;01RHV806ATI0;Vi^@32`/808R8R03Ph>031`L40`/;204I6AP0I
6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35aLD0a<C404I6AP0I6AT0^KVi0<71
`@0S8b<0>3Ph0;no_`35aLD0`<30028R8P0h>3P0`<300:n_[`0J6QX0Lg=c
0<G5a@2JVYX0410@08j>SP35aLD0ATI601TI6@2i^KT0aLG507moO`0E5AD0
Y:BT1`38b<P01@37alL0[Zj^01XJ6P1aLG40alO701H0b<S8D`39bLT/0<S8
b00307IfMP3Cdm<0cLg=0100cLg=00H0dm?C07IfMP38b<P0b<S807IfMP3C
dm<20<g=c@030?ooo`0W9bL0cLg=00L0cLg=00<0oooo02LW9`3=cLd00P3=
cLd01`3Cdm<0MWIf0<S8b038b<P0EUIF08:2PP23Ph<01@27QhL01@0X:2P0
oooo0?ooo`3oool0:2PX00D0QhN700<0Ph>308:2PP1FEUH00`39bLT04P38
b<P0d=3@0000002b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OW
i`3]kNd0lO7a0?Gem@3hn?P0n_[j0?clo0<0oOgm0@3no_h10?ooo`40oooo
00000`000000clo?0<G5a@040<G5a@0?09NGU`2g]kL0k>c/0;Vi^@2;Rh/0
]kNg0:ZZZP2TY:@0OWin075aL@34a<@0fMWI0?KfmP3inOT0YJFU00@0aLG5
00@0_kno028R8P1ADE40a/K60P35aLD01P1QHF406ATI0;Vi^@2o_kl08R8R
05eMG@80410@00@05ADE0:BTY032`/80ADE50`0@41001P1ADE40HF5Q010@
400I6AT0][Jf04E5A@<0410@00P0DE5A0;no_`0R8R80>3Ph0;no_`35aLD0
[:b/01XJ6P80410@00D0LW9b0<G5a@35aLD0`/;204E5A@03010@400301DE
5@2TY:@0a/K600D0aLG500H0[:b/05aLG02][Jd06QXJ075aL@36a/JE0<G5
a@0307IfMP3EeMD0dM7A0100dM7A00H0eMGE07IfMP35aLD0aLG507IfMP3E
eMD20=7Ad@030?ooo`0W9bL0dM7A00L0dM7A00<0oooo02LW9`3AdM400P3A
dM401`3EeMD0MWIf0<G5a@35aLD0EUIF08:2PP27QhL01@29RHT50?ooo`D0
RHV900<0QhN708:2PP1FEUH01035aLD04@3?cll000000;bl_02o_kl0`l?3
0<W9b@3@d=00emOG0=gMg@3Ti>@0j^[Z0>o_k`3clo<0m_Kf0?Shn03kno/0
o?cl0080oOgm0P3no_h20?ooo`0000<000000<g=c@31`L400`31`L40401e
MGD0VIVI0:NWY`3]kNd0bLW907=cL`2`/;00SHf=07=cL`1<C4`0HV9R0;jn
_P2e]KD0/[:b0:f][@2YZJT40<71`@040;^k^`0R8R80DE5A0<C4a080`L71
00@0GemO01TI6@2i^KT0`/;21`31`L401@2k^k/08R8R03Ph>02n_[h0`/;2
0080`L7100`0`l?30<G5a@2l_;`08R8R03Ph>02n_[h0`/;20;^k^`0R8R80
>3Ph0;jn_P32`/8;0<71`@0407alO00E5AD0Y:BT0<?3``D0`L7100H0ZJVY
01XJ6P0@4100410@075aL@34a<BE0<71`@0307IfMP3Hf=P0eMGE0100eMGE
00H0f=SH07IfMP31`L40`L7107IfMP3Hf=P20=GEe@030?ooo`0W9bL0oooo
00P0oooo00H09bLW0=GEe@3EeMD0eMGE0=SHf01fMWH20<71`@0305IFEP24
Q8@0R8R800@0RXZ:0`3oool0100X:2P0oooo0?ooo`3oool408Z:RP0308R8
R024Q8@0EUIF00@0`L710100cLg=00000036a/H0b<S80<c<c03AdM40emOG
0=cLg03Rh^80j>SX0>g]k@3alO40mOGe0?Ogm`3inOT0o?cl0`3moOd20?kn
oP80oooo00000`000000c<c<0;Zj^P020;fm_@0A0;jn_P2m_Kd0WYjN0>?S
h`2f][H0UYJF08:2PP2CTi<0Q8B408:2PP1>CTh0;2`/09bLW02[Zj/0m_Kf
0?Shn02WYjL0102m_Kd0102g]kL08B4Q055AD@34a<@20;fm_@0405eMG@0I
6AT0^KVi0;no_`P0_Kfm00<0GEeM03Lg=`2h^;P00P2m_Kd02`1MGEd06ATI
0;Vi^@2o_kl0GEeM03Lg=`2h^;P0_Kfm0;Ng]`0Q8B40>3Ph0080_[jn2`2m
_Kd0101jNWX05ADE0:BTY02o_kl60;fm_@050:FUY@0J6QX0410@071`L032
`/805P2m_KeC0;jn_R`0_Kfm00<0MWIf0=[JfP3EeMD0403EeMD01P3Jf]X0
MWIf0;fm_@2m_Kd0MWIf0=[JfP80eMGE00<0oooo079bLP1bLW80201bLW80
3P0W9bL0eMGE0=GEe@3EeMD0f][J07IfMP2m_Kd0_[jn05IFEP27QhL0RXZ:
08f=S@2>SXh0SHf=0`3oool00`0X:2P0SHf=02PX:0030?ooo`<0SHf=00@0
RXZ:08N7Q`1FEUH0_[jn0P2m_Kd04@2j^[X0c<c<0000003@d=00d];B0=GE
e@3IfMT0gMgM0>;RhP3WinL0k>c/0?3`l03dm?@0m_Kf0?Shn03jn_X0o?cl
00<0oOgm00<0o_kn0?ooo`3oool00@3oool000030000003:b/X0^;Rh00<0
^[Zj0100O7al08f=S@1hN7P0VIVI0?3`l032`/80YjNW0<_;b`2l_;`0Xj>S
06e]K@1mOGd0[:b/0?_kn`3lo?`0YZJV0`2j^[X0102k^k/0];Bd024Q8@1=
CDd30;Zj^P0305]KF`0I6AT0[jn_00h0^[Zj00<0Fe]K01TI6@2_[jl01@2j
^[X0102d];@08B4Q03Lg=`2d];@<0;Zj^P0307MgM`0D51@0Vi^K00T0^[Zj
00<0Fe]K06][J`2j^[X0U@2j^[X00`1fMWH0f=SH0=[JfP0@0=[JfP060=SH
f01fMWH0^[Zj0;Zj^P1fMWH0f=SH0P3Jf]X00`3oool0LW9b079bLP08079b
LP0=02LW9`3Jf]X0f][J0=[JfP3Hf=P0MWIf0;Zj^P2k^k/0EUIF08^;R`2?
Shl0SXj>08n?S`030?ooo`0502PX:02?Shl0Shn?08n?S`0X:2P00`3oool0
1@2?Shl0SXj>08n?S`2;Rh/0EUIF00<0^[Zj0100^;Rh0<[:bP000000fMWI
0=[JfP3MgMd0h>3P0>CTi03Xj>P0k>c/0>o_k`3clo<0m_Kf0?Shn03jn_X0
o?cl0`3moOd20?knoP<0oooo00000`000000bLW90;Rh^0030;Rh^00@08J6
QP2NWYh0e=CD0?3`l03bl_80c/k>0<71`@3MgMd0d];B0::RXP3<c<`0bLW9
08V9R@3lo?`0oOgm0:NWYmL0^;Rh00<0MWIf0=OGe`3MgMd0403MgMd01P3G
emL0MWIf0;Rh^02h^;P0MWIf0=OGe`<0gMgM2`0W9bL30=gMg@090=OGe`1f
MWH0^;Rh0;Rh^01FEUH0T92@096AT@2CTi<0U9BD0080oooo00<0:2PX09RH
V02HV9P00`2HV9P0200X:2P0oooo0?ooo`2DU9@0Ti>C096AT@2@T900EUIF
102h^;P03`39bLT000000>7Qh@3Rh^80i>CT0>OWi`3Zj^X0kNg]0?3`l03c
lo<0mOGe0?Ogm`3inOT0no_k0?clo0030?gmo@030?knoP3oool0oooo0080
oooo00000`000000bLW90;Rh^0040;Rh^00?08J6QP2DU9@0l_;b0??cl`2P
X:00j>SX0?KfmP3Kfm/0VIVI0?[jnP3kno/0g=cL0;Jf]P3no_h0YjNW0=L0
^;Rh00<0MWIf0=KFeP3QhN40403QhN401P3Fe]H0MWIf0;Rh^02h^;P0MWIf
0=KFeQ40hN7Q00T0e]KF07IfMP2h^;P0^;Rh05IFEP2EUID0UiNG09bLW02R
XZ800P0X:2P01@2[Zj/0[Jf]0:^[Z`2[Zj/0[Jf]0080Zj^[0P0X:2P01@2R
XZ80W9bL09NGU`2EUID0EUIF00@0^;Rh00h0bLW90000003Xj>P0jNWY0>_[
j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0<0oOgm0P3n
o_h40?ooo`0000<000000<W9b@2e]KD00`2e]KD03@1gMgL0X:2P0>3Ph03c
lo<0iNGU0>c/k03fm_H0n?Sh0?Win@2@T900no_k0?clo03moOd00P3no_h0
0`2WYjL0]KFe0;Fe]@3E0;Fe]@0307IfMP3]kNd0jNWY0100jNWY00H0kNg]
07IfMP2e]KD0]KFe07IfMP3]kNdA0>WYj@0:0>g]k@1fMWH0]KFe0;Fe]@1F
EUH0Vi^K09jNWP2UYJD0/[:b0;Zj^P80_kno00@0`<300;no_`2o_kl0`<30
0P2o_kl01`30`<00^[Zj0;:b/P2UYJD0WYjN09^KV`1FEUH0102e]KD03@39
bLT000000>k^kP3_knl0l?3`0?7al@3clo<0mOGe0?KfmP3hn?P0nOWi0?_k
n`3lo?`00`3moOd20?knoPD0oooo00000`000000bLW90;Zj^P040;6a/@03
08f=S@2ZZZX0ZZZZ00D0ZZZZ00<0YjNW0:ZZZP2ZZZX0102ZZZ[G0;6a/@03
07IfMP3De=@0l_;b0100l_;b00H0e=CD07IfMP2a/K40/K6a07IfMP3De=@A
0?;blP060=CDe01fMWH0/K6a0;6a/@1FEUH0Xj>S0P2XZ:P=0<?3``80Z:RX
00<0Xj>S05IFEP2a/K400P2a/K40102j^[X0bLW90000003clo<20?Cdm007
0?Gem@3fm_H0moOg0?Shn03jn_X0no_k0?clo0040?gmo@030?knoP3oool0
oooo00@0oooo00000`000000`l?30<W9b@3Z0:j^[P0408f=S@1fMWH0c/k>
0?_kn`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^[Zh0102=SHd0MWIf0<k>
cP3kno/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00H0VIVI05IFEP2e
]KD0bl_;0=oOg`3Xj>P90>OWi`060>SXj03Ogml0bl_;0;Fe]@1FEUH0VIVI
0`2^[Zh00`39bLT0`l?3000000020?KfmP050?Ogm`3hn?P0n?Sh0?Win@3j
n_X00P3lo?`40?gmo@80o_kn1P3oool0000400000033`l<0e=CD0;Ng]nX0
YjNW00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`03
08f=S@1fMWH0MWIf00l0MWIf00<0SHf=0:NWY`2WYjL00P2WYjL00`2=SHd0
EUIF05IFEP0?05IFEP0808f=S@2WYjL0YjNW0:NWY`2g]kL0e=CD0<?3``00
00030?Win@040?[jnP3kno/0o?cl0?clo0D0oOgm0P3no_h70?ooo`0000H0
00000:ZZZP3WinL0e=CD0;jn_P2c/k?o0:ZZZR/0ZZZZ00H0/k>c0;jn_P3D
e=@0inOW0:ZZZP0000020?_kn`<0o?cl1@3moOd30?knoPP0oooo00001`00
0000RXZ:0=WIf@3lo?`0i>CT0=;BdP3<c<`0o`2n_[hY0;jn_P080<c<c03B
d]80i>CT0?clo03IfMT0RXZ:0000003lo?`80?gmo@80o_kn2P3oool00008
0000001YJFT0UiNG0=WIf@3lo?`0l_;b0>c/k03Shn?o0=WIfBL0fMWI00P0
hn?S0>c/k03bl_80o?cl0=WIf@2GUiL0JFUY000000L0oOgm0`3no_h;0?oo
o`0000D000000?ooo`1YJFT0UiNG0=?Cd`020?clo?l0oooo:@3oool20?cl
o0050=?Cd`2GUiL0JFUY0?ooo`000000103moOd40?knoPd0oooo00000`00
0000oooo0?ooo`0206UYJ@0308Z:RP2GUiL0ZZZZ0?l0ZZZZ:02ZZZX0102G
UiL0RXZ:06UYJ@1YJFT20?ooo`030000003no_h0o_kn00@0o_kn3`3oool0
0?l00000=`00000E0?ooo`00\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{331, 218},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}],

Cell[TextData[{
  "Let us create the notebook from scratch using a kernel command, ",
  StyleBox["NotebookPut", "MR"],
  ". ",
  StyleBox["NotebookPut[", "MR"],
  StyleBox["expr", "IT"],
  StyleBox["]", "MR"],
  " will create a notebook corresponding to ",
  StyleBox["expr",
    FontSlant->"Italic"],
  " in the front end and make it the currently selected notebook."
}], "Text",
  CellTags->{"i:2", "b:43.0", "IPM3.10.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(nb = 
      NotebookPut[\[IndentingNewLine]Notebook[{\[IndentingNewLine]Cell["\<\
Demo notebook\>", "\<Section\>"], 
            Cell["\<This is a text cell\>", "\<Text\>"], \
\[IndentingNewLine]Cell["\<1+2+3\>", "\<Input\>"]\[IndentingNewLine]}]\
\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell[BoxData[
    InterpretationBox[\(NotebookObject[\[LeftSkeleton]"Untitled-1"\
\[RightSkeleton]]\),
      NotebookObject[ 
        FrontEndObject[ 
          LinkObject[ "3v8_shm", 1, 1]], 28],
      Editable->False]], "Output",
  CellLabel->"Out[1]=",
  CellTags->{"b:43.0", "IPM3.10.2.1"}]
}, Open  ]],

Cell["Here is the notebook as viewed in the front end.", "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005;0000fR000`400?l0
0000o`00003oo`3ooom<0?ooo`00o`3ooom<0?ooo`003P3ooooo0?knoRl0
o_kn3`3oool000`0oooo103no_ko0?gmoB/0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgm<@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo2D0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k9`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinBP0nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_HW0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clbD0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hU0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj2D0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4U0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH8`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllS0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aB<0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^b<0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a8`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYR<0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhS09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXU09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000003L000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?cl<`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[Zc80Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFTa06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj3`2OWil00`2m_Kd0k>c/0>OWi`0H0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K3`3:b/X0
0`3Kfm/0Xj>S0?_kn`0H0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo3`3@d=000`3oool0UYJF0?_kn`0H0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3oool?0<o?c`03
0?ooo`2FUYH0no_k01P0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h0203oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM40d=3@0`3AdM400`3@d=00dM7A0=3@d005
0=7Ad@030?ooo`2FUYH0n_[j01P0n_[j00<0UYJF0?ooo`3>c/h0103>c/h0
0`0n?Sh0c/k>0<k>cP050<k>cP030?ooo`2FUYH0h>3P0100h>3P01@0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3P
h03Ph>00103Ph>003`34a<@0B4Q80=WIf@3Ph>00JFUY09jNWP3Ph>00SXj>
05ADE03Ph>00JFUY09jNWP3Ph>00SXj>05ADE0040>3Ph0080:f][@1DE5@0
h>3P0:f][@1DE5@0h>3P06UYJ@2NWYhF0>3Ph00309JFUP3oool0c/k>00@0
c/k>0`0n?Sh50<k>cP040?ooo`2FUYH0UYJF0?ooo`@0e]KF00P0UIFE0=OG
e`2EUID0e]KF09FEU@3Fe]H0UIFE0=OGe`<0e]KF00<0oooo09JFUP3inOT0
603inOT00`2FUYH0oooo0<k>cP030<k>cP<0?Shn1P3>c/h00`3oool0UYJF
0>3Ph0090>3Ph080oooo0P3Ph>020?ooo`0E0>3Ph01YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
1@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103P
h>004034a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00a<C404Q8B03Bd]80
9RHV0=;BdP3Ph>00a<C404Q8B03IfMT30>3Ph0080=cLg01ADE40^KVi0:f]
[@1DE5@0h>3P06UYJ@2NWYhF0>3Ph00309JFUP3oool0clo?00<0clo?0`0n
?Sh60<o?c`040?ooo`2FUYH0UYJF0?ooo`<0g=cL00T0oooo09FEU@3oool0
UIFE0?ooo`2EUID0oooo09FEU@3MgMd00P3Lg=`00`3MgMd0oooo09JFUP0I
0?Win@0309JFUP3oool0clo?00@0clo?0`0n?Sh50<o?c`030?ooo`2FUYH0
h>3P00P0h>3P0P2c/k<06P3oool0h>3P0;>c/`2c/k<0oooo0>3Ph01YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P
0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT0
9RHV0<_;b`0V9RH0d];B0>3Ph03IfMT09RHV0=;BdP@0h>3P00L0a<C404Q8
B03Bd]806QXJ0000000=3@d0d];B00X0h>3P1`0000050>3Ph00309JFUP3o
ool0e=CD0080e=CD0`0n?Sh70=CDe0040?ooo`2FUYH0UYJF0?ooo`<0g=cL
00T0oooo09FEU@3oool0UIFE0?ooo`2EUID0oooo09FEU@3MgMd00P3Lg=`0
0`3MgMd0oooo09JFUP0I0?Shn00309JFUP3oool0e=CD00D0e=CD0`0n?Sh4
0=CDe0030?ooo`2FUYH0h>3P00P0h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00
501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00G0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h03IfMT09RHV0<_;b`0V9RH0d];B0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU9
0000000=3@d0d];B06UYJ@2NWYh03`3Ph>05000000H0h>3P00@0UYJF0?oo
o`3Kfm/0fm_K0`0n?Sh80=_Kf`050?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3S
hn<0203oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0P3Ti>@0103S
hn<0i>CT0?ooo`2FUYHI0?Ogm`0309JFUP3oool0fm_K00H0fm_K0`0n?Sh3
0=_Kf`030?ooo`2FUYH0h>3P0100h>3P01@0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000
o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>006034
a<@0B4Q80=WIf@3IfMT09RHV0=;BdP3Ph>00a<C404Q8B03Bd]809RHV0=;B
dP3Ph>00a<C404Q8B03IfMT0[Jf]05ADE03Ph>00JFUY07inOP34a<@0B4Q8
0=WIf@l0h>3P0`0000070>3Ph00309JFUP3oool0i>CT0080i>CT0`0n?Sh7
0>CTi0050?ooo`2FUYH0UYJF0?ooo`3Zj^X00P3YjNT02P3oool0UIFE0?oo
o`2EUID0oooo09FEU@3oool0UIFE0>WYj@3Zj^X20>WYj@030?ooo`2FUYH0
mOGe01P0mOGe00<0UYJF0?ooo`3Ti>@01@3Ti>@303hn?P@0i>CT00<0oooo
09JFUP3Ph>003@3Ph>020?ooo`0E0>3Ph01YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001@000000
o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>000`3Ph>006@2>
SXh0000003Ph>03IfMT0h>3P06UYJ@2NWYh0h>3P08j>SP1DE5@0h>3P06UY
J@2NWYh0h>3P08j>SP1DE5@0h>3P0:f][@1DE5@0h>3P06UYJ@1nOWh0h>3P
06UYJ@2NWYh0403Ph>000`000000h>3P0>3Ph0060>3Ph00309JFUP3oool0
kNg]00<0kNg]0`0n?Sh60>g]k@050?ooo`2FUYH0UYJF0?ooo`3Zj^X00P3Y
jNT02P3oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0>WYj@3Zj^X2
0>WYj@030?ooo`2FUYH0m?Cd01P0m?Cd00<0UYJF0?ooo`3]kNd0103]kNd3
03hn?PD0kNg]00<0oooo09JFUP3Ph>00303Ph>020;>c/`0F0?ooo`3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo
0>3Ph03Ph>00103Ph>006@3IfMT0DU9B0=WIf@3Ph>00fMWI01XJ6P000000
3@d=0=;BdP3Ph>00fMWI01XJ6P0000003@d=0=;BdP3Ph>00fMWI01XJ6P00
00003@d=0=;BdP3Ph>00a<C401dM7@3Bd]80603Ph>000`2FUYH0oooo0?Cd
m0040?Cdm0<0?Shn1@3dm?@0103oool0UYJF09JFUP3oool30>g]k@070?oo
o`3]kNd0oooo0>g]k@3oool0kNg]0?ooo`050>g]k@030?ooo`2FUYH0lO7a
01P0lO7a00<0UYJF0?ooo`3dm?@00`3dm?@303hn?PH0m?Cd00<0oooo09JF
UP3Ph>00303Ph>020;>c/`80h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl
0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`2FUYH0
oooo0?clo0050?clo00303hn?P3lo?`0o?cl00@0o?cl00@0oooo09JFUP2F
UYH0oooo3`3]kNd00`3oool0UYJF0?3`l00H0?3`l00309JFUP3oool0o?cl
00@0o?cl00<0?Shn0?clo03lo?`01@3lo?`00`3oool0UYJF0>3Ph00@0>3P
h00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph003
0?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3oool03@3oool209JFUP03
0?ooo`3gmoL0moOg00d0moOg00<0oooo09JFUP3_knl0603_knl00`2FUYH0
oooo0?ooo`0=0?ooo`0309JFUP3Ph>00h>3P00l0h>3P01@0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
00001@000000o?cl0:^[Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00
=@3Ph>000`2FUYH0emOG0?ooo`0<0?ooo`040=OGe`2FUYH0UYJF0?[jnPl0
oooo00<0n_[j09JFUP3_knl0603_knl00`2FUYH0emOG0?ooo`0<0?ooo`03
0=OGe`2FUYH0h>3P0100h>3P01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[
Z`1YJFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00=@3Ph>000`3Jf]X0TY:B
0;Jf]P0;0;Fe]@050;Bd]02?Shl0f][J0=KFeP2`/;003`2h^;P00`2YZJT0
c/k>0>k^kP0H0>k^kP030=[JfP2BTY80][Jf00/0]KFe00<0];Bd08n?S`3J
f]X04@3Ph>00501YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJIL0WYjN
00<0oooo0>3Ph03Ph>00=@3Ph>0A0>c/k1<0jn_[6@3WinLA0>c/k140h>3P
01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0
k>c/0>k^kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0g]kN00`0fm_K01H0
g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3o
ool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool0303>
c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolN0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF0?oo
o`050<k>cP0303hn?P3>c/h0c/k>00@0c/k>01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF0?oo
o`040<k>cP<0?Shn1@3>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0oooo00<0clo?1@0n
?Sh40<o?c`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooah0oooo00<0k>c/09JFUP3oool00P3De=@303hn?P040=CDe00n
?Sh0?Shn03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF0?ooo`3Kfm/303hn?P<0
fm_K0`0n?Sh20=_Kf`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool00P3Ti>@00`0n?Sh0
i>CT0>CTi0030>CTi00J03hn?P3Ti>@0i>CT0>CTi03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0
oooo00`0kNg]01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo7P3oool00`3/k>`0UYJF0?ooo`0<0?Cdm00F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/
09JFUP3oool0303lo?`05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0oooo00d0oooo01D0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo3P3oool505EE
E@80oooo1@1EEED40?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo4P3oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/0=[J
fP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFU20?ooo`030=oOg`0B4Q8000000080000000<02@T9
0=;BdP3oool01P3oool00`2RXZ80000000000002000000030451@@3gmoL0
oooo00H0oooo00<0no_k05ADE00000000P0000000`2d];@0oooo0?ooo`2[
0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Wi
n@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYA03oool00`3Ogml06a/K0=;BdP040?ooo`030=oOg`0K6a/0
d];B00<0oooo00<0N7Qh00l?3`3_knl01@3oool00`3Ogml06a/K0=;BdP02
0?ooo`070=oOg`1CDe<0moOg0?ooo`3Ogml08B4Q0>o_k`2/0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
A03oool00`3Ogml06a/K0=;BdP040?ooo`030=oOg`0K6a/0d];B00@0oooo
00<0a/K600T92@3Bd]80103oool00`3Ogml06a/K0=;BdP060?ooo`030?Og
m`0V9RH0d];B0:`0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo
0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFU40?ooo`030=oOg`0K6a/0d];B0080oooo
00<0N7Qh0000000000000`0000002@1PH600oooo0?ooo`3oool0gmoO01/K
6`3Bd]80oooo07QhN00500000003061PH03oool0oooo0080oooo00<0XZ:R
00l?3`3_knl0[03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJD@0oooo00<0gmoO01/K6`3Bd]80103oool0
0`3Ogml06a/K0=;BdP060?ooo`030::RXP11@D40moOg0080oooo00<0gmoO
01/K6`3Bd]80103oool01036a/H0000000T92@3Bd]:]0?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03o
ool00`3Ogml06a/K0=;BdP040?ooo`030=oOg`0K6a/0d];B0080oooo00L0
gmoO024Q8@3_knl0oooo0<K6aP11@D40moOg0080oooo00<0gmoO01/K6`3B
d]801P3oool00`36a/H0@D510?Ogm`2/0?ooo`0305EEE@3oool0oooo00@0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY@P3oool01@3gmoL0
7AdM000000092@T0d];B00T0oooo00H0moOg01dM7@2d];@0oooo07QhN01P
H6080?ooo`070?_kn`1DE5@0];Bd0?ooo`36a/H0@D510?Ogm`2/0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYA03oool00`1hN7P02@T90=;BdP0:0?ooo`050=oOg`0B4Q80000000l?
3`3_knl02@3oool01@3Ogml04Q8B000000092@T0d];B0:d0oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooo`h0oooo1@1EEED60?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Wi
n@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI
0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`
l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0
inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo6@3oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0EEEE
0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Wi
n@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool>
0?ooo`D0EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooa80oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTm0?ooo`050?_kn`1DE5@000000000001PH6000P3o
ool03P1hN7P00000061PH02RXZ8000000451@@2JVYX000000451@@33`l<0
?clo0000000?3`l0kno_0`3oool0202RXZ8000000451@@33`l<0?clo0000
000?3`l0kno_0`3oool01P2RXZ800000000000000000@D510?Ogm`<0oooo
0140N7Qh0000002d];@0oooo07QhN00000003`l?0>_[j`1DE5@0H61P0=oO
g`0B4Q80@D510?Ogm`1hN7P000000;Bd]0030?ooo`0C0?_kn`1DE5@00000
00T92@3Bd]80oooo07QhN00000003`l?0>o_k`2RXZ8000000451@@2JVYX0
00000451@@3GemL06a/K0=;BdP210?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?`3oool00`2RXZ80Shn?
0?ooo`020?ooo`0>0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1MGEd0
d];B0<_;b`3Bd]80oooo0<K6aP2?Shl30?ooo`080?_kn`1MGEd0d];B0<_;
b`3Bd]80oooo0<K6aP2?Shl30?ooo`0507QhN02d];@0no_k05eMG@3Bd]80
0`3oool04@3kno/0GEeM0=;BdP3oool0a/K6061PH03oool0moOg07emO@3O
gml0LW9b0?_kn`1MGEd0d];B0?_kn`1MGEd0d];B00@0oooo0100XZ:R061P
H03oool0moOg0:BTY036a/H0H61P0?ooo`3gmoL0OGem0?_kn`1MGEd0d];B
0?_kn`1MGEd0d];BQ03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJCl0oooo00<0XZ:R08n?S`3oool00P3o
ool02@3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP020?oo
o`030::RXP000000];Bd00<0oooo00<0no_k05eMG@3Bd]800P3oool00`2R
XZ8000000;Bd]0030?ooo`050<K6aP0?3`l0jn_[05eMG@3Bd]800`3oool0
1P3kno/0GEeM0=;BdP3oool0XZ:R0;Bd]0@0oooo00L0bl_;03lo?`2?Shl0
oooo0?_kn`1MGEd0d];B00@0oooo00<0N7Qh0;Bd]03oool00P3oool00`2R
XZ80];Bd0?ooo`020?ooo`060?_kn`1MGEd0d];B0?_kn`1MGEd0d];BQ03o
ool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJCl0oooo00<0XZ:R08n?S`3oool00P3oool03@3gmoL0;2`/0>o_
k`3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP3Ogml04Q8B0451@@3gmoL0103o
ool01`3kno/0GEeM0=;BdP3Ogml04Q8B0451@@3gmoL01@3oool0103gmoL0
7AdM00T92@3Bd]830?ooo`050?_kn`1MGEd0d];B0?ooo`2RXZ800`000000
2@1PH600oooo0<K6aP0?3`l0kno_0?ooo`3kno/0GEeM0=;BdP040?ooo`03
0861P@3Bd]80oooo0080oooo00/0XZ:R0000000000000000061PH03kno/0
GEeM0=;BdP3kno/0GEeM0=;BdP240?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?`3oool00`2RXZ80Shn?
0?ooo`020?ooo`0>0?Ogm`0M7Ad0H61P0?Ogm`0V9RH0d];B07QhN0092@T0
d];B0<K6aP2?Shl0oooo0861P@3Bd]830?ooo`0807QhN0092@T0d];B0<K6
aP2?Shl0oooo0861P@3Bd]830?ooo`050::RXP2?Shl0no_k05eMG@3Bd]80
0`3oool04@3Kfm/0GEeM0=;BdP3oool0a/K608n?S`3oool0N7Qh08n?S`3g
moL0;2`/0>OWi`2TY:@0oooo0=_Kf`1MGEd0d];B00@0oooo0100XZ:R08n?
S`3oool0N7Qh0;Bd]036a/H0Shn?0?ooo`1hN7P0Shn?0?_kn`1MGEd0d];B
0?_kn`1MGEd0d];BQ03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJCl0oooo00<0XZ:R08n?S`3oool00P3o
ool01@3gmoL0;2`/0;Jf]P000000@D510080moOg00L09RHV0=;BdP3kno/0
E5AD0000002FUYH0fm_K00<0oooo00P0moOg02HV9P3Bd]80no_k05ADE000
0000UYJF0=_Kf`<0oooo00D0gmoO018B4P000000@D510?Ogm`030?ooo`0?
0<K6aP00000000000;Bd]03oool0N7Qh00000011@D40lo?c05ADE0000000
TY:B018B4P2?Shl0a/K60080000000<0];Bd0?ooo`3oool00P3oool03`1h
N7P0000000l?3`3_knl0oooo07QhN0000000@D510?Ogm`3kno/0GEeM0=;B
dP3kno/0GEeM0=;BdP240?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY?`3oool00`2RXZ80Shn?0?ooo`02
0?ooo`030?Ogm`0/;2`0kno_0280oooo00<0PH610=;BdP3oool02`3oool0
0`21PH40d];B0?ooo`0=0?ooo`060?_kn`1MGEd0d];B0?_kn`1MGEd0d];B
Q03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJC`0oooo00/0Z:RX0=;BdP3oool0XZ:R08n?S`3oool0a/K6
08^;R`1DE5@03`l?0>o_k`0R0?ooo`030:RXZ03Bd]80oooo00/0oooo00<0
Z:RX0=;BdP3oool03@3oool01P1hN7P02@T90=;BdP1hN7P02@T90=;BdX@0
oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030::RXP000000000000@0000000@0Shn?0=oOg`0Q
8B40kno_0`3oool00`3gmoL09RHV0=;BdP080?ooo`030?Ogm`0V9RH0d];B
02l0oooo00H0moOg02HV9P3Bd]80moOg02HV9P3Bd]:40?ooo`0305EEE@3o
ool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3o
ool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool?0?ooo`@0EEEE1P3o
ool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00
lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0
EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooo`h0oooo1@1EEED60?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolI0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_
0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3k
no/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo6@3o
ool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00
lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OW
i`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3o
ool0oooo0080oooo00<0k>c/0?3`l02>SXh01@2PX:000`2OWil0X:2P0:2P
X0040:2PX00F092@T03/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0=kN
gP0<0=?Cd`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI
0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo6@3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?oooaT0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gM
g@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolI0?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OG
e`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo6@3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooaT0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolI0?oo
o`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo3P3oool505EEE@H0
oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
703oool00`1hN7P000000000000500000003061PH03oool0oooo00@0oooo
00<0no_k05ADE00000000P0000005@0?3`l0kno_0?ooo`3oool0moOg01dM
7@000000@D510?Ogm`3oool0moOg01dM7@000000@D510?Ogm`3oool0moOg
01dM7@000000@D510?Ogm`030?ooo`030<K6aP00000000000080000000<0
];Bd0?ooo`3oool0203oool0101hN7P0000000l?3`3_knl20?ooo`050?Og
m`0M7Ad000000451@@3gmoL00`3oool00`36a/H000000000000200000003
0;Bd]03oool0oooo00<0oooo00T0a/K600000000000000000451@@3gmoL0
oooo0?_kn`1DE5@00`0000000`0?3`l0kno_0?ooo`020?ooo`0507QhN000
00003`l?0>_[j`1DE5@00P0000000`0?3`l0kno_0?ooo`040?ooo`030<K6
aP00000000000080000000<0];Bd0?ooo`3oool0103oool00`36a/H00000
00000002000000030;Bd]03oool0oooo0080oooo00@0N7Qh000000092@T0
d];B0P3oool01036a/H000000000002?ShmQ0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703oool00`1hN7P0
0000000000060000000300T92@3Bd]80oooo0080oooo00<0gmoO018B4P00
00001@0000005P2d];@0oooo0?Ogm`0M7Ad000000451@@3gmoL0oooo0?Og
m`0M7Ad000000451@@3gmoL0oooo0?Ogm`0M7Ad000000451@@3gmoL0oooo
0?_kn`1DE5@6000000030451@@3gmoL0oooo00L0oooo00@0N7Qh0000000?
3`l0kno_0P3oool0203gmoL07AdM00000011@D40moOg0?ooo`3kno/0E5AD
1P0000001P11@D40moOg0?ooo`3oool0moOg01dM7@<0000000@0H61P0?oo
o`3Ogml04Q8B1P0000001`2d];@0oooo0?ooo`1hN7P0000000T92@0o?cl0
100000001P092@T0d];B0?ooo`3oool0no_k05ADE0H0000000H0@D510?Og
m`3oool0oooo0?_kn`1DE5@60000000>0451@@3gmoL0oooo0?ooo`1hN7P0
000000T92@3Bd]80oooo0?_kn`1DE5@0000000l?3`3_knmQ0?ooo`0305EE
E@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703o
ool00`1hN7P00000000000070000000500l?3`3_knl0oooo0?_kn`1DE5@0
0P0000006P2d];@0oooo0?Ogm`0M7Ad0000000l?3`3_knl0moOg01dM7@00
0000@D510?Ogm`3oool0moOg01dM7@000000@D510?Ogm`3oool0moOg01dM
7@000000@D510?Ogm`3oool0XZ:R0P000000101PH600oooo0?ooo`2RXZ82
0000000308n?S`3oool0oooo00H0oooo00@0N7Qh0000000?3`l0kno_0P3o
ool01`3gmoL07AdM00000011@D40moOg0?ooo`2RXZ800P000000101PH600
oooo0?ooo`2RXZ820000000708n?S`3oool0oooo0=oOg`0B4Q800000061P
H0020?ooo`0=0?_kn`1DE5@000000000002d];@0oooo0?Ogm`0M7Ad00000
00l?3`3_knl0oooo07QhN0020000000400l?3`3_knl0oooo0<K6aP800000
00@0@D510?Ogm`3oool0XZ:R0P000000101PH600oooo0?ooo`2RXZ820000
000408n?S`3oool0oooo0::RXP80000000@0H61P0?ooo`3oool0XZ:R0P00
00002@2?Shl0oooo0?ooo`1hN7P0000000T92@3Bd]80oooo0<K6aP020000
000308n?S`3oool0oooo0600oooo00<0EEEE0?ooo`3oool0103oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`0407QhN00000002@T90=;B
dP<0oooo00X0moOg01dM7@000000000008n?S`3oool0gmoO018B4P000000
H61P1`3oool0403gmoL07AdM00000011@D40moOg0?ooo`3gmoL07AdM0000
0011@D40moOg0?ooo`3gmoL07AdM00000011@D420?Ogm`0401dM7@000000
@D510?Ogm`80oooo00D0no_k05ADE00000003`l?0>o_k`070?ooo`0407Qh
N00000003`l?0>o_k`80oooo00@0moOg01dM7@000000@D510P3gmoL0100M
7Ad000000451@@3gmoL20?ooo`0:0?_kn`1DE5@0000000l?3`3_knl0oooo
0=oOg`0B4Q800000061PH080oooo00@0gmoO018B4P000000H61P203oool0
101hN7P000000000002d];@30?ooo`0907QhN00000002@T90=;BdP3gmoL0
7AdM00000011@D40moOg0080oooo00X0no_k05ADE00000003`l?0>o_k`3g
moL07AdM00000011@D40moOg0P3oool03`3kno/0E5AD0000000?3`l0kno_
0?ooo`1hN7P0000000T92@3Bd]80no_k05ADE00000003`l?0>o_k`1R0?oo
o`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUY703oool0101hN7P0000000T92@3Bd]840?ooo`060=oOg`0B4Q800000
0451@@3gmoL0a/K60P0000000`2?Shl0oooo0?ooo`050?ooo`0E0?Ogm`0M
7Ad000000451@@3gmoL0oooo0?Ogm`0M7Ad000000451@@3gmoL0oooo0?Og
m`0M7Ad000000451@@3gmoL0gmoO018B4P000000H61P00@0oooo00@0XZ:R
000000000000];Bd1`3oool0101hN7P0000000l?3`3_knl20?ooo`090?Og
m`0M7Ad000000451@@3gmoL0gmoO018B4P000000H61P00@0oooo00T0XZ:R
000000000000];Bd0?ooo`3Ogml04Q8B0000001PH6000P3oool01036a/H0
00000000002?Shl80?ooo`0407QhN00000002@T90=;BdP<0oooo00P0XZ:R
000000000000];Bd0=oOg`0B4Q800000061PH0@0oooo00P0XZ:R00000000
0000];Bd0=oOg`0B4Q800000061PH0@0oooo00H0XZ:R000000000000];Bd
0?ooo`1hN7P20000000500l?3`2BTY8000000000002d];@0H`3oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0
oooo00@0N7Qh000000092@T0d];B103oool01P3kno/0E5AD0000000?3`l0
kno_0<K6aPT0000001<0];Bd0?Ogm`0M7Ad000000451@@3gmoL0oooo0?Og
m`0M7Ad000000451@@3gmoL0oooo0?Ogm`0M7Ad000000451@@3gmoL0a/K6
0080000000<0Shn?0?ooo`3oool00P3oool0102RXZ8000000000002d];@7
0?ooo`0407QhN00000003`l?0>o_k`80oooo00H0moOg01dM7@000000@D51
0?Ogm`36a/H20000000308n?S`3oool0oooo0080oooo00T0XZ:R00000000
0000];Bd0?ooo`3Ogml04Q8B0000001PH6000P3oool00`36a/H000000000
0007000000060;Bd]03oool0N7Qh0000000?3`l0kno_0`3oool01@2RXZ80
00000000002d];@0a/K60080000000<0Shn?0?ooo`3oool00P3oool01@2R
XZ8000000000002d];@0a/K60080000000<0Shn?0?ooo`3oool00P3oool0
1P2RXZ8000000000002d];@0oooo07QhN0D0000000<0@D510?Ogm`3oool0
HP3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJA`0oooo00@0N7Qh000000092@T0d];B1@3oool01@1hN7P00000
00T92@3Bd]80a/K600T0000001H0];Bd0?Ogm`0M7Ad000000451@@3gmoL0
oooo0?Ogm`0M7Ad000000451@@3gmoL0oooo0?Ogm`0M7Ad000000451@@3g
moL0gmoO018B4P000000H61P103oool0102RXZ8000000000002d];@70?oo
o`0407QhN00000002@T90=;BdP80oooo00T0moOg01dM7@000000@D510?Og
m`3Ogml04Q8B0000001PH600103oool02@2RXZ8000000000002d];@0oooo
0=oOg`0B4Q800000061PH0020?ooo`030<K6aP000000000000L0000000H0
];Bd0?ooo`1hN7P0000000l?3`3_knl30?ooo`080::RXP00000000000;Bd
]03Ogml04Q8B0000001PH6040?ooo`080::RXP00000000000;Bd]03Ogml0
4Q8B0000001PH6040?ooo`090::RXP00000000000;Bd]03oool0N7Qh0000
00092@T0?clo0080000000<0];Bd0?ooo`3oool0HP3oool00`1EEED0oooo
0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo00@0
N7Qh000000092@T0d];B1@3oool0201hN7P0000000T92@3Bd]80gmoO018B
4P000000Shn?0`3oool0502RXZ80000000T92@3Bd]80moOg01dM7@000000
3`l?0>o_k`3oool0moOg01dM7@0000003`l?0>o_k`3oool0moOg01dM7@00
0000@D510P3gmoL0100M7Ad000000451@@3gmoL20?ooo`050?_kn`1DE5@0
000000l?3`3_knl01`3oool0101hN7P000000000002d];@20?ooo`040?Og
m`0M7Ad000000451@@80moOg00@07AdM00000011@D40moOg0P3oool02P3k
no/0E5AD0000000?3`l0kno_0?ooo`3Ogml04Q8B0000001PH6020?ooo`04
0=oOg`0B4Q80000008n?S`<0oooo00H0XZ:R000000092@T0d];B0?ooo`1h
N7P20000000=0;Bd]03oool0oooo0?ooo`1hN7P0000000T92@3Bd]80moOg
01dM7@000000@D510?Ogm`020?ooo`0:0?_kn`1DE5@0000000l?3`3_knl0
moOg01dM7@000000@D510?Ogm`80oooo00d0no_k05ADE00000003`l?0>o_
k`3oool0N7Qh000000092@T0b/[:01dM7@000000Shn?06@0oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?oo
o`0407QhN00000002@T90=;BdPD0oooo0140N7Qh000000092@T0d];B0?oo
o`1hN7P0000000l?3`3_knl0oooo0?Ogm`0M7Ad00000061PH03oool0moOg
01dM7@020000000308n?S`3oool0a/K600<0000000<0Shn?0?ooo`36a/H0
0P000000101PH600oooo0?ooo`2RXZ8200000004061PH03oool0oooo0::R
XP80000000<0H61P0?ooo`3oool01P3oool01`1hN7P000000000000?3`l0
kno_0?ooo`2RXZ800P000000101PH600oooo0?ooo`2RXZ8200000004061P
H03oool0oooo0::RXP80000000L0H61P0?ooo`3oool0gmoO018B4P000000
H61P00<0oooo00T0N7Qh0000000?3`l0kno_0?ooo`3gmoL07AdM0000001P
H6000P3oool01`1hN7P000000000000?3`l0kno_0?ooo`36a/H00P000000
1011@D40moOg0?ooo`2RXZ8200000004061PH03oool0oooo0::RXP800000
00@0H61P0?ooo`3oool0XZ:R0P000000101PH600oooo0?ooo`2RXZ820000
0008061PH03oool0oooo07QhN00000002@T90=;BdP36a/H20000000308n?
S`3oool0oooo0640oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTL0?ooo`0407QhN00000002@T90=;BdP@0oooo
00P0no_k05ADE00000003`l?0>o_k`3oool0moOg01dM7@D0000000D03`l?
0>o_k`3oool0moOg01dM7@0=000000050;Bd]03oool0oooo0?_kn`1DE5@0
1P0000000`11@D40moOg0?ooo`070?ooo`0407QhN00000002@T903lo?`D0
000000D0];Bd0?ooo`3oool0no_k05ADE006000000040451@@3gmoL0oooo
07QhN0D0000000@0H61P0?ooo`3gmoL07AdM1@0000001@0?3`l0kno_0?oo
o`3oool0N7Qh00L0000000H02@T90=;BdP3oool0oooo0?_kn`1DE5@60000
00060451@@3gmoL0oooo0?ooo`3kno/0E5AD1P0000002P11@D40moOg0?oo
o`3oool0N7Qh000000092@T0d];B0?ooo`1hN7P2000000030;Bd]03oool0
oooo0600oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3@d=00clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJA`0oooo
00@0N7Qh000000092@T0d];B103oool01@3Ogml04Q8B00000011@D40moOg
0080oooo00<0no_k05ADE00000000P0000002@11@D40moOg0?ooo`3oool0
moOg01dM7@000000Shn?0<K6aP030000000308n?S`3oool0a/K600<00000
00<0];Bd0?ooo`3oool00`3oool00`36a/H00000000000020000000308n?
S`3oool0oooo00P0oooo00D0N7Qh0000000?3`l0kno_07QhN00300000003
0;Bd]03oool0oooo00<0oooo00<0a/K60000000000000P0000001@2?Shl0
oooo0?ooo`3oool0N7Qh00D0000000D0H61P0?ooo`3oool0no_k05ADE003
000000030451@@3gmoL0oooo0080oooo00D0N7Qh0000000?3`l0jn_[05AD
E0020000000300l?3`3_knl0oooo00@0oooo00<0a/K60000000000000P00
00000`2?Shl0oooo0?ooo`040?ooo`030<K6aP00000000000080000000<0
Shn?0?ooo`3oool00P3oool02P1hN7P0000000T92@3Bd]80oooo0?Ogm`0M
7Ad0000000T92@3Bd]9Q0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTL0?ooo`0407QhN00000002@T90=;BdP<0oooo00D0gmoO018B
4P000000000008n?S`180?ooo`040=oOg`0B4Q800000061PH0h0oooo00@0
N7Qh0000000?3`l0kno_803oool0101hN7P0000000T92@3Bd]9W0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0d=3@0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTL0?ooo`0307QhN0000000
000000L0000000<03`l?0>o_k`3oool0A`3oool0103Ogml04Q8B0000001P
H60>0?ooo`0407QhN00000003`l?0>o_kb00oooo00@0N7Qh000000092@T0
d];BI`3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00D0k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY703o
ool00`1hN7P00000000000060000000300T92@3Bd]80oooo04T0oooo00<0
XZ:R0000001PH6003P3oool0101hN7P0000000l?3`3_knlP0?ooo`0407Qh
N00000002@T90=;BdVL0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJA`0oooo00<0N7Qh0000000000001@0000000`11@D40moOg0?ooo`1;
0?ooo`030=oOg`1bLW80oooo00d0oooo00@0N7Qh0000000?3`l0kno_803o
ool0101hN7P0000000T92@3Bd]9W0?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0
oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2E
UID06@3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo4P3oool00`1EEED0oooo0?ooo`040?ooo`0305EE
E@3oool0oooo0080oooo00D0k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE
0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0
i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo
00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolB0?ooo`03
05EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03L
g=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolB0?ooo`0305EEE@3oool0oooo00@0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3oool?0?ooo`@0EEEE1P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooa80oooo00<0EEEE0?oo
o`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTK0?ooond0
00001@3oool505EEE@80oooo1@1EEED40?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
oah0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0
gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?oooah0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?S
h`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooah0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[Z
jP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolN0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM
0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?oooah0oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolN0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3T
i>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3GemL0
303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolN0?ooo`030>c/k03moOd0Ti>C00D0^;Rh00<0]KFe0;Rh^02h^;P0
102h^;P05P2CTi<0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolN0?ooo`030>c/k03^k^h0SHf=00`0WinO01H0SHf=0>KViP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF
0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/
k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo
00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02FUYH0oooo0080c/k>
00<0?Shn0<k>cP3>c/h00`3>c/h06P0n?Sh0c/k>0<k>cP3>c/h0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool0103/
k>`0UYJF0?ooo`3?cll303hn?P<0clo?0`0n?Sh20<o?c`0F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/
09JFUP3oool00P3De=@303hn?P040=CDe00n?Sh0?Shn03hn?P<0e=CD01H0
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo7P3o
ool00`3/k>`0UYJF0?ooo`030=_Kf`D0?Shn103Kfm/05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolN0?ooo`030>c/k02F
UYH0oooo00@0i>CT0`0n?Sh50>CTi00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool01@3]
kNd00`0n?Sh0kNg]0>g]k@040>g]k@0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooah0oooo00<0k>c/09JFUP3oool0303d
m?@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolN0?ooo`030>c/k02FUYH0oooo00`0o?cl01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo7P3oool00`3/k>`0UYJF0?oo
o`0=0?ooo`0E09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooah0oooo00<0k>c/09JFUP3GemL0303oool05P3GemL0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`00000N000000040>c/k03Jf]X0
TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3Jf]X0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000<00000
0?Ogm`2JVYX0o`1jNWXa07YjNP0C09ZJVP3gmoL0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`2o_kl0
o`2o_klb0;no_`0B0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000030000003gmoL0hn?S0?l0hn?S<P3Shn<04P3g
moL0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
00000`000000lO7a0?ooo`3Z0?ooo`030>?Sh`2UYJD0MWIf00<0MWIf00@0
MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0hn?S0:FU
Y@@0MWIf00@0MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3o
ool0jn_[0:FUY@@0EUIF00@0EeMG05IFEP1GEeL0EeMG2@1FEUH00`2UYJD0
jn_[0?ooo`030?ooo`0B0?7al@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003QhN40o?cl0?_kn`80o?cl00<0
no_k0?clo03lo?`0103lo?`00`3kno/0o?cl0?clo0020?clo0030?_kn`3l
o?`0no_k00D0o?cl00<0no_k0?clo03kno/0D@3kno]o0?clo0030:FUY@1f
MWH0SHf=0100SHf=00H0MWIf0:FUY@3lo?`0o?cl0:FUY@1fMWHA08f=S@06
07IfMP2UYJD0o?cl0?_kn`2UYJD0EUIF4@1KFe/00`1FEUH0YJFU0?clo003
0?clo00B0>7Qh@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000030000003Ph>00moOg0>X0moOg00<0MWIf08f=S@2RXZ80
402RXZ801P2=SHd0MWIf0?Ogm`3gmoL0MWIf08f=SA40XZ:R00P0SHf=07If
MP3gmoL0moOg05IFEP1KFe/0Kfm_075aL@d0LW9b00@0LG5a06m_K`1KFe/0
EUIF103gmoL04P3Ph>00000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo00000`000000g]kN0?7al@020?7al@80l_;b00<0lO7a
0?;blP3bl_80103alO400`3bl_80lO7a0?7al@030?7al@80l_;b00<0lO7a
0?;blP3alO400P3bl_;B0?7al@0307IfMP2h^;P0Zj^[0100Zj^[00H0^;Rh
07IfMP3alO40lO7a07IfMP2h^;PA0:^[Z`050;Rh^01fMWH0lO7a0?;blP1F
EUH00P1_Kfl00`1bLW80MgMg07MgM`0;07MgM`04079bLP1_Kfl0Kfm_05IF
EP<0lO7a01<0l_;b0=kNgP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo00001@000000g=cL0>_[j`3[jn/0k>c/0080jn_[
00<0YjNW0::RXP2RXZ800P2QXJ400`2PX:00WinO09fMW@0209nOW`80X:2P
0P2QXJ400`2WYjL0jn_[0>_[j`050>_[j`050=CDe035aLD0aLG50<G5a@3G
emL00P3[jn/00`3Zj^X0b<S80=3@d0020>_[j`050=[JfP36a/H0hn?S0>_[
j`3Ph>000P35aLD01`3<c<`0jNWY0<k>cP37alL0jNWY0>_[j`3Ph>000P35
aLD01`3<c<`0j^[Z0=CDe035aLD0gMgM0>_[j`3Ph>000P35aLD01P36a/H0
hn?S0>_[j`3[jn/0j^[Z0<S8b0<0aLG500<0a/K60>?Sh`3[jn/01`3[jn/0
0`3UiND0alO70=OGe`0F0>_[je<0k>c/;03[jn/0101fMWH0^k^k0;6a/@2a
/K480?ooo`L0/K6a00H0^k^k07IfMP3[jn/0jn_[07IfMP2k^k/20;6a/@`0
oooo0`2a/K402@2k^k/0MWIf0>_[j`3[jn/0EUIF07MgM`1bLW80M7Ad07Mg
M`0;07QhN00507MgM`1dM7@0LW9b07MgM`1FEUH0103[jn/04P3Lg=`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`00
0000f][J0>GUi@030>GUi@0308Z:RP2KVi/0[:b/00<0jNWY00X0j^[Z0<C4
a025QHD0k>c/0>g]k@3^k^h0inOW0?3`l03bl_80Xj>S1P3UiND04@1XJ6P0
410@010@400@4100P8200<K6aP3?cll0g]kN02HV9P1ADE40clo?0>GUi@2D
U9@05ADE0:BTY03Ng]h0/[:b0080410@00L0@D510=_Kf`1@D5006ATI0;Zj
^P3Shn<0/[:b0080410@00L0@D510=oOg`1`L700410@09:BTP3Ti>@0[jn_
0080410@00H05ADE0;Zj^P3<c<`0i>CT0=gMg@0T92@3010@400301DE5@2T
Y:@0g]kN00L0iNGU00@0bLW901`L701bLW80e=CDU@3UiND01@1fMWH0`<30
0;Fe]@2e]KD0oooo00H0LW9b00<09bLW0;Fe]@2e]KD01@2e]KD01P30`<00
MWIf0>GUi@3UiND0MWIf0<30`080]KFe00<0oooo02LW9`0W9bL02@0W9bL3
0;Fe]@050<30`01fMWH0iNGU0>GUi@1FEUH00P1gMgL0101iNGT0NWYj0?oo
o`3oool707YjNP80oooo00D0NWYj07UiN@1gMgL0MgMg05IFEP040>GUi@0B
0=[JfP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000030000003IfMT0gmoO00<0gmoO00D0g]kN0:2PX03Zj^X0jNWY0>GU
i@020>[ZjP0:0=3@d01dM7@0kNg]0>k^kP2][Jd0d=3@0?;blP3clo<0Xj>S
0=oOg`80g]kN03@0gmoO0=kNgP1/K6`05ADE0:^[Z`3Ng]h0`l?301`L701D
E5@0a/K60=KFeP0U9BD0DE5A0<g=c@3Ng]h0Shn?01DE5@2TY:@0eMGE04i>
CP0h>3P0a<C40=gMg@3Kfm/0CTi>01TI6@2j^[X0fMWI04i>CP0h>3P0a<C4
0=gMg@3GemL09BDU03Ph>034a<@0`/;201/K6`1ADE40fMWI0<?3``1<C4`0
f=SH0=OGe`0U9BD06ATI0<G5a@3Ng]h0Shn?01DE5@2TY:@0fMWI1`3Ng]h0
1033`l<071`L079bLP3AdM6E0=kNgP0507IfMP33`l<0^k^k0;^k^`3oool0
1P1bLW800`0W9bL0^k^k0;^k^`050;^k^`060<?3``1fMWH0g]kN0=kNgP1f
MWH0`l?30P2k^k/00`3oool09bLW0;^k^`070;^k^`030?ooo`0W9bL0^k^k
0080^k^k00T0`l?307IfMP3Ogml0g]kN05IFEP1gMgL0N7Qh07]kN`0`<300
0`3oool507emO@<0oooo00D0<30`07]kN`1hN7P0MgMg05IFEP020=kNgP0D
0=oOg`3Ng]h0fMWI0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000=OGe`3Hf=P0fMWI0P3Hf=P0402@T900Shn?
08j>SP3YjNT0PH61055AD@27QhL0f][J0:JVYP2YZJT0[:b/05mOG`3bl_80
lo?c0?Cdm02TY:@30=SHf0050=WIf@3Bd]8092@T055AD@3;bl/00P3Hf=P0
801ZJVX06ATI0;Zj^P3@d=0092@T055AD@3;bl/0f=SH08^;R`0E5AD0Y:BT
0<g=c@0T92@0>3Ph0<?3``3GemL0eMGE04a<C00I6AT0^[Zj0=3@d00T92@0
>3Ph0<?3``3GemL0dM7A02@T900h>3P0`l?308Z:RP0E5AD0Y:BT1035aLD0
2P2NWYh0410@075aL@3AdM40f=SH08^;R`0E5AD0Y:BT0=CDe03Bd]820<G5
a@080<K6aP3GemL0f=SH0=SHf02m_Kd06a/K079bLP3>c/hF0=SHf5<0fMWI
;03Hf=P01@1fMWH0alO70;no_`2o_kl0oooo00L09bLW1`2o_kl01P37alL0
MWIf0=SHf03Hf=P0MWIf0<O7a`80_kno00<0oooo02LW9`2o_kl01`2o_kl0
0`3oool09bLW0;no_`020;no_`050<O7a`1fMWH0f=SH0=SHf01FEUH00P1k
Ng/00`1nOWh0Ogmo02PX:0030?ooo`<0Ogmo0`3oool00`0X:2P0Ogmo07in
OP0207]kN`0305IFEP3IfMT0f=SH0080f=SH0180emOG0000002JVYX0X:2P
0:RXZ02b/[80_;bl0<K6aP3@d=00fMWI0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=GEe@3C
dm<00`3Cdm<00`25QHD0Xj>S0>KViP020>[ZjP0;03De=@1oOgl0clo?0;jn
_P2;Rh/0GUiN05QHF01XJ6P0Xj>S0<W9b@2ZZZX0103Cdm<0103<c<`092@T
055AD@3:b/X20=?Cd`0N06MWI`0I6AT0^KVi0<_;b`0T92@0DE5A0<[:bP3C
dm<0R8R801DE5@2TY:@0bLW902@T900h>3P0`L710=?Cd`3?cll0BTY:01TI
6@2i^KT0bl_;02@T900h>3P0`L710=?Cd`3<c<`092@T03Ph>031`L40IfMW
1@0@41002P1ADE40WIfM010@402>SXh0c<c<0=?Cd`28R8P05ADE0:BTY02Q
XJ42010@400301XJ6P36a/H0dm?C0080dm?C00@0^;Rh01/K6`1bLW80c<c<
U@3Cdm<00`1fMWH0bl_;0<K6aP0@0<K6aP060<_;b`1fMWH0dm?C0=?Cd`1f
MWH0bl_;0P36a/H00`3oool09bLW0<K6aP070<K6aP030?ooo`0W9bL0a/K6
0080a/K600L0bl_;07IfMP3Cdm<0dm?C05IFEP1mOGd0Ogmo00<0Ph>300D0
:2PX0?ooo`3oool0oooo08>3P`030?ooo`0702PX:023Ph<0Ph>308>3P`1o
Ogl0OGem05IFEP040=?Cd`0A0=GEe@000000X:2P0:FUY@2][Jd0][Jf0;no
_`38b<P0d];B0=[JfP3Rh^80jNWY0>o_k`3dm?@0m_Kf0?Win@3kno/00`3m
oOd10?knoP40oooo0@3oool000050000003Cdm<0c/k>0<k>cP3?cll00P3>
c/h03`27QhL0Ti>C0=?Cd`3Kfm/0[Jf]0<71`@2`/;00ZJVY0:ZZZP2=SHd0
TI6A0;2`/03;bl/0kno_0:FUY@030<k>cP050<o?c`37alL08b<S055AD@38
b<P00P3>c/h0;@1UIFD06ATI0;Vi^@37alL08b<S055AD@37alL0c/k>08F5
Q@0E5AD0Y:BT0<O7a`0S8b<0>3Ph0<71`@3>c/h0bl_;04U9B@0I6AT0^KVi
0<O7a`0S8b<0>3Ph0<71`@3>c/h0alO702<S8`0h>3P0`L7108F5Q@0E5AD0
Y:BT0<[:bP37alL08b<S055AD@2a/K406QXJ079bLP35aLD0c/k>08F5Q@0E
5AD0Y:BT0<g=c@070<k>cP040;Bd]00J6QX0LW9b0<[:bQH0c/k>D`3?cll/
0<k>cP0307IfMP3>c/h0bl_;0100bl_;00H0c/k>07IfMP3>c/h0c/k>07If
MP3>c/h20<_;b`030?ooo`0W9bL0bl_;00L0bl_;00<0oooo02LW9`3;bl/0
0P3;bl/02@3>c/h0MWIf0<o?c`3>c/h0EUIF07emO@22PX80QHF508B4Q002
08F5Q@0302PX:03oool0oooo00<0oooo00P0:2PX08F5Q@25QHD0Q8B408F5
Q@22PX80OGem05IFEP@0c/k>0140dm?C0000002XZ:P0[Jf]0;>c/`2k^k/0
`l?30<c<c03EeMD0gMgM0>CTi03[jn/0l?3`0?Cdm03gmoL0nOWi0?clo003
0?gmo@40o_kn0@3oool10?ooo`0000<000000=3@d039bLT00`39bLT04@1c
Lg<0U9BD07inOP1iNGT0X:2P0=7Ad@2][Jd0YJFU0:JVYP25QHD0GUiN0:f]
[@3UiND0m_Kf0?Shn02TY:@0bLW900<0b<S800@0`/;202<S8`1ADE40alO7
0P38b<P0;01RHV806ATI0;Vi^@32`/808R8R03Ph>031`L40`/;204I6AP0I
6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35aLD0a<C404I6AP0I6AT0^KVi0<71
`@0S8b<0>3Ph0;no_`35aLD0`<30028R8P0h>3P0`<300:n_[`0J6QX0Lg=c
0<G5a@2JVYX0410@08j>SP35aLD0ATI601TI6@2i^KT0aLG507moO`0E5AD0
Y:BT1`38b<P01@37alL0[Zj^01XJ6P1aLG40alO701H0b<S8D`39bLT/0<S8
b00307IfMP3Cdm<0cLg=0100cLg=00H0dm?C07IfMP38b<P0b<S807IfMP3C
dm<20<g=c@030?ooo`0W9bL0cLg=00L0cLg=00<0oooo02LW9`3=cLd00P3=
cLd01`3Cdm<0MWIf0<S8b038b<P0EUIF08:2PP23Ph<01@27QhL01@0X:2P0
oooo0?ooo`3oool0:2PX00D0QhN700<0Ph>308:2PP1FEUH00`39bLT04P38
b<P0d=3@0000002b/[80][Jf0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OW
i`3]kNd0lO7a0?Gem@3hn?P0n_[j0?clo0<0oOgm0@3no_h10?ooo`40oooo
00000`000000clo?0<G5a@040<G5a@0?09NGU`2g]kL0k>c/0;Vi^@2;Rh/0
]kNg0:ZZZP2TY:@0OWin075aL@34a<@0fMWI0?KfmP3inOT0YJFU00@0aLG5
00@0_kno028R8P1ADE40a/K60P35aLD01P1QHF406ATI0;Vi^@2o_kl08R8R
05eMG@80410@00@05ADE0:BTY032`/80ADE50`0@41001P1ADE40HF5Q010@
400I6AT0][Jf04E5A@<0410@00P0DE5A0;no_`0R8R80>3Ph0;no_`35aLD0
[:b/01XJ6P80410@00D0LW9b0<G5a@35aLD0`/;204E5A@03010@400301DE
5@2TY:@0a/K600D0aLG500H0[:b/05aLG02][Jd06QXJ075aL@36a/JE0<G5
a@0307IfMP3EeMD0dM7A0100dM7A00H0eMGE07IfMP35aLD0aLG507IfMP3E
eMD20=7Ad@030?ooo`0W9bL0dM7A00L0dM7A00<0oooo02LW9`3AdM400P3A
dM401`3EeMD0MWIf0<G5a@35aLD0EUIF08:2PP27QhL01@29RHT50?ooo`D0
RHV900<0QhN708:2PP1FEUH01035aLD04@3?cll000000;bl_02o_kl0`l?3
0<W9b@3@d=00emOG0=gMg@3Ti>@0j^[Z0>o_k`3clo<0m_Kf0?Shn03kno/0
o?cl0080oOgm0P3no_h20?ooo`0000<000000<g=c@31`L400`31`L40401e
MGD0VIVI0:NWY`3]kNd0bLW907=cL`2`/;00SHf=07=cL`1<C4`0HV9R0;jn
_P2e]KD0/[:b0:f][@2YZJT40<71`@040;^k^`0R8R80DE5A0<C4a080`L71
00@0GemO01TI6@2i^KT0`/;21`31`L401@2k^k/08R8R03Ph>02n_[h0`/;2
0080`L7100`0`l?30<G5a@2l_;`08R8R03Ph>02n_[h0`/;20;^k^`0R8R80
>3Ph0;jn_P32`/8;0<71`@0407alO00E5AD0Y:BT0<?3``D0`L7100H0ZJVY
01XJ6P0@4100410@075aL@34a<BE0<71`@0307IfMP3Hf=P0eMGE0100eMGE
00H0f=SH07IfMP31`L40`L7107IfMP3Hf=P20=GEe@030?ooo`0W9bL0oooo
00P0oooo00H09bLW0=GEe@3EeMD0eMGE0=SHf01fMWH20<71`@0305IFEP24
Q8@0R8R800@0RXZ:0`3oool0100X:2P0oooo0?ooo`3oool408Z:RP0308R8
R024Q8@0EUIF00@0`L710100cLg=00000036a/H0b<S80<c<c03AdM40emOG
0=cLg03Rh^80j>SX0>g]k@3alO40mOGe0?Ogm`3inOT0o?cl0`3moOd20?kn
oP80oooo00000`000000c<c<0;Zj^P020;fm_@0A0;jn_P2m_Kd0WYjN0>?S
h`2f][H0UYJF08:2PP2CTi<0Q8B408:2PP1>CTh0;2`/09bLW02[Zj/0m_Kf
0?Shn02WYjL0102m_Kd0102g]kL08B4Q055AD@34a<@20;fm_@0405eMG@0I
6AT0^KVi0;no_`P0_Kfm00<0GEeM03Lg=`2h^;P00P2m_Kd02`1MGEd06ATI
0;Vi^@2o_kl0GEeM03Lg=`2h^;P0_Kfm0;Ng]`0Q8B40>3Ph0080_[jn2`2m
_Kd0101jNWX05ADE0:BTY02o_kl60;fm_@050:FUY@0J6QX0410@071`L032
`/805P2m_KeC0;jn_R`0_Kfm00<0MWIf0=[JfP3EeMD0403EeMD01P3Jf]X0
MWIf0;fm_@2m_Kd0MWIf0=[JfP80eMGE00<0oooo079bLP1bLW80201bLW80
3P0W9bL0eMGE0=GEe@3EeMD0f][J07IfMP2m_Kd0_[jn05IFEP27QhL0RXZ:
08f=S@2>SXh0SHf=0`3oool00`0X:2P0SHf=02PX:0030?ooo`<0SHf=00@0
RXZ:08N7Q`1FEUH0_[jn0P2m_Kd04@2j^[X0c<c<0000003@d=00d];B0=GE
e@3IfMT0gMgM0>;RhP3WinL0k>c/0?3`l03dm?@0m_Kf0?Shn03jn_X0o?cl
00<0oOgm00<0o_kn0?ooo`3oool00@3oool000030000003:b/X0^;Rh00<0
^[Zj0100O7al08f=S@1hN7P0VIVI0?3`l032`/80YjNW0<_;b`2l_;`0Xj>S
06e]K@1mOGd0[:b/0?_kn`3lo?`0YZJV0`2j^[X0102k^k/0];Bd024Q8@1=
CDd30;Zj^P0305]KF`0I6AT0[jn_00h0^[Zj00<0Fe]K01TI6@2_[jl01@2j
^[X0102d];@08B4Q03Lg=`2d];@<0;Zj^P0307MgM`0D51@0Vi^K00T0^[Zj
00<0Fe]K06][J`2j^[X0U@2j^[X00`1fMWH0f=SH0=[JfP0@0=[JfP060=SH
f01fMWH0^[Zj0;Zj^P1fMWH0f=SH0P3Jf]X00`3oool0LW9b079bLP08079b
LP0=02LW9`3Jf]X0f][J0=[JfP3Hf=P0MWIf0;Zj^P2k^k/0EUIF08^;R`2?
Shl0SXj>08n?S`030?ooo`0502PX:02?Shl0Shn?08n?S`0X:2P00`3oool0
1@2?Shl0SXj>08n?S`2;Rh/0EUIF00<0^[Zj0100^;Rh0<[:bP000000fMWI
0=[JfP3MgMd0h>3P0>CTi03Xj>P0k>c/0>o_k`3clo<0m_Kf0?Shn03jn_X0
o?cl0`3moOd20?knoP<0oooo00000`000000bLW90;Rh^0030;Rh^00@08J6
QP2NWYh0e=CD0?3`l03bl_80c/k>0<71`@3MgMd0d];B0::RXP3<c<`0bLW9
08V9R@3lo?`0oOgm0:NWYmL0^;Rh00<0MWIf0=OGe`3MgMd0403MgMd01P3G
emL0MWIf0;Rh^02h^;P0MWIf0=OGe`<0gMgM2`0W9bL30=gMg@090=OGe`1f
MWH0^;Rh0;Rh^01FEUH0T92@096AT@2CTi<0U9BD0080oooo00<0:2PX09RH
V02HV9P00`2HV9P0200X:2P0oooo0?ooo`2DU9@0Ti>C096AT@2@T900EUIF
102h^;P03`39bLT000000>7Qh@3Rh^80i>CT0>OWi`3Zj^X0kNg]0?3`l03c
lo<0mOGe0?Ogm`3inOT0no_k0?clo0030?gmo@030?knoP3oool0oooo0080
oooo00000`000000bLW90;Rh^0040;Rh^00?08J6QP2DU9@0l_;b0??cl`2P
X:00j>SX0?KfmP3Kfm/0VIVI0?[jnP3kno/0g=cL0;Jf]P3no_h0YjNW0=L0
^;Rh00<0MWIf0=KFeP3QhN40403QhN401P3Fe]H0MWIf0;Rh^02h^;P0MWIf
0=KFeQ40hN7Q00T0e]KF07IfMP2h^;P0^;Rh05IFEP2EUID0UiNG09bLW02R
XZ800P0X:2P01@2[Zj/0[Jf]0:^[Z`2[Zj/0[Jf]0080Zj^[0P0X:2P01@2R
XZ80W9bL09NGU`2EUID0EUIF00@0^;Rh00h0bLW90000003Xj>P0jNWY0>_[
j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?clo0<0oOgm0P3n
o_h40?ooo`0000<000000<W9b@2e]KD00`2e]KD03@1gMgL0X:2P0>3Ph03c
lo<0iNGU0>c/k03fm_H0n?Sh0?Win@2@T900no_k0?clo03moOd00P3no_h0
0`2WYjL0]KFe0;Fe]@3E0;Fe]@0307IfMP3]kNd0jNWY0100jNWY00H0kNg]
07IfMP2e]KD0]KFe07IfMP3]kNdA0>WYj@0:0>g]k@1fMWH0]KFe0;Fe]@1F
EUH0Vi^K09jNWP2UYJD0/[:b0;Zj^P80_kno00@0`<300;no_`2o_kl0`<30
0P2o_kl01`30`<00^[Zj0;:b/P2UYJD0WYjN09^KV`1FEUH0102e]KD03@39
bLT000000>k^kP3_knl0l?3`0?7al@3clo<0mOGe0?KfmP3hn?P0nOWi0?_k
n`3lo?`00`3moOd20?knoPD0oooo00000`000000bLW90;Zj^P040;6a/@03
08f=S@2ZZZX0ZZZZ00D0ZZZZ00<0YjNW0:ZZZP2ZZZX0102ZZZ[G0;6a/@03
07IfMP3De=@0l_;b0100l_;b00H0e=CD07IfMP2a/K40/K6a07IfMP3De=@A
0?;blP060=CDe01fMWH0/K6a0;6a/@1FEUH0Xj>S0P2XZ:P=0<?3``80Z:RX
00<0Xj>S05IFEP2a/K400P2a/K40102j^[X0bLW90000003clo<20?Cdm007
0?Gem@3fm_H0moOg0?Shn03jn_X0no_k0?clo0040?gmo@030?knoP3oool0
oooo00@0oooo00000`000000`l?30<W9b@3Z0:j^[P0408f=S@1fMWH0c/k>
0?_kn`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^[Zh0102=SHd0MWIf0<k>
cP3kno/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00H0VIVI05IFEP2e
]KD0bl_;0=oOg`3Xj>P90>OWi`060>SXj03Ogml0bl_;0;Fe]@1FEUH0VIVI
0`2^[Zh00`39bLT0`l?3000000020?KfmP050?Ogm`3hn?P0n?Sh0?Win@3j
n_X00P3lo?`40?gmo@80o_kn1P3oool0000400000033`l<0e=CD0;Ng]nX0
YjNW00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`03
08f=S@1fMWH0MWIf00l0MWIf00<0SHf=0:NWY`2WYjL00P2WYjL00`2=SHd0
EUIF05IFEP0?05IFEP0808f=S@2WYjL0YjNW0:NWY`2g]kL0e=CD0<?3``00
00030?Win@040?[jnP3kno/0o?cl0?clo0D0oOgm0P3no_h70?ooo`0000H0
00000:ZZZP3WinL0e=CD0;jn_P2c/k?o0:ZZZR/0ZZZZ00H0/k>c0;jn_P3D
e=@0inOW0:ZZZP0000020?_kn`<0o?cl1@3moOd30?knoPP0oooo00001`00
0000RXZ:0=WIf@3lo?`0i>CT0=;BdP3<c<`0o`2n_[hY0;jn_P080<c<c03B
d]80i>CT0?clo03IfMT0RXZ:0000003lo?`80?gmo@80o_kn2P3oool00008
0000001YJFT0UiNG0=WIf@3lo?`0l_;b0>c/k03Shn?o0=WIfBL0fMWI00P0
hn?S0>c/k03bl_80o?cl0=WIf@2GUiL0JFUY000000L0oOgm0`3no_h;0?oo
o`0000D000000?ooo`1YJFT0UiNG0=?Cd`020?clo?l0oooo:@3oool20?cl
o0050=?Cd`2GUiL0JFUY0?ooo`000000103moOd40?knoPd0oooo00000`00
0000oooo0?ooo`0206UYJ@0308Z:RP2GUiL0ZZZZ0?l0ZZZZ:02ZZZX0102G
UiL0RXZ:06UYJ@1YJFT20?ooo`030000003no_h0o_kn00@0o_kn3`3oool0
0?l00000=`00000E0?ooo`00\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{331, 218},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}],

Cell[TextData[{
  "There is actually quite a lot going on behind the scenes here in terms of \
the interaction between the kernel and the front end. As stated in Chapter 1, \
the kernel and the front end are two separate programs that communicate with \
each other through a protocol called ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  ". For purposes of efficiency, ",
  StyleBox["MathLink",
    FontSlant->"Italic"],
  " itself does not store the notebook in memory but instead refers to it by \
means of a ",
  StyleBox["handle", "TermDefinition"],
  ". These handles are called ",
  StyleBox["notebook objects", "TermDefinition"],
  " and are given as ",
  StyleBox["NotebookObject[", "MR"],
  StyleBox["fe", "IT"],
  StyleBox[",", "MR"],
  StyleBox["id", "IT"],
  StyleBox["]", "MR"],
  ", where ",
  StyleBox["fe",
    FontSlant->"Italic"],
  " is an object that refers to the entire front end and ",
  StyleBox["id",
    FontSlant->"Italic"],
  " is an integer that is a unique identifier for that notebook. In the \
example above, looking at the ",
  StyleBox["InputForm", "MR"],
  " displays this information stored with the notebook object."
}], "Text",
  CellTags->{"i:3", "b:43.0", "IPM3.10.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(InputForm[nb]\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell["\<\
NotebookObject[FrontEndObject[LinkObject[\"3v8_shm\", 
   1, 1]], 28]\
\>", "Output",
  CellLabel->"Out[2]//InputForm=",
  CellTags->{"b:43.0", "IPM3.10.2.1"}]
}, Open  ]],

Cell[TextData[{
  "Since we have assigned a symbol, ",
  StyleBox["nb", "MR"],
  ", to this object, we can refer to it through this symbol. ",
  StyleBox["NotebookGet", "MR"],
  " gets the expression corresponding to this notebook and reads it into the \
kernel. You should think of it as analogous to ",
  StyleBox["Get", "MR"],
  " for packages."
}], "Text",
  CellTags->{"i:5", "b:43.0", "IPM3.10.2.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(NotebookGet[nb]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],

Cell[BoxData[
    \(Notebook[{Cell[
          CellGroupData[{Cell["Demo notebook", "Section"], 
              Cell["This is a text cell", "Text"], Cell["1+2+3", "Input"]}, 
            Open]]}, FrontEndVersion \[Rule] "5.0 for Microsoft Windows", 
      ScreenRectangle \[Rule] {{0.`, 1024.`}, {0.`, 681.`}}]\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:43.0", "IPM3.10.2.1"}]
}, Open  ]],

Cell[TextData[{
  "Notice that the front end has added two options to this notebook: ",
  StyleBox["FrontEndVersion", "MR"],
  " and ",
  StyleBox["ScreenRectangle", "MR"],
  ". It has also added some grouping information for the cells. These are \
default behaviors of the front end and may vary from one front end to \
another. They are also user-settable."
}], "Text",
  CellTags->{"i:6", "b:43.0", "IPM3.10.2.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Manipulating notebooks", "Subsection",
  CellTags->{"b:43.1", "IPM3.10.2.2", "c:5"}],

Cell[TextData[{
  StyleBox["NotebookPut", "MR"],
  " and ",
  StyleBox["NotebookGet", "MR"],
  " are general functions for dealing with entire notebooks at once. There \
are a host of additional functions for manipulating parts of notebooks. You \
might first think that we can simply use functions like ",
  StyleBox["Part", "MR"],
  " to extract a particular part of a notebook we are interested in. There \
are several reasons why this is not generally practical. First, because a \
notebook can contain many, many cells, it is often quite difficult to \
determine precisely which part you want to work on. Secondly, since the \
notebook resides in the front end, not the kernel, it is often not very \
efficient to manipulate the notebook directly by the kernel (although, if the \
notebook is small enough, this is certainly possible)."
}], "TextFirst",
  CellTags->{"i:7", "b:43.1", "IPM3.10.2.2"}],

Cell["\<\
As it turns out, there is a way around these issues and that is through \
something referred to as the \[OpenCurlyDoubleQuote]current selection,\
\[CloseCurlyDoubleQuote] which is essentially a reference to the notebook \
object. You could then think of the notebook manipulation functions as \
operating on streams.\
\>", "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[TextData[{
  "To see a list of the open notebooks, use ",
  StyleBox["Notebooks[]", "MR"],
  "."
}], "Text",
  CellTags->{"i:8", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Notebooks[]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[BoxData[
    RowBox[{"{", 
      RowBox[{
        InterpretationBox[\(NotebookObject[\[LeftSkeleton]"Untitled-1"\
\[RightSkeleton]]\),
          NotebookObject[ 
            FrontEndObject[ 
              LinkObject[ "3v8_shm", 1, 1]], 28],
          Editable->False], ",", 
        InterpretationBox[\(NotebookObject[\[LeftSkeleton]"10FEProgramming.\
nb"\[RightSkeleton]]\),
          NotebookObject[ 
            FrontEndObject[ 
              LinkObject[ "3v8_shm", 1, 1]], 27],
          Editable->False], ",", 
        InterpretationBox[\(NotebookObject[\[LeftSkeleton]"Messages"\
\[RightSkeleton]]\),
          NotebookObject[ 
            FrontEndObject[ 
              LinkObject[ "3v8_shm", 1, 1]], 7],
          Editable->False]}], "}"}]], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]],

Cell[TextData[{
  "Again, using ",
  StyleBox["InputForm", "MR"],
  ", you can see the actual handles to each of the notebooks."
}], "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Notebooks[] // InputForm\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell["\<\
{NotebookObject[FrontEndObject[LinkObject[\"3v8_shm\", 
    1, 1]], 28], NotebookObject[
  FrontEndObject[LinkObject[\"3v8_shm\", 1, 1]], 27], 
 NotebookObject[FrontEndObject[LinkObject[\"3v8_shm\", 
    1, 1]], 7]}\
\>", "Output",
  CellLabel->"Out[5]//InputForm=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]],

Cell[TextData[{
  "Let us walk through some of the most common notebook operations you should \
learn about. The first is ",
  StyleBox["NotebookCreate", "MR"],
  ". As its name implies, this function will create a new untitled notebook \
in the front end. We assign ",
  StyleBox["nb", "MR"],
  " to be the handle to this notebook."
}], "Text",
  CellTags->{"i:9", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(nb = NotebookCreate[]\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[BoxData[
    InterpretationBox[\(NotebookObject[\[LeftSkeleton]"Untitled-2"\
\[RightSkeleton]]\),
      NotebookObject[ 
        FrontEndObject[ 
          LinkObject[ "3v8_shm", 1, 1]], 29],
      Editable->False]], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005S0000hb000`400?l0
0000o`00003oo`3ooomT0?ooo`00o`3ooomT0?ooo`003P3ooooo0?knoTL0
o_kn3`3oool000`0oooo103no_ko0?gmoD<0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgmB@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo3d0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k?`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinD00nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_Ho0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clcd0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hm0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj3d0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4m0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH>`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllk0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aC/0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^c/0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a>`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYS/0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhk09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXm09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000004l000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?clB`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[ZdX0Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFU906UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj6`2OWil00`2m_Kd0k>c/0>OWi`0T0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K6`3:b/X0
0`3Kfm/0Xj>S0?_kn`0T0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo6`3@d=000`3oool0UYJF0?_kn`0T0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3ooolK0<o?c`03
0?ooo`2FUYH0no_k02@0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM400`3@d=070=7Ad@80d=3@0P3AdM430=3@
d0L0dM7A00<0oooo09JFUP3jn_X0903jn_X00`2FUYH0oooo0<k>cP040<k>
cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JFUP3Ph>00403Ph>00501Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0
h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3Ph01YJFT0WYjN0>3Ph02>
SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0
103>c/h303hn?PD0c/k>00@0oooo09JFUP2FUYH0oooo203Fe]H20=OGe`07
09FEU@3Fe]H0UIFE0=KFeP2EUID0e]KF09FEU@050=KFeP80emOG0`3Fe]H0
0`3oool0UYJF0?Win@0T0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n?Sh6
0<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h034a<@0B4Q80=;BdP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0
g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?oo
o`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3Lg=`6
0=gMg@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID70=gMg@80
g=cL00<0gMgM0?ooo`2FUYH09@3inOT00`2FUYH0oooo0<o?c`040<o?c`<0
?Shn1@3?cll00`3oool0UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c
/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3I
fMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV
9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103o
ool0UYJF09JFUP3oool30=cLg0H0gMgM00P0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@L0gMgM0P3Lg=`00`3MgMd0oooo09JFUP0U0?Shn003
09JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0h>3P00P0
h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00G0<C4
a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B
0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2NWYh0
3`3Ph>05000000H0h>3P00@0UYJF0?ooo`3Kfm/0fm_K0`0n?Sh80=_Kf`05
0?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0<0hn?S00T0i>CT0?oo
o`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Ti>@30>?Sh`<0i>CT
00@0hn?S0>CTi03oool0UYJF9@3gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0
?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P
01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV
9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0a<C4
04Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WY
j@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY
00D0j^[Z0>WYj@3YjNT0oooo09JFUP0U0?Gem@0309JFUP3oool0i>CT00D0
i>CT0`0n?Sh40>CTi0030?ooo`2FUYH0h>3P00d0h>3P0P3oool05@3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P01T0SXj>0000000h>3P0fMWI0>3Ph01YJFT0WYjN
0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph02][Jd0
E5AD0>3Ph01YJFT0OWin0>3Ph01YJFT0WYjN0100h>3P00<000000>3Ph03P
h>001P3Ph>000`2FUYH0oooo0>g]k@030>g]k@<0?Shn1P3]kNd01@3oool0
UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WYj@080?ooo`2EUID0oooo
09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY00D0j^[Z0>WYj@3YjNT0
oooo09JFUP0U0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]k@03
0?ooo`2FUYH0h>3P00`0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0
h>3P01T0fMWI059BDP3IfMT0h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P
0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3Bd]80
h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@303hn
?PD0m?Cd00@0oooo09JFUP2FUYH0oooo2@3]kNd01`3oool0kNg]0?ooo`3]
kNd0oooo0>g]k@3oool02`3]kNd00`3oool0UYJF0?7al@0T0?7al@0309JF
UP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2FUYH0h>3P00`0h>3P
0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jN
WP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3lo?`01@3l
o?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?oooa/0kNg]
00<0oooo09JFUP3`l?00903`l?000`2FUYH0oooo0?clo0040?clo00303hn
?P3lo?`0o?cl00D0o?cl00<0oooo09JFUP3Ph>00403Ph>00501YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph00309JFUP3oool0oooo00d0oooo0P2FUYH00`3oool0moOg0?Og
m`0I0?Ogm`030?ooo`2FUYH0kno_02@0kno_00<0UYJF0?ooo`3oool03@3o
ool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?cl
o02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF
0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XK0?ooo`030?[jnP2F
UYH0kno_02@0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF0>3P
h00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F
0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e]KD0
1@2d];@0Shn?0=[JfP3Fe]H0/;2`01/0^;Rh00<0ZJVY0<k>cP3^k^h0903^
k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0f][J0140h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFVG09jNWP030?ooo`3Ph>00
h>3P03D0h>3P4@3/k>`O0>_[jbD0inOW4@3/k>`A0>3Ph00D06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`060>c/k03bl_80eMGE
0<[:bP37alL0`l?31P31`L406@32`/80a<C40<k>cP3emOD0jn_[06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0hN7Q
0?ooo`0<0?ooo`0F0=SHf03[jn/0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00@0k>c/0?ooo`3alO40i>CT1`3UiND20>CTi00H
0>;RhP3`l?00oooo0=3@d01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool01@3/k>`0oooo0>7Qh@3YjNT0j^[Z0080jn_[0P3Zj^X0
703YjNT0j>SX0>OWi`3Vi^H0iNGU0>;RhP3oool0b/[:06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k03oool0i>CT0>_[
j`@0k>c/01d0jn_[0>[ZjP3Zj^X0jNWY0>SXj03Vi^H0hn?S0?ooo`3:b/X0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/
k03oool0i>CT0>c/k0@0kNg]01d0b<S80>_[j`3Zj^X0jNWY0>SXj03WinL0
i>CT0?ooo`3:b/X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`040>c/k03oool0iNGU0>g]k@<0k^k^0`38b<P30>WYj@0H0>SX
j03UiND0oooo0<[:bP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0oooo0>GUi@030>k^kPD0b<S80P3Zj^X0603YjNT0
iNGU0?ooo`3:b/X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00<0k>c/0?ooo`3Vi^H00P3_knl30<S8b00M0>c/k038b<P0b<S8
0<S8b03Zj^X0jNWY0>GUi@3oool0b/[:06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0oooo0>OWi`3_knl30<S8b080
k>c/01`0j^[Z0<S8b038b<P0b<S80>[ZjP3Vi^H0oooo0<[:bP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0oooo0>WY
j@020?3`l0030<S8b03_knl0kno_00<0jn_[01X0b<S80>_[j`3Zj^X0i^KV
0?ooo`3;bl/0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/0?ooo`3YjNT00`3`l?030>o_k`D0jn_[01L0i^KV0?ooo`3=
cLd0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`06
0>c/k03oool0jn_[0?;blP3alO40l?3`0`3_knl00`3/k>`0jn_[0>_[j`02
0>_[j`0G0>KViP3oool0dM7A06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool01P3/k>`0oooo0>g]k@3emOD0m?Cd0?7al@80l?3`
01d0kno_0>k^kP3^k^h0kNg]0>c/k03[jn/0iNGU0?ooo`3Fe]H0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`070>c/k03oool0
l?3`0?Ogm`3emOD0l_;b0?7al@030?3`l00K0>o_k`3^k^h0kNg]0>_[j`3U
iND0oooo0=gMg@1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00P0k>c/0?ooo`3gmoL0l?3`0>g]k@3[jn/0j^[Z0>WYj@80inOW
0P3Vi^H06@3UiND0hn?S0?;blP3oool0i>CT06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0o?cl0?ooo`0=0?ooo`0E
0>_[j`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo
00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0
k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
0`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/
k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`03
0>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/
k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0
k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/
k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/
0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0
no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03k
no/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_k
n`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k
0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0
no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3k
no/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_k
n`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k
00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/0
3@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=
0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0
no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3k
no/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_k
n`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k
01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/0
5@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E
0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo
00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0
k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
0`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/
k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`03
0>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/
k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0
k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/
k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/
0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0
no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03k
no/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_k
n`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k
0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0
no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3k
no/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_k
n`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k
00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/0
3@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=
0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0
no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3k
no/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_k
n`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k
01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/0
5@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E
0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo
00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0
k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
0`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/
k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`03
0>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/
k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0
k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/
k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/
0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0
no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03k
no/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_k
n`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k
0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0
no_k00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3k
no/03@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`030>c/k03kno/0no_k00d0no_k01D0k>c/06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0no_k0?_k
n`0=0?_kn`0E0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00<0k>c/0?_kn`3kno/03@3kno/05@3/k>`0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03kno/0no_k
00d0no_k01D0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0no_k0?_kn`0=0?_kn`0E0>c/k01YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?_kn`3kno/0
3@3kno/05@3/k>`0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`060>c/k03bl_80eMGE0<[:bP37alL0`l?31P31`L406@32`/80
a<C40<k>cP3emOD0jn_[06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0hN7Q0?ooo`0<0?ooo`0F0=SHf03[jn/0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/0?oo
o`3alO40i>CT1`3UiND20>CTi00H0>;RhP3`l?00oooo0=3@d01YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool01@3/k>`0oooo0>7Q
h@3YjNT0j^[Z0080jn_[0P3Zj^X0703YjNT0j>SX0>OWi`3Vi^H0iNGU0>;R
hP3oool0b/[:06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k03oool0i>CT0>_[j`@0k>c/01d0jn_[0>[ZjP3Zj^X0jNWY
0>SXj03Vi^H0hn?S0?ooo`3:b/X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`040>c/k03oool0i>CT0>c/k0@0kNg]01d0k>c/
0>_[j`3Zj^X0jNWY0>SXj03WinL0i>CT0?ooo`3:b/X0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`070>c/k03oool0iNGU0>g]
k@3^k^h0b<S80>k^kP020>g]k@0L0>_[j`3YjNT0b<S80>WYj@3Xj>P0iNGU
0?ooo`3:b/X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00@0k>c/0?ooo`3UiND0k^k^0`38b<P00`3^k^h0kNg]0>_[j`030<S8
b00H0>WYj@3UiND0oooo0<[:bP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool00`3/k>`0oooo0>KViP020>o_k`<0b<S801d0k>c/
0<S8b038b<P0b<S80>[ZjP3YjNT0iNGU0?ooo`3:b/X0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k03oool0inOW0>o_
k`80l?3`1@38b<P30>[ZjP0G0>KViP3oool0b/[:06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0oooo0>WYj@020?3`
l080kno_0`38b<P06`3[jn/0j^[Z0>_[j`3Zj^X0i^KV0?ooo`3;bl/0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03o
ool0jNWY00<0l?3`0P3_knl00`38b<P0jn_[0>_[j`030>_[j`0G0>KViP3o
ool0cLg=06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool01P3/k>`0oooo0>_[j`3bl_80lO7a0?3`l0<0kno_00<0k>c/0>_[j`3[
jn/00P3[jn/05`3Vi^H0oooo0=7Ad@1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo000003H0000000H0k>c/0?ooo`3]kNd0mOGe0?Cdm03alO42
0?3`l00M0>o_k`3^k^h0k^k^0>g]k@3/k>`0jn_[0>GUi@3oool0e]KF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool01`3/k>`0
oooo0?3`l03gmoL0mOGe0?;blP3alO400`3`l?006`3_knl0k^k^0>g]k@3[
jn/0iNGU0?ooo`3MgMd0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`080>c/k03oool0moOg0?3`l03]kNd0jn_[0>[ZjP3YjNT2
0>OWi`80i^KV01T0iNGU0>?Sh`3bl_80oooo0>CTi01YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?clo03oool03@3o
ool05@3[jn/0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`2JVYX0o`1jNWY907Yj
NP0C09ZJVP3gmoL0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000<000000?Ogm`2o_kl0o`2o_km:0;no_`0B0?Ogm`00
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00003
0000003gmoL0hn?S0?l0hn?SBP3Shn<04P3gmoL0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000lO7a0?ooo`3o
0?ooo`<0oooo00<0hn?S0:FUY@1fMWH00`1fMWH0101gMgL0MWIf07MgM`1g
MgL907IfMP060:FUY@3Shn<0oooo0?ooo`3Shn<0YJFU101fMWH0101gMgL0
MWIf07MgM`1gMgL907IfMP060:FUY@3Shn<0oooo0?ooo`3[jn/0YJFU101F
EUH0101GEeL0EUIF05MGE`1GEeL905IFEP030:FUY@3[jn/0oooo00<0oooo
0180lO7a0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000>7Qh@3lo?`0no_k0P3lo?`00`3kno/0o?cl0?clo004
0?clo0030?_kn`3lo?`0o?cl0080o?cl00<0no_k0?clo03kno/01@3lo?`0
0`3kno/0o?cl0?_kn`1I0?_knhl0o?cl00<0YJFU07IfMP2=SHd0402=SHd0
1P1fMWH0YJFU0?clo03lo?`0YJFU07IfMQ40SHf=00H0MWIf0:FUY@3lo?`0
no_k0:FUY@1FEUHA05]KF`0305IFEP2UYJD0o?cl00<0o?cl0180hN7Q0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000>3Ph03gmoL0o`3gmoL30?Ogm`0307IfMP2=SHd0XZ:R0100XZ:R00H0
SHf=07IfMP3gmoL0moOg07IfMP2=SHdA0::RXP0808f=S@1fMWH0moOg0?Og
m`1FEUH0Fe]K06m_K`1aLG4=079bLP04075aL@1_Kfl0Fe]K05IFEP@0moOg
0180h>3P0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000<000000=kNgP3alO400P3alO420?;blP030?7al@3bl_80l_;b
00@0lO7a00<0l_;b0?7al@3alO400`3alO420?;blP030?7al@3bl_80lO7a
0080l_;bjP3alO400`1fMWH0^;Rh0:^[Z`0@0:^[Z`060;Rh^01fMWH0lO7a
0?7al@1fMWH0^;Rh4@2[Zj/01@2h^;P0MWIf0?7al@3bl_80EUIF0080Kfm_
00<0LW9b07MgM`1gMgL02`1gMgL0101bLW80Kfm_06m_K`1FEUH30?7al@0C
0?;blP3Ng]h0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000D000000=cLg03[jn/0jn_[0>c/k0020>_[j`030:NWY`2R
XZ80XZ:R0080XJ6Q00<0X:2P09nOW`2MWId00P2OWil20:2PX080XJ6Q00<0
YjNW0>_[j`3[jn/01@3[jn/01@3De=@0aLG50<G5a@35aLD0emOG0080jn_[
00<0j^[Z0<S8b03@d=000P3[jn/01@3Jf]X0a/K60>?Sh`3[jn/0h>3P0080
aLG500L0c<c<0>WYj@3>c/h0alO70>WYj@3[jn/0h>3P0080aLG500L0c<c<
0>[ZjP3De=@0aLG50=gMg@3[jn/0h>3P0080aLG500H0a/K60>?Sh`3[jn/0
jn_[0>[ZjP38b<P30<G5a@030<K6aP3Shn<0jn_[00@0jn_[00<0j^[Z0<S8
b035aLD00`35aLD00`36a/H0hn?S0>_[j`0K0>_[je/0k>c/=03[jn/0101f
MWH0^k^k0;6a/@2a/K480?ooo`L0/K6a00H0^k^k07IfMP3[jn/0jn_[07If
MP2k^k/20;6a/@`0oooo0`2a/K402@2k^k/0MWIf0>_[j`3[jn/0EUIF07Mg
M`1bLW80M7Ad07MgM`0;07QhN00507MgM`1dM7@0LW9b07MgM`1FEUH0103[
jn/04P3Lg=`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo00000`000000f][J0>GUi@030>GUi@0308Z:RP2KVi/0[:b/00<0
jNWY00X0j^[Z0<C4a025QHD0k>c/0>g]k@3^k^h0inOW0?3`l03bl_80Xj>S
1P3UiND04@1XJ6P0410@010@400@4100P8200<K6aP3?cll0g]kN02HV9P1A
DE40clo?0>GUi@2DU9@05ADE0:BTY03Ng]h0/[:b0080410@00L0@D510=_K
f`1@D5006ATI0;Zj^P3Shn<0/[:b0080410@00L0@D510=oOg`1`L700410@
09:BTP3Ti>@0[jn_0080410@00H05ADE0;Zj^P3<c<`0i>CT0=gMg@0T92@3
010@400301DE5@2TY:@0g]kN00@0iNGU00<0g]kN02HV9P0@41000`0@4100
0`0E5AD0_[jn0>GUi@2[0>GUi@0507IfMP30`<00]KFe0;Fe]@3oool01P1b
LW800`0W9bL0]KFe0;Fe]@050;Fe]@060<30`01fMWH0iNGU0>GUi@1fMWH0
`<300P2e]KD00`3oool09bLW02LW9`0902LW9`<0]KFe00D0`<3007IfMP3U
iND0iNGU05IFEP0207MgM`0407UiN@1jNWX0oooo0?ooo`L0NWYj0P3oool0
1@1jNWX0NGUi07MgM`1gMgL0EUIF00@0iNGU0180f][J0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=WIf@3O
gml00`3Ogml01@3Ng]h0X:2P0>[ZjP3YjNT0iNGU0080j^[Z00X0d=3@07Ad
M03]kNd0k^k^0:f][@3@d=00l_;b0??cl`2SXj<0gmoO0P3Ng]h0=03Ogml0
g]kN06a/K00E5AD0Zj^[0=kNgP33`l<071`L05ADE036a/H0e]KF02DU9@1A
DE40cLg=0=kNgP2?Shl05ADE0:BTY03EeMD0CTi>03Ph>034a<@0gMgM0=_K
f`1>CTh06ATI0;Zj^P3IfMT0CTi>03Ph>034a<@0gMgM0=OGe`0U9BD0>3Ph
0<C4a032`/806a/K055AD@3IfMT0`l?304a<C03Hf=P0emOG02DU9@0I6AT0
aLG50=kNgP2?Shl05ADE0:BTY03IfMT50=kNgP050:f][@0@41006ATI0;Vi
^@3AdM40[P3Ng]h01@1fMWH0`l?30;^k^`2k^k/0oooo00H0LW9b00<09bLW
0;^k^`2k^k/01@2k^k/01P33`l<0MWIf0=kNgP3Ng]h0MWIf0<?3``80^k^k
00<0oooo02LW9`2k^k/01`2k^k/00`3oool09bLW0;^k^`020;^k^`090<?3
``1fMWH0gmoO0=kNgP1FEUH0MgMg07QhN01kNg/0<30`00<0oooo1@1mOGd3
0?ooo`05030`<01kNg/0N7Qh07MgM`1FEUH00P3Ng]h0503Ogml0g]kN0=WI
f@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003GemL0f=SH0=WIf@80f=SH0100T92@08n?S`2>SXh0jNWY0861
P@1ADE40QhN70=[JfP2VYZH0ZJVY0:b/[01OGel0l_;b0??cl`3dm?@0Y:BT
0`3Hf=P01@3IfMT0d];B02@T901ADE40bl_;0080f=SH0200JVYZ01TI6@2j
^[X0d=3@02@T901ADE40bl_;0=SHf02;Rh/05ADE0:BTY03=cLd092@T03Ph
>033`l<0emOG0=GEe@1<C4`06ATI0;Zj^P3@d=0092@T03Ph>033`l<0emOG
0=7Ad@0T92@0>3Ph0<?3``2:RXX05ADE0:BTY0@0aLG500X0WYjN010@401a
LG40dM7A0=SHf02;Rh/05ADE0:BTY03De=@0d];B0P35aLD02036a/H0emOG
0=SHf01ZJVX0410@079bLP35aLD0e=CD7P3Hf=QK0=WIfC@0f=SH00D0MWIf
0<O7a`2o_kl0_kno0?ooo`0702LW9`L0_kno00H0alO707IfMP3Hf=P0f=SH
07IfMP37alL20;no_`030?ooo`0W9bL0_kno00L0_kno00<0oooo02LW9`2o
_kl00P2o_kl01@37alL0MWIf0=SHf03Hf=P0EUIF0080Ng]k00<0OWin07mo
O`0X:2P00`3oool307moO`<0oooo00<0:2PX07moO`1nOWh00P1kNg/00`1F
EUH0fMWI0=SHf0020=SHf00B0=OGe`000000VYZJ0:2PX02XZ:P0/[:b0;bl
_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000030000003EeMD0dm?C00<0dm?C00<0
QHF50:>SX`3Vi^H00P3Zj^X02`0e=CD0Ogmo0<o?c`2n_[h0Rh^;05iNGP1H
F5P0J6QX0:>SX`39bLT0ZZZZ00@0dm?C00@0c<c<02@T901ADE40b/[:0P3C
dm<07P1WIfL06ATI0;Vi^@3;bl/092@T055AD@3:b/X0dm?C08R8R00E5AD0
Y:BT0<W9b@0T92@0>3Ph0<71`@3Cdm<0clo?04Y:BP0I6AT0^KVi0<_;b`0T
92@0>3Ph0<71`@3Cdm<0c<c<02@T900h>3P0`L7106MWI`D0410@00X0DE5A
09fMW@0@4100SXj>0<c<c03Cdm<0R8R801DE5@2TY:@0XJ6Q0P0@41002@0J
6QX0a/K60=?Cd`3Cdm<0c<c<02@T900E5AD0Y:BT0<O7a`2]0=?Cd`0307If
MP3;bl/0a/K60100a/K600H0bl_;07IfMP3Cdm<0dm?C07IfMP3;bl/20<K6
aP030?ooo`0W9bL0a/K600L0a/K600<0oooo02LW9`36a/H00P36a/H01`3;
bl/0MWIf0=?Cd`3Cdm<0EUIF07emO@1oOgl00`23Ph<01@0X:2P0oooo0?oo
o`3oool0Ph>300<0oooo00L0:2PX08>3P`23Ph<0Ph>307moO`1mOGd0EUIF
00@0dm?C0140eMGE0000002PX:00YJFU0:f][@2f][H0_kno0<S8b03Bd]80
f][J0>;RhP3YjNT0kno_0?Cdm03fm_H0nOWi0?_kn`030?gmo@40o_kn0@3o
ool10?ooo`0000D000000=?Cd`3>c/h0c/k>0<o?c`020<k>cP0?08N7Q`2C
Ti<0dm?C0=_Kf`2][Jd0`L710;2`/02YZJT0ZZZZ08f=S@2ATI40/;2`0<_;
b`3_knl0YJFU00<0c/k>00D0clo?0<O7a`0S8b<0DE5A0<S8b0020<k>cP0]
06EUI@0I6AT0^KVi0<O7a`0S8b<0DE5A0<O7a`3>c/h0QHF501DE5@2TY:@0
alO702<S8`0h>3P0`L710<k>cP3;bl/0BDU901TI6@2i^KT0alO702<S8`0h
>3P0`L710<k>cP37alL08b<S03Ph>031`L40QHF501DE5@2TY:@0b/[:0<O7
a`0S8b<0DE5A0;6a/@0J6QX0LW9b0<G5a@3>c/h0QHF501DE5@2TY:@0cLg=
00L0c/k>00D0X:2P010@400i>CT0_kno0<_;b`0M0<k>cU/0clo?=03>c/h0
0`1fMWH0c/k>0<_;b`0@0<_;b`060<k>cP1fMWH0c/k>0<k>cP1fMWH0c/k>
0P3;bl/00`3oool09bLW0<_;b`070<_;b`030?ooo`0W9bL0bl_;0080bl_;
00T0c/k>07IfMP3?cll0c/k>05IFEP1mOGd0PX:208F5Q@24Q8@00P25QHD0
0`0X:2P0oooo0?ooo`030?ooo`0802PX:025QHD0QHF508B4Q025QHD0PX:2
07emO@1FEUH40<k>cP0A0=?Cd`000000Z:RX0:f][@2c/k<0^k^k0<?3``3<
c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Win@3lo?`00`3moOd1
0?knoP40oooo0@3oool000030000003@d=00bLW900<0bLW90140Lg=c09BD
U01nOWh0NGUi0:2PX03AdM40[Jf]0:FUY@2VYZH0QHF505iNGP2][Jd0iNGU
0?KfmP3hn?P0Y:BT0<W9b@030<S8b0040<;2`P0S8b<0DE5A0<O7a`80b<S8
02`0HV9R01TI6@2i^KT0`/;2028R8P0h>3P0`L710<;2`P16ATH06ATI0;bl
_031`L408b<S03Ph>02o_kl0aLG50<C4a016ATH06ATI0;Vi^@31`L408b<S
03Ph>02o_kl0aLG50<30`00R8R80>3Ph0<30`02_[jl06QXJ07=cL`35aLD0
VYZJ010@402>SXh0aLG504I6AP0I6AT0^KVi0<G5a@1oOgl05ADE0:BTY0T0
b<S800@0HV9R010@402=SHd0alO77@38b<QK0<W9bC@0b<S800<0MWIf0=?C
d`3=cLd0403=cLd01P3Cdm<0MWIf0<S8b038b<P0MWIf0=?Cd`80cLg=00<0
oooo02LW9`3=cLd01`3=cLd00`3oool09bLW0<g=c@020<g=c@070=?Cd`1f
MWH0b<S80<S8b01FEUH0PX:208>3P`0508N7Q`0502PX:03oool0oooo0?oo
o`0X:2P01@27QhL00`23Ph<0PX:205IFEP030<W9b@0B0<S8b03@d=000000
0;:b/P2f][H0^k^k0<;2`P39bLT0dM7A0=WIf@3Ph>00inOW0>g]k@3alO40
mOGe0?Shn03jn_X0o?cl0`3moOd10?knoP40oooo0@3oool000030000003?
cll0aLG500@0aLG500l0UiNG0;Ng]`3/k>`0^KVi08^;R`2g]kL0ZZZZ0:BT
Y01nOWh0LG5a0<C4a03IfMT0m_Kf0?Win@2UYJD01035aLD0102o_kl08R8R
055AD@36a/H20<G5a@06065QH@0I6AT0^KVi0;no_`0R8R80GEeM0P0@4100
100E5AD0Y:BT0<;2`P15ADD3010@4006055AD@1QHF40410@01TI6@2f][H0
ADE50`0@4100201ADE40_kno028R8P0h>3P0_kno0<G5a@2/[:`06QXJ0P0@
41001@1bLW80aLG50<G5a@32`/80ADE500<0410@00<05ADE0:BTY036a/H0
2035aLD00`2/[:`06QXJ055AD@2]0<G5a@0307IfMP3EeMD0dM7A0100dM7A
00H0eMGE07IfMP35aLD0aLG507IfMP3EeMD20=7Ad@030?ooo`0W9bL0dM7A
00L0dM7A00<0oooo02LW9`3AdM400P3AdM401`3EeMD0MWIf0<G5a@35aLD0
EUIF08:2PP27QhL01@29RHT50?ooo`D0RHV900<0QhN708:2PP1FEUH01035
aLD04@3?cll000000;bl_02o_kl0`l?30<W9b@3@d=00emOG0=gMg@3Ti>@0
j^[Z0>o_k`3clo<0m_Kf0?Shn03kno/0o?cl0080oOgm0P3no_h20?ooo`00
00<000000<g=c@31`L400`31`L40401eMGD0VIVI0:NWY`3]kNd0bLW907=c
L`2`/;00SHf=07=cL`1<C4`0HV9R0;jn_P2e]KD0/[:b0:f][@2YZJT40<71
`@040;^k^`0R8R80DE5A0<C4a080`L7100@0GemO01TI6@2i^KT0`/;21`31
`L401@2k^k/08R8R03Ph>02n_[h0`/;20080`L7100`0`l?30<G5a@2l_;`0
8R8R03Ph>02n_[h0`/;20;^k^`0R8R80>3Ph0;jn_P32`/8;0<71`@0407al
O00E5AD0Y:BT0<?3``H0`L710P34a<@0102n_[h0A4A403Ph>02n_[j/0<71
`@0307IfMP3Hf=P0eMGE0100eMGE00H0f=SH07IfMP31`L40`L7107IfMP3H
f=P20=GEe@030?ooo`0W9bL0oooo00P0oooo00H09bLW0=GEe@3EeMD0eMGE
0=SHf01fMWH20<71`@0305IFEP24Q8@0R8R800@0RXZ:0`3oool0100X:2P0
oooo0?ooo`3oool408Z:RP0308R8R024Q8@0EUIF00@0`L710100cLg=0000
0036a/H0b<S80<c<c03AdM40emOG0=cLg03Rh^80j>SX0>g]k@3alO40mOGe
0?Ogm`3inOT0o?cl0`3moOd20?knoP80oooo00000`000000c<c<0;Zj^P02
0;fm_@0A0;jn_P2m_Kd0WYjN0>?Sh`2f][H0UYJF08:2PP2CTi<0Q8B408:2
PP1>CTh0;2`/09bLW02[Zj/0m_Kf0?Shn02WYjL0102m_Kd0102g]kL08B4Q
055AD@34a<@20;fm_@0405eMG@0I6AT0^KVi0;no_`P0_Kfm00<0GEeM03Lg
=`2h^;P00P2m_Kd02`1MGEd06ATI0;Vi^@2o_kl0GEeM03Lg=`2h^;P0_Kfm
0;Ng]`0Q8B40>3Ph0080_[jn2`2m_Kd0101jNWX05ADE0:BTY02o_kl40;fm
_@070;Ng]`0Q8B40Kfm_0<G5a@2o_kl08b<S071`L00N0;fm_E/0_[jn=02m
_Kd00`1fMWH0f][J0=GEe@0@0=GEe@060=[JfP1fMWH0_Kfm0;fm_@1fMWH0
f][J0P3EeMD00`3oool0LW9b079bLP08079bLP0>02LW9`3EeMD0eMGE0=GE
e@3Jf]X0MWIf0;fm_@2n_[h0EUIF08N7Q`2:RXX0SHf=08j>SP2=SHd30?oo
o`0302PX:02=SHd0:2PX00<0oooo0`2=SHd0102:RXX0QhN705IFEP2n_[h2
0;fm_@0A0;Zj^P3<c<`000000=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`00`3moOd00`3no_h0oooo
0?ooo`010?ooo`0000<000000<[:bP2h^;P00`2j^[X0401lO7`0SHf=07Qh
N02IVIT0l?3`0<;2`P2WYjL0bl_;0;bl_02SXj<0KFe]07emO@2/[:`0no_k
0?clo02VYZH30;Zj^P040;^k^`2d];@08B4Q04e=C@<0^[Zj00<0Fe]K01TI
6@2_[jl03P2j^[X00`1KFe/06ATI0:n_[`050;Zj^P040;Bd]00Q8B40=cLg
0;Bd]0`0^[Zj00<0MgMg01@D502KVi/01P2j^[X01@2ATI40410@010@400@
4100CDe=0:h0^[Zj00<0MWIf0=SHf03Jf]X0403Jf]X01P3Hf=P0MWIf0;Zj
^P2j^[X0MWIf0=SHf080f][J00<0oooo079bLP1bLW80201bLW803@0W9bL0
f][J0=[JfP3Jf]X0f=SH07IfMP2j^[X0^k^k05IFEP2;Rh/0Shn?08j>SP2?
Shl00`3oool01@0X:2P0Shn?08n?S`2?Shl0:2PX00<0oooo00D0Shn?08j>
SP2?Shl0Rh^;05IFEP030;Zj^P0@0;Rh^03:b/X000000=WIf@3Jf]X0gMgM
0>3Ph03Ti>@0j>SX0>c/k03_knl0lo?c0?KfmP3hn?P0n_[j0?clo0<0oOgm
0P3no_h30?ooo`0000<000000<W9b@2h^;P00`2h^;P04026QXH0WYjN0=CD
e03`l?00l_;b0<k>cP31`L40gMgM0=;BdP2RXZ80c<c<0<W9b@29RHT0o?cl
0?gmo@2WYjO_0;Rh^00307IfMP3GemL0gMgM0100gMgM00H0emOG07IfMP2h
^;P0^;Rh07IfMP3GemL30=gMg@/09bLW0`3MgMd02@3GemL0MWIf0;Rh^02h
^;P0EUIF092@T02ATI40Ti>C09BDU0020?ooo`0302PX:02HV9P0V9RH00<0
V9RH00P0:2PX0?ooo`3oool0U9BD09>CT`2ATI40T92@05IFEP@0^;Rh00l0
bLW90000003QhN40h^;R0>CTi03WinL0j^[Z0>g]k@3`l?00lo?c0?Gem@3g
moL0nOWi0?_kn`3lo?`00`3moOd00`3no_h0oooo0?ooo`020?ooo`0000<0
00000<W9b@2h^;P0102h^;P03`26QXH0U9BD0?;blP3clo<0X:2P0>SXj03f
m_H0fm_K09VIV@3jn_X0no_k0=cLg02f][H0o_kn0:NWY`3_0;Rh^00307If
MP3Fe]H0hN7Q0100hN7Q00H0e]KF07IfMP2h^;P0^;Rh07IfMP3Fe]HA0>7Q
h@090=KFeP1fMWH0^;Rh0;Rh^01FEUH0UIFE09NGU`2LW9`0XZ:R0080:2PX
00D0Zj^[0:f][@2[Zj/0Zj^[0:f][@020:^[Z`80:2PX00D0XZ:R09bLW02G
UiL0UIFE05IFEP040;Rh^00>0<W9b@000000j>SX0>WYj@3[jn/0kNg]0>o_
k`3alO40m?Cd0?KfmP3gmoL0nOWi0?_kn`3lo?`30?gmo@80o_kn103oool0
000300000039bLT0]KFe00<0]KFe00d0MgMg0:2PX03Ph>00lo?c0>GUi@3/
k>`0m_Kf0?Shn03inOT0T92@0?_kn`3lo?`0oOgm0080o_kn00<0YjNW0;Fe
]@2e]KD0k@2e]KD00`1fMWH0kNg]0>WYj@0@0>WYj@060>g]k@1fMWH0]KFe
0;Fe]@1fMWH0kNg]4@3YjNT02P3]kNd0MWIf0;Fe]@2e]KD0EUIF09^KV`2N
WYh0YJFU0;:b/P2j^[X20;no_`040<30`02o_kl0_kno0<30`080_kno00L0
`<300;Zj^P2b/[80YJFU09jNWP2KVi/0EUIF00@0]KFe00d0bLW90000003^
k^h0kno_0?3`l03alO40lo?c0?Gem@3fm_H0n?Sh0?Win@3kno/0o?cl00<0
oOgm0P3no_h50?ooo`0000<000000<W9b@2j^[X0102a/K400`2=SHd0ZZZZ
0:ZZZP050:ZZZP030:NWY`2ZZZX0ZZZZ00@0ZZZZk`2a/K400`1fMWH0e=CD
0?;blP0@0?;blP060=CDe01fMWH0/K6a0;6a/@1fMWH0e=CD4@3bl_801P3D
e=@0MWIf0;6a/@2a/K40EUIF0:>SX`80Z:RX3@33`l<20:RXZ0030:>SX`1F
EUH0/K6a0080/K6a00@0^[Zj0<W9b@000000lo?c0P3dm?@01`3emOD0m_Kf
0?Ogm`3hn?P0n_[j0?_kn`3lo?`0103moOd00`3no_h0oooo0?ooo`040?oo
o`0000<000000<?3``39bLT0o`2^[Zh30:j^[P0408f=S@1fMWH0c/k>0?_k
n`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^[Zh0102=SHd0MWIf0<k>cP3k
no/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00H0VIVI05IFEP2e]KD0
bl_;0=oOg`3Xj>P90>OWi`060>SXj03Ogml0bl_;0;Fe]@1FEUH0VIVI0`2^
[Zh00`39bLT0`l?3000000020?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X0
0P3lo?`40?gmo@80o_kn1P3oool0000400000033`l<0e=CD0;Ng]ol0YjNW
0`2WYjL00`2=SHd0MWIf07IfMP0?07IfMP0308f=S@2WYjL0YjNW0080YjNW
00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=
S@1FEUH0EUIF00l0EUIF00P0SHf=0:NWY`2WYjL0YjNW0;Ng]`3De=@0`l?3
000000<0nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo0000
1P000000ZZZZ0>OWi`3De=@0_[jn0;>c/ol0ZZZZ@`2ZZZX01P2c/k<0_[jn
0=CDe03WinL0ZZZZ00000080no_k0`3lo?`50?gmo@<0o_kn203oool00007
0000002:RXX0fMWI0?clo03Ti>@0d];B0<c<c03o0;jn_T40_[jn00P0c<c<
0=;BdP3Ti>@0o?cl0=WIf@2:RXX000000?clo0P0oOgm0P3no_h:0?ooo`00
00P0000006UYJ@2GUiL0fMWI0?clo03bl_80k>c/0>?Shol0fMWI?`3IfMT0
203Shn<0k>c/0?;blP3lo?`0fMWI09NGU`1YJFT000001`3moOd30?knoP/0
oooo00001@000000oooo06UYJ@2GUiL0dm?C0080o?clo`3ooom10?ooo`80
o?cl00D0dm?C09NGU`1YJFT0oooo000000040?gmo@@0o_kn3@3oool00003
0000003oool0oooo0080JFUY00<0RXZ:09NGU`2ZZZX0o`2ZZZY00:ZZZP04
09NGU`2:RXX0JFUY06UYJ@80oooo00<000000?knoP3no_h0103no_h?0?oo
o`00o`00001?000001D0oooo0000\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{355, 227},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell[TextData[{
  "Now let us write to the notebook. ",
  StyleBox["NotebookWrite", "MR"],
  " takes two arguments: the first argument is the notebook object that we \
are writing to; the second argument is what we are writing. We will create a \
few different examples below."
}], "Text",
  CellTags->{"i:10", "b:43.1", "IPM3.10.2.2"}],

Cell[TextData[{
  "A ",
  StyleBox["Cell", "MR"],
  " is an expression with two arguments. The first argument is the contents \
of the cell; the second argument is the cell style, a listing of which is \
under the ",
  StyleBox["Format\[RightTriangle]Style", "SR"],
  " menu in the front end."
}], "Text",
  CellTags->{"i:11", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(NotebookWrite[nb, 
      Cell["\<Here is some text.\>", "\<Text\>"]]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005S0000hb000`400?l0
0000o`00003oo`3ooomT0?ooo`00o`3ooomT0?ooo`003P3ooooo0?knoTL0
o_kn3`3oool000`0oooo103no_ko0?gmoD<0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgmB@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo3d0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k?`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinD00nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_Ho0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clcd0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hm0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj3d0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4m0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH>`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllk0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aC/0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^c/0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a>`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYS/0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhk09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXm09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000004l000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?clB`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[ZdX0Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFU906UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj6`2OWil00`2m_Kd0k>c/0>OWi`0T0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K6`3:b/X0
0`3Kfm/0Xj>S0?_kn`0T0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo6`3@d=000`3oool0UYJF0?_kn`0T0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3ooolK0<o?c`03
0?ooo`2FUYH0no_k02@0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM400`3@d=070=7Ad@80d=3@0P3AdM430=3@
d0L0dM7A00<0oooo09JFUP3jn_X0903jn_X00`2FUYH0oooo0<k>cP040<k>
cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JFUP3Ph>00403Ph>00501Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0
h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3Ph01YJFT0WYjN0>3Ph02>
SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0
103>c/h303hn?PD0c/k>00@0oooo09JFUP2FUYH0oooo203Fe]H20=OGe`07
09FEU@3Fe]H0UIFE0=KFeP2EUID0e]KF09FEU@050=KFeP80emOG0`3Fe]H0
0`3oool0UYJF0?Win@0T0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n?Sh6
0<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h034a<@0B4Q80=;BdP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0
g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?oo
o`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3Lg=`6
0=gMg@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID70=gMg@80
g=cL00<0gMgM0?ooo`2FUYH09@3inOT00`2FUYH0oooo0<o?c`040<o?c`<0
?Shn1@3?cll00`3oool0UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c
/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3I
fMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV
9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103o
ool0UYJF09JFUP3oool30=cLg0H0gMgM00P0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@L0gMgM0P3Lg=`00`3MgMd0oooo09JFUP0U0?Shn003
09JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0h>3P00P0
h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00G0<C4
a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B
0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2NWYh0
3`3Ph>05000000H0h>3P00@0UYJF0?ooo`3Kfm/0fm_K0`0n?Sh80=_Kf`05
0?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0<0hn?S00T0i>CT0?oo
o`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Ti>@30>?Sh`<0i>CT
00@0hn?S0>CTi03oool0UYJF9@3gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0
?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P
01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV
9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0a<C4
04Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WY
j@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY
00D0j^[Z0>WYj@3YjNT0oooo09JFUP0U0?Gem@0309JFUP3oool0i>CT00D0
i>CT0`0n?Sh40>CTi0030?ooo`2FUYH0h>3P00d0h>3P0P3oool05@3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P01T0SXj>0000000h>3P0fMWI0>3Ph01YJFT0WYjN
0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph02][Jd0
E5AD0>3Ph01YJFT0OWin0>3Ph01YJFT0WYjN0100h>3P00<000000>3Ph03P
h>001P3Ph>000`2FUYH0oooo0>g]k@030>g]k@<0?Shn1P3]kNd01@3oool0
UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WYj@080?ooo`2EUID0oooo
09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY00D0j^[Z0>WYj@3YjNT0
oooo09JFUP0U0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]k@03
0?ooo`2FUYH0h>3P00`0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0
h>3P01T0fMWI059BDP3IfMT0h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P
0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3Bd]80
h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@303hn
?PD0m?Cd00@0oooo09JFUP2FUYH0oooo2@3]kNd01`3oool0kNg]0?ooo`3]
kNd0oooo0>g]k@3oool02`3]kNd00`3oool0UYJF0?7al@0T0?7al@0309JF
UP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2FUYH0h>3P00`0h>3P
0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jN
WP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3lo?`01@3l
o?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?oooa/0kNg]
00<0oooo09JFUP3`l?00903`l?000`2FUYH0oooo0?clo0040?clo00303hn
?P3lo?`0o?cl00D0o?cl00<0oooo09JFUP3Ph>00403Ph>00501YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph00309JFUP3oool0oooo00d0oooo0P2FUYH00`3oool0moOg0?Og
m`0I0?Ogm`030?ooo`2FUYH0kno_02@0kno_00<0UYJF0?ooo`3oool03@3o
ool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?cl
o02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF
0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XK0?ooo`030?[jnP2F
UYH0kno_02@0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF0>3P
h00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F
0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e]KD0
1@2d];@0Shn?0=[JfP3Fe]H0/;2`01/0^;Rh00<0ZJVY0<k>cP3^k^h0903^
k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0f][J0140h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFVG09jNWP030?ooo`3Ph>00
h>3P03D0h>3P4@3/k>`O0>_[jbD0inOW4@3/k>`A0>3Ph00D06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03^k^h0SHf=
00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JF
UP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool01@3>c/h00`0n
?Sh0c/k>0<k>cP040<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool0103>c/h303hn
?PD0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0?ooo`030<o?c`D0?Shn103?cll05P3o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh3
0=CDe00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn0P3K
fm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`030>c/k02FUYH0oooo0080i>CT00<0?Shn0>CTi03Ti>@00`3T
i>@06P0n?Sh0i>CT0>CTi03Ti>@0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?ooo`0<0>g]k@0F
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0
o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3GemL0
303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3J
f]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0
l?3`08j>SP050:2PX00309nOW`2PX:00X:2P00@0X:2P01H0T92@0>c/k01Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0
UYJF0=kNgP0<0=?Cd`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>CTi03MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL
0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7A
d@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3T
i>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0
emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gM
g@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3M
gMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gM
g@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF
0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3M
gMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT
0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0
g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?S
h`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03F
e]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>?Sh`3Lg=`0g=cL0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL
0=cLg03Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3@d=00clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00D0
k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J0=KFeP3@d=00clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3@d=00clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd609FEU@0I0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`050>c/k02FUYH0oooo0?Ogm`3]kNd01P3oool06P3Fe]H0dM7A
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0dM7A
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocH0oooo00D0k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03L
g=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KFeP3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT
0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z
01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0
g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CT
i03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3G
emL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM
0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7A
d@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3S
hn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gM
g@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM
0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3M
gMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl00000=P000000103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gM
g@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3oool]0?ooo`D0EEEE103oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?ooo`0305EEE@3o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CT
i03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gM
g@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@
d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo
00H0E5AD000000000000Shn?0?ooo`1hN7P20000000?0451@@3gmoL0N7Qh
0000000?3`l0kno_07QhN0000000@D510?Ogm`3oool0N7Qh0000000?3`l0
kno_00<0oooo00P0XZ:R00000011@D40`l?303lo?`0000003`l?0>o_k`<0
oooo00L0bl_;03lo?`0000003`l?0>o_k`3oool0XZ:R008000000100];Bd
0?ooo`1hN7P00000061PH01hN7P000000451@@2JVYX000000451@@3gmoL0
N7Qh0000000?3`l0kno_103oool0501hN7P000000;Bd]03oool0N7Qh0000
000?3`l0jn_[05ADE01PH600gmoO018B4P11@D40moOg07QhN0000000];Bd
0=oOg`0K6a/0d];BWP3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTm
0?ooo`0307QhN02d];@0oooo0080oooo0140XZ:R08n?S`3oool0a/K6061P
H03oool0moOg07emO@3gmoL0;2`/0>o_k`3oool0a/K6061PH03oool0moOg
07emO@030?ooo`080?_kn`1MGEd0d];B0<_;b`3Bd]80oooo0<K6aP2?Shl3
0?ooo`0I0<_;b`3Bd]80oooo0<K6aP2?Shl0gmoO01/K6`3Bd]80no_k05eM
G@3Bd]80moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0a/K6
061PH03oool0moOg07emO@030?ooo`0A0?_kn`1MGEd0d];B0?ooo`36a/H0
H61P0?ooo`3gmoL0OGem0=oOg`1bLW80no_k05eMG@3Bd]80no_k05eMG@3B
d]80XP3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTm0?ooo`0307Qh
N02d];@0oooo0080oooo00D0XZ:R08n?S`3oool0XZ:R0;Bd]0030?ooo`06
0?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd1P3oool00`3kno/0GEeM0=;BdP02
0?ooo`030::RXP000000];Bd00D0oooo00D0XZ:R0000002d];@0XZ:R061P
H0020?ooo`0=0::RXP2?Shl0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k
05ADE02d];@0XZ:R0;Bd]0060?ooo`060?_kn`1MGEd0d];B0?ooo`2RXZ80
];Bd103oool01`3;bl/0?clo08n?S`3oool0no_k05eMG@3Bd]80XP3oool0
0`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo00<0N7Qh
0;Bd]03oool00P3oool0102RXZ80Shn?0?ooo`2RXZ8300000006061PH03g
moL0;2`/0>o_k`3oool0XZ:R0`0000002`1PH600oooo0?ooo`3oool0no_k
05eMG@3Bd]80gmoO018B4P11@D40moOg00@0oooo00L0gmoO018B4P11@D40
moOg0?ooo`2RXZ80Shn?0080oooo00`0N7Qh08n?S`3gmoL0;2`/0>o_k`3k
no/0GEeM0=;BdP3kno/0E5AD0;Bd]02RXZ8300000009061PH03oool0oooo
0?ooo`3kno/0GEeM0=;BdP3oool0XZ:R00<0000000T0H61P0?ooo`36a/H0
3`l?0>o_k`3oool0no_k05eMG@3Bd]80XP3oool00`1EEED0oooo0?ooo`02
0?ooo`030>c/k03moOd0Ti>C00D0^;Rh00<0]KFe0;Rh^02h^;P0102h^;P0
5P2CTi<0k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?@3oool0
0`1hN7P00000000000030000000@08n?S`3oool0a/K608n?S`3oool0N7Qh
08^;R`1DE5@0000008n?S`3oool0a/K608n?S`3oool0N7Qh08n?S`<0oooo
00P0N7Qh00T92@3Bd]80a/K608n?S`3oool0PH610=;BdP<0oooo01T0a/K6
08n?S`3oool0PH610=;BdP3Ogml0De=C0?Ogm`3Ogml04Q8B0;2`/01DE5@0
000008n?S`3gmoL07AdM061PH03kno/0GEeM0=;BdP36a/H0Shn?0?ooo`1h
N7P0Shn?00<0oooo0140fm_K05eMG@3Bd]80oooo0<K6aP2?Shl0oooo07Qh
N02?Shl0moOg02`/;03WinL0Y:BT0?ooo`3Kfm/0GEeM0=;BdP2R0?ooo`03
05EEE@3oool0oooo0080oooo00<0k>c/0>k^kP2=SHd0302OWil05P2=SHd0
i^KV06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?@3oool00`1hN7P0
];Bd0?ooo`020?ooo`070::RXP2?Shl0oooo0?ooo`1hN7P000000451@@02
0?Ogm`0802`/;02BTY802@T90=;BdP1hN7P000000451@@3gmoL30?ooo`08
0?Ogm`0V9RH0d];B0?_kn`1DE5@0000009JFUP3Kfm/30?ooo`070?_kn`1D
E5@0000009JFUP3Kfm/0oooo0::RXP020000000@08n?S`3oool0gmoO05=C
D`2JVYX000000451@@2n_[h000000451@@3gmoL0oooo07QhN0000000@D51
0?Ogm`<0oooo00l0a/K6000000000000];Bd0?ooo`1hN7P000000451@@3c
lo<0E5AD0000002BTY804Q8B08n?S`36a/H00P0000000`2d];@0oooo0?oo
o`2O0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3Ng]h0303K
fm/05P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?@3o
ool00`1hN7P0];Bd0?ooo`020?ooo`030::RXP2?Shl0oooo03T0oooo00<0
PH610=;BdP3oool02`3oool00`21PH40d];B0?ooo`2Q0?ooo`0305EEE@3o
ool0oooo0080oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY?@3oool00`1hN7P0];Bd0?oo
o`020?ooo`030::RXP2?Shl0oooo03T0oooo00<0Z:RX0=;BdP3oool02`3o
ool00`2XZ:P0d];B0?ooo`2Q0?ooo`0305EEE@3oool0oooo0080oooo00<0
k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY?03oool01P1DE5@000000000002?Shl0oooo07QhN080
000000<0@D510?Ogm`3oool03`3oool00`3gmoL09RHV0=;BdP3G0?ooo`03
05EEE@3oool0oooo0080oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?ooo`03
05EEE@3oool0oooo0080oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?ooo`03
05EEE@3oool0oooo0080oooo00<0k>c/09JFUP3oool00P3>c/h00`0n?Sh0
c/k>0<k>cP030<k>cP0J03hn?P3>c/h0c/k>0<k>cP3oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3oool^0?ooo`@0EEEE103oool0
103/k>`0UYJF0?ooo`3?cll303hn?P<0clo?0`0n?Sh20<o?c`0F0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc40oooo00<0
EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`020=CDe0<0?Shn00@0
e=CD03hn?P0n?Sh0?Shn0`3De=@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3oool]0?ooo`D0EEEE103oool00`3/k>`0UYJF
0?ooo`030=_Kf`D0?Shn103Kfm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00@0i>CT
0`0n?Sh50>CTi00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool01@3]kNd00`0n?Sh0kNg]
0>g]k@040>g]k@0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/
k02FUYH0oooo00`0o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0
k>c/09JFUP3GemL0303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe
01L0];Bd08n?S`3Jf]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`2JVYX0o`1j
NWY907YjNP0C09ZJVP3gmoL0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000<000000?Ogm`2o_kl0o`2o_km:0;no_`0B
0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000030000003gmoL0hn?S0?l0hn?SBP3Shn<04P3gmoL0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000lO7a
0?ooo`3o0?ooo`<0oooo00<0hn?S0:FUY@1fMWH00`1fMWH0101gMgL0MWIf
07MgM`1gMgL907IfMP060:FUY@3Shn<0oooo0?ooo`3Shn<0YJFU101fMWH0
101gMgL0MWIf07MgM`1gMgL907IfMP060:FUY@3Shn<0oooo0?ooo`3[jn/0
YJFU101FEUH0101GEeL0EUIF05MGE`1GEeL905IFEP030:FUY@3[jn/0oooo
00<0oooo0180lO7a0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000>7Qh@3lo?`0no_k0P3lo?`00`3kno/0o?cl
0?clo0040?clo0030?_kn`3lo?`0o?cl0080o?cl00<0no_k0?clo03kno/0
1@3lo?`00`3kno/0o?cl0?_kn`1I0?_knhl0o?cl00<0YJFU07IfMP2=SHd0
402=SHd01P1fMWH0YJFU0?clo03lo?`0YJFU07IfMQ40SHf=00H0MWIf0:FU
Y@3lo?`0no_k0:FUY@1FEUHA05]KF`0305IFEP2UYJD0o?cl00<0o?cl0180
hN7Q0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000<000000>3Ph03gmoL0o`3gmoL30?Ogm`0307IfMP2=SHd0XZ:R0100
XZ:R00H0SHf=07IfMP3gmoL0moOg07IfMP2=SHdA0::RXP0808f=S@1fMWH0
moOg0?Ogm`1FEUH0Fe]K06m_K`1aLG4=079bLP04075aL@1_Kfl0Fe]K05IF
EP@0moOg0180h>3P0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000<000000=kNgP3alO400P3alO420?;blP030?7al@3b
l_80l_;b00@0lO7a00<0l_;b0?7al@3alO400`3alO420?;blP030?7al@3b
l_80lO7a0080l_;bjP3alO400`1fMWH0^;Rh0:^[Z`0@0:^[Z`060;Rh^01f
MWH0lO7a0?7al@1fMWH0^;Rh4@2[Zj/01@2h^;P0MWIf0?7al@3bl_80EUIF
0080Kfm_00<0LW9b07MgM`1gMgL02`1gMgL0101bLW80Kfm_06m_K`1FEUH3
0?7al@0C0?;blP3Ng]h0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000D000000=cLg03[jn/0jn_[0>c/k0020>_[j`03
0:NWY`2RXZ80XZ:R0080XJ6Q00<0X:2P09nOW`2MWId00P2OWil20:2PX080
XJ6Q00<0YjNW0>_[j`3[jn/01@3[jn/01@3De=@0aLG50<G5a@35aLD0emOG
0080jn_[00<0j^[Z0<S8b03@d=000P3[jn/01@3Jf]X0a/K60>?Sh`3[jn/0
h>3P0080aLG500L0c<c<0>WYj@3>c/h0alO70>WYj@3[jn/0h>3P0080aLG5
00L0c<c<0>[ZjP3De=@0aLG50=gMg@3[jn/0h>3P0080aLG500H0a/K60>?S
h`3[jn/0jn_[0>[ZjP38b<P30<G5a@030<K6aP3Shn<0jn_[00@0jn_[00<0
j^[Z0<S8b035aLD00`35aLD00`36a/H0hn?S0>_[j`0K0>_[je/0k>c/=03[
jn/0101fMWH0^k^k0;6a/@2a/K480?ooo`L0/K6a00H0^k^k07IfMP3[jn/0
jn_[07IfMP2k^k/20;6a/@`0oooo0`2a/K402@2k^k/0MWIf0>_[j`3[jn/0
EUIF07MgM`1bLW80M7Ad07MgM`0;07QhN00507MgM`1dM7@0LW9b07MgM`1F
EUH0103[jn/04P3Lg=`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo00000`000000f][J0>GUi@030>GUi@0308Z:RP2KVi/0
[:b/00<0jNWY00X0j^[Z0<C4a025QHD0k>c/0>g]k@3^k^h0inOW0?3`l03b
l_80Xj>S1P3UiND04@1XJ6P0410@010@400@4100P8200<K6aP3?cll0g]kN
02HV9P1ADE40clo?0>GUi@2DU9@05ADE0:BTY03Ng]h0/[:b0080410@00L0
@D510=_Kf`1@D5006ATI0;Zj^P3Shn<0/[:b0080410@00L0@D510=oOg`1`
L700410@09:BTP3Ti>@0[jn_0080410@00H05ADE0;Zj^P3<c<`0i>CT0=gM
g@0T92@3010@400301DE5@2TY:@0g]kN00@0iNGU00<0g]kN02HV9P0@4100
0`0@41000`0E5AD0_[jn0>GUi@2[0>GUi@0507IfMP30`<00]KFe0;Fe]@3o
ool01P1bLW800`0W9bL0]KFe0;Fe]@050;Fe]@060<30`01fMWH0iNGU0>GU
i@1fMWH0`<300P2e]KD00`3oool09bLW02LW9`0902LW9`<0]KFe00D0`<30
07IfMP3UiND0iNGU05IFEP0207MgM`0407UiN@1jNWX0oooo0?ooo`L0NWYj
0P3oool01@1jNWX0NGUi07MgM`1gMgL0EUIF00@0iNGU0180f][J0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<00000
0=WIf@3Ogml00`3Ogml01@3Ng]h0X:2P0>[ZjP3YjNT0iNGU0080j^[Z00X0
d=3@07AdM03]kNd0k^k^0:f][@3@d=00l_;b0??cl`2SXj<0gmoO0P3Ng]h0
=03Ogml0g]kN06a/K00E5AD0Zj^[0=kNgP33`l<071`L05ADE036a/H0e]KF
02DU9@1ADE40cLg=0=kNgP2?Shl05ADE0:BTY03EeMD0CTi>03Ph>034a<@0
gMgM0=_Kf`1>CTh06ATI0;Zj^P3IfMT0CTi>03Ph>034a<@0gMgM0=OGe`0U
9BD0>3Ph0<C4a032`/806a/K055AD@3IfMT0`l?304a<C03Hf=P0emOG02DU
9@0I6AT0aLG50=kNgP2?Shl05ADE0:BTY03IfMT50=kNgP050:f][@0@4100
6ATI0;Vi^@3AdM40[P3Ng]h01@1fMWH0`l?30;^k^`2k^k/0oooo00H0LW9b
00<09bLW0;^k^`2k^k/01@2k^k/01P33`l<0MWIf0=kNgP3Ng]h0MWIf0<?3
``80^k^k00<0oooo02LW9`2k^k/01`2k^k/00`3oool09bLW0;^k^`020;^k
^`090<?3``1fMWH0gmoO0=kNgP1FEUH0MgMg07QhN01kNg/0<30`00<0oooo
1@1mOGd30?ooo`05030`<01kNg/0N7Qh07MgM`1FEUH00P3Ng]h0503Ogml0
g]kN0=WIf@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000040000003GemL0f=SH0=WIf@80f=SH0100T92@08n?S`2>SXh0
jNWY0861P@1ADE40QhN70=[JfP2VYZH0ZJVY0:b/[01OGel0l_;b0??cl`3d
m?@0Y:BT0`3Hf=P01@3IfMT0d];B02@T901ADE40bl_;0080f=SH0200JVYZ
01TI6@2j^[X0d=3@02@T901ADE40bl_;0=SHf02;Rh/05ADE0:BTY03=cLd0
92@T03Ph>033`l<0emOG0=GEe@1<C4`06ATI0;Zj^P3@d=0092@T03Ph>033
`l<0emOG0=7Ad@0T92@0>3Ph0<?3``2:RXX05ADE0:BTY0@0aLG500X0WYjN
010@401aLG40dM7A0=SHf02;Rh/05ADE0:BTY03De=@0d];B0P35aLD02036
a/H0emOG0=SHf01ZJVX0410@079bLP35aLD0e=CD7P3Hf=QK0=WIfC@0f=SH
00D0MWIf0<O7a`2o_kl0_kno0?ooo`0702LW9`L0_kno00H0alO707IfMP3H
f=P0f=SH07IfMP37alL20;no_`030?ooo`0W9bL0_kno00L0_kno00<0oooo
02LW9`2o_kl00P2o_kl01@37alL0MWIf0=SHf03Hf=P0EUIF0080Ng]k00<0
OWin07moO`0X:2P00`3oool307moO`<0oooo00<0:2PX07moO`1nOWh00P1k
Ng/00`1FEUH0fMWI0=SHf0020=SHf00B0=OGe`000000VYZJ0:2PX02XZ:P0
/[:b0;bl_036a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003EeMD0dm?C00<0
dm?C00<0QHF50:>SX`3Vi^H00P3Zj^X02`0e=CD0Ogmo0<o?c`2n_[h0Rh^;
05iNGP1HF5P0J6QX0:>SX`39bLT0ZZZZ00@0dm?C00@0c<c<02@T901ADE40
b/[:0P3Cdm<07P1WIfL06ATI0;Vi^@3;bl/092@T055AD@3:b/X0dm?C08R8
R00E5AD0Y:BT0<W9b@0T92@0>3Ph0<71`@3Cdm<0clo?04Y:BP0I6AT0^KVi
0<_;b`0T92@0>3Ph0<71`@3Cdm<0c<c<02@T900h>3P0`L7106MWI`D0410@
00X0DE5A09fMW@0@4100SXj>0<c<c03Cdm<0R8R801DE5@2TY:@0XJ6Q0P0@
41002@0J6QX0a/K60=?Cd`3Cdm<0c<c<02@T900E5AD0Y:BT0<O7a`2]0=?C
d`0307IfMP3;bl/0a/K60100a/K600H0bl_;07IfMP3Cdm<0dm?C07IfMP3;
bl/20<K6aP030?ooo`0W9bL0a/K600L0a/K600<0oooo02LW9`36a/H00P36
a/H01`3;bl/0MWIf0=?Cd`3Cdm<0EUIF07emO@1oOgl00`23Ph<01@0X:2P0
oooo0?ooo`3oool0Ph>300<0oooo00L0:2PX08>3P`23Ph<0Ph>307moO`1m
OGd0EUIF00@0dm?C0140eMGE0000002PX:00YJFU0:f][@2f][H0_kno0<S8
b03Bd]80f][J0>;RhP3YjNT0kno_0?Cdm03fm_H0nOWi0?_kn`030?gmo@40
o_kn0@3oool10?ooo`0000D000000=?Cd`3>c/h0c/k>0<o?c`020<k>cP0?
08N7Q`2CTi<0dm?C0=_Kf`2][Jd0`L710;2`/02YZJT0ZZZZ08f=S@2ATI40
/;2`0<_;b`3_knl0YJFU00<0c/k>00D0clo?0<O7a`0S8b<0DE5A0<S8b002
0<k>cP0]06EUI@0I6AT0^KVi0<O7a`0S8b<0DE5A0<O7a`3>c/h0QHF501DE
5@2TY:@0alO702<S8`0h>3P0`L710<k>cP3;bl/0BDU901TI6@2i^KT0alO7
02<S8`0h>3P0`L710<k>cP37alL08b<S03Ph>031`L40QHF501DE5@2TY:@0
b/[:0<O7a`0S8b<0DE5A0;6a/@0J6QX0LW9b0<G5a@3>c/h0QHF501DE5@2T
Y:@0cLg=00L0c/k>00D0X:2P010@400i>CT0_kno0<_;b`050<k>cP050<g=
c@37alL0cLg=0<O7a`3:b/X04`3>c/iK0<o?cc@0c/k>00<0MWIf0<k>cP3;
bl/0403;bl/01P3>c/h0MWIf0<k>cP3>c/h0MWIf0<k>cP80bl_;00<0oooo
02LW9`3;bl/01`3;bl/00`3oool09bLW0<_;b`020<_;b`090<k>cP1fMWH0
clo?0<k>cP1FEUH0OGem08:2PP25QHD0Q8B40080QHF500<0:2PX0?ooo`3o
ool00`3oool0200X:2P0QHF508F5Q@24Q8@0QHF508:2PP1mOGd0EUIF103>
c/h04@3Cdm<000000:RXZ02][Jd0/k>c0;^k^`33`l<0c<c<0=GEe@3MgMd0
i>CT0>_[j`3`l?00m?Cd0?Ogm`3inOT0o?cl00<0oOgm0@3no_h10?ooo`40
oooo00000`000000d=3@0<W9b@030<W9b@0A07=cL`2DU9@0OWin07UiN@2P
X:00dM7A0:f][@2UYJD0YZJV08F5Q@1NGUh0[Jf]0>GUi@3fm_H0n?Sh0:BT
Y039bLT00`38b<P01032`/808b<S055AD@37alL20<S8b00/069RHP0I6AT0
^KVi0<;2`P0R8R80>3Ph0<71`@32`/80ATI601TI6@2l_;`0`L7102<S8`0h
>3P0_kno0<G5a@34a<@0ATI601TI6@2i^KT0`L7102<S8`0h>3P0_kno0<G5
a@30`<008R8R03Ph>030`<00[jn_01XJ6P1cLg<0aLG509ZJVP0@4100SXj>
0<G5a@16ATH06ATI0;Vi^@35aLD0Ogmo01DE5@2TY:@90<S8b004069RHP0@
4100SHf=0<O7a`@0b<S800H0/k>c04E5A@2QXJ40C4a<06i^KP37alLC0<S8
b5/0bLW9=038b<P00`1fMWH0dm?C0<g=c@0@0<g=c@060=?Cd`1fMWH0b<S8
0<S8b01fMWH0dm?C0P3=cLd00`3oool09bLW0<g=c@070<g=c@030?ooo`0W
9bL0cLg=0080cLg=00L0dm?C07IfMP38b<P0b<S805IFEP22PX80Ph>300D0
QhN700D0:2PX0?ooo`3oool0oooo02PX:00508N7Q`0308>3P`22PX80EUIF
00<0bLW90180b<S80=3@d0000000/[:b0;Jf]P2k^k/0`/;20<W9b@3AdM40
fMWI0>3Ph03WinL0kNg]0?7al@3emOD0n?Sh0?[jnP3lo?`30?gmo@40o_kn
0@3oool10?ooo`0000<000000<o?c`35aLD01035aLD03`2GUiL0]kNg0>c/
k02i^KT0Rh^;0;Ng]`2ZZZX0Y:BT07inOP1aLG40a<C40=WIf@3fm_H0nOWi
0:FUY@040<G5a@040;no_`0R8R80DE5A0<K6aP80aLG500H0HF5Q01TI6@2i
^KT0_kno028R8P1MGEd2010@400401DE5@2TY:@0`/;204E5A@<0410@00H0
DE5A065QH@0@41006ATI0;Jf]P15ADD3010@4008055AD@2o_kl08R8R03Ph
>02o_kl0aLG50:b/[00J6QX2010@4005079bLP35aLD0aLG50<;2`P15ADD0
0`0@41000`0E5AD0Y:BT0<K6aP080<G5a@030:b/[00J6QX0DE5A00D0aLG5
00D0_kno028R8P0`<3006QXJ08j>SP2S0<G5a@0307IfMP3EeMD0dM7A0100
dM7A00H0eMGE07IfMP35aLD0aLG507IfMP3EeMD20=7Ad@030?ooo`0W9bL0
dM7A00L0dM7A00<0oooo02LW9`3AdM400P3AdM401`3EeMD0MWIf0<G5a@35
aLD0EUIF08:2PP27QhL01@29RHT50?ooo`D0RHV900<0QhN708:2PP1FEUH0
1035aLD04@3?cll000000;bl_02o_kl0`l?30<W9b@3@d=00emOG0=gMg@3T
i>@0j^[Z0>o_k`3clo<0m_Kf0?Shn03kno/0o?cl0080oOgm0P3no_h20?oo
o`0000<000000<g=c@31`L400`31`L40401eMGD0VIVI0:NWY`3]kNd0bLW9
07=cL`2`/;00SHf=07=cL`1<C4`0HV9R0;jn_P2e]KD0/[:b0:f][@2YZJT4
0<71`@040;^k^`0R8R80DE5A0<C4a080`L7100@0GemO01TI6@2i^KT0`/;2
1`31`L401@2k^k/08R8R03Ph>02n_[h0`/;20080`L7100`0`l?30<G5a@2l
_;`08R8R03Ph>02n_[h0`/;20;^k^`0R8R80>3Ph0;jn_P32`/8;0<71`@04
07alO00E5AD0Y:BT0<?3``H0`L710P34a<@0102n_[h0A4A403Ph>02n_[h4
0<71`@0705mOG`0@4100410@010@400h>3P0_[jn0<;2`P2Q0<71`@0307If
MP3Hf=P0eMGE0100eMGE00H0f=SH07IfMP31`L40`L7107IfMP3Hf=P20=GE
e@030?ooo`0W9bL0oooo00P0oooo00H09bLW0=GEe@3EeMD0eMGE0=SHf01f
MWH20<71`@0305IFEP24Q8@0R8R800@0RXZ:0`3oool0100X:2P0oooo0?oo
o`3oool408Z:RP0308R8R024Q8@0EUIF00@0`L710100cLg=00000036a/H0
b<S80<c<c03AdM40emOG0=cLg03Rh^80j>SX0>g]k@3alO40mOGe0?Ogm`3i
nOT0o?cl0`3moOd20?knoP80oooo00000`000000c<c<0;Zj^P020;fm_@0A
0;jn_P2m_Kd0WYjN0>?Sh`2f][H0UYJF08:2PP2CTi<0Q8B408:2PP1>CTh0
;2`/09bLW02[Zj/0m_Kf0?Shn02WYjL0102m_Kd0102g]kL08B4Q055AD@34
a<@20;fm_@0405eMG@0I6AT0^KVi0;no_`P0_Kfm00<0GEeM03Lg=`2h^;P0
0P2m_Kd02`1MGEd06ATI0;Vi^@2o_kl0GEeM03Lg=`2h^;P0_Kfm0;Ng]`0Q
8B40>3Ph0080_[jn2`2m_Kd0101jNWX05ADE0:BTY02o_kl40;fm_@070;Ng
]`0Q8B40Kfm_0<G5a@2o_kl08b<S071`L0050;fm_@0607YjNP0d=3@0?3`l
05eMG@0h>3P0^;Rh4`2m_KeK0;jn_S@0_Kfm00<0MWIf0=[JfP3EeMD0403E
eMD01P3Jf]X0MWIf0;fm_@2m_Kd0MWIf0=[JfP80eMGE00<0oooo079bLP1b
LW80201bLW803P0W9bL0eMGE0=GEe@3EeMD0f][J07IfMP2m_Kd0_[jn05IF
EP27QhL0RXZ:08f=S@2>SXh0SHf=0`3oool00`0X:2P0SHf=02PX:0030?oo
o`<0SHf=00@0RXZ:08N7Q`1FEUH0_[jn0P2m_Kd04@2j^[X0c<c<0000003@
d=00d];B0=GEe@3IfMT0gMgM0>;RhP3WinL0k>c/0?3`l03dm?@0m_Kf0?Sh
n03jn_X0o?cl00<0oOgm00<0o_kn0?ooo`3oool00@3oool000030000003:
b/X0^;Rh00<0^[Zj0100O7al08f=S@1hN7P0VIVI0?3`l032`/80YjNW0<_;
b`2l_;`0Xj>S06e]K@1mOGd0[:b/0?_kn`3lo?`0YZJV0`2j^[X0102k^k/0
];Bd024Q8@1=CDd30;Zj^P0305]KF`0I6AT0[jn_00h0^[Zj00<0Fe]K01TI
6@2_[jl01@2j^[X0102d];@08B4Q03Lg=`2d];@<0;Zj^P0307MgM`0D51@0
Vi^K00H0^[Zj00D0TI6A010@400@4100410@04e=C@070;Zj^P030;Bd]00Q
8B40Jf][0:@0^[Zj00<0MWIf0=SHf03Jf]X0403Jf]X01P3Hf=P0MWIf0;Zj
^P2j^[X0MWIf0=SHf080f][J00<0oooo079bLP1bLW80201bLW803@0W9bL0
f][J0=[JfP3Jf]X0f=SH07IfMP2j^[X0^k^k05IFEP2;Rh/0Shn?08j>SP2?
Shl00`3oool01@0X:2P0Shn?08n?S`2?Shl0:2PX00<0oooo00D0Shn?08j>
SP2?Shl0Rh^;05IFEP030;Zj^P0@0;Rh^03:b/X000000=WIf@3Jf]X0gMgM
0>3Ph03Ti>@0j>SX0>c/k03_knl0lo?c0?KfmP3hn?P0n_[j0?clo0<0oOgm
0P3no_h30?ooo`0000<000000<W9b@2h^;P00`2h^;P04026QXH0WYjN0=CD
e03`l?00l_;b0<k>cP31`L40gMgM0=;BdP2RXZ80c<c<0<W9b@29RHT0o?cl
0?gmo@2WYjO_0;Rh^00307IfMP3GemL0gMgM0100gMgM00H0emOG07IfMP2h
^;P0^;Rh07IfMP3GemL30=gMg@/09bLW0`3MgMd02@3GemL0MWIf0;Rh^02h
^;P0EUIF092@T02ATI40Ti>C09BDU0020?ooo`0302PX:02HV9P0V9RH00<0
V9RH00P0:2PX0?ooo`3oool0U9BD09>CT`2ATI40T92@05IFEP@0^;Rh00l0
bLW90000003QhN40h^;R0>CTi03WinL0j^[Z0>g]k@3`l?00lo?c0?Gem@3g
moL0nOWi0?_kn`3lo?`00`3moOd00`3no_h0oooo0?ooo`020?ooo`0000<0
00000<W9b@2h^;P0102h^;P03`26QXH0U9BD0?;blP3clo<0X:2P0>SXj03f
m_H0fm_K09VIV@3jn_X0no_k0=cLg02f][H0o_kn0:NWY`3_0;Rh^00307If
MP3Fe]H0hN7Q0100hN7Q00H0e]KF07IfMP2h^;P0^;Rh07IfMP3Fe]HA0>7Q
h@090=KFeP1fMWH0^;Rh0;Rh^01FEUH0UIFE09NGU`2LW9`0XZ:R0080:2PX
00D0Zj^[0:f][@2[Zj/0Zj^[0:f][@020:^[Z`80:2PX00D0XZ:R09bLW02G
UiL0UIFE05IFEP040;Rh^00>0<W9b@000000j>SX0>WYj@3[jn/0kNg]0>o_
k`3alO40m?Cd0?KfmP3gmoL0nOWi0?_kn`3lo?`30?gmo@80o_kn103oool0
000300000039bLT0]KFe00<0]KFe00d0MgMg0:2PX03Ph>00lo?c0>GUi@3/
k>`0m_Kf0?Shn03inOT0T92@0?_kn`3lo?`0oOgm0080o_kn00<0YjNW0;Fe
]@2e]KD0k@2e]KD00`1fMWH0kNg]0>WYj@0@0>WYj@060>g]k@1fMWH0]KFe
0;Fe]@1fMWH0kNg]4@3YjNT02P3]kNd0MWIf0;Fe]@2e]KD0EUIF09^KV`2N
WYh0YJFU0;:b/P2j^[X20;no_`040<30`02o_kl0_kno0<30`080_kno00L0
`<300;Zj^P2b/[80YJFU09jNWP2KVi/0EUIF00@0]KFe00d0bLW90000003^
k^h0kno_0?3`l03alO40lo?c0?Gem@3fm_H0n?Sh0?Win@3kno/0o?cl00<0
oOgm0P3no_h50?ooo`0000<000000<W9b@2j^[X0102a/K400`2=SHd0ZZZZ
0:ZZZP050:ZZZP030:NWY`2ZZZX0ZZZZ00@0ZZZZk`2a/K400`1fMWH0e=CD
0?;blP0@0?;blP060=CDe01fMWH0/K6a0;6a/@1fMWH0e=CD4@3bl_801P3D
e=@0MWIf0;6a/@2a/K40EUIF0:>SX`80Z:RX3@33`l<20:RXZ0030:>SX`1F
EUH0/K6a0080/K6a00@0^[Zj0<W9b@000000lo?c0P3dm?@01`3emOD0m_Kf
0?Ogm`3hn?P0n_[j0?_kn`3lo?`0103moOd00`3no_h0oooo0?ooo`040?oo
o`0000<000000<?3``39bLT0o`2^[Zh30:j^[P0408f=S@1fMWH0c/k>0?_k
n`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^[Zh0102=SHd0MWIf0<k>cP3k
no/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00H0VIVI05IFEP2e]KD0
bl_;0=oOg`3Xj>P90>OWi`060>SXj03Ogml0bl_;0;Fe]@1FEUH0VIVI0`2^
[Zh00`39bLT0`l?3000000020?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X0
0P3lo?`40?gmo@80o_kn1P3oool0000400000033`l<0e=CD0;Ng]ol0YjNW
0`2WYjL00`2=SHd0MWIf07IfMP0?07IfMP0308f=S@2WYjL0YjNW0080YjNW
00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=
S@1FEUH0EUIF00l0EUIF00P0SHf=0:NWY`2WYjL0YjNW0;Ng]`3De=@0`l?3
000000<0nOWi00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo0000
1P000000ZZZZ0>OWi`3De=@0_[jn0;>c/ol0ZZZZ@`2ZZZX01P2c/k<0_[jn
0=CDe03WinL0ZZZZ00000080no_k0`3lo?`50?gmo@<0o_kn203oool00007
0000002:RXX0fMWI0?clo03Ti>@0d];B0<c<c03o0;jn_T40_[jn00P0c<c<
0=;BdP3Ti>@0o?cl0=WIf@2:RXX000000?clo0P0oOgm0P3no_h:0?ooo`00
00P0000006UYJ@2GUiL0fMWI0?clo03bl_80k>c/0>?Shol0fMWI?`3IfMT0
203Shn<0k>c/0?;blP3lo?`0fMWI09NGU`1YJFT000001`3moOd30?knoP/0
oooo00001@000000oooo06UYJ@2GUiL0dm?C0080o?clo`3ooom10?ooo`80
o?cl00D0dm?C09NGU`1YJFT0oooo000000040?gmo@@0o_kn3@3oool00003
0000003oool0oooo0080JFUY00<0RXZ:09NGU`2ZZZX0o`2ZZZY00:ZZZP04
09NGU`2:RXX0JFUY06UYJ@80oooo00<000000?knoP3no_h0103no_h?0?oo
o`00o`00001?000001D0oooo0000\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{355, 227},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell[TextData[{
  "Adding options to ",
  StyleBox["Cell", "MR"],
  " allows us to change some of the properties of the cell. For example, here \
are several of the options that you can add."
}], "Text",
  PageBreakAbove->True,
  CellTags->{"i:12", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Take[Options[Cell], {10, 15}]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[BoxData[
    \({Deletable \[Rule] True, PageWidth \[Rule] WindowWidth, 
      Visible \[Rule] True, CellFrame \[Rule] False, 
      CellDingbat \[Rule] None, ShowCellBracket \[Rule] True}\)], "Output",
  CellLabel->"Out[8]=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(NotebookWrite[
      nb, \[IndentingNewLine]Cell["\<Here is some more text.\>", "\<Text\>", \
\[IndentingNewLine]CellFrame \[Rule] True, 
        CellDingbat \[Rule] \[HappySmiley]]]\)], "Input",
  CellLabel->"In[9]:=",
  CellDingbat->None,
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005S0000hb000`400?l0
0000o`00003oo`3ooomT0?ooo`00o`3ooomT0?ooo`003P3ooooo0?knoTL0
o_kn3`3oool000`0oooo103no_ko0?gmoD<0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgmB@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo3d0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k?`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinD00nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_Ho0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clcd0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hm0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj3d0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4m0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH>`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllk0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aC/0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^c/0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a>`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYS/0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhk09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXm09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000004l000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?clB`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[ZdX0Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFU906UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj6`2OWil00`2m_Kd0k>c/0>OWi`0T0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K6`3:b/X0
0`3Kfm/0Xj>S0?_kn`0T0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo6`3@d=000`3oool0UYJF0?_kn`0T0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3ooolK0<o?c`03
0?ooo`2FUYH0no_k02@0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM400`3@d=070=7Ad@80d=3@0P3AdM430=3@
d0L0dM7A00<0oooo09JFUP3jn_X0903jn_X00`2FUYH0oooo0<k>cP040<k>
cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JFUP3Ph>00403Ph>00501Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0
h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3Ph01YJFT0WYjN0>3Ph02>
SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0
103>c/h303hn?PD0c/k>00@0oooo09JFUP2FUYH0oooo203Fe]H20=OGe`07
09FEU@3Fe]H0UIFE0=KFeP2EUID0e]KF09FEU@050=KFeP80emOG0`3Fe]H0
0`3oool0UYJF0?Win@0T0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n?Sh6
0<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h034a<@0B4Q80=;BdP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0
g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?oo
o`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3Lg=`6
0=gMg@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID70=gMg@80
g=cL00<0gMgM0?ooo`2FUYH09@3inOT00`2FUYH0oooo0<o?c`040<o?c`<0
?Shn1@3?cll00`3oool0UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c
/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3I
fMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV
9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103o
ool0UYJF09JFUP3oool30=cLg0H0gMgM00P0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@L0gMgM0P3Lg=`00`3MgMd0oooo09JFUP0U0?Shn003
09JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0h>3P00P0
h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00G0<C4
a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B
0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2NWYh0
3`3Ph>05000000H0h>3P00@0UYJF0?ooo`3Kfm/0fm_K0`0n?Sh80=_Kf`05
0?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0<0hn?S00T0i>CT0?oo
o`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Ti>@30>?Sh`<0i>CT
00@0hn?S0>CTi03oool0UYJF9@3gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0
?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P
01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV
9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0a<C4
04Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WY
j@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY
00D0j^[Z0>WYj@3YjNT0oooo09JFUP0U0?Gem@0309JFUP3oool0i>CT00D0
i>CT0`0n?Sh40>CTi0030?ooo`2FUYH0h>3P00d0h>3P0P3oool05@3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P01T0SXj>0000000h>3P0fMWI0>3Ph01YJFT0WYjN
0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph02][Jd0
E5AD0>3Ph01YJFT0OWin0>3Ph01YJFT0WYjN0100h>3P00<000000>3Ph03P
h>001P3Ph>000`2FUYH0oooo0>g]k@030>g]k@<0?Shn1P3]kNd01@3oool0
UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WYj@080?ooo`2EUID0oooo
09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY00D0j^[Z0>WYj@3YjNT0
oooo09JFUP0U0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]k@03
0?ooo`2FUYH0h>3P00`0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0
h>3P01T0fMWI059BDP3IfMT0h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P
0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3Bd]80
h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@303hn
?PD0m?Cd00@0oooo09JFUP2FUYH0oooo2@3]kNd01`3oool0kNg]0?ooo`3]
kNd0oooo0>g]k@3oool02`3]kNd00`3oool0UYJF0?7al@0T0?7al@0309JF
UP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2FUYH0h>3P00`0h>3P
0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jN
WP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3lo?`01@3l
o?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?oooa/0kNg]
00<0oooo09JFUP3`l?00903`l?000`2FUYH0oooo0?clo0040?clo00303hn
?P3lo?`0o?cl00D0o?cl00<0oooo09JFUP3Ph>00403Ph>00501YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph00309JFUP3oool0oooo00d0oooo0P2FUYH00`3oool0moOg0?Og
m`0I0?Ogm`030?ooo`2FUYH0kno_02@0kno_00<0UYJF0?ooo`3oool03@3o
ool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?cl
o02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF
0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XK0?ooo`030?[jnP2F
UYH0kno_02@0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF0>3P
h00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F
0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e]KD0
1@2d];@0Shn?0=[JfP3Fe]H0/;2`01/0^;Rh00<0ZJVY0<k>cP3^k^h0903^
k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0f][J0140h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFVG09jNWP030?ooo`3Ph>00
h>3P03D0h>3P4@3/k>`O0>_[jbD0inOW4@3/k>`A0>3Ph00D06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03^k^h0SHf=
00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JF
UP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool01@3>c/h00`0n
?Sh0c/k>0<k>cP040<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool0103>c/h303hn
?PD0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0?ooo`030<o?c`D0?Shn103?cll05P3o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh3
0=CDe00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn0P3K
fm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`030>c/k02FUYH0oooo0080i>CT00<0?Shn0>CTi03Ti>@00`3T
i>@06P0n?Sh0i>CT0>CTi03Ti>@0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?ooo`0<0>g]k@0F
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0
o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3GemL0
303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3J
f]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0
l?3`08j>SP050:2PX00309nOW`2PX:00X:2P00@0X:2P01H0T92@0>c/k01Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0
UYJF0=kNgP0<0=?Cd`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>CTi03MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL
0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7A
d@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3T
i>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0
emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gM
g@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM
0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl00000=P000000103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3M
gMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M
0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03ooooT
000000`0oooo1@1EEED40?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Z
j^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0
gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03F
e]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0g=cL0=cLg03Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTa0?ooo`050::R
XP000000000000T92@3Bd]801P3oool00`000000oooo0?ooo`050?ooo`06
05ADE0000000000008n?S`3oool0N7Qh0P0000003`11@D40moOg07QhN000
00003`l?0>o_k`1hN7P000000451@@3gmoL0oooo07QhN00000003`l?0>o_
k`030?ooo`080::RXP000000@D510<?3``0o?cl0000000l?3`3_knl30?oo
o`070<_;b`0o?cl0000000l?3`3_knl0oooo0::RXP020000000@0;Bd]03o
ool0N7Qh0000001PH600N7Qh00000011@D40VYZJ00000011@D40moOg07Qh
N00000003`l?0>o_k`<0oooo00/0N7Qh0000001PH600N7Qh00000011@D40
VYZJ00000011@D40moOg0::RXP020000000;0;Bd]03oool0N7Qh00000011
@D40moOg0?ooo`1hN7P0000000l?3`3_knl0103oool0501hN7P000000;Bd
]03oool0N7Qh0000000?3`l0jn_[05ADE01PH600gmoO018B4P11@D40moOg
07QhN0000000];Bd0=oOg`0K6a/0d];BJP3oool00`000000oooo0?ooo`0>
0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]
kNd609FEU@0I0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY<03oool01P3kno/0GEeM0=;BdP3oool0XZ:R08n?
S`H0oooo00<000000?ooo`3oool01P3oool00`1hN7P0];Bd0?ooo`020?oo
o`0A0::RXP2?Shl0oooo0<K6aP1PH600oooo0?Ogm`1mOGd0moOg02`/;03_
knl0oooo0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool0203kno/0GEeM0=;B
dP3;bl/0d];B0?ooo`36a/H0Shn?0`3oool06@3;bl/0d];B0?ooo`36a/H0
Shn?0=oOg`0K6a/0d];B0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?_kn`1M
GEd0d];B0?_kn`1DE5@0];Bd0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool0
603gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]03Ogml06a/K
0=;BdP3kno/0GEeM0=;BdP3gmoL0;2`/0>o_k`3oool0a/K6061PH03oool0
moOg07emO@<0oooo0140no_k05eMG@3Bd]80oooo0<K6aP1PH600oooo0?Og
m`1mOGd0gmoO079bLP3kno/0GEeM0=;BdP3kno/0GEeM0=;BdP1^0?ooo`03
0000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0
UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT`0?ooo`060?Ogm`0/
;2`0TY:B00l?3`2f][H0H61P1P3oool00`000000oooo0?ooo`060?ooo`03
07QhN02d];@0oooo0080oooo00D0XZ:R08n?S`3oool0XZ:R0;Bd]0030?oo
o`060?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd1P3oool00`3kno/0GEeM0=;B
dP020?ooo`030::RXP000000];Bd00D0oooo00D0XZ:R0000002d];@0XZ:R
061PH0020?ooo`0=0::RXP2?Shl0moOg02`/;03_knl0no_k05eMG@3Bd]80
no_k05ADE02d];@0XZ:R0;Bd]0060?ooo`0;0?Ogm`0/;2`0kno_0?_kn`1M
GEd0d];B0?_kn`1DE5@0];Bd0::RXP1PH6000P3oool0202RXZ80Shn?0?Og
m`0/;2`0kno_0?ooo`2RXZ80];Bd1P3oool01P3kno/0GEeM0=;BdP3oool0
XZ:R0;Bd]0@0oooo00L0bl_;03lo?`2?Shl0oooo0?_kn`1MGEd0d];B06h0
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3@d=00clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC00oooo00H0moOg
01dM7@1PH600moOg01dM7@1PH6060?ooo`030000003oool0oooo00H0oooo
00<0N7Qh0;Bd]03oool00P3oool0102RXZ80Shn?0?ooo`2RXZ8300000006
061PH03gmoL0;2`/0>o_k`3oool0XZ:R0`0000002`1PH600oooo0?ooo`3o
ool0no_k05eMG@3Bd]80gmoO018B4P11@D40moOg00@0oooo00L0gmoO018B
4P11@D40moOg0?ooo`2RXZ80Shn?0080oooo00`0N7Qh08n?S`3gmoL0;2`/
0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]02RXZ830000000?061PH03o
ool0oooo0?ooo`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd
]02RXZ80Shn?0080oooo00L0N7Qh08n?S`3gmoL0;2`/0>o_k`3oool0XZ:R
00<0000000T0H61P0?ooo`3oool0oooo0?_kn`1MGEd0d];B0?ooo`2RXZ80
0`0000002@1PH600oooo0<K6aP0?3`l0kno_0?ooo`3kno/0GEeM0=;BdP1^
0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0
1@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT`0?ooo`06
0?Ogm`0/;2`0kno_0?ooo`36a/H0H61P1P3oool00`000000oooo0?ooo`06
0?ooo`0307QhN0000000000000<000000100Shn?0?ooo`36a/H0Shn?0?oo
o`1hN7P0Rh^;05ADE0000000Shn?0?ooo`36a/H0Shn?0?ooo`1hN7P0Shn?
0`3oool0201hN7P02@T90=;BdP36a/H0Shn?0?ooo`21PH40d];B0`3oool0
6@36a/H0Shn?0?ooo`21PH40d];B0=oOg`1CDe<0moOg0=oOg`0B4Q80/;2`
05ADE0000000Shn?0?Ogm`0M7Ad0H61P0?_kn`1MGEd0d];B0<K6aP2?Shl0
oooo07QhN02?Shl00P3oool06@3kno/0E5AD0000002?Shl0moOg01dM7@1P
H600no_k05eMG@3Bd]80gmoO05=CD`3gmoL0gmoO018B4P2`/;00E5AD0000
002?Shl0oooo0<K6aP2?Shl0oooo07QhN02?Shl00`3oool04@3Kfm/0GEeM
0=;BdP3oool0a/K608n?S`3oool0N7Qh08n?S`3gmoL0;2`/0>OWi`2TY:@0
oooo0=_Kf`1MGEd0d];B06h0oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID0
6@3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJC00oooo00H0moOg01dM7@11@D40emOG018B4P1PH6060?ooo`03
0000003oool0oooo00H0oooo00<0N7Qh0;Bd]03oool00P3oool01`2RXZ80
Shn?0?ooo`3oool0N7Qh00000011@D400P3gmoL0200/;2`0TY:B00T92@3B
d]80N7Qh00000011@D40moOg0`3oool0203gmoL09RHV0=;BdP3kno/0E5AD
0000002FUYH0fm_K0`3oool01`3kno/0E5AD0000002FUYH0fm_K0?ooo`2R
XZ800P000000402?Shl0oooo0=oOg`1CDe<0VYZJ00000011@D40_[jn0000
0011@D40moOg0?ooo`1hN7P000000451@@3gmoL30?ooo`0;0=oOg`1CDe<0
VYZJ00000011@D40_[jn00000011@D40moOg0?ooo`2RXZ800P0000002`2?
Shl0oooo0?Ogm`0/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg00<0
oooo00l0a/K6000000000000];Bd0?ooo`1hN7P000000451@@3clo<0E5AD
0000002BTY804Q8B08n?S`36a/H00P0000000`2d];@0oooo0?ooo`1[0?oo
o`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool01@3/
k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT`0?ooo`060?_k
n`1MGEd0d];B0?ooo`2RXZ80Shn?1P3oool00`000000oooo0?ooo`060?oo
o`0307QhN02d];@0oooo0080oooo00<0XZ:R08n?S`3oool0E03oool00`21
PH40d];B0?ooo`0;0?ooo`030861P@3Bd]80oooo06d0oooo00<000000?oo
o`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]1P2EUID06@3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJC40oooo00D0XZ:R0000000000002@T9
0=;BdP060?ooo`030000003oool0oooo00H0oooo00<0N7Qh0;Bd]03oool0
0P3oool00`2RXZ80Shn?0?ooo`1D0?ooo`030:RXZ03Bd]80oooo00/0oooo
00<0Z:RX0=;BdP3oool0K@3oool00`000000oooo0?ooo`0>0?ooo`0305EE
E@3oool0oooo0080oooo00D0k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY?03oool00`000000oooo0?ooo`050?ooo`0605ADE0000000
000008n?S`3oool0N7Qh0P0000000`11@D40moOg0?ooo`0?0?ooo`030?Og
m`0V9RH0d];B0;h0oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Z
j^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03F
e]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Z
j^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3G
emL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0
gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool02`3oool405EEE@@0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<00000
0?ooo`3oool0h03oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0
oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03M
gMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJC`0ooooi000000<0?ooo`D0EEEE103oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7A
d@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3S
hn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gM
g@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@
d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM
0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03M
gMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo;@3oool505EEE@@0oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc40oooo
00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?ooo`0305EEE@3o
ool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CT
i03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cL
g03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFTl0?ooo`0605ADE0000000000008n?S`3oool0N7Qh0P00
00003`11@D40moOg07QhN00000003`l?0>o_k`1hN7P000000451@@3gmoL0
oooo07QhN00000003`l?0>o_k`030?ooo`080::RXP000000@D510<?3``0o
?cl0000000l?3`3_knl30?ooo`070<_;b`0o?cl0000000l?3`3_knl0oooo
0::RXP020000000@0;Bd]03oool0N7Qh0000001PH600N7Qh00000011@D40
VYZJ00000011@D40moOg07QhN00000003`l?0>o_k`@0oooo01@0N7Qh0000
002d];@0oooo07QhN00000003`l?0>_[j`1DE5@0H61P0=oOg`0B4Q80@D51
0?Ogm`1hN7P000000;Bd]03Ogml06a/K0=;BdYh0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cL
g03Lg=`0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY?@3oool00`1hN7P0];Bd0?ooo`020?ooo`0A0::R
XP2?Shl0oooo0<K6aP1PH600oooo0?Ogm`1mOGd0moOg02`/;03_knl0oooo
0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool0203kno/0GEeM0=;BdP3;bl/0
d];B0?ooo`36a/H0Shn?0`3oool06@3;bl/0d];B0?ooo`36a/H0Shn?0=oO
g`0K6a/0d];B0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B
0?_kn`1DE5@0];Bd0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool04@3kno/0
GEeM0=;BdP3oool0a/K6061PH03oool0moOg07emO@3Ogml0LW9b0?_kn`1M
GEd0d];B0?_kn`1MGEd0d];B0:80oooo00<0EEEE0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY?@3oool00`1hN7P0];Bd0?ooo`020?ooo`050::RXP2?Shl0oooo
0::RXP2d];@00`3oool01P3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]0H0oooo
00<0no_k05eMG@3Bd]800P3oool00`2RXZ8000000;Bd]0050?ooo`050::R
XP000000];Bd0::RXP1PH6000P3oool03@2RXZ80Shn?0?Ogm`0/;2`0kno_
0?_kn`1MGEd0d];B0?_kn`1DE5@0];Bd0::RXP2d];@01P3oool01P3kno/0
GEeM0=;BdP3oool0XZ:R0;Bd]0@0oooo00L0bl_;03lo?`2?Shl0oooo0?_k
n`1MGEd0d];B0:80oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF
0=OGe`0<0?ooo`0F0=OGe`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTm0?ooo`0307QhN02d];@0oooo0080oooo00@0XZ:R08n?S`3oool0
XZ:R0`0000001P1PH600moOg02`/;03_knl0oooo0::RXP<0000000/0H61P
0?ooo`3oool0oooo0?_kn`1MGEd0d];B0=oOg`0B4Q80@D510?Ogm`040?oo
o`070=oOg`0B4Q80@D510?Ogm`3oool0XZ:R08n?S`020?ooo`0<07QhN02?
Shl0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R0`00
00002@1PH600oooo0?ooo`3oool0no_k05eMG@3Bd]80oooo0::RXP030000
0009061PH03oool0a/K600l?3`3_knl0oooo0?_kn`1MGEd0d];B0:80oooo
00<0EEEE0?ooo`3oool00P3oool00`3/k>`0oOgm09>CT`050;Rh^0030;Fe
]@2h^;P0^;Rh00@0^;Rh01H0Ti>C0>c/k01YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJCd0oooo00<0N7Qh0000000000000`000000402?Shl0oooo
0<K6aP2?Shl0oooo07QhN02;Rh/0E5AD0000002?Shl0oooo0<K6aP2?Shl0
oooo07QhN02?Shl30?ooo`0807QhN0092@T0d];B0<K6aP2?Shl0oooo0861
P@3Bd]830?ooo`0I0<K6aP2?Shl0oooo0861P@3Bd]80gmoO05=CD`3gmoL0
gmoO018B4P2`/;00E5AD0000002?Shl0moOg01dM7@1PH600no_k05eMG@3B
d]80a/K608n?S`3oool0N7Qh08n?S`030?ooo`0A0=_Kf`1MGEd0d];B0?oo
o`36a/H0Shn?0?ooo`1hN7P0Shn?0?Ogm`0/;2`0inOW0:BTY03oool0fm_K
05eMG@3Bd]80XP3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k03^k^h0
SHf=00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool01`2RXZ80Shn?0?ooo`3o
ool0N7Qh00000011@D400P3gmoL0200/;2`0TY:B00T92@3Bd]80N7Qh0000
0011@D40moOg0`3oool0203gmoL09RHV0=;BdP3kno/0E5AD0000002FUYH0
fm_K0`3oool01`3kno/0E5AD0000002FUYH0fm_K0?ooo`2RXZ800P000000
402?Shl0oooo0=oOg`1CDe<0VYZJ00000011@D40_[jn00000011@D40moOg
0?ooo`1hN7P000000451@@3gmoL30?ooo`0?0<K6aP00000000000;Bd]03o
ool0N7Qh00000011@D40lo?c05ADE0000000TY:B018B4P2?Shl0a/K60080
000000<0];Bd0?ooo`3oool0W`3oool00`1EEED0oooo0?ooo`020?ooo`03
0>c/k02FUYH0g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool00`2RXZ80
Shn?0?ooo`0i0?ooo`030861P@3Bd]80oooo00/0oooo00<0PH610=;BdP3o
ool0X@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0
c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0
oooo00<0N7Qh0;Bd]03oool00P3oool00`2RXZ80Shn?0?ooo`0i0?ooo`03
0:RXZ03Bd]80oooo00/0oooo00<0Z:RX0=;BdP3oool0X@3oool00`1EEED0
oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00H0E5AD00000000
0000Shn?0?ooo`1hN7P2000000030451@@3gmoL0oooo00l0oooo00<0moOg
02HV9P3Bd]80e`3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0
oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0
oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0
oooo0080c/k>00<0?Shn0<k>cP3>c/h00`3>c/h06P0n?Sh0c/k>0<k>cP3>
c/h0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
;P3oool405EEE@@0oooo00@0k>c/09JFUP3oool0clo?0`0n?Sh30<o?c`<0
?Shn0P3?cll05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3o
ool00P3De=@303hn?P040=CDe00n?Sh0?Shn03hn?P<0e=CD01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo;@3oool505EE
E@@0oooo00<0k>c/09JFUP3oool00`3Kfm/503hn?P@0fm_K01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0UYJF0?ooo`040>CTi0<0?Shn1@3Ti>@05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo
00D0kNg]00<0?Shn0>g]k@3]kNd0103]kNd05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo
00`0m?Cd01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0?ooo`0<0?clo00F0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JF
UP3oool03@3oool05@2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
f][J09:BTP2f][H:0;Fe]@0G0;Bd]02?Shl0f][J06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00003
0000003gmoL0VYZJ0?l0NWYjB@1jNWX04`2JVYX0moOg0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000030000003gmoL0
_kno0?l0_knoBP2o_kl04P3gmoL0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo00000`000000moOg0>?Sh`3o0>?ShdX0hn?S
0180moOg0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000<000000?7al@3oool0o`3oool30?ooo`030>?Sh`2UYJD0MWIf
00<0MWIf00@0MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3o
ool0hn?S0:FUY@@0MWIf00@0MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0
hn?S0?ooo`3oool0jn_[0:FUY@@0EUIF00@0EeMG05IFEP1GEeL0EeMG2@1F
EUH00`2UYJD0jn_[0?ooo`030?ooo`0B0?7al@000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003QhN40o?cl0?_k
n`80o?cl00<0no_k0?clo03lo?`0103lo?`00`3kno/0o?cl0?clo0020?cl
o0030?_kn`3lo?`0no_k00D0o?cl00<0no_k0?clo03kno/0F@3kno^?0?cl
o0030:FUY@1fMWH0SHf=0100SHf=00H0MWIf0:FUY@3lo?`0o?cl0:FUY@1f
MWHA08f=S@0607IfMP2UYJD0o?cl0?_kn`2UYJD0EUIF4@1KFe/00`1FEUH0
YJFU0?clo0030?clo00B0>7Qh@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000030000003Ph>00moOg0?l0moOg0`3gmoL0
0`1fMWH0SHf=0::RXP0@0::RXP0608f=S@1fMWH0moOg0?Ogm`1fMWH0SHf=
4@2RXZ80202=SHd0MWIf0?Ogm`3gmoL0EUIF05]KF`1_Kfl0LG5a3@1bLW80
101aLG40Kfm_05]KF`1FEUH40?Ogm`0B0>3Ph0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003Ng]h0lO7a0080
lO7a0P3bl_800`3alO40l_;b0?;blP040?7al@030?;blP3alO40lO7a00<0
lO7a0P3bl_800`3alO40l_;b0?7al@020?;bl^X0lO7a00<0MWIf0;Rh^02[
Zj/0402[Zj/01P2h^;P0MWIf0?7al@3alO40MWIf0;Rh^140Zj^[00D0^;Rh
07IfMP3alO40l_;b05IFEP0206m_K`03079bLP1gMgL0MgMg00/0MgMg00@0
LW9b06m_K`1_Kfl0EUIF0`3alO404`3bl_80g]kN0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000050000003Lg=`0jn_[
0>_[j`3/k>`00P3[jn/00`2WYjL0XZ:R0::RXP020:6QX@030:2PX02OWil0
WIfM0080WinO0P2PX:020:6QX@030:NWY`3[jn/0jn_[00D0jn_[00D0e=CD
0<G5a@35aLD0aLG50=OGe`020>_[j`030>[ZjP38b<P0d=3@0080jn_[00D0
f][J0<K6aP3Shn<0jn_[0>3Ph0020<G5a@070<c<c03YjNT0c/k>0<O7a`3Y
jNT0jn_[0>3Ph0020<G5a@070<c<c03Zj^X0e=CD0<G5a@3MgMd0jn_[0>3P
h0020<G5a@060<K6aP3Shn<0jn_[0>_[j`3Zj^X0b<S80`35aLD00`36a/H0
hn?S0>_[j`040>_[j`030>[ZjP38b<P0aLG500<0aLG500<0a/K60>?Sh`3[
jn/06`3[jn]K0>c/k3@0jn_[00@0MWIf0;^k^`2a/K40/K6a203oool70;6a
/@060;^k^`1fMWH0jn_[0>_[j`1fMWH0^k^k0P2a/K4<0?ooo`<0/K6a00T0
^k^k07IfMP3[jn/0jn_[05IFEP1gMgL0LW9b07AdM01gMgL02`1hN7P01@1g
MgL0M7Ad079bLP1gMgL0EUIF00@0jn_[0180g=cL0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=[JfP3UiND0
0`3UiND00`2:RXX0Vi^K0:b/[0030>WYj@0:0>[ZjP34a<@0QHF50>c/k03]
kNd0k^k^0>OWi`3`l?00l_;b0:>SX`H0iNGU0140J6QX010@400@4100410@
0820P036a/H0clo?0=kNgP0V9RH0DE5A0<o?c`3UiND0U9BD01DE5@2TY:@0
g]kN0;:b/P02010@40070451@@3Kfm/0D51@01TI6@2j^[X0hn?S0;:b/P02
010@40070451@@3Ogml0L71`010@402BTY80i>CT0:n_[`02010@400601DE
5@2j^[X0c<c<0>CTi03MgMd092@T0`0@41000`0E5AD0Y:BT0=kNgP040>GU
i@030=kNgP0V9RH0410@00<0410@00<05ADE0;jn_P3UiND0Z`3UiND01@1f
MWH0`<300;Fe]@2e]KD0oooo00H0LW9b00<09bLW0;Fe]@2e]KD01@2e]KD0
1P30`<00MWIf0>GUi@3UiND0MWIf0<30`080]KFe00<0oooo02LW9`0W9bL0
2@0W9bL30;Fe]@050<30`01fMWH0iNGU0>GUi@1FEUH00P1gMgL0101iNGT0
NWYj0?ooo`3oool707YjNP80oooo00D0NWYj07UiN@1gMgL0MgMg05IFEP04
0>GUi@0B0=[JfP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000030000003IfMT0gmoO00<0gmoO00D0g]kN0:2PX03Zj^X0
jNWY0>GUi@020>[ZjP0:0=3@d01dM7@0kNg]0>k^kP2][Jd0d=3@0?;blP3c
lo<0Xj>S0=oOg`80g]kN03@0gmoO0=kNgP1/K6`05ADE0:^[Z`3Ng]h0`l?3
01`L701DE5@0a/K60=KFeP0U9BD0DE5A0<g=c@3Ng]h0Shn?01DE5@2TY:@0
eMGE04i>CP0h>3P0a<C40=gMg@3Kfm/0CTi>01TI6@2j^[X0fMWI04i>CP0h
>3P0a<C40=gMg@3GemL09BDU03Ph>034a<@0`/;201/K6`1ADE40fMWI0<?3
``1<C4`0f=SH0=OGe`0U9BD06ATI0<G5a@3Ng]h0Shn?01DE5@2TY:@0fMWI
1@3Ng]h01@2][Jd0410@01TI6@2i^KT0dM7A0:h0g]kN00D0MWIf0<?3``2k
^k/0^k^k0?ooo`06079bLP0302LW9`2k^k/0^k^k00D0^k^k00H0`l?307If
MP3Ng]h0g]kN07IfMP33`l<20;^k^`030?ooo`0W9bL0^k^k00L0^k^k00<0
oooo02LW9`2k^k/00P2k^k/02@33`l<0MWIf0=oOg`3Ng]h0EUIF07MgM`1h
N7P0Ng]k030`<0030?ooo`D0OGem0`3oool01@0`<300Ng]k07QhN01gMgL0
EUIF0080g]kN01@0gmoO0=kNgP3IfMT0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000emOG0=SHf03IfMT20=SH
f00@092@T02?Shl0SXj>0>WYj@21PH40DE5A08N7Q`3Jf]X0YZJV0:VYZ@2/
[:`0GemO0?;blP3clo<0m?Cd0:BTY0<0f=SH00D0fMWI0=;BdP0T92@0DE5A
0<_;b`020=SHf00P06YZJP0I6AT0^[Zj0=3@d00T92@0DE5A0<_;b`3Hf=P0
Rh^;01DE5@2TY:@0cLg=02@T900h>3P0`l?30=OGe`3EeMD0C4a<01TI6@2j
^[X0d=3@02@T900h>3P0`l?30=OGe`3AdM4092@T03Ph>033`l<0RXZ:01DE
5@2TY:@40<G5a@0:09jNWP0@4100LG5a0=7Ad@3Hf=P0Rh^;01DE5@2TY:@0
e=CD0=;BdP80aLG500P0a/K60=OGe`3Hf=P0JVYZ010@401bLW80aLG50=CD
e1h0f=SHF`3IfMTd0=SHf00507IfMP37alL0_kno0;no_`3oool01`0W9bL7
0;no_`060<O7a`1fMWH0f=SH0=SHf01fMWH0alO70P2o_kl00`3oool09bLW
0;no_`070;no_`030?ooo`0W9bL0_kno0080_kno00D0alO707IfMP3Hf=P0
f=SH05IFEP0207]kN`0307inOP1oOgl0:2PX00<0oooo0`1oOgl30?ooo`03
02PX:01oOgl0OWin0080Ng]k00<0EUIF0=WIf@3Hf=P00P3Hf=P04P3GemL0
000009ZJVP2PX:00Z:RX0;:b/P2l_;`0a/K60=3@d03IfMT0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
0`000000eMGE0=?Cd`030=?Cd`0308F5Q@2SXj<0i^KV0080j^[Z00/0=CDe
07moO`3?cll0_[jn08^;R`1NGUh0F5QH06QXJ02SXj<0bLW90:ZZZP040=?C
d`040<c<c00T92@0DE5A0<[:bP80dm?C01h0IfMW01TI6@2i^KT0bl_;02@T
901ADE40b/[:0=?Cd`28R8P05ADE0:BTY039bLT092@T03Ph>031`L40dm?C
0<o?c`1:BTX06ATI0;Vi^@3;bl/092@T03Ph>031`L40dm?C0<c<c00T92@0
>3Ph0<71`@1WIfL5010@400:055AD@2MWId0410@08j>SP3<c<`0dm?C08R8
R00E5AD0Y:BT0:6QX@80410@00T06QXJ0<K6aP3Cdm<0dm?C0<c<c00T92@0
5ADE0:BTY037alL0[@3Cdm<00`1fMWH0bl_;0<K6aP0@0<K6aP060<_;b`1f
MWH0dm?C0=?Cd`1fMWH0bl_;0P36a/H00`3oool09bLW0<K6aP070<K6aP03
0?ooo`0W9bL0a/K60080a/K600L0bl_;07IfMP3Cdm<0dm?C05IFEP1mOGd0
Ogmo00<0Ph>300D0:2PX0?ooo`3oool0oooo08>3P`030?ooo`0702PX:023
Ph<0Ph>308>3P`1oOgl0OGem05IFEP040=?Cd`0A0=GEe@000000X:2P0:FU
Y@2][Jd0][Jf0;no_`38b<P0d];B0=[JfP3Rh^80jNWY0>o_k`3dm?@0m_Kf
0?Win@3kno/00`3moOd10?knoP40oooo0@3oool000050000003Cdm<0c/k>
0<k>cP3?cll00P3>c/h03`27QhL0Ti>C0=?Cd`3Kfm/0[Jf]0<71`@2`/;00
ZJVY0:ZZZP2=SHd0TI6A0;2`/03;bl/0kno_0:FUY@030<k>cP050<o?c`37
alL08b<S055AD@38b<P00P3>c/h0;@1UIFD06ATI0;Vi^@37alL08b<S055A
D@37alL0c/k>08F5Q@0E5AD0Y:BT0<O7a`0S8b<0>3Ph0<71`@3>c/h0bl_;
04U9B@0I6AT0^KVi0<O7a`0S8b<0>3Ph0<71`@3>c/h0alO702<S8`0h>3P0
`L7108F5Q@0E5AD0Y:BT0<[:bP37alL08b<S055AD@2a/K406QXJ079bLP35
aLD0c/k>08F5Q@0E5AD0Y:BT0<g=c@070<k>cP050:2PX00@4100>CTi0;no
_`3;bl/01@3>c/h01@3=cLd0alO70<g=c@37alL0b/[:01<0c/k>F`3?clld
0<k>cP0307IfMP3>c/h0bl_;0100bl_;00H0c/k>07IfMP3>c/h0c/k>07If
MP3>c/h20<_;b`030?ooo`0W9bL0bl_;00L0bl_;00<0oooo02LW9`3;bl/0
0P3;bl/02@3>c/h0MWIf0<o?c`3>c/h0EUIF07emO@22PX80QHF508B4Q002
08F5Q@0302PX:03oool0oooo00<0oooo00P0:2PX08F5Q@25QHD0Q8B408F5
Q@22PX80OGem05IFEP@0c/k>0140dm?C0000002XZ:P0[Jf]0;>c/`2k^k/0
`l?30<c<c03EeMD0gMgM0>CTi03[jn/0l?3`0?Cdm03gmoL0nOWi0?clo003
0?gmo@40o_kn0@3oool10?ooo`0000<000000=3@d039bLT00`39bLT04@1c
Lg<0U9BD07inOP1iNGT0X:2P0=7Ad@2][Jd0YJFU0:JVYP25QHD0GUiN0:f]
[@3UiND0m_Kf0?Shn02TY:@0bLW900<0b<S800@0`/;202<S8`1ADE40alO7
0P38b<P0;01RHV806ATI0;Vi^@32`/808R8R03Ph>031`L40`/;204I6AP0I
6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35aLD0a<C404I6AP0I6AT0^KVi0<71
`@0S8b<0>3Ph0;no_`35aLD0`<30028R8P0h>3P0`<300:n_[`0J6QX0Lg=c
0<G5a@2JVYX0410@08j>SP35aLD0ATI601TI6@2i^KT0aLG507moO`0E5AD0
Y:BT2@38b<P0101RHV80410@08f=S@37alL40<S8b0060;>c/`15ADD0XJ6Q
04a<C01^KVh0alO74`38b<QK0<W9bC@0b<S800<0MWIf0=?Cd`3=cLd0403=
cLd01P3Cdm<0MWIf0<S8b038b<P0MWIf0=?Cd`80cLg=00<0oooo02LW9`3=
cLd01`3=cLd00`3oool09bLW0<g=c@020<g=c@070=?Cd`1fMWH0b<S80<S8
b01FEUH0PX:208>3P`0508N7Q`0502PX:03oool0oooo0?ooo`0X:2P01@27
QhL00`23Ph<0PX:205IFEP030<W9b@0B0<S8b03@d=0000000;:b/P2f][H0
^k^k0<;2`P39bLT0dM7A0=WIf@3Ph>00inOW0>g]k@3alO40mOGe0?Shn03j
n_X0o?cl0`3moOd10?knoP40oooo0@3oool000030000003?cll0aLG500@0
aLG500l0UiNG0;Ng]`3/k>`0^KVi08^;R`2g]kL0ZZZZ0:BTY01nOWh0LG5a
0<C4a03IfMT0m_Kf0?Win@2UYJD01035aLD0102o_kl08R8R055AD@36a/H2
0<G5a@06065QH@0I6AT0^KVi0;no_`0R8R80GEeM0P0@4100100E5AD0Y:BT
0<;2`P15ADD3010@4006055AD@1QHF40410@01TI6@2f][H0ADE50`0@4100
201ADE40_kno028R8P0h>3P0_kno0<G5a@2/[:`06QXJ0P0@41001@1bLW80
aLG50<G5a@32`/80ADE500<0410@00<05ADE0:BTY036a/H02035aLD00`2/
[:`06QXJ055AD@050<G5a@050;no_`0R8R80<30`01XJ6P2>SXh0X`35aLD0
0`1fMWH0eMGE0=7Ad@0@0=7Ad@060=GEe@1fMWH0aLG50<G5a@1fMWH0eMGE
0P3AdM400`3oool09bLW0=7Ad@070=7Ad@030?ooo`0W9bL0dM7A0080dM7A
00L0eMGE07IfMP35aLD0aLG505IFEP22PX80QhN700D0RHV91@3oool508V9
R@0308N7Q`22PX80EUIF00@0aLG50140clo?0000002l_;`0_kno0<?3``39
bLT0d=3@0=OGe`3MgMd0i>CT0>[ZjP3_knl0lo?c0?KfmP3hn?P0no_k0?cl
o0020?gmo@80o_kn0P3oool000030000003=cLd0`L7100<0`L710100MGEe
09VIV@2WYjL0kNg]0<W9b@1cLg<0/;2`08f=S@1cLg<0C4a<069RHP2n_[h0
]KFe0;:b/P2][Jd0ZJVY1031`L40102k^k/08R8R055AD@34a<@20<71`@04
05mOG`0I6AT0^KVi0<;2`PL0`L7100D0^k^k028R8P0h>3P0_[jn0<;2`P02
0<71`@0<0<?3``35aLD0_;bl028R8P0h>3P0_[jn0<;2`P2k^k/08R8R03Ph
>02n_[h0`/;22`31`L40101lO7`05ADE0:BTY033`l<60<71`@80a<C400@0
_[jn04A4A00h>3P0_[jn1031`L401`1OGel0410@010@400@4100>3Ph0;jn
_P32`/80X@31`L400`1fMWH0f=SH0=GEe@0@0=GEe@060=SHf01fMWH0`L71
0<71`@1fMWH0f=SH0P3EeMD00`3oool09bLW0?ooo`080?ooo`0602LW9`3E
eMD0eMGE0=GEe@3Hf=P0MWIf0P31`L400`1FEUH0Q8B408R8R00408Z:RP<0
oooo00@0:2PX0?ooo`3oool0oooo102:RXX00`28R8P0Q8B405IFEP040<71
`@0@0<g=c@000000a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<00000
0<c<c02j^[X00P2m_Kd04@2n_[h0_Kfm09jNWP3Shn<0][Jf09JFUP22PX80
Ti>C08B4Q022PX80CTi>02`/;02LW9`0Zj^[0?KfmP3hn?P0YjNW00@0_Kfm
00@0]kNg024Q8@1ADE40a<C40P2m_Kd0101MGEd06ATI0;Vi^@2o_kl80;fm
_@0305eMG@0g=cL0^;Rh0080_Kfm00/0GEeM01TI6@2i^KT0_kno05eMG@0g
=cL0^;Rh0;fm_@2g]kL08B4Q03Ph>0020;jn_P/0_Kfm00@0NWYj01DE5@2T
Y:@0_kno102m_Kd01`2g]kL08B4Q06m_K`35aLD0_kno02<S8`1`L7001@2m
_Kd01P1jNWX0=3@d03`l?01MGEd0>3Ph0;Rh^1<0_KfmF`2n_[hd0;fm_@03
07IfMP3Jf]X0eMGE0100eMGE00H0f][J07IfMP2m_Kd0_Kfm07IfMP3Jf]X2
0=GEe@030?ooo`1bLW80LW9b00P0LW9b00h09bLW0=GEe@3EeMD0eMGE0=[J
fP1fMWH0_Kfm0;jn_P1FEUH0QhN708Z:RP2=SHd0SXj>08f=S@<0oooo00<0
:2PX08f=S@0X:2P00`3oool308f=S@0408Z:RP27QhL0EUIF0;jn_P80_Kfm
0140^[Zj0<c<c0000000d=3@0=;BdP3EeMD0fMWI0=gMg@3Rh^80inOW0>c/
k03`l?00m?Cd0?KfmP3hn?P0n_[j0?clo0030?gmo@030?knoP3oool0oooo
0040oooo00000`000000b/[:0;Rh^0030;Zj^P0@07alO02=SHd0N7Qh09VI
V@3`l?00`/;20:NWY`3;bl/0_;bl0:>SX`1]KFd0OGem0:b/[03kno/0o?cl
0:JVYP<0^[Zj00@0^k^k0;Bd]00Q8B40CDe=0`2j^[X00`1KFe/06ATI0:n_
[`0>0;Zj^P0305]KF`0I6AT0[jn_00D0^[Zj00@0];Bd024Q8@0g=cL0];Bd
302j^[X00`1gMgL051@D09^KV`060;Zj^P05096AT@0@4100410@010@401=
CDd01`2j^[X00`2d];@08B4Q06][J`2T0;Zj^P0307IfMP3Hf=P0f][J0100
f][J00H0f=SH07IfMP2j^[X0^[Zj07IfMP3Hf=P20=[JfP030?ooo`1bLW80
LW9b00P0LW9b00d09bLW0=[JfP3Jf]X0f][J0=SHf01fMWH0^[Zj0;^k^`1F
EUH0Rh^;08n?S`2>SXh0Shn?00<0oooo00D0:2PX08n?S`2?Shl0Shn?02PX
:0030?ooo`0508n?S`2>SXh0Shn?08^;R`1FEUH00`2j^[X0402h^;P0b/[:
0000003IfMT0f][J0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0
n?Sh0?[jnP3lo?`30?gmo@80o_kn0`3oool0000300000039bLT0^;Rh00<0
^;Rh0100QXJ609jNWP3De=@0l?3`0?;blP3>c/h0`L710=gMg@3Bd]80XZ:R
0<c<c039bLT0RHV90?clo03moOd0YjNWk`2h^;P00`1fMWH0emOG0=gMg@0@
0=gMg@060=OGe`1fMWH0^;Rh0;Rh^01fMWH0emOG0`3MgMd;02LW9`<0gMgM
00T0emOG07IfMP2h^;P0^;Rh05IFEP2@T900TI6A09>CT`2DU9@00P3oool0
0`0X:2P0V9RH09RHV00309RHV00802PX:03oool0oooo09BDU02CTi<0TI6A
092@T01FEUH40;Rh^00?0<W9b@000000hN7Q0>;RhP3Ti>@0inOW0>[ZjP3]
kNd0l?3`0??cl`3emOD0moOg0?Win@3kno/0o?cl00<0oOgm00<0o_kn0?oo
o`3oool00P3oool0000300000039bLT0^;Rh00@0^;Rh00l0QXJ609BDU03b
l_80lo?c0:2PX03Xj>P0m_Kf0=_Kf`2IVIT0n_[j0?_kn`3Lg=`0][Jf0?kn
oP2WYjL0k`2h^;P00`1fMWH0e]KF0>7Qh@0@0>7Qh@060=KFeP1fMWH0^;Rh
0;Rh^01fMWH0e]KF4@3QhN402@3Fe]H0MWIf0;Rh^02h^;P0EUIF09FEU@2G
UiL0W9bL0::RXP0202PX:0050:^[Z`2][Jd0Zj^[0:^[Z`2][Jd00P2[Zj/2
02PX:0050::RXP2LW9`0UiNG09FEU@1FEUH0102h^;P03P39bLT000000>SX
j03YjNT0jn_[0>g]k@3_knl0lO7a0?Cdm03fm_H0moOg0?Win@3kno/0o?cl
0`3moOd20?knoP@0oooo00000`000000bLW90;Fe]@030;Fe]@0=07MgM`2P
X:00h>3P0??cl`3UiND0k>c/0?KfmP3hn?P0nOWi092@T03kno/0o?cl0?gm
o@020?knoP030:NWY`2e]KD0]KFe0>d0]KFe00<0MWIf0>g]k@3YjNT0403Y
jNT01P3]kNd0MWIf0;Fe]@2e]KD0MWIf0>g]kA40jNWY00X0kNg]07IfMP2e
]KD0]KFe05IFEP2KVi/0WYjN0:FUY@2b/[80^[Zj0P2o_kl01030`<00_kno
0;no_`30`<020;no_`070<30`02j^[X0/[:b0:FUY@2NWYh0Vi^K05IFEP04
0;Fe]@0=0<W9b@000000k^k^0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Sh
n03inOT0no_k0?clo0030?gmo@80o_kn1@3oool0000300000039bLT0^[Zj
00@0/K6a00<0SHf=0:ZZZP2ZZZX01@2ZZZX00`2WYjL0ZZZZ0:ZZZP040:ZZ
Z^l0/K6a00<0MWIf0=CDe03bl_80403bl_801P3De=@0MWIf0;6a/@2a/K40
MWIf0=CDe140l_;b00H0e=CD07IfMP2a/K40/K6a05IFEP2SXj<20:RXZ0d0
`l?30P2XZ:P00`2SXj<0EUIF0;6a/@020;6a/@040;Zj^P39bLT000000??c
l`80m?Cd00L0mOGe0?KfmP3gmoL0n?Sh0?[jnP3kno/0o?cl00@0oOgm00<0
o_kn0?ooo`3oool0103oool0000300000033`l<0bLW90?l0[Zj^0`2^[Zh0
102=SHd0MWIf0<k>cP3kno/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^
00@0SHf=07IfMP3>c/h0no_k3@3oool0103kno/0c/k>07IfMP2=SHd20:j^
[P0609VIV@1FEUH0]KFe0<_;b`3Ogml0j>SX2@3WinL01P3Xj>P0gmoO0<_;
b`2e]KD0EUIF09VIV@<0[Zj^00<0bLW90<?3``0000000P3fm_H01@3gmoL0
n?Sh0?Shn03inOT0n_[j0080o?cl103moOd20?knoPH0oooo000010000000
`l?30=CDe02g]kOo0:NWY`<0YjNW00<0SHf=07IfMP1fMWH03`1fMWH00`2=
SHd0YjNW0:NWY`020:NWY`0308f=S@1fMWH0MWIf00l0MWIf00<0SHf=0:NW
Y`2WYjL00P2WYjL00`2=SHd0EUIF05IFEP0?05IFEP0808f=S@2WYjL0YjNW
0:NWY`2g]kL0e=CD0<?3``0000030?Win@040?[jnP3kno/0o?cl0?clo0D0
oOgm0P3no_h70?ooo`0000H000000:ZZZP3WinL0e=CD0;jn_P2c/k?o0:ZZ
ZT<0ZZZZ00H0/k>c0;jn_P3De=@0inOW0:ZZZP0000020?_kn`<0o?cl1@3m
oOd30?knoPP0oooo00001`000000RXZ:0=WIf@3lo?`0i>CT0=;BdP3<c<`0
o`2n_[i10;jn_P080<c<c03Bd]80i>CT0?clo03IfMT0RXZ:0000003lo?`8
0?gmo@80o_kn2P3oool000080000001YJFT0UiNG0=WIf@3lo?`0l_;b0>c/
k03Shn?o0=WIfCl0fMWI00P0hn?S0>c/k03bl_80o?cl0=WIf@2GUiL0JFUY
000000L0oOgm0`3no_h;0?ooo`0000D000000?ooo`1YJFT0UiNG0=?Cd`02
0?clo?l0oooo@@3oool20?clo0050=?Cd`2GUiL0JFUY0?ooo`000000103m
oOd40?knoPd0oooo00000`000000oooo0?ooo`0206UYJ@0308Z:RP2GUiL0
ZZZZ0?l0ZZZZ@02ZZZX0102GUiL0RXZ:06UYJ@1YJFT20?ooo`030000003n
o_h0o_kn00@0o_kn3`3oool00?l00000C`00000E0?ooo`00\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{355, 227},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell[TextData[{
  "If we simply give a string as the second argument to ",
  StyleBox["NotebookWrite", "MR"],
  ", ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will use the default cell type, ",
  StyleBox["Input", "MR"],
  "."
}], "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(NotebookWrite[nb, "\<Here is some text.\>"]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005S0000hb000`400?l0
0000o`00003oo`3ooomT0?ooo`00o`3ooomT0?ooo`003P3ooooo0?knoTL0
o_kn3`3oool000`0oooo103no_ko0?gmoD<0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgmB@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo3d0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k?`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinD00nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_Ho0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clcd0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hm0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj3d0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4m0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH>`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllk0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aC/0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^c/0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a>`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYS/0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhk09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXm09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000004l000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?clB`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[ZdX0Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFU906UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj6`2OWil00`2m_Kd0k>c/0>OWi`0T0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K6`3:b/X0
0`3Kfm/0Xj>S0?_kn`0T0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo6`3@d=000`3oool0UYJF0?_kn`0T0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3ooolK0<o?c`03
0?ooo`2FUYH0no_k02@0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM400`3@d=070=7Ad@80d=3@0P3AdM430=3@
d0L0dM7A00<0oooo09JFUP3jn_X0903jn_X00`2FUYH0oooo0<k>cP040<k>
cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JFUP3Ph>00403Ph>00501Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0
h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3Ph01YJFT0WYjN0>3Ph02>
SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0
103>c/h303hn?PD0c/k>00@0oooo09JFUP2FUYH0oooo203Fe]H20=OGe`07
09FEU@3Fe]H0UIFE0=KFeP2EUID0e]KF09FEU@050=KFeP80emOG0`3Fe]H0
0`3oool0UYJF0?Win@0T0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n?Sh6
0<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h034a<@0B4Q80=;BdP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0
g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?oo
o`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3Lg=`6
0=gMg@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID70=gMg@80
g=cL00<0gMgM0?ooo`2FUYH09@3inOT00`2FUYH0oooo0<o?c`040<o?c`<0
?Shn1@3?cll00`3oool0UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c
/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3I
fMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV
9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103o
ool0UYJF09JFUP3oool30=cLg0H0gMgM00P0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@L0gMgM0P3Lg=`00`3MgMd0oooo09JFUP0U0?Shn003
09JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0h>3P00P0
h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00G0<C4
a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B
0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2NWYh0
3`3Ph>05000000H0h>3P00@0UYJF0?ooo`3Kfm/0fm_K0`0n?Sh80=_Kf`05
0?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0<0hn?S00T0i>CT0?oo
o`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Ti>@30>?Sh`<0i>CT
00@0hn?S0>CTi03oool0UYJF9@3gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0
?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P
01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV
9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0a<C4
04Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WY
j@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY
00D0j^[Z0>WYj@3YjNT0oooo09JFUP0U0?Gem@0309JFUP3oool0i>CT00D0
i>CT0`0n?Sh40>CTi0030?ooo`2FUYH0h>3P00d0h>3P0P3oool05@3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P01T0SXj>0000000h>3P0fMWI0>3Ph01YJFT0WYjN
0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph02][Jd0
E5AD0>3Ph01YJFT0OWin0>3Ph01YJFT0WYjN0100h>3P00<000000>3Ph03P
h>001P3Ph>000`2FUYH0oooo0>g]k@030>g]k@<0?Shn1P3]kNd01@3oool0
UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WYj@080?ooo`2EUID0oooo
09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY00D0j^[Z0>WYj@3YjNT0
oooo09JFUP0U0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]k@03
0?ooo`2FUYH0h>3P00`0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0
h>3P01T0fMWI059BDP3IfMT0h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P
0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3Bd]80
h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@303hn
?PD0m?Cd00@0oooo09JFUP2FUYH0oooo2@3]kNd01`3oool0kNg]0?ooo`3]
kNd0oooo0>g]k@3oool02`3]kNd00`3oool0UYJF0?7al@0T0?7al@0309JF
UP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2FUYH0h>3P00`0h>3P
0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jN
WP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3lo?`01@3l
o?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?oooa/0kNg]
00<0oooo09JFUP3`l?00903`l?000`2FUYH0oooo0?clo0040?clo00303hn
?P3lo?`0o?cl00D0o?cl00<0oooo09JFUP3Ph>00403Ph>00501YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph00309JFUP3oool0oooo00d0oooo0P2FUYH00`3oool0moOg0?Og
m`0I0?Ogm`030?ooo`2FUYH0kno_02@0kno_00<0UYJF0?ooo`3oool03@3o
ool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?cl
o02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF
0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XK0?ooo`030?[jnP2F
UYH0kno_02@0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF0>3P
h00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F
0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e]KD0
1@2d];@0Shn?0=[JfP3Fe]H0/;2`01/0^;Rh00<0ZJVY0<k>cP3^k^h0903^
k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0f][J0140h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFVG09jNWP030?ooo`3Ph>00
h>3P03D0h>3P4@3/k>`O0>_[jbD0inOW4@3/k>`A0>3Ph00D06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03^k^h0SHf=
00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JF
UP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool01@3>c/h00`0n
?Sh0c/k>0<k>cP040<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool0103>c/h303hn
?PD0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0?ooo`030<o?c`D0?Shn103?cll05P3o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh3
0=CDe00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn0P3K
fm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`030>c/k02FUYH0oooo0080i>CT00<0?Shn0>CTi03Ti>@00`3T
i>@06P0n?Sh0i>CT0>CTi03Ti>@0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?ooo`0<0>g]k@0F
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0
o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3GemL0
303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3J
f]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooobd0oooo1@1EEED40?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFVg0?ooo`030000003oool0oooo07H0oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY]`3oool00`000000oooo0?ooo`1f0?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJKL0oooo00<000000?ooo`3o
ool0MP3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFVg0?ooo`03
0000003oool0oooo07H0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYA03oool02@1hN7P000000000002BTY804Q8B0000001PH600oooo0<K6
aP03000000040451@@3gmoL0oooo0::RXP80000000D0@D510?Ogm`3oool0
oooo0<K6aP03000000030451@@3gmoL0oooo0080oooo00<0a/K600000000
00000`0000000`2d];@0gmoO018B4P03000000030;Bd]03oool0oooo0080
oooo00<0gmoO018B4P0000000P0000004P2d];@0oooo0?ooo`3Ogml04Q8B
000000092@T0d];B0=oOg`0B4Q80000006UYJ@0B4Q80@D51071`L0092@T0
d];B0<K6aP<0000000<0@D510?Ogm`3oool0103oool01`36a/H000000000
0011@D40moOg0?ooo`36a/H00`0000003011@D40lo?c05ADE00000002@T9
0<[:bP0M7Ad000000451@@3gmoL0oooo0<K6aP80000000L0@D510?Ogm`3o
ool0oooo0<K6aP000000];Bd00<0oooo00<000000?ooo`3oool0MP3oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFU40?ooo`0B0?_kn`1DE5@0
];Bd0?ooo`3Ogml08B4Q0>o_k`36a/H000000;Bd]03oool0moOg01dM7@2d
];@0oooo0?_kn`1DE5@0];Bd0`3oool01`36a/H000000;Bd]03oool0moOg
01dM7@2d];@01@3oool00`3Ogml06a/K0=;BdP020?ooo`070=oOg`0Q8B40
kno_0?ooo`3Ogml06a/K0=;BdP030?ooo`0L0=oOg`0Q8B40kno_0?ooo`3O
gml06a/K0=;BdP3Ogml04Q8B0;Bd]03oool0a/K600T92@3Bd]80XZ:R061P
H03Ogml06a/K0=;BdP1hN7P0Shn?0<K6aP000000];Bd0?ooo`3gmoL07AdM
0;Bd]0@0oooo01<0no_k05ADE02d];@0oooo07QhN02?Shl0a/K60000002d
];@0oooo0?Ogm`0M7Ad0];Bd0?ooo`2RXZ80@D510??cl`1DE5@0Shn?0080
oooo00H0no_k05ADE02d];@0oooo07QhN02?Shl20?ooo`030<K6aP000000
];Bd00<0oooo00<000000?ooo`3oool0MP3oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFU40?ooo`0:0?_kn`1DE5@0];Bd0?ooo`3Ogml08B4Q
0>o_k`1hN7P0@D510?Ogm`D0oooo00<0no_k05ADE02d];@00`3oool00`1h
N7P0@D510?Ogm`090?ooo`030=oOg`0K6a/0d];B00@0oooo00D0moOg01dM
7@0000003`l?0>o_k`050?ooo`070?Ogm`0M7Ad0000000l?3`3_knl0XZ:R
061PH0030?ooo`0<07QhN02?Shl0XZ:R061PH03Ogml06a/K0=;BdP1hN7P0
Shn?07QhN011@D40moOg203oool00`3gmoL09RHV0=;BdP030?ooo`0307Qh
N011@D40moOg00H0oooo00@0N7Qh04Q8B011@D40moOg0P3oool00`3gmoL0
9RHV0=;BdP0;0?ooo`030000003oool0oooo07H0oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYA03oool0203kno/0E5AD0;Bd]03oool0gmoO
024Q8@3_knl0N7Qh1@0000001@2?Shl0oooo0?_kn`1DE5@0H61P00<0oooo
00<0N7Qh0000000000000`0000000`2?Shl0oooo0?ooo`030?ooo`030=oO
g`0K6a/0d];B0080oooo00H0no_k05ADE0000000000000l?3`3_knl40?oo
o`090?_kn`1DE5@000000000000?3`l0kno_0?ooo`2RXZ80H61P0080oooo
00/0no_k05ADE02?Shl0XZ:R061PH03Ogml06a/K0=;BdP1hN7P0Shn?07Qh
N0050000000308n?S`3oool0oooo0080oooo00<0moOg02HV9P3Bd]800`3o
ool00`1hN7P00000000000030000000608n?S`3oool0oooo0::RXP000000
Shn?0`3oool00`3gmoL09RHV0=;BdP0;0?ooo`030000003oool0oooo07H0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool00`3kno/0
E5AD000000020000000G00l?3`3_knl0a/K600T92@3Bd]80oooo0<K6aP09
2@T0d];B0?ooo`3kno/0E5AD0000001PH600a/K6061PH036a/H02@T90=;B
dP3oool0a/K600T92@3Bd]801@3oool00`3Ogml06a/K0=;BdP020?ooo`07
0?_kn`1DE5@0];Bd0?ooo`36a/H0@D510?Ogm`030?ooo`0L0?_kn`1DE5@0
];Bd0?ooo`36a/H0@D510?Ogm`3Ogml04Q8B0;Bd]03oool0a/K600T92@3B
d]80XZ:R00l?3`3?cll04Q8B0;Bd]01hN7P0Shn?0<K6aP092@T0d];B0?oo
o`36a/H02@T90=;BdP@0oooo00<0moOg02HV9P3Bd]800`3oool03@36a/H0
2@T90=;BdP3oool0a/K600T92@3Bd]80oooo0<K6aP092@T0b/[:01dM7@2d
];@00P3oool00`3gmoL09RHV0=;BdP0;0?ooo`030000003oool0oooo07H0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool02@3kno/0
E5AD0;Bd]03oool0gmoO024Q8@3_knl0oooo0<K6aP030000000;0;Bd]03o
ool0gmoO018B4P000000[:b/01dM7@000000];Bd0?ooo`36a/H00`000000
0`2d];@0oooo0?ooo`020?ooo`050?_kn`1DE5@0000000T92@3Bd]800`3o
ool01P3Ogml04Q8B000000000000@D510?Ogm`@0oooo01H0gmoO018B4P00
000000000451@@3gmoL0oooo0=oOg`0B4Q80000000T92@3Bd]80gmoO018B
4P1GEeL07AdM0;2`/01DE5@03`l?0>o_k`3oool0a/K60`0000000`2d];@0
oooo0?ooo`020?ooo`030=oOg`0B4Q8000000080000000@0@D510?Ogm`3o
ool0a/K60`0000002`2d];@0oooo0::RXP0000002@T90;:b/P0B4Q800000
08n?S`3Ogml04Q8B00<0000000<0@D510?Ogm`3oool01`3oool00`000000
oooo0?ooo`1f0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0
oooo00L0no_k05ADE02d];@0oooo0=oOg`0Q8B40kno_04T0oooo00<0moOg
02HV9P3Bd]804P3oool00`3gmoL09RHV0=;BdP0;0?ooo`030000003oool0
oooo07H0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool0
1`1hN7P000000000002BTY804Q8B0000001PH6006P3oool00`2RXZ803`l?
0>o_k`0/0?ooo`030?Ogm`0V9RH0d];B0180oooo00<0moOg02HV9P3Bd]80
2`3oool00`000000oooo0?ooo`1f0?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJKL0oooo00<000000?ooo`3oool0M@3oool205EEE@@0oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJKL0oooo00<000000?ooo`3oool0M03oool00`1EEED0oooo
05EEE@040?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0
moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFVg0?ooo`030000003oool0oooo07<0oooo
00@0EEEE0?ooo`3oool0EEEE103oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY]`3oool00`000000
oooo0?ooo`1b0?ooo`0505EEE@3oool0oooo0?ooo`1EEED0103oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oool/0?ooo`H0EEEE103oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k03`l?00SXj>00D0X:2P00<0WinO0:2PX02PX:00102PX:005P2@
T900k>c/06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k02FUYH0g]kN00`0dm?C01H0g]kN09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0i>CT0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M
0>CTi03Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gM
g@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0
gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo
i000000<0?ooo`D0EEEE103oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?oo
o`3P0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`030000003oool0
oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`00
0000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S
0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`03
0000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
?03oool00`000000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo
00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?oo
o`3P0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`030000003oool0
oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`00
0000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT
0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`03
0000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
?03oool00`000000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo
00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?oo
o`3P0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0
e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`030000003oool0
oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY<@3oool01@2R
XZ800000000000092@T0d];B00H0oooo00<000000?ooo`3oool01@3oool0
1P1DE5@000000000002?Shl0oooo07QhN080000000l0@D510?Ogm`1hN7P0
000000l?3`3_knl0N7Qh00000011@D40moOg0?ooo`1hN7P0000000l?3`3_
knl00`3oool0202RXZ8000000451@@33`l<0?clo0000000?3`l0kno_0`3o
ool01`3;bl/0?clo0000000?3`l0kno_0?ooo`2RXZ800P000000402d];@0
oooo07QhN0000000H61P07QhN0000000@D5109ZJVP000000@D510?Ogm`1h
N7P0000000l?3`3_knl30?ooo`0;07QhN0000000H61P07QhN0000000@D51
09ZJVP000000@D510?Ogm`2RXZ800P0000002`2d];@0oooo07QhN0000000
@D510?Ogm`3oool0N7Qh0000000?3`l0kno_00@0oooo01@0N7Qh0000002d
];@0oooo07QhN00000003`l?0>_[j`1DE5@0H61P0=oOg`0B4Q80@D510?Og
m`1hN7P000000;Bd]03Ogml06a/K0=;BdVX0oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT`0?ooo`060?_kn`1M
GEd0d];B0?ooo`2RXZ80Shn?1P3oool00`000000oooo0?ooo`060?ooo`03
07QhN02d];@0oooo0080oooo0140XZ:R08n?S`3oool0a/K6061PH03oool0
moOg07emO@3gmoL0;2`/0>o_k`3oool0a/K6061PH03oool0moOg07emO@03
0?ooo`080?_kn`1MGEd0d];B0<_;b`3Bd]80oooo0<K6aP2?Shl30?ooo`0I
0<_;b`3Bd]80oooo0<K6aP2?Shl0gmoO01/K6`3Bd]80no_k05eMG@3Bd]80
moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0a/K6061PH03o
ool0moOg07emO@030?ooo`0H0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_k
n`1DE5@0];Bd0=oOg`0K6a/0d];B0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_
0?ooo`36a/H0H61P0?ooo`3gmoL0OGem0`3oool04@3kno/0GEeM0=;BdP3o
ool0a/K6061PH03oool0moOg07emO@3Ogml0LW9b0?_kn`1MGEd0d];B0?_k
n`1MGEd0d];B06h0oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFT`0?ooo`060?Ogm`0/;2`0TY:B00l?3`2f][H0
H61P1P3oool00`000000oooo0?ooo`060?ooo`0307QhN02d];@0oooo0080
oooo00D0XZ:R08n?S`3oool0XZ:R0;Bd]0030?ooo`060?Ogm`0/;2`0kno_
0?ooo`2RXZ80];Bd1P3oool00`3kno/0GEeM0=;BdP020?ooo`030::RXP00
0000];Bd00D0oooo00D0XZ:R0000002d];@0XZ:R061PH0020?ooo`0=0::R
XP2?Shl0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R
0;Bd]0060?ooo`0;0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0
];Bd0::RXP1PH6000P3oool0202RXZ80Shn?0?Ogm`0/;2`0kno_0?ooo`2R
XZ80];Bd1P3oool01P3kno/0GEeM0=;BdP3oool0XZ:R0;Bd]0@0oooo00L0
bl_;03lo?`2?Shl0oooo0?_kn`1MGEd0d];B06h0oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT`0?ooo`060?Og
m`0M7Ad0H61P0?Ogm`0M7Ad0H61P1P3oool00`000000oooo0?ooo`060?oo
o`0307QhN02d];@0oooo0080oooo00@0XZ:R08n?S`3oool0XZ:R0`000000
1P1PH600moOg02`/;03_knl0oooo0::RXP<0000000/0H61P0?ooo`3oool0
oooo0?_kn`1MGEd0d];B0=oOg`0B4Q80@D510?Ogm`040?ooo`070=oOg`0B
4Q80@D510?Ogm`3oool0XZ:R08n?S`020?ooo`0<07QhN02?Shl0moOg02`/
;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R0`0000003`1PH600
oooo0?ooo`3oool0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d
];@0XZ:R08n?S`020?ooo`0707QhN02?Shl0moOg02`/;03_knl0oooo0::R
XP0300000009061PH03oool0oooo0?ooo`3kno/0GEeM0=;BdP3oool0XZ:R
00<0000000T0H61P0?ooo`36a/H03`l?0>o_k`3oool0no_k05eMG@3Bd]80
KP3oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJC00oooo00H0moOg02`/;03_knl0oooo0<K6aP1PH6060?ooo`03
0000003oool0oooo00H0oooo00<0N7Qh0000000000000`000000402?Shl0
oooo0<K6aP2?Shl0oooo07QhN02;Rh/0E5AD0000002?Shl0oooo0<K6aP2?
Shl0oooo07QhN02?Shl30?ooo`0807QhN0092@T0d];B0<K6aP2?Shl0oooo
0861P@3Bd]830?ooo`0I0<K6aP2?Shl0oooo0861P@3Bd]80gmoO05=CD`3g
moL0gmoO018B4P2`/;00E5AD0000002?Shl0moOg01dM7@1PH600no_k05eM
G@3Bd]80a/K608n?S`3oool0N7Qh08n?S`020?ooo`0I0?_kn`1DE5@00000
08n?S`3gmoL07AdM061PH03kno/0GEeM0=;BdP3Ogml0De=C0?Ogm`3Ogml0
4Q8B0;2`/01DE5@0000008n?S`3oool0a/K608n?S`3oool0N7Qh08n?S`03
0?ooo`0A0=_Kf`1MGEd0d];B0?ooo`36a/H0Shn?0?ooo`1hN7P0Shn?0?Og
m`0/;2`0inOW0:BTY03oool0fm_K05eMG@3Bd]80KP3oool00`000000oooo
0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3@d=00clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC00oooo00H0
moOg01dM7@11@D40emOG018B4P1PH6060?ooo`030000003oool0oooo00H0
oooo00<0N7Qh0;Bd]03oool00P3oool01`2RXZ80Shn?0?ooo`3oool0N7Qh
00000011@D400P3gmoL0200/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40
moOg0`3oool0203gmoL09RHV0=;BdP3kno/0E5AD0000002FUYH0fm_K0`3o
ool01`3kno/0E5AD0000002FUYH0fm_K0?ooo`2RXZ800P000000402?Shl0
oooo0=oOg`1CDe<0VYZJ00000011@D40_[jn00000011@D40moOg0?ooo`1h
N7P000000451@@3gmoL30?ooo`0;0=oOg`1CDe<0VYZJ00000011@D40_[jn
00000011@D40moOg0?ooo`2RXZ800P0000002`2?Shl0oooo0?Ogm`0/;2`0
TY:B00T92@3Bd]80N7Qh00000011@D40moOg00<0oooo00l0a/K600000000
0000];Bd0?ooo`1hN7P000000451@@3clo<0E5AD0000002BTY804Q8B08n?
S`36a/H00P0000000`2d];@0oooo0?ooo`1[0?ooo`030000003oool0oooo
00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@H0UIFE01T0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFT`0?ooo`060?_kn`1MGEd0d];B0?ooo`2RXZ80
Shn?1P3oool00`000000oooo0?ooo`060?ooo`0307QhN02d];@0oooo0080
oooo00<0XZ:R08n?S`3oool0E03oool00`21PH40d];B0?ooo`0;0?ooo`03
0861P@3Bd]80oooo06d0oooo00<000000?ooo`3oool03P3oool00`1EEED0
oooo0?ooo`020?ooo`050>c/k02FUYH0oooo0?Ogm`3]kNd01P3oool06P3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJC40oooo00D0XZ:R0000000000002@T90=;BdP060?ooo`030000
003oool0oooo00H0oooo00<0N7Qh0;Bd]03oool00P3oool00`2RXZ80Shn?
0?ooo`1D0?ooo`030:RXZ03Bd]80oooo00/0oooo00<0Z:RX0=;BdP3oool0
K@3oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd609FEU@0I0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`00
0000oooo0?ooo`050?ooo`0605ADE0000000000008n?S`3oool0N7Qh0P00
00000`11@D40moOg0?ooo`0?0?ooo`030?Ogm`0V9RH0d];B0;h0oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`050>c/k02F
UYH0oooo0?Ogm`3]kNd01P3oool06P3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<000000?oo
o`3oool0h03oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo
0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd609FEU@0I0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03o
ool00`000000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo00<0
EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo
01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?oo
o`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]1P2EUID06@3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJC`0oooo00<000000?ooo`3oool0h03o
ool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00D0
k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J0=KFeP3AdM40clo?0=3@d03o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`00
0000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT
0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`03
0000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
?03oool00`000000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo
00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?oo
o`3P0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3o
ool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0
e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`030000003oool0
oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`00
0000oooo0?ooo`3P0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT
0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`03
0000003oool0oooo00/0oooo101EEED40?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0
gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooon@00000303oool505EEE@@0oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3M
gMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gM
g@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0
gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooobd0oooo1@1EEED40?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?oooc40oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?oo
o`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<@3oool00`1EEED0
oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3T
i>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03L
g=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY?03oool01P1DE5@000000000002?Shl0oooo07QhN080
000000l0@D510?Ogm`1hN7P0000000l?3`3_knl0N7Qh00000011@D40moOg
0?ooo`1hN7P0000000l?3`3_knl00`3oool0202RXZ8000000451@@33`l<0
?clo0000000?3`l0kno_0`3oool01`3;bl/0?clo0000000?3`l0kno_0?oo
o`2RXZ800P000000402d];@0oooo07QhN0000000H61P07QhN0000000@D51
09ZJVP000000@D510?Ogm`1hN7P0000000l?3`3_knl40?ooo`0D07QhN000
0000];Bd0?ooo`1hN7P0000000l?3`3[jn/0E5AD061PH03Ogml04Q8B0451
@@3gmoL0N7Qh0000002d];@0gmoO01/K6`3Bd]:N0?ooo`0305EEE@3oool0
oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3L
g=`0g=cL0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool04@2R
XZ80Shn?0?ooo`36a/H0H61P0?ooo`3gmoL0OGem0?Ogm`0/;2`0kno_0?oo
o`36a/H0H61P0?ooo`3gmoL0OGem00<0oooo00P0no_k05eMG@3Bd]80bl_;
0=;BdP3oool0a/K608n?S`<0oooo01T0bl_;0=;BdP3oool0a/K608n?S`3O
gml06a/K0=;BdP3kno/0GEeM0=;BdP3gmoL0;2`/0>o_k`3kno/0GEeM0=;B
dP3kno/0E5AD0;Bd]036a/H0H61P0?ooo`3gmoL0OGem00<0oooo0140no_k
05eMG@3Bd]80oooo0<K6aP1PH600oooo0?Ogm`1mOGd0gmoO079bLP3kno/0
GEeM0=;BdP3kno/0GEeM0=;BdP2R0?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool01@2RXZ80Shn?0?oo
o`2RXZ80];Bd00<0oooo00H0moOg02`/;03_knl0oooo0::RXP2d];@60?oo
o`030?_kn`1MGEd0d];B0080oooo00<0XZ:R0000002d];@01@3oool01@2R
XZ8000000;Bd]02RXZ80H61P0080oooo00d0XZ:R08n?S`3gmoL0;2`/0>o_
k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]02RXZ80];Bd00H0oooo00H0no_k
05eMG@3Bd]80oooo0::RXP2d];@40?ooo`070<_;b`0o?cl0Shn?0?ooo`3k
no/0GEeM0=;BdP2R0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JF
UP3GemL0303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY?@3oool00`1hN7P0];Bd0?ooo`020?ooo`040::RXP2?Shl0oooo
0::RXP<0000000H0H61P0?Ogm`0/;2`0kno_0?ooo`2RXZ830000000;061P
H03oool0oooo0?ooo`3kno/0GEeM0=;BdP3Ogml04Q8B0451@@3gmoL0103o
ool01`3Ogml04Q8B0451@@3gmoL0oooo0::RXP2?Shl00P3oool0301hN7P0
Shn?0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0];Bd0::RXP<0
000000T0H61P0?ooo`3oool0oooo0?_kn`1MGEd0d];B0?ooo`2RXZ800`00
00002@1PH600oooo0<K6aP0?3`l0kno_0?ooo`3kno/0GEeM0=;BdP2R0?oo
o`0305EEE@3oool0oooo0080oooo00<0k>c/0?gmo@2CTi<01@2h^;P00`2e
]KD0^;Rh0;Rh^0040;Rh^00F09>CT`3/k>`0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFTm0?ooo`0307QhN0000000000000<000000100Shn?0?oo
o`36a/H0Shn?0?ooo`1hN7P0Rh^;05ADE0000000Shn?0?ooo`36a/H0Shn?
0?ooo`1hN7P0Shn?0`3oool0201hN7P02@T90=;BdP36a/H0Shn?0?ooo`21
PH40d];B0`3oool06@36a/H0Shn?0?ooo`21PH40d];B0=oOg`1CDe<0moOg
0=oOg`0B4Q80/;2`05ADE0000000Shn?0?Ogm`0M7Ad0H61P0?_kn`1MGEd0
d];B0<K6aP2?Shl0oooo07QhN02?Shl00`3oool04@3Kfm/0GEeM0=;BdP3o
ool0a/K608n?S`3oool0N7Qh08n?S`3gmoL0;2`/0>OWi`2TY:@0oooo0=_K
f`1MGEd0d];B0:80oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0k^k^
08f=S@0<09nOW`0F08f=S@3Vi^H0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTm0?ooo`0307QhN02d];@0oooo0080oooo00L0XZ:R08n?S`3oool0
oooo07QhN0000000@D510080moOg00P0;2`/09:BTP092@T0d];B07QhN000
0000@D510?Ogm`<0oooo00P0moOg02HV9P3Bd]80no_k05ADE0000000UYJF
0=_Kf`<0oooo00L0no_k05ADE0000000UYJF0=_Kf`3oool0XZ:R00800000
0100Shn?0?ooo`3Ogml0De=C09ZJVP000000@D510;jn_P000000@D510?Og
m`3oool0N7Qh00000011@D40moOg0`3oool03`36a/H000000000002d];@0
oooo07QhN0000000@D510??cl`1DE5@0000009:BTP0B4Q80Shn?0<K6aP02
000000030;Bd]03oool0oooo09l0oooo00<0EEEE0?ooo`3oool00P3oool0
0`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTm0?ooo`0307QhN02d];@0oooo0080oooo00<0XZ:R
08n?S`3oool0>@3oool00`21PH40d];B0?ooo`0;0?ooo`030861P@3Bd]80
oooo0:40oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0<
0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTm
0?ooo`0307QhN02d];@0oooo0080oooo00<0XZ:R08n?S`3oool0>@3oool0
0`2XZ:P0d];B0?ooo`0;0?ooo`030:RXZ03Bd]80oooo0:40oooo00<0EEEE
0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`0605ADE0000000
000008n?S`3oool0N7Qh0P0000000`11@D40moOg0?ooo`0?0?ooo`030?Og
m`0V9RH0d];B0=L0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF
0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF
0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF
0?ooo`020<k>cP0303hn?P3>c/h0c/k>00<0c/k>01X0?Shn0<k>cP3>c/h0
c/k>0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
obh0oooo101EEED40?ooo`040>c/k02FUYH0oooo0<o?c`<0?Shn0`3?cll3
03hn?P80clo?01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0
oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh30=CDe00F0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobd0oooo1@1E
EED40?ooo`030>c/k02FUYH0oooo00<0fm_K1@0n?Sh40=_Kf`0F0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0
k>c/09JFUP3oool0103Ti>@303hn?PD0i>CT01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?oo
o`050>g]k@0303hn?P3]kNd0kNg]00@0kNg]01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?oo
o`0<0?Cdm00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocH0oooo00<0k>c/09JFUP3oool0303lo?`05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02F
UYH0oooo00d0oooo01D0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool00`3/k>`0UYJF0=OGe`0<0?ooo`0F0=OGe`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/
0=[JfP2BTY80][Jf2P2e]KD05`2d];@0Shn?0=[JfP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
0`000000moOg09ZJVP3o07YjNTT0NWYj01<0VYZJ0?Ogm`000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00000`000000moOg
0;no_`3o0;no_dX0_kno0180moOg0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000<000000?Ogm`3Shn<0o`3Shn=:0>?S
h`0B0?Ogm`000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000030000003alO40oooo0?l0oooo0`3oool00`3Shn<0YJFU07If
MP0307IfMP0407MgM`1fMWH0MgMg07MgM`T0MWIf00H0YJFU0>?Sh`3oool0
oooo0>?Sh`2UYJD407IfMP0407MgM`1fMWH0MgMg07MgM`T0MWIf00H0YJFU
0>?Sh`3oool0oooo0>_[j`2UYJD405IFEP0405MGE`1FEUH0EeMG05MGE`T0
EUIF00<0YJFU0>_[j`3oool00`3oool04P3alO40000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000hN7Q0?clo03k
no/20?clo0030?_kn`3lo?`0o?cl00@0o?cl00<0no_k0?clo03lo?`00P3l
o?`00`3kno/0o?cl0?_kn`050?clo0030?_kn`3lo?`0no_k05T0no_kS`3l
o?`00`2UYJD0MWIf08f=S@0@08f=S@0607IfMP2UYJD0o?cl0?clo02UYJD0
MWIf4@2=SHd01P1fMWH0YJFU0?clo03kno/0YJFU05IFEQ40Fe]K00<0EUIF
0:FUY@3lo?`00`3lo?`04P3QhN40000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo00000`000000h>3P0?Ogm`3o0?Ogm`<0moOg
00<0MWIf08f=S@2RXZ80402RXZ801P2=SHd0MWIf0?Ogm`3gmoL0MWIf08f=
SA40XZ:R00P0SHf=07IfMP3gmoL0moOg05IFEP1KFe/0Kfm_075aL@d0LW9b
00@0LG5a06m_K`1KFe/0EUIF103gmoL04P3Ph>00000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000g]kN0?7al@02
0?7al@80l_;b00<0lO7a0?;blP3bl_80103alO400`3bl_80lO7a0?7al@03
0?7al@80l_;b00<0lO7a0?;blP3alO400P3bl_;Z0?7al@0307IfMP2h^;P0
Zj^[0100Zj^[00H0^;Rh07IfMP3alO40lO7a07IfMP2h^;PA0:^[Z`050;Rh
^01fMWH0lO7a0?;blP1FEUH00P1_Kfl00`1bLW80MgMg07MgM`0;07MgM`04
079bLP1_Kfl0Kfm_05IFEP<0lO7a01<0l_;b0=kNgP000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001@000000g=cL0>_[
j`3[jn/0k>c/0080jn_[00<0YjNW0::RXP2RXZ800P2QXJ400`2PX:00WinO
09fMW@0209nOW`80X:2P0P2QXJ400`2WYjL0jn_[0>_[j`050>_[j`050=CD
e035aLD0aLG50<G5a@3GemL00P3[jn/00`3Zj^X0b<S80=3@d0020>_[j`05
0=[JfP36a/H0hn?S0>_[j`3Ph>000P35aLD01`3<c<`0jNWY0<k>cP37alL0
jNWY0>_[j`3Ph>000P35aLD01`3<c<`0j^[Z0=CDe035aLD0gMgM0>_[j`3P
h>000P35aLD01P36a/H0hn?S0>_[j`3[jn/0j^[Z0<S8b0<0aLG500<0a/K6
0>?Sh`3[jn/0103[jn/00`3Zj^X0b<S80<G5a@030<G5a@030<K6aP3Shn<0
jn_[01/0jn_[F`3/k>`d0>_[j`0407IfMP2k^k/0/K6a0;6a/@P0oooo1`2a
/K401P2k^k/0MWIf0>_[j`3[jn/0MWIf0;^k^`80/K6a303oool30;6a/@09
0;^k^`1fMWH0jn_[0>_[j`1FEUH0MgMg079bLP1dM7@0MgMg00/0N7Qh00D0
MgMg07AdM01bLW80MgMg05IFEP040>_[j`0B0=cLg0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003Jf]X0iNGU
00<0iNGU00<0RXZ:09^KV`2/[:`00`3YjNT02P3Zj^X0a<C408F5Q@3/k>`0
kNg]0>k^kP3WinL0l?3`0?;blP2SXj<60>GUi@0A06QXJ00@4100410@010@
4020P800a/K60<o?c`3Ng]h09RHV055AD@3?cll0iNGU09BDU00E5AD0Y:BT
0=kNgP2b/[800P0@41001`11@D40fm_K051@D00I6AT0^[Zj0>?Sh`2b/[80
0P0@41001`11@D40gmoO071`L00@4100TY:B0>CTi02_[jl00P0@41001P0E
5AD0^[Zj0<c<c03Ti>@0gMgM02@T90<0410@00<05ADE0:BTY03Ng]h0103U
iND00`3Ng]h09RHV010@4003010@400301DE5@2n_[h0iNGU0:/0iNGU00D0
MWIf0<30`02e]KD0]KFe0?ooo`06079bLP0302LW9`2e]KD0]KFe00D0]KFe
00H0`<3007IfMP3UiND0iNGU07IfMP30`<020;Fe]@030?ooo`0W9bL09bLW
00T09bLW0`2e]KD01@30`<00MWIf0>GUi@3UiND0EUIF0080MgMg00@0NGUi
07YjNP3oool0oooo1`1jNWX20?ooo`0507YjNP1iNGT0MgMg07MgM`1FEUH0
103UiND04P3Jf]X0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo00000`000000fMWI0=oOg`030=oOg`050=kNgP2PX:00j^[Z
0>WYj@3UiND00P3Zj^X02P3@d=00M7Ad0>g]k@3^k^h0[Jf]0=3@d03bl_80
lo?c0:>SX`3Ogml20=kNgP0d0=oOg`3Ng]h0K6a/01DE5@2[Zj/0g]kN0<?3
``0L71`0E5AD0<K6aP3Fe]H09BDU055AD@3=cLd0g]kN08n?S`0E5AD0Y:BT
0=GEe@1>CTh0>3Ph0<C4a03MgMd0fm_K04i>CP0I6AT0^[Zj0=WIf@1>CTh0
>3Ph0<C4a03MgMd0emOG02DU9@0h>3P0a<C40<;2`P0K6a/0DE5A0=WIf@33
`l<0C4a<0=SHf03GemL09BDU01TI6@35aLD0g]kN08n?S`0E5AD0Y:BT0=WI
f@D0g]kN00D0[Jf]010@400I6AT0^KVi0=7Ad@2^0=kNgP0507IfMP33`l<0
^k^k0;^k^`3oool01P1bLW800`0W9bL0^k^k0;^k^`050;^k^`060<?3``1f
MWH0g]kN0=kNgP1fMWH0`l?30P2k^k/00`3oool09bLW0;^k^`070;^k^`03
0?ooo`0W9bL0^k^k0080^k^k00T0`l?307IfMP3Ogml0g]kN05IFEP1gMgL0
N7Qh07]kN`0`<3000`3oool507emO@<0oooo00D0<30`07]kN`1hN7P0MgMg
05IFEP020=kNgP0D0=oOg`3Ng]h0fMWI0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000=OGe`3Hf=P0fMWI0P3H
f=P0402@T900Shn?08j>SP3YjNT0PH61055AD@27QhL0f][J0:JVYP2YZJT0
[:b/05mOG`3bl_80lo?c0?Cdm02TY:@30=SHf0050=WIf@3Bd]8092@T055A
D@3;bl/00P3Hf=P0801ZJVX06ATI0;Zj^P3@d=0092@T055AD@3;bl/0f=SH
08^;R`0E5AD0Y:BT0<g=c@0T92@0>3Ph0<?3``3GemL0eMGE04a<C00I6AT0
^[Zj0=3@d00T92@0>3Ph0<?3``3GemL0dM7A02@T900h>3P0`l?308Z:RP0E
5AD0Y:BT1035aLD02P2NWYh0410@075aL@3AdM40f=SH08^;R`0E5AD0Y:BT
0=CDe03Bd]820<G5a@080<K6aP3GemL0f=SH06YZJP0@4100LW9b0<G5a@3D
e=@N0=SHf5/0fMWI=03Hf=P01@1fMWH0alO70;no_`2o_kl0oooo00L09bLW
1`2o_kl01P37alL0MWIf0=SHf03Hf=P0MWIf0<O7a`80_kno00<0oooo02LW
9`2o_kl01`2o_kl00`3oool09bLW0;no_`020;no_`050<O7a`1fMWH0f=SH
0=SHf01FEUH00P1kNg/00`1nOWh0Ogmo02PX:0030?ooo`<0Ogmo0`3oool0
0`0X:2P0Ogmo07inOP0207]kN`0305IFEP3IfMT0f=SH0080f=SH0180emOG
0000002JVYX0X:2P0:RXZ02b/[80_;bl0<K6aP3@d=00fMWI0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00<000000=GEe@3Cdm<00`3Cdm<00`25QHD0Xj>S0>KViP020>[ZjP0;03De
=@1oOgl0clo?0;jn_P2;Rh/0GUiN05QHF01XJ6P0Xj>S0<W9b@2ZZZX0103C
dm<0103<c<`092@T055AD@3:b/X20=?Cd`0N06MWI`0I6AT0^KVi0<_;b`0T
92@0DE5A0<[:bP3Cdm<0R8R801DE5@2TY:@0bLW902@T900h>3P0`L710=?C
d`3?cll0BTY:01TI6@2i^KT0bl_;02@T900h>3P0`L710=?Cd`3<c<`092@T
03Ph>031`L40IfMW1@0@41002P1ADE40WIfM010@402>SXh0c<c<0=?Cd`28
R8P05ADE0:BTY02QXJ42010@400901XJ6P36a/H0dm?C0=?Cd`3<c<`092@T
01DE5@2TY:@0alO70:d0dm?C00<0MWIf0<_;b`36a/H04036a/H01P3;bl/0
MWIf0=?Cd`3Cdm<0MWIf0<_;b`80a/K600<0oooo02LW9`36a/H01`36a/H0
0`3oool09bLW0<K6aP020<K6aP070<_;b`1fMWH0dm?C0=?Cd`1FEUH0OGem
07moO`0308>3P`0502PX:03oool0oooo0?ooo`23Ph<00`3oool01`0X:2P0
Ph>308>3P`23Ph<0Ogmo07emO@1FEUH0103Cdm<04@3EeMD000000:2PX02U
YJD0[Jf]0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?Kf
mP3inOT0no_k00<0oOgm0@3no_h10?ooo`40oooo00001@000000dm?C0<k>
cP3>c/h0clo?0080c/k>00l0QhN709>CT`3Cdm<0fm_K0:f][@31`L40/;2`
0:VYZ@2ZZZX0SHf=096AT@2`/;00bl_;0>o_k`2UYJD00`3>c/h01@3?cll0
alO702<S8`1ADE40b<S80080c/k>02d0IFEU01TI6@2i^KT0alO702<S8`1A
DE40alO70<k>cP25QHD05ADE0:BTY037alL08b<S03Ph>031`L40c/k>0<_;
b`19BDT06ATI0;Vi^@37alL08b<S03Ph>031`L40c/k>0<O7a`0S8b<0>3Ph
0<71`@25QHD05ADE0:BTY03:b/X0alO702<S8`1ADE40/K6a01XJ6P1bLW80
aLG50<k>cP25QHD05ADE0:BTY03=cLd01`3>c/h01@2PX:00410@03Ti>@2o
_kl0bl_;00D0c/k>00D0cLg=0<O7a`3=cLd0alO70<[:bP0C0<k>cU/0clo?
=03>c/h00`1fMWH0c/k>0<_;b`0@0<_;b`060<k>cP1fMWH0c/k>0<k>cP1f
MWH0c/k>0P3;bl/00`3oool09bLW0<_;b`070<_;b`030?ooo`0W9bL0bl_;
0080bl_;00T0c/k>07IfMP3?cll0c/k>05IFEP1mOGd0PX:208F5Q@24Q8@0
0P25QHD00`0X:2P0oooo0?ooo`030?ooo`0802PX:025QHD0QHF508B4Q025
QHD0PX:207emO@1FEUH40<k>cP0A0=?Cd`000000Z:RX0:f][@2c/k<0^k^k
0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Win@3lo?`0
0`3moOd10?knoP40oooo0@3oool000030000003@d=00bLW900<0bLW90140
Lg=c09BDU01nOWh0NGUi0:2PX03AdM40[Jf]0:FUY@2VYZH0QHF505iNGP2]
[Jd0iNGU0?KfmP3hn?P0Y:BT0<W9b@030<S8b0040<;2`P0S8b<0DE5A0<O7
a`80b<S802`0HV9R01TI6@2i^KT0`/;2028R8P0h>3P0`L710<;2`P16ATH0
6ATI0;bl_031`L408b<S03Ph>02o_kl0aLG50<C4a016ATH06ATI0;Vi^@31
`L408b<S03Ph>02o_kl0aLG50<30`00R8R80>3Ph0<30`02_[jl06QXJ07=c
L`35aLD0VYZJ010@402>SXh0aLG504I6AP0I6AT0^KVi0<G5a@1oOgl05ADE
0:BTY0T0b<S800@0HV9R010@402=SHd0alO71038b<P01P2c/k<0ADE50:6Q
X@1<C4`0KVi^0<O7aa<0b<S8F`39bLTd0<S8b00307IfMP3Cdm<0cLg=0100
cLg=00H0dm?C07IfMP38b<P0b<S807IfMP3Cdm<20<g=c@030?ooo`0W9bL0
cLg=00L0cLg=00<0oooo02LW9`3=cLd00P3=cLd01`3Cdm<0MWIf0<S8b038
b<P0EUIF08:2PP23Ph<01@27QhL01@0X:2P0oooo0?ooo`3oool0:2PX00D0
QhN700<0Ph>308:2PP1FEUH00`39bLT04P38b<P0d=3@0000002b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0<0oOgm0@3no_h10?ooo`40oooo00000`000000clo?0<G5a@04
0<G5a@0?09NGU`2g]kL0k>c/0;Vi^@2;Rh/0]kNg0:ZZZP2TY:@0OWin075a
L@34a<@0fMWI0?KfmP3inOT0YJFU00@0aLG500@0_kno028R8P1ADE40a/K6
0P35aLD01P1QHF406ATI0;Vi^@2o_kl08R8R05eMG@80410@00@05ADE0:BT
Y032`/80ADE50`0@41001P1ADE40HF5Q010@400I6AT0][Jf04E5A@<0410@
00P0DE5A0;no_`0R8R80>3Ph0;no_`35aLD0[:b/01XJ6P80410@00D0LW9b
0<G5a@35aLD0`/;204E5A@03010@400301DE5@2TY:@0a/K600P0aLG500<0
[:b/01XJ6P1ADE401@35aLD01@2o_kl08R8R030`<00J6QX0SXj>0:<0aLG5
00<0MWIf0=GEe@3AdM40403AdM401P3EeMD0MWIf0<G5a@35aLD0MWIf0=GE
e@80dM7A00<0oooo02LW9`3AdM401`3AdM400`3oool09bLW0=7Ad@020=7A
d@070=GEe@1fMWH0aLG50<G5a@1FEUH0PX:208N7Q`0508V9R@D0oooo1@29
RHT00`27QhL0PX:205IFEP040<G5a@0A0<o?c`000000_;bl0;no_`33`l<0
bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3fm_H0n?Sh0?_kn`3l
o?`00P3moOd20?knoP80oooo00000`000000cLg=0<71`@030<71`@0@07Ee
M@2IVIT0YjNW0>g]k@39bLT0Lg=c0;2`/02=SHd0Lg=c04a<C01RHV80_[jn
0;Fe]@2b/[80[Jf]0:VYZ@@0`L7100@0^k^k028R8P1ADE40a<C40P31`L40
101OGel06ATI0;Vi^@32`/870<71`@050;^k^`0R8R80>3Ph0;jn_P32`/80
0P31`L403033`l<0aLG50;bl_00R8R80>3Ph0;jn_P32`/80^k^k028R8P0h
>3P0_[jn0<;2`P/0`L7100@0O7al01DE5@2TY:@0`l?31P31`L420<C4a004
0;jn_P14A4@0>3Ph0;jn_P@0`L7100L0GemO010@400@4100410@03Ph>02n
_[h0`/;20:40`L7100<0MWIf0=SHf03EeMD0403EeMD01P3Hf=P0MWIf0<71
`@31`L40MWIf0=SHf080eMGE00<0oooo02LW9`3oool0203oool01P0W9bL0
eMGE0=GEe@3EeMD0f=SH07IfMP80`L7100<0EUIF08B4Q028R8P0102:RXX3
0?ooo`0402PX:03oool0oooo0?ooo`@0RXZ:00<0R8R808B4Q01FEUH01031
`L40403=cLd000000<K6aP38b<P0c<c<0=7Ad@3GemL0g=cL0>;RhP3Xj>P0
kNg]0?7al@3emOD0moOg0?Win@3lo?`30?gmo@80o_kn0P3oool000030000
003<c<`0^[Zj0080_Kfm0140_[jn0;fm_@2NWYh0hn?S0;Jf]P2FUYH0PX:2
09>CT`24Q8@0PX:204i>CP0/;2`0W9bL0:^[Z`3fm_H0n?Sh0:NWY`040;fm
_@040;Ng]`0Q8B40DE5A0<C4a080_Kfm00@0GEeM01TI6@2i^KT0_kno202m
_Kd00`1MGEd0=cLg0;Rh^0020;fm_@0;05eMG@0I6AT0^KVi0;no_`1MGEd0
=cLg0;Rh^02m_Kd0]kNg024Q8@0h>3P00P2n_[h;0;fm_@0407YjNP0E5AD0
Y:BT0;no_`@0_Kfm00L0]kNg024Q8@1_Kfl0aLG50;no_`0S8b<0L71`00D0
_Kfm00H0NWYj03@d=00l?3`0GEeM03Ph>02h^;PC0;fm_E/0_[jn=02m_Kd0
0`1fMWH0f][J0=GEe@0@0=GEe@060=[JfP1fMWH0_Kfm0;fm_@1fMWH0f][J
0P3EeMD00`3oool0LW9b079bLP08079bLP0>02LW9`3EeMD0eMGE0=GEe@3J
f]X0MWIf0;fm_@2n_[h0EUIF08N7Q`2:RXX0SHf=08j>SP2=SHd30?ooo`03
02PX:02=SHd0:2PX00<0oooo0`2=SHd0102:RXX0QhN705IFEP2n_[h20;fm
_@0A0;Zj^P3<c<`000000=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OWi`3/
k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`00`3moOd00`3no_h0oooo0?oo
o`010?ooo`0000<000000<[:bP2h^;P00`2j^[X0401lO7`0SHf=07QhN02I
VIT0l?3`0<;2`P2WYjL0bl_;0;bl_02SXj<0KFe]07emO@2/[:`0no_k0?cl
o02VYZH30;Zj^P040;^k^`2d];@08B4Q04e=C@<0^[Zj00<0Fe]K01TI6@2_
[jl03P2j^[X00`1KFe/06ATI0:n_[`050;Zj^P040;Bd]00Q8B40=cLg0;Bd
]0`0^[Zj00<0MgMg01@D502KVi/01P2j^[X01@2ATI40410@010@400@4100
CDe=00L0^[Zj00<0];Bd024Q8@1[Jf/0Y02j^[X00`1fMWH0f=SH0=[JfP0@
0=[JfP060=SHf01fMWH0^[Zj0;Zj^P1fMWH0f=SH0P3Jf]X00`3oool0LW9b
079bLP08079bLP0=02LW9`3Jf]X0f][J0=[JfP3Hf=P0MWIf0;Zj^P2k^k/0
EUIF08^;R`2?Shl0SXj>08n?S`030?ooo`0502PX:02?Shl0Shn?08n?S`0X
:2P00`3oool01@2?Shl0SXj>08n?S`2;Rh/0EUIF00<0^[Zj0100^;Rh0<[:
bP000000fMWI0=[JfP3MgMd0h>3P0>CTi03Xj>P0k>c/0>o_k`3clo<0m_Kf
0?Shn03jn_X0o?cl0`3moOd20?knoP<0oooo00000`000000bLW90;Rh^003
0;Rh^00@08J6QP2NWYh0e=CD0?3`l03bl_80c/k>0<71`@3MgMd0d];B0::R
XP3<c<`0bLW908V9R@3lo?`0oOgm0:NWYnl0^;Rh00<0MWIf0=OGe`3MgMd0
403MgMd01P3GemL0MWIf0;Rh^02h^;P0MWIf0=OGe`<0gMgM2`0W9bL30=gM
g@090=OGe`1fMWH0^;Rh0;Rh^01FEUH0T92@096AT@2CTi<0U9BD0080oooo
00<0:2PX09RHV02HV9P00`2HV9P0200X:2P0oooo0?ooo`2DU9@0Ti>C096A
T@2@T900EUIF102h^;P03`39bLT000000>7Qh@3Rh^80i>CT0>OWi`3Zj^X0
kNg]0?3`l03clo<0mOGe0?Ogm`3inOT0no_k0?clo0030?gmo@030?knoP3o
ool0oooo0080oooo00000`000000bLW90;Rh^0040;Rh^00?08J6QP2DU9@0
l_;b0??cl`2PX:00j>SX0?KfmP3Kfm/0VIVI0?[jnP3kno/0g=cL0;Jf]P3n
o_h0YjNW0>l0^;Rh00<0MWIf0=KFeP3QhN40403QhN401P3Fe]H0MWIf0;Rh
^02h^;P0MWIf0=KFeQ40hN7Q00T0e]KF07IfMP2h^;P0^;Rh05IFEP2EUID0
UiNG09bLW02RXZ800P0X:2P01@2[Zj/0[Jf]0:^[Z`2[Zj/0[Jf]0080Zj^[
0P0X:2P01@2RXZ80W9bL09NGU`2EUID0EUIF00@0^;Rh00h0bLW90000003X
j>P0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k0?cl
o0<0oOgm0P3no_h40?ooo`0000<000000<W9b@2e]KD00`2e]KD03@1gMgL0
X:2P0>3Ph03clo<0iNGU0>c/k03fm_H0n?Sh0?Win@2@T900no_k0?clo03m
oOd00P3no_h00`2WYjL0]KFe0;Fe]@3]0;Fe]@0307IfMP3]kNd0jNWY0100
jNWY00H0kNg]07IfMP2e]KD0]KFe07IfMP3]kNdA0>WYj@0:0>g]k@1fMWH0
]KFe0;Fe]@1FEUH0Vi^K09jNWP2UYJD0/[:b0;Zj^P80_kno00@0`<300;no
_`2o_kl0`<300P2o_kl01`30`<00^[Zj0;:b/P2UYJD0WYjN09^KV`1FEUH0
102e]KD03@39bLT000000>k^kP3_knl0l?3`0?7al@3clo<0mOGe0?KfmP3h
n?P0nOWi0?_kn`3lo?`00`3moOd20?knoPD0oooo00000`000000bLW90;Zj
^P040;6a/@0308f=S@2ZZZX0ZZZZ00D0ZZZZ00<0YjNW0:ZZZP2ZZZX0102Z
ZZ[_0;6a/@0307IfMP3De=@0l_;b0100l_;b00H0e=CD07IfMP2a/K40/K6a
07IfMP3De=@A0?;blP060=CDe01fMWH0/K6a0;6a/@1FEUH0Xj>S0P2XZ:P=
0<?3``80Z:RX00<0Xj>S05IFEP2a/K400P2a/K40102j^[X0bLW90000003c
lo<20?Cdm0070?Gem@3fm_H0moOg0?Shn03jn_X0no_k0?clo0040?gmo@03
0?knoP3oool0oooo00@0oooo00000`000000`l?30<W9b@3o0:j^[P<0[Zj^
00@0SHf=07IfMP3>c/h0no_k3@3oool0103kno/0c/k>07IfMP2=SHd20:j^
[P0408f=S@1fMWH0c/k>0?_kn`d0oooo00@0no_k0<k>cP1fMWH0SHf=0P2^
[Zh01P2IVIT0EUIF0;Fe]@3;bl/0gmoO0>SXj0T0inOW00H0j>SX0=oOg`3;
bl/0]KFe05IFEP2IVIT30:j^[P030<W9b@33`l<000000080m_Kf00D0moOg
0?Shn03hn?P0nOWi0?[jnP020?clo0@0oOgm0P3no_h60?ooo`0000@00000
0<?3``3De=@0]kNgo`2WYjL30:NWY`0308f=S@1fMWH0MWIf00l0MWIf00<0
SHf=0:NWY`2WYjL00P2WYjL00`2=SHd0MWIf07IfMP0?07IfMP0308f=S@2W
YjL0YjNW0080YjNW00<0SHf=05IFEP1FEUH03`1FEUH0202=SHd0YjNW0:NW
Y`2WYjL0]kNg0=CDe033`l<000000`3inOT0103jn_X0no_k0?clo03lo?`5
0?gmo@80o_kn1`3oool000060000002ZZZX0inOW0=CDe02n_[h0/k>co`2Z
ZZY30:ZZZP060;>c/`2n_[h0e=CD0>OWi`2ZZZX000000P3kno/30?clo0D0
oOgm0`3no_h80?ooo`0000L0000008Z:RP3IfMT0o?cl0>CTi03Bd]80c<c<
0?l0_[jn@@2n_[h0203<c<`0d];B0>CTi03lo?`0fMWI08Z:RP000000o?cl
203moOd20?knoPX0oooo000020000000JFUY09NGU`3IfMT0o?cl0?;blP3/
k>`0hn?So`3IfMTo0=WIf@080>?Sh`3/k>`0l_;b0?clo03IfMT0UiNG06UY
J@0000070?gmo@<0o_kn2`3oool000050000003oool0JFUY09NGU`3Cdm<0
0P3lo?co0?oood40oooo0P3lo?`01@3Cdm<0UiNG06UYJ@3oool0000000@0
oOgm103no_h=0?ooo`0000<000000?ooo`3oool00P1YJFT00`2:RXX0UiNG
0:ZZZP3o0:ZZZT00ZZZZ00@0UiNG08Z:RP1YJFT0JFUY0P3oool00`000000
o_kn0?knoP040?knoPl0oooo003o000004l000005@3oool00001\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{355, 227},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell[TextData[{
  "Now suppose we wanted to insert an input cell with the expression ",
  Cell[BoxData[
      \(TraditionalForm\`2\^100\)]],
  " in it. "
}], "Text",
  PageBreakAbove->True,
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(2\^100\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[BoxData[
    \(1267650600228229401496703205376\)], "Output",
  CellLabel->"Out[11]=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]],

Cell[TextData[{
  "If you were to look at the underlying expression of the above cell (under \
the ",
  StyleBox["Format", "SR"],
  " menu, choose ",
  StyleBox["Show Expression", "SR"],
  "), it would look like this:"
}], "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell["\<\
Cell[BoxData[
    SuperscriptBox[\"2\", \"100\"]], \"Input\"]\
\>", "Program",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[TextData[{
  "We will talk about ",
  StyleBox["BoxData", "MR"],
  " in just a moment, but we should be able to insert a cell like this \
directly into our notebook object. Before we do this, notice that the \
insertion point has been left inside the Input cell after the last ",
  StyleBox["NotebookWrite", "MR"],
  ". To move the cell insertion bar after the current cell, we will use ",
  StyleBox["SelectionMove", "MR"],
  " which takes three arguments: the notebook we are operating on, the \
direction to move, and the unit by which we should move. The direction can be \
any of ",
  StyleBox["Next", "MR"],
  ", ",
  StyleBox["Previous", "MR"],
  ", ",
  StyleBox["After", "MR"],
  ", ",
  StyleBox["Before", "MR"],
  ", ",
  StyleBox["All", "MR"],
  ". The units are things like ",
  StyleBox["Word", "MR"],
  ", ",
  StyleBox["Cell", "MR"],
  ", ",
  StyleBox["CellGroup", "MR"],
  ", ",
  StyleBox["Notebook", "MR"],
  " (see the Help Browser under ",
  StyleBox["SelectionMove", "MR"],
  " for a complete description)."
}], "Text",
  CellTags->{"i:13", "b:43.1", "IPM3.10.2.2"}],

Cell["\<\
So, in our example, we want to move the selection just after the present \
cell.\
\>", "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SelectionMove[nb, After, Cell]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005S0000hb000`400?l0
0000o`00003oo`3ooomT0?ooo`00o`3ooomT0?ooo`003P3ooooo0?knoTL0
o_kn3`3oool000`0oooo103no_ko0?gmoD<0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgmB@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo3d0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k?`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinD00nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_Ho0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clcd0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hm0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj3d0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4m0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH>`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllk0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aC/0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^c/0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a>`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYS/0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhk09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXm09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000004l000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?clB`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[ZdX0Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFU906UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj6`2OWil00`2m_Kd0k>c/0>OWi`0T0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K6`3:b/X0
0`3Kfm/0Xj>S0?_kn`0T0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo6`3@d=000`3oool0UYJF0?_kn`0T0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3ooolK0<o?c`03
0?ooo`2FUYH0no_k02@0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM400`3@d=070=7Ad@80d=3@0P3AdM430=3@
d0L0dM7A00<0oooo09JFUP3jn_X0903jn_X00`2FUYH0oooo0<k>cP040<k>
cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JFUP3Ph>00403Ph>00501Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0
h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3Ph01YJFT0WYjN0>3Ph02>
SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0
103>c/h303hn?PD0c/k>00@0oooo09JFUP2FUYH0oooo203Fe]H20=OGe`07
09FEU@3Fe]H0UIFE0=KFeP2EUID0e]KF09FEU@050=KFeP80emOG0`3Fe]H0
0`3oool0UYJF0?Win@0T0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n?Sh6
0<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h034a<@0B4Q80=;BdP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0
g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?oo
o`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3Lg=`6
0=gMg@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID70=gMg@80
g=cL00<0gMgM0?ooo`2FUYH09@3inOT00`2FUYH0oooo0<o?c`040<o?c`<0
?Shn1@3?cll00`3oool0UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c
/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3I
fMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV
9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103o
ool0UYJF09JFUP3oool30=cLg0H0gMgM00P0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@L0gMgM0P3Lg=`00`3MgMd0oooo09JFUP0U0?Shn003
09JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0h>3P00P0
h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00G0<C4
a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B
0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2NWYh0
3`3Ph>05000000H0h>3P00@0UYJF0?ooo`3Kfm/0fm_K0`0n?Sh80=_Kf`05
0?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0<0hn?S00T0i>CT0?oo
o`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Ti>@30>?Sh`<0i>CT
00@0hn?S0>CTi03oool0UYJF9@3gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0
?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P
01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV
9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0a<C4
04Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WY
j@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY
00D0j^[Z0>WYj@3YjNT0oooo09JFUP0U0?Gem@0309JFUP3oool0i>CT00D0
i>CT0`0n?Sh40>CTi0030?ooo`2FUYH0h>3P00d0h>3P0P3oool05@3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P01T0SXj>0000000h>3P0fMWI0>3Ph01YJFT0WYjN
0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph02][Jd0
E5AD0>3Ph01YJFT0OWin0>3Ph01YJFT0WYjN0100h>3P00<000000>3Ph03P
h>001P3Ph>000`2FUYH0oooo0>g]k@030>g]k@<0?Shn1P3]kNd01@3oool0
UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WYj@080?ooo`2EUID0oooo
09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY00D0j^[Z0>WYj@3YjNT0
oooo09JFUP0U0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]k@03
0?ooo`2FUYH0h>3P00`0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0
h>3P01T0fMWI059BDP3IfMT0h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P
0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3Bd]80
h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@303hn
?PD0m?Cd00@0oooo09JFUP2FUYH0oooo2@3]kNd01`3oool0kNg]0?ooo`3]
kNd0oooo0>g]k@3oool02`3]kNd00`3oool0UYJF0?7al@0T0?7al@0309JF
UP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2FUYH0h>3P00`0h>3P
0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jN
WP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3lo?`01@3l
o?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?oooa/0kNg]
00<0oooo09JFUP3`l?00903`l?000`2FUYH0oooo0?clo0040?clo00303hn
?P3lo?`0o?cl00D0o?cl00<0oooo09JFUP3Ph>00403Ph>00501YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph00309JFUP3oool0oooo00d0oooo0P2FUYH00`3oool0moOg0?Og
m`0I0?Ogm`030?ooo`2FUYH0kno_02@0kno_00<0UYJF0?ooo`3oool03@3o
ool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?cl
o02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF
0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XK0?ooo`030?[jnP2F
UYH0kno_02@0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF0>3P
h00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F
0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e]KD0
1@2d];@0Shn?0=[JfP3Fe]H0/;2`01/0^;Rh00<0ZJVY0<k>cP3^k^h0903^
k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0f][J0140h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFVG09jNWP030?ooo`3Ph>00
h>3P03D0h>3P4@3/k>`O0>_[jbD0inOW4@3/k>`A0>3Ph00D06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03^k^h0SHf=
00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JF
UP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool01@3>c/h00`0n
?Sh0c/k>0<k>cP040<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool0103>c/h303hn
?PD0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0?ooo`030<o?c`D0?Shn103?cll05P3o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh3
0=CDe00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn0P3K
fm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`030>c/k02FUYH0oooo0080i>CT00<0?Shn0>CTi03Ti>@00`3T
i>@06P0n?Sh0i>CT0>CTi03Ti>@0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?ooo`0<0>g]k@0F
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0
o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3GemL0
303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3J
f]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl00000=P0000002P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooobd0oooo1@1EEED40?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc40
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool02@1hN7P0
00000000002BTY804Q8B0000001PH600oooo0<K6aP03000000040451@@3g
moL0oooo0::RXP80000000D0@D510?Ogm`3oool0oooo0<K6aP0300000003
0451@@3gmoL0oooo0080oooo00<0a/K60000000000000`0000000`2d];@0
gmoO018B4P03000000030;Bd]03oool0oooo0080oooo00<0gmoO018B4P00
00000P0000004P2d];@0oooo0?ooo`3Ogml04Q8B000000092@T0d];B0=oO
g`0B4Q80000006UYJ@0B4Q80@D51071`L0092@T0d];B0<K6aP<0000000<0
@D510?Ogm`3oool0103oool01`36a/H0000000000011@D40moOg0?ooo`36
a/H00`0000003011@D40lo?c05ADE00000002@T90<[:bP0M7Ad000000451
@@3gmoL0oooo0<K6aP80000000L0@D510?Ogm`3oool0oooo0<K6aP000000
];Bd07`0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool0
4P3kno/0E5AD0;Bd]03oool0gmoO024Q8@3_knl0a/K60000002d];@0oooo
0?Ogm`0M7Ad0];Bd0?ooo`3kno/0E5AD0;Bd]0<0oooo00L0a/K60000002d
];@0oooo0?Ogm`0M7Ad0];Bd00D0oooo00<0gmoO01/K6`3Bd]800P3oool0
1`3Ogml08B4Q0>o_k`3oool0gmoO01/K6`3Bd]800`3oool0703Ogml08B4Q
0>o_k`3oool0gmoO01/K6`3Bd]80gmoO018B4P2d];@0oooo0<K6aP092@T0
d];B0::RXP1PH600gmoO01/K6`3Bd]80N7Qh08n?S`36a/H000000;Bd]03o
ool0moOg01dM7@2d];@40?ooo`0C0?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?
0<K6aP000000];Bd0?ooo`3gmoL07AdM0;Bd]03oool0XZ:R0451@@3clo<0
E5AD08n?S`020?ooo`060?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0P3oool0
0`36a/H000000;Bd]01l0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW
0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3j
n_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJD@0oooo00X0no_k05ADE02d];@0oooo0=oOg`0Q8B40kno_07QhN011
@D40moOg1@3oool00`3kno/0E5AD0;Bd]0030?ooo`0307QhN011@D40moOg
00T0oooo00<0gmoO01/K6`3Bd]80103oool01@3gmoL07AdM0000000?3`l0
kno_00D0oooo00L0moOg01dM7@0000003`l?0>o_k`2RXZ80H61P00<0oooo
00`0N7Qh08n?S`2RXZ80H61P0=oOg`0K6a/0d];B07QhN02?Shl0N7Qh0451
@@3gmoL80?ooo`030?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL0
1P3oool0101hN7P0B4Q80451@@3gmoL20?ooo`030?Ogm`0V9RH0d];B08@0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool0203kno/0
E5AD0;Bd]03oool0gmoO024Q8@3_knl0N7Qh1@0000001@2?Shl0oooo0?_k
n`1DE5@0H61P00<0oooo00<0N7Qh0000000000000`0000000`2?Shl0oooo
0?ooo`030?ooo`030=oOg`0K6a/0d];B0080oooo00H0no_k05ADE0000000
000000l?3`3_knl40?ooo`090?_kn`1DE5@000000000000?3`l0kno_0?oo
o`2RXZ80H61P0080oooo00/0no_k05ADE02?Shl0XZ:R061PH03Ogml06a/K
0=;BdP1hN7P0Shn?07QhN0050000000308n?S`3oool0oooo0080oooo00<0
moOg02HV9P3Bd]800`3oool00`1hN7P00000000000030000000608n?S`3o
ool0oooo0::RXP000000Shn?0`3oool00`3gmoL09RHV0=;BdP240?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00<0no_k05ADE000
00000P0000005`0?3`l0kno_0<K6aP092@T0d];B0?ooo`36a/H02@T90=;B
dP3oool0no_k05ADE0000000H61P0<K6aP1PH600a/K600T92@3Bd]80oooo
0<K6aP092@T0d];B00D0oooo00<0gmoO01/K6`3Bd]800P3oool01`3kno/0
E5AD0;Bd]03oool0a/K60451@@3gmoL00`3oool0703kno/0E5AD0;Bd]03o
ool0a/K60451@@3gmoL0gmoO018B4P2d];@0oooo0<K6aP092@T0d];B0::R
XP0?3`l0clo?018B4P2d];@0N7Qh08n?S`36a/H02@T90=;BdP3oool0a/K6
00T92@3Bd]840?ooo`030?Ogm`0V9RH0d];B00<0oooo00d0a/K600T92@3B
d]80oooo0<K6aP092@T0d];B0?ooo`36a/H02@T90<[:bP0M7Ad0];Bd0080
oooo00<0moOg02HV9P3Bd]80Q03oool00`1EEED0oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFU40?ooo`090?_kn`1DE5@0];Bd0?ooo`3Ogml08B4Q0>o_k`3o
ool0a/K600<0000000/0];Bd0?ooo`3Ogml04Q8B0000002/[:`07AdM0000
002d];@0oooo0<K6aP03000000030;Bd]03oool0oooo0080oooo00D0no_k
05ADE00000002@T90=;BdP030?ooo`060=oOg`0B4Q80000000000011@D40
moOg103oool05P3Ogml04Q8B000000000000@D510?Ogm`3oool0gmoO018B
4P0000002@T90=;BdP3Ogml04Q8B05MGE`0M7Ad0/;2`05ADE00?3`l0kno_
0?ooo`36a/H3000000030;Bd]03oool0oooo0080oooo00<0gmoO018B4P00
00000P0000001011@D40moOg0?ooo`36a/H30000000;0;Bd]03oool0XZ:R
000000092@T0/[:b018B4P000000Shn?0=oOg`0B4Q800`0000000`11@D40
moOg0?ooo`200?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0
oooo00L0no_k05ADE02d];@0oooo0=oOg`0Q8B40kno_04T0oooo00<0moOg
02HV9P3Bd]804P3oool00`3gmoL09RHV0=;BdP240?ooo`0305EEE@3oool0
oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3g
moL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJD@0oooo00L0N7Qh000000000000TY:B018B
4P000000H61P01X0oooo00<0XZ:R00l?3`3_knl0;03oool00`3gmoL09RHV
0=;BdP0B0?ooo`030?Ogm`0V9RH0d];B08@0oooo00<0EEEE0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3oool`0?ooo`80EEEE103oool02P3WinL0k^k^
0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3k
no/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3oool_0?ooo`0305EEE@3oool0EEEE00@0oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
;P3oool0101EEED0oooo0?ooo`1EEED40?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobd0
oooo00D0EEEE0?ooo`3oool0oooo05EEE@040?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ob`0oooo1P1EEED40?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?3`
l02>SXh01@2PX:000`2OWil0X:2P0:2PX0040:2PX00F092@T03/k>`0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JF
UP3Ng]h0303Cdm<05P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Ti>@0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03L
g=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT
0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OG
e`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0
gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03ooooT000000`0oooo
1@1EEED40?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Z
j^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3F
e]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0
gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Y
jNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTa0?ooo`050::RXP0000000000
00T92@3Bd]801P3oool00`000000oooo0?ooo`050?ooo`0605ADE0000000
000008n?S`3oool0N7Qh0P0000003`11@D40moOg07QhN00000003`l?0>o_
k`1hN7P000000451@@3gmoL0oooo07QhN00000003`l?0>o_k`030?ooo`08
0::RXP000000@D510<?3``0o?cl0000000l?3`3_knl30?ooo`070<_;b`0o
?cl0000000l?3`3_knl0oooo0::RXP020000000@0;Bd]03oool0N7Qh0000
001PH600N7Qh00000011@D40VYZJ00000011@D40moOg07QhN00000003`l?
0>o_k`<0oooo00/0N7Qh0000001PH600N7Qh00000011@D40VYZJ00000011
@D40moOg0::RXP020000000;0;Bd]03oool0N7Qh00000011@D40moOg0?oo
o`1hN7P0000000l?3`3_knl0103oool0501hN7P000000;Bd]03oool0N7Qh
0000000?3`l0jn_[05ADE01PH600gmoO018B4P11@D40moOg07QhN0000000
];Bd0=oOg`0K6a/0d];BJP3oool00`000000oooo0?ooo`0>0?ooo`0305EE
E@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJC00oooo00H0no_k05eMG@3Bd]80oooo
0::RXP2?Shl60?ooo`030000003oool0oooo00H0oooo00<0N7Qh0;Bd]03o
ool00P3oool04@2RXZ80Shn?0?ooo`36a/H0H61P0?ooo`3gmoL0OGem0?Og
m`0/;2`0kno_0?ooo`36a/H0H61P0?ooo`3gmoL0OGem00<0oooo00P0no_k
05eMG@3Bd]80bl_;0=;BdP3oool0a/K608n?S`<0oooo01T0bl_;0=;BdP3o
ool0a/K608n?S`3Ogml06a/K0=;BdP3kno/0GEeM0=;BdP3gmoL0;2`/0>o_
k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]036a/H0H61P0?ooo`3gmoL0OGem
00<0oooo01P0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0
gmoO01/K6`3Bd]80no_k05eMG@3Bd]80moOg02`/;03_knl0oooo0<K6aP1P
H600oooo0?Ogm`1mOGd30?ooo`0A0?_kn`1MGEd0d];B0?ooo`36a/H0H61P
0?ooo`3gmoL0OGem0=oOg`1bLW80no_k05eMG@3Bd]80no_k05eMG@3Bd]80
KP3oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJC00oooo00H0moOg02`/;02BTY803`l?0;Jf]P1PH6060?ooo`03
0000003oool0oooo00H0oooo00<0N7Qh0;Bd]03oool00P3oool01@2RXZ80
Shn?0?ooo`2RXZ80];Bd00<0oooo00H0moOg02`/;03_knl0oooo0::RXP2d
];@60?ooo`030?_kn`1MGEd0d];B0080oooo00<0XZ:R0000002d];@01@3o
ool01@2RXZ8000000;Bd]02RXZ80H61P0080oooo00d0XZ:R08n?S`3gmoL0
;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]02RXZ80];Bd00H0oooo
00/0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R061P
H0020?ooo`080::RXP2?Shl0moOg02`/;03_knl0oooo0::RXP2d];@60?oo
o`060?_kn`1MGEd0d];B0?ooo`2RXZ80];Bd103oool01`3;bl/0?clo08n?
S`3oool0no_k05eMG@3Bd]80KP3oool00`000000oooo0?ooo`0>0?ooo`03
05EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJC00oooo00H0moOg01dM7@1PH600
moOg01dM7@1PH6060?ooo`030000003oool0oooo00H0oooo00<0N7Qh0;Bd
]03oool00P3oool0102RXZ80Shn?0?ooo`2RXZ8300000006061PH03gmoL0
;2`/0>o_k`3oool0XZ:R0`0000002`1PH600oooo0?ooo`3oool0no_k05eM
G@3Bd]80gmoO018B4P11@D40moOg00@0oooo00L0gmoO018B4P11@D40moOg
0?ooo`2RXZ80Shn?0080oooo00`0N7Qh08n?S`3gmoL0;2`/0>o_k`3kno/0
GEeM0=;BdP3kno/0E5AD0;Bd]02RXZ830000000?061PH03oool0oooo0?oo
o`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]02RXZ80Shn?
0080oooo00L0N7Qh08n?S`3gmoL0;2`/0>o_k`3oool0XZ:R00<0000000T0
H61P0?ooo`3oool0oooo0?_kn`1MGEd0d];B0?ooo`2RXZ800`0000002@1P
H600oooo0<K6aP0?3`l0kno_0?ooo`3kno/0GEeM0=;BdP1^0?ooo`030000
003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY<03o
ool01P3gmoL0;2`/0>o_k`3oool0a/K6061PH0H0oooo00<000000?ooo`3o
ool01P3oool00`1hN7P00000000000030000000@08n?S`3oool0a/K608n?
S`3oool0N7Qh08^;R`1DE5@0000008n?S`3oool0a/K608n?S`3oool0N7Qh
08n?S`<0oooo00P0N7Qh00T92@3Bd]80a/K608n?S`3oool0PH610=;BdP<0
oooo01T0a/K608n?S`3oool0PH610=;BdP3Ogml0De=C0?Ogm`3Ogml04Q8B
0;2`/01DE5@0000008n?S`3gmoL07AdM061PH03kno/0GEeM0=;BdP36a/H0
Shn?0?ooo`1hN7P0Shn?0080oooo01T0no_k05ADE0000000Shn?0?Ogm`0M
7Ad0H61P0?_kn`1MGEd0d];B0=oOg`1CDe<0moOg0=oOg`0B4Q80/;2`05AD
E0000000Shn?0?ooo`36a/H0Shn?0?ooo`1hN7P0Shn?00<0oooo0140fm_K
05eMG@3Bd]80oooo0<K6aP2?Shl0oooo07QhN02?Shl0moOg02`/;03WinL0
Y:BT0?ooo`3Kfm/0GEeM0=;BdP1^0?ooo`030000003oool0oooo00h0oooo
00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=cLg03Lg=`0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY<03oool01P3gmoL07AdM0451
@@3GemL04Q8B061PH0H0oooo00<000000?ooo`3oool01P3oool00`1hN7P0
];Bd0?ooo`020?ooo`070::RXP2?Shl0oooo0?ooo`1hN7P000000451@@02
0?Ogm`0802`/;02BTY802@T90=;BdP1hN7P000000451@@3gmoL30?ooo`08
0?Ogm`0V9RH0d];B0?_kn`1DE5@0000009JFUP3Kfm/30?ooo`070?_kn`1D
E5@0000009JFUP3Kfm/0oooo0::RXP020000000@08n?S`3oool0gmoO05=C
D`2JVYX000000451@@2n_[h000000451@@3gmoL0oooo07QhN0000000@D51
0?Ogm`<0oooo00/0gmoO05=CD`2JVYX000000451@@2n_[h000000451@@3g
moL0oooo0::RXP020000000;08n?S`3oool0moOg02`/;02BTY802@T90=;B
dP1hN7P000000451@@3gmoL00`3oool03`36a/H000000000002d];@0oooo
07QhN0000000@D510??cl`1DE5@0000009:BTP0B4Q80Shn?0<K6aP020000
00030;Bd]03oool0oooo06/0oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID0
6@3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJC00oooo00H0no_k05eMG@3Bd]80oooo0::RXP2?Shl60?ooo`03
0000003oool0oooo00H0oooo00<0N7Qh0;Bd]03oool00P3oool00`2RXZ80
Shn?0?ooo`1D0?ooo`030861P@3Bd]80oooo00/0oooo00<0PH610=;BdP3o
ool0K@3oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080
oooo00D0k>c/09JFUP3oool0moOg0>g]k@060?ooo`0J0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY<@3o
ool01@2RXZ800000000000092@T0d];B00H0oooo00<000000?ooo`3oool0
1P3oool00`1hN7P0];Bd0?ooo`020?ooo`030::RXP2?Shl0oooo05@0oooo
00<0Z:RX0=;BdP3oool02`3oool00`2XZ:P0d];B0?ooo`1]0?ooo`030000
003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@H0UIFE01T0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
00D0oooo00H0E5AD000000000000Shn?0?ooo`1hN7P2000000030451@@3g
moL0oooo00l0oooo00<0moOg02HV9P3Bd]80_P3oool00`000000oooo0?oo
o`0>0?ooo`0305EEE@3oool0oooo0080oooo00D0k>c/09JFUP3oool0moOg
0>g]k@060?ooo`0J0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?oo
o`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/
k>`0UYJF0?ooo`3gmoL20>g]k@H0UIFE01T0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`050>c/k02FUYH0oooo0?Ogm`3]kNd01P3oool06P3Fe]H0dM7A
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JC`0oooo00<000000?ooo`3oool0h03oool00`000000oooo0?ooo`0>0?oo
o`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd6
09FEU@0I0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3P0?ooo`030000003o
ool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?oo
o`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03F
e]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000
003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<0
00000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Z
j^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo
0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?oo
o`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03F
e]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3o
ool02`3oool405EEE@@0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WY
j@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJC`0oooo00<000000?ooo`3oool0
h03oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OG
e`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJC`0ooooi000000<0?ooo`D0EEEE103oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gM
g@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3M
gMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S
0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Og
m`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KF
eP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo;@3oool505EEE@@0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<@3oool0
0`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Z
j^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?oo
o`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT
0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM
0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTl0?ooo`0605ADE0000000000008n?S`3oool0N7Qh0P0000003`11@D40
moOg07QhN00000003`l?0>o_k`1hN7P000000451@@3gmoL0oooo07QhN000
00003`l?0>o_k`030?ooo`080::RXP000000@D510<?3``0o?cl0000000l?
3`3_knl30?ooo`070<_;b`0o?cl0000000l?3`3_knl0oooo0::RXP020000
000@0;Bd]03oool0N7Qh0000001PH600N7Qh00000011@D40VYZJ00000011
@D40moOg07QhN00000003`l?0>o_k`@0oooo01@0N7Qh0000002d];@0oooo
07QhN00000003`l?0>_[j`1DE5@0H61P0=oOg`0B4Q80@D510?Ogm`1hN7P0
00000;Bd]03Ogml06a/K0=;BdYh0oooo00<0EEEE0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cLg03Lg=`0e]KF
0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY?@3oool00`1hN7P0];Bd0?ooo`020?ooo`0A0::RXP2?Shl0oooo
0<K6aP1PH600oooo0?Ogm`1mOGd0moOg02`/;03_knl0oooo0<K6aP1PH600
oooo0?Ogm`1mOGd00`3oool0203kno/0GEeM0=;BdP3;bl/0d];B0?ooo`36
a/H0Shn?0`3oool06@3;bl/0d];B0?ooo`36a/H0Shn?0=oOg`0K6a/0d];B
0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0
];Bd0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool04@3kno/0GEeM0=;BdP3o
ool0a/K6061PH03oool0moOg07emO@3Ogml0LW9b0?_kn`1MGEd0d];B0?_k
n`1MGEd0d];B0:80oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?@3o
ool00`1hN7P0];Bd0?ooo`020?ooo`050::RXP2?Shl0oooo0::RXP2d];@0
0`3oool01P3gmoL0;2`/0>o_k`3oool0XZ:R0;Bd]0H0oooo00<0no_k05eM
G@3Bd]800P3oool00`2RXZ8000000;Bd]0050?ooo`050::RXP000000];Bd
0::RXP1PH6000P3oool03@2RXZ80Shn?0?Ogm`0/;2`0kno_0?_kn`1MGEd0
d];B0?_kn`1DE5@0];Bd0::RXP2d];@01P3oool01P3kno/0GEeM0=;BdP3o
ool0XZ:R0;Bd]0@0oooo00L0bl_;03lo?`2?Shl0oooo0?_kn`1MGEd0d];B
0:80oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0=OGe`0<0?oo
o`0F0=OGe`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTm0?oo
o`0307QhN02d];@0oooo0080oooo00@0XZ:R08n?S`3oool0XZ:R0`000000
1P1PH600moOg02`/;03_knl0oooo0::RXP<0000000/0H61P0?ooo`3oool0
oooo0?_kn`1MGEd0d];B0=oOg`0B4Q80@D510?Ogm`040?ooo`070=oOg`0B
4Q80@D510?Ogm`3oool0XZ:R08n?S`020?ooo`0<07QhN02?Shl0moOg02`/
;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R0`0000002@1PH600
oooo0?ooo`3oool0no_k05eMG@3Bd]80oooo0::RXP0300000009061PH03o
ool0a/K600l?3`3_knl0oooo0?_kn`1MGEd0d];B0:80oooo00<0EEEE0?oo
o`3oool00P3oool00`3/k>`0oOgm09>CT`050;Rh^0030;Fe]@2h^;P0^;Rh
00@0^;Rh01H0Ti>C0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JCd0oooo00<0N7Qh0000000000000`000000402?Shl0oooo0<K6aP2?Shl0
oooo07QhN02;Rh/0E5AD0000002?Shl0oooo0<K6aP2?Shl0oooo07QhN02?
Shl30?ooo`0807QhN0092@T0d];B0<K6aP2?Shl0oooo0861P@3Bd]830?oo
o`0I0<K6aP2?Shl0oooo0861P@3Bd]80gmoO05=CD`3gmoL0gmoO018B4P2`
/;00E5AD0000002?Shl0moOg01dM7@1PH600no_k05eMG@3Bd]80a/K608n?
S`3oool0N7Qh08n?S`030?ooo`0A0=_Kf`1MGEd0d];B0?ooo`36a/H0Shn?
0?ooo`1hN7P0Shn?0?Ogm`0/;2`0inOW0:BTY03oool0fm_K05eMG@3Bd]80
XP3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k03^k^h0SHf=00`0WinO
01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo
00<0N7Qh0;Bd]03oool00P3oool01`2RXZ80Shn?0?ooo`3oool0N7Qh0000
0011@D400P3gmoL0200/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg
0`3oool0203gmoL09RHV0=;BdP3kno/0E5AD0000002FUYH0fm_K0`3oool0
1`3kno/0E5AD0000002FUYH0fm_K0?ooo`2RXZ800P000000402?Shl0oooo
0=oOg`1CDe<0VYZJ00000011@D40_[jn00000011@D40moOg0?ooo`1hN7P0
00000451@@3gmoL30?ooo`0?0<K6aP00000000000;Bd]03oool0N7Qh0000
0011@D40lo?c05ADE0000000TY:B018B4P2?Shl0a/K60080000000<0];Bd
0?ooo`3oool0W`3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0
g]kN00`0fm_K01H0g]kN09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool00`2RXZ80Shn?0?ooo`0i
0?ooo`030861P@3Bd]80oooo00/0oooo00<0PH610=;BdP3oool0X@3oool0
0`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo00<0N7Qh
0;Bd]03oool00P3oool00`2RXZ80Shn?0?ooo`0i0?ooo`030:RXZ03Bd]80
oooo00/0oooo00<0Z:RX0=;BdP3oool0X@3oool00`1EEED0oooo0?ooo`02
0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJC`0oooo00H0E5AD000000000000Shn?0?oo
o`1hN7P2000000030451@@3gmoL0oooo00l0oooo00<0moOg02HV9P3Bd]80
e`3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>
01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
<@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo00`0c/k>
01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
<@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo0080c/k>
00<0?Shn0<k>cP3>c/h00`3>c/h06P0n?Sh0c/k>0<k>cP3>c/h0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo;P3oool405EE
E@@0oooo00@0k>c/09JFUP3oool0clo?0`0n?Sh30<o?c`<0?Shn0P3?cll0
5P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola
0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3oool00P3De=@3
03hn?P040=CDe00n?Sh0?Shn03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo;@3oool505EEE@@0oooo00<0
k>c/09JFUP3oool00`3Kfm/503hn?P@0fm_K01H0oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?oo
o`040>CTi0<0?Shn1@3Ti>@05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00D0kNg]00<0
?Shn0>g]k@3]kNd0103]kNd05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0m?Cd01H0
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3o
ool00`3/k>`0UYJF0?ooo`0<0?clo00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool03@3o
ool05@2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0f][J09:BTP2f
][H:0;Fe]@0G0;Bd]02?Shl0f][J06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000030000003gmoL0
VYZJ0?l0NWYjB@1jNWX04`2JVYX0moOg0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000030000003gmoL0_kno0?l0_kno
BP2o_kl04P3gmoL0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo00000`000000moOg0>?Sh`3o0>?ShdX0hn?S0180moOg0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?7al@3oool0o`3oool30?ooo`030>?Sh`2UYJD0MWIf00<0MWIf00@0
MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0hn?S0:FU
Y@@0MWIf00@0MgMg07IfMP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3o
ool0jn_[0:FUY@@0EUIF00@0EeMG05IFEP1GEeL0EeMG2@1FEUH00`2UYJD0
jn_[0?ooo`030?ooo`0B0?7al@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000040000003QhN40o?cl0?_kn`80o?cl00<0
no_k0?clo03lo?`0103lo?`00`3kno/0o?cl0?clo0020?clo0030?_kn`3l
o?`0no_k00D0o?cl00<0no_k0?clo03kno/0F@3kno^?0?clo0030:FUY@1f
MWH0SHf=0100SHf=00H0MWIf0:FUY@3lo?`0o?cl0:FUY@1fMWHA08f=S@06
07IfMP2UYJD0o?cl0?_kn`2UYJD0EUIF4@1KFe/00`1FEUH0YJFU0?clo003
0?clo00B0>7Qh@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000030000003Ph>00moOg0?l0moOg0`3gmoL00`1fMWH0SHf=
0::RXP0@0::RXP0608f=S@1fMWH0moOg0?Ogm`1fMWH0SHf=4@2RXZ80202=
SHd0MWIf0?Ogm`3gmoL0EUIF05]KF`1_Kfl0LG5a3@1bLW80101aLG40Kfm_
05]KF`1FEUH40?Ogm`0B0>3Ph0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000030000003Ng]h0lO7a0080lO7a0P3bl_80
0`3alO40l_;b0?;blP040?7al@030?;blP3alO40lO7a00<0lO7a0P3bl_80
0`3alO40l_;b0?7al@020?;bl^X0lO7a00<0MWIf0;Rh^02[Zj/0402[Zj/0
1P2h^;P0MWIf0?7al@3alO40MWIf0;Rh^140Zj^[00D0^;Rh07IfMP3alO40
l_;b05IFEP0206m_K`03079bLP1gMgL0MgMg00/0MgMg00@0LW9b06m_K`1_
Kfl0EUIF0`3alO404`3bl_80g]kN0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000050000003Lg=`0jn_[0>_[j`3/k>`0
0P3[jn/00`2WYjL0XZ:R0::RXP020:6QX@030:2PX02OWil0WIfM0080WinO
0P2PX:020:6QX@030:NWY`3[jn/0jn_[00D0jn_[00D0e=CD0<G5a@35aLD0
aLG50=OGe`020>_[j`030>[ZjP38b<P0d=3@0080jn_[00D0f][J0<K6aP3S
hn<0jn_[0>3Ph0020<G5a@070<c<c03YjNT0c/k>0<O7a`3YjNT0jn_[0>3P
h0020<G5a@070<c<c03Zj^X0e=CD0<G5a@3MgMd0jn_[0>3Ph0020<G5a@06
0<K6aP3Shn<0jn_[0>_[j`3Zj^X0b<S80`35aLD00`36a/H0hn?S0>_[j`04
0>_[j`030>[ZjP38b<P0aLG500<0aLG500<0a/K60>?Sh`3[jn/06`3[jn]K
0>c/k3@0jn_[00@0MWIf0;^k^`2a/K40/K6a203oool70;6a/@060;^k^`1f
MWH0jn_[0>_[j`1fMWH0^k^k0P2a/K4<0?ooo`<0/K6a00T0^k^k07IfMP3[
jn/0jn_[05IFEP1gMgL0LW9b07AdM01gMgL02`1hN7P01@1gMgL0M7Ad079b
LP1gMgL0EUIF00@0jn_[0180g=cL0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000<000000=[JfP3UiND00`3UiND00`2:
RXX0Vi^K0:b/[0030>WYj@0:0>[ZjP34a<@0QHF50>c/k03]kNd0k^k^0>OW
i`3`l?00l_;b0:>SX`H0iNGU0140J6QX010@400@4100410@0820P036a/H0
clo?0=kNgP0V9RH0DE5A0<o?c`3UiND0U9BD01DE5@2TY:@0g]kN0;:b/P02
010@40070451@@3Kfm/0D51@01TI6@2j^[X0hn?S0;:b/P02010@40070451
@@3Ogml0L71`010@402BTY80i>CT0:n_[`02010@400601DE5@2j^[X0c<c<
0>CTi03MgMd092@T0`0@41000`0E5AD0Y:BT0=kNgP040>GUi@030=kNgP0V
9RH0410@00<0410@00<05ADE0;jn_P3UiND0Z`3UiND01@1fMWH0`<300;Fe
]@2e]KD0oooo00H0LW9b00<09bLW0;Fe]@2e]KD01@2e]KD01P30`<00MWIf
0>GUi@3UiND0MWIf0<30`080]KFe00<0oooo02LW9`0W9bL02@0W9bL30;Fe
]@050<30`01fMWH0iNGU0>GUi@1FEUH00P1gMgL0101iNGT0NWYj0?ooo`3o
ool707YjNP80oooo00D0NWYj07UiN@1gMgL0MgMg05IFEP040>GUi@0B0=[J
fP000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool0
00030000003IfMT0gmoO00<0gmoO00D0g]kN0:2PX03Zj^X0jNWY0>GUi@02
0>[ZjP0:0=3@d01dM7@0kNg]0>k^kP2][Jd0d=3@0?;blP3clo<0Xj>S0=oO
g`80g]kN03@0gmoO0=kNgP1/K6`05ADE0:^[Z`3Ng]h0`l?301`L701DE5@0
a/K60=KFeP0U9BD0DE5A0<g=c@3Ng]h0Shn?01DE5@2TY:@0eMGE04i>CP0h
>3P0a<C40=gMg@3Kfm/0CTi>01TI6@2j^[X0fMWI04i>CP0h>3P0a<C40=gM
g@3GemL09BDU03Ph>034a<@0`/;201/K6`1ADE40fMWI0<?3``1<C4`0f=SH
0=OGe`0U9BD06ATI0<G5a@3Ng]h0Shn?01DE5@2TY:@0fMWI1@3Ng]h01@2]
[Jd0410@01TI6@2i^KT0dM7A0:h0g]kN00D0MWIf0<?3``2k^k/0^k^k0?oo
o`06079bLP0302LW9`2k^k/0^k^k00D0^k^k00H0`l?307IfMP3Ng]h0g]kN
07IfMP33`l<20;^k^`030?ooo`0W9bL0^k^k00L0^k^k00<0oooo02LW9`2k
^k/00P2k^k/02@33`l<0MWIf0=oOg`3Ng]h0EUIF07MgM`1hN7P0Ng]k030`
<0030?ooo`D0OGem0`3oool01@0`<300Ng]k07QhN01gMgL0EUIF0080g]kN
01@0gmoO0=kNgP3IfMT0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000emOG0=SHf03IfMT20=SHf00@092@T02?
Shl0SXj>0>WYj@21PH40DE5A08N7Q`3Jf]X0YZJV0:VYZ@2/[:`0GemO0?;b
lP3clo<0m?Cd0:BTY0<0f=SH00D0fMWI0=;BdP0T92@0DE5A0<_;b`020=SH
f00P06YZJP0I6AT0^[Zj0=3@d00T92@0DE5A0<_;b`3Hf=P0Rh^;01DE5@2T
Y:@0cLg=02@T900h>3P0`l?30=OGe`3EeMD0C4a<01TI6@2j^[X0d=3@02@T
900h>3P0`l?30=OGe`3AdM4092@T03Ph>033`l<0RXZ:01DE5@2TY:@40<G5
a@0:09jNWP0@4100LG5a0=7Ad@3Hf=P0Rh^;01DE5@2TY:@0e=CD0=;BdP80
aLG500P0a/K60=OGe`3Hf=P0JVYZ010@401bLW80aLG50=CDe1h0f=SHF`3I
fMTd0=SHf00507IfMP37alL0_kno0;no_`3oool01`0W9bL70;no_`060<O7
a`1fMWH0f=SH0=SHf01fMWH0alO70P2o_kl00`3oool09bLW0;no_`070;no
_`030?ooo`0W9bL0_kno0080_kno00D0alO707IfMP3Hf=P0f=SH05IFEP02
07]kN`0307inOP1oOgl0:2PX00<0oooo0`1oOgl30?ooo`0302PX:01oOgl0
OWin0080Ng]k00<0EUIF0=WIf@3Hf=P00P3Hf=P04P3GemL0000009ZJVP2P
X:00Z:RX0;:b/P2l_;`0a/K60=3@d03IfMT0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000eMGE
0=?Cd`030=?Cd`0308F5Q@2SXj<0i^KV0080j^[Z00/0=CDe07moO`3?cll0
_[jn08^;R`1NGUh0F5QH06QXJ02SXj<0bLW90:ZZZP040=?Cd`040<c<c00T
92@0DE5A0<[:bP80dm?C01h0IfMW01TI6@2i^KT0bl_;02@T901ADE40b/[:
0=?Cd`28R8P05ADE0:BTY039bLT092@T03Ph>031`L40dm?C0<o?c`1:BTX0
6ATI0;Vi^@3;bl/092@T03Ph>031`L40dm?C0<c<c00T92@0>3Ph0<71`@1W
IfL5010@400:055AD@2MWId0410@08j>SP3<c<`0dm?C08R8R00E5AD0Y:BT
0:6QX@80410@00T06QXJ0<K6aP3Cdm<0dm?C0<c<c00T92@05ADE0:BTY037
alL0[@3Cdm<00`1fMWH0bl_;0<K6aP0@0<K6aP060<_;b`1fMWH0dm?C0=?C
d`1fMWH0bl_;0P36a/H00`3oool09bLW0<K6aP070<K6aP030?ooo`0W9bL0
a/K60080a/K600L0bl_;07IfMP3Cdm<0dm?C05IFEP1mOGd0Ogmo00<0Ph>3
00D0:2PX0?ooo`3oool0oooo08>3P`030?ooo`0702PX:023Ph<0Ph>308>3
P`1oOgl0OGem05IFEP040=?Cd`0A0=GEe@000000X:2P0:FUY@2][Jd0][Jf
0;no_`38b<P0d];B0=[JfP3Rh^80jNWY0>o_k`3dm?@0m_Kf0?Win@3kno/0
0`3moOd10?knoP40oooo0@3oool000050000003Cdm<0c/k>0<k>cP3?cll0
0P3>c/h03`27QhL0Ti>C0=?Cd`3Kfm/0[Jf]0<71`@2`/;00ZJVY0:ZZZP2=
SHd0TI6A0;2`/03;bl/0kno_0:FUY@030<k>cP050<o?c`37alL08b<S055A
D@38b<P00P3>c/h0;@1UIFD06ATI0;Vi^@37alL08b<S055AD@37alL0c/k>
08F5Q@0E5AD0Y:BT0<O7a`0S8b<0>3Ph0<71`@3>c/h0bl_;04U9B@0I6AT0
^KVi0<O7a`0S8b<0>3Ph0<71`@3>c/h0alO702<S8`0h>3P0`L7108F5Q@0E
5AD0Y:BT0<[:bP37alL08b<S055AD@2a/K406QXJ079bLP35aLD0c/k>08F5
Q@0E5AD0Y:BT0<g=c@070<k>cP050:2PX00@4100>CTi0;no_`3;bl/01@3>
c/h01@3=cLd0alO70<g=c@37alL0b/[:01<0c/k>F`3?clld0<k>cP0307If
MP3>c/h0bl_;0100bl_;00H0c/k>07IfMP3>c/h0c/k>07IfMP3>c/h20<_;
b`030?ooo`0W9bL0bl_;00L0bl_;00<0oooo02LW9`3;bl/00P3;bl/02@3>
c/h0MWIf0<o?c`3>c/h0EUIF07emO@22PX80QHF508B4Q00208F5Q@0302PX
:03oool0oooo00<0oooo00P0:2PX08F5Q@25QHD0Q8B408F5Q@22PX80OGem
05IFEP@0c/k>0140dm?C0000002XZ:P0[Jf]0;>c/`2k^k/0`l?30<c<c03E
eMD0gMgM0>CTi03[jn/0l?3`0?Cdm03gmoL0nOWi0?clo0030?gmo@40o_kn
0@3oool10?ooo`0000<000000=3@d039bLT00`39bLT04@1cLg<0U9BD07in
OP1iNGT0X:2P0=7Ad@2][Jd0YJFU0:JVYP25QHD0GUiN0:f][@3UiND0m_Kf
0?Shn02TY:@0bLW900<0b<S800@0`/;202<S8`1ADE40alO70P38b<P0;01R
HV806ATI0;Vi^@32`/808R8R03Ph>031`L40`/;204I6AP0I6AT0_;bl0<71
`@0S8b<0>3Ph0;no_`35aLD0a<C404I6AP0I6AT0^KVi0<71`@0S8b<0>3Ph
0;no_`35aLD0`<30028R8P0h>3P0`<300:n_[`0J6QX0Lg=c0<G5a@2JVYX0
410@08j>SP35aLD0ATI601TI6@2i^KT0aLG507moO`0E5AD0Y:BT2@38b<P0
101RHV80410@08f=S@37alL40<S8b0060;>c/`15ADD0XJ6Q04a<C01^KVh0
alO74`38b<QK0<W9bC@0b<S800<0MWIf0=?Cd`3=cLd0403=cLd01P3Cdm<0
MWIf0<S8b038b<P0MWIf0=?Cd`80cLg=00<0oooo02LW9`3=cLd01`3=cLd0
0`3oool09bLW0<g=c@020<g=c@070=?Cd`1fMWH0b<S80<S8b01FEUH0PX:2
08>3P`0508N7Q`0502PX:03oool0oooo0?ooo`0X:2P01@27QhL00`23Ph<0
PX:205IFEP030<W9b@0B0<S8b03@d=0000000;:b/P2f][H0^k^k0<;2`P39
bLT0dM7A0=WIf@3Ph>00inOW0>g]k@3alO40mOGe0?Shn03jn_X0o?cl0`3m
oOd10?knoP40oooo0@3oool000030000003?cll0aLG500@0aLG500l0UiNG
0;Ng]`3/k>`0^KVi08^;R`2g]kL0ZZZZ0:BTY01nOWh0LG5a0<C4a03IfMT0
m_Kf0?Win@2UYJD01035aLD0102o_kl08R8R055AD@36a/H20<G5a@06065Q
H@0I6AT0^KVi0;no_`0R8R80GEeM0P0@4100100E5AD0Y:BT0<;2`P15ADD3
010@4006055AD@1QHF40410@01TI6@2f][H0ADE50`0@4100201ADE40_kno
028R8P0h>3P0_kno0<G5a@2/[:`06QXJ0P0@41001@1bLW80aLG50<G5a@32
`/80ADE500<0410@00<05ADE0:BTY036a/H02035aLD00`2/[:`06QXJ055A
D@050<G5a@050;no_`0R8R80<30`01XJ6P2>SXh0X`35aLD00`1fMWH0eMGE
0=7Ad@0@0=7Ad@060=GEe@1fMWH0aLG50<G5a@1fMWH0eMGE0P3AdM400`3o
ool09bLW0=7Ad@070=7Ad@030?ooo`0W9bL0dM7A0080dM7A00L0eMGE07If
MP35aLD0aLG505IFEP22PX80QhN700D0RHV91@3oool508V9R@0308N7Q`22
PX80EUIF00@0aLG50140clo?0000002l_;`0_kno0<?3``39bLT0d=3@0=OG
e`3MgMd0i>CT0>[ZjP3_knl0lo?c0?KfmP3hn?P0no_k0?clo0020?gmo@80
o_kn0P3oool000030000003=cLd0`L7100<0`L710100MGEe09VIV@2WYjL0
kNg]0<W9b@1cLg<0/;2`08f=S@1cLg<0C4a<069RHP2n_[h0]KFe0;:b/P2]
[Jd0ZJVY1031`L40102k^k/08R8R055AD@34a<@20<71`@0405mOG`0I6AT0
^KVi0<;2`PL0`L7100D0^k^k028R8P0h>3P0_[jn0<;2`P020<71`@0<0<?3
``35aLD0_;bl028R8P0h>3P0_[jn0<;2`P2k^k/08R8R03Ph>02n_[h0`/;2
2`31`L40101lO7`05ADE0:BTY033`l<60<71`@80a<C400@0_[jn04A4A00h
>3P0_[jn1031`L401`1OGel0410@010@400@4100>3Ph0;jn_P32`/80X@31
`L400`1fMWH0f=SH0=GEe@0@0=GEe@060=SHf01fMWH0`L710<71`@1fMWH0
f=SH0P3EeMD00`3oool09bLW0?ooo`080?ooo`0602LW9`3EeMD0eMGE0=GE
e@3Hf=P0MWIf0P31`L400`1FEUH0Q8B408R8R00408Z:RP<0oooo00@0:2PX
0?ooo`3oool0oooo102:RXX00`28R8P0Q8B405IFEP040<71`@0@0<g=c@00
0000a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]kNd0lO7a0?Ge
m@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<000000<c<c02j^[X0
0P2m_Kd04@2n_[h0_Kfm09jNWP3Shn<0][Jf09JFUP22PX80Ti>C08B4Q022
PX80CTi>02`/;02LW9`0Zj^[0?KfmP3hn?P0YjNW00@0_Kfm00@0]kNg024Q
8@1ADE40a<C40P2m_Kd0101MGEd06ATI0;Vi^@2o_kl80;fm_@0305eMG@0g
=cL0^;Rh0080_Kfm00/0GEeM01TI6@2i^KT0_kno05eMG@0g=cL0^;Rh0;fm
_@2g]kL08B4Q03Ph>0020;jn_P/0_Kfm00@0NWYj01DE5@2TY:@0_kno102m
_Kd01`2g]kL08B4Q06m_K`35aLD0_kno02<S8`1`L7001@2m_Kd01P1jNWX0
=3@d03`l?01MGEd0>3Ph0;Rh^1<0_KfmF`2n_[hd0;fm_@0307IfMP3Jf]X0
eMGE0100eMGE00H0f][J07IfMP2m_Kd0_Kfm07IfMP3Jf]X20=GEe@030?oo
o`1bLW80LW9b00P0LW9b00h09bLW0=GEe@3EeMD0eMGE0=[JfP1fMWH0_Kfm
0;jn_P1FEUH0QhN708Z:RP2=SHd0SXj>08f=S@<0oooo00<0:2PX08f=S@0X
:2P00`3oool308f=S@0408Z:RP27QhL0EUIF0;jn_P80_Kfm0140^[Zj0<c<
c0000000d=3@0=;BdP3EeMD0fMWI0=gMg@3Rh^80inOW0>c/k03`l?00m?Cd
0?KfmP3hn?P0n_[j0?clo0030?gmo@030?knoP3oool0oooo0040oooo0000
0`000000b/[:0;Rh^0030;Zj^P0@07alO02=SHd0N7Qh09VIV@3`l?00`/;2
0:NWY`3;bl/0_;bl0:>SX`1]KFd0OGem0:b/[03kno/0o?cl0:JVYP<0^[Zj
00@0^k^k0;Bd]00Q8B40CDe=0`2j^[X00`1KFe/06ATI0:n_[`0>0;Zj^P03
05]KF`0I6AT0[jn_00D0^[Zj00@0];Bd024Q8@0g=cL0];Bd302j^[X00`1g
MgL051@D09^KV`060;Zj^P05096AT@0@4100410@010@401=CDd01`2j^[X0
0`2d];@08B4Q06][J`2T0;Zj^P0307IfMP3Hf=P0f][J0100f][J00H0f=SH
07IfMP2j^[X0^[Zj07IfMP3Hf=P20=[JfP030?ooo`1bLW80LW9b00P0LW9b
00d09bLW0=[JfP3Jf]X0f][J0=SHf01fMWH0^[Zj0;^k^`1FEUH0Rh^;08n?
S`2>SXh0Shn?00<0oooo00D0:2PX08n?S`2?Shl0Shn?02PX:0030?ooo`05
08n?S`2>SXh0Shn?08^;R`1FEUH00`2j^[X0402h^;P0b/[:0000003IfMT0
f][J0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3l
o?`30?gmo@80o_kn0`3oool0000300000039bLT0^;Rh00<0^;Rh0100QXJ6
09jNWP3De=@0l?3`0?;blP3>c/h0`L710=gMg@3Bd]80XZ:R0<c<c039bLT0
RHV90?clo03moOd0YjNWk`2h^;P00`1fMWH0emOG0=gMg@0@0=gMg@060=OG
e`1fMWH0^;Rh0;Rh^01fMWH0emOG0`3MgMd;02LW9`<0gMgM00T0emOG07If
MP2h^;P0^;Rh05IFEP2@T900TI6A09>CT`2DU9@00P3oool00`0X:2P0V9RH
09RHV00309RHV00802PX:03oool0oooo09BDU02CTi<0TI6A092@T01FEUH4
0;Rh^00?0<W9b@000000hN7Q0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??c
l`3emOD0moOg0?Win@3kno/0o?cl00<0oOgm00<0o_kn0?ooo`3oool00P3o
ool0000300000039bLT0^;Rh00@0^;Rh00l0QXJ609BDU03bl_80lo?c0:2P
X03Xj>P0m_Kf0=_Kf`2IVIT0n_[j0?_kn`3Lg=`0][Jf0?knoP2WYjL0k`2h
^;P00`1fMWH0e]KF0>7Qh@0@0>7Qh@060=KFeP1fMWH0^;Rh0;Rh^01fMWH0
e]KF4@3QhN402@3Fe]H0MWIf0;Rh^02h^;P0EUIF09FEU@2GUiL0W9bL0::R
XP0202PX:0050:^[Z`2][Jd0Zj^[0:^[Z`2][Jd00P2[Zj/202PX:0050::R
XP2LW9`0UiNG09FEU@1FEUH0102h^;P03P39bLT000000>SXj03YjNT0jn_[
0>g]k@3_knl0lO7a0?Cdm03fm_H0moOg0?Win@3kno/0o?cl0`3moOd20?kn
oP@0oooo00000`000000bLW90;Fe]@030;Fe]@0=07MgM`2PX:00h>3P0??c
l`3UiND0k>c/0?KfmP3hn?P0nOWi092@T03kno/0o?cl0?gmo@020?knoP03
0:NWY`2e]KD0]KFe0>d0]KFe00<0MWIf0>g]k@3YjNT0403YjNT01P3]kNd0
MWIf0;Fe]@2e]KD0MWIf0>g]kA40jNWY00X0kNg]07IfMP2e]KD0]KFe05IF
EP2KVi/0WYjN0:FUY@2b/[80^[Zj0P2o_kl01030`<00_kno0;no_`30`<02
0;no_`070<30`02j^[X0/[:b0:FUY@2NWYh0Vi^K05IFEP040;Fe]@0=0<W9
b@000000k^k^0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0030?gmo@80o_kn1@3oool0000300000039bLT0^[Zj00@0/K6a00<0
SHf=0:ZZZP2ZZZX01@2ZZZX00`2WYjL0ZZZZ0:ZZZP040:ZZZ^l0/K6a00<0
MWIf0=CDe03bl_80403bl_801P3De=@0MWIf0;6a/@2a/K40MWIf0=CDe140
l_;b00H0e=CD07IfMP2a/K40/K6a05IFEP2SXj<20:RXZ0d0`l?30P2XZ:P0
0`2SXj<0EUIF0;6a/@020;6a/@040;Zj^P39bLT000000??cl`80m?Cd00L0
mOGe0?KfmP3gmoL0n?Sh0?[jnP3kno/0o?cl00@0oOgm00<0o_kn0?ooo`3o
ool0103oool0000300000033`l<0bLW90?l0[Zj^0`2^[Zh0102=SHd0MWIf
0<k>cP3kno/=0?ooo`040?_kn`3>c/h0MWIf08f=S@80[Zj^00@0SHf=07If
MP3>c/h0no_k3@3oool0103kno/0c/k>07IfMP2=SHd20:j^[P0609VIV@1F
EUH0]KFe0<_;b`3Ogml0j>SX2@3WinL01P3Xj>P0gmoO0<_;b`2e]KD0EUIF
09VIV@<0[Zj^00<0bLW90<?3``0000000P3fm_H01@3gmoL0n?Sh0?Shn03i
nOT0n_[j0080o?cl103moOd20?knoPH0oooo000010000000`l?30=CDe02g
]kOo0:NWY`<0YjNW00<0SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NW
Y`020:NWY`0308f=S@1fMWH0MWIf00l0MWIf00<0SHf=0:NWY`2WYjL00P2W
YjL00`2=SHd0EUIF05IFEP0?05IFEP0808f=S@2WYjL0YjNW0:NWY`2g]kL0
e=CD0<?3``0000030?Win@040?[jnP3kno/0o?cl0?clo0D0oOgm0P3no_h7
0?ooo`0000H000000:ZZZP3WinL0e=CD0;jn_P2c/k?o0:ZZZT<0ZZZZ00H0
/k>c0;jn_P3De=@0inOW0:ZZZP0000020?_kn`<0o?cl1@3moOd30?knoPP0
oooo00001`000000RXZ:0=WIf@3lo?`0i>CT0=;BdP3<c<`0o`2n_[i10;jn
_P080<c<c03Bd]80i>CT0?clo03IfMT0RXZ:0000003lo?`80?gmo@80o_kn
2P3oool000080000001YJFT0UiNG0=WIf@3lo?`0l_;b0>c/k03Shn?o0=WI
fCl0fMWI00P0hn?S0>c/k03bl_80o?cl0=WIf@2GUiL0JFUY000000L0oOgm
0`3no_h;0?ooo`0000D000000?ooo`1YJFT0UiNG0=?Cd`020?clo?l0oooo
@@3oool20?clo0050=?Cd`2GUiL0JFUY0?ooo`000000103moOd40?knoPd0
oooo00000`000000oooo0?ooo`0206UYJ@0308Z:RP2GUiL0ZZZZ0?l0ZZZZ
@02ZZZX0102GUiL0RXZ:06UYJ@1YJFT20?ooo`030000003no_h0o_kn00@0
o_kn3`3oool00?l00000C`00000E0?ooo`00\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{355, 227},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell["Now we can write the input cell to the notebook.", "Text",
  PageBreakAbove->True,
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(NotebookWrite[nb, 
      Cell[BoxData[
          SuperscriptBox["\<2\>", "\<100\>"]], "\<Input\>"]]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005S0000hb000`400?l0
0000o`00003oo`3ooomT0?ooo`00o`3ooomT0?ooo`003P3ooooo0?knoTL0
o_kn3`3oool000`0oooo103no_ko0?gmoD<0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgmB@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo3d0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k?`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinD00nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_Ho0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clcd0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hm0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj3d0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4m0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH>`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllk0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aC/0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^c/0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a>`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYS/0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhk09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXm09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000004l000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?clB`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[ZdX0Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFU906UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj6`2OWil00`2m_Kd0k>c/0>OWi`0T0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K6`3:b/X0
0`3Kfm/0Xj>S0?_kn`0T0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo6`3@d=000`3oool0UYJF0?_kn`0T0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3ooolK0<o?c`03
0?ooo`2FUYH0no_k02@0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM400`3@d=070=7Ad@80d=3@0P3AdM430=3@
d0L0dM7A00<0oooo09JFUP3jn_X0903jn_X00`2FUYH0oooo0<k>cP040<k>
cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JFUP3Ph>00403Ph>00501Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0
h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3Ph01YJFT0WYjN0>3Ph02>
SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0
103>c/h303hn?PD0c/k>00@0oooo09JFUP2FUYH0oooo203Fe]H20=OGe`07
09FEU@3Fe]H0UIFE0=KFeP2EUID0e]KF09FEU@050=KFeP80emOG0`3Fe]H0
0`3oool0UYJF0?Win@0T0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n?Sh6
0<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h034a<@0B4Q80=;BdP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0
g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?oo
o`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3Lg=`6
0=gMg@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID70=gMg@80
g=cL00<0gMgM0?ooo`2FUYH09@3inOT00`2FUYH0oooo0<o?c`040<o?c`<0
?Shn1@3?cll00`3oool0UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c
/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3I
fMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV
9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103o
ool0UYJF09JFUP3oool30=cLg0H0gMgM00P0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@L0gMgM0P3Lg=`00`3MgMd0oooo09JFUP0U0?Shn003
09JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0h>3P00P0
h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00G0<C4
a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B
0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2NWYh0
3`3Ph>05000000H0h>3P00@0UYJF0?ooo`3Kfm/0fm_K0`0n?Sh80=_Kf`05
0?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0<0hn?S00T0i>CT0?oo
o`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Ti>@30>?Sh`<0i>CT
00@0hn?S0>CTi03oool0UYJF9@3gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0
?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P
01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV
9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0a<C4
04Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WY
j@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY
00D0j^[Z0>WYj@3YjNT0oooo09JFUP0U0?Gem@0309JFUP3oool0i>CT00D0
i>CT0`0n?Sh40>CTi0030?ooo`2FUYH0h>3P00d0h>3P0P3oool05@3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P01T0SXj>0000000h>3P0fMWI0>3Ph01YJFT0WYjN
0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph02][Jd0
E5AD0>3Ph01YJFT0OWin0>3Ph01YJFT0WYjN0100h>3P00<000000>3Ph03P
h>001P3Ph>000`2FUYH0oooo0>g]k@030>g]k@<0?Shn1P3]kNd01@3oool0
UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WYj@080?ooo`2EUID0oooo
09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY00D0j^[Z0>WYj@3YjNT0
oooo09JFUP0U0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]k@03
0?ooo`2FUYH0h>3P00`0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0
h>3P01T0fMWI059BDP3IfMT0h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P
0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3Bd]80
h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@303hn
?PD0m?Cd00@0oooo09JFUP2FUYH0oooo2@3]kNd01`3oool0kNg]0?ooo`3]
kNd0oooo0>g]k@3oool02`3]kNd00`3oool0UYJF0?7al@0T0?7al@0309JF
UP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2FUYH0h>3P00`0h>3P
0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jN
WP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3lo?`01@3l
o?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?oooa/0kNg]
00<0oooo09JFUP3`l?00903`l?000`2FUYH0oooo0?clo0040?clo00303hn
?P3lo?`0o?cl00D0o?cl00<0oooo09JFUP3Ph>00403Ph>00501YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph00309JFUP3oool0oooo00d0oooo0P2FUYH00`3oool0moOg0?Og
m`0I0?Ogm`030?ooo`2FUYH0kno_02@0kno_00<0UYJF0?ooo`3oool03@3o
ool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?cl
o02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF
0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XK0?ooo`030?[jnP2F
UYH0kno_02@0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF0>3P
h00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F
0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e]KD0
1@2d];@0Shn?0=[JfP3Fe]H0/;2`01/0^;Rh00<0ZJVY0<k>cP3^k^h0903^
k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0f][J0140h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFVG09jNWP030?ooo`3Ph>00
h>3P03D0h>3P4@3/k>`O0>_[jbD0inOW4@3/k>`A0>3Ph00D06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03^k^h0SHf=
00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JF
UP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool01@3>c/h00`0n
?Sh0c/k>0<k>cP040<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool0103>c/h303hn
?PD0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0?ooo`030<o?c`D0?Shn103?cll05P3o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh3
0=CDe00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn0P3K
fm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`030>c/k02FUYH0oooo0080i>CT00<0?Shn0>CTi03Ti>@00`3T
i>@06P0n?Sh0i>CT0>CTi03Ti>@0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?ooo`0<0>g]k@0F
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0
o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3GemL0
303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3J
f]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl00000=P0000002P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobd0
oooo1@1EEED40?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYA03oool00`2RXZ80000000000002000000030451@@3gmoL0oooo0>@0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA@3oool00`1hN7P0
3`l?0>o_k`3X0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJDH0
oooo00<0a/K600T92@3Bd]80i`3oool00`1EEED0oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFU70?ooo`030=oOg`0K6a/0d];B0>H0oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYB03oool00`2RXZ80@D510?Ogm`3U0?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00L0gmoO024Q8@3_
knl0oooo0<K6aP11@D40moOg00D0000000<0oooo0000000000000`3oool2
00000=P0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool0
1P3gmoL07AdM0;Bd]03oool0N7Qh061PH0<0oooo00@000000?ooo`3oool0
00000P3oool00`000000oooo000000020?ooo`030000003oool0oooo0=D0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA@3oool01@3Ogml0
4Q8B0000000?3`l0kno_00<0oooo00@000000?ooo`3oool000000P3oool0
0`000000oooo000000020?ooo`030000003oool0oooo0=D0oooo00<0EEEE
0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0
mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYC@3oool010000000oooo0?ooo`00
00020?ooo`030000003oool000000080oooo00<000000?ooo`3oool0e03o
ool205EEE@@0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJDd0oooo00@000000?ooo`3oool00000
0P3oool00`000000oooo000000020?ooo`030000003oool0oooo0=<0oooo
00<0EEEE0?ooo`1EEED0103oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYB`3oool3000000<0oooo
0P0000030?ooo`800000e@3oool0101EEED0oooo0?ooo`1EEED40?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooobd0oooo00D0EEEE0?ooo`3oool0oooo05EEE@040?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooob`0oooo1P1EEED40?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^
kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooobd0oooo1@1EEED40?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
oc40oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?oo
o`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<@3oool00`1E
EED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?oo
o`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYA03oool02@1hN7P000000000002BTY80
4Q8B0000001PH600oooo0<K6aP03000000040451@@3gmoL0oooo0::RXP80
000000D0@D510?Ogm`3oool0oooo0<K6aP03000000030451@@3gmoL0oooo
0080oooo00<0a/K60000000000000`0000000`2d];@0gmoO018B4P030000
00030;Bd]03oool0oooo0080oooo00<0gmoO018B4P0000000P0000004P2d
];@0oooo0?ooo`3Ogml04Q8B000000092@T0d];B0=oOg`0B4Q80000006UY
J@0B4Q80@D51071`L0092@T0d];B0<K6aP<0000000<0@D510?Ogm`3oool0
103oool01`36a/H0000000000011@D40moOg0?ooo`36a/H00`0000003011
@D40lo?c05ADE00000002@T90<[:bP0M7Ad000000451@@3gmoL0oooo0<K6
aP80000000L0@D510?Ogm`3oool0oooo0<K6aP000000];Bd07`0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool04P3kno/0E5AD0;Bd
]03oool0gmoO024Q8@3_knl0a/K60000002d];@0oooo0?Ogm`0M7Ad0];Bd
0?ooo`3kno/0E5AD0;Bd]0<0oooo00L0a/K60000002d];@0oooo0?Ogm`0M
7Ad0];Bd00D0oooo00<0gmoO01/K6`3Bd]800P3oool01`3Ogml08B4Q0>o_
k`3oool0gmoO01/K6`3Bd]800`3oool0703Ogml08B4Q0>o_k`3oool0gmoO
01/K6`3Bd]80gmoO018B4P2d];@0oooo0<K6aP092@T0d];B0::RXP1PH600
gmoO01/K6`3Bd]80N7Qh08n?S`36a/H000000;Bd]03oool0moOg01dM7@2d
];@40?ooo`0C0?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0<K6aP000000];Bd
0?ooo`3gmoL07AdM0;Bd]03oool0XZ:R0451@@3clo<0E5AD08n?S`020?oo
o`060?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0P3oool00`36a/H000000;Bd
]01l0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_
0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3k
no/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00X0
no_k05ADE02d];@0oooo0=oOg`0Q8B40kno_07QhN011@D40moOg1@3oool0
0`3kno/0E5AD0;Bd]0030?ooo`0307QhN011@D40moOg00T0oooo00<0gmoO
01/K6`3Bd]80103oool01@3gmoL07AdM0000000?3`l0kno_00D0oooo00L0
moOg01dM7@0000003`l?0>o_k`2RXZ80H61P00<0oooo00`0N7Qh08n?S`2R
XZ80H61P0=oOg`0K6a/0d];B07QhN02?Shl0N7Qh0451@@3gmoL80?ooo`03
0?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL01P3oool0101hN7P0
B4Q80451@@3gmoL20?ooo`030?Ogm`0V9RH0d];B08@0oooo00<0EEEE0?oo
o`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYA03oool0203kno/0E5AD0;Bd]03oool0
gmoO024Q8@3_knl0N7Qh1@0000001@2?Shl0oooo0?_kn`1DE5@0H61P00<0
oooo00<0N7Qh0000000000000`0000000`2?Shl0oooo0?ooo`030?ooo`03
0=oOg`0K6a/0d];B0080oooo00H0no_k05ADE0000000000000l?3`3_knl4
0?ooo`090?_kn`1DE5@000000000000?3`l0kno_0?ooo`2RXZ80H61P0080
oooo00/0no_k05ADE02?Shl0XZ:R061PH03Ogml06a/K0=;BdP1hN7P0Shn?
07QhN0050000000308n?S`3oool0oooo0080oooo00<0moOg02HV9P3Bd]80
0`3oool00`1hN7P00000000000030000000608n?S`3oool0oooo0::RXP00
0000Shn?0`3oool00`3gmoL09RHV0=;BdP240?ooo`0305EEE@3oool0oooo
0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJD@0oooo00<0no_k05ADE00000000P0000005`0?
3`l0kno_0<K6aP092@T0d];B0?ooo`36a/H02@T90=;BdP3oool0no_k05AD
E0000000H61P0<K6aP1PH600a/K600T92@3Bd]80oooo0<K6aP092@T0d];B
00D0oooo00<0gmoO01/K6`3Bd]800P3oool01`3kno/0E5AD0;Bd]03oool0
a/K60451@@3gmoL00`3oool0703kno/0E5AD0;Bd]03oool0a/K60451@@3g
moL0gmoO018B4P2d];@0oooo0<K6aP092@T0d];B0::RXP0?3`l0clo?018B
4P2d];@0N7Qh08n?S`36a/H02@T90=;BdP3oool0a/K600T92@3Bd]840?oo
o`030?Ogm`0V9RH0d];B00<0oooo00d0a/K600T92@3Bd]80oooo0<K6aP09
2@T0d];B0?ooo`36a/H02@T90<[:bP0M7Ad0];Bd0080oooo00<0moOg02HV
9P3Bd]80Q03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU40?oo
o`090?_kn`1DE5@0];Bd0?ooo`3Ogml08B4Q0>o_k`3oool0a/K600<00000
00/0];Bd0?ooo`3Ogml04Q8B0000002/[:`07AdM0000002d];@0oooo0<K6
aP03000000030;Bd]03oool0oooo0080oooo00D0no_k05ADE00000002@T9
0=;BdP030?ooo`060=oOg`0B4Q80000000000011@D40moOg103oool05P3O
gml04Q8B000000000000@D510?Ogm`3oool0gmoO018B4P0000002@T90=;B
dP3Ogml04Q8B05MGE`0M7Ad0/;2`05ADE00?3`l0kno_0?ooo`36a/H30000
00030;Bd]03oool0oooo0080oooo00<0gmoO018B4P0000000P0000001011
@D40moOg0?ooo`36a/H30000000;0;Bd]03oool0XZ:R000000092@T0/[:b
018B4P000000Shn?0=oOg`0B4Q800`0000000`11@D40moOg0?ooo`200?oo
o`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00L0no_k05AD
E02d];@0oooo0=oOg`0Q8B40kno_04T0oooo00<0moOg02HV9P3Bd]804P3o
ool00`3gmoL09RHV0=;BdP240?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJD@0oooo00L0N7Qh000000000000TY:B018B4P000000H61P01X0
oooo00<0XZ:R00l?3`3_knl0;03oool00`3gmoL09RHV0=;BdP0B0?ooo`03
0?Ogm`0V9RH0d];B08@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3oool`0?ooo`80EEEE103oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool_0?ooo`03
05EEE@3oool0EEEE00@0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo;P3oool0101EEED0
oooo0?ooo`1EEED40?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobd0oooo00D0EEEE0?oo
o`3oool0oooo05EEE@040?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooob`0oooo1P1EEED4
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?3`l02>SXh01@2PX:00
0`2OWil0X:2P0:2PX0040:2PX00F092@T03/k>`0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3Ng]h0303Cdm<0
5P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gM
g@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0
gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3M
gMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S
0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?03ooooT000000`0oooo1@1EEED40?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM
0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM
0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL
0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTa0?ooo`050::RXP000000000000T92@3Bd]801P3o
ool00`000000oooo0?ooo`050?ooo`0605ADE0000000000008n?S`3oool0
N7Qh0P0000003`11@D40moOg07QhN00000003`l?0>o_k`1hN7P000000451
@@3gmoL0oooo07QhN00000003`l?0>o_k`030?ooo`080::RXP000000@D51
0<?3``0o?cl0000000l?3`3_knl30?ooo`070<_;b`0o?cl0000000l?3`3_
knl0oooo0::RXP020000000@0;Bd]03oool0N7Qh0000001PH600N7Qh0000
0011@D40VYZJ00000011@D40moOg07QhN00000003`l?0>o_k`<0oooo00/0
N7Qh0000001PH600N7Qh00000011@D40VYZJ00000011@D40moOg0::RXP02
0000000;0;Bd]03oool0N7Qh00000011@D40moOg0?ooo`1hN7P0000000l?
3`3_knl0103oool0501hN7P000000;Bd]03oool0N7Qh0000000?3`l0jn_[
05ADE01PH600gmoO018B4P11@D40moOg07QhN0000000];Bd0=oOg`0K6a/0
d];BJP3oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL
0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJC00oooo00H0no_k05eMG@3Bd]80oooo0::RXP2?Shl60?oo
o`030000003oool0oooo00H0oooo00<0N7Qh0;Bd]03oool00P3oool04@2R
XZ80Shn?0?ooo`36a/H0H61P0?ooo`3gmoL0OGem0?Ogm`0/;2`0kno_0?oo
o`36a/H0H61P0?ooo`3gmoL0OGem00<0oooo00P0no_k05eMG@3Bd]80bl_;
0=;BdP3oool0a/K608n?S`<0oooo01T0bl_;0=;BdP3oool0a/K608n?S`3O
gml06a/K0=;BdP3kno/0GEeM0=;BdP3gmoL0;2`/0>o_k`3kno/0GEeM0=;B
dP3kno/0E5AD0;Bd]036a/H0H61P0?ooo`3gmoL0OGem00<0oooo01P0moOg
02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0gmoO01/K6`3Bd]80
no_k05eMG@3Bd]80moOg02`/;03_knl0oooo0<K6aP1PH600oooo0?Ogm`1m
OGd30?ooo`0A0?_kn`1MGEd0d];B0?ooo`36a/H0H61P0?ooo`3gmoL0OGem
0=oOg`1bLW80no_k05eMG@3Bd]80no_k05eMG@3Bd]80KP3oool00`000000
oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd609FEU@0I0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY<03oool01P3gmoL0;2`/09:BTP0?
3`l0][Jf061PH0H0oooo00<000000?ooo`3oool01P3oool00`1hN7P0];Bd
0?ooo`020?ooo`050::RXP2?Shl0oooo0::RXP2d];@00`3oool01P3gmoL0
;2`/0>o_k`3oool0XZ:R0;Bd]0H0oooo00<0no_k05eMG@3Bd]800P3oool0
0`2RXZ8000000;Bd]0050?ooo`050::RXP000000];Bd0::RXP1PH6000P3o
ool03@2RXZ80Shn?0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0
];Bd0::RXP2d];@01P3oool02`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3k
no/0E5AD0;Bd]02RXZ80H61P0080oooo00P0XZ:R08n?S`3gmoL0;2`/0>o_
k`3oool0XZ:R0;Bd]0H0oooo00H0no_k05eMG@3Bd]80oooo0::RXP2d];@4
0?ooo`070<_;b`0o?cl0Shn?0?ooo`3kno/0GEeM0=;BdP1^0?ooo`030000
003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF
0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFT`0?ooo`060?Ogm`0M7Ad0
H61P0?Ogm`0M7Ad0H61P1P3oool00`000000oooo0?ooo`060?ooo`0307Qh
N02d];@0oooo0080oooo00@0XZ:R08n?S`3oool0XZ:R0`0000001P1PH600
moOg02`/;03_knl0oooo0::RXP<0000000/0H61P0?ooo`3oool0oooo0?_k
n`1MGEd0d];B0=oOg`0B4Q80@D510?Ogm`040?ooo`070=oOg`0B4Q80@D51
0?Ogm`3oool0XZ:R08n?S`020?ooo`0<07QhN02?Shl0moOg02`/;03_knl0
no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R0`0000003`1PH600oooo0?oo
o`3oool0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R
08n?S`020?ooo`0707QhN02?Shl0moOg02`/;03_knl0oooo0::RXP030000
0009061PH03oool0oooo0?ooo`3kno/0GEeM0=;BdP3oool0XZ:R00<00000
00T0H61P0?ooo`36a/H03`l?0>o_k`3oool0no_k05eMG@3Bd]80KP3oool0
0`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd609FEU@0I0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY<03oool01P3gmoL0;2`/
0>o_k`3oool0a/K6061PH0H0oooo00<000000?ooo`3oool01P3oool00`1h
N7P00000000000030000000@08n?S`3oool0a/K608n?S`3oool0N7Qh08^;
R`1DE5@0000008n?S`3oool0a/K608n?S`3oool0N7Qh08n?S`<0oooo00P0
N7Qh00T92@3Bd]80a/K608n?S`3oool0PH610=;BdP<0oooo01T0a/K608n?
S`3oool0PH610=;BdP3Ogml0De=C0?Ogm`3Ogml04Q8B0;2`/01DE5@00000
08n?S`3gmoL07AdM061PH03kno/0GEeM0=;BdP36a/H0Shn?0?ooo`1hN7P0
Shn?0080oooo01T0no_k05ADE0000000Shn?0?Ogm`0M7Ad0H61P0?_kn`1M
GEd0d];B0=oOg`1CDe<0moOg0=oOg`0B4Q80/;2`05ADE0000000Shn?0?oo
o`36a/H0Shn?0?ooo`1hN7P0Shn?00<0oooo0140fm_K05eMG@3Bd]80oooo
0<K6aP2?Shl0oooo07QhN02?Shl0moOg02`/;03WinL0Y:BT0?ooo`3Kfm/0
GEeM0=;BdP1^0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3o
ool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7A
d@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFT`0?ooo`060?Ogm`0M7Ad0@D510=OGe`0B4Q80H61P1P3oool00`000000
oooo0?ooo`060?ooo`0307QhN02d];@0oooo0080oooo00L0XZ:R08n?S`3o
ool0oooo07QhN0000000@D510080moOg00P0;2`/09:BTP092@T0d];B07Qh
N0000000@D510?Ogm`<0oooo00P0moOg02HV9P3Bd]80no_k05ADE0000000
UYJF0=_Kf`<0oooo00L0no_k05ADE0000000UYJF0=_Kf`3oool0XZ:R0080
00000100Shn?0?ooo`3Ogml0De=C09ZJVP000000@D510;jn_P000000@D51
0?Ogm`3oool0N7Qh00000011@D40moOg0`3oool02`3Ogml0De=C09ZJVP00
0000@D510;jn_P000000@D510?Ogm`3oool0XZ:R0080000000/0Shn?0?oo
o`3gmoL0;2`/09:BTP092@T0d];B07QhN0000000@D510?Ogm`030?ooo`0?
0<K6aP00000000000;Bd]03oool0N7Qh00000011@D40lo?c05ADE0000000
TY:B018B4P2?Shl0a/K60080000000<0];Bd0?ooo`3oool0J`3oool00`00
0000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd609FEU@0I0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY<03oool01P3kno/0GEeM0=;B
dP3oool0XZ:R08n?S`H0oooo00<000000?ooo`3oool01P3oool00`1hN7P0
];Bd0?ooo`020?ooo`030::RXP2?Shl0oooo05@0oooo00<0PH610=;BdP3o
ool02`3oool00`21PH40d];B0?ooo`1]0?ooo`030000003oool0oooo00h0
oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]
00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTa0?ooo`050::RXP000000000000T92@3Bd]80
1P3oool00`000000oooo0?ooo`060?ooo`0307QhN02d];@0oooo0080oooo
00<0XZ:R08n?S`3oool0E03oool00`2XZ:P0d];B0?ooo`0;0?ooo`030:RX
Z03Bd]80oooo06d0oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JC`0oooo00<000000?ooo`3oool01@3oool01P1DE5@000000000002?Shl0
oooo07QhN080000000<0@D510?Ogm`3oool03`3oool00`3gmoL09RHV0=;B
dP2n0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3o
ool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL
0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM
0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool02`3oool405EE
E@@0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0
gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJC`0oooo00<000000?ooo`3oool0h03oool00`000000
oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo
i000000<0?ooo`D0EEEE103oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3M
gMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gM
g@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3@d=00clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0oOgm09>CT`050;Rh^0030;Fe]@2h^;P0^;Rh00@0^;Rh01H0Ti>C0>c/
k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo;@3oool505EE
E@@0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/0>k^
kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JF
UP3Ng]h0303Kfm/05P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY?03oool01P1DE5@000000000002?Shl0oooo07QhN080000000l0
@D510?Ogm`1hN7P0000000l?3`3_knl0N7Qh00000011@D40moOg0?ooo`1h
N7P0000000l?3`3_knl00`3oool0202RXZ8000000451@@33`l<0?clo0000
000?3`l0kno_0`3oool01`3;bl/0?clo0000000?3`l0kno_0?ooo`2RXZ80
0P000000402d];@0oooo07QhN0000000H61P07QhN0000000@D5109ZJVP00
0000@D510?Ogm`1hN7P0000000l?3`3_knl40?ooo`0D07QhN0000000];Bd
0?ooo`1hN7P0000000l?3`3[jn/0E5AD061PH03Ogml04Q8B0451@@3gmoL0
N7Qh0000002d];@0gmoO01/K6`3Bd]:N0?ooo`0305EEE@3oool0oooo0080
oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?@3oool00`1hN7P0];Bd0?ooo`020?ooo`0A
0::RXP2?Shl0oooo0<K6aP1PH600oooo0?Ogm`1mOGd0moOg02`/;03_knl0
oooo0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool0203kno/0GEeM0=;BdP3;
bl/0d];B0?ooo`36a/H0Shn?0`3oool06@3;bl/0d];B0?ooo`36a/H0Shn?
0=oOg`0K6a/0d];B0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?_kn`1MGEd0
d];B0?_kn`1DE5@0];Bd0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool04@3k
no/0GEeM0=;BdP3oool0a/K6061PH03oool0moOg07emO@3Ogml0LW9b0?_k
n`1MGEd0d];B0?_kn`1MGEd0d];B0:80oooo00<0EEEE0?ooo`3oool00P3o
ool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTm0?ooo`0307QhN02d];@0oooo0080oooo00D0
XZ:R08n?S`3oool0XZ:R0;Bd]0030?ooo`060?Ogm`0/;2`0kno_0?ooo`2R
XZ80];Bd1P3oool00`3kno/0GEeM0=;BdP020?ooo`030::RXP000000];Bd
00D0oooo00D0XZ:R0000002d];@0XZ:R061PH0020?ooo`0=0::RXP2?Shl0
moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R0;Bd]006
0?ooo`060?_kn`1MGEd0d];B0?ooo`2RXZ80];Bd103oool01`3;bl/0?clo
08n?S`3oool0no_k05eMG@3Bd]80XP3oool00`1EEED0oooo0?ooo`020?oo
o`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool0102R
XZ80Shn?0?ooo`2RXZ8300000006061PH03gmoL0;2`/0>o_k`3oool0XZ:R
0`0000002`1PH600oooo0?ooo`3oool0no_k05eMG@3Bd]80gmoO018B4P11
@D40moOg00@0oooo00L0gmoO018B4P11@D40moOg0?ooo`2RXZ80Shn?0080
oooo00`0N7Qh08n?S`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD
0;Bd]02RXZ8300000009061PH03oool0oooo0?ooo`3kno/0GEeM0=;BdP3o
ool0XZ:R00<0000000T0H61P0?ooo`36a/H03`l?0>o_k`3oool0no_k05eM
G@3Bd]80XP3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo
00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JCd0oooo00<0N7Qh0000000000000`000000402?Shl0oooo0<K6aP2?Shl0
oooo07QhN02;Rh/0E5AD0000002?Shl0oooo0<K6aP2?Shl0oooo07QhN02?
Shl30?ooo`0807QhN0092@T0d];B0<K6aP2?Shl0oooo0861P@3Bd]830?oo
o`0I0<K6aP2?Shl0oooo0861P@3Bd]80gmoO05=CD`3gmoL0gmoO018B4P2`
/;00E5AD0000002?Shl0moOg01dM7@1PH600no_k05eMG@3Bd]80a/K608n?
S`3oool0N7Qh08n?S`030?ooo`0A0=_Kf`1MGEd0d];B0?ooo`36a/H0Shn?
0?ooo`1hN7P0Shn?0?Ogm`0/;2`0inOW0:BTY03oool0fm_K05eMG@3Bd]80
XP3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo0080c/k>
00<0?Shn0<k>cP3>c/h00`3>c/h06P0n?Sh0c/k>0<k>cP3>c/h0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo00<0N7Qh0;Bd
]03oool00P3oool01`2RXZ80Shn?0?ooo`3oool0N7Qh00000011@D400P3g
moL0200/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg0`3oool0203g
moL09RHV0=;BdP3kno/0E5AD0000002FUYH0fm_K0`3oool01`3kno/0E5AD
0000002FUYH0fm_K0?ooo`2RXZ800P000000402?Shl0oooo0=oOg`1CDe<0
VYZJ00000011@D40_[jn00000011@D40moOg0?ooo`1hN7P000000451@@3g
moL30?ooo`0?0<K6aP00000000000;Bd]03oool0N7Qh00000011@D40lo?c
05ADE0000000TY:B018B4P2?Shl0a/K60080000000<0];Bd0?ooo`3oool0
W`3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0<o?c`<0
?Shn0`3?cll303hn?P80clo?01H0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool00`2RXZ80
Shn?0?ooo`0i0?ooo`030861P@3Bd]80oooo00/0oooo00<0PH610=;BdP3o
ool0X@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo0080
e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh30=CDe00F0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTm0?ooo`0307QhN02d];@0oooo
0080oooo00<0XZ:R08n?S`3oool0>@3oool00`2XZ:P0d];B0?ooo`0;0?oo
o`030:RXZ03Bd]80oooo0:40oooo00<0EEEE0?ooo`3oool00P3oool00`3/
k>`0UYJF0?ooo`030=_Kf`D0?Shn103Kfm/05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUY?03oool01P1DE5@000000000002?Shl0
oooo07QhN080000000<0@D510?Ogm`3oool03`3oool00`3gmoL09RHV0=;B
dP3G0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3oool0103T
i>@303hn?PD0i>CT01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02F
UYH0oooo00D0kNg]00<0?Shn0>g]k@3]kNd0103]kNd05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?ooo`0305EEE@3o
ool0oooo0080oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool^0?ooo`@0EEEE103o
ool00`3/k>`0UYJF0?ooo`0<0?clo00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3o
ool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooobd0oooo1@1EEED40?ooo`030>c/k02FUYH0
emOG00`0oooo01H0emOG09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool0103/k>`0f][J09:BTP2f][H:0;Fe]@0G0;Bd]02?
Shl0f][J06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000030000003gmoL0VYZJ0?l0NWYjB@1jNWX0
4`2JVYX0moOg0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000030000003gmoL0_kno0?l0_knoBP2o_kl04P3gmoL00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`00
0000moOg0>?Sh`3o0>?ShdX0hn?S0180moOg0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000?7al@3oool0o`3o
ool30?ooo`030>?Sh`2UYJD0MWIf00<0MWIf00@0MgMg07IfMP1gMgL0MgMg
2@1fMWH01P2UYJD0hn?S0?ooo`3oool0hn?S0:FUY@@0MWIf00@0MgMg07If
MP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0jn_[0:FUY@@0EUIF
00@0EeMG05IFEP1GEeL0EeMG2@1FEUH00`2UYJD0jn_[0?ooo`030?ooo`0B
0?7al@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003QhN40o?cl0?_kn`80o?cl00<0no_k0?clo03lo?`0103l
o?`00`3kno/0o?cl0?clo0020?clo0030?_kn`3lo?`0no_k00D0o?cl00<0
no_k0?clo03kno/0F@3kno^?0?clo0030:FUY@1fMWH0SHf=0100SHf=00H0
MWIf0:FUY@3lo?`0o?cl0:FUY@1fMWHA08f=S@0607IfMP2UYJD0o?cl0?_k
n`2UYJD0EUIF4@1KFe/00`1FEUH0YJFU0?clo0030?clo00B0>7Qh@000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000
003Ph>00moOg0?l0moOg0`3gmoL00`1fMWH0SHf=0::RXP0@0::RXP0608f=
S@1fMWH0moOg0?Ogm`1fMWH0SHf=4@2RXZ80202=SHd0MWIf0?Ogm`3gmoL0
EUIF05]KF`1_Kfl0LG5a3@1bLW80101aLG40Kfm_05]KF`1FEUH40?Ogm`0B
0>3Ph0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000030000003Ng]h0lO7a0080lO7a0P3bl_800`3alO40l_;b0?;blP04
0?7al@030?;blP3alO40lO7a00<0lO7a0P3bl_800`3alO40l_;b0?7al@02
0?;bl^X0lO7a00<0MWIf0;Rh^02[Zj/0402[Zj/01P2h^;P0MWIf0?7al@3a
lO40MWIf0;Rh^140Zj^[00D0^;Rh07IfMP3alO40l_;b05IFEP0206m_K`03
079bLP1gMgL0MgMg00/0MgMg00@0LW9b06m_K`1_Kfl0EUIF0`3alO404`3b
l_80g]kN0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000050000003Lg=`0jn_[0>_[j`3/k>`00P3[jn/00`2WYjL0XZ:R
0::RXP020:6QX@030:2PX02OWil0WIfM0080WinO0P2PX:020:6QX@030:NW
Y`3[jn/0jn_[00D0jn_[00D0e=CD0<G5a@35aLD0aLG50=OGe`020>_[j`03
0>[ZjP38b<P0d=3@0080jn_[00D0f][J0<K6aP3Shn<0jn_[0>3Ph0020<G5
a@070<c<c03YjNT0c/k>0<O7a`3YjNT0jn_[0>3Ph0020<G5a@070<c<c03Z
j^X0e=CD0<G5a@3MgMd0jn_[0>3Ph0020<G5a@060<K6aP3Shn<0jn_[0>_[
j`3Zj^X0b<S80`35aLD00`36a/H0hn?S0>_[j`040>_[j`030>[ZjP38b<P0
aLG500<0aLG500<0a/K60>?Sh`3[jn/06`3[jn]K0>c/k3@0jn_[00@0MWIf
0;^k^`2a/K40/K6a203oool70;6a/@060;^k^`1fMWH0jn_[0>_[j`1fMWH0
^k^k0P2a/K4<0?ooo`<0/K6a00T0^k^k07IfMP3[jn/0jn_[05IFEP1gMgL0
LW9b07AdM01gMgL02`1hN7P01@1gMgL0M7Ad079bLP1gMgL0EUIF00@0jn_[
0180g=cL0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000<000000=[JfP3UiND00`3UiND00`2:RXX0Vi^K0:b/[0030>WY
j@0:0>[ZjP34a<@0QHF50>c/k03]kNd0k^k^0>OWi`3`l?00l_;b0:>SX`H0
iNGU0140J6QX010@400@4100410@0820P036a/H0clo?0=kNgP0V9RH0DE5A
0<o?c`3UiND0U9BD01DE5@2TY:@0g]kN0;:b/P02010@40070451@@3Kfm/0
D51@01TI6@2j^[X0hn?S0;:b/P02010@40070451@@3Ogml0L71`010@402B
TY80i>CT0:n_[`02010@400601DE5@2j^[X0c<c<0>CTi03MgMd092@T0`0@
41000`0E5AD0Y:BT0=kNgP040>GUi@030=kNgP0V9RH0410@00<0410@00<0
5ADE0;jn_P3UiND0Z`3UiND01@1fMWH0`<300;Fe]@2e]KD0oooo00H0LW9b
00<09bLW0;Fe]@2e]KD01@2e]KD01P30`<00MWIf0>GUi@3UiND0MWIf0<30
`080]KFe00<0oooo02LW9`0W9bL02@0W9bL30;Fe]@050<30`01fMWH0iNGU
0>GUi@1FEUH00P1gMgL0101iNGT0NWYj0?ooo`3oool707YjNP80oooo00D0
NWYj07UiN@1gMgL0MgMg05IFEP040>GUi@0B0=[JfP000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003IfMT0gmoO
00<0gmoO00D0g]kN0:2PX03Zj^X0jNWY0>GUi@020>[ZjP0:0=3@d01dM7@0
kNg]0>k^kP2][Jd0d=3@0?;blP3clo<0Xj>S0=oOg`80g]kN03@0gmoO0=kN
gP1/K6`05ADE0:^[Z`3Ng]h0`l?301`L701DE5@0a/K60=KFeP0U9BD0DE5A
0<g=c@3Ng]h0Shn?01DE5@2TY:@0eMGE04i>CP0h>3P0a<C40=gMg@3Kfm/0
CTi>01TI6@2j^[X0fMWI04i>CP0h>3P0a<C40=gMg@3GemL09BDU03Ph>034
a<@0`/;201/K6`1ADE40fMWI0<?3``1<C4`0f=SH0=OGe`0U9BD06ATI0<G5
a@3Ng]h0Shn?01DE5@2TY:@0fMWI1@3Ng]h01@2][Jd0410@01TI6@2i^KT0
dM7A0:h0g]kN00D0MWIf0<?3``2k^k/0^k^k0?ooo`06079bLP0302LW9`2k
^k/0^k^k00D0^k^k00H0`l?307IfMP3Ng]h0g]kN07IfMP33`l<20;^k^`03
0?ooo`0W9bL0^k^k00L0^k^k00<0oooo02LW9`2k^k/00P2k^k/02@33`l<0
MWIf0=oOg`3Ng]h0EUIF07MgM`1hN7P0Ng]k030`<0030?ooo`D0OGem0`3o
ool01@0`<300Ng]k07QhN01gMgL0EUIF0080g]kN01@0gmoO0=kNgP3IfMT0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000emOG0=SHf03IfMT20=SHf00@092@T02?Shl0SXj>0>WYj@21PH40
DE5A08N7Q`3Jf]X0YZJV0:VYZ@2/[:`0GemO0?;blP3clo<0m?Cd0:BTY0<0
f=SH00D0fMWI0=;BdP0T92@0DE5A0<_;b`020=SHf00P06YZJP0I6AT0^[Zj
0=3@d00T92@0DE5A0<_;b`3Hf=P0Rh^;01DE5@2TY:@0cLg=02@T900h>3P0
`l?30=OGe`3EeMD0C4a<01TI6@2j^[X0d=3@02@T900h>3P0`l?30=OGe`3A
dM4092@T03Ph>033`l<0RXZ:01DE5@2TY:@40<G5a@0:09jNWP0@4100LG5a
0=7Ad@3Hf=P0Rh^;01DE5@2TY:@0e=CD0=;BdP80aLG500P0a/K60=OGe`3H
f=P0JVYZ010@401bLW80aLG50=CDe1h0f=SHF`3IfMTd0=SHf00507IfMP37
alL0_kno0;no_`3oool01`0W9bL70;no_`060<O7a`1fMWH0f=SH0=SHf01f
MWH0alO70P2o_kl00`3oool09bLW0;no_`070;no_`030?ooo`0W9bL0_kno
0080_kno00D0alO707IfMP3Hf=P0f=SH05IFEP0207]kN`0307inOP1oOgl0
:2PX00<0oooo0`1oOgl30?ooo`0302PX:01oOgl0OWin0080Ng]k00<0EUIF
0=WIf@3Hf=P00P3Hf=P04P3GemL0000009ZJVP2PX:00Z:RX0;:b/P2l_;`0
a/K60=3@d03IfMT0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo00000`000000eMGE0=?Cd`030=?Cd`0308F5
Q@2SXj<0i^KV0080j^[Z00/0=CDe07moO`3?cll0_[jn08^;R`1NGUh0F5QH
06QXJ02SXj<0bLW90:ZZZP040=?Cd`040<c<c00T92@0DE5A0<[:bP80dm?C
01h0IfMW01TI6@2i^KT0bl_;02@T901ADE40b/[:0=?Cd`28R8P05ADE0:BT
Y039bLT092@T03Ph>031`L40dm?C0<o?c`1:BTX06ATI0;Vi^@3;bl/092@T
03Ph>031`L40dm?C0<c<c00T92@0>3Ph0<71`@1WIfL5010@400:055AD@2M
WId0410@08j>SP3<c<`0dm?C08R8R00E5AD0Y:BT0:6QX@80410@00T06QXJ
0<K6aP3Cdm<0dm?C0<c<c00T92@05ADE0:BTY037alL0[@3Cdm<00`1fMWH0
bl_;0<K6aP0@0<K6aP060<_;b`1fMWH0dm?C0=?Cd`1fMWH0bl_;0P36a/H0
0`3oool09bLW0<K6aP070<K6aP030?ooo`0W9bL0a/K60080a/K600L0bl_;
07IfMP3Cdm<0dm?C05IFEP1mOGd0Ogmo00<0Ph>300D0:2PX0?ooo`3oool0
oooo08>3P`030?ooo`0702PX:023Ph<0Ph>308>3P`1oOgl0OGem05IFEP04
0=?Cd`0A0=GEe@000000X:2P0:FUY@2][Jd0][Jf0;no_`38b<P0d];B0=[J
fP3Rh^80jNWY0>o_k`3dm?@0m_Kf0?Win@3kno/00`3moOd10?knoP40oooo
0@3oool000050000003Cdm<0c/k>0<k>cP3?cll00P3>c/h03`27QhL0Ti>C
0=?Cd`3Kfm/0[Jf]0<71`@2`/;00ZJVY0:ZZZP2=SHd0TI6A0;2`/03;bl/0
kno_0:FUY@030<k>cP050<o?c`37alL08b<S055AD@38b<P00P3>c/h0;@1U
IFD06ATI0;Vi^@37alL08b<S055AD@37alL0c/k>08F5Q@0E5AD0Y:BT0<O7
a`0S8b<0>3Ph0<71`@3>c/h0bl_;04U9B@0I6AT0^KVi0<O7a`0S8b<0>3Ph
0<71`@3>c/h0alO702<S8`0h>3P0`L7108F5Q@0E5AD0Y:BT0<[:bP37alL0
8b<S055AD@2a/K406QXJ079bLP35aLD0c/k>08F5Q@0E5AD0Y:BT0<g=c@07
0<k>cP050:2PX00@4100>CTi0;no_`3;bl/01@3>c/h01@3=cLd0alO70<g=
c@37alL0b/[:01<0c/k>F`3?clld0<k>cP0307IfMP3>c/h0bl_;0100bl_;
00H0c/k>07IfMP3>c/h0c/k>07IfMP3>c/h20<_;b`030?ooo`0W9bL0bl_;
00L0bl_;00<0oooo02LW9`3;bl/00P3;bl/02@3>c/h0MWIf0<o?c`3>c/h0
EUIF07emO@22PX80QHF508B4Q00208F5Q@0302PX:03oool0oooo00<0oooo
00P0:2PX08F5Q@25QHD0Q8B408F5Q@22PX80OGem05IFEP@0c/k>0140dm?C
0000002XZ:P0[Jf]0;>c/`2k^k/0`l?30<c<c03EeMD0gMgM0>CTi03[jn/0
l?3`0?Cdm03gmoL0nOWi0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0
00000=3@d039bLT00`39bLT04@1cLg<0U9BD07inOP1iNGT0X:2P0=7Ad@2]
[Jd0YJFU0:JVYP25QHD0GUiN0:f][@3UiND0m_Kf0?Shn02TY:@0bLW900<0
b<S800@0`/;202<S8`1ADE40alO70P38b<P0;01RHV806ATI0;Vi^@32`/80
8R8R03Ph>031`L40`/;204I6AP0I6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35
aLD0a<C404I6AP0I6AT0^KVi0<71`@0S8b<0>3Ph0;no_`35aLD0`<30028R
8P0h>3P0`<300:n_[`0J6QX0Lg=c0<G5a@2JVYX0410@08j>SP35aLD0ATI6
01TI6@2i^KT0aLG507moO`0E5AD0Y:BT2@38b<P0101RHV80410@08f=S@37
alL40<S8b0060;>c/`15ADD0XJ6Q04a<C01^KVh0alO74`38b<QK0<W9bC@0
b<S800<0MWIf0=?Cd`3=cLd0403=cLd01P3Cdm<0MWIf0<S8b038b<P0MWIf
0=?Cd`80cLg=00<0oooo02LW9`3=cLd01`3=cLd00`3oool09bLW0<g=c@02
0<g=c@070=?Cd`1fMWH0b<S80<S8b01FEUH0PX:208>3P`0508N7Q`0502PX
:03oool0oooo0?ooo`0X:2P01@27QhL00`23Ph<0PX:205IFEP030<W9b@0B
0<S8b03@d=0000000;:b/P2f][H0^k^k0<;2`P39bLT0dM7A0=WIf@3Ph>00
inOW0>g]k@3alO40mOGe0?Shn03jn_X0o?cl0`3moOd10?knoP40oooo0@3o
ool000030000003?cll0aLG500@0aLG500l0UiNG0;Ng]`3/k>`0^KVi08^;
R`2g]kL0ZZZZ0:BTY01nOWh0LG5a0<C4a03IfMT0m_Kf0?Win@2UYJD01035
aLD0102o_kl08R8R055AD@36a/H20<G5a@06065QH@0I6AT0^KVi0;no_`0R
8R80GEeM0P0@4100100E5AD0Y:BT0<;2`P15ADD3010@4006055AD@1QHF40
410@01TI6@2f][H0ADE50`0@4100201ADE40_kno028R8P0h>3P0_kno0<G5
a@2/[:`06QXJ0P0@41001@1bLW80aLG50<G5a@32`/80ADE500<0410@00<0
5ADE0:BTY036a/H02035aLD00`2/[:`06QXJ055AD@050<G5a@050;no_`0R
8R80<30`01XJ6P2>SXh0X`35aLD00`1fMWH0eMGE0=7Ad@0@0=7Ad@060=GE
e@1fMWH0aLG50<G5a@1fMWH0eMGE0P3AdM400`3oool09bLW0=7Ad@070=7A
d@030?ooo`0W9bL0dM7A0080dM7A00L0eMGE07IfMP35aLD0aLG505IFEP22
PX80QhN700D0RHV91@3oool508V9R@0308N7Q`22PX80EUIF00@0aLG50140
clo?0000002l_;`0_kno0<?3``39bLT0d=3@0=OGe`3MgMd0i>CT0>[ZjP3_
knl0lo?c0?KfmP3hn?P0no_k0?clo0020?gmo@80o_kn0P3oool000030000
003=cLd0`L7100<0`L710100MGEe09VIV@2WYjL0kNg]0<W9b@1cLg<0/;2`
08f=S@1cLg<0C4a<069RHP2n_[h0]KFe0;:b/P2][Jd0ZJVY1031`L40102k
^k/08R8R055AD@34a<@20<71`@0405mOG`0I6AT0^KVi0<;2`PL0`L7100D0
^k^k028R8P0h>3P0_[jn0<;2`P020<71`@0<0<?3``35aLD0_;bl028R8P0h
>3P0_[jn0<;2`P2k^k/08R8R03Ph>02n_[h0`/;22`31`L40101lO7`05ADE
0:BTY033`l<60<71`@80a<C400@0_[jn04A4A00h>3P0_[jn1031`L401`1O
Gel0410@010@400@4100>3Ph0;jn_P32`/80X@31`L400`1fMWH0f=SH0=GE
e@0@0=GEe@060=SHf01fMWH0`L710<71`@1fMWH0f=SH0P3EeMD00`3oool0
9bLW0?ooo`080?ooo`0602LW9`3EeMD0eMGE0=GEe@3Hf=P0MWIf0P31`L40
0`1FEUH0Q8B408R8R00408Z:RP<0oooo00@0:2PX0?ooo`3oool0oooo102:
RXX00`28R8P0Q8B405IFEP040<71`@0@0<g=c@000000a/K60<S8b03<c<`0
dM7A0=OGe`3Lg=`0h^;R0>SXj03]kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0
oOgm0P3no_h20?ooo`0000<000000<c<c02j^[X00P2m_Kd04@2n_[h0_Kfm
09jNWP3Shn<0][Jf09JFUP22PX80Ti>C08B4Q022PX80CTi>02`/;02LW9`0
Zj^[0?KfmP3hn?P0YjNW00@0_Kfm00@0]kNg024Q8@1ADE40a<C40P2m_Kd0
101MGEd06ATI0;Vi^@2o_kl80;fm_@0305eMG@0g=cL0^;Rh0080_Kfm00/0
GEeM01TI6@2i^KT0_kno05eMG@0g=cL0^;Rh0;fm_@2g]kL08B4Q03Ph>002
0;jn_P/0_Kfm00@0NWYj01DE5@2TY:@0_kno102m_Kd01`2g]kL08B4Q06m_
K`35aLD0_kno02<S8`1`L7001@2m_Kd01P1jNWX0=3@d03`l?01MGEd0>3Ph
0;Rh^1<0_KfmF`2n_[hd0;fm_@0307IfMP3Jf]X0eMGE0100eMGE00H0f][J
07IfMP2m_Kd0_Kfm07IfMP3Jf]X20=GEe@030?ooo`1bLW80LW9b00P0LW9b
00h09bLW0=GEe@3EeMD0eMGE0=[JfP1fMWH0_Kfm0;jn_P1FEUH0QhN708Z:
RP2=SHd0SXj>08f=S@<0oooo00<0:2PX08f=S@0X:2P00`3oool308f=S@04
08Z:RP27QhL0EUIF0;jn_P80_Kfm0140^[Zj0<c<c0000000d=3@0=;BdP3E
eMD0fMWI0=gMg@3Rh^80inOW0>c/k03`l?00m?Cd0?KfmP3hn?P0n_[j0?cl
o0030?gmo@030?knoP3oool0oooo0040oooo00000`000000b/[:0;Rh^003
0;Zj^P0@07alO02=SHd0N7Qh09VIV@3`l?00`/;20:NWY`3;bl/0_;bl0:>S
X`1]KFd0OGem0:b/[03kno/0o?cl0:JVYP<0^[Zj00@0^k^k0;Bd]00Q8B40
CDe=0`2j^[X00`1KFe/06ATI0:n_[`0>0;Zj^P0305]KF`0I6AT0[jn_00D0
^[Zj00@0];Bd024Q8@0g=cL0];Bd302j^[X00`1gMgL051@D09^KV`060;Zj
^P05096AT@0@4100410@010@401=CDd01`2j^[X00`2d];@08B4Q06][J`2T
0;Zj^P0307IfMP3Hf=P0f][J0100f][J00H0f=SH07IfMP2j^[X0^[Zj07If
MP3Hf=P20=[JfP030?ooo`1bLW80LW9b00P0LW9b00d09bLW0=[JfP3Jf]X0
f][J0=SHf01fMWH0^[Zj0;^k^`1FEUH0Rh^;08n?S`2>SXh0Shn?00<0oooo
00D0:2PX08n?S`2?Shl0Shn?02PX:0030?ooo`0508n?S`2>SXh0Shn?08^;
R`1FEUH00`2j^[X0402h^;P0b/[:0000003IfMT0f][J0=gMg@3Ph>00i>CT
0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`30?gmo@80o_kn0`3o
ool0000300000039bLT0^;Rh00<0^;Rh0100QXJ609jNWP3De=@0l?3`0?;b
lP3>c/h0`L710=gMg@3Bd]80XZ:R0<c<c039bLT0RHV90?clo03moOd0YjNW
k`2h^;P00`1fMWH0emOG0=gMg@0@0=gMg@060=OGe`1fMWH0^;Rh0;Rh^01f
MWH0emOG0`3MgMd;02LW9`<0gMgM00T0emOG07IfMP2h^;P0^;Rh05IFEP2@
T900TI6A09>CT`2DU9@00P3oool00`0X:2P0V9RH09RHV00309RHV00802PX
:03oool0oooo09BDU02CTi<0TI6A092@T01FEUH40;Rh^00?0<W9b@000000
hN7Q0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3k
no/0o?cl00<0oOgm00<0o_kn0?ooo`3oool00P3oool0000300000039bLT0
^;Rh00@0^;Rh00l0QXJ609BDU03bl_80lo?c0:2PX03Xj>P0m_Kf0=_Kf`2I
VIT0n_[j0?_kn`3Lg=`0][Jf0?knoP2WYjL0k`2h^;P00`1fMWH0e]KF0>7Q
h@0@0>7Qh@060=KFeP1fMWH0^;Rh0;Rh^01fMWH0e]KF4@3QhN402@3Fe]H0
MWIf0;Rh^02h^;P0EUIF09FEU@2GUiL0W9bL0::RXP0202PX:0050:^[Z`2]
[Jd0Zj^[0:^[Z`2][Jd00P2[Zj/202PX:0050::RXP2LW9`0UiNG09FEU@1F
EUH0102h^;P03P39bLT000000>SXj03YjNT0jn_[0>g]k@3_knl0lO7a0?Cd
m03fm_H0moOg0?Win@3kno/0o?cl0`3moOd20?knoP@0oooo00000`000000
bLW90;Fe]@030;Fe]@0=07MgM`2PX:00h>3P0??cl`3UiND0k>c/0?KfmP3h
n?P0nOWi092@T03kno/0o?cl0?gmo@020?knoP030:NWY`2e]KD0]KFe0>d0
]KFe00<0MWIf0>g]k@3YjNT0403YjNT01P3]kNd0MWIf0;Fe]@2e]KD0MWIf
0>g]kA40jNWY00X0kNg]07IfMP2e]KD0]KFe05IFEP2KVi/0WYjN0:FUY@2b
/[80^[Zj0P2o_kl01030`<00_kno0;no_`30`<020;no_`070<30`02j^[X0
/[:b0:FUY@2NWYh0Vi^K05IFEP040;Fe]@0=0<W9b@000000k^k^0>o_k`3`
l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k0?clo0030?gmo@80o_kn
1@3oool0000300000039bLT0^[Zj00@0/K6a00<0SHf=0:ZZZP2ZZZX01@2Z
ZZX00`2WYjL0ZZZZ0:ZZZP040:ZZZ^l0/K6a00<0MWIf0=CDe03bl_80403b
l_801P3De=@0MWIf0;6a/@2a/K40MWIf0=CDe140l_;b00H0e=CD07IfMP2a
/K40/K6a05IFEP2SXj<20:RXZ0d0`l?30P2XZ:P00`2SXj<0EUIF0;6a/@02
0;6a/@040;Zj^P39bLT000000??cl`80m?Cd00L0mOGe0?KfmP3gmoL0n?Sh
0?[jnP3kno/0o?cl00@0oOgm00<0o_kn0?ooo`3oool0103oool000030000
0033`l<0bLW90?l0[Zj^0`2^[Zh0102=SHd0MWIf0<k>cP3kno/=0?ooo`04
0?_kn`3>c/h0MWIf08f=S@80[Zj^00@0SHf=07IfMP3>c/h0no_k3@3oool0
103kno/0c/k>07IfMP2=SHd20:j^[P0609VIV@1FEUH0]KFe0<_;b`3Ogml0
j>SX2@3WinL01P3Xj>P0gmoO0<_;b`2e]KD0EUIF09VIV@<0[Zj^00<0bLW9
0<?3``0000000P3fm_H01@3gmoL0n?Sh0?Shn03inOT0n_[j0080o?cl103m
oOd20?knoPH0oooo000010000000`l?30=CDe02g]kOo0:NWY`<0YjNW00<0
SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=S@1f
MWH0MWIf00l0MWIf00<0SHf=0:NWY`2WYjL00P2WYjL00`2=SHd0EUIF05IF
EP0?05IFEP0808f=S@2WYjL0YjNW0:NWY`2g]kL0e=CD0<?3``0000030?Wi
n@040?[jnP3kno/0o?cl0?clo0D0oOgm0P3no_h70?ooo`0000H000000:ZZ
ZP3WinL0e=CD0;jn_P2c/k?o0:ZZZT<0ZZZZ00H0/k>c0;jn_P3De=@0inOW
0:ZZZP0000020?_kn`<0o?cl1@3moOd30?knoPP0oooo00001`000000RXZ:
0=WIf@3lo?`0i>CT0=;BdP3<c<`0o`2n_[i10;jn_P080<c<c03Bd]80i>CT
0?clo03IfMT0RXZ:0000003lo?`80?gmo@80o_kn2P3oool000080000001Y
JFT0UiNG0=WIf@3lo?`0l_;b0>c/k03Shn?o0=WIfCl0fMWI00P0hn?S0>c/
k03bl_80o?cl0=WIf@2GUiL0JFUY000000L0oOgm0`3no_h;0?ooo`0000D0
00000?ooo`1YJFT0UiNG0=?Cd`020?clo?l0oooo@@3oool20?clo0050=?C
d`2GUiL0JFUY0?ooo`000000103moOd40?knoPd0oooo00000`000000oooo
0?ooo`0206UYJ@0308Z:RP2GUiL0ZZZZ0?l0ZZZZ@02ZZZX0102GUiL0RXZ:
06UYJ@1YJFT20?ooo`030000003no_h0o_kn00@0o_kn3`3oool00?l00000
C`00000E0?ooo`00\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{355, 227},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell[TextData[{
  "Notice that at the end of each ",
  StyleBox["NotebookWrite", "MR"],
  ", the cell insertion bar was placed just after the cell that was written, \
except in the case of writing input cells. Oftentimes, you will need to move \
around within the notebook or select a particular cell (or other expression) \
and perform some operation on it. For example, suppose we would like to \
select the previous cell (the one containing the ",
  Cell[BoxData[
      \(TraditionalForm\`2\^100\)]],
  ") in ",
  StyleBox["nb", "MR"],
  " and evaluate it. We can do this with the ",
  StyleBox["SelectionMove", "MR"],
  " function."
}], "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SelectionMove[nb, Previous, Cell]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005S0000hb000`400?l0
0000o`00003oo`3ooomT0?ooo`00o`3ooomT0?ooo`003P3ooooo0?knoTL0
o_kn3`3oool000`0oooo103no_ko0?gmoD<0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgmB@3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo3d0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k?`3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinD00nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_Ho0?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clcd0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^hm0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj3d0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4m0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH>`3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllk0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aC/0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^c/0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a>`2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYS/0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhk09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXm09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o000004l000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?clB`3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[ZdX0Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFU906UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj6`2OWil00`2m_Kd0k>c/0>OWi`0T0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K6`3:b/X0
0`3Kfm/0Xj>S0?_kn`0T0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo6`3@d=000`3oool0UYJF0?_kn`0T0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3ooolK0<o?c`03
0?ooo`2FUYH0no_k02@0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM400`3@d=070=7Ad@80d=3@0P3AdM430=3@
d0L0dM7A00<0oooo09JFUP3jn_X0903jn_X00`2FUYH0oooo0<k>cP040<k>
cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JFUP3Ph>00403Ph>00501Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0
h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3Ph01YJFT0WYjN0>3Ph02>
SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0
103>c/h303hn?PD0c/k>00@0oooo09JFUP2FUYH0oooo203Fe]H20=OGe`07
09FEU@3Fe]H0UIFE0=KFeP2EUID0e]KF09FEU@050=KFeP80emOG0`3Fe]H0
0`3oool0UYJF0?Win@0T0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n?Sh6
0<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3P
h034a<@0B4Q80=;BdP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P00P0
g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?oo
o`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3Lg=`6
0=gMg@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID70=gMg@80
g=cL00<0gMgM0?ooo`2FUYH09@3inOT00`2FUYH0oooo0<o?c`040<o?c`<0
?Shn1@3?cll00`3oool0UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3Ph02c
/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1YJFT0
WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WIf@3I
fMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI02HV
9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:0>3P
h0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0103o
ool0UYJF09JFUP3oool30=cLg0H0gMgM00P0oooo09FEU@3oool0UIFE0?oo
o`2EUID0oooo09FEU@L0gMgM0P3Lg=`00`3MgMd0oooo09JFUP0U0?Shn003
09JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0h>3P00P0
h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000003l
o?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00G0<C4
a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0d];B
0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2NWYh0
3`3Ph>05000000H0h>3P00@0UYJF0?ooo`3Kfm/0fm_K0`0n?Sh80=_Kf`05
0?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0<0hn?S00T0i>CT0?oo
o`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID00P3Ti>@30>?Sh`<0i>CT
00@0hn?S0>CTi03oool0UYJF9@3gmoL00`2FUYH0oooo0=_Kf`060=_Kf`<0
?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0h>3P
01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B02HV
9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0a<C4
04Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CTi0<0
?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WY
j@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY
00D0j^[Z0>WYj@3YjNT0oooo09JFUP0U0?Gem@0309JFUP3oool0i>CT00D0
i>CT0`0n?Sh40>CTi0030?ooo`2FUYH0h>3P00d0h>3P0P3oool05@3Ph>00
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P01T0SXj>0000000h>3P0fMWI0>3Ph01YJFT0WYjN
0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph02][Jd0
E5AD0>3Ph01YJFT0OWin0>3Ph01YJFT0WYjN0100h>3P00<000000>3Ph03P
h>001P3Ph>000`2FUYH0oooo0>g]k@030>g]k@<0?Shn1P3]kNd01@3oool0
UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X40>WYj@080?ooo`2EUID0oooo
09FEU@3oool0UIFE0?ooo`2EUID20>[ZjPD0jNWY00D0j^[Z0>WYj@3YjNT0
oooo09JFUP0U0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]k@03
0?ooo`2FUYH0h>3P00`0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0
h>3P01T0fMWI059BDP3IfMT0h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P
0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3Bd]80
h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@303hn
?PD0m?Cd00@0oooo09JFUP2FUYH0oooo2@3]kNd01`3oool0kNg]0?ooo`3]
kNd0oooo0>g]k@3oool02`3]kNd00`3oool0UYJF0?7al@0T0?7al@0309JF
UP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2FUYH0h>3P00`0h>3P
0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jN
WP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3lo?`01@3l
o?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?oooa/0kNg]
00<0oooo09JFUP3`l?00903`l?000`2FUYH0oooo0?clo0040?clo00303hn
?P3lo?`0o?cl00D0o?cl00<0oooo09JFUP3Ph>00403Ph>00501YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3P
h00e0>3Ph00309JFUP3oool0oooo00d0oooo0P2FUYH00`3oool0moOg0?Og
m`0I0?Ogm`030?ooo`2FUYH0kno_02@0kno_00<0UYJF0?ooo`3oool03@3o
ool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?cl
o02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF
0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XK0?ooo`030?[jnP2F
UYH0kno_02@0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF0>3P
h00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F
0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e]KD0
1@2d];@0Shn?0=[JfP3Fe]H0/;2`01/0^;Rh00<0ZJVY0<k>cP3^k^h0903^
k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0f][J0140h>3P01@0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFVG09jNWP030?ooo`3Ph>00
h>3P03D0h>3P4@3/k>`O0>_[jbD0inOW4@3/k>`A0>3Ph00D06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k03^k^h0SHf=
00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JF
UP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool01@3>c/h00`0n
?Sh0c/k>0<k>cP040<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3oool0103>c/h303hn
?PD0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool00`3/k>`0UYJF0?ooo`030<o?c`D0?Shn103?cll05P3o
ool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh3
0=CDe00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn0P3K
fm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolf0?ooo`030>c/k02FUYH0oooo0080i>CT00<0?Shn0>CTi03Ti>@00`3T
i>@06P0n?Sh0i>CT0>CTi03Ti>@0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/k>`0UYJF0?ooo`0<0>g]k@0F
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0
oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0oooo00`0
o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo=P3oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3GemL0
303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?S`3J
f]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo;@3oool5000000@0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo;03oool7000000<0
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo;03oool00`000000ZZZZ0:ZZZP030:ZZZP06
0000003oool0oooo0?ooo`3WinL0k^k^0P3_knl01P3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooob`0oooo1@0000000`2ZZZX0
00000?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooob`0oooo1@0000000`2ZZZX0
00000?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooob`0oooo1@0000000`2ZZZX0
00000?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooob`0oooo1@0000000`2ZZZX0
00000?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFU40?ooo`030::RXP00000000000080
000000<0@D510?Ogm`3oool0g`3oool5000000030:ZZZP000000oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJDD0oooo00<0N7Qh00l?3`3_knl0h`3oool500000003
0:ZZZP000000oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJDH0oooo00<0a/K600T92@3B
d]80hP3oool5000000030:ZZZP000000oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJDL0
oooo00<0gmoO01/K6`3Bd]80h@3oool5000000030:ZZZP000000oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJDP0oooo00<0XZ:R0451@@3gmoL0h03oool500000003
0:ZZZP000000oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00L0gmoO024Q8@3_
knl0oooo0<K6aP11@D40moOg00D0000000<0oooo0000000000000`3oool2
00000=<0oooo1@0000000`2ZZZX000000?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU4
0?ooo`060?Ogm`0M7Ad0];Bd0?ooo`1hN7P0H61P0`3oool010000000oooo
0?ooo`0000020?ooo`030000003oool000000080oooo00<000000?ooo`3o
ool0d03oool5000000030:ZZZP000000oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJDD0
oooo00D0gmoO018B4P0000003`l?0>o_k`030?ooo`040000003oool0oooo
00000080oooo00<000000?ooo`0000000P3oool00`000000oooo0?ooo`3@
0?ooo`D0000000<0ZZZZ0000003oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYC@3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool000000080oooo00<0
00000?ooo`3oool0d03oool400000080ZZZZ00H000000?ooo`3oool0oooo
0>OWi`3^k^h20>o_k`060?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJDd0oooo00@000000?ooo`3oool000000P3oool00`000000oooo
000000020?ooo`030000003oool0oooo0=00oooo0`000000102ZZZX00000
0:ZZZP0000030?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFU;0?ooo`<000000`3oool2000000<0
oooo0P00003C0?ooo`80000000D0ZZZZ000000000000ZZZZ000000030?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooob`0oooo00<000000:ZZZP0000000P0000000`2Z
ZZX000000?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooob`0oooo1P2ZZZX01P00
0000oooo0?ooo`3oool0inOW0>k^kP80kno_00H0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3oool/0?ooo`L000000`3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3oool]0?ooo`D00000103oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00X0inOW0>k^
kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocH0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooobd0oooo1@1EEED40?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
oc40oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0
l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_k
n`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?oo
o`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<@3oool00`1E
EED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?oo
o`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYA03oool02@1hN7P000000000002BTY80
4Q8B0000001PH600oooo0<K6aP03000000040451@@3gmoL0oooo0::RXP80
000000D0@D510?Ogm`3oool0oooo0<K6aP03000000030451@@3gmoL0oooo
0080oooo00<0a/K60000000000000`0000000`2d];@0gmoO018B4P030000
00030;Bd]03oool0oooo0080oooo00<0gmoO018B4P0000000P0000004P2d
];@0oooo0?ooo`3Ogml04Q8B000000092@T0d];B0=oOg`0B4Q80000006UY
J@0B4Q80@D51071`L0092@T0d];B0<K6aP<0000000<0@D510?Ogm`3oool0
103oool01`36a/H0000000000011@D40moOg0?ooo`36a/H00`0000003011
@D40lo?c05ADE00000002@T90<[:bP0M7Ad000000451@@3gmoL0oooo0<K6
aP80000000L0@D510?Ogm`3oool0oooo0<K6aP000000];Bd07`0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool04P3kno/0E5AD0;Bd
]03oool0gmoO024Q8@3_knl0a/K60000002d];@0oooo0?Ogm`0M7Ad0];Bd
0?ooo`3kno/0E5AD0;Bd]0<0oooo00L0a/K60000002d];@0oooo0?Ogm`0M
7Ad0];Bd00D0oooo00<0gmoO01/K6`3Bd]800P3oool01`3Ogml08B4Q0>o_
k`3oool0gmoO01/K6`3Bd]800`3oool0703Ogml08B4Q0>o_k`3oool0gmoO
01/K6`3Bd]80gmoO018B4P2d];@0oooo0<K6aP092@T0d];B0::RXP1PH600
gmoO01/K6`3Bd]80N7Qh08n?S`36a/H000000;Bd]03oool0moOg01dM7@2d
];@40?ooo`0C0?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0<K6aP000000];Bd
0?ooo`3gmoL07AdM0;Bd]03oool0XZ:R0451@@3clo<0E5AD08n?S`020?oo
o`060?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0P3oool00`36a/H000000;Bd
]01l0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_
0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3k
no/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00X0
no_k05ADE02d];@0oooo0=oOg`0Q8B40kno_07QhN011@D40moOg1@3oool0
0`3kno/0E5AD0;Bd]0030?ooo`0307QhN011@D40moOg00T0oooo00<0gmoO
01/K6`3Bd]80103oool01@3gmoL07AdM0000000?3`l0kno_00D0oooo00L0
moOg01dM7@0000003`l?0>o_k`2RXZ80H61P00<0oooo00`0N7Qh08n?S`2R
XZ80H61P0=oOg`0K6a/0d];B07QhN02?Shl0N7Qh0451@@3gmoL80?ooo`03
0?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL01P3oool0101hN7P0
B4Q80451@@3gmoL20?ooo`030?Ogm`0V9RH0d];B08@0oooo00<0EEEE0?oo
o`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe
0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYA03oool0203kno/0E5AD0;Bd]03oool0
gmoO024Q8@3_knl0N7Qh1@0000001@2?Shl0oooo0?_kn`1DE5@0H61P00<0
oooo00<0N7Qh0000000000000`0000000`2?Shl0oooo0?ooo`030?ooo`03
0=oOg`0K6a/0d];B0080oooo00H0no_k05ADE0000000000000l?3`3_knl4
0?ooo`090?_kn`1DE5@000000000000?3`l0kno_0?ooo`2RXZ80H61P0080
oooo00/0no_k05ADE02?Shl0XZ:R061PH03Ogml06a/K0=;BdP1hN7P0Shn?
07QhN0050000000308n?S`3oool0oooo0080oooo00<0moOg02HV9P3Bd]80
0`3oool00`1hN7P00000000000030000000608n?S`3oool0oooo0::RXP00
0000Shn?0`3oool00`3gmoL09RHV0=;BdP240?ooo`0305EEE@3oool0oooo
0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJD@0oooo00<0no_k05ADE00000000P0000005`0?
3`l0kno_0<K6aP092@T0d];B0?ooo`36a/H02@T90=;BdP3oool0no_k05AD
E0000000H61P0<K6aP1PH600a/K600T92@3Bd]80oooo0<K6aP092@T0d];B
00D0oooo00<0gmoO01/K6`3Bd]800P3oool01`3kno/0E5AD0;Bd]03oool0
a/K60451@@3gmoL00`3oool0703kno/0E5AD0;Bd]03oool0a/K60451@@3g
moL0gmoO018B4P2d];@0oooo0<K6aP092@T0d];B0::RXP0?3`l0clo?018B
4P2d];@0N7Qh08n?S`36a/H02@T90=;BdP3oool0a/K600T92@3Bd]840?oo
o`030?Ogm`0V9RH0d];B00<0oooo00d0a/K600T92@3Bd]80oooo0<K6aP09
2@T0d];B0?ooo`36a/H02@T90<[:bP0M7Ad0];Bd0080oooo00<0moOg02HV
9P3Bd]80Q03oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_
0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3k
no/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU40?oo
o`090?_kn`1DE5@0];Bd0?ooo`3Ogml08B4Q0>o_k`3oool0a/K600<00000
00/0];Bd0?ooo`3Ogml04Q8B0000002/[:`07AdM0000002d];@0oooo0<K6
aP03000000030;Bd]03oool0oooo0080oooo00D0no_k05ADE00000002@T9
0=;BdP030?ooo`060=oOg`0B4Q80000000000011@D40moOg103oool05P3O
gml04Q8B000000000000@D510?Ogm`3oool0gmoO018B4P0000002@T90=;B
dP3Ogml04Q8B05MGE`0M7Ad0/;2`05ADE00?3`l0kno_0?ooo`36a/H30000
00030;Bd]03oool0oooo0080oooo00<0gmoO018B4P0000000P0000001011
@D40moOg0?ooo`36a/H30000000;0;Bd]03oool0XZ:R000000092@T0/[:b
018B4P000000Shn?0=oOg`0B4Q800`0000000`11@D40moOg0?ooo`200?oo
o`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00L0no_k05AD
E02d];@0oooo0=oOg`0Q8B40kno_04T0oooo00<0moOg02HV9P3Bd]804P3o
ool00`3gmoL09RHV0=;BdP240?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJD@0oooo00L0N7Qh000000000000TY:B018B4P000000H61P01X0
oooo00<0XZ:R00l?3`3_knl0;03oool00`3gmoL09RHV0=;BdP0B0?ooo`03
0?Ogm`0V9RH0d];B08@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3oool`0?ooo`80EEEE103oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool_0?ooo`03
05EEE@3oool0EEEE00@0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo;P3oool0101EEED0
oooo0?ooo`1EEED40?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobd0oooo00D0EEEE0?oo
o`3oool0oooo05EEE@040?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooob`0oooo1P1EEED4
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/0?3`l02>SXh01@2PX:00
0`2OWil0X:2P0:2PX0040:2PX00F092@T03/k>`0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocH0oooo00<0k>c/09JFUP3Ng]h0303Cdm<0
5P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gM
g@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3AdM40clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0
gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0i>CT0=gMg@3MgMd0emOG
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo
=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3M
gMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]
0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S
0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?03ooooT000000`0oooo1@1EEED40?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM
0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM
0=gMg@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL
0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTa0?ooo`050::RXP000000000000T92@3Bd]801P3o
ool00`000000oooo0?ooo`050?ooo`0605ADE0000000000008n?S`3oool0
N7Qh0P0000003`11@D40moOg07QhN00000003`l?0>o_k`1hN7P000000451
@@3gmoL0oooo07QhN00000003`l?0>o_k`030?ooo`080::RXP000000@D51
0<?3``0o?cl0000000l?3`3_knl30?ooo`070<_;b`0o?cl0000000l?3`3_
knl0oooo0::RXP020000000@0;Bd]03oool0N7Qh0000001PH600N7Qh0000
0011@D40VYZJ00000011@D40moOg07QhN00000003`l?0>o_k`<0oooo00/0
N7Qh0000001PH600N7Qh00000011@D40VYZJ00000011@D40moOg0::RXP02
0000000;0;Bd]03oool0N7Qh00000011@D40moOg0?ooo`1hN7P0000000l?
3`3_knl0103oool0501hN7P000000;Bd]03oool0N7Qh0000000?3`l0jn_[
05ADE01PH600gmoO018B4P11@D40moOg07QhN0000000];Bd0=oOg`0K6a/0
d];BJP3oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL
0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJC00oooo00H0no_k05eMG@3Bd]80oooo0::RXP2?Shl60?oo
o`030000003oool0oooo00H0oooo00<0N7Qh0;Bd]03oool00P3oool04@2R
XZ80Shn?0?ooo`36a/H0H61P0?ooo`3gmoL0OGem0?Ogm`0/;2`0kno_0?oo
o`36a/H0H61P0?ooo`3gmoL0OGem00<0oooo00P0no_k05eMG@3Bd]80bl_;
0=;BdP3oool0a/K608n?S`<0oooo01T0bl_;0=;BdP3oool0a/K608n?S`3O
gml06a/K0=;BdP3kno/0GEeM0=;BdP3gmoL0;2`/0>o_k`3kno/0GEeM0=;B
dP3kno/0E5AD0;Bd]036a/H0H61P0?ooo`3gmoL0OGem00<0oooo01P0moOg
02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0gmoO01/K6`3Bd]80
no_k05eMG@3Bd]80moOg02`/;03_knl0oooo0<K6aP1PH600oooo0?Ogm`1m
OGd30?ooo`0A0?_kn`1MGEd0d];B0?ooo`36a/H0H61P0?ooo`3gmoL0OGem
0=oOg`1bLW80no_k05eMG@3Bd]80no_k05eMG@3Bd]80KP3oool00`000000
oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd609FEU@0I0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY<03oool01P3gmoL0;2`/09:BTP0?
3`l0][Jf061PH0H0oooo00<000000?ooo`3oool01P3oool00`1hN7P0];Bd
0?ooo`020?ooo`050::RXP2?Shl0oooo0::RXP2d];@00`3oool01P3gmoL0
;2`/0>o_k`3oool0XZ:R0;Bd]0H0oooo00<0no_k05eMG@3Bd]800P3oool0
0`2RXZ8000000;Bd]0050?ooo`050::RXP000000];Bd0::RXP1PH6000P3o
ool03@2RXZ80Shn?0?Ogm`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0
];Bd0::RXP2d];@01P3oool02`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3k
no/0E5AD0;Bd]02RXZ80H61P0080oooo00P0XZ:R08n?S`3gmoL0;2`/0>o_
k`3oool0XZ:R0;Bd]0H0oooo00H0no_k05eMG@3Bd]80oooo0::RXP2d];@4
0?ooo`070<_;b`0o?cl0Shn?0?ooo`3kno/0GEeM0=;BdP1^0?ooo`030000
003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF
0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFT`0?ooo`060?Ogm`0M7Ad0
H61P0?Ogm`0M7Ad0H61P1P3oool00`000000oooo0?ooo`060?ooo`0307Qh
N02d];@0oooo0080oooo00@0XZ:R08n?S`3oool0XZ:R0`0000001P1PH600
moOg02`/;03_knl0oooo0::RXP<0000000/0H61P0?ooo`3oool0oooo0?_k
n`1MGEd0d];B0=oOg`0B4Q80@D510?Ogm`040?ooo`070=oOg`0B4Q80@D51
0?Ogm`3oool0XZ:R08n?S`020?ooo`0<07QhN02?Shl0moOg02`/;03_knl0
no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R0`0000003`1PH600oooo0?oo
o`3oool0moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R
08n?S`020?ooo`0707QhN02?Shl0moOg02`/;03_knl0oooo0::RXP030000
0009061PH03oool0oooo0?ooo`3kno/0GEeM0=;BdP3oool0XZ:R00<00000
00T0H61P0?ooo`36a/H03`l?0>o_k`3oool0no_k05eMG@3Bd]80KP3oool0
0`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/
09JFUP3oool0moOg0P3]kNd609FEU@0I0=3@d03?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY<03oool01P3gmoL0;2`/
0>o_k`3oool0a/K6061PH0H0oooo00<000000?ooo`3oool01P3oool00`1h
N7P00000000000030000000@08n?S`3oool0a/K608n?S`3oool0N7Qh08^;
R`1DE5@0000008n?S`3oool0a/K608n?S`3oool0N7Qh08n?S`<0oooo00P0
N7Qh00T92@3Bd]80a/K608n?S`3oool0PH610=;BdP<0oooo01T0a/K608n?
S`3oool0PH610=;BdP3Ogml0De=C0?Ogm`3Ogml04Q8B0;2`/01DE5@00000
08n?S`3gmoL07AdM061PH03kno/0GEeM0=;BdP36a/H0Shn?0?ooo`1hN7P0
Shn?0080oooo01T0no_k05ADE0000000Shn?0?Ogm`0M7Ad0H61P0?_kn`1M
GEd0d];B0=oOg`1CDe<0moOg0=oOg`0B4Q80/;2`05ADE0000000Shn?0?oo
o`36a/H0Shn?0?ooo`1hN7P0Shn?00<0oooo0140fm_K05eMG@3Bd]80oooo
0<K6aP2?Shl0oooo07QhN02?Shl0moOg02`/;03WinL0Y:BT0?ooo`3Kfm/0
GEeM0=;BdP1^0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3o
ool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7A
d@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFT`0?ooo`060?Ogm`0M7Ad0@D510=OGe`0B4Q80H61P1P3oool00`000000
oooo0?ooo`060?ooo`0307QhN02d];@0oooo0080oooo00L0XZ:R08n?S`3o
ool0oooo07QhN0000000@D510080moOg00P0;2`/09:BTP092@T0d];B07Qh
N0000000@D510?Ogm`<0oooo00P0moOg02HV9P3Bd]80no_k05ADE0000000
UYJF0=_Kf`<0oooo00L0no_k05ADE0000000UYJF0=_Kf`3oool0XZ:R0080
00000100Shn?0?ooo`3Ogml0De=C09ZJVP000000@D510;jn_P000000@D51
0?Ogm`3oool0N7Qh00000011@D40moOg0`3oool02`3Ogml0De=C09ZJVP00
0000@D510;jn_P000000@D510?Ogm`3oool0XZ:R0080000000/0Shn?0?oo
o`3gmoL0;2`/09:BTP092@T0d];B07QhN0000000@D510?Ogm`030?ooo`0?
0<K6aP00000000000;Bd]03oool0N7Qh00000011@D40lo?c05ADE0000000
TY:B018B4P2?Shl0a/K60080000000<0];Bd0?ooo`3oool0J`3oool00`00
0000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd609FEU@0I0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUY<03oool01P3kno/0GEeM0=;B
dP3oool0XZ:R08n?S`H0oooo00<000000?ooo`3oool01P3oool00`1hN7P0
];Bd0?ooo`020?ooo`030::RXP2?Shl0oooo05@0oooo00<0PH610=;BdP3o
ool02`3oool00`21PH40d];B0?ooo`1]0?ooo`030000003oool0oooo00h0
oooo00<0EEEE0?ooo`3oool00P3oool01@3/k>`0UYJF0?ooo`3gmoL0kNg]
00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTa0?ooo`050::RXP000000000000T92@3Bd]80
1P3oool00`000000oooo0?ooo`060?ooo`0307QhN02d];@0oooo0080oooo
00<0XZ:R08n?S`3oool0E03oool00`2XZ:P0d];B0?ooo`0;0?ooo`030:RX
Z03Bd]80oooo06d0oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo
0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3AdM40
clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JC`0oooo00<000000?ooo`3oool01@3oool01P1DE5@000000000002?Shl0
oooo07QhN080000000<0@D510?Ogm`3oool03`3oool00`3gmoL09RHV0=;B
dP2n0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3o
ool01@3/k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X0
7@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL
0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1E
EED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT0
7@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00
oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<000000?ooo`3oool0
3P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003o
ool0oooo0>00oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM
0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>00oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?oo
o`030000003oool0oooo0>00oooo00<000000?ooo`3oool02`3oool405EE
E@@0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0
gMgM0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJC`0oooo00<000000?ooo`3oool0h03oool00`000000
oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JFUP3o
ool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3AdM40clo?0=3@
d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJC`0oooo
i000000<0?ooo`D0EEEE103oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80
jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`040>c/k02F
UYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0d=3@0<o?
c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo
0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3M
gMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf0?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0
d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M
0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0UYJF0?oo
o`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@
0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolf
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gM
g@3Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd2
0>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0103/k>`0
UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?
cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolf0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0
g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?ooocH0oooo00@0k>c/09JFUP3oool0moOg
0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL0=KFeP3@d=00clo?0=3@d03oool0
UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=cLg03Lg=`0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolf0?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo=P3oool00`3/
k>`0oOgm09>CT`050;Rh^0030;Fe]@2h^;P0^;Rh00@0^;Rh01H0Ti>C0>c/
k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo;@3oool505EE
E@@0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/0>k^
kP2=SHd0302OWil05P2=SHd0i^KV06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooola0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JF
UP3Ng]h0303Kfm/05P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY?03oool01P1DE5@000000000002?Shl0oooo07QhN080000000l0
@D510?Ogm`1hN7P0000000l?3`3_knl0N7Qh00000011@D40moOg0?ooo`1h
N7P0000000l?3`3_knl00`3oool0202RXZ8000000451@@33`l<0?clo0000
000?3`l0kno_0`3oool01`3;bl/0?clo0000000?3`l0kno_0?ooo`2RXZ80
0P000000402d];@0oooo07QhN0000000H61P07QhN0000000@D5109ZJVP00
0000@D510?Ogm`1hN7P0000000l?3`3_knl40?ooo`0D07QhN0000000];Bd
0?ooo`1hN7P0000000l?3`3[jn/0E5AD061PH03Ogml04Q8B0451@@3gmoL0
N7Qh0000002d];@0gmoO01/K6`3Bd]:N0?ooo`0305EEE@3oool0oooo0080
oooo00<0k>c/09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?@3oool00`1hN7P0];Bd0?ooo`020?ooo`0A
0::RXP2?Shl0oooo0<K6aP1PH600oooo0?Ogm`1mOGd0moOg02`/;03_knl0
oooo0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool0203kno/0GEeM0=;BdP3;
bl/0d];B0?ooo`36a/H0Shn?0`3oool06@3;bl/0d];B0?ooo`36a/H0Shn?
0=oOg`0K6a/0d];B0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?_kn`1MGEd0
d];B0?_kn`1DE5@0];Bd0<K6aP1PH600oooo0?Ogm`1mOGd00`3oool04@3k
no/0GEeM0=;BdP3oool0a/K6061PH03oool0moOg07emO@3Ogml0LW9b0?_k
n`1MGEd0d];B0?_kn`1MGEd0d];B0:80oooo00<0EEEE0?ooo`3oool00P3o
ool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFTm0?ooo`0307QhN02d];@0oooo0080oooo00D0
XZ:R08n?S`3oool0XZ:R0;Bd]0030?ooo`060?Ogm`0/;2`0kno_0?ooo`2R
XZ80];Bd1P3oool00`3kno/0GEeM0=;BdP020?ooo`030::RXP000000];Bd
00D0oooo00D0XZ:R0000002d];@0XZ:R061PH0020?ooo`0=0::RXP2?Shl0
moOg02`/;03_knl0no_k05eMG@3Bd]80no_k05ADE02d];@0XZ:R0;Bd]006
0?ooo`060?_kn`1MGEd0d];B0?ooo`2RXZ80];Bd103oool01`3;bl/0?clo
08n?S`3oool0no_k05eMG@3Bd]80XP3oool00`1EEED0oooo0?ooo`020?oo
o`030>c/k02FUYH0oooo00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool0102R
XZ80Shn?0?ooo`2RXZ8300000006061PH03gmoL0;2`/0>o_k`3oool0XZ:R
0`0000002`1PH600oooo0?ooo`3oool0no_k05eMG@3Bd]80gmoO018B4P11
@D40moOg00@0oooo00L0gmoO018B4P11@D40moOg0?ooo`2RXZ80Shn?0080
oooo00`0N7Qh08n?S`3gmoL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD
0;Bd]02RXZ8300000009061PH03oool0oooo0?ooo`3kno/0GEeM0=;BdP3o
ool0XZ:R00<0000000T0H61P0?ooo`36a/H03`l?0>o_k`3oool0no_k05eM
G@3Bd]80XP3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo
00`0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JCd0oooo00<0N7Qh0000000000000`000000402?Shl0oooo0<K6aP2?Shl0
oooo07QhN02;Rh/0E5AD0000002?Shl0oooo0<K6aP2?Shl0oooo07QhN02?
Shl30?ooo`0807QhN0092@T0d];B0<K6aP2?Shl0oooo0861P@3Bd]830?oo
o`0I0<K6aP2?Shl0oooo0861P@3Bd]80gmoO05=CD`3gmoL0gmoO018B4P2`
/;00E5AD0000002?Shl0moOg01dM7@1PH600no_k05eMG@3Bd]80a/K608n?
S`3oool0N7Qh08n?S`030?ooo`0A0=_Kf`1MGEd0d];B0?ooo`36a/H0Shn?
0?ooo`1hN7P0Shn?0?Ogm`0/;2`0inOW0:BTY03oool0fm_K05eMG@3Bd]80
XP3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo0080c/k>
00<0?Shn0<k>cP3>c/h00`3>c/h06P0n?Sh0c/k>0<k>cP3>c/h0oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJCd0oooo00<0N7Qh0;Bd
]03oool00P3oool01`2RXZ80Shn?0?ooo`3oool0N7Qh00000011@D400P3g
moL0200/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg0`3oool0203g
moL09RHV0=;BdP3kno/0E5AD0000002FUYH0fm_K0`3oool01`3kno/0E5AD
0000002FUYH0fm_K0?ooo`2RXZ800P000000402?Shl0oooo0=oOg`1CDe<0
VYZJ00000011@D40_[jn00000011@D40moOg0?ooo`1hN7P000000451@@3g
moL30?ooo`0?0<K6aP00000000000;Bd]03oool0N7Qh00000011@D40lo?c
05ADE0000000TY:B018B4P2?Shl0a/K60080000000<0];Bd0?ooo`3oool0
W`3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo0<o?c`<0
?Shn0`3?cll303hn?P80clo?01H0oooo09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJCd0oooo00<0N7Qh0;Bd]03oool00P3oool00`2RXZ80
Shn?0?ooo`0i0?ooo`030861P@3Bd]80oooo00/0oooo00<0PH610=;BdP3o
ool0X@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02FUYH0oooo0080
e=CD0`0n?Sh0103De=@0?Shn03hn?P0n?Sh30=CDe00F0?ooo`2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTm0?ooo`0307QhN02d];@0oooo
0080oooo00<0XZ:R08n?S`3oool0>@3oool00`2XZ:P0d];B0?ooo`0;0?oo
o`030:RXZ03Bd]80oooo0:40oooo00<0EEEE0?ooo`3oool00P3oool00`3/
k>`0UYJF0?ooo`030=_Kf`D0?Shn103Kfm/05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUY?03oool01P1DE5@000000000002?Shl0
oooo07QhN080000000<0@D510?Ogm`3oool03`3oool00`3gmoL09RHV0=;B
dP3G0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3oool0103T
i>@303hn?PD0i>CT01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo<@3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k02F
UYH0oooo00D0kNg]00<0?Shn0>g]k@3]kNd0103]kNd05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooola0?ooo`0305EEE@3o
ool0oooo0080oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3oool^0?ooo`@0EEEE103o
ool00`3/k>`0UYJF0?ooo`0<0?clo00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000o?cl0:^[Z`1YJFWo0?oooc40oooo00<0EEEE0?ooo`3oool00P3o
ool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooobd0oooo1@1EEED40?ooo`030>c/k02FUYH0
emOG00`0oooo01H0emOG09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo=P3oool0103/k>`0f][J09:BTP2f][H:0;Fe]@0G0;Bd]02?
Shl0f][J06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5
a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3m
oOd10?knoP40oooo0@3oool000030000003gmoL0VYZJ0?l0NWYjB@1jNWX0
4`2JVYX0moOg0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000030000003gmoL0_kno0?l0_knoBP2o_kl04P3gmoL00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00000`00
0000moOg0>?Sh`3o0>?ShdX0hn?S0180moOg0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000?7al@3oool0o`3o
ool30?ooo`030>?Sh`2UYJD0MWIf00<0MWIf00@0MgMg07IfMP1gMgL0MgMg
2@1fMWH01P2UYJD0hn?S0?ooo`3oool0hn?S0:FUY@@0MWIf00@0MgMg07If
MP1gMgL0MgMg2@1fMWH01P2UYJD0hn?S0?ooo`3oool0jn_[0:FUY@@0EUIF
00@0EeMG05IFEP1GEeL0EeMG2@1FEUH00`2UYJD0jn_[0?ooo`030?ooo`0B
0?7al@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003QhN40o?cl0?_kn`80o?cl00<0no_k0?clo03lo?`0103l
o?`00`3kno/0o?cl0?clo0020?clo0030?_kn`3lo?`0no_k00D0o?cl00<0
no_k0?clo03kno/0F@3kno^?0?clo0030:FUY@1fMWH0SHf=0100SHf=00H0
MWIf0:FUY@3lo?`0o?cl0:FUY@1fMWHA08f=S@0607IfMP2UYJD0o?cl0?_k
n`2UYJD0EUIF4@1KFe/00`1FEUH0YJFU0?clo0030?clo00B0>7Qh@000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000
003Ph>00moOg0?l0moOg0`3gmoL00`1fMWH0SHf=0::RXP0@0::RXP0608f=
S@1fMWH0moOg0?Ogm`1fMWH0SHf=4@2RXZ80202=SHd0MWIf0?Ogm`3gmoL0
EUIF05]KF`1_Kfl0LG5a3@1bLW80101aLG40Kfm_05]KF`1FEUH40?Ogm`0B
0>3Ph0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000030000003Ng]h0lO7a0080lO7a0P3bl_800`3alO40l_;b0?;blP04
0?7al@030?;blP3alO40lO7a00<0lO7a0P3bl_800`3alO40l_;b0?7al@02
0?;bl^X0lO7a00<0MWIf0;Rh^02[Zj/0402[Zj/01P2h^;P0MWIf0?7al@3a
lO40MWIf0;Rh^140Zj^[00D0^;Rh07IfMP3alO40l_;b05IFEP0206m_K`03
079bLP1gMgL0MgMg00/0MgMg00@0LW9b06m_K`1_Kfl0EUIF0`3alO404`3b
l_80g]kN0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000050000003Lg=`0jn_[0>_[j`3/k>`00P3[jn/00`2WYjL0XZ:R
0::RXP020:6QX@030:2PX02OWil0WIfM0080WinO0P2PX:020:6QX@030:NW
Y`3[jn/0jn_[00D0jn_[00D0e=CD0<G5a@35aLD0aLG50=OGe`020>_[j`03
0>[ZjP38b<P0d=3@0080jn_[00D0f][J0<K6aP3Shn<0jn_[0>3Ph0020<G5
a@070<c<c03YjNT0c/k>0<O7a`3YjNT0jn_[0>3Ph0020<G5a@070<c<c03Z
j^X0e=CD0<G5a@3MgMd0jn_[0>3Ph0020<G5a@060<K6aP3Shn<0jn_[0>_[
j`3Zj^X0b<S80`35aLD00`36a/H0hn?S0>_[j`040>_[j`030>[ZjP38b<P0
aLG500<0aLG500<0a/K60>?Sh`3[jn/06`3[jn]K0>c/k3@0jn_[00@0MWIf
0;^k^`2a/K40/K6a203oool70;6a/@060;^k^`1fMWH0jn_[0>_[j`1fMWH0
^k^k0P2a/K4<0?ooo`<0/K6a00T0^k^k07IfMP3[jn/0jn_[05IFEP1gMgL0
LW9b07AdM01gMgL02`1hN7P01@1gMgL0M7Ad079bLP1gMgL0EUIF00@0jn_[
0180g=cL0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000<000000=[JfP3UiND00`3UiND00`2:RXX0Vi^K0:b/[0030>WY
j@0:0>[ZjP34a<@0QHF50>c/k03]kNd0k^k^0>OWi`3`l?00l_;b0:>SX`H0
iNGU0140J6QX010@400@4100410@0820P036a/H0clo?0=kNgP0V9RH0DE5A
0<o?c`3UiND0U9BD01DE5@2TY:@0g]kN0;:b/P02010@40070451@@3Kfm/0
D51@01TI6@2j^[X0hn?S0;:b/P02010@40070451@@3Ogml0L71`010@402B
TY80i>CT0:n_[`02010@400601DE5@2j^[X0c<c<0>CTi03MgMd092@T0`0@
41000`0E5AD0Y:BT0=kNgP040>GUi@030=kNgP0V9RH0410@00<0410@00<0
5ADE0;jn_P3UiND0Z`3UiND01@1fMWH0`<300;Fe]@2e]KD0oooo00H0LW9b
00<09bLW0;Fe]@2e]KD01@2e]KD01P30`<00MWIf0>GUi@3UiND0MWIf0<30
`080]KFe00<0oooo02LW9`0W9bL02@0W9bL30;Fe]@050<30`01fMWH0iNGU
0>GUi@1FEUH00P1gMgL0101iNGT0NWYj0?ooo`3oool707YjNP80oooo00D0
NWYj07UiN@1gMgL0MgMg05IFEP040>GUi@0B0=[JfP000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003IfMT0gmoO
00<0gmoO00D0g]kN0:2PX03Zj^X0jNWY0>GUi@020>[ZjP0:0=3@d01dM7@0
kNg]0>k^kP2][Jd0d=3@0?;blP3clo<0Xj>S0=oOg`80g]kN03@0gmoO0=kN
gP1/K6`05ADE0:^[Z`3Ng]h0`l?301`L701DE5@0a/K60=KFeP0U9BD0DE5A
0<g=c@3Ng]h0Shn?01DE5@2TY:@0eMGE04i>CP0h>3P0a<C40=gMg@3Kfm/0
CTi>01TI6@2j^[X0fMWI04i>CP0h>3P0a<C40=gMg@3GemL09BDU03Ph>034
a<@0`/;201/K6`1ADE40fMWI0<?3``1<C4`0f=SH0=OGe`0U9BD06ATI0<G5
a@3Ng]h0Shn?01DE5@2TY:@0fMWI1@3Ng]h01@2][Jd0410@01TI6@2i^KT0
dM7A0:h0g]kN00D0MWIf0<?3``2k^k/0^k^k0?ooo`06079bLP0302LW9`2k
^k/0^k^k00D0^k^k00H0`l?307IfMP3Ng]h0g]kN07IfMP33`l<20;^k^`03
0?ooo`0W9bL0^k^k00L0^k^k00<0oooo02LW9`2k^k/00P2k^k/02@33`l<0
MWIf0=oOg`3Ng]h0EUIF07MgM`1hN7P0Ng]k030`<0030?ooo`D0OGem0`3o
ool01@0`<300Ng]k07QhN01gMgL0EUIF0080g]kN01@0gmoO0=kNgP3IfMT0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo0000
10000000emOG0=SHf03IfMT20=SHf00@092@T02?Shl0SXj>0>WYj@21PH40
DE5A08N7Q`3Jf]X0YZJV0:VYZ@2/[:`0GemO0?;blP3clo<0m?Cd0:BTY0<0
f=SH00D0fMWI0=;BdP0T92@0DE5A0<_;b`020=SHf00P06YZJP0I6AT0^[Zj
0=3@d00T92@0DE5A0<_;b`3Hf=P0Rh^;01DE5@2TY:@0cLg=02@T900h>3P0
`l?30=OGe`3EeMD0C4a<01TI6@2j^[X0d=3@02@T900h>3P0`l?30=OGe`3A
dM4092@T03Ph>033`l<0RXZ:01DE5@2TY:@40<G5a@0:09jNWP0@4100LG5a
0=7Ad@3Hf=P0Rh^;01DE5@2TY:@0e=CD0=;BdP80aLG500P0a/K60=OGe`3H
f=P0JVYZ010@401bLW80aLG50=CDe1h0f=SHF`3IfMTd0=SHf00507IfMP37
alL0_kno0;no_`3oool01`0W9bL70;no_`060<O7a`1fMWH0f=SH0=SHf01f
MWH0alO70P2o_kl00`3oool09bLW0;no_`070;no_`030?ooo`0W9bL0_kno
0080_kno00D0alO707IfMP3Hf=P0f=SH05IFEP0207]kN`0307inOP1oOgl0
:2PX00<0oooo0`1oOgl30?ooo`0302PX:01oOgl0OWin0080Ng]k00<0EUIF
0=WIf@3Hf=P00P3Hf=P04P3GemL0000009ZJVP2PX:00Z:RX0;:b/P2l_;`0
a/K60=3@d03IfMT0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080
oOgm0@3no_h10?ooo`40oooo00000`000000eMGE0=?Cd`030=?Cd`0308F5
Q@2SXj<0i^KV0080j^[Z00/0=CDe07moO`3?cll0_[jn08^;R`1NGUh0F5QH
06QXJ02SXj<0bLW90:ZZZP040=?Cd`040<c<c00T92@0DE5A0<[:bP80dm?C
01h0IfMW01TI6@2i^KT0bl_;02@T901ADE40b/[:0=?Cd`28R8P05ADE0:BT
Y039bLT092@T03Ph>031`L40dm?C0<o?c`1:BTX06ATI0;Vi^@3;bl/092@T
03Ph>031`L40dm?C0<c<c00T92@0>3Ph0<71`@1WIfL5010@400:055AD@2M
WId0410@08j>SP3<c<`0dm?C08R8R00E5AD0Y:BT0:6QX@80410@00T06QXJ
0<K6aP3Cdm<0dm?C0<c<c00T92@05ADE0:BTY037alL0[@3Cdm<00`1fMWH0
bl_;0<K6aP0@0<K6aP060<_;b`1fMWH0dm?C0=?Cd`1fMWH0bl_;0P36a/H0
0`3oool09bLW0<K6aP070<K6aP030?ooo`0W9bL0a/K60080a/K600L0bl_;
07IfMP3Cdm<0dm?C05IFEP1mOGd0Ogmo00<0Ph>300D0:2PX0?ooo`3oool0
oooo08>3P`030?ooo`0702PX:023Ph<0Ph>308>3P`1oOgl0OGem05IFEP04
0=?Cd`0A0=GEe@000000X:2P0:FUY@2][Jd0][Jf0;no_`38b<P0d];B0=[J
fP3Rh^80jNWY0>o_k`3dm?@0m_Kf0?Win@3kno/00`3moOd10?knoP40oooo
0@3oool000050000003Cdm<0c/k>0<k>cP3?cll00P3>c/h03`27QhL0Ti>C
0=?Cd`3Kfm/0[Jf]0<71`@2`/;00ZJVY0:ZZZP2=SHd0TI6A0;2`/03;bl/0
kno_0:FUY@030<k>cP050<o?c`37alL08b<S055AD@38b<P00P3>c/h0;@1U
IFD06ATI0;Vi^@37alL08b<S055AD@37alL0c/k>08F5Q@0E5AD0Y:BT0<O7
a`0S8b<0>3Ph0<71`@3>c/h0bl_;04U9B@0I6AT0^KVi0<O7a`0S8b<0>3Ph
0<71`@3>c/h0alO702<S8`0h>3P0`L7108F5Q@0E5AD0Y:BT0<[:bP37alL0
8b<S055AD@2a/K406QXJ079bLP35aLD0c/k>08F5Q@0E5AD0Y:BT0<g=c@07
0<k>cP050:2PX00@4100>CTi0;no_`3;bl/01@3>c/h01@3=cLd0alO70<g=
c@37alL0b/[:01<0c/k>F`3?clld0<k>cP0307IfMP3>c/h0bl_;0100bl_;
00H0c/k>07IfMP3>c/h0c/k>07IfMP3>c/h20<_;b`030?ooo`0W9bL0bl_;
00L0bl_;00<0oooo02LW9`3;bl/00P3;bl/02@3>c/h0MWIf0<o?c`3>c/h0
EUIF07emO@22PX80QHF508B4Q00208F5Q@0302PX:03oool0oooo00<0oooo
00P0:2PX08F5Q@25QHD0Q8B408F5Q@22PX80OGem05IFEP@0c/k>0140dm?C
0000002XZ:P0[Jf]0;>c/`2k^k/0`l?30<c<c03EeMD0gMgM0>CTi03[jn/0
l?3`0?Cdm03gmoL0nOWi0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0
00000=3@d039bLT00`39bLT04@1cLg<0U9BD07inOP1iNGT0X:2P0=7Ad@2]
[Jd0YJFU0:JVYP25QHD0GUiN0:f][@3UiND0m_Kf0?Shn02TY:@0bLW900<0
b<S800@0`/;202<S8`1ADE40alO70P38b<P0;01RHV806ATI0;Vi^@32`/80
8R8R03Ph>031`L40`/;204I6AP0I6AT0_;bl0<71`@0S8b<0>3Ph0;no_`35
aLD0a<C404I6AP0I6AT0^KVi0<71`@0S8b<0>3Ph0;no_`35aLD0`<30028R
8P0h>3P0`<300:n_[`0J6QX0Lg=c0<G5a@2JVYX0410@08j>SP35aLD0ATI6
01TI6@2i^KT0aLG507moO`0E5AD0Y:BT2@38b<P0101RHV80410@08f=S@37
alL40<S8b0060;>c/`15ADD0XJ6Q04a<C01^KVh0alO74`38b<QK0<W9bC@0
b<S800<0MWIf0=?Cd`3=cLd0403=cLd01P3Cdm<0MWIf0<S8b038b<P0MWIf
0=?Cd`80cLg=00<0oooo02LW9`3=cLd01`3=cLd00`3oool09bLW0<g=c@02
0<g=c@070=?Cd`1fMWH0b<S80<S8b01FEUH0PX:208>3P`0508N7Q`0502PX
:03oool0oooo0?ooo`0X:2P01@27QhL00`23Ph<0PX:205IFEP030<W9b@0B
0<S8b03@d=0000000;:b/P2f][H0^k^k0<;2`P39bLT0dM7A0=WIf@3Ph>00
inOW0>g]k@3alO40mOGe0?Shn03jn_X0o?cl0`3moOd10?knoP40oooo0@3o
ool000030000003?cll0aLG500@0aLG500l0UiNG0;Ng]`3/k>`0^KVi08^;
R`2g]kL0ZZZZ0:BTY01nOWh0LG5a0<C4a03IfMT0m_Kf0?Win@2UYJD01035
aLD0102o_kl08R8R055AD@36a/H20<G5a@06065QH@0I6AT0^KVi0;no_`0R
8R80GEeM0P0@4100100E5AD0Y:BT0<;2`P15ADD3010@4006055AD@1QHF40
410@01TI6@2f][H0ADE50`0@4100201ADE40_kno028R8P0h>3P0_kno0<G5
a@2/[:`06QXJ0P0@41001@1bLW80aLG50<G5a@32`/80ADE500<0410@00<0
5ADE0:BTY036a/H02035aLD00`2/[:`06QXJ055AD@050<G5a@050;no_`0R
8R80<30`01XJ6P2>SXh0X`35aLD00`1fMWH0eMGE0=7Ad@0@0=7Ad@060=GE
e@1fMWH0aLG50<G5a@1fMWH0eMGE0P3AdM400`3oool09bLW0=7Ad@070=7A
d@030?ooo`0W9bL0dM7A0080dM7A00L0eMGE07IfMP35aLD0aLG505IFEP22
PX80QhN700D0RHV91@3oool508V9R@0308N7Q`22PX80EUIF00@0aLG50140
clo?0000002l_;`0_kno0<?3``39bLT0d=3@0=OGe`3MgMd0i>CT0>[ZjP3_
knl0lo?c0?KfmP3hn?P0no_k0?clo0020?gmo@80o_kn0P3oool000030000
003=cLd0`L7100<0`L710100MGEe09VIV@2WYjL0kNg]0<W9b@1cLg<0/;2`
08f=S@1cLg<0C4a<069RHP2n_[h0]KFe0;:b/P2][Jd0ZJVY1031`L40102k
^k/08R8R055AD@34a<@20<71`@0405mOG`0I6AT0^KVi0<;2`PL0`L7100D0
^k^k028R8P0h>3P0_[jn0<;2`P020<71`@0<0<?3``35aLD0_;bl028R8P0h
>3P0_[jn0<;2`P2k^k/08R8R03Ph>02n_[h0`/;22`31`L40101lO7`05ADE
0:BTY033`l<60<71`@80a<C400@0_[jn04A4A00h>3P0_[jn1031`L401`1O
Gel0410@010@400@4100>3Ph0;jn_P32`/80X@31`L400`1fMWH0f=SH0=GE
e@0@0=GEe@060=SHf01fMWH0`L710<71`@1fMWH0f=SH0P3EeMD00`3oool0
9bLW0?ooo`080?ooo`0602LW9`3EeMD0eMGE0=GEe@3Hf=P0MWIf0P31`L40
0`1FEUH0Q8B408R8R00408Z:RP<0oooo00@0:2PX0?ooo`3oool0oooo102:
RXX00`28R8P0Q8B405IFEP040<71`@0@0<g=c@000000a/K60<S8b03<c<`0
dM7A0=OGe`3Lg=`0h^;R0>SXj03]kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0
oOgm0P3no_h20?ooo`0000<000000<c<c02j^[X00P2m_Kd04@2n_[h0_Kfm
09jNWP3Shn<0][Jf09JFUP22PX80Ti>C08B4Q022PX80CTi>02`/;02LW9`0
Zj^[0?KfmP3hn?P0YjNW00@0_Kfm00@0]kNg024Q8@1ADE40a<C40P2m_Kd0
101MGEd06ATI0;Vi^@2o_kl80;fm_@0305eMG@0g=cL0^;Rh0080_Kfm00/0
GEeM01TI6@2i^KT0_kno05eMG@0g=cL0^;Rh0;fm_@2g]kL08B4Q03Ph>002
0;jn_P/0_Kfm00@0NWYj01DE5@2TY:@0_kno102m_Kd01`2g]kL08B4Q06m_
K`35aLD0_kno02<S8`1`L7001@2m_Kd01P1jNWX0=3@d03`l?01MGEd0>3Ph
0;Rh^1<0_KfmF`2n_[hd0;fm_@0307IfMP3Jf]X0eMGE0100eMGE00H0f][J
07IfMP2m_Kd0_Kfm07IfMP3Jf]X20=GEe@030?ooo`1bLW80LW9b00P0LW9b
00h09bLW0=GEe@3EeMD0eMGE0=[JfP1fMWH0_Kfm0;jn_P1FEUH0QhN708Z:
RP2=SHd0SXj>08f=S@<0oooo00<0:2PX08f=S@0X:2P00`3oool308f=S@04
08Z:RP27QhL0EUIF0;jn_P80_Kfm0140^[Zj0<c<c0000000d=3@0=;BdP3E
eMD0fMWI0=gMg@3Rh^80inOW0>c/k03`l?00m?Cd0?KfmP3hn?P0n_[j0?cl
o0030?gmo@030?knoP3oool0oooo0040oooo00000`000000b/[:0;Rh^003
0;Zj^P0@07alO02=SHd0N7Qh09VIV@3`l?00`/;20:NWY`3;bl/0_;bl0:>S
X`1]KFd0OGem0:b/[03kno/0o?cl0:JVYP<0^[Zj00@0^k^k0;Bd]00Q8B40
CDe=0`2j^[X00`1KFe/06ATI0:n_[`0>0;Zj^P0305]KF`0I6AT0[jn_00D0
^[Zj00@0];Bd024Q8@0g=cL0];Bd302j^[X00`1gMgL051@D09^KV`060;Zj
^P05096AT@0@4100410@010@401=CDd01`2j^[X00`2d];@08B4Q06][J`2T
0;Zj^P0307IfMP3Hf=P0f][J0100f][J00H0f=SH07IfMP2j^[X0^[Zj07If
MP3Hf=P20=[JfP030?ooo`1bLW80LW9b00P0LW9b00d09bLW0=[JfP3Jf]X0
f][J0=SHf01fMWH0^[Zj0;^k^`1FEUH0Rh^;08n?S`2>SXh0Shn?00<0oooo
00D0:2PX08n?S`2?Shl0Shn?02PX:0030?ooo`0508n?S`2>SXh0Shn?08^;
R`1FEUH00`2j^[X0402h^;P0b/[:0000003IfMT0f][J0=gMg@3Ph>00i>CT
0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`30?gmo@80o_kn0`3o
ool0000300000039bLT0^;Rh00<0^;Rh0100QXJ609jNWP3De=@0l?3`0?;b
lP3>c/h0`L710=gMg@3Bd]80XZ:R0<c<c039bLT0RHV90?clo03moOd0YjNW
k`2h^;P00`1fMWH0emOG0=gMg@0@0=gMg@060=OGe`1fMWH0^;Rh0;Rh^01f
MWH0emOG0`3MgMd;02LW9`<0gMgM00T0emOG07IfMP2h^;P0^;Rh05IFEP2@
T900TI6A09>CT`2DU9@00P3oool00`0X:2P0V9RH09RHV00309RHV00802PX
:03oool0oooo09BDU02CTi<0TI6A092@T01FEUH40;Rh^00?0<W9b@000000
hN7Q0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3emOD0moOg0?Win@3k
no/0o?cl00<0oOgm00<0o_kn0?ooo`3oool00P3oool0000300000039bLT0
^;Rh00@0^;Rh00l0QXJ609BDU03bl_80lo?c0:2PX03Xj>P0m_Kf0=_Kf`2I
VIT0n_[j0?_kn`3Lg=`0][Jf0?knoP2WYjL0k`2h^;P00`1fMWH0e]KF0>7Q
h@0@0>7Qh@060=KFeP1fMWH0^;Rh0;Rh^01fMWH0e]KF4@3QhN402@3Fe]H0
MWIf0;Rh^02h^;P0EUIF09FEU@2GUiL0W9bL0::RXP0202PX:0050:^[Z`2]
[Jd0Zj^[0:^[Z`2][Jd00P2[Zj/202PX:0050::RXP2LW9`0UiNG09FEU@1F
EUH0102h^;P03P39bLT000000>SXj03YjNT0jn_[0>g]k@3_knl0lO7a0?Cd
m03fm_H0moOg0?Win@3kno/0o?cl0`3moOd20?knoP@0oooo00000`000000
bLW90;Fe]@030;Fe]@0=07MgM`2PX:00h>3P0??cl`3UiND0k>c/0?KfmP3h
n?P0nOWi092@T03kno/0o?cl0?gmo@020?knoP030:NWY`2e]KD0]KFe0>d0
]KFe00<0MWIf0>g]k@3YjNT0403YjNT01P3]kNd0MWIf0;Fe]@2e]KD0MWIf
0>g]kA40jNWY00X0kNg]07IfMP2e]KD0]KFe05IFEP2KVi/0WYjN0:FUY@2b
/[80^[Zj0P2o_kl01030`<00_kno0;no_`30`<020;no_`070<30`02j^[X0
/[:b0:FUY@2NWYh0Vi^K05IFEP040;Fe]@0=0<W9b@000000k^k^0>o_k`3`
l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k0?clo0030?gmo@80o_kn
1@3oool0000300000039bLT0^[Zj00@0/K6a00<0SHf=0:ZZZP2ZZZX01@2Z
ZZX00`2WYjL0ZZZZ0:ZZZP040:ZZZ^l0/K6a00<0MWIf0=CDe03bl_80403b
l_801P3De=@0MWIf0;6a/@2a/K40MWIf0=CDe140l_;b00H0e=CD07IfMP2a
/K40/K6a05IFEP2SXj<20:RXZ0d0`l?30P2XZ:P00`2SXj<0EUIF0;6a/@02
0;6a/@040;Zj^P39bLT000000??cl`80m?Cd00L0mOGe0?KfmP3gmoL0n?Sh
0?[jnP3kno/0o?cl00@0oOgm00<0o_kn0?ooo`3oool0103oool000030000
0033`l<0bLW90?l0[Zj^0`2^[Zh0102=SHd0MWIf0<k>cP3kno/=0?ooo`04
0?_kn`3>c/h0MWIf08f=S@80[Zj^00@0SHf=07IfMP3>c/h0no_k3@3oool0
103kno/0c/k>07IfMP2=SHd20:j^[P0609VIV@1FEUH0]KFe0<_;b`3Ogml0
j>SX2@3WinL01P3Xj>P0gmoO0<_;b`2e]KD0EUIF09VIV@<0[Zj^00<0bLW9
0<?3``0000000P3fm_H01@3gmoL0n?Sh0?Shn03inOT0n_[j0080o?cl103m
oOd20?knoPH0oooo000010000000`l?30=CDe02g]kOo0:NWY`<0YjNW00<0
SHf=07IfMP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=S@1f
MWH0MWIf00l0MWIf00<0SHf=0:NWY`2WYjL00P2WYjL00`2=SHd0EUIF05IF
EP0?05IFEP0808f=S@2WYjL0YjNW0:NWY`2g]kL0e=CD0<?3``0000030?Wi
n@040?[jnP3kno/0o?cl0?clo0D0oOgm0P3no_h70?ooo`0000H000000:ZZ
ZP3WinL0e=CD0;jn_P2c/k?o0:ZZZT<0ZZZZ00H0/k>c0;jn_P3De=@0inOW
0:ZZZP0000020?_kn`<0o?cl1@3moOd30?knoPP0oooo00001`000000RXZ:
0=WIf@3lo?`0i>CT0=;BdP3<c<`0o`2n_[i10;jn_P080<c<c03Bd]80i>CT
0?clo03IfMT0RXZ:0000003lo?`80?gmo@80o_kn2P3oool000080000001Y
JFT0UiNG0=WIf@3lo?`0l_;b0>c/k03Shn?o0=WIfCl0fMWI00P0hn?S0>c/
k03bl_80o?cl0=WIf@2GUiL0JFUY000000L0oOgm0`3no_h;0?ooo`0000D0
00000?ooo`1YJFT0UiNG0=?Cd`020?clo?l0oooo@@3oool20?clo0050=?C
d`2GUiL0JFUY0?ooo`000000103moOd40?knoPd0oooo00000`000000oooo
0?ooo`0206UYJ@0308Z:RP2GUiL0ZZZZ0?l0ZZZZ@02ZZZX0102GUiL0RXZ:
06UYJ@1YJFT20?ooo`030000003no_h0o_kn00@0o_kn3`3oool00?l00000
C`00000E0?ooo`00\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{355, 227},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell[TextData[{
  "To evaluate the currently selected expression, use ",
  StyleBox["SelectionEvaluate", "MR"],
  ". "
}], "Text",
  PageBreakAbove->True,
  CellTags->{"i:14", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SelectionEvaluate[nb]\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40005U0000hb000`400?l0
0000o`00003oo`3ooomV0?ooo`00o`3ooomV0?ooo`003P3ooooo0?knoTT0
o_kn3`3oool000`0oooo103no_ko0?gmoDD0oOgm103no_h=0?ooo`002P3o
ool30?kno_l0oOgmB`3moOd30?knoP/0oooo00090?ooo`80o_kn203moOgo
0?clo3l0o?cl203moOd20?knoPX0oooo00070?ooo`<0o_kn1@3moOd30?cl
o?l0no_k@@3kno/30?clo0D0oOgm0`3no_h80?ooo`001P3oool20?knoPD0
oOgm0P3lo?`00`3kno/0n_[j0?Win@3o0?WinD80nOWi00@0n_[j0?_kn`3l
o?`0o?cl1@3moOd20?knoPL0oooo00050?ooo`80o_kn103moOd20?clo005
0?[jnP3inOT0n?Sh0?Shn03gmoL0o`3fm_I10?KfmP050?Ogm`3hn?P0n?Sh
0?Win@3jn_X00P3lo?`40?gmo@80o_kn1P3oool000D0oooo00<0o_kn0?gm
o@3moOd00P3moOd01`3lo?`0no_k0?[jnP3hn?P0moOg0?KfmP3emOD00P3d
m?Co0??clcl0lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0n_[j0?_kn`3l
o?`0103moOd00`3no_h0oooo0?ooo`040?ooo`00103oool20?knoP<0oOgm
00X0o?cl0?_kn`3inOT0n?Sh0?KfmP3emOD0lo?c0?7al@3`l?00kno_o`3^
k^ho0>k^kP0:0>o_k`3`l?00lO7a0??cl`3emOD0m_Kf0?Shn03inOT0no_k
0?clo0<0oOgm0P3no_h50?ooo`000`3oool20?knoP<0oOgm00/0o?cl0?_k
n`3inOT0moOg0?KfmP3dm?@0lO7a0>o_k`3]kNd0jn_[0>WYj@3o0>SXj3l0
j>SX00/0jNWY0>_[j`3]kNd0kno_0?7al@3dm?@0m_Kf0?Ogm`3inOT0no_k
0?clo0030?gmo@80o_kn103oool000<0oooo0100o_kn0?gmo@3moOd0oOgm
0?clo03kno/0nOWi0?Ogm`3emOD0lo?c0?3`l03]kNd0j^[Z0>OWi`3Ti>@0
h^;Ro`3QhN4o0>7Qh@0<0>;RhP3Ti>@0inOW0>[ZjP3]kNd0l?3`0??cl`3e
mOD0moOg0?Win@3kno/0o?cl0`3moOd00`3no_h0oooo0?ooo`020?ooo`00
0P3oool20?knoP<0oOgm00d0o?cl0?[jnP3hn?P0m_Kf0??cl`3_knl0k>c/
0>SXj03Ti>@0h>3P0=gMg@3Jf]X0fMWI0?l0f=SH?@3Hf=P03@3IfMT0f][J
0=gMg@3Ph>00i>CT0>SXj03/k>`0kno_0??cl`3fm_H0n?Sh0?[jnP3lo?`0
0`3moOd20?knoP<0oooo00020?ooo`0B0?knoP3moOd0oOgm0?gmo@3lo?`0
n_[j0?Shn03fm_H0m?Cd0?3`l03/k>`0inOW0>;RhP3MgMd0fMWI0=GEe@3B
d]80d=3@o`3?cllm0<o?c`0>0=3@d03Bd]80eMGE0=WIf@3MgMd0h^;R0>OW
i`3/k>`0l?3`0?Cdm03fm_H0n?Sh0?[jnP3lo?`30?gmo@030?knoP3oool0
oooo0040oooo00000`3oool0o_kn0?knoP030?gmo@0>0?clo03inOT0moOg
0?Gem@3alO40kNg]0>SXj03Rh^80g=cL0=OGe`3AdM40c<c<0<S8b036a/Ko
0<G5aCd0aLG500h0a/K60<S8b03<c<`0dM7A0=OGe`3Lg=`0h^;R0>SXj03]
kNd0lO7a0?Gem@3gmoL0nOWi0?clo0<0oOgm0P3no_h20?ooo`0000<0oooo
0?knoP3no_h00P3moOd03`3lo?`0no_k0?Shn03fm_H0lo?c0>o_k`3Zj^X0
i>CT0=gMg@3GemL0d=3@0<W9b@33`l<0_kno0;bl_03o0;^k^cd0^k^k00l0
_;bl0;no_`33`l<0bLW90=3@d03GemL0gMgM0>CTi03Zj^X0kno_0??cl`3f
m_H0n?Sh0?_kn`3lo?`00P3moOd20?knoP80oooo00000`3oool0o_kn0?gm
o@020?gmo@0?0?clo03jn_X0n?Sh0?Gem@3alO40kNg]0>OWi`3Ph>00fMWI
0=7Ad@39bLT0`/;20;^k^`2f][H0/[:b0?l0/K6a?@2a/K403`2b/[80][Jf
0;^k^`32`/80bLW90=7Ad@3IfMT0h>3P0>OWi`3]kNd0lO7a0?Gem@3hn?P0
n_[j0?clo0030?gmo@40o_kn0@3oool10?ooo`0000<0oooo0?knoP3moOd0
0P3moOd03`3lo?`0nOWi0?Ogm`3dm?@0l?3`0>_[j`3Ti>@0gMgM0=GEe@3<
c<`0`l?30;^k^`2c/k<0[Jf]0:RXZ03o0:JVYSd0YZJV00l0Z:RX0:f][@2c
/k<0^k^k0<?3``3<c<`0eMGE0=gMg@3Ti>@0jn_[0?3`l03dm?@0moOg0?Wi
n@3lo?`00`3moOd10?knoP40oooo0@3oool000030?ooo`3no_h0oOgm0080
oOgm00l0no_k0?Win@3fm_H0m?Cd0>o_k`3YjNT0h^;R0=[JfP3Bd]80b<S8
0;no_`2f][H0[Jf]0:FUY@2PX:00o`2NWYhm09jNWP0?0:2PX02UYJD0[Jf]
0;Jf]P2o_kl0b<S80=;BdP3Jf]X0h^;R0>WYj@3_knl0m?Cd0?KfmP3inOT0
no_k00<0oOgm0@3no_h10?ooo`40oooo00004`3oool0o_kn0?gmo@3moOd0
o?cl0?_kn`3inOT0m_Kf0??cl`3^k^h0j>SX0>7Qh@3IfMT0d=3@0<K6aP2l
_;`0/[:b0:RXZ02PX:00o`2JVYXo09ZJVP0?0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo003o0000054000000100VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000030000003lo?`0o?cl0?l0
o?clC@3lo?`04@000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo00000`000000o?cl0:^[Z`3o0:^[Zd`0Zj^[0180o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<0
00000?clo02[Zj/0o`1YJFU;06UYJ@0C0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYV03ooolg0>3Ph0030>k^kP2=SHd0WinO00`0WinO00@0SHf=0>KV
iP3/k>`0^[Zj702OWil00`2m_Kd0k>c/0>OWi`0U0>OWi`030>k^kP2=SHd0
WinO00`0WinO00<0SHf=0>KViP3Ph>00403Ph>00501YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00005
0000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph00e0>3P
h00309JFUP3Ng]h0fm_K00`0fm_K00@0g]kN09JFUP2FUYH0fm_K703:b/X0
0`3Kfm/0Xj>S0?_kn`0U0?_kn`0309JFUP3Ng]h0fm_K00`0fm_K00<0g]kN
09JFUP3Ph>001@3Ph>020?ooo`80h>3P0P3oool20>3Ph080oooo01D0h>3P
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3o
ool0h>3P0>3Ph00e0>3Ph00309JFUP3oool0c/k>00`0c/k>00@0oooo09JF
UP2FUYH0oooo703@d=000`3oool0UYJF0?_kn`0U0?_kn`0309JFUP3oool0
c/k>00`0c/k>00<0oooo09JFUP3Ph>00103Ph>020;>c/`060?ooo`3Ph>00
/k>c0;>c/`3oool0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D0
00000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P
00<0UYJF0?ooo`3>c/h0303>c/h0103oool0UYJF09JFUP3ooolL0<o?c`03
0?ooo`2FUYH0no_k02D0no_k00<0UYJF0?ooo`3>c/h0303>c/h00`3oool0
UYJF0>3Ph0040>3Ph080/k>c0P3Ph>020;>c/`80h>3P0P2c/k<20>3Ph00D
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?oo
o`3Ph>00h>3P00<0h>3P0100SXj>000000000000000009jNWP3IfMT06QXJ
0000000=3@d0d];B0>3Ph03IfMT06QXJ0000000=3@d0d];B0`3Ph>002@3L
g=`0DE5A0;Vi^@3Ph>00fMWI01XJ6P000000>3Ph0=WIf@0F0>3Ph00309JF
UP3oool0c/k>00D0c/k>00<0?Shn0<k>cP3>c/h0103>c/h01`3oool0UYJF
09JFUP3oool0dM7A0=3@d03AdM40103@d=070=7Ad@80d=3@0P3AdM430=3@
d0L0dM7A00<0oooo09JFUP3jn_X09@3jn_X00`2FUYH0oooo0<k>cP040<k>
cP0303hn?P3>c/h0c/k>00D0c/k>00<0oooo09JFUP3Ph>00403Ph>00501Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40
oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0
h>3P0>3Ph0040>3Ph00?0<C4a018B4P0fMWI0>3Ph01YJFT0WYjN0>3Ph02>
SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD00@0h>3P00P0[Jf]05AD
E03Ph>00[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF0?ooo`3>c/h0
103>c/h303hn?PD0c/k>00@0oooo09JFUP2FUYH0oooo203Fe]H20=OGe`08
0=KFeP2EUID0e]KF09FEU@3Fe]H0UIFE0=KFeP2EUID50=KFeP80emOG0`3F
e]H00`3oool0UYJF0?Win@0U0?Win@0309JFUP3oool0c/k>00<0c/k>0`0n
?Sh60<k>cP030?ooo`2FUYH0h>3P00T0h>3P0P3oool20>3Ph080oooo01D0
h>3P06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?
cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd1
0?knoP40oooo0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>00
0`3oool0h>3P0>3Ph0040>3Ph00@0<C4a018B4P0fMWI0=WIf@0V9RH0d];B
0>3Ph034a<@0B4Q80=;BdP0V9RH0d];B0>3Ph034a<@0B4Q80=WIf@<0h>3P
00P0g=cL055AD@2i^KT0[Jf]05ADE03Ph>00JFUY09jNWQH0h>3P00<0UYJF
0?ooo`3?cll00`3?cll303hn?PH0clo?00@0oooo09JFUP2FUYH0oooo0`3L
g=`70=gMg@080?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID70=gM
g@80g=cL00<0gMgM0?ooo`2FUYH09P3inOT00`2FUYH0oooo0<o?c`040<o?
c`<0?Shn1@3?cll00`3oool0UYJF0>3Ph0080>3Ph080/k>c01X0oooo0>3P
h02c/k<0/k>c0?ooo`3Ph>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001@000000o?cl0:^[Z`1Y
JFT0WYjN09H0h>3P00<0oooo0>3Ph03Ph>00103Ph>004034a<@0B4Q80=WI
f@3IfMT09RHV0=;BdP3Ph>00fMWI02HV9P3;bl/09RHV0=;BdP3Ph>00fMWI
02HV9P3Bd]840>3Ph0070<C4a018B4P0d];B01XJ6P0000003@d=0=;BdP0:
0>3Ph0L000001@3Ph>000`2FUYH0oooo0=CDe0020=CDe0<0?Shn1`3De=@0
103oool0UYJF09JFUP3oool30=cLg0L0gMgM00P0oooo09FEU@3oool0UIFE
0?ooo`2EUID0oooo09FEU@L0gMgM0P3Lg=`00`3MgMd0oooo09JFUP0V0?Sh
n00309JFUP3oool0e=CD00D0e=CD0`0n?Sh40=CDe0030?ooo`2FUYH0h>3P
00P0h>3P0P2c/k<20>3Ph080/k>c0P3Ph>00501YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000050000
003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P0>3Ph0040>3Ph00G
0<C4a018B4P0fMWI0=WIf@0V9RH0d];B0>3Ph03IfMT09RHV0<_;b`0V9RH0
d];B0>3Ph03IfMT09RHV0=;BdP3Lg=`0BDU90000000=3@d0d];B06UYJ@2N
WYh03`3Ph>05000000H0h>3P00@0UYJF0?ooo`3Kfm/0fm_K0`0n?Sh80=_K
f`050?ooo`2FUYH0UYJF0?ooo`3Ti>@00P3Shn<20>CTi0<0hn?S0P3Ti>@0
2@3oool0UIFE0?ooo`2EUID0oooo09FEU@3oool0UIFE0>CTi0040>?Sh`<0
i>CT00@0hn?S0>CTi03oool0UYJF9P3gmoL00`2FUYH0oooo0=_Kf`060=_K
f`<0?Shn0`3Kfm/00`3oool0UYJF0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`00
00D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P00@0
h>3P01P0a<C404Q8B03IfMT0fMWI02HV9P3Bd]80h>3P0<C4a018B4P0d];B
02HV9P3Bd]80h>3P0<C4a018B4P0fMWI0:f][@1DE5@0h>3P06UYJ@1nOWh0
a<C404Q8B03IfMT?0>3Ph0<000001`3Ph>000`2FUYH0oooo0>CTi0020>CT
i0<0?Shn1`3Ti>@01@3oool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X5
0>WYj@090?ooo`2EUID0oooo09FEU@3oool0UIFE0?ooo`2EUID0j^[Z00H0
jNWY00D0j^[Z0>WYj@3YjNT0oooo09JFUP0V0?Gem@0309JFUP3oool0i>CT
00D0i>CT0`0n?Sh40>CTi0030?ooo`2FUYH0h>3P00d0h>3P0P3oool05@3P
h>00JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph003
0?ooo`3Ph>00h>3P00<0h>3P01T0SXj>0000000h>3P0fMWI0>3Ph01YJFT0
WYjN0>3Ph02>SXh0E5AD0>3Ph01YJFT0WYjN0>3Ph02>SXh0E5AD0>3Ph02]
[Jd0E5AD0>3Ph01YJFT0OWin0>3Ph01YJFT0WYjN0100h>3P00<000000>3P
h03Ph>001P3Ph>000`2FUYH0oooo0>g]k@030>g]k@<0?Shn1P3]kNd01@3o
ool0UYJF09JFUP3oool0j^[Z0080jNWY0P3Zj^X50>WYj@090?ooo`2EUID0
oooo09FEU@3oool0UIFE0?ooo`2EUID0j^[Z00H0jNWY00D0j^[Z0>WYj@3Y
jNT0oooo09JFUP0V0?Cdm00309JFUP3oool0kNg]00@0kNg]0`0n?Sh50>g]
k@030?ooo`2FUYH0h>3P00`0h>3P0P2c/k<05P3oool0h>3P06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000D000000?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P
00@0h>3P01T0fMWI059BDP3IfMT0h>3P0=WIf@0J6QX0000000d=3@3Bd]80
h>3P0=WIf@0J6QX0000000d=3@3Bd]80h>3P0=WIf@0J6QX0000000d=3@3B
d]80h>3P0<C4a00M7Ad0d];B01P0h>3P00<0UYJF0?ooo`3dm?@0103dm?@3
03hn?PD0m?Cd00@0oooo09JFUP2FUYH0oooo2P3]kNd01`3oool0kNg]0?oo
o`3]kNd0oooo0>g]k@3oool02`3]kNd00`3oool0UYJF0?7al@0U0?7al@03
09JFUP3oool0m?Cd00<0m?Cd0`0n?Sh60?Cdm0030?ooo`2FUYH0h>3P00`0
h>3P0P2c/k<20>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY
09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0UYJF0?ooo`3lo?`0
1@3lo?`00`0n?Sh0o?cl0?clo0040?clo0040?ooo`2FUYH0UYJF0?oooa`0
kNg]00<0oooo09JFUP3`l?009@3`l?000`2FUYH0oooo0?clo0040?clo003
03hn?P3lo?`0o?cl00D0o?cl00<0oooo09JFUP3Ph>00403Ph>00501YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000050000003lo?`0Zj^[06UYJ@2NWYh0UP3Ph>000`3oool0h>3P
0>3Ph00e0>3Ph00309JFUP3oool0oooo00d0oooo0P2FUYH00`3oool0moOg
0?Ogm`0J0?Ogm`030?ooo`2FUYH0kno_02D0kno_00<0UYJF0?ooo`3oool0
3@3oool00`2FUYH0h>3P0>3Ph00?0>3Ph00D06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000D00000
0?clo02[Zj/0JFUY09jNWP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0
UYJF0=OGe`3oool0303oool0103GemL0UYJF09JFUP3jn_XL0?ooo`030?[j
nP2FUYH0kno_02D0kno_00<0UYJF0=OGe`3oool0303oool00`3GemL0UYJF
0>3Ph00@0>3Ph00D06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`20?gmo@40o_kn0@3oool10?ooo`0000D000000?clo02[Zj/0JFUY09jN
WP2F0>3Ph0030?ooo`3Ph>00h>3P03D0h>3P00<0f][J09:BTP2f][H02`2e
]KD01@2d];@0Shn?0=[JfP3Fe]H0/;2`01`0^;Rh00<0ZJVY0<k>cP3^k^h0
9@3^k^h00`3Jf]X0TY:B0;Jf]P0;0;Fe]@030;Bd]02?Shl0f][J0140h>3P
01@0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFVG09jNWP030?ooo`3P
h>00h>3P03D0h>3P4@3/k>`P0>_[jbH0inOW4@3/k>`A0>3Ph00D06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh0?ooo`030>c/k03^k^h0
SHf=00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo>03oool00`3/k>`0UYJF0=kNgP0<0=_Kf`0F0=kNgP2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00<0k>c/
09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUYo`3ooolh0?ooo`030>c/k02FUYH0oooo00`0c/k>01H0oooo
09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0
clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd1
0?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo>03oool0
0`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocP0oooo00<0k>c/09JFUP3oool01@3>c/h0
0`0n?Sh0c/k>0<k>cP040<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFWo0?ooocP0oooo00<0k>c/09JFUP3oool0103>c/h3
03hn?PD0c/k>01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo>03oool00`3/k>`0UYJF0?ooo`030<o?c`D0?Shn103?cll0
5P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh
0?ooo`030>c/k02FUYH0oooo0080e=CD0`0n?Sh0103De=@0?Shn03hn?P0n
?Sh30=CDe00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocP0oooo00@0k>c/09JFUP3oool0fm_K0`0n?Sh30=_Kf`<0?Shn
0P3Kfm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY
o`3ooolh0?ooo`030>c/k02FUYH0oooo0080i>CT00<0?Shn0>CTi03Ti>@0
0`3Ti>@06P0n?Sh0i>CT0>CTi03Ti>@0oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo>03oool00`3/k>`0UYJF0?ooo`0<0>g]
k@0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocP0oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`00000h000000030>c/k02FUYH0oooo
00`0o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo>03oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00<0k>c/09JFUP3G
emL0303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolh0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?
S`3Jf]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh0?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocP0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo>03oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolX0?ooo`D0EEEE0P3oool505EEE@@0oooo00X0inOW0>k^
kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo;03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo
0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJBT0oooo0P1EEED:0?ooo`80EEEEm03oool00`1E
EED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^
kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JBT0oooo00<0EEEE0?ooo`3oool02P3oool00`1EEED0oooo0?ooo`3b0?oo
o`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY6`3oool305EEE@<0oooo0`1EEED00`3oool0EEEE05EEE@020?oo
o`0505EEE@3oool0oooo0?ooo`1EEED0103oool01@1EEED0oooo0?ooo`3o
ool0EEEE00h0oooo00P0E5AD000000000000000000l?3`3_knl0gmoO018B
4P<0000000D03`l?0>o_k`3oool0oooo0::RXP0200000006061PH03oool0
oooo0?ooo`3gmoL0GUiN1@3oool0102RXZ8000000000001PH6020?ooo`05
0=oOg`0B4Q80000000T92@3Bd]800P3oool01@3Ogml04Q8B000000092@T0
d];B00<0oooo00@0XZ:R000000000000H61P0P3oool01@3Ogml04Q8B0000
00092@T0d];B0080oooo00P0gmoO018B4P0000002@T90=;BdP3oool0gmoO
018B4P<0000000@03`l?0>o_k`3Ogml04Q8B0`000000100?3`l0kno_0?oo
o`36a/H3000000040;Bd]03oool0gmoO018B4P<0000000@03`l?0>o_k`3O
gml04Q8B0`000000100?3`l0kno_0?ooo`2RXZ82000000030451@@3oool0
oooo0080oooo00@0moOg01dM7@000000@D510P3oool01@3Ogml04Q8B0000
00092@T0d];B0080oooo00H0E5AD000000000000000000l?3`3_knl20?oo
o`040?Ogm`0M7Ad000000451@@80oooo00@0XZ:R000000000000@D51103o
ool0102RXZ8000000000001PH6030?ooo`030?Ogm`1NGUh0oooo00<0oooo
00D0gmoO018B4P0000002@T90=;BdP020?ooo`080=oOg`0B4Q80000000T9
2@3Bd]80oooo0=oOg`0B4Q830000000800l?3`3_knl0oooo0=oOg`0B4Q80
000000T92@3Bd]820?ooo`050=oOg`0B4Q80000000T92@3Bd]800P3oool0
1@3Ogml04Q8B000000092@T0d];B00<0oooo00<0moOg05iNGP3oool0103o
ool0102RXZ8000000000001PH60?0?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6P3oool01`1EEED0oooo
0?ooo`3oool0EEEE0?ooo`1EEED00P3oool00`1EEED0oooo05EEE@030?oo
o`0505EEE@3oool0oooo0?ooo`1EEED0103oool01@1EEED0oooo0?ooo`3o
ool0EEEE0100oooo00<0Hf=S0>o_k`3oool00P3oool00`1DE5@0H61P0?oo
o`040?ooo`060=oOg`1CDe<0oooo0?ooo`27QhL0kno_0`3oool00`21PH40
d];B0?ooo`020?ooo`0<0=oOg`1CDe<0oooo0?ooo`27QhL0kno_0?Ogm`0V
9RH0d];B0?ooo`3Ogml0De=C0P3oool01@2XZ:P0d];B0?ooo`3Ogml0VYZJ
0080oooo00`0gmoO05=CD`3oool0oooo08N7Q`3_knl0oooo0:RXZ03Bd]80
oooo0=oOg`2JVYX20?ooo`050:RXZ03Bd]80oooo0=oOg`2JVYX00P3oool0
0`1DE5@0H61P0?ooo`040?ooo`0305ADE01PH600oooo00@0oooo00D0QhN7
0>o_k`3oool0moOg07emO@020?ooo`0305ADE01PH600oooo00@0oooo00<0
E5AD061PH03oool01`3oool00`2RXZ80Shn?0?ooo`040?ooo`0906=SH`3_
knl0oooo0?ooo`2XZ:P0d];B0?ooo`3Ogml0VYZJ00@0oooo00<0Hf=S0>o_
k`3oool01@3oool00`1SHf<0kno_0?ooo`040?ooo`060::RXP2?Shl0oooo
0?ooo`3Ogml0De=C0P3oool00`27QhL0kno_0?ooo`020?ooo`030861P@3B
d]80oooo0080oooo00`0Z:RX0=;BdP3oool0gmoO09ZJVP3oool0moOg02HV
9P3Bd]80oooo0<K6aP11@D420?ooo`0305ADE01PH600oooo00@0oooo01<0
Z:RX0=;BdP3oool0gmoO09ZJVP3oool0moOg02HV9P3Bd]80oooo0=oOg`1C
De<0oooo0?Ogm`0V9RH0d];B0?ooo`36a/H0@D5100@0oooo00<0PH610=;B
dP3oool00P3oool01P3Ogml0De=C0?ooo`3oool0QhN70>o_k`h0oooo00<0
EEEE0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^
k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j
0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFTJ0?ooo`0705EEE@3oool0oooo0?ooo`1EEED0oooo05EEE@020?ooo`03
05EEE@3oool0EEEE00<0oooo00D0EEEE0?ooo`3oool0oooo05EEE@040?oo
o`0605EEE@3oool0oooo0?ooo`1EEED0oooo101EEED;0?ooo`0306=SH`3_
knl0oooo00<0oooo00<0N7Qh0451@@3oool00`3oool01P1hN7P0Shn?0?oo
o`3oool0PH610=;BdP<0oooo00<0a/K608n?S`3oool00P3oool01P1hN7P0
Shn?0?ooo`3oool0PH610=;BdPD0oooo00P0Hf=S0>o_k`3oool0Hf=S0>o_
k`3oool0moOg05iNGP80oooo00`0N7Qh08n?S`3oool0oooo0861P@3Bd]80
oooo06=SH`3_knl0oooo0?Ogm`1NGUh20?ooo`0506=SH`3_knl0oooo0?Og
m`1NGUh00`3oool00`1hN7P0@D510?ooo`040?ooo`0307QhN011@D40oooo
00<0oooo00D0Hf=S0>o_k`3oool0moOg07emO@030?ooo`0307QhN011@D40
oooo00@0oooo00<0N7Qh0451@@3oool01`3oool0101SHf<0kno_0?ooo`1D
E5@3000000080451@@3oool0oooo06=SH`3_knl0oooo0?Ogm`1NGUh40?oo
o`0306=SH`3_knl0oooo0080oooo00D0E5AD00000000000000000451@@06
0?ooo`0506=SH`3_knl0oooo07QhN02?Shl00P3oool00`21PH40d];B0?oo
o`020?ooo`030<K6aP2?Shl0oooo0080oooo00D0Hf=S0>o_k`3oool0moOg
05iNGP060?ooo`0606=SH`3_knl0oooo0?ooo`1hN7P0@D51103oool01@1S
Hf<0kno_0?ooo`3gmoL0GUiN00H0oooo00<0Hf=S0>o_k`3oool0103oool0
0`1SHf<0kno_0?ooo`020?ooo`030<K6aP2?Shl0oooo0080oooo00H0N7Qh
08n?S`3oool0oooo0861P@3Bd]8>0?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY6P3oool01`1EEED0oooo
0?ooo`3oool0EEEE0?ooo`1EEED00P3oool00`1EEED0oooo05EEE@030?oo
o`0505EEE@3oool0oooo0?ooo`1EEED0103oool01@1EEED0oooo0?ooo`3o
ool0EEEE0100oooo00<0Hf=S0>o_k`3oool0103oool00`2RXZ80@D510?oo
o`020?ooo`0607QhN00?3`l0kno_0?Ogm`0/;2`0kno_0`3oool00`3Ogml0
LW9b0?ooo`020?ooo`0607QhN00?3`l0kno_0?Ogm`0/;2`0kno_103oool0
1P3gmoL0;2`/0>o_k`3oool0Hf=S0>o_k`80oooo00`0Hf=S0>o_k`3oool0
N7Qh00l?3`3_knl0moOg02`/;03_knl0oooo06=SH`3_knl20?ooo`0506=S
H`3_knl0oooo06=SH`3_knl00P3oool00`1SHf<0kno_0?ooo`020?ooo`03
0::RXP11@D40oooo00@0oooo00<0XZ:R0451@@3oool00P3oool01@2XZ:P0
d];B0?ooo`3gmoL0alO700@0oooo00<0XZ:R0451@@3oool0103oool00`2R
XZ80@D510?ooo`020?ooo`0<0?Ogm`0M7Ad0000000T92@18B4P0d];B0?oo
o`21PH40d];B0?ooo`1SHf<0kno_0P3oool01P1SHf<0kno_0?ooo`3oool0
Hf=S0>o_k`<0oooo00<0Hf=S0>o_k`3oool00P3oool01@21PH40d];B0?oo
o`1SHf<0kno_0080oooo00d0moOg01dM7@0000002@T904Q8B03Bd]80oooo
07QhN00?3`l0kno_0?Ogm`0/;2`0kno_00<0oooo00<0gmoO079bLP3oool0
0P3oool01P1SHf<0kno_0?ooo`3oool0Hf=S0>o_k`@0oooo00<0a/K6061P
H03oool00`3oool00`2RXZ80@D510?ooo`020?ooo`0606=SH`3_knl0oooo
0?ooo`1SHf<0kno_103oool00`3gmoL0;2`/0>o_k`040?ooo`030<K6aP1P
H600oooo00<0oooo00<0gmoO079bLP3oool00P3oool01P1hN7P03`l?0>o_
k`3gmoL0;2`/0>o_k`h0oooo00<0EEEE0?ooo`3oool0103oool00`1EEED0
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFTJ0?ooo`0705EEE@3oool0oooo0?oo
o`1EEED0oooo05EEE@020?ooo`@0EEEE0P3oool01@1EEED0oooo0?ooo`3o
ool0EEEE00@0oooo00H0EEEE0?ooo`3oool0oooo05EEE@3oool405EEE@/0
oooo00<0Hf=S0>o_k`3oool01@3oool01P2RXZ80];Bd0?ooo`3oool0Z:RX
06m_K`80000000<0];Bd0?ooo`3oool00`3oool01P1SHf<0kno_0?ooo`3o
ool0Z:RX06m_K`80000000@0];Bd0?ooo`3oool0XZ:R0`0000001@2d];@0
oooo0?ooo`1SHf<0kno_0080oooo00D0Hf=S0>o_k`3oool0Z:RX06m_K`02
000000050;Bd]03oool0oooo06=SH`3_knl00P3oool01@1SHf<0kno_0?oo
o`1SHf<0kno_0080oooo00<0Hf=S0>o_k`3oool00`3oool00`2RXZ80];Bd
0?ooo`040?ooo`090::RXP2d];@0oooo0?ooo`3Ogml04Q8B000000092@T0
d];B00D0oooo00<0XZ:R0;Bd]03oool0103oool0402RXZ80];Bd0?ooo`3o
ool0XZ:R08n?S`3oool0a/K600T92@3Bd]80oooo0=oOg`1CDe<0oooo06=S
H`3_knl20?ooo`0606=SH`3_knl0oooo0?ooo`1SHf<0kno_0`3oool00`1S
Hf<0kno_0?ooo`020?ooo`050=oOg`1CDe<0oooo06=SH`3_knl00P3oool0
2@2RXZ80Shn?0?ooo`36a/H02@T90=;BdP3oool0Z:RX06m_K`0200000003
0;Bd]03oool0oooo00<0oooo00H0Hf=S0>o_k`3oool0oooo06=SH`3_knl2
0?ooo`0806=SH`3_knl0oooo0?ooo`3Ogml04Q8B00l?3`3_knl50?ooo`06
0::RXP2d];@0oooo0?ooo`1SHf<0kno_0P3oool0101SHf<0kno_0?ooo`2R
XZ83000000080;Bd]03oool0oooo0?ooo`3Ogml04Q8B00l?3`3_knl50?oo
o`0606=SH`3_knl0oooo0?ooo`2XZ:P0Kfm_0P0000000`2d];@0oooo0?oo
o`0=0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3o
ool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY6P3oool01@1EEED0oooo0?ooo`3oool0EEEE00H0oooo
00D0EEEE0?ooo`3oool0oooo05EEE@020?ooo`80EEEE0`3oool205EEE@<0
oooo00<0EEEE0?ooo`3oool03P3oool00`1SHf<0kno_0?ooo`050?ooo`06
0?Ogm`1NGUh0oooo0?ooo`36a/H0Shn?203oool01P2RXZ80];Bd0?ooo`3o
ool0a/K608n?S`D0oooo00<0XZ:R08n?S`3oool0103oool01@1SHf<0kno_
0?ooo`3gmoL0GUiN0080oooo00<0a/K608n?S`3oool0103oool01@1SHf<0
kno_0?ooo`3gmoL0GUiN0080oooo00D0Hf=S0>o_k`3oool0moOg05iNGP05
0?ooo`030?Ogm`1NGUh0oooo00@0oooo00T0moOg05iNGP3oool0oooo0861
P@3Bd]80oooo0=oOg`1bLW801@3oool00`3gmoL0GUiN0?ooo`040?ooo`06
0?Ogm`1NGUh0oooo0?ooo`21PH40d];B0P3oool0201SHf<0kno_0?ooo`3o
ool0XZ:R0;Bd]01SHf<0kno_0P3oool01@1SHf<0kno_0?ooo`3gmoL0GUiN
00@0oooo00<0Hf=S0>o_k`3oool00`3oool0102RXZ80];Bd06=SH`3_knl2
0?ooo`090861P@3Bd]80oooo0?ooo`1SHf<0kno_0?ooo`36a/H0Shn?00P0
oooo00T0XZ:R0;Bd]03oool0oooo06=SH`3_knl0oooo0?Ogm`1NGUh01@3o
ool00`36a/H0H61P0?ooo`040?ooo`090?Ogm`1NGUh0oooo0?ooo`1SHf<0
kno_0?ooo`3gmoL0GUiN0080oooo00<0XZ:R08n?S`3oool01`3oool00`36
a/H0H61P0?ooo`040?ooo`060::RXP2d];@0oooo0?ooo`36a/H0Shn?4P3o
ool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080oooo00X0
inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT0
6@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJA/0oooo0`1EEED;0?ooo`80EEEE0P3oool00`1EEED0oooo0?oo
o`020?ooo`0305EEE@3oool0oooo0080EEEE3P3oool0101DE5@0];Bd06=S
H`3_knl30?ooo`0505ADE02d];@0oooo0<K6aP11@D400`3oool00`1DE5@0
Shn?0?ooo`020?ooo`060?Ogm`1NGUh0oooo0?ooo`3Ogml0LW9b0`3oool0
0`1DE5@0Shn?0?ooo`030?ooo`030::RXP2?Shl0oooo00@0oooo00D0XZ:R
061PH03oool0a/K608n?S`030?ooo`0305ADE02?Shl0oooo00<0oooo00D0
XZ:R061PH03oool0a/K608n?S`020?ooo`050::RXP1PH600oooo0<K6aP2?
Shl00P3oool01@1DE5@0];Bd0?ooo`36a/H0@D510080oooo00D0E5AD0;Bd
]03oool0a/K60451@@020?ooo`0507QhN02d];@0oooo0::RXP2?Shl00P3o
ool01@1DE5@0];Bd0?ooo`36a/H0@D510080oooo00D0E5AD0;Bd]03oool0
a/K60451@@020?ooo`050::RXP2?Shl0oooo0<K6aP1PH6000`3oool0103g
moL0GUiN06=SH`3_knl20?ooo`050::RXP1PH600oooo0<K6aP2?Shl00P3o
ool0101DE5@0];Bd06=SH`3_knl40?ooo`040?Ogm`1NGUh0Hf=S0>o_k`80
oooo00D0XZ:R08n?S`3oool0a/K6061PH0030?ooo`0305ADE02?Shl0oooo
0080oooo00H0moOg05iNGP3oool0oooo0=oOg`1bLW820?ooo`050::RXP1P
H600oooo0<K6aP2?Shl00P3oool01@1hN7P0H61P0?ooo`36a/H0H61P0080
oooo00D0E5AD0;Bd]03oool0a/K60451@@020?ooo`050::RXP1PH600oooo
0<K6aP2?Shl00P3oool00`2RXZ80Shn?0?ooo`040?ooo`0807QhN01PH600
oooo0<K6aP1PH600oooo0?Ogm`1NGUh20?ooo`030=oOg`1bLW80oooo0080
oooo00<0E5AD08n?S`3oool03@3oool405EEE@H0oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYA@3oool0103gmoL07AdM00l?3`3_knl30?oo
o`050?Ogm`0M7Ad0000000T92@3Bd]80103oool01P1DE5@0000000T92@3B
d]80moOg01dM7@<0000000<0H61P0?ooo`3oool00P3oool01P1DE5@00000
00T92@3Bd]80oooo0::RXP<0000000P0Shn?0?ooo`3oool0oooo07QhN000
00003`l?0>o_k`@0oooo00@0E5AD000000092@T0d];B0P3oool0101hN7P0
000000l?3`3_knl30?ooo`0407QhN00000003`l?0>o_k`80oooo00D0moOg
01dM7@0000002@T90=;BdP020?ooo`050?Ogm`0M7Ad0000000T92@3Bd]80
0P3oool0103gmoL07AdM00000011@D430?ooo`050?Ogm`0M7Ad0000000T9
2@3Bd]800P3oool01@3gmoL07AdM000000092@T0d];B00<0oooo00@0E5AD
0000000?3`l0kno_103oool00`2RXZ803`l?0>o_k`030?ooo`0407QhN000
00003`l?0>o_k`80oooo00@0moOg01dM7@0?3`l0kno_1@3oool00`2RXZ80
3`l?0>o_k`030?ooo`0405ADE00000003`l?0>o_k`@0oooo00H0E5AD0000
00092@T0d];B0?Ogm`0M7Ad300000008061PH03oool0oooo0?ooo`1hN7P0
000000l?3`3_knl30?ooo`0407QhN00000002@T90=;BdP80oooo00D0moOg
01dM7@0000002@T90=;BdP030?ooo`0407QhN00000003`l?0>o_k`80oooo
00D0XZ:R000000000000000008n?S`030?ooo`0707QhN00000002@T90=;B
dP3oool0moOg01dM7@0300000003061PH03oool0oooo0080oooo00@0E5AD
000000092@T0d];B3P3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3o
ool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Ge
m@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJOl0oooo:`3oool205EEE@H0oooo00<0
EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3d
m?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolZ0?ooo`0305EEE@3o
ool0EEEE00H0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_
k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0
no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3o
oolY0?ooo`0405EEE@3oool0oooo05EEE@H0oooo00<0EEEE0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolX0?ooo`0505EEE@3oool0oooo0?ooo`1E
EED01P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobL0
oooo1P1EEED60?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo<`3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oooc<0
oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`
0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3W
inL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolc0?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo<`3oool00`1EEED0
oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03e
mOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?oooc<0oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooolc0?ooo`0305EEE@3oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo<`3oool00`1EEED0oooo0?ooo`020?ooo`0:
0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi
01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jN
WP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf
0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?oooc<0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0
k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[j
nP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV
0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0
no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolc0?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo<`3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobP0
oooo1@1EEED60?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_
knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k
0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo;03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo;03oool00`1EEED0oooo0?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJBH0oooo0P1EEED:0?ooo`80
EEEEm`3oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJBH0oooo00<0EEEE0?ooo`3oool02P3oool00`1EEED0
oooo0?ooo`3e0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY7P3oool00`1EEED0oooo05EEE@020?ooo`04
05EEE@3oool0oooo05EEE@<0oooo00<0EEEE0?ooo`3oool00P3oool01@1E
EED0oooo0?ooo`3oool0EEEE0080oooo00<0EEEE0?ooo`3oool02`3oool0
0`2RXZ80000000000002000000030451@@3gmoL0oooo0=l0oooo00<0EEEE
0?ooo`3oool0103oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTN
0?ooo`0305EEE@3oool0EEEE0080oooo00@0EEEE0?ooo`3oool0EEEE0`3o
ool00`1EEED0oooo0?ooo`020?ooo`0505EEE@3oool0oooo0?ooo`1EEED0
4@3oool00`1hN7P03`l?0>o_k`3S0?ooo`0305EEE@3oool0oooo00@0oooo
00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7a
l@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY7P3oool00`1EEED0oooo
05EEE@020?ooo`0405EEE@3oool0oooo05EEE@<0oooo00<0EEEE0?ooo`3o
ool00P3oool01@1EEED0oooo0?ooo`3oool0EEEE00@0oooo101EEED:0?oo
o`030<K6aP092@T0d];B0>80oooo00<0EEEE0?ooo`3oool0103oool00`1E
EED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFTN0?ooo`0305EEE@3oool0EEEE
0080oooo00@0EEEE0?ooo`3oool0EEEE0`3oool00`1EEED0oooo0?ooo`02
0?ooo`0505EEE@3oool0oooo0?ooo`1EEED04`3oool00`3Ogml06a/K0=;B
dP3Q0?ooo`0305EEE@3oool0oooo00@0oooo00<0EEEE0?ooo`3oool00P3o
ool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUY7P3oool00`1EEED0oooo05EEE@0205EEE@<0oooo00D0
EEEE0?ooo`3oool0oooo05EEE@040?ooo`0505EEE@3oool0oooo0?ooo`1E
EED00P3oool00`1EEED0oooo05EEE@0305EEE@`0oooo00<0XZ:R0451@@3g
moL0h03oool00`1EEED0oooo0?ooo`040?ooo`0305EEE@3oool0oooo0080
oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh
0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJAh0oooo00<0EEEE0?ooo`3oool01@3oool00`1EEED0
oooo0?ooo`0205EEE@<0oooo0P1EEED30?ooo`0305EEE@3oool0oooo00h0
oooo00L0gmoO024Q8@3_knl0oooo0<K6aP11@D40moOg00D0000000<0oooo
0000000000000`3oool200000=<0oooo00<0EEEE0?ooo`3oool0103oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTN0?ooo`0305EEE@3oool0
oooo00D0oooo0P1EEED20?ooo`0305EEE@3oool0oooo0080oooo00<0EEEE
0?ooo`3oool00P1EEED@0?ooo`060?Ogm`0M7Ad0];Bd0?ooo`1hN7P0H61P
0`3oool010000000oooo0?ooo`0000020?ooo`030000003oool000000080
oooo00<000000?ooo`3oool0d03oool00`1EEED0oooo0?ooo`040?ooo`03
05EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40
m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UY
J@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH
0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40
oooo0@3oool000040000003lo?`0Zj^[06UYJDD0oooo00D0gmoO018B4P00
00003`l?0>o_k`030?ooo`040000003oool0oooo00000080oooo00<00000
0?ooo`0000000P3oool00`000000oooo0?ooo`3@0?ooo`0305EEE@3oool0
oooo00@0oooo00<0EEEE0?ooo`3oool00P3oool02P3WinL0k^k^0>o_k`3_
knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I0?[jnP3kno/0no_k
0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0
aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo002
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYC@3oool0
10000000oooo0?ooo`0000020?ooo`030000003oool000000080oooo00<0
00000?ooo`3oool0c`3oool205EEE@H0oooo00<0EEEE0?ooo`3oool00P3o
ool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P2
0?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYC@3oool010000000oooo0?ooo`0000020?ooo`030000
003oool000000080oooo00<000000?ooo`3oool0cP3oool00`1EEED0oooo
05EEE@060?ooo`0305EEE@3oool0oooo0080oooo00X0inOW0>k^kP3_knl0
kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_k
n`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`0
0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD/0oooo
0`0000030?ooo`8000000`3oool200000=00oooo00@0EEEE0?ooo`3oool0
EEEE1P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_
k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0
no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobP0
oooo00D0EEEE0?ooo`3oool0oooo05EEE@060?ooo`0305EEE@3oool0oooo
0080oooo00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0
n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo9`3oool605EEE@80oooo1@1EEED40?oo
o`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080
nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFWo0?ooocP0oooo00X0inOW0>k^kP3_knl0kno_0?3`l03a
lO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0no_k0?_kn`3kno/0inOW
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo>03oool02P3W
inL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Win@0I
0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolh0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cd
m03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00X0inOW0>k^
kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3inOT06@3jn_X0
no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a
/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_k
n`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo>03oool00`3/k>`0l?3`08j>SP050:2PX00309nOW`2PX:00X:2P
00@0X:2P01H0T92@0>c/k01YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo>03oool00`3/k>`0UYJF0=kNgP0<0=?Cd`0F0=kNgP2FUYH0JFUY
0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=KFeP3AdM40clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0
oooo>03oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cL
g03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUYo`3ooolh0?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@d=00oooo09JF
UP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?
0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3n
o_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0gMgM0=OGe`3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo>03oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0
i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolh0?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooobl0
oooo1@1EEED40?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3T
i>@0gMgM0=gMg@3GemL0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?cl
o0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX
0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?oooc<0oooo00<0EEEE0?ooo`3oool0
0P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3M
gMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJ
VP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c
0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@00000
0?clo02[Zj/0JFUYo`3ooolc0?ooo`0305EEE@3oool0oooo0080oooo00@0
k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3A
dM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJOl0oooo<`3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0gMgM0=gMg@3Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
oc<0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool02@1hN7P0
00000000002BTY804Q8B0000001PH600oooo0<K6aP03000000040451@@3g
moL0oooo0::RXP80000000D0@D510?Ogm`3oool0oooo0<K6aP0300000003
0451@@3gmoL0oooo0080oooo00<0a/K60000000000000`0000000`2d];@0
gmoO018B4P03000000030;Bd]03oool0oooo0080oooo00<0gmoO018B4P00
00000P0000004P2d];@0oooo0?ooo`3Ogml04Q8B000000092@T0d];B0=oO
g`0B4Q80000006UYJ@0B4Q80@D51071`L0092@T0d];B0<K6aP<0000000<0
@D510?Ogm`3oool0103oool01`36a/H0000000000011@D40moOg0?ooo`36
a/H00`0000003011@D40lo?c05ADE00000002@T90<[:bP0M7Ad000000451
@@3gmoL0oooo0<K6aP80000000L0@D510?Ogm`3oool0oooo0<K6aP000000
];Bd07h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYA03oool04P3k
no/0E5AD0;Bd]03oool0gmoO024Q8@3_knl0a/K60000002d];@0oooo0?Og
m`0M7Ad0];Bd0?ooo`3kno/0E5AD0;Bd]0<0oooo00L0a/K60000002d];@0
oooo0?Ogm`0M7Ad0];Bd00D0oooo00<0gmoO01/K6`3Bd]800P3oool01`3O
gml08B4Q0>o_k`3oool0gmoO01/K6`3Bd]800`3oool0703Ogml08B4Q0>o_
k`3oool0gmoO01/K6`3Bd]80gmoO018B4P2d];@0oooo0<K6aP092@T0d];B
0::RXP1PH600gmoO01/K6`3Bd]80N7Qh08n?S`36a/H000000;Bd]03oool0
moOg01dM7@2d];@40?ooo`0C0?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0<K6
aP000000];Bd0?ooo`3gmoL07AdM0;Bd]03oool0XZ:R0451@@3clo<0E5AD
08n?S`020?ooo`060?_kn`1DE5@0];Bd0?ooo`1hN7P0Shn?0P3oool00`36
a/H000000;Bd]01n0?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3MgMd0gMgM0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0
oooo00X0no_k05ADE02d];@0oooo0=oOg`0Q8B40kno_07QhN011@D40moOg
1@3oool00`3kno/0E5AD0;Bd]0030?ooo`0307QhN011@D40moOg00T0oooo
00<0gmoO01/K6`3Bd]80103oool01@3gmoL07AdM0000000?3`l0kno_00D0
oooo00L0moOg01dM7@0000003`l?0>o_k`2RXZ80H61P00<0oooo00`0N7Qh
08n?S`2RXZ80H61P0=oOg`0K6a/0d];B07QhN02?Shl0N7Qh0451@@3gmoL8
0?ooo`030?Ogm`0V9RH0d];B00<0oooo00<0N7Qh0451@@3gmoL01P3oool0
101hN7P0B4Q80451@@3gmoL20?ooo`030?Ogm`0V9RH0d];B08H0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYA03oool0203kno/0E5AD0;Bd]03o
ool0gmoO024Q8@3_knl0N7Qh1@0000001@2?Shl0oooo0?_kn`1DE5@0H61P
00<0oooo00<0N7Qh0000000000000`0000000`2?Shl0oooo0?ooo`030?oo
o`030=oOg`0K6a/0d];B0080oooo00H0no_k05ADE0000000000000l?3`3_
knl40?ooo`090?_kn`1DE5@000000000000?3`l0kno_0?ooo`2RXZ80H61P
0080oooo00/0no_k05ADE02?Shl0XZ:R061PH03Ogml06a/K0=;BdP1hN7P0
Shn?07QhN0050000000308n?S`3oool0oooo0080oooo00<0moOg02HV9P3B
d]800`3oool00`1hN7P00000000000030000000608n?S`3oool0oooo0::R
XP000000Shn?0`3oool00`3gmoL09RHV0=;BdP260?ooo`0305EEE@3oool0
oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3M
gMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJD@0oooo00<0no_k05ADE00000000P0000005`0?
3`l0kno_0<K6aP092@T0d];B0?ooo`36a/H02@T90=;BdP3oool0no_k05AD
E0000000H61P0<K6aP1PH600a/K600T92@3Bd]80oooo0<K6aP092@T0d];B
00D0oooo00<0gmoO01/K6`3Bd]800P3oool01`3kno/0E5AD0;Bd]03oool0
a/K60451@@3gmoL00`3oool0703kno/0E5AD0;Bd]03oool0a/K60451@@3g
moL0gmoO018B4P2d];@0oooo0<K6aP092@T0d];B0::RXP0?3`l0clo?018B
4P2d];@0N7Qh08n?S`36a/H02@T90=;BdP3oool0a/K600T92@3Bd]840?oo
o`030?Ogm`0V9RH0d];B00<0oooo00d0a/K600T92@3Bd]80oooo0<K6aP09
2@T0d];B0?ooo`36a/H02@T90<[:bP0M7Ad0];Bd0080oooo00<0moOg02HV
9P3Bd]80QP3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0oooo
0?Ogm`80kNg]0P3Zj^X07@3Ti>@0gMgM0=gMg@3Fe]H0d=3@0<o?c`3@d=00
oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35
aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080
oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFU40?ooo`09
0?_kn`1DE5@0];Bd0?ooo`3Ogml08B4Q0>o_k`3oool0a/K600<0000000/0
];Bd0?ooo`3Ogml04Q8B0000002/[:`07AdM0000002d];@0oooo0<K6aP03
000000030;Bd]03oool0oooo0080oooo00D0no_k05ADE00000002@T90=;B
dP030?ooo`060=oOg`0B4Q80000000000011@D40moOg103oool05P3Ogml0
4Q8B000000000000@D510?Ogm`3oool0gmoO018B4P0000002@T90=;BdP3O
gml04Q8B05MGE`0M7Ad0/;2`05ADE00?3`l0kno_0?ooo`36a/H300000003
0;Bd]03oool0oooo0080oooo00<0gmoO018B4P0000000P0000001011@D40
moOg0?ooo`36a/H30000000;0;Bd]03oool0XZ:R000000092@T0/[:b018B
4P000000Shn?0=oOg`0B4Q800`0000000`11@D40moOg0?ooo`220?ooo`03
05EEE@3oool0oooo0080oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>[Z
jP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?0=3@d03oool0UYJF06UYJ@2[
Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Q
h@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJD@0oooo00L0no_k05ADE02d];@0
oooo0=oOg`0Q8B40kno_04T0oooo00<0moOg02HV9P3Bd]804P3oool00`3g
moL09RHV0=;BdP260?ooo`0305EEE@3oool0oooo0080oooo00@0k>c/09JF
UP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03MgMd0gMgM0=KFeP3@d=00clo?
0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40
^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3l
o?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UYJD@0
oooo00L0N7Qh000000000000TY:B018B4P000000H61P01X0oooo00<0XZ:R
00l?3`3_knl0;03oool00`3gmoL09RHV0=;BdP0B0?ooo`030?Ogm`0V9RH0
d];B08H0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3g
moL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolb0?oo
o`80EEEE103oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S
0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUYo`3ooola0?ooo`0305EEE@3oool0EEEE00@0
oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>?Sh`3Lg=`0g=cL
0=KFeP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo<03oool0101EEED0oooo0?ooo`1EEED40?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooobl0oooo00D0EEEE0?ooo`3oool0oooo05EEE@040?ooo`04
0>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cLg03Fe]H0
dM7A0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[
Z`1YJFWo0?ooobh0oooo1P1EEED40?ooo`040>c/k02FUYH0oooo0?Ogm`80
kNg]1P2EUID06@3@d=00clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000
002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^
0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo>03oool01@3/k>`0UYJF0?ooo`3gmoL0
kNg]00H0oooo01X0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo
000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00@0k>c/09JFUP3oool0
moOg0P3]kNd609FEU@0I0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3l
o?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SX
j03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolh0?ooo`050>c/k02FUYH0oooo
0?Ogm`3]kNd01P3oool06P3Fe]H0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo
0@3oool000040000003lo?`0Zj^[06UYJOl0oooo>03oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@H0UIFE01T0dM7A0<o?c`3@d=00oooo09JFUP1YJFT0
Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03Q
hN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?oo
o`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00D0k>c/09JF
UP3oool0moOg0>g]k@060?ooo`0J0=KFeP3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh0?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]1P2EUID06@3AdM40clo?0=3@d03oool0UYJF
06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0
f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?kn
oP40oooo0@3oool000040000003lo?`0Zj^[06UYJOl0oooo>03oool01@3/
k>`0UYJF0?ooo`3gmoL0kNg]00H0oooo01X0e]KF0=7Ad@3?cll0d=3@0?oo
o`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm
0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo
00@0k>c/09JFUP3oool0moOg0P3]kNd20>[ZjP0M0>CTi03Lg=`0g=cL0=KF
eP3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003lo?`0
Zj^[06UYJOl0oooo>03oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z
01d0i>CT0=cLg03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh0?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocP0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CTi03MgMd0
gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000
003lo?`0Zj^[06UYJOl0oooo>03oool0103/k>`0UYJF0?ooo`3gmoL20>g]
k@80jNWY01d0i>CT0=gMg@3MgMd0emOG0=7Ad@3?cll0d=3@0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn
0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh0?ooo`040>c/
k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Ti>@0gMgM0=gMg@3GemL0dM7A
0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1Y
JFWo0?ooocP0oooo00@0k>c/09JFUP3oool0moOg0P3]kNd20>WYj@0M0>CT
i03MgMd0gMgM0=OGe`3AdM40clo?0=3@d03oool0UYJF06UYJ@2[Zj/0o?cl
0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0
k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool0
00040000003lo?`0Zj^[06UYJC`0ooooiP00000<0?ooo`D0EEEE103oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY?03oool00`000000oooo0?ooo`3R0?ooo`030000003oool0oooo
00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000
oooo0?ooo`3R0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gM
g@3MgMd0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3R0?ooo`030000
003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=7Ad@3?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03o
ool00`000000oooo0?ooo`3R0?ooo`030000003oool0oooo00h0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY
01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3R
0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF
0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY?03oool00`000000oooo0?ooo`3R0?ooo`030000003oool0oooo
00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80jNWY01d0hn?S0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000
oooo0?ooo`3R0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=gM
g@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3R0?ooo`030000
003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF
0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0
d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03o
ool00`000000oooo0?ooo`3R0?ooo`030000003oool0oooo00h0oooo00<0
EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z
01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[
Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`3R
0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0
103/k>`0UYJF0?ooo`3gmoL20>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF
0=3@d03?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0
YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3i
nOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUY?03oool00`000000oooo0?ooo`3R0?ooo`030000003oool0oooo
00h0oooo00<0EEEE0?ooo`3oool00P3oool0103/k>`0UYJF0?ooo`3gmoL2
0>g]k@80j^[Z01d0i>CT0=gMg@3MgMd0e]KF0=3@d03?cll0d=3@0?ooo`2F
UYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?
c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40
o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool00`000000
oooo0?ooo`3R0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3o
ool00P3oool0103/k>`0UYJF0?ooo`3gmoL20>g]k@80jNWY01d0hn?S0=cL
g03Lg=`0e]KF0=7Ad@3?cll0d=3@0?ooo`2FUYH0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY<@3oool01@2RXZ800000000000092@T0d];B00H0
oooo00<000000?ooo`3oool01@3oool01P1DE5@000000000002?Shl0oooo
07QhN080000000l0@D510?Ogm`1hN7P0000000l?3`3_knl0N7Qh00000011
@D40moOg0?ooo`1hN7P0000000l?3`3_knl00`3oool0202RXZ8000000451
@@33`l<0?clo0000000?3`l0kno_0`3oool01`3;bl/0?clo0000000?3`l0
kno_0?ooo`2RXZ800P000000402d];@0oooo07QhN0000000H61P07QhN000
0000@D5109ZJVP000000@D510?Ogm`1hN7P0000000l?3`3_knl30?ooo`0;
07QhN0000000H61P07QhN0000000@D5109ZJVP000000@D510?Ogm`2RXZ80
0P0000002`2d];@0oooo07QhN0000000@D510?Ogm`3oool0N7Qh0000000?
3`l0kno_00@0oooo01@0N7Qh0000002d];@0oooo07QhN00000003`l?0>_[
j`1DE5@0H61P0=oOg`0B4Q80@D510?Ogm`1hN7P000000;Bd]03Ogml06a/K
0=;BdV`0oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`02
0?ooo`040>c/k02FUYH0oooo0?Ogm`80kNg]0P3YjNT07@3Shn<0g=cL0=cL
g03Fe]H0d=3@0<o?c`3@d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000
o?cl0:^[Z`1YJFT`0?ooo`060?_kn`1MGEd0d];B0?ooo`2RXZ80Shn?1P3o
ool00`000000oooo0?ooo`060?ooo`0307QhN02d];@0oooo0080oooo0140
XZ:R08n?S`3oool0a/K6061PH03oool0moOg07emO@3gmoL0;2`/0>o_k`3o
ool0a/K6061PH03oool0moOg07emO@030?ooo`080?_kn`1MGEd0d];B0<_;
b`3Bd]80oooo0<K6aP2?Shl30?ooo`0I0<_;b`3Bd]80oooo0<K6aP2?Shl0
gmoO01/K6`3Bd]80no_k05eMG@3Bd]80moOg02`/;03_knl0no_k05eMG@3B
d]80no_k05ADE02d];@0a/K6061PH03oool0moOg07emO@030?ooo`0H0?Og
m`0/;2`0kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0];Bd0=oOg`0K6a/0d];B
0?_kn`1MGEd0d];B0?Ogm`0/;2`0kno_0?ooo`36a/H0H61P0?ooo`3gmoL0
OGem0`3oool04@3kno/0GEeM0=;BdP3oool0a/K6061PH03oool0moOg07em
O@3Ogml0LW9b0?_kn`1MGEd0d];B0?_kn`1MGEd0d];B0700oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`040>c/k02FUYH0
oooo0?Ogm`80kNg]0P3Zj^X07@3Ti>@0g=cL0=cLg03Fe]H0dM7A0<o?c`3@
d=00oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k
^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl
0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFT`0?oo
o`060?Ogm`0/;2`0TY:B00l?3`2f][H0H61P1P3oool00`000000oooo0?oo
o`060?ooo`0307QhN02d];@0oooo0080oooo00D0XZ:R08n?S`3oool0XZ:R
0;Bd]0030?ooo`060?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd1P3oool00`3k
no/0GEeM0=;BdP020?ooo`030::RXP000000];Bd00D0oooo00D0XZ:R0000
002d];@0XZ:R061PH0020?ooo`0=0::RXP2?Shl0moOg02`/;03_knl0no_k
05eMG@3Bd]80no_k05ADE02d];@0XZ:R0;Bd]0060?ooo`0;0?Ogm`0/;2`0
kno_0?_kn`1MGEd0d];B0?_kn`1DE5@0];Bd0::RXP1PH6000P3oool0202R
XZ80Shn?0?Ogm`0/;2`0kno_0?ooo`2RXZ80];Bd1P3oool01P3kno/0GEeM
0=;BdP3oool0XZ:R0;Bd]0@0oooo00L0bl_;03lo?`2?Shl0oooo0?_kn`1M
GEd0d];B0700oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?oo
o`020?ooo`030>c/k02FUYH0emOG00`0oooo01H0emOG09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJC00oooo00H0moOg01dM7@1PH600moOg
01dM7@1PH6060?ooo`030000003oool0oooo00H0oooo00<0N7Qh0;Bd]03o
ool00P3oool0102RXZ80Shn?0?ooo`2RXZ8300000006061PH03gmoL0;2`/
0>o_k`3oool0XZ:R0`0000002`1PH600oooo0?ooo`3oool0no_k05eMG@3B
d]80gmoO018B4P11@D40moOg00@0oooo00L0gmoO018B4P11@D40moOg0?oo
o`2RXZ80Shn?0080oooo00`0N7Qh08n?S`3gmoL0;2`/0>o_k`3kno/0GEeM
0=;BdP3kno/0E5AD0;Bd]02RXZ830000000?061PH03oool0oooo0?ooo`3g
moL0;2`/0>o_k`3kno/0GEeM0=;BdP3kno/0E5AD0;Bd]02RXZ80Shn?0080
oooo00L0N7Qh08n?S`3gmoL0;2`/0>o_k`3oool0XZ:R00<0000000T0H61P
0?ooo`3oool0oooo0?_kn`1MGEd0d];B0?ooo`2RXZ800`0000002@1PH600
oooo0<K6aP0?3`l0kno_0?ooo`3kno/0GEeM0=;BdP1`0?ooo`030000003o
ool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0oOgm09>C
T`050;Rh^0030;Fe]@2h^;P0^;Rh00@0^;Rh01H0Ti>C0>c/k01YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3o
ool000040000003lo?`0Zj^[06UYJC00oooo00H0moOg02`/;03_knl0oooo
0<K6aP1PH6060?ooo`030000003oool0oooo00H0oooo00<0N7Qh00000000
00000`000000402?Shl0oooo0<K6aP2?Shl0oooo07QhN02;Rh/0E5AD0000
002?Shl0oooo0<K6aP2?Shl0oooo07QhN02?Shl30?ooo`0807QhN0092@T0
d];B0<K6aP2?Shl0oooo0861P@3Bd]830?ooo`0I0<K6aP2?Shl0oooo0861
P@3Bd]80gmoO05=CD`3gmoL0gmoO018B4P2`/;00E5AD0000002?Shl0moOg
01dM7@1PH600no_k05eMG@3Bd]80a/K608n?S`3oool0N7Qh08n?S`020?oo
o`0I0?_kn`1DE5@0000008n?S`3gmoL07AdM061PH03kno/0GEeM0=;BdP3O
gml0De=C0?Ogm`3Ogml04Q8B0;2`/01DE5@0000008n?S`3oool0a/K608n?
S`3oool0N7Qh08n?S`030?ooo`0A0=_Kf`1MGEd0d];B0?ooo`36a/H0Shn?
0?ooo`1hN7P0Shn?0?Ogm`0/;2`0inOW0:BTY03oool0fm_K05eMG@3Bd]80
L03oool00`000000oooo0?ooo`0>0?ooo`0305EEE@3oool0oooo0080oooo
00X0inOW0>k^kP3_knl0kno_0?3`l03alO40m?Cd0?Gem@3gmoL0n?Sh0P3i
nOT06@3jn_X0no_k0?_kn`3kno/0inOW06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`00P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJC00oooo00H0moOg01dM7@11@D40emOG018B4P1PH6060?oo
o`030000003oool0oooo00H0oooo00<0N7Qh0;Bd]03oool00P3oool01`2R
XZ80Shn?0?ooo`3oool0N7Qh00000011@D400P3gmoL0200/;2`0TY:B00T9
2@3Bd]80N7Qh00000011@D40moOg0`3oool0203gmoL09RHV0=;BdP3kno/0
E5AD0000002FUYH0fm_K0`3oool01`3kno/0E5AD0000002FUYH0fm_K0?oo
o`2RXZ800P000000402?Shl0oooo0=oOg`1CDe<0VYZJ00000011@D40_[jn
00000011@D40moOg0?ooo`1hN7P000000451@@3gmoL30?ooo`0;0=oOg`1C
De<0VYZJ00000011@D40_[jn00000011@D40moOg0?ooo`2RXZ800P000000
2`2?Shl0oooo0?Ogm`0/;2`0TY:B00T92@3Bd]80N7Qh00000011@D40moOg
00<0oooo00l0a/K6000000000000];Bd0?ooo`1hN7P000000451@@3clo<0
E5AD0000002BTY804Q8B08n?S`36a/H00P0000000`2d];@0oooo0?ooo`1]
0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool0
2P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3hn?P20?Wi
n@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY<03oool01P3kno/0GEeM0=;BdP3oool0XZ:R08n?S`H0oooo
00<000000?ooo`3oool01P3oool00`1hN7P0];Bd0?ooo`020?ooo`030::R
XP2?Shl0oooo05@0oooo00<0PH610=;BdP3oool02`3oool00`21PH40d];B
0?ooo`1_0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3oool0
0P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Ogm`3h
n?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`00000
09ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0
lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`0000@0
00000?clo02[Zj/0JFUY<@3oool01@2RXZ800000000000092@T0d];B00H0
oooo00<000000?ooo`3oool01P3oool00`1hN7P0];Bd0?ooo`020?ooo`03
0::RXP2?Shl0oooo05@0oooo00<0Z:RX0=;BdP3oool02`3oool00`2XZ:P0
d];B0?ooo`1_0?ooo`030000003oool0oooo00h0oooo00<0EEEE0?ooo`3o
ool00P3oool02P3WinL0k^k^0>o_k`3_knl0l?3`0?7al@3dm?@0mOGe0?Og
m`3hn?P20?Win@0I0?[jnP3kno/0no_k0?_kn`3WinL0JFUY0:^[Z`3lo?`0
000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^
k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool10?ooo`00
00@000000?clo02[Zj/0JFUY?03oool00`000000oooo0?ooo`050?ooo`06
05ADE0000000000008n?S`3oool0N7Qh0P0000000`11@D40moOg0?ooo`0?
0?ooo`030?Ogm`0V9RH0d];B0<00oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0
oooo0>80oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>80oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>80oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0
oooo0>80oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>80oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0
kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_k
n`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a
0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0
o?cl0080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooo`030000003oool0oooo0>80oooo00<000000?ooo`3oool03P3oool0
0`1EEED0oooo0?ooo`020?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a
0?Cdm03emOD0moOg0?Shn080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1Y
JFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SH
f03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0
oooo0>80oooo00<000000?ooo`3oool03P3oool00`1EEED0oooo0?ooo`02
0?ooo`0:0>OWi`3^k^h0kno_0>o_k`3`l?00lO7a0?Cdm03emOD0moOg0?Sh
n080nOWi01T0n_[j0?_kn`3kno/0no_k0>OWi`1YJFT0Zj^[0?clo0000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40oooo00001000
0000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0oooo0>80oooo00<00000
0?ooo`3oool03P3oool00`1EEED0oooo0?ooo`020?ooo`030>c/k03^k^h0
SHf=00`0WinO01H0SHf=0>KViP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2V
YZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Wi
n@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[
06UYJC`0oooo00<000000?ooo`3oool0hP3oool00`000000oooo0?ooo`0>
0?ooo`0305EEE@3oool0oooo0080oooo00<0k>c/09JFUP3Ng]h0303Kfm/0
5P3Ng]h0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUY?03oool0
0`000000oooo0?ooo`3R0?ooo`030000003oool0oooo00h0oooo00<0EEEE
0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0<0<k>cP0F0?ooo`2FUYH0
JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3H
f=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h1
0?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl0?ooo`030000003oool0
oooo0>80oooo00<000000?ooo`3oool02`3oool405EEE@@0oooo00<0k>c/
09JFUP3oool0303>c/h05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0
WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3f
m_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?cl
o02[Zj/0JFUY?03oool00`000000oooo0?ooo`3R0?ooo`030000003oool0
oooo00h0oooo00<0EEEE0?ooo`3oool00P3oool00`3/k>`0UYJF0?ooo`0<
0<k>cP0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFTl
0?ooonH00000303oool505EEE@@0oooo00<0k>c/09JFUP3oool0303>c/h0
5P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k
0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`2
0?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0JFUYo`3ooolh
0?ooo`030>c/k02FUYH0oooo0080c/k>00<0?Shn0<k>cP3>c/h00`3>c/h0
6P0n?Sh0c/k>0<k>cP3>c/h0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ
09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0
m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003l
o?`0Zj^[06UYJOl0oooo>03oool0103/k>`0UYJF0?ooo`3?cll303hn?P<0
clo?0`0n?Sh20<o?c`0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2N
WYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?Kf
mP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40oooo000010000000o?cl
0:^[Z`1YJFWo0?ooocP0oooo00<0k>c/09JFUP3oool00P3De=@303hn?P04
0=CDe00n?Sh0?Shn03hn?P<0e=CD01H0oooo09JFUP1YJFT0Zj^[0?clo000
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00004
0000003lo?`0Zj^[06UYJOl0oooo>03oool00`3/k>`0UYJF0?ooo`030=_K
f`D0?Shn103Kfm/05P3oool0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN
0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0
nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[
Zj/0JFUYo`3ooolh0?ooo`030>c/k02FUYH0oooo00@0i>CT0`0n?Sh50>CT
i00F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocP0oooo00<0k>c/09JFUP3oool01@3]kNd00`0n?Sh0kNg]0>g]k@040>g]
k@0F0?ooo`2FUYH0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k
^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?cl
o080oOgm0@3no_h10?ooo`40oooo000010000000o?cl0:^[Z`1YJFWo0?oo
ocP0oooo00<0k>c/09JFUP3oool0303dm?@05P3oool0UYJF06UYJ@2[Zj/0
o?cl0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000?clo02[Zj/0JFUYo`3ooolh0?ooo`030>c/k02FUYH0oooo
00`0o?cl01H0oooo09JFUP1YJFT0Zj^[0?clo0000000VYZJ09jNWP2VYZH0
/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3k
no/0o?cl0P3moOd10?knoP40oooo0@3oool000040000003lo?`0Zj^[06UY
JOl0oooo>03oool00`3/k>`0UYJF0?ooo`0=0?ooo`0E09JFUP1YJFT0Zj^[
0?clo0000000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40
j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0080oOgm0@3no_h10?ooo`40
oooo000010000000o?cl0:^[Z`1YJFWo0?ooocP0oooo00<0k>c/09JFUP3G
emL0303oool05P3GemL0UYJF06UYJ@2[Zj/0o?cl0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000@000000?clo02[Zj/0
JFUYo`3ooolh0?ooo`040>c/k03Jf]X0TY:B0;Jf]PX0]KFe01L0];Bd08n?
S`3Jf]X0JFUY0:^[Z`3lo?`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG5
0<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gm
o@40o_kn0@3oool10?ooo`0000<000000?Ogm`2JVYX0o`1jNWY;07YjNP0C
09ZJVP3gmoL0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0
hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3o
ool10?ooo`0000<000000?Ogm`2o_kl0o`2o_km<0;no_`0B0?Ogm`000000
VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^kP3c
lo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool000030000
003gmoL0hn?S0?l0hn?SC03Shn<04P3gmoL0000009ZJVP2NWYh0YZJV0;6a
/@2k^k/0aLG50<o?c`3Hf=P0hN7Q0>SXj03^k^h0lo?c0?KfmP3inOT0no_k
0?clo080oOgm0@3no_h10?ooo`40oooo00000`000000lO7a0?ooo`3o0?oo
o`D0oooo00<0hn?S0:FUY@1fMWH00`1fMWH0101gMgL0MWIf07MgM`1gMgL9
07IfMP060:FUY@3Shn<0oooo0?ooo`3Shn<0YJFU101fMWH0101gMgL0MWIf
07MgM`1gMgL907IfMP060:FUY@3Shn<0oooo0?ooo`3[jn/0YJFU101FEUH0
101GEeL0EUIF05MGE`1GEeL905IFEP030:FUY@3[jn/0oooo00<0oooo0180
lO7a0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000@000000>7Qh@3lo?`0no_k0P3lo?`00`3kno/0o?cl0?clo0040?cl
o0030?_kn`3lo?`0o?cl0080o?cl00<0no_k0?clo03kno/01@3lo?`00`3k
no/0o?cl0?_kn`1J0?_kni00o?cl00<0YJFU07IfMP2=SHd0402=SHd01P1f
MWH0YJFU0?clo03lo?`0YJFU07IfMQ40SHf=00H0MWIf0:FUY@3lo?`0no_k
0:FUY@1FEUHA05]KF`0305IFEP2UYJD0o?cl00<0o?cl0180hN7Q0000002J
VYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??c
l`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<00000
0>3Ph03gmoL0o`3gmoL50?Ogm`0307IfMP2=SHd0XZ:R0100XZ:R00H0SHf=
07IfMP3gmoL0moOg07IfMP2=SHdA0::RXP0808f=S@1fMWH0moOg0?Ogm`1F
EUH0Fe]K06m_K`1aLG4=079bLP04075aL@1_Kfl0Fe]K05IFEP@0moOg0180
h>3P0000002JVYX0WYjN0:JVYP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3X
j>P0k^k^0??cl`3fm_H0nOWi0?_kn`3lo?`20?gmo@40o_kn0@3oool10?oo
o`0000<000000=kNgP3alO400P3alO420?;blP030?7al@3bl_80l_;b00@0
lO7a00<0l_;b0?7al@3alO400`3alO420?;blP030?7al@3bl_80lO7a0080
l_;bk03alO400`1fMWH0^;Rh0:^[Z`0@0:^[Z`060;Rh^01fMWH0lO7a0?7a
l@1fMWH0^;Rh4@2[Zj/01@2h^;P0MWIf0?7al@3bl_80EUIF0080Kfm_00<0
LW9b07MgM`1gMgL02`1gMgL0101bLW80Kfm_06m_K`1FEUH30?7al@0C0?;b
lP3Ng]h0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo0020?gmo@40o_kn0@3oool1
0?ooo`0000D000000=cLg03[jn/0jn_[0>c/k0020>_[j`030:NWY`2RXZ80
XZ:R0080XJ6Q00<0X:2P09nOW`2MWId00P2OWil20:2PX080XJ6Q00<0YjNW
0>_[j`3[jn/01@3[jn/01@3De=@0aLG50<G5a@35aLD0emOG0080jn_[00<0
j^[Z0<S8b03@d=000P3[jn/01@3Jf]X0a/K60>?Sh`3[jn/0h>3P0080aLG5
00L0c<c<0>WYj@3>c/h0alO70>WYj@3[jn/0h>3P0080aLG500L0c<c<0>[Z
jP3De=@0aLG50=gMg@3[jn/0h>3P0080aLG500H0a/K60>?Sh`3[jn/0jn_[
0>[ZjP38b<P30<G5a@030<K6aP3Shn<0jn_[00@0jn_[00<0j^[Z0<S8b035
aLD00`35aLD00`36a/H0hn?S0>_[j`0L0>_[je/0k>c/=@3[jn/0101fMWH0
^k^k0;6a/@2a/K480?ooo`L0/K6a00H0^k^k07IfMP3[jn/0jn_[07IfMP2k
^k/20;6a/@`0oooo0`2a/K402@2k^k/0MWIf0>_[j`3[jn/0EUIF07MgM`1b
LW80M7Ad07MgM`0;07QhN00507MgM`1dM7@0LW9b07MgM`1FEUH0103[jn/0
4P3Lg=`0000009ZJVP2NWYh0YZJV0;6a/@2k^k/0aLG50<o?c`3Hf=P0hN7Q
0>SXj03^k^h0lo?c0?KfmP3inOT0no_k0?clo080oOgm0@3no_h10?ooo`40
oooo00000`000000f][J0>GUi@030>GUi@0308Z:RP2KVi/0[:b/00<0jNWY
00X0j^[Z0<C4a025QHD0k>c/0>g]k@3^k^h0inOW0?3`l03bl_80Xj>S1P3U
iND04@1XJ6P0410@010@400@4100P8200<K6aP3?cll0g]kN02HV9P1ADE40
clo?0>GUi@2DU9@05ADE0:BTY03Ng]h0/[:b0080410@00L0@D510=_Kf`1@
D5006ATI0;Zj^P3Shn<0/[:b0080410@00L0@D510=oOg`1`L700410@09:B
TP3Ti>@0[jn_0080410@00H05ADE0;Zj^P3<c<`0i>CT0=gMg@0T92@3010@
400301DE5@2TY:@0g]kN00@0iNGU00<0g]kN02HV9P0@41000`0@41000`0E
5AD0_[jn0>GUi@2]0>GUi@0507IfMP30`<00]KFe0;Fe]@3oool01P1bLW80
0`0W9bL0]KFe0;Fe]@050;Fe]@060<30`01fMWH0iNGU0>GUi@1fMWH0`<30
0P2e]KD00`3oool09bLW02LW9`0902LW9`<0]KFe00D0`<3007IfMP3UiND0
iNGU05IFEP0207MgM`0407UiN@1jNWX0oooo0?ooo`L0NWYj0P3oool01@1j
NWX0NGUi07MgM`1gMgL0EUIF00@0iNGU0180f][J0000002JVYX0WYjN0:JV
YP2a/K40^k^k0<G5a@3?cll0f=SH0>7Qh@3Xj>P0k^k^0??cl`3fm_H0nOWi
0?_kn`3lo?`20?gmo@40o_kn0@3oool10?ooo`0000<000000=WIf@3Ogml0
0`3Ogml01@3Ng]h0X:2P0>[ZjP3YjNT0iNGU0080j^[Z00X0d=3@07AdM03]
kNd0k^k^0:f][@3@d=00l_;b0??cl`2SXj<0gmoO0P3Ng]h0=03Ogml0g]kN
06a/K00E5AD0Zj^[0=kNgP33`l<071`L05ADE036a/H0e]KF02DU9@1ADE40
cLg=0=kNgP2?Shl05ADE0:BTY03EeMD0CTi>03Ph>034a<@0gMgM0=_Kf`1>
CTh06ATI0;Zj^P3IfMT0CTi>03Ph>034a<@0gMgM0=OGe`0U9BD0>3Ph0<C4
a032`/806a/K055AD@3IfMT0`l?304a<C03Hf=P0emOG02DU9@0I6AT0aLG5
0=kNgP2?Shl05ADE0:BTY03IfMT50=kNgP050:f][@0@41006ATI0;Vi^@3A
dM40/03Ng]h01@1fMWH0`l?30;^k^`2k^k/0oooo00H0LW9b00<09bLW0;^k
^`2k^k/01@2k^k/01P33`l<0MWIf0=kNgP3Ng]h0MWIf0<?3``80^k^k00<0
oooo02LW9`2k^k/01`2k^k/00`3oool09bLW0;^k^`020;^k^`090<?3``1f
MWH0gmoO0=kNgP1FEUH0MgMg07QhN01kNg/0<30`00<0oooo1@1mOGd30?oo
o`05030`<01kNg/0N7Qh07MgM`1FEUH00P3Ng]h0503Ogml0g]kN0=WIf@00
0000VYZJ09jNWP2VYZH0/K6a0;^k^`35aLD0clo?0=SHf03QhN40j>SX0>k^
kP3clo<0m_Kf0?Win@3kno/0o?cl0P3moOd10?knoP40oooo0@3oool00004
0000003GemL0f=SH0=WIf@80f=SH0100T92@08n?S`2>SXh0jNWY0861P@1A
DE40QhN70=[JfP2VYZH0ZJVY0:b/[01OGel0l_;b0??cl`3dm?@0Y:BT0`3H
f=P01@3IfMT0d];B02@T901ADE40bl_;0080f=SH0200JVYZ01TI6@2j^[X0
d=3@02@T901ADE40bl_;0=SHf02;Rh/05ADE0:BTY03=cLd092@T03Ph>033
`l<0emOG0=GEe@1<C4`06ATI0;Zj^P3@d=0092@T03Ph>033`l<0emOG0=7A
d@0T92@0>3Ph0<?3``2:RXX05ADE0:BTY0@0aLG500X0WYjN010@401aLG40
dM7A0=SHf02;Rh/05ADE0:BTY03De=@0d];B0P35aLD02036a/H0emOG0=SH
f01ZJVX0410@079bLP35aLD0e=CD7`3Hf=QK0=WIfCD0f=SH00D0MWIf0<O7
a`2o_kl0_kno0?ooo`0702LW9`L0_kno00H0alO707IfMP3Hf=P0f=SH07If
MP37alL20;no_`030?ooo`0W9bL0_kno00L0_kno00<0oooo02LW9`2o_kl0
0P2o_kl01@37alL0MWIf0=SHf03Hf=P0EUIF0080Ng]k00<0OWin07moO`0X
:2P00`3oool307moO`<0oooo00<0:2PX07moO`1nOWh00P1kNg/00`1FEUH0
fMWI0=SHf0020=SHf00B0=OGe`000000VYZJ0:2PX02XZ:P0/[:b0;bl_036
a/H0d=3@0=WIf@3QhN40j>SX0>k^kP3clo<0m_Kf0?Win@3kno/0o?cl0P3m
oOd10?knoP40oooo0@3oool000030000003EeMD0dm?C00<0dm?C00<0QHF5
0:>SX`3Vi^H00P3Zj^X02`0e=CD0Ogmo0<o?c`2n_[h0Rh^;05iNGP1HF5P0
J6QX0:>SX`39bLT0ZZZZ00@0dm?C00@0c<c<02@T901ADE40b/[:0P3Cdm<0
7P1WIfL06ATI0;Vi^@3;bl/092@T055AD@3:b/X0dm?C08R8R00E5AD0Y:BT
0<W9b@0T92@0>3Ph0<71`@3Cdm<0clo?04Y:BP0I6AT0^KVi0<_;b`0T92@0
>3Ph0<71`@3Cdm<0c<c<02@T900h>3P0`L7106MWI`D0410@00X0DE5A09fM
W@0@4100SXj>0<c<c03Cdm<0R8R801DE5@2TY:@0XJ6Q0P0@41002@0J6QX0
a/K60=?Cd`3Cdm<0c<c<02@T900E5AD0Y:BT0<O7a`2_0=?Cd`0307IfMP3;
bl/0a/K60100a/K600H0bl_;07IfMP3Cdm<0dm?C07IfMP3;bl/20<K6aP03
0?ooo`0W9bL0a/K600L0a/K600<0oooo02LW9`36a/H00P36a/H01`3;bl/0
MWIf0=?Cd`3Cdm<0EUIF07emO@1oOgl00`23Ph<01@0X:2P0oooo0?ooo`3o
ool0Ph>300<0oooo00L0:2PX08>3P`23Ph<0Ph>307moO`1mOGd0EUIF00@0
dm?C0140eMGE0000002PX:00YJFU0:f][@2f][H0_kno0<S8b03Bd]80f][J
0>;RhP3YjNT0kno_0?Cdm03fm_H0nOWi0?_kn`030?gmo@40o_kn0@3oool1
0?ooo`0000D000000=?Cd`3>c/h0c/k>0<o?c`020<k>cP0?08N7Q`2CTi<0
dm?C0=_Kf`2][Jd0`L710;2`/02YZJT0ZZZZ08f=S@2ATI40/;2`0<_;b`3_
knl0YJFU00<0c/k>00D0clo?0<O7a`0S8b<0DE5A0<S8b0020<k>cP0]06EU
I@0I6AT0^KVi0<O7a`0S8b<0DE5A0<O7a`3>c/h0QHF501DE5@2TY:@0alO7
02<S8`0h>3P0`L710<k>cP3;bl/0BDU901TI6@2i^KT0alO702<S8`0h>3P0
`L710<k>cP37alL08b<S03Ph>031`L40QHF501DE5@2TY:@0b/[:0<O7a`0S
8b<0DE5A0;6a/@0J6QX0LW9b0<G5a@3>c/h0QHF501DE5@2TY:@0cLg=00L0
c/k>00D0X:2P010@400i>CT0_kno0<_;b`050<k>cP050<g=c@37alL0cLg=
0<O7a`3:b/X0503>c/iK0<o?ccD0c/k>00<0MWIf0<k>cP3;bl/0403;bl/0
1P3>c/h0MWIf0<k>cP3>c/h0MWIf0<k>cP80bl_;00<0oooo02LW9`3;bl/0
1`3;bl/00`3oool09bLW0<_;b`020<_;b`090<k>cP1fMWH0clo?0<k>cP1F
EUH0OGem08:2PP25QHD0Q8B40080QHF500<0:2PX0?ooo`3oool00`3oool0
200X:2P0QHF508F5Q@24Q8@0QHF508:2PP1mOGd0EUIF103>c/h04@3Cdm<0
00000:RXZ02][Jd0/k>c0;^k^`33`l<0c<c<0=GEe@3MgMd0i>CT0>_[j`3`
l?00m?Cd0?Ogm`3inOT0o?cl00<0oOgm0@3no_h10?ooo`40oooo00000`00
0000d=3@0<W9b@030<W9b@0A07=cL`2DU9@0OWin07UiN@2PX:00dM7A0:f]
[@2UYJD0YZJV08F5Q@1NGUh0[Jf]0>GUi@3fm_H0n?Sh0:BTY039bLT00`38
b<P01032`/808b<S055AD@37alL20<S8b00/069RHP0I6AT0^KVi0<;2`P0R
8R80>3Ph0<71`@32`/80ATI601TI6@2l_;`0`L7102<S8`0h>3P0_kno0<G5
a@34a<@0ATI601TI6@2i^KT0`L7102<S8`0h>3P0_kno0<G5a@30`<008R8R
03Ph>030`<00[jn_01XJ6P1cLg<0aLG509ZJVP0@4100SXj>0<G5a@16ATH0
6ATI0;Vi^@35aLD0Ogmo01DE5@2TY:@90<S8b004069RHP0@4100SHf=0<O7
a`@0b<S800H0/k>c04E5A@2QXJ40C4a<06i^KP37alLD0<S8b5/0bLW9=@38
b<P00`1fMWH0dm?C0<g=c@0@0<g=c@060=?Cd`1fMWH0b<S80<S8b01fMWH0
dm?C0P3=cLd00`3oool09bLW0<g=c@070<g=c@030?ooo`0W9bL0cLg=0080
cLg=00L0dm?C07IfMP38b<P0b<S805IFEP22PX80Ph>300D0QhN700D0:2PX
0?ooo`3oool0oooo02PX:00508N7Q`0308>3P`22PX80EUIF00<0bLW90180
b<S80=3@d0000000/[:b0;Jf]P2k^k/0`/;20<W9b@3AdM40fMWI0>3Ph03W
inL0kNg]0?7al@3emOD0n?Sh0?[jnP3lo?`30?gmo@40o_kn0@3oool10?oo
o`0000<000000<o?c`35aLD01035aLD03`2GUiL0]kNg0>c/k02i^KT0Rh^;
0;Ng]`2ZZZX0Y:BT07inOP1aLG40a<C40=WIf@3fm_H0nOWi0:FUY@040<G5
a@040;no_`0R8R80DE5A0<K6aP80aLG500H0HF5Q01TI6@2i^KT0_kno028R
8P1MGEd2010@400401DE5@2TY:@0`/;204E5A@<0410@00H0DE5A065QH@0@
41006ATI0;Jf]P15ADD3010@4008055AD@2o_kl08R8R03Ph>02o_kl0aLG5
0:b/[00J6QX2010@4005079bLP35aLD0aLG50<;2`P15ADD00`0@41000`0E
5AD0Y:BT0<K6aP080<G5a@030:b/[00J6QX0DE5A00D0aLG500D0_kno028R
8P0`<3006QXJ08j>SP2U0<G5a@0307IfMP3EeMD0dM7A0100dM7A00H0eMGE
07IfMP35aLD0aLG507IfMP3EeMD20=7Ad@030?ooo`0W9bL0dM7A00L0dM7A
00<0oooo02LW9`3AdM400P3AdM401`3EeMD0MWIf0<G5a@35aLD0EUIF08:2
PP27QhL01@29RHT50?ooo`D0RHV900<0QhN708:2PP1FEUH01035aLD04@3?
cll000000;bl_02o_kl0`l?30<W9b@3@d=00emOG0=gMg@3Ti>@0j^[Z0>o_
k`3clo<0m_Kf0?Shn03kno/0o?cl0080oOgm0P3no_h20?ooo`0000<00000
0<g=c@31`L400`31`L40401eMGD0VIVI0:NWY`3]kNd0bLW907=cL`2`/;00
SHf=07=cL`1<C4`0HV9R0;jn_P2e]KD0/[:b0:f][@2YZJT40<71`@040;^k
^`0R8R80DE5A0<C4a080`L7100@0GemO01TI6@2i^KT0`/;21`31`L401@2k
^k/08R8R03Ph>02n_[h0`/;20080`L7100`0`l?30<G5a@2l_;`08R8R03Ph
>02n_[h0`/;20;^k^`0R8R80>3Ph0;jn_P32`/8;0<71`@0407alO00E5AD0
Y:BT0<?3``H0`L710P34a<@0102n_[h0A4A403Ph>02n_[h40<71`@0705mO
G`0@4100410@010@400h>3P0_[jn0<;2`P2S0<71`@0307IfMP3Hf=P0eMGE
0100eMGE00H0f=SH07IfMP31`L40`L7107IfMP3Hf=P20=GEe@030?ooo`0W
9bL0oooo00P0oooo00H09bLW0=GEe@3EeMD0eMGE0=SHf01fMWH20<71`@03
05IFEP24Q8@0R8R800@0RXZ:0`3oool0100X:2P0oooo0?ooo`3oool408Z:
RP0308R8R024Q8@0EUIF00@0`L710100cLg=00000036a/H0b<S80<c<c03A
dM40emOG0=cLg03Rh^80j>SX0>g]k@3alO40mOGe0?Ogm`3inOT0o?cl0`3m
oOd20?knoP80oooo00000`000000c<c<0;Zj^P020;fm_@0A0;jn_P2m_Kd0
WYjN0>?Sh`2f][H0UYJF08:2PP2CTi<0Q8B408:2PP1>CTh0;2`/09bLW02[
Zj/0m_Kf0?Shn02WYjL0102m_Kd0102g]kL08B4Q055AD@34a<@20;fm_@04
05eMG@0I6AT0^KVi0;no_`P0_Kfm00<0GEeM03Lg=`2h^;P00P2m_Kd02`1M
GEd06ATI0;Vi^@2o_kl0GEeM03Lg=`2h^;P0_Kfm0;Ng]`0Q8B40>3Ph0080
_[jn2`2m_Kd0101jNWX05ADE0:BTY02o_kl40;fm_@070;Ng]`0Q8B40Kfm_
0<G5a@2o_kl08b<S071`L0050;fm_@0607YjNP0d=3@0?3`l05eMG@0h>3P0
^;Rh502m_KeK0;jn_SD0_Kfm00<0MWIf0=[JfP3EeMD0403EeMD01P3Jf]X0
MWIf0;fm_@2m_Kd0MWIf0=[JfP80eMGE00<0oooo079bLP1bLW80201bLW80
3P0W9bL0eMGE0=GEe@3EeMD0f][J07IfMP2m_Kd0_[jn05IFEP27QhL0RXZ:
08f=S@2>SXh0SHf=0`3oool00`0X:2P0SHf=02PX:0030?ooo`<0SHf=00@0
RXZ:08N7Q`1FEUH0_[jn0P2m_Kd04@2j^[X0c<c<0000003@d=00d];B0=GE
e@3IfMT0gMgM0>;RhP3WinL0k>c/0?3`l03dm?@0m_Kf0?Shn03jn_X0o?cl
00<0oOgm00<0o_kn0?ooo`3oool00@3oool000030000003:b/X0^;Rh00<0
^[Zj0100O7al08f=S@1hN7P0VIVI0?3`l032`/80YjNW0<_;b`2l_;`0Xj>S
06e]K@1mOGd0[:b/0?_kn`3lo?`0YZJV0`2j^[X0102k^k/0];Bd024Q8@1=
CDd30;Zj^P0305]KF`0I6AT0[jn_00h0^[Zj00<0Fe]K01TI6@2_[jl01@2j
^[X0102d];@08B4Q03Lg=`2d];@<0;Zj^P0307MgM`0D51@0Vi^K00H0^[Zj
00D0TI6A010@400@4100410@04e=C@070;Zj^P030;Bd]00Q8B40Jf][0:H0
^[Zj00<0MWIf0=SHf03Jf]X0403Jf]X01P3Hf=P0MWIf0;Zj^P2j^[X0MWIf
0=SHf080f][J00<0oooo079bLP1bLW80201bLW803@0W9bL0f][J0=[JfP3J
f]X0f=SH07IfMP2j^[X0^k^k05IFEP2;Rh/0Shn?08j>SP2?Shl00`3oool0
1@0X:2P0Shn?08n?S`2?Shl0:2PX00<0oooo00D0Shn?08j>SP2?Shl0Rh^;
05IFEP030;Zj^P0@0;Rh^03:b/X000000=WIf@3Jf]X0gMgM0>3Ph03Ti>@0
j>SX0>c/k03_knl0lo?c0?KfmP3hn?P0n_[j0?clo0<0oOgm0P3no_h30?oo
o`0000<000000<W9b@2h^;P00`2h^;P04026QXH0WYjN0=CDe03`l?00l_;b
0<k>cP31`L40gMgM0=;BdP2RXZ80c<c<0<W9b@29RHT0o?cl0?gmo@2WYjOa
0;Rh^00307IfMP3GemL0gMgM0100gMgM00H0emOG07IfMP2h^;P0^;Rh07If
MP3GemL30=gMg@/09bLW0`3MgMd02@3GemL0MWIf0;Rh^02h^;P0EUIF092@
T02ATI40Ti>C09BDU0020?ooo`0302PX:02HV9P0V9RH00<0V9RH00P0:2PX
0?ooo`3oool0U9BD09>CT`2ATI40T92@05IFEP@0^;Rh00l0bLW90000003Q
hN40h^;R0>CTi03WinL0j^[Z0>g]k@3`l?00lo?c0?Gem@3gmoL0nOWi0?_k
n`3lo?`00`3moOd00`3no_h0oooo0?ooo`020?ooo`0000<000000<W9b@2h
^;P0102h^;P03`26QXH0U9BD0?;blP3clo<0X:2P0>SXj03fm_H0fm_K09VI
V@3jn_X0no_k0=cLg02f][H0o_kn0:NWY`3a0;Rh^00307IfMP3Fe]H0hN7Q
0100hN7Q00H0e]KF07IfMP2h^;P0^;Rh07IfMP3Fe]HA0>7Qh@090=KFeP1f
MWH0^;Rh0;Rh^01FEUH0UIFE09NGU`2LW9`0XZ:R0080:2PX00D0Zj^[0:f]
[@2[Zj/0Zj^[0:f][@020:^[Z`80:2PX00D0XZ:R09bLW02GUiL0UIFE05IF
EP040;Rh^00>0<W9b@000000j>SX0>WYj@3[jn/0kNg]0>o_k`3alO40m?Cd
0?KfmP3gmoL0nOWi0?_kn`3lo?`30?gmo@80o_kn103oool0000300000039
bLT0]KFe00<0]KFe00d0MgMg0:2PX03Ph>00lo?c0>GUi@3/k>`0m_Kf0?Sh
n03inOT0T92@0?_kn`3lo?`0oOgm0080o_kn00<0YjNW0;Fe]@2e]KD0k`2e
]KD00`1fMWH0kNg]0>WYj@0@0>WYj@060>g]k@1fMWH0]KFe0;Fe]@1fMWH0
kNg]4@3YjNT02P3]kNd0MWIf0;Fe]@2e]KD0EUIF09^KV`2NWYh0YJFU0;:b
/P2j^[X20;no_`040<30`02o_kl0_kno0<30`080_kno00L0`<300;Zj^P2b
/[80YJFU09jNWP2KVi/0EUIF00@0]KFe00d0bLW90000003^k^h0kno_0?3`
l03alO40lo?c0?Gem@3fm_H0n?Sh0?Win@3kno/0o?cl00<0oOgm0P3no_h5
0?ooo`0000<000000<W9b@2j^[X0102a/K400`2=SHd0ZZZZ0:ZZZP050:ZZ
ZP030:NWY`2ZZZX0ZZZZ00@0ZZZZl@2a/K400`1fMWH0e=CD0?;blP0@0?;b
lP060=CDe01fMWH0/K6a0;6a/@1fMWH0e=CD4@3bl_801P3De=@0MWIf0;6a
/@2a/K40EUIF0:>SX`80Z:RX3@33`l<20:RXZ0030:>SX`1FEUH0/K6a0080
/K6a00@0^[Zj0<W9b@000000lo?c0P3dm?@01`3emOD0m_Kf0?Ogm`3hn?P0
n_[j0?_kn`3lo?`0103moOd00`3no_h0oooo0?ooo`040?ooo`0000<00000
0<?3``39bLT0o`2^[Zh50:j^[P0408f=S@1fMWH0c/k>0?_kn`d0oooo00@0
no_k0<k>cP1fMWH0SHf=0P2^[Zh0102=SHd0MWIf0<k>cP3kno/=0?ooo`04
0?_kn`3>c/h0MWIf08f=S@80[Zj^00H0VIVI05IFEP2e]KD0bl_;0=oOg`3X
j>P90>OWi`060>SXj03Ogml0bl_;0;Fe]@1FEUH0VIVI0`2^[Zh00`39bLT0
`l?3000000020?KfmP050?Ogm`3hn?P0n?Sh0?Win@3jn_X00P3lo?`40?gm
o@80o_kn1P3oool0000400000033`l<0e=CD0;Ng]ol0YjNW1@2WYjL00`2=
SHd0MWIf07IfMP0?07IfMP0308f=S@2WYjL0YjNW0080YjNW00<0SHf=07If
MP1fMWH03`1fMWH00`2=SHd0YjNW0:NWY`020:NWY`0308f=S@1FEUH0EUIF
00l0EUIF00P0SHf=0:NWY`2WYjL0YjNW0;Ng]`3De=@0`l?3000000<0nOWi
00@0n_[j0?_kn`3lo?`0o?cl1@3moOd20?knoPL0oooo00001P000000ZZZZ
0>OWi`3De=@0_[jn0;>c/ol0ZZZZA@2ZZZX01P2c/k<0_[jn0=CDe03WinL0
ZZZZ00000080no_k0`3lo?`50?gmo@<0o_kn203oool000070000002:RXX0
fMWI0?clo03Ti>@0d];B0<c<c03o0;jn_T<0_[jn00P0c<c<0=;BdP3Ti>@0
o?cl0=WIf@2:RXX000000?clo0P0oOgm0P3no_h:0?ooo`0000P0000006UY
J@2GUiL0fMWI0?clo03bl_80k>c/0>?Shol0fMWI@@3IfMT0203Shn<0k>c/
0?;blP3lo?`0fMWI09NGU`1YJFT000001`3moOd30?knoP/0oooo00001@00
0000oooo06UYJ@2GUiL0dm?C0080o?clo`3ooom30?ooo`80o?cl00D0dm?C
09NGU`1YJFT0oooo000000040?gmo@@0o_kn3@3oool000030000003oool0
oooo0080JFUY00<0RXZ:09NGU`2ZZZX0o`2ZZZY20:ZZZP0409NGU`2:RXX0
JFUY06UYJ@80oooo00<000000?knoP3no_h0103no_h?0?ooo`00o`00001A
000001D0oooo0000\
\>"], "Picture",
  Evaluatable->False,
  ImageSize->{357, 227},
  ImageMargins->{{0, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}}]
}, Open  ]],

Cell["\<\
Let us put a few of these pieces together and create a function that will \
evaluate the next input cell. In Section 10.5 we will turn this code into a \
button. \
\>", "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[TextData[{
  "For this example we will operate in the current notebook. We can refer to \
the notebook in which these commands are being evaluated by ",
  StyleBox["EvaluationNotebook[]", "MR"],
  ". First we select the current unit; that is, the cell in which the \
following code lives. "
}], "Text",
  CellTags->{"i:15", "b:43.1", "IPM3.10.2.2"}],

Cell["SelectionMove[EvaluationNotebook[],All,Cell]", "Program",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell["\<\
Then we move the selection insertion to the next cell (at the moment, this \
code will only work if it is immediately followed by an input cell).\
\>", "TextFirst",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell["SelectionMove[EvaluationNotebook[],Next,Cell]", "Program",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell["Finally, we evaluate the currently selected input.", "TextFirst",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell["SelectionEvaluate[EvaluationNotebook[]]", "Program",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[TextData[{
  "Here we bundle this code up into the function ",
  StyleBox["EvaluateNext", "MR"],
  "."
}], "TextFirst",
  PageBreakAbove->True],

Cell[BoxData[
    \(EvaluateNext := \((\[IndentingNewLine]SelectionMove[
          EvaluationNotebook[], All, Cell]; \[IndentingNewLine]SelectionMove[
          EvaluationNotebook[], Next, 
          Cell]; \[IndentingNewLine]SelectionEvaluate[
          EvaluationNotebook[]];\[IndentingNewLine])\)\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[TextData[{
  "Evaluating the cell containing ",
  StyleBox["EvaluateNext", "MR"],
  " causes the immediately following cell to be evaluated."
}], "TextFirst",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[BoxData[
    \(EvaluateNext\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(2 + 2\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[BoxData[
    \(4\)], "Output",
  CellLabel->"Out[18]=",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:43.1", "IPM3.10.2.3", "c:6"}],

Cell[TextData[{
  "Using ",
  StyleBox["NotebookPut", "MR"],
  ", create a notebook with one Title cell, one Section cell, one Text cell \
and two Input cells."
}], "ExerciseItem",
  CellTags->{"b:43.1", "IPM3.10.2.3"}],

Cell[TextData[{
  "Use ",
  StyleBox["NotebookGet", "MR"],
  " to read the notebook you created in Exercise 1 into the kernel. Then \
programmatically change the Section cells to Subsection cells either using ",
  StyleBox["Cases", "MR"],
  " or an appropriate rule."
}], "ExerciseItem",
  CellTags->{"b:43.1", "IPM3.10.2.3"}],

Cell[TextData[{
  "Take either of the notebooks you created in the above exercises and use ",
  StyleBox["SelectionMove", "MR"],
  " and ",
  StyleBox["SelectionEvaluate", "MR"],
  " to evaluate all of the Input cells in the notebook."
}], "ExerciseItem",
  CellTags->{"b:43.1", "IPM3.10.2.3"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Cell data types", "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 2}},
  CellTags->{
  "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3", "IPM3.10.3.4", 
    "c:7"}],

Cell[TextData[{
  "The cells in your notebooks often contain different kinds of data. \
Sometimes they will only contain text. Other times they may contain formatted \
mathematical expressions, or possibly a graphical object. Since the ",
  StyleBox["Cell", "MR"],
  " data object has to handle each one of these kinds of data, there is a \
mechanism that enables the front end to deal with these objects in a \
consistent manner \[Dash] cell data types. We will look at a few of the most \
important and useful cell data types in the next few sections."
}], "TextFirst",
  CellTags->"IPM3.10.3.0"],

Cell[CellGroupData[{

Cell["TextData", "Subsection",
  CellTags->{"i:16", "b:44.0", "IPM3.10.3.1", "c:8"}],

Cell["\<\
Let us first look at a text cell that contains no special formatting. \
\>", "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell["Cell[\"Here is some text.\", \"Text\"]", "Program",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell["\tHere is some text.", "Text",
  CellMargins->{{Inherited, Inherited}, {14, 14}},
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell[TextData[{
  "Adding some formatting to this cell causes a ",
  StyleBox["TextData", "MR"],
  " wrapper to be added."
}], "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell["\<\
Cell[TextData[{
  \"Here is some \", StyleBox[\"italicized\", FontSlant->\"Italic\"],
  \" text.\"
}], \"Text\"]\
\>", "Program",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell[TextData[{
  "\tHere is some ",
  StyleBox["italicized",
    FontSlant->"Italic"],
  " text."
}], "Text",
  CellMargins->{{Inherited, Inherited}, {14, 14}},
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell[TextData[{
  "Cells with ",
  StyleBox["TextData", "MR"],
  " can contain a number of other data objects embedded in the cell. For \
example, here is a text cell that contains a ",
  StyleBox["ValueBox", "MR"],
  ". ",
  StyleBox["ValueBox", "MR"],
  "es provide a means of embedding evaluations inside of your text cells."
}], "TextFirst",
  CellTags->{"i:17", "b:44.0", "IPM3.10.3.1"}],

Cell["\<\
Cell[TextData[{
  \"The current version is: \", ValueBox[\"$Version\"]
}], \"Text\"]\
\>", "Program",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell[TextData[{
  "\tThe current version is: ",
  
  ValueBox["$Version"]
}], "Text",
  CellMargins->{{Inherited, Inherited}, {14, 14}},
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell[TextData[{
  "A listing of all of the possible ",
  StyleBox["ValueBox", "MR"],
  " names that can be used can be found choosing ",
  StyleBox["Create Value Display Object", "SR"],
  " from the ",
  StyleBox["Input", "SR"],
  " menu. Looking under the list of global variables that can be used as the \
argument to ",
  StyleBox["ValueBox", "MR"],
  ", you will see ",
  StyleBox["Date", "MR"],
  ", for example."
}], "Text",
  CellTags->{"i:18", "b:44.0", "IPM3.10.3.1"}],

Cell["\<\
Cell[TextData[{
  \"The current date is: \",
  ValueBox[\"DateLong\"]
}], \"Text\"]\
\>", "Program",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],

Cell[TextData[{
  "\tThe current date is: ",
  
  ValueBox["DateLong"]
}], "Text",
  CellMargins->{{Inherited, Inherited}, {Inherited, 14}},
  CellTags->{"b:44.0", "IPM3.10.3.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["BoxData", "Subsection",
  CellTags->{"b:44.1", "IPM3.10.3.2", "c:9"}],

Cell[TextData[{
  "Many of your cells in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " will contain formatted mathematical expressions. Whenever you work with \
these two-dimensional typeset objects, a different editor is invoked, called \
the math editor. This is indicated in the front end by a pink background in \
",
  StyleBox["Text", "MR"],
  " cell style on the typeset expression (you can enter a math typeset \
expression by pressing Control-9). This is also indicated in the underlying \
cell structure by means of the ",
  StyleBox["BoxData", "MR"],
  " wrapper. For example, consider the following cell containing a \
superscript expression."
}], "TextFirst",
  CellTags->{"i:19", "b:44.1", "IPM3.10.3.2"}],

Cell["\<\
Cell[BoxData[
    RowBox[{
      SuperscriptBox[\"x\", \"2\"], \"+\", \"y\"}]], \"Input\"]\
\>", "Program",
  CellTags->{"i:20", "b:44.1", "IPM3.10.3.2"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[BoxData[
    \(x\^2 + y\)], "Input",
  Evaluatable->False,
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[TextData[{
  "There are several things to note here. First, we see that ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " has automatically placed the elements ",
  Cell[BoxData[
      \(TraditionalForm\`x\^2\)]],
  ", + and y all in something called a ",
  StyleBox["RowBox", "MR"],
  ". This is how ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " represents box objects or a series of strings."
}], "Text",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[TextData[{
  "Secondly, the ",
  Cell[BoxData[
      \(TraditionalForm\`x\^2\)]],
  " object is represented internally as another box object, specifically ",
  StyleBox["SuperscriptBox[x,2]", "MR"],
  ".",
  " ",
  "You can use ",
  StyleBox["DisplayForm", "MR"],
  " to print box expressions in an explicit two-dimensional form."
}], "Text",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(SuperscriptBox[x, 2] // DisplayForm\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[BoxData[
    TagBox[\(x\^2\),
      DisplayForm]], "Output",
  CellLabel->"Out[1]//DisplayForm=",
  CellTags->{"b:44.1", "IPM3.10.3.2"}]
}, Open  ]],

Cell[TextData[{
  "There are many different box objects in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". Below are just a few commonly used box objects."
}], "Text",
  PageBreakAbove->True,
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell["\<\
Cell[BoxData[
    SqrtBox[\"2\"]], \"Input\"]\
\>", "Program",
  CellTags->{"i:21", "b:44.1", "IPM3.10.3.2"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[BoxData[
    \(\@2\)], "Input",
  Evaluatable->False,
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell["\<\
Cell[BoxData[
    FractionBox[\"x\", \"y\"]], \"Input\"]\
\>", "Program",
  CellTags->{"i:22", "b:44.1", "IPM3.10.3.2"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[BoxData[
    \(x\/y\)], "Input",
  Evaluatable->False,
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell["\<\
Cell[BoxData[
    RowBox[{
      SubsuperscriptBox[\"\[Integral]\", \"a\", \"b\"], 
      RowBox[{\"x\", \" \", 
        RowBox[{\"\[DifferentialD]\", \"x\"}]
      }]
    }]
  ], \"Input\"]\
\>", "Program",
  CellTags->{"i:23", "b:44.1", "IPM3.10.3.2"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[BoxData[
    \(\[Integral]\_a\%b x\ \[DifferentialD]x\)], "Input",
  Evaluatable->False,
  CellTags->{"b:44.1", "IPM3.10.3.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["GraphicsData", "Subsection",
  CellTags->{"b:44.2", "IPM3.10.3.3", "c:10"}],

Cell[TextData[{
  "Another type of data wrapper that you will encounter is ",
  StyleBox["GraphicsData", "MR"],
  ", used to indicate a graphical object in the cell. For example, creating a \
graphics object in the front end displays a plot."
}], "TextFirst",
  CellTags->{"i:24", "b:44.2", "IPM3.10.3.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Plot[Sin[x], {x, 0, 2  \[Pi]}];\)\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
0.0238095 0.151576 0.309016 0.294303 [
[.17539 .29652 -4.5 -11.875 ]
[.17539 .29652 4.5 0 ]
[.32696 .29652 -4.5 -11.875 ]
[.32696 .29652 4.5 0 ]
[.47854 .29652 -4.5 -11.875 ]
[.47854 .29652 4.5 0 ]
[.63011 .29652 -4.5 -11.875 ]
[.63011 .29652 4.5 0 ]
[.78169 .29652 -4.5 -11.875 ]
[.78169 .29652 4.5 0 ]
[.93327 .29652 -4.5 -11.875 ]
[.93327 .29652 4.5 0 ]
[.01131 .01471 -15.1875 -5.9375 ]
[.01131 .01471 0 5.9375 ]
[.01131 .16187 -22.6875 -5.9375 ]
[.01131 .16187 0 5.9375 ]
[.01131 .45617 -16.5 -5.9375 ]
[.01131 .45617 0 5.9375 ]
[.01131 .60332 -9 -5.9375 ]
[.01131 .60332 0 5.9375 ]
[ 0 0 0 0 ]
[ 1 .61803 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.17539 .30902 m
.17539 .31527 L
s
gsave
.17539 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.32696 .30902 m
.32696 .31527 L
s
gsave
.32696 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(2) show
1.000 setlinewidth
grestore
.47854 .30902 m
.47854 .31527 L
s
gsave
.47854 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(3) show
1.000 setlinewidth
grestore
.63011 .30902 m
.63011 .31527 L
s
gsave
.63011 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(4) show
1.000 setlinewidth
grestore
.78169 .30902 m
.78169 .31527 L
s
gsave
.78169 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(5) show
1.000 setlinewidth
grestore
.93327 .30902 m
.93327 .31527 L
s
gsave
.93327 .29652 -65.5 -15.875 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(6) show
1.000 setlinewidth
grestore
.125 Mabswid
.05412 .30902 m
.05412 .31277 L
s
.08444 .30902 m
.08444 .31277 L
s
.11476 .30902 m
.11476 .31277 L
s
.14507 .30902 m
.14507 .31277 L
s
.2057 .30902 m
.2057 .31277 L
s
.23602 .30902 m
.23602 .31277 L
s
.26633 .30902 m
.26633 .31277 L
s
.29665 .30902 m
.29665 .31277 L
s
.35728 .30902 m
.35728 .31277 L
s
.38759 .30902 m
.38759 .31277 L
s
.41791 .30902 m
.41791 .31277 L
s
.44822 .30902 m
.44822 .31277 L
s
.50885 .30902 m
.50885 .31277 L
s
.53917 .30902 m
.53917 .31277 L
s
.56948 .30902 m
.56948 .31277 L
s
.5998 .30902 m
.5998 .31277 L
s
.66043 .30902 m
.66043 .31277 L
s
.69074 .30902 m
.69074 .31277 L
s
.72106 .30902 m
.72106 .31277 L
s
.75138 .30902 m
.75138 .31277 L
s
.81201 .30902 m
.81201 .31277 L
s
.84232 .30902 m
.84232 .31277 L
s
.87264 .30902 m
.87264 .31277 L
s
.90295 .30902 m
.90295 .31277 L
s
.96358 .30902 m
.96358 .31277 L
s
.9939 .30902 m
.9939 .31277 L
s
.25 Mabswid
0 .30902 m
1 .30902 L
s
.02381 .01471 m
.03006 .01471 L
s
gsave
.01131 .01471 -76.1875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.02381 .16187 m
.03006 .16187 L
s
gsave
.01131 .16187 -83.6875 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Mathematica1
%%IncludeFont: Mathematica1
/Mathematica1 findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(-) show
69.188 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.02381 .45617 m
.03006 .45617 L
s
gsave
.01131 .45617 -77.5 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(0.5) show
1.000 setlinewidth
grestore
.02381 .60332 m
.03006 .60332 L
s
gsave
.01131 .60332 -70 -9.9375 Mabsadd m
1 1 Mabs scale
currentpoint translate
0 19.875 translate 1 -1 scale
/g { setgray} bind def
/k { setcmykcolor} bind def
/p { gsave} bind def
/r { setrgbcolor} bind def
/w { setlinewidth} bind def
/C { curveto} bind def
/F { fill} bind def
/L { lineto} bind def
/rL { rlineto} bind def
/P { grestore} bind def
/s { stroke} bind def
/S { show} bind def
/N {currentpoint 3 -1 roll show moveto} bind def
/Msf { findfont exch scalefont [1 0 0 -1 0 0 ] makefont setfont} bind def
/m { moveto} bind def
/Mr { rmoveto} bind def
/Mx {currentpoint exch pop moveto} bind def
/My {currentpoint pop exch moveto} bind def
/X {0 rmoveto} bind def
/Y {0 exch rmoveto} bind def
63.000 13.000 moveto
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9.000 scalefont
[1 0 0 -1 0 0 ] makefont setfont
0.000 0.000 0.000 setrgbcolor
(1) show
1.000 setlinewidth
grestore
.125 Mabswid
.02381 .04414 m
.02756 .04414 L
s
.02381 .07357 m
.02756 .07357 L
s
.02381 .103 m
.02756 .103 L
s
.02381 .13243 m
.02756 .13243 L
s
.02381 .1913 m
.02756 .1913 L
s
.02381 .22073 m
.02756 .22073 L
s
.02381 .25016 m
.02756 .25016 L
s
.02381 .27959 m
.02756 .27959 L
s
.02381 .33845 m
.02756 .33845 L
s
.02381 .36788 m
.02756 .36788 L
s
.02381 .39731 m
.02756 .39731 L
s
.02381 .42674 m
.02756 .42674 L
s
.02381 .4856 m
.02756 .4856 L
s
.02381 .51503 m
.02756 .51503 L
s
.02381 .54446 m
.02756 .54446 L
s
.02381 .57389 m
.02756 .57389 L
s
.25 Mabswid
.02381 0 m
.02381 .61803 L
s
0 0 m
1 0 L
1 .61803 L
0 .61803 L
closepath
clip
newpath
.5 Mabswid
.02381 .30902 m
.06244 .38322 L
.10458 .45852 L
.14415 .51889 L
.18221 .56357 L
.20178 .58047 L
.21267 .58793 L
.22272 .59354 L
.23293 .59796 L
.23832 .59976 L
.24402 .60127 L
.24641 .60178 L
.24897 .60225 L
.25119 .60258 L
.25361 .60288 L
.25507 .60302 L
.25643 .60313 L
.25773 .60321 L
.2584 .60324 L
.25912 .60327 L
.25983 .60329 L
.26048 .60331 L
.26176 .60332 L
.26293 .60331 L
.26416 .60329 L
.26538 .60324 L
.26607 .60321 L
.26671 .60317 L
.26912 .60299 L
.27131 .60275 L
.27364 .60244 L
.27847 .60156 L
.2838 .60025 L
.29326 .59705 L
.30213 .59301 L
.32232 .58025 L
.34082 .56432 L
.38045 .51781 L
.41855 .45965 L
.45912 .38744 L
.49816 .31259 L
.53966 .23289 L
.57964 .1614 L
.61811 .10221 L
.63759 .07707 L
.65902 .05386 L
.67938 .03652 L
.69843 .02473 L
.70884 .02018 L
.71396 .01844 L
Mistroke
.71872 .01712 L
.72301 .01617 L
.72758 .01542 L
.7302 .01511 L
.73148 .01499 L
.73266 .0149 L
.73371 .01484 L
.73487 .01478 L
.73603 .01474 L
.73725 .01472 L
.73856 .01472 L
.73977 .01473 L
.7404 .01475 L
.7411 .01477 L
.74251 .01484 L
.74375 .01492 L
.74508 .01503 L
.74748 .01528 L
.74985 .0156 L
.75235 .01601 L
.7568 .01695 L
.76198 .01836 L
.7668 .01998 L
.77773 .02472 L
.78828 .0307 L
.79823 .03757 L
.81683 .05353 L
.85534 .09846 L
.89632 .16101 L
.93577 .23146 L
.97371 .3042 L
.97619 .30902 L
Mfstroke
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[2]:=",
  ImageSize->{288, 177.938},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`40004P0000/A000`40O003h00Oogoo8Goo003oOolQ
Ool00?moob5oo`00ogoo8Goo003oOolQOol001Yoo`03001oogoo0?moo`Aoo`006Woo00<007ooOol0
ogoo17oo000AOol00`00Oomoo`06Ool00`00Oomoo`3oOol4Ool0015oo`03001oogoo00Ioo`03001o
ogoo0?moo`Aoo`002Woo10000goo00<007ooOol01Woo0P00^Woo1@00AGoo000AOol00`00Oomoo`06
Ool00`00Oomoo`2eOol50004Ool50010Ool0015oo`03001oogoo00Ioo`03001oogoo0;=oo`8000io
o`8003ioo`0047oo0P0027oo00<007ooOol0/Goo0P004Woo0P00?7oo000AOol00`00Oomoo`06Ool0
0`00Oomoo`2_Ool2000FOol2000jOol001Yoo`03001oogoo0:eoo`8001Yoo`8003Qoo`006Woo00<0
07ooOol0[7oo00<007ooOol077oo0P00=Woo000JOol00`00Oomoo`2[Ool00`00Oomoo`0OOol00`00
Oomoo`0cOol001Yoo`800:]oo`03001oogoo025oo`03001oogoo039oo`006Woo00<007ooOol0Z7oo
0P009Goo00<007ooOol0<Goo000JOol00`00Oomoo`2WOol00`00Oomoo`0VOol00`00Oomoo`0`Ool0
01Yoo`03001oogoo0:Ioo`03001oogoo02Qoo`03001oogoo02moo`006Woo00<007ooOol0YGoo00<0
07ooOol0:Woo00<007ooOol0;Woo000JOol00`00Oomoo`2TOol00`00Oomoo`0/Ool00`00Oomoo`0]
Ool001Yoo`03001oogoo0:=oo`03001oogoo02eoo`03001oogoo02eoo`006Woo00<007ooOol0Xgoo
00<007ooOol0;Woo00<007ooOol0;7oo000JOol2002SOol00`00Oomoo`0`Ool00`00Oomoo`0[Ool0
01Yoo`03001oogoo0:5oo`03001oogoo039oo`03001oogoo02Yoo`006Woo00<007ooOol0X7oo00<0
07ooOol0=7oo00<007ooOol0:Goo000JOol00`00Oomoo`2OOol00`00Oomoo`0fOol00`00Oomoo`0X
Ool001Yoo`03001oogoo09ioo`03001oogoo03Moo`03001oogoo02Qoo`006Woo00<007ooOol0WWoo
00<007ooOol0>7oo00<007ooOol09goo000JOol00`00Oomoo`2MOol00`00Oomoo`0jOol00`00Oomo
o`0VOol001Yoo`03001oogoo09aoo`03001oogoo03aoo`03001oogoo02Eoo`006Woo0P00W7oo00<0
07ooOol0?Woo00<007ooOol097oo000JOol00`00Oomoo`2JOol00`00Oomoo`0oOol00`00Oomoo`0T
Ool001Yoo`03001oogoo09Yoo`03001oogoo041oo`03001oogoo02=oo`006Woo00<007ooOol0VGoo
00<007ooOol0@Woo00<007ooOol08Woo000JOol00`00Oomoo`2HOol00`00Oomoo`13Ool00`00Oomo
o`0ROol001Yoo`03001oogoo09Qoo`03001oogoo04Aoo`03001oogoo025oo`006Woo00<007ooOol0
Ugoo00<007ooOol0AWoo00<007ooOol087oo000JOol00`00Oomoo`2GOol00`00Oomoo`16Ool00`00
Oomoo`0POol001Yoo`8009Moo`03001oogoo04Qoo`03001oogoo01moo`006Woo00<007ooOol0UGoo
00<007ooOol0BWoo00<007ooOol07Woo000JOol00`00Oomoo`2EOol00`00Oomoo`1;Ool00`00Oomo
o`0MOol001Yoo`03001oogoo09Aoo`03001oogoo04aoo`03001oogoo01eoo`006Woo00<007ooOol0
Tgoo00<007ooOol0CWoo00<007ooOol077oo000JOol00`00Oomoo`2COol00`00Oomoo`1?Ool00`00
Oomoo`0KOol000Qoo`<000=oo`03001oogoo008000Moo`03001oogoo099oo`03001oogoo051oo`03
001oogoo01]oo`0027oo00@007ooOol000Aoo`04001oogoo0006Ool00`00Oomoo`2BOol00`00Oomo
o`1AOol00`00Oomoo`0JOol000=oo`@00005Ool007ooOol00007Ool00`00Oomoo`04Ool2002BOol0
0`00Oomoo`1COol00`00Oomoo`0IOol000Qoo`04001oogoo0004Ool01000Oomoo`001Woo00<007oo
Ool0T7oo00<007ooOol0E7oo00<007ooOol06Goo0008Ool01000Oomoo`0017oo0`001goo00<007oo
Ool0T7oo00<007ooOol0EGoo00<007ooOol067oo0008Ool01000Oomoo`001Goo00<007ooOol01Woo
00<007ooOol0Sgoo00<007ooOol0EWoo00<007ooOol067oo0009Ool20006Ool30006Ool00`00Oomo
o`2?Ool00`00Oomoo`1GOol00`00Oomoo`0GOol001Yoo`03001oogoo08ioo`03001oogoo05Qoo`03
001oogoo01Moo`006Woo00<007ooOol0SWoo00<007ooOol0FGoo00<007ooOol05Woo000JOol00`00
Oomoo`2=Ool00`00Oomoo`1JOol00`00Oomoo`0FOol001Yoo`8008eoo`03001oogoo05aoo`03001o
ogoo01Eoo`006Woo00<007ooOol0S7oo00<007ooOol0G7oo00<007ooOol05Goo000JOol00`00Oomo
o`2;Ool00`00Oomoo`1NOol00`00Oomoo`0DOol001Yoo`03001oogoo08]oo`03001oogoo05ioo`03
001oogoo01Aoo`006Woo00<007ooOol0RWoo00<007ooOol0H7oo00<007ooOol04goo000JOol00`00
Oomoo`29Ool00`00Oomoo`1QOol00`00Oomoo`0COol001Yoo`03001oogoo08Uoo`03001oogoo069o
o`03001oogoo019oo`006Woo0P00RGoo00<007ooOol0Hgoo00<007ooOol04Woo000JOol00`00Oomo
o`28Ool00`00Oomoo`1TOol00`00Oomoo`0AOol001Yoo`03001oogoo08Moo`03001oogoo06Eoo`03
001oogoo015oo`006Woo00<007ooOol0Qgoo00<007ooOol0IWoo00<007ooOol047oo000JOol00`00
Oomoo`26Ool00`00Oomoo`1WOol00`00Oomoo`0@Ool001Yoo`03001oogoo08Eoo`03001oogoo06Uo
o`03001oogoo00moo`006Woo00<007ooOol0QGoo00<007ooOol0JGoo00<007ooOol03goo000JOol0
0`00Oomoo`24Ool00`00Oomoo`1[Ool00`00Oomoo`0>Ool001Yoo`8008Eoo`03001oogoo06]oo`03
001oogoo00ioo`006Woo00<007ooOol0Pgoo00<007ooOol0KGoo00<007ooOol03Goo000JOol00`00
Oomoo`23Ool00`00Oomoo`1]Ool00`00Oomoo`0=Ool001Yoo`03001oogoo089oo`03001oogoo06mo
o`03001oogoo00aoo`006Woo00<007ooOol0PWoo00<007ooOol0Kgoo00<007ooOol037oo000JOol0
0`00Oomoo`0UOol00`00Oomoo`0SOol4000VOol2000:Ool00`00Oomoo`0KOol00`00Oomoo`0TOol2
000VOol20005Ool00`00Oomoo`0;Ool001Yoo`03001oogoo02Eoo`03001oogoo02Aoo`03001oogoo
02Eoo`04001oogoo0009Ool00`00Oomoo`0KOol00`00Oomoo`0SOol01000Oomoo`0097oo00@007oo
Ool000Aoo`03001oogoo00]oo`006Woo00<007ooOol09Goo00<007ooOol09Goo00<007ooOol09goo
00<007ooOol01Woo00<007ooOol06Goo1@009goo00<007ooOol08Woo00@007ooOol000Eoo`03001o
ogoo00Yoo`006Woo0P009Woo00<007ooOol09Goo00<007ooOol09Woo00<007ooOol01goo00<007oo
Ool06Goo00@007ooOol002Eoo`04001oogoo000TOol01000Oomoo`001Goo00<007ooOol02Woo000J
Ool00`00Oomoo`0UOol00`00Oomoo`0VOol00`00Oomoo`0VOol00`00Oomoo`05Ool00`00Oomoo`0K
Ool00`00Ool0000UOol3000UOol30007Ool00`00Oomoo`09Ool001Yoo`03001oogoo02Aoo`8002Eo
o`04001oogoo000UOol01000Oomoo`001goo00<007ooOol077oo0P009Woo00<007ooOol097oo00@0
07ooOol000Ioo`03001oogoo00Uoo`006Woo00<007ooOol09Goo00<007ooOol097oo0P009goo0P00
1goo00<007ooOol07Woo00<007ooOol097oo0`009Goo0P0027oo00<007ooOol027oo000JOol00`00
Oomoo`1nOol00`00Oomoo`1gOol00`00Oomoo`08Ool001Yoo`03001oogoo07eoo`03001oogoo07Uo
o`03001oogoo00Moo`006Woo00<007ooOol0OGoo00<007ooOol0NGoo00<007ooOol01goo000JOol0
0`00Oomoo`1lOol00`00Oomoo`1kOol00`00Oomoo`06Ool001=oool000h000006Woo00<007ooOol0
1Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol0
1Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Woo00<007ooOol0
1Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol01Goo00<007ooOol0
0Woo00@007ooOol000Moo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Eoo`03001oogoo00Eoo`03001oogoo00Ioo`03001oogoo00Eoo`03001oogoo
00Eoo`03001oogoo00Eoo`40005oo`5oo`006Woo0P009Woo00<007ooOol09Woo00<007ooOol09Goo
00D007ooOomoo`0002Aoo`03001oogoo02Eoo`03001oogoo02Ioo`03001oogoo011oo`006Woo0P00
Ngoo00<007ooOol0QWoo000JOol00`00Ool0001iOol00`00Oomoo`27Ool001Yoo`03001oo`0007Uo
o`03001oogoo08Moo`006Woo00@007ooOol007Moo`03001oogoo08Qoo`006Woo00@007ooOol007Mo
o`03001oogoo08Qoo`006Woo0P000Woo00<007ooOol0Lgoo00<007ooOol0RGoo000JOol01@00Oomo
ogoo0000MGoo00<007ooOol0RGoo000JOol00`00Oomoo`02Ool00`00Oomoo`1aOol00`00Oomoo`2:
Ool001Yoo`03001oogoo009oo`03001oogoo075oo`03001oogoo08Yoo`006Woo00<007ooOol00goo
00<007ooOol0Kgoo00<007ooOol0Rgoo000JOol00`00Oomoo`03Ool00`00Oomoo`1_Ool00`00Oomo
o`2;Ool001Yoo`03001oogoo00Aoo`03001oogoo06eoo`03001oogoo08aoo`006Woo00<007ooOol0
17oo00<007ooOol0KGoo00<007ooOol0S7oo000JOol20006Ool00`00Oomoo`1[Ool00`00Oomoo`2=
Ool001Yoo`03001oogoo00Eoo`03001oogoo06]oo`03001oogoo08eoo`006Woo00<007ooOol01Woo
00<007ooOol0JGoo00<007ooOol0SWoo000JOol00`00Oomoo`06Ool00`00Oomoo`1YOol00`00Oomo
o`2>Ool001Yoo`03001oogoo00Moo`03001oogoo06Moo`03001oogoo08moo`006Woo00<007ooOol0
27oo00<007ooOol0IWoo00<007ooOol0Sgoo000JOol00`00Oomoo`08Ool00`00Oomoo`1UOol00`00
Oomoo`2@Ool001Yoo`03001oogoo00Uoo`03001oogoo06Aoo`03001oogoo091oo`006Woo0P002Woo
00<007ooOol0Hgoo00<007ooOol0TGoo000JOol00`00Oomoo`0:Ool00`00Oomoo`1ROol00`00Oomo
o`2AOol001Yoo`03001oogoo00Yoo`03001oogoo065oo`03001oogoo099oo`006Woo00<007ooOol0
2goo00<007ooOol0H7oo00<007ooOol0TWoo000JOol00`00Oomoo`0;Ool00`00Oomoo`1OOol00`00
Oomoo`2COol001Yoo`03001oogoo00aoo`03001oogoo05eoo`03001oogoo09Aoo`006Woo00<007oo
Ool037oo00<007ooOol0GGoo00<007ooOol0U7oo000JOol00`00Oomoo`0=Ool00`00Oomoo`1KOol0
0`00Oomoo`2EOol001Yoo`8000moo`03001oogoo05Yoo`03001oogoo09Eoo`006Woo00<007ooOol0
3Woo00<007ooOol0FGoo00<007ooOol0UWoo000JOol00`00Oomoo`0?Ool00`00Oomoo`1GOol00`00
Oomoo`2GOol001Yoo`03001oogoo00moo`03001oogoo05Moo`03001oogoo09Moo`006Woo00<007oo
Ool047oo00<007ooOol0EGoo00<007ooOol0V7oo0008Ool30003Ool00`00Oomoo`020007Ool00`00
Oomoo`0@Ool00`00Oomoo`1EOol00`00Oomoo`2HOol000Qoo`04001oogoo0004Ool01000Oomoo`00
1Woo00<007ooOol04Goo00<007ooOol0Dgoo00<007ooOol0VGoo0008Ool01000Oomoo`001goo00<0
07ooOol017oo0P004Woo00<007ooOol0Dgoo00<007ooOol0VGoo0008Ool01000Oomoo`0017oo00@0
07ooOol000Ioo`03001oogoo019oo`03001oogoo055oo`03001oogoo09Yoo`0027oo00@007ooOol0
00Aoo`<000Moo`03001oogoo01=oo`03001oogoo04moo`03001oogoo09]oo`0027oo00@007ooOol0
00Eoo`03001oogoo00Ioo`03001oogoo01=oo`03001oogoo04moo`03001oogoo09]oo`002Goo0P00
1Woo0`001Woo00<007ooOol057oo00<007ooOol0CGoo00<007ooOol0W7oo000JOol00`00Oomoo`0E
Ool00`00Oomoo`1;Ool00`00Oomoo`2MOol001Yoo`03001oogoo01Eoo`03001oogoo04]oo`03001o
ogoo09eoo`006Woo00<007ooOol05Woo00<007ooOol0BGoo00<007ooOol0WWoo000JOol2000HOol0
0`00Oomoo`18Ool00`00Oomoo`2NOol001Yoo`03001oogoo01Moo`03001oogoo04Moo`03001oogoo
09moo`006Woo00<007ooOol067oo00<007ooOol0AGoo00<007ooOol0X7oo000JOol00`00Oomoo`0I
Ool00`00Oomoo`14Ool00`00Oomoo`2POol001Yoo`03001oogoo01Yoo`03001oogoo049oo`03001o
ogoo0:5oo`006Woo00<007ooOol06Woo00<007ooOol0@Goo00<007ooOol0XWoo000JOol00`00Oomo
o`0KOol00`00Oomoo`10Ool00`00Oomoo`2ROol001Yoo`03001oogoo01aoo`03001oogoo03ioo`03
001oogoo0:=oo`006Woo0P007Goo00<007ooOol0?Woo00<007ooOol0Xgoo000JOol00`00Oomoo`0M
Ool00`00Oomoo`0lOol00`00Oomoo`2TOol001Yoo`03001oogoo01ioo`03001oogoo03Yoo`03001o
ogoo0:Eoo`006Woo00<007ooOol07goo00<007ooOol0>7oo00<007ooOol0YWoo000JOol00`00Oomo
o`0OOol00`00Oomoo`0gOol00`00Oomoo`2WOol001Yoo`03001oogoo021oo`03001oogoo03Eoo`03
001oogoo0:Qoo`006Woo00<007ooOol08Goo00<007ooOol0<goo00<007ooOol0ZGoo000JOol00`00
Oomoo`0ROol00`00Oomoo`0aOol00`00Oomoo`2ZOol001Yoo`8002Aoo`03001oogoo031oo`03001o
ogoo0:Yoo`006Woo00<007ooOol097oo00<007ooOol0;Woo00<007ooOol0Zgoo000JOol00`00Oomo
o`0TOol00`00Oomoo`0]Ool00`00Oomoo`2/Ool001Yoo`03001oogoo02Eoo`03001oogoo02]oo`03
001oogoo0:eoo`006Woo00<007ooOol09Woo00<007ooOol0:Goo00<007ooOol0[Woo000JOol00`00
Oomoo`0WOol00`00Oomoo`0WOol00`00Oomoo`2_Ool001Yoo`03001oogoo02Qoo`03001oogoo02Eo
o`03001oogoo0;1oo`006Woo00<007ooOol0:Goo00<007ooOol08goo00<007ooOol0/Goo000JOol2
000[Ool00`00Oomoo`0QOol00`00Oomoo`2bOol001Yoo`03001oogoo02]oo`03001oogoo01moo`03
001oogoo0;=oo`006Woo00<007ooOol0;7oo00<007ooOol07Goo00<007ooOol0]7oo000JOol00`00
Oomoo`0]Ool2000KOol2002gOol001Yoo`03001oogoo02moo`8001Moo`800;Uoo`006Woo00<007oo
Ool0<Goo0P0057oo00<007ooOol0^Goo000AOol00`00Oomoo`06Ool00`00Oomoo`0cOol2000?Ool3
002lOol0015oo`03001oogoo00Ioo`03001oogoo03Eoo`D000Eoo`D00;moo`004Goo00<007ooOol0
1Woo0P00>Woo1`00`goo000AOol00`00Oomoo`06Ool00`00Oomoo`3oOol4Ool0015oo`03001oogoo
00Ioo`03001oogoo0?moo`Aoo`0047oo0P0027oo00<007ooOol0ogoo17oo000AOol00`00Oomoo`06
Ool00`00Oomoo`3oOol4Ool00?moob5oo`00ogoo8Goo003oOolQOol00?moob5oo`00ogoo8Goo0000
\
\>"],
  ImageRangeCache->{{{164.938, 342.438}, {239.375, 130.063}} -> {-7.22671, \
1.55001, 0.0247448, 0.0127444}},
  CellTags->{"b:44.2", "IPM3.10.3.3"}]
}, Open  ]],

Cell["\<\
If you unformat the graphics cell, the first few lines would look like the \
following:\
\>", "TextFirst",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],

Cell["\<\
Cell[GraphicsData[\"PostScript\", \"\\<\\
%!
%%Creator: Mathematica
%%AspectRatio: .61803 
MathPictureStart
...\
\>", "Program",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],

Cell[TextData[{
  "Normally you will not create graphics objects from scratch so it would \
seem as if there is not too much you could do with ",
  StyleBox["GraphicsData", "MR"],
  " objects manually. But suppose you were interested in displaying your \
graphics to a notebook other than the one in which you evaluate the graphics \
input. For example, we could use ",
  StyleBox["NotebookPut", "MR"],
  " to write out a new notebook containing a graphics cell object as \
follows:"
}], "Text",
  CellTags->{"i:25", "b:44.2", "IPM3.10.3.3"}],

Cell[BoxData[
    \(MyDisplayChannel[gr_]\  := \n\t
      NotebookPut[
        Notebook[{Cell[
              GraphicsData["\<PostScript\>", 
                DisplayString[gr]], "\<Graphics\>"]}]]\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],

Cell[TextData[{
  "This is now used by giving ",
  StyleBox["MyDisplayChannel", "MR"],
  " as the value of ",
  StyleBox["DisplayFunction", "MR"],
  " for any plot your create."
}], "Text",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Plot3D[Sin[x\ y], {x, 0, 2\ \[Pi]}, {y, 0, 2\ \[Pi]}, 
      DisplayFunction \[Rule] MyDisplayChannel]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],

Cell[BoxData[
    InterpretationBox[\(NotebookObject[\[LeftSkeleton]"Untitled-5"\
\[RightSkeleton]]\),
      NotebookObject[ 
        FrontEndObject[ 
          LinkObject[ "3v8_shm", 1, 1]], 31],
      Editable->False]], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:44.2", "IPM3.10.3.3"}]
}, Open  ]],

Cell[TextData[{
  "Evaluating the above expression will cause a new notebook window to be \
created in your front end containing just the output of the ",
  StyleBox["Plot3D", "MR"],
  " command, a graphic of the surface ",
  Cell[BoxData[
      \(TraditionalForm\`sin(x\ y)\)]],
  "."
}], "Text",
  CellTags->{"b:44.2", "IPM3.10.3.3"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:44.2", "IPM3.10.3.4", "c:11"}],

Cell[TextData[{
  "Using ",
  StyleBox["NotebookPut", "MR"],
  ", create a notebook with several Text cells each containing a ",
  StyleBox["ValueBox", "MR"],
  " such as ",
  StyleBox["$Version", "MR"],
  ", ",
  StyleBox["$OperatingSystem", "MR"],
  ", and ",
  StyleBox["$UserName", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:44.2", "IPM3.10.3.4"}],

Cell[TextData[{
  "Using ",
  StyleBox["NotebookPut", "MR"],
  ", create a notebook with an Input cell containing the integral ",
  Cell[BoxData[
      \(TraditionalForm\`\[Integral]\(1\/\(1 - 
                x\^3\)\) \[DifferentialD]x\)]],
  ". Then evaluate the integral using ",
  StyleBox["SelectionMove", "MR"],
  " and ",
  StyleBox["SelectionEvaluate", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:44.2", "IPM3.10.3.4"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["GridBoxes", "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 3}},
  CellTags->{"IPM3.10.4.1", "IPM3.10.4.2", "IPM3.10.4.3", "c:12"}],

Cell[CellGroupData[{

Cell["ShowTable", "Subsection",
  CellTags->{"b:45.0", "IPM3.10.4.1", "c:13"}],

Cell[TextData[{
  "Whenever you create a two-dimensional expression consisting of some number \
of rows and columns, ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " represents that expression as a ",
  StyleBox["GridBox", "MR"],
  " object. For example, if you used the BasicInput palette to create a ",
  Cell[BoxData[
      \(TraditionalForm\`2\[Cross]2\)]],
  " matrix, it would be represented as follows:"
}], "TextFirst",
  CellTags->{"i:26", "b:45.0", "IPM3.10.4.1"}],

Cell["\<\
Cell[BoxData[GridBox[{
        {\"a\", \"b\"},
        {\"c\", \"d\"}
        }]], \"Input\"]\
\>", "Program",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell["The formatted version of this cell looks like this:", "TextFirst",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[GridBox[{
        {"a", "b"},
        {"c", "d"}
        }]], "Input",
  Evaluatable->False,
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[TextData[{
  "Looking at the ",
  StyleBox["GridBox", "MR"],
  " object, you should see that it is identical (structurally) to a matrix in \
",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ", which is really just a list of lists. "
}], "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    RowBox[{"FullForm", "[", GridBox[{
          {"a", "b"},
          {"c", "d"}
          }], "]"}]], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    TagBox[
      StyleBox[\(List[List[a, b], List[c, d]]\),
        ShowSpecialCharacters->False,
        ShowStringCharacters->True,
        NumberMarks->True],
      FullForm]], "Output",
  CellLabel->"Out[1]//FullForm=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \({{a, b}, {c, d}} // MatrixForm\)], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    TagBox[
      RowBox[{"(", "\[NoBreak]", GridBox[{
            {"a", "b"},
            {"c", "d"}
            }], "\[NoBreak]", ")"}],
      Function[ BoxForm`e$, 
        MatrixForm[ BoxForm`e$]]]], "Output",
  CellLabel->"Out[2]//MatrixForm=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[TextData[{
  "Using ",
  StyleBox["GridBox", "MR"],
  "es, let us create a function for displaying arrays of data in a formatted \
table. First we create some sample data."
}], "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    \(\(data = {{"\<\[Alpha]\>", "\<\[Beta]\>", "\<\[Gamma]\>"}, {1.234, 
            2.3451, 3.4567801}, {SqrtBox["\<\[Pi]\>"], \
\*"\"\<\!\(x\/y\)\>\"", "\<\[CapitalGamma](n)\>"}};\)\)], "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[TextData[{
  "We can put this data into a ",
  StyleBox["GridBox", "MR"],
  " and immediately print it in a two-dimensional grid using ",
  StyleBox["DisplayForm", "MR"],
  "."
}], "Text",
  CellTags->{"i:27", "b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(GridBox[data] // DisplayForm\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    TagBox[GridBox[{
          {"\[Alpha]", "\[Beta]", "\[Gamma]"},
          {"1.234", "2.3451", "3.4567801"},
          {\(\@\[Pi]\), \(x\/y\), \(\[CapitalGamma] \((n)\)\)}
          }],
      DisplayForm]], "Output",
  CellLabel->"Out[4]//DisplayForm=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[TextData[{
  StyleBox["GridBox", "MR"],
  " can be given several options that control its appearance. "
}], "Text",
  CellTags->{"i:28", "b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[GridBox]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    \({GridBaseline \[Rule] Axis, RowSpacings \[Rule] 1.`, 
      ColumnSpacings \[Rule] 0.800000011920929`, 
      ColumnWidths \[Rule] Automatic, RowAlignments \[Rule] Baseline, 
      ColumnAlignments \[Rule] {Center}, GridFrame \[Rule] False, 
      GridFrameMargins \[Rule] {{0.4000000059604645`, 
            0.4000000059604645`}, {0.5`, 0.5`}}, RowLines \[Rule] False, 
      ColumnLines \[Rule] False, RowMinHeight \[Rule] 1.`, 
      RowsEqual \[Rule] False, ColumnsEqual \[Rule] False, 
      AutoDelete \[Rule] True, AllowScriptLevelChange \[Rule] True, 
      MultilineFunction \[Rule] None, 
      GridDefaultElement \[RuleDelayed] "\[Placeholder]"}\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[TextData[{
  "Let us add a frame, make the margins around each grid element a bit larger \
than the default, and add some lines between the rows and columns. Usually \
you will set the values for ",
  StyleBox["GridFrame", "MR"],
  ", ",
  StyleBox["RowLines", "MR"],
  ", and ",
  StyleBox["ColumnLines", "MR"],
  " to either ",
  StyleBox["True", "MR"],
  " or ",
  StyleBox["False", "MR"],
  " to enable or disable these elements. Giving an explicit number as the \
value of each of these options gives the thickness of the line that is drawn \
for that object."
}], "Text",
  CellTags->{"i:29", "b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(GridBox[data, \[IndentingNewLine]GridFrame \[Rule] 1.2, 
        GridFrameMargins \[Rule] {{1, 1}, {1, 
              1}}, \[IndentingNewLine]RowLines \[Rule] 1, 
        ColumnLines \[Rule] 1] // DisplayForm\)], "Input",
  CellLabel->"In[6]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    TagBox[GridBox[{
          {"\[Alpha]", "\[Beta]", "\[Gamma]"},
          {"1.234", "2.3451", "3.4567801"},
          {\(\@\[Pi]\), \(x\/y\), \(\[CapitalGamma] \((n)\)\)}
          },
        GridFrame->1.2,
        GridFrameMargins->True,
        RowLines->True,
        ColumnLines->True],
      DisplayForm]], "Output",
  CellLabel->"Out[6]//DisplayForm=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[TextData[{
  "Now we can bundle up this code and turn all of it into a function, ",
  StyleBox["ShowTable", "MR"],
  ". If we wish, we can add some formatting, but to do so we have to wrap the \
",
  StyleBox["GridBox", "MR"],
  " in a ",
  StyleBox["StyleBox", "MR"],
  ". ",
  StyleBox["FontSize", "MR"],
  ", ",
  StyleBox["FontFamily", "MR"],
  ", ",
  StyleBox["Background", "MR"],
  ", and ",
  StyleBox["SingleLetterItalics", "MR"],
  " are all options to ",
  StyleBox["StyleBox", "MR"],
  ". "
}], "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    \(ShowTable[data_] := 
      DisplayForm[
        StyleBox[\[IndentingNewLine]GridBox[
            data, \[IndentingNewLine]GridFrame \[Rule] 1.2, 
            GridFrameMargins \[Rule] {{1, 1}, {1, 
                  1}}, \[IndentingNewLine]RowLines \[Rule] 1, 
            ColumnLines \[Rule] 
              1], \[IndentingNewLine]FontFamily \[Rule] "\<Times\>", \
\[IndentingNewLine]Background \[Rule] GrayLevel[ .8], 
          SingleLetterItalics \[Rule] True\[IndentingNewLine]]]\)], "Input",
  CellLabel->"In[7]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowTable[data]\)], "Input",
  CellLabel->"In[8]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    TagBox[
      StyleBox[GridBox[{
            {"\[Alpha]", "\[Beta]", "\[Gamma]"},
            {"1.234", "2.3451", "3.4567801"},
            {\(\@\[Pi]\), \(x\/y\), \(\[CapitalGamma] \((n)\)\)}
            },
          GridFrame->1.2,
          GridFrameMargins->True,
          RowLines->True,
          ColumnLines->True],
        SingleLetterItalics->True,
        FontFamily->"Times",
        Background->GrayLevel[0.8]],
      DisplayForm]], "Output",
  CellLabel->"Out[8]//DisplayForm=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[TextData[{
  "Sometimes the data you work with will need to be manipulated in some way \
to display it. The following is another example of the use of ",
  StyleBox["ShowTable", "MR"],
  ", but one for which we first need to think about the dimensions of our \
data. Consider displaying a table of reciprocals of rep units, numbers \
consisting entirely of 1s. "
}], "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    \(RepUnit[n_?Positive] := Nest[10\ #1 + 1 &, 1, n - 1]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"i:30", "b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = Map[1\/RepUnit[#] &, Range[12]]\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    \({1, 1\/11, 1\/111, 1\/1111, 1\/11111, 1\/111111, 1\/1111111, 
      1\/11111111, 1\/111111111, 1\/1111111111, 1\/11111111111, 
      1\/111111111111}\)], "Output",
  CellLabel->"Out[10]=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell["\<\
Since the above output contains 12 expressions, we need to explicitly \
partition it to be rectangular. First we partition the data into rows of \
three elements (columns) each.\
\>", "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowTable[Partition[expr, 3]]\)], "Input",
  CellLabel->"In[11]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    TagBox[
      StyleBox[GridBox[{
            {"1", \(1\/11\), \(1\/111\)},
            {\(1\/1111\), \(1\/11111\), \(1\/111111\)},
            {\(1\/1111111\), \(1\/11111111\), \(1\/111111111\)},
            {\(1\/1111111111\), \(1\/11111111111\), \(1\/111111111111\)}
            },
          GridFrame->1.2,
          GridFrameMargins->True,
          RowLines->True,
          ColumnLines->True],
        SingleLetterItalics->True,
        FontFamily->"Times",
        Background->GrayLevel[0.8]],
      DisplayForm]], "Output",
  CellLabel->"Out[11]//DisplayForm=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell["Here we partition the data into rows of four elements each.", "Text",
  PageBreakAbove->True,
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowTable[Partition[expr, 4]]\)], "Input",
  CellLabel->"In[12]:=",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[BoxData[
    TagBox[
      StyleBox[GridBox[{
            {"1", \(1\/11\), \(1\/111\), \(1\/1111\)},
            {\(1\/11111\), \(1\/111111\), \(1\/1111111\), \(1\/11111111\)},
            {\(1\/111111111\), \(1\/1111111111\), \(1\/11111111111\), \
\(1\/111111111111\)}
            },
          GridFrame->1.2,
          GridFrameMargins->True,
          RowLines->True,
          ColumnLines->True],
        SingleLetterItalics->True,
        FontFamily->"Times",
        Background->GrayLevel[0.8]],
      DisplayForm]], "Output",
  CellLabel->"Out[12]//DisplayForm="]
}, Open  ]],

Cell["\<\
In the above tables, we are manually partitioning the rows and columns into \
sublists that will be rectangular when they are put into the table. It would \
be good programming style to take that task from the user and do it \
automatically. We leave this as an exercise.\
\>", "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["TriangleForm", "Subsection",
  CellTags->{"b:45.1", "IPM3.10.4.2", "c:14"}],

Cell[TextData[{
  "In this section we will use ",
  StyleBox["GridBox", "MR"],
  " to develop a function for displaying an array in a triangular format. \
Such a function is quite useful for displaying the elements of Pascal\
\[CloseCurlyQuote]s triangle in the familiar triangular array. "
}], "TextFirst",
  CellTags->{"i:31", "b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    TagBox[GridBox[{
          {"", "", "", "", "", "", "", "", "1", "", "", "", "", "", "", "", 
            ""},
          {"", "", "", "", "", "", "", "1", "", "1", "", "", "", "", "", "", 
            ""},
          {"", "", "", "", "", "", "1", "", "2", "", "1", "", "", "", "", "", 
            ""},
          {"", "", "", "", "", "1", "", "3", "", "3", "", "1", "", "", "", "",
             ""},
          {"", "", "", "", "1", "", "4", "", "6", "", "4", "", "1", "", "", 
            "", ""},
          {"", "", "", "1", "", "5", "", "10", "", "10", "", "5", "", "1", "",
             "", ""},
          {"", "", "1", "", "6", "", "15", "", "20", "", "15", "", "6", "", 
            "1", "", ""},
          {"", "1", "", "7", "", "21", "", "35", "", "35", "", "21", "", "7", 
            "", "1", ""},
          {"1", "", "8", "", "28", "", "56", "", "70", "", "56", "", "28", "",
             "8", "", "1"}
          }],
      DisplayForm]], "Output"],

Cell[TextData[{
  "First let us create a function for generating the first ",
  Cell[BoxData[
      \(TraditionalForm\`n\)]],
  " rows of Pascal\[CloseCurlyQuote]s triangle."
}], "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \(PascalTable[rows_] := 
      Table[Binomial[n, m], {n, 0, rows}, {m, 0, n}]\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->{"i:32", "b:45.1", "IPM3.10.4.2"}],

Cell["Here are the first four rows (including the 0th row).", "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = PascalTable[3]\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({{1}, {1, 1}, {1, 2, 1}, {1, 3, 3, 1}}\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell["\<\
If we put empty strings around the elements in the appropriate places we can \
see what the grid should look like.\
\>", "Text",
  PageBreakAbove->True,
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(GridBox[{\[IndentingNewLine]{"\<\>", "\<\>", 
            1, "\<\>", "\<\>"}, \[IndentingNewLine]{"\<\>", 1, "\<\>", 
            1, "\<\>"}, \[IndentingNewLine]{1, "\<\>", 2, "\<\>", 
            1}\[IndentingNewLine]}] // DisplayForm\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    TagBox[GridBox[{
          {"", "", "1", "", ""},
          {"", "1", "", "1", ""},
          {"1", "", "2", "", "1"}
          }],
      DisplayForm]], "Output",
  CellLabel->"Out[15]//DisplayForm=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell["\<\
So we need to develop a function to insert these empty strings between each \
element in each row and we also need to pad out each row to the length of the \
longest row in the entire table. First we write the function to pad each row.\
\
\>", "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \(pad[lis_] := 
      PadLeft[lis, 2  Length[expr] - 1, "\<\>", 
        Round[\((2  Length[expr] - 1 - Length[lis])\)/2]]\)], "Input",
  CellLabel->"In[16]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(pad[expr[\([1]\)]]\)], "Input",
  CellLabel->"In[17]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({\*"\<\"\"\>", \*"\<\"\"\>", \*"\<\"\"\>", 
      1, \*"\<\"\"\>", \*"\<\"\"\>", \*"\<\"\"\>"}\)], "Output",
  CellLabel->"Out[17]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(pad[expr[\([2]\)]]\)], "Input",
  CellLabel->"In[18]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({\*"\<\"\"\>", \*"\<\"\"\>", \*"\<\"\"\>", 1, 
      1, \*"\<\"\"\>", \*"\<\"\"\>"}\)], "Output",
  CellLabel->"Out[18]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell["\<\
Now to insert the appropriate number of empty strings between elements, let \
us first manually insert space in a few rows.\
\>", "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Insert[expr[\([2]\)], "\<\>", {{2}}]\)], "Input",
  CellLabel->"In[19]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({1, \*"\<\"\"\>", 1}\)], "Output",
  CellLabel->"Out[19]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Insert[expr[\([3]\)], "\<\>", {{2}, {3}}]\)], "Input",
  CellLabel->"In[20]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({1, \*"\<\"\"\>", 2, \*"\<\"\"\>", 1}\)], "Output",
  CellLabel->"Out[20]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(Insert[expr[\([4]\)], "\<\>", {{2}, {3}, {4}}]\)], "Input",
  CellLabel->"In[21]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({1, \*"\<\"\"\>", 3, \*"\<\"\"\>", 3, \*"\<\"\"\>", 1}\)], "Output",
  CellLabel->"Out[21]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[TextData[{
  "Here is the function to create the third argument for ",
  StyleBox["Insert", "MR"],
  "."
}], "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[List, Rest[Range[Length[{1, 3, 3, 1}]]]]\)], "Input",
  CellLabel->"In[22]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({{2}, {3}, {4}}\)], "Output",
  CellLabel->"Out[22]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell["\<\
Here is the function to add the appropriate amount of space between elements \
in each row.\
\>", "Text",
  PageBreakAbove->True,
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \(addspace[lis_] := 
      Insert[lis, "\<\>", Map[List, Rest[Range[Length[lis]]]]]\)], "Input",
  CellLabel->"In[23]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(addspace[expr[\([3]\)]]\)], "Input",
  CellLabel->"In[24]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({1, \*"\<\"\"\>", 2, \*"\<\"\"\>", 1}\)], "Output",
  CellLabel->"Out[24]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
    \(addspace[expr[\([1]\)]]\)], "Input",
  CellLabel->"In[25]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({1}\)], "Output",
  CellLabel->"Out[25]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[TextData[{
  "This maps the ",
  StyleBox["addspace", "MR"],
  " function across each row of the Pascal table."
}], "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(expr = Map[addspace, PascalTable[3]]\)], "Input",
  CellLabel->"In[26]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({{1}, {1, \*"\<\"\"\>", 1}, {1, \*"\<\"\"\>", 2, \*"\<\"\"\>", 
        1}, {1, \*"\<\"\"\>", 3, \*"\<\"\"\>", 3, \*"\<\"\"\>", 
        1}}\)], "Output",
  CellLabel->"Out[26]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[TextData[{
  "Then we pad out each row using our ",
  StyleBox["pad", "MR"],
  " function developed above."
}], "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[pad, expr]\)], "Input",
  CellLabel->"In[27]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \({{\*"\<\"\"\>", \*"\<\"\"\>", \*"\<\"\"\>", 
        1, \*"\<\"\"\>", \*"\<\"\"\>", \*"\<\"\"\>"}, {\*"\<\"\"\>", \
\*"\<\"\"\>", 1, \*"\<\"\"\>", 1, \*"\<\"\"\>", \*"\<\"\"\>"}, {\*"\<\"\"\>", 
        1, \*"\<\"\"\>", 2, \*"\<\"\"\>", 1, \*"\<\"\"\>"}, {1, \*"\<\"\"\>", 
        3, \*"\<\"\"\>", 3, \*"\<\"\"\>", 1}}\)], "Output",
  CellLabel->"Out[27]=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[TextData[{
  "Finally we put this expression into a ",
  StyleBox["GridBox", "MR"],
  " and display it."
}], "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(GridBox[%] // DisplayForm\)], "Input",
  CellLabel->"In[28]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    TagBox[GridBox[{
          {"", "", "", "1", "", "", ""},
          {"", "", "1", "", "1", "", ""},
          {"", "1", "", "2", "", "1", ""},
          {"1", "", "3", "", "3", "", "1"}
          }],
      DisplayForm]], "Output",
  CellLabel->"Out[28]//DisplayForm=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[TextData[{
  "Here is the ",
  StyleBox["TriangleForm", "MR"],
  " function then consisting of the above pieces."
}], "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    \(TriangleForm[lis_List] := 
      Module[{addspace, expr, 
          len = Length[lis]}, \[IndentingNewLine]addspace[l_] := 
          Insert[l, "\<\>", Map[List, Rest[Range[Length[l]]]]]; 
        expr = Map[addspace, lis]; \[IndentingNewLine]DisplayForm[
          GridBox[Map[
              PadLeft[#, 2\ len - 1, "\<\>", 
                  Round[1\/2\ \((2\ len - 1 - Length[#])\)]] &, 
              expr]]\[IndentingNewLine]]\[IndentingNewLine]]\)], "Input",
  CellLabel->"In[29]:=",
  CellTags->{"i:33", "b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(PascalTable[5] // TriangleForm\)], "Input",
  CellLabel->"In[30]:=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[BoxData[
    TagBox[GridBox[{
          {"", "", "", "", "", "1", "", "", "", "", ""},
          {"", "", "", "", "1", "", "1", "", "", "", ""},
          {"", "", "", "1", "", "2", "", "1", "", "", ""},
          {"", "", "1", "", "3", "", "3", "", "1", "", ""},
          {"", "1", "", "4", "", "6", "", "4", "", "1", ""},
          {"1", "", "5", "", "10", "", "10", "", "5", "", "1"}
          }],
      DisplayForm]], "Output",
  CellLabel->"Out[30]//DisplayForm=",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:45.1", "IPM3.10.4.3", "c:15"}],

Cell[TextData[{
  "Modify ",
  StyleBox["ShowTable", "MR"],
  " so that it can display a user-specified heading in the first row of the \
grid. Include formatting to set the style of the strings in the heading to be \
different than the rest of the elements displayed by ",
  StyleBox["ShowTable", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[TextData[{
  "Modify ",
  StyleBox["ShowTable", "MR"],
  " so that it automatically partitions the list it is passed to be \
rectangular, with the number of rows and columns as close to each other as \
possible."
}], "ExerciseItem",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{

Cell[TextData[{
  "Create a function ",
  StyleBox["TruthTable[", "MR"],
  StyleBox["expr", "IT"],
  StyleBox[",", "MR"],
  StyleBox["vars", "IT"],
  StyleBox["]", "MR"],
  " that displays the logical expression ",
  StyleBox["expr",
    FontSlant->"Italic"],
  " together with all the possible truth values for the variables in the list \
",
  StyleBox["vars",
    FontSlant->"Italic"],
  ". For example, here is the truth table for the expression ",
  Cell[BoxData[
      \(TraditionalForm\`\((A \[Or] B)\) \[Implies] C\)]],
  "."
}], "ExerciseItem",
  CellTags->{"i:34", "b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(TruthTable[Implies[A || B, C], {A, B, C}]\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[BoxData[
    TagBox[GridBox[{
          {"A", "B", "C", 
            TagBox[
              FormBox[\(\((A \[Or] B)\) \[Implies] C\),
                "TraditionalForm"],
              TraditionalForm,
              Editable->True]},
          {"T", "T", "T", "T"},
          {"T", "T", "F", "F"},
          {"T", "F", "T", "T"},
          {"T", "F", "F", "F"},
          {"F", "T", "T", "T"},
          {"F", "T", "F", "F"},
          {"F", "F", "T", "T"},
          {"F", "F", "F", "T"}
          },
        GridFrame->True,
        RowLines->{2, False},
        ColumnLines->{False, False, 2}],
      DisplayForm]], "Output",
  CellLabel->"Out[1]//DisplayForm=",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]],

Cell[TextData[{
  "You will first need to create a list of all possible truth value \
assignments for the variables, ",
  StyleBox["A", "MR"],
  ", ",
  StyleBox["B", "MR"],
  ", ",
  StyleBox["C", "MR"],
  " in this case. One approach would be to use ",
  StyleBox["Distribute", "MR"],
  ". So, essentially, this is the left-hand side, or first three columns of \
the above table (not counting the first row containing the table headings)."
}], "ExercisesNoLabel",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{

Cell[BoxData[{
    \(\(vars = {A, B, C};\)\), "\[IndentingNewLine]", 
    \(len = Length[vars]; 
    ins = Distribute[Table[{True, \ False}, \ {len}], \ List, \ List, \ 
        List]\)}], "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[BoxData[
    \({{True, True, True}, {True, True, False}, {True, False, True}, {True, 
        False, False}, {False, True, True}, {False, True, False}, {False, 
        False, True}, {False, False, False}}\)], "Output",
  CellLabel->"Out[3]=",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]],

Cell["\<\
You can then create a list of rules associating each of these triples of \
truth values with a triple of variables.\
\>", "ExercisesNoLabel",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Map[Thread[vars \[Rule] #1] &, ins]\)], "Input",
  CellLabel->"In[4]:=",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[BoxData[
    \({{A \[Rule] True, B \[Rule] True, C \[Rule] True}, {A \[Rule] True, 
        B \[Rule] True, C \[Rule] False}, {A \[Rule] True, B \[Rule] False, 
        C \[Rule] True}, {A \[Rule] True, B \[Rule] False, 
        C \[Rule] False}, {A \[Rule] False, B \[Rule] True, 
        C \[Rule] True}, {A \[Rule] False, B \[Rule] True, 
        C \[Rule] False}, {A \[Rule] False, B \[Rule] False, 
        C \[Rule] True}, {A \[Rule] False, B \[Rule] False, 
        C \[Rule] False}}\)], "Output",
  CellLabel->"Out[4]=",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]],

Cell["\<\
Substituting these rules into the logical expression produces a truth value \
for each of the above rows.\
\>", "ExercisesNoLabel",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Implies[A || B, C] /. \[InvisibleSpace]Map[Thread[vars \[Rule] #1] &, 
        ins]\)], "Input",
  CellLabel->"In[5]:=",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[BoxData[
    \({True, False, True, False, True, False, True, True}\)], "Output",
  CellLabel->"Out[5]=",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]],

Cell[TextData[{
  "Your task is to put all these pieces together in a ",
  StyleBox["GridBox", "MR"],
  " with appropriate formatting."
}], "ExercisesNoLabel",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Buttons", "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
  CellTags->{
  "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", "IPM3.10.5.4", 
    "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],

Cell["\<\
Buttons are very user-friendly objects whose functionality is familiar to any \
computer user. From the programmer\[CloseCurlyQuote]s point of view, they \
allow you to hide your code behind a graphical element, the button. Instead \
of writing a function and evaluating it by pressing Shift-Enter from the \
keyboard, you pass the mouse cursor over the button and simply click. \
Whatever code is hidden underneath the button is then evaluated. \
\>", "TextFirst",
  CellTags->"IPM3.10.5.0"],

Cell[TextData[{
  "In this section we will first look at the structure of ",
  StyleBox["ButtonBox", "MR"],
  "es and then create some examples to demonstrate the variety of tasks that \
can be accomplished with buttons."
}], "Text",
  CellTags->"IPM3.10.5.0"],

Cell[CellGroupData[{

Cell["Making buttons the easy way", "Subsection",
  CellTags->{"b:46.0", "IPM3.10.5.1", "c:17"}],

Cell[TextData[{
  "The simplest way to create buttons is to select an expression in your ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " notebook, choose ",
  StyleBox["Create Button", "SR"],
  " from the ",
  StyleBox["Input", "SR"],
  " menu, and then activate your button. Let us walk through these steps to \
create a button that pastes an expression into your notebook."
}], "TextFirst",
  CellTags->{"i:35", "b:46.0", "IPM3.10.5.1"}],

Cell[TextData[{
  "Suppose you were writing a paper in which you are discussing sequences and \
you need to use an expression such as the following repeatedly in your \
notebook: ",
  Cell[BoxData[
      \(TraditionalForm\`{a\_1, a\_2, \[Ellipsis], a\_n}\)]],
  ". To create a button that would allow you to paste this expression into \
your notebook by simply clicking that button, we first write down the \
expression we will work with below in a regular input cell."
}], "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],

Cell[BoxData[
    \({a\_1, a\_2, \[Ellipsis], a\_n}\)], "Input",
  Evaluatable->False,
  CellTags->{"b:46.0", "IPM3.10.5.1"}],

Cell[TextData[{
  "Now select the entire expression and choose ",
  StyleBox["Create Button \[RightTriangle] Paste", "SR"],
  " from the ",
  StyleBox["Input", "SR"],
  " menu. "
}], "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],

Cell[BoxData[
    ButtonBox[\({a\_1, a\_2, \[Ellipsis], a\_n}\),
      ButtonStyle->"Paste"]], "Input",
  Evaluatable->False,
  CellTags->{"b:46.0", "IPM3.10.5.1"}],

Cell[TextData[{
  "Finally, to activate the button so that you can click it to have an action \
occur, select the cell in which the button occurs and then choose ",
  StyleBox["Cell Properties \[RightTriangle] Cell Active", "SR"],
  " from the ",
  StyleBox["Cell", "SR"],
  " menu."
}], "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],

Cell[BoxData[
    ButtonBox[\({a\_1, a\_2, \[Ellipsis], a\_n}\),
      ButtonStyle->"Paste"]], "Input",
  Active->True,
  Evaluatable->False,
  CellTags->{"b:46.0", "IPM3.10.5.1"}],

Cell[TextData[{
  "Clicking the above button will paste the following at the insertion point: \
",
  Cell[BoxData[
      \(TraditionalForm\`{a\_1, a\_2, \[Ellipsis], a\_n}\)]],
  "."
}], "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],

Cell[TextData[{
  "If you wished, you could create a free-standing palette from this button \
by choosing ",
  StyleBox["Generate Palette from Selection", "SR"],
  " from the ",
  StyleBox["File", "SR"],
  " menu."
}], "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],

Cell["\<\
Although the above procedure for creating buttons is quite straightforward, \
it is only convenient for fairly simple buttons. For more complicated buttons \
you will find that you need a good understanding of the structure of buttons \
and the various options that control their actions and display. We turn to \
those topics in the next few sections.\
\>", "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["The structure of buttons", "Subsection",
  CellTags->{"b:46.1", "IPM3.10.5.2", "c:18"}],

Cell[TextData[{
  "Buttons are created with the ",
  StyleBox["ButtonBox", "MR"],
  " function in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". ",
  StyleBox["ButtonBox", "MR"],
  " takes one argument and by default, that argument is pasted at the current \
selection point."
}], "TextFirst",
  CellTags->{"i:36", "b:46.1", "IPM3.10.5.2"}],

Cell[TextData[{
  "In the examples that follow, we will use ",
  StyleBox["DisplayForm", "MR"],
  " to display the button as an interactive element. If you were to unformat \
your button (",
  StyleBox["Show Expression", "SR"],
  " from the ",
  StyleBox["Format", "SR"],
  " menu), you would see essentially all that precedes the ",
  StyleBox["DisplayForm", "MR"],
  " below."
}], "Text",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ButtonBox["\<some text\>", Active \[Rule] True] // 
      DisplayForm\)], "Input",
  CellLabel->"In[1]:=",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell[BoxData[
    TagBox[
      ButtonBox[\(some\ text\),
        Active->True],
      DisplayForm]], "Output",
  CellLabel->"Out[1]//DisplayForm=",
  CellTags->{"b:46.1", "IPM3.10.5.2"}]
}, Open  ]],

Cell[TextData[{
  "Note that we have added the option ",
  StyleBox["Active\[Rule]True", "MR"],
  ". This makes the resulting button uneditable, one that is clickable. You \
will need to add this option to all your buttons to activate them. Clicking \
this button causes the following to be pasted at the current selection \
point."
}], "Text",
  CellTags->{"i:37", "b:46.1", "IPM3.10.5.2"}],

Cell[BoxData[
    \(some\ text\)], "Input",
  Evaluatable->False,
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell["\<\
Let us create a button that can serve as a template for a definite \
integral.\
\>", "Text",
  CellTags->{"i:38", "b:46.1", "IPM3.10.5.2"}],

Cell[CellGroupData[{

Cell["\<\
ButtonBox[\"Integrate[fun,{x,xmin,xmax}]\", Active->True]  //DisplayForm\
\>", "Input",
  CellLabel->"In[2]:=",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell[BoxData[
    TagBox[
      ButtonBox[\(Integrate[fun, {x, xmin, xmax}]\),
        Active->True],
      DisplayForm]], "Output",
  CellLabel->"Out[2]//DisplayForm=",
  CellTags->{"b:46.1", "IPM3.10.5.2"}]
}, Open  ]],

Cell["Clicking the button causes the following to be pasted in.", "Text",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell[BoxData[
    \(Integrate[fun, {x, xmin, xmax}]\)], "Input",
  Evaluatable->False,
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell[TextData[{
  "We can use placeholders in our template button so that the user can move \
from one placeholder to the next by pressing the Tab key. The placeholder \
character \[Placeholder] can be entered either from the Complete Characters \
palette (look under Letter-like Forms and then Keyboard Forms), or directly \
from the keyboard by typing \[EscapeKey]-",
  StyleBox["sp", "MR"],
  "-\[EscapeKey] (pressing the Escape key, then the characters ",
  StyleBox["s", "MR"],
  " and then ",
  StyleBox["p", "MR"],
  ", and finally the closing Escape key)."
}], "Text",
  CellTags->{"i:39", "b:46.1", "IPM3.10.5.2"}],

Cell[CellGroupData[{

Cell["\<\
ButtonBox[\"Integrate[\[Placeholder],{\[Placeholder],\[Placeholder],\
\[Placeholder]}]\", Active->True]  //DisplayForm\
\>", "Input",
  CellLabel->"In[3]:=",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell[BoxData[
    TagBox[
      ButtonBox[\(Integrate[\[Placeholder], {\[Placeholder], \[Placeholder], \
\[Placeholder]}]\),
        Active->True],
      DisplayForm]], "Output",
  CellLabel->"Out[3]//DisplayForm=",
  CellTags->{"b:46.1", "IPM3.10.5.2"}]
}, Open  ]],

Cell["\<\
Clicking on this button causes the following expression to be pasted. You can \
move from one placeholder to another by pressing the Tab key.\
\>", "Text",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell[BoxData[
    \(Integrate[\[Placeholder], {\[Placeholder], \[Placeholder], \
\[Placeholder]}]\)], "Input",
  Evaluatable->False,
  CellTags->{"b:46.1", "IPM3.10.5.2"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["ButtonStyle", "Subsection",
  CellTags->{"b:46.2", "IPM3.10.5.3", "c:19"}],

Cell[TextData[{
  "Although having buttons that paste their contents at the current selection \
point is useful, there is much more that buttons can do. For example, they \
can wrap the contents of the ",
  StyleBox["ButtonBox", "MR"],
  " around a selected expression and then evaluate that expression. To change \
the default behavior of buttons from simply pasting their contents to other \
actions, we have to use the ",
  StyleBox["ButtonStyle", "MR"],
  " option. ",
  StyleBox["ButtonStyle", "MR"],
  " is used to control both the style and the actions associated with your \
buttons. In the following example, ",
  StyleBox["ButtonStyle", "MR"],
  " is set to ",
  StyleBox["CopyEvaluateCell", "MR"],
  ". "
}], "TextFirst",
  CellTags->{"i:40", "b:46.2", "IPM3.10.5.3"}],

Cell[BoxData[
    \(ButtonBox["\<Integrate[\[SelectionPlaceholder],x]\>", 
        Active \[Rule] True, ButtonStyle \[Rule] "\<CopyEvaluateCell\>"] // 
      DisplayForm\)], "Input",
  CellLabel->"In[4]:=",
  Evaluatable->False,
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[TextData[{
  "The \[SelectionPlaceholder] character is entered either from palettes or \
directly from the keyboard by typing \[EscapeKey]-",
  StyleBox["spl", "MR"],
  "-\[EscapeKey]. Evaluating the above input produces the cell below. \
Selecting the input cell containing ",
  Cell[BoxData[
      FormBox[
        StyleBox[\(Cos[x\^2] + x\^5\),
          "MR"], TraditionalForm]]],
  " and then clicking the button causes the template to be wrapped around the \
selected expression and then it is evaluated."
}], "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[BoxData[
    TagBox[
      ButtonBox[\(Integrate[\[SelectionPlaceholder], x]\),
        Active->True,
        ButtonStyle->"CopyEvaluateCell"],
      DisplayForm]], "Output",
  CellLabel->"Out[4]//DisplayForm=",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[BoxData[
    \(Cos[x\^2] + x\^5\)], "Input",
  CellLabel->"In[5]:=",
  Evaluatable->False,
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(Integrate[Cos[x\^2] + x\^5, x]\)], "Input",
  CellLabel->"In[6]:=",
  Evaluatable->False,
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[BoxData[
    \(x\^6\/6 + \@\(\[Pi]\/2\)\ FresnelC[\@\(2\/\[Pi]\)\ x]\)], "Output",
  CellLabel->"Out[6]=",
  CellTags->{"b:46.2", "IPM3.10.5.3"}]
}, Open  ]],

Cell[TextData[{
  "If you were to use ",
  StyleBox["ButtonStyle\[Rule]EvaluateCell", "MR"],
  " instead of ",
  StyleBox["CopyEvaluateCell", "MR"],
  ", the button action would erase the selection and replace it with the new \
input and the result."
}], "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[TextData[{
  "Another very useful ",
  StyleBox["ButtonStyle", "MR"],
  " is ",
  StyleBox["Hyperlink", "MR"],
  ". Making a hyperlink is accomplished by creating a button out of some \
expression and setting the ",
  StyleBox["ButtonStyle", "MR"],
  " option to ",
  StyleBox["Hyperlink", "MR"],
  " and adding the ",
  StyleBox["ButtonData", "MR"],
  " option."
}], "Text",
  CellTags->{"i:41", "b:46.2", "IPM3.10.5.3"}],

Cell["\<\
Cell[TextData[{
  \"Search for button on \",
  ButtonBox[\"Google\",
    ButtonData:>{
      URL[ \"http://www.google.com\"], None},
    ButtonStyle->\"Hyperlink\"]
}], \"Text\"]\
\>", "Program",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell["The formatted version of this cell looks like this:", "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[TextData[{
  "\tSearch for button on ",
  StyleBox[ButtonBox["Google",
    ButtonData:>{
      URL[ "http://www.google.com"], None},
    ButtonStyle->"Hyperlink"],
    FontVariations->{"Underline"->True}]
}], "Text",
  CellMargins->{{Inherited, Inherited}, {6, 6}},
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[TextData[{
  "Setting ",
  StyleBox["ButtonStyle", "MR"],
  " to ",
  StyleBox["Hyperlink", "MR"],
  " sets the button action to jump to some location. That location is \
specified as the value of the option ",
  StyleBox["ButtonData", "MR"],
  ". In this example, that is set to ",
  StyleBox["URL[\"http://www.google.com\"]", "MR"],
  ". ",
  StyleBox["ButtonData", "MR"],
  " set to a URL will cause your web browser to be launched and opened to the \
location given as the argument to URL \[Dash] in this case \
http://www.google.com."
}], "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[TextData[{
  "A list of all the possible ",
  StyleBox["ButtonStyle", "MR"],
  " values is displayed in Table 10.1."
}], "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[BoxData[
    FormBox[GridBox[{
          {
            StyleBox[\(ButtonStyle\ values\),
              "TB"], 
            StyleBox["Action",
              "TB"]},
          {
            StyleBox["Paste",
              "MR"], 
            StyleBox[\(pastes\ the\ contents\ \((default)\)\),
              "TR"]},
          {
            StyleBox["Evaluate",
              "MR"], 
            StyleBox[\(pastes, \ then\ evaluates\ in\ place\),
              "TR"]},
          {
            StyleBox["EvaluateCell",
              "MR"], 
            StyleBox[\(paste, \ then\ evaluate\ entire\ cell\),
              "TR"]},
          {
            StyleBox["CopyEvaluate",
              "MR"], 
            
            StyleBox[\(copy\ current\ selection\ into\ new\ cell, \
\[IndentingNewLine]then\ paste\ and\ evaluate\),
              "TR"]},
          {
            StyleBox["CopyEvaluateCell",
              "MR"], 
            
            StyleBox[\(copy\ current\ selection\ into\ new\ cell, \
\[IndentingNewLine]then\ paste\ and\ evaluate\ cell\),
              "TR"]},
          {
            StyleBox["Hyperlink",
              "MR"], 
            StyleBox[\(jump\ to\ different\ location\),
              "TR"]}
          },
        RowSpacings->1.2,
        ColumnAlignments->{Left},
        GridFrame->True,
        GridFrameMargins->True,
        RowLines->True,
        ColumnLines->True], DefaultTraditionalFormatType]], "DisplayTable",
  LineSpacing->{0.8, 5},
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[TextData[{
  "Possible ",
  StyleBox["ButtonStyle", "MR"],
  "s and associated actions"
}], "NumberedTable",
  CellTags->{"b:46.2", "IPM3.10.5.3"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["ButtonFunction", "Subsection",
  CellTags->{"b:46.3", "IPM3.10.5.4", "c:20"}],

Cell[TextData[{
  "Whenever you need to put some ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " code inside your button, you will need to do so as the value of the \
option ",
  StyleBox["ButtonFunction", "MR"],
  ". You will also need to explicitly set the option ",
  StyleBox["ButtonEvaluator", "MR"],
  " which is set to ",
  StyleBox["None", "MR"],
  " by default. The ",
  StyleBox["ButtonEvaluator", "MR"],
  " option tells the front end what program it should communicate with to \
process the contents of the button function. Setting it to ",
  StyleBox["None", "MR"],
  " tells the front end to communicate with itself which is fine for \
operations like copying and pasting. But for operations that need to \
communicate with a kernel, you will have to specify that explicitly. A value \
of ",
  StyleBox["Automatic", "MR"],
  " sends the code to the default kernel for the current notebook. If you had \
other kernels set up, you could direct the button function at one of those."
}], "TextFirst",
  CellTags->{"i:42", "b:46.3", "IPM3.10.5.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ButtonBox["\<Compute 5!\>", \[IndentingNewLine]Active \[Rule] 
          True, \[IndentingNewLine]ButtonFunction \[RuleDelayed] 
          Factorial[5], \[IndentingNewLine]ButtonEvaluator \[Rule] 
          Automatic] // DisplayForm\)], "Input",
  CellLabel->"In[7]:=",
  Evaluatable->False,
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[BoxData[
    TagBox[
      ButtonBox[\(Compute\ \(5!\)\),
        ButtonFunction:>Factorial[ 5],
        ButtonEvaluator->Automatic,
        Active->True],
      DisplayForm]], "Output",
  CellLabel->"Out[7]//DisplayForm=",
  CellTags->{"b:46.3", "IPM3.10.5.4"}]
}, Open  ]],

Cell[TextData[{
  "Clicking this button will not cause any output to be displayed. This is \
because these buttons are not evaluated in the kernel in the usual way as \
part of the main loop. In this case, you can use ",
  StyleBox["Print", "MR"],
  " to see the side effect of this computation."
}], "Text",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(ButtonBox["\<Compute 5!\>", \[IndentingNewLine]Active \[Rule] 
          True, \[IndentingNewLine]ButtonFunction \[RuleDelayed] 
          Print[Factorial[5]], \[IndentingNewLine]ButtonEvaluator \[Rule] 
          Automatic] // DisplayForm\)], "Input",
  CellLabel->"In[8]:=",
  Evaluatable->False,
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[BoxData[
    TagBox[
      ButtonBox[\(Compute\ \(5!\)\),
        ButtonFunction:>Print[ 
            Factorial[ 5]],
        ButtonEvaluator->Automatic,
        Active->True],
      DisplayForm]], "Output",
  CellLabel->"Out[8]//DisplayForm=",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[BoxData[
    \(120\)], "Print",
  CellLabel->"From In[1]:=",
  CellTags->{"b:46.3", "IPM3.10.5.4"}]
}, Open  ]],

Cell[TextData[{
  "You can use any ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " function you wish as the value of the ",
  StyleBox["ButtonFunction", "MR"],
  " option. But, in addition to the above issue with displaying output, you \
should be aware of another important issue. As it turns out, the front end \
does not know how to parse the special shorthand notation we often use for \
arithmetic and other operations. You will be forced to use the ",
  StyleBox["FullForm", "MR"],
  " of such expressions inside of your ",
  StyleBox["ButtonFunction", "MR"],
  ". So instead of ",
  StyleBox["2+2", "MR"],
  ", use ",
  StyleBox["Plus[2,2]", "MR"],
  "; instead of ",
  StyleBox["{<<Graphics`;\[ThinSpace]LogPlot[Exp[x],{x,1,2}]}", "MR"],
  " use ",
  StyleBox["CompoundExpression[Get[\"Graphics`\",\[ThinSpace]LogPlot[Exp[x],{\
x,1,2}]]", "MR"],
  ". Fortunately, the parser for the front end can recognize the shorthand \
notation for ",
  StyleBox["List", "MR"],
  ", ",
  StyleBox["Rule", "MR"],
  ", and ",
  StyleBox["RuleDelayed", "MR"],
  ", so you can use the shorthand notations ",
  StyleBox["{}", "MR"],
  ", ",
  StyleBox["\[Rule]", "MR"],
  ", and ",
  StyleBox["\[RuleDelayed]", "MR"],
  ", respectively."
}], "Text",
  CellTags->{"i:43", "b:46.3", "IPM3.10.5.4"}],

Cell["\<\
As a final example, we will create a button that loads a package and then \
performs a computation with some functions from that package. Here is the \
code that we want to encapsulate in our button.\
\>", "Text",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[BoxData[
    \(Needs["\<Graphics`Polyhedra`\>"]\)], "Input",
  CellLabel->"In[9]:=",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(Show[Graphics3D[Stellate[Icosahedron[]]]];\)\)], "Input",
  CellLabel->"In[10]:=",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.00162 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
-0.0206914 1.08755 -7.41594e-017 1.08755 [
[ 0 0 0 0 ]
[ 1 1.00162 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.38643 1.00162 L
s
.38643 1.00162 m
.40217 .57872 L
s
.40217 .57872 m
.07192 .25073 L
s
.68538 0 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.68538 0 m
.07192 .25073 L
s
.40217 .57872 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.38643 1.00162 L
s
.38643 1.00162 m
.40217 .57872 L
s
0 0 m
1 0 L
1 1.00162 L
0 1.00162 L
closepath
clip
newpath
.5 Mabswid
.786 .442 .405 r
.44491 .44907 m .36477 .52184 L .55473 .57484 L closepath p F P
0 g
s
.37 .395 .757 r
.44491 .44907 m .55473 .57484 L .52308 .38117 L closepath p F P
0 g
s
0 0 0 r
.44491 .44907 m .52308 .38117 L .36477 .52184 L closepath p F P
0 g
s
.366 .042 .277 r
.39521 .65616 m .44391 .71565 L .55473 .57484 L closepath p F P
0 g
s
.702 .725 .868 r
.39521 .65616 m .55473 .57484 L .36477 .52184 L closepath p F P
0 g
s
0 0 .535 r
.39521 .65616 m .36477 .52184 L .44391 .71565 L closepath p F P
0 g
s
.299 .033 .331 r
.65615 .42467 m .55473 .57484 L .70544 .48388 L closepath p F P
0 g
s
.859 .738 .729 r
.65615 .42467 m .52308 .38117 L .55473 .57484 L closepath p F P
0 g
s
.641 .082 0 r
.65615 .42467 m .70544 .48388 L .52308 .38117 L closepath p F P
0 g
s
.873 .707 .674 r
.57791 .76621 m .55473 .57484 L .44391 .71565 L closepath p F P
0 g
s
.311 .298 .683 r
.57791 .76621 m .65788 .69455 L .55473 .57484 L closepath p F P
0 g
s
.6 .813 .982 r
.57791 .76621 m .44391 .71565 L .65788 .69455 L closepath p F P
0 g
s
.721 .348 .342 r
.74263 .62049 m .55473 .57484 L .65788 .69455 L closepath p F P
0 g
s
.643 .687 .876 r
.74263 .62049 m .70544 .48388 L .55473 .57484 L closepath p F P
0 g
s
.971 .845 .659 r
.74263 .62049 m .65788 .69455 L .70544 .48388 L closepath p F P
0 g
s
.698 .565 .701 r
.33115 .32429 m .36477 .52184 L .52308 .38117 L closepath p F P
0 g
s
0 0 .52 r
.33115 .32429 m .38251 .38653 L .36477 .52184 L closepath p F P
0 g
s
.608 .076 0 r
.33115 .32429 m .52308 .38117 L .38251 .38653 L closepath p F P
0 g
s
.706 .538 .659 r
.24235 .68284 m .44391 .71565 L .36477 .52184 L closepath p F P
0 g
s
0 0 0 r
.24235 .68284 m .36477 .52184 L .32852 .60768 L closepath p F P
0 g
s
.817 .41 .277 r
.19481 .46702 m .32852 .60768 L .36477 .52184 L closepath p F P
0 g
s
.712 .79 .912 r
.19481 .46702 m .36477 .52184 L .38251 .38653 L closepath p F P
0 g
s
.654 .533 .704 r
.69731 .27878 m .52308 .38117 L .70544 .48388 L closepath p F P
0 g
s
0 0 0 r
.69731 .27878 m .60797 .35949 L .52308 .38117 L closepath p F P
0 g
s
.217 .339 .774 r
.48589 .20782 m .52308 .38117 L .60797 .35949 L closepath p F P
0 g
s
.899 .805 .747 r
.48589 .20782 m .38251 .38653 L .52308 .38117 L closepath p F P
0 g
s
.559 .729 .954 r
.24235 .68284 m .32852 .60768 L .44391 .71565 L closepath p F P
0 g
s
.927 .746 .637 r
.33664 .82736 m .44391 .71565 L .32852 .60768 L closepath p F P
0 g
s
.666 .487 .633 r
.56015 .87859 m .65788 .69455 L .44391 .71565 L closepath p F P
0 g
s
.079 .258 .746 r
.56015 .87859 m .44391 .71565 L .52308 .72469 L closepath p F P
0 g
s
.145 0 0 r
.33664 .82736 m .52308 .72469 L .44391 .71565 L closepath p F P
0 g
s
.952 .764 .613 r
.69731 .27878 m .70544 .48388 L .60797 .35949 L closepath p F P
0 g
s
.633 .483 .662 r
.85199 .62068 m .70544 .48388 L .65788 .69455 L closepath p F P
0 g
s
.804 .349 .159 r
.85199 .62068 m .69976 .56825 L .70544 .48388 L closepath p F P
0 g
s
0 0 .084 r
.82611 .39367 m .70544 .48388 L .69976 .56825 L closepath p F P
0 g
s
.593 .717 .93 r
.82611 .39367 m .60797 .35949 L .70544 .48388 L closepath p F P
0 g
s
.702 .816 .936 r
.85199 .62068 m .65788 .69455 L .69976 .56825 L closepath p F P
0 g
s
.92 .834 .744 r
.56015 .87859 m .52308 .72469 L .65788 .69455 L closepath p F P
0 g
s
.67 .196 .118 r
.73868 .79032 m .65788 .69455 L .52308 .72469 L closepath p F P
0 g
s
.065 .115 .604 r
.73868 .79032 m .69976 .56825 L .65788 .69455 L closepath p F P
0 g
s
.624 .472 .656 r
.19481 .46702 m .38251 .38653 L .32852 .60768 L closepath p F P
0 g
s
.661 .476 .624 r
.48589 .20782 m .60797 .35949 L .38251 .38653 L closepath p F P
0 g
s
.635 .791 .957 r
.45664 .27423 m .38251 .38653 L .60797 .35949 L closepath p F P
0 g
s
.22 .231 .659 r
.45664 .27423 m .48535 .50673 L .38251 .38653 L closepath p F P
0 g
s
.947 .817 .683 r
.25792 .45112 m .32852 .60768 L .38251 .38653 L closepath p F P
0 g
s
.707 .293 .259 r
.25792 .45112 m .38251 .38653 L .48535 .50673 L closepath p F P
0 g
s
.628 .699 .896 r
.25792 .45112 m .48535 .50673 L .32852 .60768 L closepath p F P
0 g
s
.648 .528 .703 r
.33664 .82736 m .32852 .60768 L .52308 .72469 L closepath p F P
0 g
s
.722 .218 .033 r
.35521 .69402 m .52308 .72469 L .32852 .60768 L closepath p F P
0 g
s
.184 0 .222 r
.35521 .69402 m .32852 .60768 L .48535 .50673 L closepath p F P
0 g
s
.705 .534 .654 r
.82611 .39367 m .69976 .56825 L .60797 .35949 L closepath p F P
0 g
s
.892 .722 .664 r
.45664 .27423 m .60797 .35949 L .48535 .50673 L closepath p F P
0 g
s
0 .115 .646 r
.68527 .40123 m .60797 .35949 L .69976 .56825 L closepath p F P
0 g
s
.266 0 .159 r
.68527 .40123 m .48535 .50673 L .60797 .35949 L closepath p F P
0 g
s
.697 .564 .701 r
.73868 .79032 m .52308 .72469 L .69976 .56825 L closepath p F P
0 g
s
0 0 0 r
.62474 .66693 m .69976 .56825 L .52308 .72469 L closepath p F P
0 g
s
.296 .351 .75 r
.62474 .66693 m .52308 .72469 L .48535 .50673 L closepath p F P
0 g
s
.876 .757 .728 r
.35521 .69402 m .48535 .50673 L .52308 .72469 L closepath p F P
0 g
s
.697 .742 .886 r
.68527 .40123 m .69976 .56825 L .48535 .50673 L closepath p F P
0 g
s
.786 .409 .341 r
.62474 .66693 m .48535 .50673 L .69976 .56825 L closepath p F P
0 g
s
.25 Mabswid
.68538 0 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.68538 0 m
.07192 .25073 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[10]:=",
  ImageSize->{199.5, 199.75},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000370000aa000`40O003h00OQgoo0P00?Woo0025
Ool200000goo0000000mOol0089oo`<000=oo`8003eoo`00P7oo0P001Woo00<007oo0000?7oo001n
Ool20008Ool00`00Ool0000lOol007]oo`<000Yoo`04001oogoo000kOol007Uoo`8000eoo`05001o
ogooOol0000jOol007Ioo`<000moo`05001oogooOol0000jOol007Aoo`80019oo`03001oogoo009o
o`03001oogoo03Moo`00LGoo0`0057oo00<007ooOol00goo00<007ooOol0=Woo001_Ool2000HOol0
0`00Oomoo`02Ool00`00Oomoo`0fOol006aoo`<001Yoo`03001oogoo00=oo`03001oogoo03Eoo`00
JWoo0P007Goo00<007ooOol00goo00<007ooOol0=Goo001XOol2000OOol00`00Oomoo`04Ool00`00
Oomoo`0dOol006Eoo`<0025oo`03001oogoo00Eoo`03001oogoo03=oo`00Hgoo0P0097oo00<007oo
Ool01Goo00<007ooOol0<goo001POol3000VOol00`00Oomoo`06Ool00`00Oomoo`0bOol005ioo`80
02Uoo`03001oogoo00Moo`03001oogoo035oo`00Fgoo0`00:goo00<007ooOol01goo00<007ooOol0
<Goo001IOol2000^Ool00`00Oomoo`08Ool00`00Oomoo`0`Ool005Ioo`<0031oo`03001oogoo00Qo
o`03001oogoo031oo`00E7oo0P00<goo00<007ooOol02Goo00<007ooOol0;goo001BOol2000eOol0
0`00Oomoo`0:Ool00`00Oomoo`0^Ool004moo`<003Moo`03001oogoo00Yoo`03001oogoo02ioo`00
CGoo0P00>Woo00<007ooOol02goo00<007ooOol0;Goo001:Ool3000lOol00`00Oomoo`0<Ool00`00
Oomoo`0/Ool004Qoo`8003moo`03001oogoo00aoo`03001oogoo02aoo`00AGoo0`00@Goo00<007oo
Ool03Goo00<007ooOol0:goo0013Ool20014Ool00`00Oomoo`0=Ool00`00Oomoo`0[Ool0045oo`80
04Ioo`03001oogoo00ioo`03001oogoo02Yoo`00?Woo0`00BGoo00<007ooOol03Woo00<007ooOol0
:Goo000lOol2001<Ool00`00Oomoo`0>Ool00`00Oomoo`0YOol003Uoo`<004ioo`03001oogoo00mo
o`03001oogoo02Qoo`00=goo0P00DGoo00<007ooOol047oo00<007ooOol09goo000dOol3001COol0
0`00Oomoo`0@Ool00`00Oomoo`0WOol0039oo`8005Ioo`03001oogoo015oo`03001oogoo02Ioo`00
;goo0`00F7oo00<007ooOol04Goo00<007ooOol09Woo000]Ool2001KOol00`00Oomoo`0BOol00`00
Oomoo`0UOol002]oo`8005eoo`03001oogoo01=oo`03001oogoo02Aoo`00:7oo0`00Ggoo00<007oo
Ool04goo00<007ooOol097oo000VOol2001ROol00`00Oomoo`0DOol00`00Oomoo`0SOol002=oo`<0
03Yoo`03001oogoo02Moo`03001oogoo01Eoo`03001oogoo029oo`008Goo0P00?7oo00<005Gc0000
:7oo00<007ooOol05Goo00<007ooOol08Woo000NOol3000nOol01000EO=El`009goo00<007ooOol0
5Woo00<007ooOol08Goo000LOol20010Ool01@00EO=EleGc00009goo00<007ooOol05Woo00<007oo
Ool08Goo000IOol30012Ool00`00EO=El`02EO<00`00Oomoo`0TOol00`00Oomoo`0GOol00`00Oomo
o`0POol001Moo`8004Aoo`03001EleGc00AEl`03001oogoo02=oo`03001oogoo01Qoo`03001oogoo
01moo`005Goo0P00AGoo00<005GcEO<01UGc00<007ooOol08Woo00<007ooOol067oo00<007ooOol0
7goo000BOol30017Ool00`00EO=El`07EO<00`00Oomoo`0QOol00`00Oomoo`0IOol00`00Oomoo`0N
Ool0011oo`8004Uoo`03001EleGc00QEl`03001oogoo029oo`03001oogoo01Uoo`03001oogoo01eo
o`003Woo0P00Bgoo00<005GcEO<02EGc00<007ooOol08Goo00<007ooOol06Goo00<007ooOol07Goo
000>Ool2001:Ool00`00EO=El`0;EO<00`00Oomoo`0POol00`00Oomoo`0JOol00`00Oomoo`0LOol0
00ioo`03001oo`0004Qoo`03001EleGc00eEl`03001oogoo01moo`03001oogoo01Yoo`03001oogoo
01aoo`003Woo00@007ooOol004Moo`03001EleGc00iEl`03001oogoo01ioo`03001oogoo01]oo`03
001oogoo01]oo`003Goo00<007ooOol00Woo00<007ooOol0@goo00<005Gc00003eGc00<007ooOol0
7Woo00<007ooOol077oo00<007ooOol06Woo000=Ool00`00Oomoo`03Ool00`00Oomoo`12Ool5000>
EO<00`00Oomoo`0LOol2000NOol00`00Oomoo`0JOol000eoo`03001oogoo00Aoo`03001oogoo041o
o`04001El`000002L_D2000=EO<00`00Oomoo`0IOol4000OOol00`00Oomoo`0IOol000eoo`03001o
ogoo00Eoo`03001oogoo03ioo`05001El`007?D00004L_D2000<EO<00`00Oomoo`0FOol300000g/C
0000000POol00`00Oomoo`0HOol000eoo`03001oogoo00Ioo`03001oogoo03eoo`8000<Lm@03001b
mG;e00=bm@03001EleGc00YEl`03001oogoo01=oo`@0009k4`80021oo`03001oogoo01Qoo`003Goo
00<007ooOol01goo00<007ooOol0>goo00<005Gc00000ace00<007;eL_D017;e0P002UGc00<007oo
Ool04Woo10000g/C0P008Goo00<007ooOol05goo000<Ool00`00Oomoo`09Ool00`00Oomoo`0jOol2
00047?D00`00L_Ebm@06L_D20009EO<00`00Oomoo`0?Ool40005Na<2000QOol00`00Oomoo`0GOol0
00aoo`03001oogoo00Yoo`03001oogoo03Qoo`8000DLm@03001bmG;e00Qbm@8000QEl`03001oogoo
00aoo`D000Ik4`80029oo`03001oogoo01Ioo`0037oo00<007ooOol02goo00<007ooOol0=Woo00<0
05Gc00001Ace00<007;eL_D02W;e00<005GcEO<01UGc00<007ooOol02Goo1P001g/C0P008goo00<0
07ooOol05Goo000<Ool00`00Oomoo`0<Ool00`00Oomoo`0eOol200067?D00`00L_Ebm@0;L_D20007
EO<00`00Oomoo`07Ool60008Na<2000SOol00`00Oomoo`0EOol000aoo`03001oogoo00eoo`03001o
ogoo029oo`8000moo`8000LLm@03001bmG;e00ebm@8000EEl`03001oogoo00Eoo`L000Uk4`8002Ao
o`03001oogoo01Aoo`0037oo00<007ooOol03Woo00<007ooOol08Goo1P002goo00<001ce7?D01Qce
00<007;eL_D03g;e0P0015Gc00<007ooOol00Woo20002W/C0P009Goo00<007ooOol04goo000<Ool0
0`00Oomoo`0?Ool00`00Oomoo`0POol00`0001000003C4030007Ool200097?D00`00L_Ebm@0@L_D2
0003EO<00`00Ool00008000<Na<00`00Oomoo`0SOol00`00Oomoo`0COol000]oo`03001oogoo015o
o`03001oogoo021oo`8000I<@0@0009oo`8000XLm@03001bmG;e019bm@04001EleGcEO<9000=Na<0
0`00Oomoo`0TOol00`00Oomoo`0BOol000]oo`03001oogoo019oo`03001oogoo01moo`0300004000
00U<@0<000/Lm@03001bmG;e01=bm@80009El`H000mk4`8002Eoo`03001oogoo019oo`002goo00<0
07ooOol04goo00<007ooOol07Woo00@0000@010000U<@08000/Lm@03001bmG;e01Ebm@L0011k4`80
02Ioo`03001oogoo015oo`002goo00<007ooOol057oo00<007ooOol07Goo00D0000@0100400000Q<
@003000LmAce00XLm@03001bmG;e01Mbm@@0015k4`8002Moo`03001oogoo011oo`002goo00<007oo
Ool05Goo00<007ooOol077oo00<0000@01000P0@00<004a0C40014a000<001ce7?D02ace00<007;e
L_D06G;e10003g/C0P009goo00<007ooOol047oo000;Ool00`00Oomoo`0FOol00`00Oomoo`0KOol0
0`000100400301000`00C41<@002C402000=7?D00`00L_Ebm@0HL_D01@0080D00000FS@01P002G/C
0P00:7oo00<007ooOol03goo000:Ool00`00Oomoo`0HOol00`00Oomoo`0KOol00`00010040020100
0`00C41<@002C4000`007?DLm@0<7?D00`00L_Ebm@0GL_D00`0080DP1@0280D20005FS@60003Na<2
000YOol00`00Oomoo`0>Ool000Yoo`03001oogoo01Uoo`03001oogoo01Yoo`030000400@00<04004
001<@4a0000@7?D00`00L_Ebm@0FL_D00`0080DP1@0480D20009FS@6000XOol00`00Oomoo`0>Ool0
00Yoo`03001oogoo01Yoo`03001oogoo01Uoo`030000400@00@04003001<@000010Lm@03001bmG;e
01Ebm@03000P1B0500LP1@8000]J=003001J=00000D002=oo`03001oogoo00eoo`002Woo00<007oo
Ool06goo00<007ooOol067oo00<0000@0100100@0P004Ace00<007;eL_D057;e00<0020580D02R05
0P002EXd00<005XdFS@01EXd1P007Goo00<007ooOol03Goo000:Ool00`00Oomoo`0LOol00`00Oomo
o`0GOol00`0001004002010300000eTX000Lm@0?7?D00`00L_Ebm@0CL_D00`0080DP1@0=80D20007
FS@00`00FSAJ=00;FS@6000HOol00`00Oomoo`0<Ool000Yoo`03001oogoo01eoo`03001oogoo01Mo
o`0300004000008000AI:003000LmAce00dLm@03001bmG;e019bm@03000P1B05010P1@8000EJ=003
001J=5Xd015J=0@001Eoo`03001oogoo00]oo`002Woo00<007ooOol07Woo00<007ooOol05Goo1000
1eTX00<001ce7?D031ce00<007;eL_D04G;e00<0020580D04b050P000eXd00<005XdFS@04eXd00<0
07ooOol057oo00<007ooOol02goo0009Ool00`00Oomoo`0POol00`00Oomoo`0BOol40009FBP00`00
7?DLm@0<7?D00`00L_Ebm@0AL_D00`0080DP1@0B80D30003FS@00`00FSAJ=00CFS@00`00Oomoo`0D
Ool2000<Ool000Uoo`03001oogoo025oo`03001oogoo00moo`@000aI:003000LmAce00`Lm@03001b
mG;e00mbm@03000P1B05014P1@80009Jo005001J=5XdFS@0000DFS@00`00Oomoo`0BOol300000goo
001oo`0;Ool000Uoo`03001oogoo029oo`03001oogoo00]oo`<00003CO@005TX00iI:003000LmAce
00/Lm@03001bmG;e00ibm@03000P1B05010P1@8000AJo005001J=5XdFS@0000CFS@00`00Oomoo`0@
Ool30004Ool00`00Oomoo`0:Ool000Uoo`03001oogoo02=oo`03001oogoo00Qoo`80009=m080015I
:003000LmAce00XLm@03001bmG;e00ebm@03000P1B0500lP1@8000IJo005001J=5XdFS@0000BFS@0
0`00Oomoo`0>Ool30007Ool00`00Oomoo`0:Ool000Uoo`03001oogoo02Aoo`03001oogoo00Eoo`80
009=m08001AI:003000LmAce00TLm@03001bmG;e00abm@03000P1B0500hP1@8000QJo005001J=5Xd
FS@0000BFS@00`00Oomoo`0;Ool3000;Ool00`00Oomoo`09Ool000Uoo`03001oogoo02Eoo`03001o
ogoo009oo`80009=m08001MI:003000LmAce00PLm@03001bmG;e00]bm@03000P1B0500dP1@8000YJ
o005001J=5XdFS@0000AFS@00`00Oomoo`09Ool3000>Ool00`00Oomoo`09Ool000Uoo`03001oogoo
02Ioo`@0009=m08001YI:003000LmAce00LLm@03001bmG;e00]bm@03000P1B0500/P1@8000aJo005
001J=5XdFS@0000@FS@00`00Oomoo`07Ool3000AOol00`00Oomoo`09Ool000Qoo`03001oogoo02Io
o`8000==m08001eI:003000LmAce00HLm@03001bmG;e00Ybm@03000P1B0500XP1@8000iJo005001J
=5XdFS@0000@FS@00`00Oomoo`04Ool3000DOol00`00Oomoo`09Ool000Qoo`03001oogoo02Aoo`80
00==m08001mI:003000LmAce00HLm@03001bmG;e00Ubm@03000P1B0500TP1@80015Jo005001J=5Xd
FS@0000>FS@00`00Oomoo`02Ool3000GOol00`00Oomoo`09Ool000Qoo`03001oogoo025oo`<000A=
m0<0021I:003000LmAce00HLm@03001bmG;e00Mbm@03000P1B0500TP1@03001Jo5[l015Jo005001J
=5XdFS@0000=FS@00`00Ool00003000JOol00`00Oomoo`09Ool000Qoo`03001oogoo01moo`8000M=
m003001Bg5;L00@001eI:003000LmAce00DLm@03001bmG;e00Ibm@03000P1B0500PP1@8001AJo005
001J=5XdFS@0000<FS@3000NOol00`00Oomoo`09Ool000Qoo`03001oogoo01eoo`8000Y=m003001B
g5;L00=Bg0@001YI:003000LmAce00@Lm@03001bmG;e00Ebm@03000P1B0500LP1@8001IJo005001J
=5XdFS@0000<FS@00`00Oomoo`0NOol00`00Oomoo`09Ool000Qoo`03001oogoo01]oo`8000a=m003
001Bg5;L00MBg0@001MI:003000LmAce00<Lm@03001bmG;e00Ebm@03000P1B0500DP1@8001QJo005
001J=5XdFS@0000;FS@00`00Oomoo`0POol00`00Oomoo`08Ool000Moo`03001oogoo01eoo`03001=
m4gd00]=m003001Bg5;L00YBg0@001AI:003000LmAce008Lm@03001bmG;e00Abm@03000P1B0500@P
1@8001YJo005001J=5XdFS@0000:FS@00`00Oomoo`0QOol00`00Oomoo`08Ool000Moo`03001oogoo
01ioo`03001=m4gd00Y=m003001Bg5;L00iBg0@0015I:005000LmAce7?D00005L_D00`0080DP1@03
80D2000LF_`01@00FSAJ=5Xd00002UXd00<007ooOol08Goo00<007ooOol027oo0007Ool00`00Oomo
o`0OOol00`00COA=m00:CO@00`00D]aBg00AD]`4000>FBP010007?DLm@0017;e00<0020580D00R05
0P007U[l00D005XdFSAJ=00000UJ=003001oogoo029oo`03001oogoo00Qoo`001goo00<007ooOol0
87oo00<004gdCO@02Dgd00<005;LD]`05E;L10002UTX00@001ce7?D000=bm@04000P1B0580D2000P
F_`01@00FSAJ=5Xd000025Xd00<007ooOol08goo00<007ooOol027oo0007Ool00`00Oomoo`0QOol0
0`00COA=m009CO@00`00D]aBg00HD]`40007FBP01P007?DLm@00L_D0008P1@80029Jo003001J=5Xd
008000MJ=003001oogoo02Aoo`03001oogoo00Qoo`001goo00<007ooOol08Woo00<004gdCO@024gd
00<005;LD]`075;L100015TX00@001ce001bm@<002AJo0<0009J=003001J=5Xd00AJ=003001oogoo
02Aoo`03001oogoo00Qoo`001goo00<007ooOol08goo00<004gdCO@024gd00<005;LD]`07e;L1000
00=I:00000000P008e[l10001EXd00<005XdFS@00UXd00<007ooOol09Woo00<007ooOol01goo0006
Ool00`00Oomoo`0UOol00`00COA=m007CO@00`00D]aBg00SD]`4000QF_`30003F_`00`00FSAJ=003
FS@01@00FSAJ=5Xd0000:Goo00<007ooOol01goo0006Ool00`00Oomoo`0VOol00`00COA=m006CO@0
0`00D]aBg00SD]`7000JF_`40006F_`00`00FSAJ=004FS@01000FSAJ=000:Goo00<007ooOol01goo
0006Ool00`00Oomoo`0WOol00`00COA=m006CO@00`00D]aBg00QD]`01P0050L000009GL0009UZP@0
01=Jo0<000YJo003001J=5Xd00EJ=08002Yoo`03001oogoo00Moo`001Woo00<007ooOol0:7oo00<0
04gdCO@01Dgd00<005;LD]`07e;L0P0000@D1`00LaL0008UM`03001UZVFZ00=UZP<000eJo0<000eJ
o003001J=5Xd00EJ=0@002Qoo`03001oogoo00Moo`001Woo00<007ooOol0:Goo00<004gdCO@01Dgd
00<005;LD]`07E;L00@001@750L000=c5`03000UM`0000QUZP<000MJo0<0011Jo003001J=5Xd00AJ
=006001UIFEU001UI@009goo00<007ooOol01goo0006Ool00`00Oomoo`0ZOol00`00COA=m004CO@0
0`00D]aBg00KD]`2000250L00`00LaMc5`02LaL010009GLUM`002VFZ0`0000AJo000000001AJo003
001J=5Xd00=J=004001UIFEU0002IFD00`00Oomoo`0TOol00`00Oomoo`07Ool000Eoo`03001oogoo
02]oo`03001=m4gd00E=m003001Bg5;L01QBg08000@D1`03001c5g<G009c5`05000UMbEg9GL0000:
IJX5000EF_`00`00FSAJ=002FS@00`00IFEUI@02IFD01000IFEUI@009Woo00<007ooOol01Woo0005
Ool00`00Oomoo`0/Ool00`00COA=m004CO@00`00D]aBg00GD]`00`0050LD1`0350L00`00LaMc5`03
LaL00`009GLUM`029GL00`00IJYUZP04IJX30005IJX4000AF_`01@00FSAJ=5Xd00001VEU00@006EU
IFD002Eoo`03001oogoo00Ioo`001Goo00<007ooOol0;Goo00<004gdCO@014gd00<005;LD]`055;L
0P001A@700<007<GLaL017<G00<002Eg9GL00bEg00<006FZ00000`0036FZ0`003U[l00D005XdFSAJ
=00000MUI@04001UIFEU000TOol00`00Oomoo`06Ool000Eoo`03001oogoo02ioo`03001=m4gd00==
m003001Bg5;L01=Bg003000D1a@700DD1`03001c5g<G00Ac5`03000UMbEg008UM`<001=UZP<000]J
o004001J=5Xd0008IFD01@00IFEUIFEU00008goo00<007ooOol01Woo0005Ool00`00Oomoo`0_Ool0
0`00COA=m002CO@00`00D]aBg00AD]`2000750L00`00LaMc5`06LaL400039GL00`00IJYUZP0CIJX3
0008F_`00`00FS@0000:IFD01@00IFEUIFEU00008Woo00<007ooOol01Woo0005Ool00`00Oomoo`0`
Ool00`00COA=m002CO@00`00D]aBg00?D]`00`0050LD1`0650L00`00LaMc5`05LaL300079GL00`00
IJYUZP0EIJX40004F_`2000<IFD01@00IFEUIFEU00008Goo00<007ooOol01Woo0005Ool00`00Oomo
o`0aOol01@00COA=m4gd00003e;L0P002A@700<007<GLaL00W<G0`000W<G00<002Eg9GL01BEg00<0
06FZIJX06FFZ0`0000=Jo00000003FEU00D006EUIFEUI@00021oo`03001oogoo00Ioo`0017oo00<0
07ooOol0<goo00D004gdCOA=m00000eBg003000D1a@700PD1`03001c5g<G00<000Ec5`03000UMbEg
00HUM`03001UZVFZ01]UZP<000iUI@05001UIFEUIFD0000OOol00`00Oomoo`06Ool000Aoo`03001o
ogoo03Aoo`04001=m4gd000;D]`2000:50L40008LaL00`009GLUM`079GL00`00IJYUZP0JIJX20000
15]M000000002fEU00<006EUIFD00VEU00<007ooOol07Goo00<007ooOol01Goo0004Ool00`00Oomo
o`0eOol01000COA=m00025;L0P002Q@70`002g<G00<002Eg9GL022Eg00<006FZIJX06FFZ0P0015]M
0`002FEU00<006EUIFD00VEU00<007ooOol077oo00<007ooOol01Goo0004Ool00`00Oomoo`0fOol0
0`00CO@00007D]`00`0050LD1`0650L400000a@7001c5`0<LaL00`009GLUM`089GL00`00IJYUZP0G
IJX01000FUIJEP001U]M0`001fEU00<006EUIFD00VEU0P0077oo00<007ooOol01Goo0004Ool00`00
Oomoo`0fOol200000dgd001Bg003D]`2000650L3000450L00`00LaMc5`0<LaL00`009GLUM`099GL0
0`00IJYUZP0EIJX01@00FUIJEUYF00002E]M0`001FEU00<006EUIFD00fEU00<007ooOol06Goo00<0
07ooOol01Goo0004Ool00`00Oomoo`0fOol40003D]`00`0050LD1`0350L3000650L00`00LaMc5`0=
LaL00`009GLUM`0:9GL00`00IJYUZP0CIJX00`00FUIJEP02FUH00`00FeeKG@0:Fed40002IFD00`00
IFEUI@03IFD00`00Oomoo`0HOol00`00Oomoo`05Ool000=oo`03001oogoo03Ioo`P000<D1`<000TD
1`03001c5g<G00ec5`03000UMbEg00/UM`03001UZVFZ019UZP03001JEUYF009JEP03001KGE]M00iK
G@<000IUI@03001oogoo01Moo`03001oogoo00Eoo`000goo00<007ooOol0=Goo1`000Q@70`002a@7
00<007<GLaL03W<G00<002Eg9GL02bEg00<006FZIJX04FFZ00<005YFFUH00eYF00<005]MFed04E]M
0`0016EU00<007ooOol05Woo00<007ooOol01Goo0003Ool00`00Oomoo`0dOol:000=50L00`00LaMc
5`0?LaL00`009GLUM`0<9GL00`00IJYUZP0?IJX00`00FUIJEP04FUH00`00FeeKG@0BFed00`00Fed0
00020002IFD00`00Oomoo`0FOol00`00Oomoo`04Ool000=oo`03001oogoo03Aoo`T000hD1`03001c
5g<G00mc5`03000UMbEg00dUM`03001UZVFZ00eUZP03001JEUYF00IJEP03001KGE]M019KG@04001K
GE]MFed4000GOol00`00Oomoo`04Ool000=oo`03001oogoo03=oo`H00096oP8000dD1`03001c5g<G
015c5`03000UMbEg00dUM`03001UZVFZ00aUZP03001JEUYF00IJEP03001KGE]M01=KG@03001KGE]M
00=KG@8001Ioo`03001oogoo00Aoo`000goo00<007ooOol0<Goo1`000dKn0P0031@700<007<GLaL0
4W<G00<002Eg9GL03REg00<006FZIJX02VFZ00<005YFFUH01eYF00<005]MFed04e]M00@005]MFeeK
G@8001Qoo`03001oogoo00Aoo`000goo00<007ooOol0;Woo20001DKn0P002a@700<007<GLaL04g<G
00<002Eg9GL03bEg00<006FZIJX026FZ00<005YFFUH025YF00<005]MFed055]M0`006Woo00<007oo
Ool017oo0002Ool00`00Oomoo`0/Ool30003Ool40006A_h00`00DQH0000:50L00`00LaMc5`0CLaL0
0`009GLUM`0@9GL00`00IJYUZP06IJX00`00FUIJEP09FUH00`00FeeKG@0AFed300000goo001oo`0J
Ool00`00Oomoo`04Ool0009oo`03001oogoo02Uoo`<000Eoo`@000M6oP03001B5P0000TD1`03001c
5g<G01Ac5`03000UMbEg014UM`03001UZVFZ00EUZP03001JEUYF00YJEP03001KGE]M00iKG@8000Eo
o`03001oogoo01Qoo`03001oogoo00Aoo`000Woo00<007ooOol09Woo0`001goo100024Kn00<0058F
000021@700<007<GLaL05G<G00<002Eg9GL04BEg00<006FZIJX016FZ00<005YFFUH02eYF00<005]M
Fed02e]M0`0027oo00<007ooOol05goo00<007ooOol017oo0002Ool00`00Oomoo`0SOol30009Ool4
0009A_h01000DQIB5P001a@700<007<GLaL05W<G00<002Eg9GL04BEg00<006FZIJX00VFZ00<005YF
FUH035YF00<005]MFed02E]M0P002goo00<007ooOol067oo00<007ooOol00goo0002Ool00`00Oomo
o`0POol3000<Ool3000:A_h01000DQIB5P001Q@700<007<GLaL05g<G00<002Eg9GL04REg00@006FZ
IJX000mJEP03001KGE]M00IKG@<000ioo`03001oogoo01Moo`03001oogoo00=oo`000Woo00<007oo
Ool077oo10003Woo0`002dKn00@0058FDQH000DD1`03001c5g<G01Qc5`03000UMbEg01<UM`03001U
ZP0000mJEP03001KGE]M00=KG@<0019oo`03001oogoo01Ioo`03001oogoo00=oo`000Woo00<007oo
Ool06Goo0`004Goo0P003DKn00D0058FDQIB5P0000@D1`03001c5g<G01Qc5`03000UMbEg01@UM`03
001JEUYF00iJEP04001KGE]MFed2000FOol00`00Oomoo`0EOol00`00Oomoo`03Ool00003Ool007oo
01Qoo`<001=oo`8000i6oP05001B5U8FDQH0000350L00`00LaMc5`0ILaL00`009GLUM`0B9GL2000B
FUH3000HOol00`00Oomoo`0EOol00`00Oomoo`03Ool00003Ool007oo01Eoo`<001Ioo`030016oTKn
00e6oP05001B5U8FDQH0000250L00`00LaMc5`0JLaL00`009GLUM`0@9GL2000DFUH00`00Oomoo`0I
Ool00`00Oomoo`0DOol00`00Oomoo`03Ool00003Ool007oo019oo`<001Qoo`<000i6oP07001B5U8F
DQH001@700007W<G00<002Eg9GL03BEg0P005UYF00<007ooOol06Woo00<007ooOol04goo00<007oo
Ool00goo00000goo001oo`0?Ool3000NOol60008A_h00`00DQIB5P02DQH2000NLaL00`009GLUM`0<
9GL00`00FUIJEP0FFUH00`00Oomoo`0KOol00`00Oomoo`0COol00`00Oomoo`02Ool00003Ool007oo
00aoo`<002Moo`H00096oP03001B5U8F009B5P<001ec5`03000UMbEg00XUM`8001UJEP03001oogoo
01aoo`03001oogoo019oo`03001oogoo009oo`0000=oo`00Ool027oo1000<7oo0`001e8F1@0067<G
00<002Eg9GL022Eg0P0075YF00<007ooOol06goo00<007ooOol04Woo00<007ooOol00Woo00000`00
Oomoo`05Ool3000fOol00`00DQIB5P0:DQH4000DLaL00`009GLUM`069GL2000NFUH00`00Oomoo`0L
Ool00`00Oomoo`0AOol00`00Oomoo`02Ool00003001oogoo009oo`<003Uoo`03001B5U8F00iB5PD0
00mc5`03000UMbEg00DUM`03001JEUYF01iJEP03001oogoo01eoo`03001oogoo011oo`03001oogoo
009oo`0000<007oo00000P00?7oo00<0058FDQH04e8F1@002g<G00<002Eg9GL00REg0P008EYF00<0
07ooOol07Woo00<007ooOol03goo00<007ooOol00Woo0002000oOol00`00DQIB5P0HDQH40007LaL0
0`009GLUM`02000SFUH00`00Oomoo`0OOol00`00Oomoo`0>Ool00`00Oomoo`02Ool00003Ool00000
03moo`03001B5U8F01]B5PD0009c5`<002EJEP03001oogoo01moo`03001oogoo00ioo`03001oogoo
009oo`000goo00<007ooOol0?7oo00<0058FDQH0858F10009eYF00<007ooOol07goo00<007ooOol0
3Woo00<007ooOol00Goo0004Ool00`00Oomoo`0kOol00`00DQIB5P0PDQH3000017MG0000000095YF
00<007ooOol087oo00<007ooOol03Goo00<007ooOol00Goo0005Ool2000kOol00`00DQIB5P0NDQH2
00032AL01000MeMgEgMG100085YF00<007ooOol08Goo00<007ooOol037oo00<007ooOol00Goo0007
Ool00`00Oomoo`0hOol00`00DQIB5P0LDQH200052AL00`00MeMgE`05MeL3000MFUH00`00Oomoo`0R
Ool00`00Oomoo`0;Ool00`00Oomoo`01Ool000Qoo`03001oogoo03Moo`03001B5U8F01YB5P8000L9
5`03001gEgMG00QgE`<001YJEP03001oogoo029oo`03001oogoo00]oo`03001oogoo005oo`002Goo
0P00=goo00<0058FDQH0658F0P002@TG00<007MGMeL02gMG10005eYF00<007ooOol08Woo00<007oo
Ool02Woo00<007ooOol00Goo000;Ool00`00Oomoo`0dOol00`00DQIB5P0FDQH200000goo00095`09
2AL00`00MeMgE`0>MeL3000DFUH00`00Oomoo`0SOol00`00Oomoo`09Ool00`00Oomoo`01Ool000ao
o`03001oogoo03=oo`03001B5U8F01EB5P05001oogooOol0000:2AL00`00MeMgE`0?MeL00`00Ool0
0002000AFUH00`00Oomoo`0TOol00`00Oomoo`09Ool10001Ool1Ool000eoo`8003=oo`03001B5U8F
01=B5P8000Eoo`0300095`TG00L95`03001gEgMG00mgE`03001oogoo009oo`@000eJEP03001oogoo
02Eoo`03001oogoo00Qoo`40005oo`5oo`003goo00<007ooOol0<7oo00<0058FDQH04E8F0P0027oo
00<000TG2AL01PTG00<007MGMeL03WMG00<007ooOol01goo0`002UYF00<007ooOol09Goo00<007oo
Ool027oo0@000Goo0Goo000@Ool00`00Oomoo`0_Ool00`00DQIB5P0?DQH2000;Ool00`002AL95`06
2AL00`00MeMgE`0=MeL00`00Oomoo`0:Ool30007FUH00`00Oomoo`0VOol00`00Oomoo`07Ool10001
Ool1Ool0015oo`8002moo`03001B5U8F00eB5P8000eoo`0300095`TG00H95`03001gEgMG00agE`03
001oogoo00ioo`@000AJEP03001oogoo02Ioo`03001oogoo00Ioo`40005oo`5oo`004goo00<007oo
Ool0;7oo00<0058FDQH02e8F0P0047oo00<000TG2AL01@TG00<007MGMeL02gMG00<007ooOol04goo
0`0000=JEP00Ool0:7oo00<007ooOol01Goo0@000Goo0Goo000DOol2000/Ool00`00DQIB5P0:DQH0
0`00Oomoo`0AOol00`002AL95`042AL00`00MeMgE`0;MeL00`00Oomoo`0FOol2000YOol00`00Oomo
o`05Ool10001Ool1Ool001Ioo`03001oogoo02Uoo`03001B5U8F00QB5P8001Aoo`0300095`TG00D9
5`03001gEgMG00UgE`03001oogoo04=oo`03001oogoo00Eoo`40005oo`005goo00<007ooOol0:7oo
00<0058FDQH01U8F0`005Woo00<000TG2AL010TG00<007MGMeL02GMG00<007ooOol0A7oo00<007oo
Ool017oo0@000Goo000HOol2000XOol00`00DQIB5P04DQH20002Ool00`00Oomoo`0EOol00`002AL9
5`032AL00`00MeMgE`08MeL00`00Oomoo`16Ool00`00Oomoo`03Ool10001Ool001Yoo`03001oogoo
02Eoo`03001B5U8F009B5P8000Aoo`03001oogoo01Ioo`0300095`TG00<95`03001gEgMG00MgE`03
001oogoo04Moo`03001oogoo009oo`40005oo`006goo00<007ooOol097oo00<0058FDQH00P001Woo
00<007ooOol05Woo00<000TG2AL00`TG00<007MGMeL01WMG00<007ooOol0B7oo00<007ooOol00Woo
0@000Goo000LOol2000TOol30008Ool00`00Oomoo`0GOol00`002AL95`022AL00`00MeMgE`06MeL0
0`00Oomoo`19Ool01@00Oomoogoo00000Goo000NOol00`00Oomoo`0QOol00`00Oomoo`08Ool00`00
Oomoo`0HOol01@002AL95`TG00001gMG00<007ooOol0Bgoo00@007ooOol0005oo`007goo00<007oo
Ool0:goo00<007ooOol06Goo00D000TG2AL95`0000IgE`03001oogoo04aoo`03001oo`00005oo`00
87oo0P00:goo00<007ooOol06Goo00D000TG2AL95`0000EgE`03001oogoo04ioo`40005oo`400000
8Woo00<007ooOol0:7oo00<007ooOol06Woo00@000TG2AL000EgE`03001oogoo04ioo`40005oo`40
00008goo00<007ooOol09goo00<007ooOol06goo00<000TG000017MG00<007ooOol0D7oo0P00000T
Ool2000WOol00`00Oomoo`0KOol010002AL95`000gMG00<007ooOol0Cgoo0`00000VOol00`00Oomo
o`0TOol00`00Oomoo`0LOol00`002AL00002MeL00`00Oomoo`1<Ool40003Ool002Moo`8002Aoo`03
001oogoo01eoo`80009gE`03001oogoo04Qoo`@000Moo`00:Goo00<007ooOol08Goo00<007ooOol0
7Woo00<007MG0000Agoo10002goo000ZOol00`00Oomoo`0POol00`00Oomoo`0NOol30013Ool4000?
Ool002]oo`80021oo`03001oogoo01moo`03001oogoo03eoo`D001=oo`00;Goo00<007ooOol07Goo
00<007ooOol0Fgoo100067oo000^Ool00`00Oomoo`0LOol00`00Oomoo`1GOol4000LOol002moo`80
01aoo`03001oogoo05=oo`@0021oo`00<Goo00<007ooOol06Goo00<007ooOol0Cgoo100097oo000b
Ool00`00Oomoo`0HOol00`00Oomoo`1:Ool5000XOol003=oo`8001Qoo`03001oogoo04Ioo`@002eo
o`00=Goo00<007ooOol05Goo00<007ooOol0@Woo1000<Goo000fOol00`00Oomoo`0DOol00`00Oomo
o`0nOol4000eOol003Moo`8001Aoo`03001oogoo03Yoo`@003Uoo`00>Goo00<007ooOol04Goo00<0
07ooOol0=Goo1@00?Goo000jOol2000@Ool00`00Oomoo`0bOol40012Ool003aoo`03001oogoo00eo
o`03001oogoo02ioo`@004Ioo`00?Goo00<007ooOol037oo00<007ooOol0:Woo1000BWoo000nOol2
000<Ool00`00Oomoo`0UOol5001>Ool0041oo`03001oogoo00Uoo`03001oogoo025oo`@005=oo`00
@Goo00<007ooOol027oo00<007ooOol07Goo1000Egoo0012Ool20008Ool00`00Oomoo`0IOol4001K
Ool004Aoo`03001oogoo00Eoo`03001oogoo01Eoo`@005moo`00AGoo00<007ooOol017oo00<007oo
Ool047oo1@00Hgoo0016Ool20004Ool00`00Oomoo`0<Ool4001XOol004Qoo`05001oogooOol0000:
Ool4001/Ool004Uoo`04001oogoo0006Ool4001`Ool004Yoo`<0009oo`@007Aoo`00C7oo0`00N7oo
0000\
\>"],
  ImageRangeCache->{{{164.938, 287.625}, {252.875, 130.063}} -> {-1.21816, \
0.975361, 0.00464951, 0.00464951}},
  CellTags->{"b:46.3", "IPM3.10.5.4"}]
}, Open  ]],

Cell[TextData[{
  "Here is the button code. Note that we have also added an option to ",
  StyleBox["ButtonBox", "MR"],
  " to set the background and set the entire cell to use the Times font \
family."
}], "Text",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell["\<\
Cell[BoxData[
    ButtonBox[RowBox[{\"Stellate\",\" \",\"Icosahedron\"}],
      ButtonFunction->
        CompoundExpression[Needs[\"Graphics`Polyhedra`\"],
          Show[Graphics3D[Stellate[Icosahedron[]]]]],
      ButtonEvaluator->Automatic,
      Background->GrayLevel[.5]]],
      \"Input\",Active->True,
      FontFamily->\"Times\",
      FontColor->GrayLevel[1]]\
\>", "Program",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell["\<\
And here is the button with a result of clicking it just below.\
\>", "Text",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[CellGroupData[{

Cell[BoxData[
    ButtonBox[\(Stellate\ Icosahedron\),
      ButtonFunction:>CompoundExpression[ 
          Needs[ "Graphics`Polyhedra`"], 
          Show[ 
            Graphics3D[ 
              Stellate[ 
                Icosahedron[ ]]]]],
      ButtonEvaluator->Automatic,
      Background->GrayLevel[0.500008]]], "Input",
  CellLabel->"In[11]:=",
  Active->True,
  Evaluatable->False,
  FontFamily->"Times",
  FontColor->GrayLevel[1],
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[GraphicsData["PostScript", "\<\
%!
%%Creator: Mathematica
%%AspectRatio: 1.00162 
MathPictureStart
/Mabs {
Mgmatrix idtransform
Mtmatrix dtransform
} bind def
/Mabsadd { Mabs
3 -1 roll add
3 1 roll add
exch } bind def
%% Graphics3D
%%IncludeResource: font Helvetica
%%IncludeFont: Helvetica
/Helvetica findfont 9  scalefont  setfont
% Scaling calculations
-0.0206914 1.08755 -7.41594e-017 1.08755 [
[ 0 0 0 0 ]
[ 1 1.00162 0 0 ]
] MathScale
% Start of Graphics
1 setlinecap
1 setlinejoin
newpath
0 g
.25 Mabswid
[ ] 0 setdash
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.38643 1.00162 L
s
.38643 1.00162 m
.40217 .57872 L
s
.40217 .57872 m
.07192 .25073 L
s
.68538 0 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.68538 0 m
.07192 .25073 L
s
.40217 .57872 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.38643 1.00162 L
s
.38643 1.00162 m
.40217 .57872 L
s
0 0 m
1 0 L
1 1.00162 L
0 1.00162 L
closepath
clip
newpath
.5 Mabswid
.786 .442 .405 r
.44491 .44907 m .36477 .52184 L .55473 .57484 L closepath p F P
0 g
s
.37 .395 .757 r
.44491 .44907 m .55473 .57484 L .52308 .38117 L closepath p F P
0 g
s
0 0 0 r
.44491 .44907 m .52308 .38117 L .36477 .52184 L closepath p F P
0 g
s
.366 .042 .277 r
.39521 .65616 m .44391 .71565 L .55473 .57484 L closepath p F P
0 g
s
.702 .725 .868 r
.39521 .65616 m .55473 .57484 L .36477 .52184 L closepath p F P
0 g
s
0 0 .535 r
.39521 .65616 m .36477 .52184 L .44391 .71565 L closepath p F P
0 g
s
.299 .033 .331 r
.65615 .42467 m .55473 .57484 L .70544 .48388 L closepath p F P
0 g
s
.859 .738 .729 r
.65615 .42467 m .52308 .38117 L .55473 .57484 L closepath p F P
0 g
s
.641 .082 0 r
.65615 .42467 m .70544 .48388 L .52308 .38117 L closepath p F P
0 g
s
.873 .707 .674 r
.57791 .76621 m .55473 .57484 L .44391 .71565 L closepath p F P
0 g
s
.311 .298 .683 r
.57791 .76621 m .65788 .69455 L .55473 .57484 L closepath p F P
0 g
s
.6 .813 .982 r
.57791 .76621 m .44391 .71565 L .65788 .69455 L closepath p F P
0 g
s
.721 .348 .342 r
.74263 .62049 m .55473 .57484 L .65788 .69455 L closepath p F P
0 g
s
.643 .687 .876 r
.74263 .62049 m .70544 .48388 L .55473 .57484 L closepath p F P
0 g
s
.971 .845 .659 r
.74263 .62049 m .65788 .69455 L .70544 .48388 L closepath p F P
0 g
s
.698 .565 .701 r
.33115 .32429 m .36477 .52184 L .52308 .38117 L closepath p F P
0 g
s
0 0 .52 r
.33115 .32429 m .38251 .38653 L .36477 .52184 L closepath p F P
0 g
s
.608 .076 0 r
.33115 .32429 m .52308 .38117 L .38251 .38653 L closepath p F P
0 g
s
.706 .538 .659 r
.24235 .68284 m .44391 .71565 L .36477 .52184 L closepath p F P
0 g
s
0 0 0 r
.24235 .68284 m .36477 .52184 L .32852 .60768 L closepath p F P
0 g
s
.817 .41 .277 r
.19481 .46702 m .32852 .60768 L .36477 .52184 L closepath p F P
0 g
s
.712 .79 .912 r
.19481 .46702 m .36477 .52184 L .38251 .38653 L closepath p F P
0 g
s
.654 .533 .704 r
.69731 .27878 m .52308 .38117 L .70544 .48388 L closepath p F P
0 g
s
0 0 0 r
.69731 .27878 m .60797 .35949 L .52308 .38117 L closepath p F P
0 g
s
.217 .339 .774 r
.48589 .20782 m .52308 .38117 L .60797 .35949 L closepath p F P
0 g
s
.899 .805 .747 r
.48589 .20782 m .38251 .38653 L .52308 .38117 L closepath p F P
0 g
s
.559 .729 .954 r
.24235 .68284 m .32852 .60768 L .44391 .71565 L closepath p F P
0 g
s
.927 .746 .637 r
.33664 .82736 m .44391 .71565 L .32852 .60768 L closepath p F P
0 g
s
.666 .487 .633 r
.56015 .87859 m .65788 .69455 L .44391 .71565 L closepath p F P
0 g
s
.079 .258 .746 r
.56015 .87859 m .44391 .71565 L .52308 .72469 L closepath p F P
0 g
s
.145 0 0 r
.33664 .82736 m .52308 .72469 L .44391 .71565 L closepath p F P
0 g
s
.952 .764 .613 r
.69731 .27878 m .70544 .48388 L .60797 .35949 L closepath p F P
0 g
s
.633 .483 .662 r
.85199 .62068 m .70544 .48388 L .65788 .69455 L closepath p F P
0 g
s
.804 .349 .159 r
.85199 .62068 m .69976 .56825 L .70544 .48388 L closepath p F P
0 g
s
0 0 .084 r
.82611 .39367 m .70544 .48388 L .69976 .56825 L closepath p F P
0 g
s
.593 .717 .93 r
.82611 .39367 m .60797 .35949 L .70544 .48388 L closepath p F P
0 g
s
.702 .816 .936 r
.85199 .62068 m .65788 .69455 L .69976 .56825 L closepath p F P
0 g
s
.92 .834 .744 r
.56015 .87859 m .52308 .72469 L .65788 .69455 L closepath p F P
0 g
s
.67 .196 .118 r
.73868 .79032 m .65788 .69455 L .52308 .72469 L closepath p F P
0 g
s
.065 .115 .604 r
.73868 .79032 m .69976 .56825 L .65788 .69455 L closepath p F P
0 g
s
.624 .472 .656 r
.19481 .46702 m .38251 .38653 L .32852 .60768 L closepath p F P
0 g
s
.661 .476 .624 r
.48589 .20782 m .60797 .35949 L .38251 .38653 L closepath p F P
0 g
s
.635 .791 .957 r
.45664 .27423 m .38251 .38653 L .60797 .35949 L closepath p F P
0 g
s
.22 .231 .659 r
.45664 .27423 m .48535 .50673 L .38251 .38653 L closepath p F P
0 g
s
.947 .817 .683 r
.25792 .45112 m .32852 .60768 L .38251 .38653 L closepath p F P
0 g
s
.707 .293 .259 r
.25792 .45112 m .38251 .38653 L .48535 .50673 L closepath p F P
0 g
s
.628 .699 .896 r
.25792 .45112 m .48535 .50673 L .32852 .60768 L closepath p F P
0 g
s
.648 .528 .703 r
.33664 .82736 m .32852 .60768 L .52308 .72469 L closepath p F P
0 g
s
.722 .218 .033 r
.35521 .69402 m .52308 .72469 L .32852 .60768 L closepath p F P
0 g
s
.184 0 .222 r
.35521 .69402 m .32852 .60768 L .48535 .50673 L closepath p F P
0 g
s
.705 .534 .654 r
.82611 .39367 m .69976 .56825 L .60797 .35949 L closepath p F P
0 g
s
.892 .722 .664 r
.45664 .27423 m .60797 .35949 L .48535 .50673 L closepath p F P
0 g
s
0 .115 .646 r
.68527 .40123 m .60797 .35949 L .69976 .56825 L closepath p F P
0 g
s
.266 0 .159 r
.68527 .40123 m .48535 .50673 L .60797 .35949 L closepath p F P
0 g
s
.697 .564 .701 r
.73868 .79032 m .52308 .72469 L .69976 .56825 L closepath p F P
0 g
s
0 0 0 r
.62474 .66693 m .69976 .56825 L .52308 .72469 L closepath p F P
0 g
s
.296 .351 .75 r
.62474 .66693 m .52308 .72469 L .48535 .50673 L closepath p F P
0 g
s
.876 .757 .728 r
.35521 .69402 m .48535 .50673 L .52308 .72469 L closepath p F P
0 g
s
.697 .742 .886 r
.68527 .40123 m .69976 .56825 L .48535 .50673 L closepath p F P
0 g
s
.786 .409 .341 r
.62474 .66693 m .48535 .50673 L .69976 .56825 L closepath p F P
0 g
s
.25 Mabswid
.68538 0 m
.93948 .40631 L
s
.93948 .40631 m
1 .85355 L
s
1 .85355 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.07192 .25073 m
0 .71646 L
s
0 .71646 m
.71506 .4883 L
s
.71506 .4883 m
.68538 0 L
s
.68538 0 m
.07192 .25073 L
s
% End of Graphics
MathPictureEnd
\
\>"], "Graphics",
  CellLabel->"From In[22]:=",
  ImageSize->{199.5, 199.75},
  ImageMargins->{{62, 0}, {0, 0}},
  ImageRegion->{{0, 1}, {0, 1}},
  ImageCache->GraphicsData["Bitmap", "\<\
CF5dJ6E]HGAYHf4PAg9QL6QYHg<PAVmbKF5d0`4000370000aa000`40O003h00OQgoo0P00?Woo0025
Ool200000goo0000000mOol0089oo`<000=oo`8003eoo`00P7oo0P001Woo00<007oo0000?7oo001n
Ool20008Ool00`00Ool0000lOol007]oo`<000Yoo`04001oogoo000kOol007Uoo`8000eoo`05001o
ogooOol0000jOol007Ioo`<000moo`05001oogooOol0000jOol007Aoo`80019oo`03001oogoo009o
o`03001oogoo03Moo`00LGoo0`0057oo00<007ooOol00goo00<007ooOol0=Woo001_Ool2000HOol0
0`00Oomoo`02Ool00`00Oomoo`0fOol006aoo`<001Yoo`03001oogoo00=oo`03001oogoo03Eoo`00
JWoo0P007Goo00<007ooOol00goo00<007ooOol0=Goo001XOol2000OOol00`00Oomoo`04Ool00`00
Oomoo`0dOol006Eoo`<0025oo`03001oogoo00Eoo`03001oogoo03=oo`00Hgoo0P0097oo00<007oo
Ool01Goo00<007ooOol0<goo001POol3000VOol00`00Oomoo`06Ool00`00Oomoo`0bOol005ioo`80
02Uoo`03001oogoo00Moo`03001oogoo035oo`00Fgoo0`00:goo00<007ooOol01goo00<007ooOol0
<Goo001IOol2000^Ool00`00Oomoo`08Ool00`00Oomoo`0`Ool005Ioo`<0031oo`03001oogoo00Qo
o`03001oogoo031oo`00E7oo0P00<goo00<007ooOol02Goo00<007ooOol0;goo001BOol2000eOol0
0`00Oomoo`0:Ool00`00Oomoo`0^Ool004moo`<003Moo`03001oogoo00Yoo`03001oogoo02ioo`00
CGoo0P00>Woo00<007ooOol02goo00<007ooOol0;Goo001:Ool3000lOol00`00Oomoo`0<Ool00`00
Oomoo`0/Ool004Qoo`8003moo`03001oogoo00aoo`03001oogoo02aoo`00AGoo0`00@Goo00<007oo
Ool03Goo00<007ooOol0:goo0013Ool20014Ool00`00Oomoo`0=Ool00`00Oomoo`0[Ool0045oo`80
04Ioo`03001oogoo00ioo`03001oogoo02Yoo`00?Woo0`00BGoo00<007ooOol03Woo00<007ooOol0
:Goo000lOol2001<Ool00`00Oomoo`0>Ool00`00Oomoo`0YOol003Uoo`<004ioo`03001oogoo00mo
o`03001oogoo02Qoo`00=goo0P00DGoo00<007ooOol047oo00<007ooOol09goo000dOol3001COol0
0`00Oomoo`0@Ool00`00Oomoo`0WOol0039oo`8005Ioo`03001oogoo015oo`03001oogoo02Ioo`00
;goo0`00F7oo00<007ooOol04Goo00<007ooOol09Woo000]Ool2001KOol00`00Oomoo`0BOol00`00
Oomoo`0UOol002]oo`8005eoo`03001oogoo01=oo`03001oogoo02Aoo`00:7oo0`00Ggoo00<007oo
Ool04goo00<007ooOol097oo000VOol2001ROol00`00Oomoo`0DOol00`00Oomoo`0SOol002=oo`<0
03Yoo`03001oogoo02Moo`03001oogoo01Eoo`03001oogoo029oo`008Goo0P00?7oo00<005Gc0000
:7oo00<007ooOol05Goo00<007ooOol08Woo000NOol3000nOol01000EO=El`009goo00<007ooOol0
5Woo00<007ooOol08Goo000LOol20010Ool01@00EO=EleGc00009goo00<007ooOol05Woo00<007oo
Ool08Goo000IOol30012Ool00`00EO=El`02EO<00`00Oomoo`0TOol00`00Oomoo`0GOol00`00Oomo
o`0POol001Moo`8004Aoo`03001EleGc00AEl`03001oogoo02=oo`03001oogoo01Qoo`03001oogoo
01moo`005Goo0P00AGoo00<005GcEO<01UGc00<007ooOol08Woo00<007ooOol067oo00<007ooOol0
7goo000BOol30017Ool00`00EO=El`07EO<00`00Oomoo`0QOol00`00Oomoo`0IOol00`00Oomoo`0N
Ool0011oo`8004Uoo`03001EleGc00QEl`03001oogoo029oo`03001oogoo01Uoo`03001oogoo01eo
o`003Woo0P00Bgoo00<005GcEO<02EGc00<007ooOol08Goo00<007ooOol06Goo00<007ooOol07Goo
000>Ool2001:Ool00`00EO=El`0;EO<00`00Oomoo`0POol00`00Oomoo`0JOol00`00Oomoo`0LOol0
00ioo`03001oo`0004Qoo`03001EleGc00eEl`03001oogoo01moo`03001oogoo01Yoo`03001oogoo
01aoo`003Woo00@007ooOol004Moo`03001EleGc00iEl`03001oogoo01ioo`03001oogoo01]oo`03
001oogoo01]oo`003Goo00<007ooOol00Woo00<007ooOol0@goo00<005Gc00003eGc00<007ooOol0
7Woo00<007ooOol077oo00<007ooOol06Woo000=Ool00`00Oomoo`03Ool00`00Oomoo`12Ool5000>
EO<00`00Oomoo`0LOol2000NOol00`00Oomoo`0JOol000eoo`03001oogoo00Aoo`03001oogoo041o
o`04001El`000002L_D2000=EO<00`00Oomoo`0IOol4000OOol00`00Oomoo`0IOol000eoo`03001o
ogoo00Eoo`03001oogoo03ioo`05001El`007?D00004L_D2000<EO<00`00Oomoo`0FOol300000g/C
0000000POol00`00Oomoo`0HOol000eoo`03001oogoo00Ioo`03001oogoo03eoo`8000<Lm@03001b
mG;e00=bm@03001EleGc00YEl`03001oogoo01=oo`@0009k4`80021oo`03001oogoo01Qoo`003Goo
00<007ooOol01goo00<007ooOol0>goo00<005Gc00000ace00<007;eL_D017;e0P002UGc00<007oo
Ool04Woo10000g/C0P008Goo00<007ooOol05goo000<Ool00`00Oomoo`09Ool00`00Oomoo`0jOol2
00047?D00`00L_Ebm@06L_D20009EO<00`00Oomoo`0?Ool40005Na<2000QOol00`00Oomoo`0GOol0
00aoo`03001oogoo00Yoo`03001oogoo03Qoo`8000DLm@03001bmG;e00Qbm@8000QEl`03001oogoo
00aoo`D000Ik4`80029oo`03001oogoo01Ioo`0037oo00<007ooOol02goo00<007ooOol0=Woo00<0
05Gc00001Ace00<007;eL_D02W;e00<005GcEO<01UGc00<007ooOol02Goo1P001g/C0P008goo00<0
07ooOol05Goo000<Ool00`00Oomoo`0<Ool00`00Oomoo`0eOol200067?D00`00L_Ebm@0;L_D20007
EO<00`00Oomoo`07Ool60008Na<2000SOol00`00Oomoo`0EOol000aoo`03001oogoo00eoo`03001o
ogoo029oo`8000moo`8000LLm@03001bmG;e00ebm@8000EEl`03001oogoo00Eoo`L000Uk4`8002Ao
o`03001oogoo01Aoo`0037oo00<007ooOol03Woo00<007ooOol08Goo1P002goo00<001ce7?D01Qce
00<007;eL_D03g;e0P0015Gc00<007ooOol00Woo20002W/C0P009Goo00<007ooOol04goo000<Ool0
0`00Oomoo`0?Ool00`00Oomoo`0POol00`0001000003C4030007Ool200097?D00`00L_Ebm@0@L_D2
0003EO<00`00Ool00008000<Na<00`00Oomoo`0SOol00`00Oomoo`0COol000]oo`03001oogoo015o
o`03001oogoo021oo`8000I<@0@0009oo`8000XLm@03001bmG;e019bm@04001EleGcEO<9000=Na<0
0`00Oomoo`0TOol00`00Oomoo`0BOol000]oo`03001oogoo019oo`03001oogoo01moo`0300004000
00U<@0<000/Lm@03001bmG;e01=bm@80009El`H000mk4`8002Eoo`03001oogoo019oo`002goo00<0
07ooOol04goo00<007ooOol07Woo00@0000@010000U<@08000/Lm@03001bmG;e01Ebm@L0011k4`80
02Ioo`03001oogoo015oo`002goo00<007ooOol057oo00<007ooOol07Goo00D0000@0100400000Q<
@003000LmAce00XLm@03001bmG;e01Mbm@@0015k4`8002Moo`03001oogoo011oo`002goo00<007oo
Ool05Goo00<007ooOol077oo00<0000@01000P0@00<004a0C40014a000<001ce7?D02ace00<007;e
L_D06G;e10003g/C0P009goo00<007ooOol047oo000;Ool00`00Oomoo`0FOol00`00Oomoo`0KOol0
0`000100400301000`00C41<@002C402000=7?D00`00L_Ebm@0HL_D01@0080D00000FS@01P002G/C
0P00:7oo00<007ooOol03goo000:Ool00`00Oomoo`0HOol00`00Oomoo`0KOol00`00010040020100
0`00C41<@002C4000`007?DLm@0<7?D00`00L_Ebm@0GL_D00`0080DP1@0280D20005FS@60003Na<2
000YOol00`00Oomoo`0>Ool000Yoo`03001oogoo01Uoo`03001oogoo01Yoo`030000400@00<04004
001<@4a0000@7?D00`00L_Ebm@0FL_D00`0080DP1@0480D20009FS@6000XOol00`00Oomoo`0>Ool0
00Yoo`03001oogoo01Yoo`03001oogoo01Uoo`030000400@00@04003001<@000010Lm@03001bmG;e
01Ebm@03000P1B0500LP1@8000]J=003001J=00000D002=oo`03001oogoo00eoo`002Woo00<007oo
Ool06goo00<007ooOol067oo00<0000@0100100@0P004Ace00<007;eL_D057;e00<0020580D02R05
0P002EXd00<005XdFS@01EXd1P007Goo00<007ooOol03Goo000:Ool00`00Oomoo`0LOol00`00Oomo
o`0GOol00`0001004002010300000eTX000Lm@0?7?D00`00L_Ebm@0CL_D00`0080DP1@0=80D20007
FS@00`00FSAJ=00;FS@6000HOol00`00Oomoo`0<Ool000Yoo`03001oogoo01eoo`03001oogoo01Mo
o`0300004000008000AI:003000LmAce00dLm@03001bmG;e019bm@03000P1B05010P1@8000EJ=003
001J=5Xd015J=0@001Eoo`03001oogoo00]oo`002Woo00<007ooOol07Woo00<007ooOol05Goo1000
1eTX00<001ce7?D031ce00<007;eL_D04G;e00<0020580D04b050P000eXd00<005XdFS@04eXd00<0
07ooOol057oo00<007ooOol02goo0009Ool00`00Oomoo`0POol00`00Oomoo`0BOol40009FBP00`00
7?DLm@0<7?D00`00L_Ebm@0AL_D00`0080DP1@0B80D30003FS@00`00FSAJ=00CFS@00`00Oomoo`0D
Ool2000<Ool000Uoo`03001oogoo025oo`03001oogoo00moo`@000aI:003000LmAce00`Lm@03001b
mG;e00mbm@03000P1B05014P1@80009Jo005001J=5XdFS@0000DFS@00`00Oomoo`0BOol300000goo
001oo`0;Ool000Uoo`03001oogoo029oo`03001oogoo00]oo`<00003CO@005TX00iI:003000LmAce
00/Lm@03001bmG;e00ibm@03000P1B05010P1@8000AJo005001J=5XdFS@0000CFS@00`00Oomoo`0@
Ool30004Ool00`00Oomoo`0:Ool000Uoo`03001oogoo02=oo`03001oogoo00Qoo`80009=m080015I
:003000LmAce00XLm@03001bmG;e00ebm@03000P1B0500lP1@8000IJo005001J=5XdFS@0000BFS@0
0`00Oomoo`0>Ool30007Ool00`00Oomoo`0:Ool000Uoo`03001oogoo02Aoo`03001oogoo00Eoo`80
009=m08001AI:003000LmAce00TLm@03001bmG;e00abm@03000P1B0500hP1@8000QJo005001J=5Xd
FS@0000BFS@00`00Oomoo`0;Ool3000;Ool00`00Oomoo`09Ool000Uoo`03001oogoo02Eoo`03001o
ogoo009oo`80009=m08001MI:003000LmAce00PLm@03001bmG;e00]bm@03000P1B0500dP1@8000YJ
o005001J=5XdFS@0000AFS@00`00Oomoo`09Ool3000>Ool00`00Oomoo`09Ool000Uoo`03001oogoo
02Ioo`@0009=m08001YI:003000LmAce00LLm@03001bmG;e00]bm@03000P1B0500/P1@8000aJo005
001J=5XdFS@0000@FS@00`00Oomoo`07Ool3000AOol00`00Oomoo`09Ool000Qoo`03001oogoo02Io
o`8000==m08001eI:003000LmAce00HLm@03001bmG;e00Ybm@03000P1B0500XP1@8000iJo005001J
=5XdFS@0000@FS@00`00Oomoo`04Ool3000DOol00`00Oomoo`09Ool000Qoo`03001oogoo02Aoo`80
00==m08001mI:003000LmAce00HLm@03001bmG;e00Ubm@03000P1B0500TP1@80015Jo005001J=5Xd
FS@0000>FS@00`00Oomoo`02Ool3000GOol00`00Oomoo`09Ool000Qoo`03001oogoo025oo`<000A=
m0<0021I:003000LmAce00HLm@03001bmG;e00Mbm@03000P1B0500TP1@03001Jo5[l015Jo005001J
=5XdFS@0000=FS@00`00Ool00003000JOol00`00Oomoo`09Ool000Qoo`03001oogoo01moo`8000M=
m003001Bg5;L00@001eI:003000LmAce00DLm@03001bmG;e00Ibm@03000P1B0500PP1@8001AJo005
001J=5XdFS@0000<FS@3000NOol00`00Oomoo`09Ool000Qoo`03001oogoo01eoo`8000Y=m003001B
g5;L00=Bg0@001YI:003000LmAce00@Lm@03001bmG;e00Ebm@03000P1B0500LP1@8001IJo005001J
=5XdFS@0000<FS@00`00Oomoo`0NOol00`00Oomoo`09Ool000Qoo`03001oogoo01]oo`8000a=m003
001Bg5;L00MBg0@001MI:003000LmAce00<Lm@03001bmG;e00Ebm@03000P1B0500DP1@8001QJo005
001J=5XdFS@0000;FS@00`00Oomoo`0POol00`00Oomoo`08Ool000Moo`03001oogoo01eoo`03001=
m4gd00]=m003001Bg5;L00YBg0@001AI:003000LmAce008Lm@03001bmG;e00Abm@03000P1B0500@P
1@8001YJo005001J=5XdFS@0000:FS@00`00Oomoo`0QOol00`00Oomoo`08Ool000Moo`03001oogoo
01ioo`03001=m4gd00Y=m003001Bg5;L00iBg0@0015I:005000LmAce7?D00005L_D00`0080DP1@03
80D2000LF_`01@00FSAJ=5Xd00002UXd00<007ooOol08Goo00<007ooOol027oo0007Ool00`00Oomo
o`0OOol00`00COA=m00:CO@00`00D]aBg00AD]`4000>FBP010007?DLm@0017;e00<0020580D00R05
0P007U[l00D005XdFSAJ=00000UJ=003001oogoo029oo`03001oogoo00Qoo`001goo00<007ooOol0
87oo00<004gdCO@02Dgd00<005;LD]`05E;L10002UTX00@001ce7?D000=bm@04000P1B0580D2000P
F_`01@00FSAJ=5Xd000025Xd00<007ooOol08goo00<007ooOol027oo0007Ool00`00Oomoo`0QOol0
0`00COA=m009CO@00`00D]aBg00HD]`40007FBP01P007?DLm@00L_D0008P1@80029Jo003001J=5Xd
008000MJ=003001oogoo02Aoo`03001oogoo00Qoo`001goo00<007ooOol08Woo00<004gdCO@024gd
00<005;LD]`075;L100015TX00@001ce001bm@<002AJo0<0009J=003001J=5Xd00AJ=003001oogoo
02Aoo`03001oogoo00Qoo`001goo00<007ooOol08goo00<004gdCO@024gd00<005;LD]`07e;L1000
00=I:00000000P008e[l10001EXd00<005XdFS@00UXd00<007ooOol09Woo00<007ooOol01goo0006
Ool00`00Oomoo`0UOol00`00COA=m007CO@00`00D]aBg00SD]`4000QF_`30003F_`00`00FSAJ=003
FS@01@00FSAJ=5Xd0000:Goo00<007ooOol01goo0006Ool00`00Oomoo`0VOol00`00COA=m006CO@0
0`00D]aBg00SD]`7000JF_`40006F_`00`00FSAJ=004FS@01000FSAJ=000:Goo00<007ooOol01goo
0006Ool00`00Oomoo`0WOol00`00COA=m006CO@00`00D]aBg00QD]`01P0050L000009GL0009UZP@0
01=Jo0<000YJo003001J=5Xd00EJ=08002Yoo`03001oogoo00Moo`001Woo00<007ooOol0:7oo00<0
04gdCO@01Dgd00<005;LD]`07e;L0P0000@D1`00LaL0008UM`03001UZVFZ00=UZP<000eJo0<000eJ
o003001J=5Xd00EJ=0@002Qoo`03001oogoo00Moo`001Woo00<007ooOol0:Goo00<004gdCO@01Dgd
00<005;LD]`07E;L00@001@750L000=c5`03000UM`0000QUZP<000MJo0<0011Jo003001J=5Xd00AJ
=006001UIFEU001UI@009goo00<007ooOol01goo0006Ool00`00Oomoo`0ZOol00`00COA=m004CO@0
0`00D]aBg00KD]`2000250L00`00LaMc5`02LaL010009GLUM`002VFZ0`0000AJo000000001AJo003
001J=5Xd00=J=004001UIFEU0002IFD00`00Oomoo`0TOol00`00Oomoo`07Ool000Eoo`03001oogoo
02]oo`03001=m4gd00E=m003001Bg5;L01QBg08000@D1`03001c5g<G009c5`05000UMbEg9GL0000:
IJX5000EF_`00`00FSAJ=002FS@00`00IFEUI@02IFD01000IFEUI@009Woo00<007ooOol01Woo0005
Ool00`00Oomoo`0/Ool00`00COA=m004CO@00`00D]aBg00GD]`00`0050LD1`0350L00`00LaMc5`03
LaL00`009GLUM`029GL00`00IJYUZP04IJX30005IJX4000AF_`01@00FSAJ=5Xd00001VEU00@006EU
IFD002Eoo`03001oogoo00Ioo`001Goo00<007ooOol0;Goo00<004gdCO@014gd00<005;LD]`055;L
0P001A@700<007<GLaL017<G00<002Eg9GL00bEg00<006FZ00000`0036FZ0`003U[l00D005XdFSAJ
=00000MUI@04001UIFEU000TOol00`00Oomoo`06Ool000Eoo`03001oogoo02ioo`03001=m4gd00==
m003001Bg5;L01=Bg003000D1a@700DD1`03001c5g<G00Ac5`03000UMbEg008UM`<001=UZP<000]J
o004001J=5Xd0008IFD01@00IFEUIFEU00008goo00<007ooOol01Woo0005Ool00`00Oomoo`0_Ool0
0`00COA=m002CO@00`00D]aBg00AD]`2000750L00`00LaMc5`06LaL400039GL00`00IJYUZP0CIJX3
0008F_`00`00FS@0000:IFD01@00IFEUIFEU00008Woo00<007ooOol01Woo0005Ool00`00Oomoo`0`
Ool00`00COA=m002CO@00`00D]aBg00?D]`00`0050LD1`0650L00`00LaMc5`05LaL300079GL00`00
IJYUZP0EIJX40004F_`2000<IFD01@00IFEUIFEU00008Goo00<007ooOol01Woo0005Ool00`00Oomo
o`0aOol01@00COA=m4gd00003e;L0P002A@700<007<GLaL00W<G0`000W<G00<002Eg9GL01BEg00<0
06FZIJX06FFZ0`0000=Jo00000003FEU00D006EUIFEUI@00021oo`03001oogoo00Ioo`0017oo00<0
07ooOol0<goo00D004gdCOA=m00000eBg003000D1a@700PD1`03001c5g<G00<000Ec5`03000UMbEg
00HUM`03001UZVFZ01]UZP<000iUI@05001UIFEUIFD0000OOol00`00Oomoo`06Ool000Aoo`03001o
ogoo03Aoo`04001=m4gd000;D]`2000:50L40008LaL00`009GLUM`079GL00`00IJYUZP0JIJX20000
15]M000000002fEU00<006EUIFD00VEU00<007ooOol07Goo00<007ooOol01Goo0004Ool00`00Oomo
o`0eOol01000COA=m00025;L0P002Q@70`002g<G00<002Eg9GL022Eg00<006FZIJX06FFZ0P0015]M
0`002FEU00<006EUIFD00VEU00<007ooOol077oo00<007ooOol01Goo0004Ool00`00Oomoo`0fOol0
0`00CO@00007D]`00`0050LD1`0650L400000a@7001c5`0<LaL00`009GLUM`089GL00`00IJYUZP0G
IJX01000FUIJEP001U]M0`001fEU00<006EUIFD00VEU0P0077oo00<007ooOol01Goo0004Ool00`00
Oomoo`0fOol200000dgd001Bg003D]`2000650L3000450L00`00LaMc5`0<LaL00`009GLUM`099GL0
0`00IJYUZP0EIJX01@00FUIJEUYF00002E]M0`001FEU00<006EUIFD00fEU00<007ooOol06Goo00<0
07ooOol01Goo0004Ool00`00Oomoo`0fOol40003D]`00`0050LD1`0350L3000650L00`00LaMc5`0=
LaL00`009GLUM`0:9GL00`00IJYUZP0CIJX00`00FUIJEP02FUH00`00FeeKG@0:Fed40002IFD00`00
IFEUI@03IFD00`00Oomoo`0HOol00`00Oomoo`05Ool000=oo`03001oogoo03Ioo`P000<D1`<000TD
1`03001c5g<G00ec5`03000UMbEg00/UM`03001UZVFZ019UZP03001JEUYF009JEP03001KGE]M00iK
G@<000IUI@03001oogoo01Moo`03001oogoo00Eoo`000goo00<007ooOol0=Goo1`000Q@70`002a@7
00<007<GLaL03W<G00<002Eg9GL02bEg00<006FZIJX04FFZ00<005YFFUH00eYF00<005]MFed04E]M
0`0016EU00<007ooOol05Woo00<007ooOol01Goo0003Ool00`00Oomoo`0dOol:000=50L00`00LaMc
5`0?LaL00`009GLUM`0<9GL00`00IJYUZP0?IJX00`00FUIJEP04FUH00`00FeeKG@0BFed00`00Fed0
00020002IFD00`00Oomoo`0FOol00`00Oomoo`04Ool000=oo`03001oogoo03Aoo`T000hD1`03001c
5g<G00mc5`03000UMbEg00dUM`03001UZVFZ00eUZP03001JEUYF00IJEP03001KGE]M019KG@04001K
GE]MFed4000GOol00`00Oomoo`04Ool000=oo`03001oogoo03=oo`H00096oP8000dD1`03001c5g<G
015c5`03000UMbEg00dUM`03001UZVFZ00aUZP03001JEUYF00IJEP03001KGE]M01=KG@03001KGE]M
00=KG@8001Ioo`03001oogoo00Aoo`000goo00<007ooOol0<Goo1`000dKn0P0031@700<007<GLaL0
4W<G00<002Eg9GL03REg00<006FZIJX02VFZ00<005YFFUH01eYF00<005]MFed04e]M00@005]MFeeK
G@8001Qoo`03001oogoo00Aoo`000goo00<007ooOol0;Woo20001DKn0P002a@700<007<GLaL04g<G
00<002Eg9GL03bEg00<006FZIJX026FZ00<005YFFUH025YF00<005]MFed055]M0`006Woo00<007oo
Ool017oo0002Ool00`00Oomoo`0/Ool30003Ool40006A_h00`00DQH0000:50L00`00LaMc5`0CLaL0
0`009GLUM`0@9GL00`00IJYUZP06IJX00`00FUIJEP09FUH00`00FeeKG@0AFed300000goo001oo`0J
Ool00`00Oomoo`04Ool0009oo`03001oogoo02Uoo`<000Eoo`@000M6oP03001B5P0000TD1`03001c
5g<G01Ac5`03000UMbEg014UM`03001UZVFZ00EUZP03001JEUYF00YJEP03001KGE]M00iKG@8000Eo
o`03001oogoo01Qoo`03001oogoo00Aoo`000Woo00<007ooOol09Woo0`001goo100024Kn00<0058F
000021@700<007<GLaL05G<G00<002Eg9GL04BEg00<006FZIJX016FZ00<005YFFUH02eYF00<005]M
Fed02e]M0`0027oo00<007ooOol05goo00<007ooOol017oo0002Ool00`00Oomoo`0SOol30009Ool4
0009A_h01000DQIB5P001a@700<007<GLaL05W<G00<002Eg9GL04BEg00<006FZIJX00VFZ00<005YF
FUH035YF00<005]MFed02E]M0P002goo00<007ooOol067oo00<007ooOol00goo0002Ool00`00Oomo
o`0POol3000<Ool3000:A_h01000DQIB5P001Q@700<007<GLaL05g<G00<002Eg9GL04REg00@006FZ
IJX000mJEP03001KGE]M00IKG@<000ioo`03001oogoo01Moo`03001oogoo00=oo`000Woo00<007oo
Ool077oo10003Woo0`002dKn00@0058FDQH000DD1`03001c5g<G01Qc5`03000UMbEg01<UM`03001U
ZP0000mJEP03001KGE]M00=KG@<0019oo`03001oogoo01Ioo`03001oogoo00=oo`000Woo00<007oo
Ool06Goo0`004Goo0P003DKn00D0058FDQIB5P0000@D1`03001c5g<G01Qc5`03000UMbEg01@UM`03
001JEUYF00iJEP04001KGE]MFed2000FOol00`00Oomoo`0EOol00`00Oomoo`03Ool00003Ool007oo
01Qoo`<001=oo`8000i6oP05001B5U8FDQH0000350L00`00LaMc5`0ILaL00`009GLUM`0B9GL2000B
FUH3000HOol00`00Oomoo`0EOol00`00Oomoo`03Ool00003Ool007oo01Eoo`<001Ioo`030016oTKn
00e6oP05001B5U8FDQH0000250L00`00LaMc5`0JLaL00`009GLUM`0@9GL2000DFUH00`00Oomoo`0I
Ool00`00Oomoo`0DOol00`00Oomoo`03Ool00003Ool007oo019oo`<001Qoo`<000i6oP07001B5U8F
DQH001@700007W<G00<002Eg9GL03BEg0P005UYF00<007ooOol06Woo00<007ooOol04goo00<007oo
Ool00goo00000goo001oo`0?Ool3000NOol60008A_h00`00DQIB5P02DQH2000NLaL00`009GLUM`0<
9GL00`00FUIJEP0FFUH00`00Oomoo`0KOol00`00Oomoo`0COol00`00Oomoo`02Ool00003Ool007oo
00aoo`<002Moo`H00096oP03001B5U8F009B5P<001ec5`03000UMbEg00XUM`8001UJEP03001oogoo
01aoo`03001oogoo019oo`03001oogoo009oo`0000=oo`00Ool027oo1000<7oo0`001e8F1@0067<G
00<002Eg9GL022Eg0P0075YF00<007ooOol06goo00<007ooOol04Woo00<007ooOol00Woo00000`00
Oomoo`05Ool3000fOol00`00DQIB5P0:DQH4000DLaL00`009GLUM`069GL2000NFUH00`00Oomoo`0L
Ool00`00Oomoo`0AOol00`00Oomoo`02Ool00003001oogoo009oo`<003Uoo`03001B5U8F00iB5PD0
00mc5`03000UMbEg00DUM`03001JEUYF01iJEP03001oogoo01eoo`03001oogoo011oo`03001oogoo
009oo`0000<007oo00000P00?7oo00<0058FDQH04e8F1@002g<G00<002Eg9GL00REg0P008EYF00<0
07ooOol07Woo00<007ooOol03goo00<007ooOol00Woo0002000oOol00`00DQIB5P0HDQH40007LaL0
0`009GLUM`02000SFUH00`00Oomoo`0OOol00`00Oomoo`0>Ool00`00Oomoo`02Ool00003Ool00000
03moo`03001B5U8F01]B5PD0009c5`<002EJEP03001oogoo01moo`03001oogoo00ioo`03001oogoo
009oo`000goo00<007ooOol0?7oo00<0058FDQH0858F10009eYF00<007ooOol07goo00<007ooOol0
3Woo00<007ooOol00Goo0004Ool00`00Oomoo`0kOol00`00DQIB5P0PDQH3000017MG0000000095YF
00<007ooOol087oo00<007ooOol03Goo00<007ooOol00Goo0005Ool2000kOol00`00DQIB5P0NDQH2
00032AL01000MeMgEgMG100085YF00<007ooOol08Goo00<007ooOol037oo00<007ooOol00Goo0007
Ool00`00Oomoo`0hOol00`00DQIB5P0LDQH200052AL00`00MeMgE`05MeL3000MFUH00`00Oomoo`0R
Ool00`00Oomoo`0;Ool00`00Oomoo`01Ool000Qoo`03001oogoo03Moo`03001B5U8F01YB5P8000L9
5`03001gEgMG00QgE`<001YJEP03001oogoo029oo`03001oogoo00]oo`03001oogoo005oo`002Goo
0P00=goo00<0058FDQH0658F0P002@TG00<007MGMeL02gMG10005eYF00<007ooOol08Woo00<007oo
Ool02Woo00<007ooOol00Goo000;Ool00`00Oomoo`0dOol00`00DQIB5P0FDQH200000goo00095`09
2AL00`00MeMgE`0>MeL3000DFUH00`00Oomoo`0SOol00`00Oomoo`09Ool00`00Oomoo`01Ool000ao
o`03001oogoo03=oo`03001B5U8F01EB5P05001oogooOol0000:2AL00`00MeMgE`0?MeL00`00Ool0
0002000AFUH00`00Oomoo`0TOol00`00Oomoo`09Ool10001Ool1Ool000eoo`8003=oo`03001B5U8F
01=B5P8000Eoo`0300095`TG00L95`03001gEgMG00mgE`03001oogoo009oo`@000eJEP03001oogoo
02Eoo`03001oogoo00Qoo`40005oo`5oo`003goo00<007ooOol0<7oo00<0058FDQH04E8F0P0027oo
00<000TG2AL01PTG00<007MGMeL03WMG00<007ooOol01goo0`002UYF00<007ooOol09Goo00<007oo
Ool027oo0@000Goo0Goo000@Ool00`00Oomoo`0_Ool00`00DQIB5P0?DQH2000;Ool00`002AL95`06
2AL00`00MeMgE`0=MeL00`00Oomoo`0:Ool30007FUH00`00Oomoo`0VOol00`00Oomoo`07Ool10001
Ool1Ool0015oo`8002moo`03001B5U8F00eB5P8000eoo`0300095`TG00H95`03001gEgMG00agE`03
001oogoo00ioo`@000AJEP03001oogoo02Ioo`03001oogoo00Ioo`40005oo`5oo`004goo00<007oo
Ool0;7oo00<0058FDQH02e8F0P0047oo00<000TG2AL01@TG00<007MGMeL02gMG00<007ooOol04goo
0`0000=JEP00Ool0:7oo00<007ooOol01Goo0@000Goo0Goo000DOol2000/Ool00`00DQIB5P0:DQH0
0`00Oomoo`0AOol00`002AL95`042AL00`00MeMgE`0;MeL00`00Oomoo`0FOol2000YOol00`00Oomo
o`05Ool10001Ool1Ool001Ioo`03001oogoo02Uoo`03001B5U8F00QB5P8001Aoo`0300095`TG00D9
5`03001gEgMG00UgE`03001oogoo04=oo`03001oogoo00Eoo`40005oo`005goo00<007ooOol0:7oo
00<0058FDQH01U8F0`005Woo00<000TG2AL010TG00<007MGMeL02GMG00<007ooOol0A7oo00<007oo
Ool017oo0@000Goo000HOol2000XOol00`00DQIB5P04DQH20002Ool00`00Oomoo`0EOol00`002AL9
5`032AL00`00MeMgE`08MeL00`00Oomoo`16Ool00`00Oomoo`03Ool10001Ool001Yoo`03001oogoo
02Eoo`03001B5U8F009B5P8000Aoo`03001oogoo01Ioo`0300095`TG00<95`03001gEgMG00MgE`03
001oogoo04Moo`03001oogoo009oo`40005oo`006goo00<007ooOol097oo00<0058FDQH00P001Woo
00<007ooOol05Woo00<000TG2AL00`TG00<007MGMeL01WMG00<007ooOol0B7oo00<007ooOol00Woo
0@000Goo000LOol2000TOol30008Ool00`00Oomoo`0GOol00`002AL95`022AL00`00MeMgE`06MeL0
0`00Oomoo`19Ool01@00Oomoogoo00000Goo000NOol00`00Oomoo`0QOol00`00Oomoo`08Ool00`00
Oomoo`0HOol01@002AL95`TG00001gMG00<007ooOol0Bgoo00@007ooOol0005oo`007goo00<007oo
Ool0:goo00<007ooOol06Goo00D000TG2AL95`0000IgE`03001oogoo04aoo`03001oo`00005oo`00
87oo0P00:goo00<007ooOol06Goo00D000TG2AL95`0000EgE`03001oogoo04ioo`40005oo`400000
8Woo00<007ooOol0:7oo00<007ooOol06Woo00@000TG2AL000EgE`03001oogoo04ioo`40005oo`40
00008goo00<007ooOol09goo00<007ooOol06goo00<000TG000017MG00<007ooOol0D7oo0P00000T
Ool2000WOol00`00Oomoo`0KOol010002AL95`000gMG00<007ooOol0Cgoo0`00000VOol00`00Oomo
o`0TOol00`00Oomoo`0LOol00`002AL00002MeL00`00Oomoo`1<Ool40003Ool002Moo`8002Aoo`03
001oogoo01eoo`80009gE`03001oogoo04Qoo`@000Moo`00:Goo00<007ooOol08Goo00<007ooOol0
7Woo00<007MG0000Agoo10002goo000ZOol00`00Oomoo`0POol00`00Oomoo`0NOol30013Ool4000?
Ool002]oo`80021oo`03001oogoo01moo`03001oogoo03eoo`D001=oo`00;Goo00<007ooOol07Goo
00<007ooOol0Fgoo100067oo000^Ool00`00Oomoo`0LOol00`00Oomoo`1GOol4000LOol002moo`80
01aoo`03001oogoo05=oo`@0021oo`00<Goo00<007ooOol06Goo00<007ooOol0Cgoo100097oo000b
Ool00`00Oomoo`0HOol00`00Oomoo`1:Ool5000XOol003=oo`8001Qoo`03001oogoo04Ioo`@002eo
o`00=Goo00<007ooOol05Goo00<007ooOol0@Woo1000<Goo000fOol00`00Oomoo`0DOol00`00Oomo
o`0nOol4000eOol003Moo`8001Aoo`03001oogoo03Yoo`@003Uoo`00>Goo00<007ooOol04Goo00<0
07ooOol0=Goo1@00?Goo000jOol2000@Ool00`00Oomoo`0bOol40012Ool003aoo`03001oogoo00eo
o`03001oogoo02ioo`@004Ioo`00?Goo00<007ooOol037oo00<007ooOol0:Woo1000BWoo000nOol2
000<Ool00`00Oomoo`0UOol5001>Ool0041oo`03001oogoo00Uoo`03001oogoo025oo`@005=oo`00
@Goo00<007ooOol027oo00<007ooOol07Goo1000Egoo0012Ool20008Ool00`00Oomoo`0IOol4001K
Ool004Aoo`03001oogoo00Eoo`03001oogoo01Eoo`@005moo`00AGoo00<007ooOol017oo00<007oo
Ool047oo1@00Hgoo0016Ool20004Ool00`00Oomoo`0<Ool4001XOol004Qoo`05001oogooOol0000:
Ool4001/Ool004Uoo`04001oogoo0006Ool4001`Ool004Yoo`<0009oo`@007Aoo`00C7oo0`00N7oo
0000\
\>"],
  ImageRangeCache->{{{164.938, 287.625}, {608.5, 485.688}} -> {-1.21816, \
3.64227, 0.00464951, 0.00464951}},
  CellTags->{"b:46.3", "IPM3.10.5.4"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Example: an evaluate button", "Subsection",
  CellTags->{"b:46.4", "IPM3.10.5.5", "c:21"}],

Cell[TextData[{
  "At the end of Section 10.2 we created a function ",
  StyleBox["EvaluateNext", "MR"],
  ", which evaluated the immediately following input cell. In this section we \
will turn the code from that function into a button."
}], "TextFirst",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["Here was the code we developed in that section.", "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["\<\
EvaluateNext:=(
    SelectionMove[EvaluationNotebook[],All,Cell];
    SelectionMove[EvaluationNotebook[],Next,Cell];
    SelectionEvaluate[EvaluationNotebook[]];
    )\
\>", "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[TextData[{
  "To put this code inside a button, we need to make a few modifications. \
First, remember that the front end does not know how to parse shorthand \
notation such as ",
  StyleBox[";", "MR"],
  ". Instead we need to use ",
  StyleBox["CompoundExpression", "MR"],
  ". Second, instead of ",
  StyleBox["EvaluationNotebook", "MR"],
  ", we will use ",
  StyleBox["ButtonNotebook", "MR"],
  ", which gives the notebook in which the current button lives. Finally, we \
need to use ",
  StyleBox["ButtonCell", "MR"],
  " to refer to the cell containing the button itself. Putting all these \
pieces together, here is the ",
  StyleBox["ButtonFunction", "MR"],
  "."
}], "Text",
  CellTags->{"i:44", "b:46.4", "IPM3.10.5.5"}],

Cell[BoxData[
    \(\(ButtonFunction \[RuleDelayed] 
        CompoundExpression[{SelectionMove[ButtonNotebook[], All, ButtonCell], 
            SelectionMove[ButtonNotebook[], Next, Cell], 
            SelectionEvaluate[ButtonNotebook[]]}];\)\)], "Input",
  CellLabel->"In[13]:=",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["Here then is the code to generate our evaluate button.", "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["\<\
Cell[TextData[{
  Cell[BoxData[
      ButtonBox[\"EVALUATE\",
        ButtonFunction:>CompoundExpression[ {
            SelectionMove[ 
              ButtonNotebook[ ], All, ButtonCell], 
            SelectionMove[ 
              ButtonNotebook[ ], Next, Cell], 
            SelectionEvaluate[ 
              ButtonNotebook[ ]]}],
        Active->True]]],
  \" MATHEMATICA INPUT\"
}], \"Text\"]\
\>", "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["\<\
And here is the formatted button. Clicking the Evaluate button causes the \
cell just below the button cell to be evaluated.\
\>", "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[TextData[{
  Cell[BoxData[
      ButtonBox["EVALUATE",
        ButtonFunction:>CompoundExpression[ {
            SelectionMove[ 
              ButtonNotebook[ ], All, ButtonCell], 
            SelectionMove[ 
              ButtonNotebook[ ], Next, Cell], 
            SelectionEvaluate[ 
              ButtonNotebook[ ]]}],
        Active->True]]],
  " MATHEMATICA INPUT"
}], "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(2 + 2\)], "Input",
  CellLabel->"In[14]:=",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[BoxData[
    \(4\)], "Output",
  CellLabel->"Out[14]=",
  CellTags->{"b:46.4", "IPM3.10.5.5"}]
}, Open  ]],

Cell["\<\
Finally, let us add some formatting to make this cell a little nicer looking.\
\
\>", "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["\<\
Cell[TextData[{
  Cell[BoxData[
      ButtonBox[
        StyleBox[\"EVALUATE\",
          FontFamily->\"Helvetica\",
          FontSize->10,
          FontWeight->\"Bold\"],
        ButtonFunction:>CompoundExpression[ {
            SelectionMove[ 
              ButtonNotebook[ ], All, ButtonCell], 
            SelectionMove[ 
              ButtonNotebook[ ], Next, Cell], 
            SelectionEvaluate[ 
              ButtonNotebook[ ]]}],
        Active->True,
        Background->GrayLevel[0.500008]]]],
  StyleBox[\" MATHEMATICA INPUT\",
    FontFamily->\"Helvetica\",
    FontSize->10,
    FontWeight->\"Bold\",
    FontSlant->\"Italic\",
    FontColor->GrayLevel[1]]
}], \"Text\",
  Background->GrayLevel[0.500008]]\
\>", "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["\<\
Here is the formatted version of this code with the result of clicking the \
button.\
\>", "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[TextData[{
  Cell[BoxData[
      ButtonBox[
        StyleBox["EVALUATE",
          FontFamily->"Helvetica",
          FontSize->9,
          FontWeight->"Bold"],
        ButtonFunction:>CompoundExpression[ {
            SelectionMove[ 
              ButtonNotebook[ ], All, ButtonCell], 
            SelectionMove[ 
              ButtonNotebook[ ], Next, Cell], 
            SelectionEvaluate[ 
              ButtonNotebook[ ]]}],
        Active->True,
        Background->GrayLevel[0.500008]]]],
  StyleBox[" MATHEMATICA INPUT",
    FontFamily->"Helvetica",
    FontSize->9,
    FontWeight->"Bold",
    FontSlant->"Italic",
    FontColor->GrayLevel[1]]
}], "Text",
  CellMargins->{{Inherited, 90}, {6, 6}},
  Background->GrayLevel[0.500008],
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[CellGroupData[{

Cell[BoxData[
    \(2 + 5\)], "Input",
  CellLabel->"In[15]:=",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[BoxData[
    \(7\)], "Output",
  CellLabel->"Out[15]=",
  CellTags->{"b:46.4", "IPM3.10.5.5"}]
}, Open  ]],

Cell[TextData[{
  "There is a little inefficiency in our code as we are calling the kernel \
several times (two instances of ",
  StyleBox["SelectionMove", "MR"],
  " and one of ",
  StyleBox["SelectionEvaluate", "MR"],
  ") for what are essentially front end operations, moving and selecting. You \
can send these sorts of commands directly to the front end by wrapping them \
in ",
  StyleBox["FrontEndExecute", "MR"],
  ". To distinguish between the kernel command and the front end command you \
also need to append the ",
  StyleBox["FrontEnd`", "MR"],
  " context to the function. So for example, instead of using ",
  StyleBox["SelectionMove[\[Ellipsis]]", "MR"],
  " in the kernel, you can send it directly to the front end with the \
following."
}], "Text",
  CellTags->{"i:45", "b:46.4", "IPM3.10.5.5"}],

Cell["\<\
FrontEndExecute[FrontEnd`SelectionMove[\[Ellipsis]]]\
\>", "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[TextData[{
  "With this in mind, the EVALUATE button can be rewritten by only changing \
the ",
  StyleBox["ButtonFunction", "MR"],
  "."
}], "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["\<\
ButtonFunction:>FrontEndExecute[ {
\tFrontEnd`SelectionMove[ 
\t\tButtonNotebook[ ], All, ButtonCell], 
\tFrontEnd`SelectionMove[ 
\t\tButtonNotebook[ ], Next, Cell],
\tFrontEnd`SelectionEvaluate[ 
\t\tButtonNotebook[ ]]}]\
\>", "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell["\<\
Another method of directly accessing front end commands is via front end \
tokens. These tokens allow you to perform any menu command directly from the \
kernel. We will not discuss them here, but for a detailed discussion of front \
end tokens, see the Front End category of the Help Browser.\
\>", "Text",
  CellTags->{"i:46", "b:46.4", "IPM3.10.5.5"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Exercises", "ExerciseSubsection",
  CellTags->{"b:46.4", "IPM3.10.5.6", "c:22"}],

Cell[TextData[{
  "Create a button that will serve as a template for the ",
  StyleBox["Plot3D", "MR"],
  " function."
}], "ExerciseItem",
  CellTags->{"b:46.4", "IPM3.10.5.6"}],

Cell[TextData[{
  "Create a button that will wrap ",
  StyleBox["Expand[]", "MR"],
  " around any selected expression and evaluate that expression."
}], "ExerciseItem",
  CellTags->{"b:46.4", "IPM3.10.5.6"}],

Cell[TextData[{
  "Using ",
  StyleBox["GridBox", "MR"],
  ", create a palette of buttons that operate on polynomials like that in \
Exercise 2. Include in your palette a button for each of ",
  StyleBox["Expand", "MR"],
  ", ",
  StyleBox["Factor", "MR"],
  ", ",
  StyleBox["Apart", "MR"],
  ", and ",
  StyleBox["Together", "MR"],
  "."
}], "ExerciseItem",
  CellTags->{"b:46.4", "IPM3.10.5.6"}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 685}},
AutoGeneratedPackage->None,
ScreenStyleEnvironment->"Working",
PrintingStyleEnvironment->"Production",
ShowPageBreaks->False,
WindowSize->{620, 592},
WindowMargins->{{3, Automatic}, {Automatic, 1}},
PrintingCopies->1,
PrintingStartingPageNumber->311,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"An Introduction to Programming with ", 
          StyleBox[ "Mathematica", "IT"]}], "Header"]}, {Cell[ 
        TextData[ {"10 Front end programming"}], "Header"], Inherited, 
      Inherited}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
TaggingRules:>{"NextIndexingNumber" -> 47, "IndexEntries" -> {{"i:1", 
  StyleBox[ "Notebook", "MR"]}, {"i:10", 
  StyleBox[ "NotebookWrite", "MR"]}, {"i:11", 
  StyleBox[ "Cell", "MR"]}, {"i:13", 
  StyleBox[ "SelectionMove", "MR"]}, {"i:14", 
  StyleBox[ "SelectionEvaluate", "MR"]}, {"i:15", 
  StyleBox[ "EvaluationNotebook", "MR"]}, {"i:16", 
  StyleBox[ "TextData", "MR"]}, {"i:17", 
  StyleBox[ "ValueBox", "MR"]}, {"i:18", 
  StyleBox[ "Date", "MR"]}, {"i:19", 
  StyleBox[ "BoxData", "MR"]}, {"i:2", 
  StyleBox[ "NotebookPut", "MR"]}, {"i:20", 
  StyleBox[ "SuperscriptBox", "MR"]}, {"i:21", 
  StyleBox[ "SqrtBox", "MR"]}, {"i:22", 
  StyleBox[ "FractionBox", "MR"]}, {"i:23", 
  StyleBox[ "SubsuperscriptBox", "MR"]}, {"i:24", 
  StyleBox[ "GraphicsData", "MR"]}, {"i:25", "Display channels"}, {"i:27", 
  StyleBox[ 
  "DisplayForm", "MR"]}, {"i:3", "Handles"}, {"i:30", "Rep units"}, {"i:36", 
  StyleBox[ "ButtonBox", "MR"]}, {"i:40", 
  StyleBox[ "ButtonStyle", "MR"]}, {"i:41", 
  StyleBox[ "ButtonData", "MR"]}, {"i:42", 
  StyleBox[ "ButtonEvaluator", "MR"]}, {"i:42", 
  StyleBox[ "ButtonFunction", "MR"]}, {"i:44", 
  StyleBox[ "ButtonCell", "MR"]}, {"i:44", 
  StyleBox[ "ButtonNotebook", "MR"]}, {"i:45", 
  StyleBox[ "FrontEndExecute", "MR"]}, {"i:46", "Front end tokens"}, {"i:5", 
  StyleBox[ "NotebookGet", "MR"]}, {"i:8", 
  StyleBox[ "Notebooks", "MR"]}, {"i:9", 
  StyleBox[ 
  "NotebookCreate", 
    "MR"]}, {"i:13", "Notebook expressions", "moving around within"}, \
{"i:14", "Notebook expressions", "evaluating selections"}, {"i:16", "Cell \
expressions", 
  StyleBox[ 
  "TextData", 
    "MR"]}, {"i:17", "Cell expressions", "embedding evaluations"}, {"i:19", \
"Boxes", "superscripts"}, {"i:19", "Cell expressions", 
  StyleBox[ "BoxData", "MR"]}, {"i:21", "Boxes", 
  StyleBox[ "SqrtBox", "MR"]}, {"i:22", "Boxes", 
  StyleBox[ "FractionBox", "MR"]}, {"i:23", "Boxes", 
  StyleBox[ 
  "SubsuperscriptBox", 
    "MR"]}, {"i:25", "Graphics", "displaying to new window"}, {"i:27", {
  StyleBox[ "GridBox", "MR"], "es"}, "displaying"}, {"i:28", {
  StyleBox[ "GridBox", "MR"], "es"}, "options"}, {"i:29", {
  StyleBox[ "GridBox", "MR"], "es"}, "formatting"}, {"i:30", "Programs", 
  StyleBox[ 
  "RepUnit", 
    "MR"]}, {"i:31", "Pascal\[CloseCurlyQuote]s triangle", "displaying \
traditionallly"}, {"i:32", "Programs", 
  StyleBox[ "PascalTable", "MR"]}, {"i:33", "Programs", 
  StyleBox[ "TriangleForm", "MR"]}, {"i:34", "Programs", 
  StyleBox[ 
  "TruthTable", 
    "MR"]}, {"i:34", "Truth tables", " constructing"}, {"i:35", "Buttons", \
"creating from menus"}, {"i:36", "Buttons", "structure"}, {"i:37", "Buttons", \
"activating"}, {"i:38", "Buttons", "as templates"}, {"i:39", "Buttons", \
"placeholders"}, {"i:40", "Buttons", "actions"}, {"i:41", "Buttons", \
"hyperlinks"}, {"i:41", "Hyperlinks", "creating"}, {"i:42", "Buttons", \
"embedding code"}, {"i:42", "Buttons", "evaluation options"}, {"i:43", \
"Buttons", "front end parsing"}, {"i:45", "Buttons", "using front end \
commands"}, {"i:1", "Notebook expressions", "structure", ""}, {"i:11", "Cell \
expressions", "", ""}, {"i:12", "Cell expressions", "options", ""}, {"i:2", \
"Notebook expressions", "creating", ""}, {"i:24", "Cell expressions", 
  StyleBox[ "GraphicsData", "MR"], ""}, {"i:26", {
  StyleBox[ 
  "GridBox", 
    "MR"], "es"}, "structure", ""}, {"i:3", "Notebook expressions", "as \
objects", ""}, {"i:5", "Notebook expressions", "reading into kernel", ""}, \
{"i:6", "Notebook expressions", "options", ""}, {"i:7", "Notebook \
expressions", "manipulating", ""}, {"i:8", "Notebook expressions", "listing \
open", ""}}, "PaginationCache" -> {{2004, 9, 2, 17, 39, \
21.6195888`9.087422361388612}, {311, 311, 311, 311, 311, 311, 311, 311, 312, \
312, 312, 312, 312, 312, 312, 312, 313, 313, 313, 313, 313, 313, 313, 313, \
314, 314, 314, 314, 314, 314, 314, 314, 315, 315, 315, 315, 315, 315, 315, \
315, 315, 315, 315, 315, 316, 316, 316, 316, 316, 316, 316, 317, 317, 317, \
317, 317, 317, 317, 317, 318, 318, 318, 318, 318, 318, 318, 318, 318, 319, \
319, 319, 319, 319, 319, 320, 320, 320, 320, 320, 320, 320, 320, 320, 320, \
321, 321, 321, 321, 321, 321, 321, 321, 321, 321, 321, 321, 321, 321, 321, \
321, 321, 322, 322, 322, 322, 322, 322, 322, 322, 322, 322, 322, 322, 322, \
322, 323, 323, 323, 323, 323, 323, 323, 323, 323, 323, 323, 323, 323, 324, \
324, 324, 324, 324, 324, 324, 324, 324, 324, 324, 324, 324, 325, 325, 325, \
325, 325, 325, 325, 325, 325, 325, 325, 326, 326, 326, 326, 326, 326, 326, \
326, 326, 326, 326, 326, 326, 326, 326, 326, 327, 327, 327, 327, 327, 327, \
327, 327, 327, 327, 327, 328, 328, 328, 328, 328, 328, 328, 328, 328, 328, \
328, 329, 329, 329, 329, 329, 329, 329, 329, 329, 329, 329, 329, 329, 330, \
330, 330, 330, 330, 330, 330, 330, 330, 330, 330, 330, 330, 330, 330, 330, \
330, 330, 330, 331, 331, 331, 331, 331, 331, 331, 331, 331, 331, 331, 331, \
331, 331, 331, 331, 331, 332, 332, 332, 332, 332, 332, 332, 332, 332, 332, \
332, 332, 333, 333, 333, 333, 333, 333, 333, 333, 333, 333, 333, 333, 333, \
333, 333, 334, 334, 334, 334, 334, 334, 334, 334, 334, 334, 334, 334, 334, \
334, 334, 335, 335, 335, 335, 335, 335, 335, 335, 335, 335, 335, 335, 335, \
335, 335, 336, 336, 336, 336, 336, 336, 336, 336, 336, 336, 336, 337, 337, \
337, 337, 337, 337, 337, 337, 337, 338, 338, 338, 338, 338, 338, 338, 338, \
338, 339, 339, 339, 339, 339, 339, 339, 339, 339, 340, 340, 340, 340, 340, \
340, 340, 340, 341, 341, 341, 341, 341, 341, 341, 341, 341, 341, 342, 342, \
342, 342, 342, 342, 342, 342, 342, 342, 342, 342, 342, 342, 342}, \
Automatic}},
CellLabelAutoDelete->False,
Magnification->1,
StyleDefinitions -> "IPM3Styles.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "IPM3.10.0"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[2248, 65, 159, 6, 96, "Chapter",
      CounterAssignments->{{"Chapter", 9}},
      CellTags->{"IPM3.10.0", "c:1"}],
    Cell[2410, 73, 730, 17, 116, "ChapterAbstract",
      CellTags->"IPM3.10.0"]},
  "IPM3.10.1"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[3165, 94, 130, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 0}},
      CellTags->{"b:42", "IPM3.10.1", "c:2"}],
    Cell[3298, 98, 555, 11, 90, "TextFirst",
      CellTags->{"b:42", "IPM3.10.1"}],
    Cell[3856, 111, 439, 9, 71, "Text",
      CellTags->{"b:42", "IPM3.10.1"}],
    Cell[4298, 122, 789, 14, 128, "Text",
      CellTags->{"b:42", "IPM3.10.1"}],
    Cell[5090, 138, 336, 6, 71, "Text",
      CellTags->{"b:42", "IPM3.10.1"}],
    Cell[5429, 146, 1213, 24, 185, "Text",
      CellTags->{"b:42", "IPM3.10.1"}]},
  "IPM3.10.2.0"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[6679, 175, 196, 3, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 1}},
      CellTags->{
      "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", "IPM3.10.2.3", "c:3"}],
    Cell[6878, 180, 1013, 29, 109, "TextFirst",
      CellTags->"IPM3.10.2.0"]},
  "IPM3.10.2.1"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[6679, 175, 196, 3, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 1}},
      CellTags->{
      "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", "IPM3.10.2.3", "c:3"}],
    Cell[7916, 213, 88, 1, 44, "Subsection",
      CellTags->{"b:43.0", "IPM3.10.2.1", "c:4"}],
    Cell[8007, 216, 419, 10, 71, "TextFirst",
      CellTags->{"i:1", "b:43.0", "IPM3.10.2.1"}],
    Cell[8429, 228, 383, 17, 127, "Program",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[8815, 247, 517, 14, 71, "Text",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[9335, 263, 190, 4, 52, "Text",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[9528, 269, 201, 7, 110, "Program",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[9732, 278, 206, 6, 33, "TextFirst",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[89433, 1594, 426, 12, 52, "Text",
      CellTags->{"i:2", "b:43.0", "IPM3.10.2.1"}],
    Cell[89884, 1610, 364, 8, 150, "Input",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[90251, 1620, 294, 8, 29, "Output",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[90560, 1631, 132, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[170187, 2943, 1218, 32, 147, "Text",
      CellTags->{"i:3", "b:43.0", "IPM3.10.2.1"}],
    Cell[171430, 2979, 109, 3, 30, "Input",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[171542, 2984, 169, 5, 60, "Output",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[171726, 2992, 406, 10, 71, "Text",
      CellTags->{"i:5", "b:43.0", "IPM3.10.2.1"}],
    Cell[172157, 3006, 111, 3, 30, "Input",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[172271, 3011, 385, 7, 86, "Output",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[172671, 3021, 417, 9, 71, "Text",
      CellTags->{"i:6", "b:43.0", "IPM3.10.2.1"}]},
  "IPM3.10.2.2"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[6679, 175, 196, 3, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 1}},
      CellTags->{
      "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", "IPM3.10.2.3", "c:3"}],
    Cell[173125, 3035, 90, 1, 44, "Subsection",
      CellTags->{"b:43.1", "IPM3.10.2.2", "c:5"}],
    Cell[173218, 3038, 904, 16, 147, "TextFirst",
      CellTags->{"i:7", "b:43.1", "IPM3.10.2.2"}],
    Cell[174125, 3056, 379, 7, 71, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[174507, 3065, 159, 5, 33, "Text",
      CellTags->{"i:8", "b:43.1", "IPM3.10.2.2"}],
    Cell[174691, 3074, 107, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[174801, 3079, 827, 22, 48, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[175643, 3104, 179, 5, 33, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[175847, 3113, 120, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[175970, 3118, 315, 8, 111, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[176300, 3129, 391, 9, 71, "Text",
      CellTags->{"i:9", "b:43.1", "IPM3.10.2.2"}],
    Cell[176716, 3142, 117, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[176836, 3147, 294, 8, 29, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[237426, 4151, 336, 7, 71, "Text",
      CellTags->{"i:10", "b:43.1", "IPM3.10.2.2"}],
    Cell[237765, 4160, 352, 9, 52, "Text",
      CellTags->{"i:11", "b:43.1", "IPM3.10.2.2"}],
    Cell[238142, 4173, 164, 4, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[307508, 5319, 274, 7, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"i:12", "b:43.1", "IPM3.10.2.2"}],
    Cell[307807, 5330, 125, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[307935, 5335, 269, 5, 48, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[308241, 5345, 299, 7, 70, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[384350, 6602, 292, 10, 52, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[384667, 6616, 140, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[464830, 7938, 228, 7, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465083, 7949, 103, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465189, 7954, 129, 3, 29, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465333, 7960, 269, 8, 52, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465605, 7970, 127, 4, 59, "Program",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465735, 7976, 1094, 33, 128, "Text",
      CellTags->{"i:13", "b:43.1", "IPM3.10.2.2"}],
    Cell[466832, 8011, 143, 4, 33, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[467000, 8019, 127, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[546768, 9335, 127, 2, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[546920, 9341, 196, 5, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[627896, 10678, 687, 16, 109, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[628608, 10698, 130, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[709762, 12037, 201, 6, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"i:14", "b:43.1", "IPM3.10.2.2"}],
    Cell[709988, 12047, 118, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[799304, 13520, 225, 5, 52, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[799532, 13527, 354, 7, 71, "Text",
      CellTags->{"i:15", "b:43.1", "IPM3.10.2.2"}],
    Cell[799889, 13536, 102, 1, 42, "Program",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[799994, 13539, 213, 4, 52, "TextFirst",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[800210, 13545, 103, 1, 42, "Program",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[800316, 13548, 110, 1, 33, "TextFirst",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[800429, 13551, 97, 1, 42, "Program",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[800680, 13561, 376, 7, 110, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801059, 13570, 202, 5, 33, "TextFirst",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801264, 13577, 109, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801398, 13584, 102, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801503, 13589, 99, 3, 29, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}]},
  "IPM3.10.2.3"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[6679, 175, 196, 3, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 1}},
      CellTags->{
      "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", "IPM3.10.2.3", "c:3"}],
    Cell[801651, 13598, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:43.1", "IPM3.10.2.3", "c:6"}],
    Cell[801739, 13601, 219, 6, 46, "ExerciseItem",
      CellTags->{"b:43.1", "IPM3.10.2.3"}],
    Cell[801961, 13609, 326, 8, 46, "ExerciseItem",
      CellTags->{"b:43.1", "IPM3.10.2.3"}],
    Cell[802290, 13619, 294, 7, 46, "ExerciseItem",
      CellTags->{"b:43.1", "IPM3.10.2.3"}]},
  "IPM3.10.3.0"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[802633, 13632, 195, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 2}},
      CellTags->{
      "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3", 
        "IPM3.10.3.4", "c:7"}],
    Cell[802831, 13638, 598, 10, 109, "TextFirst",
      CellTags->"IPM3.10.3.0"]},
  "IPM3.10.3.1"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[802633, 13632, 195, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 2}},
      CellTags->{
      "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3", 
        "IPM3.10.3.4", "c:7"}],
    Cell[803454, 13652, 84, 1, 44, "Subsection",
      CellTags->{"i:16", "b:44.0", "IPM3.10.3.1", "c:8"}],
    Cell[803541, 13655, 138, 3, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[803682, 13660, 96, 1, 42, "Program",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[803781, 13663, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[803895, 13666, 126, 2, 47, "Text",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804024, 13670, 178, 5, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804205, 13677, 178, 6, 93, "Program",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804386, 13685, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804500, 13688, 200, 7, 47, "Text",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804703, 13697, 392, 10, 52, "TextFirst",
      CellTags->{"i:17", "b:44.0", "IPM3.10.3.1"}],
    Cell[805098, 13709, 150, 5, 76, "Program",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[805251, 13716, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[805365, 13719, 175, 6, 47, "Text",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[805543, 13727, 477, 14, 71, "Text",
      CellTags->{"i:18", "b:44.0", "IPM3.10.3.1"}],
    Cell[806023, 13743, 149, 6, 93, "Program",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[806175, 13751, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[806289, 13754, 179, 6, 40, "Text",
      CellTags->{"b:44.0", "IPM3.10.3.1"}]},
  "IPM3.10.3.2"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[802633, 13632, 195, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 2}},
      CellTags->{
      "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3", 
        "IPM3.10.3.4", "c:7"}],
    Cell[806505, 13765, 75, 1, 44, "Subsection",
      CellTags->{"b:44.1", "IPM3.10.3.2", "c:9"}],
    Cell[806583, 13768, 731, 16, 109, "TextFirst",
      CellTags->{"i:19", "b:44.1", "IPM3.10.3.2"}],
    Cell[807317, 13786, 164, 5, 76, "Program",
      CellTags->{"i:20", "b:44.1", "IPM3.10.3.2"}],
    Cell[807484, 13793, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[807598, 13796, 102, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[807703, 13801, 472, 14, 71, "Text",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[808178, 13817, 386, 12, 52, "Text",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[808589, 13833, 131, 3, 30, "Input",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[808723, 13838, 141, 4, 43, "Output",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[808879, 13845, 242, 7, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809124, 13854, 119, 4, 59, "Program",
      CellTags->{"i:21", "b:44.1", "IPM3.10.3.2"}],
    Cell[809246, 13860, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809360, 13863, 97, 3, 32, "Input",
      Evaluatable->False,
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809460, 13868, 130, 4, 59, "Program",
      CellTags->{"i:22", "b:44.1", "IPM3.10.3.2"}],
    Cell[809593, 13874, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809707, 13877, 98, 3, 42, "Input",
      Evaluatable->False,
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809808, 13882, 264, 10, 161, "Program",
      CellTags->{"i:23", "b:44.1", "IPM3.10.3.2"}],
    Cell[810075, 13894, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[810189, 13897, 132, 3, 42, "Input",
      Evaluatable->False,
      CellTags->{"b:44.1", "IPM3.10.3.2"}]},
  "IPM3.10.3.3"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[802633, 13632, 195, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 2}},
      CellTags->{
      "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3", 
        "IPM3.10.3.4", "c:7"}],
    Cell[810358, 13905, 81, 1, 44, "Subsection",
      CellTags->{"b:44.2", "IPM3.10.3.3", "c:10"}],
    Cell[810442, 13908, 306, 6, 52, "TextFirst",
      CellTags->{"i:24", "b:44.2", "IPM3.10.3.3"}],
    Cell[810773, 13918, 131, 3, 30, "Input",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[810907, 13923, 24743, 803, 181, 13969, 664, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[835665, 14729, 155, 4, 33, "TextFirst",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[835823, 14735, 177, 8, 127, "Program",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[836003, 14745, 542, 11, 90, "Text",
      CellTags->{"i:25", "b:44.2", "IPM3.10.3.3"}],
    Cell[836548, 14758, 271, 7, 90, "Input",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[836822, 14767, 228, 7, 52, "Text",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[837075, 14778, 198, 4, 50, "Input",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[837276, 14784, 294, 8, 29, "Output",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[837585, 14795, 336, 9, 52, "Text",
      CellTags->{"b:44.2", "IPM3.10.3.3"}]},
  "IPM3.10.3.4"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[802633, 13632, 195, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 2}},
      CellTags->{
      "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3", 
        "IPM3.10.3.4", "c:7"}],
    Cell[837958, 14809, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:44.2", "IPM3.10.3.4", "c:11"}],
    Cell[838047, 14812, 357, 13, 46, "ExerciseItem",
      CellTags->{"b:44.2", "IPM3.10.3.4"}],
    Cell[838407, 14827, 432, 13, 49, "ExerciseItem",
      CellTags->{"b:44.2", "IPM3.10.3.4"}]},
  "IPM3.10.4.1"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[838888, 14846, 152, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 3}},
      CellTags->{"IPM3.10.4.1", "IPM3.10.4.2", "IPM3.10.4.3", "c:12"}],
    Cell[839065, 14852, 78, 1, 44, "Subsection",
      CellTags->{"b:45.0", "IPM3.10.4.1", "c:13"}],
    Cell[839146, 14855, 484, 12, 71, "TextFirst",
      CellTags->{"i:26", "b:45.0", "IPM3.10.4.1"}],
    Cell[839633, 14869, 159, 6, 93, "Program",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[839795, 14877, 111, 1, 33, "TextFirst",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[839909, 14880, 144, 5, 41, "Input",
      Evaluatable->False,
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[840056, 14887, 296, 9, 52, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[840377, 14900, 189, 6, 41, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[840569, 14908, 277, 8, 43, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[840883, 14921, 126, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[841012, 14926, 302, 9, 55, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[841329, 14938, 228, 6, 52, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[841560, 14946, 274, 5, 64, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[841837, 14953, 240, 7, 52, "Text",
      CellTags->{"i:27", "b:45.0", "IPM3.10.4.1"}],
    Cell[842102, 14964, 124, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[842229, 14969, 309, 8, 83, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[842553, 14980, 167, 4, 33, "Text",
      CellTags->{"i:28", "b:45.0", "IPM3.10.4.1"}],
    Cell[842745, 14988, 112, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[842860, 14993, 753, 13, 143, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[843628, 15009, 629, 17, 90, "Text",
      CellTags->{"i:29", "b:45.0", "IPM3.10.4.1"}],
    Cell[844282, 15030, 304, 6, 70, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[844589, 15038, 416, 12, 101, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[845020, 15053, 558, 20, 71, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[845581, 15075, 578, 12, 150, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[846184, 15091, 111, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[846298, 15096, 549, 16, 107, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[846862, 15115, 418, 8, 71, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[847283, 15125, 156, 3, 30, "Input",
      CellTags->{"i:30", "b:45.0", "IPM3.10.4.1"}],
    Cell[847464, 15132, 135, 3, 44, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[847602, 15137, 247, 5, 76, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[847864, 15145, 240, 5, 52, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[848129, 15154, 126, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[848258, 15159, 626, 17, 149, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[848899, 15179, 138, 2, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[849062, 15185, 126, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[849781, 15209, 334, 6, 71, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}]},
  "IPM3.10.4.2"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[838888, 14846, 152, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 3}},
      CellTags->{"IPM3.10.4.1", "IPM3.10.4.2", "IPM3.10.4.3", "c:12"}],
    Cell[850152, 15220, 81, 1, 44, "Subsection",
      CellTags->{"b:45.1", "IPM3.10.4.2", "c:14"}],
    Cell[850236, 15223, 354, 7, 52, "TextFirst",
      CellTags->{"i:31", "b:45.1", "IPM3.10.4.2"}],
    Cell[851570, 15255, 225, 6, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[851798, 15263, 180, 4, 30, "Input",
      CellTags->{"i:32", "b:45.1", "IPM3.10.4.2"}],
    Cell[851981, 15269, 108, 1, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852114, 15274, 118, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852235, 15279, 136, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852386, 15285, 201, 5, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852612, 15294, 332, 6, 110, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852947, 15302, 257, 8, 71, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[853219, 15313, 300, 6, 71, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[853522, 15321, 217, 5, 50, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[853764, 15330, 115, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[853882, 15335, 192, 4, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854111, 15344, 115, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854229, 15349, 181, 4, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854425, 15356, 186, 4, 52, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854636, 15364, 133, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854772, 15369, 118, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854927, 15377, 138, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855068, 15382, 135, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855240, 15390, 143, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855386, 15395, 152, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855553, 15401, 160, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855738, 15410, 141, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855882, 15415, 113, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856010, 15421, 178, 5, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856191, 15428, 178, 4, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856394, 15436, 120, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856517, 15441, 135, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856689, 15449, 120, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856812, 15454, 101, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856928, 15460, 167, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857120, 15469, 133, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857256, 15474, 238, 5, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857509, 15482, 163, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857697, 15491, 111, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857811, 15496, 417, 7, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[858243, 15506, 160, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[858428, 15515, 122, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[858553, 15520, 325, 9, 87, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[858893, 15532, 169, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[859065, 15539, 555, 11, 209, "Input",
      CellTags->{"i:33", "b:45.1", "IPM3.10.4.2"}],
    Cell[859645, 15554, 127, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[859775, 15559, 514, 11, 119, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}]},
  "IPM3.10.4.3"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[838888, 14846, 152, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 3}},
      CellTags->{"IPM3.10.4.1", "IPM3.10.4.2", "IPM3.10.4.3", "c:12"}],
    Cell[860338, 15576, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:45.1", "IPM3.10.4.3", "c:15"}],
    Cell[860427, 15579, 369, 9, 65, "ExerciseItem",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[860799, 15590, 276, 7, 46, "ExerciseItem",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[861100, 15601, 599, 19, 65, "ExerciseItem",
      CellTags->{"i:34", "b:45.1", "IPM3.10.4.3"}],
    Cell[861724, 15624, 137, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[861864, 15629, 707, 22, 177, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[862586, 15654, 504, 13, 65, "ExercisesNoLabel",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[863115, 15671, 260, 6, 70, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[863378, 15679, 287, 5, 67, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[863680, 15687, 190, 4, 27, "ExercisesNoLabel",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[863895, 15695, 131, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[864029, 15700, 572, 10, 86, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[864616, 15713, 180, 4, 27, "ExercisesNoLabel",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[864821, 15721, 179, 4, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[865003, 15727, 148, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[865166, 15733, 198, 5, 27, "ExercisesNoLabel",
      CellTags->{"b:45.1", "IPM3.10.4.3"}]},
  "IPM3.10.5.0"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[865425, 15745, 218, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
      CellTags->{
      "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", 
        "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],
    Cell[865646, 15751, 502, 8, 90, "TextFirst",
      CellTags->"IPM3.10.5.0"],
    Cell[866151, 15761, 260, 6, 52, "Text",
      CellTags->"IPM3.10.5.0"]},
  "IPM3.10.5.1"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[865425, 15745, 218, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
      CellTags->{
      "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", 
        "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],
    Cell[866436, 15771, 96, 1, 44, "Subsection",
      CellTags->{"b:46.0", "IPM3.10.5.1", "c:17"}],
    Cell[866535, 15774, 450, 11, 71, "TextFirst",
      CellTags->{"i:35", "b:46.0", "IPM3.10.5.1"}],
    Cell[866988, 15787, 520, 10, 90, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[867511, 15799, 125, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[867639, 15804, 229, 7, 33, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[867871, 15813, 164, 4, 32, "Input",
      Evaluatable->False,
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[868038, 15819, 334, 8, 52, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[868375, 15829, 180, 5, 32, "Input",
      Evaluatable->False,
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[868558, 15836, 233, 7, 33, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[868794, 15845, 265, 8, 52, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[869062, 15855, 415, 7, 90, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}]},
  "IPM3.10.5.2"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[865425, 15745, 218, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
      CellTags->{
      "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", 
        "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],
    Cell[869514, 15867, 93, 1, 44, "Subsection",
      CellTags->{"b:46.1", "IPM3.10.5.2", "c:18"}],
    Cell[869610, 15870, 353, 11, 52, "TextFirst",
      CellTags->{"i:36", "b:46.1", "IPM3.10.5.2"}],
    Cell[869966, 15883, 429, 12, 71, "Text",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[870420, 15899, 165, 4, 30, "Input",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[870588, 15905, 187, 6, 46, "Output",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[870790, 15914, 391, 8, 71, "Text",
      CellTags->{"i:37", "b:46.1", "IPM3.10.5.2"}],
    Cell[871184, 15924, 104, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[871291, 15929, 149, 4, 33, "Text",
      CellTags->{"i:38", "b:46.1", "IPM3.10.5.2"}],
    Cell[871465, 15937, 160, 4, 30, "Input",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[871628, 15943, 208, 6, 46, "Output",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[871851, 15952, 112, 1, 33, "Text",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[871966, 15955, 125, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[872094, 15960, 623, 13, 90, "Text",
      CellTags->{"i:39", "b:46.1", "IPM3.10.5.2"}],
    Cell[872742, 15977, 206, 5, 30, "Input",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[872951, 15984, 254, 7, 46, "Output",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[873220, 15994, 204, 4, 52, "Text",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[873427, 16000, 171, 4, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:46.1", "IPM3.10.5.2"}]},
  "IPM3.10.5.3"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[865425, 15745, 218, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
      CellTags->{
      "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", 
        "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],
    Cell[873635, 16009, 80, 1, 44, "Subsection",
      CellTags->{"b:46.2", "IPM3.10.5.3", "c:19"}],
    Cell[873718, 16012, 779, 18, 128, "TextFirst",
      CellTags->{"i:40", "b:46.2", "IPM3.10.5.3"}],
    Cell[874500, 16032, 267, 6, 50, "Input",
      Evaluatable->False,
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[874770, 16040, 567, 13, 71, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[875340, 16055, 255, 7, 46, "Output",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[875598, 16064, 134, 4, 31, "Input",
      Evaluatable->False,
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[875757, 16072, 148, 4, 31, "Input",
      Evaluatable->False,
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[875908, 16078, 150, 3, 52, "Output",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[876073, 16084, 301, 8, 52, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[876377, 16094, 427, 14, 71, "Text",
      CellTags->{"i:41", "b:46.2", "IPM3.10.5.3"}],
    Cell[876807, 16110, 244, 9, 144, "Program",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[877054, 16121, 106, 1, 33, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[877163, 16124, 309, 9, 31, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[877475, 16135, 595, 16, 90, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[878073, 16153, 172, 5, 33, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[878248, 16160, 1521, 49, 213, "DisplayTable",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[879772, 16211, 152, 5, 22, "NumberedTable",
      CellTags->{"b:46.2", "IPM3.10.5.3"}]},
  "IPM3.10.5.4"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[865425, 15745, 218, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
      CellTags->{
      "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", 
        "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],
    Cell[879961, 16221, 83, 1, 44, "Subsection",
      CellTags->{"b:46.3", "IPM3.10.5.4", "c:20"}],
    Cell[880047, 16224, 1068, 24, 147, "TextFirst",
      CellTags->{"i:42", "b:46.3", "IPM3.10.5.4"}],
    Cell[881140, 16252, 350, 7, 90, "Input",
      Evaluatable->False,
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[881493, 16261, 267, 8, 46, "Output",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[881775, 16272, 347, 7, 71, "Text",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[882147, 16283, 357, 7, 90, "Input",
      Evaluatable->False,
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[882507, 16292, 288, 9, 46, "Output",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[882798, 16303, 104, 3, 20, "Print",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[882917, 16309, 1297, 37, 166, "Text",
      CellTags->{"i:43", "b:46.3", "IPM3.10.5.4"}],
    Cell[884217, 16348, 262, 5, 52, "Text",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[884482, 16355, 128, 3, 30, "Input",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[884635, 16362, 143, 3, 30, "Input",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[884781, 16367, 27500, 618, 203, 6511, 353, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[912296, 16988, 253, 6, 52, "Text",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[912552, 16996, 434, 12, 195, "Program",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[912989, 17010, 126, 3, 33, "Text",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[913140, 17017, 478, 15, 34, "Input",
      Evaluatable->False,
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[913621, 17034, 27497, 618, 203, 6511, 353, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:46.3", "IPM3.10.5.4"}]},
  "IPM3.10.5.5"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[865425, 15745, 218, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
      CellTags->{
      "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", 
        "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],
    Cell[941167, 17658, 96, 1, 44, "Subsection",
      CellTags->{"b:46.4", "IPM3.10.5.5", "c:21"}],
    Cell[941266, 17661, 294, 6, 52, "TextFirst",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[941563, 17669, 102, 1, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[941668, 17672, 233, 7, 110, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[941904, 17681, 736, 19, 109, "Text",
      CellTags->{"i:44", "b:46.4", "IPM3.10.5.5"}],
    Cell[942643, 17702, 319, 6, 90, "Input",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[942965, 17710, 109, 1, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[943077, 17713, 460, 15, 246, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[943540, 17730, 187, 4, 52, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[943730, 17736, 427, 13, 34, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[944182, 17753, 102, 3, 30, "Input",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[944287, 17758, 99, 3, 29, "Output",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[944401, 17764, 142, 4, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[944546, 17770, 789, 26, 433, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[945338, 17798, 147, 4, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[945488, 17804, 786, 25, 46, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[946299, 17833, 102, 3, 30, "Input",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[946404, 17838, 99, 3, 29, "Output",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[946518, 17844, 813, 18, 128, "Text",
      CellTags->{"i:45", "b:46.4", "IPM3.10.5.5"}],
    Cell[947334, 17864, 118, 3, 42, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[947455, 17869, 193, 6, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[947651, 17877, 288, 9, 144, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[947942, 17888, 364, 6, 71, "Text",
      CellTags->{"i:46", "b:46.4", "IPM3.10.5.5"}]},
  "IPM3.10.5.6"->{
    Cell[1754, 51, 469, 10, 29, "TitleBar",
      CellTags->{
      "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
        "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", 
        "IPM3.10.3.3", "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", 
        "IPM3.10.4.3", "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", 
        "IPM3.10.5.3", "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6"}],
    Cell[865425, 15745, 218, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
      CellTags->{
      "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", 
        "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],
    Cell[948343, 17899, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:46.4", "IPM3.10.5.6", "c:22"}],
    Cell[948432, 17902, 177, 5, 27, "ExerciseItem",
      CellTags->{"b:46.4", "IPM3.10.5.6"}],
    Cell[948612, 17909, 207, 5, 27, "ExerciseItem",
      CellTags->{"b:46.4", "IPM3.10.5.6"}],
    Cell[948822, 17916, 398, 14, 46, "ExerciseItem",
      CellTags->{"b:46.4", "IPM3.10.5.6"}]},
  "c:1"->{
    Cell[2248, 65, 159, 6, 96, "Chapter",
      CounterAssignments->{{"Chapter", 9}},
      CellTags->{"IPM3.10.0", "c:1"}]},
  "b:42"->{
    Cell[3165, 94, 130, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 0}},
      CellTags->{"b:42", "IPM3.10.1", "c:2"}],
    Cell[3298, 98, 555, 11, 90, "TextFirst",
      CellTags->{"b:42", "IPM3.10.1"}],
    Cell[3856, 111, 439, 9, 71, "Text",
      CellTags->{"b:42", "IPM3.10.1"}],
    Cell[4298, 122, 789, 14, 128, "Text",
      CellTags->{"b:42", "IPM3.10.1"}],
    Cell[5090, 138, 336, 6, 71, "Text",
      CellTags->{"b:42", "IPM3.10.1"}],
    Cell[5429, 146, 1213, 24, 185, "Text",
      CellTags->{"b:42", "IPM3.10.1"}]},
  "c:2"->{
    Cell[3165, 94, 130, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 0}},
      CellTags->{"b:42", "IPM3.10.1", "c:2"}]},
  "c:3"->{
    Cell[6679, 175, 196, 3, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 1}},
      CellTags->{
      "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", "IPM3.10.2.3", "c:3"}]},
  "b:43.0"->{
    Cell[7916, 213, 88, 1, 44, "Subsection",
      CellTags->{"b:43.0", "IPM3.10.2.1", "c:4"}],
    Cell[8007, 216, 419, 10, 71, "TextFirst",
      CellTags->{"i:1", "b:43.0", "IPM3.10.2.1"}],
    Cell[8429, 228, 383, 17, 127, "Program",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[8815, 247, 517, 14, 71, "Text",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[9335, 263, 190, 4, 52, "Text",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[9528, 269, 201, 7, 110, "Program",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[9732, 278, 206, 6, 33, "TextFirst",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[89433, 1594, 426, 12, 52, "Text",
      CellTags->{"i:2", "b:43.0", "IPM3.10.2.1"}],
    Cell[89884, 1610, 364, 8, 150, "Input",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[90251, 1620, 294, 8, 29, "Output",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[90560, 1631, 132, 2, 33, "TextFirst",
      PageBreakAbove->True,
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[170187, 2943, 1218, 32, 147, "Text",
      CellTags->{"i:3", "b:43.0", "IPM3.10.2.1"}],
    Cell[171430, 2979, 109, 3, 30, "Input",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[171542, 2984, 169, 5, 60, "Output",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[171726, 2992, 406, 10, 71, "Text",
      CellTags->{"i:5", "b:43.0", "IPM3.10.2.1"}],
    Cell[172157, 3006, 111, 3, 30, "Input",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[172271, 3011, 385, 7, 86, "Output",
      CellTags->{"b:43.0", "IPM3.10.2.1"}],
    Cell[172671, 3021, 417, 9, 71, "Text",
      CellTags->{"i:6", "b:43.0", "IPM3.10.2.1"}]},
  "c:4"->{
    Cell[7916, 213, 88, 1, 44, "Subsection",
      CellTags->{"b:43.0", "IPM3.10.2.1", "c:4"}]},
  "i:1"->{
    Cell[8007, 216, 419, 10, 71, "TextFirst",
      CellTags->{"i:1", "b:43.0", "IPM3.10.2.1"}]},
  "i:2"->{
    Cell[89433, 1594, 426, 12, 52, "Text",
      CellTags->{"i:2", "b:43.0", "IPM3.10.2.1"}]},
  "i:3"->{
    Cell[170187, 2943, 1218, 32, 147, "Text",
      CellTags->{"i:3", "b:43.0", "IPM3.10.2.1"}]},
  "i:5"->{
    Cell[171726, 2992, 406, 10, 71, "Text",
      CellTags->{"i:5", "b:43.0", "IPM3.10.2.1"}]},
  "i:6"->{
    Cell[172671, 3021, 417, 9, 71, "Text",
      CellTags->{"i:6", "b:43.0", "IPM3.10.2.1"}]},
  "b:43.1"->{
    Cell[173125, 3035, 90, 1, 44, "Subsection",
      CellTags->{"b:43.1", "IPM3.10.2.2", "c:5"}],
    Cell[173218, 3038, 904, 16, 147, "TextFirst",
      CellTags->{"i:7", "b:43.1", "IPM3.10.2.2"}],
    Cell[174125, 3056, 379, 7, 71, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[174507, 3065, 159, 5, 33, "Text",
      CellTags->{"i:8", "b:43.1", "IPM3.10.2.2"}],
    Cell[174691, 3074, 107, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[174801, 3079, 827, 22, 48, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[175643, 3104, 179, 5, 33, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[175847, 3113, 120, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[175970, 3118, 315, 8, 111, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[176300, 3129, 391, 9, 71, "Text",
      CellTags->{"i:9", "b:43.1", "IPM3.10.2.2"}],
    Cell[176716, 3142, 117, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[176836, 3147, 294, 8, 29, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[237426, 4151, 336, 7, 71, "Text",
      CellTags->{"i:10", "b:43.1", "IPM3.10.2.2"}],
    Cell[237765, 4160, 352, 9, 52, "Text",
      CellTags->{"i:11", "b:43.1", "IPM3.10.2.2"}],
    Cell[238142, 4173, 164, 4, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[307508, 5319, 274, 7, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"i:12", "b:43.1", "IPM3.10.2.2"}],
    Cell[307807, 5330, 125, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[307935, 5335, 269, 5, 48, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[308241, 5345, 299, 7, 70, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[384350, 6602, 292, 10, 52, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[384667, 6616, 140, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[464830, 7938, 228, 7, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465083, 7949, 103, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465189, 7954, 129, 3, 29, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465333, 7960, 269, 8, 52, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465605, 7970, 127, 4, 59, "Program",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[465735, 7976, 1094, 33, 128, "Text",
      CellTags->{"i:13", "b:43.1", "IPM3.10.2.2"}],
    Cell[466832, 8011, 143, 4, 33, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[467000, 8019, 127, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[546768, 9335, 127, 2, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[546920, 9341, 196, 5, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[627896, 10678, 687, 16, 109, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[628608, 10698, 130, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[709762, 12037, 201, 6, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"i:14", "b:43.1", "IPM3.10.2.2"}],
    Cell[709988, 12047, 118, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[799304, 13520, 225, 5, 52, "Text",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[799532, 13527, 354, 7, 71, "Text",
      CellTags->{"i:15", "b:43.1", "IPM3.10.2.2"}],
    Cell[799889, 13536, 102, 1, 42, "Program",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[799994, 13539, 213, 4, 52, "TextFirst",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[800210, 13545, 103, 1, 42, "Program",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[800316, 13548, 110, 1, 33, "TextFirst",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[800429, 13551, 97, 1, 42, "Program",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[800680, 13561, 376, 7, 110, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801059, 13570, 202, 5, 33, "TextFirst",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801264, 13577, 109, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801398, 13584, 102, 3, 30, "Input",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801503, 13589, 99, 3, 29, "Output",
      CellTags->{"b:43.1", "IPM3.10.2.2"}],
    Cell[801651, 13598, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:43.1", "IPM3.10.2.3", "c:6"}],
    Cell[801739, 13601, 219, 6, 46, "ExerciseItem",
      CellTags->{"b:43.1", "IPM3.10.2.3"}],
    Cell[801961, 13609, 326, 8, 46, "ExerciseItem",
      CellTags->{"b:43.1", "IPM3.10.2.3"}],
    Cell[802290, 13619, 294, 7, 46, "ExerciseItem",
      CellTags->{"b:43.1", "IPM3.10.2.3"}]},
  "c:5"->{
    Cell[173125, 3035, 90, 1, 44, "Subsection",
      CellTags->{"b:43.1", "IPM3.10.2.2", "c:5"}]},
  "i:7"->{
    Cell[173218, 3038, 904, 16, 147, "TextFirst",
      CellTags->{"i:7", "b:43.1", "IPM3.10.2.2"}]},
  "i:8"->{
    Cell[174507, 3065, 159, 5, 33, "Text",
      CellTags->{"i:8", "b:43.1", "IPM3.10.2.2"}]},
  "i:9"->{
    Cell[176300, 3129, 391, 9, 71, "Text",
      CellTags->{"i:9", "b:43.1", "IPM3.10.2.2"}]},
  "i:10"->{
    Cell[237426, 4151, 336, 7, 71, "Text",
      CellTags->{"i:10", "b:43.1", "IPM3.10.2.2"}]},
  "i:11"->{
    Cell[237765, 4160, 352, 9, 52, "Text",
      CellTags->{"i:11", "b:43.1", "IPM3.10.2.2"}]},
  "i:12"->{
    Cell[307508, 5319, 274, 7, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"i:12", "b:43.1", "IPM3.10.2.2"}]},
  "i:13"->{
    Cell[465735, 7976, 1094, 33, 128, "Text",
      CellTags->{"i:13", "b:43.1", "IPM3.10.2.2"}]},
  "i:14"->{
    Cell[709762, 12037, 201, 6, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"i:14", "b:43.1", "IPM3.10.2.2"}]},
  "i:15"->{
    Cell[799532, 13527, 354, 7, 71, "Text",
      CellTags->{"i:15", "b:43.1", "IPM3.10.2.2"}]},
  "c:6"->{
    Cell[801651, 13598, 85, 1, 77, "ExerciseSubsection",
      CellTags->{"b:43.1", "IPM3.10.2.3", "c:6"}]},
  "c:7"->{
    Cell[802633, 13632, 195, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 2}},
      CellTags->{
      "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3", 
        "IPM3.10.3.4", "c:7"}]},
  "i:16"->{
    Cell[803454, 13652, 84, 1, 44, "Subsection",
      CellTags->{"i:16", "b:44.0", "IPM3.10.3.1", "c:8"}]},
  "b:44.0"->{
    Cell[803454, 13652, 84, 1, 44, "Subsection",
      CellTags->{"i:16", "b:44.0", "IPM3.10.3.1", "c:8"}],
    Cell[803541, 13655, 138, 3, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[803682, 13660, 96, 1, 42, "Program",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[803781, 13663, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[803895, 13666, 126, 2, 47, "Text",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804024, 13670, 178, 5, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804205, 13677, 178, 6, 93, "Program",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804386, 13685, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804500, 13688, 200, 7, 47, "Text",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[804703, 13697, 392, 10, 52, "TextFirst",
      CellTags->{"i:17", "b:44.0", "IPM3.10.3.1"}],
    Cell[805098, 13709, 150, 5, 76, "Program",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[805251, 13716, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[805365, 13719, 175, 6, 47, "Text",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[805543, 13727, 477, 14, 71, "Text",
      CellTags->{"i:18", "b:44.0", "IPM3.10.3.1"}],
    Cell[806023, 13743, 149, 6, 93, "Program",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[806175, 13751, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.0", "IPM3.10.3.1"}],
    Cell[806289, 13754, 179, 6, 40, "Text",
      CellTags->{"b:44.0", "IPM3.10.3.1"}]},
  "c:8"->{
    Cell[803454, 13652, 84, 1, 44, "Subsection",
      CellTags->{"i:16", "b:44.0", "IPM3.10.3.1", "c:8"}]},
  "i:17"->{
    Cell[804703, 13697, 392, 10, 52, "TextFirst",
      CellTags->{"i:17", "b:44.0", "IPM3.10.3.1"}]},
  "i:18"->{
    Cell[805543, 13727, 477, 14, 71, "Text",
      CellTags->{"i:18", "b:44.0", "IPM3.10.3.1"}]},
  "b:44.1"->{
    Cell[806505, 13765, 75, 1, 44, "Subsection",
      CellTags->{"b:44.1", "IPM3.10.3.2", "c:9"}],
    Cell[806583, 13768, 731, 16, 109, "TextFirst",
      CellTags->{"i:19", "b:44.1", "IPM3.10.3.2"}],
    Cell[807317, 13786, 164, 5, 76, "Program",
      CellTags->{"i:20", "b:44.1", "IPM3.10.3.2"}],
    Cell[807484, 13793, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[807598, 13796, 102, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[807703, 13801, 472, 14, 71, "Text",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[808178, 13817, 386, 12, 52, "Text",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[808589, 13833, 131, 3, 30, "Input",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[808723, 13838, 141, 4, 43, "Output",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[808879, 13845, 242, 7, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809124, 13854, 119, 4, 59, "Program",
      CellTags->{"i:21", "b:44.1", "IPM3.10.3.2"}],
    Cell[809246, 13860, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809360, 13863, 97, 3, 32, "Input",
      Evaluatable->False,
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809460, 13868, 130, 4, 59, "Program",
      CellTags->{"i:22", "b:44.1", "IPM3.10.3.2"}],
    Cell[809593, 13874, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809707, 13877, 98, 3, 42, "Input",
      Evaluatable->False,
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[809808, 13882, 264, 10, 161, "Program",
      CellTags->{"i:23", "b:44.1", "IPM3.10.3.2"}],
    Cell[810075, 13894, 111, 1, 33, "TextFirst",
      CellTags->{"b:44.1", "IPM3.10.3.2"}],
    Cell[810189, 13897, 132, 3, 42, "Input",
      Evaluatable->False,
      CellTags->{"b:44.1", "IPM3.10.3.2"}]},
  "c:9"->{
    Cell[806505, 13765, 75, 1, 44, "Subsection",
      CellTags->{"b:44.1", "IPM3.10.3.2", "c:9"}]},
  "i:19"->{
    Cell[806583, 13768, 731, 16, 109, "TextFirst",
      CellTags->{"i:19", "b:44.1", "IPM3.10.3.2"}]},
  "i:20"->{
    Cell[807317, 13786, 164, 5, 76, "Program",
      CellTags->{"i:20", "b:44.1", "IPM3.10.3.2"}]},
  "i:21"->{
    Cell[809124, 13854, 119, 4, 59, "Program",
      CellTags->{"i:21", "b:44.1", "IPM3.10.3.2"}]},
  "i:22"->{
    Cell[809460, 13868, 130, 4, 59, "Program",
      CellTags->{"i:22", "b:44.1", "IPM3.10.3.2"}]},
  "i:23"->{
    Cell[809808, 13882, 264, 10, 161, "Program",
      CellTags->{"i:23", "b:44.1", "IPM3.10.3.2"}]},
  "b:44.2"->{
    Cell[810358, 13905, 81, 1, 44, "Subsection",
      CellTags->{"b:44.2", "IPM3.10.3.3", "c:10"}],
    Cell[810442, 13908, 306, 6, 52, "TextFirst",
      CellTags->{"i:24", "b:44.2", "IPM3.10.3.3"}],
    Cell[810773, 13918, 131, 3, 30, "Input",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[810907, 13923, 24743, 803, 181, 13969, 664, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[835665, 14729, 155, 4, 33, "TextFirst",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[835823, 14735, 177, 8, 127, "Program",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[836003, 14745, 542, 11, 90, "Text",
      CellTags->{"i:25", "b:44.2", "IPM3.10.3.3"}],
    Cell[836548, 14758, 271, 7, 90, "Input",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[836822, 14767, 228, 7, 52, "Text",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[837075, 14778, 198, 4, 50, "Input",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[837276, 14784, 294, 8, 29, "Output",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[837585, 14795, 336, 9, 52, "Text",
      CellTags->{"b:44.2", "IPM3.10.3.3"}],
    Cell[837958, 14809, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:44.2", "IPM3.10.3.4", "c:11"}],
    Cell[838047, 14812, 357, 13, 46, "ExerciseItem",
      CellTags->{"b:44.2", "IPM3.10.3.4"}],
    Cell[838407, 14827, 432, 13, 49, "ExerciseItem",
      CellTags->{"b:44.2", "IPM3.10.3.4"}]},
  "c:10"->{
    Cell[810358, 13905, 81, 1, 44, "Subsection",
      CellTags->{"b:44.2", "IPM3.10.3.3", "c:10"}]},
  "i:24"->{
    Cell[810442, 13908, 306, 6, 52, "TextFirst",
      CellTags->{"i:24", "b:44.2", "IPM3.10.3.3"}]},
  "i:25"->{
    Cell[836003, 14745, 542, 11, 90, "Text",
      CellTags->{"i:25", "b:44.2", "IPM3.10.3.3"}]},
  "c:11"->{
    Cell[837958, 14809, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:44.2", "IPM3.10.3.4", "c:11"}]},
  "c:12"->{
    Cell[838888, 14846, 152, 2, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 3}},
      CellTags->{"IPM3.10.4.1", "IPM3.10.4.2", "IPM3.10.4.3", "c:12"}]},
  "b:45.0"->{
    Cell[839065, 14852, 78, 1, 44, "Subsection",
      CellTags->{"b:45.0", "IPM3.10.4.1", "c:13"}],
    Cell[839146, 14855, 484, 12, 71, "TextFirst",
      CellTags->{"i:26", "b:45.0", "IPM3.10.4.1"}],
    Cell[839633, 14869, 159, 6, 93, "Program",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[839795, 14877, 111, 1, 33, "TextFirst",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[839909, 14880, 144, 5, 41, "Input",
      Evaluatable->False,
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[840056, 14887, 296, 9, 52, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[840377, 14900, 189, 6, 41, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[840569, 14908, 277, 8, 43, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[840883, 14921, 126, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[841012, 14926, 302, 9, 55, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[841329, 14938, 228, 6, 52, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[841560, 14946, 274, 5, 64, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[841837, 14953, 240, 7, 52, "Text",
      CellTags->{"i:27", "b:45.0", "IPM3.10.4.1"}],
    Cell[842102, 14964, 124, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[842229, 14969, 309, 8, 83, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[842553, 14980, 167, 4, 33, "Text",
      CellTags->{"i:28", "b:45.0", "IPM3.10.4.1"}],
    Cell[842745, 14988, 112, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[842860, 14993, 753, 13, 143, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[843628, 15009, 629, 17, 90, "Text",
      CellTags->{"i:29", "b:45.0", "IPM3.10.4.1"}],
    Cell[844282, 15030, 304, 6, 70, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[844589, 15038, 416, 12, 101, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[845020, 15053, 558, 20, 71, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[845581, 15075, 578, 12, 150, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[846184, 15091, 111, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[846298, 15096, 549, 16, 107, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[846862, 15115, 418, 8, 71, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[847283, 15125, 156, 3, 30, "Input",
      CellTags->{"i:30", "b:45.0", "IPM3.10.4.1"}],
    Cell[847464, 15132, 135, 3, 44, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[847602, 15137, 247, 5, 76, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[847864, 15145, 240, 5, 52, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[848129, 15154, 126, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[848258, 15159, 626, 17, 149, "Output",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[848899, 15179, 138, 2, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[849062, 15185, 126, 3, 30, "Input",
      CellTags->{"b:45.0", "IPM3.10.4.1"}],
    Cell[849781, 15209, 334, 6, 71, "Text",
      CellTags->{"b:45.0", "IPM3.10.4.1"}]},
  "c:13"->{
    Cell[839065, 14852, 78, 1, 44, "Subsection",
      CellTags->{"b:45.0", "IPM3.10.4.1", "c:13"}]},
  "i:26"->{
    Cell[839146, 14855, 484, 12, 71, "TextFirst",
      CellTags->{"i:26", "b:45.0", "IPM3.10.4.1"}]},
  "i:27"->{
    Cell[841837, 14953, 240, 7, 52, "Text",
      CellTags->{"i:27", "b:45.0", "IPM3.10.4.1"}]},
  "i:28"->{
    Cell[842553, 14980, 167, 4, 33, "Text",
      CellTags->{"i:28", "b:45.0", "IPM3.10.4.1"}]},
  "i:29"->{
    Cell[843628, 15009, 629, 17, 90, "Text",
      CellTags->{"i:29", "b:45.0", "IPM3.10.4.1"}]},
  "i:30"->{
    Cell[847283, 15125, 156, 3, 30, "Input",
      CellTags->{"i:30", "b:45.0", "IPM3.10.4.1"}]},
  "b:45.1"->{
    Cell[850152, 15220, 81, 1, 44, "Subsection",
      CellTags->{"b:45.1", "IPM3.10.4.2", "c:14"}],
    Cell[850236, 15223, 354, 7, 52, "TextFirst",
      CellTags->{"i:31", "b:45.1", "IPM3.10.4.2"}],
    Cell[851570, 15255, 225, 6, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[851798, 15263, 180, 4, 30, "Input",
      CellTags->{"i:32", "b:45.1", "IPM3.10.4.2"}],
    Cell[851981, 15269, 108, 1, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852114, 15274, 118, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852235, 15279, 136, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852386, 15285, 201, 5, 52, "Text",
      PageBreakAbove->True,
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852612, 15294, 332, 6, 110, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[852947, 15302, 257, 8, 71, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[853219, 15313, 300, 6, 71, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[853522, 15321, 217, 5, 50, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[853764, 15330, 115, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[853882, 15335, 192, 4, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854111, 15344, 115, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854229, 15349, 181, 4, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854425, 15356, 186, 4, 52, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854636, 15364, 133, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854772, 15369, 118, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[854927, 15377, 138, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855068, 15382, 135, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855240, 15390, 143, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855386, 15395, 152, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855553, 15401, 160, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855738, 15410, 141, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[855882, 15415, 113, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856010, 15421, 178, 5, 33, "Text",
      PageBreakAbove->True,
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856191, 15428, 178, 4, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856394, 15436, 120, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856517, 15441, 135, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856689, 15449, 120, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856812, 15454, 101, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[856928, 15460, 167, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857120, 15469, 133, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857256, 15474, 238, 5, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857509, 15482, 163, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857697, 15491, 111, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[857811, 15496, 417, 7, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[858243, 15506, 160, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[858428, 15515, 122, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[858553, 15520, 325, 9, 87, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[858893, 15532, 169, 5, 33, "Text",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[859065, 15539, 555, 11, 209, "Input",
      CellTags->{"i:33", "b:45.1", "IPM3.10.4.2"}],
    Cell[859645, 15554, 127, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[859775, 15559, 514, 11, 119, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.2"}],
    Cell[860338, 15576, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:45.1", "IPM3.10.4.3", "c:15"}],
    Cell[860427, 15579, 369, 9, 65, "ExerciseItem",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[860799, 15590, 276, 7, 46, "ExerciseItem",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[861100, 15601, 599, 19, 65, "ExerciseItem",
      CellTags->{"i:34", "b:45.1", "IPM3.10.4.3"}],
    Cell[861724, 15624, 137, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[861864, 15629, 707, 22, 177, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[862586, 15654, 504, 13, 65, "ExercisesNoLabel",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[863115, 15671, 260, 6, 70, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[863378, 15679, 287, 5, 67, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[863680, 15687, 190, 4, 27, "ExercisesNoLabel",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[863895, 15695, 131, 3, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[864029, 15700, 572, 10, 86, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[864616, 15713, 180, 4, 27, "ExercisesNoLabel",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[864821, 15721, 179, 4, 30, "Input",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[865003, 15727, 148, 3, 29, "Output",
      CellTags->{"b:45.1", "IPM3.10.4.3"}],
    Cell[865166, 15733, 198, 5, 27, "ExercisesNoLabel",
      CellTags->{"b:45.1", "IPM3.10.4.3"}]},
  "c:14"->{
    Cell[850152, 15220, 81, 1, 44, "Subsection",
      CellTags->{"b:45.1", "IPM3.10.4.2", "c:14"}]},
  "i:31"->{
    Cell[850236, 15223, 354, 7, 52, "TextFirst",
      CellTags->{"i:31", "b:45.1", "IPM3.10.4.2"}]},
  "i:32"->{
    Cell[851798, 15263, 180, 4, 30, "Input",
      CellTags->{"i:32", "b:45.1", "IPM3.10.4.2"}]},
  "i:33"->{
    Cell[859065, 15539, 555, 11, 209, "Input",
      CellTags->{"i:33", "b:45.1", "IPM3.10.4.2"}]},
  "c:15"->{
    Cell[860338, 15576, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:45.1", "IPM3.10.4.3", "c:15"}]},
  "i:34"->{
    Cell[861100, 15601, 599, 19, 65, "ExerciseItem",
      CellTags->{"i:34", "b:45.1", "IPM3.10.4.3"}]},
  "c:16"->{
    Cell[865425, 15745, 218, 4, 40, "Section",
      CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
      CellTags->{
      "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", 
        "IPM3.10.5.4", "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}]},
  "b:46.0"->{
    Cell[866436, 15771, 96, 1, 44, "Subsection",
      CellTags->{"b:46.0", "IPM3.10.5.1", "c:17"}],
    Cell[866535, 15774, 450, 11, 71, "TextFirst",
      CellTags->{"i:35", "b:46.0", "IPM3.10.5.1"}],
    Cell[866988, 15787, 520, 10, 90, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[867511, 15799, 125, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[867639, 15804, 229, 7, 33, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[867871, 15813, 164, 4, 32, "Input",
      Evaluatable->False,
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[868038, 15819, 334, 8, 52, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[868375, 15829, 180, 5, 32, "Input",
      Evaluatable->False,
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[868558, 15836, 233, 7, 33, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[868794, 15845, 265, 8, 52, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}],
    Cell[869062, 15855, 415, 7, 90, "Text",
      CellTags->{"b:46.0", "IPM3.10.5.1"}]},
  "c:17"->{
    Cell[866436, 15771, 96, 1, 44, "Subsection",
      CellTags->{"b:46.0", "IPM3.10.5.1", "c:17"}]},
  "i:35"->{
    Cell[866535, 15774, 450, 11, 71, "TextFirst",
      CellTags->{"i:35", "b:46.0", "IPM3.10.5.1"}]},
  "b:46.1"->{
    Cell[869514, 15867, 93, 1, 44, "Subsection",
      CellTags->{"b:46.1", "IPM3.10.5.2", "c:18"}],
    Cell[869610, 15870, 353, 11, 52, "TextFirst",
      CellTags->{"i:36", "b:46.1", "IPM3.10.5.2"}],
    Cell[869966, 15883, 429, 12, 71, "Text",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[870420, 15899, 165, 4, 30, "Input",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[870588, 15905, 187, 6, 46, "Output",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[870790, 15914, 391, 8, 71, "Text",
      CellTags->{"i:37", "b:46.1", "IPM3.10.5.2"}],
    Cell[871184, 15924, 104, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[871291, 15929, 149, 4, 33, "Text",
      CellTags->{"i:38", "b:46.1", "IPM3.10.5.2"}],
    Cell[871465, 15937, 160, 4, 30, "Input",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[871628, 15943, 208, 6, 46, "Output",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[871851, 15952, 112, 1, 33, "Text",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[871966, 15955, 125, 3, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[872094, 15960, 623, 13, 90, "Text",
      CellTags->{"i:39", "b:46.1", "IPM3.10.5.2"}],
    Cell[872742, 15977, 206, 5, 30, "Input",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[872951, 15984, 254, 7, 46, "Output",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[873220, 15994, 204, 4, 52, "Text",
      CellTags->{"b:46.1", "IPM3.10.5.2"}],
    Cell[873427, 16000, 171, 4, 30, "Input",
      Evaluatable->False,
      CellTags->{"b:46.1", "IPM3.10.5.2"}]},
  "c:18"->{
    Cell[869514, 15867, 93, 1, 44, "Subsection",
      CellTags->{"b:46.1", "IPM3.10.5.2", "c:18"}]},
  "i:36"->{
    Cell[869610, 15870, 353, 11, 52, "TextFirst",
      CellTags->{"i:36", "b:46.1", "IPM3.10.5.2"}]},
  "i:37"->{
    Cell[870790, 15914, 391, 8, 71, "Text",
      CellTags->{"i:37", "b:46.1", "IPM3.10.5.2"}]},
  "i:38"->{
    Cell[871291, 15929, 149, 4, 33, "Text",
      CellTags->{"i:38", "b:46.1", "IPM3.10.5.2"}]},
  "i:39"->{
    Cell[872094, 15960, 623, 13, 90, "Text",
      CellTags->{"i:39", "b:46.1", "IPM3.10.5.2"}]},
  "b:46.2"->{
    Cell[873635, 16009, 80, 1, 44, "Subsection",
      CellTags->{"b:46.2", "IPM3.10.5.3", "c:19"}],
    Cell[873718, 16012, 779, 18, 128, "TextFirst",
      CellTags->{"i:40", "b:46.2", "IPM3.10.5.3"}],
    Cell[874500, 16032, 267, 6, 50, "Input",
      Evaluatable->False,
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[874770, 16040, 567, 13, 71, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[875340, 16055, 255, 7, 46, "Output",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[875598, 16064, 134, 4, 31, "Input",
      Evaluatable->False,
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[875757, 16072, 148, 4, 31, "Input",
      Evaluatable->False,
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[875908, 16078, 150, 3, 52, "Output",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[876073, 16084, 301, 8, 52, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[876377, 16094, 427, 14, 71, "Text",
      CellTags->{"i:41", "b:46.2", "IPM3.10.5.3"}],
    Cell[876807, 16110, 244, 9, 144, "Program",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[877054, 16121, 106, 1, 33, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[877163, 16124, 309, 9, 31, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[877475, 16135, 595, 16, 90, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[878073, 16153, 172, 5, 33, "Text",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[878248, 16160, 1521, 49, 213, "DisplayTable",
      CellTags->{"b:46.2", "IPM3.10.5.3"}],
    Cell[879772, 16211, 152, 5, 22, "NumberedTable",
      CellTags->{"b:46.2", "IPM3.10.5.3"}]},
  "c:19"->{
    Cell[873635, 16009, 80, 1, 44, "Subsection",
      CellTags->{"b:46.2", "IPM3.10.5.3", "c:19"}]},
  "i:40"->{
    Cell[873718, 16012, 779, 18, 128, "TextFirst",
      CellTags->{"i:40", "b:46.2", "IPM3.10.5.3"}]},
  "i:41"->{
    Cell[876377, 16094, 427, 14, 71, "Text",
      CellTags->{"i:41", "b:46.2", "IPM3.10.5.3"}]},
  "b:46.3"->{
    Cell[879961, 16221, 83, 1, 44, "Subsection",
      CellTags->{"b:46.3", "IPM3.10.5.4", "c:20"}],
    Cell[880047, 16224, 1068, 24, 147, "TextFirst",
      CellTags->{"i:42", "b:46.3", "IPM3.10.5.4"}],
    Cell[881140, 16252, 350, 7, 90, "Input",
      Evaluatable->False,
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[881493, 16261, 267, 8, 46, "Output",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[881775, 16272, 347, 7, 71, "Text",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[882147, 16283, 357, 7, 90, "Input",
      Evaluatable->False,
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[882507, 16292, 288, 9, 46, "Output",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[882798, 16303, 104, 3, 20, "Print",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[882917, 16309, 1297, 37, 166, "Text",
      CellTags->{"i:43", "b:46.3", "IPM3.10.5.4"}],
    Cell[884217, 16348, 262, 5, 52, "Text",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[884482, 16355, 128, 3, 30, "Input",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[884635, 16362, 143, 3, 30, "Input",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[884781, 16367, 27500, 618, 203, 6511, 353, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[912296, 16988, 253, 6, 52, "Text",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[912552, 16996, 434, 12, 195, "Program",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[912989, 17010, 126, 3, 33, "Text",
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[913140, 17017, 478, 15, 34, "Input",
      Evaluatable->False,
      CellTags->{"b:46.3", "IPM3.10.5.4"}],
    Cell[913621, 17034, 27497, 618, 203, 6511, 353, "GraphicsData", 
    "PostScript", "Graphics",
      CellTags->{"b:46.3", "IPM3.10.5.4"}]},
  "c:20"->{
    Cell[879961, 16221, 83, 1, 44, "Subsection",
      CellTags->{"b:46.3", "IPM3.10.5.4", "c:20"}]},
  "i:42"->{
    Cell[880047, 16224, 1068, 24, 147, "TextFirst",
      CellTags->{"i:42", "b:46.3", "IPM3.10.5.4"}]},
  "i:43"->{
    Cell[882917, 16309, 1297, 37, 166, "Text",
      CellTags->{"i:43", "b:46.3", "IPM3.10.5.4"}]},
  "b:46.4"->{
    Cell[941167, 17658, 96, 1, 44, "Subsection",
      CellTags->{"b:46.4", "IPM3.10.5.5", "c:21"}],
    Cell[941266, 17661, 294, 6, 52, "TextFirst",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[941563, 17669, 102, 1, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[941668, 17672, 233, 7, 110, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[941904, 17681, 736, 19, 109, "Text",
      CellTags->{"i:44", "b:46.4", "IPM3.10.5.5"}],
    Cell[942643, 17702, 319, 6, 90, "Input",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[942965, 17710, 109, 1, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[943077, 17713, 460, 15, 246, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[943540, 17730, 187, 4, 52, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[943730, 17736, 427, 13, 34, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[944182, 17753, 102, 3, 30, "Input",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[944287, 17758, 99, 3, 29, "Output",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[944401, 17764, 142, 4, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[944546, 17770, 789, 26, 433, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[945338, 17798, 147, 4, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[945488, 17804, 786, 25, 46, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[946299, 17833, 102, 3, 30, "Input",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[946404, 17838, 99, 3, 29, "Output",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[946518, 17844, 813, 18, 128, "Text",
      CellTags->{"i:45", "b:46.4", "IPM3.10.5.5"}],
    Cell[947334, 17864, 118, 3, 42, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[947455, 17869, 193, 6, 33, "Text",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[947651, 17877, 288, 9, 144, "Program",
      CellTags->{"b:46.4", "IPM3.10.5.5"}],
    Cell[947942, 17888, 364, 6, 71, "Text",
      CellTags->{"i:46", "b:46.4", "IPM3.10.5.5"}],
    Cell[948343, 17899, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:46.4", "IPM3.10.5.6", "c:22"}],
    Cell[948432, 17902, 177, 5, 27, "ExerciseItem",
      CellTags->{"b:46.4", "IPM3.10.5.6"}],
    Cell[948612, 17909, 207, 5, 27, "ExerciseItem",
      CellTags->{"b:46.4", "IPM3.10.5.6"}],
    Cell[948822, 17916, 398, 14, 46, "ExerciseItem",
      CellTags->{"b:46.4", "IPM3.10.5.6"}]},
  "c:21"->{
    Cell[941167, 17658, 96, 1, 44, "Subsection",
      CellTags->{"b:46.4", "IPM3.10.5.5", "c:21"}]},
  "i:44"->{
    Cell[941904, 17681, 736, 19, 109, "Text",
      CellTags->{"i:44", "b:46.4", "IPM3.10.5.5"}]},
  "i:45"->{
    Cell[946518, 17844, 813, 18, 128, "Text",
      CellTags->{"i:45", "b:46.4", "IPM3.10.5.5"}]},
  "i:46"->{
    Cell[947942, 17888, 364, 6, 71, "Text",
      CellTags->{"i:46", "b:46.4", "IPM3.10.5.5"}]},
  "c:22"->{
    Cell[948343, 17899, 86, 1, 77, "ExerciseSubsection",
      CellTags->{"b:46.4", "IPM3.10.5.6", "c:22"}]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"IPM3.10.0", 956232, 18070},
  {"IPM3.10.1", 956874, 18083},
  {"IPM3.10.2.0", 957871, 18104},
  {"IPM3.10.2.1", 958585, 18118},
  {"IPM3.10.2.2", 960876, 18167},
  {"IPM3.10.2.3", 965876, 18277},
  {"IPM3.10.3.0", 966906, 18297},
  {"IPM3.10.3.1", 967651, 18312},
  {"IPM3.10.3.2", 969893, 18359},
  {"IPM3.10.3.3", 972459, 18415},
  {"IPM3.10.3.4", 974282, 18453},
  {"IPM3.10.4.1", 975247, 18472},
  {"IPM3.10.4.2", 979106, 18555},
  {"IPM3.10.4.3", 983832, 18658},
  {"IPM3.10.5.0", 985975, 18701},
  {"IPM3.10.5.1", 986825, 18718},
  {"IPM3.10.5.2", 988595, 18756},
  {"IPM3.10.5.3", 990927, 18806},
  {"IPM3.10.5.4", 993264, 18856},
  {"IPM3.10.5.5", 995785, 18910},
  {"IPM3.10.5.6", 998576, 18969},
  {"c:1", 999659, 18990},
  {"b:42", 999797, 18994},
  {"c:2", 1000369, 19008},
  {"c:3", 1000531, 19012},
  {"b:43.0", 1000743, 19017},
  {"c:4", 1002412, 19055},
  {"i:1", 1002520, 19058},
  {"i:2", 1002629, 19061},
  {"i:3", 1002735, 19064},
  {"i:5", 1002844, 19067},
  {"i:6", 1002951, 19070},
  {"b:43.1", 1003060, 19073},
  {"c:5", 1007834, 19180},
  {"i:7", 1007945, 19183},
  {"i:8", 1008058, 19186},
  {"i:9", 1008164, 19189},
  {"i:10", 1008271, 19192},
  {"i:11", 1008379, 19195},
  {"i:12", 1008487, 19198},
  {"i:13", 1008623, 19202},
  {"i:14", 1008734, 19205},
  {"i:15", 1008871, 19209},
  {"c:6", 1008979, 19212},
  {"c:7", 1009099, 19215},
  {"i:16", 1009337, 19221},
  {"b:44.0", 1009460, 19224},
  {"c:8", 1011052, 19259},
  {"i:17", 1011173, 19262},
  {"i:18", 1011288, 19265},
  {"b:44.1", 1011400, 19268},
  {"c:9", 1013316, 19312},
  {"i:19", 1013429, 19315},
  {"i:20", 1013545, 19318},
  {"i:21", 1013657, 19321},
  {"i:22", 1013769, 19324},
  {"i:23", 1013881, 19327},
  {"b:44.2", 1013997, 19330},
  {"c:10", 1015474, 19362},
  {"i:24", 1015588, 19365},
  {"i:25", 1015702, 19368},
  {"c:11", 1015812, 19371},
  {"c:12", 1015934, 19374},
  {"b:45.0", 1016129, 19378},
  {"c:13", 1019384, 19451},
  {"i:26", 1019498, 19454},
  {"i:27", 1019613, 19457},
  {"i:28", 1019722, 19460},
  {"i:29", 1019831, 19463},
  {"i:30", 1019941, 19466},
  {"b:45.1", 1020053, 19469},
  {"c:14", 1025701, 19594},
  {"i:31", 1025815, 19597},
  {"i:32", 1025929, 19600},
  {"i:33", 1026039, 19603},
  {"c:15", 1026151, 19606},
  {"i:34", 1026273, 19609},
  {"c:16", 1026391, 19612},
  {"b:46.0", 1026662, 19618},
  {"c:17", 1027752, 19644},
  {"i:35", 1027866, 19647},
  {"b:46.1", 1027983, 19650},
  {"c:18", 1029635, 19688},
  {"i:36", 1029749, 19691},
  {"i:37", 1029864, 19694},
  {"i:38", 1029973, 19697},
  {"i:39", 1030082, 19700},
  {"b:46.2", 1030194, 19703},
  {"c:19", 1031851, 19741},
  {"i:40", 1031965, 19744},
  {"i:41", 1032081, 19747},
  {"b:46.3", 1032193, 19750},
  {"c:20", 1034034, 19792},
  {"i:42", 1034148, 19795},
  {"i:43", 1034265, 19798},
  {"b:46.4", 1034379, 19801},
  {"c:21", 1036888, 19856},
  {"i:44", 1037002, 19859},
  {"i:45", 1037113, 19862},
  {"i:46", 1037224, 19865},
  {"c:22", 1037333, 19868}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 469, 10, 29, "TitleBar",
  CellTags->{
  "IPM3.10.0", "IPM3.10.1", "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", 
    "IPM3.10.2.3", "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3",
     "IPM3.10.3.4", "IPM3.10.4.1", "IPM3.10.4.2", "IPM3.10.4.3", 
    "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", "IPM3.10.5.4",
     "IPM3.10.5.5", "IPM3.10.5.6"}],

Cell[CellGroupData[{
Cell[2248, 65, 159, 6, 96, "Chapter",
  CounterAssignments->{{"Chapter", 9}},
  CellTags->{"IPM3.10.0", "c:1"}],
Cell[2410, 73, 730, 17, 116, "ChapterAbstract",
  CellTags->"IPM3.10.0"],

Cell[CellGroupData[{
Cell[3165, 94, 130, 2, 40, "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 0}},
  CellTags->{"b:42", "IPM3.10.1", "c:2"}],
Cell[3298, 98, 555, 11, 90, "TextFirst",
  CellTags->{"b:42", "IPM3.10.1"}],
Cell[3856, 111, 439, 9, 71, "Text",
  CellTags->{"b:42", "IPM3.10.1"}],
Cell[4298, 122, 789, 14, 128, "Text",
  CellTags->{"b:42", "IPM3.10.1"}],
Cell[5090, 138, 336, 6, 71, "Text",
  CellTags->{"b:42", "IPM3.10.1"}],
Cell[5429, 146, 1213, 24, 185, "Text",
  CellTags->{"b:42", "IPM3.10.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[6679, 175, 196, 3, 40, "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 1}},
  CellTags->{
  "IPM3.10.2.0", "IPM3.10.2.1", "IPM3.10.2.2", "IPM3.10.2.3", "c:3"}],
Cell[6878, 180, 1013, 29, 109, "TextFirst",
  CellTags->"IPM3.10.2.0"],

Cell[CellGroupData[{
Cell[7916, 213, 88, 1, 44, "Subsection",
  CellTags->{"b:43.0", "IPM3.10.2.1", "c:4"}],
Cell[8007, 216, 419, 10, 71, "TextFirst",
  CellTags->{"i:1", "b:43.0", "IPM3.10.2.1"}],
Cell[8429, 228, 383, 17, 127, "Program",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[8815, 247, 517, 14, 71, "Text",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[9335, 263, 190, 4, 52, "Text",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[9528, 269, 201, 7, 110, "Program",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[9732, 278, 206, 6, 33, "TextFirst",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[9941, 286, 79489, 1306, 230, 79375, 1302, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False],
Cell[89433, 1594, 426, 12, 52, "Text",
  CellTags->{"i:2", "b:43.0", "IPM3.10.2.1"}],

Cell[CellGroupData[{
Cell[89884, 1610, 364, 8, 150, "Input",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[90251, 1620, 294, 8, 29, "Output",
  CellTags->{"b:43.0", "IPM3.10.2.1"}]
}, Open  ]],
Cell[90560, 1631, 132, 2, 33, "TextFirst",
  PageBreakAbove->True,
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[90695, 1635, 79489, 1306, 230, 79375, 1302, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False],
Cell[170187, 2943, 1218, 32, 147, "Text",
  CellTags->{"i:3", "b:43.0", "IPM3.10.2.1"}],

Cell[CellGroupData[{
Cell[171430, 2979, 109, 3, 30, "Input",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[171542, 2984, 169, 5, 60, "Output",
  CellTags->{"b:43.0", "IPM3.10.2.1"}]
}, Open  ]],
Cell[171726, 2992, 406, 10, 71, "Text",
  CellTags->{"i:5", "b:43.0", "IPM3.10.2.1"}],

Cell[CellGroupData[{
Cell[172157, 3006, 111, 3, 30, "Input",
  CellTags->{"b:43.0", "IPM3.10.2.1"}],
Cell[172271, 3011, 385, 7, 86, "Output",
  CellTags->{"b:43.0", "IPM3.10.2.1"}]
}, Open  ]],
Cell[172671, 3021, 417, 9, 71, "Text",
  CellTags->{"i:6", "b:43.0", "IPM3.10.2.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[173125, 3035, 90, 1, 44, "Subsection",
  CellTags->{"b:43.1", "IPM3.10.2.2", "c:5"}],
Cell[173218, 3038, 904, 16, 147, "TextFirst",
  CellTags->{"i:7", "b:43.1", "IPM3.10.2.2"}],
Cell[174125, 3056, 379, 7, 71, "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[174507, 3065, 159, 5, 33, "Text",
  CellTags->{"i:8", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[174691, 3074, 107, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[174801, 3079, 827, 22, 48, "Output",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]],
Cell[175643, 3104, 179, 5, 33, "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[175847, 3113, 120, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[175970, 3118, 315, 8, 111, "Output",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]],
Cell[176300, 3129, 391, 9, 71, "Text",
  CellTags->{"i:9", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[176716, 3142, 117, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[176836, 3147, 294, 8, 29, "Output",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[177133, 3157, 60278, 991, 239, 60164, 987, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False]
}, Open  ]],
Cell[237426, 4151, 336, 7, 71, "Text",
  CellTags->{"i:10", "b:43.1", "IPM3.10.2.2"}],
Cell[237765, 4160, 352, 9, 52, "Text",
  CellTags->{"i:11", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[238142, 4173, 164, 4, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[238309, 4179, 69184, 1137, 239, 69070, 1133, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False]
}, Open  ]],
Cell[307508, 5319, 274, 7, 52, "Text",
  PageBreakAbove->True,
  CellTags->{"i:12", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[307807, 5330, 125, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[307935, 5335, 269, 5, 48, "Output",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[308241, 5345, 299, 7, 70, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[308543, 5354, 75792, 1245, 239, 75678, 1241, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False]
}, Open  ]],
Cell[384350, 6602, 292, 10, 52, "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[384667, 6616, 140, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[384810, 6621, 80005, 1314, 239, 79891, 1310, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False]
}, Open  ]],
Cell[464830, 7938, 228, 7, 33, "Text",
  PageBreakAbove->True,
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[465083, 7949, 103, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[465189, 7954, 129, 3, 29, "Output",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]],
Cell[465333, 7960, 269, 8, 52, "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[465605, 7970, 127, 4, 59, "Program",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[465735, 7976, 1094, 33, 128, "Text",
  CellTags->{"i:13", "b:43.1", "IPM3.10.2.2"}],
Cell[466832, 8011, 143, 4, 33, "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[467000, 8019, 127, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[467130, 8024, 79623, 1308, 239, 79509, 1304, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False]
}, Open  ]],
Cell[546768, 9335, 127, 2, 33, "Text",
  PageBreakAbove->True,
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[546920, 9341, 196, 5, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[547119, 9348, 80762, 1327, 239, 80648, 1323, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False]
}, Open  ]],
Cell[627896, 10678, 687, 16, 109, "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[628608, 10698, 130, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[628741, 10703, 81006, 1331, 239, 80892, 1327, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False]
}, Open  ]],
Cell[709762, 12037, 201, 6, 33, "Text",
  PageBreakAbove->True,
  CellTags->{"i:14", "b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[709988, 12047, 118, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[710109, 12052, 89180, 1465, 239, 89066, 1461, "GraphicsData", "Bitmap", \
"Picture",
  Evaluatable->False]
}, Open  ]],
Cell[799304, 13520, 225, 5, 52, "Text",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[799532, 13527, 354, 7, 71, "Text",
  CellTags->{"i:15", "b:43.1", "IPM3.10.2.2"}],
Cell[799889, 13536, 102, 1, 42, "Program",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[799994, 13539, 213, 4, 52, "TextFirst",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[800210, 13545, 103, 1, 42, "Program",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[800316, 13548, 110, 1, 33, "TextFirst",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[800429, 13551, 97, 1, 42, "Program",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[800529, 13554, 148, 5, 33, "TextFirst",
  PageBreakAbove->True],
Cell[800680, 13561, 376, 7, 110, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[801059, 13570, 202, 5, 33, "TextFirst",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[801264, 13577, 109, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],

Cell[CellGroupData[{
Cell[801398, 13584, 102, 3, 30, "Input",
  CellTags->{"b:43.1", "IPM3.10.2.2"}],
Cell[801503, 13589, 99, 3, 29, "Output",
  CellTags->{"b:43.1", "IPM3.10.2.2"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[801651, 13598, 85, 1, 77, "ExerciseSubsection",
  CellTags->{"b:43.1", "IPM3.10.2.3", "c:6"}],
Cell[801739, 13601, 219, 6, 46, "ExerciseItem",
  CellTags->{"b:43.1", "IPM3.10.2.3"}],
Cell[801961, 13609, 326, 8, 46, "ExerciseItem",
  CellTags->{"b:43.1", "IPM3.10.2.3"}],
Cell[802290, 13619, 294, 7, 46, "ExerciseItem",
  CellTags->{"b:43.1", "IPM3.10.2.3"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[802633, 13632, 195, 4, 40, "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 2}},
  CellTags->{
  "IPM3.10.3.0", "IPM3.10.3.1", "IPM3.10.3.2", "IPM3.10.3.3", "IPM3.10.3.4", 
    "c:7"}],
Cell[802831, 13638, 598, 10, 109, "TextFirst",
  CellTags->"IPM3.10.3.0"],

Cell[CellGroupData[{
Cell[803454, 13652, 84, 1, 44, "Subsection",
  CellTags->{"i:16", "b:44.0", "IPM3.10.3.1", "c:8"}],
Cell[803541, 13655, 138, 3, 33, "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[803682, 13660, 96, 1, 42, "Program",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[803781, 13663, 111, 1, 33, "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[803895, 13666, 126, 2, 47, "Text",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[804024, 13670, 178, 5, 33, "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[804205, 13677, 178, 6, 93, "Program",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[804386, 13685, 111, 1, 33, "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[804500, 13688, 200, 7, 47, "Text",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[804703, 13697, 392, 10, 52, "TextFirst",
  CellTags->{"i:17", "b:44.0", "IPM3.10.3.1"}],
Cell[805098, 13709, 150, 5, 76, "Program",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[805251, 13716, 111, 1, 33, "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[805365, 13719, 175, 6, 47, "Text",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[805543, 13727, 477, 14, 71, "Text",
  CellTags->{"i:18", "b:44.0", "IPM3.10.3.1"}],
Cell[806023, 13743, 149, 6, 93, "Program",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[806175, 13751, 111, 1, 33, "TextFirst",
  CellTags->{"b:44.0", "IPM3.10.3.1"}],
Cell[806289, 13754, 179, 6, 40, "Text",
  CellTags->{"b:44.0", "IPM3.10.3.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[806505, 13765, 75, 1, 44, "Subsection",
  CellTags->{"b:44.1", "IPM3.10.3.2", "c:9"}],
Cell[806583, 13768, 731, 16, 109, "TextFirst",
  CellTags->{"i:19", "b:44.1", "IPM3.10.3.2"}],
Cell[807317, 13786, 164, 5, 76, "Program",
  CellTags->{"i:20", "b:44.1", "IPM3.10.3.2"}],
Cell[807484, 13793, 111, 1, 33, "TextFirst",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[807598, 13796, 102, 3, 30, "Input",
  Evaluatable->False,
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[807703, 13801, 472, 14, 71, "Text",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[808178, 13817, 386, 12, 52, "Text",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],

Cell[CellGroupData[{
Cell[808589, 13833, 131, 3, 30, "Input",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[808723, 13838, 141, 4, 43, "Output",
  CellTags->{"b:44.1", "IPM3.10.3.2"}]
}, Open  ]],
Cell[808879, 13845, 242, 7, 33, "Text",
  PageBreakAbove->True,
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[809124, 13854, 119, 4, 59, "Program",
  CellTags->{"i:21", "b:44.1", "IPM3.10.3.2"}],
Cell[809246, 13860, 111, 1, 33, "TextFirst",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[809360, 13863, 97, 3, 32, "Input",
  Evaluatable->False,
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[809460, 13868, 130, 4, 59, "Program",
  CellTags->{"i:22", "b:44.1", "IPM3.10.3.2"}],
Cell[809593, 13874, 111, 1, 33, "TextFirst",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[809707, 13877, 98, 3, 42, "Input",
  Evaluatable->False,
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[809808, 13882, 264, 10, 161, "Program",
  CellTags->{"i:23", "b:44.1", "IPM3.10.3.2"}],
Cell[810075, 13894, 111, 1, 33, "TextFirst",
  CellTags->{"b:44.1", "IPM3.10.3.2"}],
Cell[810189, 13897, 132, 3, 42, "Input",
  Evaluatable->False,
  CellTags->{"b:44.1", "IPM3.10.3.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[810358, 13905, 81, 1, 44, "Subsection",
  CellTags->{"b:44.2", "IPM3.10.3.3", "c:10"}],
Cell[810442, 13908, 306, 6, 52, "TextFirst",
  CellTags->{"i:24", "b:44.2", "IPM3.10.3.3"}],

Cell[CellGroupData[{
Cell[810773, 13918, 131, 3, 30, "Input",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],
Cell[810907, 13923, 24743, 803, 181, 13969, 664, "GraphicsData", \
"PostScript", "Graphics",
  CellTags->{"b:44.2", "IPM3.10.3.3"}]
}, Open  ]],
Cell[835665, 14729, 155, 4, 33, "TextFirst",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],
Cell[835823, 14735, 177, 8, 127, "Program",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],
Cell[836003, 14745, 542, 11, 90, "Text",
  CellTags->{"i:25", "b:44.2", "IPM3.10.3.3"}],
Cell[836548, 14758, 271, 7, 90, "Input",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],
Cell[836822, 14767, 228, 7, 52, "Text",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],

Cell[CellGroupData[{
Cell[837075, 14778, 198, 4, 50, "Input",
  CellTags->{"b:44.2", "IPM3.10.3.3"}],
Cell[837276, 14784, 294, 8, 29, "Output",
  CellTags->{"b:44.2", "IPM3.10.3.3"}]
}, Open  ]],
Cell[837585, 14795, 336, 9, 52, "Text",
  CellTags->{"b:44.2", "IPM3.10.3.3"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[837958, 14809, 86, 1, 77, "ExerciseSubsection",
  CellTags->{"b:44.2", "IPM3.10.3.4", "c:11"}],
Cell[838047, 14812, 357, 13, 46, "ExerciseItem",
  CellTags->{"b:44.2", "IPM3.10.3.4"}],
Cell[838407, 14827, 432, 13, 49, "ExerciseItem",
  CellTags->{"b:44.2", "IPM3.10.3.4"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[838888, 14846, 152, 2, 40, "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 3}},
  CellTags->{"IPM3.10.4.1", "IPM3.10.4.2", "IPM3.10.4.3", "c:12"}],

Cell[CellGroupData[{
Cell[839065, 14852, 78, 1, 44, "Subsection",
  CellTags->{"b:45.0", "IPM3.10.4.1", "c:13"}],
Cell[839146, 14855, 484, 12, 71, "TextFirst",
  CellTags->{"i:26", "b:45.0", "IPM3.10.4.1"}],
Cell[839633, 14869, 159, 6, 93, "Program",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[839795, 14877, 111, 1, 33, "TextFirst",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[839909, 14880, 144, 5, 41, "Input",
  Evaluatable->False,
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[840056, 14887, 296, 9, 52, "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{
Cell[840377, 14900, 189, 6, 41, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[840569, 14908, 277, 8, 43, "Output",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[840883, 14921, 126, 3, 30, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[841012, 14926, 302, 9, 55, "Output",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],
Cell[841329, 14938, 228, 6, 52, "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[841560, 14946, 274, 5, 64, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[841837, 14953, 240, 7, 52, "Text",
  CellTags->{"i:27", "b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{
Cell[842102, 14964, 124, 3, 30, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[842229, 14969, 309, 8, 83, "Output",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],
Cell[842553, 14980, 167, 4, 33, "Text",
  CellTags->{"i:28", "b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{
Cell[842745, 14988, 112, 3, 30, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[842860, 14993, 753, 13, 143, "Output",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],
Cell[843628, 15009, 629, 17, 90, "Text",
  CellTags->{"i:29", "b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{
Cell[844282, 15030, 304, 6, 70, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[844589, 15038, 416, 12, 101, "Output",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],
Cell[845020, 15053, 558, 20, 71, "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[845581, 15075, 578, 12, 150, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{
Cell[846184, 15091, 111, 3, 30, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[846298, 15096, 549, 16, 107, "Output",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],
Cell[846862, 15115, 418, 8, 71, "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[847283, 15125, 156, 3, 30, "Input",
  CellTags->{"i:30", "b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{
Cell[847464, 15132, 135, 3, 44, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[847602, 15137, 247, 5, 76, "Output",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],
Cell[847864, 15145, 240, 5, 52, "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{
Cell[848129, 15154, 126, 3, 30, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[848258, 15159, 626, 17, 149, "Output",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],
Cell[848899, 15179, 138, 2, 33, "Text",
  PageBreakAbove->True,
  CellTags->{"b:45.0", "IPM3.10.4.1"}],

Cell[CellGroupData[{
Cell[849062, 15185, 126, 3, 30, "Input",
  CellTags->{"b:45.0", "IPM3.10.4.1"}],
Cell[849191, 15190, 575, 16, 121, "Output"]
}, Open  ]],
Cell[849781, 15209, 334, 6, 71, "Text",
  CellTags->{"b:45.0", "IPM3.10.4.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[850152, 15220, 81, 1, 44, "Subsection",
  CellTags->{"b:45.1", "IPM3.10.4.2", "c:14"}],
Cell[850236, 15223, 354, 7, 52, "TextFirst",
  CellTags->{"i:31", "b:45.1", "IPM3.10.4.2"}],
Cell[850593, 15232, 974, 21, 153, "Output"],
Cell[851570, 15255, 225, 6, 33, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[851798, 15263, 180, 4, 30, "Input",
  CellTags->{"i:32", "b:45.1", "IPM3.10.4.2"}],
Cell[851981, 15269, 108, 1, 33, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[852114, 15274, 118, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[852235, 15279, 136, 3, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[852386, 15285, 201, 5, 52, "Text",
  PageBreakAbove->True,
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[852612, 15294, 332, 6, 110, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[852947, 15302, 257, 8, 71, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[853219, 15313, 300, 6, 71, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[853522, 15321, 217, 5, 50, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[853764, 15330, 115, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[853882, 15335, 192, 4, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[854111, 15344, 115, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[854229, 15349, 181, 4, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[854425, 15356, 186, 4, 52, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[854636, 15364, 133, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[854772, 15369, 118, 3, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[854927, 15377, 138, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[855068, 15382, 135, 3, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[855240, 15390, 143, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[855386, 15395, 152, 3, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[855553, 15401, 160, 5, 33, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[855738, 15410, 141, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[855882, 15415, 113, 3, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[856010, 15421, 178, 5, 33, "Text",
  PageBreakAbove->True,
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[856191, 15428, 178, 4, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[856394, 15436, 120, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[856517, 15441, 135, 3, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[856689, 15449, 120, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[856812, 15454, 101, 3, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[856928, 15460, 167, 5, 33, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[857120, 15469, 133, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[857256, 15474, 238, 5, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[857509, 15482, 163, 5, 33, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[857697, 15491, 111, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[857811, 15496, 417, 7, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[858243, 15506, 160, 5, 33, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[858428, 15515, 122, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[858553, 15520, 325, 9, 87, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]],
Cell[858893, 15532, 169, 5, 33, "Text",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[859065, 15539, 555, 11, 209, "Input",
  CellTags->{"i:33", "b:45.1", "IPM3.10.4.2"}],

Cell[CellGroupData[{
Cell[859645, 15554, 127, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.2"}],
Cell[859775, 15559, 514, 11, 119, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.2"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[860338, 15576, 86, 1, 77, "ExerciseSubsection",
  CellTags->{"b:45.1", "IPM3.10.4.3", "c:15"}],
Cell[860427, 15579, 369, 9, 65, "ExerciseItem",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],
Cell[860799, 15590, 276, 7, 46, "ExerciseItem",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{
Cell[861100, 15601, 599, 19, 65, "ExerciseItem",
  CellTags->{"i:34", "b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{
Cell[861724, 15624, 137, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],
Cell[861864, 15629, 707, 22, 177, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]],
Cell[862586, 15654, 504, 13, 65, "ExercisesNoLabel",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{
Cell[863115, 15671, 260, 6, 70, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],
Cell[863378, 15679, 287, 5, 67, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]],
Cell[863680, 15687, 190, 4, 27, "ExercisesNoLabel",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{
Cell[863895, 15695, 131, 3, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],
Cell[864029, 15700, 572, 10, 86, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]],
Cell[864616, 15713, 180, 4, 27, "ExercisesNoLabel",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],

Cell[CellGroupData[{
Cell[864821, 15721, 179, 4, 30, "Input",
  CellTags->{"b:45.1", "IPM3.10.4.3"}],
Cell[865003, 15727, 148, 3, 29, "Output",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]],
Cell[865166, 15733, 198, 5, 27, "ExercisesNoLabel",
  CellTags->{"b:45.1", "IPM3.10.4.3"}]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[865425, 15745, 218, 4, 40, "Section",
  CounterAssignments->{{"Chapter", 10}, {"Section", 4}},
  CellTags->{
  "IPM3.10.5.0", "IPM3.10.5.1", "IPM3.10.5.2", "IPM3.10.5.3", "IPM3.10.5.4", 
    "IPM3.10.5.5", "IPM3.10.5.6", "c:16"}],
Cell[865646, 15751, 502, 8, 90, "TextFirst",
  CellTags->"IPM3.10.5.0"],
Cell[866151, 15761, 260, 6, 52, "Text",
  CellTags->"IPM3.10.5.0"],

Cell[CellGroupData[{
Cell[866436, 15771, 96, 1, 44, "Subsection",
  CellTags->{"b:46.0", "IPM3.10.5.1", "c:17"}],
Cell[866535, 15774, 450, 11, 71, "TextFirst",
  CellTags->{"i:35", "b:46.0", "IPM3.10.5.1"}],
Cell[866988, 15787, 520, 10, 90, "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],
Cell[867511, 15799, 125, 3, 30, "Input",
  Evaluatable->False,
  CellTags->{"b:46.0", "IPM3.10.5.1"}],
Cell[867639, 15804, 229, 7, 33, "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],
Cell[867871, 15813, 164, 4, 32, "Input",
  Evaluatable->False,
  CellTags->{"b:46.0", "IPM3.10.5.1"}],
Cell[868038, 15819, 334, 8, 52, "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],
Cell[868375, 15829, 180, 5, 32, "Input",
  Evaluatable->False,
  CellTags->{"b:46.0", "IPM3.10.5.1"}],
Cell[868558, 15836, 233, 7, 33, "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],
Cell[868794, 15845, 265, 8, 52, "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}],
Cell[869062, 15855, 415, 7, 90, "Text",
  CellTags->{"b:46.0", "IPM3.10.5.1"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[869514, 15867, 93, 1, 44, "Subsection",
  CellTags->{"b:46.1", "IPM3.10.5.2", "c:18"}],
Cell[869610, 15870, 353, 11, 52, "TextFirst",
  CellTags->{"i:36", "b:46.1", "IPM3.10.5.2"}],
Cell[869966, 15883, 429, 12, 71, "Text",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],

Cell[CellGroupData[{
Cell[870420, 15899, 165, 4, 30, "Input",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],
Cell[870588, 15905, 187, 6, 46, "Output",
  CellTags->{"b:46.1", "IPM3.10.5.2"}]
}, Open  ]],
Cell[870790, 15914, 391, 8, 71, "Text",
  CellTags->{"i:37", "b:46.1", "IPM3.10.5.2"}],
Cell[871184, 15924, 104, 3, 30, "Input",
  Evaluatable->False,
  CellTags->{"b:46.1", "IPM3.10.5.2"}],
Cell[871291, 15929, 149, 4, 33, "Text",
  CellTags->{"i:38", "b:46.1", "IPM3.10.5.2"}],

Cell[CellGroupData[{
Cell[871465, 15937, 160, 4, 30, "Input",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],
Cell[871628, 15943, 208, 6, 46, "Output",
  CellTags->{"b:46.1", "IPM3.10.5.2"}]
}, Open  ]],
Cell[871851, 15952, 112, 1, 33, "Text",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],
Cell[871966, 15955, 125, 3, 30, "Input",
  Evaluatable->False,
  CellTags->{"b:46.1", "IPM3.10.5.2"}],
Cell[872094, 15960, 623, 13, 90, "Text",
  CellTags->{"i:39", "b:46.1", "IPM3.10.5.2"}],

Cell[CellGroupData[{
Cell[872742, 15977, 206, 5, 30, "Input",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],
Cell[872951, 15984, 254, 7, 46, "Output",
  CellTags->{"b:46.1", "IPM3.10.5.2"}]
}, Open  ]],
Cell[873220, 15994, 204, 4, 52, "Text",
  CellTags->{"b:46.1", "IPM3.10.5.2"}],
Cell[873427, 16000, 171, 4, 30, "Input",
  Evaluatable->False,
  CellTags->{"b:46.1", "IPM3.10.5.2"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[873635, 16009, 80, 1, 44, "Subsection",
  CellTags->{"b:46.2", "IPM3.10.5.3", "c:19"}],
Cell[873718, 16012, 779, 18, 128, "TextFirst",
  CellTags->{"i:40", "b:46.2", "IPM3.10.5.3"}],
Cell[874500, 16032, 267, 6, 50, "Input",
  Evaluatable->False,
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[874770, 16040, 567, 13, 71, "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[875340, 16055, 255, 7, 46, "Output",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[875598, 16064, 134, 4, 31, "Input",
  Evaluatable->False,
  CellTags->{"b:46.2", "IPM3.10.5.3"}],

Cell[CellGroupData[{
Cell[875757, 16072, 148, 4, 31, "Input",
  Evaluatable->False,
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[875908, 16078, 150, 3, 52, "Output",
  CellTags->{"b:46.2", "IPM3.10.5.3"}]
}, Open  ]],
Cell[876073, 16084, 301, 8, 52, "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[876377, 16094, 427, 14, 71, "Text",
  CellTags->{"i:41", "b:46.2", "IPM3.10.5.3"}],
Cell[876807, 16110, 244, 9, 144, "Program",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[877054, 16121, 106, 1, 33, "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[877163, 16124, 309, 9, 31, "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[877475, 16135, 595, 16, 90, "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[878073, 16153, 172, 5, 33, "Text",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[878248, 16160, 1521, 49, 213, "DisplayTable",
  CellTags->{"b:46.2", "IPM3.10.5.3"}],
Cell[879772, 16211, 152, 5, 22, "NumberedTable",
  CellTags->{"b:46.2", "IPM3.10.5.3"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[879961, 16221, 83, 1, 44, "Subsection",
  CellTags->{"b:46.3", "IPM3.10.5.4", "c:20"}],
Cell[880047, 16224, 1068, 24, 147, "TextFirst",
  CellTags->{"i:42", "b:46.3", "IPM3.10.5.4"}],

Cell[CellGroupData[{
Cell[881140, 16252, 350, 7, 90, "Input",
  Evaluatable->False,
  CellTags->{"b:46.3", "IPM3.10.5.4"}],
Cell[881493, 16261, 267, 8, 46, "Output",
  CellTags->{"b:46.3", "IPM3.10.5.4"}]
}, Open  ]],
Cell[881775, 16272, 347, 7, 71, "Text",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[CellGroupData[{
Cell[882147, 16283, 357, 7, 90, "Input",
  Evaluatable->False,
  CellTags->{"b:46.3", "IPM3.10.5.4"}],
Cell[882507, 16292, 288, 9, 46, "Output",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],
Cell[882798, 16303, 104, 3, 20, "Print",
  CellTags->{"b:46.3", "IPM3.10.5.4"}]
}, Open  ]],
Cell[882917, 16309, 1297, 37, 166, "Text",
  CellTags->{"i:43", "b:46.3", "IPM3.10.5.4"}],
Cell[884217, 16348, 262, 5, 52, "Text",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],
Cell[884482, 16355, 128, 3, 30, "Input",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[CellGroupData[{
Cell[884635, 16362, 143, 3, 30, "Input",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],
Cell[884781, 16367, 27500, 618, 203, 6511, 353, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->{"b:46.3", "IPM3.10.5.4"}]
}, Open  ]],
Cell[912296, 16988, 253, 6, 52, "Text",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],
Cell[912552, 16996, 434, 12, 195, "Program",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],
Cell[912989, 17010, 126, 3, 33, "Text",
  CellTags->{"b:46.3", "IPM3.10.5.4"}],

Cell[CellGroupData[{
Cell[913140, 17017, 478, 15, 34, "Input",
  Evaluatable->False,
  CellTags->{"b:46.3", "IPM3.10.5.4"}],
Cell[913621, 17034, 27497, 618, 203, 6511, 353, "GraphicsData", "PostScript", \
"Graphics",
  CellTags->{"b:46.3", "IPM3.10.5.4"}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[941167, 17658, 96, 1, 44, "Subsection",
  CellTags->{"b:46.4", "IPM3.10.5.5", "c:21"}],
Cell[941266, 17661, 294, 6, 52, "TextFirst",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[941563, 17669, 102, 1, 33, "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[941668, 17672, 233, 7, 110, "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[941904, 17681, 736, 19, 109, "Text",
  CellTags->{"i:44", "b:46.4", "IPM3.10.5.5"}],
Cell[942643, 17702, 319, 6, 90, "Input",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[942965, 17710, 109, 1, 33, "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[943077, 17713, 460, 15, 246, "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[943540, 17730, 187, 4, 52, "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[943730, 17736, 427, 13, 34, "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[CellGroupData[{
Cell[944182, 17753, 102, 3, 30, "Input",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[944287, 17758, 99, 3, 29, "Output",
  CellTags->{"b:46.4", "IPM3.10.5.5"}]
}, Open  ]],
Cell[944401, 17764, 142, 4, 33, "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[944546, 17770, 789, 26, 433, "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[945338, 17798, 147, 4, 33, "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[945488, 17804, 786, 25, 46, "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],

Cell[CellGroupData[{
Cell[946299, 17833, 102, 3, 30, "Input",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[946404, 17838, 99, 3, 29, "Output",
  CellTags->{"b:46.4", "IPM3.10.5.5"}]
}, Open  ]],
Cell[946518, 17844, 813, 18, 128, "Text",
  CellTags->{"i:45", "b:46.4", "IPM3.10.5.5"}],
Cell[947334, 17864, 118, 3, 42, "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[947455, 17869, 193, 6, 33, "Text",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[947651, 17877, 288, 9, 144, "Program",
  CellTags->{"b:46.4", "IPM3.10.5.5"}],
Cell[947942, 17888, 364, 6, 71, "Text",
  CellTags->{"i:46", "b:46.4", "IPM3.10.5.5"}]
}, Open  ]],

Cell[CellGroupData[{
Cell[948343, 17899, 86, 1, 77, "ExerciseSubsection",
  CellTags->{"b:46.4", "IPM3.10.5.6", "c:22"}],
Cell[948432, 17902, 177, 5, 27, "ExerciseItem",
  CellTags->{"b:46.4", "IPM3.10.5.6"}],
Cell[948612, 17909, 207, 5, 27, "ExerciseItem",
  CellTags->{"b:46.4", "IPM3.10.5.6"}],
Cell[948822, 17916, 398, 14, 46, "ExerciseItem",
  CellTags->{"b:46.4", "IPM3.10.5.6"}]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

