(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     22586,        851]*)
(*NotebookOutlinePosition[     31745,       1137]*)
(*  CellTagsIndexPosition[     30053,       1075]*)
(*WindowFrame->Normal*)



Notebook[{
Cell[TextData[{
  "Introduction to Programming with ",
  StyleBox["Mathematica",
    FontSlant->"Italic"]
}], "TitleBar"],

Cell[CellGroupData[{

Cell["References", "Chapter",
  CellTags->"c:1"],

Cell[TextData[{
  StyleBox["Abbott, P",
    FontFamily->"JansonText RomanSC"],
  ". Finding roots in an interval. ",
  StyleBox["The Mathematica Journal",
    FontSlant->"Italic"],
  ", ",
  StyleBox["7",
    FontWeight->"Bold"],
  "(2)108\[Dash]112, 1998."
}], "References",
  CellTags->"Bentley79"],

Cell[TextData[{
  StyleBox["Bentley, J.\[ThinSpace]L",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Friedman, J.\[ThinSpace]H",
    FontFamily->"JansonText RomanSC"],
  ". Algorithms for reporting and counting geometric intersections. Technical \
Report C-28, ",
  StyleBox["IEEE Transactions on Computing",
    FontSlant->"Italic"],
  ", 1979."
}], "References",
  CellTags->"Bentley79"],

Cell[TextData[{
  StyleBox["Burden, R.\[ThinSpace]L",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Faires, J.\[ThinSpace]D",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Numerical Analysis",
    FontSlant->"Italic"],
  ". Pacific Grove, CA: Brooks/Cole Publishing Co., 7th edition, 2000."
}], "References",
  CellTags->"Burden89"],

Cell[TextData[{
  StyleBox["Casti, J",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Reality Rules I, Picturing the World in Mathematics \[Dash] The \
Fundamentals",
    FontSlant->"Italic"],
  ". New York: John Wiley & Sons, Inc., 1992."
}], "References",
  CellTags->"Casti92"],

Cell[TextData[{
  StyleBox["Casti, J",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Reality Rules II, Picturing the World in Mathematics \[Dash] The \
Frontier",
    FontSlant->"Italic"],
  ". New York: John Wiley & Sons, Inc., 1992."
}], "References",
  CellTags->"Casti292"],

Cell[TextData[{
  StyleBox["Crandall, R.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  ". Personal communication, March 1993."
}], "References",
  CellTags->"Crandall93"],

Cell[TextData[{
  StyleBox["Crandall, R.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Projects in Scientific Computation",
    FontSlant->"Italic"],
  ". Santa Clara: TELOS/Springer-Verlag, 1994."
}], "References",
  CellTags->"Crandall94"],

Cell[TextData[{
  StyleBox["Crandall, R.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Topics in Advanced Scientific Computation",
    FontSlant->"Italic"],
  ". Santa Clara: TELOS/Springer-Verlag, 1996."
}], "References",
  CellTags->"Crandall96"],

Cell[TextData[{
  StyleBox["Crandall",
    FontFamily->"JansonText RomanSC"],
  ", ",
  StyleBox["R.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Pomerance",
    FontFamily->"JansonText RomanSC"],
  ", ",
  StyleBox["C",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Prime Numbers: A Computational Perspective.",
    FontSlant->"Italic"],
  " New York: Springer-Verlag, 2001"
}], "References",
  CellTags->"Crandall96"],

Cell[TextData[{
  StyleBox["de Berg, M.",
    FontFamily->"JansonText RomanSC"],
  ", ",
  StyleBox["van Kreveld, M",
    FontFamily->"JansonText RomanSC"],
  "., ",
  StyleBox["Overmars,",
    FontFamily->"JansonText RomanSC"],
  " and ",
  StyleBox["M",
    FontFamily->"JansonText RomanSC"],
  "., ",
  StyleBox["Schwarzkopf, O",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Computational Geometry, Algorithms and Applications",
    FontSlant->"Italic"],
  ". Heidelberg: Springer-Verlag, 2nd edition, 2000."
}], "References",
  CellTags->"deBerg00"],

Cell[TextData[{
  StyleBox["Gardner, M",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Fractal Music, Hypercards and More\[Ellipsis].",
    FontSlant->"Italic"],
  " New York: W. H. Freeman and Company, 1992."
}], "References",
  CellTags->"Gardner92"],

Cell[TextData[{
  StyleBox["Garey, M",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Johnson, D",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Computers and Intractability: A Guide to the Theory of \
NP-Completeness",
    FontSlant->"Italic"],
  ". San Francisco: W. H. Freeman, 1979."
}], "References",
  CellTags->"Garey79"],

Cell[TextData[{
  StyleBox["Gaylord, R",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Wellin, P",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Computer Simulations with Mathematica, Explorations in Complex \
Physical and Biological Systems",
    FontSlant->"Italic"],
  ". Santa Clara: TELOS/Springer-Verlag, 1995."
}], "References",
  CellTags->"Gaylord95"],

Cell[TextData[{
  StyleBox["Glynn, J",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Gray, T",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Beginner\[CloseCurlyQuote]s Guide to Mathematica",
    FontSlant->"Italic"],
  ",",
  StyleBox[" ",
    FontSlant->"Italic"],
  "Version 4. Cambridge University Press, 1999."
}], "References",
  CellTags->"Gaylord95"],

Cell[TextData[{
  StyleBox["Golin, M",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Sedgewick, R",
    FontFamily->"JansonText RomanSC"],
  ". Analysis of a simple yet efficient convex hull algorithm. In ",
  StyleBox["4th Annual Symposium on Computational Geometry",
    FontSlant->"Italic"],
  ". ACM, 1988."
}], "References",
  CellTags->"Golin88"],

Cell[TextData[{
  StyleBox["Golub, G.\[ThinSpace]H",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Van Loan, C.\[ThinSpace]F",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Matrix Computations",
    FontSlant->"Italic"],
  ". Baltimore: Johns Hopkins University Press, 3rd edition, 1996."
}], "References",
  CellTags->"Golub83"],

Cell[TextData[{
  StyleBox["Graham, R",
    FontFamily->"JansonText RomanSC"],
  ". An efficient algorithm for determining the convex hull of a finite \
planar set. ",
  StyleBox["Information Processing Letters",
    FontSlant->"Italic"],
  ", ",
  StyleBox["1",
    FontWeight->"Bold"],
  ", 1972."
}], "References",
  CellTags->"Graham72"],

Cell[TextData[{
  StyleBox["Graham, R",
    FontFamily->"JansonText RomanSC"],
  "., ",
  StyleBox["Knuth, D.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  "., and ",
  StyleBox["Patashnik, O",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Concrete Mathematics",
    FontSlant->"Italic"],
  ". Reading: Addison-Wesley Publishing Company, 2nd edition, 1994."
}], "References",
  CellTags->"Graham89"],

Cell[TextData[{
  StyleBox["Gray, A",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Modern Differential Geometry of Curves and Surfaces with \
Mathematica",
    FontSlant->"Italic"],
  ". Boca Raton: CRC Press, 2nd edition, 1997."
}], "References",
  CellTags->"Gray97"],

Cell[TextData[{
  StyleBox["Gray, T",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Glynn, J",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Exploring Mathematics with Mathematica",
    FontSlant->"Italic"],
  ". Redwood City: Addison-Wesley Publishing Company, 1992."
}], "References",
  CellTags->"Gray92"],

Cell[TextData[{
  StyleBox["Hayes, A",
    FontFamily->"JansonText RomanSC"],
  ". Sums of cubes of digits, driven to abstraction. ",
  StyleBox["Mathematica in Education",
    FontSlant->"Italic"],
  " ",
  StyleBox["1",
    FontWeight->"Bold"],
  "(4) 3\[Dash]11, 1992."
}], "References",
  CellTags->"Hayes92"],

Cell[TextData[{
  StyleBox["Hibbard, A.",
    FontFamily->"JansonText RomanSC"],
  " and ",
  StyleBox["Levasseur, K",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Exploring Abstract Algebra with Mathematica",
    FontSlant->"Italic"],
  ". New York: Springer-Verlag, 1999."
}], "References",
  CellTags->"Hibbard99"],

Cell[TextData[{
  StyleBox["Honsberger, R",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Mathematical Gems II",
    FontSlant->"Italic"],
  ". The Dolciani Mathematical Expositions, Number Two. Providence: The \
Mathematical Association of America, 1976."
}], "References",
  CellTags->"Honsberger76"],

Cell[TextData[{
  StyleBox["Jarvis, R.\[ThinSpace]A",
    FontFamily->"JansonText RomanSC"],
  ". On the identification of the convex hull of a finite set of points in \
the plane. ",
  StyleBox["Information Processing Letters",
    FontSlant->"Italic"],
  ",",
  " ",
  StyleBox["2",
    FontWeight->"Bold"],
  ", 1973."
}], "References",
  CellTags->"Jarvis73"],

Cell[TextData[{
  StyleBox["Knuth, D.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Art of Computer Programming, Volume 1, Fundamental \
Algorithms",
    FontSlant->"Italic"],
  ". Reading: Addison-Wesley Publishing Company, 3rd edition, 1997."
}], "References",
  CellTags->"KnuthVol1"],

Cell[TextData[{
  StyleBox["Knuth, D.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Art of Computer Programming, Volume 2, Seminumerical \
Algorithms",
    FontSlant->"Italic"],
  ". Reading: Addison-Wesley Publishing Company, 3rd edition, 1997."
}], "References",
  CellTags->"Knuth81"],

Cell[TextData[{
  StyleBox["Korhfage, R.\[ThinSpace]R",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Discrete Computational Structures",
    FontSlant->"Italic"],
  ". Orlando: Academic Press, Inc., 2nd edition, 1984."
}], "References",
  CellTags->"Korhfage84"],

Cell[TextData[{
  StyleBox["Lagarias, J",
    FontFamily->"JansonText RomanSC"],
  ". The ",
  Cell[BoxData[
      \(TraditionalForm\`\(\(\[VeryThinSpace]\)\(3\[VeryThinSpace]x + 
        1\)\)\)],
    FormatType->StandardForm],
  " problem. ",
  StyleBox["The American Mathematical Monthly",
    FontSlant->"Italic"],
  ",",
  " ",
  StyleBox["92",
    FontWeight->"Bold"],
  " 3\[Dash]23, 1985."
}], "References",
  CellTags->"Lagarias85"],

Cell[TextData[{
  StyleBox["Lawler, E",
    FontFamily->"JansonText RomanSC"],
  "., ",
  StyleBox["Lenstra, J.\[ThinSpace]K",
    FontFamily->"JansonText RomanSC"],
  "., ",
  StyleBox["Rinnooy Kan, A.\[ThinSpace]H.\[ThinSpace]G",
    FontFamily->"JansonText RomanSC"],
  "., and ",
  StyleBox["Shmoys, D.\[ThinSpace]B",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Traveling Salesman Problem: A Guided Tour of Combinatorial \
Optimization",
    FontSlant->"Italic"],
  ". New York: John Wiley & Sons, Inc., 1985."
}], "References",
  CellTags->"Lawler85"],

Cell[TextData[{
  StyleBox["Lin, S",
    FontFamily->"JansonText RomanSC"],
  ". Computer solutions of the traveling salesman problem. ",
  StyleBox["Bell System Technical Journal",
    FontSlant->"Italic"],
  ", ",
  StyleBox["44",
    FontWeight->"Bold"],
  " 2245\[Dash]2269, 1965."
}], "References",
  CellTags->"Lin65"],

Cell[TextData[{
  StyleBox["McMahon, T.\[ThinSpace]A",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Bonner, J.\[ThinSpace]T",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["On Size and Life",
    FontSlant->"Italic"],
  ". New York: Scientific American Books, Inc., 1983."
}], "References",
  CellTags->"McMahon83"],

Cell[TextData[{
  StyleBox["Maeder, R",
    FontFamily->"JansonText RomanSC"],
  ". The design of the ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " programming language. ",
  StyleBox["Dr. Dobbs Journal",
    FontSlant->"Italic"],
  ", ",
  StyleBox["17",
    FontWeight->"Bold"],
  "(4) 86, 1992."
}], "References",
  CellTags->"Maeder92"],

Cell[TextData[{
  StyleBox["Maeder, R",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Mathematica Programmer",
    FontSlant->"Italic"],
  ". Cambridge, MA: Academic Press, Inc., 1994."
}], "References",
  CellTags->"Maeder94"],

Cell[TextData[{
  StyleBox["Maeder, R",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Programming in Mathematica",
    FontSlant->"Italic"],
  ". Reading, MA: Addison-Wesley Publishing Company, 3rd edition, 1997."
}], "References",
  CellTags->"Maeder94"],

Cell[TextData[{
  StyleBox["Maeder, R",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Computer Science with Mathematica:",
    FontSlant->"Italic"],
  StyleBox[" Theory and Practice for Science, Mathematics, and Engineering",
    FontSlant->"Italic"],
  ". Cambridge University Press, 2000."
}], "References",
  CellTags->"Maeder94"],

Cell[TextData[{
  StyleBox["Mandelbrot, B",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Fractal Geometry of Nature",
    FontSlant->"Italic"],
  ". New York: W. H. Freeman and Company, 1988."
}], "References",
  CellTags->"Mandelbrot82"],

Cell[TextData[{
  StyleBox["The Mathematica Journal",
    FontSlant->"Italic"],
  ". Wolfram Media, Champaign."
}], "References",
  CellTags->"TMJ"],

Cell[TextData[{
  StyleBox["Mathews, M.\[ThinSpace]V",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Technology of Computer Music",
    FontSlant->"Italic"],
  ". Cambridge, MA: MIT Press, 1969."
}], "References",
  CellTags->"Mathews69"],

Cell[TextData[{
  StyleBox["Miyaji, C",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Abbott, P",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["MathLink: Network Programming with Mathematica",
    FontSlant->"Italic"],
  ". Cambridge University Press, 2001."
}], "References",
  CellTags->"Miyaji01"],

Cell[TextData[{
  StyleBox["Nijenhuis, A",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Wilf, H",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Combinatorial Algorithms",
    FontSlant->"Italic"],
  ". New York: Academic Press, Inc., 2nd edition, 1978."
}], "References",
  CellTags->"Nijenhuis78"],

Cell[TextData[{
  StyleBox["O\[CloseCurlyQuote]Rourke, J",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Computational Geometry in C",
    FontSlant->"Italic"],
  ". Cambridge University Press, 2nd edition, 1998."
}], "References",
  CellTags->"ORourke94"],

Cell[TextData[{
  StyleBox["Pemmaraju, S",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Skiena, S",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Computational Discrete Mathematics: Combinatorics and Graph \
Theory with Mathematica",
    FontSlant->"Italic"],
  ". Cambridge University Press, 2003."
}], "References",
  CellTags->"Skiena90"],

Cell[TextData[{
  StyleBox["Pierce, J.\[ThinSpace]R",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Science of Musical Sound",
    FontSlant->"Italic"],
  ". New York: W. H. Freeman & Company, 1983."
}], "References",
  CellTags->"Pierce83"],

Cell[TextData[{
  StyleBox["Platzman, L.\[ThinSpace]K",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Bartholdi, J.\[ThinSpace]J",
    FontFamily->"JansonText RomanSC"],
  ". Spacefilling curves and the planar traveling salesman problem. ",
  StyleBox["Journal Assoc. for Computing Machinery",
    FontSlant->"Italic"],
  ", ",
  StyleBox["36",
    FontWeight->"Bold"],
  " 719\[Dash]737, 1989."
}], "References",
  CellTags->"Platzman89"],

Cell[TextData[{
  StyleBox["Poundstone, W",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Recursive Universe: Cosmic Complexity and the Limits of \
Scientific Knowledge",
    FontSlant->"Italic"],
  ". Oxford: Oxford University Press, 1985."
}], "References",
  CellTags->"Poundstone85"],

Cell[TextData[{
  StyleBox["Preparata, F.\[ThinSpace]P",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Shamos, M.\[ThinSpace]I",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Computational Geometry: An Introduction",
    FontSlant->"Italic"],
  ". New York: Springer-Verlag, 1985."
}], "References",
  CellTags->"Preparata85"],

Cell[TextData[{
  StyleBox["Reingold, E.\[ThinSpace]M",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Tilford, J.\[ThinSpace]S",
    FontFamily->"JansonText RomanSC"],
  ". Tidier drawings of trees. ",
  StyleBox["IEEE Trans. Software Eng",
    FontSlant->"Italic"],
  ", ",
  StyleBox["SE7",
    FontWeight->"Bold"],
  " 223\[Dash]228, 1981."
}], "References",
  CellTags->"Reingold81"],

Cell[TextData[{
  StyleBox["Rosenkrantz, D.\[ThinSpace]J",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Stearns, R.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  ". An analysis of several heuristics for the traveling salesman problem. ",
  StyleBox["SIAM Journal of Computing",
    FontSlant->"Italic"],
  ", ",
  StyleBox["6",
    FontWeight->"Bold"],
  " 563\[Dash]581, 1977."
}], "References",
  CellTags->"Rosenkrantz77"],

Cell[TextData[{
  StyleBox["Rossing, T.\[ThinSpace]D",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Science of Sound",
    FontSlant->"Italic"],
  ". Reading: Addison-Wesley Publishing Company, 2nd edition, 1990."
}], "References",
  CellTags->"Rossing90"],

Cell[TextData[{
  StyleBox["Rust, B",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Burrus, W.\[ThinSpace]R",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Mathematical Programming and the Numerical Solution of Linear \
Equations",
    FontSlant->"Italic"],
  ". New York: American Elsevier Publishing Co., 1972."
}], "References",
  CellTags->"Rust72"],

Cell[TextData[{
  StyleBox["Sedgewick, R",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Algorithms",
    FontSlant->"Italic"],
  ". Reading: Addison-Wesley Publishing Company, 1988."
}], "References",
  CellTags->"Sedgewick88"],

Cell[TextData[{
  StyleBox["Shamos, M.\[ThinSpace]I",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Hoey, D",
    FontFamily->"JansonText RomanSC"],
  ". Closest-point problems. In ",
  StyleBox["16th Annual Symposium on Foundations of Computer Science",
    FontSlant->"Italic"],
  ". IEEE, 1975."
}], "References",
  CellTags->"Shamos75"],

Cell[TextData[{
  StyleBox["Shepard, R",
    FontFamily->"JansonText RomanSC"],
  ". The analysis of proximities: multidimensional scaling with an unknown \
distance factor. ",
  StyleBox["Psychometrics",
    FontSlant->"Italic"],
  ", ",
  StyleBox["27",
    FontWeight->"Bold"],
  " 125\[Dash]140, 1962."
}], "References",
  CellTags->"Shepard62"],

Cell[TextData[{
  StyleBox["Skeel, R.\[ThinSpace]D",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Keiper, J.\[ThinSpace]B",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Elementary Numerical Computing with Mathematica",
    FontSlant->"Italic"],
  ". New York: McGraw-Hill, Inc., 1993."
}], "References",
  CellTags->"Skeel93"],

Cell[TextData[{
  StyleBox["Thomsen, D.\[ThinSpace]E",
    FontFamily->"JansonText RomanSC"],
  ". Making music fractally. ",
  StyleBox["Science News",
    FontSlant->"Italic"],
  ", ",
  StyleBox["117",
    FontWeight->"Bold"],
  " 187, 1980."
}], "References",
  CellTags->"Thomsen80"],

Cell[TextData[{
  StyleBox["Voss, R.\[ThinSpace]F",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Clarke, J",
    FontFamily->"JansonText RomanSC"],
  ". ",
  Cell[BoxData[
      \(TraditionalForm\`1/f\)]],
  " noise in music and speech. ",
  StyleBox["Nature",
    FontSlant->"Italic"],
  ", ",
  StyleBox["258",
    FontWeight->"Bold"],
  " 317\[Dash]318, 1975."
}], "References",
  CellTags->"Voss75"],

Cell[TextData[{
  StyleBox["Voss, R.\[ThinSpace]F",
    FontFamily->"JansonText RomanSC"],
  ". and ",
  StyleBox["Clarke, J",
    FontFamily->"JansonText RomanSC"],
  ". ",
  Cell[BoxData[
      \(TraditionalForm\`1/f\)]],
  " noise in music and speech. ",
  StyleBox["The Journal of the Acoustical Society of America",
    FontSlant->"Italic"],
  ", ",
  StyleBox["63",
    FontWeight->"Bold"],
  " 258\[Dash]263, 1978."
}], "References",
  CellTags->"Voss78"],

Cell[TextData[{
  StyleBox["Wagon, S",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["Mathematica in Action",
    FontSlant->"Italic"],
  ". Santa Clara: TELOS/Springer-Verlag, 2nd edition, 1999."
}], "References",
  CellTags->"Wagon96"],

Cell[TextData[{
  StyleBox["Weiss, G.\[ThinSpace]H",
    FontFamily->"JansonText RomanSC"],
  ". Random walks and their applications. ",
  StyleBox["American Scientist",
    FontSlant->"Italic"],
  ", ",
  StyleBox["71",
    FontWeight->"Bold"],
  " 65\[Dash]71, 1983."
}], "References",
  CellTags->"Weiss83"],

Cell[TextData[{
  StyleBox["Wickham-Jones,",
    FontFamily->"JansonText RomanSC"],
  " T. ",
  StyleBox["Computer Graphics with Mathematica",
    FontSlant->"Italic"],
  ". Santa Clara: TELOS/Springer-Verlag, 1994."
}], "References",
  CellTags->"Wickham-Jones94"],

Cell[TextData[{
  StyleBox["Wolfram, S",
    FontFamily->"JansonText RomanSC"],
  ". ",
  StyleBox["The Mathematica Book",
    FontSlant->"Italic"],
  ". Wolfram Media, Inc., 5th edition, 2003."
}], "References",
  CellTags->"Wolfram98"]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 685}},
ScreenStyleEnvironment->"Working",
PrintingStyleEnvironment->"Production",
ShowPageBreaks->False,
WindowSize->{595, 484},
WindowMargins->{{69, Automatic}, {Automatic, 5}},
PrintingCopies->1,
PrintingStartingPageNumber->429,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, Inherited, Cell[ 
        TextData[ {"An Introduction to Programming with ", 
          StyleBox[ "Mathematica", "IT"]}], "Header"]}, {Cell[ 
        TextData[ {"References"}], "Header"], Inherited, Inherited}},
PrintingOptions->{"PrintingMargins"->{{54, 54}, {72, 72}},
"PrintCellBrackets"->False,
"PrintRegistrationMarks"->False,
"PrintMultipleHorizontalPages"->False,
"FirstPageHeader"->False,
"FacingPages"->True},
TaggingRules:>{"PaginationCache" -> {{2004, 9, 2, 17, 39, \
25.1947296`9.153884622664965}, {429, 429, 429, 429, 429, 429, 429, 429, 429, \
429, 429, 429, 429, 429, 429, 429, 429, 429, 430, 430, 430, 430, 430, 430, \
430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 430, 431, \
431, 431, 431, 431, 431, 431, 431, 431, 431, 431, 431, 431, 431, 431, 431, \
431, 431, 431, 431, 431, 431, 432, 432, 432}, Automatic}},
Magnification->1,
StyleDefinitions -> "IPM3Styles.nb"
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "c:1"->{
    Cell[1900, 59, 48, 1, 96, "Chapter",
      CellTags->"c:1"]},
  "Bentley79"->{
    Cell[1951, 62, 300, 11, 25, "References",
      CellTags->"Bentley79"],
    Cell[2254, 75, 406, 12, 44, "References",
      CellTags->"Bentley79"]},
  "Burden89"->{
    Cell[2663, 89, 362, 11, 44, "References",
      CellTags->"Burden89"]},
  "Casti92"->{
    Cell[3028, 102, 292, 9, 44, "References",
      CellTags->"Casti92"]},
  "Casti292"->{
    Cell[3323, 113, 290, 9, 44, "References",
      CellTags->"Casti292"]},
  "Crandall93"->{
    Cell[3616, 124, 180, 5, 25, "References",
      CellTags->"Crandall93"]},
  "Crandall94"->{
    Cell[3799, 131, 269, 8, 25, "References",
      CellTags->"Crandall94"]},
  "Crandall96"->{
    Cell[4071, 141, 276, 8, 25, "References",
      CellTags->"Crandall96"],
    Cell[4350, 151, 465, 17, 44, "References",
      CellTags->"Crandall96"]},
  "deBerg00"->{
    Cell[4818, 170, 567, 20, 44, "References",
      CellTags->"deBerg00"]},
  "Gardner92"->{
    Cell[5388, 192, 265, 8, 25, "References",
      CellTags->"Gardner92"]},
  "Garey79"->{
    Cell[5656, 202, 357, 12, 44, "References",
      CellTags->"Garey79"]},
  "Gaylord95"->{
    Cell[6016, 216, 390, 12, 44, "References",
      CellTags->"Gaylord95"],
    Cell[6409, 230, 392, 14, 25, "References",
      CellTags->"Gaylord95"]},
  "Golin88"->{
    Cell[6804, 246, 369, 11, 44, "References",
      CellTags->"Golin88"]},
  "Golub83"->{
    Cell[7176, 259, 359, 11, 44, "References",
      CellTags->"Golub83"]},
  "Graham72"->{
    Cell[7538, 272, 341, 12, 44, "References",
      CellTags->"Graham72"]},
  "Graham89"->{
    Cell[7882, 286, 422, 14, 44, "References",
      CellTags->"Graham89"]},
  "Gray97"->{
    Cell[8307, 302, 283, 9, 44, "References",
      CellTags->"Gray97"]},
  "Gray92"->{
    Cell[8593, 313, 338, 11, 44, "References",
      CellTags->"Gray92"]},
  "Hayes92"->{
    Cell[8934, 326, 313, 11, 25, "References",
      CellTags->"Hayes92"]},
  "Hibbard99"->{
    Cell[9250, 339, 331, 11, 44, "References",
      CellTags->"Hibbard99"]},
  "Honsberger76"->{
    Cell[9584, 352, 315, 9, 44, "References",
      CellTags->"Honsberger76"]},
  "Jarvis73"->{
    Cell[9902, 363, 363, 13, 44, "References",
      CellTags->"Jarvis73"]},
  "KnuthVol1"->{
    Cell[10268, 378, 319, 9, 44, "References",
      CellTags->"KnuthVol1"]},
  "Knuth81"->{
    Cell[10590, 389, 319, 9, 44, "References",
      CellTags->"Knuth81"]},
  "Korhfage84"->{
    Cell[10912, 400, 276, 8, 25, "References",
      CellTags->"Korhfage84"]},
  "Lagarias85"->{
    Cell[11191, 410, 441, 17, 25, "References",
      CellTags->"Lagarias85"]},
  "Lawler85"->{
    Cell[11635, 429, 575, 18, 44, "References",
      CellTags->"Lawler85"]},
  "Lin65"->{
    Cell[12213, 449, 324, 11, 44, "References",
      CellTags->"Lin65"]},
  "McMahon83"->{
    Cell[12540, 462, 345, 11, 25, "References",
      CellTags->"McMahon83"]},
  "Maeder92"->{
    Cell[12888, 475, 353, 14, 25, "References",
      CellTags->"Maeder92"]},
  "Maeder94"->{
    Cell[13244, 491, 244, 8, 25, "References",
      CellTags->"Maeder94"],
    Cell[13491, 501, 268, 8, 44, "References",
      CellTags->"Maeder94"],
    Cell[13762, 511, 346, 10, 44, "References",
      CellTags->"Maeder94"]},
  "Mandelbrot82"->{
    Cell[14111, 523, 256, 8, 25, "References",
      CellTags->"Mandelbrot82"]},
  "TMJ"->{
    Cell[14370, 533, 148, 5, 25, "References",
      CellTags->"TMJ"]},
  "Mathews69"->{
    Cell[14521, 540, 255, 8, 25, "References",
      CellTags->"Mathews69"]},
  "Miyaji01"->{
    Cell[14779, 550, 330, 11, 44, "References",
      CellTags->"Miyaji01"]},
  "Nijenhuis78"->{
    Cell[15112, 563, 329, 11, 25, "References",
      CellTags->"Nijenhuis78"]},
  "ORourke94"->{
    Cell[15444, 576, 269, 8, 25, "References",
      CellTags->"ORourke94"]},
  "Skiena90"->{
    Cell[15716, 586, 372, 12, 44, "References",
      CellTags->"Skiena90"]},
  "Pierce83"->{
    Cell[16091, 600, 258, 8, 25, "References",
      CellTags->"Pierce83"]},
  "Platzman89"->{
    Cell[16352, 610, 456, 14, 44, "References",
      CellTags->"Platzman89"]},
  "Poundstone85"->{
    Cell[16811, 626, 304, 9, 44, "References",
      CellTags->"Poundstone85"]},
  "Preparata85"->{
    Cell[17118, 637, 356, 11, 44, "References",
      CellTags->"Preparata85"]},
  "Reingold81"->{
    Cell[17477, 650, 404, 14, 44, "References",
      CellTags->"Reingold81"]},
  "Rosenkrantz77"->{
    Cell[17884, 666, 453, 14, 44, "References",
      CellTags->"Rosenkrantz77"]},
  "Rossing90"->{
    Cell[18340, 682, 274, 8, 25, "References",
      CellTags->"Rossing90"]},
  "Rust72"->{
    Cell[18617, 692, 383, 12, 44, "References",
      CellTags->"Rust72"]},
  "Sedgewick88"->{
    Cell[19003, 706, 241, 8, 25, "References",
      CellTags->"Sedgewick88"]},
  "Shamos75"->{
    Cell[19247, 716, 357, 11, 44, "References",
      CellTags->"Shamos75"]},
  "Shepard62"->{
    Cell[19607, 729, 349, 12, 44, "References",
      CellTags->"Shepard62"]},
  "Skeel93"->{
    Cell[19959, 743, 358, 11, 44, "References",
      CellTags->"Skeel93"]},
  "Thomsen80"->{
    Cell[20320, 756, 288, 11, 25, "References",
      CellTags->"Thomsen80"]},
  "Voss75"->{
    Cell[20611, 769, 421, 17, 25, "References",
      CellTags->"Voss75"]},
  "Voss78"->{
    Cell[21035, 788, 462, 17, 44, "References",
      CellTags->"Voss78"]},
  "Wagon96"->{
    Cell[21500, 807, 249, 8, 25, "References",
      CellTags->"Wagon96"]},
  "Weiss83"->{
    Cell[21752, 817, 310, 11, 25, "References",
      CellTags->"Weiss83"]},
  "Wickham-Jones94"->{
    Cell[22065, 830, 265, 8, 25, "References",
      CellTags->"Wickham-Jones94"]},
  "Wolfram98"->{
    Cell[22333, 840, 237, 8, 25, "References",
      CellTags->"Wolfram98"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"c:1", 24296, 890},
  {"Bentley79", 24379, 893},
  {"Burden89", 24548, 898},
  {"Casti92", 24639, 901},
  {"Casti292", 24730, 904},
  {"Crandall93", 24824, 907},
  {"Crandall94", 24920, 910},
  {"Crandall96", 25016, 913},
  {"deBerg00", 25188, 918},
  {"Gardner92", 25282, 921},
  {"Garey79", 25374, 924},
  {"Gaylord95", 25467, 927},
  {"Golin88", 25637, 932},
  {"Golub83", 25728, 935},
  {"Graham72", 25820, 938},
  {"Graham89", 25913, 941},
  {"Gray97", 26004, 944},
  {"Gray92", 26092, 947},
  {"Hayes92", 26182, 950},
  {"Hibbard99", 26275, 953},
  {"Honsberger76", 26373, 956},
  {"Jarvis73", 26469, 959},
  {"KnuthVol1", 26563, 962},
  {"Knuth81", 26656, 965},
  {"Korhfage84", 26750, 968},
  {"Lagarias85", 26847, 971},
  {"Lawler85", 26943, 974},
  {"Lin65", 27034, 977},
  {"McMahon83", 27126, 980},
  {"Maeder92", 27221, 983},
  {"Maeder94", 27315, 986},
  {"Mandelbrot82", 27565, 993},
  {"TMJ", 27657, 996},
  {"Mathews69", 27746, 999},
  {"Miyaji01", 27840, 1002},
  {"Nijenhuis78", 27937, 1005},
  {"ORourke94", 28035, 1008},
  {"Skiena90", 28129, 1011},
  {"Pierce83", 28223, 1014},
  {"Platzman89", 28318, 1017},
  {"Poundstone85", 28418, 1020},
  {"Preparata85", 28518, 1023},
  {"Reingold81", 28617, 1026},
  {"Rosenkrantz77", 28718, 1029},
  {"Rossing90", 28818, 1032},
  {"Rust72", 28910, 1035},
  {"Sedgewick88", 29005, 1038},
  {"Shamos75", 29101, 1041},
  {"Shepard62", 29196, 1044},
  {"Skeel93", 29290, 1047},
  {"Thomsen80", 29384, 1050},
  {"Voss75", 29477, 1053},
  {"Voss78", 29567, 1056},
  {"Wagon96", 29658, 1059},
  {"Weiss83", 29749, 1062},
  {"Wickham-Jones94", 29849, 1065},
  {"Wolfram98", 29950, 1068}
  }
*)

(*NotebookFileOutline
Notebook[{
Cell[1754, 51, 121, 4, 29, "TitleBar"],

Cell[CellGroupData[{
Cell[1900, 59, 48, 1, 96, "Chapter",
  CellTags->"c:1"],
Cell[1951, 62, 300, 11, 25, "References",
  CellTags->"Bentley79"],
Cell[2254, 75, 406, 12, 44, "References",
  CellTags->"Bentley79"],
Cell[2663, 89, 362, 11, 44, "References",
  CellTags->"Burden89"],
Cell[3028, 102, 292, 9, 44, "References",
  CellTags->"Casti92"],
Cell[3323, 113, 290, 9, 44, "References",
  CellTags->"Casti292"],
Cell[3616, 124, 180, 5, 25, "References",
  CellTags->"Crandall93"],
Cell[3799, 131, 269, 8, 25, "References",
  CellTags->"Crandall94"],
Cell[4071, 141, 276, 8, 25, "References",
  CellTags->"Crandall96"],
Cell[4350, 151, 465, 17, 44, "References",
  CellTags->"Crandall96"],
Cell[4818, 170, 567, 20, 44, "References",
  CellTags->"deBerg00"],
Cell[5388, 192, 265, 8, 25, "References",
  CellTags->"Gardner92"],
Cell[5656, 202, 357, 12, 44, "References",
  CellTags->"Garey79"],
Cell[6016, 216, 390, 12, 44, "References",
  CellTags->"Gaylord95"],
Cell[6409, 230, 392, 14, 25, "References",
  CellTags->"Gaylord95"],
Cell[6804, 246, 369, 11, 44, "References",
  CellTags->"Golin88"],
Cell[7176, 259, 359, 11, 44, "References",
  CellTags->"Golub83"],
Cell[7538, 272, 341, 12, 44, "References",
  CellTags->"Graham72"],
Cell[7882, 286, 422, 14, 44, "References",
  CellTags->"Graham89"],
Cell[8307, 302, 283, 9, 44, "References",
  CellTags->"Gray97"],
Cell[8593, 313, 338, 11, 44, "References",
  CellTags->"Gray92"],
Cell[8934, 326, 313, 11, 25, "References",
  CellTags->"Hayes92"],
Cell[9250, 339, 331, 11, 44, "References",
  CellTags->"Hibbard99"],
Cell[9584, 352, 315, 9, 44, "References",
  CellTags->"Honsberger76"],
Cell[9902, 363, 363, 13, 44, "References",
  CellTags->"Jarvis73"],
Cell[10268, 378, 319, 9, 44, "References",
  CellTags->"KnuthVol1"],
Cell[10590, 389, 319, 9, 44, "References",
  CellTags->"Knuth81"],
Cell[10912, 400, 276, 8, 25, "References",
  CellTags->"Korhfage84"],
Cell[11191, 410, 441, 17, 25, "References",
  CellTags->"Lagarias85"],
Cell[11635, 429, 575, 18, 44, "References",
  CellTags->"Lawler85"],
Cell[12213, 449, 324, 11, 44, "References",
  CellTags->"Lin65"],
Cell[12540, 462, 345, 11, 25, "References",
  CellTags->"McMahon83"],
Cell[12888, 475, 353, 14, 25, "References",
  CellTags->"Maeder92"],
Cell[13244, 491, 244, 8, 25, "References",
  CellTags->"Maeder94"],
Cell[13491, 501, 268, 8, 44, "References",
  CellTags->"Maeder94"],
Cell[13762, 511, 346, 10, 44, "References",
  CellTags->"Maeder94"],
Cell[14111, 523, 256, 8, 25, "References",
  CellTags->"Mandelbrot82"],
Cell[14370, 533, 148, 5, 25, "References",
  CellTags->"TMJ"],
Cell[14521, 540, 255, 8, 25, "References",
  CellTags->"Mathews69"],
Cell[14779, 550, 330, 11, 44, "References",
  CellTags->"Miyaji01"],
Cell[15112, 563, 329, 11, 25, "References",
  CellTags->"Nijenhuis78"],
Cell[15444, 576, 269, 8, 25, "References",
  CellTags->"ORourke94"],
Cell[15716, 586, 372, 12, 44, "References",
  CellTags->"Skiena90"],
Cell[16091, 600, 258, 8, 25, "References",
  CellTags->"Pierce83"],
Cell[16352, 610, 456, 14, 44, "References",
  CellTags->"Platzman89"],
Cell[16811, 626, 304, 9, 44, "References",
  CellTags->"Poundstone85"],
Cell[17118, 637, 356, 11, 44, "References",
  CellTags->"Preparata85"],
Cell[17477, 650, 404, 14, 44, "References",
  CellTags->"Reingold81"],
Cell[17884, 666, 453, 14, 44, "References",
  CellTags->"Rosenkrantz77"],
Cell[18340, 682, 274, 8, 25, "References",
  CellTags->"Rossing90"],
Cell[18617, 692, 383, 12, 44, "References",
  CellTags->"Rust72"],
Cell[19003, 706, 241, 8, 25, "References",
  CellTags->"Sedgewick88"],
Cell[19247, 716, 357, 11, 44, "References",
  CellTags->"Shamos75"],
Cell[19607, 729, 349, 12, 44, "References",
  CellTags->"Shepard62"],
Cell[19959, 743, 358, 11, 44, "References",
  CellTags->"Skeel93"],
Cell[20320, 756, 288, 11, 25, "References",
  CellTags->"Thomsen80"],
Cell[20611, 769, 421, 17, 25, "References",
  CellTags->"Voss75"],
Cell[21035, 788, 462, 17, 44, "References",
  CellTags->"Voss78"],
Cell[21500, 807, 249, 8, 25, "References",
  CellTags->"Wagon96"],
Cell[21752, 817, 310, 11, 25, "References",
  CellTags->"Weiss83"],
Cell[22065, 830, 265, 8, 25, "References",
  CellTags->"Wickham-Jones94"],
Cell[22333, 840, 237, 8, 25, "References",
  CellTags->"Wolfram98"]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

