(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.0'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     46213,       1744]*)
(*NotebookOutlinePosition[     46911,       1768]*)
(*  CellTagsIndexPosition[     46867,       1764]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell[TextData[{
  "Style Definitions\n\n",
  StyleBox["An Introduction to Programming with Mathematica, ",
    FontSlant->"Italic"],
  "Third edition"
}], "Subtitle",
  CellMargins->{{0, 0}, {20, 0}},
  Background->RGBColor[1, 1, 0]],

Cell[CellGroupData[{

Cell["Style Environment Names", "Section"],

Cell[StyleData[All, "Working"],
  PageWidth->WindowWidth,
  CellLabelMargins->{{12, Inherited}, {Inherited, Inherited}},
  ScriptMinSize->9],

Cell[StyleData[All, "Presentation"],
  PageWidth->WindowWidth,
  CellLabelMargins->{{24, Inherited}, {Inherited, Inherited}},
  ScriptMinSize->12],

Cell[StyleData[All, "Printout"],
  PageWidth->PaperWidth,
  CellLabelMargins->{{2, Inherited}, {Inherited, Inherited}},
  ScriptMinSize->5,
  PrivateFontOptions->{"FontType"->"Outline"}]
}, Closed]],

Cell[CellGroupData[{

Cell["Notebook Options", "Section"],

Cell["\<\
The options defined for the style below will be used at the Notebook level.\
\>", "Text"],

Cell[StyleData["Notebook"],
  PageHeaders->{{Cell[ 
          TextData[ {
            CounterBox[ "Page"]}], "PageNumber"], None, Cell[ 
          TextData[ {
            ValueBox[ "FileName"]}], "Header"]}, {Cell[ 
          TextData[ {
            ValueBox[ "FileName"]}], "Header"], None, Cell[ 
          TextData[ {
            CounterBox[ "Page"]}], "PageNumber"]}},
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Headings", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Chapter"],
  ShowCellBracket->False,
  CellMargins->{{0, 0}, {24, 0}},
  CellGroupingRules->{"TitleGrouping", 0},
  ConversionRules:>{"TeX" -> {"\\chapter{", "}\n\n"}},
  CellFrameMargins->20,
  LineSpacing->{1, 6},
  CounterIncrements->"Chapter",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subtitle", 0}, {"Subsubtitle", 0}, {"Head1", 0}},
  CounterStyleMenuListing->None,
  FontFamily->"Times",
  FontSize->24,
  FontWeight->"Bold",
  Background->RGBColor[0.890196, 0.878431, 0.717647]],

Cell[StyleData["Chapter", "Presentation"],
  FontSize->30,
  FontWeight->"Bold"],

Cell[StyleData["Chapter", "Printout"],
  FontSize->22,
  Background->None]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subtitle"],
  CellMargins->{{12, Inherited}, {20, 15}},
  CellGroupingRules->{"TitleGrouping", 10},
  PageBreakBelow->False,
  DefaultNewInlineCellStyle->"None",
  InputAutoReplacements->{"TeX"->StyleBox[ 
          RowBox[ {"T", 
            AdjustmentBox[ 
            "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
              0.5], "X"}]], "LaTeX"->StyleBox[ 
          RowBox[ {"L", 
            StyleBox[ 
              AdjustmentBox[ 
              "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
            AdjustmentBox[ 
            "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
              0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
      "MMA"->"Mathematica"},
  LanguageCategory->"NaturalLanguage",
  CounterIncrements->"Subtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subsubtitle", 0}},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->24],

Cell[StyleData["Subtitle", "Presentation"],
  CellMargins->{{24, 10}, {20, 20}},
  LineSpacing->{1, 0},
  FontSize->36],

Cell[StyleData["Subtitle", "Printout"],
  CellMargins->{{2, 10}, {12, 8}},
  FontSize->18]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsubtitle"],
  CellMargins->{{12, Inherited}, {20, 15}},
  CellGroupingRules->{"TitleGrouping", 20},
  PageBreakBelow->False,
  DefaultNewInlineCellStyle->"None",
  InputAutoReplacements->{"TeX"->StyleBox[ 
          RowBox[ {"T", 
            AdjustmentBox[ 
            "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
              0.5], "X"}]], "LaTeX"->StyleBox[ 
          RowBox[ {"L", 
            StyleBox[ 
              AdjustmentBox[ 
              "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
            AdjustmentBox[ 
            "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
              0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
      "MMA"->"Mathematica"},
  LanguageCategory->"NaturalLanguage",
  CounterIncrements->"Subsubtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->14,
  FontSlant->"Italic"],

Cell[StyleData["Subsubtitle", "Presentation"],
  CellMargins->{{24, 10}, {20, 20}},
  LineSpacing->{1, 0},
  FontSize->24],

Cell[StyleData["Subsubtitle", "Printout"],
  CellMargins->{{2, 10}, {12, 8}},
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Section"],
  CellDingbat->None,
  CellMargins->{{12, Inherited}, {0, 15}},
  CellGroupingRules->{"SectionGrouping", 30},
  ConversionRules:>{"TeX" -> {"\\section{", "}\n\n"}},
  PageBreakBelow->False,
  CellFrameLabels->{{Cell[ 
          TextData[ {
            CounterBox[ "Chapter"], ".", 
            CounterBox[ "Section"]}], "Section", CellBaseline -> Baseline], 
        None}, {None, None}},
  CounterIncrements->"Section",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}, {
      "Exercise", 0}},
  CounterStyleMenuListing->None,
  FontFamily->"Times",
  FontSize->20,
  FontWeight->"Bold"],

Cell[StyleData["Section", "Presentation"],
  CellMargins->{{24, Inherited}, {Inherited, Inherited}},
  LineSpacing->{1, 0},
  FontSize->24],

Cell[StyleData["Section", "Printout"],
  CellMargins->{{Inherited, 0}, {7, 18}},
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsection"],
  CellMargins->{{12, Inherited}, {5, 18}},
  CellGroupingRules->{"SectionGrouping", 40},
  ConversionRules:>{"TeX" -> {"\\subsection{", "}\n\n"}},
  PageBreakBelow->False,
  CounterAssignments->{{"Subsubsection", 0}},
  CounterStyleMenuListing->None,
  FontSize->16,
  FontWeight->"Bold",
  FontSlant->"Italic"],

Cell[StyleData["Subsection", "Presentation"],
  CellMargins->{{24, 10}, {11, 32}},
  LineSpacing->{1, 0},
  FontSize->22],

Cell[StyleData["Subsection", "Printout"],
  CellMargins->{{40, 0}, {7, 22}},
  FontSize->12]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Subsubsection"],
  CellDingbat->"\[FilledSmallSquare]",
  CellMargins->{{22, Inherited}, {8, 18}},
  CellGroupingRules->{"SectionGrouping", 50},
  PageBreakBelow->False,
  DefaultNewInlineCellStyle->"None",
  InputAutoReplacements->{"TeX"->StyleBox[ 
          RowBox[ {"T", 
            AdjustmentBox[ 
            "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
              0.5], "X"}]], "LaTeX"->StyleBox[ 
          RowBox[ {"L", 
            StyleBox[ 
              AdjustmentBox[ 
              "A", BoxMargins -> {{-0.36, -0.1}, {0, -0}}, 
                BoxBaselineShift -> -0.2], FontSize -> Smaller], "T", 
            AdjustmentBox[ 
            "E", BoxMargins -> {{-0.075, -0.085}, {0, 0}}, BoxBaselineShift -> 
              0.5], "X"}]], "mma"->"Mathematica", "Mma"->"Mathematica", 
      "MMA"->"Mathematica"},
  LanguageCategory->"NaturalLanguage",
  CounterIncrements->"Subsubsection",
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Bold"],

Cell[StyleData["Subsubsection", "Presentation"],
  CellMargins->{{34, 10}, {11, 26}},
  LineSpacing->{1, 0},
  FontSize->18],

Cell[StyleData["Subsubsection", "Printout"],
  CellMargins->{{9, 0}, {7, 14}},
  FontSize->11]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["TitleBar"],
  ShowCellBracket->False,
  CellMargins->{{0, 0}, {0, 0}},
  CellFrameMargins->{{18, 0}, {6, 6}},
  LanguageCategory->"NaturalLanguage",
  StyleMenuListing->None,
  FontFamily->"Times",
  FontSize->12,
  FontWeight->"Bold",
  FontSlant->"Italic",
  FontColor->GrayLevel[0.900008],
  Background->RGBColor[0.501961, 0.501961, 0.752941]],

Cell[StyleData["TitleBar", "Presentation"],
  CellMargins->{{0, 0}, {0, 0}},
  LineSpacing->{1, 0},
  FontSize->14],

Cell[StyleData["TitleBar", "Printout"],
  CellMargins->{{Inherited, Inherited}, {0, 0}},
  CellElementSpacings->{"CellMinHeight"->1,
  "ClosedCellHeight"->0},
  CellOpen->False]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Body Text", "Section"],

Cell[CellGroupData[{

Cell[StyleData["Text"],
  CellMargins->{{12, 10}, {7, 7}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphIndent->24,
  CounterIncrements->"Text",
  FontFamily->"Times"],

Cell[StyleData["Text", "Presentation"],
  CellMargins->{{24, 10}, {10, 10}},
  LineSpacing->{1, 5},
  FontSize->16],

Cell[StyleData["Text", "Printout"],
  CellMargins->{{40, 2}, {6, 6}},
  TextJustification->0.5,
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["TextFirst"],
  CellMargins->{{12, 10}, {7, 7}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  CounterIncrements->"Text",
  FontFamily->"Times"],

Cell[StyleData["TextFirst", "Presentation"],
  CellMargins->{{24, 10}, {10, 10}},
  LineSpacing->{1, 5},
  FontSize->16],

Cell[StyleData["TextFirst", "Printout"],
  CellMargins->{{40, 2}, {6, 6}},
  TextJustification->0.5,
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["BulletedList"],
  CellDingbat->"\[Bullet]",
  CellMargins->{{48, 4}, {6, 6}},
  TextJustification->1,
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  ParagraphIndent->0,
  CounterIncrements->"Text",
  FontFamily->"Times"],

Cell[StyleData["BulletedList", "Presentation"],
  FontSize->16],

Cell[StyleData["BulletedList", "Printout"],
  CellMargins->{{50, 4}, {Inherited, Inherited}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ItemizedText"],
  CellMargins->{{26, 4}, {0, 8}},
  CellGroupingRules->{"SectionGrouping", 45},
  CellFrameLabels->{{Cell[ 
          TextData[ {
            CounterBox[ "ItemNumber"], "."}], "ExerciseItemNumber", 
          CellBaseline -> Baseline], None}, {None, None}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterIncrements->"ItemNumber"],

Cell[StyleData["ItemizedText", "Presentation"],
  FontSize->16],

Cell[StyleData["ItemizedText", "Printout"],
  CellMargins->{{20, 4}, {0, 8}},
  FontSize->11]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["BulletedSubItem"],
  CellDingbat->"\[Bullet]",
  CellMargins->{{48, 4}, {6, 6}},
  TextJustification->1,
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  ParagraphIndent->0,
  CounterIncrements->"Text",
  FontFamily->"Times"],

Cell[StyleData["BulletedSubItem", "Presentation"],
  FontSize->16],

Cell[StyleData["BulletedSubItem", "Printout"],
  CellMargins->{{50, 4}, {Inherited, Inherited}},
  FontSize->10]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Unique Styles", "Section"],

Cell[CellGroupData[{

Cell[StyleData["ChapterHeading"],
  CellFrame->{{0, 0}, {1, 0}},
  CellGroupingRules->{"TitleGrouping", 0},
  ConversionRules:>{"TeX" -> {{"", CompoundExpression[ #, ""]&, ""}, {"", 
    CompoundExpression[ #, ""]&, ""}}},
  PageBreakAbove->True,
  CellFrameMargins->False,
  CounterIncrements->"ChapterHeading",
  StyleMenuListing->None,
  CounterStyleMenuListing->None,
  FontSize->30,
  FontWeight->"Bold",
  GridBoxOptions->{ColumnSpacings->0.2,
  ColumnAlignments->{Left}}],

Cell[StyleData["ChapterHeading", "Printout"],
  CellMargins->{{2, Inherited}, {Inherited, 0}}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ChapterHead"],
  CounterIncrements->"ChapterHeading",
  StyleMenuListing->None,
  CounterStyleMenuListing->None,
  FontFamily->"Times",
  FontSize->24,
  FontWeight->"Bold"],

Cell[StyleData["ChapterHead", "Printout"]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["PrefaceSection"],
  CellMargins->{{12, Inherited}, {5, 18}},
  CellGroupingRules->{"SectionGrouping", 30},
  StyleMenuListing->None,
  FontSize->20,
  FontWeight->"Bold"],

Cell[StyleData["PrefaceSection", "Presentation"],
  LineSpacing->{1, 0},
  FontSize->24],

Cell[StyleData["PrefaceSection", "Printout"],
  CellMargins->{{12, 0}, {7, 22}},
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ChapterAbstract"],
  CellMargins->{{40, 100}, {20, 0}},
  TextJustification->1,
  Hyphenation->True,
  LineSpacing->{1, 2},
  StyleMenuListing->None,
  FontFamily->"Times",
  FontSize->11],

Cell[StyleData["ChapterAbstract", "Presentation"],
  FontSize->14],

Cell[StyleData["ChapterAbstract", "Printout"],
  TextJustification->1,
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Quote"],
  CellMargins->{{36, 36}, {9, 18}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterIncrements->"Text",
  StyleMenuListing->None,
  FontSlant->"Italic"],

Cell[StyleData["Quote", "Presentation"],
  FontSize->14],

Cell[StyleData["Quote", "Printout"],
  CellMargins->{{20, 4}, {0, 8}},
  ParagraphIndent->-14,
  FontSize->11]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["References"],
  CellMargins->{{20, 10}, {0, 6}},
  DefaultNewInlineCellStyle->"None",
  Hyphenation->True,
  LineSpacing->{1, 3},
  StyleMenuListing->None,
  FontFamily->"Times",
  FontSize->12],

Cell[StyleData["References", "Presentation"],
  CellMargins->{{24, 10}, {8, 8}},
  LineSpacing->{1, 5},
  FontSize->14],

Cell[StyleData["References", "Printout"],
  CellMargins->{{2, 2}, {5, 5}},
  TextJustification->0.5,
  FontSize->7]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["DisplayTable"],
  CellMargins->{{30, Inherited}, {0, 18}},
  CellHorizontalScrolling->True,
  DefaultFormatType->DefaultTraditionalFormatType,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  LanguageCategory->"Formula",
  ScriptLevel->0,
  SingleLetterItalics->True,
  UnderoverscriptBoxOptions->{LimitsPositioning->True}],

Cell[StyleData["DisplayTable", "Presentation"],
  LineSpacing->{1, 5},
  FontSize->16],

Cell[StyleData["DisplayTable", "Printout"],
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Picture"],
  CellMargins->{{24, 10}, {6, 6}},
  CellGroupingRules->"GraphicsGrouping",
  CellHorizontalScrolling->True],

Cell[StyleData["Picture", "Printout"],
  CellMargins->{{70, 2}, {6, 6}},
  Magnification->0.5]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["NonPrint Styles", "Section"],

Cell[CellGroupData[{

Cell[StyleData["NonPrintSection"],
  CellMargins->{{25, 4}, {4, 8}},
  CellGroupingRules->{"SectionGrouping", 30},
  PageBreakBelow->False,
  DefaultNewInlineCellStyle->"None",
  LineSpacing->{1, 7},
  LanguageCategory->"NaturalLanguage",
  CounterIncrements->"Section",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}},
  FontSize->20,
  FontWeight->"Bold"],

Cell[StyleData["NonPrintSection", "Presentation"],
  FontSize->20]
}, Open  ]],

Cell[CellGroupData[{

Cell[StyleData["NonPrintInput"],
  CellMargins->{{60, 10}, {5, 7}},
  Evaluatable->True,
  CellGroupingRules->"InputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  DefaultFormatType->DefaultInputFormatType,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  AutoItalicWords->{},
  LanguageCategory->"Formula",
  FormatType->InputForm,
  ShowStringCharacters->True,
  NumberMarks->True,
  LinebreakAdjustments->{0.85, 2, 10, 0, 1},
  CounterIncrements->"Input",
  FontSize->14,
  FontWeight->"Bold",
  FontColor->RGBColor[1, 0, 0]],

Cell[StyleData["NonPrintInput", "Presentation"],
  CellMargins->{{72, Inherited}, {8, 10}},
  LineSpacing->{1, 0},
  FontSize->16]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Styles for Automatic Numbering", "Section"],

Cell["\<\
The following styles are useful for numbered equations, figures, etc.  They \
automatically give the cell a FrameLabel containing a reference to a \
particular counter, and also increment that counter.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["NumberedEquation"],
  CellMargins->{{55, 10}, {0, 10}},
  CellFrameLabels->{{None, Cell[ 
          TextData[ {"(", 
            CounterBox[ "NumberedEquation"], ")"}]]}, {None, None}},
  DefaultFormatType->DefaultInputFormatType,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  CounterIncrements->"NumberedEquation",
  FormatTypeAutoConvert->False],

Cell[StyleData["NumberedEquation", "Printout"],
  CellMargins->{{55, 55}, {0, 10}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["NumberedFigure"],
  CellMargins->{{52, 4}, {0, 8}},
  PageBreakAbove->False,
  CellFrameLabels->{{Cell[ 
          TextData[ {"Figure ", 
            CounterBox[ "Chapter"], ".", 
            CounterBox[ "NumberedFigure"], ":"}], FontWeight -> "Bold", 
          CellBaseline -> Baseline], None}, {None, None}},
  CounterIncrements->"NumberedFigure",
  FormatTypeAutoConvert->False,
  FontFamily->"Helvetica",
  FontSize->9],

Cell[StyleData["NumberedFigure", "Printout"],
  FontSize->8]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["NumberedTable"],
  CellMargins->{{42, 4}, {0, 8}},
  PageBreakAbove->False,
  CellFrameMargins->8,
  CellFrameLabels->{{Cell[ 
          TextData[ {"Table ", 
            CounterBox[ "Chapter"], ".", 
            CounterBox[ "NumberedTable"], ":"}], FontWeight -> "Bold", 
          CellBaseline -> Baseline], None}, {None, None}},
  CounterIncrements->"NumberedTable",
  FormatTypeAutoConvert->False,
  FontFamily->"Helvetica",
  FontSize->9],

Cell[StyleData["NumberedTable", "Presentation"],
  CellMargins->{{42, 4}, {0, 8}},
  PageBreakAbove->False,
  CellFrameMargins->8,
  CellFrameLabels->{{Cell[ 
          TextData[ {"Table ", 
            CounterBox[ "Chapter"], ".", 
            CounterBox[ "NumberedTable"], ":"}], FontWeight -> "Bold", 
          CellBaseline -> Baseline], None}, {None, None}},
  CounterIncrements->"NumberedTable",
  FormatTypeAutoConvert->False,
  FontFamily->"Helvetica",
  FontSize->11],

Cell[StyleData["NumberedTable", "Printout"],
  CellMargins->{{18, Inherited}, {Inherited, Inherited}},
  FontSize->8]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ProgramListing"],
  CellMargins->{{42, 4}, {12, 6}},
  CellFrameMargins->8,
  CellFrameLabels->{{Cell[ 
          TextData[ {"Program Listing ", 
            CounterBox[ "Chapter"], ".", 
            CounterBox[ "NumberedProgram"], ":"}], FontWeight -> "Bold", 
          CellBaseline -> Baseline], None}, {None, None}},
  CounterIncrements->"NumberedProgram",
  FormatTypeAutoConvert->False,
  FontFamily->"Helvetica",
  FontSize->9],

Cell[StyleData["ProgramListing", "Printout"],
  CellMargins->{{18, Inherited}, {Inherited, Inherited}},
  FontSize->10]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Input/Output", "Section"],

Cell["\<\
The cells in this section define styles used for input and output to the \
kernel.  Be careful when modifying, renaming, or removing these styles, \
because the front end associates special meanings with these style names. \
Some attributes for these styles are actually set in FormatType Styles (in \
the last section of this stylesheet). \
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["Input"],
  CellMargins->{{60, 10}, {5, 7}},
  Evaluatable->True,
  CellGroupingRules->"InputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  DefaultFormatType->DefaultInputFormatType,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  AutoItalicWords->{},
  LanguageCategory->"Formula",
  FormatType->InputForm,
  ShowStringCharacters->True,
  NumberMarks->True,
  LinebreakAdjustments->{0.85, 2, 10, 0, 1},
  CounterIncrements->"Input",
  FontWeight->"Bold"],

Cell[StyleData["Input", "Presentation"],
  CellMargins->{{72, Inherited}, {8, 10}},
  LineSpacing->{1, 0},
  FontSize->16],

Cell[StyleData["Input", "Printout"],
  CellMargins->{{48, 0}, {4, 6}},
  LinebreakAdjustments->{0.85, 2, 10, 1, 1},
  FontSize->9]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Output"],
  CellMargins->{{60, 10}, {7, 5}},
  CellEditDuplicate->True,
  CellGroupingRules->"OutputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  DefaultFormatType->DefaultOutputFormatType,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  AutoItalicWords->{},
  LanguageCategory->"Formula",
  FormatType->InputForm,
  CounterIncrements->"Output"],

Cell[StyleData["Output", "Presentation"],
  CellMargins->{{72, Inherited}, {10, 8}},
  LineSpacing->{1, 0},
  FontSize->16],

Cell[StyleData["Output", "Printout"],
  CellMargins->{{48, 0}, {6, 4}},
  FontSize->9]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Message"],
  CellMargins->{{60, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"OutputGrouping",
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  AutoItalicWords->{},
  FormatType->InputForm,
  CounterIncrements->"Message",
  StyleMenuListing->None,
  FontSize->11,
  FontColor->RGBColor[0, 0, 1]],

Cell[StyleData["Message", "Presentation"],
  LineSpacing->{1, 0},
  FontSize->16],

Cell[StyleData["Message", "Printout"],
  CellMargins->{{39, Inherited}, {Inherited, Inherited}},
  FontSize->9,
  FontColor->GrayLevel[0]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Print"],
  CellMargins->{{45, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"OutputGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GroupPageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  AutoItalicWords->{},
  FormatType->InputForm,
  CounterIncrements->"Print",
  StyleMenuListing->None],

Cell[StyleData["Print", "Presentation"],
  CellMargins->{{72, Inherited}, {Inherited, Inherited}},
  LineSpacing->{1, 0},
  FontSize->16],

Cell[StyleData["Print", "Printout"],
  CellMargins->{{39, Inherited}, {Inherited, Inherited}},
  FontSize->8]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Graphics"],
  CellMargins->{{40, Inherited}, {Inherited, Inherited}},
  CellGroupingRules->"GraphicsGrouping",
  CellHorizontalScrolling->True,
  PageBreakWithin->False,
  GeneratedCell->True,
  CellAutoOverwrite->True,
  ShowCellLabel->False,
  DefaultFormatType->DefaultOutputFormatType,
  LanguageCategory->None,
  FormatType->InputForm,
  CounterIncrements->"Graphics",
  ImageMargins->{{43, Inherited}, {Inherited, 0}},
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontSize->10],

Cell[StyleData["Graphics", "Presentation"],
  ImageMargins->{{62, Inherited}, {Inherited, 0}}],

Cell[StyleData["Graphics", "Printout"],
  ImageMargins->{{30, Inherited}, {Inherited, 0}},
  Magnification->0.8]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["CellLabel"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->9,
  FontColor->RGBColor[0, 0, 1]],

Cell[StyleData["CellLabel", "Presentation"],
  FontSize->12],

Cell[StyleData["CellLabel", "Printout"],
  FontFamily->"Courier",
  FontSize->8,
  FontSlant->"Italic",
  FontColor->GrayLevel[0]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Inline Formatting", "Section"],

Cell["\<\
These styles are for modifying individual words or letters in a cell \
exclusive of the cell tag.\
\>", "Text"],

Cell[StyleData["RM"],
  StyleMenuListing->None,
  FontWeight->"Plain",
  FontSlant->"Plain"],

Cell[StyleData["BF"],
  StyleMenuListing->None,
  FontWeight->"Bold"],

Cell[StyleData["IT"],
  StyleMenuListing->None,
  FontSlant->"Italic"],

Cell[StyleData["TR"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Plain",
  FontSlant->"Plain"],

Cell[StyleData["TI"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["TB"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Bold",
  FontSlant->"Plain"],

Cell[StyleData["TBI"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontWeight->"Bold",
  FontSlant->"Italic"],

Cell[StyleData["MR"],
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontWeight->"Plain",
  FontSlant->"Plain"],

Cell[StyleData["MO"],
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["MB"],
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontWeight->"Bold",
  FontSlant->"Plain"],

Cell[StyleData["MBO"],
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontWeight->"Bold",
  FontSlant->"Italic"],

Cell[StyleData["SR"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontWeight->"Plain",
  FontSlant->"Plain"],

Cell[StyleData["SO"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["SB"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontWeight->"Bold",
  FontSlant->"Plain"],

Cell[StyleData["SBO"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontWeight->"Bold",
  FontSlant->"Italic"],

Cell[CellGroupData[{

Cell[StyleData["SO10"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->10,
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["SO10", "Presentation"],
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->10,
  FontWeight->"Plain",
  FontSlant->"Italic"]
}, Closed]],

Cell[StyleData["TermDefinition"],
  FontFamily->"Times",
  FontWeight->"Plain",
  FontSlant->"Italic"],

Cell[StyleData["SmallCaps"],
  StyleMenuListing->None,
  FontSize->9],

Cell[StyleData["InputComment"],
  StyleMenuListing->None,
  FontColor->GrayLevel[0.2]]
}, Closed]],

Cell[CellGroupData[{

Cell["Formulas and Programming", "Section"],

Cell[CellGroupData[{

Cell[StyleData["InlineFormula"],
  CellMargins->{{10, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  LanguageCategory->"Formula",
  ScriptLevel->1,
  SingleLetterItalics->True],

Cell[StyleData["InlineFormula", "Presentation"],
  CellMargins->{{24, 10}, {10, 10}},
  LineSpacing->{1, 5},
  FontSize->16],

Cell[StyleData["InlineFormula", "Printout"],
  CellMargins->{{2, 0}, {6, 6}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["DisplayFormula"],
  CellMargins->{{36, Inherited}, {Inherited, Inherited}},
  CellHorizontalScrolling->True,
  DefaultFormatType->DefaultTraditionalFormatType,
  HyphenationOptions->{"HyphenationCharacter"->"\[Continuation]"},
  LanguageCategory->"Formula",
  ScriptLevel->0,
  SingleLetterItalics->True,
  UnderoverscriptBoxOptions->{LimitsPositioning->True}],

Cell[StyleData["DisplayFormula", "Presentation"],
  LineSpacing->{1, 5},
  FontSize->16],

Cell[StyleData["DisplayFormula", "Printout"],
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["Program"],
  CellFrame->{{0, 0}, {0.5, 0.5}},
  CellMargins->{{42, 4}, {0, 8}},
  CellHorizontalScrolling->True,
  Hyphenation->False,
  LanguageCategory->"Formula",
  ScriptLevel->1,
  FontFamily->"Courier"],

Cell[StyleData["Program", "Presentation"],
  CellMargins->{{24, 10}, {10, 10}},
  LineSpacing->{1, 5},
  FontSize->16],

Cell[StyleData["Program", "Printout"],
  CellMargins->{{Inherited, 0}, {6, 6}},
  FontSize->9]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Hyperlink Styles", "Section"],

Cell["\<\
The cells below define styles useful for making hypertext ButtonBoxes.  The \
\"Hyperlink\" style is for links within the same Notebook, or between \
Notebooks.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["Hyperlink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0.269993, 0.308507, 0.6],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`NotebookLocate[ #2]}]&),
  Active->True,
  ButtonNote->ButtonData}],

Cell[StyleData["Hyperlink", "Presentation"],
  FontSize->16],

Cell[StyleData["Hyperlink", "Printout"],
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell["\<\
The following styles are for linking automatically to the on-line help \
system.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["MainBookLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0.269993, 0.308507, 0.6],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "MainBook", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["MainBookLink", "Presentation"],
  FontSize->16],

Cell[StyleData["MainBookLink", "Printout"],
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["AddOnsLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontFamily->"Courier",
  FontColor->RGBColor[0.269993, 0.308507, 0.6],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["AddOnsLink", "Presentation"],
  FontSize->16],

Cell[StyleData["AddOnsLink", "Printout"],
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["RefGuideLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontFamily->"Courier",
  FontColor->RGBColor[0.269993, 0.308507, 0.6],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "RefGuide", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["RefGuideLink", "Presentation"],
  FontSize->16],

Cell[StyleData["RefGuideLink", "Printout"],
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["GettingStartedLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0.269993, 0.308507, 0.6],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "GettingStarted", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["GettingStartedLink", "Presentation"],
  FontSize->16],

Cell[StyleData["GettingStartedLink", "Printout"],
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["OtherInformationLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0.269993, 0.308507, 0.6],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "OtherInformation", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["OtherInformationLink", "Presentation"],
  FontSize->16],

Cell[StyleData["OtherInformationLink", "Printout"],
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["MasterIndexLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0.269993, 0.308507, 0.6],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "MasterIndex", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["MasterIndexLink", "Printout"],
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["SolutionsLink"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  FontColor->RGBColor[0.269993, 0.308507, 0.6],
  FontVariations->{"Underline"->True},
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`HelpBrowserLookup[ "AddOns", #]}]&),
  Active->True,
  ButtonFrame->"None"}],

Cell[StyleData["SolutionsLink", "Presentation"],
  FontSize->16],

Cell[StyleData["SolutionsLink", "Printout"],
  FontSize->10,
  FontColor->GrayLevel[0],
  FontVariations->{"Underline"->False}]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Styles for Headers and Footers", "Section"],

Cell[StyleData["Header"],
  CellMargins->{{0, 0}, {4, 1}},
  DefaultNewInlineCellStyle->"None",
  LanguageCategory->"NaturalLanguage",
  StyleMenuListing->None,
  FontFamily->"Helvetica",
  FontSize->8],

Cell[StyleData["PageFooter"],
  StyleMenuListing->None,
  FontFamily->"Times",
  FontSize->7.5,
  FontSlant->"Italic"],

Cell[StyleData["PageNumber"],
  CellMargins->{{0, 0}, {4, 1}},
  StyleMenuListing->None,
  FontFamily->"Times",
  FontSize->9.5,
  FontWeight->"Bold"]
}, Closed]],

Cell[CellGroupData[{

Cell["Palette Styles", "Section"],

Cell["\<\
The cells below define styles that define standard ButtonFunctions, for use \
in palette buttons.\
\>", "Text"],

Cell[StyleData["Paste"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`NotebookApply[ 
        FrontEnd`InputNotebook[ ], #, After]}]&)}],

Cell[StyleData["Evaluate"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`NotebookApply[ 
        FrontEnd`InputNotebook[ ], #, All], 
      SelectionEvaluate[ 
        FrontEnd`InputNotebook[ ], All]}]&)}],

Cell[StyleData["EvaluateCell"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`NotebookApply[ 
        FrontEnd`InputNotebook[ ], #, All], 
      FrontEnd`SelectionMove[ 
        FrontEnd`InputNotebook[ ], All, Cell, 1], 
      FrontEnd`SelectionEvaluateCreateCell[ 
        FrontEnd`InputNotebook[ ], All]}]&)}],

Cell[StyleData["CopyEvaluate"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`SelectionCreateCell[ 
        FrontEnd`InputNotebook[ ], All], 
      FrontEnd`NotebookApply[ 
        FrontEnd`InputNotebook[ ], #, All], 
      FrontEnd`SelectionEvaluate[ 
        FrontEnd`InputNotebook[ ], All]}]&)}],

Cell[StyleData["CopyEvaluateCell"],
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic,
  ButtonBoxOptions->{ButtonFunction:>(FrontEndExecute[ {
      FrontEnd`SelectionCreateCell[ 
        FrontEnd`InputNotebook[ ], All], 
      FrontEnd`NotebookApply[ 
        FrontEnd`InputNotebook[ ], #, All], 
      FrontEnd`SelectionEvaluateCreateCell[ 
        FrontEnd`InputNotebook[ ], All]}]&)}]
}, Closed]],

Cell[CellGroupData[{

Cell["Placeholder Styles", "Section"],

Cell["\<\
The cells below define styles useful for making placeholder objects in \
palette templates.\
\>", "Text"],

Cell[CellGroupData[{

Cell[StyleData["Placeholder"],
  Placeholder->True,
  StyleMenuListing->None,
  FontSlant->"Italic",
  FontColor->RGBColor[0.890623, 0.864698, 0.384756],
  TagBoxOptions->{Editable->False,
  Selectable->False,
  StripWrapperBoxes->False}],

Cell[StyleData["Placeholder", "Presentation"]],

Cell[StyleData["Placeholder", "Printout"]]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["PrimaryPlaceholder"],
  StyleMenuListing->None,
  DrawHighlighted->True,
  FontSlant->"Italic",
  Background->RGBColor[0.912505, 0.891798, 0.507774],
  TagBoxOptions->{Editable->False,
  Selectable->False,
  StripWrapperBoxes->False}],

Cell[StyleData["PrimaryPlaceholder", "Presentation"]],

Cell[StyleData["PrimaryPlaceholder", "Printout"]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["FormatType Styles", "Section"],

Cell["\<\
The cells below define styles that are mixed in with the styles of most \
cells.  If a cell's FormatType matches the name of one of the styles defined \
below, then that style is applied between the cell's style and its own \
options. This is particularly true of Input and Output.\
\>", "Text"],

Cell[StyleData["CellExpression"],
  PageWidth->Infinity,
  CellMargins->{{6, Inherited}, {Inherited, Inherited}},
  ShowCellLabel->False,
  ShowSpecialCharacters->False,
  AllowInlineCells->False,
  Hyphenation->False,
  AutoItalicWords->{},
  StyleMenuListing->None,
  FontFamily->"Courier",
  FontSize->12,
  Background->GrayLevel[1]],

Cell[StyleData["InputForm"],
  InputAutoReplacements->{},
  AllowInlineCells->False,
  Hyphenation->False,
  StyleMenuListing->None,
  FontFamily->"Courier"],

Cell[StyleData["OutputForm"],
  PageWidth->Infinity,
  TextAlignment->Left,
  LineSpacing->{0.6, 1},
  StyleMenuListing->None,
  FontFamily->"Courier"],

Cell[StyleData["StandardForm"],
  InputAutoReplacements->{
    "->"->"\[Rule]", ":>"->"\[RuleDelayed]", "<="->"\[LessEqual]", 
      ">="->"\[GreaterEqual]", "!="->"\[NotEqual]", "=="->"\[Equal]", 
      Inherited},
  "TwoByteSyntaxCharacterAutoReplacement"->True,
  LineSpacing->{1.25, 0},
  StyleMenuListing->None,
  FontFamily->"Courier"],

Cell[StyleData["TraditionalForm"],
  InputAutoReplacements->{
    "->"->"\[Rule]", ":>"->"\[RuleDelayed]", "<="->"\[LessEqual]", 
      ">="->"\[GreaterEqual]", "!="->"\[NotEqual]", "=="->"\[Equal]", 
      Inherited},
  "TwoByteSyntaxCharacterAutoReplacement"->True,
  LineSpacing->{1.25, 0},
  SingleLetterItalics->True,
  TraditionalFunctionNotation->True,
  DelimiterMatching->None,
  StyleMenuListing->None],

Cell["\<\
The style defined below is mixed in to any cell that is in an inline cell \
within another.\
\>", "Text"],

Cell[StyleData["InlineCell"],
  ScriptLevel->1,
  StyleMenuListing->None],

Cell[StyleData["InlineCellEditing"],
  StyleMenuListing->None,
  Background->RGBColor[1, 0.749996, 0.8]]
}, Closed]],

Cell[CellGroupData[{

Cell["Automatic Styles", "Section"],

Cell["\<\
The cells below define styles that are used to affect the display of certain \
types of objects in typeset expressions.  For example, \"UnmatchedBracket\" \
style defines how unmatched bracket, curly bracket, and parenthesis \
characters are displayed (typically by coloring them to make them stand out).\
\
\>", "Text"],

Cell[StyleData["UnmatchedBracket"],
  StyleMenuListing->None,
  FontColor->RGBColor[0.760006, 0.330007, 0.8]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section"],

Cell[CellGroupData[{

Cell[StyleData["ExerciseSubsection"],
  CellFrame->{{0, 0}, {0, 0.5}},
  CellMargins->{{12, 0}, {8, 36}},
  CellGroupingRules->{"SectionGrouping", 40},
  LineSpacing->{1, 6},
  LanguageCategory->"NaturalLanguage",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}, {
      "ExerciseItem", 0}, {"ExerciseSubItem", 0}},
  FontFamily->"Times",
  FontSize->16,
  FontWeight->"Bold",
  FontSlant->"Italic"],

Cell[StyleData["ExerciseSubsection", "Presentation"],
  LineSpacing->{1, 0},
  FontSize->24],

Cell[StyleData["ExerciseSubsection", "Printout"],
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ExerciseItem"],
  CellMargins->{{22, 4}, {0, 8}},
  CellGroupingRules->{"SectionGrouping", 45},
  CellFrameLabels->{{Cell[ 
          TextData[ {
            CounterBox[ "ExerciseItem"], "."}], "ExerciseItemNumber", 
          CellBaseline -> Baseline], None}, {None, None}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterIncrements->"ExerciseItem"],

Cell[StyleData["ExerciseItem", "Presentation"],
  LineSpacing->{1, 5},
  FontSize->14],

Cell[StyleData["ExerciseItem", "Printout"],
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ExerciseSubItem"],
  CellMargins->{{40, 4}, {0, 8}},
  CellGroupingRules->{"SectionGrouping", 45},
  CellFrameLabels->{{Cell[ 
          TextData[ {
            CounterBox[ "ExerciseSubItem", CounterFunction -> (Part[ 
                CharacterRange[ "a", "z"], #]&)], "."}], 
          "ExerciseSubItemNumber", CellBaseline -> Baseline], None}, {
      None, None}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterIncrements->"ExerciseSubItem"],

Cell[StyleData["ExerciseSubItem", "Presentation"],
  LineSpacing->{1, 5},
  FontSize->14],

Cell[StyleData["ExerciseSubItem", "Printout"],
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ExercisesNoLabel"],
  CellMargins->{{44, 4}, {0, 8}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterStyleMenuListing->None],

Cell[StyleData["ExercisesNoLabel", "Presentation"],
  FontSize->14],

Cell[StyleData["ExercisesNoLabel", "Printout"],
  CellMargins->{{38, 4}, {0, 8}},
  ParagraphIndent->-14,
  FontSize->11]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ExerciseItemNumber"],
  CellSize->{20, Inherited},
  TextAlignment->Right,
  FontWeight->"Bold"],

Cell[StyleData["ExerciseItemNumber", "Printout"],
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["ExerciseSubItemNumber"],
  CellSize->{18, Inherited},
  TextAlignment->Right,
  FontWeight->"Bold"],

Cell[StyleData["ExerciseSubItemNumber", "Printout"],
  FontSize->10]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Solutions", "Section"],

Cell[CellGroupData[{

Cell[StyleData["SolutionsSection"],
  CellDingbat->None,
  CellMargins->{{12, Inherited}, {5, 18}},
  CellGroupingRules->{"SectionGrouping", 30},
  PageBreakBelow->False,
  FontFamily->"Times",
  FontSize->20,
  FontWeight->"Bold"],

Cell[StyleData["SolutionsSection", "Presentation"],
  CellMargins->{{24, Inherited}, {Inherited, Inherited}},
  LineSpacing->{1, 0},
  FontSize->24],

Cell[StyleData["SolutionsSection", "Printout"],
  CellMargins->{{Inherited, 0}, {7, 18}},
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["SolutionsSubsection"],
  CellMargins->{{24, 0}, {12, 6}},
  CellGroupingRules->{"SectionGrouping", 50},
  PageBreakBelow->False,
  DefaultNewInlineCellStyle->"None",
  LineSpacing->{1, 7},
  LanguageCategory->"NaturalLanguage",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}, {
      "SolutionItem", 0}},
  FontFamily->"Helvetica",
  FontSize->14,
  FontWeight->"Bold"],

Cell[StyleData["SolutionsSubsection", "Presentation"],
  LineSpacing->{1, 0},
  FontSize->16],

Cell[StyleData["SolutionsSubsection", "Printout"],
  CellMargins->{{13, 0}, {7, 22}},
  FontSize->14]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["SolutionItem"],
  CellMargins->{{18, 4}, {0, 8}},
  CellGroupingRules->{"SectionGrouping", 55},
  CellFrameLabels->{{Cell[ 
          TextData[ {
            CounterBox[ "SolutionItem"], "."}], "ExerciseItemNumber", 
          CellBaseline -> Baseline], None}, {None, None}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterIncrements->"SolutionItem"],

Cell[StyleData["SolutionItem", "Presentation"],
  CellMargins->{{22, 4}, {0, 8}},
  FontSize->14],

Cell[StyleData["SolutionItem", "Printout"],
  CellMargins->{{22, 4}, {0, 8}},
  FontSize->10]
}, Closed]],

Cell[CellGroupData[{

Cell[StyleData["SolutionNoLabel"],
  CellMargins->{{30, 4}, {0, 8}},
  Hyphenation->True,
  LineSpacing->{1, 3},
  ParagraphSpacing->{0, 8},
  CounterStyleMenuListing->None],

Cell[StyleData["SolutionNoLabel", "Presentation"],
  FontSize->14],

Cell[StyleData["SolutionNoLabel", "Printout"],
  CellMargins->{{38, 4}, {0, 8}},
  ParagraphIndent->-14,
  FontSize->11]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Spacer Cells", "Section"],

Cell[StyleData["SectionSpacer", "Working"],
  CellElementSpacings->{"CellMinHeight"->1,
  "ClosedCellHeight"->0},
  CellOpen->False],

Cell[StyleData["SectionSpacer", "Production"],
  CellMargins->{{Inherited, Inherited}, {24, 0}},
  PageBreakAbove->False],

Cell[StyleData["SubsectionSpacer", "Working"],
  CellElementSpacings->{"CellMinHeight"->1,
  "ClosedCellHeight"->0},
  CellOpen->False],

Cell[StyleData["SubsectionSpacer", "Production"],
  CellMargins->{{Inherited, Inherited}, {0, 0}},
  CellElementSpacings->{"CellMinHeight"->0},
  PageBreakAbove->False]
}, Open  ]]
}, Open  ]]
},
FrontEndVersion->"5.0 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 685}},
ScreenStyleEnvironment->"Working",
WindowSize->{524, 544},
WindowMargins->{{22, Automatic}, {46, Automatic}},
Magnification->1
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{}
*)

(*CellTagsIndex
CellTagsIndex->{}
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 233, 7, 154, "Subtitle"],

Cell[CellGroupData[{
Cell[2034, 64, 42, 0, 73, "Section"],
Cell[2079, 66, 140, 3, 35, 31, 0, "StyleData", "All", "Working"],
Cell[2222, 71, 146, 3, 35, 36, 0, "StyleData", "All", "Presentation"],
Cell[2371, 76, 186, 4, 33, 32, 0, "StyleData", "All", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[2594, 85, 35, 0, 43, "Section"],
Cell[2632, 87, 99, 2, 33, "Text"],
Cell[2734, 91, 398, 10, 35, 27, 0, "StyleData", "Notebook", "All",
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{
Cell[3169, 106, 38, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[3232, 110, 541, 14, 107, 26, 0, "StyleData", "Chapter", "All",
  CounterIncrements->"Chapter",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subtitle", 0}, {"Subsubtitle", 0}, {"Head1", 0}}],
Cell[3776, 126, 80, 2, 116, 42, 0, "StyleData", "Chapter", "Presentation"],
Cell[3859, 130, 74, 2, 66, 38, 0, "StyleData", "Chapter", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[3970, 137, 1061, 25, 68, 27, 0, "StyleData", "Subtitle", "All",
  CounterIncrements->"Subtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}, {
      "Subsubtitle", 0}},
  StyleMenuListing->None],
Cell[5034, 164, 119, 3, 98, 43, 0, "StyleData", "Subtitle", "Presentation"],
Cell[5156, 169, 90, 2, 58, 39, 0, "StyleData", "Subtitle", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[5283, 176, 1063, 25, 57, 30, 0, "StyleData", "Subsubtitle", "All",
  CounterIncrements->"Subsubtitle",
  CounterAssignments->{{"Section", 0}, {"Equation", 0}, {"Figure", 0}},
  StyleMenuListing->None],
Cell[6349, 203, 122, 3, 85, 46, 0, "StyleData", "Subsubtitle", \
"Presentation"],
Cell[6474, 208, 93, 2, 53, 42, 0, "StyleData", "Subsubtitle", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[6604, 215, 630, 17, 44, 26, 0, "StyleData", "Section", "All",
  CounterIncrements->"Section",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}, {
      "Exercise", 0}}],
Cell[7237, 234, 139, 3, 58, 42, 0, "StyleData", "Section", "Presentation"],
Cell[7379, 239, 96, 2, 59, 38, 0, "StyleData", "Section", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[7512, 246, 341, 9, 45, 29, 0, "StyleData", "Subsection", "All",
  CounterAssignments->{{"Subsubsection", 0}}],
Cell[7856, 257, 121, 3, 85, 45, 0, "StyleData", "Subsection", \
"Presentation"],
Cell[7980, 262, 92, 2, 61, 41, 0, "StyleData", "Subsection", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[8109, 269, 1014, 24, 44, 32, 0, "StyleData", "Subsubsection", "All",
  CounterIncrements->"Subsubsection",
  StyleMenuListing->None],
Cell[9126, 295, 124, 3, 74, 48, 0, "StyleData", "Subsubsection", \
"Presentation"],
Cell[9253, 300, 94, 2, 51, 44, 0, "StyleData", "Subsubsection", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[9384, 307, 362, 11, 48, 27, 0, "StyleData", "TitleBar", "All",
  StyleMenuListing->None],
Cell[9749, 320, 115, 3, 46, 43, 0, "StyleData", "TitleBar", "Presentation"],
Cell[9867, 325, 177, 4, 5, 39, 0, "StyleData", "TitleBar", "Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[10093, 335, 39, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[10157, 339, 177, 6, 46, 23, 0, "StyleData", "Text", "All",
  CounterIncrements->"Text"],
Cell[10337, 347, 115, 3, 56, 39, 0, "StyleData", "Text", "Presentation"],
Cell[10455, 352, 111, 3, 41, 35, 0, "StyleData", "Text", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[10603, 360, 159, 5, 46, 28, 0, "StyleData", "TextFirst", "All",
  CounterIncrements->"Text"],
Cell[10765, 367, 120, 3, 56, 44, 0, "StyleData", "TextFirst", \
"Presentation"],
Cell[10888, 372, 116, 3, 41, 40, 0, "StyleData", "TextFirst", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[11041, 380, 263, 9, 44, 31, 0, "StyleData", "BulletedList", "All",
  CounterIncrements->"Text"],
Cell[11307, 391, 63, 1, 48, 47, 0, "StyleData", "BulletedList", \
"Presentation"],
Cell[11373, 394, 109, 2, 41, 43, 0, "StyleData", "BulletedList", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[11519, 401, 396, 10, 40, 31, 0, "StyleData", "ItemizedText", "All",
  CounterIncrements->"ItemNumber"],
Cell[11918, 413, 63, 1, 44, 47, 0, "StyleData", "ItemizedText", \
"Presentation"],
Cell[11984, 416, 93, 2, 39, 43, 0, "StyleData", "ItemizedText", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[12114, 423, 266, 9, 44, 34, 0, "StyleData", "BulletedSubItem", "All",
  CounterIncrements->"Text"],
Cell[12383, 434, 66, 1, 48, 50, 0, "StyleData", "BulletedSubItem", \
"Presentation"],
Cell[12452, 437, 112, 2, 41, 46, 0, "StyleData", "BulletedSubItem", \
"Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[12613, 445, 32, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[12670, 449, 478, 13, 56, 33, 0, "StyleData", "ChapterHeading", "All",
  CounterIncrements->"ChapterHeading",
  StyleMenuListing->None],
Cell[13151, 464, 94, 1, 53, 45, 0, "StyleData", "ChapterHeading", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[13282, 470, 189, 6, 46, 30, 0, "StyleData", "ChapterHead", "All",
  CounterIncrements->"ChapterHeading",
  StyleMenuListing->None],
Cell[13474, 478, 42, 0, 46, 42, 0, "StyleData", "ChapterHead", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[13553, 483, 186, 5, 63, 33, 0, "StyleData", "PrefaceSection", "All",
  StyleMenuListing->None],
Cell[13742, 490, 88, 2, 66, 49, 0, "StyleData", "PrefaceSection", \
"Presentation"],
Cell[13833, 494, 96, 2, 63, 45, 0, "StyleData", "PrefaceSection", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[13966, 501, 204, 7, 51, 34, 0, "StyleData", "ChapterAbstract", "All",
  StyleMenuListing->None],
Cell[14173, 510, 66, 1, 53, 50, 0, "StyleData", "ChapterAbstract", \
"Presentation"],
Cell[14242, 513, 86, 2, 49, 46, 0, "StyleData", "ChapterAbstract", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[14365, 520, 210, 7, 59, 24, 0, "StyleData", "Quote", "All",
  CounterIncrements->"Text",
  StyleMenuListing->None],
Cell[14578, 529, 56, 1, 60, 40, 0, "StyleData", "Quote", "Presentation"],
Cell[14637, 532, 110, 3, 39, 36, 0, "StyleData", "Quote", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[14784, 540, 210, 7, 38, 29, 0, "StyleData", "References", "All",
  StyleMenuListing->None],
Cell[14997, 549, 119, 3, 49, 45, 0, "StyleData", "References", \
"Presentation"],
Cell[15119, 554, 115, 3, 35, 41, 0, "StyleData", "References", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[15271, 562, 359, 8, 50, 31, 0, "StyleData", "DisplayTable", "All"],
Cell[15633, 572, 86, 2, 54, 47, 0, "StyleData", "DisplayTable", \
"Presentation"],
Cell[15722, 576, 59, 1, 47, 43, 0, "StyleData", "DisplayTable", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[15818, 582, 135, 3, 44, 26, 0, "StyleData", "Picture", "All"],
Cell[15956, 587, 94, 2, 29, 38, 0, "StyleData", "Picture", "Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[16099, 595, 34, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[16158, 599, 373, 10, 52, 34, 0, "StyleData", "NonPrintSection", "All",
  CounterIncrements->"Section",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}}],
Cell[16534, 611, 66, 1, 52, 50, 0, "StyleData", "NonPrintSection", \
"Presentation"]
}, Open  ]],

Cell[CellGroupData[{
Cell[16637, 617, 603, 18, 46, 32, 0, "StyleData", "NonPrintInput", "All",
  CounterIncrements->"Input"],
Cell[17243, 637, 130, 3, 54, 48, 0, "StyleData", "NonPrintInput", \
"Presentation"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{
Cell[17422, 646, 49, 0, 73, "Section"],
Cell[17474, 648, 225, 4, 71, "Text"],

Cell[CellGroupData[{
Cell[17724, 656, 386, 8, 42, 35, 0, "StyleData", "NumberedEquation", "All",
  CounterIncrements->"NumberedEquation"],
Cell[18113, 666, 99, 2, 39, 47, 0, "StyleData", "NumberedEquation", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[18249, 673, 440, 11, 37, 33, 0, "StyleData", "NumberedFigure", "All",
  CounterIncrements->"NumberedFigure"],
Cell[18692, 686, 60, 1, 35, 45, 0, "StyleData", "NumberedFigure", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[18789, 692, 459, 12, 37, 32, 0, "StyleData", "NumberedTable", "All",
  CounterIncrements->"NumberedTable"],
Cell[19251, 706, 476, 12, 39, 48, 0, "StyleData", "NumberedTable", \
"Presentation",
  CounterIncrements->"NumberedTable"],
Cell[19730, 720, 117, 2, 35, 44, 0, "StyleData", "NumberedTable", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[19884, 727, 450, 11, 47, 33, 0, "StyleData", "ProgramListing", "All",
  CounterIncrements->"NumberedProgram"],
Cell[20337, 740, 119, 2, 48, 45, 0, "StyleData", "ProgramListing", \
"Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[20505, 748, 42, 0, 43, "Section"],
Cell[20550, 750, 364, 6, 90, "Text"],

Cell[CellGroupData[{
Cell[20939, 760, 547, 16, 44, 24, 0, "StyleData", "Input", "All",
  CounterIncrements->"Input"],
Cell[21489, 778, 122, 3, 54, 40, 0, "StyleData", "Input", "Presentation"],
Cell[21614, 783, 130, 3, 39, 36, 0, "StyleData", "Input", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[21781, 791, 489, 14, 44, 25, 0, "StyleData", "Output", "All",
  CounterIncrements->"Output"],
Cell[22273, 807, 123, 3, 54, 41, 0, "StyleData", "Output", "Presentation"],
Cell[22399, 812, 86, 2, 39, 37, 0, "StyleData", "Output", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[22522, 819, 521, 15, 34, 26, 0, "StyleData", "Message", "All",
  CounterIncrements->"Message",
  StyleMenuListing->None],
Cell[23046, 836, 81, 2, 39, 42, 0, "StyleData", "Message", "Presentation"],
Cell[23130, 840, 138, 3, 32, 38, 0, "StyleData", "Message", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[23305, 848, 502, 14, 35, 24, 0, "StyleData", "Print", "All",
  CounterIncrements->"Print",
  StyleMenuListing->None],
Cell[23810, 864, 137, 3, 39, 40, 0, "StyleData", "Print", "Presentation"],
Cell[23950, 869, 109, 2, 30, 36, 0, "StyleData", "Print", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[24096, 876, 507, 15, 32, 27, 0, "StyleData", "Graphics", "All",
  CounterIncrements->"Graphics",
  StyleMenuListing->None],
Cell[24606, 893, 94, 1, 32, 43, 0, "StyleData", "Graphics", "Presentation"],
Cell[24703, 896, 112, 2, 27, 39, 0, "StyleData", "Graphics", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[24852, 903, 128, 4, 32, 28, 0, "StyleData", "CellLabel", "All",
  StyleMenuListing->None],
Cell[24983, 909, 60, 1, 35, 44, 0, "StyleData", "CellLabel", "Presentation"],
Cell[25046, 912, 130, 4, 28, 40, 0, "StyleData", "CellLabel", "Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[25225, 922, 36, 0, 43, "Section"],
Cell[25264, 924, 121, 3, 52, "Text"],
Cell[25388, 929, 92, 3, 35, 21, 0, "StyleData", "RM", "All",
  StyleMenuListing->None],
Cell[25483, 934, 69, 2, 35, 21, 0, "StyleData", "BF", "All",
  StyleMenuListing->None],
Cell[25555, 938, 70, 2, 35, 21, 0, "StyleData", "IT", "All",
  StyleMenuListing->None],
Cell[25628, 942, 115, 4, 35, 21, 0, "StyleData", "TR", "All",
  StyleMenuListing->None],
Cell[25746, 948, 116, 4, 35, 21, 0, "StyleData", "TI", "All",
  StyleMenuListing->None],
Cell[25865, 954, 114, 4, 35, 21, 0, "StyleData", "TB", "All",
  StyleMenuListing->None],
Cell[25982, 960, 116, 4, 35, 22, 0, "StyleData", "TBI", "All",
  StyleMenuListing->None],
Cell[26101, 966, 184, 5, 35, 21, 0, "StyleData", "MR", "All",
  StyleMenuListing->None],
Cell[26288, 973, 185, 5, 36, 21, 0, "StyleData", "MO", "All",
  StyleMenuListing->None],
Cell[26476, 980, 183, 5, 36, 21, 0, "StyleData", "MB", "All",
  StyleMenuListing->None],
Cell[26662, 987, 185, 5, 36, 22, 0, "StyleData", "MBO", "All",
  StyleMenuListing->None],
Cell[26850, 994, 119, 4, 35, 21, 0, "StyleData", "SR", "All",
  StyleMenuListing->None],
Cell[26972, 1000, 120, 4, 35, 21, 0, "StyleData", "SO", "All",
  StyleMenuListing->None],
Cell[27095, 1006, 118, 4, 35, 21, 0, "StyleData", "SB", "All",
  StyleMenuListing->None],
Cell[27216, 1012, 120, 4, 35, 22, 0, "StyleData", "SBO", "All",
  StyleMenuListing->None],

Cell[CellGroupData[{
Cell[27361, 1020, 138, 5, 33, 23, 0, "StyleData", "SO10", "All",
  StyleMenuListing->None],
Cell[27502, 1027, 154, 5, 33, 39, 0, "StyleData", "SO10", "Presentation",
  StyleMenuListing->None]
}, Closed]],
Cell[27671, 1035, 102, 3, 35, 33, 0, "StyleData", "TermDefinition", "All"],
Cell[27776, 1040, 69, 2, 32, 28, 0, "StyleData", "SmallCaps", "All",
  StyleMenuListing->None],
Cell[27848, 1044, 86, 2, 35, 31, 0, "StyleData", "InputComment", "All",
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{
Cell[27971, 1051, 43, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[28039, 1055, 244, 6, 40, 32, 0, "StyleData", "InlineFormula", "All"],
Cell[28286, 1063, 124, 3, 56, 48, 0, "StyleData", "InlineFormula", \
"Presentation"],
Cell[28413, 1068, 93, 2, 41, 44, 0, "StyleData", "InlineFormula", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[28543, 1075, 376, 8, 35, 33, 0, "StyleData", "DisplayFormula", "All"],
Cell[28922, 1085, 88, 2, 39, 49, 0, "StyleData", "DisplayFormula", \
"Presentation"],
Cell[29013, 1089, 61, 1, 32, 45, 0, "StyleData", "DisplayFormula", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[29111, 1095, 224, 7, 57, 26, 0, "StyleData", "Program", "All"],
Cell[29338, 1104, 118, 3, 72, 42, 0, "StyleData", "Program", "Presentation"],
Cell[29459, 1109, 94, 2, 58, 38, 0, "StyleData", "Program", "Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[29602, 1117, 35, 0, 43, "Section"],
Cell[29640, 1119, 184, 4, 52, "Text"],

Cell[CellGroupData[{
Cell[29849, 1127, 318, 8, 35, 28, 0, "StyleData", "Hyperlink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[30170, 1137, 60, 1, 39, 44, 0, "StyleData", "Hyperlink", \
"Presentation"],
Cell[30233, 1140, 107, 2, 35, 40, 0, "StyleData", "Hyperlink", "Printout"]
}, Closed]],
Cell[30355, 1145, 104, 3, 33, "Text"],

Cell[CellGroupData[{
Cell[30484, 1152, 332, 8, 35, 31, 0, "StyleData", "MainBookLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[30819, 1162, 63, 1, 39, 47, 0, "StyleData", "MainBookLink", \
"Presentation"],
Cell[30885, 1165, 126, 3, 32, 43, 0, "StyleData", "MainBookLink", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[31048, 1173, 353, 9, 35, 29, 0, "StyleData", "AddOnsLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[31404, 1184, 61, 1, 38, 45, 0, "StyleData", "AddOnsLink", \
"Presentation"],
Cell[31468, 1187, 124, 3, 32, 41, 0, "StyleData", "AddOnsLink", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[31629, 1195, 357, 9, 35, 31, 0, "StyleData", "RefGuideLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[31989, 1206, 63, 1, 38, 47, 0, "StyleData", "RefGuideLink", \
"Presentation"],
Cell[32055, 1209, 126, 3, 32, 43, 0, "StyleData", "RefGuideLink", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[32218, 1217, 344, 8, 35, 37, 0, "StyleData", "GettingStartedLink", "All",
  
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[32565, 1227, 69, 1, 39, 53, 0, "StyleData", "GettingStartedLink", \
"Presentation"],
Cell[32637, 1230, 132, 3, 32, 49, 0, "StyleData", "GettingStartedLink", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[32806, 1238, 348, 8, 35, 39, 0, "StyleData", "OtherInformationLink", \
"All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[33157, 1248, 71, 1, 39, 55, 0, "StyleData", "OtherInformationLink", \
"Presentation"],
Cell[33231, 1251, 134, 3, 32, 51, 0, "StyleData", "OtherInformationLink", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[33402, 1259, 338, 8, 35, 34, 0, "StyleData", "MasterIndexLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[33743, 1269, 113, 2, 35, 46, 0, "StyleData", "MasterIndexLink", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[33893, 1276, 331, 8, 35, 32, 0, "StyleData", "SolutionsLink", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[34227, 1286, 64, 1, 39, 48, 0, "StyleData", "SolutionsLink", \
"Presentation"],
Cell[34294, 1289, 127, 3, 32, 44, 0, "StyleData", "SolutionsLink", \
"Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[34470, 1298, 49, 0, 43, "Section"],
Cell[34522, 1300, 202, 6, 32, 25, 0, "StyleData", "Header", "All",
  StyleMenuListing->None],
Cell[34727, 1308, 118, 4, 29, 29, 0, "StyleData", "PageFooter", "All",
  StyleMenuListing->None],
Cell[34848, 1314, 150, 5, 34, 29, 0, "StyleData", "PageNumber", "All",
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{
Cell[35035, 1324, 33, 0, 43, "Section"],
Cell[35071, 1326, 121, 3, 52, "Text"],
Cell[35195, 1331, 226, 5, 35, 24, 0, "StyleData", "Paste", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[35424, 1338, 295, 7, 35, 27, 0, "StyleData", "Evaluate", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[35722, 1347, 400, 9, 35, 31, 0, "StyleData", "EvaluateCell", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[36125, 1358, 387, 9, 35, 31, 0, "StyleData", "CopyEvaluate", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic],
Cell[36515, 1369, 401, 9, 35, 35, 0, "StyleData", "CopyEvaluateCell", "All",
  StyleMenuListing->None,
  ButtonStyleMenuListing->Automatic]
}, Closed]],

Cell[CellGroupData[{
Cell[36953, 1383, 37, 0, 43, "Section"],
Cell[36993, 1385, 115, 3, 52, "Text"],

Cell[CellGroupData[{
Cell[37133, 1392, 238, 7, 35, 30, 0, "StyleData", "Placeholder", "All",
  StyleMenuListing->None],
Cell[37374, 1401, 46, 0, 35, 46, 0, "StyleData", "Placeholder", \
"Presentation"],
Cell[37423, 1403, 42, 0, 35, 42, 0, "StyleData", "Placeholder", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[37502, 1408, 250, 7, 51, 37, 0, "StyleData", "PrimaryPlaceholder", "All",
  
  StyleMenuListing->None],
Cell[37755, 1417, 53, 0, 51, 53, 0, "StyleData", "PrimaryPlaceholder", \
"Presentation"],
Cell[37811, 1419, 49, 0, 51, 49, 0, "StyleData", "PrimaryPlaceholder", \
"Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[37909, 1425, 36, 0, 43, "Section"],
Cell[37948, 1427, 305, 5, 90, "Text"],
Cell[38256, 1434, 336, 11, 35, 33, 0, "StyleData", "CellExpression", "All",
  StyleMenuListing->None],
Cell[38595, 1447, 157, 5, 35, 28, 0, "StyleData", "InputForm", "All",
  StyleMenuListing->None],
Cell[38755, 1454, 151, 5, 35, 29, 0, "StyleData", "OutputForm", "All",
  StyleMenuListing->None],
Cell[38909, 1461, 341, 8, 35, 31, 0, "StyleData", "StandardForm", "All",
  StyleMenuListing->None],
Cell[39253, 1471, 412, 10, 35, 34, 0, "StyleData", "TraditionalForm", "All",
  StyleMenuListing->None],
Cell[39668, 1483, 115, 3, 33, "Text"],
Cell[39786, 1488, 73, 2, 35, 29, 0, "StyleData", "InlineCell", "All",
  StyleMenuListing->None],
Cell[39862, 1492, 104, 2, 51, 36, 0, "StyleData", "InlineCellEditing", "All",
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{
Cell[40003, 1499, 35, 0, 43, "Section"],
Cell[40041, 1501, 330, 6, 90, "Text"],
Cell[40374, 1509, 109, 2, 35, 35, 0, "StyleData", "UnmatchedBracket", "All",
  StyleMenuListing->None]
}, Closed]],

Cell[CellGroupData[{
Cell[40520, 1516, 28, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[40573, 1520, 414, 11, 88, 37, 0, "StyleData", "ExerciseSubsection", \
"All",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}, {
      "ExerciseItem", 0}, {"ExerciseSubItem", 0}}],
Cell[40990, 1533, 92, 2, 97, 53, 0, "StyleData", "ExerciseSubsection", \
"Presentation"],
Cell[41085, 1537, 65, 1, 86, 49, 0, "StyleData", "ExerciseSubsection", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[41187, 1543, 400, 10, 40, 31, 0, "StyleData", "ExerciseItem", "All",
  CounterIncrements->"ExerciseItem"],
Cell[41590, 1555, 86, 2, 41, 47, 0, "StyleData", "ExerciseItem", \
"Presentation"],
Cell[41679, 1559, 59, 1, 37, 43, 0, "StyleData", "ExerciseItem", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[41775, 1565, 495, 12, 40, 34, 0, "StyleData", "ExerciseSubItem", "All",
  CounterIncrements->"ExerciseSubItem"],
Cell[42273, 1579, 89, 2, 41, 50, 0, "StyleData", "ExerciseSubItem", \
"Presentation"],
Cell[42365, 1583, 62, 1, 37, 46, 0, "StyleData", "ExerciseSubItem", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[42464, 1589, 174, 5, 40, 35, 0, "StyleData", "ExercisesNoLabel", "All"],
Cell[42641, 1596, 67, 1, 41, 51, 0, "StyleData", "ExercisesNoLabel", \
"Presentation"],
Cell[42711, 1599, 121, 3, 39, 47, 0, "StyleData", "ExercisesNoLabel", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[42869, 1607, 112, 3, 35, 37, 0, "StyleData", "ExerciseItemNumber", \
"All"],
Cell[42984, 1612, 65, 1, 32, 49, 0, "StyleData", "ExerciseItemNumber", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[43086, 1618, 115, 3, 35, 40, 0, "StyleData", "ExerciseSubItemNumber", \
"All"],
Cell[43204, 1623, 68, 1, 32, 52, 0, "StyleData", "ExerciseSubItemNumber", \
"Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[43321, 1630, 28, 0, 43, "Section"],

Cell[CellGroupData[{
Cell[43374, 1634, 231, 7, 63, 35, 0, "StyleData", "SolutionsSection", "All"],
Cell[43608, 1643, 148, 3, 66, 51, 0, "StyleData", "SolutionsSection", \
"Presentation"],
Cell[43759, 1648, 105, 2, 59, 47, 0, "StyleData", "SolutionsSection", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[43901, 1655, 401, 11, 51, 38, 0, "StyleData", "SolutionsSubsection", \
"All",
  CounterAssignments->{{"Subsection", 0}, {"Subsubsection", 0}, {
      "SolutionItem", 0}}],
Cell[44305, 1668, 93, 2, 54, 54, 0, "StyleData", "SolutionsSubsection", \
"Presentation"],
Cell[44401, 1672, 101, 2, 62, 50, 0, "StyleData", "SolutionsSubsection", \
"Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[44539, 1679, 400, 10, 40, 31, 0, "StyleData", "SolutionItem", "All",
  CounterIncrements->"SolutionItem"],
Cell[44942, 1691, 97, 2, 41, 47, 0, "StyleData", "SolutionItem", \
"Presentation"],
Cell[45042, 1695, 93, 2, 37, 43, 0, "StyleData", "SolutionItem", "Printout"]
}, Closed]],

Cell[CellGroupData[{
Cell[45172, 1702, 173, 5, 40, 34, 0, "StyleData", "SolutionNoLabel", "All"],
Cell[45348, 1709, 66, 1, 41, 50, 0, "StyleData", "SolutionNoLabel", \
"Presentation"],
Cell[45417, 1712, 120, 3, 39, 46, 0, "StyleData", "SolutionNoLabel", \
"Printout"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[45586, 1721, 31, 0, 43, "Section"],
Cell[45620, 1723, 132, 3, 8, 43, 0, "StyleData", "SectionSpacer", "Working"],
Cell[45755, 1728, 121, 2, 56, 46, 0, "StyleData", "SectionSpacer", \
"Production"],
Cell[45879, 1732, 135, 3, 8, 46, 0, "StyleData", "SubsectionSpacer", \
"Working"],
Cell[46017, 1737, 168, 3, 32, 49, 0, "StyleData", "SubsectionSpacer", \
"Production"]
}, Open  ]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

