(************** Content-type: application/mathematica **************
                     CreatedBy='Mathematica 5.1'

                    Mathematica-Compatible Notebook

This notebook can be used with any Mathematica-compatible
application, such as Mathematica, MathReader or Publicon. The data
for the notebook starts with the line containing stars above.

To get the notebook into a Mathematica-compatible application, do
one of the following:

* Save the data starting with the line of stars above into a file
  with a name ending in .nb, then open the file inside the
  application;

* Copy the data starting with the line of stars above to the
  clipboard, then use the Paste menu command inside the application.

Data for notebooks contains only printable 7-bit ASCII and can be
sent directly in email or through ftp in text mode.  Newlines can be
CR, LF or CRLF (Unix, Macintosh or MS-DOS style).

NOTE: If you modify the data for this notebook not in a Mathematica-
compatible application, you must delete the line below containing
the word CacheID, otherwise Mathematica-compatible applications may
try to use invalid cache data.

For more information on notebooks and Mathematica-compatible 
applications, contact Wolfram Research:
  web: http://www.wolfram.com
  email: info@wolfram.com
  phone: +1-217-398-0700 (U.S.)

Notebook reader applications are available free of charge from 
Wolfram Research.
*******************************************************************)

(*CacheID: 232*)


(*NotebookFileLineBreakTest
NotebookFileLineBreakTest*)
(*NotebookOptionsPosition[     15577,        516]*)
(*NotebookOutlinePosition[     16522,        549]*)
(*  CellTagsIndexPosition[     16410,        542]*)
(*WindowFrame->Normal*)



Notebook[{

Cell[CellGroupData[{
Cell["The RandomWalks Package", "Subsection",
  CellTags->"c:11"],

Cell[TextData[{
  "From the book, ",
  StyleBox["Introduction to Programming with Mathematica",
    FontSlant->"Italic"],
  ", 3rd Edition. \n\[Copyright] Cambridge University Press 2004"
}], "Text",
  CellFrame->True,
  Background->GrayLevel[0.833326]],

Cell[TextData[{
  "In this section, we list the full ",
  StyleBox["RandomWalks", "MR"],
  " package, including expressions for options and usage statements."
}], "Text"],

Cell[CellGroupData[{

Cell["BeginPackage", "Subsubsection"],

Cell[TextData[{
  "First, we set the value of ",
  StyleBox["Context`", "MR"],
  " which causes ",
  StyleBox["$ContextPath", "MR"],
  " to be set to ",
  StyleBox["{IPM3`RandomWalks`, System`}", "MR"],
  "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(BeginPackage["\<IPM3`RandomWalks`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("IPM3`RandomWalks`"\)], "Output"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Importing Other Packages", "Subsubsection"],

Cell[TextData[{
  "You could import a package by using an optional argument to ",
  StyleBox["BeginPackage", "MR"],
  ". In that case, you would have ",
  StyleBox["BeginPackage[\"IPM3`RandomWalks`\",{Graphics`ArgColors`,Graphics`\
Polyhedra`\"}]", "MR"],
  " above. The argument against this approach is that the two packages ",
  StyleBox["Graphics`ArgColors`", "MR"],
  " and ",
  StyleBox["Graphics`Polyhedra`", "MR"],
  " will be left on the search path after the ",
  StyleBox["RandomWalks`", "MR"],
  " package is read in. It is considered poor programming style to alter the \
user\[CloseCurlyQuote]s environment by simply reading in a \
package\[LongDash]at least you should try to alter it as little as possible. \
There is another method of loading a package within a package, and that is to \
call ",
  StyleBox["Needs", "MR"],
  " ",
  StyleBox["after",
    FontSlant->"Italic"],
  " the call to ",
  StyleBox["BeginPackage", "MR"],
  ". Using this mechanism, the ",
  StyleBox["Graphics`ArgColors`", "MR"],
  " context will not remain on the context path after the ",
  StyleBox["RandomWalks", "MR"],
  " package is read in."
}], "Text"],

Cell[BoxData[
    \(Needs["\<Graphics`ArgColors`\>"]\)], "Input",
  InitializationCell->True],

Cell["Needs[\"Graphics`Polyhedra`\"]", "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Usage statements", "Subsubsection"],

Cell["\<\
Defining usage messages for the functions in your packages creates symbols \
for the functions in the current context. Each of the functions for which you \
define a usage message will then be exported for public use; that is, those \
functions are visible and usable immediately after loading the package. This \
is in distinction to any functions that are defined in your package for which \
you do not have usage messages. Those functions will be private, unavailable \
for the user of your package to access. \
\>", "Text"],

Cell[TextData[{
  "Making your functions behave much like the built-in functions will make it \
easier for users of your packages since they will expect usage messages and \
general functionality similar to that of ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  "\[CloseCurlyQuote]s functions. It is also a good way for you to document \
what you have done. We would go so far as to suggest that you consider \
writing your usage messages ",
  StyleBox["before",
    FontSlant->"Italic"],
  " you write the function definitions in ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  ". This will help you to clearly understand what it is you want your \
functions to do."
}], "Text"],

Cell[BoxData[
    \(\(RandomWalk::"\<usage\>" = "\<RandomWalk[n] generates an n-step walk \
in two dimensions. The default behavior gives a lattice walk with steps in \
one of the four compass directions. Setting the option \
LatticeWalk\[Rule]False will generate an off-lattice walk. Setting the option \
Dimensions\[Rule]3 will generate a 3-dimensional walk; Dimensions\[Rule]1 \
gives a 1-dimensional walk.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(LatticeWalk::"\<usage\>" = "\<LatticeWalk\[Rule]val is an option to \
RandomWalk that determines whether the random walk will be a lattice walk or \
an off-lattice walk. Possible values are True and False.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(ShowWalk::"\<usage\>" = "\<ShowWalk[walk] displays a two-dimensional \
random walk connecting each site with a line. Any Graphics options can be \
passed to ShowWalk. E.g., ShowWalk[walk,  Background\[Rule]GrayLevel[0]] to \
produce a black background.\>";\)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(AnimateWalk::"\<usage\>" = "\<AnimateWalk[walk, opts] creates an \
animation of a two-dimensional random walk. A red ball will be seen to move \
to the current position in the walk to aid in visualizing the \
animation.\>";\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Warning Messages", "Subsubsection"],

Cell[BoxData[
    \(\(RandomWalk::rwn = "\<Argument `1` is not a positive \
integer.\>";\)\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["Options", "Subsubsection"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Options[RandomWalk] = {LatticeWalk \[Rule] True, 
        Dimension \[Rule] 2}\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \({LatticeWalk \[Rule] True, Dimension \[Rule] 2}\)], "Output"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Begin private context", "Subsubsection"],

Cell[TextData[{
  "The ",
  StyleBox["Begin", "MR"],
  " command changes the current context without affecting the context path. \
By starting the argument ",
  StyleBox["`Private`", "MR"],
  " with a context mark ",
  StyleBox["`", "MR"],
  ", we change to a subcontext of the current context. This new subcontext is \
",
  StyleBox["IPM3`RandomWalks`Private`", "MR"],
  ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Begin["\<`Private`\>"]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("IPM3`RandomWalks`Private`"\)], "Output"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{

Cell["The Function Definitions", "Subsubsection"],

Cell["walk1D[n_] := NestList[#1 + (-1)^Random[Integer] & , 0, n]", "Input",
  InitializationCell->True],

Cell[BoxData[
    \(walk1DOffLattice[n_] := \[IndentingNewLine]FoldList[Plus, 0, 
        Table[Random[Real, {\(-1\), 1}], {n}]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(walk2D[
        n_] := \[IndentingNewLine]Module[{NSEW = {{0, 1}, {1, 
                0}, {0, \(-1\)}, {\(-1\), 0}}}, \[IndentingNewLine]FoldList[
          Plus, {0, 0}, \[IndentingNewLine]NSEW\[LeftDoubleBracket]
            Table[Random[
                Integer, {1, 4}], {n}]\[RightDoubleBracket]]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(walk2DOffLattice[n_] := \[IndentingNewLine]FoldList[Plus, {0, 0}, 
        Map[{Cos[#], Sin[#]} &, 
          Table[Random[Real, {0, 2\ \[Pi]}], {n}]]]\)], "Input",
  InitializationCell->True],

Cell["\<\
walk3D[n_] := Module[{NSEW3 = Sqrt[2]*Vertices[Cube]}, 
   FoldList[Plus, {0, 0, 0}, 
    NSEW3[[Table[Random[Integer, {1, 8}], {n}]]]]]\
\>", "Input",
  InitializationCell->True],

Cell["\<\
walk3DOffLattice[n_] := Module[{x, y, z}, 
   x[\[Phi]_, \[Theta]_] := Sqrt[1 - Cos[\[Phi]]^2]*Cos[\[Theta]]; 
    y[\[Phi]_, \[Theta]_] := Sqrt[1 - Cos[\[Phi]]^2]*Sin[\[Theta]]; 
    z[\[Phi]_] := Cos[\[Phi]]; FoldList[Plus, {0, 0, 0}, 
     Apply[{x[#1, #2], y[#1, #2], z[#1]} & , 
      Table[Random[Real, {0, 2*Pi}], {n}, {2}], {1}]]]\
\>", "Input",
  InitializationCell->True],

Cell[BoxData[
    \(RandomWalk[n_, opts___?OptionQ] := 
      Module[{dim, latticeQ}, \[IndentingNewLine]If[
          Not[IntegerQ[n] && n > 0], 
          Message[RandomWalk::rwn, 
            n], {latticeQ, dim} = {LatticeWalk, Dimension} /. 
              Flatten[{opts, 
                  Options[
                    RandomWalk]}]; \[IndentingNewLine]Which[\
\[IndentingNewLine]dim == 1, 
            If[latticeQ, walk1D[n], 
              walk1DOffLattice[n]], \[IndentingNewLine]dim == 2, 
            If[latticeQ, walk2D[n], 
              walk2DOffLattice[n]], \[IndentingNewLine]dim == 3, 
            If[latticeQ, walk3D[n], 
              walk3DOffLattice[n]]\[IndentingNewLine]]]]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(\(\(ShowWalk[coords_, opts___] := 
      Which[\[IndentingNewLine]Length[Dimensions[coords]] \[Equal] 1, 
        ListPlot[coords, opts, 
          PlotJoined \[Rule] True], \(Dimensions[coords]\)[\([2]\)] \[Equal] 
          2, Show[Graphics[Line[coords], opts, 
            AspectRatio \[Rule] Automatic]], \(Dimensions[
              coords]\)[\([2]\)] \[Equal] 3, 
        Show[Graphics3D[Line[coords], opts, 
            AspectRatio \[Rule] Automatic]]]\)\(\n\)
    \)\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \(AnimateWalk[coords_, opts___] := 
      Scan[Show[
            Graphics[{{RGBColor[1, 0, 0], PointSize[0.025], 
                  Point[coords\[LeftDoubleBracket]#1\[RightDoubleBracket]]}, 
                Line[Take[coords, #1]]}], opts, 
            AspectRatio \[Rule] Automatic, 
            PlotRange \[Rule] \(({Min[#1] - 0.2, Max[#1] + 0.2} &)\) /@ 
                Transpose[coords]] &, Range[2, Length[coords]]]\)], "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{

Cell["End private context", "Subsubsection"],

Cell[TextData[{
  "The ",
  StyleBox["End[]", "MR"],
  " command closes the ",
  StyleBox["Begin[]", "MR"],
  " and puts us back in the context ",
  StyleBox["RandomWalks`", "MR"],
  ". Any symbols that were defined in the subcontext ",
  StyleBox["IPM3`RandomWalks`Private`", "MR"],
  " can no longer be accessed."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(End[]\)], "Input",
  InitializationCell->True],

Cell[BoxData[
    \("IPM3`RandomWalks`Private`"\)], "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["End package", "Subsubsection"],

Cell[TextData[{
  "The ",
  StyleBox["EndPackage[]", "MR"],
  " command puts us back in the context we were in prior to the ",
  StyleBox["BeginPackage[]", "MR"],
  " command."
}], "Text"],

Cell[BoxData[
    \(EndPackage[]\)], "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection"],

Cell[TextData[{
  "Starting with a new session, and making sure that the ",
  StyleBox["RandomWalks", "MR"],
  " package is in a directory/folder where ",
  StyleBox["Mathematica",
    FontSlant->"Italic"],
  " can find it, this loads the package."
}], "Text"],

Cell[BoxData[
    \(<< IPM3`RandomWalks`\)], "Input"],

Cell[TextData[{
  "Here is the usage message for the ",
  StyleBox["RandomWalk", "MR"],
  " function."
}], "Commentary"],

Cell[CellGroupData[{

Cell[BoxData[
    \(\(?RandomWalk\)\)], "Input"],

Cell[BoxData[
    \("RandomWalk[n] generates an n-step walk in two dimensions. The default \
behavior gives a lattice walk with steps in one of the four compass \
directions. The option LatticeWalk takes values True or False. The value of \
the option Dimensions can be any of 1, 2, or 3."\)], "Print",
  CellTags->"Info3287377863-9274400"]
}, Closed]],

Cell["This gives a random walk of length 10 in two dimensions.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(RandomWalk[10, Dimension \[Rule] 2]\)], "Input"],

Cell[BoxData[
    \({{0, 
        0}, {0, \(-1\)}, {1, \(-1\)}, {1, \(-2\)}, {1, \(-3\)}, {1, \(-2\)}, \
{1, \(-3\)}, {2, \(-3\)}, {2, \(-2\)}, {3, \(-2\)}, {3, \(-1\)}}\)], "Output"]
}, Closed]],

Cell["\<\
This shows a 250-step off-lattice random walk using the default of two \
dimensions.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowWalk[RandomWalk[250, LatticeWalk \[Rule] False]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell["\<\
A 500-step two-dimensional random walk with some graphics options.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(ShowWalk[RandomWalk[500], Frame \[Rule] True]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell["A 100 step off-lattice random walk in three dimensions.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(walk3 = 
      ShowWalk[RandomWalk[10\^3, Dimension \[Rule] 3, 
          LatticeWalk \[Rule] False]]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell["\<\
Using a transformation rule, it is straightforward to make each line red and \
thicker.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
    \(Show[
      walk3 /. Line[x_] :> {Hue[0], Thickness[ .01], Line[x]}]\)], "Input"],

Cell[BoxData[
    TagBox[\(\[SkeletonIndicator]  Graphics3D  \[SkeletonIndicator]\),
      False,
      Editable->False]], "Output"]
}, Closed]],

Cell["\<\
This animation shows a 100-step off-lattice random walk in two dimensions. \
(We only show the final frame of the animation in the printed book.)\
\>", "Text"],

Cell[BoxData[
    \(AnimateWalk[\n\t
      RandomWalk[100, LatticeWalk \[Rule] False, 
        Dimension \[Rule] 2]]\)], "Input"],

Cell[TextData[{
  "Although we have omitted them here, several additional functions are \
available in the package ",
  StyleBox["IPM3`RandomWalks`", "MR"],
  " for performing numerical analysis on random walks. "
}], "Text"]
}, Closed]]
}, Open  ]]
},
FrontEndVersion->"5.1 for Microsoft Windows",
ScreenRectangle->{{0, 1024}, {0, 681}},
AutoGeneratedPackage->Automatic,
WindowSize->{707, 582},
WindowMargins->{{15, Automatic}, {Automatic, 0}}
]

(*******************************************************************
Cached data follows.  If you edit this Notebook file directly, not
using Mathematica, you must remove the line containing CacheID at
the top of  the file.  The cache data will then be recreated when
you save this file from within Mathematica.
*******************************************************************)

(*CellTagsOutline
CellTagsIndex->{
  "c:11"->{
    Cell[1776, 53, 65, 1, 38, "Subsection",
      CellTags->"c:11"]},
  "Info3287377863-9274400"->{
    Cell[12825, 413, 340, 5, 77, "Print",
      CellTags->"Info3287377863-9274400"]}
  }
*)

(*CellTagsIndex
CellTagsIndex->{
  {"c:11", 16199, 532},
  {"Info3287377863-9274400", 16299, 535}
  }
*)

(*NotebookFileOutline
Notebook[{

Cell[CellGroupData[{
Cell[1776, 53, 65, 1, 38, "Subsection",
  CellTags->"c:11"],
Cell[1844, 56, 253, 7, 68, "Text"],
Cell[2100, 65, 170, 4, 33, "Text"],

Cell[CellGroupData[{
Cell[2295, 73, 37, 0, 29, "Subsubsection"],
Cell[2335, 75, 220, 8, 33, "Text"],

Cell[CellGroupData[{
Cell[2580, 87, 98, 2, 30, "Input",
  InitializationCell->True],
Cell[2681, 91, 53, 1, 29, "Output"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[2783, 98, 49, 0, 29, "Subsubsection"],
Cell[2835, 100, 1151, 28, 166, "Text"],
Cell[3989, 130, 93, 2, 30, "Input",
  InitializationCell->True],
Cell[4085, 134, 75, 1, 30, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[4197, 140, 41, 0, 29, "Subsubsection"],
Cell[4241, 142, 537, 8, 109, "Text"],
Cell[4781, 152, 698, 16, 109, "Text"],
Cell[5482, 170, 456, 7, 130, "Input",
  InitializationCell->True],
Cell[5941, 179, 274, 4, 90, "Input",
  InitializationCell->True],
Cell[6218, 185, 321, 5, 110, "Input",
  InitializationCell->True],
Cell[6542, 192, 288, 5, 90, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[6867, 202, 41, 0, 29, "Subsubsection"],
Cell[6911, 204, 131, 3, 30, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[7079, 212, 32, 0, 29, "Subsubsection"],

Cell[CellGroupData[{
Cell[7136, 216, 139, 3, 30, "Input",
  InitializationCell->True],
Cell[7278, 221, 81, 1, 29, "Output"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[7408, 228, 46, 0, 29, "Subsubsection"],
Cell[7457, 230, 388, 12, 71, "Text"],

Cell[CellGroupData[{
Cell[7870, 246, 83, 2, 30, "Input",
  InitializationCell->True],
Cell[7956, 250, 61, 1, 29, "Output"]
}, Closed]]
}, Open  ]],

Cell[CellGroupData[{
Cell[8066, 257, 49, 0, 29, "Subsubsection"],
Cell[8118, 259, 103, 1, 30, "Input",
  InitializationCell->True],
Cell[8224, 262, 169, 3, 50, "Input",
  InitializationCell->True],
Cell[8396, 267, 364, 7, 90, "Input",
  InitializationCell->True],
Cell[8763, 276, 212, 4, 50, "Input",
  InitializationCell->True],
Cell[8978, 282, 189, 5, 66, "Input",
  InitializationCell->True],
Cell[9170, 289, 391, 8, 120, "Input",
  InitializationCell->True],
Cell[9564, 299, 735, 16, 170, "Input",
  InitializationCell->True],
Cell[10302, 317, 534, 11, 150, "Input",
  InitializationCell->True],
Cell[10839, 330, 480, 9, 130, "Input",
  InitializationCell->True]
}, Open  ]],

Cell[CellGroupData[{
Cell[11356, 344, 44, 0, 29, "Subsubsection"],
Cell[11403, 346, 327, 10, 52, "Text"],

Cell[CellGroupData[{
Cell[11755, 360, 66, 2, 30, "Input",
  InitializationCell->True],
Cell[11824, 364, 61, 1, 29, "Output"]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{
Cell[11934, 371, 36, 0, 23, "Subsubsection"],
Cell[11973, 373, 188, 6, 33, "Text"],
Cell[12164, 381, 73, 2, 30, "Input",
  InitializationCell->True]
}, Closed]],

Cell[CellGroupData[{
Cell[12274, 388, 33, 0, 23, "Subsubsection"],
Cell[12310, 390, 260, 7, 52, "Text"],
Cell[12573, 399, 53, 1, 30, "Input"],
Cell[12629, 402, 120, 4, 20, "Commentary"],

Cell[CellGroupData[{
Cell[12774, 410, 48, 1, 30, "Input"],
Cell[12825, 413, 340, 5, 77, "Print",
  CellTags->"Info3287377863-9274400"]
}, Closed]],
Cell[13180, 421, 72, 0, 30, "Text"],

Cell[CellGroupData[{
Cell[13277, 425, 68, 1, 30, "Input"],
Cell[13348, 428, 183, 3, 48, "Output"]
}, Closed]],
Cell[13546, 434, 108, 3, 30, "Text"],

Cell[CellGroupData[{
Cell[13679, 441, 85, 1, 30, "Input"],
Cell[13767, 444, 130, 3, 29, "Output"]
}, Closed]],
Cell[13912, 450, 90, 2, 30, "Text"],

Cell[CellGroupData[{
Cell[14027, 456, 78, 1, 30, "Input"],
Cell[14108, 459, 130, 3, 29, "Output"]
}, Closed]],
Cell[14253, 465, 71, 0, 30, "Text"],

Cell[CellGroupData[{
Cell[14349, 469, 134, 3, 31, "Input"],
Cell[14486, 474, 132, 3, 29, "Output"]
}, Closed]],
Cell[14633, 480, 111, 3, 30, "Text"],

Cell[CellGroupData[{
Cell[14769, 487, 101, 2, 30, "Input"],
Cell[14873, 491, 132, 3, 29, "Output"]
}, Closed]],
Cell[15020, 497, 169, 3, 49, "Text"],
Cell[15192, 502, 129, 3, 50, "Input"],
Cell[15324, 507, 225, 5, 52, "Text"]
}, Closed]]
}, Open  ]]
}
]
*)



(*******************************************************************
End of Mathematica Notebook file.
*******************************************************************)

